\ CdcMscConfig.fth - USB CDC and MSC composite configuration

((
Copyright (c) 2008. 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20110825 MPE002 Serial numbering and OSX changes.
))

only forth definitions
decimal

\ ==========
\ *! cdcmscconfig
\ *T USB composite CDC and MSC configuration
\ =========
\ *P The file *\i{CdcMscConfig.fth} contains a USB configuration for
\ ** a composite Communications Device Class (CDC) and Mass Storage
\ ** Class (MSC) application. For CDC and MSC only applications see
\ ** *\i{CdcConfig.fth} and *\i{MscConfig.fth}.

\ *P Running composite devices on Windows without using a custom
\ ** driver for Windows can be problematic.
\ ** Windows 7, Vista and XP SP3 are fine, but XP SP2 is not. For XP,
\ ** either upgrade to SP3 or install hotfixes. It seems you need:
\ *(
\ *B Hotfix KB935892 (which brings usbccgp.sys 5.1.2600.3116)
\ *B Hotfix KB918365 (which brings usbser.sys 5.1.2600.2930)
\ *)

\ *P We installed SP3 for testing and had to copy the new version
\ ** of *\i{usbser.sys} manually from *\i{SP3.cab}. Use Windows
\ ** search and copy the new version to replace the one in
\ ** *\i{Windows\System32\Drivers}.

\ *P The trick for composite devices is to use an Interface
\ ** Aggregation Descriptor (IAD) for each instance of any class
\ ** with more than one interface, e.g. CDC. Read the source code
\ ** for the gory details!

\ *P You also need to take care with the start up code for
\ ** composite devices. USB device start up consists of three
\ ** steps.
\ *( 1
\ *B Initialise data buffers, some of which may be in *\fo{UDATA}
\ ** section, and so will not be initialised by the default MPE
\ ** initialisation code. This code should also be part of the
\ ** USB reset action for the class or interface.
\ *B The primary initialisation of the USB hardware and code is
\ ** performed by *\fo{InitUSB}.
\ *B Then perform the USB bus connection process using
\ ** *\fo{USBconnect}.
\ *)

\ *P Your start up code for a composite device then consists of
\ *C   initA initB ... InitUSB USBConnect
\ *P Versions of this stack from November 2010 onwards include
\ ** in *\i{UsbEP0.fth} the word *\fo{startUSBsys} which performs
\ ** these operations.


\ ************************
\ *S Primary Configuration
\ ************************
\ *P Note that interface numbers must be a contiguous set
\ ** starting at 0.

0 equ dbgUHW?	\ -- flag
\ *G Set true to get debug information from the USB hardware
\ ** layer. Some of the debug output *\b{must} be done using
\ ** polled serial drivers.
0 equ dbgCore?	\ -- flag
\ *G Set true to get debug information from the USB core layer.
\ ** Some of the debug output *\b{must} be done using polled
\ ** serial drivers.
0 equ UsbPowered?	\ -- 0/1 ; bus powered?
\ *G Set to one if bus powered.
1 equ UsbDMA?		\ -- 0/1 ; uses DMA?
\ *G Set to one if any endpoints other than EP0 use DMA. At the
\ ** moment we assume that EP0 does not use DMA.
$0000:0000 equ UsbDMAEP	\ -- x ; DMA endpoints

#64 equ /maxpacket0	\ -- u
\ *G Maximum size of packets on endpoint 0.
#64 equ /maxpacket	\ -- u
\ *G Maximum size of packets on endpoints other than 0.

#32 equ #EPs	\ -- u
\ *G Number of endpoints we are are going to deal with. On a
\ ** 32 bit CPU, there are 32 bits per word. We need separate
\ ** bits for IN and OUT, so there's a practical maximum here
\ ** of 32, and USB supports 16 endpoints, each with IN and
\ ** OUT capability. Endpoint 0 is always IN and OUT capable.
4 equ #USBifs	\ -- u
\ *G Number of interfaces we can expand to eventually.

0 equ HID?	\ -- flag
\ *G True if a HID device, e.g. mouse, is to be supported.
1 equ CDC?	\ -- flag
\ *G True if a communications device class (CDC) is to be supported.
\ ** The most common of these is a USB serial port.
1 equ MSC?	\ -- flag
\ *G True if a mass storage device is to be supported.
0 equ AUDIO?	\ -- flag
\ *G True if an audio device is to be supported.

resetUIFs
\ *G Force the interface numbers to start at zero.

CDC? [if]
NextUIF: CdcCIf#	\ -- b
\ *G Interface number of the CDC control interface.
NextUIF: CdcDIf#	\ -- b
\ *G Interface number of the CDC data interface.
$81 equ CDCIinEP	\ -- ep
\ *G Optional Interrupt IN event notification endpoint (CIF).
$85 equ CDCBinEP	\ -- ep
\ *G CDC Bulk IN endpoint (DIF).
$05 equ CDCBoutEP	\ -- ep
\ *G CDC Bulk OUT endpoint (DIF).
1 equ dbgCDC?		\ -- flag
\ *G set non-zero to get debug information from the CDC driver.

CDCIinEP EP>mask equ CDCIinEPMask	\ -- mask
\ *G Core layer bit mask for the CDC Interrupt IN endpoint.
\ ** Used to avoid run-time calculation.
CDCBinEP EP>mask equ CDCBinEPMask	\ -- mask
\ *G Core layer bit mask for the CDC bulk IN endpoint.
\ ** Used to avoid run-time calculation.
CDCBoutEP EP>mask equ CDCBoutEPMask	\ -- mask
\ *G Core layer bit mask for the CDC bulk OUT endpoint.
\ ** Used to avoid run-time calculation.
[then]

MSC? [if]
NextUIF: MscIf#		\ -- b
\ *G Interface number of the mass storage class.
$82 equ MscBinEP	\ -- ep
\ *G MSC Bulk IN endpoint.
$02 equ MscBoutEP	\ -- ep
\ *G MSC Bulk OUT endpoint.
0 equ dbgMSC?		\ -- flag
\ *G set non-zero to get debug information from the MSC driver.
MscBinEP EP>mask equ MscBinEPMask	\ -- mask
\ *G Core layer bit mask for the MSC bulk IN endpoint.
\ ** Used to avoid run-time calculation.
MscBoutEP EP>mask equ MscBoutEPMask	\ -- mask
\ *G Core layer bit mask for the MSC bulk OUT endpoint.
\ ** Used to avoid run-time calculation.
[then]


\ ******************
\ *S USB Descriptors
\ ******************
\ *P This section contains the USB descriptors. There is a descriptor
\ ** for the device, for its configuration, and for various strings.
\ ** There are various other descriptors, both general and class
\ ** class specific. Their use is absurdly badly documented and
\ ** the actual requirement as to which are required is often
\ ** operating system dependent. If it works with Linux, it will
\ ** (probably) work with everything else.

\ *P Strings are accessed using a "string index". Apart from zero, which
\ ** has a predefined meaning to define the language, string indices
\ ** (1..255) are defined by the application. Here the indices are the
\ ** offset in bytes from the start of a string table. If you need more
\ ** than 255 bytes of string descriptors, change the index to be the
\ ** offset into a table of string descriptor addresses.

Create CfgDesc	\ -- addr
\ *G The configuration descriptor has a primary configuration, three
\ ** interfaces, and five endpoints. If there is more than one
\ ** configuration, additional ones must follow the first one.
\ -- configuration
  9 c,			\ descriptor size
  2 c,			\ configuration descriptor
  0 lew,		\ total size, this + interfaces + x endpoints - filled in later
  #UIFs c,		\ number of interfaces, 2 for CDC, 1 for MSC
  1 c,			\ configuration ID
  0 c,			\ no configuration string
  $C0 c,		\ self powered, $80 for bus powered
  #10 c,		\ and up to 2*10=20ma from the bus

\ -- Interface Association Descriptor - associates both CDC iinterfaces
  8 c,			\ descriptor size
  11 c,			\ IAD descriptor
  CdcCIf# c,		\ CDC control interface number
  2 c,			\ number of interfaces in association
  2 c,			\ Class - Communication Device Class
  2 c,			\ Subclass - ACM Abstract Control Model
  1 c,			\ protocol - modem, needed by Macs
  0 c,			\ no interface string

\ -- interface 0, CDC control interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  CdcCIf# c,		\ CDC control interface number
  0 c,			\ no alternate setting
  1 c,			\ number of endpoints
  2 c,			\ Class - Communication Device Class
  2 c,			\ Subclass - ACM Abstract Control Model
  1 c,			\ protocol - modem, needed by Macs
  0 c,			\ no interface string
\ -- HeaderFunctional Descriptor, CDC spec
  5 c,			\ descriptor size
  $024 c,		\ type - CS_INTERFACE
  0 c,			\ subtype - Header Functional Descriptor
  $0110 lew,		\ bcdCDC - CDC spec release number as BCD
\ -- Call Management Functional Descriptor, PSTN spec
  5 c,			\ descriptor size
  $024 c,		\ type - CS_INTERFACE
  1 c,			\ subtype - Call Management Descriptor
  1 c,			\ bmCapabilities - D0=1 for handles call management
  CdcDIf# c,		\ bDataInterface - data interface number
\ -- Abstract Control Management Functional Descriptor
  4 c,			\ length
  $024 c,		\ type - CS_INTERFACE
  2 c,			\ subtype - ACM
  2 c,			\ bmCapabilities - SET/GET_LINE_xxx
\ -- Union Functional Descriptor
  5 c,			\ length
  $024 c,		\ type - CS_INTERFACE
  6 c,			\ subtype - union
  CdcCIf# c,		\ bMasterInterface - control interface number
  CdcDIf# c,		\ bSlaveInterface - data interface number
\ -- Endpoint 1 Descriptor - event notification (optional)
  7 c,			\ desc size
  5 c,			\ endpoint descriptor
  CDCIinEP c,		\ Endpoint I/O+number
  3 c,			\ EP type - interrupt
  /MaxPacket lew,	\ max packet size
  #10 c,		\ poll interval in ms

\ -- interface 1, CDC data interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  CdcDIf# c,		\ CDC data interface number
  0 c,			\ no alternate setting
  2 c,			\ number of endpoints
  #10 c,		\ Class - CDC Data Interface
  0 c,			\ Subclass - none
  0 c,			\ protocol - none
  0 c,			\ no interface string
\ -- endpoint 5 in
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  CDCBinEP c,		\ dir and ep# - IN 5
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- endpoint 5 out
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  CDCBoutEP c,		\ dir and ep# - OUT 5
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints

\ -- interface 2, MSC interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  MscIf# c,		\ MSC interface number
  0 c,			\ alternate setting number
  2 c,			\ number of endpoints
  8 c,			\ Class - Mass Storage Class
  6 c,			\ SCSI transparent command set
  $50 c,		\ protocol - bulk only
  0 c,			\ no interface string
\ -- endpoint 2 in
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  MscBinEP c,		\ dir and ep# - IN 2
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- endpoint 2 out
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  MscBoutEP c,		\ dir and ep# - OUT 2
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints

\ -- end, or insert more configuration descriptors
here CfgDesc -  CfgDesc 2+ lew!		\ set config. desc. length
  0 lew,		\ needed by software

create StrDesc	\ -- addr
\ *G String descriptor and following strings.
\ *(
\ *B Str0 - identifies the language. This is usually left alone.
\ *B Str1 - the USB serial number, usually a 16 character string.
\ ** In the default code, the first 8 bytes are fixed and the
\ ** second 8 bytes can be derived from a 32 bit number.
\ ** Str2 - the product identification string.
\ ** Str3 - the manufacturer string.
\ *)
l: Str0		\ -- string 0, language
  4 c,			\ descriptor size
  3 c,			\ string
  $0409 lew,		\ type - US English
align l: str1	\ -- string 1, serial number
  usb", 0123456789AC0001" \ 16 Unicode digits
align l: str2	\ -- string 2, product
  usb", MPE CDC/MSC composite demo"
align l: Str3	\ -- string 3, manufacturer - must match .INF file
  usb", MPE - www.mpeforth.com"
		\ -- end
  0 lew,		\ needed by software


create DevDescDefault	\ -- addr
\ *G The device descriptor for Windows and Linux.
  #18 c,		\ descriptor size
  1 c,			\ device descriptor
  $0200 lew,		\ USB 2.00 - to use IAD
  $EF c,		\ Miscellaneous class - see iadclasscode_r10.pdf
  $02 c,		\ Common Class
  $01 c,		\ IAD
  /MaxPacket0 c,	\ endpoint 0 max size
  $0D59 lew,		\ Vendor ID (from www.usb.org) - VID=0D59 for MPE developers
  $FFF2 lew,		\ Product ID - PID=FFF0..FFFF for MPE developers
  $0002 lew,		\ BCD Release ID - assigned by vendor
  3 c,			\ Manufacturer string index
  2 c,			\ Product string index
  1 c,			\ serial number string index - 0=none
  1 c,			\ number of configurations
\ -- end
  0 lew,		\ needed by software
  align

\ MPE version for Mac OS X (up to 10.5.5)
create DevDescOSX	\ -- addr
\ *G The device descriptor for OSX.
  #18 c,		\ descriptor size
  1 c,			\ device descriptor
  $0200 lew,		\ USB 2.00
  2 c, 			\ CDC class
  0 c,			\ subclass is specified in the interface
  0 c,			\ protocol is specified in the interface
  /MaxPacket0 c,	\ endpoint 0 max size
  $0D59 lew,		\ Vendor ID (from www.usb.org) - VID=0D59 for MPE developers
  $FFF2 lew,		\ Product ID - PID=FFF0..FFFF for MPE developers
  $0002 lew,		\ BCD Release ID - assigned by vendor
  3 c,			\ Manufacturer string index
  2 c,			\ Product string index
  1 c,			\ serial number string index - 0=none
  1 c,			\ number of configurations
\ -- end
  0 lew,		\ needed by software
align

0 value DevDescReqLen	\ -- u
\ *G Holds request length of first device descriptor after reset.
\ ** It is used to auto-select the device descriptor

:noname 0 to DevDescReqLen  ; AtUSBreset
\ *G Performed at USB reset.

: DevDesc	\ -- addr
\ *G Return the address of the device descriptor. This word uses
\ ** a kluge to provide operation on Windows, Linux, and versions
\ ** of OS X before 10.5.6 which recognises the IAD descriptor.
  \ save request length of first device descriptor req after reset
  DevDescReqLen 0= if
    SetupPacket usp.wLength lew@ to DevDescReqLen
  endif
  \ pick descriptor based on this
  DevDescReqLen 8 =
  if  DevDescOSX  else  DevDescDefault  endif
;


\ ***************************
\ *S Serial number formatting
\ ***************************
\ *P If the word *\fo{unit#} is defined, it should return a
\ ** 32 bit unit serial number. This is used by the USB code
\ ** to return a serial number to the host. If the word
\ ** *\fo{unit#} has not been defined, a default fixed serial
\ ** number is returned.

[defined] unit# [if]
#34 buffer: UTFbuf	\ -- addr
\ *G Holds a string descriptor with up to 16 little-endian Unicode
\ ** 16-bit characters.

: usbSerialNum	\ -- caddr len
\ *G Returns the default device USB serial number if *\fo{unit#}
\ ** is not defined.
  Str1 UTFbuf #34 cmove		\ copy string to buffer
  unit#  UTFbuf #18 + #16 bounds do	\ -- sn
    dup #28 rshift $0F and		\ top 4 bits
    dup 9 >				\ convert to hex digit
    if  7 +  endif
    [char] 0 + i c!			\ char in first of two bytes
    4 lshift				\ ready for next 4 bits
  2 +loop
  drop  UTFbuf #34
;
[else]
: usbSerialNum	\ -- caddr len
\ *G Returns the default device USB serial number if *\i{unit#}
\ ** is not defined.
  Str1 dup c@
;
[then]

: USBStr@	\ index -- caddr len
\ *G Return the USB string descriptor corresponding to the given
\ ** string index. By convention the following order is used:
\ *D 0    language identifier - do not change
\ *D 1    serial number
\ *D 2    product
\ *D 3    manufacturer
\ *P If you configure your products with serial numbers extracted
\ ** from non-USB strings, e.g. MAC numbers, modify this word to
\ ** perform the string extraction as required.
  case
    0 of  Str0 dup c@  endof
    1 of  usbSerialNum  endof
    2 of  Str2 dup c@  endof
    3 of  Str3 dup c@  endof
      0 0 rot
  endcase
;


\ ======
\ *> ###
\ ======

decimal
