\ CdcConfig.fth2 - USB Communications Device Class configuration

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ==========
\ *! cdcconfig
\ *T USB CDC configuration
\ =========
\ *P The file *\i{CDCconfig.fth} contains a USB configuration for
\ ** a Communications Device Class (CDC) application.


\ ************************
\ *S Primary Configuration
\ ************************
\ *P Note that interface numbers must be a contiguous set
\ ** starting at 0.

0 equ dbgUHW?	\ -- flag
\ *G Set true to get debug information from the USB hardware
\ ** layer. Some of the debug output *\b{must} be done using
\ ** polled serial drivers.
0 equ dbgCore?	\ -- flag
\ *G Set true to get debug information from the USB core layer.
\ ** Some of the debug output *\b{must} be done using polled
\ ** serial drivers.
0 equ UsbPowered?	\ -- 0/1 ; bus powered?
\ *G Set to one if bus powered.
0 equ UsbDMA?		\ -- 0/1 ; uses DMA?
\ *G Set to one if any endpoints other than EP0 use DMA. At the
\ ** moment we assume that EP0 does not use DMA.
$0000:0000 equ UsbDMAEP	\ -- x ; DMA endpoints

#64 equ /maxpacket0	\ -- u
\ *G Maximum size of packets on endpoint 0.
#64 equ /maxpacket	\ -- u
\ *G Maximum size of packets on endpoints other than 0.

#32 equ #EPs	\ -- u
\ *G Number of endpoints we are are going to deal with. On a
\ ** 32 bit CPU, there are 32 bits per word. We need separate
\ ** bits for IN and OUT, so there's a practical maximum here
\ ** of 32, and USB supports 16 endpoints, each with IN and
\ ** OUT capability. Endpoint 0 is always IN and OUT capable.
4 equ #USBifs		\ -- u
\ *G Number of interfaces we can expand to eventually.

0 equ HID?	\ -- flag
\ *G True if a HID device, e.g. mouse, is to be supported.
0 equ MSC?	\ -- flag
\ *G True if a mass storage device is to be supported.
1 equ CDC?	\ -- flag
\ *G True if a communications device class (CDC) is to be supported.
\ ** The most common of these is a USB serial port.
0 equ AUDIO?	\ -- flag
\ *G True if an audio device is to be supported.
1 equ CLASS?	\ -- flag
\ *G Set this to 1 if any of the classes above are set to 1.

MSC? [if]
0 equ MscIf#		\ -- b
\ *G Interface number of the mass storage class.
$82 equ MscBinEP	\ -- ep
\ *G MSC Bulk IN endpoint.
$02 equ MscBoutEP	\ -- ep
\ *G MSC Bulk OUT endpoint.
0 equ dbgMSC?		\ -- flag
\ *G set non-zero to get debug information from the MSC driver.
MscBinEP EP>mask equ MscBinEPMask	\ -- mask
\ *G Core layer bit mask for the MSC bulk IN endpoint.
\ ** Used to avoid run-time calculation.
MscBoutEP EP>mask equ MscBoutEPMask	\ -- mask
\ *G Core layer bit mask for the MSC bulk OUT endpoint.
\ ** Used to avoid run-time calculation.
[then]

CDC? [if]
0 equ CdcCIf#		\ -- b
\ *G Interface number of the CDC control interface.
1 equ CdcDIf#		\ -- b
\ *G Interface number of the CDC data interface.
$81 equ CDCIinEP	\ -- ep
\ *G Optional Interrupt IN event notification endpoint (CIF).
$85 equ CDCBinEP	\ -- ep
\ *G CDC Bulk IN endpoint (DIF).
$05 equ CDCBoutEP	\ -- ep
\ *G CDC Bulk OUT endpoint (DIF).
1 equ dbgCDC?		\ -- flag
\ *G set non-zero to get debug information from the CDC driver.

CDCIinEP EP>mask equ CDCIinEPMask	\ -- mask
\ *G Core layer bit mask for the CDC Interrupt IN endpoint.
\ ** Used to avoid run-time calculation.
CDCBinEP EP>mask equ CDCBinEPMask	\ -- mask
\ *G Core layer bit mask for the CDC bulk IN endpoint.
\ ** Used to avoid run-time calculation.
CDCBoutEP EP>mask equ CDCBoutEPMask	\ -- mask
\ *G Core layer bit mask for the CDC bulk OUT endpoint.
\ ** Used to avoid run-time calculation.
[then]


\ **************
\ *S USB Descriptors
\ **************
\ *P This section contains the USB descriptors. There is a descriptor
\ ** for the device, for its configuration, and for various strings.

\ *P Strings are accessed using a "string index". Aprt from zero, which
\ ** has a predefined meaning to define the language, string indices
\ ** (1..255) are defined by the application. Here the indices are the
\ ** offset in bytes from the start of a string table. If you need more
\ ** than 255 bytes of string descriptors, change the index to be the
\ ** offset into a table of string descriptor addresses.

Create CfgDesc	\ -- addr
\ *G The configuration descriptor has a primary configuration, one
\ ** interface, and two endpoints. If there is more than one
\ ** configuration, additional ones must follow the first one.
\ -- configuration
  9 c,			\ descriptor size
  2 c,			\ configuration descriptor
  0 lew,		\ total size, this + interfaces + x endpoints - filled in later
  2 c,			\ number of interfaces
  1 c,			\ configuration ID
  0 c,			\ no configuration string
  $C0 c,		\ self powered
  #10 c,		\ and up to 2*10=20ma from the bus

\ -- interface 1, CDC control interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  CdcCIf# c,		\ CDC control interface number
  0 c,			\ no alternate setting
  1 c,			\ number of endpoints
  2 c,			\ Class - Communication Device Class
  2 c,			\ Subclass - ACM Abstract Control Model
  1 c,			\ protocol - modem, needed by Macs
  0 c,			\ no interface string
\ -- HeaderFunctional Descriptor, CDC spec
  5 c,			\ descriptor size
  $024 c,		\ type - CS_INTERFACE
  0 c,			\ subtype - Header Functional Descriptor
  $0110 lew,		\ bcdCDC - spec release number
\ -- Call Management Functional Descriptor, PSTN spec
  5 c,			\ descriptor size
  $024 c,		\ type - CS_INTERFACE
  1 c,			\ subtype - Call Management Descriptor
  1 c,			\ bmCapabilities - D0, handles call management
  CdcDIf# c,		\ bDataInterface - data interface number
\ -- Abstract Control Management Functional Descriptor
  4 c,			\ length
  $024 c,		\ type - CS_INTERFACE
  2 c,			\ subtype - ACM
  2 c,			\ bmCapabilities - SET/GET_LINE_xxx
\ -- Union Functional Descriptor
  5 c,			\ length
  $024 c,		\ type - CS_INTERFACE
  6 c,			\ subtype - union
  CdcCIf# c,		\ bMasterInterface - control interface number
  CdcDIf# c,		\ bSlaveInterface - data interface number
\ -- Endpoint 1 Descriptor - event notification (optional)
  7 c,			\ desc size
  5 c,			\ endpoint descriptor
  CDCIinEP c,		\ Endpoint I/O+number
  3 c,			\ EP type - interrupt
  /maxpacket lew,	\ max packet size
  #10 c,		\ poll interval in ms

\ -- interface 2, CDC data interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  CdcDIf# c,		\ CDC data interface number
  0 c,			\ no alternate setting
  2 c,			\ number of endpoints
  #10 c,		\ Class - CDC Data Interface
  0 c,			\ Subclass - none
  0 c,			\ protocol - none
  0 c,			\ no interface string
\ -- endpoint 5 in
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  CDCBinEP c,		\ dir and ep# - IN 5
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- endpoint 5 out
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  CDCBoutEP c,		\ dir and ep# - OUT 5
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- end, or insert more configuration descriptors
here CfgDesc -  CfgDesc 2+ lew!		\ set config. desc. length
  0 lew,		\ needed by software


create StrDesc	\ -- addr
\ *G String descriptor and following strings.
l: Str0		\ -- string 0, language
  4 c,			\ descriptor size
  3 c,			\ string
  $0409 lew,		\ type - US English
l: Str1		\ -- string 1, manufacturer - must match .INF file
  usb", MPE - www.mpeforth.com"
align l: str2	\ -- string 2, product
  usb", MPE CDC class demo"
align l: str3	\ -- string3, serial number
  usb", 0123456789AB0001"	\ 16 digits
		\ -- end
  0 lew,		\ needed by software

Str1 Str0 - equ Str1idx
Str2 Str0 - equ Str2idx
Str3 Str0 - equ Str3idx

create DevDesc	\ -- addr
\ *G The device descriptor.
  #18 c,		\ descriptor size
  1 c,			\ device descriptor
  $0110 lew,		\ USB 1.1
  2 c,			\ CDC class
  0 c,			\ subclass is specified in the interface
  0 c,			\ protocol is specified in the interface
  /MaxPacket0 c,	\ endpoint 0 max size
  $0D59 lew,		\ Vendor ID (from www.usb.org) - VID=0D59 for MPE developers
  $FFF1 lew,		\ Product ID - PID=FFF0..FFFF for MPE developers
  $0001 lew,		\ Release ID - assigned by vendor
  Str1idx c,		\ Manufacturer string index
  Str2idx c,		\ Product string index
  Str3idx c,		\ serial number string index - 0=none
  1 c,			\ number of configurations
\ -- end
  0 lew,		\ needed by software


\ ======
\ *> ###
\ ======

decimal
