\ devSTM32F072disco.fth - USB set up for STM32F072B Discovery board

((
Copyright (c) 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! devstm32f072disco
\ *T USB set up for STM32F072B Discovery
\ ===========
\ *P The file *\i{devSTM32F072disco.fth} provides USB initialisation
\ ** and LED drivers for an STM32F072B Discovery board.
\ ** It will require minor changes for other STM32F0 devices and
\ ** boards, in particular in the USB Connect function and LEDs.

\ USB - CDC or MSC only
  include %UsbDir%/UsbDefs		\ USB definitions/equates
  include %CommonDir%/CQueues		\ character queues
  include %UsbDir%/VCOMdev		\ UART device emulation
((
  include %UsbDir%\MscConfig		\ MSC configuration file
  include %UsbDir%\UsbHw2xxx		\ USB hardware layer for LPC2148
  include %UsbDir%\UsbBase		\ USB base operations
MSC? [if]
  include %UsbDir%\MscDrv		\ Mass Storage Class
[then]
CDC? [if]
  include %UsbDir%\VCOMdev		\ UART device emulation
  include %UsbDir%\CdcDrv		\ Communications Device Class
[then]
  include %UsbDir%\UsbEP0		\ USB control and initialisation
))

\ **********************
\ *S USB interrupt level
\ **********************

0 equ USBpri#	\ -- u
\ *G Slot/Priority (0..15, 0=high) used for the USB interrupt handler.


\ *******
\ *S LEDs
\ *******
\ *P The hardware uses two LEDs to indicate USB activity (LED1) and
\ ** disk activity (LED2).

: usbLED1on	\ --
\ *G Turn LED1 on.
  LDL setPin  ;
: usbLED1off	\ --
\ *G Turn LED1 off.
  LDL clrPin  ;
: usbLED2on	\ --
\ *G Turn LED2 on.
  LDR setPin  ;
: usbLED2off	\ --
\ *G Turn LED2 off.
  LDR clrPin  ;

: initLEDs	\ --
\ *G Initialise the two LEDs to off, output=low. These are on
\ ** P0.10,11. Executed at power up.
  LedActive off  usbLED1off  usbLED2off
  LDU clrPin  LDD clrPin
;
' initLEDs AtCold



\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary STM32F0xx USB setup is performed in
\ ** *\i{UsbHwSTM32F0xx.fth}. Because STM32F0xx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.

: udelay	\ --
\ *G Short delay to let hardware react.
  1000 0 do loop
;

: USBhwInit	\ --
\ *G Enable access to the USB peripheral.
  _RCC
  bit16 over rccCR2 + or!		\ enable HSI48
  begin					\ wait until ready
    dup rccCR2 + @ bit17 and
  until
  bit7 over rccCFGR3 + bic!		\ select HSI48 for USB
\ switch to the HSI48 output
  $0003 $0003 2 pick rccCFGR + setMask	\ set SW=3
  begin
    dup rccCFGR + @ $000C and $000C =	\ wait for SWS=3
  until
  bit23 over rccAPB1EN + or!		\ enable USB
  bit23 over rccAPB1RST + bic!		\ clear USB reset
  bit27 over rccAPB1EN + or!		\ enable CRS
  bit27 over rccAPB1RST + bic!		\ clear CRS reset
  drop

  _USBdev
  $0001 over usbCNTR + !		\ reset USB peripheral
  udelay
  $0000 over usbCNTR + !
  drop

  _CRS
  $0000:2040 over crsCR + !		\ default trim, autotrim
  $2022:BB7F over crsCFGR + !		\ src=SOF, 1 ms, 48 MHz
  $0000:000F over crsICR + !		\ clear interrupts
  bit5 over crsCR + or!			\ default trim + CEN
  drop
;

: .CRS		\ --
\ *G Display CRS state.
  _CRS #16 pdump
;

: usbHWconnect	\ --
\ *G Connect pull up on USB DP line.
  bit15 _USBdev usbBCDR + or!
;

: usbHWdisconnect	\ --
\ *G Disconnect pull up on USB DP line.
  bit15 _USBdev usbBCDR + bic!
;

_USBSRAM constant _USBSRAM	\ -- addr
\ *G Base address of 768 bytes of USB packet memory, which must
\ ** be addressed/read/written in 16 bit units.
_USBdev constant _USBdev	\ -- addr
\ *G Base address of USB peripheral.
_CRS constant _CRS		\ -- addr
\ *G Base address of USB Clock Recovery System, which tunes the
\ ** HSI48 oscillator using the SOF pulses.


\ =========
\ Main code
\ =========

\ USB driver

create usb:dev
  18 c,  \ bLength
  $01 c, \ USB_DEVICE_DESCRIPTOR_TYPE
  $00 c,
  $02 c, \ bcdUSB = 2.00
  $02 c, \ bDeviceClass: CDC
  $00 c, \ bDeviceSubClass
  $00 c, \ bDeviceProtocol
  $40 c, \ bMaxPacketSize0
  $83 c,
  $04 c, \ idVendor = 0x0483
  $40 c,
  $57 c, \ idProduct = 0x7540
  $00 c,
  $02 c, \ bcdDevice = 2.00
  1 c,     \ Index of string descriptor describing manufacturer
  2 c,     \ Index of string descriptor describing product
  3 c,     \ Index of string descriptor describing the device's serial number
  $01 c, \ bNumConfigurations
calign

create usb:conf  \ total length = 67 bytes
\ USB Configuration Descriptor
  9 c,   \ bLength: Configuration Descriptor size
  $02 c, \ USB_CONFIGURATION_DESCRIPTOR_TYPE
  67 c,  \ VIRTUAL_COM_PORT_SIZ_CONFIG_DESC
  0 c,
  2 c,   \ bNumInterfaces: 2 interface
  1 c,   \ bConfigurationValue
  0 c,   \ iConfiguration
  $C0 c, \ bmAttributes: self powered
  $32 c, \ MaxPower 0 mA
\ Interface Descriptor
  9 c,   \ bLength: Interface Descriptor size
  $04 c, \ USB_INTERFACE_DESCRIPTOR_TYPE
  $00 c, \ bInterfaceNumber: Number of Interface
  $00 c, \ bAlternateSetting: Alternate setting
  $01 c, \ bNumEndpoints: One endpoints used
  $02 c, \ bInterfaceClass: Communication Interface Class
  $02 c, \ bInterfaceSubClass: Abstract Control Model
  $01 c, \ bInterfaceProtocol: Common AT commands
  $00 c, \ iInterface:
\ Header Functional Descriptor
  5 c,   \ bLength: Endpoint Descriptor size
  $24 c, \ bDescriptorType: CS_INTERFACE
  $00 c, \ bDescriptorSubtype: Header Func Desc
  $10 c, \ bcdCDC: spec release number
  $01 c,
\ Call Management Functional Descriptor
  5 c,   \ bFunctionLength
  $24 c, \ bDescriptorType: CS_INTERFACE
  $01 c, \ bDescriptorSubtype: Call Management Func Desc
  $00 c, \ bmCapabilities: D0+D1
  $01 c, \ bDataInterface: 1
\ ACM Functional Descriptor
  4 c,   \ bFunctionLength
  $24 c, \ bDescriptorType: CS_INTERFACE
  $02 c, \ bDescriptorSubtype: Abstract Control Management desc
  $02 c, \ bmCapabilities
\ Union Functional Descriptor
  5 c,   \ bFunctionLength
  $24 c, \ bDescriptorType: CS_INTERFACE
  $06 c, \ bDescriptorSubtype: Union func desc
  $00 c, \ bMasterInterface: Communication class interface
  $01 c, \ bSlaveInterface0: Data Class Interface
\ Endpoint 2 Descriptor
  7 c,   \ bLength: Endpoint Descriptor size
  $05 c, \ USB_ENDPOINT_DESCRIPTOR_TYPE
  $82 c, \ bEndpointAddress: (IN2)
  $03 c, \ bmAttributes: Interrupt
  8 c,   \ VIRTUAL_COM_PORT_INT_SIZE
  0 c,
  $FF c, \ bInterval:
\ Data class interface descriptor
  9 c,   \ bLength: Endpoint Descriptor size
  $04 c, \ USB_INTERFACE_DESCRIPTOR_TYPE
  $01 c, \ bInterfaceNumber: Number of Interface
  $00 c, \ bAlternateSetting: Alternate setting
  $02 c, \ bNumEndpoints: Two endpoints used
  $0A c, \ bInterfaceClass: CDC
  $00 c, \ bInterfaceSubClass:
  $00 c, \ bInterfaceProtocol:
  $00 c, \ iInterface:
\ Endpoint 3 Descriptor
  7 c,   \ bLength: Endpoint Descriptor size
  $05 c, \ USB_ENDPOINT_DESCRIPTOR_TYPE
  $03 c, \ bEndpointAddress: (OUT3)
  $02 c, \ bmAttributes: Bulk
  64 c,  \ VIRTUAL_COM_PORT_DATA_SIZE
  0 c,
  $00 c, \ bInterval: ignore for Bulk transfer
\ Endpoint 1 Descriptor
  7 c,   \ bLength: Endpoint Descriptor size
  $05 c, \ USB_ENDPOINT_DESCRIPTOR_TYPE
  $81 c, \ bEndpointAddress: (IN1)
  $02 c, \ bmAttributes: Bulk
  64 c,  \ VIRTUAL_COM_PORT_DATA_SIZE
  0 c,
  $00 c, \ bInterval
calign

create usb:langid
  4 c, 3 c,  \ USB_STRING_DESCRIPTOR_TYPE,
  $0409 w, \ LangID = U.S. English

create usb:vendor
  40 c, 3 c,  \ USB_STRING_DESCRIPTOR_TYPE,
  char M w, char e w, char c w, char r w, char i w, char s w, char p w,
  bl     w, char ( w, char S w, char T w, char M w, char 3 w, char 2 w,
  char F w, char 1 w, char 0 w, char x w, char ) w,

create usb:product
  36 c, 3 c,  \ USB_STRING_DESCRIPTOR_TYPE,
  char F w, char o w, char r w, char t w, char h w, bl     w, char S w,
  char e w, char r w, char i w, char a w, char l w, bl     w, char P w,
  char o w, char r w, char t w,

create usb:line
  hex 00 c, C2 c, 01 c, 00 c, 01 c, 00 c, 08 c, 00 c, decimal

create usb:init
hex
  0080 w,  \ ADDR0_TX   control - rx: 64b @ +040/080, tx: 64b @ +080/100
  0000 w,  \ COUNT0_TX
  0040 w,  \ ADDR0_RX
  8400 w,  \ COUNT0_RX
  00C0 w,  \ ADDR1_TX   bulk - tx: 64b @ +0C0/180
  0000 w,  \ COUNT1_TX
  0000 w,  \ ADDR1_RX
  0000 w,  \ COUNT1_RX
  0140 w,  \ ADDR2_TX   interrupt - tx: 8b @ +140/280
  0000 w,  \ COUNT2_TX
  0000 w,  \ ADDR2_RX
  0000 w,  \ COUNT2_RX
  0000 w,  \ ADDR3_TX   bulk - rx: 64b @ +100/200
  0000 w,  \ COUNT3_TX
  0100 w,  \ ADDR3_RX
  8400 w,  \ COUNT3_RX
decimal

((
$40005C00 constant USB
     USB $00 + constant USB-EP0R
     USB $40 + constant USB-CNTR
     USB $44 + constant USB-ISTR
     USB $48 + constant USB-FNR
     USB $4C + constant USB-DADDR
     USB $50 + constant USB-BTABLE
$40006000 constant USBMEM
))

$1FFF:F7AC equ _udevid	\ -- addr
\ *G Address holds 96 bit unique ID
$1FFF:F7CC equ _flashkb	\ -- addr
\ *G Holds the flash size in kilobytes.
: hwid		\ -- u
\ *G Returns a "fairly unique" hardware ID as single 32-bit number.
  _udevid dup @ over cell + @ xor over 2 cells + @ xor
;

: flash-kb ( -- u )  \ return size of flash memory in KB
  _flashkb w@ ;

: even		\ addr -- addr'
  1+ -2 and
;

: usb-pma ( pos -- addr ) dup 1 and negate swap 2* + _USBSRAM + ;
: usb-pma@ ( pos -- u ) usb-pma w@ ;
: usb-pma! ( u pos -- ) usb-pma w! ;

: ep-addr ( ep -- addr ) cells _USBdev usbEP0R + + ;
: ep-reg ( ep n -- addr ) 2* swap 8 * + usb-pma ;

\ endpoint status values, both TX and RX
0 equ epDISABLED
1 equ epSTALL
2 equ epNAK
3 equ epVALID

\ endpoint type, both TX and RX
0 equ epBULK
1 equ epCONTROL
2 equ epISO
3 equ epINTERRUPT


: rxstat!	\ ep u --
\ *G Set stat_rx without toggling/setting any other fields.
  swap ep-addr >r
  12 lshift  r@ w@ tuck  xor
\  R^rrseekT^ttnnnn
\  5432109876543210
  %0011000000000000 and swap
  %0000111100001111 and
  %1000000010000000 or
  or r> w! ;

: txstat!	\ ep u --
\ *G Set stat_tx without toggling/setting any other fields.
  swap ep-addr >r
  4 lshift  r@ w@ tuck  xor		\ -- orig mask ; R: -- addr
\  R^rrseekT^ttnnnn
\  5432109876543210
  %0000000000110000 and swap		\ -- mask' orig
  %0000111100001111 and			\ zero bits that toggle on 1
  %1000000010000000 or			\ set bits that clear on zero
  or r> w! ;

: ep-reset-rx# ( ep -- ) $8400 over 3 ep-reg w! 3 rxstat! ;
: rxclear	\ ep --
\ *G Clear the endpoint receive
cr ." rxc ep" dup .
  ep-addr dup w@ $0F8F and swap w! ;
: txclear ( ep -- ) ep-addr dup w@ $8F0F and swap w! ;
\ : rxclear ( ep -- ) ep-addr dup w@ $7FFF and $8F8F and swap w! ;
\ : txclear ( ep -- ) ep-addr dup w@ $FF7F and $8F8F and swap w! ;

2variable usb-pend
  0 0 usb-pend 2!
18 buffer: usb-serial

: set-serial ( -- addr )  \ fill serial number in as UTF-16 descriptor
  base @ hex
  hwid 0 <# 8 0 do # loop #> ( base addr 8 )
  0 do dup c@ i 1+ 2* usb-serial + w! 1+ loop
  drop base !
  usb-serial $0312 over w! ;

: send-data ( addr n -- )
cr ." > " over .word space dup .word space ." - "
  usb-pend 2! ;
: send-next ( -- )
  usb-pend 2@
over .word space dup .word ."  <" cr
   64 min $46 usb-pma@ min
  >r ( addr R: num )
  r@ even 0 ?do
    dup i + w@ $80 i + usb-pma!
  2 +loop drop
  r@ $02 usb-pma! 0 3 txstat!
  usb-pend 2@ r> dup negate d+ usb-pend 2! ;

: send-desc ( -- )
  $42 usb-pma@ case
    $0100 of usb:dev     18 endof
    $0200 of usb:conf    67 endof
    $0300 of usb:langid  4  endof
    $0301 of usb:vendor  40 endof
    $0302 of usb:product 36 endof
    $0303 of set-serial  18 endof
     0 0 rot
  endcase
  send-data ;

create usb:init
  $0080 w,  \ ADDR0_TX   control - rx: 64b @ +040/080, tx: 64b @ +080/100
  $0000 w,  \ COUNT0_TX
  $0040 w,  \ ADDR0_RX
  $8400 w,  \ COUNT0_RX
  $00C0 w,  \ ADDR1_TX   bulk - tx: 64b @ +0C0/180
  $0000 w,  \ COUNT1_TX
  $0000 w,  \ ADDR1_RX
  $0000 w,  \ COUNT1_RX
  $0140 w,  \ ADDR2_TX   interrupt - tx: 8b @ +140/280
  $0000 w,  \ COUNT2_TX
  $0000 w,  \ ADDR2_RX
  $0000 w,  \ COUNT2_RX
  $0000 w,  \ ADDR3_TX   bulk - rx: 64b @ +100/200
  $0000 w,  \ COUNT3_TX
  $0100 w,  \ ADDR3_RX
  $8400 w,  \ COUNT3_RX

: usb-reset ( -- )
  256 0 do  0 i 2* usb-pma! loop
  0 _USBdev usbBTABLE + w!
  usb:init  64 0 do
    dup w@  i _USBSRAM + w!
    2+
  4 +loop  drop
  $3210 0 ep-addr w!		\ RX valid, CONTROL, TX stall, EP0
  $0021 1 ep-addr w!		\ RX dis, BULK, TX NAK, EP1
  $0622 2 ep-addr w!		\ RX dis, INTERRUPT, TX NAK, EP2
  $3003 3 ep-addr w!		\ RX valid, BULK, TX dis, EP3
  $80 _USBdev usbDADDR + w! ;
((
: usb-reset ( -- )
  256 0 do  0 i 2* usb-pma! loop  0 USB-BTABLE h!
  usb:init  64 0 do
    dup h@  i USBMEM + h!
    2+
  4 +loop  drop
  $3210 0 ep-addr h!
  $0021 1 ep-addr h!
  $0622 2 ep-addr h!
  $3003 3 ep-addr h!
  $80 USB-DADDR h! ;
))
create zero 0 ,

128 4 + buffer: usb-in-ring   \ RX ring buffer, ample for mecrisp input lines
 64 4 + buffer: usb-out-ring  \ TX ring buffer, for outbound bytes

: ep-setup	\ ep --
\ *G Handle a setup packet, sent from host to config this device.
\  dup rxclear
  $41 usb-pma c@ case
    $00 of zero 2 send-data endof
    $06 of send-desc endof
    ( default ) 0 0 send-data
  endcase
  dup rxclear
  ep-reset-rx#
  send-next ;

variable tx.pend
  tx.pend off
variable usb.ticks
  usb.ticks off

: usb-pma-c! ( b pos -- )  \ careful, can't write high bytes separately
  dup 1 and if
    1- dup usb-pma@ rot 8 lshift or swap
  then usb-pma! ;

: usb-fill ( -- )  \ fill the USB outbound buffer from the TX ring buffer
  UsbOutQ0 cqchars ?dup if
\  usb-out-ring ring# ?dup if
    dup tx.pend !
    dup 0 do
      UsbOutQ0 (cqueue>) $C0 i + usb-pma-c!
\      usb-out-ring ring> $C0 i + usb-pma-c!
    loop
    1 1 ep-reg w!  1 3 txstat!
  then ;

: ep-out	\ ep --
\ *G Outgoing packets, sent from host to this device
cr ." ep-out "
\ dup 2 rxstat!  \ set RX state to NAK
  dup if  \ only pick up data for endpoint 3
    UsbInpQ0 cqchars 60 > if  drop exit  then \ reject if no room in ring
\    usb-in-ring ring# 60 > if drop exit then  \ reject if no room in ring
    dup 3 ep-reg w@ $7F and 0 ?do
      i $100 + usb-pma c@ UsbInpQ0 (>cqueue)
\      i $100 + usb-pma c@ usb-in-ring >ring
    loop
  then
  dup rxclear
  ep-reset-rx# ;

: ep-in		\ ep --
\ *G Incoming polls, sent from this device to host
cr ." ep-in "
  dup if				\ -- ep ; EP1..7
    0 usb.ticks !  0 tx.pend !  usb-fill
  else					\ -- ep ; EP0
    $41 usb-pma c@ $05 = if
      $42 usb-pma@ $80 or _USBdev usbDADDR + w!
    then
    send-next
  then
  txclear ;

: usb-ctr	\ istr --
cr ." usb-ctr "
  dup $07 and  swap $10 and if		\ -- ep ; OUT or SETUP packet
    dup ep-addr w@ $800 and
    if  ep-setup  else  ep-out then
  else
    ep-in
  then ;

: usb-poll
\ *G Main USB driver polling.
\    tx.pend @ if			\ if TX pending
\      1 usb.ticks +!
\      usb.ticks @ 10000 u>		\ waited 10 secs?
\      if  initVCOM0  then		\ clear buffers
\    then
\ main poll
  _USBdev usbISTR + w@
$9E1F and dup
if  cr dup .word  then
  dup $8000 and if			\ good endpoint transfer
    dup usb-ctr
  then
  dup $0200 and if			\ bit 9; SOF
    tx.pend @ if			\ if TX pending
      1 usb.ticks +!
      usb.ticks @ 10000 u>		\ waited 10 secs?
      if  initVCOM0  then		\ clear buffers
    then
    $FDFF _USBdev usbISTR + w!		\ clear interrupt
  then
  dup $0400 and if			\ bit 10; USB reset
    usb-reset
    $FBFF _USBdev usbISTR + w!		\ clear interrupt
  then
  dup $0800 and if			\ bit 11; USB suspend
    %1100 _USBdev usbCNTR + wor!	\ suspend and low power modes
    $F7FF _USBdev usbISTR + w!		\ clear interrupt
  then
  $1000 and if				\ bit 12; USB wake up
    %1000 _USBdev usbCNTR + wbic!
    $EFFF _USBdev usbISTR + w!		\ clear interrupt
  then ;

((
: usb-key? ( -- f )  pause usb-poll usb-in-ring ring# 0<> ;
: usb-key ( -- c )  begin usb-key? until  usb-in-ring ring> ;
: usb-emit? ( -- f )  usb-poll usb-out-ring ring? ;
: usb-emit ( c -- )  begin usb-emit? until  usb-out-ring >ring
                     tx.pend @ 0= if usb-fill then ;
))

((
: USBinterrupt	\ --
\ *G The USB interrupt handler. It calls *\fo{usb-poll} above to
\ ** permit better factoring.
  usbLED1on
  usb-poll
  usbLED1off
;
' USBinterrupt USB_EXTI_18vec# EXC: USB_ISR	\ -- addr
\ *G The entry point for the USB interrupt.

: usbIntInit	\ --
\ *G Initialise USB hardware interrupt handler.
  $9E00 _USBdev usbCNTR + w! 		\ enable USB ints
  USB_EXTI_18vec# EnInt			\ enable USB handler
;
))

: usb-io	\ --
\ *G Start up USB
  usbHWinit usb-reset
  initVCOM0
\  usbIntInit
  usbHWconnect
  $9E00 _USBdev usbCNTR + or!
  1000000 0 do  usb-poll  key? ?leave  loop
;

: +usb	usb-io  ;
: -usb  usbHWdisconnect  ;

: WDUMP		\ addr len -- ; dump 16 bit half words
\ *G Display (dump) len bytes of memory starting at addr
\ ** as 16 bit half-words.
[defined] FlushKeys [if] FlushKeys [then]	\ SFP002
  bounds ?do				\ get limits
    cr  i .lword  space   i 8 + i       \ print address
    do  space i w@ .word  2 +loop	\ first eight bytes
    space  i $010 + i 8 +               \ gap
    do  space i w@ .word  2 +loop	\ second eight bytes
\    2 spaces  i $010 + i                \ gap
\    do  i w@ dup $FF and .ascii  8 rshift .ascii  2 +loop  \ ascii o/p
    halt? ?leave			\ quit or halt
  $010 +loop                            \ until done
  cr  					\ restore base
;

: .usb		\ offset len --
  swap _USBSRAM + swap wdump
;

: .usb1		0 $80 .usb  ;
: .dev		_usbdev $100 pdump ;


\ ======
\ *> ###
\ ======

decimal

