\ UsbDefs.fth - Generic USB defines

((
Copyright (c) 2008, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20100906 SFP002 Added USBmin?.
20081114 SFP001 Added NEXTUIF: and friends.
))

only forth definitions
decimal

\ ==========
\ *! usbdefs
\ *T Generic USB definitions
\ =========
\ *P The file *\i{UsbDefs.fth} contains generic USB defines and tools
\ ** used to insulate the hardware drivers *\i{UsbHwxxxx.fth} from the
\ ** core layers in *\i{UsbBase.fth} and *\i{UsbEP0.fth}.

[undefined] USBmin? [if]
0 equ USBMin?	\ -- flag
\ *G If your control file sets this equate non-zero, much of the
\ ** USB code is compiled without headers.
[then]
USBmin? [if]  internal  [then]

interpreter
: byte	 1 field  ;
: hword  2 field  ;
target

struct /USBSetupPacket	\ -- len
\ *G USB Default Control Pipe Setup Packet structure.
\ *[
  byte  usp.bmRequestType	\ bit7=Dir,0=to dev, bit6:5=Type, bit4:0=Recipient
  byte  usp.bRequest
  hword usp.wValue
  hword usp.wIndex
  hword usp.wLength
end-struct
\ *]

\ bmRequestType.Dir
#define REQUEST_HOST_TO_DEVICE     0
#define REQUEST_DEVICE_TO_HOST     1
$80 equ dirIN
$00 equ dirOUT

\ bmRequestType.Type
3 5 lshift equ ReqTypeMask
0 5 lshift equ ReqTypeStd
1 5 lshift equ ReqTypeClass
2 5 lshift equ ReqTypeVendor
3 5 lshift equ ReqTypeRes

\ bmRequestType.Recipient
$01F equ ReqRecipMask
#define REQUEST_TO_DEVICE          0
#define REQUEST_TO_INTERFACE       1
#define REQUEST_TO_ENDPOINT        2
#define REQUEST_TO_OTHER           3

\ USB Standard Request Codes
#define USB_REQUEST_GET_STATUS                 0
#define USB_REQUEST_CLEAR_FEATURE              1
#define USB_REQUEST_SET_FEATURE                3
#define USB_REQUEST_SET_ADDRESS                5
#define USB_REQUEST_GET_DESCRIPTOR             6
#define USB_REQUEST_SET_DESCRIPTOR             7
#define USB_REQUEST_GET_CONFIGURATION          8
#define USB_REQUEST_SET_CONFIGURATION          9
#define USB_REQUEST_GET_INTERFACE              10
#define USB_REQUEST_SET_INTERFACE              11
#define USB_REQUEST_SYNC_FRAME                 12

\ USB GET_STATUS Bit Values
#define USB_GETSTATUS_SELF_POWERED             0x01
#define USB_GETSTATUS_REMOTE_WAKEUP            0x02
#define USB_GETSTATUS_ENDPOINT_STALL           0x01

\ USB Standard Feature selectors
#define USB_FEATURE_ENDPOINT_STALL             0
#define USB_FEATURE_REMOTE_WAKEUP              1


\ USB Descriptor Types
1 equ USBDevDesc#
2 equ USBCfgDesc#
3 equ USBStrDesc#
4 equ USBIfaceDesc#
5 equ USBEPDesc#
6 equ USBDevQualDesc#
7 equ USBOtherSpdCfgDesc#
8 equ USBIfacePowerDesc#

\ USB Device Classes
#define USB_DEVICE_CLASS_RESERVED              0x00
#define USB_DEVICE_CLASS_AUDIO                 0x01
#define USB_DEVICE_CLASS_COMMUNICATIONS        0x02
#define USB_DEVICE_CLASS_HUMAN_INTERFACE       0x03
#define USB_DEVICE_CLASS_MONITOR               0x04
#define USB_DEVICE_CLASS_PHYSICAL_INTERFACE    0x05
#define USB_DEVICE_CLASS_POWER                 0x06
#define USB_DEVICE_CLASS_PRINTER               0x07
#define USB_DEVICE_CLASS_STORAGE               0x08
#define USB_DEVICE_CLASS_HUB                   0x09
#define USB_DEVICE_CLASS_VENDOR_SPECIFIC       0xFF

\ bmAttributes in Configuration Descriptor
#define USB_CONFIG_POWERED_MASK                0xC0
#define USB_CONFIG_BUS_POWERED                 0x80
#define USB_CONFIG_SELF_POWERED                0x40
#define USB_CONFIG_REMOTE_WAKEUP               0x20

\ bEndpointAddress in Endpoint Descriptor
#define USB_ENDPOINT_DIRECTION_MASK            0x80

\ MSC Request Codes
#define MSC_REQUEST_RESET               0xFF
#define MSC_REQUEST_GET_MAX_LUN         0xFE

\ CDC class-specific request codes
\ (usbcdc11.pdf, 6.2, Table 46)
\ see Table 45 for info about the specific requests.
#define SEND_ENCAPSULATED_COMMAND           0x00
#define GET_ENCAPSULATED_RESPONSE           0x01
#define SET_COMM_FEATURE                    0x02
#define GET_COMM_FEATURE                    0x03
#define CLEAR_COMM_FEATURE                  0x04
#define SET_AUX_LINE_STATE                  0x10
#define SET_HOOK_STATE                      0x11
#define PULSE_SETUP                         0x12
#define SEND_PULSE                          0x13
#define SET_PULSE_TIME                      0x14
#define RING_AUX_JACK                       0x15
#define SET_LINE_CODING                     0x20
#define GET_LINE_CODING                     0x21
#define SET_CONTROL_LINE_STATE              0x22
#define SEND_BREAK                          0x23
#define SET_RINGER_PARMS                    0x30
#define GET_RINGER_PARMS                    0x31
#define SET_OPERATION_PARMS                 0x32
#define GET_OPERATION_PARMS                 0x33
#define SET_LINE_PARMS                      0x34
#define GET_LINE_PARMS                      0x35
#define DIAL_DIGITS                         0x36
#define SET_UNIT_PARAMETER                  0x37
#define GET_UNIT_PARAMETER                  0x38
#define CLEAR_UNIT_PARAMETER                0x39
#define GET_PROFILE                         0x3A
#define SET_ETHERNET_MULTICAST_FILTERS      0x40
#define SET_ETHERNET_PMP_FILTER             0x41
#define GET_ETHERNET_PMP_FILTER             0x42
#define SET_ETHERNET_PACKET_FILTER          0x43
#define GET_ETHERNET_STATISTIC              0x44
#define SET_ATM_DATA_FORMAT                 0x50
#define GET_ATM_DEVICE_STATISTICS           0x51
#define SET_ATM_DEFAULT_VC                  0x52
#define GET_ATM_VC_STATISTICS               0x53


struct /UsbEpd	\ -- len
\ *G USB Standard Endpoint Descriptor
\ *[
  BYTE  epd.Len
  BYTE  epd.Type
  BYTE  epd.Addr
  BYTE  epd.Attribs
  HWORD epd./MaxPacket
  BYTE  epd.Interval
end-struct
\ *]


\ ****************
\ *S Support tools
\ ****************

compiler
: bitn		\ n -- 2^n
\ *G Generate bit n from n.
  1 swap lshift
;
target

aligning? Big-Endian? or [if]
compiler
: lew@		\ addr -- w
\ *G 16 bit unaligned little-endian fetch.
\ ** USB control data is in little-endian form and may be unaligned.
  dup c@ swap 1+ c@ 8 lshift or  ;
: lew!		\ w addr --
\ *G 16 bit unaligned little endian store.
\ ** USB control data is in little-endian form and may be unaligned.
  2dup c!  swap 8 lshift swap 1+ c!  ;
: lel@		\ addr -- x
\ *G 32 bit unaligned little-endian fetch.
  dup c@  over 1 + c@ 8 lshift or
  over 2+ c@ #16 lshift or  swap 3 + c@ #24 lshift or  ;
: lel!		\ x addr --
\ *G 32 bit unaligned little endian store.
  2dup c!  swap 8 rshift swap  2dup 1 + c!
  swap 8 rshift swap  2dup 2 + c!  swap 8 rshift swap  3 + c!  ;
target
[else]
compiler
: lew@	w@  ;
: lew!  w!  ;
: lel@  @  ;
: lel!  !  ;
target
[then]

aligning? Little-Endian? or [if]
compiler
: bew@		\ addr -- w
  dup c@ 8 lshift  swap 1+ c@ or  ;
: bew!		\ w addr --
  over 8 rshift over  c!  1+ c!  ;
: bel@		\ addr -- x
  dup c@  8 lshift over 1 + c@ or
  8 lshift over 2 + c@ or  8 lshift swap 3 + c@ or  ;
: bel!		\ x addr --
  swap 8 rol tuck over c!    swap 8 rol tuck over 1 + c!
  swap 8 rol tuck over 2 + c!    swap 8 rol swap 3 + c!  ;
target
[else]
compiler
: bew@  w@  ;
: bew!  w!  ;
: bel@  @  ;
: bel!  !  ;
target
[then]

host
: USB$,		\ --
\ Host action to comma a USB string into the dictionary.
  [char] " word count
  dup 2* 2+ c,c(t)  3 c,c(t)		\ length in bytes, string type
  bounds
  ?do  i c@ c,c(t)  0 c,c(t)  loop	\ little-endian Unicode chars
;
target

interpreter
: lew@		\ addr -- w
\ *G 16 bit fetch.
\ ** USB control data is in little-endian form and may be unaligned.
  dup c@ swap 1+ c@ 8 lshift or
;

: lew!		\ w addr --
\ *G 16 bit store.
\ ** USB control data is in little-endian form and may be unaligned.
  2dup c!  swap 8 lshift swap 1+ c!
;

: lew,		\ w --
\ *G Lay unaligned little-endian 16 bit data.
  dup c,  8 rshift c,
;

: lel@		\ addr -- x
\ *G 32 bit unaligned little-endian fetch.
  dup c@  over 1 + c@ 8 lshift or
  over 2+ c@ #16 lshift or  swap 3 + c@ #24 lshift or
;

: lel!		\ x addr --
\ *G 32 bit unaligned little endian store.
  2dup c!  swap 8 rshift swap  2dup 1 + c!
  swap 8 rshift swap  2dup 2 + c!  swap 8 rshift swap  3 + c!
;

: lel,		\ x --
\ *G Lay unaligned little-endian 32 bit data.
  dup c,  8 rshift dup  c,  8 rshift dup  c,  8 rshift c,
;

: bew@		\ addr -- w
\ *G Unaligned big-endian 16 bit fetch.
  dup c@ 8 lshift  swap 1+ c@ or
;

: bew!		\ w addr --
\ *G Unaligned big-endian 16 bit store.
  over 8 rshift over  c!  1+ c!  ;

: bew,		\ w --
\ *G Lay unaligned big-endian 16 bit data.
  dup 8 rshift c,  c,
;

: bel@		\ addr -- x
\ *G Unaligned big-endian 32 bit fetch.
  dup c@  8 lshift over 1 + c@ or
  8 lshift over 2 + c@ or  8 lshift swap 3 + c@ or
;

: bel!		\ x addr --
\ *G Unaligned big-endian 32 bit store.
  swap 8 rol tuck over c!    swap 8 rol tuck over 1 + c!
  swap 8 rol tuck over 2 + c!    swap 8 rol swap 3 + c!
;

: bel,		\ x --
\ *G Lay unaligned big-endian 32 bit data.
  dup #24 rshift c,  dup #16 rshift c,  dup 8 rshift c,  c,
;

: USB",		USB$,  ;
\ *P Strings are laid as little-endian 16 bit Unicode, preceded by
\ ** a count byte and a string type marker.
target

host
variable NextUIF	\ -- addr

: resetUIFs	\ --
  0 NextUIF !
;

: stepUIF	\ -- if#
  NextUIF @  NextUIF incr
;

: #UIFs		\ -- u
  NextUIF @
;
target

interpreter
: resetUIFs	\ --
\ *G Start the USB interface sequence numbering at 0.
  resetUIFs  ;

: NextUIF:	\ "<name>" --
\ *G Create an equate with the next USB interface number.
  stepUIF equ
;

: #UIFs		\ -- u
\ *G Returns the number of interfaces defined so far.
  #UIFs
;
target


\ *********************
\ *S Hardware isolation
\ *********************
\ *P The five *\fo{VALUE}s below are used to provide status information
\ ** for the core layer, but are actually contained here in *\i{UsbDefs.fth}
\ ** to avoid forward references and to permit easy reference by the
\ ** class drivers. Note that the endpoint status values are bit masks
\ ** in a hardware independent form. See *\fo{EP>mask} below.

0 value DevAddr		\ -- x
\ *G Set to $80+u where u is the new device address. Used by
\ ** low level code to set the USB device address at a time
\ ** suitable for the hardware. After setting *\fo{UsbDevAddr}
\ ** below, *\fo{DevAddr} is reset to 0.
0 value UsbDevAddr	\ -- u
\ *G USB device address. Set by the host.
0 value UsbConfig	\ -- x
\ *G USB Configuration set. Non-zero when the USB device has been
\ ** configured.
0 value UsbEPmask	\ -- x
\ *G Endpoint mask. Bits are set when the endpoint is enabled. Used
\ ** by the core layer.
0 value UsbEPhalt	\ -- x
\ *G Endpoint halt status. Bits are set when the endpoint is halted/stalled.
0 value Usb#IfSet	\ -- n
\ *G Number of interfaces set.

: EP>mask	\ ep# -- mask
\ *G Decode an endpoint number to a bit mask such that if bit 7
\ ** is clear (OUT endpoints), the bit is in the low 16 bits, and
\ ** if set (IN endpoints) in the upper 16 bits.
  dup $0F and  swap $80 and		\ if bit 7 set (IN endpoint)
  if  #16 +  endif			\ shift by 16 more bits
  1 swap lshift				\ -- mask
;
interpreter
: EP>mask	\ ep# -- mask
  dup $0F and  swap $80 and		\ if bit 7 set (IN endpoint)
  if  #16 +  endif			\ shift by 16 more bits
  1 swap lshift				\ -- mask
;
target


\ ***************
\ *S Linked lists
\ ***************
\ *P Linked lists are used by the USB driver to avoid forward
\ ** references to higher level code, especially various reset
\ ** actions. Note that the chains are managed at compile
\ ** time. The linked list uses facilities provided in
\ ** *\i{Common\Kernel62.fth}.

[undefined] ExecChain [if]
: ExecChain	\ anchor --
\ *G Execute the contents of chain with the following structure:
\ *C   link | xt | ...
\ *P Each word that is run has the stack effect
\ *C   link -- link
\ *P Where link is the address of the link field in the structure.
\ ** Thus, data that follows the xt can easily be accessed.
  @
  begin
    dup
  while
    dup cell + @ execute
    @
  repeat
  drop
;
[then]

create UsbResetChain	\ -- addr
\ *G Anchors the chain of words executed at device reset.
  0 ,

create UsbFrameChain	\ -- addr
\ *G Anchors the chain of words executed in the frame interrupt.
  0 ,

interpreter
: link,		\ addr --
\ *G Extend chain anchored at *\i{addr}.
  here  over @ ,  swap !
;

: AtUSBreset	\ xt --
\ *G Add the word whose *\i{xt} is given to the USB reset chain.
  align  UsbResetChain link,  ,
;

: AtUSBframe	\ xt --
\ *G Add the word whose *\i{xt} is given to the USB frame
\ ** interrupt chain. Words in this chain are mostly used to
\ ** enable Bulk or Interrupt endpoint NAK interrupts. Careful
\ ** use of this chain can significantly reduce USB interrupt
\ ** handling overhead.
  align  UsbFrameChain link,  ,
;
target


\ ======
\ *> ###
\ ======

decimal
