\ devmcb2388.fth - USB set up for Keil MCB2300/2388 board

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

BC mod from devLCI2148.fth

Make sure ETM jumper is NOT installed it will make pin2.9 not
work for soft connect

This uses the standard size usb-b connector

Jumpers umode:toward LCD, d-,d+ :toward speaker

LEDs not quite right - work but don't flash visibly

To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! devmcb2388
\ *T USB set up for Keil MCB2388
\ ===========
\ *P The file *\i{devMCB2388.fth} provides USB initialisation
\ ** and LED drivers for a Keil MCB2300 board with an LPC2388.
\ ** It will require minor changes for other LPC devices and
\ ** boards, in particular in the USB Connect function and LEDs.


\ **********************
\ *S USB interrupt level
\ **********************

4 equ USBpri#	\ -- u
\ *G Slot/Priority (0..15, 0=high) used for the USB interrupt handler.


\ *******
\ *S LEDs
\ *******
\ *P The USB software uses two LEDs to indicate USB activity
\ ** (LED1) and disk activity (LED2).
\ ** LED1 is turned on during the interrupt service routine.

bit18 equ LED1mask			\ P1.18, PINSEL3, UGL
0 equ LED2mask				\ not present
LED1mask equ LEDiomask			\ P1.18 only
0 [ps  18 +ps  ps]
  equ LEDpinval  equ LEDpinmask		\ bits in PINSEL3.

: initLEDs	\ --
\ *G Initialise the two LEDs to off, output=high.
  bit0 _SYSCON SCS + or!		\ enable fast I/O for GPIO0
  LEDpinval LEDpinmask _PINSEL3 setMask	\ configure I/O
  LEDIOmask _FIO1
  2dup FIODIR + or!			\ set DIR bits
  FIOSET + !				\ set high
;
' initLEDs AtCold

: usbLED1on	\ --
\ *G Turn LED1 on.
  LED1mask _FIO1 FIOCLR + !  ;
: usbLED1off	\ --
\ *G Turn LED1 off.
  LED1mask _FIO1 FIOSET + !  ;
compiler
: usbLED1on	\ --
  LED1mask _FIO1 FIOCLR + !  ;
: usbLED1off	\ --
  LED1mask _FIO1 FIOSET + !  ;
: usbLED2on	\ --
  ;
: usbLED2off	\ --
  ;
target


\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary LPC2xxx USB setup is performed in
\ ** *\i{UsbHwLPC2xxx.fth}. Because LPC2xxx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.

  1 [ps 29 +ps  30 +ps ps]  equ USBps1val  equ USBps1mask
\ usb data pins pinsel1
  1 [ps 30 +ps ps]  equ USBps3val  equ USBps3mask  \ pwrdetect
  1 [ps 9 +ps  ps]  equ USBps4val  equ USBps4mask   \ usbconnect

: USBhwInit	\ --
\ *G Enable access to the USB peripheral.
\ ** The USB clock PLL is set in the start up code.
  ~pcusb _PCONP  or!  \ usb power on
  begin               \ turn on clocks
    _USB
    $012 over USBClkCtrl + !		\ enable AHB, USB device clocks
  \ begin					\ wait for all to settle
    pause   USBClkSt + @
    $012 and $012 =
  until
  $2000:0000  $3000:0000  _pinsel pinmode3 + setmask \ pu disabled on p1.30
  USBps1val  USBps1mask  _PINSEL1 setMask	 \ select pins
  USBps3val  USBps3mask  _PINSEL3 setMask
  USBps4val  USBps4mask  _PINSEL4 setMask  \ soft connect output p2.9
;
((  \ not needed if software connect works right
: USBhwconnect	\ --
\ *G Connect to the USB bus.
;
: USBhwdisconnect	\ --
\ *G Disconnect from the USB bus.
;
))

\ ======
\ *> ###
\ ======

decimal

