\ devOli1766D.fth - USB set up for Olimex LPC-1766STK board

((
Copyright (c) 2008, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20110817 SFP001 Conversion to device connection of Olimex
                LPC-1766STK board.
))

only forth definitions
decimal

\ ===========
\ *! devoli1766d
\ *T USB device set up for Olimex LPC-1766STK
\ ===========
\ *P The file *\i{devOli1766D.fth} provides USB device mode
\ ** initialisation and LED drivers for an Olimex LPC-1766STK
\ ** board. It will require minor changes for other LPC devices
\ ** and boards, in particular in the USB Connect function and
\ ** LEDs.


\ *******
\ *S LEDs
\ *******
\ *P The USB software uses two LEDs to indicate USB activity
\ ** (LED1, P1.25) and disk activity (LED2, P0.4).
\ ** LED1 is turned on during the interrupt service routine.

0 [ps  25 +ps  ps] equ LED1val  equ LED1mask	\ bits in PINSEL3
bit25 equ LED1bit				\ bit in FIO1
0 [ps  4 +ps  ps] equ LED2val  equ LED2mask	\ bits in PINSEL0
bit4 equ LED2bit				\ bit in FIO1

: initLEDs	\ --
\ *G Initialise the two LEDs to off, output=high.
  LED1val LED1mask  _PINSEL3 setMask	\ configure LED1
  LED2val LED2mask  _PINSEL0 setMask	\ configure LED2
  LED1bit _FIO1
  2dup FIODIR + or!			\ set DIR bits
  FIOSET + !				\ set high
  LED2bit _FIO0
  2dup FIODIR + or!			\ set DIR bits
  FIOSET + !				\ set high
;
' initLEDs AtCold

: usbLED1on	\ --
\ *G Turn LED1 on.
  LED1bit _FIO1 FIOCLR + !  ;
: usbLED1off	\ --
\ *G Turn LED1 off.
  LED1bit _FIO1 FIOSET + !  ;
: usbLED2on	\ --
\ *G Turn LED2 on.
  LED2bit _FIO0 FIOCLR + !  ;
: usbLED2off	\ --
\ *G Turn LED2 off.
  LED2bit _FIO0 FIOSET + !  ;


\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary LPC2xxx USB setup is performed in
\ ** *\i{UsbHwLPC2xxx.fth}. Because LPC2xxx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.
\ *E USB_UP_LED   P1.18
\ ** USB_CONNECT  P2.9
\ ** VBUS         P1.30
\ ** D+           P0.29
\ ** D-           P0.30

  1 [ps 29 +ps  30 +ps ps]  equ USBps1val  equ USBps1mask	\ PINSEL1
\ usb data pins pinsel1
  1 [ps 18 +ps  30 +ps ps]  equ USBps3val  equ USBps3mask 	\ PINSEL3
  1 [ps 9 +ps  ps]  equ USBps4val  equ USBps4mask   		\ PINSEL4

: USBhwInit	\ --
\ *G Enable access to the USB peripheral.
\ ** The USB clock PLL is set in the start up code.
  ~pcusb _PCONP  or!			\ usb power on

  begin					\ turn on clocks
    $012 _USB USBClkCtrl + !		\ enable AHB, USB device clocks
    pause
    _USB USBClkSt + @			\ check status
    $012 and $012 =
  until

  _SYSCON				\ power up and configure 48Mhz PLL
  USBCLKval over USBCLKCFG + !		\ USB clock divider from PLL0
  [i
  0 over PLL1CON + !			\ disable PLL1
  $AA over PLL1FEED + !			\ feed sequence
  $55 over PLL1FEED + !
  i]
  drop

  $2000:0000  $3000:0000  _pinsel pinmode3 + setmask \ pu disabled on p1.30
  USBps1val  USBps1mask  _PINSEL1 setMask	\ select pins
  USBps3val  USBps3mask  _PINSEL3 setMask
  USBps4val  USBps4mask  _PINSEL4 setMask	\ soft connect output p2.9
;

((  \ not needed if software connect works right
: USBhwconnect	\ --
\ *G Connect to the USB bus.
;
: USBhwdisconnect	\ --
\ *G Disconnect from the USB bus.
;
))

\ ======
\ *> ###
\ ======

decimal

