\ CODEARM.FTH - ARM7+ code file

((
Copyright (c) 2000, 2002, 2003, 2004, 2007, 2008, 2009, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
Recode FILL to check for word start address, and copy words if possible.
This change may have a good speed/size trade-off.

Change history
==============
20160122 SFP022 Reserved R8 as float stack pointer FSP.
20090219 SFP021 Enhanced UM/MOD to full 64/32 operation.
		Minor DocGen enhancements.
20081016 SFP020 Moved mul/div code to avoid forward references.
20081007 SFP019 Added SETMASK.
20080816 SFP018 Recoded M+.
20080401 SFP017 Corrected D<.
20070302 MPE016 Recoded CMOVE and CMOVE> for speed.
20040923 MPE015 Made C_?OF C_END-CASE C_NEXTCASE only compiled
		when the equate FULLCASE? is non-zero.
20040206 MPE014 Added target long jumps/calls for standalone
		compilation in kernels with ROM and RAM more
		than 32Mb apart, e.g. Philips LPC210x.
		Overhauled and retested. Corrected USER.
20030109 SBD013 Corrected sign extension in M+.
20021010 SFP012 Removed Cache synchronisation from comma words, as this
		is now done by QUIT and EXIT.
20020916 SFP011 Corrected >NAME for endian order sensitivity
20020628 SFP010 Started DOCGEN additions.
20000620 SFP009 Matched USER to updated compiler [build 561]
20000608 SFP008 Converted to v6.1 generic I/O model
))

\ ==========
\ *! codearm
\ *T ARM code definitions
\ ==========

\ ********
\ *S Notes
\ ********
\ *P Some words and code routines are marked in the documentation
\ ** as *\fo{INTERNAL}. These are factors used by other words and do
\ ** not have dictionary entries in the standalone Forth. They
\ ** are only accessible to users of the VFX Forth ARM Cross
\ ** Compiler. This also applies to definitions of the form:
\ *C n EQU <name>
\ *C PROC <name>
\ *C L: <name>

\ *****************
\ *S Register usage
\ *****************
\ *P On the ARM the following register usage is the default:
\ *E   r15         pc      program counter
\ **   r14         link    link register
\ **   r13         rsp     return stack pointer
\ **   r12         psp     data stack pointer
\ **   r11         up      user area pointer
\ **   r10         tos     cached top of stack
\ **   r9          lp      locals pointer
\ **   r8          fsp     float stack pointer
\ **   r0-r7       scratch
\ *P The VFX optimiser reserves R0 and R1 for internal operations.
\ ** *\fo{CODE} definitions must use R10 as TOS with NOS pointed to by
\ ** R12 as a full descending stack in ARM terminology. R0..R8 are
\ ** free for use by *\fo{CODE} definitions and need not be preserved or
\ ** restored. You should assume that any register can be affected
\ ** by other words.


\ ****************
\ *S Configuration
\ ****************
\ *P These equates are set false (zero) if they have not already
\ ** been defined.

[undefined] HIGH-LEVEL-ROLL [if]
false equ HIGH-LEVEL-ROLL	\ -- flag
\ *G Set this equate true to compile a high level version of
\ ** *\fo{ROLL}.
[then]

[undefined] CLZ? [if]
false equ CLZ?	\ -- flag
\ *G Set this non-zero to compile *\fo{CLZ}.
[then]

[undefined] DSQRT? [if]
false equ DSQRT?	\ -- flag
\ *G Set this non-zero to compile *\fo{DSQRT}.
[then]

[undefined] FastCmove? [if]
false equ FastCmove?	\ -- flag
\ *G Set this flag true to use a fast but vast (~1kb) version of
\ ** *\fo{CMOVE}. If your application uses either *\fo{CMOVE} or
\ ** *\fo{MOVE} in time-critical code, the fast version offers
\ ** an overall speed up of about four times. The code for the
\ ** fast but vast version was written by Rowley Associates,
\ ** whose permission to adapt and publish the code with the
\ ** MPE cross compiler is much appreciated.
[then]

only forth definitions
decimal


\ ***********************************
\ *S Logical and relational operators
\ ***********************************

CODE AND	\ x1 x2 -- x3
\ *G Perform a logical AND between the top two stack items and retain
\ ** the result in top of stack.
  ldmfd psp ! { r0 }
  and tos, tos, r0
  next,
END-CODE

CODE OR		\ x1 x2 -- x3
\ *G Perform a logical OR between the top two stack items and retain
\ ** the result in top of stack.
  ldmfd psp ! { r0 }
  orr tos, tos, r0
  next,
END-CODE

CODE XOR	\ x1 x2 -- x3
\ *G Perform a logical XOR between the top two stack items and retain
\ ** the result in top of stack.
  ldmfd psp ! { r0 }
  eor tos, tos, r0
  next,
END-CODE

CODE NOT 	\ x -- x'
\ *G Perform a bitwise NOT on the top stack item and retain result.
  mvn tos, tos
  next,
END-CODE

CODE INVERT 	\ x -- x'
\ *G Perform a bitwise NOT on the top stack item and retain result.
  mvn tos, tos
  next,
END-CODE

CODE 0= 	\ x -- flag
\ *G Compare the top stack item with 0 and return TRUE if equals.
  mov .s tos, tos
  mvn .eq tos, # 0
  mov .ne tos, # 0
  next,
END-CODE

CODE 0<> 	\ x -- flag
\ *G Compare the top stack item with 0 and return TRUE if not-equal.
  mov .s tos, tos
  mvn .ne tos, # 0
  mov .eq tos, # 0
  next,
END-CODE

CODE 0< 	\ x -- flag
\ *G Return TRUE if the top of stack is less-than-zero.
  mov .s tos, tos
  mvn .mi tos, # 0
  mov .pl tos, # 0
  next,
END-CODE

CODE 0> 	\ x -- flag
\ *G Return TRUE if the top of stack is greater-than-zero.
  cmp tos, # 0
  mvn .gt tos, # 0
  mov .le tos, # 0
  next,
END-CODE

CODE = 		\ x1 x2 -- flag
\ *G Return TRUE if the two topmost stack items are equal.
  ldmfd psp ! { r0 }
  cmp tos, r0
  mvn .eq tos, # 0
  mov .ne tos, # 0
  next,
END-CODE

CODE <> 	\ x1 x2 -- flag
\ *G Return TRUE if the two topmost stack items are different.
  ldmfd psp ! { r0 }
  cmp tos, r0
  mvn .ne tos, # 0
  mov .eq tos, # 0
  next,
END-CODE

CODE < 		\ n1 n2 -- flag
\ *G Return TRUE if n1 is less than n2.
  ldmfd psp ! { r0 }
  cmp r0, tos
  mvn .lt tos, # 0
  mov .ge tos, # 0
  next,
END-CODE

CODE > 		\ n1 n2 -- flag
\ *G Return TRUE if n1 is greater than n2.
  ldmfd psp ! { r0 }
  cmp r0, tos
  mvn .gt tos, # 0
  mov .le tos, # 0
  next,
END-CODE

CODE <= 	\ n1 n2 -- flag
\ *G Return TRUE if n1 is less than or equal to n2.
  ldmfd psp ! { r0 }
  cmp r0, tos
  mvn .le tos, # 0
  mov .gt tos, # 0
  next,
END-CODE

CODE >= 	\ x1 x2 -- flag
\ *G Return TRUE if n1 is greater than or equal to n2.
  ldmfd psp ! { r0 }
  cmp r0, tos
  mvn .ge tos, # 0
  mov .lt tos, # 0
  next,
END-CODE

CODE U> 	\ u2 u2 -- flag
\ *G An UNSIGNED version of >.
  ldmfd psp ! { r0 }
  cmp r0, tos
  mvn .hi tos, # 0
  mov .ls tos, # 0
  next,
END-CODE

CODE U< 	\ u1 u2 -- flag
\ *G An UNSIGNED version of <.
  ldmfd psp ! { r0 }
  cmp r0, tos
  mvn .lo tos, # 0
  mov .hs tos, # 0
  next,
END-CODE

CODE DU< 	\ ud1 ud2 -- flag
\ *G Returns true if ud1 (unsigned double) is less than ud2.
  ldmfd psp ! { r0-r2 }                 \ r0=d2 low, r1=d1 high, r2=d1 low
  sub .s r2, r2, r0                     \ d1l - d2l
  sbc .s r1, r1, tos                    \ d1h - d2h
  mvn .cc tos, # 0
  mov .cs tos, # 0
  next,
END-CODE

CODE D0< 	\ d -- flag
\ *G Returns true if signed double d is less than zero.
  add psp, psp, # 4                     \ lose low
  mov .s tos, tos
  mvn .mi tos, # 0
  mov .pl tos, # 0
  next,
END-CODE

CODE D0=	\ xd -- flag
\ *G Returns true if xd is 0.
  ldmfd psp ! { r0 }
  orr .s r0, r0, tos
  mvn .eq tos, # 0                      \ true
  mov .ne tos, # 0                      \ false
  next,
END-CODE

CODE D= 	\ xd1 xd2 -- flag
\ *G Return TRUE if the two double numbers are equal.
  ldmfd psp ! { r0-r2 }                 \ r0=d2 low, r1=d1 high, r2=d1 low
  sub .s r0, r0, r2                     \ d2l - d1l
  sbc tos, tos, r1                      \ d2h - d1h
  orr .s r0, r0, tos
  mvn .eq tos, # 0                      \ true
  mov .ne tos, # 0                      \ false
  next,
END-CODE

CODE D< 	\ d1 d2 -- flag
\ *G Return TRUE if the double number d1 is (signed) less than the
\ ** double number d2.
  ldmfd psp ! { r0-r2 }                 \ r0=d2 low, r1=d1 high, r2=d1 low
  sub .s r2, r2, r0                     \ d1l - d2l
  sbc .s r1, r1, tos			\ d1h - d2h
  mvn .lt tos, # 0                      \ true
  mov .ge tos, # 0                      \ false
\  sbc r1, r1, tos                       \ d1h - d2h
\  mov .s r1, r1
\  mvn .mi tos, # 0                      \ true
\  mov .pl tos, # 0                      \ false
  next,
END-CODE

CODE DMAX 	\ d1 d2 -- d3 ; d3=max of d1/d2
\ *G Return the maximum double number from the two supplied.
  ldmfd psp ! { r0-r1 }                 \ r0=d2 low, r1=d1 high, r2=d1 low
  cmp tos, r1                           \ check high
  str .gt r0, [ psp ]                   \ d2>d1
  mov .gt pc, link                      \ ret d2
  mov .lt tos, r1                       \ d2<d1
  mov .lt pc, link                      \ ret d1
  ldr r2, [ psp ]                       \ d2h=d1h
  cmp r0, r2                            \ check low
  mov .lt tos, r1                       \ d2<d1
  mov .lt pc, link                      \ ret d1
  str r0, [ psp ]                       \ d2>=d1
  next,                                 \ ret d2
END-CODE

CODE DMIN 	\ d1 d2 -- d3 ; d3=min of d1/d2
\ *G Return the minimum double number from the two supplied.
  ldmfd psp ! { r0-r1 }                 \ r0=d2 low, r1=d1 high, r2=d1 low
  cmp tos, r1                           \ check high
  str .lt r0, [ psp ]                   \ d2<d1
  mov .lt pc, link                      \ ret d2
  mov .gt tos, r1                       \ d2>d1
  mov .gt pc, link                      \ ret d1
  ldr r2, [ psp ]                       \ d2h=d1h
  cmp r0, r2                            \ check low
  mov .gt tos, r1                       \ d2>d1
  mov .gt pc, link                      \ ret d1
  str r0, [ psp ]                       \ d2<=d1
  next,
END-CODE

CODE MIN	\ n1 n2 -- n1|n2
\ *G Given two data stack items preserve only the smaller.
  ldmfd psp ! { r0 }
  cmp tos, r0
  mov .gt tos, r0
  next,
END-CODE

CODE MAX	\ n1 n2 -- n1|n2
\ *G Given two data stack items preserve only the larger.
  ldmfd psp ! { r0 }
  cmp tos, r0
  mov .lt tos, r0
  next,
END-CODE

CODE WITHIN? 	\ n1 n2 n3 -- flag
\ *G Return TRUE if N1 is within the range N2..N3.
\ ** This word uses signed arithmetic.
  ldmfd psp ! { r0, r1 }
  mov r2, # 0
  mov r3, # 0
  cmp r1, r0
  mov .ge r2, # 1
  cmp r1, tos
  mov .le r3, # 1
  tst r2, r3
  mvn .ne tos, # 0
  mov .eq tos, # 0
  next,
END-CODE

CODE WITHIN 	\ n1|u1 n2|u2 n3|u3 -- flag
\ *G The ANS version of WITHIN?.
\ ** This word uses unsigned arithmetic, so that signed compares are
\ ** treated as existing on a number circle.
  ldmfd psp ! { r0, r1 }                \ tos = n3, r0 = n2, r1 = n1
  sub tos, tos, r0			\ tos-n2
  sub r1, r1, r0			\ n1-n2
  cmp r1, tos				\ n1 u< tos
  mvn .lo tos, # 0
  mov .hs tos, # 0
  next,
END-CODE

CODE LSHIFT 	\ x1 u -- x2
\ *G Logically shift X1 by U bits left.
  ldmfd psp ! { r0 }
  mov tos, r0, lsl tos
  next,
END-CODE

CODE RSHIFT 	\ x1 u -- x2
\ *G Logically shift X1 by U bits right.
  ldmfd psp ! { r0 }
  mov tos, r0, lsr tos
  next,
END-CODE

((
CODE <<		\ x1 u -- x1<<u
\ *G Logically shift X1 by U bits left.
\ ** Obsolete - replaced by LSHIFT.
  ldmfd psp ! { r0 }
  mov tos, r0, lsl tos
  next,
END-CODE

CODE >>		\ x1 u -- x1<<u
\ *G Logically shift X1 by U bits right.
\ ** Obsolete - replaced by RSHIFT.
  ldmfd psp ! { r0 }
  mov tos, r0, lsr tos
  next,
END-CODE
))


\ ***************
\ *S Control flow
\ ***************

CODE EXECUTE 	\ xt --
\ *G Execute the code described by the XT. This is a Forth equivalent
\ ** to an assembler JSR/CALL instruction.
  mov r0, tos			        \ move CFA
  ldmfd psp ! { tos }		        \ get new TOS
  mov pc, r0                            \ execute CFA - link contains ret addr of execute
END-CODE

internal

CODE BRANCH	\ --
\ *G The run time action of unconditional branches compiled on the target.
\ ** INTERNAL.
  ldr pc, [ link ]                      \ get address and branch
END-CODE

CODE ?BRANCH	\ n --
\ *G The run time action of conditional branches compiled on the target.
\ ** INTERNAL.
  mov .s tos, tos                       \ set flags
  ldmfd psp ! { tos }                   \ get new tos
  add .ne pc, link, # 4                 \ skip branch and jump back (do if ... else section )
  ldr pc, [ link ]                      \ get address and branch (do else ... then section )
END-CODE

CODE (OF) 	\ n1 n2 -- n1|--
\ *G The run time action of OF compiled on the target.
\ ** INTERNAL.
  ldmfd psp ! { r0 }
  cmp r0, tos                           \ compare n1 and n2
  mov .ne tos, r0                       \ not equal so tos=n1
  ldr .ne pc, [ link ]                  \ get branch addr and jump
  ldmfd psp ! { tos }                   \ equal so get new tos
  add pc, link, # 4                     \ jump over branch addr to code
END-CODE

CODE (LOOP) 	\ --
\ *G The run time action of *\fo{LOOP} compiled on the target.
\ ** INTERNAL.
  ldr r1, [ rsp ]                       \ fetch index
  add .s r1, r1, # 1                    \ increment index
  str .vc r1, [ rsp ]                   \ no overflow, store new index
  ldr .vc pc, [ link ]                  \ no overflow, take branch
  add rsp, rsp, # $0c                   \ drop 3 items from return stack
  add pc, link, # 4                     \ skip branch
END-CODE

CODE (+LOOP) 	\ n --
\ *G The run time action of *\fo{+LOOP} compiled on the target.
\ ** INTERNAL.
  ldr r1, [ rsp ]                       \ fetch index
  add .s r1, r1, tos                    \ increment index by n
  str .vc r1, [ rsp ]                   \ no overflow, store new index
  ldmfd psp ! { tos }                   \ get new tos
  ldr .vc pc, [ link ]                  \ no overflow, take branch
  add rsp, rsp, # $0c                   \ drop 3 items from return stack
  add pc, link, # 4                     \ skip branch
END-CODE

CODE (DO)	\ limit index --
\ *G The run time action of *\fo{DO} compiled on the target.
\ ** INTERNAL.
  ldmfd psp ! { r1 }                    \ get limit
L: PDO
  ldr r2, [ link ], # 4                 \ get LEAVE address and incr BRANCH addr
  add r1, r1, # $80000000
  sub r0, tos, r1

  stmfd rsp ! { r0-r2 }                 \ push LEAVE then limit, then index on ret. stack
  ldmfd psp ! { tos }                   \ update tos
  next,
END-CODE

CODE (?DO) 	\ limit index --
\ *G The run time action of *\fo{?DO} compiled on the target.
\ ** INTERNAL.
  ldmfd psp ! { r1 }                    \ get limit
  cmp r1, tos                           \ check not equal
  b .ne pdo                             \ take DO ?
  ldmfd psp ! { tos }                   \ update tos
  ldr pc, [ link ]                      \ and take leave branch
END-CODE

external

CODE LEAVE 	\ --
\ *G Remove the current *\fo{DO..LOOP} parameters and jump to the
\ ** end of the *\fo{DO..LOOP} structure.
  add rsp, rsp, # 8                     \ remove limit, index
  ldmfd rsp ! { pc }                    \ jump to exit address
END-CODE

CODE ?LEAVE 	\ flag --
\ *G If flag is non-zero, remove the current *\fo{DO..LOOP} parameters
\ ** and jump to the end of the *\fo{DO..LOOP} structure.
  mov .s tos, tos                       \ set flags
  ldmfd psp ! { tos }                   \ restore TOS
  mov .eq pc, link                      \ flag false so continue
  add rsp, rsp, # 8                     \ flag true so remove limit, index  - if old TOS<>0
  ldmfd rsp ! { pc }                    \ flag true so jump to exit address - if old TOS<>0
END-CODE

CODE I		\ -- n
\ *G Return the current index of the inner-most DO..LOOP.
\  stmfd psp ! { tos }
  str  tos, [ psp, # -4 ] !
  ldr tos, [ rsp, # 0 ]
  ldr r0, [ rsp, # 4 ]
  add tos, tos, r0
  next,
END-CODE

CODE J 		\ -- n
\ *G Return the current index of the second DO..LOOP.
\  stmfd psp ! { tos }                   \ push TOS
  str  tos, [ psp, # -4 ] !
  ldr tos, [ rsp, # $0C ]               \ index
  ldr r0, [ rsp, # $010 ]
  add tos, tos, r0
  next,
END-CODE

CODE UNLOOP 	\ -- ; R: loop-sys --
\ *G Remove the DO..LOOP control parameters from the return stack.
  add rsp, rsp, # $0C                   \ remove loop parameters from return stack
  next,                                 \ jump to return address
END-CODE


\ *******************
\ *S Basic arithmetic
\ *******************

CODE S>D 	\ n -- d
\ *G Convert a single number to a double one.
  stmfd psp ! { tos }                   \ push n
  mov .s tos, tos                       \ test for negative n
  mvn .mi tos, # 0                      \ make tos=-1
  mov .pl tos, # 0
  next,
END-CODE

CODE D>S 	\ d -- n
\ *G Convert a double number to a single.
  ldmfd psp ! { tos }                   \ low to Tos
  next,
END-CODE

CODE NOOP	\ --
\ *G A NOOP, null instruction.                                          )
  next,
END-CODE

CODE M+     \ d1|ud1 n -- d2|ud2
\ *G Add double d1 to sign extended single n to form double d2.
   ldmfd psp ! { r0 r1 }                \ r0 = d1 high, r1 = d1 low
   add .s r1, r1, tos                   \ d2l = d1l + n ; SFP018
   adc tos, r0, tos asr # #31           \ d2h = d1h + sex(n) + C ; SFP018
   stmfd psp ! { r1 }                   \ push d2l
   next,
END-CODE

CODE 1+ 	\ n1|u1 -- n2|u2
\ *G Add one to top-of stack.
  add tos, tos, # 1
  next,
END-CODE

CODE 2+ 	\ n1|u1 -- n2|u2
\ *G Add two to top-of stack.
  add tos, tos, # 2
  next,
END-CODE

CODE 4+ 	\ n1|u1 -- n2|u2
\ *G Add four to top-of stack.
  add tos, tos, # 4
  next,
END-CODE

CODE 1- 	\ n1|u1 -- n2|u2
\ *G Subtract one from top-of stack.
  sub tos, tos, # 1
  next,
END-CODE

CODE 2- 	\ n1|u1 -- n2|u2
\ *G Subtract two from top-of stack.
  sub tos, tos, # 2
  next,
END-CODE

CODE 4- 	\ n1|u1 -- n2|u2
\ *G Subtract four from top-of stack.
  sub tos, tos, # 4
  next,
END-CODE

CODE 2* 	\ x1 -- x2
\ *G Signed multiply top of stack by 2.
  mov tos, tos, lsl # 1
  next,
END-CODE

CODE 4* 	\ x1 -- x2
\ *G Signed multiply top of stack by 4.
  mov tos, tos, lsl # 2
  next,
END-CODE

CODE 2/ 	\ x1 -- x2
\ *G Signed divide top of stack by 2.
  mov tos, tos, asr # 1
  next,
END-CODE

CODE U2/ 	\ x1 -- x2
\ *G Unsigned divide top of stack by 2.
  mov tos, tos, lsr # 1
  next,
END-CODE

CODE 4/ 	\ x1 -- x2
\ *G Signed divide top of stack by 4.
  mov tos, tos, asr # 2
  next,
END-CODE

CODE U4/ 	\ x1 -- x2
\ *G Unsigned divide top of stack by 4.
  mov tos, tos, lsr # 2
  next,
END-CODE

CODE +  	\ n1|u1 n2|u2 -- n3|u3
\ *G Add two single precision integer numbers.
  ldmfd psp ! { r0 }
  add tos, tos, r0
  next,
END-CODE

CODE -  	\ n1|u1 n2|u2 -- n3|u3
\ *G Subtract two single precision integer numbers. N3|u3=n1|u1-n2|u2.
  ldmfd psp ! { r0 }
  sub tos, r0, tos
  next,
END-CODE

CODE NEGATE 	\ n1 -- n2
\ *G Negate a single precision integer number.
  rsb tos, tos, # 0
  next,
END-CODE

CODE D+ 	\ d1 d2 -- d3
\ *G Add two double precision integers.
  ldmfd psp ! { r0-r2 }                 \ r0 = d2 low, r1 = d1 high, r2 = d1 low
  add .s r0, r0, r2                     \ d3l = d2l + d1l
  adc tos, tos, r1                      \ d3h = d2h + d1h
  stmfd psp ! { r0 }                    \ push d3l
  next,
END-CODE

CODE D- 	\ d1 d2 -- d3
\ *G Subtract two double precision integers. D3=D1-D2.
  ldmfd psp ! { r0-r2 }                 \ r0 = d2 low, r1 = d1 high, r2 = d1 low
  sub .s r0, r2, r0                     \ d3l = d2l - d1l
  sbc tos, r1, tos                      \ d3h = d2h - d1h
  stmfd psp ! { r0 }                    \ push d3l
  next,
END-CODE

CODE DNEGATE 	\ d1 -- -d1
\ *G Negate a double number.
L: DNEG1
  ldmfd psp ! { r0 }                    \ r0 = d1l, tos = d1h
  rsb .s r0, r0, # 0                    \ negate low
  rsc tos, tos, # 0                     \ negate high carrying through
  stmfd psp ! { r0 }
  next,
END-CODE

CODE ?NEGATE 	\ n1 flag -- n1|n2
\ *G If flag is negative, then negate n1.
  mov .s tos, tos                       \ set processor flags
  ldmfd psp ! { tos }                   \ discard flag
  rsb .mi tos, tos, # 0                 \ negate n1 if flag=true
  next,
END-CODE

CODE ?DNEGATE 	\ d1 flag -- d1|d2
\ *G If flag is negative, then negate d1.
  mov .s tos, tos                       \ set processor flags
  ldmfd psp ! { tos }                   \ discard flag
  mov r1, link                          \ preserve link
  bl .mi DNEG1                          \ negate d1 if flag=true
  mov pc, r1                            \ restore link and exit
END-CODE

CODE ABS	\ n -- u
\ *G If n is negative, return its positive equivalent (absolute value).
  teq tos, # 0
  rsb .mi tos, tos, # 0
  next,
END-CODE

CODE DABS 	\ d -- ud
\ *G If d is negative, return its positive equivalent (absolute value).
  mov .s tos, tos                       \ set processor flags
  mov r1, link                          \ preserve link
  bl .mi DNEG1                          \ negate d if flag=true
  mov pc, r1                            \ restore link and exit
END-CODE

CODE D2* 	\ xd1 -- xd2
\ *G Multiply the given double number by two.
  ldmfd psp ! { r0 }                    \ get low
  mov .s r0, r0, asl # 1
  mov tos, tos, asl # 1                 \ shift left
  orr .cs tos, tos, # 1                 \ add carry if set
  stmfd psp ! { r0 }
  next,
END-CODE

CODE D2/ 	\ xd1 -- xd2
\ *G Divide the given double number by two.
  ldmfd psp ! { r0 }                    \ get low
  mov .s tos, tos, asr # 1
  mov r0, r0, rrx
  stmfd psp ! { r0 }
  next,
END-CODE


\ *****************
\ *S Multiplication
\ *****************

CODE UM*	\ u1 u2 -- ud
\ *G Perform unsigned-multiply between two numbers and return double
\ ** result.
  ldmfd psp ! { r0 }
  umull r1, tos, r0, tos                \ hi:lo = u1 * u2
  stmfd psp ! { r1 }                    \ use r1 because of umull register restrictions
  next,
END-CODE

CODE *		\ n1 n2 -- n3
\ *G Standard signed multiply. N3 = n1 * n2.
  ldmfd psp ! { r0 }
  mul tos, r0, tos
  next,
END-CODE

CODE m*		\ n1 n2 -- d
\ *G Signed multiply yielding double result.
  ldmfd psp ! { r0 }
  smull r1, tos, r0, tos                \ hi:lo = n1 * n2
  stmfd psp ! { r1 }
  next,
END-CODE


\ ***********
\ *S Division
\ ***********
\ *P Division on an ARM is really slow. Avoid it if you can where
\ ** performance matters. Multiplication by the reciprocal and
\ ** polynomial approximations are usually the recommended
\ ** techniques. We have opted for performance over code size.
\ ** The memory behaviour of many embedded ARM devices compunds
\ ** the impact of the additional test and branch overheads.
\ ** Consequently, the test and subtract loops are unrolled.
\ ** Unlike most other processors, signed division is faster than
\ ** unsigned!

\ *P For more details of implementing division on an ARM, and for
\ ** many other numerical coding techniques, see the
\ ** "ARM System Developer's Guide", ISBN 1-55860-874-5

macro: udiv64_step	\ --
\ *G Cross compiler macro to perform one step of the unsigned
\ ** 64 bit by 32 bit division
    add .s  r1, r1, r1			\ 64 bit shift of dividend
    adc .s  r0, r0, r0
    adc     r3, r2, r2			\ preserve dividend bit 63, R2=0
    sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
    adc .s  r3, r3, r3			\ success if bit 0 or 1 set
    orr .ne r1, r1, # 1
    add .eq r0, r0, tos			\ failed, undo subtraction
;m

((
code um/mod	\ ud1 u2 -- urem uquot
\ *G Slow and short - Full 64 by 32 unsigned division subroutine.
\ ** This routine uses a loop for code size. This version is
\ ** commented out by default.
  ldr     r0, [ psp ], # 4		\ dividend high
  ldr     r1, [ psp ]			\ dividend low
  mov	  r4, # #32			\ loop counter
  mov     r2, # 0			\ always 0
  begin,
    udiv64_step
    sub .s  r4, r4, # 1
  eq, until,
  mov     tos, r1			\ move quotient
  str     r0, [ psp ]			\ move remainder
  next,
end-code
))

code um/mod	\ ud1 u2 -- urem uquot
\ *G Fast and big - Full 64 by 32 unsigned division subroutine.
\ ** Unrolled for speed. This routine uses 920 bytes of code space.
  ldr     r0, [ psp ], # 4		\ dividend high
  ldr     r1, [ psp ]			\ dividend low
  mov     r2, # 0			\ always 0

  udiv64_step	\ 1
  udiv64_step	\ 2
  udiv64_step	\ 3
  udiv64_step	\ 4
  udiv64_step	\ 5
  udiv64_step	\ 6
  udiv64_step	\ 7
  udiv64_step	\ 8
  udiv64_step	\ 9
  udiv64_step	\ 10
  udiv64_step	\ 11
  udiv64_step	\ 12
  udiv64_step	\ 13
  udiv64_step	\ 14
  udiv64_step	\ 15
  udiv64_step	\ 16
  udiv64_step	\ 17
  udiv64_step	\ 18
  udiv64_step	\ 19
  udiv64_step	\ 20
  udiv64_step	\ 21
  udiv64_step	\ 22
  udiv64_step	\ 23
  udiv64_step	\ 24
  udiv64_step	\ 25
  udiv64_step	\ 26
  udiv64_step	\ 27
  udiv64_step	\ 28
  udiv64_step	\ 29
  udiv64_step	\ 30
  udiv64_step	\ 31
  udiv64_step	\ 32

\ all done
  mov     tos, r1			\ move quotient
  str     r0, [ psp ]			\ move remainder
  next,
end-code

proc Udiv63/31	\ r0:r1/tos ; 64/32 unsigned divide -> tos=quot, r0=rem
\ *G Unsigned division primitive - unrolled for speed.
\ ** Note that this routine does not handle the top bit of the divisor
\ ** and dividend correctly. *\fo{Udiv63/31} is used for signed divide
\ ** operations for which the top bits are always zero.
  rsb .s  tos, tos, # 0          \ negate divisor
  add .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 1
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 2
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 3
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 4
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 5
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 6
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 7
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 8
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 9
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 10
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 11
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 12
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 13
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 14
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 15
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 16
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 17
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 18
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 19
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 20
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 21
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 22
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 23
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 24
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 25
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 26
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 27
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 28
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 29
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 30
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 31
  sub .cc r0, r0, tos
  adc .s  r1, r1, r1
  adc .s  r0, tos, r0, lsl # 1   \ 32
  sub .cc r0, r0, tos
  adc     tos, r1, r1
  next,
end-code

CODE FM/MOD 	\  d1 n2 -- rem quot ; floored division
\ *G Perform a signed division of double number D1 by single number N2
\ ** and return remainder and quotient using floored division. See the
\ ** ANS Forth specification for more details of floored division.
  ldmfd psp ! { r0, r1 }                \ r0:r1=dividend
\ Make dividend and divisor absolute, saving signs
l: fm/mod1
  mov .s r3, r0                         \ r3=sign of dividend
  b .pl fm/mod2                         \ form |dividend|
  rsb .s r1, r1, # 0
  rsc r0, r0, # 0
l: fm/mod2
  mov .s r2, tos                        \ r2=sign of divisor
  rsb .mi tos, tos, # 0                 \ form |divisor|

  mov r6, link                          \ store link - r6 is free for use here
  bl Udiv63/31

  teq r0, # 0                           \ remainder = 0?
  b .eq fm/mod3                         \ NO
  cmp r2, # 0
  rsb .mi r0, r0, # 0                   \   remainder takes sign of divisor
  eor .s r3, r3, r2                     \   r3=sign of quotient
  rsb .mi tos, tos, # 0                 \   form quotient if neg.
  sub .mi r0, r2, r0                    \   form remainder if neg.
  sub .mi tos, tos, # 1                 \   form quotient if neg.
  stmfd psp ! { r0 }                    \   push rem
  mov pc, r6                            \   exit
l: fm/mod3                              \ YES
  eor .s r3, r3, r2                     \   r3=sign of quotient
  rsb .mi tos, tos, # 0                 \   form quotient
  stmfd psp ! { r0 }                    \   push rem
  mov pc, r6                            \   exit
END-CODE

CODE SM/REM 	\ d1 n2 -- rem quot ; symmetric division
\ *G Perform a signed division of double number D1 by single number N2
\ ** and return remainder and quotient using symmetric (normal) division.
  ldmfd psp ! { r0, r1 }		\ r0:r1 = d1
\ Make dividend and divisor absolute, saving signs
l: sm/rem1
  mov .s r3, r0				\ sign of dividend
  b .pl sm/rem2
  rsb .s r1, r1, # 0
  rsc r0, r0, # 0
l: sm/rem2
  mov .s r2, tos			\ sign of divisor
  rsb .mi tos, tos, # 0

  mov r6, link
  bl Udiv63/31				\ tos=quot, r0=rem

  teq r0, # 0
  b .eq sm/rem3				\ skip if rem=zero
  eor .s r3, r3, r2			\ sign of result
  rsb .mi tos, tos, # 0			\ negate quot and rem
  rsb .mi r0, r0, # 0
  cmp r2, # 0				\ if divisor negative
  rsb .mi r0, r0, # 0			\ negate remainder
  stmfd psp ! { r0 }
  mov pc, r6
l: sm/rem3
  eor .s r3, r3, r2
  rsb .mi tos, tos, # 0
  stmfd psp ! { r0 }
  mov pc, r6
END-CODE

CODE /MOD 	\ n1 n2 -- rem quot
\ *G Signed division of N1 by N2 single-precision yielding remainder
\ ** and quotient.
  ldmfd psp ! { r1 }                    \ get dividend
  mov .s r1, r1                         \ sign extend
  mvn .mi r0, # 0
  mov .pl r0, # 0
  b sm/rem1				\ join common code
END-CODE

(( for DocGen only
CODE M/MOD 	\ d1 n2 -- rem quot
\ *G A synonym for *\fo{FM/MOD} for Forth-83 compatibility.
\ ** Obsolete - Use *\fo{FM/MOD} instead.
  ....
END-CODE
))

: / 		\ n1 n2  -- n3
\ *G Standard signed division operator. n3 = n1/n2.
  /mod nip
;

: MOD 		\ n1 n2 -- n3
\ *G Return remainder of division of N1 by N2. n3 = n1 mod n2.
  /mod drop
;

: M/ 		\ d n1 -- n2
\ *G Signed divide of a double by a single integer.
  sm/rem nip
;

: MU/MOD 	\ d n -- rem d#quot
\ *G Perform an unsigned divide of a double by a single, returning
\ ** a single remainder and a double quotient.
  >r  0  r@  um/mod  r> swap >r  um/mod  r>
;


\ *********************************
\ *S Scaling - multiply then divide
\ *********************************
\ *P These operations perform a multiply followed by a divide.
\ ** The intermediate result is in an extended form.  The point
\ ** of this operation is to avoid loss of precision.

: */MOD 	\ n1 n2 n3 -- n4 n4
\ *G Multiply n1 by n2 to give a double precision result, and then
\ ** divide it by n3 returning the remainder and quotient.
  >r m* r> sm/rem
;

: */ 		\ n1 n2 n3 -- n4
\ *G Multiply n1 by n2 to give a double precision result, and then
\ ** divide it by n3 returning the quotient.
  */mod nip
;

: m*/		\ d1 n2 n3 -- dquot
\ *G The result dquot=(d1*n2)/n3. The intermediate value d1*n2
\ ** is triple-precision to avoid loss of precision. In an ANS
\ ** Forth standard program n3 can only be a positive signed
\ ** number and a negative value for n3 generates an ambiguous
\ ** condition, which may cause an error on some implementations,
\ ** but not in this one.
  s>d >r abs >r				\ -- d1 n2 ; R: -- sign(n3) |n3|
  s>d >r abs				\ -- d1 |n2| ; R: -- sign(n3) |n3| sign(n2)
  -rot					\ -- |n2| d1 ; R: -- sign(n3) |n3| sign(n2)
  s>d r> xor				\ -- |n2| d1 sign(d1*n2) ; R: -- sign(n3) |n3|
  r> swap >r >r				\ -- |n2| d1 ; R: -- sign(n3) sign(d1*n2) |n3|
  dabs rot				\ -- |d1| |n2| ; R: -- sign(n3) sign(d1*n2) |n3|
  tuck um* 2swap um*			\ -- d1h*n2 d1l*n2 ; R: -- sign(n3) sign(d1*n2) |n3|
  swap >r  0 d+ r> -rot			\ -- t ; R: -- sign (n3) sign(d1*n2) |n3|
  r@ um/mod -rot r> um/mod nip swap	\ -- d ; R: -- sign(n3) sign(d1*n2)
  r> r> xor IF dnegate THEN		\ -- d
;

((
internal
CODE TNEGATE 	\ t1 -- -t1 ; negate value of triple cell number
 \ *G Negate a triple (3 cell) number.
 \ ** INTERNAL.
  ldmfd psp ! { r0, r1 }		\ tos=t1.h, r0=t1.m, r1=t1.l
  rsb .s r1, r1, # 0
  rsc .s r0, r0, # 0
  rsc tos, tos, # 0
  stmfd psp ! { r0, r1 }
  next,
END-CODE
external
))


\ *********************
\ *S Stack manipulation
\ *********************

CODE NIP 	\ x1 x2 -- x2
\ *G Dispose of the second item on the data stack.
  add psp, psp, # 4
  next,
END-CODE

CODE TUCK 	\ x1 x2 -- x2 x1 x2
\ *G Insert a copy of the top data stack item underneath the current
\ ** second item.
  ldmfd psp ! { r0 }
  stmfd psp ! { r0, tos }
  next,
END-CODE

CODE PICK 	\ xu .. x0 u -- xu .. x0 xu
\ *G Get a copy of the Nth data stack item and place on top of stack.
\ ** 0 PICK is equivalent to DUP.
  ldr tos, [ psp, ++ tos lsl # 2 ]
  next,
END-CODE

HIGH-LEVEL-ROLL [if]
: ROLL		\ xu xu-1 .. x0 u -- xu-1 .. x0 xu
\ *G Rotate the order of the top N stack items by one place such that
\ ** the current top of stack becomes the second item and the Nth item
\ ** becomes TOS. See also *\fo{ROT}.
  >R R@ PICK
  SP@ DUP CELL+   R> 1+ CELLS CMOVE>  DROP
;
[else]
CODE ROLL 	\ xu xu-1 .. x0 u -- xu-1 .. x0 xu
  mov r0, tos, asl # 2                \ r0=position of xu in bytes
  cmp r0, # 0                         \ u is valid?
  ldmfd .le psp ! { tos }             \   no - get new tos
  mov .le pc, link                    \   no - exit
  ldr tos, [ psp, ++ r0 ]             \ put xu in tos
L: ROLL1
  sub r1, r0, # 4
  ldr r2, [ psp, ++ r1 ]
  str r2, [ psp, ++ r0 ]
  mov .s r0, r1
  b .ne ROLL1
  add psp, psp, # 4
  next,
END-CODE
[then]

CODE ROT 	\ x1 x2 x3 -- x2 x3 x1
\ *G ROTate the positions of the top three stack items such that the
\ ** current top of stack becomes the second item. See also *\fo{ROLL}.
  ldmfd psp ! { r0 r1 }                 \ R1=n1, R0=n2, TOS=n3
  stmfd psp ! { r0 }
  stmfd psp ! { tos }
  mov tos, r1
  next,
END-CODE

CODE -ROT 	\ x1 x2 x3 -- x3 x1 x2
\ *G The inverse of *\fo{ROT}.
  ldmfd psp ! { r0 r1 }               \ R1=n1, R0=n2, TOS=n3
  stmfd psp ! { r1 tos }
  mov tos, r0
  next,
END-CODE

CODE >R 	\ x -- ; R: -- x
\ *G Push the current top item of the data stack onto the top of the
\ ** return stack.
  stmfd rsp ! { tos }
  ldmfd psp ! { tos }
  next,
END-CODE

CODE R> 	\ -- x ; R: x --
\ *G Pop the top item from the return stack to the data stack.
  stmfd psp ! { tos }
  ldmfd rsp ! { tos }
  next,
END-CODE

CODE R@ 	\ --  x  ; R:  x -- x
\ *G Copy the top item from the return stack to the data stack.
  stmfd psp ! { tos }
  ldr tos, [ rsp ]
  next,
END-CODE

CODE 2>R 	\ x1 x2 -- ; R:  -- x1 x2
\ *G Transfer the two top data stack items to the return stack.
  ldmfd psp ! { r0, r1 }                \ r0=x1, r1=new tos
  stmfd rsp ! { r0 }                    \ push x1
  stmfd rsp ! { tos }                   \ push x2
  mov tos, r1                           \ new tos=r1
  next,
END-CODE

CODE 2R> 	\ -- x1 x2 ; R: x1 x2 --
\ *G Transfer the top two return stack items to the data stack.
  ldmfd rsp ! { r0 r1 }                 \ r0=x2, r1=x1
  stmfd psp ! { r1 tos }                \ push old tos, x1
  mov tos, r0                           \ tos=x2
  next,
END-CODE

CODE 2R@ 	\ --  x1 x2  ; R:  x1 x2 -- x1 x2
\ *G Copy the top two return stack items to the data stack.
  ldmfd rsp { r0, r1 }                  \ r0=x2, r1=x1
  stmfd psp ! { r1 tos }                \ push old tos, x1
  mov tos, r0                           \ tos=x2
  next,
END-CODE

CODE 2ROT 	\ x1 x2 x3 x4 x5 x6 -- x3 x4 x5 x6 x1 x2
\ *G Perform the *\fo{ROT} operation on three cell-pairs.
  ldmfd psp ! { r0-r4 }                 \ r0=x5, r4=x1
  stmfd psp ! { r0-r2 }                 \ push x3-x5
  stmfd psp ! { r4, tos }               \ push x6, x1
  mov tos, r3                           \ x2 to tos
  next,
END-CODE

CODE SWAP 	\ x1 x2 -- x2 x1
\ *G Exchange the top two data stack items.
  swp tos, tos, [ psp ]
  next,
END-CODE

CODE DUP 	\ x -- x x
\ *G DUPlicate the top stack item.
  stmfd psp ! { tos }
  next,
END-CODE

CODE OVER 	\ x1 x2 -- x1 x2 x1
\ *G Copy NOS to a new top-of-stack item.
  stmfd psp ! { tos }
  ldr tos, [ psp, # 4 ]
  next,
END-CODE

CODE DROP 	\ x --
\ *G Lose the top data stack item and promote NOS to TOS.
  ldmfd psp ! { tos }
  next,
END-CODE

CODE 2DROP 	\ x1 x2 -- )
\ *G Discard the top two data stack items.
  add psp, psp, # 4
  ldmfd psp ! { tos }
  next,
END-CODE

CODE 2SWAP 	\ x1 x2 x3 x4 -- x3 x4 x1 x2
\ *G Exchange the top two cell-pairs on the data stack.
  ldmfd psp ! { r0 }                    \ r0=x3
  swp tos, tos, [ psp ]                 \ swap x4 with x2
  add r1, psp, # 4                      \ r1 points to x1
  swp r0, r0, [ r1 ]                    \ swap x3 and x1
  stmfd psp ! { r0 }                    \ push x1
  next,
END-CODE

CODE ?DUP 	\ x -- | x
\ *G DUPlicate the top stack item only if it non-zero.
  mov .s tos, tos
  stmfd .ne psp ! { tos }
  next,
END-CODE

CODE 2DUP 	\ x1 x2 -- x1 x2 x1 x2
\ *G DUPlicate the top cell-pair on the data stack.
  ldr r0, [ psp ]                       \ r0=x1, tos=x2
  stmfd psp ! { r0, tos }               \ push tos, r0
  next,
END-CODE

CODE 2OVER 	\ x1 x2 x3 x4 -- x1 x2 x3 x4 x1 x2
\ *G As *\fo{OVER} but works with cell-pairs rather than single-cell items.
  stmfd psp ! { tos }
  ldr r0, [ psp, # $0C ]
  ldr tos, [ psp, # $08 ]
  stmfd psp ! { r0 }
  next,
END-CODE

CODE SP@ 	\ -- x
\ *G Get the current address value of the data-stack pointer.
  stmfd psp ! { tos }
  mov tos, psp
  next,
END-CODE

CODE SP! 	\ x --
\ *G Set the current address value of the data-stack pointer.
  mov psp, tos
  ldmfd psp ! { tos }
  next,
END-CODE

CODE RP@ 	\ -- x
\ *G Get the current address value of the return-stack pointer.
  stmfd psp ! { tos }
  mov tos, rsp
  next,
END-CODE

CODE RP! 	\ x --
\ *G Set the current address value of the return-stack pointer.
  mov rsp, tos
  ldmfd psp ! { tos }
  next,
END-CODE


\ ******************************
\ *S String and memory operators
\ ******************************

CODE COUNT 	\ c-addr1 -- c-addr2' u
\ *G Given the address of a counted string in memory this word will
\ ** return the address of the first character and the length in
\ ** characters of the string.
  add r0, tos, # 1
  stmfd psp ! { r0 }                    \ return c-addr2'
  ldrb tos, [ tos ]
  next,
END-CODE

CODE /STRING 	\ c-addr1 u1 n -- c-addr2 u2
\ *G Modify a string address and length to remove the first N characters
\ ** from the string.
  ldmfd psp ! { r0, r1 }                \ r0=u1, r1=c-addr1
  add r1, r1, tos                       \ update addr
  stmfd psp ! { r1 }                    \ push addr
  sub tos, r0, tos                      \ update len
  next,
END-CODE

CODE SKIP 	\ c-addr1 u1 char -- c-addr2 u2
\ *G Modify the string description by skipping over leading occurrences of
\ ** 'char'.
  ldmfd psp ! { r0, r1 }                \ tos=char, r0=len, r1=adr1
  add r0, r0, # 1
L: SK1
  sub .s r0, r0, # 1
  b .eq DONE
  ldrb r2, [ r1 ]
  cmp r2, tos
  add .eq r1, r1, # 1
  b .eq SK1
L: DONE
  stmfd psp ! { r1 }
  mov tos, r0
  next,
END-CODE

CODE SCAN 	\ c-addr1 u1 char -- c-addr2 u2
\ *G Look for first occurrence of *\i{char} in the string and
\ ** return a new string. *\i{C-addr2/u2} describes the string
\ ** with *\i{char} as the first character.
  ldmfd psp ! { r0, r1 }                \ tos=char, r0=len, r1=adr1
  add r0, r0, # 1
L: SC1
  sub .s r0, r0, # 1
  b .eq SCANDONE
  ldrb r2, [ r1 ]
  cmp r2, tos
  add .ne r1, r1, # 1
  b .ne SC1
L: SCANDONE
  stmfd psp ! { r1 }
  mov tos, r0
  next,
END-CODE

CODE S=		\ c-addr1 c-addr2 u -- flag
\ *G Compare two same-length strings/memory blocks, returning TRUE if
\ ** they are identical.
  ldmfd psp ! { r0, r1 }                \ r0=adr2, r1=adr1
  mov .s tos, tos
  b .eq S=X                             \ TRUE as len=0
L: S=1
  ldrb r2, [ r0 ], # 1
  ldrb r3, [ r1 ], # 1
  cmp r2, r3
  b .ne S=X                             \ FALSE as mismatched chars
  sub .s tos, tos, # 1
  b .ne S=1                             \ if count=0 then TRUE
L: S=X
  mvn .eq tos, # 0                      \ TRUE
  mov .ne tos, # 0                      \ FALSE
  next,
END-CODE

: compare       \ c-addr1 u1 c-addr2 u2 -- n                    17.6.1.0935
\ *G Compare two strings. The return result is 0 for a match or can be
\ ** -ve/+ve indicating string differences.
\ ** If the two strings are identical, n is zero. If the two strings
\ ** are identical up to the length of the shorter string, n is
\ ** minus-one (-1) if u1 is less than u2 and one (1) otherwise.
\ ** If the two strings are not identical up to the length of the
\ ** shorter string, n is minus-one (-1) if the first non-matching
\ ** character in the string specified by c-addr1 u1 has a lesser
\ ** numeric value than the corresponding character in the string
\ ** specified by c-addr2 u2 and one (1) otherwise.
  rot swap                      \ c-addr1 c-addr2 u1 u2
  2dup - >r min                 \ c-addr1 c-addr2 minlen -- R: lendiff? --

  begin
    dup
  while
    -rot over c@ over c@ -      \ length c-addr1 c-addr2 (char1-char2)
    ?dup if                     \ If chars are different
        r> drop >r              \  replace lendiff result with error code
        drop 0                  \  and put 0 on TOS (make len==0 at BEGIN)
    else                        \ otherwise
        1+ swap 1+ swap rot 1-  \ increment addresses and decrement length
    then
  repeat
  drop 2drop                    \ remove addresses and null count from stack
                                \ -- ; R: result --
  r> dup if 0< 1 or then        \ make nice flag, 0 becomes 0, -ve becomes -1
                                \            and  +ve becomes 1
;

internal
: (search)	{ c-addr1 u1 c-addr2 u2 -- c-addr3 u3 flag }
  c-addr1 u1 u2 - 1+ bounds     	\ Search for first char of $2 in $1.  Note that
  ?do                           	\ we only check LEN($1)-LEN($2)+1 chars of $1.
    i c@  c-addr2 c@  = if        	\ Compare next char of $1 with first char of $2.
      i c-addr2 u2 s= if           	\ Is the whole of $2 present?
        i u1 i c-addr1 - - true 	\ Yes - build return stack parameters...
        unloop exit             	\ ...and bail out.
      then
    then
  loop
  c-addr1 u1 false            		\ End of loop and first char not found - no match.
;
external

: SEARCH	( c-addr1 u1 c-addr2 u2 -- c-addr3 u3 flag  )
\ *G Search the string c-addr1/u1 for the string c-addr2/u2. If a match
\ ** is found return c-addr3/u3, the address of the start of the match
\ ** and the number of characters remaining in c-addr1/u1, plus flag f
\ ** set to true. If no match was found return c-addr1/u1 and f=0.
  2 pick over <         		\ Is $1 shorter than $2?
  if  2drop false exit  then   		\ Yes - $2 *can't* be in $1.
  dup 0 <=              		\ Is $2 zero length?
  if  2drop true exit  then    		\ Yes - we have found it at the start of $1.
  2 pick 0 <=           		\ Is $1 zero length?
  if  2drop false exit  then   		\ Yes - string not found.
  (search)
;

FastCmove? [if]
  include %CpuDir%\fcmove.fth
[else]
code cmove	\ asrc adest len --
\ *G Copy *\i{len} bytes of memory forwards from *\i{asrc} to *\i{adest}.
\ ** If the performance of *\fo{CMOVE} is important in your
\ ** application, set the equate *\fo{FastCmove?} non-zero and
\ ** a much faster but much larger version will be compiled.
\ ** See *\i{Arm\fcmove.fth} for the details.
  mov .s  r3, tos                	\ r3=len
  ldmfd   psp ! { r0, r1, tos }		\ r0=addr2, r1=addr1, r3=len, tos restored
  mov .eq pc, link			\ return if len=0, nothing to do
  orr     r2, r1, r0			\ check alignment
  orr     r2, r2, r3			\ R2 := adr1|addr2|len
  and .s  r4, r2, # 3			\ will be zero if aligned
  b .ne   cmovx3			\ if not aligned
L: CMOVX1
  ldr     r2, [ r1 ], # 4
  sub .s  r3, r3, # 4			\ hoisted for ARM9
  str     r2, [ r0 ], # 4
  b .ne   CMOVX1
  next,
L: CMOVX3
  ldrb    r2, [ r1 ], # 1
  sub .s  r3, r3, # 1			\ hoisted for ARM9
  strb    r2, [ r0 ], # 1
  b .ne   CMOVX3
  next,
END-CODE
[then]

CODE CMOVE>	\ c-addr1 c-addr2 u --
\ *G As *\fo{CMOVE} but working in the opposite direction,
\ ** copying the last character in the string first.
  mov .s  r3, tos                	\ r3=len
  ldmfd   psp ! { r0, r1, tos }		\ r0=addr2, r1=addr1, r3=len, tos restored
  mov .eq pc, link			\ return if len=0, nothing to do
  add     r0, r0, r3			\ addr2+len
  add     r1, r1, r3			\ addr1+len
  orr     r2, r1, r0			\ check alignment
  orr     r2, r2, r3			\ R2 := adr1|addr2|len
  and .s  r4, r2, # 3			\ will be zero if aligned
  b .ne   CMV>3				\ byte by byte if not aligned
L: CMV>1	\ copy by 4 byte units
  ldr     r2, [ r1, # -4 ] !
  sub .s  r3, r3, # 4			\ hoisted for ARM9 and above
  str     r2, [ r0, # -4 ] !
  b .ne   CMV>1
  next,
L: CMV>3	\ copy byte by byte
  ldrb    r2, [ r1, # -1 ] !
  sub .s  r3, r3, # 1			\ hoisted for ARM9 and above
  strb    r2, [ r0, # -1 ] !
  b .ne   CMV>3
  next,
END-CODE

CODE ON		\ a-addr --
\ *G Given the address of a CELL this will set its contents to TRUE (-1).
  mvn r0, # 0                           \ -1
  str r0, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE OFF 	\ a-addr --
\ *G Given the address of a CELL this will set its contents to FALSE (0).
  mov r0, # 0
  str r0, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE C+!	\ b c-addr --
\ *G Add N to the character (byte) at memory address ADDR.
  ldmfd psp ! { r0 }
  ldrb r1, [ tos ]
  add r1, r1, r0
  strb r1, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE 2@		\ a-addr -- x1 x2
\ *G Fetch and return the two CELLS from memory ADDR and ADDR+sizeof(CELL).
\ ** The cell at the lower address is on the top of the stack.
  ldmfd tos { r0, r1 }                  \ r0=x2, r1=x1
  stmfd psp ! { r1 }
  mov tos, r0
  next,
END-CODE

CODE 2! 	\ x1 x2 a-addr --
\ *G Store the two CELLS x1 and x2 at memory ADDR.
\ ** X2 is stored at ADDR and X1 is stored at ADDR+CELL.
  ldmfd psp ! { r0-r1 }                 \ r0=x2, r1=x1
  str r0, [ tos ], # 4
  str r1, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE FILL	\ c-addr u char --
\ *G Fill LEN bytes of memory starting at ADDR with the byte information
\ ** specified as CHAR.
  ldmfd psp ! { r0, r1 }                \ tos=char, r0=count, r1=addr
L: FIL1
  sub .s r0, r0, # 1
  strb .pl tos, [ r1 ], # 1
  b .pl FIL1
  ldmfd psp ! { tos }
  next,
END-CODE

CODE +!		\ n|u a-addr --
\ *G Add N to the CELL at memory address ADDR.
  ldmfd psp ! { r0 }
  ldr r1, [ tos ]
  add r1, r1, r0
  str r1, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE INCR 	\ a-addr --
\ *G Increment the data cell at a-addr by one.
  ldr r1, [ tos ]
  add r1, r1, # 1
  str r1, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE DECR	\ a-addr --
\ *G Decrement the data cell at a-addr by one.
  ldr r1, [ tos ]
  sub r1, r1, # 1
  str r1, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE @		\ a-addr -- x
\ *G Fetch and return the CELL at memory ADDR.
  ldr tos, [ tos ]
  next,
END-CODE

CODE W@ 	\ a-addr -- w
\ *G Fetch and 0 extend the word (16 bit) at memory ADDR.
  ldrh tos, [ tos ]
  next,
END-CODE

CODE C@ 	\ c-addr -- char
\ *G Fetch and 0 extend the character at memory ADDR and return.
  ldrb tos, [ tos ]
  next,
END-CODE

CODE ! 		\ x a-addr --
\ *G Store the CELL quantity X at memory A-ADDR.
  ldmfd psp ! { r0 }
  str r0, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE W! 	\ w a-addr --
\ *G Store the word (16 bit) quantity w at memory ADDR.
  ldmfd psp ! { r0 }
  strh r0, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE C! 	\ char c-addr --
\ *G Store the character CHAR at memory C-ADDR.
  ldmfd psp ! { r0 }
  strb r0, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE

CODE UPPER 	\ c-addr u --
\ *G Convert the ASCII string described to upper-case. This operation
\ ** happens in place.
  ldmfd psp ! { r0 }                    \ r0=addr, tos=len
L: UPPER1
  sub .s tos, tos, # 1
  b .mi UPPER3                          \ len=0 so exit
  ldrb r1, [ r0 ], # 1                  \ fetch char and incr addr
  cmp r1, # $60                         \ check lower bound
  b .ls UPPER1                          \ skip if lower
  cmp r1, # $7A                         \ check upper bound
  and .ls r1, r1, # $DF                 \ if OK then mask
  strb .ls r1, [ r0, # -1 ]             \ if OK then store char
  b UPPER1                              \ repeat
L: UPPER3
  ldmfd psp ! { tos }                   \ get new tos
  next,
END-CODE

CODE TEST-BIT	\ mask c-addr -- flag
\ *G AND the mask with the contents of addr and return true if the
\ ** result is non-zero (-1) or false (0) if the result is zero.
\ ** Byte operation.
  ldmfd psp ! { r0 }                    \ get mask
  ldrb tos, [ tos ]                     \ get byte
  and tos, tos, r0                      \ apply mask
  next,
END-CODE

CODE SET-BIT	\ mask c-addr --
\ *G Apply the mask ORred with the contents of c-addr.
\ ** Byte operation.
  ldmfd psp ! { r0 }                    \ get mask
  ldrb r1, [ tos ]                      \ get byte
  orr r1, r1, r0                        \ apply mask
  strb r1, [ tos ]                      \ store byte
  ldmfd psp ! { tos }                   \ get new tos
  next,
END-CODE

CODE RESET-BIT	\ mask c-addr --
\ *G Apply the mask inverted and ANDed with the contents of c-addr.
\ ** Byte operation.
  ldmfd psp ! { r0 }                    \ get mask
  ldrb r1, [ tos ]                      \ get-byte
  bic r1, r1, r0                        \ apply mask
  strb r1, [ tos ]                      \ store byte
  ldmfd psp ! { tos }                   \ get new tos
  next,
END-CODE

CODE TOGGLE-BIT	\ u c-addr --
\ *G Invert the bits at c-addr specified by the mask. Byte operation.
  ldmfd psp ! { r0 }                    \ get mask
  ldrb r1, [ tos ]                      \ get-byte
  eor r1, r1, r0                        \ apply mask
  strb r1, [ tos ]                      \ store byte
  ldmfd psp ! { tos }                   \ get new tos
  next,
END-CODE


\ **********************
\ *S Miscellaneous words
\ **********************

CODE NAME>	\ nfa -- cfa
\ *G Move a pointer from an NFA to the CFA or "XT" in ANS parlance.
  ldrb r0, [ tos ]                      \ fetch count byte
  and r0, r0, # $1f                     \ strip top 3 bits
  add r0, r0, tos                       \ add base address
  add r0, r0, # 4                       \ add 1 for c.b., 3 to 4byte align
  mvn r1, # 3                           \ mov -4 into r1
  and tos, r0, r1                       \ force even boundary
  next,
END-CODE

\ On ARM, name field is a multiple of 4 bytes so we only need to
\ check top bit of every fourth byte when working back from the cfa
\ to find the nfa
CODE >NAME	\ cfa -- nfa
\ *G Move a pointer from an XT back to the NFA or name-pointer.
\ ** If the original pointer was not an XT or if the definition in
\ ** question has no name header in the dictionary the returned pointer
\ ** will be useless. Care should be taken when manipulating or scanning
\ ** the Forth dictionary in this way.
L: TO-NAME-1
  ldr r0, [ tos, # -4 ] !               \ get prev. 4 bytes
little-endian? [if]
  tst r0, # $00000080			\ top bit of low byte set?
[else]
  tst r0, # $80000000			\ top bit of high byte set?
[then]
  b .eq TO-NAME-1                       \ loop if not
  next,
END-CODE

internal
: (SEARCH-WORDLIST)	\ c-addr u ^^nfa -- 0 | xt 1 | xt -1
  begin
    @ dup
  while
    dup c@ $20 and
    0= if
      dup c@ $1F and            	\ -- c-addr u ^nfa nfalen ;
      2 pick =
      if                        	\ Are the names the same *length*?
        dup 1+ 3 pick 3 pick s=
        if                      	\ Are they the same *name*?
          nip nip
          dup name> swap
          c@ $40 and 0= if -1 else 1 then
          exit
        then
      then
    then
    n>link
  repeat
  drop 2drop 0
;
external

: SEARCH-WORDLIST	\ c-addr u wid -- 0|xt 1|xt -1
\ *G Search the given wordlist for a definition. If the definition is
\ ** not found then 0 is returned, otherwise the XT of the definition
\ ** is returned along with a non-zero code. A -ve code indicates a
\ ** "normal" definition and a +ve code indicates an *\fo{IMMEDIATE} word.
  dup 0= if
    nip nip
  else
    over 3 pick c@ + over @ 1- and      \ -- c-addr u wid thread# ;
    1+ cells +                          \ -- c-addr u ^nfa ;
    (search-wordlist)
  then
;

CODE DIGIT 	\ char n -- 0|n true
\ *G If the ASCII value *\i{CHAR} can be treated as a digit for a number
\ ** within the radix *\i{N} then return the digit and true, otherwise
\ ** return false.
  ldmfd psp ! { r0 }                    \ base in tos, char in r0
  sub .s r0, r0, # $30                  \
  b .mi DIG2                            \ fail if char < '0'
  cmp r0, # $0A
  b .lt DIG1                            \ true, '0' <= char <= '9@
  sub r0, r0, # 7                       \ convert 'A'..'F'
  cmp r0, # $0A
  b .lt DIG2                            \ fail if result < 'A'
L: DIG1
  cmp r0, tos                           \ in range of base?
  mvn .lt tos, # 0                      \ yes, tos=true
  stmfd .lt psp ! { r0 }                \ push n
  mov .lt pc, link                      \ exit
L: DIG2
  mov tos, # 0                          \ tos=false
  next,                                 \ exit
END-CODE


\ **********************
\ *S Portability helpers
\ **********************
\ *P Using these words will make code easier to port between
\ ** 16, 32 and 64 bit targets.

CODE CELL+ 	\ a-addr1 -- a-addr2
\ *G Add the size of a CELL to the top-of stack.
  add tos, tos, # 4
  next,
END-CODE

CODE CELLS 	\ n1 -- n2
\ *G Return the size in address units of N1 cells in memory.
  mov tos, tos, asl # 2
  next,
END-CODE

CODE CELL- 	\ a-addr1 -- a-addr2
\ *G Decrement an address by the size of a cell.
  sub tos, tos, # 4
  next,
END-CODE

CODE CELL 	\ -- n
\ *G Return the size in address units of one CELL.
  stmfd psp ! { tos }           	\ save TOS
  mov tos, # 4                  	\ return cell size
  next,
END-CODE

CODE CHAR+ 	\ c-addr1 -- c-addr2
\ *G Increment an address by the size of a character.
  add tos, tos, # 1
  next,
END-CODE

CODE CHARS 	\ n1 -- n2
\ *G Return size in address units of N1 characters.
  next,
END-CODE


\ ********************
\ *S Runtime for VALUE
\ ********************

internal

CODE VAL!	\ n -- ; store value address in-line
\ *G Store n at the inline address following this word.
\ ** INTERNAL.
  ldr r0, [ link ], # 4         \ get data address (in-line)
  str tos, [ r0 ]               \ and write data from tos
  ldmfd psp ! { tos }           \ pop tos
  mov pc, link                  \ and continue
END-CODE

CODE VAL@	\ -- n ; read value data address in-line
\ *G Read n from the inline address following this word.
\ ** INTERNAL.
  stmfd psp ! { tos }           \ push tos
  ldr r0, [ link ], # 4         \ get data address (in-line)
  ldr tos, [ r0 ]               \ read data into tos
  mov pc, link
END-CODE

\ VAL generates a literal, and so LIT is used instead.

external


\ *************************************
\ *S Defining words and runtime support
\ *************************************

ASMCODE
L: DOCREATE	\ -- addr
\ *G The run time action of *\fo{CREATE}.
\ ** INTERNAL.
  stmfd psp ! { tos }
  ldr tos, [ link ]
  ldmfd rsp ! { pc }
end-code

internal

CODE LIT 	\ -- x
\ *G Code which when CALLED at runtime will return an inline cell
\ ** value.
\ ** INTERNAL.
  stmfd psp ! { tos }                   \ save TOS
  ldr tos, [ link ], # 4                \ get literal and get new return address
  mov pc, link
END-CODE

CODE (") 	\ -- a-addr ; return address of string, skip over it
\ *G Return the address of a counted string that is inline after the
\ ** CALLING word, and adjust the CALLING word's return address to
\ ** step over the inline string. See the definition of *\fo{(.")} for
\ ** an example.
\ ** INTERNAL.
  stmfd psp ! { tos }                   \ save tos
  ldr tos, [ rsp ]                      \ get a-addr
  ldrb r0, [ tos ]                      \ get count byte
  add r0, r0, # 4                       \ 1 for count, 3 to make aligned
  mvn r1, # 3                           \ mov -4 into r1
  and r0, r0, r1                        \ force 4 byte even boundary
  add r0, r0, tos                       \ add to a-addr
  str r0, [ rsp ]                       \ skip over
  next,
END-CODE

external

aligning? [if]
: aligned       \ addr -- addr'
\ *G Given an address pointer this word will return the next ALIGNED
\ ** address subject to system wide alignment restrictions.
  3 + -4 and
;
compiler
: aligned       \ addr -- addr'
  3 + -4 and
;
target
[then]

: CONSTANT 	\ x "<spaces>name" -- ; Exec: -- x
\ *G Create a new *\fo{CONSTANT} called *\fo{name} which has the
\ ** value *\i{x}. When *\fo{NAME} is executed the value is
\ ** returned.
  CREATE -4 allot , ;CODE
  stmfd psp ! { tos }                   \ save old TOS
  ldr tos, [ link ]                     \ get const val from PFA
  ldmfd rsp ! { pc }
END-CODE

: VARIABLE 	\ "<spaces>name" -- ; Exec: -- a-addr
\ *G Create a new variable called *\fo{name}. When *\fo{name}" is
\ ** executed the address of the data-cell is returned for use
\ ** with *\fo{@} and *\fo{!} operators.
  CREATE 0 , ;CODE
  stmfd psp ! { tos }                   \ save old tos
  ldr tos, [ link ]                     \ get var addr from PFA
  ldmfd rsp ! { pc }
END-CODE

: USER 		\ u "<spaces>name" -- ; Exec: -- addr ; SFP009
\ *G Create a new *\fo{USER} variable called *\fo{name}. The *\i{u}
\ ** parameter specifies the index into the user-area table at which
\ ** to place the* data. *\fo{USER} variables are located in a
\ ** separate area of memory for each task or interrupt. Use in
\ ** the form:
\ *C   $400 USER TaskData
  CREATE  -4 allot ,  ;CODE		\ replce data address by offset
  stmfd psp ! { tos }                   \ save old tos
  ldr r0, [ link ]                      \ get ptr to user val from PFA
  add tos, up, r0                       \ add user pointer to user value
  ldmfd rsp ! { pc }
end-code

interpreter
: u#		\ "<name>"-- u
\ *G An *\fo{INTERPRETER} word that returns the index of the
\ ** *\fo{USER} variable whose name follows, e.g.
\ *C   u# S0
  ' >body
;
target

interpreter
also asm-access		\ black magic here!
  get-rsp  get-psp  get-tos
previous
target
equ tos-reg
equ psp-reg
equ rsp-reg

internal
: DOCOLON, 	\ --
\ *G Compile the runtime entry code required by colon definitions.
\ ** INTERNAL.
  [ $e9204000 rsp-reg $10 <<n or ] literal ,	\ stmfd rsp ! { link }
;
external

((
\ Short form
  bl      <dest>
\ Long form
  mov     link, pc			\ $+0, link := $+8
  ldr     pc, [ link ], # 4		\ $+4, link := $+12
  dd      <dest>			\ $+8, contains target address
  ...					\ $+12, returns here
))
: !call		\ dest addr --
\ *G Install a BL DEST opcode at addr.
  tuck					\ -- addr xt addr
  8 +  -				\ -- addr offset
  2 rshift $00FFFFFF and $EB000000 or 	\ -- addr BLins
  swap !				\ set call instruction
;

LongCalls? [if]
: scall,	\ addr --
\ *G Compile a machine code BL to addr. No range checking is performed.
  here 8 + -				\ subtract HERE (pc in effect)
  2 rshift $00FFFFFF and		\ shift addr and mask
  $EB000000 or ,			\ make BL and lay down
;

: compile,	\ xt --
\ *G Compile the word specified by xt into the current definition.
  dup here 8 + - $FC000000 and		\ -- dest offset'
  dup 0=
  if  drop scall,  exit  endif
  $FC000000 =
  if  scall,  exit  endif
  $E1A0E00F ,				\ $+00: mov  link, pc
  $E49EF004 ,				\ $+04: ldr  pc, [ link ], # 4
  ,					\ $+08: <dest>
;
[else]
: compile,	\ xt --
  here 8 + -				\ subtract HERE (pc in effect)
  2 rshift $00FFFFFF and		\ shift addr and mask
  $EB000000 or ,			\ make BL and lay down
;
[then]

compiler
: call,		\ dest --
\ *G Compile a BL or long CALL into the current definition.
  compile,
;
target

LongCalls? [if]
: >BODY		\ xt -- a-addr
\ *G Move a pointer from a CFA or "XT" to the definition BODY. This
\ ** should only be used with children of *\fo{CREATE}. E.g. if
\ ** *\fo{FOOBAR} is defined by *\fo{CREATE foobar}, then the
\ ** phrase *\fo{' foobar >body} would yield the same result as
\ ** executing *\fo{foobar}.
  dup 4 + @				\ instruction after STM
  $FF000000 and $EB000000 =	 	\ BL opcode?
  if  8 + @  else  #16 + @  endif	\ address at xt+8 or xt +16
;
[else]
: >BODY		\ xt -- a-addr
  8 + @
;
[then]

target-only
: :		\ C: "<spaces>name" -- colon-sys ; Exec: i*x -- j*x ; R: -- nest-sys
\ *G Begin a new definition called *\fo{name}.
  ?exec  !csp
  bl word count makeheader smudge
  [compile] ] docolon,
;
host&target

target-only
: :NONAME	\ C: -- colon-sys ; Exec: i*x -- i*x  ; R: -- nest-sys
\ *G Begin a new code definition which does not have a name. After the
\ ** definition is complete the semi-colon operator returns the XT of
\ ** newly compiled code on the stack.
  align here cell+                      \ return XT
  !csp                                  \ compiler stack security
  here last ! 0 ,                       \ lay null header to fool SMUDGE
  ] docolon,				\ compiler on, lay entry code
;
host&target

internal
: DOCREATE, 	\ --
\ *G Compile the run time action of *\fo{CREATE}.
\ ** INTERNAL.
  [ $e9204000 rsp-reg $10 <<n or ] literal ,	\ stmfd rsp ! { link }
  DOCREATE CALL,  HERE 4+ ,
;

: (;CODE) 	\ -- ; R: a-addr --
\ *G Performed at compile time by *\fo{;CODE} and *\fo{DOES>}.
\ ** Patch the last word defined (by *\fo{CREATE}) to have the
\ ** run time actions that follow immediately after *\fo{(;CODE)}.
\ ** INTERNAL.
  r> latest name>			\ -- runtime childxt
  dup 4 + @				\ instruction after STM
  $FF000000 and $EB000000 = 	 	\ BL opcode or long form?
  if  4 + !call  else  #12 + !  endif
;
external

: DOES>		\ C: colon-sys1 -- colon-sys2 ; Run: -- ; R:  nest-sys --
\ *G Begin definition of the runtime-action of a child of a defining word.
\ ** See the section about defining words in *\i{Programming Forth}.
\ ** You should not use *\fo{RECURSE} after *\fo{DOES>}.
  compile (;code)
  [ $e9200000 psp-reg $10 <<n or 1 tos-reg <<n or ]
  literal ,                                             \ stmfd psp ! { tos }
  [ $e59e0000 tos-reg $c <<n or ] literal ,             \ ldr tos, [ link ]
;
IMMEDIATE

: CRASH 	\ -- ; used as action of DEFER
\ *G The default action of a *\fo{DEFER}ed word. This will simply
\ ** *\fo{THROW} a code back to the system.
  $0C throw
;

external

: DEFER 	\ Comp: "<spaces>name" -- ; Run:  i*x -- j*x
\ *G Creates a new *\fo{DEFER}ed word. A default action,
\ ** *\fo{CRASH}, is automatically assigned.
  CREATE
    ['] CRASH ,
  ;CODE
  ldr r0, [ link ]
  ldmfd rsp ! { link }                  \ allows return to 'top-level' from word
  ldr pc, [ r0 ]                        \ jump to word
END-CODE

: 2CONSTANT	\ Comp: x1 x2 "<spaces>name" -- ; Run: -- x1 x2
\ *G A two-cell equivalent to *\fo{CONSTANT}.
  create
    cell negate allot , ,
  ;code
  stmfd psp ! { tos }                   \ push tos
  ldr r0, [ link, # 4 ]                 \ get low
  stmfd psp ! { r0 }                    \ push low
  ldr tos, [ link ]                     \ get high
  ldmfd rsp ! { pc }                    \ exit
END-CODE

: 2VARIABLE	\ Comp: "<spaces>name" -- ; Run: -- a-addr
\ *G A two-cell equivalent to *\fo{VARIABLE}.
  create
    0 , 0 ,
  ;code
  stmfd psp ! { tos }                   \ save old tos
  ldr tos, [ link ]                     \ get var addr from pfa
  ldmfd rsp ! { pc }                    \ exit
END-CODE

: FIELD		\ size n "<spaces>name" -- size+n ; Exec: addr -- addr+n
\ *G Create a new field within a structure definition of size n bytes.
  CREATE  -4 allot over , +  ;CODE
  ldr r0, [ link ]                      \ get offset from pfa
  add tos, tos, r0                      \ add to base addr
  ldmfd rsp ! { pc }                    \ exit
END-CODE


\ ************************
\ *S Structure compilation
\ ************************
\ *P These words define high level branches. They are used by the structure
\ ** words such as *\fo{IF} and *\fo{AGAIN}.

hex

internal

: >mark         \ -- addr
\ *G Mark the start of a forward branch. HIGH LEVEL CONSTRUCTS ONLY.
\ ** INTERNAL.
  here  0 ,
;

: >resolve      \ addr --
\ *G Resolve absolute target of forward branch. HIGH LEVEL CONSTRUCTS ONLY.
\ here over 2+ - swap !                 \ relative
\ ** INTERNAL.
  here swap !  ;                        \ absolute

: <mark         \ -- addr
\ *G Mark the start (destination) of a backward branch.
\ ** HIGH LEVEL CONSTRUCTS ONLY.
\ ** INTERNAL.
  here
;

: <resolve      \ addr --
\ *G Resolve a backward branch to addr.
\ ** HIGH LEVEL CONSTRUCTS ONLY.
\ ** INTERNAL.
\  here swap 2+ -  ,  		        \ relative
   ,                    		\ absolute for target
;

synonym >c_res_branch >resolve	\ addr -- ; fix up forward referenced branch
\ *G See >RESOLVE.
\ ** INTERNAL.
synonym c_mrk_branch< <mark	\ -- addr ; mark destination of backward branch
\ *G See <MARK
\ ** INTERNAL.


\ **********************
\ *S Branch constructors
\ **********************
\ *P Used when compiling code on the target.

: c_branch<     \ addr --
\ *G Lay the code for BRANCH.
\ ** INTERNAL.
  compile branch ,
;

: c_?branch<    \ addr --
\ *G Lay the code for ?BRANCH.
  compile ?branch ,
;

: c_branch>     \ -- addr
\ *G Lay the code for a forward referenced unconditional branch.
\ ** INTERNAL.
  compile branch  >mark
;

: c_?branch>    \ -- addr
\ *G Lay the code for a forward referenced conditional branch.
\ ** INTERNAL.
  compile ?branch  >mark
;


\ ***************************
\ *S Main structure compilers
\ ***************************

: c_lit		\ lit --
\ *G Compile the code for a LITERAL.
\ ** INTERNAL.
  compile lit  ,
;

: c_drop        \ --
\ *G Compile the code for *\fo{DROP}.
\ ** INTERNAL.
  compile drop
;

: c_exit        \ --
\ *G Compile the code for *\fo{EXIT}.
\ ** INTERNAL.
  [ $e8b08000 rsp-reg $10 <<n or ] literal ,	\ ldmfd rsp ! { pc }
;

: c_do          \ C: -- do-sys ; Run: n1|u1 n2|u2 -- ; R: -- loop-sys
\ *G Compile the code for *\fo{DO}.
\ ** INTERNAL.
  compile (do) >mark <mark
;

: c_?DO         \ C: -- do-sys ; Run: n1|u1 n2|u2 -- ; R: -- | loop-sys
\ *G Compile the code for *\fo{?DO}.
\ ** INTERNAL.
  compile (?do) >mark <mark
;

: c_LOOP        \ C: do-sys -- ; Run: -- ; R: loop-sys1 -- | loop-sys2
\ *G Compile the code for *\fo{LOOP}.
\ ** INTERNAL.
  compile (loop) <resolve >resolve
;

: c_+LOOP       \ C: do-sys -- ; Run: -- ; R: loop-sys1 -- | loop-sys2
\ *G Compile the code for *\fo{+LOOP}.
\ ** INTERNAL.
  compile (+loop) <resolve >resolve
;

variable NextCaseTarg	\ -- addr
\ *G Holds the entry point of the current *\fo{CASE} structure.
\ ** INTERNAL.

: c_case        \ -- addr
\ *G Compile the code for *\fo{CASE}.
\ ** INTERNAL.
  NextCaseTarg @  <mark NextCaseTarg !
;

: c_OF          \ C: -- of-sys ; Run: x1 x2 -- | x1
\ *G Compile the code for *\fo{OF}.
\ ** INTERNAL.
  compile (of) >mark
;

: c_ENDOF	\ C: case-sys1 of-sys -- case-sys2 ; Run: --
\ *G Compile the code for *\fo{ENDOF}.
\ ** INTERNAL.
  c_branch> swap >c_res_branch
;

: FIX-EXITS     \ n1..nn --
\ *G Compile the code to resolve the forward branches at the end
\ ** of a *\fo{CASE} structure.
\ ** INTERNAL.
  begin
    sp@ csp @ <>
  while
    >c_res_branch
  repeat
;

: c_ENDCASE     \ C: case-sys -- ; Run: x --
\ *G Compile the code for *\fo{ENDCASE}.
\ ** INTERNAL.
  c_drop  fix-exits  NextCaseTarg !
;

FullCase? [if]
: c_END-CASE    \ C: case-sys -- ; Run: x --
\ *G Compile the code for *\fo{END-CASE}.
\ ** INTERNAL. Only compiled if equate *\fo{FullCase?} is non-zero.
  fix-exits  NextCaseTarg !
;

: c_NEXTCASE    \ C: case-sys -- ; Run: x --
\ *G Compile the code for *\fo{NEXTCASE}.
\ ** INTERNAL. Only compiled if equate *\fo{FullCase?} is non-zero.
  c_drop  NextCaseTarg @ c_branch<  fix-exits  NextCaseTarg !
;

: c_?OF         \ C: -- of-sys ; Run: flag --
\ *G Compile the code for *\fo{?OF}.
\ ** INTERNAL. Only compiled if equate *\fo{FullCase?} is non-zero.
  c_?branch>
;
[then]

external


\ ****************
\ *S Miscellaneous
\ ****************

CLZ? [if]
code clz	\ x -- #lz
\ *G Count the number of leading zeros in x.
\ ** The equate *\fo{CLZ?} must be set true to compile this word.
   mov .s r2, tos
   mov .eq tos, # #1
   mov .ne tos, # #0
   mov .s r1, r2, lsr # #16
   add .eq tos, tos, # #16
   mov .eq r1, r2
   mov .s r2, r1, lsr # #8
   add .eq tos, tos, # #8
   mov .eq r2, r1
   mov .s r1, r2, lsr # #4
   add .eq tos, tos, # #4
   mov .eq r1, r2
   mov .s r2, r1, lsr # #2
   add .eq tos, tos, # #2
   mov .eq r2, r1
   mov .s r1, r2, lsr # #1
   add .eq tos, tos, # #1
   next,
end-code
[then]

((
32bit? [if] 32 [else] 16 [then] equ #bits	\ -- u
\ *G Number of bits in a cell.

: dsqrt		\ d -- n
 \ *G Single square root of a double number.
 \ ** Taken from comp.lang.forth and we apologise
 \ ** for losing the attribution to the original author.
  { | a p -- }
  0 to a  0 to p			\ a=error accumulator, p=result
  #bits 0 do				\ -- d
    a 2* over 0< if 1+ then to a	\ shift MSB of d into a
    2dup d+				\ shift d
    a 2* over 0< if 1+ then to a	\ shift MSB of d into a
    2dup d+				\ shift d
    p 2* to p				\ shift p one place
    a p 2* > if
      a p 2* 1+ - to a
      p 1+ to p
    then
  loop
  2drop  p
;
))

DSQRT? [if]
code dsqrt	\ +d -- n
\ *G Single square root of a double number. 62 bits -> 31 bits.
\ ** The equate *\fo{DSQRT?} must be set true to compile this word.
  ldr     r0, [ psp ], # 4		\ r1:r0=d
  mov     r1, tos
  mov     r2, # 0			\ r2=error accumulator
  mov     tos, # 0			\ tos=result
  mov     r3, # #32			\ loop counter
  begin,
\ shift d two bits lift, accumulating the shifted out bits in erracc
    add .s  r0, r0, r0			\ top bit of r1:r0
    adc .s  r1, r1, r1
    adc     r2, r2, r2			\ ends up in bit 1 of erracc
    add .s  r0, r0, r0			\ top bit of r1:r0
    adc .s  r1, r1, r1
    adc     r2, r2, r2			\ ends up in bit0 of erracc
\ shift result one bit left
    mov     tos, tos lsl # 1
\ compare erracc against 2*res
    cmp     r2, tos lsl # 1
    gt, if,
      sub     r2, r2, tos lsl # 1
      sub     r2, r2, # 1
      add     tos, tos, # 1
    endif,
    sub .s  r3, r3, # 1
  eq, until,
  next,
end-code
[then]

: setMask	\ value mask addr --
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;


\ ******
\ *> ###
\ ******

