\ ADuc7024 - standalone Forth control file

((
Copyright (c) 2004, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20090604 DVA001 Changed SYSTEM-SPEED for current EVBs.
))


\ =========
\ *! aduc7024
\ =========
\ *T Standalone Forth for ADuC7024 Flash
\ *P The control file ADUC7024.CTL generates a standalone
\ ** Forth for Analog Devices ADuC7024 systems.

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LeafCalls			\ shorter and faster
  -LongCalls			\ do not permit standalone Forth to handle
  				\ calls outside 26 bit range.

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ Crystal, clock, serial and ticker rates
#32768 equ xtal-speed		\ crystal clock rate in HZ.
\ #45088000 equ system-speed	\ System clock rate in HZ. Original EVB
  #41780000 equ system-speed	\ System clock rate in HZ. Current EVB
  0 equ CDval			\ core speed is 45.088MHz/(2^CD)
  1 equ 2^CD			\ needed by UART baud rate generator, must match 2^CDval
#38400 equ console0-speed	\ Serial port speed in BPS.
0 equ console-port		\ Designate comms port for terminal (0..n).
				\ 0 is on-chip UART
#10 equ tick-ms			\ TIMEBASE tick in ms

\ define stack and user area sizes for TASKs
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ **********
\ Memory map
\ **********

  $00000000 equ link-address	\ Address at which code will start\be linked at
  $00000000 $0000F7FF cdata section ADuC7020	\ code section
  $00010000 $000107FF idata section PROGd	\ 2k IDATA
  $00010800 $00011FFF udata section PROGu	\ 4k UDATA

interpreter
: prog ADuC7020 ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrAD702x  		\ Special function registers.
  include %HwDir%\initAD702x		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  0 equ ARM-VIC?			\ no vectored controller
  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\Drivers\serAD702xi	\ interrupt RX serial driver
\  include %CpuDir%\Drivers\serAD702xp	\ polled serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
[then]
  include %CpuDir%\drivers\TickAD702x	\ timer tick

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ *S End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE ARM ANS ROM PowerForth v6.20 "
;
external

((
: ANS-FORTH	\ -- ; marker
;
))


\ *******************
\ *S Application code
\ *******************

  include %CpuDir%\Drivers\miscAD702x	\ miscellaneous drivers


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

((
\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
sec-base 0				\ -- addr 0
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $18 + @ +
  over $1C + @ +
  negate
swap $14 + !
))

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

FINIS


\ ======
\ *> ###
\ ======

