\ ADuc7020 - standalone Forth control file

((
Copyright (c) 2004, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20090604 DVA001 Changed SYSTEM-SPEED for current EVBs.
))


\ =========
\ *! aduc7020uf
\ =========
\ *T Umbilical Forth for ADuC7020 Flash
\ *P The control file ADUC7024UF.CTL generates an Umbilical
\ ** Forth for Analog Devices ADuC7020 systems.

only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-heads			\ no word headers at all
  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LeafCalls			\ shorter and faster
  -LongCalls			\ do not permit standalone Forth to handle
  				\ calls outside 26 bit range.

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

\ Do we need interactive debugging over the XTL?
 1 equ umbilical?		\ true if debugging over XTL
   $40 Equ max-depth		\   max depth for UF data stack before UF barfs
   0 equ SPI?			\   true if XTL over SPI port

\ Does the initialised data area actually need to be initialised?
 1 equ init-idata?		\ required for INTARM and other extensions!

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 0 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ Crystal, clock, serial and ticker rates
#32768 equ xtal-speed		\ crystal clock rate in HZ.
\ #45088000 equ system-speed	\ System clock rate in HZ. Original EVB
  #41780000 equ system-speed	\ System clock rate in HZ. Current EVB
  0 equ CDval			\ core speed is 45.088MHz/(2^CD)
  1 equ 2^CD			\ needed by UART baud rate generator, must match 2^CDval
#38400 equ console0-speed	\ Serial port speed in BPS.
1 equ console-port		\ Designate comms port for terminal (0..n).
				\ 0 is on-chip UART
#10 equ tick-ms			\ TIMEBASE tick in ms

\ define stack and user area sizes for TASKs
$0040 equ UP-SIZE		\ size of each task's user area
$00E0 equ SP-SIZE		\ size of each task's data stack
$00E0 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused



\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ **********
\ Memory map
\ **********

  $00000000 equ link-address	\ Address at which code will start\be linked at
  $00000000 $0000F7FF cdata section ADUF7020	\ code section
  $00010000 $000107FF idata section PROGd	\ 2k IDATA
  $00010800 $00011FFF udata section PROGu	\ 4k UDATA

interpreter
: prog ADUF7020  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrAD702x  		\ Special function registers.
  include %HwDir%\initAD702x		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\MinARM		\ low level kernel definitions
  include %CommonDir%\UFkernel		\ high level kernel definitions
  0 equ ARM-VIC?	\ -- 0 ; no vectored controller
  1 equ test-isr?	\ -- n ; non-zero to compile test code
  1 equ SimpleAborts?	\ -- n
  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\Drivers\serAD702xi	\ interrupt RX serial driver
  include %CpuDir%\Drivers\miscAD702x	\ miscellaneous drivers, inc. Flash
Umbilical? [if]
  Synonym wait-byte  serkey		\ Say which XTL drivers to use
  Synonym send-byte  seremit
  Synonym Wait-Byte? serkey?
  Synonym Init-XTL   Init-Ser
  Include %CommonDir%\messages		\ Equates only
\  Include %CommonDir%\Targend		\ Message driver
  Include %HwDir%\TargendFlash		\ Message driver
[Endif]
  UFconsole constant console

\  include %CommonDir%\methods		\ target support for methods
\  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
[then]
  include %CpuDir%\drivers\TickAD702x	\ timer tick

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]


\ *************
\ *S End of kernel
\ *************

: .CPU          \ -- ; sign on
  ." MPE ARM ANS "
  [ tasking?   ] [if] ." Multi-Tasking " [else] ." Single-Task " [then]
  [ Umbilical? ] [if] ." Umbilical "     [else] ." Production "  [then]
  ." Forth"
;


\ *******************
\ *S Application code
\ *******************


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\UFlib
end-libs

Umbilical? [if]
  make-turnkey run-umbilical            \ in TARGEND.FTH
  "" COM1" console0-speed serial        \ specified required target link
  umbilical-forth                       \ start interactive session
[else]
  Make-turnkey Run                      \ application start up word
[then]

FINIS


\ ======
\ *> ###
\ ======

