\ 28F320.FTH - Intel StrataFlash 28F320B3 Flash driver HPO 25 Apr 2003
\ One chip operated in 16 bit mode.

((
Copyright (c) 2000, 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============

))

\ ************
\ *! 28F320
\ *T 28F320 Flash programming
\ ************

\ *P This code is designed for use with a single Intel 28F320 2M x 16 bit Flash chip
\ ** on a 16 bit bus. The size of each sector is 64K bytes,
\ ** To configure 16 bit mode on the CDB89712 board :
\ ** Link ( low ) JP27, unlink ( high ) JP28.
\ ** Unlink J15 ( high ), link pins 1+2 on JP16, JP17 and JP18.

\ *P This code is hardware and CPU dependent because of bus width
\ ** and cache considerations.

\ *P Compiler and Interpreter macros must be set up before
\ ** this code is compiled to force an uncached address. Some
\ ** CPUs, e.g. MIPS and some ARMs use a particular address
\ ** range to determine whether the access should be cached
\ ** or uncached. During Flash programming, ALL accesses to
\ ** the flash should be uncached and write buffers should
\ ** be turned off.
\ ** The following is for for a Samsung S3C4510B CPU.
\ *E compiler
\ ** : ForceUncached        \ addr -- addr'
\ **   bit26 or  ;
\ ** target
\ ** interpreter
\ ** : ForceUncached        \ addr -- addr'
\ **   bit26 or  ;
\ ** target

\ *P If your CPU does not have a cache or if caching is
\ ** never enabled for Flash access, use the following code.
\ *E compiler
\ ** : ForceUncached    ;   \ addr -- addr'
\ ** target
\ ** interpreter
\ ** : ForceUncached    ;   \ addr -- addr'
\ ** target

\ *P Note that when programming these devices, programming part
\ ** of a ( two chip ) sector ( 256K byte ) will leave the unprogrammed parts
\ ** set to bytes of FF.

\ *P Note : the 28F320B3 does not have Lock bits but the 28F320J3 does.

decimal

\ *************
\ *S Primitives
\ *************

variable vFlashBase
\ *G the address of the block of Flash that we are accessing

: FlCmd     \ command --
\ *G write a command to the Flash chips ( both simultaneously ).
   vFlashBase @ c!  \ command
;

: FlWaitForReady    \ -- status
\ *G read the status of the chip and clear if required
\   $70 FlCmd                           \ read status command - NOT REQUIRED ( ! )
   begin
      vFlashBase @ @ 0x0080 and     \ read the status of the chip
   0x0080 = until                   \ SR.7 is true from the chip = ready
   vFlashBase @ @ 0x007E and        \ check error bits for the chip 0, 0 = OK
   dup if  $50 FlCmd  then              \ Clear Status
   $FF FlCmd \ Reset both Flash chips to normal "Read Array" mode simultaneously.
;

: FlClearLockBits       \ -- status ;  0 for success
\ *G Erase the sector at the given address on both Flash chips, 128K bytes each.
\  [I
   $50 FlCmd            \ Clear Status ( just in case )
   $60 FlCmd            \ select lock bits
   $D0 FlCmd            \ clear all lock bits
   FlWaitForReady
\  I]
;

: FlSetLockBit       \ addr -- status ;  0 for success
\ *G Erase the sector at the given address on both Flash chips, 128K bytes each.
\  [I
   vFlashBase @ >r
   vFlashBase !
   $50 FlCmd            \ Clear Status ( just in case )
   $60 FlCmd            \ select lock bits
   $01 FlCmd            \ set the lock bit for the sector at addr
   FlWaitForReady
   r> vFlashBase !
\  I]
;

: FlErase       \ addr -- status ;  0 for success
\ *G Erase the sector at the given address on the Flash chip, 64K bytes each sector,
\ ** except for the first 64K which is divided up into 8 x 8K chunks.
\  [I
   vFlashBase !
\   FlClearLockBits      \ the lock bits will be set \ 28F640J3 needs this, 28F320B3 does not as it has no lock bits
   $50 FlCmd            \ Clear Status ( just in case )
   $20 FlCmd            \ erase setup
   $D0 FlCmd            \ confirm erase command
   FlWaitForReady
\   or                  \ 28F640J3 needs this, 28F320B3 does not as it has no lock bits
\  I]
;

((
\ Halfword access using the special opcodes
CODE W@h    \ a-addr -- w
\ *G Fetch and 0 extend the word (16 bit) at memory ADDR.
  ldrh tos, [ tos ]
  next,
END-CODE

CODE W!h    \ w a-addr --
\ *G Store the word (16 bit) quantity w at memory ADDR.
  ldmfd psp ! { r0 }
  strh r0, [ tos ]
  ldmfd psp ! { tos }
  next,
END-CODE
))

: FlWrite16        \ w addr -- flag ; 0 for success
\ *G Write 16 bits to the Flash chip at the given address.
\  [I
   vFlashBase !
   $10 FlCmd            \ write command
   vFlashBase @ w!       \ write the 16 bits of data
   \ Note : Must use a 16 bit halfword store, not two c!'s ARMTDM not ARM7 .
   FlWaitForReady
\  I]
;

: FlWrite         \ n addr -- flag ; 0 for success
\ *G Write 32 bits to the Flash chip at the given address.
\  [I
   vFlashBase !
   $10 FlCmd            \ write command
   dup vFlashBase @ w!       \ write the low 16 bits of data
   FlWaitForReady >r
   $10 FlCmd            \ write command
   0x10 rshift vFlashBase @ 2+ w!       \ write the high 16 bits of data
   FlWaitForReady r> or
\  I]
;

: FlId      \ addr -- manid0 devid0
\ *G read identifier codes for the chip
\ ** $89 $16 for the Intel 28F128
\ ** $89 $17 for the Intel 28F640
\ ** $89 $18 for the Intel 28F320
\ ** $89 $8897 for the Intel 28F320 bottom boot block, $8896 for top boot block
\ ** device.
\   [I
   vFlashBase !
   $90 FlCmd                         \ read ID command for the chip
   vFlashBase @ $0000 + w@           \ manufacturer's ID for chip 0
   vFlashBase @ $0004 + w@           \ device IDfor chip 0 16 bit mode

\   vFlashBase @ 2+ $0000 + w@        \ manufacturer's ID for chip 1
\   vFlashBase @ 2+ $0004 + w@        \ device ID for chip 1
   $FF FlCmd \ Reset both Flash chips to normal "Read Array" mode simultaneously.
\   I]
;

: FlBlk      \ addr -- blkstat0 
\ *G read the block lock status for the chip
\   [I
   vFlashBase @ >r
   vFlashBase !
   $90 FlCmd                         \ read ID command for the chip
   vFlashBase @ $0004 + c@           \ block lock status for chip 0
\   vFlashBase @ 2+ $0008 + c@        \ block lock status for chip 1
   $FF FlCmd \ Reset both Flash chips to normal "Read Array" mode simultaneously.
   r> vFlashBase !
\   I]
;

create sectab   \ -- addr
\ *G Table containing the number of sectors and starting offset
\ ** of each sector, plus a dummy start address which enables
\ ** the size of the last sector to be calculated.
  #8 ,
  $00000000 ,  $00010000 ,  $00020000 ,  $00030000 ,
  $00040000 ,  $00050000 ,  $00060000 ,  $00070000 ,
  $00080000 ,               \ dummy sector start

: SectorN   \ n -- addr len
\ *G Convert sector number to base address and length
  1+ cells sectab + 2@          \ -- end start
  tuck -
;

((
: nErase    \ n --
\ *G Erase sector n of the Flash.
  SectorN drop SecErase
;
))

: FindSector    \ addr -- start len
\ *G Find the start and length of the sector containing address
\ ** addr. If len is zero, the sector was not found and start
\ ** is set to addr.
  vFlashBase @ - 0             \ convert to offset, assume failure
  sectab @ 0 ?do
    over i SectorN over + within if \ test sector
      2drop i SectorN
      leave
    endif
  loop
  swap vFlashBase @ + swap         \ make offset absolute
;
((
[undefined] .lword [if]
: .lword    \ x --
\ *G Display x as an 8 digit hexadecimal number in the form
\ ** "abcd:efgh".
  base @ hex  swap
  0 <# # # # # ascii : hold # # # # #> type
  base !
;
[then]

[undefined] .byte [if]
: .byte     \ x --
\ *G Display x as a 2 digit hexadecimal number in the form
\ ** "ab".
  base @ hex  swap
  0 <# # # #> type
  base !
;
[then]

: .src/dest \ src dest -- src dest
\ *G Display the addresses and contents of the source
\ ** and destination.
  ."  source: " over .lword  over c@ ." (" .byte ." )"
  ."  dest: " dup .lword  dup c@ ." (" .byte ." )"
;

\ *************************
\ *S User/Application words
\ *************************
\ *P These words have heads by default and are in the target
\ ** dictionary.

external

: (EraseFlash)  \ dest dlen --
\ *G Erase the flash sectors covering dlen bytes at dest.
  ?dup if
    swap ForceUncached swap     \ *** force bypass of cache **
    over + 1- >r            \ -- dest ; R: -- end
    begin               \ -- addr ; R: -- end
      cr ."  Erasing sector at: " dup .lword
      dup SecErase          \ erase current sector
      FindSector +          \ step to start of next sector
      dup r@ u>             \ until out of range
    until
    drop  r> drop
  endif
;

: (ProgFlash)   \ src dest len --
\ *G Write len bytes from memory at src to Flash at dest.
\ ** Note that the Flash is assumed to be erased, and that
\ ** no verification is performed except when each byte is
\ ** programmed.
  swap ForceUncached swap       \ *** force bypass of cache **
  0 ?do
    over c@ over write8 if
      cr ."  Write failed:" .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    1 + swap  1 + swap
  1 +loop
  2drop
;

: (VerifyFlash) \ src dest len --
\ *G Verify len bytes from memory at src to Flash at dest.
  swap ForceUncached swap       \ *** force bypass of cache **
  3dup s= if  3drop exit  endif
  cr ."  Verify failed"
  0 ?do
    over c@ over c@ <> if
      cr 2 spaces .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    1 + swap  1 + swap
  1 +loop
  2drop
;

: ProgramFlash  \ src dest len --
\ *G Using the memory buffer at src, program the Flash at dest
\ ** with len bytes. The relevant Flash sectors are erased, the
\ ** Flash is programmed, and the result verified.
  cr ." Erasing " 2dup (EraseFlash)
  cr ." Programming "  3dup (ProgFlash)
  cr ." Verifying "  (VerifyFlash)
;
))

\ ******
\ *> ###
\ ******

