\ BootLd.fth  20 May 2003 Boot loader for CS89712
\ This version loads into SDRAM at 0xC0000000
\ Use in conjunction with the CS89712.com PC program.
\ Connect the CS89712's serial port A ( next to Ethernet socket ) to PC COM1.

\ ***********************************************************
\ CS89712 polled RX, polled TX, serial communication drivers
\ ***********************************************************

: init-ser  \ --
\ *G Initialise both UARTs, change the baud rate to 115.2k.

   0x000E0001 UBRLCR1 !         \ 115200 baud
\   0x000E0001 UBRLCR2 !         \ 115200 baud

   0x00000100 SYSCON1 !         \ UART1EN on ( bit 8 ) , DBGEN off ( bit 11 )
\   0x00000100 SYSCON2 !         \ UART2EN on ( bit 8 ) , DBGEN off ( bit 11 )

\   0x000E0005 UBRLCR1 !         \ 38400 baud
\   0x000E000B UBRLCR1 !         \ 19200 baud
\   0x000E0017 UBRLCR1 !         \ 9600 baud
;

\ *****************
\ Serial primitives
\ *****************

: seremitWait0      \ --
   begin
\      pause
      SYSFLG1 @ 0x00000800 and 0=    \ bit 11
   until
;

: seremit0      \ char --
   seremitWait0
   UARTDR1 c!
;

: sertype0      \ c-addr len --
   bounds
   ?do  i c@ seremit0  loop
;

: serkey?0      \ -- flag
  SYSFLG1 @ 0x00400000 and 0=     \ bit 22
;

: serkey0       \ -- char
   begin
\      pause
   serkey?0  until
   UARTDR1 w@  $FF and
;

: serCR0   \ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D seremit0  $0A seremit0
;

variable RAMptr     \ pointer to next byte of RAM to store data to

: DEPTH         \ -- +n
( *G Return the number of items on the data stack, excluding the count. )
  INIT-S0 sp@ - cell- cell /
;

: .char         \ c --
   0x0F and  dup 0x09 > if  0x07 + then  [char] 0 + seremit0
;

: .dig          \ c --
   dup 4 Rshift .char
                .char
;

: UploadBlock    \ --
\ *G upload 1024 bytes to current RAMptr address, update RAMptr.
   [char] { seremit0
    1024 0 do  serkey0 RAMptr @ c!  1 RAMptr +!  loop
    DEPTH 1 - if  depth .dig  then
   [char] } seremit0
;

code JumpToProgram      \ --
\ *G jump to the start of the downloaded program at RAM start + 2K.
  mvl R0, # $C0000000                   \ use macro to load full 32 bit address
  mov pc, R0                            \ into R0 ( scratch ), then put into pc
END-CODE

: BootLoader

   0xC0000000 RAMptr !

   serkey0  serkey0 8 lshift or  2047 -        \ end address, not count

   0 ?do
      serkey0 RAMptr @ c!  1 RAMptr +!
   loop

   JumpToProgram
;


((
: BootLoader

   0x10000800 RAMptr !

   UploadBlock

   begin
      serkey0 >r
         [char] Q r@ = if
            JumpToProgram
         else
            [char] { r@ = if
               UploadBlock
            else
               seremit0
            then
         then
      r> drop
   again
;
))

\ : COLD  BootLoader ;

