\ FLASHjmp.ctl  HPO  2 Jun 2003 Control file to create a FLASH jump table

((
  The code is set up to run from FLASH at address 0x00000000
  and initialises the external SDRAM at 0xC0000000 to 0xC03FFFFF
  then jumps to a corresponding table in SDRAM
))

only forth definitions

\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ include macros     \ compiler and assembler macros

\ FILE: SYMBOLS.LOG           \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790
1 equ CPU=ARM           \ enables some CPU specific code
  32bit-mode            \ running in 32 bit mode

\ *****************
\ default constants
\ *****************

cell equ cell               \ size of a cell (16 bits)
0 equ false
-1 equ true

\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?         \ true for ARM AIF format

\ Kernel components
 0 equ ColdChain?       \ true to use the ColdChain mechanism
 0 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size     \   for internal consistency check
   0 equ event-handler?     \   true to include event handler
   0 equ message-handler?   \   true to include message handler
   0 equ semaphores?        \ true to include semaphores
 0 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?          \ true for software floating point
 0 equ FullCase?        \ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?       \ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?                  \ true if BLOCK needed
 $0000 equ sizeofheap       \ 0=no heap, nz=size of heap
   0 equ heap-diags?        \   true to include diagnostic code
 0 equ paged?           \ true if ROM or RAM is paged/banked
 0 equ MPE-SET?         \ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?     \ true if ANS ENVIRONMENT system required

 \ Clock, serial and ticker rates
#18240000 equ system-speed      \ System clock rate in HZ.
#9600 equ console0-speed        \ Serial port speed in BPS.
#9600 equ console1-speed        \ Serial port speed in BPS.
\ #9600 equ console2-speed      \ Serial port speed in BPS.
0 equ console-port              \ Designate serial port for terminal.
\ #2 equ tick-ms                \ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel     \ x in Vx.yz
 char 1 equ mpe-ver     \ y in Vx.yz
 char 0 equ usrver      \ z in Vx.yz

\ define stack and user area sizes
$0080 equ UP-SIZE               \ size of each task's user area    ( was 100 )
$0080 equ SP-SIZE               \ size of each task's data stack   ( was 200 )
$0080 equ RP-SIZE               \ size of each task's return stack ( was 200 )
up-size rp-size + sp-size +
  equ task-size         \ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#10 equ sp-guard

$0080 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting level for interrupts and SWIs
3 equ #IRQs         \ number of IRQ stacks, shared by all IRQs (1 min) ( was 4 )
2 equ #SWIs         \ number of SWI nestings permitted (0 is ok)       ( was 1 )

\ ********************
\ Define memory layout
\ ********************

$00000000 equ link-address      \ for a binary image - usually starts at zero on the ARM
                    \ Used by the AIF header

\ Flash is one Intel 28F320B3
\ $00000000 $003FFFFF cdata section FLASHjmp  \ 4 Mbyte for Application
\ $C01E0000 $C01EFFFF idata section PROGd    \ 64K initialised RAM and dictionary
\ $C01F0000 $C01FFFFF udata section PROGu    \ 64K uninitialised RAM
\ $70000000 $70FFFFFF cdata section PROGboot \ 128 byte boot program

\ Flash is one Intel 28F320B3
$00000000 $003FFFFF cdata section FLASHjmp  \ 4 Mbyte for Application
$60009800 $60009FFF idata section PROGd  \ 1K initialised RAM and dictionary
$6000A000 $6000B7FF udata section PROGu  \ 6K uninitialised RAM

FLASHjmp sec-base  equ flashbase       \ -- addr ; Base address of flash memory after mapping
sec-len     equ flashlen        \ -- u ; size of Flash
\ Where the boot program is in flash and where it is run
flashbase   equ BootFlash       \ -- addr ; base address of boot Flash after mapping
FLASHjmp sec-len   equ BootLen     \ -- u ; length of boot Flash
FLASHjmp sec-base  equ BootRAM     \ -- addr ; base address of boot code after mapping

interpreter
: prog  FLASHjmp  ;        \ alias primary code section - PROG is default name
target

PROG PROGd PROGu  CDATA     \ use Code for HERE , and so on

\ **********************************
\ USER area and Multi tasker equates
\ **********************************

\ Main User/Task stack for USR/SYS mode operation.
\ In order to permit fast interrupt nesting the
\ return stack must be the lowest of RSP, PSP and UP
\ In order for the initialisation code in MULTIARM.FTH
\ to work, INIT-U0 must be the highest of INIT-UO,
\ INIT-S0 and INIT-R0.
rp-size sp-size + equ TASK-U0       \ initial offset of user area
rp-size sp-size + equ TASK-S0       \ initial offset of data stack
rp-size           equ TASK-R0       \ initial offset of return stack

task-size reserve equ INIT-T0       \ base of main task area
  init-t0 task-u0 + equ INIT-U0     \ base of main user area
  init-t0 task-s0 + equ INIT-S0     \ top of main data stack
  init-t0 task-r0 + equ INIT-R0     \ top of main return stack

\ SVC stack ; nestable up to #SWIs
0 reserve equ SVC_StackTop      \ top of stack
task-size #SWIs * reserve equ SVC_StackBase \ bottom of stack

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_StackTop      \ top of IRQ stacks
task-size #IRQs * reserve       \ bottom of IRQ stacks
  equ IRQ_StackBase
IRQ_StackTop equ IRQ_Stack_Top
IRQ_StackBase equ IRQ_Stack_Base

\ FIQ stack
0 reserve equ FIQ_StackTop      \ top of FIQ stack
task-size reserve equ FIQ_StackBase \ bottom of FIQ stack

\ UND stack
0 reserve equ UND_StackTop      \ top of stack
task-size reserve equ UND_StackBase \ bottom of stack

\ ABT stack
0 reserve equ ABT_StackTop      \ top of stack
task-size reserve equ ABT_StackBase \ bottom of stack

\ TIB - terminal input buffer
tib-len reserve equ INIT-TIB        \ base of TIB

PROGd  sec-top 1+ equ UNUSED-TOP  PROG  \ top of memory for UNUSED

\ ************
\ Kernel files
\ ************

   include sfr89712             \ CS89712 Special function registers.
   include initjmp              \ Generic startup code (*required*).

: ANS-FORTH \ -- ; marker
;


\ ****************************
\ *S Add application code here
\ ****************************


((
BuildFile Build.no
create version$ \ -- addr
\ *G Counted string holding the version number. This version number
\ ** is automatically updated after each successful build.
  Build$,
))
((
: .signon   \ --
\ *G System sign on. Displays system, version and author information.
  cr ." Phoenix ARM board"
  cr ." Inventio Software Ltd "  Version
;
\ ' .signon AtCold
))

\ *********************
\ *S End of compilation
\ *********************

\ ****************
\ *S Sanity checks
\ ****************

decimal

((
  cr ." Required USER size is     : " next-user @ .
  cr ." Current USER allocation is: " up-size .
Next-user @ up-size > [if]
\ *G Check that the USER area is large enough.
  cr ." *** Increase USER area size UP-SIZE in control file ***
  abort
[then]
))

\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words
((
\ ********
\ All done
\ ********

decimal
update-build

cr
cr ." ==============================="
cr ." Build:" version$ count type
cr ." ==============================="
cr
))

FINIS

\ ******
\ *> ###
\ ******

