\ ForthFl.ctl  HPO  2 Jun 2003 Control file to an Axotec ARM Forth to run in FLASH at 0

((
  The code is set up to run from FLASH at address 0x00000000
  using external SDRAM at 0xC0000000 to 0xC03FFFFF
))

only forth definitions

\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include macros     \ compiler and assembler macros

\ FILE: SYMBOLS.LOG           \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790
1 equ CPU=ARM           \ enables some CPU specific code
  32bit-mode            \ running in 32 bit mode

\ *****************
\ default constants
\ *****************

cell equ cell               \ size of a cell (16 bits)
0 equ false
-1 equ true

\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?         \ true for ARM AIF format

\ Kernel components
 1 equ ColdChain?       \ true to use the ColdChain mechanism
 1 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size     \   for internal consistency check
   0 equ event-handler?     \   true to include event handler
   0 equ message-handler?   \   true to include message handler
   0 equ semaphores?        \ true to include semaphores
 0 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?          \ true for software floating point
 0 equ FullCase?        \ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?       \ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?                  \ true if BLOCK needed
 $0000 equ sizeofheap       \ 0=no heap, nz=size of heap
   0 equ heap-diags?        \   true to include diagnostic code
 0 equ paged?           \ true if ROM or RAM is paged/banked
 0 equ MPE-SET?         \ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?     \ true if ANS ENVIRONMENT system required

 \ Clock, serial and ticker rates
#18240000 equ system-speed      \ System clock rate in HZ.
#9600 equ console0-speed        \ Serial port speed in BPS.
#9600 equ console1-speed        \ Serial port speed in BPS.
\ #9600 equ console2-speed      \ Serial port speed in BPS.
0 equ console-port              \ Designate serial port for terminal.
\ #2 equ tick-ms                \ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel     \ x in Vx.yz
 char 1 equ mpe-ver     \ y in Vx.yz
 char 0 equ usrver      \ z in Vx.yz

\ define stack and user area sizes
$0080 equ UP-SIZE               \ size of each task's user area    ( was 100 )
$0080 equ SP-SIZE               \ size of each task's data stack   ( was 200 )
$0080 equ RP-SIZE               \ size of each task's return stack ( was 200 )
up-size rp-size + sp-size +
  equ task-size         \ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#10 equ sp-guard

$0080 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting level for interrupts and SWIs
3 equ #IRQs         \ number of IRQ stacks, shared by all IRQs (1 min) ( was 4 )
2 equ #SWIs         \ number of SWI nestings permitted (0 is ok)       ( was 1 )

\ ********************
\ Define memory layout
\ ********************

$00000000 equ link-address      \ for a binary image - usually starts at zero on the ARM
                    \ Used by the AIF header

\ Flash is one Intel 28F320B3
 $00000000 $003FFFFF cdata section ForthFL  \ 4 Mbyte for Application
 $C01E0000 $C01EFFFF idata section PROGd    \ 64K initialised RAM and dictionary
 $C01F0000 $C01FFFFF udata section PROGu    \ 64K uninitialised RAM
\ $70000000 $70FFFFFF cdata section PROGboot \ 128 byte boot program

\ Flash is one Intel 28F320B3
\ $00000000 $003FFFFF cdata section ForthFL  \ 4 Mbyte for Application
\ $60009800 $60009FFF idata section PROGd  \ 1K initialised RAM and dictionary
\ $6000A000 $6000B7FF udata section PROGu  \ 6K uninitialised RAM

ForthFL sec-base  equ flashbase       \ -- addr ; Base address of flash memory after mapping
sec-len     equ flashlen        \ -- u ; size of Flash
\ Where the boot program is in flash and where it is run
flashbase   equ BootFlash       \ -- addr ; base address of boot Flash after mapping
ForthFL sec-len   equ BootLen     \ -- u ; length of boot Flash
ForthFL sec-base  equ BootRAM     \ -- addr ; base address of boot code after mapping

interpreter
: prog  ForthFL  ;        \ alias primary code section - PROG is default name
target

PROG PROGd PROGu  CDATA     \ use Code for HERE , and so on

\ **********************************
\ USER area and Multi tasker equates
\ **********************************

\ Main User/Task stack for USR/SYS mode operation.
\ In order to permit fast interrupt nesting the
\ return stack must be the lowest of RSP, PSP and UP
\ In order for the initialisation code in MULTIARM.FTH
\ to work, INIT-U0 must be the highest of INIT-UO,
\ INIT-S0 and INIT-R0.
rp-size sp-size + equ TASK-U0       \ initial offset of user area
rp-size sp-size + equ TASK-S0       \ initial offset of data stack
rp-size           equ TASK-R0       \ initial offset of return stack

task-size reserve equ INIT-T0       \ base of main task area
  init-t0 task-u0 + equ INIT-U0     \ base of main user area
  init-t0 task-s0 + equ INIT-S0     \ top of main data stack
  init-t0 task-r0 + equ INIT-R0     \ top of main return stack

\ SVC stack ; nestable up to #SWIs
0 reserve equ SVC_StackTop      \ top of stack
task-size #SWIs * reserve equ SVC_StackBase \ bottom of stack

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_StackTop      \ top of IRQ stacks
task-size #IRQs * reserve       \ bottom of IRQ stacks
  equ IRQ_StackBase
IRQ_StackTop equ IRQ_Stack_Top
IRQ_StackBase equ IRQ_Stack_Base

\ FIQ stack
0 reserve equ FIQ_StackTop      \ top of FIQ stack
task-size reserve equ FIQ_StackBase \ bottom of FIQ stack

\ UND stack
0 reserve equ UND_StackTop      \ top of stack
task-size reserve equ UND_StackBase \ bottom of stack

\ ABT stack
0 reserve equ ABT_StackTop      \ top of stack
task-size reserve equ ABT_StackBase \ bottom of stack

\ TIB - terminal input buffer
tib-len reserve equ INIT-TIB        \ base of TIB

PROGd  sec-top 1+ equ UNUSED-TOP  PROG  \ top of memory for UNUSED

\ ************
\ Kernel files
\ ************

   include sfr89712             \ CS89712 Special function registers.
   include initARM              \ Generic startup code (*required*).

   include codeARM              \ low level kernel definitions

   include Ser89712             \ polled serial driver

   include kernel62             \ high level kernel definitions
   include methods              \ target support for methods

   include devtools             \ DUMP .S etc development tools
\   include voctools             \ ORDER VOCS etc
((
 : Init-Multi ;
 : pause ;
INTERPRETER
 : TASK   POSTPONE VARIABLE ;
TARGET
 : INITIATE   DROP ;
))
   include MultiArm             \ tasks

   include Tik89712             \ timer

\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE ARM ANS ROM PowerForth v6.10 "
;
external

: ANS-FORTH \ -- ; marker
;


\ ****************************
\ *S Add application code here
\ ****************************

task SerialPort2task

: >serial0
   Console0 opvec !  Console0 ipvec !
;

: >serial1
   Console1 opvec !  Console1 ipvec !
;

$80 Buffer: tib2

: SerialPort2
   tib2 'tib !
   0 >in !
   out off
   echoing on
   >serial1
   Debug
   quit
;

: InitSerialPort2task
   ['] SerialPort2 SerialPort2task initiate
;
' InitSerialPort2task AtCold

\ ***********************************************

task Timertask

: (TimerTask)
   0 >in !
   out off
   echoing on
   begin
      100 ms  0x01 PDDR c!
      100 ms  0x00 PDDR c!
   again
;

: InitTimerTask
   ['] (TimerTask) TimerTask Initiate
;
' InitTimerTask AtCold

\ ***********************************************

: ShortDelay
\ *G ~ 10ms delay
   20000 0 do  1 1 + drop  loop
;

CODE Reboot      \ --
\ *G The run time action of unconditional branches compiled on the target.
  mvl R0, # $10000800            \ load address into scratch register using
                                 \ macro instruction.
  mov pc, R0                     \ Then copy into PC
END-CODE

: .char         \ c --
   0x0F and  dup 0x09 > if  0x07 + then  [char] 0 + emit
;

: .dig          \ c --
   dup 4 Rshift .char
                .char
;

: .x    \ u --
\ *G display U as a hex number
    dup 0x18 Rshift .dig
    dup 0x10 Rshift .dig
    dup 0x08 Rshift .dig
                    .dig
    space
;


: DUMP  \ addr len --
   0 DO
      dup 15 AND 0= IF  CR  dup .x  space  THEN
      dup 0x03 AND 0= IF  space  THEN
      DUP @ .dig  space  1+
   LOOP
   DROP
   space
;

include 28F640                  \ Intel StrataFlash

: LongDelay
   cr s" start" type
   100 0 do  ShortDelay  loop
   s"  end" type
;

((
: DEPTH         \ -- +n
( *G Return the number of items on the data stack, excluding the count. )
  INIT-S0 sp@ - cell- cell /
;
))

: RDEPTH         \ -- +n
( *G Return the number of items on the data stack, excluding the count. )
  INIT-R0 rp@ - cell- cell /
;

: Version  ." V1.0  2 Jun 2003 Flash" ;
: who    ." Crystal CDB89712 board, CS89712 ARM7tdmi processor" ;

: .Banner
   cr  who  cr  version
;

\ include blocks
include RAMtests

: InitForthRAM
   .Banner
   0x00000522 SDCONF !  \ initialise the SDRAM controller
\   0xC0010000 DP ! 	\ HERE set SDRAM
;

' InitForthRAM AtCold

: ShowLockBits   \ l+1 f --
   cr s" LockBits for chips 0 and 1 :  " type
   ?do
      cr i .dig  space i 0x40000 * dup .x  vFlashBase @ + space FlBlk swap .dig space  .dig
   loop
;

: .Result       \ result --
   dup .x  if  s"  Failed! "  else  s"  done. "  then  type ;

variable ExitFlag

: Debug
   .Banner
   cr  ." Debug : press Q to quit."

   0x00000000 vFlashBase !              \ Flash is at 0 when running from Flash

   ExitFlag off

   begin
\      pause
\      key? if
         key  DUP [char] a [char] z 1+ within if  223 and  then
         case
             [char] A of  LongDelay  endof
             [char] B of cr s" Boot0" type   0x70000000 0x80 dump  endof
\            [char] C of cr s" Boot1" type   0x70000080 0x80 dump  endof
            [char] D of cr s" Depth = " type  depth .x
            INIT-S0 .x  s" Rdepth = " type Rdepth .x  INIT-R0 .x
            endof

            [char] R of cr s" RAM" type  0xC0000000 0xA0 dump  endof
            [char] Q of cr ."  done. "  ExitFlag on   endof
            [char] S of cr s" RAM" type  0xC0000080 0xA0 dump  endof
            [char] T of cr s" SysFlg1.3 " type  SYSFLG1 @ .x  endof
            [char] I of cr s" ID0,1 :  " type  vFlashBase @ FlId swap 2swap swap .dig space .dig  space space  .dig space .dig  endof
            [char] L of 16 0 ShowLockBits endof
            [char] K of cr s" Set lock bit " type  vFlashBase @ FlSetLockBit 2 0 ShowLockBits  endof

            [char] E of cr s" Erase - I don't think so! " type ( vFlashBase @ FlErase .Result ) endof
            [char] W of cr s" Write - I don't think so! " type ( 0x12385678 vFlashBase @ FlWrite .Result ) endof
            [char] F of cr s" Flash"  type  vFlashBase @ 0x80 dump  endof

            emit
         endcase
\      then
\      TC1D w@ 0x200 and if  0x01  else  0x00  then  PDDR c!  \ flash the LED
      \ 2kHz freerunning counter
\   again
   ExitFlag @ until
;


BuildFile Build.no
create version$ \ -- addr
\ *G Counted string holding the version number. This version number
\ ** is automatically updated after each successful build.
  Build$,

((
: .signon   \ --
\ *G System sign on. Displays system, version and author information.
  cr ." Phoenix ARM board"
  cr ." Inventio Software Ltd "  Version
;
\ ' .signon AtCold
))

\ *********************
\ *S End of compilation
\ *********************

\ ****************
\ *S Sanity checks
\ ****************

decimal

((
  cr ." Required USER size is     : " next-user @ .
  cr ." Current USER allocation is: " up-size .
Next-user @ up-size > [if]
\ *G Check that the USER area is large enough.
  cr ." *** Increase USER area size UP-SIZE in control file ***
  abort
[then]
))

\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

\ ********
\ All done
\ ********

decimal
update-build

cr
cr ." ==============================="
cr ." Build:" version$ count type
cr ." ==============================="
cr


FINIS

\ ******
\ *> ###
\ ******

