\ ARM jump table startup code

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20020610 SFP001 Changed S0 to point to LAST available cell
))

\ -----------------------------------------------------------------------------
\     Generic start up table and entry points
\ -----------------------------------------------------------------------------


\ This code branches by loading the PC with an ABSOLUTE address contained
\ in a location at a RELATIVE offset from the LDR instruction. This avoids
\ the use of a relative branch which may be out of range if the execution
\ address is high in memory.
asmcode
L: Vector_Start                         \ 'Hard wired' ARM Processor vectors
  ldr pc, ResetV                        \ 00 - Reset
  ldr pc, UndefV                        \ 04 - Undefined Instructions
  ldr pc, SWIV                          \ 08 - SWI instructions
  ldr pc, PAbortV                       \ 0C - Instruction fetch Aborts
  ldr pc, DAbortV                       \ 10 - Data access aborts
  ldr pc, UnusedV                       \ 14 - Reserved (address exception if 26bit)
  ldr pc, IRQV                          \ 18 - IRQ interrupts
  ldr pc, FIQV                          \ 1C - FIQ interrupts

\ For faster response speed, you may wish to place your FIQ handler here, replacing
\ the ldr pc, FIQV instruction with the first instruction of the FIQ handler. This saves
\ the ARM from the extra overhead of having to branch to the handler.

\ The following vectors usually end up in RAM after the EPROM/Flash is remapped
\ into high memory and RAM is remapped to low memory.
L: Soft_Vectors                         \ Handler routines can be 'socketed' in here
L: ResetV       dd 0xC0000000                    \ HandlerReset
L: UndefV       dd 0xC0000004                    \ HandlerUndef
L: SWIV         dd 0xC0000008                    \ HandlerSWI
L: PAbortV      dd 0xC000000C                    \ HandlerPAbort
L: DAbortV      dd 0xC0000010                    \ HandlerDAbort
L: UnusedV      dd 0xC0000014                    \ HandlerUnused
L: IRQV         dd 0xC0000018                    \ HandlerIRQ
L: FIQV         dd 0xC000001C                    \ HandlerFIQ
L: Vector_End

L: DefHandlerReset   
  adr r13, # ECLD
  mov pc, r13
end-code

chere vector_start - equ vector_len

DefHandlerReset    ResetV  !

\ -----------------------------------------------------------------------------
\       start of Forth system
\ -----------------------------------------------------------------------------

L: CLD1     \ XT of cold start word is placed here
  0xC0000000 ,     \ jump to start of code in SDRAM
L: DATASTACK
  init-s0 ,
L: RETURNSTACK
  init-r0 ,
L: USERPOINTER
  init-u0 ,
L: REG_SDCONF
  SDCONF ,
L: VAL_0x522
  0x522 ,
L: EXTRAM_START_ADDRESS
   0xC0000000 ,

asmcode
L: WHILE(1)
   mov pc, # 0xC0000000

L: ECLD

\ enable the SDRAM
  ldr psp, REG_SDCONF
  ldr rsp, VAL_0x522
  str rsp, [ psp ] 
		   
\ copy the smallest possible program into SDRAM
\ this will loop until the Lauterbach loads the real program into SDRAM
  ldr psp, EXTRAM_START_ADDRESS
  ldr rsp, WHILE(1)
  str rsp, [ psp ] 
					       
\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK                  \ set return stack
  ldr up, USERPOINTER                   \ set user pointer
  ldr pc, CLD1                  \ get address of cold start word and jump
end-code

\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------
((
[undefined] sp-guard [if]
0 equ sp-guard
[then]

\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,  \ s0
  init-r0 ,             \ r0
  0 ,  0 ,                              \ #tib, tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
))
