\ lapload.ctl  HPO 20 May 2003 Control file to build the 2K CS89712 boot loader
\ This version initialises the SDRAM and loads into it at 0x60000000

((
The boot code is set up to run in a 2k section of RAM
  $1000:0000 $1000:07FF
  which is loaded by the on-chip boot rom program
  When using the on-chip boot program ( nMEDCHG pin is low ), the
  48K internal SRAM is mapped to :
  $1000:0000 $1000:BFFF
  and the 16 MByte FLASH ( CS0  ) is mapped to :
  $7000:0000 $70FF:FFFF
))

only forth definitions

\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include macros     \ compiler and assembler macros

\ FILE: SYMBOLS.LOG           \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790
1 equ CPU=ARM           \ enables some CPU specific code
  32bit-mode            \ running in 32 bit mode

\ *****************
\ default constants
\ *****************

cell equ cell               \ size of a cell (16 bits)
0 equ false
-1 equ true

\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?         \ true for ARM AIF format

\ Kernel components
 0 equ ColdChain?       \ true to use the ColdChain mechanism
 0 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size     \   for internal consistency check
   0 equ event-handler?     \   true to include event handler
   0 equ message-handler?   \   true to include message handler
   0 equ semaphores?        \ true to include semaphores
 0 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?          \ true for software floating point
 0 equ FullCase?        \ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?       \ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?          \ true if BLOCK needed
 $0000 equ sizeofheap       \ 0=no heap, nz=size of heap
   0 equ heap-diags?        \   true to include diagnostic code
 0 equ paged?           \ true if ROM or RAM is paged/banked
 0 equ MPE-SET?         \ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?     \ true if ANS ENVIRONMENT system required

 \ Clock, serial and ticker rates
#18240000 equ system-speed      \ System clock rate in HZ.
#9600 equ console0-speed        \ Serial port speed in BPS.
#9600 equ console1-speed        \ Serial port speed in BPS.
\ #9600 equ console2-speed      \ Serial port speed in BPS.
0 equ console-port              \ Designate serial port for terminal.
\ #2 equ tick-ms                \ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel     \ x in Vx.yz
 char 1 equ mpe-ver     \ y in Vx.yz
 char 0 equ usrver      \ z in Vx.yz

\ define stack and user area sizes
$0040 equ UP-SIZE               \ size of each task's user area    ( was 100 )
$0030 equ SP-SIZE               \ size of each task's data stack   ( was 200 )
$0030 equ RP-SIZE               \ size of each task's return stack ( was 200 )
up-size rp-size + sp-size +
  equ task-size         \ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#0 equ sp-guard

$0004 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting level for interrupts and SWIs
1 equ #IRQs         \ number of IRQ stacks, shared by all IRQs (1 min) ( was 4 )
0 equ #SWIs         \ number of SWI nestings permitted (0 is ok)       ( was 1 )

\ ********************
\ Define memory layout
\ ********************

$00000000 equ link-address      \ for a binary image - usually starts at zero on the ARM
                    \ Used by the AIF header
\ 2k of RAM at 0100:0000, 16Mbyte of FLASH at 0000:0000 after remapping
\ Flash is one AMD 29F040B.
$10000000 $100007F7 cdata section lapload \ ~2K for Application
$100007F8 $100007FF idata section PROGd  \ 8 bytes initialised RAM and dictionary
$10000800 $1000B7EF udata section PROGu  \ ~4k uninitialised RAM
$70000000 $70FFFFFF cdata section PROGf  \ 16MByte Flash

ProgF sec-base  equ flashbase       \ -- addr ; Base address of flash memory after mapping
sec-len     equ flashlen        \ -- u ; size of Flash
\ Where the boot program is in flash and where it is run
flashbase   equ BootFlash       \ -- addr ; base address of boot Flash after mapping
lapload sec-len   equ BootLen     \ -- u ; length of boot Flash
lapload sec-base  equ BootRAM     \ -- addr ; base address of boot code after mapping

interpreter
: prog  lapload  ;        \ alias primary code section - PROG is default name
target

PROG PROGd PROGu  CDATA     \ use Code for HERE , and so on

\ **********************************
\ USER area and Multi tasker equates
\ **********************************

\ Main User/Task stack for USR/SYS mode operation.
\ In order to permit fast interrupt nesting the
\ return stack must be the lowest of RSP, PSP and UP
\ In order for the initialisation code in MULTIARM.FTH
\ to work, INIT-U0 must be the highest of INIT-UO,
\ INIT-S0 and INIT-R0.
rp-size sp-size + equ TASK-U0       \ initial offset of user area
rp-size sp-size + equ TASK-S0       \ initial offset of data stack
rp-size           equ TASK-R0       \ initial offset of return stack

task-size reserve equ INIT-T0       \ base of main task area
  init-t0 task-u0 + equ INIT-U0     \ base of main user area
  init-t0 task-s0 + equ INIT-S0     \ top of main data stack
  init-t0 task-r0 + equ INIT-R0     \ top of main return stack

\ SVC stack ; nestable up to #SWIs
0 reserve equ SVC_StackTop      \ top of stack
task-size #SWIs * reserve equ SVC_StackBase \ bottom of stack

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_StackTop      \ top of IRQ stacks
task-size #IRQs * reserve       \ bottom of IRQ stacks
  equ IRQ_StackBase
IRQ_StackTop equ IRQ_Stack_Top
IRQ_StackBase equ IRQ_Stack_Base

\ FIQ stack
0 reserve equ FIQ_StackTop      \ top of FIQ stack
task-size reserve equ FIQ_StackBase \ bottom of FIQ stack

\ UND stack
0 reserve equ UND_StackTop      \ top of stack
task-size reserve equ UND_StackBase \ bottom of stack

\ ABT stack
0 reserve equ ABT_StackTop      \ top of stack
task-size reserve equ ABT_StackBase \ bottom of stack

\ TIB - terminal input buffer
tib-len reserve equ INIT-TIB        \ base of TIB

PROGd  sec-top 1+ equ UNUSED-TOP  PROG  \ top of memory for UNUSED

\ ************
\ Kernel files
\ ************

   include sfr89712             \ CS89712 Special function registers.
   include initARM              \ Generic startup code (*required*).
   include MinArm               \ minimum run-time code

\ *************
\ End of kernel
\ *************


\ ****************************
\ *S Add application code here
\ ****************************

include BootLoad

: ShortDelay
\ *G ~ 10ms delay
   20000 0 do  1 1 + drop  loop
;

: MyTest

   seremitWait0        \ wait for ">" character to be sent by 128 byte loader

   Init-Ser
   ShortDelay          \ delay
   ShortDelay          \ delay
\   s"  115200 baud" sertype0  \ PC must change baud rate to display this correctly

   0x00000522 SDCONF !  	\ initialise the SDRAM controller
   0xC0000000 0x40000 erase     \ clean out 256K bytes of SDRAM

\   serCR0  s" CS89712 boot loader V1.0 20 May 2003 ( ARM end ) "  sertype0

   BootLoader
;

' MyTest CLD1 !
((
BuildFile Build.no
create version$ \ -- addr
\ *G Counted string holding the version number. This version number
\ ** is automatically updated after each successful build.
  Build$,

: .signon   \ --
\ *G System sign on. Displays system, version and author information.
  cr ." Phoenix ARM board"
  cr ." Inventio Software Ltd "  Version
;
\ ' .signon AtCold
))

\ *********************
\ *S End of compilation
\ *********************

\ ****************
\ *S Sanity checks
\ ****************

decimal

((
  cr ." Required USER size is     : " next-user @ .
  cr ." Current USER allocation is: " up-size .
Next-user @ up-size > [if]
\ *G Check that the USER area is large enough.
  cr ." *** Increase USER area size UP-SIZE in control file ***
  abort
[then]
))

\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

\ ********
\ All done
\ ********

decimal

((
update-build

cr
cr ." ==============================="
cr ." Build:" version$ count type
cr ." ==============================="
cr
))

FINIS

\ ******
\ *> ###
\ ******

