\ RAMtests.fth HPO 5 Mar 2003
\ Not absolutely thorough, but tests some common errors.

\ *S Test Axotec SDRAM from 0xC0000000
\ ** Note these are destructive tests.

0xC0000000 constant SDRAMstart
0x1000000 constant SDRAMsize \ 2 x 64 Mbit D4564163 G5 - A10B devices = 16Mbyte

2variable Addr+Len
\ *G address and length of RAM to test

variable FailFlag
\ *G counts the number of failures

variable TotalFail
\ *G counts the total number of failures

#1 cells value RAMstep
\ *G # cells to increment the address by for each test

variable Test#
\ *G incremented for each test

variable CellTest#
\ *G incremented for each cell test

variable Mangle#
\ *G which set of values is being used for the test

: .Mangle#
   Mangle# @ case
   1 of  ." unique to each address"  endof
   2 of  ." 55AA55AA / AA55AA55 "  endof
   3 of  ." FFFFFFFF / 00000000 "  endof
   4 of  ." 00000000 / FFFFFFFF "  endof
         ." ??? "
   endcase
;

defer RAMmangle

: RAMmangle1
\ *G a simple change, to avoid "address = data" pickup
    0x31415926 xor
;

: RAMmangle2
\ *G alternating bit patterns
    drop Celltest# @ 1 and if  0x55AA55AA  else  0XAA55AA55  then
;

: RAMmangle3
\ *G all high or all low
    drop Celltest# @  1 and if  0xFFFFFFFF  else  0x00000000  then
;

: RAMmangle4
\ *G all high or all low
    drop Celltest# @  1 and if  0x0000000  else  0xFFFFFFF  then
;

: InitRAMtests
   assign RAMmangle3 to-do RAMmangle
   3 Mangle# !
   SDRAMstart SDRAMsize Addr+Len 2!
;

' InitRAMtests AtCold


: ?Esc     key? if  key 27 = abort"  Escaped."  then ;


: RAMset
   ?Esc  ." Filling selected cells..."

   0 CellTest# !

   Addr+Len 2@ bounds  do
      i RAMmangle i !
      1 CellTest# +!
   RAMstep +loop
;

: RAMcheck
   0 CellTest# !

   ?Esc  ." Checking...    "

   0 FailFlag !

   Addr+Len 2@ bounds  do
      i @ i RAMmangle = 0= if  cr ." Error at "  i u.  1 FailFlag +!  then
      1 CellTest# +!

      RAMstep 4 = if   \ every cell is filled
         i SDRAMstart SDRAMsize + cell - = 0= if
            i cell + @ i cell + RAMmangle = 0= if  cr ." Error at + "  i cell + u.  1 FailFlag +! then
         then
         i SDRAMstart = 0= if
            -2 CellTest# +!
            i cell - @ i cell - RAMmangle = 0= if  cr ." Error at - "  i cell - u.  1 FailFlag +! then
            2 CellTest# +!
         then
      else              \ intermediate cells should be zero
         i SDRAMstart SDRAMsize + cell - = 0= if
            i cell + @ 0= 0= if  cr ." Zero at + "  i cell + u.  1 FailFlag +!  then
         then
         i SDRAMstart = 0= if
            i cell - @ 0= 0= if  cr ." Zero at - "  i cell - u.  1 FailFlag +!  then
         then
      then

      FailFlag @ 5 > if  leave  then
   RAMstep +loop
;

: .RAMheader
   decimal
   Addr+Len 2@
   cr  ." Test "  1 Test# +!  Test# @ .  hex
   ." : "  u. ." bytes from "  .x  ." , step = " RAMstep decimal .
   .Mangle#
   hex
;

: RAMerase
   RAMstep 1 cells = 0= if
      cr ."    Erasing..."
      Addr+Len 2@ erase
   else
      cr ."              "
   then
;

: RAMtest     \ --
\ *G check that the RAM is there by filling it with a value ( mangled )
\ ** then reading the value back
   base @ >r  

   .RAMheader
   RAMerase
   RAMset
   RAMCheck

   FailFlag @ dup TotalFail +!  if  ." Failed!"  else  ." OK."  then

   r> base !

   ?Esc
;

: RAMthere?
   SDRAMstart SDRAMsize Addr+Len 2!  RAMtest
;

: RAMthere?fail
   SDRAMstart SDRAMsize RAMstep 10 *  + Addr+Len 2!  RAMtest
;

: RAMtestGroup          \ stepsize --
   cells to RAMstep
   SDRAMstart SDRAMsize  Addr+Len 2!

   assign RAMmangle1 to-do RAMmangle  1 Mangle# !
   RAMtest

   assign RAMmangle2 to-do RAMmangle  2 Mangle# !
   RAMtest

   assign RAMmangle3 to-do RAMmangle  3 Mangle# !
   RAMtest

   assign RAMmangle4 to-do RAMmangle  4 Mangle# !
   RAMtest
;

: RAMtests
   0 TotalFail !
   0 Test# !

   0x1000 RAMtestGroup
   255    RAMtestGroup
   2      RAMtestGroup
   1      RAMtestGroup

   TotalFail @ 0= if
      cr ." All tests passed."
   else
      cr  TotalFail @ .  ." failures!!!"  then
;

((
: BRK
   seremitWait0
   0x000E1001 UBRLCR1 !         \ 115200 baud  + break
   0x00 UARTDR1 c!
   seremitWait0
   0x000E0001 UBRLCR1 !         \ 115200 baud
;

: breaks
   single
   begin
      BRK
   0x56 emit
\      1 ms
   KEY? UNTIL
;

: BRKS  BEGIN BRK 20 MS  KEY? UNTIL ;
))
