\ Cirrus CS89712 (ARM7DI-based) special function registers - extracted from
\ CS89712-2.pdf document

((
        Copyright (c) 1988-1998
        MicroProcessor Engineering
        133 Hill Lane
        Southampton SO15 5AF
        England

        tel: +44 1703 631441
        fax: +44 1703 339691
        net: mpe@mpeforth.com
             tech-support@mpeforth.com
        web: www.mpeforth.com
))

\ ***********
\ Bit equates
\ ***********

$00000001 equ BIT0
$00000002 equ BIT1
$00000004 equ BIT2
$00000008 equ BIT3
$00000010 equ BIT4
$00000020 equ BIT5
$00000040 equ BIT6
$00000080 equ BIT7
$00000100 equ BIT8
$00000200 equ BIT9
$00000400 equ BIT10
$00000800 equ BIT11
$00001000 equ BIT12
$00002000 equ BIT13
$00004000 equ BIT14
$00008000 equ BIT15
$00010000 equ BIT16
$00020000 equ BIT17
$00040000 equ BIT18
$00080000 equ BIT19
$00100000 equ BIT20
$00200000 equ BIT21
$00400000 equ BIT22
$00800000 equ BIT23
$01000000 equ BIT24
$02000000 equ BIT25
$04000000 equ BIT26
$08000000 equ BIT27
$10000000 equ BIT28
$20000000 equ BIT29
$40000000 equ BIT30
$80000000 equ BIT31

\ *******
\ ARM CPU
\ *******

$80 equ I_BIT
$40 equ F_BIT
$20 equ T_BIT

$00000080 equ IRQDisable
$00000040 equ FIQDisable
$000000C0 equ InterruptMask
$0000001F equ ModeMask

$00000000 equ usr_26
$00000001 equ fiq_26
$00000002 equ irq_26
$00000003 equ svc_26
$00000010 equ usr_32
$00000011 equ fiq_32
$00000012 equ irq_32
$00000013 equ svc_32
$00000017 equ abt_32
$0000001B equ und_32
$0000001F equ sys_32

$10 equ ARM_MODE_USER
$11 equ ARM_MODE_FIQ
$12 equ ARM_MODE_IRQ
$13 equ ARM_MODE_SVC
$14 equ ARM_MODE_ABORT
$15 equ ARM_MODE_UNDEF
$1F equ ARM_MODE_SYS

\ --------------------------------------------------------------------------
\ CS89712 specific registers

\ Address      Name    Default RD/WR Size Comments
0x80000000 equ PADR        \ 0 RW 8 Port A data register.
0x80000001 equ PBDR        \ 0 RW 8 Port B data register.
\ 0x80000002               \  --- 8 Reserved.
0x80000003 equ PDDR        \ 0 RW 8 Port D data register.
0x80000040 equ PADDR       \ 0 RW 8 Port A data direction register.
0x80000041 equ PBDDR       \ 0 RW 8 Port B data direction register.
\ 0x80000042               \  --- 8 Reserved.
0x80000043 equ PDDDR       \ 0 RW 8 Port D data direction register.
0x80000080 equ PEDR        \ 0 RW 3 Port E data register.
0x800000C0 equ PEDDR       \ 0 RW 3 Port E data direction register.
0x80000100 equ SYSCON1     \ 0 RW 32 System control register 1.
0x80000140 equ SYSFLG1     \ 0 RD 32 System status flags register 1.
0x80000180 equ MEMCFG1     \ 0 RW 32 Expansion memory configuration register 1.
0x800001C0 equ MEMCFG2     \ 0 RW 32 Expansion memory configuration register 2.
\ 0x80000200               \ 0 RW 32 Reserved.
0x80000240 equ INTSR1      \ 0 RD 32 Interrupt status register 1.
0x80000280 equ INTMR1      \ 0 RW 32 Interrupt mask register 1.
0x800002C0 equ LCDCON      \ 0 RW 32 LCD control register.
0x80000300 equ TC1D        \ 0 RW 16 Read / Write register sets and reads data to TC1.
0x80000340 equ TC2D        \ 0 RW 16 Read / Write register sets and reads data to TC2.
0x80000380 equ RTCDR       \ - RW 32 Real Time Clock data register.
0x800003C0 equ RTCMR       \ - RW 32 Real Time Clock match register.
0x80000400 equ PMPCON      \ 0 RW 12 PWM pump control register.
0x80000440 equ CODR        \ 0 RW 8 CODEC data I/O register.
0x80000480 equ UARTDR1     \ 0 RW 16 UART1 FIFO data register.
0x800004C0 equ UBRLCR1     \ 0 RW 32 UART1 bit rate and line control register.
0x80000500 equ SYNCIO      \ 0 RW 32 Synchronous serial I/O data reg for master only SSI.
0x80000540 equ PALLSW      \ 0 RW 32 Least significant 32bit word of LCD palette register.
0x80000580 equ PALMSW      \ 0 RW 32 Most significant 32bit word of LCD palette register.
0x800005C0 equ STFCLR      \ - WR -- Write to clear all start up reason flags.
0x80000600 equ BLEOI       \ - WR -- Write to clear battery low interrupt.
0x80000640 equ MCEOI       \ - WR -- Write to clear media changed interrupt.
0x80000680 equ TEOI        \ - WR -- Write to clear tick and watchdog interrupt.
0x800006C0 equ TC1EOI      \ - WR -- Write to clear TC1 interrupt.
0x80000700 equ TC2EOI      \ - WR -- Write to clear TC2 interrupt.
0x80000740 equ RTCEOI      \ - WR -- Write to clear RTC match interrupt.
0x80000780 equ UMSEOI      \ - WR -- Write to clear UART modem status changed interrupt.
0x800007C0 equ COEOI       \ - WR -- Write to clear CODEC sound interrupt.
0x80000800 equ HALT        \ - WR -- Write to enter the Idle State.
0x80000840 equ STDBY       \ - WR -- Write to enter the Standby State.
\ 0x80000880  -- 0x80000FFF \ Reserved Write will have no effect, read is undefined.
0x80001000 equ FBADDR      \ 0xC RW 4 LCD frame buffer start address.
0x80001100 equ SYSCON2     \ 0 RW 16 System control register 2.
0x80001140 equ SYSFLG2     \ 0 RD 24 System status register 2.
0x80001240 equ INTSR2      \ 0 RD 16 Interrupt status register 2.
0x80001280 equ INTMR2      \ 0 RW 16 Interrupt mask register 2.
\ 0x800012C0 -- 0x8000147F \ Reserved Write will have no effect, read is undefined.
0x80001480 equ UARTDR2     \ 0 RW 16 UART2 Data register.
0x800014C0 equ UBRLCR2     \ 0 RW 32 UART2 bit rate and line control register.
0x80001500 equ SS2DR       \ 0 RW 16 Master / slave SSI2 data register.
0x80001600 equ SRXEOF      \ - WR -- Write to clear RX FIFO overflow flag.
0x800016C0 equ SS2POP      \ - WR -- Write to pop SSI2 residual byte into RX FIFO.
0x80001700 equ KBDEOI      \ - WR -- Write to clear keyboard interrupt.
\ 0x80001800 Reserved  \ - WR -- Do not write to this location. A write will cause the
\ processor to go into an unsupported power state.
\ 0x80001840-- 0x80001FFF \ Reserved --- Write will have no effect, read is undefined.
0x80002000 equ DAIR        \ 0 RW 32 DAI control register.
0x80002040 equ DAIR0       \ 0 RW 32 DAI data register 0.
0x80002080 equ DAIDR1      \ 0 RW 32 DAI data register 1.
0x800020C0 equ DAIDR2      \ 0 WR 21 DAI data register 2.
0x80002100 equ DAISR       \ 0 RW 32 DAI status register.
0x80002200 equ SYSCON3     \ 0 RW 16 System control register 3.
0x80002240 equ INTSR3      \ 0 RD 32 Interrupt status register 3.
0x80002280 equ INTMR3      \ 0 RW 8 Interrupt mask register 3.
0x800022C0 equ LEDFLSH     \ 0 RW 7 LED Flash register.
0x80002300 equ SDCONF      \ 0 RW 11 SDRAM configuration register.
0x80002340 equ SDRFPR      \ 0 RW 11 SDRAM refresh period register.

\ Note that the Ports A, B, C and D have their addresses reversed in Big Endian mode
\ for both the data and direction registers.

\ --------------------------------------------------------------------------

