\ CS89712 ARM Ticker interrupt handlers

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

This ticker is based on using Timer 1, generating
an interrupt every TICK-MS milliseconds. It can be adapted for the
TIMEBASE system by adding DO-TIMERS to the TICKER-ISR.

period=divisor/clock

divisor = period * clock
))

interpreter
: gen-CTC   \ us clock -- ctc#
  [ #256 #36 * 1000 * ] literal */      \ divisor = period * clock / 9216000
  1000 /                                \ convert us to ms
  1-                                    \ actual count is divisor less 1
;
target

tick-ms #1000 *  system-speed gen-CTC
  equ #Timer1div

[undefined] ticks [if]  \ TICKS is also defined in TIMEBASE.FTH
0 value ticks   \ -- ; returns the ticker value
( *G Get current clock value in milliseconds. )
[then]

: ticker-isr    \ --
  INTSR1 @ bit8 and if          \ test for this interrupt
    0 TC1EOI w!                 \ reset Timer1 interrupt ( write anything )
    tick-ms addr ticks +!       \ update tick count
[ timebase? ] [if]
    do-timers
[then]
  endif
;
' ticker-isr add-irq            \ add it to the IRQ chain

0 value ticking?    \ -- n ; nonzero when ticker running

: start-clock   \ -- ; start clock interrupt
  di
  bit4 SYSCON1 orr!         \ set bit 4 for Timer1  1 = prescale
  bit5 SYSCON1 bic!         \ clear bit 5 for Timer1  0 = 2kHz
  #Timer1div TC1D w!          \ set clock divider, f = ( 2kHz ) / ( n+1 ) = 100Hz
  bit11 INTMR1 orr!         \ enable interrupt
  ei                    \ enable interrupts
  -1 to ticking?
;

: stop-clock    \ -- ; stop clock interrupt
  [i
  bit11 INTMR1 bic!         \ enable interrupt
  0 to ticking?
  i]
;

timebase? [if]
: start-timers  \ -- ; Start internal time clock
  stop-clock init-timers start-clock
;

: stop-Timers   \ -- ; disable timer system
  stop-clock
;
[else]
((
: start-timers  \ -- ; Start internal time clock
  start-clock
;

: stop-Timers   \ -- ; disable timer system
  stop-clock
;

: later         \ n -- n'
( *G Generates the timebase value for termination in n millseconds time. )
  ticks +
;

: expired       \ n -- flag ; true if timed out
( *G Flag is returned true if the timebase value n has timed out. Calls PAUSE. )
  pause  ticks - 0<         \ must use circular arithmetic
;

: timedout?     \ n -- flag ; true if timed out
( *G Flag is returned true if the timebase value n has timed out. Does not )
( ** call PAUSE, so can be used in interrupts, winprocs and callbacks. )
( In particular, )
( ** TIMEDOUT? should be used rather than EXPIRED inside timer action words )
( ** to reduce timer jitter. )
  ticks - 0<                            \ must use circular arithmetic
;
optimised? [if]
compiler
: later         \ n -- n'
  ticks +
;

: timedout?     \ n -- flag ; true if timed out
  ticks - 0<                            \ must use circular arithmetic
;
target
[then]

[undefined] ms [if]
: ms            \ n --
( *G Waits for n milliseconds. Uses PAUSE. )
  later
  begin
    dup expired
  until
  drop
;
[then]
))
[then]
