\ AT91M40400 generic startup code for Atmel EB01

(( 
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20030508 SFP001 Removed AIF header code - now in AIFHEAD.FTH.
))


\ -----------------------------------------------------------------------------
\     Generic start up table and entry points
\ -----------------------------------------------------------------------------

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr 	pc, ResetV              \ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  ldr 	pc, UnusedV             \ 14 - Reserved (address exception if 26bit)
  ldr 	pc, [ pc, # $-F20 ]	\ 18 - IRQ interrupts
  ldr 	pc, [ pc, # $-F20 ]	\ 1C - FIQ interrupts

  L: Soft_Vectors               \ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code

L: InitTableEBI
  $01002531 ,			\ 0x0100:0000, 16MB, 2 tdf, 16 bits, 5 WS
  $02002121 ,			\ 0x0200:0000, 16MB, 0 hold, 16 bits, 1 WS
  $20000000 ,			\ Unused
  $30000000 ,			\ Unused
  $40000000 ,			\ Unused
  $50000000 ,			\ Unused
  $60000000 ,			\ Unused
  $70000000 ,			\ Unused
  $00000001 ,			\ Remap command
  $00000006 ,			\ 6 memory regions, standard read
  $FFE00000 ,			\ EBI base address

asmcode
L: InitReset
\  bl		InitLeds
\  bl		FlashLeds

  ldr 		r10, PtInitTableEBI
  mov .s 	r0, pc, LSR # 20
  mov .eq 	r10, r10, LSL # 12
  mov .eq 	r10, r10, LSR # 12

\ these four instructions MUST be together
  ldr 		r12, PtInitRemap
  ldmia 	r10, ! { r0-r9, r11 }
  stmia 	r11, ! { r0-r9 }
  mov 		pc, r12
end-code

InitReset prog sec-base - ResetV !	\ INITRESET is in LOW memory starting at 0.

L: PtInitTableEBI
  InitTableEBI ,

L: PtInitRemap
  0 ,

L: PtInitVector
  Vector_Start ,
L: CLD1		                        \ cold start word
  0 ,				        \ compiler will fill up with address of word to execute at cold start
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,
l: ^int_stack_top			\ interrupt stacks top
  int_stack_top ,
l: ^sys/user_stack_top			\ system\user mode stacks
  sys/user_stack_top ,

l: ^aic_base				\ base address of AIC
  _AIC ,
l: ^aic_all				\ mask to select all interrupts
  $701FF ,
l: ^pio_base
  _PIO ,

asmcode
L: InitRemap
  mvn		r0, # 0					\ -1, prevent pio interrupts
  ldr		r1, ^pio_base
  str		r0, [ r1, # PIO_PER ]			\ all pins in PIO mode
  str		r0, [ r1, # PIO_IDR ]			\ all interrupts disabled

  ldr		r0, ^aic_all				\ disable all interrupts
  ldr		r1, ^aic_base
  str		r0, [ r1, # AIC_IDCR ]
  str		r0, [ r1, # AIC_ICCR ]

  mov		r0, # #8				\ will write 8 times to AIC_EOICR
L: LoopAIC0	
  str		r1, [ r1, # AIC_EOICR ]			\ write any value to the end of interrupt
  sub .s	r0, r0, # #1
  b .hi		LoopAIC0

\ Set up IRQ, FIQ, ABORT and UNDef stacks
  mov		r1, # ARM_MODE_IRQ I_BIT or F_BIT or	\ IRQ Interrupt Mode + Stack Setup
  msr		CPSR, _c r1
  ldr		r13, ^INT_STACK_TOP
  mov		r0, r13
  sub		r0, r0, # IRQ_STACK_SIZE
  
  mov		r1, # ARM_MODE_FIQ I_BIT or F_BIT or 	\ FIQ Interrupt Mode + Stack Setup
  msr		CPSR, _c r1
  mov		r13, r0
  sub		r0, r0, # FIQ_STACK_SIZE

  mov		r1, # ARM_MODE_ABORT I_BIT or F_BIT or	\ Abort Mode + Stack Setup
  msr		CPSR, _c r1
  mov		r13, r0
  sub		r0, r0, # ABT_STACK_SIZE

  mov		r1, # ARM_MODE_UNDEF I_BIT or F_BIT or	\ Undefined Mode + Stack
  msr		CPSR, _c r1
  mov		r13, r0
  sub		r0, r0, # UND_STACK_SIZE

\ Set up SYStem and USER mode stacks
  mov		r0, # ARM_MODE_SYS I_BIT or F_BIT or	\ System Mode
  msr		CPSR, _c r0
  ldr		r13, ^SYS/USER_STACK_TOP
  sub		r4, r13, # SYS_STACK_SIZE

\ Include these lines if you want to run in USER mode,
\ otherwise comment them out.
\  mov		r0, # ARM_MODE_USER			\ User Mode - Set application mode and enable interrupts
\  msr		CPSR, _c r0

 
\ -----------------------------------------------------------------------------
\       start of Forth system
\ -----------------------------------------------------------------------------

L: ECLD
\  bl		FlashLeds

\ Copy vector area into low RAM
aif? [if]
  adr		r0, # vector_start	\ source
[else]
  mov		r0, # vector_start	\ source
[then]
  mov		r1, # 0			\ dest
  mov		r2, # #18		\ eight vectors + spurious, instruction and data cells
l: vector_loop
  ldr		r3, [ r0 ], # 4
  str		r3, [ r1 ], # 4
  sub .s	r2, r2, # #1
  b .ne		vector_loop

\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

InitRemap PtInitRemap !


\ -----------------------------------------------------------------------------
\       reset values for user and system variables
\ -----------------------------------------------------------------------------

\ initial values of user variables
L: USER-RESET
  init-s0 ,   init-r0 ,                 \ s0, r0
  0 ,  0 ,                              \ #tib, span
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link


proc InitLeds
  mov		r0, # $16			\ LED mask for P4,2,1
  mvl		r1, # $FFFF0000			\ point r1 at PIO
  str		r0, [ r1, # $00 ]		\ enable PIO of pins
  str		r0, [ r1, # $10 ]		\ set pins as outputs
  mov		pc, link
end-code

$80000 equ FlashDelay
proc FlashLeds
  mov		r0, # $16			\ LED mask for P4,2,1
  mvl		r1, # $FFFF0000			\ point r1 at PIO
  str		r0, [ r1, # $30 ]		\ set those pins

  mov		r2, # FlashDelay
l: onloop
  sub .s	r2, r2, # 1
  b .ne		onloop

  str		r0, [ r1, # $34 ]		\ reset those pins

  mov		r2, # FlashDelay
l: offloop
  sub .s	r2, r2, # 1
  b .ne		offloop

  mov		pc, link
end-code

: FlashIt	\ --
  [asm
    bl FlashLeds
  asm]
;

