\ EDSL Boot loader code

((
Copyright (c) 2000, 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ **********
\ *! EDSLboot
\ *T Application loader and program maintenance
\ **********
\ *P The file EDSLBOOT.FTH provides facilities for file transfer into Flash
\ ** using XMODEM with HyperTerm or similar terminal emulators.
\ ** Menu options allow various facilities for 
\ ** boot management, replacement of user applications, and replacement of
\ ** boot code.
\ ** The timebase system and a Flash driver are required. The multitasker
\ ** is disabled by the maintenance menu.

decimal


\ ****************
\ *S Configuration
\ ****************

True equ BootMenu?	\ -- flag
\ *G Set this equate non-zero to include the power up menu code.
\ ** The word BOOTSTRAP should then be set as the application
\ ** start up word using the phrase "MAKE-TURNKEY BOOTSTRAP" 
\ ** in your control file.


\ ****************************
\ *S Handling the Flash memory
\ ****************************

: n>app		\ n -- addr
\ *G Given an application number, convert it to the required
\ ** address in the Flash.
  ext_prog_size * ext_base +
;

: RunApp	\ n --
\ *G Copy flash for app n into RAM at address APPRAM, and execute at
\ ** address APPRAM. If the data at the start of the application
\ ** Flash area is $FFFFFFFF, which is the signature of unprogrammed
\ ** flash, RUN-APP issues an error message and returns.
  n>app dup @ $FFFFFFFF = if		\ Check for application
    cr ." Application at " .dword ."  not present"
    exit
  then
cr ." Running application at " dup .dword
  stop-timers  di			\ disable all interrupts
  AppRam ext_prog_size cmove		\ copy Flash to RAM
  AppRam execute			\ go do it
;

: RemoveApp	\ n --
\ *G Erase the flash for application n.
  n>app ext_prog_size (EraseFlash)
;

: GetApp	\ n --
\ *G Download application n by Xmodem.
  AppRAM ext_prog_size RecvXmodem if	\ -- n length
    2drop  cr ." Bad transfer "  exit
  endif
  cr dup . ." Bytes received"
  AppRAM rot n>app rot ProgramFlash
;

: ValidApp#?	\ n -- flag
  0 #ext_progs 1- within?
;

: EnterApp#	\ -- n true | 0
\ *G Enter an application number. If the number is in range,
\ ** return the application number and true, otherwise just
\ ** return zero.
  cr ." Enter Application number 0.." #ext_progs 1- . ." : "
  key dup emit  [char] 0 -
  dup ValidApp#? dup 0= if
    nip  cr ." Bad application number"
  endif
;
    
: NewApp	\ --
\ *G Get a new application by Xmodem.
  EnterApp# if  GetApp  endif
;

: EraseApp	\ --
\ *G Wipe an application.
  EnterApp# if  RemoveApp  endif
;

: ExecApp	\ --
  EnterApp# if  RunApp  endif
;

variable DefaultApp#

: DefaultApp	\ --
\ *G Set the default application number in the first
\ ** four bytes of the serial EEPROM.
  EnterApp# if
    DefaultApp# !
    DefaultApp# cell 0 SEPROM!
  endif
;

: RunDefault	\ --
\ *G Run the default application
  DefaultApp# cell 0 SEPROM@
  DefaultApp# @ dup ValidApp#? if
    RunApp
  else
    drop
    cr ." No valid application number in EEPROM"
  endif
;
  
: NewBootFlash	\ --
\ *G Update the boot Flash area by Xmodem.
  AppRAM Boot_Flash_Len Bin-UP
  if  cr ." Bad transfer "  exit  endif
  AppRAM Flash_start Boot_Flash_Len ProgramFlash
;


\ ************************
\ *S User maintenance menu
\ ************************
\ *P The maintenance system provides the following features,
\ ** accessed from the menu.
\ *B F - run the Forth monitor
\ *B R - run an application
\ *B E - erase an application
\ *B D - download and Flash a new user program
\ *B B - download and Flash a new boot program
\ *B S - Set the default application to run
\ *B Q - quit maintenance system

decimal

: .maintenance-menu	\ --
\ *G Display the maintenance menu.
  cr ." Command keys:"
  cr ." F - run the Forth monitor"
  cr ." R - run an application"
  cr ." E - erase an application"
  cr ." D - download and Flash a new user program"
  cr ." B - download and Flash a new boot program"
  cr ." S - Set the defult application"
  cr ." Q - quit maintenance system"
;

: (maintenance)	\ mode -- flag ; flag true if Q pressed
\ *G Enter a single command and process it. If mode is non-zero,
\ ** an invalid key runs an application program if it exists.
\ ** Flag is returned non-zero if Q was pressed.
  >r
  0					\ Q not pressed yet
  case  cr ." Enter command: " key upc dup emit
    [char] F of
      cr ." Forth Monitor"  
      r> drop  s0 @ sp!  cr .CPU quit
    endof
    [char] R of
      cr ." Run application" ExecApp
    endof
    [char] E of
      cr ." Erase application" EraseApp
    endof
    [char] D of
      cr ." Download new application"  NewApp
    endof
    [char] B of
      cr ." Download new boot program:"
      cr ."   Are you sure? Y/N: " key upc  dup emit  [char] Y =
      if  NewBootFlash  endif
    endof
    [char] S of
      cr ." Set default Application number" DefaultApp
    endof
    [char] Q of  cr  drop true  endof
      cr ." Invalid command"
      r@ if
        ."  - Running User program"
        drop RunDefault
      endif
  endcase
  r> drop
;

: Maintenance 	\ mode -- flag ; true if Q pressed
\ *G Enter the maintenance system. If mode is non-zero,
\ ** the system will time out after 3 seconds and run
\ ** an application program if it exists. Flag is
\ ** returned non-zero if Q was pressed.
\ ** N.B. the multitasker is turned off, but is restored
\ ** on exit, and the timebase system is reinitialised.
  .maintenance-menu
  Start-Timers
  if					\ mode is non-zero
    cr ." You have 3 seconds ..."
    0
    begin
      #10 ms
      1+ dup #300 =
      key? or
    until
    #300 < if
      1 (maintenance)
    else
      cr ." no key pressed - Running User program"
      RunDefault
    then
  else
    0 (maintenance)
  endif
  stop-timers
;

: maintain	\ --
\ *G Run the maintenance system until Q is pressed.
  begin  0 maintenance  until
;

BootMenu? [if]
: bootstrap	\ --
  (init) init-ser			\ initialise Forth and serial line
  console ipvec !  console opvec !	\ select default I/O device
[ tasking? ] [if]
  init-multi				\ initialise multi-tasker
[then]
[defined] WalkColdChain [if]
  WalkColdChain				\ execute user specified initialisation
[then]
  s0 @ sp!                              \ reset data stack
  cr
  cr .cpu				\ system sign on message
  cr ." MPE ARM Serial BootStrap v1.000, 15 October 2002"
  cr ." Copyright (c) 2002  EDSL & MicroProcessor Engineering Ltd."
  begin  1 maintenance drop  again
;
[then]


\ ******
\ *> ###
\ ******

