\ INIT55800EDSL.FTH - AT91M55800A generic startup code, copy ROM to RAM

(( 
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE


Much of this code is derived from the CSTARTUP.S files provided
on the Atmel EB55 CD in the SOFTWARE folder.
))

decimal

\ ---------------------------------------
\ Generic start up table and entry points
\ ---------------------------------------

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr 	pc, ResetV              \ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  ldr 	pc, UnusedV             \ 14 - Reserved (address exception if 26bit)
  ldr 	pc, [ pc, # $-F20 ]	\ 18 - IRQ interrupts
  ldr 	pc, [ pc, # $-F20 ]	\ 1C - FIQ interrupts

  L: Soft_Vectors               \ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code

Vector_End Vector_Start - equ Vector_Len

$01000000 equ Flash_start	\ address of flash after REMAP

L: InitTableEBI
  $01002529 ,			\ 0x0100:0000, 16MB, 2 tdf, 16 bits, 2 WS
  $02002121 ,			\ 0x0200:0000, 16MB, 0 hold, 16 bits, 1 WS
  $20000000 ,			\ Unused
  $30000000 ,			\ Unused
  $40000000 ,			\ Unused
  $50000000 ,			\ Unused
  $60000000 ,			\ Unused
  $70000000 ,			\ Unused
  $00000001 ,			\ Remap command
  $00000006 ,			\ standard read
  _EBI_BASE ,			\ EBI base address

asmcode
L: InitReset
(( \ See EB55_LED.FTH
  bl	  InitLeds
l: x1
  mov     r0, # $200
  bl	  FlashLeds
  mov     r0, # $200
  bl	  FlashLeds
  b       x1
))

\ massage address in case ...
  ldr 		r10, PtInitTableEBI
  mov .s 	r0, pc, LSR # 20
  mov .eq 	r10, r10, LSL # 12
  mov .eq 	r10, r10, LSR # 12

\ these four instructions MUST be together
  ldr 		r12, PtInitRemap
  ldmia 	r10, ! { r0-r9, r11 }
  stmia 	r11, ! { r0-r9 }	\ REMAP occurs here, and relies on
  mov 		pc, r12			\ this instruction being in the pipeline
end-code

InitReset prog sec-base -  ResetV !	\ INITRESET is in LOW memory starting at 0.


\ Rather than use long literal macro instructions to access data,
\ it is shorter to use PC relative accesses to the data.
L: PtInitTableEBI
  InitTableEBI ,

L: PtInitRemap
  0 ,		\ filled in later with address of code run after REMAP command

\ L: PtInitVector
\   Vector_Start ,
L: CLD1		                        \ cold start word
  0 ,				        \ compiler will fill up with
  					\ address of word to execute at cold start
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,

l: ^int_stack_top			\ interrupt stacks top
  int_stack_top ,
l: ^sys/user_stack_top			\ system\user mode stacks top
  sys/user_stack_top ,

l: ^aic_base				\ base address of AIC
  _AIC_BASE ,
l: ^aic_all				\ mask to select all interrupts
  AIC_INTALL ,
l: ^pioa_base
  _PIOA_BASE ,
l: ^piob_base
  _PIOB_BASE ,

l: ^apmc	\ holds APMC base address
  _apmc_base ,
l: ^apmc_init	\ holds APMC initial value
  $002F0002 ,				\ MOSCEN=1, OSCOUNT=47 (1.4ms/30us)
l: ^apmc_main	\ holds APMC main oscillator value
  $002F4002 ,				\ MOSCEN=1, OSCOUNT=47 (1.4ms/30us),
  					\ CSS=01
l: ^apmc_pll	\ holds APMC PLL oscillator value
  $032F4102 ,				\ MOSCEN=1, OSCOUNT=47 (1.4ms/30us),
  					\ CSS=01, MUL=1, PLLCOUNT=3
l: ^apmc_pll32	\ holds APMC PLL oscillator value
  $032F8102 ,				\ MOSCEN=1, OSCOUNT=47 (1.4ms/30us),
  					\ CSS=10, MUL=1, PLLCOUNT=3

asmcode
L: AfterRemap
(( \ See EB55_LED.FTH
  mov     r0, # $200
  bl	  FlashLeds
  mov     r0, # $200
  bl	  FlashLeds
))

\ Speed up the clock
  ldr		r1, ^apmc		\ base address of APMC
  mvn		r0, # 0
  str		r0, [ r1, # APMC_PCDR ]	\ disable all peripheral clocks
  mov		r0, # BIT13		\ mask for PIOA in APMC
  str		r0, [ r1, # apmc_pcer ]	\ enable PIOA clocks

  ldr		r0, ^apmc_init		\ set clock configuration
  str		r0, [ r1, # APMC_CGMR ]
  begin,
    ldr		r0, [ r1, # APMC_SR ]	\ read APMC status
    and .s	r0, r0, # 1		\ extract MOSCS bit
  ne, until,				\ wait until set
\ select main clock
  ldr		r0, ^apmc_main		\ set clock configuration
  str		r0, [ r1, # APMC_CGMR ]
\ select PLL clock
  ldr		r0, ^apmc_pll		\ set clock configuration
  str		r0, [ r1, # APMC_CGMR ]
  begin,
    ldr		r0, [ r1, # APMC_SR ]	\ read APMC status
    and .s	r0, r0, # 2		\ extract LOCK bit
  ne, until,				\ wait until set
  ldr		r0, ^apmc_pll32		\ set clock configuration to 32 MHz
  str		r0, [ r1, # APMC_CGMR ]

\ prevent PIO interrupts
  mvn		r0, # 0					\ -1, prevent pio interrupts
  ldr		r1, ^pioa_base
  str		r0, [ r1, # PIO_PER ]			\ all pins in PIO mode
  str		r0, [ r1, # PIO_IDR ]			\ all interrupts disabled
  ldr		r1, ^piob_base
  str		r0, [ r1, # PIO_PER ]			\ all pins in PIO mode
  str		r0, [ r1, # PIO_IDR ]			\ all interrupts disabled

\ Set up the AIC
  ldr		r0, ^aic_all				\ disable all interrupts
  ldr		r1, ^aic_base
  str		r0, [ r1, # AIC_IDCR ]
  str		r0, [ r1, # AIC_ICCR ]

  mov		r0, # #8				\ will write 8 times to AIC_EOICR
L: LoopAIC0	
  str		r1, [ r1, # AIC_EOICR ]			\ write any value to the end of interrupt
  sub .s	r0, r0, # #1
  b .ne		LoopAIC0

\ Set up IRQ, FIQ, ABORT and UNDef stacks
  mov		r1, # ARM_MODE_IRQ I_BIT or F_BIT or	\ IRQ Interrupt Mode + Stack Setup
  msr		CPSR, _c r1
  ldr		r13, ^INT_STACK_TOP
  mov		r0, r13
  sub		r0, r0, # IRQ_STACK_SIZE
  
  mov		r1, # ARM_MODE_FIQ I_BIT or F_BIT or 	\ FIQ Interrupt Mode + Stack Setup
  msr		CPSR, _c r1
  mov		r13, r0
  sub		r0, r0, # FIQ_STACK_SIZE

  mov		r1, # ARM_MODE_ABORT I_BIT or F_BIT or	\ Abort Mode + Stack Setup
  msr		CPSR, _c r1
  mov		r13, r0
  sub		r0, r0, # ABT_STACK_SIZE

  mov		r1, # ARM_MODE_UNDEF I_BIT or F_BIT or	\ Undefined Mode + Stack
  msr		CPSR, _c r1
  mov		r13, r0
  sub		r0, r0, # UND_STACK_SIZE

\ Set up SYStem and USER mode stacks
  mov		r0, # ARM_MODE_SYS I_BIT or F_BIT or	\ System Mode
  msr		CPSR, _c r0
  ldr		r13, ^SYS/USER_STACK_TOP
  sub		r4, r13, # SYS_STACK_SIZE

\ Include these lines if you want to run in USER mode,
\ otherwise comment them out.
\  mov		r0, # ARM_MODE_USER			\ User Mode - Set application mode and enable interrupts
\  msr		CPSR, _c r0

 
\ ---------------------
\ start of Forth system
\ ---------------------

L: ECLD
(( \ See EB55_LED.FTH
  bl            InitLeds		\ (re)initialise APMC for PIOB
l: x2
  mov		r0, # $80000
  bl		FlashLeds
  mov		r0, # $80000
  bl		FlashLeds
\  b             x2
))

\ Copy vector area into low RAM
  mov		r0, # Flash_start	\ source
  mov		r1, # 0			\ dest
  mov		r2, # Vector_len	\ eight vectors + soft vectors
l: vector_loop
  ldr		r3, [ r0 ], # 4
  str		r3, [ r1 ], # 4
  sub .s	r2, r2, # 4
  b .ne		vector_loop

\ If required, copy the code into RAM for execution
RunFromRAM? [if]
  mov		r0, # Flash_start	\ source
  mov		r1, # sec-base		\ dest
  mov		r2, # sec-len		\ len
l: Flash_loop
  ldr		r3, [ r0 ], # 4
  str		r3, [ r1 ], # 4
  sub .s	r2, r2, # 4
  b .ne		flash_loop
[then]

\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

AfterRemap sec-base -  Flash_Start + PtInitRemap !


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] sp-guard [if]
0 equ sp-guard
[then]

\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, span/tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link

