\ InitSAM7x.fth - generic AT91SAM7x startup code

(( 
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Much of this code is derived from sample files provided
on the Atmel AT91 CD.
))


decimal

\          MOSC * ( mul+1)
\ PLLout = ---------------
\               div
\ 
\          PLLout * div
\ mul+1  = ------------
\              MOSC
\ 
\          PLLout * div
\ mul    = ------------ - 1
\              MOSC

interpreter
: genMUL	\ freq div -- mul
\ Given a required frequency and a division ratio, calculate
\ the PLL multiplier.
  /MOSC */mod  swap /MOSC 2/ >=
  if  1+  endif
  1-
;

: genFREQ	\ mul div -- freq
\ Given a multiplier and divider, calculate the PLL output frequency.
  /MOSC -rot
  swap 1+ swap */
;

: genPLL	\ mul div cnt base -- regval
\ Given a multiplier, divider, slow clock count, and a base value,
\ form the PLLR value.
  swap $3F and 8 lshift or		\ cnt
  swap $FF and or			\ div
  swap $7FF and #16 lshift or		\ mul
;
target

\ Calculate the PLL settings
pll-speed /PLLdiv genMUL equ /PLLmul
/PLLmul /PLLdiv genFREQ equ /PLLCK
/PLLmul /PLLdiv $3E USBdiv #28 lshift genPLL
  equ initPLLcalc

\ Calculate the MCKR settings
PCKpres 2 lshift  MCKsel or
  equ initMCKRcalc

/PLLCK  PCKpres rshift equ /MCK
/PLLCK  PCKpres rshift equ /PCK
/PLLCK  USBdiv rshift equ /UCK

showPLLinfo? [if]
cr
cr ." Requested PLLCK = " pll-speed . ." Hz"
cr ." Obtained PCK  = " /PLLCK . ." Hz"
cr ." mul = " /PLLmul .
cr ." div = " /PLLdiv .
cr ." PLLR = " initPLLcalc .dword
cr
cr ." Obtained MCK = " /MCK . ." Hz"
cr ." Obtained UCK = " /UCK . ." Hz"
cr ." Obtained PCK = " /PCK . ." Hz"
cr ." MCKR = " initMCKRcalc .dword
cr
[then]


\ ---------------------------------------
\ Generic start up table and entry points
\ ---------------------------------------

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr   pc, ResetV		\ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  ldr 	pc, UnusedV             \ 14 - Reserved (address exception if 26bit)
  ldr 	pc, [ pc, # $-F20 ]	\ 18 - IRQ interrupts
  ldr 	pc, [ pc, # $-F20 ]	\ 1C - FIQ interrupts

  L: Soft_Vectors               \ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset ; must be in ProgFlash
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code
Vector_End Vector_Start - equ Vector_Len

proc init-io-ll	\ R0 contains address of initialisation table
\ *G Low-level initialisation routine. On entry, R0 points to a
\ ** table which contains address/value pairs. The tables finish
\ ** with a zero address. Uses R1 and R2.
  bic     r0, r0, # $FF000000	\ force address to Flash
  and     r1, pc, # $FF000000
  orr     r0, r0, r1
l: init-io-ll-1
  ldr	  r1, [ r0 ], # 4	\ get address
  cmp	  r1, # 0		\ get out if 0
  mov .eq pc, link
  ldr     r2, [ r0 ], # 4	\ get data
  str	  r2, [ r1, # 0 ]	\ store data
  b       init-io-ll-1
l: init-io-ll-x
  next,
end-code

\ Rather than use long literal macro instructions to access data,
\ it is shorter to use PC relative accesses to the data.
l: ^MC		\ Pointer to Memory controller
  _MC ,
l: ^PMC		\ pointer to PMC
  _PMC ,
l: ^AIC		\ pointer to AIC
  _AIC ,
l: ^SP		\ pointer to System Peripherals
  _SP ,
l: ^initMOR	\ initial PMC
  $4001 ,	\ OSCOUNT=$40, MOSCEN=1
l: ^AIC_all	\ mask for all interrupts
  $FFFFFFFF ,

\ Forth start up data
L: CLD1		\ cold start word
  0 ,		\ compiler will fill up with address cold start word
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,

Proc InitReset
\ *G Initialisation code. For the SAM7X series, Flash appears at
\ ** at 0x0 and at 0x0010:0000 at boot time. The instruction at 0x0
\ ** forces a branch to the Flash at at *\fo{InitReset} in the
\ ** 0x0010:0000 region.
  mov   r0, # $D3			\ I/F set, mode=SVC
  msr   cpsr _all r0			\ force CPSR

\ Set number of Flash wait states
  ldr     r0, ^MC
  mov     r1, # #FWS #8 lshift
  str     r1, [ r0, # MC_FMR0 ]		\ set Flash mode ; all
\  str     r1, [ r0, # MC_FMR1 ]	\ set Flash mode ; SAM7X512
\ we're running in high flash now, so remap
  mov     r1, # 1
  str     r1, [ r0, # MC_RCR ]		\ remap command

\ Enable the main ocillator
  ldr     r0, ^PMC
  ldr     r1, ^initMOR
  str     r1, [ r0, # CKGR_MOR ]
\ Wait for main oscillator to come ready
  begin,
    ldr     r1, [ r0, # PMC_SR ]
    and .s  r1, r1, # bit0		\ MOSCS bit must be 1
  ne, until,

\ Set up the PLLs
l: SetupPLLs
  ldr     r0, ^PMC
  mvl     r1, # initPLLcalc		\ PLLA
  str     r1, [ r0, # CKGR_PLLR ]
  begin,
    ldr     r1, [ r0, # PMC_SR ]	\ LOCK bit must be 1
    and .s  r1, r1, # bit2
  ne, until,
  begin,
    ldr     r1, [ r0, # PMC_SR ]	\ MCKRDY bit must be 1
    and .s  r1, r1, # bit3
  ne, until,

  mov     r1, # PCKpres 2 lshift	\ two stage write required
  str     r1, [ r0, # PMC_MCKR ]
  begin,
    ldr     r1, [ r0, # PMC_SR ]	\ MCKRDY bit must be 1
    and .s  r1, r1, # bit3
  ne, until,
  mov     r1, # initMCKRcalc
  str     r1, [ r0, # PMC_MCKR ]
  begin,
    ldr     r1, [ r0, # PMC_SR ]	\ MCKRDY bit must be 1
    and .s  r1, r1, # bit3
  ne, until,

  ldr     r0, ^SP			\ disable watchdog (write once)
  mov     r1, # bit15
  str     r1, [ r0, # WDT_MR ]
  mov     r1,     # $A5000000		\ key
  orr     r1, r1, # $00000100		\ ERSTL=1
  orr     r1, r1, # $00000011		\ URSTIEN=1, URSTEN=1
  str     r1, [ r0, # RSTC_MR ]		\ set up reset controller mode

\ Set up the AIC
  ldr	  r0, ^aic_all			\ disable all interrupts
  ldr	  r1, ^aic
  str	  r0, [ r1, # AIC_IDCR ]
  str	  r0, [ r1, # AIC_ICCR ]

  mov	  r0, # #8			\ will write 8 times to AIC_EOICR
L: LoopAIC0	
  str	  r1, [ r1, # AIC_EOICR ]	\ write any value to the end of interrupt
  sub .s  r0, r0, # 1
  b .ne	  LoopAIC0


\ The vector area is copied to low RAM (REMAPped internal RAM)
\ for interrupt handlers to patch.
l: CopyVectorsToZero
  mov	  r0, # vector_start		\ source
  mov	  r1, # 0			\ dest
  mov	  r2, # Vector_len		\ eight vectors + soft vectors
l: vector_loop
  ldr	  r3, [ r0 ], # 4
  str	  r3, [ r1 ], # 4
  sub .s  r2, r2, # 4
  b .ne	  vector_loop

 
\ ---------------------
\ start of Forth system
\ ---------------------

L: ECLD
\ Disable interrupts
  mrs   r0, cpsr			\ get CPSR
  orr   r0, r0, # I_BIT F_BIT or	\ set I bit ; SFP009
  msr   cpsr _c r0			\ modify control bits
\ Set up the Forth registers
  bl initStacks				\ use defaults in ARM\InitStacks.fth
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

InitReset ResetV !			\ set reset address


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] sp-guard [if]
0 equ sp-guard
[then]

\ initial values of user variables for standalone systems only
[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, span/tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]

