\ ReprogApp.fth - reprogram SAM7x Flash

((
Copyright (c) 2004, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
At end of download, check for a partial block to program.

Change max program size to depend on CPU.

Change history
==============
20060701 SFP002 Added CopyFlash.
20040212 MPE001 First release
))

only forth definitions  decimal


\ ****************
\ *S Flash handler
\ ****************

cell buffer: FlDest	\ -- addr
\ *G Holds next destination address

cell buffer: #PrgErrs	\ -- addr
\ *G Holds error count

: reboot	\ --
\ *G Reboots the CPU by activating the watchdog.
\ ** Interrupts are assumed to be disabled.
  $A500000D _SP RSTC_CR + !		\ start reset
  begin again				\ wait for reset
;

Code Prog256	\ src dest -- ior
\ *G This procedure is run from RAM and contains the
\ ** Flash erase/program tools. It is passed TOS=dest, NOS=
\ ** src, and returns a zero/one status, where zero=success.
\ ** 256 bytes are programmed. *\i{Dest} must be 256 byte
\ ** aligned. The EFC unit must already be initialised.

\ Uses R8=src, R7=dest, R6=count, R0, R1, R2
  mov     r7, TOS		\ preserve dest for page number
  ldr     r8, [ psp ], # 4	\ R8 := src
  mov     r6, # #256		\ 256 bytes per page
  begin,
    ldr     r0, [ r8 ], # 4	\ read data from RAM
    str     r0, [ TOS ], # 4	\ write to page buffer
    sub .s  r6, r6, # 4
  eq, until,
  mov     r0, r7		\ page base address
  mvn     r2, # $FF		\ r2 := FFFF:FF00 = _MC
  begin,			\ wait for MC_FSR0 FRDY bit to be set 
    ldr     r1, [ r2, # MC_FSR0 ]
    and .s  r1, r1, # 1
  ne, until,
  \ form the command
  bic     r0, r0, # $FF000000	\ clear key field
  bic     r0, r0, # $00F00000	\ clear top of 1Mb range
  bic     r0, r0, # $000000FF	\ clear command field
  orr     r0, r0, # $5A000000	\ set key field
  orr     r0, r0, # $00000001	\ set command field
  str     r0, [ r2, # MC_FCR0 ]	\ write command
  begin,			\ wait for MC_FSR0 FRDY bit to be set 
    ldr     TOS, [ r2, # MC_FSR0 ]
    and .s  r1, TOS, # 1
  ne, until,
  and .s  TOS, TOS, # $0C	\ check status bits
  mov .ne TOS, # 1		\ if either set, return 1
  mov   pc, link		\ return
End-Code

: ProgBuff	\ src dest len --
\ *G Program the given region from the source buffer in RAM.
  bounds ?do
    dup i Prog256  #PrgErrs +!  #256 +
  #256 +loop
  drop
;

: SetFMCN	\ nmcn --
\ Write the value into the FMCN field of MC_FMR0.
  $FF and #16 lshift  #FWS 8 lshift or	\ FMR0 field construction
  _MC MC_FMR0 + !
  begin  _MC MC_FSR0 + @ 1 and  until	\ wait for Flash ready
;

: InitXBuff	\ --
\ *G Initialise the Flash buffer
  Xbase ^FlBuffer !
;

: BinAligned	\ u n -- u'
\ *G Force *\i{u} to the next *\i{n} byte boundary, where *\i{n}
\ ** must be a power of two.
  tuck 1- +  swap negate and
;

: toFlash	\ src dest --
\ *G Handle 128 bytes at *\i{src} to go to *\i{dest} in Flash.
  swap #128 +				\ update RAM pointer
  dup Xtop = if				\ -- dest src ; got complete unit?
    2drop
    Xbase FlDest @ Xlen ProgBuff	\ program unit
    InitXbuff  Xlen FlDest +!		\ update pointers
  else
    ^FlBuffer !  drop			\ set next rx buffer
  endif
;

: LastToFlash	\ -- 
\ *G Program the partial buffer.
  Xbase ^FlBuffer @ over - dup if	\ -- src len
    #256 binAligned
    FlDest @ swap ProgBuff
  else
    2drop
  endif
;


\ *****************
\ *S Numeric output
\ *****************

: .nibble	\ n --
\ *G Convert a nibble to a hex ASCII digit and display it.
  $0F and  dup 9 >
  if  7 +  then
  $30 + emit
;

: .WORD		\ w --
\ *G Display w as a 4 digit unsigned hexadecimal number.
  dup #12 rshift .nibble
  dup 8 rshift .nibble
  dup 4 rshift .nibble
  .nibble
;

: .dword	\ x --
\ *G Display x as an 8 digit unsigned hexadecimal number.
  dup #16 rshift .word .word
;


\ ***************
\ *S Reprogrammer
\ ***************

l: m2
  ",  done "
l: m3
  ", Start Xmodem TX "
[defined] REBOOT [if]
l: m4
  ", Press R to reboot, other to reprogram "
[else]
l: m4
  ", Reset or press key to reprogram"
[then]
l: m5
  ", Errors"
l: m6
  ", No "

: $.	\ c-addr --
\ *G Display counted string.
  count type
;

: .PrgErrs
\ *G Display number of errors.
  cr  #PrgErrs @ .word  bl emit
  m5 $.
;

: ReprogFlash	\ --
\ *G Erase and program the Flash.
  CLD_FMCN1.5 @ SetFMCN
  init-ser  #20 ms
  begin
    InitXbuff  #PrgErrs off
    cr m3 $.  $01A emit			\ send message and trigger
    FlashBase dup FlDest !		\ set start address
    $00400000 Bin-Up drop		\ receive and program
    LastToFlash				\ program residue
    .PrgErrs  #100 ms
    cr m4 $.
[defined] REBOOT [if]
    key $DF and dup emit  [char] R =
  until
  #100 ms
  reboot
[else]
    key drop
  again
[then]
;

\ SFP002...
l: msg7
  ", Copy "

: (CopyFlash)	\ src dest len --
\ *G Copy the flash. The parameters are as for *\fo{CMOVE},
\ ** but *\i{dest} and *\i{len} must be 256 byte aligned.
  init-ser  #100 ms
  #PrgErrs off
  cr msg7 $.				\ "Copy" message
  0 do					\ -- src dest
    over Xbase #256 cmove
    Xbase over Prog256 #PrgErrs +!
    #256 +  swap #256 + swap
  #256 +loop
  2drop
  .PrgErrs  #20 ms			\ error count
;

: CopyFlash	\ src dest len --
\ *G Copy the flash. The parameters are as for *\fo{CMOVE},
\ ** but *\i{dest} and *\i{len} must be 256 byte aligned.
\ ** The process is repeated until there are no errors, and
\ ** the system is then rebooted.
  CLD_FMCN1.5 @ SetFMCN
  begin
    3dup (CopyFlash)  #PrgErrs @ 0=
  until
  #100 ms  reboot
;
\ ...SFP002


\ ======
\ *> ###
\ ======

