\ Reprogram AT91SAM7xxx over serial line.

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20061101 SFP001 First cut
))


\ =============
\ *! reprogsam7ser
\ *T Reprogramming the AT91SAM7 serially
\ =============
\ *S Introduction
\ *P *\i{ReprogSAM7ser.ctl} is the control file for the serial loader
\ ** which reprograms the on-chip Flash memory.
\ ** Because the on-chip Flash cannot be accessed during
\ ** programming, and because the initial loader code may
\ ** itself be replaced, the Flash programming code is copied
\ ** into RAM for execution. The only exit from the code is a
\ ** reset of the CPU to execute the newly Flashed program. All
\ ** interrupts are disabled during reprogramming, and so
\ ** polled serial drivers are used.

\ *P Access to the reprogramming software is defined in
\ ** *\i{REFLASH.FTH} for the main system code and in
\ ** this control file and the two must match.

\ *P Return is by rebooting the system. The contents of the
\ ** internal RAM and Flash are destroyed, therefore any data
\ ** that must be preserved should be saved externally to the
\ ** chip.


only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

"" \buildkit.dev\software\rom\arm"	setmacro CpuDir
"" \buildkit.dev\software\rom\common"	setmacro CommonDir
"" ."					setmacro AppDir
"" .."					setmacro HwDir


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  no-heads			\ do not generate heads
  no inlining			\ saves some space
  -ram-table			\ saves some space
  +LeafCalls			\ saves some space

0         equ false
false not equ true


\ ****************
\ *S Configuration
\ ****************

#48100000 equ /MCK	\ -- hz
\ *G Default CPU speed.

$00100000 equ FlashBase	\ -- addr
\ *G Base address of Flash.

1 equ #FWS	\ -- u
\ *G Default number of Flash wait states.


\ *************
\ *N Memory map
\ *************
\ *P Take care to ensure these memory regions do not overlap the stacks
\ ** of the calling program or the interrupt vector and FIQ area.
\ ** The *\fo{Xbuff} section must be a multiple of 256 bytes because
\ ** the Flash is programmed in 256 byte units.

$00200400 $00201FFF cdata section ReprogSAM7ser
$00202000 $002027FF idata section PROGd
$00202800 $00202FFF udata section PROGu
$00200000 $002003FF udata section Xbuff	\ 1024 byte Xmodem buffer
  sec-base equ Xbase			\ base address of buffer
  sec-len equ Xlen			\ length of buffer
  Xbase Xlen + equ Xtop			\ top of buffer

interpreter
: prog		ReprogSAM7ser  ;
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ***********************************
\ *N Configure stacks and kernel buffers
\ ***********************************
\ *P The stacks have already been set up by the calling program.


\ ************
\ *S File List
\ ************

include %CpuDir%\ArmDef			\ ARM CPU equates
include %CpuDir%\sfrSAM7x		\ Special function registers

l: CLD1		\ -- addr
\ *G Filled in later with the xt of *\fo{ReprogFlash}
  0 ,
l: CLD_CopyFlash	\ -- addr
\ *G Filled in later with the xt of *\fo{CopyFlash}
  0 ,
l: CLD_UART	\ -- addr
\ *G Filled in later with 0 or UART base address to use.
\ ** If set to 0, the default is UART0. This allows an
\ ** application to select which UART to use.
  0 ,
l: CLD_usdel	\ -- addr
\ *G Filled in with the number of number of iterations required
\ ** by *\fo{UDELAY} for a one microsecond delay. The default
\ ** assumes 5 clocks per iteration and a 50MHz clock
   #10 ,
l: CLD_FMCN1.5	\ -- addr
\ *G Filled in with the number of clocks in 1.5us. Used to set
\ ** set up the Flash controller. The default assumes a 50MHz
\ ** clock.
   #75 ,

include %AppDir%\primitives		\ Forth primitives from CODEARM.FTH
include %AppDir%\MinSerSAM7xp		\ polled serial line driver
include %AppDir%\delays			\ software delays

synonym ser-emit emit
\ *G A synonym required by the Xmodem code.
synonym ser-key? key?
\ *G A synonym required by the Xmodem code.
synonym ser-key  key
\ *G A synonym required by the Xmodem code.

include %AppDir%\MinXmodemRx		\ Xmodem receiver
include %AppDir%\ReprogApp		\ Application

make-turnkey ReprogFlash		\ start up action
\ *G Define the action of the reprogramming code.
' CopyFlash CLD_CopyFlash !
\ *G Define the action of the Flash copy code.


\ *******
\ The end
\ *******

decimal

FINIS


\ ******
\ *> ###
\ ******
