\ Sam7x256.ctl - Atmel AT91SAM7X256 standalone Forth control file

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20060630 MPE001 First release.
))


\ =========
\ *! sam7x256
\ =========
\ *T Standalone Forth for AT91SAM7X256

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives
"" C:\buildkit.dev\software\AddOns\PowerNet\v30dev"
		setmacro IpStack	\ where PowerNet lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
\  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused

\ =============
\ *N Memory map
\ =============
\ *P Code is run from Flash at its high address, and RAM is REMAPped.
\ ** This allows interrupt vectors to be in RAM. The first 256 bytes
\ ** of RAM are reserved for the vectors. If you add an FIQ routine
\ ** and use more than 256 bytes, adjust the start address of the
\ ** *\fo{PROGd} section accordingly.

\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $0020:0000..$0020:2FFF. Ensure that your stacks are
\ ** outside this region.

  $00100000 equ link-address	\ Address at which code will start\be linked at
  $00100000 $0013FBFF cdata section EKSAM7X	\ 256k code section, less 1024 bytes
    sec-top 1+ equ CfgFlash			\ for configuration data
  $00200100 $00200FFF idata section PROGd	\ 4k IDATA, less 256 byte vector space
  $00201000 $0020FFFF udata section PROGu	\ 48k UDATA

interpreter
: prog EKSAM7X ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ==========================================
\ *N Crystal, clock, serial and ticker rates
\ ==========================================
\ *P The AT91SAM7x has several clocks derived from two sources:
\ *(
\ *B SCK - Slow clock, usually an RC oscillator,
\ *B MOSC - Master oscillator, 18.432MHz crystal on the EK board,
\ *B PLLCK - PLL output derived from MOSC. If the USB port is to be used,
\ ** this must be set to 48, 96 or 192MHz,
\ *B MCK - derived from PLLCK,
\ *B UCK - 48 or 96 MHz USB clock derived from PLLCK.
\ *)

#18432000 equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
xtal-speed equ /MOSC		\ -- hz
\ *G Synonym used by PLL calculator.

#96200000 equ pll-speed		\ -- hz
\ *G Requested PLL clock speed in HZ. This is the target speed
\ ** of PLLCK. The equate *\fo{/PLLCK} is calculated in *\i{InitSAM7x.fth)
\ ** and represents the actual PCK speed.
1 equ USBdiv		\ -- 0/1
\ ** If set, the PLL clock will be divided by 2 or 4 to generate the 48MHz
\ ** USB clock.
\ *(
\ *B Set this to 0 if PLLCK should generate 48MHz.
\ *B Set this to 1 if PLLCK should generate 96MHz.
\ *B Set this to 2 if PLLCK should generate 192MHz.
\ *)

((
#48100000 equ system-speed	\ -- hz
\ *G Requested processor clock in HZ. This is the target speed
\ ** of PCK. The equate *\fo{/PCK} is calculated in *\i{InitSAM7x.fth)
\ ** and represents the actual PCK speed.
))
1 equ PCKpres		\ -- n
\ *G The prescaler applied to PLLCK to generate PCK. Must be in the range
\ ** 0..6, dividing by that power of two, e.g. if set to 3, PCK will be
\ ** PLLCK divided by 8.
3 equ MCKsel		\ -- n
\ *G Defines the source of the master clock
\ *(
\ *B 0 - Slow Clock
\ *B 1 - Main Clock
\ *B 2 - do not use
\ *B 3 - PLL Clock
\ *)

#14 equ /PLLdiv		\ -- n
\ *G The divider used for the PLL.
1 equ showPLLinfo?	\ -- flag
\ *G If set non-zero the results of the PLL calculations are
\ ** displayed.

1 equ #FWS		\ -- n
\ *G Number of Flash wait states. Should be 1 if *\fo{system-speed}
\ ** above is greater than 30MHz.

0 equ console-port		\ Designate serial port for terminal (0..n).
\ *G Ports 0, 1 and 2 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

1 equ useDBGU?		\ -- n
\ *G Set non-zero to compile code for the two-wire debug UART,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUSART0?	\ -- n
\ *G Set non-zero to compile code for USART0,
\ ** device *\fo{Console1}.
#38400 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

#10 equ tick-ms		\ -- ms
\ *G TIMEBASE tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Define CPU type in use
$00 #16 lshift equ 7s
$01 #16 lshift equ 7x
$11 #16 lshift equ 7xc

7x #256 or equ SAMtype		\ -- cpu# ; selects Flash stuff

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $9000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 1 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrSAM7x  		\ Special function registers.
  include %HwDir%\initSAM7x		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
.sections
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  include %CpuDir%\Drivers\rebootSAM7	\ reboot using software reset
  1 equ ARM-VIC?			\ select AT91 AIC Vectored Interrupts
  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\Drivers\serSAM7xqi	\ queued interrupt RX serial driver
\  include %CpuDir%\Drivers\serSAM7xi	\ interrupt RX serial driver
\  include %CpuDir%\Drivers\serSAM7xp	\ polled serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%\delays
  include %CpuDir%\drivers\Tick9200	\ SAM7x TCB0 is same as RM9200
  ' start-timers AtCold
[else]
  include %CommonDir%\delays
  include %CpuDir%\drivers\Tick9200	\ SAM7x TCB0 is same as RM9200
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ *S End of kernel
\ *************

buildfile ekSAM7x.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for AT91SAM7X"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%\XmodemTxRx	\ Xmodem driver

  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support
  include %HwDir%\FlashProg		\ SAM7 Flash code
  include %HwDir%\ReProg\ReFlash	\ SAM7 ReFlash utility
1 equ CPU=ARM	\ to get ARM specific code
  include %ExampleDir%\NetCode		\ network order stuff
PowerNet? [if]
  include %AppDir%\WebConfig		\ configuration tools
  include %CpuDir%\drivers\EtherSAM7x	\ Ethernet driver
  include %AppDir%\PNconfig		\ PowerNet configuration
  include %IpStack%\PowerNet.bld	\ PowerNet build
[else]
1 equ EthDiags?
1 equ Sniff?
1 equ Pingit?
  include %CpuDir%\drivers\EtherSAM7x	\ Ethernet driver
[then]


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap

\  include %ExampleDir%\benchmarks\xbench32	\ performance tests


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======

