\ WebConfig.fth

((
Copyright (c) 2006, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441    tel
  011 44 23 8033 9691    fax
  901 313 4312           US voice access to UK office


To do
=====

Change history
==============
))

decimal

\ ============
\ *! webconfig
\ *T Ethernet and Internet configuration
\ ============
\ *P The file *\i{WebConfig.fth} defines configuration data and
\ ** tools for configuring the Ethernet controller, TCP/IP
\ ** settings and other unit details such a remote web site
\ ** that your equipment connects to. These tools are useful
\ ** during both development and production for initial unit setup.

\ *P You can use this file as a model for your own hardware.
\ ** Please copy this file from the installation folder to your
\ ** application folder before use.

\ *P To avoid problems with defaults defined in other files,
\ ** compile *\i{WebConfig.fth} before the Ethernet driver or
\ ** other PowerNet code.
\ *E include %ExampleDir%\NetCode        \ network order stuff
\ ** include %AppDir%\WebConfig          \ configuration tools
\ ** include %CpuDir%\drivers\EtherSAM7x \ Ethernet driver
\ ** include %AppDir%\PNconfig           \ PowerNet configuration
\ ** include %IpStack%\PowerNet.bld      \ PowerNet build


\ *******************************
\ *S EEPROM/Flash area definition
\ *******************************
\ *P The structure declared here defines the layout of the
\ ** configuration data region.

interpreter
: [cfg		\ -- 0
  0
;

: Cfg:		\ addr size -- addr+size
  over equ  +			\ could be CONSTANT rather than EQU
;

: cfg]		\ x --
  drop
;
target

\ *P Layout of serial EEPROM or a Flash region which holds configuration
\ ** information.

#1024 constant /CfgData	\ -- n
\ *G Size of the configuration data area.

#256 equ /URL	\ -- len
\ *G Size of a buffer of a domain or page name.

\ *[
[cfg
  4 cfg: Emagic			\ $5555AAAA
  4 cfg: Eunit#			\ unit serial number
  4 cfg: EcustID		\ customer ID number
  6 cfg: EMACaddress		\ 6 byte Ethernet MAC address
  1 cfg: EDNS?			\ non-zero if we can do DNS lookups
  1 cfg: EDHCP?			\ non-zero for DHCP
  4 cfg: EIPaddress		\ 4 byte IPv4 IP addresss, 0/-1 for DHCP
  4 cfg: EEnetIPMask		\ 4 byte network IP mask
  4 cfg: EIPGateway		\ 4 byte IP gateway
  4 cfg: EPollServer		\ 4 byte Poll server IP address
  4 cfg: EPollPort		\ 4 byte Poll server port
/URL cfg: EPollURL		\ server domain name
/URL cfg: EPollPage		\ server page name
  4 cfg: ETransaction#		\ Transaction number
  0 cfg: Elast			\ defines size used
cfg]
\ *]
Elast /CfgData u> [if]
  cr ." Urk! Configuration data too big" abort
[then]

create CfgTemplate	\ -- addr
\ *G Default configuration which must match the configuration
\ ** structure above.
  here /CfgData erase
  $5555AAAA ,				\ Magic# - configuration marker
  1 ,					\ unit serial number
  1 ,					\ customer number
  $00 c, $10 c, $8B c, $F1 c, $44 c, $51 c,	\ six bytes for 48 bit MAC address
  0 c,					\ DNS byte
  0 c,					\ DHCP byte
  192 c,  168 c,  1 c,  251 c,		\ My IP address
  $FF c,  $FF c,  0 c,  0 c,		\ local IP mask
  192 c,  168 c,  0 c,  50 c,		\ IP gateway
  #213 c,  #171 c,  #222 c,  #246 c,	\ client poll server
  #80 ,					\ client poll server port
  here  ", www.safersystems.net"  /URL + org	\ poll server domain name
  here  ", /test1.aspx"  /URL + org	\ poll server page name
  1 ,					\ transaction number
here CfgTemplate - Elast <> [if]
  cr ." Urk! Template does not match configuration table" abort
[then]


\ ***************
\ *S Runtime data
\ ***************

[undefined] EtherAddress [if]
6 buffer: EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
[then]

[undefined] DNS? [if]
1 buffer: DNS?		\ -- addr
\ *G If non-zero, the first byte indicates that DNS should be
\ ** used to obtain the poll server's IP address.
[then]

[undefined] DHCP? [if]
1 buffer: DHCP?		\ -- addr
\ *G If non-zero, the first byte indicates that DHCP should be
\ ** used to obtain the unit's IP address from a local DHCP
\ ** server
[then]

[undefined] IpAddress [if]
4 buffer: IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
\ ** The range 192.168.xxx.yyy is commonly used for
\ ** private networks. The data is in network order.
[then]

[undefined] ENetIPMask [if]
4 buffer: EnetIPMask	\ -- addr
\ *G IP mask for addresses on the Ethernet port. 
\ ** The data is in network order.
[then]

[undefined] IPGateway [if]
4 buffer: IPGateway	\ -- addr
\ *G Gateway attached to Ethernet port.
\ ** The data is in network order.
\ ** Set to 0.0.0.0 if there is no gateway.
[then]

[undefined] PollServer [if]
4 buffer: PollServer	\ -- addr
\ *G Polling server IP address in network order. This is the server
\ ** that the reader unit will connect to.
\ ** The data is in network order.
[then]

[undefined] PollPort [if]
4 buffer: PollPort	\ -- addr
\ *G Polling server port address, defaults to 80.
[then]

[undefined] PollURL [if]
/URL buffer: PollURL	\ -- addr
\ *G Buffer for the poll server's domain name.
[then]

[undefined] PollPage [if]
/URL buffer: PollPage	\ -- addr
\ *G Buffer for the page to be accessed by polling operations.
[then]

[undefined] Unit# [if]
0 value Unit#	\ -- u
\ *G Unit serial number.
[then]
[undefined] CustID# [if]
0 value CustID#	\ -- u
\ *G Reader customer ID number.
[then]
[undefined] Transaction# [if]
0 value Transaction#	\ -- u
\ *G Transaction number used by the client code.
[then]


\ ****************************
\ *S Flash and EEPROM routines
\ ****************************
\ *P Select one set of these routines as required.

\ ========
\ *N Flash
\ ========

[undefined] CfgFlash [if]
SecTab dup @ cells + @ FlashBase + constant CfgFlash	\ -- addr
\ *G For configurations in Flash, we default to using the last
\ ** sector of the flash for configuration storage. Systems
\ ** that use fixed size pages, e.g. Atmel SAM7X CPUs, should
\ ** predefine *\fo{CfgFlash}.
[then]

Elast buffer: CfgBuff	\ -- addr
\ *G Buffer used during configuration read and write.

: SetDefaults	\ --
\ *G Flash: Write the default configuration to Flash.
\ ** Usually run during production configuration before
\ ** *\fo{NetSetup} below.
  CfgTemplate CfgBuff Elast cmove	\ get defaults to RAM.
  CfgBuff CfgFlash Elast ProgramFlash	\ or save to Flash
;

: DataFlash>RAM		\ --
\ *G Flash: Read the TCP/IP stack settings from the Flash configuration
\ ** area. Data is only copied to RAM if the magic number is correct.
  CfgFlash Emagic + @ $5555AAAA = if
    CfgFlash
    dup Eunit# + @ to Unit#
    dup ECustID + @ to CustID#
    dup EMACaddress + EtherAddress 6 cmove
    dup EDNS? + c@ DNS? c!
    dup EDHCP? + c@ DHCP? c!
    dup EIPaddress + IPaddress 4 cmove
    dup EEnetIPMask + ENetIPMask 4 cmove
    dup EIPGateway + IPGateway 4 cmove
    dup EPollServer + PollServer 4 cmove
    dup EPollPort + PollPort 4 cmove
    dup EPollURL + PollURL /URL cmove
    dup EPollPage + PollPage /URL cmove
    dup ETransaction# + @ to Transaction#
    drop
  then
;
' DataFlash>RAM AtCold

: RAM>DataFlash		\ --
\ *G Flash: Write the TCP/IP stack settings to the configuration Flash.
  CfgBuff
  $5555AAAA over Emagic + !
  Unit# over Eunit# + !
  CustID# over ECustID + !
  EtherAddress over EMACaddress + 6 cmove
  DNS? c@ over EDNS? + c!
  DHCP? c@ over EDHCP? + c!
  IPaddress over EIPaddress + 4 cmove
  EnetIPMask over EENetIPMask + 4 cmove
  IPGateway over EIPGateway + 4 cmove
  PollServer over EPollServer + 4 cmove
  PollPort over EPollPort + 4 cmove
  PollURL over EPollURL + /URL cmove
  PollPage over EPollPage + /URL cmove
  Transaction# over ETransaction# + !
  CfgFlash Elast ProgramFlash
;

\ ================
\ *N Serial EEPROM
\ ================

(( \ EEPROM version
: DataFlash>RAM		\ --
\ *G EEPROM: Read the TCP/IP stack settings from the configuration
\ ** EEPROM.
  addr Reader# 4 Eunit# eeprom@
  addr CustID# 4 ECustID eeprom@
  addr Transaction# 4 ETransaction# eeprom@

  EtherAddress 6 EMACaddress eeprom@
  DNS? 1 EDNS? eeprom@
  DHCP? 1 EDHCP? eeprom@
  IPaddress 4 EIPaddress eeprom@
  EnetIPMask 4 EENetIPMask eeprom@
  IPGateway 4 EIPGateway eeprom@
  PollServer 4 EPollServer eeprom@
  PollPort 4 EPollPort eeprom@
  PollURL dup c@ 1+ /URL min EPollURL eeprom@
  PollPage dup c@ 1+ /URL min EPOllPage eeprom@
;
' DataFlash>RAM AtCold

: RAM>DataFlash		\ --
\ *G EEPROM: Write the TCP/IP stack settings to the configuration
\ ** EEPROM.
  EtherAddress 6 EMACaddress eeprom!
  DNS? 1 EDNS? eeprom!
  DHCP? 1 EDHCP? eeprom!
  IPaddress 4 EIPaddress eeprom!
  EnetIPMask 4 EENetIPMask eeprom!
  IPGateway 4 EIPGateway eeprom!
  PollServer 4 EPollServer eeprom!
  PollPort 4 EPollPort eeprom!
  PollURL dup c@ 1+ /URL min EPollURL eeprom!
  PollPage dup c@ 1+ /URL min EPOllPage eeprom!
;

: SetDefaults	\ --
\ *G EEPROM: Write the default configuration to EEPROM.
\ ** Usually run during production configuration before
\ ** *\fo{NetSetup} below.
  CfgTemplate Elast 0 eeprom!		\ save to EEPROM
;
))


\ ********************
\ *S Set up operations
\ ********************

\ =======================================
\ *N Displaying and Entering IP addresses
\ =======================================

: .IP		\ caddr --
\ *G Display the IP address at *\i{caddr}.
  @(n) .IPaddress
;

: (GetIPaddress)	\ -- ipaddr|INADDR_NONE|0
\ *G Get an IPv4 address from the user. *\FO{INADDR_NONE} (-1) is
\ ** is returned if the entry is bad, and zero is returned for a
\ ** zero length entry. Use *\fo{.IPaddress ( ipaddr -- )} to
\ ** display an IP address.
  { | str[ #20 ] -- ipaddr }
  str[ #20 erase  str[ #20 accept
  str[ swap dup
  if  >inet_aton  else  nip  endif
;

: GetIPaddress	\ caddr -- ior
\ *G Get an IP address from the console in the form
\ ** "192.168.0.55" (base is decimal). A good result is saved at
\ ** *\i{caddr} and ior is returned 0 for success or a null entry,
\ ** or other for a bad entry. Note that *\i{caddr} will always
\ ** be modified.
  >r
  case  (getIPaddress) dup
    0 of  endof				\ no change = success
    -1 of  endof			\ bad entry
      r@ !(n)  drop 0			\ success
  end-case
  r> drop
;

: GetIP		\ caddr -- ior ; 0=success
\ *G Get an IP address from the console in the form
\ ** "192.168.0.55" (base is decimal). A good result is saved at
\ ** *\i{caddr}.
  { | temp[ 4 ] -- }
  begin
    dup temp[ 4 cmove
    cr ." Current = " temp[ .IP ."  New: " temp[ getIPaddress
   while
    cr ." Invalid IP address "
  repeat
  cr ." Using: " temp[ over 4 cmove  .IP
;


\ ========================================
\ *N Displaying and Entering MAC addresses
\ ========================================

: .MACaddress	\ caddr --
\ *G Display the MAC address at *\i{caddr}.
  base @ >r  hex
  dup     c@ .byte [char] - emit
  dup 1 + c@ .byte [char] - emit
  dup 2 + c@ .byte [char] - emit
  dup 3 + c@ .byte [char] - emit
  dup 4 + c@ .byte [char] - emit
  5 + c@ .byte 
  r> base !
;

: getByte	\ addr char -- addr'
\ *G Collect an integer delimited by char and store it
\ ** at addr, returning addr+1.
  word integer? 1 <> throw		\ collect and convert
  dup #255 u> throw			\ restrict to 0..255
  over c!  1+				\ stash data and update address
;

: getData	\ addr char n --
\ *G Collect n integers delimited by char, and store the bytes
\ ** sequentially at addr.
  1- 0 max 0 ?do
    tuck getByte swap
  loop
  drop  bl getByte drop
;

: GetMACaddress	\ caddr -- ior ; 0=success
\ *G Get an Ethernet MAC address from the console in the form
\ ** "aa-bb-cc-dd-ee-ff" (base is hex). The result is saved at
\ ** *\i{caddr} and ior is returned 0 for success or a null entry,
\ ** or other for a bad entry. Note that *\i{caddr} will always
\ ** be modified.
  base @ >r  hex
  FlushKeys Query  #tib @ if
    tib #tib @ upper
    [char] - 6 ['] getData catch dup
    if  nip nip nip  then
  else
    drop 0
  endif
  r> base !
;

: GetMac	\ caddr -- ; input new MAC address
\ *G Get a new MAC address from the user console.
  { | temp[ 8 ] -- }
  begin
    dup temp[ 6 cmove
    cr ." Current = " temp[ .MACaddress ."  New: " temp[ getMACaddress
   while
    cr ." Invalid MAC address "
  repeat
  cr ." Using: " temp[ over 6 cmove  .MacAddress
;


\ ===============
\ *N Setup proper
\ ===============

[undefined] y/n? [if]
: y/n?		\ -- flag
\ *G Wait for a key and return true if the key is Y.
  ."  Y/N: " key upc  dup emit  [char] Y =
;
[then]

: EtherSetup	\ --
\ *G Set up the unit's Ethernet address.
  cr ." Enter Ethernet MAC address in the form: aa-bb-cc-dd-ee-ff"
  EtherAddress GetMac
;

: .IPformat	\ --
  ." Enter IP addresses in the form: aaa.bbb.ccc.ddd"
;

sizeofheap 0<>  /URL PADSIZE > and [if]
: getC$		\ caddr --
\ *G Get a counted string of up to *\fo{/URL} characters from the
\ ** keyboard and save it at *\i{caddr}.
\ ** This version is compiled if a heap is present.
  /URL 2 + allocate
  if  ." Heap error"  2drop exit  endif
  dup >r  /URL accept dup if
    r@ swap #255 min rot place
  else
    2drop
  endif
  r> free drop
;
[else]
: getC$		\ caddr --
\ *G Get a counted string of up to 62 characters from the keyboard
\ ** and save it at *\i{caddr}.
\ ** This version is compiled if a heap is not present.
  pad /URL PADSIZE 2 - min accept dup if
    pad swap rot place
  else
    2drop
  endif
;
[then]

: (getU)	\ caddr -- ior
\ *G Get an unsigned number and place it at caddr.
\ ** Return 0 for success or a null entry.
  { | str[ #20 ] -- ipaddr }
  str[ #20 erase  str[ #20 accept
  str[ swap dup if			\ -- caddr a n
    case  (integer?)			\ -- caddr [d 2|n 1|0]
      0 of  drop -1  endof
      1 of  swap !  0  endof
      2 of  2drop drop -2  endof
    endcase
  else
    nip nip
  endif
;

: getU		\ caddr --
\ *G Get an unsigned number and place it at caddr.
  { | temp[ cell ] -- }
  begin
    dup @ temp[ !
    ." Curr: " temp[ @ u. ." New: " temp[ (getU)
   while
    cr ." Invalid number, retry" cr
  repeat
  temp[ @ over !  cr ." Using " @ u.
;

: .BadE$	\ --
\ *G Warning for unconfigured string.
  ." *** Not yet set ***
;

: E$.		\ caddr --
\ *G Display the counted string unless the count is $00 or $FF,
\ ** in which case display a warning.
  case  count
    $00 of  drop .BadE$  endof
    $FF of  drop .BadE$  endof
       type
  end-case
;

: (NetSetup)	\ --
  cr ." Use <Enter> alone to accept existing values"
  EtherSetup
  cr ." Use DHCP to get an IP address: "
  y/n?  dup DHCP? c!  0= if
    cr .IPformat
    cr ." Unit IP address: " IPaddress getIP
    cr ." Network mask   : " ENetIPmask getIP
    cr ." Gateway address: " IPGateway getIP
  else
    0  dup IPaddress !  dup ENetIPmask !  IPGateway !
  endif
  cr ." Poll server name: " PollURL E$.  space PollURL getC$
  cr ." Poll server page: "  PollPage E$. space  PollPage getC$
  cr ." Poll port number (default=80): " PollPort getU
  cr ." Use DNS for poll server: " y/n?  dup DNS? c!  0= if
    cr .IPformat
    cr ." Poll Server IP address: " PollServer getIP
  else
    0 PollServer !
  endif
;

: NetSetup	\ --
\ *G Configure the unit for the network and server.
  begin
    cr ." Network set up"
    cr ." =============="
    (NetSetup)
    cr ." =============="
    cr ." Accept " y/n?
  until
  RAM>DataFlash
;


\ ======
\ *> ###
\ ======

decimal

