\ Embedded Artists 2138+10 Ethernet - Standalone Forth control file

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20070109 MPE001 Converted from 100Mb file
))


\ ============
\ *! ea2138e10
\ ============
\ *T Standalone Forth for LPC2138 User Flash
\ *P The control file EA2138e100.CTL generates a standalone
\ ** Forth for the Embedded Artists 2138 10Mbps Ethernet QuickStart
\ ** board.

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives
"" \buildkit.dev\software\AddOns\PowerNet\Dev"
                setmacro IpStack	\ where the PowerNet stack lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  Hex-I32			\ produce Intel Hex 32 bit as well

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

\ Define CPU type in use
 #2138 equ LPCtype		\ -- cpu# ; selects Flash stuff

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   1 equ event-handler?		\   true to include event handler
   1 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $4400 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?		\ true for PowerNet


\ Crystal, clock, serial and ticker rates
#14745600 equ xtal-speed	\ crystal clock rate in HZ.
\ for greatest baud rate flexibility use 14.7456 MHz
\ #60000000 equ system-speed	\ System clock rate in HZ.
xtal-speed 4 * equ system-speed	\ System clock rate in HZ.
  1 equ /vpb			\ VPB clock divider    1  2  4
  %01 equ init-vpb		\ VPB bits           % 01 10 00
#115200 equ console0-speed	\ Serial port speed in BPS.
#38400 equ console1-speed	\ Serial port speed in BPS.

0 equ console-port		\ Designate serial port for terminal (0/1).
\ *G Ports 0 and 1 are the on-chip UARTs. The internal/external USB device
\ ** is port 2, and bit-banged ports are defined from 10 onwards.
1 equ useUART0?	\ -- n
\ *G Compile UART0 code if non-zero.
1 equ useUART1?	\ -- n
\ *G Compile UART1 code if non-zero.

#10 equ tick-ms			\ TIMEBASE tick in ms


\ define stack and user area sizes for TASKs
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true

\ **********
\ Memory map
\ **********

  $00000000 equ link-address	\ Address at which code will start\be linked at
  $00000000 $0007DFFF cdata section EA2138E10	\ code section
    sec-len equ Boot_Flash_len
  $7FFFE000 $7FFFFFFF udata section BootBlock	\ 8k ROM boot loader
    $7FFFFFF0 equ IAPentry			\   Thumb code entry point
  $40000000 equ ISPram				\ 512 bytes for ISP loader
  $40000200 $400007FF idata section PROGd	\ IDATA less 512 for ISP loader
  $40000800 $40007FFF udata section PROGu	\ UDATA
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP
    #256 reserve equ ISPstack			\   256 bytes for ISP stack
    						\   IAP stack is Forth return stack, max 128 bytes

interpreter
: prog EA2138E10  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrLPC2138  		\ Special function registers.
  include %HwDir%\initLPC210x		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  include %CpuDir%\Drivers\rebootLPC210x	\ reboot using watchdog
  3 equ ARM-VIC?			\ select PL190 Vectored Interrupts
  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\Drivers\serLPC210xi	\ interrupt RX serial driver
\  include %CpuDir%\Drivers\serLPC210xp	\ polled serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
[then]
  include %CommonDir%\delays		\ tie delays
  include %CpuDir%\drivers\TickLPC210x	\ timer tick

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ *S End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE ARM 2138 PowerForth v6.30 "
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  include %HwDir%\IAP210x		\ IAP code
  include %AppDir%\Reprog\Reflash	\ Reflash system
  include %HwDir%\FlashTables		\ Flash sector tables
  include %HwDir%\FlashProg		\ higher level Flash routines

((
Thumb16
proc inThumb
  mov  r0, # $55
  add  r1, r0, # 1
  add  r2, r1, # 1
  add  r3, r2, # 1
  bx   lr
endproc
ARM32

code TestThumb
\ A2C8
  mov  r7, lr
  mvl  r0, inThumb
  orr  r0, r0, # 1
  mov  lr, pc		\ points to first NOP
  bx   r0
  nop  nop  nop
  mov  lr, r7
  next,
end-code
))


\ *******************
\ *S PowerNet support
\ *******************

PowerNet? [if]

[undefined] later [if]
optimised? [if]
compiler
: later         \ n -- n'
  ticks +
;

: timedout?     \ n -- flag ; true if timed out
  ticks - 0<                            \ must use circular arithmetic
;
target
[then]
[then]

timebase? [if]
' start-timers atcold
[else]
Synonym Start-Timers Start-Clock
' start-clock atcold
[then]

: DataFlash>RAM	\ --
;

[then]


\ ***********
\ *S PowerNet
\ ***********
\ *P First of all you must add a text macro to your main control file
\ ** to tell it where the stack is located. The code below was used
\ ** during development.
\ *E "" D:\buildkit.dev\software\AddOns\PowerNet\v30dev"
\ **                setmacro IpStack	\ where the PowerNet stack lives
\ *P The macro *\fo{IPSTACK} tells the cross compiler where the file
\ ** *\i{POWERNET.BLD} is located.

PowerNet? [if]

include EA2138PNconfig			\ PowerNet configuration settings

\ *P Now define the default Ethernet MAC and IP addresses. If DHCP or
\ ** APIPA are being used, the IP addresses *\b{must} be in RAM.

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources. This definition is usually provided by the Ethernet
\ ** driver if compiled first.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $3D c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!
[then]

useDHCP? 0= [if]

create IPaddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
\ ** The range 192.168.xxx.yyy is commonly used for
\ ** private networks. This definition is usually
\ ** provided by the Ethernet driver which must be
\ ** compiled first.
\ ** The data is in network order.
  192 c,  168 c,  1 c,  251 c,		\ assign these as required

create EnetIPMask	\ -- addr
\ *G IP mask for addresses on Ethernet port.
\ ** The data is in network order.
  $FF c,  $FF c,  0 c,  0 c,

create IPGateway	\ -- addr
\ *G Gateway attached to the Ethernet port.
\ ** The data is in network order.
\ ** Set to 0.0.0.0 if there is no gateway.
  0 ,

[else]

variable IPaddress	\ -- addr
  0 IPaddress !
variable EnetIPMask	\ -- addr
  0 EnetIPMask !
variable IPGateway	\ -- addr
  0 IPGateway !

[then]

  include %CpuDir%\Drivers\Ether8019AS	\ RealTek Ethernet driver
  include %IpStack%\PowerNet.bld	\ PowerNet build file

[then]


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap
\ include %ExampleDir%\tasking


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
sec-base 0				\ -- addr 0
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $18 + @ +
  over $1C + @ +
  negate
swap $14 + !

decimal
\ cr ." PC = " here dup .dword ."  " .

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

FINIS


\ ======
\ *> ###
\ ======

