\ IAP210x.FTH - LPC210x IAP routines

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20040809 MPE001 Released for use with bootloaders v1.52/1.63.
))

decimal

\ ======
\ *! iap210x
\ *T Philips LPC2xxx IAP routines
\ ======
\ *P The IAP is accessed by calling a Thumb routine at the
\ ** IAPentry address with the address of a command block in R0
\ ** and the address of a status/result block (RAM) in R1.
\ ** All the IAPxxx words return a 0 result on success. Note
\ ** also that all interrupts are disabled for the duration
\ ** of an IAP call, and therefore that ticker interrupts will
\ ** not be serviced. In particular, the sector erase command
\ ** may take 400ms, and a write of a 512 byte line may take 1ms.

\ *P The LPC210x CPUs have 128kb Flash in 8kb sectors numbered
\ ** from 0..15.  These sectors may be programmed in units of
\ ** 512 bytes. Sector 15 is the boot sector which cannot be erased
\ ** or programmed. See the LPC2106 User Manual for more details.

[undefined] FullIAP? [if]
1 equ FullIAP?		\ -- n
\ *G If this EQUate is set non-zero, additional IAP routines
\ ** are compiled, e.g. to get the bootloader version number
\ ** and the device part number. The definition here is only
\ ** used if it has not been previously defined.
[then]

5 cells buffer: IAPcmd	\ -- addr ; max 5 cells
\ *G Command input buffer for IAP routines.
3 cells buffer: IAPres	\ -- addr ; max 3 cells
\ *G Result output buffer from IAP routines.

l: ^IAP
  IAPentry 1 or ,			\ entry address, Thumb mode

code IAP	\ *cmd *res --
\ *G The primitive to call the IAP routines. Interrupts are
\ ** disabled for the duration of the IAP call.
  str	link, [ rsp, # -4 ] !		\ save return address

\ Disable interrupts
  mrs   r0, cpsr			\ save interrupt status
  str   r0, [ rsp, # -4 ] !
  orr   r0, r0, # $C0			\ disable interrupts
  msr   cpsr, _c r0

\ Set up parameters and call the IAP Thumb routine
  ldr   r0, [ psp ], # 4		\ R0 = *cmd
  mov   r1, tos				\ R1 = *res
  stmfd rsp ! { r9-r12 }		\ preserve regs
  ldr   r12, ^IAP			\ get entry address
  mov   link, pc			\ link set to after BX
  bx    r12				\ jump to Thumb code
  mov   r0, r0
  mov   r0, r0
  mov   r0, r0
  mov   r0, r0
  ldmfd rsp ! { r9-r12 }		\ restore regs

\ restore interrupts
  ldr   r0, [ rsp ], # 4
  msr   cpsr, _c r0

  ldr   tos, [ psp ], # 4		\ restore TOS
  ldr   pc, [ rsp ], # 4		\ return
end-code

system-speed equ prog-speed
\ xtal-speed equ prog-speed

: IAPprep	\ start end -- res
\ *G Prepare sectors for erase/write.
  IAPcmd
  #50 over !				\ -- start end addr ; cmd#
  rot over cell + !			\ -- end addr ; start
  tuck  2 cells + !			\ -- addr ; end
  IAPres IAP  IAPres @
;

: IAPcopy	\ Rsrc Fdest len -- res
\ *G Copy/program len bytes from Rsrc in RAM to Fdest in Flash.
  IAPcmd
  #51 over !				\ cmd
  tuck  3 cells + !			\ len
  tuck  cell + !			\ Fdest
  tuck  2 cells + !			\ Rsrc
  prog-speed #1000 / over 4 cells + !	\ -- addr ; cclk/1000
  IAPres IAP  IAPres @
;

: IAPerase	\ start end -- res
\ *G Erase the inclusive range of sectors.
  IAPcmd
  #52 over !				\ cmd
  tuck  2 cells + !			\ end
  tuck  cell + !			\ start
  prog-speed #1000 / over 3 cells + !	\ cclk/1000
  IAPres IAP  IAPres @
;

: IAPcheck	\ start end -- res
\ *G Blank check the inclusive range of sectors.
  IAPcmd
  #53 over !				\ cmd
  tuck  2 cells + !			\ end
  tuck  cell + !			\ start
  IAPres IAP  IAPres @
;

: IAPcompare	\ Rsrc Fdest len -- res
\ *G Compare len bytes from Rsrc in RAM to Fdest in Flash.
  IAPcmd
  #56 over !				\ cmd
  tuck  3 cells + !			\ len
  tuck  cell + !			\ Fdest
  tuck  2 cells + !			\ Rsrc
  IAPres IAP  IAPres @
;

FullIAP? [if]

: IAPbootver	\ -- bootver
\ *G Return the 16 bit boot code version. The high byte is the
\ ** major version and the low byte is the minor version.
  IAPcmd
  #55 over !
  IAPres IAP  IAPres cell + w@
;

: IAPpartno	\ -- part#
\ *G Return the Philips part number.
  IAPcmd
  #54 over !
  IAPres IAP  IAPres cell + @
;

[then]


\ **********
\ *S Gotchas
\ **********
\ *P If you have problems with the IAP routines, check the
\ ** bootloader version using the Philips ISP software or by typing
\ *C   IAPBootVer .dword
\ *P which will give something of the form:
\ *C   0000:xxyy
\ *P xx is the major version number and yy is the minor version
\ ** number. If this number is less than 0000:0134 (hexadecimal) or
\ ** 1.52 (decimal) you should update the bootloader using ISP
\ ** software version 2.2.0 or greater. These are available on
\ ** the MPE CDs and from
\ *E   www.semiconductors.philips.com
\ **     /files/products/standard/microcontrollers/utilities/
\ **       philips_flash_utility.zip
\ **       lpc2000_bl_update.zip

\ *P If you still have problems, use the Philips ISP software to
\ ** erase the whole of the Flash, and then reinstall an appropriate
\ ** .HEX file. Remember to convert the latest .IMG file to .HEX.


\ ======
\ *> ###
\ ======

decimal

