\ INITLP210x.FTH - Philips LPC210x generic startup code

(( 
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ ***************************************
\ Generic start up table and entry points
\ ***************************************

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr 	pc, ResetV              \ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  dd    0                       \ 14 - checksum
  ldr 	pc, [ pc, # $-FF0 ]	\ 18 - IRQ interrupts from VicVectAddr @ $FFFFF030
  ldr 	pc, FIQV		\ 1C - FIQ interrupts

L: Soft_Vectors	\ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code

Vector_End Vector_Start - equ Vector_Len


\ ************
\ Startup code
\ ************

\ Required Forth pointers
L: CLD1		                        \ cold start word
  0 ,				        \ compiler will fill up with address of word to execute at cold start
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,

\ Required peripheral pointers
l: ^SYSCON				\ system control block
  _SYSCON ,

interpreter
: gen-PLL	\ -- mval pval
  system-speed xtal-speed /
  dup 0= if
    cr ." system speed less than xtal speed" abort
  endif
\  dup 1 and if
\    cr ." system/xtal must be even" abort
\  endif
  1-
  system-speed 2 * #156000000 >
  if  0  exit  endif
  system-speed 4 * #156000000 >
  if  1  exit  endif
  system-speed 8 * #156000000 >
  if  2  exit  endif
  system-speed #16 * #156000000 >
  if  3  exit  endif
  cr ." Can't find PLL P value" abort
;

: gen-mam	\ -- mamtin
  system-speed 1- 20000000 / 1+ 
;
target

system-speed xtal-speed <> [if]
  gen-pll equ Pval  equ Mval
  Pval 5 <<n  Mval or  equ PLLCFGval
[then]
  gen-mam equ MAMval

Proc InitReset	\ power up entry point through vector
\ Get address of system control block
  ldr		r0, ^SYSCON

\ Switch to "User FLASH mode" so interrupt vectors are not remapped
  mov		r1, # 1
  str		r1, [ r0, # MEMMAP ]

\ Set the VPB divisor
  mov		r1, # init-vpb
  str		r1, [ r0, # VPBDIV ]

\ Set up the PLL and MAM
\ Turn off the MAM
  mov	        r1, # 0
  str		r1, [ r0, # MAMCR ]
\ Set the PLL
system-speed xtal-speed <> [if]
  mov		r4, # $AA		\ initialise feed sequence
  mov		r5, # $55
  mov		r3, # PLLCFGval		\ set required configuration
  str		r3, [ r0, # PLLCFG ]
  str		r4, [ r0, # PLLFEED ]
  str		r5, [ r0, # PLLFEED ]
  mov		r3, # 1			\ PLLE=1, PLLC=0
  str		r3, [ r0, # PLLCON ]
  str		r4, [ r0, # PLLFEED ]
  str		r5, [ r0, # PLLFEED ]
l: waitlock
  ldr		r3, [ r0, # PLLSTAT ]
  and .s	r3, r3, # bit10		\ test LOCK status
  b .eq		waitlock
  mov		r3, # 3			\ PLLE=1, PLLC=1
  str		r3, [ r0, # PLLCON ]
  str		r4, [ r0, # PLLFEED ]	\ feed sequence
  str		r5, [ r0, # PLLFEED ]
l: waitconnect
  ldr		r3, [ r0, # PLLSTAT ]
  and .s	r3, r3, # bit9		\ test CONNECT status
  b .eq		waitconnect
[then]
\ Re-enable the MAM
  mov		r1, # MAMval		\ n CCLK cycles
  str		r1, [ r0, # MAMTIM ]
  mov		r1, # 2			\ MAM fully enabled
  str		r1, [ r0, # MAMCR ]

\ Initialise the stacks for INTARM3.FTH
  bl		InitStacks		\ see ARM\INITSTACKS.FTH


\ *****************************
\ Optional code copy
\ *****************************

\ If required, copy code into RAM for execution
((
  mov		r0, # Flash_start	\ source
  mov		r1, # RAM_start		\ dest
  mov		r2, # Flash_len		\ len, must be multiple of 4
l: Flash_loop
  ldr		r3, [ r0 ], # 4
  str		r3, [ r1 ], # 4
  sub .s	r2, r2, # 4
  b .ne		flash_loop
))

 
\ *********************
\ Start of Forth system
\ *********************

L: ECLD
\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

InitReset prog sec-base -  ResetV !	\ INITRESET is in LOW memory starting at 0.


\ ******************************************
\ Reset values for user and system variables
\ ******************************************

[undefined] sp-guard [if]
  0 equ sp-guard
[then]

\ initial values of user variables for standalone systems only
[undefined] umbilical? [if]
L: USER-RESET
  init-s0   tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,                 		\ r0
  0 ,  0 ,                              \ #tib, span
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ *********************
\ Low level serial code
\ *********************

(( \ Is the UART working?
  bl init-uart0
l: initreseta
  mov           r0, # ascii A
  bl emit0
  b initreseta
))

((
L: ^UART0
  _UART0 ,
L: ^PCONP
  _PCONP ,
L: ^PINSEL0
  _PINSEL0 ,

proc init-uart0	\ --
\ *G Initialise serial port 0
\ enable peripheral clock
  ldr    r1, ^PCONP
  ldr    r0, [ r1 ]
  orr    r0, r0, # bit3
  str    r0, [ r1 ]
\ enable alternate functions on P0:1.0
  ldr    r1, ^PINSEL0
  ldr    r0, [ r1 ]
  bic    r0, r0, # $0F
  orr    r0, r0, # $05
  str    r0, [ r1 ]
\ initialise the UART
  ldr    r1, ^UART0
  mov    r0, # 1
  str    r0, [ r1, # UnFCR ]		\ essential
  mov    r0, # $083
  str    r0, [ r1, # UnLCR ]		\ 8 bit, 1 stop bit, no parity, enable DLAB
  mov    r0, # 0
  str    r0, [ r1, # UnDLM ]
  mov    r0, # 4
  str    r0, [ r1, # UnDLL ]
  mov    r0, # $003
  str    r0, [ r1, # UnLCR ]
  mov    pc, link
end-code

proc emit0	\ char in R0
  ldr    r1, ^UART0
l: emit0a
  ldr    r2, [ r1, # UnLSR ]
  and .s r2, r2, # $020
  b .eq  emit0a
  str    r0, [ r1, # UnTHR ]
  mov    pc, link
end-code
))

  
\ ======
\ *> ###
\ ======

decimal

