\ LPC2106uf.ctl - MPE LPC2106 Umbilical Forth target control file

((
Copyright (c) 2004, 2008

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 80 631441
fax: +44 (0)23 80 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20081127 MPE001 Converted for v7 and Stamp compiler
))


\ =========
\ *! lpc2106uf
\ *T Umbilical Forth for LPC2106 User Flash
\ =========
\ *P The control file LPC2106uf.ctl generates an Umbilical
\ ** Forth for Philips LPC2106 systems.

only forth definitions decimal


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE                   \ turn compiler on

only forth definitions          \ starting search order
  no-heads			\ heads not needed for Umbilical Forth
  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if]
  +xrefs			\ Not Stamp/IRTC compilers
[then]
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  Hex-I32			\ generate Intel Hex file

\  nt-access-ports		\ for Windows NT, 2000 ...


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N LPC type being used
\ ======================

#2106 equ LPCtype	\ -- n
\ *G Define the LPC type being used. This is used in the
\ ** CPU specific code such as the IAP access code to
\ ** select CPU specific data and routines.


\ =====================
\ *N Speed selection
\ =====================

\ Crystal, clock, serial and ticker rates
#10000000 equ xtal-speed	\ crystal clock rate in HZ.
\ for greatest baud rate flexibility use 14.7456 MHz
#60000000 equ system-speed	\ System clock rate in HZ.
  4 equ /vpb			\ VPB clock divider    1  2  4
  %00 equ init-vpb		\ VPB bits           % 01 10 00

#115200 equ console0-speed	\ Serial port speed in BPS.
#38400 equ console1-speed	\ Serial port speed in BPS.
0 equ console-port		\ Designate serial port for terminal (0/1).
\ *G Ports 0 and 1 are the on-chip UARTs. The internal/external USB device
\ ** is port 2, and bit-banged ports are defined from 10 onwards.
1 equ useUART0?	\ -- n
\ *G Compile UART0 code if non-zero.
0 equ useUART1?	\ -- n
\ *G Compile UART1 code if non-zero.

#10 equ tick-ms			\ TIMEBASE tick in ms


\ ======================================
\ *N Stack, task and interrupt selection
\ ======================================

\ define stack and user area sizes for TASKs
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$30 equ max-depth		\ max depth for UF data stack before UF barfs


\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused


\ **********
\ *N Memory map
\ **********

  $00000000 $0001DFFF cdata section LPC2106uf	\ code section
  $40000200 $40000FFF idata section PROGd	\ 4k IDATA
  $40001000 $4000FFFF udata section PROGu	\ 60k UDATA
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP
    $7FFFFFF0 equ IAPentry			\   Thumb code entry point

interpreter
: prog LPC2106uf ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ =====================
\ *N Software selection
\ =====================

 1 equ umbilical?               \ true = Umbilical Forth, zero = standalone
   0 umbilical? and equ SPI?	\ true for SPI Umbilical, false for serial
   SPI? 0= equ Serial?		\ true for Serial Umbilical, false for SPI
 1 equ init-idata?		\ true if IDATA to be initialised

\ Kernel components
 0 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

interpreter
: AtCold	\ xt --
\ *G Most Umbilical Forth systems do not use the *\fo{ColdChain}
\ ** mechanism provided in *\i{Kernel62.fth} for standalone
\ ** Forths. To prevent compilation problems, define *\fo{AtCold}
\ ** as *\fo{DROP}. Compilers built from December 2008 onwards
\ ** will warn you if *\fo{AtCold} is used before being defined
\ ** in the target code.
  drop
;
target

\ *****************
\ default constants
\ *****************

8 equ BSOUT			  \ backspace character
cell equ cell			  \ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrLPC2106  		\ Special function registers.
  include %HwDir%\initLPC210x		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CommonDir%\messages		\ Umbilical messages
  include %CpuDir%\minARM		\ main code defs.
  include %CommonDir%\UFkernel		\ Forth high level kernel

  3 equ ARM-VIC?			\ select PL190 Vectored Interrupts
0 equ test-isr?	\ -- n ; non-zero to compile test code
\ *G Non-zero to compile ISR test and diagnostic code.
1 equ SimpleAborts?	\ -- n
\ *G If this equate is non-zero, simple DAbort, PAbort, and Undef
\ ** handlers are installed.
  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\Drivers\serLPC210xi	\ interrupt RX serial driver
\  include %AppDir%\Reprog\Reflash	\ Reflash system
  include %HwDir%\IAP210x		\ IAP code
  include %HwDir%\FlashTables		\ Flash sector tables
  include %HwDir%\FlashProg		\ higher level Flash routines

umbilical? [if]
  Synonym wait-byte  serkey0		\ Say which XTL drivers to use
  Synonym send-byte  seremit0
  Synonym Wait-Byte? serkey?0
  Synonym Init-XTL   Init-Ser
  include %CommonDir%\targend		\ message driver
[then]
\  include T2periodic			\ T2 periodic timer


\ *******************
\ *S Application code
\ *******************

umbilical? 0= [if]
: cold	\ -- ; N.B. remove INIT-SER for interactive testing
  initVIC init-ser			\ initialise VIC and UART
  SerConsole opvec !			\ select I/O devices
  SerConsole ipvec !
  $0A base c!				\ default number conversion base
  cr cr ." MicroProcessor Engineering Umbilical Forth"
  begin
    cr  $0A 0
    do  i $030 + emit  loop		\ display numbers 0..9
    cr ." Press any key to continue " key drop
  again
;
[then]

umbilical? [if]
: runUmb	\ --
\ *G Starts Umbilical Forth with additional initialisation.
  init-idata initVIC run-umbilical
;
[then]


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
sec-base 0				\ -- addr 0
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $18 + @ +
  over $1C + @ +
  negate
swap $14 + !


\ *******************************************************************
\ Define what the system will do at power up.
\ For the final application, use UMBILICAL? = 0
\ and TARGEND.FTH will not be included when the interactive Umbilical
\ Forth is no longer needed. RUN-UMBILICAL is defined in TARGEND.FTH.
\ *******************************************************************

decimal

umbilical? [if]
  make-turnkey runUmb			\ cold start to Umbilical Forth
  "" COM4:" console0-speed serial	\ define link driver
  umbilical-forth                       \ switch to interactive mode
[else]
  make-turnkey cold                     \ cold start to application
  finis
[then]

FINIS					\ catch all in case ...


\ ======
\ *> ###
\ ======


