\ Reflash.fth - handles reflashing the system

((
Copyright (c) 2004, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20060701 SFP001 Added CopyFlash.
))


\ ==========
\ *! reflash
\ *T Philips LPC2xxx Reflashing
\ ==========
\ *S Introduction
\ *P If you destroy the application in the internal Flash, you
\ ** must use the Philips ISP loader to reload an Intel Hex file
\ ** supplied on the ARM Stamp CD. A suitable baud rate is 38400 baud.
\ ** To use this, ensure that port P0.14 is low. There is a link
\ ** on the board that (when installed) enforces this. Then reset
\ ** the board and use the Philips loader. Then close the Philips
\ ** loader. Ensure that the P0.14 link is removed before resetting
\ ** the board. Then connect using AIDE's PowerTerm or HyperTerm.

\ *P Once the Forth system is running again, you can use the word
\ ** *\fo{REFLASH ( -- )} to download a new binary image.
\ ** Note that *\fo{REFLASH} only handles binary memory image
\ ** files. These should transferred using the XModem 128
\ ** (checksum) protocol. If you are using AIDE with the file
\ ** server enabled, a file selection dialog will appear
\ ** automatically after you have executed *\fo{REFLASH}.

\ *P The LPC2xxx can only be reflashed from an application by
\ ** a program running from RAM. A separate application built
\ ** by a control file *\i{REPROG\REPROG*.CTL} is used to do this. A
\ ** binary image *\fo{REPROG*.IMG} is inserted into the application.
\ ** When required, the code is copied into the internal RAM and
\ ** executed from RAM.

\ *P Return is by rebooting the system. The contents of the
\ ** internal RAM and Flash are destroyed, therefore any data
\ ** that must be preserved should be saved externally to the
\ ** chip. Alternatively, modify the reprogramming code to
\ ** use the last sector to hold data to be
\ ** preserved.


\ ***************************
\ *S Code in main application
\ ***************************
\ *P The layout of the RAM code is defined in the source
\ ** file *\i{InitRp210x.fth}. This defines the first cell as
\ ** the Forth word to execute.

create reprog2xxx.img	\ -- addr
\ *G The start address of the reprogramming code

LPCtype #2106 = [if]
console-port 2 = [if]	\ USB Stamp port?
  data-file %HwDir%/ReProg/REPROGUSB.IMG equ /reprog	\ -- len
[else]
  data-file %HwDir%/ReProg/REPROG210X.IMG equ /reprog	\ -- len
\ *G The length of the reprogramming code loaded into
\ ** the dictionary.
[then]
[then]

LPCtype #2134 = [if]
  data-file %HwDir%/ReProg/REPROG2134.IMG equ /reprog	\ -- len
\ *G The length of the reprogramming code loaded into
\ ** the dictionary.
[then]

LPCtype #2138 =
LPCtype #2148 = or [if]
  data-file %HwDir%/ReProg/REPROG213X.IMG equ /reprog	\ -- len
\ *G The length of the reprogramming code loaded into
\ ** the dictionary.
[then]

$40000200 equ reprogrun	\ -- addr
\ *G The run time address of the reprogramming code.
\ ** This *\b{must} match the start address of the *\fo{CDATA}
\ ** section defined in *\i{REPROG210x.CTL}, and that section
\ ** *\b{must not} overlap the run-time stacks and user area.

0 value ReflashDev	\ -- addr
\ *G Holds 0 or a UART base address. If set to 0, the reflash
\ ** code will use the default device for the XModem transfer.
\ ** Otherwise it will use the value here as the base address
\ ** of the LPC2xxx UART to use.
\ ** N.B. No effect on the MPE USB Stamp.

useUART1? [if]
console-port 1 = [if]
  _UART1 to ReflashDev
[then]
[then]

: callit	\ xt --
\ *G Load the reprogramming code and execute the xt.
  [defined] dfi [if] dfi [then]		\ disable FIQ
  di					\ disable IRQ [and FIQ]
  ReflashDev				\ get device selection BEFORE copy
  reprog2xxx.img reprogrun /reprog cmove \ copy code to RAM
  reprogrun 2 cells + !			\ set device base address
  execute
  [defined] reboot [if]  reboot  [else]  0 execute  [then]
;

: reflash	\ -- ; no exit
\ *G Copies the reprogramming code to the run-time address
\ ** and executes it.
  reprog2xxx.img @ callit
;

: CopyFlash	\ src dest len --
\ *G Copy the flash. The parameters are as for *\fo{CMOVE}.
\ ** The process is repeated until there are no errors, and
\ ** the system is then rebooted.
  reprog2xxx.img cell + @ callit
;


\ ======
\ *> ###
\ ======

