\ Reprogram LPC2xxx over serial line.

((
Copyright (c) 2003, 2004, 2005, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20060818 SFP002 Added UART selection.
20040809 SFP001 Added equate FULLIAP?
20030728 SFP000 First cut
))

\ =============
\ *! reprog210x
\ *T Reprogramming the LPC2xxx serially
\ =============
\ *S Introduction
\ *P *\i{REPROG210x.CTL} is the control file for the serial loader
\ ** which reprograms the on-chip Flash memory.
\ ** Because the on-chip Flash cannot be accessed during
\ ** programming, and because the initial loader code may
\ ** itself be replaced, the Flash programming code is copied
\ ** into RAM for execution. The only exit from the code is a
\ ** reset of the CPU to execute the newly Flashed program. All
\ ** interrupts are disabled during reprogramming, and so
\ ** polled serial drivers are used.

\ *P Access to the reprogramming software is defined in
\ ** *\i{REFLASH.FTH} for the main system code and in
\ ** this control file and the two must match.

\ *P Return is by rebooting the system. The contents of the
\ ** internal RAM and Flash are destroyed, therefore any data
\ ** that must be preserved should be saved externally to the
\ ** chip.


only forth definitions

[undefined] LPCtype [if]
#2138 constant LPCtype	\ -- u
\ *G Define default LPC type if none defined on command line
[then]


\ ******************************
\ Define the default directories
\ ******************************

"" \buildkit.dev\software\rom\arm"	setmacro CpuDir
"" \buildkit.dev\software\rom\common"	setmacro CommonDir
"" ."					setmacro AppDir
"" .."					setmacro HwDir


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  no-heads			\ do not generate heads
  no inlining			\ saves some space
  -ram-table			\ saves some space
  +LeafCalls			\ saves some space

0         equ false
false not equ true


\ ****************
\ *S Configuration
\ ****************

LPCtype equ LPCtype	\ -- n
\ *G Define the LPC type being used. This is used to
\ ** select CPU specific data and routines.

\ Crystal, clock, serial and ticker rates
#10000000 equ xtal-speed	\ -- Hz
\ *G Crystal clock rate in HZ. For greatest baud rate flexibility
\ ** use 14.7456 MHz
#60000000 equ system-speed	\ -- Hz
\ *G System operating speed in HZ. Must be the same as, or an
\ ** even multiple of *\fo{XTAL-SPEED}. If not the same, the
\ ** initialisation code will set up the PLL as required.

0 equ FullIAP?		\ -- n
\ *G Set this equate non-zero to compile additional IAP routines.
\ ** See *\i{IAP210x.FTH} for more details.


\ **********
\ Memory map
\ **********

LPCtype #2106 = [if]
$40000200 $40001FFF cdata section Reprog210x
[then]
LPCtype #2134 = [if]
$40000200 $40001FFF cdata section Reprog2134
[then]
LPCtype #2138 = [if]
$40000200 $40001FFF cdata section Reprog213x
[then]
$40002000 $400027FF idata section PROGd
$40002800 $40002FFF udata section PROGu
$40003000 $400031FF udata section Xbuff	\ 512 byte Xmodem buffer
  sec-base equ Xbase			\ base address of buffer
  sec-len equ Xlen			\ length of buffer
  Xbase Xlen + equ Xtop			\ top of buffer


LPCtype #2106 = [if]
interpreter
: prog		Reprog210x  ;
target
[then]
LPCtype #2134 = [if]
interpreter
: prog		Reprog2134  ;
target
[then]
LPCtype #2138 = [if]
interpreter
: prog		Reprog213x  ;
target
[then]

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ***********************************
\ Configure stacks and kernel buffers
\ ***********************************
\ The stacks have already been set up by the calling program.


\ *****
\ Files
\ *****
\ ********************
\ *S Items of interest
\ ********************

include %CpuDir%\ArmDef			\ ARM CPU equates
include %CpuDir%\sfrLPC2106		\ Special function registers

l: CLD1		\ -- addr
\ *G Filled in later with the xt of *\fo{ReprogFlash}
  0 ,
l: CLD_CopyFlash	\ -- addr
\ *G Filled in later with the xt of *\fo{CopyFlash}
  0 ,
l: CLD_UART	\ -- addr
\ *G Filled in later with 0 or UART base address to use.
\ ** If set to 0, the default is UART0. This allows an
\ ** application to select which UART to use.
  0 ,

include %AppDir%\primitives		\ Forth primitives from CODEARM.FTH
include %AppDir%\MinSerLPC210xp		\ polled serial line driver
include %AppDir%\delays			\ software delays
include %CpuDir%\Drivers\rebootLPC210x	\ reboot using watchdog
synonym ser-emit emit
\ *G A synonym required by the Xmodem code.
synonym ser-key? key?
\ *G A synonym required by the Xmodem code.
synonym ser-key  key
\ *G A synonym required by the Xmodem code.
 include %AppDir%\MinXmodemRx		\ Xmodem receiver
\ include %CommonDir%\MinXmodemRx	\ Xmodem receiver
$7FFFFFF0 equ IAPentry			\ IAP entry point
include %HwDir%\IAP210x			\ IAP access
include %HwDir%\FlashTables		\ Flash sector tables
include %AppDir%\ReprogApp		\ Application


make-turnkey ReprogFlash		\ start up action
\ *G Define the action of the reprogramming code.
' CopyFlash CLD_CopyFlash !
\ *G Define the action of the Flash copy code.


\ *******
\ The end
\ *******

decimal

FINIS


\ ******
\ *> ###
\ ******
