\ Forth Stamp LPC2106/2138 standalone Forth control file

((
Copyright (c) 2004, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20070413 SFP001 Updated to select between 2106 and 2138 versions.
))


\ =========
\ *! serstamp
\ =========
\ *T Forth Stamp for Philips LPC2106 and LPC2138
\ *P The control file *\i{SerStamp.ctl} generates a standalone
\ ** Forth for the NMI LPC2106 and LPC2138 boards.

only forth definitions

[undefined] LpcCpu [if]
#2106 constant LpcCpu	\ -- n
\ *G The LPC type can be defined on the compiler command line.
\ ** If it has not been defined, the LPC2106 is assumed.
[then]


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if]
  +xrefs                        \ enable cross references
[then]
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  +LeafCalls			\ improves performance, reduces size.

  Hex-I32			\ generate 32 bit Intel Hex file

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N LPC type being used
\ ======================

LpcCpu equ LPCtype	\ -- n
\ *G Define the LPC type being used. This is used in the
\ ** CPU specific code such as the IAP access code to
\ ** select CPU specific data and routines.


\ =====================
\ *N Software selection
\ =====================

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ --------------------
\ *H Kernel components
\ --------------------
 1 equ ArmStamp?	\ -- n
\ *G nz to include ARM Stamp specifics
 0 equ USBStamp?	\ -- n
\ *G nz to include USB ARM Stamp specifics
 1 equ SerStamp?	\ -- n
\ *G nz to include Serial ARM Stamp specifics
 1 equ ColdChain?	\ -- n
\ *G nz to use cold chain mechanism
 1 equ tasking?
\ *G true if multitasker needed. Event handler and message
\ ** passing code is not compiled, but semaphores are.
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\   true to include semaphores
 1 equ timebase?
\ *G true for TIMEBASE code
 1 equ softfp?
\ *G true for software floating point
 1 equ FullCase?
\ *G true to include ?OF END-CASE NEXTCASE extensions
 1 equ target-locals?
\ *G true if target local variable sources needed
 0 equ romforth?
\ *G true for ROMForth handler
 0 equ blocks?
\ *G true if BLOCK needed
 $0000 equ sizeofheap
\ *G 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ ------------------------------------------
\ *H Crystal, clock, serial and ticker rates
\ ------------------------------------------

#10000000 equ xtal-speed
\ *G Crystal clock rate in HZ. For greatest baud rate flexibility
\ ** use 14.7456 MHz,
#60000000 equ system-speed
\ *G System operating speed in HZ. Must be the same as, or an
\ ** even multiple of XTAL-SPEED. If not the same, the
\ ** initialisation code will set up the PLL as required.

\ #14745600 equ xtal-speed
\ xtal-speed 4 * equ system-speed

  4 equ /vpb		\ -- n ; VPB clock divider    1  2  4
\ *G Required VPB bus division ratio.
  %00 equ init-vpb	\ -- n ; VPB bits           % 01 10 00
\ *G Initial value set into the VPB register. MUST match the
\ ** selection of /VPB.

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0.
1 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1.
#115200 equ console0-speed
\ #9600 equ console0-speed
\ *G UART0 speed in BPS.
#38400 equ console1-speed
\ *G UART1 speed in BPS.
0 equ console-port
\ *G Designate serial port for terminal (0/1).
#10 equ tick-ms
\ *G TIMEBASE tick in ms


\ ---------------------------------------------
\ *H Define stack and user area sizes for TASKs
\ ---------------------------------------------

$0100 equ UP-SIZE
\ *G size of each task's user area
$0100 equ SP-SIZE
\ *G size of each task's data stack
$0200 equ RP-SIZE
\ *G size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN
\ *G Terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs
\ *G Number of IRQ stacks, shared by all IRQs (1 min)
1 equ #FIQs
\ *G number of FIQ stacks, shared by all FIQs (1 min)
0 equ #SWIs
\ *G number of SWI nestings permitted. 0 is ok if SWIs are unused


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ *************
\ *S Memory map
\ *************

  $00000000 equ link-address	\ Address at which code will start\be linked at

#2106 LPCtype = [if]
  $00000000 $0001DFFF cdata section SerStamp2106
\ *G Primary code section in on-chip Flash
    sec-len equ Boot_Flash_len
  $40000200 $4000DFFF idata section PROGd
\ *G 56k IDATA less 512 for ISP loader. This area contains
\ ** initialised data and user applications. The first five cells
\ ** are reserved for use by the program storage mechanism. The next
\ ** three cells are reserved for future use.
    here equ CommitXt
      0 , 0 , 0 , 0 ,				\ len, xt, inverted xt, CRCapp
      -1 , -1 , -1 , -1 ,			\ RFU
    4 cells equ /Commit				\ size of used header
    8 cells equ /Header				\ total size of header
    /Commit equ MinAppSize			\ minimum application size
    sec-end CommitXt - equ MaxAppSize		\ maximum application size
  $4000E000 $4000FFFF udata section PROGu
\ *G 8k UDATA. This area contains uninitialised data.
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP
    #256 reserve equ ISPstack			\   256 bytes for ISP stack
    						\   IAP stack is Forth return stack, max 128 bytes
interpreter
: prog SerStamp2106  ;			\ synonym
target
[then]

#2138 LPCtype = [if]
  $00000000 $0001DFFF cdata section SerStamp2138
\ *G Primary code section in on-chip Flash
    sec-len equ Boot_Flash_len
  $40000200 $40005FFF idata section PROGd
\ *G 24k IDATA less 512 for ISP loader. This area contains
\ ** initialised data and user applications. The first five cells
\ ** are reserved for use by the program storage mechanism. The next
\ ** three cells are reserved for future use.
    here equ CommitXt
      0 , 0 , 0 , 0 ,				\ len, xt, inverted xt, CRCapp
      -1 , -1 , -1 , -1 ,			\ RFU
    4 cells equ /Commit				\ size of used header
    8 cells equ /Header				\ total size of header
    /Commit equ MinAppSize			\ minimum application size
    sec-end CommitXt - equ MaxAppSize		\ maximum application size
  $40006000 $40007FFF udata section PROGu
\ *G 8k UDATA. This area contains uninitialised data.
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP
    #256 reserve equ ISPstack			\   256 bytes for ISP stack
    						\   IAP stack is Forth return stack, max 128 bytes
interpreter
: prog SerStamp2138  ;			\ synonym
target
[then]

Stamp? 0= [if]
  $7FFFE000 $7FFFFFFF udata section BootBlock
\ *G 8k ROM boot loader
[then]
  $7FFFFFF0 equ IAPentry			\ Thumb code entry point
  $40000000 equ ISPram				\ 512 bytes for ISP loader

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrLPC2106  		\ Special function registers.
  include %HwDir%\initLPC210x		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  include %CpuDir%\Drivers\rebootLPC210x	\ reboot using watchdog
  3 equ ARM-VIC?			\ select PL190 Vectored Interrupts
  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\Drivers\serLPC210xi	\ interrupt RX serial driver
\  include %CpuDir%\Drivers\serLPC210xp	\ polled serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%\delays
  include %CpuDir%\drivers\TickLPC210x	\ timer tick
  ' start-timers AtCold
[else]
  include %CommonDir%\delays
  include %CpuDir%\drivers\TickLPC210x	\ timer tick
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

#2106 LPCtype = [if]
  buildfile SerStamp2106.no

internal
create version$
  build$,

: .banner	\ -- ; display line of asterisks
  cr ." *******************************************"
;

: .CPU          \ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Serial LPC2106 Stamp "  version$ $.
  .banner
;

external

[then]

#2138 LPCtype = [if]
  buildfile SerStamp2138.no

internal
create version$
  build$,

: .banner	\ -- ; display line of asterisks
  cr ." *******************************************"
;

: .CPU          \ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Serial LPC2138 Stamp "  version$ $.
  .banner
;

external
[then]

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ Application code
\ *******************

  include %CommonDir%\DebugTools
  include %HwDir%\IAP210x		\ IAP code
  include %AppDir%\Reprog\Reflash	\ Reflash system
  include %HwDir%\FlashTables		\ Flash sector tables
  include %HwDir%\FlashProg		\ higher level Flash routines
\ Redefinitions already done for ROMFORTH
\  Synonym Ser-Key? key?
\  Synonym Ser-Key  key
\  Synonym Ser-Emit emit
  1 equ XmodemRx?
  1 equ XmodemTx?
  include %CommonDir%\XmodemTxRx
\ I2C
  include %HwDir%\i2cLPCbb		\ bit-banged driver
  include %ExampleDir%\i2c\I2Cbase	\ common driver layer, always needed

(( \ Disaster recovery version
: Recover	\ --
  cr ." Starting recovery"
  empty
  cr ." Recovery complete"
;
' Recover AtCold
))

\ Application save/restore - MUST be last file that uses ATCOLD
  include %HWDir%\StampApps		\ application save/restore


\ **********************
\ Select example code
\ **********************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
sec-base 0				\ -- addr 0
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $18 + @ +
  over $1C + @ +
  negate
swap $14 + !


\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build				\ increment build number
FINIS


\ ======
\ *> ###
\ ======

