\ USBgpio.fth

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

\ ==========
\ *! usbgpio
\ ==========
\ *T GPIO initialisation and USB driver

\ **********************
\ *S GPIO initialisation
\ **********************
\ *P The following table describes how the GPIO pins are used
\ ** by the USB stamp board.
\ *E Pinsel  00      01      10      11      LPC pin Board
\ ** ===========================================================
\ ** 0.1:0   P0.0    TxD0    PWM1    RFU     13	     RS232 o/p
\ ** 0.3:2   P0.1    RxD0    PWM3	     14      RS232 i/p
\ ** 0.5:4   P0.2    SCL     Cap0.0	     18      U2p6
\ ** 0.7:6   P0.3    SDA     Mat0.0	     21      U2p5
\ ** 0.9:8   P0.4    WP      Cap0.1	     22      U2p7
\ ** 0.11:10 P0.5    MISO    Mat0.1	     23      usbRD#
\ ** 0.13:12 P0.6    MOSI    Cap0.2	     24      usbWR
\ ** 0.15:14 P0.7    SSEL    PWM2	     28	     usbTXE#
\ ** 0.17:16 P0.8    TxD1    PWM4	     29      usbD0
\ ** 0.19:18 P0.9    RxD1    PWM6	     30      usbD1
\ ** 0.21.20 P0.10   RTS1    Cap1.0	     35      usbD2
\ ** 0.23:22 P0.11   CTS1    Cap1.1	     36      usbD3
\ ** 0.25:24 P0.12   DSR1    Mat1.0	     37      usbD4
\ ** 0.27:26 P0.13   DTR1    Mat1.1	     41      usbD5
\ ** 0.29:28 P0.14   CD1     EINT1	     44      usbD6
\ ** 0.31:30 P0.15   RI1     EINT2	     45      usbD7
\ ** 1.1:0   P0.16   EINT0   Mat0.2	     46	     usbRXF#
\ ** 1.3:2   P0.17   Cap1.2  --	             47	     pTRST
\ ** 1.5:4   P0.18   Cap1.3  --	             48	     pTMS
\ ** 1.7:6   P0.19   Mat1.2  --	             1	     pTCK
\ ** 1.9:8   P0.20   Mat1.3  --	             2	     pTDI
\ ** 1.11.10 P0.21   PWM5		     3	     pTDO
\ ** 1.13:12 P0.22   --		             32      CPLD cTMS o/p
\ ** 1.15:14 P0.23   --		             33      CPLD cTCK o/p
\ ** 1.17:16 P0.24   --		             34      CPLD cRTCK i/p
\ ** 1.19:18 P0.25   --		             38      CPLD cTDI o/p
\ ** 1.21:20 P0.26   --		             39      CPLD cNTRST o/p
\ ** 1.23:22 P0.27   TRST		     8       CPLD cTDO i/p
\ ** 1.25:24 P0.28   TMS		     9       CPLD cNSRST o/p
\ ** 1.27:26 P0.29   TCK		     10      CPLD cX
\ ** 1.29:28 P0.30   TDI		     15      CPLD cM0
\ ** 1.31:30 P0.31   TDO		     16      CPLD cM1

\ *P On versions of the MPE USB Stamp board for which the UART1
\ ** pins conflict with the USB chip pins, UART1 is not enabled.
\ ** This is controlled by the equate USEUART1? in the control file.

\ I2C interface equates
bit2 equ i2cSCL
bit3 equ i2cSDA
bit4 equ i2cWP

i2cSCL  i2cWP or equ i2cHiMask
\ *G Mask for I2C bits set high at initialisation
0 equ i2cLoMask
\ *G Mask for I2C bits set low at initialisation

\ USB interface equates
bit5  equ usbRD#
bit6  equ usbWR
bit7  equ usbTXE#
bit16 equ usbRXF#
$00000FF00 equ usbDbus
8 equ usbDbShift

usbRD# equ UsbHiMask	\ -- mask
\ *G Mask for USB bits set high at initialisation
usbWR equ UsbLoMask	\ -- mask
\ *G Mask for USB bits set low at initialisation

\ JTAG interface equates
bit22 equ cTMSo		#22 equ #cTMSo
bit23 equ cTCKo		#23 equ #cTCKo
bit24 equ cRTCKi	#24 equ #cRTCKi
bit25 equ cTDIo		#25 equ #cTDIo
bit26 equ cNTRSTo	#26 equ #cNTRSTo
bit27 equ cTDOi		#27 equ #cTDOi
bit28 equ cNRSTo	#28 equ #cNRSTo
bit29 equ cNSRSTo	#29 equ #cNSRSTo
bit30 equ cDBGrqo	#30 equ #cDBGrqo
bit31 equ cDBGacki	#31 equ #cDBGacki

cTMSo  CTCKo or  cTDIo or
cNTRSTo or  cNRSTo or
  equ JtagHiMask	\ -- mask
\ *G Mask for JTAG bits set high at initialisation
0 equ JtagLoMask	\ -- mask
\ *G Mask for JTAG bits set low at initialisation

\ GPIO Output direction masks
0
i2cSCL or  i2cWP or
usbRD# or  usbWR or
cTMSo or  CTCKo or  cTDIo or
cNTRSTo or  cNRSTo or
\ cX or cM0 or  cM1 or
  equ GpioDirMask
\ *G Mask with bits set for output. 1=o/p, 0=i/p.

I2cHiMask UsbHiMask or JtagHiMask or 
  equ GpioHiMask	\ -- mask
\ *G Mask for GPIO bits set high at initialisation
I2cLoMask UsbLoMask or JtagLoMask or
  equ GpioLoMask	\ -- mask
\ *G Mask for GPIO bits set high at initialisation

create ^gpio	\ -- addr
\ *G Contains the base address of the GPIO registers.
  _gpio ,

: init-gpio	\ --
\ *G Initialise the GPIO bits except for the serial lines
\ ** which are handled by the startup code.
  ^gpio @
  GpioDirMask over IODIR + !		\ set to o/p
  GpioLoMask  over IOCLR + !		\ set to 0
  GpioHiMask  over IOSET + !		\ set to 1
  drop
;
' init-gpio AtCold


\ *******************
\ *S USB comms driver
\ *******************
\ *P The USB comms driver follows the generic I/O model used
\ ** by the serial ports.

IsLeaf : usbEmit	\ char --
\ *G Write a byte to the USB connection.
  ^gpio @
  begin					\ poll until ready
    dup IOPIN + @ usbTXE# and 0=
  until
  usbDbus over IODIR + or!		\ set data bus to out
  usbWR over IOSET + !			\ WR high
  swap usbDbShift lshift	 	\ -- _gpio data ; align data
  2dup usbDbus and swap IOSET + !	\ -- _gpio data ; set '1' bits
  invert usbDbus and over IOCLR + !	\ -- _gpio ;      set '0' bits
  [asm  mov r0, r0  asm]		\ 16 ns
  usbWR swap IOCLR + !			\ WR low
;

: usbKey?	\ -- flag
\ *G Return non-zero if a character is available to be read from
\ ** the USB connection.
  ^gpio @ IOPIN + @ usbRXF# and 0=
;

tasking? [if]
: usbKey	\ -- char
\ *G Read (wait) for a character from the USB connection.
  ^gpio @
  begin					\ poll until ready
    dup IOPIN + @ usbRXF# and
   while
    pause
  repeat
  usbDbus over IODIR + bic!		\ set data bus to in
  usbRD# over IOCLR + !			\ RD# low
  [asm  mov  r0, r0			\ ~48ns
        mov  r0, r0
        mov  r0, r0
  asm]
  dup IOPIN + @				\ get data
  usbRD# rot IOSET + !			\ RD high
  usbDBshift rshift $FF and		\ extract byte
;
[else]
IsLeaf : usbKey	\ -- char
  ^gpio @
  begin					\ poll until ready
    dup IOPIN + @ usbRXF# and 0=
  until
  usbDbus over IODIR + bic!		\ set data bus to in
  usbRD# over IOCLR + !			\ RD# low
  [asm  mov  r0, r0			\ ~48ns
        mov  r0, r0
        mov  r0, r0
  asm]
  dup IOPIN + @				\ get data
  usbRD# rot IOSET + !			\ RD high
  usbDBshift rshift $FF and		\ extract byte
;
[then]

: usbCR		\ --
\ *G Send a CR/LF pair on the USB connection.
  $0D usbEmit  $0A usbEmit
;

: usbTYPE	\ c-addr len --
\ *G Type a string to the USB connection.
  bounds
  ?do  i c@ usbEmit  loop
;

[undefined] useUSB? [if]
1 equ useUSB?
[then]

cdata
useUSB? [if]
create UsbConsole	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for USB.
  ' usbKey ,		\ -- char ; receive char
  ' usbKey? ,		\ -- flag ; check receive char
  ' usbEmit ,		\ -- char ; display char
  ' usbType ,		\ caddr len -- ; display string
  ' usbCR ,		\ -- ; display new line

console-port 2 = [if]
  UsbConsole constant Console
\ *G *\fo{CONSOLE} is the device used by the Forth system for
\ ** interaction. It may be changed by a phrase of the form:
\ *C   <device>  dup opvec !  ipvec !
[then]

[then]


\ =======
\ *> ###
\ =======

decimal
