\ CDS2388.ctl - LPC2388 standalone Forth control file - CDS 2388 board

((
Copyright (c) 2008, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20101119 MPE002 Updated for board revisions
20081006 MPE001 First release.
))


\ =========
\ *! cds2388
\ =========
\ *T Standalone Forth for LPC2388 User Flash
\ *P The control file *\i{CDS2388.ctl} generates a standalone
\ ** Forth for Philips LPC2388 systems using a 16 MHz crystal.
\ ** SD card drivers
\ ** are used with the *\i{FATfiler} FAT file system. Note
\ ** that *\i{FATfiler} is provided with the Professional
\ ** editions of the cross compiler from v6.4 onwards and
\ ** Developer editions from v7.0 onwards.

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives
mc" %ExampleDir%/FATfiler"
		setmacro FatFiler	\ where FAT file system code lives
mc" %ExampleDir%/USB"
                setmacro UsbDir		\ where USB code lives
"" C:\buildkit.dev\software\AddOns\PowerNet\Dev"
		setmacro IpStack	\ where PowerNet lives



\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  Hex-I32			\ Generate 32 bit Intel Hex

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

\ Define CPU type in use
 #2388 equ LPCtype		\ -- cpu# ; selects Flash configuration
 2 equ HwRev#			\ -- rev# ; CDS board hardware revision

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 1 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $8000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ Crystal, clock, serial and ticker rates
#16000000 equ xtal-speed	\ -- hz
\ *G Crystal clock rate in HZ.
#72000000 equ system-speed	\ -- hz
\ *G Desired system clock rate in HZ. After the PLL calculations
\ ** have been performed in *\i{InitLPC23xx.fth}, the value here
\ ** is reset to the actual speed to improve the accuracy of baud
\ ** rate calculations.
#48000000 equ usb-speed		\ USB clock rate in HZ, set to
				\ zero if USB is unused.

0 equ console-port		\ Designate serial port for terminal.
\ *G Ports 0..3 are the on-chip UARTs. The internal/external USB devices
\ ** are ports 10, 11, 12, and bit-banged ports are defined from 20 onwards.
1 equ useUART0?	\ -- n
\ *G Compile UART0 code if non-zero.
  #115200 equ console0-speed	\ Serial port speed in BPS.
1 equ useUART1?	\ -- n
\ *G Compile UART1 code if non-zero.
  #115200 equ console1-speed	\ Serial port speed in BPS.
1 equ useUART2?	\ -- n
\ *G Compile UART2 code if non-zero.
  #31250 equ console2-speed	\ MIDI port speed in BPS.
1 equ useUART3?	\ -- n
\ *G Compile UART3 code if non-zero.
  #250000 equ console3-speed	\ DMX port speed in BPS.

#1 equ tick-ms			\ TIMEBASE tick in ms, 1 is good for PowerNet


\ define stack and user area sizes for TASKs
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused


\ **********
\ Memory map
\ **********

  $00000000 $0007CFFF cdata section CDS2388	\ code section
    sec-len equ Boot_Flash_len
    $7FFFFFF0 equ IAPentry			\   Thumb code entry point
  $40000000 equ ISPram				\ 512 bytes for ISP loader
  $40000200 $40003FFF idata section PROGd	\ 16k IDATA less 512 for ISP loader
  $40004000 $4000FFFF udata section PROGu	\ 32k UDATA
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP
  $0007:D000 $0007:DFFF udata section UnitCfg	\ Unit configuration flash
  $07FE0:0000 $07FE0:3FFF udata section ERAM	\ 16k Ethernet RAM
  $07FD0:0000 $07FD0:1FFF udata section URAMP	\ 8k primary USB RAM - for USB use
  $07FD0:2000 $07FD0:3FFF udata section URAME	\ 8k extended USB RAM - used for sectors

interpreter
: prog	CDS2388 ;				\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrLPC23xx	  	\ Special function registers.
  include %HwDir%\initLPC23xx		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  include %CpuDir%\Drivers\rebootLPC210x	\ reboot using watchdog
  5 equ ARM-VIC?			\ select PL192 Vectored Interrupts
\  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CpuDir%\IntArm4		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\drivers\serLPC23xxqi	\ queued interrupt driven serial
\  include %CpuDir%\drivers\serLPC23xxp	\ polled serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%\delays		\ delay common code
  include %CpuDir%\drivers\TickLPC23xx	\ timer tick
  ' Start-Timers AtCold
[else]
  include %CommonDir%\delays		\ delay common code
  include %CpuDir%\drivers\TickLPC23xx	\ timer tick
  ' Start-Clock AtCold
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
\  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
\  include %CommonDir%\RomForth\filetran	\ ASCII file uploader
\  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
\  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile CDS2388.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ******************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for LPC23xx"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  include %HwDir%\IAP23xx		\ IAP code
  include %HwDir%\Reprog\Reflash	\ Reflash system
  include %HwDir%\FlashTables		\ Flash sector tables
  include %HwDir%\FlashProg		\ higher level Flash routines

  include %CpuDir%\drivers\RTC23xx	\ real time clock
  include %CpuDir%\drivers\i2cLPC23xx	\ I2C on channel 0.

  include %CpuDir%\Drivers\dmaLPC23xx	\ DMA driver
HwRev# 2 = [if]
  include %FatFiler%\Drivers\hwMciCDS0813	\ CDS 2388 hardware specifics
[then]
HwRev# 1 = [if]
  include %FatFiler%\Drivers\hwMciCDS0807	\ CDS 2388 hardware specifics
[then]
  include %HwDir%\mci23xx.bld		\ FAT file system code

((
create pagedir$  ", \HTTP"	\ -- caddr
\ *G The base directory for pages as a counted string.
\ ** The directory name must not end in a separator.
\ ** This is a FAT path name.
  align
create HomePage$  ", /home.asp"	\ -- caddr
\ *G Holds the counted string for the home page.
\ ** This string is part of a URL.
  align

: SVvalid?	\ addr|0 -- flag
\ *G Check that addr is a valid service data address.
  dup if
    STARTOFHEAP ENDOFHEAP within
  endif
;

1 equ XC?
1 equ CPU=ARM
0 equ Phy#	\ -- n
\ *G Single PHY: The number of the PHY in use: 0..31
  include %AppDir%\CDSPNconfig		\ PowerNet configuration file
  include %IpStack%\NetCode		\ network order stuff
  include %AppDir%\WebConfig		\ configuration tools
  include %CpuDir%\Drivers\EtherLPC23xx	\ Ethernet driver
  include %IPstack%\PowerNet.bld	\ PowerNet build file
  include %IPstack%\Examples\WebPost	\ File upload
))

[undefined] ?StackEmpty [if]
  include %CommonDir%\DebugTools
[then]

\ USB - CDC or MSC only
  include %UsbDir%\UsbDefs		\ USB definitions/equates
\  include %UsbDir%\MSCconfig		\ USB configuration file
\  include %UsbDir%\CDCconfig		\ USB configuration file
  include %UsbDir%\CdcMscConfig		\ USB configuration file
  include %UsbDir%\otgISP1301		\ USB OTG set up
  include %UsbDir%\UsbHwLPC2xxx		\ USB DMA hardware layer for LPC23xx
  include %UsbDir%\UsbBase		\ USB base operations
[sections  URAME CDATA		\ buffers in USB upper RAM
MSC? [if]
  include %UsbDir%\MscDrv		\ Mass Storage Class
[then]
CDC? [if]
  include %UsbDir%\VCOMdev		\ UART device emulation
  include %UsbDir%\CdcDrv		\ Communications Device Class
[then]
sections]
  include %UsbDir%\UsbEP0		\ USB control and initialisation

\  include CDStests			\ Code to test CDS web pages

: StartApp	\ --
  initFATfs .
[defined] PowerNet [if]  PowerNet  [then]
  StartUSBsys
;


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\tests\ANS\tester	\ John Hayes' ANS test engine
\ include %CommonDir%\tests\MPE\7-1-a	\ MPE test engine
\ include %CommonDir%\tests\MPE\7-1-b	\ MPE tests
\ include %CpuDir%\TestDiv		\ division tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\tests\MPE\testtask
\ include %ExampleDir%\tests\MPE\testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
sec-base 0				\ -- addr 0
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $18 + @ +
  over $1C + @ +
  negate
swap $14 + !

decimal
\ cr ." PC = " here dup .dword ."  " .

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======

