\ DispSAA1064 - I2C display driver

((
Copyright (c) 2008
CDS Advanced Technology bv

Copyright (c) 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! dispsaa1064
\ *T SAA1064 Display Driver
\ ===========
\ *P The tests were performed using an SAA1064 display controller
\ ** at I2C address $70.


\ **********************************
\ *S Character to segment conversion
\ **********************************

create SegmentTable	\ -- addr
\ *G Table used to convert chars/digits to 7 segment info.
\ The hex/decimal mix is correct!
  $3F C, $06 C, $5B C, $4F C, $66 C, $6D C, $7D C, $07 C,  \ 0..7 data
  $7F C, $6F C, $7B C, $77 C, $7C C, $39 C, $5E C, $79 C,  \ 8..F data
  098 C, 098 C, 098 C, 098 C, 098 C, 098 C, 098 C, 098 C,  \ 10..17 ascii
  098 C, 098 C, 098 C, 098 C, 098 C, 098 C, 098 C, 098 C,  \ 18..1F ascii
  000 C, 098 C, 034 C, $09 C, 098 C, 098 C, 098 C, 032 C,  \ 20..27 ascii
  057 C, 015 C, 098 C, 098 C, 098 C, 064 C, 098 C, 098 C,  \ 28..2F ascii
  063 C, 006 C, 091 C, 079 C, 102 C, 109 C, 125 C, 007 C,  \ 30..37 ascii
  127 C, 111 C, 098 C, 098 C, 098 C, 072 C, 098 C, 083 C,  \ 38..3F ascii
  123 C, 119 C, 124 C, 057 C, 094 C, 121 C, 113 C, 061 C,  \ 40..47 ascii
  118 C, 048 C, 030 C, 112 C, 056 C, 055 C, 084 C, 063 C,  \ 48..4F ascii
  115 C, 103 C, 080 C, 109 C, 120 C, 062 C, $3E C, $3E C,  \ 50..57 ascii
  098 C, $6E C, 098 C, 057 C, 098 C, 015 C, 001 C, 008 C,  \ 58..5F ascii
  002 C, 119 C, 124 C, 088 C, 094 C, 121 C, 113 C, 111 C,  \ 60..67 ascii
  116 C, 016 C, 012 C, 112 C, 056 C, 084 C, 084 C, 092 C,  \ 68..6F ascii
  115 C, 103 C, 080 C, 109 C, 120 C, 028 C, $1C C, $1C C,  \ 70..77 ascii
  098 C, $6E C, 098 C, 098 C, 006 C, 098 C, 098 C, 098 C,  \ 78..7F ascii

: Char>Seg	\ char -- segments
\ *G Convert an ASCII char/digit to the corresponding 8 bit
\ ** segment data.
  $7F and SegmentTable + c@  ;		\ limit range and do table lookup


\ ******************
\ *S Display handler
\ ******************

$70 equ addrSAA	\ -- u
\ *G I2C address of the SAA1064 in 8 bit format, bit 0=R/W.

: .i2cstat	\ --
\ *G Display I2C status byte.
  _I2Cx I2STAT + @ .byte  ;

: start_disp	\ reg --
\ *G Start an I2C transaction to the display at the given register.
  +ack_i2c				\ all bytes acknowledged
  start_i2c  addrSAA write_byte_i2c	\ SAA1064 address +w
  $07 and write_byte_i2c		\ first reg
;

: $>disp	\ caddr len --
\ *G Send a character string to the display.
  $00 start_disp  $37 write_byte_i2c	\ consumes reg
  0 max 4 min  bounds
  ?do  i c@ Char>Seg write_byte_i2c  loop
  stop_i2c
;

: >disp		\ nn..n1 n reg --
\ *G Send *\i{n} bytes starting at register *\i{reg}.
  start_disp				\ consumes reg
  0 ?do  write_byte_i2c  loop		\ n bytes
  stop_i2c
;

: DispStat	\ -- b
\ *G Read the display status, $80 if enabled.
  +ack_i2c				\ all bytes acknowledged
  start_i2c  $71 write_byte_i2c		\ SAA1064 address +r
  read_byte_i2c  stop_i2c
;

: .Boot		\ --
  s" boot" $>disp  ;

: BootSign	\ --
\ *G Initialise the display and display a boot message at start up.
  init_i2cx .Boot  ;
' BootSign AtCold

COMPILER
: MSG"		\ "<text>" --
  S" $>disp  ;
TARGET

: MSG"		\ "<text>" --
\ *G Display fixed string on LED display
  POSTPONE S" STATE @ IF
    POSTPONE $>disp
  ELSE
    $>disp
  THEN
; IMMEDIATE


\ ======
\ *> ###
\ ======

decimal
