\ InitLP23xx.fth - Philips LPC23xx generic startup code

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ ***************************************
\ Generic start up table and entry points
\ ***************************************

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr 	pc, ResetV              \ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  dd    0                       \ 14 - checksum
  ldr 	pc, [ pc, # $-120 ]	\ 18 - IRQ interrupts from VicVectAddr @ $FFFF:FF00
  ldr 	pc, FIQV		\ 1C - FIQ interrupts

L: Soft_Vectors	\ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code

Vector_End Vector_Start - equ Vector_Len


\ ************
\ Startup code
\ ************

\ Required Forth pointers
L: CLD1		                        \ cold start word
  0 ,				        \ compiler will fill up with address of word to execute at cold start
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,

\ Required peripheral pointers
l: ^SYSCON				\ system control block
  _SYSCON ,

\ Find the even clock divider that generates the CCO clock at
\ 288 Mhz. We look for 144 MHz and multiply by two. The value in
\ the register is one less than this.
#144000000 system-speed / 2* 1- equ CCLKCFGval
CCLKCFGval 1+ system-speed * equ cco-speed

interpreter

: genPLL	\ -- mval nval
\ For the moment, set N=1 (nval=0), and calculate the Mval
\ only.
  system-speed xtal-speed u< if
    cr ." system speed less than xtal speed" abort
  endif
  cco-speed xtal-speed 2* / 1-		\ Mval-1
  0					\ Nval-1
;

: gen-mam	\ -- mamtin
  system-speed 1- 20000000 / 1+
\ 4   \ fixed 60-80mhz
;

target

system-speed xtal-speed <> [if]
  genPLL equ Nval  equ Mval
  Nval #16 lshift  Mval or  equ PLLCFGval
\ recalculate the system-speed
  xtal-speed Nval 1+ /			\ PLL input clock
  Mval 1+ 2* *				\ PLL output speed
    equ cco-real
  cco-real CCLKCFGval 1+ /
    equ system-speed
  usb-speed [if]
    cco-real usb-speed / 1- equ USBCLKval
  [else]
    $0F equ USBCLKval
  [then]
[then]
  gen-mam equ MAMval

Proc InitReset	\ power up entry point through vector
\ Get address of system control block
  ldr		r0, ^SYSCON

\ Switch to "User FLASH mode" so interrupt vectors are not remapped
  mov		r1, # 1
  str		r1, [ r0, # MEMMAP ]

\ Set up the PLL and MAM
\ Turn off the MAM
  mov	        r1, # 0
  str		r1, [ r0, # MAMCR ]
\ Set the PLL
system-speed xtal-speed <> [if]

\ turn on xosc and test for ready
  ldr		r3, [ r0, # SCS ]
  orr		r3, r3, # bit5
  str    	r3, [ r0, # SCS ]

l: waitxosc
  ldr		r3, [ r0, # SCS ]
  and .s	r3, r3, # bit6		\ test xtal status
  b .eq		waitxosc

  mov		r3, # 1			\ select xtal osc
  str		r3, [ r0, # CLKSRCSEL ]	\ CLKSRCSEL

  mov		r4, # $AA		\ initialise feed sequence
  mov		r5, # $55
  mov		r3, # PLLCFGval		\ set required configuration
  str		r3, [ r0, # PLLCFG ]
  str		r4, [ r0, # PLLFEED ]
  str		r5, [ r0, # PLLFEED ]
  mov		r3, # 1			\ PLLE=1, PLLC=0
  str		r3, [ r0, # PLLCON ]
  str		r4, [ r0, # PLLFEED ]
  str		r5, [ r0, # PLLFEED ]
l: waitlock
  ldr		r3, [ r0, # PLLSTAT ]
  and .s	r3, r3, # bit26		\ test LOCK status
  b .eq		waitlock

l: mnlock
  ldr		r3, [ r0, # PLLSTAT ]
  mvl		r2, # $FF7FFF
  and		r3, r3, r2		\ test mn status
  mov		r2, # PLLCFGval
  eor .s	r3, r3, r2
  b .ne	mnlock

\ setup cpu clk
  mov		r3, # CCLKCFGval	\ set cclock divisor
  str		r3, [ r0, # CCLKCFG ]	\ CCLKCFG

\ setup usb clk
  mov		r3, # USBCLKval		\ set usb clock divisor
  str		r3, [ r0, # USBCLKCFG ]	\ USBCLKCFG

\ setup basic peripheral clocks
  mov		r3, # $0140		\ set uart0&1 cclock div to 1 div1
  str		r3, [ r0, # PCLKSEL0 ]	\ PCLKSEL0
  mov		r3, # $050000		\ set uart 2&3 cclock div to 1 div1
  str		r3, [ r0, # PCLKSEL1 ]	\ PCLKSEL1

  mov		r3, # 3			\ PLLE=1, PLLC=1
  str		r3, [ r0, # PLLCON ]
  str		r4, [ r0, # PLLFEED ]	\ feed sequence
  str		r5, [ r0, # PLLFEED ]

\ Re-enable the MAM
  mov		r1, # MAMval		\ n CCLK cycles
  str		r1, [ r0, # MAMTIM ]
  mov		r1, # 2			\ MAM fully enabled
  str		r1, [ r0, # MAMCR ]

\ Initialise the stacks for INTARM3.FTH
  bl		InitStacks		\ see ARM\INITSTACKS.FTH


\ *****************************
\ Optional code copy
\ *****************************

\ If required, copy code into RAM for execution
((
  mov		r0, # Flash_start	\ source
  mov		r1, # RAM_start		\ dest
  mov		r2, # Flash_len		\ len, must be multiple of 4
l: Flash_loop
  ldr		r3, [ r0 ], # 4
  str		r3, [ r1 ], # 4
  sub .s	r2, r2, # 4
  b .ne		flash_loop
))


\ *********************
\ Start of Forth system
\ *********************

L: ECLD
\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

InitReset prog sec-base -  ResetV !	\ INITRESET is in LOW memory starting at 0.


\ ******************************************
\ Reset values for user and system variables
\ ******************************************

[undefined] sp-guard [if]
  0 equ sp-guard
[then]

\ initial values of user variables for standalone systems only
[undefined] umbilical? [if]
L: USER-RESET
  init-s0   tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,                 		\ r0
  0 ,  0 ,                              \ #tib, span
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ *********************
\ Low level serial code
\ *********************

(( \ Is the UART working?
  bl init-uart0
l: initreseta
  mov           r0, # char A
  bl emit0
  b initreseta
))

((
L: ^UART0
  _UART0 ,
L: ^PCONP
  _PCONP ,
L: ^PINSEL0
  _PINSEL0 ,

proc init-uart0	\ --
\ *G Initialise serial port 0
\ enable peripheral clock
\  ldr    r0, [ r1 ]
\  orr    r0, r0, # $50
\  str    r0, [ r1 ]
\ enable alternate functions on P0:1.0
  ldr    r1, ^PCONP
  ldr    r1, ^PINSEL0
  ldr    r0, [ r1 ]
  bic    r0, r0, # $F0
  orr    r0, r0, # $50
  str    r0, [ r1 ]
\ initialise the UART
  ldr    r1, ^UART0
  mov    r0, # 1
  str    r0, [ r1, # UnFCR ]		\ essential
  mov    r0, # $083
  str    r0, [ r1, # UnLCR ]		\ 8 bit, 1 stop bit, no parity, enable DLAB
  mov    r0, # 0
  str    r0, [ r1, # UnDLM ]
  mov    r0, # 104
  str    r0, [ r1, # UnDLL ]
  mov    r0, # $003
  str    r0, [ r1, # UnLCR ]
  mov    pc, link
end-code

proc emit0	\ char in R0
  ldr    r1, ^UART0
l: emit0a
  ldr    r2, [ r1, # UnLSR ]
  and .s r2, r2, # $020
  b .eq  emit0a
  str    r0, [ r1, # UnTHR ]
  mov    pc, link
end-code
))


\ ======
\ *> ###
\ ======

decimal

