\ emc-E2468.fth - Olimex LPC-E2468 external memory controller setup

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! emce2468
\ *T LPC-E2468 External Memory
\ ===========
\ *P Much of this code is derived from the function *\b{ConfigureEMC()}
\ ** in the Olimex start-up code in the file
\ ** *\i{u-boot-oli2468\u-boot-1.3.1-olimex\board\lpc_e2468\lowlevel_init.c}.

\ *P The LPC-E2468 pin usage is as follows:
\ *E SDRAM
\ ** P2.16     CASn
\ ** P2.17     RASn
\ ** P2.18     SDclk
\ ** P2.20     DyCS0/SDCS
\ ** P2.24     SDclkEn
\ ** P2.28     DQMN0
\ ** P2.29     DQMN1
\ ** P3.0..15  Dn
\ ** P4.0..14  An
\ **
\ ** Flash
\ ** D0..7 as above
\ ** P4.18     CLE
\ ** P4.19     ALE
\ ** P4.24     /OE
\ ** P4.25     /WE
\ ** P4.31     /CS1

1 [ps
  16 +ps  17 +ps  18 +ps  20 +ps  24 +ps  28 +ps  29 +ps
ps]  equ emcPS5val  equ emcPS5mask
\ *G EMC configuration for PINSEL5
2 [ps
  16 +ps  17 +ps  18 +ps  20 +ps  24 +ps  28 +ps  29 +ps
ps]  equ emcPM5val  equ emcPM5mask
\ *G No pull up/down on EMC pins
$5555:5555 equ initPS6
\ *G EMC configuration for PINSEL6, D0..15
$AAAA:AAAA equ initPM6
\ *G No pull up/down on D0..15
$1555:5555 equ emcPS8val  $3FFF:FFFF equ emcPS8mask
\ *G EMC configuration for PINSEL8, A0..14
$2AAA:AAAA equ emcPM8val  $3FFF:FFFF equ emcPM8mask
\ *G No pull up/down on A0..14.
1 [ps  18 +ps  19 +ps  24 +ps  25 +ps  31 +ps  ps]
  equ emcPS9val  equ emcPS9mask
\ *G EMC configuration for PINSEL9, CLE, ALE, /OE, /WE and /CS1.
2 [ps  18 +ps  19 +ps  24 +ps  25 +ps  31 +ps  ps]
  equ emcPM9val  equ emcPM9mask
\ *G No pull up/down for CLE, ALE, /OE, /WE and /CS1.

$0303 equ initDynRasCas0
0 #14 lshift			\ 16 bit data
0 #12 lshift or			\ high performance
2   9 lshift or			\ 8M*16, 4 banks, row=12, col=9
1   7 lshift or
  equ initDynCfg0

\ Refresh period is 64 ms / 2 ^ rows.
\ Refresh timer is set at CCLK/16.
system-speed #64 #1000 */ 4096 / 16  / equ initDynRfrsh
\ *G Refresh counter for 64 ms period, 12 bit row, 16 clocks per unit.

DMB0base constant SDRAMbase	\ -- addr
\ *G Base address of the SDRAM block.

: initEMC	\ --
\ *G Initialise the EMC interface for SDRAM and Flash.
\ ** Performed at power up.
  \ power up and pin config
  ~pcEMC _PCONP or!			\ enable EMC interface
  emcPS5val emcPS5mask _PINSEL5 setMask
  emcPM5val emcPM5mask _PINMODE5 setMask
  initPS6 _PINSEL6 !
  initPM6 _PINMODE6 !
  emcPS8val emcPS8mask _PINSEL8 setMask
  emcPM8val emcPM8mask _PINMODE8 setMask
  emcPS9val emcPS9mask _PINSEL9 setMask
  emcPM9val emcPM9mask _PINMODE9 setMask

  \ EMC Dynamic SDRAM setup
  _EMC					\ @72MHz, 1 CCLK ~ 13ns
  $01 over emcControl + !		\ enabled
  $0000 over emcConfig + !		\ little-endian, CLKOUT=CCLK
  $01 over emcDynRP + !			\ 1+1, >20ns = 2 clk
  $02 over emcDynRAS + !		\ 2+1, >45ns = 3 clk
  $03 over emcDynSREX + !		\ 3+1, >80-100ns = 5 clk
  $01 over emcDynAPR + !
  $05 over emcDynDAL + !		\ 5+1, >75ns
  $03 over emcDynWr + !			\ 3+1
  $03 over emcDynRC + !			\ 3+1
  $03 over emcDynRFC + !		\ 3+1
  $03 over emcDynXSR + !		\ 3+1
  $00 over emcDynRRD + !		\ 0+1
  $03 over emcDynMRD + !		\ 3+1
  $01 over emcDynRdCfg + !

  initDynRasCas0 over emcDynRasCas0 + !
  initDynCfg0 over emcDynCfg0 + !
  #100 ms
  3 7 lshift  3 or  over emcDynCtrl + !	\ NOP cmd + clocks
  #200 ms
  2 7 lshift  3 or  over emcDynCtrl + !	\ PRECHARGE cmd + clocks
  1 over emcDynRfrsh + !		\ min refresh period
  #128 0 do loop			\ wait at least 128 bus clocks - 512?
  initDynRfrsh over emcDynRfrsh + !	\ final refresh period
  1 7 lshift  3 or  over emcDynCtrl + !	\ MODE cmd + clocks
  DMB0base $33 #12 lshift or @ drop	\ set mode by reading
  0 over emcDynCtrl + !			\ NORMAL cmd
  bit19 over emcDynCfg0 + or!		\ enable buffers
  tick-ms ms				\ settle

  \ Static memory set up
  $02 over emcStaWaitWen0 + !
  $02 over emcStaWaitOen0 + !
  $1F over emcStaWaitRd0 + !
  $1F over emcStaWaitPage0 + !
  $1F over emcStaWaitWr0 + !
  $0F over emcStaWaitTurn0 + !
  $81 over emcStaCfg0 + !

  $02 over emcStaWaitWen1 + !
  $02 over emcStaWaitOen1 + !
  $08 over emcStaWaitRd1 + !
  $1F over emcStaWaitPage1 + !
  $08 over emcStaWaitWr1 + !
  $0F over emcStaWaitTurn1 + !
  $80 over emcStaCfg1 + !

  drop					\ discard base address
;
' initEMC AtCold


\ Original C code

((
void ConfigureEMC(void)
{
	volatile unsigned int i, dummy = dummy;

  PCONP   |= 0x00000800;		/* Turn on EMC PCLK */

//  SCS     &= ~0x00000002;
//  SCS     |= 0x00000002;

  PINSEL5  = 0x05050555;
  PINMODE5 = 0xAA02022A;
  PINSEL6  = 0x55555555;
  PINMODE6 = 0xAAAAAAAA;
  PINSEL8  = 0x15555555;
  PINMODE8 = 0x2AAAAAAA;
  PINSEL9  = 0x40050050;
  PINMODE9 = 0x800A00A0;

  EMC_CTRL = 0x00000001;
  EMC_CONFIG = 0;

  EMC_DYN_RP     = 1;   //>20ns = 2 clk
  EMC_DYN_RAS    = 2;   //>45ns = 3 clk
  EMC_DYN_SREX   = 3;   //>80-100ns = 5 clk
  EMC_DYN_APR    = 1;   //
  EMC_DYN_DAL    = 5;   //2 clk
  EMC_DYN_WR     = 3;   //2 clk
  EMC_DYN_RC     = 3;   //>65ns = 4 clk
  EMC_DYN_RFC    = 3;   //>80-100ns = 6 clk
  EMC_DYN_XSR    = 3;   //>80-100ns = 6 clk
  EMC_DYN_RRD    = 0;   //>15ns = 1-2 clk
  EMC_DYN_MRD    = 3;   //2 clk
  EMC_DYN_RD_CFG = 1;   //or 1,2,3

  //
  EMC_DYN_RASCAS0 = 0x00000303;

  //
  EMC_DYN_CFG0 = (0u << 14)
	  		| (0u << 12)
			| (2u << 9)
			| (1u << 7);

  //wait 100mS
  delayMs(100);

  //Send command: NOP
  EMC_DYN_CTRL = 3 | (3<<7);

  //wait 200mS
  delayMs(200);

  //Send command: PRECHARGE-ALL, shortest possible refresh period
  EMC_DYN_CTRL = 3 | (2<<7);
  EMC_DYN_RFSH = 0x00000001;

  //wait at least 128 ABH clock cycles
  for(i=0; i<0x40; i++)
    asm volatile(" nop");

  //Set correct refresh period
  //EMC_DYN_RFSH = 28;		//--ivan
  EMC_DYN_RFSH = 7813/16;	//--stanimir

  //Send command: MODE
  EMC_DYN_CTRL = 3 | (1<<7);

  //Set mode register in SDRAM
  dummy = *((volatile unsigned int*)(SDRAM_BASE_ADDR | (0x33 << 12) ) );

  //Send command: NORMAL
  EMC_DYN_CTRL = 0x00000000;

  //Enable buffer
  EMC_DYN_CFG0 |= (1u<<19);

  //initial system delay
  delayMs(1);

  EMC_STA_WAITWEN0  = 0x2;
  EMC_STA_WAITOEN0  = 0x2;
  EMC_STA_WAITRD0   = 0x1f;
  EMC_STA_WAITPAGE0 = 0x1f;
  EMC_STA_WAITWR0   = 0x1f;
  EMC_STA_WAITTURN0 = 0xf;
  EMC_STA_CFG0      = 0x00000081;

  EMC_STA_WAITWEN1  = 0x2;
  EMC_STA_WAITOEN1  = 0x2;
  EMC_STA_WAITRD1   = 0x8;
  EMC_STA_WAITPAGE1 = 0x1f;
  EMC_STA_WAITWR1   = 0x8;
  EMC_STA_WAITTURN1 = 0xf;
  EMC_STA_CFG1      = 0x00000080;

}
))


\ ======
\ *> ###
\ ======

decimal

