\ S3C4510 initialisation on KEB50100 board

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


Position Independent Code
-------------------------
Although this code is compiled at the the load address
of the Forth system in DRAM, this code is executed out
of Flash at base address 0000:0000. Consequently the code
in this file must be position independent.


To do
=====

Change history
==============
))


decimal


\ *************
\ Memory set up
\ *************

\ ******************************************************
\ DRAM Memory Bank 0 area MAP for Exception vector table
\ and Stack, User code area.                            
\ ******************************************************

0x01000000 equ DRAM_BASE
0x01400000 equ DRAM_LIMIT
\ 0x01800000 equ DRAM_LIMIT


\ *****************************************
\ SYSTEM MEMORY CONTROL REGISTER EQU TABLES
\ *****************************************

\ from SNDS.A
\ EXTDBWTH : Memory Bus Width register
\ 0 : Disable
\ 1 : Byte
\ 2 : Half-Word
\ 3 : Word

1      		equ DSR0	\ ROM0, 8 bit
2  2 <<n	equ DSR1	\ ROM1, 16 bit
3  4 <<n	equ DSR2	\ ROM2, 32 bit
3  6 <<n	equ DSR3	\ ROM3, 32 bit
3  8 <<N	equ DSR4	\ ROM4, 32 bit
3 10 <<n	equ DSR5	\ ROM5, 32 bit
3 12 <<n	equ DSD0	\ DRAM0, 32 bit
3 14 <<n	equ DSD1	\ DRAM1, 32 bit
3 16 <<n	equ DSD2	\ DRAM2, 32 bit
3 18 <<n	equ DSD3	\ DRAM3, 32 bit
3 20 <<n	equ DSX0	\ EXTIO0
3 22 <<n	equ DSX1	\ EXTIO1
3 24 <<n	equ DSX2	\ EXTIO2
3 26 <<n	equ DSX3 	\ EXTIO3

0
DSR0 +  DSR1 +  DSR2 +  DSR3 +  DSR4 +  DSR5 +
DSD0 +  DSD1 +  DSD2 +  DSD3 + 
DSX0 +  DSX1 +  DSX2 +  DSX3 +
  equ rEXTDBWTH


\ ROMCON0 : ROM Bank 0 Control register
\ The xxx_S values are used after remapping
$0000 10 <<n equ ROMBasePtr0	\ 0x0000:0000 in 64k units
$0020 20 <<n equ ROMEndPtr0	\ 0x0020:0000 in 64k units

$0100 10 <<n equ ROMBasePtr0_S	\ 0x0100:0000  ;REMAP 
$0120 20 <<n equ ROMEndPtr0_S	\ 0x0120:0000  

0        equ PMC0		\ 0x0=Normal ROM, 0x1=4Word Page 
				\ 0x2=8Word Page, 0x3=16Word Page
0  2 <<n equ rTpa0		\ 0x0=5Cycle, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle 
6  4 <<n equ rTacc0		\ 0x0=Disable, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle
				\ 0x4=5Cycle, 0x5=6Cycle
				\ 0x6=7Cycle, 0x7=Reserved
0  ROMEndPtr0 +  ROMBasePtr0 +  rTacc0 +  rTpa0 +  PMC0 +
  equ rROMCON0
0  ROMEndPtr0_S +  ROMBasePtr0_S +  rTacc0 +  rTpa0 +  PMC0 +
  equ rROMCON0_S


\ ROMCON1 : ROM Bank 1 Control register
\ The xxx_S values are used after remapping
$0020 10 <<n equ ROMBasePtr1	\ 0x0020:0000 in 64k units
$0040 20 <<n equ ROMEndPtr1	\ 0x0040:0000 in 64k units

$0120 10 <<n equ ROMBasePtr1_S	\ 0x0120:0000  ;REMAP 
$0140 20 <<n equ ROMEndPtr1_S	\ 0x0140:0000  

0        equ PMC1	\ 0x0=Normal ROM, 0x1=4Word Page 
			\ 0x2=8Word Page, 0x3=16Word Page
0  2 <<n equ rTpa1	\ 0x0=5Cycle, 0x1=2Cycle
			\ 0x2=3Cycle, 0x3=4Cycle 
6  4 <<n equ rTacc1	\ 0x0=Disable, 0x1=2Cycle
			\ 0x2=3Cycle, 0x3=4Cycle
			\ 0x4=5Cycle, 0x5=6Cycle
			\ 0x6=7Cycle, 0x7=Reserved
0  ROMEndPtr1 +  ROMBasePtr1 +  rTacc1 +  rTpa1 +  PMC1 +
  equ rROMCON1
0  ROMEndPtr1_S +  ROMBasePtr1_S +  rTacc1 +  rTpa1 +  PMC1 +
  equ rROMCON1_S


\ ROMCON2 : ROM Bank 2 Control register
\ The xxx_S values are used after remapping
$0040 10 <<n equ ROMBasePtr2	\ 0x0040:0000 in 64k units
$0060 20 <<n equ ROMEndPtr2	\ 0x0060:0000 in 64k units

$0140 10 <<n equ ROMBasePtr2_S	\ 0x0140:0000  ;REMAP 
$0160 20 <<n equ ROMEndPtr2_S	\ 0x0160:0000  

0        equ PMC2		\ 0x0=Normal ROM, 0x1=4Word Page 
				\ 0x2=8Word Page, 0x3=16Word Page
0  2 <<n equ rTpa2		\ 0x0=5Cycle, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle 
4  4 <<n equ rTacc2		\ 0x0=Disable, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle
				\ 0x4=5Cycle, 0x5=6Cycle
				\ 0x6=7Cycle, 0x7=Reserved
0  ROMEndPtr2 +  ROMBasePtr2 +  rTacc2 +  rTpa2 +  PMC2 +
  equ rROMCON2
0  ROMEndPtr2_S +  ROMBasePtr2_S +  rTacc2 +  rTpa2 +  PMC2 +
  equ rROMCON2_S


\ ROMCON3 : ROM Bank 3 Control register
\ The xxx_S values are used after remapping
$0060 10 <<n equ ROMBasePtr3	\ 0x0060:0000 in 64k units
$0080 20 <<n equ ROMEndPtr3	\ 0x0080:0000 in 64k units

$0160 10 <<n equ ROMBasePtr3_S	\ 0x0160:0000  ;REMAP 
$0180 20 <<n equ ROMEndPtr3_S	\ 0x0180:0000  

0        equ PMC3		\ 0x0=Normal ROM, 0x1=4Word Page 
				\ 0x2=8Word Page, 0x3=16Word Page
0  2 <<n equ rTpa3		\ 0x0=5Cycle, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle 
2  4 <<n equ rTacc3		\ 0x0=Disable, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle
				\ 0x4=5Cycle, 0x5=6Cycle
				\ 0x6=7Cycle, 0x7=Reserved
0  ROMEndPtr3 +  ROMBasePtr3 +  rTacc3 +  rTpa3 +  PMC3 +
  equ rROMCON3
0  ROMEndPtr3_S +  ROMBasePtr3_S +  rTacc3 +  rTpa3 +  PMC3 +
  equ rROMCON3_S


\ ROMCON4 : ROM Bank 4 Control register
\ The xxx_S values are used after remapping
$0080 10 <<n equ ROMBasePtr4	\ 0x0080:0000 in 64k units
$00A0 20 <<n equ ROMEndPtr4	\ 0x00A0:0000 in 64k units

$0180 10 <<n equ ROMBasePtr4_S	\ 0x0180:0000  ;REMAP 
$01A0 20 <<n equ ROMEndPtr4_S	\ 0x01A0:0000  

0        equ PMC4		\ 0x0=Normal ROM, 0x1=4Word Page 
				\ 0x2=8Word Page, 0x3=16Word Page
0  2 <<n equ rTpa4		\ 0x0=5Cycle, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle 
4  4 <<n equ rTacc4		\ 0x0=Disable, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle
				\ 0x4=5Cycle, 0x5=6Cycle
				\ 0x6=7Cycle, 0x7=Reserved
0  ROMEndPtr4 +  ROMBasePtr4 +  rTacc4 +  rTpa4 +  PMC4 +
  equ rROMCON4
0  ROMEndPtr4_S +  ROMBasePtr4_S +  rTacc4 +  rTpa4 +  PMC4 +
  equ rROMCON4_S


\ ROMCON5 : ROM Bank 5 Control register
\ The xxx_S values are used after remapping
$00A0 10 <<n equ ROMBasePtr5	\ 0x00A0:0000 in 64k units
$00C0 20 <<n equ ROMEndPtr5	\ 0x00C0:0000 in 64k units

$01A0 10 <<n equ ROMBasePtr5_S	\ 0x01A0:0000  ;REMAP 
$01C0 20 <<n equ ROMEndPtr5_S	\ 0x01C0:0000  

0        equ PMC5		\ 0x0=Normal ROM, 0x1=4Word Page 
				\ 0x2=8Word Page, 0x3=16Word Page
0  2 <<n equ rTpa5		\ 0x0=5Cycle, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle 
0  4 <<n equ rTacc5		\ 0x0=Disable, 0x1=2Cycle
				\ 0x2=3Cycle, 0x3=4Cycle
				\ 0x4=5Cycle, 0x5=6Cycle
				\ 0x6=7Cycle, 0x7=Reserved
0  ROMEndPtr5 +  ROMBasePtr5 +  rTacc5 +  rTpa5 +  PMC5 +
  equ rROMCON5
0  ROMEndPtr5_S +  ROMBasePtr5_S +  rTacc5 +  rTpa5 +  PMC5 +
  equ rROMCON5_S


\ DRAMCON0 : DRAM Bank 0 control register */
$0100 10 <<n equ DRAMBasePtr0	\ 0x0100:0000  
$0140 20 <<n equ DRAMEndPtr0	\ 0x0140:0000  

$0000 10 <<n equ DRAMBasePtr0_S	\ 0x0000:0000   ;; REMAP
$0040 20 <<n equ DRAMEndPtr0_S	\ 0x0040:0000

1  7 <<n equ STrc0		\ (Trc)0=1cycle,1=2cycle ; RAS to CAS delay
3  8 <<n equ STrp0		\ (Trp)0=1cycle ~ 3=4cycle ; RAS precharge
0 30 <<n equ SCAN0		\ 0=8bit,1=9bit,2=10bit,3=11bits ; # col addrs

0  SCAN0 +  DRAMEndPtr0 +  DRAMBasePtr0 +  STrp0 +  STrc0 +
  equ rSDRAMCON0
0  SCAN0 +  DRAMEndPtr0_S +  DRAMBasePtr0_S +  STrp0 +  STrc0 +
  equ rSDRAMCON0_S


\ DRAMCON1 : DRAM Bank 1 control register
$0140 10 <<n equ DRAMBasePtr1	\ 0x0140:0000  
$0180 20 <<n equ DRAMEndPtr1	\ 0x0180:0000  

$0040 10 <<n equ DRAMBasePtr1_S	\ 0x0040:0000   ;; REMAP
$0080 20 <<n equ DRAMEndPtr1_S	\ 0x0080:0000  

1  7 <<n equ STrc1		\ (Trc)0=1cycle,1=2cycle ; RAS to CAS delay
3  8 <<n equ STrp1		\ (Trp)0=1cycle ~ 3=4cycle ; RAS precharge
0 30 <<n equ SCAN1		\ 0=8bit,1=9bit,2=10bit,3=11bits ; # col addrs

0  SCAN1 +  DRAMEndPtr1 +  DRAMBasePtr1 +  STrp1 +  STrc1 +
  equ rSDRAMCON1
0  SCAN1 +  DRAMEndPtr1_S +  DRAMBasePtr1_S +  STrp1 +  STrc1 +
  equ rSDRAMCON1_S


\ DRAMCON2 : DRAM Bank 2 control register
$0180 10 <<n equ DRAMBasePtr2	\ 0x0180:0000  
$01C0 20 <<n equ DRAMEndPtr2	\ 0x01C0:0000  

$0080 10 <<n equ DRAMBasePtr2_S	\ 0x0080:0000   ;; REMAP
$00C0 20 <<n equ DRAMEndPtr2_S	\ 0x00C0:0000  

1  7 <<n equ STrc2		\ (Trc)0=1cycle,1=2cycle ; RAS to CAS delay
3  8 <<n equ STrp2		\ (Trp)0=1cycle ~ 3=4cycle ; RAS precharge
0 30 <<n equ SCAN2		\ 0=8bit,1=9bit,2=10bit,3=11bits ; # col addrs

0  SCAN2 +  DRAMEndPtr2 +  DRAMBasePtr2 +  STrp2 +  STrc2 +
  equ rSDRAMCON2
0  SCAN2 +  DRAMEndPtr2_S +  DRAMBasePtr2_S +  STrp2 +  STrc2 +
  equ rSDRAMCON2_S


\ DRAMCON3 : DRAM Bank 3 control register
$01C0 10 <<n equ DRAMBasePtr3	\ 0x01C0:0000  
$0200 20 <<n equ DRAMEndPtr3	\ 0x0200:0000  

$00C0 10 <<n equ DRAMBasePtr3_S	\ 0x00C0:0000   ;; REMAP
$0100 20 <<n equ DRAMEndPtr3_S	\ 0x0100:0000  

1  7 <<n equ STrc3		\ (Trc)0=1cycle,1=2cycle ; RAS to CAS delay
3  8 <<n equ STrp3		\ (Trp)0=1cycle ~ 3=4cycle ; RAS precharge
0 30 <<n equ SCAN3		\ 0=8bit,1=9bit,2=10bit,3=11bits ; # col addrs

0  SCAN3 +  DRAMEndPtr3 +  DRAMBasePtr3 +  STrp3 +  STrc3 +
  equ rSDRAMCON3
0  SCAN3 +  DRAMEndPtr3_S +  DRAMBasePtr3_S +  STrp3 +  STrc3 +
  equ rSDRAMCON3_S


\ REFEXTCON : External I/O & Memory Refresh cycle Control Register
\ refresh period = (2^11 - value + 1)/fmclk_mhz
\   for period in us:
\ value = 2^11 + 1 - period*fmclk

system-speed #1000000 / equ fmclk

$0360 equ ExtIOBase		\ external I/O at 0x0360:0000
1 15 <<n equ SREN		\ 1=refresh enable
1 16 <<n equ SVSF		\ 1=I/O accessible
3 17 <<n equ Strc		\ 0=1cycle, 1=2cycle, 2=3cycle, Row cycle time
				\ 3=4cycle, 4=5cycle,
8 fmclk *  #2049 swap -
  21 <<n equ SRefCycleValue	\ Unit [us], 4k refresh 64ms
0  SRefCycleValue +  STrc +  SVSF +  SREN +  ExtIOBase +
  equ rSREFEXTCON


\ ===============================================
\ SDRAM System Initialize Data	(KS32C50100 only)
\ ===============================================

l: SystemInitDataSDRAM
  rEXTDBWTH ,		\ DRAM1(Half), ROM5(Byte), ROM1(Half), else 32bit
  rROMCON0 ,		\ 0x0000:0000 ~ 0x001F:FFFF, ROM0,4Mbit,2cycle
  rROMCON1 ,		\ 0x0020:0000 ~ 0x003F:FFFF, ROM0,4Mbit
  rROMCON2 ,		\ 0x0040:0000 ~ 0x005F:FFFF, ROM2
  rROMCON3 ,		\ 0x0060:0000 ~ 0x007F:FFFF, ROM3
  rROMCON4 ,		\ 0x0080:0000 ~ 0x009F:FFFF, ROM4
  rROMCON5 ,		\ 
  rSDRAMCON0 ,  	\ 0x0100:0000 ~ 0x013F:FFFF, DRAM0 4M,
  rSDRAMCON1 ,		\ 0x0140:0000 ~ 0x017F:FFFF, DRAM1 4M,
  rSDRAMCON2 ,		\ 0x0180:0000 ~ 0x01EF:FFFF, DRAM2 16M
  rSDRAMCON3 ,		\ 0x01C0:0000 ~ 0x01FF:FFFF
  rSREFEXTCON , 	\ External I/O, Refresh	

l: ^SystemInitDataSDRAM
  SystemInitDataSDRAM ,

dram-low? [if]
l: SystemInitDataSDRAM_S
  rEXTDBWTH ,		\ DRAM1(Half), ROM5(Byte), ROM1(Half), else 32bit
  rROMCON0_S ,		\ 0x0100:0000 ~ 0x011F:FFFF, ROM0,4Mbit,2cycle
  rROMCON1_S ,		\ 0x0120:0000 ~ 0x013F:FFFF
  rROMCON2_S ,		\ 0x0140:0000 ~ 0x015F:FFFF, ROM2
  rROMCON3_S ,		\ 0x0160:0000 ~ 0x017F:FFFF, ROM3
  rROMCON4_S ,		\ 0x0180:0000 ~ 0x019F:FFFF, ROM4
  rROMCON5_S ,		\ 
  rSDRAMCON0_S ,  	\ 0x0000:0000 ~ 0x003F:FFFF, DRAM0 4M,
  rSDRAMCON1_S ,	\ 0x0040:0000 ~ 0x007F:FFFF, DRAM1 4M,
  rSDRAMCON2_S ,	\ 0x0080:0000 ~ 0x00EF:FFFF, DRAM2 16M
  rSDRAMCON3_S ,	\ 0x00C0:0000 ~ 0x00FF:FFFF
  rSREFEXTCON , 	\ External I/O, Refresh	

l: ^SystemInitDataSDRAM_S
  SystemInitDataSDRAM_S ,
[then]


\ *************************************
\ End of memory set up data definitions
\ *************************************


\ **********
\ Entry code
\ **********

PROC Reset_4510
  mvl     r1, # _interrupts
  mvn     r0, # 0			\ disable all interrupts
  str     r0, [ r1, # intmsk ]

\ LED Display
  mvl     r1, # _ioports
  mov     r0, # $FF			\ bottom 8 bits as outputs
  str     r0, [ r1, # iopmod ]
  mov     r0, # $55			\ inverted to give AA on board LEDs
  str     r0, [ r1, # iopdata ]

\ Special Register Configuration for SYNC DRAM
l:  sync_dram_config
  mvl     r0, # 0x3FF0000		\ SYSCFG reg after reset
  mvl     r1, # 0x83FFFF90		\ SDRAM, regs 3FF:0000, SRAM 3FE:0000, 8k cache
\ UCache?      [if]  bit1 or  [then]	\ set CE bit if required
\ WriteBuffer? [if]  bit2 or  [then]	\ Set WE bit if required
  STR     r1, [ r0 ]			\ Cache,WB disable,Start_addr = 0x3FF00000

\ Configure memory with Flash at low memory
\ ROM and RAM Configuration(Multiple Load and Store)
  ldr     r0, ^SystemInitDataSDRAM	\ point to initial table values
  LDMIA   r0, { r1-r12 }
  mvl     r0, # _SYSTEM EXTDBWTH +	\ table starts with EXTDBWTH register
  STMIA   r0, { r1-r12 }		\ load registers in one instruction
l: mem_setup_done

dram-low? [if]
\ If required, copy ROM to RAM
run-from-dram? [if]
  mov     r0, # $10000			\ waste time to initialise SDRAM
l: wloop1
  sub .s  r0, r0, # 1
  b .ne   wloop1

  mov	  r0, # 0			\ Flash at 0000:0000
  mvl     r1, # 0x01000000		\ start of code in DRAM
  mvl     r2, # CODE-SIZE		\ amount to copy
  add	  r2, r2, # 3			\ round to cell
  bic     r2, r2, # 3
l: dram-copy-loop
  ldr	  r3, [ r0 ], # 4
  str     r3, [ r1 ], # 4
  sub .s  r2, r2, # 4
  b .ne   dram-copy-loop
[then]

\ Now reconfigure memory with the Flash high and DRAM at 0
\ ROM and RAM Configuration (Multiple Load and Store)
  ldr     r0, ^SystemInitDataSDRAM_S	\ point to initial table values
  LDMIA   r0, { r1-r12 }
  mvl     r0, # _SYSTEM EXTDBWTH +	\ table starts with EXTDBWTH register
  STMIA   r0, { r1-r12 }		\ load registers in one instruction
[else]
\ If required, copy ROM to RAM
run-from-dram? [if]
  mov     r0, # $10000			\ waste time to initialise SDRAM
l: wloop1
  sub .s  r0, r0, # 1
  b .ne   wloop1

  mov	  r0, # 0			\ Flash at 0000:0000
  mvl     r1, # CODE-START		\ start of code in DRAM
  mvl     r2, # CODE-SIZE		\ amount to copy
  add	  r2, r2, # 3			\ round to cell
  bic     r2, r2, # 3
l: dram-copy-loop
  ldr	  r3, [ r0 ], # 4
  str     r3, [ r1 ], # 4
  sub .s  r2, r2, # 4
  b .ne   dram-copy-loop
[then]
[then]

\ Reconfigure with cache enabled
  mvl     r0, # 0x3FF0000		\ SYSCFG reg after reset
  mvl     r1, # 0x83FFFF90		\ SDRAM, regs 3FF:0000, SRAM 3FE:0000, 8k cache
UCache?      [if]  bit1 or  [then]	\ set CE bit if required
WriteBuffer? [if]  bit2 or  [then]	\ Set WE bit if required
  STR     r1, [ r0 ]			\ Cache,WB disable,Start_addr = 0x3FF00000


\ **********************************************
\ All the boot level hardware is now initialised
\ **********************************************

\ This code sets up the CPSRs of each mode and the RSP pointer.
\ The code in INTARM2.FTH provides a nest of stacks which are
\ set up by the interrupt entry code. Only compile this code block
\ if you are going to handle interrupt entry and exit yourself.
\ Note the CPSR registers are also set up in ECLD in INITARM.FTH
\ in order to support systems without stack handlers.

  mov r0, # InterruptMask irq_32 or
  msr CPSR, _c r0                         
  mvl r0, # IRQ_StackTop		\ IRQ mode stack
  mov rsp, r0 

  mov r0, # InterruptMask fiq_32 or 
  msr CPSR, _c r0                         
  mvl r0, # FIQ_StackTop		\ FIQ mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask und_32 or
  msr CPSR, _c r0                         
  mvl r0, # UND_StackTop		\ UND mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask abt_32 or
  msr CPSR, _c r0                         
  mvl r0, # ABT_StackTop		\ ABT mode stack
  mov rsp, r0                

  mov r0, # InterruptMask svc_32 or	\ Supervisor mode
  msr CPSR, _c r0
  mvl r0, # SVC_StackTop		\ SVC mode stack
  mov rsp, r0

  mov r0, # InterruptMask sys_32 or
  msr CPSR, _c r0                         
\ If you intend to run Forth in USR mode rather than SYS mode then you
\ need to switch here. The code at ECLD in INITARM.FTH will set up the
\ stacks for the current mode.

\ Done this way in case a relative branch is out of range
  adr	  r0, # ECLD			\ join code in INITARM.FTH
  mov	  pc, r0

  next,					\ keeps disassembler happy!
end-code

Reset_4510 CODE-START - ResetV !	\ Slot into the vector table

