\ MX211B control file - run from SDRAM

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

N.B. The Samsung S3C4510 core is run in big-endian mode, and
the correct version of the cross compiler XARMBE.EXE must be
used with this control file.

The cross compiler produces a big-endian file.
If the MX211B is run with 16 bit Flash, the image file must
be put into the two ROMs such that the even bytes are on
XDATA[15:8] which is the ODD device (U18) on the board.
The odd bytes are on XDATA[7:0] which is the EVEN device
(U17) on the board.

Tested on Maestro MX211B.


To do
=====

Change history
==============
20030120 SFP002 Moved ETHER4510.FTH and NETBOOT.FTH
20020703 SFP001 First release
))

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

"" ..\common"   setmacro CommonDir	\ where the common code lives
"" ..\examples" setmacro ExampleDir	\ where the examples live
"" ."           setmacro CpuDir		\ where the CPU specific code lives
"" .\Hardware\MX211b" setmacro HWDir	\ where board specific code lives
"" ..\Examples" setmacro ExampleDir	\ Examples
"" ..\MyApp"    setmacro AppDir		\ where application code lives
"" \buildkit.dev\software\AddOns\PowerNet\v30dev"
                setmacro IpStack	\ PowerNet code
"" ..\examples\Filesys"
                setmacro FileSysDir	\ where the File System lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

  include %CpuDir%\macros		\ compiler / assembler macros

\ file: PROG.log

CROSS-COMPILE

only forth definitions			\ default search order

\  log
  no-log				\ no output log
  rommed				\ split ROM/RAM system
  align-long				\ code is 32bit aligned
  arm7tdmi				\ ARM7 core
  32bit-mode				\ running in 32 bit mode
  interactive                   	\ enter interactive mode at end
  +xrefs                        	\ enable cross references

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

 1 equ CPU=ARM		\ nz=tell source to use ARM specific code

$03FF0000 equ _SFRBASE_DEFAULT		\ where registers live after reset
_SFRBASE_DEFAULT equ _SFRBASE_USER	\ change if application remaps them

$03FE0000 equ _RAMBASE_DEFAULT		\ where internal RAM lives after reset
_RAMBASE_DEFAULT equ _RAMBASE_USER	\ change if application remaps them

 1 equ run-from-dram?	\ 0=from flash, nz=copy to DRAM
 1 equ dram-low?	\ 0=DRAM high, nz=DRAM at address 0

 1 equ UCache?		\ true to turn on Unified CACHE
 \ N.B. If UCache? is true peripherals MUST be addressed with
 \ address bit 26 set (0400:0000) to avoid the cache. This can be
 \ done by defining all peripherals at their physical address + 0400:000.
 1 equ WriteBuffer?	\ true to enable write back buffer

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 1 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $20000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

\ System clocks
#50000000 equ system-speed	\ System clock rate in Hz.
#10000000 equ xtal-speed	\ system crystal clock rate in Hz

\ Serial ports on UARTs 0/1
#115200 equ uart0-speed		\ Serial port speed in BPS.
#38400 equ uart1-speed		\ Serial port speed in BPS.
0 equ console-port		\ Designate serial port 0 for console.

\ Timebase
#10 equ tick-ms			\ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0200 equ UP-SIZE		\ size of each task's user area
$0200 equ SP-SIZE		\ size of each task's data stack
$0400 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#4 equ sp-guard

4 equ #IRQs			\ number of IRQ stacks, shared by all IRQs
4 equ #SWIs			\ number of SWI nestings permitted

UP-SIZE equ INTRAM		\ space used by interrupt page

$0100 equ TIB-LEN		\ terminal i/p buffer length


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)


\ ********************
\ Define memory layout
\ ********************

dram-low? [if]
  $00000000 equ link-address			\ usually starts at zero for AIF
  $00000000 $0003FFFF cdata section MXnet	\ allow 512k - N.B. Affects INITxxx file
    sec-base equ CODE-START
    $40000   equ CODE-SIZE
  $00400000 $004FFFFF idata section iram	\ 1M - N.B. Affects INITxxx file
  $00500000 $005FFFFF udata section uram	\ 1M - N.B. Affects INITxxx file
[else]
  $01000000 equ link-address			\ usually starts at zero for AIF
  $01000000 $0101FFFF cdata section MXnet	\ allow 128k - N.B. Affects INITxxx file
    sec-base equ CODE-START
    $20000   equ CODE-SIZE
  $01100000 $0111FFFF idata section iram	\ 128k - N.B. Affects INITxxx file
  $01200000 $0121FFFF udata section uram	\ 128k - N.B. Affects INITxxx file
[then]

interpreter
: prog	MXnet ;			\ synonym: PROG is alias for KEB
: progd	iram  ;				\ synonym:
: progu	uram  ;				\ synonym:
target

romforth? [if]
  $00600000 $006FFFFF udata section progr	\ 128k - N.B. Affects INITxxx file
  sec-base equ APPL-ROM                 \ start of application space
  sec-top  equ APPL-ROM-END             \ end of application space
  sec-top  equ EM
[then]

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************************
\ USER area and Multi tasker equates
\ **********************************
\ Assume stacks grow down: user area, sp stack, rp-stack

\ Main User/Task stack for USR/SYS mode operation.
\ In order to permit fast interrupt nesting the
\ return stack must be the lowest of RSP, PSP and UP
\ In order for the initialisation code in MULTIARM.FTH
\ to work, INIT-U0 must be the highest of INIT-UO,
\ INIT-S0 and INIT-R0.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack

\ SVC stack ; nestable up to #SWIs
0 reserve equ SVC_StackTop		\ top of stack
task-size #SWIs * reserve equ SVC_StackBase	\ bottom of stack

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_StackTop		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_StackBase

\ FIQ stack
0 reserve equ FIQ_StackTop		\ top of FIQ stack
task-size reserve equ FIQ_StackBase	\ bottom of FIQ stack

\ UND stack
0 reserve equ UND_StackTop		\ top of stack
task-size reserve equ UND_StackBase	\ bottom of stack

\ ABT stack
0 reserve equ ABT_StackTop		\ top of stack
task-size reserve equ ABT_StackBase	\ bottom of stack

\ TIB - terminal input buffer
tib-len reserve equ INIT-TIB		\ base of TIB

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfrS3C4510		\ Special function registers.
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HWDir%\InitMXsdram		\ S3C4510 start up code - H/W specific

  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  include %CpuDir%\intS3C4510		\ exception handlers
\  include %CpuDir%\drivers\ser4510p	\ S3C4510 polled UART driver
  include %CommonDir%\CQueues
  include %CpuDir%\drivers\ser4510qi	\ S3C4510 queued interrupt UART driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variable support

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
[then]
  include %CpuDir%\drivers\Tick4510	\ timer tick - must be after TIMEBASE.FTH

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
\  include %CommonDir%\tests\mpe\testheap \ memory allocation test
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
console-port 0 = [if]
  synonym ser-key  serkey0
  synonym ser-key? serkey?0
  synonym ser-emit seremit0
[then]
console-port 1 = [if]
  synonym ser-key  serkey1
  synonym ser-key? serkey?1
  synonym ser-emit seremit1
[then]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ XSHELL textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE S3C4510 ANS ROM PowerForth v6.10 "
;
external

: ANS-FORTH	\ -- ; marker
;


\ *************************
\ Add application code here
\ *************************

\  include %FileSysDir%\Filesys.bld	\ CEM Filing system

  include %CpuDir%\drivers\Ether4510	\ Samsung Ethernet driver

\ *** Define these constants carefully! ***
\ These assume that the bottom 256k of Flash is used for the
\ boot code, the next 192k of Flash is used for applications
\ loaded by the network boot system, and the final 64k is used
\ for data storage.
\ N.B. These constants are affected by the SECTION definitions.

\ How the Flash is laid out
dram-low? [if]
$06000000 constant flashbase	\ -- addr ; Base address of flash memory.
flashbase constant BootFlash	\ base address of boot Flash after mapping
$00040000 constant BootLen	\ length of boot Flash
$00000000 constant BootRAM	\ base address of boot code after mapping
$06040000 constant userflash	\ -- addr ; base address of user flash
$00030000 constant userflashlen	\ -- n ; size of user flash
$06070000 constant dataflash	\ -- addr ; base address of data flash
$00010000 constant datalen	\ -- n ; size of data flash
\ Where applications are copied to from the user flash
$00400000 constant AppRam	\ -- addr ; application area
$00100000 constant Applen	\ -- n ; length of application area
[else]
$00000000 constant flashbase	\ -- addr ; Base address of flash memory.
flashbase constant BootFlash	\ base address of boot Flash after mapping
$00020000 constant BootLen	\ length of boot Flash
$01000000 constant BootRAM	\ base address of boot code after mapping
$00020000 constant userflash	\ -- addr ; base address of user flash
$0001FF00 constant userflashlen	\ -- n ; size of user flash
$0003FF00 constant dataflash	\ -- addr ; base address of data flash
$00000100 constant datalen	\ -- n ; size of data flash
\ Where applications are copied to from the user flash
$01400000 constant AppRam	\ -- addr ; application area
$00100000 constant Applen	\ -- n ; length of application area
[then]

compiler
: ForceUncached		\ addr -- addr'
  bit26 or  ;
target
interpreter
: ForceUncached		\ addr -- addr'
  bit26 or  ;
target

  include %CpuDir%\drivers\29f040b	\ Flash programming
  include %CpuDir%\drivers\netcode	\ Network order and CPU dependent
  include %CpuDir%\drivers\NetBoot	\ Network boot loader

\ PowerNet configuration and setup

 1 equ ethernet?	\ nz for Ethernet systems
 0 equ slip?		\ nz to include SLIP

 0 equ tftp?		\ nz to include TFTP
 1 equ tcp?		\ nz for TCP as well as UDP
 1 equ telnet?		\ nz to include Telnet
 1 equ echo?		\ nz to include Echo
 0 equ snmp?	        \ nz to include SNMP
 1 equ diags?		\ nz to include diagnostics (recommended)
  include %IpStack%\PowerNet.bld


\ ********************************************
\ The following code can be included if wanted
\ ********************************************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %CommonDir%\tests\mpe\testheap


\ *******************
\ Clean up and finish
\ *******************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs


\ ****************
\ *S Sanity checks
\ ****************

decimal

  cr ." Required USER size is     : " next-user @ .
  cr ." Current USER allocation is: " up-size .
Next-user @ up-size > [if]
\ *G Check that the USER area is large enough.
  cr ." *** Increase USER area size UP-SIZE in control file ***
  abort
[then]


\ ********
\ All done
\ ********

AIF? [if]
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  cdata here link-address -             \ complete AIF header structure
    AIFimagereadonlysize !
[then]


((
flush-idata
hex chere $1FF + $-200 and corg		\ SFTP utility requires 512 byte units
))

\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

FINIS

