\ Startup Code for the Boot Loader in the MPE ARM7 Development Kit

((
Copyright (c) 2000, 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


1) Sets the PPI controller for PortA/B as input and PortC as output.
2) Configures Flash as Segment1 mapped in high memory.
3) Maps external SRAM from address 0
4) Enables internal SRAM as combined cache
5) Configures Ethernet Chip Select Area
6) Copies app from Flash to Ram:0000

The code is set up to run in a 48k section of Flash
  $0000:0000 $0000:BFFF at boot, copied to RAM at same address
The boot code remaps the chip select unit and segment mapper to put:
  1Mb Flash	at 0100:0000 to 010F:FFFF	Segment 1	read/write, not cached
  128kb RAM	at 0000:0000 to 0001:FFFF	Segment 2, BCR2, CS3:2	16b, r/w, cached
  1M-128kb RAM	at 0002:0000 to 000F:FFFF	Segment 3, BCR2, CS3:2	16b, r/w, not cached
  2k local SRAM	at 6000:0000 to 6000:07FF	Cache mode
  1k Ethernet	at 5000:0000 to 5000:03FF	Segment 4	read/write, not cached
The vector table is then copied to address 0.

N.B. Because of a bug in the LH77790A/B parts, only the
CDATA areas can be cached.


To do
=====

Change history
==============
20030602 SFP001 Fixed cache problems
))


aif? [if]

\ Use default Reset handler in INITARM.FTH (branches to ECLD)
\ because the AIF loader will have done all the CPU specific
\ set up.

[else]

proc SetupIO	\ R0 contains address of initialisation table
\ Initialise a device from a table. The table consists of
\ pairs of address/data cells, terminated by a 0 address
\ which is unused. Destroys R0, R1.
l: SetupIO-1
  ldr	  r1, [ r0 ], # 4	\ get address
  cmp	  r1, # 0		\ get out if 0
  b .eq   SetupIO-x
  ldr     r2, [ r0 ], # 4	\ get data
  str	  r2, [ r1, # 0 ]	\ store data
  b       SetupIO-1
l: SetupIO-x
  next,
end-code


\ This code relies on being position independent up to
\ the final "MOV  PC, R0". Although compiled at the runtime
\ address, this code is executed out of Flash after it has
\ been remapped.
\ The Flash has already been mapped into Segment 1 and Bank 1
proc InFLASH
\ Set RAM as segment 2/3 using Bank 2, CS3=HI CS2=LO, 16 bit, 1 waits
  adr r0, $9090         adr r1, _BCR2       str r0, [ r1 ]
\ 0000:0000..0001:FFFF - read/write, cached, 32 bit, Bank 2
  adr r0, $7C04         adr r1, _SDR2       str r0, [ r1 ]
  adr r0, prog_start    adr r1, _START2     str r0, [ r1 ]
  adr r0, prog_stop     adr r1, _STOP2      str r0, [ r1 ]
\ 0002:0000..000F:FFFF - read/write, uncached, 32 bit, Bank 2
  adr r0, $7804         adr r1, _SDR3       str r0, [ r1 ]
  adr r0, ram_start     adr r1, _START3     str r0, [ r1 ]
  adr r0, ram_stop      adr r1, _STOP3      str r0, [ r1 ]
\ Set Ethernet as Bank 4 using CS1, 8 bit, 4 wait
  adr     r0, %0100000000001100
  adr     r1, _BCR4
  str     r0, [ r1 ]
\ Setup Segment4 as read/write, non-cachable, Bank4
  adr     r0, %0111100000010000
  adr     r1, _SDR4
  str     r0, [ r1 ]
\ 1k from $5000:0000
  adr     r0, $50000000
  adr     r1, _START4
  str     r0, [ r1 ]
  adr     r0, $50000400
  adr     r1, _STOP4
  str     r0, [ r1 ]

\ Local SRAM and Cache
  adr r1, _CCR
  mov r0, # $08        str r0, [ r1 ]	\ invalidate cache
  mov r0, # $01        str r0, [ r1 ]	\ enable cache
\ Local SRAM enabled, mapped at 6000:0000 to 6000:07FF
  mov r0, # $03         adr r1, _LSCR       str r0, [ r1 ]

\ Copy boot code to low RAM if it exists
  mov     r0, # Flash_Start		\ Source address.
  mov     r1, # Prog_Start ( 0 )	\ Destination address.
  mov     r2, # Prog_len		\ Length to copy - first 128kb/192kb
L: CopyJumpTable
  ldr     r3, [ r0 ], # 4
  str     r3, [ r1 ], # 4
  sub .s  r2, r2, # 4
  b .ne CopyJumpTable

\ The code in INTARM2.FTH provides a nest of stacks which are
\ set up by the interrupt entry code. This code sets up those
\ stacks.
  mov r0, # InterruptMask irq_32 or
  msr CPSR, _c r0                         
  mvl r0, # IRQ_Stack_Top		\ IRQ mode stack
  mov rsp, r0 

  mov r0, # InterruptMask fiq_32 or 
  msr CPSR, _c r0                         
  mvl r0, # FIQ_Stack_Top		\ FIQ mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask und_32 or
  msr CPSR, _c r0                         
  mvl r0, # UND_StackTop		\ UND mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask abt_32 or
  msr CPSR, _c r0                         
  mvl r0, # ABT_StackTop		\ ABT mode stack
  mov rsp, r0                

  mov r0, # InterruptMask svc_32 or	\ Supervisor mode
  msr CPSR, _c r0
  mvl r0, # SVC_StackTop		\ ABT mode stack
  mov rsp, r0

\  mov r0, # InterruptMask svc_32 or
\  msr CPSR, _c r0                         
\ If you intend to run Forth in USR mode rather than SYS mode then you
\ need to switch here.

\ Jump to Forth startup code
l: StartForth
  adr r0, # ECLD
  mov pc, r0
end-code


\ To understand this code, you will need the LH77790 User manual,
\ section 5. This is the hardware reset code.
proc HandlerReset
\ Disable all interrupt sources.
\ The watchdog timer is disabled by a hardware reset.
  mov     r0, # 0
  mvl     r1, _FIQER    str r0, [ r1 ]
  mvl     r1, _IRQER    str r0, [ r1 ]

\ Set up PPI (8255 clone)
  mov  r0, # $80        adr r1, _PPI_CTLR   strb r0, [ r1 ]
  mov  r0, # $00        adr r1, _PPI_PC     strb r0, [ r1 ]

\ Set up Flash as Segment 1 using CS0, 8 bit, 5 waits
  adr r0, $5003         adr r1, _BCR1       str r0, [ r1 ]
\ Read Write, non-cacheable, 32 bit, Bank 1
  adr r0, $7802         adr r1, _SDR1       str r0, [ r1 ]
\ set start and end addresses of segment 1
  adr r0, FLASH_START   adr r1, _START1     str r0, [ r1 ]
  adr r0, FLASH_STOP    adr r1, _STOP1      str r0, [ r1 ]

\ Jump out of segment 8 (reset at 0) into the Flash at segment 1
  adr   r0, # InFlash Prog_Start - Flash_start +
  mov   pc, r0
end-code

\ The initial reset address MUST point into boot code
\ in low memory, which is where the Flash is initially
\ addressed.
HandlerReset prog_start -  ResetV !

[then]

