\ Serial Driver for Debug Uart on MPE ARM7 Development Kit.
\ Performs interrupt driven receive and polled transmit on UART channel 2.
\ This uart is the device pinned on the core board for development.

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20020625 SFP002 Added interrupt locking to SERKEY.
		Enhanced GEN-BAUD-RATE.
))


only forth definitions  decimal


\ *************************
\ Required data definitions
\ *************************

variable rx-char	\ -- addr ; character last read
variable rx-avail	\ -- addr ; true when character received


\ **************
\ Initialisation
\ **************

\ This next word takes the system clock rate (in HZ) and the required serial
\ comms rate (in BPS) and returns a system-dependant number of parameters that
\ should be sufficent to effect the baud rate selection by initialising the
\ correct hardware.

interpreter
: gen-baud-rate	\ clk-rate bps-rate -- 16bit-divisor
  0 swap um/mod nip			\ clk/bps
  #16 /mod swap 8 >=			\ clk/bps/16, check for rounding
  if  1+  endif
;
target

system-speed console-speed gen-baud-rate
  equ serial_divisor

\ internal

: init-ser	\ -- ; initialise system serial port on UART2
  di
  0 $FFFF0C00 !                         \ Channel2 in UART Mode
  _PCSR @				\ select internal clock source
  PCSR_mask_U2 invert and
  _PCSR  !
  _XCLK/1                 _U2CCR !      \ baud rate prescaler
  $07 $80 or              _LCR2  !      \ 8N2, "$80 OR" => divisor access via DLAB.
  serial_divisor 8 rshift _DLM2  !      \ set divisor
  serial_divisor $FF and  _DLL2  !
  $07                     _LCR2  !	\ disable divisor access
  $01                     _IER2  !      \ Enable receive interrupt

  _fiqer @ IRQ_mask_U2 invert and _fiqer !      \ disable UART1 on FIQ
  _irqer @ IRQ_mask_U2 or _irqer !              \ enable UART1 on IRQ

  rx-avail off
  ei					\ enable interrupts
;


\ *****************
\ Serial primitives
\ *****************

: ser2-isr      \ --
  _irqsr @ IRQ_mask_U2 and if           \ test for this interrupt
    _RBR2 @ $FF and rx-char !		\ read character (resets interrupt)
    rx-avail on
  endif
;
' ser2-isr add-irq      \ add this ISR to the IRQ list

: (serkey?)	\ -- t/f
  rx-avail @
;

: (serkey)	\ -- char
  begin
    [I
    rx-avail @ 0=
   while
    I]
[ tasking? ] [if] pause [then]
  repeat
  rx-char @
  rx-avail off
  I]
;

((
: (seremit)	\ char --
  begin
    _LSR2 @ %00100000 and 0=
   while
  repeat
  _THR2 !
;
))
: (seremit)	\ char --
  begin
    _LSR2 @ %00100000 and
  until
  _THR2 !
;

: (serTYPE)	\ c-addr len --
  bounds
  ?do  i c@ (seremit)  loop
;

: (serCR)		\ --
  $0D (seremit)  $0A (seremit)
;

external


\ ***********************
\ Generic i/o assignments
\ ***********************


cdata
create Console	\ -- addr ; OUT managed by upper driver
  ' (serkey) ,		\ -- char ; receive char
  ' (serkey?) ,		\ -- flag ; check receive char
  ' (seremit) ,		\ -- char ; display char
  ' (sertype) ,		\ caddr len -- ; display string
  ' (sercr) ,		\ -- ; display new line

decimal

