\ LcdQvgaMono4.fth - 320 * 240 * 2 Monochrome QVGA driver for Sharp LH77790B

((
Copyright (c) 1999, 2000, 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050303 MPE001 Overhauled and font separated to Font16x16x2.fth
))

decimal

\ ======
\ *> gui
\ *S Example Monochrome LCD driver
\ ======
\ *P The file *\i{LcdQvgaMono} is a driver for a 320*240*2 bits
\ ** monochrome QVGA panel for the Sharp LH77790B CPU which has
\ ** an on-chip LCD controller.

decimal

#320 equ pix/row	\ -- u ; number of pixels per row (line)
#240 equ pix/col	\ -- u ; number of pixels per column
#2 equ bits/pix		\ -- u ; number of bits per pixel
#4 equ pix/byte		\ -- u ; number of pixels per byte
#16 equ pix/cell	\ -- u ; number of pixels in a 32 bit cell

pix/row pix/byte /
  equ bytes/row		\ -- u ; number of bytes per row

bytes/row  pix/col *  #1000 +
  equ szVIDEO_BUFFER	\ -- u ; Memory Size of Frame Buffer

0 equ COLOR_WHITE
1 equ COLOR_LTGRAY
2 equ COLOR_GRAY
3 equ COLOR_BLACK


\ *******************
\ *N Low level driver
\ *******************

PROGv
szVIDEO_BUFFER buffer: VIDEO_BUFFER	\ -- addr
PROG

variable color		\ -- addr
\ *G Holds the colour used for lines.
  COLOR_BLACK color !

variable back_color	\ -- addr
\ *G Holds the colour used for the background.
  color_white back_color !


: fill-lcd	\ colour --
  $3 and				\ %000000xy
  dup 2 lshift or			\ %0000xyxy
  dup 4 lshift or			\ %xyxyxyxy
  VIDEO_BUFFER szVIDEO_BUFFER rot fill
;

: clear-lcd	\ --
\ *G Clear the LCD screen.
  back_color @ fill-lcd
;

: init-lcd	\ --
\ *G Initialise the LCD hardware. Performed at power-up.
  _PCSR @ %01000000 or _PCSR !		\ Enable LCD Clock in CControl
  $8A _LCD_MODE c!			\ Gray, 4-bit, Division, Single, InACTIVE
  $4F _LCD_BC c!			\ Byte Count = 80-1
  $01 _LCD_CP1W c!			\ CP1 Pulse Width = 2
  $EF _LCD_DUTY_7_0 c!			\ Duty Cycle 1/240
  %00 _LCD_DUTY_9_8 c!
  VIDEO_BUFFER _LCD_SADR1_7_0 c!	\ Video buffer Address
  VIDEO_BUFFER 8 rshift _LCD_SADR1_15_8 c!
  VIDEO_BUFFER 16 rshift _LCD_SADR1_23_16 c!
  VIDEO_BUFFER 24 rshift _LCD_SADR1_31_24 c!
  $EF _LCD_VLC1_7_0 c!			\ Set VLC1 = DUTY
  %00 _LCD_VLC1_9_8 c!
  $00 _LCD_VDLT c!			\ Video Memory Delta
  %01010101 _LCD_GRAY1 c!		\ ??
  %11101110 _LCD_GRAY2 c!		\ ??

  $00 _LCD_CLKDIV C!			\ Clock divisor
  $8B _LCD_MODE c!			\ Gray, 4-bit, Division, Single, ACTIVE
  clear-lcd				\ Clear the screen
;
' init-lcd AtCold

: SetColor      \ colour --
\ *G Set the drawing colour.
  color !
;

: SetBackColor  \ colour --
\ *G Set the background colour.
  back_color !
;

: SetTextColor	\ colour --
\ *G Set the text drawing colour.
  $03 and
  dup 2 lshift or
  dup 4 lshift or
  dup 8 lshift or
  dup 16 lshift or
  text_color !
;

: PutPixel      \ x y --
\ *G Set the pixel to the current drawing (line) colour.
  bytes/row *				\ x yofs --
  over 2 rshift +			\ x byteofs --
  swap 3 and				\ byteofs pixelofs --
  over VIDEO_BUFFER + c@		\ byteofs pixelofs byte --
  swap case
    0 of %00111111 and color @ 6 lshift or ( %11000000 or ) endof
    1 of %11001111 and color @ 4 lshift or ( %00110000 or ) endof
    2 of %11110011 and color @ 2 lshift or ( %00001100 or ) endof
    3 of %11111100 and color @          or ( %00000011 or ) endof
    drop
  end-case
  swap VIDEO_BUFFER + c!
;

: PutByte      \ x y --
\ *G Set a byte (4 pixels) to the current drawing (line) colour.
  bytes/row *  swap 2 rshift +		\ -- offset
  VIDEO_BUFFER + 			\ -- address
  color @
  dup 2 lshift or  dup 4 lshift or
  swap c!
;

: PutCell	\ x y --
\ *G Set a cell (16 pixels) to the current drawing (line) colour.
  bytes/row *  swap 2 rshift +		\ -- offset
  VIDEO_BUFFER + 			\ -- address
  color @
  dup 2 lshift or  dup 4 lshift or
  dup 8 lshift or  dup #16 lshift or
  swap !
;

: hPixStep	\ y x len -- y x' len'
  -rot  2dup swap putpixel  rot
  1 /string
;

: hByteStep	\ y x len -- y x' len'
  -rot  2dup swap putbyte  rot
  pix/byte /string
;

: hCellStep	\ y x len -- y x' len'
  -rot  2dup swap putcell  rot
  pix/cell /string
;

: h-line        \ x y len --
\ *G Draw a horizontal line of length *\i{len} pixels starting
\ ** at *\i{x,y}.
  over pix/col u< not
  if  drop 2drop  exit  then		\ line off screen

  dup 0< if				\ check for negative length
    2 pick +  rot                       \ y xstart xend --
  else
    2 pick +  rot swap			\ y xstart xend --
  then

  over pix/row u< not
  if  drop 2drop  exit  then		\ line starts off screen

  dup pix/row u< not
  if  drop pix/row 1-  then		\ clip to edge of screen
  over 0<
  if  nip 0 swap then           	\ clip to start of screen

  over -				\ y xstart xlen --
\ step to byte boundary
  begin  over pix/byte 1- and while  hPixStep  repeat
\ step to cell boundary
  begin  over pix/cell 1- and while  hByteStep  repeat
\ fill complete cells
  begin  dup pix/cell >= while  hCellStep  repeat
\ finish off bytes
  begin  dup pix/byte >= while  hByteStep  repeat
\ finish off pixels
  begin  dup while  hPixStep  repeat

  3drop
;

: v-line        \ x y len --
\ *G Draw a vertical line of length *\i{len} pixels starting
\ ** at *\i{x,y}.
  2 pick pix/row u< not
  if  drop 2drop  exit  then		\ line off screen

  dup 0< if
    over + swap				\ x ystart yend --
  else
    over +				\ x ystart yend --
  then

  dup pix/col u< not
  if  drop  pix/col 1-  then		\ clip to bottom of screen
  over 0<
  if  nip 0 swap  then			\ clip to top of screen

  over -				\ x ystart len --
  0 ?do
    2dup putpixel  1+
  loop
  2drop
;

: rect          \ x y w h --
\ *G Draw a rectangle at *\i{x,y} with width and height *\i{w,h}.
  2>r					\ -- x y  ; R: -- w h
  dup pix/col u< not
  if  r> r> 2drop  2drop  exit  then
  over pix/row u< not
  if  r> r> 2drop  2drop  exit  then
\ -- x y  ; R: -- w h
\ 1st vertical: x y h v-line
  2dup r@ v-line
\ 2nd vertical: x+w y h v-line
  2dup swap 2r@				\ -- x y y x w h ; R: -- w h
  -rot + -rot v-line
\ horizontal: x y+h w h-line
  2dup r> + r@ h-line			\ -- x y ; R: -- w
\ horizontal: x y w h-line
  r> h-line
;

: FilledRect	\ x y w h --
\ *G Draw a filled rectangle at *\i{x,y} with width and height
\ ** *\i{w,h}.
  rot swap bounds ?do		\ -- x w
    2dup i swap h-line
  loop
  2drop
;


\ ***************
\ *N Font display
\ ***************

: .fchar        \ px py char --
\ *G Write a character at the given pixel address, which is
\ ** stepped to the nearest byte boundary.
  -rot					\ -- char px py
  bytes/row *				\ starting video row offset
  swap pix/byte / +			\ starting row offset in bytes
  VIDEO_BUFFER +			\ base address of video buffer
  bytes/row DrawChar
;

: sout          \ px py caddr len --
\ *G Write a string at the given pixel address, which is
\ ** stepped to the nearest byte boundary.
  bounds ?do			\ -- x y
    2dup i c@ .fchar
    swap /fwidth + swap
  loop
  2drop
;


\ *****************
\ *N Image handling
\ *****************
\ *P Before use, images created on a PC must be converted to
\ ** the format required by the particular LCD panel. There
\ ** are tools to do this in the *\i{Tools\BMPcnv} folder.

\ This code needs a rewrite!
\ If it ain't broke, don't fix it!

: load-image	{ x y imageaddr | %we %width %height %size %pad -- }
\ *G Display a graphics image in memory at imageaddr at pixel
\ ** position x,y. If part of the image is off the screen, the
\ ** displayed image is clipped to the screen size.
  0 to %we
  imageaddr dup @ to %height
  dup 1 cells + @ to %width 
  2 cells + @ to %size
  %width pix/row = if
    %size pix/byte / 0 do
      imageaddr 3 i + cells + @
      video_buffer i cells + !
    loop
    exit
  then
  imageaddr 3 cells + to imageaddr
  %size pix/byte * 			\ #pixels --
  %height /				\ #pixels/line --
  %width -				\ pad --
  to %pad
  %width x + pix/row > if 
    %width x + pix/row - pix/byte /  to %we 
    pix/row x - to %width 
  then
  %height y + pix/col > if 
    pix/col y - to %height
  then
  imageaddr
  video_buffer
  y bytes/row * + x pix/byte / +	\ *img *vid --
  %height 0 ?do
    %width pix/byte / 0 ?do
      over c@  over c!
      1+ swap 1+ swap
    loop
    %pad if
      over c@ over c@ and
      over c!
      1+ swap 1+ swap
    then
    pix/row %width %pad + - pix/byte / +
    swap %we + swap
  loop
  2drop
;


\ ======
\ *> ###
\ ======

decimal

