\ TSEPOS 2317 13 42 \

decimal

80 240 * 1000 + equ szVIDEO_BUFFER	\ Memory Size of Frame Buffer

PROGv
szVIDEO_BUFFER buffer: VIDEO_BUFFER
PROG

: init-lcd	\ --
  _PCSR @ %01000000 or _PCSR !  \ Enable LCD Clock in CControl
  $8A _LCD_MODE c!              \ Gray, 4-bit, Division, Single, InACTIVE
  $4F _LCD_BC c!                \ Byte Count = 80-1
  $01 _LCD_CP1W c!              \ CP1 Pulse Width = 2
  $EF _LCD_DUTY_7_0 c!          \ Duty Cycle 1/240
  %00 _LCD_DUTY_9_8 c!
  VIDEO_BUFFER _LCD_SADR1_7_0 c!                \ Video buffer Address
  VIDEO_BUFFER 8 rshift _LCD_SADR1_15_8 c!
  VIDEO_BUFFER 16 rshift _LCD_SADR1_23_16 c!
  VIDEO_BUFFER 24 rshift _LCD_SADR1_31_24 c!
  $EF _LCD_VLC1_7_0 c!          \ Set VLC1 = DUTY
  %00 _LCD_VLC1_9_8 c!
  $00 _LCD_VDLT c!              \ Video Memory Delta
  %01010101 _LCD_GRAY1 c!             \ ??
  %11101110 _LCD_GRAY2 c!             \ ??

  $00 _LCD_CLKDIV C!            \ Clock divisor
  $8B _LCD_MODE c!              \ Gray, 4-bit, Division, Single, ACTIVE
  clear-lcd                     \ Clear the screen
;

0       constant        COLOR_WHITE
1       constant        COLOR_LTGRAY
2       constant        COLOR_GRAY
3       constant        COLOR_BLACK

\ ***********************************************************************
\ * clear-lcd                                                           *
\ * Function:   Retrieves set background colour and fills the lcd with  *
\ *             that colour                                             *
\ * Recieves: None                                                      *
\ * Returns:  None                                                      *
\ ***********************************************************************

: clear-lcd             \ --
  back_color @ fill-lcd
;

\ ***********************************************************************
\ * fill-lcd                                                            *
\ * Function: Recieves a colour code and fills the lcd with that colour *
\ * Recieves: Colour                                                    *
\ * Returns:  None                                                      *
\ ***********************************************************************

: fill-lcd	\ Colour-code --
  $3 and                        \ %000000xy
  dup 2 lshift or               \ %0000xyxy
  dup 4 lshift or               \ %xyxyxyxy
  VIDEO_BUFFER szVIDEO_BUFFER rot fill
;

variable        color   COLOR_BLACK color !
variable        text_color -1 text_color !
variable        back_color color_white back_color !

: SetColor      \ c --
  color !
;

: SetBackColor  \ c --
  back_color !
;

: SetTextColor  \ c --
  dup 2 lshift or
  dup 4 lshift or
  dup 8 lshift or
  dup 16 lshift or
  text_color !
;

: PutPixel      \ x y --
  80 *                  \ x yofs --
  over 2 rshift +         \ x byteofs --
  swap 3 and            \ byteofs pixelofs --
  over VIDEO_BUFFER + c@ \ byteofs pixelofs byte --
  swap case
    0 of %00111111 and color @ 6 lshift or ( %11000000 or ) endof
    1 of %11001111 and color @ 4 lshift or ( %00110000 or ) endof
    2 of %11110011 and color @ 2 lshift or ( %00001100 or ) endof
    3 of %11111100 and color @          or ( %00000011 or ) endof
    drop
  end-case
  swap VIDEO_BUFFER + c!
;

: pixeloff

;

: h-line        \ x y len --
  over 0 239 within? not
  if drop 2drop exit then     \ line off screen

  dup 0< if                   \ negative length
    2 pick +                  \ xend y xstart --
    rot                       \ y xstart xend --
  else
    2 pick +                  \ xstart y xend --
    rot swap                  \ y xstart xend --
  then                        \ y xstart xend --

  over 319 > if drop 2drop exit then    \ line starts off screen

  dup 319 > if drop 319 then            \ clip to edge of screen
  over 0 < if nip 0 swap then           \ clip to start of screen

  over -                      \ y xstart len --
  0 ?do
  2dup swap putpixel 1 +
  loop
  2drop
;

: v-line        \ x y len --
  2 pick 0 319 within? not
  if drop 2drop exit then       \ line off screen

  dup 0< if
    over +                      \ x yend ystart --
    swap                        \ x ystart yend --
  else
    over +                      \ x ystart yend --
  then

  dup 239 > if drop 239 then    \ clip to bottom of screen
  over 0 < if nip 0 swap then   \ clip to top of screen

  over -                          \ x ystart len --
  0 ?do
    2dup putpixel 1 +
  loop
  2drop
;

: rect          \ x y len hiht --
  >r >r         \ x y  ; r: hiht len--
  dup 0 239 within? not if r> 2drop 2drop exit then
  over 0 319 within? not if r> 2drop 2drop exit then

  2dup r@ h-line        \ x y ; r: hiht len --
  over r@ + 1 - swap        \ x x+len-1 y -- ; r: hiht len--
  r> r> swap >r >r
  2dup r@ v-line      \ x x+len-1 y -- ; r: len hiht --
  r@ + 1 -
  r> r> swap >r >r
  dup rot swap r> negate 1 + h-line
  r> negate  1 + v-line
;

\ ----------------------------------------------------------------------------

interpreter
: font,         \ u32 -- ;
  dup #24 rshift $FF and        \ aabbccdd aa --
  over #8 rshift $FF00 and      \ aabbccdd aa bb?? --
  or                            \ aabbccdd bbaa --
  over #8 lshift $FF0000 and or \ aabbccdd ccbbaa --
  swap #24 lshift $FF000000 and or      \ ddccbbaa -- ; Flip32 complete
  ,c(t)
;
target

create font

\ space 32
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ ! 33
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ " 34
%00000000000000000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011000011000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
\ # 35
%00000000000000000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ $ 36
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000000111100000000001100000000 font,
%00000000001111111111000000000000 font,
%00000000000011111111110000000000 font,
%00000000110000000000111100000000 font,
%00000000111100000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ % 37
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000011110000000000000011110000 font,
%00001100001100000000001111000000 font,
%00001100001100000000111100000000 font,
%00000011110000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011110000000000000000 font,
%00000000001111000000001111000000 font,
%00000000111100000000110000110000 font,
%00000011110000000000110000110000 font,
%00001111000000000000001111000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ & 38
%00000000000000000000000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000001111111100000000000000 font,
%00000000001111111100000000000000 font,
%00000000111100001111000011110000 font,
%00000000111100000011110011110000 font,
%00000000111100000000001111000000 font,
%00000000111100000000111100000000 font,
%00000000001111111111111111110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ ' 39
%00000000000000000000000000000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
\ ( 40
%00000000000000000000000000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011110000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000001111000000000000 font,
%00000000000000000000000000000000 font,
\ ) 41
%00000000000000000000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000001111000000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011110000000000000000 font,
%00000000000000000000000000000000 font,
\ * 42
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000111100000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000111111111111111100000000 font,
%00000000111111111111111100000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ + 43
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000111111111111111100000000 font,
%00000000111111111111111100000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ , 44
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ - 45
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000111111111111111100000000 font,
%00000000111111111111111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ . 46
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000011111111000000000000 font,
%00000000000011111111000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ / 47
%00000000000000000000000000000000 font,
%00000000000000000000000000111100 font,
%00000000000000000000000011110000 font,
%00000000000000000000001111000000 font,
%00000000000000000000111100000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011110000000000000000 font,
%00000000001111000000000000000000 font,
%00000000111100000000000000000000 font,
%00000011110000000000000000000000 font,
%00001111000000000000000000000000 font,
%00111100000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 0 48
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000111111111111111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000111111000000 font,
%00000011110000000011001111000000 font,
%00000011110000001100001111000000 font,
%00000011110000110000001111000000 font,
%00000011110011000000001111000000 font,
%00000011111100000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111111111111111100000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 1 49
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111100000000000000 font,
%00000000001111111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 2 50
%00000000000000000000000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000000000000000000111100000000 font,
%00000000000000000000111100000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011110000000000000000 font,
%00000000001111000000001100000000 font,
%00000000111100000000001100000000 font,
%00000000111111111111111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,


\ 3 51
%00000000000000000000000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111111111110000000000 font,
%00000000111100000000111100000000 font,
%00000000110000000000111100000000 font,
%00000000000000000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000000011110000000000 font,
%00000000110000000000111100000000 font,
%00000000111100000000111100000000 font,
%00000000001111111111110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 4 52
%00000000000000000000000000000000 font,
%00000000000000000000111100000000 font,
%00000000000000000011111100000000 font,
%00000000000000001100111100000000 font,
%00000000000000110000111100000000 font,
%00000000000011000000111100000000 font,
%00000000001100000000111100000000 font,
%00000000110000000000111100000000 font,
%00000011000000000000111100000000 font,
%00000011111111111111111111000000 font,
%00000011111111111111111111000000 font,
%00000000000000000000111100000000 font,
%00000000000000000011111111000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 5 53
%00000000000000000000000000000000 font,
%00000011111111111111111111000000 font,
%00000011111111111111111111000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011111111111111110000000000 font,
%00000000111111111111111100000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111100000000001111000000 font,
%00000000001111111111111100000000 font,
%00000000000011111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 6 54
%00000000000000000000000000000000 font,
%00000000000000001111110000000000 font,
%00000000000000111111000000000000 font,
%00000000000011111100000000000000 font,
%00000000001111110000000000000000 font,
%00000000111111000000000000000000 font,
%00000011111111111111110000000000 font,
%00000011111111111111111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111111111111111100000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 7 55
%00000000000000000000000000000000 font,
%00000011111111111111111100000000 font,
%00000011111111111111111100000000 font,
%00000011110000000000111100000000 font,
%00000000000000000000111100000000 font,
%00000000000000000000111100000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ 8 56
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111100000000111100000000 font,
%00000000001111111111110000000000 font,
%00000000001111111111110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111100000000111100000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,


\ 9 57
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000111111111111111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111111111111111111000000 font,
%00000000000011111111111100000000 font,
%00000000000000000011111100000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000111111110000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ : 58
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ ; 59
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111111000000000000 font,
%00000000000000001100000000000000 font,
%00000000000000110000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ < 60
%00000000000000000000000000000000 font,
%00000000000000000000000011110000 font,
%00000000000000000000001111000000 font,
%00000000000000000000111100000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000001111000000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000000111100000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000000011110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ = 61
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000011111111111111111111000000 font,
%00000011111111111111111111000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000011111111111111111111000000 font,
%00000011111111111111111111000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ > 62
%00000000000000000000000000000000 font,
%00001111000000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000000111100000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011110000000000000000 font,
%00000000001111000000000000000000 font,
%00000000111100000000000000000000 font,
%00000011110000000000000000000000 font,
%00001111000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ ? 63
%00000000000000000000000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000111100000000 font,
%00000000000000000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ @ 64
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111111111100000000 font,
%00000000111100000000000000000000 font,
%00000000001111111111111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ A 65
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011111111111111111111000000 font,
%00000011111111111111111111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ B 66
%00000000000000000000000000000000 font,
%00001111111111111111111100000000 font,
%00001111111111111111111111000000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00000000111111111111111111000000 font,
%00000000111111111111111111000000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00001111111111111111111111000000 font,
%00001111111111111111111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ C 67
%00000000000000000000000000000000 font,
%00000000000011111111110000000000 font,
%00000000001111000000111100000000 font,
%00000000111100000000001111000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000000111100000000001111000000 font,
%00000000001111000000111100000000 font,
%00000000000011111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ D 68
%00000000000000000000000000000000 font,
%00001111111111111111110000000000 font,
%00001111111111111111111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000000011110000 font,
%00000011110000000000000011110000 font,
%00000011110000000000000011110000 font,
%00000011110000000000000011110000 font,
%00000011110000000000000011110000 font,
%00000011110000000000000011110000 font,
%00000011110000000000001111000000 font,
%00001111111111111111111100000000 font,
%00001111111111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ E 69
%00000000000000000000000000000000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00000011110000000000001111110000 font,
%00000011110000000011000000110000 font,
%00000011110000001111000000000000 font,
%00000011111111111111000000000000 font,
%00000011111111111111000000000000 font,
%00000011110000001111000000000000 font,
%00000011110000000011000000110000 font,
%00000011110000000000000011110000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ F 70
%00000000000000000000000000000000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00000011110000000000000011110000 font,
%00000011110000000011000000110000 font,
%00000011110000001111000000000000 font,
%00000011111111111111000000000000 font,
%00000011111111111111000000000000 font,
%00000011110000001111000000000000 font,
%00000011110000000011000000000000 font,
%00000011110000000000000000000000 font,
%00001111111100000000000000000000 font,
%00001111111100000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ G 71
%00000000000000000000000000000000 font,
%00000000000000111111000000000000 font,
%00000000000011111111110000000000 font,
%00000000001111000000111100000000 font,
%00000000111100000000001111000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000001111111111000000 font,
%00000011110000000000001111000000 font,
%00000000111100000000001111000000 font,
%00000000001111000000001111000000 font,
%00000000000011111111111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ H 72
%00000000000000000000000000000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011111111111111111111000000 font,
%00000011111111111111111111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ I 73
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000001111111111110000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000001111111111110000000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ J 74
%00000000000000000000000000000000 font,
%00000000000000111111111111000000 font,
%00000000000000111111111111000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ K 75
%00000000000000000000000000000000 font,
%00001111110000000000111100000000 font,
%00000011110000000011110000000000 font,
%00000011110000001111000000000000 font,
%00000011110000111100000000000000 font,
%00000011110011110000000000000000 font,
%00000011111111000000000000000000 font,
%00000011111111000000000000000000 font,
%00000011110011110000000000000000 font,
%00000011110000111100000000000000 font,
%00000011110000001111000000000000 font,
%00000011110000000011110000000000 font,
%00001111110000000000111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ L 76
%00000000000000000000000000000000 font,
%00001111111100000000000000000000 font,
%00001111111100000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000011110000000000000000110000 font,
%00000011110000000000000011110000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ M 77
%00000000000000000000000000000000 font,
%00001111000000000000000011110000 font,
%00001111110000000000001111110000 font,
%00001111111100000000111111110000 font,
%00001111001111000011110011110000 font,
%00001111000011111111000011110000 font,
%00001111000000111100000011110000 font,
%00001111000000000000000011110000 font,
%00001111000000000000000011110000 font,
%00001111000000000000000011110000 font,
%00001111000000000000000011110000 font,
%00001111000000000000000011110000 font,
%00001111000000000000000011110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ N 78
%00000000000000000000000000000000 font,
%00001111110000000000000011110000 font,
%00001111111100000000000011110000 font,
%00001111001111000000000011110000 font,
%00001111000011110000000011110000 font,
%00001111000000111100000011110000 font,
%00001111000000001111000011110000 font,
%00001111000000000011110011110000 font,
%00001111000000000000111111110000 font,
%00001111000000000000001111110000 font,
%00001111000000000000000011110000 font,
%00001111000000000000000011110000 font,
%00001111000000000000000011110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ O 79
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111100000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ P 80
%00000000000000000000000000000000 font,
%00001111111111111111111100000000 font,
%00001111111111111111111111000000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00000000111111111111111111000000 font,
%00000000111111111111111111000000 font,
%00000000111100000000000000000000 font,
%00000000111100000000000000000000 font,
%00000000111100000000000000000000 font,
%00001111111111110000000000000000 font,
%00001111111111110000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ Q 81
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000111111111111111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000110000001111000000 font,
%00000011110000001100001111000000 font,
%00000000111111111111111100000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000111100000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ R 82
%00000000000000000000000000000000 font,
%00001111111111111111111100000000 font,
%00001111111111111111111111000000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00000000111111111111111111000000 font,
%00000000111111111111111100000000 font,
%00000000111100000000001111000000 font,
%00000000111100000000000011110000 font,
%00000000111100000000000011110000 font,
%00001111111100000000000011110000 font,
%00001111111100000000000011110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ S 83
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000000000000000 font,
%00000000111100000000000000000000 font,
%00000000001111111100000000000000 font,
%00000000000000111111110000000000 font,
%00000000000000000000111100000000 font,
%00000000000000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111100000000111100000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ T 84
%00000000000000000000000000000000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00001111000000111100000011110000 font,
%00001100000000111100000000110000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000001111111111110000000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ U 85
%00000000000000000000000000000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111111111111111100000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ V 86
%00000000000000000000000000000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000000111100000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ W 87
%00000000000000000000000000000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000000000001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111100001111000000 font,
%00000011110000111100001111000000 font,
%00000000111111111111111100000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ X 88
%00000000000000000000000000000000 font,
%00111100000000000000000000111100 font,
%00001111000000000000000011110000 font,
%00000011110000000000001111000000 font,
%00000000111100000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000011110000000000001111000000 font,
%00001111000000000000000011110000 font,
%00111100000000000000000000111100 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ Y 89
%00000000000000000000000000000000 font,
%00000000111100000000111100000000 font,
%00000000111100000000111100000000 font,
%00000000111100000000111100000000 font,
%00000000111100000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000001111111111110000000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ Z 90
%00000000000000000000000000000000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00001111000000000000000011110000 font,
%00001100000000000000111100000000 font,
%00000000000000000000111100000000 font,
%00000000000000001111000000000000 font,
%00000000000000001111000000000000 font,
%00000000000011110000000000000000 font,
\ cr ." hello"
%00000000111100000000000000110000 font,
%00000000111100000000000011110000 font,
%00001111111111111111111111110000 font,
%00001111111111111111111111110000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ [ 91
%00000000000000000000000000000000 font,
%00000000000000011111111111100000 font,
%00000000000000011111111111100000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011110000000000000 font,
%00000000000000011111111111100000 font,
%00000000000000011111111111100000 font,
%00000000000000000000000000000000 font,

\ \ 92
%00000000000000000000000000000000 font,
%00111100000000000000000000000000 font,
%00001111000000000000000000000000 font,
%00000011110000000000000000000000 font,
%00000000111100000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000001111000000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000000111100000000 font,
%00000000000000000000001111000000 font,
%00000000000000000000000011110000 font,
%00000000000000000000000000111100 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ ] 93
%00000000000000000000000000000000 font,
%00001111111111110000000000000000 font,
%00001111111111110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00000000000011110000000000000000 font,
%00001111111111110000000000000000 font,
%00001111111111110000000000000000 font,
%00000000000000000000000000000000 font,

\ ^ 94
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ _ 95
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%11111111111111111111111111111111 font,
%11111111111111111111111111111111 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ ` 96
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000001111000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ a 97
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000000011110000000000 font,
%00000000000011111111110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011110011110000000000 font,
%00000000000000111100111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ b 98
%00000000000000000000000000000000 font,
%00000000000011111100000000000000 font,
%00000000000011111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111111111100000000 font,
%00000000000000111100001111000000 font,
%00000000000000111100000011110000 font,
%00000000000000111100000011110000 font,
%00000000000000111100000011110000 font,
%00000000000000111100001111000000 font,
%00000000000011110011110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ c 99
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ d 100
%00000000000000000000000000000000 font,
%00000000000011111100000000000000 font,
%00000000000011111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000111111111100000000000000 font,
%00000011110000111100000000000000 font,
%00001111000000111100000000000000 font,
%00001111000000111100000000000000 font,
%00001111000000111100000000000000 font,
%00000011110000111100000000000000 font,
%00000000001111001111000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ e 101
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111111111110000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ f 102
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000000000000000000 font,
%00000000111111111100000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000111111110000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ g 103
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100110000000000 font,
%00000000000011111111110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111110000000000 font,
%00000000000000000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,

\ h 104
%00000000000000000000000000000000 font,
%00000011111100000000000000000000 font,
%00000000111100000000000000000000 font,
%00000000111100000000000000000000 font,
%00000000111100000000000000000000 font,
%00000000111100111100000000000000 font,
%00000000111111111111000000000000 font,
%00000000111111000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000011111100000011110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ i 105
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000011111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ j 106
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,

\ k 107
%00000000000000000000000000000000 font,
%00000000111111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111001111000000000000 font,
%00000000001111111100000000000000 font,
%00000000001111111100000000000000 font,
%00000000001111001111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111111000011110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ l 108
%00000000000000000000000000000000 font,
%00000000000011111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ m 109
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000011110000000011000000000000 font,
%00000011111111001111110000000000 font,
%00000011110011111100111100000000 font,
%00000011110000110000111100000000 font,
%00000011110000110000111100000000 font,
%00000011110000110000111100000000 font,
%00000011110000110000111100000000 font,
%00000011110000110000111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ n 110
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000111100111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ o 111
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ p 112
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000111111001111000000000000 font,
%00000000001111111111110000000000 font,
%00000000001111000000111100000000 font,
%00000000001111000000111100000000 font,
%00000000001111000000111100000000 font,
%00000000001111000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000001111111111000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000111111110000000000000000 font,

\ q 113
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000011110011111100000000 font,
%00000000001111111111110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000011111111100000000 font,

\ r 114
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000111100111100000000000000 font,
%00000000001111001111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000111111110000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ s 115
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ t 116
%00000000000000000000000000000000 font,
%00000000000011000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000111111111100000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ u 117
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011110011110000000000 font,
%00000000000000111100111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ v 118
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ w 119
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100000011110000000000 font,
%00000000111100110011110000000000 font,
%00000000111100110011110000000000 font,
%00000000111100110011110000000000 font,
%00000000111100110011110000000000 font,
%00000000001111001111000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ x 120
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000111100000000111100000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011111111000000000000 font,
%00000000001111000011110000000000 font,
%00000000111100000000111100000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ y 121
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000011111111110000000000 font,
%00000000000000000011110000000000 font,
%00000000001111000011110000000000 font,
%00000000000000111100000000000000 font,

\ z 122
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000001111111111110000000000 font,
%00000000001100000011110000000000 font,
%00000000000000001111000000000000 font,
%00000000000000111100000000000000 font,
%00000000000011110000000000000000 font,
%00000000001111000000110000000000 font,
%00000000001111111111110000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ { 123
%00000000000000000000000000000000 font,
%00000000000000001111110000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000001111000000000000000000 font,
%00000000001111000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000001111110000000000 font,
%00000000000000000000000000000000 font,

\ | 124
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

\ } 125
%00000000000000000000000000000000 font,
%00000000001111110000000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000000011110000000000 font,
%00000000000000000011110000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000000000111100000000000000 font,
%00000000001111110000000000000000 font,
%00000000000000000000000000000000 font,

\ ~ 126
%00000000000000000000000000000000 font,
%00000000001111000011000000000000 font,
%00000000111111111111000000000000 font,
%00000000110000111100000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,
%00000000000000000000000000000000 font,

: v@		\ addr -- 32bit
  dup c@
  over 1 + c@ 8 lshift or
  over 2 + c@ 16 lshift or
  swap 3 + c@ 24 lshift or

;

: v!		\ 32bit addr--
  dup v@ rot swap over invert and
          \ addr char masked --
  swap text_color @
          \ addr masked char color --
  and or swap
          \ color-char addr --
  2dup c!
  1 + over 8 rshift over c!
  1 + over 16 rshift over c!
  1 + swap 24 rshift swap c!
;

: .fchar        \ x y ascii --

  32 -           \ x y asciiofs --
  cells 16 *     \ x y charofs --
  font +        \ x y font --
  swap 80 * rot 4 / + VIDEO_BUFFER +

  16 0 do       \ font ptr --
    over @ over v!
    80 + swap 1 cells + swap
  loop
  2drop
;

: sout          \ x y addr len --
  >r rot
  rot r> \ addr x y len --
  0 ?do
    2 pick c@   \ addr x y char --
    2 pick 2 pick
                \ addr x y char x y --
    rot .fchar
    swap 16 + swap
    rot 1 chars + rot rot
  loop
  drop 2drop
;

: button        \ x y len hiht saddr slen --
  swap >r >r over r@ 16 * < abort" String too long to fit in button"
                \ x y len hiht -- ; r: saddr slen --
  >r 2 pick 2 pick 2 pick r@ rect
                \ x y len -- ; r: saddr slen hiht --
  r> rot swap 2 / +
                \ x len y0 -- ; r: saddr slen --
  rot rot 2 / + \ y0 x0 -- ; r: saddr slen --
  swap
  8 -           \ x0 y0-8  -- ; r: saddr slen --
  swap          \ y0-8 x0 -- ; r: saddr slen --
  r@ 8 * -      \ y0-8 x0-8slen -- ; r: saddr slen --
  swap
  r> r> swap    \ x y saddr slen --
  sout

;

: -leading
  bl skip
;

: (sout)
  2swap 16 * swap 16 * swap 2swap
  \ x y c-addr u
  sout
;

: (.fchar)
  rot 16 * rot 16 * rot .fchar
;

: nInter        \ c-addr u -- n ; Gets number of inter-words in a string
  0 swap               \ c-addr n u --
  0 ?do                \ c-addr n --
    over c@ 32 = if 1+ then
    swap 1+ swap
  loop
  nip
;

: wout          \ x y c-addr -- l ; Prints next token and returns its length
  0 >r                  \ x y c-addr -- ; r: len --
  begin dup c@ 32 <>    \ x y c-addr flag -- ; r: len --
  while                 \ x y c-addr -- ; r: len --
  dup c@                \ x y c-addr char -- ; r: len --
  3 pick 3 pick rot (.fchar)
                        \ x y c-addr -- ; r: len --
  1+                    \ x y 'c-addr -- ; r: len --
  r> 1+ >r              \ x y 'c-addr -- ; r: 'len --
  rot 1+ rot rot        \ 'x y 'c-addr -- ; r: 'len --
  repeat
  drop 2drop
  r>
;

: JustifyPrint	\ x y w c-addr u -- ; Fully justified text out!!!!
2dup nInter dup >r      \ x y w c-addr u nInter -- ; r: nInter --
rot swap 2swap -        \ x y c-addr nInter nExtra -- ; r: nInter --
over +                  \ x y c-addr nInter nTot -- ; r: nInter --
swap /                  \ x y c-addr spaces\word -- ; r: nInter --
1 MAX
r> 1+ 0 ?do
  3 pick 3 pick 3 pick wout
                        \ x y c-addr spaces\word wlen --
  swap >r               \ x y c-addr wlen -- ; r: spaces --
  rot r> swap >r >r     \ x c-addr wlen -- ; r: y spaces --
  swap over             \ x wlen c-addr wlen -- ; r: y spaces --
  + 1+                  \ x wlen 'caddr -- ; r: y spaces --
  -rot +                \ 'caddr 'x -- ; r: y spaces --
  r@ +                  \ 'caddr 'x -- ; r: y spaces --
  r> r> swap >r         \ 'caddr 'x y -- ; r: spaces --
  rot                   \ 'x y 'caddr -- ; r: spaces --
  r>                    \ 'x y 'caddr spaces --
  loop
  2drop 2drop
;

256 constant MAX_PATH
MAX_PATH buffer:	textline-out

[undefined] c+! [if]
: c+!		\ n c-addr --
  tuck c@ + swap c!
;
[then]

: cu$+		\ c-addr u $ --
  2dup 2>r count + swap move
  2r> c+!
;

: NextLine	\ w c-addr u -- 'c-addr u ; New string in textline-out
  0 textline-out c!		\ reset output line
  begin
    textline-out c@ 3 pick <		\ whilst text line is < width
    over 0<> and            		\ AND string has more text
  while
    2dup 2>R				\ remember original string in case we overflow
    -leading
    2dup bl scan			\ w c-addr u rcaddr ru --
    2swap 2 pick -			\ w rcaddr ru c-addr u-ru --

    dup textline-out c@ +		\ w rcaddr ru c-addr u-ru newlen! --
    5 pick <= if
	    textline-out cu$+		\ w rcaddr ru --
	    S"  " textline-out cu$+	\ and add a space dude
	2r> 2drop
    else
	2drop 2drop drop 2r> exit
    then
  repeat
  rot drop				\ remaining c-addr u :
;

: prettyprint	\ x y w h c-addr u -- ; XYWH are in character units

  rot 4 pick + 1- -rot		\ modify H to become Y limit

  \ cr 3 pick 0 ?do [char] * emit loop

  -leading			\ strip leading space

  begin
    ?dup
  while
    3 pick -rot		\ x y w h c-addr u w c-addr u --
    NextLine			\ x y w h 'c-addr 'u -- ; next line is in textline buffer
    2>R 2over 3 pick		\ x y w h x y w -- ; R: c-addr u --
    textline-out
    count -trailing JustifyPrint 	\ x y w h -- ; R: c-addr u --
    rot 1+ -rot 2r>		\ x next-y w h c-addr u --
    4 pick 3 pick > if 2drop 2drop 2drop exit then
  repeat
  2drop 2drop drop
;

: 4Dup		\ a b c d -- a b c d a b c d
  2over 2over ;

: 3D-Button	\ x y len hiht saddr slen --
  3 pick 16 / 1- swap min 0 max swap >r >r      \ x y l h -- ; R: saddr slen --
  COLOR_BLACK SetColor
  4dup rect
  COLOR_LTGRAY SetColor
  3 pick 5 + 3 pick 5 + 3 pick 10 - 3 pick 10 - Rect
  COLOR_WHITE SetColor
  3 pick 1 + 3 pick 1 + 2 pick 2 - v-line
  3 pick 2 + 3 pick 1 + 2 pick 3 - v-line
  3 pick 3 + 3 pick 1 + 2 pick 4 - v-line
  3 pick 4 + 3 pick 1 + 2 pick 5 - v-line
  3 pick 1 + 3 pick 1 + 3 pick 2 - h-line
  3 pick 1 + 3 pick 2 + 3 pick 3 - h-line
  3 pick 1 + 3 pick 3 + 3 pick 4 - h-line
  3 pick 1 + 3 pick 4 + 3 pick 5 - h-line
  COLOR_GRAY SetColor
  3 pick 2 pick + 2 - 3 pick 1 + 2 pick 2 - v-line
  3 pick 2 pick + 3 - 3 pick 2 + 2 pick 3 - v-line
  3 pick 2 pick + 4 - 3 pick 3 + 2 pick 4 - v-line
  3 pick 2 pick + 5 - 3 pick 4 + 2 pick 5 - v-line
  3 pick 1 + 3 pick 2 pick + 2 - 3 pick 2 - h-line
  3 pick 2 + 3 pick 2 pick + 3 - 3 pick 3 - h-line
  3 pick 3 + 3 pick 2 pick + 4 - 3 pick 4 - h-line
  3 pick 4 + 3 pick 2 pick + 5 - 3 pick 5 - h-line

  COLOR_LTGRAY SetColor
  dup 12 - 0 do
    3 pick 6 + 3 pick i + 6 + 3 pick 12 - h-line
  loop

  \ x y l h -- ; R: saddr slen --
  2 / rot +             \ x l h/2+y -- ; R: saddr slen --
  swap 2/ rot +         \ y0 x0 -- ; R: saddr slen --
  swap 8 - swap         \ y0-8 x0 -- ; R: saddr slen --
  r@ 8 * -
  swap r> r> swap
  COLOR_BLACK SetTextColor sout
;

: test-3d
0 0 200 100 s" This string is far too long to fit in this button" 3d-button
;

: Screen        \ caddr1 u1 caddr2 u2 caddr3 u3 caddr4 u4 caddrb ub --
  clear-lcd
  color_black setcolor
  0 48 320 144 rect
  5 53 310 134 rect
    1   4  18  7 2rot prettyprint
  160 200 160 40 2rot 3D-Button
    0 200 160 40 2rot 3D-Button
  160   0 160 40 2rot 3D-Button
    0   0 160 40 2rot 3D-Button
;

\ =============================================================================

interpreter

variable        menu-define-state       \ 0 = not doing a menu
                menu-define-state off   \ 1 = doing button 1
                                        \ 2...4 doing button X
                                        \ 5 doing body text
                                        \ 6 complete
: $place                \ $ --
  count dup c,
  bounds ?do
    i c@ c,
  loop
;

: MENU          \ "name" --
  menu-define-state @(h) abort" Already defining a menu"
  create
    here cell+ ,
    1 menu-define-state !(h)            \ flag new defining state
  does>
    @
;

: END-MENU      \ --
  menu-define-state @(h) 6 <> abort" Incomplete menu definition"
  0 menu-define-state !(h)
;

: BUTTON"       \ <"text"> --
  menu-define-state @(h) 0= abort" Not defining a menu when using BUTTON"
  menu-define-state @(h) 4 > abort" Attempt to define too many buttons"
  [char] " word $place align
  menu-define-state @(h) 1+ menu-define-state !(h)
;

: RUNS          \ <"name"> --
  menu-define-state @(h) 2 5 within? not abort" RUNS must follow BUTTON within a MENU"
  ' ,
;

: BODY"         \ <"text"> --
  menu-define-state @(h) 5 <> abort" MENU BODY text out of place"
  [char] " word $place align
  menu-define-state @(h) 1+ menu-define-state !(h)
;

target

: +item		\ addr -- caddr len addr'
  count 2dup + aligned cell+    \ c-addr1 u1 nextptr --
;

: ShowMenu      \ menu-id --
  +item +item +item +item
  count Screen
;

: ActOnMenu     \ menu-id button-pressed --
  1-                            \ get into range 0..3
  0 ?do
    count + aligned cell+       \ skip over N buttons in stuct
  loop
  count + aligned @             \ grab XT
  execute                       \ and run it!
;

\ ===========================================================================


: test1 cr ." ONE" ;
: test2 cr ." TWO" ;
: test3 cr ." THREE" ;
: test4 cr ." FOUR" ;



MENU    secondmenu
        BUTTON" Button2.1" RUNS test1
        BUTTON" Button2.2" RUNS test2
        BUTTON" Button2.3" RUNS test3
        BUTTON" Button2.4" RUNS test4
        BODY" Second Menu Selection"
END-MENU

: menu2
  secondmenu showmenu ;

MENU    mainmenu

        BUTTON" Menu 2"  RUNS menu2
        BUTTON" Button 2" RUNS test2
        BUTTON" Button 3"  RUNS test3
        BUTTON" Button 4" RUNS test4
        BODY" Press a button and see some stuff"

END-MENU


: tt
  mainmenu ShowMenu
;

: rt
  mainmenu swap ActOnMenu
;

((
: setpixel \ x y -- x and y are from origin of phone screen
  2* 55 + swap 2* 64 + swap	\ xofs yofs --
  2dup putpixel
  over 1+ over putpixel
  2dup 1 + putpixel
  1+ swap 1+ swap putpixel
;

: clearpixel
  color @ >r
  color_white setColor setpixel  
  r> color !
;

: load-image { x y imageaddr | %we %width %height %size %pad -- }
  0 to %we
  imageaddr dup @ to %height
  dup 1 cells + @ to %width 
  2 cells + @ to %size
\  %size $4B00 = if 
  %width 320 = if
\    %height 240 = if 0 to y then 
\    clear-lcd
\   4800 0 do
\    imageaddr video_buffer 80 y * +
    %size 4 / 0 do
      imageaddr 3 i + cells + @
      video_buffer i cells + !
    loop
 \   2drop
    exit
  then
  imageaddr 3 cells + to imageaddr
  %size 4 * 				\ #pixels --
  %height /				\ #pixels/line --
  %width -				\ pad --
  to %pad
  %width x + 320 > if 
    %width x + 320 - 4 /  to %we 
    320 x - to %width 
  then
  %height y + 240 > if 
    240 y - to %height
  then
  imageaddr
  video_buffer y 80 * + x 4 / +		\ *img *vid --
  %height 0 ?do
    %width 4 / 0 ?do
      over c@  over c!
      1+ swap 1+ swap
    loop
    pad 0 <> if
      over c@ over c@ and
      over c!
      1+ swap 1+ swap
    then
    320 %width %pad + - 4 / +
    swap %we + swap
  loop
  2drop
;

: fillscreen
  65 0 do
    i 96 0 do i over setpixel loop
    drop
  loop
;

: clearscreen
  65 0 do
    i 96 0 do i over clearpixel loop
    drop
  loop
;
))

((
: init-lcd
  init-lcd
  start-timers
  0 0 nok7110 load-image
  color_ltgray setcolor
  63 54 194 132 rect
  color_gray setcolor
  85 60 mpe load-image
  3000 ms
  clearscreen
;
))

((
: demo
  init-lcd
  start-timers
  clear-lcd
  begin
    80 50 mpe load-image
    4 9 10 3 "" More Real .......... Less Time" count prettyprint	  
    0 320 do
      i 0 demo1 load-image
      10 ms
    -4 +loop
    1000 ms
    0 12 20 3 "" Welcome to the MPE Sharp ARM7 evaluation board demo." count prettyprint
    3000 ms
    clear-lcd
    0 0 image1 load-image
    3000 ms
    clear-lcd
    0 0 Fth6 load-image
    3000 ms
    clear-lcd
    35 100 sharp load-image
    0 9 20 6 "" Powered by the low-cost, low-power Sharp ARM7 Micro. A comprehensive ARM7 evaluation kit" count prettyprint
    6000 ms
    clear-lcd
    0 0 phone load-image
    0 9 20 6 "" The Sharp Arm7 is ideal for mobile communications .." count prettyprint
    3000 ms
    clear-lcd
    10 10 beat load-image
    1000 ms
    110 10 beat load-image
    1000 ms
    210 10 beat load-image
	 0 6 20 8 "" .. for medical instrumentation .." count prettyprint
    1000 ms
    10 160 beat load-image
    1000 ms
    110 160 beat load-image
    1000 ms
    210 160 beat load-image
    1000 ms
    clear-lcd
    0 0 20 4 "" .. even for vending machines!" count prettyprint 
    97 70 bottle load-image
    0 11 20 4 "" The ARM7 is suitable for almost any application" count prettyprint
    3000 ms
    0 0 stack load-image
    4000 ms
    0 0 final load-image
    6000 ms
    clear-lcd
  key? until
  clear-lcd
;
))



