\ Motion.fth - vending machine motion control

((
Control software for Cosmic beverage machine model CT01.   
   
(c) 2005   
   
Vallid Ltd   
15 Brookside Centre,   
Temple Farm Industrial Estate,   
Southend. ESSEX.   
SS2 5RR   
tel: 01702 612562   
   
and   
   
Drassington Ltd 
   
Written in FORTH with compilers supplied by:-   
   
MicroProcessor Engineering Ltd.   
133 Hill Lane, Southampton,   
SO1 5AF.   
   

To Do
=====   
Change History   
==============
20050505 JNW001 First cut
))

((
\ sachet interface pcb
: .su1 $040 read_8574 . ;
: .su2 $042 read_8574 . ;

: >su1 $040 write_8574  ;
: >su2 $042 write_8574  ;
 
\ clamp pcb
: .cu4 $044 read_8574 . ;
: >su4 $044 write_8574  ;

\ slide pcb
: .tu4 $046 read_8574 . ;
: >tu4 $046 write_8574  ;

\ main control pcb
: .mu5 $048 read_8574 . ;
: .mu7 $04a read_8574 . ;
: .mu8 $04c read_8574 . ;
: >mu5 $048 write_8574  ;
: >mu7 $04a write_8574  ;
: >mu8 $04c write_8574  ;

\ repeat read loops
: .rsu1 hex begin .su1 key? until ;
: .rsu2 hex begin .su2 key? until ;
: .rcu2 hex begin .cu4 key? until ;
: .rtu2 hex begin .tu4 key? until ;
: .rmu5 hex begin .mu5 key? until ;
: .rmu7 hex begin .mu7 key? until ;
: .rmu8 hex begin .mu8 key? until ;


: fridgetemp	\ -- w
\ *G Returns a temperature in half-degree units.
  $95 readtemp0.06
;

: .temp2 fridgetemp . ;

: tanktemp	\ -- w
\ *G Returns a temperature in half-degree units.
  Temp75 $093 read_reg16_75 7 rshift
;

: .temp1 tanktemp . ;
))



\ *********************
\ *S Sachet orientation
\ *********************

0 value TiltPos	\ --
\ *G Contains a indicator of the sachet orientation, zero
\ ** indicating unknown.
  1 equ TiltUp
  2 equ TiltDn

: Up?		\ -- 
\ *G True if tilted up.
  tilted_up ibit? 0=
;

: toUp		\ --
\ *G Motor will move until it gets there.
  tilted_up iBit? if
    tilt ibitset
    begin  #10 ms  Up? until
  endif
  TiltUp -> TiltPos
;

: dn?		\ -- 
\ *G True if tilted down.
  tilted_dn ibit? 0=
;


: toDn		\ --
\ *G Motor will move until it gets there.
  tilted_dn iBit? if
    tilt ibitclr
    begin  #10 ms  dn? until
  endif
  TiltDn -> TiltPos
;


\ *****************
\ *S Clamp position
\ *****************

0 value ClampPos
\ *G Contains a indicator of where the clamp is, zero
\ ** indicating unknown.
  1 equ ClampClosed
  2 equ ClampOpen

#-379 cve ClampRecvCnt
#-500 cve ClampPierceCnt

: closeClamp	\ --
  ClampClosed ClampPos <> if
    clamping ibitset  move_clamp ibitset
    begin
      #10 ms  Tacho1 ClampRecvCnt <=
    until
    move_clamp ibitclr
    ClampClosed -> ClampPos
  endif
;

: openClamp	\ --
  clamp_open ibit? if			\ 0=open
    clamping ibitclr  move_clamp ibitset
    begin
      #10 ms  clamp_open ibit? 0=
    until
    move_clamp ibitclr
  endif
  0 -> Tacho1
  ClampOpen -> ClampPos
;

: PierceSachet	\ --
  clamping ibitset  move_clamp ibitset
  begin
    #10 ms  Tacho1 ClampPierceCnt <=
    CycleFault? or
  until
  move_clamp ibitclr
;
  


\ ******************
\ *S Fridge movement
\ ******************

: FridgeParked? 	\ -- flag
  ft_sw2  ibit? 0=
;

: FridgeDeployed?	\ -- flag
  ft_sw1  ibit? 0=
;


: ParkFridge	\ --
\ *G Move fridge out of way
  ft_r ibitset 
  begin
    #10 ms  FridgeParked?
  until
;

: DeployFridge	\ --
\ *G Put fridge in dispensing position.
  ft_r ibitclr
  begin
    #10 ms  FridgeDeployed?
  until
;


\ *****************
\ *S Slide position
\ *****************

0 value SlidePos	\ -- n
\ *G Contains an indicator of where the slide is, zero
\ ** indicating unknown.
  1 equ SlideFront	\ at front of machine (dispense)
  2 equ SlideReceive	\ at sachet position (wait for sachet)
  3 equ BinFront	\ at front of disposal bin
  4 equ BinBack		\ at back of disposal bin
  5 equ SlideBack	\ at back of machine

: atFront?	\ -- n
\ *G non-zero if slide at front.
  at_front ibit? 0=
;

: atBack?	\ -- n
\ *G non-zero if slide at back.
  at_back ibit? 0=
;

: toFront	\ --
  toDn					\ sachet must be down
  at_front ibit? if			\ 0=at front
    to_back ibitclr  move_slide ibitset
    begin
      #10 ms  at_front ibit? 0=
    until
    move_slide ibitclr
  endif
  SlideFront -> SlidePos
;

: StartBack	\ --
  to_back ibitset  move_slide ibitset
;

: StartFront	\ --
  to_back ibitclr  move_slide ibitset
;

: StopSlide	\ --
  move_slide ibitclr
;

: toBack	\ --
  at_back ibit? if			\ 0=at back
    StartBack
    begin
      #10 ms  at_back ibit? 0=
    until
    StopSlide
  endif
  SlideBack -> SlidePos
;

: toHome		\ --
\ *G Sachet up, at back, clamp open
  ParkFridge
  tilt ibitset				\ up
  clamping ibitclr  move_clamp ibitset  \ open
  to_back ibitset  move_slide ibitset	\ back
  begin
    #10 ms
    at_back ibit? 0=
    clamp_open ibit? 0= and
    tilted_up ibit? 0= and
  until
  move_clamp ibitclr  move_slide ibitclr
  TiltUp -> TiltPos
  ClampOpen -> ClampPos
  SlideBack -> SlidePos
  0 -> Tacho1
;


\ *****************
\ *S Vend movements
\ *****************

: AtOptic?	\ -- flag
\ Return true if at optic.
  _PPI_PB c@ bit3 and 0=
;

: WaitOptic	\ --
\ *G Wait until we hit the optic - Receive, Bin start or Bin finish.
  begin  AtOptic? 0= while  #10 ms  repeat
;

: WaitOffOptic	\ --
\ *G Wait until are off the optic
  begin  AtOptic? while  #10 ms  repeat
;

: NextOptic	\ --
\ *G Go from one optic to the next.
  WaitOffOptic WaitOptic
;

: Arrived	\ posid --
\ *G Stop slide and mark position.
  StopSlide  -> SlidePos
;

: binfront>receive	\ --
  StartFront  NextOptic  StopSlide
  SlideReceive -> SlidePos
;

: front>binfront	\ --
  StartBack
    WaitOptic NextOptic
  BinFront arrived
;

: binback>receive	\ --
  StartFront  NextOptic NextOptic  SlideReceive arrived
;

: back>receive	\ --
  StartFront  WaitOptic NextOptic NextOptic  SlideReceive arrived
;

: receive>binfront	\ --
  StartBack  NextOptic  SlideReceive arrived
;

((
: toXX	\ --
  case  SlidePos
    SlideFront   of  endof
    SlideReceive of  endof
    BinFront     of  endof
    BinBack      of  endof
    SlideBack    of  endof
      toBack 
  endcase
;
))

: toReceive	\ --
  case  SlidePos
    SlideFront   of  front>binfront toUp binfront>receive  endof
    SlideReceive of
      Up? 0= if
        receive>binfront toUp binfront>receive
      endif
    endof
    BinFront     of  toUp binfront>receive  endof
    BinBack      of  toUp binback>receive  endof
    SlideBack    of  toUp back>receive  endof
      toBack toUp back>receive
  endcase
;

: toVend
  case  SlidePos
    SlideFront   of
      dn? 0=
      if  front>binfront endif
    endof
    SlideReceive of
      Dn? 0=
      if  receive>binfront  endif
    endof
    BinFront     of  endof
    BinBack      of  endof
    SlideBack    of  endof
      tohome
  endcase
  toDn toFront
;

: binback>binfront	\ --
  StartFront  WaitOffOptic NextOptic  BinFront arrived
;

: back>binfront	\ --
  StartFront  NextOptic NextOptic  BinFront arrived
;

: RandBack	\ --
\ *G At BINFRONT position, move a random distance backwards
\ ** within the range NINFRONT to BINBACK - ??.
  Startback
  ticks tick-ms / 3 and  #300 *  #150 + ms
  StopSlide
;
  
: DumpSachet	\ --
\ *G Dump sachet in waste bin
  openClamp  #1000 ms
;

: DoDump	\ --
  toDn  RandBack  DumpSachet  toUp
  BinBack -> SlidePos
;

: toBinfront	\ --
  case  SlidePos
    SlideFront   of  front>binfront  endof
    SlideReceive of  receive>binfront  endof
    BinFront     of  endof
    BinBack      of  binback>binfront  endof
    SlideBack    of  back>binfront  endof
      toBack  back>binFront
  endcase
;

: toDump	\ --
  toBinfront doDump
;


: Go?		\ -- flag
\ *G Return true if go button active.
  _PPI_PA c@ bit2 and 0=
;

: WaitGo	\ --
  begin  #10 ms  Go? until
;

: WaitNoGo	\ --
  begin  #10 ms  Go? 0= until
;


\ ****************
\ *S Abbreviations
\ ****************

: Lost!		\ --
\ *G We don't know where we are!
  0 -> TiltPos  0 -> ClampPos  0 -> SlidePos  
;

: up		( -- )  toUp  ;
: dn		( -- )  toDn  ;
: cc		( -- )  closeClamp  ;
: oc		( -- )  openClamp  ;
: fr 		( -- )  toFront  ;
: bk		( -- )  toBack  ;
: dance		( -- )  cc fr dn oc fr up cc bk  ;
: home		( -- )  toHome  ;

