\ UCN5821.FTH - 8 bit serial latch driver

((
Copyright (c) 2005
MicroProcessor Engineering Ltd
133 Hill Lane
Southampton
SO1 5AF
England
tel: +44 (0)23 80 631441
fax: +44 (0)23 80 339691


Port_C is all o/ps, initialised by the I2C driver.

CLK	PC.0
DIN	PC.1
ST	PC.4
OE	PC.5

This file must be loaded AFTER the I2C driver
))

decimal

\ **************
\ *S Port driver
\ **************
\ *P Outputs are driven by the Port C bit access system. Bits are
\ ** defined for this, not as bit masks.

\ compiler
: setLO		\ bit#*2 --
\ *G Set the specified bit low. There is inverter on each output.
  1 or  _PPI_CTLR c!
;

: setHI		\ bit#*2 --
\ *G Set the specified bit high. There is inverter on each output.
  _PPI_CTLR c!
;
\ target

$00 equ clk5821	\ -- bit#*2
$02 equ din5821	\ -- bit#*2
$08 equ st5821	\ -- bit#*2
$0A equ oe5821	\ -- bit#*2

: En5821	\ --
\ *G Enable 5821 o/ps
  oe5821 setLO
;

: Dis5821	\ --
\ *G Disable 5821 o/ps
  oe5821 setHI
;

: Write5821	\ b --
\ *G Write the given data byte to the 5821, msb first, and
\ ** strobe the data in.
  8 0 do
    dup $80 and				\ test ms bit
    if  din5821 setHI  else  din5821 setLO  endif
    clk5821 setHI  clk5821 setLO	\ data latches at lo->hi edge
    1 lshift
  loop
  drop
  st5821 setHI  st5821 setLO		\ strobe data line
;


\ *************
\ *S I/O driver
\ *************
\ *P Because this is an output only device, a shadow variable
\ ** contains the current device state.

variable state5821	\ -- addr
\ *G Holds the required state of the I/O bits controlled by
\ ** the 5821.

: set		\ bitmask --
\ *G Set the set bits in bitmask in the shadow register.
  state5821 or!
;

: clr		\ bitmask --
\ *G Clear the set bits in bitmask in the shadow register.
  state5821 bic!
;

: setIO		\ --
\ *G Write the shadow register to the device.
  state5821 @ write5821
;

: Init5821	\ --
\ *G Initialise the 5821
  oe5821 setHI				\ disabled
  clk5821 setLO				\ data latches at lo->hi edge
  st5821 setLO				\ inactive
  state5821 off  setIO  en5821		\ initialise outputs
;  ' Init5821 AtCold



