\ Startup Code for the Boot Loader in the MPE ARM7 Development Kit

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE


1) Sets the PPI controller for PortA/B as input and PortC as output.
2) Configures Flash as Segment1 mapped in high memory.
3) Maps external SRAM from address 0
4) Enables internal SRAM as combined cache
5) Configures Ethernet Chip Select Area
6) Copies vector table from Flash to Ram:0000

The code is set up to run in a 48k section of Flash
  $1000000 $100BFFF
The boot code remaps the chip select unit and segment mapper to put:
  1Mb Flash	at 0100:0000 to 010F:FFFF	Segment 1	read/write, not cached
  512k RAM	at 0000:0000 to 0007:FFFF	Segment 2	read/write, cached
  2k local SRAM	at 0000:0000 to 6000:07FF	Cache mode
  1k Ethernet	at 5000:0000 to 5000:03FF	Segment 4	read/write, not cached
The vector table is then copied to address 0.
))


aif? [if]

\ Use default Reset handler in INITARM.FTH (branches to ECLD)
\ because the AIF loader will have done all the CPU specific
\ set up.

[else]

PROG sec-base   equ FLASH_START		\ First address in FLASH.
FLASH_START 1024 1024 * + 1+ equ FLASH_STOP
\ PROG sec-top 1+ equ FLASH_STOP	\ Next address past end of FLASH.

\ To understand this code, you will need the LH77790 User manual,
\ section 5.
asmcode
L: HandlerReset
\ Disable all interrupt sources.
\ The watchdog timer is disabled by a hardware reset.
  mov     r0, # 0
  mvl     r1, _FIQER    str r0, [ r1 ]
  mvl     r1, _IRQER    str r0, [ r1 ]

\ Set up PPI (8255 clone)
  mov r0, # $80         adr r1, _PPI_CTLR   strb r0, [ r1 ]
  mov r0, # $00         adr r1, _PPI_PC     strb r0, [ r1 ]

\ Set up Flash as Segment 1 using CS0, 8 bit, 3 waits
  adr r0, $3003         adr r1, _BCR1       str r0, [ r1 ]
\ Read Write, non-cacheable, 32 bit, Bank 1
  adr r0, $7802         adr r1, _SDR1       str r0, [ r1 ]
\ set start and end addresses of segment 1
  adr r0, FLASH_START   adr r1, _START1     str r0, [ r1 ]
  adr r0, FLASH_STOP    adr r1, _STOP1      str r0, [ r1 ]

  b InFLASH

L: InFLASH
\ Set RAM as segment 2 using CS3=HI CS2=LO , 16 bit, 1 wait
  adr r0, $9090         adr r1, _BCR2       str r0, [ r1 ]
\ read/write, cachable, 32 bit, Bank 2
  adr r0, $7C04         adr r1, _SDR2       str r0, [ r1 ]
\ 512k from $00000 to $80000
  adr r0, $00000        adr r1, _START2     str r0, [ r1 ]
\  adr r0, $80000        adr r1, _STOP2      str r0, [ r1 ]
  adr r0, $100000        adr r1, _STOP2      str r0, [ r1 ]

\ Local SRAM enabled, mapped at 6000:0000 to 6000:07FF
  mov r0, # $03         adr r1, _LSCR       str r0, [ r1 ]
\ Enable local SRAM as Cache Mode
  mov r0, # $01         adr r1, _CCR        str r0, [ r1 ]

\ Set Ethernet as segment 4 using CS1, 8 bit, 4 wait
  adr r0, %0100000000001100             adr r1, _BCR4       str r0, [ r1 ]
\ Setup Segment4 as read/write, non-cachable, Bank4
  adr r0, %0111100000010000             adr r1, _SDR4       str r0, [ r1 ]
\ 1k from $5000:0000
  adr r0, $50000000                     adr r1, _START4     str r0, [ r1 ]
  adr r0, $50000400                     adr r1, _STOP4      str r0, [ r1 ]

\ Copy vector table to low RAM if it exists
  mvl r0, # Vector_Start                \ Source address.
  mov r1, # $0                          \ Destination address.
  mov r2, # Vector_End Vector_Start -   \ Length to copy.
L: CopyJumpTable
  ldr r3, [ r0 ], # 4
  str r3, [ r1 ], # 4
  sub .s r2, r2, # 4
  b .ne CopyJumpTable

  b ECLD
end-code

HandlerReset ResetV  !

[then]

