\ AT49BV162AT.FTH - JTAG Widget Atmel AT49BV162AT Flash write utility

\ Copyright (c) 2006
\ MicroProcessor Engineering
\ 133 Hill Lane
\ Southampton SO15 5AF
\ England
\ 
\ tel: +44 (0)23 8063 1441
\ fax: +44 (0)23 8033 9691
\ net: mpe@mpeforth.com
\      tech-support@mpeforth.com
\ web: www.mpeforth.com
\ 
\ From North America, our telephone and fax numbers are:
\   011 44 23 8063 1441
\   011 44 23 8033 9691
\ 
\ Change history
\ ==============
\ 20061025 SFP001 First release.

only forth definitions  decimal


\ ************
\ *! 49bv162at
\ *T AT49BV162AT Flash programming
\ ************
\ *P This code is used by the JTAG Widget to write to the Flash
\ ** for storing applications. The code is designed for use with
\ ** an Atmel AT49BV162AT device on a 16 bit bus. The size of each
\ ** sector is given by the sector table.

\ *P This code is hardware and CPU dependent because of bus width
\ ** and cache considerations. This code is being tested on a New Micros
\ ** NMI9200 board. This is ALPHA TEST code.


\ ****************
\ *S Configuration
\ ****************
\ *P This section configures the base address of the target
\ ** Flash, the start address of the RAM that holds the
\ ** target Flash programming routine, and disables caching
\ ** of the Flash.
\ ** Edit this file before compiling it so that the target hardware
\ ** is used correctly.

$10000000 to FlashBase	\ default CS0
\ *G Define the default base address of the current Flash device.
$00200000 to RAMbase	\ default internal SRAM
\ *G Define the default base address of the RAM used for the
\ ** programing code.

\ *P Some CPUs, e.g. MIPS and some ARMs use a particular address
\ ** range to determine whether the access should be cached
\ ** or uncached. During Flash programming, *\b{all} accesses to
\ ** the Flash should be uncached and write buffers should
\ ** be turned off.
\ ** The following is for for a Samsung S3C4510B CPU.
\ *E : ForceUncached  \ addr -- addr'
\ **   bit26 or  ;

\ *P If your CPU does not have a cache or if caching is
\ ** never enabled for Flash access, use the following code.
\ *E : ForceUncached   \ addr -- addr'
\ ** ; immediate

: ForceUncached	\ addr -- addr'
\ *G Convert a target address into its uncached form. Defaults to
\ ** *\fo{NOOP}.
; immediate


\ ========================
\ *S Flash Access routines
\ ========================

create FL49bv162at	\ -- addr
\ *G Table containing the number of sectors and starting offset
\ ** of each sector, plus a dummy start address which enables
\ ** the size of the last sector to be calculated.
  #39 ,
  $00000000 ,  $00010000 ,  $00020000 ,  $00030000 ,
  $00040000 ,  $00050000 ,  $00060000 ,  $00070000 ,
  $00080000 ,  $00090000 ,  $000A0000 ,  $000B0000 ,
  $000C0000 ,  $000D0000 ,  $000E0000 ,  $000F0000 ,
  $00100000 ,  $00110000 ,  $00120000 ,  $00130000 ,
  $00140000 ,  $00150000 ,  $00160000 ,  $00170000 ,
  $00180000 ,  $00190000 ,  $001A0000 ,  $001B0000 ,
  $001C0000 ,  $001D0000 ,  $001E0000 ,  $001F0000 ,
  $001F2000 ,  $001F4000 ,  $001F6000 ,  $001F8000 ,
  $001FA000 ,  $001FC000 ,  $001FE000 ,  $00200000 ,
FL49bv162at to SecTab

create FL49bv162a	\ -- addr
\ *G Table containing the number of sectors and starting offset
\ ** of each sector, plus a dummy start address which enables
\ ** the size of the last sector to be calculated.
  #39 ,
  $00000000 ,  $00002000 ,  $00004000 ,  $00006000 ,
  $00008000 ,  $0000A000 ,  $0000C000 ,  $0000E000 ,
  $00010000 ,  $00020000 ,  $00030000 ,  $00040000 ,
  $00050000 ,  $00060000 ,  $00070000 ,  $00080000 ,
  $00090000 ,  $000A0000 ,  $000B0000 ,  $000C0000 ,
  $000D0000 ,  $000E0000 ,  $000F0000 ,  $00100000 ,
  $00110000 ,  $00120000 ,  $00130000 ,  $00140000 ,
  $00150000 ,  $00160000 ,  $00170000 ,  $00180000 ,
  $00190000 ,  $001A0000 ,  $001B0000 ,  $001C0000 ,
  $001D0000 ,  $001E0000 ,  $001F0000 ,  $00200000 ,
\ FL49bv162a to SecTab

: FlUnlock	\ --
\ *G Unlock the flash.
  $AA flashbase $5555 1 lshift + w!c(t)	\ unlock
  $55 flashbase $2AAA 1 lshift + w!c(t)
;

: FlCmd		\ command --
\ *G Unlock the flash and write the command.
  $AA flashbase $5555 1 lshift + w!c(t)		\ unlock
  $55 flashbase $2AAA 1 lshift + w!c(t)
      flashbase $5555 1 lshift + w!c(t)		\ command
;

: FlReset	\ --
\ *G Reset device after AutoSelect command.
  $F0 flashbase $5555 1 lshift + w!c(t)	\ reset
;

: FlId		\ -- manid devid
\ *G Get manufacturer's ID, should be $1F $D6 for AT49BV162AT
  $90 FlCmd
  flashbase $000 + w@c(t)
  flashbase $002 + w@c(t)
  FlReset
;

: FlCfg		\ 0/1 -- 
\ *G Set the device configuration register to 0 or 1.
  $D0 FlCmd  FlashBase w!c(t)
;

variable FlResult	\ -- addr
\ *G Holds the last two reads from a faulty erase.

: SecErase	\ secaddr -- ; erase sector
\ *G Erase the sector at the given address.
  0 FlCfg				\ configuration mode 0.
  $80 FlCmd  FlUnlock $30 over w!c(t)	\ Sector erase sequence
  dup >r  w@c(t)			\ first status read
  begin					\ -- prev
    r@ w@c(t) tuck			\ -- curr prev curr
    xor $40 and	0=			\ -- curr flag ; done if DQ6 not toggled
    over $0028 and  or			\ -- curr flag' ; done if bit 3 or 5 set
  until
  $0028 and if
    r@ w@c(t) #16 lshift  r@ w@c(t) or
    Flresult !    
  endif
  r> drop
  FlReset
;

: (EraseFlash)	\ dest dlen --
\ *G Erase the flash sectors covering dlen bytes at dest.
  dup if
    swap ForceUncached swap		\ *** force bypass of cache **
    over + 1- >r			\ -- dest ; R: -- end
    begin				\ -- addr ; R: -- end
      cr ."  Erasing sector at: " dup .lword
      dup SecErase			\ erase current sector
      FindSector +			\ step to start of next sector
      dup r@ u>				\ until out of range
    until
    drop  r> drop
  else
    2drop
  endif
;


\ ******************************
\ *S Code loaded into target RAM
\ ******************************
\ *P This section contains code that is copied to target
\ ** RAM for execution. It is position independent.
\ ** The two subroutines labelled *\i{DCCemit} and
\ ** *\i{DCCkey} are the same for all ARM7/9 CPUs and
\ ** Flash devices.

\ DCC control register
\ Bit1  The W bit is 0 if the DCC write register can accept data
\       from the CPU.
\ Bit0  The R bit is 0 if the DCC read register is empty.

\ *[ 
arm7tdmi	\ select min CPU required

Proc Write162at	\ do not execute this on host
ahead,		\ forward unconditional branch
\ DCCemit - in: R0=x32, destroys: R1
l$1:
  begin,
    mrc    #14 0 r1 cr0 cr0 0	\ read DCC control reg
    and .s r1, r1, # $02	\ test bit 1
  eq, until,			\ 0 for ready to write
  mcr   #14 0 r0 cr1 cr0 0	\ write DCC data reg
  mov	pc, link		\ return

\ DCCkey - out: R0=x32, destroys: R1
l$2:
  begin,
    mrc    #14 0 r1 cr0 cr0 0	\ read DCC control reg
    and .s r1, r1, # $01	\ test bit 0
  ne, until,			\ 1 for ready to read
  mrc   #14 0 r0 cr1 cr0 0	\ read DCC data reg
  mov	pc, link		\ return

\ Wr16 - in: R4=FlashBase, R5=addr, R6=x16, R8=#errs
\      - out: R5=addr+2, R8=#errs
\      - destroys: R0, R1, R2, R3
\ unlock, write command, write data
l$3:
  mov   r1, # $55		\ offset = $5555
  orr   r1, r1, # $5500
  mov	r1, r1 lsl # 1		\ but address is for 16 bit unit
  mov   r0, # $AA		\ first unlock byte
  strh  r0, [ r4 ++ r1 ]
  mov   r2, # $AA		\ offset = $2AAA
  orr   r2, r2, # $2A00
  mov	r2, r2 lsl # 1		\ but address is for 16 bit unit
  mov   r0, # $55		\ second unlock byte
  strh  r0, [ r4 ++ r2 ]
  mov   r0, # $A0		\ write command code
  orr   r0, r0, # $A000
  strh  r0, [ r4 ++ r1 ]
  bic   r2, r6, # $FF000000	\ clear high 16 bits for later compare
  bic   r2, r2, # $00FF0000
  strh  r2, [ r5 ]		\ write data

\ poll data until done or errored out
  mov   r0, # $4000		\ time out counter
  begin,
    ldrh  r3, [ r5 ]		\ read data back
    cmp   r2, r3		\ same?
   ne, while,
    cmp   r0, # 0		\ timed out?
   ne, while,
    sub   r0, r0, # 1
  repeat, then,
  cmp   r2, r3			\ same?
  add .ne r8, r8, # 1		\ no, update error count

  add   r5, r5, # 2		\ step destination address
  mov	pc, link		\ return

\ Main - in: R4=FlashBase, R5=dest, uses: R7=count, R8=#errs
then,		\ patch up AHEAD,
begin,		\ main loop
  mov	r8, # 0			\ clear error counter
  bl	L$2			\ DCCkey for block size in cells
  mov .s r7, r0			\ R7 := loopcount
  b .eq	$			\ spin if zero - indicates last block
  begin,
    bl	  L$2			\ DCCkey for 32 bit data
    mov   r6, r0
    bl	  l$3			\ write low 16 bits
    mov   r6, r6 lsr # #16
    bl	  l$3			\ write high 16 bits
    sub .s r7, r7, # 1
  eq, until,
  mov	r0, r8			\ error count
  bl    l$1			\ DCCemit for error count
again,
  mov   pc, link		\ keeps disassembler happy
EndProc
here Write162at - constant Len162at  \ image length to copy to target.
\ *] 


\ ************************
\ *S Programing the device
\ ************************

: Prog162at	\ addr len -- 
\ *G Given the address and maximum length of the target memory to
\ ** be programed, erase the target Flash, and read and program
\ ** the target memory from a host file in binary image format.
\ ** The file is uploaded into the JTAG Widget using the XModem
\ ** 1024 byte block protocol. See *\fo{RecvXmodem} in the XModem
\ ** chapter of the manual for more details of the XModem system.
  Xmodem-1k				\ faster than 128
  HaltCPU
  cr ." Erasing Flash "
  2dup (EraseFlash)
  swap ForceUncached swap		\ force bypass of cache
  over Write162at Len162at +WriteTarget	\ set target portion
  RecvXmodem				\ receive file and program
  -WriteTarget .Xresult			\ stop CPU, show results
;


\ ******
\ *> ###
\ ******

decimal
