\ Init9200ek.fth - generic AT91RM9200 startup code

(( 
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Much of this code is derived from sample files provided
on the Atmel AT91 CD.
))

(( Memory controller area set up by UBOOT
ffffff00: 00000000 00000000 00000000 00003210    .............2..
ffffff10: 00000000 00000000 00000000 00000000    ................
ffffff20: 00000000 00000000 00000000 00000000    ................
ffffff30: 00000000 00000000 00000000 00000000    ................
ffffff40: 00000000 00000000 00000000 00000000    ................
ffffff50: 00000000 00000000 00000000 00000000    ................
ffffff60: 00000002 00000000 00000000 00000000    ................
ffffff70: 00003284 00002000 00002000 00002000    .2... ... ... ..
ffffff80: 00002000 00002000 00002000 00002000    . ... ... ... ..
ffffff90: 00000000 000002e0 2188c155 00000000    ........U..!....
ffffffa0: 00000000 00000000 00000000 00000000    ................
ffffffb0: 00000000 00000000 00000000 00000000    ................
ffffffc0: 00000000 00000000 00000000 00000000    ................
ffffffd0: 00000000 00000000 00000000 00000000    ................
ffffffe0: 00000000 00000000 00000000 00000000    ................
fffffff0: 00000000 00000000 00000000 00000000    ................
))

decimal

\          MOSC * ( mul+1)
\ PLLout = ---------------
\               div
\ 
\          PLLout * div
\ mul+1  = ------------
\              MOSC
\ 
\          PLLout * div
\ mul    = ------------ - 1
\              MOSC

interpreter
: genMUL	\ freq div -- mul
\ Given a required frequency and a division ratio, calculate
\ the PLL multiplier.
  /MOSC */mod  swap /MOSC 2/ >=
  if  1+  endif
  1-
;

: genFREQ	\ mul div -- freq
\ Given a multiplier and divider, calculate the PLL output frequency.
  /MOSC -rot
  swap 1+ swap */
;

: genPLLx	\ mul div cnt base -- regval
\ Given a multiplier, divider, slow clock count, and a base value,
\ form the PLLAR/PLLBR value.
  2over genFREQ #150000000 >		\ check freq>150MHz
  if  bit15 or  endif			\ and set OUTA accordingly
  swap $3F and 8 lshift or		\ cnt
  swap $FF and or			\ div
  swap $7FF and #16 lshift or		\ mul
;
target

\ Calculate the PLLA settings
system-speed /PLLAdiv genMUL equ /PLLAmul
/PLLAmul /PLLAdiv genFREQ equ /PCK
/PLLAmul /PLLAdiv $3E $20000000 genPLLx
  equ initPLLAcalc

\ Calculate the PLLB settings
/PLLBdiv2 [if] 96000000 [else] 48000000 [then]
  equ usb-speed		\ -- hz
usb-speed /PLLBdiv genMUL equ /PLLBmul
/PLLBmul /PLLBdiv genFREQ equ /USB
/PLLBmul /PLLBdiv $3E
/PLLBdiv2 [if] $10000000 [else] $00000000 [then]
 genPLLx
  equ initPLLBcalc

\ Calculate the MCKR settings
/MCKdiv 1- 3 and 8 lshift  $0002 or
  equ initMCKRcalc

/PCK /MCKdiv / equ /MCK	\ -- hz

showPLLinfo? [if]
cr
cr ." Requested PCK = " system-speed . ." Hz"
cr ." Obtained PCK  = " /PCK . ." Hz"
cr ." mul = " /PLLAmul .
cr ." div = " /PLLAdiv .
cr ." PLLAR = " initPLLAcalc .dword
cr
cr ." Requested USB clk = " usb-speed . ." Hz"
cr ." Obtained USB clk  = " /USB . ." Hz"
cr ." mul = " /PLLBmul .
cr ." div = " /PLLBdiv .
cr ." PLLBR = " initPLLBcalc .dword
cr
cr ." Obtained MCK = " /PCK /MCKdiv / . ." Hz"
cr ." MCKR = " initMCKRcalc .dword
cr
[then]

(( observed on EK set up
\ PLL setup for 18.432 MHz Xtal
#define	initPLLAR	0x20263E04	\ 179,712000 MHz for PCK 
					\ out=0, cnt=$3E, mul-1=$26=38, div=$04
#define	initPLLBR	0x10483E0E	\ 48,054857 MHz (divider by 2 for USB)
					\ out=0, cnt=$3E, mul-1=$48=72, div=$0E
#define	initMCKR	0x00000202	\ PCK/3 = MCK Master Clock = 59,904000MHz
					\ with PLLA selected.
))
#define CS1area		0x20000000	\ start address of CS1 memory
#define DBW_SDRC_MR	0x00		\ 0x00 for 32 bit SDRAM, 0x10 for 16 bit


\ ---------------------------------------
\ Generic start up table and entry points
\ ---------------------------------------

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr   pc, ResetV		\ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  ldr 	pc, UnusedV             \ 14 - Reserved (address exception if 26bit)
  ldr 	pc, [ pc, # $-F20 ]	\ 18 - IRQ interrupts
  ldr 	pc, [ pc, # $-F20 ]	\ 1C - FIQ interrupts

  L: Soft_Vectors               \ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset ; must be in ProgFlash
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code
Vector_End Vector_Start - equ Vector_Len

proc init-io-ll	\ R0 contains address of initialisation table
\ *G Low-level initialisation routine. On entry, R0 points to a
\ ** table which contains address/value pairs. The tables finish
\ ** with a zero address. Uses R1 and R2.
  bic     r0, r0, # $FF000000	\ force address to Flash
  and     r1, pc, # $FF000000
  orr     r0, r0, r1
l: init-io-ll-1
  ldr	  r1, [ r0 ], # 4	\ get address
  cmp	  r1, # 0		\ get out if 0
  mov .eq pc, link
  ldr     r2, [ r0 ], # 4	\ get data
  str	  r2, [ r1, # 0 ]	\ store data
  b       init-io-ll-1
l: init-io-ll-x
  next,
end-code

\ Rather than use long literal macro instructions to access data,
\ it is shorter to use PC relative accesses to the data.
l: ^MC		\ Pointer to Memory controller
  _MC ,
l: ^REMAPx	\ filled in later with branch address after REMAP
  0 ,
l: ^PIOC	\ Pointer to PIOC
   _PIOC ,
l: ^PMC		\ pointer to PMC
  _PMC ,
l: ^initMOR	\ initial PMC
  $FF01 ,				\ OSCOUNT=$FF, MOSCEN=1
l: ^AIC		\ pointer to AIC
  _AIC ,
l: ^AIC_all	\ mask for all interrupts
  $FFFFFFFF ,

l: InitFlashTab
\  _MC MC_PUIA + ,	0 ,		\ Undocumented incantation
\  _MC MC_PUP + ,	0 ,
\  _MC MC_PUER + ,	0 ,
\  _MC MC_ASR + ,	0 ,
\  _MC MC_AASR + ,	0 ,
  _MC EBI_CSA + ,	bit1 ,		\ Setup MEMC to support CS1=SDRAM, CS0=Flash
  _MC EBI_CFGR + ,	0 ,		\ Data bus pulled up
  _MC SMC_CSR0 + ,	$3284 ,		\ No early read, DBW=1, BAT=1, 2 data float, 4 w/s
  0 , 0 ,				\ end of table
l: ^InitFlashTab
  InitFlashTab ,

((
#define 	AT91C_SDRC_MODE_NORMAL_CMD           (0x0) // (SDRC) Normal Mode
#define 	AT91C_SDRC_MODE_NOP_CMD              (0x1) // (SDRC) NOP Command
#define 	AT91C_SDRC_MODE_PRCGALL_CMD          (0x2) // (SDRC) All Banks Precharge Command
#define 	AT91C_SDRC_MODE_LMR_CMD              (0x3) // (SDRC) Load Mode Register Command
#define 	AT91C_SDRC_MODE_RFSH_CMD             (0x4) // (SDRC) Refresh Command
))
l: initSDRAMtab	\ -- addr
\ *G SDRAM initialisation table. See INIT_FLASH.C and INIT_RAM.C
\ ** on the AT91 CD.
\  _MC EBI_CSA + ,	bit1 ,		\ Setup MEMC to support CS1=SDRAM, CS0=Flash
\  _MC EBI_CFGR + ,	0 ,		\ Data bus pulled up
  _MC SDRC_CR + ,	0x2188C155 ,	\ TXSR=4, TRAS=3, TRCD=1, TRP=9, TRC=8,
					\ TWR=#2; CAS2, NB4; NR12, NC9 
  _MC SDRC_MR + ,	DBW_SDRC_MR 2 + ,	\ Precharge all
  CS1area ,		0 ,		\ dummy write
  _MC SDRC_MR + ,	DBW_SDRC_MR 4 +	, \ 8 Refresh cycles
  CS1area ,		0 ,		\ dummy write, #1
  CS1area ,		0 ,		\ dummy write, #2
  CS1area ,		0 ,		\ dummy write, #3
  CS1area ,		0 ,		\ dummy write, #4
  CS1area ,		0 ,		\ dummy write, #5
  CS1area ,		0 ,		\ dummy write, #6
  CS1area ,		0 ,		\ dummy write, #7
  CS1area ,		0 ,		\ dummy write, #8
  _MC SDRC_MR + ,	DBW_SDRC_MR 3 +	, \ Load Mode Register
  CS1area ,		0 ,		\ dummy write
  _MC SDRC_TR + ,	0x2E0		\ refresh counter: MCK, refresh rate, refresh burst len
  CS1area 0x080 + ,	0 ,		\ write offset to SDRAM device mode register
  _MC SDRC_MR + ,	DBW_SDRC_MR 0 +	, \ into Normal mode
  CS1area ,		0 ,		\ dummy write
  0 , 0 ,				\ end of table
l: ^initSDRAMtab
  initSDRAMtab ,

\ Application details, filled in at the end of the control file.
l: ^ProgFlash	\ source where app is stored in external Flash
  0 ,
l: ^ProgBase	\ dest where app is copied to
  0 ,
l: ^ProgLen	\ length of app
  0 ,

\ Forth start up data
L: CLD1		\ cold start word
  0 ,		\ compiler will fill up with address cold start word
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,

Proc InitReset
\ *G Initialisation. Note that this code must be
\ ** position independent as it can be executed at another
\ ** address as part of the boot process.
  mov   r0, # $D3			\ I/F set, mode=SVC
  msr   cpsr _all r0			\ force CPSR

\ *** Enable the main ocillator
  ldr     r0, ^PMC
  ldr     r1, ^initMOR
  str     r1, [ r0, # CKGR_MOR ]
\ *** Wait for main oscillator to come ready
  begin,
    ldr     r1, [ r0, # CKGR_MCFR ]
    and .s  r1, r1, # bit16		\ ready bit in CKGR_MCFR must be 1
  ne, until,

\ ** Enable the Flash on NCS0 - must be done before clocks are changed
  ldr     r0, ^InitFlashTab
  bl      init-io-ll

\ *** Set up the PLLs
l: SetupPLLs
  ldr     r0, ^PMC
\  mov     r1, # $0001			\ disable PCK
\  str     r1, [ r0, # PMC_SCDR ]
  mvl     r1, # initPLLAcalc		\ PLLA
  str     r1, [ r0, # CKGR_PLLAR ]
  begin,
    ldr     r1, [ r0, # PMC_SR ]	\ LOCKA bit must be 1
    and .s  r1, r1, # bit1
  ne, until,
  mvl     r1, # initPLLBcalc		\ PLLB
  str     r1, [ r0, # CKGR_PLLBR ]
  begin,
    ldr     r1, [ r0, # PMC_SR ]	\ LOCKB bit must be 1
    and .s  r1, r1, # bit2
  ne, until,
  mvl     r1, # initMCKRcalc		\ MCK
  str     r1, [ r0, # PMC_MCKR ]
  begin,
    ldr     r1, [ r0, # PMC_SR ]	\ MCKRDY bit must be 1
    and .s  r1, r1, # bit3
  ne, until,
  mov     r1, # $0001			\ enable PCK
  str     r1, [ r0, # PMC_SCER ]

\ *** Initialise the SDRAM
\ Set up I/O pins
  and .s  r0, pc, # $E0000000		\ mask PC
  eq, if,
    ldr     r0, ^PIOC			\ Select SDRAM for pins
    mvl     r1, # $FFFF0000		\ bits 31..16 are peripheral A
    mvl     r2, # 0			\ no periperal B yet
    str     r1, [ r0, # PIO_ASR ]
    str     r2, [ r0, # PIO_BSR ]
    orr     r3, r1, r2			\ enable non-PIO peripheral functions
    str     r3, [ r0, # PIO_PDR ]

    ldr     r0, ^InitSDRAMTab
    bl      init-io-ll
  then,
 
\ *** Set up the AIC
  ldr	  r0, ^aic_all			\ disable all interrupts
  ldr	  r1, ^aic
  str	  r0, [ r1, # AIC_IDCR ]
  str	  r0, [ r1, # AIC_ICCR ]

  mov	  r0, # #8			\ will write 8 times to AIC_EOICR
L: LoopAIC0	
  str	  r1, [ r1, # AIC_EOICR ]	\ write any value to the end of interrupt
  sub .s  r0, r0, # 1
  b .ne	  LoopAIC0

\ If we have booted from external 16 bit Flash, we will need to
\ do a REMAP. We assume that we are booting with BMS=0, in which
\ case the Flash on NCS0 is visible both at $0xxxxxxx and at
\ $1xxxxxxx. We test for this by checking if we are running in
\ low memory with address bits 31..28=0. Because we have not yet
\ copied the vectors and the application code into SRAM and SDRAM
\ the destination address of the branch after remapping must be in
\ the *\fo{BootStore} section. This address is calculated at
\ compile time
  and .s  r0, pc, # $E0000000		\ mask PC
  eq, if,				\ at $0xxxxxxx
\    ldr     r6, ^REMAPx			\ address we want to branch to in BootStore
    ldr     r4, ^MC			\ memory controller address
    mov     r5, # 1			\ RCB=bit0
\ Do NOT separate the next two instructions.
    str     r5, [ r4, # MC_RCR ]	\ remap command
\    mov     pc, r6			\ executed from the pipeline
  endif,
l: REMAPx	\ This address is massaged into ^REMAPx later on


\ The code in Flash is now copied to SDRAM for execution.
\ The vector area is copied to low RAM (REMAPped internal RAM)
\ for interrupt handlers to patch.

l: CopiesToRAM

\ Copy application into code RAM
l: CopyFlashToRAM
  ldr	  r0, ^ProgFlash		\ source
  ldr	  r1, ^ProgBase			\ dest
  ldr	  r2, ^ProgLen			\ len
  add     r2, r2, # 3			\ round to 4 byte unit
  bic     r2, r2, # 3
l: appcopy_loop
  ldr	  r3, [ r0 ], # 4
  str	  r3, [ r1 ], # 4
  sub .s  r2, r2, # 4
  b .ne	  appcopy_loop


\ Copy vector area into low RAM.
\ Must be after code copy, otherwise may not be there!
l: CopyVectorsToZero
  mov	  r0, # vector_start		\ source
  mov	  r1, # 0			\ dest
  mov	  r2, # Vector_len		\ eight vectors + soft vectors
l: vector_loop
  ldr	  r3, [ r0 ], # 4
  str	  r3, [ r1 ], # 4
  sub .s  r2, r2, # 4
  b .ne	  vector_loop

 
\ ---------------------
\ start of Forth system
\ ---------------------

L: ECLD
\ Disable interrupts
  mrs   r0, cpsr			\ get CPSR
  orr   r0, r0, # I_BIT F_BIT or	\ set I bit ; SFP009
  msr   cpsr _c r0			\ modify control bits
\ Set up the Forth registers
  bl initStacks				\ use defaults in ARM\InitStacks.fth
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

REMAPx vector_start -			\ offset of REMAPx from boot address
BootStore sec-base  + prog		\ rebased into BOOTSTORE section
^REMAPx !

InitReset prog sec-base -		\ offset from start of code
BootStore sec-base  + prog		\ plus base address of Flash
ResetV !				\ set reset address


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] sp-guard [if]
0 equ sp-guard
[then]

\ initial values of user variables for standalone systems only
[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, span/tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]

