\ NMI9200 standalone Forth control file

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ =========
\ *! EK9200
\ =========
\ *T Forth for NMI9200

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives
"" C:\buildkit.dev\software\AddOns\PowerNet\v30dev"
		setmacro IpStack	\ where PowerNet lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  +LeafCalls			\ improves performance, reduces size.

  Hex-I32			\ generate 32 bit Intel Hex file

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

((
\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format
))

\ --------------------
\ *H Kernel components
\ --------------------

 1 equ ColdChain?	\ -- n
\ *G nz to use cold chain mechanism
 1 equ tasking?
\ *G true if multitasker needed. Event handler and message
\ ** passing code is not compiled, but semaphores are.
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\   true to include semaphores
 0 equ timebase?
\ *G true for TIMEBASE code
 0 equ softfp?
\ *G true for software floating point
 0 equ FullCase?
\ *G true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?
\ *G true if target local variable sources needed
 0 equ romforth?
\ *G true for ROMForth handler
 0 equ blocks?
\ *G true if BLOCK needed
 $18000 equ sizeofheap
\ *G 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ -----------------------
\ *H Extension components
\ -----------------------

 1 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.

\ ------------------------------------------
\ *H Crystal, clock, serial and ticker rates
\ ------------------------------------------
\ *P The AT91RM9200 has five clocks derived from two crystal
\ ** sources:
\ *(
\ *B SCK - Slow clock, usually a 32768Hz crystal,
\ *B MOSC - Master oscillator, 18.432MHz crystal on the EK board,
\ *B PCK - set by PLLA from MOSC,
\ *B MCK - derived from PCK,
\ *B UCK - 48 or 96 MHz USB clock set by PLLB from MOSC.
\ *)

#32768 equ slow-clock		\ -- hz
\ *G Slow clock rate in Hz.
#18432000 equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
xtal-speed equ /MOSC		\ -- hz
\ *G Synonym used by PLL calculator.
#179712000 equ system-speed	\ -- hz
\ *G Requested processor clock in HZ. This is the target speed
\ ** of PCK. The equate *\fo{/PCK} is calculated in *\i{InitNMI9200.fth)
\ ** and represents the actual PCK speed.
3 equ /MCKdiv		\ -- n
\ *G The divider applied to PCK to generate MCK. Must be in the
\ ** range 1..4. The prescaler in this implementation is always 1,
\ ** and the use of PLLA is assumed.
4 equ /PLLAdiv		\ -- n
\ *G The divider used for PLLA.
#14 equ /PLLBdiv	\ -- n
\ The divider used for PLLB.
1 equ /PLLBdiv2		\ -- 0/1
\ *G Set this to 1 if PLLB should generate 96MHz rather than 48MHz.
\ ** If set, the PLL clock will be divided by two to generate 48MHz.
1 equ showPLLinfo?	\ -- flag
\ *G If set non-zero the results of the PLL calculations are
\ ** displayed.

1 equ useDBGU?		\ -- n
\ *G Set non-zero to compile code for the two-wire debug UART,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUSART0?	\ -- n
\ *G Set non-zero to compile code for USART0,
\ ** device *\fo{Console1}.
#38400 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console3}.
#9600 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console4}.
#9600 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

0 equ console-port
\ *G Designate serial port for terminal (0..4). The DBGU is selected
\ ** by 0, the UARTS by 1..4. 10 is reserved for the USB device port.

#10 equ tick-ms
\ *G TIMEBASE tick in ms.


\ ---------------------------------------------
\ *H Define stack and user area sizes for TASKs
\ ---------------------------------------------

$0100 equ UP-SIZE
\ *G size of each task's user area
$0100 equ SP-SIZE
\ *G size of each task's data stack
$0200 equ RP-SIZE
\ *G size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN
\ *G Terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs
\ *G Number of IRQ stacks, shared by all IRQs (1 min)
1 equ #FIQs
\ *G number of FIQ stacks, shared by all FIQs (1 min)
0 equ #SWIs
\ *G number of SWI nestings permitted. 0 is ok if SWIs are unused.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ *************
\ *S Memory map
\ *************

  $20000000 $200FFFFF cdata section NMI9200	\ 1Mb code space on CS1=SDRAM
\ *G Primary code section in external 16 Mb SDRAM.
\ ** The first 16Mb are currently reserved for the NMI9200 software.
  $10000000 $100FFFFF cdata section BootStore	\ 1Mb where program is held
\ *G The 16 bit Flash region holding the code image when the CPU
\ ** comes out of reset. This usually starts at $10000000 on NCS0.
\ ** *\b{N.B.} This must not be the first CDATA section.
  $20200000 $2020FFFF idata section PROGd	\ 64k Idata
\ *G 64k IDATA. This area contains initialised data.
  $20210000 $2024FFFF udata section PROGu	\ 256k Udata
\ *G 64k UDATA. This area contains uninitialised data.
  $20250000 $2025FFFF udata section MMUdata	\ 16k used on 64k boundary
\ *G MMU tables must be on a 64k boundary. We usually put this
\ ** section in the last 64k of the cached area.
    MMUdata sec-base equ TBASE			\ equate for Cache control
  $00000000 $00003FFF udata section CpuRAM	\ internal SRAM
\ *G Internal SRAM after REMAP operation. This RAM holds interrupt
\ ** vectors and data which must not be cached.
    $0200 allot				\ reserved for vectors and FIQ handler.

interpreter
: prog nmi9200  ;			\ synonym
\ *G This sysnonym is used to refer to the master CDATA section which
\ ** will contain the boot code and the IDATA section initialisation
\ ** data.
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfr9200  		\ Special function registers.
  include %HwDir%\initNMI9200		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CpuDir%\drivers\Cache9200	\ cache control
  include %CommonDir%\kernel62          \ high level kernel definitions
' EnableCache AtCold			\ Enable cache at power up
  include %CpuDir%\drivers\reboot9200	\ reboot code
1 equ ARM-VIC?				\ select AT91 Vectored Interrupts
  include %CpuDir%\IntArm3		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
\  include %CpuDir%\drivers\ser9200p	\ polled serial driver
\  include %CpuDir%\drivers\ser9200i	\ interrupt RX serial driver
  include %CpuDir%\drivers\ser9200qi	\ queued interrupt RX serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%\delays
  include %CpuDir%\drivers\Tick9200	\ timer tick
  ' start-timers AtCold
[else]
  include %CommonDir%\delays
  include %CpuDir%\drivers\Tick9200	\ timer tick
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

buildfile nmi9200.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." *************************************"
;
external

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for NMI AT91RM9200"
  cr version$ $. space BuildDate$ $.
  .banner
;

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ Application code
\ *******************

  include %CommonDir%\XmodemTxRx	\ Xmodem driver
  include %CpuDir%\drivers\AT49bv6416	\ Flash driver and REFLASH
1 equ CPU=ARM	\ to get ARM specific code
  include %ExampleDir%\NetCode		\ network order stuff
  include %CpuDir%\drivers\Ether9200	\ Ethernet driver

PowerNet? [if]
  include %AppDir%\PNconfig		\ PowerNet configuration
  include %IpStack%\PowerNet.bld	\ PowerNet build
  include %AppDir%\WebConfig		\ configuration tools
[then]


\ **********************
\ Select example code
\ **********************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit. Then set program details.
prog flush-idata			\ force IDATA sections to be laid NOW
here $1FF +  $-0200 and org		\ force to 512 byte boundary

sec-base ^ProgBase !
here sec-base - ^ProgLen !
BootStore sec-base ^ProgFlash !
prog


\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build				\ increment build number
FINIS


\ ======
\ *> ###
\ ======

