\ INITLP210x.FTH - Philips LPC210x generic startup code
\ adapted for OXUF922 ### mgs 26/04/04

(( 
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ ***************************************
\ Generic start up table and entry points
\ ***************************************

asmcode
L: Vector_Start			\ 'Hard wired' ARM Processor vectors
  ldr 	pc, ResetV              \ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  dd    0                       	\ 14 - checksum
  ldr 	pc, IRQV				\ [ pc, # $-FF0 ]		\ 18 - IRQ interrupts from VicVectAddr @ $FFFFF030
  ldr 	pc, FIQV				\ 1C - FIQ interrupts

L: Soft_Vectors			\ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code

Vector_End Vector_Start - equ Vector_Len


\ ************
\ Startup code
\ ************

\ Required Forth pointers
L: CLD1		                        \ cold start word
  0 ,				        \ compiler will fill up with address of word to execute at cold start
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,

l: ^WDT			      		\ initial config w/dog, clks, & memory
	_WDT ,
	WD_init ,
	WD_disable ,
l: ^clk_cntl
	_clk_start ,
	clk_enable ,
l: ^LED_base
	_led_base ,
l: ^RAMbank
	_RAMbank0 ,
	timing_value ,
l: ^rps_base
	rps_ic_base ,

Proc InitReset	\ power up entry point through vector

\ OxsemiHardwareInit:
\ @ - Initialise the watchdog timer
    LDR r0, ^wdt   				\ @ - watchdog hardware reg
    LDR r1, ^wdt 4 + 			\ @ - slow timeout, reset
    STR r1, [ r0 ]               	\ @ - init watchdog
    LDR r1, ^wdt 8 +  			\ @ - disable watchdog val
    STR r1, [ r0 ]
	
\ @ -  enable the clocks
	LDR r0, ^CLK_cntl
	LDR R1, ^clk_cntl 4 +
     STR R1, [ R0 ]

\ diagnostics	
    LDR r0, ^LED_BASE
    MOV r1, # LED0
    STRB r1, [ r0 ]

\ setup memory access 
   	LDR r0, ^rambank
	LDR r1, ^RAMbank 4 +
	STR r1, [ r0 ]
	
\ @ -  Set up the GPIO port    do we need this here?
\    LDR r0, # _GPIOoe
\    LDR r1, # LEDARRAY
\    STR r1,[r0]


\     * Allow the board a little extra time to stablize before accessing RAM
(( 
    LDR r1, # $FFFF          \ /* timed loop */
l: dl0
    SUBS r1, r1, # 1
    BNE dl0

    LDR r1, # _RAMbank0

    LDR r2, # $10020585
    STR r2, [r1, #(SRC_UART - STATIC_RAM_REG_BASE)]

    LDR r2, =0x40010301
    STR r2, [r1, #(SRC_FLASH - STATIC_RAM_REG_BASE)]
 ))

\ * CPU INTERRUPTS EXPLICITLY DISABLED
\ * disable individual interrupts in the interrupt controller
 
    ldr     r2, ^RPS_BASE                               \ /* R2->interrupt controller */
    MVN     r1, # 0                                         \ /* &FFFFFFFF */
    STR     r1, [ r2, # RPS_FIQ_DISABLE  RPS_IC_BASE - ]      \ /* disable all FIQ sources */
    STR     r1, [ r2, # RPS_IRQ_DISABLE  RPS_IC_BASE - ]      \ /* disable all IRQ sources */


 \ * Disable timers
 ((
     MOV     r2, # RPS_BASE
     ADD     r2, r2, # RPS_TIMER1_BASE - RPS_BASE	
     LDR     r1, [ r2, # RPS_TIMER_CONTROL_OFFSET ]
     BIC     r1, r1, # 0x80 							\ @ -  clear the enable bit
     STR     r1,[ r2 ]

     MOV     r2, #RPS_BASE
     ADD     r2, r2, #RPS_TIMER2_BASE - RPS_BASE	
     LDR     r1, [r2,#RPS_TIMER_CONTROL_OFFSET]
     BIC     r1, r1, #0x80 @ -  clear the enable bit
     STR     r1,[r2]
  ))


\ Initialise the stacks for INTARM3.FTH
  bl		InitStacks		\ see ARM\INITSTACKS.FTH

\ diagnostics	
    LDR r0, ^LED_BASE
    MOV r1, # LED1
    STRB r1, [ r0 ]

 
\ *********************
\ Start of Forth system
\ *********************

L: ECLD
\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

InitReset prog sec-base -  ResetV !	\ INITRESET is in LOW memory starting at 0.


\ ******************************************
\ Reset values for user and system variables
\ ******************************************

[undefined] sp-guard [if]
  0 equ sp-guard
[then]

\ initial values of user variables for standalone systems only
[undefined] umbilical? [if]
L: USER-RESET
  init-s0   tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,                 		\ r0
  0 ,  0 ,                              \ #tib, span
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ *********************
\ Low level serial code
\ *********************

(( \ Is the UART working?
  bl init-uart0
l: initreseta
  mov           r0, # ascii A
  bl emit0
  b initreseta
))

((
L: ^UART0
  _UART0 ,
L: ^PCONP
  _PCONP ,
L: ^PINSEL0
  _PINSEL0 ,

proc init-uart0	\ --
\ *G Initialise serial port 0
\ enable peripheral clock
  ldr    r1, ^PCONP
  ldr    r0, [ r1 ]
  orr    r0, r0, # bit3
  str    r0, [ r1 ]
\ enable alternate functions on P0:1.0
  ldr    r1, ^PINSEL0
  ldr    r0, [ r1 ]
  bic    r0, r0, # $0F
  orr    r0, r0, # $05
  str    r0, [ r1 ]
\ initialise the UART
  ldr    r1, ^UART0
  mov    r0, # 1
  str    r0, [ r1, # UnFCR ]		\ essential
  mov    r0, # $083
  str    r0, [ r1, # UnLCR ]		\ 8 bit, 1 stop bit, no parity, enable DLAB
  mov    r0, # 0
  str    r0, [ r1, # UnDLM ]
  mov    r0, # 4
  str    r0, [ r1, # UnDLL ]
  mov    r0, # $003
  str    r0, [ r1, # UnLCR ]
  mov    pc, link
end-code

proc emit0	\ char in R0
  ldr    r1, ^UART0
l: emit0a
  ldr    r2, [ r1, # UnLSR ]
  and .s r2, r2, # $020
  b .eq  emit0a
  str    r0, [ r1, # UnTHR ]
  mov    pc, link
end-code
))

  
\ ======
\ *> ###
\ ======

decimal

