((
      MPE_2468 Development Board
      ~~~~~~~~~~~~~~~~~~~~~~~~~~
      
Words for on-board I2C devices.

Based on MPE code from AT24LC64.fth

))

DECIMAL

$57  1 LSHIFT  EQU   EE_SL_ADDR      
\ *G on-board 24LC32 EEPROM
$68  1 LSHIFT  EQU   ADC_SL_ADDR
\ *G on-board MCP3427 A to D converter


\ ********************
\ *S EEPROM Primitives
\ ********************
5 constant 24c32_twr		\ -- n ;
\ *G The max. write time in milliseconds over temperature and voltage.

: 24c32_send_addr	\ Eaddr devaddr --
\ *G Issue a device address and EEPROM address.
  write_byte_i2c			\ device address
  dup 8 rshift write_byte_i2c		\ addr hi
  $FF and write_byte_i2c		\ addr lo
;

: 24c32_write_byte	\ byte Eaddr devaddr --
\ *G Write byte to the EEPROM devaddr at EEPROM address Eaddr.
\ ** Note that the system must wait at least *\fo{24c32_TWR} milliseconds
\ ** before accessing the same device again.
  +ack_i2c  start_i2c
  $0FE and 24c32_send_addr		\ devaddr and addr
  write_byte_i2c			\ byte
  stop_i2c
;

: 24c32_read_byte	\ Eaddr devaddr -- byte
\ *G Using EEPROM devaddr, read the byte at EEPROM address Eaddr.
  +ack_i2c  start_i2c
  tuck $0FE and 24c32_send_addr		\ devaddr and addr by dummy write
  start_i2c
  $001 or write_byte_i2c		\ devaddr again
  -ack_i2c  read_byte_i2c		\ read byte
  stop_i2c
;


\ *******************
\ *S EEPROM utilities
\ *******************
\ *P These are simple words to read and write data into the serial
\ ** EEPROM. They are not optimised for speed, but they do permit
\ ** easy use. Much faster operation may be achieved using SEPROM@
\ ** and SEPROM! below.

: eeb@		\ eaddr -- b
\ *G Read a byte (8 bits) from address EADDR in the chip.
  EE_SL_ADDR 24c32_read_byte
;

: eeb!		\ b eaddr --
\ *G Write a byte (8 bits) to address EADDR in the chip.
  EE_SL_ADDR 24c32_write_byte  24c32_twr ms
;


\ ****************
\ *S ADC utilities
\ ****************
: I2CADC-READ     \ -- data
\ *G Read the result of the most recent conversion; up to 16 sig. bits
  start_i2c
  ADC_SL_ADDR bit0 or write_byte_i2c    \ address device for read
  +ack_i2c read_byte_i2c                \ read ms byte of result
  8 lshift                              \ align
  -ack_i2c read_byte_i2c                \ ls byte
  stop_i2c
  or                                    \ merge to word result
;

: I2CADC-CONVERT  \ gain rate chl --
\ *G Initiate conversion on given channel, gain and rate/#bits
\ ** !Must then wait for conversion to complete before reading!
\ ** First time this word cancels default continuous conversion mode
\ ** subsequent calls initiate a single conversion
\ **   chl: 0 or 1
\ **   rate: 0=12 bits in 1/240 second,  1=14 bits in 1/60 second
\ **         2=16 bits in 1/15 second
\ *    gain: 0=1x (3.3V), 1=2x (1.15V), 2=4x (825mV), 3=8x (412.5mV)
\ ** For full details, see Microchip MCP3427 data sheet:
\ **   http://ww1.microchip.com/downloads/en/DeviceDoc/22226a.pdf
  5 lshift                      \ align chl to bits 6:5
  swap 2 lshift or              \ merge rate in bits 3:2 
  swap or                       \ merge gain
  bit7 or                       \ convert bit set
  start_i2c
  ADC_SL_ADDR write_byte_i2c    \ address device for write
  write_byte_i2c                \ set configuration register
  stop_i2c
;
  


  
