\ MPE_2468.ctl - M.P.E. LPC-2468 Development Board

((
Copyright (c) 2008, 2009, 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20111107 SFP004	Adapted for MPE distribution.
20111014 BJC003 Converted for MPE LPC-2468 Dev. Board
20090205 SFP002 Converted to boot loader use.
20081006 MPE001 First release.
))


\ =========
\ *! mpe_2468
\ *T M.P.E. LPC-2468 User Flash
\ =========
\ *P The control file *\i{MPE_2468.ctl} generates a standalone
\ ** Forth for NXP LPC2468 systems using a 12 MHz crystal.
\ ** SD card drivers are used with the *\i{FATfiler} FAT file
\ ** system. A developer version of the compiler is required.

only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

\ Only set one constant non-zero to select the macros
0 constant BJC?			\ Barry Culver
1 constant SFP?			\ Stephen Pelc
0 constant PRW?			\ Phil Walker


BJC? [if]	\ Culver macros
"" J:\ForthLibs\MultiArm\Common"
		setmacro CommonDir	\ where the common code lives
"" J:\ForthLibs\MultiArm\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" J:\ForthLibs\MultiARM\ARM\Hardware\LPC23xx"
		setmacro HWDir		\ where board specific code lives
"" J:\ForthLibs\MultiARM\ARM\Hardware\LPC23xx"
		setmacro DevDir		\ where board specific code lives during development
"" J:\ForthLibs\MultiARM\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives
mc" %ExampleDir%\FATfiler"
		setmacro FatFiler	\ where FAT file system code lives
mc" %ExampleDir%\USB"
                setmacro UsbDir		\ where USB code lives
"" C:\buildkit.dev\software\AddOns\PowerNet\Dev"
		setmacro IpStack	\ where PowerNet lives
[then]

SFP? [if]	\ MPE macros
c" \buildkit.dev\software\ROM"
		setMacro ROMdir
mc" %ROMdir%\Common"
		setmacro CommonDir	\ where the common code lives
mc" %ROMdir%\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
mc" %ROMdir%\ARM\Hardware\LPC23xx"
		setmacro HWDir		\ where board specific code lives
\ mc" C:\Consult\CMon\PB_2468\Code"
mc" ."
		setmacro DevDir		\ where board specific code lives during development
mc" %ROMdir%\Examples"
		setmacro ExampleDir	\ Examples
mc" ."    	setmacro AppDir		\ where application code lives
mc" %ExampleDir%\FATfiler"
		setmacro FatFiler	\ where FAT file system code lives
mc" %ExampleDir%\USB"
                setmacro UsbDir		\ where USB code lives
mc" \buildkit.dev\software\AddOns\PowerNet\Dev"
		setmacro IpStack	\ where PowerNet lives
[then]


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ comment out to have an output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  Hex-I32			\ Generate 32 bit Intel Hex file
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

\ Define CPU type in use
 #2468 equ LPCtype		\ -- cpu# ; selects Flash configuration

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\   true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $8000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 1 equ PowerNet?		\ true if PowerNet needed.


\ ==========================================
\ *N Crystal, clock, serial and ticker rates
\ ==========================================

#12000000 equ xtal-speed	\ -- hz
\ *G Crystal clock rate in HZ.
#72000000 equ system-speed	\ -- hz
\ *G Desired system clock rate in HZ. After the PLL calculations
\ ** have been performed in *\i{InitLPC23xx.fth}, the value here
\ ** is reset to the actual speed to improve the accuracy of baud
\ ** rate calculations.
#48000000 equ usb-speed		\ USB clock rate in HZ, set to
				\ zero if USB is unused.

0 equ console-port		\ Designate serial port for terminal.
\ *G Ports 0..3 are the on-chip UARTs. The internal/external USB devices
\ ** are ports 10, 11, 12, and bit-banged ports are defined from 20 onwards.
1 equ useUART0?	\ -- n
\ *G Compile UART0 code if non-zero.
  #115200 equ console0-speed	\ Serial port speed in BPS.
0 equ useUART1?	\ -- n
\ *G Compile UART1 code if non-zero.
  #115200 equ console1-speed	\ Serial port speed in BPS.
0 equ useUART2?	\ -- n
\ *G Compile UART2 code if non-zero.
  #31250 equ console2-speed	\ MIDI port speed in BPS.
0 equ useUART3?	\ -- n
\ *G Compile UART3 code if non-zero.
  #250000 equ console3-speed	\ DMX port speed in BPS.

#1 equ tick-ms			\ TIMEBASE tick in ms, 1 is good for PowerNet


\ ======================================
\ *N Stack and user area sizes for TASKs
\ ======================================

\ *[
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused
\ *]

\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (4 bytes / 32 bits)
0 equ false
-1 equ true


\ **********
\ *N Memory map
\ **********

\ *[
  $0000:0000 $0007:CFFF cdata section MPE_2468	\ code section
    $7FFFFFF0 equ IAPentry			\   Thumb code entry point
  $4000:0200 $4000:3FFF idata section PROGd	\ 16k IDATA less 64+512 for vectors and buffer
    $4000:0000 equ ISPram			\ 512 bytes for ISP loader
  $4000:4000 $4000:FFFF udata section PROGu	\ 48k UDATA
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP
    						\   IAP stack is Forth return stack, max 128 bytes
  $0007:D000 $0007:DFFF udata section UnitCfg	\ Unit configuration flash
  $7FE0:0000 $7FE0:3FFF udata section ERAM	\ 16k Ethernet RAM
  $7FD0:0000 $7FD0:1FFF udata section URAMP	\ 8k primary USB RAM - for USB use
  $7FD0:2000 $7FD0:3FFF udata section URAME	\ 8k extended USB RAM - used for sectors
  $A000:0000 $A0FF:FFFF udata section SDRAM	\ 16Mb off-chip SDRAM, WHEN ENABLED
interpreter
: prog	MPE_2468 ;				\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
\ *]

\ ***************
\ *S Kernel files
\ ***************

\ *[
  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrLPC24xx	  	\ Special function registers.
  include %HwDir%\initLPC23xx		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
1 equ FastCmove?
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  include %CpuDir%\Drivers\rebootLPC210x	\ reboot using watchdog

  5 equ ARM-VIC?			\ select PL192 Vectored Interrupts
\  1 equ SimpleAborts?			\ reduce code space
  include %CpuDir%\IntArm4		\ interrupt handlers
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\drivers\serLPC23xxqi	\ queued interrupt driven serial
\  include %CpuDir%\drivers\serLPC23xxp	\ polled serial driver

1 equ DUMP?
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%\delays		\ delay common code
  include %CpuDir%\drivers\TickLPC23xx	\ timer tick
  ' Start-Timers AtCold
[else]
  include %CommonDir%\delays		\ delay common code
  include %CpuDir%\drivers\TickLPC23xx	\ timer tick
  ' Start-Clock AtCold
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support
[then]
\ *]


\ ****************
\ *S End of kernel
\ ****************

\ *[
buildfile MPE_2468.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." **************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE LPC2468 ROM PowerForth"
  cr version$ $. space BuildDate$ $.
  .banner
;
external
\ *]


\ *******************
\ *S Application code
\ *******************

\ *[
  \ to download code to the target
  Synonym Ser-Emit  seremit0
  Synonym Ser-Key   serkey0
  Synonym Ser-Key?  serkey?0
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support

  include %HwDir%\IAP23xx		\ IAP code
  include %HwDir%\Reprog\Reflash	\ Reflash system
  include %HwDir%\FlashTables		\ Flash sector tables
  include %HwDir%\FlashProg		\ higher level Flash routines

  include %CpuDir%\drivers\RTC23xx	\ real time clock
  include %CpuDir%\drivers\i2cLPC23xx	\ I2C on channel 0.
\ *[
  include %AppDir%\EMC-MPE_2468		\ external memory
\  include %HwDir%/emc-E2468    \ external memory
  include %CpuDir%/LPC-GPIO		\ easy pin access
  include %CpuDir%/drivers/pwm23xx	\ PWM driver
  include %CpuDir%/drivers/adc23xx	\ ADC driver
  include %AppDir%\UserLEDs.fth		\ on-board User LED driver
\ *]

  include %CpuDir%\Drivers\dmaLPC23xx	\ DMA driver
  include %AppDir%\hwMci-MPE_2468	\ MCI hardware config
  include %FatFiler%\mci23xx.bld	\ FAT file system code
  include %FatFiler%\FATtests		\ test code


\ *P  USB Device Stack
PowerNet? 0= [if]
  include %CommonDir%/DebugTools
[then]
  include %AppDir%\USB_HW-MPE_LPC2468
  include %UsbDir%\UsbDefs         \ USB definitions/equates
  include %AppDir%\MscConfig-MPE_2468       \ configuration file
\  include %UsbDir%\CDCconfig      \ USB configuration file
\  include %UsbDir%\CdcMscConfig   \ USB configuration file
  include %UsbDir%\UsbHwLPC2xxx    \ USB hardware layer for LPC2xxx
  include %UsbDir%\UsbBase         \ USB base operations
CDC? [if]
  include %UsbDir%\CdcDrv          \ Communications Device Class
[then]
MSC? [if]
  include %UsbDir%\MscDrv          \ Mass Storage Class
[then]
  include %UsbDir%\UsbEp0          \ USB Endpoint 0 layer

PowerNet? [if]
1 equ XC?
1 equ CPU=ARM
1 equ Phy#	\ -- n
\ *G Single PHY: The number of the PHY in use: 0..31
  include %AppDir%\PNconfig		\ PowerNet configuration file
  include %IpStack%\NetCode		\ network order stuff
  include %AppDir%\WebConfig		\ configuration tools
  include %AppDir%\Ether-MPE_LPC2468	\ Ethernet driver
  include %IPstack%\PowerNet.bld	\ PowerNet build file
  include %IPstack%\Examples\WebPost	\ File upload
[else]
1 equ XC?
1 equ CPU=ARM
1 equ sniff?
1 equ pingit?
  include %ExampleDir%\NetCode		\ network order stuff
  include %AppDir%\Ether-MPE_LPC2468	\ Ethernet driver
[then]
\ *]

\ *P By default, the HTTP server is set up to use files from the
\ ** directory *\i{\PAGES} on the SD card. To run the HTTP server
\ ** initially copy the files from the PowerNet *\i{TestPages}
\ ** directory into the *\i{\PAGES} directory on a card.

\ *P You can test the card and FAT file system.
\ *E   initFATfs .    \ return 0 on success
\ **   dir            \ list root directory
\ **   cwd PAGES      \ switch to PAGES directory
\ **   dir            \ list pages directory
\ *P When this is good, you can proceed to start the web server.
\ *C   PowerNet
\ *P By default, this acquires an IP address by DHCP. Make sure
\ ** that a DHCP server is running on your network, e.g. by enabling
\ ** the DHCP server on your network server or ADSL router.

((
: debugprint	\ --
  10000 0 do  i .  loop
;
))


\ **********************
\ *S Select example code
\ **********************

  include %AppDir%\SDRAM-test
  include %AppDir%\I2C-MPE_2468		\ code for on-board I2C devices
  include %AppDir%\PinTest

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ *S Finishing up
\ ***************

: startAll	\ --
\ *G Start all the the selected components.
  initFATfs abort" Can't start file system"
  PowerNet
  startUSBsys
;


\ *[
libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
sec-base 0				\ -- addr 0
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $18 + @ +
  over $1C + @ +
  negate
swap $14 + !

decimal
\ cr ." PC = " here dup .dword ."  " .

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build

\ Define device, speed (0=auto) and project file
\ Download the target image when compilation finishes
jlink LPC2468 100 Project.jlink	\ JLINK and device setup
\ -download-prompts			\ no prompting for download
afterwards  download-sections		\ download image after FINIS

FINIS
\ *]


\ ======
\ *> ###
\ ======

