\ MscConfig.fth - USB Mass Storage Class configuration

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ==========
\ *! mscconfig
\ *T USB MSC configuration
\ =========
\ *P The file *\i{MscConfig.fth} contains a USB configuration for
\ ** a Mass Storage Class (MSC) application.


\ ************************
\ *S Primary Configuration
\ ************************
\ *P Note that interface numbers must be a contiguous set
\ ** starting at 0.

0 equ dbgUHW?	\ -- flag
\ *G Set true to get debug information from the USB hardware
\ ** layer. Some of the debug output *\b{must} be done using
\ ** polled serial drivers.
0 equ dbgCore?	\ -- flag
\ *G Set true to get debug information from the USB core layer.
\ ** Some of the debug output *\b{must} be done using polled
\ ** serial drivers.
0 equ UsbPowered?	\ -- 0/1 ; bus powered?
\ *G Set to one if bus powered.
1 equ UsbDMA?		\ -- 0/1 ; uses DMA?
\ *G Set to one if any endpoints other than EP0 use DMA. At the
\ ** moment we assume that EP0 does not use DMA.
$0000:0000 equ UsbDMAEP	\ -- x ; DMA endpoints

#64 equ /maxpacket0	\ -- u
\ *G Maximum size of packets on endpoint 0.
#64 equ /maxpacket	\ -- u
\ *G Maximum size of packets on endpoints other than 0.

#32 equ #EPs	\ -- u
\ *G Number of endpoints we are are going to deal with. On a
\ ** 32 bit CPU, there are 32 bits per word. We need separate
\ ** bits for IN and OUT, so there's a practical maximum here
\ ** of 32, and USB supports 16 endpoints, each with IN and
\ ** OUT capability. Endpoint 0 is always IN and OUT capable.
4 equ #USBifs		\ -- u
\ *G Number of interfaces we can expand to eventually.

0 equ HID?	\ -- flag
\ *G True if a HID device, e.g. mouse, is to be supported.
1 equ MSC?	\ -- flag
\ *G True if a mass storage class (MSC) device is to be supported.
0 equ CDC?	\ -- flag
\ *G True if a communications device class (CDC) is to be supported.
\ ** The most common of these is a USB serial port.
0 equ AUDIO?	\ -- flag
\ *G True if an audio device is to be supported.
1 equ CLASS?	\ -- flag
\ *G Set this to 1 if any of the classes above are set to 1.

MSC? [if]
0 equ MscIf#		\ -- b
\ *G Interface number of the mass storage class.
$82 equ MscBinEP	\ -- ep
\ *G MSC Bulk IN endpoint.
$02 equ MscBoutEP	\ -- ep
\ *G MSC Bulk OUT endpoint.
0 equ dbgMSC?		\ -- flag
\ *G set non-zero to get debug information from the MSC driver.
MscBinEP EP>mask equ MscBinEPMask	\ -- mask
\ *G Core layer bit mask for the MSC bulk IN endpoint.
\ ** Used to avoid run-time calculation.
MscBoutEP EP>mask equ MscBoutEPMask	\ -- mask
\ *G Core layer bit mask for the MSC bulk OUT endpoint.
\ ** Used to avoid run-time calculation.
[then]


\ **************
\ *S USB Descriptors
\ **************
\ *P This section contains the USB descriptors. There is a descriptor
\ ** for the device, for its configuration, and for various strings.

\ *P Strings are accessed using a "string index". Aprt from zero, which
\ ** has a predefined meaning to define the language, string indices
\ ** (1..255) are defined by the application. Here the indices are the
\ ** offset in bytes from the start of a string table. If you need more
\ ** than 255 bytes of string descriptors, change the index to be the
\ ** offset into a table of string descriptor addresses.

Create CfgDesc	\ -- addr
\ *G The configuration descriptor has a primary configuration, one
\ ** interface, and two endpoints. If there is more than one
\ ** configuration, they must follow the first one
\ -- configuration
  9 c,			\ descriptor size
  2 c,			\ configuration descriptor
  #32 lew,		\ total size, this + interface + 2 endpoints
  1 c,			\ number of interfaces
  1 c,			\ configuration ID
  0 c,			\ no configuration string
  $C0 c,		\ $80=bus powered, $C0=self powered
  #5 c,		  \ 2*5=10ma power needed
\ -- interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  MscIf# c,		\ MSC interface number
  0 c,			\ alternate setting number
  2 c,			\ number of endpoints
  8 c,			\ Class - Mass Storage Class
  6 c,			\ SCSI transparent command set
  $50 c,		\ protocol - bulk only
  0 c,			\ no interface string
\ -- endpoint 2 in
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  MscBinEP c,		\ dir and ep# - IN 2
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- endpoint 2 out
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  MscBoutEP c,		\ dir and ep# - OUT 2
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- end, or insert more configuration descriptors
  0 lew,		\ needed by software

create StrDesc	\ -- addr
\ *G String descriptor and following strings.
l: Str0		\ -- string 0, language
  4 c,			\ descriptor size
  3 c,			\ string
  $0409 lew,		\ type - US English
l: Str1		\ -- string 1, manufacturer
  usb", www.mpeforth.com"
align  l: str2	\ -- string 2, product
  usb", MPE Mass Storage demo"
align l: str3	\ -- string3, serial number
  usb", 0123456789ABCDEF"
		\ -- end
  0 lew,		\ needed by software

Str1 Str0 - equ Str1idx
Str2 Str0 - equ Str2idx
Str3 Str0 - equ Str3idx

create DevDesc	\ -- addr
\ *G The device descriptor.
  #18 c,		\ descriptor size
  1 c,			\ device descriptor
  $0110 lew,		\ USB 1.1
  0 c,			\ class is specified in the interface
  0 c,			\ subclass is specified in the interface
  0 c,			\ protocol is specified in the interface
  /MaxPacket0 c,	\ endpoint 0 max size
  $0D59 lew,		\ Vendor ID (from www.usb.org) - VID=0D59 for MPE developers
  $FF00 lew,		\ Product ID - PID=FF00..FFFF for MPE developers
  $0001 lew,		\ Release ID - assigned by vendor
  Str1idx c,		\ Manufacturer string index
  Str2idx c,		\ Product string index
  Str3idx c,		\ serial number string index - 0=none
  1 c,			\ number of configurations
\ -- end
  0 lew,		\ needed by software


\ ======
\ *> ###
\ ======

decimal
