((
                SDRAM Test Code
                ~~~~~~~~~~~~~~~

Written by:   Barry Culver
              Culver Consultancy Ltd.
              www.culverconsultancy.co.uk
              
Use as you wish, strictly at your own risk; this code is not warranted or
supported in any way.

This file contains forth words to perform a test on the 16M byte SDRAM area.
Exclusive access to the SDRAM during the test is assumed and contents are lost.

The test is fairly 'quick* and dirty', it is primarily intended to test the
hardware, rather than for a faulty SDRAM chip.  A good chip test needs knowledge
of the specific SDRAM device layout and operation.
(* Quick to write, not to run!)

This test has pauses to check the operation of the refresh circuits.  However
for a good chip at room temperature, memory may hold up for seconds without
refresh.  Running this test with the board at an elevated temperature will give
additional confidence.  Take care if doing this - although the chips are rated
for 70C, power supply circuits may not be designed to cope with this ambient
temperature and may overheat!  Don't blame me if you melt your board!

The test is of course fine for testing SRAM and other types of direct-access,
read/write memory.  You may want to remove the 5s pauses for testing non-DRAM
memory types.  Redefine the identifiers TESTbase, TESTsize as required.

Tests are made for Byte, Word and Long access.  The test will fail/crash in a
hardware specific way if the test memory region does not support all forms of
access.

** THIS TEST TAKES A LONG TIME TO RUN AND ONLY OCCASIONALLY PRODUCES PROGRESS
MESSAGES **

ASSUMES:
  Timers running, timer interrupt enabled (i.e. delays available)
  SDRAM enabled, AND configured correctly for the connected device
  SDRAMbase, SDRAMsize externally defined
  32 bit forth
  
Changes
~~~~~~~
27Oct11   BjC   First release

))
DECIMAL


SDRAMbase   EQU   TESTbase    \ test base address
SDRAMsize   EQU   TESTsize    \ bytes to test

    \ The data used for the test - 257 bytes, a prime number
    \ Also used as an array of 127 words and 61 longs.
HEX
create  TestData_ 
 01 c,  FF c,  02 c,  00 c,  04 c,  55 c,  08 c,  00 c,
 10 c,  A5 c,  20 c,  05 c,  40 c,  50 c,  80 c,  0A c,
 01 c,  A0 c,  03 c,  AA c,  06 c,  55 c,  0C c,  3C c,
 18 c,  C3 c,  30 c,  F0 c,  60 c,  0F c,  C0 c,  01 c,
 03 c,  80 c,  07 c,  40 c,  0E c,  20 c,  1C c,  10 c,
 38 c,  08 c,  70 c,  04 c,  E0 c,  02 c,  C0 c,  01 c,
 FE c,  01 c,  FD c,  02 c,  FB c,  04 c,  F7 c,  08 c,
 EF c,  10 c,  DF c,  20 c,  BF c,  40 c,  7F c,  80 c,
 FC c,  03 c,  F9 c,  06 c,  F3 c,  0C c,  E7 c,  18 c,
 CF c,  30 c,  9F c,  60 c,  3F c,  C0 c,  7F c,  80 c,
 00 c,  00 c,  00 c,  00 c,  00 c,  00 c,  00 c,  80 c,
 80 c,  80 c,  80 c,  80 c,  80 c,  80 c,  FF c,  00 c,
 00 c,  FF c,  FF c,  00 c,  FF c,  AA c,  55 c,  55 c,
 AA c,  AA c,  3C c,  3C c,  3C c,  C3 c,  3C c,  00 c,
 81 c,  81 c,  42 c,  42 c,  24 c,  24 c,  18 c,  18 c,
 99 c,  66 c,  66 c,  66 c,  99 c,  99 c,  69 c,  96 c,
 06 c,  60 c,  30 c,  03 c,  03 c,  63 c,  63 c,  63 c,
 36 c,  00 c,  00 c,  00 c,  00 c,  00 c,  FF c,  FF c,
 FF c,  FF c,  FF c,  00 c,  01 c,  02 c,  03 c,  04 c,
 05 c,  06 c,  FF c,  FE c,  FD c,  FC c,  FB c,  FA c,
 F8 c,  07 c,  08 c,  09 c,  0A c,  0B c,  F7 c,  F6 c,
 F5 c,  F4 c,  F3 c,  0C c,  0D c,  0E c,  0F c,  10 c,
 F2 c,  F1 c,  F0 c,  EF c,  ED c,  11 c,  12 c,  13 c,
 14 c,  15 c,  EC c,  EB c,  EA c,  E9 c,  E8 c,  E7 c,
 E6 c,  16 c,  17 c,  18 c,  19 c,  1A c,  1B c,  1C c,
 E5 c,  E4 c,  E3 c,  E2 c,  E1 c,  E0 c,  DF c,  1D c,
 1E c,  1F c,  20 c,  21 c,  DE c,  DD c,  DC c,  DB c,
 DA c,  22 c,  23 c,  24 c,  25 c,  26 c,  27 c,  28 c,
 29 c,  D9 c,  D8 c,  D7 c,  D6 c,  D5 c,  D5 c,  D4 c,
 D3 c,  D2 c,  2A c,  2B c,  2C c,  2D c,  2E c,  2F c,
 30 c,  31 c,  32 c,  D2 c,  D1 c,  D0 c,  CF c,  CE c,
0CD c,  CC c,  CB c,  CA c,  33 c,  34 c,  35 c,  36 c,
 37 c,  38 c,  39 c,  3A c,  3B c,  C9 c,  C8 c,  C7 c,
 C6 c,
DECIMAL

157   equ   TEST_BYTES
127   equ   TEST_WORDS
61    equ   TEST_LONGS


variable    Errors

: FILL-SDRAM-BYTES     \ table_offset | -;  fill with test bytes (byte access)
  TESTsize 0 do
    TestData_ over + c@      \ test byte
    i TESTbase + c!
    1+                        \ increment table offset
    dup TEST_BYTES >= if    \ wrap
      drop 0            
    endif
  loop
  drop                      \ offset
;

: FILL-SDRAM-WORDS     \ table_offset | -;  fill with test data (word access)
  TESTsize 0 do
    TestData_ over + w@      \ test byte
    $FFFF xor                \ invert bits
    i TESTbase + w!
    2+                        \ increment table offset
    dup TEST_WORDS >= if    \ wrap
      drop 0            
    endif
  2 +loop
  drop                      \ offset
;

: FILL-SDRAM-LONGS  \ table_offset | -;  fill with test data (longword access)
  TESTsize 0 do
    TestData_ over + @      \ test byte
    i TESTbase + !
    4+                        \ increment table offset
    dup TEST_LONGS >= if    \ wrap
      drop 0            
    endif
  4 +loop
  drop                      \ offset
;


: COMPARE-SDRAM-BYTES    \ table_offset | errors; compare with source data (byte access)
  TESTsize 0 do
    TestData_ over + c@      \ test byte
    i TESTbase + c@ <> if   \ compare with saved
      1 Errors +!
    endif
    1+                      \ increment table offset
     dup TEST_BYTES >= if   \ wrap
      drop 0             
    endif
  loop
  drop
;

: COMPARE-SDRAM-WORDS    \ table_offset | errors; compare with source data (byte access)
  TESTsize 0 do
    TestData_ over + w@      \ test byte
    $FFFF xor                \ invert bits
    i TESTbase + w@ <> if   \ compare with saved
      1 Errors +!
    endif
    2+                      \ increment table offset
     dup TEST_WORDS >= if   \ wrap
      drop 0             
    endif
  2 +loop
  drop
;

: COMPARE-SDRAM-LONGS    \ table_offset | errors; compare with source data (byte access)
  TESTsize 0 do
    TestData_ over + @      \ test byte
    i TESTbase + @ <> if   \ compare with saved
      1 Errors +!
    endif
    4+                      \ increment table offset
     dup TEST_LONGS >= if   \ wrap
      drop 0             
    endif
  4 +loop
  drop
;


: TEST-SDRAM      \ - | -;
  Errors off              \ reset count
  63 0 do
    i fill-sdram-bytes
    5000 ms               \ pause to test refresh
    i compare-sdram-bytes
    
  \ repeat for word access
    i 2* fill-sdram-words
    5000 ms
    i 2* compare-sdram-words
    
  \ repeat for long access
    i 4* fill-sdram-longs
    5000 ms
    i 4* compare-sdram-longs
    
    cr ." SDRAM test: pass " i .
    ."  Errors = " Errors @ u.
  loop
  Errors @ ?dup if
    cr ." Test *FAILED* - total errors = " .
  else
    cr ." Test PASSED - no errors."
  endif
;

: SHOW-EMC-REGS   \ - | -; display all EMC SDRAM registers
  binary
  _EMC
  cr ."               1  -   bit  -  0"
  cr ." register      5      87      0"
  dup emcControl + @
  cr ." EMC_Control:  " 16 u.r
  dup emcConfig + @
  cr ." EMC_Config:   " 16 u.r
  dup emcDynRP + @
  cr ." EMC_DynRP:    " 16 u.r
  dup emcDynRAS + @
  cr ." EMC_DynRAS:   " 16 u.r
  dup emcDynSREX + @
  cr ." EMC_DynSREX:  " 16 u.r
  dup emcDynAPR + @
  cr ." EMC_DynAPR:   " 16 u.r
  dup emcDynDAL + @
  cr ." EMC_DynDAL:   " 16 u.r
  dup emcDynWr + @
  cr ." EMC_DynWr:    " 16 u.r
  dup emcDynRC + @
  cr ." EMC_DynRC:    " 16 u.r
  dup emcDynRFC + @
  cr ." EMC_DynRFC:   " 16 u.r
  dup emcDynXSR + @
  cr ." EMC_DynXSR:   " 16 u.r
  dup emcDynRRD + @
  cr ." EMC_DynRRD:   " 16 u.r
  dup emcDynMRD + @
  cr ." EMC_DynMRD:   " 16 u.r
  dup emcDynRdCfg + @
  cr ." EMC_DynRdCfg: " 16 u.r
  dup emcDynRasCas0 + @
  cr ." EMCDynRasCas0:" 16 u.r
  dup emcDynCfg0 + @
  cr ." EMC_DynCfg0:  " 16 u.r
  dup emcDynCtrl + @
  cr ." EMC_DynCtrl:  " 16 u.r
  dup emcDynRfrsh + @
  cr ." EMC_DynRfrsh: " 16 u.r
  dup emcStatus + @
  cr ." EMC_Status:   " 16 u.r 
  drop
  decimal
;



