\ hwMci-MPE_2468.fth - hardware set up for M.P.E. LPC-2368 MCI driver

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! hwMci-MPE_2468
\ *T Configuration of LPC2468 MCI driver for M.P.E. LPC-2468 Dev. Board
\ ===========
\ *P The MCI unit's SD card pins can be assigned to more than one
\ ** set of pins. In addition, the Card Detect and Write Protect
\ ** pins are assigned to a wide range of GPIO lines on different
\ ** boards.

\ *P The *\i{hwMcixxx.fth} files contain the hardware
\ ** configurations for various LPC23xx/4xx boards. They are
\ ** compiled before the main MCI driver file *\i{mciLPC23xx.fth}.

\ *P on this board, MCI lines are assigned as follows:
\ **    Board       rev F       rev I      Iss 2
\ **    MCI_D0      P1.6        P1.6       P1.6
\ **    MCI_D1      P1.7        P1.7       P1.7
\ **    MCI_D2      P1.11       P1.11      P1.11
\ **    MCI_D3      P1.12       P1.12      P1.12
\ **    MCI_CLK     P1.2        P1.2       P1.2
\ **    MCI_CMD     P1.3        P1.3       P1.3
\ **    MCI_Power   P0.21       P0.21      P0.21
\ *P In this implementation, P4.15 is used as a separate Card Detect
\ ** input.

\ ** In most cases, the default value of *\fo{/pcdMCI} below is
\ ** appropriate.

1 equ /pcdMCI	\ -- u
\ *G Peripheral clock divider for MCI unit. Set this to one if
\ ** performance is more important than power consumption.
/pcdMCI PCLKdiv 24 lshift equ mciPCLK1val	\ -- val
\ *G Bit value for MCI peripheral clock divider.
$0300:0000 equ mciPCLK1mask		\ -- mask
\ *G Bit mask for MCI peripheral clock divider.

2 [ps
  2 +ps  3 +ps  6 +ps  7 +ps  11 +ps  12 +ps
ps] equ mciPS2val  equ mciPS2mask
\ *G MPE LPC-2468: PINSEL2 register value and mask for MCI.

2 [ps
  21 +ps
ps] equ mciPS1val  equ mciPS1mask
\ *G MPE LPC-2468: PINSEL1 register value and mask for MCI.

: initMCIconfig	\ --
  bit3 bit0 or _SCS or!			\ MCI power active high, FIO for P0/1

  bit15 _FIO4				\ set P4.15 as (card detect) input
  2dup FIODIR + bic!  FIOMASK + bic!

  mciPCLK1val mciPCLK1mask _PCLKSEL1 setMask	\ peripheral clock
  mciPS2val  mciPS2mask _PINSEL2 setMask	\ select pins
  mciPS1val  mciPS1mask _PINSEL1 setMask
;

: CDin?		\ -- flag
\ *G Return non-zero if a card is present.
  _FIO4 FIOPIN + @ bit15 and 0=  ;    \ P4.15

: WPin?		\ -- flag
\ *G Return non-zero if the write protect slider is set.
\ ** For this board, we always return zero.
\ ** Micro SD cards have no write protect switch and
\ ** the PCB has no link or switch
  0  ;


\ ======
\ *> ###
\ ======

decimal
