\ SA1100 startup code

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))

aif? [if]

\ Use default Reset handler in INITARM.FTH (branches to ECLD)

[else]

asmcode 
L: Reset_SA1100
  bl Init_General			\ GPIO and so on
  bl Init_DRAM				\ Initialise memory etc.
  bl Init_Interrupt			\ Initialise interrupts
  b ECLD
end-code

Reset_SA1100    ResetV  !		\ Slot into the vector table       
    
PROC Init_General
  mvl r0, # OS_T			\ Set OS-Timer counter to zero
  mov r1, # 0
  str r1, [ r0, # _oscr ]
  
  mvl r0, # MC				\ Default adjustments for CS1 and CS0
  mvl r1, # MSC0_xxMhz
  str r1, [ r0, # _msc0 ]
  
  mvl r0, # GPIO			\ Set GPIO Direction Register
  mvl r1, # GPIO_Dir
  str r1, [ r0, # _gpdr ]

  mvl r1, # $FFFFFFFF 			\ GPIO Pin Output Set Register (all inputs on)
  str r1, [ r0, # _gpsr ]

  mov r1, # 0				
  str r1, [ r0, # _grer ]		\ GPIO Rising Edge Detect Register (disable)
  str r1, [ r0, # _gfer ]		\ GPIO Falling Edge Detect Register (disable)
  next,
end-code

PROC Init_DRAM
  mvl r0, # MC
  mvl r1, # DRAM_CTL			
  str r1, [ r0, # _mdcnfg ]		\ MDCNFG DRAM Configuration Register
  mvl r1, # DRAM_CAS0
  str r1, [ r0, # _mdcas0 ]		\ MDCAS0 DRAM CAS Waveform Shift Register 0
  mvl r1, # DRAM_CAS1
  str r1, [ r0, # _mdcas1 ]		\ MDCAS1 DRAM CAS Waveform Shift Register 1
  mvl r1, # DRAM_CAS2
  str r1, [ r0, # _mdcas2 ]		\ MDCAS1 DRAM CAS Waveform Shift Register 2

((
\ This code sets up the CPSRs of each mode and the RSP pointer.
\ The code in INTARM.FTH provides a nest of stacks which are
\ set up by the interrupt entry code. Only compile this code block
\ if you are going to handle interrupt entry and exit yourself.
\ Note the CPSR registers are set up in ECLD in INITARM.FTH.
  mov r0, # InterruptMask irq_32 or
  msr CPSR, r0                         
  mvl r0, # IRQ_StackBase		\ IRQ mode stack
  mov rsp, r0 

  mov r0, # InterruptMask svc_32 or
  msr SPSR, r0                         

  mov r0, # InterruptMask fiq_32 or 
  msr CPSR, r0                         
  mvl r0, # FIQ_StackBase		\ FIQ mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask und_32 or
  msr CPSR, r0                         
  mvl r0, # UND_StackBase		\ UND mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask abt_32 or
  msr CPSR, r0                         
  mvl r0, # ABT_StackBase		\ ABT mode stack
  mov rsp, r0                

  mov r0, # InterruptMask svc_32 or
  msr CPSR, r0

\ If you intend to run Forth in USR mode rather than SVC mode then you
\ need to set up a stack that can cope with switches to SVC mode (i.e. SWI
\ calls). The following two lines will do this providing SVC_StackBase is
\ declared in the control file. As Forth will normally run in SVC mode, it will
\ set up its own stacks.

  mvl r0, # SVC_StackBase		\ SVC mode stack
  mov rsp, r0
))
  next,
end-code  

PROC Init_Interrupt
  mvl r0, # IC				\ Interrupt Controller Register
  mov r1, # 0
  str r1, [ r0, # _icmr ]		\ Interrupt Controller Mask Register
  					\ pending interrupt is masked from becoming active
  str r1, [ r0, # _iclr ]		\ Interrupt Controller Level Register
  					\ all interrupts are IRQ-Interrupts

\ Request all Interrupt-Sources

\ Request for GPIO-Interrupts
  mvl r0, # GPIO			
  mvl r1, # $ffffffff
  str r1, [ r0, # _gedr ]		\ Set GPIO Edge Detect Status Register

\ Request for OS-Timer-Interrupts
  mvl r0, # OS_T			 
  mov r1, # $f
  str r1, [ r0, # _ossr ]		\ OS Timer Status Register (Match Status Ch0 ..3)

\ Enable IRQ-interrupts
\  mrs	r0, CPSR
\  bic	r0, r0, # IRQDisable
\  msr	CPSR, r0
  next,
end-code
  
PROC SwitchLED				\ R0 = LED mask
  mvl r1, # GPIO
  ldr r2, [ r1, # _gplr ]		\ read status
  and r2, r2, r0			\ LED & Status
  cmp r2, r0
  str .eq r0, [ r1, # _gpcr ]		\ Clear GPIO
  str .ne r0, [ r1, # _gpsr ]		\ Set GPIO
  next,
end-code

[then]

