\ HitexLEDs.fth - LED driver for Hitex STR912 EVB

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20070626 SFP001 Initial release.
))

only forth definitions
decimal

\ ============
\ *! hitexleds
\ *T Hitex STR912 LED driver
\ ============
\ *P The seven segment LEDs are attached to port 8, and the digit
\ ** enables are connected to P9.1 (msd) and P9.0 (lsd). The data
\ ** bits are 0 for on and 1 for off. The decimal point is bit 7.
\ *E       0
\ **      --- 
\ **     |    |
\ **   5 |    |1
\ **     | 6  |
\ **      --- 
\ **     |    |
\ **   4 |    |2
\ **     | 3  |
\ **      ---   * 7

create numbers	\ -- addr
\ *G Table of digit patterns for the LEDs. Set bits are off.
  0x40 c,	\ 0
  0x79 c,	\ 1
  0x24 c,	\ 2
  0x30 c,	\ 3
  0x19 c,	\ 4
  0x12 c,	\ 5
  0x02 c,	\ 6
  0x78 c,	\ 7
  0x00 c,	\ 8
  0x10 c,	\ 9

: initLEDs	\ --
\ *G Initialise the GPI0 ports for the LEDs.
  GPIO8mask GPIO9mask or  _SCU		\ -- mask addr
  2dup SCU_PCGR1 + or!			\ enable clocks
  tuck SCU_PRR1 + or!			\ take out of reset
  0 over SCU_GPIOTYPE + 8 cells + !	\ port 8.7:0 push-pull o/ps
  3 swap SCU_GPIOTYPE + 9 cells + bic!	\ port 9.1:0 push-pull o/ps
  $FF _GPIO8 GPIO_DIR + !		\ port 8 = o/p
  $03 _GPIO9 GPIO_DIR + or!		\ port 9.1:0 = o/p
;

$03 2 lshift equ SelMask	\ -- mask
\ *G Bit mask added to GPIO9 DR address to belect bits 0 and 1
$FF 2 lshift equ DataMask	\ -- mask
\ *G Bit mask added to GPIO8 DR address to belect bits 7..0.

: SelLED	\ n --
\ *G Select LED 0 or 1.
  1 swap lshift invert			\ -- mask ; bit=0 to select
  _GPIO9 GPIO_DR + SelMask + c!		\ write out
;

: n>led		\ n --
\ *G Send value n=0..9 to LED.
  numbers + c@				\ get pattern from table
  _GPIO8 GPIO_DR + DataMask + c!	\ write out to port
;

: TestLEDs	\ --
\ *G Display a repeating count of 0..9 on LED0.
  initLEDs  1 SelLED  init-ser1
  0
  begin
    #1000000 0 do loop			\ 9 clocks per loop, approx .1 second
\    #10000000 0 do loop			\ 9 clocks per loop, approx 1 second
    dup 1 and SelLED
    1+ dup 9 > if
      drop 0  sercr1
    endif
    dup n>led
    dup [char] 0 + seremit1
  again
;


\ ======
\ *> ###
\ ======

decimal


