\ InitSTR91x.fth - STR91x generic startup code

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ =============
\ *! initstr91x
\ *T STR91x initialisation
\ =============

interpreter
: binpower	\ u -- power
\ Convert a power of two to its power, e.g. 32 -> 5. Aborts
\ on failure.
  -1 swap  #31 0 do		\ -- -1 u
    dup 1 i lshift = if
      nip i swap  leave
    endif
  loop
  drop  dup -1 = if
    cr ." Invalid power of two" abort
  endif
;
target


\ ***************************************
\ Generic start up table and entry points
\ ***************************************

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr 	pc, ResetV              \ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  dd    0                       \ 14 - unused
  ldr 	pc, [ pc, # $-FF0 ]	\ 18 - IRQ interrupts from VicVectAddr @ $FFFFF030
  ldr 	pc, FIQV		\ 1C - FIQ interrupts

L: Soft_Vectors	\ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
\  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code

Vector_End Vector_Start - equ Vector_Len


\ ***************
\ *S Startup data
\ ***************

\ Required Forth pointers
L: CLD1		                        \ cold start word
  0 ,				        \ compiler will fill up with address of word to execute at cold start
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,


\ ================
\ *N System config
\ ================

\ Required peripheral pointers
l: ^SCU				\ system control unit
  _SCU ,
l: ^FMI
  _FMI ,


\ ==============
\ *N SRAM set up
\ ==============

\ $0180 equ SCU_SCR0_def	\ -- x
$0181 equ SCU_SCR0_def	\ -- x
\ *G Default config register 0: EMI 2/high/mux; SRAM=32kb, 1 w/s, PFQBC=on

SCU_SCR0_def  IRAMkb #32 <> [if]
  IRAMkb #64 = [if]
    1 3 lshift or
  [else]
    IRAMkb #96 = [if]
      2 3 lshift or
    [else]
      cr ." IRAMkb value not 32/64/96 kb" abort
    [then]
  [then]
[then]
 equ SCU_SCR0_val

l: ^SCU_SCR0_val
  SCU_SCR0_val ,


\ ===============
\ *N Flash set up
\ ===============

system-speed 66000000 >
[if] 3 [else] 2 [then]		\ number of wait states
1- #11 lshift			\ in bits 12:11
bit4 or				\ bit 4 set for high speed operation
2 lshift equ FMIcfgval	\ -- x
\ *G The flash configuration register is written as an address offset.
\ ** Consequently it is shifted 2 bits left as the Flash is 32 bits
\ ** wide.

l: ^FMIcfgval
  FMIcfgval ,


\ =============
\ *N PLL set up
\ =============
\ *P The PLL is laid out in the following way, using three dividers,
\ ** M, N and P. Note that P represents a power of two.
\ *E          ---     -----    -----
\ ** fOSC ->-| M |-X-| VCO |-+-| 2^P |->- fPLL
\ **          ---  |  -----  |  -----
\ **               |         |
\ **               |  ----   |
\ **                -| 2N |--
\ **                  ----
\ *P When the PLL is active, it generates an output frequency
\ ** (fPLL) according to the following equation:
\ *C  fPLL = (2 x N x fOSC)/(M x 2^P)
\ *P Where the values of M, N and P must satisfy the following
\ ** constraints:
\ *B 1 <= M <= 255
\ *B 1 <= N <= 255
\ *B 0 <= P <= 5
\ *B 1 MHz <= fOSC/M <= 2 MHz
\ *B 200 MHz <= (2 x N x fOSC) / M <= 622 MHz
\ *B 4 MHz <= fOSC <= 25 MHz

\ *P You program the M, N and P values by writing to the PLL
\ ** configuration register (SCU_PLLCONF). The default value of
\ ** this register is $3C019, corresponding to PLL disabled, P=3,
\ ** N=192, M=25, which for a 25Mhz crystal leads to:
\ ** PLL freq = 384MHz, FPLL=48. For 96MHz operation, change
\ ** the P value to 2, and set the USB divider to reduce the USB
\ ** clock to 48MHz.

$3C019 equ PLLCONF_def	\ -- x
\ *G The default/reset value of SCU_PLLCONF.
$AC019 equ PLLCONF_val	\ -- x
\ *G The application value of SCU_PLLCONF, 25MHz crystal, 96MHz CPU.
$20002 equ CLKCNTR_def	\ -- x
\ *G Default value of SCU_CLKCNTR.
$20000  RCLKDIVmask or  AHBDIVmask or  APBDIVmask or  BRSELmask or
  equ CLKCNTR_val	\ -- x
\ *G Application value of SCU_CLKCNTR.

l: ^PLLconfs
  CLKCNTR_def ,		\ +00: FMSTR=FOSC, EMIclk=HCLK/2
  PLLCONF_def ,		\ +04: reset value
  PLLCONF_val ,		\ +08: application value
  CLKCNTR_val ,		\ +0C: FMSTR=FPLL, EMIclk=HCLK/2


\ ************
\ Startup code
\ ************

Proc SmallWait	\ used when a few clocks have to be wasted for stabilisation
  mov	    r0, # #10
  begin,
    sub .s    r0, r0, # 1
  eq, until,
  mov       pc, lr
end-code

Proc InitReset	\ power up entry point through vector
\ The C examples waste a few cycles here, so we do too.
  bl       SmallWait

\ Set bits 17 (DTCM) and/or 18 (ITCM) of the Core Configuration
\ Control Register to preserve RAM read/write ordering.
\ XBench32 slows from 7770 ms to 7830
  MOV     r0,     # 0x60000		\ set bits DTCM, ITCM
  ORR     r0, r0, # 0x00004		\ set bit FIM to default
  MCR     #15 0x1 r0, cr15, cr1, #0

\ Enable AHB write buffer and DTCM. See ARM966EStrm 5.3 for next bit.
\ If the ARM9E-S core performs a read from AHB address space or
\ an unbuffered write to AHB address space, the core stalls until
\ all write buffer entries are written. Draining the write buffer
\ ensures data coherency.
  MRC      #15 #0 r0, cr1, cr0, #0	\ Read CP15 register 1 into r0
\  bic      r0, r0, # $FF000000		\ clear unpredictable SBZ bits
\  bic      r0, r0, # $00FF0000
\  bic      r0, r0, # $00004000
\  bic      r0, r0, # $00000001
\  orr      r0, r0, # $00000F70		\ set unpredictable SBO bits
  ORR      r0, r0, # 0xC		\ Enable Write Buffer on AHB and DTCM
  MCR      #15 #0 r0, cr1, cr0, #0	\ Write CP15 register 1

\ set SCU_SCR0 configuration, basically SRAM size.
  ldr      r2, ^SCU			\ address of system control block
  ldr      r0, ^SCU_SCR0_val		\ required value in SCU_SCR0
  str      r0, [ r2, # SCU_SCR0 ]

\ set FMI configuration
  ldr      r3, ^FMI
  mov      r0, # BootFlashKb #32 / binpower
  str      r0, [ r3, # FMI_BBSR ]	\ set boot Flash memory size
  mov      r0, # NonBootFlashKb 8 / binpower
  str      r0, [ r3, # FMI_NBBSR ]	\ set non-boot Flash memory size
  mov      r0, # FlashBase 2 rshift
  str      r0, [ r3, # FMI_BBADR ]	\ set boot block base address
  mov      r0, # NBFlashBase 2 rshift
  str      r0, [ r3, # FMI_NBBADR ]	\ set non-boot block base address
  mov      r0, # $0018			\ enable both banks in Flash control reg.
  str      r0, [ r3, # FMI_CR ]
  mov      r0, # 3			\ clear FMI error status bits
  str      r0, [ r3, # FMI_SR ]

  mov      r1, # NBFlashBase
  mov      r0, # $060			\ Write Flash Config reg command (16 bits)
  strh     r0, [ r1, # 0 ]		\ to any Bank 1 address
  mov      r0, # 3			\ Write Flash Config confirm reg command (16 bits)
  ldr      r4, ^FMIcfgval		\ get config value as an address
  strh     r0, [ r1, ++ r4 ]		\ write config value to Flash Config reg.

\ Set up the PLL; R2 already points to the SCU
  ldr      r0, ^PLLconfs $00 +		\ table of PLL config, first is for SCU_CLKCNTR
  str      r0, [ r2, # SCU_CLKCNTR ]	\ write $0002:0002
  bl       SmallWait			\ for stabilisation
  ldr      r0, ^PLLconfs $04 +		\ table of PLL config, second is default config
  str      r0, [ r2, # SCU_PLLCONF ]	\ write default config
  ldr      r0, ^PLLconfs $08 +		\ table of PLL config, third is application config
  str      r0, [ r2, # SCU_PLLCONF ]	\ write application config
\ only do this phase if PLLCONF bit 19 is set
  begin,
    ldr      r0, [ r2, # SCU_SYSSTATUS ]	\ wait until locked
    and .s   r0, r0, # bit0
  ne, until,
  ldr      r0, ^PLLconfs $0C +		\ CLKCNTR application value
  str      r0, [ r2, # SCU_CLKCNTR ]	\ write $0002:0000
  bl       SmallWait
\ PLL is now configured

\ Set standard clock gating
  mov      r0, # $00DB bit5 or		\ add VIC to list
  str      r0, [ r2, # SCU_PCGR0 ]
\ Enable minimal set of system devices
  mov      r0, # $1000			\ take minimal set out of reset
  orr      r0, r0, # $53 bit5 or	\ add VIC to list
  str      r0, [ r2, # SCU_PRR0 ]

\ Initialise the stacks for INTARM3.FTH
  bl       InitStacks			\ see ARM\INITSTACKS.FTH


\ *****************************
\ Optional code copy
\ *****************************

\ If required, copy code into RAM for execution
((
  mov		r0, # Flash_start	\ source
  mov		r1, # RAM_start		\ dest
  mov		r2, # Flash_len		\ len, must be multiple of 4
l: Flash_loop
  ldr		r3, [ r0 ], # 4
  str		r3, [ r1 ], # 4
  sub .s	r2, r2, # 4
  b .ne		flash_loop
))


\ *********************
\ Start of Forth system
\ *********************

L: ECLD
\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

InitReset prog sec-base -  ResetV !	\ INITRESET is in LOW memory starting at 0.


\ ******************************************
\ Reset values for user and system variables
\ ******************************************

[undefined] sp-guard [if]
  0 equ sp-guard
[then]

\ initial values of user variables for standalone systems only
[undefined] umbilical? [if]
L: USER-RESET
  init-s0   tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,                 		\ r0
  0 ,  0 ,                              \ #tib, span
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

