\ MinSerSTR91xp.fth - STR91x polled RX, polled TX, serial communication drivers

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20070725 VRW002 Ported from SAM7xxxx
20060818 SFP001 Added UART selection code.
))

only forth definitions


\ *****************
\ Serial primitives
\ *****************

cell buffer: UARTaddr	\ -- addr
\ *G Holds address of UART to use.

l: ppUART
\ *G Holds the RAM address at which the UART address may be found.
  UARTaddr ,

IsLeaf : init-ser	\ --
\ *G The host has already set up the UART. Force the UART into
\ ** polled operation.
  CLD_UART @				\ if non-zero
  if  CLD_UART @  else _UART1  endif	\ use different UART
  dup ppUART @ !
  $0000 swap UART_IMSC + w!		\ disable interrupts
;

IsLeaf : emit 	\ char --
\ *G Transmit a character
  ppUART @ @
  begin
    dup UART_FR + w@ bit5 and 0=	\ bit 5 is TxFifo full
  until
  swap $FF and swap UART_DR + w!
;

IsLeaf : key?	\ -- t/f
\ *G Return true if a character is available.
  ppUART @ @ UART_FR + @ bit4 and 0=	\ bit 4 is RxFifo empty

;

IsLeaf : key	\ -- char
\ *G Wait for and return a character.
  ppUART @ @
  begin
    dup UART_FR + @ bit4 and 0=
  until
  UART_DR + w@ $FF and
;

: type		\ c-addr len --
\ *G Transmit a string.
  bounds
  ?do  i c@ emit  loop
;

: CR		\ --
\ *G Transmit a CR/LF pair.
  $0D emit  $0A emit
;

