\ Reflash.fth - handles reflashing the system

((
Copyright (c) 2004, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20061101 SFP002 Converted for SAM7.
20060701 SFP001 Added CopyFlash.
))


\ ==========
\ *! reflash
\ *T STR91x Reflashing
\ ==========
\ *S Introduction
\ *P If you destroy the application in the internal primary Flash,
\ ** you must use a JTAG programmer to reload a binary file. If you
\ ** have corrupted the secondary Flash, you must use a JTAG
\ ** programmer to reload a reprogram utility into it.

\ *P Once the Forth system is running again, you can use the word
\ ** *\fo{REFLASH ( -- )} to download a new binary image.
\ ** Note that *\fo{REFLASH} only handles binary memory image
\ ** files. These should be transferred using the XModem 128
\ ** (checksum) protocol. If you are using AIDE with the file
\ ** server enabled, a file selection dialog will appear
\ ** automatically after you have executed *\fo{REFLASH}.

\ *P The STR91x can only be reflashed from an application by
\ ** a program running from RAM or another Flash bank. By default
\ ** the reprogram utilities occupy the first 4kb of the
\ ** secondary Flash, which is mapped at $0040:0000 by the
\ ** default MPE code. If you change this, you will have to
\ ** rebuild and reflash the program loader in the secondary
\ ** Flash. The default serial loader is built by the control
\ ** file *\i{ReProg\ReprogSTR91x*.CTL}

\ *P Return from REFLASH or COPYFLASH is by rebooting the system.
\ ** The contents of the internal RAM and Flash are destroyed,
\ ** therefore any data that must be preserved should be saved.
\ ** If needed, you can modify the reprogramming code to
\ ** use the last sector of the primary Flashto hold data to be
\ ** preserved. Alternatively, use the remainder of the secondary
\ ** Flash.


\ ***************************
\ *S Code in main application
\ ***************************
\ *P The layout of the RAM code is defined in the source files
\ ** *\i{ReprogSTR91x*.ctl}. These define the first cell as
\ ** containg the Forth word to execute.

$00400000 equ reprogrun	\ -- addr
\ *G The run time address of the reprogramming code.
\ ** This *\b{must} match the start address of the *\fo{CDATA}
\ ** section defined for the reprogramming code, and that section
\ ** *\b{must not} overlap the run-time stacks and user area.

((
0 value ReflashDev	\ -- addr
 \ *G Holds 0 or a UART base address. If set to 0, the reflash
 \ ** code will use the default device for the XModem transfer.
 \ ** Otherwise it will use the value here as the base address
 \ ** of the UART to use.
))

: callit	\ xt --
\ *G Load the reprogramming code and execute the xt.
  [defined] dfi [if] dfi [then]		\ disable FIQ
  di					\ disable IRQ [and FIQ]
  execute
  [defined] reboot [if]  reboot  [else]  0 execute  [then]
;

: ReFlash	\ -- ; no exit
\ *G Copies the reprogramming code to the run-time address
\ ** and executes it.
  reprogrun @ callit
;

: CopyFlash	\ src dest len --
\ *G Copy the flash. The parameters are as for *\fo{CMOVE}.
\ ** The process is repeated until there are no errors, and
\ ** the system is then rebooted.
  reprogrun cell + @ callit
;


\ ======
\ *> ###
\ ======

