\ Reprogram STR91x over serial line.

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20061101 SFP001 First cut
))


\ =============
\ *! reprogstr91xser
\ *T Reprogramming the STR91x serially
\ =============
\ *S Introduction
\ *P *\i{ARM\Hardware\STR91x\Regrog\ReprogSTR91xser.ctl} is the
\ ** control file for a serial loader which reprograms the primary
\ ** on-chip Flash memory. The batch file *\i{bs1.bat} builds the
\ ** image and places it in *\i{ARM\Hardware\STR91x}.

\ ** Because the primary Flash cannot be accessed during
\ ** programming, the Flash programming code is in the secondary
\ ** Flash block. All interrupts are disabled during reprogramming,
\ ** and so polled serial drivers are used. The only exit from the
\ ** code is a reset of the CPU to execute the newly Flashed
\ ** program. The contents of internal RAM and primary Flash are
\ ** destroyed. Thus any data that must be preserved should be
\ ** saved outside the primary Flash block. The secondary Flash
\ ** block can be programmed using the code in
\ ** *\i{ARM\Hardware\STR91x\FlashProg.fth}.

\ *P Access to the reprogramming software is defined in
\ ** *\i{RARM\Hardware\STR91x\Reprog\Reflash.fth} for the main
\ ** system code and in this control file. The two sets of
\ ** definitions *\b{must} match.


only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

"" ..\..\.."		setmacro CpuDir
"" ..\..\..\..\common"	setmacro CommonDir
"" ."			setmacro AppDir
"" .."			setmacro HwDir


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ArmArch5                      \ ARM9 processor type
  32bit-mode			\ running in 32 bit mode
  no-heads			\ do not generate heads
  no inlining			\ saves some space
  -ram-table			\ saves some space
  +LeafCalls			\ saves some space
  Hex-I32			\ also produce Intel Hex-32 obj format

0         equ false
false not equ true


\ ****************
\ *S Configuration
\ ****************

#96000000 equ system-speed	\ -- hz
\ *G CPU clock speed in HZ. This must match the speed set up by
\ ** the calling application.

#96 equ IRAMkb		\ -- n
\ *G Number of kilobytes of internal RAM
#512 equ BootFlashKb	\ -- u
\ *G Number of kilobytes of boot area internal Flash.
#32 equ NonBootFlashkb
\ *G Number of kilobytes of non-boot area internal Flash.

$00000000 equ FlashBase		\ -- addr
\ *G Base address of the boot (primary) Flash bank. This is the
\ ** address at which the bank is mapped.
$00400000 equ NBFlashBase	\ -- addr
\ *G Base address of the non-boot (secondary) bank, in which this
\ ** code will run. This is the address at which the bank is
\ ** mapped. The mapping here *\b{must} match that of the
\ ** calling application as it is the caller which performs
\ ** the CPU initialisation.


\ *************
\ *N Memory map
\ *************
\ *P Take care to ensure these memory regions do not overlap the stacks
\ ** of the calling program or the interrupt vector and FIQ area.
\ ** The *\fo{Xbuff} section must be a multiple of 256 bytes because
\ ** the Flash is programmed in 256 byte units.


$00400000 $00400FFF cdata section ReprogSTR91xSer
$40000000 $400007FF idata section PROGd	\ 2k IDATA
$40000800 $40000FFF udata section PROGu	\ 2k UDATA
$40001000 $400013FF udata section Xbuff	\ 1024 byte Xmodem buffer
  sec-base equ Xbase			\ base address of buffer
  sec-len equ Xlen			\ length of buffer
  Xbase Xlen + equ Xtop			\ top of buffer

interpreter
: prog		ReprogSTR91xSer  ;
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ***********************************
\ *N Configure stacks and kernel buffers
\ ***********************************
\ *P The stacks have already been set up by the calling program.


\ ************
\ *S File List
\ ************

include %CpuDir%\ArmDef			\ ARM CPU equates
include %CpuDir%\sfrSTR91x		\ Special function registers

l: CLD1		\ -- addr
\ *G Filled in later with the xt of *\fo{ReprogFlash}
  0 ,
l: CLD_CopyFlash	\ -- addr
\ *G Filled in later with the xt of *\fo{CopyFlash}
  0 ,
l: CLD_UART	\ -- addr
\ *G Filled in later with 0 or UART base address to use.
\ ** If set to 0, the default is UART0. This allows an
\ ** application to select which UART to use.
  0 ,
l: CLD_usdel	\ -- addr
\ *G Filled in with the number of number of iterations required
\ ** by *\fo{UDELAY} for a one microsecond delay. The default
\ ** assumes 5 clocks per iteration and a 50MHz clock
   #10 ,

include %AppDir%\primitives		\ Forth primitives from CODEARM.FTH
include %AppDir%\MinSerSTR91xp		\ polled serial line driver
include %AppDir%\delays			\ software delays
include %CpuDir%\Drivers\RebootSTR91x	\ software reboot

synonym ser-emit emit
\ *G A synonym required by the Xmodem code.
synonym ser-key? key?
\ *G A synonym required by the Xmodem code.
synonym ser-key  key
\ *G A synonym required by the Xmodem code.

include %AppDir%\MinXmodemRx		\ Xmodem receiver
include %AppDir%\ReprogApp		\ Application

make-turnkey ReprogFlash		\ start up action
\ *G Define the action of the reprogramming code.
' CopyFlash CLD_CopyFlash !
\ *G Define the action of the Flash copy code.


\ *******
\ The end
\ *******

decimal

FINIS


\ ******
\ *> ###
\ ******
