\ STR912FW44.ctl - STR912FW44 standalone Forth control file

((
Copyright (c) 2006, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20070528 MPE001 First release.
))


\ =============
\ *! str912fw44
\ =============
\ *T Standalone Forth for STR912FW44
\ *P The code in *\i{ARM\Hardware\STR91x\STR912FW44.ctl is a
\ ** control file for STR91xxxx CPU systems. It was tested on
\ ** a Hitex STR912 evaluation board with an STR912FW44 Rev. D
\ ** part.

\ *P Unlike the LPC2xxx family, The STR91x CPUs have two Flash
\ ** regions. Code in one bank can be used to program the other.
\ ** We have chosen to place the Flash reprogramming code in the
\ ** bottom 8kb of the 32kb secondary Flash region, reserving
\ ** the top 8kb for application configuration. As a result of
\ ** this, initial Flash programming requires *\b{two} images
\ ** to be programmed, one to the primary Flash and one to the
\ ** secondary Flash. If you are using the MPE JTAG Widget,
\ ** load the file *\i{CPUs\jwSTR91x.fth} for the STR91x
\ ** configuration.

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives
"" C:\buildkit.dev\software\AddOns\PowerNet\Dev"
		setmacro IpStack	\ where PowerNet lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ArmArch5                      \ ARM9 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  #8 LoopAlignment		\ faster but bigger
  Hex-I32			\ also produce Intel Hex-32 obj format

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N STR91x definitions
\ =====================

#96 equ IRAMkb		\ -- n
\ *G Number of kilobytes of internal RAM
#512 equ BootFlashKb	\ -- u
\ *G Number of kilobytes of boot area internal Flash.
#32 equ NonBootFlashkb
\ *G Number of kilobytes of non-boot area internal Flash.

$00000000 equ FlashBase		\ -- addr
\ *G Base address of the boot Flash bank. This is the address at
\ ** which the bank is mapped. It is not necessarily part of a
\ ** *\fo{SECTION}.
$00400000 equ NBFlashBase	\ -- addr
\ *G Base address of the non-boot bank. This is the address at
\ ** which the bank is mapped. It is not necessarily part of a
\ ** *\fo{SECTION}.


\ *P The STR91x has several input clocks, of which we only use
\ ** the first two.
\ *(
\ *B FRTC - Slow clock, usually a 32.768kHz crystal,
\ *B FOSC - Master oscillator, 25MHz crystal on most dev boards,
\ **        and must be 25MHz for the Ethernet PHY.
\ *B FUSB - optional 48 MHz input for USB.
\ *B FTIMx - optional clock inputs for Timers 0/1 and 2/3.
\ *)

\ *P From the first two clock sources the working clocks are
\ ** derived.
\ *(
\ *B FPLL - PLL output derived from FOSC.
\ *B FMSTR - master clock selected between FOSC, FPLL, and FRTC.
\ **         We only use FPLL.
\ *B USBCK - 48 MHz USB clock derived from FMSTR divided by 1 or 2.
\ **         Thus if USB is to be used, FMSTR must be 48 or 96 MHz.
\ *B BRCLK - UART baud rate clock. FMSTR or FMSTR/2.
\ *B RCLK - Divided from FMSTR, and then divided to form the AHB bus (HCLK),
\ **       APB bus (PCLK), and Flash memory bus (FMCLK) clocks.
\ *B HCLK - Derived from RCLK. Used by AHB peripherals.
\ *B PCLK - Derived from RCLK. Used by APB peripherals.
\ *B CPUCLK - FMSTR or RCLK, we only use FMSTR.
\ *)

#25000000 equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
xtal-speed equ /MOSC		\ -- hz
\ *G Synonym used by PLL calculator.

#96000000 equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values. STM provide the CAPS program
\ ** for this from http://www.st.com. Throughout this code
\ ** it is assumed that *\fo{system-speed}=FMSTR.

1 equ /RCLKdiv	\ -- n
\ *G Division ratio between FMSTR and RCLK. Default is 1. This must
\ ** match *\fo{RCLKDIVmask} below.
0 2 lshift equ RCLKDIVmask	\ -- mask
\ *G Bit mask for the 3 bit RCLK divider field in the SCU_CLKCNTR
\ ** register.
\ *E   field   division by
\ **    000      1
\ **    001      2
\ **    010      4
\ **    011      8
\ **    100      16
\ **    101      1024

1 equ /HCLKdiv	\ -- n
\ *G Division ratio between RCLK and the AHB clock. Default is 1. This must
\ ** match *\fo{AHBDIVmask} below.
0 5 lshift equ AHBDIVmask	\ -- mask
\ *G Bit mask for the 2 bit APBdiv divider field in the SCU_CLKCNTR
\ ** register.
\ *E   field   division by
\ **    00       1
\ **    01       2
\ **    10       4
\ **    11       8

1 equ /PCLKdiv	\ -- n
\ *G Division ratio between RCLK and the APB clock. Default is 1. This must
\ ** match *\fo{APBDIVmask} below..
0 7 lshift equ APBDIVmask	\ -- mask
\ *G Bit mask for the 2 bit APBdiv divider field in the SCU_CLKCNTR
\ ** register.
\ *E   field   division by
\ **    00       1
\ **    01       2
\ **    10       4
\ **    11       reserved

2 equ /BRdiv	\ -- u
\ *G Divider for baud rate, corresponding to bit 9 of SCU_CLKCNTR.
\ ** Default is 2.
0 9 lshift equ BRSELmask	\ -- mask
\ *G Bit mask for the 1 bit BRSEL divider field in the SCU_CLKCNTR
\ ** register.
\ *E   field   division by
\ **     0       2
\ **     1       1

system-speed /RCLKdiv /  /HCLKdiv / equ HCLKspeed	\ -- hz
\ *G HCLK speed in Hz.
system-speed /RCLKdiv /  /PCLKdiv / equ PCLKspeed	\ -- hz
\ *G PCLK speed in Hz.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $0400:0000..$0400:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

  $00000000 equ link-address	\ Address at which code will start\be linked at
  $00000000 $0007FFFF cdata section STR9W44	\ 512kb code section in boot bank Flash
\    sec-base equ SWBootBase			\ base address of boot Flash
  $00400000 $00407FFF cdata section NBBFlash	\ 32 kb non-boot Flash
\    sec-base equ SWNBBbase			\ for configuration data
    sec-top 1+ 8 kb - equ CfgFlash		\ 8 kb of configuration data at top

\ *P Note that the internal RAM can be accessed in three regions.
\ *(
\ *B TCM (tightly coupled RAM) at $0400:0000 - fastest but unsafe with DMA.
\ *B AHB buffered at $4000:0000 - uses write buffering, coherent except with DMA.
\ *B AHB unbuffered at $5000:0000 - coherent and safe but slowest.
\ *)
\ RAM in AHB space - slower
  $40000000 $40000FFF idata section PROGd	\ 4k IDATA
  $40001000 $40013FFF udata section PROGu	\ 80k-4k UDATA, N.B. Buffered
\ DMA unbuffered area
  $50014000 $50017FFF udata section AHBramU	\ 16k UDATA for DMA buffers
(( \ faster but unsafe with DMA and Ethernet
\ RAM in TCM - faster
  $04000000 $04000FFF idata section PROGd	\ 4k IDATA
  $04001000 $0400FFFF udata section PROGu	\ 64k-4k UDATA
))


interpreter
: prog STR9W44  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
1 equ #FIQs			\ number of FIQ stacks,
				\ shared by all FIQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused

\ ================
\ *N Define stacks
\ ================

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ==========================
\ *N Serial and ticker rates
\ ===========================

0 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

1 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

1 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 0, 1 and 2 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G TIMEBASE tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $8000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 1 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %CpuDir%\sfrSTR91x  		\ Special function registers.
  include %HwDir%\initSTR91x		\ startup code
  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  include %CpuDir%\Drivers\rebootSTR91x	\ reboot using software reset
  3 equ ARM-VIC?			\ select PL190 Vectored Interrupts
  2 equ #VICs				\ with a second VIC
  include %CpuDir%\IntSTR91x		\ interrupt handlers for dual PL190.
  include %CommonDir%\cqueues		\ character queues
  include %CpuDir%\Drivers\serSTR91xqi	\ RX queued interrupt driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%\delays		\ time delays
  include %CpuDir%\Drivers\TickSTR91x	\ STR91x ticker
  ' start-timers AtCold
[else]
  include %CommonDir%\delays
  include %CpuDir%\Drivers\TickSTR91x
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ *S End of kernel
\ *************

buildfile ekSTR91x.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." *****************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for STR91x"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

((
code GetCR0	\ -- cr0 ; ID register
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr0, cr0, 0
  next,
end-code

code GetCR1	\ -- cr1
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr1, cr0, 0
  next,
end-code

code SetCR1	\ cr1 --
  mcr     #15 0 tos, cr1, cr0, 0
  ldmfd   psp ! { tos }
  next,
end-code
))

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%\XmodemTxRx	\ Xmodem driver
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support

FlashBase constant PriFlash	\ -- addr
\ *G Mapped base address of primary Flash (512kb).
NBFlashBase constant SecFlash
\ *G Mapped base address of secondary Flash (32kb).
  include %HwDir%\FlashProg		\ STR91x Flash code
  include %HwDir%\ReProg\ReFlash	\ STR91x ReFlash utility
\  variable lseconds
\  include %ExampleDir%\UnixTime

PowerNet? [if]
  1 equ CPU=ARM	\ to get ARM specific code
  0 equ FilePages?			\ no file system of HTTP page storage
  include %AppDir%\PNconfig		\ PowerNet configuration
  include %IpStack%\NetCode		\ network order stuff
  include %AppDir%\WebConfig		\ configuration tools
  include %CpuDir%\drivers\EtherSTR91x	\ Ethernet driver
  include %IpStack%\PowerNet.bld	\ PowerNet build
  include %IpStack%\Examples\WebPost	\ application update

[else]
((
1 equ CPU=ARM	\ to get ARM specific code
1 equ EthDiags?
1 equ Sniff?
1 equ Pingit?
  include %ExampleDir%\NetCode		\ network order stuff
  include %CpuDir%\drivers\EtherSTRx91x	\ Ethernet driver
))
[then]


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\benchmarks\xbench32	\ performance tests

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======

