\ Init75401lolo.FTH - LPD75401 with LoLo startup code

(( 
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ ***************************************
\ Generic start up table and entry points
\ ***************************************

asmcode
L: Vector_Start	\ 'Hard wired' ARM Processor vectors
  ldr 	pc, ResetV              \ 00 - Reset
  ldr 	pc, UndefV              \ 04 - Undefined Instructions
  ldr 	pc, SWIV                \ 08 - SWI instructions
  ldr 	pc, PAbortV             \ 0C - Instruction fetch Aborts
  ldr 	pc, DAbortV             \ 10 - Data access aborts
  dd    0                       \ 14 - checksum
  ldr 	pc, IRQV		\ 18 - unvectored interrupts
\  ldr 	pc, [ pc, # $-FF0 ]	\ 18 - IRQ interrupts from VicVectAddr @ $FFFFF030
  ldr 	pc, FIQV		\ 1C - FIQ interrupts

L: Soft_Vectors	\ Handler routines can be 'socketed' in here
  L: ResetV       dd 0          \ HandlerReset
  L: UndefV       dd 0          \ HandlerUndef
  L: SWIV         dd 0          \ HandlerSWI
  L: PAbortV      dd 0          \ HandlerPAbort
  L: DAbortV      dd 0          \ HandlerDAbort
  L: UnusedV      dd 0          \ HandlerUnused
  L: IRQV         dd 0          \ HandlerIRQ
  L: FIQV         dd 0          \ HandlerFIQ
  L: Vector_End
end-code

Vector_End Vector_Start - equ Vector_Len


\ ************
\ Startup code
\ ************

\ Required Forth pointers
L: CLD1		                        \ cold start word
  0 ,				        \ compiler will fill up with address of word to execute at cold start
L: DATASTACK				\ top of data stack
  init-s0 ,
L: RETURNSTACK				\ top of return stack
  init-r0 ,
L: USERPOINTER				\ user area pointer
  init-u0 ,

l: ^UART2
  _UART2 ,

((
proc SendUart	\ send character in R4
  ldr   r1, ^UART2
  mov   r2, # $20			\ select bank 1
  str   r2, [ r1, # U2GIR ]
\  ldr   r0, [ r1, # $018 ]		\ set bit 1 in TCM, enable TX
\  orr   r0, r0, # $02
\  str   r0, [ r1, # $018 ]
l: waitemit1
  ldr   r3, [ r1, # $010 ]		\ if full, bit 2 in FLR is set
  and .s r3, r3, # $004
  b .ne waitemit1
  str   r4, [ r1, # U2THR ]
\ l: waitemit2
\  ldr   r3, [ r1, # $010 ]		\ if not empty, bit 2:0 in FLR is set
\  and .s r3, r3, # $007
\  b .ne waitemit2
\  ldr   r0, [ r1, # $018 ]		\ clear bit 1 in TCM, disable TX
\  bic   r0, r0, # $02
\  str   r0, [ r1, # $018 ]
  mov	pc, link
end-code
))

proc SendUart	\ send character in R4
  ldr   r1, ^UART2
  mov   r2, # $00			\ select bank 0
  str   r2, [ r1, # U2GIR ]
l: waitemit1
  ldr   r0, [ r1, # U2LSR ]		\ if empty, bit 5 in LSR is set
  and .s r0, r0, # bit5
  b .eq waitemit1
  str   r4, [ r1, # U2THR ]
  mov	pc, link
end-code

((
  0 _UART2 U2GIR + !
  _UART2
  begin
    dup U2LSR + @ %00100000 and
  until
  U2THR + !
))

Proc InitReset	\ power up entry point through vector
\ Disable interrupts
  mrs   r0, cpsr
  orr   r0, r0, # $C0			\ set I and F bits
  msr   cpsr _c r0			\ modify control bits

l: nextchar
  mov	r4, # char M
  bl	SendUart
  mov	r4, # char P
  bl	SendUart
  mov	r4, # char E
  bl	SendUart
  mov	r4, # $0D
  bl	SendUart
  mov	r4, # $0A
  bl	SendUart
\  b     nextchar


\ Initialise the stacks for INTARM3.FTH
\  bl		InitStacks		\ see ARM\INITSTACKS.FTH


\ *****************************
\ Optional code copy
\ *****************************

\ If required, copy code into RAM for execution
((
  mov		r0, # Flash_start	\ source
  mov		r1, # RAM_start		\ dest
  mov		r2, # Flash_len		\ len, must be multiple of 4
l: Flash_loop
  ldr		r3, [ r0 ], # 4
  str		r3, [ r1 ], # 4
  sub .s	r2, r2, # 4
  b .ne		flash_loop
))

((
\ Copy vector table to low RAM if it exists
  mvl r0, # Vector_Start                \ Source address.
  mov r1, # $0                          \ Destination address.
  mov r2, # Vector_End Vector_Start -   \ Length to copy.
L: CopyJumpTable
  ldr r3, [ r0 ], # 4
  str r3, [ r1 ], # 4
  sub .s r2, r2, # 4
  b .ne CopyJumpTable

))

 
\ *********************
\ Start of Forth system
\ *********************

L: ECLD
\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code

InitReset ( prog sec-base - ) ResetV !	\ INITRESET is in LOW memory starting at 0.


\ ******************************************
\ Reset values for user and system variables
\ ******************************************

[undefined] sp-guard [if]
  0 equ sp-guard
[then]

\ initial values of user variables for standalone systems only
[undefined] umbilical? [if]
L: USER-RESET
  init-s0   tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,                 		\ r0
  0 ,  0 ,                              \ #tib, span
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ *********************
\ Low level serial code
\ *********************

  
\ ======
\ *> ###
\ ======

decimal

