\ LogicPD 75401 Lolo Forth control file

((
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ =========
\ *! lpd75401lolo
\ =========
\ *T Forth for LogicPD 75401 and Lolo
\ *P The control file Lpd75401lolo.ctl generates the standalone
\ ** Forth for the LogicPD 75401-11 board.

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
"" ..\..\..\Common"
		setmacro CommonDir	\ where the common code lives
"" ..\..\..\ARM"
		setmacro CpuDir		\ where the CPU specific code lives
"" ."		setmacro HWDir		\ where board specific code lives
"" ..\..\..\Examples"
		setmacro ExampleDir	\ Examples
"" ."    	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  +LeafCalls			\ improves performance, reduces size.

 0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ --------------------
\ *H Kernel components
\ --------------------

 1 equ ColdChain?	\ -- n
\ *G nz to use cold chain mechanism
 0 equ tasking?
\ *G true if multitasker needed. Event handler and message
\ ** passing code is not compiled, but semaphores are.
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\   true to include semaphores
 0 equ timebase?
\ *G true for TIMEBASE code
 0 equ softfp?
 \ *G true for software floating point
 0 equ FullCase?
\ *G true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?
 \ *G true if target local variable sources needed
 0 equ romforth?
\ *G true for ROMForth handler
 0 equ blocks?
\ *G true if BLOCK needed
 $0000 equ sizeofheap
 \ *G 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ ------------------------------------------
\ *H Crystal, clock, serial and ticker rates
\ ------------------------------------------

#70000000 equ xtal-speed
\ *G Crystal clock rate in HZ.
#70000000 equ system-speed
\ *G System operating speed in HZ.

0 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0.
0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1.
1 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2.

#115200 equ console0-speed
\ *G Default baud rate for UART0.
#115200 equ console1-speed
\ *G Default baud rate for UART1.
#115200 equ console2-speed
\ *G Default baud rate for UART2.

2 equ console-port
\ *G Designate serial port for terminal (0/1/2...).

#10 equ tick-ms
\ *G TIMEBASE tick in ms.


\ ---------------------------------------------
\ *H Define stack and user area sizes for TASKs
\ ---------------------------------------------

$0100 equ UP-SIZE
\ *G size of each task's user area.
$0100 equ SP-SIZE
\ *G size of each task's data stack.
$0200 equ RP-SIZE
\ *G size of each task's return stack.
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN
\ *G Terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs
\ *G Number of IRQ stacks, shared by all IRQs (1 min)
1 equ #FIQs
\ *G number of FIQ stacks, shared by all FIQs (1 min)
0 equ #SWIs
\ *G number of SWI nestings permitted. 0 is valid if SWIs are unused


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ *************
\ *S Memory map
\ *************

  $480C0000 equ link-address	\ Address at which code will start\be linked at
  $480C0000 $480DFFFF cdata section Lpd75401
\ *G Primary code section. Set LINK-ADDRESS above to be this start address.
  $480E0000 $480E7FFF idata section PROGd
\ *G Primary IDATA section
  $480E8000 $480EFFFF udata section PROGu
\ *G Primary UDATA section. This area contains unitialised data.

interpreter
: prog	Lpd75401  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%\StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\ARMdef		\ ARM generic equates
  include %HwDir%\sfr75401lolo 		\ Special function registers.
  include %HwDir%\init75401lolo		\ startup code
\  include %CpuDir%\initstacks		\ default stack initialisation
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62          \ high level kernel definitions
  0 equ ARM-VIC?			\ select PL190 Vectored Interrupts
\  include %CpuDir%\IntArm3		\ interrupt handlers
\  include %CommonDir%\cqueues		\ character queues
\  include %CpuDir%\Drivers\ser75401i	\ interrupt RX serial driver
  include %HwDir%\ser75401lolop		\ polled serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code, MUST be before TICKxxx
  include %CpuDir%\drivers\TickLPC210x	\ timer tick
  ' start-timers AtCold
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

buildfile Lpd75401Lolo.no
internal
create version$
  build$,
external

: .CPU          \ -- ; display CPU type
  cr ." ****************************************"
  cr ." MPE ROM PowerForth for LPD75401 and Lolo "  version$ $.
  cr ." ****************************************"
;

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ Application code
\ *******************

\  include %CommonDir%\DebugTools
\  include %HwDir%\IAP210x		\ IAP code
\  include %AppDir%\Reprog\Reflash	\ Reflash system
\  include %HwDir%\FlashProg		\ higher level Flash routines

\ Redefinitions already done for ROMFORTH
\  Synonym Ser-Key? key?
\  Synonym Ser-Key  key
\  Synonym Ser-Emit emit
\  1 equ XmodemRx?
\  1 equ XmodemTx?
\  include %CommonDir%\XmodemTxRx

  include %HwDir%\Smc91C111


\ **********************
\ Select example code
\ **********************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build				\ increment build number
FINIS


\ ======
\ *> ###
\ ======

