\ SA1110 startup code for StrongBox rev 2

((
Copyright (c) 2002, 2003
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE


Position Independent Code
-------------------------
Although this code is compiled at the the load address
of the Forth system in DRAM, this code is executed out
of Flash at base address 0000:0000. Consequently the code
in this file must be position independent. Note also that
the reset vector location RESETV is loaded with the entry
address in low memory.
  Reset_SA1110 CODE-START - ResetV !

Power Up Oscillator
-------------------
For boards with the 1Hz boot oscillator the nBooted line
on GPIO2 *must* be asserted after reset to disable the 3v3
power supply pulsing.
))


\ ********************
\ Configure the system
\ ********************

\ -----------
\ GPIO set up
\ -----------

BIT24 equ Led1			\ Green LED on GPIO24
BIT22 equ Led2			\ Red LED on GPIO22

Led1 LED2 or			\ initial output bits
  equ init_gpdr			\ Direction for GPIO 


\ ---------------------------------------------
\ DRAM Timing for ISSI IS41LV6105 (1Mx16, 50ns) for 206Mhz (ca. 10ns/memory-cycle)
\ ---------------------------------------------

\ MDCNFG - banks 2/3 are unused
\ N.B.
\ 1) Set all other MC registers first. EAPD=0 and KAPD=0 in MDREFR
\ 2) Set MDCNFG with DEx=0
\ 3) Perform 8 reads from unenabled RAM to start refresh
\ 4) Set MDCBFG with DEx=1 as appropriate
\ 5) Rewrite MDREFR if needed

decimal

$3        equ DRAM_DE0		\ bank0 and bank1 enable 
$0 2  <<n equ DRAM_DTIM0        \ 0=FPM/EDO, 1=SDRAM
$0 3  <<n equ DRAM_DWID0	\ 32 bit bus width
$1 4  <<n equ DRAM_DRAC0        \ 10x10 drams on banks 0/1
$0 7  <<n equ DRAM_CDB20        \ cas clock is 0=CPU-CLK 1=MEM-CLK
$8 8  <<n equ DRAM_TRP0         \ mem cycles for ras precharge(Trp) - 30ns
$0 12 <<n equ DRAM_TDL0         \ cycles after cas to latch data
$3 14 <<n equ DRAM_TWR0		\ SDRAM write recovery

DRAM_DTIM0
DRAM_DWID0 or
DRAM_DRAC0 or
DRAM_CDB20 or
DRAM_TRP0 or
DRAM_TDL0 or
DRAM_TWR0 or
  equ init_MDCNFG_first		\ DE fields = 00

DRAM_DE0 init_MDCNFG_first or
  equ init_MDCNFG_second	\ DE fields =xx
 
\ MDREFR - refresh control reg, no SDRAM, no SMROM
$5        equ DRAM_TRASR	\ cycles - 1 for refresh   - 50ns
50 4 <<n  equ DRAM_DRI		\ cycles between DRAM refreshes
				\ DRI = (reftime-burst/rows)*memclock/32
				\     = (16*10^-3/2^10)*10^8/32
DRAM_TRASR DRAM_DRI or
  equ init_MDREFR

((
\ MDCAS0..2 (slow)
$3F03F03F equ init_MDCAS0	\ for an 8 beat burst 
$F03F03F0 equ init_MDCAS1                       
$F03F03F0 equ init_MDCAS2                       
))

\ MDCAS0..2 (normal)
$F0F0F00F equ init_MDCAS0	\ for an 8 beat burst 
$F0F0F0F0 equ init_MDCAS1                       
$FFFFFFF0 equ init_MDCAS2                       

((
\ MDCAS0..2 (fast)
$CCCCCCCF equ init_MDCAS0	\ for an 8 beat burst 
$FFFFFFFC equ init_MDCAS1                       
$FFFFFFFF equ init_MDCAS2                       
))


\ -------------------------------
\ Static memory/peripheral timing
\ -------------------------------

decimal

\ MSC0, CS0=32 bit flash, CS1=16 bit SRAM
0         equ RT0		\ flash
0 2 <<n   equ RBW0		\ 32 bit
9 3 <<n   equ RDF0		\ cycles address->data valid
9 8 <<n   equ RDN0		\ cycles rd/wr width
2 13 <<n  equ RRR0		\ cycles min wr high time

1 16 <<n  equ RT1		\ SRAM
1 18 <<n  equ RBW1		\ 16 bit
9 19 <<n  equ RDF1		\ cycles address->data valid
9 24 <<n  equ RDN1		\ cycles rd/wr width
2 29 <<n  equ RRR1		\ 2* cycles min wr high time

RT0     RBW0 or  RDF0 or  RDN0 or  RRR0 or
RT1 or  RBW1 or  RDF1 or  RDN1 or  RRR1 or
  equ init_MSC0

\ MSC1, CS2=none CS3=16 bit Ethernet
0         equ RT2		\ flash
1 2 <<n   equ RBW2		\ 32 bit
16 3 <<n  equ RDF2		\ 3 cycle address set up
16 8 <<n  equ RDN2		\ 9 cycle rd/wr width
2 13 <<n  equ RRR2		\ 2 cycle min wr high time

1 16 <<n  equ RT3		\ Ethernet
1 18 <<n  equ RBW3		\ 16 bit
3 19 <<n  equ RDF3		\ cycle address set up (30ns)
10 24 <<n equ RDN3		\ cycle rd/wr width (100ns)
2 29 <<n  equ RRR3		\ cycle min wr high time

RT2     RBW2 or  RDF2 or  RDN2 or  RRR2 or
RT3 or  RBW3 or  RDF3 or  RDN3 or  RRR3 or
  equ init_MSC1

\ MSC2, CS4=none CS5=none
$FFF8FFF8 equ init_MSC2


\ --------------------------------------------
\ PC card set up for HAL bus and CAN interface
\ --------------------------------------------
((
Note:
  wait states are inserted by the CPLD interface,
  All accesses must be byte or half-word.

For FASTx=0
  address set up  = 3*(BSxx+1)+1 memory clocks (2* on second access)
  strobe width    = 3*(BSxx+1)+1 memory clocks
  address hold    = 1*(BSxx+1)+1 memory clocks
  2nd addr set up = 2*(BSxx+1)+1 memory clocks
                    16 bit access to 8 bit I/O
For FASTx=1
  address set up  = 1*(BSxx+1)+1 memory clocks
  strobe width    = 3*(BSxx+1)+1 memory clocks
  address hold    = 1*(BSxx+1)+1 memory clocks
  2nd addr set up = 1*(BSxx+1)+1 memory clocks
                    16 bit access to 8 bit I/O

For 206 MHz, say MCLK period is 10ns. For IOW/IOR of 100ns,
we need 10 MCLKs, 10/3=3.3, take 4 (next whole number).
Now BS=4-1=3.

The CAN bus interface is in Slot 0 I/O space at
  $2000:0000 to $2000:FFFF	(repeated every $0100 bytes)
The HAL bus memory interface is in Slot 0 I/O space at
  $2002:0000 to $2002:FFFF	(A15:0 on the HAL bus)
The HAL bus I/O interface (BUSIOB active) is in Slot 0 I/O space at
  $2003:0000 to $2003:FFFF	(A15:0 on the HAL bus)
))

\ slot 0
3         equ BSIO0	\ I/O space
3  #5 <<n equ BSA0	\ attribute space
3 #10 <<n equ BSM0	\ memory space
1 #15 <<n equ FAST0	\ set to reduce address set up to BSxx+1 MCLKs
\ slot 1
3 #16 <<n equ BSIO1	\ I/O space
3 #21 <<n equ BSA1	\ attribute space
3 #26 <<n equ BSM1	\ memory space
0 #31 <<n equ FAST1	\ set to reduce address set up

0
BSIO0 or  BSA0 or  BSM0 or  FAST0 or
BSIO1 or  BSA1 or  BSM1 or  FAST1 or
  equ init_MECR


\ *************************************
\ End of memory set up data definitions
\ *************************************


aif? 0= [if]	\ if standalone

\ ************
\ LED handling
\ ************

PROC SwitchLED	\ R0 = LED mask
  mvl r1, # _GPIO
  ldr r2, [ r1, # gplr ]		\ read status
  and r2, r2, r0			\ LED & Status
  cmp r2, r0				\ if masked result = mask
  str .eq r0, [ r1, # gpcr ]		\   clear GPIO
  str .ne r0, [ r1, # gpsr ]		\ else set GPIO
  next,
end-code

PROC SetLED	\ R0 = LED mask
  mvl r1, # _GPIO
  str r0, [ r1, # gpcr ]		\ clear GPIO
  next,
end-code

PROC ClearLED	\ R0 = LED mask
  mvl r1, # _GPIO
  str r0, [ r1, # gpsr ]		\ set GPIO
  next,
end-code

PROC wasteTime
  mvl	  r0, # #400000			\ delay count for LED
l: loop1
  sub .s  r0, r0, # 1
  b .ne   loop1
  next,
end-code


\ *******************
\ Debug UART handling
\ *******************

((
interpreter
: gen-brd	\ baud -- utcr2 utcr1
  #16 * xtal-speed swap / 1-
  dup $0FF and				\ UTCR2
  swap 8 rshift $00F and		\ UTCR1
;
target

l: uart3-setup	\ -- addr
\ *G Initialisation table for UART3 - usually the debug UART
  _uart3 utcr3 + ,	0 ,		\ disable it
  _uart3 utcr0 + ,      UART3_UTCR0 ,	\ 8 bit, no parity, 1 stop 
  #38400 gen-brd			\ define baud rate
  _uart3 utcr1 + ,	,
  _uart3 utcr2 + ,	,
  _uart3 utsr0 + ,	$0FF ,		\ reset all bits
\  _uart3 utsr1 + ,	$0FF ,		\ reset all bits
  _uart3 utdr + ,	0 ,		\ write dummy character!
  _uart3 utcr3 + ,	UTCR3_TXE UTCR3_RXE or ,  \ enable TX and RX straight to the UART pins
  0 ,					\ end of table

\ Initialisation and tables
proc init-io-ll	\ R0 contains address of initialisation table
l: init-io-ll-1
  ldr	  r1, [ r0 ], # 4	\ get address
  cmp	  r1, # 0		\ get out if 0
  b .eq   init-io-ll-x
  ldr     r2, [ r0 ], # 4	\ get data
  str	  r2, [ r1, # 0 ]	\ store data
  b       init-io-ll-1
l: init-io-ll-x
  next,
end-code

proc serkey-ll	\ R0=base address -> R1=char
l: serkey-ll-1
  ldr	  r1, [ r0, # utsr1 ]
  and .s  r1, r1, # UTSR1_RNE
  b .eq   serkey-ll-1
  ldr     r1, [ r0, # utdr ]
  next,
end-code

proc seremit-ll	\ R0=base, R1=char
l: seremit-ll-1
  ldr	  r2, [ r0, # utsr1 ]
  and .s  r2, r2, # UTSR1_TNF
  b .eq   seremit-ll-1
  str     r1, [ r0, # utdr ]
  next,
end-code
))


\ **********
\ Entry code
\ **********

PROC Reset_SA1110
\ System timer
  mvl r0, # _OS_T			\ Set OS-Timer counter to zero
  mov r1, # 0
  str r1, [ r0, # oscr ]

\ GPIOs
  mvl r0, # _GPIO			\ Set GPIO Direction Register
  mvl r1, # init_gpdr
  str r1, [ r0, # gpdr ]		\ direction register
  str r1, [ r0, # gpcr ]		\ clear output bits to 0
  mvn r1, # 0				\ clear GPIO Edge Detect Status Register
  str r1, [ r0, # gedr ]

  mov r1, # 0				\ clear edge detection reg. enables
  str r1, [ r0, # grer ]		\ GPIO Rising Edge Detect Register (disable)
  str r1, [ r0, # gfer ]		\ GPIO Falling Edge Detect Register (disable)

\ PLL clock
  mvl r0, # _PM
  mov r1, # cc_206MHz			\ set to 206 MHz
  str r1, [ r0, # ppcr ]		\ PPCR *must* be set BEFORE enabling!
  mcr #15 %000 r0 cr15 cr1 %010		\ Enable clock switching
  mov r0, r0				\ settle time
  mov r0, r0				\ settle time
  mov r0, r0				\ settle time

\ DRAM setup
  mov r1, # $4000			\ waste 100-200us to let DRAM clocks settle
l: clock-settle-loop
  sub .s r1, r1, # 1
  b .ne clock-settle-loop

  mvl r0, # _MC
  mvl r1, # init_MDREFR
  str r1, [ r0, # mdrefr ]		\ MDREFR
  mvl r1, # init_mdcnfg_first
  str r1, [ r0, # mdcnfg ]		\ MDCNFG DRAM Configuration Register
  mvl r1, # init_MDCAS0
  str r1, [ r0, # mdcas0 ]		\ MDCAS0 DRAM CAS Waveform Shift Register 0
  mvl r1, # init_MDCAS1
  str r1, [ r0, # mdcas1 ]		\ MDCAS1 DRAM CAS Waveform Shift Register 1
  mvl r1, # init_MDCAS2
  str r1, [ r0, # mdcas2 ]		\ MDCAS2 DRAM CAS Waveform Shift Register 2
  mvl r1, # init_MDREFR
  str r1, [ r0, # mdrefr ]		\ MDREFR
  mvl r2, # DRAM0-START			\ 8 refresh cycles
  ldr r3, [ r2 ], # 4
  ldr r3, [ r2 ], # 4
  ldr r3, [ r2 ], # 4
  ldr r3, [ r2 ], # 4
  ldr r3, [ r2 ], # 4
  ldr r3, [ r2 ], # 4
  ldr r3, [ r2 ], # 4
  ldr r3, [ r2 ], # 4
  mvl r1, # init_mdcnfg_second
  str r1, [ r0, # mdcnfg ]		\ MDCNFG DRAM Configuration Register

\ static memory (SRAM not on first board)
  mvl r0, # _MC
  mvl r1, # init_MSC0			\ Default adjustments for CS1 and CS0
  str r1, [ r0, # msc0 ]
  mvl r1, # init_MSC1			\ Default adjustments for CS2 and CS3
  str r1, [ r0, # msc1 ]
  mvl r1, # init_MSC2			\ Default adjustments for CS4 and CS5
  str r1, [ r0, # msc2 ]
  mvl r1, # init_MECR			\ PC-Card interface for HAL and CAN
  str r1, [ r0, # mecr ]

\ Turn on the instruction cache
\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2
ICache? [if]
  mrc  #15 %000 r0 Cr1 Cr0 %000		\ read CP15, CR1=control reg
  orr  r0, r0, # bit12			\ set bit 12
  mcr  #15 %000 r0 Cr1 Cr0 %000		\ write CP15, CR1=control reg
[then]


\ ************************
\ Interrupt initialisation
\ ************************

  mvl r0, # _IC				\ Interrupt Controller base
  mov r1, # 0
  str r1, [ r0, # icmr ]		\ Interrupt Controller Mask Register
  					\ pending interrupt is masked from becoming active
  str r1, [ r0, # iclr ]		\ Interrupt Controller Level Register
  					\ all interrupts are IRQ-Interrupts
  str r1, [ r0, # icmr ]		\ disable all interrupts


\ **********************************************
\ All the boot level hardware is now initialised
\ **********************************************

((
\ Tell user we're alive - this code is useful during hardware
\ bring up.
  mvl	  r0, # uart3-setup code-start -
  bl      init-io-ll
l: spinit
  mov	  r0, # LED1 LED2 or
  bl	  switchLed
  bl      wasteTime
  mov	  r0, # LED1 LED2 or
  bl	  switchLed
  bl      wasteTime
  mov	  r0, # LED2
  bl	  ClearLed
\ ((
  mvl     r0, # _uart3
  mov     r1, # char A
  bl      seremit-ll
  mov     r1, # char R
  bl      seremit-ll
  mov     r1, # char M
  bl      seremit-ll
\ b spinit
))


\ The code in INTARM2.FTH provides a nest of stacks which are
\ set up by the interrupt entry code. This code sets up those
\ stacks.
  mov r0, # InterruptMask irq_32 or
  msr CPSR, _c r0                         
  mvl r0, # IRQ_StackTop		\ IRQ mode stack
  mov rsp, r0 

  mov r0, # InterruptMask fiq_32 or 
  msr CPSR, _c r0                         
  mvl r0, # FIQ_StackTop		\ FIQ mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask und_32 or
  msr CPSR, _c r0                         
  mvl r0, # UND_StackTop		\ UND mode stack
  mov rsp, r0                          

  mov r0, # InterruptMask abt_32 or
  msr CPSR, _c r0                         
  mvl r0, # ABT_StackTop		\ ABT mode stack
  mov rsp, r0                

  mov r0, # InterruptMask svc_32 or	\ Supervisor mode
  msr CPSR, _c r0
  mvl r0, # SVC_StackTop		\ ABT mode stack
  mov rsp, r0

  mov r0, # InterruptMask sys_32 or
  msr CPSR, _c r0                         
\ If you intend to run Forth in USR mode rather than SYS mode then you
\ need to switch here.


\ ************************************************
\ If required, copy ROM to RAM, and then run Forth
\ ************************************************

run-from-dram? [if]
  mov	  r0, # 0			\ Flash at 0000:0000
  mvl     r1, # CODE-START		\ start of code in DRAM
  mvl     r2, # CODE-SIZE		\ amount to copy
  add	  r2, r2, # 3			\ round to cell
  bic     r2, r2, # 3
l: dram-copy-loop
  ldr	  r3, [ r0 ], # 4
  str     r3, [ r1 ], # 4
  sub .s  r2, r2, # 4
  b .ne   dram-copy-loop
[then]

\ clear LED1 to show boot has finished
  mov     r0, # LED1
  bl      ClearLED

\ done this way in case a relative branch is out of range
  adr	  r0, # ECLD			\ join code in INITARM.FTH
  mov	  pc, r0

  next,
end-code

Reset_SA1110 CODE-START - ResetV !	\ Slot into the vector table
    
[then]

LED1 constant LED1
LED2 constant LED2

Code LEDchange	\ mask --
  mvl r1, # _GPIO
  ldr r2, [ r1, # gplr ]		\ read status
  and r2, r2, tos			\ LED & Status
  cmp r2, tos				\ if masked result = mask
  str .eq tos, [ r1, # gpcr ]		\   clear GPIO
  str .ne tos, [ r1, # gpsr ]		\ else set GPIO
  ldmfd   psp ! { tos }
  next,
end-code

Code LEDon	\ mask --
  mvl r1, # _GPIO
  str tos, [ r1, # gpcr ]		\ clear GPIO
  ldmfd   psp ! { tos }
  next,
end-code

Code LEDoff	\ mask --
  mvl r1, # _GPIO
  str tos, [ r1, # gpsr ]		\ set GPIO
  ldmfd   psp ! { tos }
  next,
end-code


