\ Boot loader application code for MPE ARM7 Development Kit.
\ The required ethernet MAC Address and IPv4 addresses are configured
\ by the two CREATEs directly beneath the copyright notice below.

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))

decimal

\ =============================================================================

create EtherAddress	\ -- addr
  $00 c, $10 c, $8B c, $F1 c, $44 c, $20 c,	\ six bytes for 48 bit MAC address
  					\ you must obtain these yourself!

create IpAddress	\ -- addr
  192 c,  168 c,  1 c,  251 c,		\ assign these as required


\ =============================================================================
(( 
: flash!	\ byte offset --
  2dup
  flashbaseA -rot 65536 +		\ -- base byte offset
  (writebyte) execute if		\ -- byte offset
    cr ." Failed flash write of " swap . ." at " dup .
    ." read " flashbaseA + c@ .
  else
    2drop
  endif
;
 ))

: flash!	\ byte offset --
  begin
    2dup	\ byte offset byte offset --
    FlashBaseA 	\ byte offset byte offset base --
    -rot $10000 +
    (writebyte) execute
    0=
  until
  2drop
;


: erase-user-flash      \ --
  init-flash-writer
  8 1 do
    begin
      cr ." Erase Flash Sector " i .
      FlashBaseA i (erasesector) execute 0=
    until
  loop
;

code run-user   \ -- ; copy flash into ram from address 0 and remap ram segment
        mvl     r0, # FlashBaseA 65536 +
        mvl     r1, # $010000
        mvl     r2, # 448 1024 *        ( 512k Flash - 64k boot loader )
l: cpy
        ldr     r3, [ r0 ], # 4
        str     r3, [ r1 ], # 4
        sub .s  r2, r2, # 4
        b .ne cpy

        mov     link, # $010000
        next,
end-code

cell buffer: DestPtr

: WriteFlashBlock       \ addr len --
\  cr ." Write a flash Block at" over . ." of" dup . ." Bytes"
  bounds ?do
    i c@ destptr @ flash!
    1 destptr +!
  loop
;


\ *************************************
\ Network order (big-endian) operations
\ *************************************

: w@(n)         \ addr -- u16
  dup c@ 8 lshift swap 1+ c@ or
;

: w!(n)         \ u16 addr --
  over 255 and over 1+ c!
  swap 8 rshift swap c!
;

: @(n)          \ addr -- u32
  dup w@(n) 16 lshift swap 2 + w@(n) or
;

: !(n)		\ u32 addr --
  over 65535 and over 2 + w!(n)
  swap 16 rshift swap w!(n)
;

: memswap       \ a1 a2 len --
  bounds ?do
    i c@                \ a1 c2 --
    over c@             \ a1 c2 c1 --
    i c!                \ a1 c2 --
    over c!             \ a1 --
    1+
  loop
  drop
;

: .z$           \ addr --
  begin
    dup c@ ?dup
  while
    emit 1+
  repeat
  drop
;

: (cksum)	\ sum addr len -- cksum ; computed into sum as 16 bit ints
  swap 2- swap				\ start address 2- so we can inc it more efficiently
  begin
    dup 1 >				\ check if more than 1 byte left
  while
    2- >r				\ dec & shift the length out of the way
    2+ tuck w@(n)			\ get 16 bits of data                   \ SJC001
    +					\ and add it into cksum
    dup $80000000 and			\ check top bit
    if
      dup $0FFFF and			\ get low 16 bits
      swap $10000 / +			\ and add in the high 16 bits !!!
    endif
    swap 				\ re-order values
    r>					\ retrieve length
  repeat
  if					\ odd byte left
    2+ c@ $0100 * +
    dup $80000000 and			\ check top bit
    if
      dup $0FFFF and			\ get low 16 bits
      swap $10000 / +			\ and add in the high 16 bits !!!
    endif
  else					\ all done
    drop
  endif
  begin
    dup $10000 /
  while
    dup $0FFFF and			\ get low 16 bits
    swap $10000 / +			\ and add in the high 16 bits !!!
  repeat
;

\ cksum is deferred to allow a m/c version (if required) for speed
defer cksum

\ compute the checksum
: hlcksum	\ addr len -- cksum ; computed as 16 bit ints
  0 -rot (cksum)
  $FFFF xor				\ and finally invert it !!
;
assign hlcksum to-do cksum

\ ------------------------------------------------------------------------------

create EtherAddressBC	\ -- addr ; for checking for broadcast addresses
  255 c, 255 c, 255 c, 255 c, 255 c, 255 c,

: MyIp?         \ *ip -- flag ; true for match
  4 IpAddress 4 compare 0=
;

: BroadCastMac? \ addr -- flag
  dup cell+ w@ swap @           \ u16 u32 --
  EtherAddressBC @ =
  swap EtherAddressBC cell+ w@ =
  and
;

cell buffer: pktInSize
1536 buffer: pktIn
cell buffer: udp-data
cell buffer: udp-datalen

: SendEtherPacket	\ -- 
  PktIn PktInSize @ send_ether_pkt
;

: SendPacket	\ --
  cr ." Send A Packet"
  pktinsize @ EHDR - pktin IPHDR.len w!(n)      \ new ip header length
  PktIn EHDR + IPHDR EHDR - cksum               \ .. and recalculate for header
  PktIn IPHDR.cksum w!(n)
  SendEtherPacket
;

128 equ T_IGNOREPACKET	\ -- n ; error codes
129 equ T_IP_BADSIZE
130 equ T_IP_BADVERSION
131 equ T_IP_OFFSET
132 equ T_IP_NOTFORME
133 equ T_IP_UNHANDLED
134 equ T_ICMP_BADSIZE
135 equ T_ICMP_BADTYPE
136 equ T_UNKNOWN_DOMAIN

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0001	equ ARPHARD_ETHER
$0800	equ ARPPROTO_IP

$0001	equ ARPOP_REQUEST
$0002	equ ARPOP_REPLY

struct  IPHDR
  EHDR field      _ehdr2
  1 field IPHDR.version
  1 field IPHDR.tos
  2 field IPHDR.len
  2 field IPHDR.id
  2 field IPHDR.off
  1 field IPHDR.ttl
  1 field IPHDR.p
  2 field IPHDR.cksum
  4 field IPHDR.src
  4 field IPHDR.dst
end-struct

1	equ IPPROTO_ICMP	\ control message protocol
17	equ IPPROTO_UDP		\ user datagram protocol

struct  ICMP
  IPHDR field     _iphdr1
  1 field ICMP.type
  1 field ICMP.code
  2 field ICMP.cksum
  2 field ICMP.id
  2 field ICMP.seq
  0 field ICMP.data
end-struct

$0000 equ ICMP_ECHOREPLY
$0008 equ ICMP_ECHOREQUEST

struct  UDP
  IPHDR field     _iphdr2
  2 field UDP.sport
  2 field UDP.dport
  2 field UDP.ulen
  2 field UDP.usum
  0 field UDP.data
end-struct

\ =======================================================================
( **                                                                    )
\ =======================================================================

$001 equ TFTP_RRQ                \ Read ReQuest
$002 equ TFTP_WRQ                \ Write ReQuest
$003 equ TFTP_DATA               \ DATA packet
$004 equ TFTP_ACK                \ ACKnowledge
$005 equ TFTP_ERROR              \ ERROR condition

cell buffer: Uploading?
cell buffer: blockid
cell buffer: UploadComplete?

: ResetTFTP	\ --
  0 DestPtr !
  false Uploading? !
  0 UpLoadComplete? !
  0 blockid !
;

: smERROR       \ --
  cr ." Error Packet"
  cr ." TFTP Error: "
  udp-data @ 4 + .z$
  ResetTFTP
  T_IGNOREPACKET throw
;

: smDone        \ --
  cr ." Upload Complete"
  ResetTFTP
  UpLoadComplete? on
;

: AckData       \ --
  cr ." Ack Data"
  TFTP_ACK udp-data @ w!(n)
  blockid @ 
  cr ." Block ID = " dup .
  udp-data @ 2+ w!(n)
  4 udp-datalen !
\  1 blockid +!
;

: smBegin       \ --
  cr ." Begin Packet"
  UpLoading? @ if
    cr ." TFTP Error: Upload begun before previous complete."
    ResetTFTP T_IGNOREPACKET throw
  then

  cr ." TFTP Receiving file: " udp-data @ 2+ .z$ space
  true Uploading? !
  AckData
;

: smData        \ --
  cr ." Data Packet"
udp-data @ 2 + w@(n) BlockID @ <> if
  udp-data @ cell+ udp-datalen @ cell -
  WriteFlashBlock
  udp-data @ 
  2 + w@(n) 
  dup ." BlockID?" . BlockID !
  udp-datalen @ 516 <> >r
  AckData
  r> if smDone then
else
  cr ." Duplicate Packet"
  AckData
then
;

: process-ip-udp-tftp   \ --
  udp-data @ w@(n)
  case

    TFTP_WRQ of         smBegin         endof
    TFTP_ERROR of       smError         endof
    TFTP_DATA of        smData          endof

    drop                cr ." TFTP Error: Unexpected packet"
                        ResetTFTP
                        T_IGNOREPACKET throw
  end-case
;

\ =======================================================================
( **                                                                    )
\ =======================================================================

20 buffer: temp-ip

: process-ip-udp        \ --
  pktInSize @ UDP                       <  if T_IGNOREPACKET throw then

  pktin udp.sport dup w@(n)     	\ *s s --
  pktin udp.dport dup w@(n)     	\ *s s *d d --
  dup >r                        	\ *s s *d d -- ; d --
  -rot w!(n)                    	\ swap port numbers for reply
  swap w!(n)
  r>                            	\ dest-port --

  pktin UDP.data udp-data !             \ split buffer to header and data
  pktinsize @ UDP - udp-datalen !       \ sections
  UDP pktinsize !

  case
    #69 of      process-ip-udp-tftp     endof
    drop T_IGNOREPACKET throw
  end-case

  \ recombine udp data with original header (assumes data in same place)
  udp-datalen @ pktinsize +!

  \ Update UDP data length ( data length+sizeofUDP header )
  udp-datalen @ 8 + pktin UDP.ulen w!(n)

  \ Update UDP checksum
  0 pktin UDP.usum w!(n)

  pktin IPHDR.src @(n) temp-ip !(n)
  pktin IPHDR.dst @(n) temp-ip 1 cells + !(n)
  IPPROTO_UDP temp-ip 2 cells + w!(n)
  udp-datalen @ 8 + temp-ip 10 + w!(n)          \ size of udp data + udp header
  pktin IPHDR + temp-ip 12 + 8 cmove

  temp-ip 20 cksum $FFFF xor
  udp-data @ udp-datalen @ cksum $FFFF xor +
  dup $FFFF > if 1+ then $FFFF and $FFFF xor

  pktin UDP.usum w!(n)

  \ .. and send
  SendPacket
;

: process-ip-icmp	\ --
  pktInSize @ ICMP                      <  if T_ICMP_BADSIZE throw then
  PktIn ICMP.type c@ ICMP_ECHOREQUEST   <> if T_ICMP_BADTYPE throw then

  ICMP_ECHOREPLY PktIn ICMP.type c!     \ switch type to reply
  0 PktIn ICMP.cksum w!(n)              \ reset checksum

  PktIn IPHDR +                         \ pointer to icmp header
  PktInSize @ IPHDR -                   \ size of ICMP block
  cksum PktIn ICMP.cksum w!(n)          \ recalc checksum

  SendPacket
;

: process-ip            \ --
  pktInSize @ IPHDR                     <  if T_IP_BADSIZE    throw then
  pktIn IPHDR.version c@ $45            <> if T_IP_BADVERSION throw then
  pktIn IPHDR.off w@(n)                    if T_IP_OFFSET     throw then
  pktIn IPHDR.dst MyIp?                 0= if T_IP_NOTFORME   throw then


  PktIn IPHDR.src                       \ swap addresses ready for reply
  PktIn IPHDR.dst 4 memswap

  0 PktIn IPHDR.cksum w!(n)             \ clear checksum field

  pktIn IPHDR.p c@
  case
    IPPROTO_ICMP of     process-ip-icmp         endof
    IPPROTO_UDP  of     process-ip-udp          endof
    drop T_IP_UNHANDLED throw
  end-case
;

\ =======================================================================
( **                                                                    )
\ =======================================================================

: process-arp-request   \ --
  pktIn AHDR.htype w@(n) ARPHARD_ETHER <> if T_IGNOREPACKET throw then
  pktIn AHDR.ptype w@(n) ARPPROTO_IP   <> if T_IGNOREPACKET throw then
  pktIn AHDR.hlen c@ 6                 <> if T_IGNOREPACKET throw then
  pktIn AHDR.plen c@ 4                 <> if T_IGNOREPACKET throw then
  pktIn ARP.targetIP MyIp?             0= if T_IGNOREPACKET throw then
  pktIn ARP.senderMAC                           \ swap in target MAC
  pktIn ARP.targetMAC 10 cmove
  EtherAddress PktIn ARP.senderMAC 6 cmove       \ put in my details
  IpAddress    PktIn ARP.senderIP  4 cmove

  ARPOP_REPLY PktIn AHDR.op w!(n)               \ set ARPOP_REPLY

  pktIn ARP.targetMAC pktIn EHDR.DstMac 6 cmove  \ set ethernet packet addresses
  pktIn ARP.senderMAC pktIn EHDR.SrcMac 6 cmove
  SendEtherPacket
;

: process-arp   \ --
  pktInSize @ ARP < if T_IGNOREPACKET throw then
  ARP PktInSize !
  pktIn AHDR.op w@(n) ARPOP_REQUEST = if
    process-arp-request
  else
    T_IGNOREPACKET throw
  then
;

\ =======================================================================
( **                                                                    )
\ =======================================================================

: boot-process	\ --

  pktInSize @ EHDR < if T_IGNOREPACKET throw then       \ too short

  pktIn EHDR.DstMac BroadCastMAC? 0= if         \ swap if not broadcast
        PktIn EHDR.DstMac
        Pktin EHDR.SrcMac
        6 memswap
  then

  pktIn EHDR.type w@(n)
  case

    ET_IP       of      process-ip      endof
    ET_ARP      of      process-arp     endof

    drop T_UNKNOWN_DOMAIN throw
  end-case
;

: download      \ --
  cr
  cr ." New Code Loader Running. Use Serial DownLoad or TFTP"
  cr
  erase-user-flash
  ResetTFTP
  init-CS8900 EtherAddress etheradd>CS
  cr ." Using IP address: "  decimal
    IpAddress    c@ 0 .r ." ."
    IpAddress 1 + c@ 0 .r ." ."
    IpAddress 2 + c@ 0 .r ." ."
    IpAddress 3 + c@ 0 .r
  init-EtherRx init-EtherTx
  begin

    key? if	\ check for serial download
          0 0 do                        \ I KNOW THIS IS A VERY LONG LOOP!!!!
            key i flash!
            i $FF and 0= if [char] . emit then
          loop
          begin  again			\ loaded, spin until reset
    then

    isrx? if
      pktIn 1536 get_ether_pkt pktInSize !
      ['] boot-process catch
      ?dup if
        case
 \         T_IGNOREPACKET   of   cr ." >> Ignored"               endof
 \         T_UNKNOWN_DOMAIN of   cr ." >> Unknown Domain"        endof
          T_IP_BADSIZE     of   cr ." >> Bad IP Size"           endof
          T_IP_BADVERSION  of   cr ." >> Bad IP Version"        endof
          T_IP_OFFSET      of   cr ." >> Bad IP Offset"         endof
 \         T_IP_NOTFORME    of   cr ." >> IP Not for me"         endof
          T_IP_UNHANDLED   of   cr ." >> Unhandled IP"          endof
          T_ICMP_BADSIZE   of   cr ." >> Bad ICMP Size"         endof
          T_ICMP_BADTYPE   of   cr ." >> Bad ICMP Type"         endof
          drop
        end-case
      then
    then
    UpLoadComplete? @
  until
;

: bflash!	\ byte offset --
  cr ." Do a flash store of" over . ." at" dup .
  flashbaseA -rot (writebyte) execute
;

: bWriteFlashBlock       \ addr len --
  cr ." Write a flash Block at" over . ." of" dup . ." Bytes"
  bounds ?do
    i c@ destptr @ bflash!
    1 destptr +!
  loop
;

: CopyRam>Boot
  0 65536 bWriteFlashBlock
;

: CurrentIP
  cr ." Current IP Address = " 
  IPAddress 
  dup c@ 0 .r [char] . emit
  dup 1 + c@ 0 .r [char] . emit
  dup 2 + c@ 0 .r [char] . emit
  3 + c@ 0 .r 
;

: getByte	\ addr char -- addr'
  word integer? 1 <> throw		\ collect and convert
  over c!  1+				\ stash data and update address
;

: getData	\ addr char n --
  1- 0 max 0 ?do
    tuck getByte swap
  loop
  drop  bl getByte drop
;

: GetIp
  base @ >r  decimal
  begin
    cr ." Please Enter New IP Address: xxx.xxx.xxx.xxx " cr
    Query   IPaddress [char] . 4 ['] getData catch
   while
     2drop drop  cr ." Invalid IP address"
  repeat
  CurrentIP
  r> base !
;

: CurrentMAC
  cr ." Current MAC Address = " 
  EtherAddress
  dup c@ 0 .r [char] , emit
  dup 1 + c@ 0 .r [char] , emit
  dup 2 + c@ 0 .r [char] , emit
  dup 3 + c@ 0 .r [char] , emit
  dup 4 + c@ 0 .r [char] , emit
  5 + c@ 0 .r 
;

: GetMac
  base @ >r  hex
  CurrentMAC
  begin
    cr ." Please Enter New MAC Address: aa-bb-cc-dd-ee-ff (base is hexadecimal) " cr
    Query   EtherAddress [char] - 6 ['] getData catch
   while
     2drop drop  cr ." Invalid MAC address"
  repeat
  CurrentMAC
  r> base !
;

: SetupEther
  CurrentIP
  GetIP
  init-flash-writer
  cr ." Erasing Flash sector 0"
  FlashBaseA 0 (erasesector) execute
  0 destptr !
  CopyRam>Boot
  cr ." Finished Copying BootLoader to Flash"
;

: SetupMAC
  GetMAC
  init-flash-writer
  cr ." Erasing Flash sector 0"
  FlashBaseA 0 (erasesector) execute
  0 destptr !
  CopyRam>Boot
  cr ." Finished Copying BootLoader to Flash"
;

: BootOptions 
  cr ." Hit E to Setup the Ethernet Address"
  cr ." Hit M to Setup the MAC Address"
  cr ." Hit F to Run Forth"
  cr ." Hit R to Run the Application"
  cr ." Hit D to Download New App"
  cr ." You Have 3 Seconds..."
  Start-Timers
  0
  begin
    10 ms
    1+ dup 300 =
    key? or
  until
  300 < if
    key upc case
      [char] E of  
        cr ." Setup Ethernet"
        SetupEther
	cr ." Please Reset ADK"
	begin FALSE until
      endof
      [char] M of
        cr ." Setup MAC"
	SetupMAC
	cr ." Please Reset ADK"
	begin FALSE until
      endof
      [char] F of
        cr ." Forth Monitor"
	cold
      endof
      [char] R of
        cr ." Run User Code"
        run-user
      endof
      [char] D of
	 cr ." Download new user app"
	 download
	 cr ." Please Reset ADK"
	 begin FALSE until
      endof
      cr ." Key Not recognised - Running User App"
      drop run-user
    end-case
  else
    cr ." no key pressed - Running User App"
    run-user
  then
  stop-timers
;

: bootstrap	\ --
  (init) init-ser			\ initialise Forth and serial line
  console ipvec !  console opvec !	\ select default I/O device
  cr cr ." ARM7 Development Kit BootStrap"
  $92 $ffff1c0c c!			\ Set PPI mode
  $ffff1c00 c@ 128 and 0=		\ Download if bit 7 is grounded
  if  download  then
  BootOptions
\  run-user
;

: QWE		\ --
  Resettftp
  erase-user-flash
  0
  $20000 $10000 do
    FlashBaseA 
    $55
    i (writebyte) execute
    1 and +  
  loop
  cr . ." errors"
;

: QWER		\ --
  Resettftp
  erase-user-flash
  0
  $20000 $10000 do
    begin
      FlashBaseA 
      $55
      i (writebyte) execute
      0=
    until
  loop
;

: QWERT		\ --
  Resettftp
  erase-user-flash
  0 0
  $20000 $10000 do
    FlashBaseA 
    $55
    i (writebyte) execute
    if
      1+
      begin
        swap 1+ swap
        FlashBaseA
        $55
        i (writebyte) execute
        0=
      until
    then
  loop
  cr . ." First Time Failures, " . ." Total Retries"
;

variable ECount

: QWERTY		\ --
  0 Ecount !
  100 0 do
    Resettftp
    erase-user-flash
    0 0
    $20000 $10000 do
      FlashBaseA 
      $55
      i (writebyte) execute
      if
        1+ cr ." Error at address: " i .
        begin
          swap 1+ swap
          FlashBaseA 
          $55
          i (writebyte) execute
          0=
        until
      then
    loop
    FlushCache
    Ecount +! drop
  loop
  Ecount @ cr ." Dropout: " . ." %"
;

: Qtest
  50 1 do
    cr ." Flash Delay of: " i .
    i fdelay !
    qwert
  loop
;

: QWERTYU		\ --
  Resettftp
  erase-user-flash
  0 0
  $80000 $10000 do
 \   FlashBaseA 
    $01000000
    $55
    i (writebyte) execute
    if
      1+ cr ." Error at address: " i .
      begin
        swap 1+ swap
    \    FlashBaseA 
        $01000000
        $55
        i (writebyte) execute
        0=
      until
    then
  loop
  cr . ." First Time Failures, " . ." Total Retries"
;
