\ LAN91C92/94/96 Ethernet Driver for the MPE ARM7 Development Kit

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))



$50000000 equ EtherBase

$0E equ BankSelect
$00 equ Bank0
$01 equ Bank1
$02 equ Bank2
$03 equ Bank3

\ Bank 0 registers
$00 equ TCRL
$01 equ TCRH
$02 equ TxStatus
$05 equ RCR
$0A equ MCR

\ Bank 1 registers
$00 equ Configl
$01 equ Configh
$04 equ IA0
$05 equ IA1
$06 equ IA2
$07 equ IA3
$08 equ IA4
$09 equ IA5
$0A equ gprl
$0B equ gprh
$0C equ Ctrl
$0D equ Ctrh

\ Bank 2 registers

$00 equ MMU
$02 equ PNR
$03 equ ARR
$06 equ Pointerl
$07 equ Pointerh
$08 equ Datal
$09 equ Datah
$0C equ IntStatus
$0D equ IntMask

\ Various bits and bobs...
$80 equ RelRx
$E0 equ RxRd
$40 equ TxWr
$08 equ AllocIntMask
#1530 equ MaxMsgSize
$20 equ AllocTx
$01 equ bit0
$02 equ bit1
$04 equ bit2
$08 equ bit3
$10 equ bit4
$20 equ bit5
$40 equ bit6
$80 equ bit7
$00 equ EvenTx
$20 equ OddTx
$01 equ RxTask#

\ =======================================================================
( **                                                                    )
( *S Hardware Interface Layer                                           )
( **                                                                    )
\ =======================================================================

variable e!-delay       10 e!-delay !
variable e@-delay        0 e@-delay !

internal

: SetBank               \ bank -- ; bank = 0 to 3
( ** Select active register bank.                                       )
  $03 and
  EtherBase BankSelect + c!
;

: e!                    \ val offset --
( ** Set a registers contents                                           )
  e!-delay @ 0 ?do loop
  EtherBase + c!
;

: e@                    \ offset -- val
( ** Read a register from selected bank.                                )
  e@-delay @ 0 ?do loop
  EtherBase + c@
;

external

\ =======================================================================
( **                                                                    )
( *S Diagnostics                                                        )
( **                                                                    )
\ =======================================================================

internal

: .bank                 \ --
( ** Display register contents for currently selected SMC bank.         )
  $10 0 do
    i e@ .byte space loop cr
;

external

: .reg                  \ --
( ** Display contents of all SMC Ethernet Controller's registers        )
  base @ >r hex
  cr ." Offset  " $10 0 do i .byte space loop cr cr
  #4 0
  do
    i setbank ." Bank " i . space .bank
  loop
  r> base !
;

\ =======================================================================
( **                                                                    )
( *S Driver Layer                                                       )
( **                                                                    )
\ =======================================================================

internal

: AllocateMemory        \ n --
( ** Allocate n bytes of memory to hold a packet for transmission.      )
  bank2 setbank
  $10 +                         \ because we will add to the msg length later
  $100 /                        \ because we allocate in 256 byte chuncks
  AllocTx or                    \ add command and length together
  MMU e!                        \ send the request to ether

  begin
    IntStatus e@                \ get interupt status register
    AllocIntMask and            \ loop until bit set
    0=
  while
    pause
  repeat

  ARR e@ $1F and                \ extract allocated packet number
  PNR e!                        \ store at packet number register
;

: QueueTx               \ --
( ** Enqueue last written packet for transmission.                      )
  $C0 MMU e!                    \ enqueue packet command
;

: WritePacket           \ addr len --
( ** Write an Ethernet frame to chip for transmission                   )
  >r                            \ store len for later
  $00 Datah e!                  \ write status word
  $00 Datah e!
  r@ 6 +                        \ len+6,=status,length and control word
  $FFFE and                     \ ensure an even number
  $100 /mod swap                \ split len to two 8 bit
  Datah e! Datah e!             \ store low then high count byte
  r@ bounds                     \ addr len bounds for loop
  do
    i c@ Datah e!               \ get buffer entry, store to chip
  loop
  r> 2 mod                      \ was length odd ?
  if                            \ y:
    OddTx Datah e!              \ packet control byte
  else                          \ n:
    $00 Datah e!                \ dummy write
    EvenTx Datah e!             \ packet control byte
  endif
;
external

: init-CS8900           \ --
( ** Initialise SMC Chip                                                )
  cr ." Configure SMC Ether Device"
  cr ."   Reset MMU"
  Bank2 SetBank $40 MMU e!      \ Reset MMU
  cr ."   Interrupt Source (unused)"
  bank1 SetBank $00 Configl e!  \ Active HI ints
  cr ."   Twisted Pair media"
  Bank1 SetBank $00 Configh e!  \ select 10BaseT transmission media
  cr ."   Reserve TX frame space"
  Bank0 SetBank $06 MCR e!      \ reserves 6*256=1536 bytes for TX
;


: init-EtherTx          \ --
( ** Enable Ethernet TX module                                          )
  cr ." SMC Ether Device, Enable TX"
  Bank0 SetBank
  Tcrl e@ 1 or Tcrl e!
  Bank1 SetBank  $08 Ctrh e!    \ set auto release of transmit memory
;


: init-EtherRx          \ --
( ** Enable Ethernet RX module                                          )
  cr ." SMC Ether Device, Enable RX"
  Bank0 SetBank
  $03 Rcr e!                    \ enable receiver and strip rx crc
  Bank2 SetBank
  bit0 IntMask e!               \ enable rx ints on ether chip

;

: etheradd>CS           \ addr --
( ** Store 6 byte Ethernet address into SMC chip from memory buffer.    )
  cr ." SMC Ether Device, Hardware Address "
  Bank1 SetBank
  dup c@ dup .byte IA0 e! 1+ [char] - emit
  dup c@ dup .byte IA1 e! 1+ [char] - emit
  dup c@ dup .byte IA2 e! 1+ [char] - emit
  dup c@ dup .byte IA3 e! 1+ [char] - emit
  dup c@ dup .byte IA4 e! 1+ [char] - emit
      c@ dup .byte IA5 e!
;

: IsRx?                 \ -- t|f
( ** Check if incoming data is present                                  )
  Bank2 SetBank
  IntStatus e@ 1 and
;

: get_ether_pkt         { *dest maxlen | plen -- len }
( ** Get pending receive packet to a buffer                             )

  Bank2 SetBank
  $00 Pointerl e!               \ reset pointer value
  RxRd Pointerh e!              \ read rx ram area, auto inc low pntr
  Datah e@ drop                 \ discard hash code value
  Datah e@ drop                 \ high status contains odd/even info
  Datah e@ Datah e@ $100 * +    \ 2 * 8 bit -> 16 bit number
  $07FF and                     \ byte count only uses 11 bits !
  $06 -
  dup maxlen > if               \ ensure length < maxlen
    drop maxlen
  endif

  dup -> plen

  0 ?do                         \ loop for rest of message
   datah e@                     \ read byte from ether chip ram
   *dest i + c!                 \ copy byte to buffer
  loop

  RelRx MMU e!                  \ remove and release memory from rx

  plen                          \ return packet length
;

: DiscardRX             \ --
( ** Throw away pending input packet due to lack of memory.             )
  Bank2 SetBank
  RelRx MMU e!                  \ remove and release memory from rx
;

: send_ether_pkt        \ addr len --
( ** Send packet from supplied buffer                                   )

  dup                           \ ensure packet is over 64 bytes or it will
  $40 < if                      \ be ignored by the SMC chipset
    drop $40
  endif

  Bank2 SetBank
  dup AllocateMemory            \ get memory for transmission
    $00 Pointerl e!             \ reset offset value
    TxWr Pointerh e!            \ set to write Tx ram area
    WritePacket                 \ send packet to ether chip
    QueueTx
;

: set-ether             \ e1 e2 e3 e4 e5 e6 --
  Bank1 SetBank
  IA5 e!
  IA4 e!
  IA3 e!
  IA2 e!
  IA1 e!
  IA0 e!
;

1536 buffer: pbuff

: packet-listen         \ --
  init-cs8900
  1 2 3 4 5 6 set-ether
  init-EtherRx				\ now init the receiver
  init-EtherTx				\ & the transmitter
  begin
    isrx?
    if
      pbuff 1536 erase
      pbuff 1536 get_ether_pkt
      pbuff swap 1 max dump
    then
    key?
  until
;


: ee!           \ word offset --
  Bank2 SetBank pointerl e!
  Bank1 SetBank dup 255 and gprl e!
  8 rshift gprh e!
  ctrl e@ $F8 and $5 or ctrl e!
  begin ctrl e@ 1 and 0= until
  ctrl e@ $F8 and ctrl e!
;

: ee@           \ offset -- word
  Bank2 SetBank pointerl e! Bank1 SetBank
  ctrl e@ $F8 and $6 or ctrl e!
  begin ctrl e@ 2 and 0= until
  ctrl e@ $F8 and ctrl e!
  gprh e@ 8 lshift gprl e@ or
;


