\ Startup Code for User Applications in the MPE ARM7 Development Kit

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))


\ Segment Descriptor Registers:
\
\               15:             Reserved (0)
\               14:             SVC Write Allowed?
\               13:             SVC Read Allowed?
\               12:             USR Write Allowed?
\               11:             USR Read Allowed?
\               10:             Cached?
\               9:              Reserved (0)
\               8:              HalfWordMode?
\               0..7:           Bank Select     %0000:0001 = Bank0
\                                               %0000:0010 = Bank1
\                                               %0000:0100 = Bank2
\                                               %0000:1000 = Bank3
\                                               %0001:0000 = Bank4
\                                               %0010:0000 = Bank5
\                                               %0100:0000 = Bank6
\                                               %1000:0000 = Bank7
\
\ Bank Configuration Register
\
\               15:             16Bit? ( 0 = 8bit )
\               14,13,12:       Wait states (0..7)
\               11...0:         Chip Select Configuration

asmcode
L: HandlerReset
\ Set up PPI (8255 clone)
  \ PortA/B Inputs, PortC Outputs
  mov r0, # $92         adr r1, _PPI_CTLR   strb r0, [ r1 ]
  mov r0, # $00         adr r1, _PPI_PC     strb r0, [ r1 ]

  \ Channel2 in UART Mode by default
  mov r0, # $00         adr r1, $FFFF0C00   strb r0, [ r1 ]
(( 
\ Set RAM as segment 2 using CS3=HI CS2=LO , 16 bit, 1 wait
  adr r0, $9090         adr r1, _BCR2       str r0, [ r1 ]
\ read/write, cachable, 32 bit, Bank 2
  adr r0, $7C04         adr r1, _SDR2       str r0, [ r1 ]
\ 448k from $00000 to $70000
  adr r0, $00000        adr r1, _START2     str r0, [ r1 ]
  adr r0, $70000        adr r1, _STOP2      str r0, [ r1 ]

\ Set VRAM as segment 3 using CS3=HI CS2=LO , 16 bit, 1 wait
  adr r0, $9090         adr r1, _BCR3       str r0, [ r1 ]
\ read/write, cachable, 32 bit, Bank 3
  adr r0, $7808         adr r1, _SDR3       str r0, [ r1 ]
\ 64k from $70000 to $80000
  adr r0, $70000        adr r1, _START3     str r0, [ r1 ]
  adr r0, $80000        adr r1, _STOP3      str r0, [ r1 ]
 ))

\ Set DRAM as segment 6 using CS5=HI CS4=LO , 16 bit, 1 wait
  adr r0, $A900         adr r1, _BCR6a      str r0, [ r1 ]
  adr r0, $0033         adr r1, _BCR6b      str r0, [ r1 ]
\ read/write, cacheable, 32 bit, Bank 6
  adr r0, $7C40         adr r1, _SDR6       str r0, [ r1 ]
\ 2Mb from $0200.0000 to $0220.0000-1
  adr r0, $00000000     adr r1, _START6     str r0, [ r1 ]
  adr r0, $00200000     adr r1, _STOP6      str r0, [ r1 ]
\ DRR refresh = (tr/cycles)*clock
\             = (16*10^-3/1024)*24*10^6 = 375
  adr r0, #375          adr r1, _DRR        str r0, [ r1 ] 
\ wait 200us min
  mov     r2, # #1024
l: drw1
  sub .s  r2, r2, # 1
  b .ne   drw1
\ Perform 8 read accesses to wake it up
  adr     r1, # $02000000
  mov     r2, # 8
l: drw2
  ldr     r0, [ r1 ], # 4
  sub .s  r2, r2, # 1
  b .ne   drw2

\ erase 2M of DRAM
  mov r0, # $0
  mov r1, # $0
  mvl r2, # $200000
L: DRErase
  str r0, [ r1 ], # 4
  sub .s r2, r2, # 4
  b .ne DRErase


\ Local SRAM enabled, mapped at 6000:0000 to 6000:07FF
  mov r0, # $03         adr r1, _LSCR       str r0, [ r1 ]
\ Enable local SRAM as Cache Mode
  mov r0, # $01         adr r1, _CCR        str r0, [ r1 ]

\ Set Ethernet as segment 4 using CS1, 8 bit, 4 wait
  adr r0, %0100000000001100             adr r1, _BCR4       str r0, [ r1 ]
\ Setup Segment4 as read/write, non-cachable, Bank4
  adr r0, %0111100000010000             adr r1, _SDR4       str r0, [ r1 ]
\ 1k from $5000:0000
  adr r0, $50000000                     adr r1, _START4     str r0, [ r1 ]
  adr r0, $50000400                     adr r1, _STOP4      str r0, [ r1 ]

\ Copy vector table to low RAM if it exists
  mvl r0, # Vector_Start                \ Source address.
  mov r1, # $0                         \ Destination address.
  mvl r2, # Vector_Len		       \ Length to copy.
L: CopyJumpTable
  ldr r3, [ r0 ], # 4
  str r3, [ r1 ], # 4
  sub .s r2, r2, # 4
  b .ne CopyJumpTable

  b ECLD
end-code

HandlerReset ResetV  !

