\ ARM generic startup code

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20030508 SFP002 Removed AIF header code - now in AIFHEAD.FTH.
20020610 SFP001 Changed S0 to point to LAST available cell
))


\ ==========
\ *! initarm
\ *T Generic start up
\ ==========
\ *P The file ARM\INITARM.FTH contains the generic vector table,
\ ** Forth start up code and Forth initialisation tables. It must
\ ** be the first file compiled that generates code unless you
\ ** are compiling operating specific code, such as with an AIF
\ ** header.

\ *P For CPUs such as the Atmel AT91 series which have a vectored
\ ** interrupt controller, this file is not used, and a variant of
\ ** this code will be included in a CPU and hardware specific file
\ ** INITxxxx.FTH. Such files may be found in the HARDWARE directory
\ ** e.g. Hardware\EB55\InitARM55800.fth is used for the EB55 board
\ ** which has an AT91x55800A CPU.

\ *S Vectors
\ *P This code branches by loading the PC with an ABSOLUTE address contained
\ ** in a location at a RELATIVE offset from the LDR instruction. This avoids
\ ** the use of a relative branch which may be out of range if the execution
\ ** address is high in memory.
asmcode
L: Vector_Start                         \ 'Hard wired' ARM Processor vectors
  ldr pc, ResetV                        \ 00 - Reset
  ldr pc, UndefV                        \ 04 - Undefined Instructions
  ldr pc, SWIV                          \ 08 - SWI instructions
  ldr pc, PAbortV                       \ 0C - Instruction fetch Aborts
  ldr pc, DAbortV                       \ 10 - Data access aborts
  ldr pc, UnusedV                       \ 14 - Reserved (address exception if 26bit)
  ldr pc, IRQV                          \ 18 - IRQ interrupts
  ldr pc, FIQV                          \ 1C - FIQ interrupts

\ *P For faster response speed, you may wish to place your FIQ handler here, replacing
\ ** the ldr pc, FIQV instruction with the first instruction of the FIQ handler. This saves
\ ** the ARM from the extra overhead of having to branch to the handler.

\ *P The following vectors usually end up in RAM after the EPROM/Flash is remapped
\ ** into high memory and RAM is remapped to low memory.
L: Soft_Vectors                         \ Handler routines can be 'socketed' in here
L: ResetV       dd 0                    \ HandlerReset
L: UndefV       dd 0                    \ HandlerUndef
L: SWIV         dd 0                    \ HandlerSWI
L: PAbortV      dd 0                    \ HandlerPAbort
L: DAbortV      dd 0                    \ HandlerDAbort
L: UnusedV      dd 0                    \ HandlerUnused
L: IRQV         dd 0                    \ HandlerIRQ
L: FIQV         dd 0                    \ HandlerFIQ
L: Vector_End

L: DefHandlerReset
  adr r13, # ECLD
  mov pc, r13
end-code

chere vector_start - equ vector_len

DefHandlerReset    ResetV  !

 
\ ------------------------
\ *S Start of Forth system
\ ------------------------
\ *P This section contains initial values of Forth
\ ** registers and the code to initialise and 
\ ** run Forth.
L: CLD1		\ XT of cold start word is placed here
  0 ,			\ compiler inserts address of word to execute at cold start
L: DATASTACK
  init-s0 ,
L: RETURNSTACK
  init-r0 ,
L: USERPOINTER
  init-u0 ,

asmcode
L: ECLD
\ Set up the Forth registers
  ldr psp, DATASTACK                    \ set data stack
  ldr rsp, RETURNSTACK	                \ set return stack
  ldr up, USERPOINTER	                \ set user pointer
  ldr pc, CLD1			        \ get address of cold start word and jump
end-code


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] sp-guard [if]
0 equ sp-guard
[then]

[undefined] umbilical? [if]
\   true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

