\ InitStacks.fth - Default ARM stack initialisation

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20041004 MPE001 Reduced size of code.
))

decimal

l: ^int_stack_top			\ interrupt stacks top
  int_stack_top ,
l: ^sys/user_stack_top			\ system\user mode stacks
  sys/user_stack_top ,

int_stack_top equ irq_stack_top
irq_stack_top irq_stack_size - equ fiq_stack_top
fiq_stack_top fiq_stack_size - equ abt_stack_top
abt_stack_top abt_stack_size - equ und_stack_top
und_stack_top und_stack_size - equ svc_stack_top


Proc InitStacks	\ default stack initialisation
  mov	r5, link		\ preserve return address

\ Set up IRQ, FIQ, ABORT and UNDef stacks
  msr	CPSR, _c  # ARM_MODE_IRQ I_BIT or F_BIT or	\ IRQ Interrupt Mode + Stack Setup

  ldr	r13, ^INT_STACK_TOP
  mov	r0, r13
  sub	r0, r0, # IRQ_STACK_SIZE

  msr	CPSR, _c  # ARM_MODE_FIQ I_BIT or F_BIT or 	\ FIQ Interrupt Mode + Stack Setup
  bl    initFIQ						\ sets up FIQ bank
  sub	r0, r0, # FIQ_STACK_SIZE

  msr	CPSR, _c  # ARM_MODE_ABORT I_BIT or F_BIT or	\ Abort Mode + Stack Setup
  mov	r13, r0
  sub	r0, r0, # ABT_STACK_SIZE

  msr	CPSR, _c  # ARM_MODE_UNDEF I_BIT or F_BIT or	\ Undefined Mode + Stack
  mov	r13, r0
\ The following line is only required if the return value is required
\  sub	r3, r0, # UND_STACK_SIZE			\ R3=bottom of exception stack space

\ Set up SVC (supervisor) stack
  msr	CPSR, _c  # ARM_MODE_SVC  I_BIT or F_BIT or	\ Supervisor Mode + Stack
  mov	r13, r0
  sub	r0, r0, # SVC_STACK_SIZE

\ Set up SYStem and USER mode stack
  msr	CPSR, _c  # ARM_MODE_SYS I_BIT or F_BIT or	\ System/User Mode
  ldr	r13, ^SYS/USER_STACK_TOP
\ The following line is only required if the return value is required
\  sub	r4, r13, # SYS/USER_STACK_SIZE			\ R4=bottom of SYS/USER stack space

\ The CPU is in SYSTEM mode with all stacks set up and interrupts
\ disabled.

\ Remove the comments from  these lines if you want to run in
\ USER mode, otherwise comment them out. NOT RECOMMENDED.
\ In USER mode, the C field of the CPSR cannot be modified.
\ Consequently, interrupts cannot be disabled or enabled.
\ Unless you are coding for an operating system with SWIs to
\ handle interrupt enable/disable, USER mode is NOT recommended.
\  mov	r1, # ARM_MODE_USER			\ User Mode - Set application mode and enable interrupts
\  msr	CPSR, _c r1

  mov		pc, r5			\ return
end-code

