\ ARM exception (interrupt) handlers

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20040216 MPE001 Derived from INTARM3.FTH
))


\ =========
\ *! intarm3min
\ *T Minimal Interrupt handlers
\ =========
\ *P The file ARM\INTARM3MIN.FTH contains ARM interrupt
\ ** handlers in the v6.2 style for CPUs with vectored interrupt
\ ** controllers. Unlike INTARM3.FTH, INTARM3MIN makes the
\ ** following assumptions:
\ *B FIQ interrupts are not nested and never reenable FIQ or IRQ
\ ** interrupts.
\ *B FIQ interrupts can be enable inside IRQ handlers.
\ *B Code density is more important than overall interrupt latency.

\ *P INTARM3MIN.FTH requires ARM\ARMDEF.FTH to be compiled before
\ ** the SFRxxxx file for your particular CPU. The file ARM/STACKDEF.FTH
\ ** provides default main task and stack layouts and should be
\ ** compiled from the control file or copied into your control file.
\ ** Default stack initialisation code is provided by the assembler
\ ** routine INITSTACKS in ARM\INITSTACKS.FTH. This routine can be
\ ** referenced from your initialisation code.

\ *P Each exception (interrupt) type requires a predefined stack
\ ** frame on which the interrupt handler builds the Forth stacks
\ ** and USER area. The system initialisation code must preset the
\ ** banked R13 registers to the top of the frames. This code
\ ** requires the Forth return stack pointer to be R13.
\ ** See ARM\CONFIGS\LPC2106U.CTL (contains stack layout) and
\ ** ARM\HARDWARE\INITLPC210X.FTH (contains initialisation code)
\ ** for examples.

\ *P In order to support exception nesting the equate #IRQs in the
\ ** control file must be set to the maximum number of nestings
\ ** required for ALL modes. When #IRQs is greater than 1 (to indicate
\ ** IRQ nesting), the exception handler entry and exit code is
\ ** extended to handle all exceptions in SVC mode. This avoids
\ ** corruption of R14 (the LINK register) when nesting occurs.
\ ** The equate #IRQs is used to calculate the size and layout
\ ** of the IRQ and SVC mode stacks along with the equates #SWIs
\ ** and #FIQs in the control file.

\ *P Note that it is implicit throughout the code that the three
\ ** system registers UP, PSP, RSP are always set so that:
\ *C   UP > PSP > RSP
\ *P Because of this layout, data stack underflows may corrupt
\ ** the first part of the USER area. You have been warned.
\ ** During testing, it may be as well to set the equate
\ ** SP-GUARD to 2 or 3 in your control file to leave a few
\ ** guard cells on the data stack.

only forth definitions
decimal


\ ****************
\ *S Configuration
\ ****************
\ *P The following equates define values used for the
\ ** equate ARM-VIC? that controls how IRQ and FIQ
\ ** exceptions are processed. Note that if ARM-VIC?
\ ** contains any other values, no IRQ or FIQ code will
\ ** be compiled, permitting you to define different versions
\ ** for other CPUs in another file. If you find and code other
\ ** vectored interrupt controllers for ARMs, we will be happy
\ ** to include your code in this file in future releases.

[undefined] NO-VIC [if]
0 equ NO-VIC	\ -- n
\ *G specifies that no vectored interrupt controller is present.
1 equ AT91-VIC	\ -- n
\ *G specifies that the AT91 AIC vectored interrupt controller
\ ** is present.
2 equ S3C-VIC	\ -- n
\ *G specifies that the Samsung vectored interrupt controller
\ ** as used in the S3C4510B is present.
3 equ PL190-VIC	\ -- n
\ *G Specifies that the ARM PL190 vectored interrupt controller
\ ** as used by the Philips LPC210x is present.
[then]

[undefined] ARM-VIC? [if]
NO-VIC equ ARM-VIC?	\ -- n
\ *G If the equate ARM-VIC? is undefined before this file is compiled,
\ ** the generic (no vectored controller) will be compiled for FIQ and
\ ** IRQ handling.
[then]

1 equ test-isr?	\ -- n ; non-zero to compile test code
\ *G Non-zero to compile test and diagnostic code.

0 equ Reserved_ISR?	\ -- n
\ *G When non-zero, a handler will be installed for the RESERVED
\ ** exception (vector at $0000:0014). This vector was used by the
\ ** 28 bit architecture, but is now reserved for future expansion.

interpreter
: !call		\ xt addr --
\ *G An INTERPRETER word to create a call opcode to xt at addr.
  tuck					\ -- addr xt addr
  8 +  -				\ -- addr offset
  2 rshift $00FFFFFF and $EB000000 or 	\ -- addr BLins
  swap !				\ set call instruction
;
target


\ ***********************
\ *S Interrupt management
\ ***********************

[undefined] EI [if]
code EI		\ -- ; enable interrupts
\ *G Global enable interrupts, both FIQ and IRQ.
  mrs  r0, cpsr
  bic  r0, r0, # $C0			\ clear I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code

code DI         \ -- ; disable interrupts
\ *G Global disable interrupts, both FIQ and IRQ.
  mrs  r0, cpsr
  orr  r0, r0, # $C0			\ set I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code

code [I		\ R: -- x ; preserve I/F status on return stack, disable ints
\ *G Preserve I/F status on return stack, disable interrupts.
\ ** The state is restored by I].
  mrs   r0, cpsr			\ get CPSR
  stmfd rsp ! { r0 }			\ save
  orr   r0, r0, # $C0			\ set I and F bits
  msr   cpsr _c r0			\ modify control bits
  next,
end-code

code I]		\ R: x -- ; restore int. status from r. stack
\ *G Restore interrupt status saved by [I from the return stack.
  ldmfd rsp ! { r0 }
  msr   cpsr _c r0
  next,
end-code
[then]



\ *************************************
\ *S Support for complex abort handlers
\ *************************************
\ *P The complex exception handlers store the ARM CPU state in a
\ ** data frame. The frame is 72 bytes long (18 4 byte cells),
\ ** whose format is:
\ *E --- high ---
\ ** ABORT mode R14/link = faulting PC+8
\ ** faulting R12
\ ** faulting R11
\ ** faulting R10
\ ** faulting R9
\ ** faulting R8
\ ** faulting R7
\ ** faulting R6
\ ** faulting R5
\ ** faulting R4
\ ** faulting R3
\ ** faulting R2
\ ** faulting R1
\ ** faulting R0
\ ** faulting R14
\ ** faulting R13
\ ** faulting CPSR
\ ** ABORT mode CPSR on entry
\ ** --- low --

test-isr? [if]

struct /exframe	\ -- n
\ *G structure defining the exception frame
  int exf.abtCPSR
  int exf.CPSR
  int exf.r13
  int exf.r14
  int exf.r0
  int exf.r1
  int exf.r2
  int exf.r3
  int exf.r4
  int exf.r5
  int exf.r6
  int exf.r7
  int exf.r8
  int exf.r9
  int exf.r10
  int exf.r11
  int exf.r12
  int exf.abtLink
end-struct

: .item		\ addr -- addr+4
\ *G Display a cell item at addr and increment addr.
  dup @ .dword space  cell +
;

: .items	\ addr n -- addr+4n
\ *G Display n items at addr and increment addr.
  0 ?do  .item  loop
;

: .frame	\ ^frame --
\ *G Display the CPU state pointed to by the data frame.
  cr ." === Registers ==="
  cr ." CPSR = " dup exf.CPSR @ .dword	\ CPSR
  cr ." R0  = "  dup exf.R0 4 .items	\ R0..R3
  cr ." R4  = "  4 .items		\ R4..R7
  cr ." R8  = "  4 .items		\ R8..R11
  cr ." R12 = "  .item drop             \ R12
     dup exf.R13 2 .items drop		\ R13 R14
     exf.abtLink .item drop		\ R15
;

[undefined] ip>nfa [if]
: name?         \ addr -- flag
\ *G Check to see if the supplied address is a valid NFA.
\ ** This word is implementation dependent.
\ ** A valid NFA for MPE embedded systems satisfies the following:
\ *B All characters within string are printable ASCII within range 33..126
\ *B String Length is non-zero in range 1..31 and bit 7 is set, ignore bits 6, 5
  dup aligned over <>
  if  drop  FALSE exit  endif
  count					\ c-addr u --
  dup  $9F and  $81 $9F within? 0=	\ NFA first byte = 1SIxxxxx, count = xxxxx
  					\ mask           = 10011111
  if  2drop  0  exit  then
  $01F and bounds ?do
    i c@ #33 #126 within? 0=		\ check all ascii chars
    if  unloop  FALSE exit  then
  loop
  TRUE
;

: ip>nfa	\ addr -- nfa
\ *G Attempt to move backwards from an address within a definition
\ ** to the relevant NFA.
  2-				\ NFA must be at least 'n' bytes backwards
  begin
    dup name? 0=
   while
    1-
  repeat
;
[then]

: check-aligned	\ addr -- addr'
\ *G Check addr for cell alignment, report and correct if
\ ** misaligned.
  dup aligned over <>
  if  ." (Misaligned) "  aligned  then
;

' and equ start-of-code
prog sec-end  equ end-of-code

: .rsframe	\ ^frame --
\ *G Display the return stack indicated by the frame, assuming
\ ** the Forth RSP=R13 and RUP=R11.
  dup exf.R13 @				\ -- ^frame rsp
  cr ." RSP = " dup .dword space check-aligned
  swap exf.R11 @ r0-offset + @		\ -- rsp rsTop
  ."   R0 = " dup .dword space check-aligned
\ Note that for negative steps, +LOOP processes the limit value.
  cell -				\ rsTop is not used
  cr ." --- Return stack high ---"
  ?do
    cr i .dword space  i @ dup .dword space
    dup start-of-code end-of-code within
    if  ip>nfa .name  else  drop  endif
  cell negate +loop
  cr ." --- Return stack low --- "
;

: .psframe	\ ^frame --
\ *G Display the data stack indicated by the frame, assuming
\ ** the Forth PSP=R12 and RUP=R11.
  dup exf.R12 @				\ -- ^frame psp
  cr ." PSP = " dup .dword space check-aligned
  over exf.R11 @ s0-offset + @		\ -- ^frame psp psTop
  ."   S0 = " dup .dword space check-aligned
\ Because of the stack caching S0 is a cell LOWER than PSP
\ for an empty stack. Note also that for negative steps,
\ +LOOP processes the limit value.
  2dup cell + u>
  if  2drop  cr ." Data stack underflowed"  exit  endif
  2dup cell + =
  if  2drop  cr ." Data stack empty"  exit  endif
  cr ." --- Data stack high ---"	\ -- ^frame psp psTop/S0
  2dup <> if
    cell - do
      cr i .dword space  i @ .dword
      cell negate
    +loop
  else
    2drop
  endif
  cr ." --- Data stack low --- "
  cr ." rTOS/R10  " exf.R10 @ .dword
;

[then]


\ ********************************
\ *S Undefined instruction handler
\ ********************************

defer Undef_handler	\ ^frame ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to a data frame and a pointer to the instruction,
\ ** returns true if a fixup has been performed. If the return value
\ ** is non-zero, the instruction is rerun otherwise the instruction
\ ** is skipped.
  assign drop to-do Undef_handler

PROC Undef_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r2, CPSR			\ save CPSR for mode switch
  mrs   r3, SPSR			\ save SPSR for nesting
  bic   r1, r2, # $01F			\ clear mode bits from CPSR
  and   r0, r3, # $01F			\ extract mode bits from SPSR
  orr   r0, r0, r1			\ switch to old mode to extract R13, R14
  msr   CPSR, _c r0
  mov   r4, r13
  mov   r5, r14
  msr   CPSR, _c r2			\ switch mode back
  stmfd rsp ! { r2, r3, r4, r5 }	\ save data on abort stack
  mov   r0, rsp				\ base address of data frame

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0

  sub   psp, psp, # 8			\ indicate depth=2
  str   r0, [ psp ]			\ save pointer to data frame
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' Undef_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r2, r3, r4, r5 }	\ restore SPSR and others saved
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 4		\ return, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

Undef_exception UndefV !


\ ---------------
\ Undef test code
\ ---------------

test-isr? [if]
: testundef	\ ^frame ^ins -- fixed? ; handle undefined instruction
\ *G Test handler for Undefined Instructions.
  hex  console opvec !  console ipvec !
  cr ." Undef exception at " dup .lword
  cr ." Instruction        " @ .lword
  dup .frame  dup cr .psframe  cr .rsframe
  cr ." Restarting system ..."		\ skip instruction
[defined] reboot [if] reboot [else] 0 execute [then]
  0
;
assign testundef to-do Undef_handler

: run-undef	\ --
\ *G Causes an Undefined Instruction exception.
  cr ." before"
  [asm
    mrc  7 0 r0, cr0, cr0, 0		\ CP7 doesn't exist, so should fault
  asm]
  cr ." after"
;
[then]


\ *************************
\ *S Prefetch Abort handler
\ *************************

defer PAbort_handler	\ ^frame ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to a data frame and a pointer to the instruction,
\ ** returns true if a fixup has been performed. If the return value
\ ** is non-zero, the instruction is rerun otherwise the instruction
\ ** is skipped.
  assign drop to-do PAbort_handler

PROC PAbort_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r2, CPSR			\ save CPSR for mode switch
  mrs   r3, SPSR			\ save SPSR for nesting
  bic   r1, r2, # $01F			\ clear mode bits from CPSR
  and   r0, r3, # $01F			\ extract mode bits from SPSR
  orr   r0, r0, r1			\ switch to old mode to extract R13, R14
  msr   CPSR, _c r0
  mov   r4, r13
  mov   r5, r14
  msr   CPSR, _c r2			\ switch mode back
  stmfd rsp ! { r2, r3, r4, r5 }	\ save data on abort stack
  mov   r0, rsp				\ base address of data frame

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0

  sub   psp, psp, # 8			\ indicate depth=2
  str   r0, [ psp ]			\ save pointer to data frame
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' PAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r2, r3, r4, r5 }	\ restore SPSR and others saved
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 4		\ return, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

PAbort_exception PAbortV !

test-isr? [if]

: testPabort	\ ^frame ^ins -- fixed?
\ *G Test PABORT handler.
  decimal  console opvec !  console ipvec !
  cr ." PAbort exception at " .lword
  dup .frame  dup cr .psframe  cr .rsframe
  cr ." Restarting system ..."		\ skip instruction
[defined] reboot [if] reboot [else] 0 execute [then]
  0
;
assign testPabort to-do PAbort_handler

: run-Pabort   $05000000 execute  ;	\ hardware specific
\ *G Run test PAbort handler.
[then]


\ ******************
\ *S Data Abort handler
\ ******************

defer DAbort_handler	\ ^frame ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to a data frame and a pointer to the instruction,
\ ** returns true if a fixup has been performed. If the return value
\ ** is non-zero, the instruction is rerun otherwise the instruction
\ ** is skipped. The frame format is as for the PAbort handler above.
  assign drop to-do DAbort_handler

PROC DAbort_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r2, CPSR			\ save CPSR for mode switch
  mrs   r3, SPSR			\ save SPSR for nesting
  bic   r1, r2, # $01F			\ clear mode bits from CPSR
  and   r0, r3, # $01F			\ extract mode bits from SPSR
  orr   r0, r0, r1			\ switch to old mode to extract R13, R14
  msr   CPSR, _c r0
  mov   r4, r13
  mov   r5, r14
  msr   CPSR, _c r2			\ switch mode back
  stmfd rsp ! { r2, r3, r4, r5 }	\ save data on abort stack
  mov   r0, rsp				\ base address of data frame

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0

  sub   psp, psp, # 8			\ indicate depth=2
  str   r0, [ psp ]			\ save pointer to data frame
  sub   tos, link, # 8			\ point to bad instruction
  bl    ' DAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r2, r3, r4, r5 }	\ restore SPSR and others saved
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 8		\ return, rerun instruction
  sub .s      pc, link, # 4		\ return, skip instruction
end-code

DAbort_exception DAbortV !

test-isr? [if]
: testdabort	\ ^frame ^ins -- fixed?
\ *G Test DABORT handler.
  decimal  console opvec !  console ipvec !
  cr ." DAbort exception at " dup .lword
  cr ." Instruction         " @ .lword
  dup .frame  dup cr .psframe  cr .rsframe
  cr ." Restarting system ..."
[defined] reboot [if] reboot [else] 0 execute [then]
  cr ." Instruction skipped"  0		\ skip instruction
;
assign testdabort to-do DAbort_handler

: run-dabort   $05000000 @ drop  ;		\ hardware specific
\ *G Run test PAbort handler.
[then]


\ **********************************************
\ *S Reserved (26 bit address exception) handler
\ **********************************************
\ *P This exception is only used for systems which
\ ** support the 26 bit PC mode of early ARM cores.

Reserved_ISR? [if]

defer Unused_handler	\ ^ins -- fixed?
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do Unused_handler

PROC Unused_exception	\ --
\ *G The UNUSED exception handler calls UNUSED_handler above.
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' Unused_handler

  cmp   tos, # 0			\ check return result
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .ne .s  pc, link, # 4		\ return from exception, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

Unused_exception UnusedV !

test-isr? [if]
: testunused	\ --
\ *G Test code for the UNUSED exception.
  decimal  console opvec !  console ipvec !
  cr ." Unused exception"
;
assign testunused to-do Unused_handler
[then]

[then]


\ ****************************
\ *S AT91 IRQ and FIQ handlers
\ ****************************
\ *P This section is not a treatise on the Atmel Advanced
\ ** Interrupt Controller. These words provide a fairly basic
\ ** set of tools for using the AIC.
\ ** The notes in the Generic IRQ section about interrupt nesting
\ ** apply here.

\ *P This code requires KERNEL62.FTH and the equate
\ ** COLDCHAIN? must be set non-zero in the control file.

\ *P Basic clock enabling of the AIC should be performed in
\ ** the CPU specific startup file, e.g.
\ ** ARM\HARDWARE\EB55\INITARM55800.FTH.

\ *P If the EQUate #IRQs is greater than one, IRQ nesting is
\ ** supported with the penalty of greater overhead.
\ ** IF IRQ nesting is required, the IRQ stack size should be
\ ** at least 64*#IRQs bytes, and TASK-SIZE*#IRQs bytes should
\ ** be added to the SWI stack. If IRQ nesting is not required
\ ** the IRQ stack must be at least TASK-SIZE bytes.

\ *P If the EQUate #FIQs is greater than one, FIQ nesting is
\ ** supported with the penalty of greater overhead.
\ ** IF FIQ nesting is required, the FIQ stack size should be
\ ** at least 64*#FIQs bytes, and TASK-SIZE*#FIQs bytes should
\ ** be added to the SWI stack. If FIQ nesting is not required
\ ** the FIQ stack must be at least TASK-SIZE bytes.

ARM-VIC? AT91-VIC = [if]

l: ^IRQ_AIC	\ holds address of AIC
  _AIC ,

PROC IRQ_entry	\ --
\ *G This is the template code for vectored IRQ interrupt handlers.
\ ** It is also used as the spurious interrupt handler.
  stmfd   rsp ! { r0-r12, link }	\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

#IRQs 1 > [if]
  msr   CPSR, _c # I_BIT ARM_MODE_SVC or \ Switch to SVC mode
  stmfd rsp ! { link }			\ save Supervisor LINK register
[then]

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack

l: IRQ_call
  ldr	r4, ^IRQ_AIC			\ get AIC address
  mov	link, pc			\ will return after next instruction
  ldr	pc, [ r4, # AIC_IVR ]		\ call the high level handler
\ *** return point
  ldr	r4, IRQ_template		\ issue end of interrupt to AIC
  str	r3, [ r4, # AIC_EOICR ]		\ value written is irrelevant

  add     rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags

#IRQs 1 > [if]
  ldmfd rsp ! { link }                  \ restore Supervisor LINK register
  msr   CPSR, _c # I_BIT ARM_MODE_IRQ or \ Switch back to IRQ mode
[then]

  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd   rsp ! { r0-r12, link }	\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

$E59FF018 Vector_start $18 + !		\ LDR PC, [ PC, # $18 ]
					\ LDR PC, IRQV
IRQ_entry IRQV !			\ Set entry address


PROC FIQ_entry	\ --
\ *G This is the code for vectored FIQ interrupt handlers.
  stmfd   rsp ! { r0-r7, link }		\ save unbanked registers and LINK
  mrs     r3, SPSR			\ save SPSR for nesting
  stmfd   rsp ! { r3 }

#FIQs 1 > [if]
  msr   CPSR, _c # I_BIT F_BIT or ARM_MODE_SVC or \ Switch to SVC mode
  stmfd   rsp ! { link }		\ save Supervisor LINK register
[then]

  sub     up, rsp, # up-size		\ USER area at top
  sub     psp, up, # sp-guard cells	\ with DATA stack next
  sub     rsp, up, # sp-size		\ reserve space for data stack

l: FIQ_call
  ldr	r4, ^IRQ_AIC			\ get AIC address
  mov	link, pc			\ will return after next instruction
  ldr	pc, [ r4, # AIC_IVR ]		\ call the high level handler
\ *** return point
  ldr	r4, ^IRQ_AIC			\ get AIC address
  str	  r3, [ r4, # AIC_EOICR ]	\ value written is irrelevant

  add     rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags

#FIQs 1 > [if]
  ldmfd   rsp ! { link }		\ restore Supervisor LINK register
  msr   CPSR, _c # I_BIT F_BIT or ARM_MODE_FIQ or \ Switch to FIQ mode
[then]

  ldmfd   rsp ! { r3 }			\ restore SPSR
  msr     SPSR, _c _f r3
  ldmfd   rsp ! { r0-r7 link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

$E59FF018 Vector_start $1C + !		\ LDR PC, [ PC, # $18 ]
					\ LDR PC, IRQV
FIQ_entry FIQV !			\ Set entry address


: InitSPU	\ --
\ *G Initialise the spurious interrupt vector.
\ ** This word is executed as part of the cold chain.
  ['] noop _aic aic_spu + !
;
' InitSPU AtCold

: EnInt		\ int# --
\ *G Enable the requested AIC interrupt.
  1 swap lshift				\ generate bit mask
  _aic aic_iecr + !			\ for enable register.
;

: DisInt	\ int# --
\ *G Disable the requested AIC interrupt.
  1 swap lshift				\ generate bit mask
  _aic aic_idcr + !			\ for disable register.
;

: SetIRQ	\ xt mode int# --
\ *G Set the xt to be the action of IRQ INT# with mode being
\ ** set into the relevant AIC SMR register. Then enable the
\ ** interrupt. The FIQ interrupt is set with INT#=0 for which
\ ** the priority is unused.
\ ** Interrupt numbers may be found in the AIC section of the
\ ** CPU data sheet.
  dup >r DisInt				\ disable this interrupt
  cells _aic + tuck			\ -- isr addr mode addr
  aic_smr0 + !  aic_svr0 + !		\ set mode and isr
  r> EnInt				\ enable this interrupt
;

\ *P The mode value is a combination of priority 0..7 and
\ ** interrupt type as follows. Only the first two should be
\ ** used for internal interrupts.
$000 equ ISR_low	\ -- n
\ *G Low level sensitive.
$020 equ ISR_nedge	\ -- n
\ *G Negative edge triggered.
$040 equ ISR_high	\ -- n
\ *G High level sensitive.
$080 equ ISR_pedge	\ -- n
\ *G Positive edge triggered.

\ *P An example of setting up an interrupt follows.
\ *E : ISRaction  \ -- ; the Forth word
\ **   ...
\ ** ;
\ ** ' ISRaction IRQ: MyIsr  \ -- isr
\ ** #15 equ MyIsr#  \ -- n
\ **
\ ** : SetupISR   \ -- ; initialise
\ **   MyIsr ISR_low 7 or MyIsr#
\ ** ;

[then]


\ ***************************************
\ *S Samsung S3C4510 IRQ and FIQ handlers
\ ***************************************
\ *P This implementation does NOT support nested FIQ interrupts.
\ ** The notes in the Generic IRQ section about interrupt nesting
\ ** apply here.

\ *P If the EQUate #IRQs is greater than one, IRQ nesting is
\ ** supported with the penalty of greater overhead.
\ ** IF IRQ nesting is required, the IRQ stack size should be
\ ** at least 64*#IRQs bytes, and TASK-SIZE*#IRQs bytes should
\ ** be added to the SWI stack. If IRQ nesting is not required
\ ** the IRQ stack must be at least TASK-SIZE bytes.


ARM-VIC? S3C-VIC = [if]		\ if using Samsung controller

cdata
create despatch_table	\ -- addr ; 21 entry interrupt despatch table
\ *G This table holds the addresses (XTs) of the service routines
\ ** for each of the 20 interrupts plus a dummy for false interrupts.
\ ** Equates for the interrupt numbers and their bit positions may
\ ** be found in the file SFRS3C4510.FTH. This table is defined
\ ** in the CDATA section because it is assumed that the code is run
\ ** from RAM. If this is not so, change the CDATA above the definition
\ ** as required, usually to IDATA.
  #21 cells allot&erase
cdata

l: ^_interrupts		\ holds address of interrupt controller regs
  _interrupts ,
l: ^isr_despatch
  despatch_table ,

PROC IRQ_exception	\ IRQ entry code
\ state save
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

#IRQs 1 > [if]
  msr   CPSR, _c # I_BIT ARM_MODE_SVC or \ Switch to SVC mode
  stmfd rsp ! { link }			\ save Supervisor LINK register
[then]

  sub   up, rsp, # up-size		\ USER area at top
  sub   psp, up, # sp-guard cells	\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack

\ clear and despatch
  ldr   r8, ^_interrupts		\ R8 := _interrupt
  ldr   r0, [ r8, # intoffset ]		\ returns R0=[0..54] in steps of 4
  mov   r1, r0 lsr # 2			\ R1 := interrupt bit#
  mov   r2, # 1				\ perform mask = 1 lsl bit#
  mov   r3, r2 lsl r1
  str   r3, [ r8, # intpnd ]		\ write 1 to reset bit, 0 unaffected
  ldr   r4, ^isr_despatch		\ address of despatch table
  mov   link, pc			\ link := PC+8, state restore
  ldr   pc, [ r4 ++ r0 ]		\ execute routine

\ state restore
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags

#IRQs 1 > [if]
  ldmfd rsp ! { link }                  \ restore Supervisor LINK register
  msr   CPSR, _c # I_BIT ARM_MODE_IRQ or \ Switch back to IRQ mode
[then]

  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
IRQ_exception IRQV !

PROC FIQ_exception	\ FIQ entry code
\ state save
  stmfd rsp ! { r0-r7, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub   psp, up, # sp-guard cells	\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
\ clear and despatch
  ldr   r8, ^_interrupts		\ R8 := _interrupt
  ldr   r0, [ r8, # intoffset ]		\ returns R0=[0..54]
  mov   r1, r0 lsr # 2			\ R1 := interrupt bit#
  mov   r2, # 1				\ perform mask = 1 lsl bit#
  mov   r3, r2 lsl r1
  str   r3, [ r8, # intpnd ]		\ write 1 to reset bit, 0 unaffected
  ldr   r4, ^isr_despatch		\ address of despatch table
  mov   link, pc			\ link := PC+8, state restore
  ldr   pc, [ r4 ++ r0 ]		\ execute routine

\ state restore
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags

  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r7, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
FIQ_exception FIQV !

: SetFIQ	\ xt int# --
\ *G Set the XT to be the action of FIQ INT#.
  1 over lshift				\ -- xt int# mask
  _interrupts intmod +			\ interrupt mode register
  dup @  rot or  swap !			\ 1 for FIQ
  cells despatch_table + !		\ place XT in table
;

: SetIRQ	\ xt int# --
\ *G Set the XT to be the action of IRQ INT#.
  1 over lshift				\ -- xt int# ~mask
  _interrupts intmod +			\ interrupt mode register
  dup @  rot invert and  swap !		\ 0 for IRQ
  cells despatch_table + !		\ place XT in table
;

: EnInt		\ int# --
\ *G Enable the requested interrupt.
  1 swap lshift invert			\ generate bit mask
  _interrupts intmsk +
  dup @  rot and  swap !
;

: DisInt	\ int# --
\ *G Disable the requested interrupt.
  1 swap lshift invert			\ generate bit mask
  _interrupts intmsk +
  dup @  rot and  swap !
;

[then]


\ ****************************
\ *S ARM PL190 IRQ and FIQ handlers
\ ****************************
\ *P This section is not a treatise on the ARM PL190 Vectored
\ ** Interrupt Controller. These words provide a fairly basic
\ ** set of tools for using the ARM VIC.
\ ** The notes in the Generic IRQ section about interrupt nesting
\ ** apply here.

\ *P This code requires KERNEL62.FTH and the equate
\ ** COLDCHAIN? must be set non-zero in the control file.
\ ** The VIC addresses should be defined in the SFRxxxx file
\ ** which defines the peripheral address for the silicon.

\ *P If the EQUate #IRQs is greater than one, IRQ nesting is
\ ** supported with the penalty of greater overhead.
\ ** IF IRQ nesting is required, the IRQ stack size should be
\ ** at least 64*#IRQs bytes, and TASK-SIZE*#IRQs bytes should
\ ** be added to the SWI stack. If IRQ nesting is not required
\ ** the IRQ stack must be at least TASK-SIZE bytes.

ARM-VIC? PL190-VIC = [if]

L: ^IRQ_VIC	\ -- addr ; holds VIC address
  _VIC ,

PROC IRQ_entry	\ --
\ *G This is the code for vectored IRQ interrupt handlers.
\ --- save state ---
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }
#IRQs 1 > [if]
  msr   CPSR, _c # I_BIT ARM_MODE_SVC or \ Switch to SVC mode
  stmfd rsp ! { link }			\ save Supervisor LINK register
[then]
\ --- set up Forth ---
  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
\  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
\  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
\  str   rsp, [ up, # r0-offset ]	\ set R0
\ --- call high level handler ---
l: IRQ_call
  ldr	r4, ^IRQ_VIC			\ call vectored routine
  mov   link, pc
  ldr	pc, [ r4, # VICVectAddr ]	\ call the high level handler
\ *** returns here
  ldr	r4, ^IRQ_VIC			\ issue end of interrupt to VIC
  str	r3, [ r4, # VICVectAddr ]	\ value written is irrelevant
\ --- restore from Forth ---
  add     rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
\ --- restore state ---
#IRQs 1 > [if]
  ldmfd rsp ! { link }                  \ restore Supervisor LINK register
  msr   CPSR, _c # I_BIT ARM_MODE_IRQ or \ Switch back to IRQ mode
[then]

  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd   rsp ! { r0-r12, link }	\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

$E59FF018 Vector_start $18 + !		\ LDR PC, [ PC, # $18 ]
					\ LDR PC, IRQV
IRQ_entry IRQV !			\ Set entry address

PROC FIQ_entry	\ --
\ *G This is the template code for FIQ interrupt handlers.
  stmfd   rsp ! { r0-r7, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack

l: FIQ_call
  bl      ' noop			\ call the high level handler

  add     rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags

  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd   rsp ! { r0-r7 link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
FIQ_entry FIQV !			\ set FIQ entry point

interpreter
: SetFIQ	\ xt --
\ *G Set the FIQ interrupt to call the Forth word whose xt is
\ ** given. Use interpretively in the form:
\ *C   ' <word> SetFIQ
  FIQ_call !call
;
target

: InitVIC	\ --
\ *G Initialise the VIC Default interrupt vector.
\ ** This word is executed as part of the cold chain.
  ['] noop _VICDefVectAddr !
;
' InitVIC AtCold

: EnInt		\ src# --
\ *G Enable the requested VIC interrupt source.
  1 swap lshift				\ generate bit mask
  _VICIntEnable !			\ for enable register.
;

: DisInt	\ src# --
\ *G Disable the requested VIC interrupt source.
  1 swap lshift				\ generate bit mask
  _VICIntEnClr !			\ for disable register.
;

: SetIrqAction	\ xt src# slot# --
\ *G Set the xt to be the action of the source interrupt number
\ ** src# (0..31) being set into the corresponding slot
\ ** (slot# = 0..15) where slot# corresponds to priority
\ ** (0 = highest priority). Then enable the
\ ** interrupt.
\ ** Interrupt source numbers may be found in the VIC section
\ ** of the CPU data sheet and the relevant SFRxxxx.FTH file.
  swap dup >r DisInt			\ disable this interrupt
  cells _vic + tuck			\ -- base isr base ; R: -- src#
  VICVectAddr0 + !			\ -- base ; R: -- src# ; set isr entry point
  r> tuck $20 or swap VICVectCntl0 + !	\ set src and enable vector
  EnInt					\ enable interrupt source
;

\ *P An example of setting up an interrupt follows.
\ *E : ISRaction  \ -- ; the Forth word
\ **   ...
\ ** ;
\ ** #15 equ MySrc#  \ -- n ; what triggers it
\ ** #3 equ MySlot#  \ -- n ; what priority (0=highest)
\ **
\ ** : SetupISR   \ -- ; initialise
\ **   ['] ISRaction MySrc# MySlot# SetIrqAction
\ ** ;

[then]


\ ======
\ *> ###
\ ======

