\ Atmel AT91 vectored exception (interrupt) handlers

((
Copyright (c) 2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20040127 MPE001 Updated SWI, PAbort and DAbort handlers.
		Fixed MSRs to SPSR to set Cand F fields.
))


\ ==========
\ *! intAT91
\ ==========
\ *T Atmel AT91 interrupt handlers
\ *P The IRQ handlers all share a common "stack of stacks". On entry to
\ ** the interrupt handler, the next stack is allocated. It is assumed
\ ** that an interrupt handler may not itself be interrupted by an interrupt
\ ** of the same level. If this will happen, the return stack pointers
\ ** of each banked interrupt must be pre-allocated and set up by the
\ ** interrupt initialisation routine INIT-INT-USER which is called
\ ** by the start-up code. This will require each interrupt routine
\ ** to have its own stack area, and for the user areas to be allocated
\ ** dynamically. The start up code is responsible for initialising
\ ** R13 in each ARM register space.

\ *P The number of stacks allocated for IRQs is defined by the equate #IRQs
\ ** in the control file.

\ *P Note that each interrupt has its own USER area, but that
\ ** NO USER VARIABLES ARE INITIALISED.


1 equ test-isr?	\ -- n ; true to compile test code
\ *G Set this equate true to compile the test code.

true equ AIC?	\ -- n ; true if using AIC
\ *G Set this equate true if using the AIC for the FIQ
\ ** and IRQ interrupts.


only forth definitions
decimal

: init-int-user \ -- ; initialise interrupt user area
;


\ ***********************
\ *S Interrupt management
\ ***********************

[undefined] EI [if]
code EI		\ -- ; enable interrupts
\ *G Global enable interrupts, both FIQ and IRQ.
  mrs  r0, cpsr
  bic  r0, r0, # $C0			\ clear I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] DI [if]
code DI         \ -- ; disable interrupts
\ *G Global disable interrupts, both FIQ and IRQ.
  mrs  r0, cpsr
  orr  r0, r0, # $C0			\ set I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] SAVE-INT [if]
code SAVE-INT	\ -- x ; return interrupt status and disable interrupts

\ *G Geturn interrupt status and then disable interrupts.
  stmfd psp ! { tos }			\ preserve TOS
  mrs   r0, cpsr			\ get status register
  mov   tos, r0
  orr   r0, r0, # $C0			\ set I and F bits
  msr   cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] RESTORE-INT [if]
code RESTORE-INT	\ x -- ; restore state returned by SAVE-INT
\ *G Restore state returned by SAVE-INT.
  msr   cpsr _c  tos
  ldmfd psp ! { tos }
  next,
end-code
[then]

[undefined] [I [if]
code [I		\ R: -- x ; preserve I/F status on return stack, disable ints
\ *G Preserve I/F status on return stack, disable interrupts.
\ ** The state is restored by I].
  mrs   r0, cpsr			\ get CPSR
  stmfd rsp ! { r0 }			\ save
  orr   r0, r0, # $C0			\ set I and F bits
  msr   cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] I] [if]
code I]		\ R: x -- ; restore int. status from r. stack
\ *G Restore interrupt status saved by [I from the return stack.
  ldmfd rsp ! { r0 }
  msr   cpsr _c r0
  next,
end-code
[then]


\ **************
\ *S SWI handler
\ **************
\ *P This SWI handler assumes that we may already be in supervisor mode,
\ ** and that the RSP must be preserved. A new RSP stack is allocated, and
\ ** the original RSP, R4..R12 and LINK are saved on the new RSP stack.
\ ** New data stack and USER area are then allocated, parameters R0..R2
\ ** and the SWI# are put on the new data stack, and the handler is executed.
\ ** On return, registers RSP and R4..12 are restored. R2 and R3 will be
\ ** destroyed. Return data (if any) may be placed in R0 and R1.

\ ** By default, on entry to SWI_HANDLER, TOS contains the SWI#, and the
\ ** next three items contain R0..2. This is done so that the standard
\ ** ARM SWI calls can be emulated. Although it is not strictly necessary
\ ** to remove this data, a canonical handler will have the stack effect:
\ *E   SWI_ACTION	\ r2 r1 r0 swi# -- r1 r0


defer SWI_handler	\ r2 r1 r0 swi# -- r1 r0 ; default action
\ *G Default action of SWI interrupt
  assign 2drop to-do SWI_handler	\ set to do nothing

PROC SWI_exception	\ -- ; parameter passing in registers
\ *G Assumes RSP=R13, R0-2 are parameters, R2 will be destroyed.
\ ** Return parameters are in R0-R1, R3..R13 will be preserved. R13
\ ** must have been set up by the startup code.
  stmfd rsp ! { r3-r12, link }		\ save unbanked registers + LINK (11 regs)

  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r3 }
  mov	r3, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer
  stmfd r13 ! { r3 }			\ save orignal value on NEW return stack

  ldr tos, [ link, # -4 ]		\ Get SWI instruction
  bic tos, tos, # $FF000000		\ mask out instruction bits to get number
  stmfd psp ! { r0-r2 }			\ tos=SWI#, r0..2=parameters
  bl    ' SWI_handler			\ run required word
  mov   r0, tos				\ r0 is in TOS
  ldmfd psp ! { r1 }			\ r1 is in NOS

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd r13 ! { r3-r12, link }		\ restore unbanked registers and LINK

  mov .s  pc, link			\ return from exception
end-code

SWI_exception SWIV !


\ -------------
\ SWI test code
\ -------------

test-isr? [if]
: testswi	\ r2 r1 r0 swi# -- 0 1
\ *G An example SWI handler.
  decimal  console opvec !  console ipvec !
  cr ." SWI exception"
  cr ." SWI number = " .
  cr ." args = " . . .
  cr ." returning -- 0 1" 0 1
;
assign testswi to-do SWI_handler

code run-swi0	\ r2 r1 r0 -- r1 r0
\ *G Test execution of SWI 0.
  stmfd rsp ! { link }
  mov   r0, tos
  ldmfd psp ! { r1, r2 }
  swi   # 0
  mov   tos, r0
  stmfd psp ! { r1 }
  ldmfd rsp ! { pc }
end-code
[then]


\ ***************************
\ *S Other exception handlers
\ ***************************
\ *P All other exception handlers preserve and restore
\ ** all registers. Some words are passed a pointer to the
\ ** instruction or data that caused the exception.

\ ================================
\ *N Undefined instruction handler
\ ================================

defer Undef_handler	\ ^ins --
\ *G Is passed a pointer to the offending instruction.
  assign drop to-do Undef_handler

PROC Undef_exception
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to undefined instruction
  bl    ' Undef_handler

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, _c _f r0
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK

  mov .s pc, link			\ return from exception, after bad instruction
end-code

Undef_exception UndefV !


\ ---------------
\ Undef test code
\ ---------------

test-isr? [if]
: testundef	\ ^ins -- ; handle undefined instruction
\ *G Test handler for Undefined Instructions.
  hex  console opvec !  console ipvec !
  cr ." Undef exception at " dup .lword
  cr ." Instruction        " @ .lword
;
assign testundef to-do Undef_handler

: run-undef	\ --
\ *G Causes an Undefined Instruction exception.
  cr ." before"
  [asm
    mrc  7 0 r0, cr0, cr0, 0		\ CP7 doesn't exist, so should fault
  asm]
  cr ." after"
;
[then]


\ =========================
\ *N Prefetch Abort handler
\ =========================

defer PAbort_handler	\ ^ins -- fixed?
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do PAbort_handler

PROC PAbort_exception
\ The PABORT exception code calls PAbort_handler above.
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' PAbort_handler

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, _c _f r0

  cmp   tos, # 0			\ check return value
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .ne .s  pc, link, # 4		\ return from exception, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

PAbort_exception PAbortV !

test-isr? [if]
: testpabort	\ ^ins -- fixed?
\ *G Test PAbort handler.
  decimal  console opvec !  console ipvec !
  cr ." PAbort exception at " dup .lword
  cr ." Instruction         " dup @ .lword
  drop 0				\ skip instruction
;
assign testpabort to-do PAbort_handler

((
ArmArch5? [if]
: run-pabort	\ --
\ *G Run the test PAbort handler.
  [asm
    bkpt # $55AA	
    mov r0, r0
  asm]
;
[then]
))
[then]


\ =====================
\ *N Data Abort handler
\ =====================

defer DAbort_handler	\ ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do DAbort_handler

PROC DAbort_exception
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 8			\ point to bad instruction
  bl    ' DAbort_handler

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, _c _f r0

  cmp   tos, # 0
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 8		\ return, rerun instruction
  sub .s      pc, link, # 4		\ return, skip instruction
end-code

DAbort_exception DAbortV !

variable DAup
variable DAsp
variable DArp

test-isr? [if]
: testdabort	\ ^ins -- fixed?
\ *G Test DAbort handler.
  self DAup !  sp@ DAsp !  rP@ DArp !
  decimal  console opvec !  console ipvec !
  cr ." DAbort exception at " dup .lword
  cr ." Instruction         " dup @ .lword
  drop 0				\ skip instruction
;
assign testdabort to-do DAbort_handler

: run-dabort   $04000000 @  ;		\ hardware specific
\ *G Run test PAbort handler.
[then]



\ ============================================
\ *N Unused (26 bit address exception) handler
\ ============================================
\ *P This exception is only used for systems which
\ ** support the 26 bit PC mode of early ARM cores.

defer Unused_handler	\ ^ins -- fixed?
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do Unused_handler

PROC Unused_exception
\ *G The UNUSED exception handler calls UNUSED_handler above.
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' Unused_handler

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, _c _f r0

  cmp   tos, # 0			\ check return result
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .ne .s  pc, link, # 4		\ return from exception, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

Unused_exception UnusedV !

test-isr? [if]
: testunused	\ --
\ *G Test handler for UNUSED exception
  decimal  console opvec !  console ipvec !
  cr ." Unused exception"
;
assign testunused to-do Unused_handler
[then]


\ **************
\ *S IRQ handler
\ **************

AIC? 0= [if]

variable next-irq	\ -- addr
\ *G Holds pointer to next ISR to run.
\ ** ISRs are added to this chain.
l: ^next-irq
  next-irq ,

PROC IRQ_exception	\ IRQ entry code
\ *G The IRQ handler entry point which executes the chain.
\ ** The USER variables S0 and R0 are initialised.
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  ldr   r0, ^next-irq			\ get address of variable NEXT-ISR
  ldr	pc, [ r0 ]			\ branch to contents
end-code

proc IRQ_reti		\ IRQ exit code
\ *G This code cleans up after the IRQ and is the first item added
\ ** to the IRQ chain.
  ldmfd r13 { r13 }			\ get original R13 back
  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, _c _f r0
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
IRQ_reti next-irq !			\ no handlers defined yet
IRQ_exception IRQV !

interpreter
: br24,		\ xt opcode --
  swap
  chere 8 + -				\ subtract HERE (pc in effect)
  4/ $00FFFFFF and                      \ shift addr and mask
  or ,					\ make B/BL and lay down
;

: add-isr	\ xt chain -- ; ' <name> <chain_var> ADD-ISR adds name to the ISR handler list
  cdata align
  dup @ >r  chere swap !		\ get current chain end, and update
  $EB000000 br24,			\ BL xt
  r> $EA000000 br24,			\ B previous
;

: add-irq	\ xt -- ; ' <name> ADD-IRQ adds name to the IRQ handler list
  next-irq add-isr
;
target
heads? [if]
: br24,		\ xt opcode --
\ *G Given an opcode (B or BL) and an execution address,
\ ** compiles a branch or call to it.
  swap
  here 8 + -				\ subtract HERE (pc in effect)
  4/ $00FFFFFF and                      \ shift addr and mask
  or ,					\ make B/BL and lay down
;

: add-isr	\ xt chain -- ; ' <name> <chain_var> ADD-ISR adds name to the ISR handler list
\ *G An action is added to the IRQ or FIQ chain by a phrase of the form:
\ *C   ' <name> <chain_var> ADD-ISR
  align					\ force cell boundary
  dup @ >r  here swap !			\ get current chain end, and update
  $EB000000 br24,			\ BL xt
  r> $EA000000 br24,			\ B previous
;

: add-irq	\ xt -- ; ' <name> ADD-IRQ adds name to the IRQ handler list
\ *G An action is added to the IRQ chain by a phrase of the form:
\ *C   ' <name> ADD-IRQ
  next-irq add-isr
;
[then]

[then]


\ ***********
\ FIQ handler
\ ***********

AIC? 0= [if]

variable next-fiq	\ -- addr
\ *G Holds pointer to next ISR to run. FIQ ISRs are
\ ** added to this chain
l: ^next-fiq
  next-fiq ,

PROC FIQ_exception	\ FIQ entry code
\ *G FIQ entry code.
  stmfd r13 ! { r0-r7, link }		\ save unbanked registers and LINK
  mov	r1, r13				\ preserve R13 for later
  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer
  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  ldr   r0, ^next-fiq			\ get address of variable NEXT-FIQ
  ldr	pc, [ r0 ]			\ branch to contents
end-code

proc FIQ_reti		\ FIQ exit code
\ *G FIQ exit code.
  ldmfd r13 { r13 }			\ get original R13 back
  ldmfd r13 ! { r0-r7, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
FIQ_reti next-fiq !			\ no handlers defined yet
FIQ_exception FIQV !			\ set vector

interpreter
: add-fiq	\ xt -- ; ' <name> ADD-FIQ adds name to the FIQ handler list
  next-fiq add-isr
;
target

heads? [if]
: add-fiq	\ xt -- ; ' <name> ADD-FIQ adds name to the FIQ handler list
\ *G An action is added to the FIQ chain by a phrase of the form:
\ *C   ' <name> ADD-FIQ
  next-fiq add-isr
;
[then]

[then]


\ ======
\ *> ###
\ ======

