\ Samsung S3C4510 vectored exception (interrupt) handlers

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20020711 SFP000 First release
))

\ ==========
\ *! intS3C4510
\ ==========
\ *T S3C4510 interrupt handlers
\ *P The IRQ handlers all share a common "stack of stacks". On entry to
\ ** the interrupt handler, the next stack is allocated. It is assumed
\ ** that an interrupt handler may not itself be interrupted by an interrupt
\ ** of the same level. If this will happen, the return stack pointers
\ ** of each banked interrupt must be pre-allocated and set up by the
\ ** interrupt initialisation routine INIT-INT-USER which is called
\ ** by the start-up code. This will require each interrupt routine
\ ** to have its own stack area, and for the user areas to be allocated
\ ** dynamically. The start up code is responsible for initialising
\ ** R13 in each ARM register space.

\ *P The number of stacks allocated for IRQs is defined by the equate #IRQs
\ ** in the control file.

\ *P Note that each interrupt has its own USER area, but that
\ ** NO USER VARIABLES ARE INITIALISED.


0 equ test-isr?	\ -- n ; true to compile test code
\ *G Set this equate true to compile the test code.

true equ SIC?	\ -- n ; true if using SIC (Samsung Interrupt Controller)
\ *G Set this equate true if using the SIC for the FIQ
\ ** and IRQ interrupts.


only forth definitions
decimal

: init-int-user \ -- ; initialise interrupt user area
;


\ ***********************
\ *S Interrupt management
\ ***********************

[undefined] EI [if]
code EI		\ -- 
\ *G Enable interrupts within the ARM core.
  mrs  r0, cpsr
  bic  r0, r0, # $C0			\ clear I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] DI [if]
code DI         \ --
\ *G Disable interrupts within the ARM core.
  mrs  r0, cpsr
  orr  r0, r0, # $C0			\ set I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] SAVE-INT [if]
code SAVE-INT	\ -- x
\ *G Return interrupt status and disable ARM core interrupts.
  stmfd psp ! { tos }			\ preserve TOS
  mrs   r0, cpsr			\ get status register
  mov   tos, r0
  orr   r0, r0, # $C0			\ set I and F bits
  msr   cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] RESTORE-INT [if]
code RESTORE-INT	\ x --
\ *G Restore state returned by SAVE-INT.
  msr   cpsr _c  tos
  ldmfd psp ! { tos }
  next,
end-code
[then]

[undefined] [I [if]
code [I		\ R: -- x
\ *G Preserve ARM I/F interrupt status on return stack, and 
\ ** disable interrupts.
  mrs   r0, cpsr			\ get CPSR
  stmfd rsp ! { r0 }			\ save
  orr   r0, r0, # $C0			\ set I and F bits
  msr   cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] I] [if]
code I]		\ R: x --
\ *G Restore interrupt status from the return stack.
  ldmfd rsp ! { r0 }
  msr   cpsr _c r0
  next,
end-code
[then]


\ **************
\ *S SWI handler
\ **************
\ *P This SWI handler assumes that we may already be in supervisor mode,
\ ** and that the RSP must be preserved. A new RSP stack is allocated, and
\ ** the original RSP, R4..R12 and LINK are saved on the new RSP stack.
\ ** New data stack and USER area are then allocated, parameters R0..R2
\ ** and the SWI# are put on the new data stack, and the handler is executed.
\ ** On return, registers RSP and R4..12 are restored. R2 and R3 will be
\ ** destroyed. Return data (if any) may be placed in R0 and R1.

\ ** By default, on entry to SWI_HANDLER, TOS contains the SWI#, and the
\ ** next three items contain R0..2. This is done so that the standard
\ ** ARM SWI calls can be emulated. Although it is not strictly necessary
\ ** to remove this data, a canonical handler will have the stack effect:
\ *E   SWI_ACTION	\ r2 r1 r0 swi# -- r1 r0


defer SWI_handler	\ r2 r1 r0 swi# -- r1' r0' ; default action
\ *G A DEFERred word used as the action of the SWI interrupt.
  assign 2drop to-do SWI_handler	\ set to do nothing

PROC SWI_exception
\ assumes RSP=R13, R0-2 are parameters.X
\ Return parameters are in R0-R1, R2 destroyed

  stmfd rsp ! { r3-r12, link }		\ save unbanked registers + LINK (11 regs)
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  mov   psp, up				\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
  ldr tos, [ link, # -4 ]		\ Get SWI instruction
  bic tos, tos, # $FF000000		\ mask out instruction bits to get number
  stmfd psp ! { r0-r2 }			\ tos=SWI#, r0..2=parameters
  bl    ' SWI_handler			\ run required word
  mov   r0, tos				\ get return data into registers
  ldmfd psp ! { r1 }

  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r3-r12, link }		\ restore registers

  mov .s  pc, link			\ return from exception
end-code

SWI_exception SWIV !


\ -------------
\ SWI test code
\ -------------

test-isr? [if]
: testswi	\ r2 r1 r0 -- r1' r0' ; executes swi0
  decimal  console opvec !  console ipvec !
  hex
  cr ." In SWI: "
  ." RSP = " rp@ u.  ."  PSP = " sp@ u.  ."  UP = " up@ u.
  cr ." SWI exception"
  cr ." SWI number = " .
  cr ." args = " . . .
  cr
  10 11
;
assign testswi to-do SWI_handler

code (run-swi0)	\ r2 r1 r0 -- ; executes swi0
  ldmfd psp ! { r1-r2 }		\ massage parameters
  mov   r0, tos
  stmfd rsp ! { link }		\ nest return address
  swi   # 0
  mov   tos, r0			\ set return data
  stmfd psp ! { r1 }
  ldmfd rsp ! { pc }		\ unnest
end-code

: run-swi0	\ r2 r1 r0 -- ; executes swi0
  base @ >r
  hex
  cr ." Before: "
  ." RSP = " rp@ u.  ."  PSP = " sp@ u.  ."  UP = " up@ u.
  (run-swi0)
  decimal
  cr ." Returns: " . .
  hex
  cr ." After : "
  ." RSP = " rp@ u.  ."  PSP = " sp@ u.  ."  UP = " up@ u.
  r> base !
;
[then]


\ ***************************
\ *S Other interrupt handlers
\ ***************************
\ *P All other interrupt handlers preserve and restore
\ ** all registers. Some are passed a pointer to the
\ ** instruction or data that caused the exception.


\ ********************************
\ *N Undefined instruction handler
\ ********************************

defer Undef_handler	\ ^ins --
\ *G Is passed a pointer to the offending instruction.
  assign drop to-do Undef_handler

PROC Undef_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  mov   psp, up				\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to undefined instruction
  bl    ' Undef_handler

  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  mov .s pc, link			\ return from exception
end-code

Undef_exception UndefV !


\ ---------------
\ Undef test code
\ ---------------

test-isr? [if]
: testundef	\ ^ins -- ; handle undefined instruction
  hex  console opvec !  console ipvec !
  cr ." Undef exception at " dup .lword
  cr ." Instruction        " @ .lword
;
assign testundef to-do Undef_handler

: run-undef	\ --
  cr ." before"
  [asm
    mrc  7 0 r0, cr0, cr0, 0		\ CP7 doesn't exist, so should fault
  asm]
  cr ." after"
;
[then]


\ *************************
\ *N Prefetch Abort handler
\ *************************

defer PAbort_handler	\ ^ins -- fixed?
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do PAbort_handler

PROC PAbort_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  mov   psp, up				\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' PAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .ne .s  pc, link, # 4		\ return from exception, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

PAbort_exception PAbortV !

test-isr? [if]
: testpabort	\ --
  decimal  console opvec !  console ipvec !
  cr ." PAbort exception"
;
assign testpabort to-do PAbort_handler
[then]


\ *********************
\ *N Data Abort handler
\ *********************

defer DAbort_handler	\ ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do DAbort_handler

PROC DAbort_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  mov   psp, up				\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' DAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .ne .s  pc, link, # 4		\ return from exception, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

DAbort_exception DAbortV !

test-isr? [if]
: testdabort
  decimal  console opvec !  console ipvec !
  cr ." DAbort exception"
;
assign testdabort to-do DAbort_handler
[then]


\ **********************************************
\ *N Reserved (26 bit address exception) handler
\ **********************************************

defer Unused_handler	\ ^ins -- fixed?
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do Unused_handler

PROC Unused_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  mov   psp, up				\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' Unused_handler

  cmp   tos, # 0			\ check return result
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .ne .s  pc, link, # 4		\ return from exception, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

Unused_exception UnusedV !

test-isr? [if]
: testunused
  decimal  console opvec !  console ipvec !
  cr ." Unused exception"
;
assign testunused to-do Unused_handler
[then]


\ **************
\ *N IRQ and FIQ handlers
\ **************
\ *P This implementation permits interrupt nesting and uses
\ ** the same despatch handler for both IRQ and FIQ because
\ ** the INTOFFSET register is sensitive to the interrupt mode.

SIC? [if]		\ if using Samsung controller

cdata
create despatch_table	\ -- addr ; 21 entry interrupt despatch table
\ *G This table holds the addresses (XTs) of the service routines
\ ** for each of the 20 interrupts plus a dummy for false interrupts.
\ ** Equates for the interrupt numbers and their bit positions may
\ ** be found in the file SFRS3C4510.FTH. This table is defined
\ ** in the CDATA section because it is assumed that the code is run
\ ** from RAM. If this is not so, change the CDATA above the definition
\ ** as required, usually to IDATA.
  #21 cells allot&erase
cdata

l: ^_interrupts		\ holds address of interrupt controller regs
  _interrupts ,
l: ^isr_despatch
  despatch_table ,

PROC IRQ_exception	\ IRQ/FIQ entry code
\ state save
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  mov   psp, up				\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
\ clear and despatch
  ldr   r8, ^_interrupts		\ R8 := _interrupt
  ldr   r0, [ r8, # intoffset ]		\ returns R0=[0..54]
  mov   r1, r0 lsr # 2			\ R1 := interrupt bit#
  mov   r2, # 1				\ perform mask = 1 lsl bit#
  mov   r3, r2 lsl r1
  str   r3, [ r8, # intpnd ]		\ write 1 to reset bit, 0 unaffected
  ldr   r4, ^isr_despatch		\ address of despatch table
  mov   link, pc			\ link := PC+8, state restore
  ldr   pc, [ r4 ++ r0 ]		\ execute routine

\ state restore
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
IRQ_exception IRQV !
IRQ_exception FIQV !

: SetFIQ	\ xt int# --
\ *G Set the XT to be the action of FIQ INT#.
  1 over lshift				\ -- xt int# mask
  _interrupts intmod +			\ interrupt mode register
  dup @  rot or  swap !			\ 1 for FIQ
  cells despatch_table + !		\ place XT in table
;

: SetIRQ	\ xt int# --
\ *G Set the XT to be the action of IRQ INT#.
  1 over lshift				\ -- xt int# ~mask
  _interrupts intmod +			\ interrupt mode register
  dup @  rot invert and  swap !		\ 0 for IRQ
  cells despatch_table + !		\ place XT in table
;

: EnInt		\ int# --
\ *G Enable the requested interrupt.
  1 swap lshift invert			\ generate bit mask
  _interrupts intmsk +
  dup @  rot and  swap !
;

: DisInt	\ int# --
\ *G Disable the requested interrupt.
  1 swap lshift invert			\ generate bit mask
  _interrupts intmsk +
  dup @  rot and  swap !
;

[else]

variable next-irq	\ -- addr ; holds pointer to next ISR to run
			\ ISRs are added to this chain
l: ^next-irq
  next-irq ,

PROC IRQ_exception	\ IRQ entry code
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  mov   psp, up				\ with DATA stack next
  sub   rsp, up, # sp-size		\ reserve space for data stack
  					\ leaving the return stack lowest
  ldr   r0, ^next-irq			\ get address of variable NEXT-ISR
  ldr	pc, [ r0 ]			\ branch to contents
end-code

proc IRQ_reti		\ IRQ exit code
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
IRQ_reti next-irq !			\ no handlers defined yet
IRQ_exception IRQV !

interpreter
: br24,		\ xt opcode --
  swap
  chere 8 + -				\ subtract HERE (pc in effect)
  4/ $00FFFFFF and                      \ shift addr and mask
  or ,					\ make B/BL and lay down
;

: add-isr	\ xt chain -- ; ' <name> <chain_var> ADD-ISR adds name to the ISR handler list
  cdata align
  dup @ >r  chere swap !		\ get current chain end, and update
  $EB000000 br24,			\ BL xt
  r> $EA000000 br24,			\ B previous
;

: add-irq	\ xt -- ; ' <name> ADD-IRQ adds name to the IRQ handler list
  next-irq add-isr
;
target
heads? [if]
: br24,		\ xt opcode --
  swap
  here 8 + -				\ subtract HERE (pc in effect)
  4/ $00FFFFFF and                      \ shift addr and mask
  or ,					\ make B/BL and lay down
;

: add-isr	\ xt chain -- ; ' <name> <chain_var> ADD-ISR adds name to the ISR handler list
  align					\ force cell boundary
  dup @ >r  here swap !			\ get current chain end, and update
  $EB000000 br24,			\ BL xt
  r> $EA000000 br24,			\ B previous
;

: add-irq	\ xt -- ; ' <name> ADD-IRQ adds name to the IRQ handler list
  next-irq add-isr
;
[then]

[then]


\ ***********
\ FIQ handler
\ ***********

SIC? 0= [if]

variable next-fiq	\ -- addr ; holds pointer to next ISR to run
			\ ISRs are added to this chain
l: ^next-fiq
  next-fiq ,

PROC FIQ_exception	\ FIQ entry code
  stmfd r13 ! { r0-r7, link }		\ save unbanked registers and LINK
  mov	r1, r13				\ preserve R13 for later
  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer
  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  ldr   r0, ^next-fiq			\ get address of variable NEXT-FIQ
  ldr	pc, [ r0 ]			\ branch to contents
end-code

proc FIQ_reti		\ FIQ exit code
  ldmfd r13 { r13 }			\ get original R13 back
  ldmfd r13 ! { r0-r7, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
FIQ_reti next-fiq !			\ no handlers defined yet
FIQ_exception FIQV !			\ set vector

interpreter
: add-fiq	\ xt -- ; ' <name> ADD-FIQ adds name to the FIQ handler list
  next-fiq add-isr
;
target

heads? [if]
: add-fiq	\ xt -- ; ' <name> ADD-FIQ adds name to the FIQ handler list
  next-fiq add-isr
;
[then]

[then]


