\ ARM exception (interrupt) handlers

((
Copyright (c) 2002, 2003, 2004, 2005, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20070703 SFP010 Split Dual PL190 from INTARM3.FTH.
20050930 SFP009 Split FIQ and IRQ enable/disable.
20050223 SFP008 Moved save of SPSR into conditional code for #IRQs>1.
20040729 SFP007 Changed display to show return stack before data
		stack.
20040205 MPE006 Added ARM PL190 VIC as in Philips LPC2xxx family.
		Added higher diagnostic actions for DAbort PAbort and Undef.
		Made reserved exception optional.
20040127 MPE005 Updated SWI, PAbort and DAbort handlers.
		Fixed MSRs to SPSR to set Cand F fields.
20040115 MPE004 Major overhaul of interrupt nesting.
20030910 SFP003 Incorporated AT91 and Samsung
		vectored interrupt controllers.
20021009 SFP002 Initialised S0 and R0.
20020430 SFP001 Simplified entry/exit code.
		Added better nesting support.
))

\ =========
\ *! IntSTR91x
\ *T STR91x Interrupt handlers
\ =========
\ *P The file ARM\IntSTR91x.FTH contains ARM interrupt handlers
\ ** for systems with dual PL190 VIC units.
\ ** IntSTR91x.FTH requires ARM\ARMDEF.FTH to be compiled before
\ ** the SFRxxxx file for your particular CPU. The file ARM/STACKDEF.FTH
\ ** provides default main task and stack layouts and should be
\ ** compiled from the control file or copied into your control file.
\ ** Default stack initialisation code is provided by the assembler
\ ** routine INITSTACKS in ARM\INITSTACKS.FTH. This routine can be
\ ** referenced from your initialisation code.

\ *P Each exception (interrupt) type requires a predefined stack
\ ** frame on which the interrupt handler builds the Forth stacks
\ ** and USER area. The system initialisation code must preset the
\ ** banked R13 registers to the top of the frames. This code
\ ** requires the Forth return stack pointer to be R13.

\ *P In order to support exception nesting the equate #IRQs in the
\ ** control file must be set to the maximum number of nestings
\ ** required for ALL modes. When #IRQs is greater than 1 (to indicate
\ ** IRQ nesting), the exception handler entry and exit code is
\ ** extended to handle all exceptions in SVC mode. This avoids
\ ** corruption of R14 (the LINK register) when nesting occurs.
\ ** The equate #IRQs is used to calculate the size and layout
\ ** of the IRQ and SVC mode stacks along with the equates *\fo{#SWIs}
\ ** and *\fo{#FIQs} in the control file.

\ *P Note that each interrupt has its own *\fo{USER} area, but that
\ ** NO USER VARIABLES ARE INITIALISED except for *\fo{S0} and *\fo{R0} in
\ ** SWI handlers.

\ *P Note that it is implicit throughout the code that the three
\ ** system registers UP, PSP, RSP are always set so that:
\ *C   UP > PSP > RSP
\ *P Because of this layout, data stack underflows may corrupt
\ ** the first part of the *\fo{USER} area. You have been warned.
\ ** During testing, it may be as well to set the equate
\ ** *\fo{SP-GUARD} to 2 or 3 in your control file to leave a few
\ ** guard cells on the data stack.

only forth definitions
decimal


\ *P The following equates define what code is compiled. If they
\ ** are defined before IntSTR91x.FTH the default values below will
\ ** be overridden.

[undefined] test-isr? [if]
1 equ test-isr?	\ -- n ; non-zero to compile test code
\ *G Non-zero to compile test and diagnostic code.
[then]

[undefined] SimpleAborts? [if]
0 equ SimpleAborts?	\ -- n
\ *G If this equate is non-zero, simple DAbort, PAbort, and Undef
\ ** handlers are installed.
\ ** The simple high level action receives only a pointer to the instruction
\ ** that caused the abort and issues a simple message.
\ ** If this equate is zero, a more complex DAbort handler is installed.
\ ** The complex high level action receives a pointer to a status
\ ** frame and a pointer to the instruction that caused the abort.
\ ** The complex handlers are compiled if *\fo{TEST-ISR?} is nonzero.
\ ** They perform a register dump and a return stack trace.
\ ** The additional code space required for complex handlers is
\ ** about 2300 bytes.
[then]

[undefined] Reserved_ISR? [if]
0 equ Reserved_ISR?	\ -- n
\ *G When non-zero, a handler will be installed for the RESERVED
\ ** exception (vector at $0000:0014). This vector was used by the
\ ** 26 bit architecture, but is now reserved for future expansion.
\ ** It is used for other purposes by some CPUs, e.g. Philips LPC2xxx.
[then]

interpreter
: !call		\ xt addr --
\ *G An *\fo{INTERPRETER} word to create a call opcode to xt at addr.
  tuck					\ -- addr xt addr
  8 +  -				\ -- addr offset
  2 rshift $00FFFFFF and $EB000000 or 	\ -- addr BLins
  swap !				\ set call instruction
;
target


\ ***********************
\ *S Interrupt management
\ ***********************

\ SFP009...
code EFI	\ -- ; enable FIQ interrupt
\ *G Global enable FIQ interrupt.
  mrs  r0, cpsr
  bic  r0, r0, # F_BIT			\ clear F bit
  msr  cpsr _c r0			\ modify control bits
  next,
end-code

code DFI	\ -- ; disable interrupts
\ *G Global disable FIQ interrupt.
  mrs  r0, cpsr
  orr  r0, r0, # F_BIT			\ set F bit
  msr  cpsr _c r0			\ modify control bits
  next,
end-code
\ ...SFP009

[undefined] EI [if]
code EI		\ -- ; enable interrupts
\ *G Global enable IRQ.
  mrs  r0, cpsr
  bic  r0, r0, # I_BIT			\ clear I bit ; SFP009
  msr  cpsr _c r0			\ modify control bits
  next,
end-code

code DI         \ -- ; disable interrupts
\ *G Global disable IRQ.
  mrs  r0, cpsr
  orr  r0, r0, # I_BIT			\ set I bit ; SFP009
  msr  cpsr _c r0			\ modify control bits
  next,
end-code

code [I		\ R: -- x ; preserve I/F status on return stack, disable ints
\ *G Preserve I/F status on return stack, disable IRQ.
\ ** The state is restored by I].
  mrs   r0, cpsr			\ get CPSR
  stmfd rsp ! { r0 }			\ save
  orr   r0, r0, # I_BIT			\ set I bit ; SFP009
  msr   cpsr _c r0			\ modify control bits
  next,
end-code

code I]		\ R: x -- ; restore int. status from r. stack
\ *G Restore interrupt status saved by [I from the return stack.
  ldmfd rsp ! { r0 }
  msr   cpsr _c r0
  next,
end-code
[then]


\ **************
\ *S SWI handler
\ **************
\ *P The SWI handler is only compiled if the equate #SWIs is non-zero.

\ *P The SWI handler assumes that we may already be in supervisor mode,
\ ** and that the RSP must be preserved. A new RSP stack is allocated, and
\ ** the original RSP, R4..R12 and LINK are saved on the new RSP stack.
\ ** New data stack and USER area are then allocated, parameters R0..R2
\ ** and the SWI# are put on the new data stack, and the handler is executed.
\ ** On return, registers RSP and R3..12 are restored. R2 will be
\ ** destroyed. Return data (if any) may be placed in R0 and R1.

\ *P By default, on entry to *\fo{SWI_HANDLER}, TOS contains the SWI#, and the
\ ** next three items contain R0..2. This is done so that the standard
\ ** ARM SWI calls can be emulated. Although it is not strictly necessary
\ ** to remove this data, a canonical handler will have the stack effect:
\ *C   SWI_ACTION	\ r2 r1 r0 swi# -- r1' r0'

#SWIs [if]

defer SWI_handler	\ r2 r1 r0 swi# -- r1' r0' ; default action
\ *G The default action is *\fo{2DROP}. Assign your own action
\ ** to this word.
\ *C   assign mySwi to-do SWI_handler
  assign 2drop to-do SWI_handler	\ set to do nothing

PROC SWI_exception	\ --
\ *G The actual SWI handler which calls SWI_handler above.
\ ** It assumes RSP=R13, R0-2 are parameters, R2 will be destroyed,
\ ** Return parameters are in R0-R1, R3..R13 will be preserved
  stmfd rsp ! { r3-r12, link }		\ save unbanked registers + LINK (11 regs)
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0

  ldr tos, [ link, # -4 ]		\ Get SWI instruction
  bic tos, tos, # $FF000000		\ mask out instruction bits to get number
  stmfd psp ! { r0-r2 }			\ tos=SWI#, r0..2=parameters
  bl    ' SWI_handler			\ run required word
  mov   r0, tos				\ r0 is in TOS
  ldmfd psp ! { r1 }			\ r1 is in NOS

  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r3-r12, link }		\ restore registers

  mov .s  pc, link			\ return from exception
end-code

SWI_exception SWIV !


\ -------------
\ SWI test code
\ -------------

test-isr? [if]
: testswi	\ r2 r1 r0 -- r1' r0' ; executes swi0
\ *G An example SWI handler.
  decimal  console opvec !  console ipvec !
  cr ." SWI exception"
  cr ." SWI number = " .
  cr ." args = " . . .
  cr ." returning -- 0 1" 0 1
;
assign testswi to-do SWI_handler

code run-swi0	\ r2 r1 r0 -- r1 r0
\ *G Test execution of SWI 0.
  stmfd rsp ! { link }
  mov   r0, tos
  ldmfd psp ! { r1, r2 }
  swi   # 0
  mov   tos, r0
  stmfd psp ! { r1 }
  ldmfd rsp ! { pc }
end-code
[then]

[then]


\ *************************************
\ *S Support for complex abort handlers
\ *************************************
\ *P The complex exception handlers store the ARM CPU state in a
\ ** data frame. The frame is 72 bytes long (18 4 byte cells),
\ ** whose format is:
\ *E --- high ---
\ ** ABORT mode R14/link = faulting PC+8
\ ** faulting R12
\ ** faulting R11
\ ** faulting R10
\ ** faulting R9
\ ** faulting R8
\ ** faulting R7
\ ** faulting R6
\ ** faulting R5
\ ** faulting R4
\ ** faulting R3
\ ** faulting R2
\ ** faulting R1
\ ** faulting R0
\ ** faulting R14
\ ** faulting R13
\ ** faulting CPSR
\ ** ABORT mode CPSR on entry
\ ** --- low --

SimpleAborts? 0= test-isr? and [if]

struct /exframe	\ -- n
\ *G structure defining the exception frame
  int exf.abtCPSR
  int exf.CPSR
  int exf.r13
  int exf.r14
  int exf.r0
  int exf.r1
  int exf.r2
  int exf.r3
  int exf.r4
  int exf.r5
  int exf.r6
  int exf.r7
  int exf.r8
  int exf.r9
  int exf.r10
  int exf.r11
  int exf.r12
  int exf.abtLink
end-struct

: .item		\ addr -- addr+4
\ *G Display a cell item at addr and increment addr.
  dup @ .dword space  cell +
;

: .items	\ addr n -- addr+4n
\ *G Display n items at addr and increment addr.
  0 ?do  .item  loop
;

: .frame	\ ^frame --
\ *G Display the CPU state pointed to by the data frame.
  cr ." === Registers ==="
  cr ." CPSR = " dup exf.CPSR @ .dword	\ CPSR
  cr ." R0  = "  dup exf.R0 4 .items	\ R0..R3
  cr ." R4  = "  4 .items		\ R4..R7
  cr ." R8  = "  4 .items		\ R8..R11
  cr ." R12 = "  .item drop             \ R12
     dup exf.R13 2 .items drop		\ R13 R14
     exf.abtLink .item drop		\ R15
;

[undefined] ip>nfa [if]
: name?         \ addr -- flag
\ *G Check to see if the supplied address is a valid NFA.
\ ** This word is implementation dependent.
\ ** A valid NFA for MPE embedded systems satisfies the following:
\ *B All characters within string are printable ASCII within range 33..126
\ *B String Length is non-zero in range 1..31 and bit 7 is set, ignore bits 6, 5
  dup aligned over <>
  if  drop  FALSE exit  endif
  count					\ c-addr u --
  dup  $9F and  $81 $9F within? 0=	\ NFA first byte = 1SIxxxxx, count = xxxxx
  					\ mask           = 10011111
  if  2drop  0  exit  then
  $01F and bounds ?do
    i c@ #33 #126 within? 0=		\ check all ascii chars
    if  unloop  FALSE exit  then
  loop
  TRUE
;

: ip>nfa	\ addr -- nfa
\ *G Attempt to move backwards from an address within a definition
\ ** to the relevant NFA.
  2-				\ NFA must be at least 'n' bytes backwards
  begin
    dup name? 0=
   while
    1-
  repeat
;
[then]

: check-aligned	\ addr -- addr'
\ *G Check addr for cell alignment, report and correct if
\ ** misaligned.
  dup aligned over <>
  if  ." (Misaligned) "  aligned  then
;

: ?Clip32	\ xsp xspTop -- xsp xspTop'
\ *G Clip the stack display to 32 items.
  2dup swap - #32 cells u> if
    drop  dup #32 cells +
    cr ." Clipped to 32 items"
  endif
;

[defined] Umbilical? [if]
' (do) equ start-of-code
[else]
' and equ start-of-code
[then]
prog sec-end  equ end-of-code

: .rsframe	\ ^frame --
\ *G Display the return stack indicated by the frame, assuming
\ ** the Forth RSP=R13 and RUP=R11.
  dup exf.R13 @				\ -- ^frame rsp
  cr ." RSP = " dup .dword space check-aligned
  swap exf.R11 @ r0-offset + @		\ -- rsp rsTop
  ."   R0 = " dup .dword space check-aligned
\ Note that for negative steps, +LOOP processes the limit value.
  cell -				\ rsTop is not used
  cr ." --- Return stack high ---"
  ?Clip32
  ?do
    cr i .dword space  i @ dup .dword space
    dup start-of-code end-of-code within
    if  ip>nfa .name  else  drop  endif
  cell negate +loop
  cr ." --- Return stack low --- "
;

: .psframe	\ ^frame --
\ *G Display the data stack indicated by the frame, assuming
\ ** the Forth PSP=R12 and RUP=R11.
  dup exf.R12 @				\ -- ^frame psp
  cr ." PSP = " dup .dword space  check-aligned
  over exf.R11 @ s0-offset + @		\ -- ^frame psp psTop
  ."   S0 = " dup .dword space  check-aligned
\ Because of the stack caching S0 is a cell LOWER than PSP
\ for an empty stack. Note also that for negative steps,
\ +LOOP processes the limit value.
  2dup cell + u>
  if  drop 2drop  cr ." Data stack underflowed"  exit  endif
  2dup cell + =
  if  drop 2drop  cr ." Data stack empty"  exit  endif
  2dup swap - SP-SIZE u>
  if  drop 2drop  cr ." Data stack overflowed"  exit  endif
  cr ." --- Data stack high ---"	\ -- ^frame psp psTop/S0
  2dup <> if
    cell - ?Clip32 do
      cr i .dword space  i @ .dword
      cell negate
    +loop
  else
    2drop
  endif
  cr ." --- Data stack low --- "
  cr ." rTOS/R10  " exf.R10 @ .dword
;

[then]


\ ********************************
\ *S Undefined instruction handler
\ ********************************

\ =======================
\ *N Simple UNDEF handler
\ =======================

SimpleAborts? [if]

defer Undef_handler	\ ^ins --
\ *G The place holder for the Undefined Instruction handler has
\ ** a default action of DROP.
  assign drop to-do Undef_handler

PROC Undef_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
\  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
\  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
\  str   rsp, [ up, # r0-offset ]	\ set R0

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to undefined instruction
  bl    ' Undef_handler

  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  mov .s pc, link			\ return from exception
end-code

Undef_exception UndefV !


\ ---------------
\ Undef test code
\ ---------------

test-isr? [if]
: testundef	\ ^ins -- ; handle undefined instruction
\ *G Test handler for Undefined Instructions.
  hex  console opvec !  console ipvec !
  cr ." Undef exception at " dup .lword
  cr ." Instruction        " -4 and @ .lword
  cr ." Restarting system ..."
\ [defined] reboot [if] reboot [else] 0 execute [then]
;
assign testundef to-do Undef_handler

: run-undef	\ --
\ *G Causes an Undefined Instruction exception.
  [asm
    mrc  7 0 r0, cr0, cr0, 0		\ CP7 doesn't exist, so should fault
  asm]
;
[then]

[then]


\ ========================
\ *N Complex UNDEF handler
\ ========================

SimpleAborts? 0= [if]

defer Undef_handler	\ ^frame ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to a data frame and a pointer to the instruction,
\ ** returns true if a fixup has been performed. If the return value
\ ** is non-zero, the instruction is rerun otherwise the instruction
\ ** is skipped.
  assign drop to-do Undef_handler

PROC Undef_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r2, CPSR			\ save CPSR for mode switch
  mrs   r3, SPSR			\ save SPSR for nesting
  bic   r1, r2, # $01F			\ clear mode bits from CPSR
  and   r0, r3, # $01F			\ extract mode bits from SPSR
  orr   r0, r0, r1			\ switch to old mode to extract R13, R14
  msr   CPSR, _c r0
  mov   r4, r13
  mov   r5, r14
  msr   CPSR, _c r2			\ switch mode back
  stmfd rsp ! { r2, r3, r4, r5 }	\ save data on abort stack
  mov   r0, rsp				\ base address of data frame

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0

  sub   psp, psp, # 8			\ indicate depth=2
  str   r0, [ psp ]			\ save pointer to data frame
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' Undef_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r2, r3, r4, r5 }	\ restore SPSR and others saved
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 4		\ return, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

Undef_exception UndefV !


\ ---------------
\ Undef test code
\ ---------------

test-isr? [if]
: testundef	\ ^frame ^ins -- fixed? ; handle undefined instruction
\ *G Test handler for Undefined Instructions.
  hex  console opvec !  console ipvec !
  cr ." Undef exception at " dup .lword
  cr ." Instruction        " -4 and @ .lword
  dup .frame  dup cr .rsframe  cr .psframe
  cr ." Restarting system ..."		\ skip instruction
[defined] reboot [if] reboot [else] 0 execute [then]
  0
;
assign testundef to-do Undef_handler

: run-undef	\ --
\ *G Causes an Undefined Instruction exception.
  cr ." before"
  [asm
    mrc  7 0 r0, cr0, cr0, 0		\ CP7 doesn't exist, so should fault
  asm]
  cr ." after"
;
[then]

[then]

[then]


\ *************************
\ *S Prefetch Abort handler
\ *************************

\ ========================
\ *N Simple PABORT handler
\ ========================

SimpleAborts? [if]

defer PAbort_handler	\ ^ins -- fixed?
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do PAbort_handler

PROC PAbort_exception	\ --
\ The PABORT exception code calls PAbort_handler above.
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' PAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .ne .s  pc, link, # 4		\ return from exception, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

PAbort_exception PAbortV !

test-isr? [if]
: testpabort	\ ^ins -- fixed?
\ *G Test PAbort handler.
  decimal  console opvec !  console ipvec !
  cr ." PAbort exception at " .lword
  drop 0				\ skip instruction
[defined] reboot [if] reboot [else] 0 execute [then]
;
assign testpabort to-do PAbort_handler

ArmArch5? [if]
: run-pabort	\ --
\ *G Run the test PAbort handler.
  [asm
    bkpt # $55AA	
    mov r0, r0
  asm]
;
[else]
: run-pabort	\ --
\ *G Run the test PAbort handler.
  $05000000 execute			\ hardware specific
;
[then]

[then]

[then]


\ =========================
\ *N Complex PAbort handler
\ =========================

SimpleAborts? 0= [if]

defer PAbort_handler	\ ^frame ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to a data frame and a pointer to the instruction,
\ ** returns true if a fixup has been performed. If the return value
\ ** is non-zero, the instruction is rerun otherwise the instruction
\ ** is skipped.
  assign drop to-do PAbort_handler

PROC PAbort_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r2, CPSR			\ save CPSR for mode switch
  mrs   r3, SPSR			\ save SPSR for nesting
  bic   r1, r2, # $01F			\ clear mode bits from CPSR
  and   r0, r3, # $01F			\ extract mode bits from SPSR
  orr   r0, r0, r1			\ switch to old mode to extract R13, R14
  msr   CPSR, _c r0
  mov   r4, r13
  mov   r5, r14
  msr   CPSR, _c r2			\ switch mode back
  stmfd rsp ! { r2, r3, r4, r5 }	\ save data on abort stack
  mov   r0, rsp				\ base address of data frame

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0

  sub   psp, psp, # 8			\ indicate depth=2
  str   r0, [ psp ]			\ save pointer to data frame
  sub   tos, link, # 4			\ point to bad instruction
  bl    ' PAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r2, r3, r4, r5 }	\ restore SPSR and others saved
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 4		\ return, rerun instruction
  mov .s      pc, link			\ return, skip instruction
end-code

PAbort_exception PAbortV !

test-isr? [if]

: testPabort	\ ^frame ^ins -- fixed?
\ *G Test PABORT handler.
  decimal  console opvec !  console ipvec !
  cr ." PAbort exception at " .lword
  dup .frame  dup cr .rsframe  cr .psframe
  cr ." Restarting system ..."		\ skip instruction
[defined] reboot [if] reboot [else] 0 execute [then]
  0
;
assign testPabort to-do PAbort_handler

: run-Pabort   $05000000 execute  ;	\ hardware specific
\ *G Run test PAbort handler.
[then]

[then]


\ ******************
\ *S Data Abort handler
\ ******************

\ ========================
\ *N Simple DAbort handler
\ ========================

SimpleAborts? [if]

defer DAbort_handler	\ ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to an instruction, returns true if a fixup has been
\ ** performed. If the return value is non-zero, the instruction is rerun
\ ** otherwise the instruction is skipped.
  assign noop to-do DAbort_handler

PROC DAbort_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack

  sub   psp, psp, # 4			\ indicate depth=1
  sub   tos, link, # 8			\ pointer to bad instruction
  bl    ' DAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 8		\ return, rerun instruction
  sub .s      pc, link, # 4		\ return, skip instruction
end-code

DAbort_exception DAbortV !

test-isr? [if]
: testdabort	\ ^ins -- fixed?
\ *G Test DABORT handler.
  decimal  console opvec !  console ipvec !
  cr ." DAbort exception at " dup .lword
  cr ." Instruction         " dup -4 and @ .lword
  cr ." Restarting system ..."
[defined] reboot [if] reboot [else] 0 execute [then]
  drop 0				\ skip instruction
;
assign testdabort to-do DAbort_handler
: run-dabort   $04000000 @  ;		\ hardware specific
\ *G Run test PAbort handler.
[then]

[then]


\ ========================
\ *N Complex DAbort handler
\ ========================

SimpleAborts? 0= [if]

defer DAbort_handler	\ ^frame ^ins -- fixed? ; true if instruction fixed up
\ *G Given a pointer to a data frame and a pointer to the instruction,
\ ** returns true if a fixup has been performed. If the return value
\ ** is non-zero, the instruction is rerun otherwise the instruction
\ ** is skipped. The frame format is as for the *\fo{PAbort handler} above.
  assign drop to-do DAbort_handler

PROC DAbort_exception
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r2, CPSR			\ save CPSR for mode switch
  mrs   r3, SPSR			\ save SPSR for nesting
  bic   r1, r2, # $01F			\ clear mode bits from CPSR
  and   r0, r3, # $01F			\ extract mode bits from SPSR
  orr   r0, r0, r1			\ switch to old mode to extract R13, R14
  msr   CPSR, _c r0
  mov   r4, r13
  mov   r5, r14
  msr   CPSR, _c r2			\ switch mode back
  stmfd rsp ! { r2, r3, r4, r5 }	\ save data on abort stack
  mov   r0, rsp				\ base address of data frame

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0

  sub   psp, psp, # 8			\ indicate depth=2
  str   r0, [ psp ]			\ save pointer to data frame
  sub   tos, link, # 8			\ point to bad instruction
  bl    ' DAbort_handler

  cmp   tos, # 0
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
  ldmfd rsp ! { r2, r3, r4, r5 }	\ restore SPSR and others saved
  msr   SPSR, _c _f r3
  ldmfd rsp ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s .ne  pc, link, # 8		\ return, rerun instruction
  sub .s      pc, link, # 4		\ return, skip instruction
end-code

DAbort_exception DAbortV !

test-isr? [if]
: testdabort	\ ^frame ^ins -- fixed?
\ *G Test DABORT handler.
  decimal  console opvec !  console ipvec !
  cr ." DAbort exception at PC = " .lword
\  cr ." DAbort exception at " dup .lword
\  cr ." Instruction         " -4 and @ .lword
  dup .frame  dup cr .rsframe  cr .psframe
  cr ." Restarting system ..."
[defined] reboot [if] reboot [else] 0 execute [then]
  cr ." Instruction skipped"  0		\ skip instruction
;
assign testdabort to-do DAbort_handler

: run-dabort   $05000000 @ drop  ;		\ hardware specific
\ *G Run test DAbort handler.
[then]

[then]


\ ****************************
\ *S ARM Dual PL190 IRQ and FIQ handlers
\ ****************************
\ *P This section is not a treatise on the ARM PL190 Vectored
\ ** Interrupt Controller. These words provide a fairly basic
\ ** set of tools for using the ARM PL190 VIC, which is fully
\ ** documented in the ARM Technical Publications CD available
\ ** free of charge from *\i{www.arm.com}.
\ ** The notes in the Generic IRQ section about interrupt nesting
\ ** apply here.

\ *P This code requires KERNEL62.FTH and the equate
\ ** *\fo{COLDCHAIN?} must be set non-zero in the control file.
\ ** The VIC addresses should be defined in the SFRxxxx file
\ ** which defines the peripheral address for the silicon.

\ *P If the EQUate *\fo{#IRQs} is greater than one, IRQ nesting is
\ ** supported with the penalty of greater overhead.
\ ** IF IRQ nesting is required, the IRQ stack size should be
\ ** at least 64*#IRQs bytes, and TASK-SIZE*#IRQs bytes should
\ ** be added to the SWI stack. If IRQ nesting is not required
\ ** the IRQ stack must be at least *\fo{TASK-SIZE} bytes.

\ *P FIQ nesting is not supported by this code. If required a
\ ** suitable starting point is the AT91 code.
\ ** The FIQ stack must be at least *\fo{TASK-SIZE} bytes.

L: IRQ_template	\ -- addr ; start of area to copy
  _VIC ,

PROC IRQ_entry	\ --
\ *G This is the template code for VIC0 vectored IRQ interrupt handlers.
\ ** It is also used as the VIC0 default/spurious interrupt handler.
\ --- save state ---
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
#IRQs 1 > [if]
  mrs   r3, SPSR			\ save SPSR for nesting ; SFP008
  stmfd rsp ! { r3 }
  mrs   r0, CPSR			\ Switch to SVC mode
  bic   r0, r0, # ARM_MODE_MASK
  orr   r0, r0, # ARM_MODE_SVC
  msr   CPSR, _c r0
  stmfd rsp ! { link }			\ save Supervisor LINK register
[then]
\ --- set up Forth ---
  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0
\ --- call high level handler ---
l: IRQ_call
  bl      ' noop			\ call the high level handler
  ldr	r4, IRQ_template		\ issue end of interrupt to VIC
  str	r3, [ r4, # VICVectAddr ]	\ value written is irrelevant
\ --- restore from Forth ---
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
\ --- restore state ---
#IRQs 1 > [if]
  ldmfd rsp ! { link }                  \ restore Supervisor LINK register
  mrs   r0, CPSR			\ Switch back to IRQ mode
  bic   r0, r0, # ARM_MODE_MASK
  orr   r0, r0, # ARM_MODE_IRQ
  msr   CPSR, _c r0
  ldmfd rsp ! { r3 }			\ restore SPSR ; SFP008
  msr   SPSR, _c _f r3
[then]

  ldmfd   rsp ! { r0-r12, link }	\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
chere IRQ_template - equ /IRQ_template	\ size of template
IRQ_call IRQ_template - equ /IRQ_call	\ offset to call instruction

L: IRQ1_template	\ -- addr ; start of area to copy
  _VIC1 ,

PROC IRQ1_entry	\ --
\ *G This is the template code for VIC1 vectored IRQ interrupt handlers.
\ ** It is also used as the VIC1 default/spurious interrupt handler.
\ --- save state ---
  stmfd rsp ! { r0-r12, link }		\ save unbanked registers and LINK
#IRQs 1 > [if]
  mrs   r3, SPSR			\ save SPSR for nesting ; SFP008
  stmfd rsp ! { r3 }
  mrs   r0, CPSR			\ Switch to SVC mode
  bic   r0, r0, # ARM_MODE_MASK
  orr   r0, r0, # ARM_MODE_SVC
  msr   CPSR, _c r0
  stmfd rsp ! { link }			\ save Supervisor LINK register
[then]
\ --- set up Forth ---
  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
  str   rsp, [ up, # r0-offset ]	\ set R0
\ --- call high level handler ---
  ldr	r0, IRQ1_template		\ read VIC1 - important
  ldr   r1, [ r0, # VICVectAddr ]	\ value is discarded
l: IRQ1_call
  bl      ' noop			\ call the high level handler
  ldr	r4, IRQ1_template		\ issue end of interrupt to VIC
  str	r3, [ r4, # VICVectAddr ]	\ value written is irrelevant
\ --- restore from Forth ---
  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags
\ --- restore state ---
#IRQs 1 > [if]
  ldmfd rsp ! { link }                  \ restore Supervisor LINK register
  mrs   r0, CPSR			\ Switch back to IRQ mode
  bic   r0, r0, # ARM_MODE_MASK
  orr   r0, r0, # ARM_MODE_IRQ
  msr   CPSR, _c r0
  ldmfd rsp ! { r3 }			\ restore SPSR ; SFP008
  msr   SPSR, _c _f r3
[then]

  ldmfd   rsp ! { r0-r12, link }	\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
chere IRQ1_template - equ /IRQ1_template	\ size of template
IRQ1_call IRQ1_template - equ /IRQ1_call	\ offset to call instruction

L: FIQ_template
  _VIC ,
PROC FIQ_entry	\ --
\ *G This is the template code for FIQ interrupt handlers.
  stmfd   rsp ! { r0-r7, link }		\ save unbanked registers and LINK
  mrs   r3, SPSR			\ save SPSR for nesting
  stmfd rsp ! { r3 }

  sub   up, rsp, # up-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP ; SFP002
\  sub	r4, psp, # tos-cached? cells	\ generate new S0 ; SFP002
\  str   r4, [ up, # s0-offset ]		\ that is compatible with SP! ; SFP002
  sub   rsp, up, # sp-size		\ reserve space for data stack
\  str   rsp, [ up, # r0-offset ]	\ set R0

l: FIQ_call
  bl      ' noop			\ call the high level handler
\  ldr	r4, FIQ_template		\ issue end of interrupt to VIC
\  str	r3, [ r4, # VICVectAddr ]	\ value written is irrelevant

  add   rsp, rsp, # sp-size up-size +	\ restore rsp - does not affect flags

  ldmfd rsp ! { r3 }			\ restore SPSR
  msr   SPSR, _c _f r3
  ldmfd   rsp ! { r0-r7 link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
FIQ_entry FIQV !			\ set FIQ entry point

interpreter
: IRQ:		\ xt -- ; -- isr
\ *G Creates an IRQ ISR for VIC0 that runs the given Forth word. At run
\ ** time the entry point of the ISR is returned. Use in the form:
\ *C   ' <action> IRQ: <actionISR>
  chere  /IRQ_template allot		\ reserve space
  IRQ_template over /IRQ_template move	\ copy template
  tuck /IRQ_call + !call		\ form call instruction
  cell + label				\ returns entry point
;

: IRQ1:		\ xt -- ; -- isr
\ *G Creates an IRQ ISR for VIC1 that runs the given Forth word. At run
\ ** time the entry point of the ISR is returned. Use in the form:
\ *C   ' <action> IRQ: <actionISR>
  chere  /IRQ1_template allot		\ reserve space
  IRQ1_template over /IRQ1_template move	\ copy template
  tuck /IRQ1_call + !call		\ form call instruction
  cell + label				\ returns entry point
;

: SetDefIRQ	\ xt --
\ *G Set the default IRQ handler to call the Forth word whose xt
\ ** is given. Use interpretively in the form:
\ *C   ' <word> SetDefIRQ
\ *P The compiler sets this action to *\fo{NOOP} unless you use
\ ** *\fo{SETDEFIRQ}.
  IRQ_call !call
;

: SetFIQ	\ xt --
\ *G Set the FIQ interrupt to call the Forth word whose xt is
\ ** given. Use interpretively in the form:
\ *C   ' <word> SetFIQ
  FIQ_call !call
;
target

: InitVIC	\ --
\ *G Initialise the VIC Default interrupt vector.
\ ** This word is executed as part of the cold chain.
  IRQ_entry _VIC0 VICDefVectAddr + !
  IRQ1_entry _VIC1 VICDefVectAddr + !
;
' InitVIC AtCold

L: VICtab	\ -- addr
\ *G Table of VIC addresses.
  _VIC0 ,
  _VIC1 ,

: src>vic	\ src# -- vic src#'
\ *G Given a source number, return an updated source number and
\ ** the base address of the corresponding VIC unit.
  dup $01F invert and 3 rshift VICtab + @	\ -- src# vic
  swap $01F and
;

: EnInt		\ src# --
\ *G Enable the requested VIC interrupt source.
  src>vic  1 swap lshift		\ generate bit mask
  swap VICIntEnable + !			\ for enable register.
;

: DisInt	\ src# --
\ *G Disable the requested VIC interrupt source.
  src>vic  1 swap lshift		\ generate bit mask
  swap VICIntEnClr + !			\ for disable register.
;

: SetIrqIsr	\ isr src# slot# --
\ *G Set the ISR to be the action of the source interrupt number
\ ** src# (0..n) being set into the corresponding slot
\ ** (slot# = 0..15) where slot# corresponds to priority
\ ** (0 = highest priority). Then enable the
\ ** interrupt. Note that Forth IRQ interrupt handlers
\ ** must be created with IRQ: <name>.
\ ** Assembler routines may be created using:
\ *C PROC <name> ... END-CODE
\ *P Interrupt source numbers may be found in the VIC section
\ ** of the CPU data sheet and the relevant SFRxxxx.FTH file.
  swap dup DisInt			\ -- isr slot# src# ; disable this interrupt
  dup >r src>vic >r			\ -- isr slot# vic ; R: -- src# src#'
  swap cells + tuck			\ -- vbase isr vbase ; R: -- src# src#'
  VICVectAddr0 + !			\ -- vbase ; R: -- src# src#' ; set isr entry point
  r> $20 or swap VICVectCntl0 + !	\ set src and enable vector
  r> EnInt				\ enable interrupt source
;

: setFIQsrc	\ src# --
\ *G Set *\i{src#} to be an enabled FIQ interrupt. During
\ ** cross compilation the ISR address must have been set with
\ ** *\fo{SetFIQ} above. The FIQ interrupt can be enabled and
\ ** disabled by *\fo{EnInt} and *\fo{DisInt}.
  src>vic  1 swap lshift  swap		\ -- mask vic ; generate bit mask
  2dup VICIntSelect + !			\ set as FIQ
  VICIntEnable + !			\ for enable register.
;

\ *P An example of setting up an interrupt follows.
\ ** In the example below, the line
\ *C ' ISRaction IRQ: MyIsr  \ -- isr
\ *P is for VIC0 only. If the ISR is for VIC1 use
\ ** ' ISRaction IRQ1: MyIsr  \ -- isr
\ *P for VIC1 interrupt sources.

\ *E : ISRaction  \ -- ; the Forth word
\ **   ...
\ ** ;
\ **
\ ** ' ISRaction IRQ: MyIsr  \ -- isr
\ ** #15 equ MySrc#  \ -- n ; what triggers it
\ ** #3 equ MySlot#  \ -- n ; what priority (0=highest)
\ ** 
\ ** : SetupISR   \ -- ; initialise
\ **   MyIsr MySrc# MySlot# SetIRQisr 
\ ** ;

: (.VIC)		\ vic --
\ *G Show the status of the VIC whose address is given.
  base @ >r  binary
  cr ."                 FEDCBA9876543210FEDCBA9876543210"
  cr ." VICIntEnable:   " dup VICIntEnable + @ 32 U.R
  cr ." VICIntSelect:   " dup VICIntSelect + @ 32 U.R
  cr ." VICDefVectAddr: " dup VICDefVectAddr + @ .dword
  cr ." Slot IRQ#  Vect'd  Vector
  cr
  $10 0 do
    cr i .byte 4 spaces
    dup VICVectCntl0 + i cells + @
    dup $1F and .byte 3 spaces  $20 and
    if ." yes" else ." No " endif
    5 spaces
    dup VICVectAddr0 + i cells + @ .dword
  loop
  cr ." VicProtect: " dup VICProtection + @ 1 and
  if ." yes" else ." no" endif 
  drop
  r> base !
;

: .VIC0		\ --
\ *G Show the status of the primary VIC.
  _VIC (.vic)
;

: .VIC1		\ --
\ *G Show the status of the secondary VIC.
  _VIC1 (.vic)
;

: SWINT		\ src# --
\ *G Generate interrupt from software.
  src>vic  1 swap lshift  swap VICSoftInt + !
;

[then]


\ ======
\ *> ###
\ ======

