\ ARM v6/7 Multi-Tasker

((
Copyright (c) 2000, 2002, 2003, 2008, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20160125 SFP010 Updated for R8=FSP
20141121 MPE009 Improved context switch speed.
20080613 SFP008 Documentation tweaks.
20031216 SFP007 Added DOCGEN documentation.
20030729 SFP006 Corrected TASK for TASK-U0 non-zero.
20030709 SFP005 Added .RUNNING to display running tasks.
20020919 SFP004 Added HIS to provide access to other task's
		USER variables.
		Added debugging words .TASK ( task -- )
		and .TASKS ( -- ).
20020915 SFP003 Fixed PSP initialisation bug in INIT-TASK.
20020712 SFP002 Added CDATA to end of some INTERPRETER definitions.
20000619 SFP001 Converted to v6.1 model


********************
Resource definitions
********************

On the ARM the following register usage is assumed
  r15         pc      program counter
  r14         link    link register
  r13         rsp     return stack pointer
  r12         psp     data stack pointer
  r11         up      user area pointer
  r10         tos     cached top of stack
  r9          lp      locals pointer
  r8          fsp     float stack pointer
  r0-r7       scratch
))

\ ===========
\ *! multiarm
\ *T ARM multitasker
\ ===========
\ *P The ARM multitasker follows the model introduced
\ ** with the v6.1 compilers. A few extensions are also
\ ** provided.

\ *P The code in *\i{MultiARM.fth} only works with the ARM32
\ ** register set usage. If you need interworking with one of the
\ ** Cortex register set usages, please contact MPE or modify the
\ ** code yourself.


\ *************
\ *S Configuration - normally performed earlier
\ *************

[undefined] test-multi? [if]
0 equ test-multi?	\ true to compile test code
\ *G If previously undefined, *\fo{TEST-MULTI?} is set to zero
\ ** and test code is not compiled.
[then]


\ ****************************
\ *S TCB data structure layout
\ ****************************
\ *E cell       LINK    link to next task
\ ** cell       SSP     Saved Stack Pointer
\ ** cell       STAT    Bit 0     1 = running, 0 = halted
\ **                    Bit 1     1 = message pending
\ **                    Bit 2     1 = event triggered
\ **                    Bit 3     1 = event handler run
\ **                    Bit 4..7  Reserved
\ ** 	        others  1 = set to run task, available to user
\ ** cell       TASK    Task that sent message here
\ ** cell       MESG    Message address
\ ** cell       EVNTw   CFA of word run as event handler
\ *P This structure is allocated at the start of the *\fo{USER}
\ ** area. Consequently the *\fo{TCB} of the current task is given
\ ** by *\fo{UP}.

struct /TCB	\ -- size
\ *G The structure used by the code that matches the
\ ** description above.
  ptr tcb.link		\ link to next task ; MUST BE FIRST
  ptr tcb.ssp	        \ Saved Stack Pointer
  int tcb.status	\ status word
  int tcb.msrc		\ message source
  int tcb.mesg		\ message
  ptr tcb.event		\ xt of word which is event handler
end-struct

\ bit masks for the status cell
$0001 equ run-mask  run-mask invert equ ~run-mask	\ running
  0 equ run-bit#
$0002 equ msg-mask  msg-mask invert equ ~msg-mask	\ message available
  1 equ msg-bit#
$0004 equ trg-mask  trg-mask invert equ ~trg-mask	\ event triggered
  2 equ trg-bit#
$0008 equ evt-mask  evt-mask invert equ ~evt-mask	\ event run
  3 equ evt-bit#


\ ******************
\ Consistency checks
\ ******************

/tcb tcb-size <> [if]
  cr ." Control file and tasker disagree as to TCB size"  abort
[endif]


\ ***************************
\ *S Task handling primitives
\ ***************************

cdata		\ so that comma is to code space

init-u0 constant main	\ -- addr ; tcb of main task
\ *G Returns the base address of the main task's *\fo{USER} area.
l: main-link
  0 ,					\ end of task chain

0 value multi?	\ -- flag
\ *G Returns true if the tasker is enabled.

l: ^multi	\ -- addr ; holds pointer to MULTI? data area
  ' multi? >body ,

: single	\ --
\ *G Disable scheduler.
  0 to multi?
;

: multi		\ --
\ *G Enable scheduler.
  -1 to multi?
;

((
On the ARM the following register usage is assumed
  r15         pc      program counter
  r14         link    link register
  r13         rsp     return stack pointer
  r12         psp     data stack pointer
  r11         up      user area pointer
  r10         tos     cached top of stack
  r9          lp      locals pointer
  r8          fsp     float stack pointer
  r0-r7       scratch
))
CODE pause	\ -- ; the scheduler itself
\ *G The software scheduler itself.
l: [schedule]
  ldr   r0, ^multi			\ inspect contents of MULTI?
  ldr   r1, [ r0 ]
  cmp   r1, # 0				\ single tasking if 0
  mov .eq pc, link                      \ single tasking so exit
\
\ save previously running task ip, rp, up, sp
\ can use with interrupts if we stack everything
\
  stmfd rsp ! { r8, r9, r10, r12, link } \ FSP, LP, TOS, PSP, LR ; MPE009, SFP010
\  stmfd rsp ! { r0-r12, link }          \ stack the lot
  str   rsp, [ up, # 0 tcb.ssp ]	\ save SP in TCB
\
\ select next task to run
\
l: [schedule]1
  ldr   up, [ up, # 0 tcb.link ]	\ get next task
  ldr   r0, [ up, # 0 tcb.status ]	\ inspect status
  cmp   r0, # 0				\ 0 = not running
  b .eq [schedule]1
\
\ run selected task - sp, up, rp, ip
\
  ldr   rsp, [ up, # 0 tcb.ssp ]	\ restore SSP
  ldmfd rsp ! { r8, r9, r10, r12, link } \ FSP, LP, TOS, PSP, LR ; MPE009, SFP010
\  ldmfd rsp ! { r0-r12, link }		\ restore registers

event-handler? [if]	\ if user wants the event handler
\
\ event handler exit
\
  ldr   r0, [ up, # 0 tcb.status ]	\ look at status
  and .s  r1, r0, # trg-mask		\ inspect event trigger bit
  ne, if,
    bic   r0, r0, # trg-mask		\ clear trigger bit
    orr   r0, r0, # evt-mask		\ set event bit
    str   r0, [ up, # 0 tcb.status ]	\ restore status
    ldr   pc, [ up, # 0 tcb.event ]	\ run event handler
  endif,
[then]
  next,
end-code

code status	\ -- task-status
\ *G Returns the current task's status cell, but with the run
\ ** bit masked out.
  stmfd psp ! { tos }                   \ save tos
  ldr   tos, [ up, # 0 tcb.status ]	\ fetch task status byte
  bic   tos, tos, # run-mask		\ mask running bit
  next,
end-code

CODE restart    \ task -- ; mark task TCB as running
\ *G Sets the RUN bit in the task's status cell.
  ldr   r0, [ tos, # 0 tcb.status ]	\ get task status
  orr   r0, r0, # run-mask		\ set task running bit
  str   r0, [ tos, # 0 tcb.status ]	\ put back status
  ldmfd psp ! { tos }                   \ restore tos
  next,
END-CODE

CODE halt       \ task -- ; reset running bit in TCB
\ *G Clears the RUN bit in the task's status cell.
  ldr   r0, [ tos, # 0 tcb.status ]	\ get task status
  bic   r0, r0, # run-mask		\ reset task running bit
  str   r0, [ tos, # 0 tcb.status ]	\ put back status
  ldmfd psp ! { tos }                   \ restore tos
  next,
END-CODE

: stop          \ -- ; halt oneself
\ *G *\fo{HALT}s the current task, and executes *\fo{PAUSE}.
  self halt  pause
;


\ **************
\ *S Event handling
\ **************
\ *P Event handling is only compiled if the equate
\ ** *\fo{EVENT-HANDLER?} is set non-zero in the control
\ ** file.

event-handler? [if]

: set-event	\ task --
\ *G Set the event trigger in task TCB.
  tcb.status dup @  trg-mask or  swap !
;

: event?	\ task -- flag
\ *G Returns true if true if task has received an event
\ ** trigger which has not been cleared yet.
  tcb.status @  trg-mask and 0<>
;

: clr-event-run	\ --
\ *G Reset the current task's *\fo{EVENT_RUN} flag.
  up@ tcb.status dup @  ~evt-mask and  swap !
;

: to-event      \ xt task -- ; define action of a task
\ *G Sets XT as the event handler for the task.
  tcb.event !				\ set into TCB of task
;
[then]


\ ****************
\ *S Message handling
\ ****************
\ *P Message handling is only compiled if the equate
\ ** *\fo{MESSAGE-HANDLER?} is set non-zero in the control
\ ** file.

message-handler? [if]
: msg?		\ task -- flag
\ *G Returns true if task has received a message.
  tcb.status @ msg-mask and 0<>
;

internal
: (send-msg)	\ mesg task -- ; store message in task
  self over tcb.msrc !			\ set source
  dup
  tcb.status dup @  msg-mask or  swap !	\ set message bit
  tcb.mesg !				\ set message
;

: (get-message)	\ -- mesg task
  up@ tcb.mesg @			\ message
  up@ tcb.msrc @
  up@ tcb.status
  dup @  ~msg-mask and  swap !		\ reset message bit
;
external

: send-message  \ addr task --
\ *G Send a message to a task.
  begin  dup msg?               	\ if task already has a message
    while  pause                	\ wait
  repeat                        	\ and try again
  (send-msg)				\ send the message
;

: get-message   \ -- addr task
\ *G Wait for any message and return the message and the task it
\ ** came from.
  begin  self msg? 0=           	\ check for a pending message
    while  pause                	\ wait for it
  repeat                        	\ try again
  (get-message)                 	\ read it
;
[then]

event-handler? message-handler? and [if]
: wait-event/msg        \ --
\ *G Wait for a message or an event trigger.
  begin
    self msg?  self event?  or 0=
   while
    pause
  repeat
;
[then]


\ *************************
\ *S Task structure management
\ *************************

internal

(( Extract from scheduler save
  stmfd rsp ! { r9, r10, r12, link }	\ LP, TOS, PSP, LR ; MPE009
  str   rsp, [ up, # 0 tcb.ssp ]	\ save SP in TCB
))
code init-task	\ xt task -- ; Initialise a task stack
\ *G Initialise a task's stack before running it and
\ ** set it to execute the word whose XT is given.
  ldmfd psp ! { r1 }			\ get execution address
  mov   r2, rsp				\ save return stack pointer

\ Generate the RSP of the new task
\ the next line works because TASK-U0 is greater than TASK-R0
  sub   rsp, tos, # task-u0 task-r0 -	\ generate new task RSP

  mov   r0, # 0				\ will need this many times

  str   rsp, [ tos, # r0-offset ]	\ save new R0 (taskID=UP)
  str   r0, [ tos, # 0 tcb.status ]	\ clear new task status

  stmfd rsp ! { r1 }			\ R14 LINK, push xt=link
\ the next line works because TASK-U0 is greater than TASK-S0
  sub   r1, tos, # task-u0 task-s0 -	\ calculate new PSP
  sub	r1, r1, # sp-guard cells	\ generate new PSP ; SFP003
  stmfd rsp ! { r1 }			\ R12, new PSP ; SFP003
  sub	r1, r1, # tos-cached? cells	\ generate new S0 ; SFP003
  str   r1, [ tos, # s0-offset ]	\ that is compatible with SP!
\  stmfd rsp ! { tos }			\ R11, new R11=UP ; del MPE009
  stmfd rsp ! { r0 }			\ R10, new TOS
  stmfd rsp ! { r0 }			\ R9
\ the next line works because TASK-U0 is greater than TASK-F0
  sub   r1, tos, # task-u0 task-f0 -	\ calculate new FSP
  stmfd rsp ! { r1 }			\ R8, new FSP ; SFP010
\  stmfd rsp ! { r0 }			\ R7
\  stmfd rsp ! { r0 }			\ R6
\  stmfd rsp ! { r0 }			\ R5
\  stmfd rsp ! { r0 }			\ R4
\  stmfd rsp ! { r0 }			\ R3
\  stmfd rsp ! { r0 }			\ R2
\  stmfd rsp ! { r0 }			\ R1
\  stmfd rsp ! { r0 }			\ R0
  str   rsp, [ tos, # 0 tcb.ssp ]	\ save RSP

  mov   rsp, r2				\ restore RSP
  ldmfd psp ! { tos }			\ restore TOS
  next,
end-code

: add-task      \ task -- ; insert into list
\ *G Add the task to the list of tasks after the current task.
  self tcb.link @			\ save task currently pointed to
  over tcb.link !			\ and make new task point to it
  self tcb.link !			\ current task points to new task
;

: sub-task      \ task -- ; remove task from chain
\ *G Remove the task from the task list.
\ look for task which points to required task
  self					\ head of chain ; -- target current
  begin  2dup tcb.link @ <>		\ while curr. does not point to target
   while				\ step through chain
    tcb.link @  dup self =		\ but if task not in list, check for end
    if  2drop  exit  endif		\ and get out
  repeat                                \ -- target points-to-it
\ set task that points to target to point to task pointed to by the target
  swap tcb.link @			\ task pointed to by target
  swap tcb.link !			\ set into link field
;

external

: initiate	\ xt task -- ; start task from scratch
\ *G Start the given task executing the word whose XT is given, e.g.
\ *C   ['] <name> <task> INITIATE
  tuck init-task			\ reset TCB
  dup add-task                          \ add to chain
  run-mask swap tcb.status !		\ mark as running
  pause					\ let it run
;

: sleeper	\ xt task --
\ *G Start task from scratch, but leave it *\fo{HALT}ed. Use in
\ ** the form:
\ *C   ['] <action> <taskname> SLEEPER
\ *P to put a task on the active task list, but as if *\fo{HALT}ed.
\ ** *\fo{SLEEPER} allows you to make a task ready for waking up
\ ** later, perhaps by another task. This avoids having to put
\ ** *\fo{STOP} as the first word in a task. Note that *\fo{SLEEPER}
\ ** does not call *\fo{PAUSE}. See also *\fo{INITIATE}.
  tuck init-task			\ reset TCB
  dup add-task                          \ add to chain
  tcb.status off			\ mark as idle
;

: terminate	\ task --
\ *G Stop a task, and remove it from the list.
  sub-task pause			\ halt task, and remove from chain
;

: init-multi    \ -- ; initialisation with multi-tasking
\ *G Initialise the multitasker and start it. If tasking is
\ ** selected by setting the equate *\fo{TASKING?} in the control
\ ** file, *\i{KERNEL62.FTH} will automatically run this word.
\ ** Make sure that your initialisation code includes
\ ** *\fo{INIT-MULTI} or your code will crash.
  single				\ tasker disabled
  main /tcb erase			\ clean TCB
  main dup tcb.link !			\ initialise MAIN to point to itself
  run-mask main tcb.status !		\ and reset status field
  multi					\ mark tasker enabled
;

: his		\ task uservar -- addr
\ *G Given a task id and a *\fo{USER} variable, returns the
\ ** address of that variable in the given task. This word is
\ ** used to set up *\fo{USER} variables in other tasks.
  self - +
;


\ **********
\ *S Semaphores
\ **********
\ *P The semaphore code is only compiled if the equate
\ ** *\fo{SEMAPHORES?} is set non-zero in the control
\ ** file.

semaphores? [if]

\ *P A *\fo{SEMAPHORE} is an extended variable used for signalling
\ ** between tasks, and for resource allocation. It contains two
\ ** cells, a *\b{counter} and an *\b{arbiter}. The counter field
\ ** is used as a count of the number of times the resource may
\ ** be used, and the arbiter field contains the TCB of the task
\ ** that currently owns it. This field can be used for priority
\ ** arbitration and deadlock detection/arbitration. The count
\ ** field allows the semaphore to be used as a*\b{counted}
\ ** semaphore or as an exclusive access semaphore.

\ *P For example a character buffer may be used where the semaphore
\ ** counter contains the number of available characters.

\ *P An exclusive access semaphore is used to share resources.
\ ** The semaphore is initialised to one, usually by *\fo{SIGNAL}.
\ ** The first task to *\fo{REQUEST} it gains access, and all
\ ** other tasks must wait until the accessing task *\fo{SIGNAL}s
\ ** that it has finished with the resource.

interpreter
: semaphore     \ -- ; -- addr [child]
\ *G Creates a semaphore which returns its address at runtime.
\ ** The count field is initialised to zero for use as counted
\ ** semaphore. Use in the form:
\ *C Semaphore <name>
\ *P If you want this to be an exclusive access semaphore, follow
\ ** this with:
\ *C   1 <name> !
  idata create
    0 ,  0 ,				\ count and arbiter fields
  cdata
;
target

heads? [if]
: semaphore	\ -- ; -- addr [child]
  create
    0 ,  0 ,				\ count and arbiter fields
;
[then]

: request       \ sem -- ; get access to resource, wait if count = 0
\ *G *\fo{REQUEST} waits until the counter field of a semaphore
\ ** is non-zero, and then decrements the counter field by one.
  begin
    [I  dup @ 0=			\ n.b. test and set
   while
    I]  pause				\ operations must be
  repeat                                \ non-interruptible and indivisible
  dup decr				\ got it, decrement counter
  self swap cell+ !			\ mark resource as mine
  I]					\ re-enable interrupts
;

: signal        \ addr --
\ *G *\fo{SIGNAL} increments the counter field of a semaphore, indicating
\ ** either that another item has been allocated to the resource, or
\ ** that it is available for use again, 0 indicating in use by a task.
  [I					\ must be interrupt safe
    dup incr  cell+ off			\ inc. counter, release
  I]
;

[then]


\ ******************
\ *S TASK and START:
\ ******************
\ *P *\fo{TASK <name>} builds a named task user area.
\ ** The action of a task is assigned and the task started
\ ** by the word *\fo{INITIATE}
\ *C   ['] <action> <task> INITIATE

\ *P *\fo{START:} is used inside a colon definition. The code
\ ** before *\fo{START:} is the task's initialisation, performed
\ ** by the current task. The code after *\fo{START:} up to the
\ ** closing *\fo{;} is the action of the task. For example:

\ *E   TASK FOO
\ **   : RUN-FOO
\ **     ...
\ **     FOO START:
\ **     ...
\ **     begin ... pause again
\ **   ;

\ *P All tasks must run in an endless loop, except for initialisation
\ ** code.
\ ** When *\fo{RUN-FOO} is executed, the code after *\fo{START:}
\ ** is set up as the action of task *\fo{FOO} and started.
\ ** *\fo{RUN-FOO} then exits.

\ *P If you want to perform additional actions after starting the task, you
\ ** should use *\fo{INITIATE} to start the task.

variable task-chain	\ -- addr
\ *G Anchors list of all tasks created by *\fo{TASK} and friends.
  main-link task-chain !		\ MAIN already defined

interpreter	\ host version
: task		\ -- ; -- task ; TASK <name> builds a task
\ *G Note that the cross-interpreter's version of *\fo{TASK} has
\ ** been modified from v6.2 onwards to leave the current
\ ** section as *\fo{CDATA}.
  udata
  task-size reserve task-u0 + constant	\ build pointer ; SFP006
  cdata
  here  task-chain @ ,  task-chain !	\ link task
;
target

heads? [if]	\ target version
: task		\ -- ; -- task ; TASK <name> builds a task
\ *G Creates a new task and data area, returning the address
\ ** of the user area at run time. The task is also linked into
\ ** the task chain anchored by *\fo{TASK-CHAIN}.
  here task-u0 +			\ task's user area ; SFP006
  task-size allot
  constant				\ build pointer
  here  task-chain @ ,  task-chain !	\ link task
;
[then]

: start:	\ task -- ; exits from caller
\ *G Used inside a colon definition. The code following
\ ** *\fo{START:} up to the ending semi-colon forms the action
\ ** of the task. The word containing *\fo{START:} finishes at
\ ** *\fo{START:}.
  r> swap initiate
;


\ ***************
\ *S Debugging tools
\ ***************

[defined] ip>nfa [if]
: .taskBody	\ addr --
\ *G Display the name of a task, given the address of the link.
  dup $0040:0000 u< if			\ in low 4Mb?
    #12 - dup ip>nfa			\ -- xt nfa ; ARM specific
    2dup - #32 u>			\ may be headerless
    if  drop .dword  else nip .name  endif
  else
    .dword
  endif
  space
;
[else]
: .taskBody	\ addr --
\ *G Display the name of a task, given the address of the link.
  dup $0040:0000 u< if			\ in low 4Mb?
    #12 - >name .name space		\ ARM specific
  else
    .dword
  endif
;
[then]

: .task		\ task --
\ *G Display task's name if it has one, otherwise display its
\ ** address.
  task-chain
  begin					\ -- task link
    @ dup
   while				\ -- task link
    2dup cell - @ = if
      nip  .taskBody  exit
    endif
  repeat
  drop .lword ."  is not in TASK chain"
;

: .tasks	\ task -- ; display all task names
\ *G Display all the tasks anchored by *\fo{TASK-CHAIN}.
  task-chain
  begin
    @ dup
   while
    dup .taskBody
  repeat
  drop
;

: .running	\ --
\ *G Display all the running tasks.
  main
  begin
    cr dup .task
    tcb.link @
    dup main =
  until
  drop
;


\ ********
\ all done
\ ********

decimal


\ *********
\ Test code
\ *********

test-multi? [if]

task task1	\ -- addr ; build task area

500000 value rate1	\ -- n ; number of idle iterations

: action1	\ -- ; action of task1
  hex  console opvec !
  begin
    [char] * emit
    rate1 0 do pause loop
  again
;

task task2	\ -- addr ; build task area

2000000 value rate2	\ -- n ; number of idle iterations

: run-task2	\ -- ; example of using START:
  task2 start:
    console opvec !
    begin
      [char] / emit
      rate2 0 do pause loop
    again
;

[then]


\ ======
\ *> ###
\ ======

