\ STACKDEF.FTH - default stack layout

((
Copyright (c) 2004, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20160125 SFP001 Updated for R8=FSP
))

decimal

\ ======
\ *! stackdef
\ *T Default stack layout
\ ======

[undefined] FP-SIZE [if]
0 equ FP-SIZE	\ -- len
[then]


\ ***************************
\ Task area and Stack equates
\ ***************************
\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, FSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size fp-size + sp-size + equ TASK-U0	\ initial offset of user area
rp-size fp-size + sp-size + equ TASK-S0	\ initial offset of data stack
rp-size fp-size + equ TASK-F0		\ initial offset of float stack
rp-size           equ TASK-R0		\ initial offset of return stack

\ This defines the size and offsets of task areas,
\ and reserves space for the main task.
task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-f0 + equ INIT-F0		\ top of main float stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack

\ All tasks run in System (SYS) or User (USR) mode, which
\ share the same stack pointer. Defining additional tasks
\ reserves RAM for the tasks.
INIT-R0 equ SYS/USER_STACK_TOP
rp-size equ SYS/USER_STACK_SIZE
INIT-R0 rp-size - equ SYS/USER_STACK_BASE

\ Standalone Forths require a terminal input buffer
[defined] tib-len [if]
tib-len reserve equ INIT-TIB		\ base of TIB
[then]


\ ************************************
\ Exception/Interrupt stack allocation
\ ************************************
\ IRQ stacks ; nestable up to #IRQs
\ We assume that high level interrupt handlers will be used
#IRQs 1 > [if]
#20 cells #IRQs * equ IRQ_STACK_SIZE	\ main action on SVC stack
[else]
task-size #IRQs * equ IRQ_STACK_SIZE	\ total RAM required
[then]

\ FIQ, ABT, UND stacks
\ We assume that high level interrupt handlers will be used
#FIQs 1 > [if]
#20 cells #FIQs * equ FIQ_STACK_SIZE	\ main action on SVC stack
[else]
task-size #FIQs * equ FIQ_STACK_SIZE	\ total RAM required
[then]
task-size equ ABT_STACK_SIZE
task-size equ UND_STACK_SIZE

\ SVC/SWI stack space ; nestable up to #SWIs + #IRQs + #FIQs
\ We assume that high level interrupt handlers will be used
task-size #SWIs *			\ SWI nesting space
#IRQs 1 > [if]				\ allow for nestable IRQs
  task-size #IRQs * +
[then]
#FIQs 1 > [if]				\ allow for nestable FIQs
  task-size #FIQs * +
[then]
  equ SVC_STACK_SIZE

0 reserve equ INT_STACK_TOP		\ top of exception stacks
IRQ_STACK_SIZE FIQ_STACK_SIZE +		\ IRQ and FIQ
SVC_STACK_SIZE +			\ SWI/SVC
ABT_STACK_SIZE + UND_STACK_SIZE +	\ ABT and UND
  reserve equ INT_STACK_BASE		\ bottom of exception stacks


\ ======
\ *> ###
\ ======

decimal

