\ TestDiv.fth - ARM division performance tests

((
Copyright (c) 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ==========
\ *! testdiv
\ *T ARM division tests
\ ==========
\ *P This code tests division correctness and performance.
\ ** The code requires the files *\i{Common\TESTS\MPE\7-1-A.FTH}
\ ** and *\i{Common\TESTS\MPE\7-1-B.FTH}.

10000 value #t	\ -- u
\ *G Number of times each test is performed.


\ ******************
\ *S Signed division
\ ******************

CODE u/mod 	\ u1 u2 -- rem quot
  \ *G Unsigned division of *\i{u1} by *\i{u2} single-precision
  \ ** yielding remainder and quotient.
  mov     r1, tos			\ d=R1=denominator  (numerator/denominator)
  ldr     tos, [ psp ]			\ n/r=TOS=numerator/remainder
  mov     r0, # 0			\ q=R0=current quotient
					\ t=R2=temp
  \ N.B. after subtraction, Carry=1 indicates Borrow=0
  rsb .s  r2, r1, tos lsr # 3		\ if ((n>>3)>=d, B=0, C=1
  b .cc   div_3bits			\ quotient fits in 3 bits
  rsb .s  r2, r1, tos lsr # 8		\ if ((n>>8)>=d, B=0, C=1
  b .cc   div_8bits			\ quotient fits in 8 bits
  mov     r1, r1 lsl # 8		\ d := d*256
  orr     r0, r0, # $FF00:0000		\ make loop iterate twice
  rsb .s  r2, r1, tos lsr # 4		\ if ((n>>4)>=d, B=0, C=1
  b .cc   div_4bits			\ quotient fits in 12 bits
  rsb .s  r2, r1, tos lsr # 8		\ if ((n>>8)>=d, B=0, C=1
  b .cc   div_8bits			\ quotient fits in 16 bits
  mov     r1, r1 lsl # 8		\ d := d*256
  orr     r0, r0, # $00FF:0000		\ make loop iterate 3 times
  rsb .s  r2, r1, tos lsr # 8		\ if ((n>>8)>=d), B=0, C=1
  mov .cs r1, r1, lsl # 8		\ d := d*256
  orr .cs r0, r0, # $0000:FF00		\ make loop iterate 4 times
  rsb .s  r2, r1, tos lsr # 4		\ if ((n>>4)>=d
  b .cc   div_4bits			\   r/d quotient fits in 4 bits
  rsb .s  r2, r1, # 0			\ if 0>=d, divide by zero
  b .cs   div_by_0
  \ at this point, C=0
l: div_loop
  mov .cs r1, r1 lsr # 8		\ set for next 8 bits
l: div_8bits				\ calculate next 8 quotient bits
  rsb .s  r2, r1, tos lsr # 7		\ if ((n>>7)>=d), B=0, C=1
  sub .cs tos, tos, r1 lsl # 7		\ if C, r -= d<<7
  adc     r0, r0, r0			\ q := (q<<1)+C
  rsb .s  r2, r1, tos lsr # 6		\ if ((n>>6)>=d), B=0, C=1
  sub .cs tos, tos, r1 lsl # 6		\ if C, r -= d<<6
  adc     r0, r0, r0			\ q := (q<<1)+C
  rsb .s  r2, r1, tos lsr # 5		\ if ((n>>5)>=d), B=0, C=1
  sub .cs tos, tos, r1 lsl # 5		\ if C, r -= d<<5
  adc     r0, r0, r0			\ q := (q<<1)+C
  rsb .s  r2, r1, tos lsr # 4		\ if ((n>>4)>=d), B=0, C=1
  sub .cs tos, tos, r1 lsl # 4		\ if C, r -= d<<4
  adc     r0, r0, r0			\ q := (q<<1)+C
l: div_4bits
  rsb .s  r2, r1, tos lsr # 3		\ if ((n>>3)>=d), B=0, C=1
  sub .cs tos, tos, r1 lsl # 3		\ if C, r -= d<<3
  adc     r0, r0, r0			\ q := (q<<1)+C
l: div_3bits
  rsb .s  r2, r1, tos lsr # 2		\ if ((n>>2)>=d), B=0, C=1
  sub .cs tos, tos, r1 lsl # 2		\ if C, r -= d<<2
  adc     r0, r0, r0			\ q := (q<<1)+C
  rsb .s  r2, r1, tos lsr # 1		\ if ((n>>1)>=d), B=0, C=1
  sub .cs tos, tos, r1 lsl # 1		\ if C, r -= d<<1
  adc     r0, r0, r0			\ q := (q<<1)+C
  rsb .s  r2, r1, tos			\ if (n>=d), B=0, C=1
  sub .cs tos, tos, r1			\ if C, r -= d
  adc .s  r0, r0, r0			\ q := (q<<1)+C, C := old q bit31
l: div_next
  b .cs   div_loop
  str     tos, [ psp ]			\ stash remainder
  mov     tos, r0			\ quot -> TOS
  next,
l: div_by_0
  mvn     tos, # 0			\ quot := -1
  str     tos, [ psp ]			\ rem := -1
  next,
end-code

[defined] u/mod [if]
: /x		\ n1 n2 -- n3 ; signed divide
  2dup xor -rot swap abs swap abs	\ -- sign u1 u2
  u/mod nip
  swap ?negate
;

: modx		\ n1 n2 -- n3 ; signed mod
  over abs swap abs			\ -- sd u1 u2
  u/mod drop
  swap ?negate				\ rem takes sign of dividend
;

: /modx		\ n1 n2 -- rem quot
  over swap  2dup xor -rot			\ -- sd sq n1 n2
  swap abs swap abs u/mod		\ -- sd sq urem uquot
  rot ?negate -rot			\ -- squot sd urem
  swap ?negate swap			\ -- srem squot
;

: TestDivX
  ds( 0 5 --) /x (-- 0 )ds
  ds( 2147483647 -1 --) /x (-- -2147483647 )ds
  ds( -2147483647 -1 --) /x (-- 2147483647 )ds
  ds( 32000 59 --) / (-- 542 )ds
  ds( -1 2147483648 --) /x (-- 0 )ds
\  ds( -1 2147483647 --) /x (-- -1 )ds	\ floored
  ds( -1 2147483647 --) /x (-- 0 )ds	\ symmetric

  ds( 1903 5 --) modx (-- 3 )ds
\  ds( 1903 -5 --) modx (-- -2 )ds	\ floored
  ds( 1903 -5 --) modx (-- 3 )ds		\ symmetric
  ds( -1903 -5 --) modx (-- -3 )ds
  ds( 2000 345 --) modx (-- 275 )ds
  ds( 0 1 --) modx (-- 0 )ds
  ds( 2147483647 2 --) modx (-- 1 )ds
  ds( 2147483647 1000000000 --) modx (-- 147483647 )ds

\  ds( -2147483647 5 --) /modx (-- 3 -429496730 )ds	\ floored
  ds( -2147483647 5 --) /modx (-- -2 -429496729 )ds	\ symmetric
  ds( -2147483647 -5 --) /modx (-- -2 429496729 )ds
\  ds( 2147483647 -5 --) /modx (-- -3 -429496730 )ds	\ floored
  ds( 2147483647 -5 --) /modx (-- 2 -429496729 )ds	\ symmetric
  ds( 32500 1 --) /modx (-- 0 32500 )ds
  ds( -8 -9 --) /modx (-- -8 0 )ds
  ds( 3 2147483647 --) /modx (-- 3 0 )ds
  ds( 2147483647 2 --) /modx (-- 1 1073741823 )ds
  ds( 2147483647 1000000000 --) /modx (-- 147483647 2 )ds
;

: PtDivX	\ -- ; one cycle of division test
  0 5  /x drop
  2147483647 -1 /x drop
  -2147483647 -1 /x drop
  32000 59 /x drop
  -1 2147483648 /x drop
  -1 2147483647 /x drop		\ symmetric

  1903 5 modx drop
  1903 -5 modx drop 		\ symmetric
  -1903 -5 modx drop
  2000 345 modx drop
  0 1 modx drop
  2147483647 2 modx drop
  2147483647 1000000000 modx drop

  -2147483647 5 /modx 2drop
  -2147483647 -5 /modx 2drop
  2147483647 -5 /modx 2drop
  32500 1 /modx 2drop
  -8 -9 /modx 2drop
  3 2147483647 /modx 2drop
  2147483647 2 /modx 2drop
  2147483647 1000000000 /modx 2drop
;

: tmx
  cr ." New division "
  ticks
  #t 0 do  PtDivx  loop
  ticks swap - . ." ms"
;

[then]

: PtDiv		\ -- ; one cycle of division test
  0 5 / drop
  2147483647 -1 / drop
  -2147483647 -1 / drop
  32000 59 / drop
  -1 2147483648 / drop
  -1 2147483647 / drop		\ symmetric

  1903 5 mod drop
  1903 -5 mod drop		\ symmetric
  -1903 -5 mod drop
  2000 345 mod drop
  0 1 mod drop
  2147483647 2 mod drop
  2147483647 1000000000 mod drop

  -2147483647 5 /mod 2drop
  -2147483647 -5 /mod 2drop
  2147483647 -5 /mod 2drop
  32500 1 /mod 2drop
  -8 -9 /mod 2drop
  3 2147483647 /mod 2drop
  2147483647 2 /mod 2drop
  2147483647 1000000000 /mod 2drop
;

: tm
  cr ." Kernel division "
  ticks
  #t 0 do  PtDiv  loop
  ticks swap - . ." ms"
;


\ *********
\ *S UM/MOD
\ *********

: TestDivU	\ --
  ds( 0 0 1 --)                           UM/Mod (-- 0 0 )ds
  ds( 1 0 1 --)                           UM/Mod (-- 0 1 )ds
  ds( 1 0 2 --)                           UM/Mod (-- 1 0 )ds
  ds( 3 0 2 --)                           UM/Mod (-- 1 1 )ds
  ds( MAX-UINT 2 UM* 2 --)                UM/Mod (-- 0 MAX-UINT )ds
  ds( MAX-UINT 2 UM* MAX-UINT --)         UM/Mod (-- 0 2 )ds
  ds( MAX-UINT MAX-UINT UM* MAX-UINT --)  UM/Mod (-- 0 MAX-UINT )ds
;

: PtDivU	\ --
  0 0 1                            UM/Mod 2drop
  1 0 1                            UM/Mod 2drop
  1 0 2                            UM/Mod 2drop
  3 0 2                            UM/Mod 2drop
  MAX-UINT 2 UM* 2                 UM/Mod 2drop
  MAX-UINT 2 UM* MAX-UINT          UM/Mod 2drop
  MAX-UINT MAX-UINT UM* MAX-UINT   UM/Mod 2drop
;

(( corrected but slooow!
proc udiv64/32	\ r0:r1/tos ; 64/32 unsigned divide: tos=quot, r0=rem
\ *G Full 64 by 32 unsigned division subroutine. It runs about
\ ** four times slower than *\fo{UDIV63/31}. *\fo{Udiv64/32}
\ ** is used for unsigned mixed magnitude divide operations such
\ ** as *\fo{UM/MOD}.
\ -- destroys R4, must not destroy R3, R6
  mov	  r4, # #32			\ loop counter
  begin,
    add .s  r1, r1, r1			\ 64 shift of dividend
    adc .s  r0, r0, r0
    sbc     r7, r7, r7			\ r7=0 if there was a carry from bit 63
    sub .s  r0, r0, tos			\ trial subtraction
    cs, if,				\ carry set = no borrow, so trial succeeds
      orr     r1, r1, # 1		\ increment quotient
    else,				\ carry clear = borrow, check higher bit
      cmp     r7, # 0			\ zero = previous carry from old bit 63
      orr .eq r1, r1, # 1		\ still succeds, increment quotient
      add .ne r0, r0, tos		\ failed, undo subtraction
    endif,
    sub .s  r4, r4, # 1
  eq, until,
  mov     tos, r1			\ move quotient
  next,
end-code
))

((
code um/modx	\ ud1 u2 -- urem uquot
\ *G Slow and short - Full 64 by 32 unsigned division subroutine.
  ldr     r0, [ psp ], # 4		\ dividend high
  ldr     r1, [ psp ]			\ dividend low
  mov	  r4, # #32			\ loop counter
  mov     r2, # 0			\ always 0
  begin,
    add .s  r1, r1, r1			\ 64 bit shift of dividend
    adc .s  r0, r0, r0
    adc     r3, r2, r2			\ preserve dividend bit 63
    sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
    adc .s  r3, r3, r3			\ success if bit 0 or 1 set
    orr .ne r1, r1, # 1
    add .eq r0, r0, tos			\ failed, undo subtraction
    sub .s  r4, r4, # 1
  eq, until,
  mov     tos, r1			\ move quotient
  str     r0, [ psp ]			\ move remainder
  next,
end-code
))
code um/modx	\ ud1 u2 -- urem uquot
\ *G Fast and big - Full 64 by 32 unsigned division subroutine.
  ldr     r0, [ psp ], # 4		\ dividend high
  ldr     r1, [ psp ]			\ dividend low
  mov     r2, # 0			\ always 0

\ 1
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 2
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 3
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 4
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 5
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 6
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 7
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 8
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 9
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 10
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 11
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 12
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 13
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 14
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 15
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 16
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 17
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 18
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 19
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 20
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 21
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 22
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 23
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 24
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 25
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 26
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 27
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 28
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 29
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 30
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 31
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction
\ 32
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  adc     r3, r2, r2			\ preserve dividend bit 63
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  orr .ne r1, r1, # 1
  add .eq r0, r0, tos			\ failed, undo subtraction

\ all done
  mov     tos, r1			\ move quotient
  str     r0, [ psp ]			\ move remainder
  next,
end-code

CODE UM/MODy	\ ud un -- urem uquot
\ *G Old bad version
  ldmfd psp ! { r0, r1 }                \ HI:LO=r0:r1=dividend, tos=divisor
  mov r3, link                          \ save link - assumes r3 not corrupted
  bl udiv63/31
  stmfd psp ! { r0 }                    \ store remainder, quotient in tos
  mov pc, r3
END-CODE

: TestDivUx	\ --
  ds( 0 0 1 --)                           UM/Modx (-- 0 0 )ds
  ds( 1 0 1 --)                           UM/Modx (-- 0 1 )ds
  ds( 1 0 2 --)                           UM/Modx (-- 1 0 )ds
  ds( 3 0 2 --)                           UM/Modx (-- 1 1 )ds
  ds( MAX-UINT 2 UM* 2 --)                UM/Modx (-- 0 MAX-UINT )ds
  ds( MAX-UINT 2 UM* MAX-UINT --)         UM/Modx (-- 0 2 )ds
  ds( MAX-UINT MAX-UINT UM* MAX-UINT --)  UM/Modx (-- 0 MAX-UINT )ds
;

: PtDivUx	\ --
  0 0 1                            UM/Modx 2drop
  1 0 1                            UM/Modx 2drop
  1 0 2                            UM/Modx 2drop
  3 0 2                            UM/Modx 2drop
  MAX-UINT 2 UM* 2                 UM/Modx 2drop
  MAX-UINT 2 UM* MAX-UINT          UM/Modx 2drop
  MAX-UINT MAX-UINT UM* MAX-UINT   UM/Modx 2drop
;

: PtDivUy	\ --
  0 0 1                            UM/Mody 2drop
  1 0 1                            UM/Mody 2drop
  1 0 2                            UM/Mody 2drop
  3 0 2                            UM/Mody 2drop
  MAX-UINT 2 UM* 2                 UM/Mody 2drop
  MAX-UINT 2 UM* MAX-UINT          UM/Mody 2drop
  MAX-UINT MAX-UINT UM* MAX-UINT   UM/Mody 2drop
;

: tu		\ --
\ 587 ms
  cr ." Old good UM/MOD "
  ticks
  #t 0 do  PtDivU  loop
  ticks swap - . ." ms"
;

: tux		\ --
\ 467 ms looped, 254 ms unrolled
  cr ." New UM/MODx "
  ticks
  #t 0 do  PtDivUx  loop
  ticks swap - . ." ms"
;

: tuy		\ --
\ 140 ms
  cr ." Old bad UM/MODy "
  ticks
  #t 0 do  PtDivUy  loop
  ticks swap - . ." ms"
;


\ ======
\ *> ###
\ ======

decimal

