\ Control file used to build uploadable applications for the MPE ARM7
\ Development Kit.

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))


only forth definitions

\ ******************************
\ Define the default directories
\ ******************************

"" ..\common"                   setmacro CommonDir      \ where the common code lives
"" ."                           setmacro CpuDir		\ where the CPU specific code lives
"" .\hardware\MpeArmDevKit"     setmacro HWDir	        \ where board specific code lives
"" .\hardware\MpeArmDevKit\Hware\Opt\Pics" setmacro PicDir        \ where pictures lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
\  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790A
  32bit-mode			\ running in 32 bit mode

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 1 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 1 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $20000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 \ Clock, serial and ticker rates
#24000000 equ system-speed	\ System clock rate in HZ.
#38400 equ console-speed	\ Serial port speed in BPS.
1 equ console-port		\ Designate serial port for terminal.
#10 equ tick-ms			\ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE               \ size of each task's data stack
$0100 equ RP-SIZE               \ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
UP-SIZE equ INTRAM		\ space used by interrupt page

$0100 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
4 equ #IRQs			\ number of IRQ stacks, shared by all IRQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted (0 is ok)


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ********************
\ Define memory layout
\ ********************

  $010000 equ link-address              \ for a binary image - usually starts at zero on the ARM

\ e27040 8bit define-emulator		\ ADK set for 27040 EPROM
  $010000 $037FFF cdata section PROG    \ Program code
  $038000 $0537FF idata section PROGd   \ IData (on target code space)
  $053800 $07AFFF udata section PROGu   \ UData/Heap
  $07B000 $07FFFF udata section PROGv   \ 64k Video Ram (uncached udata)

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************************
\ USER area and Multi tasker equates
\ **********************************

\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack
task-size #SWIs * reserve drop		\ space for SWI nesting

tib-len reserve equ INIT-TIB		\ base of TIB

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_STACK_TOP		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_STACK_BASE

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfr790A		\ LH77790A Special function registers.
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HWDir%\User\InitUser	        \ User application startup cde
  include %CpuDir%\cache790.fth         \ Cache Control Code
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel            \ high level kernel definitions
  include %CpuDir%\intARM		\ exception handlers
  include %HwDir%\hware\console.fth     \ Serial Port for FORTH Console (UART2)
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables


tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[ELSE]
  : pause ;
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\Tick790	\ timer tick
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%\softfp		\ floating point
  include %CommonDir%\softcom		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ XSHELL textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE ARM ANS ROM PowerForth v6.10"
;
external

: ANS-FORTH	\ -- ; marker
;


\ ***********************
\ Custom Hardware Drivers
\ ***********************

 include %HwDir%\hware\SmcEther.fth
 include %HWDir%\hware\Led.fth
 include %HwDir%\hware\opt\lcdqvga4.fth


 idata create image1
 data-file %PicDir%\vfx.bin
 cr . ." Bytes Loaded"
 
 idata create demo1
 data-file %PicDir%\demo1.bin
 cr . ." Bytes Loaded"

 idata create phone
 data-file %PicDir%\phone.bin
 cr . ." Bytes Loaded"

 idata create beat
 data-file %PicDir%\beat.bin
 cr . ." Bytes Loaded"

 idata create sharp
 data-file %PicDir%\sharp.bin
 cr . ." Bytes Loaded"

 idata create mpe
 data-file %PicDir%\mpe.bin
 cr . ." Bytes Loaded"
 
 idata create bottle
 data-file %PicDir%\bottle.bin
 cr . ." Bytes Loaded"

 idata create final
 data-file %PicDir%\final.bin
 cr . ." Bytes Loaded"
 
  idata create stack
  data-file %PicDir%\stack.bin
  cr . ." Bytes Loaded"

  idata create Fth6
  data-file %PicDir%\f6.bin
  cr . ." Bytes Loaded"
 
FINIS

