\ Control file to build the Ethernet/Serial Boot Loader used by the
\ MPE ARM7 Development Kit.

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


The boot code is set up to run in a 48k section of Flash copied to RAM
  $0000000 $0000:BFFF
The boot code remaps the chip select unit and segment mapper to put:
  1Mb Flash	at 0100:0000 to 010F:FFFF	Segment 1	read/write, not cached
  512k RAM	at 0000:0000 to 0007:FFFF	Segment 2	read/write, cached
  2k local SRAM	at 6000:0000 to 6000:07FF	Cache mode
  1k Ethernet	at 5000:0000 to 5000:03FF	Segment 4	read/write, not cached
The vector table is then copied to address 0.
))


only forth definitions

\ ******************************
\ Define the default directories
\ ******************************

"" ..\common"                   setmacro CommonDir      \ where the common code lives
"" ."                           setmacro CpuDir		\ where the CPU specific code lives
"" .\hardware\MpeArmDevKit"     setmacro HWDir	        \ where board specific code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
\  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790A
  32bit-mode			\ running in 32 bit mode

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 0 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 1 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 \ Clock, serial and ticker rates
#24000000 equ system-speed	\ System clock rate in HZ.
#38400 equ console-speed	\ Serial port speed in BPS.
#38400 equ console0-speed	\ Serial port speed in BPS.
#38400 equ console1-speed	\ Serial port speed in BPS.
#38400 equ console2-speed	\ Serial port speed in BPS.
2 equ console-port		\ Designate serial port for terminal.
#10 equ tick-ms			\ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0100 equ UP-SIZE               \ size of each task's user area
$0100 equ SP-SIZE               \ size of each task's data stack
$0200 equ RP-SIZE               \ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard

UP-SIZE equ INTRAM		\ space used by interrupt page

$0100 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting level for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks, shared by all IRQs (1 min)
1 equ #SWIs			\ number of SWI nestings permitted (0 is ok)


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ********************
\ Define memory layout
\ ********************

$00000000 equ link-address		\ for a binary image - usually starts at zero on the ARM
  					\ Used by the AIF header
$00000000 $0000BFFF cdata section ADKboot
$01000000 $010FFFFF cdata section PROGf	\ 1Mb of Flash
$0000C000 $0000CFFF idata section PROGd
$0000D000 $0000EFFF udata section PROGu

ADKboot PROGd PROGu  CDATA		\ use Code for HERE , and so on

interpreter
: prog adkboot ;			\ alias primary code section - PROG is default name
target


\ **********************************
\ USER area and Multi tasker equates
\ **********************************

\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack
task-size #SWIs * reserve drop		\ space for SWI nesting

tib-len reserve equ INIT-TIB		\ base of TIB

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_STACK_TOP		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_STACK_BASE

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED

((
\ Main User/Task stack for USR/SYS mode operation.
\ In order to permit fast interrupt nesting the
\ return stack must be the lowest of RSP, PSP and UP
\ In order for the initialisation code in MULTIARM.FTH
\ to work, INIT-U0 must be the highest of INIT-UO,
\ INIT-S0 and INIT-R0. 
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack

\ SVC stack ; nestable up to #SWIs
0 reserve equ SVC_StackTop		\ top of stack
task-size #SWIs * reserve equ SVC_StackBase	\ bottom of stack

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_StackTop		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_StackBase
IRQ_StackTop equ IRQ_Stack_Top
IRQ_StackBase equ IRQ_Stack_Base

\ FIQ stack
0 reserve equ FIQ_StackTop		\ top of FIQ stack
task-size reserve equ FIQ_StackBase	\ bottom of FIQ stack

\ UND stack
0 reserve equ UND_StackTop		\ top of stack
( task-size ) 0 reserve equ UND_StackBase	\ bottom of stack

\ ABT stack
0 reserve equ ABT_StackTop		\ top of stack
( task-size ) 0 reserve equ ABT_StackBase	\ bottom of stack

\ TIB - terminal input buffer  
tib-len reserve equ INIT-TIB		\ base of TIB

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
))

\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfr790A		\ LH77790A Special function registers.
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HwDir%\boot\InitBoot		\ Devkit start up code for boot loader
  include %CpuDir%\Cache790		\ Cache code
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel62		\ high level kernel definitions
  include %CpuDir%\intARM2		\ exception handlers
\  include %HwDir%\hware\console 	\ Debug Uart - channel 2
\  include %CommonDir%\cqueues		\ circular byte queues
  include %CpuDir%\drivers\Ser790i	\ Debug Uart - channel 2
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[ELSE]
  : pause ;
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\Tick790	\ timer tick
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%\softfp		\ floating point
  include %CommonDir%\softcom		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ XSHELL textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE ARM ANS ROM PowerForth v6.20 "
;
external

: ANS-FORTH	\ -- ; marker
;


\ *************************
\ Add application code here
\ *************************

interpreter
: const equ  ;
target

ProgF
  sec-base equ Flashbase
Prog
compiler
: ForceUncached  ;	\ addr -- addr'
target
interpreter
: ForceUncached  ;	\ addr -- addr'
target
   include %CpuDir%\drivers\29F040B.fth

create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $20 c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!

create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
  192 c,  168 c,  1 c,  251 c,		\ assign these as required

$50000000 equ EtherBase	\ -- addr
0 equ SMC16?	\ -- flag ; true for 16 bit access code
0 equ fastCPU?	\ -- n    ; true for fast CPU
0 equ smcDiags?	\ -- flag ; true for Ethernet diagnostics
0 equ eeprom?	\ -- flag ; true for attached EEPROM
1 equ sniff?		\ -- flag
   include %CpuDir%\drivers\smc91c9x.fth

: reboot	\ --
\ *G Reboot the CPU (equivalent to a hardware reset). This word
\ ** is used by NETBOOT.FTH if present.
  $07 $FFFFAC30 !  begin again
;

1 equ BootMenu?			\ -- n ; nz to compile boot menu
flashbase constant BootFlash	\ base address of boot Flash after mapping
$0000C000 constant BootLen	\ length of boot Flash
$00000000 constant BootRAM	\ base address of boot code after mapping
$01010000 constant userflash	\ -- addr ; base address of user flash
$00060000 constant userflashlen	\ -- n ; size of user flash
$01070000 constant dataflash	\ -- addr ; base address of data flash
$00010000 constant datalen	\ -- n ; size of data flash
\ Where applications are copied to from the user flash
$00010000 constant AppRam	\ -- addr ; application area
$00060000 constant Applen	\ -- n ; length of application area
(( \ old version
   include %CpuDir%\drivers\netboot.old.fth
))
1 equ CPU=ARM			\ if defined, selects ARM specific code
   include %CpuDir%\drivers\netcode	\ Network order and CPU dependent
   include %CpuDir%\drivers\netboot	\ Network boot loader
   
   include %HWDir%\hware\led


\ *********************
\ *S End of compilation
\ *********************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

make-turnkey bootstrap


\ ****************
\ *S Sanity checks
\ ****************

decimal

  cr ." Required USER size is     : " next-user @ .
  cr ." Current USER allocation is: " up-size .
Next-user @ up-size > [if]
\ *G Check that the USER area is large enough.
  cr ." *** Increase USER area size UP-SIZE in control file ***
  abort
[then]

\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words


\ ********
\ All done
\ ********

decimal

FINIS

