\ Control file used to build uploadable applications for the MPE ARM7
\ Development Kit.

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))


only forth definitions

\ ******************************
\ Define the default directories
\ ******************************

"" ..\common"                   setmacro CommonDir      \ where the common code lives
"" ."                           setmacro CpuDir		\ where the CPU specific code lives
"" .\hardware\MpeArmDevKit"     setmacro HWDir	        \ where board specific code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
\  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790A
  32bit-mode			\ running in 32 bit mode

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 1 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 1 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $20000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 \ Clock, serial and ticker rates
#24000000 equ system-speed	\ System clock rate in HZ.
#38400 equ console-speed	\ Serial port speed in BPS.
#38400 equ console0-speed	\ Serial port 0 speed in BPS.
#38400 equ console1-speed	\ Serial port 1 speed in BPS.
#38400 equ console2-speed	\ Serial port 2 speed in BPS.
2 equ console-port		\ Designate serial port for terminal.
#10 equ tick-ms			\ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 2 equ mpe-ver		\ y in Vx.yz
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE               \ size of each task's data stack
$0100 equ RP-SIZE               \ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
UP-SIZE equ INTRAM		\ space used by interrupt page

$0100 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks, shared by all IRQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted (0 is ok)


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ********************
\ Define memory layout
\ ********************

  $010000 equ link-address              \ for a binary image - usually starts at zero on the ARM

\ e27040 8bit define-emulator		\ ADK set for 27040 EPROM
$00010000 $0001FFFF cdata section AdkUser \ 128k Program RAM
\ $0000 in-emulator
$00020000 $0003FFFF idata section PROGd	\ 128k IData (on target code space)
$00040000 $0006FFFF udata section PROGu	\ 192k UData/Heap
$00070000 $0007FFFF udata section PROGv	\ 64k Video Ram (uncached udata)
$01000000 $010FFFFF cdata section PROGf	\ 1Mb of Flash

AdkUser PROGd PROGu  CDATA              \ use Code for HERE , and so on

interpreter
: prog adkuser ;
target


\ **********************************
\ USER area and Multi tasker equates
\ **********************************

\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack
task-size #SWIs * reserve drop		\ space for SWI nesting

tib-len reserve equ INIT-TIB		\ base of TIB

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_STACK_TOP		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_STACK_BASE

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfr790A		\ LH77790A Special function registers.
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HWDir%\User\InitUser	        \ User application startup cde
  include %CpuDir%\cache790		\ Cache Control Code
  include %CpuDir%\codeARM		\ low level kernel definitions
\  include %CommonDir%\kernel            \ high level kernel definitions
  include %CommonDir%\kernel62		\ high level kernel definitions
  include %CpuDir%\intARM		\ exception handlers
\  include %HwDir%\hware\console         \ Serial Port for FORTH Console (UART2)
  include %CpuDir%\drivers\Ser790i	\ Debug Uart - channel 2
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables


tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[ELSE]
  : pause ;
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\Tick790	\ timer tick
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%\softfp		\ floating point
  include %CommonDir%\softcom		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ XSHELL textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE ARM ANS ROM PowerForth v6.20"
;
external

: ANS-FORTH	\ -- ; marker
;


\ ***********************
\ Custom Hardware Drivers
\ ***********************

interpreter
: const equ  ;
\ *G Define this as CONSTANT to get interactive access to the
\ ** constants.
target

create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $20 c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!

create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
  192 c,  168 c,  1 c,  251 c,		\ assign these as required

$50000000 equ EtherBase	\ -- addr
0 equ SMC16?	\ -- flag ; true for 16 bit access code
0 equ fastCPU?	\ -- n    ; true for fast CPU
0 equ smcDiags?	\ -- flag ; true for Ethernet diagnostics
0 equ eeprom?	\ -- flag ; true for attached EEPROM
1 equ sniff?	\ -- flag
  include %CpuDir%\drivers\smc91c9x.fth
  include %HWDir%\hware\Led.fth


\ *********************
\ *S End of compilation
\ *********************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs


\ ****************
\ *S Sanity checks
\ ****************

decimal

  cr ." Required USER size is     : " next-user @ .
  cr ." Current USER allocation is: " up-size .
Next-user @ up-size > [if]
\ *G Check that the USER area is large enough.
  cr ." *** Increase USER area size UP-SIZE in control file ***
  abort
[then]

\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words


\ ********
\ All done
\ ********

decimal

FINIS

