\ ARM Evaluation Board (AEB) Umbilical Forth control file

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))

only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

"" ..\common16" setmacro CommonDir	\ where the common code lives
"" ."           setmacro CpuDir		\ where the CPU specific code lives
"" .\Hardware\AEB" setmacro HWDir	\ where board specific code lives
"" ..\Examples" setmacro ExampleDir	\ Examples
"" ..\MyApp"    setmacro AppDir		\ where application code lives



\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  no-heads			\ no heads needed for UF
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790A
  32bit-mode			\ running in 32 bit mode

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

\ Do we need interactive debugging over the XTL?
 1 equ umbilical?		\ true if debugging over XTL
   $40 Equ max-depth		\   max depth for UF data stack before UF barfs
   0 equ SPI?			\   true if XTL over SPI port

\ Does the initialised data area actually need to be initialised?
 0 equ init-idata?

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code


 \ Clock, serial and ticker rates
#24000000 equ system-speed	\ System clock rate in HZ.
#9600 equ console-speed		\ Serial port speed in BPS.
1 equ console-port		\ Designate serial port for terminal.
#10 equ tick-ms			\ TIMEBASE tick in ms 

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz 
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
UP-SIZE equ INTRAM		\ space used by interrupt page

\ define nesting levels for interrupts and SWIs
4 equ #IRQs			\ number of IRQ stacks, shared by all IRQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted (0 is ok)


\ *****************
\ default constants
\ *****************

0 equ false
-1 equ true


\ ********************
\ Define memory layout
\ ********************

  $000000 equ link-address              \ for a binary image - usually starts at zero on the ARM
\ e27020 8bit define-emulator		\ AEB set for 27020 EPROM
  $1000000 $101FFFF cdata section PROG
\ $0 in-emulator
  $010000 $013FFF idata section PROGd
  $014000 $017FFF udata section PROGu

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************************
\ USER area and Multi tasker equates
\ **********************************
\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack
task-size #SWIs * reserve drop		\ space for SWI nesting

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_STACK_TOP		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_STACK_BASE  

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfr790A		\ LH77790A Special function registers.
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HWDir%\InitAEB		\ ARM Evaluation Board start up code
  include %CpuDir%\MinARM		\ low level kernel definitions
  include %CommonDir%\UFkernel		\ high level kernel definitions
  include %CpuDir%\intARM		\ exception handlers
  include %CpuDir%\drivers\ser790Ap	\ LH77790A UART driver (polled)
\  include %CpuDir%\drivers\ser790Aui	\ LH77790A UART driver (interrupt RX)
  include %CpuDir%\local		\ local variables
Umbilical? [if]
  Synonym wait-byte  (serkey)		\ Say which XTL drivers to use
  Synonym send-byte  (seremit)
  Synonym Wait-Byte? (serkey?)
  Synonym Init-XTL   Init-Ser
  Include %CommonDir%\messages		\ Equates only
  Include %CommonDir%\Targend		\ Message driver
[Endif]

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\Tick790	\ timer tick
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%\softfp		\ floating point
  include %CommonDir%\softcom		\ common floating point code
[then]


\ *************
\ End of kernel
\ *************

: .CPU          \ -- ; sign on
  ." MPE H8/300H ANS "
  [ tasking?   ] [if] ." Multi-Tasking " [else] ." Single-Task " [then]
  [ Umbilical? ] [if] ." Umbilical "     [else] ." Production "  [then]
  ." Forth"
;


\ *************************
\ Add application code here
\ *************************

\ include %CpuDir%\drivers\aeb_led	\ AEB LED drivers
\ include AppDir\MyApp                  \ you can use a macro for APPDIR


\ *******************
\ Load libraries here
\ *******************

libraries
  include %CommonDIR%\UFlib		\ high level library
  include %CpuDir%\libARM		\ CPU specific library
end-libs


\ **************************************
\ Define what happens at target power up
\ **************************************

Umbilical? [if]
  make-turnkey run-umbilical            \ in TARGEND.FTH
  "" COM1" console-speed serial         \ specified required target link
  umbilical-forth                       \ start interactive session
[else]
  Make-turnkey Run                      \ application start up word
[then]

finis

