\ Atmel EB55 with AT91M55800A standalone Forth control file

((
Copyright (c) 2003
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20021001 SFP002 Changed CONSOLE-PORT to use UART0.
		Added COLDCHAIN mechanism.
20020930 SFP001 Reduced memory requirements.
		Added Serial EEPROM and DataFlash drivers.
))


\ =======
\ *! eb55
\ =======

\ *T Atmel EB55 control file

\ *S Working with the ARM Angel debugger
\ *P The Angel debugger supplied with the Atmel EB55 usually works on
\ ** the board serial Port A on UART0. The default serial port for
\ ** this configuration file is UART1 so that the MPE AIDE environment
\ ** can be used alongside the ARM tools.

\ *P Image files produced by the MPE Forth cross compiler can be downloaded
\ ** into the upper Flash area for testing as follows.
\ *B Run the Angel debugger as detailed in the Atmel and ARM
\ ** documentation. The STD/USER link must be in the STD position.
\ *B Configure the flash downloader to use a command line of the form:
\ ** "d:\xArm\arm\prog.img 0x01100000" which will put the image file in
\ ** upper section of the Flash memory.
\ *B Set the STD/USER link on the EB55 to the USER position
\ ** and reset the EB55. The board will reset with the upper half of
\ ** the Flash mapped at address 0. The supplied startup code in the
\ ** folder ARM\HARDWARE\EB55 remaps this area to address
\ ** 0100:0000h.
\ *B Run AIDE and configure PowerTerm to use the other serial port
\ ** at 38400 baud using the Properties dialog. Make the connection
\ ** and you will be able to talk to the console.

\ *S Working with BINCOM
\ *P If you are using the evaluation board with the ARM tools and exceed
\ ** the time out period, the BINCOM utility is still available. This will
\ ** load code into RAM. Write a flash downloader ... ?

\ *S Things to note
\ *N Atmel AT91 AIC Interrupt controller
\ *P The Atmel AIC interrupt controller provides vectored interrupts
\ ** through the AIC. Because of this, and because of the REMAP command
\ ** usually required to set up the chip select units and to place
\ ** RAM at address zero, the usual INITARM.FTH file is not used for
\ ** CPU initialisation, and the file INTARM.FTH for the interrupt
\ ** handlers is not required.

\ *N Atmel AT91 start up code
\ *P Each Atmel AT91 variant uses a CPU specific start up file
\ ** INITxxxx.FTH which enables enough of the chip to perform
\ ** the REMAP command and to waggle an LED pin on the Atmel
\ ** EVBs.

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

"" ..\common" setmacro CommonDir	\ where the common code lives
"" ."           setmacro CpuDir		\ where the CPU specific code lives
"" .\Hardware\EB55" setmacro HWDir	\ where board specific code lives
"" ..\Examples" setmacro ExampleDir	\ Examples
"" ..\MyApp"    setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
\  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7TDMI                      \ ARM7 processor type
  32bit-mode			\ running in 32 bit mode

0 equ false
-1 equ true


\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 1 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


 \ Clock, serial and ticker rates
#32768000 equ system-speed	\ System clock rate in HZ.
#38400 equ console-speed	\ Serial port speed in BPS.
0 equ console-port		\ Designate serial port for terminal.
#10 equ tick-ms			\ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes for TASKs
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

\ define stack and user area sizes for high level INTERRUPTs
$0100 equ INT-UP-SIZE		\ size of each interrupt's user area
$0100 equ INT-SP-SIZE		\ size of each interrupt's data stack
$0100 equ INT-RP-SIZE		\ size of each interrupt's return stack
int-up-size int-rp-size + int-sp-size +
  equ INTRAM			\ space used by interrupt

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
4 equ #IRQs			\ number of IRQ stacks, shared by all IRQs (1 min)
1 equ #SWIs			\ number of SWI nestings permitted (0 is ok)


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ **********
\ Memory map
\ **********

  $01000000 equ link-address            \ Address at which code will start\be linked at
  					\ for Angel downloader
  $01000000 $01007FFF cdata section EB55eval
  $02000000 $02003FFF idata section PROGd
  $02004000 $02007FFF udata section PROGu

romforth? [if]
  $02010000 $0201FFFF udata section PROGr
  sec-base equ APPL-ROM                 \ start of application space
  sec-top  equ APPL-ROM-END             \ end of application space
[then]

interpreter
: prog EB55eval ;
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************************
\ USER area and Multi tasker equates
\ **********************************
\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack

\ SWI stacks ; nestable up to #SWIs
task-size #SWIs * reserve drop		\ space for SWI nesting

\ IRQ stacks ; nestable up to #IRQs
\ We assume that high level interrupt handlers will be used
task-size #IRQs * equ IRQ_STACK_SIZE	\ total RAM required

0 reserve equ INT_STACK_TOP		\ top of IRQ stacks
IRQ_STACK_SIZE reserve			\ bottom of IRQ stacks
  equ IRQ_STACK_BASE

\ FIQ, ABT, UND stacks
task-size equ FIQ_STACK_SIZE
task-size equ ABT_STACK_SIZE
task-size equ UND_STACK_SIZE

FIQ_STACK_SIZE ABT_STACK_SIZE + UND_STACK_SIZE +
  reserve equ INT_STACK_BASE

\ SYStem and USER mode stacks
task-size equ SYS_STACK_SIZE
task-size equ USER_STACK_SIZE

0 reserve equ SYS/USER_STACK_TOP
SYS_STACK_SIZE USER_STACK_SIZE +
  reserve equ SYS/USER_STACK_BASE

tib-len reserve equ INIT-TIB		\ base of TIB

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfr55800a  		\ Atmel AT91M55800A Special function registers.
aif? [if]
  include %CpuDir%\AifHead		\ AIF header
[then]
  include %HWDir%\initARM55800		\ Generic 55800 startup code (*required*).
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel            \ high level kernel definitions
\  include %CpuDir%\drivers\ser55800p	\ AT91x55800A polled serial code
\  include %CpuDir%\drivers\ser55800i	\ AT91x55800A interrupt RX serial code
  include %CommonDir%\cqueues		\ Character queues
  include %CpuDir%\Drivers\ser55800qi	\ AT91x55800 interrupt queued serial driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variables

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\Tick55800a	\ timer tick
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ AIDE textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support, AIDE doesn't
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE ARM ANS ROM PowerForth v6.20 "
;
external

: ANS-FORTH	\ -- ; marker
;


\ *************************
\ Add application code here
\ *************************

\  include %HwDir%\EB55_DataFlash	\ SPI Flash
  include %HwDir%\i2cEB55drv		\ EB55 I2C specifics
  include %HwDir%\i2cBase		\ I2C base driver
  include %HwDir%\AT24C512drv		\ AT24C512 device driver


\ ****************************************************
\ The following example code can be included if wanted
\ ****************************************************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

AIF? [if]
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  cdata here link-address -             \ complete AIF header structure
    AIFimagereadonlysize !
[then]

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

FINIS

\ ======
\ *> ###
\ ======

