\ KEB50100 control file - standalone

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

N.B. The Samsung S3C4510 core is run in big-endian mode, and
the correct version of the cross compiler XARMBE.EXE must be
used with this control file.


To do
=====

Change history
==============
20020522 SFP001 First release
))

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

"" ..\common"   setmacro CommonDir	\ where the common code lives
"" ."           setmacro CpuDir		\ where the CPU specific code lives
"" .\Hardware\KEB50100" setmacro HWDir	\ where board specific code lives
"" ..\Examples" setmacro ExampleDir	\ Examples
"" ..\MyApp"    setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

  include %CpuDir%\macros		\ compiler / assembler macros

\ file: PROG.log

CROSS-COMPILE

only forth definitions			\ default search order

\  log
  no-log				\ no output log
  rommed				\ split ROM/RAM system
  align-long				\ code is 32bit aligned
  arm7tdmi				\ ARM7 core
  32bit-mode				\ running in 32 bit mode
  interactive                   	\ enter interactive mode at end
  +xrefs                        	\ enable cross references

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

$03FF0000 equ _SFRBASE_DEFAULT		\ where registers live after reset
_SFRBASE_DEFAULT equ _SFRBASE_USER	\ change if application remaps them

$03FE0000 equ _RAMBASE_DEFAULT		\ where internal RAM lives after reset
_RAMBASE_DEFAULT equ _RAMBASE_USER	\ change if application remaps them

 1 equ run-from-dram?	\ 0=from flash, nz=copy to DRAM
 0 equ ICache?		\ true to turn on ICACHE
 0 equ DCache?		\ true to turn on DCACHE, MMU, Write back buffer

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   0 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

\ System clocks
#50000000 equ system-speed	\ System clock rate in Hz.
#10000000 equ xtal-speed	\ system crystal clock rate in Hz

\ Serial ports on UARTs 1..3
#38400 equ uart0-speed		\ Serial port speed in BPS.
#38400 equ uart1-speed		\ Serial port speed in BPS.
1 equ console-port		\ Designate serial port 1 for console.

\ Timebase
#10 equ tick-ms			\ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0200 equ UP-SIZE		\ size of each task's user area
$0200 equ SP-SIZE		\ size of each task's data stack
$0400 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area

4 equ #IRQs			\ number of IRQ stacks, shared by all IRQs
4 equ #SWIs			\ number of SWI nestings permitted

UP-SIZE equ INTRAM		\ space used by interrupt page

$0100 equ TIB-LEN		\ terminal i/p buffer length


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)


\ ********************
\ Define memory layout
\ ********************

run-from-dram? [if]
  $C0000000 equ link-address			\ usually starts at zero
  $C0000000 $C001FFFF cdata section KEB		\ allow 128k of 1Mb before DRAM0
    sec-base equ CODE-START
    $20000  equ CODE-SIZE
  $C0100000 $C01FFFFF idata section DRAM0	\ 4-1Mb DRAM bank 0
    sec-base equ DRAM0-START			\ start of usable RAM
  $C0200000 $C03FFFFF udata section DRAM1	\ 4Mb DRAM bank 1
     sec-base equ DRAM1-START
[else]
  $000000 equ link-address			\ usually starts at zero
  $000000 $01FFFF cdata section KEB		\ allow 128k
    sec-base equ CODE-START
    $20000  equ CODE-SIZE
  $C0100000 $C03FFFFF idata section DRAM0	\ 4Mb DRAM bank 0
    sec-base equ DRAM0-START			\ start of usable RAM
  $C8000000 $C83FFFFF udata section DRAM1	\ 4Mb DRAM bank 1
     sec-base equ DRAM1-START
[then]

  $08000000 $0801FFFF idata section SRAMd	\ 256k SRAM on CS1
  $08020000 $0803FFFF udata section SRAMu
  $80000000 $8000FFFF udata section periph	\ 64k I/O

interpreter
: prog	keb ;				\ synonym: PROG is alias for KEB
: progd	dram0  ;			\ synonym:
: progu	dram1  ;			\ synonym:
target

romforth? [if]
  $040000 link-address + $01FFFF link-address + udata section PROGr
  sec-base equ APPL-ROM                 \ start of application space
  sec-top  equ APPL-ROM-END             \ end of application space
[then]

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************************
\ USER area and Multi tasker equates
\ **********************************
\ Assume stacks grow down: user area, sp stack, rp-stack

\ Main User/Task stack for USR/SYS mode operation.
\ In order to permit fast interrupt nesting the
\ return stack must be the lowest of RSP, PSP and UP
\ In order for the initialisation code in MULTIARM.FTH
\ to work, INIT-U0 must be the highest of INIT-UO,
\ INIT-S0 and INIT-R0.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack

\ SVC stack ; nestable up to #SWIs
0 reserve equ SVC_StackTop		\ top of stack
task-size #SWIs * reserve equ SVC_StackBase	\ bottom of stack

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_StackTop		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_StackBase

\ FIQ stack
0 reserve equ FIQ_StackTop		\ top of FIQ stack
task-size reserve equ FIQ_StackBase	\ bottom of FIQ stack

\ UND stack
0 reserve equ UND_StackTop		\ top of stack
task-size reserve equ UND_StackBase	\ bottom of stack

\ ABT stack
0 reserve equ ABT_StackTop		\ top of stack
task-size reserve equ ABT_StackBase	\ bottom of stack

\ TIB - terminal input buffer
tib-len reserve equ INIT-TIB		\ base of TIB

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfrS3C4510		\ Special function registers.
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HWDir%\InitS3C4510		\ S3C4510 start up code - H/W specific

  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel            \ high level kernel definitions
  include %CpuDir%\intARM2		\ exception handlers
  include %CpuDir%\drivers\ser4510p	\ S3C4510 polled UART driver
\  include %CommonDir%\CQueues
\  include %CpuDir%\drivers\ser4510qi	\ S3C4510 queued interrupt UART driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variable support

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\Tick4510	\ timer tick - must be after TIMEBASE.FTH
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CpuDir%\sfp32ARM		\ floating point primitives - coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ XSHELL textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE S3C4510 ANS ROM PowerForth v6.10 "
;
external

: ANS-FORTH	\ -- ; marker
;


\ *************************
\ Add application code here
\ *************************


\ ********************************************
\ The following code can be included if wanted
\ ********************************************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ *******************
\ Clean up and finish
\ *******************

libraries	\ to resolve common forward references
  include %CpuDir%\libARM
  include %CommonDir%\library
end-libs

AIF? [if]
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  cdata here link-address -             \ complete AIF header structure
    AIFimagereadonlysize !
[then]


\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

FINIS

