\ ADKUFROM.CTL - ROM only Umbilical Forth for the MPE ARM7 Development Kit

((
Copyright (c) 2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE


This control file is set up to run Umbilical Forth using
1) The MPE ADK hardware
2) no External SRAM
3) UART2 (the debug port) for serial comms
4) Code space from $00000..$7FFFF
5) Internal SRAM only from $6000.0000..$6000.07FF

))


only forth definitions

\ ******************************
\ Define the default directories
\ ******************************

"" ..\common"           setmacro CommonDir      \ where the common code lives
"" ."                   setmacro CpuDir		\ where the CPU specific code lives
"" .\hardware\TAS"	setmacro HWDir	        \ where board specific code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%\macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  no-heads			\ no word headers at all
\ interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  ARM7                          \ Core of Sharp's LH77790A
  32bit-mode			\ running in 32 bit mode

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

\ Do we need interactive debugging over the XTL?
 1 equ umbilical?		\ true if debugging over XTL
   $40 Equ max-depth		\   max depth for UF data stack before UF barfs
   0 equ SPI?			\   true if XTL over SPI port

\ Does the initialised data area actually need to be initialised?
 1 equ init-idata?		\ required for INTARM and other extensions!

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 0 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ blocks?			\ true if BLOCK needed
 $00000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets

\ Clock, serial and ticker rates
#24000000 equ system-speed	\ System clock rate in HZ.
#38400 equ console-speed	\ Serial port speed in BPS.
1 equ console-port		\ Designate serial port for terminal.
#10 equ tick-ms			\ TIMEBASE tick in ms

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0040 equ UP-SIZE		\ size of each task's user area
$00E0 equ SP-SIZE               \ size of each task's data stack
$00E0 equ RP-SIZE               \ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks, shared by all IRQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted (0 is ok)


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ********************
\ Define memory layout
\ ********************

 $000000 equ link-address		\ for a binary image - usually starts at zero on the ARM

\ e27040 8bit define-emulator		\ ADK set for 27040 EPROM
$000000 $01FFFF cdata section PROG	\ 512k Program space
\ $0000 in-emulator
$60000000 $600003FF idata section PROGd	\ 1k IData (on target code space)
$60000400 $600007FF udata section PROGu	\ 1k UData/Heap
$02000000 $021FFFFF udata section PROGDRAM

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************************
\ USER area and Multi tasker equates
\ **********************************

\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack
task-size #SWIs * reserve drop		\ space for SWI nesting

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_STACK_TOP		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_STACK_BASE

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfr790A		\ LH77790A Special function registers.
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HWDir%\ROMonly\InitROM	\ ROM only application startup cde
  include %CpuDir%\cache790.fth         \ Cache Control Code
  include %CpuDir%\MinARM		\ low level kernel definitions
  include %CommonDir%\UFkernel		\ high level kernel definitions
  include %CpuDir%\intARM		\ exception handlers
  include %HwDir%\Hware\console		\ LH77790A UART2 driver (ADK interrupt RX)
  include %CpuDir%\local		\ local variables
Umbilical? [if]
  Synonym wait-byte  (serkey)		\ Say which XTL drivers to use
  Synonym send-byte  (seremit)
  Synonym Wait-Byte? (serkey?)
  Synonym Init-XTL   Init-Ser
  Include %CommonDir%\messages		\ Equates only
  Include %CommonDir%\Targend		\ Message driver
[Endif]

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[ELSE]
  : pause ;
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\Tick790	\ timer tick
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%\sfp32ARM		\ floating point - ARM specific
  include %CommonDir%\sfp32com		\ common floating point code
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

: .CPU          \ -- ; sign on
  ." MPE ARM ANS "
  [ tasking?   ] [if] ." Multi-Tasking " [else] ." Single-Task " [then]
  [ Umbilical? ] [if] ." Umbilical "     [else] ." Production "  [then]
  ." Forth"
;


\ ***********************
\ Custom Hardware Drivers
\ ***********************

\ include %HwDir%\hware\SmcEther.fth
\ include %HWDir%\hware\Led.fth

Umbilical? [if]
  make-turnkey run-umbilical            \ in TARGEND.FTH
\ interactive finis			\ debug use only
  "" COM1" console-speed serial         \ specified required target link
  umbilical-forth                       \ start interactive session
[else]
  Make-turnkey Run                      \ application start up word
[then]
 
FINIS

