\ MPE/HTWK SA1100 module control file - standalone

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE
))

only forth definitions


\ ***********************************************************
\ Uncomment these lines if you want to use the MPE test suite
\ ***********************************************************

\ for 5-1-x tests...
\ also c-c definitions
\ required-symbol --)-later
\ previous definitions
\ ...for 5-1-x tests


\ ******************************
\ Define the default directories
\ ******************************

"" ..\common16" setmacro CommonDir	\ where the common code lives
"" ."           setmacro CpuDir		\ where the CPU specific code lives
"" .\Hardware\SA1100Module" setmacro HWDir	\ where board specific code lives
"" ..\Examples" setmacro ExampleDir	\ Examples
"" ..\MyApp"    setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

  include %CpuDir%\macros		\ compiler / assembler macros

\ file: PROG.log

CROSS-COMPILE

only forth definitions			\ default search order


  no-log				\ no output log
  rommed				\ split ROM / RAM system
  align-long				\ code is 32bit aligned
  sa-110				\ StrongARM core
  32bit-mode				\ running in 32 bit mode
  interactive                   	\ enter interactive mode at end
  +xrefs                        	\ enable cross references

0         equ false
false not equ true


\ ****************
\ Configure target
\ ****************

\ What sort of header do we need, default is memory image with no header
 0 equ AIF?			\ true for ARM AIF format

\ Kernel components
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   1 equ event-handler?		\   true to include event handler
   1 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $1000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

#24000000 equ system-speed	\ System clock rate in HZ.
#9600 equ console-speed		\ Serial port speed in BPS.
1 equ console-port		\ Designate serial port for terminal.
#10 equ tick-ms			\ TIMEBASE tick in ms 

\ version numbers
 char 6 equ mpe-rel		\ x in Vx.yz
 char 1 equ mpe-ver		\ y in Vx.yz 
 char 0 equ usrver		\ z in Vx.yz

\ define stack and user area sizes
$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
UP-SIZE equ INTRAM		\ space used by interrupt page

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
4 equ #IRQs			\ number of IRQ stacks, shared by all IRQs (1 min)
0 equ #SWIs			\ number of SWI nestings permitted (0 is ok)


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)


\ ********************
\ Define memory layout
\ ********************

AIF? [if]
  $008000 equ link-address		\ Address at which code will start\be linked at
[else]
  $000000 equ link-address		\ for a binary image - usually starts at zero on the ARM
[then]

  $000000  link-address +  $00FFFF link-address + cdata section PROG
  $0c0100000 $0c0113fff idata section PROGd
  $0c0114000 $0c0117FFF udata section PROGu
  $000 $07F cdata section AIFHEADER

romforth? [if]
  $018000 link-address + $01FFFF link-address + udata section PROGr
  sec-base equ APPL-ROM                 \ start of application space
  sec-top  equ APPL-ROM-END             \ end of application space
[then]

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************************
\ USER area and Multi tasker equates
\ **********************************
\ Assume stacks grow down: user area, sp stack, rp-stack
\ Main User/Task stack for USR/SVC operation
\ The return stack must be the lowest of RSP, PSP and UP
\ in order to permit fast interrupt nesting. In order for
\ the initialisation code in MULTIARM.FTH to work, INIT-U0
\ must be the highest.
rp-size sp-size + equ TASK-U0		\ initial offset of user area
rp-size sp-size + equ TASK-S0		\ initial offset of data stack
rp-size           equ TASK-R0		\ initial offset of return stack

task-size reserve equ INIT-T0		\ base of main task area
  init-t0 task-u0 + equ INIT-U0		\ base of main user area
  init-t0 task-s0 + equ INIT-S0		\ top of main data stack
  init-t0 task-r0 + equ INIT-R0		\ top of main return stack
task-size #SWIs * reserve drop		\ space for SWI nesting

tib-len reserve equ INIT-TIB		\ base of TIB

\ IRQ stacks ; nestable up to #IRQs
0 reserve equ IRQ_STACK_TOP		\ top of IRQ stacks
task-size #IRQs * reserve		\ bottom of IRQ stacks
  equ IRQ_STACK_BASE  

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ ************
\ Kernel files
\ ************

  include %CpuDir%\sfrSA1100		\ SA1100 Special function registers.
aif? [if]
  include %CpuDir%\AifHead
[then]
  include %CpuDir%\initARM		\ Generic startup code (*required*).
  include %HWDir%\InitSA1100		\ SA1100 start up code
  include %CpuDir%\codeARM		\ low level kernel definitions
  include %CommonDir%\kernel            \ high level kernel definitions
  include %CpuDir%\intARM		\ exception handlers
  include %CpuDir%\drivers\serSA1100p	\ SA1100 UART driver
  include %CommonDir%\devtools		\ DUMP .S etc development tools
  include %CommonDir%\voctools		\ ORDER VOCS etc
  include %CommonDir%\methods		\ target support for methods
  include %CpuDir%\local		\ local variable support

tasking? [if]
  include %CpuDir%\multiARM		\ multi-tasker, MUST be before TIMEBASE
[then]

timebase? [if]
  include %CommonDir%\timebase		\ time base common code
  include %CpuDir%\drivers\TickSA1100	\ timer tick
[then]

environment? [if]
  include %CommonDir%\environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%\heap32		\ memory allocation set
[then]

softfp? [if]
  include %CommonDir%\sfp32hi		\ floating point primitives - replace with coded version
  include %CommonDir%\sfp32com		\ common floating point code
[then]

romforth? [if]
  include %CommonDir%\RomForth\link	\ appl. rom link
  include %CommonDir%\RomForth\iodef	\ link i/o
  include %CommonDir%\RomForth\filetran	\ ascii file uploader
  include %CommonDir%\RomForth\xmodem	\ XMODEM downloader
  include %CommonDir%\RomForth\intelhex	\ Intel Hex downloader
  include %CommonDir%\RomForth\textfile	\ XSHELL textfile support
\ include %CommonDir%\RomForth\blocks	\ XSHELL blocks support
[then]

mpe-set? [if]
  include %CpuDir%\mpe_supp		\ MPE v5 compatibility word set
[then]


\ *************
\ End of kernel
\ *************

internal
: .CPU          \ -- ; display CPU type
  ." MPE StrongARM ANS ROM PowerForth v6.10 "
;
external

: ANS-FORTH	\ -- ; marker
;


\ *************************
\ Add application code here
\ *************************

\ include ...


\ ****************************************
\ The following code be included if wanted
\ ****************************************

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ *******************
\ Clean up and finish
\ *******************

libraries
  include %CommonDir%\library
end-libs


\ XREF DUP                              \ where is DUP used
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

FINIS

