\ AT49BV6416.FTH - Atmel AT49BV6416 Flash write utility

\ Copyright (c) 2006
\ MicroProcessor Engineering
\ 133 Hill Lane
\ Southampton SO15 5AF
\ England
\ 
\ tel: +44 (0)23 8063 1441
\ fax: +44 (0)23 8033 9691
\ net: mpe@mpeforth.com
\      tech-support@mpeforth.com
\ web: www.mpeforth.com
\ 
\ From North America, our telephone and fax numbers are:
\   011 44 23 8063 1441
\   011 44 23 8033 9691
\ 
\ Change history
\ ==============
\ 20061012 SFP001 First release.

only forth definitions  decimal


\ ************
\ *! 49bv6416
\ *T AT49BV6416 Flash programming
\ ************
\ *P This code is used to write to the Flash for storing
\ ** applications and data. The code is designed for use with
\ ** an Atmel AT49BV6416 device on a 16 bit bus. The size of each
\ ** sector is given by the sector table.

\ *P This code is hardware and CPU dependent because of bus width
\ ** and cache considerations. This code was tested on an Atmel
\ ** AT91RM9200-EK board.

\ *P Care must be taken when modifying or using this code. The actual primitives
\ ** used to perform the write and erase operations are specifically written to
\ ** be executed from RAM or memory that is not in the Flash itself. The
\ ** Flash devices do not handle simultaneous read/write operation.
\ ** Because the flash may still present at 0000:0000, interrupts are
\ ** disabled during programming. The flash memory is copied into RAM
\ ** by the boot code and all code execution (except for interrupt
\ ** vector despatch) occurs from RAM.

\ *P The address *\fo{FLASHBASE} must be defined before this file is
\ ** compiled. If it is not defined a base address of $1000:0000
\ ** will be used.

\ *P Compiler and Interpreter macros must be set up before
\ ** this code is compiled to force an uncached address. Some
\ ** CPUs, e.g. MIPS and some ARMs use a particular address
\ ** range to determine whether the access should be cached
\ ** or uncached. During Flash programming, ALL accesses to
\ ** the flash should be uncached and write buffers should
\ ** be turned off.
\ ** The following is for for a Samsung S3C4510B CPU.
\ *E compiler
\ ** : ForceUncached		\ addr -- addr'
\ **   bit26 or  ;
\ ** target
\ ** interpreter
\ ** : ForceUncached		\ addr -- addr'
\ **   bit26 or  ;
\ ** target

\ *P If your CPU does not have a cache or if caching is
\ ** never enabled for Flash access, use the following code.
\ *E compiler
\ ** : ForceUncached	;	\ addr -- addr'
\ ** target
\ ** interpreter
\ ** : ForceUncached	;	\ addr -- addr'
\ ** target

\ *P Note that when programming these devices, programming part
\ ** of a sector (64kb) will leave the unprogrammed part
\ ** set to bytes of FF.

decimal


\ ****************
\ *S Configuration
\ ****************
\ *P This section configures the base address of the target
\ ** Flash, the start address of the RAM that holds the
\ ** target Flash programming routine, and disables caching
\ ** of the Flash.
\ ** Edit this file before compiling it so that the target hardware
\ ** is used correctly.

[undefined] FlashBase [if]
$10000000 value FlashBase	\ default CS0
\ *G Define the default base address of the current Flash device.
[then]

[undefined] ForceUncached [if]
compiler
: ForceUncached		\ addr -- addr'
\ *G Convert a target address into its uncached form. Defaults to
\ ** *\fo{NOOP}.
  ( bit26 or )  ;
target
interpreter
: ForceUncached		\ addr -- addr'
  ( bit26 or )  ;
target
[then]


\ ========================
\ *S Flash Access routines
\ ========================

create SecTab	\ -- addr
\ *G Table containing the number of sectors and starting offset
\ ** of each sector, plus a dummy last address which enables
\ ** the size of the last sector to be calculated.
  #135 ,
  $00000000 ,  $00002000 ,  $00004000 ,  $00006000 ,
  $00008000 ,  $0000A000 ,  $0000C000 ,  $0000E000 ,
  $00010000 ,  $00020000 ,  $00030000 ,  $00040000 ,
  $00050000 ,  $00060000 ,  $00070000 ,  $00080000 ,
  $00090000 ,  $000A0000 ,  $000B0000 ,  $000C0000 ,
  $000D0000 ,  $000E0000 ,  $000F0000 ,  $00100000 ,
  $00110000 ,  $00120000 ,  $00130000 ,  $00140000 ,
  $00150000 ,  $00160000 ,  $00170000 ,  $00180000 ,
  $00190000 ,  $001A0000 ,  $001B0000 ,  $001C0000 ,
  $001D0000 ,  $001E0000 ,  $001F0000 ,  $00200000 ,
  $00210000 ,  $00220000 ,  $00230000 ,  $00240000 ,
  $00250000 ,  $00260000 ,  $00270000 ,  $00280000 ,
  $00290000 ,  $002A0000 ,  $002B0000 ,  $002C0000 ,
  $002D0000 ,  $002E0000 ,  $002F0000 ,  $00300000 ,
  $00310000 ,  $00320000 ,  $00330000 ,  $00340000 ,
  $00350000 ,  $00360000 ,  $00370000 ,  $00380000 ,
  $00390000 ,  $003A0000 ,  $003B0000 ,  $003C0000 ,
  $003D0000 ,  $003E0000 ,  $003F0000 ,  $00400000 ,
  $00410000 ,  $00420000 ,  $00430000 ,  $00440000 ,
  $00450000 ,  $00460000 ,  $00470000 ,  $00480000 ,
  $00490000 ,  $004A0000 ,  $004B0000 ,  $004C0000 ,
  $004D0000 ,  $004E0000 ,  $004F0000 ,  $00500000 ,
  $00510000 ,  $00520000 ,  $00530000 ,  $00540000 ,
  $00550000 ,  $00560000 ,  $00570000 ,  $00580000 ,
  $00590000 ,  $005A0000 ,  $005B0000 ,  $005C0000 ,
  $005D0000 ,  $005E0000 ,  $005F0000 ,  $00600000 ,
  $00610000 ,  $00620000 ,  $00630000 ,  $00640000 ,
  $00650000 ,  $00660000 ,  $00670000 ,  $00680000 ,
  $00690000 ,  $006A0000 ,  $006B0000 ,  $006C0000 ,
  $006D0000 ,  $006E0000 ,  $006F0000 ,  $00700000 ,
  $00710000 ,  $00720000 ,  $00730000 ,  $00740000 ,
  $00750000 ,  $00760000 ,  $00770000 ,  $00780000 ,
  $00790000 ,  $007A0000 ,  $007B0000 ,  $007C0000 ,
  $007D0000 ,  $007E0000 ,  $007F0000 ,  $00800000 ,

: SectorN	\ n -- addr len
\ *G Convert sector number to base address and length
  1+ cells sectab + 2@			\ -- end start
  tuck -
;

: FindSector	\ addr -- start len
\ *G Find the start and length of the sector containing address
\ ** addr. If len is zero, the sector was not found and start
\ ** is set to addr.
  flashbase - 0				\ convert to offset, assume failure
  sectab @ 0 ?do
    over i SectorN over + within if	\ test sector
      2drop i SectorN
      leave
    endif
  loop
  swap flashbase + swap			\ make offset absolute
;

: FlUnlock	\ --
\ *G Unlock the flash.
  $AA flashbase $5555 1 lshift + w!	\ unlock
  $55 flashbase $2AAA 1 lshift + w!
;

: FlCmd		\ command --
\ *G Unlock the flash and write the command.
((
  $AA flashbase $5555 1 lshift + w!	\ unlock
  $55 flashbase $2AAA 1 lshift + w!
))
  FlUnlock  flashbase $5555 1 lshift + w!
;

: FlReset	\ --
\ *G Reset device after AutoSelect command.
  $F0 flashbase $5555 1 lshift + w!	\ reset
;

: FlId		\ -- manid devid
\ *G Get manufacturer's ID, should be $1F $D6 for AT49BV6416
  $90 FlCmd
  flashbase $000 + w@
  flashbase $002 + w@
  FlReset
;

: FlCfg		\ 0/1 -- 
\ *G Set the device configuration register to 0 or 1.
  $E0 FlCmd  FlashBase w!
;

: Write16	\ w address -- flag ; 0 for success
\ *G Write a 16 bit word to the given absolute address.
  [I
  $A0 FlCmd				\ write command
  2dup w!				\ write data
  $4000 >r				\ counter
  begin
    2dup w@ <>  r@ 0<> and		\ while not same and not timed out
   while
    r> 1- >r				\ count down, go round again
  repeat
  2drop  r> 0=				\ success if counter non-zero
  I]
;

variable FlResult	\ -- addr
\ *G Holds the last two reads from a faulty erase.

: SecErase	\ secaddr -- ; erase sector
\ *G Erase the sector at the given address.
  0 FlCfg				\ configuration mode 0.
  $AA flashbase $5555 1 lshift + w!	\ unlock access
  $70 over w!				\ unlock sector
  $80 FlCmd  FlUnlock $30 over w!	\ Sector erase sequence
  dup >r  w@				\ first status read
  begin					\ -- prev
    r@ w@ tuck				\ -- curr prev curr
    xor $40 and	0=			\ -- curr flag ; done if DQ6 not toggled
    over $0028 and  or			\ -- curr flag' ; done if bit 3 or 5 set
  until
  $0028 and if
    r@ w@ #16 lshift  r@ w@ or
    Flresult !    
  endif
  r> drop
  FlReset
;

[undefined] .lword [if]
: .lword	\ x --
\ *G Display x as an 8 digit hexadecimal number in the form
\ ** "abcd:efgh".
  base @ hex  swap
  0 <# # # # # [char] : hold # # # # #> type
  base !
;
[then]

[undefined] .word [if]
: .word		\ w --
\ *G Display w as a 4 digit hexadecimal number in the form
\ ** "abcd".
  base @ hex  swap
  0 <# # # # # #> type
  base !
;
[then]

((
[undefined] .byte [if]
: .byte		\ x --
\ *G Display x as a 2 digit hexadecimal number in the form
\ ** "ab".
  base @ hex  swap
  0 <# # # #> type
  base !
;
[then]
))

: .src/dest	\ src dest -- src dest
\ *G Display the addresses and contents of the source
\ ** and destination.
  ."  source: " over .lword  over w@ ." (" .word ." )"
  ."  dest: " dup .lword  dup w@ ." (" .word ." )"
;


\ *************************
\ *S User/Application words
\ *************************
\ *P These words have heads by default and are in the target
\ ** dictionary.

\ external

: (EraseFlash)	\ dest dlen --
\ *G Erase the flash sectors covering dlen bytes at dest.
  ?dup if
    swap ForceUncached swap		\ *** force bypass of cache **
    over + 1- >r			\ -- dest ; R: -- end
    begin				\ -- addr ; R: -- end
\      cr ."  Erasing sector at: " dup .lword
      ." ."
      dup SecErase			\ erase current sector
      FindSector +			\ step to start of next sector
      dup r@ u>				\ until out of range
    until
    drop  r> drop
  endif
;

: (ProgFlash)	\ src dest len --
\ *G Write len bytes from memory at src to Flash at dest.
\ ** Note that the Flash is assumed to be erased, and that
\ ** no verification is performed except when each byte is
\ ** programmed.
  swap ForceUncached swap		\ *** force bypass of cache **
  0 ?do
    over w@ over write16 if
      cr ."  Write failed:" .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    2 + swap  2 + swap
  2 +loop
  2drop
;

: (VerifyFlash)	\ src dest len --
\ *G Verify len bytes from memory at src to Flash at dest.
  swap ForceUncached swap		\ *** force bypass of cache **
  3dup s= if  3drop exit  endif
  cr ."  Verify failed"
  0 ?do
    over w@ over w@ <> if
      cr 2 spaces .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    2 + swap  2 + swap
  2 +loop
  2drop
;

: ProgramFlash	\ src dest len --
\ *G Using the memory buffer at src, program the Flash at dest
\ ** with len bytes. The relevant Flash sectors are erased, the
\ ** Flash is programmed, and the result verified.
  cr ." Erasing " 2dup (EraseFlash)
  cr ." Programming "  3dup (ProgFlash)
  cr ." Verifying "  (VerifyFlash)
;


\ ******************
\ *S Reflash utility
\ ******************

variable #PrgErrs	\ -- addr
\ *G Holds number of erros during writes.

: (WrFl)	\ src dest len --
\ *G Write len bytes from memory at src to Flash at dest.
\ ** Note that the Flash is assumed to be erased, and that
\ ** no verification is performed except when each byte is
\ ** programmed.
  swap ForceUncached swap		\ *** force bypass of cache **
  0 ?do
    over w@ over write16
    if  #PrgErrs incr  endif
    2 + swap  2 + swap
  2 +loop
  2drop
;

: WrFl		\ --
\ *G Move 128/1024 bytes from *\fo{X-BUFFER} into the memory
\ ** pointed to by *\fo{CUR-ADDRESS}.
  X-buffer CUR-ADDRESS @ /Xblk (WrFl)	\ Program 128/1024 bytes
  /Xblk cur-address +!
;

: .X.yy		\ u --
  base @ >r decimal
  0 <# # # [char] . hold #s #> type space
  r> base !
;

: .Xresult	\ len' status --
\ *G Display the results returned by *\fo{RecvXmodem}.
\ ** See the XModem chapter for more details.
  base @ >r decimal
  cr ." Transfer "
  if ." un" endif ." successful"
  cr dup . ." bytes transferred in "
  /RXms #10 / .X.yy ." seconds"
  cr #100 /RXms */ .X.yy ." kbytes/second"
  cr #PrgErrs ? ." programming errors"
  cr
  r> base !
;

$80000 value /Reflash	\ -- len
\ *G The maximum image size handled by *\fo{Reflash} below.

: Reflash	\ --
\ *G Wipe the existing Flash image and download a new one
\ ** by Xmodem-1k transfer. The maximum size that can be programmed
\ ** can be changed in *\fo{/Reflash} above.
  Xmodem-1k
  ['] from-buffer >body @ >r		\ action-of from-buffer >r
  assign WrFl to-do from-buffer  
  Flashbase /Reflash			\ -- caddr len
  cr ." Erasing " 2dup (EraseFlash)
  #PrgErrs off  RecvXmodem .Xresult
  r> to-do from-buffer
;


\ ******
\ *> ###
\ ******

decimal
