\ AM29LV400B.FTH - AMD 29VL400B 256k*16 Flash driver in 256k*32 pair

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20020617 SFP000 First release
))


\ ************
\ *! Am29lv400
\ *T AM29LV400BB Flash programming
\ ************

\ *P This file is for programming two 29LV400BB devices organised in
\ ** 16 bit mode to form a 32 bit pair. The boot block is at the low
\ ** end of memory. The flash memory is copied into DRAM by the boot
\ ** code and all code execution (except for interrupt vector despatch)
\ ** occurs from DRAM.

\ *P The sectors are laid out at the following offsets:
\ *E Sector	size	size*2	start		end
\ ** SA0	16kb	32kb	0000:0000	0000:7FFF
\ ** SA1	8kb	16kb	0000:8000	0000:BFFF
\ ** SA2	8kb	16kb	0000:C000	0000:FFFF
\ ** SA3	32kb	64kb	0001:0000	0001:FFFF
\ ** SA4	64kb	128kb	0002:0000	0003:FFFF
\ ** SA5	64kb	128kb	0004:0000	0005:FFFF
\ ** SA6	64kb	128kb	0006:0000	0007:FFFF
\ ** SA7	64kb	128kb	0008:0000	0009:FFFF
\ ** SA8	64kb	128kb	000A:0000	000B:FFFF
\ ** SA9	64kb	128kb	000C:0000	000D:FFFF
\ ** SA10	64kb	128kb	000E:0000	000F:FFFF


decimal

((
$00000000 constant flashbase	\ -- addr
\ *G Base address of flash memory.
))

create sectab	\ -- addr
\ *G Table containing the number of sectors and starting offset
\ ** of each sector, plus a dummy start address which enables
\ ** the size of the last sector to be calculated.
  #11 ,
  $00000000 ,  $00008000 ,  $0000C000 ,  $00010000 ,
  $00020000 ,  $00040000 ,  $00060000 ,  $00080000 ,
  $000A0000 ,  $000C0000 ,  $000E0000 ,
  $00100000 ,				\ dummy sector start

: SectorN	\ n -- addr len
\ *G Convert sector number to base address and length
  1+ cells sectab + 2@			\ -- end start
  tuck -
;

: FindSector	\ addr -- start len
\ *G Find the start and length of the sector containing address
\ ** addr. If len is zero, the sector was not found and start
\ ** is set to addr.
  flashbase - 0				\ convert to offset, assume failure
  sectab @ 0 ?do
    over i SectorN over + within if	\ test sector
      2drop i SectorN
      leave
    endif
  loop
  swap flashbase + swap			\ make offset absolute
;

: FlUnlock	\ --
\ *G Unlock the flash.
  $AAAAAAAA [ flashbase $555 4 * + ] literal !	\ unlock
  $55555555 [ flashbase $2AA 4 * + ] literal !
;

: FlCmd		\ command --
\ *G Unlock the flash and write the command.
  $AAAAAAAA [ flashbase $555 4 * + ] literal !	\ unlock
  $55555555 [ flashbase $2AA 4 * + ] literal !
  [ flashbase $555 4 * + ] literal !	\ command
;

: FlReset	\ --
\ *G Reset device after AautoSelect command
  $F0F0F0F0 [ flashbase $555 4 * + ] literal !	\ reset
;

: FlId		\ -- manid devid
\ *G Get manufacturer's ID, should be $00010001 $22BA22BA.
  [I
  $90909090 FlCmd
  [ flashbase $000 4 * + ] literal @
  [ flashbase $001 4 * + ] literal @
  FlReset
  I]
;

: SecErase	\ secaddr -- ; erase sector
\ *G Erase the sector at the given address.
  [I
  $80808080 FlCmd
  FlUnlock
  $30303030 over !
  dup >r
  @				\ first status read
  begin
    r@ @ tuck xor
    $00400040 and		\ nz if DQ6 toggled
    0=
  until
  drop
  r> drop
  I]
;

: nErase	\ n --
\ *G Erase sector n of the Flash.
  SectorN drop SecErase
;

: write32	\ x addr -- flag ; 0=success
\ *G Write a single cell to Flash. Addr must be cell aligned.
  [I
  $A0A0A0A0 FlCmd
  2dup !
  #10000 0 do
    2dup @ = ?leave
  loop
  @ <>
  I]
;

[undefined] .lword [if]
: .LWORD	\ x --
  base @ hex  swap
  0 <# # # # # ascii : hold # # # # #> type
  base !
;
[then]

: .src/dest	\ src dest -- src dest
  ."  source: " over .lword  over @ ." (" .lword ." )"
  ."  dest: " dup .lword  dup @ ." (" .lword ." )"
;

: (EraseFlash)	\ dest dlen --
\ *G Erase the flash sectors covering dlen bytes at dest.
\ ** Dest must be cell aligned, and len must be a multiple number of
\ ** cells.
  ?dup if
    over + 1- >r			\ -- dest ; R: end
    begin				\ -- addr ; R: end
      cr ."  Erasing sector at: " dup .lword
      dup SecErase			\ erase current sector
      FindSector +			\ step to start of next sector
      dup r@ u>				\ until out of range
    until
    drop  r> drop
  endif
;

: (ProgFlash)	\ src dest len --
\ *G Write len bytes from memory at src to Flash at dest.
\ ** Note that the Flash is assumed to be erased, and that
\ ** no verication is performed except when each cell is
\ ** programmed. The source and destination addresses must
\ ** be cell aligned, and len must be a multiple number of
\ ** cells.
  0 ?do
    over @ over write32 if
      cr ."  Write failed:" .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    cell + swap  cell + swap
  cell +loop
  2drop
;

: (VerifyFlash)	\ src dest len --
\ *G Verify len bytes from memory at src to Flash at dest.
\ ** The source and destination addresses must
\ ** be cell aligned, and len must be a multiple number of
\ ** cells.
  3dup s= if  3drop exit  endif
  cr ."  Verify failed"
  0 ?do
    over @ over @ <> if
      cr 2 spaces .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    cell + swap  cell + swap
  cell +loop
  2drop
;

: ProgramFlash	\ src dest len --
\ *G Using the memory buffer at src, program the Flash at dest
\ ** with len bytes. The relevant Flash sectors are erased, the
\ ** Flash is programmed, and the result verified.
\ ** The source and destination addresses must be
\ ** cell aligned, and len must be a multiple number of
\ ** cells.
  cr ." Erasing " 2dup (EraseFlash)
  cr ." Programming "  3dup (ProgFlash)
  cr ." Verifying "  (VerifyFlash)
;


\ ******
\ *> ###
\ ******


