\ CS8900.fth contains the code to drive the ethernet controller

((
first issue 0.01 May 1997

Copyright (C) DelaComms Ltd. 1997
Copyright (C) Oxford Net Solutions Ltd. 2003
and
Copyright (C) MicroProcessor Engineering Ltd. 2005
		
))

$308300 constant EtherIoBase		\ start of IO space
$300000 constant EtherPPBase		\ start of packet page space

\ the status & control registers
$120	equ CS_Reg0
$120	equ CS_ISQ

$102	equ CS_Reg3
$102	equ CS_RxCFG

$124	equ CS_Reg4
$124	equ CS_RxEvent

$104	equ CS_Reg5
$104	equ CS_RxCTL

$106	equ CS_Reg7
$106	equ CS_TxCFG

$128	equ CS_Reg8
$128	equ CS_TxEvent

$108	equ CS_Reg9
$108	equ CS_TxCMD

$10A	equ CS_RegB
$10A	equ CS_BufCFG

$12C	equ CS_RegC
$12C	equ CS_BufEvent

$130	equ CS_Reg10
$130	equ CS_RxMISS

$132	equ CS_Reg12
$132	equ CS_TxCOL

$112	equ CS_Reg13
$112	equ CS_LineCTL

$134	equ CS_Reg14
$134	equ CS_LineST

$114	equ CS_Reg15
$114	equ CS_SelfCTL

$136	equ CS_Reg16
$136	equ CS_SelfST

$116	equ CS_Reg17
$116	equ CS_BusCTL

$138	equ CS_Reg18
$138	equ CS_BusST

$118	equ CS_Reg19
$118	equ CS_TestCTL

$13C	equ CS_Reg1C

\ transmit control
\ $144	equ CS_TxCMD
\ $146	equ CS_TxLen

: byteswap	\ w -- w' ; swap bytes
  $100 /mod swap $100 * +
;

\ do ethernet registers dump (byte swapped)
: RDUMP		\ addr u -- ; display u words at a-addr
  bounds				\ get limits
  do
    cr
    i .lword space			\ print address
    i 8 + i
    do
      space i w@ byteswap
      .word				\ first four words
    2
    +loop
    space				\ gap
    i $10 + i 8 +
    do
      space i w@ byteswap
      .word				\ second four words
    2
    +loop
    halt? ?leave			\ quit or halt
  $10
  +loop					\ until done
  cr  					\ restore base
;

\ do ethernet registers dump (NOT byte swapped)
: EDUMP		\ addr u -- ; display u words at a-addr
  bounds				\ get limits
  do
    cr
    i .lword space			\ print address
    i 8 + i
    do
      space i w@ \ byteswap
      .word				\ first four words
    2
    +loop
    space				\ gap
    i $10 + i 8 +
    do
      space i w@ \ byteswap
      .word				\ second four words
    2
    +loop
    halt? ?leave			\ quit or halt
  $10
  +loop					\ until done
  cr  					\ restore base
;

\ accesses to/from ethernet io space
: !Eio		\ w offset -- ; store word into offset in IO space
  swap byteswap swap
  EtherIoBase + w!
;

: @Eio		\ offset -- w ; read word from offset in IO space
  EtherIoBase + w@
  byteswap
;

\ accesses to/from ethernet io space
: !PP		\ w offset -- ; store word into offset in PacketPage space
  swap byteswap swap
  EtherPPBase + w!
;

: @PP		\ offset -- w ; read word from offset in PacketPage space
  EtherPPBase + w@
  byteswap
;

: setPP		\ bit(s) offset -- ; set bits into offset in PacketPage space
  swap over @PP or swap !PP
;

: clrPP		\ bit(s) offset -- ; clr bits into offset in PacketPage space
  swap $FFFF xor over @PP and swap !PP
;

: init-memmode	\ -- ; initialise CS8900 for memory mode access
  $0116 $0A !Eio			\ pp pointer to 1116 (reg 17)
  $0417 $0C !Eio			\ write reg 17
;


code tcs	\ -- ; write/read loop to test cs8900
  # $0116 d0 move,	\ data into d0
  # $30830a a0 movea,	\ address into a0
  # $400000 a1 movea,	\ to pulse TP
begin,
  (a1) d1 w. move,
  d0 (a0) w. move,
  (a0) d1 w. move,
again,
  next,
end-code

: init-CS8900	\ -- ;
  init-memmode			\ enable memory block interface
  $02C0 CS_LineCTL !PP		\ Rxon, Txon, AutoAUI/10baseT
;

: init-EtherRx	\ -- ;
  $0D00 CS_RxCTL !PP		\ BCast, RxOKA, RxIndividual
  $0000 CS_RxCFG !PP		\ no ints !!
;

: init-EtherTx	\ -- ;
  $0000 CS_TxCFG !PP		\ no ints !!
;

: etheradd>CS	\ addr -- ; store ethernet address into CS8900
  dup 	 w@ byteswap $0158 !PP
  2+ dup w@ byteswap $015A !PP
  2+ 	 w@ byteswap $015C !PP
;

: @RxEvent	\ -- n ; get receive frame event
  CS_RxEvent @PP
;

: IsRx?		\ -- n ; get receive frame event
  @RxEvent $0100 and 0= not
;

: RF		\ -- ; dump for now
  @RxEvent dup
  ." Event=" .word
  $0100 and
  if
    $0400 @PP			\ get RxStatus
     ."  Status="  .word
    $0402 @PP dup		\ get Rx Length
     ."  Len="  .hex
    $300404 swap edump
  endif
;

: get_ether_pkt	\ dest maxlen -- len ; read next packet into supplied buffer
\ cr ." In get_ether_pkt(), Len="
  $0402 @PP			\ get Rx Length
\  dup .hex
  min				\ only up to buffer size
  dup >r			\ save packet len for exit
  $300404 swap bounds		\ start of data / len
  ?do
    i w@ over w!		\ copy word by word
    2+				\ & bump dest address
  2
  +loop
  drop				\ lose dest address
  r>				\ retrieve length (bytes
;

\ we HAVE to use machine code loop here
\ or the chip times out & will not Tx !!!!
code (send_ether_pkt)	\ src len -- ; send packet from supplied buffer
  # 1 d7 lsr,			\ d7 = length/2
  (a7)+ a0 movea,		\ a0 -> data
  # $300A00 a1 movea,		\ a1 = dest
L: EthTx1
  (a0)+ (a1)+ w. move,		\ move words
  d7 EthTx1 dbra,		\ loop til done
  (a7)+ d7 move,		\ pop TOS
  next,
end-code

: send_ether_pkt	\ src len -- ; send packet from supplied buffer
\ cr ." In send_ether_pkt(), Len="
\  dup .hex cr
  $00C0 $0144 !PP		\ Set TxCMD
  dup $0146 !PP			\ set packet length
  begin
    $0138 @PP $0100 and	0=	\ test Rdy4TxNow bit in BusST
  while
    pause
  repeat
  (send_ether_pkt)		\ MUST be m/c or chip does NOT tx
;

: ethOff	\ -- ; disable Ether Tx & Rx
  $00C0 CS_LineCTL clrPP	\ Rxoff, Txoff 
;

: ethOn		\ -- ; enable Ether Tx & Rx
  $00C0 CS_LineCTL setPP	\ Rxon, Txon 
;

