\ CACHE9200.FTH - AT91RM9200 Cache handling

((
Copyright (c) 1997-2002, 2003, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20061027 SFP002 Enabled Async mode - wow!
20061018 SFP001 Derived from CacheSA1110.fth.
))

\ **************
\ *! Cache9200
\ *T AT91RM9200 Cache
\ **************
\ *P For CPUs that have a cache, the word *\fo{FLUSHCACHE} should be provided,
\ ** which should write back all modified data and invalidate at least
\ ** the instruction cache. Newly compiled code was written to data
\ ** space and the data cache. Invalidating the caches causes the newly
\ ** compiled code to be executed rather than the previous contents of
\ ** the instruction cache, which may contain data now modified by the
\ ** newly compiled code.

\ *P For the AT91RM9200, see the Intel and the ARM Architecture Reference
\ ** Manual, 2nd edition, ed. David Seal, chapters B2, B3 and B5.

\ *P The source code is in the file ARM\Drivers\Cache9200.FTH.

\ *P The equate *\fo{TBASE} defines the start address of the MMU
\ ** tables and must be defined before this file is compiled,
\ ** and must be on a 64k boundary.

\ *P *\b{N.B.} This set up performs no memory remapping, and
\ ** that the adress and size of the DRAM area are set in
\ ** this file. If you want to use this file with multiple
\ ** cacheable sections or with other adress assignments,
\ ** you will have to modify this file.

\ *P *\b{N.B.} For interactive use, this file must be compiled
\ ** before *\i{Kernel62.fth}. 


\ ********************
\ *S Cache desrciption
\ ********************
\ *P This section contains equates that describe the cache and the
\ ** cached area(s) of memory. In this implementation *\fo{EnableCache}
\ ** only enables caching of one region of memory defined by the
\ ** equates *\fo{DRAM-START} and *\fo{/DRAM}.

#16 kb equ /DCache	\ -- /dcache
\ *G Size of the data cache.
#16 kb equ /ICache	\ -- /icache
\ *G Size of the instruction cache.
#32 equ Dline-size	\ -- /line
\ *G Size of a Data Cache line.
#8 equ #Csegs		\ -- n
\ *G Number of segments in the caches.
#64 equ #Cindices	\ -- n
\ *G Number of indices in the caches.

$20000000 equ DRAM-START	\ --addr
\ *G Defines the start address of the DRAM which is to be cached.
#32 Mb equ /DRAM	\ -- len
\ *G Defines the size of the DRAM to be cached.
DRAM-START /DRAM + equ DRAM-END	\ --addr
\ *G Defines the end address+1 of cacheable DRAM.

\ *P The equate *\fo{TBASE} defines the start address of the MMU
\ ** tables and must be defined before this file is compiled,
\ ** and must be on a 64k boundary.

#16 kb equ /TMMU	\ -- len
\ *G Size of the MMU tables. 4Gb in 4096 1Mb sections at 4 bytes/section.
1 Mb equ /section	\ -- len
\ *G A section controls a region of this size.
$0C02 equ mmu_sect_desc	\ -- mask
\ *G Initial setting of section descriptor.
\ ** AP=%11, Domain=0, C/B=0, type=%10
$000C equ mmu_cache_mask	\ -- mask
\ *G ORred in to descriptor to enable cache for section.
\ ** Bit 3 = Cache bit, bit 2 = Buffer bit

DRAM-START equ cacheable	\ -- addr
\ *G Base address of cacheable area.
/DRAM equ /cacheable	\ -- len
\ *G size of cacheable area, which must be a unit number of sections.
/cacheable 1 Mb u/ equ #cacheable	\ -- n
\ *G Number of cacheable sections
TBASE  cacheable 1 Mb u/ cells +
  equ TCBASE	\ -- addr
\ *G Start address of cacheable MMU descriptors.

\ describe the initial state of the MMU and CACHE control register
\ N.B. not all bits are active on all CPUs
\ bit 31	iA	Async clock select
\ bit 30	nF	notFastBus select
\ bit 15	L4	1 for architecture v5+
\ bit 14	RR	1 to select predictable (round robin) replacement
\ bit 13	V	1 to put interrupt vectors at $FFFF:0000
\ bit 12	I	1 to enable instruction cache
\ bit 11	Z	1 to enable branch prediction (if implemented)
\ bit 10	F	keep 0
\ bit 9		R	ROM protection - ignored if AP=%11
\ bit 8		S	System protection - ignored if AP=%11
\ bit 7		B	1 for big-endian
\ bit 6		L	1 for current abort model - usually 1 ***
\ bit 5		D	1 to DISABLE 26 bit exceptions - usually 1 ***
\ bit 4		P	1 for 32 bit exceptions - usually 1 ***
\ bit 3		W	1 to enable write buffer
\ bit 2		C	1 to enable cache
\ bit 1		A	1 to enable alignment checking
\ bit 0		M	1 to enable MMU
\   $107D		Enable MMU with all cache etc
\   $007D		Enable MMU and ICache only
\   $1075		Enable MMU and I & DCaches not WB
\   $0075               Enable MMU and DCache not WB
$007D equ init_cp15_cr1		\ $007D required for AT91RM9200

cdata
l: ^tbase	\ holds TBASE
  TBASE ,
l: ^MMU_init	\ holds the default section descriptor
  mmu_sect_desc ,
l: ^cacheable	\ holds base address of cacheable area
  cacheable ,
l: ^tcbase	\ holds base address of cacheable descriptors
  TCBASE ,

variable CE?	\ -- addr ; holds true if cache enabled
\ *G This variable holds -1 when the cache has been enabled,
\ ** or 0 when it is disabled.
  0 CE? !
l: ^ce?
  ce? ,

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code EnableCache	\ -- ; Switch on cache & MMU for DRAM
\ *G Enable the data cache by constructing a default MMU table
\ ** and enabling it. If variable CE? is already set, no action
\ ** is taken.
  ldr     r0, ^CE?			\ do nothing if CE? <> 0
  ldr     r1, [ r0 ]
  cmp     r1, # 0
  mov .ne pc, link

\ Save interrupt state and disable both interrupts
  mrs     r7, cpsr
  orr     r0, r7, # $C0			\ set I and F bits
  msr     cpsr, _c r0

\ Disable existing cache state
  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
\  bic     r1, r1, # $1000		\ disable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ set up the CP#15 cache and MMU registers
  ldr     r1, ^tbase			\ base address of MMU
  mcr     #15 $0 r1, cr2, cr0, #0	\ write table base

  mov     r0, # $00000003		\ Domain 0 only
  mcr     #15 $0 r0, cr3, cr0, #0	\ write domain-access

  mov     r0, # $0
  mcr     #15 $0 r0, cr8, cr7, #0	\ invalidate ITB and DTB
  mcr     #15 $0 r0, cr7, cr7, #0	\ invalidate IC and DC
  mov     r0, r0			\ let MMU settle
  mov     r0, r0
  mov     r0, r0

\ construct default MMU table with no caching enabled
  ldr     r3, ^MMU_init			\ default descriptor
  mov     r0, # 0			\ section 0 for address 0
l: mmuloop
  orr     r2, r0, r3			\ r2 := section_addr + def_desc
  str     r2, [ r1 ], # 4		\ step through table
  add .s  r0, r0, # /section		\ update memory address
  b .ne   mmuloop			\ for all of memory

\ fix up the cacheable area
  ldr     r1, ^tcbase			\ base of cacheable area of MMU table
  mov     r4, # #cacheable		\ number of cacheable sections
l: fixloop
  ldr     r2, [ r1 ]			\ read current descriptor
  orr     r2, r2, # mmu_cache_mask	\ apply mask to enable caching/buffering
  str     r2, [ r1 ], # 4		\ write descriptor back
  sub .s  r4, r4, # 1
  b .ne   fixloop

\ Flush/Invalidate the caches and MMU TLBs
  mov     r0, # $0
  mcr     #15 $0 r0, cr8, cr7, #0	\ invalidate ITB and DTB
  mcr     #15 $0 r0, cr7, cr7, #0	\ invalidate IC and DC
  mov     r0, r0			\ let MMU settle
  mov     r0, r0
  mov     r0, r0

\ Enable the caches and MMU
  mrc     #15 $0 r1, cr1, cr0, #0
  orr     r1, r1, # $C0000000		\ Enable Async mode ; SFP002
  orr	  r1, r1, # init_cp15_cr1	\ Enable MMU, DCache, WB ...
  orr     r1, r1, # $1000		\ enable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ Set the Cache Enabled flag
  mvn     r1, # 0
  ldr     r0, ^CE?
  str     r1, [ r0 ]

\ restore interrupt state
  msr     cpsr, _c r7

  next,
end-code
\ ' EnableCache AtCold			\ add to initialisation chain

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code FlushCache	\ -- ; flush DCache and ICache
\ *G Flush the ICache and DCache. See the the SA-1110 Developer
\ ** Manual section 5.2.3. If the variable CE? is 0, no action
\ ** is taken.
  ldr     r0, ^CE?			\ do nothing if CE? = 0
  ldr     r1, [ r0 ]
  cmp     r1, # 0
  mov .eq pc, link

\ Clean the data cache
  mov     r0, # 0			\ index<<26 | seg<<5
  mov     r1, # 0                       \ seg
  begin,
    mov     r2, # 0                     \ index
    begin,
      mov     r0, r1 lsl # 5		\ seg<<5
      orr     r0, r0, r2 lsl # #26	\ index<<26 | seg<<5
      mcr     #15 $0 r0, cr7, cr10, 2	\ clean entry
      add     r2, r2, # 1
      cmp     r2, # #Cindices
    eq, until,
    add     r1, r1, # 1
    cmp     r1, # #Csegs
  eq, until,

\ Drain the write buffer
  mov     r0, # 0
  mcr     #15 $0 r0, cr7, cr10, #4

\ Invalidate the I/D caches
  mov	  r0, # 0
  mcr     #15 $0 r0, cr7, cr7, #0	\ invalidate I/D caches
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

  next,
end-code

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code DisableCache	\ -- ; Switch off DATA cache & MMU
\ *G Disable the data cache.
  stmfd   rsp ! { link }

  bl      ' flushcache

  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ Clear the Cache Enabled flag
  mov     r1, # 0
  ldr     r0, ^CE?
  str     r1, [ r0 ]

  ldmfd   rsp ! { pc }
end-code


\ ****************************
\ *S Diagnostics and Test code
\ ****************************
\ *P COmpilation of this code is controlled by conditional
\ ** compilation.

1 [if]

code GetCPUID	\ -- cr0
\ *G Read ID register, same as *\b{AT91F_ARM_ReadIDCode}.
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr0, cr0, 0
  next,
end-code

code GetCacheType	\ -- cr0
\ *G Read Cache Type register, same as *\b{AT91F_ARM_ReadCacheType}.
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr0, cr0, 1
  next,
end-code

code GetControl	\ -- cr1
\ *G Read control register, same as *\b{AT91F_ARM_ReadControl}.
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr1, cr0, 0
  next,
end-code

code SetControl	\ cr1 --
\ *G Write control register, same as *\b{AT91F_ARM_WriteControl}.
  mcr     #15 0 tos, cr1, cr0, 0
  ldmfd   psp ! { tos }
  next,
end-code

: +Icache	\ --
\ *G Enable the I cache.
  GetControl $1000 or SetControl
;

: -Icache	\ --
\ *G Disable the I cache.
  GetControl $1000 invert and SetControl
;

: .CacheType	\ --
\ *G Display the cache configuration.
  GetCacheType
  cr ." Type = " dup #25 rshift $F and .
  cr ." Harvard = " dup #24 rshift 1 and .
  cr ." /Dcache = " dup #18 rshift 7 and 1 swap lshift 9 lshift .
  cr ."  Dassoc = " dup #15 rshift 7 and 1 swap lshift .
  cr ."  /Dline = " dup #12 rshift 3 and 1 swap 1+ lshift . ." words"
  cr ." /Icache = " dup #6 rshift 7 and 1 swap lshift 9 lshift .
  cr ."  Iassoc = " dup #3 rshift 7 and 1 swap lshift .
  cr ."  /Iline = " dup 3 and 1 swap 1+ lshift . ." words"
  drop
;

: (testcache)	\ -- ; test primitive
\ *G Very simple speed test with loop and memory fetch.
  ticks
  #2000000 0 do
    DRAM-START @ drop
  loop
  ticks swap - cr ." time " . ." ms"
;

: testcache	\ -- ; main test
\ *G Tests performance under several cache condition settings.
  EnableCache cr ." Caches enabled " (testcache)
  DisableCache cr ." DCache disabled " (testcache)
  -Icache cr ." I/DCache disabled " (testcache)
  EnableCache
;

[then]


\ ======
\ *> ###
\ ======

decimal
