\ CACHESA1110.FTH - SA1100 StrongARM Cache handling

((
Copyright (c) 1997-2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20030704 SFP001 Modified while testing StrongBox2.
))

\ **************
\ *! CacheSA1110
\ **************
\ *T SA-1110 Cache
\ *P For CPUs that have a cache, the word FLUSHCACHE should be provided,
\ ** which should write back all modified data and invalidate at least
\ ** the instruction cache. Newly compiled code was written to data
\ ** space and the data cache. Invalidating the ICACHE causes the newly
\ ** compiled code to be executed rather than the previous contents of
\ ** the instruction cache, which may contain data now modified by the
\ ** newly compiled code.

\ *P For the SA-1110, see the Intel and the ARM Architecture Reference
\ ** Manual, 2nd edition, ed. David Seal, chapters B2, B3 and B5.

\ *P The source code is in the file ARM\CACHESA1110.FTH.

\ *P The equate TBASE defines the start address of the MMU
\ ** tables and must be defined before this file is compiled,
\ ** and must be on a 64k boundary.
\ ** Note that this set up performs no memory remapping, and
\ ** that the adress and size of the DRAM area are set in
\ ** this file. If you want to use this file with multiple
\ ** cacheable sections or with other adress assignments,
\ ** you will have to modify this file.


\ describe the Caches
$E0000000 equ ReadZeros			\ magic address for reading 0
#8 kb     equ /DCache			\ size of data cache
#16 kb    equ /ICache			\ size of ICache
#32       equ Dline-size		\ size of Data Cache line

\ Describe the DRAM
$C0000000 equ DRAM-START		\ start of DRAM
\ *G Defines the start address of the DRAM which is to be cached.
4 Mb      equ /DRAM			\ size of DRAM
\ *G Defines the size of the DRAM to be cached.
DRAM-START /DRAM + equ DRAM-END		\ end of DRAM + 1
\ *G Defines the end address+1 of cacheable DRAM.

\ Describe the MMU table area
#16 kb equ /TMMU			\ 4Gb in 4096 1Mb sections @ 4 bytes/section
1 Mb equ /section			\ section controls a region of this size
((
DRAM-END /TMMU - equ TBASE		\ base address of MMU table
))
$0C02 equ mmu_sect_desc			\ initial setting of section descriptor
					\ AP=%11, Domain=0, C/B=0, type=%10
$000C equ mmu_cache_mask		\ ORred in to desc. to enable cache for section
					\ bit 3 = Cache bit, bit 2 = Buffer bit
\ describe the cacheable area
DRAM-START equ cacheable		\ base address of cacheable area
/DRAM      equ /cacheable		\ size of cacheable area
					\ must be unit number of sections
/cacheable 1 Mb u/ equ #cacheable	\ number of cacheable sections
TBASE  cacheable 1 Mb u/ cells +
  equ TCBASE				\ start of cacheable MMU descriptors

TBASE  ReadZeros 1 Mb u/ cells +
  equ T0BASE				\ start of ReadZeros MMU descriptors


\ describe the initial state of the MMU and CACHE control register
\ N.B. not all bits are active on all CPUs
\ bit 15	L4	1 for architecture v5+
\ bit 14	RR	1 to select predictable (round robin) replacement
\ bit 13	V	1 to put interrupt vectors at $FFFF:0000
\ bit 12	I	1 to enable instruction cache
\ bit 11	Z	1 to enable branch prediction (if implemented)
\ bit 10	F	keep 0
\ bit 9		R	ROM protection - ignored if AP=%11
\ bit 8		S	System protection - ignored if AP=%11
\ bit 7		B	1 for big-endian
\ bit 6		L	1 for current abort model - usually 1 ***
\ bit 5		D	1 to DISABLE 26 bit exceptions - usually 1 ***
\ bit 4		P	1 for 32 bit exceptions - usually 1 ***
\ bit 3		W	1 to enable write buffer
\ bit 2		C	1 to enable cache
\ bit 1		A	1 to enable alignment checking
\ bit 0		M	1 to enable MMU
\   $107D		Enable MMU with all cache etc
\   $007D		Enable MMU and ICache only
\   $1075		Enable MMU and I & DCaches not WB
\   $0075               Enable MMU and DCache not WB
\ N.B. Initialisation sets up the ICache.
$007D equ init_cp15_cr1		\ $007D required for SA1110

cdata
l: ^tbase	\ holds TBASE
  TBASE ,
l: ^MMU_init	\ holds the default section descriptor
  mmu_sect_desc ,
l: ^cacheable	\ holds base address of cacheable area
  cacheable ,
l: ^tcbase	\ holds base address of cacheable descriptors
  TCBASE ,
l: ^t0base	\ holds base address of ReadZeros descriptors
  T0BASE ,

variable CE?	\ -- addr ; holds true if cache enabled
\ *G This variable holds -1 when the cache has been enabled,
\ ** or 0 when it is disabled.
  0 CE? !
l: ^ce?
  ce? ,

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code EnableCache	\ -- ; Switch on cache & MMU for DRAM
\ *G Enable the data cache by constructing a default MMU table
\ ** and enabling it. If variable CE? is already set, no action
\ ** is taken.
  ldr     r0, ^CE?			\ do nothing if CE? <> 0
  ldr     r1, [ r0 ]
  cmp     r1, # 0
  mov .ne pc, link

\ Disable existing cache state
  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
\  bic     r1, r1, # $1000		\ disable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ set up the CP#15 cache and MMU registers
  ldr     r1, ^tbase			\ base address of MMU
  mcr     #15 $0 r1, cr2, cr0, #0	\ write table base

  mov     r0, # $00000003		\ Domain 0 only
  mcr     #15 $0 r0, cr3, cr0, #0	\ write domain-access

  mov     r0, # $0
  mcr     #15 $0 r0, cr8, cr7, #0	\ flush ITB and DTB
  mcr     #15 $0 r0, cr7, cr7, #0	\ flush IC and DC
  mov     r0, r0			\ let MMU settle
  mov     r0, r0
  mov     r0, r0

\ construct default MMU table with no caching enabled
  ldr     r3, ^MMU_init			\ default descriptor
  mov     r0, # 0			\ section 0 for address 0
l: mmuloop
  orr     r2, r0, r3			\ r2 := section_addr + def_desc
  str     r2, [ r1 ], # 4		\ step through table
  add .s  r0, r0, # /section		\ update memory address
  b .ne   mmuloop			\ for all of memory

\ fix up the cacheable area
  ldr     r1, ^tcbase			\ base of cacheable area of MMU table
  mov     r4, # #cacheable		\ number of cacheable sections
l: fixloop
  ldr     r2, [ r1 ]			\ read current descriptor
  orr     r2, r2, # mmu_cache_mask	\ apply mask to enable caching/buffering
  str     r2, [ r1 ], # 4		\ write descriptor back
  sub .s  r4, r4, # 1
  b .ne   fixloop

\ fix up the ReadZeros area - this is necessary, but we only use the first 8kb,
\ so we only need to make one section cacheable.
  ldr     r1, ^T0BASE			\ base of ReadZeros area of MMU table
  ldr     r2, [ r1 ]			\ read current descriptor
  orr     r2, r2, # mmu_cache_mask	\ apply mask to enable caching/buffering
  str     r2, [ r1 ], # 4		\ write desciptor back

\ Flush/Invalidate the caches and MMU TLBs
  mov     r0, # $0
  mcr     #15 $0 r0, cr8, cr7, #0	\ flush ITB and DTB
  mcr     #15 $0 r0, cr7, cr7, #0	\ flush IC and DC
  mov     r0, r0			\ let MMU settle
  mov     r0, r0
  mov     r0, r0

\ Enable the caches and MMU
  mrc     # 15 $0 r1, cr1, cr0, #0
  orr	  r1, r1, # init_cp15_cr1	\ Enable MMU, DCache, WB ...
  orr     r1, r1, # $1000		\ enable ICache
  mcr     # 15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ Set the Cache Enabled flag
  mvn     r1, # 0
  ldr     r0, ^CE?
  str     r1, [ r0 ]

  next,
end-code

\ ' EnableCache AtCold			\ add to initialisation chain

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code FlushCache	\ -- ; flush DCache and ICache
\ *G Flush the ICache and DCache. See the the SA-1110 Developer
\ ** Manual section 5.2.3. If the variable CE? is 0, no action
\ ** is taken.
  ldr     r0, ^CE?			\ do nothing if CE? = 0
  ldr     r1, [ r0 ]
  cmp     r1, # 0
  mov .eq pc, link

\ Flush written data by reading all lines
  mov     r0, # ReadZeros		\ magic address for reading 0
  add     r1, r0, # /Dcache		\ r1 := end limit
  begin,
    ldr     r2, [ r0 ], # Dline-size	\ read a line
    teq     r0, r1
  eq, until,

\ Clean the I/D caches
  mov	  r0, # 0
\  mcr     #15 $0 r0, cr7, cr6, #0	\ invalidate D cache
\  mcr     #15 $0 r0, cr5, cr6, #0	\ invalidate I cache
  mcr     #15 $0 r0, cr7, cr7, #0	\ invalidate I/D caches
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

  next,
end-code

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code DisableCache	\ -- ; Switch off DATA cache & MMU
\ *G Disable the data cache.
  stmfd   rsp ! { link }

  bl      ' flushcache

  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ Clear the Cache Enabled flag
  mov     r1, # 0
  ldr     r0, ^CE?
  str     r1, [ r0 ]

  ldmfd   rsp ! { pc }
end-code


\ *********
\ Test code
\ *********

code GetCR0	\ -- cr0 ; ID register
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr0, cr0, 0
  next,
end-code

code GetCR1	\ -- cr1
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr1, cr0, 0
  next,
end-code

code SetCR1	\ cr1 --
  mcr     #15 0 tos, cr1, cr0, 0
  ldmfd   psp ! { tos }
  next,
end-code

: (testcache)	\ -- ; test primitive
  ticks
  #200000 0 do
    DRAM-START @ drop
  loop
  ticks swap - cr ." time " . ." ms"
;

: testcache	\ -- main test
  Start-Timers
  EnableCache cr ." Cache enabled " (testcache)
  DisableCache cr ." Cache disabled " (testcache)
  EnableCache
;

((
code -cache
\ Disable existing cache state
  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
  bic     r1, r1, # $1000		\ disable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0
  next,
end-code

code -icache
\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2
  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $1000		\ disable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
\  mov	  r0, # 0
\  mcr     #15 $0 r0, cr5, cr6, #0	\ invalidate I cache
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0
  next,
end-code

\  mrc  #15 %000 r0 Cr1 Cr0 %000		\ read CP15, CR1=control reg
\  orr  r0, r0, # bit12			\ set bit 12
\  mcr  #15 %000 r0 Cr1 Cr0 %000		\ write CP15, CR1=control reg

code +icache
\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2
\  mov	  r0, # 0
\  mcr     #15 $0 r0, cr5, cr6, #0	\ invalidate I cache

  mrc     #15 $0 r1, cr1, cr0, #0
  orr     r1, r1, # $1000		\ enable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0
  next,
end-code
))

