\ CACHESA1110.FTH - SA1100 StrongARM Cache handling

((
Copyright (c) 1997-2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20030704 SFP001 Modified while testing StrongBox2.
20030722 SBD001 Modified to set up descriptors from a table.
))

\ **************
\ *! CacheSA1110
\ **************
\ *T SA-1110 Cache
\ *P For CPUs that have a cache, the word FLUSHCACHE should be provided,
\ ** which should write back all modified data and invalidate at least
\ ** the instruction cache. Newly compiled code was written to data
\ ** space and the data cache. Invalidating the ICACHE causes the newly
\ ** compiled code to be executed rather than the previous contents of
\ ** the instruction cache, which may contain data now modified by the
\ ** newly compiled code.

\ *P For the SA-1110, see the Intel and the ARM Architecture Reference
\ ** Manual, 2nd edition, ed. David Seal, chapters B2, B3 and B5.

\ *P The source code is in the file ARM\CACHESA1110.FTH.

\ *P The equate TBASE defines the start address of the MMU
\ ** tables and must be defined before this file is compiled,
\ ** XXXX and must be on a    64K boundary. XXXX 
\ ** and must be on a 16k boundary.
\ ** The Primary descriptors start at TBASE, there is a primary 
\ ** descriptor for eack 1Mb block of memory. Each descriptor occupies
\ ** 4 bytes so there are 16Kb of descriptors.
\ ** The secondary descriptors follow the primary descriptors.
\ ** The secondary descriptors may describe Large ( 64Kb ) Small ( 4Kb )
\ ** or Tiny ( 1 Kb ) pages. Because of the way addresses are decoded
\ ** there is a 4 bit overlap in the Coarse Second level table index and
\ ** the page index with Large page descriptors, so each descriptor must
\ ** be repeated 16 times. Each descriptor describes a 64Kb block, so 
\ ** 16 are required to describes 1Mb . Therefore Large descriptors 
\ ** 16 * 16 * 4 bytes = 1Kb
\ ** Small page indexes in a coarse second level table do not overlap so
\ ** no repeat is required.
\ ** The Fine Second Level table is 4x bigger. Large descriptors must be repeated 
\ ** 64 times, a Small descriptor 4 times, but a tiny descriptor only once.
\ ** EnableCache initialises the entire first level descriptor table with
\ ** cache disabled entries for every 1Mb block; these entries are then modified 
\ ** as required by entries read from the DESCRIPTOR_TABLE. The format of
\ ** the DESCRIPTOR_TABLE is <addr> , <repeat> , <descriptor> ,
\ ** <addr> is an address in the descriptor table beginning at TBASE, 
\ ** Calculate <addr>  from the index of the 1Mb block
\ ** <Index> = <block-addr> /section U/     
\ ** Use this as index into TBASE
\ ** <addr> = <index> cells TBASE +
\ ** <descriptor> is a value to write to <addr>. this will be section
\ ** descriptor, or a pointer to a coarse or fine page table in the 
\ ** First-level descriptor format.
\ ** For entries in the First-level table <repeat> should be 1.
\ ** If the First-Level descriptor contains a Coarse or Fine page descriptor 
\ ** then the Coarse or Fine Page descriptor table must be built at TBASE +
\ ** /TMMU onward. The <repeat> entry may be used to create the required number of
\ ** duplicate Large page descriptors in Coarse tables, or Large and Small 
\ ** in Fine tables.
\ ** Note that this set up performs no memory remapping.
\ ** This file contains an example DESCRIPTOR_TABLE for 4 cached 1Mb blocks
\ ** in the primary table plus 512K in a secondary table   
\ Modifications
\	SM251 Steve Doughton 1st December 2003
\  FlushCache: The invalidate I cache line is clearly wrong, it is writing to 
\ register 5, the fault status. Modified code to invalide both D & I in one op.



\ describe the Caches
$E0000000 equ ReadZeros			\ magic address for reading 0
#8 kb     equ /DCache			\ size of data cache
#16 kb    equ /ICache			\ size of ICache
#32       equ Dline-size		\ size of Data Cache line



$0C02 equ mmu_sect_desc			\ initial setting of section descriptor
					\ AP=%11, Domain=0, C/B=0, type=%10
$0C0E equ mmu_cache_mask		\ enable cache for section
					\ AP=%11, Domain=0, type=%10
					\ bit 3 = Cache bit, bit 2 = Buffer bit

$0001 equ mmu_coarse_desc		\ coarse page table descriptor
					\ Domain=0, type=%01
$0003 equ mmu_fine_desc		\ coarse page table descriptor
					\ Domain=0, type=%11

$0FFD equ large_page_cache_mask		\ enable cache for large page
					\ AP0=%11, AP1=%11, AP2=%11, AP3=%11,
					\ type=%01
					\ bit 3 = Cache bit, bit 2 = Buffer bit
$0001 equ large_page_invalid_mask		\ enable cache for large page
					\ AP0=%00, AP1=%00, AP2=%00, AP3=%00,
					\ type=%01


\ Describe the MMU table area
#16 kb equ /TMMU			\ 4Gb in 4096 1Mb sections @ 4 bytes/section
1 Mb equ /section			\ section controls a region of this size
1 equ SectionEntries
#64 kb equ /LargePage
#16 equ LargeCoarseEntries	\ Large page entries repeated 16 times in coarse table
#64 equ LargeFineEntries	\ Large page entries repeated 64 times in fine table
#4 kb equ /SmallPage 
#1 equ SmallCoarseEntries	\ Small page entries repeated 1 times in coarse table
#4 equ SmallFineEntries		\ Small page entries repeated 4 times in fine table
#1 kb equ /TinyPage 
#1 equ TinyFineEntries		\ Tiny page entries repeated 1 times in fine table

[UNDEFINED] TBASE [IF]
DRAM-END /TMMU 2 * - equ TBASE		\ base address of MMU table
[THEN]


TBASE /TMMU + EQU 2ND-LEVEL	\ Start of 2nd level descriptors

[UNDEFINED]	DESCRIPTOR_TABLE [IF]
\ Describe the RAM
$C0000000 equ DRAM-START		\ start of DRAM
$08080000 equ SRAM-START        \ 808:0000H 512K bytes sram

cdata
L: DESCRIPTOR_TABLE
\ Format:  address-in-table,  #of repeats,  entry in table
\ First Level Descriptors
SRAM-START /section U/         cells TBASE + , SectionEntries , 2ND-LEVEL mmu_coarse_desc OR ,	
DRAM-START /section U/     DUP cells TBASE + , SectionEntries , /section *    mmu_cache_mask OR ,	
DRAM-START /section U/ 1 + DUP cells TBASE + , SectionEntries , /section *    mmu_cache_mask OR ,	
DRAM-START /section U/ 2 + DUP cells TBASE + , SectionEntries , /section *    mmu_cache_mask OR ,	
DRAM-START /section U/ 3 + DUP cells TBASE + , SectionEntries , /section *    mmu_cache_mask OR ,	
\ fix up the ReadZeros area - this is necessary, but we only use the first 8kb,
\ so we only need to make one section cacheable.
ReadZeros /section U/ DUP cells TBASE + , SectionEntries , /section *    mmu_cache_mask OR ,	
\ Second Level Descriptors - Large page: 1 section = 16 pages
2ND-LEVEL 0 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START large_page_cache_mask OR ,	
2ND-LEVEL 1 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage 1 * + large_page_cache_mask OR ,	
2ND-LEVEL 2 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage 2 * + large_page_cache_mask OR ,	
2ND-LEVEL 3 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage 3 * + large_page_cache_mask OR ,	
2ND-LEVEL 4 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage 4 * + large_page_cache_mask OR ,	
2ND-LEVEL 5 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage 5 * + large_page_cache_mask OR ,	
2ND-LEVEL 6 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage 6 * + large_page_cache_mask OR ,	
2ND-LEVEL 7 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage 7 * + large_page_cache_mask OR ,	
2ND-LEVEL #8 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #8 * + large_page_invalid_mask OR ,	
2ND-LEVEL #9 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #9 * + large_page_invalid_mask OR ,	
2ND-LEVEL #10 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #10 * + large_page_invalid_mask OR ,	
2ND-LEVEL #11 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #11 * + large_page_invalid_mask OR ,	
2ND-LEVEL #12 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #12 * + large_page_invalid_mask OR ,	
2ND-LEVEL #13 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #13 * + large_page_invalid_mask OR ,	
2ND-LEVEL #14 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #14 * + large_page_invalid_mask OR ,	
2ND-LEVEL #15 LargeCoarseEntries * cells + , LargeCoarseEntries , SRAM-START /LargePage #15 * + large_page_invalid_mask OR ,	
0 ,  \ End of table

[THEN]

cdata
L: ^DESCRIPTOR_TABLE DESCRIPTOR_TABLE ,


\ describe the initial state of the MMU and CACHE control register
\ N.B. not all bits are active on all CPUs
\ bit 15	L4	1 for architecture v5+
\ bit 14	RR	1 to select predictable (round robin) replacement
\ bit 13	V	1 to put interrupt vectors at $FFFF:0000
\ bit 12	I	1 to enable instruction cache
\ bit 11	Z	1 to enable branch prediction (if implemented)
\ bit 10	F	keep 0
\ bit 9		R	ROM protection - ignored if AP=%11
\ bit 8		S	System protection - ignored if AP=%11
\ bit 7		B	1 for big-endian
\ bit 6		L	1 for current abort model - usually 1 ***
\ bit 5		D	1 to DISABLE 26 bit exceptions - usually 1 ***
\ bit 4		P	1 for 32 bit exceptions - usually 1 ***
\ bit 3		W	1 to enable write buffer
\ bit 2		C	1 to enable cache
\ bit 1		A	1 to enable alignment checking
\ bit 0		M	1 to enable MMU
\   $107D		Enable MMU with all cache etc
\   $007D		Enable MMU and ICache only
\   $1075		Enable MMU and I & DCaches not WB
\   $0075               Enable MMU and DCache not WB
\ N.B. Initialisation sets up the ICache.
$007D equ init_cp15_cr1		\ $007D required for SA1110

cdata
l: ^tbase	\ holds TBASE
  TBASE ,
l: ^MMU_init	\ holds the default section descriptor
  mmu_sect_desc ,

variable CE?	\ -- addr ; holds true if cache enabled
\ *G This variable holds -1 when the cache has been enabled,
\ ** or 0 when it is disabled.
  0 CE? !
l: ^ce?
  ce? ,

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code EnableCache	\ -- ; Switch on cache & MMU for DRAM
\ *G Enable the data cache by constructing a default MMU table
\ ** and enabling it. If variable CE? is already set, no action
\ ** is taken.
  ldr     r0, ^CE?			\ do nothing if CE? <> 0
  ldr     r1, [ r0 ]
  cmp     r1, # 0
  mov .ne pc, link

\ Disable existing cache state
  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
\  bic     r1, r1, # $1000		\ disable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ set up the CP#15 cache and MMU registers
  ldr     r1, ^tbase			\ base address of MMU
  mcr     #15 $0 r1, cr2, cr0, #0	\ write table base

  mov     r0, # $00000003		\ Domain 0 only
  mcr     #15 $0 r0, cr3, cr0, #0	\ write domain-access

  mov     r0, # $0
  mcr     #15 $0 r0, cr8, cr7, #0	\ flush ITB and DTB
  mcr     #15 $0 r0, cr7, cr7, #0	\ flush IC and DC
  mov     r0, r0			\ let MMU settle
  mov     r0, r0
  mov     r0, r0

\ construct default MMU table with no caching enabled
  ldr     r3, ^MMU_init			\ default descriptor
  mov     r0, # 0			\ section 0 for address 0
l: mmu_default_loop
  orr     r2, r0, r3			\ r2 := section_addr + def_desc
  str     r2, [ r1 ], # 4		\ step through table
  add .s  r0, r0, # /section		\ update memory address
  b .ne   mmu_default_loop			\ for all of memory

\ fix up the cacheable area
  ldr	  r1, ^DESCRIPTOR_TABLE
  
  ldr     r2, [ r1 ], # 4		\ read address in translation table
l: mmu_copy_loop
  ldr 	  r3, [ r1 ], # 4	    \ read number of repeats
  ldr 	  r4, [ r1 ], # 4		\ read descriptor
l: mmu_repeat_loop
  str     r4, [ r2 ], # 4		\ write descriptor to translation table
  sub .s  r3, r3, # 1
  b .ne   mmu_repeat_loop
  ldr     r2, [ r1 ], # 4		\ read address in translation table
  cmp	  r2, # 0				\ check for End Of Table 
  b .ne   mmu_copy_loop

\ Flush/Invalidate the caches and MMU TLBs
  mov     r0, # $0
  mcr     #15 $0 r0, cr8, cr7, #0	\ flush ITB and DTB
  mcr     #15 $0 r0, cr7, cr7, #0	\ flush IC and DC
  mov     r0, r0			\ let MMU settle
  mov     r0, r0
  mov     r0, r0

\ Enable the caches and MMU
  mrc     # 15 $0 r1, cr1, cr0, #0
  orr	  r1, r1, # init_cp15_cr1	\ Enable MMU, DCache, WB ...
  orr     r1, r1, # $1000		\ enable ICache
  mcr     # 15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ Set the Cache Enabled flag
  mvn     r1, # 0
  ldr     r0, ^CE?
  str     r1, [ r0 ]

  next,
end-code

\ ' EnableCache AtCold			\ add to initialisation chain

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code FlushCache	\ -- ; flush DCache and ICache
\ *G Flush the ICache and DCache. See the the SA-1110 Developer
\ ** Manual section 5.2.3. If the variable CE? is 0, no action
\ ** is taken.  
\ SM251  The invalidate I cache line is clearly wrong, it is writing to 
\ register 5, the fault status. Modified code to invalide both D & I in one op.
  ldr     r0, ^CE?			\ do nothing if CE? = 0
  ldr     r1, [ r0 ]
  cmp     r1, # 0
  mov .eq pc, link

\ Flush written data by reading all lines
  mov     r0, # ReadZeros		\ magic address for reading 0
  add     r1, r0, # /Dcache		\ r1 := end limit
  begin,
    ldr     r2, [ r0 ], # Dline-size	\ read a line
    teq     r0, r1
  eq, until,

\ Clean the I/D caches
  mov	  r0, # 0
 \ mcr     #15 $0 r0, cr7, cr6, #0	\ SM251 invalidate D cache
 \ mcr     #15 $0 r0, cr5, cr6, #0	\ SM251 invalidate I cache
   mcr     #15 $0 r0, cr7, cr7, #0	\ flush IC and DC   SM251 
 
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

  next,
end-code

\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2 ; MRC = read from CP, MCR = write to CP.
code DisableCache	\ -- ; Switch off DATA cache & MMU
\ *G Disable the data cache.
  stmfd   rsp ! { link }

  bl      ' flushcache

  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0

\ Clear the Cache Enabled flag
  mov     r1, # 0
  ldr     r0, ^CE?
  str     r1, [ r0 ]

  ldmfd   rsp ! { pc }
end-code


\ *********
\ Test code
\ *********

code GetCR0	\ -- cr0 ; ID register
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr0, cr0, 0
  next,
end-code

code GetCR1	\ -- cr1
  stmfd   psp ! { tos }
  mrc     #15 0 tos, cr1, cr0, 0
  next,
end-code

code SetCR1	\ cr1 --
  mcr     #15 0 tos, cr1, cr0, 0
  ldmfd   psp ! { tos }
  next,
end-code

: (testcache)	\ -- ; test primitive
  ticks
  #200000 0 do
    DRAM-START @ drop
  loop
  ticks swap - cr ." time " . ." ms"
;

: testcache	\ -- main test
  Start-Timers
  EnableCache cr ." Cache enabled " (testcache)
  DisableCache cr ." Cache disabled " (testcache)
  EnableCache
;

((
code -cache
\ Disable existing cache state
  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $000D		\ disable DCache & MMU & WB
  bic     r1, r1, # $1000		\ disable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0
  next,
end-code

code -icache
\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2
  mrc     #15 $0 r1, cr1, cr0, #0
  bic     r1, r1, # $1000		\ disable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
\  mov	  r0, # 0
\  mcr     #15 $0 r0, cr5, cr6, #0	\ invalidate I cache
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0
  next,
end-code

\  mrc  #15 %000 r0 Cr1 Cr0 %000		\ read CP15, CR1=control reg
\  orr  r0, r0, # bit12			\ set bit 12
\  mcr  #15 %000 r0 Cr1 Cr0 %000		\ write CP15, CR1=control reg

code +icache
\ MRC/MCR cp#, op1, Rd, Crn, Crm, op2
\  mov	  r0, # 0
\  mcr     #15 $0 r0, cr5, cr6, #0	\ invalidate I cache

  mrc     #15 $0 r1, cr1, cr0, #0
  orr     r1, r1, # $1000		\ enable ICache
  mcr     #15 $0 r1, cr1, cr0, #0
  mov     r0, r0			\ let pipeline settle down
  mov     r0, r0
  mov     r0, r0
  next,
end-code
))

