\ DCCIOP.FTH - Debug Comms Channel: polled RX, polled TX

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ ==========
\ *! dccio
\ *T Debug Comms Channel (DCC)
\ ==========
\ *P The Debug Comms Channel acts as a UART that takes no pins on
\ ** the CPU. It can transfer up to 32 bits at a time, and can be
\ ** accessed by a host through the ICE registers and by the
\ ** target CPU using coprocessor instructions. The ARM Debug
\ ** module is required. It is present in all ARM7 CPUs with a
\ ** D in the description, e.g. ARM7TDMI, and all ARM9s and up.
\ ** Note that StrongARM CPUs do not contain this module.

only forth definitions


\ *****************
\ *S Initialisation
\ *****************

: initDCC	\ --
\ *G Initialise DCC comms. A *\fo{NOOP} for now.
;


\ *******************
\ *S Comms primitives
\ *******************

code readDCCcr	\ -- x
\ *G Read the DCC control register and return the value read.
\ *D Bit1  The W bit is 0 if the DCC write register can accept data
\ **       from the CPU.
\ *D Bit0  The R bit is 0 if the DCC read register is empty.
  str    tos, [ psp, # -4 ] !
  mrc    #14 0 tos cr0 cr0 0
  next,
end-code

code writeDCCdr	\ x --
\ *G Write a 32 bit value to the DCC data register.
  mcr    #14 0 tos cr1 cr0 0
  ldr    tos, [ psp ], # 4
  next,
end-code

code readDCCdr	\ -- x
\ *G Read the DCC data register and return the 32 bit value read.
  str    tos, [ psp, # -4 ] !
  mrc    #14 0 tos cr1 cr0 0
  next,
end-code

: DCCemit	\ char --
\ *G Write a character to the DCC.
  begin
    readDCCcr bit1 and 0=
  until
  writeDCCdr
;

: DCCkey?	\ -- flag
\ *G Return true if the DCC read register has a character
\ ** available to be read.
  readDCCcr bit0 and 0<>
;

: DCCkey	\ -- char
\ *G Wait until the host/debugger has written a character to
\ ** the DCC and return it.
  begin
    DCCkey? 0=
   while
[ tasking? ] [if]  pause  [then]
  repeat
  readDCCdr
;

: DCCtype	\ caddr len --
\ *G Write a string to the DCC channel.
  bounds ?do  i c@ DCCemit  loop
;

: DCCcr		\ --
\ *G Write a CR/LF pair of characters to the DCC channel.
  #13 DCCemit  #10 DCCemit
;


\ **************************
\ *S Generic i/o assignments
\ **************************

cdata
create ConsoleDCC	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for the ARM Debug Comms Channel.
  ' DCCkey ,		\ -- char ; receive char
  ' DCCkey? ,		\ -- flag ; check receive char
  ' DCCemit ,		\ -- char ; display char
  ' DCCtype ,		\ caddr len -- ; display string
  ' DCCcr ,		\ -- ; display new line

console-port -1 = [if]  consoleDCC constant console  [then]
\ *G If the equate *\fo{CONSOLE-PORT} in the control file
\ ** is set to -1, the ARM DCC will be used as the default
\ ** console.

: DCCio		\ --
\ *G Set the current task's I/O to go through the DCC channel.
  ConsoleDCC  dup ipvec !  opvec !
;


\ ======
\ *> ###
\ ======

decimal

