\ ETHER4510.FTH - driver for Samsung S3C4510B Ethernet controller

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20040801 CJD004 Fixed MAC_INTS.
20030514 SFP003 Trial for inter-packet gap.
20020831 SFP002 Faster, smaller
20020830 SFP001 Released to beta test.
20020709 SFP000 Derived from Samsung example code files
		MACINIT.C, MAC.C, MACLIB.C, MAC.H
		
))

\ ************
\ *! Ether4510
\ ************
\ *T S3C4510B Ethernet Driver Code


\ ****************
\ *S Configuration
\ ****************

interpreter
: const equ ;	\ n -- ; -- n
\ *G A synonym for EQU (default) or CONSTANT.
\ ** If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** CONST as CONSTANT, otherwise by default CONST is defined as EQU.
target

1 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.

create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $3D c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!

create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
  192 c,  168 c,  1 c,  251 c,		\ assign these as required


\ *************************
\ *S Equates and structures
\ *************************

\ *P See SFRS3C4510.FTH for a full definition of the BDMA and MAC
\ ** register addresses and offsets. This section defines many 
\ ** equates needed to build the data structures and provides
\ ** bit masks for many registers. The data originates from
\ ** the Samsung files distributed with the KEB-50100 board, and
\ ** requires build 0625 or later of the compiler.

#define  MAC_ADDR_SIZE         6     /* dst,src addr is 6 bytes each */ 

#define MaxRxFrameSize  	1520 // Rx Frame Max Size = 1520
#define MaxRxFrameDescriptors  	16   // Max number of Rx Frame Descriptors 
#define MaxRxFrameData         	16   // Max number of Rx Frame Data
#define MaxTxFrameDescriptors  	8    // Max number of Tx Frame Descriptors
#define MaxTxFrameData         	8    // Max number of Tx Frame Data

// TxRx Frame Descriptor 
#define	fOwnership_BDMA	0x80000000 // 0:CPU, 1:BDMA
#define	fOwnership_CPU	0x7fffffff // 0:CPU, 1:BDMA

// Tx Frame Descriptor's control bits
#define	PaddingMode		0x00
#define	NoPaddingMode		0x01
#define	NoCRCMode		0x02
#define	CRCMode			0x00
#define	MACTxIntEn		0x04
#define	LittleEndian		0x08
#define	BigEndian		0x00
#define	SourceAddrDecrement 	0x00
#define	SourceAddrIncrement 	0x10
#define	WidgetAlign00		0x00 // No Invalid bytes
#define	WidgetAlign01		0x01 // 1 Invalid byte
#define	WidgetAlign10		0x10 // 2 Invalid bytes
#define	WidgetAlign11		0x11 // 3 Invalid bytes

// Tx Frame descriptor's Status
#define	ExColl		0x0010	// Excessive Collision
#define	TxDeffer	0x0020	// Transmit deffered 
#define Paused		0x0040	// Paused : holding data transmission DMA to MAC
#define	IntTx		0x0080	// Interrupt on Transmit 
#define	Under		0x0100	// Underrun */
#define	DeferOn		0x0200	// Mac defers for Max_DEFERRAL:=0.32768ms 
				// for 100Mbits/s, := 3.2768ms for 10Mbits/s
#define	NCarr		0x0400	// No Carrier sense is detected during the 
				// entire transmission of a packet from SFD 
				// to CRC
#define	SQErr		0x0800	// fake collision signal didn't come from 
				// PHY for 1.6us.
#define LateColl	0x1000	// Late collision 
#define	TxPar		0x2000	// Transmit Parity Error
#define	Comp		0x4000	// MAC transmit or discards one packet
#define TxHalted	0x8000	// Transmission was halted by clearing TxEn..

// Rx Frame descriptor's Status 
#define	OvMax		0x0008	// Over Maximum Size
#define	CtlRecd		0x0020	// set if packet received is a 
				// MAC control frame.
#define	IntRx		0x0040	// Interrupt on Receive 
#define Rx10Stat	0x0080	// set if packet was received via the 
				// 10bits interface reset if packet 
				// was received via MII
#define	AlignErr	0x0100	// Alignment Error
#define	CRCErr		0x0200	// CRC error
#define	Overflow	0x0400	// MAC receive FIFO was full when it 
				// needed to store a received byte
								
#define	LongErr		0x0800	// received a frame longer than 1518 bytes 
#define	RxPar		0x2000	// MAC receive FIFO has detected a parity error
#define RxGood		0x4000	// successfully received a packet with no errors
#define RxHalted	0x8000	// Transmission was halted by clearing TxEn...

// Buffered DMA Receiver Control Register 
#define	BRxBRST		0x0000F	// BDMA Rx Burst Size * BRxBRST 
				// = Burst Data Size 16
#define	BRxSTSKO	0x00020	// BDMA Rx Stop/Skip  Frame or Interrupt(=1) 
				// case of not OWNER the current Frame 
#define	BRxMAINC	0x00040	// BDMA Rx Memory Address Inc/Dec 
#define	BRxDIE		0x00080	// BDMA Rx Every Received Frame Interrupt Enable
#define	BRxNLIE		0x00100	// BDMA Rx NULL List Interrupt Enable 
#define	BRxNOIE		0x00200	// BDMA Rx Not Owner Interrupt Enable
#define	BRxMSOIE	0x00400	// BDMA Rx Maximum Size over Interrupr Enable 
#define	BRxLittle	0x00800	// BDMA Rx Big/Little Endian 
#define	BRxBig 		0x00000	// BDMA Rx Big/Little Endian
#define BRxWA01		0x01000	// BDMA Rx Word Alignment- one invalid byte 
#define BRxWA10		0x02000	// BDMA Rx Word Alignment- two invalid byte
#define BRxWA11		0x03000	// BDMA Rx Word Alignment- three invalid byte 
#define	BRxEn		0x04000	// BDMA Rx Enable
#define	BRxRS		0x08000	// BDMA Rx Reset
#define	RxEmpty		0x10000	// BDMA Rx Buffer empty interrupt 
#define	BRxEarly	0x20000	// BDMA Rx Early notify Interrupt

// Buffered DMA	Trasmit	Control Register(BDMATXCON) 
#define	BTxBRST		0x0000F	// BDMA Tx Burst Size = 16 
#define	BTxSTSKO	0x00020	// BDMA Tx Stop/Skip Frame or Interrupt in case
				// of not Owner the current frame 
#define	BTxCPIE		0x00080	// BDMA Tx Complete to send control 
				// packet Enable
#define	BTxNOIE		0x00200	// BDMA Tx Buffer Not Owner
#define	BTxEmpty	0x00400	// BDMA Tx Buffer Empty Interrupt 

// BDMA Tx buffer can be moved to the MAC Tx IO when the new frame comes in. 
#define	BTxMSL000	0x00000	// No wait to fill the BDMA 
#define	BTxMSL001	0x00800	// wait to fill 1/8 of the BDMA 
#define	BTxMSL010	0x01000	// wait to fill 2/8 of the BDMA
#define	BTxMSL011	0x01800	// wait to fill 3/8 of the BDMA
#define	BTxMSL100	0x02000	// wait to fill 4/8 of the BDMA
#define	BTxMSL101	0x02800	// wait to fill 5/8 of the BDMA
#define	BTxMSL110	0x03000	// wait to fill 6/8 of the BDMA
#define	BTxMSL111	0x03800	// wait to fill 7/8 of the BDMA
#define	BTxEn		0x04000	// BDMA Tx Enable 
#define	BTxRS		0x08000	// BDMA Rx Reset 

// BDMA Status Register 
#define	S_BRxRDF	0x00001	// BDMA Rx Done Every Received Frame 
#define	S_BRxNL		0x00002	// BDMA Rx NULL List 
#define	S_BRxNO		0x00004	// BDMA Rx Not Owner 
#define	S_BRxMSO	0x00008	// BDMA Rx Maximum Size Over 
#define	S_BRxEmpty	0x00010	// BDMA Rx Buffer Empty 
#define	S_BRxSEarly 	0x00020	// Early Notify 
#define	S_BRxFRF	0x00080 // One more frame data in BDMA receive buffer 
#define	S_BTxCCP	0x10000	// BDMA Tx Complete to send Control Packet 
#define	S_BTxNL		0x20000	// BDMA Tx Null List 
#define	S_BTxNO		0x40000	// BDMA Tx Not Owner
#define	S_BTxEmpty	0x100000 // BDMA Tx Buffer Empty 

// MAC Control Register 
#define	HaltReg		0x0001	// stop transmission and reception 
				// after completion of ant current packets 
#define	HaltImm		0x0002	// Stop transmission and reception immediately 
#define	SwReset 	0x0004 	// reset all Ethernet controller state machines
				// and FIFOs 
#define	FullDup		0x0008	// allow transmission to begin while reception
				// is occurring 
#define	MACLoop		0x0010	// MAC loopback
#define	ConnM00		0x0000	// Automatic-default 
#define	ConnM01		0x0020	// Force 10Mbits endec
#define	ConnM10		0x0040	// Force MII (rate determined by MII clock 
#define	MIIOFF		0x0040	// Force MII (rate determined by MII clock 
#define	Loop10		0x0080	// Loop 10Mbps 
#define	MissRoll	0x0400	// Missed error counter rolled over 
#define	MDCOFF		0x1000	// MII Station Management Clock Off
#define	EnMissRoll	0x2000	// Interrupt when missed error counter rolls 
				// over 
#define	Link10		0x8000	// Link status 10Mbps 

// CAM control register(CAMCON) 
#define	StationAcc	0x0001	// Accept any packet with a unicast station 
				// address 
#define	GroupAcc	0x0002	// Accept any packet with multicast-group 
				// station address  
#define	BroadAcc	0x0004	// Accept any packet with a broadcast station
				// address 
#define	NegCAM		0x0008	// 0: Accept packets CAM recognizes, 
				//    reject others
				// 1: reject packets CAM recognizes, 
				//    accept others 
#define	CompEn		0x0010	// Compare Enable mode

// Transmit Control Register(MACTXCON)
#define	TxEn		0x0001	// transmit Enable 
#define TxHalt		0x0002	// Transmit Halt Request 
#define	NoPad		0x0004	// suppress Padding 
#define	NoCRC		0x0008	// Suppress CRC 
#define	FBack		0x0010	// Fast Back-off
#define	NoDef		0x0020	// Disable the defer counter
#define	SdPause		0x0040	// Send Pause
#define	MII10En		0x0080	// MII 10Mbps mode enable
#define	EnUnder		0x0100	// Enable Underrun
#define	EnDefer		0x0200	// Enable Deferral
#define	EnNCarr		0x0400	// Enable No Carrier 
#define	EnExColl	0x0800	// interrupt if 16 collision occur 
				// in the same packet 
#define EnLateColl	0x1000	// interrupt if collision occurs after 
				// 512 bit times(64 bytes times) 
#define	EnTxPar		0x2000	// interrupt if the MAC transmit FIFO 
				// has a parity error 
#define	EnComp		0x4000	// interrupt when the MAC transmits or 
				// discards one packet 

// Transmit Status Register(MACTXSTAT)
#define	ExColl		0x0010	// Excessive collision 
#define	TxDeffered	0x0020	// set if 16 collisions occur for same packet
#define	Paused		0x0040	// packet waited because of pause during 
				// transmission 
#define	IntTx		0x0080	// set if transmission of packet causes an 
				// interrupt condiftion 
#define	Under		0x0100	// MAC transmit FIFO becomes empty during 
				// transmission 
#define DeferOn		0x0200	// MAC defers for MAC deferral 
#define	NCarr		0x0400	// No carrier sense detected during the 
				// transmission of a packet 
#define	SQE		0x0800	// Signal Quality Error
#define	LateColl	0x1000	// a collision occures after 512 bit times 
#define	TxPar		0x2000	// MAC transmit FIFO has detected a parity error
#define	Comp		0x4000	// MAC transmit or discards one packet 
#define	TxHalted	0x8000	// Transmission was halted by clearing 
				// TxEn or Halt immedite 

// Receive Control Register (MACRXCON) 
#define	RxEn		0x0001
#define	RxHalt		0x0002
#define	LongEn		0x0004
#define	ShortEn		0x0008
#define	StripCRC	0x0010
#define	PassCtl		0x0020
#define	IgnoreCRC	0x0040
#define	EnAlign		0x0100
#define	EnCRCErr	0x0200
#define	EnOver		0x0400
#define	EnLongErr	0x0800
#define	EnRxPar		0x2000
#define	EnGood		0x4000

// Receive Status Register(MACRXSTAT)
#define MCtlRecd	0x0020
#define	MIntRx		0x0040
#define	MRx10Stat	0x0080
#define	MAllignErr	0x0100
#define	MCRCErr		0x0200
#define	MOverflow	0x0400
#define	MLongErr	0x0800
#define	MRxPar		0x2000
#define	MRxGood		0x4000
#define	MRxHalted	0x8000

struct /FrameDescriptor	\ -- len
\ *G Tx/Rx common descriptor structure 
  int FD.FrameDataPtr
  int FD.Reserved		// cf: RX-reserved, TX-Reserved
  				// (25bits) + Control bits(7bits)
  int FD.StatusAndFrameLength
  int FD.NextFrameDescriptor
end-struct

struct /ETH_HEADER	\ -- len
\ *G MAC Frame Structure.
  6 field DestinationAddr
  6 field SourceAddr
  2 field LengthOrType
end-struct

struct /MACFrame	\ -- len
  /ETH_HEADER field MF.Header
        #1506 field MF.LLCData
end-struct

struct /MACTxStatus	\ -- len
\ *G MAC Tx Error Structure.
  int MTS.MacTxGood
  int MTS.ExCollErr
  int MTS.TxDeferredErr
  int MTS.sPaused
  int MTS.UnderErr
  int MTS.DeferErr
  int MTS.NCarrErr
  int MTS.sSQE
  int MTS.LateCollErr
  int MTS.TxParErr
  int MTS.sTxHalted
end-struct

struct /MACRxStatus	\ -- len
\ *G MAC Rx Error Structure.
  int MRS.OvMaxSize
  int MRS.sCtlRecd
  int MRS.sRx10Stat
  int MRS.AllgnErr
  int MRS.sCRCErr
  int MRS.OverflowErr
  int MRS.sLongErr
  int MRS.RxParErr
  int MRS.sRxHalted
end-struct

struct /BDMATxStatus	\ -- len
\ *G BDMA Tx Error Structure.
  int BTS.BTxNLErr
  int BTS.BTxNOErr
  int BTS.BTxEmptyErr
end-struct

struct /BDMARxStatus	\ -- len
\ *G BDMA Rx Error Structure.
  int BRS.BdmaRxCnt
  int BRS.BdmaRxGood
  int BRS.BRxNLErr
  int BRS.BRxNOErr
  int BRS.BRxMSOErr
end-struct


\ **********
\ *S Globals
\ **********

BTxBRST BTxMSL110 or BTxSTSKO or
  value gBdmaTxCon	\ -- mask
\ *G Holds the value used to set the BdmaTxCon register.

EnComp value gMacTxCon	\ -- mask
\ *G Holds the value used to set the MacTxCon register

BRxDIE      BRxEn or    BRxBig or   BRxMAINC or
BRxBRST or  BRxNLIE or  BRxNOIE or  BRxSTSKO or
  ( value ) equ gBdmaRxCon	\ -- mask
\ *G Holds the value used to set the BdmaRxCon register.

RxEn StripCRC or
  ( value ) equ gMacRxCon	\ -- mask
\ *G Holds the value used to set the MacRxCon register

0 value gCTxFDPtr	\ -- addr
\ *G Current (to be used by BDMA) transmit frame descriptor.
0 value gWTxFDPtr
\ *G Current (to be used by user write) transmit frame descriptor.
0 value gCRxFDPtr	\ -- addr
\ *G Current (to be used by BDMA) receive frame descriptor.
0 value gRRxFDPtr	\ -- addr
\ *G Current (to be used by user read) receive frame descriptor.

0 value gErrorPacketCnt	\ -- n
\ *G Returns the number of receive errors.

0 value RxStopped	\ -- flag
\ *G Set true when receiver has been stopped because 
\ ** all available buffers are full.

\ Global variable structures to store status
idata
create gsMacTxStatus   /MACTxStatus allot&erase
create gsMacRxStatus   /MACRxStatus allot&erase
\ create gsBdmaTxStatus  /BDMATxStatus allot&erase
create gsBdmaRxStatus  /BDMARxStatus allot&erase
cdata

/FrameDescriptor MaxRxFrameDescriptors *
  buffer: RxFDBaseAddr	\ -- addr
\ *G Rx Frame Descriptors

/FrameDescriptor MaxTxFrameDescriptors *
  buffer: TxFDBaseAddr	\ -- addr
\ *G Tx Frame Descriptors.

/MACFrame MaxRxFrameData *
  buffer: RxFBABaseAddr	\ -- addr
\ *G Rx Frame Buffer.

/MACFrame MaxTxFrameData *
  buffer: TxFBABaseAddr	\ -- addr
\ *G Tx Frame Buffer


\ ********************
\ MAC address handling
\ ********************

: ua@		\ addr -- n ; unaligned address 32-bit fetch
  dup c@
  8 lshift  swap 1+ tuck  c@ or
  8 lshift  swap 1+ tuck  c@ or
  8 lshift  swap 1+       c@ or
;

: uaw@		\ addr -- n ; unaligned address 16 bit fetch
  dup c@ 8 lshift  swap 1+ c@ or
;

: SetCam	\ macaddr n --
\ *G Use the MAC address at MACADDR, put in slot n of the CAM
\ ** and enable CAM entry n.
\ ** N.B. relies on BIG-ENDIAN operation and the MAC address
\ ** must be on a 2-byte boundary.
  dup >r				\ preserve entry number
  6 * _BDMA CAM + +			\ base address of CAM entry
  dup 2 and if				\ if halfword aligned ; -- macaddr camaddr
    2-					\ step back to cell boundary
    over w@				\ first half word of MAC in low word
    over @ $FFFF0000 and or  over !
    4 + swap 2+ ua@ swap !		\ last 4 bytes of MAC address
  else
    over ua@ over !			\ first cell
    4 + swap 4 + w@ #16 lshift		\ last two bytes
    over @ $FFFF and or swap !
  endif
  $0001 r> lshift  _mac CAMEN + or!	\ enable CAM[n]
;

: ResetCam	\ n --
\ *G Disable entry n in the CAM.
  $0001 swap lshift  _mac CAMEN + bic!	\ disable CAM[n]
;


\ **********
\ PHY Access
\ **********

\ STACON bits
0x20 	const PHYHWADDR		\ 1 << 5
bit10   const PHYREGWRITE
bit11	const MiiBusy

PHYHWADDR PHYREGWRITE or MiiBusy or
  const WRPHY	\ -- n ; PHY write command
PHYHWADDR MiiBusy or
  const RDPHY	\ -- n ; PHY read command

\ PHY registers
0x00 const PHY_CNTL_REG
0x01 const PHY_STATUS_REG
0x02 const PHY_ID_REG1
0x03 const PHY_ID_REG2
0x04 const PHY_ANA_REG
0x05 const PHY_ANLPAR_REG
0x06 const PHY_ANE_REG
0x10 const PHY_ECNTL_REG1
0x11 const PHY_QPDS_REG
0x12 const PHY_10BOP_REG
0x13 const PHY_ECNTL_REG2

\ PHY control register
bit15 const RESET_PHY
bit14 const ENABLE_LOOPBACK
bit13 const DR_100MB
bit12 const ENABLE_AN
bit10 const PHY_MAC_ISOLATE
bit9  const RESTART_AN
bit8  const PHY_FULLDUPLEX
bit7  const PHY_COL_TEST

\ PHY Status register
bit5 const AN_COMPLETE

\ PHY Auto-negotiation Advertisement register
bit8 const DR100_TX_FULL
bit7 const DR100_TX_HALF
bit6 const DR10_TX_FULL
bit5 const DR10_TX_HALF
DR100_TX_FULL DR100_TX_HALF or DR10_TX_FULL or
  const ANE_ALL	\ request everything


: WritePhy	\ data reg --
\ *G Write data to reg in the PHY.
  _mac tuck				\ -- data mac reg mac
  2swap stadata + !			\ -- reg mac ; set data
  swap WRPHY or over stacon + !		\ -- mac ; set write command
  begin
    dup stacon + @ MiiBusy and 0=	\ wait for busy bit to clear
  until
  drop
\  #1000 0 do loop			\ waste time *** huh? ***
;

: ReadPhy	\ reg -- data
\ *G Read data from reg in the PHY.
  _mac					\ -- reg mac
  swap RDPHY or over StaCon + !
  begin
    dup stacon + @ MiiBusy and 0=	\ wait for busy bit to clear
  until
  StaData + @				\ read data
;

: OrPhy		\ mask reg -- ; or mask into register
\ *G Or mask into reg in the PHY.
  tuck ReadPhy  or  swap WritePhy
;


\ *****************
\ *S MAC primitives
\ *****************

: ReadyMacTx	\ --
\ *G Set the Tx Registers related to BDMA & MAC ready to transmit
\ ** a packet.
  gBdmaTxCon _bdma BDMATXCON + !
  gMacTxCon  _mac MACTXCON + !
;

: ReadyMacRx	\ --
\ *G Set the Rx Registers related to BDMA & MAC ready to receive
\ ** a packet.
  gBdmaRxCon _bdma BDMARXCON + !
  gMacRxCon  _mac MACRXCON + !
  0 -> RxStopped
;

: DisMacRx	\ --
\ *G Disable the Rx and MAC receive registers.
  0 _bdma BDMARXCON + !
  0 _mac MACRXCON + !
  -1 -> RxStopped
;


\ *******************
\ *S Diagnostic hooks
\ *******************

((
: MacDebugStatus	\ --
\ *G Hook for diagnostic info. display.
;
))


\ *********************
\ *S Interrupt handlers
\ *********************

: MACTxErrors	\ status --
\ *G Process status errors for the MAC TX.
  dup Under and  if  gsMacTxStatus MTS.UnderErr incr  endif
  dup ExColl and if  gsMacTxStatus MTS.ExCollErr incr  endif
  dup TxDeffer and if  gsMacTxStatus MTS.TxDeferredErr incr  endif
  dup Paused and if  gsMacTxStatus MTS.sPaused incr  endif
  dup DeferOn and  if  gsMacTxStatus MTS.DeferErr incr  endif
  dup NCarr and  if  gsMacTxStatus MTS.NCarrErr incr  endif
  dup SQErr and  if  gsMacTxStatus MTS.sSQE incr  endif
  dup LateColl and if  gsMacTxStatus MTS.LateCollErr incr  endif
  dup TxPar and  if  gsMacTxStatus MTS.TxParErr incr  endif
  dup TxHalted and if  gsMacTxStatus MTS.sTxHalted incr  endif
  drop
;

: MAC_Tx_isr	\ --
\ *G Interrupt Service Routine for MAC Tx.
  _bdma BDMATXPTR + @			\ DMA address
  gCTxFDPtr				\ current tx fd
  begin					\ -- dmaaddr fd
    2dup <>
   while
    dup FD.FrameDataPtr @		\ if frame is owned by BDMA
    fOwnership_BDMA and 0=		\ we cannot carry on
   while
    dup FD.StatusAndFrameLength @	\ inspect status
    #16 rshift $FFFF and
    dup Comp and if			\ good
      drop
      gsMacTxStatus MTS.MacTxGood incr
    else				\ bad
      MACTxErrors			\ process errors
      ( MacDebugStatus ) ReadyMacTx	\ Set MAC/BDMA Tx control regs for next use
    endif
    0 over FD.StatusAndFrameLength !	\ Clear Framedata pointer already used.
    FD.NextFrameDescriptor @		\ step to next frame
  repeat then				\ -- dmaaddr fd
  to gCTxFdPtr  drop
;

: MAC_Rx_isr	\ --
\ *G Interrupt Service Routine for MAC Rx - Unused
;

: BDMA_Tx_isr	\ --
\ *G Interrupt Service Routine for BDMA Tx - unused
;

: BDMARXErrors	\ status --
\ *G Handle BDMA Receive errors.
  gErrorPacketCnt 1+ to gErrorPacketCnt	\ count total number of errors
  dup OvMax and    if  gsMacRxStatus MRS.OvMaxSize incr  endif
  dup CtlRecd and  if  gsMacRxStatus MRS.sCtlRecd incr  endif
  dup Rx10Stat and if  gsMacRxStatus MRS.sRx10Stat incr  endif
  dup AlignErr and if  gsMacRxStatus MRS.AllgnErr incr  endif
  dup CRCErr and   if  gsMacRxStatus MRS.sCRCErr incr  endif
  dup OverFlow and if  gsMacRxStatus MRS.OverflowErr incr  endif
  dup LongErr and  if  gsMacRxStatus MRS.sLongErr incr  endif
  dup RxPar and    if  gsMacRxStatus MRS.RxParErr incr  endif
  dup RxHalted and if  gsMacRxStatus MRS.sRxHalted incr  endif
  drop
;

: BDMA_Rx_isr	\ --
\ *G Interrupt Service Routine for BDMA Rx.
\ ** Ethernet frames are received by this ISR.

\ Get current frame descriptor and status
  _bdma BDMARXPTR + @ >r		\ get RX frame desc. pointer
  _bdma BDMASTAT + @			\ BDMA status

\ Clear BDMA status register bit by write 1
  S_BRxRDF _bdma BDMASTAT + or!
  gsBdmaRxStatus BRS.BdmaRxCnt Incr

\ Check Null List Interrupt - next frame address is 0.
  dup S_BRxNL and if
    drop  r> drop			\ discard BDMA status, RxFD
    S_BRxNL _bdma BDMASTAT + or!	\ clear by writing 1, 
    gsBDMARxstatus BRS.BRxNLErr incr
    MacInitialize			\ too much, surely??
    exit
  endif

\ Check NotOwner status - all buffers in use
  S_BRxNO and if
    S_BRxNO _bdma BDMASTAT + or!
    gsBdmaRxStatus BRS.BRxNOErr incr
    r> drop
    DisMacRx				\ disable receiver, restarted later
    exit
  endif

\ Get Rx Frame Descriptor
  gCRxFDPtr				\ current RX frame desc.
  begin					\ -- rxFD
    dup FD.StatusAndFrameLength @
    #16 rshift 0xffff and		\ -- rxFD rxstatus

\ If Rx frame is good, then process received frame else process errors
    dup RxGood OvMax or and		\ extract RxGood and OvMax bits
    RxGood = if				\ if no overflow
      gsBdmaRxStatus BRS.BdmaRxGood Incr
      drop
    else
      BDMARxErrors			\ Save Error status
      fOwnership_BDMA over FD.FrameDataPtr or!
    endif
    FD.NextFrameDescriptor @
    dup r@ =
  until					\ -- rxFD
  to gCRxFDPtr  r> drop			\ --
;


\ *****************
\ *S Initialisation
\ *****************

\ variable ResetStatus		\ -- addr ; status after reset complete
\ variable ResetCntl		\ -- addr ; Control after reset complete
\ variable ResetPartner		\ -- addr ; Partner after reset complete
\ variable ResetCntlSet		\ -- addr ; value set to Control after AN

((
: ResetPhy	\ -- ; primitive version
  RESET_PHY PHY_CNTL_REG OrPhy		\ software reset the chip
  begin					\ wait until reset complete
    PHY_CNTL_REG ReadPhy RESET_PHY and 0=
  until
  0 PHY_CNTL_REG WritePhy		\ select 10Mb half duplex
;
))

: ResetPhy	\ -- ; autonegotiation version
\ *G Initialise the PHY with autonegotiation
  RESET_PHY PHY_CNTL_REG OrPhy		\ software reset the chip
  begin					\ wait until reset complete
    PHY_CNTL_REG ReadPhy RESET_PHY and 0=
  until
  ANE_ALL PHY_ANA_REG OrPhy		\ request 10/100BaseT features
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
  RESTART_AN PHY_CNTL_REG OrPhy		\ start autonegotiation
  begin					\ wait until done
    PHY_STATUS_REG ReadPhy
    AN_COMPLETE and
  until
\  PHY_STATUS_REG ReadPhy ResetStatus !	\ store Reset results
\  PHY_CNTL_REG ReadPhy ResetCntl !
  PHY_ANLPAR_REG ReadPhy \ dup ResetPartner !
\ set up control reg using the results
  0					\ -- partner cntlset
  over DR100_TX_FULL and
  if  DR_100MB PHY_FULLDUPLEX or or  endif
  over DR100_TX_HALF and
  if  DR_100MB or  endif
  over DR10_TX_FULL and
  if  PHY_FULLDUPLEX or  endif
\  dup ResetCntlSet !			\ store results used
  PHY_CNTL_REG WritePhy
  drop
;

: TxFDInitialize	\ --
\ *G Initialize Tx frame descriptor area-buffers.
{ | pFd pStartFd pLastFd pFrameData -- }
  0 -> pLastFd
\ Get Frame descriptor's base address
\ +0x4000000 is for setting this area to non-cacheable
  TxFDBaseAddr 0x4000000 + 
  dup _bdma BDMATXPTR + !
  dup -> gWTxFDPtr  dup -> gCTxFDPtr
  dup -> pFd  -> pStartFd
\ Get Transmit buffer base address.
  TxFBABaseAddr 0x4000000 + -> pFrameData
\ Generate linked list.
  MaxTxFrameDescriptors 0 do
    pLastFd 0= if
      pFd -> pLastFd
    else
      pFd pLastFd FD.NextFrameDescriptor !
    endif
    pFrameData fOwnership_CPU and  pFd FD.FrameDataPtr !
    0 pFd FD.Reserved !
    0 pFd FD.StatusAndFrameLength !
    0 pFd FD.NextFrameDescriptor !
    pFd -> pLastFd
    pFd /FrameDescriptor + -> pFd
    pFrameData /MacFrame + -> pFrameData
  loop
\ Turn Frame descriptor list into a ring
  pStartFd pLastFd FD.NextFrameDescriptor !
;

: RxFDInitialize	\ --
\ *G Initialize Rx frame descriptor area-buffers.
{ | pFd pStartFd pLastFd pFrameData -- }
  0 -> pLastFd
\ Get Frame descriptor's base address
\ +0x4000000 is for setting this area to non-cacheable
  RxFDBaseAddr 0x4000000 +		\ non-cacheable FD base address
  dup _bdma BDMARXPTR + !		\ initialise BDMA pointer
  dup -> gCRxFDPtr  dup -> gRRxFDPtr	\ receive FD pointers
  dup -> pFd  -> pStartFd
\ Get buffer base address
  RxFBABaseAddr 0x4000000 + -> pFrameData
\ Generate linked list
  MaxRxFrameDescriptors 0 do
    pLastFd 0= if
      pFd -> pLastFd
    else
      pFd pLastFd FD.NextFrameDescriptor !
    endif
    pFrameData fOwnership_BDMA or 0x4000000 or  pFd FD.FrameDataPtr !
    0 pFd FD.Reserved !
    0 pFd FD.StatusAndFrameLength !
    0 pFd FD.NextFrameDescriptor !
    pFd -> pLastFd
    pFd /FrameDescriptor + -> pFd
    pFrameData /MacFrame + -> pFrameData
  loop
\ Turn Frame descriptor list into a ring
  pStartFd pLastFd FD.NextFrameDescriptor !
;

mac_rx_int  mac_tx_int or  bdma_rx_int or  bdma_tx_int or	\ CJD004
  const mac_ints	\ -- mask ; bit mask for Ethernet interrupts

: MacInitialize	\ -- ; initialise 4510 for Ethernet
\ *G Initialise interrupts, BDMA, MAC and CAM
\ Disable MAC and BDMA interrupts
  mac_ints _interrupts intmsk + bic!	\ bit clear
\ BDMA and MAC interrupt vector setup
  ['] MAC_Rx_isr  nMAC_RX_INT  SetIRQ
  ['] MAC_Tx_isr  nMAC_TX_INT  SetIRQ
  ['] BDMA_Rx_isr nBDMA_RX_INT SetIRQ
  ['] BDMA_Tx_isr nBDMA_TX_INT SetIRQ
\ Set the initial condition of the BDMA, and MAC
  BRxRS _bdma BDMARXCON + !		\ Reset BDMAC Receiver
  BTxRS	_bdma BDMATXCON + !		\ Reset BDMAC Transceiver
  SwReset _mac MACCON + !
  MaxRxFrameSize #40 + _bdma BDMARXLSZ + !	\ 1520
  FullDup _mac MACCON + !		\ request full duplex
\ Set the BDMA Tx/Rx Frame Descriptors
  TxFDInitialize RxFDInitialize
\ Set the CAM Control registers
  0 _mac CAMEN + !			\ disable all CAM entries
  EtherAddress 0 SetCam			\ enable CAM[0] only
  CompEn BroadAcc or GroupAcc or	\ Enable comparison, broadcast, multicast
  _mac CAMCON + !
\ 6. Enable interrupt BDMA Rx and MAC Tx interrupt.
  nBDMA_RX_INT EnInt
  nMAC_TX_INT  EnInt
  nGlobal_Int  EnInt
\ 7. Configure the BDMA and MAC control registers
  ReadyMacTx ReadyMacRx
;

: InitEther	\ -- ; full Ethernet initialisation
\ *G Perform a full initialisation of the Ethernet system.
\ ** This word must be run before the Ethernet is used,
\ ** otherwise the code may hang or crash!
  ResetPhy MacInitialize
;

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O IPDGetAddr function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the stored data
\ ** will not be changed.
  drop					\ flags
  ?dup if				\ macaddr
    EtherAddress 6 cmove
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;


\ ***********************************
\ *S Application transmit and receive
\ ***********************************

: MacTxGo	\ --
\ *G Permit transmission to start.
  _bdma bdmatxcon + @ BTxEn and 0=
  if  BTxEn _bdma bdmatxcon + or!  endif
  TxEn _mac mactxcon + or!
;

little-endian? [if]
PaddingMode      CRCMode or        SourceAddrIncrement or
LittleEndian or  WidgetAlign00 or  MACTxIntEn or
  equ TxFDreserved
[then]
big-endian? [if]
PaddingMode      CRCMode or        SourceAddrIncrement or
BigEndian or     WidgetAlign00 or  MACTxIntEn or
  equ TxFDreserved
[then]

: (send_ether_pkt)	\ addr len -- ior ; ior=0=success
\ *G The raw non-blocking send routine.
\  3 + -4 and				\ force len to 4 byte unit
  $40 max tuck				\ min 64 bytes ; -- len addr len
  gWTxFDPtr dup >r			\ next transmit frame descriptor
  FD.FrameDataPtr @ 			\ -- len addr len dest/flag ; R: -- fd
  dup fOwnership_BDMA and if		\ if owned by BDMA, we cannot xmit
    2drop 2drop  r> drop  -1  exit
  endif					\ -- len addr len dest ; R: -- fd
  swap cmove				\ copy to DMA buffer ; -- len ; R: -- fd

  TxFDreserved r@ FD.reserved !		\ Set TX Frame flag & Length Field
  $FFFF and r@ FD.StatusAndFrameLength !
  fOwnership_BDMA r@ FD.FrameDataPtr or! \ Change ownership to BDMA
  MacTxGo				\ Enable MAC and BDMA Tx control registers

  r> FD.NextFrameDescriptor @		\ Change the Tx frame descriptor for next use
  to gWTxFDPtr

  ( 1 ms )				\ Ensure interpacket gap ; SFP003
  0					\ success
;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop
  gWTxFDPtr FD.FrameDataPtr @ fOwnership_BDMA and 0=
;

: send_ether_pkt        \ addr len --
\ *G Send packet from supplied buffer.
\ ** The word will block until sufficient packet memory is available.
\ ** It returns after the the packet has been copied to a DMA buffer, but
\ ** return from this word does not mean that the packet has been 
\ ** transmitted.
  begin
    2dup (send_ether_pkt)
   while
[ tasking? ] [if] pause [then]
  repeat
  2drop
;

: IsRx?         \ -- t|f
\ *G Check if incoming data is present, return true if data is available.
  gRRxFDPtr gCRxFDPtr <>
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get pending receive packet to a buffer.
\ ** Note that data is valid only when ISRX? returns true,
\ ** so that you must poll with ISRX? before using
\ ** GET_ETHER_PKT.
\ ** If reception has been stopped because all the DMA
\ ** buffers are in use, reception will be restarted after
\ ** each buffer is read.
  IsRx? if				\ -- *dest maxlen
    gRRxFDPtr dup >r			\ FD to receive from
    FD.FrameDataPtr @ -rot		\ -- src *dest maxlen ; R: -- fd
    r@ FD.StatusAndFrameLength @
    $FFFF and min			\ -- src *dest len' ; R: -- fd
    dup >r cmove r>			\ copy, preserve len to return
    r@ FD.NextFrameDescriptor @		\ update Rx pointer
    to gRRxFDPtr
    fOwnership_BDMA r> FD.FrameDataPtr or!	\ Change ownership to BDMA for next use
    RxStopped      			\ restart Rx if stopped
    if  ReadyMacRx  then
  else
    2drop 0
  endif
;

: DiscardRX	\ --
\ *G Discard (throw away) pending receive packet.
\ ** You must poll with ISRX? before using DISCARDRX.
\ ** If reception has been stopped because all the DMA
\ ** buffers are in use, reception will be restarted after
\ ** each buffer is discarded.
  IsRx? if
    gRRxFDPtr				\ FD to receive from
    dup FD.NextFrameDescriptor @	\ update Rx pointer
      to gRRxFDPtr
    fOwnership_BDMA swap FD.FrameDataPtr or!	\ Change ownership to BDMA for next use
    RxStopped      			\ restart Rx if stopped
    if  ReadyMacRx  then
  endif
;


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************

\ *P The layout and usage of this structure is defined in the
\ ** file COMMON\ETHERCOM.FTH.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

create EtherVector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** COMMON\ETHERCOM.FTH.
  ' InitEther ,         \ initialisation      
  ' noop ,	        \ shutdown            
  ' IsRx? ,	        \ receive test        
  ' get_ether_pkt ,     \ receive packet      
  ' IsTx? ,	        \ transmit test       
  ' send_ether_pkt ,    \ transmit packet     
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state
  ' DiscardRx ,		\ Discard current receive packet

EtherVector constant IPDevice	\ -- addr
\ *G IPDevice is the default device name required by ETHERCOM.FTH.
\ ** If you have multiple ports, only one should be called IPDevice.

  
\ **************
\ *S System test
\ **************
\ *P The simplest way to test the system is to test reception
\ ** by sniffing broadcast packets on the network. Once this
\ ** is working, you can move on to testing transmission
\ ** using the PING code in NETBOOT.FTH.

sniff? [if]

#1536 buffer: pbuff	\ -- addr
\ *G Temporary buffer for SNIFF.

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** SNIFF can be used to test reception.
  InitEther
  begin
    isrx? if
      pbuff #1536 erase
      pbuff #1536 get_ether_pkt
      pbuff swap 1 max dump
    then
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]

((
: emptyEtherRx	\ -- ; flush all receive buffers
  0
  begin
    isrx?
   while
    pbuff #1536 get_ether_pkt drop
    1+
  repeat
  cr . ." buffers flushed"
;

: testrx	\ --
  start-timers
  begin
    gsbdmarxstatus @ .  #1000 ms
    RxStopped
    if  emptyEtherRx cr  endif
    key?
  until
;

: g?		\ --
  isrx? if
    pbuff 100 get_ether_pkt  pbuff swap dump
  then
;

: tx		\ --
  gWTxFDptr
  dup 16 dump
  sendArpReq
  cr ." Key" key drop
  16 dump
  gsmactxstatus /mactxstatus dump
;
))


\ **********
\ *S Gotchas
\ **********
\ *P For reasons we do not understand yet, transmission of multiple
\ ** packets is considerably faster if a delay is introduced at the
\ ** end of (SEND_ETHER_PKT), particularly if 100Mb full-duplex is
\ ** in use. To avoid conflicts with NETBOOT.FTH and interrupt
\ ** disabling, this should be a software delay which is tuned for
\ ** your hardware.


\ ******
\ *> ###
\ ******

