\ RTL8019AS and NE2000 compatible Ethernet driver

((
Copyright (c) 2004, 2005, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeltd.demon.co.uk
     tech-support@mpeltd.demon.co.uk
web: www.mpeltd.demon.co.uk

Portions of this code have been licensed from:
  Oxford Network Solutions
  Tel: 01869 320461
  Fax: 01869 327065
  Internet: grahams@oxns.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20070111 MPE001 Added code for EA2138 board.
))


\ =========
\ *! ne2000
\ *T NE2000 Compatible Ethernet Driver
\ =========
\ *P The file *\i{ARM\Drivers\Ether8019AS.fth} contains the
\ ** hardware driver layer for NE2000 compatible Ethernet
\ ** controller chips such as the Realtek RTL8019AS used on the
\ ** Embedded Artists EA2138-E10 boards.
\ ** The data sheets for these devices may be found at
\ *C   www.realtek.com.tw


\ ===================
\ *S Hardware gotchas
\ ===================
\ *P This code assumes byte accesses to the chip. This restricts
\ ** internal RAM to 8kb. When operating with fast CPUs note that
\ ** the back to back access time may need to be controlled.

\ *P The NE2000 access words *\fo{EC@}, *\fo{EC!}, *\fo{EW@} and
\ ** *\fo{EW!} assume that the chip is driven from I/O
\ ** ports. If you are not using the EtherStamp boards, you will
\ ** have to modify these for your own use.

\ *P Not all "NE2000 compatible" devices are the same. They may/will
\ ** have additional registers which must be configured during set up.
\ ** If you convert this code for other chips, be very careful.


\ ****************
\ *S Configuration
\ ****************
\ *P The following can be defined before the file *\i{drivers\Ether8019AS.fth}
\ ** is compiled.

\ =====================
\ *N Hardware selection
\ =====================

[undefined] TiniARM? [if]
0 equ TiniARM?	\ -- flag
\ *G Set this non-zero for the TiniARM with an EDTP board.
\ ** Zero by default.
[then]

[undefined] EA2138? [if]
1 equ EA2138?	\ -- flag
\ *G Set this non-zero for the EA2138 board with an 8019AS.
\ ** Non-zero by default.
[then]

[undefined] EEprom? [if]
0 equ eeprom?	\ -- flag
\ *G Set this equate true if the NE2000 has an attached EEPROM
\ ** for configuration data storage. False by default.
\ ** Note that no EEPROM driver code is provided and if set,
\ ** stubs will be compiled.
[then]


\ =====================
\ *N Software selection
\ =====================

[undefined] EtherDiags? [if]
0 equ EtherDiags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. Non-zero by default.
[then]

[undefined] Sniff? [if]
0 equ Sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $20 c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!
[then]

[undefined] IpAddress [if]
create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
  192 c,  168 c,  1 c,  251 c,		\ assign these as required
[then]

#100 equ #DMAwait	\ -- u ; 1..u, NOT 0.
\ *G Defines the number of iterations for which DMA operation
\ ** is polled in *\fo{waitDMA} below before returning failure.
\ ** This value may need to be increased on fast hardware.


\ *******************
\ NE2000 registers
\ *******************

\ Page 0 and common registers
#define ne2kCR		0x00	\ all pages
#define ne2kCLDA0	0x01	\ r/o
#define ne2kPSTART	0x01	\ w/o - page 2 r/o
#define ne2kCLDA1	0x02	\ r/o
#define ne2kPSTOP	0x02	\ w/o - page 2 r/o
#define ne2kBNRY	0x03
#define ne2kTSR		0x04	\ r/o
#define ne2kTPSR	0x04	\ w/o
#define ne2kNCR		0x05	\ r/o
#define ne2kTBCR0	0x05	\ w/o
#define ne2kFIFO	0x06	\ r/o
#define ne2kTBCR1	0x06	\ w/o
#define ne2kISR		0x07	\ r/w
#define ne2kCRDA0	0x08	\ r/o
#define ne2kRSAR0	0x08	\ w/o
#define ne2kCRDA1	0x09	\ r/o
#define ne2kRSAR1	0x09	\ w/o
#define ne2kRBCR0	0x0A
#define ne2kRBCR1	0x0B
#define ne2kRSR		0x0C	\ r/o
#define ne2kRCR		0x0C	\ w/o
#define ne2kCNTR0	0x0D	\ r/o
#define ne2kTCR		0x0D	\ w/o
#define ne2kCNTR1	0x0E	\ r/o
#define ne2kDCR		0x0E	\ w/o
#define ne2kCNTR2	0x0F	\ r/o
#define ne2kIMR		0x0F	\ w/o
#define ne2kRDMAPORT	0X10	\ all pages
#define ne2kRSTPORT	0x1F	\ all pages 

\ Page 1 is r/w
#define ne2kPAR0	0x01    \ 1..6
#define ne2kCURR	0x07
#define ne2kMAR0	0x08	\ 8..F

\ Page 2 is to read W/O regs in Page 0

\ Page 3
#define ne2kCR9346	0x01
#define ne2kCONFIG0	0x03
#define ne2kCONFIG1	0x04
#define ne2kCONFIG2	0x05
#define ne2kCONFIG3	0x06


\ NE2000 initial register values
$04 equ RCRval
$00 equ TCRval
$58 equ DCRval
\ PXR and OVW interrupt enabled
$11 equ IMRval
$40 equ Txtstart
$46 equ Rxstart
$60 equ Rxstop
\ NE2000 ISR Register definitions
$80 equ Rst
$40 equ Rdc
$10 equ Ovw
$01 equ Prx


\ *********************
\ *S Hardware Interface
\ *********************
\ *P The default code is for a bit-banged device attached to the
\ ** GPIO system on a TiniARM board. Change this to suit your
\ ** hardware.

\ ==========================
\ *N TiniARM with EDTP board
\ ==========================

TiniARM? [if]

$0000FF00 equ edbmask	\ -- mask ; ethernet data bus mask
$200000F0 equ eabmask	\ -- mask ; ethernet address bus mask
bit31     equ nIOWR	\ -- mask ; IOWR bit - active low
bit30     equ nIORD	\ -- mask ; IORD bit - active low
bit29     equ SA4	\ -- mask ; SA4 address line
bit16     equ nINT	\ -- mask ; interrupt bit - active low
nIORD  nIOWR or  eabmask or
  equ EOPs		\ -- mask ; always outputs
EOPs  nINT or  edbmask or
  dup equ etherbits	\ -- mask ; all ethernet bits
  invert equ ~etherbits	\ -- mask ; all non-ethernet bits

$000000FF equ PS0gpio	\ -- mask ; ensure iopins 4-15 are GPIO
$03FFFFFC equ PS1gpio	\ -- mask ; ensure iopins 16,29-31 are GPIO

L: ^GPIOe	\ -- addr ; saves target space
  _GPIO ,

: erdmode	\ --
\ *G Turns data bus for reading.
  ^GPIOe @
  dup IODIR + @
  edbmask invert and
  swap IODIR + !
;

: ewrmode	\ --
\ *G Turns data bus for writing
  ^GPIOe @
  dup IODIR + @
  edbmask or
  swap IODIR + !
;

: ne2kaddr	\ reg --
\ *G Write NE2000 address.
  $1F and				\ clip to 5 bits
  4 lshift				\ A3..0 to P0.7..4
  dup #21 lshift or			\ A25 to P0.29
  eabmask dup ^GPIOe @ tuck		\ -- n' mask addr mask addr
  IOCLR + !				\ -- n' mask addr ; clear all address bits
  -rot  and  swap IOSET + !
;

: ec@		\ reg -- data
\ *G Read byte from NE2000 register n.
  erdmode				\ set up iopin dir for reading
  ne2kaddr		 		\ set address on pins
  ^GPIOe @
  nIORD over IOCLR + !			\ nIORD low
  [asm  nop  nop  nop  asm]		\ 48 ns
  dup IOPIN + @				\ read
  nIORD rot IOSET + !			\ nIORD high
  8 rshift $000000FF and
;

: ec!		\ data n --
\ *G Write byte to NE2000 register n.
  ewrmode				\ set up iopin dir for writing
  ne2kaddr		 		\ set address on pins
  ^GPIOe @				\ -- data addr
  edbmask over IOCLR + !		\ clear all data lines
  swap $FF and 8 lshift over IOSET + !	\ set 1 bits
  nIOWR over IOCLR + !			\ nIOWR low
  [asm  nop  nop  nop  asm]		\ 48 ns
  nIOWR swap IOSET + !			\ nIOWR high
;

: ew@		\ reg -- data16
\ *G Read 16 bits from NE2000 register pair n, n+1.
  dup ec@				\ low byte from low address
  swap 1+ ec@ 8 lshift or		\ high byte from high address
;

: ew!		\ data16 reg --
\ *G Write 16 bits to NE2000 register pair n, n+1.
  2dup ec!				\ low byte to low address
  swap 8 rshift swap 1+ ec!		\ high byte to high address
;

: eblkwrite	\ addr dreg len --
\ *G Write len bytes starting at addr into consecutive 
\ ** registers starting at dreg.
  bounds
  ?do  dup c@ i ec!  1+  loop
  drop
;

: InitNE2000IO	\ --
\ *G Initialise access to the NE2000 hardware. For i/o port
\ ** access this involves setting up port I/O direction registers
\ ** and so on.
  _PINSEL
    dup PINSEL0 + @			\ set GPIO4..15 to GPIO mode
    PS0gpio and
    over PINSEL0 + !
    dup PINSEL1 + @			\ set GPIO16,29..31 to GPIO mode
    PS1gpio and
    swap PINSEL1 + !
  ^GPIOe @
  nIORD nIOWR or nINT or over IOSET + !	\ set read and write high
  eabmask edbmask or  over IOCLR + !	\ set other o/ps low
  dup IODIR + @
    ~etherbits and			\ mask Ethernet bits
    EOPs or				\ set Ethernet outputs
  swap IODIR + !
;

: initNE2000	\ --
\ *G Initialise the NE2000 chip.
  ne2kRSTPORT ec@ ne2kRSTPORT ec!	\ initialisation
  #10 ms
  ne2kISR ec@ bit7 and 0= if		\ test we entered reset
    cr ." Cannot reset Ethernet" exit
  endif

  $21 ne2kCR ec!  2 ms			\ stop command, page 0
  DCRval ne2kDCR ec!			\ Data Config: little endian byte transfers
  0 dup ne2kRBCR0 ec!  ne2kRBCR1 ec!	\ DMA count=0
  $04 ne2kRCR ec!			\ Rx Config: for us and broadcast
  Txtstart ne2kTPSR ec!			\ Tx Config: start page in local RAM
  $02 ne2kTCR ec!			\ loopback to inhibit reception during set up
  Rxstart dup ne2kPSTART ec! 		\ set up receive ring
    ne2kBNRY ec!
  Rxstop ne2kPSTOP ec!

  $61 ne2kCR ec!  2 ms			\ stop command, page 1
  Rxstart ne2kCURR ec!			\ first receive ring page
  EtherAddress ne2kPAR0 6 eblkwrite	\ Ethernet MAC address

  $21 ne2kCR ec!			\ stop command, page 0
  DCRval ne2kDCR ec!			\ Data Config: little endian byte transfers

  $22 ne2kCR ec!			\ start command, page 0
  $FF ne2kISR ec!			\ clear all interrupt flags
  Imrval ne2kIMR ec!			\ enable overrun and receive interrupts
  $00 ne2kTCR ec!			\ normal operation
;

[then]


\ ==============
\ *N EA2138 10Mb
\ ==============

EA2138? [if]

\ P0 usage

$00FF0000 equ EDBmask	\ -- mask ; i/o Ethernet data bus mask. P0.16..23

bit15 equ ETH_INT	\ -- mask ; i/p Interrupt INT0
bit31 equ ETH_RST	\ -- mask ; o/p Reset

$C0000000 equ PS0gpio	\ -- mask ; ensure P0.15 is GPIO
$C000FFFF equ PS1gpio	\ -- mask ; ensure P0.16..23,31 are GPIO

\ P1 usage

$001F0000 equ EABmask	\ -- mask ; o/p Ethernet address bus mask. P1.16..20

bit21 equ ETH_nCS	\ -- mask ; o/p CS active low
bit22 equ ETH_nRD	\ -- mask ; o/p RD bit - active low
bit23 equ ETH_nWR	\ -- mask ; o/p WR bit - active low

\ Port bit masks

ETH_RST ETH_INT or EDBmask or
  equ EIOP0s		\ -- mask ; port 0 Ethernet i/o
ETH_RST equ EOP0s	\ -- mask ; port 0 outputs

ETH_nRD  ETH_nWR or  ETH_nCS or  EABmask or
  equ EIOP1s		\ -- mask ; port 1 Ethernet i/o
ETH_nRD  ETH_nWR or  ETH_nCS or  EABmask or
  equ EOP1s		\ -- mask ; port 1 Ethernet ooutputs

L: ^GPIO0	\ -- addr ; saves target space
  _GPIO0 ,

L: ^GPIO1	\ -- addr ; saves target space
  _GPIO1 ,

: erdmode	\ --
\ *G Turns data bus for reading.
  EDBmask ^GPIO0 @ IODIR + bic!
;

: ewrmode	\ --
\ *G Turns data bus for writing.
  EDBmask ^GPIO0 @ IODIR + or!
;

: ne2kaddr	\ reg --
\ *G Write NE2000 address.
  #16 lshift EABmask and		\ align, clip to 5 bits
  ^GPIO1 @ EABmask over IOCLR + !	\ clear all bits
  IOSET + !
;

: ec@		\ reg -- data
\ *G Read byte from NE2000 register *\i{reg}.
  erdmode				\ set up iopin dir for reading
  ne2kaddr		 		\ set address on pins
  ^GPIO1 @
  ETH_nCS over IOCLR + !		\ nCS/A5 low
  ETH_nRD over IOCLR + !		\ nRD low
  [asm  nop  nop  nop  nop  nop  asm]	\ 48\80 ns
  ^GPIO0 @ IOPIN + @ swap		\ -- data gpio1 ; read
  ETH_nRD over IOSET + !		\ nRD high
  ETH_nCS swap IOSET + !		\ nCS/A5 high
  #16 rshift $000000FF and
;

: ec!		\ data reg --
\ *G Write byte to NE2000 register *\i{reg}.
  ewrmode				\ set up iopin dir for writing
  ne2kaddr		 		\ set address on pins
  $FF and #16 lshift  ^GPIO0 @		\ -- data' gpio0
  EDBmask over IOCLR + !  IOSET + !	\ clear all data lines, set 1 bits
  ^GPIO1 @
  ETH_nCS over IOCLR + !		\ nCS/A5 low
  ETH_nWR over IOCLR + !		\ nWR low
  [asm  nop  nop  nop  nop  nop  asm]	\ 48\80 ns
  ETH_nWR over IOSET + !		\ nWR high
  ETH_nCS swap IOSET + !		\ nCS/A5 high
;

: ew@		\ reg -- data16
\ *G Read 16 bits from NE2000 register pair *\i{reg, reg+1}.
  dup ec@				\ low byte from low address
  swap 1+ ec@ 8 lshift or		\ high byte from high address
;

: ew!		\ data16 reg --
\ *G Write 16 bits to NE2000 register pair *\i{reg, reg+1}.
  2dup ec!				\ low byte to low address
  swap 8 rshift swap 1+ ec!		\ high byte to high address
;

: eblkwrite	\ addr dreg len --
\ *G Write *\i{len} bytes starting at *\i{addr} into consecutive 
\ ** registers starting at *\i{dreg}.
  bounds
  ?do  dup c@ i ec!  1+  loop
  drop
;

: InitNE2000IO	\ --
\ *G Initialise access to the NE2000 hardware. For i/o port
\ ** access this involves setting up port I/O direction registers
\ ** and so on.
  _PINSEL
    $0C over PINSEL2 + bic!		\ P1 is GPIO
    PS0gpio over PINSEL0 + bic!		\ force GPIO usage
    PS1gpio swap PINSEL1 + bic!		\ force GPIO usage
  ^GPIO1 @
    dup IODIR + @
      EIOP1s invert and			\ clear all as inputs
      EOP1s or				\ set bits for o/ps
    over IODIR + !
    ETH_nCS ETH_nRD or ETH_nWR or	\ set control lines high
    swap IOSET + !
  ^GPIO0 @
    dup IODIR + @
      EIOP0s invert and			\ clear all as inputs
      EOP0s or				\ set bits for o/ps
    swap IODIR + !
;

: initNE2000	\ --
\ *G Initialise the NE2000 chip.

  ETH_RST ^GPIO0 @
  2dup IOSET + !  #100 ms		\ set RST high then low
  IOCLR + !  #500 ms

  $00 ne2kIMR ec!			\ mask all interrupts
  $FF ne2kISR ec!			\ clear all interrupt flags

\ 8019 specific configuration
  $E1 ne2kCR ec!			\ select page 3, Abort, STOP
  $C0 ne2kCR9346 ec!			\ enable config register access
  $00 ne2kCONFIG3 ec!			\ setup LEDs, clear sleep and power down
  $20 ne2kCONFIG2 ec!			\ force 10BaseT with test, disable BROM
  $00 ne2kCR9346 ec!			\ disable config register access
  #500 ms

  $21 ne2kCR ec!			\ select page 0, Abort, STOP
  $48 ne2kDCR ec!			\ Data Config: little endian byte transfers
  0 ne2kRBCR0 ew!			\ 16 bit DMA count=0

  $20 ne2kRCR ec!			\ Rx temporarily in MON mode
  $02 ne2kTCR ec!			\ Tx temporarily in internal loopback
  TXTstart ne2kTPSR ec!			\ Tx Config: start page in local RAM
  RXstart ne2kBNRY ec!			\ Rx config: set up receive ring
  RXstart ne2kPSTART ec!
  RXstop ne2kPSTOP ec!
  $FF ne2kISR ec!			\ clear all interrupt flags again

  $61 ne2kCR ec!  #10 ms		\ Abort. STOP, page 1
  EtherAddress ne2kPAR0 6 eblkwrite	\ Ethernet MAC address
  Rxstart ne2kCURR ec!			\ first receive ring page

\ standard NE2k initialisation
  $21 ne2kCR ec!			\ Abort, STOP, page 0
  $06 ne2kRCR ec!			\ Rx: Accept broadcast and multicast
  $FF ne2kISR ec!			\ clear all interrupt flags
  $1F ne2kIMR ec!			\ enable overrun and receive interrupts
  $22 ne2kCR ec!			\ start command, page 0
  $00 ne2kTCR ec!			\ normal operation
;


\ *************************
\ *S Generic NE2000 driver
\ *************************

: InitEther	\ --
\ *G Initialise I/O to NE2000 and initialise the NE2000.
\ ** By default this is done at powerup. During testing you
\ ** may wish to comment out the line:
\ *C   ' InitEther AtCold
  cr ." Initialising Ethernet ... "
  InitNE2000IO  InitNE2000
  ." done"
;
' InitEther AtCold

: IsRx?         \ -- t|f
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
  ne2kBNRY ec@
  $62 ne2kCR ec!			\ select page 1
  ne2kCURR ec@ <>
  $22 ne2kCR ec!			\ select page 0
;

: setDMA	\ addrne2k len --
\ *G Set up the remote DMA for *\b{len} bytes to/from
\ ** *\b{addrne2k} in the NE2000 RAM. The transfer is not performed.
  $40 ne2kISR ec!			\ clear RDMA interrupt
  $51 ne2kIMR ec!			\ enable RDMA interrupt
  ne2kRBCR0 ew!  ne2kRSAR0 ew!  
;

: waitDMA	\ -- flag
\ *G Wait a short time for DMA to complete, returning zero
\ ** if the DMA has completed.
  #DMAwait 0 do
    ne2kISR ec@ $40 and
    if  0  unloop exit  endif
  loop
  -1
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get pending receive packet to a buffer.
\ ** Note that data is valid only when *\fo{ISRX?} returns true,
\ ** so that you must poll with *\fo{ISRX?} before using
\ ** *\fo{GET_ETHER_PKT}. The received size of the packet
\ ** (not clipped to the buffer size) is returned.
  { | lenb nxtp rxstat -- }
  -> lenb
\ read the header
  ne2kBNRY ec@ 8 lshift dup >r
  4 setDMA				\ prep. for four bytes
  $0A ne2kCR ec!			\ start the remote read transfer
  ne2kRDMAPORT ec@ -> rxstat		\ receive status
  ne2kRDMAPORT ec@ -> nxtp		\ next packet
  ne2kRDMAPORT ec@ $FF and		\ get received packet size
  ne2kRDMAPORT ec@ 8 lshift or		\ -- *dest lenr
  waitDMA drop \ abort" Header DMA read failure"
\ read the data
  tuck
  r> 4 + over setDMA			\ prep. for packet read
  $0A ne2kCR ec!			\ start the remote read transfer
  0 ?do					\ -- lenr *dest
    ne2kRDMAPORT ec@			\ -- lenr *dest byte
    i lenb u<				\ save/discard data?
    if  over c!  1+  else  drop  endif
  loop
  drop					\ -- lenr
  waitDMA drop \ abort" Data DMA read failure"
\ update the ring buffer
  nxtp ne2kBNRY ec!
  $FF ne2kISR ec!			\ clear NE2000 interrupt reg
  4 -					\ fixup num RX bytes for leading status etc.
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer memory.
((
\ read the header
  ne2kBNRY ec@ 8 lshift 4 setDMA	\ prep. for four bytes
  $0A ne2kCR ec!			\ start the remote read transfer
  ne2kRDMAPORT ec@ drop			\ receive status
  ne2kRDMAPORT ec@ 			\ next packet
  ne2kRDMAPORT ec@ drop			\ get received packet size
  ne2kRDMAPORT ec@ drop			\ -- *dest lenr
  waitDMA drop \ abort" Header DMA read failure"
\ update the ring buffer
  ne2kBNRY ec!
  $FF ne2kISR ec!			\ clear NE2000 interrupt reg
))
  0 0 get_ether_pkt drop		\ len=0 for no transfer
;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length *\i{n} can be sent.
  drop true
;

: send_ether_pkt        \ caddr len --
\ *G Send packet from supplied buffer.
\ ** The word will block until sufficient packet memory is available.
\ ** Note that you should read incoming packets regularly, otherwise
\ ** it is possible to get into a situation in which *\fo{SEND_ETHER_PKT}
\ ** blocks for ever because there is not enough space in the
\ ** device for the transmission packet. Theoretically, this should
\ ** not happen because 1536 bytes are reserved for transmission,
\ ** but ...
  $40 max				\ force min packet size
  $FF ne2kISR ec!			\ clear interrupts
  Txtstart 8 lshift  over setDMA	\ setup for Tx DMA
  dup ne2kTBCR0 ew!			\ transmit byte count
  $012 ne2kCR ec!			\ start the DMA
  bounds ?do
    i c@ ne2kRDMAPORT ec!
  loop
  waitDMA drop \ abort" Tx DMA write failure"
  Txtstart ne2kTPSR ec!			\ set Tx start page
  $24 ne2kCR ec!			\ start transmission
  begin					\ wait until done
    ne2kISR ec@ $0A and 0=
   while
[ tasking? ] [if]  pause  [then]
  repeat
  $FF ne2kISR ec!			\ clear interrupts
;


\ **************
\ *S Diagnostics
\ **************
\ *P These are only compiled if the equate *\fo{EtherDiags?} is
\ ** non-zero.

EtherDiags? [if]

: edump		\ --
\ *G Dump NE2000 registers.
  cr  $10 0 do
    i ec@ .byte space
  loop
  cr  $20 $10 do
    i ec@ .byte space
  loop
  cr
;

: epage		\ p --
\ *G Select NE2000 page.
  6 lshift
  0 ec@ $3F and or 0 ec!
;

: ep		\ --
\ *G Dump NE2000 registers selected page.
  cr  $10 0 do
    i 8 = if ." : " endif
    i ec@ .byte space
  loop
;

: ed		\ --
\ *G Dump NE2000 registers for all pages.
  0 epage ep
  1 epage ep
  2 epage ep
  3 epage ep
  0 epage
  cr
;

[then]


\ ******************
\ *S Attached EEPROM
\ ******************

eeprom? [if]

cr ." NE2000 EEPROM code not implemented"  abort

: ee!           \ word offset --
\ *G Write a 16 bit word into the EEPROM at EEPROM address
\ ** offset. The EEPROM may not be present in all SMC91C9x
\ ** implementations.
  2drop
;

: ee@           \ offset -- word
\ *G Read a 16 bit word from the EEPROM at EEPROM address
\ ** offset. The EEPROM may not be present in all implementations.
  drop $FFFF
;

[then]


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************
\ *P The code in this section is only compiled if the equate
\ ** *\fo{GenericIP?) has been defined and is non-zero.

\ *P The layout and usage of this structure is defined in the
\ ** file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

[undefined] GenericIP? [if]
0 equ GenericIP?
[then]

GenericIP? [if]

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O *\fo{IPDGetAddr} function.
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the stored data
\ ** will not be changed.
  drop					\ flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove
    etheradd>CS
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;

create NE2vector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** COMMON\ETHERCOM.FTH.
  ' InitEther ,         \ initialisation      
  ' noop ,	        \ shutdown            
  ' IsRx? ,	        \ receive test        
  ' get_ether_pkt ,     \ receive packet      
  ' IsTx? ,	        \ transmit test       
  ' send_ether_pkt ,    \ transmit packet     
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

NE2vector constant IPDevice	\ -- addr
\ *G IPDevice is the default device name required by *\i{ETHERCOM.FTH}.
\ ** If you have multiple ports, only one should be called *\fo{IPDevice}.

[then]

  
\ **************
\ *S System test
\ **************
\ *P This code is only compiled if the equate *\fo{SNIFF?}
\ ** is non-zero.

sniff? [if]

#1536 buffer: pbuff	\ -- addr
\ *G Buffer for *\fo{SNIFF}.

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  InitEther
  begin
    isrx? if
      pbuff 1536 erase
      pbuff 1536 get_ether_pkt
cr ." Packet length = " dup .dword
      pbuff swap 1 max $1518 min dump
    then
[ tasking? ] [if] pause [then]
    key?
  until
;

: Tsniff	\ --
\ *G Checks the network and displays a received packet.
  isrx? if
    pbuff 1536 erase
    pbuff 1536 get_ether_pkt .hex
    pbuff $40 dump
  then
[ tasking? ] [if] pause [then]
;

[then]


\ ******
\ *> ###
\ ******

decimal
