\ AX88976 and NE2000 compatible Ethernet driver

((
Copyright (c) 2004, 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeltd.demon.co.uk
     tech-support@mpeltd.demon.co.uk
web: www.mpeltd.demon.co.uk

Portions of this code have been licensed from:
  Oxford Network Solutions
  Tel: 01869 320461
  Fax: 01869 327065
  Internet: grahams@oxns.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ =========
\ *! ne2000
\ *T NE2000 Compatible Ethernet Driver
\ =========
\ *S Introduction
\ *P This is the hardware driver layer for NE2000 compatible
\ ** Ethernet controller chips such as the Realtek RTL8019AS and
\ ** the Asix AX88796 chips used on the MPE ARM EtherStamp boards.
\ ** The data sheets for these devices may be found at
\ *C   www.realtek.com.tw
\ *C   www.asix.com.tw

\ ===================
\ *S Hardware gotchas
\ ===================
\ *P This code assumes byte accesses to the chip. This restricts
\ ** internal RAM to 8kb. When operating with fast CPUs note that
\ ** the back to back access time may need to be controlled.

\ *B The NE2000 access words *\fo{EC@}, *\fo{EC!}, *\fo{EW@} and
\ ** *\fo{EW!} assume that the chip is driven from I/O
\ ** ports. If you are not using the EtherStamp boards, you will
\ ** have to modify these for your own use.


\ ****************
\ *S Configuration
\ ****************
\ *P The following can be defined before the file *\i{DRIVERS\NE2000.FTH}
\ ** is compiled.

\ =====================
\ *N Hardware selection
\ =====================

[undefined] HasPHY? [if]
1 equ HasPHY?	\ -- flag
\ *G The original NE2000 was a 10 Mbps device. Later chips for
\ ** 10/100 Mbps usually perform autonegotiation at powerup. Set
\ ** this equate non-zero to include chip specific PHY handling
\ ** code. The example code is for the Asix AX88796.
[then]

[undefined] EEprom? [if]
0 equ eeprom?   \ -- flag
\ *G Set this equate true if the NE2000 has an attached EEPROM
\ ** for configuration data storage. False by default.
\ ** Note that no EEPROM driver code is provided and if set,
\ ** stubs will be compiled.
[then]


\ =====================
\ *N Software selection
\ =====================

[undefined] EtherDiags? [if]
1 equ EtherDiags?   \ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. Non-zero by default.
[then]

[undefined] Sniff? [if]
1 equ Sniff?   \ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress   \ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $20 c,   \ six bytes for 48 bit MAC address
					\ you must obtain these yourself!
[then]

[undefined] IpAddress [if]
create IpAddress   \ -- addr
\ *G Holds the Ethernet IP address (four bytes).
  192 c,  168 c,  1 c,  251 c,		\ assign these as required
[then]

[undefined] #DMAwait [if]
#1000 equ #DMAwait	\ -- u ; 1..u, NOT 0.
\ *G Defines the number of iterations for which DMA operation
\ ** is polled in *\fo{waitDMA} below before returning failure.
\ ** This value may need to be increased on fast hardware.
[then]

[undefined] GenericIP? [if]
0 equ GenericIP?	\ -- n
\ *G Some versions of PowerNet v3+ with multiple IP devices use
\ ** a generic IP device structure. For these versions set this
\ ** equate non-zero. The default is zero.
[then]


\ *******************
\ NE2000 compatible registers
\ *******************

\ Page 0 and common registers
#define ne2kCR      0x00	\ all pages
#define ne2kCLDA0   0x01	\ r/o
#define ne2kPSTART  0x01	\ w/o - page 2 r/o
#define ne2kCLDA1   0x02	\ r/o
#define ne2kPSTOP   0x02	\ w/o - page 2 r/o
#define ne2kBNRY    0x03
#define ne2kTSR     0x04	\ r/o
#define ne2kTPSR    0x04	\ w/o
#define ne2kNCR     0x05	\ r/o
#define ne2kTBCR0   0x05	\ w/o
#define ne2kFIFO    0x06	\ r/o
#define ne2kTBCR1   0x06	\ w/o
#define ne2kISR     0x07	\ r/w
#define ne2kCRDA0   0x08	\ r/o
#define ne2kRSAR0   0x08	\ w/o
#define ne2kCRDA1   0x09	\ r/o
#define ne2kRSAR1   0x09	\ w/o
#define ne2kRBCR0   0x0A
#define ne2kRBCR1   0x0B
#define ne2kRSR     0x0C	\ r/o
#define ne2kRCR     0x0C	\ w/o
#define ne2kCNTR0   0x0D	\ r/o
#define ne2kTCR     0x0D	\ w/o
#define ne2kCNTR1   0x0E	\ r/o
#define ne2kDCR     0x0E	\ w/o
#define ne2kCNTR2   0x0F	\ r/o
#define ne2kIMR     0x0F	\ w/o
#define ne2kRDMAPORT 0X10	\ all pages
#define ne2kRSTPORT 0x1F	\ all pages 

\ Page 1 is r/w
#define ne2kPAR0    0x01	\ 1..6
#define ne2kCURR    0x07
#define ne2kMAR0    0x08	\ 8..F

\ Page 2 is to read W/O regs in Page 0

\ Page 3
#define ne2kCR9346   0x01


\ initial register values
$04 equ Rcrval
$00 equ Tcrval
$58 equ Dcrval
\ PXR and OVW interrupt enabled
$11 equ Imrval
$40 equ Txtstart
$46 equ Rxstart
$60 equ Rxstop
\ ISR Register definitions
$80 equ Rst
$40 equ Rdc
$10 equ Ovw
$01 equ Prx


\ *********************
\ *S Hardware Interface
\ *********************
\ *P The default code is for a bit-banged device attached to the
\ ** GPIO system on a TiniARM board. Change this to suit your
\ ** hardware.

$0000FF00 equ edbmask		\ -- mask ; ethernet data bus mask
$200000F0 equ eabmask		\ -- mask ; ethernet address bus mask
bit31     equ nIOWR		\ -- mask ; IOWR bit - active low
bit30     equ nIORD		\ -- mask ; IORD bit - active low
bit29     equ SA4		\ -- mask ; SA4 address line
bit16     equ nINT		\ -- mask ; interrupt bit - active low
nIORD  nIOWR or  eabmask or
  equ EOPs			\ -- mask ; always outputs
EOPs  nINT or  edbmask or
  dup equ etherbits		\ -- mask ; all ethernet bits
  invert equ ~etherbits		\ -- mask ; all non-ethernet bits

$000000FF equ PS0gpio		\ -- mask ; ensure iopins 4-15 are GPIO
$03FFFFFC equ PS1gpio		\ -- mask ; ensure iopins 16,29-31 are GPIO

L: ^GPIOe	\ -- addr ; saves target space
  _GPIO ,

: erdmode	\ --
\ *G Turns data bus for reading.
  ^GPIOe @
  dup IODIR + @
  edbmask invert and
  swap IODIR + !
;

: ewrmode	\ --
\ *G Turns data bus for writing
  ^GPIOe @
  dup IODIR + @
  edbmask or
  swap IODIR + !
;

: ne2kaddr	\ reg --
\ *G Write NE2000 address.
  $1F and			\ clip to 5 bits
  4 lshift			\ A3..0 to P0.7..4
  dup #21 lshift or		\ A25 to P0.29
  eabmask dup ^GPIOe @ tuck	\ -- n' mask addr mask addr
  IOCLR + !			\ -- n' mask addr ; clear all address bits
  -rot  and  swap IOSET + !
;

: ec@      \ reg -- data
\ *G Read byte from NE2000 register n.
  erdmode			\ set up iopin dir for reading
  NE2000addr			\ set address on pins
  ^GPIOe @
  nIORD over IOCLR + !		\ nIORD low
  [asm  nop  nop  nop  asm]	\ 48 ns
  dup IOPIN + @			\ read
  nIORD rot IOSET + !		\ nIORD high
  8 rshift $000000FF and
;

: ec!      \ data n --
\ *G Write byte to NE2000 register n.
  ewrmode			\ set up iopin dir for writing
  NE2000addr			\ set address on pins
  ^GPIOe @			\ -- data addr
  edbmask over IOCLR + !	\ clear all data lines
  swap $FF and 8 lshift 
  over IOSET + !		\ set 1 bits
  nIOWR over IOCLR + !		\ nIOWR low
  [asm  nop  nop  nop  asm]	\ 48 ns
  nIOWR swap IOSET + !		\ nIOWR high
;

: ew@      \ reg -- data16
\ *G Read 16 bits from NE2000 register pair n, n+1.
  dup ec@			\ low byte from low address
  swap 1+ ec@ 8 lshift or	\ high byte from high address
;

: ew!      \ data16 reg --
\ *G Write 16 bits to NE2000 register pair n, n+1.
  2dup ec!			\ low byte to low address
  swap 8 rshift swap 1+ ec!	\ high byte to high address
;

: eblkwrite   \ addr dreg len --
\ *G Write len bytes starting at addr into consecutive 
\ ** registers starting at dreg.
  bounds
  ?do  dup c@ i ec!  1+  loop
  drop
;

: InitNE2000IO   \ --
\ *G Initialise access to the NE2000 hardware. For i/o port
\ ** access this involves setting up port I/O direction registers
\ ** and so on.
  _PINSEL
    dup PINSEL0 + @		\ set GPIO4..15 to GPIO mode
    PS0gpio and
    over PINSEL0 + !
    dup PINSEL1 + @		\ set GPIO16,29..31 to GPIO mode
    PS1gpio and
    swap PINSEL1 + !
  ^GPIOe @
  nIORD nIOWR or nINT or 
  over IOSET + !		\ set read and write high
  eabmask edbmask or
  over IOCLR + !		\ set other o/ps low
  dup IODIR + @
    ~etherbits and		\ mask Ethernet bits
    EOPs or			\ set Ethernet outputs
  swap IODIR + !
;


\ ************************
\ *S AX88796 PHY interface
\ ************************
\ *P PHY register mappings are mostly very similar. However,
\ ** the interface between the MAC and the PHY varies widely.
\ ** If you are not using the Asix chip, modify this code for
\ ** the chip you are using. This code is only compiled if the
\ ** equate *\fo{HasPHY?} is set non-zero.

HasPHY? [if]

#define MII_MEMR 0x14	\ all pages

#0 equ #phydelay	\ -- n
\ *G Set this equate non-zero if a delay is required
\ ** after each clock transition when reading/writing
\ ** PHY registers. Tune this value for your hardware.

#phydelay [if]
: phyclkdelay	\ --
\ *G Delay routine for clocking the PHY registers.
  #phydelay 0 do loop
;
[else]
compiler
: phyclkdelay	\ --
;
target
[then]

: PHYout	\ val --
\ *G Write VAL to the MEMR register.
  MII_MEMR ec!  phyclkdelay
;

\ MEMR bits
\ 7, $80 - op, EECK, leave low
\ 6, $40 - op, EEO, leave high
\ 5, $20 - ip, EEI, ignore
\ 4, $10 - op, EECS, must be low
\ 3, $08 - op, MDO, data to PHY
\ 2, $04 - ip, MDI, data from PHY
\ 1, $02 - op, MDIR, 1 for read, 0= write PHY
\ 0, $01 - op, MDC, clock state

: ckPHYout	\ x --
\ *G Write a bit to the PHY, 1 if x=nz or 0 if x=0.
  if
    $48 PHYout			\ op data bit
    $49 PHYout			\ raise clock bit
    $48 PHYout			\ drop clock again
  else
    $40 PHYout			\ op data bit
    $41 PHYout			\ raise clock bit
    $40 PHYout			\ drop clock again
  endif
;

: ckPHYin	\ -- 0|1
\ *G Read a bit from the PHY. The first data bit is available
\ ** after the 16th clock pulse, i.e. read must set the turn
\ ** around bit = 0.
  $42 PHYout			\ MDIR=1, MDO=0
  $43 PHYout			\ raise clock bit
  $42 PHYout			\ drop clock again
  MII_MEMR ec@ $04 and 2 rshift	\ read data bit from PHY
;

: prepPHY	\ --
\ *G Prepare a PHY transfer.
  #32 0
  do  1 ckPHYout  loop
;

: readPHY	\ reg -- w
\ *G Read the requested PHY register (16 bits).
  \ first form the data in as 32 bits
  #18 lshift  $68000000 or	\ ST = 01(2), op = 10(2), PHYadd = 10(5), REGadd = reg(5), TA=00(2)
  prepPHY
  #15 0 do			\ 16th bit provided by read routine (MDO=0)
    dup $80000000 and ckPHYout
    1 lshift
  loop
  drop  0
  #16 0 do
    1 lshift  ckPHYin or
  loop
  1 ckPHYout			\ 32nd clock bit
;

: writePHY	\ val reg --
\ *G Write VAL to PHY register REG.
  \ first form the data in as 32 bits
  $1F and  #18 lshift			\ set up register address
  swap $FFFF and  or			\ merge in data
  $58020000 or	\ ST = 01(2), op = 01(2), PHYadd = 10(5), REGadd = reg(5), TA=10(2)
  prepPHY
  #32 0 do
    dup $80000000 and ckPHYout
    1 lshift
  loop
  drop
;

: AutoNegDone?	\ -- flag
\ *G Return non-zero if autonegotiation is complete.
  1 readPHY bit5 and
;

: resetPHY	\ --
\ *G Resets the PHY interface and autonegotiates the connection.
  $0800 0 writePHY  #2500 ms		\ power down for 2.5 secs
  $01E1 4 writePHY			\ select what we need
  $3300 0 writePHY			\ start autonegotiation
  #25 0 do				\ allow 2.5 seconds to complete
    AutoNegDone? ?leave			\ check for completion
    #100 ms
  loop
  cr ." Autonegotiation " AutoNegDone? if
    5 ReadPHY				\ get autonegotiation results
    dup bit8 bit7 or and		\ check for 100/10
    if  ." 100"  else  ." 10"  endif
    bit8 bit6 or and			\ check duplex
    if  ." /FDX"  else  ." /HDX"  endif
  else
    ." bad"
  endif
;

[then]


\ *************************
\ *S Generic NE2000 driver
\ *************************

variable next_pkt	\ -- addr
\ *G Holds the next RDMA packet to be read when receiving
\ ** packets. The use of this variable avoids problems with
\ ** some NE2000 clones.


\ *****************
\ *N Initialisation
\ *****************

: (initNE2000)	\ --
\ *G The initialisation primitive is used at powerup and for
\ ** error recovery.
  $21 ne2kCR ec!  2 ms			\ stop command, page 0
  DCRval ne2kDCR ec!			\ Data Config: little endian byte transfers
  0 dup ne2kRBCR0 ec!  ne2kRBCR1 ec!	\ DMA count=0

  $04 ne2kRCR ec!			\ Rx Config: for us and broadcast
  Txtstart ne2kTPSR ec!			\ Tx Config: start page in local RAM
  $02 ne2kTCR ec!			\ loopback to inhibit reception during set up
  Rxstart dup ne2kPSTART ec! 		\ set up receive ring
    ne2kBNRY ec!
  Rxstop ne2kPSTOP ec!

  $61 ne2kCR ec!  2 ms			\ stop command, page 1
  Rxstart 1+ dup
  ne2kCURR ec!				\ first receive ring page
  next_pkt !				\ first pkt to be received
  EtherAddress ne2kPAR0 
  6 eblkwrite				\ Ethernet MAC address

  $21 ne2kCR ec!			\ stop command, page 0
  DCRval ne2kDCR ec!			\ Data Config: little endian byte transfers

  $22 ne2kCR ec!			\ start command, page 0
  $FF ne2kISR ec!			\ clear all interrupt flags
  Imrval ne2kIMR ec!			\ enable overrun and receive interrupts
  $00 ne2kTCR ec!			\ normal operation
;

: initNE2000	\ --
\ *G Reset and initialise the NE2000 chip.
  ne2kRSTPORT ec@ ne2kRSTPORT ec!	\ initialisation
  #100 ms
  ne2kISR ec@ bit7 and 0=		\ test we entered reset
  if  cr ." Cannot reset Ethernet"  exit  endif
  (initNE2000)
[ HasPHY? ] [if]  ResetPHY  [then]
;

: InitEther   \ --
\ *G Initialise I/O to NE2000 and initialise the NE2000.
\ ** By default this is done at powerup. During testing you
\ ** may wish to comment out the line:
\ *C   ' InitEther AtCold
  InitNE2000IO  InitNE2000
;
' InitEther AtCold


\ **********************
\ *N Receive buffer ring
\ **********************

EtherDiags? [if]
: epage		\ p --
\ *G Select NE2000 page.
  6 lshift
  0 ec@ $3F and or 0 ec!
;

: .ring		\ code --
\ *G Display the given code and the state of the receive buffer
\ ** ring.
  . 
  next_pkt @ .byte space
  ne2kBNRY ec@ .byte space
  1 epage
  ne2kCURR ec@ .byte space
  0 epage
;
[then]

: IsRx?         \ -- flag
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
\  ne2kISR ec@ $01 and
  $62 ne2kCR ec!  ne2kCURR ec@		\ select page 2, read CURR
  $22 ne2kCR ec!  ne2kBNRY ec@		\ select page 0, read BNRY
  2dup <=				\ check for wraparound
  if  RxStop RxStart - -  endif
  1+ <>					\ if BNRY +1 = CURR we are empty
;

: UpdateRXring	\ nextp --
\ *G Given the next receive packet, update the receive ring.
  dup next_pkt !			\ save this for next receive
  1- dup RxStart <
  if  drop  RxStop 1-  endif
  ne2kBNRY ec!
;


\ ***********************
\ *N Receive and Transmit
\ ***********************

: setDMA	\ addrne2k len --
\ *G Set up the RDMA for *\b{len} bytes to/from
\ ** *\b{addrne2k} in the NE2000 RAM. The transfer is not performed.
  $40 ne2kISR ec!			\ clear RDMA interrupt
  $51 ne2kIMR ec!			\ enable RDMA interrupt
  ne2kRBCR0 ew!  ne2kRSAR0 ew!  
;

: waitDMA	\ -- flag
\ *G Wait a short time for RDMA to complete, returning zero
\ ** if the DMA has completed.
  #DMAwait 0 do
    ne2kISR ec@ $40 and
    if  0  unloop exit  endif
  loop
  -1
;

: getRXheader	\ -- len status nextp
  next_pkt @ $FF and 8 lshift		\ RDMA address for next read
  4 setDMA				\ prep. for four bytes
  $0A ne2kCR ec!			\ start the remote read transfer
  ne2kRDMAPORT ec@			\ receive status
  ne2kRDMAPORT ec@ 			\ next packet
  ne2kRDMAPORT ec@ $FF and		\ get received packet size
  ne2kRDMAPORT ec@ 8 lshift or		\ -- status nextp len
  -rot 					\ -- len status nextp
  waitDMA drop 				\ abort" Header DMA read failure"
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get a pending receive packet to a buffer of size *\b{maxlen).
\ ** Return the number of bytes received, or zero for a bad packet.
\ ** Note that data is valid only when *\fo{ISRX?} returns true,
\ ** so that you must poll with *\fo{ISRX?} before using
\ ** *\fo{GET_ETHER_PKT}. The received size of the packet
\ ** (not clipped to the buffer size) is returned.
  getRXheader >r  1 and
  over #1518 u> not and
  if		\ -- *dest maxlen len ; good packet?
    4- dup rot min			\ -- *dest len len' ; clip len to buffer size
    swap -rot				\ -- len *dest len' ; save length actually read
    next_pkt @ $FF and 8 lshift 4 +	\ RDMA address for next read
    over setDMA				\ prep. for packet read
    $0A ne2kCR ec!			\ start the remote read transfer
    bounds
    ?do  ne2kRDMAPORT ec@ i c!  loop	\ -- len
    waitDMA drop 			\ abort" Data DMA read failure"
  else		\ -- *dest maxlen len ; bad packet
[ EtherDiags? ] [if]  cr ." RXerr " dup .ring  [then]
    drop 2drop  0			\ return failed len
    r> drop  (initNE2000)  exit		\ brute force recovery
  endif
\ update the ring buffer and clean up
  r> UpdateRXring			\ now update the next pkt stuff
  $FF ne2kISR ec!			\ clear NE2000 interrupt reg
\ [ EtherDiags? ] [if]  cr ." rx " dup .ring  [then]
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer
\ ** memory.
  getRXheader nip nip UpdateRXring
  $FF ne2kISR ec!			\ clear NE2000 interrupt reg
;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop true
;

: send_ether_pkt        \ caddr len --
\ *G Send packet from supplied buffer.
\ ** The word will block until sufficient packet memory is available.
\ ** Note that you should read incoming packets regularly, otherwise
\ ** it is possible to get into a situation in which *\fo{SEND_ETHER_PKT}
\ ** blocks for ever because there is not enough space in the
\ ** device for the transmission packet. Theoretically, this should
\ ** not happen because 1536 bytes are reserved for transmission,
\ ** but ...
  $40 max  #1518 min			\ force min packet size
  $FF ne2kISR ec!			\ clear interrupts
  Txtstart 8 lshift  over setDMA	\ setup for Tx DMA
  dup ne2kTBCR0 ew!			\ transmit byte count
  $012 ne2kCR ec!			\ start the DMA
  bounds ?do			        \ transfer data to RDMA
    i c@ ne2kRDMAPORT ec!
  loop
  waitDMA drop \ abort" Tx DMA write failure"
  Txtstart ne2kTPSR ec!			\ set Tx start page
  $24 ne2kCR ec!			\ start transmission
  begin					\ wait until done
    ne2kISR ec@ $0A and 0=
   while
[ tasking? ] [if]  pause  [then]
  repeat
  $FF ne2kISR ec!			\ clear interrupts
;


\ ******************
\ *S Attached EEPROM
\ ******************

eeprom? [if]

cr ." NE2000 EEPROM code not implemented"  abort

: ee!           \ word offset --
\ *G Write a 16 bit word into the EEPROM at EEPROM address
\ ** offset. The EEPROM may not be present in all
\ ** implementations.
  2drop
;

: ee@           \ offset -- word
\ *G Read a 16 bit word from the EEPROM at EEPROM address
\ ** offset. The EEPROM may not be present in all implementations.
  drop $FFFF
;

[then]


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************
\ *P The code in this section is only compiled if the equate
\ ** *\fo{GenericIP?) has been defined and is non-zero.

\ *P The layout and usage of this structure is defined in the
\ ** file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,		\ 0: initialisation
\ **   ' MyTerm ,		\ 1: shutdown
\ **   ' MyRx? ,		\ 2: receive test
\ **   ' MyRx			\ 3: receive packet
\ **   ' MyTx? ,		\ 4: transmit test
\ **   ' MyTx ,			\ 5: transmit packet
\ **   ' MyGetAddr ,		\ 6: Get device addresses
\ **   ' MySetAddr ,		\ 7: Set device addresses
\ **   ' MySave ,		\ 8: Save IP device state

GenericIP? [if]

: GetAddrs   \ -- ipaddr macaddr 0
\ *G For Generic I/O *\fo{IPDGetAddr} function.
  IpAddress EtherAddress 0
;

: SetAddrs   \ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the stored data
\ ** will not be changed.
  drop				\ flags
  ?dup if			\ macaddr
    dup EtherAddress 6 cmove
    etheradd>CS
  endif
  ?dup if			\ ipaddr
    IpAddress 4 cmove
  endif
;

create NE2vector   \ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** COMMON\ETHERCOM.FTH.
  ' InitEther ,			\ initialisation      
  ' noop ,			\ shutdown            
  ' IsRx? ,			\ receive test        
  ' get_ether_pkt ,		\ receive packet      
  ' IsTx? ,			\ transmit test       
  ' send_ether_pkt ,		\ transmit packet     
  ' GetAddrs ,			\ Get device addresses
  ' SetAddrs ,			\ Set device addresses
  ' noop ,			\ Save IP device state

NE2vector constant IPDevice   \ -- addr
\ *G IPDevice is the default device name required by *\i{ETHERCOM.FTH}.
\ ** If you have multiple ports, only one should be called *\fo{IPDevice}.

[then]

  
\ **************
\ *S System test
\ **************
\ *P This code is only compiled if the equate *\fo{SNIFF?}
\ ** is non-zero.

sniff? [if]

#1536 buffer: pbuff   \ -- addr
\ *G Buffer for *\fo{SNIFF}.

: sniff      \ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  InitEther
  begin
    isrx? if
      pbuff 1536 erase
      pbuff 1536 get_ether_pkt
cr ." Packet length = " dup .dword
      pbuff swap 1 max $1518 min dump
    then
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]


\ *******************
\ Diagnostic routines
\ *******************
\ *P This code is only compiled if the equate *\fo{EtherDiags?} is
\ ** set non-zero.

EtherDiags? [if]

: edump      \ --
\ *G Dump NE2000 registers.
  cr  $10 0 do
    i ec@ .byte space
  loop
  cr  $20 $10 do
    i ec@ .byte space
  loop
  cr
;

: ep		\ --
\ *G Dump NE2000 registers in currently selected page.
  cr  $10 0 do
    i ec@ .byte space
  loop
;

: ed		\ --
\ *G Dump NE2000 registers for all pages.
  0 epage ep
  1 epage ep
  2 epage ep
  3 epage ep
  0 epage
  cr
;

: .phy		\ --
\ *G Display contents of all Ethernet Controller PHY registers.
  cr ."  0 Control: " 0 readPHY .word
  cr ."  1  Status: " 1 readPHY .word
  cr ."  2 PhyID#1: " 2 readPHY .word
  cr ."  3 PhyID#2: " 3 readPHY .word
  cr ."  4 AutoNeg: " 4 readPHY .word
  cr ."  5 AutoRem: " 5 readPHY .word
  cr ." 16 Config1: " #16 readPHY .word
  cr ." 17 Config2: " #17 readPHY .word
  cr ." 18 StatOut: " #18 readPHY .word
  cr ." 19    Mask: " #19 readPHY .word
;

[then]


\ ******
\ *> ###
\ ******


