\ Ether9200.fth - AT91RM9200 Ethernet driver

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ =========
\ *! ether9200
\ *T AT91RM9200 Ethernet Driver Code
\ =========
\ *S Introduction
\ *P This code is derived from the sample C driver on the
\ ** Atmel AT91DVD. It was tested on the Atmel AT91RM9200-EK
\ ** board.

\ *P There are several gotchas when using the AT91RM9200 EMAC.
\ *(
\ *B Cache coherency for ARM920T cores is achieved by software. See
\ ** ARM DDI 0151C section 4.4 for more details. Because DMA causes
\ ** cache problems, the Ethernet receive headers and buffers should be
\ ** in uncached memory to avoid requiring cache flush operations.
\ ** Similarly, a transmit buffer is defined in uncached memory.
\ *B If interrupt operation is not used, as in this case, interrupts
\ ** disabled in the EMAC block do not set bits in the EMAC interrupt
\ ** status registers.
\ *B The EMAC ISR (Interrupt status register) clears all bits when
\ ** read.
\ *)


\ ****************
\ *S Configuration
\ ****************
\ *P The following must be defined before the file
\ ** *\i{DRIVERS\Ether9200.FTH} is compiled.

interpreter
: const equ ;	\ n -- ; -- n
\ *G If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** *\fo{CONST} as *\fo{CONSTANT}, otherwise by default *\fo{CONST}
\ ** is defined as *\fo{EQU}.
target

[undefined] E92Diags? [if]
1 equ E92Diags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. False by default.
[then]

[undefined] RMII? [if]
1 equ RMII?	\ -- flag
\ *G Set this if the hardware uses 2-bit data access to the PHY
\ ** rather than four-bit.
[then]

[undefined] MultiPhy? [if]
1 equ MultiPhy?	\ -- flag
\ *G Set this flag true if you are using multiple PHYs or do not
\ ** know the address of the PHY.
[then]

[undefined] sniff? [if]
1 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] pingit? [if]
1 equ pingit?	\ -- flag
\ *G Set this equate true to compile the PING code,
\ ** which tests Ethernet transmission and reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $51 c,	\ six bytes for 48 bit MAC address
\  $00 c, $08 c, $EF c, $00 c, $5E c, $C4 c,	\ six bytes for 48 bit MAC address
[then]						\ you must obtain these yourself!

[undefined] IpAddress [if]
create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
\  10 c,  102 c,  0 c,  11 c,		\ assign these as required
  192 c,  168 c,  1 c,  251 c,		\ assign these as required
[then]


\ ************
\ *S Buffers
\ ************

#1536 equ /ERxBuff	\ -- u
\ *G Size of an Ethernet receive buffer rounded up to 16 byte unit.
\ ** If the buffers are in cached memory, they should also be rounded
\ ** to a cache line (64 bytes).
#8 equ #ERxBuffs		\ -- u
\ *G Number of receive buffers. Must be at least two. The
\ ** maximum number is 1024.
#1536 equ /ETxBuff	\ -- u
\ *G Size of an Ethernet transmit buffer rounded up to 16 byte unit.
#1 equ #ETxBuffs		\ -- u
\ *G Number of transmit buffers. Must be at least one. Transmit buffers
\ ** are managed in software.

struct /ERxHdr	\ -- len
\ *G Structure defining a receive header.
\ *[
  int rh.addr		\ address of buffer
  int rh.status		\ receive status
end-struct
\ *]

[sections  CpuRAM	\ put headers and buffers in uncached RAM
uhere #63 + #-64 and uorg	\ force UDATA to 64 byte boundary
uhere constant ERxHdrs	\ -- addr
\ *G Base address of the receive headers. The receiver steps
\ ** through these headers to find a free buffer. The headers
\ ** *\b{must} start on a 64 byte boundary because of a chip bug.
\ ** The headers should be in *\b{uncached} memory.
  #ERxBuffs /ERxHdr * uallot

uhere #63 + #-64 and uorg	\ force UDATA to 64 byte boundary
uhere constant ERxBuffs	\ -- addr
\ *G Base address of the receive buffers. Some buffer handling
\ ** techniques may require these buffers to be in uncached
\ ** memory.
  #ERxBuffs /ERxBuff * uallot

uhere #63 + #-64 and uorg	\ force UDATA to 64 byte boundary
uhere constant ETxBuffs	\ -- addr
\ *G Base address of the transmit buffers. These should not be
\ ** in cached memory.
  #ETxBuffs /ETxBuff * uallot
sections]

variable FirstRxHdr	\ -- addr
\ *G Holds the address of the first read header to check.
  ERxHdrs FirstRxHdr !
variable NextRxHdr	\ -- addr
\ *G Holds the address of the next read header containing unhandled
\ ** data. Set to zero to indicate none or unknown.
  0 NextRxHdr !

: InitERxHdrs	\ --
\ *G Initialise the receive headers.
  ERxHdrs FirstRxHdr !  0 NextRxHdr !	\ mark no data available
  ERxBuffs ERxHdrs  #ERxBuffs 1- 0 do	\ -- buff hdr
    2dup rh.addr !  dup rh.status off	\ initialise structure
    swap /ERxBuff + swap /ERxHdr +	\ step to next
  loop					\ -- buff' hdr'
  swap bit1 or over rh.addr !		\ last buffer has WRAP bit set
  rh.status off
;


\ *************
\ *S PHY access
\ *************

$60020000 equ ~RdPhy	\ -- mask
\ *G Mask for PHY read operations. Assumes use of PHY0.
$50020000 equ ~WrPhy	\ -- mask
\ *G Mask for PHY write operations. Assumes use of PHY0.

\ PHY registers
0x00 const PHY_CNTL_REG
0x01 const PHY_STATUS_REG
0x02 const PHY_ID_REG1
0x03 const PHY_ID_REG2
0x04 const PHY_ANA_REG
0x05 const PHY_ANLPAR_REG
0x06 const PHY_ANE_REG
0x10 const PHY_ECNTL_REG1
0x11 const PHY_QPDS_REG
0x12 const PHY_10BOP_REG
0x13 const PHY_ECNTL_REG2

\ PHY control register
bit15 const RESET_PHY
bit14 const ENABLE_LOOPBACK
bit13 const DR_100MB
bit12 const ENABLE_AN
bit10 const PHY_MAC_ISOLATE
bit9  const RESTART_AN
bit8  const PHY_FULLDUPLEX
bit7  const PHY_COL_TEST

\ PHY Status register
bit5 const AN_COMPLETE

\ PHY Auto-negotiation Advertisement register
bit8 const DR100_TX_FULL
bit7 const DR100_TX_HALF
bit6 const DR10_TX_FULL
bit5 const DR10_TX_HALF
DR100_TX_FULL DR100_TX_HALF or DR10_TX_FULL or
  const ANE_ALL	\ request everything

: WaitPhyDone	\ --
\ *G Wait until the current PHY operation has completed.
((
  begin
    _EMAC EMAC_ISR + @ bit0 and 0=	\ interrupt must be enabled
  until
))
  #10 ms
;

\ *P According to the AT91RM9200-EK example software, the default
\ ** PHY is 0. By experimentation, we found this to be uncertain, 
\ ** and the default may be 8 under some reset conditions. Two
\ ** versions of the following two words are provided, the first
\ ** allowing use of a current PHY, the second for a single known
\ ** PHY. The first version is used if the equate *\fo{MultiPhy?}
\ ** is non-zero.

MultiPhy? [if]
0 value Phy#	\ -- n
\ *G Multi PHY: The number of the PHY in use: 0..31
: ~Phy#		\ -- mask
\ *G Multi PHY: Mask used for PHY command generation.
  Phy# #23 lshift
;
[else]
0 equ Phy#	\ -- n
\ *G Single PHY: The number of the PHY in use: 0..31
Phy# #23 lshift equ ~Phy#	\ -- mask
\ *G Single PHY: Mask used for PHY command generation.
[then]

: ReadPhy	\ reg -- w
\ *G Read a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  $1F and #18 lshift			\ form reg# mask
  ~RdPhy or  ~Phy# or			\ set up reg command
  _EMAC EMAC_MAN + !			\ set command
  WaitPhyDone				\ let operation complete
  _EMAC EMAC_MAN + @ $FFFF and		\ return data
;

: WritePhy	\ w reg --
\ *G Write a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  $1F and #18 lshift			\ form reg# mask
  ~WrPhy or  ~Phy# or			\ set up reg command
  swap $FFFF and or			\ merge in data
  _EMAC EMAC_MAN + !			\ set command
;

: OrPhy		\ mask reg --
\ *G Or mask into reg in the PHY. Set bits in mask are set in the
\ ** register.
  tuck ReadPhy  or  swap WritePhy
;

: InitPhy	\ -- ; autonegotiation version
\ *G Initialise the PHY with autonegotiation
  RESET_PHY PHY_CNTL_REG OrPhy		\ software reset the chip
  begin					\ wait until reset complete
    PHY_CNTL_REG ReadPhy RESET_PHY and 0=
  until
  ANE_ALL PHY_ANA_REG OrPhy		\ request 10/100BaseT features
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
  RESTART_AN PHY_CNTL_REG OrPhy		\ start autonegotiation
  begin					\ wait until done
    PHY_STATUS_REG ReadPhy
    AN_COMPLETE and
  until
  PHY_ANLPAR_REG ReadPhy
\ set up control reg using the results
  0					\ -- partner cntlset
  over DR100_TX_FULL and
  if  DR_100MB PHY_FULLDUPLEX or or  endif
  over DR100_TX_HALF and
  if  DR_100MB or  endif
  over DR10_TX_FULL and
  if  PHY_FULLDUPLEX or  endif
  PHY_CNTL_REG WritePhy
  drop
;


\ *****************
\ *S Initialisation
\ *****************

$00001FF80 equ EMACpins	\ -- mask
\ *G Bit mask for Ethernet pins, PA16..7 on PIOA

RMII? [if] bit13 [else] 0 [then] equ ~RMII	\ -- mask
\ *G Initialisation bit mask for CTL reg if RMII is used.

/MCK 2* 8 / 5000000 <= [if]
  0
[else]
  /MCK 2* 16 / 5000000 <= [if]
    1
  [else]
    /MCK 2* 32 / 5000000 <=
    [if]  2  [else]  3  [then]
  [then]
[then]
#10 lshift equ ~MDC	\ -- mask
\ *G Clock mask for MDC. Calculated from *\fo{/MCK}.
      
((
For a MAC address conventionally defined as:
  aa-bb-cc-dd-ee-ff
the layout of the SAxL/H registers is:
SAxL:
        ---------------------
    MSB | cc | dd | ee | ff | LSB
        ---------------------
SAxH:
        ---------------------
    MSB | 00 | 00 | aa | bb | LSB
        ---------------------
The low register must be set first.
))

: setMACAddress	\ addr --
\ *G Set the MAC address in the EMAC unit.
  dup 5 + c@
  over 4 + c@ 8 lshift or
  over 3 + c@ #16 lshift or
  over 2 + c@ #24 lshift or
  _EMAC EMAC_SA1L + !
  dup 1 + c@  swap 0 + c@ 8 lshift or
  _EMAC EMAC_SA1H + !
;

create InitEtherIOTab	\ -- addr
\ *G Initialisation table for Ethernet I/O access. Disables
\ ** Ethernet interrupt.
  _PMC PMC_PCER + ,		EMACmask ,	\ power up Ethernet
  _PIOA PIO_ASR + ,		EMACpins ,	\ EMAC pins are periph A
  _PIOA PIO_BSR + ,		0 ,
  _PIOA PIO_ODR + ,		EMACpins ,	\ disable PIO outputs
  _PIOA PIO_IDR + ,		EMACpins ,	\ disable PIO interrupt
  _PIOA PIO_PDR + ,		EMACpins ,	\ disable PIO
  _AIC AIC_IDCR + ,		EMACmask ,	\ Disable EMAC interrupt at AIC
  0 ,

create InitEtherTab	\ -- addr
\ *G Initialisation table for Ethernet controller. Does not
\ ** perform the PIO disabling.
  _AIC AIC_IDCR + ,		EMACmask ,	\ Disable EMAC interrupt at AIC
  _EMAC EMAC_IDR + ,		$0FFF ,		\ Disable all EMAC interrupt sources
  _EMAC EMAC_CTL + ,		$0030 ,		\ Enable MDIO/PHY access, clear stats
  _EMAC EMAC_CFG + ,   $0013 ~RMII or ~MDC or ,	\ set CFG register, 100M Half Duplex.
  _EMAC EMAC_TSR + ,		$007F ,		\ clear transmit status bits
  _EMAC EMAC_RSR + ,		$0007 ,		\ clear receive status bits
  _EMAC EMAC_RBQP + ,		ERxHdrs ,	\ receive queue head
((
  _AIC AIC_SMR0 + ID_EMAC cells + ,  $0007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + ID_EMAC cells + ,  EmacISR ,	\ ISR handler
  _AIC AIC_IECR + ,		EMACmask , 	\ Enable the DBGU interrupt
))
  0 ,

: InitEMAC	\ -- flag ; nz=success
\ *S Initialisation of the Ethernet and PHY, including the MAC
\ ** address. Note that interrupts are disabled for a while. If
\ ** this is problematic, replace the interrupt disables with
\ ** a semaphore.
  [i  InitEtherIoTab init-io  i]	\ get access to EMAC unit
  InitERxHdrs				\ set up receive headers
  EtherAddress setMACaddress		\ set up MAC address
  [i
    InitEtherTab init-io		\ run the table
    _EMAC EMAC_ISR + @ drop		\ clear any pending ints.
  i]
  InitPhy  PHY_CNTL_REG ReadPhy
  dup bit13 and #13 rshift		\ bit13 -> bit0 ; 100Mbps?
  swap bit8 and 7 rshift or		\ bit8 -> bit1 ; Full Duplex?
((
  dup bit13 and				\ 100Mbps?
  if  bit0 else 0  endif		\ for E9200 CFG register bit
  swap bit8 and				\ Full duplex?
  if  bit1 or  endif
))
  _EMAC EMAC_CFG + tuck			\ -- cfg mask cfg
  @ -4 and or swap !			\ write FD and SPD bits.
  $000C _EMAC EMAC_CTL + or!		\ enable Tx and Rx
  -1
;

: InitEther	\ --
\ *G Perform a full initialisation of the chip, enabling
\ ** Rx and Tx, and setting up the Ethernet MAC address
\ ** from *\fo{EtherAddress}.
  cr ." Initialising Ethernet ... "
  InitEMAC 0=
  if ." un" then ." successful"
;


\ ********************
\ *S Packet read/write
\ ********************

: StepRxHdr	\ hdr -- hdr'
  dup @ bit1 and			\ if WRAP bit set
  if drop ERxHdrs else /ERxHdr + endif
;

: IsRx?         \ -- flag ; nz = has data
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
  _EMAC EMAC_RSR + @ $07 and
  if  $07 _EMAC EMAC_RSR + !  endif
  NextRxHdr @				\ already got one
  if  true exit  endif
  FirstRxHdr @  #ERxBuffs 0 do		\ -- hdr
    dup rh.addr @ bit0 and if		\ new received packet?
      dup NextRxHdr !			\ point to packet to read
      StepRxHdr FirstRxHdr !		\ step to next one
      true  unloop exit			\ get out
    endif
    StepRxHdr
  loop
  drop 0				\ nothing received.
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get a pending receive packet to a buffer of size *\b{maxlen).
\ ** Return the number of bytes received, or zero for a bad packet.
\ ** Note that data is valid only when *\fo{ISRX?} returns true,
\ ** so that you must poll with *\fo{ISRX?} before using
\ ** *\fo{GET_ETHER_PKT}. The received size of the packet
\ ** (not clipped to the buffer size) is returned.
\ ** In this implementation, a destination address of 0 indicates
\ ** that the packet should be discarded, and no memory transfer
\ ** occurs.
  NextRxHdr @ dup 0=			\ sanity check
  if  drop 2drop 0  exit  endif
  \ -- *dest maxlen hdr
  dup rh.addr @ $FFFFFFFC and		\ -- *dest maxlen hdr buff
  swap rh.status @ $000007FF and 4 -	\ -- *dest maxlen buff len
  dup >r  2swap rot min			\ buff *dest len' -- ; R: -- len'
  over					\ check dest address
  if  cmove  else  3drop  endif		\ -- ; R: -- len'
  bit0 NextRxHdr @ rh.addr bic!		\ give buffer back to DMA
  0 NextRxHdr !				\ mark no current Rx packet
  r>					\ return length
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer
\ ** memory.
  0 0 get_ether_pkt drop
;

0 value LastTxStat

: TxStat	\ -- mask
\ *G Read Tx Status register.
  _EMAC EMAC_TSR + @  dup -> LastTxStat
;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop  TxStat bit4 and			\ TSR BNQ bit set
;

: WaitTxDone	\ --
\ *G Wait until the frame has been transmitted.
  begin
    TxStat $0067 and 0=
   while
    pause
  repeat
;

: ClearTx	\ --
\ *G Clear transmit status register.
  $0067 _EMAC EMAC_TSR + !
;

: send_ether_pkt	\ *pBuf len --
\ *G Send packet from supplied buffer. Note that we wait until 
\ ** completion so that the buffer can be released immediately.

\ Perform length sanity checks
  dup 0=
  if  2drop exit  endif
  dup /ETxBuff u>
  if  2drop exit  endif
\ Copy packet to uncached memory
  tuck  ETxBuffs swap aligned cmove	\ copy, alignment forces fast CMOVE
  ETxBuffs swap				\ -- etxb len
\ Now send the packet from uncached memory
  begin  dup IsTx? 0= while  pause  repeat	\ wait until Tx available
  ClearTx				\ clear Tx status
  #64 max				\ min packet size
  swap _EMAC EMAC_TAR + !		\ set buffer 
  $07FF and _EMAC EMAC_TCR + !		\ set len, starts Tx
  WaitTxDone				\ wait until sent
;


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O IPDGetAddr function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the corresponding stored data
\ ** will not be changed.
  drop					\ mode flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove		\ save
    SetMACaddress			\ put in EMAC
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;


\ *P The layout and usage of the Ethernet common I/O structure is
\ ** defined in the file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

create E92vector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** *\i{COMMON\ETHERCOM.FTH}.
  ' InitEther ,         \ initialisation      
  ' noop ,	        \ shutdown            
  ' IsRx? ,	        \ receive test        
  ' get_ether_pkt ,     \ receive packet      
  ' IsTx? ,	        \ transmit test       
  ' send_ether_pkt ,    \ transmit packet     
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

E92vector constant IPDevice	\ -- addr
\ *G IPDevice is the default device name required by ETHERCOM.FTH.
\ ** If you have multiple ports, only one should be called IPDevice.

  
\ ==============
\ *S Diagnostics
\ ==============

E92Diags? [if]

create EregNames	\ -- addr
  ", CTL CFG SR  TAR TCR TSR RBQP--- RSR ISR IER IDR IMR MAN "

: .ename	\ offset --
  EregNames 1+ + 4 type space
;

: .emac		\ --
\ *G Display contents of all Ethernet Controller registers.
  cr ." Emac regs"
  _EMAC  $038 0 do			\ -- emac
    cr i .byte space  i .ename 
    dup i + @ .dword
  4 +loop
  drop
((
  cr ." Stats regs"
  _EMAC $040 +  $048 0 do		\ -- stat
    cr i .byte space  dup i + @ .dword
  4 +loop
  drop
))
;

: .phy		\ --
\ *G Display contents of all Ethernet Controller's PHY registers.
  cr ."  0 Control: " 0 readphy .word
  cr ."  1  Status: " 1 readphy .word
  cr ."  2 PhyID#1: " 2 readphy .word
  cr ."  3 PhyID#2: " 3 readphy .word
  cr ."  4 AutoNeg: " 4 readphy .word
  cr ."  5 Ability: " 5 readphy .word
  cr ."  6 Autoexp: " 6 readphy .word
  cr ." 16 Config1: " #16 readphy .word
  cr ." 17 Conf/St: " #17 readphy .word
  cr ." 18 10TC/St: " #18 readphy .word
;

MultiPhy? [if]
: .Phys		\ --
\ *G Compiled if the equate *\fo{MultiPhy?} is non-zero.
\ ** Displays the register contents of all PHYs.
  #32 0 do
    cr ." PhyA " i .byte space
    i to Phy#  .Phy
  loop
;
[then]

[then]


\ **************
\ *S System test
\ **************
\ *P The packet sniffer code is only compiled if the equate
\ ** *\fo{SNIFF?} is non-zero.

sniff? [if]

1536 buffer: pbuff	\ -- addr
\ *G Temporary buffer for *\fo{SNIFF}.

: NextRxPkt	\ --
\ *G Collect and display packet
  pbuff 1536 erase
  pbuff 1536 get_ether_pkt dup .word
  pbuff swap 1 max dump
;

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  begin
    isrx?
    if  NextRxPkt  endif
    #10 ms
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]

\ *P The PING example is used to test Ethernet transmission and
\ ** reception. Before testing transmission, test reception by
\ ** using the maintenance system to download a user application
\ ** and then pinging the target board instead of TFTPing to it.
\ *E Example ARP request:  #42/$2A bytes
\ ** C020.0600  FF FF FF FF FF FF 00 C0  F0 0D 9D 7E 08 06 00 01
\ ** C020.0610  08 00 06 04 00 01 00 C0  F0 0D 9D 7E C0 A8 01 7F
\ ** C020.0620  00 00 00 00 00 00 C0 A8  FF FE 11 0A 75 9C C0 A8

\ *P Note that the equates *\fo{SNIFF?} and *\fo{PINGIT?} must both
\ ** be set non-zero to compile the PING code.

pingit? [if]

create requested-ip	\ -- addr
\ *G Holds IP address to be pinged.
  #192 c,  #168 c,  0 c,  50 c,

create zero-mac		\ -- addr
\ *G Holds MAC address 00-00-00-00-00-00
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

create EtherBC		\ -- addr
\ *G Holds MAC address FF-FF-FF-FF-FF-FF
  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

$0001 equ ARPHARD_ETHER
$0800 equ ARPPROTO_IP

$0001 equ ARPOP_REQUEST
$0002 equ ARPOP_REPLY

: SendArpReq	\ -- ; send ARP request
\ *G Send an ARP request.
\ ** All initialisation must have been performed before
\ ** using this word.
  pbuff >r

  EtherBC r@ EHDR.DstMac 6 cmove
  EtherAddress   r@ EHDR.SrcMac 6 cmove
  ET_ARP         r@ EHDR.type w!(n)

  ARPHARD_ETHER  r@ AHDR.htype w!(n)
  ARPPROTO_IP    r@ AHDR.ptype w!(n)
  6              r@ AHDR.hlen c!
  4              r@ AHDR.plen c!
  ARPOP_REQUEST  r@ AHDR.op w!(n)

  EtherAddress   r@ ARP.SenderMAC 6 cmove
  IpAddress      r@ ARP.SenderIP 4 cmove
  zero-mac       r@ ARP.TargetMAC 6 cmove
  requested-ip   r@ ARP.TargetIP 4 cmove

  r> ARP send_ether_pkt
;

: pingonce	\ -- ; an ARP request
\ *G Send an ARP request and process the response once.
\ ** All initialisation must have been performed before
\ ** using this word.
  cr  ticks				\ starting time
  SendArpReq

  #1000 later
  begin
    isrx? if
      pbuff #1536 get_ether_pkt .
      pbuff EHDR.type w@(n) ET_ARP =
      pbuff AHDR.op w@(n) ARPOP_REPLY =  and
      pbuff ARP.targetMAC EtherAddress 6 s=  and
    else
      dup expired dup
      if  ." Timed out "  endif
    endif
  until
  drop
  ticks swap - . ." ms"			\ elapsed time
;

: pingit	\ -- 
\ *G Ping the IP address at *\fo{REQUESTED-IP} until a key is pressed.
  InitEther
  begin
    pingonce  #500 ms
    key?
  until
  ( Stop-Timers )
;

[then]


\ ******
\ *> ###
\ ******

decimal
