\ DM9000 Ethernet driver

((
Copyright (c) 2005, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20060620 MPE002 Fixes for DM9000 transmit issues.
20050911 EGS001 Fixup for IO ports on Embedded Artists ARM 2138 board AND for DM9000 !!
))


\ =========
\ *! dm9000
\ *T DM9000 Ethernet Driver Code
\ =========
\ *S Introduction
\ *P This code is derived from the sample C driver from Embedded
\ ** Artists for their LPC2138 Ethernet quickstart board.

\ *P This code assumes a CPU with byte addressing. A cell
\ ** addressed machine, e.g. most DSPs, will require considerable
\ ** changes to the source code for memory buffer transfers.


\ ****************
\ *S Configuration
\ ****************
\ *P The following must be defined before the file
\ ** *\i{DRIVERS\EtherDM9000.FTH} is compiled.

interpreter
: const equ ;	\ n -- ; -- n
\ *G If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** *\fo{CONST} as *\fo{CONSTANT}, otherwise by default *\fo{CONST}
\ ** is defined as *\fo{EQU}.
target

[undefined] fastCPU? [if]
1 equ fastCPU?	\ -- n
\ *G Set this value false if no software intervention is
\ ** required to meet the 400ns timing requirement before and after
\ ** changing the pointer register.
[then]

[undefined] eeprom? [if]
0 equ eeprom?	\ -- flag
\ *G Set this equate true if the DM9000 has an attached EEPROM
\ ** for configuration data storage. False by default.
[then]

[undefined] dmDiags? [if]
1 equ dmDiags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. False by default.
[then]

[undefined] sniff? [if]
0 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] pingit? [if]
0 equ pingit?	\ -- flag
\ *G Set this equate true to compile the PING code,
\ ** which tests Ethernet transmission and reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
\  $00 c, $10 c, $8B c, $F1 c, $44 c, $20 c,	\ six bytes for 48 bit MAC address
  $00 c, $08 c, $EF c, $00 c, $5E c, $C4 c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!
[then]

[undefined] IpAddress [if]
create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
  10 c,  102 c,  0 c,  11 c,		\ assign these as required
\  192 c,  168 c,  1 c,  251 c,		\ assign these as required
[then]


\ ************
\ *S Constants
\ ************

/* register offsets */
$00 equ NIC_NCR
$01 equ NIC_NSR
$02 equ NIC_TCR
$03 equ NIC_TSR1
$04 equ NIC_TSR2
$05 equ NIC_RCR
$06 equ NIC_RSR
$07 equ NIC_ROCR
$08 equ NIC_BPTR
$09 equ NIC_FCTR
$0A equ NIC_FCR
$0B equ NIC_EPCR
$0C equ NIC_EPAR
$0D equ NIC_EPDRL
$0E equ NIC_EPDRH
$0F equ NIC_WCR
$10 equ NIC_PAR0
$11 equ NIC_PAR1
$12 equ NIC_PAR2
$13 equ NIC_PAR3
$14 equ NIC_PAR4
$15 equ NIC_PAR5
$16 equ NIC_MAR0
$17 equ NIC_MAR1
$18 equ NIC_MAR2
$19 equ NIC_MAR3
$1A equ NIC_MAR4
$1B equ NIC_MAR5
$1C equ NIC_MAR6
$1D equ NIC_MAR7
$1E equ NIC_GPCR
$1F equ NIC_GPR
$22 equ NIC_TRPAL
$23 equ NIC_TRPAH
$24 equ NIC_RWPAL
$25 equ NIC_RWPAH
$28 equ NIC_VIDL
$29 equ NIC_VIDH
$2A equ NIC_PIDL
$2B equ NIC_PIDH
$2C equ NIC_CHIPR
$2F equ NIC_SMCR
$F0 equ NIC_MRCMDX
$F2 equ NIC_MRCMD
$F4 equ NIC_MRRL
$F5 equ NIC_MRRH
$F6 equ NIC_MWCMDX
$F8 equ NIC_MWCMD
$FA equ NIC_MWRL
$FB equ NIC_MWRH
$FC equ NIC_TXPLL
$FD equ NIC_TXPLH
$FE equ NIC_ISR
$FF equ NIC_IMR

\ =======================================================================

#1530 const MaxMsgSize

[undefined] bit0 [if]
$01 const bit0
$02 const bit1
$04 const bit2
$08 const bit3
$10 const bit4
$20 const bit5
$40 const bit6
$80 const bit7
[then]


\ ***************************
\ *S Hardware Interface Layer
\ ***************************
\ IO pins for Embedded Artists 2138 board
\ Data Bus   P0.16..23    r/w   Bitmask = $00FF0000
\ A2         P1.16        w/o   selects command/data
\ IRQ        P0.15        r/o
\ RST        P0.31        w/o   active high
\ nWR        P1.23        w/o   active low
\ nRD        P1.22        w/o   active low
\ nCS        P1.21        w/o   active low

\ port 1
bit23     equ DM_WR	\ -- mask ; IOWR bit - active low - p1
bit22     equ DM_RD	\ -- mask ; IORD bit - active low - p1
bit21     equ DM_CE	\ -- mask ; Chip Select bit - active low - p1
bit16     equ DM_A2	\ -- mask ; Command/Data bit - p1

\ port 0
bit31     equ DM_RS	\ -- mask ; RESET bit - active low - p1
bit15     equ DM_INT	\ -- mask ; interrupt bit - active low
$00FF0000 equ edbmask	\ -- mask ; ethernet data bus mask bits 16-23
edbmask invert equ ~edbmask
#16 equ /edbshift	\ -- n ; shift count for data bus

DM_RD DM_WR or DM_CE or DM_A2 or
  equ p1.EOPs		\ -- mask ; always outputs
p1.EOPs  DM_INT or  edbmask or
  dup equ etherbits	\ -- mask ; all ethernet bits
  invert equ ~etherbits	\ -- mask ; all non-ethernet bits

$3FFFFFFF equ PS0gpio	\ -- mask ; 15..00: ensure iopin  P0.15 is GPIO
$3FFF0000 equ PS1gpio	\ -- mask ; 31..16: ensure iopins P0.16-23 & 31 are GPIO
$FFFFFFF4 equ PS2gpio	\ -- mask ; See PINSEL2 register description

L: ^GPIO0	\ -- addr ; saves target space
  _GPIO0 ,

L: ^GPIO1	\ -- addr ; saves target space
  _GPIO1 ,

: InitDM9000IO	\ -- ; initialise IO ports
\ *G Initialise access to the DM9000 hardware. For i/o port
\ ** access this involves setting up port I/O direction registers
\ ** and so on.
  _PINSEL
    PS0gpio over PINSEL0 + and!		\ set GPIOxxx to GPIO mode
    PS1gpio over PINSEL1 + and!		\ set GPIOxxx to GPIO mode
    PS2gpio swap PINSEL2 + and!		\ set GPIOxxx to GPIO mode
  DM_RD DM_WR or DM_CE or DM_A2 or
  ^GPIO1 @
    2dup IOSET + !			\ set read, write, CS and A2 high
    IODIR + or!				\ then make them outputs
  ^GPIO0 @
    DM_RS DM_INT or over IOSET + !	\ set reset and int high
    dup IODIR + @
      DM_RS or  DM_INT invert and	\ RS is output, INT is input
    swap IODIR + !			\ set as needed
;

: edb>out	\ --
\ *G Set data bus to output.
  edbmask ^GPIO0 @ IODIR + or!	\ set direction bits
;

: edb>in
\ *G Set data bus to input.
  edbmask ^GPIO0 @ IODIR + bic!	\ clear direction bits
;

: edb>hi	\ --
\ *G Set data bus high.
  edbmask ^GPIO0 @ IOSET + !	\ set bits to 1
;

: edb>lo
\ *G Set data bus low.
  edbmask ^GPIO0 @ IOCLR + !	\ set bits to 0
;

: DM1>lo	\ bit -- ;
  ^GPIO1 @ IOCLR + !
;
: DM1>hi	\ bit -- ;
  ^GPIO1 @ IOSET + !
;
: DM0>lo	\ bit -- ;
  ^GPIO0 @ IOCLR + !
;
: DM0>hi	\ bit -- ;
  ^GPIO0 @ IOSET + !
;

: ((ec@))	\ -- data ;
  ^GPIO0 @ IOPIN + @
  #16 rshift $FF and
;

((
: ((ec!))	\ data -- ;
  $FF and /edbshift lshift
  ^GPIO0 @ IOPIN + tuck
  @ + swap !
;
))
: ((ec!))	\ data --
  ^GPIO0 @				\ -- data addr
  edbmask over IOCLR + !		\ all bits low
  swap $FF and /edbshift lshift swap	\ form bits to set
  IOSET + !				\ and set them
;

: (ec@)         \ -- val ;
\ *G Read an 8 bit direct at curr address
  noop
  edb>in			\ data bus for input
  edb>lo			\ set all to 0
  DM_A2 DM1>hi
  DM_CE DM1>lo			\ pull CE lo
  noop
  DM_RD DM1>lo			\ pull RD lo
  noop
  ((ec@))			\ go read back data byte
  DM_RD DM1>hi			\ end of RD\
  noop
  DM_CE DM1>hi			\ end of CE\
  noop
  DM_A2 DM1>lo
  edb>out			\ set bus back to output
  edb>hi			\ set all to 1
;

: ec@           \ reg -- val
\ *G Read an 8 bit register at the selected reg adess
  edb>out
  edb>lo
  DM_A2 DM1>lo			\ A2 pin low
  ((ec!))			\ write reg value to port
  DM_CE DM1>lo			\ pull CE lo
  noop
  DM_WR DM1>lo			\ pull WE lo
  noop
  DM_WR DM1>hi			\ end of WR\
  noop
  DM_CE DM1>hi			\ end of CE\
  (ec@)
;

: (ec!)           \ b --
\ *G Set an 8 bit register contents to b, in the current addr.
  noop
  edb>out
  edb>lo
  ((ec!))
  DM_A2 DM1>hi
  DM_CE DM1>lo
  noop
  DM_WR DM1>lo
  noop
  DM_WR DM1>hi
  noop
  DM_CE DM1>hi
  DM_A2 DM1>lo
  edb>hi
;

: ec!		\ b reg --
\ *G Set an 8 bit register contents to b, in the reg addr.
  edb>out
  edb>lo
  DM_A2 DM1>lo
  ((ec!))
  DM_CE DM1>lo
  noop
  DM_WR DM1>lo
  noop
  DM_WR DM1>hi
  noop
  DM_CE DM1>hi
  (ec!)
;


\ *************
\ *S PHY access
\ *************

: phyclkdelay	\ --
  100 0 do loop
;

#500000 equ FAILSAFE_LIMIT

: write_phy	\ val reg -- ; val = 16 bit value
  $1F and $40 OR NIC_EPAR ec!	\ set address
  dup 8 rshift NIC_EPDRH ec!	\ write high data
  $00FF and NIC_EPDRL ec!	\ write low data
  $0A NIC_EPCR ec!		\ write command to PHY

  FAILSAFE_LIMIT 0 do
    NIC_EPCR c@ $01 and 0= if
      1 ms
      $08 NIC_EPCR ec!
      leave
    endif
  loop
;

: read_phy	\ reg -- val ; val = 16 bit value
  $1F and $40 OR NIC_EPAR ec!	\ set address
  $0C NIC_EPCR ec!		\ write command to PHY
  0									\ dummy 0
  FAILSAFE_LIMIT 0 do
    NIC_EPCR ec@ $01 and 0= if
      1 ms
      $08 NIC_EPCR ec!
      drop			\ lose the dummy result
      NIC_EPDRH ec@ 8 lshift 	\ read high data
      NIC_EPDRL ec@ +		\ read low data
      leave
    endif
  loop
;


\ *****************
\ *S Initialisation
\ *****************

: InitDM9000	\ -- t|f
\ *S Initialisation of the DM9000, including MAC address.
\ ** Note that this takes nearly three seconds. The flag is
\ ** returned true if initialisation succeeds.

\ now do the chip reset
  DM_RS DM0>lo
  #2000 ms
  DM_RS DM0>hi
  #250 ms
  DM_RS DM0>lo
  #500 ms

\ then do a SW reset
  $01 NIC_NCR ec!  1 ms
  $00 NIC_NCR ec!  1 ms
  $01 NIC_NCR ec!  1 ms
  $00 NIC_NCR ec!  1 ms

  0				\ return false in case it all goes t**s up
\ check if we got contact with DM9000 chip
\ check vendor and product id
  NIC_VIDH ec@ ( dup .byte space ) $0A <>
  if  ( cr ." VIDH failed " )  exit  endif
  NIC_VIDL ec@ ( dup .byte space ) $46 <>
  if  ( cr ." VIDL failed " )  exit  endif
  NIC_PIDH ec@ ( dup .byte space ) $90 <>
  if  ( cr ." PIDH failed " )  exit  endif
  NIC_PIDL ec@ ( dup .byte space ) $00 <>
  if  ( cr ." PIDL failed " )  exit  endif
  NIC_ISR ec@ ( dup .byte space ) 6 rshift $02 <>
  if  ( cr ." ISR failed " )  exit  endif

  $01E1 $04 write_phy		\ PHY Reset
  $1200 $00 write_phy		\ PHY Reset
  \ activate internal phy
  $01 NIC_GPCR ec!
  $00 NIC_GPR ec!
  1 ms
  \ Clear Tx Status
  $00 NIC_TCR ec!
  \ Set Normal mode
  $00 NIC_SMCR ec!
  \ Set MAC Address
  EtherAddress
  dup c@ NIC_PAR0 ec!
  1+ dup c@ NIC_PAR1 ec!
  1+ dup c@ NIC_PAR2 ec!
  1+ dup c@ NIC_PAR3 ec!
  1+ dup c@ NIC_PAR4 ec!
  1+     c@ NIC_PAR5 ec!
  \ set multicast hash table
  $00 NIC_MAR0 ec!
  $00 NIC_MAR1 ec!
  $00 NIC_MAR2 ec!
  $00 NIC_MAR3 ec!
  $00 NIC_MAR4 ec!
  $00 NIC_MAR5 ec!
  $00 NIC_MAR6 ec!
  $00 NIC_MAR7 ec!
  \ Clear Tx Status
  $2C NIC_NSR ec!
  \ Set flow control
  $FF NIC_FCR ec!
  \ Set interrupt status
  $0F NIC_ISR ec!
  $31 NIC_BPTR ec!
  $38 NIC_FCTR ec!
  \ Start NIC
  \ Enable Rx
  $39 NIC_RCR ec!		\ enabled & multicast
  \ Set interrupt mask
  $83 NIC_IMR ec!		\ enable Tx, Rx auto-rollover for rx add pointer

  drop -1			\ indicate success
;

: InitEther	\ --
\ *G Perform a full initialisation of the chip, enabling
\ ** Rx and Tx, and setting up the Ethernet MAC address
\ ** from ETHERADDRESS.
  cr ." Initialising Ethernet ... "
  InitDM9000IO  InitDM9000
  0= if ." un" then ." successful"
;


\ ********************
\ *S Packet read/write
\ ********************

: IsRx?         \ -- t|f
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
  NIC_MRCMDX ec@ drop			\ dummy read
  (ec@) $0F and $01 =			\ get current value
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get a pending receive packet to a buffer of size *\b{maxlen).
\ ** Return the number of bytes received, or zero for a bad packet.
\ ** Note that data is valid only when *\fo{ISRX?} returns true,
\ ** so that you must poll with *\fo{ISRX?} before using
\ ** *\fo{GET_ETHER_PKT}. The received size of the packet
\ ** (not clipped to the buffer size) is returned.
\ ** In this implementation, a destination address of 0 indicates
\ ** that the packet should be discarded, and no memory transfer
\ ** occurs.
  { *pb pblen | rxst rxlen ignoreFrame -- len }
  0 -> ignoreFrame
  0 -> rxlen

  \ get rx chain status
  IsRx? if
    NIC_MRCMD ec@ (ec@) 8 lshift or -> rxst
    (ec@) (ec@) 8 lshift or -> rxlen
    \ check if pkt length is valid
    rxlen MaxMsgSize > if
      InitDM9000 drop  0 -> rxlen
    else
      rxlen #64 < 			\ check for runt frame
      if  -1 -> ignoreFrame  endif
      rxst $bf00 and
      if  -1 -> ignoreFrame  endif
      *pb 0= ignoreFrame or 0= if
        *pb rxlen bounds
        ?do  (ec@) i c!  loop		\ read byte and write to buffer
      else
        rxlen 0
        ?do (ec@) drop  loop
        4 -> rxlen
      endif
      rxlen 4 - -> rxlen
    endif
  endif
  rxlen
;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop  NIC_NSR ec@ $0C and
;

: (send_ether_pkt)	\ *pBuf len --
\ *G Send packet from supplied buffer.
\ ** Unlike other Ethernet controller drivers, the DM9000 has a
\ ** peculiarity in that after reset, no transmit buffer is seen to
\ ** be free.
  { *pb len -- }
  len #64 max -> len
  \ activate write command with auto-inc of write pointer
  *pb dup c@ NIC_MWCMD ec!
  1+ len 1- bounds
  ?do  i c@ (ec!)  loop
  \ then write num bytes to be sent
  len $FF and NIC_TXPLL ec!
  len 8 rshift NIC_TXPLH ec!
  \ start transmission
  $01 NIC_TCR ec!
;

: TxTimeout?	\ -- flag
\ *G Return true if we cannot transmit for a while.
  #2000 later
  begin
    NIC_NSR ec@ $0C and 0=
   while
    dup expired
  until then
  timedout?
;

: send_ether_pkt  \ *pBuf len --
\ *G Send packet from supplied buffer. If the transmission times
\ ** out, the DM9000 is reset and the packet is resent. Zero
\ ** length and overlength packets are discarded. This driver
\ ** waits for a free transmit buffer *\b{after} a packet has
\ ** been sent.
  dup if
    dup $0600 <= if
      begin
        2dup (send_ether_pkt)  TxTimeout?
       while
\ cr ." Fixing TX fault"
        InitDM9000IO  InitDM9000 drop
      repeat
    endif
  endif
  2drop
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer
\ ** memory.
  0 0 get_ether_pkt drop
;



\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O IPDGetAddr function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the stored data
\ ** will not be changed.
  drop					\ flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove
    drop \ etheradd>CS
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;


\ *P The layout and usage of the Ethernet common I/O structure is
\ ** defined in the file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

create DM9000vector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** *\i{COMMON\ETHERCOM.FTH}.
  ' InitEther ,         \ initialisation
  ' noop ,	        \ shutdown
  ' IsRx? ,	        \ receive test
  ' get_ether_pkt ,     \ receive packet
  ' IsTx? ,	        \ transmit test
  ' send_ether_pkt ,    \ transmit packet
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

DM9000vector constant IPDevice	\ -- addr
\ *G IPDevice is the default device name required by ETHERCOM.FTH.
\ ** If you have multiple ports, only one should be called IPDevice.


\ ==============
\ *S Diagnostics
\ ==============

dmDiags? [if]

: .bank         \ bank -- ; bank is 1..n
\ *G Display register contents for currently selected DM bank.
  dup $10 -
  ?do  i ec@ .byte space  loop
  cr
;

: .reg          \ --
\ *G Display contents of all DM Ethernet Controller's registers.
  base @  hex
  cr ."    " $10 0 do i .byte space loop cr
  #51 0 do  [char] = emit  loop  cr
  #4 1 do
    i 1- . space
    i $10 * .bank
  loop
  cr $F0 ec@ ." F0: " .byte
  cr $F5 ec@ 8 lshift $F4 ec@ + ." F4: " .word
  cr $FE ec@ ." FE: " .byte
  cr $FF ec@ ." FF: " .byte
  base !
;

: .phy			\ --
\ *G Display contents of all Ethernet Controller's PHY registers.
  cr ."  0 Control: " 0 read_phy .word
  cr ."  1  Status: " 1 read_phy .word
  cr ."  2 PhyID#1: " 2 read_phy .word
  cr ."  3 PhyID#2: " 3 read_phy .word
  cr ."  4 AutoNeg: " 4 read_phy .word
  cr ."  5 Ability: " 5 read_phy .word
  cr ."  6 Autoexp: " 6 read_phy .word
  cr ." 16 Config1: " #16 read_phy .word
  cr ." 17 Conf/St: " #17 read_phy .word
  cr ." 18 10TC/St: " #18 read_phy .word
;

[then]


\ **************
\ *S System test
\ **************
\ *P The packet sniffer code is only compiled if the equate
\ ** *\fo{SNIFF?} is non-zero.

sniff? [if]

1536 buffer: pbuff	\ -- addr
\ *G Temporary buffer for *\fo{SNIFF}.

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  begin
    isrx? if
      pbuff 1536 erase
      pbuff 1536 get_ether_pkt dup .word
      pbuff swap 1 max dump
    then
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]

\ *P The PING example is used to test Ethernet transmission and
\ ** reception. Before testing transmission, test reception by
\ ** using the maintenance system to download a user application
\ ** and then pinging the target board instead of TFTPing to it.
\ *E Example ARP request:  #42/$2A bytes
\ ** C020.0600  FF FF FF FF FF FF 00 C0  F0 0D 9D 7E 08 06 00 01
\ ** C020.0610  08 00 06 04 00 01 00 C0  F0 0D 9D 7E C0 A8 01 7F
\ ** C020.0620  00 00 00 00 00 00 C0 A8  FF FE 11 0A 75 9C C0 A8

\ *P Note that the equates *\fo{SNIFF?} and *\fo{PINGIT?} must both
\ ** be set non-zero to compile the PING code.

pingit? [if]

create requested-ip	\ -- addr
\ *G Holds IP address to be pinged.
  #192 c,  #168 c,  0 c,  1 c,

create zero-mac		\ -- addr
\ *G Holds MAC address 00-00-00-00-00-00
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

create EtherBC		\ -- addr
\ *G Holds MAC address FF-FF-FF-FF-FF-FF
  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

$0001 equ ARPHARD_ETHER
$0800 equ ARPPROTO_IP

$0001 equ ARPOP_REQUEST
$0002 equ ARPOP_REPLY

: SendArpReq	\ -- ; send ARP request
\ *G Send an ARP request.
\ ** All initialisation must have been performed before
\ ** using this word.
  pbuff >r

  EtherBC r@ EHDR.DstMac 6 cmove
  EtherAddress   r@ EHDR.SrcMac 6 cmove
  ET_ARP         r@ EHDR.type w!(n)

  ARPHARD_ETHER  r@ AHDR.htype w!(n)
  ARPPROTO_IP    r@ AHDR.ptype w!(n)
  6              r@ AHDR.hlen c!
  4              r@ AHDR.plen c!
  ARPOP_REQUEST  r@ AHDR.op w!(n)

  EtherAddress   r@ ARP.SenderMAC 6 cmove
  IpAddress      r@ ARP.SenderIP 4 cmove
  zero-mac       r@ ARP.TargetMAC 6 cmove
  requested-ip   r@ ARP.TargetIP 4 cmove

  r> ARP send_ether_pkt
;

: pingonce	\ -- ; an ARP request
\ *G Send an ARP request and process the response once.
\ ** All initialisation must have been performed before
\ ** using this word.
  cr  ticks				\ starting time
  SendArpReq

  #1000 later
  begin
    isrx? if
      pbuff #1536 get_ether_pkt .
      pbuff EHDR.type w@(n) ET_ARP =
      pbuff AHDR.op w@(n) ARPOP_REPLY =  and
      pbuff ARP.targetMAC EtherAddress 6 s=  and
    else
      dup expired dup
      if  ." Timed out "  endif
    endif
  until
  drop
  ticks swap - . ." ms"			\ elapsed time
;

: pingit	\ --
\ *G Ping the IP address at *\fo{REQUESTED-IP} until a key is pressed.
  Start-Timers  InitEther
  begin
    pingonce  #500 ms
    key?
  until
  ( Stop-Timers )
;

[then]


\ ******
\ *> ###
\ ******


