\ EtherSAM7X.fth - AT91SAM7X256 Ethernet driver

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ =========
\ *! ethersam7x
\ *T AT91SAM7x Ethernet Driver Code
\ =========
\ *S Introduction
\ *P This code is derived from the sample C driver on the Atmel
\ ** AT91DVD, and code from other Internet sources. The chip 
\ ** datasheet is not enough! This code was tested on the Atmel
\ ** AT91SAM7X-EK board.

\ *P There are several gotchas when using the EMAC.
\ *(
\ *B If interrupt operation is not used, interrupts disabled in
\ ** the EMAC block may not set bits in the EMAC interrupt
\ ** status registers.
\ *B The EMAC ISR (Interrupt status register) clears all bits when
\ ** read.
\ *B Receive and transmit headers and buffers must be on an
\ ** 8 byte boundary, otherwise the internal DMA will fail.
\ *B Reliable initialisation of the PHY and its address is
\ ** board specific and requires great attention to detail.
\ *B As of November 2006, the RMII PHY interface does not work.
\ ** See the current data sheet for more details.
\ *)


\ ****************
\ *S Configuration
\ ****************
\ *P If you do not want the default values of any of these
\ ** equates, defined them before the file *\i{Drivers\EtherSAM7x.fth}
\ ** is compiled.

interpreter
: const equ ;	\ n -- ; -- n
\ *G If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** *\fo{CONST} as *\fo{CONSTANT}, otherwise by default *\fo{CONST}
\ ** is defined as *\fo{EQU}.
target

[undefined] EtherCom? [if]
0 equ EtherCom?	\ -- flag
\ *G Only needed if the EtherCom generic I/O device is being used.
[then]

[undefined] EthDiags? [if]
0 equ EthDiags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. False by default.
[then]

[undefined] RMII? [if]
0 equ RMII?	\ -- flag
\ *G Set this if the hardware uses two-bit data access to the PHY
\ ** rather than four-bit.
[then]

[undefined] MultiPhy? [if]
0 equ MultiPhy?	\ -- flag
\ *G Set this flag true if you are using multiple PHYs or do not
\ ** know the address of the PHY.
[then]

[undefined] sniff? [if]
0 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] pingit? [if]
0 equ pingit?	\ -- flag
\ *G Set this equate true to compile the PING code,
\ ** which tests Ethernet transmission and reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $51 c,	\ six bytes for 48 bit MAC address
\  $00 c, $08 c, $EF c, $00 c, $5E c, $C4 c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!
[then]

[undefined] IpAddress [if]
create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
\  10 c,  102 c,  0 c,  11 c,		\ assign these as required
  192 c,  168 c,  1 c,  251 c,		\ assign these as required
[then]

#128 equ /ERxBuff	\ -- u
\ *G Size of an Ethernet receive buffer, fixed at 128 bytes. A full
\ ** Ethernet frame requires 12 buffers.
#48 equ #ERxBuffs	\ -- u
\ *G Number of receive headers and buffers. Must be at least two. The
\ ** maximum number is 1024.

#1520 equ /ETxBuff	\ -- u
\ *G Size of an Ethernet transmit buffer rounded up to a 16 byte unit.
#2 equ #ETxBuffs		\ -- u
\ *G Number of transmit headers. Must be at least one. The
\ ** maximum number is 1024.

struct /ETRxHdr	\ -- len
\ *G Structure defining a transmit or receive header.
\ *[
  int rh.addr		\ address of buffer
  int rh.status		\ receive status
end-struct
\ *]


\ **************
\ *S Packet read
\ **************

uhere #15 + #-16 and uorg	\ force UDATA to 8 byte boundary
uhere constant ERxHdrs	\ -- addr
\ *G Base address of the receive headers. The receiver steps
\ ** through these headers to find a free buffer. The headers
\ ** *\b{must} start on an 8 byte boundary.
  #ERxBuffs /ETRxHdr * uallot

uhere constant ERxBuffs	\ -- addr
\ *G Base address of the receive buffers, which must be on an
\ ** 8 byte boundary.
  #ERxBuffs /ERxBuff * uallot

variable FirstRxHdr	\ -- addr
\ *G Holds the address of the first read header to check.
  ERxHdrs FirstRxHdr !
variable #ERx	\ -- addr
\ *G Holds the number of packets available for read. Used with the
\ ** EMAC interrupt to reduce overheads.

: InitERxHdrs	\ --
\ *G Initialise the receive headers.
  ERxHdrs FirstRxHdr !  0 #ERx !	\ mark no data available
  ERxBuffs ERxHdrs  #ERxBuffs 1- 0 do	\ -- buff hdr
    2dup rh.addr !  dup rh.status off	\ initialise structure
    swap /ERxBuff + swap /ETRxHdr +	\ step to next
  loop					\ -- buff' hdr'
  swap bit1 or over rh.addr !		\ last buffer has WRAP bit set
  rh.status off
;

((
: IsRx?         \ -- flag ; nz = has data
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
\ ** return value is zero if no packets are available, or the
\ ** number of available packets.
  #ERx @
;
))
compiler
: IsRx?         \ -- flag ; nz = has data
  #ERx @
;
target

: StepRxHdr	\ rhdr -- rhdr'
\ *G Step to the next receive header, wrapping as required.
  dup rh.addr @ bit1 and		\ if WRAP bit set
  if drop ERxHdrs else /ETRxHdr + endif
;

: FindRxStart	\ rxhdr -- rxhdr'|0
\ *G Skip over and clear any receive fragments. These are
\ ** identified by the ownership bit being set, and the start of
\ ** frame bit being clear. On exit *\i{rxhdr') points to a header
\ ** with the SOF bit set. If no valid header can be found, the
\ ** return value is 0.
  #ERxBuffs 0 do
    dup rh.addr @ bit0 and if		\ bit 0 set = EMAC filled it
      dup rh.status @ bit14 and if	\ bit 14 set = first frame in packet
        unloop exit			\ return valid header
      endif
    endif
    bit0 over rh.addr bic!		\ clear
    StepRxHdr
  loop
  drop 0				\ nothing found
;

: doRxBuffer	\ caddr maxlen len rxhdr -- caddr' maxlen' len' rxhdr
\ *G Process the given receive buffer, adding it to the application
\ ** buffer at *\i{caddr/maxlen} if there is enough room. *\i{Len} is
\ ** the accumulating size of the packet, which is updated by the
\ ** size of the receive buffer, even if not all bytes were copied.
\ ** *\i{rxhdr} is the header controlling the receive buffer.
  { | len rxhdr lb -- }
  dup -> rxhdr rh.status @		\ read status
  $FFF and dup 0=			\ extract buffer length
  if  drop #128  endif			\ take 0 as 128 bytes
  dup -> lb + -> len			\ set rxbuff length, and accumulate
  rxhdr rh.addr @ -4 and lb		\ -- caddr maxlen rxb lb
  2over rot min dup >r cmove		\ -- caddr maxlen R: -- clen ; copy what we can
  r> /string				\ -- caddr' maxlen' ; step by copied length
  len rxhdr				\ -- caddr' maxlen' len' rxhdr
  bit0 over rh.addr bic!		\ give header back to DMA
;

: ReceivePacket	\ caddr maxlen rxhdr -- len
\ *G Read a complete packet into the destination buffer, returning
\ ** the actual length of the packet. If the packet is bigger than
\ ** the buffer, the copy is truncated. After each copy, the header
\ ** is returned to the EMAC.
  0 swap  #ERxBuffs 0 do		\ -- caddr maxlen len rxhdr
    dup rh.status @ >r
    doRxBuffer				\ add Rx buffer to application
    StepRxHdr dup FirstRxHdr !		\ step to next buffer
    r> bit15 and ?leave			\ done if EOF bit was set
  loop
  drop nip nip
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get a pending receive packet to a buffer of size *\b{maxlen).
\ ** Return the number of bytes received, or zero for a bad packet.
\ ** Note that data is valid only when *\fo{ISRX?} returns true,
\ ** so that you must poll with *\fo{ISRX?} before using
\ ** *\fo{GET_ETHER_PKT}. The received size of the packet
\ ** (not clipped to the buffer size) is returned.
\ ** In this implementation, a destination address of 0 indicates
\ ** that the packet should be discarded, and no memory transfer
\ ** occurs.
  IsRx? 0=				\ sanity check
  if  2drop 0  exit  endif
  [i #ERx decr i]
  FirstRxHdr @ FindRxStart dup if	\ -- *dest maxlen rxhdr
    ReceivePacket
  else
    nip nip
  endif
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer
\ ** memory.
  0 0 get_ether_pkt drop
;


\ ***************
\ *S Packet write
\ ***************
\ *P Note that in theory, *\b{no} transmit buffers need to be
\ ** allocated as the transmit routine could use the buffers
\ ** passed to it. In practice, there is a requirement for 
\ ** transmit (and receive) buffers to be on an eight-byte
\ ** boundary. To avoid this problem, transmit data is copied
\ ** to aligned buffers.

uhere #15 + #-16 and uorg	\ force UDATA to 8 byte boundary
uhere constant ETxHdrs	\ -- addr
\ *G Base address of the transmit headers. The receiver steps
\ ** through these headers to find the next buffer. The headers
\ ** *\b{must} start on an 8 byte boundary.
  #ETxBuffs /ETRxHdr * uallot
uhere constant ETxBuffs	\ -- addr
\ *G Base address of the transmit buffers. The buffers
\ ** *\b{must} start on an 8 byte boundary.
  #ETxBuffs /ETxbuff * uallot

variable FirstTxHdr	\ -- addr
\ *G Holds the address of the first transmit header to check.
  ETxHdrs FirstTxHdr !
variable NextTxHdr	\ -- addr
\ *G Holds the address of the next tranmit header to use.
\ ** Set to zero to indicate none or unknown.
  FirstTxHdr NextTxHdr !

: InitETxHdrs	\ --
\ *G Initialise the transmit headers. Note that there are *\b{no} transmit
\ ** buffers actually allocated as the transmit routine uses the
\ ** buffer passed to it.
  ETxHdrs dup FirstTxHdr !  NextTxHdr !	\ mark ready
  ETxBuffs ETxHdrs  #ETxBuffs 1- 0 do	\ -- buff hdr
    2dup rh.addr !			\ rh.addr
    bit31 over rh.status !		\ rh.status=used
    swap /ETxBuff + swap /ETRxHdr +	\ step to next buffer and header
  loop					\ -- buff' hdr'
  tuck rh.addr !
  bit31 bit30 or swap rh.status !	\ last buffer has WRAP bit set
;

: StepTxHdr	\ thdr -- thdr'
\ *G Step to the next transmit header, wrapping as required.
  dup rh.status @ bit30 and		\ if WRAP bit set
  if drop ETxHdrs else /ETRxHdr + endif
;

((
: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop
  NextTxHdr @ rh.status @ bit31 and 0<>	\ Used bit set?
;
))
compiler
: IsTx?		\ n -- flag
  drop
  NextTxHdr @ rh.status @ bit31 and 0<>	\ Used bit set?
;
target

: send_ether_pkt	\ *pBuf len --
\ *G Send packet from supplied buffer.
  dup 1 1515 within 0=
  if  2drop exit  endif
  #64 max				\ min 64 bytes

\ wait for previous TX to finish
  NextTxHdr @ 
  begin
    dup rh.status @ bit31 and 0=
   while
    pause
  repeat

\ copy data to buffer and start transmission
  >r  tuck aligned			\ -- len caddr len' ; R: -- txhdr
  r@ rh.addr @ swap cmove  r>		\ -- len txhdr ; copy data
  tuck rh.status @ bit30 and  bit15 or	\ -- txhdr len stat ; preserve wrap bit, set last
  swap $7FF and or			\ -- txhdr stat' ; merge length
  over rh.status !			\ -- txhdr ; set status/length
  bit9 _EMAC EMAC_NCR + or!		\ start transmission with TSTART bit

\ step to next header
  StepTxHdr NextTxHdr !
;


\ ***********
\ *S EMAC ISR
\ ***********
\ *P If performance is a real issue, *\fo{CODE} this routine to
\ ** avoid the Forth entry and exit overhead.

IsLeaf : EMAChandler	\ --
\ *G The high-level Forth part of the EMAC ISR.
\ ** Note that there is a bug in the chip that requires two bits to
\ ** be tested for a valid receive. Note also that the EMAC_ISR 
\ ** register is self-clearing on read, and thus may only be read
\ ** once for each interrupt.
  _EMAC dup EMAC_ISR + @ bit1 and	\ test RCOMP bit
  over EMAC_RSR + @ bit1 and or if	\ test REC bit
    $0007 swap  EMAC_RSR + !		\ clear status bits
    #ERx Incr  exit			\ count received packets
  endif
  drop
;

' EMAChandler IRQ: EmacISR	\ -- addr
\ *G The entry point for the EMAC interrupt handler.


\ *************
\ *S PHY access
\ *************
\ *P The PHY is usually only accessed during initialisation.

$60020000 equ ~RdPhy	\ -- mask
\ *G Mask for PHY read operations. Assumes use of PHY0.
$50020000 equ ~WrPhy	\ -- mask
\ *G Mask for PHY write operations. Assumes use of PHY0.

\ PHY registers
0x00 const PHY_CNTL_REG
0x01 const PHY_STATUS_REG
0x02 const PHY_ID_REG1
0x03 const PHY_ID_REG2
0x04 const PHY_ANA_REG
0x05 const PHY_ANLPAR_REG
0x06 const PHY_ANE_REG
0x10 const PHY_ECNTL_REG1
0x11 const PHY_QPDS_REG
0x12 const PHY_10BOP_REG
0x13 const PHY_ECNTL_REG2

\ PHY control register
bit15 const RESET_PHY
bit14 const ENABLE_LOOPBACK
bit13 const DR_100MB
bit12 const ENABLE_AN
bit10 const PHY_MAC_ISOLATE
bit9  const RESTART_AN
bit8  const PHY_FULLDUPLEX
bit7  const PHY_COL_TEST

\ PHY Status register
bit5 const AN_COMPLETE

\ PHY Auto-negotiation Advertisement register
bit8 const DR100_TX_FULL
bit7 const DR100_TX_HALF
bit6 const DR10_TX_FULL
bit5 const DR10_TX_HALF
DR100_TX_FULL  DR100_TX_HALF or  DR10_TX_FULL or  DR10_TX_HALF or
  const ANE_ALL	\ request everything

\ *P According to the example software, the default
\ ** PHY is 0. By experimentation, we found this to be uncertain, 
\ ** and the default may be 8 or 31 under some reset conditions. Two
\ ** versions of the following two words are provided, the first
\ ** for a single known PHY, and the second allowing use of a
\ ** current PHY. The first version is used if the equate
\ ** *\fo{MultiPhy?} is non-zero.

0 equ DefPhy#	\ -- n
\ *G Default PHY address. Normally 0.

MultiPhy? [if]
DefPhy# value Phy#	\ -- n
\ *G Multi PHY: The number of the PHY in use: 0..31
: ~Phy#		\ -- mask
\ *G Multi PHY: Mask used for PHY command generation.
  Phy# #23 lshift
;
[else]
DefPhy# equ Phy#	\ -- n
\ *G Single PHY: The number of the PHY in use: 0..31
Phy# #23 lshift equ ~Phy#	\ -- mask
\ *G Single PHY: Mask used for PHY command generation.
[then]

: WaitPhyDone	\ --
\ *G Wait until the current PHY operation has completed.
\  begin	
\    pause _EMAC EMAC_SR + @ bit2 and 0=	\ interrupt must be enabled
\  until
  #10 ms				\ slow but safe
;

: ReadPhy	\ reg -- w
\ *G Read a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  $1F and #18 lshift			\ form reg# mask
  ~RdPhy or  ~Phy# or			\ set up reg command
  _EMAC EMAC_MAN + !			\ set command
  WaitPhyDone				\ let operation complete
  _EMAC EMAC_MAN + @ $FFFF and		\ return data
;

: WritePhy	\ w reg --
\ *G Write a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  $1F and #18 lshift			\ form reg# mask
  ~WrPhy or  ~Phy# or			\ set up reg command
  swap $FFFF and or			\ merge in data
  _EMAC EMAC_MAN + !			\ set command
;

: OrPhy		\ mask reg --
\ *G Or mask into reg in the PHY. Set bits in mask are set in the
\ ** register.
  tuck ReadPhy  or  swap WritePhy
;

((
: BicPhy	\ mask reg --
 \ *G Invert/And mask into reg in the PHY. Set bits in mask are 
 \ ** cleared in the register.
  tuck ReadPhy  swap invert and  swap WritePhy
;
))

: InitPhy	\ -- ctl ; autonegotiation version
\ *G Initialise the PHY with autonegotiation.

\ Perform an external reset, otherwise the PHY address is not
\ properly set up
  $A5000111 _SP RSTC_MR + !		\ set external reset time
  $A5000008 _SP RSTC_CR + !		\ issue external reset
  begin					\ wait reset complete
    [asm  nop nop nop nop  asm]
    _SP RSTC_SR + @ bit16 and
  until
  bit4 _EMAC EMAC_NCR + or!		\ force MPE active
  #100 ms				\ seems to be needed

\ Reset the PHY
  PHY_CNTL_REG ReadPhy drop		\ discard old state
  $3100 PHY_CNTL_REG WritePhy  #20 ms	\ select state
  $8000 PHY_CNTL_REG OrPhy  #20 ms	\ force reset
  begin					\ wait reset complete
    pause
    PHY_CNTL_REG ReadPhy RESET_PHY and 0=
  until

\ start autonegotiation - results are not same as from hardware reset
  ANE_ALL PHY_ANA_REG OrPhy		\ request 10/100BaseT features
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
  RESTART_AN PHY_CNTL_REG OrPhy		\ start autonegotiation
  begin					\ wait until done
    PHY_STATUS_REG ReadPhy
    AN_COMPLETE and
  until

\ set up control reg using the results
  PHY_ANLPAR_REG ReadPhy  0		\ -- partner cntlset
  over DR100_TX_FULL and
  if  DR_100MB PHY_FULLDUPLEX or or  endif
  over DR100_TX_HALF and
  if  DR_100MB or  endif
  over DR10_TX_FULL and
  if  PHY_FULLDUPLEX or  endif
  ENABLE_AN or				\ so that partner ability is visible
  dup PHY_CNTL_REG WritePhy
  nip					\ -- ctl
;


\ *****************
\ *S Initialisation
\ *****************

/MCK 2* 8 / 5000000 <= [if]
  0
[else]
  /MCK 2* 16 / 5000000 <= [if]
    1
  [else]
    /MCK 2* 32 / 5000000 <=
    [if]  2  [else]  3  [then]
  [then]
[then]
#10 lshift equ ~MDC	\ -- mask
\ *G Clock mask for MDC. Calculated from *\fo{/MCK}.
      
RMII? [if] bit0 [else] 0 [then] equ ~RMII	\ -- mask
\ *G Initialisation bit mask for USRIO reg if RMII is used.

((
For a MAC address conventionally defined as:
  aa-bb-cc-dd-ee-ff
the layout of the SAxL/H registers is:
SAxL:
        ---------------------
    MSB | cc | dd | ee | ff | LSB
        ---------------------
SAxH:
        ---------------------
    MSB | 00 | 00 | aa | bb | LSB
        ---------------------
The low register must be set first.
))

: setMACAddress	\ addr --
\ *G Set the MAC address in the EMAC unit.
  dup 5 + c@
  over 4 + c@ 8 lshift or
  over 3 + c@ #16 lshift or
  over 2 + c@ #24 lshift or
  _EMAC EMAC_SA1L + !
  dup 1 + c@  swap 0 + c@ 8 lshift or
  _EMAC EMAC_SA1H + !
;

$00007FFFF equ EMACpins	\ -- mask
\ *G Bit mask for Ethernet pins, PB18..0 on PIOB.
\ ** PB18 is the PWRDN pin, which must be cleared.

create InitEtherTab	\ -- addr
\ *G Initialisation table for Ethernet I/O access. Disables
\ ** Ethernet interrupt, clears powerdown and initialise the
\ ** Ethernet controller.
  _AIC AIC_IDCR + ,		EMACmask ,	\ Disable EMAC interrupt at AIC
  _PMC PMC_PCER + ,	EMACmask PIOBmask or ,	\ power up Ethernet and PIOB
  _PIOB PIO_ASR + ,		EMACpins ,	\ EMAC pins are periph A
  _PIOB PIO_ODR + ,		EMACpins ,	\ disable PIO outputs
  _PIOB PIO_IDR + ,		EMACpins ,	\ disable PIO interrupt
  _PIOB PIO_PDR + ,		EMACpins ,	\ disable PIO
  _PIOB PIO_PPUDR + ,		EMACpins ,	\ disable PIO pullups
  _PIOB PIO_PER + ,		bit18 ,		\ enable PIO bt18/PWRDN
  _PIOB PIO_OER + ,		bit18 ,		\ enable output PIO bt18/PWRDN
  _PIOB PIO_CODR + ,		bit18 ,		\ clear output PIO bt18/PWRDN
  _EMAC EMAC_IDR + ,		$3CFF ,		\ Disable all EMAC interrupt sources
  _EMAC EMAC_NCR + ,		$0030 ,		\ Enable MDIO/PHY access, clear stats
  _EMAC EMAC_CFG + ,		$0013 ~MDC or ,	\ set CFG register, 100M Half Duplex.
  _EMAC EMAC_TSR + ,		$0077 ,		\ clear transmit status bits
  _EMAC EMAC_RSR + ,		$0007 ,		\ clear receive status bits
  _EMAC EMAC_RBQP + ,		ERxHdrs ,	\ receive queue head
  _EMAC EMAC_TBQP + ,		ETxHdrs ,	\ transmit queue head
  _EMAC EMAC_IER + ,		bit1 bit0 or ,	\ enable RCOMP and MFD interrupts
  _EMAC EMAC_USRIO + ,		bit1 ~RMII or ,	\ enable clock, perhaps RMII
  _AIC AIC_SMR0 + ID_EMAC cells + ,  $0007 ,	\ interrupt priority
  _AIC AIC_SVR0 + ID_EMAC cells + ,  EmacISR ,	\ ISR handler
  _AIC AIC_IECR + ,		EMACmask , 	\ enable the EMAC interrupt
  0 ,

: InitEMAC	\ -- flag ; nz=success
\ *S Initialisation of the Ethernet and PHY, including the MAC
\ ** address. Note that interrupts are disabled for a while. If
\ ** this is problematic, replace the interrupt disables with
\ ** a semaphore.
  [i
    InitERxHdrs InitETxHdrs		\ set up RX and TX headers
    EtherAddress setMACaddress		\ set up MAC address
    InitEtherTab init-io		\ run the table
    _EMAC EMAC_ISR + @ drop		\ clear any pending ints.
  i]
  InitPhy dup bit13 and #13 rshift	\ bit13 -> bit0 ; 100Mbps?
  swap bit8 and 7 rshift or		\ bit8 -> bit1 ; Full Duplex?
  _EMAC EMAC_CFG + tuck			\ -- cfg mask cfg
  @ -4 and or swap !			\ write FD and SPD bits.
  $000C _EMAC EMAC_NCR + or!		\ enable Tx and Rx
  -1
;

: InitEther	\ --
\ *G Perform a full initialisation of the chip, enabling
\ ** Rx and Tx, and setting up the Ethernet MAC address
\ ** from *\fo{EtherAddress}.
  cr ." Initialising Ethernet ... "
  InitEMAC 0=
  if ." un" then ." successful"
;


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************

EtherCom? [if]

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O IPDGetAddr function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the corresponding stored data
\ ** will not be changed.
  drop					\ mode flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove		\ save
    SetMACaddress			\ put in EMAC
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;

\ *P The layout and usage of the Ethernet common I/O structure is
\ ** defined in the file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

create ESAM7Xvector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** *\i{COMMON\ETHERCOM.FTH}.
  ' InitEther ,         \ initialisation      
  ' noop ,	        \ shutdown            
  ' IsRx? ,	        \ receive test        
  ' get_ether_pkt ,     \ receive packet      
  ' IsTx? ,	        \ transmit test       
  ' send_ether_pkt ,    \ transmit packet     
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

ESAM7Xvector constant IPDevice	\ -- addr
\ *G IPDevice is the default device name required by
\ ** *\i{ETHERCOM.FTH}. If you have multiple ports, only one
\ ** should be called *\fo{IPDevice}.

[then]

  
\ ==============
\ *S Diagnostics
\ ==============

EthDiags? [if]

create EregNames	\ -- addr
  ", CTL CFG SR  TAR TCR TSR RBQPTBQPRSR ISR IER IDR IMR MAN "

: .ename	\ offset --
  EregNames 1+ + 4 type space
;

: .emac		\ --
\ *G Display contents of all Ethernet Controller registers.
  cr ." Emac regs"
  _EMAC  $038 0 do			\ -- emac
    cr i .byte space  i .ename 
    dup i + @ .dword
  4 +loop
  drop
((
  cr ." Stats regs"
  _EMAC $040 +  $048 0 do		\ -- stat
    cr i .byte space  dup i + @ .dword
  4 +loop
  drop
))
;

: .phy		\ --
\ *G Display contents of all Ethernet Controller's PHY registers.
  cr ."  0 Control: " 0 readphy .word
  cr ."  1  Status: " 1 readphy .word
  cr ."  2 PhyID#1: " 2 readphy .word
  cr ."  3 PhyID#2: " 3 readphy .word
  cr ."  4 AutoNeg: " 4 readphy .word
  cr ."  5 Ability: " 5 readphy .word
  cr ."  6 Autoexp: " 6 readphy .word
  cr ." 16 Config1: " #16 readphy .word
  cr ." 17 Conf/St: " #17 readphy .word
  cr ." 18 10TC/St: " #18 readphy .word
;

MultiPhy? [if]
: .Phys		\ --
\ *G Compiled if the equate *\fo{MultiPhy?} is non-zero.
\ ** Displays the register contents of all PHYs.
  #32 0 do
    cr ." PhyA " i .byte space
    i to Phy#  .Phy
  loop
;
[then]

[then]


\ **************
\ *S System test
\ **************

\ =================
\ *N Packet sniffer
\ =================
\ *P The packet sniffer code is only compiled if the equate
\ ** *\fo{SNIFF?} is non-zero.

sniff? [if]

1536 buffer: pbuff	\ -- addr
\ *G Temporary buffer for *\fo{SNIFF}.

: NextRxPkt	\ --
\ *G Collect and display packet
  pbuff 1536 erase
  pbuff 1536 get_ether_pkt dup .word
  pbuff swap 1 max dump
;

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  begin
    isrx?
    if  NextRxPkt  endif
    #10 ms
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]


\ =======
\ *N Ping
\ =======
\ *P The PING example is used to test Ethernet transmission and
\ ** reception. Before testing transmission, test reception by
\ ** using the maintenance system to download a user application
\ ** and then pinging the target board instead of TFTPing to it.
\ *E Example ARP request:  #42/$2A bytes
\ ** C020.0600  FF FF FF FF FF FF 00 C0  F0 0D 9D 7E 08 06 00 01
\ ** C020.0610  08 00 06 04 00 01 00 C0  F0 0D 9D 7E C0 A8 01 7F
\ ** C020.0620  00 00 00 00 00 00 C0 A8  FF FE 11 0A 75 9C C0 A8

\ *P Note that the equates *\fo{SNIFF?} and *\fo{PINGIT?} must both
\ ** be set non-zero to compile the PING code.

pingit? [if]

create requested-ip	\ -- addr
\ *G Holds IP address to be pinged.
  #192 c,  #168 c,  0 c,  50 c,

create zero-mac		\ -- addr
\ *G Holds MAC address 00-00-00-00-00-00
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

create EtherBC		\ -- addr
\ *G Holds MAC address FF-FF-FF-FF-FF-FF
  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

$0001 equ ARPHARD_ETHER
$0800 equ ARPPROTO_IP

$0001 equ ARPOP_REQUEST
$0002 equ ARPOP_REPLY

: SendArpReq	\ -- ; send ARP request
\ *G Send an ARP request.
\ ** All initialisation must have been performed before
\ ** using this word.
  pbuff >r

  EtherBC r@ EHDR.DstMac 6 cmove
  EtherAddress   r@ EHDR.SrcMac 6 cmove
  ET_ARP         r@ EHDR.type w!(n)

  ARPHARD_ETHER  r@ AHDR.htype w!(n)
  ARPPROTO_IP    r@ AHDR.ptype w!(n)
  6              r@ AHDR.hlen c!
  4              r@ AHDR.plen c!
  ARPOP_REQUEST  r@ AHDR.op w!(n)

  EtherAddress   r@ ARP.SenderMAC 6 cmove
  IpAddress      r@ ARP.SenderIP 4 cmove
  zero-mac       r@ ARP.TargetMAC 6 cmove
  requested-ip   r@ ARP.TargetIP 4 cmove

  r> ARP send_ether_pkt
;


: pingonce	\ -- ; an ARP request
\ *G Send an ARP request and process the response once.
\ ** All initialisation must have been performed before
\ ** using this word.
  cr  ticks			\ start time
  SendArpReq
  #1000 later			\ end of 1 second
  begin
    isrx? if
      pbuff #1536 get_ether_pkt . 
      pbuff EHDR.type w@(n) ET_ARP =
      pbuff AHDR.op w@(n) ARPOP_REPLY =  and
      pbuff ARP.targetMAC EtherAddress 6 s=  and
    else
      dup expired dup
      if  ." Timed out "  endif
    endif
  until
  drop
  ticks swap - . ." ms"			\ elapsed time
;

: pingit	\ -- 
\ *G Ping the IP address at *\fo{REQUESTED-IP} until a key is pressed.
  InitEther
  begin
    pingonce  #500 ms
    key?
  until
  ( Stop-Timers )
;

[then]


\ ******
\ *> ###
\ ******

decimal
