\ EtherSTR91x.fth - STR91xxx Ethernet driver

((
Copyright (c) 2007, 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20081125 SFP001 Added ETHERLINK?
		Overhauled initialisation for PowerNet v4.5
))

decimal


\ ==============
\ *! etherstr91x
\ *T STR91x Ethernet Driver Code
\ ==============
\ *S Introduction
\ *P The Ethernet code in *\i{ARM\Drivers\EtherSTR91x} is derived
\ ** from the sample C driver from the ST website http://www.st.com.
\ ** It was tested on a Hitex board.

\ *P There are several gotchas when using the STR91x EMAC.
\ *(
\ *B Data coherency for ARM966 cores is achieved by draining the
\ ** write buffers, which requires a write into unbuffered AHB
\ ** space, or a read from AHB space. Thus global data and
\ ** buffers used by Ethernet interrupts, DMA and mainline code
\ ** must be mapped into AHB unbuffered memory.
\ *B If interrupt operation is not used, as in this case, interrupts
\ ** disabled in the EMAC block do not set bits in the EMAC interrupt
\ ** status registers.
\ *B The EMAC ISR (Interrupt status register) clears all bits when
\ ** read.
\ *)


\ ****************
\ *S Configuration
\ ****************
\ *P The following must be defined before the file
\ ** *\i{ARM\Drivers\EtherSTR91x} is compiled.

interpreter
: const equ ;	\ n -- ; -- n
\ *G If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** *\fo{CONST} as *\fo{CONSTANT}, otherwise by default *\fo{CONST}
\ ** is defined as *\fo{EQU}.
target

[undefined] EtherCom? [if]
0 equ EtherCom?	\ -- flag
\ *G Only needed if the EtherCom generic I/O device is being used.
\ ** This only happens for systems with multiple IP ports and
\ ** PowerNet v3.x upwards.
[then]

[undefined] EtherDiags? [if]
0 equ EtherDiags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. False by default.
[then]

[undefined] MultiPhy? [if]
0 equ MultiPhy?	\ -- flag
\ *G Set this flag true if you are using multiple PHYs or do not
\ ** know the address of the PHY.
[then]

[undefined] sniff? [if]
0 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] pingit? [if]
0 equ pingit?	\ -- flag
\ *G Set this equate true to compile the PING code,
\ ** which tests Ethernet transmission and reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $51 c,	\ six bytes for 48 bit MAC address
\  $00 c, $08 c, $EF c, $00 c, $5E c, $C4 c,	\ six bytes for 48 bit MAC address
[then]						\ you must obtain these yourself!

[undefined] IpAddress [if]
create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
\  10 c,  102 c,  0 c,  11 c,		\ assign these as required
  192 c,  168 c,  0 c,  251 c,		\ assign these as required
[then]


\ ************
\ *S Buffers
\ ************

#1536 equ /ERxBuff	\ -- u
\ *G Size of an Ethernet receive buffer rounded up to 16 byte unit.
#4 equ #ERxBuffs		\ -- u
\ *G Number of receive buffers. Must be at least two.
#1536 equ /ETxBuff	\ -- u
\ *G Size of an Ethernet transmit buffer rounded up to 16 byte unit.
#2 equ #ETxBuffs		\ -- u
\ *G Number of transmit buffers. Must be at least one. Transmit buffers
\ ** are managed in software.

struct /ETRxHdr	\ -- len
\ *G Structure defining a transmit or receive descriptor, which must
\ ** be on a cell boundary.
\ *[
  int trh.control	\ receive control reg
  int trh.addr		\ address of buffer
  int trh.next		\ next descriptor
  int trh.status	\ receive status, bit16=1=DMA, bit16=0=CPU
end-struct
\ *]

[sections  AHBramU	\ headers and buffers in AHB unbuffered RAM
uhere constant ERxHdrs	\ -- addr
\ *G Base address of the receive headers. The receiver steps
\ ** through these headers to find a free buffer. The headers
\ ** *\b{must} start on a 64 byte boundary because of a chip bug.
\ ** The headers should be in *\b{uncached} memory.
  #ERxBuffs /ETRxHdr * uallot

uhere constant ETxHdrs	\ -- addr
\ *G Base address of the transmit headers. The transmitter steps
\ ** through these headers to find a pending buffer. The headers
\ ** *\b{must} start on a 64 byte boundary because of a chip bug.
\ ** The headers should be in *\b{uncached} memory.
  #ERxBuffs /ETRxHdr * uallot

uhere #63 + #-64 and uorg	\ force UDATA to 64 byte boundary
uhere constant ERxBuffs	\ -- addr
\ *G Base address of the receive buffers. Some buffer handling
\ ** techniques may require these buffers to be in uncached
\ ** memory.
  #ERxBuffs /ERxBuff * uallot

uhere constant ETxBuffs	\ -- addr
\ *G Base address of the transmit buffers. These should not be
\ ** in cached memory.
  #ETxBuffs /ETxBuff * uallot
sections]


variable FirstRxHdr	\ -- addr
\ *G Holds the address of the first read header to check.
  ERxHdrs FirstRxHdr !
variable NextRxHdr	\ -- addr
\ *G Holds the address of the next read header to be processed
\ ** by the CPU. This is a change from previous drivers.
  0 NextRxHdr !
variable FirstTxHdr	\ -- addr
\ *G Holds the address of the first transmit header to check.
  ETxHdrs FirstTxHdr !
variable NextTxHdr	\ -- addr
\ *G Holds the address of the next transmit header to use.
  0 NextTxHdr !

bit16 equ ~HDR_VALID
bit14 equ ~NXT_EN
bit0 equ ~NPOL_EN

\ InitERxHdrs and InitETxHdrs are basically the same function
\ We need to factor this code!

: InitERxHdrs	\ --
\ *G Initialise the receive headers.
  ERxHdrs dup FirstRxHdr !  NextRxHdr !	\ Set both first & next pointers to start of buffers
  ERxBuffs ERxHdrs  #ERxBuffs 0 ?do	\ -- buff hdr
    /ERxBuff ~NXT_EN or over trh.control !	\ control=buffer length, auto-fetch
    2dup trh.addr !			\ buffer start address
    dup /ETRxHdr + ~NPOL_EN or		\ next descriptor - and set NPOL_EN
    over trh.next !			\ Store it in "next" field in header
    ~HDR_VALID over trh.status !	\ status = DMA
    swap /ERxBuff + swap /ETRxHdr +	\ step to next
  loop					\ -- buff' hdr'
  ERxHdrs ~NPOL_EN or swap /ETRxHdr - trh.next ! \ set last header points back to start
  drop
;

: InitETxHdrs	\ --
\ *G Initialise the transmit headers.
  ETxHdrs dup FirstTxHdr !  NextTxHdr !	\ Set both first & next pointers to start of buffers
  ETxBuffs ETxHdrs  #ETxBuffs 0 ?do	\ -- buff hdr
    ~NXT_EN over trh.control !		\ control=enable automatic fetch of next header
					\ Length needs to be set when packet is sent
    2dup trh.addr !			\ buffer start address
    dup /ETRxHdr + ~NPOL_EN or		\ next descriptor - and set NPOL_EN
    over trh.next !			\ Store it in "next" field in header
    0 over trh.status !			\ status = CPU
    swap /ETxBuff + swap /ETRxHdr +	\ step to next
  loop					\ -- buff' hdr'
  ETxHdrs ~NPOL_EN or swap /ETRxHdr - trh.next !  \ set last header points back to start
  drop
;


\ *************
\ *S PHY access
\ *************
\ *P This section is specifically for the STE100P PHY as fitted to
\ ** the STR912 Development Board. Other PHYs will need to be
\ ** checked to make sure that they correspond to the same
\ ** general model.

\ PHY registers
0x00 const PHY_CNTL_REG
0x01 const PHY_STATUS_REG
0x02 const PHY_ID_REG1
0x03 const PHY_ID_REG2
0x04 const PHY_ANA_REG
0x05 const PHY_ANLPAR_REG
0x06 const PHY_ANE_REG
0x11 const PHY_XCIIS_REG
0x12 const PHY_XIE_REG
0x13 const PHY_100CTR_REG
0x14 const PHY_XMC_REG

\ PHY control register
bit15 const RESET_PHY
bit14 const ENABLE_LOOPBACK
bit13 const DR_100MB
bit12 const ENABLE_AN
bit11 const PHY_POWERDOWN
bit10 const PHY_MAC_ISOLATE
bit9  const RESTART_AN
bit8  const PHY_FULLDUPLEX
bit7  const PHY_COL_TEST

\ PHY Status register
bit5 const AN_COMPLETE

\ PHY Auto-negotiation Advertisement register bits
bit8 const DR100_TX_FULL
bit7 const DR100_TX_HALF
bit6 const DR10_TX_FULL
bit5 const DR10_TX_HALF
DR100_TX_FULL DR100_TX_HALF or DR10_TX_FULL or
  const ANE_ALL	\ request everything

\ PHY XCIIS register
bit9 const XCIIS_100Mb
bit8 const XCIIS_FULL_DUPLEX
bit7 const XCIIS_PAUSE
bit6 const XCIIS_AN_COMPLETE
bit5 const XCIIS_REMOTE_FAULT
bit4 const XCIIS_LINK_DOWN
bit3 const XCIIS_AN_ACK_RECD
bit2 const XCIIS_PD_FAULT
bit1 const XCIIS_AN_PAGE_RECD
bit0 const XCIIS_REC_ERR_FULL

\ PHY_100CTR_REG
bit4 bit3 bit2 or or const CMODE

: WaitPhyDone	\ --
\ *G Wait until the current PHY operation has completed.
  begin
    _ENET ENET_MIIA + @ bit0 and	\ bit 0 = busy
   while
    pause
  repeat
;


\ *P Two versions of the following two words are provided, the
\ ** first allowing use of a current PHY, the second for a single
\ ** known PHY. The first version is used if the equate
\ ** *\fo{MultiPhy?} is non-zero.

MultiPhy? [if]
0 value Phy#	\ -- n
\ *G Multi PHY: The number of the PHY in use: 0..31
: ~Phy#		\ -- mask
\ *G Multi PHY: Mask used for PHY command generation.
  Phy# #11 lshift
;
[else]
0 equ Phy#	\ -- n
\ *G Single PHY: The number of the PHY in use: 0..31
Phy# #11 lshift equ ~Phy#	\ -- mask
\ *G Single PHY: Mask used for PHY command generation.
[then]

bit0 equ ~Busy	\ -- n
\ *G Set bit 0 in MIIA register to make the command execute

: ReadPhy	\ reg -- w
\ *G Read a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  $1F and #6 lshift			\ form reg# mask
  ~Phy# or				\ set up reg read command, bit 1 = 0 = read
  ~Busy or				\ Busy bit needs to be set to make command active
  _ENET ENET_MIIA + !			\ set command
  WaitPhyDone				\ let operation complete
  _ENET ENET_MIID + @ $FFFF and		\ return data
;

: WritePhy	\ w reg --
\ *G Write a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  swap _ENET tuck			\ -- reg base w base
  ENET_MIID + !				\ -- reg base ; write data
  swap $1F and #6 lshift		\ form reg# mask
  ~Phy# or  bit1 or			\ set up reg command, bit 1 = 1 = write
  ~Busy or				\ Busy bit needs to be set to make command active
  ENET_MIIA + !				\ set command
;

: OrPhy		\ mask reg --
\ *G Or mask into reg in the PHY. Set bits in mask are set in the
\ ** register.
  tuck ReadPhy  or  swap WritePhy
;

\ SFP001...
: resetPhy	\ --
\ *G Initialise the PHY with autonegotiation enabled.
  RESET_PHY PHY_CNTL_REG OrPhy		\ software reset the chip
  begin					\ wait until reset complete
    PHY_CNTL_REG ReadPhy RESET_PHY and 0=
  until
  ANE_ALL PHY_ANA_REG OrPhy		\ request 10/100BaseT features
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
;

: autoNegotiate	\ -- ior ; 0=success
\ *G Perform the autonegotiation sequence. If no link can be
\ ** established, return ior=non-zero. On success, return zero.
\ ** For most PHYs this is done by a timeout, say 5 seconds.
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
\ cr ." Starting AutoNegotiation"
  RESTART_AN PHY_CNTL_REG OrPhy		\ start autonegotiation
  5000 later				\ limit to 5 seconds
  begin
    pause				\ wait until done
    dup timedout? 0=
   while
    PHY_STATUS_REG ReadPhy AN_COMPLETE and
  until then
  timedout?
  if  -1  exit  endif
\ cr ." AutoNegotiation done"
\ set up control reg using the results
  PHY_ANLPAR_REG ReadPhy
  0					\ -- partner cntlset
  over DR100_TX_FULL and
  if  DR_100MB PHY_FULLDUPLEX or or  endif
  over DR100_TX_HALF and
  if  DR_100MB or  endif
  over DR10_TX_FULL and
  if  PHY_FULLDUPLEX or  endif
  PHY_CNTL_REG WritePhy
  drop 0
;

: EtherLink?	\ -- flag
\ *G Return true if the Ethernet link is established. For most
\ ** PHYs, this is performed by testing bit 2 of register 1.
  1 ReadPhy bit2 and 0<>
;
\ ...SFP001


\ *****************
\ *S Initialisation
\ *****************

$82a8 equ ~GPIO1Outputpins
\ *G Bit mask for pins 1,2,3,4,7 to be Alternate Output 2
$00a0 equ ~GPIO5Outputpins
\ *G Bit mask for pins 2,3 to be Alternate Output 2

$fff3 equ ~GPIO0InterestingPins
\ *G Bit mask designating all pins we're interested in for port 0
$ffff equ ~GPIO1InterestingPins
\ *G Bit mask designating all pins we're interested in for port 1
$00f0 equ ~GPIO5InterestingPins
\ *G Bit mask designating all pins we're interested in for port 5

interpreter
: ccrdivisor ( -- ccrbitfield )
\ *G INTERPRETER only: Find divisor field for ENET_CCR from the
\ ** values of *\fo{/HCLKdiv} and *\fo{/PCLKdiv}.
\ ** Aborts compilation if an invalid ratio has been specified.
  /HCLKdiv /PCLKdiv  2dup = if		\ values are equal - bitfield is "00"
    2drop 0
  else
    4 lshift swap 4 lshift swap		\ Both now 16 times bigger, so integer div works
    / 2 = if				\ pclk is half hclk - bitfield is "01"
      1
    else				\ Invalid ratio selected
      cr  true abort" invalid HCLK to PCLK ratio for ENET unit"
    then
  then
;
target

ccrdivisor equ ccrdivresult

((
For a MAC address conventionally defined as:
  aa-bb-cc-dd-ee-ff
the layout of the MAL/H registers is:
MAL:
        ---------------------
    MSB | cc | dd | ee | ff | LSB
        ---------------------
MAH:
        ---------------------
    MSB | 00 | 00 | aa | bb | LSB
        ---------------------

N.B. This is drawn from the ST evaluation code; the STR91x User Manual is
incorrect (it only allows 32-bit MACs)
))

: setMACAddress	\ addr --
\ *G Set the MAC address in the EMAC unit.
  dup 5 + c@
  over 4 + c@ 8 lshift or
  over 3 + c@ #16 lshift or
  over 2 + c@ #24 lshift or
  _ENET ENET_MAL + !
  dup 1 + c@  swap 0 + c@ 8 lshift or
  _ENET ENET_MAH + !
;

: EnPer1	\ mask --
\ *G Enable peripherals on set 1 by enabling the clock and removing reset.
  _SCU
  2dup SCU_PCGR1 + or!			\ enable clock
  SCU_PRR1 + or!			\ remove reset
;

: SetGPIOPins	\ mask GPIOnum --
\ *G Set up GPIO output register according to mask
\ ** N.B. This assumes the appropriate bits are zero to start, so the mask
\ ** is just ORed in
  2 lshift _SCU SCU_GPIOOUT + +		\ Address of appropriate register
  or!					\ Put bits in
;

: ClearGPIOPins	\ mask GPIOnum --
\ *G Clear GPIO output register according to *\i{mask}. Note that
\ ** this will clear those bits that are *\b{set} in the input
\ ** mask and leave the others alone.
  2 lshift _SCU SCU_GPIOOUT + +		\ Address of appropriate register
  bic!					\ clear bits in register
;


bit0 equ ~DMA_EN
bit2 equ ~START_FETCH
bit2 equ RX_ENABLE
bit3 equ TX_ENABLE
bit6 equ ~RUNT_FRAME
bit7 equ ~COLLISION_SEEN

: StartRxEngine	\ -- flag ; nz=success
\ *G Kick off the RX DMA Engine.
  NextRxHdr @ dup 0= if	\ Check that RX Ring buffers have been initialised
    drop false  exit			\ Wrong calling sequence
  then
  dup trh.addr @ _ENET ENET_RXSAR + !	\ Start address for first DMA operation
  dup trh.control @ _ENET ENET_RXCR + !	\ And control register
  ~NPOL_EN or				\ Set bit to enable descriptor autofetch
  _ENET ENET_RXNDAR + !			\ Load first descriptor into ENET_RXNDAR
  RX_ENABLE _ENET ENET_MCR + or!	\ Enable receiver
  _ENET ENET_RXSTR + 			\ Write the START_FETCH bit, DMA_EN,
  ~DMA_EN ~START_FETCH or
  ~RUNT_FRAME or			\ RUNT_FRAME and
  ~COLLISION_SEEN or			\ COLLISION_SEEN bits into ENET_RXSTR
  swap or!
  true			\ If we got here, it worked
;

: StartTxEngine	\ -- flag ; nz=success
\ *G Kick off the TX DMA Engine.
  NextTxHdr @ dup 0= if			\ Check that TX Ring buffers have been initialised
    drop false  exit			\ Wrong calling sequence
  then
  dup trh.addr @ _ENET ENET_TXSAR + !	\ Start address for first DMA operation
  dup trh.control @ _ENET ENET_TXCR + !	\ And control register
  ~NPOL_EN or _ENET ENET_TXNDAR + !	\ Load first descriptor into ENET_TXNDAR
  TX_ENABLE _ENET ENET_MCR + or!	\ Enable transmitter
  _ENET ENET_TXSTR + 			\ Write the START_FETCH and
  ~DMA_EN ~START_FETCH or swap or!	\ DMA_EN bits into ENET_TXSTR
  true					\ If we got here, it worked
;
bit11 equ ~MACmask
bit15 equ ~GPIO1mask
bit19 equ ~GPIO5mask
bit12 equ ~PHYCLKmask
bit0  equ ~MACRESET

\ SFP001...
: resetEMAC	\ --
\ *G Enable access to the EMAC unit.
  \ Take GPIO 1 and 5 out of reset
  ~GPIO1mask ~GPIO5mask or EnPer1
  \ Turn on PHYCLK
  ~PHYCLKmask _SCU SCU_CLKCNTR + or!	\ Clock control register - holds PHYSEL flag
  \ Set relevant pins in GPIO 0, 1, and 5 to input (default condition)
  ~GPIO0InterestingPins 0 ClearGPIOPins
  ~GPIO1InterestingPins 1 ClearGPIOPins
  ~GPIO5InterestingPins 5 ClearGPIOPins
  \ Configure GPIO1 and 5 output pins to Alt Output 2
  ~GPIO1Outputpins 1 SetGPIOPins
  ~GPIO5Outputpins 5 SetGPIOPins

  \ Force reset of MAC device
  ~MACmask _SCU SCU_PRR0 + bic!		\ Seems to be a problem if we don't reset

  \ Enable MAC device in SCU
  ~MACmask _SCU
  2dup SCU_PRR0 + or!			\ take it out of reset
  SCU_PCGR0 + or!			\ set bit to clock MAC

  \ Set ENET_CCR to set up clocks
  ccrdivresult 2 lshift _ENET ENET_CCR + !	\ Rest of register is forced to zero
  \ Enable MAC in MAC
  ~MACRESET _ENET ENET_SCR + bic!
;

: InitEMAC	\ -- flag ; zero=success
\ *S Initialisation of the Ethernet and PHY, including the MAC
\ ** address.
  resetEMAC
  EtherAddress setMACaddress		\ set up MAC address
  resetPhy autoNegotiate
  if  -1  exit  endif
  InitERxHdrs				\ set up receive headers
  InitETxHdrs				\ set up receive headers
\  EtherAddress setMACaddress		\ set up MAC address
  StartRxEngine				\ Kick off RX DMA
  StartTxEngine				\ Kick off TX DMA
  and 0=
;

: InitEther	\ --
\ *G Perform a full initialisation of the chip, enabling
\ ** Rx and Tx, and setting up the Ethernet MAC address
\ ** from *\fo{EtherAddress}.
  cr ." Initialising Ethernet ... "
  InitEMAC
  if ." un" then ." successful"
;
\ ...SFP001


\ ********************
\ *S Packet read/write
\ ********************

compiler
: StepRxHdr	\ hdr -- hdr'
  trh.next @				\ Point to next buffer in ring
  ~NPOL_EN invert and			\ Strip off ring enable flag
;
target

: StepOnRxHdr				\ --
\ *G Give current header back to the DMA engine and move on to
\ ** the next one.
  ~HDR_VALID NextRxHdr @ trh.status or!	\ give buffer back to DMA
  NextRxHdr dup @ StepRxHdr swap !	\ Point to next buffer in ring
;

$852ac000 equ ~ERROR_BITS
\ *G Good Data in the RX buffer is defined as the data having the
\ ** VALID flag set, i.e. data has been received into this buffer.

: IsRx?         \ -- flag ; nz = has data
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
  NextRxHdr @ trh.status @ 		\ Look at status word in next header
  dup ~HDR_VALID and 0=			\ Check if it's marked "not for DMA", i.e. has data
  dup if				\ And if it is, make sure the data in there is real
    drop  ~ERROR_BITS and dup		\ Look at all the error bits in the word
    if  StepOnRxHdr  then		\ if any set, ditch duff packet
    0=
  else
    nip					\ Discard status word
  then
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get a pending receive packet to a buffer of size *\b{maxlen).
\ ** Return the number of bytes received, or zero for a bad packet.
\ ** The received size of the packet (not clipped to the buffer
\ ** size) is returned.
\ ** In this implementation, a destination address of 0 indicates
\ ** that the packet should be discarded, and no memory transfer
\ ** occurs.
  IsRx? 0= if  2drop 0 exit  endif	\ Make sure we've got some data to read
  NextRxHdr @				\ -- *dest maxlen hdr
  dup trh.addr @ $FFFFFFFC and		\ -- *dest maxlen hdr buff
  swap trh.status @ $000007FF and 4 -	\ -- *dest maxlen buff len
  dup >r  2swap rot min			\ buff *dest len' -- ; R: -- len'
  over					\ check dest address
  if  cmove  else  3drop  endif		\ -- ; R: -- len'
  StepOnRxHdr
  r>					\ return length
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer
\ ** memory.
  0 0 get_ether_pkt drop
;

compiler
: TxStat	\ -- mask
\ *G Read Tx Status register.
  _ENET ENET_TXSR + @
;
target

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop  TxStat bit0 and			\ TXSR "FIFO Empty" bit
;

: send_ether_pkt	\ *pBuf len --
\ *G Send packet from supplied buffer. Note that we copy the
\ ** data to a private area so that the buffer can be released
\ ** immediately.

\ Perform length sanity checks
  dup 0=
  if  2drop exit  endif
  dup /ETxBuff u>
  if  2drop exit  endif
\ Copy packet to uncached memory
  NextTxHdr @ 				\ -- *pbuf len NextHeader

\ Wait for TX buffer to become available
  begin
  dup trh.status @
  ~HDR_VALID and 0= until		\ Wait for VALID flag in status word to be clear
					\ i.e. owned by CPU, not DMA
  trh.addr @				\ -- *pbuf len etxb
  rot swap 2 pick			\ -- len *pbuf etxb len
  aligned cmove				\ copy, alignment forces fast CMOVE
  					\ -- len
  ~NXT_EN or NextTxHdr @ trh.control !	\ Put length into header word
  NextTxHdr dup @ swap
  dup @ trh.next @ 			\ Find next header block
  ~NPOL_EN invert and			\ and remove NPOL_EN flag
  swap !				\ Step NextTxHdr to next header in ring
  trh.status ~HDR_VALID swap or!	\ Set VALID flag to DMA ownership
\ At this point, the TX DMA will pick the packet up & send it if it is running.
;


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************

EtherCom? [if]

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O IPDGetAddr function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the corresponding stored data
\ ** will not be changed.
  drop					\ mode flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove		\ save
    SetMACaddress			\ put in EMAC
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;


\ *P The layout and usage of the Ethernet common I/O structure is
\ ** defined in the file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

create E92vector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** *\i{COMMON\ETHERCOM.FTH}.
  ' InitEther ,         \ initialisation
  ' noop ,	        \ shutdown
  ' IsRx? ,	        \ receive test
  ' get_ether_pkt ,     \ receive packet
  ' IsTx? ,	        \ transmit test
  ' send_ether_pkt ,    \ transmit packet
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

E92vector constant IPDevice	\ -- addr
\ *G IPDevice is the default device name required by *\{ETHERCOM.FTH}.
\ ** If you have multiple ports, only one should be called *\fo{IPDevice}.

[then]


\ ==============
\ *S Diagnostics
\ ==============

EtherDiags? [if]

((
: tn
\ *G No access to structure methods from compiled code, so this
\ ** is to work out where the pointer is going...
  trh.next
;

: vh
\ *G A test word to dump the current state of the headers
  ERxHdrs  #ERxBuffs /ETRxHdr * pdump
;

: vth
\ *G A test word to dump the current state of the TX headers
  ETxHdrs  #ETxBuffs /ETRxHdr * pdump
;

: killit
\ *G Another test word - this one zeroes the RX buffers so I can
\ ** check it's receiving
  ERxBuffs #ERxBuffs /ERxBuff * erase
;

: viewpackets
\ *G Show the first 16 bytes of each buffer
  cr
  #ERxBuffs 0 do
    i dup . ." : " /ERxBuff * ERxBuffs +		\ -- buffptr
    #16 0 do
      dup i + c@ .
    loop drop cr
  loop
;

: readnext
\ *G Display value of next packet status and drop buffer if there
\ ** is one
  IsRx?
  if
    ." Status is " NextRxHdr @ trh.status @ .dword cr
    DiscardRX
  then
;
))

: .phy		\ --
\ *G Display contents of all Ethernet Controller's PHY registers.
  cr ."  0 Control: " 0 readphy .word
  cr ."  1  Status: " 1 readphy .word
  cr ."  2 PhyID#1: " 2 readphy .word
  cr ."  3 PhyID#2: " 3 readphy .word
  cr ."  4 AutoNeg: " 4 readphy .word
  cr ."  5 Ability: " 5 readphy .word
  cr ."  6 Autoexp: " 6 readphy .word
  cr ." 17 Conf/St: " #17 readphy .word
  cr ." 18 TCIE:    " #18 readphy .word
;

MultiPhy? [if]
: .Phys		\ --
\ *G Compiled if the equate *\fo{MultiPhy?} is non-zero.
\ ** Displays the register contents of all PHYs.
  #32 0 do
    cr ." PhyA " i .byte space
    i to Phy#  .Phy
  loop
;
[then]

[then]


\ **************
\ *S System test
\ **************
\ *P The packet sniffer code is only compiled if the equate
\ ** *\fo{SNIFF?} is non-zero.

sniff? [if]

1536 buffer: pbuff	\ -- addr
\ *G Temporary buffer for *\fo{SNIFF}.

: NextRxPkt	\ --
\ *G Collect and display packet
  pbuff 1536 erase
  pbuff 1536 get_ether_pkt dup .word
  pbuff swap 1 max dump
;

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  begin
    isrx?
    if  NextRxPkt  endif
    #10 ms
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]

\ *P The PING example is used to test Ethernet transmission and
\ ** reception. Before testing transmission, test reception by
\ ** using the maintenance system to download a user application
\ ** and then pinging the target board instead of TFTPing to it.
\ *E Example ARP request:  #42/$2A bytes
\ ** C020.0600  FF FF FF FF FF FF 00 C0  F0 0D 9D 7E 08 06 00 01
\ ** C020.0610  08 00 06 04 00 01 00 C0  F0 0D 9D 7E C0 A8 01 7F
\ ** C020.0620  00 00 00 00 00 00 C0 A8  FF FE 11 0A 75 9C C0 A8

\ *P Note that the equates *\fo{SNIFF?} and *\fo{PINGIT?} must both
\ ** be set non-zero to compile the PING code.

pingit? [if]

create requested-ip	\ -- addr
\ *G Holds IP address to be pinged.
  #192 c,  #168 c,  0 c,  50 c,

create zero-mac		\ -- addr
\ *G Holds MAC address 00-00-00-00-00-00
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

create EtherBC		\ -- addr
\ *G Holds MAC address FF-FF-FF-FF-FF-FF
  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

$0001 equ ARPHARD_ETHER
$0800 equ ARPPROTO_IP

$0001 equ ARPOP_REQUEST
$0002 equ ARPOP_REPLY

: SendArpReq	\ -- ; send ARP request
\ *G Send an ARP request.
\ ** All initialisation must have been performed before
\ ** using this word.
  pbuff >r

  EtherBC r@ EHDR.DstMac 6 cmove
  EtherAddress   r@ EHDR.SrcMac 6 cmove
  ET_ARP         r@ EHDR.type w!(n)

  ARPHARD_ETHER  r@ AHDR.htype w!(n)
  ARPPROTO_IP    r@ AHDR.ptype w!(n)
  6              r@ AHDR.hlen c!
  4              r@ AHDR.plen c!
  ARPOP_REQUEST  r@ AHDR.op w!(n)

  EtherAddress   r@ ARP.SenderMAC 6 cmove
  IpAddress      r@ ARP.SenderIP 4 cmove
  zero-mac       r@ ARP.TargetMAC 6 cmove
  requested-ip   r@ ARP.TargetIP 4 cmove

  r> ARP send_ether_pkt
;

: pingonce	\ -- ; an ARP request
\ *G Send an ARP request and process the response once.
\ ** All initialisation must have been performed before
\ ** using this word.
  cr  ticks				\ starting time
  SendArpReq
  #1000 later begin
    isrx? if
      pbuff #1536 get_ether_pkt .
      pbuff EHDR.type w@(n) ET_ARP =
      pbuff AHDR.op w@(n) ARPOP_REPLY =  and
      pbuff ARP.targetMAC EtherAddress 6 s=  and
    else
      dup expired dup
      if  ." Timed out "  endif
    endif
  until
  drop
  ticks swap - . ." ms"			\ elapsed time
;

: pingit	\ --
\ *G Ping the IP address at *\fo{REQUESTED-IP} until a key is pressed.
  InitEther
  begin
    pingonce  #500 ms  key?
  until
  ( Stop-Timers )
;

[then]


\ ******
\ *> ###
\ ******

decimal
