\ Boot loader application code for Ethernet systems

((
Copyright (c) 2000, 2002, 2003, 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050225 SFP009 Added checks for presence of ticker and/or timebase.
20030621 SFP008 Permitted no USER flash.
20030620 SFP007 Moved Network order/CPU dependent code to NETCODE.FTH.
		Corrected fault when no application is present.
20030111 SFP006 Put code in separate vocabulary to ease 
		integration with PowerNet.
20021102 SFP005 Added reboot facility
20020918 SFP004 Generalised and moved to the DRIVERS directory.
20020831 SFP003 Faster, smaller checksum code.
20020830 SFP002 Modified for Samsung board.
		Made the network order fetch/store operations
		usable on 16 bit alignments rather than 32 bit
		alignments as these depend on the ARM CPU bus
		interface permissions. Some ARMs with 16 bit
		bus interfaces permit 16 bit aligned 32 bit
		operations, but 32 bit bus systems may/will fail.
		Improved high level checksum code for ARM.
20020627 SFP001 Updated and tested on StrongBox rev 1.
20020617 SFP000 First release
))


\ check for required code.
[undefined] start-timers [if]
  [undefined] start-clock [if]
    cr ." This code requires START-CLOCK or START-TIMERS and MS"  abort
  [then]
  Synonym Start-Timers Start-Clock
  Synonym Stop-Timers  Stop-Clock
[then]


\ **********
\ *! Netboot
\ *T Network boot code and TFTP
\ **********
\ *P The file *\i{NETBOOT.FTH} provides facilities for file transfer into Flash
\ ** using Ethernet and TFTP. Menu options allow various facilities for 
\ ** boot management, replacement of user applications, replacement of
\ ** boot code, and Ethernet MAC and IP address configuration. The 
\ ** timebase system and a Flash driver are required. The multitasker
\ ** is disabled by the maintenance menu, and should be disabled
\ ** if you use the TFTP downloader words. *\i{NETBOOT.FTH} requires
\ ** *\i{NETCODE.FTH} for network order and CPU dependent operations.

\ *P If the CPU can be reset under software control, you can provide a
\ ** word called *\fo{REBOOT} to reset the CPU. If *\fo{REBOOT} is present, the
\ ** maintenance menu will offer to reboot the the CPU after a new boot
\ ** flash program has been installed.

\ *P If the equate *\fo{PING?} is non-zero, a simple PING test utility is
\ ** compiled so that the Ethernet layer can be tested. The examples
\ ** folder contains the file *\i{Drivers\Smc91C9x.fth} which can be
\ ** used as a model for your Ethernet driver.

decimal


\ ****************
\ *S Configuration
\ ****************
\ *P The required ethernet MAC Address and the IPv4 address are configured
\ ** by two *\fo{CREATE}s in the control file.

\ *P If these equates are defined before *\i{NETBOOT.FTH} is
\ ** compiled, the default settings below will be ignored.

[undefined] PING? [if]
true equ ping?	\ -- flag
\ *G Set this equate non-zero to include the PING code.
[then]

[undefined] BOOTMENU? [if]
false equ BootMenu?	\ -- flag
\ *G Set this equate non-zero to include the power up menu code.
\ ** The word *\fo{BOOTSTRAP} should then be set as the application
\ ** start up word using the phrase "MAKE-TURNKEY BOOTSTRAP" 
\ ** in your control file.
[then]


\ **************
\ *S NETBOOT.VOC
\ **************

vocabulary netboot.voc	\ --
\ *G The NETBOOT code is held in a separate vocabulary
\ ** *\fo{NETBOOT.VOC} to avoid name conflicts with the PowerNet
\ ** source code.
also netboot.voc definitions


\ ****************************
\ *S Handling the Flash memory
\ ****************************

variable DestPtr	\ -- addr
\ *G Holds the next address in Flash to be written to.

#512 buffer: FlWriteBuff
\ *G Scratch buffer to hold TFTP data when the source address
\ ** is not cell aligned. This is usually the case because of
\ ** the TFTP header layout.

: WriteFlashBlock       \ addr len --
\ *G Write the buffer addr/len to Flash at the address
\ ** in *\fo{DESTPTR} and then add len to *\fo{DESTPTR}.
\ ** Len must not be greater than 512 bytes.
  over 3 and if				\ If unaligned
    >r
    FlWriteBuff r@ cmove		\ copy to aligned buffer
    FlWriteBuff r>
  endif
  tuck
  DestPtr @ swap (ProgFlash)
  DestPtr +!
;

UserFlashLen [if]
: run-user	\ --
\ *G Copy flash into ram at address APPRAM, and execute at address APPRAM.
\ ** If the data at the start of the application Flash area is $FFFFFFFF,
\ ** which is the signature of unprogrammed flash, *\fo{RUN-USER} issues an
\ ** error message and returns.
  UserFlash @ $FFFFFFFF =		\ Check for application
  if  cr ." No application present" exit  then

  stop-timers  di			\ disable all interrupts
  UserFlash AppRam UserFlashLen cmove	\ copy Flash to RAM
  AppRam execute			\ go do it
;
[then]


\ ****************
\ *S Miscellaneous
\ ****************

: memswap       \ a1 a2 len --
\ *G Exchange the contents of two buffers.
  bounds ?do
    i c@                \ a1 c2 --
    over c@             \ a1 c2 c1 --
    i c!                \ a1 c2 --
    over c!             \ a1 --
    1+
  loop
  drop
;

: .z$           \ addr --
\ *G Display a zero terminated string at addr.
  begin
    dup c@ dup
  while
    emit 1+
  repeat
  2drop
;


\ *****************************
\ *S Misc. IP and MAC functions
\ *****************************

create EtherAddressBC	\ -- addr
\ *G The Ethernet broadcast adress for checking for broadcast addresses.
  255 c, 255 c, 255 c, 255 c, 255 c, 255 c,

: MyIp?         \ *ip -- flag ; true for match
\ *G Returns true if the IPv4 address at *ip is the same as the one
\ ** at *\fo{IPADDRESS}.
  IpAddress 4 s=
;

: BroadCastMac? \ *ip -- flag
\ *G Returns true if the IPv4 address at *ip is the same as the one
\ ** at *\fo{ETHERADDRESSBC}.
  EtherAddressBC 6 s=
;

: CurrentIP	\ -- ; display IP address
\ *G Display the current IP address.
  base @ >r  decimal
  cr ." Current IP address: "
  IpAddress
  dup c@     0 .r ." ."
  dup 1 + c@ 0 .r ." ."
  dup 2 + c@ 0 .r ." ."
      3 + c@ 0 .r
  r> base !
;

: CurrentMAC	\ --
\ *G Display current MAC address.
  base @ >r  hex
  cr ." Current MAC Address: " 
  EtherAddress
  dup     c@ 0 .r [char] - emit
  dup 1 + c@ 0 .r [char] - emit
  dup 2 + c@ 0 .r [char] - emit
  dup 3 + c@ 0 .r [char] - emit
  dup 4 + c@ 0 .r [char] - emit
  5 + c@ 0 .r 
  r> base !
;


\ *******************************************
\ *S Ethernet and IP structures and constants
\ *******************************************

128 equ T_IGNOREPACKET	\ -- n ; error codes
129 equ T_IP_BADSIZE
130 equ T_IP_BADVERSION
131 equ T_IP_OFFSET
132 equ T_IP_NOTFORME
133 equ T_IP_UNHANDLED
134 equ T_ICMP_BADSIZE
135 equ T_ICMP_BADTYPE
136 equ T_UNKNOWN_DOMAIN

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0001	equ ARPHARD_ETHER
$0800	equ ARPPROTO_IP

$0001	equ ARPOP_REQUEST
$0002	equ ARPOP_REPLY

struct  IPHDR
  EHDR field      _ehdr2
  1 field IPHDR.version
  1 field IPHDR.tos
  2 field IPHDR.len
  2 field IPHDR.id
  2 field IPHDR.off
  1 field IPHDR.ttl
  1 field IPHDR.p
  2 field IPHDR.cksum
  4 field IPHDR.src
  4 field IPHDR.dst
end-struct

1	equ IPPROTO_ICMP	\ control message protocol
17	equ IPPROTO_UDP		\ user datagram protocol

struct  ICMP
  IPHDR field     _iphdr1
  1 field ICMP.type
  1 field ICMP.code
  2 field ICMP.cksum
  2 field ICMP.id
  2 field ICMP.seq
  0 field ICMP.data
end-struct

$0000 equ ICMP_ECHOREPLY
$0008 equ ICMP_ECHOREQUEST

struct  UDP
  IPHDR field     _iphdr2
  2 field UDP.sport
  2 field UDP.dport
  2 field UDP.ulen
  2 field UDP.usum
  0 field UDP.data
end-struct


\ ******************************
\ *S Buffers and sending packets
\ ******************************

variable udp-data	\ -- addr
\ *G Holds the address of the UDP data payload.
variable udp-datalen	\ -- addr
\ *G Holds the length of the UDP data payload.
variable pktInSize	\ -- addr
\ *G Holds the length of the Ethernet packet.
#1536 buffer: pktIn	\ -- addr
\ *G Buffer containing the last of current Ethernet packet.

: SendEtherPacket	\ -- 
\ *G Send a raw Ethernet packet from the buffer *\fo{PKTIN} whose
\ ** size is contained in the variable *\fo{PKTINSIZE}.
  PktIn PktInSize @ send_ether_pkt
;

: SendPacket	\ --
\ *G Send an IP packet over Ethernet from the buffer *\fo{PKTIN}
\ ** whose size is contained in the variable *\fo{PKTINSIZE}.
\ ** The IP checksum is calculated and inserted into the IP
\ ** header before transmission.
  pktinsize @ EHDR - pktin IPHDR.len w!(n)      \ new ip header length
  PktIn EHDR +  IPHDR EHDR -  cksum             \ .. and recalculate for header
  PktIn IPHDR.cksum w!(n)
  SendEtherPacket
;


\ *******
\ *S TFTP
\ *******

$001 equ TFTP_RRQ                \ Read ReQuest
$002 equ TFTP_WRQ                \ Write ReQuest
$003 equ TFTP_DATA               \ DATA packet
$004 equ TFTP_ACK                \ ACKnowledge
$005 equ TFTP_ERROR              \ ERROR condition

variable Uploading?	\ -- addr
\ *G Holds true if a TFTP upload is in progress.
variable blockid	\ -- addr
\ *G Holds the block identifier
variable UploadComplete?	\ -- addr
\ *G Holds TRUE if the TFTP upload has completed.

: ResetTFTP	\ --
\ *G Reset (initialise) the TFTP state variables.
  false Uploading? !
  0 UpLoadComplete? !
  0 blockid !
;

: smERROR       \ --
\ *G The host has reported a TFTP error. The error string is displayed.
  cr ." TFTP Error: "
  udp-data @ 4 + .z$			\ display error text from server
  ResetTFTP  T_IGNOREPACKET throw
;

: smDone        \ --
\ *G The TFTP transfer has completed.
  cr ." Upload Complete"
  ResetTFTP  UpLoadComplete? on
;

: AckData       \ --
\ *G Generate a TFTP acknowledge and increment the block number.
  TFTP_ACK udp-data @ w!(n)
  blockid @ udp-data @ 2+ w!(n)
  4 udp-datalen !
  1 blockid +!
;

: smBegin       \ --
\ *G A TFTP transfer start has been detected.
  UpLoading? @ if
    cr ." TFTP Error: Upload begun before previous complete."
    ResetTFTP  T_IGNOREPACKET throw
  then

  cr ." TFTP Receiving file: " udp-data @ 2+ .z$ space
  true Uploading? !
  AckData
;

: smData        \ --
\ *G Process a TFTP data packet.
  udp-data @ cell+ udp-datalen @ cell -
\ dup .
  WriteFlashBlock
  udp-datalen @ #516 <> >r
  AckData
  r> if smDone then
;

: process-ip-udp-tftp   \ --
\ *G Process a TFTP packet.
  udp-data @ w@(n)
  case
    TFTP_WRQ of         smBegin         endof
    TFTP_ERROR of       smError         endof
    TFTP_DATA of        smData          endof
      cr ." TFTP Error: Unexpected packet"
      ResetTFTP  T_IGNOREPACKET throw
  endcase
;


\ ********************
\ *S IP, UDP, and ICMP
\ ********************

#20 buffer: temp-ip	\ -- addr
\ *G A scratch buffer for manipulating headers.

: process-ip-udp        \ --
\ *G Process UDP packets.
  pktInSize @ UDP                       <  if T_IGNOREPACKET throw then

  pktin udp.sport dup w@(n)     	\ *s s --
  pktin udp.dport dup w@(n)     	\ *s s *d d --
  dup >r                        	\ *s s *d d -- ; d --
  -rot w!(n)                    	\ swap port numbers for reply
  swap w!(n)
  r>                            	\ dest-port --

  pktin UDP.data udp-data !             \ split buffer to header and data
  pktinsize @ UDP - udp-datalen !       \ sections
  UDP pktinsize !

  case
    #69 of      process-ip-udp-tftp     endof
    drop T_IGNOREPACKET throw
  end-case

  \ recombine udp data with original header (assumes data in same place)
  udp-datalen @ pktinsize +!

  \ Update UDP data length ( data length+sizeofUDP header )
  udp-datalen @ 8 + pktin UDP.ulen w!(n)

  \ Update UDP checksum
  0 pktin UDP.usum w!(n)

  pktin IPHDR.src @(n) temp-ip !(n)
  pktin IPHDR.dst @(n) temp-ip 1 cells + !(n)
  IPPROTO_UDP temp-ip 2 cells + w!(n)
  udp-datalen @ 8 + temp-ip #10 + w!(n)	\ size of udp data + udp header
  pktin IPHDR + temp-ip #12 + 8 cmove

  temp-ip #20 cksum $FFFF xor
  udp-data @ udp-datalen @ cksum $FFFF xor +
  dup $FFFF > if 1+ then $FFFF and $FFFF xor

  pktin UDP.usum w!(n)

  \ .. and send
  SendPacket
;

: process-ip-icmp	\ --
\ *G Process ICMP packets.
  pktInSize @ ICMP                      <  if T_ICMP_BADSIZE throw then
  PktIn ICMP.type c@ ICMP_ECHOREQUEST   <> if T_ICMP_BADTYPE throw then

  ICMP_ECHOREPLY PktIn ICMP.type c!     \ switch type to reply
  0 PktIn ICMP.cksum w!(n)              \ reset checksum

  PktIn IPHDR +                         \ pointer to icmp header
  PktInSize @ IPHDR -                   \ size of ICMP block
  cksum PktIn ICMP.cksum w!(n)          \ recalc checksum

  SendPacket
;

: process-ip	\ --
\ *G Process IP packets.
  pktInSize @ IPHDR                     <  if T_IP_BADSIZE    throw then
  pktIn IPHDR.version c@ $45            <> if T_IP_BADVERSION throw then
  pktIn IPHDR.off w@(n)                    if T_IP_OFFSET     throw then
  pktIn IPHDR.dst MyIp?                 0= if T_IP_NOTFORME   throw then


  PktIn IPHDR.src                       \ swap addresses ready for reply
  PktIn IPHDR.dst 4 memswap

  0 PktIn IPHDR.cksum w!(n)             \ clear checksum field

  pktIn IPHDR.p c@
  case
    IPPROTO_ICMP of     process-ip-icmp         endof
    IPPROTO_UDP  of     process-ip-udp          endof
    drop T_IP_UNHANDLED throw
  end-case
;


\ ******
\ *S ARP
\ ******

: process-arp-request   \ --
\ *G Process an ARP request packet.
  pktIn AHDR.htype w@(n) ARPHARD_ETHER <> if T_IGNOREPACKET throw then
  pktIn AHDR.ptype w@(n) ARPPROTO_IP   <> if T_IGNOREPACKET throw then
  pktIn AHDR.hlen c@ 6                 <> if T_IGNOREPACKET throw then
  pktIn AHDR.plen c@ 4                 <> if T_IGNOREPACKET throw then
  pktIn ARP.targetIP MyIp?             0= if T_IGNOREPACKET throw then
  pktIn ARP.senderMAC			\ copy sender MAC and IP to target
  pktIn ARP.targetMAC #10 cmove
  EtherAddress PktIn ARP.senderMAC 6 cmove       \ put in my details
  IpAddress    PktIn ARP.senderIP  4 cmove

  ARPOP_REPLY PktIn AHDR.op w!(n)               \ set ARPOP_REPLY

  pktIn ARP.targetMAC pktIn EHDR.DstMac 6 cmove  \ set ethernet packet addresses
  pktIn ARP.senderMAC pktIn EHDR.SrcMac 6 cmove
  SendEtherPacket
;

: process-arp   \ --
\ *G Process ARP packets.
  pktInSize @ ARP < if T_IGNOREPACKET throw then
  ARP PktInSize !
  pktIn AHDR.op w@(n) ARPOP_REQUEST = if
    process-arp-request
  else
    T_IGNOREPACKET throw
  then
;


\ *********************
\ *S Tieing it together
\ *********************

: boot-process	\ --
\ *G Process an Ethernet packet.
  pktInSize @ EHDR < if T_IGNOREPACKET throw then       \ too short

  pktIn EHDR.DstMac BroadCastMAC? 0= if         \ swap if not broadcast
    PktIn EHDR.DstMac
    Pktin EHDR.SrcMac
    6 memswap
  then

  case  pktIn EHDR.type w@(n)
    ET_IP  of  process-ip   endof
    ET_ARP of  process-arp  endof
      T_UNKNOWN_DOMAIN throw
  endcase
;


\ *************************************
\ Serial and Ethernet Flash programming
\ *************************************

((
: nextSerUnit	{ *flash | buff[ 4 ] -- flag }
\ *G Program the next serial data into a Flash unit,
\ ** assuming that the data is being sent as a memory
\ ** image. The returned flag is 0 for success.
   buff[ 4 bounds
   do  key i c!  loop
   buff[ @ *flash 4 write32
;

: serial-download	{ *flash len -- }
  *flash len (eraseFlash)
  cr ." Start serial transfer now"
  cr ." Reset board when transfer is complete"
  cr
  *flash len bounds ?do
    nextSerUnit if
      cr ." Write failed at address: " i .lword
      unloop exit
    endif
    i $FF and 0= if [char] . emit then
  4 +loop				\ must match size of nextSerUnit
;
))

: ether-download	{ *flash len -- }
  *flash 0= if  [I  endif		\ protect if overwriting interrupt vectors

  *flash DestPtr !
  *flash len (eraseFlash)

  ResetTFTP  InitEther  CurrentIP
  cr ." Start Ethernet TFTP transfer now"
  cr ." Press <ESC> to abort transfer"
  cr

  begin
    isrx? if	\ check for ethernet download
      pktIn #1536 get_ether_pkt pktInSize !
      [I  ['] boot-process catch  I]
      ?dup if
        case
\          T_IGNOREPACKET   of   cr ." >> Ignored"         endof
\          T_UNKNOWN_DOMAIN of   cr ." >> Unknown Domain"  endof
          T_IP_BADSIZE     of   cr ." >> Bad IP Size"     endof
          T_IP_BADVERSION  of   cr ." >> Bad IP Version"  endof
          T_IP_OFFSET      of   cr ." >> Bad IP Offset"   endof
\          T_IP_NOTFORME    of   cr ." >> IP Not for me"   endof
          T_IP_UNHANDLED   of   cr ." >> Unhandled IP"    endof
          T_ICMP_BADSIZE   of   cr ." >> Bad ICMP Size"   endof
          T_ICMP_BADTYPE   of   cr ." >> Bad ICMP Type"   endof
          drop
        end-case
      then
    then
    *flash if
      key? if
        key $1B =
        if  UpLoadComplete? on  then
      then
    then
    UpLoadComplete? @
  until
  base @ decimal
  cr  DestPtr @ *flash - . ." bytes programmed"
  base !

  *flash 0= if  I]  then
;

: download      \ *flash len --
\ *G Download a file by Ethernet/TFTP
\ or by serial line
\ ** and program it into the Flash Block at *flash/len,
\ ** which will be erased before the file transfer starts.
  ether-download
;
((
: download      \ *flash len --
  cr ." Select"
  cr ."  E - Ethernet and TFTP"
  cr ."  S - Serial 8 bit binary"
  cr ."  other - no action"
  cr ." S/E " key upc dup emit
  case
    [char] S of  serial-download  endof
    [char] E of  ether-download   endof
      nip nip
  endcase
;
))

: NewBootFlash	\ --
\ *G Update the boot Flash area by TFTP.
[ BootFlash 0= ] [if]  Stop-Timers  [then]
  BootFlash BootLen download  ( nowReset )
\ [ BootFlash 0= ] [if]  Start-Timers  [then]
;


\ ***********************************************
\ *S Displaying and Entering IP and MAC addresses
\ ***********************************************

: getByte	\ addr char -- addr'
\ *G Collect an integer delimited by char and store it
\ ** at addr, returning addr+1.
  word integer? 1 <> throw		\ collect and convert
  over c!  1+				\ stash data and update address
;

: getData	\ addr char n --
\ *G Collect n integers delimited by char, and store the bytes
\ ** sequentially at addr.
  1- 0 max 0 ?do
    tuck getByte swap
  loop
  drop  bl getByte drop
;

: GetIp		\ -- ; input new IP address
\ *G Ask the user for an IP address and place it in
\ ** *\fo{IPADDRESS} if valid.
  base @ >r  decimal
  begin
    cr ." Please Enter New IP Address: xxx.xxx.xxx.xxx "
    cr FlushKeys Query  #tib @ if
      IPaddress [char] . 4 ['] getData catch dup
      if  nip nip nip  then
    else
      0
    endif
   while
     2drop drop
     cr ." Invalid IP address"
     CurrentIP
  repeat
  CurrentIP
  r> base !
;

: GetMac	\ -- ; input new MAC address
\ *G Get a new MAC address from the user console.
  base @ >r  hex
  begin
    cr ." Please Enter New MAC Address: aa-bb-cc-dd-ee-ff (base is hex) "
    cr FlushKeys Query  #tib @ if
      EtherAddress [char] - 6 ['] getData catch dup
      if  nip nip nip  then
    else
      0
    endif
   while
    2drop drop
    cr ." Invalid MAC address"
    CurrentMAC
  repeat
  CurrentMAC
  r> base !
;

also forth definitions

: RAM>DataFlash	{ | buff[ #10 ] -- }
\ *G When data such as the Ethernet IP or MAC address is changed,
\ ** the data sector must be reprogrammed. Note that *\fo{DATAFLASH>RAM}
\ ** must be used to reload the data at powerup.
  IpAddress buff[ 4 cmove
  EtherAddress buff[ 4 + 6 cmove
  buff[ DataFlash #12 ProgramFlash	\ force integer number of cells
;

: DataFlash>RAM	\ --
\ *G Set up the IP and Ethernet addresses from the data flash area.
  DataFlash @ $FFFFFFFF <> if
    DataFlash @ IpAddress !
  then
  DataFlash 4 + @ $FFFFFFFF <> if
    DataFlash 4 + EtherAddress 6 cmove
  then
;

previous definitions

: SetupIP	\ --
\ *G Get a new Ethernet IP address from the user and reprogram
\ ** the Flash boot sector.
  CurrentIP GetIP RAM>DataFlash
;

: SetupMAC
\ *G Get a new Ethernet MAC address from the user and reprogram
\ ** the Flash boot sector.
  CurrentMAC GetMAC RAM>DataFlash
;


\ ************************
\ *S User maintenance menu
\ ************************
\ *P The maintenance system provides the following features,
\ ** accessed from the menu.
\ *B I - setup the Ethernet IP address
\ *B M - setup the Ethernet MAC address
\ *B F - run the Forth monitor
\ *B R - run the application
\ *B E - erase the user Flash area
\ *B D - download and Flash a new user program
\ *B B - download and Flash a new boot program
\ *B Q - quit maintenance system


decimal

((
: nowReset	\ -- 
  cr ." Please Reset board: "
  begin again
;
))

: y/n?		\ -- flag
\ *G Wait for a key and return true if the key is Y.
  ."  Y/N: " key upc  dup emit  [char] Y =
;

: .maintenance-menu	\ --
\ *G Display the maintenance menu.
  cr
  cr ." Command keys:"
  cr ." I - setup the Ethernet IP address"
  cr ." M - setup the Ethernet MAC address"
  cr ." F - run the Forth monitor"
[ UserFlashLen ] [if]			\ SFP008
  cr ." R - run the application"
  cr ." E - erase the user Flash area"
  cr ." D - download and Flash a new user program"
[then]					\ SFP008
  cr ." B - download and Flash a new boot program"
  cr ." Q - quit maintenance system"
;

: (maintenance)	\ mode -- flag ; flag true if Q pressed
\ *G Enter a single command and process it. If mode is non-zero,
\ ** an invalid key runs an application program if it exists.
\ ** Flag is returned non-zero if Q was pressed.
  >r
  0					\ Q not pressed yet
  case  cr ." Enter command: " key upc dup emit
    [char] I of  
      cr ." Setup Ethernet IP address" SetupIP  ( nowReset )
    endof
    [char] M of
      cr ." Setup MAC address" SetupMAC  ( nowReset )
    endof
    [char] F of
      cr ." Forth Monitor"  cr  s0 @ sp! quit
    endof
[ UserFlashLen ] [if]			\ SFP008
    [char] R of
      cr ." Run user application" run-user
    endof
    [char] D of
      cr ." Download new user program"
      userflash userflashlen download  ( nowReset )
    endof
    [char] E of
      cr ." Erasing user Flash area"
      userflash userflashlen (EraseFlash)  ( nowReset )
    endof
[then]				        \ SFP008
    [char] B of
      cr ." Download new boot program:"
      cr ."   Are you sure?" y/n? if
        NewBootFlash
[defined] REBOOT [if]
        cr ." Reboot now?" y/n?
	if  space reboot  endif
[then]
      endif
    endof
    [char] Q of  cr  drop true  endof
      cr ." Invalid command"
[ UserFlashLen ] [if]			\ SFP008
      r@ if
        ."  - Running User program"
        drop  run-user  0
      endif
[then]					\ SFP008
  endcase
  r> drop
;

: Maintenance 	\ mode -- flag ; true if Q pressed
\ *G Enter the maintenance system. If mode is non-zero,
\ ** the system will time out after 3 seconds and run
\ ** an application program if it exists. Flag is
\ ** returned non-zero if Q was pressed.
\ ** N.B. the multitasker is turned off, but is restored
\ ** on exit, and the timebase system is reinitialised.
[ tasking? ] [if]
  multi? >r  single
[then]
  CurrentIP CurrentMAC
  .maintenance-menu
  Start-Timers				\ stop so that start is interrupt safe
[ UserFlashLen ] [if]			\ SFP008
  if					\ mode is non-zero
    cr ." You have 3 seconds ..."
    0
    begin
      #10 ms
      1+ dup #300 =
      key? or
    until
    #300 < if
      1 (maintenance)
    else
      cr ." no key pressed - Running User program"
      run-user  0			\ SFP007
    then
  else
    0 (maintenance)
  endif
[else]					\ SFP008
  drop  0 (maintenance)			\ SFP008
[then]					\ SFP008
  stop-timers
[ tasking? ] [if]
  r> if  multi  endif
[then]
;


\ ***************************************
\ *S Public words in the FORTH vocabulary
\ ***************************************
\ *P The following words are held in the *\fo{FORTH} vocabulary.

also forth definitions

: maintain	\ --
\ *G Run the maintenance system until Q is pressed.
  DataFlash>RAM
  begin  0 maintenance  until
;

BootMenu? [if]
: bootstrap	\ --
\ *G If the equate *\fo{BOOTMENU?} is non-zero, *\fo{BOOTSTRAP}
\ ** is compiled and may be set as the power up action by
\ ** *\fo{MAKE-TURNKEY}. *\fo{BOOTSTRAP} performs minimal
\ ** system initialisation.
  (init) init-ser			\ initialise Forth and serial line
  console ipvec !  console opvec !	\ select default I/O device
  s0 @ sp!                              \ reset data stack
  cr
  cr .cpu				\ system sign on message
  cr ." MPE Ethernet BootStrap v1.800, 25 February 2005"
  cr ." Copyright (c) 1999-2005 MicroProcessor Engineering Ltd."
  DataFlash>RAM
  begin  1 maintenance drop  again
;
[then]

only forth definitions


\ *******
\ *S PING
\ *******
\ *P The PING example is used to test Ethernet transmission and
\ ** reception. Before testing transmission, test reception by
\ ** using the maintenance system to download a user application
\ ** and then pinging the target board instead of TFTPing to it.
\ *E Example ARP request:  $42/#2A bytes
\ ** C020.0600  FF FF FF FF FF FF 00 C0  F0 0D 9D 7E 08 06 00 01
\ ** C020.0610  08 00 06 04 00 01 00 C0  F0 0D 9D 7E C0 A8 01 7F
\ ** C020.0620  00 00 00 00 00 00 C0 A8  FF FE 11 0A 75 9C C0 A8

\ *P Note that the equate PING? must be set true to compile
\ ** the PING code.

ping? [if]

also netboot.voc

create requested-ip	\ -- addr
\ *G Holds IP address to be pinged.
  #192 c,  #168 c,  0 c,  1 c,

create zero-mac
\ *G Holds MAC address 00-00-00-00-00-00
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

: SendArpReq	\ -- ; send ARP request
\ *G Send an ARP request.
\ ** All initialisation must have been performed before
\ ** using this word.
  pktIn >r

  EtherAddressBC r@ EHDR.DstMac 6 cmove
  EtherAddress   r@ EHDR.SrcMac 6 cmove
  ET_ARP         r@ EHDR.type w!(n)

  ARPHARD_ETHER  r@ AHDR.htype w!(n)
  ARPPROTO_IP    r@ AHDR.ptype w!(n)
  6              r@ AHDR.hlen c!
  4              r@ AHDR.plen c!
  ARPOP_REQUEST  r@ AHDR.op w!(n)

  EtherAddress   r@ ARP.SenderMAC 6 cmove
  IpAddress      r@ ARP.SenderIP 4 cmove
  zero-mac       r@ ARP.TargetMAC 6 cmove
  requested-ip   r@ ARP.TargetIP 4 cmove

  r> ARP send_ether_pkt
;

: pingonce	\ -- ; an ARP request
\ *G Send an ARP request and process the response once.
\ ** All initialisation must have been performed before
\ ** using this word.
  cr  ticks				\ starting time
  SendArpReq

  #1000 later
  begin
    isrx? if
      pktIn #1536 get_ether_pkt pktInSize !
      pktIn EHDR.type w@(n) ET_ARP =
      pktIn AHDR.op w@(n) ARPOP_REPLY =  and
      pktIn ARP.targetMAC EtherAddress 6 s=  and
    else
      dup expired dup
      if  ." Timed out "  endif
    endif
  until
  drop
  ticks swap - . ." ms"			\ elapsed time
;

: pingit	\ -- 
\ *G Ping the IP address at *\fo{REQUESTED-IP} until a key is pressed.
  Start-Timers  InitEther
  begin
    pingonce  #100 ms
    key?
  until
  Stop-Timers
;

previous

[then]


\ **********
\ *S Gotchas
\ **********
\ *P If this code is run from Flash, you can only reprogram
\ ** the flash if dual-plane Flash is used and the sectors
\ ** being programmed are are in a different plane or chip
\ ** from this code.

\ *P If the Flash is mapped at 0000:0000 at run-time, the
\ ** the interrupt vectors will be inaccessible during
\ ** programming of the boot area. The code in *\i{NETBOOT.FTH}
\ ** is paranoid and disables interrupts during processing
\ ** of incoming Ethernet packets, which may upset some
\ ** Ethernet drivers which require enabled interrupts for
\ ** packet transmission.

\ *P In general, RAM is faster than Flash and systems which
\ ** copy code into RAM for execution will be less troublesome.


\ ******
\ *> ###
\ ******

