\ RiscOS support words for the ARM7/SA110 targets
\
\ Released for use with the MPE Forth Cross Compiler by:
\
\ MicroProcessor Engineering
\ 133 Hill Lane
\ Southampton SO15 5AF
\ England
\
\ tel: (+44) 1703 631441   (international)
\           01703 631441   (domestic)
\
\ email: mpe@mpeforth.com
\

\ The following words allow code generated by the ARM cross compiler to access
\ the RiscOS CLI routines (and therefore named SWIs), and to do a very simple version
\ of INCLUDE (i.e. no nesting of files) for testing purposes. These words are somewhat
\ fragile and provide no error checking whatsoever. They do on the other hand allow code
\ to be tested on Acorn ARM based machines running RiscOS 3.5 and above with all the various
\ debugging aids that are available. Much quicker than trying to debug using hardware alone.



\ RiscOS CLI access - any CLI command can be executed, including named SWIs.
\ No error checking is implemented, and if a command fails for any reason the forth will
\ quit and drop back into the RiscOS CLI

create riscos-buff
  256 allot

code os_cli     ( $-addr -- ; string must be terminated by a carriage return )
  mov r0, tos
  swi $05
  ldmfd psp ! { tos }
  next,
end-code

: oscli         ( $-addr -- ; takes addr of a counted string to be executed )
  count dup >r riscos-buff swap cmove
  $0d riscos-buff r> + c!
  riscos-buff
  os_cli
;


\ File handler. Works by redirecting character input to a named file using *EXEC.
\ A modified ACCEPT then reads lines of characters which are interpreted as normal.
\ As there is no EOF character, the end of a file is determined when no character has
\ been read for one second. Input is then directed back to the keyboard. This is a very
\ makeshift solution but appears to work well enough to INCLUDE various test files.

code timed-key  ( -- char|error ; waits for up to one second for a key )
  stmfd psp ! { tos }
  mov r0, # #129                                        \ OS_Byte 129
  mov r1, # $64                                         \ one second timeout - 100cs
  mov r2, # $00
  swi $006                                              \ OS_Byte
  mov .s r2, r2
  mov .eq tos, r1                                       \ valid char
  mov .ne tos, r2                                       \ not valid FF=timeout, 1B=escape cond.
  next,
end-code

variable end-of-file

: file-ACCEPT  (  c-addr +n1 -- +n2 ; read up to LEN chars into ADDR using TIMED-KEY )
  0 -rot bounds                                         \ form address limits
  ?do
    case timed-key 				        \ get character
      $FF   of end-of-file on leave endof               \ EOF (read char timeout)
      bsin  of ?bs               endof	                \ BS
      atab  of bl i save-ch 1+ 1 endof	                \ TAB, convert to space
      alf   of bl i save-ch 1+ 1 endof	                \ LF, convert to space
      acr   of bl ?emit  leave   endof	                \ CR
      ffeed of bl ?emit  leave   endof	                \ FF, treat like CR
      delin of ?bs               endof	                \ DEL
             i save-ch  1+ 1 0		                \ stash char, echo it?
    endcase
  +loop                                                 \ round again
;

: include       ( "<spaces>filename" -- ; include a file from the CSD or path given in the file name )
  bl word
  dup c@ dup >r swap 1+ riscos-buff 6 + rot cmove
  "" exec " riscos-buff 6 cmove
  r> 6 + riscos-buff c!
  riscos-buff
  oscli                                                 \ redirect input to the file using *EXEC <filename>
  end-of-file off                                       \ reset eof flag
  begin
    echoing off                                         \ force off echoing
    tib tib-len 1- file-accept                          \ read line of characters into tib
    dup span ! #tib !
    0 to source-id   tib #tib @ to-source  >in off
    interpret                                           \ interpret line
  end-of-file @ until
  "" exec" oscli                                        \ redirect input back to keyboard with *EXEC
  echoing on
;
