\ LH77790A interrupt RX, polled TX, serial communication drivers

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE

20000618 SFP001 First release
))

only forth definitions


\ *************************
\ Required data definitions
\ *************************

variable rx-char	\ -- addr ; character last read
variable rx-avail	\ -- addr ; true when character received


\ **************
\ Initialisation
\ **************

\ This next word takes the system clock rate (in HZ) and the required serial
\ comms rate (in BPS) and returns a system-dependant number of parameters that
\ should be sufficent to effect the baud rate selection by initialising the
\ correct hardware.

interpreter
: gen-baud-rate	\ clk-rate bps-rate -- 16bit-divisor
  0 swap um/mod nip			\ clk/bps
  #16 /mod swap 8 >=			\ clk/bps/16, check for rounding
  if  1+  endif
;
target

system-speed console-speed gen-baud-rate
  equ serial_divisor

\ internal

: init-ser	\ -- ; initialise system serial port on UART1
  di					\ disable interrupts
  _PCSR @				\ select internal clock source
  PCSR_mask_U1 invert and
  _PCSR  !
  _XCLK/1                 _U1CCR !	\ baud rate prescaler
  $03 $80 or              _LCR1  !      \ "$80 OR" => divisor access via DLAB.
  serial_divisor 8 rshift _DLM1  !	\ set divisor
  serial_divisor $FF and  _DLL1  !
  $03                     _LCR1  !
  $01			  _IER1  !	\ Enable receive interrupt

  _fiqer @ IRQ_mask_U1 invert and _fiqer !	\ disable UART1 on FIQ
  _irqer @ IRQ_mask_U1 or _irqer !		\ enable UART1 on IRQ

  rx-avail off
  ei					\ enable interrupts
;


\ *****************
\ Serial primitives
\ *****************

: (seremit)	\ char --
  begin
    _LSR1 @ %00100000 and
  until
  _THR1 !
;

: ser1-isr	\ --
  _irqsr @ IRQ_mask_U1 and if		\ test for this interrupt
    _RBR1 c@ rx-char !			\ read character (resets interrupt)
    rx-avail on
  endif
;
' ser1-isr add-irq	\ add this ISR to the IRQ list

: (serkey?)	\ -- t/f
  rx-avail @
;

: (serkey)	\ -- char
  begin
    [I
    rx-avail @ 0=
   while
    I]
[ tasking? ] [if] pause [then]
  repeat
  rx-char @
  rx-avail off
  I]
;

external


\ ***********************
\ Generic i/o assignments
\ ***********************

internal

: (serTYPE)	\ c-addr len --
  bounds
  ?do  i c@ (seremit)  loop
;

: (serCR)		\ --
  $0D (seremit)  $0A (seremit)
;

external

cdata
create Console	\ -- addr ; OUT managed by upper driver
  ' (serkey) ,		\ -- char ; receive char
  ' (serkey?) ,		\ -- flag ; check receive char
  ' (seremit) ,		\ -- char ; display char
  ' (sertype) ,		\ caddr len -- ; display string
  ' (sercr) ,		\ -- ; display new line

decimal

