\ LH77790B interrupt RX, polled TX, serial communication drivers

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE

20020917 SFP002 Updated to handle all three UARTS
20000618 SFP001 First release
))

only forth definitions


\ *************************
\ Required data definitions
\ *************************

variable rx-char0	\ -- addr ; character last read
variable rx-avail0	\ -- addr ; true when character received
variable rx-char1	\ -- addr ; character last read
variable rx-avail1	\ -- addr ; true when character received
variable rx-char2	\ -- addr ; character last read
variable rx-avail2	\ -- addr ; true when character received


\ **************
\ Initialisation
\ **************

\ This next word takes the system clock rate (in HZ) and the required serial
\ comms rate (in BPS) and returns a system-dependent number of parameters that
\ should be sufficent to effect the baud rate selection by initialising the
\ correct hardware.

interpreter
: gen-baud-rate	\ clk-rate bps-rate -- 16bit-divisor
  0 swap um/mod nip			\ clk/bps
  #16 /mod swap 8 >=			\ clk/bps/16, check for rounding
  if  1+  endif
;
target

system-speed console0-speed gen-baud-rate
  equ serial_divisor0
system-speed console1-speed gen-baud-rate
  equ serial_divisor1
system-speed console2-speed gen-baud-rate
  equ serial_divisor2

l: uart0_tab	\ -- addr ; initialisation table
  _U0CCR ,		_XCLK/1 ,		\ baud rate prescaler
  _LCR0 ,		$07 $80 or ,        	\ divisor access via DLAB
  serial_divisor0 dup				\ set divisor
  _DLM0 ,		8 rshift ,		\ high
  _DLL0 ,		$FF and ,		\ low
  _LCR0 ,		$07 ,			\ disable DLAB access
  _IER0 ,		$01 ,			\ enable receive interrupt
  0 ,

l: uart1_tab	\ -- addr ; initialisation table
  _U1CCR ,		_XCLK/1 ,		\ baud rate prescaler
  _LCR1 ,		$07 $80 or ,        	\ divisor access via DLAB
  serial_divisor1 dup				\ set divisor
  _DLM1 ,		8 rshift ,		\ high
  _DLL1 ,		$FF and ,		\ low
  _LCR1 ,		$07 ,			\ disable DLAB access
  _IER1 ,		$01 ,			\ enable receive interrupt
  0 ,

l: uart2_tab	\ -- addr ; initialisation table
  _SIR_CTLR ,		0 ,			\ Disable IrDA
  _U2CCR ,		_XCLK/1 ,		\ baud rate prescaler
  _LCR2 ,		$07 $80 or ,        	\ divisor access via DLAB
  serial_divisor2 dup				\ set divisor
  _DLM2 ,		8 rshift ,		\ high
  _DLL2 ,		$FF and ,		\ low
  _LCR2 ,		$07 ,			\ disable DLAB access
  _IER2 ,		$01 ,			\ enable receive interrupt
  0 ,

: init-ser	\ initialise all UARTS
  di
\ UART0
  rx-avail0 off
  PCSR_mask_U0 _PCSR bic!		\ select internal clock source
  uart0_tab init-io			\ use table
  IRQ_mask_U0 _fiqer bic!		\ disable UART on FIQ
  IRQ_mask_U0 _irqer or!		\ enable UART on IRQ
\ UART1
  rx-avail1 off
  PCSR_mask_U1 _PCSR bic!		\ select internal clock source
  uart1_tab init-io			\ use table
  IRQ_mask_U1 _fiqer bic!		\ disable UART on FIQ
  IRQ_mask_U1 _irqer or!		\ enable UART on IRQ
\ UART2
  rx-avail2 off
  PCSR_mask_U2 _PCSR bic!		\ select internal clock source
  uart2_tab init-io			\ use table
  IRQ_mask_U2 _fiqer bic!		\ disable UART1 on FIQ
  IRQ_mask_U2 _irqer or!		\ enable UART1 on IRQ
  ei
;


\ *****************
\ Serial primitives
\ *****************

: serial-isr	\ --
  _irqsr @ IRQ_mask_U0 and if		\ test for this interrupt
    _RBR0 c@ rx-char0 !			\ read character (resets interrupt)
    rx-avail0 on
  endif
  _irqsr @ IRQ_mask_U1 and if		\ test for this interrupt
    _RBR1 c@ rx-char1 !			\ read character (resets interrupt)
    rx-avail1 on
  endif
  _irqsr @ IRQ_mask_U2 and if		\ test for this interrupt
    _RBR2 c@ rx-char2 !			\ read character (resets interrupt)
    rx-avail2 on
  endif
;
' serial-isr add-irq	\ add this ISR to the IRQ list


: serkey?0	\ -- t/f
  rx-avail0 @
;

: serkey?1	\ -- t/f
  rx-avail1 @
;

: serkey?2	\ -- t/f
  rx-avail2 @
;

: serkey0	\ -- char
  begin
    [I  rx-avail0 @ 0=
   while
    I]
[ tasking? ] [if] pause [then]
  repeat
  rx-char0 @
  rx-avail0 off
  I]
;

: serkey1	\ -- char
  begin
    [I  rx-avail1 @ 0=
   while
    I]
[ tasking? ] [if] pause [then]
  repeat
  rx-char1 @
  rx-avail1 off
  I]
;

: serkey2	\ -- char
  begin
    [I  rx-avail2 @ 0=
   while
    I]
[ tasking? ] [if] pause [then]
  repeat
  rx-char2 @
  rx-avail2 off
  I]
;

: (seremit)	\ char base --
  begin
    dup _LSR_offset + @ %00100000 and
  until
  _THR_offset + !
;

: serEmit0	\ char --
  _UART0 (serEmit)
;

: serEmit1	\ char --
  _UART1 (serEmit)
;

: serEmit2	\ char --
  _UART2 (serEmit)
;


\ ***********************
\ Generic i/o assignments
\ ***********************

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (serCR)	\ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D over (seremit)  $0A swap (seremit)
;

: sertype0	\ addr len --
\ *G Type a string to the UART.
  _UART0 (serType)
;

: serTYPE1	\ c-addr len --
\ *G Type a string to the UART.
  _UART1 (serType)
;

: serTYPE2	\ c-addr len --
\ *G Type a string to the UART.
  _UART2 (serType)
;

: serCR0	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART0 (serCR)
;

: serCR1	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART1 (serCR)
;

: serCR2	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART2 (serCR)
;


cdata
create Console0	\ -- addr ; OUT managed by upper driver
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

create Console1	\ -- addr ; OUT managed by upper driver
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

create Console2	\ -- addr ; OUT managed by upper driver
  ' serkey2 ,		\ -- char ; receive char
  ' serkey?2 ,		\ -- flag ; check receive char
  ' seremit2 ,		\ -- char ; display char
  ' sertype2 ,		\ caddr len -- ; display string
  ' sercr2 ,		\ -- ; display new line

console-port 0 = [if]
  Console0 constant Console
[then]

console-port 1 = [if]
  Console1 constant Console
[then]

console-port 2 = [if]
  Console2 constant Console
[then]

decimal

