\ LH77790B queued interrupt RX, polled TX, serial communication drivers

((
Copyright (c) 2000, 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE

20021009 SFP002 Tightened queue locking
20020918 SFP001 First release
))

only forth definitions


\ ***********************************
\ *S Serial interrupt service routine
\ ***********************************

#256 cqueue: SerInpQ0	\ -- addr ; circular byte queue
#256 cqueue: SerInpQ1	\ -- addr ; circular byte queue
#256 cqueue: SerInpQ2	\ -- addr ; circular byte queue

: >RxQ	\ char queue --
\ *G Put character from UART into queue.
  dup (cqFull?) if			\ if queue full
    2drop				\  discard character
  else
    (>cqueue)				\  queue character
  then
;

: serial-isr	\ --
\ *G ISR added to IRQ chain. SERIAL-ISR handles all three serial
\ ** channels.
  _irqsr @ IRQ_mask_U0 and if		\ test for this interrupt
    case  _IIR0 c@
      0 of  _MSR0 c@ drop  endof	\ Modem Status interrupt
      2 of  endof			\ Tx interrupt
      4 of  _RBR0 c@ SerInpQ0 >RxQ  endof	\ read character (resets interrupt)
      6 of  _LSR0 c@ drop  endof	\ Line Status interrupt
    endcase
  endif
  _irqsr @ IRQ_mask_U1 and if		\ test for this interrupt
    case  _IIR1 c@
      0 of  _MSR1 c@ drop  endof	\ Modem Status interrupt
      2 of  endof			\ Tx interrupt
      4 of  _RBR1 c@ SerInpQ1 >RxQ  endof	\ read character (resets interrupt)
      6 of  _LSR1 c@ drop  endof	\ Line Status interrupt
    endcase
  endif
  _irqsr @ IRQ_mask_U2 and if		\ test for this interrupt
    case  _IIR2 c@
      0 of  _MSR2 c@ drop  endof	\ Modem Status interrupt
      2 of  endof			\ Tx interrupt
      4 of  _RBR2 c@ SerInpQ2 >RxQ  endof	\ read character (resets interrupt)
      6 of  _LSR2 c@ drop  endof	\ Line Status interrupt
    endcase
  endif
;
' serial-isr add-irq	\ add this ISR to the IRQ list


\ *****************
\ *S Initialisation
\ *****************

interpreter
: gen-baud-rate	\ clk-rate bps-rate -- 16bit-divisor
\ *G Takes the system clock rate (in HZ) and the required serial
\ ** comms rate (in BPS) and returns a system-dependent number of parameters that
\ ** should be sufficent to effect the baud rate selection by initialising the
\ ** correct hardware.
  0 swap um/mod nip			\ clk/bps
  #16 /mod swap 8 >=			\ clk/bps/16, check for rounding
  if  1+  endif
;
target

system-speed console0-speed gen-baud-rate
  equ serial_divisor0
system-speed console1-speed gen-baud-rate
  equ serial_divisor1
system-speed console2-speed gen-baud-rate
  equ serial_divisor2

l: uart0_tab	\ -- addr ; initialisation table
  _U0CCR ,		_XCLK/1 ,		\ baud rate prescaler
  _LCR0 ,		$07 $80 or ,        	\ divisor access via DLAB
  serial_divisor0 dup				\ set divisor
  _DLM0 ,		8 rshift ,		\ high
  _DLL0 ,		$FF and ,		\ low
  _LCR0 ,		$07 ,			\ disable DLAB access
  _IER0 ,		$01 ,			\ enable receive interrupt
  0 ,

l: uart1_tab	\ -- addr ; initialisation table
  _U1CCR ,		_XCLK/1 ,		\ baud rate prescaler
  _LCR1 ,		$07 $80 or ,        	\ divisor access via DLAB
  serial_divisor1 dup				\ set divisor
  _DLM1 ,		8 rshift ,		\ high
  _DLL1 ,		$FF and ,		\ low
  _LCR1 ,		$07 ,			\ disable DLAB access
  _IER1 ,		$01 ,			\ enable receive interrupt
  0 ,

l: uart2_tab	\ -- addr ; initialisation table
  _SIR_CTLR ,		0 ,			\ Disable IrDA
  _U2CCR ,		_XCLK/1 ,		\ baud rate prescaler
  _LCR2 ,		$07 $80 or ,        	\ divisor access via DLAB
  serial_divisor2 dup				\ set divisor
  _DLM2 ,		8 rshift ,		\ high
  _DLL2 ,		$FF and ,		\ low
  _LCR2 ,		$07 ,			\ disable DLAB access
  _IER2 ,		$01 ,			\ enable receive interrupt
  0 ,

: init-ser	\ --
\ *G Initialise all UARTs.
  di
\ UART0
  SerInpQ0 init-cqueue			\ initialise character queues
  PCSR_mask_U0 _PCSR bic!		\ select internal clock source
  uart0_tab init-io			\ use table
  IRQ_mask_U0 _fiqer bic!		\ disable UART on FIQ
  IRQ_mask_U0 _irqer or!		\ enable UART on IRQ
\ UART1
  SerInpQ1 init-cqueue			\ initialise character queues
  PCSR_mask_U1 _PCSR bic!		\ select internal clock source
  uart1_tab init-io			\ use table
  IRQ_mask_U1 _fiqer bic!		\ disable UART on FIQ
  IRQ_mask_U1 _irqer or!		\ enable UART on IRQ
\ UART2
  SerInpQ2 init-cqueue			\ initialise character queues
  PCSR_mask_U2 _PCSR bic!		\ select internal clock source
  uart2_tab init-io			\ use table
  IRQ_mask_U2 _fiqer bic!		\ disable UART1 on FIQ
  IRQ_mask_U2 _irqer or!		\ enable UART1 on IRQ
  ei
;


\ ********************
\ *S Serial primitives
\ ********************

: serkey?0	\ -- t/f
  SerInpQ0 cqnotempty?
;

: serkey?1	\ -- t/f
  SerInpQ1 cqnotempty?
;

: serkey?2	\ -- t/f
  SerInpQ2 cqnotempty?
;

: serkey0	\ -- char
  SerInpQ0 cqueue>
;

: serkey1	\ -- char
  SerInpQ1 cqueue>
;

: serkey2	\ -- char
  SerInpQ2 cqueue>
;

: (seremit)	\ char base --
  begin
    dup _LSR_offset + @ %00100000 and
  until
  _THR_offset + c!
;

: serEmit0	\ char --
  _UART0 (serEmit)
;

: serEmit1	\ char --
  _UART1 (serEmit)
;

: serEmit2	\ char --
  _UART2 (serEmit)
;

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (serCR)	\ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D over (seremit)  $0A swap (seremit)
;

: sertype0	\ addr len --
\ *G Type a string to the UART.
  _UART0 (serType)
;

: serTYPE1	\ c-addr len --
\ *G Type a string to the UART.
  _UART1 (serType)
;

: serTYPE2	\ c-addr len --
\ *G Type a string to the UART.
  _UART2 (serType)
;

: serCR0	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART0 (serCR)
;

: serCR1	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART1 (serCR)
;

: serCR2	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART2 (serCR)
;


\ ***********************
\ Generic i/o assignments
\ ***********************

cdata
create Console0	\ -- addr ; OUT managed by upper driver
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

create Console1	\ -- addr ; OUT managed by upper driver
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

create Console2	\ -- addr ; OUT managed by upper driver
  ' serkey2 ,		\ -- char ; receive char
  ' serkey?2 ,		\ -- flag ; check receive char
  ' seremit2 ,		\ -- char ; display char
  ' sertype2 ,		\ caddr len -- ; display string
  ' sercr2 ,		\ -- ; display new line

console-port 0 = [if]
  Console0 constant Console
[then]

console-port 1 = [if]
  Console1 constant Console
[then]

console-port 2 = [if]
  Console2 constant Console
[then]

decimal

\ : tt begin  ."   ok" cr  key? until key drop ;

