\ LPC210x queued interrupt RX, polled TX, serial communication drivers

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20040614 MPE003 Fixed SETBAUD.
20040427 MPE002 Added selection of which UARTs are used.
20040209 MPE001 First release
))

only forth definitions  decimal


\ =========
\ *! serlpc210xi
\ =========
\ *T Serial driver
\ *P By default the serial drivers are initialised to 115200
\ ** baud for UART0 which is the serial console, and UART1
\ ** is initialised to 38400 baud. These baud rates can be
\ ** changed by setting the relevant UART DLL and DLM registers.
\ ** See the chip user manual for more details.

\ *S Configuration
\ *P The following equates control which UARTs are required
\ ** and initialised. If the equates are undefined, they are
\ ** generated and set to 1.

[undefined] useUART0? [if]
1 equ useUART0?	\ -- n
\ *G Compile UART0 code if non-zero.
[then]

[undefined] useUART1? [if]
1 equ useUART1?	\ -- n
\ *G Compile UART1 code if non-zero.
[then]


\ ***********************************
\ *S Serial interrupt service routines
\ ***********************************

internal

useUART0? [if]
#256 cqueue: SerInpQ0	\ -- addr ; circular byte queue
[then]
useUART1? [if]
#128 cqueue: SerInpQ1	\ -- addr ; circular byte queue
[then]

: >RxQ		\ char queue --
\ *G Put character from UART into queue.
\ ** INTERNAL.
  dup (cqFull?)				\ if queue full
  if  2drop  exit  endif		\  discard character
  (>cqueue)				\  queue character
;

: FIFO>RxQ      \ base queue --
\ *G Given a UART base address and a character queue,
\ ** empty the UART FIFO into the queue.
\ ** INTERNAL.
  begin
    over UnLSR + @ $01 and
   while
    over UnRBR + @ $FF and over >RxQ
  repeat
  2drop
;

useUART0? [if]
: ser0-isrh	\ --
\ *G UART0 high level ISR.
\ ** INTERNAL.
  case  _UART0 dup UnIIR + @ 7 and
\   0 of  UnMSR + @ drop  endof		\ No MSR for UART0
    2 of  endof				\ Tx interrupt
    4 of  SerInpQ0 FIFO>RxQ  endof	\ read characters (resets interrupt)
    6 of  UnLSR + @ drop  endof		\ Line Status interrupt
          nip
  endcase
;
' ser0-isrh IRQ: ser0-isr
\ *G UART0 interrupt service routine entry point.
[then]

useUART1? [if]
: ser1-isrh	\ --
\ *G UART1 high level ISR.
\ ** INTERNAL.
  case  _UART1 dup UnIIR + @ 7 and
    0 of  U1MSR + @ drop  endof		\ Modem Status interrupt
    2 of  endof				\ Tx interrupt
    4 of  SerInpQ1 FIFO>RxQ  endof	\ read characters (resets interrupt)
    6 of  UnLSR + @ drop  endof		\ Line Status interrupt
          nip
  endcase
;
' ser1-isrh IRQ: ser1-isr
\ *G UART1 interrupt service routine entry point.
[then]

external


\ *****************
\ *S Initialisation
\ *****************

interpreter
: gen-baud-rate	\ bps-rate -- divisor16
\ *G Takes the required serial comms rate (in BPS) and returns
\ ** a system-dependent number of parameters that should be
\ ** sufficent to effect the baud rate selection by initialising
\ ** the hardware.
  system-speed /vpb / swap / #16 /mod
  swap 8 >=
  if  1+  endif
;
target

useUART0? [if]
console0-speed gen-baud-rate
  equ u0-divisor

l: uart0_tab	\ -- addr ; initialisation table
  _U0FCR ,		$81 ,			\ FIFO enable (8 char delay)
  _U0LCR ,		$03 $80 or ,        	\ divisor access via DLAB
  u0-divisor dup				\ set divisor
  _U0DLM ,		8 rshift $FF and ,	\ high
  _U0DLL ,		$FF and ,		\ low
  _U0LCR ,		$03 ,			\ disable DLAB access
  _U0IER ,		$01 ,			\ enable receive interrupt
  0 ,
[then]

useUART1? [if]
console1-speed gen-baud-rate
  equ u1-divisor

l: uart1_tab	\ -- addr ; initialisation table
  _U1FCR ,		$81 ,			\ FIFO enable (8 char delay)
  _U1LCR ,		$03 $80 or ,        	\ divisor access via DLAB
  u1-divisor dup				\ set divisor
  _U1DLM ,		8 rshift $FF and ,	\ high
  _U1DLL ,		$FF and ,		\ low
  _U1LCR ,		$03 ,			\ disable DLAB access
  _U1IER ,		$01 ,			\ enable receive interrupt
  0 ,
[then]

: init-ser	\ --
\ *G Initialise both UARTs.
  di

\ UART0
[ useUART0? ] [if]
  SerInpQ0 init-cqueue			\ initialise character queues
  bit3 _PCONP or!			\ enable clock
  _PINSEL0 dup @			\ enable pins
    $0F invert and  $5 or
  swap !
  uart0_tab init-io			\ use table
  ser0-isr U0Int# #14 SetIrqIsr		\ set up interrupt
[then]

\ UART1
[ useUART1? ] [if]
  SerInpQ1 init-cqueue			\ initialise character queues
  bit4 _PCONP or!			\ enable clock
  _PINSEL0 dup @			\ enable pins
    $000F0000 invert and  $00050000 or
  swap !
  uart1_tab init-io			\ use table
  ser1-isr U1Int# #13 SetIrqIsr		\ set up interrupt
[then]

  ei
;


\ ********************
\ *S Serial primitives
\ ********************

0 value FaultConsole?	\ -- flag
\ *G When set non-zero, transmit is in polled mode. This is
\ ** required when fault exceptions provide debug information.

: +FaultConsole	\ --
\ *G For use in fault exception handlers, the multi-tasker must
\ ** be turned off and *\fo{EMIT) and friends must run in polled
\ ** mode.
  -1 to FaultConsole?
  [ tasking? ] [if]  single  [then]
;

((
: -FaultConsole	\ --
\ *G Undoes the action of *\fo{+FaultConsole}. This word is only
\ ** needed if SWIs are used and perform serial I/O. See *\i{IntARM4/5.fth}.
  0 to FaultConsole?
  [ tasking? ] [if]  multi  [then]
;
))

internal

: (seremit)	\ char base --
\ *G Send a character on the UART whose base address is given.
\ ** INTERNAL
  begin
    dup UnLSR + @ %00100000 and
  until
  UnTHR + !
;

: (serCR)	\ base --
\ *G Send a CR/LF pair on the UART whose base address is given.
\ ** INTERNAL
  $0D over (seremit)  $0A swap (seremit)
;

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
\ ** INTERNAL
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

internal

useUART0? [if]

: serEmit0	\ char --
\ *G Transmit a character on UART0.
  _UART0 (serEmit)
;

: serkey?0	\ -- t/f
\ *G Return true if UART0 has a character available.
  SerInpQ0 cqnotempty?
;

: serkey0	\ -- char
\ Wait for a character on UART0.
  SerInpQ0 cqueue>
;

: sertype0	\ addr len --
\ *G Type a string to the UART.
  _UART0 (serType)
;

: serCR0	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART0 (serCR)
;

[then]

useUART1? [if]

: serkey?1	\ -- t/f
\ *G Return true if UART1 has a character available.
  SerInpQ1 cqnotempty?
;

: serkey1	\ -- char
\ Wait for a character on UART1.
  SerInpQ1 cqueue>
;

: serEmit1	\ char --
\ *G Transmit a character on UART1.
  _UART1 (serEmit)
;

: serTYPE1	\ c-addr len --
\ *G Type a string to the UART.
  _UART1 (serType)
;

: serCR1	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART1 (serCR)
;

[then]

external


\ **************************
\ *S Generic i/o assignments
\ **************************

cdata
useUART0? [if]
create Console0	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for UART0.
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line
[then]

useUART1? [if]
create Console1	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for UART1.
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line
[then]

console-port 0 = [if]
  Console0 constant Console
\ *G *\fo{CONSOLE} is the device used by the Forth system for
\ ** interaction. It may be changed by a phrase of the form:
\ *C   <device>  dup opvec !  ipvec !
[then]

console-port 1 = [if]
  Console1 constant Console
[then]


\ ****************************
\ *S Forth Stamp specific code
\ ****************************
\ *P This code is only compiled if the equate *\fo{ArmStamp?} is
\ ** set true.

[undefined] ArmStamp? [if]  0 equ ArmStamp?  [then]
ArmStamp? [if]

_UART0 constant UART0	\ -- addr
\ *G Base address of the UART0 peripheral.
_UART1 constant UART1	\ -- addr
\ *G Base address of the UART1 peripheral.

: SetBaud	\ bps-rate uart --
\ *G Takes the required serial comms rate (in BPS) and the UART
\ ** base and sets the baud rate.
  swap system-speed /vpb / swap / #16 /mod
  swap 8 >=
  if  1+  endif
  dup 8 rshift $FF and  swap $FF and swap	\ -- uart divlo divhi
  [i
    rot						\ -- divlo divhi uart
    $83 over UnLCR + !				\ enable DLAB access
    tuck UnDLM + !				\ -- divlo uart ; high divisor ; MPE003
    tuck UnDLL + !				\ -- uart ; MPE003
    $03 swap UnLCR + !
  i]
;

[then]

