\ LPC210x queued interrupt RX, polled TX, serial communication drivers
\ adapted to OXUF922 '950 onchip uart ### mgs 26/04/04

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20040209 MPE001 First release
))

only forth definitions  decimal


\ =========
\ *! serlpc210xi
\ =========
\ *T Serial driver
\ *P By default the serial drivers are initialised to 115200
\ ** baud for UART0 which is the serial console, and UART1
\ ** is initialised to 38400 baud. These baud rates can be
\ ** changed by setting the relevant UART DLL and DLM registers.
\ ** See the chip user manual for more details.


\ ***********************************
\ *S Serial interrupt service routines
\ ***********************************

internal

#64 cqueue: SerInpQ0	\ -- addr ; circular byte queue
\ #128 cqueue: SerInpQ1	\ -- addr ; circular byte queue

: >RxQ		\ char queue --
\ *G Put character from UART into queue.
\ ** INTERNAL.
  dup (cqFull?)				\ if queue full
  if  2drop  exit  endif		\  discard character
  (>cqueue)				\  queue character
;

: FIFO>RxQ      \ base queue --
\ *G Given a UART base address and a character queue,
\ ** empty the UART FIFO into the queue.
\ ** INTERNAL.
  begin
    over UnLSR + c@ $01 and	   		\ byte access on 922 ###
   while
    over UnRBR + c@  over >RxQ
  repeat
  2drop
;

: ser0-isrh	\ --
\ *G UART0 high level ISR.
\ ** INTERNAL.
led4 _led_base c!
  case  _UART0 dup UnIIR + c@ 7 and
    0 of  UnMSR + c@ drop  endof		\ MSR for UART0
    2 of  endof				\ Tx interrupt
    4 of  SerInpQ0 FIFO>RxQ  endof	\ read characters (resets interrupt)
    6 of  UnLSR + c@ drop  endof		\ Line Status interrupt
          nip
  endcase 
led4 invert _led_base c!
;
' ser0-isrh add-irq			\ IRQ: ser0-isr
\ *G UART0 interrupt service routine entry point.


external


\ *****************
\ *S Initialisation
\ *****************

interpreter
: gen-baud-rate	\ bps-rate -- divisor16
\ *G Takes the required serial comms rate (in BPS) and returns
\ ** a system-dependent number of parameters that should be
\ ** sufficent to effect the baud rate selection by initialising
\ ** the hardware.
  system-speed  swap 16 * /			\ '650' compatible after reset
;
target

console0-speed gen-baud-rate
  equ u0-divisor

bit9 equ u0intmask
((
: init-ser0	\ --
\ *G Initialise serial port 0
  1                   _U0FCR c!
  $03 $80 or          _U0LCR c!		\ 8 bit, 1 stop bit, no parity, enable DLAB
  u0-divisor 8 rshift _U0DLM c!
  u0-divisor $FF and  _U0DLL c!
  $03                 _U0LCR c!
;
))

: inituart0 ( --)					\ regs are byte accessed on ox950
	$81 _u0fcr c!
	$03 $80 or _u0lcr c!
	u0-divisor dup 8 rshift $FF and _u0dlm c!
	$FF and _u0dll c!
	$03 _u0lcr c!
	$01 _u0ier c!
	u0intmask RPS_IRQ_ENABLE !		\ int controller
;

: init-ser	\ --
\ *G Initialise both UARTs.
  di
  led2 _led_base c!				\ diags
\ UART0
  SerInpQ0 init-cqueue			\ initialise character queues
  inituart0			
  led3 _led_base c!
  [char] X _u0thr c!
 \ [char] B _u0thr c!
  ei
;


\ ********************
\ *S Serial primitives
\ ********************

: serkey?0	\ -- t/f
\ *G Return true if UART0 has a character available.
  SerInpQ0 cqnotempty?
;

: serkey0	\ -- char
\ Wait for a character on UART0.
  SerInpQ0 cqueue>
;

: (seremit)	\ char base --
  begin
    dup UnLSR + c@ %00100000 and
  until
  UnTHR + c!
;

: serEmit0	\ char --
\ *G Transmit a character on UART0.
  _UART0 (serEmit)
;

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: sertype0	\ addr len --
\ *G Type a string to the UART.
  _UART0 (serType)
;

: (serCR)	\ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D over (seremit)  $0A swap (seremit)
;

: serCR0	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART0 (serCR)
;


\ **************************
\ *S Generic i/o assignments
\ **************************

cdata
create Console0	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for UART0.
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

console-port 0 = [if]
  Console0 constant Console
\ *G CONSOLE is the device used by the Forth system for interaction.
\ ** It may be changed by a phrase of the form:
\ *C   <device>  dup opvec !  ipvec !
[then]


