\ Atmel AT91 40807 ARM Ticker interrupt handlers

((
Copyright (c) 2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

Ticker uses channel 0 of the TC unit.

period=(prescale*RC)/clock

RC = (period * clock)/prescale

Note that timer interrupts are cleared by reading the
TC_SR register.
))


\ =========
\ *! tick40x
\ =========
\ *T Ticker Interrupt for Atmel AT9140xxx
\ *P The ticker is the basis of the TIMEBASE system. In order
\ ** start the ticker the START-CLOCK word is used. This
\ ** initialises the TC0 timer counter channel to produce
\ ** a periodic interrupt which updates the value TICKS.
\ ** The period is set by the equate TICK-MS in milliseconds,
\ ** and the value of TICKS is updated by TICK-MS at every
\ ** interrupt.

\ *P Other files named TICKxxx in the DRIVERS folder provide
\ ** the same functionality for other CPUs.

\ *S Glossary


\ Select one prescaler line
decimal
\    2 equ prescale0	0 equ tcclks0
\    8 equ prescale0	1 equ tcclks0
\   32 equ prescale0	2 equ tcclks0
\  128 equ prescale0	3 equ tcclks0
  1024 equ prescale0	4 equ tcclks0
\ *G The initial division ratio of the TC0 prescaler, and the
\ ** value required in the TCCLKS field.

interpreter
: gen-RC0	\ us clock -- rc
\ *G An INTERPRETER definition that takes a period in microseconds
\ ** and the clock speed in herz to produce the required value to be
\ ** loaded in the RC register of TC0.
  prescale0 #1000000 * */
  dup $0FFFF u> if
      cr ." Divider exceeds 16 bits, increase PRESCALE0 and retry"
    abort
  endif
;
target

tick-ms #1000 *  system-speed gen-RC0 equ #RC0
\ *G The initial value of the RC register calculated from the
\ ** required ticker period TICK-MS in milliseconds and the
\ ** system clock speed SYSTEM-SPEED in herz.


\ ************************
\ Ticker interrupt handler
\ ************************

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- u ; returns the ticker value
\ *G Return current clock value in milliseconds. This value
\ ** can treated as a 32 bit unsigned value that will wrap
\ ** when it overflows.
[then]

: ticker-isr	\ -- ; high level interrupt handler
\ *G The high level interrupt handler for
  _TC0 TC_SR + @ drop			\ read SR to clear int
  ticks tick-ms + to ticks		\ update ticker by period in ms
[ timebase? ] [if]
  do-timers
[then]
;

l: ^AIC
  _AIC ,

proc TC0ISR	\ -- ; TC0 interrupt handler, R13 already set up
\ *G The low level interrupt handler for TC0 assumes uses the same
\ ** template as all the other AT91 interrupt handlers, assuming
\ ** that the interrupt R13 (RSP) has already been set up and that
\ ** registers R0..12 may be destroyed. New values of R11 (UP) and
\ ** R12 (PSP) are calculated.
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  bl	' ticker-isr

  ldr	r4, ^AIC			\ issue end of interrupt to AIC
  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, r0
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

0 value ticking?
\ *G True when the clock interrupt is running.

: start-clock	\ -- ; start clock interrupt
\ *G Start the ticker periodic interrupt.
  di
  $00000010 _PS_PCER !			\ Enable clock to TC0
  $00000010 _AIC AIC_IDCR + !		\ Disable TC0 interrrupts
  $00000010 _AIC AIC_ICCR + !		\ Clear pending TC0 interrrupts
  $00000001 _AIC AIC_SMR0 + 4 cells + !	\ Set TC0 interrupt to priority 1 (lowest)
  TC0ISR    _AIC AIC_SVR0 + 4 cells + !	\ Store address of interrupt handler routine
  $0000C000 tcclks0 or			\ WAVE | CPCTRG | TCCLKS
            _TC0 TC_CMR + !
  #RC0      _TC0 TC_RC + !		\ set initial period
  $000000FF _TC0 TC_IDR + !		\ disable all TC0 interrupts
  $00000010 _TC0 TC_IER + !		\ enable interrupt on RC compare
  $00000005 _TC0 TC_CCR + !		\ enable SWTRG and CLKEN
  $00000010 _AIC AIC_IECR + !		\ Enable the TC0 interrupt
  -1 to ticking?
  ei
;

: stop-clock	\ -- ; stop clock interrupt
\ *G Stop the ticker periodic interrupt.
  [I
  $00000002 _TC0 TC_CCR + !		\ set CLKDIS
  $00000010 _AIC AIC_IDCR + !		\ Disable TC0 interrrupts
  0 to ticking?
  I]
;


\ ********************
\ Time base extensions
\ ********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  stop-clock init-timers start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

