\ Samsung S3C4510 ARM Ticker interrupt handlers

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

Ticker uses channel 0 of the TC unit.

period=(prescale*RC)/clock

RC = (period * clock)/prescale

Note that timer interrupts are cleared by reading the
TC_SR register.
))

decimal


\ ===========
\ *! tick4510
\ ===========
\ *T Ticker Interrupt for Samsung S3C4510
\ *P The ticker is the basis of the TIMEBASE system. In order
\ ** start the ticker the START-CLOCK word is used. This
\ ** initialises the TC0 timer counter channel to produce
\ ** a periodic interrupt which updates the value TICKS.
\ ** The period is set by the equate TICK-MS in milliseconds,
\ ** and the value of TICKS is updated by TICK-MS at every
\ ** interrupt.

\ *P Other files named TICKxxx in the DRIVERS folder provide
\ ** the same functionality for other CPUs.

\ *P The timer period is calculaled from:
\ *E   period = clktime*(cntx+1) = (cntx+1)/fmclk
\ **
\ **   cntx = (period*fmclk)-1

\ *S Glossary


interpreter
: gen-cntx	\ system-speed us -- cntx
\ *G An INTERPRETER definition that takes the clock spped and a
\ ** period in microseconds to produce the required value to be
\ ** set in the TDATAx register of the timer.
  #1000000 */mod
  swap #499999 u>
  if  1+  endif
  1-
;
target

tick-ms #1000 *  system-speed gen-cntx equ cnt-t0
\ *G The initial value of the TDATAx register calculated from the
\ ** required ticker period TICK-MS in milliseconds and the
\ ** system clock speed SYSTEM-SPEED in herz.


\ ************************
\ Ticker interrupt handler
\ ************************

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- u ; returns the ticker value
\ *G Return current clock value in milliseconds. This value
\ ** can treated as a 32 bit unsigned value that will wrap
\ ** when it overflows.
[then]

: timer0-isr	\ -- ; high level interrupt handler
\ *G The high level interrupt handler for timer 0
 tick-ms addr ticks +!			\ update ticker by period in ms
[ timebase? ] [if]
  do-timers
[then]
;

0 value ticking?
\ *G True when the clock interrupt is running.

: start-clock	\ -- ; start clock interrupt
\ *G Start the ticker periodic interrupt.
  di
  ['] Timer0-isr nTimer0_int SetIRQ	\ Set as IRQ interrupt
  _timers
  $07 over tmod + bic!			\ disable timer 0, select interval mode
  cnt-t0 over 2dup TDATA0 + ! TCNT0 + !	\ set timer count registers
  $01 swap tmod + or!			\ enable in interval mode
  nTimer0_int EnInt			\ Enable the interrupt
  nGlobal_int EnInt
  -1 to ticking?
  ei
;

: stop-clock	\ -- ; stop clock interrupt
\ *G Stop the ticker periodic interrupt.
  [I
  $01 _timers tmod + bic!		\ disable timer 0
  nTimer0_int DisInt
  0 to ticking?
  I]
;


\ ********************
\ Time base extensions
\ ********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  stop-clock  init-timers  start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

