\ LH77790 ARM Ticker interrupt handlers

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


This ticker is based on using the CTC channel 0 in mode 2, generating
an interrupt every TICK-MS milliseconds. It can be adapted for the
TIMEBASE system by adding DO-TIMERS to the TICKER-ISR.

period=divisor/clock

divisor = period * clock
))

#16 equ ctc_prescale	\ MUST be binary prescaler

interpreter
: gen-CTC	\ us clock -- ctc_lo ctc_hi
  0 ctc_prescale um/mod nip		\ divide by prescaler
  um*  #1000000 um/mod nip		\ divisor = period * clock / 1000000
  dup $0FFFF u> if
    cr ." CTC divider exceeds 16 bits, increase CTC_PRESCALE and retry"
    abort
  endif
  0 $0100 um/mod
;
target

tick-ms #1000 *  system-speed gen-CTC
  equ #ctc0div.hi
  equ #ctc0div.lo

[undefined] ticks [if]
0 value ticks	\ -- ; returns the ticker value
\ *G Get current clock value in milliseconds.
\ ** TICKS may have been defined in TIMEBASE.FTH or DELAYS.FTH.
[then]

: ticker-isr	\ --
  _irqsr @ IRQ_mask_CTC0 and if		\ test for this interrupt
    IRQ_mask_CTC0 _iclr !		\ reset interrupt
    tick-ms addr ticks +!		\ update tick count
[ timebase? ] [if]
    do-timers
[then]
  endif
;
' ticker-isr add-irq			\ add it to the IRQ chain

0 value ticking?	\ -- n ; nonzero when ticker running

: start-clock	\ -- ; start clock interrupt
  di
  bit3 _pcsr bic!			\ clear bit 3 for internal clock
  ctc_prescale _ct0ccr c!		\ set clock prescaler
  $034 _ct_cwr c!			\ select ch0, mode 2, write LSB/MSB
  #ctc0div.lo _ct_cntr0 c!		\ low byte
  #ctc0div.hi _ct_cntr0 c!		\ high byte
  _iocr @ $0600 or _iocr !		\ enable gate

  _icr1 @ $0001 or _icr1 !		\ CTC0 edge triggered, low to high
  _fiqer @ $FFBF and _fiqer !		\ disable CTC0 on FIQ
  _irqer @ $0040 or _irqer !		\ enable CTC0 on IRQ
  ei					\ enable interrupts
  -1 to ticking?
;

: stop-clock	\ -- ; stop clock interrupt
  [i
  _iocr @ $F9FF and $0400 or _iocr !	\ disable gate, bits 10:9 = 10
  _irqer @ $FFBF and _irqer !		\ disable CTC0 on IRQ
  0 to ticking?
  i]
;

timebase? [if]
: start-timers  \ -- ; Start internal time clock
  stop-clock init-timers start-clock
;

: stop-Timers   \ -- ; disable timer system
  stop-clock
;
[else]
((
: start-timers  \ -- ; Start internal time clock
  start-clock
;

: stop-Timers   \ -- ; disable timer system
  stop-clock
;

: later         \ n -- n'
( *G Generates the timebase value for termination in n milliseconds time. )
  ticks +
;

: expired       \ n -- flag ; true if timed out
( *G Flag is returned true if the timebase value n has timed out. Calls PAUSE. )
  pause  ticks - 0<			\ must use circular arithmetic
;

: timedout?     \ n -- flag ; true if timed out
( *G Flag is returned true if the timebase value n has timed out. Does not )
( ** call PAUSE, so can be used in interrupts, winprocs and callbacks. In particular, )
( ** TIMEDOUT? should be used rather than EXPIRED inside timer action words )
( ** to reduce timer jitter. )
  ticks - 0<                            \ must use circular arithmetic
;
optimised? [if]
compiler
: later         \ n -- n'
  ticks +
;

: timedout?     \ n -- flag ; true if timed out
  ticks - 0<                            \ must use circular arithmetic
;
target
[then]

[undefined] ms [if]
: ms            \ n --
( *G Waits for n milliseconds. Uses PAUSE. )
  later
  begin
    dup expired
  until
  drop
;
[then]
))
[then]
