\ Atmel AT91RM9200 Ticker interrupt handlers

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

Ticker uses channel 0 of the TC unit.

period=(prescale*RC)/clock

RC = (period * clock)/prescale

Note that timer interrupts are cleared by reading the
TC_SR register.
))


\ ===========
\ *! tick9200
\ *T Ticker Interrupt for Atmel AT91RM9200
\ ===========
\ *P The ticker is the basis of the TIMEBASE system. To start
\ ** the ticker use *\fo{START-CLOCK}, which initialises the TC0
\ ** timer counter channel to produce a periodic interrupt which
\ ** updates the value *\fo{TICKS}. The period is set by the
\ ** equate *\fo{TICK-MS} in milliseconds, and the value of 
\ ** *\fo{TICKS} is updated by *\fo{TICK-MS} at every interrupt.

\ *P Other files named *\i{TICKxxx.fth} in the *\i{DRIVERS} folder
\ ** provide the same functionality for other CPUs.

(( clock selection
0	TIMER_CLOCK1	MCK/2
1	TIMER_CLOCK2	MCK/8
2	TIMER_CLOCK3	MCK/32
3	TIMER_CLOCK4	MCK/128
4	TIMER_CLOCK5	SCLK
5	XC0
6	XC1
7	XC2
))

\ Select one prescaler line
decimal
\    2 equ prescale0	0 equ tcclks0
\    8 equ prescale0	1 equ tcclks0
   32 equ prescale0	2 equ tcclks0
\  128 equ prescale0	3 equ tcclks0
\  1024 equ prescale0	4 equ tcclks0
\ *G The initial division ratio of the TC0 prescaler, and the
\ ** value required in the TCCLKS field.

interpreter
: gen-RC0	\ us clock -- rc
\ *G An INTERPRETER definition that takes a period in microseconds
\ ** and the clock speed in herz to produce the required value to be
\ ** loaded in the RC register of TC0.
  prescale0 #1000000 * */
  dup $0FFFF u> if
    cr ." Divider " . ." exceeds 16 bits, increase PRESCALE0 and retry"
    abort
  endif
;
target

tick-ms #1000 *  /MCK gen-RC0 equ #RC0
\ *G The initial value of the RC register calculated from the
\ ** required ticker period *\fo{TICK-MS} in milliseconds and the
\ ** system clock speed *\fo{SYSTEM-SPEED} in Herz.


\ ************************
\ Ticker interrupt handler
\ ************************

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- u ; returns the ticker value
\ *G Return current clock value in milliseconds. This value
\ ** can be treated as a 32 bit unsigned value that will wrap
\ ** when it overflows.
[then]

: ticker-isr	\ -- ; high level interrupt handler
\ *G The high level interrupt handler for the ticker.
  _TCB0 TC0 + TC_SR + @ drop		\ read SR to clear int
  ticks tick-ms + to ticks		\ update ticker by period in ms
[ timebase? ] [if]
  do-timers
[then]
;

' ticker-isr IRQ: TCB0_0isr

0 value ticking?
\ *G True when the clock interrupt is running.

: start-clock	\ -- ; start clock interrupt
\ *G Start the ticker periodic interrupt.
  di
  TC0mask _PMC PMC_PCER + !		\ Enable clock to TC0
  TC0mask _AIC AIC_IDCR + !		\ Disable TC0 interrrupts
  TC0mask _AIC AIC_ICCR + !		\ Clear pending TC0 interrrupts
  $00000001 _AIC AIC_SMR0 + ID_TC0 cells + !	\ Set TC0 interrupt to priority 1 (lowest)
  TCB0_0isr _AIC AIC_SVR0 + ID_TC0 cells + !	\ Store address of interrupt handler routine
  $0000C000 tcclks0 or			\ WAVE | CPCTRG | TCCLKS
            _TCB0 TC0 + TC_CMR + !
  #RC0      _TCB0 TC0 + TC_RC + !	\ set initial period
  $000000FF _TCB0 TC0 + TC_IDR + !	\ disable all TC0 interrupts
  $00000010 _TCB0 TC0 + TC_IER + !	\ enable interrupt on RC compare
  $00000005 _TCB0 TC0 + TC_CCR + !	\ enable SWTRG and CLKEN
  TC0mask _AIC AIC_IECR + !		\ Enable the TC0 interrupt
  -1 to ticking?
  ei
;

: stop-clock	\ -- ; stop clock interrupt
\ *G Stop the ticker periodic interrupt.
  [I
  $00000002 _TCB0 TC0 + TC_CCR + !	\ set CLKDIS
  TC0mask _AIC AIC_IDCR + !		\ Disable TC0 interrrupts
  0 to ticking?
  I]
;


\ ********************
\ Time base extensions
\ ********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  init-timers                           \ initialise time base system
  start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

