\ Analog Devices ADuC702x Ticker interrupt handlers

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))

decimal

\ =========
\ *! tickad702x
\ =========
\ *T ADuC702 Ticker Interrupt
\ *P The ticker is the basis of the TIMEBASE system. In order
\ ** start the ticker the START-CLOCK word is used. This
\ ** initialises Timer 0 to produce a periodic interrupt which
\ ** updates the value TICKS.
\ ** The period is set by the equate TICK-MS in milliseconds,
\ ** and the value of TICKS is updated by TICK-MS at every
\ ** interrupt.

\ *P Other files named TICKxxx in the DRIVERS folder provide
\ ** the same functionality for other CPUs.

\ *P This code requires the interrupt facilities in INTARM3.


\ ************************
\ *S Configuration equates
\ ************************

_timer equ _ticker	\ -- addr
\ *G Select the timer/counter unit to use for the ticker

BIT2 equ TickerBit	\ -- mask
\ *G The bit to be set in interrupt enable/disable/status registers.

interpreter
: gen-ticks	\ us clock -- t0ld prescale
\ *G An INTERPRETER definition that takes a period in microseconds
\ ** and the clock speed in herz to produce the required value to be
\ ** loaded in the match register.
  2dup 1 #1000000 * */ dup $10000 <
  if  nip nip  $C0  exit  endif
  drop
  2dup #16 #1000000 * */ dup $10000 <
  if  nip nip  $C4  exit  endif
  drop
  2dup #256 #1000000 * */ dup $10000 <
  if  nip nip  $C8  exit  endif
  drop
  2drop true abort" Cannot find valid T0LD value"
;
target

tick-ms #1000 * system-speed gen-ticks  equ T0Conval  equ T0LdVal
\ *G The initial value of the T0CON and T0LD registers calculated
\ ** from the required ticker period *\fo{TICK-MS} in milliseconds
\ ** and the system clock speed *\fo{SYSTEM-SPEED} in herz.


\ ***************************
\ *S Ticker interrupt handler
\ ***************************

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- u ; returns the ticker value
\ *G Return current clock value in milliseconds. This value
\ ** can treated as a 32 bit unsigned value that will wrap
\ ** when it overflows.
[then]

internal
: ticker-isr	\ -- ; high level interrupt handler
\ *G The high level interrupt handler for the ticker.
  _INTCON IrQSta + @ TickerBit and if
    0 _ticker T0ClrI + !		\ reset T0 interrupt
    ticks tick-ms + to ticks		\ update ticker by period in ms
[ timebase? ] [if]
    do-timers
[then]
  endif
;
external

' ticker-isr add-irq
\ *G Creates the IRQ handler for the ticker.

: start-clock	\ -- ; start clock interrupt
\ *G Start the ticker periodic interrupt.
  di
  _ticker
    T0Ldval over T0Ld + !		\ Counter value
    T0ConVal over T0Con + !
  drop
  TickerBit _INTCON IrqEn + or!		\ enable interrupt
  ei
;
[undefined] Umbilical? [if]
' Start-Clock AtCold
[then]

: stop-clock	\ -- ; stop clock interrupt
\ *G Stop the ticker periodic interrupt.
  TickerBit _INTCON IrqClr + !
  0 _ticker T0Con + !
;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]



\ ***********************
\ *S Time base extensions
\ ***********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  init-timers  start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

