\ Philips LPC210x Ticker interrupt handlers

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))

decimal

\ =========
\ *! ticklpc210x
\ =========
\ *T LPC210x Ticker Interrupt
\ *P The ticker is the basis of the TIMEBASE system. In order
\ ** start the ticker the START-CLOCK word is used. This
\ ** initialises the TC0 timer counter channel to produce
\ ** a periodic interrupt which updates the value TICKS.
\ ** The period is set by the equate TICK-MS in milliseconds,
\ ** and the value of TICKS is updated by TICK-MS at every
\ ** interrupt.

\ *P Other files named TICKxxx in the DRIVERS folder provide
\ ** the same functionality for other CPUs.

\ *P For the LPC210x, the default ticker uses Timer 0, Match
\ ** Register 0 being used to reset the count and generate an
\ ** interrupt. Note that if the PWM outputs are not being used,
\ ** the PWM unit can be used as it includes the same register
\ ** set as Timer 0 and Timer 1.

\ *P This code requires the interrupt facilities in INTARM3.


\ ************************
\ *S Configuration equates
\ ************************

_timer0 equ _ticker	\ -- addr
\ *G Select the timer/counter unit to use for the ticker

BIT1 equ TickerBit	\ -- mask
\ *G The bit to be set in PCONP to power the counter/timer.

t0int# equ TickInt#	\ -- src
\ *G select the source number of the ticker interrupt.

#15 equ TickSlot#	\ -- n
\ *G Select the VIC slot/priority for the ticker interrupt.
\ ** This is usually the lowest priority interrupt in slot 15.

1 equ /preTick	\ -- n
\ *G Select the prescaler division ratio.
/pretick 1- equ init_preDiv
\ *G The initial value of the prescaler divider.

decimal

interpreter
: gen-ticks	\ us clock -- n
\ *G An INTERPRETER definition that takes a period in microseconds
\ ** and the clock speed in herz to produce the required value to be
\ ** loaded in the match register.
  /vpb /				\ pclk speed
  /pretick #1000000 * */
;
target

tick-ms #1000 *  system-speed gen-ticks equ #MRtick
\ *G The initial value of the match register calculated from the
\ ** required ticker period TICK-MS in milliseconds and the
\ ** system clock speed SYSTEM-SPEED in herz.


\ ***************************
\ *S Ticker interrupt handler
\ ***************************

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- u ; returns the ticker value
\ *G Return current clock value in milliseconds. This value
\ ** can treated as a 32 bit unsigned value that will wrap
\ ** when it overflows.
[then]

: ticker-isr	\ -- ; high level interrupt handler
\ *G The high level interrupt handler for the ticker.
  1 _ticker TnIR + !			\ reset MR0 interrupt
  ticks tick-ms + to ticks		\ update ticker by period in ms
[ timebase? ] [if]
  do-timers
[then]
;

' ticker-isr IRQ: ticker-irq	\ -- addr
\ *G Creates the IRQ handler for the ticker.

: start-clock	\ -- ; start clock interrupt
\ *G Start the ticker periodic interrupt.
  di
  TickerBit _PCONP or!			\ power timer block
  _ticker
  0 over TnTCR + !			\ disable counter
  init_preDiv over TnPC + !		\ set prescaler
  #MRtick over TnMR0 + !		\ set match register
  3 over TnMCR + !			\ interrupt and reset on MR0
  ticker-irq TickInt# TickSlot# SetIrqIsr \ set up and enable interrupt
  1 swap TnTCR + !			\ enable counter
  ei
;

: stop-clock	\ -- ; stop clock interrupt
\ *G Stop the ticker periodic interrupt.
  TickInt# DisInt
;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]



\ ***********************
\ *S Time base extensions
\ ***********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  init-timers  start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

