\ Philips LPC210x Ticker interrupt handlers    - for Ref Peripheral Set

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))

decimal

\ =========
\ *! tickrps
\ =========
\ *T RPS Ticker Interrupt
\ *P The ticker is the basis of the TIMEBASE system. In order
\ ** start the ticker the START-CLOCK word is used. This
\ ** initialises the TC0 timer counter channel to produce
\ ** a periodic interrupt which updates the value TICKS.
\ ** The period is set by the equate TICK-MS in milliseconds,
\ ** and the value of TICKS is updated by TICK-MS at every
\ ** interrupt.

\ *P Other files named TICKxxx in the DRIVERS folder provide
\ ** the same functionality for other CPUs.

\ *P For the RPS, the default ticker uses Timer 1 in periodic mode
\ ** with the values calculated by GEN-TICKS below.

\ *P This code requires the interrupt facilities in INTARM3.


\ ************************
\ *S Configuration equates
\ ************************

_timer1 equ _ticker	\ -- addr
\ *G Select the timer/counter unit to use for the ticker

BIT4 equ TickerBit	\ -- mask
\ *G The bit to be set in RPS interrupt reg to enable the counter/timer.

interpreter
: gen-ticks	\ us clock -- load prescale
\ *G An INTERPRETER definition that takes a period in microseconds
\ ** and the clock speed in herz to produce the required values to be
\ ** loaded in the load and control registers.
  2dup #1000000 */  dup $10000 < 
  if   nip nip  0 exit  endif		\ control 3:2=00
  drop
  2dup #16000000 */  dup $10000 < 
  if   nip nip  $04 exit  endif		\ control 3:2=01
  drop
  #256000000 */  $08			\ control 3:2=10
;
target

tick-ms #1000 *  system-speed gen-ticks
  $C0 +  equ init_T1_control		\ in periodic mode and enabled
  equ init_T1_load			\ 16 bit load value
\ *G The initial value of the match register calculated from the
\ ** required ticker period TICK-MS in milliseconds and the
\ ** system clock speed SYSTEM-SPEED in herz.


\ ***************************
\ *S Ticker interrupt handler
\ ***************************

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- u ; returns the ticker value
\ *G Return current clock value in milliseconds. This value
\ ** can treated as a 32 bit unsigned value that will wrap
\ ** when it overflows.
[then]

: ticker-isr	\ -- ; high level interrupt handler
\ *G The high level interrupt handler for the ticker.
  RPS_IRQ_STATUS @ TickerBit and if
    1 _ticker $0C + !			\ reset interrupt
    ticks tick-ms + to ticks		\ update ticker by period in ms
[ timebase? ] [if]
    do-timers
[then]
endif
;

' ticker-isr add-irq	\ --
\ *G Creates the IRQ handler for the ticker.

: start-clock	\ -- ; start clock interrupt
\ *G Start the ticker periodic interrupt.
  di
  _ticker
  init_T1_load over !			\ set count value
  init_T1_control swap 8 + !		\ set control value
  TickerBit RPS_IRQ_ENABLE !		\ enable bit in interrupt controller
  ei
;

: stop-clock	\ -- ; stop clock interrupt
\ *G Stop the ticker periodic interrupt.
  bit7 _ticker 8 + bic!
;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]



\ ***********************
\ *S Time base extensions
\ ***********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  init-timers  start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

