\ SA1110 Ticker interrupt handlers

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

The ticker is based on the OS timer in the SA1110 core, and
is connected to the IRQ system using OSMR0

period=divisor/clock

divisor = period * clock
))

interpreter
: gen-os_t	\ us -- step
  xtal-speed um*			\ hz * time
  #1000000 um/mod nip			\ divisor = period * clock / 1000000
;
target

tick-ms #1000 * gen-os_t equ tick_step	\ -- n
\ *G TICK_STEP is the step required to update the OS_T match register
\ ** for the next compare.

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- ; returns the ticker value
( *G Get current clock value in milliseconds. )
[then]

: ticker-isr	\ --
  _OS_T					\ timer base address
  dup ossr + @ $1 and if		\ if OSMR0 set
    tick_step over osmr0 + +!		\ update the match register
    $1 over ossr + !			\ clear interrupt bit
    tick-ms addr ticks +!		\ update tick count
[ timebase? ] [if]
    do-timers
[then]
  endif
  drop					\ discard _OS_T base address
;
' ticker-isr add-irq			\ add it to the IRQ chain

0 value ticking?
\ *G True when the clock interrupt is running.

: start-clock	\ -- ; start clock interrupt
  di
  _OS_T [o/f]
  dup oscr + @  tick_step +		\ intialise OSMR0
    over osmr0 + !			\ to OSCR + step
  0 over ower + !			\ 
  $0F over ossr + !			\ reset match bits
  dup oier + @  $1 or  swap oier + !	\ set bit 0
  _IC icmr + [o/f] dup @ bit26 or swap ! \ set ICMR bit 26
  -1 to ticking?
  ei					\ enable interrupts
;
\ The use of [O/F] is to force a literal into a register to
\ reduce code size.

: stop-clock	\ -- ; stop clock interrupt
  [I
  _OS_T oier + [o/f]
  dup @  $1 invert and  swap !		\ clear OIER bit 0
  0 to ticking?
  I]
;

timebase? [if]
: start-timers  \ -- ; Start internal time clock
  stop-clock init-timers start-clock
;

: stop-Timers   \ -- ; disable timer system
  stop-clock
;
[then]

