\ STR91x Ticker interrupt handlers

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))

decimal

\ =========
\ *! ticklstr91x
\ =========
\ *T STR91x Ticker Interrupt
\ *P The ticker is the basis of the TIMEBASE system. In order
\ ** start the ticker *\fo{START-CLOCK} is used. This initialises
\ ** the Timer0 to produce a periodic interrupt which updates the
\ ** value *\fo{TICKS}. The period is set by the equate
\ ** *\fo{TICK-MS} in milliseconds, and the value of *\fo{TICKS}
\ ** is updated by *\fo{TICK-MS} at every interrupt.

\ *P Other files named *\i{TICKxxx} in the *\i{DRIVERS} folder provide
\ ** the same functionality for other CPUs.

\ *P For the STR91x, the default ticker uses Timer 0, Output
\ ** Compare Register 2 being used in PWM mode to reset the count
\ ** and to generate an interrupt.

\ *P This code requires the interrupt facilities in *\i{INTSTR91x.FTH}.


\ ****************
\ *S Configuration
\ ****************

interpreter
: gen-ticks	\ us -- n #pre
\ *G An *\fo{INTERPRETER} definition that takes a period in
\ ** microseconds to produce the required value to be loaded in
\ ** the compare register TIM_OC2R and the CC field prescaler
\ ** value in TIM_CR2.
  PCLKspeed #1000000 */			\ calculate 32 bit divisor
  1  #256 0 do				\ -- div pre
    2dup / $FFF0 u< ?leave
    1+
  loop					\ -- div pre'
  dup #256 > abort" Can't generate timer values"
  tuck / dup #10 < abort" Period too small"	\ -- pre' div
  5 -					\ allow for reset to FFFC
  swap 1-
;
target

tick-ms #1000 *  gen-ticks  equ #CCtick  equ #OCtick
\ *G The initial value of the compare register and CC field in CR1
\ ** calculated from the required ticker period *\fo{TICK-MS} in
\ ** milliseconds.

_TIM0 equ _ticker	\ -- addr
\ *G Base address of timer used for the ticker.
TIM01mask equ TickerBit	\ -- mask
\ *G Ticker power up bit mask.

TIM0int# equ TickInt#	\ -- n ; 0..31
\ *G Interrupt device number for the ticker device. Devices
\ ** 0..15 are on VIC0 and 16..31 on VIC1.
7 equ TickSlot#		\ -- n ; 0..15
\ *G Priority slot used for this interrupt, where 0 is the highest
\ ** priority and 15 the lowest.


\ ***************************
\ *S Ticker interrupt handler
\ ***************************

[undefined] ticks [if]	\ TICKS is also defined in TIMEBASE.FTH
0 value ticks	\ -- u ; returns the ticker value
\ *G Return current clock value in milliseconds. This value
\ ** can treated as a 32 bit unsigned value that will wrap
\ ** when it overflows.
[then]

: ticker-isr	\ -- ; high level interrupt handler
\ *G The high level interrupt handler for the ticker.
  bit11 _ticker TIM_SR + bic!		\ reset OCF2 interrupt
  tick-ms addr ticks +!			\ update ticker by period in ms
[ timebase? ] [if]
  do-timers
[then]
;

' ticker-isr IRQ: ticker-irq	\ -- addr
\ *G Creates the IRQ handler for the ticker.

: start-clock	\ -- ; start clock interrupt
\ *G Start the ticker periodic interrupt.
  di
  TickerBit _SCU			\ power timer block
  2dup SCU_PCGR1 + or!			\ enable clocks
  SCU_PRR1 + or!			\ take out of reset

  \ Set all the timer 0 registers to their default values
  _ticker
  $8000 over TIM_OC1R + !
  $8000 over TIM_OC2R + !
  0 over TIM_CR1 + !
  #CCtick over TIM_CR2 + !		\ prescaler
  $FF over TIM_CNTR + !			\ any value will do
  0 over TIM_SR + !
  \ now configure timer so that every x ms an int is generated.
  bit11 #CCtick or over TIM_CR2 + !	\ enable OC2 interrupt, set prescaler
  #OCtick over TIM_OC2R + !		\ period
  #OCtick 1 rshift over TIM_OC1R + !	\ half period
  \ Set up VIC
  ticker-irq TickInt# TickSlot# SetIrqIsr \ set up and enable interrupt
  \ enable timer 0
  bit15 bit4 or swap TIM_CR1 + !	\ enable counter in PWM mode
  ei
;

: stop-clock	\ -- ; stop clock interrupt
\ *G Stop the ticker periodic interrupt.
  TickInt# DisInt
;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]



\ ***********************
\ *S Time base extensions
\ ***********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  init-timers  start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

