\ W29EE011.FTH - Winbond Flash driver, 2 devices as 16 bit pair

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

\ ************
\ *! W29EE011
\ *T W29EE011 Flash programming
\ ************

\ *P The file W29EE011.FTH is for programming a 16 bit pair of devices.
\ ** The equate FLASHBASE must be set up before this file is
\ ** compiled. If it is not defined a base address of 0400:0000
\ ** will be used. Ensure that bit 26 of this address is set to
\ ** bypass the cache. Uncached addresses may be passed to the
\ ** programming routines as these force bit 26 of Flash addresses
\ ** to a '1'.

\ *P The code must be run from RAM or any area that
\ ** is not part of the Flash. Because the flash is still
\ ** present at 0000:0000, interrupts are disabled during
\ ** programming. The flash memory is copied into DRAM by the boot
\ ** code and all code execution (except for interrupt vector despatch)
\ ** occurs from DRAM.

\ *P Note that when programming these devices, programming part
\ ** of a page (128bytes * 2) will leave the unprogrammed part
\ ** set to bytes of FF. We recommend that all programming be
\ ** done in units of 256 bytes starting at an address aligned to
\ ** a 256 byte boundary.


decimal

#256 equ FlProgUnit	\ -- n ; Flash programming unit size
\ *G Define the size of the smallest unit that can be programmed.
\ ** For the W29E011 on a 16 bit bus this corresponds to two 128
\ ** byte pages, 256 bytes.

[undefined] flashbase [if]
$00000000 constant flashbase	\ -- addr ; base of flash memory
\ *G The base address of the Flash memory. This is the base address
\ ** in memory before any manipulation to produce an uncached
\ ** address.
[then]

flashbase bit26 or equ u/c-flashbase	\ -- addr ; base of flash memory
\ *G The base address of the Flash memory. This address must be
\ ** uncached during programming, or caching must be turned off.
\ ** For the S3C4510, setting bit 26 of the address (0400:0000)
\ ** forces the cache to be bypassed.

: 10us		\ --
\ *G Wait 10 microseconds. A 50MHz cached CPU is assumed.
  #160 0 do loop
;

: FlCmd		\ command --
\ *G Unlock the flash and write the command.
  $AAAA [ u/c-flashbase $5555 2* + ] literal w!	\ unlock
  $5555 [ u/c-flashbase $2AAA 2* + ] literal w!
        [ u/c-flashbase $5555 2* + ] literal w!	\ command
;

: FlId		\ -- manid devid
\ *G Get manufacturer's ID, should be $DADA $C1C1.
  [I
  $8080 FlCmd  $6060 FlCmd  10us
  [ u/c-flashbase $000 2* + ] literal w@
  [ u/c-flashbase $001 2* + ] literal w@
  $F0F0 FlCmd  10us
  I]
;

: (EraseFlash)	\ dest dlen --
\ *G Erase the flash sectors covering dlen bytes at dest.
\ ** This a noop for code compatibility with NETBOOT.FTH,
\ ** as the W29E011 is erased while it is being programmed.
  2drop
;

: Write256	\ src dest -- flag
\ *G Program a 128 word (256 byte) page, returning 0 for success.
  $04000000 or			\ *** make dest an uncached address ***
  [I				\ interrupt safe
  $A0A0 FlCmd

  [asm
\ copy 128 words = 256 bytes
    ldr     r0, [ psp ]			\ src=r0, dest=tos, r2=len
    mov     r2, # #128
l: s29-1
    ldrh    r1, [ r0 ], # 2
    strh    r1, [ tos ], # 2
    sub .s  r2, r2, # 1
    b .ne   s29-1

\ wait 100us min @ 20ns/instruction, 40ns per loop, 25 loops/us, 25000 loops/ms
    mov     r2, # $0A00
l: s29-2
    sub .s  r2, r2, # 1
    b .ne   s29-2

\ put back last source address + 2
    str     r0, [ psp ]
  asm]					\ -- src+256 dest+256

  swap 2- w@  swap 2-			\ -- w last
  #100000 >r				\ wait for prog or fail
  begin
    r> 1- >r
    2dup w@ =				\ leave if same
    r@ 0= or 
  until
  r> drop
  w@ <>					\ fail if different
  I]
;

: (ProgFlash)	\ src dest len --
\ *G Write len bytes from memory at src to Flash at dest.
\ ** Note that the Flash page will be erased, and that
\ ** no verication is performed except when each cell is
\ ** programmed. The source and destination addresses must
\ ** be word aligned, and len must be a multiple number of
\ ** words. A dot is displayed at the start of each 4kb block
\ ** and an E is displyed for a badly programmed 256 byte page.
  0 ?do					\ -- src dest
    i $0FFF and 0= if  ." ."  then	\ one '.' per 4k
    2dup Write256
    if  ." E"  then
    #256 + swap #256 + swap
  #256 +loop
  2drop
;

[undefined] .lword [if]
: .LWORD	\ x --
\ *G Display x as an 8 digit hexadecimal number in the form
\ ** "abcd:efgh".
  base @ hex  swap
  0 <# # # # # ascii : hold # # # # #> type
  base !
;
[then]

[undefined] .word [if]
: .WORD		\ x --
\ *G Display x as an 4 digit hexadecimal number in the form
\ ** "abcd".
  base @ hex  swap
  0 <# # # # # #> type
  base !
;
[then]

: .src/dest	\ src dest -- src dest
\ *G Display the source and destination addresses and contents,
\ ** returning the addresses unchanged.
  ."  source: " over .lword  over w@ ." (" .word ." )"
  ."  dest: " dup .lword  dup w@ ." (" .word ." )"
;

: (VerifyFlash)	\ src dest len --
\ *G Verify len bytes from memory at src to Flash at dest.
\ ** The source and destination addresses must
\ ** be word aligned, and len must be a multiple number of
\ ** words.
  swap bit26 or swap			\ *** bypass the cache for dest ***
  3dup s= if  3drop exit  endif
  cr ."  Verify failed"
  0 ?do
    over w@ over w@ <> if
      cr 2 spaces .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    2+ swap  2+ swap
  2 +loop
  2drop
;

: ProgramFlash	\ src dest len --
\ *G Using the memory buffer at src, program the Flash at dest
\ ** with len bytes. The relevant Flash sectors are erased, the
\ ** Flash is programmed, and the result verified.
\ ** The source and destination addresses must be
\ ** word aligned, and len must be a multiple number of
\ ** words.
  cr ." Erasing " 2dup (EraseFlash)
  cr ." Programming "  3dup (ProgFlash)
  cr ." Verifying "  (VerifyFlash)
;


\ ******
\ *> ###
\ ******


