\ dmaLPC23xx.fth - LPC23xx GPDMA driver

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))


\ =============
\ *! dmalpc23xx
\ *T DMA driver for LPC23xx
\ =============

\ *****************
\ *S DMA primitives
\ *****************
\ *P The DMA engine has problems transferring data to the primary
\ ** internal RAM block. We recommend that you use the upper portion
\ ** of the USB block, which has to be powered up.

\ *P By default, other MPE code for LPC23xx devices uses DMA
\ ** channel 0 for the MCI driver and DMA channel 1 for the USB
\ ** driver.

: initDMA	\ --
\ *G Initialise the GPDMA controller. The USB block is also
\ ** powered up so that the USB RAM can be used.
  ~pcUSB ~pcGPDMA or _PCONP or!		\ power up USB and GPDMA
  _GPDMA
  $03 over dmaIntTCClr + !		\ clear interrupts
  $03 over dmaIntErrClr + !
  $01 over dmaConfig + !		\ LE, enabled
  drop  10 0 do loop
;

$02  12 lshift equ /SBsize	\ source burst = 8
$02  15 lshift equ /DBsize	\ destination burst = 8
$02  18 lshift equ /SBwidth	\ source data = 32 bits
$02  21 lshift equ /DBwidth	\ dest data = 32 bits
1    26 lshift equ SIm2p	\ increment source after transfer
0    27 lshift equ DIm2p	\ do not increment dest after transfer
%101 ( 0 ) 28 lshift equ ProtM2p	\ cacheable, non-buff, privileged
1    31 lshift equ TCIntEn	\ enable termal count interrupt

0    26 lshift equ SIp2m	\ do not increment source after transfer
1    27 lshift equ DIp2m	\ increment dest after transfer
%011 28 lshift equ ProtP2M	\ non-cacheable, buff, privileged

TCIntEn  ProtM2p or  DIm2p or SIm2p or
/DBwidth or  /SBwidth or  /DBsize or  /SBsize or
  equ CtrlValM2P	\ -- mask
\ *G Channel control register value for M2P transfers.

TCIntEn  ProtP2m or  DIp2m or SIp2m or
/DBwidth or  /SBwidth or  /DBsize or  /SBsize or
  equ CtrlValP2M	\ -- mask
\ *G Channel control register value for P2M transfers.

: +m2p0		\ src dest cfg --
\ *G Initialise DMA channel 0 for a memory to peripheral transfer.
\ ** The peripheral is the flow controller, so a length is not needed.
\ ** The *\i{cfg} value is set in the *\fo{dma0Config} register.
  -rot _GPDMA				\ -- cfg src dest gpdma
  tuck dma0DestAddr + !			\ set dest (periph) address
  tuck dma0SrcAddr + !			\ set source (memory) address
  0 over dma0LLI + !			\ no linked list
  CtrlValM2P over dma0Ctrl + !		\ set control reg
  dma0Config + !			\ must be last
;

: +p2m0		\ src dest cfg --
\ *G Initialise DMA channel 0 for a peripheral to memory transfer.
\ ** The peripheral is the flow controller, so a length is not needed.
\ ** The *\i{cfg} value is set in the *\fo{dma0Config} register.
  -rot swap _GPDMA			\ -- cfg dest src gpdma
  bit0 over dmaIntTCClr + !		\ clear indicators
  bit0 over dmaIntErrClr + !
  tuck dma0SrcAddr + !			\ set source (memory) address
  tuck dma0DestAddr + !			\ set dest (periph) address
  0 over dma0LLI + !			\ no linked list
  CtrlValP2M over dma0Ctrl + !		\ set control reg
  dma0Config + !			\ must be last
;

: -dma0		\ --
\ *G Finish the transfer on DMA channel 0.
  $00 _GPDMA dma0Config + !		\ turn everything off
;

: dma0Err?	\ -- flag ; true on error
  _GPDMA dmaRawIntErrSt + @ bit0 and
;


\ **************
\ *S DMA for MCI
\ **************

$1E901 equ M2PvalMCI	\ -- x
\ *G DMA configuration register value for an MCI M2P transfer,
\ ** i.e. a write to the card.
$1F009 equ P2MvalMCI	\ -- x
\ *G DMA configuration register value for an MCI P2M transfer,
\ ** i.e. a read from the card.


\ ======
\ *> ###
\ ======

decimal
