\ hwMciCDS2388.fth - hardware set up for CDS Olimex E2368 MCI driver

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! hwmcicds2388
\ *T Configuration of CDS LPC2388 MCI driver
\ ===========
\ *P The MCI unit's SD card pins can be assigned to more than one
\ ** set of pins. In addition, the Card Detect and Write Protect
\ ** pins are assigned to a wide range of GPIO lines on different
\ ** boards.

\ *P The *\i{hwMcixxx.fth} files contain the hardware
\ ** configurations for various LPC23xx/4xx boards. They are
\ ** compiled before the main MCI driver file *\i{mciLPC23xx.fth}.

\ *P In this implementation, P3.25 is used as a separate Card Detect
\ ** input, and P3.26 is used as the Write protect input.

\ ** In most cases, the default value of *\fo{/pcdMCI} below is
\ ** appropriate.

1 equ /pcdMCI	\ -- u
\ *G Peripheral clock divider for MCI unit. Set this to one if
\ ** performance is more important than power consumption.
/pcdMCI PCLKdiv 24 lshift equ mciPCLK1val	\ -- val
\ *G Bit value for MCI peripheral clock divider.
$0300:0000 equ mciPCLK1mask		\ -- mask
\ *G Bit mask for MCI peripheral clock divider.

2 [ps  19 +ps  20 +ps  21 +ps  22 +ps  0 [psm]  29 +ps  30 +ps  ps]
  equ mciPS1val  equ mciPS1mask
\ *G PINSEL1 register value and mask for MCI.
2 [ps  11 +ps  12 +ps  13 +ps  ps]
  equ mciPS4val  equ mciPS4mask
\ *G PINSEL4 register value and mask for MCI.

: initMCIconfig	\ --
  bit3 bit0 or _SCS or!			\ MCI power active high, FIO for P0/1
\ bit3 _SCS bic!			\ MCI power active low
\ bit0 _SCS or!				\ FIO for P0/1
  bit25 bit26 or  _FIO3			\ set P3.25 and P3.26 as inputs
  2dup FIODIR + bic!  FIOMASK + bic!
  mciPCLK1val mciPCLK1mask _PCLKSEL1 setMask	\ peripheral clock
  mciPS1val mciPS1mask _PINSEL1 setMask	\ select pins
  mciPS4val mciPS4mask _PINSEL4 setMask
;

: CDin?		\ -- flag
\ *G Return non-zero if a card is present.
  _FIO3 FIOPIN + @ bit25 and 0=
;

: WPin?		\ -- flag
\ *G Return non-zero if the write protect slider is set.
  _FIO3 FIOPIN + @ bit26 and 0<>
;


\ ======
\ *> ###
\ ======

decimal
