\ Analog Devices ADuC702x - Miscellaneous

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

only forth definitions

\ =============
\ *! miscad702x
\ =============
\ *T Miscellaneous AD702x peripherals
\ *P This file containains miscellaneous functions
\ ** and small peripheral drivers. Copy this code into your own
\ ** files as the functionality of this file will change between
\ ** releases


\ *************
\ *S Reboot CPU
\ *************
\ *P Rebooting uses a software reset.

: reboot	\ --
\ *G Force a software reset.
  $04 _SYSCON RstSta + !
  begin again
;



\ *************
\ *S DAC output
\ *************

: >DACmode	\ mode n --
\ *G Set the mode of DAC n (0, 1).
  3 lshift _DAC + Dac0Dat + !
;

: >DAC		\ data n --
\ *G Set the data value (0..4095) DAC n (0, 1).
  swap $FFF and #16 lshift
  swap 3 lshift _DAC +
  Dac0Dat + !
;


\ ************
\ *S ADC input
\ ************
\ *P Before use the relevant GPIO set up must be performed.

l: ^ADC
  _ADC ,

: SetConv	\ x --
\ *G Place x in the ADC conversion register, defining the mode of
\ ** the conversions(s).
  ^ADC @ AdcCon + !
;

: InitADC	\ --
\ *G Power up Vref and the ADC and wait at least 500us.
  $01 _REFERENCE RefCon + !
  $20 SetConv  1 ms
;

: SetChannels	\ +chan -chan --
\ *G Set the positive and negative ADC channels.
  ^ADC @ tuck  AdcCn + !  AdcCp + !
;

IsLeaf : ADCwait	\ --
\ *G Wait for ADC conversion to complete
  begin
    ^ADC @ AdcSta + @ 1 and
  until
;

: ADCread	\ -- n
\ *G Read the result of the last ADC conversion
  ^ADC @ AdcDat + @ #16 arshift
;

: ADC@		\ chan -- n
\ *G Perform a single software read of the given channel.
  0 SetChannels
  $A3 SetConv  ADCwait  ADCread		\ One single-ended s/w conversion
;


\ ***************
\ *S Flash access
\ ***************
\ *P The Flash is 16 bits wide and must be written in units of
\ ** 16 bits. The Flash is addressed with normal byte addresses.

l: ^FCI
  _FCI ,

IsLeaf : FeeW@	\ addr -- w
\ *G Read the the 16 bit value from Flash.
  ^FCI @				\ -- addr iob
  tuck FeeAdr + !			\ set address
  $01 over FeeCon + !			\ set command
  begin  dup FeeSta + @ 1 and  until	\ wait for completion
  FeeDat + @  #16 lshift #16 rshift	\ read, extract 16 bits
;

IsLeaf : FeeW!	\ w addr --
\ *G write 16 bits into Flash.
  ^FCI @				\ -- w addr iob
  tuck FeeAdr + !  tuck FeeDat + !	\ -- iob
  $02 over FeeCon + !			\ set command
  begin  dup FeeSta + @ 1 and  until	\ wait for completion
  drop
;

IsLeaf : ErasePage	\ addr --
\ *G Erase the page containing addr.
  ^FCI @				\ -- addr iob
  tuck FeeAdr + !			\ -- iob
  $05 over FeeCon + !			\ set command
  begin  dup FeeSta + @ 1 and  until	\ wait for completion
  drop
;

: FeeC@		\ addr -- b
\ *G Read an 8 bit item from Flash.
  dup $FFFFFFFE and FeeW@
  swap 1 and				\ if odd address
  if  8 rshift  endif			\ take high byte
  $FF and
;

: FeeC!		\ b addr --
\ *G Write an 8 bit item into Flash.
  dup $FFFFFFFE and dup FeeW@		\ -- b addr addr' w
  rot 1 and if				\ -- b addr' w ; high byte
    $00FF and rot 8 lshift $FF00 and or	\ -- addr' w'
  else
    $FF00 and  rot $00FF and  or
  endif
  swap FeeW!
;

: Fee@		\ addr -- x
\ *G Read a 32 bit item from Flash.
  dup FeeW@  swap 2+ FeeW@ #16 lshift  or
;

: Fee!		\ x addr --
\ *G Write a 32 bit item into Flash.
  2dup FeeW!  swap #16 rshift swap 2+ FeeW!
;

compiler
: inFlash?	\ addr -- flag
\ *G Returns true if addr is in Flash.
  $00010000 u<
;
target

: !F		\ x addr --
\ *G Write 32 bits into Flash or other memory. Addresses in the range
\ ** 0000:0000..0000:FFFF are treated as Flash writes and all
\ ** others as normal memory writes.
  dup inFlash?
  if  Fee!  else  !  endif
;

: W!F		\ w addr --
\ *G Write 16 bits into Flash or other memory. Addresses in the range
\ ** 0000:0000..0000:FFFF are treated as Flash writes and all
\ ** others as normal memory writes.
  dup inFlash?
  if  FeeW!  else  W!  endif
;

: C!F		\ x addr --
\ *G Write 8 bits into Flash or other memory. Addresses in the range
\ ** 0000:0000..0000:FFFF are treated as Flash writes and all
\ ** others as normal memory writes.
  dup inFlash?
  if  FeeC!  else  C!  endif
;


\ ======
\ *> ###
\ ======

decimal

