\ Atmel EVB01 interrupt RX, polled TX, serial communication drivers

((
Copyright (c) 2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE

))


\ ********************
\ Interrupt management
\ ********************

[undefined] ei [if]
code EI		\ -- ; enable interrupts
  mrs  r0, cpsr
  bic  r0, r0, # $C0			\ clear I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] di [if]
code DI         \ -- ; disable interrupts
  mrs  r0, cpsr
  orr  r0, r0, # $C0			\ set I and F bits
  msr  cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] [I [if]
code [I		\ R: -- x ; preserve I/F status on return stack, disable ints
  mrs   r0, cpsr			\ get CPSR
  stmfd rsp ! { r0 }			\ save
  orr   r0, r0, # $C0			\ set I and F bits
  msr   cpsr _c r0			\ modify control bits
  next,
end-code
[then]

[undefined] I] [if]
code I]		\ R: x -- ; restore int. status from r. stack
  ldmfd rsp ! { r0 }
  msr   cpsr _c r0
  next,
end-code
[then]


\ ************************
\ Initialisation constants and common code
\ ************************

\ USART Inital values for Mode Register
US_CLKS_MCK US_CHRL_8 or US_ASYNCH or US_PAR_NO or US_NBSTOP1 or US_CHMODE_NORMAL or US_MODE8 or US_CLKNO or equ US_INITMODE

\ USART Initial values for Control Register
US_RXDIS US_TXDIS or US_RSTRX or US_RSTTX or US_RSTSTA or equ US_RESET

US_RXEN US_TXEN or equ US_GO

\ USART Initial values for Interrupt Disable Register
$3FF equ US_INTDIS


decimal

: Baud-Rate-Gen \ baud -- CD
  system-speed swap / 16 /
;


\ *****
\ UART0
\ *****

256 cqueue: SerQ0

: ISR-Key0
  SerQ0 (cqFull?) if
    _USART0 US_RHR + @ drop
  else
    _USART0 US_RHR + @ SerQ0 (>cqueue)
  then
;

l: ^AIC
  _AIC ,

proc Ser0ISR	\ -- ; UART 0 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  bl	' ISR-Key0

  ldr	r4, ^AIC			\ issue end of interrupt to AIC
  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, r0
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code


: (initser0qi)	\ --
  di
  $00000004 _AIC AIC_IDCR + !			\ Disable UART0 interrupt interrrupts
  $00000007 _AIC AIC_SMR0 + 2 cells + !		\ Set USART0 inerrupt to priority 7 (highest)
  Ser0ISR   _AIC AIC_SVR0 + 2 cells + !		\ Store address of interrupt handler routine
  $0000000C _PS_PCER !		\ Enable clocks for both USARTS
  $0000C000 _PIO PIO_PDR + !	\ disable PIO for USART0 channels
  $00000080 _PIO PIO_IDR + !	\ Disable interrupts for the PIO channels
  US_RESET _USART0 US_CR + !	\ Reset the USART0 channels
  US_INITMODE _USART0 US_MR + !	\ Set up initial mode for USART0
  console-speed Baud-Rate-gen 
  _USART0 US_BRGR + !		\ Set up baud rate
  0 _USART0 US_RTOR + !		\ Disable reciever timeout
  0 _USART0 US_TTGR + !		\ Disable transmitter time-guard
  0 _USART0 US_RPR + !		\ Disbale receive pointer register
  0 _USART0 US_RCR + !		\ Zero receiver counter register
  0 _USART0 US_TPR + !		\ Disable transmitter pointer register
  0 _USART0 US_TCR + !		\ Zero transmitter counter register
  $00000004 _AIC AIC_IECR + !	\ Enable the USART0 interrupt
  $00000001 _USART0 US_IER + !	\ Enable interrupt on RX0
  US_GO _USART0 US_CR + !	\ GO GO GO
  ei
;

: key?0qi	\ -- flag
  SerQ0 cqempty? 0=
;

: Key0qi	\ -- char
  SerQ0 cqueue>
;

: emit0qi	\ char --
  _USART0
  begin
    dup US_CSR + @ US_TXRDY and
  until
  US_THR + !
;

: type0qi	\ addr len --
  bounds
  ?do  i c@ emit0qi  loop
;

: cr0qi		\ --
  $0D emit0qi  $0A emit0qi
;


cdata create Console0qi
  ' key0qi ,
  ' key?0qi ,
  ' emit0qi ,
  ' type0qi ,
  ' CR0qi ,


\ *****
\ UART1
\ *****

256 cqueue: SerQ1

: ISR-Key1	\ --
  SerQ1 (cqFull?) if
    _USART1 US_RHR + @ drop
  else
    _USART1 US_RHR + @ SerQ1 (>cqueue)
  then
;

l: ^AIC
  _AIC ,

proc Ser1ISR	\ -- ; UART 1 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  mrs   r0, SPSR			\ save SPSR for nesting
  stmfd r13 ! { r0 }
  mov	r1, r13				\ preserve R13 for later

  sub	r13, r13, # INT-UP-SIZE		\ allocate USER area pointer
  mov	up, r13
  mov	psp, r13			\ allocate DATA stack pointer at TOP of stack
  sub   r13, r13, # INT-SP-SIZE		\ allocate RETURN stack pointer

  stmfd r13 ! { r1 }			\ save orignal value on NEW return stack

  bl	' ISR-Key1

  ldr	r4, ^AIC			\ issue end of interrupt to AIC
  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 { r13 }			\ get original R13 back

  ldmfd r13 ! { r0 }			\ restore SPSR
  msr   SPSR, r0
  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code


: (initser1qi)	\ --
  di
  $00000008 _AIC AIC_IDCR + !			\ Disable UART interrupt interrrupts
  $00000007 _AIC AIC_SMR0 + 3 cells + !		\ Set USART interrupt to priority 7 (highest)
  Ser1ISR   _AIC AIC_SVR0 + 3 cells + !		\ Store address of interrupt handler routine
  $0000000C _PS_PCER !		\ Enable clocks for both USARTS
  $00600000 _PIO PIO_PDR + !	\ disable PIO for USART channels
  $00600000 _PIO PIO_IDR + !	\ Disable interrupts for the PIO channels
  US_RESET _USART1 US_CR + !	\ Reset the USART0 channels
  US_INITMODE _USART1 US_MR + !	\ Set up initial mode for USART0
  console-speed Baud-Rate-gen 
  _USART1 US_BRGR + !		\ Set up baud rate
  0 _USART1 US_RTOR + !		\ Disable reciever timeout
  0 _USART1 US_TTGR + !		\ Disable transmitter time-guard
  0 _USART1 US_RPR + !		\ Disbale receive pointer register
  0 _USART1 US_RCR + !		\ Zero receiver counter register
  0 _USART1 US_TPR + !		\ Disable transmitter pointer register
  0 _USART1 US_TCR + !		\ Zero transmitter counter register
  $00000008 _AIC AIC_IECR + !	\ Enable the USART interrupt
  $00000001 _USART1 US_IER + !	\ Enable interrupt on RX0
  US_GO _USART1 US_CR + !	\ GO GO GO
  ei
;

: key?1qi	\ -- flag
  SerQ1 cqempty? 0=
;

: Key1qi	\ -- char
  SerQ1 cqueue>
;

: emit1qi	\ char --
  _USART1
  begin
    dup US_CSR + @ US_TXRDY and
  until
  US_THR + !
;

: type1qi	\ addr len --
  bounds
  ?do  i c@ emit1qi  loop
;

: cr1qi		\ --
  $0D emit1qi  $0A emit1qi
;


cdata create Console1qi
  ' key1qi ,
  ' key?1qi ,
  ' emit1qi ,
  ' type1qi ,
  ' CR1qi ,


\ **************
\ Console set up
\ **************

: init-ser	\ -- ; initialise serial ports
  (initser0qi)  (initser1qi)
;

console-port 0 = [if]
console0qi constant Console	\ -- addr ; select default serial port
[then]

console-port 1 = [if]
console1qi constant Console	\ -- addr ; select default serial port
[then]


