\ S3C4510B polled RX, polled TX, serial communication drivers

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

only forth definitions


\ **************
\ Initialisation
\ **************

\ This next word takes the system clock rate (in HZ) and the required serial
\ comms rate (in BPS) and returns a system-dependent number of parameters that
\ should be sufficent to effect the baud rate selection by initialising the
\ correct hardware.

\ S3C4510B
\ BRGOUT=MCLK2/((cnt0+1)*16*(16^cnt1))
\ where
\   cnt0=divisor
\   if cnt1=0, no prescale, otherwise prescale by 16
\ 
\ assuming cnt1=0
\ cnt0=(system-speed/2*baud*16)-1

interpreter
: gen-baud-rate	\ clk-rate bps-rate -- cnt0<<4+cnt1
  / #32 /mod swap #15 >
  if  1+  endif				\ round up if necessary
  1 -
  dup $FFF > if
    1+
    #16 /mod swap 7 >
    if  1+  endif
    1-  1
  else
    0
  endif
  swap $FFF and  4 <<n			\ shift cnt0
  or					\ merge cnt1
;
target

system-speed uart0-speed gen-baud-rate
  equ init-ubrdiv0
system-speed uart1-speed gen-baud-rate
  equ init-ubrdiv1

create uart0-setup
  _uart0 utxbuf + ,	0 ,		\ dummy transmit character
  _uart0 ubrdiv + ,     init-ubrdiv0 ,	\ baud rate divisor
  _uart0 ulcon + ,	%00000011 , 	\ nonIRDA, internal clock, 8N1
  _uart0 ucon + ,	%00001001 ,	
  0 ,

create uart1-setup
  _uart1 utxbuf + ,	0 ,		\ dummy transmit character
  _uart1 ubrdiv + ,     init-ubrdiv1 ,	\ baud rate divisor
  _uart1 ulcon + ,	%00000011 , 	\ nonIRDA, internal clock, 8N1
  _uart1 ucon + ,	%00001001 ,	
  0 ,

: init-ser	\ --
\ *G Initialise the serial line(s).
  _interrupts intmod + dup @
    bit4 bit5 or invert and		\ clear bits 4/5 for IRQ ints
    swap !
  _interrupts intmsk + dup @
    bit4 bit5 or or			\ set bits 4/5 to disable interrupts
    swap !
  uart0-setup init-io
  _interrupts intmod + dup @
    bit6 bit7 or invert and		\ clear bits 4/5 for IRQ ints
    swap !
  _interrupts intmsk + dup @
    bit6 bit7 or or			\ set bits 4/5 to disable interrupts
    swap !
  uart1-setup init-io
  #100000 0 do loop			\ waste at least a character time
;


\ *****************
\ Serial primitives
\ *****************

: (serkey?)	\ uart -- t/f
\ *G Return true if a character has been received by the UART whose
\ ** base address is given.
  ustat + @ $020 and
;

tasking? [if]
: (serkey)	\ -- char
\ *G Wait until a character has been received by the UART whose
\ ** base address is given, and return the character.
  begin
    dup ustat + @ $020 and 0=
   while
    pause
  repeat
  urxbuf + @ $FF and
;
[else]
: (serkey)	\ -- char
  begin  dup ustat + @ $020 and  until
  urxbuf + @ $FF and
;
[then]

: (seremit)	\ char uart --
\ *G Send the character to the UART whose base address is given.
  begin
    dup ustat + @ $040 and
  until
  utxbuf + !
;

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (serCR)	\ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D over (seremit)  $0A swap (seremit)
;

: serkey?0	( -- flag )	_uart0 (serkey?) 0<>  ;
: serkey0	( -- char )	_uart0 (serkey)  ;
: seremit0	( char -- )	_uart0 (seremit)  ;
: sertype0	( addr len -- )	_uart0 (sertype)  ;
: sercr0	( -- )		_uart0 (sercr)  ;

: serkey?1	( -- flag )	_uart1 (serkey?) 0<>  ;
: serkey1	( -- char )	_uart1 (serkey)  ;
: seremit1	( char -- )	_uart1 (seremit)  ;
: sertype1	( addr len -- )	_uart1 (sertype)  ;
: sercr1	( -- )		_uart1 (sercr)  ;


\ ***********************
\ Generic i/o assignments
\ ***********************

cdata
create Console0	\ -- addr ; OUT managed by upper driver
\ *G Function despatch table for UART0.
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

create Console1	\ -- addr ; OUT managed by upper driver
\ *G Function despatch table for UART1.
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

console-port 0 = [if]
Console0 constant Console
[then]
console-port 1 = [if]
Console1 constant Console
[then]

decimal

