\ S3C4510B queued interrupt RX, polled TX, serial communication drivers

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

only forth definitions


\ *********************
\ Baud Rate Calculation
\ *********************

\ This next word takes the system clock rate (in HZ) and the required serial
\ comms rate (in BPS) and returns a system-dependent number of parameters that
\ should be sufficent to effect the baud rate selection by initialising the
\ correct hardware.

\ S3C4510B
\ BRGOUT=MCLK2/((cnt0+1)*16*(16^cnt1))
\ where
\   cnt0=divisor
\   if cnt1=0, no prescale, otherwise prescale by 16
\ 
\ assuming cnt1=0
\ cnt0=(system-speed/2*baud*16)-1

interpreter
: gen-baud-rate	\ clk-rate bps-rate -- cnt0<<4+cnt1
\ *G INTERPRETER definition to calculate the initial value of
\ ** the baud rate generator register.
  / #32 /mod swap #15 >
  if  1+  endif				\ round up if necessary
  1 -
  dup $FFF > if
    1+
    #16 /mod swap 7 >
    if  1+  endif
    1-  1
  else
    0
  endif
  swap $FFF and  4 <<n			\ shift cnt0
  or					\ merge cnt1
;
target

system-speed uart0-speed gen-baud-rate
  equ init-ubrdiv0	\ -- n
\ *G Initial value of baud rate generator register for UART0.
system-speed uart1-speed gen-baud-rate
  equ init-ubrdiv1	\ -- n
\ *G Initial value of baud rate generator register for UART1.


\ *****************************
\ *S Interrupt Service Routines
\ *****************************

#256 cqueue: SerQ0	\ -- addr ; circular byte queue
#256 cqueue: SerQ1	\ -- addr ; circular byte queue

: ReadCh	\ queue uart --
\ *G Read character from UART into queue. If the queue is full
\ ** or the receive status is bad, discard the character.
  dup ustat + @ %00100000 and if	\ good character?
    over (cqFull?) if			\ queue full?
      urxbuf + @ drop			\ read character and discard it
      drop				\ discard queue
      exit
    endif
    urxbuf + @ $FF and swap (>cqueue)	\ read and stash character
    exit
  endif
  2drop					\ bad char, discard.
;

: Uart0-rx-isr	\ --
\ *G UART0 receive interrupt service routine.
  SerQ0 _uart0 ReadCh
;

: Uart1-rx-isr	\ --
\ *G UART1 receive interrupt service routine.
  SerQ1 _uart1 ReadCh
;


\ *****************
\ *S Initialisation
\ *****************

create uart0-setup	\ -- addr
\ *G Initialisation table for UART0.
  _uart0 utxbuf + ,	0 ,		\ dummy transmit character
  _uart0 ubrdiv + ,     init-ubrdiv0 ,	\ baud rate divisor
  _uart0 ulcon + ,	%00000011 , 	\ nonIRDA, internal clock, 8N1
  _uart0 ucon + ,	%00001101 ,	\ TX, RX and Status Ints enabled
  0 ,

create uart1-setup	\ -- addr
\ *G Initialisation table for UART1.
  _uart1 utxbuf + ,	0 ,		\ dummy transmit character
  _uart1 ubrdiv + ,     init-ubrdiv1 ,	\ baud rate divisor
  _uart1 ulcon + ,	%00000011 , 	\ nonIRDA, internal clock, 8N1
  _uart1 ucon + ,	%00001101 ,	
  0 ,

: init-ser	\ --
\ *G Initialise the serial line(s).
  di
  uart0-setup init-io
  uart1-setup init-io
  ['] uart0-rx-isr nUart0_RX_Err_Int SetIRQ
  ['] uart1-rx-isr nUart1_RX_Err_Int SetIRQ
  nUart0_rx_err_int EnInt
  nUart1_rx_err_int EnInt
  nGlobal_Int Enint
  #100000 0 do loop			\ waste at least a character time
  ei
;


\ ********************
\ *S Serial primitives
\ ********************

: serkey?0	\ base -- t/f
\ *G Return true if a character has been received by UART0.
  SerQ0 cqempty? 0=
;

: serkey0	\ -- char
\ *G Wait until a character has been received by UART1,
\ ** and return the character.
  SerQ0 cqueue>
;

: serkey?1	\ base -- t/f
\ *G Return true if a character has been received by UART1.
  SerQ1 cqempty? 0=
;

: serkey1	\ -- char
\ *G Wait until a character has been received by UART1,
\ ** and return the character.
  SerQ1 cqueue>
;

: (seremit)	\ char uart --
\ *G Send the character to the UART whose base address is given.
  begin
    dup ustat + @ $040 and
  until
  utxbuf + !
;

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (serCR)	\ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D over (seremit)  $0A swap (seremit)
;

: seremit0	( char -- )	_uart0 (seremit)  ;
\ *G Transmit a character using UART0.
: sertype0	( addr len -- )	_uart0 (sertype)  ;
\ *G Transmit a string using UART0.
: sercr0	( -- )		_uart0 (sercr)  ;
\ *G Transmit a CR/LF pair using UART0.

: seremit1	( char -- )	_uart1 (seremit)  ;
\ *G Transmit a character using UART1.
: sertype1	( addr len -- )	_uart1 (sertype)  ;
\ *G Transmit a string using UART1.
: sercr1	( -- )		_uart1 (sercr)  ;
\ *G Transmit a CR/LF pair using UART1.


\ **************************
\ *S Generic i/o assignments
\ **************************

cdata
create Console0	\ -- addr
\ *G Function despatch table for UART0.
\ ** OUT is managed by the upper level driver.
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

create Console1	\ -- addr ; OUT managed by upper driver
\ *G Function despatch table for UART1.
\ ** OUT is managed by the upper level driver.
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

console-port 0 = [if]
  Console0 constant Console
[then]
console-port 1 = [if]
  Console1 constant Console
[then]

decimal

