\ Atmel EB55 console definitions, interrupt driven rx, polled tx serial

((
Copyright (c) 2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ ********************
\ Baud rate generation
\ ********************

decimal

: Baud-Rate-Gen \ baud -- CD
  system-speed swap / 16 /
;
interpreter
: Baud-Rate-Gen \ baud -- CD ; can be used while interpreting
  system-speed swap / 16 /
;
target


\ ****************************
\ UART initial register values
\ ****************************

\ USART Inital values for Mode Register
US_CLKS_MCK US_CHRL_8 or US_ASYNCH or US_PAR_NO or US_NBSTOP1 or US_CHMODE_NORMAL or US_MODE8 or US_CLKNO or equ US_INITMODE

\ USART Initial values for Control Register
US_RXDIS US_TXDIS or US_RSTRX or US_RSTTX or US_RSTSTA or equ US_RESET

US_RXEN US_TXEN or equ US_GO

\ USART Initial values for Interrupt Disable Register
$3FF equ US_INTDIS


\ **************************
\ Interrupt Service routines
\ **************************

l: ^AIC
  _AIC ,

cell buffer: rx0-char	\ -- addr ; last character received
cell buffer: rx0-avail	\ -- addr ; character available?

l: ^rx0-char		\ pointers to data
  rx0-char ,
l: ^rx0-avail
  rx0-avail ,
l: ^USART0
   _USART0 ,

proc Ser0ISR	\ -- ; UART 0 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  ldr	r0, ^rx0-char			\ address of data items
  ldr	r1, ^rx0-avail
  ldr	r2, ^USART0
  ldr	r4, ^AIC

  ldr	r3, [ r2, # US_RHR ]		\ read UART RHR register
  str   r3, [ r0 ]			\ save character
  mvn	r3, # 0				\ -1
  str	r3, [ r1 ]			\ set character available flag

  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

cell buffer: rx1-char	\ -- addr ; last character received
cell buffer: rx1-avail	\ -- addr ; character available?

l: ^rx1-char		\ pointers to data
  rx1-char ,
l: ^rx1-avail
  rx1-avail ,
l: ^USART1
   _USART1 ,

proc Ser1ISR	\ -- ; UART 0 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  ldr	r0, ^rx1-char			\ address of data items
  ldr	r1, ^rx1-avail
  ldr	r2, ^USART1
  ldr	r4, ^AIC

  ldr	r3, [ r2, # US_RHR ]		\ read UART RHR register
  str   r3, [ r0 ]			\ save character
  mvn	r3, # 0				\ -1
  str	r3, [ r1 ]			\ set character available flag

  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

cell buffer: rx2-char	\ -- addr ; last character received
cell buffer: rx2-avail	\ -- addr ; character available?

l: ^rx2-char		\ pointers to data
  rx2-char ,
l: ^rx2-avail
  rx2-avail ,
l: ^USART2
   _USART2 ,

proc Ser2ISR	\ -- ; UART 0 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  ldr	r0, ^rx2-char			\ address of data items
  ldr	r1, ^rx2-avail
  ldr	r2, ^USART2
  ldr	r4, ^AIC

  ldr	r3, [ r2, # US_RHR ]		\ read UART RHR register
  str   r3, [ r0 ]			\ save character
  mvn	r3, # 0				\ -1
  str	r3, [ r1 ]			\ set character available flag

  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code


\ **************
\ Initialisation
\ **************

[undefined] init-io [if]
: init-io	\ addr --
\ *G Copy the contents of the I/O set up table to an I/O device.
\ ** Each element of the table is of the form addr (cell) followed
\ ** by data (cell). The table is terminated by an address of 0.
\ ** A table of a single 0 address performs no action.
  begin
    dup @
   while
    dup 2@ !  2 cells +
  repeat
  drop
;
[then]

l: uart0-init-tab	\ -- addr ; hold initialisation data
\ address			data
  rx0-avail ,			0 ,
  rx0-char ,			0 ,
  _APMC_BASE APMC_PCER + ,	$2000 BIT2 + ,	\ Enable peripheral clocks for UART and PIOA
  _pioa_base pio_pdr + ,	$00018000 ,	\ disable PIO for TXD0 and RXD0
  _pioa_base pio_idr + ,	$00018000 ,	\ Disable interrupts for the PIO channels
  _AIC AIC_IDCR + ,		BIT2 ,		\ Disable interrupt
  _AIC AIC_SMR0 + 2 cells + ,	$00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + 2 cells + ,	Ser0ISR ,	\ ISR handler
  _USART0 US_CR + ,		US_RESET , 	\ Reset the USART0 channels
  _USART0 US_MR + ,		US_INITMODE , 	\ Set up initial mode for USART0
  _USART0 US_BRGR + ,				\ Set up baud rate
		  console-speed Baud-Rate-gen ,
  _USART0 US_RTOR + ,		0 , 		\ Disable receiver timeout
  _USART0 US_TTGR + ,		0 , 		\ Disable transmitter time-guard
  _USART0 US_RPR + ,		0 , 		\ Disbale receive pointer register
  _USART0 US_RCR + ,		0 ,		\ Zero receiver counter register
  _USART0 US_TPR + ,		0 ,		\ Disable transmitter pointer register
  _USART0 US_TCR + ,		0 ,		\ Zero transmitter counter register
  _AIC AIC_IECR + ,		BIT2 ,	 	\ Enable the USART0 interrupt
  _USART0 US_IER + ,		$00000001 ,	\ Enable interrupt on RX0
  _USART0 US_CR + ,		US_GO , 	\ GO GO GO
  0 ,						\ end of table

l: uart1-init-tab	\ -- addr ; hold initialisation data
\ address			data
  rx1-avail ,			0 ,
  rx1-char ,			0 ,
  _APMC_BASE APMC_PCER + ,	$2000 BIT3 + ,	\ Enable peripheral clocks for UART and PIOA
  _pioa_base pio_pdr + ,	$000C0000 ,	\ disable PIO for TXD1 and RXD1
  _pioa_base pio_idr + ,	$000C0000 ,	\ Disable interrupts for the PIO channels
  _AIC AIC_IDCR + ,		BIT3 ,		\ Disable interrupt
  _AIC AIC_SMR0 + 3 cells + ,	$00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + 3 cells + ,	Ser1ISR ,	\ ISR handler

  _USART1 US_CR + ,		US_RESET , 	\ Reset the USART channels
  _USART1 US_MR + ,		US_INITMODE , 	\ Set up initial mode for USART
  _USART1 US_BRGR + ,				\ Set up baud rate
		  console-speed Baud-Rate-gen ,
  _USART1 US_RTOR + ,		0 , 		\ Disable receiver timeout
  _USART1 US_TTGR + ,		0 , 		\ Disable transmitter time-guard
  _USART1 US_RPR + ,		0 , 		\ Disbale receive pointer register
  _USART1 US_RCR + ,		0 ,		\ Zero receiver counter register
  _USART1 US_TPR + ,		0 ,		\ Disable transmitter pointer register
  _USART1 US_TCR + ,		0 ,		\ Zero transmitter counter register
  _AIC AIC_IECR + ,		BIT3 ,	 	\ Enable the USART1 interrupt
  _USART1 US_IER + ,		$00000001 ,	\ Enable interrupt on RX
  _USART1 US_CR + ,		US_GO , 	\ Enable TX and RX
  0 ,						\ end of table

l: uart2-init-tab	\ -- addr ; hold initialisation data
\ address			data
  rx2-avail ,			0 ,
  rx2-char ,			0 ,
  _APMC_BASE APMC_PCER + ,	$2000 BIT4 + ,	\ Enable peripheral clocks for UART and PIOA
  _pioa_base pio_pdr + ,	$00600000 ,	\ disable PIO for TXD1 and RXD1
  _pioa_base pio_idr + ,	$00600000 ,	\ Disable interrupts for the PIO channels
  _AIC AIC_IDCR + ,		BIT4 ,		\ Disable interrupt
  _AIC AIC_SMR0 + 4 cells + ,	$00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + 4 cells + ,	Ser2ISR ,	\ ISR handler

  _USART2 US_CR + ,		US_RESET , 	\ Reset the USART channels
  _USART2 US_MR + ,		US_INITMODE , 	\ Set up initial mode for USART
  _USART2 US_BRGR + ,				\ Set up baud rate
		  console-speed Baud-Rate-gen ,
  _USART2 US_RTOR + ,		0 , 		\ Disable receiver timeout
  _USART2 US_TTGR + ,		0 , 		\ Disable transmitter time-guard
  _USART2 US_RPR + ,		0 , 		\ Disbale receive pointer register
  _USART2 US_RCR + ,		0 ,		\ Zero receiver counter register
  _USART2 US_TPR + ,		0 ,		\ Disable transmitter pointer register
  _USART2 US_TCR + ,		0 ,		\ Zero transmitter counter register
  _AIC AIC_IECR + ,		BIT4 ,	 	\ Enable the USART1 interrupt
  _USART2 US_IER + ,		$00000001 ,	\ Enable interrupt on RX
  _USART2 US_CR + ,		US_GO , 	\ Enable TX and RX
  0 ,						\ end of table

: init-ser	\ -- ; initialise on-chip UARTs
  di
  uart0-init-tab init-io
  uart1-init-tab init-io
  uart2-init-tab init-io
  ei
;

: serkey?0i	\ -- flag
  rx0-avail @
;

: serkey?1i	\ -- flag
  rx1-avail @
;

: serkey?2i	\ -- flag
  rx2-avail @
;

: serkey0i	\ -- char
  begin
    [I
    rx0-avail @ 0=
   while
    I]
[ tasking? ] [if]
    pause
[then]
  repeat
  rx0-char @ $FF and
  rx0-avail off
  I]
;

: serkey1i	\ -- char
  begin
    [I
    rx1-avail @ 0=
   while
    I]
[ tasking? ] [if]
    pause
[then]
  repeat
  rx1-char @ $FF and
  rx1-avail off
  I]
;

: serkey2i	\ -- char
  begin
    [I
    rx2-avail @ 0=
   while
    I]
[ tasking? ] [if]
    pause
[then]
  repeat
  rx2-char @ $FF and
  rx2-avail off
  I]
;

: (seremit) 	\ char UBase* -- ; transmit 
  begin
    dup US_CSR + @ US_TXRDY and
  until
  US_THR + !
;

: seremit0	\ char --
  _USART0 (seremit)
;

: seremit1	\ char --
  _USART1 (seremit)
;

: seremit2	\ char --
  _USART2 (seremit)
;

: (sertype)	\ c-addr len base --
  -rot  bounds
  ?do  i c@ over (seremit) loop
  drop
;

: sertype0	\ c-addr len --
  _USART0 (sertype)
;

: sertype1	\ c-addr len --
  _USART1 (sertype)
;

: sertype2	\ c-addr len --
  _USART2 (sertype)
;

: serCR0	\ --
  $0D seremit0  $0A seremit0
;

: serCR1	\ --
  $0D seremit1  $0A seremit1
;

: serCR2	\ --
  $0D seremit2  $0A seremit2
;

cdata
create Console0	\ -- addr 
  ' serkey0i ,
  ' serkey?0i ,
  ' seremit0 ,
  ' sertype0 ,
  ' serCR0 ,

create Console1
  ' serkey1i ,
  ' serkey?1i ,
  ' seremit1 ,
  ' sertype1 ,
  ' serCR1 ,

create Console2
  ' serkey2i ,
  ' serkey?2i ,
  ' seremit2 ,
  ' sertype2 ,
  ' serCR2 ,

console-port 0 = [if]
console0 constant Console	\ -- addr ; select default serial port
[then]

console-port 1 = [if]
console1 constant Console	\ -- addr ; select default serial port
[then]

console-port 2 = [if]
console2 constant Console	\ -- addr ; select default serial port
[then]


