\ Atmel AT91RM9200 console definitions, interrupt driven rx, polled tx serial

((
Copyright (c) 2001, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20061017 SFP001 Converted to AT91RM9200.
20040127 MPE001 Changed MSR instructions to set C and F fields.
))

only forth definitions  decimal

\ ===========
\ *! ser9200qi
\ *T AT91RM9200 queued interrupt receive serial driver
\ ===========
\ *P By default the serial drivers are initialised to 115200
\ ** baud for the DBGU (default console), 38400 for USART0 and
\ ** 9600 baud for USART1..3. These baud rates can be
\ ** changed by setting the relevant UART registers.
\ ** See the chip user manual for more details.

\ ****************
\ *S Configuration
\ ****************
\ *P The following equates control which USARTs are required
\ ** and initialised. If the equates are undefined, they are
\ ** generated and set to 1 for DBGU and 0 for all others.

[undefined] useDBGU? [if]
1 equ useDBGU?	\ -- n
\ *G Compile DBGU code if non-zero.
[then]

[undefined] useUSART0? [if]
0 equ useUSART0?	\ -- n
\ *G Compile USART0 code if non-zero.
[then]

[undefined] useUSART1? [if]
0 equ useUSART1?	\ -- n
\ *G Compile USART1 code if non-zero.
[then]

[undefined] useUSART2? [if]
2 equ useUSART2?	\ -- n
\ *G Compile USART2 code if non-zero.
[then]

[undefined] useUSART3? [if]
0 equ useUSART3?	\ -- n
\ *G Compile USART3 code if non-zero.
[then]


only forth definitions
decimal


\ **************************
\ *S PIO Configuration tools
\ **************************

: IsPeriph	\ Amask Bmask piobase --
\ *G Configure peripheral functions on a PIO port and disable the
\ ** PIO functions.
  3dup tuck 2swap			\ -- am bm p bm p am p
  PIO_ASR + !  PIO_BSR + !		\ set A and B selects
  -rot or swap PIO_ODR + !		\ disable peripheral
;

: IsPIOout	\ mask piobase --
\ *G Configure PIO pins as outputs.
  2dup PIO_PER + !  PIO_OER + !
;

: IsPIOin	\ mask piobase --
\ *G Configure PIO pins as inputs.
  2dup PIO_ODR + !  PIO_PER + !
;


\ ***********************
\ *S Baud rate generators
\ ***********************

: GenBAUD	\ baud -- CD
\ *G Calculate the baud rate. Can also be interpreted.
  /MCK swap / 16 /mod  swap 7 >
  if  1+  endif
;
interpreter
: GenBAUD	\ baud -- CD
  /MCK swap / 16 /mod  swap 7 >
  if  1+  endif
;
target


\ *************************
\ *S USART masks for 8N1 use, clk = MCK
\ *************************

US_CLKS_MCK  US_CHRL_8 or  US_ASYNCH or US_PAR_NO or US_NBSTOP1 or
US_CHMODE_NORMAL or US_MODE8 or US_CLKNO or
  equ US_INITMODE	\ -- mask
\ *G USART inital values for Mode Register.

US_RXDIS  US_TXDIS or  US_RSTRX or  US_RSTTX or  US_RSTSTA or
  equ US_RESET		\ -- mask
\ *G USART inital value for RESET operation in Control Register.

US_RXEN US_TXEN or
  equ US_GO		\ -- mask
\ *G USART inital value for START operation in Control Register.

$000F3FFF equ US_INTDIS	\ -- mask
\ *G USART initial value for Interrupt Disable Register


\ ***********************
\ *S Common polled output
\ ***********************

: (seremit) 	\ char UBase* -- ; transmit 
  begin
    dup US_CSR + @ US_TXRDY and
  until
  US_THR + !
;

: (sertype)	\ c-addr len base --
  -rot  bounds
  ?do  i c@ over (seremit) loop
  drop
;

: (serCR)	\ base --
  #13 over (seremit)  #10 swap (seremit)
;



\ *********************************
\ *S ISRS and initialisation tables
\ *********************************

l: ^AIC
  _AIC ,

useDBGU? [if]
#256 cqueue: SerQiD	\ -- addr
\ *G Input queue for Debug UART.

: ISR-KeyD	\ -- ; high level ISR
\ *G Handler for Debug UART.
  SerQiD (cqFull?) if
    _DBGU US_RHR + @ drop
  else
    _DBGU US_RHR + @ SerQiD (>cqueue)
  then
;

' ISR-KeyD IRQ: SerDISR
\ *G ISR for Debug UART.

$C0000000 equ DBGUpins
\ *G Pin mask in PIOA for the TXD and RXD pins.

l: initDBGUtab	\ -- addr
\ *G Debug UART initialisation table
  _PMC PMC_PCER + ,		SYSmask ,	\ power up system peripherals, inc DBGU
  _PIOA PIO_ASR + ,		DBGUpins ,	\ PIOA bits 31/30 are periph A
  _PIOA PIO_BSR + ,		0 ,
  _PIOA PIO_ODR + ,		DBGUpins ,	\ disable PIO outputs
  _PIOA PIO_IDR + ,		DBGUpins ,	\ disable PIO interrupt
  _PIOA PIO_PDR + ,		DBGUpins ,	\ disable PIO
  _AIC AIC_IDCR + ,		SYSmask ,	\ Disable interrupt
  _AIC AIC_SMR0 + ID_SYS cells + ,  $00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + ID_SYS cells + ,  SerdISR ,	\ ISR handler
  _DBGU US_CR + ,		US_RESET ,	\ reset UART
  _DBGU US_MR + ,		US_INITMODE ,	\ mode register
  _DBGU US_IDR + ,		US_INTDIS ,	\ disable interrupts
  _DBGU US_BRGR + ,	Console0-speed GenBAUD , \ baud rate
  _DBGU US_IER + ,		$00000001 ,	\ Enable interrupt on RX0
  _AIC AIC_IECR + ,		SYSmask , 	\ Enable the DBGU interrupt
  _DBGU US_CR + ,		US_GO ,		\ enable TX and RX
  0 ,						\ end of table

: serkey?D	\ -- flag
\ *G *\fo{KEY?} for Debug UART.
  SerQiD cqempty? 0=
;

: serKeyD	\ -- char
\ *G *\fo{KEY} for Debug UART.
  SerQiD cqueue>
;

: seremitD	\ char --
\ *G *\fo{EMIT} for Debug UART.
  _DBGU (seremit)
;

: sertypeD	\ c-addr len --
\ *G *\fo{TYPE} for Debug UART.
  _DBGU (sertype)
;

: serCRD	\ --
\ *G *\fo{CR} for Debug UART.
  _DBGU (sercr)
;

cdata create ConsoleD	\ -- addr 
\ *G Device structure for debug UART.
  ' serkeyD ,
  ' serkey?D ,
  ' seremitD ,
  ' sertypeD ,
  ' serCRD ,
[then]

useUSART0? [if]
#256 cqueue: SerQi0
\ *G Input queue for USART0.

: ISR-Key0	\ -- ; high level ISR
\ *G Handler for USART0.
  SerQi0 (cqFull?) if
    _USART0 US_RHR + @ drop
  else
    _USART0 US_RHR + @ SerQi0 (>cqueue)
  then
;

' ISR-Key0 IRQ: Ser0ISR
\ *G ISR for USART0.

$00060000 equ US0pins
\ *G Pin mask in PIOA for the USART0 TXD and RXD pins - PA17/18.

l: InitUSART0tab	\ -- addr ; hold initialisation data
\ address			data
  _PMC_BASE PMC_PCER + ,	US0mask ,	\ Enable peripheral clocks for UART and PIOA
  _PIOA PIO_ASR + ,		US0pins ,	\ PIOA bits 18/17 are periph A
  _PIOA PIO_BSR + ,		0 ,
  _PIOA PIO_ODR + ,		US0pins ,	\ disable PIO outputs
  _PIOA PIO_IDR + ,		US0pins ,	\ disable PIO interrupt
  _PIOA PIO_PDR + ,		US0pins ,	\ disable PIO
  _AIC AIC_IDCR + ,		US0mask ,	\ Disable interrupt
  _AIC AIC_SMR0 + ID_US0 cells + ,  $00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + ID_US0 cells + ,  Ser0ISR ,	\ ISR handler
  _USART0 US_CR + ,		US_RESET , 	\ Reset the USART0 channels
  _USART0 US_MR + ,		US_INITMODE , 	\ Set up initial mode for USART0
  _USART0 US_BRGR + ,				\ Set up baud rate
		  console1-speed Baud-Rate-gen ,
  _USART0 US_RTOR + ,		0 , 		\ Disable receiver timeout
  _USART0 US_TTGR + ,		0 , 		\ Disable transmitter time-guard
  _USART0 US_RPR + ,		0 , 		\ Disbale receive pointer register
  _USART0 US_RCR + ,		0 ,		\ Zero receiver counter register
  _USART0 US_TPR + ,		0 ,		\ Disable transmitter pointer register
  _USART0 US_TCR + ,		0 ,		\ Zero transmitter counter register
  _AIC AIC_IECR + ,		US0mask , 	\ Enable the USART0 interrupt
  _USART0 US_IER + ,		$00000001 ,	\ Enable interrupt on RX0
  _USART0 US_CR + ,		US_GO , 	\ GO GO GO
  0 ,						\ end of table

: serkey?0	\ -- flag
\ *G *\fo{KEY?} for USART0.
  SerQi0 cqempty? 0=
;

: serKey0	\ -- char
\ *G *\fo{KEY} for USART0.
  SerQi0 cqueue>
;

: seremit0	\ char --
\ *G *\fo{EMIT} for USART0.
  _USART0 (seremit)
;

: sertype0	\ c-addr len --
\ *G *\fo{TYPE} for USART0.
  _USART0 (sertype)
;

: serCR0	\ --
\ *G *\fo{CR} for USART0.
  _USART0 (sercr)
;

cdata create Console0	\ -- addr
\ *G Device structure for USART0.
  ' serkey0 ,
  ' serkey?0 ,
  ' seremit0 ,
  ' sertype0 ,
  ' serCR0 ,
[then]

((
useUSART1? [if]
#256 cqueue: SerQi1
\ *G Input queue for USART1.

: ISR-Key1	\ -- ; high level ISR
\ *G Handler for USART1.
  SerQi1 (cqFull?) if
    _USART1 US_RHR + @ drop
  else
    _USART1 US_RHR + @ SerQi1 (>cqueue)
  then
;

' ISR-Key1 IRQ: Ser1ISR
\ *G ISR for USART1.

$00300000 equ US1pins
\ *G Pin mask in PIOB for the USART1 TXD and RXD pins - PB20/21.

: serkey?1qi	\ -- flag
\ *G *\fo{KEY?} for USART1.
  SerQi1 cqempty? 0=
;

: serKey1qi	\ -- char
\ *G *\fo{KEY} for USART0.
  SerQi1 cqueue>
;

: seremit1	\ char --
\ *G *\fo{EMIT} for USART1.
  _USART1 (seremit)
;

: sertype1	\ c-addr len --
\ *G *\fo{TYPE} for USART1.
  _USART1 (sertype)
;

: serCR1	\ --
\ *G *\fo{CR} for USART1.
  _USART1 (sercr)
;

cdata create Console1	\ -- addr
\ *G Device structure for USART1.
  ' serkey1 ,
  ' serkey?1 ,
  ' seremit1 ,
  ' sertype1 ,
  ' serCR1 ,
[then]

useUSART2? [if]
#256 cqueue: SerQi2
\ *G Input queue for USART2.

: ISR-Key2	\ -- ; high level ISR
\ *G Handler for USART2.
  SerQi2 (cqFull?) if
    _USART2 US_RHR + @ drop
  else
    _USART2 US_RHR + @ SerQi2 (>cqueue)
  then
;

' ISR-Key2 IRQ: Ser2ISR
\ *G ISR for USART2.

$01800000 equ US2pins
\ *G Pin mask in PIOA for the USART2 TXD and RXD pins - PA23/24.

: serkey?2	\ -- flag
\ *G *\fo{KEY?} for USART2.
  SerQi2 cqempty? 0=
;

: serKey2	\ -- char
\ *G *\fo{KEY} for USART2.
  SerQi2 cqueue>
;

: seremit2	\ char --
\ *G *\fo{EMIT} for USART2.
  _USART2 (seremit)
;

: sertype2	\ c-addr len --
\ *G *\fo{TYPE} for USART2.
  _USART2 (sertype)
;

: serCR2	\ --
\ *G *\fo{CR} for USART2.
  _USART2 (sercr)
;

cdata create Console2	\ -- addr
\ *G Device structure for USART2.
  ' serkey2 ,
  ' serkey?2 ,
  ' seremit2 ,
  ' sertype2 ,
  ' serCR2 ,
[then]

useUSART3? [if]
#256 cqueue: SerQi3
\ *G Input queue for USART3.

: ISR-Key3	\ -- ; high level ISR
\ *G Handler for USART3.
  SerQi3 (cqFull?) if
    _USART3 US_RHR + @ drop
  else
    _USART3 US_RHR + @ SerQi3 (>cqueue)
  then
;

' ISR-Key3 IRQ: Ser3ISR
\ *G ISR for USART3.

$00000060 equ US3pins
\ *G Pin mask in PIOA for the USART0 TXD and RXD pins - PA5/6.

: serkey?3	\ -- flag
\ *G *\fo{KEY?} for USART3.
  SerQi3 cqempty? 0=
;

: serKey3	\ -- char
\ *G *\fo{KEY} for USART3.
  SerQi3 cqueue>
;

: seremit3	\ char --
\ *G *\fo{EMIT} for USART3.
  _USART3 (seremit)
;

: sertype3	\ c-addr len --
\ *G *\fo{TYPE} for USART3.
  _USART3 (sertype)
;

: serCR3	\ --
\ *G *\fo{CR} for USART3.
  _USART3 (sercr)
;

cdata create Console3	\ -- addr
\ *G Device structure for USART3.
  ' serkey3 ,
  ' serkey?3 ,
  ' seremit3 ,
  ' sertype3 ,
  ' serCR3 ,
[then]
))

: init-ser	\ --
\ *G Initialise on-chip UARTs.
  di
[ useDBGU? ] [if]
  serQiD init-cqueue  InitDBGUtab init-io
[then]
[ useUSART0? ] [if]
  serQi0 init-cqueue  InitUSART0tab init-io
[then]
[ useUSART1? ] [if]
  serQi1 init-cqueue  InitUSART1tab init-io
[then]
[ useUSART2? ] [if]
  serQi2 init-cqueue  InitUSART2tab init-io
[then]
[ useUSART3? ] [if]
  serQi3 init-cqueue  InitUSART3tab init-io
[then]
  ei
;

console-port 0 = [if]
  consoleD constant Console	\ -- addr ; select default serial port
[then]

console-port 1 = [if]
  console0 constant Console	\ -- addr ; select default serial port
[then]

console-port 2 = [if]
  console1 constant Console	\ -- addr ; select default serial port
[then]

console-port 3 = [if]
  console2 constant Console	\ -- addr ; select default serial port
[then]

console-port 4 = [if]
  console3 constant Console	\ -- addr ; select default serial port
[then]


