\ Analog Devices ADuC702x polled RX, polled TX, serial communication drivers

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

only forth definitions

\ =============
\ *! serad702xp
\ =============
\ *T Serial driver
\ *P By default the serial driver is initialised to 9600
\ ** baud for the serial console. ese baud rate can be
\ ** changed by setting the relevant UART DLL and DLM registers.
\ ** See the chip user manual for more details.


\ *****************
\ *S Initialisation
\ *****************

0 equ use-frac?
\ *G Set this non-zero to use the fractional baud rate generator.
\ ** Note that on current silicon (September 2004) the fractional
\ ** baud rate generator does not work and should not be used. See
\ ** errata ER010 in the current errate sheet.

use-frac? [if]

interpreter
: gen-baud-rate	\ bps-rate -- divisor16
\ *G Takes the the required serial comms rate (in BPS) and returns
\ ** the primary divisor value for the baud rate generator.
\ *E                    system-speed
\ ** divisor =      --------------------
\ **                2^CD * 32 * baudrate
  system-speed swap /			\ 45.088Mhz/baud
  2^CD / #32 /				\ if fractional divider in use
;

: gen-frac	\ reqbaud divisor -- frac
\ *G Takes the required baud rate and the divisor, and calculates
\ ** the required fraction value for M=1. From the data sheet:
\ *E       N           system-speed
\ ** 1 + ---- = --------------------------------
\ **     2048   req-baud * 2^CD * 32 * u0-divisor
\ ** 
\ ** let  denom = req-baud * 2^CD * 32 * u0-divisor
\ ** 
\ **   N    systemspeed - denom
\ ** ---- = ------------------
\ ** 2048        denom
\ ** 
\ **     (systemspeed - denom) * 2048
\ ** N = ----------------------------
\ **               denom
  * 2^CD * #32 *			\ -- denom
  system-speed over - #2048 rot */
;
target

[else]

interpreter
: gen-baud-rate	\ bps-rate -- divisor16
  system-speed swap /			\ 45.088Mhz/baud
  2^CD / #32 /mod			\ if fractional divider not in use
  swap #16 >=				\ rounding check
  if  1+  endif
;

: gen-frac	\ reqbaud divisor -- frac
  2drop 0
;
target

[then]


console0-speed gen-baud-rate
  equ u0-divisor
console0-speed u0-divisor gen-frac
  equ u0-frac
u0-frac $8800 or
  equ ComDiv2val

\ internal

: init-ser	\ --
\ *G Initialise serial port.
  _GPIO dup Gp1Con + @
    $FF invert and  $11 or		\ P1.0/1 -> Uart
  swap Gp1Con + !

  _UART
  $03 $80 or           over ComCon0 + !	\ 8 bit, 1 stop bit, no parity, enable DLAB
  u0-divisor 8 rshift  over ComDiv1 + !	\ divisor high
  u0-divisor $FF and   over ComDiv0 + !	\ divisor low
  $03                  over ComCon0 + !	\ 8 bit, 1 stop bit, no parity, disable DLAB
[ use-frac? ] [if]
  ComDiv2val           over ComDiv2 + ! \ fraction enable
[then]
  drop
;


\ ********************
\ *S Serial primitives
\ ********************

: seremit	\ char --
\ *G Transmit a character on UART.
  _UART
  begin
    dup ComSta0 + @ %00100000 and
  until
  ComTx + !
;

: serkey?	\ -- t/f
\ *G Return true if the UART has a character available.
  _UART ComSta0 + @ %00000001 and 0<>
;

tasking? [if]
: serkey	\ -- char
\ Wait for a character on the UART and return it.
  _UART
  begin
    dup ComSta0 + @ %00000001 and 0=
   while
    pause
  repeat
  ComRx + @ $FF and
;
[else]
: serkey	\ -- char
  _UART
  begin
    dup ComSta0 + @ %00000001 and
  until
  ComRx + @ $FF and
;
[then]

\ external

: serTYPE	\ c-addr len --
\ *G Type a string to the UART.
  bounds
  ?do  i c@ seremit  loop
;

: serCR		\ --
\ *G Issue a CR/LF pair to the UART.
  $0D seremit  $0A seremit
;


\ **************************
\ *S Generic i/o assignments
\ **************************

cdata
create Console0	\ -- addr ;
\ *G Generic I/O device for the UART. *\fo{OUT} is managed
\ ** by the upper level driver in KERNEL62.FTH.
  ' serkey ,		\ -- char ; receive char
  ' serkey? ,		\ -- flag ; check receive char
  ' seremit ,		\ -- char ; display char
  ' sertype ,		\ caddr len -- ; display string
  ' sercr ,		\ -- ; display new line

console-port 0 = [if]
  console0 constant console
\ *G *\fo{CONSOLE} is the device used by the Forth system for
\ ** interaction. It may be changed by a phrase of the form:
\ *C   <device>  dup opvec !  ipvec !
[then]

decimal


\ ======
\ *> ###
\ ======

decimal

