\ SERLPC210XP.FTH - LPC210x polled RX, polled TX, serial communication drivers

((
Copyright (c) 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

only forth definitions


\ **************
\ Initialisation
\ **************

\ This next word takes the system clock rate (in HZ) and the required serial
\ comms rate (in BPS) and returns a system-dependent number of parameters that
\ should be sufficent to effect the baud rate selection by initialising the
\ correct hardware.

\ divisor =          system-speed
\                -------------------
\                16 * vpb * baudrate

interpreter
: gen-baud-rate	\ bps-rate -- divisor16
  system-speed /vpb / swap / #16 /mod
  swap 8 >=
  if  1+  endif
;
target

console0-speed gen-baud-rate
  equ u0-divisor
console1-speed gen-baud-rate
  equ u1-divisor

internal

: init-ser0	\ --
\ *G Initialise serial port 0
  bit3 _PCONP or!			\ enable clock
  _PINSEL0 dup @			\ enable pins
    $0F invert and  $5 or
  swap !
  1                   _U0FCR !
  $03 $80 or          _U0LCR !		\ 8 bit, 1 stop bit, no parity, enable DLAB
  u0-divisor 8 rshift _U0DLM !
  u0-divisor $FF and  _U0DLL !
  $03                 _U0LCR !
;

: init-ser1	\ --
\ *G Initialise serial port 1
  bit4 _PCONP or!			\ enable clock
  _PINSEL0 dup @			\ enable pins
    $000F0000 invert and  $00050000 or
  swap !
  1                   _U1FCR !
  $03 $80 or          _U1LCR !		\ 8 bit, 1 stop bit, no parity, enable DLAB
  u1-divisor 8 rshift _U1DLM !
  u1-divisor $FF and  _U1DLL !
  $03                 _U1LCR !
;

: init-ser	\ --
\ *G Initialise both serial ports
  init-ser0  init-ser1
;

compiler
: +FaultConsole  ;
\ *G For a polled serial driver, *\fo{+FaultConsole} is a *\fo{NOOP}.
target


\ *****************
\ Serial primitives
\ *****************

: (seremit)	\ char base --
  begin
    dup UnLSR + @ %00100000 and
  until
  UnTHR + !
;

: (serkey?)	\ base -- t/f
  UnLSR + @ %00000001 and 0<>
;

: (serkey)	\ base -- char
  begin
[ tasking? ] [if]  pause  [then]
    dup UnLSR + @ %00000001 and
  until
  UnRBR + @ $FF and
;

external

: seremit0	\ char --
  _UART0 (seremit)
;

: seremit1	\ char --
  _UART1 (seremit)
;

: serkey?0	\ -- flag
  _UART0 (serkey?)
;

: serkey?1	\ -- flag
  _UART1 (serkey?)
;

: serkey0	\ -- char
  _UART0 (serkey)
;

: serkey1	\ -- char
  _UART1 (serkey)
;

: serTYPE0	\ c-addr len --
  bounds
  ?do  i c@ seremit0  loop
;

: serTYPE1	\ c-addr len --
  bounds
  ?do  i c@ seremit1  loop
;

: serCR0	\ --
  $0D seremit0  $0A seremit0
;

: serCR1	\ --
  $0D seremit1  $0A seremit1
;


\ ***********************
\ Generic i/o assignments
\ ***********************

cdata
create Console0	\ -- addr ; OUT managed by upper driver
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

create Console1	\ -- addr ; OUT managed by upper driver
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

console-port 0 = [if]
  console0 constant console
[then]

console-port 1 = [if]
  console1 constant console
[then]

decimal

