\ SerLPC23xxp.fth - LPC23xx polled RX, polled TX, serial communication drivers

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20131208 SFP001 Fixed INIT-SERx.
))

only forth definitions


\ =============
\ *! serlpc23xx
\ *T LPC23xx serial driver
\ =============
\ *P This driver provides polled serial comms for LPC23xx CPUs
\ ** using the internal UARTs. Because there are alternate pin
\ ** assignments for some UARTs, you *\b{must} check that the
\ ** supplied code pin setup is correct for your board.


\ ********
\ *S Tools
\ ********

[undefined] setMask [if]
: setMask	\ value mask addr --
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;
[then]


\ *P *\fo{GenBaudRate} uses the system clock rate (in Hz), the
\ ** perpheral clock divider ration, and the required serial speed
\ ** (in BPS) and returns the 16 bit divider value for the baud
\ ** rate generator
\ *E              system-speed
\ ** divisor = -------------------
\ **           16 * pcd * baudrate

interpreter
: GenBaudRate	\ bps-rate pcd -- divisor16
\ *G Given a baud rate and the peripheral clock divider
\ ** (1, 2, 4, 8), calculate the 16 bit baud rate divisor.
\ **
  system-speed swap / swap / #16 /mod
  swap 8 >=
  if  1+  endif
;
target


\ ********************
\ *S Serial primitives
\ ********************

internal

: +FaultConsole	( -- )  ;
\ *G Because this is a polled driver, *\fo{+FaultConsole} for
\ ** fault handling is a *\fo{NOOP}. See *\i{ARM/IntARM4.fth}
\ ** for more details.

: (seremit)	\ char base --
  begin
    dup UnLSR + c@ %00100000 and
  until
  UnTHR + c!
;

: (sertype)	\ caddr len base --
  -rot bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (sercr)	\ base --
  $0D over (seremit)  $0A swap (seremit)
;

: (serkey?)	\ base -- t/f
  UnLSR + c@ %00000001 and 0<>
;

: (serkey)	\ base -- char
  begin
[ tasking? ] [if]  pause  [then]
    dup UnLSR + c@ %00000001 and
  until
\ UnRBR + @ $FF and
  UnRBR + c@
;

external

useUART0? [if]
1 equ pcdU0	\ -- n
\ *G Peripheral clock division ratio for UART0.
console0-speed pcdU0 GenBaudRate equ u0-divisor	\ -- n
\ *G UART0 bad rate divisor.
pcdu0 PCLKdiv 6 lshift equ /pcdu0	\ -- mask
\ *G Bit mask for UART0 peripheral clock divider.
: init-ser0	\ --
\ *G Initialise serial port 0
  bit3 _PCONP or!				\ enable clock & pwr
  /pcdU0 $00C0 _SYSCON PCLKSEL0 + setMask \ set peripheral clock speed
  $0050 $00F0 _PINSEL0 setMask		\ PINSEL0 bits 7:4
  _UART0
    1 over UnFCR + !
    $83 over UnLCR + !			\ 8 bit, 1 stop bit, no parity, enable DLAB
    u0-divisor 8 rshift over UnDLM + !
    u0-divisor $FF and over UnDLL + !
    $03 over UnLCR + !
  drop
;
: seremit0	\ char --
  _UART0 (seremit)  ;
: sertype0	\ c-addr len --
  _UART0 (sertype)  ;
: sercr0	\ --
  _UART0 (sercr)  ;
: serkey?0	\ -- flag
  _UART0 (serkey?)  ;
: serkey0	\ -- char
  _UART0 (serkey)  ;
create Console0	\ -- addr ; OUT managed by upper driver
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line
console-port 0 = [if]
  console0 constant console
[then]
[then]

useUART1? [if]
1 equ pcdU1	\ -- n
\ *G Peripheral clock division ratio for UART.
console1-speed pcdU1 GenBaudRate equ u1-divisor	\ -- n
\ *G UART baud rate divisor.
pcdu1 PCLKdiv 8 lshift equ /pcdu1	\ -- mask
\ *G Bit mask for UART peripheral clock divider.
: init-ser1	\ --
\ *G Initialise serial port 1
  bit4 _PCONP or!			\ enable clock & pwr
  /pcdU1 $0300 _SYSCON PCLKSEL0 + setMask \ set peripheral clock speed
  $000A $000F _PINSEL4 setMask		\ TXD1, mode 2 ; SFPxxx
  _UART1
    1 over UnFCR + !
    $83 over UnLCR + !			\ 8 bit, 1 stop bit, no parity, enable DLAB
    u1-divisor 8 rshift over UnDLM + !
    u1-divisor $FF and over UnDLL + !
    $03 over UnLCR + !
  drop
;
: seremit1	\ char --
  _UART1 (seremit)  ;
: sertype1	\ c-addr len --
  _UART1 (sertype)  ;
: sercr1	\ --
  _UART1 (sercr)  ;
: serkey?1	\ -- flag
  _UART1 (serkey?)  ;
: serkey1	\ -- char
  _UART1 (serkey)  ;

create Console1	\ -- addr ; OUT managed by upper driver
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

console-port 1 = [if]
  console1 constant console
[then]
[then]

useUART2? [if]
1 equ pcdU2	\ -- n
\ *G Peripheral clock division ratio for UART.
console2-speed pcdU2 GenBaudRate equ u2-divisor	\ -- n
\ *G UART baud rate divisor.
pcdu2 PCLKdiv #16 lshift equ /pcdu2	\ -- mask
\ *G Bit mask for UART peripheral clock divider.
: init-ser2	\ --
\ *G Initialise serial port 0
  bit24 _PCONP + or!				\ enable clock & pwr
  /pcdU2 $30000 _SYSCON PCLKSEL1 + setMask	\ set peripheral clock speed
  $0050:0000 $00F0:0000 _PINSEL0 setMask	\ PINSEL0 bits 23:20 ; SFPxxx
  _UART2
    1 over UnFCR + !
    $83 over UnLCR + !			\ 8 bit, 1 stop bit, no parity, enable DLAB
    u2-divisor 8 rshift over UnDLM + !
    u2-divisor $FF and over UnDLL + !
    $03 over UnLCR + !
  drop
;
: seremit2	\ char --
  _UART2 (seremit)  ;
: sertype2	\ c-addr len --
  _UART2 (sertype)  ;
: sercr2	\ --
  _UART2 (sercr)  ;
: serkey?2	\ -- flag
  _UART2 (serkey?)  ;
: serkey2	\ -- char
  _UART2 (serkey)  ;

create Console2	\ -- addr ; OUT managed by upper driver
  ' serkey2 ,		\ -- char ; receive char
  ' serkey?2 ,		\ -- flag ; check receive char
  ' seremit2 ,		\ -- char ; display char
  ' sertype2 ,		\ caddr len -- ; display string
  ' sercr2 ,		\ -- ; display new line

console-port 2 = [if]
  console2 constant console
[then]
[then]

useUART3? [if]
1 equ pcdU3	\ -- n
\ *G Peripheral clock division ratio for UART.
console3-speed pcdU3 GenBaudRate equ u3-divisor	\ -- n
\ *G UART baud rate divisor.
pcdu3 PCLKdiv #18 lshift equ /pcdu3	\ -- mask
\ *G Bit mask for UART peripheral clock divider.
: init-ser3	\ --
\ *G Initialise serial port 3.
  bit25 _PCONP + or!				\ enable clock & pwr
  /pcdU2 $C0000 _SYSCON PCLKSEL1 + setMask	\ set peripheral clock speed
  $003C:0000 $003C:0000 _PINSEL1 + setMask	\ PINSEL1 bits 21:18, mode 3
  _UART3
    1 over UnFCR + !
    $83 over UnLCR + !			\ 8 bit, 1 stop bit, no parity, enable DLAB
    u2-divisor 8 rshift over UnDLM + !
    u2-divisor $FF and over UnDLL + !
    $03 over UnLCR + !
  drop
;
: seremit3	\ char --
  _UART3 (seremit)  ;
: sertype3	\ c-addr len --
  _UART0 (sertype)  ;
: sercr3	\ --
  _UART3 (sercr)  ;
: serkey?3	\ -- flag
  _UART3 (serkey?)  ;
: serkey3	\ -- char
  _UART3 (serkey)  ;

create Console3	\ -- addr ; OUT managed by upper driver
  ' serkey3 ,		\ -- char ; receive char
  ' serkey?3 ,		\ -- flag ; check receive char
  ' seremit3 ,		\ -- char ; display char
  ' sertype3 ,		\ caddr len -- ; display string
  ' sercr3 ,		\ -- ; display new line

console-port 3 = [if]
  console3 constant console
[then]
[then]


: init-ser	\ --
\ *G Initialise all serial ports
  [ useUART0? ] [if]  init-ser0  [then]
  [ useUART1? ] [if]  init-ser1  [then]
  [ useUART2? ] [if]  init-ser2  [then]
  [ useUART3? ] [if]  init-ser3  [then]
;


\ ======
\ *> ###
\ ======

decimal
