\ SA1110 polled serial I/O handling

((
Copyright (c) 2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE

20000608 SFP002 Converted to v6.1 generic I/O model
))

only forth definitions  decimal


((
Baud rate calculations
======================

The required baud rate setting BRD is caculated by:

    xtal-speed
    ---------- - 1
      16*baud

where the xtal-speed is normally 3.686400 MHz.

The low 12 bits of BRD are significant. The upper 4 are placed in UTCR1
and the low 8 bits in UTCR2.
))

: gen-brd	\ baud -- utcr2 utcr1
\ *G The given baud rate in bits per second, e.g. 38400, is converted
\ ** into the initial values of BRD for UTCR1 and UTCR2. An interpreted
\ ** version of this word is also available.
  #16 * xtal-speed swap / 1 -
  dup $0FF and				\ UTCR2
  swap 8 rshift $00F and		\ UTCR1
;
interpreter
: gen-brd	\ baud -- utcr2 utcr1
  #16 * xtal-speed swap / 1 -
  dup $0FF and				\ UTCR2
  swap 8 rshift $00F and		\ UTCR1
;
target

\ Initialisation and tables
: init-io	\ addr --
\ *G Copy the contents of the I/O set up table to an I/O device.
\ ** Each element of the table is of the form addr (cell) followed
\ ** by data (cell). The table is terminated by an address of 0.
\ ** A table of a single 0 address performs no action.
  begin
    dup @
   while
    dup 2@ !  2 cells +
  repeat
  drop
;

l: uart3-setup	\ -- addr
\ *G Initialisation table for UART3 - usually the debug UART
  _uart3 utcr3 + ,	0 ,		\ disable it
  _uart3 utcr0 + ,      UART3_UTCR0 ,	\ 8 bit, no parity, 1 stop 
  #38400 gen-brd			\ define baud rate
  _uart3 utcr1 + ,	,
  _uart3 utcr2 + ,	,
  _uart3 utsr0 + ,	$0FF ,		\ reset all bits
\  _uart3 utsr1 + ,	$0FF ,		\ reset all bits
  _uart3 utdr + ,	0 ,		\ write dummy character!
  _uart3 utcr3 + ,	UTCR3_TXE UTCR3_RXE or ,	\ enable TX and RX straight to the UART pins
  0 ,					\ end of table

hex


: INIT-SER	\ -- ; initialise serial line(s)
\ *G Initialise the serial line(s).
  uart3-setup init-io
  #100000 0 do loop			\ waste a character time - ICACHE ON
\  #10000 0 do loop			\ waste a character time - ICACHE OFF
;

: (serkey?)	\ base -- t/f
\ *G Return true if a character has been received by the UART whose
\ ** base address is given.
  utsr1 + @ UTSR1_RNE and 0<>
;

tasking? [if]
: (serkey)	\ base -- char
  begin
\ *G Wait until a character has been received by the UART whose
\ ** base address is given, and return the character.
    dup utsr1 + @ UTSR1_RNE and 0=
   while
    pause
  repeat
  utdr + @ ( $FF and )
;
[else]
: (serkey)	\ base -- char
  begin
    dup utsr1 + @ UTSR1_RNE and
  until
  utdr + @ ( $FF and )
;
[then]

: (seremit)	\ char base --
\ *G Send the character to the UART whose base address is given.
  begin
    dup utsr1 + @ UTSR1_TNF and
  until
  utdr + !
;

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (serCR)	\ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D over (seremit)  $0A swap (seremit)
;


\ ***********************
\ Generic i/o assignments
\ ***********************

: serkey3	( -- char )	_UART3 (serkey)  ;
: serkey?3	( -- flag )	_UART3 (serkey?)  ;
: seremit3	( char -- )	_UART3 (seremit)  ;
: sertype3	( addr len -- )	_UART3 (sertype)  ;
: sercr3	( -- )		_UART3 (sercr)  ;

((
0 ipfunc KEY	\ -- char ; receive char
1 ipfunc KEY?	\ -- flag ; check receive char
2 opfunc EMIT	\ -- char ; display char
3 opfunc TYPE	\ caddr len -- ; display string
4 opfunc CR	\ -- ; display new line
5 opfunc TYPEC	\ caddr len -- ; display string in code space (Harvard only)
))
cdata
create Console3	\ -- addr ; OUT managed by upper driver
  ' serkey3 ,		\ -- char ; receive char
  ' serkey?3 ,		\ -- flag ; check receive char
  ' seremit3 ,		\ -- char ; display char
  ' sertype3 ,		\ caddr len -- ; display string
  ' sercr3 ,		\ -- ; display new line

decimal

Console3 constant Console

