\ SA1110 consoles on UARTs 1..3, queued interrupt driven rx, polled tx serial

((
Copyright (c) 2001, 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20020430 SFP003 SA1110 version
20000608 SFP002 Converted to v6.1 generic I/O model
))

only forth definitions  decimal


((
Baud rate calculations
======================

The required baud rate setting BRD is caculated by:

    xtal-speed
    ---------- - 1
      16*baud

where the xtal-speed is normally 3.686400 MHz.

The low 12 bits of BRD are significant. The upper 4 are placed in UTCR1
and the low 8 bits in UTCR2.
))

: gen-brd	\ baud -- utcr2 utcr1
\ *G The given baud rate in bits per second, e.g. 38400, is converted
\ ** into the initial values of BRD for UTCR1 and UTCR2. An interpreted
\ ** version of this word is also available.
  #16 * xtal-speed swap / 1 -
  dup $0FF and				\ UTCR2
  swap 8 rshift $00F and		\ UTCR1
;
interpreter
: gen-brd	\ baud -- utcr2 utcr1
  #16 * xtal-speed swap / 1 -
  dup $0FF and				\ UTCR2
  swap 8 rshift $00F and		\ UTCR1
;
target


\ *************************
\ Interrupt Service Routine(s)
\ *************************

#256 cqueue: SerQ1	\ -- addr ; circular byte queue
#256 cqueue: SerQ2	\ -- addr ; circular byte queue
#256 cqueue: SerQ3	\ -- addr ; circular byte queue

: ReadChars	\ uart queue --
\ *G Read characters from UART into queue.
  >r
  begin
    dup utsr1 + @ UTSR1_RNE and		\ more to get?
   while
    r@ (cqFull?) if			\ if queue full
      dup utdr + @ drop			\  discard character
    else
      dup utdr + @ r@ (>cqueue)		\  queue character
    then
  repeat
  drop  r> drop
;

: ISR-Keys	\ --
  _UART1 SerQ1 ReadChars
  _UART2 SerQ2 ReadChars
  _UART3 SerQ3 ReadChars
;
' ISR-Keys add-irq


\ *************************
\ Initialisation and tables
\ *************************

l: uart1-setup	\ -- addr
\ *G Initialisation table for UART1.
  _gpclk gpclkr0 ,	1 ,		\ set SUS=1
  _uart1 utcr3 + ,	0 ,		\ disable it
  _uart1 utcr0 + ,      UART1_UTCR0 ,	\ 8 bit, no parity, 1 stop 
  console1-speed gen-brd		\ define baud rate
  _uart1 utcr1 + ,	,
  _uart1 utcr2 + ,	,
  _uart1 utsr0 + ,	$0FF ,		\ reset all bits
  _uart1 utdr + ,	0 ,		\ write dummy character!
  _uart1 utcr3 + ,	UTCR3_TXE UTCR3_RXE or UTCR3_RIE or ,
					\ enable TX and RX straight to the UART pins
  0 ,					\ end of table

l: uart2-setup	\ -- addr
\ *G Initialisation table for UART2.
  _hssp hscr0 + ,	0 ,		\ disable HSSP
  _uart2 utcr3 + ,	0 ,		\ disable UART
  _uart2 utcr4 + ,	0 ,		\ turn off IrDA facilities
  _uart2 utcr0 + ,      UART2_UTCR0 ,	\ 8 bit, no parity, 1 stop 
  console2-speed gen-brd		\ define baud rate
  _uart2 utcr1 + ,	,
  _uart2 utcr2 + ,	,
  _uart2 utsr0 + ,	$0FF ,		\ reset all bits
  _uart2 utdr + ,	0 ,		\ write dummy character!
  _uart2 utcr3 + ,	UTCR3_TXE UTCR3_RXE or UTCR3_RIE or ,
					\ enable TX and RX straight to the UART pins
  0 ,					\ end of table

l: uart3-setup	\ -- addr
\ *G Initialisation table for UART3 - usually the debug UART
  _uart3 utcr3 + ,	0 ,		\ disable it
  _uart3 utcr0 + ,      UART3_UTCR0 ,	\ 8 bit, no parity, 1 stop 
  console3-speed gen-brd		\ define baud rate
  _uart3 utcr1 + ,	,
  _uart3 utcr2 + ,	,
  _uart3 utsr0 + ,	$0FF ,		\ reset all bits
  _uart3 utdr + ,	0 ,		\ write dummy character!
  _uart3 utcr3 + ,	UTCR3_TXE UTCR3_RXE or UTCR3_RIE or ,
					\ enable TX and RX straight to the UART pins
  0 ,					\ end of table

: init-ser1	\ --
\ *G Initialise serial line on UART1.
(( \ This bit to use GPIO 14/15 for UART data.
  _gpio gafr +	           		\ Select GPIO alternate function
    dup @ $0000C000 or swap !		\ on GPIO 14 and 15 for serial 1
  _gpio gpdr +				\ Set alternate function directions
    dup @
    bit15 invert and  bit14 or		\ bit 15=0=input, bit14=1=output
    swap !
))
  di
  uart1-setup init-io
  _IC icmr + [o/f] dup @ bit15 or swap ! \ set ICMR bit 15
  ei
;

: init-ser2	\ --
\ *G Initialise serial line on UART2.
  di
  uart2-setup init-io
  _IC icmr + [o/f] dup @ bit16 or swap ! \ set ICMR bit 15
  ei
;

: init-ser3	\ --
\ *G Initialise serial line on UART3.
  di
  uart3-setup init-io
  _IC icmr + [o/f] dup @ bit17 or swap ! \ set ICMR bit 17
  ei
;

: init-ser	\ -- ; initialise serial lines
\ *G Initialise the serial line(s).
  init-ser1 init-ser2 init-ser3
  #100000 0 do loop			\ waste a character time - ICACHE ON
\  #10000 0 do loop			\ waste a character time - ICACHE OFF
;


\ **************
\ Serial drivers
\ **************

\ Common routines

: (seremit)	\ char base --
\ *G Send the character to the UART whose base address is given.
  begin
    dup utsr1 + @ UTSR1_TNF and
  until
  utdr + !
;

: (serTYPE)	\ c-addr len base --
\ *G Type a string to the UART whose base address is given.
  -rot  bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (serCR)	\ base --
\ *G Issue a CR/LF pair to the UART whose base address is given.
  $0D over (seremit)  $0A swap (seremit)
;


\ -----
\ UART1
\ -----

: serkey?1	\ base -- t/f
\ *G Return true if a character has been received by the UART whose
\ ** base address is given.
  SerQ1 cqempty? 0=
;

: serkey1	\ -- char
\ *G Wait until a character has been received, and return the character.
  SerQ1 cqueue>
;

: seremit1	\ char --
\ *G Send the character to UART3.
  _UART1 (seremit)
;

: serTYPE1	\ c-addr len --
\ *G Type a string to the UART.
  _UART1 (serType)
;

: serCR1	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART1 (serCR)
;


\ -----
\ UART2
\ -----

: serkey?2	\ base -- t/f
\ *G Return true if a character has been received by the UART whose
\ ** base address is given.
  SerQ2 cqempty? 0=
;

: serkey2	\ -- char
\ *G Wait until a character has been received, and return the character.
  SerQ2 cqueue>
;

: seremit2	\ char --
\ *G Send the character to UART3.
  _UART2 (seremit)
;

: serTYPE2	\ c-addr len --
\ *G Type a string to the UART.
  _UART2 (serType)
;

: serCR2	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART2 (serCR)
;


\ -----
\ UART3
\ -----

: serkey?3	\ base -- t/f
\ *G Return true if a character has been received by the UART whose
\ ** base address is given.
  SerQ3 cqempty? 0=
;

: serkey3	\ -- char
\ *G Wait until a character has been received, and return the character.
  SerQ3 cqueue>
;

: seremit3	\ char --
\ *G Send the character to UART3.
  _UART3 (seremit)
;

: serTYPE3	\ c-addr len --
\ *G Type a string to the UART.
  _UART3 (serType)
;

: serCR3	\ --
\ *G Issue a CR/LF pair to the UART.
  _UART3 (serCR)
;


\ ***********************
\ Generic i/o assignments
\ ***********************

((
0 ipfunc KEY	\ -- char ; receive char
1 ipfunc KEY?	\ -- flag ; check receive char
2 opfunc EMIT	\ -- char ; display char
3 opfunc TYPE	\ caddr len -- ; display string
4 opfunc CR	\ -- ; display new line
5 opfunc TYPEC	\ caddr len -- ; display string in code space (Harvard only)
))
cdata
create Console1	\ -- addr ; OUT managed by upper driver
\ *G Device table for UART1.
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

create Console2	\ -- addr ; OUT managed by upper driver
\ *G Device table for UART2.
  ' serkey2 ,		\ -- char ; receive char
  ' serkey?2 ,		\ -- flag ; check receive char
  ' seremit2 ,		\ -- char ; display char
  ' sertype2 ,		\ caddr len -- ; display string
  ' sercr2 ,		\ -- ; display new line

create Console3	\ -- addr ; OUT managed by upper driver
\ *G Device table for UART2.
  ' serkey3 ,		\ -- char ; receive char
  ' serkey?3 ,		\ -- flag ; check receive char
  ' seremit3 ,		\ -- char ; display char
  ' sertype3 ,		\ caddr len -- ; display string
  ' sercr3 ,		\ -- ; display new line

decimal

console-port 1 = [if]
  Console1 constant Console
[then]

console-port 2 = [if]
  Console2 constant Console
[then]

console-port 3 = [if]
  Console3 constant Console
[then]

