\ Atmel AT91SAM7x console definitions, interrupt driven rx, polled tx serial

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20061031 SFP001 Converted to AT91SAM7.
))


\ ===========
\ *! sersam7xi
\ *T AT91SAM7 interrupt receive serial driver
\ ===========
\ *P By default the serial drivers are initialised to 115200
\ ** baud for the DBGU (default console), 38400 for USART0 and
\ ** 9600 baud for USART1..3. These baud rates can be
\ ** changed by setting the relevant UART registers.
\ ** See the chip user manual for more details.

\ ****************
\ *S Configuration
\ ****************
\ *P The following equates control which USARTs are required
\ ** and initialised. If the equates are undefined, they are
\ ** generated and set to 1 for DBGU and 0 for all others.

[undefined] useDBGU? [if]
1 equ useDBGU?	\ -- n
\ *G Compile DBGU code if non-zero.
[then]

[undefined] useUSART0? [if]
0 equ useUSART0?	\ -- n
\ *G Compile USART0 code if non-zero.
[then]

[undefined] useUSART1? [if]
0 equ useUSART1?	\ -- n
\ *G Compile USART1 code if non-zero.
[then]

only forth definitions
decimal


\ ***********************
\ *S Baud rate generators
\ ***********************

: GenBAUD	\ baud -- CD
\ *G Calculate the baud rate. Can also be interpreted.
  /MCK swap / 16 /mod  swap 7 >
  if  1+  endif
;
interpreter
: GenBAUD	\ baud -- CD
  /MCK swap / 16 /mod  swap 7 >
  if  1+  endif
;
target


\ *************************
\ *S USART masks for 8N1 use, clk = MCK
\ *************************

US_CLKS_MCK  US_CHRL_8 or  US_ASYNCH or US_PAR_NO or US_NBSTOP1 or
US_CHMODE_NORMAL or US_MODE8 or US_CLKNO or
  equ US_INITMODE	\ -- mask
\ *G USART inital values for Mode Register.

US_RXDIS  US_TXDIS or  US_RSTRX or  US_RSTTX or  US_RSTSTA or
  equ US_RESET		\ -- mask
\ *G USART inital value for RESET operation in Control Register.

US_RXEN US_TXEN or
  equ US_GO		\ -- mask
\ *G USART inital value for START operation in Control Register.

$000F3FFF equ US_INTDIS	\ -- mask
\ *G USART initial value for Interrupt Disable Register


\ *****************************
\ *S Interrupt Service routines
\ *****************************

l: ^AIC
  _AIC ,

useDBGU? [if]

cell buffer: rxd-char	\ -- addr ; last character received
cell buffer: rxd-avail	\ -- addr ; character available?

l: ^rxd-char		\ pointers to data
  rxd-char ,
l: ^rxd-avail
  rxd-avail ,
l: ^DBGU
   _DBGU ,

proc SerdISR	\ -- ; DBGU interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  ldr	r0, ^rxd-char			\ address of data items
  ldr	r1, ^rxd-avail
  ldr	r2, ^DBGU
  ldr	r4, ^AIC

  ldr	r3, [ r2, # US_RHR ]		\ read USART RHR register
  str   r3, [ r0 ]			\ save character
  mvn	r3, # 0				\ -1
  str	r3, [ r1 ]			\ set character available flag

  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

$30000000 equ DBGUpins
\ *G Pin mask in PIOA for the TXD and RXD pins.

l: initDBGUtab	\ -- addr ; initialisation table
  _PMC PMC_PCER + ,		SYSmask ,	\ power up system peripherals, inc DBGU
  _PIOA PIO_ASR + ,		DBGUpins ,	\ PIOA bits 31/30 are periph A
  _PIOA PIO_BSR + ,		0 ,
  _PIOA PIO_ODR + ,		DBGUpins ,	\ disable PIO outputs
  _PIOA PIO_IDR + ,		DBGUpins ,	\ disable PIO interrupt
  _PIOA PIO_PDR + ,		DBGUpins ,	\ disable PIO
  _AIC AIC_IDCR + ,		SYSmask ,	\ Disable interrupt
  _AIC AIC_SMR0 + ID_SYS cells + ,  $00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + ID_SYS cells + ,  SerdISR ,	\ ISR handler
  _DBGU US_CR + ,		US_RESET ,	\ reset UART
  _DBGU US_MR + ,		US_INITMODE ,	\ mode register
  _DBGU US_IDR + ,		US_INTDIS ,	\ disable interrupts
  _DBGU US_BRGR + ,	Console0-speed GenBAUD , \ baud rate
  _DBGU US_IER + ,		$00000001 ,	\ Enable interrupt on RX0
  _AIC AIC_IECR + ,		SYSmask , 	\ Enable the DBGU interrupt
  _DBGU US_CR + ,		US_GO ,		\ enable TX and RX
  0 ,						\ end of table

[then]

useUSART0? [if]

cell buffer: rx0-char	\ -- addr ; last character received
cell buffer: rx0-avail	\ -- addr ; character available?

l: ^rx0-char		\ pointers to data
  rx0-char ,
l: ^rx0-avail
  rx0-avail ,
l: ^USART0
   _USART0 ,

proc Ser0ISR	\ -- ; USART 0 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  ldr	r0, ^rx0-char			\ address of data items
  ldr	r1, ^rx0-avail
  ldr	r2, ^USART0
  ldr	r4, ^AIC

  ldr	r3, [ r2, # US_RHR ]		\ read USART RHR register
  str   r3, [ r0 ]			\ save character
  mvn	r3, # 0				\ -1
  str	r3, [ r1 ]			\ set character available flag

  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

[then]

((
cell buffer: rx1-char	\ -- addr ; last character received
cell buffer: rx1-avail	\ -- addr ; character available?

l: ^rx1-char		\ pointers to data
  rx1-char ,
l: ^rx1-avail
  rx1-avail ,
l: ^USART1
   _USART1 ,

proc Ser1ISR	\ -- ; USART 0 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  ldr	r0, ^rx1-char			\ address of data items
  ldr	r1, ^rx1-avail
  ldr	r2, ^USART1
  ldr	r4, ^AIC

  ldr	r3, [ r2, # US_RHR ]		\ read USART RHR register
  str   r3, [ r0 ]			\ save character
  mvn	r3, # 0				\ -1
  str	r3, [ r1 ]			\ set character available flag

  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code

cell buffer: rx2-char	\ -- addr ; last character received
cell buffer: rx2-avail	\ -- addr ; character available?

l: ^rx2-char		\ pointers to data
  rx2-char ,
l: ^rx2-avail
  rx2-avail ,
l: ^USART2
   _USART2 ,

proc Ser2ISR	\ -- ; USART 0 interrupt handler, R13 already set up
  stmfd r13 ! { r0-r12, link }		\ save unbanked registers and LINK
  ldr	r0, ^rx2-char			\ address of data items
  ldr	r1, ^rx2-avail
  ldr	r2, ^USART2
  ldr	r4, ^AIC

  ldr	r3, [ r2, # US_RHR ]		\ read USART RHR register
  str   r3, [ r0 ]			\ save character
  mvn	r3, # 0				\ -1
  str	r3, [ r1 ]			\ set character available flag

  str	r3, [ r4, # AIC_EOICR ]

  ldmfd r13 ! { r0-r12, link }		\ restore unbanked registers and LINK
  sub .s  pc, link, # 4			\ return from exception
end-code
))

\ **************
\ Initialisation
\ **************

((
[undefined] init-io [if]
: init-io	\ addr --
\ *G Copy the contents of the I/O set up table to an I/O device.
\ ** Each element of the table is of the form addr (cell) followed
\ ** by data (cell). The table is terminated by an address of 0.
\ ** A table of a single 0 address performs no action.
  begin
    dup @
   while
    dup 2@ !  2 cells +
  repeat
  drop
;
[then]
))

((
l: usart0-init-tab	\ -- addr ; hold initialisation data
\ address			data
  rx0-avail ,			0 ,
  rx0-char ,			0 ,
  _APMC_BASE APMC_PCER + ,	$2000 BIT2 + ,	\ Enable peripheral clocks for USART and PIOA
  _pioa_base pio_pdr + ,	$00018000 ,	\ disable PIO for TXD0 and RXD0
  _pioa_base pio_idr + ,	$00018000 ,	\ Disable interrupts for the PIO channels
  _AIC AIC_IDCR + ,		BIT2 ,		\ Disable interrupt
  _AIC AIC_SMR0 + 2 cells + ,	$00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + 2 cells + ,	Ser0ISR ,	\ ISR handler
  _USART0 US_CR + ,		US_RESET , 	\ Reset the USART0 channels
  _USART0 US_MR + ,		US_INITMODE , 	\ Set up initial mode for USART0
  _USART0 US_BRGR + ,				\ Set up baud rate
		  console-speed Baud-Rate-gen ,
  _USART0 US_RTOR + ,		0 , 		\ Disable receiver timeout
  _USART0 US_TTGR + ,		0 , 		\ Disable transmitter time-guard
  _USART0 US_RPR + ,		0 , 		\ Disbale receive pointer register
  _USART0 US_RCR + ,		0 ,		\ Zero receiver counter register
  _USART0 US_TPR + ,		0 ,		\ Disable transmitter pointer register
  _USART0 US_TCR + ,		0 ,		\ Zero transmitter counter register
  _AIC AIC_IECR + ,		BIT2 ,	 	\ Enable the USART0 interrupt
  _USART0 US_IER + ,		$00000001 ,	\ Enable interrupt on RX0
  _USART0 US_CR + ,		US_GO , 	\ GO GO GO
  0 ,						\ end of table

l: usart1-init-tab	\ -- addr ; hold initialisation data
\ address			data
  rx1-avail ,			0 ,
  rx1-char ,			0 ,
  _APMC_BASE APMC_PCER + ,	$2000 BIT3 + ,	\ Enable peripheral clocks for USART and PIOA
  _pioa_base pio_pdr + ,	$000C0000 ,	\ disable PIO for TXD1 and RXD1
  _pioa_base pio_idr + ,	$000C0000 ,	\ Disable interrupts for the PIO channels
  _AIC AIC_IDCR + ,		BIT3 ,		\ Disable interrupt
  _AIC AIC_SMR0 + 3 cells + ,	$00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + 3 cells + ,	Ser1ISR ,	\ ISR handler

  _USART1 US_CR + ,		US_RESET , 	\ Reset the USART channels
  _USART1 US_MR + ,		US_INITMODE , 	\ Set up initial mode for USART
  _USART1 US_BRGR + ,				\ Set up baud rate
		  console-speed Baud-Rate-gen ,
  _USART1 US_RTOR + ,		0 , 		\ Disable receiver timeout
  _USART1 US_TTGR + ,		0 , 		\ Disable transmitter time-guard
  _USART1 US_RPR + ,		0 , 		\ Disbale receive pointer register
  _USART1 US_RCR + ,		0 ,		\ Zero receiver counter register
  _USART1 US_TPR + ,		0 ,		\ Disable transmitter pointer register
  _USART1 US_TCR + ,		0 ,		\ Zero transmitter counter register
  _AIC AIC_IECR + ,		BIT3 ,	 	\ Enable the USART1 interrupt
  _USART1 US_IER + ,		$00000001 ,	\ Enable interrupt on RX
  _USART1 US_CR + ,		US_GO , 	\ Enable TX and RX
  0 ,						\ end of table

l: usart2-init-tab	\ -- addr ; hold initialisation data
\ address			data
  rx2-avail ,			0 ,
  rx2-char ,			0 ,
  _APMC_BASE APMC_PCER + ,	$2000 BIT4 + ,	\ Enable peripheral clocks for USART and PIOA
  _pioa_base pio_pdr + ,	$00600000 ,	\ disable PIO for TXD1 and RXD1
  _pioa_base pio_idr + ,	$00600000 ,	\ Disable interrupts for the PIO channels
  _AIC AIC_IDCR + ,		BIT4 ,		\ Disable interrupt
  _AIC AIC_SMR0 + 4 cells + ,	$00000007 ,	\ Interrupt priority
  _AIC AIC_SVR0 + 4 cells + ,	Ser2ISR ,	\ ISR handler

  _USART2 US_CR + ,		US_RESET , 	\ Reset the USART channels
  _USART2 US_MR + ,		US_INITMODE , 	\ Set up initial mode for USART
  _USART2 US_BRGR + ,				\ Set up baud rate
		  console-speed Baud-Rate-gen ,
  _USART2 US_RTOR + ,		0 , 		\ Disable receiver timeout
  _USART2 US_TTGR + ,		0 , 		\ Disable transmitter time-guard
  _USART2 US_RPR + ,		0 , 		\ Disbale receive pointer register
  _USART2 US_RCR + ,		0 ,		\ Zero receiver counter register
  _USART2 US_TPR + ,		0 ,		\ Disable transmitter pointer register
  _USART2 US_TCR + ,		0 ,		\ Zero transmitter counter register
  _AIC AIC_IECR + ,		BIT4 ,	 	\ Enable the USART1 interrupt
  _USART2 US_IER + ,		$00000001 ,	\ Enable interrupt on RX
  _USART2 US_CR + ,		US_GO , 	\ Enable TX and RX
  0 ,						\ end of table
))

: init-ser	\ -- ; initialise on-chip USARTs
  di
[ useDBGU? ] [if]  InitDBGUtab init-io  [then]
[ useUSART0? ] [if]  initUSART0tab init-io  [then]
[ useUSART1? ] [if]  initUSART1tab init-io  [then]
  ei
;

: (seremit) 	\ char UBase* -- ; transmit 
  begin
    dup US_CSR + @ US_TXRDY and
  until
  US_THR + !
;

: (sertype)	\ c-addr len base --
  -rot  bounds
  ?do  i c@ over (seremit) loop
  drop
;

useDBGU? [if]
: serkey?d	\ -- flag
  rxd-avail @
;

: serkeyd	\ -- char
  begin
    [I
    rxd-avail @ 0=
   while
    I]  [ tasking? ] [if]  pause  [then]
  repeat
  rxd-char @ $FF and
  rxd-avail off
  I]
;

: seremitd	\ char --
  _DBGU (seremit)
;

: sertyped	\ c-addr len --
  _DBGU (sertype)
;

: serCRd	\ --
  $0D seremitd  $0A seremitd
;

cdata
create ConsoleD	\ -- addr 
  ' serkeyD ,
  ' serkey?D ,
  ' seremitD ,
  ' sertypeD ,
  ' serCRD ,
[then]

useUSART0? [if]
: serkey?0i	\ -- flag
  rx0-avail @
;

: serkey0i	\ -- char
  begin
    [I
    rx0-avail @ 0=
   while
    I]
[ tasking? ] [if]
    pause
[then]
  repeat
  rx0-char @ $FF and
  rx0-avail off
  I]
;

: seremit0	\ char --
  _USART0 (seremit)
;

: sertype0	\ c-addr len --
  _USART0 (sertype)
;

: serCR0	\ --
  $0D seremit0  $0A seremit0
;

cdata
create Console0	\ -- addr 
  ' serkey0i ,
  ' serkey?0i ,
  ' seremit0 ,
  ' sertype0 ,
  ' serCR0 ,
[then]

useUSART1? [if]
: serkey?1i	\ -- flag
  rx1-avail @
;

: serkey1i	\ -- char
  begin
    [I
    rx1-avail @ 0=
   while
    I]
[ tasking? ] [if]
    pause
[then]
  repeat
  rx1-char @ $FF and
  rx1-avail off
  I]
;

: seremit1	\ char --
  _USART1 (seremit)
;

: sertype1	\ c-addr len --
  _USART1 (sertype)
;

: serCR1	\ --
  $0D seremit1  $0A seremit1
;

create Console1
  ' serkey1i ,
  ' serkey?1i ,
  ' seremit1 ,
  ' sertype1 ,
  ' serCR1 ,
[then]

((
useUSART2? [if]
: serkey?2i	\ -- flag
  rx2-avail @
;

: serkey2i	\ -- char
  begin
    [I
    rx2-avail @ 0=
   while
    I]
[ tasking? ] [if]
    pause
[then]
  repeat
  rx2-char @ $FF and
  rx2-avail off
  I]
;

: seremit2	\ char --
  _USART2 (seremit)
;

: sertype2	\ c-addr len --
  _USART2 (sertype)
;

: serCR2	\ --
  $0D seremit2  $0A seremit2
;

create Console2
  ' serkey2i ,
  ' serkey?2i ,
  ' seremit2 ,
  ' sertype2 ,
  ' serCR2 ,
[then]

useUSART3? [if]
: serkey?3i	\ -- flag
  rx3-avail @
;

: serkey3i	\ -- char
  begin
    [I  rx3-avail @ 0=
   while
    I]  [ tasking? ] [if]  pause  [then]
  repeat
  rx3-char @ $FF and
  rx3-avail off
  I]
;

: seremit3	\ char --
  _USART3 (seremit)
;

: sertype3	\ c-addr len --
  _USART3 (sertype)
;

: serCR3	\ --
  $0D seremit3  $0A seremit3
;

create Console3
  ' serkey3i ,
  ' serkey?3i ,
  ' seremit3 ,
  ' sertype3 ,
  ' serCR3 ,
[then]
))

console-port 0 = [if]
  consoleD constant Console	\ -- addr ; select default serial port
[then]

console-port 1 = [if]
  console0 constant Console	\ -- addr ; select default serial port
[then]

console-port 2 = [if]
  console1 constant Console	\ -- addr ; select default serial port
[then]

