\ Atmel AT91SAM7xxxx console definitions, polled rx, polled tx serial

((
Copyright (c) 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ ===========
\ *! sersam7xp
\ *T AT91SAM7X polled serial driver
\ ===========

only forth definitions
decimal


\ *************************
\ *S UART masks for 8N1 use, clk = MCK
\ *************************

US_CLKS_MCK  US_CHRL_8 or  US_ASYNCH or US_PAR_NO or US_NBSTOP1 or
US_CHMODE_NORMAL or US_MODE8 or US_CLKNO or
  equ US_INITMODE	\ -- mask
\ *G USART inital values for Mode Register.

US_RXDIS  US_TXDIS or  US_RSTRX or  US_RSTTX or  US_RSTSTA or
  equ US_RESET		\ -- mask
\ *G USART inital value for RESET operation in Control Register.

US_RXEN US_TXEN or
  equ US_GO		\ -- mask
\ *G USART inital value for START operation in Control Register.

$000F3FFF equ US_INTDIS	\ -- mask
\ *G USART initial value for Interrupt Disable Register


\ ***********************
\ *S Baud rate generators
\ ***********************

: GenBAUD	\ baud -- CD
\ *G Calculate the baud rate. Can also be interpreted.
  /MCK swap / 16 /mod  swap 7 >
  if  1+  endif
;
interpreter
: GenBAUD	\ baud -- CD
  /MCK swap / 16 /mod  swap 7 >
  if  1+  endif
;
target

: (seremit) 	\ char UBase* -- ; transmit 
  begin
    dup US_CSR + @ US_TXRDY and
  until
  US_THR + !
;

: (sertype)	\ c-addr len base --
  -rot  bounds
  ?do  i c@ over (seremit) loop
  drop
;

: (serCR)	\ base --
  $0D over (serEmit)  $0A swap (serEmit)
;

: (serkey?)	\ UBase* -- t/f
  US_CSR + @ US_RXRDY and
;

: (serkey)	\ base -- char
  begin
    dup (serkey?)
  until
  US_RHR + @ $FF and
;

useDBGU? [if]
: seremit0	\ char --
  _DBGU (seremit)
;

: sertype0	\ c-addr len --
  _DBGU (sertype)
;

: serCR0	\ --
  _DBGU (serCR)
;

: Serkey?0	\ -- flag
  _DBGU (serkey?)
;

: serkey0	\ -- char
  _DBGU (serkey)
;

cdata
create Console0	\ -- addr 
  ' serkey0 ,
  ' serkey?0 ,
  ' seremit0 ,
  ' sertype0 ,
  ' serCR0 ,

$30000000 equ DBGUpins	\ -- mask
\ *G pin mask for DBGU port on PIOA bits 29/28.

l: initDBGUtab	\ -- addr
\ *G Initialisation table for DBGU serial port.
  _PMC PMC_PCER + ,	SYSmask ,	\ power up system peripherals, in DBGU
  _PIOA PIO_ASR + ,	DBGUpins ,	\ PIOA bits 29/28 are periph A
  _PIOA PIO_BSR + ,	0 ,
  _PIOA PIO_ODR + ,	DBGUpins ,
  _PIOA PIO_PDR + ,	DBGUpins ,
  _DBGU US_CR + ,	US_RESET ,	\ reset UART
  _DBGU US_MR + ,	US_INITMODE ,	\ mode register
  _DBGU US_IDR + ,	US_INTDIS ,	\ disable interrupts
  _DBGU US_BRGR + ,	Console0-speed GenBAUD , \ baud rate
  _DBGU US_CR + ,	US_GO ,		\ enable TX and RX
  0 ,					\ end of table
[then]

useUSART0? [if]
: seremit1	\ char --
  _USART0 (seremit)
;

: sertype1	\ c-addr len --
  _USART0 (sertype)
;

: serCR1	\ --
  _USART0 (serCR)
;

: Serkey?1	\ -- flag
  _USART0 (serkey?)
;

: serkey1	\ -- char
  _USART0 (serkey)
;

cdata create Console1
  ' serkey1 ,
  ' serkey?1 ,
  ' seremit1 ,
  ' sertype1 ,
  ' serCR1 ,

$00000003 equ US0pins	\ -- mask
\ *G Bit mask for US0 TXD/RXD on PIOA bits 1/0.

l: initUSART0tab	\ -- addr
  _PMC PMC_PCER + ,	US0mask ,	\ power up US0
  _PIOA PIO_ASR + ,	US0pins ,	\ PIOA bits 1/0 are UART0
  _PIOA PIO_BSR + ,	0 ,
  _PIOA PIO_ODR + ,	US0pins ,
  _PIOA PIO_PDR + ,	US0pins ,
  _USART0 US_CR + ,	US_RESET ,	\ reset UART
  _USART0 US_MR + ,	US_INITMODE ,	\ mode register
  _USART0 US_IDR + ,	US_INTDIS ,	\ disable interrupts
  _USART0 US_BRGR + ,	Console1-speed GenBAUD , \ baud rate
  _USART0 US_RTOR + ,	0 ,		\ Rx timout
  _USART0 US_TTGR + ,	0 ,		\ Tx timeguard
  _USART0 US_CR + ,	US_GO ,		\ enable TX and RX
  0 ,					\ end of table
[then]

useUSART1? [if]
: seremit2	\ char --
  _USART1 (seremit)
;

: sertype2	\ c-addr len --
  _USART1 (sertype)
;

: serCR2	\ --
  _USART1 (serCR)
;

: Serkey?2	\ -- flag
  _USART1 (serkey?)
;

: serkey2	\ -- char
  _USART1 (serkey)
;

cdata create Console2
  ' serkey2 ,
  ' serkey?2 ,
  ' seremit2 ,
  ' sertype2 ,
  ' serCR2 ,

$00000060 equ US1pins	\ -- mask
\ *G Bit mask for US1 TXD/RXD on PIOA bits 6/5.

l: initUSART1tab	\ -- addr
  _PMC PMC_PCER + ,	US1mask ,	\ power up US0
  _PIOA PIO_ASR + ,	US1pins ,	\ PIOA bits 1/0 are UART0
  _PIOA PIO_BSR + ,	0 ,
  _PIOA PIO_ODR + ,	US1pins ,
  _PIOA PIO_PDR + ,	US1pins ,
  _USART0 US_CR + ,	US_RESET ,	\ reset UART
  _USART0 US_MR + ,	US_INITMODE ,	\ mode register
  _USART0 US_IDR + ,	US_INTDIS ,	\ disable interrupts
  _USART0 US_BRGR + ,	Console1-speed GenBAUD , \ baud rate
  _USART0 US_RTOR + ,	0 ,		\ Rx timout
  _USART0 US_TTGR + ,	0 ,		\ Tx timeguard
  _USART0 US_CR + ,	US_GO ,		\ enable TX and RX
  0 ,					\ end of table
[then]

console-port 0 = [if]
  console0 constant Console	\ -- addr ; select default serial port
[then]

console-port 1 = [if]
  console1 constant Console	\ -- addr ; select default serial port
[then]

console-port 2 = [if]
  console2 constant Console	\ -- addr ; select default serial port
[then]

: init-ser	\ -- ; initialise on-chip UARTs
[ useDBGU? ] [if]
  initDBGUtab init-io
[then]
[ useUSART0? ] [if]
  initUSART0tab init-io
[then]
[ useUSART1? ] [if]
  initUSART1tab init-io
[then]
;


\ ======
\ *> ###
\ ======

decimal
