\ serSTR91xqi.FTH - STR91x queued interrupt RX, polled TX, serial communication drivers

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

only forth definitions


\ *****************
\ *S Initialisation
\ *****************
\ *P *\fo{GEN-BAUD-RATE} takes the system clock rate (in HZ) and
\ ** the required serial comms rate (in BPS) and returns a
\ ** system-dependent number of parameters that should be sufficent
\ ** to effect the baud rate selection by initialising the correct
\ ** hardware. The value *\f{brdiv} is 1 or 2, depending on the
\ ** setting of bit 9 in the SCU_CLKCNTR register. By default,
\ ** this bit is set to 0, leading to *\f{brdiv=2}.

\ *P The STR91x family has a fractional baud rate generator, with
\ ** a 6 bit fractional portion. To minimise rounding errors, we
\ ** calculate a seven bit fraction, with the bottom bit rounded
\ ** up.

\ *E                    system-speed
\ ** divisor =      ---------------------
\ **                16 * brdiv * baudrate
\ **
\ **                 system-speed * 128    system-speed * 8
\ ** divisor*128 = --------------------- = ----------------
\ **               16 * brdiv * baudrate   brdiv * baudrate

interpreter
: gen-baud-rate	\ bps-rate -- int16 div6
\ *G *\fo{INTERPRETER} word to generate the integer and fractional
\ ** parts of the baud rate divisor.
  system-speed 8 * /BRDIV /  swap / 1+
  dup 7 rshift $FFFF and		\ integer portion
  swap 1 rshift $03F and		\ fractional portion
;
target

console0-speed gen-baud-rate
  equ u0-divfrac  equ u0-divint
console1-speed gen-baud-rate
  equ u1-divfrac  equ u1-divint
console2-speed gen-baud-rate
  equ u2-divfrac  equ u2-divint

useUART0? [if]
: init-ser0	\ --
\ *G Initialise serial port 0 to N,8,1
  bit3 _SCU
  2dup SCU_PCGR1 + or!			\ set bit to enable clock to UART
  SCU_PRR1 + or!			\ take UART out of reset
  _UART0
  0 over UART_CR + w!			\ disable UART
  u0-divint over UART_IBRD + w!		\ set integer portion of baud rate
  u0-divfrac over UART_FBRD + w!	\ set fractional portion of baud rate
\ N.B. a write to LCR must occur after setting the baud rate.
  %01110000 over UART_LCR + w!		\ set LCR for N,8,1, FIFOs enabled
  $0012 over UART_IFLS + w!		\ set FIFOs to half way mark.
  $0301 swap UART_CR + w!		\ enable UART, TX, RX
;
[then]

\ *P On the Hitex STR912 board only UART1 is connected to RS232
\ ** drivers. The connection is P3.2=RxD (Alt Inp 1), P3.3=TxD
\ ** (Alt Out 2). Note that I/O functions should be assigned to
\ ** one pin only. The functions are described in the chip-specific
\ ** datasheet, *\b{not} in the generic STR91x User Manual.

useUART1? [if]
#256 cqueue: RX1q	\ -- addr
\ *G UART1 receive queue.

: UART1-isr	\ --
\ *G UART1 interrupt service routine.
  _UART1
  begin
    dup UART_FR + @ bit4 and 0=		\ bit 4 is RxFifo empty
   while
    dup UART_DR + @ $FF and		\ -- uart char
    Rx1q dup (cqfull?)
    if  2drop  else  (>cqueue)  endif
  repeat
  drop
;
' UART1-isr IRQ1: UART1-irq

: init-ser1	\ --
\ *G Initialise serial port 1 to N,8,1
  UART1mask _SCU
  2dup SCU_PCGR1 + or!			\ set bit to clock UART
  tuck SCU_PRR1 + or!			\ take it out of reset
\ -- addr
  dup SCU_GPIOOUT + 3 cells + dup @	\ read P3 output type
    $00F0 invert and 			\ clear existing bits for P2.3:2
    $0080 or				\ P3.3=Alt Out 2, P3.2=input
    swap !				\ put back
  $08 over SCU_GPIOTYPE + 3 cells + bic!	\ set P3.3 to push/pull
  SCU_GPIOIN + 3 cells + dup @		\ read P3 input type
    $0C invert and			\ clear bits 3:2
    $04 or				\ set bit 2 = Alt Inp 1
    swap !

  Rx1q init-cqueue			\ reset input queue
  _UART1
  0 over UART_CR + w!			\ disable UART
  u1-divint over UART_IBRD + w!		\ set integer portion of baud rate
  u1-divfrac over UART_FBRD + w!	\ set fractional portion of baud rate
\ N.B. a write to LCR must occur after setting the baud rate.
  $70 over UART_LCR + w!		\ set LCR for N,8,1, FIFOs enabled
  $0012 over UART_IFLS + w!		\ set FIFOs to half way mark.
  $0301 over UART_CR + w!		\ enable UART, TX, RX
  $0050 over UART_IMSC + w!		\ enable RX and RX timeout interrupts
  drop

  UART1-irq UART1int# 7 SetIrqIsr	\ set and enable interrupt
;
[then]

useUART2? [if]
: init-ser2	\ --
\ *G Initialise serial port 2 to N,8,1
  bit5 _SCU
  2dup SCU_PCGR1 + or!			\ set bit to clock UART
  SCU_PRR1 + or!			\ take it out of reset
  _UART2
  0 over UART_CR + w!			\ disable UART
  u2-divint over UART_IBRD + w!		\ set integer portion of baud rate
  u2-divfrac over UART_FBRD + w!	\ set fractional portion of baud rate
\ N.B. a write to LCR must occur after setting the baud rate.
  %01110000 over UART_LCR + w!		\ set LCR for N,8,1, FIFOs enabled
  $0012 over UART_IFLS + w!		\ set FIFOs to half way mark.
  $0301 swap UART_CR + w!		\ enable UART, TX, RX
;
[then]

: init-ser	\ --
\ *G Initialise serial ports
  di  InitVIC				\ INIT-SER called before cold chain
  [ useUART0? ] [if] init-ser0 [then]
  [ useUART1? ] [if] init-ser1 [then]
  [ useUART2? ] [if] init-ser2 [then]
  ei
;


\ *****************
\ Serial primitives
\ *****************

: (seremit)	\ char base --
  begin
    dup UART_FR + w@ bit5 and 0=	\ bit 5 is TxFifo full
  until
  swap $FF and swap UART_DR + w!
;

: (serkey?)	\ base -- t/f
  UART_FR + @ bit4 and 0=		\ bit 4 is RxFifo empty
;

: (serkey)	\ base -- char
  begin
[ tasking? ] [if]  pause  [then]
    dup UART_FR + @ bit4 and 0=
  until
  UART_DR + w@ $FF and
;

useUART0? [if]
: serkey?0	\ -- flag
  _UART0 (serkey?)
;

: serkey0	\ -- char
  _UART0 (serkey)
;

: seremit0	\ char --
  _UART0 (seremit)
;

: serTYPE0	\ c-addr len --
  bounds
  ?do  i c@ seremit0  loop
;

: serCR0	\ --
  $0D seremit0  $0A seremit0
;

cdata
create Console0	\ -- addr ; OUT managed by upper driver
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line
[then]

useUART1? [if]
: serkey?1	\ -- flag
  Rx1q cqnotempty?
;

: serkey1	\ -- char
  Rx1q cqueue>
;

: seremit1	\ char --
  _UART1 (seremit)
;

: serTYPE1	\ c-addr len --
  bounds
  ?do  i c@ seremit1  loop
;

: serCR1	\ --
  $0D seremit1  $0A seremit1
;

cdata create Console1	\ -- addr ; OUT managed by upper driver
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line
[then]

useUART2? [if]
: serkey?2	\ -- flag
  _UART2 (serkey?)
;

: serkey2	\ -- char
  _UART2 (serkey)
;

: seremit2	\ char --
  _UART2 (seremit)
;

: serTYPE2	\ c-addr len --
  bounds
  ?do  i c@ seremit2  loop
;

: serCR2	\ --
  $0D seremit2  $0A seremit2
;

cdata create Console2	\ -- addr ; OUT managed by upper driver
  ' serkey2 ,		\ -- char ; receive char
  ' serkey?2 ,		\ -- flag ; check receive char
  ' seremit2 ,		\ -- char ; display char
  ' sertype2 ,		\ caddr len -- ; display string
  ' sercr2 ,		\ -- ; display new line
[then]


\ ***********************
\ Generic i/o assignments
\ ***********************

console-port 0 = [if]
  console0 constant console
[then]

console-port 1 = [if]
  console1 constant console
[then]

console-port 2 = [if]
  console2 constant console
[then]

decimal

