\ fcmove.fth - Fast CMOVE for Forth

((
Copyright (c) 2001, 2002, 2008 Rowley Associates Limited.
  Released for use with the MPE Forth cross compiler by
  Paul Curtis.

Copyright (c) 2009, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


This code provides an overall speed up of about four times over
the original version of CMOVE. The code for this fast (but vast)
version was written by Rowley Associates, whose permission to
adapt and publish the code with the MPE cross compiler is much
appreciated.


To do
=====

Change history
==============
20160122 MPE002 preserve R8=FSP.
20090804 MPE001 Adapted to Forth.
))

only forth definitions
decimal

\ *P On the ARM the following register usage is the default:
\ *E   r15         pc      program counter
\ **   r14         link    link register
\ **   r13         rsp     return stack pointer
\ **   r12         psp     data stack pointer
\ **   r11         up      user area pointer
\ **   r10         tos     cached top of stack
\ **   r9          lp      locals pointer
\ **   r8          fsp     float stack pointer
\ **   r0-r7       scratch

\ Address-specific routines.  Naming convention is x_d_s,
\ where _d is the low two bits of the destination and _s is the
\ low two bits of the source address.

proc x_3_3
\ The source and destination are both aligned the same way
\ Both can be made full-word aligned.
\ r1=asrc, r0=adest, r2=len
  ldrb     r3, [ R1 ], # 1
  strb     r3, [ R0 ], # 1
  sub .s   r2, r2, # 1
  b .ne    x_0_0
  b        fcmovex
end-code

proc x_1_1
\ The source and destination are both aligned the same way
\ Both can be made half-word aligned.
\ r1=asrc, r0=adest, r2=len
  ldrb     r3, [ R1 ], # 1
  sub .s   r2, r2, # 1
  strb     r3, [ R0 ], # 1
\  sub .s   r2, r2, # 1
  b .eq    fcmovex

l: x_2_2
\ The source and destination are both halfword aligned
\ Both be made word aligned.
   cmp     r2, # 2			\ Catch case of 1-byte left
   b .cc   x_1_1
   ldrh    r3, [ R1 ], # 2
   sub .s  r2, r2, # 2
   strh    r3, [ R0 ], # 2
\   sub .s  r2, r2, # 2
   b .eq   fcmovex

l: x_0_0
\ The source and destination are both full-word aligned.
\ If we can we'll copy blocks of 32 bytes = 8 registers.
\ we use R4..R11 with R3 as a scratch register.
  sub .s   r2, r2, # #32		\ at least 32 bytes?
  b .cc    final_word_copy		\ no
  begin,
    ldmia    R1 ! { r4-r11 }
    sub .s   r2, r2, # #32
    stmia    R0 ! { r4-r11 }
\    sub .s   r2, r2, # #32
  cc, until,
\ Adjust number of words because we overshot.
l: final_word_copy
  add .s   r2, r2, # #32
  b .eq    fcmovex
\ We have at most 7 words left to copy after the loop, so
\ Figure out what's left and copy them.
  mov     r3, r2 lsr # 2
  rsb     r3, r3, # 7
  add     pc, pc, r3 lsl # 3
  nop
\ 7
  ldr     r3, [ R1 ], # 4
  str     r3, [ R0 ], # 4
\ 6
  ldr     r3, [ R1 ], # 4
  str     r3, [ R0 ], # 4
\ 5
  ldr     r3, [ R1 ], # 4
  str     r3, [ R0 ], # 4
\ 4
  ldr     r3, [ R1 ], # 4
  str     r3, [ R0 ], # 4
\ 3
  ldr     r3, [ R1 ], # 4
  str     r3, [ R0 ], # 4
\ 2
  ldr     r3, [ R1 ], # 4
  str     r3, [ R0 ], # 4
\ 1
  ldr     r3, [ R1 ], # 4
  str     r3, [ R0 ], # 4

\ We have 0, 1, 2, or 3 bytes left to copy, so copy tail.
l: copy_tail
  mov .s  r2, r2, lsl # #31
\ C = 1 if two bytes to copy,
\ N = 1 if one byte to copy,
\ C+N if three...
  ldrh .cs r3, [ R1 ], # 2
  strh .cs r3, [ R0 ], # 2
  ldrb .mi r3, [ R1 ], # 1
  strb .mi r3, [ R0 ], # 1
  b        fcmovex

l: x_1_3
\ We can make both addresses halfword aligned
  ldrb     r3, [ R1 ], # 1
  strb     r3, [ R0 ], # 1
  sub .s   r2, r2, # 1
  b .eq    fcmovex

l: x_2_0
\ The source (R1) is cell aligned, the destination (R0) is halfword
\ aligned.
  sub .s   r2, r2, # #32
  b .cc    final_halfword_copy

  begin,
    ldmia    R1 ! { r3-r10 }		\ 32 bytes
    strh     r3, [ R0 ], # 2		\ Rd is now word-aligned
\ reassemble the registers - faster than additional stores
    mov      r3, r3 lsr # 16
    orr      r3, r3, r4 lsl # 16
    mov      r4, r4 lsr # 16
    orr      r4, r4, r5 asl # 16
    mov      r5, r5 lsr # 16
    orr      r5, r5, r6 asl # 16
    mov      r6, r6 lsr # 16
    orr      r6, r6, r7 asl # 16
    mov      r7, r7 lsr # 16
    orr      r7, r7, r8 asl # 16
    mov      r8, r8 lsr # 16
    orr      r8, r8, r9 asl # 16
    mov      r9, r9 lsr # 16
    orr      r9, r9, r10 asl # 16
    stmia    R0 ! { r3-r9 }
    mov      r10, r10, lsr # 16
    strh     r10, [ R0 ], # 2
    sub .s   r2, r2, # #32
  cc, until,

\ Adjust number of words because we overshot
l: final_halfword_copy
  add .s   r2, r2, # #32
  b .eq    fcmovex
\ We have at most 15 halfwords left to copy after the loop, so
\ Figure out what's left and copy them.
  mov      r3, r2 lsr # 1
  rsb      r3, r3, # #15
  add      pc, pc, r3 lsl # 3
  nop
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2
  ldrh     r3, [ R1 ], # 2
  strh     r3, [ R0 ], # 2

\ We might have one byte left to copy.
  tst      r2, # 1
  ldrb .ne r3, [ R1 ], # 1
  strb .ne r3, [ R0 ], # 1
  b        fcmovex

l: x_3_1
  ldrb     r4, [ R1 ], # 1
  strb     r4, [ R0 ], # 1
  sub .s   r2, r2, # 1
  b .eq    fcmovex
l: x_0_2
  cmp      r2, # 2
  b .cc    x_3_1
  ldrh     r4, [ R1 ], # 2
  sub .s    r2, r2, # 2
  strh     r4, [ R0 ], # 2
\  sub .s    r2, r2, # 2
  b .ne     x_2_0
  b         fcmovex
end-code

proc x_0_1
  ldrb     r4, [ R1 ], # 1
  sub .s   r2, r2, # 1
  strb     r4, [ R0 ], # 1
\  sub .s   r2, r2, # 1
  b .eq    fcmovex
l: x_1_2
  ldrb     r4, [ R1 ], # 1
  sub .s   r2, r2, # 1
  strb     r4, [ R0 ], # 1
\  sub .s   r2, r2, # 1
  b .eq    fcmovex
l: x_2_3
  ldrb     r4, [ R1 ], # 1
  sub .s   r2, r2, # 1
  strb     r4, [ R0 ], # 1
\  sub .s   r2, r2, # 1
  b .eq    fcmovex
l: x_3_0
\ The source is full-word aligned, the destination is one byte off.
\ We read a block into and then perform a register shuffle so that
\ we can save full-words.
  sub .s   r2, r2, # #32
  b .cc    final_byte_copy
  begin,
    ldmia    R1 ! { r3-r10 }
    strb     r3, [ R0 ], # 1		\ Rd is now word aligned
    mov      r3, r3, lsr # 8
    orr      r3, r3, r4, lsl # 24
    mov      r4, r4, lsr # 8
    orr      r4, r4, r5, lsl # 24
    mov      r5, r5, lsr # 8
    orr      r5, r5, r6, lsl # 24
    mov      r6, r6, lsr # 8
    orr      r6, r6, r7, lsl # 24
    mov      r7, r7, lsr # 8
    orr      r7, r7, r8, lsl # 24
    mov      r8, r8, lsr # 8
    orr      r8, r8, r9, lsl # 24
    mov      r9, r9, lsr # 8
    orr      r9, r9, r10, lsl # 24
    stmia    R0 ! { r3-r9 }
    mov      r10, r10, lsr # 8
    strh     r10, [ R0 ], # 2
    mov      r10, r10, lsr # 16
    strb     r10, [ R0 ], # 1
    sub .s   r2, r2, # #32
  cc, until,
  b        final_byte_copy
end-code

proc x_2_1
  ldrb     r4, [ R1 ], # 1
  sub .s   r2, r2, # 1
  strb     r4, [ R0 ], # 1
\  sub .s   r2, r2, # 1
  b .eq    fcmovex
l: x_3_2
  ldrb     r4, [ R1 ], # 1
  sub .s   r2, r2, # 1
  strb     r4, [ R0 ], # 1
\  sub .s   r2, r2, # 1
  b .eq    fcmovex
l: x_0_3
  ldrb     r4, [ R1 ], # 1
  sub .s   r2, r2, # 1
  strb     r4, [ R0 ], # 1
\  sub .s   r2, r2, # 1
  b .eq    fcmovex
l: x_1_0
  sub .s   r2, r2, # #32
  b .cc    final_byte_copy
  begin,
    ldmia    R1 ! { r3-r10 }
    strb     r3, [ R0 ], # 1		\ Rd is now half-word aligned
    mov      r3, r3, lsr # 8
    strh     r3, [ R0 ], # 2		\ Rd is now full-word aligned
    mov      r3, r3 lsr # 16
    orr      r3, r3, r4 lsl # 8
    mov      r4, r4 lsr # 24
    orr      r4, r4, r5 lsl # 8
    mov      r5, r5 lsr # 24
    orr      r5, r5, r6 lsl # 8
    mov      r6, r6 lsr # 24
    orr      r6, r6, r7 lsl # 8
    mov      r7, r7 lsr # 24
    orr      r7, r7, r8 lsl # 8
    mov      r8, r8 lsr # 24
    orr      r8, r8, r9 lsl # 8
    mov      r9, r9, lsr # 24
    orr      r9, r9, r10 lsl # 8
    stmia    R0 ! { r3-r9 }
    mov      r10, r10 lsr # 24
    strb     r10, [ R0 ], # 1
    sub .s   r2, r2, # #32
  cc, until,

l: final_byte_copy
\ Adjust number of words because we overshot.
  add .s   r2, r2, # #32
  b .eq    fcmovex
\ We have at most 31 bytes left to copy after the loop, so
\ figure out what's left and copy them.
  sub .s   r2, r2, # 4
  cs, if,				\ bcc     2f
    begin,				\ 1:
      ldr      r3, [ R1 ], # 4
      strb     r3, [ R0 ], # 1
      mov      r3, r3, lsr # 8
      strh     r3, [ R0 ], # 2
      mov      r3, r3, lsr # 16
      strb     r3, [ R0 ], # 1
      b .eq    fcmovex
      sub .s   r2, r2, # 4
    cc, until,				\ bcs     1b
  endif,				\ 2:
  mov .s   r2, r2, lsl # #31
\ C = 1 if two bytes to copy,
\ N = 1 if one byte to copy,
\ C+N if three...
  ldrh .cs r3, [ R1 ], # 2		\ 2 of 2/3 bytes
  strb .cs r3, [ R0 ], # 1
  mov .cs  r3, r3 lsr # 8
  strb .cs r3, [ R0 ], # 1
  ldrb .mi r3, [ R1 ]			\ 1 of 1/3 bytes
  strb .mi r3, [ R0 ]
  b        fcmovex
end-code

code cmove	\ asrc adest len --
\ *G Copy *\i{len} bytes of memory forwards from *\i{asrc} to
\ ** *\i{adest}. In one test, this runs in 625 ms, whereas the
\ ** original code runs in 2777 ms.
  stmfd    rsp ! { r8, r9, r11 }	\ preserve FSP, LP and UP
  mov .s   r2, tos                	\ r2=len
  ldmfd    psp ! { r0, r1 }		\ r0=adest, r1=asrc, r2=len, tos restored
  b .eq    fcmovex			\ return if len=0, nothing to do

\ Use lower two bits of source and destination addresses to jump
\ into address-specific routines.
  and      r3, r1, # 3			\ bottom two bits of asrc
  orr      r3, r3, r0 lsl # 2		\ bottom two bits of adest
  and      r3, r3, # $0F		\ 2 + 2 bits = 16 cases
  ldr      pc, [ pc ++ r3 lsl # 2 ]	\ cell addressed offset
  nop

\ Table of address-specific routines.  Naming convention is x_d_s,
\ where _d is the low two bits of the destination and _s is the
\ low two bits of the source address.
  dd   x_0_0
  dd   x_0_1
  dd   x_0_2
  dd   x_0_3
  dd   x_1_0
  dd   x_1_1
  dd   x_1_2
  dd   x_1_3
  dd   x_2_0
  dd   x_2_1
  dd   x_2_2
  dd   x_2_3
  dd   x_3_0
  dd   x_3_1
  dd   x_3_2
  dd   x_3_3

l: fcmovex
  ldmfd    rsp ! { r8, r9, r11 }	\ restore FSP, LP and UP
  ldmfd    psp ! { tos }		\ restore TOS
  next,
end-code


\ *****
\ Tests
\ *****

0 [if]

decimal

code cmoveo	\ c-addr1 c-addr2 u --
\ Copy U bytes of memory forwards from C-ADDR1 to C-ADDR2.
\ This is the original CMOVE. 2777ms
  mov .s r3, tos                	\ r3=len
  ldmfd psp ! { r0, r1, tos }		\ r0=addr2, r1=addr1, r3=len, tos restored
  mov .eq pc, link			\ return if len=0, nothing to do
  orr     r2, r1, r0			\ check alignment
  orr     r2, r2, r3			\ R2 := adr1|addr2|len
  and .s  r4, r2, # 3			\ will be zero if aligned
  b .ne   cmovx3			\ if not aligned
L: CMOVX1
  ldr     r2, [ r1 ], # 4
  sub .s  r3, r3, # 4			\ hoisted for ARM9
  str     r2, [ r0 ], # 4
  b .ne CMOVX1
  next,
L: CMOVX3
  ldrb r2, [ r1 ], # 1
  sub .s r3, r3, # 1			\ hoisted for ARM9
  strb r2, [ r0 ], # 1
  b .ne CMOVX3
  next,
END-CODE

code cmovex	\ c-addr1 c-addr2 u --
\ Copy U bytes of memory forwards from C-ADDR1 to C-ADDR2.
\ This is a tweaked version of the original CMOVE. 2336 ms
\ About 14% faster than the basic one.
  mov .s r3, tos                	\ r3=len
  ldmfd    psp ! { r0, r1, tos }	\ r0=addr2, r1=addr1, r3=len, tos restored
  mov .eq  pc, link			\ return if len=0, nothing to do
  orr      r2, r1, r0			\ check alignment
  tst      r2, # 3			\ will be zero if aligned
  b .ne    CMOVXha			\ if not aligned
L: CMOVXa	\ addresses are aligned
  begin,
    cmp      r3, # #16
   cs, while,
    ldmia    r1 ! { r4-r7 }
    sub      r3, r3, # #16
    stmia    r0 ! { r4-r7 }
  repeat,
  cmp      r3, # 0
  mov .eq  pc, link
  begin,
    ldrb     r4, [ r1 ], # 1
    sub .s   r3, r3, # 1
    strb     r4, [ r0 ], # 1
  eq, until,
  next,
L: CMOVXha	\ check half aligned
  orr      r2, r1, r0			\ check alignment
  tst      r2, # 1			\ will be zero if half-aligned
  b .ne    CMOVXb			\ if not aligned, move bytes
  begin,
    cmp      r3, # 2
   cs, while,
    ldrh     r4, [ r1 ], # 2
    sub .s   r3, r3, # 2
    strh     r4, [ r0 ], # 2
  repeat,
  cmp      r3, # 0
  mov .eq  pc, link
L: CMOVXb	\ move bytes
  begin,
    ldrb     r4, [ r1 ], # 1
    sub .s   r3, r3, # 1
    strb     r4, [ r0 ], # 1
  eq, until,
  next,
END-CODE

ualign
256 buffer: fcsrc	\ -- addr
256 buffer: fcdest	\ -- addr

: initFCtest	\ --
\ Initialise the buffers
  fcsrc  256 0
  do i over c!  1+  loop
  drop
  fcdest 256 erase
;

: FCtest	\ so do --
\ Test a copy at the given offsets.
  initFCtest
  swap fcsrc +  swap fcdest +
  2dup 99 cmove  2dup 99 s= 0= if
    cr ." Bad: src=$" swap .dword ."  dest=$" .dword
  else
    2drop
  endif
;

: allFCtest	\ --
  3 0 do  3 0 do  i j FCtest  loop  loop
;

: XCtest	\ so do --
\ Test a copy at the given offsets.
  initFCtest
  swap fcsrc +  swap fcdest +
  2dup 99 cmovex  2dup 99 s= 0= if
    cr ." Bad: src=$" swap .dword ."  dest=$" .dword
  else
    2drop
  endif
;

: allXCtest	\ --
  3 0 do  3 0 do  i j XCtest  loop  loop
;

#10000 value #tests

: FCspeed	\ --
\ Test the fast CMOVE
  ticks
  #tests 0 ?do
    3 0 do  3 0 do
      fcsrc i + fcdest j + 252 fcmove
    loop  loop
  loop
  ticks swap - abs . ." ms"
;

: OCspeed	\ --
\ Test the original CMOVE
  ticks
  #tests 0 ?do
    3 0 do  3 0 do
      fcsrc i + fcdest j + 252 cmoveo
    loop  loop
  loop
  ticks swap - abs . ." ms"
;

: XCspeed	\ --
\ Test the tweaked original CMOVE
  ticks
  #tests 0 ?do
    3 0 do  3 0 do
      fcsrc i + fcdest j + 252 cmovex
    loop  loop
  loop
  ticks swap - abs . ." ms"
;

[then]


\ ======
\ *> ###
\ ======

decimal


