\ SFRSA1110.fth - SA-1110 peripheral definitions

((
Copyright (c) 2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


N.B. The memory controller for the SA-1110 is not the same as that
for the SA-1100.

Absolute addresses start with an '_' underscore character.
Offsets and masks do not.


To do
=====

Change history
==============
20010918 SFP001 First release
))

decimal


\ *******
\ ARM CPU 
\ *******

$80 equ I_BIT
$40 equ F_BIT
$20 equ T_BIT

$00000080 equ IRQDisable                      
$00000040 equ FIQDisable                      
$000000C0 equ InterruptMask                   
$0000001F equ ModeMask

$00000000 equ usr_26                        
$00000001 equ fiq_26
$00000002 equ irq_26
$00000003 equ svc_26
$00000010 equ usr_32                          
$00000011 equ fiq_32                          
$00000012 equ irq_32                          
$00000013 equ svc_32                          
$00000017 equ abt_32                          
$0000001B equ und_32                          
$0000001F equ sys_32                          
 
$10 equ ARM_MODE_USER
$11 equ ARM_MODE_FIQ
$12 equ ARM_MODE_IRQ
$13 equ ARM_MODE_SVC
$17 equ ARM_MODE_ABORT
$1B equ ARM_MODE_UNDEF
$1F equ ARM_MODE_SYS


\ ***********
\ Bit equates
\ ***********

$00000001 equ BIT0                            
$00000002 equ BIT1                            
$00000004 equ BIT2                            
$00000008 equ BIT3                            
$00000010 equ BIT4                            
$00000020 equ BIT5                            
$00000040 equ BIT6                            
$00000080 equ BIT7                            
$00000100 equ BIT8                            
$00000200 equ BIT9                            
$00000400 equ BIT10                           
$00000800 equ BIT11                           
$00001000 equ BIT12                           
$00002000 equ BIT13                           
$00004000 equ BIT14                           
$00008000 equ BIT15                           
$00010000 equ BIT16                           
$00020000 equ BIT17                           
$00040000 equ BIT18                           
$00080000 equ BIT19                           
$00100000 equ BIT20                           
$00200000 equ BIT21                           
$00400000 equ BIT22                           
$00800000 equ BIT23                           
$01000000 equ BIT24                           
$02000000 equ BIT25                           
$04000000 equ BIT26                           
$08000000 equ BIT27                           
$10000000 equ BIT28                           
$20000000 equ BIT29                           
$40000000 equ BIT30                           
$80000000 equ BIT31                           
         

\ *************
\ Memory sizing
\ *************

$00000400 equ SZ_1K                           
$00002000 equ SZ_8K                           
$00004000 equ SZ_16K                          
$00008000 equ SZ_32K                          
$00010000 equ SZ_64K                          
$00020000 equ SZ_128K                         
$00040000 equ SZ_256K                         
$00080000 equ SZ_512K                         
          
$00100000 equ SZ_1M                           
$00200000 equ SZ_2M                           
$00400000 equ SZ_4M                           
$00800000 equ SZ_8M                           
$01000000 equ SZ_16M                          
$02000000 equ SZ_32M                          
$04000000 equ SZ_64M                          
$08000000 equ SZ_128M                         
$10000000 equ SZ_256M                         
$20000000 equ SZ_512M                         
          
$40000000 equ SZ_1G                           


\ ********************************************
\ GPIO (General Purpose IO) REGISTER LOCATIONS
\ ********************************************
 
$90040000 equ _GPIO	\ GPIO Register Location
$00 equ gplr			\ GPIO Pin Level Register
$04 equ gpdr                 	\ GPIO Pin Direction Register
$08 equ gpsr			\ GPIO Pin Output Set Register
$0C equ gpcr			\ GPIO Pin Output Clear Register
$10 equ grer			\ GPIO Rising-Edge Detect Register
$14 equ gfer			\ GPIO Falling-Edge Detect Register
$18 equ gedr			\ GPIO Edge Detect Status Register
$1C equ gafr			\ GPIO Alternate Function Register


\ ========================
\ GPIO alternate functions
\ ========================

BIT27 equ gpaf_32KHZ_OUT	\ 32.768 khz Oscillator output
BIT26 equ gpaf_RCLK_OUT		\ Internal Clock/2
BIT25 equ gpaf_RTC_Clock	\ Trimmed 1 Hz clock

BIT15 equ gpaf_UART_RXD		\  UART1 RxD
BIT14 equ gpaf_UART_TXD		\  UART1 TxD


\ *********************************************************************************
\      Interrupt Controller
\ *********************************************************************************
 
$90050000 equ _IC	\ Interrupt controller
$00 equ icip			\ IRQ pending register 
$04 equ icmr			\ IRQ mask register 
$08 equ iclr			\ IRQ level register, 0=IRQ, 1=FIQ
$0C equ iccr			\ IRQ control register 
$10 equ icfp			\ FIQ pending register 
$20 equ icpr			\ IRQ pending register 

 
\ ==============
\ Interrupt bits
\ ==============
 
BIT0  equ GPIO0_INT             \ GPIO[0] Edge detect
BIT1  equ GPIO1_INT             \ GPIO[1] Edge detect
BIT2  equ GPIO2_INT             \ GPIO[2] Edge detect
BIT3  equ GPIO3_INT             \ GPIO[3] Edge detect
BIT4  equ GPIO4_INT             \ GPIO[4] Edge detect
BIT5  equ GPIO5_INT             \ GPIO[5] Edge detect
BIT6  equ GPIO6_INT             \ GPIO[6] Edge detect
BIT7  equ GPIO7_INT             \ GPIO[7] Edge detect
BIT8  equ GPIO8_INT             \ GPIO[8] Edge detect
BIT9  equ GPIO9_INT             \ GPIO[9] Edge detect
BIT10 equ GPIO10_INT            \ GPIO[10] Edge detect
BIT11 equ GPIO11_27_INT         \ or of GPIO[11] to GPIO[27] Edge detect
      
BIT12 equ LCD_INT                         
BIT13 equ UDC_INT                         
      
BIT15 equ UART1_INT                       
BIT16 equ UART2_INT                       
BIT17 equ UART3_INT                       
BIT18 equ MCP_INT                         
BIT19 equ SSP_INT                         
      
BIT20 equ DMA0_INT                        
BIT21 equ DMA1_INT                        
BIT22 equ DMA2_INT                        
BIT23 equ DMA3_INT                        
BIT24 equ DMA4_INT                        
BIT25 equ DMA5_INT                        
      
BIT26 equ OST0_INT                        
BIT27 equ OST1_INT                        
BIT28 equ OST2_INT                        
BIT29 equ OST3_INT                        
      
BIT30 equ RTC_HZ_INT                      
BIT31 equ RTC_AL_INT                      


\ ****************************************
\ RTC (Real Time Clock) Register Locations
\ ****************************************

$90010000 equ _RTC
$00 equ rtar			\ RTC Alarm Register
$04 equ rcnr			\ RTC Count Register
$08 equ rttr			\ RTC Timer Trim Register
$10 equ rtsr			\ RTC Status Register

BIT0 equ RTSR_AL		\ RTC Alarm Detected
BIT1 equ RTSR_HZ		\ 1 Hz Rising Edge Detected
BIT2 equ RTSR_ALE		\ RTC Alarm Interrupt Enable
BIT3 equ RTSR_HZE		\ 1 Hz Rising Interrupt Enable
 
 
\ **********************************************
\ OS Timer (Operating System) Register Locations
\ **********************************************
 
$90000000 equ _OS_T                            
$00 equ osmr0			\ OS-Timer Match Register 0
$04 equ osmr1			\ OS-Timer Match Register 1
$08 equ osmr2			\ OS-Timer Match Register 2
$0C equ osmr3			\ OS-Timer Match Register 3
$10 equ oscr			\ OS-Timer Counter Register
$14 equ ossr			\ OS-Timer Status Register
$18 equ ower			\ OS-Timer Watchdog Enable Register
$1C equ oier			\ OS-Timer Interrupt Enable Register
                
3687            equ mSEC_1	\ Count of Clock for 1 Millisec
mSEC_1 5 *      equ mSEC_5                          
mSEC_1 10 *     equ mSEC_10                         
mSEC_1 250 *    equ mSEC_250                        
mSEC_1 500 *    equ mSEC_500                        
mSEC_1 1000 *   equ SEC_1                           
 
\ --------------
\ OS - constants
\ --------------
\ OS Timer Watchdog Enable Register (OWER)
BIT0 equ OWER_WME		\ Watchdog Match Enable

\ OS Timer Interrupt Enable Register (OIER)
BIT0 equ OIER_E0                \ Interrupt Enable Channel 0
BIT1 equ OIER_E1                \ Interrupt Enable Channel 1
BIT2 equ OIER_E2                \ Interrupt Enable Channel 2
BIT3 equ OIER_E3                \ Interrupt Enable Channel 3

\ OS Timer Status Register (OSSR)
BIT0 equ OSSR_M0                \ Match Status Channel 0
BIT1 equ OSSR_M1                \ Match Status Channel 1
BIT2 equ OSSR_M2                \ Match Status Channel 2
BIT3 equ OSSR_M3                \ Match Status Channel 3


\ ********************************
\ POWER MANAGER REGISTER LOCATIONS
\ ********************************
 
$90020000 equ _PM	\ PM (Power Manager) Register Location
$00 equ pmcr			\ PM Control Register
$04 equ pssr			\ PM Sleep Status Register
$08 equ pspr			\ PM Scratch Pad Register
$0C equ pwer			\ PM Wakeup Enable Register
$10 equ pcfr			\ PM General Configuration Register
$14 equ ppcr			\ PM PLL Configuration Register
$18 equ pgsr			\ PM GPIO Sleep State Register
$1C equ posr			\ PM Oscillator Register
 
\ -------------------------
\ Power Manager - constants
\ -------------------------
 
BIT0 equ POSR_OOK          	\ Oscillator Status Register Oscillator OK(32.768kHz)
 
 
\ ------------------------------------------------
\ CORE CLOCK CONFIGURATION FIELD (CCF[4:0] im PPCR
\ ------------------------------------------------

\ Assumes 3.6864 MHz xtal
$0 equ cc_59Mhz                        
$1 equ cc_74Mhz                        
$2 equ cc_88Mhz                        
$3 equ cc_103Mhz                       
$4 equ cc_118Mhz                       
$5 equ cc_132Mhz                       
$6 equ cc_147Mhz                       
$7 equ cc_162Mhz                       
$8 equ cc_177Mhz                       
$9 equ cc_191Mhz                       
$A equ cc_206Mhz                       
$B equ cc_221Mhz

 
\ ***********************************
\ Reset Controller REGISTER LOCATIONS
\ ***********************************
 
$90030000 equ _RESET	\ Reset Register Location
  $00 equ rsrr			\ Reset Controller Software Reset Register
  $04 equ rcsr 			\ Reset Controller Status Register
  $08 equ tucr			\ Test Unit Control Reg.
              
BIT0 equ RCSR_HWR               \ Hardware Reset
BIT1 equ RCSR_SWR               \ Software Reset
BIT2 equ RCSR_WDR               \ Watchdog Reset
BIT3 equ RCSR_SMR               \ Sleep Mode Reset
 
\ ----------------------------
\ Reset Controller - constants
\ ----------------------------
 
BIT0 equ RSRR_SWR		\ Reset Controller Software Reset Register
 
 
\ ********************************************
\ MC (Memory Configuration) REGISTER LOCATIONS
\ ********************************************
 
$A0000000 equ _MC	\ MC Register Location
  $00 equ mdcnfg		\ MDCNFG DRAM Configuration Register
  $04 equ mdcas0		\ MDCAS0 DRAM CAS Waveform Shift Register 0
  $08 equ mdcas1		\ MDCAS1 DRAM CAS Waveform Shift Register 1
  $0C equ mdcas2		\ MDCAS2 DRAM CAS Waveform Shift Register 2
  $10 equ msc0			\ MSC0 Static Memory Control Register 0
  $14 equ msc1			\ MSC1 Static Memory Control Register 1
  $18 equ mecr			\ MECR Expansion Bus Configuration Register
  $1C equ mdrefr		\ DRAM refresh control register
  $20 equ mdcas20		\ CAS rotate reg 0 for DRAM banks 2/3
  $24 equ mdcas21		\ CAS rotate reg 1 for DRAM banks 2/3
  $28 equ mdcas22		\ CAS rotate reg 2 for DRAM banks 2/3
  $2C equ msc2			\ Static memory control reg 2
  $30 equ smcnfg		\ SMROM configuration 
       

decimal

\ **************
\ DMA controller
\ **************

$B0000000 equ _DMAC
_DMAC $00 + equ _DMAC0
_DMAC $20 + equ _DMAC1
_DMAC $40 + equ _DMAC2
_DMAC $60 + equ _DMAC3
_DMAC $80 + equ _DMAC4
_DMAC $A0 + equ _DMAC5

$00 equ DDAR
$04 equ DCSRsb
$08 equ DCSRcb
$0C equ DCSR
$10 equ DBSA
$14 equ DBTA
$18 equ DBSB
$1C equ DBTB


\ **************
\ LCD controller
\ **************

$B0100000 equ _LCD
  $00 equ lccr0
  $04 equ lcsr

  $10 equ DBAR1
  $14 equ DCAR1
  $18 equ DBAR2
  $1C equ DCAR2
  $20 equ LCCR1
  $24 equ LCCR2
  $28 equ LCCR3


\ *****
\ UARTs
\ *****

$80010000 equ _UART1	\ See also GPCLK below
$80030000 equ _UART2	\ See also HSSP below
$80050000 equ _UART3
 
$00 equ utcr0		\ UART Control Register 0
$04 equ utcr1           \ UART Control Register 1
$08 equ utcr2           \ UART Control Register 2
$0C equ utcr3           \ UART Control Register 3
$10 equ utcr4           \ UART Control Register 4
$14 equ utdr            \ UART Data Register

$1C equ utsr0           \ UART Status Register 0
$20 equ utsr1           \ UART Status Register 1
 
 
\ UART Line status bits.
\ UART Status Register 0
BIT0 equ UTSR0_TFS		\ transmit fifo service request 
BIT1 equ UTSR0_RFS              \ receive fifo service request 
BIT2 equ UTSR0_RID              \ receiver idle 
BIT3 equ UTSR0_RBB              \ receiver begin of break 
BIT4 equ UTSR0_REB              \ receiver end of break 
BIT5 equ UTSR0_EIF              \ error in fifo 
 
\ UART Status Register 1
BIT0 equ UTSR1_TBY              \ transmitter busy flag 
BIT1 equ UTSR1_RNE              \ receiver not empty (LSR_DR) 
BIT2 equ UTSR1_TNF              \ transmit fifo not full 
BIT3 equ UTSR1_PRE              \ parity read error (LSR_PE) 
BIT4 equ UTSR1_FRE              \ framing error (LSR_FE) 
BIT5 equ UTSR1_ROR              \ receive fifo overrun (LSR_OE) 


\ UART Line control bits.
\ UART Control Register 0
BIT0 equ UTCR0_PE                                       \ parity enable
BIT1 equ UTCR0_OES                                      \ Odd/Even Parity Select 
BIT2 equ UTCR0_SBS                                      \ stop bit select 
BIT3 equ UTCR0_DSS                                      \ data size select 
BIT4 equ UTCR0_SCE                                      \ sample clock enable 
BIT5 equ UTCR0_RCE                                      \ receive clock edge select 
BIT6 equ UTCR0_TCE                                      \ transmit clock edge select

\ UART Control Register 3
BIT0 equ UTCR3_RXE                                      \ receiver enable
BIT1 equ UTCR3_TXE                                      \ transmitter enable 
BIT2 equ UTCR3_BRK                                      \ break 
BIT3 equ UTCR3_RIE                                      \ receive FIFI interrupt enable 
BIT4 equ UTCR3_TIE                                      \ transmit FIFO interrupt enable 
BIT5 equ UTCR3_LBM                                      \ loop back Mode 

\ UART baud rates with 3.6864 Mhz
 
$000 equ BAUD_230400                      
$001 equ BAUD_115200                      
$003 equ BAUD_57600                       
$005 equ BAUD_38400                       
$00B equ BAUD_19200                       
$00F equ BAUD_14400                       
$017 equ BAUD_9600                        
$02F equ BAUD_4800                        
$05F equ BAUD_2400                        
$0BF equ BAUD_1200                           
$17f equ BAUD_600                           
$2ff equ BAUD_300                            
 
 
\ *********************************************************************************
\       Default adjustments for the UART-Ports
\ *********************************************************************************
\ 8 bit, no parity, 1 stop

UTCR0_DSS equ UART1_UTCR0                               \ UART1 Control Register 0
\ 8 bit, no parity, 1 stop

UTCR0_DSS equ UART2_UTCR0                               \ UART2 Control Register 0
\ 8 bit, no parity, 1 stop

UTCR0_DSS equ UART3_UTCR0                               \ UART3 Control Register 0
 
BAUD_115200 equ UART1_BAUD                              \ UART1 Control Register 1+2
BAUD_115200 equ UART2_BAUD                              \ UART2 Control Register 1+2
BAUD_115200 equ UART3_BAUD                              \ UART3 Control Register 1+2
 
\ enable TX and RX and RX-Interrupt

UTCR3_TXE UTCR3_RXE or UTCR3_RIE or equ UART1_UTCR3     \ UART1 Control Register 3
\ enable TX and RX and RX-Interrupt

UTCR3_TXE UTCR3_RXE or UTCR3_RIE or equ UART2_UTCR3     \ UART2 Control Register 3
\ enable TX and RX and RX-Interrupt

UTCR3_TXE UTCR3_RXE or UTCR3_RIE or equ UART3_UTCR3     \ UART3 Control Register 3


\ *****
\ GPCLK - see also UART1
\ *****

$80020000 equ _GPCLK

$0060 equ gpclkr0
$0064 equ gpclkr1

$006C equ gpclkr2
$0070 equ gpclkr3


\ ************************************
\ HSSP (High speed serial to parallel)
\ ************************************

$80040000 equ _HSSP                            
$90060028 equ _hscr2	\ HSSP Control Register 2 in PPC address space

$60 equ hscr0		\ HSSP Control Register 0
$64 equ hscr1		\ HSSP Control Register 1
$6C equ hsdr		\ HSSP Data Register
$74 equ hssr0		\ HSSP Status Register 0
$78 equ hssr1		\ HSSP Status Register 1

\ Line control bits.
\ HSSP Control Register 0
BIT0 equ HSCR0_ITR	\ IrDA Transmission Rate
BIT1 equ HSCR0_LBM	\ Loop Back Mode


\ *** 
\ MCP
\ ***

$80060000 equ _MCP
  $00 equ mccr0
  $08 equ mcdr0
  $0C equ mcdr1
  $10 equ mcdr2
  $18 equ mcsr
$90060030 equ _mccr1

 
\ *******************************
\ PPC (Peripheral Pin Controller)
\ *******************************

$90060000 equ PPC                             
  $00 equ ppdr		\ PPC Pin Direction Register
  $04 equ ppsr		\ PPC Pin State Register
  $08 equ ppar		\ PPC Pin Assignment Register
  $0C equ psdr		\ PPC Sleep Mode Direction Register
  $10 equ ppfr		\ PPC Pin Flag Register

\ PPC Line control bits.
\ PPC Flag Register

BIT0  equ PPFR_LCD	\ LCD Controller Flag (read only)
BIT12 equ PPFR_SP1_TX	\ Serial Port 1: SDLC/UART Transmit Flag (read only)
BIT13 equ PPFR_SP1_RX	\ Serial Port 1: SDLC/UART Receive Flag (read only)
BIT14 equ PPFR_SP2_TX	\ Serial Port 2: ICP Transmit Flag (read only)
BIT15 equ PPFR_SP2_RX	\ Serial Port 2: ICP Receive Flag (read only)
BIT16 equ PPFR_SP3_TX	\ Serial Port 3: UART Transmit Flag (read only)
BIT17 equ PPFR_SP3_RX	\ Serial Port 3: UART Receive Flag (read only)
BIT18 equ PPFR_SP4	\ Serial Port 4: MCP/SSP Flag (read only)


\ ****
\ Misc
\ ****


