\ sfrSTR91x.fth - ST STR91x special function registers

((
Copyright (c) 2006, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! sfrstr91x
\ *T STR91x special function registers
\ ===========
\ *P Absolute addresses start with an '_' underscore character.
\ ** Offsets and masks do not.

\ *P Do not put *\b{anything} in this file that generates code
\ ** for the target during compilation of this file. This file
\ ** is usually compiled first.
\ ** *\fo{INTERPRETER} and *\fo{COMPILER} definitions are fine,
\ ** but target definitions *\fo{must} be in another file.


\ ********
\ *S Tools
\ ********
\ *P All APB peripheral addresses are unbuffered addresses. Use
\ *\fo{U>B} to convert an unbuffered address to a buffered address.

interpreter
: u>b	$10000000 -  ;	\ uaddr -- baddr
\ *G Convert an unbuffered APB address to a buffered address.
target
compiler
: u>b	$10000000 -  ;
target


\ *P These tools ease conversion of C header files.

interpreter

: [per		\ -- 0
\ *G Start a new peripheral at offset 0.
  0
;

: vu32		\ offset -- offset+4
\ *G Define a 32 bit peripheral register.
  dup EQU  4 +
;

: vu16		\ offset -- offset+2
\ *G Define a 16 bit peripheral register.
  dup EQU  2 +
;

: vu8		\ offset -- offset+1
\ *G Define an 8 bit peripheral register.
  dup EQU  1 +
;

: vu32[]	\ offset n -- offset+n*4
\ *G Define an arry of n 32 bit registers
  over EQU  4 * +
;

: per]		\ offset --
\ *G Finish the definition of a peripheral block
  drop
;

target


\ ************************
\ Device numbers and masks
\ ************************
\ These are used when enabling clocks and taking peripherals in
\ and out of reset.

0 equ TIM01dev#		\ timers 0 and 1
  bit0 equ TIM01mask
1 equ TIM23dev#		\ timers 2 and 3
  bit1 equ TIM23mask
2 equ MCdev#		\ Motor Controller
  bit2 equ MCmask
3 equ UART0dev#		\ UARTs
  bit3 equ UART0mask
4 equ UART1dev#
  bit4 equ UART1mask
5 equ UART2dev#
  bit5 equ UART2mask
6 equ I2C0dev#		\ I2Cs
  bit6 equ I2C0mask
7 equ I2C1dev#
  bit7 equ I2C1mask
8 equ SSP0dev#		\ SSPs
  bit8 equ SSP0mask
9 equ SSP1dev#
  bit9 equ SSP1mask
10 equ CANdev#		\ CAN
  bit10 equ CANmask
11 equ ADCdev#		\ ADC
  bit11 equ ADCmask
12 equ WDGdev#		\ WDG
  bit12 equ WDGmask
13 equ WIUdev#		\ WIU
  bit13 equ WIUmask
14 equ GPIO0dev#	\ GPIOs
  bit14 equ GPIO0mask
15 equ GPIO1dev#
  bit15 equ GPIO1mask
16 equ GPIO2dev#
  bit16 equ GPIO2mask
17 equ GPIO3dev#
  bit17 equ GPIO3mask
18 equ GPIO4dev#
  bit18 equ GPIO4mask
19 equ GPIO5dev#
  bit19 equ GPIO5mask
20 equ GPIO6dev#
  bit20 equ GPIO6mask
21 equ GPIO7dev#
  bit21 equ GPIO7mask
22 equ GPIO8dev#
  bit22 equ GPIO8mask
23 equ GPIO9dev#
  bit23 equ GPIO9mask
24 equ RTCdev#
  bit24 equ RTCmask


\ ***
\ VIC
\ ***
\ The PL190 register names are used instead of the ST names.

$FFFFF000 equ _VIC	\ primary VIC
$FFFFF000 equ _VIC0	\ synonym
$FC000000 equ _VIC1	\ secondary VIC
  $00 equ VICIRQStatus	\ $Fffff000
  $04 equ VICFIQStatus	\ $Fffff004
  $08 equ VICRawIntr	\ $Fffff008
  $0C equ VICIntSelect	\ $Fffff00c
  $10 equ VICIntEnable	\ $Fffff010
  $14 equ VICIntEnClr	\ $Fffff014
  $18 equ VICSoftInt	\ $Fffff018
  $1C equ VICSoftIntClear \ $Fffff01c
  $20 equ VICProtection	\ $Fffff020
  $30 equ VICVectAddr	\ $Fffff030
  $34 equ VICDefVectAddr \ $Fffff034

 $100 equ VICVectAddr0	\ $Fffff100
 $104 equ VICVectAddr1	\ $Fffff104
 $108 equ VICVectAddr2	\ $Fffff108
 $10C equ VICVectAddr3	\ $Fffff10c
 $110 equ VICVectAddr4	\ $Fffff110
 $114 equ VICVectAddr5	\ $Fffff114
 $118 equ VICVectAddr6	\ $Fffff118
 $11C equ VICVectAddr7	\ $Fffff11c
 $120 equ VICVectAddr8	\ $Fffff120
 $124 equ VICVectAddr9	\ $Fffff124
 $128 equ VICVectAddr10	\ $Fffff128
 $12C equ VICVectAddr11	\ $Fffff12c
 $130 equ VICVectAddr12	\ $Fffff130
 $134 equ VICVectAddr13	\ $Fffff134
 $138 equ VICVectAddr14	\ $Fffff138
 $13C equ VICVectAddr15	\ $Fffff13c

 $200 equ VICVectCntl0	\ $Fffff200
 $204 equ VICVectCntl1	\ $Fffff204
 $208 equ VICVectCntl2	\ $Fffff208
 $20C equ VICVectCntl3	\ $Fffff20c
 $210 equ VICVectCntl4	\ $Fffff210
 $214 equ VICVectCntl5	\ $Fffff214
 $218 equ VICVectCntl6	\ $Fffff218
 $21C equ VICVectCntl7	\ $Fffff21c
 $220 equ VICVectCntl8	\ $Fffff220
 $224 equ VICVectCntl9	\ $Fffff224
 $228 equ VICVectCntl10	\ $Fffff228
 $22C equ VICVectCntl11	\ $Fffff22c
 $230 equ VICVectCntl12	\ $Fffff230
 $234 equ VICVectCntl13	\ $Fffff234
 $238 equ VICVectCntl14	\ $Fffff238
 $23C equ VICVectCntl15	\ $Fffff23c


\ The equates below satisfy the original INTARM3.FTH code.
_VIC0 VICIntEnClr + equ _VICINTENCLR
_VIC0 VICIntSelect + equ _VICINTSELECT
_VIC0 VICIntEnable + equ _VICINTENABLE
_VIC0 VICDefVectAddr + equ _VICDEFVECTADDR


\ ===========================
\ Interrupt numbers and masks
\ ===========================
\ Note that the STR91x family has only 16 interrupt sources per VIC.
\ To permit use of the dual PL190 code with CPUs that have 32 sources
\ per VIC, interrupts on the second VIC are numbered from 32 onwards.

interpreter
: inm:		\ bit# --
\ Generate the interrupt number and mask equates. Later code
\ treats interrupts 0..31 as being on VIC0, 32..63 on VIC1
\ and so on.
  dup equ				\ interrupt number
  $01F and 1 swap lshift equ		\ bit mask for relevant VIC
;
target

\ VIC0
0 inm: WDGint# WDGintMask
1 inm: SWint# SWintMask
2 inm: DccRXint# DccRXintMask
3 inm: DccTXint# DccTXintMask
4 inm: TIM0int# TIM0intMask
5 inm: TIM1int# TIM1intMask
6 inm: TIM2int# TIM2intMask
7 inm: TIM3int# TIM3intMask
8 inm: USBHIint# USBHIintMask
9 inm: USBLOint# USBLOintMask
10 inm: SCUint# SCUintMask
11 inm: EMACint# EMACintMask
12 inm: DMAint# DMAintMask
13 inm: CANint# CANintMask
14 inm: MCint# MCintMask
15 inm: ADCint# ADCintMask

32 inm: UART0int# UART0intMask
33 inm: UART1int# UART1intMask
34 inm: UART2int# UART2intMask
35 inm: I2C0int# I2C0intMask
36 inm: I2C1int# I2C1intMask
37 inm: SSP0int# SSP0intMask
38 inm: SSP1int# SSP1intMask
39 inm: LVDint# LVDintMask
40 inm: RTCint# RTCintMask
41 inm: WIUint# WIUintMask
42 inm: WIUgp0int# WIUgp0intMask
43 inm: WIUgp1int# WIUgp1intMask
44 inm: WIUgp2int# WIUgp2intMask
45 inm: WIUgp3int# WIUgp3intMask
46 inm: USBBRint# USBBRintMask
47 inm: PFQint# PFQintMask


\ *******************
\ Ethernet Controller
\ *******************

$7C000000 equ _ENET

[per
/* DMA Registers */
  vu32 ENET_SCR           /* DMA Status and Control Register         */
  vu32 ENET_IER           /* DMA Interrupt Sources Enable Register   */
  vu32 ENET_ISR           /* DMA Interrupt Status Register           */
  vu32 ENET_CCR           /* Clock Control Relation : HCLK, PCLK and ENET_CLK phase relations                */
  vu32 ENET_RXSTR         /* Rx DMA start Register                   */
  vu32 ENET_RXCR          /* Rx DMA Control Register                 */
  vu32 ENET_RXSAR         /* Rx DMA Base Address Register            */
  vu32 ENET_RXNDAR        /* Rx DMA Next Descriptor Address Register */
  vu32 ENET_RXCAR         /* Rx DMA Current Address Register         */
  vu32 ENET_RXCTCR        /* Rx DMA Current Transfer Count Register  */
  vu32 ENET_RXTOR         /* Rx DMA FIFO Time Out Register           */
  vu32 ENET_RXSR          /* Rx DMA FIFO Status Register             */
  vu32 ENET_TXSTR         /* Tx DMA start Register                   */
  vu32 ENET_TXCR          /* Tx DMA Control Register                 */
  vu32 ENET_TXSAR         /* Tx DMA Base Address Register            */
  vu32 ENET_TXNDAR        /* Tx DMA Next Descriptor Address Register */
  vu32 ENET_TXCAR         /* Tx DMA Current Address Register         */
  vu32 ENET_TXTCR         /* Tx DMA Current Transfer Count Register  */
  vu32 ENET_TXTOR         /* Tx DMA FIFO Time Out Register           */
  vu32 ENET_TXSR          /* Tx DMA FIFO Status Register             */

  drop $400
/* MAC Registers */
  vu32 ENET_MCR      /* ENET Control Register             */
  vu32 ENET_MAH      /* ENET Address High Register        */
  vu32 ENET_MAL      /* ENET Address Low Register         */
  vu32 ENET_MCHA     /* Multicast Address High Register   */
  vu32 ENET_MCLA     /* Multicast Address Low Register    */
  vu32 ENET_MIIA     /* MII Address Register              */
  vu32 ENET_MIID     /* MII Data Register                 */
  vu32 ENET_MCF      /* ENET Control Frame Register       */
  vu32 ENET_VL1      /* VLAN1 Register                    */
  vu32 ENET_VL2      /* VLAN2 register                    */
  vu32 ENET_MTS      /* ENET Transmission Status Register */
  vu32 ENET_MRS      /* ENET Reception Status Register    */
per]



\ **************************
\ DMA and Channelx Registers
\ **************************
\ AHB, 8 Channels

$78000000 equ _DMAC

\ Channelx
\ x can be ,0,1,2,3,4,5,6 or 7. There are eight Channels AHB BUS Master
\  Each channel is in a 32 byte block, where the blocks start at offset. 0x0100
[per
  vu32 DMA_SRCn      /* Channelx Source Address Register      */
  vu32 DMA_DESTn     /* Channelx Destination Address Register */
  vu32 DMA_LLIn      /* Channelx Lincked List Item Register   */
  vu32 DMA_CCn       /* Channelx Contol Register              */
  vu32 DMA_CNFn      /* Channelx Configuration Register       */
per]


\ DMA Controller 

[per
  vu32 DMA_ISR         /* Interrupt Status Register                    */
  vu32 DMA_TCISR       /* Terminal Count Interrupt Status Register     */
  vu32 DMA_TCICR       /* Terminal CountInterrupt Clear Register       */
  vu32 DMA_EISR        /* Error Interrupt Status Register              */
  vu32 DMA_EICR        /* Error Interrupt Clear Register               */
  vu32 DMA_TCRISR      /* Terminal Count Raw Interrupt Status Register */
  vu32 DMA_ERISR       /* Raw Error Interrupt Status Register          */
  vu32 DMA_ENCSR       /* Enabled Channel Status Register              */
  vu32 DMA_SBRR        /* Software Burst Request Register              */
  vu32 DMA_SSRR        /* Software Single Request Register             */
  vu32 DMA_SLBRR       /* Software Last Burst Request Register         */
  vu32 DMA_SLSRR       /* Software Last Single Request Register        */
  vu32 DMA_CNFR        /* Configuration Register                       */
  vu32 DMA_SYNR        /* Syncronization Register                      */
per]


\ *************************
\ External Memory Interface (EMI) Bankx Registers
\ *************************

$74000000 equ _EMI
  $00 equ EMI_BANK1
  $20 equ EMI_BANK2
  $40 equ EMI_BANK3
  $E0 equ EMI_BANK0

[per
  vu32 EMI_ICR		/* Bankx   Idle Cycle Control Register                    */
  vu32 EMI_RCR		/* Bankx   Read Wait State Control Register               */
  vu32 EMI_WCR		/* Bankx   Write Wait State Control Register              */
  vu32 EMI_OECR		/* Bankx   Output Enable Assertion Delay Control Register */
  vu32 EMI_WECR		/* Bankx   Write Enable Assertion Delay Control Register  */
  vu32 EMI_BCR		/* Bankx   Control Register                               */
per]


\ **********
\ USB device
\ **********

$70000000 equ _USBdev


\ *************
\ I2C interface, APB1, 2 off
\ *************

$5C00C000 equ _I2C0
$5C00D000 equ _I2C1
[per
  vu32 I2C_CR                 /* Control Register                */
  vu32 I2C_SR1                /* Status Register 1               */
  vu32 I2C_SR2                /* Status Register 2               */
  vu32 I2C_CCR                /* Clock Control Register          */
  vu32 I2C_OAR1               /* Own Address Register 1          */
  vu32 I2C_OAR2               /* Own Address Register 2          */
  vu32 I2C_DR                 /* Data Register                   */
  vu32 I2C_ECCR               /* Extended Clock Control Register */
per]


\ **************
\ WatchDog Timer
\ **************

$5C00B000 equ _WDG
[per
  vu16 WDG_CR  2 +        /* Control Register        */
  vu16 WDG_PR  2 +        /* Prescalar Register       */
  vu16 WDG_VR  2 +        /* Pre-load Value Register */
  vu16 WDG_CNT  2 +       /* Counter Register        */
  vu16 WDG_SR  2 +        /* Status Register         */
  vu16 WDG_MR  2 +        /* Mask Register           */
  vu16 WDG_KR  2 +        /* Key Register            */
per]


\ ***************************
\ Analog to Digital Converter, APB1
\ ***************************

$5C00A000 equ _ADC
[per
  vu16 ADC_CR  2 +         /* Control Register               */
  vu16 ADC_CCR  2 +        /* Channel Configuration Register */
  vu16 ADC_HTR  2 +        /* Higher Threshold Register      */
  vu16 ADC_LTR  2 +        /* Lower Threshold Register       */
  vu16 ADC_CRR  2 +        /* Compare Result Register        */
  vu16 ADC_DR0  2 +        /* Data Register for Channel 0    */
  vu16 ADC_DR1  2 +        /* Data Register for Channel 1    */
  vu16 ADC_DR2  2 +        /* Data Register for Channel 2    */
  vu16 ADC_DR3  2 +        /* Data Register for Channel 3    */
  vu16 ADC_DR4  2 +        /* Data Register for Channel 4    */
  vu16 ADC_DR5  2 +        /* Data Register for Channel 5    */
  vu16 ADC_DR6  2 +        /* Data Register for Channel 6    */
  vu16 ADC_DR7  2 +        /* Data Register for Channel 7    */
  vu16 ADC_PRS  2 +        /* Prescaler Value Register       */
per]


\ ***************************
\ CAN Controller Area Network, APB1
\ ***************************

$5C009000 equ _CAN		\ base address
  $0100 equ /CAN		\ size of CAN block

$060 equ /MsgObj		\ size of message object block
[per
  vu16 CANIF_CRR  2 +		/* IFn Command request Register       */
  vu16 CANIF_CMR  2 +		/* IFn Command Mask Register          */
  vu16 CANIF_M1R  2 +		/* IFn Message Mask 1 Register        */
  vu16 CANIF_M2R  2 +		/* IFn Message Mask 2 Register        */
  vu16 CANIF_A1R  2 +		/* IFn Message Arbitration 1 Register */
  vu16 CANIF_A2R  2 +		/* IFn Message Arbitration 2 Register */
  vu16 CANIF_MCR  2 +		/* IFn Message Control Register       */
  vu16 CANIF_DA1R  2 +	        /* IFn DATA A 1 Register              */
  vu16 CANIF_DA2R  2 +	        /* IFn DATA A 2 Register              */
  vu16 CANIF_DB1R  2 +	        /* IFn DATA B 1 Register              */
  vu16 CANIF_DB2R  2 +	        /* IFn DATA B 2 Register              */
per]

[per
  vu16 CAN_CR  2 +		/* Control Register                */
  vu16 CAN_SR  2 +	        /* Status Register                 */
  vu16 CAN_ERR  2 +		/* Error counter Register          */
  vu16 CAN_BTR  2 +		/* Bit Timing Register             */
  vu16 CAN_IDR  2 +		/* Interrupt Identifier Register   */
  vu16 CAN_TESTR  2 +		/* Test Register                   */
  vu16 CAN_BRPR  2 +		/* BRP Extension Register          */
  drop $20
  dup equ CAN_IF1  /MsgObj +	\ CAN Interface 1
  dup equ CAN_IF2  /MsgObj +	\ CAN Interface 2
  drop $100
  vu16 CAN_TXR1R  2 +		/* Transmission request 1 Register */
  vu16 CAN_TXR2R  2 +		/* Transmission Request 2 Register */
  drop $120
  vu16 CAN_ND1R  2 +		/* New Data 1 Register             */
  vu16 CAN_ND2R  2 +		/* New Data 2 Register             */
  drop $140
  vu16 CAN_IP1R  2 +		/* Interrupt Pending 1 Register    */
  vu16 CAN_IP2R  2 +		/* Interrupt Pending 2 Register    */
  drop $160
  vu16 CAN_MV1R  2 +		/* Message Valid 1 Register        */
  vu16 CAN_MV2R  2 +		/* Message VAlid 2 Register        */
per]


\ ***
\ SSP, APB1, 2 channels
\ ***

$5C007000 equ _SSP0
$5C008000 equ _SSP1
[per
  vu16 SSP_CR0  2 +        /* Control Register 1                   */
  vu16 SSP_CR1  2 +        /* Control Register 2                   */
  vu16 SSP_DR  2 +         /* Data Register                        */
  vu16 SSP_SR  2 +         /* Status Register                      */
  vu16 SSP_PR  2 +         /* Clock Prescale Register              */
  vu16 SSP_IMSCR  2 +      /* Interrupt Mask Set or Clear Register */
  vu16 SSP_RISR  2 +       /* Raw Interrupt Status Register        */
  vu16 SSP_MISR  2 +       /* Masked Interrupt Status Register     */
  vu16 SSP_ICR  2 +        /* Interrupt Clear Register             */
  vu16 SSP_DMACR  2 +      /* DMA Control Register                 */
per]


\ ****
\ UART, APB1, 3 channels
\ ****

$5C004000 equ _UART0
$5C005000 equ _UART1
$5C006000 equ _UART2

  $00 equ UART_DR	/* Data Register                                               */
  $04 equ UART_RSECR	/* Receive Status Register (read)/Error Clear Register (write) */

  $18 equ UART_FR	/* Flag Register                                               */

  $20 equ UART_ILPR	/* IrDA Low-Power counter Register                             */
  $24 equ UART_IBRD	/* Integer Baud Rate Divisor Register                          */
  $28 equ UART_FBRD	/* Fractional Baud Rate Divisor Register                       */
  $2C equ UART_LCR	/* Line Control Register, High byte                            */
  $30 equ UART_CR	/* Control Register                                            */
  $34 equ UART_IFLS	/* Interrupt FIFO Level Select Register                        */
  $38 equ UART_IMSC	/* Interrupt Mask Set/Clear Register                           */
  $3C equ UART_RIS	/* Raw Interrupt Status Register                               */
  $40 equ UART_MIS	/* Masked Interrupt Status Register                            */
  $44 equ UART_ICR	/* Interrupt Clear Register                                    */
  $48 equ UART_DMACR	/* DMA Control Register                                        */


\ *****************
\ MC, Motor Control, APB1
\ *****************

$5C003000 equ _MC
[per
  vu16 MC_TCPT  2 +          /* Tacho Capture Register           */
  vu16 MC_TCMP  2 +          /* Tacho Compare Register           */
  vu16 MC_IPR  2 +           /* Input Pending Register           */
  vu16 MC_TPRS  2 +          /* Tacho Prescaler Register         */
  vu16 MC_CPRS  2 +          /* PWM Counter Prescaler Register   */
  vu16 MC_REP  2 +           /* Repetition Counter Register      */
  vu16 MC_CMPW  2 +          /* Compare Phase W Preload Register */
  vu16 MC_CMPV  2 +          /* Compare Phase V Preload Register */
  vu16 MC_CMPU  2 +          /* Compare Phase U Preload Register */
  vu16 MC_CMP0  2 +          /* Compare 0 Preload Register       */
  vu16 MC_PCR0  2 +          /* Peripheral Control Register 0    */
  vu16 MC_PCR1  2 +          /* Peripheral Control Register 1    */
  vu16 MC_PCR2  2 +          /* Peripheral Control Register 2    */
  vu16 MC_PSR  2 +           /* Polarity Selection Register      */
  vu16 MC_OPR  2 +           /* Output Peripheral Register       */
  vu16 MC_IMR  2 +           /* Interrupt Mask Register          */
  vu16 MC_DTG  2 +           /* Dead Time Generator Register     */
  vu16 MC_ESC  2 +           /* Emergency Stop Clear Register    */
per]


\ *******************
\ System Control Unit, APB1
\ *******************

$5C002000 equ _SCU
[per
  vu32 SCU_CLKCNTR	/* Clock Control Register                       */
  vu32 SCU_PLLCONF	/* PLL Configuration Register                   */
  vu32 SCU_SYSSTATUS	/* System Status Register                       */
  vu32 SCU_PWRMNG	/* Power Management Register                    */
  vu32 SCU_ITCMSK	/* Interrupt Mask Register                      */
  vu32 SCU_PCGR0	/* Peripheral Clock Gating Register 0           */
  vu32 SCU_PCGR1	/* Peripheral Clock Gating Register 1           */
  vu32 SCU_PRR0		/* Peripheral Reset Register 0                  */
  vu32 SCU_PRR1		/* Peripheral Reset Register 1                  */
  vu32 SCU_MGR0		/* Idle Mode Mask Gating Register 0             */
  vu32 SCU_MGR1		/* Idle Mode Mask Gating Register 1             */
  vu32 SCU_PECGR0	/* Peripheral Emulation Clock Gating Register 0 */
  vu32 SCU_PECGR1	/* Peripheral Emulation Clock Gating Register 1 */
  vu32 SCU_SCR0		/* System Configuration Register 0              */
  vu32 SCU_SCR1		/* System Configuration Register 1              */
  vu32 SCU_SCR2		/* System Configuration Register 2              */
  4 +
  8 vu32[] SCU_GPIOOUT	/* GPIO Output Registers                      */
  8 vu32[] SCU_GPIOIN	/* GPIO Input Registers                       */
  #10 vu32[] SCU_GPIOTYPE	/* GPIO Type Registers                        */
  vu32 SCU_GPIOEMI	/* GPIO EMI Selector Register                 */
  vu32 SCU_WKUPSEL	/* Wake-Up Selection Register                 */
  8 +
  vu32 SCU_GPIOANA	/* GPIO Analog mode Register                  */
per]


\ ***
\ RTC, APB1
\ ***

$5C001000 equ _RTC
[per
  vu32 RTC_TR         /* Time Register       */
  vu32 RTC_DTR        /* Date Register       */
  vu32 RTC_ATR        /* Alarm time Register */
  vu32 RTC_CR         /* Control Register    */
  vu32 RTC_SR         /* Status Register     */
  vu32 RTC_MILR       /* Millisec Register   */
per]


\ **************
\ AHB/APB BRIDGE, APB0/1 registers strcture
\ **************

$5C000000 equ _AHB/APB1
$58000000 equ _AHB/APB0
[per
  vu32 APB_BSR        /* Bridge Status Register            */
  vu32 APB_BCR        /* Bridge Configuration Register     */
  vu32 APB_PAER       /* Peripheral Address Error register */
per]


\ ****
\ GPIO, ABP0, 10 off
\ ****

$58006000 equ _GPIO

$58006000 equ _GPIO0
$58007000 equ _GPIO1
$58008000 equ _GPIO2
$58009000 equ _GPIO3
$5800A000 equ _GPIO4
$5800B000 equ _GPIO5
$5800C000 equ _GPIO6
$5800D000 equ _GPIO7
$5800E000 equ _GPIO8
$5800F000 equ _GPIO9

  $000 equ GPIO_DR	/* Data Register                    */
  $400 equ GPIO_DIR	/* Data Direction Register          */
  $420 equ GPIO_SEL	/* Mode Control Register */


\ *********
\ TIM Timer, ABP0, 4 off
\ *********

$58002000 equ _TIM0
$58003000 equ _TIM1
$58004000 equ _TIM2
$58005000 equ _TIM3

[per
  vu16 TIM_IC1R  2 +        /* Input Capture 1 Register  */
  vu16 TIM_IC2R  2 +        /* Input Capture 2 Register  */
  vu16 TIM_OC1R  2 +        /* Output Compare 1 Register */
  vu16 TIM_OC2R  2 +        /* Output Compare 2 Register */
  vu16 TIM_CNTR  2 +        /* Counter Register          */
  vu16 TIM_CR1  2 +         /* Control Register 1        */
  vu16 TIM_CR2  2 +         /* Control Register 2        */
  vu16 TIM_SR  2 +          /* Status Register           */
per]


\ *********************
\ WUI Wake-Up Interrupt, APB0
\ *********************

$58001000 equ _WUI
[per
  vu32  WUI_CTRL   /* Control Register            */
  vu32  WUI_MR     /* Mask Register               */
  vu32  WUI_TR     /* Trigger Register            */
  vu32  WUI_PR     /* Pending Register            */
  vu32  WUI_INTR   /* Software Interrupt Register */
per]


\ **************************
\ FMI Flash Memory Interface, AHB
\ **************************

$54000000 equ _FMI
[per
  vu32 FMI_BBSR		/* Boot Bank Size Register                */
  vu32 FMI_NBBSR	/* Non-Boot Bank Size Register            */
  4 +
  vu32 FMI_BBADR	/* Boot Bank Base Address Register        */
  vu32 FMI_NBBADR	/* Non-Boot Bank Base Address Register    */
  4 +
  vu32 FMI_CR           /* Control Register                       */
  vu32 FMI_SR           /* Status Register                        */
  vu32 FMI_BCE5ADDR     /* BC Fifth Entry Target Address Register */
per]


((

/*******************************************************************************
*                         Memory Mapping of STR91x                             *
*******************************************************************************/

#define AHB_APB_BRDG0_U    (0x58000000) /* AHB/APB Bridge 0 UnBuffered Space */
#define AHB_APB_BRDG0_B    (0x48000000) /* AHB/APB Bridge 0 Buffered Space   */

#define AHB_APB_BRDG1_U    (0x5C000000) /* AHB/APB Bridge 1 UnBuffered Space */
#define AHB_APB_BRDG1_B    (0x4C000000) /* AHB/APB Bridge 1 Buffered Space   */

#define AHB_EMI_U          (0x74000000) /* EMI UnBuffered Space */
#define AHB_EMI_B          (0x64000000) /* EMI Buffered Space   */

#define AHB_DMA_U          (0x78000000) /* DMA UnBuffered Space */
#define AHB_DMA_B          (0x68000000) /* DMA Buffered Space   */

#define AHB_ENET_MAC_U     (0x7C000400) /* ENET_MAC  UnBuffered Space */
#define AHB_ENET_MAC_B     (0x6C000000) /* ENET_MAC  Buffered Space   */

#define AHB_ENET_DMA_U     (0x7C000000) /* ENET_DMA  Unbuffered Space */
#define AHB_ENET_DMA_B     (0x6C000400) /* ENET_DMA  Buffered Space    */

#define AHB_VIC1_U         (0xFC000000) /* Secondary VIC1 UnBuffered Space */
#define AHB_VIC0_U         (0xFFFFF000) /* Primary VIC0 UnBuffered Space   */

#define AHB_FMI_U          (0x54000000) /* FMI Unbuffered Space */
#define AHB_FMI_B          (0x44000000) /* FMI buffered Space   */

/*******************************************************************************
*                Addresses related to the VICs' peripherals                    *
*******************************************************************************/

#define VIC0_BASE          (AHB_VIC0_U)
#define VIC1_BASE          (AHB_VIC1_U)

/*******************************************************************************
*                    Addresses related to the EMI banks                        *
*******************************************************************************/

#define AHB_EMIB3_OFST      (0x00000040)   /* Offset of EMI bank3 */
#define AHB_EMIB2_OFST      (0x00000020)   /* Offset of EMI bank2 */
#define AHB_EMIB1_OFST      (0x00000000)   /* Offset of EMI bank1 */
#define AHB_EMIB0_OFST      (0x000000E0)   /* Offset of EMI bank0 */

/*******************************************************************************
*                 Addresses related to the DMA peripheral                      *
*******************************************************************************/

#define AHB_DMA_Channel0_OFST    (0x00000100)   /* Offset of Channel 0 */
#define AHB_DMA_Channel1_OFST    (0x00000120)   /* Offset of Channel 1 */
#define AHB_DMA_Channel2_OFST    (0x00000140)   /* Offset of Channel 2 */
#define AHB_DMA_Channel3_OFST    (0x00000160)   /* Offset of Channel 3 */
#define AHB_DMA_Channel4_OFST    (0x00000180)   /* Offset of Channel 4 */
#define AHB_DMA_Channel5_OFST    (0x000001A0)   /* Offset of Channel 5 */
#define AHB_DMA_Channel6_OFST    (0x000001C0)   /* Offset of Channel 6 */
#define AHB_DMA_Channel7_OFST    (0x000001E0)   /* Offset of Channel 7 */

/*******************************************************************************
*                 Addresses related to the APB0 sub-system                     *
*******************************************************************************/

#define APB_WIU_OFST       (0x00001000)   /* Offset of WIU   */
#define APB_TIM0_OFST      (0x00002000)   /* Offset of TIM0  */
#define APB_TIM1_OFST      (0x00003000)   /* Offset of TIM1  */
#define APB_TIM2_OFST      (0x00004000)   /* Offset of TIM2  */
#define APB_TIM3_OFST      (0x00005000)   /* Offset of TIM3  */
#define APB_GPIO0_OFST     (0x00006000)   /* Offset of GPIO0 */
#define APB_GPIO1_OFST     (0x00007000)   /* Offset of GPIO1 */
#define APB_GPIO2_OFST     (0x00008000)   /* Offset of GPIO2 */
#define APB_GPIO3_OFST     (0x00009000)   /* Offset of GPIO3 */
#define APB_GPIO4_OFST     (0x0000A000)   /* Offset of GPIO4 */
#define APB_GPIO5_OFST     (0x0000B000)   /* Offset of GPIO5 */
#define APB_GPIO6_OFST     (0x0000C000)   /* Offset of GPIO6 */
#define APB_GPIO7_OFST     (0x0000D000)   /* Offset of GPIO7 */
#define APB_GPIO8_OFST     (0x0000E000)   /* Offset of GPIO8 */
#define APB_GPIO9_OFST     (0x0000F000)   /* Offset of GPIO9 */

/*******************************************************************************
*                   Addresses related to the APB1 sub-system                   *
*******************************************************************************/

#define APB_RTC_OFST       (0x00001000) /* Offset of RTC               */
#define APB_SCU_OFST       (0x00002000) /* Offset of System Controller */
#define APB_MC_OFST        (0x00003000) /* Offset of Motor Control     */
#define APB_UART0_OFST     (0x00004000) /* Offset of UART0             */
#define APB_UART1_OFST     (0x00005000) /* Offset of UART1             */
#define APB_UART2_OFST     (0x00006000) /* Offset of UART2             */
#define APB_SSP0_OFST      (0x00007000) /* Offset of SSP0              */
#define APB_SSP1_OFST      (0x00008000) /* Offset of SSPI              */
#define APB_CAN_OFST       (0x00009000) /* Offset of CAN               */
#define APB_ADC_OFST       (0x0000A000) /* Offset of ADC               */
#define APB_WDG_OFST       (0x0000B000) /* Offset of WDG               */
#define APB_I2C0_OFST      (0x0000C000) /* Offset of I2C0              */
#define APB_I2C1_OFST      (0x0000D000) /* Offset of I2C1              */

/*----------------------------------------------------------------------------*/
/*----------------------------- Unbuffered Mode ------------------------------*/
/*----------------------------------------------------------------------------*/

#ifndef Buffered 

/*******************************************************************************
*                  AHBAPB peripheral Unbuffered Base Address                   *
*******************************************************************************/

#define AHBAPB0_BASE           (AHB_APB_BRDG0_U)
#define AHBAPB1_BASE           (AHB_APB_BRDG1_U)

/*******************************************************************************
*                  ENET peripheral Unbuffered Base Address                     *
*******************************************************************************/

#define ENET_MAC_BASE          (AHB_ENET_MAC_U)
#define ENET_DMA_BASE          (AHB_ENET_DMA_U)

/*******************************************************************************
*                  DMA peripheral Unbuffered Base Address                      *
*******************************************************************************/

#define DMA_BASE           (AHB_DMA_U)

/*******************************************************************************
*                  EMI peripheral Unbuffered Base Address                      *
*******************************************************************************/

#define EMI_BASE           (AHB_EMI_U)    

/*******************************************************************************
*                  FMI peripheral Unbuffered Base Address                      *
*******************************************************************************/

#define FMI_BASE           (AHB_FMI_U)


#else /* Buffered */

/*----------------------------------------------------------------------------*/
/*------------------------------ Buffered Mode -------------------------------*/
/*----------------------------------------------------------------------------*/

/*******************************************************************************
*                   AHBAPB peripheral Buffered Base Address                    *
*******************************************************************************/

#define AHBAPB0_BASE           (AHB_APB_BRDG0_B)
#define AHBAPB1_BASE           (AHB_APB_BRDG1_B)

/*******************************************************************************
*                  ENET peripheral Unbuffered Base Address                     *
*******************************************************************************/

#define ENET_MAC_BASE          (AHB_ENET_MAC_B)
#define ENET_DMA_BASE          (AHB_ENET_DMA_B)

/*******************************************************************************
*                    DMA peripheral Buffered Base Address                      *
*******************************************************************************/

#define DMA_BASE           (AHB_DMA_B)

/*******************************************************************************
*                      EMI peripheral Buffered Base Address                    *
*******************************************************************************/

#define EMI_BASE           (AHB_EMI_B)

/*******************************************************************************
*                      FMI peripheral Buffered Base Address                    *
*******************************************************************************/

#define FMI_BASE           (AHB_FMI_B)

#endif /* Buffered */

/*******************************************************************************
*                          DMA channels Base Address                           *
*******************************************************************************/
#define DMA_Channel0_BASE  (DMA_BASE + AHB_DMA_Channel0_OFST)
#define DMA_Channel1_BASE  (DMA_BASE + AHB_DMA_Channel1_OFST)
#define DMA_Channel2_BASE  (DMA_BASE + AHB_DMA_Channel2_OFST)
#define DMA_Channel3_BASE  (DMA_BASE + AHB_DMA_Channel3_OFST)
#define DMA_Channel4_BASE  (DMA_BASE + AHB_DMA_Channel4_OFST)
#define DMA_Channel5_BASE  (DMA_BASE + AHB_DMA_Channel5_OFST)
#define DMA_Channel6_BASE  (DMA_BASE + AHB_DMA_Channel6_OFST)
#define DMA_Channel7_BASE  (DMA_BASE + AHB_DMA_Channel7_OFST)

/*******************************************************************************
*                     EMI Banks peripheral Base Address                        *
*******************************************************************************/

#define EMI_Bank0_BASE  (EMI_BASE + AHB_EMIB0_OFST)
#define EMI_Bank1_BASE  (EMI_BASE + AHB_EMIB1_OFST)
#define EMI_Bank2_BASE  (EMI_BASE + AHB_EMIB2_OFST)
#define EMI_Bank3_BASE  (EMI_BASE + AHB_EMIB3_OFST)

/*******************************************************************************
*                     APB0 Peripherals' Base addresses                         *
*******************************************************************************/

#define WIU_BASE           (AHBAPB0_BASE + APB_WIU_OFST)
#define TIM0_BASE          (AHBAPB0_BASE + APB_TIM0_OFST)
#define TIM1_BASE          (AHBAPB0_BASE + APB_TIM1_OFST)
#define TIM2_BASE          (AHBAPB0_BASE + APB_TIM2_OFST)
#define TIM3_BASE          (AHBAPB0_BASE + APB_TIM3_OFST)
#define GPIO0_BASE         (AHBAPB0_BASE + APB_GPIO0_OFST)
#define GPIO1_BASE         (AHBAPB0_BASE + APB_GPIO1_OFST)
#define GPIO2_BASE         (AHBAPB0_BASE + APB_GPIO2_OFST)
#define GPIO3_BASE         (AHBAPB0_BASE + APB_GPIO3_OFST)
#define GPIO4_BASE         (AHBAPB0_BASE + APB_GPIO4_OFST)
#define GPIO5_BASE         (AHBAPB0_BASE + APB_GPIO5_OFST)
#define GPIO6_BASE         (AHBAPB0_BASE + APB_GPIO6_OFST)
#define GPIO7_BASE         (AHBAPB0_BASE + APB_GPIO7_OFST)
#define GPIO8_BASE         (AHBAPB0_BASE + APB_GPIO8_OFST)
#define GPIO9_BASE         (AHBAPB0_BASE + APB_GPIO9_OFST)

/*******************************************************************************
*                      APB1 Peripherals' Base addresses                        *
*******************************************************************************/

#define RTC_BASE           (AHBAPB1_BASE + APB_RTC_OFST)
#define SCU_BASE           (AHBAPB1_BASE + APB_SCU_OFST)
#define MC_BASE            (AHBAPB1_BASE + APB_MC_OFST)
#define UART0_BASE         (AHBAPB1_BASE + APB_UART0_OFST)
#define UART1_BASE         (AHBAPB1_BASE + APB_UART1_OFST)
#define UART2_BASE         (AHBAPB1_BASE + APB_UART2_OFST)
#define SSP0_BASE          (AHBAPB1_BASE + APB_SSP0_OFST)
#define SSP1_BASE          (AHBAPB1_BASE + APB_SSP1_OFST)
#define CAN_BASE           (AHBAPB1_BASE + APB_CAN_OFST)
#define ADC_BASE           (AHBAPB1_BASE + APB_ADC_OFST)
#define WDG_BASE           (AHBAPB1_BASE + APB_WDG_OFST)
#define I2C0_BASE          (AHBAPB1_BASE + APB_I2C0_OFST)
#define I2C1_BASE          (AHBAPB1_BASE + APB_I2C1_OFST)

/*******************************************************************************
*                                IPs' declaration                              *
*******************************************************************************/

/*------------------------------ Non Debug Mode ------------------------------*/

#ifndef DEBUG

/*********************************** AHBAPB ***********************************/

#define AHBAPB0               ((AHBAPB_TypeDef *)AHBAPB0_BASE)
#define AHBAPB1               ((AHBAPB_TypeDef *)AHBAPB1_BASE)

/************************************* EMI ************************************/

#define EMI                ((EMI_TypeDef *)EMI_BASE)

/************************************* DMA ************************************/

#define DMA                ((DMA_TypeDef *)DMA_BASE)
#define DMA_Channel0       ((DMA_Channel_TypeDef *)DMA_Channel0_BASE)
#define DMA_Channel1       ((DMA_Channel_TypeDef *)DMA_Channel1_BASE)
#define DMA_Channel2       ((DMA_Channel_TypeDef *)DMA_Channel2_BASE)
#define DMA_Channel3       ((DMA_Channel_TypeDef *)DMA_Channel3_BASE)
#define DMA_Channel4       ((DMA_Channel_TypeDef *)DMA_Channel4_BASE)
#define DMA_Channel5       ((DMA_Channel_TypeDef *)DMA_Channel5_BASE)
#define DMA_Channel6       ((DMA_Channel_TypeDef *)DMA_Channel6_BASE)
#define DMA_Channel7       ((DMA_Channel_TypeDef *)DMA_Channel7_BASE)

/************************************* EMI ************************************/

#define EMI_Bank0         ((EMI_Bank_TypeDef *)EMI_Bank0_BASE)
#define EMI_Bank1         ((EMI_Bank_TypeDef *)EMI_Bank1_BASE)
#define EMI_Bank2         ((EMI_Bank_TypeDef *)EMI_Bank2_BASE)
#define EMI_Bank3         ((EMI_Bank_TypeDef *)EMI_Bank3_BASE)

/************************************* ENET_MAC ************************************/

#define ENET_MAC              ((ENET_MAC_TypeDef *)ENET_MAC_BASE)

/************************************* ENET_DMA ************************************/

#define ENET_DMA              ((ENET_DMA_TypeDef *)ENET_DMA_BASE)

/************************************* FMI ************************************/

#define FMI                ((FMI_TypeDef *)FMI_BASE)

/************************************* VIC ************************************/

#define VIC0               ((VIC_TypeDef *)VIC0_BASE)
#define VIC1               ((VIC_TypeDef *)VIC1_BASE)

/*******************************************************************************
*                              APB0 Peripherals'                               *
*******************************************************************************/
#define WIU                ((WIU_TypeDef *)WIU_BASE)
#define TIM0               ((TIM_TypeDef *)TIM0_BASE)
#define TIM1               ((TIM_TypeDef *)TIM1_BASE)
#define TIM2               ((TIM_TypeDef *)TIM2_BASE)
#define TIM3               ((TIM_TypeDef *)TIM3_BASE)
#define GPIO0              ((GPIO_TypeDef *)GPIO0_BASE)
#define GPIO1              ((GPIO_TypeDef *)GPIO1_BASE)
#define GPIO2              ((GPIO_TypeDef *)GPIO2_BASE)
#define GPIO3              ((GPIO_TypeDef *)GPIO3_BASE)
#define GPIO4              ((GPIO_TypeDef *)GPIO4_BASE)
#define GPIO5              ((GPIO_TypeDef *)GPIO5_BASE)
#define GPIO6              ((GPIO_TypeDef *)GPIO6_BASE)
#define GPIO7              ((GPIO_TypeDef *)GPIO7_BASE)
#define GPIO8              ((GPIO_TypeDef *)GPIO8_BASE)
#define GPIO9              ((GPIO_TypeDef *)GPIO9_BASE)
/*******************************************************************************
*                              APB1 Peripherals'                               *
*******************************************************************************/
#define RTC                ((RTC_TypeDef *)RTC_BASE)
#define SCU                ((SCU_TypeDef *)SCU_BASE)
#define MC                 ((MC_TypeDef *)MC_BASE)
#define UART0              ((UART_TypeDef *)UART0_BASE)
#define UART1              ((UART_TypeDef *)UART1_BASE)
#define UART2              ((UART_TypeDef *)UART2_BASE)
#define SSP0               ((SSP_TypeDef *)SSP0_BASE)
#define SSP1               ((SSP_TypeDef *)SSP1_BASE)
#define CAN                ((CAN_TypeDef *)CAN_BASE)
#define ADC                ((ADC_TypeDef *)ADC_BASE)
#define WDG                ((WDG_TypeDef *)WDG_BASE)
#define I2C0               ((I2C_TypeDef *)I2C0_BASE)
#define I2C1               ((I2C_TypeDef *)I2C1_BASE)
#define ENET_MAC           ((ENET_MAC_TypeDef *)ENET_MAC_BASE)
#define ENET_DMA           ((ENET_DMA_TypeDef *)ENET_DMA_BASE)

#else   /* DEBUG */

/*-------------------------------- Debug Mode --------------------------------*/

EXT AHBAPB_TypeDef         *AHBAPB0;
EXT AHBAPB_TypeDef         *AHBAPB1;
EXT DMA_TypeDef            *DMA;
EXT DMA_Channel_TypeDef    *DMA_Channel0;
EXT DMA_Channel_TypeDef    *DMA_Channel1;
EXT DMA_Channel_TypeDef    *DMA_Channel2;
EXT DMA_Channel_TypeDef    *DMA_Channel3;
EXT DMA_Channel_TypeDef    *DMA_Channel4;
EXT DMA_Channel_TypeDef    *DMA_Channel5;
EXT DMA_Channel_TypeDef    *DMA_Channel6;
EXT DMA_Channel_TypeDef    *DMA_Channel7;
EXT EMI_Bank_TypeDef       *EMI_Bank0;
EXT EMI_Bank_TypeDef       *EMI_Bank1;
EXT EMI_Bank_TypeDef       *EMI_Bank2;
EXT EMI_Bank_TypeDef       *EMI_Bank3;
EXT FMI_TypeDef            *FMI;
EXT VIC_TypeDef            *VIC0;
EXT VIC_TypeDef            *VIC1;
EXT WIU_TypeDef            *WIU;
EXT TIM_TypeDef            *TIM0;
EXT TIM_TypeDef            *TIM1;
EXT TIM_TypeDef            *TIM2;
EXT TIM_TypeDef            *TIM3;
EXT GPIO_TypeDef           *GPIO0;
EXT GPIO_TypeDef           *GPIO1;
EXT GPIO_TypeDef           *GPIO2;
EXT GPIO_TypeDef           *GPIO3;
EXT GPIO_TypeDef           *GPIO4;
EXT GPIO_TypeDef           *GPIO5;
EXT GPIO_TypeDef           *GPIO6;
EXT GPIO_TypeDef           *GPIO7;
EXT GPIO_TypeDef           *GPIO8;
EXT GPIO_TypeDef           *GPIO9;
EXT RTC_TypeDef            *RTC;
EXT SCU_TypeDef            *SCU;
EXT MC_TypeDef             *MC;
EXT UART_TypeDef           *UART0;
EXT UART_TypeDef           *UART1;
EXT UART_TypeDef           *UART2;
EXT SSP_TypeDef            *SSP0;
EXT SSP_TypeDef            *SSP1;
EXT CAN_TypeDef            *CAN;
EXT ADC_TypeDef            *ADC;
EXT WDG_TypeDef            *WDG;
EXT I2C_TypeDef            *I2C0;
EXT I2C_TypeDef            *I2C1;
EXT ENET_MAC_TypeDef       *ENET_MAC;
EXT ENET_DMA_TypeDef       *ENET_DMA;


#endif  /* DEBUG */

#endif  /* __91x_MAP_H*/

/******************* (C) COPYRIGHT 2006 STMicroelectronics *****END OF FILE****/

))


\ ======
\ *> ###
\ ======

decimal

