\ ETHERCOM.FTH - Generic I/O for Ethernet drivers

((
Copyright (c) 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20020618 SFP001 First release
))

\ ***********
\ *! ethercom
\ *T Ethernet and IP devices generic I/O
\ ***********

\ ****************************
\ *S Internet Protocol Devices
\ ****************************

\ *P In order to ease changing devices and to permit
\ ** systems with multiple ports, version 3 and above of
\ ** the Powernet TCP/IP stack use a generalised interface
\ ** to the hardware, which is usually an Ethernet driver
\ ** or a serial driver.

\ *P The interface consists of a vector (table) of XTs
\ ** corresponding to the particular function required.
\ ** The layout of the table is as follows:
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device addresses and state
\ **   ' MyDiscard ,	\ 9: Discard current receive packet

\ *P by default, the system code requires a default IP device
\ ** called IPDevice. IPDevice will be used when PowerNet is
\ ** started.

decimal

cell +User IPDVec	\ -- addr
\ *G IPDVec is a USER variable which contains the address
\ ** of the current Internet Protocol Device (IPD) vector.
\ ** All tasks must initialise IPDVEC before using the 
\ ** ETHERCOM interface.
\ *E   IPDevice IPDVec !

: IPDinit	\ --
\ *G Initialise (open) the current IP device.
  IPDVec @ ( 0 cells + ) @C execute
;

: IPDterm	\ --
\ *G Shutdown (close) the current IP device.
  IPDVec @ [ 1 cells + ] literal @C execute
;

: IPDRx?	\ -- flag ; check receive char
\ *G Return true if a packet is available at the current IP
\ ** device.
  IPDVec @ [ 2 cells + ] literal @C execute
;

: IPDRx	        \ buff size -- len ; receive packet
\ *G Receive a packet into buff of size bytes, returning len the
\ ** number of bytes received. Use IPDRX? to check that a packet
\ ** is available.
  IPDVec @ [ 3 cells + ] literal @C execute
;

; IPDTx?	\ len -- flag ; check TX capability
\ *G Return true if the current IP device can send a packet
\ ** of size len bytes.
  IPDVec @ [ 4 cells + ] literal @C execute
;

: IPDTx		\ buff len -- ; send packet
\ *G Transmit the given packet.
  IPDVec @ [ 5 cells + ] literal @C execute
;

: IPDGetAddr	\ -- ipaddr netaddr flags
\ *G Returns: a pointer to the IP address used by this
\ ** device, or 0 if it has not been set yet: a pointer
\ ** to the network address of the device, e.g. the Ethernet
\ ** MAC address or 0 if not set or irrelevant: and a set of flags
\ ** which are currently 0 for IPv4 and Ethernet/SLIP/PPP devices.
  IPDVec @ [ 6 cells + ] literal @C execute
;

: IPDSetAddr	\ ipaddr netaddr flags --
\ *G Consumes: a pointer to the IP address used by this
\ ** device, or 0 for no change: a pointer
\ ** to the network address of the device, e.g. the Ethernet
\ ** MAC address or 0 for no change: and a set of flags
\ ** which are currently 0 for IPv4 and Ethernet/SLIP/PPP devices.
  IPDVec @ [ 7 cells + ] literal @C execute
;

: IPDSave	\ --
\ *G Save the device state to non-volatile storage so that
\ ** it can be reloaded at the next power up or IPDinit.
  IPDVec @ [ 8 cells + ] literal @C execute
;

: IPDDiscard	\ --
\ *G Discard the current receive packet, usually because
\ ** PowerNet has run out of buffer space. This is only
\ ** valid if IPDRX? has returned true to say that a packet
\ ** is available.
  IPDVec @ [ 9 cells + ] literal @C execute
;


\ ******
\ *> ###
\ ******


