\ XMODEMTXRX.FTH - XMODEM file receive tools

((
Copyright (c) 2001, 2002, 2004
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20040903 MGS005	adapt/improve for GSL bootloader - cope with long delays
20040209 SFP004 Modified for LPC2106 with 512 byte Flash writes,
		Receive only.
20030924 SFP003 Overhauled receive code to make	it minimal
		for single chip Flash programmers.
20021014 SFP002 Modified for COMMON directory.
20010717 EJB000 First release
))


\ ==================
\ *! minxmodemrx
\ *T XMODEM Receiver and Transmitter
\ ==================

\ *S Introduction
\ *P This file implements the XMODEM 128-byte protocol for
\ ** receive only.

\ *P No test code is provided for this file as the system has
\ ** been tested by comparison of transferred binary files.

\ *S Words in MinXmodemRx.fth

decimal


\ ==========================
\ *N Constants and variables
\ ==========================

\ Characters
$001 equ START	$001 equ SOH
$004 equ EOT
$006 equ ACK
$015 equ NAK
$018 equ CAN

\ Error codes
$0101 equ blkerror			\ error codes
$0102 equ wrongblk
$0103 equ noreply
$0104 equ crcerror
$0105 equ overflow

$0100 equ maxerrs			\ max errors before we bomb

\ Variables
cell buffer: BLK#			\ Current Block No.
cell buffer: CUR-ADDRESS                \ Current host address
cell buffer: LAST-BLOCK                 \ last block we can receive
cell buffer: rxerrcnt			\ counter for rx errors
cell buffer: ^FlBuffer			\ pointer to Flash data buffer
cell buffer: rxstatus			\ receiver status

compiler
: X-buffer	\ -- addr
\ *G Returns the address of the Xmodem data receive buffer.
\ ** This version is for systems that call *\fo{ToFlash}.
  ^FlBuffer @
;
target

((
#128 Buffer: x-buffer	\ -- addr
\ *G Returns the address of the Xmodem data receive buffer.
\ ** This version is for systems that do not call *\fo{ToFlash}.
))

\ ===================
\ *N XMODEM reception
\ ===================
\ *P This code is a much smaller version of that
\ ** provided in COMMON\XMODEMTXRX.FTH. It was originally written
\ ** for the Phips LPC2xxx family to handle installation of a new
\ ** binary image into internal Flash. For this reason it handles
\ ** all initialisation itself and assumes that interrupts are
\ ** disabled.

\ *P It is assumed that the word *\fo{ToFlash ( src dest -- )}
\ ** is used to program each 128 byte block into Flash.

\ *P It is also assumed that A routine \*fo(uDelay ( n -- )} is
\ ** available that can produce short delays. You should provide
\ ** one tuned to your hardware.

: TimedKey	\ -- ch ; as KEY but returns -1 on timeout
\ *G As KEY but returns -1 on timeout. This version is for a 60MHz LPC2xxx
  #100000 0 do				\ approx 100ms
    ser-key?
    if  ser-key  unloop exit  then
  loop
  -1
;
((
: TimedKey	\ -- ch
\ *G As KEY but returns -1 on timeout.
\ ** This version is for a 16MHz AT91.
  #10000 0       		\ AT91 @ 16MHz ; MGS005
  do				\ approx 100ms
    ser-key?
    if  ser-key  unloop exit  then
  loop
  -1
;
))

[undefined] ser-flush [if]
: ser-flush	\ -- ; flush the link input
\ *G Flush all input character from the host/target link.
\ ** Wait to see if link goes quiet for 100mS.
  begin
    timedkey 0<				\ MGS005
  until
;
[then]

: send-ack	\ -- ; send ACK
\ *G Transmit an ACK character.
  ACK ser-emit
;

: send-nak	\ -- ; Transmit a NAK character
\ *G Transmit a NAK character.
  NAK ser-emit
;

: send-can	\ -- ; send CAN character
\ *G Transmit a CAN character.
  CAN ser-emit  ( rxstatus on )		\ MGS005
;


: Get-BlockData	\ -- cksum ; get block from host
\ *G Receive a 128 byte XMODEM data block from the host.
  0
  X-buffer #128 bounds do		\ Get 128 byte Block
    TimedKey dup i c!			\ save byte to buffer
    + $0FF and                          \ Add to Check-Sum
  loop
;

: CancelXfer	\ --
\ *G Cancel transfer by sending 2 CAN characters.
  send-can send-can			\ 2 CAN's aborts transfer
  maxerrs rxstatus !			\ so we get out ; MGS005
;

: ?toomanyerrs	\ --
\ *G Cancel reception if too many errors have occurred.
  ser-flush
  1 rxerrcnt +!
  rxerrcnt @ maxerrs > if
    CancelXfer         			 \ so we exit
  else
    send-nak  rxstatus off	         \ so we try again
  endif
;

: From-Buffer	\ -- ; copy 128 bytes from X-BUFFER
\ *G Move 128 bytes from X-BUFFER into the memory pointed to
\ ** by CUR-ADDRESS. The user must supply a routine *\fo{ToFlash}
\ ** ( src dest -- ) which programs the data at src into Flash
\ ** at dest.
  X-buffer cur-address @ ToFlash	\ program 128 bytes
  #128 cur-address +!
;

: Get-Block	\ --
\ *G Receive an XMODEM 128 byte data block from the host, processing
\ ** the header and checksum data.
  TimedKey TimedKey			\ get block and inverse
  over + $0FF <> if
    ?toomanyerrs  \ send-can	        \ MGS005
    drop  exit				\ lose incoming block number
  endif
  Blk# @ $00FF and <> if		\ correct block?
     ?toomanyerrs  exit
  endif
  Get-BlockData				\ get the data
  Timedkey <> if			\ get single cksum byte
    ?toomanyerrs  exit
  endif
  Blk# @ last-block @ > if
    CancelXfer  exit
  endif
  From-Buffer				\ copy data to memory
  send-ack				\ all OK send ACK
  1 Blk# +!				\ bump for next block
  rxerrcnt off				\ clear error count again
;

#100 >udelay /100us
\ *G *\fo{>uDelay ( n -- n')} defines the value required by
\ ** *\fo(uDelay} below for a delay of n microseconds.

((
#100 equ /100us 	\ ~125uSec
\ *G If *\fo{>uDelay} is not available, determine this value
\ ** experimentally.
))

[undefined] uDelay [if]
: uDelay   \ n -- ; short delay
\ *G Provides a short delay of n iterations. The actual delay
\ ** time is CPU and memory dependent.
  begin  1- dup  0= until
  drop
;
[then]

: key/1s	\ -- flag
\ *G Waits for one second or until a key is pressed,
\ ** returning true if a key was pressed.
  #10000
  begin
    ser-key? 0= over and
   while
    /100us udelay  1-
  repeat
  drop
  ser-key?
;

((
: waitforresponse	\ -- ; wait for host to respond
\ *G Wait for the host to respond for up to 1 second. If the
\ ** host does not respond, a NOREPLY status is set in the
\ ** variable RXSTATUS.
  key/1s 0=   rxstatus !
\  if  rxstatus on  endif		\ flag no response
;
))

: waitResponse ( --)
\ *G Wait for upt 60 seconds for a reply, sending
\ ** a NAK after each 1 second timeout
  rxstatus on				\ indicates no reply
  60 0 do
    key/1s
    if  rxstatus off  leave  then	\ received something
    send-nak				\ triggers resend of previous block
  loop
;

[-short-branches
: Bin-Up	\ addr len -- status ; status 0 = GOOD
\ *G Upload (receive) a block of data of the given size into memory
\ ** using the XMODEM 128 byte block protocol. An error status is
\ ** returned, 0 indicating success.
\  cr ." Start Xmodem transmitter on host" cr
  rxstatus off				\ start with good status
  rxerrcnt off				\ and no errors
  ser-flush				\ flush input
  7 rshift last-block !			\ calculate last block number
  CUR-ADDRESS !				\ Start from addr
  1 BLK# !				\ first block number

  send-NAK 				\ NAK requests single byte checksum
  begin					\ wait for remote startup
    key/1s 0=				\ done if a reply within a second
   while
    send-NAK 				\ then send NAK if still no key
  repeat

  begin					\ outer loop handles blocks ; MGS005
    waitresponse
    rxstatus @ 0= if 		        \ check if we got a response
      case  ser-key				\ get response
        EOT of  rxerrcnt @
		if  send-nak  			\ ? out of sync
		else  EOT rxstatus !  send-ack  \ all finished
		then
	    endof
        CAN of  rxerrcnt @
		if  send-nak
		else  CAN rxstatus !  send-ack  \ remote cancelled
		then
	    endof
        SOH of  Get-block  endof		\ start of data block
        	?toomanyerrs   			\ handle other chars ; MGS005
	endcase
    else
      ?toomanyerrs			\ no response ; MGS005
    endif
    rxstatus @				\ loop until all done or fatal error
  until

  rxstatus @ dup EOT = if
    drop 0				\ returns 0 for good transfer
  else
    send-can
  endif
  ser-flush
;
short-branches]

((
: RecvXmodem	\ addr len -- len' status ; status=0=good
\ *G Upload (receive) a block of data of the given size into memory
\ ** using the XMODEM 128 byte block protocol. The number of bytes
\ ** correctly received and an error status are
\ ** returned, 0 indicating success.
  over >r  bin-up  cur-address @ r> - swap
;
))

\ ======
\ *> ###
\ ======


