\ Target block words for use with XSHELL v2/3 - NOT for AIDE and POWERTERM

((
Copyright (c) 1991..2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

20000530 SFP001 Converted to v6.1 target

Changes:	See RELEASE.TXT for full details
))

\ ===========
\ *> romforth
\ *S Block support
\ ===========
\ *P *\i{BLOCKS.FTH} provides support for the transfer of 1k blocks
\ ** between a host PC and the embedded target. Although the use
\ ** of blocks as source screens is now obsolete, blocks are
\ ** still useful for the transfer of binary data, especially
\ ** for data loggers which use paged RAM or Flash for data
\ ** storage. AIDE version 3 or above is required.

\ *P Despite the comment about the obsolescence of screens, the
\ ** code in *\i{BLOCKS.FTH} provides screen file support for legacy
\ ** systems.



\ **********
\ *N Primitives
\ **********

internal

decimal

: BLK-READ	\ addr blk# --
\ *G Reads one block from host to addr.
  ^R ser-emit			\ send read code
  synch-to-host			\ synchronise host & target
  send-block#			\ send block no.
  1024 bounds			\ formulate buffer addresses
  DO
    ser-key i c!		\ get 8 bits, save in buffer
  LOOP
  ACK ser-emit			\ send ack to complete transfer
  Wait-Ack/Nack Disk-error !	\ get status response
;

: BLK-WRITE	\ addr blk# --
\ *G Sends given block number to host.
  ^W ser-emit			\ send write code
  synch-to-host			\ synchronise host & target
  send-block#			\ send block no.
  1024 bounds			\ formulate buffer addresses
  DO
    I C@ ser-emit		\ send 8 bits from buffer
  LOOP
  Wait-Ack/Nack Disk-error !	\ get status response
;

external


\ *****************
\ *N Application words
\ *****************

32bit? [if]
  $7FFFFFFF equ initFIRST
  $80000000 equ update-mask
[then]
16bit? [if]
  $7FFF equ initFIRST
  $8000 equ update-mask
[then]

$400 equ /block			\ -- size ; size of a block
\ *G The size of a block.
cell /block + equ /first	\ -- size ; size of FIRST buffer
/first buffer: FIRST		\ -- addr ; first element is block no.
\ *G The data buffer. The first cell contains the block number and is
\ ** followed by *\fo{/BLOCK} data bytes.

: SAVE-BUFFERS	\ --
\ *G Save the data buffer if it has been modified.
  FIRST @ 0<			\ buffer marked as changed?
  IF				\ if so....
    FIRST cell+		        \  leave buffer address
    FIRST @ initFIRST AND       \  clear update flag
    DUP FIRST !		        \  buffer-addr blk# --
    BLK-WRITE		        \  write this changed block
  endif			        \ or else don't save it
;

: EMPTY-BUFFERS \ --
\ *G Mark the data buffer as empty.
  FIRST /first ERASE		\ clear buffer to zeroes
  initFIRST FIRST !		\ clear resident block number
;

: UPDATE        \ --
\ *G Mark the data buffer as modified.
  FIRST @  update-mask OR  FIRST !	\ mark resident block as changed
;

: FLUSH         \ --
\ *G Force a changed buffer to be uploaded to the host.
  SAVE-BUFFERS  EMPTY-BUFFERS  \ flush any changes to disk
;

: BLOCK		\ u -- addr
\ *G Addr is the address of the first character of the block buffer
\ ** assigned to mass-storage block u. An ambiguous condition exists
\ ** if u is not an available block number. If block u is already in
\ ** a block buffer, a-addr is the address of that block buffer. If
\ ** block u is not already in memory and there is an unassigned
\ ** block buffer, transfer block u from mass storage to an unassigned
\ ** block buffer. Addr is the address of that block buffer. If
\ ** block u is not already in memory and there are no unassigned
\ ** block buffers, unassign a block buffer. If the block in that
\ ** buffer has been UPDATEd, transfer the block to mass storage
\ ** and transfer block u from mass storage into that buffer. Addr
\ ** is the address of that block buffer. At the conclusion of the
\ ** operation, the block buffer pointed to by addr is the current
\ ** block buffer and is assigned to u.
  DUP  FIRST @ initFIRST AND =	\ already in ?
  IF
    DROP                        \ yes, discard
  ELSE
    SAVE-BUFFERS                \ no, save current buffer
    FIRST cell+ OVER BLK-READ   \ read new
    FIRST !                     \ set blk#
  ENDIF
  FIRST cell+  ;                \ return addr (fixed)


\ *********************
\ *N Block file management
\ *********************

internal

: (LINE)        \ line# blk# -- addr #chars
  >R 040 /block */MOD R> +
  BLOCK + 040                 \ step line by line through block buffer
;

: -trailing     \ addr len -- addr len' ; strip trailing spaces
\ *G This word strips the trailing spaces from a string. It is most useful when
\ ** displaying lines from a screen to strip the trailing spaces from the
\ ** 64 byte fixed length lines.
  dup 0
  ?do  2dup + 1- c@ bl <> ?leave 1-  loop
;

: .LINE		\ line# blk# --
\ *G Display line# from blk#.
  (LINE) -trailing TYPE		\ show contents of line
;

: ?LOADING	\ --
\ *G THROW #-501 if input from console.
  BLK @	 0=			\ BLK==0 : console
  #-501 ?throw			\ error if not loading
;

external

: LIST		\ blk# -- ; display screen given
\ *G Display screen blk# as 16 lines of 64 characters.
  CLS DUP SCR !			\ clear screen, set variable
  ." Screen " .			\ print header
  $010 0			\ the number of lines
  DO
    CR	I 1+ 3 .R SPACE		\ for each line print line no.
    I SCR @ .LINE		\ and line from block
    KEY? ?LEAVE			\ check if early finish
  LOOP
  CR				\ new line
;

: L		\ --
\ *G LIST the current screen in SCR.
  Scr @ List  ;

: N		\ --
\ *G LIST the Next screen.
  Scr @ 1+  initFirst and  List  ;

: P		\ --
\ *G LIST the Previous screen.
  Scr @ 1-  0 max  List	 ;

: Index		\ n1 n2 --
\ *G Display the top lines in the (inclusive) range of screens.
  Cr  1+ Swap
  ?DO  I Block Drop
       Halt? Disk-Error @ Or ?LEAVE
       Cr I 3 .R Space	0 I .Line
  LOOP	;

: Qx		\ --
\ *G INDEX all available screens.
  0 initFIRST Index  ;

: LOAD		\ blk# -- ; compile given screen
\ *G Save the current input-source specification. Store u in BLK
\ ** (thus making block u the input source and setting the input
\ ** buffer to encompass its contents), set >IN to zero, and interpret.
\ ** When the parse area is exhausted, restore the prior input
\ ** source specification. Other stack effects are due to the words LOADed.
  BLK @ >R  >IN @ >R		\ preserve blk# & input position
  0 >IN !  BLK !		\ set up for this block number
  ?LOADING			\ check for block 0
  INTERPRET			\ interpret from this block
  R> >IN !  R> BLK !		\ finished: restore pointers
;

: THRU		\ first last --
\ *G LOAD screens from first to last in that order.
  1+ SWAP			\ form limit & index
  DO  I LOAD  LOOP
;

: -->		\ --
\ *G LOAD the next screen.
  ?LOADING			\ must be loading
  0 >IN !  1 BLK +!		\ step to next block
; immediate			\ this word is immediate:
				\ a word may cover 2+ screens

: $using	\ $addr --
\ *G Select the file specified by the counted string at $addr as
\ ** the current block/screen file.
  ^u send$			\ send string to host
;
\ NB $USING in this implementation does not return a t/f flag
\ as the host version does. The host takes care of creating
\ non existent files.


: using		\ "<file>" --
\ *G Use in the form "USING <filename>" to select the current
\ ** block/screen file. The AIDE file server is required.
\ ** AIDE takes care of creating non-existent files.
  bl word			\ get string from keyboard
  $using			\ open it on host
;


\ ======
\ *> ###
\ ======

decimal

