\ ROM Forth XMODEM file upload tool       

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

20000530 SFP001 Overhaul for v6.1 targets

Changes:	See RELEASE.TXT for full details
))

\ *********************
\ Constants & Variables
\ *********************

internal

hex
01 equ SOH                         \ File transfer
04 equ EOT                         \   constants
decimal

variable BLK#                           \ Current Block No.
variable CUR-ADDRESS                    \ Current host address
variable LAST-BYTES                     \ #bytes mod 128 decimal


\ **********
\ Primitives
\ **********

decimal

: x-buffer	\ -- addr ; return address of buffer
  here 64 +  ;                          \ just above here

: To-Buffer	\ -- ;move data into buffer
  CUR-ADDRESS @ X-buffer 128 cmove      \ Move 128 bytes from
  128 cur-address +!  ;                 \ memory pointed to by
                                        \ CUR-ADDRESS into buff
: ?Ack          \ -- ; wait for ACK
  ser-key ACK <>                        \ wait for response
  IF  EOT ser-emit                      \ flag end of XMODEM
      1 abort" File Transfer error"     \ MUST be ACK
  ENDIF  ;

: Send-Block	\ Blk# -- ; send block of given number
  SOH ser-emit                          \ Start of header
  dup ser-emit                          \ Block Number
  invert ser-emit			\ 1's complement Block#
  0  X-buffer 128 bounds                \ Send 128 byte Block
  DO  i c@                              \ get byte from buffer
      dup ser-emit                      \ transmit it
      + 255 and                         \ Add to Check-Sum
  LOOP                                  \
  ser-emit                              \ Send Check-Sum
  ?Ack  ;                               \ wait response

: Calc-Last-Bytes	\ #cells -- #blks
  0 128 um/mod                          \ use 128 byte chunks
  swap last-bytes !                     \ no. of remaining bytes
  1 blk# !  ;                           \ start with block 1

: Send-Last-Blk		\ -- 
  last-bytes @ ?dup                     \ any bytes left ?
  if  x-buffer 128 blank                \ erase buffer
      cur-address @ x-buffer rot cmove  \ copy last bits to buff
      blk# @ send-block                 \ send block
  endif  ;


\ **********
\ User words
\ **********

External

: Bin-Down	\ addr #bytes --
  cr cr ." Start XMODEM receiver - [PgDn],1 with ProComm" cr
  ^Y ser-emit                           \ trigger for PowerTerm
  swap CUR-ADDRESS !                    \ Start from addr
  calc-last-bytes                       \ This is for last Block
  BEGIN ser-key NAK = UNTIL             \ Wait for receiver ...
  1+ 1                                  \ block numbers from 1
  ?DO  to-buffer                        \ Transfer mem to buffer
       i send-block                     \ Send this block
       1 BLK# +!                        \ Count blocks sent
  LOOP                                  \
  send-last-blk                         \ Check for last Block
  EOT ser-emit  ?Ack                    \ terminate XMODEM
  cr ." File transfer successful" cr  ;

