\ ROM tools

((
Released for use with the MPE Cross Compiler by:

MicroProcessor Engineering
133 Hill Lane
Shirley
Southampton SO1 5AF
England

tel: +44 (0)23 8063 1441
tel: +44 (0)23 8033 9691
))


\ ===========
\ *! romtools
\ *T Romtools.fth
\ ===========
\ *P The file *\i{Common/Romtools.fth} contains debugging tools
\ ** and various utilities.

\ ******************
\ *S Debugging tools
\ ******************
\ *P If you only want the debug tools, compile *\i{Common/dump.fth}
\ ** or *\i{Common/Devtools.fth} (32 bit systems) rather than
\ ** *\i{Common/Romtools.fth}.

: .BYTE         \ b8 --
\ *G Display a byte as two hex digits.
  base @ >r hex
  0 <# # # #> TYPE
  r> base !
;

: .WORD         \ w16 --
\ *G Display a 16 bit word as four hex digits.
  base @ >r hex
  0 <# # # # # #> TYPE
  r> base !
;

32bit? [if]
: .DWORD        \ l --
\ *G Display a 32 bit item as eight hex digits.
  base @ >r hex
  0 <# # # # # [char] . hold # # # # #> TYPE
  r> base !
;
[then]

: .ASCII        \ b --
\ *G Display a byte as a character if the code is printable,
\ ** otherwise display a dot.
  $07F AND  DUP $020 $07E WITHIN? 0=
  IF  DROP $02E  ENDIF  EMIT
;

16bit? [if]
: dump		\ addr len --
\ *G Dump a block of memory byte by byte. 16 bit version.
  BASE @ -rot  HEX  bounds		\ get limits
  ?DO
    CR  i .WORD  i 010 bounds		\ print address
    do  space  i c@ .BYTE  loop		\ print byte
    2 SPACES  I 010 bounds		\ gap
    DO  I c@ .ASCII  LOOP		\ ascii o/p
    key? ?LEAVE				\ quit or halt
  010 +LOOP				\ until done
  BASE !  CR				\ restore base
;

harvard? [if]
: Cdump		\ addr len --
\ *G Dump a block of code memory byte by byte. 16 bit Harvard
\ ** version.
  BASE @ -rot  HEX  bounds		\ get limits
  ?DO
    CR  i .WORD  i $010 bounds		\ print address
    do  space  i c@c .BYTE  loop	\ print byte
    2 SPACES  I $010 bounds		\ gap
    DO  I c@c .ASCII  LOOP		\ ascii o/p
    key? ?LEAVE				\ quit or halt
  $010 +LOOP				\ until done
  BASE !  CR				\ restore base
;
[then]
[then]

32bit? [if]
: dump		\ addr len --
\ *G Dump a block of memory byte by byte. 32 bit version.
  BASE @ -rot  HEX  bounds		\ get limits
  ?DO
    CR  i .DWORD  i $010 bounds		\ print address
    do  space  i c@ .BYTE  loop		\ print byte
    2 SPACES  I $010 bounds		\ gap
    DO  I c@ .ASCII  LOOP		\ ascii o/p
    key? ?LEAVE				\ quit or halt
  $010 +LOOP				\ until done
  BASE !  CR				\ restore base
;

harvard? [if]
: Cdump		\ addr len --
\ *G Dump a block of code memory byte by byte. 32 bit Harvard
\ ** version.
  BASE @ -rot  HEX  bounds		\ get limits
  ?DO
    CR  i .DWORD  i $010 bounds		\ print address
    do  space  i c@c .BYTE  loop	\ print byte
    2 SPACES  I $010 bounds		\ gap
    DO  I c@c .ASCII  LOOP		\ ascii o/p
    key? ?LEAVE				\ quit or halt
  $010 +LOOP				\ until done
  BASE !  CR				\ restore base
;
[then]
[then]

: .S		\ i*x -- i*x
\ *G Non-destructive stack display.
  CR
  DEPTH ?DUP IF
    ."  top"  0
    DO  I PICK CR 8 U.R  LOOP
  ELSE
    ." Empty stack"
  ENDIF
  CR
;

: ?		\ addr --
\ *G Display the contents of *\i{addr} in the current base.
  @ .
;


\ ******************
\ *S FORGET and friends
\ ******************
\ *P In an embedded system *\fo{FORGET} and friends are only
\ ** useful when compiling code into RAM. *\fo{FORGET} is used
\ ** to remove words recently compiled.

decimal

internal

: Trim		\ nfa-to-forget wid --
\ *G Trim the threads in a wordlist.
  dup >threads swap >#threads @ cells
  bounds do				\ for each thread
    ORIGIN- i @				\ fetch top link
    begin	\ -- nfa-t-f nfa2
      2dup origin- u> 0=		\ while nfa <= link
     while
       n>link @c			\  try next one
    repeat
    i !  ORIGIN+			\ update thread entry
  cell +Loop				\ step to next thread
  drop					\ release nfa
;

: (forget)      \ nfa -- ; forgets back to given nfa
\ *G Forget all words back to the given nfa.
  >r
  r@ ORIGIN-  FENCE @ ORIGIN-  U<  -121 ?throw
  VOC-LINK @
  BEGIN
    ORIGIN- R@ ORIGIN- OVER U<
   WHILE
    ORIGIN+ >voc-link @
    only forth definitions
  REPEAT
  ORIGIN+ dup voc-link !		\ voc-link -- ; pointer to top voc
  BEGIN
    r@ over				\ -- v-l nfa v-l
    trim				\ -- v-l ; cut links
    >voc-link @ ?dup 0=			\ check end of chain
  UNTIL
  r> n>link dp !			\ reset DP
;

external

: $FORGET       \ $addr --
\ *G Forget a word whose name is given as a counted string.
  count  current @ search-wordlist ?undef
  >name (forget)
;

: FORGET	\ -- ; FORGET <name>
\ *G The name to forget follows in input stream
  bl word  dup count upper  $forget
;


\ **********
\ vocs order
\ **********

also root definitions
: Vocs          \ --
\ *G Display a list of all the vocabularies in the system.
  Voc-Link @
  Begin
    ?dup
   While               		\ wid --
    dup >vocname @ .name
    >voc-link @
  Repeat
;

: .Voc		\ addr --
\ *G Try to display the vocabulary whose wid is at *\i{addr}.
  @ ?dup If
    >vocname @ .name
  Endif
;

: Order		\ --
\ *G Display the *\fo{CONTEXT} search order and the *\fo{CURRENT}
\ ** vocabulary.
  cr ." Context: "  Context Vsize bounds
  Do  i .voc  cell +Loop
  cr ." Current: " Current .voc cr
;

previous definitions


((
\ **********
\ .hex 2.hex
\ **********
\ only necessary if the target is in a very bad way indeed
\ and even . will not work!!!


hex

: .hex          \ n -- ; print n in hex
  4 0
  do  dup 0C 0 do 2/ loop               \ top 4 bits to bottom
      0F and  dup 9 >                   \ mask, check 0..9,A..F
      if  7 +  endif                    \  was A..F
      030 + (emit)                      \ convert to ASCII
      2* 2* 2* 2*                       \ get next 4 bits
  loop                                  \
  drop                                  \ drop n
;

: 2.hex
  cr 2dup  .hex space .hex
;
))


