\ MPE PowerForth v6/7 software floating point package for 32 bit targets

((
Copyright (c) 2000, 2002, 2005, 2006, 2007, 2009, 2010, 2011, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change History
==============
20151110 ALP018 Corrected 0 check in FP>IEEE.
20150330 SFP017 Reordered the code. Recognise both 'e' and 'E'.
		Updated for systems whose shift instructions do not return
		0 for a shift count of 32 or more.
		Reordered for hosted systems.
20150122 BJC016 Previous change BJC014 did not work properly. Now using 5/4
                rounding for values printed in non exponent format using F.
                Added word NORM-FROUND which applies 5/4 rounding assuming that
                the value is printed 'normally'.
20111201 MPE015 Updated documentation.
20110302 BJC014 Corrected op-prepare which assumes 8 sig figs regardless
                of the actual precision required.  It now uses PLACES to
                ensure that output from F. etc is now rounded to the places
                displayed.
20100630 MPE013 Improved compatibility with VFX Forth for Win/Lin/DOS.
                Minor refactoring.
20100628 BJC012 Changes to FNUMBER? allow use of HEX and other
                integers after REALS executed:
                  If BASE is not decimal FNUMBER? always does
                integer conversion.
                  FCHECK now recognises the integer prefixes
                ('#','$','0x' etc.) so these are converted as
                integers.
                The character used as a decimal separator is
                set in VALUE DPchar, defined here (if not previously).
                This allows use of ',' or indeed other characters.
                The US/UK convention '.' is the default.
20100526 BJC011 CDATA is selected prior to defining all FARRAYs
                for constant data (previously CDATA was assumed,
                and things broke if it wasn't).
20090729 SBD010 Fixed ROUND at limits.
		Improved accuracy in ?10PWR.
20090319 SFP009 Added INTERPRETER versions of F@ F! and F,
		Added FBUFF.
20090220 MPE008 Added Alberto Pasquale's IEEE conversion words.
20071105 SFP007 Made >FLOAT convert only in DECIMAL.
20071012 SFP006 Improved documentation.
20060328 SFP005 Reordered to remove forward references.
20050705 SFP004 Improved FVARIABLE for optimised systems.
20021112 SFP003 DOCGENned it.
20000713 SFP002 Renamed DINT to F>D for consistency. F>D is the ANS word.
		The original F>D was just a synonym. Similarly SINT was
		renamed to F>S
20000706 SFP001 Applied improvements supplied by Hiden Analytical.
		Added minor performance optimisations.
		Made F/ and DINT return clipped values on overflow.
19981029 MSD000 Brought the common stuff out of all the various versions of
         MSD000 SOFTFP.FTH into a single (that is, *this*) file.

32-bit mantissa and 32-bit exponent, combined FP and data stack

Polynomials from
  Cody and Waite
    http://ebook30.com/science/mathematics/190013/william-j.-cody-software-manual-for-the-elementary-functions.html

  Abramowitz & Stegun
    http://www.math.ucla.edu/~cbm/aands/
    http://www.convertit.com/Go/Bioresearchonline/Reference/AMS55.ASP
))


\ =========
\ *! softfp
\ =========
\ *T Software Floating Point

\ ***************
\ *S Introduction
\ ***************
\ *P Software floating-point is supplied with the cross-compiler and the
\ ** target Forth. The target floating point wordset is not fully ANS
\ ** and Forth-2012 compliant, but satisfies the needs of embedded systems
\ ** without undue complexity. The Forth data stack and the floating
\ ** point stack are the same (combined) in *\i{Common/SoftFPCX.fth}
\ ** and separate in *\i{Common/SoftFPSX.fth}. The floating point
\ ** data storage format is not IEEE, but is optimised for performance
\ ** on small controllers. If you need a separate floating point
\ ** stack or IEEE double format storage, please contact MPE. Any
\ ** variations in the implementation will be documented in the
\ ** target specific section of the manual.

\ *P The cross-compiler has a more limited floating-point support
\ ** than the target. Some words are avaliable during compilation
\ ** of colon definitions, but not while interpreting.

\ **************
\ *S Source code
\ **************
\ *P The source code is in two sets of files, one for 32 bit
\ ** Forth targets, the other for 16 bit targets. The files are:
\ *E   Common\SoftFP32CX current 32 bit code for a combined stack
\ **   Common\SoftFP32CX current 32 bit code for separate stacks
\ **   Common\sfp32hi    old 32 bit primitives
\ **   Common\sfp32com   old 32 bit high level code
\ **   Common\sfp16hi    16 bit primitives
\ **   Common\sfp16com   16 bit high level code
\ *P These files use no assembler definitions. Some targets have
\ ** code versions of the primitives, and these will be found in
\ ** the CPU specific code directory. A significant increase in
\ ** performance can be obtained by using the code files.

\ **********************************
\ *S Entering floating-point numbers
\ **********************************
\ *P Floating point number entry is enabled by *\fo{REALS} and
\ ** disabled by *\fo{INTEGERS}.

\ *P Floating-point numbers of the form 0.1234e1 are required
\ ** (see *\fo{FNUMBER?}) during interpretation and compilation
\ ** of source code. Floating-point numbers are compiled as
\ ** literal numbers when in a colon definition (compiling) and
\ ** placed on the stack when outside a definition (interpreting).
\ ** Inside a colon definition, a floating point literal number
\ ** must be preceded by *\fo{F#}.
\ *C : foo  ... f# 1.234e0 ...  ;

\ *P The more flexible word *\fo{>FLOAT} accepts numbers in two
\ ** forms, 1.234 and 0.1234e1. Both words are documented later
\ ** in this chapter. See also the section on *\i{Gotchas} later
\ ** in this chapter.

\ *P Note also that MPE Forths use ',' as the double number
\ ** indicator - it makes life much easier for Europeans.

\ *************************************
\ *S The form of floating-point numbers
\ *************************************
\ *P A floating-point number is placed on the Forth data stack. In
\ ** the Forth literature, this is referred to as a combined floating
\ ** point and data stack.
\ ** For 32 bit targets, a floating point number consists of two
\ ** 32-bit numbers, one for the mantissa and one for the exponent.
\ ** For 16 bit targets, it consists of a 32-bit double mantissa
\ ** and a single 16-bit exponent. The mantissa is normalised.
\ ** The exponent is on the top of the stack.
\ ** Note that for 16 bit targets, number conversion is affected
\ ** by the cross-compiler directives *\fo{HOST-MATH} and *\fo{TARGET-MATH}.
\ ** *\fo{HOST-MATH} leaves double numbers and floats in 32-bit form,
\ ** whereas *\fo{TARGET-MATH} leaves them in 16-bit form.

\ *******************************
\ *S Creating and using variables
\ *******************************
\ *P To create a variable, use *\fo{FVARIABLE}. *\fo{FVARIABLE} works in the
\ ** same way as *\fo{VARIABLE}. For example, to create a floating-point
\ ** variable called *\fo{VAR1} you code:
\ *C   FVARIABLE VAR1
\ *P When *\fo{VAR1} is used, it returns the address of the floating-point
\ ** number.

\ *P Two words are used to access floating-point variables,
\ ** *\fo{F@} and *\fo{F!}. These are analogous to *\fo{@} and
\ ** *\fo{!}.

\ *********************
\ *S Creating constants
\ *********************
\ *P To create a floating-point constant, use *\fo{FCONSTANT}, which
\ ** is analogous to *\fo{CONSTANT}. For example, to generate a
\ ** floating-point constant called *\fo{CON1} with a value of 1.234,
\ ** you enter:
\ *C   1.234e0 FCONSTANT CON1
\ *P When *\fo{CON1} is executed, it returns 1.234 on the Forth
\ ** stack.

\ ***************************
\ *S Using the supplied words
\ ***************************
\ *P The supplied words split into several groups:
\ *(
\ *B sines, cosines and tangents
\ *B arc sines, cosines and tangents
\ *B arithmetic functions
\ *B logarithms
\ *B powers
\ *B displaying floating-point numbers
\ *B inputting floating-point numbers
\ *)

\ *P The following functions only exist as target words so you
\ ** cannot use them in calculations in your source code when
\ ** outside a colon definition.

\ *N Calculating sines, cosines and tangents
\ *P To calculate sine, cosine and tangent, use *\fo{FSIN}, *\fo{FCOS} and
\ ** *\fo{FTAN} respectively. Angles are expressed in radians.

\ *N Calculating arc sines, cosines and tangents
\ *P To calculate arc sine, cosine and tangent, use *\fo{FASIN}, *\fo{FACOS}
\ *P and *\fo{FATAN} respectively. They return an angle in radians.

\ *N Calculating logarithms
\ *P Two words are supplied to calculate logarithms, *\fo{FLOG} and *\fo{FLN}.
\ ** *\fo{FLOG} calculates a logarithm to base 10 (decimal).
\ ** *\fo{FLN} calculates a logarithm to base e. Both take a
\ ** floating-point number in the range from 0 to Einf.

\ *N Calculating powers
\ *P Three power functions are supplied:
\ *C   FE^X F10^X X^Y

\ *********************
\ *S Degrees or radians
\ *********************
\ *P The angular measurement used in the trigonometric functions
\ ** are in radians. To convert between degrees and radians use
\ ** *\fo{RAD>DEG} or *\fo{DEG>RAD}. *\fo{RAD>DEG} converts an angle from radians
\ ** to degrees. *\fo{DEG>RAD} converts an angle from degrees to radians.

\ ************************************
\ *S Displaying floating-point numbers
\ ************************************
\ *P Two words are available for displaying floating-point numbers,
\ ** *\fo{F.} and *\fo{E.}. The word *\fo{F.} takes a floating-point
\ ** number from the stack and displays it in the form xxxx.xxxxx
\ ** or x.xxxxxEyy depending on the size of the number. The word
\ ** *\fo{E.} displays the number in the latter form.

\ ************************************
\ *S Number formats, ANS and Forth200x
\ ************************************
( *P The ANS Forth standard specifies that floating point numbers must )
( ** be entered in the form 1.234e5 and must contain a point '.' and )
( ** 'e' or 'E', and that double integers are terminated by a point '.'. )

\ *P This situation prevents the use of the standard conversion
\ ** words in international applications because of the
\ ** interchangable use of the '.' and ',' characters in numbers.
\ ** Because of this, the cross-compiler's host VFX Forth uses
\ ** two four-byte arrays, *\fo{FP-CHAR} and *\fo{DP-CHAR}, to
\ ** hold the characters used as the floating point and double
\ ** integer indicator characters. By default, *\fo{FP-CHAR} is
\ ** initialised to '.' and *\fo{DP-CHAR} is initialised to to
\ ** ',' and '.'. For strict ANS compliance, you should set them
\ ** as follows *\b{before} *\fo{CROSS-COMPILE} is run.

\ *E \ ANS standard setting
\ **   char . dp-char !
\ **   char . fp-char !
\ ** : ans-floats    \ -- ; for strict ANS compliance
\ **   [char] . dp-char !
\ **   [char] . fp-char !
\ ** ;
\ ** \ MPE defaults
\ **   char , dp-char !
\ **   char . dp-char 1+ c!
\ **   char . fp-char !
\ ** : mpe-floats    \ -- ; for existing and most legacy code
\ **   [char] , dp-char !
\ **   [char] . dp-char 1+ c!
\ **   [char] . fp-char !
\ ** ;
\ ** \ Legacy defaults, including ProForth
\ **   char , dp-char !
\ **   char . fp-char !
\ ** : legacy-floats \ -- ; for legacy code
\ **   [char] , dp-char !
\ **   [char] . fp-char !
\ ** ;

\ *P You can of course set these arrays to hold any values which
\ ** suit your application's language and locale. Note that integer
\ ** conversion is always attempted before floating point
\ ** conversion. This means that if the *\fo{FP-CHAR} and *\fo{DP-CHAR}
\ ** arrays contain the same character, floating point numbers must
\ ** contain 'e' or 'E'. If the arrays are all different, a number
\ ** containing the *\fo{FP-CHAR} will be successfully converted
\ ** as a floating point number, even if it does not contain 'e'
\ ** or 'E'.



\ ***********
\ *S Glossary
\ ***********

cdata decimal


\ =============
\ *N Separators
\ =============
\ *P Before July 2010, the floating point separator, '.', was
\ ** fixed. To ease internationalisation, it is now variable.

[undefined] fp-char [if]
variable fp-char	\ -- addr
\ *G Holds up to four character(s) to be treated as floating
\ ** point indicators. Set to '.' for ANS compatibility. Note
\ ** that this should be accessed as a one to four byte array.
\ ** The first character is used as the point character for
\ ** output.
  char . fp-char c!
[endif]

0 equ SepArray?	\ -- flag
\ *G If the equate is non-zero, *\fo{fp-char} is treated as
\ ** a four byte array, otherwise as a one byte array. This is
\ ** a flag for future expansion.

[undefined] isSep? [if]
SepArray? [if]
: isSep?	\ char addr -- flag
\ *G Return true if *\i{char} is one of the four bytes at *\i{addr}.
\ ** If less than than four bytes are needed, a zero byte acts as
\ ** a terminator. Used when *\fo{SepArray?} is true.
  cell bounds do
    i c@ 0= if
      drop false  unloop exit
    endif
    i c@ over = if
      drop true  unloop exit
    endif
  loop
  drop 0
;
[else]
compiler
: isSep?  c@ =  ;
\ *G A compiler macro used when *\fo{SepArray?} is false.
target
[then]
[then]


\ ========================
\ *N High Level primitives
\ ========================
\ *P The software floating point pack requires several support
\ ** primitives. High level versions are provided in *\i{SFP16HI.FTH}
\ ** and *\i{SFP32HI.FTH} for 16 and 32 bit targets. Some targets
\ ** have coded versions in the CPU directory and these will
\ ** provide much better performance. The support file should
\ ** be compiled before the common file.

\ *P In *\i{SoftFP32CX.fth} and *\i{SoftFP32SX.fth} a set of
\ ** high-level primitives are compiled if the primitives have
\ ** not yet been supplied.

[undefined] S-> [if]

: S->           \ n1 carry-in-flag --- n2 carry-out-flag
\ *G Perform a right shift, applying the carry in to the m.s. bit and
\ ** returning the carry out as 1 or 0.
  over 1 rshift swap		\ Save original N1 and shift it right one bit.
  1 and                         \ Only use bottom bit of carry flag.            \ MSD002
  if                            \ Should MSB (carry-in bit) be zero or one?
    $80000000 or                \ MSB must be a one.
  else
    $7FFFFFFF and               \ MSB must be a zero.
  endif
  swap $00000001 and            \ Retrieve old N1 and create carry-out flag.    \ MSD002
;

: <-S		\ n1 carry-in-flag --- n2 carry-out-flag
\ *G Perform a left shift, applying the carry in to the l.s. bit and
\ ** returning the carry out as 1 or 0.
  over 1 lshift swap		\ Save original N1 and shift it left one bit.
  if                            \ Should LSB (carry-in bit) be zero or one?
    $0000:0001 or               \ LSB must be a one.
  else
    $FFFF:FFFE and              \ LSB must be a zero.
  endif
  swap $8000:0000 and #31 rshift	\ Retrieve old N1 and create carry-out flag.
;

: d<<1		\ xd -- xd<<1
\ *G One bit double left shift.
  swap 0 <-s            	\ lsc msc -- msc lsc<<1 carry ;
  rot swap <-s          	\ -- lsc<<1 msc<<1 carry ;
  drop                  	\ -- lsc<<1 msc<<1 ;
;

: d>>1		\ xd -- xd>>1
\ *G One bit double right logical shift.
  0 s->                 	\ lsc msc -- lsc msc>>1 carry ;
  rot swap s->          	\ -- msc>>1 lsc>>1 carry ;
  drop swap             	\ -- lsc>>1 msc>>1 ;
;

: D>>N		\ d n -- d>>n
\ *G N bit double right logical shift.
  dup 0> if
    0 do d>>1 loop
  else
    drop
  endif
;

[then]

[defined] CPU=386 [if]		\ x86 targets, not ARM
: rshiftx	\ x u -- x' ; right shift
\ *G Used for right shifts that may exceed 31 bits. In the ANS and
\ ** Forth 2012 standards, this is an ambiguous condition. We need
\ ** shifts over 31 bits to return 0. On x86 targets, a check is made
\ ** for shifts over 31 bits.
  dup 31 <=
  if  rshift  else  2drop 0  then
;
[else]
compiler
: rshiftx	rshift  ;
target
[then]


\ -----------------------------------
\ *N Basic stack and memory operators
\ -----------------------------------

: F!		\ r addr --
\ *G Stores r at addr
  2!  ;

: F@		\ addr -- r
\ *G Fetches r from addr.
  2@  ;

: F,		\ r --
\ *G Lays a real number into the dictionary, reserving 8 bytes.
  here  8 allot  f!  ;
interpreter
: F!	2!  ;
: F@	2@  ;
: F,	here  8 allot  f!  ;
target

: FDUP  	\ r -- r r
\ *G Floating point equivalent of *\fo{DUP}.
  2dup  ;

: FOVER		\ r1 r2 -- r1 r2 r1
\ *G Floating point equivalent of *\fo{OVER}.
  2over  ;

: FROT		\ r1 r2 r3 -- r2 r3 r1
\ *G Floating point equivalent of *\fo{ROT}.
  5 roll 5 roll  ;

: FPICK		\ fu..f0 u -- fu..f0 fu
\ *G Floating point equivalent of *\fo{PICK}.
  2* 1+ dup >r pick  r> pick  ;

: FROLL		\ f1 f2 f3 --  f2 f3 f1
\ *G Floating point equivalent of *\fo{ROLL}.
  2* 1+ dup >r roll  r> roll  ;

: FSWAP		\ r1 r2 -- r2 r1
\ *G Floating point equivalent of *\fo{SWAP}.
  2swap  ;

: FDROP		\ r --
\ *G Floating point equivalent of *\fo{DROP}.
  2drop  ;

: FNIP		\ r1 r2 -- r2
\ *G Floating point equivalent of *\fo{NIP}.
  2swap 2drop  ;

compiler	\ internal use only!
: f>r		2>r  ;
: fr>		2r>  ;
target


\ define some primitives as compiler macros
optimised? [if]
compiler
: F!		\ r f-addr --
  2!  ;
: F@		\ f-addr -- r
  2@  ;
: FDUP  	\ r -- r r
  2dup  ;
: FDROP		\ r --
  2drop  ;
: FNIP		\ r1 r2 -- r2
  2swap 2drop  ;
target
[then]


\ --------------------------------
\ *N Floating point defining words
\ --------------------------------

\ SFP004...
: FVARIABLE	\ "<spaces>name" -- ; Run: -- f-addr
\ *G Use in the form: *\fo{FVARIABLE <name>} to create a variable
\ ** that will hold a floating point number.
  create  0 , 0 ,  ;
interpreter
: FVARIABLE	\ "<spaces>name" -- ; Run: -- f-addr
  idata create  0 , 0 ,  cdata  ;
target
\ ...SFP004

: FCONSTANT	\ r "<spaces>name" -- ; Run: -- r
\ *G Use in the form: *\fo{<float> FCONSTANT <name>} to create a
\ ** constant that returns a floating point number.
  create  , ,  does> f@  ;
interpreter
: fconstant	2constant  ;
target

: FARRAY	\ "<spaces>name" fn-1..f0 n -- ; Run: i -- ri
\ *G Create an initialised array of floating point numbers. Use
\ ** in the form:
\ *C   fn-1 .. f1 f0 n FARRAY <name>
\ *P to create an array of n floating point numbers. When the
\ ** array *\fo{name} is executed, the index i is used to return
\ ** the address of the i'th 0 zero-based element in the array.
\ ** For example:
\ *C   4e0 3e0 2e0 1e0 0e0 5 FARRAY TEST
\ *P will set up an array of five elements.
\ ** Note that the rightmost float (0e0) is element 0.
\ ** Then *\fo{i TEST} will return the *\{i}th element.
\ ** If you create this array in *\fo{IDATA}, restore *\fo{CDATA}
\ ** afterwards.
  create
    0 do  , ,  loop
  does>
    swap 2 cells * +  f@
;

: FBUFF		\ u "name" -- ; i -- addr
\ *G Creates a buffer of *\i{u} floats in the current memory
\ ** section. The child action is to return the address of the
\ ** *\i{i}th element (zero-based).
\ *C   10 fbuff foo
\ *P Creates an buffer for ten float elements in.
\ *C   3 foo
\ *P Returns the address of element 3 in the buffer.
  create
    2 cells * allot
  does>
    swap 2 cells * +
;
\ *P The default section is *\fo{CDATA}, and we recommend that
\ ** you leave it that way! To create a ten element array in
\ ** *\fo{UDATA} space, you can use:
\ *E udata
\ ** 10 fbuff MyFloats
\ ** cdata


\ -------------------
\ *N Type conversions
\ -------------------

internal

: NORM		\ n exp -- f
\ *G Normalise a single integer and a single exponent to produce a
\ ** floating point number. INTERNAL.
  over if			        \ -- n exp ; check zero
    swap				\ -- exp n
    dup >r  abs		                \ -- exp n' ; R: -- n ; save sign, make positive
    dup 0< if				\ -- exp n' ; $80000000 is a special case
      r> drop
    else				\ -- n' ; R: -- n
      begin
        swap 1- swap			\ decrement exponent
        1 lshift  dup 0<		\ shift until -ve
      until
      swap 1+ swap  1 rshift		\ step back
      r> ?negate			\ restore sign
    endif
    swap				\ -- n' exp
  else
    drop 0				\ -- 0 0
  then
;

: DNORM		\ d exp -- fn ; normalise a 64 bit double
\ *G Normalise a double integer and a single exponent to produce a
\ ** floating point number. INTERNAL.
  >R  2DUP or 0= IF
    R> DROP				\ return 0 mantissa \ 0 exp
  ELSE
    TUCK DABS				\ save sign, make mantissa poitive
    BEGIN
      DUP $40000000 AND 0=
     WHILE
      2DUP D+				\ Double it, ie shift right
      R> 1- >R				\ adjust exponent
    REPEAT
    NIP  SWAP 0< ?NEGATE  R>		\ discard ls, apply sign, merge exp
  THEN
;

external

: FSIGN		\ fn -- |fn| flag ; true if negative
\ *G Return the absolute value of fn and a flag which is true
\ ** if fn is negative.
  swap dup abs -rot 0<
;

: S>F		\ n -- fn
\ *G Converts a single integer to a float.
  #31 norm
;

: D>F		\ d -- fn
\ *G Converts a double integer to a float.
  #63 dnorm
;

: F>S		\ fn -- n
\ *G Converts a float to a single integer.
\ ** Note that *\fo{F>S} truncates the number towards zero
\ ** according to the ANS specification. If |fn| is greater
\ ** than maxint, +/-maxint is returned.
  fsign >r  dup 0< if
    r> drop  fdrop 0
  else
    31 swap -  dup 0> if
      rshiftx
    else
\     1 abort" number too large in F>S"
      2drop  $7FFFFFFF			\ return max int
    then
    r>
    if  negate  then
  then
;

: F>D		\ fn -- d
\ *G Converts a float to a double integer.
\ ** Note that *\fo{F>D} truncates the number towards zero
\ ** according to the ANS specification. If |fn| is greater
\ ** than dmaxint, +/-dmaxint is returned.
  fsign >r  dup 0< if
    r> drop  fdrop 0 0
  else
    63 swap -  dup 0> if
      0 -rot d>>n
    else
\     1 abort" number too large in F>D"
      2drop  $FFFFFFFF $7FFFFFFF	\ max dint
    then
    r>
    if  dnegate  then
  then
;

: FINT		\ f1 -- f2
\ *G Chop the number towards zero to produce a floating point
\ ** representation of an integer.
  dup #32 <
  if  f>s s>f  then
;


\ -------------
\ *N Arithmetic
\ -------------

: FNEGATE	\ r1 -- r2
\ *G Floating point negate.
  swap negate swap
;

: ?FNEGATE	\ fn n -- fn|-fn
\ *G If n is negative, negate fn.
  0< if  fnegate  then
;

: FABS		\ fn -- |fn|
\ *G Floating point absolute.
  swap abs swap
;

: F*		\ r1 r2 -- r3
\ *G Floating point multiply.
  fsign >r  fswap fsign r> + >r		\ -- |r2| |r1| ; R: -- sign
  rot + 1+ -rot				\ -- exp mant2 mant1 ; R: -- sign
  um* 2dup or if			\ -- exp ud ; R: -- sign ; 64 bit multply <> 0 ?
    begin
      dup  $40000000 and 0=		\ normalised?
     while
      rot 1- -rot  d<<1			\ 1 step
    repeat
    swap 0< if  1+  then		\ round up on discarded low half
    dup 0<                              \ check for overflow
    if  swap 1+ swap  1 rshift  then	\ undo last step
    r> -1 =				\ see if product -ve
    if  negate  then                    \ make it -ve
    swap				\ recover exponent
  else
    2drop drop  r> drop  0 0		\ return 0
  then
;

: F/		\ r1 r2 -- r3
\ *G Floating point divide.
  fsign >r  fswap fsign r> + >r fswap	\ -- |r1| |r2| ; R: -- sign
  over 0= if				\ Check r2 mantissa
    fdrop fdrop				\ discard r1, r2
    $40000000 $7FFF			\ very large positive float
  else					\ -- mant1 exp1 mant2 exp2 ; R: -- sign
    rot swap - -rot			\ -- exp mant1 mant2 ; R: -- sign ; subtract exponents
    >r  0 swap				\ -- exp dividend ; R: -- mant2 sign ; dividend double prec.
    dup r@ u< 0= if			\ avoid divide overflow
      rot 1+ -rot			\   update exp
      d>>1				\   update dividend
    then
    r@ um/mod                           \ -- exp urem uquot ; R: -- mant2 sign ; do division
    swap r> 2/ u> if			\ -- exp uquot ; R: -- sign
      1+				\ if rem>div/2 round up
    then
    dup 0< if                           \ check for overflow
      1 rshift  swap 1+			\  and correct
    else
      swap
    endif
    1- norm				\ normalise
  endif
  r> -1 =			        \ check sign
  if  fnegate  then
;

: F+		\ r1 r2 -- r3
\ *G Floating point addition.
  over 0= if				\ check for r2=0
    2drop  exit
  endif

  3 pick 0= if				\ check for r1=0
    2swap 2drop  exit
  endif

  fsign >r
  fswap fsign >r fswap			\ -- m1 e1 m2 e2 ; R: -- s1 s2
  rot 2dup - >r				\ -- m1 m2 e2 e1 ; R: -- s1 s2 e2-e1
  max -rot  r@ if			\ -- esum m1 m2
    r@ 0> if				\ reduce smaller mantissa
      swap r> rshiftx swap		\ -- esum m1' m2 ; R: -- s1 s2
    else
      r> abs rshiftx			\ -- esum m1 m2' ; R: -- s1 s2
    then
  else
    r> drop				\ -- esum m1 m2  ; R: -- s1 s2
  then
  r> r>					\ -- esum m1 m2 s2 s1 ; R: --
  over <> if				\ -- esum m1 m2 s2 ; if signs differ
    >r  negate +			\ -- esum m1-m2 ; R: -- s2
  else
    >r  +				\ -- esum m1+m2 ; R: -- s2
    swap 1+ swap			\ scale exponent
    0 s-> +  dup 0< if			\ and mantissa, check nomalisation
      swap 1+ swap  1 rshift
    endif
  endif					\ esum msum ; R: -- s2
  r>					\ apply sign
  if  negate  endif
  swap norm				\ and normalise
;

: F-		\ r1 r2 -- r3
\ *G Floating point subtraction, r3=r1-r2
  fnegate f+
;

: FSEPARATE	\ f1 f2 -- f3 f4
\ *G Leave the signed integer quotient f4 and remainder f3 when
\ ** f1 is divided by f2. The remainder has the same sign as the
\ ** dividend.
  fover fover f/
  frot frot fabs
  fswap fsign >r fswap
  2 fpick fabs fint f* f-
  r>
  if  fnegate  then
  fswap
;

: FFRAC		\ f1 f2 -- f3
\ *G Leave the fractional remainder from the division f1/f2. The
\ ** remainder takes the sign of the dividend.
  fseparate fdrop
;


\ -----------------------
\ *N Relational operators
\ -----------------------

: F0<		\ f1 -- flag
\ *G Floating point *\fo{0<}.
  drop 0<
;

: F0>		\ f1 -- flag
\ *G Floating point *\fo{0>}.
  drop 0>
;

: F0=  		\ f1 -- flag
\ *G Floating point *\fo{0=}.
  drop 0=
;

: F0<>  	\ f1 -- flag
\ *G Floating point *\fo{0<>}.
  drop 0<>
;

optimised? [if]
compiler
: F0<		\ f1 -- flag
  drop 0<
;

: F0>		\ f1 -- flag
  drop 0>
;

: F0=  		\ f1 -- flag
  drop 0=
;

: F0<>  	\ f1 -- flag
  drop 0<>
;
target
[then]

: F=  		\ f1 f2 -- flag
\ *G Floating point *\fo{=}.
  f- f0=
;

: F<  		\ r1 r2 -- flag
\ *G Floating point *\fo{<}.
  f- f0<
;

: F>  		\ f1 f2 -- flag
\ *G Floating point *\fo{>}.
  f- f0>
;

: FMAX  	\ r1 r2 -- r1|r2
\ *G Floating point *\fo{MAX}.
  fover fover f<
  if  fswap  then
  fdrop
;

: FMIN  	\ r1 r2 -- r1|r2
\ *G Floating point *\fo{MIN}.
  fover fover f>
  if  fswap  endif
  fdrop
;

\ ----------------
\ *N Miscellaneous
\ ----------------

: FDEPTH  	\ -- +n
\ *G Returns the number of floats on the stack.
  depth 2/
;

: FALIGNED	\ addr -- f-addr
\ *G Aligns the address to accept an 8-byte float.
  7 + -8 and
;
optimised? [if]
compiler
: FALIGNED	\ addr -- f-addr
  7 + -8 and
;
target
[then]

: FALIGN  	\ --
\ *G Aligns the dictionary to accept an 8-byte float.
  here dup faligned swap - allot
;

: FLOAT+  	\ f-addr1 -- f-addr2
\ *G Increments addr by 8, the size of a float.
  8 +
;

: FLOATS  	\ n1 -- n2
\ *G Returns n2, the size of n1 floats.
  8 *
;
optimised? [if]
compiler
: FLOAT+  	\ f-addr1 -- f-addr2
  8 +
;

: FLOATS  	\ n1 -- n2
  8 *
;
target
[then]


\ ===========================
\ *N Powers of ten operations
\ ===========================

1 s>f 10 s>f f/ fconstant %.1
\ *G Floating point 0.1.
          1 s>f fconstant %1
\ *G Floating point 1.0.
         10 s>f fconstant %10
\ *G Floating point 10.0.
  1250000000 34 fconstant %10^10
\ *G Floating point 10^10.
 1844674407 -33 fconstant %10^-10
\ *G Floating point 10^-10.
F# 1.0E256 FCONSTANT %10^256
\ *G Floating point 10^256.
F# 1.0E-1 FCONSTANT %10E-1
\ *G Floating point 10^-1.
F# 1.0E-10 FCONSTANT %10E-10
\ *G Floating point 10^-10.
F# 1.0E-256 FCONSTANT %10^-256
\ *G Floating point 10^-256.

CDATA
F# 1.0E15 F# 1.0E14 F# 1.0E13 F# 1.0E12 F# 1.0E11
F# 1.0E10 F# 1.0E09 F# 1.0E08 F# 1.0E07 F# 1.0E06
F# 1.0E05 F# 1.0E04 F# 1.0E03 F# 1.0E02 F# 1.0E01
F# 1.0e00
16 FARRAY POWERS-OF-10E1
\ *G An array of 16 powers of ten starting at 10^0
\ ** in steps of 1.

F# 1.0E256 F# 1.0E240 F# 1.0E224 F# 1.0E208
F# 1.0E192 F# 1.0E176 F# 1.0E160 F# 1.0E144
F# 1.0E128 F# 1.0E112 F# 1.0E96  F# 1.0E80
F# 1.0E64  F# 1.0E48  F# 1.0E32  F# 1.0E16
F# 1.0E00
17 FARRAY POWERS-OF-10E16
\ *G An array of 17 powers of ten starting at 10^0
\ ** in steps of 16.

F# 1.0E-15 F# 1.0E-14 F# 1.0E-13 F# 1.0E-12 F# 1.0E-11
F# 1.0E-10 F# 1.0E-09 F# 1.0E-08 F# 1.0E-07 F# 1.0E-06
F# 1.0E-05 F# 1.0E-04 F# 1.0E-03 F# 1.0E-02 F# 1.0E-01
F# 1.0e-00
16 FARRAY POWERS-OF-10E-1
\ *G An array of 16 powers of ten starting at 10^0
\ ** in steps of -1.

F# 1.0E-256 F# 1.0E-240 F# 1.0E-224 F# 1.0E-208
F# 1.0E-192 F# 1.0E-176 F# 1.0E-160 F# 1.0E-144
F# 1.0E-128 F# 1.0E-112 F# 1.0E-96  F# 1.0E-80
F# 1.0E-64	F# 1.0E-48 	F# 1.0E-32 	F# 1.0E-16
F# 1.0E00
17 FARRAY POWERS-OF-10E-16
\ *G An array of 17 powers of ten starting at 10^0
\ ** in steps of -16.

: RAISE_POWER	\ mant exp -- mant' exp'
\ *G Raise the power in preparation for number formatting.
  DUP >R  #15 AND ?DUP
  IF  POWERS-OF-10E1 F*  THEN
  R@ 4 rshift  #15 AND ?DUP
  IF  #16 MIN POWERS-OF-10E16 F*  THEN
  R> 8 rshift  0
  ?DO  %10^256 F*  LOOP
;

: SINK_FRACTION	\ mant exp -- mant' exp'
\ *G Reduce the power in preparation for number formatting.
  ABS DUP >R  #15 AND ?DUP
  IF  POWERS-OF-10E-1 F*  THEN
  R@ 4 rshift #15 AND ?DUP
  IF  POWERS-OF-10E-16 F*  THEN
  R> 8 rshift  0
  ?DO  %10^-256 F*  LOOP
;

: *10^X		\ float dec_exponent -- float'
\ *G Generate float' = float *10^dec_exp. INTERNAL.
  DUP 0< IF
    SINK_FRACTION
  ELSE
    RAISE_POWER
  THEN
;


\ -----------------------
\ *N Floating point input
\ -----------------------
\ *P Note that number conversion takes place in *\fo{PAD}.

: FLITERAL	\ Comp: r -- ; Run: -- r
\ *G Compiles a float as a literal into the current definition.
\ ** At execution time, a float is returned. For example,
\ ** *\fo{[ %PI F2* ] FLITERAL} will compile 2PI as a floating point
\ ** literal. Note that *\fo{FLITERAL} is immediate.
  swap  postpone literal  postpone literal
;
IMMEDIATE

internal

: CONVERT-EXP	\ c-addr --
\ *G If the character at c-addr is 'D' convert it to 'E'. INTERNAL.
  count [char] D scan if
    [char] E swap c!
  else
    drop
  then
;

: CONVERT-FPCHAR	\ c-addr --
\ *G Convert the f.p. char '.' to the double char ',' for
\ ** conversion. INTERNAL.
  count fp-char c@ scan if
    [char] , swap c!
  else
    drop
  then
;

: ALL-BLANKS?	\ c-addr len -- flag
\ *G Return true if string is all blanks (spaces). INTERNAL.
  true -rot
  bounds ?Do
    i c@ bl <>
    if  drop false leave  then
  Loop
;

[-short-branches
: FCHECK	\ -- am lm ae le e-flag .-flag
\ *G Check the input string at *\fo{PAD}, returning the separated
\ ** mantissa and exponent flags. The e-flag is returned true
\ ** if the string contained an exponent indicator 'E' and
\ ** the .-flag is returned true if a '.' was found. INTERNAL.
\ returns NOT FP, if number has base prefix
\ e.g. '$','#','0x' etc.  (Base suffix NOT supported)
  base @ >r                             \ preserve number base
  pad count 2dup check-prefix d= if     \ not integer - NOTE: WORKS ONLY FOR PREFIXES
    pad count 2dup [char] E scan dup >r   \ ap lp ae le --
    tuck >r >r - r> r>                    \ am lm ae le --
    dup if  1 /string  then               \ am lm ae' le' --
    pad count fp-char c@ scan nip
    r> swap
  else                                  \ integer prefix detected
    0 0 0 0                             \ dummies
    false false                         \ do DP or exponent
  endif
  r> base !                             \ restore number base
;
short-branches]

: doMNUM	\ c-addr u -- d 2 | 0
\ *G Convert the mantissa string to a double number and 2. If
\ ** conversion fails, just return 0. INTERNAL.
  (integer?) dup 1 =
  if  drop s>d 2  then
;

: doENUM	\ c-addr u -- n 1 | 0 ; str as above
\ *G Convert the exponent string to a single number and 1. If
\ ** conversion fails, just return 0. INTERNAL.
  dup if
    (integer?)  dup 2 =
    if  2drop drop 0  then
  else
    2drop  0 1
  then
;

: FIXEXP     \ dmant exp -- mant' exp'
\ *G Convert a double integer mantissa and a single integer
\ ** exponent into a floating point number. INTERNAL.
  >R drop s>f R> *10^X			\ d>f ?
;
external

-short-branches
: FNUMBER?	\ addr --  0/.../mant exp 2
\ *G Behaves like the integer version of *\fo{NUMBER?} except that if
\ ** the number is in F.P. format and *\fo{BASE} is decimal, a floating
\ ** point conversion is attempted. If conversion is successful,
\ ** the floating point number is left on the float stack and
\ ** the result code is 2. This word only accepts words with an
\ ** 'E' as a floating point indicator, e.g, 1.2345e0.
\ ** If BASE is not decimal all numbers are treated as integers.
\ ** The integer prefixes '#','$','0x' etc. are recognised and
\ ** cause integer conversion to be used.
  count pad place			\ copy string to pad
  pad count upper
  fcheck drop                           \ valid f.p. number?
  BASE @ #10 = and if                   \ and base is DECIMAL
    pad convert-fpchar                  \   convert f.p. char ready for conversion to a double
    doEnum if                           \   convert exponent ok?
      -rot  over c@ [char] , <> ?undef  \     must be at least one digit for d.p.
      doMnum if                         \     convert mantissa ok?
        rot dpl @  dup 0>
	if  -  else  drop  then
        fixexp  2
      else
        drop 0
      then
    else
      2drop 0
    then
    dup #l !  -1 dpl !
  else
    2drop 2drop  pad integer?           \ try to convert as integer
  then
;
+short-branches

: >FLOAT	\ c-addr u -- r true | false
\ *G Try to convert the string at c-addr/u to a floating point number.
\ ** If conversion is successful, flag is returned true, and a floating
\ ** number is returned on the float stack, otherwise just flag=0 is
\ ** returned. This word accepts several forms, e.g. 1.2345e0, 1.2345,
\ ** 12345 and converts them to a float. Note that double numbers
\ ** (containing a ',') cannot be converted.
\ ** Number conversion is decimal only, regardless of the current *\fo{BASE}.
  base @ >r decimal			\ SFP007
  pad place				\ copy string to pad
  pad count upper                       \ make string upper case
  pad convert-exp                       \ make sure exponent char is an 'E'
  fcheck or if                          \ valid f.p. number?
    pad convert-fpchar                  \   convert f.p. char ready for conversion to a double
    doEnum if                           \   convert exponent ok?
      -rot doMnum if                    \     convert mantissa ok?
        rot dpl @  dup 0>
	if  -  else  drop  then
        fixexp  true
      else
        drop 0
      then
    else
      2drop 0
    then
    dup #l !  -1 dpl !
  else
    2drop
    case  2dup 2>r (integer?)		\   try to convert integer to f.p.
      0 of  2r@ all-blanks? dup		\    all blanks is a special case returning f.p. 0
            if  0 0 rot  then
	endof
      1 of  s>f true  endof		\    convert single integer to float
      2 of  d>f true  endof		\    convert double
    endcase
    2r> 2drop
  then
  r> base !				\ SFP007
;

: (F#)  	\ addr -- fn 2 | 0
\ *G The primitive for *\fo{F#} and *\fo{F#IN} below.
  fnumber?
  case
    0  of  0      endof
    1  of  s>f 2  endof
    2  of  dpl @ -1 <>                  \ see if double
           if  d>f  then                \ convert to fp
           2
       endof
  endcase
;

: F#IN  	\ -- fn 2 | 0
\ *G Attempts to convert a token from the input stream to a
\ ** floating-point number. Numbers in integer format will be
\ ** converted to floating-point. An indicator (0 or 2/3) is
\ ** returned in the same way as an indicator is returned by
\ ** *\fo{FNUMBER?}.
  query bl word (f#)
;

: F#  		\ -- [f] ; or compiles it [ state smart ]
\ *G If interpreting, takes text from the input stream and,
\ ** if possible converts it to a f.p. number on the stack.
\ ** Numbers in integer format will be converted to floating-point.
\ ** If compiling, the converted number is compiled.
  bl word (f#) ?undef  state @
  if  [compile] fliteral  then
;
IMMEDIATE


also forth definitions

: REALS  	\ -- ; allow f.p input
\ *G Switch *\fo{NUMBER?} to permit floating point input using
\ ** *\fo{FNUMBER?}. This action can be reversed by *\fo{INTEGERS}.
\ ** Both *\fo{REALS} and *\fo{INTEGERS} are in the *\fo{FORTH}
\ ** vocabulary.
  assign fnumber? to-do number?         \ switch fnumber?
;

: INTEGERS  	\ -- ; no f.p input
\ *G Switch *\fo{NUMBER?} to restore integer only input.
  assign integer? to-do number?         \ switch in integer?
;

previous definitions


\ ------------------------
\ *N Floating point output
\ ------------------------

variable places  8 places !	\ -- addr
\ *G Number of digits output after the decimal point.

: ROUND  	\ f1 -- f2
\ *G Rounds least significant eight bits to 0 if higher 2 bits
\ ** are all 0s or all 1s.
  fsign >r >r
  dup 224 and 0=
  if  -256 and  then
  dup 224 and  224 =
  if  -256 and 256 +  then
  r> over 0<
\  if  swap 2/  swap 1+  then
  if  swap 1 rshift swap 1+  then		\ SBD010
  r>
  if  fnegate  then
;

internal
: ?10PWR  	\ exp[2] -- exp[2] exp[10]
\ *G Generate the power of ten corresponding to the power of two. INTERNAL.
  dup 0< >r
\  dup abs 301 1000 */mod nip		\ l10(2)~=.3010
  dup abs 8651 28738 */mod nip		\ l10(2)= 0.30102999512840141972301482357854 ; SBD010
  r> if  negate 1-  then
;

: SIGFIGS	\ fn n -- d dec_exponent
\ *G From fn, generate a double number corresponding to n significant digits
\ ** and a decimal exponent. INTERNAL.
  >R ?10PWR R>
  SWAP DUP >R  - ?DUP IF
    *10^X
  ENDIF
  R>
;

: op-prepare	\ fn -- d exp sign
\ *G From fn, generate a double number corresponding to n significant digits,
\ ** a decimal exponent and a sign indicator (nz=negative). INTERNAL.
  FSIGN >r
  8 SIGFIGS >r
  ROUND  F>D
  r> r>
;

: .EXP  	\ exp --
\ *G Display the exponent. INTERNAL.
  [char] E emit  dup 0<			\ if -10<e<10 then print a "0" in front of exponent
  if  [char] -  else  [char] +  then
  emit
  abs  0 <# # #s #> type
;

: N#  		\ d n -- d'
\ *G Convert n digits. INTERNAL.
  0 ?do # loop
;

: .FPsign	\ flag --
\ *G If flag is non-zero, generate a '-' otherwise a space.
  if  [char] -  else  bl  then
  hold
;

: .FPsep	\ --
\ *G Issue the FP separator, usually '.'.
  fp-char c@ hold  ;
external

: E.  		\ n exp --
\ *G Print the f.p. number on the stack in exponential form,
\ ** x.xxxxxEyy.
  op-prepare swap >r >r			\ -- d ; R: -- sign exp
  <#  8 n#  .FPsep  #
      r> .FPsign
  #>
  8 places @ min 0 max + 8 - type
  r> .exp
;

: REPRESENT	\ r c-addr u -- n flag1 flag2
\ *G Assume that the floating number is of the form +/-0.xxxxEyy.
\ ** Place the significand xxxxx at c-addr with a maximum of u digits.
\ ** Return n the signed integer version of yy. Return flag1 true
\ ** if f is negative, and return flag2 true if the results are
\ ** valid. In this implementation all errors are handled by
\ ** exceptions, and so flag2 is always true.
  2swap op-prepare >r >r                \ convert to a double
  <#  9 n#  #>                          \ generate string with max. digits
  rot min                               \ use input string size if smaller
  rot swap cmove                        \ copy string to buffer
  r> 1+                                 \ return exponent
  r>                                    \ return sign
  true                                  \ conversion always works
;

\   Table of rounding offsets - 0.5 to various powers
\   These are used to add for 5/4 rounding
\   (note these must be defined in reverse order)
CDATA
f# 0.5e-8     f# 0.5e-7       f# 0.5e-6       f# 0.5e-5
f# 0.5e-4     f# 0.5e-3       f# 0.5e-2       f# 0.5e-1
f# 0.5e-0
9 FARRAY F_Roundings

\   THIS WORD ASSUMES THAT THE OUTPUT WILL BE PRINTED IN NORMAL (NON-EXPONENT)
\   FORM - IT IS **NOT** A GENERAL PURPOSE 5/4 ROUNDING FUNCTION
: NORM-FROUND      \ fn | fn'; apply 5/4 rounding according to PLACES
  fsign >r
  Places @ 8 min 0 max
  F_Roundings f+
  r> if fnegate endif
;

: F.  		\ f --
\ *G Print the f.p. number in free format, xxxx.yyyy, if
\ ** possible. Otherwise display using the x.xxxxEyy format.
  dup -10 28 within 0= if
    e.  exit
  endif
  norm-fround                \ apply 5/4 rounding, assuming normalised output
  op-prepare swap >r -rot		\ -- sign d ; r: -- exp
  <#  8 r> - 0 max dup >r ?dup
      if  n#  .FPsep then
      #s  rot .FPsign
  #>
  r> dup if
    places @ -  0 max -
    dup 0 do
      2dup + 1- c@  [char] 0 <> ?leave
      1-
    loop
  else
    drop
  then
  2dup +  1- c@  fp-char isSep?
  if  1-  then
  type
;


\ -----------
\ *N Rounding
\ -----------

f# 0.5 fconstant %ROUNDPOINT
f# 1.0 fconstant %ONE
\ *G Floating point 1.0.

: FLOOR  	\ r1 -- r2
\ *G Floored round towards -infinity.
  fdup fint                             \ input, input rounded to zero
  fover f0> if				\ input was positive?
    fnip                                \   positive integral fine
  else
    fswap fover f- f0<>                 \   negative, but input not an integral?
    if  %one f-  then                   \     round towards -ve infinity
  then
;

: FROUND  	\ r1 --  r2
\ *G Round the number to nearest or even.
  fdup fint                             \ input, input rounded to zero
  fover fover f- fabs %roundpoint f< if	\ input fractional part closer to integral?
    fnip                                \   just return integral
  else
    fswap f0> if                        \   input positive?
      %one                              \     round towards +ve infinity
    else
      %one fnegate                      \     round towards -ve infinity
    then
    f+
  then
;



\ -------------------------
\ *N Trigonmetric functions
\ -------------------------
\ *P N.B. All angles are in radians.

 1518500249 0 fconstant %SQR.5
 1267358949 0 fconstant %S1
1792332800 -1 fconstant %S2
 1686629716 2 fconstant PI
 1686629715 1 fconstant PI/2
 1686629716 0 fconstant PI/4
1367130550 -1 fconstant 1/PI
      180 s>f fconstant %180
1686626304 18 fconstant %SINMAX
 1488522234 0 fconstant %D8
1865280597 -1 fconstant %D9
 1677721600 8 fconstant %ALNMAX
 1549082004 1 fconstant %1/LN2
 1488522236 0 fconstant %LN2
 1660944384 7 fconstant %ALGMAX
 1783446570 2 fconstant %1/LG2
1292913983 -1 fconstant %LG2
1367130550 0 fconstant 2/PI

: DEG>RAD	\ n1 -- n2
\ *G Convert degrees to radians.
  %180 f/ pi f*
;

: RAD>DEG	\ n1 -- n2
\ *G convert radians to degrees.
  %180 f* pi f/
;

: FSQR  	\ f1 -- f2 ; FSQR by Heron's formula
\ *G F2=sqrt(f1) by Heron's formula.
  over 0= if				\ check 0
    exit
  then
  over 0<				\ check -ve
  abort" square root of negative no.!"
  >r 0                                	\ .5 <=f<=1
  fdup %s1 f*  %s2 f+			\ HART est Y0
  fover fover f/ f+
  fswap fover f/ fswap 2-  f+
  r@ 1 and  dup r> + >r
  if  %sqr.5 f*  then
  r> 2/ +
;

internal
: ?TR-ERR	\ x --
  abort" Overflow in trig. function"
;

CDATA
f# -1.6666666E-1
f# 8.3333315E-3
f# -1.98409E-4
f# 2.7526E-6
f# -2.39E-8
5 farray SINARRAY

: (SIN)  	\ f1 -- f2
  fdup fdup f* fdup 0 sinarray f*
  4 1
  do  i sinarray f+ fover f*  loop
  4 sinarray f+ f* %1 f+ f*
;

: SINFN		\ f1 -- f2
  fdup 1/pi f*                          \ ang ang/pi --
  fint fdup f>s >r			\ ang int[ang/pi] --
  pi f* f-                              \ ang-pi*int[ang/pi] --
  fdup pi/2 f>
  if  pi fswap f-  then
  (sin)
  r> 1 and
  if  fnegate  then
;

: (FSIN)	\ f1 -- f2
  fsign  >r
  fdup %sinmax f> ?tr-err
  sinfn
  r> ?fnegate
;
external

: FSIN		\ f1 -- f2
\ *G f2=sin(f1).
  (FSIN)
;

: FCOS		\ f1 -- f2
\ *G f2=cos(f1).
  pi/2 f+  (fsin)
;

internal
f# 3.3333140E-1
f# 1.3339240E-1
f# 5.3374060E-2
f# 2.4565089E-2
f# 2.9005250E-3
f# 9.5168091E-3
6 farray TANARRAY

: (TAN)		\ f1 -- f2
  fdup fdup f* fdup 0 tanarray f*
  5 1
  do  i tanarray f+ fover f*  loop
  5 tanarray f+ f* %1 f+ f*
;
external

: FTAN		\ f1 -- f2
\ *G f2=tan(f1).
  fsign >r
  fdup %sinmax f> ?tr-err
  fdup 2/pi f* fint fswap fover pi/2 f* f-
  fdup pi/4 f> if
    pi/2 f- (tan) fswap %1 f+
  else
    (tan) fswap
  then
  f>s 1 and
  if  %1 fswap f/ fnegate  then
  r>
  if  fnegate  then
;

internal

f# 1.5707963E0
f# -2.1459880E-1
f# 8.8978987E-2
f# -5.0174305E-2
f# 3.0891881E-2
f# -1.7088126E-2
f# 6.6700901E-3
f# -1.2624911E-3
8 farray ASINARRAY

: ARCFN  	\ f1 -- f2
  fdup 0 asinarray f*
  7 1
  do  i asinarray f+ fover f*  loop
  7 asinarray f+
  fswap %1 fswap f- fsqr f*
;

f# -3.3333145E-1
f# 1.9993551E-1
f# -1.4208899E-1
f# 1.0656264E-1
f# -7.5289640E-2
f# 4.2909614E-2
f# -1.6165737E-2
f# 2.8662257E-3
8 farray ATANARRAY

: ATFN		\ f1 -- f2
  fdup fdup f* fdup 0 atanarray f*
  7 1
  do  i atanarray f+ fover f*  loop
  7 atanarray f+ f* %1 f+ f*
;

external

: FASIN  	\ f1 -- f2
\ *G f2=arcsin(f1).
  fsign >r
  arcfn pi/2 fswap f-
  r> if  fnegate  then
;

: FACOS  	\ f1 -- f2
\ *G f2=arccos(f1).
  fsign >r
  arcfn
  r> if  pi fswap f-  then
;

: FATAN  	\ f1 -- f2
\ *G f2=arctan(f1).
  fsign >r
  fdup %1 f> if
    %1 fswap f/  atfn pi/2 fswap f-
  else
    atfn
  then
  r> if  fnegate  then
;


\ ----------------------------------
\ *N Power and logarithmic functions
\ ----------------------------------

internal
f# 9.9999642E-1
f# -4.9987412E-1
f# 3.3179903E-1
f# -2.4073381E-1
f# 1.6765407E-1
f# -9.5329390E-2
f# 3.6088494E-2
f# -6.4535442E-3
8 farray LOGARRAY
external

: FLN  		\ f1 -- f2
\ *G Take the logarithm of f1 to base e and return the result.
  fdup f0> 0=
  abort" Invalid argument to FLN/FLOG"
  1 swap 1- >r  %1 f-
  fdup 0 logarray f*
  7 1
  do  i logarray f+ fover f*  loop
  7 logarray f+ f*  r> s>f  %d8 f* f+
;

: FLOG  	\ f1 -- f2
\ *G Take the logarithm of f1 to base 10 and return the result.
  fln %d9 f*
;

internal

f# 1.1512928E0
f# 6.6273088E-1
f# 2.5439357E-1
f# 7.2951737E-2
f# 1.7421120E-2
f# 2.5549180E-3
f# 9.3264267E-4
7 farray 10ARRAY

: (10^X)  	\ f1 -- f2
  fdup 0 10array f*
  6 1
  do  i 10array f+ fover f*  loop
  6 10array f+ f*
  %1 f+  fdup f*
;

f# -1.0000000E0
f# 4.9999992E-1
f# -1.6666530E-1
f# 4.1657348E-2
f# -8.3013598E-3
f# 1.3298820E-3
f# -1.4131610E-4
7 farray E-ARRAY

: (E^X)  	\ f1 -- f2
  fdup 0 e-array f*
  6 1
  do
    i e-array f+ fover f*
  loop
  6 e-array f+ f*
  %1 f+ %1 fswap  f/
;

external

: FE^X  	\ f1 -- f2
\ *G f2=e^f1.
  fsign >r
  fdup %alnmax f> abort" Overflow in FE^X"
  fdup  %1/ln2 f*
  fdup f>s >r
  fint %ln2 f* f-
  (e^x)
  r> +
  r> if  %1 fswap f/  then
;

: F10^X  	\ f1 -- f2
\ *G f2=10^f1
  fsign >r
  fdup %algmax f> abort" overflow in f10^x"
  fdup %1/lg2 f*
  fdup f>s >r
  fint %lg2 f* f-
  (10^x)
  r> +
  r> if  %1 fswap f/  then
;

: FX^N  	\ x-real n-integer -- fx^n
\ *G fx^n=x^n where x is a float and n is an integer.
  dup 0= if
    drop fdrop  %1  exit
  then
  dup >r
  abs >r  %1 fswap  r>
  begin
    dup 0>
  while
    dup 2 / dup 2*			\ -- fx fconst |n| |n|/2 |n|'
    rot -				\ -- fx fconst |n|/2 |n|'-|n|
    if
      >r fswap fover f* fswap r>
    then
    >r fdup f* r>
  repeat
  drop fdrop
  r> 0<
  if  %1 fswap f/  then
;

: FX^Y  	\ x-real y-real -- fn
\ *G fn=X^Y where Y and Y are both floats.
  fdup f0= if
    fdrop fdrop  %1  exit
  then
  fdup fdup fint f- f>r
  f>s >r fdup r>
  fx^n fswap
  fr> fdup f0<> if
    fover f0< abort" Result of FX^Y is complex"
    fswap fdup f0< >r
    fabs flog f*  f10^x
    r> if  %1 fswap f/  then
    f*
  else
    fdrop fdrop
  then
;


\ =========================
\ *N IEEE format conversion
\ =========================

: FP>IEEE	\ fp -- ieee32
\ *G Convert native FP value to IEEE 32 bit format.
  2dup or if				\ Don't convert 0
    #126 + #23 LSHIFT			\ IEEE biased 127 FP has 1+
    OVER 0<
    IF  $80000000 OR  THEN		\ FP signed mantissa, set IEEE sign bit
    SWAP ABS
    $3FFFFFFF AND 7 RSHIFT OR		\ clear msb IEEE hidden bit
  else
    drop				\ return IEEE 0e0
  endif
;

: IEEE>FP	\ ieee32 -- fp
\ *G Convert IEEE 32 bit float to native format.
  DUP IF				\ Don't convert 0
    DUP					\ save a copy to get the sign
    8 LSHIFT 1 RSHIFT $40000000 OR
    OVER ?NEGATE
    SWAP
    23 RSHIFT $FF AND #126 - 		\ shifted for FP exponent
  ELSE
    DUP					\ was already 0
  THEN
;



\ **********
\ *S Gotchas
\ **********
\ *P The ANS and Forth200x specifications define the format of
\ ** floating point numbers during text interpretation as:
\ *E Convertible string := <significand><exponent>
\ **
\ ** <significand> := [<sign>]<digits>[.<digits0>]
\ ** <exponent>    := E[<sign>]<digits0>
\ ** <sign>        := { + | - }
\ ** <digits>      := <digit><digits0>
\ ** <digits0>     := <digit>*
\ ** <digit>       := { 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 }

\ *P This format is handled by the word *\fo{FNUMBER?}. The word
\ ** *\fo{>FLOAT} accepts a more relaxed format.
\ *E Convertible string := <significand>[<exponent>]
\ **
\ ** <significand> := [<sign>]{<digits>[.<digits0>] | .<digits> }
\ ** <exponent>    := <marker><digits0>
\ ** <marker>      := {<e-form> | <sign-form>}
\ ** <e-form>      := <e-char>[<sign-form>]
\ ** <sign-form>   := { + | - }
\ ** <e-char>      := { D | d | E | e }

\ *P This restriction makes it difficult to use the text interpreter
\ ** during program execution as it requires floating point numbers
\ ** to contain 'D' or 'E' indicators, which is not profane practice.
\ ** A quick kluge to fix this is to change *\fo{FNUMBER?} as below.
\ *E Replace:
\ **   fcheck drop if                       \ valid f.p. number?
\ ** with:
\ **   fcheck or if                         \ valid f.p. number?
\ *P Note that this change can/will cause problems if number base
\ ** is not *\fo{DECIMAL}.


\ ****************************
\ *S Changes from v6.0 to v6.1
\ ****************************
\ *P Renamed *\fo{DINT} to *\fo{F>D} for consistency. *\fo{F>D} is the ANS word.
\ ** The original *\fo{F>D} was just a synonym. Similarly *\fo{SINT} was
\ ** renamed to *\fo{F>S}.

\ *P The word *\fo{FLOATS} that enabled floating point number conversion
\ ** has been renamed to *\fo{REALS} to avoid a name conflict with the
\ ** ANS word of the same name.

\ *P The *\fo{F-PACK} vocabulary has been removed as no one liked it,
\ ** and it could be considered contrary to the ANS Forth specification.
\ ** If you wish to retain the *\fo{F-PACK} vocabulary, add the following lines
\ ** before and after the compilation of the floating point code:
\ *E   only forth definitions         \ *** added ***
\ **   vocabulary f-pack              \ *** added ***
\ **   also f-pack definition         \ *** added ***
\ **   include %CommonDir%\Sfp32Hi    \ primitives
\ **   include %CommonDir%\Sfp32Com   \ common high level code
\ **   previous definitions           \ *** added ***

\ *P The code enabling floating point to work in degrees or
\ ** radians has been commented out for ANS compatibility. All
\ ** trig functions now operate in radians. The commented out
\ ** code may be uncommented if you need backward compatibility.

\ ==========================================
\ *N 32 bit targets: software floating point
\ ==========================================
\ *P Overhauled 32 bit software floating point and incorporated
\ ** improvements contributed by Hiden Analytical. These include
\ ** more complete special case detection, faster high level code,
\ ** and more accurate number input and output.

\ *P Removed all use of global variables except *\fo{PLACES} to make
\ ** the floating point code usable in interrupt routines and
\ ** in multitasked systems. If the output routines are to be
\ ** multitasked, change the definition of *\fo{PLACES} from:
\ *C   VARIABLE PLACES  8 PLACES !
\ *P to:
\ *C   CELL +USER PLACES
\ *P and remember to initialise *\fo{PLACES} before using the floating
\ ** point output routines.

\ *P Many words that are only useful as factors have been made
\ ** headerless to save target memory space.

\ ==========================================
\ *N 16 bit targets: software floating point
\ ==========================================
\ *P Note that the 16 bit floating point pack is not re-entrant.
\ ** If you need to use the floating point pack in a multitasking
\ ** system, you should convert the global variables to *\fo{USER}
\ ** variables. The word *\fo{+USER} can be used
\ *C   <size> +USER <name>
\ *P to define a *\fo{USER} variable of a given size (normally a CELL)
\ ** at the next free offset in the USER area. Only *\fo{PLACES} will
\ ** need initialisation.


\ ======
\ *> ###
\ ======

