\ 7-1-A.fth - target test harness for Forth 7 cross compiler

((
Copyright (c) 1991, 1992, 1995, 1998, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

Testing for Forth words

Adapted from 'Testing Toolkit' by P. Koopman Jr. - Forth Dimensions XII,3

Author:	J Lee

To do
=====

Change history
==============
20090217 SFP005	Converted to v7 cross compiler.
		DocGenned.
19980417 MRD004 Ensure that line numbers come out in *decimal*.
19951113 JOL003	Modified for EPIC Forth token compiler.
19920221 JOL002 Modified to work in COLON definitions during Cross
		Compilation.
19911213 JOL001 Original release.
	 	This wordset provides a means of testing programs in a
		formalised fashion. A word is executed surrounded by a
		formal stack definition.
		DS( specified_inputs --) <WORD> (-- required_outputs )DS
		This version works with MF3.xx.
))

only forth definitions
decimal

\ =============
\ *! targettest
\ *T Target test suites
\ =============

\ *******************
\ *S MPE test harness
\ *******************
\ *P The code in the files *\i{Common\TESTS\MPE\7-1-A.FTH}
\ ** and *\i{Common\TESTS\MPE\7-1-B.FTH} contains a test
\ ** harness and tests respectively for MPE kernels. The
\ ** files for previous v5 and v6 compilers are still present in
\ ** the directory, but are no longer supported.

\ *P Test code may be cross-compiled or used interactively.


\ ***********************
\ *N Target harness words
\ ***********************
\ *P This section just describes the mechanics of the test
\ ** harness. How to use it is covered in the next section.

variable Stack1		\ -- addr
\ *G Stack depth marker. Depth at entry to stack check.
variable Stack2		\ -- addr
\ *G Stack depth marker. Depth before entry of test word.
variable Stack3		\ -- addr
\ *G Stack depth marker. Depth after execution of test word.
variable Test-Line#	\ -- addr
\ *G Holds line number of word under test
variable Left-Loop	\ -- addr
\ *G Holds flag for early loop exit

[undefined] .decimal [if]
: .decimal	( n -- )	base @ swap  decimal .  base !  ;
\ *G Display in decimal.
[then]

: Reset-Stack	\ -- ; reset data stack )
\  s0 @ sp!				\ this would be nice
  begin  depth  while  drop  repeat	\ long winded but effective
;

: Check-Element	\ element# -- t/f
\ *G Check a value on the output stack.
  dup if				\ only do for non-zero elements
    roll <> if
      ." FAILED, Incorrect Result"
      reset-stack
      -1 Left-Loop !			\ flag that loop is left prematurely
      -1
    Else
      0
    Endif
  Endif
;

: Check-Stack	\ sentinel  --
\ *G Check stack for corruption.
  $0bad1 <> if				\ check stack corruption
    source type ."  FAILED, stack corrupted"
    reset-stack				\ reset data stack
  Else
    ." PASSED."
  Endif
;


\ ***********************
\ *N Target testing words
\ ***********************
\ *P A word is executed surrounded by formal stack definitions
\ ** containing the test inputs and required outputs. A test is
\ ** written in the form:
\ *C   DS( inputs --) <WORD> (-- outputs )DS
\ *P for example:
\ *C   ds( $05555 $0aaaa --) or (-- 0ffff )ds

\ *P Note that the input and output expressions can contain
\ ** executable Forth as well as literals. You are not restricted
\ ** to a single word under test: you can use a phrase.


: Ds(		\ -- sentinel
\ *G Start defining the input data.
  $0bad1				\ sentinel (check for stack corruption)
  depth stack1 !  			\ intial depth
;

: --)-later	\ .. line# -- ..
  test-line# !  depth stack2 !		\ pre execution depth
;

host-compilation
c-c  only forth also c-c also cc/c
: Sline#	\ --
  line# @ general-literal
;
c-c only forth also c-c definitions
target-compilation

compiler
: --)		\ --
  Sline# --)-later
;
target

: --)		\ .. -- ..
\ *G Finish defining the input data.
  -1 test-line# !  depth stack2 !	\ pre execution depth
;

: (--		\ .. -- ..
\ *G Start defining the correct outputs.
  depth stack3 !  		\ post execution depth ; )
;

: )Ds		\ ... --
\ *G Finish defining the correct outputs.
  cr  Test-Line# @ -1 <> if
    ." Testing Line# : " test-line# @ .decimal
  endif
  depth stack3 @ - dup			\ post execution elements
  stack3 @ stack1 @ -			\ pre execution elements
  <> if					\ give up if not the same
    source type
    ."  FAILED, Number of elements not the same"
    reset-stack
  Else
    0 left-loop !			 \ turn loop exit flag off
    0 swap ?Do				 \ check result against expected
      i check-element
      If  leave  Endif
    -1 +Loop
    left-loop @ 0=
    If  check-stack  Endif
  Endif
;


\ **************************
\ *S ANS tester by Jon Hayes
\ **************************
\ *P After the ANS Forth process, John Hayes at JHU/APL developed
\ ** an ANS test suite. A version slightly modified by MPE for use
\ ** with cross compilers can be found in *\i{Common\Tests\ANS}.
\ ** Cross compile *\i{Tester.fth}, and then *\fo{INCLUDE} test
\ ** files as required.


\ ======
\ *> ###
\ ======

decimal

