\ high level floating point code file - 16 bit targets, ROM model

((
Copyright (C) 1983-2000
MicroProcessor Engineering Ltd.
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)2380 631441
fax: +44 (0)2380 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))


\ ftemp f! f@ fdup fover frot fswap fdrop

((
    Many of the floating point operations run faster when implemented
    using a temporary buffer in RAM. This is particularly true of the RTX
    processor when using a common data stack for floating point and
    integer numbers.
))

decimal

3 cells constant float

: floats	\ n -- n'
  float *  ;
: float+
  float +  ;

interpreter
: floats  float *  ;
: float+  float +  ;
target

3 floats buffer: ftemp	\ -- addr ; three item buffer

: f!		\ fn addr -- ; note top of stack to low address
  >r  r@ !  r@ 2+ !  r> 4 + !  ;

: f@		\ addr -- fn ; top of stack from low address
  >r  r@ 4 + @  r@ 2+ @  r> @  ;

harvard? [if]
: f@c           \ addr -- fn ; top of stack from low address
  >r  r@ 4 + @c  r@ 2+ @c  r> @c  ;
[then]

: fdup		\ fn -- fn fn
  ftemp f!  ftemp f@  ftemp f@  ;

: fover		\ f1 f2 -- f1 f2 f1
  [ ftemp float+ ] literal f!  ftemp f!
  ftemp f@  [ ftemp float+ ] literal f@  ftemp f@  ;

decimal

: frot		\ f1 f2 f3 -- f2 f3 f1
  ftemp f!				\ save f3 in buffer
  [ ftemp float+ ] literal f!		\ save f2 in buffer
  [ ftemp 2 floats + ] literal f!	\ save f1 in buffer
  [ ftemp float+ ] literal f@		\ f2
  ftemp f@				\ f3
  [ ftemp 2 floats + ] literal f@  ;	\ f1

: fswap		\ f1 f2 -- f2 f1
  ftemp f!  [ ftemp float+ ] literal f!
  ftemp f@  [ ftemp float+ ] literal f@  ;

: fdrop		\ f1 --
  drop drop drop  ;

: fnip		\ f1 f2 -- f1
  fswap fdrop  ;


\ High level shift routines

hex

: <-S		\ d1 carry-in-flag -- d2 carry-out-flag
  -rot				\ save flag away
  dup 0< >r			\ save carry out
  2dup d+			\ shift left by doubling
  rot if  0,1 d+  endif		\ add carry in
  r> 1 and  ;			\ provide carry out

: S->		\ d1 carry-in-flag -- d2 carry-out-flag
  >r				\ save carry in flag
  over 1 and  -rot		\ save carry out flag
  swap 2/ 07FFF and		\ shift right l.s. word
  over 1 and			\ look at carry from m.s. word
  if  08000 or  endif		\ merge into l.s. word
  swap 2/ 07FFF and		\ shift right m.s. word
  r>				\ look at carry in
  if  08000 or  endif		\ merge into m.s. word
  rot  ;			\ bring back carry out

: d<<1		\ d1 -- d1<<1
  2dup d+  ;

: d>>1		\ d1 -- d1>>1
  swap 2/ 07FFF and		\ shift right l.s. word
  over 1 and			\ look at carry from m.s. word
  if  08000 or  endif		\ merge into l.s. word
  swap 2/ 07FFF and  ;		\ shift right m.s. word

: q<<1		\ q1 -- q2
  2*  over 0<			\ shift, look at carry from lower
  if  1+  endif			\ merge into higher
  >r				\ preserve result
  2*  over 0<			\ shift, look at carry from lower
  if  1+  endif			\ merge into higher
  >r				\ preserve result
  2*  over 0<			\ shift, look at carry from lower
  if  1+  endif			\ merge into higher
  >r				\ preserve result
  2*				\ shift lowest
  r> r> r>  ;			\ restore higher items

decimal


\ defining words

\ Cross compiler versions of the defining words
interpreter
harvard? [if]
: f,c		\ f --
  cdata  , , ,  idata
;

: fvariable
  idata create
    0 ,(r) 0 ,(r) 0 ,(r)
;

: fconstant	( fn -- )
  cdata CREATE
    f,c
  idata
  DOES>
    F@C
;

: farray	( fn-1..f0 n -- ; compile / n -- fn ; execute )
  cdata CREATE
    0 ?DO  f,c  LOOP
  idata
  DOES>
    SWAP float * + F@C
;
[else]
: f,		\ f --
  , , ,
;

: fvariable
  idata create
    0 , 0 , 0 ,
  cdata
;

: fconstant	( fn -- )
  cdata CREATE
    , , ,
  DOES>
    F@
;

: farray	( fn-1..f0 n -- ; compile / n -- fn ; execute )
  cdata CREATE
    0 ?DO  , , ,  LOOP
  DOES>
    SWAP  float * + F@
;
[then]
target

\ target definitions
harvard? [if]
: fvariable	( -- ; -- addr )
  there dup constant
  float allot-ram
  float erase
;

: fconstant	( fn -- )
  CREATE  , , ,  DOES> F@C
;

: farray	( fn-1..f0 n -- ; compile / n -- fn ; execute )
  CREATE
    0 ?DO  , , ,  LOOP
  DOES>
    SWAP float * + F@C
;
[else]
: fvariable	( -- ; -- addr )
  CREATE  0 , 0 , 0 ,
;

: fconstant	( fn -- )
  CREATE  , , ,  DOES> F@
;

: farray	( fn-1..f0 n -- ; compile / n -- fn ; execute )
  CREATE
    0 ?DO  , , ,  LOOP
  DOES>
    SWAP float * + F@
;
[then]


