\ ExternAAPCS.fth - Access to C libraries

((
Copyright (c) 2001-2010, 2011, 2012, 2013, 2014, 2016, 2018, 2019, 2020
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
Option to switch floats to integer registers per base standard.


Change history
==============
20200714 SFP036 64 bit host conversion.
		Single float correction in FPSREGX.
		Fixed [A].
20190130 SFP035 Corrected LongLong handling.
20181217 SFP034 Added option to not save R12.
		Refactored save of R9.
20161107 SFP033 Added PDIC( a, b ) for BBC micro:bit.
		Added code for Cortex-M0/M1.
20160321 SFP032 Added DIR( addr ) and DIC( pri sec ).
20160317 SFP031 Deferred some portions to allow expansion.
20160314 SFP030 Converted to use with cross compiler.

20160108 SFP025 Updated for FP arguments and results.
20140731 SFP024 Converted to ARM EABI.
20140521 SFP023 Enhanced parsing of *** types.
20131119 SFP022	Made LOCATE go to first line of EXTERN:
20130520 SFP021 Exposed INIT-LIB. Added CLEAR-LIB.
2012xxyy SFP020 Added compatibility words for other Forths.
20120418 SFP019 Added FRAMEWORK and FRAMEWORK: for OSX.
		Allowed library and framework redefinitions
		to be ignored.
20120411 SFP018 Prevented inlining of words containing EXTERNs.
20120312 SFP017 Added EXTERNVAR and ALIASEDEXTERNVAR.
20111104 SFP016 More testing for tabs.
20111021 SFP016 Added #BADEXTERNS #BADLIBS and .BADLIBS.
20111004 SFP015 Added PLONG to Windows types.
20110912 SFP014 Added library mode options.
20110427 SFP013 Added additional pointer parsing.
                Added __IN_OPT, __INOUT_OPT and __OUT_OPT for Windows.
20110415 SFP012 Supports C // and /*...*/ comments in argument
                parsing.
                Added __IN, __INOUT and __OUT for Windows.
                Added ** and ***.
20100223 SFP011 Added protection mechanism to Externs.
20100222 SFP010 Moved PREEXTERN and POSTEXTERN to Forth voc.
20091123 SFP009	Refactored with startup.fth.
20090610 SFP008 Added documentation for FLAG from Marcel Hendrix.
20090609 SFP007 Added .BADEXTERNS.
20090502 SFP006	Added code for Windows protection with floats.
20020323 MPE005 Refactored client specific code in prologue and
                epilogue.
20090319 MPE004 Modified after distribution list discussions.
20090310 MPE003 Refactored.
20090131 MPE002 Tightened up.
		Put most words in separate vocabulary.
20090121 MPE001 Ported from the Windows/Linux version.
))


\ ==========
\ *! sharedc
\ *T Calling functions in other languages
\ ==========

only forth also c-c definitions decimal

vocabulary types(t)	\ --
\ Where the "C" style types go for the AAPCS: notation.
vocabulary AAPCS.Voc	\ --
\ Where the guts of the AAPCS: mechanism live.

also AAPCS.Voc definitions

: (search)   { c-addr1 u1 c-addr2 u2 -- c-addr3 u3 flag }
  c-addr1 u1 u2 - 1+ bounds     \ Search for first char of $2 in $1.  Note that
  ?do                           \ we only check LEN($1)-LEN($2)+1 chars of $1.
    i c@  c-addr2 c@  =         \ Compare next char of $1 with first char of $2.
    if
      i c-addr2 u2 s=           \ Is the whole of $2 present?
      if
        i u1 i c-addr1 - - true \ Yes - build return stack parameters...
        unloop exit             \ ...and bail out.
      then
    then
  loop
  c-addr1 u1 false            \ End of loop and first char not found - no match.
;

: ANSSEARCH	( c-addr1 u1 c-addr2 u2 -- c-addr3 u3 flag  )
\ By a fuckup, SEARCH is used for something else in the cross-compiler.
  2 pick over <				\ Is $1 shorter than $2?
  if  2drop false exit  then		\ Yes - $2 *can't* be in $1.
  dup 0 <=				\ Is $2 zero length?
  if  2drop true exit  then		\ Yes - we have found it at the start of $1.
  2 pick 0 <=				\ Is $1 zero length?
  if  2drop false exit  then		\ Yes - string not found.
  (search)
;

: evaluate(t)	\ i*x c-addr u -- j*x
\ Process the supplied string as though it had been entered via the
\ interpreter.
  save-input n>r
  to-source  -1 to source-id  >in off
  'SourceFile off
  ['] interpret(t) catch
  nr> restore-input drop
  throw
;

: xcr		\ --
\ When debugging, we need to force output to be flushed before a
\ segfault can occur and destroy buffered output.
  cr  key? drop  50 ms
;

: zcr		\ --
  xcr  key drop
;

: string-up	\ caddr len --
\ Copy string to HERE and WORD-BUFFER
  2dup here place  word-buffer place
;

: $:(t)		\ caddr len --
\ Start a colon definition using caddr/len as the name.
  2dup upper 2>r
  (])  !csp
  ColonChild DefnType !
  <:noname> off
  2r> string-up xc-create
  c" : " chere(t) +Page(t) .log
\  here(t) 1 c" SYMBOL-: " -1 extsym-log        \ symbol table
  start-inline
  c_:
  'latest(s) @(m) dup 'latest: !(m)
  status+ mark-smudged
;

: ;(t)
  ?finish-locals(t)
  ?csp
  c_;  [last-req]
  end-inline                                    \ mark end of inline code
  ?b-d  If  postpone exit  Endif
  <:noname> @ 0= if
    'latest: @(m) status+ clear-smudged		\ make symbol visible to ?TARGET
  endif
  ([)  0 state!
  in-host-build @(m) If
    host-mode on(m)             \ if we're in host building mode then switch on mechanism again
    only forth also c-c also extensions definitions     \ set default host building order
    smudge                      \ have to make host word visible
  Endif
;


\ ***************
\ *S Introduction
\ ***************
\ *P The ARM/Cortex Forth cross-compiler supports calling
\ ** functions in C or any language that can provide functions
\ ** that use the AAPCS calling convention. This is an ARM
\ ** convention documented in *\i{IHI0042F_aapcs.pdf}.

\ *P This chapter documents the Forth side of the Forth to C
\ ** interface. A separate chapter discusses the C side of the
\ ** interface.

\ *P In order to provide the greatest isolation between sections
\ ** of code written in other languages, the functions foreign to
\ ** Forth are accessed by SVC calls and/or jump tables. The simple
\ ** solution just uses SVC calls for all foreign functions.

\ *P All foreign function calls are made using the AAPCS calling
\ ** convention.

\ *P Calls with a variable number of parameters (*\b{varargs}) are
\ ** not supported.

\ *P The interface is defined for Cortex-Mx CPUs only. If you need
\ ** an ARM32 interface, contact MPE.

\ *P This work is directly inspired by Robert Sexton's Sockpuppet
\ ** interface:
\ *C   https://github.com/rbsexton/sockpuppet
\ *P His contribution and permission are gratefully acknowledged.

\ ============
\ *N SVC calls
\ ============
\ *P Access to functions in a library is provided by the
\ ** *\fo{SVC( n )} syntax which is similar to a C style function
\ ** prototype, e.g.
\ *E SVC( n ) int open(
\ **   const char * pathname, int flags, mode_t mode
\ ** );
\ *P where n is the SVC call number. Everything on the source
\ ** line after the closing ')' is discarded. The example is
\ ** for the function *\b{open()} from a file API, and produces
\ ** a Forth word *\fo{open}.
\ *C   open	\ pathname flags mode -- int

\ *P The calling convention is always AAPCS. Because of the variations
\ ** between C compilers and silicon vendor conventions, there are
\ ** options for saving R9 and R12. In many cases of embedded systems
\ ** compilers these registers do not need to be saved.

\ *P The parser used to separate the tokens is not ideal. If you
\ ** have problems with a definition, make sure that *\fo{*}
\ ** tokens are white-space separated. Formal parameter names,
\ ** e.g. *\i{argv} above are ignored. Array indicators, *\i{[]}
\ ** above, are also ignored when part of the names.

\ *P A small number of SVC calls are defined by the system. Regard
\ ** entries 0..15 as being reserved by the Sockpuppet API.
\ *E SVC( 0 ) int GetSAPIversion( void );
\ ** \ Returns the Sockpuppet interface version.
\ ** SVC( 1 ) void * GetLinkList( void );
\ ** \ Returns the head of the linked list used to share data.
\ ** SVC( 7 ) uint32_t GetTimeMS( void );
\ ** \ Returns the value of the millisecond timer. On overflow,
\ ** this wraps around.
\ ** SVC( 14 ) void * GetDirFnTable( void );
\ ** \ Returns the address of the direct jump table, which may
\ ** \ be the same as the SVC jump table below.
\ ** SVC( 15 ) void * GetSVCFnTable( void );
\ ** \ Returns the address of the SVC jump table.


\ ==============
\ *N Jump tables
\ ==============
\ *P Calling through SVC calls is simple, but has overhead. In
\ ** addition, lower priority interrupts are disabled by an SVC
\ ** call and so interrupt latency suffers badly. You cannot
\ ** do ticker-based timing inside an SVC-based call without
\ ** messing about with the ARM/Cortex priority mechanisms. The
\ ** solution is is to call the functions through a jump table
\ ** as often used by an SVC call mechanism. This requires a little
\ ** bit more code than the pure SVC approach, but has *\b{much}
\ ** better interrupt latency and gives more flexibility.

\ *P The example of *\b{open()} is used again.
\ ** Access to functions in a library is provided by the
\ ** *\fo{JTI( n )} syntax which is similar to a C style function
\ ** prototype, e.g.
\ *E JTI( n ) int open(
\ **   const char * pathname, int flags, mode_t mode
\ ** );
\ *P where n is the index (0, 1, 2, ...) into the jump table. The
\ ** parsing rules are as for SVC calls. The definition results
\ ** in a Forth word *\fo{open}.
\ *C   open	\ pathname flags mode -- int

\ *P Before use, you have to tell the cross compiler where the
\ ** address of the jump table is held, and then use an SVC call
\ ** find the address of the jump table.

\ *E SVC( 15 ) void * GetDirFnTable( void );
\ ** \ Define SVC call that returns the address of the
\ ** \ jump table.
\ **
\ ** variable JT     \ -- addr
\ ** \ Holds the address of the jump table.
\ ** JT holdsJumpTable
\ ** \ Tell the cross compiler where the jump table address
\ ** \ is held.
\ **
\ ** : initJTI       \ -- ; initialise jump table calls
\ **   GetDirFnTable JT !  ;
\ **
\ ** JTI( n ) int open(
\ **   const char * pathname, int flags, mode_t mode
\ ** );


\ ==============================
\ *N Double indirect call tables
\ ==============================
\ ** This form is used when calling ROM routines in devices
\ ** from TI, NXP and others. In these the primary table entries
\ ** hold the address of other tables. The secondary index
\ ** is then used to fetch the address of the actual routine
\ ** to be called. The Forth word marshalls the parameters
\ ** and calls the routine.

\ *P Before use, you have to declare the base address of the
\ ** primary ROM table used for calling ROM functions. For
\ ** Luminary/TI CPUs, this will probably be:
\ *C  $0100:0010 setPriTable

\ *P Now you can define a set of ROM calls, for example,
\ ** again for a TI CPU.
\ *E DIC( 4, 0 ) void ROM_GPIOPinWrite(
\ **    uint32 ui32Port, uint8 ui8Pins, uint8 ui8Val
\ ** );
\ *P where
\ *(
\ *B ROM_APITABLE is an array of pointers located at 0x0100.0010.
\ *B ROM_GPIOTABLE is an array of pointers located at ROM_APITABLE[4].
\ *B ROM_GPIOPinWrite is a function pointer located at ROM_GPIOTABLE[0].
\ *)
\ *P Parameters:
\ *(
\ *B ui32Port is the base address of the GPIO port.
\ *B ui8Pins is the bit-packed representation of the pin(s).
\ *B ui8Val is the value to write to the pin(s).
\ *)
\ *P Description:*\br{}
\ ** Writes the corresponding bit values to the output pin(s)
\ ** specified by ui8Pins. Writing to a pin configured as an
\ ** input pin has no effect. The pin(s) are specified using a
\ ** bit-packed byte, where each bit that is set identifies the
\ ** pin to be accessed, and where bit 0 of the byte represents
\ ** GPIO port pin 0, bit 1 represents GPIO port pin 1, and so
\ ** on.
\ *P Returns:*\br{}
\ ** None.
\ *P To call this function, use the Forth form:
\ *C   port pins val ROM_GPIOPinWrite


\ ==============================
\ *N Pointer to double indirect call tables
\ ==============================
\ ** This form is used when calling ROM routines via double
\ ** indirect calls as in the previous section except that the
\ ** address of the primary table is found at run time, e.g.
\ ** by the C launcher passing the address at start up, or by
\ ** using an SVC call to pass it into Forth RAM. As before,
\ ** the primary table entries hold the address of other tables.
\ ** The secondary index is then used to fetch the address of the
\ ** actual routine to be called. The Forth word marshalls the
\ ** parameters and calls the routine. This form is used by
\ ** systems such as the mbed version of the Sockpuppet interface
\ ** for the BBC micro:bit.

\ *P Before use, you have to declare the address that holds the
\ ** addressof the primary ROM table used for calling ROM functions.
\ *E  variable PriPointer	\ set at powerup somehow
\ **  PriPointer setPriPointer	\ declare to cross compiler

\ *P Now you can define a set of ROM calls, for example,
\ ** the call to the serial version of *\fo{TYPE} in a BBC
\ ** micro:bit is:
\ *C PDIC( 2, 3 ) void serType( uint8_t *buff, int length );
\ *P To call this function, use the Forth form:
\ *C   <caddr> <len> serType


\ ===============
\ *N Direct calls
\ ===============
\ *P Where the address of the routine is known at the Forth
\ ** compile time, you can use a direct call.
\ *C   DIR( addr ) int foo( int a, char *b, char c );
\ *P The Forth word marshalls the parameters and calls the
\ ** subroutine at target address *\i{addr}.


\ *****************************
\ *S C comments in declarations
\ *****************************
\ *P Rudimentary support for C comments in declarations is
\ ** provided, but it is good enough for the vast majority of
\ ** declarations.
\ *(
\ *B Comments can be *\fo{// ...} or *\fo{/* ... */},
\ *B Comments must be at the end of the line,
\ *B Comments are treated as extending to the end of the line,
\ *B Comments must not contain the ')' character.
\ *)
\ *P The example below is taken from a *\i{SQLite} interface.
\ *E SVC( x ) int sqlite3_open16(
\ **   const void * filename,  /* Database filename [UTF-16] */
\ **   sqlite3 ** ppDb         /* OUT: SQLite db handle */
\ ** );


\ *********
\ *S Format
\ *********
\ *E 'SVC(' <n> ')' <return> [ <callconv> ] <name> '(' <arglist> ')' ';'
\ ** 'JTI(' <n> ')' <return> [ <callconv> ] <name> '(' <arglist> ')' ';'
\ ** 'DIC(' <n1>, <n2> ')' <return> [ <callconv> ] <name> '(' <arglist> ')' ';'
\ ** 'PDIC(' <n1>, <n2> ')' <return> [ <callconv> ] <name> '(' <arglist> ')' ';'
\ ** 'DIR(' <n> ')' <return> [ <callconv> ] <name> '(' <arglist> ')' ';'
\ **
\ ** <n>         := { literal }
\ ** <return>    := { <type> [ '*' ] |  void }
\ ** <arg>       := { <type> [ '*' ] [ <name> ] }
\ ** <args>      := { [ <arg>, ]* <arg> }
\ ** <arglist>   := { <args> | void }	Note: "void, void" etc. is illegal.
\ ** <callconv>  := { PASCAL | WINAPI | STDCALL | "PASCAL" | "C" }
\ ** <name>      := <any Forth acceptable namestring>
\ ** <type>      := ... (see below, "void" is a valid type)

\ *P The Forth <name> is case-insensitive.

\ *P As a standard Forth's string length for dictionary names is
\ ** only guaranteed up to 31 characters for portable source code,
\ ** long API names can cause problems.

\ **********************
\ *S Calling Conventions
\ **********************
\ *P In the discussion *\b{caller} refers to the Forth system
\ ** (below the application layer and *\b{callee} refers to a
\ ** a foreign function. The *\fo{SVC()} only supports the AAPCS
\ ** standard as used by C.

\ *P When using calls to floating point libraries, there are
\ ** three possible calling conventions, *\fo{noFPAPI},
\ ** *\fo{softFPABI) and *\fo{hardFPABI}.
\ *(
\ *B *\fo{noFPABI): No floating point support is provided.
\ ** The value *\fo{FPABI} is set to 0.
\ *B *\fo{softFPABI): Floating point numbers are passed to
\ ** and from the system in integer registers or on the stack.
\ ** The value *\fo{FPABI} is set to 1.
\ *B *\fo{hardFPABI}:  Floating point numbers are passed to
\ ** and from the system in the VFP registers.
\ ** The value *\fo{FPABI} is set to 2.
\ *)

\ *P For now, the only combination supported by the cross compiler
\ ** uses *\b{FPsystem=2} and *\b{FPABI=2}, i.e 32 bit VFP floats,
\ ** a separate Forth float stack, and a hardfloat ABI.


\ *************************
\ *S Promotion and Demotion
\ *************************
\ *P The system generates code to either promote or demote non-CELL
\ ** sized arguments and return results which can be either signed or
\ ** unsigned. Although Forth is an un-typed language it must deal with
\ ** libraries which do have typed calling conventions. In general
\ ** the use of non-CELL arguments should be avoided but return
\ ** results should be declared in Forth with the same size as the C or
\ ** PASCAL convention documented.

\ ********************
\ *S Argument Reversal
\ ********************
\ *P The default calling convention for AAPCS is used. The
\ ** right-most argument/parameter in the C-style prototype is
\ ** on the top the Forth data stack.
\ ** When calling an external function the parameters are reordered
\ ** as required by the AAPCS; this is to enable the
\ ** argument list to read left to right in Forth source as well
\ ** as in the C-style operating system documentation.


\ **********************************
\ *S Controlling external references
\ **********************************

2 value FPABI	\ -- u
2 value FPsystem	\ -- u

also cc/i
: setFPABI	( u -- ) to FPABI  ;
\ *G When using calls to floating point libraries, there are
\ ** three possible values, 0 for *\fo{noFPAPI}, 1 for\fo{softFPABI)
\ ** and  2 for *\fo{hardFPABI}. At present, only the *\fo{hardFPABI}
\ ** is supported.
: setFPsystem	( u -- ) to FPsystem  ;
\ *G Defines which floating point pack
\ ** is installed and active, 0 for none, 1 for software floating
\ ** point, 2 for VFP2 and 3 for VFP3. At present, only the VFP2
\ ** option is supported, which is binary compatible with VFP3.
previous definitions


\ ********************************
\ +S Import parsing and generation
\ ********************************

\ ======================
\ +N Data and structures
\ ======================

\ These are constants used as THROW codes with associated text messages.
Errdef err_ImpType	"Invalid type in foreign function import"
ErrDef err_CGsize	"Invalid size in foreign function argument preparation"
ErrDef err_RetType	"Invalid return type"
ErrDef err_RetSize	"Invalid return size"
ErrDef err_RegParm	"Invalid register for parameter"
ErrDef err_noFPABI	"No external FP ABI defined"
ErrDef err_noFPsystem	"No Forth FP system defined"
ErrDef err_varargs	"Varargs not supported"


\ argument types - BOOL could be considered a form of UINT, but
\ is O/S specific.
0 constant SINTtype	\ -- n ; default is signed int
1 constant UINTtype	\ -- n ; signed int
2 constant BOOLtype	\ -- n ; C boolean
3 constant FLOATtype	\ -- n ; float/double/ext ....

0 constant notReg	\ -- n ; parameter not in register
1 constant IntReg	\ -- n ; parameter in integer/general register
2 constant FPReg	\ -- n ; parameter is in VFP register

struct /argItem	\ -- len
  int ai.Size		\ argument size in bytes, 0=void, 1=byte/char, 2=short, 4=int ...
  int ai.type		\ 0=uint, 1=sint, 2=bool, 3=float
  int ai.FrameOff	\ offset on operating system frame to data
  int ai.Reg		\ register number to be used
  int ai.inReg?		\ true if in register (may be FPreg). 0=no, 1=Reg, 2=FPreg
  int ai.PSPoff		\ offset on Forth parameter stack as if TOS uncached
  int ai.FSPoff		\ offset on Forth float stack as if FTOS uncached
end-struct
#32 /argitem * constant /argList	\ size of the argument list, max 32 items

\ Operating system dependent items. These VALUEs are set to
\ defaults at the start of each external import declaration.

0 value CalleeCleaned?	\ -- flag
\ 0 = "C", 1 = "PASCAL", used for cleanup only. If the calling
\ mechanism preserves the Forth stack pointers, this value is not
\ needed.
4 value RegisterArgs	\ -- u
\ Maximum number of arguments passed in registers.
4 value FrameAlignment	\ -- u
\ Set to alignment of stack frame in memory. Must be a power of two.
true value AlignInFrame? \ -- flag
\ True if items on the stack frame are padded for data alignment.
true value REQUIRE_NREV	\ -- flag
\ Set true if arguments are reversed by default (C argument order),
\ i.e. the leftmost argument in the argument declaration list is
\ on the top of the C stack.
true value L>R?		\ -- flag
\ Set true when the Forth stack order matches the argument
\ declaration order.
0 value ObjPtr?		\ -- flag
\ True if an object-oriented call is required, i.e. a hidden
\ "this" pointer is used. The location of "this" is compiler and
\ OS dependent.
\   0 = none
\   1 = VC++, pointer in fixed register
\   2 = gcc, pointer on top, but below struct pointer (see below)
0 value StructRet?	\ -- flag
\ Set this true if the import returns a structure and requires a
\ hidden structure pointer. The location of the structure pointer
\ is compiler and OS dependent.
\   0 = none
\   1 = VC++, pointer in fixed register
\   2 = gcc
0 value varargs?	\ -- flag
\ Set this true if the parameter list is of variable length.
1 value forceTbits?	\ -- flag
\ Set true if the T bit in calls has to be forced.
1 value saveR9?		\ --
\ *G Set true if R9 is to be saved by calls.
1 value saveR12?		\ --
\ *G Set true if R12 is to be saved by calls.

defer setDefExt	\ --
\ Set the call type defaults in the VALUEs above.

#256 buffer: importName		\ name of DLL function
#256 buffer: externName		\ name of Forth word
variable usesFloats		\ non-zero if FP arguments or return ; SFP006
variable NumArgs		\ number of input arguments
variable NumInts		\ number of arguments on Forth data stack
variable NumFloats		\ number of arguments on Forth Float stack
variable /DataFrame		\ total size of data frame with padding
variable /ForthFrame		\ number of bytes taken from Forth data stack
variable /ForthFPFrame		\ number of bytes taken from Forth float stack
variable NCRN			\ next core register number
variable NSAA			\ next stacked argument address/offset
variable NFRN			\ next floating point register number
variable SVCindex		\ SVC or jump table index (JTI)
variable PriIndex		\ Primary index for double indirect calls
variable SecIndex		\ Secondary index for double indirect calls
variable PriTable		\ holds base address of primary table for double indirect calls
variable PriPointer		\ Holds address of pointer  to primary table
variable DIRvar			\ holds direct address.
variable JTIvar			\ holds target address of variable that
				\ holds the jump table base address.
/argItem buffer: ReturnRes	\ structure for return data
/argList buffer: ArgList	\ argument data during compilation

0 value LastImport		\ address of current/last import structure

: .ArgType	\ n --
  case
    SINTtype of  ." SINT"  endof
    UINTtype of  ." UINT"  endof
    BOOLtype of  ." BOOL"  endof
    FLOATtype of  ." Float"  endof
      ." ???"
  endcase
;

: .ArgData	\ ^ai --
\ Display the argument data
  cr ." Size: " dup ai.size @ dup 0 .r
  0= if
    drop  ."  (void)"
  else
    ." , Type: " dup ai.Type @ .ArgType
    ." , PSP: " dup ai.PSPoff @ 0 .r
    ." , FSP: " dup ai.FSPoff @ 0 .r
    ." , Frame: " dup ai.FrameOff @ 0 .r
    case  dup ai.inReg? @
      0 of  endof
      1 of  dup ai.reg @ ." , Reg: " .  endof
      2 of  dup ai.reg @ ." , Freg: " .  endof
         1 abort" invalid register type"
    endcase
    drop
  endif
;

: ArgList[]	\ u -- addr
\ Return the address of the uth item in argList
  /argItem * argList +  ;

: .ArgList	\ --
\ Display the calculated argument and return data.
  cr ." Arguments"  numArgs @ 0
  ?do  i ArgList[] .ArgData  loop
  cr ." --" cr ." Returns"
  ReturnRes .ArgData
; doNotSin

: CalcSource	\ ^ai --
\ Generate the source Forth data for the given argument.
  dup ai.type @ FLOATtype = if
    /ForthFPFrame @ over ai.FSPoff !	\ where it is on Forth FP stack
    8 /ForthFPframe +!			\ all Forth FPs are 64 bit
    drop  numFloats incr
  else
    /ForthFrame @ over ai.PSPoff !	\ where it is on Forth data stack
    ai.size @ aligned(t) /ForthFrame +!
    numInts incr
  endif
; doNotSin

: CalcForthOff	\ --
\ Calculate the offsets of data items on the Forth input stack.
\ The Forth order is only affected by the choice of L>R or R>L
\ ordering. By default we use L>R.
  numArgs @ if
    L>R? if
      0 numArgs @ 1-
      do  i ArgList[] CalcSource  -1 +loop
    else
      numArgs @ 0
      do  i ArgList[] CalcSource  loop
    endif
  endif
; doNotSin

: AlignFrame	\ n --
\ Force frame alignment of a item of the given size to an N byte
\ boundary where N is a power of two.
  /DataFrame @				\ offset w.r.t frame after call
  over 1- +  swap negate and		\ -- dp'
  /DataFrame !
;

: ?AlignFrame	\ size --
\ Force frame alignment of a item of the given size to an N byte
\ boundary where N is a power of two. Ignore void items.
  ?dup if
    aligned(t) AlignInFrame?
    if  AlignFrame  else  drop  endif
  endif
;

: frameDest	\ ^ai --
\ Generate the destination frame data for the given argument.
  dup ai.size @ ?AlignFrame
  /DataFrame @ over ai.FrameOff !
  ai.size @ aligned(t) /DataFrame +!
 ;

((
struct /argItem	\ -- len
  int ai.Size		\ argument size in bytes, 0=void, 1=byte/char, 2=short, 4=int ...
  int ai.type		\ 0=uint, 1=sint, 2=bool, 3=float
  int ai.FrameOff	\ offset on operating system frame to data
  int ai.Reg		\ register number to be used
  int ai.inReg?		\ true if in register (may be FPreg). 0=no, 1=Reg, 2=FPreg
  int ai.PSPoff		\ offset on Forth parameter stack as if TOS uncached
end-struct
))
: regDest	\ ^ai --
\ We assume that data is 4 or 8 bytes.
  dup ai.Size @ 4 > 			\ align register
  if  NCRN @ 1 + -2 and NCRN !  then
  NCRN @ RegisterArgs >=		\ if we have used all registers
  if  frameDest exit  then		\ just put it on the frame
  1 over ai.InReg? !			\ mark as integer register
  NCRN @ over ai.Reg !			\ register number
  dup ai.Size @ aligned(t) 2 rshift NCRN +!  \ number of registers needed
  drop
;

: FPregDest	\ ^ai --
\ We assume that data is 4 or 8 bytes.
  dup ai.Size @ 4 > 			\ align register
  if  NFRN @ 1 + -2 and NFRN !  then	\ align so we can mix floats and doubles
  2 over ai.InReg? !			\ mark as register
  NFRN @ over ai.Reg !			\ register number
  dup ai.Size @ 2 rshift NFRN +!	\ number of registers needed, Sx/Dx
  drop
;

: CalcDest	\ ^ai --
\ Generate the destination frame data for the given argument.
  dup ai.Type @ FLOATtype = if
    FPregDest
  else
    RegisterArgs
    if  regDest  else  frameDest  endif
  endif
;

: CalcFrameOff	\ --
\ Calculate the O/S stack offsets. These are from the C-style
\ argument list, and so are unaffected by R>L, which only affects
\ the source offsets on the Forth stack with respect to the
\ declaration list.
  REQUIRE_NREV if
    numArgs @ 0
    ?do  i ArgList[] CalcDest  loop
  else
    numArgs @ if
      0 numArgs @ 1-
      do  i ArgList[] CalcDest  -1 +loop
    endif
  endif
;

: initOffsets	\ --
\ Initialise the stack and frame offsets required.
  numInts off  numFloats off
  /DataFrame off			\ C data frame
  /ForthFrame off  /ForthFPFrame off	\ Forth frames
  NCRN off  NSAA off  NFRN off
;


\ ******************
\ +S Code generation
\ ******************

\ ========
\ +N Tools
\ ========

: [a		\ -- ; start of assembler sequence
  also asm-access  ;  immediate

: a]            \ -- ; end of assembler sequence
  previous  ;  immediate

: [a]		\ -- ; flush asm sequence
  asm_end asm_start ;			\ flush assembler


\ ==============================
\ +N Prologue, call and epilogue
\ ==============================
\ The process of making the external call is to:
\ 1) Save Forth system registers on the return (R13/SP) stack.
\ 2) If required save the FP state.
\ \ 3) Preserve the Forth TOS (normally cached in TOS) on the
\ \    deep stack so that the Forth stack is all in memory.
\ \ 4) Preserve R13/SP by extending the deep Forth stack.
\ 5) Adjust the return stack (R13/SP) to operating system requirements
\    with space allocated for the function arguments.
\    This may involve aligment to an 8 or 16 byte boundary.
\ 6) Copy the Forth and FP parameters to registers or return
\    stack locations allowing for the calling convention.
\ 7) Call the external function
\ 8) Restore R13/SP for "C" call convention
\ 9) Save the return data and process TOS for void and float returns
\ 10) Restore the float conditions as required
\ 11) Restore the Forth stack
\ 12) Restore Forth registers

((
2 cells constant /PSPextend	\ -- u
\ The number of bytes by which the Forth stack is dropped to
\ extend it. The high cell contains the cached TOS. The bottom
\ cell (offset 0) contains the saved SP.
\ MUST BE at least the size of a LongLong.
/PSPextend cell - constant /PSPtos	\ -- u
\ The offset from the dropped Forth stack to the cell containing
\ TOS.
))
-4 constant /PSPtos	\ -- u
\ The offset from the dropped Forth stack to the cell containing
\ TOS. -4 indicates TOS is in a register

: FrameDrop	\ -- n
\ The amount by which the frame is dropped before alignment.
\ This is a positive number to subtract.
  /DataFrame @ ( FrameAlignment + ) ;

: FrameMask	\ -- mask
\ The mask applied to SP after FrameDrop has been applied.
  FrameAlignment negate  ;

: -ReturnForth	\ n -- n-x
\ Remove the size of data returned on the Forth data stack.
  ReturnRes ai.Size @ aligned(t) -  ;

: #PSPrestore	\ -- n
\ generate code to restore TOS for void/float returns. This is
\ the number of bytes added to restore the Forth frame.
  /ForthFrame @				\ size of Forth stack arguments
  returnRes ai.size @ if
    case  ReturnRes ai.Type @ 		\ Return type
      SINTtype of  -ReturnForth  endof
      UINTtype of  -ReturnForth  endof
      BOOLtype of  -ReturnForth  endof
    endcase
  endif
\  /PSPextend +				\ frame overhead, >= largest return type
;

\ SVCcall    saveState, layCall, restState,
\   -1 SVC   noop       SVC      noop
\   0  JTI   R12R14     JTI      R12R14
\   1  DIR   R12R14     DIR      R12R14

defer saveState,	\ --
\ Lay code to save the target system state before the call.
defer layCall,		\ --
\ Lay code to call a function
defer restoreState,	\ ==
\ Lay code to restore system state before returning data

: saveR9R12R14,	\ --
\ Lay the save state code. Brute force and ignorance coding!
  Cortex-M0/M1? if
    SaveR9?
    if  [a  mov   r4, r9  a] [a]  then
    SaveR12?
    if  [a  mov   r5, r12  a] [a]  then
    SaveR9? SaveR12? and
    if  [a  push  {{ r4, r5, r14 }}  a] [a]  exit  then
    SaveR9? SaveR12? 0= and
    if  [a  push  {{ r4, r14 }}  a] [a]  exit  then
    SaveR9? 0= SaveR12? and
    if  [a  push  {{ r5, r14 }}  a] [a]  exit  then
    SaveR9? 0= SaveR12? 0= and
    if  [a  push  {{ r14 }}  a] [a]  exit  then
  else
    SaveR9? SaveR12? and
    if  [a  stmfd   rsp ! {{ r9 r12 r14 }}  a] [a]  exit  then
    SaveR9? SaveR12? 0= and
    if  [a  stmfd   rsp ! {{ r9 r14 }}  a] [a]  exit  then
    SaveR9? 0= SaveR12? and
    if  [a  stmfd   rsp ! {{ r12 r14 }}  a] [a]  exit  then
    SaveR9? 0= SaveR12? 0= and
    if  [a  str   r14, x[ rsp, # -4 x] !  a] [a]  exit  then
  endif
; doNotSin

: restR9R12R14,	\ --
\ Lay the restore state code. Brute force and ignorance coding!
  Cortex-M0/M1? if
    SaveR9? SaveR12? and if
      [a  pop   {{ r4, r5 }}
          mov   r9, r4
          mov   r12, r5  a] [a]
    then
    SaveR9? SaveR12? 0= and if
      [a  pop   {{ r4 }}
          mov   r9, r4  a] [a]
    then
    SaveR9? 0= SaveR12? and if
      [a  pop   {{ r5 }}
          mov   r12, r5  a] [a]
    then
    SaveR9? 0= SaveR12? 0= and
    if  then
    [a  pop   {{ r4 }}
        mov   r14, r4  a] [a]
  else
    SaveR9? SaveR12? and
    if  [a  ldmfd   rsp ! {{ r9 r12 r14 }}  a] [a]  exit  then
    SaveR9? SaveR12? 0= and
    if  [a  ldmfd   rsp ! {{ r9 r14 }}  a] [a]  exit  then
    SaveR9? 0= SaveR12? and
    if  [a  ldmfd   rsp ! {{ r12 r14 }}  a] [a]  exit  then
    SaveR9? 0= SaveR12? 0= and
    if  [a  ldr   r14, x[ rsp x], # 4  a] [a]  exit  then
  then
;

: saveR9,	\ --
\ Lay the save state code
  Cortex-M0/M1? if
    [a  mov   r4, r9
        push  {{ r4 }}
    a] [a]
  else
    [a  str  r9, x[ rsp, # -4 x] !  a] [a]
  then
; doNotSin

: restR9,	\ --
\ Lay the restore state code
  Cortex-M0/M1? if
    [a  pop   {{ r4 }}
        mov   r9, r4
    a] [a]
  else
    [a  ldr  r9, x[ rsp x], # 4  a] [a]
  then
;

: SVC#		\ -- u
\ SVC number
  SVCindex @  ;

: JTI		\ -- addr
\ address holding table base address
  JTIvar @  ;

: JTIoffset	\ -- u
\ Offset into jump table
  SVCindex @ 2 lshift  ;

: laySVCcall,	\ --
\ Lay the code to call the external function.
  [a  svc   # SVC#  a] [a]
;

: ?ForceTbit	\ --
\ If the flag is set, force the T bit in R4 to be set.
\ cr ." Into ?ForceTbit" key? drop
  ForceTbits? if
    Cortex-M0/M1? if
      [a  mov .s  r5, # 1		\ force Thumb bit
          orr .s  r4, r5
      a] [a]
    else
      [a  orr .s  r4, r4, # 1 a] [a]	\ force Thumb bit
    then
  then
\ cr ." Out of ?ForceTbit" key? drop
;

: layJTIcall,	\ --
\ Lay the code to call the external function.
\ Call external C/PASCAL proc. According to AAPCS R4..R7 are
\ preserved.
  rR4 JTI primLoadLit			\ base address of table held here
  [a
    ldr     r5, x[ r4 x]		\ fetch base address
    ldr     r4, x[ r5, # JTIoffset x]	\ fetch function address
  a] [a] ?ForceTbit [a
    blx     r4				\ call it
  a] [a]
;

: layDIRcall,	\ --
\ Lay the code to call the external function directly.
\ Call external C/PASCAL proc. According to AAPCS R4..R7 are
\ preserved.
  rR4 SVC# primLoadLit			\ SVC call number\direct
  ?ForceTbit
  [a  blx     r4  a] [a]		\ call it
;

: PriPtr	\ -- addr
\ Return address of pointer to the primary table
  PriPointer @  ;
: PriAddr	\ -- addr
\ Return address of Primary table
  PriTable @ ;
: PriOffset	\ -- offset
\ index into primary table
  PriIndex @ 2 lshift  ;
: SecOffset	\ -- offset
\ index into secondary table
  SecIndex @ 2 lshift  ;

: layDICcall,	\ --
\ Lay the code to call the external function, and clean up.
\ Call external C/PASCAL proc. According to AAPCS R4..R7 are
\ preserved.
  rR4 PriAddr primLoadLit		\ base address of table held here
  [a
    ldr     r5, x[ r4, # PriOffset x]	\ fetch address of secondary table
    ldr     r4, x[ r5, # Secoffset x]	\ fetch function address
  a] [a]
  ?ForceTbit
  [a  blx   r4  a] [a]			\ call it
;

: layPDICcall,	\ --
\ Lay the code to call the external function, and clean up.
\ Call external C/PASCAL proc. According to AAPCS R4..R7 are
\ preserved.
  rR4 PriPtr primLoadLit		\ base address of primary table held here
  [a
    ldr     r4, x[ r4, # 0 x]		\ get address of primary table
    ldr     r5, x[ r4, # PriOffset x]	\ fetch address of secondary table
    ldr     r4, x[ r5, # Secoffset x]	\ fetch function address
  a] [a]
  ?ForceTbit
  [a  blx   r4  a] [a]			\ call it
;

: preReturnData	\ --
\ Save TOS and adjust stack if data stack grows down.
\ cr ." Into preReturnData, #PSPrestore = " #PSPrestore . key? drop
  #PSPrestore 0< if
    #PSPrestore abs >r
    r@ 4 =  numints @ 0=  and if
      Cortex-M0/M1? if
        [a  sub .s  psp, psp, # 4
            str     tos, x[ psp, # 0 x]
        a] [a]
      else
        [a  str   tos, x[ psp, # -4 x] !  a] [a]
      then
    else
      [a  sub .s   psp, psp, # r@  a] [a]
      numInts @ 0= if			\ if TOS not consumed
        [a  str   tos, x[ psp, # r@ 4 - x]  a] [a]
      endif
    endif
    r> drop
  endif
\ cr ." Out of preReturnData" key? drop
;

: epilogue,	\ --
\ Restore the Forth and O/S stacks.
\ Forth data stack restoration
\ cr ." Into epilogue," key? drop
  #PSPrestore 0> if
    #PSPrestore >r
    [a add .s   psp, psp, # r> a] [a]	\ adjust Forth PSP stack: 0-4+8=4
  endif
\ cr ." Out of epilogue," key? drop
; doNotSin

\ SVCcall    saveState, layCall, restState,
\   -1 SVC   noop       SVC      noop
\   0  JTI   R12R14     JTI      R12R14
\   1  DIR   R12R14     DIR      R12R14

: setSVC	\ --
\ set up for an SVC call. Note that R0..R3, R12 are saved by the
\ interrupt/exception handler.
  saveR9? if
    ['] saveR9, is saveState,
    ['] restR9, is restoreState,
  else
    ['] noop is saveState,
    ['] noop is restoreState,
  endif
  ['] laySVCcall, is layCall,
;

: setCallSave/Rest	\ --
\ set Save and Restore actions for save
  ['] saveR9R12R14, is saveState,
  ['] restR9R12R14, is restoreState,
;

: setJTI	\ --
\ set up for an JTI call.
  setCallSave/Rest
  ['] layJTIcall, is layCall,
;

: setDIR	\ --
\ set up for a direct call.
  setCallSave/Rest
  ['] layDIRcall, is layCall,
;

: setDIC	\ --
\ set up for a double indirect call.
  setCallSave/Rest
  ['] layDICcall, is layCall,
;

: setPDIC	\ --
\ set up for a double indirect call.
  setCallSave/Rest
  ['] layPDICcall, is layCall,
;


\ ============================
\ +N Copy Forth stack to frame
\ ============================

: offsets	\ ^ai -- srcoff destoff
\ Extract the base offsets from the structure. The source offset
\ is assumed to be on the PSP stack and is adjusted to be based
\ from the saved TOS.
  dup ai.PSPoff @ /PSPtos +  swap ai.FrameOff @
;

: (intRegX)	\ reg --
  case
    0 of  [a R0 a]  endof
    1 of  [a R1 a]  endof
    2 of  [a R2 a]  endof
    3 of  [a R3 a]  endof
    4 of  [a R4 a]  endof
    5 of  [a R5 a]  endof
    6 of  [a R6 a]  endof
    7 of  [a R7 a]  endof
      err_RegParm throw
  endcase
;

: intRegX	\ ^ai --
  ai.Reg @  ( ." reg = R" dup . cr )  (intRegX)
;

: intRegX+1	\ ^ai --
  ai.Reg @ 1+ ( ." reg = R" dup . cr )  (intRegX)
;

: FPSregX	\ ^ai --
  case  ai.Reg @  ( ." reg = R" dup . cr )
     0 of  [a S0 a]  endof
     1 of  [a S1 a]  endof
     2 of  [a S2 a]  endof
     3 of  [a S3 a]  endof
     4 of  [a S4 a]  endof
     5 of  [a S5 a]  endof
     6 of  [a S6 a]  endof
     7 of  [a S7 a]  endof
     8 of  [a S8 a]  endof
     9 of  [a S9 a]  endof
    10 of  [a S10 a]  endof
    11 of  [a S11 a]  endof
    12 of  [a S12 a]  endof
    13 of  [a S13 a]  endof
    14 of  [a S14 a]  endof
    15 of  [a S15 a]  endof
      err_RegParm throw
  endcase
; doNotSin

: FPDregX		\ ^ai --
  case  ai.Reg @  ( ." reg = R" dup . cr )
     0 of  [a D0 a]  endof
     2 of  [a D1 a]  endof
     4 of  [a D2 a]  endof
     6 of  [a D3 a]  endof
     8 of  [a D4 a]  endof
    10 of  [a D5 a]  endof
    12 of  [a D6 a]  endof
    14 of  [a D7 a]  endof
      err_RegParm throw
  endcase
; doNotSin

: psp[src]	\ ^ai -- offset
  ai.PSPoff @ /PSPtos +
\ ." srcoff = " dup .dword cr
  >r [a x[ psp, # r> x] a]
;

: rsp[dest]	\ ^ai -- offset
  ai.FrameOff @
\ ." dstoff = " dup .dword cr
  >r [a x[ sp, # r> x] a]
;

: psp[src=reg?]	\ ^ai -- flag
\ Return true if the source is in a register.
  ai.PSPoff @ /PSPtos + 0<
;

: fromTOStoReg	\ ^ai --
\ load register from TOS.
  >r [a mov a]  r> intRegX  [a tos a]  [a]
;

: fromTOStoDeep	\ ^ai --
\ load deep stack from TOS.
  >r  [a  str    tos, a]  r> rsp[dest] [a]
;

: regLoadSB	\ ^ai --
  dup psp[src=reg?] if
    fromTOStoReg
  else
    >r  [a ldrb a]  r@ intRegX  r> psp[src]  [a]
  endif
;

: regLoadUB	\ ^ai --
  dup psp[src=reg?] if
    fromTOStoReg
  else
    >r  [a ldrb a]  r@ intRegX  r> psp[src]  [a]
  endif
;

: regLoadSH	\ ^ai --
  dup psp[src=reg?] if
    fromTOStoReg
  else
    >r  [a ldrsh a]  r@ intRegX  r> psp[src]  [a]
  endif
;

: regLoadUH	\ ^ai --
  dup psp[src=reg?] if
    fromTOStoReg
  else
    >r  [a ldrh a]  r@ intRegX  r> psp[src]  [a]
  endif
;

: regLoad	\ ^ai --
  dup psp[src=reg?] if
    fromTOStoReg
  else
    >r  [a ldr a]  r@ intRegX  r> psp[src]  [a]
  endif
;

: argCopySB	\ ^ai -- ; copy from data stack to return stack through r5
  dup psp[src=reg?]
  if  fromTOStoDeep exit  endif
  >r
  [a  ldrsb  r5, a]  r@ psp[src] [a]
  [a  str    r5, a]  r> rsp[dest] [a]
;

: argCopyUB	\ ^ai -- ; copy from data stack to return stack through r5
  dup psp[src=reg?]
  if  fromTOStoDeep exit  endif
  >r
  [a  ldrb   r5, a]  r@ psp[src] [a]
  [a  str    r5, a]  r> rsp[dest] [a]
;

: argCopySH	\ ^ai -- ; copy from data stack to return stack through r5
  dup psp[src=reg?]
  if  fromTOStoDeep exit  endif
  >r
  [a  ldrsh  r5, a]  r@ psp[src] [a]
  [a  str    r5, a]  r> rsp[dest] [a]
;

: argCopyUH	\ ^ai -- ; copy from data stack to return stack through r5
  dup psp[src=reg?]
  if  fromTOStoDeep exit  endif
  >r
  [a  ldrh   r5, a]  r@ psp[src] [a]
  [a  str    r5, a]  r> rsp[dest] [a]
;

: argCopy	\ ^ai -- ; copy from data stack to return stack through r5
  dup psp[src=reg?]
  if  fromTOStoDeep exit  endif
  >r
  [a  ldr    r5, a]  r@ psp[src] [a]
  [a  str    r5, a]  r> rsp[dest] [a]
;

: CopyS8	\ ^ai --
\ Copy a signed 8 bit item from data stack to return stack
  dup ai.InReg? @
  if  regLoadSB  else  argCopySB  endif
;

: CopyS16	\ ^ai --
\ Copy a signed 16 bit item from srcoff [PSP] to destoff [RSP]
  dup ai.InReg? @
  if  regLoadSH  else  argCopySH  endif
;

: CopyU8	\ ^ai --
\ Copy an unsigned 8 bit item from srcoff [PSP] to destoff [RSP]
  dup ai.InReg? @
  if  regLoadUB  else  argCopyUB  endif
;

: CopyU16	\ ^ai --
\ Copy an unsigned 16 bit item from srcoff [PSP] to destoff [RSP]
  dup ai.InReg? @
  if  regLoadUH  else  argCopyUH  endif
;

: CopyX32	\ ^ai --
\ Copy a cell from srcoff [PSP] to destoff [RSP]
  dup ai.InReg? @
  if  regLoad  else  argCopy  endif
;

: CopyX64-F	\ ^ai --
\ Copy a 64 bit double integer in Forth format from srcoff [PSP]
\ to destoff [RSP] or reg
  dup >r ai.InReg? @ if		\ dest is register
    r@ psp[src=reg?] if		\ high portion to high reg
      [a mov a]  r@ intRegX+1  [a tos a]  [a]
    else
      [a ldr a]  r@ intRegX+1  r@ psp[src]  [a]
    endif
    [a ldr a]  r@ intRegX  r@ psp[src] 4+ [a]
  else		\ -- ^ai ^ai
    [a  ldr    r5, a]  r@ psp[src]  [a]
    [a  str    r5, a]  r@ rsp[dest] 4+  [a]
    [a  ldr    r5, a]  r@ psp[src] 4+  [a]
    [a  str    r5, a]  r@ rsp[dest]  [a]
  endif
  r> drop
;

: CopySINT	\ ^ai --
\ Compile code to copy TOS to the frame as an signed INT32/64.
\ with any required promotion.
  case  dup ai.size @
    1 of  CopyS8  endof
    2 of  CopyS16  endof
    4 of  CopyX32  endof
    8 of  CopyX64-F  endof
     err_CGsize throw
  endcase
;

: CopyUINT	\ ^ai --
\ Compile code to copy TOS to the frame as a UINT32/64.
\ with any required promotion.
  case  dup ai.size @
    1 of  CopyU8  endof
    2 of  CopyU16  endof
    4 of  CopyX32  endof
    8 of  CopyX64-F  endof
     err_CGsize throw
  endcase
;

: CopyBOOL	\ ^ai --
\ Compile code to copy TOS to the frame as a BOOL.
\ with any required promotion/demotion.
  CopyU8
;

: CopyF32	\ ^ai --
\ Compile code to copy a float from FTOS or srcoff[FSP] to Sx
  dup ai.FSPoff @ 0= if
    dup >r
    [a vcvt .f32<f64 a]  r> FPSregX  [a d8 a] [a]
  else
    dup >r
    [a  vldr .f64  d9, x[ r8, # a] r> ai.FSPoff @ 8 - [a x] a]  [a]
    dup >r
    [a vcvt .f32<f64 a]  r> FPSregX  [a d9 a] [a]
  endif
  drop
;

: CopyF64	\ ^ai --
\ Compile code to copy a double from FTOS or srcoff[FSP] to Dx
  dup ai.FSPoff @ 0= if
    dup >r
    [a vmov .f64 a]  r> FPDregX  [a d8 a] [a]
  else
    dup >r
    [a vldr .f64 a]  r@ FPDregX
    [a x[ r8, # a]  r> ai.FSPoff @ 8 - [a x] a]  [a]
\    [a  vldr .f64  d9, [ r8, # a] r@ ai.FSPoff @ 8 - [a ] a]  [a]
\    [a vmov .f64 a]  r> FPDregX  [a d9 a] [a]
  endif
  drop
;

: CopyFLOAT	\ ^ai --
\ Compile code to copy a float or double
\ Copy a double from from srcoff [FSP] to Sx/Dx
  FPABI 2 <> err_noFPABI ?throw
  FPsystem 2 <> err_noFPsystem ?throw
  case  dup ai.size @
    4 of  CopyF32  endof
    8 of  CopyF64  endof
       err_CGsize throw
  endcase
;

: CopyArg	\ ^ai --
\ Compile code to copy an item from Forth to the calling frame
\ with any required promotion.
  case  dup ai.type @
    UINTtype of  CopyUINT  endof
    SINTtype of  CopySINT  endof
    BOOLtype of  CopyBOOL  endof
    FLOATtype of  CopyFloat  endof
      err_ImpType throw
  endcase
;

: CopyFrame	\ --
\ Copy the arguments from the Forth stack to the O/S stack with
\ any required promotion/demotion.
  L>R? REQUIRE_NREV xor 0= if
    numArgs @ if
      0 numArgs @ 1-
      do  i ArgList[] CopyArg  -1 +loop
    endif
  else
    numArgs @ 0
    ?do  i ArgList[] CopyArg  loop
  endif
;


\ ================
\ +N Return result
\ ================

: restoreTOS	\ --
\ For void and float return values, TOS must be restored.
  /PSPtos /ForthFrame @ +		\ problem for -ve offset and CM0/1
\ if -ve, void input. For void foo( void ), nothing to do.
  dup 0 >= if
    >r  [a  ldr  tos  x[ psp, # r> x] a] [a]
  else
    drop
  then
;

: #FSPrestore	\ -- n
\ The number of bytes to adjust the FP stack by for non-float returns
  /ForthFPframe @			\ number consumed as inputs
;

: restoreFTOS	\ --
\ For void and non-float return values, FTOS may need to be restored.
  #FSPrestore if
    [a
       vldr  d8, x[ r8, # #FSPrestore x]
       add   r8, r8, # #FSPrestore
    a] [a]
  then
;

: retLongLongF	\ --
\ Return a long long in Forth format.
  [a  mov   tos, r1  a] [a]			\ most significant  -> TOS
  [a  str   r0, x[ psp, x# #PSPrestore x]  a] [a]	\ least significant -> NOS ; SFP035
;

: ReturnSINT	\ --
\ Return a signed int.
  case  ReturnRes ai.size @
    1 of  [a  mov  tos, r0  a] [a]  endof
    2 of  [a  mov  tos, r0  a] [a]  endof
    4 of  [a  mov  tos, r0  a] [a]  endof
    8 of  retLongLongF  endof
        err_RetSize throw
  endcase
;

: ReturnUINT	\ --
\ Return an unsigned int.
  case  ReturnRes ai.size @
    1 of  [a  mov  tos, r0  a] [a]  endof
    2 of  [a  mov  tos, r0  a] [a]  endof
    4 of  [a  mov  tos, r0  a] [a]  endof
    8 of  retLongLongF  endof
        err_RetSize throw
  endcase
;

: ReturnBOOL	\ --
\ Return a BOOL as a single byte.
  [a  mov  tos, r0  a] [a]  ;

: ?saveFTOS,	\ --
\ Save old FTOS if no floats were inputs
  /ForthFPFrame @ 0= if
     [a
       sub        r8, r8, # #8
       vstr .f64  d8, x[ r8, # 0 x]
     a] [a]
  then
;

: ?updateFSP	\ --
\ update FSP to remove FP inputs
  /ForthFPFrame @ if
    /ForthFPFrame @ 8 - ?dup if
      >r  [a  add   r8, r8, # r>  a] [a]
    then
  then
;

: returnF32	\ --
\ Return a 32 bit float.
  ?saveFTOS,
  [a  vcvt .F64<F32  d8, s0  a] [a]
  ?updateFSP
;

: returnF64	\ --
\ Return a 64 bit float.
  ?saveFTOS,
  [a  vmov .f64  d8, d0  a] [a]
  ?updateFSP
;

: returnFLOAT	\ -- ; return float/double
\ For a float return, we must still restore the integer TOS correctly.
  FPABI 2 <> err_noFPABI ?throw		\ must be VFP2/3
  FPsystem 2 <> err_noFPsystem ?throw	\ must be hardFPABI
  case  returnRes ai.size @
    4 of  returnF32  endof
    8 of  returnF64  endof
      err_RetSize throw
  endcase
  restoreTOS
;

: returnData	\ -- ; process return value
\ Do any necessary demotion, or lose return result for VOID.
\ This happens before the epilogue code so that the saved frame
\ is available.
  preReturnData				\ allow for TOS being saved
  returnRes ai.size @ if		\ non-void
    case  ReturnRes ai.Type @ 		\ Return type
      SINTtype of  ReturnSINT restoreFTOS  endof
      UINTtype of  ReturnUINT restoreFTOS  endof
      BOOLtype of  ReturnBOOL restoreFTOS  endof
      FLOATtype of  returnFLOAT  endof
        err_RetType throw
    endcase
  else					\ void
    RestoreTOS restoreFTOS
  endif
; doNotSin


\ ====================
\ +N Import generation
\ ====================

: comp-extern	\ xt --
\ Compile a child of EXTERN:.
  doCall,  discard-sinline
;

: set-comp-extern	\ xt --
\ Set COMP-EXTERN as the compiler of the given XT.
  >code-len  ['] comp-extern over !  cell+ off
;

variable FirstLine#	\ -- addr ; SFP022
\ Holds line number of the first line

: setFirstLine	\ --
  line# @ FirstLine# !
;

: MakeColon(t)	\ --
  line# @ >r  FirstLine# @ line# !	\ use first line number for header
  ExternName count $:(t)		\ make header and start definition
  r> line# !
;

: MakeNormal	\ --
\ Make the word that calls a normal imported function.
\ cr ." MN1"
  MakeColon(t)				\ save xt we have just made ; SFP022
\ cr ." MN2"
  saveState,
\ cr ." MN3"
    copyFrame
\ cr ." MN4"
      layCall,
\ cr ." MN5"
  restoreState,
\ cr ." MN6"
    returnData
\ cr ." MN7"
      epilogue,
\ cr ." MN8"
 ;(t)					\ finish definition
\ cr ." MN9"
; doNotSin

: MakeVarCall	\ --
\ Make the word that calls a varargs imported function.
  err_varargs throw
; doNotSin

: MakeImport	\ --
\ Make the word that calls the imported function.
  varargs?
  if  MakeVarCall  else  MakeNormal  endif
; doNotSin


: GenerateImport        \ --
\ Sort out where data must go and generate a Forth word to do it.
\ Calculate the stack and frame offsets
  initOffsets CalcForthOff CalcFrameOff
\ Build the colon definition that interprets the function
  MakeImport
;
+short-branches
\ ...SFP004


\ =================
\ Argument handling
\ =================

0 value argSIZE		\ in bytes
0 value argTYPE		\ 0 = integer or bool
0 value argDEFSIGN	\ 0 = unsigned, 1 = signed, for ints < 4 bytes
-1 value argREQSIGN	\ 0 = unsigned, 1 = signed, -1 no request made
0 value argISPOINTER	\ 0 = no, 1 = this is a pointer

: next-item	\ c-addr u char -- 'c-addr 'u c-addr1 u1
\ Return the remaining string and the leftmost item delimited by
\ char. Both strings have leading and trailing spaces removed.
  >r 2dup r> scan			\ ca1 u1 ca2 u2 --
  2swap swap >r				\ ca2 u2 u1 -- ; R: ca1 --
  over - >r				\ ca2 u1 -- ; R: ca1 u1 --
  1 /string -white  2r> -white
;

: checkPtr	\ caddr len -- caddr' len' ; SFP013 SFP023
\ The string is a name (no spaces). Check if either or both the
\ leading and trailing characters are '*', and if so mark the
\ argument as a pointer.
  begin
    dup 1 > while			\ -- caddr len
    over c@ [char] * = while
      1 to argISPOINTER  1 /string
  repeat then
  begin
    dup 1 > while
    2dup + 1- c@ [char] * = while	\ check trailing '*'
      1 to argISPOINTER  1-
  repeat then
;

-short-branches
: parse-arg     \ c-addr u --
   4 to argSIZE
   0 to argTYPE				\ default to int
   0 to argDEFSIGN
  -1 to argREQSIGN
   0 to argISPOINTER

  begin
    -leading dup 0>
  while
    \ c-addr u --
    bl next-item checkPtr		\ SFP013
    ['] types(t) voc>wid search-wordlist
    \ -- c-addr-rem u-rem xt flag | c-addr-rem u-rem 0
    dup 0=
    if  ['] noop swap  then		\ Use NOOP if not found
    \ -- c-addr-rem u-rem xt flag
    0= 2 pick 0> and err_impType ?throw	\ bail if token not found and wasn't last!
    execute
  repeat
  2drop

  argREQSIGN -1 =			\ if no sign set,
  if  argDEFSIGN to argREQSIGN  then	\  use default
  argISPOINTER if                       \ if pointer then override to use
    0 to argTYPE			\ treat as int
    0 to argREQSIGN                     \ unsigned cell as data type
    4 to argSIZE			\ use CELL instead?
  then
  argTYPE FLOATtype =			\ SFP006
  if  usesFloats on  endif
;
+short-branches

: delimiter?    \ char -- flag
\ Return true if the character is a type delimiter
  dup [char] * =			\ pointer
  if  drop  true exit  then
  dup bl =				\ space
  if  drop  true exit  then
  dup 9 =				\ tab ; SFP017
  if  drop  true exit  then
  drop false
;

: splitRet&Name	\ c-addr u -- c-addr u' c-addr1 u1
\ Get the return type and function name from the input stream.
\ Split the string at the last delimiter, so that caddr/u'
\ contains the return type info and caddr1/u1 contains the
\ function name.

\ parse the return type and function name
  [char] ( word count -trailing		\ -- c-addr u ; return type, call conv, name )

\ find last delimiter
  2dup + 1 chars -      \ -- c-addr u last
  begin
    dup c@ delimiter? 0=
  while
    1-
  repeat
  1+			\ -- c-addr u c-addr1
\ calculate strings
  dup 3 pick -          \ -- c-addr u c-addr1 u'
  swap rot		\ -- c-addr u' c-addr1 u
  2 pick -		\ -- c-addr u' c-addr1 u-u'
;

: setArgData	\ ^ai --
  argSIZE over ai.size !
  argTYPE 0= if				\ default is int/bool
    argREQSIGN
    if  SINTtype  else  UINTtype  endif
    swap ai.type !
  else
    argTYPE swap ai.type !
  endif
;

: ?refill	\ --
\ Perform REFILL and throw on error.
  refill 0= #-411 ?throw  ;

: parse*/	\ caddr len -- caddr' len --
\ Parse out a C comment of the form "/* ... */", returning the
\ following string. May parse multiple lines.
  begin
    s" */" anssearch 0=			\ look for terminating "*/"
   while
    2drop  ?refill source
  repeat
  2 /string -white			\ step over "/", strip white space
;

: toEOL		\ --
\ Set the text input ponter to the end of the line.
  #tib @ >in !  ;

: stripLeadingComment	\ caddr len -- caddr' len'
\ Remove leading C style comments from the string. The first
\ character is not white space.
  dup 2 <
  if  exit  endif
  over s" //" s=			\ if string starts with //
  if  toEOL  drop 0  exit  endif	\ set length to zero
  over s" /*" s=			\ if string starts with /*
  if  parse*/  toEOL  endif
;

: stripTrailingComment	\ caddr len -- caddr' len'
\ Remove trailing C style comments from the string. Any leading
\ comment has already been removed. All trailing comments are
\ assumed to extend to the end of the line.
  dup 2 <
  if  exit  endif
  2dup s" //" anssearch
  if  nip - -white  toEOL  exit  endif
  2drop  2dup s" /*" anssearch
  if  nip - -white  toEOL  exit  endif
  2drop
;

-short-branches
: ExtractArgs	\ caddr u --
\ The input string is the return type information. Then parse the
\ rest of the line after the leading '(' character.

  -white parse-arg			\ parse the return type
  ReturnRes setArgData

  0 numArgs !
  begin
    [char] ) parse		        \ -- caddr len
    'tib @ >in @ 1- 0 max + c@ >r	\ terminator
    begin
      -leading dup 0>
    while				\ c-addr u --
      stripLeadingComment dup 0>
    while
      [char] , next-item
      stripTrailingComment dup if
        parse-arg			\ next argument to trailing , or )
        numArgs @ ArgList[] setArgData	\ save data in argument array
        1 numArgs +!
      else
        2drop
      endif
    repeat then
    2drop
    r> [char] ) <>
   while
    ?refill
  repeat
  numArgs @ 1 = if			\ special case, #args=1 and argsize=0 ; SFP007
    arglist @ 0= if			\ indicates void list
      numArgs off
    endif
  endif
  0 word drop				\ discard rest of line
;
+short-branches

variable <ShowExtData>

[-short-branches
: ?ShowExtData	\ --
  <ShowExtData> @ if
    cr cr ExternName count type .ArgList
    cr ." #Ints = " numints @ .  ."  #Floats = " numfloats @ .
  endif
;
short-branches]

: isSingleName	\ caddr len -- ; SFP020
\ Set as name of Forth word and DLL function.
  2dup importName place			\ set name of DLL function
  ExternName place			\ set name of Forth word
;

: SingleName	\ "name" -- ; SFP020
\ G Get a name as the name of the Forth word and the DLL function.
  parse-name isSingleName
;

: parseIndex	\ "text" --
\ Parse up to trailing ), cross-interpret the text, and save the
\ result in SVCindex.
  [char] ) parse evaluate(t) SVCindex !
;

: stripComma	\ caddr len --
  bounds ?do
    i c@ [char] , =
    if  bl i c!  endif
  loop
;

: parseDIC	\ "text" --
\ Parse up to trailing ), cross-interpret the text, and save the
\ results in PriIndex and SecIndex.
  [char] ) parse 2dup stripComma
  evaluate(t) SecIndex ! PriIndex !
;

: ExternDef	\ "text" --
\ Parse and process the definition after the SVC( ... ) or JTI( ... )
\ portions.
  setFirstLine				\ SFP022
  splitRet&Name	isSingleName     	\ -- c-addr u ; leaves return type info
  usesFloats off  setDefExt		\ set default declaraction type ; SFP006
  ExtractArgs GenerateImport
  ?ShowExtData
;

\ =============
\ +N Public API
\ =============

also cc/i

: SVC(		\ "text" -- ; )
\ *G Declare an external API reference of the form:
\ *C   SVC( n ) int foo( int a, char *b, char c );
\ *P The Forth word marshalls the parameters and calls SVC/SWI n.
\ ** The Forth word has the same name as the function in the
\ ** library, but the Forth word name is *\b{not} case-sensitive.
\ ** The length of the function's name may not be longer than a
\ ** Forth word name.
  parseIndex  setSVC  ExternDef
;

: holdsJumpTable	\ addr(t) --
\ *G The base address of the jump table is stored at the given
\ ** target address. *\fo{HoldsJumpTable} must be used before
\ ** any *\fo{JTI(} definition is made.
\ *E variable JT     \ -- addr
\ ** JT holdsJumpTable
  JTIvar !  ;

: JumpTable		\ -- addr
\ *G Returns the target address that holds the jump table
\ ** base address
  JTIvar @  ;

: JTI(		\ "text" -- ; )
\ *G Declare an external API reference of the form:
\ *C   JTI( n ) int foo( int a, char *b, char c );
\ *P The Forth word marshalls the parameters and calls entry n
\ ** (0, 1, 2, ...) in the jump table
\ ** The Forth word can have the same name as the function in the
\ ** library, but the Forth word name is *\b{not} case-sensitive.
\ ** The length of the function's name may not be longer than a
\ ** Forth word name.
  parseIndex  setJTI  ExternDef
;

: DIR(		\ "text" -- ; )
\ *G Declare an external API reference of the form:
\ *C   DIR( addr ) int foo( int a, char *b, char c );
\ *P The Forth word marshalls the paramters and calls the
\ ** subroutine at target address *\i{addr}.
\ ** The Forth word can have the same name as the function in the
\ ** library, but the Forth word name is *\b{not} case-sensitive.
\ ** The length of the function's name may not be longer than a
\ ** Forth word name.
  parseIndex  setDIR  ExternDef
;

: setPriTable	\ addr(t) --
\ *G Set the base address of the Primary ROM table used for
\ ** calling ROM functions. For Luminary/TI CPUs, this will
\ ** probably be:
\ *C  $0100:0010 setPriTable
  PriTable !  ;

: DIC(		\ "text" -- ; )
\ *G Declare an external API reference of the form:
\ *C   DIC( pri, sec ) int foo( int a, char *b, char c );
\ *P The comma betwee *\i{pri} and *\i{sec} is optional.
\ ** This form is used when calling ROM routines in devices
\ ** from TI, NXP and others. In these the primary table entries
\ ** hold the address of another table. The secondary index
\ ** is then used to fetch the address of the actual routine
\ ** to be called. The Forth word marshalls the parameters
\ ** and calls the routine.
\ ** The Forth word can have the same name as the function in the
\ ** library, but the Forth word name is *\b{not} case-sensitive.
\ ** The length of the function's name may not be longer than a
\ ** Forth word name.
  parseDIC  setDIC  ExternDef
;

: setPriPointer	\ addr(t) --
\ *G Set the base address of the Primary ROM table used for
\ ** calling ROM functions. For Luminary/TI CPUs, this will
\ ** probably be:
\ *C  $0100:0010 setPriTable
  PriPointer !  ;

: PDIC(		\ "text" -- ; )
\ *G Declare an external API reference of the form:
\ *C   PDIC( pri, sec ) int foo( int a, char *b, char c );
\ *P The comma betwee *\i{pri} and *\i{sec} is optional.
\ ** This form is used when calling ROM routines in devices
\ ** from TI, NXP and others. In these the primary table entries
\ ** hold the address of another table. The secondary index
\ ** is then used to fetch the address of the actual routine
\ ** to be called. The Forth word marshalls the parameters
\ ** and calls the routine.
\ ** The Forth word can have the same name as the function in the
\ ** library, but the Forth word name is *\b{not} case-sensitive.
\ ** The length of the function's name may not be longer than a
\ ** Forth word name.
  parseDIC  setPDIC  ExternDef
;

: +ForceTbits	\ --
\ *G Force function addresses to have the T bit (bit 0) set
\ ** by the code that performs the call. This is the
\ ** default.
  1 to ForceTbits?  ;

: -ForceTbits	\ --
\ *G Do not compile the code that forces function addresses to have
\ ** the T bit (bit 0) set by the code that performs the call.
\ ** You will have to dump a table to find this out for your system.
  0 to ForceTbits?  ;

: +SaveR9	\ --
\ *G Compile additional code to preserve R9 (platform register) across calls.
\ ** Some silicon vendors do not know what various compilers do,
\ ** so the safe thing is to use *\fo{+SaveR9}.
  1 to SaveR9?  ;
: -SaveR9	\ --
\ *G Do not compile additional code to preserve R9 across calls.
  0 to SaveR9?  ;

: +SaveR12	\ --
\ *G Compile additional code to preserve R12 (Intra-Procedure call
\ ** scratch register) across calls.
\ ** Some silicon vendors do not know what various compilers do,
\ ** so the safe thing is to use *\fo{+SaveR12}.
  1 to SaveR12?  ;
: -SaveR12	\ --
\ *G Do not compile additional code to preserve R12 across calls.
  0 to SaveR12?  ;

: +DebugExterns	\ --
\ *G Turn on display of debug information.
  <ShowExtData> on  ;
: -DebugExterns	\ --
\ *G Turn off display of debug information.
  <ShowExtData> off  ;

previous definitions


\ ******************************
\ *S Pre-Defined parameter types
\ ******************************
\ *P The types known by the system are all found in the vocabulary
\ ** *\fo{TYPES(t)}. You can add new ones at will. Each *\fo{TYPE}
\ ** definition modifies one or more of the following *\fo{VALUE}s.                        )
\ *D argSIZE      Size in bytes of data type.
\ *D argDEFSIGN   Default sign of data type if no override is supplied.
\ *D argREQSIGN   Sign OverRide. This and the previous use 0 = unsigned
\ **              and 1 = signed.
\ *D argISPOINTER 1 if type is a pointer, 0 otherwise

\ *P Each *\fo{TYPES(t)} definition can either set these flags
\ ** directly or can be made up of existing types.


\ ======================
\ *N Calling conventions
\ ======================

also types(t) definitions

: "C"           \ --
\ *G Set Calling convention to "C" standard. Arguments are
\ ** reversed, and the caller cleans up the stack. This is
\ ** the default.
  0 to CalleeCleaned?  TRUE to Require_NRev
;

: "PASCAL"      \ --
\ *G Set the calling convention to the "PASCAL" standard as used
\ ** by Pascal compilers. Arguments are *\b{not} reversed, and the
\ ** called routine cleans up the stack.
  1 to CalleeCleaned?  FALSE to Require_NRev
;

: L>R		\ --
\ *G By default, arguments are assumed to be on the Forth stack
\ ** with the top item matching the rightmost argument in the
\ ** declaration so that the Forth parameter order matches that
\ ** in the C-style declaration.
\ ** *\fo{L>R} confirms this and is the default.
  TRUE to L>R?
;

: R>L		\ --
\ *G By default, arguments are assumed to be on the Forth stack
\ ** with the top item matching the rightmost argument in the
\ ** declaration so that the Forth parameter order matches that
\ ** in the C-style declaration.
\ ** *\fo{R>L} reverses this.
  FALSE to L>R?
;

previous definitions


\ ==============
\ *N Basic Types
\ ==============

: (setDefExt)	\ --
\ set up the defaults for SVC and JTI calls.
  4 to RegisterArgs  4 to FrameAlignment  1 to AlignInFrame?
  TRUE to REQUIRE_NREV  0 to CalleeCleaned?
  TRUE to L>R?
  0 to ObjPtr?  0 to StructRet?  0 to varargs?
;
assign (setDefExt) to-do setDefExt

also types(t) definitions

: unsigned      \ --
\ *G Request current parameter as being unsigned.
  0 to argREQSIGN  ;

: signed        \ --
\ *G Request current parameter as being signed.
  1 to argREQSIGN  ;

: int           \ --
\ *G Declare parameter as integer. This is a signed 32 bit quantity
\ ** unless preceeded by *\fo{unsigned}.
  4 to argSIZE  1 to argDEFSIGN  ;

: char          \ --
\ *G Declare parameter as character. This is a signed 8 bit quantity
\ ** unless preceeded by *\fo{unsigned}.
  1 to argSIZE  1 to argDEFSIGN  ;

: void          \ --
\ *G Declare parameter as void. A *\fo{VOID} parameter has no
\ ** size. It is used to declare an empty parameter list, a null
\ ** return type or is combined with *\fo{*} to indicate a generic
\ ** pointer.
  0 to argSIZE  0 to argDEFSIGN  ;

: *             \ --
\ *G Mark current parameter as a pointer.
  1 to argISPOINTER  ;
: **		\ --
\ *G Mark current parameter as a pointer.
  1 to argISPOINTER  ;
: ***		\ --
\ *G Mark current parameter as a pointer.
  1 to argISPOINTER  ;

: const ;	\ --
\ *G Marks next item as *\b{constant} in C terminology. Ignored
\ ** by Forth.

: int32		\ --
\ *G A 32bit signed quantity.
  4 to argSIZE  1 to argDEFSIGN  ;

: int16		\ --
\ *G A 16 bit signed quantity.
   2 to argSIZE  1 to argDEFSIGN  ;

: int8		\ --
\ *G An 8 bit signed quantity.
   1 to argSIZE  1 to argDEFSIGN  ;

: uint32	\ --
\ *G 32bit unsigned quantity.
  unsigned int  ;

: uint32_t	\ --
\ *G 32bit unsigned quantity.
  unsigned int  ;

: uint16	\ --
\ *G 16bit unsigned quantity.
  unsigned int  2 to argSIZE  ;

: uint16_t	\ --
\ *G 16bit unsigned quantity.
  unsigned int  2 to argSIZE  ;

: uint8		\ --
\ *G 8bit unsigned quantity.
  1 to argSIZE  0 to argDEFSIGN  ;

: uint8_t		\ --
\ *G 8bit unsigned quantity.
  1 to argSIZE  0 to argDEFSIGN  ;

: LongLong	\ --
\ *G A 64 bit signed or unsigned integer. At run-time, the argument
\ ** is taken from the Forth data stack as a normal Forth double
\ ** with the top item on the top of the data stack.
  8 to argSIZE  1 to argDEFSIGN  ;

: LONG          int  ;
\ *G A 32 bit signed quantity.

: SHORT		\ --
\ *G For most compilers a *\b{short} is a 16 bit signed item,
\ ** unless preceded by *\fo{unsigned}.
  2 to argSIZE  1 to argDEFSIGN  ;

: BYTE		\ --
\ *G An 8 bit unsigned quantity.
  1 to argSIZE  0 to argDEFSIGN  ;

: float		\ --
\ *G 32 bit float.
  4 to argSIZE  0 to argDEFSIGN  FLOATtype to argTYPE  ;

: double	\ --
\ *G 64 bit float.
  8 to argSIZE  0 to argDEFSIGN  FLOATtype to argTYPE  ;

: bool1		\ --
\ *G One byte boolean.
  1 to argSIZE  0 to argDEFSIGN  BOOLtype to argTYPE  ;
: bool4		\ --
\ *G Four byte boolean.
  int  ;

: ...		\ --
\ *G The parameter list is of unknown size. This is an indicator
\ ** for a C varargs call. Run-time support for this varies between
\ ** CPUs and operating system implementations of VFX Forth. Test, test,
\ ** test.
  1 to varargs?
;

previous definitions


\ *********
\ Test code
\ *********

((
SVC( 14 ) void * GetDirFnTable( void );
SVC( 24 ) int open( const char * pathname, int flags, int mode );
SVC( 36 ) int foop( int a, int b, int c, int d, int e, int f );

$10000000 holdsjumptable
JTI( 25 ) int open2( const char * pathname, int flags, int mode );

DIR( $0100:0201 ) int poop( char * addr, int a , int b );

-forceTbits
$0100:0010 setPriTable
DIC( 4, 0 ) void ROM_GPIOPinWrite(
  uint32 ui32Port, uint8 ui8Pins, uint8 ui8Val
);
))

previous definitions


\ ======
\ *> ###
\ ======

decimal

