\ JLinkOps - operations using the Segger JLink

((
Copyright (c) 2013, 2015, 2016, 2022
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20220304 SFP003 Allowed for failed open.
20161108 SFP002 Do not close JLINK until we have to.
20151113 SFP001 Added AUTO device detection.
))

only forth definitions
decimal

\ ========
\ *! jlink
\ *T JLink Flash and debug interface
\ ========
\ *P The compiler can be integrated with a Segger JLink to provide
\ ** Flash programming and debugging facilities. Instructions for
\ ** downloading the required software will have been provided with
\ ** your JLink. The Segger website is at *\f{http://www.segger.com}.
\ ** Make sure that the installed Segger shared library is at least
\ ** v4.78e or v4.78.5.

\ *P These facilities are available for Windows, Mac OS X and Linux.

\ *****************************
\ *S Installing Segger software
\ *****************************
\ ==========
\ *N Windows
\ ==========
\ *P Download the latest Segger tools; you need at least v4.78e or v4.78.5.
\ ** Place the file *\i{JLinkARM.dll} in a directory in the search
\ ** path. The directory containing the compiler is always in the
\ ** search path.

\ ===========
\ *N Mac OS X
\ ===========
\ *P Download the latest Segger tools; you need at least v4.78e
\ ** or v4.78.5. We need the shared library *\i{libjlinkarm.4.xx.y.dylib}
\ ** and the link *\i{libjlinkarm.4.dylib}. Check that the
\ ** following file exists:
\ *C   /Applications/Segger/JLink/libjlinkarm.dylib
\ *P If it is present, you are all done. If it does not exist,
\ ** either the installation has failed or Segger have changed the
\ ** rules. Note that this file is a symbolic link.

\ *P For the 64 bit compiler, put the shared libraries *\i{libjlinkarm.*.dylib}
\ ** in *\i{/usr/local/lib}. If the cross-compiler cannot find the Segger
\ ** library, make a symbolic link *\i{libjlinkarm.dylib}
\ ** to it in *\i{/usr/local/lib}.

\ *P You will have to repeat this incantation whenever you update
\ ** the Segger tools.

\ ========
\ *N Linux
\ ========
\ *P Download the latest Segger tools; you need at least v4.78e or v4.78.5.
\ ** After installation or update, you should have a directory
\ ** *\i{/opt/SEGGER/JLink} containing the files *\i{libjlinkarm.so.4}
\ ** and *\i{libjlinkarm.so.4.xx.y}. Copy these to *\i{/usr/lib} or
\ ** a system directory which is searched for 32 bit libraries.
\ ** For example:
\ *C   sudo  cp /opt/SEGGER/JLink/libjlinkarm.so.* /usr/lib/
\ *P You will have to repeat this incantation whenever you update
\ ** the Segger tools.


\ *************************
\ *S Controlling the J-Link
\ *************************
\ *P JLink commands can be executed at any time, either in the
\ ** control file or interactively (at the keyboard). However,
\ ** the memory image and output files are not finalised until
\ ** the word *\fo{FINIS} has been executed. Actions that must be
\ ** performed after *\fo{FINIS} can be specified with *\fo{AFTERWARDS},
\ ** which saves the rest of the line as text to be interpreted
\ ** at the end of *\fo{FINIS}. *\fo{AFTERWARDS} can be used several
\ ** times, and each line is interpreted in order, first encountered
\ ** being interpreted first. The example below sets up the JLink
\ ** and then causes the code sections to be downloaded to Flash.
\ *E   jlink STM32F407ZG 0 Project.jlink \ device, speed, project file
\ **   afterwards  download-sections   \ download image

\ *P The list of device names can be found in the file
\ ** *\i{<jlink>\ETC\JFlash\MCU.csv} supplied by Segger. The
\ ** speed (0=automatic) and project file (Project.jlink) are
\ ** defaults and rarely need to be changed.

\ ==================
\ *N Umbilical Forth
\ ==================
\ *P At the end of the control file, you can place something like:
\ *E ...
\ **
\ ** jlink LPC1114/302 0 Project.jlink  \ device, speed, project file
\ ** useSWD                  \ use SWD rather than the default JTAG
\ ** PSUART0 set-sjl-link    \ select Umbilical link through J-Link
\ **
\ ** umbilical-forth

\ *P The word *\fo{jlink} sets the device (important), the speed
\ ** (0=autodetect to maximum) and project file. By default, the
\ ** J-Link operates in JTAG mode. For systems that require SWD
\ ** operation *\fo{useSWD} selects SWD operation. We can fake
\ ** a serial line for the Umbilical Forth link using *\fo{set-sjl-link}
\ ** which takes the address of the four-byte data used for
\ ** talking to the J-Link.

\ *P The file *\i{Cortex/Drivers/serPSUART.fth}
\ ** contains an example link driver for the target side. It's rather
\ ** slow because of the USB connection to the J-Link, but it works.

\ *P If you have a system with a real serial port, you can still use
\ ** it but you will have to replace the line
\ *C PSUART0 set-sjl-link    \ select Umbilical link through J-Link
\ *P with serial configuration lines such as
\ *C c" COM4:" console-speed serial
\ *C c" dtr=off rts=off" set-control


\ *******
\ +S Data
\ *******

256 buffer: z$DeviceName	\ -- zaddr
  s" STM32F407ZG" z$DeviceName zplace
MAX_PATH buffer: z$ImageFile	\ -- zaddr
  s" C:\buildkit.dev\software\ROM\Cortex\Hardware\STM32F4\STM32-E407.img" z$ImageFile zplace
MAX_PATH buffer: z$JLinkFile	\ -- zaddr
  s" C:\buildkit.dev\software\ROM\Cortex\Hardware\STM32F4\Project.jlink" z$JLinkFile zplace
1000 value JlinkKhz		\ -- khz
0 value TIF_type		\ -- type ; 0=JTAG, 1=SWD
\ Target Interface type.
$0800:0000 value FlashStartAddr	\ -- taddr
128 constant #MAXregs		\ -- n ; max registers allowed
#MAXregs cells constant /MAXregs
/MAXregs buffer: RegList	\ -- taddr
\ Holds a table of register indices.
/MAXregs buffer: RegData	\ -- taddr
\ Holds a table of register data.
0 value #CurrRegs		\ -- u
\ Number of supperted registers in the target CPU

0 value CoreType	\ -- x
\ Core family type returned by J-Link.
0 value DevIndex	\ -- x
\ Index into JTAG chain, usually 0.

1024 constant /ErrJL	\ -- len
\ +G Size of buffer for error string.
/ErrJL buffer: z$ErrJL	\ -- addr
\ +G Buffer for error string.

also c-c
: Cortex?	TargetISA 0<>  ;
: ARM32?	TargetISA 0=  ;
previous


\ ********
\ +S Tools
\ ********

[undefined] Num/Constant [if]
: Num/Constant	\ caddr -- n
\ +G Treat the token as a number returning the value, or as a Forth
\ +* word which returns a single item on execution.
  find if				\ -- caddr xt
    execute
  else
    number? 1 <> abort" Not a single integer."
  endif
;
[then]


\ **********************
\ +S Set up and shutdown
\ **********************

[defined] Target_386_Windows [if]
0 constant MB_OK
16 constant MB_ICONSTOP

: ErrJLink	\ z$ --
\ +G Display error string.
  hwndMain if
    hwndMain swap z" J-Link error" MB_OK MB_ICONSTOP or MessageBox drop
  else
   cr ." JLink Error: " zcount type
  endif
;
[else]
: ErrJLink	\ z$ --
\ +G Display error string.
  cr ." JLink Error: " zcount type cr
;
[then]

: ExecJLink	\ z$ -- ior ; 0=success
\ +G Pass the string to the J-Link ExecCommand. The result is
\ +* discarded. If there is an error, an error message is
\ +* displayed.
  0 z$ErrJL c!
  z$ErrJL /ErrJL JLINKARM_ExecCommand drop
  z$ErrJL c@ dup
  if  z$ErrJL ErrJLink  endif
;

: ProjectJLink	\ -- ior ; 0=success
\ +G Set the project file path.
  {: | temp[ 512 ] -- :}
  s" ProjectFile = " temp[ zplace
  z$JLinkFile zcount temp[ zAppend
  temp[ ExecJLink
;

: deviceJLink	\ -- ior ; 0=success
\ +G Set the device to be operated on, e.g. STM32F407ZG. The
\ +* device name is taken from the buffer *\fo{z$DeviceName}.
\ +* A full list can be found in the file *\i{MCU.csv}.
  {: | temp[ 512 ] -- :}
  z$DeviceName zcount s" auto" istr= 0= if
    s" Device = " temp[ zplace
    z$DeviceName zcount temp[ zAppend
    temp[ ExecJLink
  else
    0
  then
;

: speedJLink	\ kHhz --
\ Set the JLink speed in kHz. A value of zero selects automatic
\ detection of JLink speed. A value of 65535 selects adaptive
\ clocking.
  JLINKARM_SetSpeed
;

: ifaceJLink	\ --
\ +G Use one of the interface numbers to select how the JLink
\ +* connects to the target. 0=JTAG, 1=SWD.
  TIF_type JLINKARM_TIF_Select drop
;

0 value JLinkOpen?	\ -- flag
\ +G True if JLINK has been opened.

: closeJLink	\ --
\ +G Close the J-Link connection
  JLINKARM_Close  0 -> JLinkOpen?
;

: .Core/Device	\ --
  {: | temp[ 512 ] info[ 3 cells ] -- :}
  JLINKARM_CORE_GetFound -> CoreType
  temp[ off  CoreType temp[ 511 JLINKARM_Core2CoreName space
  cr ." Connected to " temp[ zcount type ."  core, device is "
  0 info[ JLINKARM_JTAG_GetDeviceInfo if
    ." Unknown"
  else
    info[ @ zcount type
  endif
;

: openJLink	\ -- ior
\ +G Open the J-Link connection, returning zero on success.
  0 -> JLinkOpen?
  JLinkARMlib 0=
  if  z" JLinkARM library not loaded" ErrJLink  1 exit  then
  JLINKARM_Open ?dup
  if  dup ErrJLink  closeJLink exit  endif
  ifaceJLink				\ select JTAG or SWD
  ProjectJLink ?dup if
    z" Cannot set project file" ErrJLink  closeJLink exit
  endif
  DeviceJLink ?dup if
    z" Cannot select device" ErrJLink  closeJLink exit
  endif
  JLINKARM_Connect ?dup if
    z" Cannot connect to target" ErrJLink  closeJLink  exit
  endif
\  .Core/Device
\  1 JLINKARM_SetInitRegsOnReset drop
\  20 JLINKARM_SetResetDelay		\ 20 ms wait after reset
  JLinkKHz speedJLink
  RegList #MAXregs JLINKARM_GetRegisterList -> #CurrRegs
  -1 -> JLinkOpen?
  0
;

: ?OpenJLink	\ --
\ +G Open the J-Link connection if not already done.
  JLinkOpen? 0= if  openJLink drop  endif
;


\ ********************
\ +S Download to Flash
\ ********************

: LoadImageFile	\ -- caddr len
\ +G Load the image file into memory. a *\fo{THROW} occurs on
\ +* error.
  z$ImageFile zcount inhale
;

: (ProgImageMem)	\ caddr len taddr --
\ +G Program the memory image *\i{caddr/len} to Flash at *\i{taddr}.
\ +* There are no messages except on error.
  0 JLINKARM_BeginDownload		\ Indicates start of flash download
  -rot swap JLINKARM_WriteMem 0<
  if  z" Flash write failed" ErrJLink  endif
  JLINKARM_EndDownload drop
;

: ProgImageMem	\ caddr len taddr --
\ +G Program the memory image *\i{caddr/len} to Flash at *\i{taddr}.
cr ." Downloading ..." key? drop
  JLINKARM_Reset drop
  (ProgImageMem)
  JLINKARM_Reset drop
."  done" key? drop
;

: (DownldJLink)	\ --
  LoadImageFile FlashStartAddr ProgImageMem
;

: DownloadJLink	\ --
\ +S Download the image file predefined by *\fo{ImageFile} to the
\ +* target
  ?OpenJLink  JLinkOpen? if		\ SFP003
    ['] (DownldJLink) catch drop
    Burp				\ release file memory
  then
;

also c-c also sections.voc
: section>JLink	{: sect | buff used -- :}
\ +G Download section memory image.
  ?OpenJLink  JLinkOpen? if
    sect savesize(ms) dup -> used		\ find size of section contents
    get-img-buff -> buff			\ put contents of section in buffer
    sect MS-low @  buff  used  sect read-gen(ms)
    buff used sect MS-low @ ProgImageMem	\ program flash
    buff free-img-buff
  then
;
assign section>JLink to-do download-section
previous previous


\ ********************
\ +S Register handling
\ ********************

: RegIndex[]	\ u -- index
\ Get the index for the uth register.
  cells RegList + @
;

: RegData[]	\ u -- data32
\ Get the data for the uth register (already read from CPU).
  cells RegData + @
;

: setRegData[]	\ data32 u --
\ Set the data for the uth register.
  cells RegData + !
;

: .RegNames	\ --
\ Display a list of supported registers
  #CurrRegs 0 ?do
    cr i RegIndex[] dup .
    8 >pos
    JLINKARM_GetRegisterName zcount type
  loop
;

19 constant #baseCMregs	\ -- u
\ R0..R15, XPSR, MSP, PSP

: getCMregs	\ --
\ Read the base registers.
  RegList RegData NULL #CurrRegs JLINKARM_ReadRegs drop
;

: setCMregs	\ --
\ Write the base registers.
  RegList RegData NULL #CurrRegs JLINKARM_WriteRegs drop
;

: .item		\ addr -- addr+4
\ Display a cell item at addr and increment addr.
  dup @ .dword space  cell +  ;

: .item+4	\ addr -- addr+4
\ Display a cell item +4  at addr and increment addr.
  dup @ 4 + .dword space  cell +  ;

: .items	\ addr n -- addr+4n
\ Display n items at addr and increment addr.
  0 ?do  .item  loop  ;

: .regName	\ u --
\ Display the name of the uth register in the index table.
  RegIndex[] JLINKARM_GetRegisterName zcount type
;

: (.CMregs)	\ u --
\ Display the given number of registers
  16 umax >r  RegData
  cr ." R0  = "  4 .items
  cr ." R4  = "  4 .items
  cr ." R8  = "  4 .items
  cr ." R12 = "  3 .items .item+4
  cr r> 16 ?do			\ -- reglist
    i .regName ."  = " .item  2 spaces
    out @ 66 > if  cr  endif
  loop
  drop
;

: .baseCMregs	\ --
  19 (.CMregs)
;

also c-c
: .nextIns	\ --
\ Display next instruction. Note that the JLink returns PC/R15
\ pointing to the instruction, and removes the effect of the
\ pipeline.
  Cortex? if				\ Thumb1/2
    15 RegData[] ( 4 - ) 2 xDISASM/al
  else					\ ARM32
    15 RegData[] ( 8 - ) 4 xDISASM/al
  endif
;

: BLins?	\ -- flag
\ Return true if the current instruction is a BL instruction.
  15 RegData[]  Cortex? if
    dup w@c(t) $F800 and $F000 =
    swap 2+ w@c(t) $D000 and $D000 = and
  else
    l@c(t) $FF00:0000 and $EB00:0000 =
  endif
;
previous

: .savedCPUstate	\ --
\ Display the CPU registers and the next instruction.
  .baseCMregs .nextIns
;


\ *************
\ +S Primitives
\ *************

: doStop		\ --
\ +G Stop the CPU.
  JLINKARM_Halt
  if  z" Can't halt" ErrJLink exit  endif
  getCMregs
;

: StopCPU	\ -- flag
\ +G Return true if the CPU was previously running. If the CPU is
\ +* running, halt it.
  JLINKARM_IsHalted 0= dup
  if  doStop  endif
;

: ESCkey?	\ -- flag
\ +G Return true if the <ESC> key has been pressed. Key presses
\ +* are consumed.
  key? dup
  if  drop key 27 =  then
;

: doWaitStop	\ --
\ +G Wait for CPU to stop or the <ESC> is pressed.
  begin
    JLINKARM_IsHalted if  getCMregs exit  then
    ESCkey? if  exit  then
    10 ms
  again
;

: go            \ -- ; target carries on
\ +G Carry on execution of the target code.
  setCMregs  JLINKARM_Go
;

: [F		\ -- ; R: -- flag ; flag=0 if previously frozen
\ +G The code between *\fo{[F} and *\fo{F]} is executed with the
\ +* target frozen. Because state information is preserved on the
\ +* return stack, these words must be used in pairs inside the
\ +* same definition.
  r>  StopCPU >r  >r
;

: F]		\ -- ; R: flag -- ; Unfreeze target if flag is true
\ +G The code between *\fo{[F} and *\fo{F]} is executed with the
\ +* target frozen. Because state information is preserved on the
\ +* return stack, these words must be used in pairs inside the
\ +* same definition.
  r>
  r> if  go  endif
  >r
;

: step		\ --
\ +G Execute a single step. There is no display.
  setCMregs  JLINKARM_Step
  if  z" Can't step" ErrJLink  endif
  getCMregs
;

: (stepOver)	\ --
\ +G If the current instruction is an unconditional BL instruction,
\ +* step without display until it returns. Otherwise perform a
\ +* single step. There is no display.
  BLins? if
    15 RegData[] 4 +  begin		\ return address
      step  15 RegData[] over =  halt? or
    until
    drop
  else
    step
  endif
;

: setBP		\ addr -- bp# ; 0=failure
\ +G Set a breakpoint at the given address and return the
\ +* breakpoint number. Note that the JLink uses the address of
\ +* the instruction, not the pipelined value (addr+4/8) in the
\ +* PC.
  JLINKARM_BP_IMP_ANY JLINKARM_SetBPEx
;

: clrBP		\ bp# --
\ +G Clear the given breakpoint.
  JLINKARM_ClrBPEx
  if  z" Cannot clear breakpoint" ErrJLink  endif
;

: clrAllBPs	\ --
\ +G Clear all breakpoints.
  JLINKARM_BP_HANDLE_ALL JLINKARM_ClrBPEx
  if  z" Cannot clear breakpoints" ErrJLink  endif
;


\ ************************
\ +S Cross compiler set up
\ ************************

defer set-JLink	\ --
\ +G Set the JLink drivers to be the default for the Umbilical
\ +* link.

also c-c also sections.voc
: JLINK		\ -- ; jlink <device> <kHz> <projfile>
\ +G Set the JLink to use the given device, connection speed
\ +* and the given project file. The list of devices can be
\ +* found in the file *\i{<jlink>\ETC\JFlash\MCU.csv} supplied by Segger.
\ +* The connection speed is specified in kHz, where 0 specifies
\ +* automatic speed detection, and 65535 specifies adaptive
\ +* clocking. For example:
\ +C jlink STM32F407ZG 1000 Project.jlink
\ +P selects an STM32F407ZG device, 1000 kHz and the project file
\ +* is *\i{Project.jlink} in the compiler's working directory.
  parse-name z$DeviceName zplace
  bl word Num/Constant -> JlinkKhz
  getPathSpec z$JLinkFile zplace
  assign section>JLink to-do download-section
  set-JLink
;
previous previous


\ ******************************
\ *S Initialisation and download
\ ******************************

only forth  also c-c  also cc/i

: JLINK		\ -- ; jlink <device> <speed> <projfile>
\ *G Set the JLink to use the given device, connection speed
\ ** and the given project file. The list of devices can be
\ ** found in the file *\i{<jlink>\ETC\JFlash\MCU.csv} supplied by Segger.
\ ** The connection speed is specified in kHz, where 0 specifies
\ ** automatic speed detection, and 65535 specifies adaptive
\ ** clocking. For example:
\ *C jlink STM32F407ZG 1000 Project.jlink
\ *P selects an STM32F407ZG device, 1000 kHz and the project file
\ ** is *\i{Project.jlink} in the compiler's working directory.
  JLink
;

: speedJLink	\ kHhz --
\ *G Set the JLink speed in kHz. A value of zero selects automatic
\ ** detection of JLink speed. A value of 65535 selects adaptive
\ ** clocking. This word is useful when a CPU runs at a low speed
\ ** after reset, e.g. at 32 kHz until the clock unit has been
\ ** set up.
  speedJLink
;

: useJTAG	\ --
\ *G The JLink connects to the target by JTAG. This is the default.
  0 -> TIF_type
;

: useSWD	\ --
\ *G The JLink connects to the target by SWD.
  1 -> TIF_type
;

(( for DocGen
: download-sections	\ --
\ *G After *\fo{JLINK} above has been used, *\fo{download-sections}
\ ** is set to use the JLink device to program the target.
\ ** *\fo{Download-sections} closes the JLink connection after
\ ** programming.
))

: InitJLink	\ --
\ *G Initialise the J-Link connection.
  ?OPenJLink				\ SFP002
;

: TermJLink	\ --
\ *G Terminate the J-Link connection.
  JLinkOpen? if  CloseJLink  then
;

: ProgramImage	\ taddr "<filename>" -- ; 0 ProgramImage File.bin
\ *G Program the file at the given target address.
  GetPathSpec rot ProgImageMem
;

: setImageFile	\ "<filename>" -- ; ImageFile <filename>
\ *G Set the predefined image file used by commands such as
\ ** *\fo{DownloadJLink} below.
  GetPathSpec z$ImageFile zplace
;

: setFlashStart	\ taddr -- ; $0800:0000 setFlashStart
\ *G Set the predefined Flash start address used by commands such
\ ** as *\fo{DownloadJLink} below.
  -> FlashStartAddr
;

: DownloadJLink	\ --
\ *G Download the image file predefined by *\fo{ImageFile} to the
\ ** target.
  DownloadJLink
;

previous previous definitions


\ *********************
\ *S Debug and stepping
\ *********************
\ *P The word *\fo{InitJLink} must have been used before any of
\ ** these words have been run.

only forth  also c-c  also cc/i

: reg@		\ reg# -- x
\ *G Return the data in the saved register (0..15). The CPU must
\ ** have been halted.
  RegData[]
;

: reg!		\ x reg# --
\ *G Apply the data to the saved register (0..15).
  setRegData[]
;

: .regs		\ --
\ *G Display the working registers.
  .savedCPUstate
;

: stop		\ --
\ *G Stop the CPU.
  doStop
;

: StopCPU	\ -- flag
\ *G Return true if the CPU was previously running. If the CPU is
\ ** running, halt it.
  StopCPU
;

: from		\ addr --
\ *G Set the target PC to the given address
  #15 setRegData[]
;

: go            \ -- ; target carries on
\ *G Carry on execution of the target code.
  go
;

: halt		\ --
\ *G Halt the CPU and dump registers.
  doStop .savedCPUstate
;

: step		\ --
\ *G Execute a single step. There is no display.
  step
;

: ss		\ --
\ *G Execute a single step and then display registers
  step .savedCPUstate
;

: stepOver	\ --
\ *G If the current instruction is an unconditional BL instruction,
\ ** step without display until it returns. Otherwise perform a
\ ** single step. There is no display until the end.
  (stepOver) .savedCPUstate
;

: so		\ --
\ *G Step past the current instruction and then display registers.
  (stepOver) .savedCPUstate
;

: resetOnly	\ --
\ *G Reset the CPU with no checks.
  JLINKARM_Reset drop
;

: resetGo	\ --
\ *G Reset the target and set it to free run if the CPU had been
\ ** halted.
  JLINKARM_IsHalted
  JLINKARM_Reset drop
  if  JLINKARM_Go  endif
;

: halted?	\ -- flag
\ *G Return true if the CPU is halted.
  JLINKARM_IsHalted
;

: running?	\ -- flag
\ *G Return true if the CPU is running.
  JLINKARM_IsHalted 0= 1 and
;

previous previous definitions


\ **************
\ *S Breakpoints
\ **************

only forth  also c-c  also cc/i

: setBP		\ addr -- bp# ; 0=failure
\ *G Set a breakpoint at the given address and return the
\ ** breakpoint number.
  setBP
;

: clrBP		\ bp# --
\ *G Clear the given breakpoint.
  clrBP
;

: clrAllBPs	\ --
\ *G Clear all breakpoints.
  clrAllBPs
;

: waitHalt	\ --
\ *G Wait until the CPU halts, e.g. for a breakpoint, or until
\ ** the <ESC> key is pressed.
  doWaitStop  JLINKARM_IsHalted
  if  .savedCPUstate  then
;

previous previous definitions


\ ****************
\ *S Memory driver
\ ****************
\ *P Unless otherwise set, the cross compiler defaults to reading
\ ** and writing target memory images in host memory. When a JLink
\ ** or other hardware provides access to real target memory, the
\ ** memory drivers can be selected to read and write the real
\ ** target memory.

also c-c also sections.voc

: jlink-c@(ms)	\ addr(t) section -- b
  {: | temp[ cell ] -- :}
  drop
  [F  1 temp[ JLINKARM_ReadMem drop  temp[ c@  F]
;

: jlink-c!(ms)	\ b addr(t) section --
  {: | temp[ cell ] -- :}
  rot temp[ c!  cdata-type? if
    [F  temp[ 1 rot (ProgImageMem)  F]
  else
    [F  1 temp[ JLINKARM_WriteMem drop  F]
  then
;

: jlink-w@(ms)	\ addr(t) section -- w
  {: | temp[ cell ] -- :}
  drop
  [F  2 temp[ JLINKARM_ReadMem drop  temp[ w@  F]
;

: jlink-w!(ms)	\ w addr(t) section --
  {: | temp[ cell ] -- :}
  rot temp[ w!  cdata-type? if
    [F  temp[ 2 rot (ProgImageMem)  F]
  else
    [F  2 temp[ JLINKARM_WriteMem drop  F]
  then
;

: jlink-l@(ms)	\ addr(t) section -- l
  {: | temp[ cell ] -- :}
  drop
  [F  4 temp[ JLINKARM_ReadMem drop  temp[ @  F]
;

: jlink-l!(ms)	\ l addr(t) section --
  {: | temp[ cell ] -- :}
  rot temp[ !  cdata-type? if
    [F  temp[ 4 rot (ProgImageMem)  F]
  else
    [F  4 temp[ JLINKARM_WriteMem drop  F]
  then
;

: read-jlink(ms)	{: addr(t) addr(h) len sect -- :}
  [F  addr(t) len addr(h) JLINKARM_ReadMem drop  F]
;

: write-jlink(ms)	{: addr(h) addr(t) len sect -- :}
  sect cdata-type? if
    [F  addr(h) len addr(t) (ProgImageMem)  F]
  else
    [F  addr(t) len addr(h) JLINKARM_WriteMem drop  F]
  then
;

: open-jlink(ms)	\ section -- ; open the link
  ?OpenJLink  
  JlinkOpen? 0= swap MS-handle !	\ SFP003
;

: close-jlink(ms)	\ section -- ; close the link
  MS-handle on
;

create MSIO-jlink	\ -- addr ; vector for I/O via link
  ' open-jlink(ms) ,
  ' close-jlink(ms) ,
  ' save-gen(ms) ,
  ' load-gen(ms) ,
  ' read-gen(ms) ,
  ' write-gen(ms) ,
  ' org-mem(ms) ,
  ' here-mem(ms) ,
  ' allot-mem(ms) ,
  ' jlink-c@(ms) ,
  ' jlink-w@(ms) ,
  ' jlink-l@(ms) ,
  ' no64support ,
  ' jlink-c!(ms) ,
  ' jlink-w!(ms) ,
  ' jlink-l!(ms) ,
  ' no64support ,

: all-via-jlink	\ -- ; switch xDATA sections to use JLink
  ?OpenJLink  MSIO-jlink section-chain use-mem-table
;

: (set-JLink)	\ -- ; select J-LInk as the Link driver.
  assign all-via-jlink to-do xdata-via-link
;
assign (set-JLink) to-do set-JLink

: all-via-default	\ -- ; switch xDATA sections to use default
  MSIO-mem section-chain use-mem-table
;

: cdata-via-default	\ -- ; switch CDATA sections to use default
  MSIO-mem section-chain use-cdata-table
;

0 value ImageStart

: [image	\ --
\ Start compiling to the image.
  chere(t) -> ImageStart  cdata-via-default
;

: image]	\ --
\ finish compiling in the target and copy the data to Flash
  chere(t) ImageStart <> if
    download-sections
  endif
  xdata-via-link
;

previous previous

only forth  also c-c  also cc/i

: all-via-jlink		\ --
\ *G Switch all memory sections to use the JLink to access target
\ ** memory.
  all-via-jlink
;

: all-via-default	\ --
\ *G Switch all memory sections to use the default memory image
\ ** driver.
  all-via-default
;

: [image	\ --
\ *G Start compiling or other actions that affect Flash.
\ ** The compilation is performed to a memory image, and the
\ ** result is written to Flash when *\fo{image]} is executed.
  chere(t) -> ImageStart  cdata-via-default
;

: image]	\ --
\ *G Finish compiling to memory, update the Flash and restore
\ ** the memory drivers.
  chere(t) ImageStart <>
  if  download-cdata  endif
  all-via-jlink
;

previous previous definitions


\ ****************************
\ *S J-Link pseudo serial port
\ ****************************
\ *P We use four bytes of target memory as a pretend UART (PSUART). This
\ ** allows us to run debug communications and Umbilical Forth
\ ** through the J-Link using no resources except the four bytes
\ ** of RAM.

0 value debug-sjl?

only forth  also c-c definitions  also cc/i
: +sjl-debug	( -- )		-1 to debug-sjl?  ;
: -sjl-debug	( -- )		0 to debug-sjl?  ;
: sjl-debug?	( -- flag )	debug-sjl?  ;
previous previous definitions

0 value serJL-RAM	\ -- addr
\ Holds the address of the four byte block in target RAM.

struct /serJL-RAM
  byte sjr.status-h>t	\ set by host when data is ready
  			\ cleared by target after data has been read
  byte sjr.data-h>t	\ written by host
  byte sjr.status-t>h	\ set by target when data is ready
  			\ cleared by host after data has been read
  byte sjr.data-t>h	\ written by target
end-struct

also c-c also sections.voc
: sjl-link-open	\ -- ; open the link if not already done
  ?OpenJLink
;

: sjl-link-close	\ -- ; close the link if not already done
;

: sjl-query-byte        \ -- t/f ; Like KEY?
  serJL-RAM sjr.status-t>h 0 jlink-c@(ms)
;

: sjl-get-byte		\ -- char
  serJL-RAM sjr.data-t>h 0 jlink-c@(ms)
  0 serJL-RAM sjr.status-t>h dup addr>section jlink-c!(ms)
;

: sjl-check-abort	\ -- ; check if abort pressed.
  key? if
    case  key
      2 of  1 abort"  ^B - Comms break"  endof
      3 of  1 abort"  ^C - Comms break"  endof
    endcase
  endif
;

: sjl-wait-byte		\ -- b ; like KEY
  begin
    sjl-query-byte 0=
   while
    sjl-check-abort  1 ms
  repeat
  sjl-get-byte  debug-sjl?
  if  dup ." [" .byte ." ]"  endif
;

: sjl-send-byte		\ b -- ; like EMIT
  begin
    serJL-RAM sjr.status-h>t 0 jlink-c@(ms)
   while
    sjl-check-abort  1 ms
  repeat
  dup serJL-RAM sjr.data-h>t dup addr>section jlink-c!(ms)
  -1 serJL-RAM sjr.status-h>t dup addr>section jlink-c!(ms)
  debug-sjl?
  if  ." <" .byte ." >"  else  drop  then
;

: sjl-link-reset	\ -- ; reset target across link
\  JLINKARM_Reset drop
\  JLINKARM_Go
;

: sjl-link-flush	\ -- ; flush any pending input characters	SFP001
  begin  sjl-query-byte while  sjl-wait-byte drop  repeat
;

: .sjl-link-state	\ -- ; show serial link state
  ." Target link at " serJL-RAM .dword
;
previous previous

also c-c
: set-sjl-link	\ addr(t) -- ; select J-Link PSUART
  to serJL-RAM
  assign sjl-link-open to-do link-open
  assign sjl-link-close to-do link-close
  assign sjl-query-byte to-do query-byte
  assign sjl-wait-byte to-do wait-byte
  assign sjl-send-byte to-do send-byte
  assign sjl-link-reset to-do link-reset
  assign sjl-link-flush to-do link-flush
  assign .sjl-link-state to-do .link-state
;
previous

also c-c definitions  also cc/i
: set-sjl-link	\ addr --
\ *G Set the base address in the target at which the pseudo-UART
\ ** four byte RAM block is located. The Umbilical Forth link
\ ** drivers are also set to use the pseudo UART.
  set-sjl-link
;
previous previous definitions


\ ************************
\ *S Interactive debugging
\ ************************
\ *P When you build a standalone target, you can still debug it
\ ** using the J-Link. If the compiler directive *\fo{INTERACTIVE}
\ ** has been used the compiler stays alive after executing *\fo{FINIS}.
\ ** you can then use any of the commands as required. In order
\ ** to permit access to any memory block in the target, we first
\ ** modify our usual *\fo{SECTION} definitions in the control file.
\ *E  $0000:0000 $FFFF:FFFF udata section CATCHALL
\ **  $0000:0000 $0000:7FFF cdata section LPC1114ufjl	\ code section
\ **  $1000:0200 $1000:0FFF idata section PROGd	\ 4k IDATA
\ **  $1000:1000 $1000:1FFF udata section PROGu	\ 4k UDATA
\ *P The *\fo{CATCHALL} section provides a valid memory area for
\ ** every target memory address. It must be the first *\fo{SECTION}
\ ** defined.

\ *P After *\fo{FINIS}, the compiler stays alive. You can now
\ ** enter commands interactively at the keyboard.
\ *E all-via-jlink
\ ** Section PROGU switched
\ ** Section PROGD switched
\ ** Section K60TWR switched
\ ** Section CATCHALL switched ok
\ ** 0 100 dump
\ ** 0000:0000 00 FB 00 20 AD 05 00 00 ED 56 00 00 3D 57 00 00  .{. -...mV..=W..
\ ** 0000:0010 8D 57 00 00 DD 57 00 00 2D 58 00 00 00 00 00 00  .W..]W..-X......
\ ** 0000:0020 00 00 00 00 00 00 00 00 00 00 00 00 0F 02 00 00  ................
\ ** 0000:0030 7D 58 00 00 00 00 00 00 13 02 00 00 EB 72 00 00  }X..........kr..
\ ** 0000:0040 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  ................
\ ** 0000:0050 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  ................
\ ** 0000:0060 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00  ................
\ **  ok

\ *P The first command, *\fo{ALL-VIA-JLINK}, switches the memory
\ ** drivers to use the J-Link. The second command dumps a block
\ ** of memory. You can also use the debugging commands.
\ *E halt
\ ** R0  = 1FFF:0120 FFFF:FFFF 0000:37A9 0000:0000
\ ** R4  = 0000:0020 0000:001E 0000:0000 1FFF:00F4
\ ** R8  = 0000:0000 0000:0000 0000:0000 2000:FF00
\ ** R12 = 2000:FEF0 2000:FDE0 0000:5B3F 0000:6E18
\ ** XPSR = 2100:0000   MSP = 2000:FB00   PSP = 2000:FDE0
\ ** ( 0000:6E14 7047 pG )        bx LR ok
\ ** ss
\ ** R0  = 1FFF:0120 FFFF:FFFF 0000:37A9 0000:0000
\ ** R4  = 0000:0020 0000:001E 0000:0000 1FFF:00F4
\ ** R8  = 0000:0000 0000:0000 0000:0000 2000:FF00
\ ** R12 = 2000:FEF0 2000:FDE0 0000:5B3F 0000:5B42
\ ** XPSR = 2100:0000   MSP = 2000:FB00   PSP = 2000:FDE0
\ ** ( 0000:5B3E F5E7 ug )        b # $5B2C ok
\ ** ss
\ ** R0  = 1FFF:0120 FFFF:FFFF 0000:37A9 0000:0000
\ ** R4  = 0000:0020 0000:001E 0000:0000 1FFF:00F4
\ ** R8  = 0000:0000 0000:0000 0000:0000 2000:FF00
\ ** R12 = 2000:FEF0 2000:FDE0 0000:5B3F 0000:5B30
\ ** XPSR = 2100:0000   MSP = 2000:FB00   PSP = 2000:FDE0
\ ** ( 0000:5B2C FCF794FC |w.| )  bl # $2458      [I ok
\ ** go  ok


\ ======
\ *> ###
\ ======

decimal

