\ xCortex introduction

\ ========
\ *! intro
\ *T Introduction
\ ========

\ ***************
\ *S What you get
\ ***************
\ *P The MPE Forth 7 cross compiler for the ARM Cortex-M
\ ** family consists of five portions which will have been installed
\ ** for you.
\ *( 1
\ *B AIDE front end (Windows only).
\ *B Forth 7 cross compiler for ARM and Cortex-M0/M1/M3/M4/M7.
\ *B Standalone Forth source code and configuration files.
\ ** Note that ARM Forth Stamp compilers do not include the PowerFile
\ ** target file system, PowerView GUI, PID loop controller and state
\ ** machine compiler. The Stamp compiler is restricted to 120k of code
\ ** space and 64k of RAM, and configurations and device drivers are
\ ** only supplied for the following CPU families: NXP LPC21xx;
\ ** NXP LPC17xx; Freescale Kinetis; ST STM32F4xx.
\ *B Umbilical Forth source code and configuration files.
\ ** The best reference for the target code is to read it yourself.
\ *B Support tools - MAKE utility, Intel Hex and Motorola S-record
\ ** converters plus a range of additional tools. See the *\i{TOOLS}
\ ** folder.
\ *)

\ *P The *\i{Docs} folder contains the cross compiler and CPU
\ ** specific manuals. This folder also contains CPU and chip
\ ** documentation, so browse it to see what is there.
\ ** The *\i{Cortex/Manual} folder contains a manual
\ ** for the Cortex-specific target code. The *\i{Common/Manual} folder
\ ** contains a manual for common high-level target code used by all
\ ** CPU targets.

\ *P Many folders contain release notes in files called *\i{Release.xxx.txt}.
\ ** These contain a change history.

\ ***********************
\ *S Running the compiler
\ ***********************
\ *P The mechanics of compilation are covered in the general cross-compiler
\ ** manual, *\i{xc7man.pdf}. This section covers how to launch the cross
\ ** itself.

\ ============================
\ *N Package and binary naming
\ ============================
\ *P The compilers are named according to the pattern below.
\ ** They can be found in the <target>/Compiler directory. Under
\ ** macOS and Linux, there will be copies in the /usr/local/bin
\ ** directory.

\ *E x<target>_<cpu>_<os>[.<bin>]
\ **
\ ** <target> = CPU for which the cross-compiler produces code, e.g.
\ **            ArmCortex
\ **            MSP430
\ **            x86
\ **            x64
\ **
\ ** <cpu>    = x86     32 bit Intel - host operating system CPU
\ **            x64     64 bit AMD/Intel
\ **            arm     32 bit ARM
\ **            aa64    64 bit ARM
\ **
\ ** <os>     = win     Windows 32/64 - host operating system
\ **            lin     Linux 32/64
\ **            mac     OS/X and macOS 32/64
\ **
\ ** <bin>    = exe     Windows 32/64 - host operating system file type
\ **            elf     ELF 32/64, e.g. Linux
\ **            mo      Mach O for Mac OS/X

\ *P Under macOS and Linux, the installation script will produce a link of the form
\ *C   x<target>
\ *P The links are the recommended way to run the compiler.
\ ** Under Linux and OS/X, the links are symlinks in the /usr/local/bin
\ ** directory.
\ ** Under Windows (must be 7+) these are symlinks in Windows\System32, e.g.
\ *C   mklink xArmCortex.exe x:\buildkit.dev\software\COMPILER\xArmCortexHfp_x86_win.exe

\ =========================
\ *N Launching the compiler
\ =========================
\ *P At launch time, the cross-compiler is a normal Forth system.
\ ** When it executes the word *\fo{CROSS-COMPILE}, it "pulls
\ ** down the shutters" and reconfigures itself as a cross-compiler.
\ ** When you run the cross-compiler you can pass Forth commands
\ ** on the command line, e.g.
\ *C   xArmCortex  include MyControlFile.ctl


\ *******************************
\ *S Downloading the target image
\ *******************************
\ *P If the board you are using does not have a pre-installed Forth
\ ** kernel, you will have to program the image produced by the cross
\ ** compiler using a suitable bootloader or JTAG unit.

\ *P Most ARM and Cortex targets supported by MPE include a utility
\ ** word *\fo{REFLASH ( -- )}, which will download a new image into
\ ** the target and reboot the board. The *\fo{REFLASH} tools are
\ ** documented in the target code manuals.

\ *P The Segger JLink tools are supported for Flash programming and
\ ** debugging. See the separate chapter for more details.


\ *****************
\ *S Supported Cortex CPUs and hardware
\ *****************
\ *P The Cortex family of CPUs is supplied by a large number of
\ ** silicon vendors. The target code will run on any of the CPUs,
\ ** and there are UART and timer drivers for several versions.
\ ** Look in the *\i{Cortex/Drivers} and *\i{Cortex/Hardware}
\ ** folders to see if your Cortex hardware is already supported.

\ *P The standard hardware boards supported by this compiler are:
\ *(
\ *B Infineon XMC11xx on XMC2Go board,
\ *B STM32072B Discovery board for STM32F0,
\ ** Also STM32F031, STM32F042, STM32051,
\ *B STM32F103 on Blue Pill and Reva boards,
\ *B STM32F3 on Nucleo board,
\ *B STM32F4 Discovery board,
\ *B STM3240G-EVAL board for STM32F4xx,
\ *B STM32F429-DISC0 or DISC1,
\ *B Olimex STM32-E407,
\ *B Olimex STM32-P107,
\ *B Olimex LPC1766-STK,
\ *B Reva STM32F103R for STM32 devices,
\ *B Keil MCB1114 for LPC1114,
\ *B Nuvoton NuMicro-SDK for NUC1100/120/130/140,
\ *B NXP/FReescale K60 Tower System,
\ *B NXP/Freescale FRDM KL25Z,
\ *B NXP 4078 on Redboard 4078,
\ *B TI LM3S9B92,
\ *B TI TM4C1294 on Launchpad board.
\ *)

\ *P Support for the following devices is planned or in
\ ** preparation.
\ *(
\ *B Energy Micro
\ *B Atmel
\ *B ST STM32F1 low power devices
\ *)

\ *****************
\ *S Supported ARM CPUs and hardware.
\ *****************
\ *P The ARM family of CPUs is supplied by a large number of silicon
\ ** vendors. The target code will run on any of the CPUs, and there
\ ** are UART and timer drivers for several versions. Look in the
\ ** *\i{ARM/Drivers} and *\i{ARM/Hardware} folders to see if your
\ ** ARM7/9 hardware is already supported.

\ *P The standard hardware boards supported by this compiler are:
\ *(
\ *B MPE USB ARM Stamp (NXP LPC2106)
\ *B Olimex LPC-P2148 (NXP LPC2148)
\ *B Keil MCB2300 with LPC2388
\ *B Hitex STR912 EvalBoard (ST STR912FAW44)
\ *B Atmel EB55 (Atmel AT91M55800A)
\ *)

\ **************
\ *S New targets
\ **************
\ *P MPE has ported the Forth system to a much wider range of ARM
\ ** CPUs than the ones mentioned above, so look in the *\i{ARM/Hardware}
\ ** and *\i{Cortex/Hardware} folders to see if your hardware is
\ ** already supported.

\ *P MPE can provide assistance with board installations and
\ ** drivers for new devices.


\ ======
\ *> ###
\ ======

