


\ ====================================== 
\ GPIOA - General-purpose I/Os
\ ====================================== 

$4800:0000 equ _GPIOA
  $0000 equ gpio.MODER           \ GPIO port mode register
  $0004 equ gpio.OTYPER          \ GPIO port output type register
  $0008 equ gpio.OSPEEDR         \ GPIO port output speed          register
  $000C equ gpio.PUPDR           \ GPIO port pull-up/pull-down          register
  $0010 equ gpio.IDR             \ GPIO port input data register
  $0014 equ gpio.ODR             \ GPIO port output data register
  $0018 equ gpio.BSRR            \ GPIO port bit set/reset          register
  $001C equ gpio.LCKR            \ GPIO port configuration lock          register
  $0020 equ gpio.AFRL            \ GPIO alternate function low          register
  $0024 equ gpio.AFRH            \ GPIO alternate function high          register
  $0028 equ gpio.BRR             \ Port bit reset register


\ ====================================== 
\ GPIOB - General-purpose I/Os
\ ====================================== 

$4800:0400 equ _GPIOB
  $0000 equ gpio.MODER           \ GPIO port mode register
  $0004 equ gpio.OTYPER          \ GPIO port output type register
  $0008 equ gpio.OSPEEDR         \ GPIO port output speed          register
  $000C equ gpio.PUPDR           \ GPIO port pull-up/pull-down          register
  $0010 equ gpio.IDR             \ GPIO port input data register
  $0014 equ gpio.ODR             \ GPIO port output data register
  $0018 equ gpio.BSRR            \ GPIO port bit set/reset          register
  $001C equ gpio.LCKR            \ GPIO port configuration lock          register
  $0020 equ gpio.AFRL            \ GPIO alternate function low          register
  $0024 equ gpio.AFRH            \ GPIO alternate function high          register
  $0028 equ gpio.BRR             \ Port bit reset register


\ ====================================== 
\ TSC - Touch sensing controller
\ ====================================== 

$4002:4000 equ _TSC
#24 equ EXTI2_TSC_IRQn
  $0000 equ tsc.CR               \ control register
  $0004 equ tsc.IER              \ interrupt enable register
  $0008 equ tsc.ICR              \ interrupt clear register
  $000C equ tsc.ISR              \ interrupt status register
  $0010 equ tsc.IOHCR            \ I/O hysteresis control          register
  $0018 equ tsc.IOASCR           \ I/O analog switch control          register
  $0020 equ tsc.IOSCR            \ I/O sampling control register
  $0028 equ tsc.IOCCR            \ I/O channel control register
  $0030 equ tsc.IOGCSR           \ I/O group control status          register
  $0034 equ tsc.IOG1CR           \ I/O group x counter register
  $0038 equ tsc.IOG2CR           \ I/O group x counter register
  $003C equ tsc.IOG3CR           \ I/O group x counter register
  $0040 equ tsc.IOG4CR           \ I/O group x counter register
  $0044 equ tsc.IOG5CR           \ I/O group x counter register
  $0048 equ tsc.IOG6CR           \ I/O group x counter register
  $004C equ tsc.IOG7CR           \ I/O group x counter register
  $0050 equ tsc.IOG8CR           \ I/O group x counter register


\ ====================================== 
\ CRC - cyclic redundancy check calculation unit
\ ====================================== 

$4002:3000 equ _CRC
  $0000 equ crc.DR               \ Data register
  $0004 equ crc.IDR              \ Independent data register
  $0008 equ crc.CR               \ Control register
  $0010 equ crc.INIT             \ Initial CRC value
  $0014 equ crc.POL              \ CRC polynomial


\ ====================================== 
\ Flash - Flash
\ ====================================== 

$4002:2000 equ _Flash
#20 equ FLASH_IRQn
  $0000 equ fl.ACR               \ Flash access control register
  $0004 equ fl.KEYR              \ Flash key register
  $0008 equ fl.OPTKEYR           \ Flash option key register
  $000C equ fl.SR                \ Flash status register
  $0010 equ fl.CR                \ Flash control register
  $0014 equ fl.AR                \ Flash address register
  $001C equ fl.OBR               \ Option byte register
  $0020 equ fl.WRPR              \ Write protection register


\ ====================================== 
\ RCC - Reset and clock control
\ ====================================== 

$4002:1000 equ _RCC
#21 equ RCC_IRQn
  $0000 equ rcc.CR               \ Clock control register
  $0004 equ rcc.CFGR             \ Clock configuration register          (RCC_CFGR)
  $0008 equ rcc.CIR              \ Clock interrupt register          (RCC_CIR)
  $000C equ rcc.APB2RSTR         \ APB2 peripheral reset register          (RCC_APB2RSTR)
  $0010 equ rcc.APB1RSTR         \ APB1 peripheral reset register          (RCC_APB1RSTR)
  $0014 equ rcc.AHBENR           \ AHB Peripheral Clock enable register          (RCC_AHBENR)
  $0018 equ rcc.APB2ENR          \ APB2 peripheral clock enable register          (RCC_APB2ENR)
  $001C equ rcc.APB1ENR          \ APB1 peripheral clock enable register          (RCC_APB1ENR)
  $0020 equ rcc.BDCR             \ Backup domain control register          (RCC_BDCR)
  $0024 equ rcc.CSR              \ Control/status register          (RCC_CSR)
  $0028 equ rcc.AHBRSTR          \ AHB peripheral reset register
  $002C equ rcc.CFGR2            \ Clock configuration register 2
  $0030 equ rcc.CFGR3            \ Clock configuration register 3


\ ====================================== 
\ DMA1 - DMA controller 1
\ ====================================== 

$4002:0000 equ _DMA1
#27 equ DMA1_CH1_IRQn
#28 equ DMA1_CH2_IRQn
#29 equ DMA1_CH3_IRQn
#30 equ DMA1_CH4_IRQn
#31 equ DMA1_CH5_IRQn
#32 equ DMA1_CH6_IRQn
#33 equ DMA1_CH7_IRQn
  $0000 equ dma.ISR              \ DMA interrupt status register          (DMA_ISR)
  $0004 equ dma.IFCR             \ DMA interrupt flag clear register          (DMA_IFCR)
  $0008 equ dma.CCR1             \ DMA channel configuration register          (DMA_CCR)
  $000C equ dma.CNDTR1           \ DMA channel 1 number of data          register
  $0010 equ dma.CPAR1            \ DMA channel 1 peripheral address          register
  $0014 equ dma.CMAR1            \ DMA channel 1 memory address          register
  $001C equ dma.CCR2             \ DMA channel configuration register          (DMA_CCR)
  $0020 equ dma.CNDTR2           \ DMA channel 2 number of data          register
  $0024 equ dma.CPAR2            \ DMA channel 2 peripheral address          register
  $0028 equ dma.CMAR2            \ DMA channel 2 memory address          register
  $0030 equ dma.CCR3             \ DMA channel configuration register          (DMA_CCR)
  $0034 equ dma.CNDTR3           \ DMA channel 3 number of data          register
  $0038 equ dma.CPAR3            \ DMA channel 3 peripheral address          register
  $003C equ dma.CMAR3            \ DMA channel 3 memory address          register
  $0044 equ dma.CCR4             \ DMA channel configuration register          (DMA_CCR)
  $0048 equ dma.CNDTR4           \ DMA channel 4 number of data          register
  $004C equ dma.CPAR4            \ DMA channel 4 peripheral address          register
  $0050 equ dma.CMAR4            \ DMA channel 4 memory address          register
  $0058 equ dma.CCR5             \ DMA channel configuration register          (DMA_CCR)
  $005C equ dma.CNDTR5           \ DMA channel 5 number of data          register
  $0060 equ dma.CPAR5            \ DMA channel 5 peripheral address          register
  $0064 equ dma.CMAR5            \ DMA channel 5 memory address          register
  $006C equ dma.CCR6             \ DMA channel configuration register          (DMA_CCR)
  $0070 equ dma.CNDTR6           \ DMA channel 6 number of data          register
  $0074 equ dma.CPAR6            \ DMA channel 6 peripheral address          register
  $0078 equ dma.CMAR6            \ DMA channel 6 memory address          register
  $0080 equ dma.CCR7             \ DMA channel configuration register          (DMA_CCR)
  $0084 equ dma.CNDTR7           \ DMA channel 7 number of data          register
  $0088 equ dma.CPAR7            \ DMA channel 7 peripheral address          register
  $008C equ dma.CMAR7            \ DMA channel 7 memory address          register


\ ====================================== 
\ DMA2 - Memory address
\ ====================================== 

$4002:0400 equ _DMA2
#72 equ DMA2_CH1_IRQn
#73 equ DMA2_CH2_IRQn
#74 equ DMA2_CH3_IRQn
#75 equ DMA2_CH4_IRQn
#76 equ DMA2_CH5_IRQn


\ ====================================== 
\ TIM2 - General purpose timer
\ ====================================== 

$4000:0000 equ _TIM2
#44 equ TIM2_IRQn
  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $0008 equ tim.SMCR             \ slave mode control register
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0018 equ tim.CCMR1_Output     \ capture/compare mode register 1 (output          mode)
  $0018 equ tim.CCMR1_Input      \ capture/compare mode register 1 (input          mode)
  $001C equ tim.CCMR2_Output     \ capture/compare mode register 2 (output          mode)
  $001C equ tim.CCMR2_Input      \ capture/compare mode register 2 (input          mode)
  $0020 equ tim.CCER             \ capture/compare enable          register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register
  $0034 equ tim.CCR1             \ capture/compare register 1
  $0038 equ tim.CCR2             \ capture/compare register 2
  $003C equ tim.CCR3             \ capture/compare register 3
  $0040 equ tim.CCR4             \ capture/compare register 4
  $0048 equ tim.DCR              \ DMA control register
  $004C equ tim.DMAR             \ DMA address for full transfer


\ ====================================== 
\ TIM3 - DMA register for burst              accesses
\ ====================================== 

$4000:0400 equ _TIM3
#45 equ TIM3_IRQn


\ ====================================== 
\ TIM4 - TIM3 global interrupt
\ ====================================== 

$4000:0800 equ _TIM4
#46 equ TIM4_IRQn


\ ====================================== 
\ TIM15 - General purpose timers
\ ====================================== 

$4001:4000 equ _TIM15
#40 equ TIM1_BRK_TIM15_IRQn
  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $0008 equ tim.SMCR             \ slave mode control register
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0018 equ tim.CCMR1_Output     \ capture/compare mode register (output          mode)
  $0018 equ tim.CCMR1_Input      \ capture/compare mode register 1 (input          mode)
  $0020 equ tim.CCER             \ capture/compare enable          register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register
  $0030 equ tim.RCR              \ repetition counter register
  $0034 equ tim.CCR1             \ capture/compare register 1
  $0038 equ tim.CCR2             \ capture/compare register 2
  $0044 equ tim.BDTR             \ break and dead-time register
  $0048 equ tim.DCR              \ DMA control register
  $004C equ tim.DMAR             \ DMA address for full transfer


\ ====================================== 
\ TIM16 - General-purpose-timers
\ ====================================== 

$4001:4400 equ _TIM16
#41 equ TIM1_UP_TIM16_IRQn
  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0018 equ tim.CCMR1_Output     \ capture/compare mode register (output          mode)
  $0018 equ tim.CCMR1_Input      \ capture/compare mode register 1 (input          mode)
  $0020 equ tim.CCER             \ capture/compare enable          register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register
  $0030 equ tim.RCR              \ repetition counter register
  $0034 equ tim.CCR1             \ capture/compare register 1
  $0044 equ tim.BDTR             \ break and dead-time register
  $0048 equ tim.DCR              \ DMA control register
  $004C equ tim.DMAR             \ DMA address for full transfer
  $0050 equ tim.OR               \ option register


\ ====================================== 
\ TIM17 - General purpose timer
\ ====================================== 

$4001:4800 equ _TIM17
#42 equ TIM1_TRG_COM_TIM17_IRQn
  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0018 equ tim.CCMR1_Output     \ capture/compare mode register (output          mode)
  $0018 equ tim.CCMR1_Input      \ capture/compare mode register 1 (input          mode)
  $0020 equ tim.CCER             \ capture/compare enable          register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register
  $0030 equ tim.RCR              \ repetition counter register
  $0034 equ tim.CCR1             \ capture/compare register 1
  $0044 equ tim.BDTR             \ break and dead-time register
  $0048 equ tim.DCR              \ DMA control register
  $004C equ tim.DMAR             \ DMA address for full transfer


\ ====================================== 
\ USART1 - Universal synchronous asynchronous receiver transmitter
\ ====================================== 

$4001:3800 equ _USART1
#53 equ USART1_EXTI25_IRQn
  $0000 equ Un.CR1               \ Control register 1
  $0004 equ Un.CR2               \ Control register 2
  $0008 equ Un.CR3               \ Control register 3
  $000C equ Un.BRR               \ Baud rate register
  $0010 equ Un.GTPR              \ Guard time and prescaler          register
  $0014 equ Un.RTOR              \ Receiver timeout register
  $0018 equ Un.RQR               \ Request register
  $001C equ Un.ISR               \ Interrupt & status          register
  $0020 equ Un.ICR               \ Interrupt flag clear register
  $0024 equ Un.RDR               \ Receive data register
  $0028 equ Un.TDR               \ Transmit data register


\ ====================================== 
\ USART2 - Transmit data value
\ ====================================== 

$4000:4400 equ _USART2
#54 equ USART2_EXTI26_IRQn


\ ====================================== 
\ USART3 - USART2 global interrupt and EXTI Line 26        interrupt
\ ====================================== 

$4000:4800 equ _USART3
#55 equ USART3_EXTI28_IRQn


\ ====================================== 
\ UART4 - USART3 global interrupt and EXTI Line 28        interrupt
\ ====================================== 

$4000:4C00 equ _UART4
#68 equ UART4_EXTI34_IRQn


\ ====================================== 
\ UART5 - UART4 global and EXTI Line 34        interrupts
\ ====================================== 

$4000:5000 equ _UART5
#69 equ UART5_EXTI35_IRQn


\ ====================================== 
\ SPI1 - Serial peripheral interface/Inter-IC      sound
\ ====================================== 

$4001:3000 equ _SPI1
#51 equ SPI1_IRQn
  $0000 equ spi.CR1              \ control register 1
  $0004 equ spi.CR2              \ control register 2
  $0008 equ spi.SR               \ status register
  $000C equ spi.DR               \ data register
  $0010 equ spi.CRCPR            \ CRC polynomial register
  $0014 equ spi.RXCRCR           \ RX CRC register
  $0018 equ spi.TXCRCR           \ TX CRC register
  $001C equ spi.I2SCFGR          \ I2S configuration register
  $0020 equ spi.I2SPR            \ I2S prescaler register


\ ====================================== 
\ SPI2 - I2S Linear prescaler
\ ====================================== 

$4000:3800 equ _SPI2
#52 equ SPI2_IRQn


\ ====================================== 
\ SPI3 - SPI2 global interrupt
\ ====================================== 

$4000:3C00 equ _SPI3
#67 equ SPI3_IRQn


\ ====================================== 
\ EXTI - External interrupt/event      controller
\ ====================================== 

$4001:0400 equ _EXTI
#18 equ TAMP_STAMP_IRQn
#22 equ EXTI0_IRQn
#23 equ EXTI1_IRQn
#24 equ EXTI2_TSC_IRQn
#25 equ EXTI3_IRQn
#26 equ EXTI4_IRQn
#39 equ EXTI9_5_IRQn
#47 equ I2C1_EV_EXTI23_IRQn
#53 equ USART1_EXTI25_IRQn
#54 equ USART2_EXTI26_IRQn
#55 equ USART3_EXTI28_IRQn
#56 equ EXTI15_10_IRQn
#68 equ UART4_EXTI34_IRQn
#69 equ UART5_EXTI35_IRQn
#92 equ USB_WKUP_EXTI_IRQn
  $0000 equ exti.IMR1            \ Interrupt mask register
  $0004 equ exti.EMR1            \ Event mask register
  $0008 equ exti.RTSR1           \ Rising Trigger selection          register
  $000C equ exti.FTSR1           \ Falling Trigger selection          register
  $0010 equ exti.SWIER1          \ Software interrupt event          register
  $0014 equ exti.PR1             \ Pending register
  $0018 equ exti.IMR2            \ Interrupt mask register
  $001C equ exti.EMR2            \ Event mask register
  $0020 equ exti.RTSR2           \ Rising Trigger selection          register
  $0024 equ exti.FTSR2           \ Falling Trigger selection          register
  $0028 equ exti.SWIER2          \ Software interrupt event          register
  $002C equ exti.PR2             \ Pending register


\ ====================================== 
\ COMP - Comparator
\ ====================================== 

$4001:001C equ _COMP
#80 equ COMP123_IRQn
#81 equ COMP456_IRQn
#82 equ COMP7_IRQn
  $0000 equ comp.COMP1_CSR       \ control and status register
  $0004 equ comp.COMP2_CSR       \ control and status register
  $0008 equ comp.COMP3_CSR       \ control and status register
  $000C equ comp.COMP4_CSR       \ control and status register
  $0010 equ comp.COMP5_CSR       \ control and status register
  $0014 equ comp.COMP6_CSR       \ control and status register
  $0018 equ comp.COMP7_CSR       \ control and status register


\ ====================================== 
\ PWR - Power control
\ ====================================== 

$4000:7000 equ _PWR
#17 equ PVD_IRQn
  $0000 equ pwr.CR               \ power control register
  $0004 equ pwr.CSR              \ power control/status register


\ ====================================== 
\ CAN - Controller area network
\ ====================================== 

$4000:6400 equ _CAN
#35 equ USB_HP_CAN_TX_IRQn
#36 equ USB_LP_CAN_RX0_IRQn
#37 equ CAN_RX1_IRQn
#38 equ CAN_SCE_IRQn
  $0000 equ can.MCR              \ master control register
  $0004 equ can.MSR              \ master status register
  $0008 equ can.TSR              \ transmit status register
  $000C equ can.RF0R             \ receive FIFO 0 register
  $0010 equ can.RF1R             \ receive FIFO 1 register
  $0014 equ can.IER              \ interrupt enable register
  $0018 equ can.ESR              \ error status register
  $001C equ can.BTR              \ bit timing register
  $0180 equ can.TI0R             \ TX mailbox identifier register
  $0184 equ can.TDT0R            \ mailbox data length control and time stamp          register
  $0188 equ can.TDL0R            \ mailbox data low register
  $018C equ can.TDH0R            \ mailbox data high register
  $0190 equ can.TI1R             \ TX mailbox identifier register
  $0194 equ can.TDT1R            \ mailbox data length control and time stamp          register
  $0198 equ can.TDL1R            \ mailbox data low register
  $019C equ can.TDH1R            \ mailbox data high register
  $01A0 equ can.TI2R             \ TX mailbox identifier register
  $01A4 equ can.TDT2R            \ mailbox data length control and time stamp          register
  $01A8 equ can.TDL2R            \ mailbox data low register
  $01AC equ can.TDH2R            \ mailbox data high register
  $01B0 equ can.RI0R             \ receive FIFO mailbox identifier          register
  $01B4 equ can.RDT0R            \ receive FIFO mailbox data length control and          time stamp register
  $01B8 equ can.RDL0R            \ receive FIFO mailbox data low          register
  $01BC equ can.RDH0R            \ receive FIFO mailbox data high          register
  $01C0 equ can.RI1R             \ receive FIFO mailbox identifier          register
  $01C4 equ can.RDT1R            \ receive FIFO mailbox data length control and          time stamp register
  $01C8 equ can.RDL1R            \ receive FIFO mailbox data low          register
  $01CC equ can.RDH1R            \ receive FIFO mailbox data high          register
  $0200 equ can.FMR              \ filter master register
  $0204 equ can.FM1R             \ filter mode register
  $020C equ can.FS1R             \ filter scale register
  $0214 equ can.FFA1R            \ filter FIFO assignment          register
  $021C equ can.FA1R             \ CAN filter activation register
  $0240 equ can.F0R1             \ Filter bank 0 register 1
  $0244 equ can.F0R2             \ Filter bank 0 register 2
  $0248 equ can.F1R1             \ Filter bank 1 register 1
  $024C equ can.F1R2             \ Filter bank 1 register 2
  $0250 equ can.F2R1             \ Filter bank 2 register 1
  $0254 equ can.F2R2             \ Filter bank 2 register 2
  $0258 equ can.F3R1             \ Filter bank 3 register 1
  $025C equ can.F3R2             \ Filter bank 3 register 2
  $0260 equ can.F4R1             \ Filter bank 4 register 1
  $0264 equ can.F4R2             \ Filter bank 4 register 2
  $0268 equ can.F5R1             \ Filter bank 5 register 1
  $026C equ can.F5R2             \ Filter bank 5 register 2
  $0270 equ can.F6R1             \ Filter bank 6 register 1
  $0274 equ can.F6R2             \ Filter bank 6 register 2
  $0278 equ can.F7R1             \ Filter bank 7 register 1
  $027C equ can.F7R2             \ Filter bank 7 register 2
  $0280 equ can.F8R1             \ Filter bank 8 register 1
  $0284 equ can.F8R2             \ Filter bank 8 register 2
  $0288 equ can.F9R1             \ Filter bank 9 register 1
  $028C equ can.F9R2             \ Filter bank 9 register 2
  $0290 equ can.F10R1            \ Filter bank 10 register 1
  $0294 equ can.F10R2            \ Filter bank 10 register 2
  $0298 equ can.F11R1            \ Filter bank 11 register 1
  $029C equ can.F11R2            \ Filter bank 11 register 2
  $02A0 equ can.F12R1            \ Filter bank 4 register 1
  $02A4 equ can.F12R2            \ Filter bank 12 register 2
  $02A8 equ can.F13R1            \ Filter bank 13 register 1
  $02AC equ can.F13R2            \ Filter bank 13 register 2
  $02B0 equ can.F14R1            \ Filter bank 14 register 1
  $02B4 equ can.F14R2            \ Filter bank 14 register 2
  $02B8 equ can.F15R1            \ Filter bank 15 register 1
  $02BC equ can.F15R2            \ Filter bank 15 register 2
  $02C0 equ can.F16R1            \ Filter bank 16 register 1
  $02C4 equ can.F16R2            \ Filter bank 16 register 2
  $02C8 equ can.F17R1            \ Filter bank 17 register 1
  $02CC equ can.F17R2            \ Filter bank 17 register 2
  $02D0 equ can.F18R1            \ Filter bank 18 register 1
  $02D4 equ can.F18R2            \ Filter bank 18 register 2
  $02D8 equ can.F19R1            \ Filter bank 19 register 1
  $02DC equ can.F19R2            \ Filter bank 19 register 2
  $02E0 equ can.F20R1            \ Filter bank 20 register 1
  $02E4 equ can.F20R2            \ Filter bank 20 register 2
  $02E8 equ can.F21R1            \ Filter bank 21 register 1
  $02EC equ can.F21R2            \ Filter bank 21 register 2
  $02F0 equ can.F22R1            \ Filter bank 22 register 1
  $02F4 equ can.F22R2            \ Filter bank 22 register 2
  $02F8 equ can.F23R1            \ Filter bank 23 register 1
  $02FC equ can.F23R2            \ Filter bank 23 register 2
  $0300 equ can.F24R1            \ Filter bank 24 register 1
  $0304 equ can.F24R2            \ Filter bank 24 register 2
  $0308 equ can.F25R1            \ Filter bank 25 register 1
  $030C equ can.F25R2            \ Filter bank 25 register 2
  $0310 equ can.F26R1            \ Filter bank 26 register 1
  $0314 equ can.F26R2            \ Filter bank 26 register 2
  $0318 equ can.F27R1            \ Filter bank 27 register 1
  $031C equ can.F27R2            \ Filter bank 27 register 2


\ ====================================== 
\ USB_FS - Universal serial bus full-speed device      interface
\ ====================================== 

$4000:5C00 equ _USB_FS
#58 equ USB_WKUP_IRQn
#90 equ USB_HP_IRQn
#91 equ USB_LP_IRQn
  $0000 equ usb_fs.USB_EP0R      \ endpoint 0 register
  $0004 equ usb_fs.USB_EP1R      \ endpoint 1 register
  $0008 equ usb_fs.USB_EP2R      \ endpoint 2 register
  $000C equ usb_fs.USB_EP3R      \ endpoint 3 register
  $0010 equ usb_fs.USB_EP4R      \ endpoint 4 register
  $0014 equ usb_fs.USB_EP5R      \ endpoint 5 register
  $0018 equ usb_fs.USB_EP6R      \ endpoint 6 register
  $001C equ usb_fs.USB_EP7R      \ endpoint 7 register
  $0040 equ usb_fs.USB_CNTR      \ control register
  $0044 equ usb_fs.ISTR          \ interrupt status register
  $0048 equ usb_fs.FNR           \ frame number register
  $004C equ usb_fs.DADDR         \ device address
  $0050 equ usb_fs.BTABLE        \ Buffer table address


\ ====================================== 
\ I2C1 - Inter-integrated circuit
\ ====================================== 

$4000:5400 equ _I2C1
#47 equ I2C1_EV_EXTI23_IRQn
#48 equ I2C1_ER_IRQn
  $0000 equ i2c.CR1              \ Control register 1
  $0004 equ i2c.CR2              \ Control register 2
  $0008 equ i2c.OAR1             \ Own address register 1
  $000C equ i2c.OAR2             \ Own address register 2
  $0010 equ i2c.TIMINGR          \ Timing register
  $0014 equ i2c.TIMEOUTR         \ Status register 1
  $0018 equ i2c.ISR              \ Interrupt and Status register
  $001C equ i2c.ICR              \ Interrupt clear register
  $0020 equ i2c.PECR             \ PEC register
  $0024 equ i2c.RXDR             \ Receive data register
  $0028 equ i2c.TXDR             \ Transmit data register


\ ====================================== 
\ I2C2 - 8-bit transmit data
\ ====================================== 

$4000:5800 equ _I2C2
#49 equ I2C2_EV_EXTI24_IRQn
#50 equ I2C2_ER_IRQn


\ ====================================== 
\ IWDG - Independent watchdog
\ ====================================== 

$4000:3000 equ _IWDG
  $0000 equ iwdg.KR              \ Key register
  $0004 equ iwdg.PR              \ Prescaler register
  $0008 equ iwdg.RLR             \ Reload register
  $000C equ iwdg.SR              \ Status register
  $0010 equ iwdg.WINR            \ Window register


\ ====================================== 
\ WWDG - Window watchdog
\ ====================================== 

$4000:2C00 equ _WWDG
#16 equ WWDG_IRQn
  $0000 equ wwdg.CR              \ Control register
  $0004 equ wwdg.CFR             \ Configuration register
  $0008 equ wwdg.SR              \ Status register


\ ====================================== 
\ RTC - Real-time clock
\ ====================================== 

$4000:2800 equ _RTC
#19 equ RTC_WKUP_IRQn
#57 equ RTCAlarm_IRQn
  $0000 equ rtc.TR               \ time register
  $0004 equ rtc.DR               \ date register
  $0008 equ rtc.CR               \ control register
  $000C equ rtc.ISR              \ initialization and status          register
  $0010 equ rtc.PRER             \ prescaler register
  $0014 equ rtc.WUTR             \ wakeup timer register
  $001C equ rtc.ALRMAR           \ alarm A register
  $0020 equ rtc.ALRMBR           \ alarm B register
  $0024 equ rtc.WPR              \ write protection register
  $0028 equ rtc.SSR              \ sub second register
  $002C equ rtc.SHIFTR           \ shift control register
  $0030 equ rtc.TSTR             \ time stamp time register
  $0034 equ rtc.TSDR             \ time stamp date register
  $0038 equ rtc.TSSSR            \ timestamp sub second register
  $003C equ rtc.CALR             \ calibration register
  $0040 equ rtc.TAFCR            \ tamper and alternate function configuration          register
  $0044 equ rtc.ALRMASSR         \ alarm A sub second register
  $0048 equ rtc.ALRMBSSR         \ alarm B sub second register
  $0050 equ rtc.BKP0R            \ backup register
  $0054 equ rtc.BKP1R            \ backup register
  $0058 equ rtc.BKP2R            \ backup register
  $005C equ rtc.BKP3R            \ backup register
  $0060 equ rtc.BKP4R            \ backup register
  $0064 equ rtc.BKP5R            \ backup register
  $0068 equ rtc.BKP6R            \ backup register
  $006C equ rtc.BKP7R            \ backup register
  $0070 equ rtc.BKP8R            \ backup register
  $0074 equ rtc.BKP9R            \ backup register
  $0078 equ rtc.BKP10R           \ backup register
  $007C equ rtc.BKP11R           \ backup register
  $0080 equ rtc.BKP12R           \ backup register
  $0084 equ rtc.BKP13R           \ backup register
  $0088 equ rtc.BKP14R           \ backup register
  $008C equ rtc.BKP15R           \ backup register
  $0090 equ rtc.BKP16R           \ backup register
  $0094 equ rtc.BKP17R           \ backup register
  $0098 equ rtc.BKP18R           \ backup register
  $009C equ rtc.BKP19R           \ backup register
  $00A0 equ rtc.BKP20R           \ backup register
  $00A4 equ rtc.BKP21R           \ backup register
  $00A8 equ rtc.BKP22R           \ backup register
  $00AC equ rtc.BKP23R           \ backup register
  $00B0 equ rtc.BKP24R           \ backup register
  $00B4 equ rtc.BKP25R           \ backup register
  $00B8 equ rtc.BKP26R           \ backup register
  $00BC equ rtc.BKP27R           \ backup register
  $00C0 equ rtc.BKP28R           \ backup register
  $00C4 equ rtc.BKP29R           \ backup register
  $00C8 equ rtc.BKP30R           \ backup register
  $00CC equ rtc.BKP31R           \ backup register


\ ====================================== 
\ TIM6 - Basic timers
\ ====================================== 

$4000:1000 equ _TIM6
#70 equ TIM6_DACUNDER_IRQn
  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register


\ ====================================== 
\ TIM7 - Low Auto-reload value
\ ====================================== 

$4000:1400 equ _TIM7
#71 equ TIM7_IRQn


\ ====================================== 
\ DAC - Digital-to-analog converter
\ ====================================== 

$4000:7400 equ _DAC
#70 equ TIM6_DACUNDER_IRQn
  $0000 equ dac.CR               \ control register
  $0004 equ dac.SWTRIGR          \ software trigger register
  $0008 equ dac.DHR12R1          \ channel1 12-bit right-aligned data holding          register
  $000C equ dac.DHR12L1          \ channel1 12-bit left aligned data holding          register
  $0010 equ dac.DHR8R1           \ channel1 8-bit right aligned data holding          register
  $0014 equ dac.DHR12R2          \ channel2 12-bit right aligned data holding          register
  $0018 equ dac.DHR12L2          \ channel2 12-bit left aligned data holding          register
  $001C equ dac.DHR8R2           \ channel2 8-bit right-aligned data holding          register
  $0020 equ dac.DHR12RD          \ Dual DAC 12-bit right-aligned data holding          register
  $0024 equ dac.DHR12LD          \ DUAL DAC 12-bit left aligned data holding          register
  $0028 equ dac.DHR8RD           \ DUAL DAC 8-bit right aligned data holding          register
  $002C equ dac.DOR1             \ channel1 data output register
  $0030 equ dac.DOR2             \ channel2 data output register
  $0034 equ dac.SR               \ status register


\ ====================================== 
\ NVIC - Nested Vectored Interrupt      Controller
\ ====================================== 

$E000:E000 equ _NVIC
  $0004 equ nvic.ICTR            \ Interrupt Controller Type          Register
  $0F00 equ nvic.STIR            \ Software Triggered Interrupt          Register
  $0100 equ nvic.ISER0           \ Interrupt Set-Enable Register
  $0104 equ nvic.ISER1           \ Interrupt Set-Enable Register
  $0108 equ nvic.ISER2           \ Interrupt Set-Enable Register
  $0180 equ nvic.ICER0           \ Interrupt Clear-Enable          Register
  $0184 equ nvic.ICER1           \ Interrupt Clear-Enable          Register
  $0188 equ nvic.ICER2           \ Interrupt Clear-Enable          Register
  $0200 equ nvic.ISPR0           \ Interrupt Set-Pending Register
  $0204 equ nvic.ISPR1           \ Interrupt Set-Pending Register
  $0208 equ nvic.ISPR2           \ Interrupt Set-Pending Register
  $0280 equ nvic.ICPR0           \ Interrupt Clear-Pending          Register
  $0284 equ nvic.ICPR1           \ Interrupt Clear-Pending          Register
  $0288 equ nvic.ICPR2           \ Interrupt Clear-Pending          Register
  $0300 equ nvic.IABR0           \ Interrupt Active Bit Register
  $0304 equ nvic.IABR1           \ Interrupt Active Bit Register
  $0308 equ nvic.IABR2           \ Interrupt Active Bit Register
  $0400 equ nvic.IPR0            \ Interrupt Priority Register
  $0404 equ nvic.IPR1            \ Interrupt Priority Register
  $0408 equ nvic.IPR2            \ Interrupt Priority Register
  $040C equ nvic.IPR3            \ Interrupt Priority Register
  $0410 equ nvic.IPR4            \ Interrupt Priority Register
  $0414 equ nvic.IPR5            \ Interrupt Priority Register
  $0418 equ nvic.IPR6            \ Interrupt Priority Register
  $041C equ nvic.IPR7            \ Interrupt Priority Register
  $0420 equ nvic.IPR8            \ Interrupt Priority Register
  $0424 equ nvic.IPR9            \ Interrupt Priority Register
  $0428 equ nvic.IPR10           \ Interrupt Priority Register
  $042C equ nvic.IPR11           \ Interrupt Priority Register
  $0430 equ nvic.IPR12           \ Interrupt Priority Register
  $0434 equ nvic.IPR13           \ Interrupt Priority Register
  $0438 equ nvic.IPR14           \ Interrupt Priority Register
  $043C equ nvic.IPR15           \ Interrupt Priority Register
  $0440 equ nvic.IPR16           \ Interrupt Priority Register
  $0444 equ nvic.IPR17           \ Interrupt Priority Register
  $0448 equ nvic.IPR18           \ Interrupt Priority Register
  $044C equ nvic.IPR19           \ Interrupt Priority Register
  $0450 equ nvic.IPR20           \ Interrupt Priority Register


\ ====================================== 
\ FPU - Floating point unit
\ ====================================== 

$E000:ED88 equ _FPU
#97 equ FPU_IRQn
  $0000 equ fpu.CPACR            \ Coprocessor Access Control          Register
  $01AC equ fpu.FPCCR            \ FP Context Control Register
  $01B0 equ fpu.FPCAR            \ FP Context Address Register
  $01B4 equ fpu.FPDSCR           \ FP Default Status Control          Register
  $01B8 equ fpu.MVFR0            \ Media and VFP Feature Register          0
  $01BC equ fpu.MVFR1            \ Media and VFP Feature Register          1


\ ====================================== 
\ DBGMCU - Debug support
\ ====================================== 

$E004:2000 equ _DBGMCU
  $0000 equ dbg.IDCODE           \ MCU Device ID Code Register
  $0004 equ dbg.CR               \ Debug MCU Configuration          Register
  $0008 equ dbg.APB1FZ           \ APB Low Freeze Register
  $000C equ dbg.APB2FZ           \ APB High Freeze Register


\ ====================================== 
\ TIM1 - Advanced timer
\ ====================================== 

$4001:2C00 equ _TIM1
#43 equ TIM1_CC_IRQn
  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $0008 equ tim.SMCR             \ slave mode control register
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0018 equ tim.CCMR1_Output     \ capture/compare mode register (output          mode)
  $0018 equ tim.CCMR1_Input      \ capture/compare mode register 1 (input          mode)
  $001C equ tim.CCMR2_Output     \ capture/compare mode register (output          mode)
  $001C equ tim.CCMR2_Input      \ capture/compare mode register 2 (input          mode)
  $0020 equ tim.CCER             \ capture/compare enable          register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register
  $0030 equ tim.RCR              \ repetition counter register
  $0034 equ tim.CCR1             \ capture/compare register 1
  $0038 equ tim.CCR2             \ capture/compare register 2
  $003C equ tim.CCR3             \ capture/compare register 3
  $0040 equ tim.CCR4             \ capture/compare register 4
  $0044 equ tim.BDTR             \ break and dead-time register
  $0048 equ tim.DCR              \ DMA control register
  $004C equ tim.DMAR             \ DMA address for full transfer
  $0054 equ tim.CCMR3_Output     \ capture/compare mode register 3 (output          mode)
  $0058 equ tim.CCR5             \ capture/compare register 5
  $005C equ tim.CCR6             \ capture/compare register 6
  $0060 equ tim.OR               \ option registers


\ ====================================== 
\ TIM8 - Advanced-timers
\ ====================================== 

$4001:3400 equ _TIM8
#59 equ TIM8_BRK_IRQn
#60 equ TIM8_UP_IRQn
#61 equ TIM8_TRG_COM_IRQn
#62 equ TIM8_CC_IRQn
  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $0008 equ tim.SMCR             \ slave mode control register
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0018 equ tim.CCMR1_Output     \ capture/compare mode register (output          mode)
  $0018 equ tim.CCMR1_Input      \ capture/compare mode register 1 (input          mode)
  $001C equ tim.CCMR2_Output     \ capture/compare mode register (output          mode)
  $001C equ tim.CCMR2_Input      \ capture/compare mode register 2 (input          mode)
  $0020 equ tim.CCER             \ capture/compare enable          register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register
  $0030 equ tim.RCR              \ repetition counter register
  $0034 equ tim.CCR1             \ capture/compare register 1
  $0038 equ tim.CCR2             \ capture/compare register 2
  $003C equ tim.CCR3             \ capture/compare register 3
  $0040 equ tim.CCR4             \ capture/compare register 4
  $0044 equ tim.BDTR             \ break and dead-time register
  $0048 equ tim.DCR              \ DMA control register
  $004C equ tim.DMAR             \ DMA address for full transfer
  $0054 equ tim.CCMR3_Output     \ capture/compare mode register 3 (output          mode)
  $0058 equ tim.CCR5             \ capture/compare register 5
  $005C equ tim.CCR6             \ capture/compare register 6
  $0060 equ tim.OR               \ option registers


\ ====================================== 
\ ADC1 - Analog-to-Digital Converter
\ ====================================== 

$5000:0000 equ _ADC1
#34 equ ADC1_2_IRQn
  $0000 equ adc.ISR              \ interrupt and status register
  $0004 equ adc.IER              \ interrupt enable register
  $0008 equ adc.CR               \ control register
  $000C equ adc.CFGR             \ configuration register
  $0014 equ adc.SMPR1            \ sample time register 1
  $0018 equ adc.SMPR2            \ sample time register 2
  $0020 equ adc.TR1              \ watchdog threshold register 1
  $0024 equ adc.TR2              \ watchdog threshold register
  $0028 equ adc.TR3              \ watchdog threshold register 3
  $0030 equ adc.SQR1             \ regular sequence register 1
  $0034 equ adc.SQR2             \ regular sequence register 2
  $0038 equ adc.SQR3             \ regular sequence register 3
  $003C equ adc.SQR4             \ regular sequence register 4
  $0040 equ adc.DR               \ regular Data Register
  $004C equ adc.JSQR             \ injected sequence register
  $0060 equ adc.OFR1             \ offset register 1
  $0064 equ adc.OFR2             \ offset register 2
  $0068 equ adc.OFR3             \ offset register 3
  $006C equ adc.OFR4             \ offset register 4
  $0080 equ adc.JDR1             \ injected data register 1
  $0084 equ adc.JDR2             \ injected data register 2
  $0088 equ adc.JDR3             \ injected data register 3
  $008C equ adc.JDR4             \ injected data register 4
  $00A0 equ adc.AWD2CR           \ Analog Watchdog 2 Configuration          Register
  $00A4 equ adc.AWD3CR           \ Analog Watchdog 3 Configuration          Register
  $00B0 equ adc.DIFSEL           \ Differential Mode Selection Register          2
  $00B4 equ adc.CALFACT          \ Calibration Factors


\ ====================================== 
\ ADC2 - CALFACT_S
\ ====================================== 

$5000:0100 equ _ADC2
#34 equ ADC1_2_IRQn


\ ====================================== 
\ ADC3 - ADC1 and ADC2 global interrupt
\ ====================================== 

$5000:0400 equ _ADC3
#63 equ ADC3_IRQn


\ ====================================== 
\ ADC4 - ADC3 global interrupt
\ ====================================== 

$5000:0500 equ _ADC4
#77 equ ADC4_IRQn


\ ====================================== 
\ ADC1_2 - Analog-to-Digital Converter
\ ====================================== 

$5000:0300 equ _ADC1_2
  $0000 equ adc.CSR              \ ADC Common status register
  $0008 equ adc.CCR              \ ADC common control register
  $000C equ adc.CDR              \ ADC common regular data register for dual          and triple modes


\ ====================================== 
\ SYSCFG - System configuration controller
\ ====================================== 

$4001:0000 equ _SYSCFG
  $0000 equ syscfg.CFGR1         \ configuration register 1
  $0008 equ syscfg.EXTICR1       \ external interrupt configuration register          1
  $000C equ syscfg.EXTICR2       \ external interrupt configuration register          2
  $0010 equ syscfg.EXTICR3       \ external interrupt configuration register          3
  $0014 equ syscfg.EXTICR4       \ external interrupt configuration register          4
  $0018 equ syscfg.CFGR2         \ configuration register 2
  $0004 equ syscfg.RCR           \ CCM SRAM protection register


\ ====================================== 
\ OPAMP - Operational amplifier
\ ====================================== 

$4001:0038 equ _OPAMP
  $0000 equ opamp.OPAMP1_CR      \ OPAMP1 control register
  $0004 equ opamp.OPAMP2_CR      \ OPAMP2 control register
  $0008 equ opamp.OPAMP3_CR      \ OPAMP3 control register
  $000C equ opamp.OPAMP4_CR      \ OPAMP4 control register