\ ARM/Cortex test control file

((
Copyright (c) 2006, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))


\ MPE macros
c" ../../ROM/Common"
		setmacro CommonDir	\ where the common code lives
c" ../../ROM/Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ../../ROM/ARM/hardware/LPC210x
		setmacro HWDir		\ where board specific code lives
c" ../../ROM/Examples"
		setmacro ExampleDir	\ Examples
c" ."   	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%/Macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  Thumb-1			\ processor type
\  Hex-I32			\ Generate Intel Hex 32 bit

1 equ CPU=Cortex		\ for coded versions of some words
0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ---------------------------------------------
\ *H Define stack and user area sizes for TASKs
\ ---------------------------------------------

$0100 equ UP-SIZE	\ -- n
\ *G size of each task's user area
$0100 equ SP-SIZE	\ -- n
\ *G size of each task's data stack
$0200 equ RP-SIZE	\ -- n
\ *G size of each task's return stack
up-size rp-size + sp-size +
  equ task-size		\ -- n
\ *G Size of TASK data area
#2 equ sp-guard		\ -- n
\ *G Define the number of cells of guard space at the top of
\ ** the data stack

$0100 equ TIB-LEN	\ -- n
\ *G Terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs	\ -- n
\ *G Number of IRQ stacks, shared by all IRQs (1 min)
1 equ #FIQs	\ -- n
\ *G number of FIQ stacks, shared by all FIQs (1 min)
0 equ #SWIs	\ -- n
\ *G number of SWI nestings permitted. 0 is ok if SWIs are unused


\ *****************
\ Default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ *************
\ *S Memory map
\ *************

  $00000000 $0001FFFF cdata section TestCx
\ *G Primary code section in on-chip Flash
  $00100000 $0010CFFF idata section PROGd
\ *G 48k IDATA. This area contains
\ ** initialised data and user applications.
  $0010D000 $0010FFFF udata section PROGu
\ *G 8k UDATA. This area contains unitialised data.

interpreter
: prog TestCx ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ***************
\ Thumb test code
\ ***************

log

$100 allot


\ ========
\ Thumb v1
\ ========

Thumb-1

proc ProcBack	\ -- addr
  bx		r14
end-code

code Thumb1Ops	\ --
l: labBack
  adc .s	r1, r2
  add .s	r1, r2, # 3
  add .s	r1, # $88
  add .s	r1, r2, r3
  add		r8, r9
  add		r1, pc, # $88
  add		r1, sp, # $88
  add		sp, # $88
  and .s	r1, r2

  asr .s	r1, r2, # $11
  asr .s 	r1, r2

  b .ne		labBack
  b .pl		labFwd
  b		labBack
  b		labFwd

  bic .s 	r1, r2

  bkpt	# $55

  bl	ProcBack
  bl	ProcFwd
  blx	ProcBack
  blx	ProcFwd
  blx	r10
  bx	r10

  cmn	r1, r2

  cmp	r1, # $88
  cmp	r1, r2
  cmp	r8, r9

  eor	r1, r2
  ldmia	r5, ! { r1-r3, r6, r7 }

  ldr	r1, [ r2, # $44 ]
  ldr	r1, [ r2, r3 ]
  ldr	r1, [ PC, # $180 ]
  ldr	r1, [ SP, # $180 ]

  ldrb	r1, [ r2, # $11 ]
  ldrb	r1, [ r2, r3 ]

  ldrh	r1, [ r2, # $12 ]
  ldrb	r1, [ r2, r3 ]

  ldrsb	r1, [ r2, r3 ]

  ldrsh	r1, [ r2, r3 ]

  lsl .s	r1, r2, # $11
  lsl .s	r1, r2

  lsr .s	r1, r2, # $11
  lsr .s	r1, r2

  mov .s	r1, # $88
  mov .s	r1, r2		\ 2 low regs = enc T2, valid always if S=1
  mov		r1, r2	Thumb-2	\ 2 low regs = enc T1, valid always if S=0 and Thumb-2
  mov		r8, r9  Thumb-1

  mul .s	r1, r2

  mvn .s	r1, r2

  neg .s	r1, r2

  orr .s	r1, r2

  pop   { pc }
  pop	{ r1-r3, r6, r7 }
  pop	{ r1-r3, r6, r7, pc }

  push  { lr }
  push	{ r1-r3, r6, r7 }
  push	{ r1-r3, r6, r7, lr }

  ror .s	r1, r2

  sbc .s	r1, r2

  stmia	r5, ! { r1-r3, r6, r7 }

  str	r1, [ r2, # $44 ]
  str	r1, [ r2, r3 ]
  str	r1, [ SP, # $180 ]

  strb	r1, [ r2, # $11 ]
  strb	r1, [ r2, r3 ]

  strh	r1, [ r2, # $12 ]
  strh	r1, [ r2, r3 ]

  sub .s	r1, r2, # 3
  sub .s	r1, # $88
  sub .s	r1, r2, r3
  sub	sp, # $88

  swi	# $88

  tst	r1, r2

l: labFwd
  bx	r14
end-code

proc ProcFwd	\ -- addr
  bx	r14
end-code


\ ========
\ Thumb v2
\ ========

Thumb-2

code t2ADC	\ --
  adc		r1, r10, # $200
  adc .s	r1, r10, # $200
  adc .s	r1, r2
  adc .s	r1, r1, r2
  adc		r1, r10, r5
  adc		r1, r10, r5 .lsl # 2
  adc .s	r1, r10, r5 .lsl # 2
  bx		r14
end-code

code t2ADD	\ --
  add .s	r1, r2, # 7
  add .s	r1, # $55
  add .s	r1, r1, # $55
  add		r1, r10, # $300
  add .s	r1, r10, # $300
  add 		r1, r10, # $765
  add .s	r1, r2, r3
  add		r10, r1
  add		r10, r10, r1
  add .s	r1, r10, r5
  add		r1, r10, r5 .lsl # 2

  add		r1, SP, # $AA
  add		sp, sp, # $55
  add .s	r1, sp, # $200
  add		r1, sp, # $888

  add		r10, sp, r10
  add		sp, r2
  add		sp, sp, r2
  add		r5, sp, r10 .lsl # 1
  bx		r14
end-code

align l: AdrBack
  $5555AAAA ,
code t2ADR	\ --
  adr           r2, AdrFwd
  adr .w        r2, AdrFwd
  adr		r1, $ $40 +
  adr		r1, $ $40 -
  adr		r1, $ $200 +
  adr		r1, $ $200 -
  adr .w        r2, AdrBack
  bx		r14
end-code
align l: AdrFwd
  $AAAA5555 ,

code t2AND	\ --
  and		r1, r10, # $300
  and .s	r1, r10, # $700
  and .s	r1, r2
  and .s	r1, r1, r2
  and .s	r1, r10, r5
  and		r1, r10, r5 .lsl # 2
  bx		r14
end-code

code t2ASR	\ --
  asr .s	r1, r2, # $13
  asr .s	r1, r10, # $12
  asr .s	r1, r2
  asr .s	r1, r1, r2
  asr .s	r1, r10, r5
  asr		r1, r10, r5
  bx		r14
end-code

proc T2Back	\ -- addr
  bx		r14
end-code

code T2B	\ --
  b		T2Back
  b		T2Fwd
  b .w		T2Back
  b .w		T2Fwd
  b .ne		T2Back
  b .eq		T2Fwd
  b .ne .w	T2Back
  b .eq	.w	T2Fwd
  bx		r14
end-code

code t2BIC	\ --
  bic		r1, r10, # $200
  bic .s	r1, r10, # $200
  bic .s	r1, r2
  bic .s	r1, r1, r2
  bic		r1, r10, r5
  bic		r1, r10, r5 .lsl # 2
  bic .s	r1, r10, r5 .lsl # 2
  bx		r14
end-code

code t2Bxx	\ --
  bl		T2back
  bl		T2Fwd
  blx		r5
  cbnz		r5, T2Fwd
  cbz		r5, T2fwd
  bx		r14
end-code

proc T2Fwd	\ -- addr
  bx		r14
end-code

code t2cmn	\ --
  cmn		r10, # $200
  cmn		r1, r2
  cmn		r10, r5
  bx		r14
end-code

code t2cmp	\ --
  cmp		r1, # $AA
  cmp		r10, # $200
  cmp		r1, r2
  cmp		r10, r5
  cmp		r10, r5 .lsl # 2
  bx		r14
end-code

code t2eor	\ --
  eor		r1, r10, # $200
  eor .s	r1, r10, # $200
  eor .s	r1, r2
  eor .s	r1, r1, r2
  eor		r1, r10, r5
  eor		r1, r10, r5 .lsl # 2
  eor .s	r1, r10, r5 .lsl # 2
  bx		r14
end-code

code t2itx
  IT .eq
  nop
  ITT .ne
  nop nop
  ITE .cs
  nop nop
  ITTT .cc
  nop nop nop
  ITET .eq
  nop nop nop
  ITTE .ne
  nop nop nop
  ITEE .eq
  nop nop nop
  ITTTT .ne
  nop nop nop nop
  ITETT .eq
  nop nop nop nop
  ITTET .ne
  nop nop nop nop
  ITEET .eq
  nop nop nop nop
  ITTTE .eq
  nop nop nop nop
  ITETE .eq
  nop nop nop nop
  ITTEE .eq
  nop nop nop nop
  ITEEE .eq
  nop nop nop nop
  bx		r14
end-code

code t2ldcx	\ --
  ldc		10  Cr5, [ r10, # $88 ]
  ldc		10  Cr5, [ r10, # $-88 ] !
  ldc		10  Cr5, [ r10 ], # $88
  ldc		10  Cr5, [ r10 ] {} $55
  ldcl		10  Cr5, [ r10, # $88 ]
  ldcl		10  Cr5, [ r10, # $-88 ] !
  ldcl		10  Cr5, [ r10 ], # $88
  ldcl		10  Cr5, [ r10 ] {} $55
  ldc2		10  Cr5, [ r10, # $88 ]
  ldc2		10  Cr5, [ r10, # $-88 ] !
  ldc2		10  Cr5, [ r10 ], # $88
  ldc2		10  Cr5, [ r10 ] {} $55
  ldc2l		10  Cr5, [ r10, # $88 ]
  ldc2l		10  Cr5, [ r10, # $-88 ] !
  ldc2l		10  Cr5, [ r10 ], # $88
  ldc2l		10  Cr5, [ r10 ] {} $55
  bx		r14
end-code

code t2ldmx	\ --
  ldm		r5 ! { r1-r3 }
  ldm		r12 ! { r1-r3 }
  ldm		r12, { r1-r3 }
  ldm		r12, { r1-r3 pc }
\  ldm		r12, { r1-r3 lr pc }	\ UNPREDICTABLE
  ldmdb		r12 ! { r1-r3 }
  ldmdb		r12, { r1-r3 }
  ldmdb		r12, { r1-r3 pc }
  bx		r14
end-code

align l: ldrBack
  $aaaa5555 ,
code t2ldrx	\ --
  ldr		r5, [ r3, # $18 ]	\ A6-88 T1
  ldr		r4, [ SP, # $20 ]	\ A6-88 T2
  ldr		r10, [ r12, # $FF0 ]	\ A6-88 T3
  ldr		r10, [ r6, # $-24 ]	\ A6-88 T4
  ldr		r10, [ r6 ], # $24	\ A6-88 T4
  ldr		r10, [ r6 ], # $-24	\ A6-88 T4
  ldr		r10, [ r6, # $24 ] !	\ A6-88 T4
  ldr		r10, [ r6, # $-24 ] !	\ A6-88 T4
  ldr		r5, ldrFwd		\ A6-90 T1
  ldr		r5, ldrBack		\ A6-90 T2
  ldr		r5, [ r3, r4 ]		\ A6-92 T1
  ldr		r5, [ r10, r4 ]		\ A6-92 T2
  ldr		r5, [ r10, r1 .lsl # 2 ]

  ldrb		r5, [ r3, # $18 ]	\ A6-94 T1
  ldrb		r10, [ r12, # $FF0 ]	\ A6-94 T2
  ldrb		r10, [ r6, # $-24 ]	\ A6-94 T3
  ldrb		r10, [ r6 ], # $24
  ldrb		r10, [ r6 ], # $-24
  ldrb		r10, [ r6, # $24 ] !
  ldrb		r10, [ r6, # $-24 ] !
  ldrb		r5, ldrFwd
  ldrb		r5, ldrBack
  ldrb		r5, [ r3, r4 ]		\ A6-98 T1
  ldrb		r5, [ r10, r4 ]		\ A6-98 T2
  ldrb		r5, [ r10, r1 .lsl # 2 ]

  ldrbt		r5, [ r3, # $18 ]

  ldrd		r10, r11, [ r6, # $-24 ]	\ A6-102 T1
  ldrd		r10, r11, [ r6 ], # $24
  ldrd		r10, r11, [ r6 ], # $-24
  ldrd		r10, r11, [ r6, # $24 ] !
  ldrd		r10, r11, [ r6, # $-24 ] !
  ldrd		r10, r11, ldrFwd
  ldrd		r10, r11, ldrBack

  ldrex		r5, [ r10, # $88 ]
  ldrexb	r5, [ r10 ]
  ldrexh	r5, [ r10 ]

  ldrh		r5, [ r3, # $18 ]	\ A6-110 T1
  ldrh		r10, [ r12, # $FF0 ]	\ A6-110 T2
  ldrh		r10, [ r6, # $-24 ]	\ A6-110 T3
  ldrh		r10, [ r6 ], # $24
  ldrh		r10, [ r6 ], # $-24
  ldrh		r10, [ r6, # $24 ] !
  ldrh		r10, [ r6, # $-24 ] !
  ldrh		r5, ldrFwd
  ldrh		r5, ldrBack
  ldrh		r5, [ r3, r4 ]		\ A6-112 T1
  ldrh		r5, [ r10, r4 ]		\ A6-112 T1
  ldrh		r5, [ r10, r1 .lsl # 2 ]

  ldrht		r5, [ r3, # $18 ]

  ldrsb		r10, [ r12, # $FF0 ]	\ A6-118 T1
  ldrsb		r10, [ r6, # $-24 ]	\ A6-118 T2
  ldrsb		r10, [ r6 ], # $24
  ldrsb		r10, [ r6 ], # $-24
  ldrsb		r10, [ r6, # $24 ] !
  ldrsb		r10, [ r6, # $-24 ] !
  ldrsb		r5, ldrFwd
  ldrsb		r5, ldrBack
  ldrsb		r5, [ r3, r4 ]		\ A6-122 T1
  ldrsb		r5, [ r10, r4 ]		\ A6-122 T2
  ldrsb		r5, [ r10, r1 .lsl # 2 ]

  ldrsbt	r5, [ r3, # $18 ]

  ldrsh		r10, [ r12, # $FF0 ]	\ A6-126 T1
  ldrsh		r10, [ r6, # $-24 ]	\ A6-126 T2
  ldrsh		r10, [ r6 ], # $24
  ldrsh		r10, [ r6 ], # $-24
  ldrsh		r10, [ r6, # $24 ] !
  ldrsh		r10, [ r6, # $-24 ] !
  ldrsh		r5, ldrFwd
  ldrsh		r5, ldrBack
  ldrsh		r5, [ r3, r4 ]		\ A6-130 T1
  ldrsh		r5, [ r10, r4 ]		\ A6-130 T2
  ldrsh		r5, [ r10, r1 .lsl # 2 ]

  ldrsht	r5, [ r3, # $18 ]

  ldrt		r5, [ r3, # $18 ]

  bx		r14
end-code
align l: ldrFwd
  $5555aaaa ,

code t2shifts	\ --
  asr .s	r1, r2, # $13
  asr .s	r1, r10, # $12
  asr .s	r1, r2
  asr .s	r1, r1, r2
  asr .s	r1, r10, r5
  asr		r1, r10, r5

  lsl .s	r1, r2, # $010
  lsl		r1, r10, # $018
  lsl .s	r1, r10, # $018
  lsl .s	r1, r2
  lsl .s	r1, r1, r2
  lsl .s	r10, r1, r2
  lsl		r10, r1, r2

  lsr .s	r1, r2, # $010
  lsr		r1, r10, # $018
  lsr .s	r1, r10, # $018
  lsr .s	r1, r2
  lsr .s	r1, r1, r2
  lsr .s	r10, r1, r2
  lsr		r10, r1, r2

  ror .s	r1, r2, # $010
  ror		r1, r10, # $018
  ror .s	r1, r10, # $018
  ror .s	r1, r2
  ror .s	r1, r1, r2
  ror .s	r10, r1, r2
  ror		r10, r1, r2

  rrx		r10, r5

  bx		r14
end-code

code t2copro	\ --
  cdp		#15 4 Cr5, Cr1, Cr10, 6
  cdp2		#15 4 Cr5, Cr1, Cr10, 6
  mcr		#15 4 r10, Cr4, Cr5, 6
  mcr2		#15 4 r10, Cr4, Cr5, 6
  mcrr		#15 4 r10, r11, Cr5
  mcrr2		#15 4 r10, r11, Cr5
  mrc		#15 4 r10, Cr4, Cr5, 6
  mrc2		#15 4 r10, Cr4, Cr5, 6
  mrrc		#15 4 r10, r11, Cr5
  mrrc2		#15 4 r10, r11, Cr5
  mrs		r10, $0F
  msr		$0E r10
  bx		r14
end-code

code t2muldiv	\ --
  mla		r10, r1, r2, r5
  mls		r10, r1, r2, r5
  mul .s	r1, r2
  mul .s	r1, r1, r2
  mul		r1, r2, r5

  sdiv		r3, r4, r5

  smlal		r8, r9, r1, r2
  smull		r8, r9, r1, r2

  udiv		r3, r4, r5
  umlal		r8, r9, r1, r2
  umull		r8, r9, r1, r2

  bx		r14
end-code

code t2mov	\ --
  mov .s	r1, # $88
  mov 		r1, # $200
  mov .s	r1, # $200
  mov		r10, # $5678

  mov		r8, r9
  mov .s	r1, r2		\ 2 low regs = enc T2, valid always if S=1
  mov		r1, r2		\ 2 low regs = enc T1, valid always if S=0 and Thumb-2
  mov .w	r10, r11
  mov .s .w	r10, r11

  mov		r10, r5 .asr # 8
  mov		r10, r5 .asr r9
  mov		r10, r5 .lsl # 8
  mov		r10, r5 .lsl r9
  mov		r10, r5 .lsr # 8
  mov		r10, r5 .lsr r9
  mov		r10, r5 .ror # 8
  mov		r10, r5 .ror r9
  mov		r10, r5 .rrx

  mov		r10, # $5678
  movt		r10, # $1234

  mvn .s	r1, # $AA
  mvn .s	r1, r2
  mvn		r10, r5 .asr # 8
  mvn		r10, r5 .lsl # 8
  mvn		r10, r5 .lsr # 8
  mvn		r10, r5 .ror # 8
  mvn		r10, r5 .rrx

  mvn		r1, r2

  bx		r14
end-code

code t2orn	\ --
  orn		r1, r10, # $200
  orn .s	r1, r10, # $200
  orn .s	r1, r2
  orn .s	r1, r1, r2
  orn		r1, r10, r5
  orn		r1, r10, r5 .lsl # 2
  orn .s	r1, r10, r5 .lsl # 2
  bx		r14
end-code

code t2orr	\ --
  orr		r1, r10, # $200
  orr .s	r1, r10, # $200
  orr .s	r1, r2
  orr .s	r1, r1, r2
  orr		r1, r10, r5
  orr		r1, r10, r5 .lsl # 2
  orr .s	r1, r10, r5 .lsl # 2
  bx		r14
end-code

code t2pop/push
  pop		{ r1-r3, r6, r7 }
  pop		{ r1-r3, r6, r7, pc }
  pop		{ r7-r11, lr }
\  pop		{ r7-r11, lr, pc }
  pop		{ r7-r11, pc }
  push		{ r1-r3, r6, r7 }
  push		{ r1-r3, r6, r7, lr }
  pop		{ r7-r11, lr }
  bx		r14
end-code

code t2rsb	\ --
  neg .s	r1
  neg .s	r1, r2

  rsb .s	r1, r2, # 0
  rsb .s	r10, r2, # 0
  rsb		r10, r2, # $55
  rsb		r1, r10, r5 .lsl # 2
  rsb .s	r1, r10, r5 .lsl # 2

  bx		r14
end-code

code t2sbc	\ --
  sbc		r1, r10, # $200
  sbc .s	r1, r10, # $200
  sbc .s	r1, r2
  sbc .s	r1, r1, r2
  sbc		r1, r10, r5
  sbc		r1, r10, r5 .lsl # 2
  sbc .s	r1, r10, r5 .lsl # 2
  bx		r14
end-code

code t2stcx	\ --
  stc		10  Cr5, [ r10, # $88 ]
  stc		10  Cr5, [ r10, # $-88 ] !
  stc		10  Cr5, [ r10 ], # $88
  stc		10  Cr5, [ r10 ] {} $55
  stcl		10  Cr5, [ r10, # $88 ]
  stcl		10  Cr5, [ r10, # $-88 ] !
  stcl		10  Cr5, [ r10 ], # $88
  stcl		10  Cr5, [ r10 ] {} $55
  stc2		10  Cr5, [ r10, # $88 ]
  stc2		10  Cr5, [ r10, # $-88 ] !
  stc2		10  Cr5, [ r10 ], # $88
  stc2		10  Cr5, [ r10 ] {} $55
  stc2l		10  Cr5, [ r10, # $88 ]
  stc2l		10  Cr5, [ r10, # $-88 ] !
  stc2l		10  Cr5, [ r10 ], # $88
  stc2l		10  Cr5, [ r10 ] {} $55
  bx		r14
end-code

code t2stmx	\ --
  stm		r5 ! { r1-r3 }
  stm		r12 ! { r1-r3 }
  stm		r12, { r1-r3 }
  stm		r12, { r1-r3 lr }
  stmia		r5 ! { r1-r3 }
  stmia		r12 ! { r1-r3 }
  stmia		r12, { r1-r3 }
  stmia		r12, { r1-r3 lr }
  stmea		r5 ! { r1-r3 }
  stmea		r12 ! { r1-r3 }
  stmea		r12, { r1-r3 }
  stmea		r12, { r1-r3 lr }

  stmdb		r12 ! { r1-r3 }
  stmdb		r12, { r1-r3 }
  stmdb		r12, { r1-r3 lr }
  stmfd		r12 ! { r1-r3 }
  stmfd		r12, { r1-r3 }
  stmfd		r12, { r1-r3 lr }
  bx		r14
end-code

code t2strx	\ --
  str		r5, [ r3, # $18 ]	\ A6-222 T1
  str		r4, [ SP, # $20 ]	\ A6-222 T2
  str		r10, [ r12, # $FF0 ]	\ A6-222 T3
  str		r10, [ r6, # $-24 ]	\ A6-222 T4
  str		r10, [ r6 ], # $24	\ A6-222 T4
  str		r10, [ r6 ], # $-24	\ A6-222 T4
  str		r10, [ r6, # $24 ] !	\ A6-222 T4
  str		r10, [ r6, # $-24 ] !	\ A6-222 T4
  str		r5, [ r3, r4 ]		\ A6-224 T1
  str		r5, [ r10, r4 ]		\ A6-224 T2
  str		r5, [ r10, r1 .lsl # 2 ]

  strb		r5, [ r3, # $18 ]
  strb		r10, [ r12, # $FF0 ]
  strb		r10, [ r6, # $-24 ]
  strb		r10, [ r6 ], # $24
  strb		r10, [ r6 ], # $-24
  strb		r10, [ r6, # $24 ] !
  strb		r10, [ r6, # $-24 ] !
  strb		r5, [ r3, r4 ]
  strb		r5, [ r10, r4 ]
  strb		r5, [ r10, r1 .lsl # 2 ]

  strbt		r5, [ r3, # $18 ]

  strd		r10, r11, [ r6, # $-24 ]
  strd		r10, r11, [ r6 ], # $24
  strd		r10, r11, [ r6 ], # $-24
  strd		r10, r11, [ r6, # $24 ] !
  strd		r10, r11, [ r6, # $-24 ] !

  strex		r4, r5, [ r10, # $88 ]
  strexb	r4, r5, [ r10 ]
  strexh	r4, r5, [ r10 ]

  strh		r5, [ r3, # $18 ]
  strh		r10, [ r12, # $FF0 ]
  strh		r10, [ r6, # $-24 ]
  strh		r10, [ r6 ], # $24
  strh		r10, [ r6 ], # $-24
  strh		r10, [ r6, # $24 ] !
  strh		r10, [ r6, # $-24 ] !
  strh		r5, [ r3, r4 ]
  strh		r5, [ r10, r4 ]
  strh		r5, [ r10, r1 .lsl # 2 ]

  strht		r5, [ r3, # $18 ]

  strt		r5, [ r3, # $18 ]

  bx		r14
end-code

code t2sub	\ --
  sub .s	r1, r2, # 7
  sub .s	r1, # $55
  sub .s	r1, r1, # $55
  sub		r1, r10, # $300
  sub .s	r1, r10, # $300
  sub 		r1, r10, # $765
  sub .s	r1, r2, r3
  sub		r10, r1
  sub		r10, r10, r1
  sub .s	r1, r10, r5
  sub		r1, r10, r5 .lsl # 2

  sub		sp, sp, # $100
  sub		r1, SP, # $AA
  sub		sp, sp, # $55
  sub .s	r1, sp, # $400
  sub		r1, sp, # $888

  sub		r10, sp, r10
  sub		sp, r2
  sub		sp, sp, r2
  sub		r5, sp, r10 .lsl # 1
  bx		r14
end-code

code t2teq	\ --
  teq		r1, # $AA
  teq		r10, # $200
  teq		r1, r2
  teq		r10, r5
  teq		r10, r5 .lsl # 2
  bx		r14
end-code

code t2tst	\ --
  tst		r1, # $AA
  tst		r10, # $200
  tst		r1, r2
  tst		r10, r5
  tst		r10, r5 .lsl # 2
  bx		r14
end-code

((
code t2ADD	\ --
  bx		r14
end-code

))

l: miscback
  $1234:5678 ,

code t2misc	\ --
  bfc		r5, # 10 # 9
  bfi		r5, r10, # 10 # 9
  bkpt		# $55
  clrex
  clz		r10, r5
  cps		.ie .f .i
  cps		.id .i
  dbg		# 15
  dmb		# 10
  dsb		# 8
  isb		# 9
  nop
  nop .n
  nop .w

  pld		[ r10, # $FF0 ]
  pld		[ r2, # $-88 ]
  pld		miscback
  pld		miscfwd
  pld		[ r10, r5 .lsl # 2 ]

  pldw		[ r10, # $FF0 ]
  pldw		[ r2, # $-88 ]

  pli		[ r10, # $FF0 ]
  pli		[ r2, # $-88 ]
  pli		miscback
  pli		miscfwd
  pli		[ r10, r5 .lsl # 2 ]

  rbit		r10, r5
  rev		r1, r2
  rev		r10, r5
  rev16		r1, r2
  rev16		r10, r5
  revsh		r1, r2
  revsh		r10, r5

  sbfx		r5, r8, # 10 # 9
  sev
  ssat		r10, # $9 r1
  ssat		r10, # $9 r1 .lsl $5
  ssat		r10, # $9 r1 .asr $5
  svc		# $88
  sxtb		r1, r2
  sxtb		r8, r9
  sxtb		r8, r10, # 2
  sxtb		r8, r10, # #16
  sxth		r1, r2
  sxth		r8, r9
  sxth		r8, r10, # 16

  tbb		[ r1, r2 ]
  tbh		[ r10, r11 ]

  ubfx		r5, r8, # 10 # 9
  usat		r10, # $9 r1
  usat		r10, # $9 r1 .lsl $5
  usat		r10, # $9 r1 .asr $5
  sxtb		r1, r2
  uxtb		r8, r9
  uxtb		r8, r10, # 2
  uxtb		r8, r10, # #16
  uxth		r1, r2
  uxth		r8, r9
  uxth		r8, r10, # 16

  wfe
  wfe .n
  wfe .w
  wfi
  wfi .n
  wfi .w
  yield
  yield .n
  yield .w

  bx		r14
end-code

l: miscfwd
  $8765:4321 ,

((
: tt 1 2 3 ;

CODE DU<  \ ud1 ud2 -- flag        AC0 CD DU<
  ldmfd psp ! { r0-r2 }                 \ r0=d2 low, r1=d1 high, r2=d1 low
  sub .s r2, r2, r0                     \ d1l - d2l
  sbc .s r1, r1, tos                    \ d1h - d2h
  ite .cc
  mvn tos, # 0
  mov tos, # 0
  bx r14
end-code

CODE D= 	\ xd1 xd2 -- flag
\ *G Return TRUE if the two double numbers are equal.
  ldmfd psp ! { r0-r2 }                 \ r0=d2 low, r1=d1 high, r2=d1 low
  sub .s r0, r0, r2                     \ d2l - d1l
  sbc .s tos, tos, r1                      \ d2h - d1h
  orr .s r0, r0, tos
  ite .eq
    mvn  tos, # 0                      \ true
    mov  tos, # 0                      \ false
  next,
END-CODE

: d=2  d- or 0=  ;
))

\ =======
\ Tidy up
\ =======

decimal
FINIS

\ ======
\ *> ###
\ ======

