\ ARM/Cortex test control file

((
Copyright (c) 2006, 2009, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20110601 MPE001 Derived from TestCortex.ctl.
))


\ MPE macros
c" ../../ROM/Common"
		setmacro CommonDir	\ where the common code lives
c" ../../ROM/Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ../../ROM/ARM/hardware/LPC210x
		setmacro HWDir		\ where board specific code lives
c" ../../ROM/Examples"
		setmacro ExampleDir	\ Examples
c" ."   	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%/Macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ processor type
\  Hex-I32			\ Generate Intel Hex 32 bit

1 equ CPU=Cortex		\ for coded versions of some words
0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ---------------------------------------------
\ *H Define stack and user area sizes for TASKs
\ ---------------------------------------------

$0100 equ UP-SIZE	\ -- n
\ *G size of each task's user area
$0100 equ SP-SIZE	\ -- n
\ *G size of each task's data stack
$0200 equ RP-SIZE	\ -- n
\ *G size of each task's return stack
up-size rp-size + sp-size +
  equ task-size		\ -- n
\ *G Size of TASK data area
#2 equ sp-guard		\ -- n
\ *G Define the number of cells of guard space at the top of
\ ** the data stack

$0100 equ TIB-LEN	\ -- n
\ *G Terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs	\ -- n
\ *G Number of IRQ stacks, shared by all IRQs (1 min)
1 equ #FIQs	\ -- n
\ *G number of FIQ stacks, shared by all FIQs (1 min)
0 equ #SWIs	\ -- n
\ *G number of SWI nestings permitted. 0 is ok if SWIs are unused


\ *****************
\ Default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ *************
\ *S Memory map
\ *************

  $00000000 $0001FFFF cdata section TestM0M1
\ *G Primary code section in on-chip Flash
  $00100000 $0010CFFF idata section PROGd
\ *G 48k IDATA. This area contains
\ ** initialised data and user applications.
  $0010D000 $0010FFFF udata section PROGu
\ *G 8k UDATA. This area contains unitialised data.

interpreter
: prog TestM0M1 ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ **********************
\ Cortex-M0/M1 test code
\ **********************

log

$100 allot


\ ========
\ Thumb v1
\ ========

proc ProcBack	\ -- addr
  bx		r14
end-code

code Thumb1Ops	\ --
l: labBack
  adc .s	r1, r2

  add .s	r1, r2, # 3
  add .s	r1, # $88
  add .s	r1, r2, r3
  add		r8, r9
  add		r1, pc, # $88
  add		r1, sp, # $88
  add		sp, # $88

  and .s	r1, r2

  asr .s	r1, r2, # $11
  asr .s 	r1, r2
  asr .s        r1, r1, r2

  b .ne		labBack
  b .pl		labFwd
  b		labBack
  b		labFwd

  bic .s 	r1, r2

  bkpt	# $55

  bl	ProcBack
  bl	ProcFwd
\  blx	ProcBack
\  blx	ProcFwd
  blx	r10
  bx	r10

  cmn	r1, r2

  cmp	r1, # $88
  cmp	r1, r2
  cmp	r8, r9

  eor .s r1, r2

  ldmia	r5, ! { r1-r3, r6, r7 }

  ldr	r1, [ r2, # $44 ]
  ldr	r1, [ r2, r3 ]
  ldr	r1, [ PC, # $180 ]
  ldr	r1, [ SP, # $180 ]

  ldrb	r1, [ r2, # $11 ]
  ldrb	r1, [ r2, r3 ]

  ldrh	r1, [ r2, # $12 ]
  ldrb	r1, [ r2, r3 ]

  ldrsb	r1, [ r2, r3 ]

  ldrsh	r1, [ r2, r3 ]

  lsl .s	r1, r2, # $11
  lsl .s	r1, r2

  lsr .s	r1, r2, # $11
  lsr .s	r1, r2

  mov .s	r1, # $88
  mov .s	r1, r2		\ 2 low regs = enc T2, valid always if S=1
  mov		r1, r2		\ 2 low regs = enc T1, valid always if S=0 and Thumb-2
  mov		r8, r9

  mul .s	r1, r2

  mvn .s	r1, r2

  neg .s	r1, r2

  nop

  orr .s	r1, r2

  pop   { pc }
  pop	{ r1-r3, r6, r7 }
  pop	{ r1-r3, r6, r7, pc }

  push  { lr }
  push	{ r1-r3, r6, r7 }
  push	{ r1-r3, r6, r7, lr }

  ror .s	r1, r2

  sbc .s	r1, r2

  stmia	r5, ! { r1-r3, r6, r7 }

  str	r1, [ r2, # $44 ]
  str	r1, [ r2, r3 ]
  str	r1, [ SP, # $180 ]

  strb	r1, [ r2, # $11 ]
  strb	r1, [ r2, r3 ]

  strh	r1, [ r2, # $12 ]
  strh	r1, [ r2, r3 ]

  sub .s	r1, r2, # 3
  sub .s	r1, # $88
  sub .s	r1, r2, r3
  sub	sp, # $88

  swi	# $88

  tst	r1, r2

l: labFwd
  bx	r14
end-code

proc ProcFwd	\ -- addr
  bx	r14
end-code


\ ========
\ Thumb v2 extensions
\ ========

align l: AdrBack
  $5555AAAA ,
code t2ADR	\ --
  adr           r2, AdrFwd
  adr		r1, $ $40 + aligned
  adr		r1, $ $200 + aligned
  bx		r14
end-code
align l: AdrFwd
  $AAAA5555 ,

proc T2Back	\ -- addr
  bx		r14
end-code

code T2B	\ --
  b		T2Back
  b		T2Fwd
  b .ne		T2Back
  b .eq		T2Fwd
  bx		r14
end-code

code t2BIC	\ --
  bic .s	r1, r2
  bic .s	r1, r1, r2
  bx		r14
end-code

code t2Bxx	\ --
  bl		T2back
  bl		T2Fwd
  blx		r5
  bx		r14
end-code

proc T2Fwd	\ -- addr
  bx		r14
end-code

code t2cmn	\ --
  cmn		r1, r2
  bx		r14
end-code

code t2cmp	\ --
  cmp		r1, # $AA
  cmp		r1, r2
  cmp		r10, r5
  cmp		r10, r11
  bx		r14
end-code

code t2ldmx	\ --
  ldm		r5 ! { r1-r3 }
\  ldm		r2, { r1-r3 }
\  ldm		r5, { r1-r3 pc }
  bx		r14
end-code

align l: ldrBack
  $aaaa5555 ,
code t2ldrx	\ --
  ldr		r5, [ r3, # $18 ]	\ A6-88 T1
  ldr		r4, [ SP, # $20 ]	\ A6-88 T2
  ldr		r5, ldrFwd		\ A6-90 T1

  ldrb		r5, [ r3, # $18 ]	\ A6-94 T1
  ldrb		r5, [ r3, r4 ]		\ A6-98 T1

  ldrh		r5, [ r3, # $18 ]	\ A6-110 T1
  ldrh		r5, [ r3, r4 ]		\ A6-112 T1

  ldrsb		r5, [ r3, r4 ]		\ A6-122 T1

  ldrsh		r5, [ r3, r4 ]		\ A6-130 T1

  bx		r14
end-code
align l: ldrFwd
  $5555aaaa ,

code t2copro	\ --
  mrs		r10, $0F
  msr		$0E r10
  bx		r14
end-code

code t2muldiv	\ --
  mul .s	r1, r2
  mul .s	r1, r1, r2
\  mul		r1, r2, r5

  bx		r14
end-code

code t2mov	\ --
  mov .s	r1, # $88
  mov		r8, r9
  mov .s	r1, r2		\ 2 low regs = enc T2, valid always if S=1
  mov		r1, r2		\ 2 low regs = enc T1, valid always if S=0 and Thumb-2

  mov .s	r4, r5 .asr # 8
  mov .s	r4, r4 .asr r5

  mvn .s	r1, r2

  bx		r14
end-code

code t2neg	\ --
  neg .s	r1
  neg .s	r1, r2
  bx		r14
end-code

code t2orr	\ --
  orr .s	r1, r2
  orr .s	r1, r1, r2
  bx		r14
end-code

code t2pop/push	\ --
  pop		{ r1-r3, r6, r7 }
  pop		{ r1-r3, r6, r7, pc }
  push		{ r1-r3, r6, r7 }
  push		{ r1-r3, r6, r7, lr }
  bx		r14
end-code

code t2rev	\ --
  rev		r1, r2
  rev16		r1, r2
  revsh		r1, r2
  bx		r14
end-code

code t2rsb	\ --
  rsb .s	r1, r2, # 0
  bx		r14
end-code

code t2sbc	\ --
  sbc .s	r1, r2
  sbc .s	r1, r1, r2
  bx		r14
end-code

code t2stmx	\ --
  stm		r5 ! { r1-r3 }
  stmia		r5 ! { r1-r3 }
  stmea		r5 ! { r1-r3 }
  bx		r14
end-code

code t2strx	\ --
  str		r5, [ r3, # $18 ]	\ A6-222 T1
  str		r4, [ SP, # $20 ]	\ A6-222 T2
  str		r5, [ r3, r4 ]		\ A6-224 T1

  strb		r5, [ r3, # $18 ]
  strb		r5, [ r3, r4 ]

  strh		r5, [ r3, # $18 ]
  strh		r5, [ r3, r4 ]
  bx		r14
end-code

code t2sub	\ --
  sub .s	r1, r2, # 7
  sub .s	r1, # $55
  sub .s	r1, r1, # $55
  sub .s	r1, r2, r3

  sub		sp, sp, # $100
  sub		sp, # 88
  bx		r14
end-code

l: miscback
  $1234:5678 ,

code t2misc	\ --
  bkpt		# $55

\  cps		.ie .f .i
  cps		.ie .i
  cps		.id .i
  dbg		# 15
  dmb		# 10
  dsb		# 8
  isb		# 9
  nop
  nop .n

  sev

  svc		# $88

  sxtb		r1, r2

  sxth		r1, r2

  uxtb		r1, r2

  uxth		r1, r2

  wfe
  wfe .n
\  wfe .w
  wfi
  wfi .n
\  wfi .w
  yield
  yield .n
\  yield .w

  bx		r14
end-code

l: miscfwd
  $8765:4321 ,


\ ****************
\ High level tests
\ ****************

\ +debug-opt	\ tutn debugging on

: dup dup  ;

: over over  ;

: swap swap  ;

: rot rot  ;

: -rot -rot  ;

: drop drop  ;

: 2drop drop drop  ;

: 3drop drop drop drop  ;

: 4drop drop drop drop drop  ;

: bounds over + swap  ;

: dig0  dup 9  ;

: dig1  dup 9 >  ;

: dig2  dup 9 > if  7 +  endif  ;

: dig3  9 over < if  7 +  endif  ;

: m1 2dup > ;
: m2 2dup > if then ;
: m3 2dup > if nip then ;
: m4 2dup > if nip exit then ;

: min
  2dup >
  if  nip exit  then
  drop  ;

: max
  2dup <
  if  nip exit  then
  drop  ;

: min2
  2dup >
  if  swap  then
  drop  ;

: ?dup ?dup ;

: tq1 ?dup if drop endif ;

: tq2 ?dup 0= if  7  endif  ;

: + + ;

: - - ;

: 4-a 4- ;

: 4-b 4 - ;

: n- 10 - ;

: t1 2dup @  ;
\ ((
: t2 2dup 2@ ;
: t3 rot 2!  ;
: t4a  +!  ;
: t4b 4 swap +!  ;
: t4c 6 swap +!  ;
: t4d dup @ rot + swap !  ;     \ high level version of +!

: t4 incr  ;
: t5 decr  ;
: t6 2dup !  ;
: t7  >r  r@ drop  r> drop  ;
\ ))

CODE (DO)	\ limit index --
\ *G The run time action of *\fo{DO} compiled on the target.
\ ** INTERNAL.
  ldmia   psp ! { r1 }			\ get limit
L: PDO
  mov     r5, link			\ return address
  sub .s  r5, r5, # 1			\ -1, remove T bit
  ldr     r2, [ r5, # 0 ]		\ get LEAVE address
  add .s  r5, r5, # 5			\ -1 + 5 = +4, step over branch address
  mov     link, r5
  mov .s  r5, # 1
  lsl .s  r5, r5, # #31			\ $80000000
  add .s  r1, r1, r5
  sub .s  r0, tos, r1

  push    { r0-r2 }			\ push LEAVE then limit, then index on ret. stack
  ldmia   psp ! { tos }			\ update tos
  next,
END-CODE

CODE (?DO) 	\ limit index --
\ *G The run time action of *\fo{?DO} compiled on the target.
\ ** INTERNAL.
  ldmia   psp ! { r1 }			\ get limit
  cmp     r1, tos			\ check not equal
  b .ne   pdo                           \ take DO ?
  ldmia   psp ! { tos }			\ update tos
  mov     r5, link
  sub .s  r5, r5, # 1			\ clear T bit
  ldr     r0, [ r5, # 0 ]		\ and take leave branch
  bx      r0
END-CODE

: CONSTANT 	\ x "<spaces>name" -- ; Exec: -- x
\ *G Create a new *\fo{CONSTANT} called *\fo{name} which has the
\ ** value *\i{x}. When *\fo{NAME} is executed *\i{x} is returned.
  CREATE -4 allot , ;CODE
  sub .s  psp, psp, # 4
  str     tos, [ psp ]			\ save TOS
  mov     r5, link			\ return address
  sub .s  r5, r5, # 1			\ remove T bit
  ldr     tos, [ r5 ]			\ get const val/addr from afer CALL
  pop     { pc }
END-CODE

: VARIABLE 	\ "<spaces>name" -- ; Exec: -- a-addr
\ *G Create a new variable called *\fo{name}. When *\fo{name} is
\ ** executed the address of the data-cell is returned for use
\ ** with *\fo{@} and *\fo{!} operators.
  CREATE  0 ,  ;CODE
  sub .s  psp, psp, # 4
  str     tos, [ psp ]			\ save TOS
  mov     r5, link			\ return address
  sub .s  r5, r5, # 1			\ remove T bit
  ldr     tos, [ r5 ]			\ get const val/addr from afer CALL
  pop     { pc }
END-CODE

: USER 		\ u "<spaces>name" -- ; Exec: -- addr ; SFP009
\ *G Create a new *\fo{USER} variable called *\fo{name}. The *\i{u}
\ ** parameter specifies the index into the user-area table at which
\ ** to place the* data. *\fo{USER} variables are located in a
\ ** separate area of memory for each task or interrupt. Use in
\ ** the form:
\ *C   $400 USER TaskData
  CREATE  -4 allot ,  ;CODE		\ replce data address by offset
  sub .s  psp, psp, # 4
  str     tos, [ psp ]			\ save TOS
  mov     r5, link			\ return address
  sub .s  r5, r5, # 1			\ remove T bit
  ldr     r0, [ r5 ]			\ get user offset
  mov     r1, up			\ get user base address
  add .s  tos, r1, r0			\ add user pointer to user offset
  pop     { pc }
end-code

: t8 5 0 do  i drop  loop  ;
: t9 6 0 ?do  i drop  5 +loop  ;

: t10  rot drop  ;

: t11  swap 1 + nip  ;

: t12  2swap  ;
: t13  #-256 + ;

#200 user t14user
: t14a t14user  ;
: t14b t14user @  ;
: t14  t14user @ 5 + ;

#600 user txuser
: tx txuser  ;

: t15  swap u2/ u2/ u2/  ;

variable t16var
create t16cr  0 , 0 , 0 ,
: t16
  $55 t16var c!
  $AA t16cr c!
  $AA t16cr 8 + c!
;

: t17           \ shuffle test
  swap 8 *  ;

: t18
  xor
  swap xor
  2dup xor
  2dup swap xor
;

: t19           \ literal tests
  $1234 dup 1- +
;

0 constant ffx
: t20   2drop ffx ;

: t21   rot dup  ;

: t22  r> @  ;

: COMPARE       \ c-addr1 u1 c-addr2 u2  --  n ; compare strings
  2swap				\ a2 l2 a1 l1 --
  rot                           \ a2 a1 l1 l2 --
  2dup - >r                     \ a2 a1 l1 l2 --
  min 0                         \ a2 a1  l1/l2 0
  ?do
    swap 1+  swap 1+            \ step to next address
    over 1- c@  over 1- c@ <>   \ a2 a1 a2<>a1 --
    ?leave                      \ until different
  loop                          \ or until done
  swap 1- c@  swap 1- c@  -     \ last two chars
  ?dup                          \ see if same
  if
    0<                          \ not same, return indicator
    if  +1  else  -1  then
    r> drop
  else
    r> dup                      \ strings =, see if same length
    if
      0<                        \ not, return indicator
      if  -1  else  +1  then
    then
  then
;

: comparev       \ c-addr1 u1 c-addr2 u2 -- n                    17.6.1.0935
  rot swap                      \ c-addr1 c-addr2 u1 u2
  2dup - >r min                 \ c-addr1 c-addr2 minlen -- R: lendiff? --

  begin
    dup
  while
    -rot over c@ over c@ -      \ length c-addr1 c-addr2 (char1-char2)

    ?dup if                     \ If chars are different
        r> drop >r              \  replace lendiff result with error code
        drop 0                  \  and put 0 on TOS (make len==0 at BEGIN)
    else                        \ otherwise
        1+ swap 1+ swap rot 1-  \ increment addresses and decrement length
    then

  repeat

  drop 2drop                    \ remove addresses and null count from stack
                                \ -- ; R: result --
  r> dup if 0< 1 or then        \ make nice flag, 0 becomes 0, -ve becomes -1
                                \            and  +ve becomes 1
;

: compares      \ c-addr1 u1 c-addr2 u2 -- n                    17.6.1.0935
  rot swap                      \ c-addr1 c-addr2 u1 u2
  2dup - >r min -rot            \ minlen c-addr1 c-addr2 -- ; R: lendiff? --

  begin
    2 pick
  while
    over c@ over c@ -		\ -- length c-addr1 c-addr2 (char1-char2)

    ?dup if                     \ If chars are different
                                \ minlen c-addr1 c-addr2 diff -- ; R: lendiff? --
        r> drop >r              \  replace lendiff result with error code
        drop  0 -rot            \  and put 0 as len (make len==0 at BEGIN)
    else                        \ otherwise
        1+ swap 1+ swap         \ increment addresses
        rot 1- -rot             \ and decrement length
    then

  repeat

  drop 2drop                    \ remove addresses and null count from stack
                                \ -- ; R: result --
  r> dup if 0< 1 or then        \ make nice flag, 0 becomes 0, -ve becomes -1
                                \            and  +ve becomes 1
;

: compareh	\ c-addr1 u1 c-addr2 u2 -- n
  rot 2dup min
  0 ?do
    2over i + c@
    swap i + c@
    2dup = if
      2drop
    else
      2swap 2drop leave
    then
  loop
  2swap 2drop  swap -
  -1 max  1 min
;

: testpick1
  over 3 pick  ;

: testpick2
  dup 1+
  3 pick
  3 pick
;

create tc1

: tc2 tc1  ;

: (")           \ -- addr ; in-line string follows caller, and skip it
  r>                                    \ return address to caller
  r> dup count + >r                     \ caller's return address
  swap >r                               \ restore return address
;

: t23  lshift  ;
: t24  0 ror  ;
: t25  3 arshift  ;
: t26  rot rshift  ;

\ *********************
\ Guarded shuffle tests
\ *********************

variable a
variable b

: ts1   1 2 3 4 5 a @ b @ > if then ;


\ =======
\ Tidy up
\ =======

decimal
FINIS

\ ======
\ *> ###
\ ======

