\ VFXhead.fth - new style header construction

((
Copyright (c) 1997-2004, 2006, 2013, 2014, 2018
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone number is:
  011 44 23 8063 1441


To do
=====

Change history
==============
20180309 SFP006 Updated for new NDCS target header bit.
                Control byte changes to 2 bits set. $0F0 -> $0C0
20140807 SFP004 Set host variable when VFX headers are installed.
20120405 SFP003 ARM/Cortex conversion
20060930 SFP002 Changed Hash function to CRC16
20040311 SFP001 Prepared for new VFX header to come.
99/09/07 SJC000 header to support SAME alignment as target code.
))


\ If you change the hash function, you MUST also change the
\ hash functions in the cross compiler VfxHead.fth, the target
\ files KERNEL.FTH and REQDCODE.FTH to be the same.
((
: VFXhash	\ caddr len -- n
  swap c@ +				\ count & first char ; SFP024
;

: VFXhash	\ caddr len -- hash
  tuck  over c@ -rot			\ -- len first caddr len
  + 1- c@				\ -- len first last
  + +					\ -- hash
;

: VFXhash	\ caddr len -- hash
  tuck  over c@ 1 lshift -rot		\ -- len first caddr len
  + 1- c@ 2 lshift			\ -- len first last
  + +					\ -- hash
;

: VFXhash	\ caddr len -- hash
  0 -rot  bounds
  do  1 lshift  i c@ +  loop
;

: VFXhash	\ caddr len -- hash
  dup 3 and 6 lshift -rot	\ -- len' caddr len
  over c@ 3 lshift -rot		\ -- len' first' caddr len
  + 1 - c@			\ -- len' first' last
  + +				\ -- hash
;

: VFXhash	\ caddr len -- hash
  tuck					\ -- len caddr len
  over c@ 2 lshift -rot			\ -- len first caddr len
  + 1- c@ 4 lshift			\ -- len first last
  + +					\ -- hash
;
))

\ $8FDB constant CrcPoly
 $A001 constant CrcPoly
\ $013C constant CrcPoly

create H-CRCTBL	\ -- addr ; CRC table
  #256 2* ALLOT

: INIT-CRC16-TABLE	\ -- ; fill CRC table
  #256 0 DO
    I  8 0 DO
      DUP 1 AND  >R
      1 RSHIFT  R>
      IF  CrcPoly XOR  THEN
    LOOP
    I 2*  H-CRCTBL + W!
  LOOP
;
INIT-CRC16-TABLE

: CRC-16	\ crc byte -- crc'
  OVER  XOR  255 AND  2* H-CRCTBL + W@  SWAP  8 RSHIFT  XOR
;

: VFXhash	\ caddr len -- hash
  dup 3 lshift -rot		\ crc caddr len
  bounds ?do			\ -- crc
    i c@ crc-16
  loop
  $FFFF and
;

: VFXthread,(t)                 \ --
  here count VFXhash		\ form hash code
  cur-#threads(t) 1- and cell*  \ mask to get hash code
  cur-voc(s) t-nfa+ +           \ index into nfas
  dup @(s) ,c(t)                \ lay down old value
  chere(t) swap !(s)            \ and update nfas table
;

: code-align(t) \ -- ; Perform CODE section alignment (use $CC for padding)
  begin
    chere(t) 3 and		\ align to 4 byte boundary
  while
    $CC c,c(t)			\ SFP003
  repeat                        \ Aligned to CELL (using INT3 for padding)
;

: data-align(t) \ -- ; As code-align(t) except on cell-1 boundary
  begin
    chere(t) 3 and 3 <>
  while
    $CC c,c(t)
  repeat                        \ Aligned to CELL (using INT3 for padding)
;

: VFXnoheader(t)	\ -- xt ; for :NONAME ; SFP072
  ?0-even  chere(t)
;

((
1 value AliasBit?
: +AliasBit	( -- )  1 to AliasBit?  ;
: -AliasBit?	( -- )  0 to AliasBit?  ;
))

: VFXheader(t)  \ -- ; text at HERE
  here DUP C@ WIDTH(T) @(m) MIN ?dup IF
    code-align(t)                                                               \ SJC000
    VFXthread,(t)               \ lay thread link
    $0C0 c,c(t)			\ control byte (Cx, x=ndcs, alias, smudge, immediate, **, ***)
    chere(t) last(t) !(m)       \ mark NFA
    dup c,c(t)                  \ lay count byte
    0
    DO  1+ DUP C@ C,C(T)  LOOP  \ and text
    DROP
    0 c,c(t)                    \ 0 terminate the name field
\ *** align data as required ***  ?ALIGN-INLINE
    line# @ w,c(t)		\ line number field for LOCATE
    1 w,c(t)                    \ info word for mpe/ccs. Bit0 = optimised? flag
    0 ,c(t)			\ XREF anchor
    chere(t) lastcg(t) !(m)	\ mark code generator
    0 ,c(t)                     \ holds address of code generator
    0 ,c(t)                     \ holds length of generated code
    code-align(t)
  ELSE
    drop  \ data-align(t)
    chere(t) lastcg(t) !(m)	\ mark code generator
    0 ,c(t)                     \ holds address of code generator
    0 ,c(t)                     \ holds length of generated code
    code-align(t)
  ENDIF
  DefnType off
;

: set-compiler(t)	\ xt(t) --
  lastcg(t) @(m) dup >r !c(t)  0 r> cell(t) + !c(t)
;

: its-xt	\ -- xt ; Get XT of last definition
  'latest(s) @(m) value+ @(s)
;

: VFXndcs(h)	\ -- ; make the last definition NDCS to the host
  NDCS-Bit 'latest(s) @(m) status+ set-bit(s)
;

: VFXndcs(t)	\ -- ; mark last target word as NDCS
  latest(t) 1- c@c(t) $20 or latest(t) 1- c!c(t)	\ bit
;

: VFXimmediate(t)       \ -- ; make last definition immediate
  VFXndcs(h)
  $40 'latest(s) @(m) >name(s) set-bit(s)	\ immediate; symbols are not VFX standard
  width(t) @(m) if
    latest(t) 1- c@c(t) $04 or latest(t) 1- c!c(t)	\ target format
    VFXndcs(t)  its-xt set-compiler(t)
  endif
;

((
: VFXimmediate(t)       \ --
\ make last definition immediate in host and target
  VFXndcs(h)  width(t) @(m) if
    VFXndcs(t)  its-xt set-compiler(t)
  endif
;
))
: VFX-headers   \ -- ; default to VFX Forth headers
  assign VFXnoheader(t) to-do noheader(t)
  assign VFXheader(t) to-do header(t)
  assign VFXimmediate(t) to-do immediate(t)     \ SJC001
  0 ImmVals(t) !(m)
  #255 width(t) !(m)
  true VFXuservars? !
;


((
Over in another thread ("What are my options for dictionary
hashing?"), we've been discussing a variety of schemes for efficiently
handling Forth dictionaries.  I wanted to start a new discussion in
this thread based on my initial findings on one approach that I'll
call "HashList/MRU."  I think the results are both surprising... and
not.  The model that I'm using was built in Perl because I find that a
fine language for quickly prototyping ideas and measuring their
effectiveness.  The next step will be to convert this to Forth--
unless someone beats me to the punch.  I'll provide a link to the
script at the end of this message so that others can independently
validate my findings and/or improve the model.

Here's a quick summary:  In the other thread, the OP (initially)
described a dictionary scheme where he used a hash function to select
from one of 16 lists that he would then do a linear search on.  I
later tacked on the idea of exploiting locality of reference and
treating each list as a MRU cache, moving found items to the front of
the list.

In my model, I use a very simple hash function chosen for speed rather
than one that produces an ideal distribution.  The hash function
simply adds the symbol's length and each of the symbol's characters
together, and then takes the result, mod 16.  It's not the best, but
it's also not the worst.

I seed the dictionary with the 359 unique words taken from ANS Forth.
For a dataset, I'm initially using all of the Forth code in Hugh
Aguilar's "novice" code at http://www.forth.org/novice.forth.zip.
This dataset was chosen mostly for ironic reasons; I'll collect a
larger and more diverse set of Forth code and try that later.  I
frankly don't expect the numbers to change all that much, but that's
the reason why I'm testing my expectations.

So let's look at the statistics the model collected.

There were 15589 lookups of symbols.  Finding those symbols took a
total of 146940 probes.  So we can say that for this dataset, the
average number of probes to find a symbol is 9.4.  This is an
interesting number because if we take the total number of symbols in
the dictionary at the end (1479), we can compare that to a height-
balanced binary tree and see that it would have resulted in log2(1479)
or 10.5 probes, worst case.  So very roughly, at least for this
dataset, the average number of probes to find a symbol is slightly
less than the worst case of a height-balanced binary tree.  Obviously,
that means the height-balanced binary tree will have better average
search performance, but the HashList/MRU scheme fares much better than
(my) intuition suggested.  And it does so with far less cost in
maintaining not only the additional memory needed for a binary tree,
but the CPU cycles needed to keep it balanced.

We can now look at the effectiveness of the MRU cache.  The simplest
measure is to turn it off and see how it affects search performance.
When I do that, the number of probes jumps to 607320, which raises the
average number of probes needed to find a symbol to 38.9.  So clearly
MRU cache behavior helps reduce the number of probes needed to find a
symbol by a significant amount.

I next wanted to get a better idea of the MRU behavior.  That is, we
can see the MRU is effective at reducing the number of probes, but I
wanted to know more about how deep into each list a symbol typically
was when it was found.  And here's those numbers:

 0: 6649 (42.65)
 1: 3022 (19.39)
 2: 1341 ( 8.60)
 3:  658 ( 4.22)
 4:  368 ( 2.36)
 5:  267 ( 1.71)
 6:  169 ( 1.08)
 7:  160 ( 1.03)

In other words, when a symbol was found, it was found 42.7% of the
time at the top of a list, 19.4% of the time at the second-from the
top of the list, and so on.  Add it up, and a whopping 81% of the
time, found symbols are within the first eight items of the list.  And
just for interest, here is the top eight items on each list:

 0 (   92): abort" over alloc list recurse 1- false case
 1 (   87): @ if allot each[ show-int prune-tree endof halve-count
 2 (   81): ! constant ['] <evens> r> >r a<b ccompare
 3 (  101): then ]each insert-node <show-tree> emit a=b new-root
promote-rite
 4 (   93): data-wbuf limit r@ init-int tree a>b true kill-symtab
 5 (   92): used each <find3> <int*> insert-ordered ?do u> spaces
 6 (  103): next-record init-buf evens and nil head init-list test.4th
 7 (   97): room-wbuf cr rnd fill-int show-tree c" .rite <smudge>
 8 (   87): w here cells marker cnt root rite equalize
 9 (   90): ( wbuf@ records ptr find3 drop int* new-int
10 (   86): wbuf! create wbuf.4th init-tree node depth nip direction
11 (   97): : * tuck .key <find-key> <= new-parent dealloc
12 (  101): ; past -- { dup [char] attach-to-parent 2drop
13 (   97): \ <show-int> find-node int> <new-symtab> show-node
else .count
14 (  100): data +! = - } variable .n loop
15 (   75): 0= base size . swap key-lim ~ find-key

The number in parenthesis is the number of symbols in each list,
useful for seeing the quality of the distribution.

One final statistic to consider is the number of times a symbol was
moved to the start of the list after it was found.  There were 7451
moves for this dataset, which means that roughly 47.8% of the time, a
search for a symbol will result in an item moving to the front of the
list.

So let's summarize:

1.  Adding the MRU cache behavior to the original HashList scheme is a
clear benefit in terms of search performance.  With this dataset, 81%
of the time the symbol is found within the first 8 items in the list.
One of the criticisms placed against the MRU for a Forth dictionary
was that there is no reason to believe that if a symbol is referenced,
it will be referenced again.  That would be true if symbols had a
uniform distribution, but they clearly don't.

2.  An improved hash function would likely improve matters somewhat,
but even with a relatively poor hash function, the search performance
is very good.  By reducing the number of lists needed to search by 16
and by reordering the list based on the temporal pattern of symbol
use, the average performance isn't terribly far from the worst case
behavior of a height-balanced binary tree.

3.  The code to implement this HashList/MRU scheme is dirt-simple,
doesn't have the memory overhead of a binary tree, and doesn't have
the CPU overhead of rebalancing the binary tree.  The biggest cost is
a simple "move to front" operation, which involves swapping three
pointers, and that is (with this dataset) only happening 47.8% of the
time.

4.  This HashList/MRU scheme is naturally adaptive without any
controls needed to adjust performance and no memory needed to track
the number of times a symbol is used.  Also, because it doesn't track
cumulative frequency, it isn't confused by symbols that have a high
frequency in one part of the code, but low frequency elsewhere.

5.  I am not making the claim that HashList/MRU is the simplest
possible scheme or the fastest possible.  Indeed, a straight hash
table will give vastly better performance.  But for those allergic to
the probabilistic nature of hash tables and who overstate their costs,
HashList/MRU is both simple and efficient.  I am saying that compared
to height-balanced binary trees it costs less in both memory and CPU
cycles while giving very good search performance.  And when compared
to a frequency-balanced binary trees, it's safe to say that HashList/
MRU also costs even less in memory and CPU cycles, and has better
average performance.  Of course, we'll know for sure when the
"inventor" of symtab provides objective measures to support his
claims.

For those who want to play with the script used to generate the
statistics I've cited, it's available here:

    http://japanisshinto.com/stuff/HashListMRU.pl

You run it (at least under Linux) like this:

    perl HashListMRU.pl novice/*.4th

))

