\ Hmodel.fth - Host dependent words for the 32-bit compiler

((
Copyright (c) 1992-2008, 2009, 2010, 2011, 2012, 2014, 2016, 2018, 2022
MicroProcessor Engineering
133 Hill Lane
Southampton
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com

Change History
==============
20220329 SFP039 Added EITHER=
20200823 SFP038 Added more 64 host selections.
20200623 SFP037 Added CELLBITS.
20200618 SFP036 Added support for 64 bit hosts.
20200220 SFP035 Reversed SFP031.
20191106 SFP034 Added debug version of THROW.
20180510 SFP033 Changes for VFX v5 host
20160922 SFP032 Changes for ARM host.
20160830 SFP032 Default to "1 constant HLLonly?"
20141120 SFP031 More macro expansion.
20140807 SFP030 Added VFXheaders?
20140530 SFP029 Added FCOPY and FDEL.
20140214 SFP028 Added ERR_TARGSYM.
20120824 SFP027 Improved resolution of GET-MS.
20111121 SFP026 Added byte reverse operations.
20101210 SFP025 made default SHOW-FTH expand macros.
20101130 SFP024 Added 16 bit CRC.
20101015 SFP024 Added List-Clear.
20101013 SFP024 Added Copy-File.
20091217 SFP023 SET-PATHNAME does not change case of file name.
20080212 SFP022 V7 conversion
20070910 SFP021 Added escaped strings.
20060320 SFP020 Added ARSHIFT.
20060105 SFP019 Added GETDATE$ and GETTIME$
20040524 SFP018 Added NOT-IRTC for Stamp versions
SFP017	4 July 2001	Added SKIP-TO-WORD
SFP016	15 Dec 2000	Added ROL and ROR
SFP015	19 May 2000	Added 3DUP
SFP014  10 April 2000	Added IRTC stuff
SFP013  12 Sept 1999	Added ANS SM/REM and FM/MOD
SFP012  31 March 1999	Added .SOURCE-LINE to show current line and position
SFP011  26 Aug 1998	Removed SKIP-TO-WORD as it is no longer needed.
MSD010  03 Mar 98	Removed the DISP-ERROR patch as I *have* put it into PFW itself.
			(This change was not flagged in the source as it was just a
			straight deletion of the DISP-ERROR word.)
SFP009  22 Feb 98	Restored the DISP-ERROR patch. This should really
			be put in PFW itself.
MSD008  18 Feb 98	Removed the ANS "is defined" word set since they have been
			incorporated into PFW.
JL007    3 Oct 97	Removed 2>R 2R> 2R@ since now in PFW 2.1.
JL006   25 Feb 97	Added 2>R 2R> 2R@
SFP005  22 Jan 97	Added [ENDIF]
SFP004  12 Jan 96	Added TAB
JL004   14 November 96	Support for conditional compilation before hitting MODULE:.
JL003   13 November 96	Minor update to volume 2 draft 4.
JL002   30 September 96	Removed redefinitions when building on PFW 1.5 alpha 18.
JL001   23 August 96	Always compiles ANS control structures HOST_IF HOST_ELSE etc.
This file started August 1992
))


only forth also c-c also forth definitions
decimal

[undefined] cellbits [if]
: cellbits	\ -- u
\ *G Count the number of bits in a cell - relies on 2s
\ ** complement arithmetic. Useful when porting code
\ ** between 16, 32 and 64 systems. Note that this is
\ ** not a constant; the calculation is made at each use.
  0 -1 begin				\ count mask --
    dup
   while
    swap 1+ swap
    1 lshift
  repeat
  drop
;
[then]

0 value show-rs-throw?	\ -- x ; if nz, throw has more debug support

: +show-rs-throw	\ -- ; enable debugged THROW
  1 to show-rs-throw?
;
: -show-rs-throw	\ -- ; disable debugged THROW
  0 to show-rs-throw?
;

: throw		\ x -- ; added debugging support
  ?dup if
    show-rs-throw? if  .rs  then
    throw
  then
;

: ?throw	\ flag x --
  swap if  throw  else  drop  then
;

: ;host,		\ -- ; lay host ; code
  postpone ;
;

[defined] target_arm_linux [if]
: (head-create)         \ -- ; INTERPRETER head
  here $create                          \ build header in dictionary
  -4 allot                              \ de-allocate DOCREATE
  state on                              \ host's compiler on
;
[else]
: (head-create)         \ -- ; INTERPRETER head
  here $create                          \ build header in dictionary
  -5 allot                              \ de-allocate DOCREATE
  state on                              \ host's compiler on
;
[then]

: .last(h)	\ -- ; display latest host NFA.
  last @ 40 dump
;
also c-c definitions
: .last(h)	\ -- ; display latest host NFA.
  last @ 40 dump
;
previous definitions

\ align(h) even(h) -- J Lee .. 8/9/93

\ set this to TRUE for even aligned hosts, FALSE for don't care hosts
FALSE constant ALIGN(H)

align(h) [if]
: EVEN(H)       \ --
  ALIGN(H) If
    here 1 and                  \   at odd address?
    If  bl c,  Endif
  Endif
;
[else]
: EVEN(H) ; immediate
[then]

\ host size independence

: cell*         cell *  ;
: cells-        cell* -  ;


\ ALIAS to reduce colon threading overhead
\
\ Use in the form:
\       ALIAS <new-name> <old-name>
\ e.g.
\       ALIAS @(M) @
\

: alias synonym ;

: rshifts	\ x1 u -- x2
  rshift  ;

: widen		\ b -- n ; widen signed byte to signed cell
  dup $80 and                           \ if byte is negative
  If  $FFFFFF00 or  Endif               \   do a negative extension
;
\ ... JL003

[undefined] u<= [if]
: u<=  ( u1 u2 -- flag )
  2dup u<  -rot  =  or
;
[then]

[undefined] u>= [if]
: u>=  ( u1 u2 -- flag )
  2dup u>  -rot  =  or
;
[then]

: umod  ( u1 u2 -- u )
  0 swap um/mod drop
;

: uwithin?  (  u1 u2 u3  --  flag  )
  >r  over u<=
  r>  rot u>=
  and
;

: U-WITHIN?	\ u1 u2 u3 -- flag
  uwithin?
;


\ **********************
\ Tab handling and WORDS
\ **********************

: DisplayStart	\ --
\ Clear the output line counter to minimise display pauses.
  1 op-line# !
;

variable tab-width
#16 tab-width !         \ Default for DOS and for XC layouts

: tab           \ n -- ; tab to given position
  c/l out @ <=                          \ beyond end of line?
  if
    cr
  else
    tab-width @ out @ over mod - spaces
  endif
;

[defined] Target_386_Windows [if]
Extern: BOOL PASCAL QueryPerformanceCounter( void * int64 );
Extern: BOOL PASCAL QueryPerformanceFrequency( void * int64 );

: get-ms	\ -- ms
\ *G Return a ticker count in milliseconds.
\ *E seconds = count / freq
\ ** ms      = (count * 1000) / freq
\ *P Note that we assume that frequency can be expressed as
\ ** a positive 32 bit number.
  { | count[ 2 cells ] freq[ 2 cells ] -- ms }
  count[ QueryPerformanceCounter drop
  freq[ QueryPerformanceFrequency drop
  count[ 2@ swap			\ count
  #1000 freq[ @ m*/ drop		\ ms
;
[then]

[defined] Target_x64_Windows [if]
Extern: BOOL PASCAL QueryPerformanceCounter( void * int64 );
Extern: BOOL PASCAL QueryPerformanceFrequency( void * int64 );

: get-ms	\ -- ms
\ *G Return a ticker count in milliseconds.
\ *E seconds = count / freq
\ ** ms      = (count * 1000) / freq
\ *P Note that we assume that frequency can be expressed as
\ ** a positive 64 bit number.
  { | count[ cell ] freq[ cell ] -- ms }
  count[ QueryPerformanceCounter drop
  freq[ QueryPerformanceFrequency drop
  count[ @ #1000 freq[ @ */ 		\ ms
;
[then]

[defined] Target_ARM_Linux
[defined] Target_386_Linux or
[defined] Target_x64_Linux or
[defined] Target_386_OSX or
[defined] Target_x64_OSX or
[if]
: get-ms        \ -- ms
  ticks
;
[then]

[undefined] .source-line [if]
: .source-line  \ -- ; show current source line
  ShowErrorLine ShowSourceOnError
;
[then]

: ShowCurrSource	\ --
\ *G Display the current text file at the current line.
  .source-line
;


\ *************
\ Date and Time
\ *************

\ SFP019...

create Months	\ -- addr
  ", JanFebMarAprMayJunJulAugSepOctNovDec"

: Mon$		\ n -- caddr len ; n=1..12
  ( 1 max #12 min ) 1- 3 * Months + 1+ 3
;

also system
: (date)	\ -- dow dd mm yyyy
  os_getlocaltime			\ -- secs min hrs day mon year dow
  swap >r swap >r swap >r
  nip nip nip
  r> r> r>
;

: (time)				\ -- hh mm ss
  os_getlocaltime 4drop			\ -- secs min hrs day mon year dow
  -rot swap
;
previous

: GetDate$	\ buff --
\ *G Get date as counted string in buffer.
  base @ >r  decimal
  0 over c!  >r				\ set empty string
  (date)				\ -- dow dd mm yyyy ; R: -- base buff
  rot (u.) r@ append			\ dd
  bl r@ addchar
  swap Mon$ r@ append			\ mm
  bl r@ addchar
  (u.) r> append			\ yyyy
  drop
  r> base !
;

: (2.r)		\ u -- caddr len
  0 <# # # #>
;

: GetTime$	\ buff --
\ *G Get time as counted string in buffer.
  base @ >r  decimal
  0 over c!  >r				\ set empty string
  (time)				\ -- hh mm ss ; R: -- base buff
  rot (2.r) r@ append			\ hh
  [char] : r@ addchar
  swap (2.r) r@ append			\ mm
  [char] : r@ addchar
  (2.r) r> append			\ ss
  r> base !
;

\ ...SFP019

: OFFSET        \ x "<spaces>name" -- ; Exec: n -- n+x*4
\ *G Create a new offset called *\i{name}. On execution of *\i{name}
\ ** the supplied address will be incremented by *\i{x} cells.
  create
    ,
  does>
    @ cells +
;


\ *****
\ Files
\ *****

: from?		\ -- flag
\ *G Return true if input is coming from a file
  source-id dup				\ 0 is console
  if  -1 <>  endif			\ -1 is EVALUATE
;

: ?ReleaseFiles	\ --
\ *G Close any open source files. At present this just closes the
\ ** current source file.
  from? if
    source-id close-file drop  0 -> source-id
  endif
;

[undefined] ptr [if]
: ptr	int  ;
[then]
\ synonym on 32 bit, not on 64 bit


MAX_PATH 1 cells+ constant PCB-LEN	\ -- len

: pcb                   \ "name" -- ; Exec: -- addr
  create
    INVALID_HANDLE_VALUE ,	\ handle
    MAX_PATH allot&erase	\ zstring
\  does>
;

: handle                \ 'pcb -- handle
  @
;

: p-name                \ 'pcb -- *filename
  cell+
;

: reset-pcb             \ 'pcb --
  INVALID_HANDLE_VALUE over !
  cell+ MAX_PATH erase
;

: set-pathname          \ $name 'pcb --
  dup >r reset-pcb
  count r> p-name       \ c-addr u 'pcb-name -- ;
  swap MAX_PATH 1- min  \ c-addr 'pcb-name 'u -- ;
\  2dup 2>r		\ del SFP023
  move                  \ -- ;
\  2r> upper		\ del SFP023
;

\ : Extension?	\ c-addr u -- len true | false

: scan-last
  >r
  BEGIN   over 1+ over 1- r@ scan ?dup
   WHILE  2swap 2drop
  REPEAT
  r> 2drop
;

: set-ext	\ $addr pcb-addr --
\ If the filename in the path control block has no extension, the
\ one in the given string is added to the filename.
  p-name dup >r max_path [char] \ scan-last [char] . scan nip
  IF    r> 2drop   exit
  ENDIF
  r> max_path 0 scan drop  >r count tuck r@ swap cmove r> + 0 swap
  c!
;


: .pcb                  \ 'pcb --
  p-name .z$
;

: open-path-pcb         \ 'pcb -- ior
  dup p-name zcount r/w open-file dup	\ -- pcb handle ior
  if  nip -1 swap  endif
  -rot  swap !
;

: open-source-pcb       \ 'pcb -- ior
  dup p-name zcount r/o open-file dup	\ -- pcb handle ior
  if  nip -1 swap  endif
  -rot  swap !
;

: create-path-pcb       \ 'pcb -- ior
  dup p-name zcount r/w create-file dup	\ -- pcb handle ior
  if  nip -1 swap  endif
  -rot  swap !
;

: close-path-pcb        \ 'pcb -- ior
  dup handle close-file
  INVALID_HANDLE_VALUE rot !
;

: create-path           \ zname -- handle 0 | -- ior
  zcount r/w create-file dup
  if  nip  endif
;

: open-path	\ zname -- handle 0 | -- ior
  zcount r/w open-file dup
  if  nip  endif
;

: read-path             \ addr len handle -- n ior
  read-file
;

[defined] Target_386_Windows 
[defined] Target_x64_Windows or
[if]
: write-path            \ addr len handle -- n ior
  { | temp[ cell ] -- }
  0 temp[ !
  -rot temp[ NULL			\ handle addr len abytes overlapped --
  WriteFile
  if temp[ l@ 0 else -1 -1 then
;
[then]
[defined] Target_ARM_Linux
[defined] Target_386_Linux or
[defined] Target_x64_Linux or
[if]
: write-path            \ addr len handle -- n ior
  -rot write dup -1 =
;
[then]

[defined] Target_386_OSX
[defined] Target_x64_OSX or
[if]
: write-path            \ addr len handle -- n ior
  -rot write dup -1 =
;
[then]

: seek-path             \ n handle -- ior
  swap 0 rot reposition-file
;

: close-path            \ handle -- ior
  close-file
;

: delete-path           \ $ -- ior
  count delete-file
;

: path-size	\ handle -- size|-1
\ Return file size as a single number, or -1 on error
  file-size
  if  2drop -1  else  drop  endif
;

[undefined] copy-file [if]

[defined] Target_386_Windows [if]
: copy-file	\ src srclen dest destlen nooverwrite -- ior ; SFP024
\ Copy the file. If nooverwrite is non-zero and the destination
\ exists, an error is returned.
  {: | src[ max_path ] dest[ max_path ] -- :}
  >r
  expand max_path 1- min dest[ zplace
  expand max_path 1- min src[ zplace
  src[ dest[ r> CopyFile
;
[then]

[defined] Target_386_Linux
[defined] Target_386_OSX
or [if]
: copy-clean	\ hs hd buff --
\ clean up after the copy
  free drop				\ buff
  dup -1 =				\ hd
  if  drop  else  close-file drop  endif
  dup -1 =				\ hs
  if  drop  else  close-file drop  endif
;

: copy-file	\ src srclen dest destlen nooverwrite -- ior
\ Copy the file. If nooverwrite is non-zero and the destination
\ exists, an error is returned.
  {: src slen dest dlen nov | hs hd nrd buff fsize[ 2 cells ] -- ior :}
  nov if
    dest dlen FileExist?
    if  -1 exit  endif
  endif
  -1 to hs  -1 to hd  0 to buff
  $1000 allocate
  if  drop  -2 exit  endif
  to buff
  src slen r/o open-file
  if  drop -3  hs hd buff copy-clean  exit  endif
  to hs
  dest dlen r/w create-file
  if  drop -4  hs hd buff copy-clean  exit  endif
  to hd
  hs file-size drop fsize[ 2!
  begin
    fsize[ 2@ d0<>
   while
    buff $1000 hs read-file
    if  drop -5  hs hd buff copy-clean  exit  endif
    to nrd
    buff nrd hd write-file
    if  -6  hs hd buff copy-clean  exit  endif
    fsize[ 2@ nrd 0 d- fsize[ 2!
  repeat
  0
  hs hd buff copy-clean
;
[then]		\ Linux/OSX
[then]		\ copy-file

: fcopy		\ -- ; fcopy source dest
  cr
  parse-name >syspad ." Copy - Source: " 2dup type
  parse-name >syspad ."  Dest: " 2dup type
  key? drop				\ force output
  0 copy-file
  if  ."  fail"  else  ."  good"  endif
  key? drop				\ force output
;

: fdel		\ -- ; fdel dest
  cr
  parse-name >syspad ." Delete - Dest: " 2dup type
  key? drop				\ force output
  delete-file
  if  ."  fail"  else  ."  good"  endif
  key? drop				\ force output
;

\ ****
\ Misc
\ ****

: v-find        \ caddr voc-xt -- cfa +/-1 | caddr 0
\ *G An equivalent to *\fo{SEARCH-WORDLIST} used by previous
\ ** MPE Forths.
  swap >r
  >body @  r@ count rot search-wordlist dup
  if  r> drop  else  r> swap  endif
;

: .CurrTextFile	\ --
\ *G Display source file name.
  line# @ -1 <>
  if  'SourceFile @ .SourceName  then
;

: .SourceFile	\ --
\ *G Display source file name with some decoration.
  line# @ -1 <> if
    cr ."  Source: "
   [char] " emit  .CurrTextFile  [char] " emit
  then
;

: $=	\ $1 $2 -- flag
\ Compare two counted strings, returning true if identical.
  dup c@ 1+ s=
;

#256 buffer: pocket

: 'word  here  ;


\ *****
\ Lists
\ *****

((
-------------------------------------------------------------------------------
	List Management

	Maintain lists in the dictionary.  Allows for the creation of, addition
	to, and traversal of lists.

	LIST-MAKE  (  --  list-id  )
		- Creates an empty list, returning a list identifier.

	LIST-ADD  (  item list-id  --  )
		- Adds item to the end of the list.

	LIST-TRAVERSE  (  xt list-id  --  )
		- Traverse the list performing the action of xt on each item.

-------------------------------------------------------------------------------
))

struct LIST-STRUCT
  int LIST-STRUCT.NEXT
  int LIST-STRUCT.ITEM
end-struct

: LIST-MAKE  (  --  list-id  ; creates a new empty list  )
  here LIST-STRUCT allot			\ create an empty node
  NULL over LIST-STRUCT.NEXT !			\ this is a dummy node
;

: LIST-ADD  (  item list-id  --  ; adds a node to the end of the list  )
  Begin						\ get to the end of the list :-
    dup LIST-STRUCT.NEXT @ NULL <>		\ while not end of list
  While
    LIST-STRUCT.NEXT @				\   move to next node
  Repeat
  here LIST-STRUCT allot			\ allocate space for new node
  dup rot LIST-STRUCT.NEXT !			\ link this to end of list
  NULL over LIST-STRUCT.NEXT !			\ make new node last in list
  LIST-STRUCT.ITEM !				\ set the item for the new node
;

: LIST-TRAVERSE  (  ... xt list-id  --  ; traverse the list and executes xt upon each node item  )
  LIST-STRUCT.NEXT @				\ The first node is a dummy
  Begin
    dup NULL <>					\ while not end of list
  While
   2>r						\   hide local stack so stack can be used by the executed word
   2r@ LIST-STRUCT.ITEM @ swap execute		\   execute action on node item
   2r>                                          \   restore local stack
   LIST-STRUCT.NEXT @				\   move to next node
  Repeat
  2drop
;

: List-Clear	\ list-id -- ; empty list
  0 0 rot 2!  ;


\ *************************
\ LOCATE and AIDE interface
\ *************************

((
1 constant dbgXlocate?

: .$Err		\ $ --
  {: | temp[ 260 ] -- :}
  count temp[ zplace
  hwndMain temp[ z" AIDE-Err" MB_OK MessageBox drop
;

: .alErr	\ caddr len --
  {: | temp[ 260 ] -- :}
  temp[ zplace
  hwndMain temp[ z" al-AIDE-Err" MB_OK MessageBox drop
;
))

defer show-fth	\ file$ line# --
\ *G Locate to file and line#.

[defined] fullPathName [if]
max_path buffer: FQpathname	\ -- addr
\ *G Buffer holding fully qualified path name
[then]

: vfx-show-fth	\ file$ line# --
\ *G Locate to file and line# using VFX internal mechanism.
\ ** This is the default action of *\fo{show-fth} above.
  { | temp[ MAX_PATH 1+ ] }
  swap $ExpandMacros
[defined] fullPathName [if]
  count fullPathName temp[ place  temp[	\ SFP031, SFP035
[then]
  C" locate_path" $setmacro	\ SFP025
  base @ >r  decimal s>d <# #S #>  r> base !
  temp[ place
  temp[ C" locate_line" $setmacro
  .locate
;
assign vfx-show-fth to-do show-fth

: whereis-show-fth	\ file$ line# --
\ *G Display the file name and line number on the console.
  ."  file: " swap count type
  ." , line: " . cr
;

[defined] Target_386_Windows
[defined] Target_x64_Windows or
[if]

expose-module UserIDE
: setXlocate	\ --
  assign EditLocate to-do .Locate  ;
' setXlocate AtCold
previous

: Cmd>AIDE	\ caddr len --
  { | temp[ cell ] -- }
  STD_ERROR_HANDLE GetStdHandle -rot  temp[ NULL WriteFile drop
;

\ The word LOCATE-VIA-AIDE is used to switch LOCATE to send
\ commands to AIDE.
\ The format of the LOCATE command is:
\   VIEW-FILE <line#> <filename>
\ e.g.
\   VIEW-FILE 100 foobar.fth
\ if the supplied file name does not contain an extension
\ (indicated by a "." in the file name), the default ".FTH"
\ is added.

\ SFP003... locate target code via AIDE

create AIDE-string      \ -- addr
\ *G Buffer for AIDE command line
  $100 allot&erase

: AIDE-show-fth \ $file line# -- ;
\ *G Force LOCATE to run through AIDE.
\ dbgXlocate? if
\  over .$Err
\ then
  base @ -rot  decimal			\ -- base $file line#
  s" VIEW-FILE " AIDE-string place	\ base command
  0 <# bl hold #s #> AIDE-string append	\ -- base $file ; add line number with trailing space
[defined] fullPathName [if]
  count fullPathName			\ SFP031, SFP035
  2dup AIDE-string append		\ -- caddr len ; add file name
[else]
  count 2dup AIDE-string append		\ -- caddr len ; add file name
[then]
  Extension?				\ add .fth if no extension
  if  drop  else  s" .fth" AIDE-string append  endif
  AIDE-string
  count expand Cmd>AIDE	\ send text to AIDE
  base !
;

: AideErrorEdit	\ -- ; run editor on error
\ *G Edit the file at an error, using the contents of the variables
\ ** *\fo{'SOURCEFILE} and *\fo{LINE#}.
  { | temp[ MAX_PATH ] -- }
  'SourceFile @ -1 <> if		\ source file?
    line# @ -1 <> if			\ line number?
      CurrSourceName temp[ place
      temp[ line# @ AIDE-show-fth
    endif
  endif
;  assign (EditOnError) to-do EditOnError

: locate-via-aide       \ -- ; force LOCATE to run through AIDE
  assign AideErrorEdit to-do EditOnError
  assign AIDE-show-fth to-do show-fth
;
[then]

[defined] Target_ARM_Linux
[defined] Target_386_Linux or
[defined] Target_x64_Linux or
[defined] Target_386_OSX or
[defined] Target_x64_OSX or [if]
: locate-via-aide       \ --
\ *G Force *\fo{LOCATE} to run through AIDE.
\ ** This is a *\fo{NOOP} for MacOS and Linux.
;
[then]


\ ************************
\ Context save and restore
\ ************************

: n!		\ xn..x1 n addr --
\ *G Save n items at addr including n.
  over 1+ cells bounds
  do  i !  cell +loop
;

: n@		\ addr --
\ *G Retrieve n items saved by N!.
  dup @ cells over +		\ -- start end
  do  i @  cell negate +loop
;

cell vsize + constant /SOnest	\ -- u
\ *G Storage required for search order preservation
8 constant #SOsaves	\ -- u
\ *G Search order maximum nesting.
/SOnest #SOsaves * constant /SOsave	\ -- size

/SOsave buffer: SO[]	\ -- addr
\ *G Array in which search orders are saved.

SO[] value CurrSO[]	\ -- addr
\ *G Points at next search order free position.

: saveSO	\ --
\ *G Save search order.
  get-current CurrSO[] !
  get-order CurrSO[] cell + n!
  /SOnest add CurrSO[]
;

: restoreSO     \ --
\ *G Restore search order.
  /SOnest sub CurrSO[]
  CurrSO[] @ set-current
  CurrSO[] cell+ n@ set-order
;


\ **************
\ Error handling
\ **************

#-406 #ErrDef err_OutOfMemory	"ALLOCATE/RESIZE failed"

ErrDef xcerr_fwdref	"Unresolved or forward reference not permitted"
ErrDef xcerr_host	"Word not in host Forth"
ErrDef xcerr_FwdType	"Unknown forward reference type"
ErrDef xcerr_IllFwd	"Forward reference not permitted"
ErrDef xcerr_LocRes	"Local label already resolved"
ErrDef xcerr_InvOp	"Invalid operator for this type"
ErrDef err_ise		"Internal software error - report to MPE"
ErrDef err_uncons	"Unconsumed reference"
ErrDef err_depth	"Code error - stack depth changed"
ErrDef err_TargSym	"Cannot compile or execute target word/symbol on host"

: error		\ bad word now!
  cr ." Replace ERROR with THROW" #-13 throw
; immediate

: ?error		\ bad word now!
  cr ." Replace ?ERROR with ?THROW" #-13 throw
; immediate


\ ====
\ Misc
\ ====

also system
: find-windows-constants        \ --
\ *G Cross compiler will/may find Windows constants.
  assign isWinInteger? to-do isNumber?	\ SFP033
\  assign winInteger? to-do number?
;

: -find-windows-constants       \ --
\ *G Cross compiler will not find Windows constants
  assign isInteger? to-do isNumber?	\ SFP033
\  assign integer? to-do number?
;
previous

-find-windows-constants

cellbits 32 = [if]
#19 constant minXt>Cntb	\ -- n
\ *G A VFX header is at least 19 bytes, from the XT to the count byte: a 3 byte
\ ** name plus two 16 bit items, plus three cells.
\ *E   Link | Ctrl | Count | <name> | Term | line# | Info | XRef | Len/xt | Sgen
\ **   -------------------------------------------------------------------------
\ **   Cell | Byte |  Byte | n Bytes| Byte |  Word | Word | Cell |  Cell  | Cell
\ **   -------------------------------------------------------------------------
\ **     4  |  1   |   1   |    n   |   1  |   2   |   2  |   4  |    4   |   4
[then]

cellbits 64 = [if]
#31 constant minXt>Cntb	\ -- n
\ *G A VFX header is at least 31 bytes, from the XT to the count byte: a 3 byte
\ ** name plus two 16 bit items, plus three cells.
\ *E   Link | Ctrl | Count | <name> | Term | line# | Info | XRef | Len/xt | Sgen
\ **   -------------------------------------------------------------------------
\ **   Cell | Byte |  Byte | n Bytes| Byte |  Word | Word | Cell |  Cell  | Cell
\ **   -------------------------------------------------------------------------
\ **     8  |  1   |   1   |    n   |   1  |   2   |   2  |   8  |    8   |   8
[then]


: IP>NFA        \ addr -- nfa                                    MPE.0000
\ *G Attempt to move backwards from an address within a definition to
\ ** the relevant NFA.
  MinXt>Cntb -			\ NFA (count byte) is at least this far back
  begin
\    dup InForth? 0=
\    if  drop  C" * outside dictionary * " exit  then
    dup name? 0=		\ -- addr flag
   while
    1-
  repeat
;

: >name         \ xt -- nfa                                      MPE.0000
( *G Move a pointer from an XT back to the NFA or name-pointer.         )
( ** If the original pointer was not an XT or if the definition in      )
( ** question has no name header in the dictionary the returned pointer )
( ** will be useless. Care should be taken when manipulating or scanning )
( ** the Forth dictionary in this way.                                  )
  unalign  ip>nfa			\ SFP024
;

: forceOP	\ --
\ *G Force console output to be flushed.
  key? drop  100 ms
;

: flushIP	\ --
\ *G Discard any concole input, flush output.
  begin  key? while  key drop  repeat  ;

: y/n?		\ -- t/f ; y = true
\ *G Wait for a key and return true if the user presses 'y' or 'Y'.
  flushIP  key upc [char] Y = dup
  If  [char] Y  Else  [char] N  Endif
  emit  forceOP
;

: +crc16	\ crc char -- crc'
\ *G Update the XMODEM CRC with the given character. The initial
\ ** value should be zero.
  8 lshift xor
  8 0 do
    dup $8000 and if
      1 lshift  $1021 xor
    else
      1 lshift
    then
  loop
  $FFFF and
;

: calcCRC16	\ addr len -- crc
\ *G Generate a 16 bit CRC for a block of memory
  0 -rot bounds				\ -- crc addr+len addr
  ?do  i c@ +crc16  loop
;

\ SFP026...
: ByteRevL	\ a:b:c:d -- d:c:b:a
\ *G 32 bit byte reversal.
  dup #24 rshift $FF and		\ -- a:b:c:d a
  over 8 rshift $FF00 and or		\ -- a:b:c:d b:a
  over 8 lshift $00FF:0000 and or	\ -- a:b:c:d c:b:a
  swap #24 lshift $FF00:0000 and or	\ -- d:c:b:a
;

: ByteRevW	\ a:b:c:d -- b:a:d:c
\ *G Byte reverse both 16 bit pairs.
  dup 8 rshift $00FF:00FF and		\ -- a:b:c:d 0:a:0:c
  swap 8 lshift $FF00:FF00 and or	\ -- b:a:d:c
;

: ByteRevWZ	\ a:b:c:d -- 0:0:d:c
\ *G Byte reverse low 16 bits and then zero extend.
  ByteRevW $0000:FFFF and
;

: ByteRevWS	\ a:b:c:d -- s:s:d:c
\ *G Byte reverse low 16 bits and then sign extend.
  ByteRevW $0000:FFFF and
  dup $8000 and negate or
;
\ ...SFP026

((
[undefined] "xxx" [if]
: "xxx"		\ "xxx" -- caddr len
\ *G Parse a string enclosed by quotes from the input stream, e.g.
\ *C   "Quoted string"
  [char] " parse 2drop			\ step up to first quote
  [char] " parse			\ string up to second quote
;
[then]
))

: bor!		\ mask c-addr --
\ *G Apply the *\i{mask} ORred with the contents of *\i{c-addr}.
\ ** Byte operation.
\  swap over
\  c@ or swap c!
  set-bit
;

: bbic!		\ mask c-addr --
\ *G Apply the *\i{mask} inverted and ANDed with the contents of
\ ** *\i{c-addr}. Byte operation.
\  swap not over
\  c@ and swap c!
  clear-bit
;

variable VFXuservars?	\ --
\ *G Set true when the original x86 USER variable structure
\ ** is being used. Aargh!
  0 VFXuservars? !

: [.s]		\ debug tool
  .s  ;  immediate

: >eol  source nip >in !  ;

[undefined] either= [if]
: either=       \ x a b -- t/f
\ *G Return true if *\i{x} is equal to either *\i{a} or *\i{b}.
  2 pick = -rot                 \ n, b
  =  or                         \ n, a
;
[then]
