\ INITIAL.FTH Cross compiler initialisation.

((
Copyright (c) 1993-2004

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
net: mpe@mpeforth.com

20090515 SFP006 Removed IRTC switches.
SFP005  10 April 2000
	Add IRTC switches.
SFP004  20 April 1999
        Add dummy vocabulary at start of symbol table. This is used
        to hold the list of *magic* vocabularies such as EQUATES
        and LABELS. This prevents other vocabularies being overwritten
        during initialisation. This bug has been lurking for a *very*
        long time.
MSD003  24 April 98
        Initilise #LOGGED with a *large* value.  (A facility has been
        added that allows the *user* to chose the number of log columns
        per line of text.  Therefore, we can't just use a (small) fixed
        value as the initial value of #LOGGED because the user may
        chose a bigger value.
JL002   5 November 96
        Add implicit TOKENS symbol useful for conditional compilation.
JL001   17 October 96
        Added external symbol table.
))


\ file control

only forth also c-c definitions
decimal

\ *** WARNING ***
\ This code is used to initialise the REQUIRED symbols.
\ It has intimate knowledge of the host's vocabulary structure and name list threading

(( \ PFW version
: enumerate-required-thread  ( thread -- )
  @                                                                     \ get the first thread
  begin                             \
    ?dup                            \   if not end of thread list (eol==0)
  while                             \
    dup name> execute drop          \     execute this word to initialise the relevant symbol
    n>link @                        \     step to next word in thrad list
  repeat                            \
;

: init-requireds  ( -- )
  ['] required.voc >body                        \ using REQUIRED.VOC vocabulary
  cell +                                                        \ step over VOC-LINK field
  dup cell + swap @ cells bounds    \ get address of first thread list and #threads bounded for loop
  ?Do                               \
    i enumerate-required-thread     \   traverse each thread list in turn
  cell +Loop                        \
;
))

\ VFX version
\ : WalkWordList  \ xt wid --
\ *E : MyDef    \ thread# nfa -- flag ; Return TRUE to continue

: InitReq	\ thread# nfa -- flag
  nip  name> execute drop  true
;

: init-requireds  ( -- )
  ['] InitReq ['] required.voc voc>wid WalkWordList
;



\ Unhook-Forth Build-Root-Voc ...........          Jon 30/01/89

decimal

: Unhook-Forth  \ --
  last(s) @(m) cell- off(s)             \ End of chain.
\ store NFA of FORTH in thread
  last(s) @(m)                     	\ FORTH nfa.
  last'(s)@ #threads(s)+ cell+          \ store into FORTH thread
  11 cell* + !(s)                       \ links. 11=5 (count) + 6 (F=ASCII 046)
  last'(s)@ vl+ off(s)                  \ voc links to zero
;                                       \ decimal 11 is hash code

: Build-Root-Voc        \ --
  16 c" ROOT" $Make-voc                 \ Build ROOT voc,
  Last'(s)@ 'Root(s) !(m)               \ a seperate case from
;                                       \ required vocs.


\ Build-Required-Vocs ...................          Jon 30/01/89

((
: Build-Required-Vocs                 \ --
  2 c" EQUATES" $Make-voc             \ All these vocs are built
  Last'(s)@ 'Equates(s) !(m)          \ on DUMMY.  All Voc-
  2 c" LABELS" $Make-voc              \ Symbol addresses are
  Last'(s)@ 'Labels(s) !(m)           \ stored in the Management
\ 16 c" PAGE" $Make-voc               \ Page Vocabulary.
\ Last'(s)@ 'Page(s) !(m)             \
  16 c" FORTH" $Make-voc              \ Variable area for easy
  Last'(s)@ 'Forth(s) !(m)            \ access.
  unhook-forth                        \ lonely forth.
;                                     \
))
: Build-Required-Vocs   \ -- ; SFP004
  Here(s) Current(s) !(m)	      \ dummy vocabulary to contain chain     \ SFP004
  0 ,(s)  0 ,(s)  #16 (Make-Voc)      \  of magic vocabularies                \ SFP004
  16 c" EQUATES" $Make-voc            \ All these vocs are built
  Last'(s)@ 'Equates(s) !(m)          \ on DUMMY.  All Voc-
  16 c" LABELS" $Make-voc             \ Symbol addresses are
  Last'(s)@ 'Labels(s) !(m)           \ stored in the Management
\ 16 c" PAGE" $Make-voc               \ Page Vocabulary.
\ Last'(s)@ 'Page(s) !(m)             \
  64 c" FORTH" $Make-voc              \ Variable area for easy
  Last'(s)@ 'Forth(s) !(m)            \ access.
  unhook-forth                        \ lonely forth.
;

\ Init-Vocs .............................          Jon 30/01/89

: Init-Vocs     \ --
  build-required-vocs                 \ First the Required Vocs.
  'Forth(s) @(m)                      \ Then set Forth(s) as
  Context(s) !(m) definitions(s)      \ Current(s) & Context(s)
  build-root-voc                      \ Root(s) on Forth(s).
;                                     \


: Init-vars     \ -- ; SFP005
  ?log on(m)  dp(t) off(m)              \ generate log table
  #31 width(t) !(m)			\ target header size
  the-equate HEADS? on(s)               \ lay target heads?
  do-build on(m)                        \ build host analogues
  host-mode off(m)                      \ turn host building mechanism off
  in-host-build off(m)                  \ flag not building into host
  9999 #logged !(m)                     \ symbol table format                   \ MSD003
  +offset off(m)                        \ signal that no offset to be laid for DO (as per EPIC Spec. V1.0)
  last'(s)@ 'latest(s) !(m)             \ set up symbol table
  Run-XC on(m)                          \ signal that XC has run
  mpe-controls off(m)                   \ use non MPE (ANS) control structures by default
;


\ Init-Symbols Init......................          Jon 30/01/89

: Init-Symbols ( -- )
  init-dp(s)
  build-dummy-voc
  init-vocs
;

\ JL002
: Init-Core  ( -- )
  init-symbol-space             \ initialise symbol table
  init-symbols                  \ initialise default symbols
  c" TOKENS" $symbol            \ create a standard symbol, this can be detected by conditional compilation words
  is-resolved                   \ make it resolved so it doesn't appear in unresolved log
  last'(s)@ 'latest(s) !(m)     \ manually mark as latest symbol
  init-vars                     \ initialise compiler variables
\  init-disk                    \ initialise target image handler
;

((
: Init-Target-Memory  ( -- ; initialise target base memory addresses )
  0 rom-base  0 ram-base  0 uram-base       \ set base addresses of sections
  max-data @(m) initialised-ram             \ allocate space for data section
  cell(t) allot                             \ grab address 0 in code space (so NULL pointer works)
  cell(t) allot-ram                         \ grab address 0 in initialised data space (so NULL pointer works)
  cell(t) allot-uram                        \ grab address 0 in unintialised data space (so NULL pointer works)
;
))

\ JL001
: Init-Extsymbols  (  --  ;  intialise external symbol table  )
\  extsym-create-file
;

: Init-ExtExports  (  --  ;  initialise external exports  )
\  extexp-create-file
;

only forth definitions

