\ Inlining for MPE cross compiler v6

((
Copyright (c) 1998
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 2380 631441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

Change history
--------------
SFP005	2 Feb 2000	Added INLINE-NEVER
SFP004	12 Nov 1999	Added protection for : foo ... r> ... >r ... ;
SJC003  99/10/07        ... not incorporated
SFP002  19 Jul 1999     Added INLINE-ALWAYS
SFP001  29 Oct 1998     Added ABSURD INLINING
SFP000  28 Sep 1998     First version


Description
-----------
The length of a definition (including the final return) is stored
in the CODE-LEN field of the symbol table.

During compilation the start address of the definition may be found from
the value field of the symbol. At the end of the definition, the length
is calculated and stored in the CODE-LEN field.

When the definition is referenced and compiled by COMPILE-REF, the
CODE-LEN field is compared against the MAX-INLINE variable. If inlining
is permitted, the code is copied from the original to the new location.
Otherwise a CALL is compiled.

Some words contain code that cannot be inlined, e.g defining words.
The variable INLINE-OK? is reset by these words, and the CODE-LEN
field is then left at 0 to indicate that inlining should not be used.
Note that since the CODE-LEN field includes the final return opcode,
the length of the return opcode must be subtracted before inlining
takes place.

The size of the return opcode is defined in the cross compiler's
build file.

Inlining is inhibited by:
  :noname
  does> ;code
  exit
  >r r@ r>  2>r 2r@ 2r>
  COMPILE-REF generating a CALL
  using a return stack item not placed there in the definition
  structures with absolute target addresses
  PC relative references
  Literal pools

CODE <name> definitions may be inlined by placing INLINE after the END-CODE
  CODE <name> ... END-CODE  INLINE

INLINE-ALWAYS performs the same function as INLINE, except that the word
will be inlined regardless of the current setting of INLINING. This is
provided to aid factoring of the kernel.
))


only forth  also c-c definitions

variable inline-ok?     \ -- addr ; true if inlining permitted
  inline-ok? off
variable max-inline     \ -- addr ; holds maximum size of code to be inlined
  max-inline off
variable rs-changed     \ -- addr ; hold change in the return stack
  rs-changed off
variable <inline-always> \ -- addr ; set for defn. that is always inlined
  <inline-always> off
variable rs-negative	\ -- addr ; true if RS went negative, i.e return address used	SFP004
  rs-negative off	\ set true if definition accesses return stack items
  			\ that were not put there by the definition.
			\ This flag is also used by the source inliner.

\ These four variables must be set up in the XOPTxxx/XGENxxx files
variable #:entry        \ -- addr ; size of entry code to COLON definition
variable #:exit         \ -- addr ; size of exit code for COLON definition
variable #code-entry    \ -- addr ; size of entry code for CODE definition
variable #code-exit     \ -- addr ; size of exit code for CODE definition


\ ********************************
\ Words used by the cross compiler
\ ********************************

: start-inline  \ -- ; tell compiler inlining of word is permitted
\  inline-ok? on  rs-changed off
  inline-ok? on  rs-changed off  rs-negative off	\ SFP004
;

: set-code-len  \ -- ; set the CODE-LEN field of any symbol
  chere(t) 'latest(s) @(m) value+ @(s) -        \ length of code
  'latest(s) @(m) code-len+ !(s)                \ store it
;

: end-inline    \ -- ; sets CODE-LEN field of last COLON definition
  inline-ok? @off                       \ inlining ok?
  rs-changed @off 0=  and               \ and no change in R.S. depth?
  rs-negative @off 0=  and		\ and R.S. never went negative	SFP004
  if  set-code-len  endif               \ set up CODE-LEN field
;

: not-inlined   \ --
  inline-ok? off
;

: inlined       \ --
  inline-ok? on
;

: inline?       \ symbol -- flag ; true if symbol can be inlined
  dup resolved?
  if
    dup code-len+ @(s)                  \ SFP002
    1 max-inline @ within?
    swap inline-always? or              \ SFP002
  else
    drop 0
  endif
;

: :inline,(t)           \ symbol -- ; inline a colon definition
  dup value+ @(s) #:entry @ +           \ start of code to be inlined
  swap code-len+ @(s)                   \ size of code to be inlined
  #:entry @ -  #:exit @ -               \ less entry and exit code
  0 max  bounds
  ?do  i c@c(t) c,c(t)  loop
;

: code-inline,(t)       \ symbol -- ; inline a code definition
  dup value+ @(s) #code-entry @ +       \ start of code to be inlined
  swap code-len+ @(s)                   \ size of code to be inlined
  #code-entry @ -  #code-exit @ -       \ less entry and exit code
  0 max  bounds
  ?do  i c@c(t) c,c(t)  loop
;

: inline,(t)            \ symbol -- ; inline a definition
  dup code-word?
  if
    code-inline,(t)
  else
    :inline,(t)
  endif
;

: ?inline-rs    \ n -- ; no inlining if <n items on return stack
  rs-changed @ >
  if  rs-negative on  not-inlined  endif	\ needed for source inliner SFP004
;

: +rs-count     \ n -- ; update return stack change
  rs-changed +!  0 ?inline-rs		\ change return stack
;


\ ***********************************
\ User directives to control inlining
\ ***********************************

\ These constants define the maximum size of a word that can be inlined
    0 constant no
  #12 constant normal
 #255 constant aggressive
#4095 constant absurd                   \ SFP001

: inlining      \ n -- ; set size of inlining
  max-inline !
;

: inlining?     \ -- n ; get size of inlining
  max-inline @
;

: UnSafe-RS?	\ -- flag ; true if RS is mistreated ; SFP004
  rs-negative @
;

no inlining

: inline        \ -- ; mark latest CODE definition for inlining
\  'latestCode @(m) Code-Word? 0=
\  abort" Not a CODE definition"
  set-code-len
;

: inline-always \ -- ; mark latest definition to be always inlined
  inline  is-inline-always
;

: inline-never \ -- ; mark latest definition to be never inlined ; SFP005
  0 'latest(s) @(m) code-len+ !(s)	\ zero code length!
;

\ test code
: .symx         \ -- ; display symbol data
  get-token current(s) @(m) v-find(t) ?undef
  $40 - $80 dump(s)
;


