((
	MARKER.FTH
	Extensible MARKER.

	Copyright (c) 1997
	MicroProcessor Engineering
	133 Hill Lane
	Southampton SO15 5AF
	England

	tel: (+44) 2380 631441
	email: mpe@mpeforth.com

	Changes
	-------
))


only forth definitions
decimal


((
-------------------------------------------------------------------------------
	List Management

	Maintain lists in the dictionary.  Allows for the creation of, addition
	to, and traversal of lists.

	LIST-MAKE  (  --  list-id  )
		- Creates an empty list, returning a list identifier.

	LIST-ADD  (  item list-id  --  )
		- Adds item to the end of the list.

	LIST-TRAVERSE  (  xt list-id  --  )
		- Traverse the list performing the action of xt on each item.

-------------------------------------------------------------------------------
))

struct LIST-STRUCT
  int LIST-STRUCT.NEXT
  int LIST-STRUCT.ITEM
end-struct

: LIST-MAKE  (  --  list-id  ; creates a new empty list  )
  here LIST-STRUCT allot			\ create an empty node 
  NULL over LIST-STRUCT.NEXT !			\ this is a dummy node
;

: LIST-ADD  (  item list-id  --  ; adds a node to the end of the list  )
  Begin						\ get to the end of the list :-
    dup LIST-STRUCT.NEXT @ NULL <>		\ while not end of list
  While
    LIST-STRUCT.NEXT @				\   move to next node
  Repeat
  here LIST-STRUCT allot			\ allocate space for new node
  dup rot LIST-STRUCT.NEXT !			\ link this to end of list
  NULL over LIST-STRUCT.NEXT !			\ make new node last in list
  LIST-STRUCT.ITEM !				\ set the item for the new node
;

: LIST-TRAVERSE  (  ... xt list-id  --  ; traverse the list and executes xt upon each node item  )
  LIST-STRUCT.NEXT @				\ The first node is a dummy
  Begin
    dup NULL <>					\ while not end of list
  While
   2>r						\   hide local stack so stack can be used by the executed word
   2r@ LIST-STRUCT.ITEM @ swap execute		\   execute action on node item
   2r>                                          \   restore local stack
   LIST-STRUCT.NEXT @				\   move to next node
  Repeat
  2drop
;


((
-------------------------------------------------------------------------------
	Marker component management

	The marker works by preserving the current state at definition of the
	marker, and restore it when the marker is executed, rather than trying
	to undo any actions.

	A marker component is something that needs to be remembered as part
	of the action of marker and will need to be restored later when
	the marker is executed.  This may be something as simple as a pointer
	or as complex as a symbol table.

	For each type of component that MARKER has to deal with a separate
	preserve and restore action is required.

	For each component the preserve action will need to present the restore
	action with some data to act upon.  This data must be returned as the
	result of the preserve action as a single cell, what the cell actually
	represents is unspecified, it may be a simple value or a pointer to a
	more complicated data structure, but it must be processed by the
	restore	action accordingly.

	So a preserve action must have the following prototype:

		'preserve-something'  (  --  x  )

	Note that the preserve action must not compile data at HERE since the
	defining word MARKER (see below) will be using this area.

	A restore action must have the following prototype:

		'restore-something'  (  x  --   )

	To add a component to a marker use the following which will add the
	component to the necessary lists of action types.

	ADD-MARKER-COMPONENT  (  preserve-xt restore-xt  --  )
		- Adds the actions for preserve and restore for a marker
		  component.
 
-------------------------------------------------------------------------------
))

list-make constant PRESERVE-LIST
list-make constant RESTORE-LIST

: ADD-MARKER-COMPONENT  (  preserve-xt restore-xt  --  )
  RESTORE-LIST list-add
  PRESERVE-LIST list-add
;


((
-------------------------------------------------------------------------------
	The MARKER defining word
	
	The data for each preserve/restore action pair is kept in the parameter
	field of a MARKER.

	When creating a marker each preserve action is executed in turn and the
	result of this action is compiled into the child's parameter field.

	When executing a child of marker each restore action is executed in
	turn with the corresponding value extracted from the parameter field
	and passed to the restore action.

	All the restore actions and preserve actions are found by using the
	list traversal function upon the appropriate list.

-------------------------------------------------------------------------------
))


: PRESERVE-COMPONENT  (  xt  --  )
  execute ,
;

: RESTORE-COMPONENT  (  addr xt  --  addr+cell  )
  over @ swap execute
  cell+
;

: MARKER  (  "<spaces>name"  --  ;  --  )
  create
    ['] preserve-component PRESERVE-LIST list-traverse
  does>
    ['] restore-component RESTORE-LIST list-traverse
    drop
;


((
-------------------------------------------------------------------------------
	Basic MARKER components
-------------------------------------------------------------------------------
))

: preserve-dict  (  --  x  )
  latest
;

: restore-dict  (  x  --  )
  (forget)
;

' preserve-dict  ' restore-dict  add-marker-component





