\ Target memory control and definition.

((
Copyright (c) 1988-1999, 2000, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 2380 631441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

20200416 SFP013 Added 64 bit alignment.
20120612 SFP012 Updated comments.
20000601 SFP011 Changed HERE(T) to CHERE(T) as appropriate

        JL001   18 Sep 96
                Added scan-(t) fopr OTA Version 2.

        JL002   23 Sep 96
                Added target>host utility to copy memory from target space to host space.

        JL003   13 November 96
                Minor update for volume 2 draft 4.

        SFP004  10 Jan 97
                Modified Initialised RAM Table (IRT) handling

        SFP005  6 Feb 97
                Removed ALIGN to avoid ANS name clash - use ALIGN-EVEN instead

        SFP006  17 DEc 97
                Added ALIGN-LONG ALIGN-LONG-1 ALIGN-LONG-2 for 32 bit CPUs
                which need PFAs on 32 bit boundaries

        SFP007  20 Jan 98
                Changed ?FORCE-EVEN to force 16 and 32 bit boundaries according to alignment mode.
                Added FORCE-16 and FORCE-32

        MSD008  04 Apr 98
                Incorporate ARM changes into the BUILDKIT.

        SFP009  28 Apr 1998
                Made FORCE-RAM-ALIGN sensitive to ALIGNING

        SFP010  20 May 1999
                Added ALIGN-INLINE-DATA to force alignment of data in code space
))

\ file control

only forth also c-c definitions
decimal


: Bounding      \ prom-size mask min-space leading-offset --
  Prom-Offset !(m)  min-space !(m)  swap  Prom-Size !(m)
  Dp(T) @(m) and  Prom-Size @(m) + Next-Prom !(m)
  ?bounding on(m)  ;


\ Pseudo-ops -- Aligning                           sfp 27/02/89

(( Removed because of name conflict with ANS ALIGN      SFP005
: ALIGN         \ -- ; align CFAs to even boundaries
  2 ?ALIGNING !(m)  ;
))

: align-xword	\ -- ; align CFAs to 64 bit boundaries
  6 ?aligning !(m)  ;

: align-long    \ -- ; align CFAs to 32 bit boundaries          SFP006
  5 ?aligning !(m)  ;

: align-long-1  \ -- ; align CFAs to 32 bit boundary -1 byte    SFP006
  4 ?aligning !(m)  ;

: align-long-2  \ -- ; align CFAs to 32 bit boundary -2 byte    SFP006
  3 ?aligning !(m)  ;

: align-even    \ -- ; align CFAs to even boundaries
  2 ?aligning !(m)  ;

: align-odd     \ -- ; align CFAs to odd boundaries
  1 ?aligning !(m)  ;

: no-align      \ -- ; turn alignment off
  0 ?aligning !(m)  ;

: align-inline-data     \ -- ; align inline data - usually in headers
  1 ?aligning-inline !(m)  ;

: no-align-inline-data  \ -- ; do not align inline data - usually in headers
  0 ?aligning-inline !(m)  ;

: Rommed        \ -- ; tell system target is split ROM/RAM
  ?Rom on(m)  ;

: Rom?          \ -- flag ; true if rom target being generated
  ?Rom @(m)  ;


\ Memory reference -- J Lee .. 9/9/93

: Fill-c(t)       \ addr len c --
\ Fill target memory (CDATA)
  -rot  bounds
  ?do  dup i c!c(t)  loop
  DROP  ;

: erase-c(t)      \ addr len --
\ Zero target memory (CDATA)
  0 fill-c(t)  ;

: Blank-c(t)  \ addr len --
\ Fill target memory (CDATA) with spaces.
  bl fill-c(t)
;

: Fill-d(t)       \ addr len c --
\ Fill target memory (I/UDATA)
  -rot  bounds
  ?do  dup i c!d(t)  loop
  DROP  ;

: erase-d(t)      \ addr len --
\ Zero target memory (I/UDATA)
  0 fill-d(t)  ;

: Blank-d(t)  \ addr len --
\ Fill target memory (I/UDATA) with spaces.
  bl fill-d(t)
;


: cmove-c(t)      \ addr1 addr2 len --
\ copy target memory (CDATA to CDATA)
  ?DUP                                  \ if something
  IF  0 DO  OVER C@C(T) OVER C!C(T)     \ copy a byte
            SWAP 1+  SWAP 1+            \ step on
        LOOP                            \ till done
  ENDIF                                 \
  2DROP  ;                              \ lose addresses

: cmove-d(t)    \ addr1 addr2 len --
\ copy target memory (I/UDATA to IDATA)
  ?DUP                                  \ if something
  IF  0 DO  OVER C@D(T) OVER C!D(T)     \ copy a byte
            SWAP 1+  SWAP 1+            \ step on
        LOOP                            \ till done
  ENDIF                                 \
  2DROP  ;                              \ lose addresses

: cmove>-d(t)	\ addr1 addr2 len --
\ copy target memory, last byte first (I/UDATA to I/UDATA)
  dup >r  +  swap r@ + swap  r> 0 ?do	\ -- addr1+len addr2+len
    swap 1-  swap 1-			\ step back
    over c@d(t) over c!d(t)		\ copy a byte
  loop
  2DROP  ;                              \ lose addresses

: cmove-(t)      \ addr1 addr2 len --
  ?DUP                                  \ if something
  IF  0 DO  OVER C@D(T) OVER C!D(T)     \ copy a byte - ????? data only ?????
            SWAP 1+  SWAP 1+            \ step on
        LOOP                            \ till done
  ENDIF                                 \
  2DROP  ;                              \ lose addresses

: Fill-(t)  \ addr len c --
  -rot bounds
  ?Do  dup i c!d(t)  Loop
  drop
;

: Erase-(t)  \ addr len --
  0 fill-(t)
;

: Blank-(t)  \ addr len --
  bl fill-(t)
;

: +string-(t)  ( addr1 len1 addr2 len2 -- addr2 len3 )
  2dup >r >r
  +
  swap dup >r
  cmove-(t)
  r> r> r> rot +
;

: c+string-(t)  ( char c-addr len -- c-addr len+1 )
  2dup +
  >r rot r> c!d(t)
  1+
;

\ JL001
: scan-(t)  (  c-addr1 len1 char  --  c-addr2 len2  )
  >r                                    \ preserve char
  begin
    over c@d(t) r@ =  >r                \ if we haven't found the character?
    dup 0=                              \ and we haven't run out of characters?
    r>  or invert
  while
    1- swap 1+ swap                     \   decrement length, increment address
  repeat
  r> drop                               \ throw away original char
;

\ JL003
: skip-(t)  (  c-addr1 len1 char  --  c-addr2 len2  )
  >r                                    \ preserve char
  begin
    over c@d(t) r@ <>  >r               \ if we haven't found the character?
    dup 0=                              \ and we haven't run out of characters?
    r>  or invert
  while
    1- swap 1+ swap                     \   decrement length, increment address
  repeat
  r> drop                               \ throw away original char
;


\ Memory reference                                 Jon 09/08/88

: CMOVE-C,C(T)  \ addr n -- ; lay host string into target CDATA
  bounds
  ?DO  I C@ C,C(T) LOOP  ;

: CMOVE-C,D(T)  \ addr n -- ; lay host string into target IDATA
  bounds
  ?DO  I C@ C,D(T) LOOP  ;

: ALLOT-C(T)      \ n --
  ?DUP
  IF  DUP 0<
      IF  DP(T) +!(m)
      ELSE  HERE SWAP CMOVE-C,C(T)
      ENDIF
  ENDIF  ;

: allot-c-0(t)    \ n --
  dup 0<
  if  allot-c(t)
  else  0
        ?do  0 c,c(t)  loop
  endif  ;

: TOGGLE-C(T)   \ addr mask
  OVER C@C(T) XOR SWAP C!C(T)  ;

: ?BOUND	\ -- ; SFP001
  ?BOUNDING @(m)
  IF  NEXT-PROM @(m) CHERE(T) U<
      IF  PROM-SIZE @(m) NEXT-PROM +!(m)  ENDIF
      NEXT-PROM @(m) CHERE(T) -  MIN-SPACE @(m) U<
      IF  BASE @  HEX
          BEGIN  CONSOLE  CR ." Target DP = "  CHERE(T) U.
                 ."  -- Step to next PROM ? "  Y/N? DUP
                 IF  NEXT-PROM @(m) PROM-OFFSET @(m) + DP(T) !(m)
                     PROM-SIZE @(m) NEXT-PROM +!(m)
                 ENDIF
                 SET-LOG-MODE   \ restore printer mode
          0= UNTIL
          BASE !
      ENDIF
  ENDIF  ;


\ target>host
\

: target>host  ( c-addr1 c-addr2 len -- ; copy from target address c-addr1 to host adddress c-addr2 )
  -rot swap rot  0                              \ swap c-addr1 and c-addr2 to better suit loop
  ?Do                                           \ for all bytes
    2dup  c@d(t) swap c!  1+ swap 1+ swap       \   copy from target to host, step both addresses
  Loop                                          \
  2drop                                         \ discard unwanted addresses
;


\ Memory reference                                 sfp 27/02/89

: Force-Even  ( -- ; force DP to an even address for 16 bit alignment )
  chere(t) 1 and
  If  addr(t)/byte callot(t)  Endif
;

\ sfp007...
: force-16      \ fill-byte -- ; force 16 bit boundary
  chere(t) 1 and
  if  c,c(t)  else  drop  endif
;

: force-32      \ fill-byte -- ; force 32 bit boundary
  begin
    chere(t) 3 and
   while
    dup c,c(t)
  repeat
  drop
;

: force-64      \ fill-byte -- ; force 64 bit boundary
  begin
    chere(t) 7 and
   while
    dup c,c(t)
  repeat
  drop
;

: ?FORCE-EVEN   \ b -- ; force even memory address if required
  case  ?aligning @(m)                  \ check alignment mode required
    1 of  force-16  endof
    2 of  force-16  endof
    3 of  force-32  endof
    4 of  force-32  endof
    5 of  force-32  endof
    6 of  force-64  endof
          nip                           \ discard unused fill-byte
  endcase
;
\ ...sfp007

\ MPEL -- 14/4/93
: Force-RAM-Even  ( -- ; force RP to even address )
  ihere(t) 1 and
  If  1 iallot(t)  Endif
;

\ *jl* 13/20/95 -- force initialised ram to be cell aligned
: Force-RAM-Align       \ -- ; force RP to cell aligned address SFP009
  ?aligning @(m) if
    ihere(t) cell(t) 1- and dup 0<> If
      cell(t) swap - iallot(t)
    Else
      drop
    Endif
  endif
;

\ *jl* 13/20/95 -- force initialised ram to be cell aligned
: Force-URAM-Align      \ -- ; force RP to cell aligned address  SFP009
  ?aligning @(m) if
    uhere(t) cell(t) 1- and dup 0<> If
      cell(t) swap - uallot(t)
    Else
      drop
    Endif
  endif
;

: ?even(r)      \ -- ; allot RAM space if required
  ihere(t) 1 and  ?aligning @(m) 0<> and
  if  addr(t)/byte iallot(t)  endif  ;

: ?force-align  \ -- ; align CFA so that PFA is on selected byte boundary
  case  ?aligning @(m)
    1 of  chere(t) 1 and 0= if  bl c,c(t)  endif  endof
    2 of  chere(t) 1 and if  bl c,c(t)  endif  endof
    3 of  begin
            chere(t) 3 and 2 <>         \ long-2 required
           while
            bl c,c(t)
          repeat
      endof
    4 of  begin
            chere(t) 3 and 3 <>         \ long-1 required
           while
            bl c,c(t)
          repeat
      endof
    5 of  begin
            chere(t) 3 and 0<>          \ long required
           while
            bl c,c(t)
          repeat
      endof
    6 of  begin
            chere(t) 7 and		\ xword required
           while
            bl c,c(t)
          repeat
      endof
  endcase  ;

( -- ; insert character as required )
: ?BL-EVEN      BL ?FORCE-EVEN  ;
: ?0-EVEN       0  ?FORCE-EVEN  ;

: ?align-inline \ -- ; pad inline data                  \ SFP010
  ?aligning-inline @(m)
  if  0 ?force-even  endif
;

