\ general output words

Pcb Output-pcb				\ hold output filename and handle
Variable Output-buff			\ temporary output buffer

\ line delimiters - now built into PFW 2.1
$0d constant cr-chr
$0a constant lf-chr


\ open and close

: Create-output-file	\ addr$ -- ; create output file
  Output-pcb set-pathname		\ set pathname
  Output-pcb Create-path-pcb		\ create file
  If
   cr ." Can't create log file" ABORT
  Endif
;

: Close-output-file	\ -- ; create output file
  Output-pcb Close-path-pcb		\ close file
  If
   cr ." Can't close log file" ABORT
  Endif
;


\ output block = type (but quicker)

: (output-block)	\ addr len --
  dup >r				\ preserve length for error check
  Output-pcb handle			\ write buffer
  Write-path				\
  0<>					\ status good?
  If
    cr ." Can't write to log file" ABORT	\ NO: error
  Endif
  r> <>					\ correct length?
  If
    cr ." Disk Full" ABORT		\ NO: error
  Endif
;


: output-block		\ addr len  -- ; send len chars to file
  2dup over		     		\ addr len addr len addr
  'tib @ dup #tib @ + dup >r		\ get lower and upper limit of block
  within?				\ are we writing from block?
  if					\ YES:
    + r> >				\   check we haven't exceeded upper limit
    if					\   YES we have
      cr ." block overflow" ABORT	\     fail
    endif				\
  else					\ NO
    2drop r> drop			\   assume its OK and clean up
  endif 				\
  (output-block)			\ write block
;

: Output-type		\ addr len -- ; send characters to file
  dup out +!
  output-block
;

: Output-emit		\ char -- ; send character to file
  Output-buff c!			\ save character in buffer
  Output-buff 1
  (output-block)			\ and write it (no buffer check)
  out incr
;

: Output-cr		\ -- ; send characters to file
  Cr-chr Output-Emit			\
  Lf-chr Output-Emit			\
  out off
;

: get-output-drivers  (  --  cfa{cr} cfa{emit} cfa{type}  ; return the output driver cfas  )
  action-of cr
  action-of emit
  action-of type
;

: set-output-drivers  (  cfa{cr} cfa{emit} cfa{type}  --  ;  set output drivers  )
  to-do type
  to-do emit
  to-do cr
;

: >Output		\ -- cfa1 cfa2 cfa3 ; redirect console to output file
  Action-of Cr				\ get current type
  Action-of Type			\ get current type
  Action-of Emit			\ get current emit
  Assign output-emit to-do emit		\ redirect output
  Assign output-type to-do type		\ redirect output
  Assign output-cr   to-do cr		\ redirect output
;

: >Console		\ cfa1 cfa2 cfa3 -- ; restore driver
  to-do emit				\ redirect output
  to-do type				\ redirect output
  to-do cr				\ redirect output
;


\ *jl* question sort out the log handler for PFW!!!!
\ Log to disk words

\ Set-Log-mode

\ these contain the CFAs of the original output handlers
variable output-cfa1
variable output-cfa2
variable output-cfa3

\ buffer for log filename, set a default filename
create log_filename  256 allot
  c" LOG.TXT" log_filename $move


: Set-Log-Mode          \ -- ; sets previous log mode
  LogOut @(m)                           \ which mode ?
  Case
    0 Of  console  Endof                               \ Case: CONSOLE
    1 Of  cr ." Can't log to printer" ABORT  Endof     \ redirect & sel. printer
    2 Of  output-cfa3 @  output-cfa2 @  output-cfa1 @ >console  Endof
  EndCase
;

: Open-Log      \ -- ; open log file according to type
  case  logout @(m)                     \ test type
    1  of  cr ." Can't log to printer" ABORT  endof            \ printer
    2  of  log_filename create-output-file
           >output  output-cfa1 !  output-cfa2 !  output-cfa3 !  \ redirect drivers, remember old ones
       endof
  endcase
;

: Close-Log     \ -- ; close log file
  case  Logout @(m)                                             \ test log type
    1 Of  cr ." Can't log to printer" ABORT  Endof              \ printer
    2 Of  close-output-file                                     \ file, so close it
          output-cfa3 @  output-cfa2 @  output-cfa1 @ >console  \ switch o/p back
      Endof
  endcase
  logout off(m)                                                 \ and back to cons.
;


\ log switches ..........................          Jon 07/02/89

only forth also c-c also forth definitions

: File          \ --
  cr ." Symbol log file is " log_filename $.
  2 LogOut !(m)                         \ LogOut := FILE
;

: File:         \ -- ; <FileName>
  bl word dup count upper log_filename $move
  file                          	\ set LogOut for file
;

: Con:          \ --
;

: Prn:          \ -- ; display log on printer
  cr ." Logging to Printer not supported ... using console"
;

only forth also c-c definitions

: Log           \ --
  ?Log on(m)  ;

: No-Log        \ --
  ?Log off(m)  ;

: logging?      \ -- flag ; true if log turned on
  ?Log @(m)  ;

only forth definitions

