\ TSEPOS 62 17 2 \

( *! portio                                                             )
( *T Hardware Level Port IO Access                                      )
( **                                                                    )
( *P This library supports direct hardware access.                      )
( ** Under Windows 95/98 machines there is no restriction for the reading)
( ** and writing of hardware IO locations. Under NT the two instructions)
( ** are normally forbidden to user applications. For work under NT a   )
( ** driver is provided in your ProForth installation as                )
( ** %VFX%\XTRA\NTPORT.EXE                                              )
( **                                                                    )

struct OSVERSIONINFO
        cell    field   dwOSVersionInfoSize
        cell    field   dwMajorVersion
        cell    field   dwMinorVersion
        cell    field   dwBuildNumber
        cell    field   dwPlatformId
        128     field   szCSDVersion
end-struct

: GiveIO        \ --
  z" \\.\giveio"
  GENERIC_READ
  0
  NULL
  OPEN_EXISTING
  FILE_ATTRIBUTE_NORMAL
  NULL CreateFile
  dup INVALID_HANDLE_VALUE = if
    cr ." Can't get access to IO Driver."
    cr ." Install the driver from the COMPILER\XTRA directory, and then"
    cr ." Execute the word PIO-TEST to confirm installation."
    abort
  endif
  CloseHandle drop
;

: pio-init      \ --
\ *G This definition should be called once by your application to       )
\ ** provide port access. It's job is to autodetect NT at runtime and
\ ** attempt to access the port io driver to grant hardware access
\ ** privilege.
  { | osv[ OSVERSIONINFO ] -- }
  OSVERSIONINFO osv[ dwOSVersionInfoSize !
  osv[ GetVersionEx if
    osv[ dwPlatformId @ VER_PLATFORM_WIN32_NT =
    if  GiveIO  then
  then
;

: nt-access-ports	\ --
  pio-init
;

[undefined] pc@ [if]
code pc@        \ port -- b ; read port
( *G Read a byte from the hardware control port supplied.               )
  mov   edx, ebx                \ port address must be in DX
  in    al, dx                  \ read port
  movzx ebx, al                 \ clear EBX
  next,                         \ back to Forth
end-code
[then]

[undefined] pc! [if]
code pc!        \ b port -- ; write port
( *G Write the supplied byte to the selected hardware control port.     )
  mov   edx, ebx                \ port address must be in DX
  mov   eax, 0 [ebp]            \ get NOS (b)
  out   dx, al                  \ write port
  mov   ebx, 4 [ebp]            \ restore TOS
  add   ebp, 8
  next,                         \ back to Forth
end-code
[then]

: PlayNote      \ freq duration --

  1193180 rot /
  $61 pc@ $3 or $61 pc!         \ Start Speaker Going

  $B6 $43 pc!                   \ set timer freq.
  dup $42 pc!
  8 rshift $42 pc!

  Sleep                         \ Sleep for duration of note

  $61 pc@ $FC and $61 pc!       \ Switch off speaker
;

: pio-test      \ --
( *G A test routine for hardware access. After a succesful PIO-INIT     )
( ** this definition can be used to confirm access. If you hear a       )
( ** familiar tune, all is well!                                        )

   897   500 PlayNote
  1007   500 PlayNote
   800   500 PlayNote
   400   500 PlayNote
   599  1000 PlayNote
;

( *> ###                                                                )

