\ PREASM.FTH - Forth cross assembler preamble

((
Copyright (c) 2002, 2010, 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441


To do
=====

Change history
==============
20170704 SFP002 64 bit update
20100212 SFP001 Corrected ?UNCONS.
))


only forth  also c-c  also assembler definitions

: .uncons	\ symbol -- ; SFP001
  cr ." At target address " cell+ @(s) .dword
  err_uncons throw
;

: ?UNCONS
  REF1 @(m) ?dup
  IF  .uncons  REF1 off(m)  ENDIF	\ SFP001
  REF2 @(m) ?dup
  IF  .uncons  REF2 off(m)  ENDIF	\ SFP001
;


\ ***********************************************************************
\ load these to redefine C,(T) W,(T) and L,(T) to display compiled values
\ at assembly time for debugging
\ ***********************************************************************

variable showing	\ -- addr ; true if displaying code
  showing off

also cc/i
: +show-code    cc/i	\ -- ; code will be displayed
  showing on  ;

: -show-code    cc/i	\ -- ; code will not be displayed
  showing off  ;
previous definitions

[undefined] .xword [if]
: .xword	\ x -- ; change on 64 bit host
  ." xxxx:xxxx:" .dword
;
[then]

: ?cr           \ --
  out @ #72 u>
  if  pcr  endif  ;

: c,c(T)         \ b --
  showing @
  if  ?cr  DUP .BYTE  endif
  c,c(T)  ;

: w,c(T)         \ u --
  showing @
  if  ?cr  DUP .WORD  endif
  w,c(T)  ;

: l,c(T)         \ u --
  showing @
  if  ?cr  DUP .LWORD  endif
  l,c(T)  ;

: x,c(T)         \ u --
  showing @
  if  ?cr  DUP .XWORD  endif
  x,c(T)  ;


\ *************************
\ Assembler list-file stuff
\ *************************

decimal

pcb log-file	\ -- addr ; the log file
  log-file reset-pcb			\ turn off for overlay

create cr-lf    \ -- addr ; data to make cr/lf in file
  13 c, 10 c,                   	\ a cr by any other name

variable asm-debug	\ -- addr ; says whether debugging or not

: space-pad	\ -- ; move pile of spaces to here
  pad off                               \ turn off
  30 out @ - 0 max dup pad c!           \ generate count and count byte
  pad 1+ swap bl fill                   \ fill rest with spaces
;

: write-text    \ addr len -- ; write data to log
  log-file handle write-path 2drop
;

hex

create sp	\ -- addr ; the space string, for writing to file
  bl c,

: .hex		\ n -- ; used for log file only
  3 out +!                              \ increment counter
  base @ >r hex                         \ preserve base
  $FF and s>d <# # # #> write-text	\ write number
  sp 1 write-text                       \ write space
  r> base !                             \ restore base
;

: log-line      \ -- ; maybe log the i/p line to the file
  asm-debug @ if
    space-pad                           \ work out how many spaces
    pad 1+ pad c@ write-text            \ pad to col.30
    tib #tib @ write-text               \ write text to file
    cr-lf 2 write-text                  \ and a new-line
    out off                             \ back to start of line
  endif
;

also forth definitions

: list:		\ <name> -- ; init debug log
  bl word                               \ get name
  log-file set-pathname                 \ set filename
  asm-debug on                          \ set flag to say so
  log-file create-path-pcb drop         \ create the file first
;

: end-list	\ -- ; end debug log
  asm-debug off                         \ finished debug
  log-file close-path-pcb drop          \ close file
;

previous definitions

: x,c(t)        \ 32b --
  asm-debug @ if
    dup space .xword
  endif
  x,c(t)                                \ 64b forth
;

: l,c(t)        \ 32b --
  asm-debug @ if
    dup space .dword
  endif
  l,c(t)                                \ 32b forth
;

: w,c(t)        \ 16b --
  asm-debug @ if
    dup space .word
  endif
  w,c(t)
;

: c,c(t)        \ 8b --
  asm-debug @ if
    dup space .byte
  endif
  c,c(t)
;


\ ***************************
\ general assembler variables
\ ***************************

hex

variable <prefix>               \ prefix/postfix mode
  <prefix> on                   \ default to prefix
create aprior  0 , 0 ,          \ see ?prefix
create last-pos 0 , 0 ,         \ holds BLK & >IN
variable pos-#tib               \ -- addr ; holds previous #TIB
create pos-buff  $0100 allot    \ -- addr ; holds previous TIB

also c-c definitions
defer a-reset	\ -- ; assembler reset
previous definitions


\ ************************************
\ PREFIX and POSTFIX notation handling
\ ************************************

\ This function is used to allow opcodes to be before (conventional)
\ or after (old Forth style) the operands. If the <PREFIX> flag is set
\ a previously stored routine is executed, thus effectively putting off
\ the execution of an opcode's function until the next opcode is encountered.
\
\ The basis of this code was presented by Bob Smith at the
\ November 1987 FORML Conference - thanks.
\ Rewritten to avoid return stack issues Feb 2002

: prefix?       \ -- t/f ; true if in prefix mode
  <prefix> @  ;

(( Original Bob Smith version
: ?prefix       \ struct -- struct' ; executes previous opcode
  prefix? if
    r>                                  \ struct ret-addr --
    aprior 2@  2swap  aprior 2!         \ exchange with contents of APRIOR
    >r                                  \ will use previous return address
  endif
;
))
: (?prefix)	\ struct -- struct' ; executes previous opcode
\ This word assumes that the return address is on the top of the
\ return stack and that an xt is inline. It exits the caller.
  r> @					\ -- struct xt
  prefix? if
    aprior 2@  2swap  aprior 2!         \ exchange with contents of APRIOR
  endif
  execute				\ use previous xt
;

: ?prefix	\ -- ; finish and start nameless word
\ This word assumes that the return address is on the top of the
\ return stack and that it can compile an xt inline.
  postpone (?prefix)  here 0 ,		\ (?PREFIX) gets xt inline
  state off  smudge
  :noname swap !  !csp
; immediate

[defined] disassembler [if]
also disassembler
[+switch dasm-switch
  ' (?prefix) run:
    disinit @long disclose
    s" ?PREFIX" >disLine
    set-Disaddr  ;
switch]
previous
[then]

also cc/i

: prefix        cc/i	\ -- ; select prefix notation
  <prefix> on  ;

: postfix       cc/i	\ -- ; select postfix notation
  <prefix> off  ;

: prefix?       cc/i	\ -- t/f ; true if in prefix mode
  <prefix> @  ;

previous definitions

: null-op       \ -- ; force execution of previous opcode
  0 ?prefix drop  a-reset  ;

: kickoff       \ n -- ; dummy operator
  drop  a-reset  ;

: set-null      \ -- ; initialise APRIOR
  0  ['] kickoff aprior 2!  ;


\ **************
\ Error handling
\ **************

: save-pos      \ -- ; preserves opcode position in text
  blk @  >in @  last-pos 2!
  blk @ 0= if				\ if not in screen file
    tib pos-buff #tib @ $100 min cmove	\ save TIB
    #tib @ $100 min pos-#tib !		\ save #TIB
  endif  ;

: restore-pos   \ -- ; restore text position
  last-pos 2@  >in !  dup blk !  0=     \ if in text file
  if  pos-buff tib pos-#tib @ $100 min cmove \ restore TIB
      pos-#tib @ #tib !
  endif  ;

: a-error       \ n -- ; error exit
  prefix?
  if  restore-pos  endif
  throw  ;

: ?a-error      \ t/f n -- ; error n if t/f true
  swap
  if  a-error  else  drop  endif  ;


\ ******************************
\ Assembler startup and shutdown
\ ******************************

also c-c definitions

: ASM_START     \ -- ; initialise assembler
  set-null  save-pos  a-reset  ;

: ASM_END       \ -- ; reset assembler
  null-op  ;

defer flush-assembler	\ -- ; flush assembler if multi-opcode, e.g. VLIW
  ' noop is flush-assembler

previous definitions


\ ***********
\ File output
\ ***********

#32 constant text-start         \ -- n ; start position of text in listing

: .show-code    \ -- ; adds text to code laid by instruction
  showing @                             \ debugging info needed?
  if  text-start out @ - 0 max spaces   \ tab to start of text display
      prefix?                           \ source of text is different
      if  pos-buff pos-#tib @ type      \  if in prefix
      else  tib #tib @ type             \  or in postfix
      endif
      pcr here(t) .LWORD space		\ show program counter location
  endif
;


\ *******************************************************
\ Macros - because of the prefix notation, some assembler
\ state needs to be preserved across macros, especially
\ what is to be done when the macro finishes. See the
\ earlier section on PREFIX notation
\ *******************************************************

decimal

32 constant #mb-entries                 \ macro nesting permitted
3 cells constant /mb-entry              \ 3 cells per entry
#mb-entries /mb-entry *                 \ size of macro buffer
  constant mb-size

create mb                               \ macro buffer
  mb-size allot&erase

: push-macro    \ -- ; save APRIOR and STATE(T), put null into APRIOR
  mb mb mb-size /mb-entry /string cmove>  \ copy buffer
  aprior 2@ mb 2!                       \ save APRIOR
  state(t) @(m) mb 2 cells+ !           \ save STATE
  2 state(t) !(m)                       \ force assembler mode
  set-null  ;                           \ insert null into assembler

: pop-macro     \ -- ; restore APRIOR and STATE(T)
  mb 2 cells+ @ state(t) !(m)           \ restore STATE
  mb 2@ aprior 2!                       \ restore APRIOR
  mb /mb-entry +  mb                    \ source and dest
  mb-size /mb-entry -  cmove            \ slide buffer back
;

: (start-macro) \ -- ; compiles prologue code
  postpone 0  postpone ?prefix  postpone drop      \ 0 ?prefix drop
  postpone push-macro
;

: (end-macro)   \ -- ; compiles epilogue code
  postpone null-op  postpone pop-macro	\ end of macro
;

: (macro:)      \ -- sys ; builds a macro header
  : (start-macro)			\ : is not IMMEDIATE in VFX
;

: (;macro)      \ sys --
  postpone null-op  postpone pop-macro    \ end of macro
  postpone ;
;

also c-c definitions

: macro:        \ -- sys
  host-mode on(m)
  only forth  also c-c  also assembler  also asm-access definitions	\ word is in assembler
  (macro:)
;  immediate

: in-line:      \ -- sys
  also postpone cc/c		\ word is in COMPILER
  (macro:)
;  immediate

: mc_gen:       \ -- sys ; builds micro code generator in C-C
  also c-c definitions
  (macro:)
;  immediate

: start-macro   \ -- ; used by code generators
  postpone null-op  postpone push-macro
  also asm-access
;  immediate

: end-macro     \ -- ; used by code generators
  (end-macro) previous  ;  immediate

: MASM:		\ --
  saveSO
  only forth  also c-c  also assembler  also asm-access definitions	\ word is assembler
  host-mode on(m) (macro:)
;

also asm-access definitions

: ;MASM		\ --
  (;macro)  host-mode off(m)		\ end of macro
  restoreSO
; immediate

: ;m		\ sys --
  (;macro)  host-mode off(m)		\ end of macro
  previous definitions			\ restore search order
;  immediate

: $		\ -- addr
  chere(t)
;

previous definitions


\ ********************
\ Assembler references
\ ********************

: asm-ref	\ symbol -- value
  state(t) @(m) >r  2 state(t) !(m)
  0 ref
  r> state(t) !(m)
;

: asm-run-ref	\ symbol -- value
  state(t) @(m) >r  2 state(t) !(m)
  0 run-time-ref
  r> state(t) !(m)
;


\ ********
\ all done
\ ********

only forth  also c-c definitions

decimal

