\ Disassembler prefix code for MPE v6 cross compilers

((
File: PREDASM.FTH

Copyright (c) 1999, 2002, 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: tech-support@mpeforth.com

Change History
==============
20080214 SFP001 VFX changes.
))


\ ***************
\ Auxiliary words
\ ***************

: digits        \ d n -- d' ; add n digits to the output stream
  0
  ?do  #  loop
;

: <C            \ char --- n ; sign extend byte to 32bits
  DUP $7F >
  IF  $FFFFFF00 OR  THEN
;

: <w            \ w -- n ; sign extend word to 32 bits
  dup $07FFF >
  if  $FFFF0000 or  endif
;

: u<=           \ u1 u2 -- flag
  u> not
;

: u>=           \ u1 u2 -- flag
  u< not
;

: .ise          \ -- ; displays an internal software error
  cr ." XDASM Internal software error"
  cr ." probably in word: "  r> ip>nfa .name
  cr abort
;

[undefined] buffer: [if]
: buffer:
  create
    allot&erase
;
[then]


\ *************************
\ DISASSEMBLER main portion - target address handling
\ *************************

VARIABLE (DisAddr)              \ current disassembly address
VARIABLE ToDisAddr              \ to address in from to words
variable StartAddr              \ address of start of opcode
variable branch-target          \ highest branch target address
variable next-found             \ true when NEXT found
variable datasize               \ this instruction's size, 0=byte, 1=word,2=long


: DisAddr       \ -- addr ; current disassembly address
   (DisAddr) @ ;

: SET-DisAddr   \ addr -- ; set current disassembly addr to addr
   (DisAddr) ! ;

: +DisAddr      \ n -- ; increment current disassembly address by n
   (DisAddr) +! ;

: >branch-target        \ addr -- ; insert if higher than current value
  dup branch-target @ u>
  if    branch-target !
  else  drop
  endif
;


\ **********************
\ opcode stream handling
\ **********************

#32 constant /ByteStream        \ -- n ; maximum number of bytes to buffer

/ByteStream buffer: ByteStream  \ -- addr ; buffer for stored opcodes

variable ^Stream        \ -- addr ; holds next free slot in byte stream
  ByteStream ^Stream !

: byte>stream   \ b -- ; add byte to stream
  ^Stream @ c!  1 ^Stream +!
;

: INIT-VARS     \ --
  ByteStream /ByteStream erase
  ByteStream ^Stream !
  2 datasize !				\ default to long for Coldfire
;

: .ByteStream   \ -- ; dump the contents of the opcode stream
  ^Stream @ ByteStream
  ?do  i c@ .byte  loop
;


\ ******************
\ DISASSEMBLY OUTPUT
\ ******************

32bit? 64bit? or [if]
#36 CONSTANT DisOpCol    \ starting column for opcode
#10 CONSTANT DisParmCol  \ offset of parameters after the opcode
[then]
16bit? [if]
#28 CONSTANT DisOpCol    \ starting column for opcode
#10 CONSTANT DisParmCol  \ offset of parameters after the opcode
[then]

#128 Buffer: (DisLine$) \ line buffer for disassembly
VARIABLE DisPos         \ output position for opcode info

: DisLine$      \ -- addr cnt
   (DisLine$) 80 ;

: >DISLINE      \ adr cnt -- ; Add opcode information to disassembly line buffer.
   DisLine$ DisPos @ /STRING
   ROT MIN DUP
   DisPos  +!                           \ increment print pos in dis buffer
   CMOVE ;

: ShowDisLine	\ -- ; testing only
  (DisLine$) 64 dump
;


\ **************
\ Numeric output
\ **************

: unibble>dis	\ nibble
  0 <# # #> >DISLINE  ;

: UBYTE>DIS	\ byte --
  0 <# 2 digits #> >DISLINE ;

: UWORD>DIS	\ word --
  0 <# 4 digits #> >disline  ;

: U24b>dis	\ 24b --
  0 <# 6 digits #> >disline  ;

: ULONG>DIS	\ long --
  0 <# 8 digits #> >disline  ;

: snibble>dis	\ nibble
  s>d tuck  dabs <# # #s rot sign #> >DISLINE  ;

: SBYTE>DIS	\ byte --
  s>d tuck  dabs <# # #s rot sign #> >DISLINE ;

: SWORD>DIS	\ word --
  s>d tuck  dabs <# # #s rot sign #> >disline  ;

: s24b>dis	\ 24b --
  s>d tuck  dabs <# # #s rot sign #> >disline  ;

: sLONG>DIS	\ long --
  s>d tuck  dabs <# # #s rot sign #> >disline  ;

: $>dis         \ string --
  count >disline  ;


\ ****************
\ Character output
\ ****************

variable char-holder

: CHAR>DIS      \ char --
   char-holder c!
   char-holder 1 >DISLINE  ;

: [>DIS         \ --
   [char] [ CHAR>DIS ;

: ]>DIS         \ --
   [char] ] CHAR>DIS ;

: :>DIS         \ --
   [char] : CHAR>DIS ;

: +>DIS         \ --
   [char] + CHAR>DIS ;

: ->DIS         \ --
   [char] - CHAR>DIS ;

: (>dis         \ --
  c" (" $>dis  ;

: )>dis         \ --
  c" )" $>dis  ;

: d(>dis        \ --
  c" d(" $>dis  ;	\ )

: bl>dis        \ --
  bl char>dis  ;

: ">dis        \ --
  [char] " char>dis  ;

: ,>DIS         \ --
  c" , " $>dis  ;

: #>dis         \ --
  c" # " $>dis  ;


: DISINIT       \ -- ; Initialize disassembly output buffer to current
                \ address and output positions for dump and opcode.
  DisLine$ BLANK                        \ initialise o/p buffer
  DisPos OFF                            \ no characters in buffer
  init-vars
  DisAddr StartAddr !
;

: disclose      \ -- ; add finishing text
;


\ *************************************
\ Fetching opcodes from the code stream
\ *************************************

: @BYTE         \ -- byte
  DisAddr C@c(t)
  dup byte>stream
  1 +DisAddr  ;

: @WORD         \ -- word
  @byte @byte
  [ also sections.voc ] 2bytes>word [ previous ]  ;

: @24b          \ -- 00aabbcc
  big-endian? if
    0 @byte @byte @byte
  else
    @byte @byte @byte 0
  endif
  [ also sections.voc ] 4bytes>long [ previous ]  ;

: @LONG         \ -- long
  @byte @byte @byte @byte
  [ also sections.voc ] 4bytes>long [ previous ]  ;


\ *****************
\ Immediate display
\ *****************

: .imm3         \ b --
  7 and  #>dis  unibble>dis  bl>dis  ;

: .imm4s        \ nibble --
  $0F and
  #>dis  snibble>dis  bl>dis  ;

: .imm8s        \ b --
  #>dis  sbyte>dis  bl>dis  ;

: .imm16s       \ w --
  #>dis  sword>dis  bl>dis  ;

: .imm32s       \ l --
  #>dis  slong>dis  bl>dis  ;

: .imm4u        \ nibble --
  $0F and
  #>dis  unibble>dis  bl>dis  ;

: .imm8u        \ b --
  #>dis  ubyte>dis  bl>dis  ;

: .imm16u       \ w --
  #>dis  uword>dis  bl>dis  ;

: .imm32u       \ l --
  #>dis  ulong>dis  bl>dis  ;


\ **************
\ Opcode display
\ **************

: parmtab>dis	\ -- ; tab to start of arguments
  DisParmCol DisPos ! ;

: .op		\ -- ; just outputs the text
  create  ",  does> $>dis  ;

: .optab	\ -- ; outputs the text and tabs
  create  ",  does> $>dis  parmtab>dis  ;

.optab .invalid Invalid opcode"

