(( PRIMSYMS.FTH - Primitives needed to build the XCompiler kernel

Copyright (c) 1992-96, 1998, 2016

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
net: mpe@mpeforth.com

First Release:

Changes:        See RELEASE.DOC for full details

20200618 SFP006 64 bit host update.
20160803 SFP005	Added high-level version of V-FIND(S).
		No measurable speed change!
SFP004  15 Nov 1998     Added XREF-ANCHOR field
SFP003  28 Sep 1998     Fixed V-FIND(S) to get SMUDGE bit from STATUS field
SFP002  13 Sep 1998     Modified V-FIND(S) to allow for code length field
JL001   23 Aug 96       Modified V-FIND(S) to check for SMUDGE bit in symbol status field.
                        Symbol won't be found in smudge bit is set, even if correct name.
        March '95       Support ProForth for Windows 3.1x
))


only forth also c-c definitions
decimal


hex

variable symbol-table   \ -- addr ; holds address of the symbol table

: @(s)       \ offset -- n ; fetch word from symbol table
  symbol-table @ + @
;

: !(s)		\ n offset -- ; write into symbol table
  symbol-table @ + !
;

cellbits 64 = [if]	\ SFP006
: l@(s)       \ offset -- n ; fetch 32 bits from symbol table
  symbol-table @ + l@
;

: l!(s)		\ n offset -- ; write 32 bits into symbol table
  symbol-table @ + l!
;
[then]

: w@(s)		\ offset -- w ; fetch 16bit word from symbol table
  symbol-table @ + w@
;

: w!(s)		\ w offset -- ; write 16bit word into symbol table
  symbol-table @ + w!
;

: c@(s)		\ offset -- n ; fetch byte from symbol table
  symbol-table @ + c@
;

: c!(s)		\ b offset -- ; write byte into symbol table
  symbol-table @ + c!
;

: 2@(s)         \ addr -- d
  dup cell+ @(s)  swap @(s)  ;

: 2!(s)         \ d addr --
  >r  r@ !(s)  r> cell+ !(s)  ;

: on(s)		\ offset -- ; set cell in symbol table to -1
  symbol-table @ + on
;

: off(s)	\ offset -- ; set cell in symbol table to 0
  symbol-table @ + off
;

: +!(s)		\ n offset -- ; add n to cell in symbol table
  symbol-table @ + +!
;

: set-bit(s)    \ mask offset --
  symbol-table @ + set-bit  ;

: fill(s)       \ offset len char --
  rot symbol-table @ + -rot fill  ;

: cmove>(s)     \ s-off d-off len -- ; source in dictionary, dest in symbols
  swap symbol-table @ + swap       \ dest in symbol table
  cmove>  ;

: cmove(s)>     \ s-off d-off len -- ; source in symbols, dest in dictionary
  rot symbol-table @ + -rot        \ dest in symbol table
  cmove>  ;

: aligned(s)	\ u1 -- u2 ; SFP005
  cell 1- +  cell negate and
;


\ searching

\ JL001
\
hex

HLLonly? [if]

\ Must match HEADER(S) ( $addr -- ) in XC_SYMBL.FTH.
: testSymSmudge	\ $addr symend -- -1 | 0
\ return true if the name is not smudged.
  nip  aligned(s) 2 cells + @ smudge-bit and 0=
;

\ Must match HEADER(S) ( $addr -- ) in XC_SYMBL.FTH.
: testSymName	\ $addr symNFAa -- flag
\ compare the text string at $addr against the symbol NFA at
\ absolute address symNFAa. On a match, -1 is returned.
  over c@ over c@ xor $3F and		\ if count bytes don't match
  if  2drop 0 exit  endif
  over 1+ c@ over 1+ c@ xor $7F and	\ if first bytes don't match
  if  2drop 0 exit  endif
  over c@ $3F and 1 = 			\ -- $addr NFA(s) ; if length=1
  if  2+  testSymSmudge  exit  endif 	\ step over count+byte, test smudge
  over c@ >r	  			\ # bytes in name to compare
  over 2+  over 2+  r@ 1- s=		\ compare rest of name
  if  r> 1+ + testSymSmudge  exit endif	\ to end of symbol
  r> drop  2drop 0
;

\ Must match HEADER(S) ( $addr -- ) in XC_SYMBL.FTH.
: V-FIND(S)  \ $addr voc-addr(s) -- symbl 1 / symbl -1 / text-addr 0
\ see if vocabulary exists at all, ie is non-zero
  dup 0= if  exit  endif		\ voc=0? Return text-addr and 0
  symbol-table @ +			\ -- $addr symVoc ; convert to Forth addr.
\
\ form the hash code by adding the count byte and first letter
\ ; -- $addr symVoc
  over dup c@ swap 1+ c@ +		\ -- $addr symVoc hash
  over @ 1- and				\ -- $addr symVoc hash' ; #threads mask
\
\ pick NFA from relevant thread
\ ; -- $addr vaddr hash
  cells +  cell + @			\ -- $addr symNFA ; get top NFA from threads : new voc def
\
\ search this thread ; -- $addr symNFA
  begin					\ -- $addr symNFA
    dup 0= if  drop 0  exit  endif	\ done if at end of thread
    symbol-table @ +			\ -- $addr symNFAa ; convert to Forth addr.
    2dup testSymName 0=
   while
    cell - @				\ -- $addr symNFA ; next name
  repeat
\ -- $addr NFA(s)
  dup  rot c@ 1+ + aligned(s) 3 cells +	\ -- NFA(s) symbol-a ; step to end of symbol name
  symbol-table @ -			\ -- NFA(s) symbol ; make address relative
  swap c@ $040 and if 1 else -1 then	\ -- symbol +/-1 ; 1 = imm, -1=normal
;

[else]

code V-FIND(S)  \ text-addr thread#-addr -- symbl 1 / symbl -1 / text-addr 0
\ see if vocabulary exists at all, ie is non-zero
        or      ebx, ebx                \ voc exists?
        jz      l$1
        add     ebx, [] symbol-table	\ convert voc to Forth addr.

\ shuffle data
        mov     edx, 0 [ebp]            \ get text address
        push    esi                     \ preserve ESI and EDI
        push    edi                     \
        mov     esi, edx                \ set ESI points to ctext
        mov     edi, ebx                \ set EDI to first NFA addr.
\
\ form the hash code by adding the count byte and first letter
        xor     ebx, ebx                \ clear offset register
        mov     bl, 0 [esi]             \ get count byte
        add     bl, 1 [esi]             \ add first byte of name
        mov     al, 0 [edi]             \ get number of threads : new voc def
        dec     al                      \ mask = #threads - 1
        and     bl, al                  \ and with thread_mask
\
\ pick NFA from relevant thread
        mov     edi, 4 [edi] [ebx*4]    \ get top NFA from threads : new voc def
        or      edi, edi                \ thread empty ?
        jz      l$2                     \ yes, search fails
\
\ search this thread
        xor	ecx, ecx		\ clear for later use
l$3:
        add     edi, [] symbol-table	\ convert symbol thread addr. to Forth addr.
        mov     ebx, edi                \ get top nfa : old and new voc defs
        mov     esi, edx                \ get text address
        lodsw                           \ get count byte + first letter
        mov     cl, al                  \ save count byte
        xor     ax, 0 [edi]             \ compare text against voc
        and     ax, $07F3F               \ masking first
        jnz     l$4                     \ different, try next word
        add     edi, # 2                \ skip bytes compared
        dec     cx                      \ first byte done, dec. count
        jcxz    l$5                     \ 1 letter, search good!
        repz cmpsb                      \ search it
        jz      l$5                     \ strings match if Z flag set
l$4:
        mov     edi, -4 [ebx]           \ fetch next NFA
        or      edi, edi                \ test it exists
        jnz     l$3                     \ go round again if it does
l$2:
        sub     ebx, ebx                \ return 0

l$6:
        pop     edi			\ restore Forth regs
        pop     esi
        mov	0 [ebp], edx		\ return addr
l$1:
        next,

l$5:
        mov     eax, 8 [edi]            \ get STATUS field value (absolute)
        and     eax, # smudge-bit       \ check to see that symbol is visible (not smudged)
        jnz     l$4                     \ try looking again if this symbol isn't visible (smudged)

        mov     edx, edi                \ EDI points to CFA after CMPS
        add	edx, # 3		\ align address
        and	edx, # -4
        add     edx, # #12              \ skip over XREF, code length, status word ; SFP004

        sub     edx, [] symbol-table	\ convert absolute to symbol table addr.
        test    byte 0 [ebx], # $040	\ see if immediate
        mov     ebx, # 1                \ return 1 (immediate) or -1 (normal)
        jnz     l$6                     \ immediate
        neg     ebx                     \ negate if normal
        jmp     l$6
end-code

[then]				\ HLLonly?

\ state@ state! -- J Lee .. 21/3/95

: state@        \ -- n
  state @  ;

: state!        \ n --
  state !  ;


