\  RELOCATE.FTH Relocation.

((

        Copyright (c) 1993-95

        MicroProcessor Engineering
        133 Hill Lane
        Southampton SO15 5AF
        England

        tel: (+44) 2380 631441
        net: mpe@mpeforth.com

Change history
==============
20190614 SFP007	Updates for 64 bit targets.
SFP006	2 July 1998	Moved ?B-D and LAY-TARGET? to XOPTxxx file
			to avoid a forward reference
SFP005  13 Mar 97       Moved COMPILE-REF to GENERATE.FTH
                        Moved COMPILE-LITERAL to GENERATE.FTH
                        Removed OTA token compiler remnants
JL004   5 November 96   Add optimisations in ADD-LITERAL and SUB-LITERAL for +1 and -1 respectively.
SFP003  29 Sep 96       Modified Compile-Literal to handle short literal tokens
JL002   11 Sep 96       Updated tokens for V2.
SFP001  14 May 96       Default to CALL xx yy for forward refs.

))

\ file control

only forth also c-c definitions
decimal


\ general relocation data structures -- J Lee .. 13/9/93
((
****************************************************
****    General Information                     ****
****************************************************
))

\ For every section (be it CODE or DATA or 'any other') there is one
\ 'Section Header', one 'set of Relocation Entries' and one 'raw data'.
\
\ All sections headers are kept in an array and so are always accessible.

4 constant max-#sections        \ maximum number of sections
variable #sections              \ number of sections allocated
variable code-section           \ current section used for CODE
variable data-section           \ current section used for DATA
variable exec-addr              \ execution address (in 1st CODE section)


\ Relocation entries for the CURRENT sections are held in the relevant array.
\ Data is normally assumed to have few relocation entries.

128 kb constant max-#code-rel-entries    \ max. no. of rel. entries for code
128 kb constant max-#data-rel-entries    \ max. no. of rel. entries for data



\ section structures -- J Lee .. 14/9/93
((
****************************************************
****    Section Headers                         ****
****************************************************
))

16 constant /section-name       \ maximum size of section name (counted $)

struct section-header
        /section-name   Field s_name            \ section name
        cell            Field s_phy-addr        \ target physical address
        cell            Field s_vir-addr        \ target virtual address
        cell            Field s_length          \ section length
        cell            Field s_^raw-data       \ pointer to section raw data
        cell            Field s_^rel-data       \ pointer to relocation info.
        cell            Field s_#rel-entries    \ no. of relocation entries
end-struct


((
******************
* section arrays *
******************
))

section-header max-#sections * constant /section-array

create section-array
        /section-array allot                    \ allocate array space
        section-array /section-array erase      \ and erase


: >Section-Header       ( section# -- addr ; get addr of section header )
  section-header *              \ index into array
  section-array +               \
;


\ relocation entry data structures -- J Lee .. 5/10/93
((
****************************************************
****    Relocation Entries                      ****
****************************************************
))

struct rel-entry
        cell Field r_addr                       \ addr in section
        cell Field r_type                       \ relocation type
end-struct


((
********************
* relocation types *
********************
))
\
\ specific relocation types
\
 0 constant rtype_none          \ no relocation
 1 constant rtype_litc16        \ code pointer - 16bit
 2 constant rtype_litd16        \ data pointer - 16bit
 3 constant rtype_litc32        \ code pointer - 32bit

 4 constant rtype_litd32        \ data pointer - 32bit

 5 constant rtype_litc64        \ code pointer - 64bit
 6 constant rtype_litd64        \ data pointer - 64bit

 7 constant rtype_glit16        \ general literal - 16bit
 8 constant rtype_glit32        \ general literal - 32bit
 9 constant rtype_glit64        \ general literal - 64bit

10 constant rtype_crel16        \ code relative offset - 16bit
11 constant rtype_crel32        \ code relative offset - 32bit
12 constant rtype_crel64        \ code relative offset - 64bit

13 constant rtype_drel16        \ data relative offset - 16bit
14 constant rtype_drel32        \ data relative offset - 32bit
15 constant rtype_drel64        \ data relative offset - 64bit

16 constant rtype_litu16        \ uninitialised data pointer - 16bit
17 constant rtype_litu32        \ uninitialised data pointer - 32bit
18 constant rtype_litu64        \ uninitialised data pointer - 64bit

19 constant rtype_litc128
20 constant rtype_litd128
21 constant rtype_glit128
22 constant rtype_crel128
23 constant rtype_drel128

\
\ general relocation types
\
defer rtype_litc-cell   \ code pointer
defer rtype_litd-cell   \ data pointer
defer rtype_litc-2cell  \ code pointer double
defer rtype_litd-2cell  \ data pointer double
defer rtype_glit-cell   \ general literal single
defer rtype_glit-2cell  \ general literal double
defer rtype_crel-cell   \ code relative offset single
defer rtype_crel-2cell  \ code relative offset double
defer rtype_drel-cell   \ data relative offset single
defer rtype_drel-2cell  \ data relative offset double
defer rtype_litu-cell   \ unintialised data pointer

: Set-Relocation16      ( -- ; set relocation for 16bit target )
  assign rtype_litc16 to-do rtype_litc-cell     \ code pointer
  assign rtype_litd16 to-do rtype_litd-cell     \ data pointer
  assign rtype_litc32 to-do rtype_litc-2cell    \ code pointer double
  assign rtype_litd32 to-do rtype_litd-2cell    \ data pointer double
  assign rtype_glit16 to-do rtype_glit-cell     \ general literal single
  assign rtype_glit32 to-do rtype_glit-2cell    \ general literal double
  assign rtype_crel16 to-do rtype_crel-cell     \ code relative offset single
  assign rtype_crel32 to-do rtype_crel-2cell    \ code relative offset double
  assign rtype_drel16 to-do rtype_drel-cell     \ data relative offset single
  assign rtype_drel32 to-do rtype_drel-2cell    \ data relative offset double
  assign rtype_litu16 to-do rtype_litu-cell             \ uninitialised data pointer
;

: Set-Relocation32      ( -- ; set relocation for 32bit target )
  assign rtype_litc32 to-do rtype_litc-cell     \ code pointer single
  assign rtype_litd32 to-do rtype_litd-cell     \ data pointer single
  assign rtype_litc64 to-do rtype_litc-2cell    \ code pointer double
  assign rtype_litd64 to-do rtype_litd-2cell    \ data pointer double
  assign rtype_glit32 to-do rtype_glit-cell     \ general literal single
  assign rtype_glit64 to-do rtype_glit-2cell    \ general literal double
  assign rtype_crel32 to-do rtype_crel-cell     \ code relative offset single
  assign rtype_crel64 to-do rtype_crel-2cell    \ code relative offset double
  assign rtype_drel32 to-do rtype_drel-cell     \ data relative offset single
  assign rtype_drel64 to-do rtype_drel-2cell    \ data relative offset double
  assign rtype_litu32 to-do rtype_litu-cell     \ uninitialised data pointer
;

: Set-Relocation64	\ -- ; set relocation for 32bit target
  assign rtype_litc64 to-do rtype_litc-cell     \ code pointer single
  assign rtype_litd64 to-do rtype_litd-cell     \ data pointer single
  assign rtype_litc128 to-do rtype_litc-2cell   \ code pointer double
  assign rtype_litd128 to-do rtype_litd-2cell   \ data pointer double
  assign rtype_glit64 to-do rtype_glit-cell     \ general literal single
  assign rtype_glit128 to-do rtype_glit-2cell   \ general literal double
  assign rtype_crel64 to-do rtype_crel-cell     \ code relative offset single
  assign rtype_crel128 to-do rtype_crel-2cell   \ code relative offset double
  assign rtype_drel64 to-do rtype_drel-cell     \ data relative offset single
  assign rtype_drel128 to-do rtype_drel-2cell   \ data relative offset double
  assign rtype_litu64 to-do rtype_litu-cell     \ uninitialised data pointer
;

: Assign-Relocation-Types       ( -- ; relocation types for 16b or 32b)
  16bit?                        \ 16bit relocation types
  If  set-relocation16  Endif
  32bit?                        \ 32bit relocation types
  If  set-relocation32  Endif
  64bit?                        \ 32bit relocation types
  If  set-relocation64  Endif
;

((
*********************
* relocation arrays *
*********************
))

\ size of relocation array for current CODE section
rel-entry max-#code-rel-entries * constant /code-rel-array

create code-rel-array
        /code-rel-array allot                   \ allocate array space
        code-rel-array /code-rel-array erase    \ and erase


\ size of relocation array for current DATA section
rel-entry max-#data-rel-entries * constant /data-rel-array

create data-rel-array
        /data-rel-array allot                   \ allocate array space
        data-rel-array /data-rel-array erase    \ and erase


: >Rel-Entry    ( array rel# -- addr ; get addr of relocation entry )
  rel-entry * +                 \ index into array
;


\ section code -- J Lee .. 14/9/93

: Get-Section#  ( -- section# ; get next aavailable section#, or error )
  #sections @                   \ get potential next free section
  dup max-#sections >=          \ no more sections available?
  If                            \ t:
    drop                        \   error
    ." Too many sections" ABORT \
  Endif                         \ otherwise:
  1 #sections +!                \ increment no. sections allocated
;

: Code-Section-Header   ( -- addr ; get addr of section header for CODE )
  code-section @ >section-header        \ current code section
;

: Data-Section-Header   ( -- addr ; get addr of section header for DATA )
  data-section @ >section-header        \ current data section
;


\ relocation entry code -- J Lee .. 14/9/93

: Get-Code-Rel# ( -- rel# ; return next free CODE rel#, or do ERROR )
  code-section-header s_#rel-entries    \ get #rel-entries addr
  dup @  max-#code-rel-entries  >=      \ get #rel-entries, too many:
  If                                    \ t:
    drop                                \   error
    ." Too many CODE entries" ABORT     \
  Endif                                 \ otherwise:
  dup @                                 \ return next free rel#
  swap 1 swap +!                        \ increment no. rels allocated
;

: Get-Data-Rel# ( -- rel# ; return next free DATA rel#, or do ERROR )
  data-section-header s_#rel-entries    \ get #rel-entries addr
  dup @  max-#data-rel-entries  >=      \ get #rel-entries, too many:
  If                                    \ t:
    drop                                \   error
    ." Too many DATA entries" ABORT     \
  Endif                                 \ otherwise:
  dup @                                 \ return next free rel#
  swap 1 swap +!                        \ increment no. rels allocated
;

: >Code-Rel     ( rel# -- addr ; get addr of code relocation entry )
  code-section-header                   \ current CODE section header
  s_^rel-data @                         \ get pointer to relocation data
  swap >rel-entry                       \ return rel-entry address
;

: >Data-Rel     ( rel# -- addr ; get addr of data relocation entry )
  data-section-header                   \ current CODE section header
  s_^rel-data @                         \ get pointer to relocation data
  swap >rel-entry                       \ return rel-entry address
;


\ Code-Ref Data-Ref -- J Lee .. 10/9/93

: Code-Ref      ( addr ref-type -- ; mark a data ref. at code addr )
  get-code-rel# >code-rel >r    \ get next data relocation entry
  r@ r_type !                   \ set type
  r> r_addr !                   \ set addr
;

: Data-Ref      ( addr ref-type -- ; mark a code ref. at code addr )
  get-data-rel# >data-rel >r    \ get next data relocation entry
  r@ r_type !                   \ set type
  r> r_addr !                   \ set addr
;


\ references -- J Lee .. 14/10/93

(( SFP005 ...
\ SFP001...
: Compile-Ref   ( symbol -- )
  0 ref >ca(t) ,c(t)
;
\ ...SFP001

: Compile-Run-Time-Ref  ( symbol -- )
  2 cells+ 0 run-time-ref               \ generate code offset
  >ca(t) ,c(t)                          \
;
... SFP005 ))


\ ?b-d ?bd-literal value, -- J Lee .. 10/9/93

((
: ?b-d          ( -- ; if CC is compiling and between CREATE ... DOES> )
  compil?  state@  0<> and
;

: lay-target?  ( -- t/f/ ; true if we can lay target code )
\  ?b-d 0=  heads?  or                  \ if we're not in def. word or headers are on
\  ?b-d 0=                              \ if we're not in defining word
  TRUE                          \ always
;
))

: ?bd-Literal   ( n -- ; compile into C-C if between CREATE ... DOES> )
  ?b-d                          \ in CREATE ... DOES> portion
  If                            \ y:
    postpone literal            \   compile as a literal in Host
  Else                          \ n:
    drop                        \   discard literal
  ENDIF
;


\ Umbilical mode relocation -- J Lee .. 30/10/93

: ?>Umbilical-Litc-cell ( addr -- addr' ; modify address in U/F )
  in-umbilical @(m)             \ in Umbilical?
  If  code-base @(m) +  Endif   \ y:   modify to absolute address
;

: ?>Umbilical-Litd-cell ( addr -- addr' ; modify address in U/F )
  in-umbilical @(m)             \ in Umbilical?
  If  data-base @(m) +  Endif   \ y:  modify address
;

: ?<Umbilical-Litc-cell ( addr' -- addr ; modify address in U/F )
  in-umbilical @(m)             \ in Umbilical?
  If  code-base @(m) -  Endif   \ y:  modify to 0 relative address
;

: ?<Umbilical-Litd-cell ( addr' -- addr ; modify address in U/F )
  in-umbilical @(m)             \ in Umbilical?
  If  data-base @(m) -  Endif   \ y:  modify to 0 relative address
;


\ Code-Literal Data-Literal Label-Literal General-Literal -- J Lee .. 8/10/93

\
\ Identify the literal type.
\ This code is used to determine which is the smallest form of the literal
\

((
\ SFP003 ... added
: LITx-type?  ( x -- t/f )
  $00 $0F within?
;
\ ... SFP003

: PLIT-type?  ( x -- t/f )
  0 255 within?
;

: NLIT-type?  ( x -- t/f )
  -255 -1 within?
;

: HLIT-type?  ( x -- t/f )
  0 65535 within?
;

: SLIT-type?  ( x -- t/f )
  dup 0 1020 within?                    \ largest value to be addressed is 1020 (255*4)
  swap $3 and 0=                                \ and it must be on a cell (4 byte) boundary
  and
;

: Signed-byte-type?  ( x -- t/f )
  -128 127 within?
;

: SCALL-type?  ( addr -- t/f ; assume call token and operand not laid yet )
  2 >ca(t)                      \ allow for token and operand
  -128 127 within?      \ in range?
;

: CALL-type?  ( addr -- t/f/ ; assume call token and operand not laid yet )
  3 >ca(t)                              \ allow for token and operand
  -32768 32767 within?  \ in range?
;
))


: Code-Offset   ( -- ; mark code position as a code offset reference )
  here(t) rtype_crel-cell code-ref
;

: Data-Offset   ( -- ; mark code poistion as a data reference offset )
  here(t) rtype_drel-cell code-ref
;

: Code-Pointer  ( -- ; mark code position as a code reference )
  here(t) rtype_litc-cell code-ref      \ make relocation entry
;

: Data-Pointer  ( -- ; mark code position as a data reference )
  here(t) rtype_litd-cell code-ref      \ make relocation entry
;

: Code-Value    ( addr -- ; mark code position as a 'cell' value )
  rtype_glit-cell code-ref              \ make relocation entry
;



: General-Literal(t)    ( n -- ; lay value as some form of LIT in target )
  lay-target?
  If
    doliteral,
  Else
    drop
  Endif
;

: General-Literal(h)    ( n -- ; lay value as a LIT in the host analogue )
  ?bd-literal                           \ lay value
;

: General-Literal  ( n -- ; compile literal into target and into host if necessary )
  dup general-literal(t) general-literal(h)
;

: General-DLiteral      ( d -- ; lay value as a LIT )
  swap general-literal
  general-literal
;


: Here-Code-Value       ( -- ; mark here as 'cell' value )
  here(t) code-value            \ make relocation entry
;

: Data-Rel-Value        ( data-addr -- ; mark addr as 'cell' value )
  rtype_glit-cell data-ref              \ make relocation entry
;

: Data-Rel-Litc  ( data-addr -- ; mark addr as code pointer )
  rtype_litc-cell data-ref
;

: Data-Rel-Litd  ( data-addr -- ; mark addr as data pointer )
  rtype_litd-cell data-ref
;

: Data-Rel-DValue       ( data-addr -- ; mark addr as '2cell' value )
  rtype_glit-2cell data-ref     \ make relocation entry
;

: Compile-Num   ( n|d -- )
  dpl @                         \ double number?
  1+                            \
  If                            \ y:
    general-dliteral            \   lay double number
  Else
    general-literal             \   lay single number
  Endif                         \
;


\ initialisation -- J Lee .. 14/9/93

: Init-Common   ( header -- ; initialise common parts of section header )
  0 over                \ get physical addr
  s_phy-addr !          \ set in section header
  0 over                \ get virtual addr
  s_vir-addr !          \ set in section header
  0 over                \ section length
  s_length !            \ set in section header
  0 over                \ pointer to raw data
  s_^raw-data !         \ set in section header
  0 over                \ number of relocation entries
  s_#rel-entries !      \ set in section header
  drop
;

: Init-Code-Section     ( -- ; initialise CODE section )
  get-section# code-section !   \ allocate code section
  code-section-header           \ get CODE section header
  c" CODE" over                 \ section name
  s_name $move                  \ set in section header
  code-rel-array over           \ get pointer to CODE Rel Data
  s_^rel-data !                 \ set in section header
  init-common                   \ initialise rest of header
;

: Init-Data-Section     ( -- ; initialise DATA section )
  get-section# data-section !   \ allocate data section
  data-section-header           \ get DATA section header
  c" DATA" over                 \ section name
  s_name $move                  \ set in section header
  data-rel-array over           \ get pointer to DATA Rel Data
  s_^rel-data !                 \ set in section header
  init-common                   \ initialise rest of header
;

: Init-Sections ( -- ; initialise sections )
  section-array /section-array erase    \ zero all fields in all records
  0 #sections !                         \ no sections allocated
  init-code-section                     \ initialise code section
  init-data-section                     \ initialise data section
;

: Init-Rel-Entries      ( -- ; initialise all relocation arrays )
  code-rel-array /code-rel-array erase  \ zero all CODE relocation array
  data-rel-array /data-rel-array erase  \ zero all DATA relocation array
;

: Init-Relocation       ( -- ; initialise relocation )
  init-sections
  init-rel-entries
;


\ finishing up -- J Lee .. 14/9/93

defer MAKE-RELOCATION
assign NOOP to-do MAKE-RELOCATION

(( for debuggin
: .Data-Relocation      ( -- ; display data relocation records )
  data-section-header                   \
  s_#rel-entries @  0                   \ for no. code rel. entries:
  ?Do                                   \
    i >data-rel                         \   get general relocation entry
    cr i . ." Addr: " dup field{ r_addr }field @ .hex
       ." Type: " field{ r_type }field @ .hex
    cr
key drop
  Loop
;
))

: Finish-Relocation     ( -- ; finish up relocation )
\ .data-relocation
  make-relocation
;


