\ REQ-DEF.FTH Required definitions handler.

((
These definitions are created when the compiler is built.
The list of required symbols is in file REQUIRED.FTH.
The symbols are held in the REQUIRED.VOC vocabulary,
and their symbols are created by code in INITIAL.FTH
when the compiler is initialised.

Copyright (c) 1993-98

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
net: mpe@mpeforth.com
web: www.mpeforth.com

SFP001  16 Nov 1998     Overhauled, XREF requirements added
))


\ ********************
\ search order control
\ ********************

only forth also c-c definitions
decimal

Vocabulary Required.Voc


: Name>Here     \ Pfa -- ; $ left at here
  body> >name                           \ value to name
  dup c@ 31 and dup >r 1+               \ >Nfa and get masked
  here swap cmove                       \ copy to here
  r> here c!                            \ count-byte = len only
;

: (Required-Symbol)     \ "<name>" -- ; REQUIRED-SYMBOL <name>
  Create
   0 , 0 ,                              \ Flag, Value.
  Does>
   dup @                                \ Symbol defined ?
   If  cell+ @                          \   Yes, just fetch it.
   Else
     dup on                             \ Flag symbol as defined
     dup name>here                      \ copy name to here
     here $symbol                       \ create symbol
     cell+ last'(s)@ dup rot !          \ Get symbol address,
   Endif ;                              \  stash as value,

: ($Required-Symbol)    \ string -- ; create symbol in REQUIRED.VOC
  $Create
   0 , 0 ,                              \ Flag, Value.
  Does>
   dup @                                \ Symbol defined ?
   If  cell+ @                          \   Yes, just fetch it.
   Else
     dup on                             \ Flag symbol as defined
     dup name>here                      \ copy name to here
     here $symbol                       \ create symbol
     cell+ last'(s)@ dup rot !          \ Get symbol address,
   Endif ;                              \  stash as value,

: (Required-Label)      \ -- ; <name>
  Create
   0 , 0 ,                              \ Flag, Value.
  Does>
   dup @                                \ Symbol defined ?
   If  cell+ @                          \   Yes, just fetch it.
    Else                                \
     dup on                             \ Flag symbol as defined
     dup name>here                      \ copy name to here
     Current(s) @(m) 'Labels(s) @(m) Current(s) !(m)
     here $symbol                       \ create symbol
     Current(s) !(m)                    \
     cell+ last'(s)@ dup rot !          \ Get symbol address,
   Endif ;                              \  stash as value,

: (Required-Equ)        \ -- ; <name>
  Create
   0 , 0 ,                              \ Flag, Value.
  Does>
   dup @                                \ Symbol defined ?
   If  cell+ @                          \   Yes, just fetch it.
    Else                                \
     dup on                             \ Flag symbol as defined
     dup name>here                      \ copy name to here
     Current(s) @(m) 'Equates(s) @(m) Current(s) !(m)
     here $symbol                       \ create symbol
     Current(s) !(m)                    \
     cell+ last'(s)@ dup rot !          \ Get symbol address,
   Endif ;                              \  stash as value,

: (Required-Def-Word)   \ -- <name>
  Create                                \
   0 , 0 ,                              \ Flag, Value.
  Does>                                 \
   dup @                                \ Symbol defined ?
   If  cell+ @                          \   Yes, just fetch it.
    Else                                \
     dup on                             \ Flag symbol as defined
     dup name>here                      \ copy name to here
     here $def-word                     \ create symbol
     cell+ last'(s)@ dup rot !          \ Get symbol address,
   Endif ;                              \  stash as value,
                                        \  & leave a copy.

: (Required-Voc)        \ -- <name>
  Create                                \
   0 , 0 ,                              \ Flag, Value.
  Does>                                 \
   dup @                                \ Symbol defined ?
   If  cell+ @                          \   Yes, just fetch it.
    Else                                \
     dup on                             \ Flag symbol as defined
     dup name>here                      \ copy name to here
     2 here $make-voc                   \ create symbol - 2 threads in voc
     cell+ last'(s)@ dup rot !          \ Get symbol address,
   Endif ;                              \  stash as value,
                                        \  & leave a copy.

: (Required-C/V)        \ -- <name>
  Create                                \
   0 , 0 ,                              \ Flag, Value.
  Does>                                 \
   dup @                                \ Symbol defined ?
   If  cell+ @                          \   Yes, just fetch it.
    Else                                \
     dup on                             \ Flag symbol as defined
     dup name>here                      \ copy name to here
     2 here $make-c/v                   \ create symbol
     cell+ last'(s)@ dup rot !          \ Get symbol address,
   Endif ;                              \  stash as value,
                                        \  & leave a copy.

: In-Required.voc       \ --
  also Required.voc definitions
;

: Restore-voc           \ --
  only forth also c-c definitions
;

: Required-Symbol       \ --
  in-required.voc
  (required-symbol)
  restore-voc
;

: $Required-Symbol      \ string --
  in-required.voc
  ($required-symbol)
  restore-voc
;

: Required-Def-Word     \ --
  in-required.voc
  (required-def-word)
  restore-voc
;

: Required-Label        \ --
  in-required.voc
  (required-label)
  restore-voc
;

: Required-Equ          \ --
  in-required.voc
  (required-equ)
  restore-voc
;

: Required-C/V          \ --
  in-required.voc
  (required-c/v)
  restore-voc
;

: Required-Voc          \ --
  in-required.voc
  (required-voc)
  restore-voc
;


\ **********************
\ $Target-Sym Target-Sym
\ **********************

: $Target-Sym   \ string -- xt
  dup count upper                       \ input name
  dup ['] Required.Voc v-find 0= if     \ only look in Req. Voc
    $. ."  is undefined SYMBOL"
    .source-line  abort
  Endif
  nip                                   \ don't need name any more
;                                       \  else, execute word

: Target-Sym    \ "<name>" -- ; TARGET-SYM <name>
  bl word $target-sym
  state@ if compile, else execute endif
;
immediate

: Init-target   \ "<name>" -- ; INIT-TARGET <name>
  postpone target-sym drop  ;          \ used to init. symbols

: The-Label	\ --
  postpone target-sym
;
immediate

: The-Equate	\ --
  postpone target-sym
;
immediate


\  The use of The-Label and The-Equate replace, for example:
\
\    [+LABEL] DODOES [PREVIOUS] with THE-LABEL DODOES


\ ***********************************************************
\ FORCE-REQUIRED to ensure that a required word entry exists.
\ These are needed to permit XREFs of COMPILER words.
\ ***********************************************************

: force-symbol  \ -- xt ; look ahead and force a REQUIRED-SYMBOL
  >in @ >r                              \ save i/p
  get-token
  ['] required.voc v-find 0= if		\ if symbol does not exist
    drop                                \ discard text address
    r@ >in !                            \ restore i/p stream
    context @ current @
    ['] required.voc voc>wid current !
    (Required-Symbol)                   \ make symbol
    current ! context !
    latest-xt				\ return xt
  endif
  r> >in !                              \ restore i/p
;

variable <last-required>        \ -- addr ; holds xt of last symbol from REQUIRED:

: required:     \ "<name" -- ; REQUIRED: <name>
  force-symbol <last-required> !  bl word drop
;

: [last-req]    \ -- ; force XREF of last symbol defined by REQUIRED:
  <last-required> @                     \ XT of last REQUIRED: symbol
  >body cell+                           \ address at which symbol address is found
  clit,                     		\ compile as literal
  postpone @  postpone add-xref         \ get symbol and add XREF info
;  immediate

only forth definitions

