\ Saving and restoring state for partial compilation

((
Copyright (c) 1993-98
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

20090709 SBD007 Changed exit behaviour of SNAPSHOT.
SFP006	09 Feb 2001	Fixed SNAPSHOT section handling
SFP005  13 Sep 1998     Used BIND/D in SNAPSHOT
SFP004  27 Aug 1998     Overhaul for XC6
SFP003  14 Nov 97       Update for SECTIONS model
JL002    3 Oct 97       Update for PFW 2.1
JL001   21 Feb 97       Update for OTA
))

only forth also c-c definitions
decimal


\ *****************
\ declare file pcbs
\ *****************

pcb Vars.Pcb
pcb Symbols.Pcb
pcb Ram.Pcb


\ ******************
\ File name handling
\ ******************

: $?File-Name   \ $addr --
  dup Symbols.pcb set-pathname          \ PCBs.
  dup Vars.pcb set-pathname
  Ram.Pcb set-pathname
  c" .SYM" Symbols.Pcb set-ext          \ set default extensions.
  c" .VAR" Vars.Pcb set-ext                   \ for all PCBs
  c" .RAM" Ram.Pcb set-ext
;

: ?File-Name    \ -- ; <name>
  bl-word/uc  $?file-name               \ set file-name for all compiler files
;


\ ************
\ Create Files
\ ************

: Create-Vars           \ --
  Vars.Pcb create-path-pcb
  abort" Can't create Vars File"
;

: Create-Symbols        \ --
  Symbols.Pcb create-path-pcb
  abort" Can't create Symbol File"
;

: Create-Ram            \ --
  Ram.Pcb create-path-pcb
  abort" Can't create Ram File"
;


\ ***********
\ Close Files
\ ***********

: Close-Vars    \ --
  Vars.Pcb handle close-path
  abort" Can't close Vars File"
;

: Close-Symbols \ --
  Symbols.Pcb handle close-path
  abort" Can't close Symbol File"
;

: Close-Ram     \ --
  Ram.Pcb handle close-path
  abort" Can't close Ram File"
;


\ ***********
\ Write Files
\ ***********

: Write-Vars    \ addr len --
  Vars.Pcb handle write-path
  abort" Can't write to Vars file"
  drop
;

: Write-Symbols \ addr len --
  Symbols.Pcb handle write-path
  abort" Can't write to Symbols File"
  drop
;

: Write-Ram     \ addr len --
  Ram.Pcb handle write-path
  abort" Can't write to Ram file"
  drop
;


\ ********************
\ Calc File parameters
\ ********************

: Calc-Vars-Pars        \ -- addr len
  {Symbol-Management                    \ addr
  }Symbol-Management over -             \ len
;

: Calc-Symbols-Pars     \ -- addr len
  Symbol-Space                          \ addr
  /Symbol-Space                         \ len
;

: Calc-Ram-Pars         \ -- addr len
  Ram-Area @(m)
  Ram-Size @(m) bytes/addr(t) *
;


\ **********
\ Save files
\ **********

: Save-Symbols  \ --
  create-symbols                        \ create file
    calc-symbols-pars                   \ write file
    write-symbols
  close-symbols  ;                      \ close file


: Save-Vars     \ --
  create-vars                           \ create file
    calc-vars-pars                      \ write file
    write-vars
  close-vars  ;                         \ close file


: Save-Ram      \ --
  create-ram                            \ create file
    calc-ram-pars                       \ write file
    write-ram
  close-ram  ;                          \ close file


\ **********
\ Open Files
\ **********

: Open-Vars     \ --
  Vars.Pcb open-path-pcb
  abort" Can't open Vars File"  ;


: Open-Symbols  \ --
  Symbols.Pcb open-path-pcb
  abort" Can't open Symbol File"  ;


: Open-Ram      \ --
  Ram.Pcb open-path-pcb
  abort" Can't open Ram File"  ;


\ **********
\ Read Files
\ **********

: Read-Vars     \ addr len --
  Vars.Pcb handle read-path
  abort" Can't read Vars File"
  drop
;

: Read-Symbols  \ addr len --
  Symbols.Pcb handle read-path
  abort" Can't read Symbols File"
  drop
;

: Read-Ram      \ addr len --
  Ram.Pcb handle read-path
  abort" Can't read Ram File"
  drop
;


\ *************************
\ Calc File Read parameters
\ *************************

: Calc-Read-Vars        \ -- addr len
  {Symbol-Management                    \ addr
  }Symbol-management over -             \ len
;

: Calc-Read-Symbols     \ -- addr len
  Symbol-Space                          \ addr
  /Symbol-Space                         \ len
;

: Calc-Read-Ram         \ -- addr len
  here dup Ram-Area !(m)                \ new address of Init-Ram
  Ram-Size @(m) cell+ dup               \ still same size
  allot                                 \ to be allotted
;


\ *************
\ Loading files
\ *************
\  NOTE:  The initialised Ram has to be rebuilt in the XCompiler.
\         This means that the Management-Vars have to be loaded
\         first, and then the Initialised-Ram.

: Load-Symbols  \ --
  open-symbols                          \ open file
    calc-read-symbols                   \ read file
    read-symbols
    Symbol-Space Symbol-Table !         \ reset pointer to symbols
  close-symbols                         \ close file
;

: Load-Vars     \ --
  open-vars                             \ open file
    calc-read-vars                      \ read file
    read-vars
  close-vars                            \ close file
;

: Load-Ram      \ --
  open-ram                              \ open file
    calc-read-ram                       \ read file
    read-ram
  close-ram                             \ close file
;


\ **************************
\ Saving and restoring state
\ **************************

: $Save-compilation     \ $addr -- ; snapshot XC
  $?file-name
  save-symbols  save-vars               \ Save Xcompilation data,
  ?Rom @(m) If  save-ram Endif          \ and init-ram for Rom.
  save-sections                         \ SFP003
  cr ." ---------------------"
  cr ." Compilation Suspended"
  cr ." ---------------------"
  cr
;

: $Restart      \ $addr --
  $?file-name load-symbols load-vars    \ Restore XCompiler.
  ?Rom @(m) If  load-ram Endif          \ Rom target has init-ram
  load-sections
  cr ." ---------------------"
  cr ." Compilation Restarted"
  cr ." ---------------------"
  cr
;

create snap-name$       \ -- addr ; holds snapshot name
  max_path allot&erase

: snapshot      \ "<file>" -- ; SNAPSHOT FOO
\ *** save action of loader
  host-mode-compiler                    \ stop cross compilation
  >in @ >r                              \ save input pointer
  get-token $save-compilation           \ make state files
  close-sections			\ otherwise EXE has open sections! SFP006
  r@ >in !  get-token snap-name$ $move  \ preserve snapshot name
\ *** install new action of loader
  r@ >in !  save                        \ save the cross compiler
  r> drop
\ *** restore action of loader
  close-image close-log xc-exit		\ SBD
\  error-exit                            \ get out quietly
;

also forth definitions                  \ must be available at start

: load-snapshot \ -- ; reload Xcompiler and sections
  (cross-compile)			\ initialise cross compilation
  open-sections				\ initialise sections before reloading ; SFP006
  snap-name$ $restart			\ reload compiler state
;

((
: Resume        \ -- ; resume <name> ; combo. of CROSS-COMPILE and RESTART
  (cross-compile)
  get-token $restart
;

: load-snapshot \ -- ; restarts cross compiler using snapshot name
  load-snapshot
;
))

previous definitions

also cc/i

(( del SFP006
: Save-Compilation      cc/i    \ --
  bl-word/uc
  $save-compilation  ;

: Suspend-Compilation   cc/i    \ --
  [+interpreter] save-compilation [previous]    \ snapshot
  error-exit                                    \ silent bye
;

: Restart-Compilation   cc/i    \ -- ; <name>
  resume  ;
))

: snapshot              cc/i    \ "file" -- ; SNAPSHOT <file>
  snapshot
;

only forth definitions


\ ******************
\ Support for MARKER
\ ******************

only forth also c-c definitions

variable PartialChain	\ -- ; anchors list of partials

: PreservePartial	\ --
  c" .var"  c" .ram"  c" .sym"  3  c" mrk"
  get-unique-filename 0= if		\ get base filename
    cr ." Couldn't get unique filename"	\   give up if we couldn't get a filename
    ABORT
  endif
  PartialChain link,			\ start list item
  here >r  c@ count $,			\ lay file name
  r> $?file-name			\ set up all the filenames with extensions
  save-symbols				\ save the symbol table
  save-vars				\ save the mamnagement variables
  ?rom @(m) If  save-ram  Endif		\ save initialised RAM if a ROM target
;

: (RestorePartial)	\ file$ -- ; restore partial compilation bits
  dup $?file-name                               \ take filename string from heap and set filenames
  load-symbols                                  \ reload symbol table
  load-vars                                     \ reload management variables
  ?rom @(m)  If  load-ram  Endif                \ reload initialied RAM if a ROM target
  >r c" .var" c" .sym" c" .ram" 3 r>
  delete-unique-filename 0= if			\ delete all the files
    cr ." Couldn't delete filename"             \   give up if we couldn't delete files
    ABORT
  Endif
;

: RestorePartial	\ start end -- start end
  PartialChain @ begin
    dup
     while				\ -- start end link
    dup @ r>				\ next link
    3dup -rot within? if		\ if in active region
      dup cell+ (RestorePartial)	\ restore
      dup PartialChain delLink		\ delete from chain
    endif
    drop r>
  repeat
  drop
;

' PreservePartial remembers
' RestorePartial prunes

only forth definitions

