\ SECTION handling for cross compilers

((
Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2006, 2008
Copyright (c) 2009, 2010, 2012, 2013, 2014, 2016, 2018, 2019
MicroProcessor Engineering
133 Hill Lane
Southampton, SO15 5AF
England

tel: +44 2380 631441
net: tech-support@mpeforth.com

To Do
=====
Force .SECTIONS to output to the CONSOLE when the log goes to a file.
This may have to be done elsewhere.

Change History
==============
20210615 SFP044 Option to save all sections in secondary files
20201127 SFP043 DATA-FILE opens file in R/O mode.
20190614 SFP042 Changes for 64 bit targets.
20180628 SFP041 Community edition changes.
20170704 SFP040 64 bit update.
20160804 SFP039 Replace coded words with high level definitions.
20140528 SFP038 Made binary file extension selectable.
20140515 SFP037 Added option to suppress download prompts
20131031 SFP036 Added ADDR>SECTION and USE-MEM-TABLE.
20131023 SFP035	Prevented search of IDATA section in MEM-OP(MS)
		for Harvard targets.
20120128 SFP034 Added .CURR-SECTIONS
20101112 SFP033 Enhanced IDATA write performance.
20101013 SFP032 Added WRITE-FAIL.
20100422 SFP031 Moved part of downloader here from Uforth.fth.
20091216 SFP030 Allowed Eval/Stamp users to set FIXED-DATA-BASE once.
20090515 SFP029 Removed IRTC references.
20080909 SFP028 Added dummy sections to allow INCLUDE before SECTIONs set.
20060316 SFP027 Added PAGEID(T).
20060206 SFP026 Added SetFlashStart.
20051202 SFP024 Added secondary file output for CDATA sections.
20020916 SFP023 Added LENGTH(T) to return length of current section.
20020904 SFP022 Added SEC_OVERLAP and SEC_LIMIT error codes.
20011108 SFP021 Stopped LAY-IDATA(MS) for zero length IDATA
20010713 SFP020 Fixed Gx,(T) for Harvard targets.
                Moved xDATA? to avoid forward refs.
20001010 SFP019 Added DATA-FILE to load binary data into IDATA or CDATA
20000822 SFP018 Added xALIGN(T)
20000410 SFP017 Added section size checking for IRTC compilers
20000204 SFP016 Converted ABORT" to common error numbers - ARM is master file
19990314 SFP015 Changed (.SECTION-REPORT) to continue if invalid section found.
19981217 SFP014 Corrected stack action of MPAGE
19981120 MSD014 Removed the diagnostic .RS from in front of the ABORT" in the
         MSD014 word MEM-OP(MS).
19981108 SFP013 Added conversion for Harvard (8051, Z8 etc) CPUs with
                separated code and data spaces.
19980924 MSD012 The following changes re-applied after The Boss had been at
         MSD012 this.  Also, some of my MSD010 seem to have gone missing and
         MSD012 I've re-applied *those*, as well:
         MSD012 Added section "sanity check" call to (.SECTION-REPORT) and
         MSD012 added the word itself.  We had trouble with the running stuff
         MSD012 under the ARM emulator because I'd arbitrarily made the code
         MSD012 and data sections contiguous.  Steve had removed the address
         MSD012 range checking the word C,C(T) that is used to lay the IDATA
         MSD012 table down in ROM and, in this case, the gap between the end
         MSD012 of the code and end of the code section was smaller than the
         MSD012 length of the IDATA RAM table.  Rather than have a performance
         MSD012 hit by replacing the address checking, we've elected to add
         MSD012 the above-mentioned validation at end-of-run.
19980827 SFP011 Moved XXXX-DEN(MS) and LAY-IDATA from EPEM.DRV.
19980810 SFP010 Added BANK-ACTIVEPAGE to bank definition
                Forced BANK and PAGE to create in INTERPRETER vocabulary
19980501 SFP009 Coded little-endian byte splitters/mergers
                Added ALL-SAVED to force whole section to be written out
                Added SAVESIZE(MS) to return size to be saved
19980306 SFP008 SINGLE-SECTION-ONLY now clears ?ROM
19980228 SFP007 Added SINGLE-SECTION-ONLY and SINGLE-SECTION?
19980221 SFP006 Added CHANGE-IO(MS)
19980218 SFP005 Added error checking to FROM-END(T)
19980218 MSD004 Replace MS-HIGH with MS-BP in MEM-END(T).
19971210  JL003 Added CDATA-TYPE? IDATA-TYPE? UDATA-TYPE?
19970912 SFP002 Added WRITE-IGNORE and WRITE-INVALID.
19970907 SFP001 Added Harvard support.


Notes
=====
The memory sections are based upon a data structure which allows
different I/O devices to be added. The following are assumed to
be the minimum set:
  In memory buffer      saved to disc or emulator at end
  EPROM emulator
  Umbilical Link

First release: September 1997

Notes:
There are five types of memory assumed:
  CDATA         contains code, assumed to be in ROM, but writable by
                the cross compiler
  IDATA         Initialised RAM, copied from ROM to RAM at start up,
                zeroed before copy
  UDATA         Unitialised RAM, may be read/written by cross compiler
  -----
  ICDATA        Initialised CODE RAM for Harvard targets only
  UCDATA        Unitialised CODE RAM for Harvard targets only

Memory is organised into SECTIONs, which are units of memory. In bank
switched systems BANKs may be defined, to which are attached PAGEs. A BANK
defines the address range and type of swtched memory, and multiple PAGEs
are defined within the BANK. There is no limit to the number of separate
banks and pages. Each PAGE behaves as a SECTION, except that only the
last referenced page in each bank is active. This allows us to bank
switch both ROM and RAM areas.

HEX
0 7FFF CDATA SECTION ROM
8000 BFFF CDATA BANK ROMBANK
  1 PAGE BANK0
  2 PAGE BANK1
  3 PAGE BANK2
C000 FFFF IDATA SECTION RAM

For each type (CDATA, IDATA ...) there is a current section. There is also
a current type.

The data structure for a SECTION is described below by the MS-DESC
structure. ALL accesses to the target memory are performed through this
structure. Two particular techniques should be noted.

1) The type of a section is held as the data address for the variable
   pointing to the current section of the type. This greatly simplifies
   the code. See MS-TYPE

2) Each section contains a pointer MS-IOVECTOR to an array of I/O functions.
   By default, all sections are created assuming that target memory is buffered
   in host memory, which will be written to disc by the SAVE operation when
   the cross compiler finishes.

   This permits new I/O mechanisms to be installed at a later date without
   having to touch this code. All that is needed is a word that installs a new
   vector into the last referenced section. The structure of the vector is
   defined in the MSIO-desc structure.

     : INSTALL-MINE     \ -- ; INSTALLS I/O INTO THE LAST REFERENCED SECTION
       MY-VECTOR  CURR-SECT @ MS-IOVECTOR !  ;
))

\ ***********
\ Error codes
\ ***********

[forth]  also c-c definitions
\ *** abort" ***
( #337 ) ErrDef sec_undefined	"Memory address not in defined section"
( #338 ) ErrDef sec_nosection	"Memory address not in section"
( #339 ) ErrDef sec_create	"Can't create image file"
( #340 ) ErrDef sec_write	"Image file write error"
( #341 ) ErrDef sec_close	"Image file close error"
( #342 ) ErrDef sec_imagesize	"Image file bigger than size of section"
( #343 ) ErrDef sec_bufferlen	"Image file larger than buffer"
( #344 ) ErrDef sec_read	"Image file read error"
( #346 ) ErrDef sec_allocate	"Can't allocate image buffer"
( #347 ) ErrDef sec_inuse	"Section already in use"
( #348 ) ErrDef sec_no64	"No 64 bit support"
( #349 ) ErrDef sec_writeinvalid  "Writes invalid in WRITE-INVALID and WRITE-FAIL sections"
( #350 ) ErrDef sec_badmem 	"Unsupported memory access (size greater than cell)"
( #351 ) ErrDef sec_open	"Can't open image file"
( #352 ) ErrDef sec_overlap	"Section DP and BP pointers crossed"
( #353 ) ErrDef sec_limit	"Section limit exceeded"


\ *******************************
\ Miscellaneous support functions
\ *******************************

[forth]  also c-c definitions

\ For testing
0 constant testing?

testing? [if]
-1 constant big-endian?
 0 constant little-endian?
[then]

: under      \ a b -- a a b ; duplicate NOS under TOS
  over swap
;

big-endian? [if]
: wflip      \ xxab -- xxba
  dup $FF and 8 lshift			\ -- xxab 00b0
  swap 8 rshift $FF and			\ -- 00b0 000a
  or
;
: lflip      \ abcd -- dcba
  dup #24 rshift $FF and	\ -- abcd 000a
  over 8 rshift $FF00 and  or	\ -- abcd 00ba
  over 8 lshift $FF0000 and  or	\ -- abcd 0cba
  swap $FF and #24 lshift  or	\ -- dcba
;
cellbits #64 = [if]
: xflip		\ abcdefgh -- hgfedcba
  dup #56 rshift $FF and	\ -- abcd:efgh 0000:000a
  over #40 rshift $FF00 and or	\ -- abcd:efgh 0000:00ba
  over #24 rshift $FF:0000 and or \ -- abcd:efgh 0000:0cba
  over 8 rshift $FF00:0000 and or \ -- abcd:efgh 0000:dcba
  over 8 lshift $FF:0000:0000 and or \ -- abcd:efgh 000e:dcba
  over #24 lshift $FF00:0000:0000 and or \ -- abcd:efgh 00fe:dcba
  over #40 lshift $FF:0000:0000:0000 and or \ -- abcd:efgh 0gfe:dcba
  swap #56 lshift $FF00:0000:0000:0000 and or \ -- hgfe:dcba
[then]
[then]

little-endian? [if]
: wflip  ; immediate
: lflip  ; immediate
: xflip  ; immediate
[then]

: .pfa          \ pfa -- ; display associated name
  ?dup
  if  body> >name .name  else  ." none "  endif  ;

hex

also forth definitions
: u*            \ u1 u2 -- u3
  um* drop  ;

: u/mod         \ u1 u2 -- u3 u4
  0 swap um/mod  ;
previous definitions


vocabulary sections.voc


\ ******************************************************************
\ The following words convert 16 and 32 bit values to byte sequences
\ ******************************************************************
\ They are used in the disassemblers.

also sections.voc definitions

big-endian? [if]

: word>2bytes	\ w -- bl bh ; BE will process high byte first
  dup $FF and  swap 8 rshift $FF and
;

: long>4bytes	\ l -- bl bml bmh bh ; BE will process high byte first
  dup $FF and
  swap dup 8 rshift $FF and
  swap dup #16 rshift $FF and
  swap #24 rshift $FF and
;

cellbits #32 = [if]
: xlong>8bytes	\ x -- bl bml bmh bh 0 0 0 0 ; BE will process high byte first
\ On a 32 bit host, only the low 32 bits of x are used.
  long>4bytes 0 0 0 0
;
[then]
cellbits #64 = [if]
: xlong>8bytes	\ x -- bl bml bmh bh 0 0 0 0 ; BE will process high byte first
\ On a 32 bit host, only the low 32 bits of x are used.
  dup >r long>4bytes  r> #32 rshift long>4bytes
;
[then]

: 2bytes>word	\ bh bl -- w ; BE high byte read first
  $FF and  swap $FF and 8 lshift or
;

: 4bytes>long	\ bh bmh bml bl -- l ; BE high byte read first
  $FF and
  swap $FF and 8 lshift or
  swap $FF and #16 lshift or
  swap $FF and #24 lshift or
;

cellbits #32 = [if]
: 8bytes>xlong	\ x x x x bh bmh bml bl -- x ; BE high byte read first
\ On a 32 bit host, the x bytes are discarded.
  4bytes>long >r  2drop 2drop  r>
;
[then]
cellbits #64 = [if]
: 8bytes>xlong	\ hbh hbmh hbml hbl lbh lbmh lbml lbl -- x ; BE high byte read first
\ On a 32 bit host, the x bytes are discarded.
  4bytes>long >r  4bytes>long #32 lshift  r> or
;
[then]

[then]

little-endian? [if]

: word>2bytes	\ w -- bh bl ; LE will process low byte first
  dup 8 rshift $FF and  swap $FF and
;

: long>4bytes	\ l -- bh bmh bml bl ; LE will process low byte first
  dup #24 rshift $FF and
  swap dup #16 rshift $FF and
  swap dup 8 rshift $FF and
  swap $FF and
;

cellbits #32 = [if]
: xlong>8bytes	\ x -- 0 0 0 0 bh bmh bml bl ; LE will process low byte first
\ On a 32 bit host, the high bytes of a 64 bit item are zero or discarded.
  >r  0 0 0 0  r> long>4bytes
;
[then]
cellbits #64 = [if]
: xlong>8bytes	\ x -- hbh hbmh hbml hbl bh bmh bml bl ; LE will process low byte first
\ On a 32 bit host, the high bytes of a 64 bit item are zero or discarded.
  dup >r  #32 rshift long>4bytes  r> long>4bytes
;
[then]

: 2bytes>word	\ bl bh -- w ; LE low byte read first
  $FF and 8 lshift  swap $FF and or
;

: 4bytes>long	\ bl bml bmh bh -- l ; LE low byte read first
  $FF and  #24 lshift
  swap $FF and  #16 lshift  or
  swap $FF and  #8 lshift  or
  swap $FF and  or
;

cellbits #32 = [if]
: 8bytes>xlong	\ bl bml bmh bh 0 0 0 0 -- x ; LE low byte read first
\ On a 32 bit host, the high bytes of a 64 bit item are zero or discarded.
  2drop 2drop  4bytes>long
;
[then]
cellbits #64 = [if]
: 8bytes>xlong	\ bl bml bmh bh 0 0 0 0 -- x ; LE low byte read first
\ On a 32 bit host, the high bytes of a 64 bit item are zero or discarded.
  4bytes>long #32 lshift >r  4bytes>long  r> or
;
[then]

[then]


\ *********************
\ Structure definitions
\ *********************

struct MS-desc          \ -- ; -- len ; describes a SECTION
  int MS-handle         \ depends on I/O type, but -1 always indicates closed, not in use
  int MS-iovector       \ pointer to an array of XTs
  int MS-type           \ type of data, CURR-ISECT=IDATA, CURR-UDATA=UDATA, ...
  1 field MS-active     \ non-zero for section active (see banked systems)
  1 field MS-flags	\ bit flags ; bit 0=write ignore
  1 field MS-spare2
  1 field MS-spare3
  int MS-FlashStart	\ start address of Flash for hex files ; SFP026
  int MS-high           \ highest target address in the section
  int MS-low            \ lowest target address in the section
  int MS-DP             \ target dictionary pointer for this section
  int MS-DPmax          \ highest target address reached by MS-DP
  int MS-BP             \ target buffer pointer, allocated from top downwards
  int MS-bank           \ which bank is this section, 0=unbanked
  int MS-pageid         \  page number within the bank
  int MS-symbol         \ pointer to section's symbol information
  int MS-LINK           \ pointer to previous section
  #16 field MS-private  \ data specific to the I/O, e.g. EPEM offset
end-struct

: (.section)    \ section -- ; show full details of section
  cr ." Section: " dup .pfa
  cr ." handle = " dup MS-handle @ .dword
     ."     I/O vector = " dup MS-iovector @ .pfa
     ."     type = " dup MS-type @ .pfa
     ."     active = " dup MS-active c@ .
  cr ." Hex file base address = " dup MS-FlashStart @ .dword	\ SFP026
  cr ." Memory limits = " dup MS-low @ .dword  space  dup MS-high @ .dword
  cr ." Dictionary pointer DP = " dup MS-DP @ .dword  ." , max = " dup MS-DPmax @ .dword
     ." , BP = " dup MS-BP @ .dword
  cr ." Bank ID = " dup MS-bank @ ?dup
     if  .pfa  ." , page ID = " dup MS-pageid @ .dword  else  ." unbanked"  endif
  cr ." Symbol data at " dup MS-symbol @ .dword
  cr
  drop
;

struct MSIO-desc        \ -- ; -- len ; describes the vector for SECTION I/O
  int MSIO-open
  int MSIO-close
  int MSIO-save
  int MSIO-load
  int MSIO-read
  int MSIO-write
  int MSIO-org          \ set DP within the section
  int MSIO-here         \ return DP
  int MSIO-allot        \ adjust DP
  int MSIO-c@(t)        \ 8 bit
  int MSIO-w@(t)        \ 16 bit
  int MSIO-l@(t)        \ 32 bit
  int MSIO-x@(t)        \ reserved for 64 bit
  int MSIO-c!(t)        \ 8 bit
  int MSIO-w!(t)        \ 16 bit
  int MSIO-l!(t)        \ 32 bit
  int MSIO-x!(t)        \ reserved for 64 bit
end-struct

struct bank-desc        \ -- ; -- len ; describes a BANK data structure
  int bank-high         \ end/highest address in the bank
  int bank-low          \ start/lowest address in the bank
  int bank-type         \ data mode (CDATA, IDATA ...)
  int bank-activepage   \ points to active page in this bank
end-struct

: (.bank)       \ bank --
  cr ." Bank: " dup .pfa
  cr ." Memory limits = " dup bank-low @ .dword space  dup bank-high @ .dword
     ."      type = " dup bank-type @ .
  cr
  drop
;


\ ***********************
\ Variables and constants
\ ***********************

MS-desc buffer: DummySection	\ -- addr
\ Dummy section that makes xHERE(t) return zero before sections
\ are defined.

variable section-chain  \ -- addr ; main memory chain
  section-chain off
variable curr-csect     \ -- addr ; current CDATA section
  DummySection curr-csect !
variable curr-isect     \ -- addr ; current IDATA section
  DummySection curr-isect !
variable curr-usect     \ -- addr ; current UDATA section
  DummySection curr-usect !

variable dsection-chain \ -- addr ; data memory chain for harvard architectures
  dsection-chain off
(( SFP013
variable curr-idsect    \ -- addr ; current IDDATA section
variable curr-udsect    \ -- addr ; current UDDATA section
))

\ *** KLUGE warning *** this CONSTANT is modified! Will be replaced by VALUE later
curr-csect value curr-sect   \ -- addr ; last selected section of ANY type

variable curr-bank      \ -- addr ; points to last created/selected bank
variable <single-section>       \ -- addr ; true if only one section
                                \ e.g. when building a hosted Forth


-1 constant closed      \ -- -1 ; -1 is used as a closed (not in use) indicator)
 1 constant hnd-ign     \ -- 1 ; handle for WRITE-IGNORE sections
 2 constant hnd-inv     \ -- 2 ; handle for WRITE-INVALID sections

-1 constant active      \ -- -1 ; -1 is used to indicate that a bank is selected
 0 constant inactive


\ ******************
\ Section primitives
\ ******************

: MSIO-exec     \ i*x section offset -- j*y ; execute method from section
  over MS-iovector @ +  @ execute  ;

: MSIOop        \ offset -- ; i*x section -- j*y ; child word executes selected vector
  create
    ,
  does>                                 \ section addr --
    @  MSIO-exec  ;

0 MSIO-open  MSIOop open(ms)            \ section --
0 MSIO-close MSIOop close(ms)           \ section --
0 MSIO-save  MSIOop save(ms)            \ section --
0 MSIO-load  MSIOop load(ms)            \ section --
0 MSIO-read  MSIOop read(ms)            \ addr(t) addr(h) len section --
0 MSIO-write MSIOop write(ms)           \ addr(h) addr(t) len section --
0 MSIO-org   MSIOop org(ms)             \ addr(t) section --
0 MSIO-here  MSIOop here(ms)            \ section -- addr(t)
0 MSIO-allot MSIOop allot(ms)           \ n section --
0 MSIO-c@(t) MSIOop c@(ms)              \ addr(t) section -- b
0 MSIO-w@(t) MSIOop w@(ms)              \ addr(t) section -- w
0 MSIO-l@(t) MSIOop l@(ms)              \ addr(t) section -- l
0 MSIO-c!(t) MSIOop c!(ms)              \ b addr(t) section --
0 MSIO-w!(t) MSIOop w!(ms)              \ w addr(t) section --
0 MSIO-l!(t) MSIOop l!(ms)              \ l addr(t) section --
64bit? [if]
0 MSIO-x@(t) MSIOop x@(ms)              \ addr(t) section -- x
0 MSIO-x!(t) MSIOop x!(ms)              \ x addr(t) section --
[then]


\ ***********************************
\ Memory Section and Chain primitives
\ ***********************************

: in-section?   \ addr(t) section -- t/f
  MS-high 2@ u-within?  ;

: closed?       \ section -- flag ; true if closed
  MS-handle @ closed =  ;

: open?         \ section -- flag ; true if open
  MS-handle @ closed <>  ;

: active?       \ section -- flag ; true if open and active
  dup MS-handle @ closed <>  swap MS-active c@  and  ;

: size(ms)      \ section -- size ; return size of section
  MS-high 2@ 1+ swap -  ;

: used(ms)      \ section -- size ; return used amount of section
  dup MS-DP @  swap MS-low @ -  ;

: savesize(ms)  \ section -- size ; return amount to save ; SFP009
  dup MS-DP @ over MS-DPmax @
  2dup u<                               \ return umax
  if  swap  endif
  drop
  swap MS-low @ -  ;

: ?addr-ok(ms)  \ addr(t) section -- ; error if addr(t) not in section
  MS-high 2@ u-within? 0=
\  abort" Memory address not in section"	\ SFP016
  sec_nosection ?throw
;

: ?addr/len-ok(ms)       \ addr(t) len section -- ; error if addr/len not in section
  >r
  over + 1- r@ ?addr-ok(ms)
  r> ?addr-ok(ms)  ;

: addr(t>h)     \ addr(t) section -- addr(h) ; convert to host buffer address ; *** CODE this ***
  tuck  MS-low @ -  swap MS-handle @ +  ;

: set-curr-sect \ section -- ; set as current section and type
  dup MS-type @                         \ -- section curr-xsect
  dup to curr-sect			\ set CURR-SECT
  !  ;                                  \ set CURR-xSECT

defer check-xdata
  assign noop to-do check-xdata

: xdata         \ curr-xsect -- ; make current xDATA section the current section and type
  to curr-sect  check-xdata
\  curr-sect @ 0= abort" xDATA: no section of this type"
;

: change-io(ms) \ vector section -- ; change I/O type of given section
  dup close(ms)                         \ close with current I/O type
  tuck MS-iovector !                    \ install new vector
  open(ms)  ;                           \ reopen

: change-curr-io(ms)     \ vector -- ; change I/O type of the current section.
  curr-sect @ change-io(ms)  ;          \ current section

\ JL003...
: cdata-type?   \ section -- flag ; return true if section is of type CDATA
  ms-type @ curr-csect =
;

: idata-type?   \ section -- flag ; return true if section is of type IDATA
  ms-type @ curr-isect =
;

: udata-type?   \ section -- flag ; return true if section is of type UDATA
  ms-type @ curr-usect =
;
\ ...JL003


\ *********************
\ Section file handling
\ *********************

\ SFP024...

defer send-blocks	\ -- ; write out the data
  assign noop to-do send-blocks
variable host-address	\ -- addr ; holds image address in host
variable host-length	\ -- addr ; holds image length
variable target-address	\ -- addr ; holds target image load address
variable exec-address	\ -- addr ; holds target start address
#16 buffer: $bin-ext	\ -- addr ; binary file extension with '.' as counted string
  s" .img" $bin-ext place	\ default to .IMG
#16 buffer: $ext-name	\ -- addr ; holds output file name extension WITH '.'
  $ext-name #16 erase
#64 buffer: CRC$buff	\ -- addr
\ Holds the last CRC generated, as a counted hex string.

also c-c definitions
0 value CRCnames?	\ -- flag ; true if names have CRCs
0 value SaveCdataOnly?	\ -- flag ; true if only CDATA sections are saved
0 value SaveAllSecData?	\ -- flag ; true to save all sections of secondary files
: setBinExt	\ "text" -- ; set binary file extension ; setBinExt .bin
  parse-name $bin-ext place
;
previous definitions

((
: formCRC$	\ crc u --
\ Save the CRC as u hex digits.
  base @ >r hex
  >r  0 <#  r> 0 ?do # loop  #> CRC$buff place
  r> base !
;
: addCRC$	\ dest --
\ Add CRC string to name at dest if required.
  CRCnames? if
    CRC$buff count rot append
  else
    drop
  endif
;
))

: genCRC16text	\ crc -- caddr len
\ Generate the CRC text to be added to the section name.
  base @ >r hex
  0 <# # # # # [char] _ hold #>
  r> base !
;

: (set-img-name)	\ sect pcb -- ; set name of section + .IMG into section
  >r
  r@ reset-pcb                          \ clear PCB
  body> >name count $01F and            \ addr + len of section name
  r> p-name swap cmove                  \ copy to PCB
;

: set-img-name  \ sect pcb -- ; set name of section + .IMG into section
  dup >r (set-img-name)			\ set base name
  $bin-ext r> set-ext			\ form .IMG extension ; SFP038
\  c" .IMG" r> set-ext			\ form .IMG extension
;

: sectName	\ sect -- caddr len
  body> >name count $01F and  ;		\ addr + len of section name

FileBuff: HexFileDev	\ -- ; genio device for file

: write-image   \ addr len sect
\ Write out host buffer to image and optional HEX/ELF file.
\ primary image file
  { addr(h) len sect | name[ 256 ] crc[ 16 ] -- }
  SaveCdataOnly? if			\ check for CDATA sections
    sect cdata-type? 0=
    if  exit  endif
  endif
  CRCnames? if				\ CRCs in names?
    addr(h) len calcCRC16		\ calculate section CRC
    genCRC16text crc[ place
  else
    crc[ off
  endif
  sect sectName name[ place		\ Image file name
  CRCnames? if				\ CRCs in names?
    crc[ count name[ append		\ CRC text if needed
  endif
  $bin-ext count name[ append		\ extension ; SFP038
\ s" .img" name[ append			\ extension
  addr(h) len name[ count exhale	\ abort" Image file write error" drop	SFP016
\ secondary image file
  sect cdata-type?  SaveAllSecData? or if
    action-of send-blocks ['] noop <> if
      sect sectName name[ place		\ HEX file name
      CRCnames? if			\ CRCs in names?
        crc[ count name[ append		\ CRC text if needed
      endif
      $ext-name count name[ append	\ extension

      name[ count fileExist? if		\ file is always to be created
        name[ count delete-file drop
      endif

      name[ count r/w HexFileDev open-gio sec_create ?throw drop
      addr(h) host-address !  len host-length !
      sect MS-FlashStart @ target-address !	\ SFP026

      op-handle @ >r  HexFileDev op-handle !
      send-blocks
      r> op-handle !

      HexFileDev close-gio sec_close ?throw
    endif
  endif
;

\ ...SFP024

: read-image    { addr(h) len sect | spcb[ pcb-len ] -- size ; read into host buffer }
  sect spcb[ set-img-name
  spcb[ open-path-pcb sec_open ?throw	\ abort" Can't open image file"	SFP016
  addr(h)                               \ address of memory buffer
  spcb[ handle path-size		\ length of existing image file
  dup sect size(ms) u> sec_imagesize ?throw	\ abort" Image file bigger than size of section"	SFP016
  dup len u> sec_bufferlen ?throw		\ abort" Image file larger than buffer"	SFP016
  spcb[ handle read-path sec_read ?throw	\ abort" Image file read error"		SFP016
  spcb[ close-path-pcb sec_close ?throw		\ abort" Image file close error"	SFP016
;

: get-img-buff  \ u -- addr(h) ; allocate a buffer of u bytes
  #256 + allocate sec_allocate ?throw
;

: free-img-buff \ addr(h) --
  free drop
;


\ *********************************************
\ Memory operators which look through the chain
\ *********************************************
\ SC = section code
\ SD = section data


: (addr>section)	\ addr(t) chain -- section 0 | ior
\ Search the chain to find which section contains the address.
\ On error, just an ior is returned. On success, a section and
\ zero are returned.
  begin
    @ dup
   while				\ -- addr(t) section
    2dup in-section?			\ address in section
    over active? and			\   and the section is open
    if  nip 0 exit  then
    MS-link
  repeat
  2drop sec_undefined
;

: addr>section	\ addr(t) -- section
\ Return the section associated with a target address.
  section-chain (addr>section) dup ?throw
;

harvard? [if]
: addr>dsection	\ addr(t) -- section
\ Return the section associated with a target data address for a
\ Harvard target.
  dsection-chain (addr>section) dup ?throw
;
[then]

: mem-op(ms)     \ offset -- ; ??? addr(t) -- ??? ; creates a memory section chain operator
  create
    ,                                   \ lay XT
  does>
    @ >r                                \ save XT
\ check current CDATA section first
    dup curr-csect @ in-section? if
      curr-csect @ r> MSIO-exec  exit	\ execute the required method and done
    endif
[ harvard? 0= ] [if]			\ SFP035
\ check current IDATA section ; SFP033
    dup curr-isect @ in-section? if
      curr-isect @ r> MSIO-exec  exit	\ execute the required method and done
    endif
[then]
((
\ check current UDATA section ; SFP033
    dup curr-usect @ in-section? if
      curr-usect @ r> MSIO-exec  exit	\ execute the required method and done
    endif
))
\ if that fails look through the whole chain
    section-chain                       \ main section chain
    begin
      @ ?dup                            \ while sections left
     while                              \ -- addr(t) section
      2dup in-section?                  \ if address is in this section
      over active? and                  \   and the section is open
      if  r> MSIO-exec                  \ execute the required method
          exit                          \ and get out
      endif
      MS-link
    repeat
    r> drop
    cr ." Address = " .dword
    sec_undefined throw			\ SFP016
;

0 MSIO-c@(t) mem-op(ms) c@(sc)          \ addr(t) -- b
0 MSIO-w@(t) mem-op(ms) w@(sc)          \ addr(t) -- w
0 MSIO-l@(t) mem-op(ms) l@(sc)          \ addr(t) -- n
0 MSIO-x@(t) mem-op(ms) x@(sc)          \ addr(t) -- x
0 MSIO-c!(t) mem-op(ms) c!(sc)          \ b addr(t) --
0 MSIO-w!(t) mem-op(ms) w!(sc)          \ w addr(t) --
0 MSIO-l!(t) mem-op(ms) l!(sc)          \ n addr(t) --
0 MSIO-x!(t) mem-op(ms) x!(sc)          \ x addr(t) --

harvard? [if]
: dmem-op(ms)    \ offset -- ; ??? addr(t) -- ??? ; creates a memory section chain operator
  create
    ,                                   \ lay XT
  does>
    @ >r                                \ save XT
\ if that fails look through the whole data chain
    dsection-chain                      \ main DATA section chain
    begin
      @ ?dup                            \ while sections left
     while                              \ -- addr(t) section
      2dup in-section?                  \ if address is in this section
      over active? and                  \   and the section is open
      if  r> MSIO-exec                  \ execute the required method
          exit                          \ and get out
      endif
      MS-link
    repeat
    r> drop
    sec_undefined throw
\    .rs  1 abort" memory address not in defined section"
;

0 MSIO-c@(t) dmem-op(ms) c@(sd)         \ addr(t) -- b
0 MSIO-w@(t) dmem-op(ms) w@(sd)         \ addr(t) -- w
0 MSIO-l@(t) dmem-op(ms) l@(sd)         \ addr(t) -- n
0 MSIO-x@(t) dmem-op(ms) x@(sd)         \ addr(t) -- n
0 MSIO-c!(t) dmem-op(ms) c!(sd)         \ b addr(t) --
0 MSIO-w!(t) dmem-op(ms) w!(sd)         \ w addr(t) --
0 MSIO-l!(t) dmem-op(ms) l!(sd)         \ n addr(t) --
0 MSIO-x!(t) dmem-op(ms) x!(sd)         \ n addr(t) --
[endif]


\ ***********************************************
\ memory section buffer handling words and vector
\ ***********************************************

: open-mem(ms)   \ section -- ; open a memory buffer
  dup open? sec_inuse ?throw		\ abort" Section already in use"	SFP016
  dup size(ms) get-img-buff swap MS-handle !  ;

: close-mem(ms)  \ section -- ; close a memory buffer
  dup open?
  if  dup MS-handle @ free-img-buff  endif
  -1 swap MS-handle !  ;

: save-mem(ms)   { sect | spcb[ pcb-len ] -- ; save a memory buffer }
  sect MS-handle @  sect savesize(ms)   \ address and length of memory buffer ; SFP009
  sect write-image  ;

: load-mem(ms)   { sect -- ; load a memory buffer }
  sect MS-handle @  sect size(ms)  sect read-image drop  ;

: read-mem(ms)   { addr(t) addr(h) len section -- }
  addr(t) len section ?addr/len-ok(ms)
  addr(t) section addr(t>h)  addr(h)  len cmove  ;

: write-mem(ms)  { addr(h) addr(t) len section -- }
  addr(t) len section ?addr/len-ok(ms)
  addr(h)  addr(t) section addr(t>h)  len cmove  ;

: org-mem(ms)    \ addr(t) section --
\  2dup ?addr-ok(ms)
  MS-DP !  ;

: here-mem(ms)   \ section -- addr(t)
  MS-DP @  ;

((
: allot-mem(ms)  \ n section --
  tuck MS-DP +!                         \ update
  dup MS-DP @ swap ?addr-ok(ms)  ;      \ range check
))
: allot-mem(ms)  \ n section --
  MS-DP +!  ;                           \ update

: c@-mem(ms)     \ addr(t) section -- b
\  2dup ?addr-ok(ms)
  addr(t>h) c@  ;

: w@-mem(ms)     \ addr(t) section -- w
\  2dup ?addr-ok(ms)
  addr(t>h) w@ wflip ;

cellbits #32 = [if]
: l@-mem(ms)     \ addr(t) section -- l
\  2dup ?addr-ok(ms)
  addr(t>h) @ lflip ;
[else]
: l@-mem(ms)     \ addr(t) section -- l
\  2dup ?addr-ok(ms)
  addr(t>h) l@ lflip ;
[then]

cellbits #32 = [if]
big-endian? [if]
: x@-mem(ms)	\ addr(t) section -- x
  addr(t>h) 4 + @ lflip  ;		\ ignore top bytes
[else]
: x@-mem(ms)	\ addr(t) section -- x
  addr(t>h) @ lflip  ;			\ ignore top bytes
[then]
[then]
cellbits #64 = [if]
: x@-mem(ms)	\ addr(t) section -- x
  addr(t>h) @ xflip  ;
[then]

: no64support   \ addr(t) section -- x1 x2
  sec_no64 throw			\ 1 abort" no 64 bit support"	SFP016
;

: c!-mem(ms)     \ b addr(t) section --
\  2dup ?addr-ok(ms)
  addr(t>h) c!  ;

: w!-mem(ms)     \ w addr(t) section --
  rot wflip -rot
\  2dup ?addr-ok(ms)
  addr(t>h) w!  ;

cellbits #32 = [if]
: l!-mem(ms)     \ l addr(t) section --
  rot lflip -rot  addr(t>h) !  ;
[then]
cellbits #64 = [if]
: l!-mem(ms)     \ l addr(t) section --
  rot lflip -rot  addr(t>h) l!  ;
[then]

cellbits #32 = [if]
big-endian? [if]
: x!-mem(ms)     \ x addr(t) section --
  addr(t>h)  0 over !  4 + !  ;		\ set high portion to 0
[else]
: x!-mem(ms)     \ x addr(t) section --
  addr(t>h) tuck !  0 swap 4 + !  ;	\ set high portion to 0
[then]
[then]
cellbits #64 = [if]
: x!-mem(ms)     \ x addr(t) section --
  rot xflip -rot  addr(t>h) !  ;
[then]

create MSIO-mem \ -- addr ; vector for I/O to host memory
  ' open-mem(ms) ,
  ' close-mem(ms) ,
  ' save-mem(ms) ,
  ' load-mem(ms) ,
  ' read-mem(ms) ,
  ' write-mem(ms) ,
  ' org-mem(ms) ,
  ' here-mem(ms) ,
  ' allot-mem(ms) ,
  ' c@-mem(ms) ,
  ' w@-mem(ms) ,
  ' l@-mem(ms) ,
  ' x@-mem(ms) ,
  ' c!-mem(ms) ,
  ' w!-mem(ms) ,
  ' l!-mem(ms) ,
  ' x!-mem(ms) ,

\ ****************************
\ WRITE-IGNORE memory handlers
\ ****************************

: open-ign(ms)   \ section --
  dup open? sec_inuse ?throw		\ abort" Section already in use"	SFP016
  hnd-ign swap MS-handle !  ;

: close-ign(ms)  \ section --
  -1 swap MS-handle !  ;

: save-ign(ms)   \ section --
  drop  ;

: load-ign(ms)   \ section --
  drop  ;

: read-ign(ms)   \ addr(t) addr(h) len section --
  drop rot drop  erase  ;

: write-ign(ms)  \ addr(h) addr(t) len section --
  2drop 2drop  ;

: @-ign(ms)      \ addr(t) section -- 0
  2drop  0  ;

: !-ign(ms)      \ x addr(t) section --
  2drop drop  ;

create MSIO-ignore \ -- addr ; vector for I/O requests ignored
  ' open-ign(ms) ,
  ' close-ign(ms) ,
  ' save-ign(ms) ,
  ' load-ign(ms) ,
  ' read-ign(ms) ,
  ' write-ign(ms) ,
  ' org-mem(ms) ,
  ' here-mem(ms) ,
  ' allot-mem(ms) ,
  ' @-ign(ms) ,
  ' @-ign(ms) ,
  ' @-ign(ms) ,
64bit? [if] ' @-ign(ms) ,  [else]  ' no64support ,  [then]
  ' !-ign(ms) ,
  ' !-ign(ms) ,
  ' !-ign(ms) ,
64bit? [if] ' @-ign(ms) ,  [else]  ' no64support ,  [then]


\ *****************************
\ WRITE-INVALID memory handlers
\ *****************************

: wr-inv(ms)     \ -- ; error
  sec_writeinvalid throw	\ 1 abort" Operation invalid in WRITE-INVALID section"	SFP016
;

create MSIO-invalid \ -- addr ; vector for I/O requests invalid
  ' open-ign(ms) ,
  ' close-ign(ms) ,
  ' save-ign(ms) ,
  ' load-ign(ms) ,
  ' read-ign(ms) ,
  ' wr-inv(ms) ,
  ' org-mem(ms) ,
  ' here-mem(ms) ,
  ' allot-mem(ms) ,
  ' @-ign(ms) ,
  ' @-ign(ms) ,
  ' @-ign(ms) ,
  ' @-ign(ms) ,
  ' wr-inv(ms) ,
  ' wr-inv(ms) ,
  ' wr-inv(ms) ,
  ' wr-inv(ms) ,

create MSIO-readonly	\ -- addr ; vector for I/O to read-only memory ; SFP032
  ' open-mem(ms) ,
  ' close-mem(ms) ,
  ' save-mem(ms) ,
  ' load-mem(ms) ,
  ' read-mem(ms) ,
  ' wr-inv(ms) ,			\ ' write-mem(ms) ,
  ' org-mem(ms) ,
  ' here-mem(ms) ,
  ' wr-inv(ms) ,			\ ' allot-mem(ms) ,
  ' c@-mem(ms) ,
  ' w@-mem(ms) ,
  ' l@-mem(ms) ,
64bit? [if] ' x@-mem(ms) ,  [else]  ' no64support ,  [then]
  ' wr-inv(ms) ,			\ ' c!-mem(ms) ,
  ' wr-inv(ms) ,			\ ' w!-mem(ms) ,
  ' wr-inv(ms) ,			\ ' l!-mem(ms) ,
  ' wr-inv(ms) ,			\ ' x!-mem(ms) ,
((
struct MSIO-desc        \ -- ; -- len ; describes the vector for SECTION I/O
  int MSIO-open
  int MSIO-close
  int MSIO-save
  int MSIO-load
  int MSIO-read
  int MSIO-write
  int MSIO-org          \ set DP within the section
  int MSIO-here         \ return DP
  int MSIO-allot        \ adjust DP
  int MSIO-c@(t)        \ 8 bit
  int MSIO-w@(t)        \ 16 bit
  int MSIO-l@(t)        \ 32 bit
  int MSIO-x@(t)        \ reserved for 64 bit
  int MSIO-c!(t)        \ 8 bit
  int MSIO-w!(t)        \ 16 bit
  int MSIO-l!(t)        \ 32 bit
  int MSIO-x!(t)        \ reserved for 64 bit
end-struct
))

\ **********************************************
\ Generic operators usable by many section types
\ **********************************************

: read-gen(ms)   { addr(t) addr(h) len sect -- }
  addr(t) addr(h)  len 0 ?do
    over sect c@(ms)  over c!
    swap 1+ swap 1+
  loop
  2drop  ;

: write-gen(ms)  { addr(h) addr(t) len sect -- }
  addr(h) addr(t)  len 0
  ?do
    over c@  over sect c!(ms)
    swap 1+ swap 1+
  loop
  2drop  ;

: save-gen(ms)   { sect | buff len -- ; save memory image }  \ SFP005
  sect used(ms) -> len
  len get-img-buff -> buff
  sect MS-low @  buff len  sect read-gen(ms)
  buff len  sect write-image
  buff free-img-buff  ;

: load-gen(ms)   { sect -- ; load a memory image }
  sect size(ms) get-img-buff             \ get buffer
  dup  dup sect size(ms) sect read-image \ read file into it
  sect write-gen(ms)                     \ copy to emulator
  free-img-buff  ;                       \ release buffer


\ *************************************************
\ Words that operate on all the sections in a chain
\ *************************************************

: all-sects-op  \ offset -- ; -- ; only operates on offsets executing ( section -- )
                \ SFP013 updated for Harvard architecture
  create
    ,
  does>
    @ >r                                \ save offset
    section-chain
    begin
      @ ?dup
     while
      dup r@ MSIO-exec                  \ execute method
      MS-link
    repeat
[ harvard? ] [if]
    dsection-chain
    begin
      @ ?dup
     while
      dup r@ MSIO-exec                  \ execute method
      MS-link
    repeat
[endif]
    r> drop
;

also c-c definitions
0 MSIO-open  all-sects-op open-sections
0 MSIO-close all-sects-op close-sections
0 MSIO-save  all-sects-op save-sections
0 MSIO-load  all-sects-op load-sections
previous definitions

: use-mem-table	\ memtable chain -- ; SFP036
\ Switch sections in chain to use the given memory table. The
\ previous table is not closed and its handle/address is left
\ unchanged. This allows us to switch back later.
  begin
    @ dup
   while				\ -- memtable section
    2dup MS-iovector @ <> if
      2dup MS-iovector !
      cr ." Section " dup .pfa ." switched"
    endif
    over MSIO-mem = if			\ check for write ignore
      dup MS-flags c@ $01 and if
        MSIO-ignore over MS-iovector !
      endif
    endif
    MS-link
  repeat
  2drop
;

: use-cdata-table	\ memtable chain -- ; SFP036
\ Switch CDATA sections in chain to use the given memory table. The
\ previous table is not closed and its handle/address is left
\ unchanged. This allows us to switch back later.
  begin
    @ dup
   while				\ -- memtable section
    dup cdata-type? if
      2dup MS-iovector @ <> if
        2dup MS-iovector !
        cr ." Section " dup .pfa ." switched"
      endif
    endif
    MS-link
  repeat
  2drop
;

\ **********************************************
\ Target versions of dictionary management words
\ **********************************************

also c-c definitions

: here(t)       \ -- addr(t)
  curr-sect @ here-mem(ms)  ;

: chere(t)      \ -- addr(t)
  curr-csect @ here-mem(ms)  ;

: ihere(t)      \ -- addr(t)
  curr-isect @ here-mem(ms)  ;

: uhere(t)      \ -- addr(t)
  curr-usect @ here-mem(ms)  ;

: allot(t)      \ n --
  curr-sect @ allot-mem(ms)  ;

: CAllot(t)     \ n --
  curr-csect @ allot-mem(ms)  ;

: IAllot(t)     \ n --
  curr-isect @ allot-mem(ms)  ;

: UAllot(t)     \ n --
  curr-usect @ allot-mem(ms)  ;

: org(t)        \ addr(t) --
  curr-sect @ org-mem(ms)  ;

: Corg(t)       \ addr(t) --
  curr-csect @ org-mem(ms)  ;

: Iorg(t)       \ addr(t) --
  curr-isect @ org-mem(ms)  ;

: Uorg(t)       \ addr(t) --
  curr-usect @ org-mem(ms)  ;

: aligned(t)    \ addr(t) -- addr(t)' ; allign a target address to a cell boundary
  cell(t) 1- +  cell(t) negate and	\ 3 + -4 and
;

: Align(t)      \ --
  here(t) aligned(t) org(t)  ;

: CAlign(t)     \ -- ; SFP018
  Chere(t) aligned(t) Corg(t)  ;

: IAlign(t)     \ -- ; SFP018
  Ihere(t) aligned(t) Iorg(t)  ;

: UAlign(t)     \ -- ; SFP018
  Uhere(t) aligned(t) Uorg(t)  ;

previous definitions


\ ******************************************************
\ Misc user words need to be here to avoid forward refs.
\ ******************************************************

also c-c definitions

: cdata?        \ -- flag ; true if CDATA is current
  curr-sect curr-csect =  ;

: idata?        \ -- flag ; true if IDATA is current
  curr-sect curr-isect =  ;

: udata?        \ -- flag ; true if UDATA is current
  curr-sect curr-usect =  ;

previous definitions


\ ****************************************************
\ Define the comma words using the DEFERred primitives
\ ****************************************************

also c-c definitions
[-sin
: c,c(t)        \ b -- ; 'c' comma into target CODE space
  chere(t) c!c(t)  addr(t)/byte callot(t)  ;

: C,D(T)        \ b -- ; 'c' comma into target DATA space
  ihere(t) c!d(t)  addr(t)/byte iallot(t)  ;

: w,c(t)        \ w -- ; 'w' comma into target CODE space
  chere(t) w!c(t)  addr(t)/word callot(t)  ;

: w,D(T)        \ w -- ; 'w' comma into target DATA space
  ihere(t) w!d(t)  addr(t)/word iallot(t)  ;

: l,c(t)        \ l -- ; 'l' comma into target CODE space
  chere(t) l!c(t)  addr(t)/long callot(t)  ;

: l,D(T)        \ l -- ; 'l' comma into target DATA space
  ihere(t) l!d(t)  addr(t)/long iallot(t)  ;

: x,c(t)        \ l -- ; 'l' comma into target CODE space
  chere(t) x!c(t)  addr(t)/xlong callot(t)  ;

: x,D(T)        \ l -- ; 'l' comma into target DATA space
  ihere(t) x!d(t)  addr(t)/xlong iallot(t)  ;

: ,c(t)         \ n -- ; cell comma into target CODE space
  chere(t) !c(t)  cell(t) callot(t)  ;

: ,D(T)         \ n -- ; cell comma into target DATA space
  ihere(t) !d(t)  cell(t) iallot(t)  ;

harvard? [if]
: gc,(t)        \ b -- ; 'b' comma into current section ; SFP020
  here(t)  cdata?
  if  c!c(t)  else  c!d(t)  endif
  addr(t)/byte allot(t)  ;

: gw,(t)        \ w -- ; 'w' comma into current section ; SFP020
  here(t)  cdata?
  if  w!c(t)  else  w!d(t)  endif
  addr(t)/word allot(t)  ;

: gl,(t)        \ l -- ; 'l' comma into current section ; SFP020
  here(t)  cdata?
  if  l!c(t)  else  l!d(t)  endif
  addr(t)/long allot(t)  ;

: gx,(t)        \ l -- ; 'l' comma into current section ; SFP020
  here(t)  cdata?
  if  x!c(t)  else  x!d(t)  endif
  addr(t)/xlong allot(t)  ;

: g,(t)         \ cell -- ; cell comma into current section ; SFP020
  here(t)  cdata?
  if  !c(t)  else  !d(t)  endif
  cell(t) allot(t)  ;
[else]
: gc,(t)        \ b -- ; 'b' comma into current section
  here(t) c!c(t) addr(t)/byte allot(t)  ;

: gw,(t)        \ w -- ; 'w' comma into current section
  here(t) w!c(t) addr(t)/word allot(t)  ;

: gl,(t)        \ l -- ; 'l' comma into current section
  here(t) l!c(t) addr(t)/long allot(t)  ;

: gx,(t)        \ x -- ; 'x' comma into current section
  here(t) x!c(t) addr(t)/xlong allot(t)  ;

: g,(t)         \ cell -- ; cell comma into current section
  here(t) !c(t) cell(t) allot(t)  ;
[then]
sin]

: bad-mem       \ -- ; bad memory access
  sec_badmem throw	\ 1 abort" Unsupported memory access (size greater than cell)"	SFP016
;

previous definitions


\ *****************************************************************
\ Saving the IDATA sections at the end of the current CDATA section
\ *****************************************************************

: lay-idata(ms) { sect | buff -- ; lay memory image }
  sect used(ms) ?dup 0=			\ if len=0, do nothing ; SFP021
  if  exit  endif
  get-img-buff -> buff
  sect MS-low @  buff  sect used(ms)  sect read-gen(ms)
  sect used(ms) ,c(t)                   \ length
  mpe_standard? 0= if
    sect MS-low @ ,c(t)                 \ address
    sect MS-pageid @ ,c(t)              \ page identifier
  endif
  buff  sect used(ms)  bounds
  ?do  i c@ c,c(t)  loop                \ lay data
\ Alignment ???
  buff free-img-buff
;

also c-c definitions

: lay-idata     \ -- ; lay IDATA at end of current code section         \ SFP013 Harvard update
[ harvard? ] [if]
  dsection-chain                        \ main DATA section chain
[else]
  section-chain                         \ main section chain
[endif]
  begin
    @ ?dup
   while                                \ while sections left
    dup MS-type @ curr-isect =          \ if it's an IDATA section
    if  dup lay-idata(ms)  endif        \   add on this section
    MS-link
  repeat
  0 ,c(t)
;

previous definitions


\ ******************************************************
\ Assign actions to the DEFERred target memory operators
\ ******************************************************

also c-c definitions

16bit? [if]     \ added Harvard checking                \ SFP013
: Assign-Code-Drivers   ( -- ; assign memory drivers for code space )
  [ also sections.voc ]
  assign c@(sc) to-do c@c(t)
  assign c!(sc) to-do c!c(t)
  assign w@(sc) to-do w@c(t)
  assign w!(sc) to-do w!c(t)
  assign bad-mem to-do l@c(t)
  assign bad-mem to-do l!c(t)
  assign w@(sc) to-do @c(t)
  assign w!(sc) to-do !c(t)
  [ previous ]
;

harvard? [if]
: Assign-Data-Drivers   ( -- ; assign memory drivers for data space )
  [ also sections.voc ]
  assign c@(sd) to-do c@d(t)
  assign c!(sd) to-do c!d(t)
  assign w@(sd) to-do w@d(t)
  assign w!(sd) to-do w!d(t)
  assign bad-mem to-do l@d(t)
  assign bad-mem to-do l!d(t)
  assign w@(sd) to-do @d(t)
  assign w!(sd) to-do !d(t)
  [ previous ]
;
[else]
: Assign-Data-Drivers   ( -- ; assign memory drivers for data space )
  [ also sections.voc ]
  assign c@(sc) to-do c@d(t)
  assign c!(sc) to-do c!d(t)
  assign w@(sc) to-do w@d(t)
  assign w!(sc) to-do w!d(t)
  assign bad-mem to-do l@d(t)
  assign bad-mem to-do l!d(t)
  assign w@(sc) to-do @d(t)
  assign w!(sc) to-do !d(t)
  [ previous ]
;
[endif]
[endif]

32bit? [if]     \ added Harvard checking                \ SFP013
: Assign-Code-Drivers   ( -- ; assign memory drivers for code space )
  [ also sections.voc ]
  assign c@(sc) to-do c@c(t)
  assign c!(sc) to-do c!c(t)
  assign w@(sc) to-do w@c(t)
  assign w!(sc) to-do w!c(t)
  assign l@(sc) to-do l@c(t)
  assign l!(sc) to-do l!c(t)
  assign l@(sc) to-do @c(t)
  assign l!(sc) to-do !c(t)
  [ previous ]
;

harvard? [if]
: Assign-Data-Drivers   ( -- ; assign memory drivers for data space )
  [ also sections.voc ]
  assign c@(sd) to-do c@d(t)
  assign c!(sd) to-do c!d(t)
  assign w@(sd) to-do w@d(t)
  assign w!(sd) to-do w!d(t)
  assign l@(sd) to-do l@d(t)
  assign l!(sd) to-do l!d(t)
  assign l@(sd) to-do @d(t)
  assign l!(sd) to-do !d(t)
  [ previous ]
;
[else]
: Assign-Data-Drivers   ( -- ; assign memory drivers for data space )
  [ also sections.voc ]
  assign c@(sc) to-do c@d(t)
  assign c!(sc) to-do c!d(t)
  assign w@(sc) to-do w@d(t)
  assign w!(sc) to-do w!d(t)
  assign l@(sc) to-do l@d(t)
  assign l!(sc) to-do l!d(t)
  assign l@(sc) to-do @d(t)
  assign l!(sc) to-do !d(t)
  [ previous ]
;
[endif]
[endif]

64bit? [if]     \ added Harvard checking                \ SFP013
: Assign-Code-Drivers   ( -- ; assign memory drivers for code space )
  [ also sections.voc ]
  assign c@(sc) to-do c@c(t)
  assign c!(sc) to-do c!c(t)
  assign w@(sc) to-do w@c(t)
  assign w!(sc) to-do w!c(t)
  assign l@(sc) to-do l@c(t)
  assign l!(sc) to-do l!c(t)
  assign x@(sc) to-do x@c(t)
  assign x!(sc) to-do x!c(t)
  assign x@(sc) to-do @c(t)
  assign x!(sc) to-do !c(t)
  [ previous ]
;

harvard? [if]
: Assign-Data-Drivers   ( -- ; assign memory drivers for data space )
  [ also sections.voc ]
  assign c@(sd) to-do c@d(t)
  assign c!(sd) to-do c!d(t)
  assign w@(sd) to-do w@d(t)
  assign w!(sd) to-do w!d(t)
  assign l@(sd) to-do l@d(t)
  assign l!(sd) to-do l!d(t)
  assign x@(sd) to-do x@d(t)
  assign x!(sd) to-do x!d(t)
  assign x@(sd) to-do @d(t)
  assign x!(sd) to-do !d(t)
  [ previous ]
;
[else]
: Assign-Data-Drivers   ( -- ; assign memory drivers for data space )
  [ also sections.voc ]
  assign c@(sc) to-do c@d(t)
  assign c!(sc) to-do c!d(t)
  assign w@(sc) to-do w@d(t)
  assign w!(sc) to-do w!d(t)
  assign l@(sc) to-do l@d(t)
  assign l!(sc) to-do l!d(t)
  assign x@(sc) to-do x@d(t)
  assign x!(sc) to-do x!d(t)
  assign x@(sc) to-do @d(t)
  assign x!(sc) to-do !d(t)
  [ previous ]
;
[endif]
[endif]

previous definitions


\ ******************
\ Section navigation
\ ******************
\ The data here is in target units. For cell-addressed targets
\ the addresses and lengths are in units of cells.

also c-c definitions

: origin(t)     \ -- addr(t) ; returns start address of CDATA section
  curr-csect @ MS-low @  ;

: bottom(t)     \ -- addr(t) ; returns start address of current section
  curr-sect @ MS-low @  ;

: top(t)        \ -- addr(t) ; returns end address of current section
  curr-sect @ MS-high @  ;

: length(t)	\ -- u ; reurns length of current section ; SFP023
  curr-sect @ size(ms)
;

: mem-end(t)    \ -- addr(t) ; returns BP of current section
  curr-sect @ MS-bp @  ;        \ "MS-high" -> "MS-bp".                         \ MSD004

: from-end(t)   \ n section -- addr(t) ; allocates memory from top of section
  swap negate  over MS-BP +!            \ allocate from top
  dup MS-DP @  over MS-BP @ u>          \ check for crash
  sec_overlap ?throw			\ SFP022
  MS-BP @  ;                            \ return base address of buffer

: reserve(t)    \ n -- addr(t) ; allocates down from top of UDATA section
  curr-usect @ from-end(t)  ;

: cbank(t)      \ -- bank ; 0 if not banked
  curr-csect @ MS-bank @  ;

: pageid(t)	\ -- page|0 ; 0 if unpaged ; SFP027
  curr-csect @ MS-bank @ if
    curr-csect @ MS-pageid @
  else
    0
  endif
;

previous definitions


\ ******************************************
\ Section defining words and user DIRECTIVES
\ ******************************************

: (section)	\ section --
\ Action of referencing a section.
  active over MS-active c!		\ make section active
  set-curr-sect				\ set as current section and type
;

: (page)	\ page --
\ Action of referencing a page.
  dup MS-bank @				\ make all other pages in this bank inactive
[ harvard? ] [if]
  dup cdata-type?			\ choose appropriate address space
  if  section-chain  else  dsection-chain  endif
[else]
  section-chain
[endif]
  begin
    @ ?dup
   while				\ -- page bank section
    2dup MS-bank @ =			\ make inactive if this section is a page in this bank
    if  inactive over MS-active c!  endif
    MS-link
  repeat
  drop					\ discard bank
  active over MS-active c!		\ make this page/section active
  dup cdata-type? if			\ if this is a code page
    dup MS-bank @
    bank-activepage @ ?dup if		\ get the previously active page
      MS-symbol @ RemoveContext(s)	\ and remove it from the search order
    endif
    dup MS-symbol @ context(s) !(m)	\ make this the top vocabulary
  endif
  dup dup MS-bank @ bank-activepage !	\ mark it as the current page in the page's bank
  set-curr-sect				\ make it the current section
;

: setSect/Page	\ section|page --
\ Action of referencing a section or page.
  dup MS-bank @
  if  (page)  else  (section)  endif
;

0 value unstored?	\ -- flag
\ Set true when section should declare no host memory and should behave
\ as a WRITE-IGNORE section.

also c-c definitions

: idata         \ -- ; make current IDATA section the current section and type
  curr-isect xdata  ;

: udata         \ -- ; make current UDATA section the current section and type
  curr-usect xdata  ;

: cdata         \ -- ; make current CDATA section the current section and type
  curr-csect xdata  ;

: ignorable	\ -- ; new section will be WRITE-IGNORE.
  -1 -> unstored?  ;

: section       \ first last -- ; -- ; define a memory section  \ SFP013 Harvard conversion
                \ e.g. 0 7FFF CDATA SECTION FOO
  also interpreter definitions
  create
  previous definitions
    here >r  MS-desc allot&erase        \ save section pointer, reserve space for descriptor
    closed r@ MS-handle !               \ mark as not in use
    active r@ MS-active c!              \ mark as active
    unstored? if
      MSIO-ignore r@ MS-iovector !	\ install write-ignore driver
      $01 r@ MS-flags bor!		\ bit 0 is write-ignore indicator
      0 -> unstored?
    else
      MSIO-mem r@ MS-iovector !		\ install memory driver by default
    then
[ harvard? ] [if]
    cdata? if
      section-chain @ r@ MS-link !      \ link into code section chain
      r@ section-chain !
    else
      dsection-chain @ r@ MS-link !     \ link into code section chain
      r@ dsection-chain !
    endif
[else]
    section-chain @ r@ MS-link !        \ link into section chain
    r@ section-chain !
[endif]
    curr-sect r@ MS-type !              \ set type, by storing address of CURR-xSECT variable
    over r@ MS-FlashStart !		\ SFP026 ; sets low address, can be overridden
    2dup r@ MS-high 2!                  \ and set memory range
    1+ r@ MS-BP !                       \ buffer pointer grows down
    dup r@ MS-DP !                      \ initial value of DP = first
    r@ MS-DPmax !                       \ initial max value of DPmax=DP
    r@ set-curr-sect                    \ make this section the current one and type
    r> open(ms)                         \ open it
  does>
   (section)
;

: bank          \ first last -- ; -- ; define a memory bank area where pages are
  also interpreter definitions
  create
  previous definitions
    here  bank-desc allot&erase         \ reserve space
    dup curr-bank !                     \ make this the current bank
    curr-sect over bank-type !          \ memory type
    bank-high 2!                        \ memory limits
  does>
    curr-bank !  ;                      \ just select it


((
Pages are handled by building vocabularies in the symbol table.
When a page is referred to, the action is to make the relevant page
active for the memory system, to remove the previously active page in
the bank from the search order, and to make the specified page
vocabulary active.

Page vocabularies are created by PAGE-VOCABULARY <name>, where
<name> is a previously defined page, or by MAKE-PAGE-VOCS, which
creates target vocabularies for all CDATA pages.

The first two cells in a CDATA page are reserved to hold information
about the vocabularies defined in the page.
origin+0        voc-link field of last vocabulary defined in page
origin+cell     voc-link field of first vocabulary defined in page

See also PAGE-VOCABULARY and MAKE-PAGE-VOCS in XDTC41-4.FTH
))

defer MakePageVoc       \ -- context(s) ; builds the host vocabulary

: mpage         \ pageid -- ; -- ; define a page within a bank          \ SFP013 Harvard conversion
                \ e.g. $080 PAGE BAR
  >in @ swap                            \ save current position
  also interpreter definitions
  create        \ -- pageid
  previous definitions
    here >r  MS-desc allot&erase        \ reserve space for descriptor
    r@ MS-pageid !                      \ save page number
    closed r@ MS-handle !               \ mark as not in use
    inactive r@ MS-active c!            \ mark page as inactive
    MSIO-mem r@ MS-iovector !           \ install memory drivers by default
[ harvard? ] [if]
[else]
    curr-bank @ bank-type @ curr-csect =  \ if a CDATA section
    if
      section-chain @ r@ MS-link !      \ link into CODE section chain
      r@ section-chain !
    else
      dsection-chain @ r@ MS-link !     \ link into DATA section chain
      r@ dsection-chain !
    endif
[endif]
    r@ curr-bank @ bank-activepage !    \ make this the active page for the bank
    curr-bank @ r@ MS-bank !            \ save bank to which this page belongs
    curr-bank @ bank-type @ r@ MS-type !           \ set type from current bank
    curr-bank @ bank-high 2@
      over r@ MS-FlashStart !		\ SFP026
      2dup r@ MS-high 2!		\ and set memory range from current bank
    1+ r@ MS-BP !                       \ buffer pointer grows down
    dup r@ MS-DP !                      \ initial value of DP = first
    r@ MS-DPmax !                       \ initial max value of DPmax=DP
    r@ set-curr-sect                    \ make this the current section
    r@ open(ms)                         \ open it
    r@ cdata-type? if                   \ if this is a code page
      cell(t) 2* r@ MS-DP +!            \   cell for last VOC-LINK, cell for first VOC-LINK
      >in !                             \   restore input stream
      MakePageVoc                       \   make host vocabulary and point to it
      r> MS-symbol !
    else
      drop  r> drop                     \ otherwise discard data
    endif
  does>         \ -- page
    (page)
;

: pages         \ pageid -- ; -- ; define a page within a bank
  mpage  ;

: write-ignore  \ -- ; make the current sections ignore writes, always reading 0
  MSIO-ignore change-curr-io(ms)
  $01 curr-sect @ MS-flags bor!		\ bit 0 is write-ignore indicator
;

: write-invalid \ -- ; make the current sections fail on writes, always reading 0
  MSIO-invalid change-curr-io(ms)  ;

: write-fail	\ -- ; make the current section read-only ; SFP032
  MSIO-readonly curr-sect @ MS-iovector !  ;

: write-mem	\ -- ; make the current section read/write ; SFP032
  MSIO-mem curr-sect @ MS-iovector !  ;

: single-section-only   \ -- ; the current section will be the only section
  curr-sect @
  dup curr-csect !
  dup curr-isect !
  curr-usect !
  ?ROM off(m)
  <single-section> on  ;

: single-section?       \ -- flag ; true if one section only
  <single-section> @  ;

: all-saved     \ -- ; force current section to save everything
  curr-sect @
  dup MS-high @ 1+              \ so that length is correct
  swap MS-DPmax !  ;

: SetFlashBase	\ addr -- ; SFP026
\ Set section start address for HEX files.
  curr-sect @ MS-FlashStart !
;

: [sections	\ -- sec Csec Isec Usec
  curr-sect @
  curr-csect @  curr-isect @  curr-usect @
;

: sections]	\ sec Csec Isec Usec --
  setSect/Page  setSect/Page  setSect/Page  setSect/Page
;

previous definitions


\ **************
\ Memory testing
\ **************

: .section-type         \ section -- ; display section type
  case  MS-type @
    curr-csect of  ." CDATA "  endof                                            \ MSD012
    curr-isect of  ." IDATA "  endof                                            \ MSD012
    curr-usect of  ." UDATA "  endof                                            \ MSD012
                   ." huh?  "                                                   \ MSD012
  endcase  ;

: section-sane?         \ section -- t/f ; Perform "sanity" check on a section. \ MSD012
  >r                                                                            \ MSD012
  r@ ms-low @  r@ ms-dp   @     u> invert                                          \ MSD012
  r@ ms-bp  @  r@ ms-dp   @     u< invert  and                                     \ MSD012
  r@ ms-bp  @  r@ ms-high @ 1+  u> invert  and                                     \ MSD012
  r> drop                                                                       \ MSD012
;                                                                               \ MSD012

: (.section-report)     \ section -- ; show section details
  base @ >r  hex
  cr  dup .pfa                          \ section name
  #16 out @ - 0 max spaces              \ under TYPE
  dup .section-type                     \ type
  dup used(ms)  8 u.r                   \ used                                  \ MSD012
  dup MS-low @  space 8 u.r             \ start address                         \ MSD012
  dup MS-dp @   space 8 u.r             \ DP                                    \ MSD012
  dup MS-bp @   space 8 u.r             \ BP                                    \ MSD012
  dup MS-high @ space 8 u.r             \ end address                           \ MSD012
  dup MS-bank @ if
    dup MS-pageid @ 6 u.r               \ page ID                               \ MSD012
  endif                                                                         \ MSD012
  dup section-sane? not                                                         \ SFP015
  if  cr dup .pfa ."  section pointers out of range, or sections may overlap"  endif    \ SFP015
  drop                                                                          \ MSD012
  r> base !
;

also c-c definitions

: .section      \ --
  curr-sect @ (.section)  ;

: .sections     \ -- ; SFP013 for Harvard
  section-chain
  begin
    @ ?dup
   while
    dup (.section)
    MS-link
  repeat
[ harvard? ] [if]
  dsection-chain
  begin
    @ ?dup
   while
    dup (.section)
    MS-link
  repeat
[endif]
;

((
  curr-sect @
  curr-csect @  curr-isect @  curr-usect @
))

: .?curr-sect	\ section --
  cr  dup curr-sect @ =
  if  ." * "  else  ."   "  endif
  space .pfa
;

: .curr-sections	\ --
\ Display the current section of each type. The current type
\ is marked with an asterisk.
  curr-csect @ .?curr-sect ." cdata"
  curr-isect @ .?curr-sect ." idata"
  curr-usect @ .?curr-sect ." udata"
;

: .bank         \ --
  curr-bank @ (.bank)  ;


: .curr-sect    \ -- ; display the current section and type
\  ." current type    = " curr-sect .pfa
  curr-sect @ .pfa ." is current section"  ;

: .section-report       \ -- ; show details of all sections     \ SFP013 Harvard update
  cr  ." Section         Type      Used    Start       DP       BP      End  Page" \ MSD012
  section-chain
  begin
    @ ?dup
   while
    dup (.section-report)
    MS-link
  repeat
[ harvard? ] [if]
  dsection-chain
  begin
    @ ?dup
   while
    dup (.section-report)
    MS-link
  repeat
[endif]
;

pcb data-file-pcb	\ -- addr ; to control loading

: data-file	\ -- size ; DATA-FILE <filename>
  bl word $expandmacros data-file-pcb set-pathname
  data-file-pcb open-source-pcb sec_open ?throw	\ SFP043
\  data-file-pcb open-path-pcb sec_open ?throw
  data-file-pcb handle path-size	\ length of existing image file
  dup allocate sec_allocate ?throw	\ -- size addr(h) ; reserve space for it
  2dup swap data-file-pcb handle read-path sec_read ?throw drop
  data-file-pcb close-path-pcb sec_close ?throw
  swap >r				\ preserve size
  here(t) r@ curr-sect @ write-gen(ms)	\ copy data to target image
  r@ allot(t)  r>
;

previous definitions


\ **********************************
\ Downloading sections to the target
\ **********************************

also c-c definitions

defer download-section	\ sect -- ; SFP031
\ Download the given section to the target.
  assign drop to-do download-section

(( example from Uforth.fth
: (download-section)	{ sect | buff used -- ; download memory image }  \ SFP018, SFP024
  sect savesize(ms) dup -> used         \ find size of section contents
  get-img-buff -> buff                  \ put contents of section in buffer
  sect MS-low @  buff  used  sect read-gen(ms)
  buff  sect MS-low @  used             \ -- addr(h) addr(t) len
  begin
    dup #1024 u< invert                    \ send in 1k chunks
   while                                \ -- addr(h) addr(t) len
    >r
    2dup #1024 copy-cmnd>t >target      \ send chunk to target
    swap #1024 +  swap #1024 +          \ update addresses
    r> #1024 -                          \ update length
  repeat
  dup                                   \ if anything left
  if
    copy-cmnd>t >target                 \ send it to the target
  else
    drop 2drop
  endif
  buff free-img-buff
;
))

0 value -download-prompts?	\ -- flag ; SFP037
\ True if there are to be no download prompts.

: ?download-section     \ sect -- ; download if needed ; SFP031 SFP037
  dup cdata-type? if			\ download if CDATA
    -download-prompts? if
      download-section
    else
      cr ." Download " dup .pfa ." section to target? (y/n): " y/n?
      if  download-section  else  drop  endif
    endif
  else
    drop
  endif
;

: download-sections     \ -- ; download all required sections ; SFP031
  section-chain
  begin
    @ ?dup
   while
    dup ?download-section
    MS-link
  repeat
;

: download-cdata	\ --
\ download all CDATA sections quietly ; SFP036
  section-chain
  begin
    @ dup
   while
    dup cdata-type?
    if  dup download-section  endif
    MS-link
  repeat
  drop
;
previous definitions


\ ****
\ Done
\ ****

only forth  also c-c definitions


\ *****
\ Tests
\ *****
((
hex
0 7fff cdata section rom
8000 BFFF cdata bank rombank
  1 page romp1
  2 page romp2
  3 page romp3
C000 DFFF idata section ram
E000 FFFF udata bank rambank
  81 page ramp1  write-ignore
  82 page ramp2  write-ignore
  83 page ramp3  write-invalid

romp1 rom ramp1 ram
))


