\ Serial drivers for XTL (Xcompiler Transfer Link)

((
Copyright (c) MicroProcessor Engineering Ltd.
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 2380 631441
net: tech-support@mpeforth.com

History: Started 12 Jan 97

20110701 SFP005 Made command tail user configurable.
SFP004	20 Dec 1999	Added debug switches for char display
SFP003  15 Nov 1999	Fixed stack fault in SERIAL
 JL002  15 Dec 97       Modified SERIAL to accept target string
SFP001	26 Jan 97	Added LINK-FLUSH
))


\ **************
\ Initialisation
\ **************

only forth also c-c definitions


\ **********************
\ Debug control switches
\ **********************

variable <debug-serial>
  <debug-serial> off

also cc/i
: +serial-debug	cc/i	<debug-serial> on  ;
: -serial-debug	cc/i	<debug-serial> off  ;
: serial-debug?	cc/i	<debug-serial> @  ;
previous definitions


\ ********************
\ Serial driver basics
\ ********************

SerDev: XtlSerDev	\ -- sid
\ Serial I/O device for link.

#256 buffer: ser-name	\ -- addr
\ buffer for serial name, e.g. COM1
  s" COM1:" ser-name place

variable b-rate		\ -- addr
\ holds numeric baud rate, e.g. 9600
  #9600 b-rate !
variable ser-parity	\ -- addr
\ +G Holds parity as character, N, E etc
  char N ser-parity !
variable ser-#data	\ -- addr
\ +G Holds number of data bits.
  8 ser-#data !
variable ser-#stop	\ -- addr
\ +G Holds number of stop bits.
  1 ser-#stop !
#256 buffer: ser-extra$	\ -- addr
\ +G Counted string for extras such as DTR/RTS.
  0 ser-extra$ c!
#256 buffer: serString	\ -- addr
\ +G Assembly buffer for consiguration string.

[defined] Target_386_Windows 
[defined] Target_x64_Windows  or 
[if]
s"  dtr=on rts=on" ser-extra$ place

: GenSerString	\ --
\ +G Build the serial configuration string
  base @ >r  decimal serString >r
  ser-name count r@ place
  s"  baud=" r@ append  b-rate @ (u.) r@ append
  s"  parity=" r@ append  ser-parity @ r@ addChar
  s"  data=" r@ append  ser-#data @ (u.) r@ append
  s"  stop=" r@ append  ser-#stop @ (u.) r@ append
  s"  " r@ append
  ser-extra$ count r@ append
\  s"  dtr=on rts=on" r@ append
  r> drop  r> base !
;
[then]

[defined] Target_ARM_Linux
[defined] Target_386_Linux or
[defined] Target_x64_Linux or
[if]
: genSerParity	\ -- caddr len
  case  ser-parity @ upc
    [char] N of  s" no"  endof
    [char] E of  s" even"  endof
    [char] O of  s" odd"  endof
      s" whoops!" rot
  endcase
;

 s" 1 dtr 1 rts " ser-extra$ place

: GenSerString	\ --
\ +G Build the serial configuration string
  base @ >r decimal  serString >r
  serString #256 blank
  ser-name count r@ place  s"  " r@ append		\ "/dev/ttyS0 "
  b-rate @ (u.) r@ append  s"  baud " r@ append		\ "115200 baud "
  genSerParity r@ append  s"  parity " r@ append	\ "no parity"
  ser-#data @ (u.) r@ append  s"  data " r@ append	\ "8 data "
  ser-#stop @ (u.) r@ append  s"  stop " r@ append	\ "1 stop "
  s"  " r@ append
  ser-extra$ count r@ append
\  s" 1 dtr 1 rts " r@ append				\ " 1 dtr 1 rts "
  r> drop  r> base !
;
[then]

[defined] Target_386_OSX
[defined] Target_x64_Osx or
[if]
: genSerParity	\ -- caddr len
  case  ser-parity @ upc
    [char] N of  s" no"  endof
    [char] E of  s" even"  endof
    [char] O of  s" odd"  endof
      s" whoops!" rot
  endcase
;

: GenSerString	\ --
\ +G Build the serial configuration string
  base @ >r decimal  serString >r
  serString #256 blank
  ser-name count r@ place  s"  " r@ append		\ "/dev/ttyS0 "
  b-rate @ (u.) r@ append  s"  baud " r@ append		\ "115200 baud "
  genSerParity r@ append  s"  parity " r@ append	\ "no parity"
  ser-#data @ (u.) r@ append  s"  data " r@ append	\ "8 data "
  ser-#stop @ (u.) r@ append  s"  stop " r@ append	\ "1 stop "
  s" 1 dtr 1 rts " r@ append				\ "1 dtr 1 rts "
  r> drop  r> base !
;
[then]

: ser-query-byte	\ -- flag
\ Equivalent of KEY?
  XtlSerDev key?-gio
;

: ser-Wait-byte		\ -- b
  begin
    ser-query-byte 0=		\ From serial port
   while
    key? if
      case  key
        2 of  1 abort"  ^B - Serial break"  endof
        3 of  1 abort"  ^C - Serial break"  endof
      endcase
    endif
    1 ms
  repeat
  XtlSerDev key-gio
  <debug-serial> @
  if  dup ." [" .byte ." ]"  endif
;

\ variable serdelay  1000 serdelay !

: ser-Send-byte		\ b --
  <debug-serial> @ if
    dup XtlSerDev emit-gio	\ To serial port
    ." <" .byte ." >"
  else
    XtlSerDev emit-gio		\ To serial port
  endif
;

: ser-link-reset	\ -- ; reset target across link
  noop				\ dummy for now
;

: ser-link-flush	\ -- ; flush any pending input characters	SFP001
  begin
    ser-query-byte
  while
    ser-wait-byte drop
  repeat
;

: .ser-link-state	\ -- ; show serial link state
  ." Target link on " ser-name $.	\ COM port
  ."  at " b-rate @ u. ." baud"		\ show baud rate
;

: ser-link-close	\ -- ; close XTL link
  XtlSerDev close-gio drop
;

: ser-link-open		\ -- ; start comms as default
  base @ >r decimal
  ser-link-close			\ ensure closed down already
  genSerString				\ generate command string

  host-mode @(m) >r  host-mode on(m)	\ force correct INTERPRET
  extension_mode @ >r  extension_mode off
  serString count r/w XtlSerDev open-gio \ open may use INTERPRET
  r> extension_mode !
  r> host-mode !(m)

  abort" Can't open XTL link" drop
  <debug-serial> @ if
    cr ." Config string = " serString count type
    cr ." Serial handle = " XtlSerDev gen-handle @ .dword
  endif
  r> base !
;

: set-serial	\ -- ; set serial drivers
  assign ser-link-open to-do link-open
  assign ser-link-close to-do link-close
  assign ser-query-byte to-do query-byte
  assign ser-wait-byte to-do wait-byte
  assign ser-send-byte to-do send-byte
  assign ser-link-reset to-do link-reset
  assign ser-link-flush to-do link-flush
  assign .ser-link-state to-do .link-state
;

also cc/i		\ this code added to the cross interpreter
: serial  	cc/i	\ port$ baud -- ; e.g. c" COM1" #9600 SERIAL
  b-rate !				        \ serial speed
  ser-name over c@d(t) 1+ target>host           \ com port - text will be in TARGET space
  set-serial				        \ driver assignments
;

: ser-control	cc/i	\ caddr(t) --
  ser-extra$ over c@d(t) 1+ target>host
;
previous definitions

