\ MPE PowerForth/32 software floating point package

((
20191110 SFP004	Converted to use deferred ISNUMBER?
20140220 SFP003
SFP002	13 Jan 2000	Added conversion to/from 16 bit format
SFP001  9 Mar 1999      Changed FLOATS to REALS

Revision 3.1 January 1987
Revision 3.2 August 1993        Trig bug fixes in SINFN
                                F. bug fixed
                                Converted for 68xxx cross compiler v5.1
                                Converted to PF386

Quick and dirty version for 32-bit Forth-83
with 32-bit mantissa and 32-bit exponent

Copyright (C) 1983, 1984, 1985, 1986, 1993
MicroProcessor Engineering Ltd.
133 Hill Lane
Southampton SO1 5AF
England
tel: +44 703 631441
net: mpe@mpeforth.com


( Load screen for floating point package......................)

(     Welcome to MPE's new floating point package             )
(       For full details see your manual !!!!!!               )
(          1 -> 21 floating point primitives                  )
(         22 -> 23 fconstants & fvariables                    )
(         24 -> 27 floating point output                      )
(         28 -> 33 floating point input                       )
(         34 -> 49 floating point trig, logs, powers          )

))


\ vocabulary

DECIMAL

only forth definitions


\ f! f@ fdup

: f!    2!  ;                           \ f addr --
: f@    2@  ;                           \ addr -- f
: fdup  2dup  ;                         \ f -- f f


\ fover frot fpick froll fswap fdrop

decimal

: fover         2over  ;                \ f1 f2 -- f1 f2 f1

: frot          5 roll 5 roll  ;        \ f1 f2 f3 -- f2 f3 f1

: fpick         2* 1+ >r                \ calculate depth
                r@ pick  r> pick  ;     \ go do it

: froll         2* 1+ >r                \ calculate depth
                r@ roll  r> roll  ;     \ go do it

: fswap         2swap  ;                \ f1 f2 -- f2 f1

: fdrop         2drop  ;                \ f1 --



\ f, fvariable fconstant farray

: F,            \ f --
  HERE  8 ALLOT  F!  ;

: FVARIABLE     \ -- ; -- faddr
  CREATE  0 , 0 ,  DOES>  ;

: FCONSTANT     \ fn -- ; -- fn
  CREATE  F,  DOES>  F@  ;

: FARRAY        \ fn-1..f0 n -- ; compile / n -- fn ; execute
  CREATE  0 DO F, LOOP  DOES>  SWAP 8 * +  F@  ;

\ High level shift routines

: <<1		\ n -- n<<1
  1 lshift  ;

: >>1		\ n -- n>>1
  1 rshift  ;

: S->           \ n1 carry-in-flag --- n2 carry-out-flag
\ *G Perform a right shift, applying the carry in to the m.s. bit and
\ ** returning the carry out as 1 or 0.
  over >>1 swap                 \ Save original N1 and shift it right one bit.
  1 and                         \ Only use bottom bit of carry flag.
  if                            \ Should MSB (carry-in bit) be zero or one?
    $80000000 or                \ MSB must be a one.
  else
    $7FFFFFFF and               \ MSB must be a zero.
  endif
  swap $00000001 and            \ Retrieve old N1 and create carry-out flag.
;

: <-S		\ n1 carry-in-flag --- n2 carry-out-flag
\ *G Perform a left shift, applying the carry in to the l.s. bit and
\ ** returning the carry out as 1 or 0.
  over <<1 swap                 \ Save original N1 and shift it left one bit.
  if                            \ Should LSB (carry-in bit) be zero or one?
    $00000001 or                \ LSB must be a one.
  else
    $FFFFFFFE and               \ LSB must be a zero.
  endif
  swap $80000000 and 0<>        \ Retrieve old N1 and create carry-out flag.
;


((
\ 386 shift routines

prefix intel

code <-s        \ n1 carry-in-flag --- n2 carry-out-flag
  shr ebx, # 1			\ shift flag into carry
  rcl 0 [ebp], # 1		\ shift n1 to become n2
  rcl ebx, # 1			\ get carry back into TOS
  and ebx, # 1			\ mask bit 0
  next,
end-code

code s->        \ n1 carry-in-flag --- n2 carry-out-flag
  shr ebx, # 1			\ shift flag into carry
  rcr 0 [ebp], # 1		\ shift n1 to become n2
  rcl ebx, # 1			\ get carry back into TOS
  and ebx, # 1			\ mask bit 0
  next,
end-code

code <<1        \ n -- n<<1
  sal ebx, # 1   next,
end-code

code >>1        \ n -- n>>1
  shr ebx, # 1   next,
end-code
))

decimal

\ variables ...................................................

( -- addr )
VARIABLE EXP
VARIABLE SGN
VARIABLE *SN
VARIABLE /SN
VARIABLE +SN1
VARIABLE +SN2
VARIABLE EXP1


\ norm-r norm-l

: norm-r        \ mant -- mant'
  1 exp +!  >>1  ;

: norm-l        \ mant -- mant'
  -1 exp +!  <<1  ;


\ dnorm - normalise d-word

: DNORM         \ n exp -- fn
  EXP !  dup                    \ check zero
  IF  DUP >r  abs               \ save sign
      BEGIN  norm-l             \ shift until -ve
             DUP
      0< UNTIL
      norm-r
      r> ?negate                \ restore sign
      EXP @                     \ recover exponent
  ELSE  0
  ENDIF  ;


\ s>f fsign

: s>f           \ n -- fn
  31 DNORM  ;

: FSIGN         \ fn -- fn flag ; true if negative
  swap dup abs -rot 0<  ;


\ fnegate fabs .....................................nms 19/04/88

: fnegate       ( fn -- -fn )
  >R NEGATE R>  ;

: ?fnegate      ( fn n -- fn|-fn )
  0< if fnegate  endif  ;

: fabs          ( fn -- |fn| )
  >R ABS R>  ;


\ f* f/

: f*            ( f1 f2 -- f3 )
  FSIGN *SN !  1+ EXP !  >r       ( save sign & exp 1 )
  FSIGN *SN +!  EXP +!  r>        ( save sign & exp 2 )
  um* 2dup or                     ( 64 bit multply = 0 ?)
  IF  BEGIN
        DUP  [ hex ] 40000000 [ decimal ] and 0=
        WHILE  -1 EXP +!  2dup d+
      REPEAT
      swap 0< if  1+  endif     \ round up on low half
      DUP 0<                    \ check for overflow
      IF  norm-r  ENDIF
      *SN @ -1 =                \ see if product -ve
      IF  NEGATE  ENDIF       \ make it -ve
      EXP @                     \ recover exponent
  ELSE  2DROP  0 0
  ENDIF  ;

: f/            ( f1 f2 -- f3 )
  FSIGN /SN !  NEGATE EXP !         \ Negate divisor & save
  DUP 0= ABORT" Divide by 0 error"  \ Check mantissa
  >r  FSIGN /SN +!  EXP +!          \ make +ve
  0 swap  dup r@ u< invert          \ dividend double prec.
  if  1 exp +!
      0 s->
      if  >r  1 s-> drop  r> endif
  endif
  r@ um/mod                         \ do division
  swap r> 2/ u>  if  1+  endif      \ if rem>div/2 round up
  dup 0<                            \ check for overflow
  if  norm-r  endif                 \  and correct
  EXP @ 1- DNORM                    \ normalise
  /SN @ -1 =                        \ check sign
  IF  FNEGATE  ENDIF  ;


\ f+ f-

: F+            ( f1 f2 -- f3 )
  over                      \ f1 non-zero ?
  IF  FSIGN +SN1 !  EXP !  >r   over
      IF  FSIGN +SN2 !  EXP1 !  EXP @ EXP1 @ - DUP
          IF  DUP 0>
              IF  rshift
              ELSE  r>  swap ABS
                    rshift
                    >r
              ENDIF
          ELSE  DROP
          ENDIF
          EXP @ EXP1 @ MAX EXP !  r> +sn1 @ +sn2 @ -
          IF  NEGATE +
          ELSE  +  1 EXP +!  0 S-> +  DUP 0<
                IF  norm-r  ENDIF
          ENDIF
          +SN2 @
          IF  NEGATE  ENDIF
          EXP @ DNORM
      ELSE  FDROP  r> +SN1 @
            IF  NEGATE  ENDIF
            EXP @
      ENDIF
  ELSE  FDROP
  ENDIF   ;

: F-            \ f1 f2 -- f3
  fnegate f+  ;


\ sint fint

: SINT          ( fn -- n )
  FSIGN SGN !  DUP 0<
  IF  FDROP 0
  ELSE  31 swap - dup 0>
        IF  rshift
        ELSE  1 abort" Number too large in SINT"
        ENDIF
        SGN @
        IF  NEGATE  ENDIF
  ENDIF  ;

: fint          ( f1 -- f2 )
  DUP 32 <
  IF  SINT s>f  ENDIF  ;


\ f-compares

( f1 -- flag )
: F<0   DROP 0<  ;
: F>0   DROP 0>  ;
: F=0   DROP 0=  ;
: f<>0  drop 0<>  ;


( f1 f2 -- flag )
: F=    F- F=0  ;
: F<    F- F<0  ;
: F>    F- F>0  ;


\ fmax fmin fliteral fseparate ffrac

: fmax          \ f1 f2 -- f1|f2
  FOVER FOVER F<
  IF  FSWAP  ENDIF
  FDROP  ;

: fmin          \ f1 f2 -- f1|f2
  FOVER FOVER F>
  IF  FSWAP  ENDIF
  FDROP  ;

: fliteral      \ fn --
  swap  postpone literal  postpone literal  ;  immediate

: fseparate     \ f1 f2 -- f3 f4
  FOVER FOVER F/
  FROT FROT FABS
  FSWAP FSIGN >R FSWAP
  2 fpick FABS FINT F* F-
  R>
  IF  FNEGATE  ENDIF
  FSWAP  ;

: ffrac         \ f1 f2 -- f3
  FSEPARATE FDROP  ;


\ fconstants .................................................

( -- fn )
1 s>f 10 s>f f/ fconstant %.1
1 s>f fconstant %1
10 s>f fconstant %10
1250000000 34 fconstant %10^10
1844674407 -33 fconstant %10^-10


\ fvariables & variables .......................................

( -- addr )
VARIABLE %EXP
VARIABLE %SGN
VARIABLE %FLG
VARIABLE %N
FVARIABLE %STK1

\ round ?ferror

( rounds least significant eight bits to 0 if higher 2 bits )
( are all 0s or all 1s                                      )
( f1 -- f2 )
: ROUND FSIGN >R >R
        DUP 224 AND 0=
        IF  -256 AND  ENDIF
        DUP 224 AND  224 =
        IF  -256 AND 256 +  endif
        R> OVER 0<
        if  SWAP 2/  SWAP 1+  endif
        R>
        IF  FNEGATE  ENDIF  ;


\ places ?10pwr oprep .exp e. f.

variable places  8 places !     \ number of digits after d.p.

: ?10pwr        \ exp(2) -- exp(2) exp(10)
  dup 0< >r
  dup abs 301 1000 */mod nip            \ l10(2)~=.3010
  r> if  negate 1- endif ;

: oprep         \ fn -- n
  FSIGN %SGN !
  ?10PWR  DUP %EXP !  8 SWAP -  ?DUP
  IF    DUP 0<
        IF  ABS 0
            DO  %.1 F*  LOOP
        ELSE  0
              DO  %10 F*  LOOP
        ENDIF
  ENDIF
  ROUND  SINT  ;

: .exp          \ --
  ( if -10<e<10 then print a "0" in front of exponent  )
  [char] e EMIT   %EXP @  DUP  0<
  IF  [char] -  ELSE  [char] +  ENDIF
  EMIT
  ABS  0 <# # #S #> TYPE  ;

: n#            \ n -- ; convert n digits
  0 ?do # loop  ;

: E.            \ n exp --
  OPREP  s>d
  <#  8 n#  [char] . HOLD  #  %sgn @
      IF  [char] -  ELSE  BL  ENDIF
      HOLD
  #>
  8 places @ min 0 max + 8 - TYPE
  .EXP  ;

: f.            \ fn --
  DUP -10 27 WITHIN?
  IF  OPREP  s>d
      <#  8 %EXP @ - 0 max dup >r ?dup
          IF  n#  [char] . HOLD  ENDIF
          #S  %SGN @
          IF  [char] -  ELSE  BL  ENDIF
          HOLD
      #>
      r> dup
      if  places @ -  0 max -
          DUP 0
          DO  2DUP + 1- C@  [char] 0 <> ?leave
              1-
          LOOP
      else  drop
      endif
      2DUP +  1- C@  [char] . =
      IF  1-  ENDIF
      TYPE
  ELSE  E.
  ENDIF  ;


\ fcheck mnum enum

: FCHECK        \ -- am lm ae le flag ; true if f.p. number
  pad count 2dup [char] E scan dup >r    \ ap lp ae le --
  tuck >r >r - r> r>                    \ am lm ae le --
  dup if 1 /string endif                \ am lm ae' le' --
  pad count [char] . scan  dup >r
  if  [char] , swap c!  else  drop  endif
  r> r> or  ;

: MNUM          ( -- d 2 | 0 ; uses fchecked str in pad   )
  (INTEGER?) DUP 1 =
  IF  DROP S>D 2  ENDIF  ;

: ENUM          ( -- n 1 | 0 ; str as above  )
  DUP
  IF  (INTEGER?)  DUP 2 =
      IF  2DROP DROP 0  ENDIF
  ELSE  2DROP  0 1
  ENDIF  ;


\ fixexp ............................................. July 86

: FIXEXP     ( dmant exp -- f )
  >R                                    \ preserve single exponent
  drop s>f                              \ convert double mantissa to float
  R>  10 /MOD  SWAP >R  ?DUP            \ deal with |exponent|>10
  IF  DUP 0<
      IF  ABS 0
          DO  %10^-10 F*  LOOP
      ELSE  0
            DO  %10^10 F*  LOOP
      ENDIF
  ENDIF
  R> ?DUP                               \ finish with exponent
  IF  DUP 0<
      IF  ABS 0
          DO  %.1 F*  LOOP
      ELSE  0
            DO  %10 F*  LOOP
      ENDIF
  ENDIF  ;


\ fnumber? ........................................... July 86

: sfp32>16	\ m e -- dm e ; convert from 32 bit format to 16 bit format
  >r  $10000 /mod  r>
;

: sfp16>32	\ dm e -- n e
  >r  #16 lshift  swap $FFFF and  or  r>
;


: ?sfp32>16	\ m e -- m e | dm e
  [ also c-c ]
  16bit?  convert-doubles? @ and	\ 16 bit target with conversion enabled?
  [ previous ]
  if  sfp32>16  endif
;

: ?sfp16>32	\ dm e -- m e | dm e
  [ also c-c ]
  16bit?  convert-doubles? @ and	\ 16 bit target with conversion enabled?
  [ previous ]
  if  sfp16>32  endif
;

: 2?sfp32>16	\ f1 f2 -- f1' f2' ; *** has carnal knowledge ***
  2>r  ?sfp32>16  2r> ?sfp32>16
;

: 2?sfp16>32	\ f1 f2 -- f1' f2' ; *** has carnal knowledge ***
  ?sfp16>32 2>r  ?sfp16>32  2r>
;

: (isFNUMBER?)	( caddr len -- 0/.../mant exp 3 )
  {: | temp[ 256 ] -- 0/.../mant exp 3 :}
  temp[ place
  temp[ COUNT UPPER  FCHECK
  IF  ENUM
      IF  -ROT MNUM
          IF  ROT DPL @  DUP 0>
              IF  -  ELSE  DROP  ENDIF
              FIXEXP
              [ also c-c ]
              16bit?                    \ 16 bit target?
              convert-doubles? @ and    \ with conversion enabled?
              [ previous ]
              if                        \ has 32 bit mantissa
                sfp32>16 3		\ and takes 3 cells
              else
                2
              endif
          ELSE  DROP 0
          ENDIF
      ELSE  2DROP 0
      ENDIF
      DUP #L !  -1 dpl !
  ELSE  2DROP 2DROP  0
  ENDIF
;

: isFnumber?	( caddr len -- 0 | n 1 | d/f32 2 | f16 3 )
  base @ >r                             \ preserve base
  2dup 2>r  pad place			\ preserve addr/len, copy string to pad
  pad count isInteger? ?dup if		\ try and convert to int
    2r> 2drop				\ discard preserved addr/len
  else
    decimal  2r> (isFnumber?)		\ try as a float instead
  endif
  r> base !                             \ restore base
;

: Fnumber?	( caddr -- 0 | n 1 | d/f32 2 | f16 3 )
  count isFnumber?
;

((
: (FNUMBER?)      ( addr -- 0/.../mant exp 3 )
  PAD OVER C@ 1+ CMOVE   ( copy string to pad )
  PAD COUNT UPPER  FCHECK
  IF  ENUM
      IF  -ROT MNUM
          IF  ROT DPL @  DUP 0>
              IF  -  ELSE  DROP  ENDIF
              FIXEXP
              [ also c-c ]
              16bit?                    \ 16 bit target?
              convert-doubles? @ and    \ with conversion enabled?
              [ previous ]
              if                        \ has 32 bit mantissa
                sfp32>16 3		\ and takes 3 cells
              else
                2
              endif
          ELSE  DROP 0
          ENDIF
      ELSE  2DROP 0
      ENDIF
      DUP #L !  -1 dpl !
  ELSE  2DROP 2DROP  0
  ENDIF
;

: fnumber?      ( addr -- 0 | n 1 | d/f32 2 | f16 3 )
  base @ >r                             \ preserve base
  >r                                    \ preserve addr
  r@ PAD OVER C@ 1+ CMOVE               \ copy string to pad
  pad integer? ?dup                     \ try and convert to int
  if
    r> drop                             \ discard preserved addr
  else
    decimal r> (fnumber?)               \ try as a float instead
  endif
  r> base !                             \ restore base
;
))

\ floats integers

also forth definitions

: reals         \ -- ; allow f.p input
  assign isFnumber? to-do isNumber?  ;      \ switch fnumber?

: integers      \ -- ; no f.p input
  assign isInteger? to-do isNumber?  ;      \ switch in integer?

previous definitions


\ (f#) f#-in f# ....................................... July 86

: (F#)     ( addr -- fn 3 )
  FNUMBER?
  case  0  OF  0  ENDOF
        1  OF  S>F 2  ENDOF
        2  of  dpl @ -1 <>                ( see if double)
               if  drop s>f  endif        ( covert to fp)
               2
           endof
        3  of  3  endof
               1 abort" (F#) can't cope with this format"
  ENDCASE  ;

: F#IN     ( -- fn 2 | 0 )
  QUERY BL WORD (F#)  ;

: F#       ( -- f or compiles it [ state smart ] )
  BL WORD (F#)  0= #-13 ?throw  STATE @
  IF  postpone FLITERAL  ENDIF  ;  IMMEDIATE

\ floating point constants
1518500249 0 fconstant %sqr.5
1267358949 0 fconstant %s1
1792332800 -1 fconstant %s2
1686629716 2 fconstant pi
1686629715 1 fconstant pi/2
1686629716 0 fconstant pi/4
1367130550 -1 fconstant 1/pi
180 s>f fconstant %180
1686626304 18 fconstant %sinmax
1488522234 0 fconstant %d8
1865280597 -1 fconstant %d9
1677721600 8 fconstant %alnmax
1549082004 1 fconstant %1/ln2
1488522236 0 fconstant %ln2
1660944384 7 fconstant %algmax
1783446570 2 fconstant %1/lg2
1292913983 -1 fconstant %lg2
1367130550 0 fconstant 2/pi

\ FSQR by Heron's Formula .....................................

: fsqr          ( f1 -- f2 )
  fdup f<>0                               ( check 0 )
  IF  OVER 0<                           ( check -ve )
      ABORT" Square root of negative no.!"
      >R 0                              ( .5 <=f<=1 )
      FDUP %S1 F*  %S2 F+               ( HART est Y0 )
      FOVER FOVER F/ F+
      FSWAP FOVER F/ FSWAP 2-  F+
      R@ 1 AND  DUP R> + >R
      IF  %SQR.5 F*  ENDIF
      R> 2/ +
  ENDIF  ;


\ [sin] .......................................................

\ n -- fn
F# -1.6666666E-1                F# 8.3333315E-3
F# -1.98409E-4                  F# 2.7526E-6
F# -2.39E-8
5 FARRAY SINARRAY

\ f1 -- f2
: (SIN)         FDUP FDUP F* FDUP 0 SINARRAY F*
                4 1  DO
                  I SINARRAY F+ FOVER F*
                LOOP
                4 SINARRAY F+ F* %1 F+ F*  ;


\ sine functions ..................................nms 19/04/88

( -- )
: ?TR-ERR       ABORT" Overflow in trig. function"  ;

( f1 -- f2 )
: SINFN         FDUP 1/PI F*            ( ang ang/pi --- )
                FINT FDUP SINT %N !     ( ang int[ang/pi] --- )
                PI F* F-                ( ang-pi*int[ang/pi] - )
                FDUP PI/2 F>
                  IF  PI fswap F- ENDIF
                (SIN)
                %N @ 1 AND
                IF  FNEGATE  ENDIF  ;

\ sine cosine .....................................nms 19/04/88

\ n1 -- n2
: DEG>RAD       %180 F/ PI F*  ;

\ n1 -- n2
: RAD>DEG       %180 F* PI F/  ;

: (FSIN)        \ f1 -- f2
        FSIGN  >r
        FDUP %SINMAX F> ?TR-ERR
        SINFN
        r> ?fnegate  ;

( f1 -- f2 )
: FSIN  DEG>RAD (FSIN)  ;

( f1 f2 )
: FCOS  DEG>RAD  PI/2 F+  (FSIN)  ;


\ (tan) tan

\ n -- fn
F# 3.3333140E-1                 F# 1.3339240E-1
F# 5.3374060E-2                 F# 2.4565089E-2
F# 2.9005250E-3                 F# 9.5168091E-3
6 FARRAY TANARRAY

\ f1 -- f2
: (TAN)         FDUP FDUP F* FDUP 0 TANARRAY F*
                5 1  DO
                  I TANARRAY F+ FOVER F*
                LOOP 5 TANARRAY F+ F* %1 F+ F*  ;

\ f1 -- f2
: FTAN          DEG>RAD FSIGN %SGN !
                FDUP %SINMAX F> ?TR-ERR
                FDUP 2/PI F* FINT FSWAP FOVER PI/2 F* F-
                FDUP PI/4 F>
                  IF  PI/2 F- (TAN) FSWAP %1 F+
                  ELSE  (TAN) FSWAP
                  ENDIF
                SINT 1 AND
                  IF  %1 FSWAP F/ FNEGATE  ENDIF
                %SGN @  IF  FNEGATE  ENDIF  ;

\ arcfn
\ n -- fn
F# 1.5707963E0                  F# -2.1459880E-1
F# 8.8978987E-2                 F# -5.0174305E-2
F# 3.0891881E-2                 F# -1.7088126E-2
F# 6.6700901E-3                 F# -1.2624911E-3
8 FARRAY ASINARRAY

( f1 -- f2 )
: ARCFN         FDUP 0 ASINARRAY F*
                7 1  DO
                  I ASINARRAY F+ FOVER F*
                LOOP
                7 ASINARRAY F+
                FSWAP %1 FSWAP F- FSQR F*  ;


\ atfn

\ n -- fn
F# -3.3333145E-1                F# 1.9993551E-1
F# -1.4208899E-1                F# 1.0656264E-1
F# -7.5289640E-2                F# 4.2909614E-2
F# -1.6165737E-2                F# 2.8662257E-3
8 FARRAY ATANARRAY

( f1 -- f2 )
: ATFN          FDUP FDUP F* FDUP 0 ATANARRAY F*
                7 1  DO
                  I ATANARRAY F+ FOVER F*
                LOOP
                7 ATANARRAY F+ F* %1 F+ F*  ;


\ fasin facos fatan ...........................................

\ f1 -- f2
: FASIN FSIGN %SGN !  ARCFN PI/2 FSWAP F-
        %SGN @
        IF  FNEGATE  ENDIF
        RAD>DEG  ;

\ f1 -- f2
: FACOS FSIGN %SGN !  ARCFN
        %SGN @
        IF  PI FSWAP F-  ENDIF
        RAD>DEG  ;

\ f1 -- f2
: FATAN FSIGN %SGN !  FDUP %1 F>
        IF  %1 FSWAP F/  ATFN PI/2 FSWAP F-
        ELSE  ATFN
        ENDIF
        %SGN @
        IF  FNEGATE  ENDIF
        RAD>DEG  ;


\ ln log
F# 9.9999642E-1                 F# -4.9987412E-1
F# 3.3179903E-1                 F# -2.4073381E-1
F# 1.6765407E-1                 F# -9.5329390E-2
F# 3.6088494E-2                 F# -6.4535442E-3
8 FARRAY LOGARRAY       \ n -- fn

\ f1 -- f2
: FLN   fdup f>0 invert
        ABORT" Invalid argument to LOG"
        1 SWAP 1- %N !  %1 F-
        FDUP 0 LOGARRAY F*
        7 1  DO
          I LOGARRAY F+ FOVER F*
        LOOP
        7 LOGARRAY F+ F* %N @ S>F  %D8 F* F+  ;

: FLOG  FLN %D9 F*  ;


\ (10^x)
\ n -- fn
F# 1.1512928E0                  F# 6.6273088E-1
F# 2.5439357E-1                 F# 7.2951737E-2
F# 1.7421120E-2                 F# 2.5549180E-3
F# 9.3264267E-4
7 FARRAY 10ARRAY

\ f1 -- f2
: (10^X)        FDUP 0 10ARRAY F*
                6 1  DO
                  I 10ARRAY F+ FOVER F*
                LOOP
                6 10ARRAY F+ F*
                %1 F+  FDUP F*  ;


\ (e^x)

\ n -- fn
F# -1.0000000E0                 F# 4.9999992E-1
F# -1.6666530E-1                F# 4.1657348E-2
F# -8.3013598E-3                F# 1.3298820E-3
F# -1.4131610E-4
7 FARRAY E-ARRAY

\ f1 -- f2
: (E^X)         FDUP 0 E-ARRAY F*
                6 1  DO
                  I E-ARRAY F+ FOVER F*
                LOOP
                6 E-ARRAY F+ F*
                %1 F+ %1 FSWAP  F/  ;


\ fe^x f10^x

\ f1 -- f2
: FE^X  FSIGN %SGN !  FDUP
        %ALNMAX F> ABORT" Overflow in FE^X"
        FDUP  %1/LN2 F*  FDUP SINT %N !  FINT %LN2 F* F-
        (E^X)  %N @ +  %SGN @  IF  %1 FSWAP F/  ENDIF  ;

\ f1 -- f2
: F10^X FSIGN %SGN !  FDUP
        %ALGMAX F> ABORT" Overflow in F10^X"
        FDUP %1/LG2 F*  FDUP SINT %N !  FINT %LG2 F* F-
        (10^X)  %N @ + %SGN @  IF  %1 FSWAP F/  ENDIF  ;


\ fx^n

( x-real n-integer --- fn )
: FX^N  DUP
          IF  DUP 0< %FLG !  ABS >R  %1 FSWAP  R>
              BEGIN  DUP 0>
                WHILE  DUP 2 / DUP 2* ROT -
                       IF  >R FSWAP FOVER F* FSWAP R>  ENDIF
                       >R FDUP F* R>
              REPEAT
              DROP FDROP  %FLG @
              IF  %1 FSWAP F/  ENDIF
          ELSE  DROP FDROP  %1
          ENDIF  ;



\ fx^y

( x-real y-real --- fn )
: FX^Y  fdup f<>0
          IF  FDUP FDUP FINT F- %STK1 F!
              SINT >R FDUP R>  FX^N FSWAP
              %STK1 F@ fdup f<>0
              IF  fover f<0 ABORT" Result of FX^Y is complex"
                  FSWAP fdup f<0 %SGN !  fabs
                  FLOG F* F10^X %SGN @
                  IF  %1 FSWAP F/  ENDIF
                  F*
              ELSE  FDROP FDROP
              ENDIF
          ELSE  FDROP FDROP  %1
          ENDIF  ;


\ =========================
\ IEEE format conversion
\ =========================

: FP>IEEE	\ fp -- ieee32
\ *G Convert native FP value to IEEE 32 bit format.
  over if				\ Don't convert 0
    #126 + #23 LSHIFT			\ IEEE biased 127 FP has 1+
    OVER 0<
    IF  $80000000 OR  THEN		\ FP signed mantissa, set IEEE sign bit
    SWAP ABS
    $3FFFFFFF AND 7 RSHIFT OR		\ clear msb IEEE hidden bit
  else
    drop				\ return IEEE 0e0
  endif
;

: IEEE>FP	\ ieee32 -- fp
\ *G Convert IEEE 32 bit float to native format.
  DUP IF				\ Don't convert 0
    DUP					\ save a copy to get the sign
    8 LSHIFT 1 RSHIFT $40000000 OR
    OVER ?NEGATE
    SWAP
    23 RSHIFT $FF AND #126 - 		\ shifted for FP exponent
  ELSE
    DUP					\ was already 0
  THEN
;

( floating point goodbye!!............................July 86 )
((
decimal

;p

That's all, folks.
Our thanks go to all those who have contributed to this package
whose efforts have not been in vain. Any bugs are still our
fault, and if you find any please notify us.
))

