\ SymbolTools.fth - tools using the symbol table

((
Copyright (c) 2022
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! filename
\ *T Chapter Title
\ ===========

((
: Words(s)      \ -- ; display words in current symbol voc.
  cr ." Target vocabulary: "                                                    \ SFP018
  context(s) @(m) .voc(s)                                                       \ SFP018
  cr ." Trailing asterisk indicates unresolved word"                            \ SFP018
  c/l out !                             \ set up
  Context(s) @(m) #Threads(s)+          \ #threads, thr0..thrn-1
  dup cell+  nfa-buff                   \ source, dest
  rot @(s) cell* cmove(s)>              \ count/length and copy
  Begin
    max-nfa(s) dup                      \ while still one left
    halt? 0= and                        \ and no stop command
   While
    <show-unresolved> @ if
      dup .word-name(s)			\ show name
      dup name>(s) resolved? 0=		\ say if unresolved
      if ."  * "  endif
    else
      dup name>(s) resolved?
      if  dup .word-name(s)  endif
    endif
    cell- @(s) swap !                   \ find next NFA in thread
  Repeat
  2drop
;
))

only forth also c-c definitions

0 value DestAddr(t)	\ -- addr
\ Address whose name we want to find.

: InSymVoc(s)	\ voc(s) xt --
\ Perform xt on all words in symbol vocabulary. Action of XT
\ must be stack neutral ( nfa(s) -- nfa(s) ).
  >r					\ preserve XT
  context(s) @(m)  swap is-voc(s)       \ set new context
  c/l out !                             \ set up
  Context(s) @(m) #Threads(s)+          \ #threads, thr0..thrn-1
  dup cell+  nfa-buff                   \ source, dest
  rot @(s) cell* cmove(s)>              \ count/length and copy
  Begin
    max-nfa(s) dup                      \ while still one left
    halt? 0= and                        \ and no stop command
   While
    r@ execute
    cell- @(s) swap !                   \ find next NFA in thread
  Repeat
  2drop
  is-voc(s)       			\ restore symbol context
  r> drop				\ discard XT
;

: checkLabel	\ nfa(s) -- nfa(s)
\ Executed once for each word in the symbol vocabulary.
  dup name>(s) resolved? if
    dup name>(s) @(s) DestAddr(t) = if
      dup .name(s) space
    then
  then
;

: showLabel	\ addr(t) --
\ Display label name, if label is at the given target address.
  to DestAddr(t)
  'labels(s) @(m)  ['] checkLabel InSymVoc(s)
;

: checkLabelLine	\ nfa(s) -- nfa(s)
\ Executed once for each word in the symbol vocabulary.
  dup name>(s) resolved? if
    dup name>(s) @(s) DestAddr(t) = if
      ." L: " dup .name(s) 3 spaces
    then
  then
;

: showLabelLine	\ addr(t) --
\ Display label name, if label is at given address.
  to DestAddr(t)
  'labels(s) @(m)  ['] checkLabelLine InSymVoc(s)
;


also cc/i

: ShowLabel	\ addr(t) --
\ Display label name, if label is at the given target address.
  showLabel
;

c-c only Forth also c-c definitions


\ ======
\ *> ###
\ ======

decimal



