((
	TEMPFILE.FTH
	Create and delete unique/temporary file names.

	Copyright (c) 1997
	MicroProcessor Engineering
	133 Hill Lane
	Southampton SO15 5AF
	England

	tel: (+44) 2380 631441
	email: mpe@mpeforth.com

	Changes
	-------
20230106 SFP001 removed search order change.
))


((
-------------------------------------------------------------------------------

	GET-UNIQUE-FILENAME  (  extn .. ext1 n prefix   --  c-addr flag  )
		- Given an address of a 3 character string prefix and a set of
		  file extensions this function will return a unique filename
		  which will be in the range 'xxx00000' to 'xxx99999'.

		- If no unique name could be generated then the function
		  will return FALSE and c-addr does not contain a vaild
		  filename.  If the function succeeded then TRUE will be
		  returned and c-addr will contain a valid filename.

		- The extensions and prefix strings must all be counted strings.

	DELETE-UNIQUE-FILENAME  (  extn .. ext1 n filename  --  flag  )
		- Given a base filename and a set of extensions this function
		  will delete all files of the form 'filename.ext' for every
		  extension.

		- If all the files were deleted successfully then TRUE will be
		  returned, otherwise FALSE will be returned.

		- All extensions and filename strings should be counted
		  strings.

-------------------------------------------------------------------------------
))


decimal

32 constant /unique-filename			\ maximum length of filename

: FILENAME-UNIQUE?  { filename | buffer[ /unique-filename ] -- flag }
  filename buffer[ $move                        \ make local copy of string
  buffer[ $>asciiz				\ need zero terminated string for file operations
  open-path 0=					\ attempt to open the file
  if						\ if it opened then it exists so it is not unique
    close-path					\   so close the file
    if                                    	\     cope if there is an error closing...
      cr ." Couldn't close temp file name"
      ABORT					\     by giving up
    endif
    FALSE
  else
    TRUE					\ file didn't open so assume unique
  endif						\  n.b may be we should explicitly check the failure code
;

: NUM>$  {  c-addr n  |  addr  len  --  }
  n s>d <# # # # # # #> -> len -> addr		\ convert to string, returned as addr, len
  addr c-addr 1+ len cmove			\ return as a counted string in c-addr
  len c-addr c!
;

: FILENAME+EXT-UNIQUE?  (  extn .. ext1 ) {  n filename  |  buffer[ /unique-filename ] unique-flag  --  flag  }
  TRUE -> unique-flag				\ assume filename unique
  n 0
  ?Do						\ for all extensions
    unique-flag                                 \   but don't bother trying if we already know file exists
    If
      filename buffer[ $move  buffer[ $+	\     concatenate filename + ext
      buffer[ filename-unique? 0=		\     does the file exist?
      If					\
        FALSE -> unique-flag                    \       then it's not unique
      Endif
    Else
      drop
    Endif
  Loop
  unique-flag					\ is the file unique..
;


create UNIQUE-FILENAME$
	/unique-filename allot

: MAKE-BASE-NAME  {  prefix$ num buffer$  |  num$[ 6 ]  --  }
  prefix$ buffer$ $move			\   start with the prefix
  num$[ num num>$			\   build the sequential number $
  num$[ buffer$ $+			\   concatenate
;

: GET-UNIQUE-FILENAME	\ extn..ext1 n prefix -- c-addr flag
  { n prefix | flag buffer$[ /unique-filename ] -- c-addr flag }
  FALSE -> flag				\ assume not unique
  10000 0				\ try file names in range xxx00000 to xxx99999
  Do
      prefix i buffer$[	make-base-name	\   buld base filename
      n 0 ?Do  i pick  Loop		\   make copies of extensions
      n buffer$[ filename+ext-unique?	\   check to see whether all extension forms unique
      buffer$[ filename-unique?		\   and just the filename unique
      and				\
      If                                \   if the filename is unique
        TRUE -> flag			\     flag filename is unique
        prefix i unique-filename$	\     re-build base filename
        make-base-name
        leave				\     get out of loop
      Endif
  Loop
  n 0 ?Do  drop  Loop			\ throw away extensions
  UNIQUE-FILENAME$ flag			\ return address and flag
;

: DELETE-UNIQUE-FILENAME	\ extn..ext1 n filename -- flag
  { n filename | flag buffer[ /unique-filename ] -- flag }
  TRUE -> flag				\ assume all files can be deleted
  n 0					\ for all the extensions
  ?Do
    filename buffer[ $move		\   make copy of filename
    buffer[ $+				\   add next extension to filename
    buffer[ delete-file 0=		\   delete the file
    If					\   if the file couldn't be deleted
      FALSE -> flag			\     flag as failure
    Endif
  Loop
  flag					\ return flag
;



