\ Local variables for MPE cross compilers

only forth  also c-c definitions  decimal

((

file: TLOCALS.FTH

Copyright (c) 1994, 1995, 1997, 1998, 1999, 2000, 2011, 2015, 2020
MicroProcessor Engineering Ltd
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com

N.B. { and LOCALS| are not IMMEDIATE at the moment in the COMPILER voc.
If they are changed in any way, check in XC_XTEND.FTH

20200320 SFP012 Saved transient area in a global
20150531 SFP011 Added Forth-2012 {:
20111012 SFP010 Forced target local variable data to be initialised
		at cross compiler start up.
May 2000	Added code to cope with CPUs in which the locals stack
		grows UPWARDS, e.g. 8051 and TMS320C3x/4x. In this case
		the offset is compiled as -ve by the XOPTxxx code, and
		the offset used is that BEFORE modification.
Oct 1999	Moved some parts to XOPTxxx/XGENxxx
July 1998	Converted to the XOPTxxx family of code generators
Nov 97          Added extra capability allowing target objects to be used in
                the construction of local array sizes.
Jan 97          Converted to cross compiler target use
                N.B. MAKELVS takes in line data defining the number of CELLS
Sep 1996        Converted to use MAKEFRAME technique from SENDIT
                ANS LOCALS| Jules Jim ... | added
Jan 1996        LOCAL-BASE changed to offset 0A4h to avoid clash with DONE-FLG
                System words moved to SYSTEM vocabulary
Sept 1995       Locals moved to the return stack
                N.B. This code is sensitive to the USER variable structure
                local arrays added
May 1994        Original release

This file implements the following syntax for named inputs and local
variables. The sequence
  { ni1 ni2 ... | lv1 lv2 ... -- o1 o2 }
defines named inputs, local variables, and outputs. The named inputs are
automatically copied from the data stack on entry. Named inputs and local
variables can be referenced by name within the word during compilation.
The output names are dummies to allow a complete stack comment to be
generated.

The items between { and | are named inputs.
The items between | and -- are local variables.
The items between -- and } are outputs.

Named inputs and locals return their values when referenced, and must
be preceded by -> to perform a store, or by ADDR to return the address.

Arrays may be defined in the form:
  arr[ n ]
Any name ending in the '[' character will be treated as an array, the
expression up to the terminating ']' will be interpreted. Arrays only
return their base address, all operators are ignored.

In the example below, a and b are named inputs, a+b and a*b are local
variables, and arr[ is a 10 byte array.

: foo           { a b | a+b a*b arr[ 10 ] -- }
  a b + -> a+b
  a b * -> a*b
  cr a+b .  a*b .
;

See the ANS specification for details of LOCALS| ... | and its usage.
))

\ `TO' variable implementation : compile time

((
variable OperatorType(t)
\ used at compile time for prefix operator
\ 0 = default
\ 1 = store
\ 2 = addr
))

: bad-method    \ --
  xcerr_InvOp throw
;

: process-lvar  \ offset -- [compiling] ; ?? offset -- ?? [interpreting]
  case  OperatorType(t) @off
    0  of  c_lv@  endof
    1  of  c_lv!  endof
    2  of  c_lv  endof
           bad-method
  endcase
;

: process-larray        \ offset -- [compiling] ; ?? offset -- ?? [interpreting]
  OperatorType(t) @off
  if  bad-method  endif
  c_lv     				\ arrays only return address
;

: $lvar         \ offset ^string -- ; -- ; build local variable
  also compiler definitions
  $create
\    immediate  ,                       \ builds word in transient area
    ,                                   \ builds word in transient area
    previous definitions
  does>         \ -- pfa
    dup body> >name                     \ point to word name
    dup c@ $01F and                     \ extract length
    + c@ [char] [ =                     \ see if last char = '['
    if
      @ process-larray
    else
      @ process-lvar
    endif
;


\ syntax handling for { ... }

variable local-offset  local-offset off		\ moved to XGEN/XOPTxxx
variable patch-address  patch-address off	\ moved to XGEN/XOPTxxx
variable OldLast

variable init-t-dp			\ start of transient area
variable t-dp-size  $02000 t-dp-size !	\ Initial size of transient dictionary.
variable t-dp  t-dp off			\ transient dictionary pointer
2variable old-transient			\ previous transient area ; SFP012

$2000 buffer: transient-area	\ -- addr
  transient-area init-t-dp !

: reset-heads   \ --
  transient-area
  dup init-t-dp !  t-dp !
;

: initTLVs	\ -- ; initialise transient area data ; SFP010
  t-dp off  init-t-dp off  $02000 t-dp-size !
; ' initTLVs atCold

: termTLVs	\ -- ; delete transient area
; ' termTLVs atExit
((
: reset-heads   \ --
  t-dp-size @ allocate
  abort" Can't allocate local variable transient area"
  dup init-t-dp !  t-dp !
;

: initTLVs	\ -- ; initialise transient area data ; SFP010
  t-dp off  init-t-dp off  $02000 t-dp-size !
; ' initTLVs atCold

: termTLVs	\ -- ; delete transient area
  init-t-dp @ free drop
; ' termTLVs atExit
))

: remove-heads	\ -- ; remove words from transient area
  init-t-dp @ t-dp @ trim-dictionary	\  remove local name heads
;

: exch-dp       \ -- ; swap dictionary pointers
  dp @  t-dp @ dp !  t-dp !
;

: array?        \ addr -- t/f ; returns true if string ends with '['
  count + 1- c@ [char] [ =
;

: get-n         \ -- n ; process until terminating ]
  begin
      get-token c" ]" 2 s= 0=       \ get token, check not ']'
    while
     ([) interpreting (])
  repeat
;

: au>cells      \ aus -- cells ; convert address units to cells
  addr(t)/cell u/
;

: start-locals  \ -- n1 n2 ; initiate local variable processing
  t-dp @ 0=                             \ set up behead area
  if  reset-heads  endif
  init-t-dp @  t-dp @  old-transient 2!	\ save from behead area ; SFP012
\  init-t-dp @  t-dp @                   \ save from behead area
  t-dp @ init-t-dp !                    \ start new transient area

  last @ OldLast !                      \ preserve for use by SMUDGE in ;

  local-offset off                      \ start at bottom of frame
  patch-address off                     \ no MAKEFRAME compiled yet
;

: restore-last  \ -- ; restore previous value of LAST
  OldLast @ Last !  ;

: ?finish-exit(t)       \ -- ; finish locals for EXIT
  patch-address @                       \ if locals demanded
  if  c_rellvs  endif			\  compile word to clean up locals
  OperatorType(t) @off
  abort" Method set but not used"
;

[undefined] PatchLocalSize(t) [if]	\ default if not yet defined
cr ." Installing Default MAKELVS patchup"
: PatchLocalSize(t)	\ #bytes/aus ^patch -- ; sets frame size
  swap au>cells swap			\ convert bytes to cells
[ 64bit? ] [if]
  2 + w!c(t)				\  set up total locals size
[then]
[ 32bit? ] [if]
  2 + w!c(t)				\  set up total locals size
[then]
[ 16bit? ] [if]
  1+ c!c(t)				\  set up total local size
[then]
;
[then]

: ?finish-locals(t)     \ -- ; finish locals processing at end of word or section
  patch-address @ if			\ if locals demanded
    c_rellvs       			\  compile word to clean up locals
    local-offset @off  aligned(t)       \  force locals size to cell boundary
    patch-address @off PatchLocalSize(t) \ defined in the XGENxxx and XOPTxxx files
    remove-heads			\  remove local name heads
    old-transient 2@ t-dp ! init-t-dp !	\  restore transient area ; SFP012
    restore-last                        \ restore LAST for use by SMUDGE in ;
  endif
  OperatorType(t) @off
  abort" Method set but not used"
;

: bar?          \ ^string -- flag ; true if string is either bar character
  dup c" |" $=
  swap c" " $= or
;

((
: makeframe,(t) \ #args #locals -- ; parameters in bytes
  target-sym makelvs compile-ref        \ compile local entry word
  chere(t) patch-address !              \ needed when frame is adjusted at compile time
[ 32bit? ] if(
  swap au>cells w,c(t)  au>cells w,c(t) \ #args then #locals
)endif
[ 16bit? ] if(
  swap au>cells c,c(t)  au>cells c,c(t)
)endif
;
))

: makeframe,(t) \ #args #locals -- ; parameters in bytes
  c_makelvs patch-address !
;

[undefined] locals-grow-up? [if]
0 constant locals-grow-up?
[then]

also cc/c
: {     cc/c    \ -- ; process local variables
  start-locals

  >in @                                 \ mark input stream
  cell(t) 2*                            \ initial frame offset
  20 0                                  \ count input arguments, max 16 locals
  do  get-token                         \ from input stream
      dup c" --" $=                     \ ignore everything after `--'
      swap bar? or  ?leave              \ or `'
      cell(t) +
  loop
  swap >in !                            \ restore input stream

  dup cell(t) 2* - local-offset !       \ keeps ARGUMENT size for now

  20 0                                  \ max no. of locals
  do  get-token                         \ from input stream
      dup c" --" $=                     \ ignore everything after `--'
      if  drop  leave  endif
      dup bar?                          \ switch from named inputs to locals
      if  2drop 0                       \ locals are -ve offset from 0
      else  [ also system ]
            dup pocket $move            \ preserve arg/local name while size is found
            [ previous ]
            array? if			\ does the name end in '['
                get-n aligned(t)        \ interpret  arr[ 7 ] ; round up to cells
            else
                cell(t)
            endif
[ locals-grow-up? ] [if]
            exch-dp                     \ build local variable in transient area
              over [ also system ] pocket [ previous ] $lvar
            exch-dp
            -                           \ move frame down by size of local
[else]
            -                           \ move frame down by size of local
            exch-dp                     \ build local variable in transient area
              dup [ also system ] pocket [ previous ] $lvar
            exch-dp
[then]
      endif
  loop

  0 min abs                             \ #locals is a negative offset

  local-offset @  over or               \ anything to lay?
  if
    local-offset @ over makeframe,(t)   \ #args #locals --
  endif

  local-offset !                        \ now set #locals, can be modified later

  [char] } parse 2drop                  \ skip to end of locals statement
  restore-last                          \ restore previous LATEST
;  ( immediate )

: {:	cc/c	\ -- ; process local variables
  [+compiler] { [previous]
;


((
  ANS LOCALS ...  implementation
))
: locals|	cc/c    \ -- ; process named inputs
  start-locals

\ gather the locals
  cell(t) 2*                            \ initial frame offset for arguments
  #17 0                                 \ for each local + bar
  do
    get-token dup bar?                  \ if bar encountered
    if  drop  negate  leave  endif      \ return -ve to show bar encountered
[ locals-grow-up? ] [if]
    exch-dp                             \ build local variables in transient area
      >r  cell(t) +  dup r> $lvar
    exch-dp
[else]
    exch-dp                             \ build local variables in transient area
      over swap $lvar                   \ n $addr $LVAR
    exch-dp
    cell(t) +                           \ increase frame offset
[then]
  loop
  dup 0> abort" Missing terminator or too many locals"

\ lay MAKEFRAME #ARGS:16 #LOCALS:16
  abs cell(t) 2* -                      \ remove initial frame offset
  0 makeframe,(t)                       \ #args #locals

  restore-last                          \ restore previous LATEST
;  ( immediate )

c-c
only forth  also c-c definitions


\ test words

((
: t2    { a b -- }
  cr ." a=" a .  ."  b=" b .
  cr ." a+b=" a b + .
;

: t3    { a b | a+b arr[ 10 ] a*b -- }
  a b + -> a+b
  a b * -> a*b
  cr ." A+B = " a+b .  ."   A*B = " a*b .
  cr ." A = " a .  ."   B = " b .
  cr ." RP@ =          " rp@ abs>rel .dword
  cr ." LOCAL-BASE =   " local-base abs>rel .dword
  cr ." addr A =       " addr a .dword
  cr ." addr B =       " addr b .dword
  cr ." addr A+B =     " addr a+b .dword
  cr ." addr ARR[ =    " arr[ .dword
     arr[     5 [char] x fill
     arr[ 5 + 5 [char] y fill
  cr ." ARR[ contains '" arr[ 10 type ." '"
  cr ." addr A*B =     " addr a*b .dword
;

: t4    \ a b --
  cr ." before T3: LOCAL-BASE =   " local-base @ abs>rel .dword
  t3
  cr ." after T3:  LOCAL-BASE =   " local-base @ abs>rel .dword
;

: t5    \ a b c --
  locals| c b a |
  a . b . c .
;
))

