\ Target Memory driver for 32-bit targets

((
File: TMEM.FTH

Copyright (c) 1997, 1998, 2006, 2008

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
net: tech-support@mpeforth.com

20080113 SFP004 Added <UMBLINK?> which is set when the link is running.
20061129 SFP003 Added LDUMP and WDUMP
SFP002	26 Aug 1998	Moved DUMP(T) CDUMP(T) from UFORTH.FTH
SFP001	6 Mar 1998	Added INCR(T) DECR(T)
))

only forth also c-c definitions


\ ****************************************************
\ Define the target memory operators as DEFERred words
\ ****************************************************

  DEFER C@C(T)		\ byte fetch from target CODE space
  DEFER C!C(T)		\ byte store to target CODE space

  DEFER C@D(T)		\ byte fetch from target DATA space
  DEFER C!D(T)		\ byte store to target DATA space

  DEFER @C(T)		\ fetch from target CODE space
  DEFER !C(T)		\ store to target CODE space

  DEFER @D(T)		\ fetch from target DATA space
  DEFER !D(T)		\ store to target DATA space

  DEFER W@C(T)		\ 16-bit fetch from CODE space
  DEFER W!C(T)		\ 16-bit store to CODE space

  DEFER W@D(T)		\ 16-bit fetch from DATA space
  DEFER W!D(T)		\ 16-bit store to DATA space

  DEFER L@C(T)		\ 32-bit fetch from CODE space
  DEFER L!C(T)		\ 32-bit store to CODE space

  DEFER L@D(T)		\ 32-bit fetch from DATA space
  DEFER L!D(T)		\ 32-bit store to DATA space

  DEFER X@C(T)		\ 64-bit fetch from CODE space
  DEFER X!C(T)		\ 64-bit store to CODE space

  DEFER X@D(T)		\ 64-bit fetch from DATA space
  DEFER X!D(T)		\ 64-bit store to DATA space


\ ******************************************************
\ Compound memory words built on the DEFERred primitives
\ ******************************************************

: +!d(t)	\ n addr(t) --
  >r  r@ @d(t) + r> !d(t)  ;

: +!c(t)	\ n addr(t) --
  >r  r@ @c(t) + r> !c(t)  ;

: 2!d(t)  ( d addr -- )
  swap over !d(t)			\ store high cell in low address
  cell(t) + !d(t)			\ store low cell in high address
;

: 2@d(t)  ( addr -- d )
  dup cell(t) + @d(t)			\ get low cell from high address
  swap @d(t)				\ get high cell from low address
;

: incr(t)	\ addr --
  1 swap +!d(t)	 ;

: decr(t)	\ addr --
  -1 swap +!d(t)  ;

: dump(t)	{ addr len | db[ $010 ] -- ; dump DATA space }
  DisplayStart
  BASE @  HEX
  addr len bounds			\ get limits
  DO  CR  i .dword			\ display address
      db[				\ bytes will be copied here
      i $10 bounds								\ MSD017
      do
	i c@d(t)			\ get byte
	2dup swap c!			\ store byte in buffer
	space .BYTE			\ print byte
	1+				\ step buffer on
      loop
      drop
      2 SPACES	db[ $10 bounds		\ gap					\ MSD017
      DO  I c@ .ASCII  LOOP		\ ascii o/p
      halt? ?leave			\ quit or halt
  $10 +LOOP				\ until done				\ MSD017
  BASE !  CR  ;				\ restore base

\ SFP003...
32bit? 64bit? or [if]
: LDUMP(t)	\ addr len -- ; dump 32 bit long words
\ *G Display (dump) len bytes of memory starting at addr
\ ** as 32 bit words.
  DisplayStart
  bounds ?do				\ get limits
    cr  i .dword  space   i 8 + i       \ print address
    do  space i l@d(t) .dword  4 +loop	\ first eight bytes
    space  i $010 + i 8 +               \ gap
    do  space i l@d(t) .dword  4 +loop	\ second eight bytes
    2 spaces  i $010 + i                \ gap
    do  i c@d(t) .ascii  loop		\ ascii o/p
    halt? ?leave			\ quit or halt
  $010 +loop                            \ until done
  cr  					\ restore base
;

: WDUMP(t)		\ addr len -- ; dump 16 bit half words
\ *G Display (dump) len bytes of memory starting at addr
\ ** as 16 bit half-words.
  DisplayStart
  bounds ?do				\ get limits
    cr  i .dword  space   i 8 + i       \ print address
    do  space i w@d(t) .word  2 +loop	\ first eight bytes
    space  i $010 + i 8 +               \ gap
    do  space i w@d(t) .word  2 +loop	\ second eight bytes
    2 spaces  i $010 + i                \ gap
    do  i c@d(t) .ascii  loop		\ ascii o/p
    halt? ?leave			\ quit or halt
  $010 +loop                            \ until done
  cr  					\ restore base
;
[then]
\ ...SFP003

: Cdump(t)	{ addr len | db[ $010 ] -- ; dump CODE space }
  DisplayStart
  BASE @  HEX
  addr len bounds			\ get limits
  DO  CR  i .dword			\ display address
      db[				\ bytes will be copied here
      i $10 bounds								\ MSD017
      do
	i c@c(t)			\ get byte
	2dup swap c!			\ store byte in buffer
	space .BYTE			\ print byte
	1+				\ step buffer on
      loop
      drop
      2 SPACES	db[ $10 bounds		\ gap					\ MSD017
      DO  I c@ .ASCII  LOOP		\ ascii o/p
      halt? ?leave			\ quit or halt
  $10 +LOOP				\ until done				\ MSD017
  BASE !  CR  ;				\ restore base

variable <UmbLink?>	\ -- addr ; SFP004
\ Set non-zero when the Umbilical link is connected.
