\ Umbilical Interpreter/Compiler

((
File: UFORTH.FTH

Copyright (c) 1988-2000, 2008, 2009, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 80 631441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

20100422 SFP030 Moved part of downloader to Sections.fth.
20090515 SFP029 Removed IRTC references.
20080912 SFP028 Changed file release handling
20000505 SFP027 Will now interpret USER vars
		Removed LABELS and EQUATES which are in CC/I
20000426 SFP026 Changed to check for target interpretation BEFORE host
20000412 SFP025 Added deferred CORD-HOOK to permit driver installation
20000409 SFP024 Deferred DOWNLOAD-SECTION
19990414 SFP023 Moved FILL and friends to INTERP.FTH
19981111 SFP022 Moved ALONE and friends to INTERP.FTH
                Moved C!C and friends to INTERP.FTH
19981108 SFP021 Added Hardvard update
19980913 SFP020 Moved .SECTIONS to INTERP.FTH
19980826 SFP019 Moved WORDS to INTERP.FTH
                Moved DUMP(T) CDUMP(T) to TMEM.FTH
                Moved DUMP CDUMP to INTERP.FTH
19980501 SFP018 Chanded USED(MS) to SAVESIZE(MS) in DOWNLOAD-SECTION
19980316 MSD017 Changed the constants in the DUMP words so that they are
         MSD017 BASE insensitive.
19980306 SFP016 Modified INTERPRET(U) for extensions behaviour
19980221 SFP015 Modified UMBILICAL-FORTH to switch EPROM emulator to
                Umbilical mode, and to switch non-emulator sections
                to use the Umbilical link
19980216 MSD014 Place serial link open and close "bracketing" around the
         MSD014 section of code that down-loads the non-EPROM sections.
19980212 MSD013 Changed name of DUMP(T) to DUMP.
19980212 SFP012 Correct search sequence in INTERPRETING(U).
19980211 SFP011 Fixed voc search order in BACK and (BACK).
19971210 JL010  Fixed bug download of sections wrongly detecting UDATA type.
19971114 SFP009 Modified for SECTIONS model.
19971003 JL008  Modified for PFW 2.100.
19970213 SFP007 Changed host/target search order so that only SC/I is
         SFP007 searched before the target.  Also, renamed some utility
         SFP007 words with (T) suffix.
19970212 SFP006 Added METHODS> support.
19970128 SFP005 Corrected emu-inRAM? and Memory drivers.
19970126 SFP004 Corrected COMPILE-NUMBER.
19970114 SFP003 Added download for non-paged targets.
19970110 SFP002 Converted to PFW 2 host and removed ?RAM.
199209?? SFP001 Converted to PowerForth/386 and v5/32.
))

decimal

only forth also c-c also umbilical definitions

cr cr .( Loading Umbilical Interpreter/Compiler .... )


\ **********************************************************************
\ MLINKDRV.FTH contains the memory access words that use the serial link
\ **********************************************************************


\ error(u) ?value(s)                               sfp 13/01/90

: error(u)      \ -- ; error handling
  .throw				\ display it
  only forth also c-c                   \ force host vocabs
  also umbilical definitions
  abort  ;

: ?value(s)     \ symbol-addr -- value
  dup resolved? ?undef                  \ check symbol exists
  @(s)  ;                               \ get its value

: umbilical-.log        \ $addr addr --
  2drop  ;

\ ?sci ?scc .............................          sfp 06/04/89

: ?sci                                \ Special INTERPRETER
  sci context-find                    \
  IF  execute  r> drop  ENDIF         \
;

: ?scc                                \ Special COMPILER
  scc context-find                    \
  IF  execute  r> drop  ENDIF         \
;


\ ?target(u)                                       sfp 06/04/89

: run-method_methods(u) \ symbol --
  >r
  r@ target-exec                        \ execute primary symbol
  r> sel-method target-exec             \ execute secondary symbol
;

: run-value(u)  \ ?? symbol -- ??
  value-addr@
  case  OperatorType(t) @off
    0 of  @d(t)  endof
    1 of  !d(t)  endof
    2 of         endof
        cr ." Invalid operator type" ABORT
  endcase
;

: run-method(u) \ symbol --
\ cr ." symbol has method bit set"
  case
    dup method_value?   ?Of  run-value(u)           Endof
    dup method_UserVar? ?Of  target-exec            Endof
    dup method_methods? ?Of  run-method_methods(u)  Endof
      cr ." Can't handle this type: " .SymName  ABORT
  end-case
;

: ?target(u)    \ --
  here find(t)                          \ Word exists ?
  IF  dup resolved? 0=
      if  drop  exit  endif
      compil?                           \ yes, compiling?
      IF
        ?target-compile
      ELSE
\ cr ." found symbol " here count type
        case
          dup voc?    ?of  is-voc(s)  endof     \ is it a vocabulary
          dup method? ?Of  run-method(u)  Endof \ special cases for methods
                        target-exec             \ no, execute target CFA
        end-case
      ENDIF
      r> drop                           \ exit outer word
  ENDIF  ;

\ ?labels

: ?LABELS(u)
  here 'LABELS(s) @(m) #threads(s)+     \ -- $addr ^threads
  v-find(s)                             \ exists ?
  IF
    dup resolved? ?undef                \ error if label forward referenced
    COMPIL?                             \ compiling ?
    IF
      doliteralsym,                     \ yes, compile LIT
    ELSE
      1 REF
    ENDIF
    R> DROP                             \ leave outer word
  ELSE
    DROP                                \ discard text addr
  ENDIF  ;


\ number                                           sfp 06/04/89

: Number        \ -- d | n | abort
  here count isNumber? dup ?undef
\  here number? dup ?undef
  2 =
  if  convert-double  endif
;

: compile-number        \ -- ; forward ref not permitted for Umbilical Forth
  here count isNumber? ?dup ?undef	\ valid number?
\  here number? ?dup ?undef              \ valid number?
  dup 2 =
  if  >r convert-double r>  endif
  abs $0FF and                          \  allow for number type classes
  0 swap 1- do                          \  adjust n for DO ... -1 +LOOP
    i roll general-literal              \  lay components of number
  -1 +loop
;


\ Outer Interpreter ........................... J Lsfp 28/06/90

: INTERPRETING(u)
\  ?SCI ?target(u) ?INTERPRETER ?EQUATES ?LABELS(u)	\ SFP012 SFP026
  ?SCI ?INTERPRETER ?target(u) ?EQUATES ?LABELS(u)	\ SFP012 SFP026
  NUMBER  ;

: COMPILING(u)
  ?SCC ?COMPILER ?TARGET(u) ?EQUATES ?LABELS(u)
  ( NUMBER COMPILE-NUM ) compile-number  ;

: ASSEMBLING(u)
  ?ASSEMBLER ?SCI ?INTERPRETER ?EQUATES ?LABELS(u)
  ?TARGET(u) ?C-C NUMBER ;


\ interpret(u)  ............................... J Lsfp 29/06/90

: Interpret(u)	\ -- ; umbilical interpreter ; SFP016
  BEGIN
    BL-WORD/UC c@
   WHILE
    host-mode @(m)              \ are we working in host mode?
    If
      host-interpret            \   go away and interpret this on the host
    Else
      compil?
      IF    compiling(u)
      ELSE  interp?
            if    interpreting(u)
            else  assembling(u)
            endif
      ENDIF
    Endif
    ?STACK
  REPEAT  ;


\ Quit ........................................ J Lsfp 29/06/90

: Quit-emu      \ --
  s0 @ sp!                            \ bullet proofing
  STATE(T) off(m)                     \ force interpretation
  blk off                             \ input from keyboard
  BEGIN
    r0 @ rp!                          \ N.B. host return stack
    cr query
    ['] interpret(u) catch ?dup if    \ deal with line
      cr ." THROW CODE: " dup . .throw
      QuitHook init-QI quit
    endif
    interp?                           \ see if prompt needed
    IF  .prompt  ENDIF
  AGAIN
;


\ UF Initialisation and shutdown

also sections.voc
: (download-section)	{ sect | buff used -- ; download memory image }  \ SFP018, SFP024
  sect savesize(ms) dup -> used         \ find size of section contents
  get-img-buff -> buff                  \ put contents of section in buffer
  sect MS-low @  buff  used  sect read-gen(ms)
  buff  sect MS-low @  used             \ -- addr(h) addr(t) len
  begin
    dup #1024 u< 0=                     \ send in 1k chunks
   while                                \ -- addr(h) addr(t) len
    >r
    2dup #1024 copy-cmnd>t >target      \ send chunk to target
    swap #1024 +  swap #1024 +          \ update addresses
    r> #1024 -                          \ update length
  repeat
  dup                                   \ if anything left
  if
    copy-cmnd>t >target                 \ send it to the target
  else
    drop 2drop
  endif
  buff free-img-buff
;
assign (download-section) to-do download-section
previous

(( Moved to Sections.fth
defer download-section	\ sect -- ; SFP024
  assign (download-section) to-do download-section

: ?download-section     \ sect -- ; download if needed  \ SFP021 Harvard changes
  [ also sections.voc ]
[ harvard? ] [if]
\  dup in-emu? 0=  over cdata-type? and       \ download if not in emu and CDATA         \ ***SFP021***
  dup cdata-type?			\ download if CDATA
[else]
  dup udata-type? 0=			\ download if not UDATA
[endif]
  if
    cr ." Download " dup .pfa ."  section to target? (y/n): " y/n?
    if  download-section  else  drop  endif
  else
    drop
  endif
  [ previous ]
;

: download-sections     \ -- ; download all required sections   \ SFP021
                        \ N.B. can only download CDATA sections if Harvard
  [ also sections.voc ]
  section-chain
  begin
    @ ?dup
   while
    dup ?download-section
    MS-link
  repeat
  [ previous ]
;
))

\ targ-prompt reset-UF

: targ-prompt   \ -- ; prompt while Umbiiical Forth runs
  ."    T:OK"  depth ?dup
  if  ." -" .  endif
  space  ;

: Reset-UF      \ -- ; reset Umbilical Forth
  no-log  internal                      \ no log, no heads
  assign targ-prompt to-do .prompt      \ reset host prompt
  assign interpret(u) to-do interpret(t)  \ and interpreter
  assign quit-emu to-do quit            \ error recovery XCompiler
;

decimal

defer cord-hook	\ -- ; hook for driver installation ; SFP025
  assign noop to-do cord-hook

: cord          \ --
\ ." CORD" cr key? drop
  reset-UF                              \ reset re-usable code
\ ." a" cr key? drop
  link-open                             \ open target link ; SFP002
  <UmbLink?> on				\ mark link as active
  #300 ms				\ allow link to settle
  cls                                   \ pretties
  cr ." Umbilical Forth " .version      \ sign on
  cr .target                            \ say which target
  cr  decimal                           \ force base
  cr .link-state                        \ show link state
  cr xdata-via-link                     \ non-emulator sections via link
  cr .section-report                    \ show section list
  cr .curr-sect                         \ and current section
  cr
  cr ." Use ^B to abort link,"
  cr ." and then RESET <CR> to restart it."
  cr ." BASE is now DECIMAL"            \ warn user
  cr cr ."      T:OK"                   \ makes it prettier
  link-flush                            \ flush input buffers
  cord-hook				\ install any needed memory drivers	SFP025
  doAfterwards
  quit-emu                              \ run interpreter
;


\ Umbilical-Forth and other directives

also c-c definitions

decimal

: Umbilical-Forth       \ -- ; directive
  (finis)                               \ finish cross compilation

  ?releaseFiles				\ SFP028
\  assign (abort) to-do abort            \ reset error handler
  only forth  also c-c  also umbilical  \ set search order
  link-close                            \ allows for user download
  cr
  cr ." Target must be powered."
  cr ." Reset target and then press <space> to run Umbilical Forth " \ wait...
  Begin  key bl =  Until                \ poll for space bar
  link-open                             \ MSD014
  download-sections                     \ download images if needed
  link-close                            \ MSD014
  cord  ;                               \ run umbilical forth

previous definitions


\ more                                             sfp 06/11/91

: more          \ --
  begin  key?  while  key drop  repeat  \ flush input buffer
  cr ." Press <space> to continue "     \ ask for input
  key bl <> abort" ABORTED!"  ;     \ wait for it

: (back)        \ -- ; close umbilical
  assign .SysPrompt to-do .prompt	\ reset host
  assign (quit) to-do quit
\  assign (abort) to-do abort
\  assign (error) to-do error
  assign (interpret) to-do interpret                                            \ SFP011
  only forth  also c-c                  \ force search order
  also umbilical definitions
  link-close                            \ turn off serial port
  <UmbLink?> off
;


\ Special INTERPRETER ......................... J Lsfp 13/01/90

only Forth also c-c also Umbilical also SCI definitions

: -umbilical	cc/si	\ --
  runQI  ;

: Download      cc/si   \ --
  download-sections  ;

: Back          cc/si   \ -- ; host Forth system
  (back)                                \ close Umbilical
  only forth also c-c also umbilical definitions                                \ SFP011
  abort  ;                              \ normal Forth system

: Reset         cc/si   \ --
  (back) cord  ;

((
: Words         cc/si   \ --
  words(s) cr  ;
))

: Help          cc/si   \ --
  Cr ." Special when Interpreting - overrides target words"
  Cr ." =================================================="
  SCI words
  Cr
  Cr ." Special when Interpreting - only done if no target version"
  Cr ." ========================================================="
  INTERPRETER words
  Cr
  Cr ." Special when Compiling - overrides any target version"
  Cr ." ====================================================="
  COMPILER words
  Cr
  Cr ." use LABELS to see labels and EQUATES to see equates"
  Cr ." ===================================================
  Cr  ;

: C,            cc/si
  gC,(T)  ;

: w,            cc/si   \ w --
  gw,(t)  ;

: ,             cc/si
  g,(t)  ;


\ Special case INTERPRETER ..................... J sfp 28/06/90

: .(t)          cc/si   \ n --
  .  ;

: u.(t)         cc/si   \ n --
  u.  ;

: ?(t)          cc/si   \ addr --
  target-sym @ target-exec  .  ;

decimal

: Hex(H)        cc/si   \ --
  16 base !  ;

: Decimal(H)    cc/si   \ --
  10 base !  ;

: Binary(H)     cc/si   \ --
  2 base !  ;

: Octal(H)      cc/si   \ --
  8 base !  ;

: Bye           cc/si   \ -- ; leave system
  Link-Close  Bye  ;                    \ be kind to DOS

: Cls           cc/si   \ -- ; clear the screen
  cls  ;

hex

: .FTH          cc/si  .SourceFile  ;
\ : Use           cc/si  Use  ;
\ : QC            cc/si  QC  ;


\ .s \ ( ((                                        sfp 05/06/91

: .S(T)         cc/si
  .s  ;

: \             cc/si  postpone \  ;
: (             cc/si  postpone (  ;		\ ) ; keep editor happy
: ((            cc/si  postpone ((  ;		\ )) ; keep editor happy


\ latest                                           sfp 28/06/90

: LATEST        cc/si  \ --
  1 abort" No NAMEs on target"  ;

: to            cc/si   \ --
  1 operatortype(t) !  ;

: ->            cc/si   \ --
  1 operatortype(t) !  ;


\ ************************************************************************
\ Defining words support so that REQUIRED.VOC words do not override things
\ ************************************************************************

: :             cc/si
  [+interpreter] : [previous]  ;

: code          cc/si
  [+interpreter] code [previous]  ;

: value         cc/si
  [+interpreter] value [previous]  ;

: variable      cc/si
  [+interpreter] variable [previous]  ;

: constant      cc/si
  [+interpreter] constant [previous]  ;

: user          cc/si
  [+interpreter] user [previous]  ;

: create        cc/si
  [+interpreter] create [previous]  ;

: 2constant     cc/si
  [+interpreter] 2constant [previous]  ;

: 2variable     cc/si
  [+interpreter] 2variable [previous]  ;

: Methods>      cc/si
  [+interpreter] Methods> [previous]  ;

: :noname       cc/si
  [+interpreter] :noname [previous]  ;

: Buffer:       cc/si
  [+interpreter] Buffer: [previous]  ;


\ End of special case INTERPRETER                  sfp 03/04/89

c-c only forth definitions
decimal

cr .( Umbilical Interpreter/Compiler loaded )

