((      XC_XTEND.FTH        Compatability layer for extending the compiler.

        Copyright (c) 1996-97, 1999, 2001, 2009

        MicroProcessor Engineering
        133 Hill Lane
        Southampton SO15 5AF
        England

        tel: +44 (0)2380 631441
        email: mpe@mpeforth.com


20191024 SFP013 VFX 5.1 changes.
20180530 SFP012 VFX v5 changes.
20160922 SFP011	Enhanced portability for ARM hosts.
SFP010	21 Mar 2001	Extended POSTPONE for EXTEND_INTERPRETERs that compile.
SFP009	1 June 2000	Corrected use of HERE(T) when CHERE(T) is right
SFP008	15 Oct 1999	Added { and LOCALS| to EXTENSIONS voc.
			Made ['] immediate in EXTENSIONS voc.
			Code tidy up.
SFP007	10 Sept 1999	Added .( to EXTENSIONS voc
JL006    2 Oct 97       Modified for PFW 2.100
SFP005  19 Feb 97       Removed compilation of EXIT after SET-METHODS in METHODS>
SFP004  15 Feb 97       Changed extension mode search orders to match FI orders.
                        Changed : behaviour to ease decompilation.
SFP003  13 Feb 97       Added ;CODE and [']
SFP002  11 Feb 97       Moved EXTENSION_MODE to XDTC41-4
SFP001  4 Feb 97        Added ( \ and (( to extensions
                        Corrected POSTPONE
))



\ *****************************************************************
\ **** Compiler extension search order control and compilation ****
\ *****************************************************************

only forth also c-c definitions

(( moved to XDTC41-4
\ extend which part of the system
0 constant EXTEND_HOST
1 constant EXTEND_COMPILER
2 constant EXTEND_INTERPRETER
3 constant EXTEND_ASSEMBLER
))

: Set-Voc-Order  (  extension-mode --  ; set the default search order for an extension mode )
  case
    EXTEND_HOST
    of  only forth  also c-c definitions  endof
    EXTEND_COMPILER
    of  only forth  also c-c  also compiler definitions  also extensions  endof
    EXTEND_INTERPRETER
    of  only forth  also c-c  also interpreter definitions  also extensions  endof
    EXTEND_ASSEMBLER
    of  only forth  also c-c  also interpreter  also asm-access definitions  also extensions  endof
        cr ." Internal Software Error: Invalid EXTEND_host type:" dup . ABORT
  endcase
;

\ Escape word for search order
\
: cc/extensions
  c-c
;  IMMEDIATE


\
\ redefinition of : so we can build extensions; this sets search order *and doesn't alter it*
\ This is the definition that must always be found and so it is built into the EXTENSIONS
\ vocabulary which is always first in search order after a compiler extension directive is
\ executed.
\

also system also extensions definitions  cc/extensions

: :  cc/extensions
  extension_mode @ set-voc-order  :
;

: ;  cc/extensions
  postpone ;
  only forth also c-c definitions also extensions
;  IMMEDIATE


\ *****************************************************
\ ****  Defining word support for runtime on host  ****
\ *****************************************************

only forth also c-c definitions

(( \ deleted SFP013 for ARM/Cortex hosts
: RunDoes>	\ addr --
  'latest(s) @(m) value+ @(s) patch-child       \ patch up the CFA
  'latest(s) @(m) status+ clear-c/v             \ C/V field now used for extension packet
  r> drop                                       \ can't return into 'target' runtime so back up one more level
;
))

: RunDoesX>  (  addr  --  ;  )
  'latest(s) @(m) value+ @(s) patch-child       \ patch up the CFA
  'latest(s) @(m) status+ clear-c/v             \ C/V field now used for extension packet
;

only forth also c-c also extensions definitions  cc/extensions

: [order] cc/extensions order ; immediate

: Does>  cc/extensions  (  --  )
\ cr ." DOES> cc/e " chere(t) .dword
  chere(t) 1 #literal                   \ compile address of runtime for RUNDOES> to use ; SFP009
  postpone RUNDOESX>  postpone exit  ( ;host, )   reveal    \ end of compile portion ; SFP013
\  compile RUNDOESX>  ;host,  smudge     \ end of compile portion
\  compile RUNDOES>                      \ end of compile portion
  0 state!                              \ host not compiling
\ cr chere(t) .dword
  dodoes,
  host-mode off(m) (])                  \ stop working on host, turn target compiler on
;  IMMEDIATE

: Interp>  cc/extensions  (  --  )
\ cr ." Interp> "  here .dword
  chere(t) 1 #literal                   \ compile address of runtime for RUNDOES> to use ; SFP009
  postpone RUNDOESX>  postpone exit  ( ;host, ) reveal	\ end of compile portion ; SFP013
\  postpone RUNDOESX>  postpone exit  reveal     \ end of compile portion
\  compile RUNDOES>                      \ end of compile portion
  0 state!                              \ host not compiling
  dodoes,
  host-mode off(m) (])                  \ stop working on host, turn target compiler on
;  IMMEDIATE

: ;code cc/extensions   \ -- ; INTERPRETER-MODE action of ;CODE
  chere(t) 1 #literal                   \ compile address of runtime for RUNDOES> to use ; SFP009
  postpone RUNDOESX>  postpone exit  ( ;host, )   reveal    \ end of compile portion ; SFP013
\  compile RUNDOESX>  ;host,  smudge     \ end of compile portion
\  compile RUNDOES>                      \ end of compile portion
  0 state!                              \ host not compiling
  host-mode off(m)                      \ stop working on host, turn target compiler on
  2 state(t) !(m)                       \ initialise target assembly
  sp@ asm_start initfwd
;  IMMEDIATE

: Methods>      cc/extensions    \ -- ; e.g. : FOO CREATE ... METHODS> @ ! ;
  make-methods                          \ create a new extension packet
  postpone set-methods  ,                \ will mark created symbol as a method, symbol address in-line
;  IMMEDIATE

: [']           cc/extensions   \ -- ; compiles a reference to CFA
  '(t)  postpone literal
;  IMMEDIATE				\ SFP008


\ SFP008...
\ When extending, we need the host's locals. N.B. ADDR MUST not be used!
\ Therefore, we cannot use LOCAL ARRAYS
: {		cc/extensions
  extension_mode @ if
    postpone {				\ host version
  else
    [+compiler] { [previous]
  endif
;  IMMEDIATE

: locals|	cc/extensions
  extension_mode @ if
    postpone locals|				\ host version
  else
    [+compiler] locals| [previous]
  endif
;  IMMEDIATE

: to		cc/extensions
  extension_mode @ if
    postpone to				\ host version
  else
    [+compiler] to [previous]
  endif
;  IMMEDIATE

: ->		cc/extensions
  extension_mode @ if
    postpone to				\ host version
  else
    [+compiler] to [previous]
  endif
;  IMMEDIATE

: addr		cc/extensions
  extension_mode @ abort" Cannot generate TARGET address for HOST local"
  [+compiler] addr [previous]
;  IMMEDIATE
\ ...SFP008


\ Put these in to avoid any unpleasantnesses
: (     cc/extensions
  postpone (  ;  immediate
: ((    cc/extensions			\ for syntax highlighter ))
  postpone ((  ;  immediate		\ for syntax highlighter ))
: \     cc/extensions
  postpone \  ;  immediate
: .(	cc/extensions
  postpone .(  ;  immediate		\ )
: ."	cc/extensions
  state @
  if  postpone ."  else  [interp] ."  endif
; immediate


\ **************************************************
\ ****    special support for compatibility     ****
\ **************************************************

: postpone  cc/extensions   (  "<spaces>name"  --  )
  extension_mode @ EXTEND_COMPILER = if	\ SFP001
    bl-word/uc ['] compiler v-find If
      compile,
    Else
      find(t)_ans If
        postpone literal  postpone compile-ref
      Else
        cr ." Can't find: " $.  ABORT
      Endif
    Endif
  else
    extension_mode @ EXTEND_INTERPRETER = if	\ SFP001
      bl-word/uc ['] compiler v-find If
        compile,
      Else
        ['] interpreter v-find if
	  compile,
	else
          find(t)_ans If
            postpone literal  postpone compile-ref
          Else
            cr ." Can't find: " $.  ABORT
          Endif
	endif
      Endif
    else
      postpone postpone
    endif
  endif
;  IMMEDIATE

c-c only forth also c-c definitions


\ *****************************************
\ ****  Compiler extension directives  ****
\ *****************************************

only forth  also c-c  also cc/i

: host  cc/i
  in-host-build on(m)                   \ we're building into the host
  host-mode on(m)                       \ turn on compiler mechanism for host building
  only forth also c-c definitions also extensions
  EXTEND_HOST extension_mode !
;

: interpreter  cc/i
  in-host-build on(m)                   \ we're building into the host
  host-mode on(m)                       \ turn on compiler mechanism for host building
  only forth  also c-c definitions  also extensions
  EXTEND_INTERPRETER extension_mode !
;

: compiler  cc/i
  in-host-build on(m)                   \ we're building into the host
  host-mode on(m)                       \ turn on compiler mechanism for host building
  only forth  also c-c definitions  also extensions
  EXTEND_COMPILER extension_mode !
;

: assembler cc/i
  in-host-build on(m)                   \ we're building into the host
  host-mode on(m)                       \ turn on compiler mechanism for host building
  only forth also c-c definitions also extensions
  EXTEND_ASSEMBLER extension_mode !
;

: target  cc/i                          \ doesn't do anything, would already be in target mode
  in-host-build off(m)
  host-mode off(m)
  0 extension_mode !
;

: target-support  cc/i                  \ not going to differentiate target and target-support
  in-host-build off(m)
  host-mode off(m)
  0 extension_mode !
;

: [comp]	cc/i
  EXTEND_COMPILER dup extension_mode !  Set-Voc-Order
;

c-c only forth  also c-c  also cc/e

: [interp]	cc/e
  EXTEND_INTERPRETER dup extension_mode !  Set-Voc-Order
; immediate

: host          cc/e
  [+interpreter] host [previous]  ;

: interpreter   cc/e
  [+interpreter] interpreter [previous]  ;

: compiler      cc/e
  [+interpreter] compiler [previous]  ;

: assembler     cc/e
  [+interpreter] assembler [previous]  ;

c-c  only forth  also c-c definitions

: target
  in-host-build off(m)
  host-mode off(m)
  0 extension_mode !
  only forth also c-c also interpreter definitions
;

: target-support
  target
;

