\ XDTC41-3.FTH - MPE Forth Cross-Compiler v6.x

((
        Copyright (c) 1983-2000

        MicroProcessor Engineering
        133 Hill Lane
        Southampton SO15 5AF
        England

        tel: (+44) 2380 631441
        net: mpe@mpeforth.com
             tech-support@mpeforth.com
        web: www.mpeforth.com

SFP009  15 June 2021    Expanded forward reference packet.
SFP008	22 March 2001	Fixed (.LOG) for :NONAME
SFP007	1 June 2000	Fixed REF! for Harvard targets when current section
			is not CDATA.
SFP006	21 Feb 2000	Added automatic reporting of missing runtime actions
SFP005  8 Feb 1999      Deferred (.LOG)
SFP004  14 Sep 1998     Moved VOCS(H) and VOCS(T) to XC-SYMBL.FTH
SFP003  13 August 1998  Added VOCS(H) and VOCS(T)
MSD002  24 Apr 98       Add logs-per-line facility.
JL001   15 Jan 97       Reinstated check for unresolved labels.
        May 96          #UNRESOLVED counter added
        Jul 92          converted to PC PowerForth/386
        July 1991       Output log converted to three columns wide

))


\ Ref! -- J Lee .. 15/10/93

Only Forth Also C-C Definitions

decimal

\ A forward reference packet consists of three primary fields
\ plus two extension fields that may be present for some oddball
\ CPUs such as the A2S. These packets are also created by the
\ assemblers - see the formaward reference words such as
\ FORWARD-SET.
\ cell 0 - link to symbol or latest forward reference
\ cell 1 - address at which reference was made - CHERE(T)
\ cell 2 - forward reference mode
\ ------
\ cell 3 - ext 1
\ cell 4 - ext 2

: Ref!          \ Symbol-Ref-Field mode -- value
  >r
  here(s) over @(s) ,(s)                \ Ref-Packet
  dup rot !(s)  chere(t) ,(s)		\ address of reference ; SFP007
  r> ,(s)                               \ reference mode (0=CFA, 1=LABEL)
  0 ,(s)  0 ,(s)			\ ext 1 and ext 2
  Assembl? if
    dup  Ref1 @(m) if                   \ up to two forward references
      Ref2 dup @(m) if                  \ REF1/2 <> 0 indicates forward ref.
        cr ." Too many assembler forward references at " CHere(t) .dword space
        #-13 throw
      endif				\ SFP007
    Else  Ref1
    Endif
    !(m)
  Endif
  1 #forward +!(m)  ;                   \ count forward references


\ (Ref) Ref Run-Time-Ref-- J Lee .. 15/10/93

: (Ref)         \ symbol-addr mode t/f -- value
  If
    drop @(s)
   Else
    swap cell+ swap ref!
  Endif
;

: Ref           \ sym-val-fld mode -- value
  over resolved? (ref)                \ Only used for normal
;                                     \  symbols.

: Run-Time-Ref  \ r-t-sym-addr mode -- value
  over 2 cells- run-time? (ref)
;


\ context-find bl-word-up word-down -- J Lee .. 17/3/95

decimal

((
: CONTEXT-FIND  \ -- cfa true | false ; find in CONTEXT voc. - text at HERE
  here context @ v-find  dup 0=
  if  nip  endif  ;
))
: CONTEXT-FIND  \ -- cfa true | false ; find in CONTEXT voc. - text at HERE
  here count context @ search-wordlist
;

create word-buffer 64 allot

: BL-WORD-UP    ( copy next text string to HERE and safety )
  get-token word-buffer OVER C@ 2+ CMOVE ;      \ *jl* use GET-TOKEN

: WORD-DOWN     ( move previously saved string to HERE )
  word-buffer here OVER C@ 2+ CMOVE ;


\ Printing                                         Jon 09/08/88

                  #12 CONSTANT LOG-WIDTH        \ characters per name
#80  LOG-WIDTH #13 +  / CONSTANT LOG/LINE	\ names per line
                  #61 CONSTANT L/P		\ lines per page

\
\ Set the spacing of the unresolved symbols display at the end of compilation
\
80 35 - 9 / CONSTANT ##S                \ 80 chars wide
                                        \ 35 chars for name
                                        \ 9 chars / address
: EJECT         WANT-PAGES @(m) IF $0C EMIT ENDIF ;                              \ MSD005

: TITLE         CR 3 SPACES .VERSION
                CR 3 SPACES .TARGET
                CR 3 SPACES .SYMBOL-SPACE
                CR CR  5 #LINES !(m) ;

: NEW-PAGE      EJECT TITLE  #LOGGED off(m)  ;

: NPAGE         WANT-PAGES @(m)                                                 \ MSD005
                IF  #LINES @(m)                                                 \ MSD005
                    IF  #LINES @(m) L/P 1- >
                        IF  NEW-PAGE
                        ELSE  1 #LINES +!(m)
                        ENDIF
                    ELSE  NEW-PAGE
                    ENDIF                                                       \ MSD005
                ENDIF  ;

: PCR           CR NPAGE  #LOGGED off(m)  ;

: 6U.R          BASE @  HEX  SWAP 6 U.R  BASE !  ;

32bit? 64bit? or  [if]
: 8U.R		\ u --
  space BASE @  HEX  SWAP 8 U.R  BASE !
;
[then]

16bit? [if]
: 8U.R		\ u --
  dup $FFFF0000 and if
    .dword
  else
    space  BASE @  HEX  SWAP 8 U.R  BASE !
  endif
;
[then]

: ?NONE         \ -- ; shows that no unresolved references were found
  ALL-RES @(m)
  IF    PCR ."    None " PCR            \ PCR added before None
  ELSE  ALL-RES on(m)
  ENDIF  ;

Defer (.log)    \ type$-addr value sym-addr -- ; SFP005

: ((.LOG))      \ type$-addr value sym-addr -- ; SFP005
  >R
  ?LOG @(m)
  IF  #LOGGED @(m)  LOGS-PER-LINE @(m) >=       \ see if new line needed ; MSD002
      IF  PCR  ENDIF
      SPACE  8U.R                               \ value
      SPACE count type SPACE                    \ type$ string (: CR DF ...)
      r@ if
        r@ >name(s) count(s) 31 and             \ symbol name from symbol table
        LOG-WIDTH MIN                           \ see if not too long
        LOG-WIDTH OVER -  ROT ROT  TYPE(S)      \ display name from symbol table
        SPACES                                  \ pad for next
      else
        log-width spaces
      endif
      1 #LOGGED +!(m)                           \ count no. of symbols logged
  ELSE  2DROP
\        ." ."  out @ 50 >=
\        if  pcr  endif
  ENDIF
  R> DROP  ;
assign ((.log)) to-do (.log)

\ *jl*
\ MPEL -- 16/4/93
: COMPILER-.LOG          'LATEST(s) @(m)  (.LOG)  ;

\ *jl*
\ MPEL -- 16/4/93
defer .LOG      assign Compiler-.log to-do .log

\ Printing                                         sfp 15/08/88

Variable #Unresolved

decimal
: .UNRES        \ sym-nfa fwd-ref-fld --
  @(s) ?DUP                             \ any forward refs?
  IF  ALL-RES off(m)                    \ yes, clear flag
      pcr 2 spaces swap .name(s)        \ display name
      BEGIN  35 out @ - SPACES  ##S 0   \ tab it
             DO  DUP cell+ @(s) 8U.R SPACE  \ show address
                 @(s) DUP 0= ?leave     \ see if at end
             LOOP
             PCR
             dup 0=  halt?  OR
      UNTIL
      #Unresolved Incr                  \ count number of forward refs
  ENDIF
  DROP  ;


\ Printing                                         sfp 15/08/88

((
: ?UNRES        \ voc(s) -- ; display unreolved references
  #threads(s)+ dup cell+  swap @(s) cell*  bounds
  Do  i                                 \ for each thread
      BEGIN  @(s) ?dup                  \ -- nfa [nfa]
        WHILE  dup dup name>(s)         \ -- nfa nfa addr(s)
               dup resolved?            \ symbol defined ?
               if  2drop                \ yes, chuck data
               else  cell+ .unres       \ no, show words
               endif
               n>link(s)                \ switch to link
      REPEAT
  cell +loop
;
))
: ?UNRES        \ voc(s) -- ; display unreolved references ; SFP006
  #threads(s)+ dup cell+  swap @(s) cell*  bounds
  Do
    i					\ for each thread
    BEGIN
      @(s) ?dup				\ -- nfa [nfa]
     WHILE
      dup dup name>(s)			\ -- nfa nfa addr(s)
      dup resolved? if			\ symbol defined ?
        dup def-word? if		\ if a defining word
	  dup 3 cells + @(s) if		\ check the runtime action, whoops
	    3 cells + .unres
	  else
	    2drop
	  endif
	else
          2drop				\ yes, chuck data
	endif
      else
        cell+ .unres			\ no, show words
      endif
      n>link(s)				\ switch to link
    REPEAT
  cell +loop
;


\ Printing -- J Lee .. 30/10/95
\

: ."UNRES"      PCR ." Unresolved " ;

: ?LINK         DUP >name(s) SWAP 3 cells+ .UNRES ;

\ *jl* 30/10/95
\ Added 2CONSTANT
\ Changed display to be accurate for a token compiler
\ Merged the old ?DOES> checks
\
: ?CFAS         ."UNRES" ." Defining Words "
                target-sym : ?LINK
                target-sym CONSTANT ?LINK
                target-sym VARIABLE ?LINK
                target-sym USER ?LINK
                target-sym DOES> ?LINK
                target-sym 2CONSTANT ?LINK
                target-sym VOCABULARY ?LINK
                target-sym PAGE-VOCABULARY ?link
                target-sym VALUE ?LINK
                target-sym DEFER ?LINK
                ?NONE  ;

(( *jl* 30/10/95 Nonsense for a token compiler
: ?DOES>        ."UNRES" ." DOES> pointers "
                target VOCABULARY ?LINK
                ?NONE   ;
))


\ Printing [ ?LABS NEEDS ?UNRES ] .......          Jon 10/08/88

: ?LABS                 \ -- ; show unresolved labels
  ."UNRES" ." labels "
  'LABELS(s) @(m) ?UNRES  ?NONE ;

: VOC-LINKS             \ -- 0 vl1 vl2 .... ; get all voc-links
  0  VOC-LINK(s) @(m)                   \ and first
  BEGIN  ?DUP                           \ while they exist
    WHILE  DUP @(s)                     \ stack and get next one
  REPEAT  ;                             \


\ Printing                                         sfp 14/04/89

: .VOC                  \ voc-link -- ; show unresolved in voc
  vl-                                   \ voc-link to voc
  pcr ." in "  dup .SymName		\ display voc name
  ?UNRES  ?NONE  ;                      \ set vocab, find unres

: ?VOCS                 \ 0 vl1 vl2 .... -- ; show all unres
  BEGIN  ?DUP                           \ while vocs exist
    WHILE  .voc                         \ check them
  REPEAT  ;                             \


\ Printing                                         sfp 15/08/88

: ?WORDS        ."UNRES" ." words "
                VOC-LINKS               \ extract all vocs
                ?VOCS  ;                \ show language vocs

\ *jl* 30/10/95
\ Don't search for unresolved labels in the token compiler
\ DOES> check merged with ?CFAS
\ *sfp* 13/05/96 #UNRESOLVED COUNTER
\ jl001
: ?RESOLVE      \ --
  0 #unresolved !
  PCR all-res on(m) ?CFAS ( ?DOES> )  ?LABS ?WORDS  ;

((
: VOCS(T)       \ -- ; display all target vocabulary names
  PCR ." Target vocabularies: "
  VOC-LINK(s) @(m)                      \ first
  BEGIN
    ?DUP                                \ while they exist
   WHILE
     dup vl- .SymName space		\ display the name
     @(s)                               \ get next one
  REPEAT
  PCR  ;

: vocs(h)       \ -- ; display host vocabularies
  PCR ." Host vocabularies: "
  vocs
  PCR  ;
))
only forth definitions

