\ XDTC41-5.FTH Cross compiler termination and clean up

((
MPE Forth Cross-Compiler.

Copyright (c) 1983-98, 1999, 2000, 2003, 2006, 2008, 2010

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
net: tech-support@mpeforth.com


Change History
==============
20120612 SFP038 Updated LOCATE info at start and end of compilation.
20101102 SFP037 Corrected alignment in LAY-RAM-TABLE and APP-LAY-RAM-TABLE.
20101014 SFP036 Added BOOTFINIS and APPFINIS.
20100105 SFP035	Added alignment check in LAY-RAM-TABLE
20080912 SFP034 Changed file release.
20061003 SFP033 Added section report to console if log file used.
20030807 SFP032 Added -RAM-TABLE to inhibit RAM table generation.
20000726 SFP031 Added INIT-WID-LINK initialisation
19991016 SFP030 Put emulator into umbilical mode after INTERACTIVE
19990911 SFP029 Added ?STACKJUNK check
19990821 SFP028 Added LOCATE-VIA-AIDE to /IDE
19990504 SFP027 Added INTERACTIVE before FINIS to permit compiler to be left
                in an interactive state without compilation after compilation
                has finished. This allows access to XREF and other tools.
19981117 SFP026 Modified.COMPILATION-TIME to show number of lines compiled
19981115 SFP025 Modified (FINIS) to save the XREF memory
                Modified FINIS to free the XREF memory
                Added XREF-REPORT to FINIS-REPORT
19981114 SFP024 Modified (CROSS-COMPILE) to initialise LOCATE info
19980825 SFP023 Added checks to FINIS-VOC for existence of target voc.
19980430 SFP022 See MSD015, in FINIS ALIGN(T) only performed if ?ALIGNING set
19980424 MSD021 Add /COLS facilities.  Also, initialise the value of
         MSD021 LOGS-PER-LINE at *run* time since the definition of LOG/LINE
         MSD021 comes too late to use in INITIAL.FTH.
19980424 MSD020 Add /PAGEOFF facilities.
19980415 MSD019 Adopt Matt's ARM changes.
19980319 MSD018 Added handling of the new /IDE parameter.
19980311 MSD017 Yet *more* fiddling with the unresolved warning message.  This
         MSD017 time, the change is to make it appear on-screen even if you
         MSD017 elected to generate a log file.  Also, I made a cosmetic
         MSD017 change to the warning message itself - the box now extends
         MSD017 *all* the way around the text and the asterisks have been
         MSD017 removed from the "all ok" message.  (I wanted them to be
         MSD017 more distinct from each other.)
19980306 SFP016 Made WARN-UNRESOLVED louder, and moved it to end of FINIS-REPORT
19980127 MSD015 Added an ALIGN(T) before anything else is done in FINIS.
         MSD015 If the dictionary isn't aligned after the last item, the
         MSD015 target image won't sign-on (on the 68K and H8/300H, anyway).
19971113 SFP014 Converted for SECTIONS memory model
19971003 JL013  Modified for PFW V2.100.
19970221 JL012  Re-factored CROSS-COMPILE for partial compilation.
19970216 SFP011 Added report of count of redefinitions
19970110 SFP010 Removed dynamic cell size changes
         SFP010 Modified laying or Initialised RAM table
         SFP010 Cleaned up FINIS and friends
19961119 JL009  Added support for ['] on imports.
19961118 JL008  Support for batch and interactive mode.
19961030 JL007  Use MODULE: instead of CROSS-COMPILE, takes a base
         JL007  filename for (nearly) all files. Multiple modules.
19961017 JL006  Added external symbol support.
19961017 JL005  Tidied compiler termination.
19961003 JL004  Made END-MODULE synonym for FINIS.
19960929 SFP003 Changed EPF to MDF
19960829 JL001  *** Currently Disabled ***
         JL001  Added CHECK-UNRES-TOKENS in (FINIS) which will build
         JL001  standalone procedures for any forward reference to a token
         JL001  and resolve the forward references.
))


\ file control

only forth also c-c definitions
decimal


\ **************************
\ Finishing off vocabularies
\ **************************

HEX

: finishVocs	\ -- ; SFP036
\ Flush all vocabulary/wordlist chains to RAM.
  VOC-LINK(S) @(m)                      \ top of chain
  BEGIN
    vl- >r                              \ voc-link points to voc-link
    r@ resolved?  r@ t-vp+ @(s) 0<> and \ symbol resolved and has target version?
    IF
      r@ t-nfa+  r@ t-vp+ @(s)          \ thread tops, source/dest
      r@ #threads(t)+ @(s) 0 ?do	\ for all threads
        over @(s)  over !d(t)           \ copy over
        swap cell+  swap cell(t) +      \ step on
      loop
      2drop
    ENDIF
    r> vl+ @(s) DUP 0=			\ get next voc link
  UNTIL
  drop
;

: setLabelData	\ value label -- ; SFP036
\ If the label is defined, set the data value into the target.
  dup resolved?				\ If target has place for this
  if  @(s) !c(T)  else  2drop  endif	\ set data
;

: FINIS-VOC     \ -- ; initialise target vocabs and pointers
  finishVocs				\ SFP036
  VOC-LINK(t) @(m) the-label INIT-VOC-LINK setLabelData
  WID-LINK(t) @(m) the-label INIT-WID-LINK setLabelData
;

: BootFinis-Voc	\ --  ; SFP036
  Finis-Voc  ;

: AppFinis-Voc	\ -- ; initialise target vocabs and pointers ; SFP036
  finishVocs				\ vocs and wordlists
  VOC-LINK(t) @(m) the-label APP-INIT-VOC-LINK setLabelData
  WID-LINK(t) @(m) the-label APP-INIT-WID-LINK setLabelData
;



\ **********************************************
\ Finishing off memory, ROM/RAM and RAM versions
\ **********************************************

variable idata-flushed          \ true after ram table laid ; MSD019
variable ram-table-size         \ holds size of ram table

: lay-ram-table \ -- ; lay down the initialised ram table ; DIRECTIVE SFP014
\ *** Needs modification to allow for 16 and 32 bit cell addressed machines
\ *** For this, see the 320C31 compiler
  idata-flushed @ 0= if                 \ if not already laid ; MSD019
    ?aligning @(m)			\ SFP035
    if  calign(t) ialign(t)  endif	\ SFP035 SFP037
    chere(t)                            \ start of table
    CHERE(T) the-label INIT-RAM resolve \ resolve INIT-RAM equate for use by target code
    lay-idata                           \ lay all IDATA sections
    idata-flushed on                    \ mark job as done ; MSD019
    chere(t) swap - ram-table-size !    \ save size of table
  endif
;

: -ram-table	\ --
\ *G Prevent the generation of an initialised RAM table.
  idata-flushed on
;

variable app-idata-flushed	\ true after ram table laid
variable app-ram-table-size	\ holds size of ram table

: app-lay-ram-table	\ -- ; lay down the initialised ram table ; SFP036
\ *** Needs modification to allow for 16 and 32 bit cell addressed machines
\ *** For this, see the 320C31 compiler
  app-idata-flushed @ 0= if		\ if not already laid
    ?aligning @(m)
    if  calign(t) ialign(t)  endif
    chere(t) dup			\ start of table
    the-label APP-INIT-RAM resolve	\ resolve APP-INIT-RAM equate
    lay-idata                           \ lay all IDATA sections
    app-idata-flushed on		\ mark job as done
    chere(t) swap - app-ram-table-size !  \ save size of table
  endif
;

: -app-ram-table	\ --
\ *G Prevent the generation of an initialised RAM table.
  app-idata-flushed on
;

harvard? [if]   \ SFP032
variable <code/data>    \ -- addr ; holds section used as shared CODE/DATA

: shared-here(t)        \ -- addr(t) ; returns HERE in shared CODE/DATA section
  <code/data> @ if			\ if defined
    [ also sections.voc ]
    <code/data> @ here(ms)              \ return here in that section
    [previous]
  else
    chere(t)                            \ otherwise use current CDATA section
  endif
;

: FINIS-ROM     \ -- ; finishes mixed ROM/RAM ; SFP036
  shared-here(t) the-label INIT-FENCE setLabelData
  shared-here(t) the-label INIT-DP setLabelData
  ihere(t)       the-label INIT-RP setLabelData
  lay-ram-table                         \ add initialised RAM to end of the section
;

: AppFinis-ROM	\ -- ; finishes mixed ROM/RAM ; SFP036
  shared-here(t) the-label APP-INIT-FENCE setLabelData
  shared-here(t) the-label APP-INIT-DP setLabelData
  ihere(t) the-label APP-INIT-RP setLabelData
  app-lay-ram-table			\ add initialised RAM to end of the section
;
[else]		\ non-Harvard
variable TargetFlash	\ non-zero if target can compile to Flash
: FINIS-ROM     \ -- ; finishes mixed ROM/RAM ; SFP036
  TargetFlash @ if
    TargetFlash @ the-label INIT-FENCE setLabelData
    TargetFlash @ the-label INIT-DP setLabelData
    ihere(t)      the-label INIT-RP setLabelData
  else
    ihere(t) the-label INIT-FENCE setLabelData
    ihere(t) the-label INIT-DP setLabelData
  endif
  lay-ram-table                         \ add initialised RAM to end of the section
;
: AppFinis-ROM	\ -- ; finishes mixed ROM/RAM ; SFP036
  ihere(t) the-label APP-INIT-FENCE setLabelData
  ihere(t) the-label APP-INIT-DP setLabelData
  app-lay-ram-table			\ add initialised RAM to end of the section
;
[endif]

: FINIS-RAM     \ -- ; finishes RAM only systems
  FENCE(T) @(m) ?dup 0=			\ if not marked, use HERE
  IF  CHERE(T) ENDIF
  the-label INIT-FENCE setLabelData
  chere(t) the-label INIT-DP setLabelData
;

: flush-Idata	\ -- ; Flush all IDATA to end of CDATA ; MSD019
  idata-flushed @ 0= if			\ if not already laid
    finis-voc
    ?Rom @(m)
    If  finis-rom  Else  finis-ram  endif
  Endif
  idata-flushed on                      \ mark job as done
;

: AppFlush-idata	\ -- ; Flush all IDATA to end of CDATA
  app-idata-flushed @ 0= if		\ if not already laid
    appFinis-voc  appFinis-ROM
  Endif
  app-idata-flushed on			\ mark job as done
;


\ ****************************************
\ Reports issued at the end of compilation
\ ****************************************

variable starting-ms	\ -- addr
\ Holds start time of compilations

: .compilation-time     \ -- ; report compilation time
  decimal
\  #lines(cc) @ . ." lines compiled. "
  ." Compilation time was "
  get-ms starting-ms @ - abs		\ time in hundredths of seconds
  0 <# # # # [char] . hold #s #> type	\ show time
  ."  seconds."
;

: .forward-count        \ -- ; show forward reference statistics
  decimal
  pcr ." There were " #forward @(m) dup u. ." forward references"
  if
    pcr ." There are "
    #unresolved @ ?dup
    if
      ." still " u.
    else
      ." no "
    endif
    ." forward references left"
  endif
;

: warn-unresolved       \ -- ; bitch loudly if unresolved references exist
  cr  #unresolved @ if
    cr ." ******************************************************************"   \ MSD017
    cr ." * Warning: FORWARD REFERENCES - your output file(s) may not work *"   \ MSD017
    cr ." ******************************************************************"   \ MSD017
  else
    cr ." Successful compilation"                                               \ MSD017
  endif
  cr                                                                            \ MSD017
;

: ?Init-Ram     \ -- ; initialised RAM table addr. and size ; SFP010
  hex
  pcr ." Initialised RAM Table - Addr: $" the-label INIT-RAM @(s) u.
\ *** Needs modification to allow for 16 and 32 bit cell addressed machines
\ *** For this, see the 320C31 compiler
  ."   Length: $" ram-table-size @ u.
;

: .redefined    \ -- ; display number of words redefined
  cr ." There were " #redefined @(m)  decimal . ." target word redefinitions"
;

: Finis-Report  \ -- ; report other compilation details ; SFP010
  decimal
  ?resolve
  .redefined                            \ SFP011
  .forward-count
  xref-report                           \ SFP025
  ?init-ram
\  pcr pcr .symbol-space
  pcr .section-report                   \ section information
  pcr pcr .compilation-time
  pcr warn-unresolved
;


\ ************************
\ Useful termination words
\ ************************

variable starting-depth	\ -- addr
\ Holds stack depth at CROSS-COMPILE

: ?StackJunk	\ -- ; SFP029 SFP035
\ Tell user if there is garbage on the stack
  depth starting-depth @ <> if		\ stack changed?
    pcr
    pcr ." WARNING: Stack depth changed during compilation"
    pcr ."   at start: " starting-depth @ .
    pcr ."   at end  : " depth .
    pcr
  endif
;

: ?logWarn	\ -- ; issue warnings to log
  logout @(m) if
    close-log  .section-report  warn-unresolved  ?StackJunk
  endif
;

\ link | xt | counted string

variable AfterChain	\ -- addr
\ *G Anchor for chain of scripts executed later.

: runAfter$	\ link -- link
  dup 2 cells + count evaluate
;

: Afterwards	\ -- ; afterwards ." hello 1" cr
\ *G The following text up to the end of the line is saved and will
\ ** later be executed when *\fo{FINIS} is run.
  align here			\ start structure
  0 ,  ['] runAfter$ ,		\ link and action
  0 word count $,		\ counted string
  AfterChain AddEndLink		\ add structure to chain.
;

: doAfterwards	\ --
\ *G Execute the actions defined by *\fo{Afterwards}.
  AfterChain execChain
;

: (Finis)       \ -- ; does most of the termination work,
                \ used by FINIS and UMBILICAL-FORTH
  new-page
  finis-voc
  ?Rom @(m)  If  finis-rom  Else  finis-ram  Endif
\ debug
  ?generate-checksums
  finis-report                          \ SFP010
  ?StackJunk				\ SFP029
\ ticks >r
  save-sections
\ cr ." Section save takes " ticks r> - . ." ms"
  save-xref                             \ save XREF info  SFP025
\  Emu>Target                            \ return emulator to host mode
  finish-relocation
  ?markEndOfSource(t)			\ update LOCATE information ; SFP038
  ?logWarn
  cr
\  doAfterwards
;

variable <interactive>  \ -- addr ; true if compiler is to remain interactive   \ SFP027
  <interactive> off

: init-QI	\ --
\ *G Initialise for interactive QUIT.
  s0 @ sp!                            \ bullet proofing
  STATE(T) off(m)                     \ force interpretation
  blk off                             \ input from keyboard
;

: Quit-interactive      \ --
  init-QI
  BEGIN
    r0 @ rp!				\ N.B. host return stack
    cr query
    ['] interpret(t) catch ?dup 0= if	\ deal with line
      interp?                           \ see if prompt needed
      IF  .prompt  ENDIF
    else
      .throw  QuitHook  init-QI
    endif
  AGAIN
;

: runQI		\ --
\ Run interactive QUIT.
  only forth  also c-c			\ set search order
  assign quit-interactive to-do quit	\ error recovery XCompiler
  assign interpret(cc) to-do interpret(t)
  cr ."  ok" quit-interactive		\ host's interpreter
;

: interact/bye	\ --
  <interactive> @ if
    cr ." Interactive mode started"
    cr
    runQI
  else
    close-sections
    term-xref                           \ close XREF    SFP025
    xc-exit                             \ exit program
  endif
;

: Finis		\ --
  ?aligning @(m)                                                                \ SFP022
  if  align(t)  endif                                                           \ MSD015
  (finis)                               \ finish cross compiling
  doAfterwards
  ?releaseFiles				\ SFP034
  interact/bye
;


((
also cc/i

: interactive   cc/i    \ -- ; tell compiler to stay alive after FINIS          \ SFP027
  <interactive> on
;

: Finis         cc/i    \ -- ; the compiler directive, so in INTERPRETER vocab  \ SFP027
  finis  ;

: bye           cc/i    \ -- ; extended for interactive mode ; SFP027
  <interactive> @
  if  close-sections  term-xref  endif
  bye
;

: End-Module    cc/i    \ -- ; synonym
  [+interpreter] finis [previous]
;

previous definitions
))

\ **************************
\ Boot/Application splitting (for CDS)
\ **************************

: must-be-rom
  ?Rom @(m) 0= abort" Split ROM/RAM system required"  ;

: BootFinis	\ -- ; SFP036
  must-be-rom
  finis-voc  finis-rom
  ?generate-checksums -checksums
  finis-report                          \ SFP010
  ?StackJunk				\ SFP029
  save-sections
  finish-relocation
  cdata write-fail
;

: AppFinis	\ -- ; SFP036
  must-be-rom
  ?aligning @(m)			\ SFP022
  if  align(t)  endif                   \ MSD015
  new-page
  appFlush-Idata
  ?generate-checksums -checksums
  the-label APP-INIT-RAM @(s)  the-label INIT-RAM !(s)
  app-ram-table-size @ ram-table-size !
  finis-report                          \ SFP010
  ?StackJunk				\ SFP029
  save-sections
  save-xref                             \ save XREF info  SFP025
  finish-relocation
  ?logWarn
  cr
  ?releaseFiles				\ SFP034
  interact/bye
;

((
also cc/i
: BootFinis	cc/i	\ --
  BootFinis  ;
: AppFinis	cc/i	\ --
  AppFinis  ;
previous definitions
))

\ *****************************
\ Cross compiler initialisation
\ *****************************

only forth also c-c definitions

\ set target type dependencies for compiler, e.g. 16bit versus 32bit
: Assign-Target/XC      \ -- ; set target type specifics
  assign-code-drivers           \ code space memory drivers
  assign-data-drivers           \ data space memory drivers
  assign-relocation-types       \ relocation types
  assign-convert-double         \ double number handling
;

only forth also c-c also forth definitions

: (CROSS-COMPILE)       \ -- ; initialise for cross compilation
  depth starting-depth !		\ SFP035
\  bl-word/uc                                   \ grab base name
\  dup $output-file                             \ set IMG filename
\  dup $output-mdf                              \ set MDF filename
\  dup extsym-set-filename                      \ set SYM filename
\  extexp-set-filename                          \ set DEF filename for exports
\  assign error-exit to-do abort                 \ errors exit compiler
\  assign xc-error to-do error                   \ set up error handler
  assign interpret(cc) to-do interpret(t)       \ set up target interpreter
  assign interpret(t) to-do interpret                   \ JL013
\  assign ?show-new-file(cc) to-do ?show-new-file        \ JL013
\  assign ?show-resume-file(cc) to-do ?show-resume-file  \ JL013
\  target-32b byte-swap-off                      \ need to do this first as memory drivers depend on values     del SFP014
  assign-target/xc                              \ set up target dependencies
  init-core                                     \ initialise compiler core
  [+assembler] reset [previous]                 \ initialise assembler
  init-relocation                               \ initialise relocation record handling
\  init-target-memory                           \ initialise target memory address handling
\  init-extsymbols                              \ JL006, initialise external symbol table
\  init-extexports                              \ JL007, initialise external exports
  init-requireds                                \ initialise all predefined symbols
\  DB-init                                      \ initialise database handling
\  TLV-Init                                     \ initialise TLV creation
\  module-init                                  \ initilaise module handling
\  socket-init                                  \ initialise socket handling
\ no-heads                                      \ never need heads (keep as directive for development)
  open-log                                      \ open log file if necessary
  ([)                                           \ target in interpreting mode
  get-ms starting-ms !                          \ save start time
  in-umbilical off(m)                           \ in cross compile mode, not umbilical forth mode
  logs-per-line @(m) 1 < if                     \ If the LOGS-PER-LINE  value is "illegal", \ MSD021
    LOG/LINE logs-per-line !(m)                 \ then replace it               \ MSD021
  endif                                         \ with the default value.       \ MSD021
  new-page pcr                                  \ new page for start of compiliation
  init-locate(s)                                \ SFP024
  ?markStartofSource(t)				\ SFP038
;

: CROSS-COMPILE         \ --
  (cross-compile)
;


\ *********************
\ Command line switches
\ *********************

only forth also c-c also forth definitions

: /PAUSEOFF     \ -- ; run in batch mode
  interactive-run off(m)
;

: /IDE          \ -- ; run from IDE host
  ide-hosted on(m) locate-via-aide	\ SFP028
;

: /PAGEOFF      \ -- ; inhibit page-throw drivel
  want-pages off(m)
;

: /COLS         \ cols -- ; specify number of log columns per line
  logs-per-line !(m)
;

: /-PAUSES	\ -- ; stop pauses in console
  0 to PauseConsole
;

: /+PAUSES	\ -- ; enable pauses in console
  op-handle @ to PauseConsole
;


decimal

assign NCCABORT to-do .cold

only Forth definitions

