\ Library file handling

((
File: XLIBS.FTH

MPE Forth Cross-Compiler v6

Copyright (c) 1998, 2008

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: http://www.mpeforth.com

Original release January 1998

20080605 SFP003 DocGenned.
20000523 SFP002 [RREQUIRED] returns true for unresolved LABELs
19980305 MSD001 Cosmetic change to stop some of the messages messin'
         MSD001 up the log file layout.


))

\ =========
\ *> interp
\ *S Library files
\ =========
\ *P A library file contains words which are compiled only if
\ ** required, i.e. they have been used but not yet defined.
\ ** Library files are scanned between the directives *\fo{LIBRARIES}
\ ** and *\fo{END-LIBS}. The code between *\fo{LIBRARIES} and
\ ** *\fo{END-LIBS} is repeatedly interpreted until the number
\ ** of forward references and unresolved symbols remains
\ ** constant. For example:
\ *E libraries
\ **   include libfile1.fth
\ **   include libfile2
\ ** end-libs


only forth definitions


\ ***********************************
\ Text input handling
\ Should be moved to FROMN.SCR in PFW
\ ***********************************

#16 cells constant /ipNest	\ -- u
\ *G Max Staorage size per source nesting
16 constant #ipNests	\ -- u
\ *G Maximum number of nesting levels.
/ipNest #ipNests * constant #ipnesting	\ -- u
\ *G Size of storage array

#ipNesting buffer: ipArray[]	\ -- addr
\ *G Array in which source input positions are saved
  iparray[] #ipnesting erase

ipArray[] value CurrIP[]	\ -- addr
\ *G Points at next free position.

(( in Hmodel.fth
: n!		\ xn..x1 n addr --
\ *G Save n items at addr including n.
  over 1+ cells bounds
  do  i !  cell +loop
;

: n@		\ addr --
\ *G Retrieve n items saved by N!.
  dup @ cells over +		\ -- start end
  do  i @  cell negate +loop
;
))

: saveip        \ -- ; save input position
  save-input CurrIP[] n!  /ipNest add CurrIP[]
;

: ?IpError	\ flag -- ; error if set
  abort" Source input position nesting error"
;

: discardip     \ -- ; discard saved input without reusing it
  CurrIP[] ipArray[] u> 0= ?IpError
  /ipNest negate add CurrIP[]
;

: getIP[]	\ addr --
\ *G Reload input position from buffer.
  n@ restore-input ?IpError
;

: restoreip     \ -- ; restore input position
  discardip CurrIP[] getIP[]
;

: reuseip       \ -- ; go back to previous position without adjusting array
  CurrIP[] /ipNest - getIP[]
;

: [FOR]         \ n -- n ; interpretive loop
  saveip
;  immediate

: [NEXT]        \ n -- n' | ; leave if n=0
  1- ?dup
  if  reuseip  else  discardip  endif
;  immediate

: [BEGIN]       \ -- ; interpretive loop
  saveip
;  immediate

: [UNTIL]       \ flag -- ; interpretive loop
  if  discardip  else  reuseip  endif
;  immediate



\ ******************************
\ counting unresolved references
\ ******************************

only forth also c-c definitions

: count-unres   \ symbol --
  cell+ @(s)                            \ forward references outstanding?
  if  #unresolved Incr  endif
;

((
Compared to ?UNRES and friends in XDTC41-3,FTH, this counting system
allows for automatic handling of defining words, which includes user
defined ones. Some extension is needed for display of names for the
run time reporting. This version is completely "quiet" as it is used
by the library loop manager.
))
: #unres(v)     \ voc(s) -- ; display unresolved references
  #threads(s)+ dup cell+  swap @(s) cell*  bounds
  Do
    i                                   \ for each thread
    BEGIN
      @(s) ?dup                         \ -- nfa [nfa]
     WHILE
      dup name>(s)                      \ -- nfa addr(s)
      dup resolved?                     \ symbol defined ?
      if
        drop                            \ yes, chuck data
      else
        dup count-unres                 \ no, count it
        dup def-word?                   \ defining word?
        if
          2 cells+ count-unres          \ check for runtime defined
        else
          drop
        endif
      endif
      n>link(s)                         \ switch to link
    REPEAT
  cell +loop
;

: #unres        \ -- #unres
  #unresolved @  #unresolved off
  'LABELS(s) @(m) #unres(v)             \ labels
  VOC-LINK(s) @(m)                      \ vocabularies
  BEGIN
    ?DUP                                \ while they exist
   WHILE
    dup vl- #unres(v)                   \ count unresolved
    @(s)                                \ get next one
  REPEAT
  #unresolved @
  swap #unresolved !
;


\ ***************************
\ Words available to the user
\ ***************************

((
The code between LIBRARIES and END-LIBS is repeatedly interpreted
until the number of forward references and unresolved symbols
remains constant.
))

: needed?	\ symbol -- flag ; true if has forward refs
  dup resolved? 0= if			\ still unresolved
    cell+ @(s) 0<>			\ and has outstanding references
  else
    drop 0
  endif
;

only forth  also c-c  also cc/i

: libraries     cc/i    \ -- 0 #unres #forward ; interpretive loop
\ *G Starts repetitive execution of the code up to *\fo{END-LIBS}.
  pcr
  pcr ." Starting library scan "
  saveip  0  #unres  #forward @(m)
;  immediate

: end-libs      cc/i    \ pass# #unres #forward -- pass# #unres' #forward'
\ *G Ends the block started by *\fo{LIBRARIES}.
  rot 1+ -rot                           \ increment pass counter
  pcr ." End of library pass " 2 pick . \ tell user
  #unres  #forward @(m)                 \ new data
  2swap 2over d=                        \ same if no change
  if
    pcr ." Library scanning finished "
    pcr
    2drop drop  discardip               \ no change, get out
  else
    reuseip                             \ changed, go round again
  endif
;  immediate

: [required]    cc/i    \ "<name>" -- true
\ *G Returns true if symbol exists and is forward referenced.
\ ** All words in the library file are usually defined in the
\ ** form:
\ *E [required] foo [if]
\ ** : foo  ...  ;
\ ** [then]
  bl-word/uc find(t)_ans if             \ symbol must exist
    needed?
  else
    'LABELS(s) @(m) v-find(t) if
      needed?
    else
      drop  0
    endif
  endif
;

c-c  only forth also c-c definitions


\ ***********************
\ Library facilities test - N.B. target code
\ ***********************

((
: foo bar1 bar2 bar3  ;

libraries
  [required] bar1 [if]
  : bar1 ;
  [then]
end-libs
))

