\ COMPILER.FTH  Main compiler file, supports "core" Forth word set.

((

J Lee   18 Sep 96

Based on:
        OTA Specification - Volume 1
        September 1996

Copyright (c) 1993-96, 1998, 1999, 2000, 2012, 2016, 2019, 2020

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
email: mpe@mpeforth.com


20200405 SFP046 Added integers to POSTPONE.
20191103 SFP045 More host v5.x changes
20190614 SFP044 Host v5.x changes
20190613 SFP043 64 bit updates.
20160309 SFP042 Added Z" and Z\" string words.
20120612 SFP041 Added S\" and C\" escaped string versions of S" and C"
SFP040	18 Oct 2000	Reorganised string handling for token compiler and for
			upcoming optimisations.
			Corrected Umbilical handling of DOES> and ;CODE
SFP039	18 June 2000	Added ADDR handling for VALUE
SFP038	30 May 2000	Split ; and EXIT handling to C_; and C_EXIT
SFP037  10 Mar 2000	Fixed :NONAME handling
SFP036	20 Sept 1999	Changed COMPILE to permit more kernel freedom
SFP035  5 June 1999     Removed long standing bug in ;CODE which prevented use of
                        children in colon definitions; caused by the use of smudging in
                        previous change JL001.
MSD034  20 Nov 1998     Removed CHARS - it's now in XOPT*.FTH.
SFP033  16 Nov 1998     XREF updates
SFP032  11 Nov 1998     Updates for Harvard Architectures
SFP031  27 Sep 1998     Added inlining control
SFP030  20 Aug 1998     Added switch for runtime resolution of defining words
SFP029  2 July 1998     Moved XC: and children to the XGEN/XOPTxxx file

        Changes
SFP     21 Apr 96       Corrected /MOD */MOD */

SFP     23 Apr 96       Added IF( )ELSE( )ENDIF

SFP     30 Apr 96       Modified DOES> to lay CLASSPROC token

SFP     13 May 96       Added support for (D ... D) as comments
                        Requires COMPILER to be loaded before INTERP

SFP     15 May 96       Fixed ALIGNED

JL001   22 August 96    Added smudge support for colon definitions (see ;).

JL002   23 August 96    Made selection between ANS and MPE versions of control structure words
                        a run time rather than compile time decision.

JL003   27 August 96    Added conditional compilation words [IF] [ELSE] [THEN] and [DEFINED].

JL004   11 Sep 96       Upgraded tokens to V2, reorganizing the compliation order.

SFP005  29 Sep 96       Used TOKEN,(T) to allow TARGET-TOKEN: to create all tokens.

JL006   30 Sep 96       Changed USERVAR to take a LIT not an inline value.

JL007   1 October 96    Fixed 2/.

JL0910  9 October  96   Added 'under the hood' protocol tokens.

JL008   24 October 96   Added compiler compatibility layer.

JL009   25 October 96   Fixed user variable assignments.

JL010   25 October 96   Reinstalled NEGATE as a token.

JL011   5 November 96   Fixed [UNDEFINED].

JL012   5 November 96   Added THROW for DEVWRITELIT in DEBUG".

JL013   5 November 96   Fixed 2>R 2R> and 2R@ in b-d mode.

JL014   8 November 96   Added a CR to DEBUG".

JL015   12-13 Nov 96    Minor updates to volume 2 draft 4.

SFP016  15 Jan 97       Added ?FINISH-LOCALS(T) to ; DOES> ;CODE and EXIT
                        Modified TO

JL017   17 January 97   Added THERE.

JL018   21 January 97   Added ,(R).

SFP019  31 Jan 97       Added METHODS>

SFP020  19 Feb 97       Removed compilation of EXIT after SET-METHODS in METHODS>

SFP021  14 Mar 97       Moved TARGET-SYM <name> COMPILE-REFs to GENERATE.FTH

JL022   24 Sep 97       Added extra uses of the GENERATE words.

JL023    7 Oct 97       Added COMPILE that now becomes effectively an IMMEDIATE
                        word.  COMPILE now forces a subroutine call to be laid
                        for the following word that will be COMPILEd.

JL024   20 Oct 97       Added C".

MSD025  26 JAN 98       Add calls to ?BL-EVEN after invocation of ",C(T).
                        (Code pointer alignment is now done by the *caller*
                        of ",C(T) instead of by ",C(T) itself.)

MSD026  18 FEB 98       Added [ENDIF].

SFP027  28 Feb 1998     Converted Forth structures to use C_MRK_BRANCH< and >C_RES_BRANCH

SFP028   6 March 1998   Added code generation for >R R@ R>
                        Restored code generation of INCR and DECR

))


only forth also c-c definitions


\ ************************************************
\ ****    Error handler for target actions    ****
\ ************************************************

: Target-Compilation-Error  ( $addr -- ; this aborts compilation with error message )
  cr cr ." Can't compile " $. ."  in target"  ABORT
;


only forth also c-c also compiler definitions  c-c


\ ***********************************
\ ****    Misc. memory access    ****
\ ***********************************

required: !                                     \ SFP033
: !  cc/c                                       \ SFP033
  lay-target?
  If  c_!  [last-req]  Endif                    \ SFP033
  ?b-d
  If  postpone dup  postpone data-rel-value  postpone !d(t)  Endif
;

64bit? [if]
required: L!                                    \ SFP033
: L!  cc/c
  lay-target?
  If  c_l!  [last-req]  Endif
  ?b-d  If  postpone l!d(t)  Endif
;
[then]

\ JL022
required: W!                                    \ SFP033
: W!  cc/c
  lay-target?
  If  c_w!  [last-req]  Endif
  ?b-d  If  postpone w!d(t)  Endif
;

required: +!                                    \ SFP033
: +!    cc/c
  lay-target?
  If  c_+!  [last-req]  Endif                   \ SFP033
  ?b-d  If  postpone dup  postpone data-rel-value  postpone +!d(t)  Endif
;

64bit? [if]
required: L+!
: l+!  cc/c
  lay-target?
  If  c_l+! [last-req]  Endif
  ?b-d If
    postpone dup  postpone l@d(t)  postpone rot  postpone +  postpone swap
    postpone l!d(t)
  Endif
;
[then]

16bit? 0= [if]
required: W+!                                   \ SFP033
: w+!  cc/c
  lay-target?
  If  c_w+! [last-req]  Endif                   \ SFP033
  ?b-d If
    postpone dup  postpone w@d(t)  postpone rot  postpone +  postpone swap
    postpone w!d(t)
  Endif
;
[then]

\ JL004
required: c+!                                   \ SFP033
: c+!  cc/c
  lay-target?
  If  c_c+! [last-req]  Endif                   \ SFP033
  ?b-d
  If
    postpone dup postpone c@d(t) postpone rot postpone + postpone swap
    postpone c!d(t)
  Endif
;

required: ,                                     \ SFP033
: ,  cc/c
  lay-target?
  If  c_,  [last-req]  Endif                    \ SFP033
  ?b-d  If  postpone here(t)  postpone data-rel-value  postpone g,(t)  Endif
;

harvard? [if]   \ SFP032
required: ,(R)                                  \ SFP033
: ,(r)  cc/c
  lay-target?  If  c_,(r)  [last-req]  Endif    \ SFP033
  ?b-d  If  postpone ihere(t)  postpone data-rel-value  postpone ,d(t)  Endif
;
[else]
required: ,(R)                                  \ SFP033
: ,(r)  cc/c
  lay-target?  If  c_,  [last-req]  Endif       \ SFP033
  ?b-d  If  postpone ihere(t)  postpone data-rel-value  postpone ,d(t)  Endif
;
[endif]

\ !! required: CHARS                                 \ SFP033                   \ MSD034
\ !! : Chars  cc/c  [last-req]  ;                    \ does *1 SFP033           \ MSD034

\ JL017  SFP032
Harvard? [if]
required: THERE                                 \ SFP033
: There  cc/c
  lay-target?
  if  c_THere  [last-req]  endif
  ?b-d
  If  postpone ihere(t)  Endif
;
[else]
required: THERE                                 \ SFP033
: There  cc/c
  lay-target?
  if  c_THere  [last-req]  endif
  ?b-d
  If  postpone ihere(t)  Endif
;
[endif]


\ ******************************
\ ****    Defining words    ****
\ ******************************

required: CREATE                                \ SFP033
: Create cc/c
  <:noname> @ if				\ SFP037
    c_create  [last-req]
    exit
  endif

  2 zeros,(s)                                   \ expand reference packet
  'latest: @(m) status+ mark-def-word           \ mark as defining word
  do-build @(m) if                              \ is image-in-host allowed ?
    sc/c  interpreter definitions
      word-down context-find 0= if              \ if no analogue exists
        (head-create)                           \  start one
        postpone create(cc)
      Else
        drop
      Endif
      rc/c
  Endif
  lay-target?					\ SFP040
  if  c_create  [last-req]  endif		\ SFP033 SFP040
;

\ JL004
\ required: DOES>                         \ SFP033
: Does>  cc/c
\ cr ." DOES> cc/c " chere(t) .dword
  <:noname> @ if			\ SFP037
    not-inlined ?finish-locals(t)
    c_(;code)  [last-req]
    dodoes,
    exit
  endif

  not-inlined                           \ inlining not permitted        SFP031
  ?finish-locals(t)                     \ SFP016
  ?csp  ?b-d if                         \ if in create ... does> build host analogue
    postpone DOES>(cc)
   'LATEST: @(m) ,   			\ needs symbol address of the defining word
\    ;host,
    postpone ;
    lay-target?				\ SFP040
    if  c_(;code)  [last-req]  endif
    0 state!
  else
    lay-target?				\ SFP040
    if  c_(;code)  [last-req]  endif
  Endif
\ cr chere(t) .dword
  do-runtime @(m) if
    chere(t) 'Latest: @(m) resolve-def-word     \ mark run time action as being here
  endif
\ cr chere(t) .dword
  dodoes,
;

\ required: INTERP>
: interp>  cc/c
  [+compiler] DOES> [previous]
;

required: ;CODE                         \ SFP033
: ;code  cc/c   \ ; reorganised SFP035 SFP040
  <:noname> @ if			\ SFP037
    not-inlined  ?finish-locals(t)
    c_(;code)  [last-req]
    2 state(t) !(m)  sp@ asm_start initfwd
    exit
  endif

  not-inlined                           \ inlining not permitted        SFP031
  ?finish-locals(t)                     \ SFP016
  ?csp  ?b-d IF
    postpone DOES>(cc) 'LATEST: @(m) ,   \ needs symbol address of the defining word
    ;host,
\    ( compile EXIT ) postpone exit
    lay-target?				\ SFP040
    if  c_(;code)  [last-req]  endif
    0 state!
  else
    lay-target?				\ SFP040
    if  c_(;code)  [last-req]  endif
  ENDIF
  'latest: @(m) status+ clear-smudged   \ SFP035 make symbol visible to ?TARGET
  do-runtime @(m) if
    chere(t) 'Latest: @(m) resolve-def-word     \ mark run time action as being here
  endif
  2 state(t) !(m)
  sp@ asm_start initfwd
;

: Methods>      cc/c    \ -- ; e.g. : FOO CREATE ... METHODS> @ ! ;
  not-inlined                           \ inlining not permitted        SFP031
  make-methods                          \ build packet
  ?b-d if                               \ if in create ... does> build host analogue
    postpone set-methods  dup ,          \ will mark created symbol as a method, symbol address in-line
  Endif
  drop
  ?finish-locals(t)
;


\ ******************************************
\ ****  Comments and string building    ****
\ ******************************************

: (     cc/c
  postpone (
;

: \     cc/c
  postpone \
;

: ((  cc/c  ( -- )
  postpone ((          \ This comment here just to keep my editor happy! ))
;

required: ."                            \ SFP033
: ."	cc/c		\ SFP040
  not-inlined  >in @
  lay-target?
  if  c_." [last-req]  endif
  ?b-d
  if  >in !  postpone ."  else  drop  endif
;

required: S"                            \ SFP033
: S"  cc/c	\ SFP040
  not-inlined  >in @
  lay-target?
  if  c_s"  [last-req]  endif
  ?b-d
  if  cr ." Can't do this in a defining word" ABORT  else  drop  endif
;

required: C"                            \ SFP033
: C"  cc/c	\ SFP040
  not-inlined  >in @
  lay-target?
  if  c_c"  [last-req]  endif
  ?b-d
  if  >in !  postpone c"  else  drop  endif
;

required: Z"                            \ SFP042
: Z"  cc/c	\ SFP042
  not-inlined  >in @
  lay-target?
  if  c_z"  [last-req]  endif
  ?b-d
  if  >in !  postpone z"  else  drop  endif
;

\ SFP041 ...
required: S\"
: S\"		cc/c	\ --
  not-inlined  >in @
  lay-target?
  if  c_s\"  [last-req]  endif
  ?b-d
  if  cr ." Can't do this in a defining word" ABORT  else  drop  endif
;

required: C\"
: C\" 		cc/c	\ --
  not-inlined  >in @
  lay-target?
  if  c_c\"  [last-req]  endif
  ?b-d
  if  >in !  postpone C\"  else  drop  endif
;
\ ...SFP041

required: Z\"		\ -- ; SFP042
: Z\" 		cc/c	\ -- ; SFP042
  not-inlined  >in @
  lay-target?
  if  c_z\"  [last-req]  endif
  ?b-d
  if  >in !  postpone Z\"  else  drop  endif
;

required: ""                            \ SFP033
: ""  cc/c	\ SFP040
  not-inlined  >in @
  lay-target?
  if  c_c"  [last-req]  endif
  ?b-d
  if  >in !  postpone c"  else  drop  endif
;

required: ABORT"			\ SFP033
: abort"  cc/c	\ SFP040
  not-inlined  >in @
  lay-target?
  if  c_abort"  [last-req]  endif
  ?b-d
  if  >in !  postpone abort"  else  drop  endif		\ "
;


((
*******************************
        deferred words
*******************************
))

required: ASSIGN                                \ SFP033
: assign  cc/c
  ['(t)]  [last-req]
;

required: TO-DO                                 \ SFP033
: to-do  cc/c
  not-inlined                           \ SFP031
  c_(to-do)  [last-req]                 \ SFP033
;


\ ***********************************
\ ****    Miscellaneous words    ****
\ ***********************************

required: ASCII                                 \ SFP033
: ascii  cc/c
  bl word 1+ c@ general-literal  [last-req]     \ SFP033
;

\ JL004, JL008
required: ;                                     \ SFP033
: ;  cc/c
  ?finish-locals(t)                             \ SFP016
  ?csp
  c_;  [last-req]				\ SFP033, SFP038
  end-inline                                    \ mark end of inline code       \ SFP031
  ?b-d  If
    ;host,
\    ( compile EXIT ) postpone exit
  Endif
  <:noname> @ 0= if			\ SFP037
    'latest: @(m) status+ clear-smudged	\ JL001 make symbol visible to ?TARGET
  endif
  ([)  0 state!
  in-host-build @(m) If
    host-mode on(m)			\ if we're in host building mode then switch on mechanism again
    only forth also c-c also extensions definitions     \ set default host building order
    reveal                      	\ have to make host word visible ; SFP045
\    smudge
  Endif
;

: [     cc/c    ( -- )
  ([)
;

required: [']                                   \ SFP033
: [']   cc/c
  ['(t)]  [last-req]                            \ SFP033
;

required: [CHAR]                                \ SFP033
: [Char]  cc/c
  bl word 1+ c@ general-literal
  [last-req]                            \ SFP033
;

required: [COMPILE]                              \ SFP033
: [compile]  cc/c
  ([compile])  [last-req]               \ SFP033
;

required: COMPILE                               \ SFP033
((
: compile  cc/c
  not-inlined                           \ SFP031
  c_compile  [last-req]                 \ SFP033
  ([compile])
;
))
: compile  cc/c	\ -- ; SFP036
  c_compile_new  [last-req]                 \ SFP033
;


((
: >body  cc/c
  cr cr ." >BODY is not supported ... see ISSUEM52" cr ABORT
;
))

((
: find  cc/c
  lay-target?   If  c" FIND" target-compilation-error  Endif
  ?b-d  If  compile find(t)_ans  Endif
;
))

((
: immediate  cc/c
  cr cr ." IMMEDIATE is not supported ..." ABORT
;

: marker  cc/c
  cr cr ." MARKER is not supported ..." ABORT
;
))

required: EXIT                                  \ SFP033
: Exit  cc/c
  not-inlined                           \ inlining not permitted        SFP031
  ?finish-exit(t)                       \ SFP016
  c_exit  [last-req]                    \ SFP033, SFP038
  ?b-d if  ( compile EXIT ) postpone exit  endif
;

required: LITERAL                               \ SFP033
: Literal  cc/c
  general-literal  [last-req]           \ SFP033
;

\ : Find(t)_ans	\ c-addr -- c-addr 0 | addr 1 | addr -1
: postpone  cc/c
  bl-word/uc find(t)_ans dup            \ symbol exists?
  If
    over resolved?                      \    symbol resolved?
    If
      1 =                               \      symbol immediate?
      If
        compile-ref
      Else
        c_compile-word
      Endif
    Else
      cr cr ." Can't find " $. ." to POSTPONE - cannot guess IMMEDIATE!" cr cr ABORT
    Endif
  Else
    drop  dup >r  count isInteger?
    case
      0 of
        cr cr ." Can't find " r> $. ." to POSTPONE - cannot guess IMMEDIATE!"
        cr cr ABORT
      endof
      1 of  r> drop  general-literal  endof
      2 of  r> drop  swap general-literal general-literal  endof
        #-13 throw
    endcase
  Endif
;

required: RECURSE                              \ SFP033
: recurse  cc/c
  not-inlined                           \ inlining not permitted        SFP031
  lay-target?
  If  'latest(s) @(m) compile-ref  [last-req]  Endif    \ SFP033
  ?b-d  If  postpone recurse  Endif
;

required: TO                                    \ SFP033
: to    cc/c    \ -- ; process TO <name>
  bl-word/uc dup find(t)                \ target word?
  If
    dup method?                         \ must be a method
    If
      nip to-compile
    Else
      drop cr ." Can't TO: " $. .source-line ABORT
    Endif
    [last-req]                          \ SFP033
  Else
    ['] compiler v-find                 \ local variables are in COMPILER voc  SFP016
    if
      1 OperatorType(t) !  execute      \ perform operation
    else
      cr ." Can't TO: " $. ."  Illegal Forward reference" .source-line ABORT
    endif
  Endif
;

: ->    cc/c    \ -- ; synonym for TO
  [+compiler] to [previous]
;

: addr  cc/c    \ -- ; process ADDR <name> ; SFP016 SFP039
  get-token ['] compiler v-find if
    2 OperatorType(t) !  execute      \ perform operation
    exit
  endif
  find(t)_ans if
    dup method? if
      addr-compile  exit
    endif
    drop  here
  endif
  cr ." Can't ADDR: " $. ."  Illegal Forward reference" ABORT
;


\ ****************************************************
\ ****    HOST suppoprt for control structures    ****
\ ****************************************************


c-c only forth also c-c definitions

\ PFW Control structure codes

 1 constant begin_statement
 2 constant if_statement
 3 constant repeat_statement
 3 constant do_statement                \ *sfp* to be changed
 4 constant case_statement
 5 constant of_statement

only forth also c-c also cc/c


\ **********************************
\ ****    DO  LOOP structure    ****
\ **********************************

required: DO                                    \ SFP033
: DO  CC/C
  c_do  [last-req]                      \ SFP033
  ?B-D IF  postpone DO DROP  ENDIF
  3
;

required: ?DO                                   \ SFP033
: ?DO  CC/C
  c_?do  [last-req]                     \ SFP033
  ?B-D IF  postpone ?DO DROP  ENDIF
  3
;

required: LOOP                                  \ SFP033
: LOOP  CC/C
  3 ?PAIRS  ?B-D
  IF  do_statement postpone LOOP  ENDIF
  c_loop  [last-req]                    \ SFP033
;

required: +LOOP                                 \ SFP033
: +LOOP  CC/C
  3 ?PAIRS  ?B-D
  IF  do_statement postpone +LOOP  ENDIF
  c_+loop  [last-req]                   \ SFP033
;

required: LEAVE                                 \ SFP033
: LEAVE  CC/C
  c_leave [last-req]                    \ SFP033
  ?B-D IF  postpone LEAVE  ENDIF
;

required: ?LEAVE                                \ SFP033
: ?LEAVE
  CC/C
  c_?leave  [last-req]                  \ SFP033
  ?B-D IF  postpone ?LEAVE  ENDIF
;


\ **************************************
\ ****    IF ELSE THEN structure    ****
\ **************************************
\

c-c only forth also c-c definitions

: ans_IF
  c_?branch>
  ?B-D  IF  postpone if  ENDIF
;

: ans_ELSE
  ?B-D IF  postpone else >R  ENDIF
  c_branch>  swap >c_res_branch
  ?B-D IF  R>  ENDIF
;

: ans_ENDIF
  ?B-D  IF  postpone endif  ENDIF
  >c_res_branch
;

: ans_THEN
  ans_ENDIF
;

: ans_BEGIN
  c_mrk_branch<
  ?B-D IF  postpone begin  ENDIF
;

: ans_WHILE
  ?b-d If  postpone while  >r >r  Endif
  c_?branch>  swap
  ?b-d If  r> r>  Endif
;

: ans_UNTIL
  ?B-D IF  postpone until  ENDIF
  c_?branch<
;

: ans_AGAIN
  ?B-D IF  postpone again  ENDIF
  c_branch<
;

: ans_END
  ans_AGAIN
;

: ans_REPEAT
  ?b-d If  postpone again  postpone endif  Endif
  c_branch<  >c_res_branch
;

c-c only forth also c-c also cc/c


\ These are the versions of IF ELSE .. that the program uses.

\ JL002

required: IF
: IF  cc/c
  ans_if
  [last-req]
;

required: ELSE
: ELSE  cc/c
  ans_else
  [last-req]
;

required: THEN
: THEN  cc/c
  ans_then
  [last-req]
;

required: BEGIN
: BEGIN  cc/c
  ans_begin
  [last-req]
;

required: WHILE
: WHILE  cc/c
  ans_while
  [last-req]
;

required: AGAIN
: AGAIN  cc/c
  ans_again
  [last-req]
;

required: UNTIL
: UNTIL  cc/c
  ans_until
  [last-req]
;

required: REPEAT
: REPEAT  cc/c
  ans_repeat
  [last-req]
;


\ ******************************
\ ****    Case Statement    ****
\ ******************************

required: CASE                                  \ SFP033
: Case  cc/c
  ?comp  c_case  csp @  !csp  4
  [last-req]                            \ SFP033
;

\ JL004
required: OF                                   \ SFP033
: Of    cc/c
  4 ?pairs
  c_of  [last-req]                      \ SFP033
  ?b-d  If  case_statement postpone OF drop  Endif
  5
;

required: ?OF                                   \ SFP033
: ?Of  cc/c
  4 ?pairs
  c_?of  [last-req]                     \ SFP033
  ?b-d if  postpone ?OF  endif
  5
;

required: ENDOF                                  \ SFP033
: Endof cc/c
  5 ?pairs  ?b-d
  If
    of_statement postpone ENDOF drop >r
  Endif
  c_endof  [last-req]                   \ SFP033
  ?b-d  If  r>  Endif
  4
;

required: ENDCASE                               \ SFP033
: Endcase       cc/c
  4 ?pairs  ?b-d
  If  postpone drop  Endif
  c_drop
  Begin
    sp@ csp @ <>        \ sensitive to stack type *sfp*
  While
    c_endcase
  Repeat
  csp !
  c_dropcase
  [last-req]                            \ SFP033
;

required: END-CASE                              \ SFP033
: End-case      cc/c
  4 ?pairs
  Begin
    sp@ csp @ <>        \ sensitive to stack type *sfp*
  While
    c_endcase
  Repeat
  csp !
  c_dropcase  [last-req]                \ SFP033
;

required: NEXTCASE                              \ SFP033
: NextCase      cc/c
  4 ?pairs
  ?b-d
  If  postpone drop  Endif
  c_drop
  c_nextcasejump
  Begin
    sp@ csp @ <>        \ sensitive to stack type *sfp*
  While
    c_endcase
  Repeat
  csp !
  c_nextcasefix  [last-req]             \ SFP033
;


((
*******************************
*** Conditional Compilation ***
*******************************
))

: [IF]  cc/c
  postpone [IF]
;

: [ELSE]  cc/c
  postpone [ELSE]
;

: [THEN]  cc/c
  postpone [THEN]
;

: [ENDIF]  cc/c                                                                 \ MSD026
  postpone [THEN]                                                              \ MSD026
;                                                                               \ MSD026

: [DEFINED]  cc/c
  bl-word/uc find(t)_ans                \ find the symbol
  If                                    \ if found then must be resolved
    resolved?                           \
  Else                                  \ otherwise not defined
    drop FALSE                          \
  Endif
;

\ JL011
: [UNDEFINED]  cc/c
  [+compiler] [defined] [previous] 0=
;


((
*****************************
*** Jon's Recommendations ***
*****************************
))

: Endif  cc/c
  [+compiler] THEN [previous]
;


\ ****************
\ Section handling
\ ****************

: origin        cc/c    \ -- addr(t) ; returns start address of CDATA section
  origin(t) general-literal  ;

: sec-base      cc/c    \ -- addr(t) ; returns start address of current section
  bottom(t) general-literal  ;

: sec-top       cc/c    \ -- addr(t) ; returns end address of current section
  top(t) general-literal  ;


\ ******************
\ 64/32 bit literals
\ ******************

[defined] c_lit64 [if]
: lit#64	\ "<text>" --
  parse-name isNumber? 2 <> abort" Must be a double integer"
  c_lit64
;
[then]


\ ********************************************************
\ Just here because it's easier - shouldn't really be here
\ ********************************************************

xc: ",          c_",            ",c(t)


c-c only forth also c-c definitions

