\ Literal pool handling for VFX optimiser
               
(( Complex one - untested and incomplete
The literal pool consists of a list of references which are resolved
either when requested as a fixed size pool by the programmer:
  cells LITPOOL
or, if the last literal pool is unavailable because it is full or too
far away, a new one will be created at the next unconditional branch.
The format of the forward reference list is:
  cell   lp_link         pointer to previous reference
  cell   lp_value        literal value
  cell   lp_addr         address of load instruction

A minimum size for the automatically created literal pools may be specified:
  n MIN-LITPOOL
A literal that is to be handled by the literal pool system is compiled by
the word C_LITPOOL (n -- ), which first attempts to use a current literal
pool, and then makes a forward reference if the pool is not available.
))

(( Complex one               
variable <min-litpool>
variable <curr-litpool>
variable <curr-litpool-size>
variable <curr-litpool-used>
variable <lp-fwd-chain>

struct /lp-fwd
  ptr lp-link                   \ link to previous ref
  cell lp-value                 \ value of literal
  cell lp-adr                   \ address of load ins.
end-struct

defer in-lit-range?             \ addr(t) -- flag
defer c_litpoolins              \ addr(t) --

: clear-litpool  \ -- ; initialise literal pool
  <curr-litpool off  ;

: get-pool-slot  \ n -- addr(t) : n -- 0
  <curr-litpool> @ dup
  if
    <curr-litpool-used> @ bounds
    ?do
      dup i @(t) =
      if
        drop i in-lit-range?
        if  i  else  clear-litpool 0  endif
        unloop  exit
      endif
    cell(t) +loop
\ need a new slot, use the first free
    <curr-litpool> @  <curr-litpool-used> @ +    tuck !(t)
    cell(t) <curr-litpool-used> +!
    <curr-litpool-used> @  <curr-litpool-size> @ >=
    if  clear-litpool  endif
    dup in-lit-range? 0=
    if  drop 0  clear-litpool  endif
  endif
;

: litpoolfwd     \ n -- ; build fwd ref

: c_litpool      \ n -- ; build literal in pool
  dup get-pool-slot ?dup
  if
    c_litpoolins drop
  else
    litpoolfwd c_litpoolfwd
  endif
;
))


\ *********************
\ Start all over again!
\ *********************

(( Simple one
The literal pool consists of a list of references which are resolved
after an unconditional return or branch. Since the pool is flushed on
a regular basis, it is not necessary for it to be large, and the pool
is currently implemented as a fixed size table of up to 64 items.

The format of a literal pool entry is:
  cell   lp_value        literal value
  cell   lp_addr         target address of load instruction
  cell   lp_prev         pointer to previous /LP-ENTRY with same LP-VALUE
  cell   lp_addrval      pointer to where constant was laid for duplicates

A literal that is to be handled by the literal pool system is compiled by
the word C_LITPOOL (n -- ).
))
               
[undefined] buffer: [if]
: buffer:       \ size -- ; -- addr ; build a buffer of the required size
  create  allot&erase  ;
[then]

variable lpoffset	\ -- addr ; offset of next entry in table
    
struct /lp-entry
  cell field lp-value		\ value of literal  
  cell field lp-addr		\ address of load instruction
  cell field lp-prev		\ 0 for first entry, pointer to previous /LP-ENTRY
end-struct

#64 /lp-entry *
  constant lpsize		\ size of pool

lpsize buffer: LiteralPool

: initlitpool    \ -- ; initialise pool
( *G Initialise the compiler's literal pool. )
  lpoffset off			\ mark unused
;
initlitpool

defer LitLoadIns,	\ dest-res -- ^ins ; lay literal load instruction
( *G A DEFERed word that loads a literal from the pool into a destination. )
( ** resource. Used as a factor of C_LITPOOL. It is deferred because it is )
( ** CPU dependent. Assignment of the action is usually performed by the ) 
( ** code generator during the cross compiler build. )

defer ResLoadIns	\ ^const ^ins -- ; resolve load offset
( *G A DEFERed word that resolves a load instruction laid by LITLOADINS, )
( ** with an unkown offset. Used as a factor of FLUSHLITPOOL. It is deferred  )
( ** because it is CPU dependent. Assignment of the action is usually )
( ** performed by the code generator during the cross compiler build. ) 

: c_litpool      \ n dest-res -- ; add n to pool
( *G Lays a literal load instruction from the literal pool that )
( ** will be resolved when FLUSHLITPOOL is next called. )
  lpoffset @ lpsize >= abort" literal pool overflow"
  literalpool lpoffset @ + >r		\ slot in pool
  LitLoadIns, r@ lp-addr !		\ lay load instruction, stash ins. addr.
  r@ lp-value !				\ stash literal value to load
  r> lp-prev off			\ clear previous entry field
  /lp-entry lpoffset +!			\ step to next slot
;
               
: flushlitpool   \ --
( *G Flushes the literal pool at the current CDATA location, and fixes )
( ** up all outstanding load instructions referencing the literal pool. )
  lpoffset @ if  
    LiteralPool lpoffset @ bounds ?do
      chere(t)
      i lp-value @ l,c(t)
      i lp-addr @ resloadins
    /lp-entry +loop
    lpoffset off
  endif
;

: #lp-entries	\ -- n
\ *G Return the number of entries in the literal pool.
  lpoffset @ /lp-entry /
;

