\ PREGEN.FTH  Words needed before the code generator/optimiser

((
Copyright (c) 1988-1998, 1999, 2000, 2003, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 2380 631441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

20140723 SFP007 Added -LEADING(T) PARSE(T) PARSE-NAME(T)
20030509 SFP006 Added SWAP-
20001018 SFP005 Restored LAY-TARGET? to be CREATE...DOES> sensitive
20000606 SFP004 Added /STRING(T)
19991015 SFP003 Fixed WORD(T)
19981120 MSD002 Added more stuff from the 386 and 68K compilers.
19981118 SFP001 Moved from XGENxxx.FTH.
))

only forth also c-c definitions

decimal


\ *********************************************************************
\ ****    Defining words used to construct compiler definitions    ****
\ *********************************************************************

((
XC: is used to cross compile a Forth word whose name is given, followed
by the name of it's compiling word and the name of it's host analogue which
is compiled into defining words. This construction allows the definition of
these words to be independent of the compiler technique in use. All the
compiling words are defined in GENERATE.FTH which contains the compilation
behaviour and will be extended with optimisers.
))

: ?b-d          \ -- flag ; if CC is compiling and between CREATE ... DOES>
  compil?  state@ 0<>  and
;

: lay-target?   \ -- t/f/ ; true if we can lay target code ; SFP005
  ?b-d 0=  heads? or			\ if we're not in def. word or headers
\  ?b-d 0=                              \ if we're not in defining word
\  TRUE					\ always
;

: XC:           \ "name" "compiler" "analogue" --
  force-symbol                          \ get xt of REQUIRED.VOC entry
  [ also forth ] create [previous]      \ built into CURRENT voc
    smudge
    ' ,  ' ,                            \ "compiler" then "analogue"
    >body cell+ ,                       \ pointer to pointer to symbol
    smudge
  does>
    lay-target? if
      dup cell+ cell+ @ @ add-xref      \ get pointer and XREF symbol
      dup @ execute                     \ execute compiling action
    endif
    ?b-d
    if  cell+ @ compile,  else  drop  endif    \ lay CREATE ... DOES> action
;


\ ****************************
\ Host analogues for XC: words
\ ****************************

: type(t)       \ targ_addr len --
  bounds ?Do  i c@d(t) emit  Loop
;

: count(t)      \ targ_add1 -- targ_addr2 len
  dup 1+ swap c@d(t)
;

harvard? [if]
: countc(t)     \ targ_add1 -- targ_addr2 len
  dup 1+ swap c@c(t)
;
[then]

: /string(t)	\ addr len n -- addr+n len-n ; not necessary until UNICODE
  tuck - -rot + swap			\ SFP004
;

: word(t)       \ char -- addr(t) ; accepts word from input, to host and target ; SFP003
  word dup c@ 1+  0
  Do  dup i + c@  ihere(t) i + c!d(t)  Loop
  drop
  ihere(t)				\ SFP003
;

: copy(h>t)	\ addr(h) len addr(t) -- addr(t) len ; copy host to target
  over >r  dup >r swap			\ -- addr(h) addr(t) len ; R: -- len addr(t)
  0 ?do
    over c@ over c!d(t)
    swap 1+ swap 1+
  loop
  2drop  r> r>
;

: parse(t)	\ char -- addr(t) len
  parse ihere(t) copy(h>t)
;

: parse-name(t)	\ -- addr(t) len
  parse-name ihere(t) copy(h>t)
;

: -trailing(t)  \ addr1 len1 -- addr2 len2
  dup 0
  ?Do
    2dup + 1- c@d(t) bl <> ?LEAVE 1-
  Loop
;

: -leading(t)	\ addr1 len1 -- addr2 len2
  begin
    dup
   while
    over c@d(t) bl =
   while
    swap 1+ swap 1-
  repeat then
;

: -zeros(t)     \ addr1 len1 -- addr2 len2
  dup 0
  ?Do
    2dup + 1- c@d(t) 0 <> ?LEAVE 1-
  Loop
;

: #>(t)         \ d -- addr len
  #>
  dup >r ihere(t) >r                    \ save target addr and length
  cmove-c,d(t)                          \ copy into target memory at saved addr
  r> r>                                 \ recover target addr and length
;

: compare(t)    \ $addr1 len1 $addr2 len2 -- -1/0/+1 ; compare $1=,<,>$2
  >r swap r>
  2dup - >r                             \ a2 a1 l1 l2 --
  min 0                                 \ a2 a1  l1/l2 0
  ?do  swap 1+  swap 1+                 \ step to next address
       over 1- c@d(t) over 1- c@d(t) <>   \ a2 a1 a2<>a1 --
       ?leave                           \ until different
  loop                                  \ or until done
  swap 1- c@d(t)  swap 1- c@d(t)  -       \ last two chars
  ?dup                                  \ see if same
  if 0< if +1 else -1 endif r> drop     \ not same, return indicator
  else  r> dup                          \ strings =, see if same length
    if  0< if -1 else +1 endif  endif   \ not, return indicator
  endif
;

: >number(t)    \ ud1 c-addr1 len1 -- ud2 c-addr2 len2 ; Number conversion
   Begin
      dup >r
      over c@d(t) upc base @ digit	\ Get next character, see if a digit
      r> 0<> and                        \ And while characters left
   While
      -rot 2>r                          \ -- ud1 n ; Save addr/len
      swap base @ um* drop              \ -- ud1l n (udh*base)
      rot base @ um*                    \ -- n (udh*base)  d(udl*base)
      d+                                \ n is ls part of first number
      2r> 1 /string                     \ Restore addr/len and step on
   Repeat
;

: cells+(t)     \ addr n -- addr'
  cell(t) * +  ;

: on(t)         \ addr --                                                       \ MSD002
  -1 swap !d(t)  ;                                                              \ MSD002
                                                                                \ MSD002
: off(t)        \ addr --                                                       \ MSD002
  0 swap !d(t)  ;                                                               \ MSD002
                                                                                \ MSD002
: none(t)       \ -- ; error if executed                                        \ MSD002
  1 abort" Urk! No host analogue for this word"                                 \ MSD002
;                                                                               \ MSD002

: swap-		\ n1 n2 -- n2-n1 ; SFP006
  swap -
;

