\ Sockets interface

((
Copyright (c) 2000
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
))


\ ===========================================================
( **								)
( *S SOCKETS - WinSock Wrapper					)
( **								)
( *P Support and API imports for the Windows Sockets 		)
( ** Interface.							)
( **								)
\ ===========================================================

[forth] definitions  decimal

library: wsock32.dll
\ extern int PASCAL WSAStartup( WORD, void * );
2 1 libfunction: WSAStartup
\ extern int PASCAL WSACleanup( void );
0 1 libfunction: WSACleanup
\ extern HANDLE PASCAL socket( int, int, int );
3 1 libfunction: socket
\ extern int PASCAL closesocket( HANDLE );
1 1 libfunction: closesocket
\ extern WORD PASCAL htons( WORD );
1 1 libfunction: htons
\ extern LONG PASCAL htonl( LONG );
1 1 libfunction: htonl
\ extern WORD PASCAL ntohs( WORD );
1 1 libfunction: ntohs
\ extern LONG PASCAL ntohl( LONG );
1 1 libfunction: ntohl
\ extern int PASCAL bind( HANDLE, void *, unsigned int );
3 1 libfunction: bind
\ extern int PASCAL accept( HANDLE, void *, unsigned int *);
3 1 libfunction: accept
\ extern int PASCAL listen( HANDLE, int );
2 1 libfunction: listen
\ extern int PASCAL ioctlsocket( HANDLE, unsigned int, void * );
3 1 libfunction: ioctlsocket
\ extern int PASCAL recv( HANDLE, char *, int, int );
4 1 libfunction: recv
\ extern int PASCAL send( HANDLE, char *, int, int );
4 1 libfunction: send
1 1 libfunction: inet_addr
1 1 libfunction: gethostbyname
3 1 libfunction: connect
0 1 libfunction: WSAGetLastError

struct WSADATA
	2	field	WSA.wVersion
	2	field WSA.wHighVersion
	WSADESCRIPTION_LEN 1+	field	WSA.szDescription
	WSASYS_STATUS_LEN 1+	field	WSA.szSystemStatus
	2	field WSA.iMaxSockets
	2	field WSA.iMaxUdpDg
	cell	field WSA.*lpVenderInfo
end-struct

struct sockaddr_in
	2	field sin_family
	2	field sin_port
	4	field sin_addr.S_un.S_addr
	8	field sin_reserved
end-struct

struct HOSTENT
        cell    field h_name
        cell    field h_aliases
        cell    field h_addrtype
        cell    field h_length
        cell    field h_addr_list
end-struct

$4004667F constant FIONREAD
$8004667E constant FIONBIO

also c-c definitions

: ?socket-break	\ -- ; ABORT on ^B or ^C
  key? if
    case  key
      2 of  .rs  1 abort"  ^B - Socket break"  endof
      3 of  .rs  1 abort"  ^C - Socket break"  endof
    endcase
  endif
;

: WSAaccept
  [ also winfunctions ] accept [ previous ]
;

: .bb		\ w --
  0 $0100 um/mod  0 .r  ." ."  0 .r
;

: InitWinSock	\ --
  { | wsa[ WSADATA ] -- }
  $0202 wsa[ rel>abs WSAStartup abort" Failed to initialise WinSock"
  cr ." Winsock v" wsa[ WSA.wHighVersion w@ .bb
     ."  with v"  wsa[ WSA.wVersion w@ .bb ."  active"
  cr wsa[ WSA.szDescription .z$
  cr wsa[ WSA.szSystemStatus .z$
  cr ." Max. number of Sockets = " wsa[ WSA.iMaxSockets w@ .
  cr ." Max. size of UDP Datagram = " wsa[ WSA.iMaxUdpDg w@ . ." bytes"
  cr
;

: KillWinSock	\ --
  WSACleanup drop
  cr ." WinSock terminated"
  cr
;

false constant Verbose?
#256 constant MAXRXSIZE

: (VMWrite)	\ c-addr u hsock --
  -rot  swap rel>abs swap  #0 send SOCKET_ERROR =
  if  cr ." VMWrite error from SEND."  endif
;

: VM::Write	\ c-addr u hsock --
( *G Write a buffer out via a socket. Used to send text strings to the  )
( ** server.                                                       )
  [ Verbose? ] [If] >r  cr ." Written: " 2dup type  r> [Then]
  (VMWrite)
;

: (VMRead)	\ c-addr u hsock -- len/err
  -rot  swap rel>abs swap  #0 recv
  dup SOCKET_ERROR = if
    drop  WSAGetLastError WSAEWOULDBLOCK <>	\ WSAEWOULDBLOCK = #10035
    dup abort" VMRead error from RECV."
  endif
;

: VM::Read	\ caddr u hsock -- len
( *G Read a buffer from a socket. The buffer is U bytes long, and )
( ** the length actually read is returned as LEN. N.B the socket is )
( ** non-blocking and will return 0 if nothing is available. )
  [ Verbose? ] [if]  2 pick >r  [then]
  (VMRead)
  [ Verbose? ] [If]  cr ." Read " r> over type  [Then]
;

: VM::Connect	\ c-addr u port# -- hsocket | 0
( *G Attempt to create a socket and connect to a server.          )
( ** C-ADDR U describes the server address either as a name or ipaddress )
( ** string and PORT# is the requested port. SMTP Servers are almost    )
( ** always found on port 0x19.                                         )
  { c-addr len p# | hsock sin[ SOCKADDR_IN ] z[ MAX_PATH ] blocking -- }

  [ Verbose? ] [If] cr ." Connecting to SEVM Server at " c-addr len type ."  on port " p# . cr [Then]

  AF_INET SOCK_STREAM IPPROTO_TCP socket	\ Create a socket
  dup -> hsock  INVALID_SOCKET = if
    [ Verbose? ] [If] cr ." VM::Connect SOCKET call failed" [Then]
    0 exit
  then
  [ Verbose? ] [If] cr ." VM::Connect SOCKET call ok, hsock = " hsock .dword [Then]

  sin[ SOCKADDR_IN erase                \ fill in address type
  AF_INET  sin[ sin_family w!		\ and port number for
  p# htons sin[ sin_port w!		\ server address struct

  z[ MAX_PATH erase                     \ make a 0 terminated
  c-addr z[ len move                    \ version of the address string.

  FALSE                                 \ Is it a name or IP??
  c-addr len bounds ?do
    i c@ [char] 9 >
    if  drop TRUE  then
  loop                                  \ -- flag

  if                                    \ Use DNS to resolve the supplied name
    z[ rel>abs gethostbyname ?dup if
      abs>rel h_addr_list @ abs>rel @
      sin[ sin_addr.S_un.S_addr !
    else
      hsock CloseHandle drop            \ DNS lookup failed!
      [ Verbose? ] [If] cr ." VM::Connect DNS call failed after GetHostByName" [Then]
      0  exit
    then
  else                                  \ If the server address
    z[ rel>abs inet_addr                \ was numeric then
    sin[ sin_addr.S_un.S_addr !         \ convert and store it
  then

  hsock sin[ rel>abs SOCKADDR_IN
  connect if				\ Attempt to connect socket using server
    hsock CloseHandle drop              \ address structure
    [ Verbose? ] [If] cr ." VM::Connect CONNECT call failed" [Then]
    0 exit                              \ and exit if failed.
  then
  [ Verbose? ] [If] cr ." VM::Connect CONNECT ok" [Then]

  1 -> blocking
  hsock FIONBIO addr blocking rel>abs
  ioctlsocket if			\ make socket non-blocking
    hsock CloseHandle drop              \ close socket on error
    [ Verbose? ] [If] cr ." VM::Connect IOCTLSOCKET call failed" [Then]
    0 exit                              \ and exit if failed.
  then
  [ verbose? ] [if] cr ." Socket set non-blocking " [then]

  c" compiler" count hsock (VMwrite)	\ tell host we're a compiler
  [ Verbose? ] [If] cr ." Sent COMPILER" [Then]

  begin				\ wait for 'r' response from server
    begin
      ?socket-break busyidle
      z[ MAX_PATH hsock (VMRead)
    until
[ verbose? ] [if] cr ." Got message: " z[ 10 dump [then]
    z[ c@ ascii r =
  until
[ verbose? ] [if] cr ." Got ready message " [then]

  hsock                                 \ return socket handle
;

: VM::Disconnect	\ hsock --
( *G Disconnect from a VM discussion by closing our socket.           )
  [ Verbose? ] [If] cr ." Disconnecting from SEVM server" cr [Then]
  CloseHandle drop
;

max_path buffer: SocketName
variable SocketPort
variable hSocketLink

: socket-link-open	\ -- ; start comms as default
  InitWinSock
  SocketName count  SocketPort @ VM::Connect
  hSocketLink !
;

: socket-link-close	\ -- ; stop socket comms
  hSocketLink @ VM::Disconnect
  hSOcketLink off
  KillWinSock
;

variable #Read			\ -- addr ; number of bytes read from socket
variable #taken		\ -- addr ; number of bytes taken from buffer
#1536 constant SRBlen	        \ -- n ; length of socket read buffer
SRBlen buffer: SocketReadBuff	\ -- addr ; socket read buffer
variable <debug-socket>		\ -- addr ; true to debug socket code
  <debug-socket> off

: socket-query-byte	\ -- flag ; true if available
  #read @
  if  true exit  endif
  SocketReadBuff SRBlen hSocketLink @ (VMRead)
  dup 0> if
    #read !  #taken off  true
  else
    drop  #read off  #taken off  false
  endif
;

: socket-wait-byte	\ -- char
  begin								\ spin until data available
    socket-query-byte 0=
   while
     ?socket-break  busyidle	\ check for user intervention, give Windows a chance
  repeat
  SocketReadBuff #taken @ + c@			\ get next character in buffer
  #taken incr  #read decr				\ update counters
  <debug-socket> @
  if  dup ." [" .byte ." ]"  endif
;

: socket-send-byte	{ char -- }
  <debug-socket> @
  if  char ." <" .byte ." >"  endif
  addr char 1 hSocketLink @ (VMwrite)
;

: socket-link-reset	\ --
;

: socket-link-flush	\ --
  begin
    socket-query-byte
   while
    socket-wait-byte drop
  repeat
;

: .socket-link-state	\ --
  base @ decimal
  ." Using socket connection to: " SocketName count type
  ."  on port: " SocketPort @ .
  ."  with handle: " hSocketLink @ .
  base !
;

: set-socketlink	\ -- ; set socket drivers
  assign socket-link-open to-do link-open
  assign socket-link-close to-do link-close
  assign socket-query-byte to-do query-byte
  assign socket-wait-byte to-do wait-byte
  assign socket-send-byte to-do send-byte
  assign socket-link-reset to-do link-reset
  assign socket-link-flush to-do link-flush
  assign .socket-link-state to-do .link-state
;

also cc/i               \ this code added to the cross interpreter
: SocketLink	cc/i	\ caddr-ip port# -- ; <name> <port> SocketLink
  SocketPort !				\ stash port number
  SocketName over c@d(t) 1+ target>host
  set-socketlink
;
: +socket-debug	cc/i	<debug-socket> on  ;
: -socket-debug	cc/i	<debug-socket> off  ;
: socket-debug?	cc/i	<debug-socket> @  ;
previous definitions

only forth  also c-c definitions


