\ XREF.FTH - cross references for MPE Forth Cross-Compiler v6 and v7.

((
Copyright (c) 1998, 2008, 2017, 2019

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: (+44) 2380 631441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

20191106 SFP007 Replace xxx-PATH words.
20190919 SFP006 Header update.
20080205 SFP005 VFX conversion.
SFP004  26 April 2000	Enabled XREF during interpretation, and prevented
			XREF <name> producing a reference.
SFP003  10 April 2000	Disabled in IRTC compiler
SFP002  18 Nov 1998     Added XREF-ALL and XREF-UNUSED
SFP001  15 Nov 1998     Original release

The compiler generates cross references by building a chain of fields
including LOCATE format (link:32, symbol-nfa:32, file#:16, line#:16) in a
separate area of memory. Links and pointers are relative to the start
of the XREF memory area.
))

only forth also c-c definitions

struct /ref(x)
  ptr >link(x)          \ link field
  ptr >sym(x)           \ symbol in which this reference occurs
  2 field >line#(x)     \ line# in file being compiled
  2 field >file#(x)     \ file# of file being compiled
\ -- from here compatible with host version ; SFP006
  cell field >in(x)             \ offset in line
  cell field >xtHere(x)         \ HERE when reference was made
  cell field >type(x)           \ 0 for xts, 1 for literals, 2 doubles,
  				\ -2 for floats on FP stack, -1 for OGL floats on data stack
  cell field >link(d)           \ decompilation link
  3 cells field >xtadded(x)     \ xt being added or literal value (1, 2, or 3 cells)
end-struct

variable <xref> \ -- addr ; true if XREF enabled
  <xref> off

variable /xref  \ -- addr ; xref memory size, default to 8Mb
  $0080:0000 /xref !

variable ^xref  \ -- addr ; holds address of XREF memory
  0 ^xref !
variable dp(x)  \ -- addr ; holds current offset into XREF memory
  0 dp(x) !

: here(x)       \ -- addr ; analogue of HERE
  dp(x) @
;

: ,(x)          \ x -- ; analogue of ,
  ^xref @ dp(x) @ + !
  cell dp(x) +!
;

: w,(x)         \ w -- ; analogue of W,
  ^xref @ dp(x) @ + w!
  2 dp(x) +!
;

: c,(x)         \ w -- ; analogue of W,
  ^xref @ dp(x) @ + c!
  1 dp(x) +!
;

: @(x)          \ offset -- x
  ^xref @ + @
;

: !(x)          \ x offset --
  ^xref @ + !
;

: w@(x)         \ offset -- w
  ^xref @ + w@
;

: w!(x)         \ w offset --
  ^xref @ + w!
;

: c@(x)         \ offset -- b
  ^xref @ + c@
;

: c!(x)         \ b offset --
  ^xref @ + c!
;

: ?MakeHeader	\ -- ; skip the header if the XREF memory is empty ; SFP006
  dp(x) @ 0= if
    dp(t) @ ,(x)			\ Target here at start of XREF
    0 ,(x)				\ anchors XREF headers
    #256 dp(x) !			\ reserve header block
  then
;

: add(x)        \ symbol -- ; add XREF location to symbol chain ; SFP004
  <xfind> @				\ don't XREF during XREF
  if  drop  exit  endif

  dp(x) @  /xref @ $100 - u>		\ overflow test
  abort" XREF overflow: Use <n> XREF-KB to increase table size"

  ?MakeHeader				\ if empty, make header space ; SFP006

  xref-anchor+                          \ point to XREF chain anchor
  here(x)   over @(s) ,(x)   swap !(s)  \ add link field
  state(t) @(m)
  if  'latest(s) @(m)  else  0  endif	\ add current symbol
  ,(x)
  from? if                              \ add LOCATE field
    line# @ w,(x)			\ line # ; SFP006
    currsourcefile# @ w,(x)             \ file # ; SFP006
  else
    0 ,(x)
  endif
\ SFP006...
  >in @ ,(x)  dp(t) @ ,(x)  0 ,(x)	\ >IN, HERE, type
  0 ,(x)  0 ,(x)  0 ,(x)  0 ,(x)  	\ decomp and literal
\ ...SFP006
;
((
  cell field >in(x)             \ offset in line
  cell field >xtHere(x)         \ HERE when reference was made
  cell field >type(x)           \ 0 for xts, 1 for literals, 2 doubles,
  				\ -2 for floats on FP stack, -1 for OGL floats on data stack
  cell field >link(d)           \ decompilation link
  3 cells field >xtadded(x)     \ xt being added or literal value (1, 2, or 3 cells)
))

: init-xref     \ -- ; initialise XREF memory - only runs once ; SFP006
  ^xref @ 0= if                         \ ignore on subsequent runs
    /xref @ allocate abort" Can't get XREF memory"
    ^xref !  0 dp(x) !			\ SFP006
  endif
;

: xrefCold	\ -- ; initialise in cold chain
  ^xref off
; ' xrefCold AtCold


max_path buffer: XREF$    \ -- addr ; buffer for XREF file name
  s" XREF.XRF" xref$ place

: term-xref     \ -- ; free up XREF memory
  ^xref @ if
    ^xref @off free abort" Can't free XREF memory"
  endif
; ' term-xref AtExit

: save-xref     \ -- ; save XREF memory to file
  ^xref @ if
    xref$ count fileExist? if
      xref$ count delete-file drop
    then
    xref$ count r/w create-file #-412 ?throw >r	\ create
    ^xref @ here(x) r@ write-file	\ -- ior ; write
    r> close-file #-416 ?throw		\ close
    #415 ?throw
  endif
;

: load-xref     \ -- ; reload XREF file from disc
  ^xref @ 0= if                         \ only if not already done
    init-xref                           \ get memory
    xref$ $>asciiz r/o open-file abort" Can't open XREF file" >r
    ^xref @ /xref @ r@ read-file abort" Can't read XREF file" drop
    r> close-file abort" Can't close XREF file"
  endif
;

: xref?         \ -- flag ;
  <xref> @ 0<>
;

: xref:         \ "filename" -- ; enable XREFs
  bl word xref$ $move
;

: xref-kb       \ n --
  #1024 * /xref !
;

\ defer add-xref  \ symbol -- ; add XREF to list - now in XC-SYMBL.FTH

: +xrefs        \ -- ; enable XREF
  <xref> on
  init-xref
  assign add(x) to-do add-xref
;

: -xrefs        \ -- ; disable XREF
  <xref> off
  assign drop to-do add-xref
;
-xrefs

: .xref         \ xref -- ; display name of symbol
  tab  >sym(x) @(x) ?dup if
    .SymName				\ show name of symbol in which used
  else
    ." interpreted"
  endif
;

: .xrefs        \ symbol -- ; display list of words that use the symbol
  cr ." XREF of " dup .SymName		\ display word name     SFP002
  c/l out !
  xref-anchor+ @(s) ?dup if             \ get head of chain
    begin
      ?dup
     while
      dup .xref
      >link(x) @(x)
    repeat
  else
    tab ." Unused or no XREF information"
  endif
;

: xref-report   \ -- ; display XREF information
  ^xref @ if
    cr ." XREF memory used = "
    here(x) #1024 / . ." Kb of "
    /xref @ #1024 / . ." Kb allocated"
  endif
;


\ *********************************************
\ XUSES <name> cross references a specific word
\ XREF-ALL cross references all target words
\ XREF-UNUSED finds all unused words
\ *********************************************

: ?xref-enabled \ -- ; exits caller if XREFfing not turned on
  ^xref @ 0= if
    cr
    cr ." Use +XREFS before this to turn on XC's XREF system"
    cr
    r> drop                             \ exits CALLER
  endif
;

: xuses         \ -- ; XUSES <name>
  ?xref-enabled
  <xfind> on				\ stop XREF producing XREF
  get-token find(t)_ans ?undef		\ error if symbol not found
  <xfind> off
  .xrefs
;

: ProcessWords(s)       \ xt -- ; apply XT to all symbols in CONTEXT voc
  >r                                    \ save XT
  cr cr ." ***********************"
     cr ." Vocabulary: " Context(s) @(m) .voc(s)
  c/l out !                             \ set up
  Context(s) @(m) #Threads(s)+          \ #threads, thr0..thrn-1
  dup cell+  nfa-buff                   \ source, dest
  rot @(s) cell* cmove(s)>              \ count/length and copy
  Begin
    max-nfa(s) dup                      \ while still one left
    halt? 0= and                        \ and no stop command
   While
    dup name>(s) r@ execute             \ process word (symbol -- )
    cell- @(s) swap !                   \ find next NFA in thread
  Repeat
  2drop
  r> drop                               \ discard XT
;

: ProcessVoc(s)         \ xt voc -- ; process all words in vocabulary
  context(s) @(m) >r  context(s) !(m)   \ set up vocabulary
  ProcessWords(s)                       \ process vocabulary
  r> context(s) !(m)                    \ restore context
;

: ProcessAllVocs(s)     \ xt -- ; process all vocabularies in the system
  >r
  VOC-LINK(s) @(m)                      \ first
  BEGIN
    ?DUP                                \ while they exist
   WHILE
     r@ over vl- ProcessVoc(s)          \ process the vocabulary
     @(s)                               \ get next one
  REPEAT
  r> drop
;

: .xrefs(1)     \ symbol -- ; no display if unresolved
  dup resolved?
  if  cr .xrefs  else  drop  endif
;

: xref-all      \ -- ; cross reference all words
  ?xref-enabled
  cr
  cr ." *************************************"
  cr ." Cross Reference listing of all words"
  cr ." *************************************"
  ['] .xrefs(1) ProcessAllVocs(s)
  cr
;

: .xrefs-none   \ symbol -- ; display word if unused
  dup resolved?                         \ if resolved
  over xref-anchor+ @(s) 0= and         \ and unused
  if
    >name(s) tab .name(s)
  else
    drop
  endif
;

: xref-unused   \ -- ; cross reference all words
  ?xref-enabled
  cr
  cr ." *******************************************************"
  cr ." The following words are unused inside colon definitions"
  cr ." *******************************************************"
  ['] .xrefs-none ProcessAllVocs(s)
  cr
;

\ **********************
\ Code stripping support
\ **********************

((

256 buffer: Keep$	\ -- addr
\ Buffer for file name
  s" keep.txt" Keep$ place
256 buffer: Unused$	\ -- addr
\ Buffer for file name
  s" unused.txt" Unused$ place

FileBuff: AFileDev	\ -- addr
\ Device for file operations.

: .xrefu-write	\ symbol --
\ Write symbol to file or console.
  dup resolved?                         \ if resolved
  over xref-anchor+ @(s) 0= and if	\ and unused
    .SymName space
    out @ 66 >
    if  cr  endif
  else
    drop
  endif
;

[undefined] !++ [if]
: !++		\ addr val -- addr'
  over !  cell +
;
[then]
: write-unused	\ -- ; list unused words to file unused.txt
  Unused$ count delete-file drop
  Unused$ count r/w AFileDev open-gio	\ -- hFile ior
  abort" Can't open file for UNUSED" drop
  [io  AFileDev setIo			\ switch output
  ['] .xrefu-write ProcessAllVocs(s)
  io]  AFileDev close-gio drop
;

variable KeepBuff	\ -- addr|0
\ Buffer for words to keep
variable UnusedBuff	\ -- addr|0
\ Buffer for words that were unused.

: line>buff	\  start curr -- start curr'
  {: start curr -- start' curr' :}
  begin
    bl word count dup
   while
     tuck curr place
     1+ dup +to curr
     start +!  1 start cell + +!
  repeat
  2drop  start curr
;

: LoadWordBuff	\ caddr len var --
  {: caddr len var | hFile -- :}
  var @ if  var @ free drop  endif
  1 Mb protAlloc dup var !
  dup  8 !++  0 !++			\ starts with data length and number of words
  caddr len r/o AFileDev open-gio
  abort" Can't read file" drop
  [io  AFileDev setIo			\ switch input
  begin
    refill
   while
    line>buff
  repeat
  2drop
  io]  AFileDev close-gio drop

;
))

((
\ test code

: test(xx)      \ symbol -- ; perform action of WORDS(S) on symbol
  dup >name(s) .word-name(s)            \ show name
  resolved? 0=                          \ say if unresolved
  if ."  * "  endif
;

: words(xx)     \ -- ; test code
  ['] test(xx) Context(s) @(m) ProcessVoc(s)
;

: all-words(xx) \ -- ; test code
  ['] test(xx) ProcessAllVocs(s)
;
))

