\ EtherK60.fth - Ethernet driver for TWR-K60N512 with TWR-SER

((
Copyright (c) 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

Portions of this code were supplied by OpenMRT,
  Cape Town, South Africa


To do
=====

Change history
==============
20111121 SFP003 Kinetis K60 conversion.
20100623 JAU002 Fixed PHY#, InitEther, InitPhy.
20100525 MPE001 First cut.
))

only forth definitions
decimal

\ ============
\ *! etherk60
\ *T Ethernet driver for Kinetis K60 devices
\ ============
\ *P The driver in *\i{Cortex\Drivers\EtherK60.fth} was tested on
\ ** a Freescale Tower System with TWR-K60N512 and TWR-SER modules.
\ ** Only RMII mode has been tested.
\ ** The 1588 timestamp modes are not implemented.

\ ****************
\ *S Configuration
\ ****************
\ *P The following must be defined before the file
\ ** *\i{Drivers\EtherK60.fth} is compiled.

interpreter
: const equ ;	\ n -- ; -- n
\ *G If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** *\fo{CONST} as *\fo{CONSTANT}, otherwise by default *\fo{CONST}
\ ** is defined as *\fo{EQU}.
target

[undefined] EtherCom? [if]
0 equ EtherCom?	\ -- flag
\ *G Only needed if the EtherCom generic I/O device is being used.
\ ** This only happens for systems with multiple IP ports and
\ ** PowerNet v3.x upwards.
[then]

[undefined] EtherDiags? [if]
1 equ EtherDiags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. False by default.
[then]

[undefined] MultiPhy? [if]
1 equ MultiPhy?	\ -- flag
\ *G Set this flag true if you are using multiple PHYs or do not
\ ** know the address of the PHY.
[then]

[undefined] Phy# [if]
MultiPhy? [if]
0 value Phy#	\ -- n
\ *G Multi PHY: The number of the PHY in use: 0..31
[else]
0 equ Phy#	\ -- n
\ *G Single PHY: The number of the PHY in use: 0..31. For the
\ ** TWR-K60N512 and TWR-SER use 0.
[then]
[then]

[undefined] FindPhy? [if]
1 equ FindPhy?	\ -- flag
\ *G Set this flag true if you want to scan for the PHY address,
\ ** because it varies or you don't know it.  Use only if you have
\ ** a single PHY connected and *\fo{MultiPhy?} must also be set
\ ** to non-zero.
[then]

[undefined] RMII? [if]
1 equ RMII?	\ -- n
\ *G Set non-zero if using RMII rather than the MII interface.
[then]

[undefined] initPHY? [if]
1 equ initPHY?	\ -- x
\ *G If it is difficult to get the PHY to respond during testing,
\ ** set *\fo{initPHY?} to zero. For normal use it should be set
\ ** to 1.
[then]

[undefined] sniff? [if]
0 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] pingit? [if]
0 equ pingit?	\ -- flag
\ *G Set this equate true to compile the PING code,
\ ** which tests Ethernet transmission and reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
\  $00 c, $04 c, $9F c, $00 c, $EC c, $C7 c,	\ supplied EVB board MAC address
   $00 c, $10 c, $8B c, $F1 c, $44 c, $51 c,	\ six bytes for 48 bit MAC address
\  $00 c, $08 c, $EF c, $00 c, $5E c, $C4 c,	\ six bytes for 48 bit MAC address
[then]						\ you must obtain these yourself!

[undefined] IpAddress [if]
create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
\  10 c,  102 c,  0 c,  11 c,		\ assign these as required
\  192 c,  168 c,  0 c,  100 c,		\ assign these as required
  192 c,  168 c,  1 c,  251 c,		\ assign these as required
[then]


\ **********
\ *S Buffers
\ **********

#1518 equ /EPacketMax	\ -- len
\ *G Max length of a normal Ethernet packet.

#1536 equ /ERxBuff	\ -- u
\ *G Size of an Ethernet receive buffer rounded up to 16 byte unit.
\ ** If the buffers are in cached memory, they should also be rounded
\ ** to a cache line (64 bytes). Although 1520 (1518 rounded up)
\ ** is 16 byte aligned, it permits normal frames but not VLAN
\ ** frames.
#4 equ #ERxBuffs		\ -- u
\ *G Number of receive buffers. Must be at least two.
#1536 equ /ETxBuff	\ -- u
\ *G Size of an Ethernet transmit buffer rounded up to 16 byte unit.
#2 equ #ETxBuffs		\ -- u
\ *G Number of transmit buffers. Must be at least one. Transmit buffers
\ ** are managed in software.

struct /EDesc	\ -- len
\ *G Structure defining an Ethernet transmit/receive descriptor.
\ ** Note that the contents are in big-endian format.
\ *[
  2 field ed.status		\ 2 byte status
  2 field ed.len		\ 2 byte length
  4 field ed.addr		\ address of buffer - 16 byte aligned
end-struct
\ *]

compiler
: be-w@   w@ byteRevW  ;
: be-w!   swap byteRevW swap w!  ;
: be-l@   @ byteRevL  ;
: be-l!   swap byteRevL swap !  ;
target

uhere #15 + #-16 and uorg
/Edesc #ErxBuffs * buffer: ERxDescs	\ -- addr
\ *G Ethernet receive descriptors.
uhere #15 + #-16 and uorg
/Edesc #ETxBuffs * buffer: ETxDescs	\ -- addr
\ *G Ethernet transmit descriptors.
uhere #15 + #-16 and uorg
/ERxBuff #ERxBuffs * buffer: ERxBuffs	\ -- addr
\ *G Ethernet receive buffers.
uhere #15 + #-16 and uorg
/ETxBuff #ETxBuffs * buffer: ETxBuffs	\ -- addr
\ *G Ethernet transmit buffers.

variable FirstRxDesc	\ -- addr
\ *G Holds the address of the first read descriptor to check.
  ERxDescs FirstRxDesc !
variable NextRxDesc	\ -- addr
\ *G Holds the address of the next read descriptor containing unhandled
\ ** data. Set to zero to indicate none or unknown.
  0 NextRxDesc !
variable NextTxDesc	\ -- addr
\ *G Holds the address of the next transmit descriptor to check.
  ETxDescs NextTxDesc !

: initRxDescs	\ --
\ *G Initialise the receive descriptors.
  ERxDescs FirstRxDesc !  0 NextRxDesc !  \ mark no data available
  ERxBuffs ERxDescs  #ERxBuffs 1- 0 do	\ -- buff hdr
    bit15 over ed.status be-w!		\ set E bit
    /ERxBuff over ed.len be-w!		\ len
    2dup ed.addr be-l!			\ address
    swap /ERxBuff + swap /EDesc +	\ step to next
  loop					\ -- buff' hdr'
  bit15 bit13 or over ed.status be-w!	\ set E and W bits
  /ERxBuff over ed.len be-w!		\ len
  ed.addr be-l!				\ address
;

: initTxDescs	\ --
\ *G Initialise the transmit descriptors.
  ETxDescs NextTxDesc !			\ mark no data available
  ETxBuffs ETxDescs  #ETxBuffs 1- 0 do	\ -- buff desc
    0 over ed.status be-w!			\ clear R,L bit
    0 over ed.len be-w!			\ len
    2dup ed.addr be-l!			\ address
    swap /ETxBuff + swap /EDesc +	\ step to next
  loop					\ -- buff' hdr'
  bit13 over ed.status be-w!		\ set W bit
  0 over ed.len be-w!			\ len
  ed.addr be-l!				\ address
;

: StepRxDesc	\ desc -- desc'
  dup ed.status be-w@ bit13 and		\ if WRAP bit set
  if drop ERxDescs else /EDesc + endif
;

: StepTxDesc	\ desc -- desc'
  dup ed.status be-w@ bit13 and		\ if WRAP bit set
  if drop ETxDescs else /EDesc + endif
;


\ ********
\ *S Tools
\ ********

((
: or!		\ x addr --
  tuck @ or swap !  ;
: wor!		\ x addr --
  tuck w@ or swap w!  ;
: bor!		\ x addr --
  tuck c@ or swap c!  ;

: bic!		\ x addr --
  tuck @  swap invert and  swap !  ;
: wbic!		\ x addr --
  tuck w@  swap invert and  swap w!  ;
: bbic!		\ x addr --
  tuck c@  swap invert and  swap c!  ;
))


\ *************
\ *S PHY access
\ *************

$60020000 equ ~RdPhy	\ -- mask
\ *G Mask for PHY read operations. Assumes use of PHY0.
$50020000 equ ~WrPhy	\ -- mask
\ *G Mask for PHY write operations. Assumes use of PHY0.

\ Standard PHY registers 0..F
0x00 const PHY_CNTL_REG
0x01 const PHY_STATUS_REG
0x02 const PHY_ID_REG1
0x03 const PHY_ID_REG2
0x04 const PHY_ANA_REG
0x05 const PHY_ANLPAR_REG
0x06 const PHY_ANE_REG

\ PHY control register
bit15 const RESET_PHY
bit14 const ENABLE_LOOPBACK
bit13 const DR_100MB
bit12 const ENABLE_AN
bit10 const PHY_MAC_ISOLATE
bit9  const RESTART_AN
bit8  const PHY_FULLDUPLEX
bit7  const PHY_COL_TEST

\ PHY Status register
bit15 const CAP_100BT4
bit14 const CAP_100BTX_FDX
bit13 const CAP_100BTX
bit12 const CAP_10BT_FDX
bit11 const CAP_10BT

bit6 const NO_PREAMBLE
bit5 const AN_COMPLETE
bit4 const REMOTE_FAULT
bit3 const AN_ABILITY
bit2 const LINK_UP
bit1 const JABBER
bit0 const CAP_EXTENDED

\ PHY Auto-negotiation Advertisement register
bit8 const DR100_TX_FULL
bit7 const DR100_TX_HALF
bit6 const DR10_TX_FULL
bit5 const DR10_TX_HALF
DR100_TX_FULL DR100_TX_HALF or DR10_TX_FULL or
  const ANE_ALL	\ request everything

MultiPhy? [if]
: ~Phy#		\ -- mask
\ *G Multi PHY: Mask used for PHY command generation.
  Phy# #23 lshift
;
[else]
Phy# #23 lshift equ ~Phy#	\ -- mask
\ *G Single PHY: Mask used for PHY command generation.
[then]

: WaitPhyDone	\ --
\ *G Wait until the current PHY operation has completed.
  begin
    pause  _FEC fecEIR + @ bit23 and 	\ interrupt must be enabled
  until
;

: ReadPhy	\ reg -- w
\ *G Read a PHY register. Note that these are 16 bit values.
  bit23 _FEC fecEIR + !  		\ reset MII interrupt
  $1F and #18 lshift			\ form reg# mask
  ~RdPhy or  ~Phy# or			\ set up reg command
\ cr ." MMFR=" dup .dword
  _FEC fecMMFR + !			\ set command
  WaitPhyDone				\ let operation complete
  _FEC fecMMFR + @ $FFFF and		\ return data
;

: WritePhy	\ w reg --
\ *G Write a PHY register. Note that these are 16 bit values.
  bit23 _FEC fecEIR + !  		\ reset MII interrupt
  $1F and #18 lshift			\ form reg# mask
  ~WrPhy or  ~Phy# or			\ set up reg command
  swap $FFFF and or			\ merge in data
\ cr ." MMFR=" dup .dword
  _FEC fecMMFR + !			\ set command
  WaitPhyDone
;

: OrPhy		\ mask reg --
\ *G Or mask into reg in the PHY. Set bits in mask are set in the
\ ** register.
  tuck ReadPhy  or  swap WritePhy
;

: InitPhy	\ -- ; autonegotiation version
\ *G Initialise the PHY with autonegotiation
  RESET_PHY PHY_CNTL_REG OrPhy		\ software reset the chip
  begin					\ wait until reset complete
    PHY_CNTL_REG ReadPhy RESET_PHY and 0=
  until
  ANE_ALL PHY_ANA_REG OrPhy		\ request 10/100BaseT features
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
  RESTART_AN PHY_CNTL_REG OrPhy		\ start autonegotiation
  begin					\ wait until done
    PHY_STATUS_REG ReadPhy
    AN_COMPLETE and
  until
  PHY_ANLPAR_REG ReadPhy
\ set up control reg using the results
  0					\ -- partner cntlset
  over DR100_TX_FULL and
  if  DR_100MB PHY_FULLDUPLEX or or  endif
  over DR100_TX_HALF and
  if  DR_100MB or  endif
  over DR10_TX_FULL and
  if  PHY_FULLDUPLEX or  endif
  PHY_CNTL_REG WritePhy
  drop
;

: EtherLink?	\ -- flag
\ *G Return true if the Ethernet link is established. For most
\ ** PHYs, this is performed by testing bit 2 of register 1.
  1 ReadPhy bit2 and 0<>
;

: FindPhy	\ -- phy|-1
\ *G Scan for PHYs. Return the number of the first PHY found, or
\ ** -1 if no PHY is found. *\fo{Phy#} is set for each PHY tested.
  -1  #32 0 do				\ range of possible Phy addresses 0..$1F
    i to Phy#
    PHY_ID_REG1 ReadPhy $FFFF <> if
\      cr ." Found Phy with addr=" i .
      drop i leave			\ no error; sucess
    endif
  loop
;


\ *****************
\ *S Initialisation
\ *****************

\            system_speed
\ 2.5Mhz = ----------------
\            mii_speed * 2
\ Rearrange
\               system_speed
\ mii_speed = ----------------
\                2.5Mhz * 2

system-speed 5 MHz /mod			\ required divider
swap 0<> 1 and + 1 lshift		\ in bits 6:1
bit7 or 				\ disable 32 bit preamble
system-speed #100 MHz /  8 lshift or	\ min 10ns MDIO hold time
  equ initMSCR	\ -- x
\ *G Initial value of MSCR.

: genFEChash	\ addr -- crc
  -1 swap 6 bounds do		\ -- crc
    i c@  8 0 do		\ -- crc b
      2dup xor 1 and if
        swap 1 rshift $EDB8:8320 xor swap
      else
        swap 1 rshift swap
      endif
      1 rshift
    loop
    drop
  loop
;

: setMACAddress	\ addr --
\ *G Set the MAC address in the EMAC unit.
  dup c@ #24 lshift			\ set for TX/RX exact match
  over 1+ c@ #16 lshift or
  over 2+ c@ 8 lshift or
  over 3 + c@ or
  _FEC fecPALR + !
  dup 4 + c@ #24 lshift
  over 5 + c@ #16 lshift or
  _FEC fecPAUR + !
  genFEChash #26 rshift dup $20 < if	\ set for RX hash
    1 swap lshift _FEC fecIALR + or!
  else
    $20 -
    1 swap lshift _FEC fecIAUR + or!
  endif
;

$0400 equ Alt4	\ -- x
\ *G Port control register value for MUX=4.

RMII? [if]
: setEtherPins	\ -- ; RMII
\ *G Set the K60 pins required for RMII mode. These are:
\ *C   RMII_MDC RMII_MDIO RMII_CRS_DV RMII_RXER RMII_TXEN
\ *C   RMII_RXD[1:0]  RMII_TXD[1:0]
\ *C   RMII_REF_CLK 1588_TMRn ENET_1588_CLKIN
  Alt4 _PORTB
  2dup portPCR0 + !			\ PB0  RMII_MDIO
  2dup portPCR1 + !			\ PB1  RMII_MDC
  2drop

  Alt4 _PORTA
  over 2 + over portPCR5 + !		\ PA5  RMII_RXER with pull down
  2dup portPCR12 + !			\ PA12 RMII_RXD1
  2dup portPCR13 + !			\ PA13 RMII_RXD0
  2dup portPCR14 + !			\ PA14 RMII_CRS_DV
  2dup portPCR15 + !			\ PA15 RMII_TXEN
  2dup portPCR16 + !			\ PA16 RMII_TXD0
  2dup portPCR17 + !			\ PA17 RMII_TXD1
  2drop
;
[else]
: setEtherPins	\ -- ; MII
\ *G Set the K60 pins required for MII mode. These are:
\ *C   MII_COL MII_CRS MII_RXCLK MII_TXCLK MII_TXER
\ *C   MII_MDC MII_MDIO MII_RXDV MII_RXER RMII_TXEN
\ *C   MII_RXD[1:0]  RMII_TXD[1:0]
\ *C   1588_TMRn ENET_1588_CLKIN
  Alt4 _PORTB
  2dup portPCR0 + !			\ PB0  RMII_MDIO
  2dup portPCR1 + !			\ PB1  RMII_MDC
  2drop

  Alt4 _PORTA
  over 2 + over portPCR5 + !		\ PA5  RMII_RXER with pull down
  2dup portPCR9 + !			\ PA9  MII_RXD3
  2dup portPCR10 + !			\ PA10 MII_RXD2
  2dup portPCR11 + !			\ PA9  MII_RXCLK
  2dup portPCR12 + !			\ PA12 MII_RXD1
  2dup portPCR13 + !			\ PA13 MII_RXD0
  2dup portPCR14 + !			\ PA14 MII_RXDV
  2dup portPCR15 + !			\ PA15 MII_TXEN
  2dup portPCR16 + !			\ PA16 MII_TXD0
  2dup portPCR17 + !			\ PA17 MII_TXD1
  2dup portPCR24 + !			\ PA24 MII_TXD2
  2dup portPCR25 + !			\ PA25 MII_TXCLK
  2dup portPCR26 + !			\ PA26 MII_TXD3
  2dup portPCR27 + !			\ PA27 MII_CRS
  2dup portPCR28 + !			\ PA28 MII_TXER
  2dup portPCR29 + !			\ PA29 MII_COL
  2drop
;
[then]

: setPHYdata	\ --
\ *G Collect the results of initialising the PHY and apply them
\ ** to the MAC.
  0 readPhy dup bit13 and if		\ 100 Mb
    bit9 _FEC fecRCR + bic!		\ clear 10M bit
  else					\ 10 Mb
    bit9 _FEC fecRCR + or!		\ set 10M bit
  endif
  bit8 and if				\ full duplex
    bit1 _FEC fecRCR + bic!		\ TX and RX are independent
    bit2 _FEC fecTCR + or!		\ set Full Duplex
  else					\ half duplex
    bit1 _FEC fecRCR + or!		\ TX and RX are together
    bit2 _FEC fecTCR + bic!		\ clear Full Duplex
  endif
;

: initEther	\ --
\ *G Initialise the FEC.
  bit0 _SIM simSCGC2 + or!		\ enable Ethernet clock
  0 _MPU mpuCESR + !			\ enable all bus masters - disable MPU
  setEtherPins				\ enable Ethernet pins

  $01 _FEC fecECR + !			\ reset the controller
  begin
    _FEC fecECR + @ bit0 and 0=
  until
  initMSCR _FEC fecMSCR + !		\ PHY read/write speed
  bit23 _FEC fecEIMR + !		\ only MII interrupt enabled
  -1 _FEC fecEIR + !			\ clear all interrupts
[ RMII? ] [if]
  $05EE:0104 _FEC fecRCR + !		\ 1518 bytes, RMII mode
[else]
  $05EE:0004 _FEC fecRCR + !		\ 1518 bytes, MII mode
[then]
  $04 _FEC fecTCR + !			\ assume full duplex for now
  0 _FEC fecIAUR + !			\ hash tables
  0 _FEC fecIALR + !
  0 _FEC fecGAUR + !
  0 _FEC fecGALR + !
  EtherAddress setMacAddress
  $0400 _FEC fecOPD + !			\ PAUSE duration, 1 unit = 512 bit times

  bit31 _FEC fecMIBC + !		\ clear MIB RAM
  _FEC fecMIBRAM + /MIBRAM bounds
  do  0 i !  cell +loop
  0 _FEC fecMIBC + !

  initRxDescs initTxDescs		\ initialise descriptors
  /ERxBuff _FEC fecEMRBR + !		\ set up rings
  ERxDescs _FEC fecERDSR + !
  ETxDescs _FEC fecETDSR + !

[ FindPhy? ] [if]  FindPhy drop  [then]	\ locate PHY
[ initPhy? ] [if]  initPHY  [then]	\ initialise the PHY
  setPHYdata				\ apply PHY data to MAC
  bit1 _FEC fecECR + or!		\ re-enable Ethernet
  #10 ms  1 _FEC fecRDAR + !		\ start receiver
;


\ ********************
\ *S Packet read/write
\ ********************

: IsRx?         \ -- flag ; nz = has data
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
  NextRxDesc @				\ already got one
  if  true exit  endif
  FirstRxDesc @  #ERxBuffs 0 do		\ -- desc
    dup ed.status be-w@ bit15 and 0= if	\ new received packet?
      dup NextRxDesc !			\ point to packet to read
      StepRxDesc FirstRxDesc !		\ step to next one
      true  unloop exit			\ get out
    endif
    StepRxDesc
  loop
  drop 0				\ nothing received.
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get a pending receive packet to a buffer of size *\b{maxlen).
\ ** Return the number of bytes received, or zero for a bad packet.
\ ** Note that data is valid only when *\fo{ISRX?} returns true,
\ ** so that you must poll with *\fo{ISRX?} before using
\ ** *\fo{GET_ETHER_PKT}. The received size of the packet
\ ** (not clipped to the buffer size) is returned.
\ ** In this implementation, a destination address of 0 indicates
\ ** that the packet should be discarded, and no memory transfer
\ ** occurs.
  NextRxDesc @ dup 0=			\ sanity check
  if  drop 2drop 0  exit  endif
  \ -- *dest maxlen desc
  dup ed.addr be-l@  swap ed.len be-w@ 	\ -- *dest maxlen buff len ; len/crc?
  dup >r  2swap rot min			\ buff *dest len' -- ; R: -- len'
  over					\ check dest address
  if  cmove  else  3drop  endif		\ -- ; R: -- len'
  /ERxBuff NextRxDesc @ ed.len be-w!	\ len
  NextRxDesc @ ed.status be-w@ bit13 and	\ extract W bit
  bit15 or  NextRxDesc @ ed.status be-w!	\ give buffer back to DMA
  NextRxDesc off			\ mark no current Rx packet
  0 _FEC fecRDAR + !			\ trigger FEC receiver
  r>					\ return length
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer
\ ** memory.
  0 0 get_ether_pkt drop  ;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop  NextTxDesc @ ed.status be-w@ bit15 and 0=  ;

: send_ether_pkt  \ caddr len --
\ *G Send packet from supplied buffer.

\ Perform length sanity checks
  dup 0=
  if  2drop exit  endif
  dup /ETxBuff u>
  if  2drop exit  endif
  #64 max

\ -- caddr len ; Wait for a free buffer
  begin  dup IsTx? 0= while  pause  repeat	\ wait until Tx available
  dup NextTxDesc @ ed.len be-w!		\ give length to DMA
  NextTxDesc @ ed.addr be-l@ swap cmove	\ copy buffer
  NextTxDesc @ ed.status be-w@ bit13 and  	\ get status W bit
  $8C00 or				\ force R, L, TC bits
  NextTxDesc @ ed.status be-w!		\ give to DMA
  0 _FEC fecTDAR + !			\ trigger FEC transmitter
  NextTxDesc @ stepTxDesc NextTxDesc !	\ step to next descriptor
;


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************

EtherCom? [if]

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O IPDGetAddr function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the corresponding stored data
\ ** will not be changed.
  drop					\ mode flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove		\ save
    SetMACaddress			\ put in EMAC
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;

\ *P The layout and usage of the Ethernet common I/O structure is
\ ** defined in the file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

create E52xxVector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** *\i{COMMON\ETHERCOM.FTH}.
  ' InitEther ,         \ initialisation
  ' noop ,	        \ shutdown
  ' IsRx? ,	        \ receive test
  ' get_ether_pkt ,     \ receive packet
  ' IsTx? ,	        \ transmit test
  ' send_ether_pkt ,    \ transmit packet
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

E52xxVector constant IPDevice	\ -- addr
\ *G *\fo{IPDevice} is the default device name required by
\ ** *\{ETHERCOM.FTH}. If you have multiple ports, only one
\ ** should be called *\fo{IPDevice}.

[then]


\ **************
\ *S Diagnostics
\ **************
\ *P This code is compiled if *\fo{EtherDiags?} is non-zero.

EtherDiags? [if]

: .phy		\ --
\ *G Display contents of all Ethernet Controller's PHY registers.
  cr ."  0 Control: " 0 readphy .word
  cr ."  1  Status: " 1 readphy .word
  cr ."  2 PhyID#1: " 2 readphy .word
  cr ."  3 PhyID#2: " 3 readphy .word
  cr ."  4 AutoNeg: " 4 readphy .word
  cr ."  5 Ability: " 5 readphy .word
  cr ."  6 Autoexp: " 6 readphy .word
\  cr ." 16 Config1: " #16 readphy .word
\  cr ." 17 Conf/St: " #17 readphy .word
\  cr ." 18 10TC/St: " #18 readphy .word
;

MultiPhy? [if]
: .phys		\ --
\ *G Compiled if the equate *\fo{MultiPhy?} is non-zero.
\ ** Displays the register contents of all PHYs.
  #32 0 do
   cr ." Phy# " i .byte  i to Phy#
   .phy
  loop
;
[then]

[then]


\ **************
\ *S System test
\ **************
\ *P The packet sniffer code is only compiled if the equate
\ ** *\fo{SNIFF?} is non-zero.

sniff? [if]

1536 buffer: pbuff	\ -- addr
\ *G Temporary buffer for *\fo{SNIFF}.

: NextRxPkt	\ --
\ *G Collect and display packet
  pbuff 1536 erase
  pbuff 1536 get_ether_pkt dup .word
  pbuff swap 1 max dump
;

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  begin
    isrx?
    if  NextRxPkt  endif
    #10 ms
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]

\ *P The PING example is used to test Ethernet transmission and
\ ** reception. Before testing transmission, test reception by
\ ** using the maintenance system to download a user application
\ ** and then pinging the target board instead of TFTPing to it.
\ *E Example ARP request:  #42/$2A bytes
\ ** C020.0600  FF FF FF FF FF FF 00 C0  F0 0D 9D 7E 08 06 00 01
\ ** C020.0610  08 00 06 04 00 01 00 C0  F0 0D 9D 7E C0 A8 01 7F
\ ** C020.0620  00 00 00 00 00 00 C0 A8  FF FE 11 0A 75 9C C0 A8

\ *P Note that the equates *\fo{SNIFF?} and *\fo{PINGIT?} must both
\ ** be set non-zero to compile the PING code.

pingit? [if]

create requested-ip	\ -- addr
\ *G Holds IP address to be pinged.
  #192 c,  #168 c,  0 c,  50 c,

create zero-mac		\ -- addr
\ *G Holds MAC address 00-00-00-00-00-00
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

create EtherBC		\ -- addr
\ *G Holds MAC address FF-FF-FF-FF-FF-FF
  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

$0001 equ ARPHARD_ETHER
$0800 equ ARPPROTO_IP

$0001 equ ARPOP_REQUEST
$0002 equ ARPOP_REPLY

: SendArpReq	\ -- ; send ARP request
\ *G Send an ARP request.
\ ** All initialisation must have been performed before
\ ** using this word.
  pbuff >r

  EtherBC r@ EHDR.DstMac 6 cmove
  EtherAddress   r@ EHDR.SrcMac 6 cmove
  ET_ARP         r@ EHDR.type w!(n)

  ARPHARD_ETHER  r@ AHDR.htype w!(n)
  ARPPROTO_IP    r@ AHDR.ptype w!(n)
  6              r@ AHDR.hlen c!
  4              r@ AHDR.plen c!
  ARPOP_REQUEST  r@ AHDR.op w!(n)

  EtherAddress   r@ ARP.SenderMAC 6 cmove
  IpAddress      r@ ARP.SenderIP 4 cmove
  zero-mac       r@ ARP.TargetMAC 6 cmove
  requested-ip   r@ ARP.TargetIP 4 cmove

  r> ARP send_ether_pkt
;

: pingonce	\ -- ; an ARP request
\ *G Send an ARP request and process the response once.
\ ** All initialisation must have been performed before
\ ** using this word.
  cr  ticks				\ starting time
  SendArpReq

  #1000 later
  begin
    isrx? if
      pbuff #1536 get_ether_pkt .
      pbuff EHDR.type w@(n) ET_ARP =
      pbuff AHDR.op w@(n) ARPOP_REPLY =  and
      pbuff ARP.targetMAC EtherAddress 6 s=  and
    else
      dup expired dup
      if  ." Timed out "  endif
    endif
  until
  drop
  ticks swap - . ." ms"			\ elapsed time
;

: pingit	\ --
\ *G Ping the IP address at *\fo{REQUESTED-IP} until a key is pressed.
  InitEther
  begin
    pingonce  #500 ms
    key?
  until
  ( Stop-Timers )
;

[then]


\ ******
\ *> ###
\ ******

decimal
