\ EtherSTM32F4xx.fth - STM32F4xx Ethernet driver

((
Copyright (c) 2012, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

http://en.wikipedia.org/wiki/Management_Data_Input/Output

To do
=====

Change history
==============
20131008 SFP002 Moved hardware set up to a separate file.
20120125 SFP001 First release
))

decimal


\ ==============
\ *! etherstm32f4xx
\ *T STM32F4xx Ethernet Driver Code
\ ==============
\ *S Introduction
\ *P The Ethernet code in *\i{Cortex\Drivers\EtherSTM32F4xx.fth}
\ ** is derived from sample C drivers from the ST MCD Application
\ ** Team; see *\f{http://www.st.com}.
\ ** The code was initially tested on the STM3240G-EVAL board.
\ ** It has also been tested on the Olimex STM32-E407 board.

\ *P Hardware set up code has now been moved to separate files.


\ ****************
\ *S Configuration
\ ****************
\ *P The following must be defined before the file
\ ** *\i{Cortex\Drivers\EtherSTM32F4xx.fth} is compiled.

interpreter
: const equ ;	\ n -- ; -- n
\ *G If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** *\fo{CONST} as *\fo{CONSTANT}, otherwise by default *\fo{CONST}
\ ** is defined as *\fo{EQU}.
target

[undefined] EtherCom? [if]
0 equ EtherCom?	\ -- flag
\ *G Only needed if the EtherCom generic I/O device is being used.
\ ** This only happens for systems with multiple IP ports and
\ ** PowerNet v3.x upwards.
[then]

[undefined] EtherDiags? [if]
0 equ EtherDiags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. False by default.
[then]

[undefined] MultiPhy? [if]
0 equ MultiPhy?	\ -- flag
\ *G Set this flag true if you are using multiple PHYs.
[then]

[undefined] FindPhy? [if]
1 equ FindPhy?	\ -- flag
\ *G Set this non-zero if you are using a single PHY but do not
\ ** know its address.
[then]

[undefined] Phy# [if]
0 value Phy#	\ -- phy#
\ *G When a known single PHY is being used, predefine *\fo{MultiPhy?}
\ ** and *\fo{FindPhy?} as zero and *\fo{Phy#} as an equate.
[then]

\ *P The equates *\fo{MultiPhy?}, *\fo{FindPhy?}, and *\fo{Phy#}
\ ** interact. Be sure to set them sensibly. Note that when
\ ** *\fo{MultiPhy?} or *\fo{FindPhy?} are true, *\fo{Phy#}
\ ** must be defined as a *\fo{VALUE}.

[undefined] RMII? [if]
0 equ RMII?	\ -- flag
\ *G Set non-zero if the board uses the RMII interface to the PHY.
\ ** The STM3240G-EVAL board is supplied pre-configured with the
\ ** PHY set to use MII communication. The board must be modified
\ ** to use RMII. This code is for MII mode only.
[then]

[undefined] sniff? [if]
0 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.
[then]

[undefined] pingit? [if]
0 equ pingit?	\ -- flag
\ *G Set this equate true to compile the PING code,
\ ** which tests Ethernet transmission and reception.
[then]

[undefined] EtherAddress [if]
create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $51 c,	\ six bytes for 48 bit MAC address
\  $00 c, $08 c, $EF c, $00 c, $5E c, $C4 c,	\ six bytes for 48 bit MAC address
[then]						\ you must obtain these yourself!

[undefined] IpAddress [if]
create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
\  10 c,  102 c,  0 c,  11 c,		\ assign these as required
  192 c,  168 c,  0 c,  251 c,		\ assign these as required
[then]


\ **********
\ *S Buffers
\ **********

#1536 equ /ERxBuff	\ -- u
\ *G Size of an Ethernet receive packet (1522) rounded up to the
\ ** next 16 byte unit.
#4 equ #ERxBuffs		\ -- u
\ *G Number of receive buffers. Must be at least two.
#1536 equ /ETxBuff	\ -- u
\ *G Size of an Ethernet transmit buffer rounded up to 16 byte unit.
#2 equ #ETxBuffs		\ -- u
\ *G Number of transmit buffers. Must be at least two. Transmit buffers
\ ** are managed in software.

struct /ERxDesc	\ -- len
\ *G Structure defining a receive descriptor, which must
\ ** be on a cell boundary.
\ *[
  int erd.Status	\ status
  int erd.CtrlSizes	\ control and buffer 1/2 sizes
  int erd.buff1Addr	\ buffer 1 address
  int erd.buff2Addr	\ buffer 2 addr, or next descriptor addr
end-struct
\ *]

struct /ETxDesc	\ -- len
\ *G Structure defining a transmit descriptor, which must
\ ** be on a cell boundary.
\ *[
  int etd.ctrlStatus	\ control and status
  int etd.buffSizes	\ buffer 1/2 sizes
  int etd.buff1Addr	\ buffer 1 address
  int etd.buff2Addr	\ buffer 2 addr, or next descriptor addr
end-struct
\ *]

udata  16 bin-align  cdata
#ERxBuffs /ERxDesc * buffer: ERxDescs	\ -- addr
\ *G Base address of the receive headers. The receiver steps
\ ** through these headers to find a free buffer.
\ ** The headers should be in *\b{uncached} memory.

#ETxBuffs /ETxDesc * buffer: ETxDescs	\ -- addr
\ *G Base address of the transmit headers. The transmitter steps
\ ** through these headers to find a pending buffer.
\ ** The headers should be in *\b{uncached} memory.

#ERxBuffs /ERxBuff * buffer: ERxBuffs	\ -- addr
\ *G Base address of the receive buffers. Some buffer handling
\ ** techniques may require these buffers to be in uncached
\ ** memory.

#ETxBuffs /ETxBuff * buffer: ETxBuffs	\ -- addr
\ *G Base address of the transmit buffers. These should not be
\ ** in cached memory.

variable NextRxDesc	\ -- addr
\ *G Holds the address of the next read header to be processed
\ ** by the CPU. This is a change from previous drivers.
  0 NextRxDesc !
variable NextTxDesc	\ -- addr
\ *G Holds the address of the next transmit header to use.
  0 NextTxDesc !

: InitERxDescs	\ --
\ *G Initialise the receive descriptors in chain mode.
  ERxDescs NextRxDesc !			\ Set pointer to first descriptor
  ERxBuffs ERxDescs  #ERxBuffs 0 ?do	\ -- buff hdr
    bit31 over erd.Status !		\ owned by DMA
    /ERxBuff bit14 or over erd.CtrlSizes !  \ buffer size, RCH set = chained
    over over erd.buff1Addr !		\ buffer address
    i #ERxBuffs 1- <> if		\ set next descriptor, last points back
      dup /ErxDesc +  over erd.Buff2Addr !
    else				\ last one
      ErxDescs  over erd.Buff2Addr !
    endif
    swap /ERxBuff + swap /ERxDesc +	\ step to next
  loop					\ -- buff' hdr'
  2drop
;

: InitETxDescs	\ --
\ *G Initialise the transmit descriptors.
  ETxDescs NextTxDesc !			\ Set pointer to first descriptor
  ETxBuffs ETxDescs  #ETxBuffs 0 ?do	\ -- buff desc
    $0010:0000 over etd.CtrlStatus !	\ owned by CPU, TCH (bit20) set
    0 over etd.BuffSizes !		\ no data
    over over etd.buff1Addr !		\ buffer address
    i #EtxBuffs 1- <> if		\ set next descriptor, last points back
      dup /ETxDesc +  over etd.buff2Addr !
    else				\ last one
      ETxDescs  over etd.buff2Addr !
    endif
    swap /ETxBuff + swap /ETxDesc +	\ step to next
  loop					\ -- buff' desc'
  2drop
;


\ *************
\ *S PHY access
\ *************

\ Standard PHY registers 0..F
0x00 const PHY_CNTL_REG
0x01 const PHY_STATUS_REG
0x02 const PHY_ID_REG1
0x03 const PHY_ID_REG2
0x04 const PHY_ANA_REG
0x05 const PHY_ANLPAR_REG
0x06 const PHY_ANE_REG

\ PHY control register
bit15 const RESET_PHY
bit14 const ENABLE_LOOPBACK
bit13 const DR_100MB
bit12 const ENABLE_AN
bit10 const PHY_MAC_ISOLATE
bit9  const RESTART_AN
bit8  const PHY_FULLDUPLEX
bit7  const PHY_COL_TEST

\ PHY Status register
bit15 const CAP_100BT4
bit14 const CAP_100BTX_FDX
bit13 const CAP_100BTX
bit12 const CAP_10BT_FDX
bit11 const CAP_10BT

bit6 const NO_PREAMBLE
bit5 const AN_COMPLETE
bit4 const REMOTE_FAULT
bit3 const AN_ABILITY
bit2 const LINK_UP
bit1 const JABBER
bit0 const CAP_EXTENDED

\ PHY Auto-negotiation Advertisement register
bit8 const DR100_TX_FULL
bit7 const DR100_TX_HALF
bit6 const DR10_TX_FULL
bit5 const DR10_TX_HALF
DR100_TX_FULL DR100_TX_HALF or DR10_TX_FULL or
  const ANE_ALL	\ request everything

MultiPhy? FindPhy? or [if]
: ~Phy#		\ -- mask
\ *G Multi PHY of unknown PHY: Mask used for PHY command generation.
  Phy# #23 lshift
;
[else]
Phy# #23 lshift equ ~Phy#	\ -- mask
\ *G Single PHY: Mask used for PHY command generation.
[then]

interpreter
: genMDC	\ -- u
\ *G Calculates the initial clock config for the PHY in MIIAR.
  system-speed dup 100 MHz >=
  if  drop %001 exit  endif
  dup 60 MHz >=
  if  drop %000 exit  endif
  dup 35 MHz >=
  if  drop %011 exit  endif
  dup 20 MHz >=
  if  drop %010 exit  endif
  abort" SYSTEM-SPEED cannot generate MDC clock"
;
target
genMDC 2 lshift equ /initMIIAR
\ *G Initial configuration of MIIAR.

: configPHY	\ --
\ *G Configure the MAC to read and write the PHY.
  /initMIIAR _ETH ethMACMIIAR + !
;

: WaitPhyDone	\ --
\ *G Wait until the current PHY operation has completed.
  begin
    pause  _ETH ethMACMIIAR + @ bit0 and 0=
  until
;

: ReadPhy	\ reg -- w
\ *G Read a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  $01F and				\ purify data
  _ETH dup ethMACMIIAR + @		\ -- reg *eth data
    $FFFF:003C and			\ keep clock info and reserved bits
    Phy# 11 lshift or			\ PHY address
    rot 6 lshift or			\ REG number
    bit0 or				\ set MII Busy bit
  swap ethMACMIIAR + !			\ write bit1=0=read
  WaitPhyDone
  _ETH ethMACMIIDR + @ $FFFF and
;

: WritePhy	\ w reg --
\ *G Write a PHY register. Note that these are 16 bit values.
  WaitPhyDone
  swap $FFFF and  swap $01F and		\ purify data
  _ETH tuck  2swap ethMACMIIDR + !	\ -- reg *eth ; store data
    dup ethMACMIIAR + @			\ -- reg *eth data
    $FFFF:003C and			\ keep clock info
    Phy# 11 lshift or			\ PHY address
    rot 6 lshift or			\ REG number
    bit1 or				\ bit1=1=write
    bit0 or				\ set MII Busy bit
  swap !				\ write
  WaitPhyDone
;

: OrPhy		\ mask reg --
\ *G Or mask into reg in the PHY. Set bits in mask are set in the
\ ** register.
  tuck ReadPhy  or  swap WritePhy
;

: resetPhy	\ --
\ *G Initialise the PHY with autonegotiation enabled.
  configPHY				\ set up the PHY
  RESET_PHY PHY_CNTL_REG OrPhy		\ software reset the chip
  begin					\ wait until reset complete
    PHY_CNTL_REG ReadPhy RESET_PHY and 0=
  until
  ANE_ALL PHY_ANA_REG OrPhy		\ request 10/100BaseT features
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
;

: autoNegotiate	\ -- ior ; 0=success
\ *G Perform the autonegotiation sequence. If no link can be
\ ** established, return ior=non-zero. On success, return zero.
\ ** For most PHYs this is done by a timeout, say 5 seconds.
  ENABLE_AN PHY_CNTL_REG OrPhy		\ enable autonegotiation
\ cr ." Starting AutoNegotiation"
  RESTART_AN PHY_CNTL_REG OrPhy		\ start autonegotiation
  5000 later				\ limit to 5 seconds
  begin
    pause  dup timedout? 0=		\ wait until done
   while
    PHY_STATUS_REG ReadPhy AN_COMPLETE and
  until then
  timedout?
  if  -1  exit  endif
\ cr ." AutoNegotiation done"
\ set up control reg using the results
  PHY_ANLPAR_REG ReadPhy
  0					\ -- partner cntlset
  over DR100_TX_FULL and
  if  DR_100MB PHY_FULLDUPLEX or or  endif
  over DR100_TX_HALF and
  if  DR_100MB or  endif
  over DR10_TX_FULL and
  if  PHY_FULLDUPLEX or  endif
  PHY_CNTL_REG WritePhy
  drop 0
;

: EtherLink?	\ -- flag
\ *G Return true if the Ethernet link is established. For most
\ ** PHYs, this is performed by testing bit 2 of register 1.
\ ** For more efficient operation, it is usually possible to
\ ** generate a link status change interrupt from the PHY; but
\ ** setting this up varies between PHYs. The polling mechanism
\ ** works everywhere.
  1 ReadPhy bit2 and 0<>
;

FindPhy? [if]
: FindPhy	\ -- phy|-1
\ *G Scan for PHYs. Return the number of the first PHY found, or
\ ** -1 if no PHY is found. *\fo{Phy#} is set for each PHY tested.
  -1  #32 0 do				\ range of possible Phy addresses 0..$1F
    i to Phy#
    PHY_ID_REG1 ReadPhy $FFFF <>	\ IDreg never FFFF
    if  drop i leave  endif
  loop
;
[then]


\ **************************
\ *S Default hardware set up
\ **************************
\ *P The word *\fo{InitEtherHw} must be provided in a separate file.
\ ** The commented out example code below is for the STM3240G-EVAL
\ ** board from ST. It operates in MII mode, 25 MHz from MCO.

((
  ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1, AF11
  ETH_MDIO -------------------------> PA2, AF11
  ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7, AF11

  ETH_PPS_OUT ----------------------> PB5, AF11
  ETH_MII_TXD3 ---------------------> PB8, AF11

  ETH_MDC --------------------------> PC1, AF11
  ETH_MII_TXD2 ---------------------> PC2, AF11
  ETH_MII_TX_CLK -------------------> PC3, AF11
  ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4, AF11
  ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5, AF11

  ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11, AF11
  ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13, AF11
  ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14, AF11

  ETH_MII_CRS ----------------------> PH2, AF11
  ETH_MII_COL ----------------------> PH3, AF11
  ETH_MII_RXD2 ---------------------> PH6, AF11
  ETH_MII_RXD3 ---------------------> PH7, AF11

  ETH_MII_RX_ER --------------------> PI10, AF11
))

((
0 1 pio: PA1		\ ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1, AF11
0 2 pio: PA2		\ ETH_MDIO -------------------------> PA2, AF11
0 7 pio: PA7		\ ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7, AF11
1 5 pio: PB5		\ ETH_PPS_OUT ----------------------> PB5, AF11
1 8 pio: PB8		\ ETH_MII_TXD3 ---------------------> PB8, AF11
2 1 pio: PC1		\ ETH_MDC --------------------------> PC1, AF11
2 2 pio: PC2		\ ETH_MII_TXD2 ---------------------> PC2, AF11
2 3 pio: PC3		\ ETH_MII_TX_CLK -------------------> PC3, AF11
2 4 pio: PC4		\ ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4, AF11
2 5 pio: PC5		\ ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5, AF11
6 11 pio: PG11		\ ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11, AF11
6 13 pio: PG13		\ ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13, AF11
6 14 pio: PG14		\ ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14, AF11
7 2 pio: PH2		\ ETH_MII_CRS ----------------------> PH2, AF11
7 3 pio: PH3		\ ETH_MII_COL ----------------------> PH3, AF11
7 6 pio: PH6		\ ETH_MII_RXD2 ---------------------> PH6, AF11
7 7 pio: PH7		\ ETH_MII_RXD3 ---------------------> PH7, AF11
8 10 pio: PI10		\ ETH_MII_RX_ER --------------------> PI10, AF11

0 8 pio: PA8		\ AF0, used for MCO - requires SYSCFG
1 14 pio: PB14		\ used as EXTINT from PHY

: isEtherPin	\ pin --
\ *G Set the pin to AF11 and 50MHz.
  11 over isFunction  50MHZ_fs swap isPinSpeed
;

: initEtherHw	\ --
\ *G Set up the clocks and pins required for Ethernet operation
\ ** in MII mode for the STM3240G-EVAL board.
\ GPIO already clocked by gpioSTM32F4xx.fth.

  bit14 _RCC 2dup
  rccAPB2ENR + or!  rccAPB2RSTR + bic!	\ enable SYSCFG clock, out of reset

  PA8					\ configure PA8 for MCO
  0 over isFunction  0 over isPinOD
  dup notPulled  100MHZ_hs swap isPinSpeed
  $0040:0000 $0760:0000 _RCC rccCFGR + setMask \ divide by 1 from HSE

  PA1 isEtherPin			\ all pins to AF11, Ethernet
  PA2 isEtherPin
  PA7 isEtherPin
  PB5 isEtherPin
  PB8 isEtherPin
  PC1 isEtherPin
  PC2 isEtherPin
  PC3 isEtherPin
  PC4 isEtherPin
  PC5 isEtherPin
  PG11 isEtherPin
  PG13 isEtherPin
  PG14 isEtherPin
  PH2 isEtherPin
  PH3 isEtherPin
  PH6 isEtherPin
  PH7 isEtherPin
  PI10 isEtherPin

  bit25 _RCC rccAHB1RSTR + or!		\ MAC into reset
[ RMII? ] [if]
  bit23 _SYSCFG sysPMC + or!
[else]
  bit23 _SYSCFG sysPMC + bic!
[then]
  bit25 _RCC rccAHB1RSTR + bic!		\ MAC out of reset
  $0E00:0000 _RCC rccAHB1ENR + or!	\ enable MAC clocks except PTP
  _ETH
  bit0 over ethDMABMR + or!		\ software reset of DMA and MAC
  begin
    dup ethDMABMR + @ bit0 and 0=
  until
  drop
;
))


\ *****************
\ *S Initialisation
\ *****************

((
For a MAC address conventionally defined as:
  ff-ee-dd-cc-bb-aa
the layout of the MAL/H registers is:
MAH:
        ---------------------
    MSB | 00 | 00 | aa | bb | LSB
        ---------------------
MAL:
        ---------------------
    MSB | cc | dd | ee | ff | LSB
        ---------------------

N.B. This is drawn from the ST evaluation code.
))

: setMACAddress	\ addr --
\ *G Set the MAC address in the EMAC unit.
  dup 5 + c@ 8 lshift  over 4 + c@ or	\ -- addr x16
  _ETH ethMACA0HR + !
  dup 3 + c@ #24 lshift
  over 2 + c@ #16 lshift or
  over 1 + c@ 8 lshift or
  swap c@ or
  _ETH ethMACA0LR + !
;

: StartRxEngine	\ -- flag ; nz=success
\ *G Kick off the RX DMA Engine.
  NextRxDesc @ 0=			\ Check that RX Ring buffers have been initialised
  if  0 exit  endif			\   not initialised, oops.
  _ETH
  bit1 over ethDMAOMR + bic!		\ stop reception
  ERxDescs over ethDMARDLAR + !		\ set pointer
  bit1 over ethDMAOMR + or!		\ start DMA Rx
  bit2 over ethMACCR + or!		\ enable MAC Rx
  0 over ethDMARPDR + !			\ start DMA polling
  drop
  true					\ If we got here, it worked
;

: StartTxEngine	\ -- flag ; nz=success
\ *G Kick off the TX DMA Engine.
  NextTxDesc @ 0=			\ Check that TX Ring buffers have been initialised
  if  0 exit  endif			\   not initialised, oops.
  _ETH
  bit13 over ethDMAOMR + bic!		\ stop transmission
  ETxDescs _ETH ethDMATDLAR + !		\ set pointer
  bit13 over ethDMAOMR + or!		\ start DMA Tx
  bit3 over ethMACCR + or!		\ enable MAC Tx
  drop
  true					\ If we got here, it worked
;

: initMACregs	\ --
\ *G Initialise the MAC registers.
  _ETH
  dup ethMACCR + @
   $02CF:7EFC invert and		\ clear non-reserved bits in MACCR
   bit14 bit11 or or			\ assume 100Mb, Full Duplex
   					\ bit3, bit2 for TE, RE
  over ethMACCR + !
  0 over ethMACFFR + !			\ receive perfect dest and broadcast only
  0 over ethMACFCR + !			\ PAUSE frames rarely used nowadays
  0 over ethMACVLANTR + !
  drop
;

: initDMAregs	\ --
\ *G Initialise the DMA registers.
  _ETH
  dup ethDMAOMR + @			\ last DMA control register
    $0731:E0DE invert and		\ clear relevant bits
					\ bit 13 and bit 1 to start Tx/Rx
  over ethDMAOMR + !
  drop
;

: cfgMACfromPHY	\ --
\ *G Configure the MAC from the PHY autonegotation results.
  PHY_CNTL_REG readPhy
  dup DR_100MB and if			\ set/clear 100Mb FES
    bit14 _ETH ethMACCR + or!
  else
    bit14 _ETH ethMACCR + bic!
  endif
  PHY_FULLDUPLEX and if			\ set/clear Duplex Mode
    bit11 _ETH ethMACCR + or!
  else
    bit11 _ETH ethMACCR + bic!
  endif
;

: InitEMAC	\ -- flag ; zero=success
\ *G Initialisation of the Ethernet and PHY, including the MAC
\ ** address.
  initEtherHw  initDMAregs  initMACregs
  EtherAddress setMACaddress		\ set up MAC address
[ FindPhy? ] [if]
  configPhy  findPhy -1 =		\ find PHY, check error
  if  -3 exit  endif
[then]
  resetPhy autoNegotiate
  if  -2  exit  endif
  cfgMACfromPHY				\ apply results from PHY
  InitERxDescs				\ set up receive headers
  InitETxDescs				\ set up receive headers
  StartRxEngine				\ Kick off RX DMA
  StartTxEngine				\ Kick off TX DMA
  and 0=				\ -1 on failure
;

: InitEther	\ --
\ *G Perform a full initialisation of the chip, enabling
\ ** Rx and Tx, and setting up the Ethernet MAC address
\ ** from *\fo{EtherAddress}.
  cr ." Initialising Ethernet ... "
  InitEMAC
  if ." un" then ." successful"
;
\ ...SFP001


\ ********************
\ *S Packet read/write
\ ********************

: doTxPoll	\ --
\ *G Force DMA to poll the transmit descriptor list.
  0 _ETH ethDMATPDR + !			\ force transmit poll
;

: doRxPoll	\ --
\ *G Force DMA to poll the receive descriptor list.
  0 _ETH ethDMARPDR + !			\ start DMA polling
;

: StepOnRxDesc	\ --
\ *G Give current header back to the DMA engine and move on to
\ ** the next one.
  NextRxDesc @
  bit31 over erd.Status !		\ owned by DMA
  erd.Buff2Addr @ NextRxDesc !		\ next descriptor
;

: IsRx?         \ -- flag ; nz = has data
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
  NextRxDesc @ erd.Status @ 		\ look at status word in next header
  dup bit31 and 0= if			\ -- status ; check marked "not for DMA", i.e. has data
    $8300 and $0300 = if		\ Look at staus/error bits in the word
      true				\ good packet
    else
      StepOnRxDesc doRxPoll false	\ ditch duff packet
    endif
  else
    drop 0				\ discard status word, return 0
  then
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get a pending receive packet to a buffer of size *\b{maxlen).
\ ** Return the number of bytes received, or zero for a bad packet.
\ ** The received size of the packet (not clipped to the buffer
\ ** size) is returned.
\ ** In this implementation, a destination address of 0 indicates
\ ** that the packet should be discarded, and no memory transfer
\ ** occurs.
  IsRx? 0= if  2drop 0 exit  endif	\ Make sure we've got some data to read
  NextRxDesc @				\ -- *dest maxlen desc
  dup erd.Buff1Addr @ swap		\ -- *dest maxlen buff desc
  erd.Status @ #16 rshift $3FFF and ( 4 - )	\ -- *dest maxlen buff len
  dup >r  2swap rot min			\ buff *dest len' -- ; R: -- len'
  over					\ check dest address
  if  aligned cmove  else  3drop  endif	\ -- ; R: -- len'
  StepOnRxDesc doRxPoll
  r>					\ return length
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of host buffer
\ ** memory.
  0 0 get_ether_pkt drop
;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  drop  NextTxDesc @ bit31 and 0=
;

: send_ether_pkt	\ buff len --
\ *G Send a packet from a supplied buffer. We copy the data to a
\ ** DMA buffer so that the buffer can be released immediately.

\ Perform length sanity checks
  dup 0=
  if  2drop exit  endif
  dup /ETxBuff u>
  if  2drop exit  endif

  #60 max				\ 64 bytes with FCS 32 bit CRC

\ Wait for TX buffer to become available
  NextTxDesc @				\ -- buff len desc
  begin  dup  bit31 and while  pause  repeat
\ Copy packet to TX buffer
  dup >r  etd.Buff1Addr @		\ -- buff len etxb ; R: -- desc
  rot swap 2 pick			\ -- len buff etxb len
  aligned cmove				\ -- len ; R: -- desc ; copy, align for fast CMOVE
  r> tuck etd.BuffSizes !		\ -- desc ; write length
  $B010:0000 over etd.CtrlStatus !	\ give packet to DMA
  etd.Buff2Addr @ NextTxDesc !		\ step on
  doTxPoll				\ force transmit poll
;


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************
\ *P You only need this section if you have multiple ports.

EtherCom? [if]

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O IPDGetAddr function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the corresponding stored data
\ ** will not be changed.
  drop					\ mode flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove		\ save
    SetMACaddress			\ put in EMAC
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;


\ *P The layout and usage of the Ethernet common I/O structure is
\ ** defined in the file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

create E92vector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** *\i{COMMON\ETHERCOM.FTH}.
  ' InitEther ,         \ initialisation
  ' noop ,	        \ shutdown
  ' IsRx? ,	        \ receive test
  ' get_ether_pkt ,     \ receive packet
  ' IsTx? ,	        \ transmit test
  ' send_ether_pkt ,    \ transmit packet
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

E92vector constant IPDevice	\ -- addr
\ *G IPDevice is the default device name required by *\{ETHERCOM.FTH}.
\ ** If you have multiple ports, only one should be called *\fo{IPDevice}.

[then]


\ ==============
\ *S Diagnostics
\ ==============

EtherDiags? [if]

_ETH constant _ETH
_RCC constant _RCC

: t1		\ --
\ *G Ethernet initialisation.
  initEtherHw initDMAregs  initMACregs
  configPHY  0 -> Phy#
;

: tPhy		\ --
\ *G Check MDC/MDIO with a scope.
  begin  2 readPHY drop  key? until  key drop
;

: tn		\ addr -- addr'
\ *G No access to structure methods from compiled code, so this
\ ** is to work out where the pointer is going...
  etd.Buff2Addr
;

: .erd		\ --
\ *G A test word to dump the current state of the Rx descriptors.
  ERxDescs  #ERxBuffs /ERxDesc * pdump
;

: .etd		\ --
\ *G A test word to dump the current state of the TX descriptors
  ETxDescs  #ETxBuffs /ETxDesc * pdump
;

: viewpackets	\ --
\ *G Show the first 16 bytes of each buffer
  cr
  #ERxBuffs 0 do
    i dup . ." : " /ERxBuff * ERxBuffs +		\ -- buffptr
    #16 0 do
      dup i + c@ .
    loop drop cr
  loop
;

: readnext	\ --
\ *G Display value of next packet status and drop buffer if there
\ ** is one.
  IsRx? if
    ." Status is " NextRxDesc @ erd.Status @ .dword cr
    DiscardRX
  then
;

: .phy		\ --
\ *G Display contents of all Ethernet Controller's PHY registers.
  cr ."  0 Control: " 0 readphy .word
  cr ."  1  Status: " 1 readphy .word
  cr ."  2 PhyID#1: " 2 readphy .word
  cr ."  3 PhyID#2: " 3 readphy .word
  cr ."  4 AutoNeg: " 4 readphy .word
  cr ."  5 Ability: " 5 readphy .word
  cr ."  6 Autoexp: " 6 readphy .word
  cr ." 17 Conf/St: " #17 readphy .word
  cr ." 18 TCIE:    " #18 readphy .word
;

MultiPhy? FindPhy? or [if]
: .Phys		\ --
\ *G Displays the register contents of all PHYs. Compiled if
\ ** either of the equates *\fo{MultiPhy?} or *\fo{FindPhy?} are
\ ** non-zero.
  #32 0 do
    cr ." PhyA " i .byte space
    i to Phy#  .Phy
  loop
;
[then]

[then]


\ **************
\ *S System test
\ **************
\ *P The packet sniffer code is only compiled if the equate
\ ** *\fo{SNIFF?} is non-zero.

sniff? [if]

1536 buffer: pbuff	\ -- addr
\ *G Temporary buffer for *\fo{SNIFF}.

: NextRxPkt	\ --
\ *G Collect and display packet
  pbuff 1536 erase
  pbuff 1536 get_ether_pkt dup .word
  pbuff swap 1 max dump
;

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  begin
    isrx?
    if  NextRxPkt  endif
    #10 ms
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]

\ *P The PING example is used to test Ethernet transmission and
\ ** reception. Before testing transmission, test reception by
\ ** using the maintenance system to download a user application
\ ** and then pinging the target board instead of TFTPing to it.
\ *E Example ARP request:  #42/$2A bytes
\ ** C020.0600  FF FF FF FF FF FF 00 C0  F0 0D 9D 7E 08 06 00 01
\ ** C020.0610  08 00 06 04 00 01 00 C0  F0 0D 9D 7E C0 A8 01 7F
\ ** C020.0620  00 00 00 00 00 00 C0 A8  FF FE 11 0A 75 9C C0 A8

\ *P Note that the equates *\fo{SNIFF?} and *\fo{PINGIT?} must both
\ ** be set non-zero to compile the PING code.

pingit? [if]

create requested-ip	\ -- addr
\ *G Holds IP address to be pinged.
  #192 c,  #168 c,  0 c,  50 c,

create zero-mac		\ -- addr
\ *G Holds MAC address 00-00-00-00-00-00
  0 c,  0 c,  0 c,  0 c,  0 c,  0 c,

create EtherBC		\ -- addr
\ *G Holds MAC address FF-FF-FF-FF-FF-FF
  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,  $FF c,

struct EHDR
  6 field EHDR.DstMAC
  6 field EHDR.SrcMAC
  2 field EHDR.type
end-struct

struct ARP
  EHDR field      _ehdr1
  2 field AHDR.htype
  2 field AHDR.ptype
  1 field AHDR.hlen
  1 field AHDR.plen
  2 field AHDR.op
  6 field ARP.senderMAC
  4 field ARP.senderIP
  6 field ARP.targetMAC
  4 field ARP.targetIP
end-struct

$0800 equ ET_IP
$0806 equ ET_ARP

$0001 equ ARPHARD_ETHER
$0800 equ ARPPROTO_IP

$0001 equ ARPOP_REQUEST
$0002 equ ARPOP_REPLY

: SendArpReq	\ -- ; send ARP request
\ *G Send an ARP request.
\ ** All initialisation must have been performed before
\ ** using this word.
  pbuff >r

  EtherBC r@ EHDR.DstMac 6 cmove
  EtherAddress   r@ EHDR.SrcMac 6 cmove
  ET_ARP         r@ EHDR.type w!(n)

  ARPHARD_ETHER  r@ AHDR.htype w!(n)
  ARPPROTO_IP    r@ AHDR.ptype w!(n)
  6              r@ AHDR.hlen c!
  4              r@ AHDR.plen c!
  ARPOP_REQUEST  r@ AHDR.op w!(n)

  EtherAddress   r@ ARP.SenderMAC 6 cmove
  IpAddress      r@ ARP.SenderIP 4 cmove
  zero-mac       r@ ARP.TargetMAC 6 cmove
  requested-ip   r@ ARP.TargetIP 4 cmove

  r> ARP send_ether_pkt
;

: pingonce	\ -- ; an ARP request
\ *G Send an ARP request and process the response once.
\ ** All initialisation must have been performed before
\ ** using this word.
  cr  ticks				\ starting time
  SendArpReq
  #1000 later begin
    isrx? if
      pbuff #1536 get_ether_pkt .
      pbuff EHDR.type w@(n) ET_ARP =
      pbuff AHDR.op w@(n) ARPOP_REPLY =  and
      pbuff ARP.targetMAC EtherAddress 6 s=  and
    else
      dup expired dup
      if  ." Timed out "  endif
    endif
  until
  drop
  ticks swap - . ." ms"			\ elapsed time
;

: pingit	\ --
\ *G Ping the IP address at *\fo{REQUESTED-IP} until a key is pressed.
  InitEther
  begin
    pingonce  #500 ms  key?
  until
  ( Stop-Timers )
;

[then]


\ ******
\ *> ###
\ ******

decimal

