\ RomCallsLM.fth - Luninary ROM services

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ =============
\ *! romcallslm
\ *T Calling the Luminary ROM services
\ =============
\ *P Luminary Stellaris devices include a ROM containing
\ ** primitives to drive most peripherals. Before using these,
\ ** consult the ROM user guide for the device you are using,
\ ** *\b{and} consult the device errata sheet.

code ROMcall-1/1	\ param1 priind secind -- res
\ *G ROM call with one parameter.
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r0, [ psp ], # 4		\ r0 = param
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code

code ROMcall-2/1	\ param1 param2 priind secind -- res
\ *G ROM call with two parameters.
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r1, [ psp ], # 4		\ r1 = param2
  ldr     r0, [ psp ], # 4		\ r0 = param1
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code

code ROMcall-3/1	\ param1 param2 param3 priind secind -- res
\ *G ROM call with three parameters.
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r2, [ psp ], # 4		\ r2 = param3
  ldr     r1, [ psp ], # 4		\ r1 = param2
  ldr     r0, [ psp ], # 4		\ r0 = param1
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code

code ROMcall-4/1	\ param1 param2 param3 param4 priind secind -- res
\ *G ROM call with four parameters.
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r3, [ psp ], # 4		\ r3 = param4
  ldr     r2, [ psp ], # 4		\ r2 = param3
  ldr     r1, [ psp ], # 4		\ r1 = param2
  ldr     r0, [ psp ], # 4		\ r0 = param1
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code


\ ====
\ GPIO
\ ====

#define SYSCTL_PERIPH_ETH       0x20105000  // ETH
#define SYSCTL_PERIPH_GPIOF     0x20000020  // GPIO F

: PeriphEnable	\ periph --
  #13 6 RomCall-1/1 drop  ;

: PeriphReset	\ periph --
  #13 5 RomCall-1/1 drop  ;

#define GPIO_PF2_LED1           0x00050801
#define GPIO_PF3_LED0           0x00050c01

: GPIOpinCfg	\ GPIO_Pxxx --
\ Not implemented in B1
  4 #26 RomCall-1/1 drop  ;

: GPIOdirModeSet	\ base pins mode --
  4 1 RomCall-3/1 drop  ;

#define GPIO_STRENGTH_2MA       0x00000001  // 2mA drive strength
#define GPIO_STRENGTH_4MA       0x00000002  // 4mA drive strength
#define GPIO_STRENGTH_8MA       0x00000004  // 8mA drive strength
#define GPIO_STRENGTH_8MA_SC    0x0000000C  // 8mA drive with slew rate control

#define GPIO_PIN_TYPE_STD       0x00000008  // Push-pull
#define GPIO_PIN_TYPE_STD_WPU   0x0000000A  // Push-pull with weak pull-up
#define GPIO_PIN_TYPE_STD_WPD   0x0000000C  // Push-pull with weak pull-down
#define GPIO_PIN_TYPE_OD        0x00000009  // Open-drain
#define GPIO_PIN_TYPE_OD_WPU    0x0000000B  // Open-drain with weak pull-up
#define GPIO_PIN_TYPE_OD_WPD    0x0000000D  // Open-drain with weak pull-down
#define GPIO_PIN_TYPE_ANALOG    0x00000000  // Analog comparator

: GPIOpadCfgSet		\ base pins strength type --
  4 5 RomCall-4/1 drop  ;


\ ========
\ Ethernet
\ ========

#define ETH_CFG_TS_TSEN         0x010000    // Enable Timestamp (CCP)
#define ETH_CFG_RX_BADCRCDIS    0x000800    // Disable RX BAD CRC Packets
#define ETH_CFG_RX_PRMSEN       0x000400    // Enable RX Promiscuous
#define ETH_CFG_RX_AMULEN       0x000200    // Enable RX Multicast
#define ETH_CFG_TX_DPLXEN       0x000010    // Enable TX Duplex Mode
#define ETH_CFG_TX_CRCEN        0x000004    // Enable TX CRC Generation
#define ETH_CFG_TX_PADEN        0x000002    // Enable TX Padding

: EtherInitExpClk	\ base EthClk --
  #15 1 RomCall-2/1 drop  ;

: EtherConfigSet	\ base config --
  #15 2 RomCall-2/1 drop  ;

: EtherMACset	\ base *MACaddr --
  #15 4 RomCall-2/1 drop  ;

((
: EtherRXavail	\ base -- bool
  #15 8 RomCall-1/1  ;

: EtherPacketGet	\ base caddr len -- len'
  #15 #11 RomCall-3/1  ;

: EtherTXavail	\ base -- bool
  #15 9 RomCall-1/1  ;

: EtherPacketSend	\ base caddr len -- len'
  #15 #13 RomCall-3/1  ;
))

: EtherPHYread	\ base reg# -- data
  #15 #18 RomCall-2/1  ;

: EtherPHYwrite	\ base reg# data --
  #15 #17 RomCall-3/1 drop  ;

: EtherEnable	\ base --
  #15 6 RomCall-1/1 drop  ;

: EtherDisable	\ base --
  #15 7 RomCall-1/1 drop  ;


\ ======
\ *> ###
\ ======

decimal
