\ SysTickCM4.fth - Cortex-M4 ticker

((
Copyright (c) 2010, 2011, 2012, 2018
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20181214 MPE006 Updated START-CLOCK.
20120517 MPE005 Update for timebase option.
20110704 MPE004 Generic CM4 version.
20110627 MPE003 LPC11xx conversion.
20110218 MPE002 Added timebase extensions.
20100125 MPE001 First release.
))

only forth definitions
decimal

\ ===========
\ *! systickcm4
\ *T M4 System Ticker
\ ===========
\ *P The system ticker uses the Cortex-M4 *\i{SysTick} timer.
\ ** Although the *\i{SysTick} timer is common to all Cortex-M4
\ ** devices, there is one clocking option. This code does not use
\ ** that option and so may be used by all CM4 devices.

\ *P The CM4 implementation is fed with the system clock and
\ ** has an option controlled by bit 2 of the
\ ** SysTick Control and Status Register (ARM DDI 0337, Rev E,
\ ** page 8-8, ARM DDI 0403D, page B3-746/7).
\ *(
\ *B Bit2=0: SysTick clock is vendor defined
\ *B Bit2=1: SysTick clock is CCLK.
\ *)
\ *P This code always uses CCLK.

tick-ms system-speed #1000 */ 1- equ /SysTick	\ -- x
\ *G Initial value of the SysTick Reload register.

[undefined] ticks [if]
0 value ticks
[then]

timebase? [if]	\ needs high-level ISR
\ *G If the timebase option is being used, we need the high-level
\ ** ISR handler.

: SysTicker	\ --
\ *G High level version of ticker ISR.
  tick-ms addr ticks +!			\ update ticker
  do-timers				\ run timebase code
;
' SysTicker SysTickvec# EXC: SysTickISR	\ -- addr
\ *G Setting the high level ISR.

[else]		\ can use low-level ISR

Proc SysTickISR	\ --
\ *G Ticker ISR in assembler.
\ --- CPU has saved ----
\   xPSR PC LR R12 R3 R2 R1 R0
  mvl32   r0, # ' ticks >body
  ldr     r1, [ r0, # 0 ]
  add .s  r1, r1, # tick-ms
  str     r1, [ r0, # 0 ]
  bx      lr
end-code
SysTickISR SysTickvec# setExcVec

\ Note that the following high level definition gives
\ essentially the same code but is dependent on register
\ allocation by the compiler.
((
: SysTicker	\ --
\ *G High level version of ticker ISR.
  tick-ms addr ticks +!			\ update ticker
;
dis t2
T2
( 0802:B820 40F22C11 @r,. )  mov r1, # $12C
( 0802:B824 C2F20001 Br.. )  movt r1, # $2000
( 0802:B828 0868 .h )        ldr r0, [ r1, # $00 ]
( 0802:B82A 401C @. )        add .s r0, r0, # $01
( 0802:B82C 0860 .` )        str r0, [ r1, # $00 ]
( 0802:B82E 7047 pG )        bx LR
16 bytes, 6 instructions.
))


[then]

: start-clock	\ --
\ *G Initialise the system ticker to run with a period of
\ ** *\fo{tick-ms} milliseconds.
  _SCS
  /SysTick over stRVR + !		\ set ticker rate
  0 over stCVR + !			\ clear current value
  $0007 over stCSR + !			\ enable it
\ bit 0 = enable, bit 1 = interrupt, bit 2 = clock source (1=sysclk, 0=sysclk/2)
  $FF over $0D23 + c!			\ set SysTick priority in SHPR3 to lowest
  drop
  ei
;

: stop-clock	\ --
\ *G Stop the sytem ticker.
  bit0 _SCS stCSR + bic!  ;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]


\ ********************
\ Time base extensions
\ ********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  stop-clock init-timers start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

decimal

