\ SysTickDisco072.fth - Cortex-M0 ticker for STM32F072B-DISCO board

((
Copyright (c) 2010, 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140207 SFP003 Converted for STM32F072B-DISCO board.
20110218 MPE002 Added timebase extensions.
))

only forth definitions
decimal

\ ===========
\ *> drivers
\ ===========
\ *S System Ticker
\ ****************
\ *P The system ticker uses the Cortex-M0 *\i{SysTick} timer.
\ ** Although the *\i{SysTick} timer is common to all Cortex-M0
\ ** devices, there is no guarantee that they have the same
\ ** clocking arrangements. Consequently there are different
\ ** drivers for different implementations.

\ *P The STM32 implementation is fed with the system clock and
\ ** has an option to divide it by 8, controlled by bit 2 of the
\ ** SysTick Control and Status Register (ARM DDI 0337, Rev E,
\ ** page 8-8).
\ *(
\ *B Bit2=0: SysTick clock is HCLK/8.
\ *B Bit2=1: SysTick clock is HCLK.
\ *)
\ *P This code always uses HCLK/8.


variable LedActive	\ -- flag
\ *P If *\fo{LedActive} contains non-zero, the ticker is used
\ ** to produce a rotating pattern on the four LEDs.
  1 LedActive !

PC 6 pio: LDU	\ -- struct ; LD3 on TIM3_CH1 AF0
\ *G Red/Upper LED.
PC 9 pio: LDR	\ -- struct ; LD5 on TIM3_CH4 AF0
\ *G Green/Right LED.
PC 7 pio: LDD	\ -- struct ; LD6 on TIM3_CH2 AF0
\ *G Blue/Down LED.
PC 8 pio: LDL	\ -- struct ; LD4 on TIM3_CH3 AF0
\ *G Orange/Left LED.

PA 0 pio: UButton	\ -- struct ; user button
\ *G User button.

variable ButtonState	\ -- addr
\ *G 1 = off, 0 = on
variable ButtonBits	\ -- addr
\ *G Last 32 bits
variable ButtonCounter	\ -- addr

[undefined] ticks [if]
0 value ticks		\ -- ms
\ *G Returns a 32 bit number of milliseconds that eventually
\ ** wraps.
[then]

: SysTicker	\ --
\ *G Ticker ISR action.
  tick-ms addr ticks +!			\ update ticker
  LedActive @ if
    case  ticks $7FF and		\ 0..2047
      $000 of  LDL clrPin  LDU setPin  endof
      $200 of  LDU clrPin  LDR setPin  endof
      $400 of  LDR clrPin  LDD setPin  endof
      $600 of  LDD clrPin  LDL setPin  endof
    endcase
  then
  ButtonBits @				\ add bit
    1 lshift UButton getPin or
  dup ButtonBits !
  case
    -1 of  1 ButtonState !  endof
     0 of
       ButtonState @ 1 = if
         0 ButtonState !
         ButtonCounter dup @ 1+ $0F and dup rot !
         console opvec !  decimal
         cr ." !set:#slider|value|" 0 .r cr
       endif
     endof
  endcase
;
' SysTicker SysTickvec# EXC: SysTickISR	\ -- addr
\ *G Setting the high level ISR.

tick-ms system-speed 8 / #1000 */ 1- equ /SysTick	\ -- x
\ +G Initial value of the SysTick Reload register.

internal
: initLD	\ struct --
  dup isOutput setPin
;
external

: start-clock	\ --
\ *G Initialise the system ticker to run with a period of
\ ** *\fo{tick-ms} milliseconds.
  LDU initLD  LDR initLD  LDD initLD  LDL initLD
  UButton isInput
  _SCS
  /SysTick over stRVR + !		\ set ticker rate
  $0003 over stCSR + !			\ enable, bit 0 = enable, bit 1=interrupt
					\ bit 2 = clock source (1=sysclk, 0=sysclk/8)
  $FF over $0D23 + c!			\ set SysTick priority to lowest
  drop
  ei
;

: stop-clock	\ --
\ *G Stop the sytem ticker.
  bit0 _SCS stCSR + bic!  ;

((
: later         \ n -- n'
\ *G Generates the timebase value for termination in
\ ** n millseconds time.
  ticks +
;

: timedout?     \ n -- flag ; true if timed out
\ *G Flag is returned true if the timebase value n has timed out.
\ ** *\fo{TIMEDOUT?} does not call *\fo{PAUSE}.
  ticks - 0<                            \ must use circular arithmetic
;

compiler
: later         \ n -- n'
  ticks +
;

: timedout?     \ n -- flag ; true if timed out
  ticks - 0<                            \ must use circular arithmetic
;
target

: ms            \ n --
\ *G Waits for n milliseconds.
  later begin
    pause  dup timedout?
  until
  drop
;
))


\ ======
\ *> ###
\ ======

decimal

