\ SysTick.fthLPC17xx - Cortex-M3 ticker for LPC17xx

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20110218 MPE002 Added timebase extensions.
20100320 MPE001 First release.
))

only forth definitions
decimal

\ ===========
\ *! systick
\ *T System Ticker
\ ===========
\ *P The system ticker uses the Cortex-M3 *\i{SysTick} timer.
\ ** Although the *\i{SysTick} timer is common to all Cortex-M3
\ ** devices, there is no guarantee that they have the same
\ ** clocking arrangements. Consequently there are different
\ ** drivers for different implementations.

\ *P The LM3S9B9x implementation is fed with the system clock and
\ ** has an option to divide it by 8, controlled by bit 2 of the
\ ** SysTick Control and Status Register (ARM DDI 0337, Rev E,
\ ** page 8-8).
\ *(
\ *B Bit2=0: SysTick clock is external/unimplemented
\ *B Bit2=1: SysTick clock is CCLK.
\ *)
\ *P This code always uses CCLK.

tick-ms system-speed #1000 */ 1- equ /SysTick	\ -- x
\ *G Initial value of the SysTick Reload register.

[undefined] ticks [if]
0 value ticks
[then]

Proc SysTickISR	\ --
  mvl32   r0, # ' ticks >body
  ldr     r1, [ r0, # 0 ]
  add .s  r1, r1, # tick-ms
  str     r1, [ r0, # 0 ]
  bx      lr
end-code
SysTickISR SysTickvec# setExcVec

: start-clock	\ --
\ *G Initialise the system ticker to run with a period of
\ ** *\fo{tick-ms} milliseconds.
  _SCS
  /SysTick over stRVR + !		\ set ticker rate
  $0007 over stCSR + !			\ enable it
\ bit 0 = enable, bit 1 = interrupt, bit 2 = clock source
  $FF over $0D23 + c!			\ set SysTick priority to lowest
  drop
  ei
;

: stop-clock	\ --
\ *G Stop the sytem ticker.
  bit0 _SCS stCSR + bic!  ;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]


\ ********************
\ Time base extensions
\ ********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  stop-clock init-timers start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

decimal

