\ adcLPC17xx.fth - ADC drivers

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20110726 MPE001 First release.
))

only forth definitions
decimal


\ =============
\ *! adclpc17xx
\ *T ADC driver for LPC17xx
\ =============

100 equ ADC0-us	\ -- us
\ *G ADC0 sample interval in microseconds.

%0010:0010 equ ADCsel			\ Select ADCs 5 and 1
\ ADC0.1 = Thermistor
\ ADC0.5 = Pot
system-speed 13 MHz /mod swap
[if] 1+ [then]
1- 8 lshift equ ADCclkdiv		\ clock divider in AD0CR 15:8
1 16 lshift equ ADCburst		\ burst mode = continuous
1 21 lshift equ ADCpdn			\ operational
%000 24 lshift equ ADCstart		\ MAT0.3 to start
0 27 lshift equ ADCedge			\ on postive edge

ADCsel  ADCclkdiv or  ADCburst or  ADCpdn or  ADCstart or  ADCedge or
  equ ADC0CRinit
\ *G Initial value of ADC0 control register

1 equ pcdTim0	\ -- n
\ *G Peripheral clock division ration for Timer0
1 equ /preTim0	\ -- n
\ *G Select the prescaler division ratio.
/preTim0 1- equ init_preDiv0
\ *G The initial value of the prescaler divider.
interpreter
: gen-T0MRx	\ us clock -- n
\ *G An *\fo{INTERPRETER} definition that takes a period in
\ ** microseconds and the clock speed in herz to produce the
\ ** required value to be loaded in the match register.
  pcdTim0 /				\ pclk speed
  /preTim0 #1000000 * */
;
target
ADC0-us system-speed gen-T0MRx equ #T0MR0tick
\ *G The initial value of match register 0 calculated from the
\ ** required ticker period *\fo{ADC0-us} in microseconds and the
\ ** system clock speed *\fo{SYSTEM-SPEED} in herz. This
\ ** defines the period of the ADC. An interrupt is generated
\ ** at the end of this period.

variable ADC0.1val	\ -- addr
\ *G Holds the accumulated value of ADC0.1.
#10 equ ADC0.1power2
\ *G The ADC is fltered over 2^x samples.
variable ADC0.5val
\ *G Holds the accumulated value of ADC0.5.
#10 equ ADC0.5power2
\ *G The ADC is fltered over 2^x samples.

: T0-isrh	\ --
\ *G High level interrupt handler for Timer0 controlling ADC0.
  1 _Timer0 TnIR + !			\ reset MR0 interrupt

  _ADC0 ADnDR1 + @ $FFF0 and		\ raw 12 bit value of ADC0.1
  ADC0.1val tuck @			\ rolling value
  dup ADC0.1power2 rshift -		\ subtract average
  +  swap !				\ add and set new value

  _ADC0 ADnDR5 + @ $FFF0 and		\ raw 12 bit value of ADC0.5
  ADC0.5val tuck @			\ rolling value
  dup ADC0.5power2 rshift -		\ subtract average
  +  swap !				\ add and set new value
;
' T0-isrh T0_vec# EXC: T0-isr

: initADC	\ --
\ *G Initialise the ADC channels.
  ~pcADC ~pcTim0 or _PCONP or!		\ Enable ADC and Timer0 power
  $0100:0004 $0300:000C _PCLKSEL0 setMask \ bits 25:24 -> %01 for CCLK ADC
  					\ bits 3:2 -> %01 for CCLK Timer0
  $0001:0000 $0003:0000 _PINSEL1 setMask  \ bits 17:16 -> %01 for AD0.1
  $C000:0000 $C000:0000 _PINSEL3 setMask  \ bits 31:30 -> %11 for AD0.5
  ADC0CRinit _ADC0 ADnCR + !		\ set ADC control register

  di
  _Timer0
  0 over TnTCR + !			\ disable counter
  init_preDiv0 over TnPC + !		\ set prescaler
  #T0MR0tick over TnMR0 + !		\ set match register 0 for period
  3 over TnMCR + !			\ interrupt and reset on MR0
  1 swap TnTCR + !			\ enable counter
  T0_vec# EnInt				\ interrupt enable in NVIC
  ei
;

: ADC0.1r	\ --
\ *G Display the current conversion value of ADC0.1
  _ADC0 ADnDR1 + @ .dword
;

: ADC0.1s	\ --
\ *G Display the smoothed conversion value of ADC0.1
  ADC0.1val @ ADC0.1power2 rshift .dword
;

: ADC0.5r	\ -- u16
\ *G Return the current conversion value of ADC0.5
  _ADC0 ADnDR5 + @
;

: ADC0.5s	\ -- u16
\ *G Return the smoothed conversion value of ADC0.5
  ADC0.5val @ ADC0.5power2 rshift
;


\ ======
\ *> ###
\ ======

decimal
