\ gpioLM3s9b9x.fth - easy access to Luminary/TI LM3S9B9x GPIO pins.

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20111011 SFP001 Adapted from Nuvoton version.
))

only forth definitions
decimal

\ ===========
\ *! gpiolm3s9b9c
\ *T TI/Luminary LM3S9B9x GPIO utilities
\ ===========
\ *P The code in *\i{Cortex\Drivers\gpioLM3s9b9x.fth} provides
\ ** utility words for accessing GPIO pins on a bit by bit basis.
\ ** The code is written for ease of use rather than performance.


\ ****************
\ *S Configuration
\ ****************
\ *P The equates in this block are defaults used if the equates
\ ** have not been defined before this file is compiled.

interpreter
: within?  within?  ;
target

((
$5000:4000 equ _GPIO

_GPIO $0000 + equ _GPIOA
_GPIO $0040 + equ _GPIOB
_GPIO $0080 + equ _GPIOC
_GPIO $00C0 + equ _GPIOD
_GPIO $0100 + equ _GPIOE
  $000 equ GPIOn_PMD
  $004 equ GPIOn_OFFD
  $008 equ GPIOn_DOUT
  $00C equ GPIOn_DMASK
  $010 equ GPIOn_PIN
  $014 equ GPIOn_DBEN
  $018 equ GPIOn_IMD
  $01C equ GPIOn_IEN
  $020 equ GPIOn_ISRC
))
_GPIOA equ PortBase	\ -- addr
\ *G Base address of port definitions.
$40 equ /PortBlock	\ -- len
\ *G Number of bytes per port for peripheral registers.
6 equ #PortShift	\ -- u
\ *G Number of bits to shift to offset by *\fo{/PortBlock}.


\ ********************
\ *S Defining I/O pins
\ ********************
\ *P FIO/GPIO acess is defined using a bit number. Bits 0..31 form
\ ** P0.0 to P0.31, bits 32..63 form P1.0 to P1.31 and so on.
\ ** Although TI/Luminary name their ports from A, here we number
\ ** them from 0. Note also that the Nuvoton pin structure only
\ ** allows 16 bits (0..15) per port.

GPIODATA 0 <> [if]
  cr ." This code relies on GPIODATA = $0000." abort
[then]

create GPIOtab	\ -- addr
  _GPIOA ,  _GPIOB ,  _GPIOC ,  _GPIOD ,
  _GPIOE ,  _GPIOF ,  _GPIOG ,  _GPIOH ,
  _GPIOJ ,

target-only
: PIO:		\ port# bit# -- ; -- iobit#
\ *G Define a port I/O bit by name. For example
\ *C   2 6 PIO: P2.11
\ ** defines an I/O bit on port 2 (GPIOC), bit 6. At run time, the I/O
\ ** address with bits 9:2 set is returned.
  swap 2 lshift GPIOtab + @		\ port base address
  1 rot 2 + lshift or			\ form address bits 9:2
  constant  ;
host&target
interpreter
: PIO:		\ port# bit# -- ; -- iobit#
  swap 2 lshift GPIOtab + @		\ port base address
  1 rot 2 + lshift or			\ form address bits 9:2
  constant  ;
target


\ ****************
\ *S IO pin access
\ ****************

compiler
: io>bit	\ iobit# -- bit# offset
  dup $1F and				\ -- iobit# bit#
  swap 5 rshift #Portshift lshift	\ -- bit# offset
;
target

: setPin	\ ioaddr --
\ *G Set the pin high.
  $FF swap c!  ;

: clrPin	\ ioaddr --
\ *G Set the pin low.
  $00 swap c!  ;

: getPin	\ ioaddr -- 0/1
\ *G Read the pin state.
  c@ 0<> 1 and  ;


\ ***********************
\ *S IO pin configuration
\ ***********************

: mode>mask	\ mode -- modemask
\ Repeats the two bit mask 16 times.
  3 and
  dup 2 lshift or			\ 4 bits
  dup 4 lshift or			\ 8 bits
  dup 8 lshift or			\ 16 bits
  dup #16 lshift or			\ 32 bits
;

: >2bitmask	\ bit# -- bitmask
\ Converts a bit number 0..15 to a 2 bit mask for a 32 bit word,
  $0F and 2* $03 swap lshift  ;

: mode>valmask	\ mode bit# -- val bitmask
\ Convert a two bit mode and bit number (0..15) into a value and
\ bitmask to use with setMask.
  swap mode>mask 			\ -- bit# modemask
  swap >2bitmask			\ -- modemask bitmask
  tuck and swap 			\ -- val bitmask
;

: io>mode	\ mode iobit# -- val mask offset
\ Converts a two bit mode.
  io>bit -rot				\ -- offset mode bit#
  mode>valmask				\ -- offset val bitmask
  rot					\ -- val bitmask offset
;

: isPinMode	\ mode iobit# --
\ *G Sets the GPIO mode for a pin. Use with the constants below.
  io>mode GPIOn_PMD + PortBase + setMask  ;

0 constant InpMode	\ -- mode#
\ *G Specifies GPIO input mode.
1 constant OutMode	\ -- mode#
\ *G Specifies GPIO input mode.
1 constant ODMode	\ -- mode#
\ *G Specifies GPIO open drain mode.
3 constant BiMode	\ -- mode#
\ *G Specifies GPIO quasi-bidirectional mode.

: isInput	\ iobit# --
\ *G Set the pin as an input.
  InpMode swap isPinMode  ;

: isOutput	\ iobit# --
\ *G Set the pin as an input.
  OutMode swap isPinMode  ;

: isPinFunc	\ func iobit# --
\ *G Set the pin configuration. Function=0 is for GPIO usage,
\ ** otherwise the alternate function. By default, nearly all pins
\ ** are in I/O pin mode after reset.
  dup $0F and 1 swap lshift		\ -- func iobit# bitmask
  swap 5 rshift cells _GCR + GPA_MFP +	\ -- func bitmask addr
  rot if or! else bic! endif
;


\ ===================================
\ *S Test code for Nuvoton NuMicro-SDK
\ ===================================
((
\ *[
decimal

1 2 pio: led1		\ GPB.2
1 3 pio: led2		\ GPB.3

: tled1		\ --
  led1 isOutput
  begin
    led1 clrPin  200 ms
    led1 setPin  200 ms
  key? until
;

: tled2		\ --
  led2 isOutput
  begin
    led2 clrPin  200 ms
    led2 setPin  200 ms
  key? until
;
\ *]
))

\ ======
\ *> ###
\ ======

decimal
