\ gpioLPC17xx.fth - easy access to LPC17xx GPIO pins.

((
Copyright (c) 2010, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20120514 PAK002 Fixed isPinMode.
20101027 SFP001 Adapted from ARM version.
))

only forth definitions
decimal

\ ===========
\ *! gpiolpc17xx
\ *T LPC17xx GPIO utilities
\ ===========
\ *P The code in *\i{Cortex\Drivers\gpioLPC17xx.fth} provides
\ ** tools to access the GPIO/FIO pins on a bit by bit basis.
\ ** The code is written for ease of use rather than performance.

\ *P There are similar files in the *\i{Cortex\Drivers} directory
\ ** for different CPU families. They all use a very similar set
\ ** of words. Do not be afraid to browse the supplied source
\ ** code!


\ ****************
\ *S Configuration
\ ****************
\ *P The equates in this block are defaults used if the equates
\ ** have not been defined before this file is compiled.

interpreter
: within?  within?  ;
target

_FIO0 equ PortBase	\ -- addr
\ *G Base address of port definitions.
$20 equ /PortBlock	\ -- len
\ *G Number of bytes per port for peripheral registers.
5 equ #PortShift	\ -- u
\ *G Number of bits to shift to offset by *\fo{/PortBlock}.


\ ********************
\ *S Defining I/O pins
\ ********************
\ *P FIO/GPIO acess is defined using a bit number. Bits 0..31 form
\ ** P0.0 to P0.31, bits 32..63 form P1.0 to P1.31 and so on.

target-only
: PIO:		\ port# bit# -- ; -- iobit#
\ *G Define a port I/O bit by name. For example
\ *C   2 11 PIO: P2.11
\ ** defines an I/O bit on port 2, bit 11. At run time, the I/O
\ ** bit number is returned.
  swap 5 lshift or constant  ;
host&target
interpreter
: PIO:		\ port# bit# -- ; -- iobit#
  swap 5 lshift or constant  ;
target


\ ****************
\ *S IO pin access
\ ****************

compiler
: io>port	\ iobit# -- mask offset
  dup $1F and 1 swap lshift		\ -- iobit# mask
  swap 5 rshift #Portshift lshift	\ -- mask offset
;
target

: setPin	\ iobit# --
\ *G Set the pin high.
  io>port FIOSET + PortBase + !  ;

: clrPin	\ iobit# --
\ *G Set the pin low.
  io>port FIOCLR + PortBase + !  ;

: getPin	\ iobit# -- 0/1
\ *G Read the pin state.
  dup $1F and				\ -- iobit# pin#
  swap 5 rshift #Portshift lshift	\ -- pin# offset
  FIOPIN + PortBase + @
  swap rshift 1 and
;

: ioport	\ iobit# -- mask offset
  io>port  ;


\ ***********************
\ *S IO pin configuration
\ ***********************

: isInput	\ iobit# --
\ *G Set the pin as an input.
  io>port FIODIR + Portbase + bic!  ;

: isOutput	\ iobit# --
\ *G Set the pin as an input.
  io>port FIODIR + Portbase + or!  ;

: mode>mask	\ mode -- modemask
  3 and
  dup 2 lshift or			\ 4 bits
  dup 4 lshift or			\ 8 bits
  dup 8 lshift or			\ 16 bits
  dup #16 lshift or			\ 32 bits
;

: io>mode	\ mode iobit# -- val mask offset
  swap mode>mask swap			\ -- modemask iobit#
  dup $0F and 2* $03 swap lshift	\ -- modemask iobit# bitmask
  swap -rot				\ -- iobit# modemask bitmask
  tuck and swap rot			\ -- val mask iobit#
  4 rshift 2 lshift			\ -- val mask offset
;

: isPinsel	\ sel# iobit# --
\ *G Set the pin configuration 0..3. By default, nearly all pins
\ ** are in I/O pin mode after reset.
  io>mode _PINSEL0 + setMask  ;

[defined] _PINMODE0 [if]
: isPinmode	\ mode# iobit# --
\ *G Set the pin mode 0..3. Not available for CPUs that do not
\ ** have *\fo{PINMODE} registers.
  io>mode _PINMODE0 + setMask  ;	\ PAK002
0 constant PulledUp	\ -- mode#
\ *G Used as the mode# with *\fo{isPinMode} above to define how
\ ** the pin operates as an input.
2 constant NotPulled	\ -- mode#
\ *G Used as the mode# with *\fo{isPinMode} above to define how
\ ** the pin operates as an input.
3 constant PulledDown	\ -- mode#
\ *G Used as the mode# with *\fo{isPinMode} above to define how
\ ** the pin operates as an input.
[then]


\ ===================================
\ *S Test code for Olimex LPC1766-STK
\ ===================================

((
\ *[
decimal

1 25 pio: led1
0  4 pio: led2
1 18 pio: Usb_Link_Led
2  9 pio: Usb_Connect_Led
0 23 pio: But1
2 13 pio: But2

: tled1		\ --
  led1 isOutput
  begin
    led1 clrPin  200 ms
    led1 setPin  200 ms
  key? until
;

: tled2		\ --
  led2 isOutput
  begin
    led2 clrPin  200 ms
    led2 setPin  200 ms
  key? until
;
\ *]
))


\ ======
\ *> ###
\ ======

decimal
