\ serSTM32F10xi.fth - STM32F10x serial: interrupt RX, polled TX

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! serstm32f10xi
\ *T STM32F10x serial driver
\ ===========

interpreter
: genBRR	\ baud -- divisor
  system-speed 1 lshift			\ for rounding
  swap /				\ brr = clock / baud
  1+  1 rshift				\ rounding
;
target

115200 genBRR equ /us1BRR	\ -- u
\ *G Baud rate divisor for USART1.

bit14 bit2 or bit0 or equ u1_apb2enA	\ --
\ *G Initialisation value for USART1 in RCC_APB2ENR. Clocks for
\ ** USART1, GPIO port A, and Alternate function are enabled.
bit14 bit3 or bit0 or equ u1_apb2enB	\ -- ; remapped to GPIO port B
\ *G Initialisation value for USART1 in RCC_APB2ENR. Clocks for
\ ** USART1, GPIO port B, and Alternate function are enabled.

variable rxd1	\ -- addr
\ *G The first byte holds the character, the second byte is
\ ** non-zero if a character is available.

: Rx1handler	\ --
\ *G Action of the USART1 receive interrupt handler.
  _USART1 usRDR + @
   $FF and $FF00 or rxd1 !
;

' Rx1handler USART1vec# EXC: U1entry

: init-ser1	\ --
\ *G Initialise USART1 for operation on GPIO pins PA9/10.
\ ** Transmit is polled, receive is interrupt-driven.

\ Default on PA10/9
  u1_apb2enA _RCC rccAPB2en + or!	\ set required bits
  bit2 _AFIO afMAPR + bic!		\ clear bit 2, no USART1 remap
  $04B0 $0FF0 _GPIOA gp.CRH + setmask	\ conf/mode RX=PA10, TX=PA9
					\ setMask is NOT AND, then OR
((
  u1_apb2enB _RCC rccAPB2en + or!	\ set required bits
  bit2 _AFIO afMAPR + or!		\ set bit 2, USART1 remapped
  $4B00:0000 $FF00:0000 _GPIOB gp.CRL + setmask	\ conf/mode RX=PB6, TX=PB6
))
  di
  _USART1
  /us1BRR over usBRR + !		\ set baud rate
  $2000 over usCR1 + !			\ 8 bit, no parity
  $0000 over usCR2 + !			\ 1 stop bit, UART mode
  $0000 over usCR3 + !			\ no interrupts, no DMA, no LIN, no smartcard
  $202C over usCR1 + !			\ 8 bit, no parity, RXNEIE, TE, RE
  drop
\  $80 USART1vec# SetPri			\ interrupt priority
  USART1vec# EnInt			\ interrupt enable in NVIC
  ei
;

: (serKey?)	\ base -- flag
  drop  rxd1 1+ c@  ;

: (serkey)	\ base -- char
  drop
[ tasking? ] [if]
  begin
    [i  rxd1 1 + c@ 0=
   while
    i] pause
  repeat
[else]
  begin
    [i  rxd1 1 + c@ 0=
   while
    i]
  repeat
[then]
  rxd1 c@  rxd1 off  i]
;

: (seremit)	\ char base --
[ tasking? ] [if]
  begin
    dup usSR + @ bit7 and 0=
   while
    pause
  repeat
[else]
  begin
    dup usSR + @ bit7 and
  until
[then]
  usTDR + !
;

: (sertype)	\ caddr len base --
  -rot bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (sercr)	\ base --
  #13 over (seremit)  #10 swap (seremit)  ;

: +FaultConsole	\ --
\ *G For use in fault exception handlers, the multi-tasker must
\ ** be turned off and *\fo{EMIT) and friends must run in polled
\ ** mode.
[ tasking? ] [if]  single  [then]
;

: serkey?1	\ -- flag
  _USART1 (serkey?)  ;

: serkey1	\ -- char
  _USART1 (serkey)  ;

: seremit1	\ char --
  _USART1 (seremit)  ;

: serType1	\ c-addr len --
  _USART1 (sertype)  ;

: sercr1	\ --
  _USART1 (sercr)  ;

create Console1	\ -- addr ; OUT managed by upper driver
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

console-port 1 = [if]
  console1 constant console
[then]

: init-ser	\ --
\ *G Initialise primary serial ports.
  init-ser1  ;


\ ======
\ *> ###
\ ======

decimal


