\ serSTM32F3xxqi.fth - STM32F30x queued interrupt RX and TX

((
Copyright (c) 2009, 2011, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

Adaptad to ST32F30x by
Synics AG

To do
=====

Change history
==============
20171005 KH002  Use newer equates extracted from STM32F30x.svd
20170831 KH001  Adapted to STM32F30x
20120415 MRT003 Verified and added USART2 and UART4.
20120123 SFP002 Upated for Eval board and Discovery board
20111201 MPE001 Converted from STM32F103

))

only forth definitions
decimal

\ ===========
\ *! serstm32f3xxqi
\ *T STM32F3xx interrupt driven serial driver
\ ===========
\ *P The serial driver *\i{Cortex/Drivers/serSTMF3xxqi.fth}
\ ** provides queued interrupt drivers for the on-chip USARTs.
\ ** The supplied driver implements:
\ *(
\ *B USART1 on PA9(Tx) and PA10(Rx)
\ *B USART2 on PA2(Tx) and PA3(Rx)
\ *B USART3 on PD8(Tx) and PD9(Rx)
\ *B UART4 on PA0(Tx) and PA2(Rx).
\ *)

\ *P When you copy this file for your own hardware do not forget
\ ** to provide a word *\fo{selio-sern) that updates the GPIO
\ ** pin assignments and alternate function selections. For each
\ ** UART there is a commented-out example.


\ ***********************
\ *S Baud rate generation
\ ***********************
\ *P The baud rate routines use 16 times sampling and the fractional
\ ** baud rate generator.

\ *E                pclkx
\ **  divisor = -------------
\ **            16 * baudrate
\ *P We then treat the divisor as an integer and four bit fraction,
\ ** effectively scaling it by 16 again.
\ *E             pclkx
\ **  BRRval = --------
\ **           baudrate

interpreter
: genBRRval     \ baud clock -- divisor
\ *G Generate the required BRR value.
  1 lshift                              \ for rounding
  swap /                                \ brr = clock / baud
  1+  1 rshift                          \ rounding
;
target


\ **************
\ *S Shared code
\ **************

struct /UxData  \ -- len
\ *G The definition of the structure that is passed to UART handlers.
  int ux.USART          \ base address of USART peripher
  int ux.RxQ            \ address of receive queue
  int ux.TxQ            \ address of transmit queue
  int ux.TxQne          \ address of transmit queue flag
end-struct

: UxHandler     \ struct --
\ *G The main part of the UART interrupt handler.
  dup ux.USART @ Un.ISR + @ bit5 and if   \ receive interrupt check
    dup ux.USART @ Un.RDR + @            \ get character
    over ux.RxQ @  dup (cqFull?)        \ -- struct char queue flag
    if  2drop  else  (>cqueue)  endif   \  discard or queue character
  endif
  dup ux.USART @ Un.CR1 + @ bit7 and if  \ transmit enabled
    dup ux.USART @ Un.ISR + @ bit7 and if \   ready for more
      dup ux.TxQ @ cqchars if           \     and more available
        dup ux.TxQ @ (cqueue>)          \       get next character
          over ux.USART @ Un.TDR + !     \       send it
      else                              \ no more characters
        bit7 over ux.USART @ Un.CR1 + bic! \ disable TX ints
      endif
    endif
  endif
  drop
;

0 value FaultConsole?   \ -- flag
\ *G When set non-zero, transmit is in polled mode. This is
\ ** required when fault exceptions provide debug information.

: +FaultConsole \ --
\ *G For use in fault exception handlers, the multi-tasker must
\ ** be turned off and *\fo{EMIT) and friends must run in polled
\ ** mode.
  -1 to FaultConsole?
  [ tasking? ] [if]  single  [then]
;

: (qseremit)    \ char struct --
\ *G Send a character on the UART whose structure is given.
  FaultConsole? 0= if
    begin                               \ wait for non-full queue
      dup ux.TxQ @ cqfull?
     while
      pause
    repeat
    [i
    tuck ux.TxQ @ (>cqueue)             \ character onto queue
    bit7 over ux.USART @ Un.CR1 + or!    \ turn on transmitter interrupt
    i]
    drop  exit
  endif
  ux.USART @
  begin
    dup Un.ISR + @ bit7 and
  until
  Un.TDR + !
;

: (qsertype)    \ caddr len struct --
\ *G Send a string on the UART whose structure is given.
  -rot bounds                           \ -- base queue
  ?do  i c@ over (qseremit)  loop
  drop
;

: (qsercr)      \ base struct --
\ *G Send a CR/LF on the UART whose structure is given.
  $0D over (qseremit)  $0A swap (qseremit)
;

: (qserkey?)    \ struct -- flag
\ *G Check if a character is available on the UART whose structure is given.
  ux.RxQ @ cqnotempty?  ;

: (qserkey)     \ struct -- char
\ *G Read the next character from the UART whose structure is given.
  ux.RxQ @ cqueue>  ;

: (init-ser)    \ brrval ^usart --
\ *G Initialise the UART whose base address is given.
  $0001 over Un.CR1 + !                 \ enable UART, 8 bits, no parity
  $0000 over Un.CR2 + !                 \ 1 stop bit, UART mode
  $0000 over Un.CR3 + !                 \ no interrupts, no DMA, no LIN, no smartcard
  tuck Un.BRR + !                       \ set baud rate

  $002d over Un.CR1 + !                 \ 8 bit, no parity, RXNEIE, TE, RE, UE, UESM

  drop
;


\ *********
\ *S USART1
\ *********
\ *P USART1 is on the APB2 bus, so the baud rate is calculated
\ ** using that bus clock.

useUSART1? [if]

console1-speed APB2-speed genBRRval equ /us1BRR \ -- u
\ *G Baud rate divisor for USART1.

#256 cqueue: SerInpQ1   \ -- addr
\ *G Receiver circular byte queue.
#128 cqueue: SerOutQ1   \ -- addr
\ *G Transmitter circular byte queue.

variable U1TxQne        \ -- addr
\ *G Set true when the TxQ contains characters.

create U1struct \ -- addr
\ *G Control structure for UART1.
  _USART1 ,  SerInpQ1 ,  SerOutQ1 ,  U1TxQne ,

: U1handler     \ --
\ *G Action of the USART1 interrupt handler.
  U1struct Uxhandler  ;
' U1handler USART1_EXTI25_IRQn EXC: U1entry
\ *G Set the UART1 interrupt vector.

[undefined] selio-ser1 [if]
: selio-ser1    \ -- ; on PA9/10
\ *G Initialise USART1 for operation on GPIO pins PA9/10.
\ ** Use this code as an example for your hardware. This word
\ ** is commented out and you must provide one as required.
\ Clocks and reset
  _RCC
  bit0 over rcc.AHBENR  + or!           \ enable clock to GPIOA
  bit0 over rcc.AHBRSTR + bic!          \ take it out of reset
  bit4 over rcc.APB2ENR  + or!           \ enable clock to USART1
  bit4 over rcc.APB2RSTR + bic!          \ take it out of reset
  drop
\ Default on PA9/10
  _GPIOA >r
  $0028:0000 $003C:0000 r@ gpio.MODER + setMask  \ alternate function on 9/10
  $0000:0770 $0000:0FF0 r> gpio.AFRH + setMask   \ AF7 for both
;
[then]
((
: selio-ser1    \ -- ; on PB6/7
  _RCC
  bit1 over rcc.AHBENR + or!            \ enable clock to GPIOB
  bit1 over rcc.AHBRSTR + bic!          \ take it out of reset
  bit4 over rcc.APB2ENR + or!            \ enable clock to USART1
  bit4 over rcc.APB2RSTR + bic!          \ take it out of reset
  drop
\ Optional on PB10/PB11
  _GPIOB >r
  $0000:A000 $0000:F000 r@ gpio.MODER + setMask  \ alternate function on PB6/7
  $7700:0000 $FF00:0000 r> gpio.AFRL + setMask   \ AF7 for both
;
))

: init-ser1     \ --
\ initialise the queues and the UART
  selio-ser1
  di
  SerInpQ1 init-cqueue
  SerOutQ1 init-cqueue
  U1TxQne off
  /us1BRR _USART1 (init-ser)
  USART1_EXTI25_IRQn EnInt                      \ interrupt enable in NVIC
  ei
;

: serkey?1      \ -- flag
  U1struct (qserkey?)  ;

: serkey1       \ -- char
  U1struct (qserkey)  ;

: seremit1      \ char --
  U1struct (qseremit)  ;

: serType1      \ c-addr len --
  U1struct (qsertype)  ;

: serCr1        \ --
  U1struct (qsercr)  ;

create Console1 \ -- addr ; OUT managed by upper driver
\ *G Generic I/O device structure for UART1.
  ' serkey1 ,           \ -- char ; receive char
  ' serkey?1 ,          \ -- flag ; check receive char
  ' seremit1 ,          \ -- char ; display char
  ' sertype1 ,          \ caddr len -- ; display string
  ' sercr1 ,            \ -- ; display new line

console-port 1 = [if]
  console1 constant console
[then]

[then]                  \ useUSART1?


\ *********
\ *S USART2
\ *********
\ *P USART2 is on the APB1 bus, so the baud rate is calculated
\ ** using that bus clock.

\ PA2 is Tx and PA3 is Rx

useUSART2? [if]

console2-speed APB1-speed genBRRval equ /us2BRR           \ -- u
\ *G Baud rate divisor for USART2.

#256 cqueue: SerInpQ2   \ -- addr
\ *G Receiver circular byte queue.

#128 cqueue: SerOutQ2   \ -- addr
\ *G Transmitter circular byte queue.

variable U2TxQne        \ -- addr
\ *G Set true when the TxQ contains characters.

create U2struct \ -- addr
\ *G UART device structure for UART2.
  _USART2 ,  SerInpQ2 ,  SerOutQ2 ,  U2TxQne ,

: U2handler     \ --
\ *G Action of the USART2 interrupt handler.
  U2struct Uxhandler  ;

' U2handler USART2_EXTI26_IRQn EXC: U2entry
\ *G Set the UART2 interrupt vector.

[undefined] selio-ser2 [if]
: selio-ser2    \ --
\ *G Initialise USART2 for operation on GPIO pins PA3 (Rx), PA2 (Tx)
  _RCC
  bit17 over rcc.AHBENR  + or!                    \ enable clock to GPIOA
  bit17 over rcc.AHBRSTR  + bic!                  \ take it out of reset
  bit17 over rcc.APB1ENR  + or!                   \ enable clock to USART2
  bit17 over rcc.APB1RSTR + bic!                  \ take it out of reset
  drop

  \ Default RS-232 port as UART2 on PA3 PA2
  _GPIOA >r
  $0000:00A0 $0000:00F0 r@ gpio.MODER   + setMask  \ alternate function on PA3:2
  $0000:0050 $0000:00F0 r@ gpio.OSPEEDR + setMask  \ 25MHz
  $0000:0010 $0000:0030 r@ gpio.PUPDR   + setMask  \ pull-up on PA2 (Tx)
  $0000:7700 $0000:FF00 r> gpio.AFRL    + setMask  \ AF7 for both (AFRL3:AFRL2)
;
[then]

: init-ser2     \ --
\ *G Initialise USART2 for queued interrupt operation on GPIO pins PA3:2
  selio-ser2
  di
  SerInpQ2 init-cqueue
  SerOutQ2 init-cqueue
  U2TxQne off
  /us2BRR _USART2 (init-ser)
  USART2_EXTI26_IRQn EnInt                   \ interrupt enable in NVIC
  ei
;

: serkey?2      \ -- flag
  U2struct (qserkey?)  ;

: serkey2       \ -- char
  U2struct (qserkey)  ;

: seremit2      \ char --
  U2struct (qseremit)  ;

: serType2      \ c-addr len --
  U2struct (qsertype)  ;

: sercr2        \ --
  U2struct (qsercr)  ;

create Console2             \ -- addr ; OUT managed by upper driver
\ *G Generic I/O device structure for UART2.
  ' serkey2 ,                     \ -- char ; receive char
  ' serkey?2 ,                    \ -- flag ; check receive char
  ' seremit2 ,                    \ -- char ; display char
  ' sertype2 ,                    \ caddr len -- ; display string
  ' sercr2 ,                      \ -- ; display new line

console-port 2 = [if]
  console2 constant console
[then]

[then]                                    \ useUSART2?


\ *********
\ *S USART3
\ *********
\ *P USART3 is on the APB1 bus, so the baud rate is calculated
\ ** using that bus clock.

useUSART3? [if]

console3-speed APB1-speed genBRRval equ /us3BRR \ -- u
\ *G Baud rate divisor for USART3.

#256 cqueue: SerInpQ3   \ -- addr
\ *G Receiver circular byte queue.
#128 cqueue: SerOutQ3   \ -- addr
\ *G Transmitter circular byte queue.

variable U3TxQne        \ -- addr
\ *G Set true when the TxQ contains characters.

create U3struct \ -- addr
\ *G UART device structure for UART3.
  _USART3 ,  SerInpQ3 ,  SerOutQ3 ,  U3TxQne ,

: U3handler     \ --
\ *G Action of the USART3 interrupt handler.
  U3struct Uxhandler  ;
' U3handler USART3_EXTI28_IRQn EXC: U3entry
\ *G Set the UART3 interrupt vector.

[undefined] selio-ser3 [if]
: selio-ser3    \ --
\ *G Initialise USART3 for operation on GPIO pins PD8 (Tx) and PD9(Rx).
\ ** Use this code as an example for your hardware. This word
\ ** is used if *\fo{selio-ser3} is not yet defined.
  _RCC
  bit3 over rcc.AHBENR + or!            \ enable clock to GPIOD
  bit3 over rcc.AHBRSTR + bic!          \ take it out of reset
  bit18 over rcc.APB1ENR + or!          \ enable clock to USART3
  bit18 over rcc.APB1RSTR + bic!        \ take it out of reset
  drop
  \ Default RS-232 port as UART3 on PD8 PD9
  _GPIOD >r
  $000A:0000 $000F:0000 r@ gpio.MODER + setMask  \ alternate function on 8/9
  $0005:0000 $000F:0000 r@ gpio.OSPEEDR + setMask  \ 25MHz
  $0001:0000 $0003:0000 r@ gpio.PUPDR + setMask    \ pull-up on PD8
  $0000:0077 $0000:00FF r> gpio.AFRH + setMask   \ AF7 for both
;
[then]

: init-ser3     \ --
\ *G Initialise USART3 for polled operation on GPIO pins PD8/9.
  selio-ser3
  di
  SerInpQ3 init-cqueue
  SerOutQ3 init-cqueue
  U3TxQne off
  /us3BRR _USART3 (init-ser)
  USART3_EXTI28_IRQn EnInt                      \ interrupt enable in NVIC
  ei
;

: serkey?3      \ -- flag
  U3struct (qserkey?)  ;

: serkey3       \ -- char
  U3struct (qserkey)  ;

: seremit3      \ char --
  U3struct (qseremit)  ;

: serType3      \ c-addr len --
  U3struct (qsertype)  ;

: sercr3        \ --
  U3struct (qsercr)  ;

create Console3 \ -- addr ; OUT managed by upper driver
\ *G Generic I/O device structure for UART3.
  ' serkey3 ,           \ -- char ; receive char
  ' serkey?3 ,          \ -- flag ; check receive char
  ' seremit3 ,          \ -- char ; display char
  ' sertype3 ,          \ caddr len -- ; display string
  ' sercr3 ,            \ -- ; display new line

console-port 3 = [if]
  console3 constant console
[then]

[then]                  \ useUSART3?


\ *********
\ *S UART4
\ *********
\ *P UART4 is on the APB1 bus, so the baud rate is calculated
\ ** using that bus clock.

useUART4? [if]

console4-speed APB1-speed genBRRval equ /us4BRR \ -- u
\ *G Baud rate divisor forUART4.

#256 cqueue: SerInpQ4   \ -- addr
\ *G Receiver circular byte queue.

#128 cqueue: SerOutQ4   \ -- addr
\ *G Transmitter circular byte queue.

variable U4TxQne        \ -- addr
\ *G Set true when the TxQ contains characters.

create U4struct \ -- addr
\ *G UART device structure for UART4.
  _UART4 ,  SerInpQ4 ,  SerOutQ4 ,  U4TxQne ,

: U4handler     \ --
\ *G Action of theUART4 interrupt handler.
  U4struct Uxhandler  ;

' U4handler UART4_EXTI34_IRQn EXC: U4entry
\ *G Set the UART4 interrupt vector.

[undefined] selio-ser4 [if]
: selio-ser4    \ --
\ *G InitialiseUART4 for operation on GPIO pins PA0 (Tx), PA1 (Rx).
  _RCC
  bit0 over rcc.AHBENR + or!                  \ enable clock to GPIOA
  bit0 over rcc.AHBRSTR + bic!                \ take it out of reset
  bit19 over rcc.APB1ENR + or!                \ enable clock toUART4
  bit19 over rcc.APB1RSTR + bic!              \ take it out of reset
  drop

  \ Default RS-232 port as UART4 on PA0 PA1
  _GPIOD >r
  $0000:000A $0000:000F r@ gpio.MODER + setMask   \ alternate function on PA0, PA1
  $0000:0005 $0000:0005 r@ gpioOSPEEDR + setMask  \ 25MHz
  $0000:0001 $0000:0003 r@ gpioPUPDR + setMask    \ pull-up on PA0
  $0000:0077 $0000:00FF r> gpio.AFRL + setMask    \ AF7 for both
;
[then]

: init-ser4     \ --
\ *G InitialiseUART4 for polled operation on GPIO pins PA0/1.
  selio-ser4
  di
  SerInpQ4 init-cqueue
  SerOutQ4 init-cqueue
  U4TxQne off
  /us4BRR _UART4 (init-ser)
  UART4_EXTI34_IRQn EnInt                         \ interrupt enable in NVIC
  ei
;

: serkey?4      \ -- flag
  U4struct (qserkey?)  ;

: serkey4       \ -- char
  U4struct (qserkey)  ;

: seremit4      \ char --
  U4struct (qseremit)  ;

: serType4      \ c-addr len --
  U4struct (qsertype)  ;

: serCr4        \ --
  U4struct (qsercr)  ;

create Console4 \ -- addr ; OUT managed by upper driver
\ *G Generic I/O device structure for UART4.
  ' serkey4 ,           \ -- char ; receive char
  ' serkey?4 ,          \ -- flag ; check receive char
  ' seremit4 ,          \ -- char ; display char
  ' sertype4 ,          \ caddr len -- ; display string
  ' serCr4 ,            \ -- ; display new line

console-port 4 = [if]
  console4 constant console
[then]

[then]                  \ useUSART4?


\ *****************
\ *S Initialisation
\ *****************

: init-ser      \ --
\ *G Initialise primary serial ports.
  [ useUSART1? ] [if] init-ser1 [then]
  [ useUSART2? ] [if] init-ser2 [then]
  [ useUSART3? ] [if] init-ser3 [then]
  [ useUART4? ] [if] init-ser4 [then]
;


\ ======
\ *> ###
\ ======

decimal
