\ serSTM32F4xxp.fth - STM32F4xx polled serial driver

((
Copyright (c) 2009, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20120125 MPE002 Changed to use external pin selection words.
20111130 MPE001 Converted from STM32F103
))

only forth definitions
decimal

\ ===========
\ *! serstm32f4xxp
\ *T STM32F4xx polled serial driver
\ ===========
\ *P The serial driver *\i{Cortex/Drivers/serSTM4xxp.fth}
\ ** provides polled serial drivers for the on-chip USARTs.
\ ** The supplied driver implements USART1 on PA9/10 and
\ ** USART3 on PD8 (Tx) and PD9 (Rx).

\ *P When you copy this file for your own hardware do not forget
\ ** update the GPIO pin assignments and alternate function
\ ** selections.


\ ***********************
\ *S Baud rate generation
\ ***********************
\ *P The baud rate routines use 16 times sampling and the fractional
\ ** baud rate generator.

\ *E                pclkx
\ **  divisor = -------------
\ **            16 * baudrate
\ *P We then treat the divisor as an integer and four bit fraction,
\ ** effectively scaling it by 16 again.
\ *E             pclkx
\ **  BRRval = --------
\ **           baudrate

interpreter
: genBRRval	\ baud clock -- divisor
\ *G Generate the required BRR value.
  1 lshift				\ for rounding
  swap /				\ brr = clock / baud
  1+  1 rshift				\ rounding
;
target


\ **************
\ *S Shared code
\ **************

: (serKey?)	\ base -- flag
  usSR + @ bit5 and  ;

: (serkey)	\ base -- char
[ tasking? ] [if]
  begin
    dup usSR + @ bit5 and 0=
   while
    pause
  repeat
[else]
  begin
    dup usSR + @ bit5 and
  until
[then]
  usRDR + @ $FF and
;

: (seremit)	\ char base --
[ tasking? ] [if]
  begin
    dup usSR + @ bit7 and 0=
   while
    pause
  repeat
[else]
  begin
    dup usSR + @ bit7 and
  until
[then]
  usTDR + !
;

: (sertype)	\ caddr len base --
  -rot bounds
  ?do  i c@ over (seremit)  loop
  drop
;

: (sercr)	\ base --
  #13 over (seremit)  #10 swap (seremit)  ;

: +FaultConsole	\ --
\ *G For use in fault exception handlers, the multi-tasker must
\ ** be turned off and *\fo{EMIT) and friends must run in polled
\ ** mode.
[ tasking? ] [if]  single  [then]
;

: (init-ser)	\ brrval ^usart --
  $2000 over usCR1 + !			\ enable UART, 8 bits, no parity
  $0000 over usCR2 + !			\ 1 stop bit, UART mode
  $0000 over usCR3 + !			\ no interrupts, no DMA, no LIN, no smartcard
  tuck usBRR + !			\ set baud rate
  $200C over usCR1 + !			\ 8 bit, no parity, TE, RE
  drop
;


\ *********
\ *S USART1
\ *********
\ *P USART1 is on the APB2 bus, so the baud rate is calculated
\ ** using that bus clock.

useUSART1? [if]

console1-speed APB2-speed genBRRval equ /us1BRR	\ -- u
\ *G Baud rate divisor for USART1.

((
: selio-ser1	\ --
\ *G Example to perform clock and pin selection for USART1 on PA9/10
\ Clocks and reset
  _RCC
  bit0 over rccAHB1ENR + or!		\ enable clock to GPIOA
  bit0 over rccAHB1RSTR + bic!		\ take it out of reset
  bit4 over rccAPB2ENR + or!		\ enable clock to USART1
  bit4 over rccAPB2RSTR + bic!		\ take it out of reset
  drop
\ Default on PA9/10
  _GPIOA >r
  $0028:0000 $003C:0000 r@ gpioMODER + setMask	\ alternate function on 9/10
  $0000:0770 $0000:0FF0 r> gpioAFRH + setMask	\ AF7 for both
;
))

: init-ser1	\ --
\ *G Initialise USART1 for polled operation on GPIO pins PA9/10.
  selio-ser1
  /us1BRR _USART1 (init-ser)
;

: serkey?1	\ -- flag
  _USART1 (serkey?)  ;

: serkey1	\ -- char
  _USART1 (serkey)  ;

: seremit1	\ char --
  _USART1 (seremit)  ;

: serType1	\ c-addr len --
  _USART1 (sertype)  ;

: sercr1	\ --
  _USART1 (sercr)  ;

create Console1	\ -- addr ; OUT managed by upper driver
  ' serkey1 ,		\ -- char ; receive char
  ' serkey?1 ,		\ -- flag ; check receive char
  ' seremit1 ,		\ -- char ; display char
  ' sertype1 ,		\ caddr len -- ; display string
  ' sercr1 ,		\ -- ; display new line

console-port 1 = [if]
  console1 constant console
[then]

[then]			\ useUSART1?


\ *********
\ *S USART3
\ *********
\ *P USART3 is on the APB1 bus, so the baud rate is calculated
\ ** using that bus clock.

useUSART3? [if]

console3-speed APB1-speed genBRRval equ /us3BRR	\ -- u
\ *G Baud rate divisor for USART3.

((
: selio-ser3	\ --
\ *G Example to perform clock and pin selection for USART3
\ ** using GPIO pins PD8/9.
  _RCC
  bit3 over rccAHB1ENR + or!		\ enable clock to GPIOD
  bit3 over rccAHB1RSTR + bic!		\ take it out of reset
  bit18 over rccAPB1ENR + or!		\ enable clock to USART3
  bit18 over rccAPB1RSTR + bic!		\ take it out of reset
  drop
\ Default on PD8/9
  _GPIOD >r
  $000A:0000 $000F:0000 r@ gpioMODER + setMask	\ alternate function on 8/9
  $0000:0077 $0000:00FF r> gpioAFRH + setMask	\ AF7 for both
;
))

: init-ser3	\ --
\ *G Initialise USART3 for polled operation on GPIO pins PD8/9.
  selio-ser3
  /us3BRR _USART3 (init-ser)
;

: serkey?3	\ -- flag
  _USART3 (serkey?)  ;

: serkey3	\ -- char
  _USART3 (serkey)  ;

: seremit3	\ char --
  _USART3 (seremit)  ;

: serType3	\ c-addr len --
  _USART3 (sertype)  ;

: sercr3	\ --
  _USART3 (sercr)  ;

create Console3	\ -- addr ; OUT managed by upper driver
  ' serkey3 ,		\ -- char ; receive char
  ' serkey?3 ,		\ -- flag ; check receive char
  ' seremit3 ,		\ -- char ; display char
  ' sertype3 ,		\ caddr len -- ; display string
  ' sercr3 ,		\ -- ; display new line

console-port 3 = [if]
  console3 constant console
[then]

[then]			\ useUSART3?


\ *****************
\ *S Initialisation
\ *****************

: init-ser	\ --
\ *G Initialise primary serial ports.
[ useUSART1? ] [if] init-ser1 [then]
[ useUSART3? ] [if] init-ser3 [then]
;


\ ======
\ *> ###
\ ======

decimal
