\ spiSTM32hard.fth - SPI for STM32F072B Discovery board

((
Copyright (c) 2012, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140225 SFP002 Conversion to F072 Discovery board.
20121016 MPE001 Initial release.
))

only forth definitions
decimal

\ =================
\ *! stm32f0hard
\ *T STM32 SPI driver
\ =================
\ *P STM32 devices and boards vary in their use of various pins.
\ ** Consequently the file *\i{Drivers/spiSTM32hard} is not
\ ** generic. The default configuration is for the STM32F072B-Disco
\ ** board to drive the L3GD20 MEMS Gyro as an SPI master with
\ ** CPOL=1 and CPHA=1.

\ *P The SPI peripheral for STM32F0 devices is not entirely
\ ** compatible with the SPI of other STM32 families.


\ ***************************
\ *S Setup and initialisation
\ ***************************
\ *P The board uses I/O for SPI as follows.
\ *E   PI.0   AF5   SSEL1      pin 131   SPI2_NSS
\ **   PI.1   AF5   SCK1       pin 132   SPI2_SCK
\ **   PI.2   AF5   MISO1      pin 133   SPI2_MISO
\ **   PI.3   AF5   MOS11      pin 134   SPI2_MOSI

\ SPI1 - APB2
\ SPI2 - APB1
\ SPI3 - APB1

: genSPIbr	\ spi-speed bus-speed -- brx
  swap /  0 7 do			\ -- divisor
    dup 1 i lshift >
    if  drop i unloop exit  endif
  -1 +loop
  drop 0
;

interpreter
: genSPIbr	\ spi-speed bus-speed -- brx
  swap /  0 7 do			\ -- divisor
    dup 1 i lshift >
    if  drop i unloop exit  endif
  -1 +loop
  drop 0
;
target

: (initSPI)	\ freq busfreq *spi --
\ *G Set up the given SPI device, where *\i{freq} is the desired
\ ** SPI speed, *\i{busfreq} is the peripheral clock speed, and
\ ** *\i{*spi} is the SPI peripheral base address. The SPI bus is
\ ** set up in master mode, MSB first.
  >r
  bit6 r@ spiCR1 + bic!			\ clear SPE bit
  genSPIbr 3 lshift $0307 or		\ SSM=1, SSI=1, SPE=0, MSTR=1, CPOL=1, CPHA=1
\ cr ." initCR1 = " dup .dword
  r@ spiCR1 + !				\ main mode control, CPOL=0, CPHA=0
  $1700 r@ spiCR2 + !			\ F1FO threshold 1 byte, 8 data bits
  bit6 r> spiCR1 + or!			\ set SPE bit
;


\ *******
\ *S SPI2
\ *******

1 MHz equ spi2-speed	\ -- hz
\ *G Define the SPI2 clock in Herz.

[undefined] selio-spi2 [if]
PC 0  pio: SPI2_NCS_MEMS			\ software driven
PB 13 pio: SPI2_SCK
PB 14 pio: SPI2_MISO
PB 15 pio: SPI2_MOSI

: isSPIpin	\ pin --
\ *G Set up the pin as an SPI pin.
  0 over isFunction  HighSpeed swap isPinSpeed
;

: selio-spi2	\ --
\ *G Example to perform clock and pin selection for SPI2 on PC0,
\ ** PB13, PB14, and PB15.
\ Clocks and reset
  $00C0:0000 _RCC
    2dup rccAHBen + or!			\ enable clock to GPIOB/C
    rccAHBrst + bic!			\ take it out of reset
  bit14 _RCC
    2dup rccAPB1en + or!		\ enable clock to SPI2
    rccAPB1rst + bic!			\ take it out of reset
\ Default on PC0, PB13, PB14, PB15
  SPI2_NCS_MEMS dup isOutput  MediumSpeed over isPinSpeed  setPin
  SPI2_SCK isSPIpin  SPI2_MISO isSPIpin  SPI2_MOSI isSPIpin
;
[then]

\ ****************
\ *S SPI transfers
\ ****************
\ *P These routines have been optimised for speed over space
\ ** considerations. For high speed use with SD cards, you will
\ ** get best performance using interrupt driven or DMA driven
\ ** operation.

(( SPI status register
Bit	Symbol	Description
0	RXNE	Receive FIFO Not Empty. This bit is 0 if the Receive
		FIFO is empty, 1 if not.
1 	TXE	Transmit FIFO Empty. This bit is 1 is the Transmit FIFO is
		empty.
7	BSY	Busy. This bit is 0 if the SSP controller is idle,
		or 1 if it is currently sending/receiving a frame
		and/or the Tx FIFO is not empty.
))

: SPIXchg	\ bout *spi -- bin
\ *G Write data to SPIx and return the received byte.
  tuck spiDR + c!		\ -- *spi ; starts TX/RX, must be byte
  noop noop noop noop		\ time for flags to react
  begin				\ Wait for BSY=0, TXE=1, RXNE=1
    dup spiSR + @ $0083 and $0003 =
  until
  spiDR + c@ $FF and		\ return received byte
;

IsLeaf : spiFlush	\ --
\ *G Flush the SPI2 input queue.
  _SPI2 begin
    dup spiSR + @ bit0 and
   while
    dup spiDR + @ drop
  repeat
  drop
;

: spi!		\ b --
\ *G Write data to SPI2 and discard the received byte.
  _SPI2 SPIXchg drop
;

: spi@		\ -- b
\ *G Write an idle byte to SPI2 and return the received byte.
  $FF _SPI2 SPIXchg
;

: InitSPI2	\ --
\ *G Initialise SPI2.
  selio-SPI2
  #1000000 APB-speed _SPI2 (initSPI)
  spiFlush
;

: .spi2		\ --
\ *G Display SPI2 peripheral registers.
  _SPI2 $40 pdump
;

system-speed spi2-speed / equ SPIclocks	\ -- n
\ *G The number of CPU clocks in a bit time.

#10 equ clocksOver
\ *G the number of clocks needed over and above the loop itself.
\ ** This number may be sensitive to the compiler version and
\ ** switch settings.

4 equ clocksPer
\ *G the number of clocks needed for each iteration of the loop
\ ** in *\fo{spiDelay} below. This number may be sensitive to the
\ ** compiler version and switch settings. On most Cortex CPUs,
\ ** a store takes 2 cycles, a load takes 3, a branch takes 3
\ ** and all others take 1. Look at the core documentation from
\ ** ARM for clock/instruction details.
\ ** Look at your board schematics and setup code for details of the
\ ** memory bandwidth. For example, running code from a 16 bit bus
\ ** with two wait states and no cache may lead to six or more
\ ** clocks per instruction. Running code from internal buffered
\ ** Flash may result in no extra clocks in practice.

SPIclocks clocksOver - clocksPer / equ /SPIbit	\ -- n
\ *G The number n for "begin 1- dup 0= until" to generate an
\ ** SPI bit time.

/SPIbit 1 < [if]
  cr ." *** SPI requested rate too fast for CPU ***" abort
[then]

IsLeaf : SPIdelay	\ --
\ *G Wait one SPI bit time.
  /SPIbit begin  1- dup 0=  until  drop
;

: CSlo_MEMS	\ --
\ *G Set CS line low.
  spidelay  SPI2_NCS_MEMS clrPin  spidelay
;

: CShi_MEMS	\ --
\ *G Set CS line hi.
  spidelay  SPI2_NCS_MEMS setPin  spidelay  ;


\ ======
\ *> ###
\ ======

decimal
