\ FlashProg.fth - Program K60 Flash

((
Copyright (c) 2004, 2008, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
At end of programming, check for a partial block to program.

Change history
==============
20111124 MPE006 Updated for Kintis K60.
20100125 MPE005 Updated for LPC17xx
20081016 MPE004 Used intermediate buffer to avoid IAP problems
		with some devices.
20081008 MPE003 Converted for LPC23xx/24xx/28xx.
20050627 MPE002 Tables separated from FlashProg.fth
20040511 MPE001 First release
))

only forth definitions  decimal


\ ===========
\ *! flashk60
\ *T Kinetis K60 Flash tools
\ ===========
\ *P These tools are provided for applications which reserve
\ ** part of the Flash for data. This code uses the Flash primitives
\ ** in *\i{FlashFTFL.fth}.

\ *P N.B. An erase causes the whole of the sector containing that
\ ** address to be erased.

\ K60N512 Flash sectors
$1400:0000 equ _AccelRAM		\ base of accelerator RAM or FlexRAM
4 kb equ /AccelRAM			\ size of accelerator RAM
2 kb equ /FlSector			\ 2 KB Flash sectors
/MainFlash equ /Pblock			\ 512 KB total program Flash
0 equ /DEblock				\ no data/EEPROM Flash
8 equ /phrase				\ unit data size in Flash


\ *************
\ *S Primitives
\ *************

\ Flash hardware algorithm operation commands
#define FTFL_VERIFY_BLOCK               0x00
#define FTFL_VERIFY_SECTION             0x01
#define FTFL_PROGRAM_CHECK              0x02
#define FTFL_READ_RESOURCE              0x03
#define FTFL_PROGRAM_LONGWORD           0x06
#define FTFL_ERASE_BLOCK                0x08
#define FTFL_ERASE_SECTOR               0x09
#define FTFL_PROGRAM_SECTION            0x0B
#define FTFL_VERIFY_ALL_BLOCK           0x40
#define FTFL_READ_ONCE                  0x41
#define FTFL_PROGRAM_ONCE               0x43
#define FTFL_ERASE_ALL_BLOCK            0x44
#define FTFL_SECURITY_BY_PASS           0x45
#define FTFL_PFLASH_SWAP                0x46
#define FTFL_PROGRAM_PARTITION          0x80
#define FTFL_SET_EERAM                  0x81

\ Return Code Definition for FTFL SSD
#define FTFL_OK                          0x0000
#define FTFL_ERR_SIZE                    0x0001
#define FTFL_ERR_RANGE                   0x0002
#define FTFL_ERR_ACCERR                  0x0004
#define FTFL_ERR_PVIOL                   0x0008
#define FTFL_ERR_MGSTAT0                 0x0010
#define FTFL_ERR_CHANGEPROT              0x0020
#define FTFL_ERR_EEESIZE                 0x0040
#define FTFL_ERR_EFLASHSIZE              0x0080
#define FTFL_ERR_ADDR                    0x0100
#define FTFL_ERR_NOEEE                   0x0200
#define FTFL_ERR_EFLASHONLY              0x0400
#define FTFL_ERR_DFLASHONLY              0x0800
#define FTFL_ERR_RDCOLERR                0x1000
#define FTFL_ERR_RAMRDY                  0x2000

align IsLeaf
: (doFlashCmd)	\ cob03 cob47 cob8B -- fstat
\ *G Perform a Flash Command and return the status byte. If bit 7
\ ** of the status is clear, a timeout occurred.
\ ** This routine is run from RAM.
  swap rot _FTFL			\ -- cob8B cob47 cob03 ^ftfl
  begin  dup ftflFSTAT + c@ $80 and  until	\ wait command complete
  $070 over ftflFSTAT + c!		\ clear previous errors
  over #24 rshift over ftflFCCOB0 + c!	\ command byte
  over #16 rshift  over ftflFCCOB1 + c!
  over 8 rshift over ftflFCCOB2 + c!
  tuck ftflFCCOB3 + c!			\ -- cob8B cob47 ^ftfl
  over #24 rshift over ftflFCCOB4 + c!
  over #16 rshift  over ftflFCCOB5 + c!
  over 8 rshift over ftflFCCOB6 + c!
  tuck ftflFCCOB7 + c!			\ -- cob8B ^ftfl
  over #24 rshift over ftflFCCOB8 + c!
  over #16 rshift  over ftflFCCOB9 + c!
  over 8 rshift over ftflFCCOBA + c!
  tuck ftflFCCOBB + c!			\ -- ^ftfl
  $080 over ftflFSTAT + c!		\ clear CCIF bit by writing 1
  10 begin 1- dup 0= until drop
  system-speed begin			\ -- ^ftfl count
    1- dup				\ check for timeout
   while
    over ftflFSTAT + c@ $080 and
  until then
  drop  ftflFSTAT + c@			\ return status
;
align
here ' (doFlashCmd) - equ /doFlashCmd	\ length of code to copy to RAM
/doFlashCmd buffer: rdoFlashCmd	\ -- addr
\ *G The RAM buffer used for the Flash programming code.

: doFlashCmd	\ cob03 cob47 cob8B -- fstat
\ *G Perform a Flash Command and return the status byte. If bit 7
\ ** of the status is clear, a timeout occurred.
  ['] (doFlashCmd) rdoFlashCmd /doFlashCmd cmove
  [i  rdoFlashCmd execute  i]
;

: eraseSector	\ addr -- stat
\ *G Erase the sector containing the address.
  $0900:0000 or  0 0 doFlashCmd
;

: writeFlash	\ dest len -- stat
\ *G Write *\i{len} bytes from the start of the programming RAM
\ ** to the Flash at *\i{dest}, returning the raw status byte.
  swap $0B00:0000 or			\ -- len cob03 ; make cmd + dest
  swap 3 rshift #16 lshift		\ -- cob03 cob47 ; number of phrases
  0 doFlashCmd
;

: writeSector	\ src dest len -- stat
\ *G Write the given data to the sector. *\i{dest} should be
\ ** the start of the sector.
  tuck 2>r				\ -- src len ; R: -- dest len
  _AccelRAM swap cmove			\ copy data to program RAM
  2r> writeFlash			\ perform write
;


\ *************
\ *S User level
\ *************

cell buffer: #PrgErrs	\ -- addr
\ *G Holds error count.

: .src/dest	\ src dest -- src dest
\ *G Display the addresses and contents of the source
\ ** and destination.
  ."  source: " over .lword  over c@ ." (" .byte ." )"
  ."  dest: " dup .lword  dup c@ ." (" .byte ." )"
;

: EraseUnit	\ dest --
\ *G Erase the flash sector starting at *\i{dest}. On error,
\ ** the variable *\fo{PrgErrs} is incremented.
  10 0 do				\ -- dest
    dup eraseSector $080 = if
      drop unloop exit
    endif
    10 ms
  loop
  drop  #PrgErrs incr
;

: ProgUnit	\ src dest --
\ *G Program *\fo{/FlashSector} bytes at *\i{src} to *\i{dest}. On error, the
\ ** variable *\fo{PrgErrs} is incremented.
  #10 0 do
    2dup /FlashSector writeSector $080 =
    if  2drop  unloop exit  endif
    10 ms
    dup EraseUnit
  loop
  2drop  #PrgErrs incr
;

: (EraseFlash)	\ dest dlen --
\ *G Erase the flash for the given range. Note that complete
\ ** sectors in the range will be erased.
  bounds
  ?do  i EraseUnit  /FlashSector +loop
;

: (ProgFlash)	\ src dest len --
\ *G Write len bytes from memory at src to Flash at *\i{dest}.
\ ** Note that the Flash is assumed to be erased, and that
\ ** no verification is performed except when each byte is
\ ** programmed. *\i{Dest} is forced to a 2k byte boundary and
\ ** *\i{len} is rounded up to the next 2k byte unit. *\i{Src}
\ ** must be on a word boundary.
  rot -4 and -rot			\ force src
  swap #-2048 and swap			\ force dest
  #2047 + #-2048 and			\ force len
  bounds ?do				\ -- src'
    dup i ProgUnit
    /FlashSector +
  /FlashSector +loop
  drop
;

: (VerifyFlash)	\ src dest len --
\ *G Verify len bytes from memory at src to Flash at dest.
  3dup s= if  3drop exit  endif
  cr ."  Verify failed"
  0 ?do
    over c@ over c@ <> if
      cr 2 spaces .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    1 + swap  1 + swap
  1 +loop
  2drop
;

: .PrgErrs	\ --
\ Display the number of erase/program errors.
  space #PrgErrs ? ." errors"
;

: ProgramFlash	\ src dest len --
\ *G Using the RAM memory buffer at *\i{src}, program the Flash
\ ** at *\i{dest} with *\i{len} bytes. The relevant Flash sectors
\ ** are erased, the Flash is programmed, and the result verified.
  0 #PrgErrs !
  cr ." Erasing " 2dup (EraseFlash) .PrgErrs
  0 #PrgErrs !
  cr ." Programming "  3dup (ProgFlash) .PrgErrs
  cr ." Verifying "  (VerifyFlash)
;


\ ======
\ *> ###
\ ======

