\ K60twrBlinky.fth - Cortex startup code and LED flasher
\ for K60 family on Tower hardware.

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20111109 MPE001 First release.
))

only forth definitions  decimal

\ ===============
\ *! k60twrblinky
\ *T Kinetis K60 Blinky
\ ===============
\ *P The file *\i{Cortex/Hardware/Kinetis/startK60twr.fth} contains
\ ** the start up and LED flasher code for Kintis K60 devices.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the ARMv7-M
\ ** Architecture Reference Manual. If placed at 0 or the start of
\ ** Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick

$0400 ExcVecs + org			\ normally at $0000:0400
\ Install the 16 byte Flash Configuration field
l: FlashCfgField
  $FF c,  $FF c,  $FF c,  $FF c,	\ 8 byte Backdoor key
  $FF c,  $FF c,  $FF c,  $FF c,	\
  $FF c,  $FF c,  $FF c,  $FF c,	\ FPROT0, FPROT1, FPROT2, FPROT3
  $FE c,  $FF c,  $FF c,  $FF c,	\ FSEC, FOPT, FEPROT, FDPROT

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. The
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
  bit0 _SIM simSOPT2 + or!		\  select main oscillator for MCG
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

: enableGPIOs	\ --
\ *G Configuring any pin or alternate function requires the
\ ** GPIOs to be clocked, so we enable them all at startup.
  $3E00 _SIM simSCGC5 + or!		\ enable clocks to all PORTs
;


: ledDelay	\ --
  5000000 begin  1- dup 0= until  drop
;

: +led1		\ -- ; turn on LED1, pin -> 0
  bit11 _PTA ptPCOR + !			\ PA.11 -> 0
;
: -led1		\ -- ; turn off LED1, pin -> 1
  bit11 _PTA ptPSOR + !			\ PA.11 -> 1
;

: initLed1	\ -- ; initialise LED E1 on PA.11
  $64 _PORTA portPCR11 + or!		\ set SRE, ODE, DSE
  $0100 _PORTA portPCR11 + or!		\ MUX(10:8) := 1 = GPIO
  bit11 _PTA ptPDDR + or!		\ DDR=1=o/p
;

: +led2		\ -- ; turn on LED E2, pin -> 0
  bit28 _PTA ptPCOR + !			\ PA.11 -> 0
;
: -led2		\ -- ; turn off LED E2, pin -> 1
  bit28 _PTA ptPSOR + !			\ PA.11 -> 1
;

: initLed2	\ -- ; initialise LED E2 on PA.11
  bit9 _SIM simSCGC5 + or!		\ enable clock to PORTA
  $64 _PORTA portPCR28 + or!		\ set SRE, ODE, DSE
  $0100 _PORTA portPCR28 + or!		\ MUX(10:8) := 1 = GPIO
  bit28 _PTA ptPDDR + or!		\ DDR=1=o/p
;

: initLEDs	\ --
\ Initialise LEDs for testing, P0.4 and P1.25 default to GPIO.
  initLed1 initLed2
  +led1 +led2
;

: flashLED	\ --
  initLEDs
  begin
    +led1 -led2 ledDelay
    -led1 +led2 ledDelay
  again
;

: disDog	\ -- ; no watchdog
  _WDOG
  $C520 over wdgUNLOCK + w!
  $D928 over wdgUNLOCK + w!
  $D2 swap wdgSTCTRLH + w!		\ unlock value per C header file
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    disDog setClocks setNVIC
    enableGPIOs				\ always needed
    flashLED
\    CLD1 @ execute
  again
;
external


INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ======
\ *> ###
\ ======

decimal

