\ K60twrNet.ctl - Example PowerNet standalone Forth for K60 Tower system

((
Copyright (c) 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140429 MPE003 Updated so that FRDIV can be varied.
20111124 MPE002 With most features.
20111103 MPE001 First release.
))


\ ============
\ *! k60twr
\ *T K60 Tower system Forth
\ ============
\ *P The Kinetis K60 has a peculiarity in that to use the Ethernet
\ ** module with a PHY in RMII mode, you have to use a 50MHz
\ ** external clock module. See the reference manual section 3.9.1.2
\ ** "RMII Clocking". This code assumes the use of a 50MHz module.

\ *P This code was built and tested for a Freescale Tower System
\ ** with TWR-K60N512 and TWR-SER modules. Note that the CPU board
\ ** must be jumpered for the 50MHz module on the the TWR-SER
\ ** board and the Ethernet PHY on the TWR-SER module set up for
\ ** RMII operation.

\ *P When you program the board using the P&E or Freescale tools,
\ ** you may have to power cycle the board.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************]
\ Make a copy of the MPE macros and adjust them for your directory
\ layout. If you have several developers with different layouts,
\ build a constant and a macro set for each developer. Only one
\ constant should be set non-zero.

1 constant MPE?	\ true if built at MPE
0 constant xxx?	\ true if built at ...

MPE? [if]	\ if built on an MPE development box

c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
mc" %ExampleDir%/FATfiler"		\ where FAT file system lives
                setMacro FatFiler
mc" %ExampleDir%/USB"
                setmacro UsbDir		\ where USB code lives
c" ."    	setmacro AppDir		\ where application code lives
[defined] Target_386_Windows [if]
c" c:/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]
[defined] Target_386_Linux [if]
c" ~/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]
[then]

xxx? [if]
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M3			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
  Hex-I32			\ produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files
  -const-visible		\ CONST = EQU

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ==================
\ *N K60 definitions
\ ==================
\ *P On the K60N512 Tower board the CPU is a PK60N512VMD or
\ ** MK60N512VMD100.

#512 kb equ /MainFlash	\ -- len
\ *G Define the size of the internal Flash.
2 kb equ /FlashSector	\ -- len
\ *G Define the size of a Flash sector.
#510 kb equ /AppFlash	\ -- len
\ *G Define how much of the Flash is used for the application.
\ ** Often the last sector is reserved for application
\ ** configuration data.
$0007:F800 equ CfgFlash	\ -- addr
\ *G Where the application configuration is stored.

#128 kb equ /MainRAM	\ -- u
\ *G Define the total size in RAM of the internal RAM blocks
\ ** based around address $2000:0000.

#128 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

#50 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ. Synced to PHY
\ ** if Ethernet is in use.
#96 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ. Max 100 MHz.
#48 MHz equ bus-speed		\ -- hz
\ *G Requested bus clock speed in HZ. Max 50 MHz.
#24 MHz equ flex-speed		\ -- hz
\ *G Requested FlexBus clock speed in HZ. Max 50 MHz.
#24 MHz equ flash-speed		\ -- hz
\ *G Requested Flash clock speed in HZ. Max 25 MHz.

\ *P The PLL is used to generate a 96 MHz system clock.
25 1 - equ /PRDIV	\ -- u
\ *G PLL input divider = 25, PLL freq = 2 MHz
48 24 - equ /VDIV	\ -- u
\ *G PLL multiplier = 48, output freq = 96 MHz

10 5 - 3 lshift equ /FRDIV
\ *G The FLL can be used for some clocks. The FLL base clock is
\ ** the crystal divided by 2^n for RANGE=0 or 2^(n+5) for RANGE<>0
\ ** and should be in the range 31.25 kHz to 39.06 kHz if the main
\ ** clock is derived from the FLL. *\fo{/FRDIV} is in bits 5:3 of
\ ** the MCG_C1 register.

#2 MHz equ irc-speed		\ -- hz
\ *G Internal RC oscillator clock rate in HZ.
1 kHz equ lpo-speed		\ -- hz
\ *G Internal low power oscillator speed in Hz.
#32768 equ rtc-speed		\ -- hz
\ *G External RTC oscillator clock rate in HZ.

#48 MHz equ usb-speed		\ -- hz
\ *G Requested clock speed to USB peripheral.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $1FFF:F000..$1FFF:FFFF. Ensure that
\ ** your stacks are outside this region.

\ *P Flash memory starts at $0000:0000.
\ *P SRAM is based around $2000:0000.

$2000:0000 equ _SRAM
/MainRAM 2/ equ /SRAMblock
_SRAM /SRAMblock - equ SRAMstart
_SRAM /SRAMblock + 1- equ SRAMend

  0 /AppFlash 1- cdata section K60twrNet	\ code section in main Flash
  SRAMstart dup $3FFF + idata section PROGd	\ IDATA
  SRAMstart $4000 + SRAMend udata section PROGu	\ UDATA

interpreter
: prog   K60twrNet  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0100 equ FP-SIZE		\ size of each task's float stack
$0200 equ RP-SIZE		\ size of each task's return stack
rp-size sp-size + fp-size + up-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

0 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

1 equ useUART3?	\ -- n
\ *G Set non-zero to compile code for UART3,
\ ** device *\fo{Console3}.
#115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUART4?	\ -- n
\ *G Set non-zero to compile code for UART4,
\ ** device *\fo{Console4}.
#9600 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

0 equ useUART5?	\ -- n
\ *G Set non-zero to compile code for UART5,
\ ** device *\fo{Console5}.
#9600 equ console5-speed
\ *G *\fo{Console5} speed in BPS.

3 equ console-port	\ -- n
\ *G Designate the serial port used for the Forth terminal (0..n).
\ ** Ports 0..5 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 1 equ softfp?			\ true for software floating point
 0 equ hardfp?			\ true for hardware IEEE floating point, e.g. M4F
 1 equ sepFPstacks?		\ true for separate data and FP stacks.
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $8000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required
 1 equ PowerNet?		\ true for the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM4def		\ Cortex-M4 generic equates and SFRs
  include %CpuDir%/sfrK60		\ K60 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %AppDir%/startK60twr		\ startup code
0 equ FastCmove?
  include %CpuDir%/CodeCortex		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootCM4	\ reboot using NVIC
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault handlers - needs serial
  include %CommonDir%/CQueues		\ character queues
  include %CpuDir%/Drivers/serK60qi	\ RX/TX queued interrupt driver
\  include %CpuDir%/Drivers/serK60p	\ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM3		\ local variables

tasking? [if]
  include %CpuDir%/MultiCortex		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
\  include %CpuDir%/sfp32Cortex		\ floating point primitives - coded version
\  include %CommonDir%/sfp32com		\ common floating point code
  sepFPstacks? [if]
    include %CommonDir%/SoftFP32SX
  [else]
    include %CommonDir%/SoftFP32CX
  [then]
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile K60twrNet.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." **********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Kinetis-K60"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  synonym ser-emit emit
  synonym ser-key? key?
  synonym ser-key  key
  include %CommonDir%/ROMFORTH/IODEF
  include %CommonDir%/ROMFORTH/textfile

  include %CpuDir%/Drivers/gpioK60	\ easy GPIO drivers
  include %HwDir%/FlashProg		\ Flash programming routines
  include %HwDir%/ReProg/ReFlash	\ Reflash system
  include %CpuDir%/Drivers/rtcK60	\ RTC

1 equ XC?
PowerNet? [if]
  1 equ CPU=Cortex	\ to compile Cortex specific code
  0 equ FilePages?			\ no file system of HTTP page storage
  include %AppDir%/K60PNcfg		\ PowerNet configuration
  include %IpStack%/NetCode		\ network order stuff
  include %AppDir%/WebConfig		\ configuration tools
  include %CpuDir%/Drivers/EtherK60	\ Ethernet driver
  include %IpStack%/PowerNet.bld	\ PowerNet build
\  include %IpStack%/SOCKETS/ws.fs
\  include %IpStack%/Examples/WebPost	\ application update
[else]
\  include %ExampleDir%/DebugTools
\  include %CommonDir%/NetCode		\ network order stuff
\  include %CpuDir%/Drivers/EtherK60	\ Ethernet driver
[then]

((
\ USB - CDC or MSC only
interpreter
: URAMP  URAM  ;			\ needed by USB code
target
  include %UsbDir%/UsbDefs		\ USB definitions/equates
\ include %UsbDir%/MSCconfig		\ USB configuration file
\ include %UsbDir%/CDCconfig		\ USB configuration file
  include %UsbDir%/CdcMscConfig		\ USB configuration file
\ include %AppDir%/CdcMscConfig		\ USB configuration file
  include %UsbDir%/devK60twr.fth	\ board specific set up
  include %UsbDir%/UsbHwK60		\ USB DMA hardware layer for LPC23xx
  include %UsbDir%/UsbBase		\ USB base operations
MSC? [if]
  include %UsbDir%/MscDrv		\ Mass Storage Class
[then]
CDC? [if]
  include %UsbDir%/VCOMdev		\ UART device emulation
  include %UsbDir%/CdcDrv		\ Communications Device Class
[then]
  include %UsbDir%/UsbEP0		\ USB control and initialisation

: StartApp	\ --
  initFATfs .
[defined] PowerNet [if]  PowerNet  [then]
  StartUSBsys
;
))


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build

\ Define device, speed (0=auto) and project file
\ Download the target image when compilation finishes
jlink MK60DN512xxx10 0 Project.jlink	\ JLINK and device setup
\ -download-prompts			\ no prompting for download
afterwards  download-sections		\ download image after FINIS
FINIS


\ ======
\ *> ###
\ ======

