\ FlashFTFL.fth - Kinetis Flash programming

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! filename
\ *T Chapter Title
\ ===========

\ K60N512 Flash sectors
$1400:0000 equ _AccelRAM		\ base of accelerator RAM or FlexRAM
4 kb equ /AccelRAM			\ size of accelerator RAM
2 kb equ /FlSector			\ 2 KB Flash sectors
/MainFlash equ /Pblock			\ 512 KB total program Flash
0 equ /DEblock				\ no data/EEPROM Flash
8 equ /phrase				\ unit data size in Flash

\ Flash hardware algorithm operation commands
#define FTFL_VERIFY_BLOCK               0x00
#define FTFL_VERIFY_SECTION             0x01
#define FTFL_PROGRAM_CHECK              0x02
#define FTFL_READ_RESOURCE              0x03
#define FTFL_PROGRAM_LONGWORD           0x06
#define FTFL_ERASE_BLOCK                0x08
#define FTFL_ERASE_SECTOR               0x09
#define FTFL_PROGRAM_SECTION            0x0B
#define FTFL_VERIFY_ALL_BLOCK           0x40
#define FTFL_READ_ONCE                  0x41
#define FTFL_PROGRAM_ONCE               0x43
#define FTFL_ERASE_ALL_BLOCK            0x44
#define FTFL_SECURITY_BY_PASS           0x45
#define FTFL_PFLASH_SWAP                0x46
#define FTFL_PROGRAM_PARTITION          0x80
#define FTFL_SET_EERAM                  0x81

\ Return Code Definition for FTFL SSD
#define FTFL_OK                          0x0000
#define FTFL_ERR_SIZE                    0x0001
#define FTFL_ERR_RANGE                   0x0002
#define FTFL_ERR_ACCERR                  0x0004
#define FTFL_ERR_PVIOL                   0x0008
#define FTFL_ERR_MGSTAT0                 0x0010
#define FTFL_ERR_CHANGEPROT              0x0020
#define FTFL_ERR_EEESIZE                 0x0040
#define FTFL_ERR_EFLASHSIZE              0x0080
#define FTFL_ERR_ADDR                    0x0100
#define FTFL_ERR_NOEEE                   0x0200
#define FTFL_ERR_EFLASHONLY              0x0400
#define FTFL_ERR_DFLASHONLY              0x0800
#define FTFL_ERR_RDCOLERR                0x1000
#define FTFL_ERR_RAMRDY                  0x2000

: doFlashCmd	\ cob03 cob47 cob8B -- fstat
\ *G Perform a Flash Command and return the status byte. If bit 7
\ ** of the status is clear, a timeout occurred.
\ ** This routine is run from RAM.
  swap rot _FTFL			\ -- cob8B cob47 cob03 ^ftfl
  begin  dup ftflFSTAT + c@ $80 and  until	\ wait command complete
  $070 over ftflFSTAT + c!		\ clear previous errors
  over #24 rshift over ftflFCCOB0 + c!	\ command byte
  over #16 rshift  over ftflFCCOB1 + c!
  over 8 rshift over ftflFCCOB2 + c!
  tuck ftflFCCOB3 + c!			\ -- cob8B cob47 ^ftfl
  over #24 rshift over ftflFCCOB4 + c!
  over #16 rshift  over ftflFCCOB5 + c!
  over 8 rshift over ftflFCCOB6 + c!
  tuck ftflFCCOB7 + c!			\ -- cob8B ^ftfl
  over #24 rshift over ftflFCCOB8 + c!
  over #16 rshift  over ftflFCCOB9 + c!
  over 8 rshift over ftflFCCOBA + c!
  tuck ftflFCCOBB + c!			\ -- ^ftfl
  $080 over ftflFSTAT + c!		\ clear CCIF bit by writing 1
  10 begin 1- dup 0= until drop
  CLD_clockspeed @ begin		\ -- ^ftfl count
    1- dup				\ check for timeout
   while
    over ftflFSTAT + c@ $080 and
  until then
  drop  ftflFSTAT + c@			\ return status
;

: eraseSector	\ addr -- stat
\ *G Erase the sector containing the address.
  $0900:0000 or  0 0 doFlashCmd
;

: writeFlash	\ dest len -- stat
\ *G Write *\i{len} bytes from the start of the programming RAM
\ ** to the Flash at *\i{dest}, returning the raw status byte.
  swap $0B00:0000 or			\ -- len cob03 ; make cmd + dest
  swap 3 rshift #16 lshift		\ -- cob03 cob47 ; number of phrases
  0 doFlashCmd
;

: writeSector	\ src dest len -- stat
\ *G Write the given data to the sector, returning the raw status
\ ** byte.
  tuck 2>r				\ -- src len ; R: -- dest len
  _AccelRAM swap cmove			\ copy data to program RAM
  2r> writeFlash			\ perform write
;


\ ======
\ *> ###
\ ======

decimal
