\ Reflash.fth - handles reflashing the system

((
Copyright (c) 2006, 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20111124 SFP003 Kinetis K60 conversion.
20100125 SFP002 Cortex conversion from LPC23xx code.
		Added FlashSize setting.
20060701 SFP001 Added CopyFlash.
))


\ ==========
\ *! reflash
\ *T Freescale Kinetis Reflashing
\ ==========
\ ***************
\ *S Introduction
\ ***************
\ *P If you destroy the application in the internal Flash, you
\ ** must use third party tools to reflash the unit.

\ *P Once the Forth system is running again, you can use the word
\ ** *\fo{REFLASH ( -- )} to download a new binary image.
\ ** Note that *\fo{REFLASH} only handles binary memory image
\ ** files. These should be transferred using the XModem 128
\ ** (checksum) protocol. If you are using AIDE with the file
\ ** server enabled, a file selection dialog will appear
\ ** automatically after you have executed *\fo{REFLASH}.

\ *P The Kinetis parts can only be reflashed from an application by
\ ** a program running from RAM. A separate application built
\ ** by a control file *\i{Reprog\Reprog*.CTL} is used to do this. A
\ ** binary image *\fo{REPROG*.IMG} is inserted into the application.
\ ** When required, the code is copied into the internal RAM and
\ ** executed from RAM.

\ *P Return is by rebooting the system. The contents of the
\ ** internal RAM and Flash are destroyed, therefore any data
\ ** that must be preserved should be saved externally to the
\ ** chip. Alternatively, modify the reprogramming code to
\ ** use the last sector to hold data to be
\ ** preserved.


\ ***************************
\ *S Code in main application
\ ***************************
\ *P The layout of the RAM code is defined in the control file
\ ** for the RAM application. This defines the first cell as
\ ** the Forth word to execute. The RAM code is included in the
\ ** Flash image and copied to RAM when needed.

create reprog.img	\ -- addr
\ *G The start address of the reprogramming code
  data-file %HwDir%/ReProg/REPROGK60.img equ /reprog	\ -- len
\ *G Generic K60: The length of the reprogramming code loaded into
\ ** the dictionary.

$1FFF:F000 equ reprogrun	\ -- addr
\ *G The run time address of the reprogramming code.
\ ** This *\b{must} match the start address of the *\fo{CDATA}
\ ** section defined in *\i{ReprogK60.ctl}, and that section
\ ** *\b{must not} overlap the run-time stacks and user area.

_UART3 value ReflashDev	\ -- addr
\ *G Holds THE UART base address used for XModem transfer. The
\ ** default is UART3.

\ *P The start of the RAM code block holds data about the
\ ** running Forth.
\ *(
\ *B Cell 0 - xt of *\fo{ReprogFlash} to run,
\ *B Cell 1 - xt of *\fo{CopyFlash} to run,
\ *B Cell 2 - 0 for default UART or base address of another UART,
\ *B Cell 3 - clock speed in Hz of the calling Forth,
\ *B Cell 4 - Length of Flash to reprogram. Often the last sector
\ ** is used to hold configuration information.
\ *B Cell 5 - Size of a Flash sector.
\ *)

: callit	\ xt --
\ *G Load the reprogramming code and execute the xt.
  [defined] dfi [if] dfi [then]		\ disable FIQ
  di					\ disable IRQ [and FIQ]
  ReflashDev				\ get device selection BEFORE copy
  reprog.img reprogrun /reprog cmove	\ copy code to RAM
  reprogrun tuck 2 cells + !		\ set device base address
  system-speed over 3 cells + !		\ set device clock speed
  /AppFlash over 4 cells + !		\ set number of Flash sectors
  /FlashSector swap 5 cells + !		\ set size of a Flash sector
  execute
  [defined] reboot [if]  reboot  [else]  ExcVecs 4 + @ execute  [then]
;

: reflash	\ -- ; no exit
\ *G Copies the reprogramming code to the run-time address
\ ** and executes it.
  reprog.img @ callit
;

: CopyFlash	\ src dest len --
\ *G Copy the flash. The parameters are as for *\fo{CMOVE}.
\ ** The process is repeated until there are no errors, and
\ ** the system is then rebooted.
  reprog.img cell + @ callit
;


\ ======
\ *> ###
\ ======

