\ ReprogApp.fth - reprogram K60 Flash

((
Copyright (c) 2004, 2008, 2009, 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
At end of download, check for a partial block to program.

Change max program size to depend on CPU.

Change history
==============
20111124 SFP006 Kinetis K60 converion.
20100125 SFP005 LPC17xx conversion.
20090205 SFP004 LPC2388 does not erase last sector at $0007:D000
20081008 SFP003 Minor documentation changes.
20060701 SFP002 Added CopyFlash.
20050627 MPE002 Now uses sector tables.
20040212 MPE001 First release
))

only forth definitions  decimal


\ ****************
\ *S Flash handler
\ ****************

cell buffer: FlDest	\ -- addr
\ *G Holds next destination address

cell buffer: #PrgErrs	\ -- addr
\ *G Holds error count

: EraseUnit	\ dest --
\ *G Erase the sector starting at *\i{dest}. On error,
\ ** the variable *\fo{PrgErrs} is incremented.
  10 0 do				\ -- dest
    dup eraseSector $080 = if
      drop unloop exit
    endif
    10 ms
  loop
  drop  #PrgErrs incr
;

: (EraseFlash)	\ dest dlen --
\ *G Erase the Flash starting at *\i{dest} for *\i{dlen} bytes.
  bounds
  ?do  i EraseUnit  CLD_/FlSector @ +loop
;

: EraseFlash	\ --
\ *G Erase the Flash for reprogramming.
  0 CLD_/FlUsed @ (EraseFlash)
;

: InitXBuff	\ --
\ *G Initialise the Flash buffer
  Xbase ^FlBuffer !
;

: RAM>Flash	\ dest len --
\ *G Write the programming RAM to Flash.
  writeFlash $080 <>
  if  #PrgErrs incr  endif
;

: toFlash	\ src dest --
\ *G Handle 128 bytes at src to go to dest in Flash.
  nip					\ src is always Xbase is always _AccelRAM
  #128 RAM>Flash
;


\ *****************
\ *S Numeric output
\ *****************

: .nibble	\ n --
\ *G Convert a nibble to a hex ASCII digit and display it.
  $0F and  dup 9 >
  if  7 +  then
  $30 + emit
;

: .WORD		\ w --
\ *G Display w as a 4 digit unsigned hexadecimal number.
  dup #12 rshift .nibble
  dup 8 rshift .nibble
  dup 4 rshift .nibble
  .nibble
;

: .dword	\ x --
\ *G Display x as an 8 digit unsigned hexadecimal number.
  dup #16 rshift .word .word
;


\ ***************
\ *S Reprogrammer
\ ***************

l: m1
  ", Erasing "
l: m2
  ",  done "
l: m3
  ", Start Xmodem TX "
[defined] REBOOT [if]
l: m4
  ", Press R to reboot, other to reprogram "
[else]
l: m4
  ", Reset or press key to reprogram"
[then]
l: m5
  ", Errors"
l: m6
  ", No "

: $.	\ c-addr --
\ *G Display counted string.
  count type
;

: .PrgErrs
\ *G Display number of errors.
  cr  #PrgErrs @ .word  bl emit
  m5 $.
;

: initReprog	\ --
\ *G Get the setup data and calculate the requirements
  init-ser #50 ms
;

: ReprogFlash	\ --
\ *G Erase and program the Flash.
  initReprog
  begin
    InitXbuff  #PrgErrs off
    cr m1 $.  #20 ms
    EraseFlash
    m2 $.  .PrgErrs  #PrgErrs off
    cr m3 $.  $01A emit			\ send message and trigger
    0 dup FlDest !  $00400000 Bin-Up drop
    .PrgErrs
    #100 ms
    cr m4 $.
[defined] REBOOT [if]
    key $DF and dup emit  [char] R =
  until
  #100 ms
  reboot
[else]
    key drop
  again
[then]
;

\ SFP002...
l: msg7
  ", Copy "

: (CopyFlash)	\ src dest len --
\ *G Copy one Flash area to another. The parameters are as for *\fo{CMOVE}.
  initReprog
  cr m1 $.  #20 ms			\ "Erase" message
  #PrgErrs off
  2dup (EraseFlash) .PrgErrs  #10 ms	\ -- src dest len
  #PrgErrs @
  if  3drop exit  endif
  cr msg7 $.				\ "Copy" message
  0 do					\ -- src dest
    2dup #512 writeSector
    #512 +  swap #512 + swap
  #512 +loop
  2drop
  .PrgErrs  #20 ms			\ error count
;

: CopyFlash	\ src dest len --
\ *G Copy the flash. The parameters are as for *\fo{CMOVE}.
\ ** The process is repeated until there are no errors, and
\ ** the system is then rebooted.
  begin
    3dup (CopyFlash)  #PrgErrs @ 0=
  until
  #100 ms  reboot
;
\ ...SFP002


\ ======
\ *> ###
\ ======

