\ StartK60twr.fth - Cortex startup code for K60 family on Tower hardware.

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140429 MPE002 Updated so that FRDIV can be varied.
20111111 MPE001 First release.
))

only forth definitions  decimal

\ ==============
\ *! startk60twr
\ *T Cortex start up for Kinetis K60
\ ==============
\ *P The file *\i{Cortex/Hardware/Kinetis/startK60twr.fth} contains
\ ** start up code for Kinetis K60 devices. Start up code for other
\ ** Cortex-M3/M4 implementations, e.g. STM32, can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M4
\ ** is in *\i{Cortex/StartCortex.fth}.

\ *P Note that this start up code is specific to the K60N512 Tower
\ ** Module. Use this file as the basis of your own code after copying
\ ** and renaming the file.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the ARMv7-M
\ ** Architecture Reference Manual. If placed at 0 or the start of
\ ** Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
\ *G Holds the xt of the main action after initial booting.
\ ** For a standalone Forth, this is usually the xt of *\fo{COLD}.
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick

$0400 ExcVecs + org			\ normally at $0000:0400
\ Install the 16 byte Flash Configuration field
l: FlashCfgField
  $FF c,  $FF c,  $FF c,  $FF c,	\ 8 byte Backdoor key
  $FF c,  $FF c,  $FF c,  $FF c,	\
  $FF c,  $FF c,  $FF c,  $FF c,	\ FPROT0, FPROT1, FPROT2, FPROT3
  $FE c,  $FF c,  $FF c,  $FF c,	\ FSEC, FOPT, FEPROT, FDPROT

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. The
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

\ Calculate the values of the dividers after MCGOUTCLK.
system-speed system-speed / 1- equ vOUTDIV1	\ value of OUTDIV1, system clock
system-speed bus-speed / 1- equ vOUTDIV2	\ value of OUTDIV2, bus clock
system-speed flex-speed / 1- equ vOUTDIV3	\ value of OUTDIV3, FlexBus clock
system-speed flash-speed / 1- equ vOUTDIV4	\ value of OUTDIV1, Flash clock

vOUTDIV1 #28 lshift
vOUTDIV2 #24 lshift or
vOUTDIV3 #20 lshift or
vOUTDIV4 #16 lshift or
  equ vCLKDIV1	\ -- x
\ *G Initial value for clock divider registers.

internal

align
IsLeaf : (setClocks)	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required. This routine is run from RAM. See errata sheet
\ ** *\i{KINETIS_0M33Z.pdf}, e2448.
  _FMC fmcPFAPR + @ dup >r		\ save prefetch default
  $00FF:0000 or _FMC fmcPFAPR + @	\ disable all prefetch

  vCLKDIV1 _SIM simCLKDIV1 + !		\ set clock dividers
  _SIM simCLKDIV2 + dup @		\ set USBDIV=1, USBFRAC=0, for divide by 2
    $FFFF:FFF0 and  $02 or		\ from 96MHz to 48MHz
  swap !

  _MCG					\ -- ^mcg ; R: -- prefetch

  \ Switch to FBE Mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=0, C2[LP]=0
  $0A0 _OSC oscCR + c!			\ ERCLKEN=1,EREFSTEN=1,SC2P=0,SC4P=0,SC8P=0,SC16P=0
  bit0 _SIM simSOPT2 + bic!		\ select OSCCLK oscillator for MCG
  $020 over mcgC2 + c!			\ MCG_C2: RANGE=2,HGO=0,EREFS=0,LP=0,IRCS=0
  %10000010 /FRDIV or over mcgC1 + c!	\ MCG_C1: CLKS=2,FRDIV=3/5,IREFS=0,IRCLKEN=1,IREFSTEN=0
  $0E0 over mcgC4 + bbic!		\ MCG_C4: DMX32=0,DRST_DRS=0
  $020 /PRDIV + over mcgC5 + c!		\ MCG_C5: PLLCLKEN=0,PLLSTEN=1,PRDIV=x
  bit6 over mcgC5 + bor!		\ MCG_C5: PLLCLKEN=1, enable PLL
  $000 /VDIV + over mcgC6 + c!		\ MCG_C6: LOLIE=0,PLLS=0,CME=0,VDIV=y
  begin  dup mcgS + c@ bit1 and  until	\ Check that the oscillator is running
  begin  dup mcgS + c@ bit4 and 0=  until \ Check that the source of the FLL reference clock is the external reference clock.
  begin  dup mcgS + c@ $0C and $08 = until \ Wait until external reference clock is selected as MCG output */
\ ok
  \           FBE mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=0, C2[LP]=0
  \ Switch to PBE mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=1, C2[LP]=0
  bit6 over mcgC5 + bbic!		\ clear PLLCLKEN - why?
  bit6 over mcgC6 + bor!		\ set PLLS
  begin  dup mcgS + c@ $0C and $08 = until \ Wait until external reference clock is selected as MCG output
  \           PBE mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=1, C2[LP]=0
  \ Switch to PEE mode: C1[CLKS]=00, C1[IREFS]=0, C6[PLLS]=1, C2[LP]=0
  $0C0 over mcgC1 + bbic!		\ clear CLKS bits
  begin  dup mcgS + c@ $0C and $0C = until \ Wait until PLL output is selected as MCG output
  begin  dup mcgS + c@ bit6 and  until	\ Wait for LOCK
\ ok, now in PEE mode

  drop					\ R: -- prefetch
  r> _FMC fmcPFAPR + !			\ re-enable prefetch
;
align
here ' (setClocks) - equ /SetClocks	\ length of code to copy to RAM
uhere equ rSetClocks	\ -- addr
\ *G The start of the RAM buffer used for the clock set up code.
\ ** By default, the buffer space is reserved. If you know that
\ ** *\fo{setClocks} will only be run once at start up, you may be
\ ** able to get away with not reserving the space!
  /SetClocks uallot	\ reserve RAM space

: setClocks	\ --
\ *G Initialise the clocks and peripheral dividers. All GPIO port
\ ** clocks are enabled.
  ['] (setClocks) rSetClocks /SetClocks cmove
  rSetClocks execute
  bit16 _SIM simSOPT2 + or!		\ use PLL, not FLL for peripherals
  bit19 _SIM simSOPT1 + or!		\ use RTC clock as 32kHz clock
  $3E00 _SIM simSCGC5 + or!		\ enable clocks to all PORTs
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

: disDog	\ -- ; no watchdog
\ *G Disable the watchdog at start up.
  _WDOG
  $C520 over wdgUNLOCK + w!
  $D928 over wdgUNLOCK + w!
  $D2 swap wdgSTCTRLH + w!		\ unlock value per C header file
;

((
: ledDelay	\ --
  20000000 begin  1- dup 0= until  drop
;

: +led1		\ -- ; turn on LED1, pin -> 0
  bit11 _PTA ptPCOR + !			\ PA.11 -> 0
;
: -led1		\ -- ; turn off LED1, pin -> 1
  bit11 _PTA ptPSOR + !			\ PA.11 -> 1
;

: initLed1	\ -- ; initialise LED E1 on PA.11
  $64 _PORTA portPCR11 + or!		\ set SRE, ODE, DSE
  $0100 _PORTA portPCR11 + or!		\ MUX(10:8) := 1 = GPIO
  bit11 _PTA ptPDDR + or!		\ DDR=1=o/p
;

: +led2		\ -- ; turn on LED E2, pin -> 0
  bit28 _PTA ptPCOR + !			\ PA.11 -> 0
;
: -led2		\ -- ; turn off LED E2, pin -> 1
  bit28 _PTA ptPSOR + !			\ PA.11 -> 1
;

: initLed2	\ -- ; initialise LED E2 on PA.11
  bit9 _SIM simSCGC5 + or!		\ enable clock to PORTA
  $64 _PORTA portPCR28 + or!		\ set SRE, ODE, DSE
  $0100 _PORTA portPCR28 + or!		\ MUX(10:8) := 1 = GPIO
  bit28 _PTA ptPDDR + or!		\ DDR=1=o/p
;

: initLEDs	\ --
\ Initialise LEDs for testing, P0.4 and P1.25 default to GPIO.
  initLed1 initLed2
  +led1 +led2
;

: flashLEDs	\ --
  initLEDs  ( init-ser3 )
  begin
    +led1 -led2 ledDelay  ( [char] A seremit3 )
    -led1 +led2 ledDelay  ( [char] Z seremit3 )
  again
;
))

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    disDog setClocks setNVIC
\    flashLEDs
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

