\ FlashFTFA.fth - Kinetis FTFA module Flash programming

((
Copyright (c) 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140627 MPE001 First release.
))

only forth definitions
decimal

\ ===========
\ *! filename
\ *T Chapter Title
\ ===========

\ Flash hardware algorithm operation commands
#define FTFA_VERIFY_SECTION             0x01
#define FTFA_PROGRAM_CHECK              0x02
#define FTFA_READ_RESOURCE              0x03
#define FTFA_PROGRAM_LONGWORD           0x06
#define FTFA_ERASE_SECTOR               0x09
#define FTFA_VERIFY_ALL_BLOCK           0x40
#define FTFA_READ_ONCE                  0x41
#define FTFA_PROGRAM_ONCE               0x43
#define FTFA_ERASE_ALL_BLOCK            0x44
#define FTFA_SECURITY_BY_PASS           0x45

\ Return Code Definition for FTFA SSD
#define FTFA_OK                          0x0000
#define FTFA_ERR_SIZE                    0x0001
#define FTFA_ERR_RANGE                   0x0002
#define FTFA_ERR_ACCERR                  0x0004
#define FTFA_ERR_PVIOL                   0x0008
#define FTFA_ERR_MGSTAT0                 0x0010
#define FTFA_ERR_CHANGEPROT              0x0020
#define FTFA_ERR_EEESIZE                 0x0040
#define FTFA_ERR_EFLASHSIZE              0x0080
#define FTFA_ERR_ADDR                    0x0100
#define FTFA_ERR_NOEEE                   0x0200
#define FTFA_ERR_EFLASHONLY              0x0400
#define FTFA_ERR_DFLASHONLY              0x0800
#define FTFA_ERR_RDCOLERR                0x1000
#define FTFA_ERR_RAMRDY                  0x2000

internal
align IsLeaf
: (doFlashCmd)	\ cob03 cob47 cob8B -- fstat
\ *G Perform a Flash Command and return the status byte. If bit 7
\ ** of the status is clear, a timeout occurred.
\ ** This routine is run from RAM.
  swap rot _FTFA			\ -- cob8B cob47 cob03 ^FTFA
  begin  dup ftfaFSTAT + c@ $80 and  until	\ wait command complete
  $070 over ftfAFSTAT + c!		\ clear previous errors
  over #24 rshift over ftfaFCCOB0 + c!	\ command byte
  over #16 rshift  over ftfaFCCOB1 + c!
  over 8 rshift over ftfaFCCOB2 + c!
  tuck ftfaFCCOB3 + c!			\ -- cob8B cob47 ^ftfl
  over #24 rshift over ftfaFCCOB4 + c!
  over #16 rshift  over ftfaFCCOB5 + c!
  over 8 rshift over ftfaFCCOB6 + c!
  tuck ftfaFCCOB7 + c!			\ -- cob8B ^ftfl
  over #24 rshift over ftfaFCCOB8 + c!
  over #16 rshift  over ftfaFCCOB9 + c!
  over 8 rshift over ftfaFCCOBA + c!
  tuck ftfaFCCOBB + c!			\ -- ^ftfl
  $080 over ftfaFSTAT + c!		\ clear CCIF bit by writing 1
  10 begin 1- dup 0= until drop
  system-speed begin			\ -- ^ftfl count
\  CLD_clockspeed @ begin		\ -- ^ftfl count
    1- dup				\ check for timeout
   while
    over ftfaFSTAT + c@ $080 and
  until then
  drop  ftfaFSTAT + c@			\ return status
;
align
here ' (doFlashCmd) - equ /doFlashCmd	\ length of code to copy to RAM
external
/doFlashCmd buffer: rdoFlashCmd	\ -- addr
\ *G The RAM buffer used for the Flash programming code.

: doFlashCmd	\ cob03 cob47 cob8B -- fstat
\ *G Perform a Flash Command and return the status byte. If bit 7
\ ** of the status is clear, a timeout occurred.
  ['] (doFlashCmd) rdoFlashCmd /doFlashCmd cmove
  [i  rdoFlashCmd execute  i]
;

: flPageErase	\ addr -- stat
\ *G Erase the sector containing the address.
  $0900:0000 or  0 0 doFlashCmd
;

: prog32	\ x addr -- stat
\ *G Program a 32 bit unit at a 32-bit aligned address.
  $0600:0000 or  swap  0 doFlashCmd
;

: inFlash?	\ addr -- flag
\ +G Returns true if the address is in the Flash area.
  FLASHSTART FLASHEND within		\ Flash range
;

external

: c!f		\ b addr --
\ *G Program any address, including Flash, with an 8 bit value.
  dup InFlash? abort" No 8 bit Flash write"
  c!
;

: w!f		\ w addr --
\ *G Program any address, including Flash, with a 16 bit value.
  dup InFlash? abort" No 16 bit Flash write"
  w!
;

: !f		\ x addr --
\ *G Program any address, including Flash, with a 32 bit value.
  dup InFlash? if
    dup @ $FFFF:FFFF <> abort" Flash not erased"
    prog32 drop
  else
    !
  endif
;

: FlErase	\ addr len --
\ *G Erase the Flash sectors in the given range.
  bounds ?do
    i inFlash? if
      i flPageErase drop  /FlashPage
    else
      -1 i !  4
    endif
  +loop
;

: FlProg	\ src dest len --
\ *G Program the Flash at *\i{dest} from memory at *\i{src}.
  bounds ?do
    dup @ i prog32 drop  4+
  4 +loop
  drop
;

((
\ internal
: >Flash	\ src dest len --
\ +G Save a memory block to Flash. The address *\i{src}must be
\ +* even. The address *\i{dest} must be the start address of
\ +* empty (erased) Flash. The kernel CRC is added at the end.
  3dup s= if
    2dup + @ crcslot @ = if
      3drop exit
    endif
  endif
  over @ $FFFF:FFFF <> if
    2dup FlErase
  endif
  2dup + crcslot @ swap !f		\ save kernel checksum
  bounds ?do
    dup @ i !f  cell +
  cell +loop
  drop
;
external
))


$0001:0000 constant ta


\ ======
\ *> ###
\ ======

decimal
