\ KL25Blinker.fth - Cortex startup code and LED flasher
\ for FRDM-KL25Z board

((
Copyright (c) 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20111109 MPE001 First release.
))

only forth definitions  decimal

\ ===============
\ *! kl25blinker
\ *T Freescale FRDM-KL25Z Blinky
\ ===============
\ *P The file *\i{Cortex/Hardware/KinetisKL20/KL25Blinker.fth} contains
\ ** the start up and LED flasher code for Kinetis KL25 devices.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the ARMv6-M
\ ** Architecture Reference Manual. If placed at 0 or the start of
\ ** Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick

$0400 ExcVecs + org			\ normally at $0000:0400
\ Install the 16 byte Flash Configuration field
l: FlashCfgField
  $FF c,  $FF c,  $FF c,  $FF c,	\ 8 byte Backdoor key
  $FF c,  $FF c,  $FF c,  $FF c,	\
  $FF c,  $FF c,  $FF c,  $FF c,	\ FPROT0, FPROT1, FPROT2, FPROT3
  $FE c,  $FF c,  $FF c,  $FF c,	\ FSEC, FOPT, FEPROT, FDPROT

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. The
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

internal
((
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
;
))

: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
  $0A0 _OSC oscCR + c!			\ ERCLKEN=1,EREFSTEN=1,SC2P=0,SC4P=0,SC8P=0,SC16P=0

  _MCG					\ -- ^mcg

  \ Switch to FBE Mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=0, C2[LP]=0
  $01C over mcgC2 + c!			\ MCG_C2: RANGE=1,HGO=1,EREFS0=1,LP=0,IRCS=0
  					\ osc range=0, 32-40kHz; range=1, 3-8 MHz; range=2, 8-32 Mhz
  %10000000 /FRDIV or over mcgC1 + c!	\ MCG_C1: CLKS=2,FRDIV=3/5,IREFS=0,IRCLKEN=0,IREFSTEN=0
  $0E0 over mcgC4 + bbic!		\ MCG_C4: DMX32=0,DRST_DRS=0
  $020 /PRDIV + over mcgC5 + c!		\ MCG_C5: PLLCLKEN=0,PLLSTEN=1,PRDIV=x
  bit6 over mcgC5 + bor!		\ MCG_C5: PLLCLKEN=1, enable PLL
  $000 /VDIV + over mcgC6 + c!		\ MCG_C6: LOLIE=0,PLLS=0,CME=0,VDIV=y
  begin  dup mcgS + c@ bit1 and  until	\ OSCINIT0: Check that the oscillator is running
  begin  dup mcgS + c@ bit4 and 0=  until \ IREFST: Check that the source of the FLL reference clock is the external reference clock.
  begin  dup mcgS + c@ $0C and $08 = until \ Wait until external reference clock is selected as MCG output */
\ ok
  \           FBE mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=0, C2[LP]=0
  \ Switch to PBE mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=1, C2[LP]=0
  bit6 over mcgC5 + bbic!		\ clear PLLCLKEN - why?
  bit6 over mcgC6 + bor!		\ set PLLS
  begin  dup mcgS + c@ $0C and $08 = until \ Wait until external reference clock is selected as MCG output
  \           PBE mode: C1[CLKS]=10, C1[IREFS]=0, C6[PLLS]=1, C2[LP]=0
  \ Switch to PEE mode: C1[CLKS]=00, C1[IREFS]=0, C6[PLLS]=1, C2[LP]=0
  $0C0 over mcgC1 + bbic!		\ clear CLKS bits
  begin  dup mcgS + c@ $0C and $0C = until \ Wait until PLL output is selected as MCG output
  begin  dup mcgS + c@ bit6 and  until	\ Wait for LOCK
\ ok, now in PEE mode

  drop					\ --

  $0001:0000 _SIM simCLKDIV1 + !	\ Force final dividers
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
;

: enableGPIOs	\ --
\ *G Configuring any pin or alternate function requires the
\ ** GPIOs to be clocked, so we enable them all at startup.
  $3E00 _SIM simSCGC5 + or!		\ enable clocks to all PORTs
;


IsLeaf : ledDelay	\ --
  5000000 begin  1- dup 0= until  drop
;


\ ====
\ LEDs
\ ====
\ Red LED1 on PTB.18
\ Green LED2 on PTB.19

: +led1		\ -- ; turn on LED1, pin -> 0
  bit18 _PTB ptPCOR + !			\ PTB.18 -> 0
;
: -led1		\ -- ; turn off LED1, pin -> 1
  bit18 _PTB ptPSOR + !			\ PTB.18 -> 1
;

: initLed1	\ -- ; initialise LED E1 on PA.11
\  $3E00 _SIM simSCGC5 + or!		\ enable clocks to PORTA..E
  $44 _PORTB portPCR18 + or!		\ set DSE, SRE
  $0100 _PORTB portPCR18 + or!		\ MUX(10:8) := 1 = GPIO
  bit18 _PTB ptPDDR + or!		\ DDR=1=o/p
;

: +led2		\ -- ; turn on LED2, pin -> 0
  bit19 _PTB ptPCOR + !			\ PTB.19 -> 0
;
: -led2		\ -- ; turn off LED2, pin -> 1
  bit19 _PTB ptPSOR + !			\ PTB.19 -> 1
;

: initLed2	\ -- ; initialise LED2 on PTB.19
\  $3E00 _SIM simSCGC5 + or!		\ enable clocks to PORTA..E
  $44 _PORTB portPCR19 + or!		\ set DSE, SRE
  $0100 _PORTB portPCR19 + or!		\ MUX(10:8) := 1 = GPIO
  bit19 _PTB ptPDDR + or!		\ DDR=1=o/p
;

: initLEDs	\ --
\ Initialise LEDs for testing, P0.4 and P1.25 default to GPIO.
  initLed1 initLed2
  +led1 +led2
;


\ =====
\ UART0
\ =====

\ Baud = systemspeed / 2 ( oversample * divisor )
\ Divisor = systemspeed / 2 ( oversample * baud )
\         = systemspeed / ( 32 * baud )

((
system-speed 32 / console0-speed / equ U0divisor
U0divisor $1FFF and
dup 8 rshift equ /U0BDH
     $FF and equ /U0BDL

: setMask	\ value mask addr -- ; cell operation
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;

: initUART0	\ --
\ *G Initialise UART0.
  $0401:0000 $0C01:0000 _SIM simSOPT2 + setMask	\ UART0SRC = 01
  bit10 _SIM simSCGC4 + or!		\ enable UART clock
  _PORTA
  $0200 over portPCR1 + !		\ Alt 2, no ints
  $0200 over portPCR2 + !		\ Alt 2, no ints
  drop
  _UART0
  0 over UnC2 + c!			\ disable
  0 over UnC1 + c!			\ 8 bit, no parity
  0 over UnC3 + c!			\ no Tx invert, no ints
  0 over UnS2 + c!			\ no Rx invert
  #15 over U0C4 + c!			\ oversample by 16
  /U0BDH over UnBDH + c!		\ baud rate divisor
  /U0BDL over UnBDL + c!
  $0C over UnC2 + c!			\ Enable Tx, Rx
  drop
;

: serEmit0	\ char --
  _UART0 begin				\ -- char uart
    dup UnS1 + c@ bit7 and
  until
  UnD + c!
;

: serKey?0	\ -- flag
  _UART0 UnS1 + c@ bit5 and 0<>
;

: serKey0	\ -- char
  _UART0
  $0F over UnS1 + c!			\ clear error flags
  begin					\ -- uart
    dup UnS1 + c@ bit5 and
  until
  UnD + c@
;
))


\ ===========
\ Application
\ ===========

: AppLoop	\ --
  initLEDs ( initUART0 ) ledDelay
  begin
\    $55 serEmit0
    +led1 -led2 ledDelay
\    $56 serEmit0
    -led1 +led2 ledDelay
  again
;

: disDog	\ -- ; no watchdog
  0 _SIM simCOPC + !
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
\ start of Forth setup
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
\ end of Forth setup
    disDog enableGPIOs
    setClocks setNVIC
    AppLoop
  again
;
external


INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ======
\ *> ###
\ ======

decimal

