\ KL25Blinky.ctl - Flash an LED on the FRDM-KL25Z bosrd

((
Copyright (c) 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140529 MPE002 Converted to FRDM-KL25Z board
20111103 MPE001 First release.
))


\ ============
\ *! k25blinky
\ *T FRDM-KL25Z Blinky control file
\ ============

only forth definitions  decimal

\ ******************************
\ Define the default directories
\ ******************************]
\ Make a copy of the MPE macros and adjust them for your directory
\ layout. If you have several developers with different layouts,
\ build a constant and a macro set for each developer. Only one
\ constant should be set non-zero.

1 constant MPE?	\ true if built at MPE

MPE? [if]	\ if built on an MPE development box

c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
mc" %ExampleDir%/FATfiler"		\ where FAT file system lives
                setMacro FatFiler
mc" %ExampleDir%/USB"
                setmacro UsbDir		\ where USB code lives
c" ."    	setmacro AppDir		\ where application code lives
[defined] Target_386_Windows [if]
c" c:/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]
[defined] Target_386_Linux [if]
c" ~/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  -LongCalls			\ do not permit standalone Forth to handle
  				\ calls outside 25 bit range.
  Hex-S19			\ produce S-record
  setBinExt .bin		\ binary files have .bin extension
  +SaveCdataOnly		\ no data area image files
  -const-visible		\ CONST = EQU

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ==================
\ *N KL25 definitions
\ ==================
\ *P On the MPE Tower board the CPU is a PK60N512VMD or MK60N512VMD100.

#128 kb equ /MainFlash	\ -- u
\ *G Define the size of the internal Flash. This is used
\ ** to control the use of  the Flash sector table.
1 kb equ /FlashPage	\ -- u
\ *G Size of a page in Flash.
#16 kb equ /MainRAM	\ -- u
\ *G Define the total size in RAM of the internal RAM blocks
\ ** based around address $2000:0000. Put one quarter below, three
\ ** quarters above.

#64 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

#8 MHz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
#4 MHz equ irc-speed		\ -- hz
\ *G Internal RC oscillator clock rate in HZ.
#32768 equ rtc-speed		\ -- hz
\ *G External RTC oscillator clock rate in HZ.

#48 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
#24 MHz equ bus-speed		\ -- hz
\ *G Requested bus clock speed in HZ. Max 50 MHz.
#24 MHz equ flex-speed		\ -- hz
\ *G Requested FlexBus clock speed in HZ. Max 50 MHz.
#24 MHz equ flash-speed		\ -- hz
\ *G Requested Flash clock speed in HZ. Max 25 MHz.
system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.
#48 MHz equ usb-speed		\ -- hz
\ *G Requested clock speed to USB peripheral.

\ *P The PLL is used to generate a 48 MHz system clock.
4 1 - equ /PRDIV	\ -- u
\ *G PLL input divider = 4, PLL freq = 2 MHz. This value is set
\ ** in MCG_C5, field PRDIV0 (bits 4:0).
24 24 - equ /VDIV	\ -- u
\ *G PLL multiplier = 24, output freq = 48 MHz. This value is set
\ ** in MCG_C6, field VDIV0 (bits 4:0).

8 5 - 3 lshift equ /FRDIV
\ *G The FLL can be used for some clocks. The FLL base clock is
\ ** the crystal divided by 2^n for RANGE=0 or 2^(n+5) for RANGE<>0.
\ ** The divider output should be in the range 31.25 kHz to 39.06 kHz
\ ** if the main clock is derived from the FLL. *\fo{/FRDIV} is
\ ** in bits 5:3 of the MCG_C1 register.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0000:0000.
\ *P SRAM is based around $2000:0000.

\ $2000:0000 equ _SRAM
\ /MainRAM 2/ equ /SRAMblock

  $0000:0000 $0001:FFFF cdata section KL25Blinky	\ code section in boot Flash
  $1FFF:F000 $1FFF:FFFF idata section PROGd	\ 4 kb IDATA
  $2000:0000 $2000:0FFF udata section PROGu	\ 12 kb UDATA

interpreter
: prog   KL25Blinky  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$00F0 equ UP-SIZE		\ size of each task's user area
$00F0 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#9600 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 0..2 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM0def		\ Cortex-M4 generic equates and SFRs
  include %CpuDir%/sfrKL25Z4		\ K25 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

\ PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %AppDir%/KL25Blinker		\ startup and Blinky

\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
((
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader
))

afterwards  fcopy KL25Blinky.s19 KL25Blinky.srec
afterwards  fcopy KL25Blinky.srec F:\KL25Blinky.srec

decimal

FINIS


\ ======
\ *> ###
\ ======

