\ Reprogram Kinetis KL25 over serial line.

((
Copyright (c) 2003-2006, 2008, 2010, 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140629 SFP005 COnverted for Kinetix KL25
20100125 SFP004 Converted for LPC17xx
20081008 SFP003 Converted for LPC23xx/24xx/28xx
20060818 SFP002 Added UART selection.
20040809 SFP001 Added equate FULLIAP?
20030728 SFP000 First cut
))

\ =============
\ *! reprogkl25
\ *T Reprogramming the Kinetis K6L25 serially
\ =============
\ *S Introduction
\ *P *\i{ReprogKL25.ctl} is the control file for the serial loader
\ ** which reprograms the on-chip Flash memory.
\ ** Because the on-chip Flash cannot be accessed during
\ ** programming, and because the initial loader code may
\ ** itself be replaced, the Flash programming code is copied
\ ** into RAM for execution. The only exit from the code is a
\ ** reset of the CPU to execute the newly Flashed program. All
\ ** interrupts are disabled during reprogramming, and so
\ ** polled serial drivers are used.

\ *P Access to the reprogramming software is defined in
\ ** *\i{ReFlash.fth} for the main system code and in
\ ** this control file. The two files must match.

\ *P Return is by rebooting the system. The contents of the
\ ** internal RAM and Flash are destroyed, therefore any data
\ ** that must be preserved should be saved externally to the
\ ** chip.


only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

c" ../../.."	setmacro CpuDir
c" ../../../.."	setmacro CommonDir
c" ."		setmacro AppDir
c" .."		setmacro HwDir


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to show output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ words start 32bit aligned
  Cortex-M0			\ Cortex-M processor type
  no-heads			\ do not generate heads
  no inlining			\ saves some space
  -ram-table			\ saves some space
  +LeafCalls			\ saves some space
  +SaveCdataOnly		\ no data area image files

0         equ false
false not equ true


\ *************
\ *S Memory map
\ *************

\ *[
$2000:0000 $2000:0BFF cdata section ReprogKL25	\ 3 k bytes
$1FFF:FC00 $1FFF:FC7F udata section PROGd	\ 128 bytes
$1FFF:FC80 $1FFF:FCFF udata section PROGu	\ 128 bytes
$1FFF:FD00 $1FFF:FFFF udata section Xbuff	\ 768 byte Xmodem buffer
  sec-base equ Xbase			\ base address of buffer
  sec-len equ Xlen			\ length of buffer
  Xbase Xlen + equ Xtop			\ top of buffer
interpreter
: prog  ReprogKL25  ;
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on
\ *]


\ ********************
\ *S Items of interest
\ ********************
\ *P The stacks have already been set up by the calling program.
\ ** The stacks should not be in the last 4kb of the lower half
\ ** of the Kinetis SRAM, otherwise they will overlap the program
\ ** copied into RAM.

\ *[
include %CpuDir%\CM0Def		\ Cortex CPU equates
include %CpuDir%\sfrKL25Z4		\ Special function registers
\ *]

l: CLD1			\ -- addr
\ *G Filled in later with the xt of *\fo{ReprogFlash}. This
\ ** label marks the start of the RAM code area.
  0 ,
l: CLD_CopyFlash	\ -- addr
\ *G Filled in later with the xt of *\fo{CopyFlash}
  0 ,
l: CLD_UART		\ -- addr
\ *G Filled in later with the UART base address to use.
\ ** The default is UART0.
  _UART0 ,
l: CLD_clockspeed	\ -- addr
\ *G Contains the clock speed to use. Filled in by the calling
\ ** application. Default is 48 MHz.
  48 Mhz ,
l: CLD_/FlUsed		\ -- addr
\ *G Holds the length of Flash for the reprogram operation.
\ ** Flash is assumed to start at $0000:0000. Default is 126 kb.
  126 kb ,
l: CLD_/FlSector	\ -- addr
\ *G Holds the size of a Flash sector. Default is 1 kb.
  1 kb ,

compiler
: prog-speed	\ -- hz
\ *G A macro to fetch the system clock speed from the entry table.
  CLD_clockspeed @  ;
target

\ *[
include %AppDir%\primitives		\ Forth primitives from CODEARM.FTH
include %AppDir%\MinSerKL25p		\ polled serial line driver
include %AppDir%\delays			\ software delays
include %CpuDir%\Drivers\rebootCM0	\ reboot somehow
\ *]

synonym ser-emit emit
\ *G A synonym required by the Xmodem code.
synonym ser-key? key?
\ *G A synonym required by the Xmodem code.
synonym ser-key  key
\ *G A synonym required by the Xmodem code.

\ *[
include %AppDir%\MinXmodemRx		\ Xmodem receiver
include %AppDir%\FlashFTFAminRAM	\ Flash primitives
include %AppDir%\ReprogApp		\ Application
\ *]


\ *******
\ The end
\ *******

make-turnkey ReprogFlash		\ start up action
\ *G Define the action of the reprogramming code.
' CopyFlash CLD_CopyFlash !
\ *G Define the action of the Flash copy code.

decimal

FINIS


\ ******
\ *> ###
\ ******
