\ EKK-LM3s9b2.ctl - standalone Forth for Luminary EKK LM3S9B2 board

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20100317 MPE001 First release.
))


\ =============
\ *! ekk-lm3s9b2
\ =============

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives
c" ~/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  T2-mode			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
\  #8 LoopAlignment		\ faster but bigger
\ For ISP load
   Hex-I32			\ produce Intel Hex-32 obj format

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N System definitions
\ =====================

$0000:0000 equ Flashbase	\ -- addr
\ *G Base address of the Flash at run-time.
1 kb equ /FlashSector		\ -- len
\ *G Size of a Flash sector, default is 1kb
#256 equ #FlashSectors		\ -- u
\ *G Number of Flash sectors. By default, the *\fo{REFLASH}
\ ** utility does not erase the last one, which can be used
\ ** for configuration information.
1 equ #ReservedSectors		\ -- u
\ *G Number of reserved sectors at top of Flash.

#128 cells equ /ExcVecs		\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

#16000000 equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
\ ** If you change this, you will have to change the set up of
\ ** *\fo{ClockParam} in *\i{Cortex\Hardware\startLM3s9b9x.fth}.
#50000000 equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
\ ** If you change this, you will have to change the set up of
\ ** *\fo{ClockParam} in *\i{Cortex\Hardware\startLM3s9b9x.fth}.
\ ** *\b{N.B.} the *\fo{REFLASH} utility does not work correctly
\ ** on some devices above 50 MHz. See the errata sheet for your
\ ** device and check the chip's markings.

#32768 equ rtc-speed		\ -- hz
\ *G External RTC oscillator clock rate in HZ.

#48000000 equ usb-speed		\ -- hz
\ *G Requested clock speed to USB peripheral.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0000:0000.

  $0000:0000 $0003:FBFF cdata section EKK-LM3s9b92  \ code section in boot Flash
    $0003:FC00 equ CfgFlash
  $2000:0000 $2000:1FFF idata section PROGd	\ 8k IDATA
  $2000:2000 $2001:7FFF udata section PROGu	\ rest for UDATA

interpreter
: prog EKK-LM3s9b92  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
4 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 0..3 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $8000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.
\ ** N.B. Set heap size too! 32 kb is ok.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CortexDef		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrLM3s9b92		\ special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startLM3s9bx		\ startup code
  include %CpuDir%/Drivers/RomCallsLM	\ Stellaris ROM calls
1 equ FastCmove?
  include %CpuDir%/CodeCortex		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootLM3s9b9x \ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault handlers - needs serial
  include %CommonDir%/CQueues		\ character queues
  include %CpuDir%/Drivers/serLM3s9bxqi	\ RX/TX queued interrupt driver
\  include %CpuDir%/Drivers/serLM3s9bxi	\ RX/TX interrupt Rx driver
\  include %CpuDir%/Drivers/serLM3s9bxp	\ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM3		\ local variables

tasking? [if]
  include %CpuDir%/MultiCortex		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickLM3s9b9x
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickLM3s9b9x
' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile ekk-lm3s9b92.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M3"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  include %HwDir%/IapLM			\ IAP code
  include %HwDir%/FlashProg		\ higher level Flash routines
  include %HwDir%/ReProg/ReFlash	\ Reflash system

((
  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%/XmodemTxRx	\ Xmodem driver
  include %CommonDir%/RomForth/iodef	\ link i/o
  include %CommonDir%/RomForth/textfile	\ AIDE textfile support
))

PowerNet? [if]
  1 equ CPU=Cortex	\ to compile Cortex specific code
  0 equ FilePages?			\ no file system of HTTP page storage
  include %AppDir%/PNconfig		\ PowerNet configuration
  include %IpStack%/NetCode		\ network order stuff
  include %AppDir%/WebConfig		\ configuration tools
  include %CpuDir%/Drivers/EtherLMrom	\ Ethernet driver
  include %IpStack%/PowerNet.bld	\ PowerNet build
\  include %IpStack%/Examples/WebPost	\ application update
[else]
((
1 equ XC?		\ cross compiled
1 equ CPU=Cortex	\ to get Cortex specific code
1 equ EthDiags?
1 equ Sniff?
1 equ Pingit?
  include %ExampleDir%/NetCode		\ network order stuff
  include %CpuDir%/Drivers/EtherLMrom	\ Ethernet driver
))
[then]


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\benchmarks\xbench32	\ performance tests

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
ExcVecs 0				\ -- addr csum
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $14 + @ +
  over $18 + @ +
  negate
swap $1C + !

\ Set the reset address for ELF
  ' StartCortex setBoot


decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======

