\ FlashProg.fth - Program LM39Bx Flash

((
Copyright (c) 2004, 2008, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
At end of programming, check for a partial block to program.

Change history
==============
20100323 MPE006 Updated for LM3D9B9x
20100125 MPE005 Updated for LPC17xx
20081016 MPE004 Used intermediate buffer to avoid IAP problems
		with some devices.
20081008 MPE003 Converted for LPC23xx/24xx/28xx.
20050627 MPE002 Tables separated from FlashProg.fth
20040511 MPE001 First release
))

only forth definitions  decimal


\ ===========
\ *! flashlpc
\ *T LM3S9B9x Flash tools
\ ===========
\ *P These tools are provided for applications which reserve
\ ** part of the Flash for data. This code uses the IAP routines
\ ** in *\i{IapLM.fth}.

\ *P N.B. An erase causes the whole of the sector containing that
\ ** address to be erased.


\ *******************
\ *S Flash primitives
\ *******************
\ *P Although some of these routines are not the most efficient
\ ** for the whole series, they are designed to be easily expanded
\ ** for future LM3S9B9x parts with as yet unknown sector sizes.

\ *P Sector tables contain the number of sectors and starting offset
\ ** of each sector, plus a dummy start address which enables
\ ** the size of the last sector to be calculated.
\ ** The boot block sector is *\b{not} included in the table.
\ ** The sector table is given by *\fo{SecTab} which is defined in
\ ** *\i{FlashTables.fth}.


\ ***************
\ *S Flash driver
\ ***************

cell buffer: FlDest	\ -- addr
\ *G Holds next destination address.

cell buffer: #PrgErrs	\ -- addr
\ *G Holds error count.

: (EraseFlash)	\ dest dlen --
\ *G Erase the flash for the given range. Note that complete
\ ** sectors in the range will be erased. If you want to write
\ ** partial sectors, check that they contain $FF in all bytes
\ ** before programming in order avoid having to erase them first.
  bounds ?do
    i FlErase1k if  #PrgErrs Incr  endif
  $0400 +loop
;

: (ProgFlash)	\ src dest len --
\ *G Write len bytes from memory at src to Flash at *\i{dest}.
\ ** Note that the Flash is assumed to be erased, and that
\ ** no verification is performed except when each byte is
\ ** programmed. *\i{Dest} is forced to a 128 byte boundary and
\ ** *\i{len} is rounded up to the next 128 byte unit. *\i{Src}
\ ** must be on a word boundary.
  rot -4 and -rot			\ force src
  swap #-128 and swap			\ force dest
  #511 + #-128 and			\ force len
  bounds ?do				\ -- src'
    dup i Flprog128
    if  #PrgErrs incr  endif
    #128 +
  #128 +loop
  drop
;

: .src/dest	\ src dest -- src dest
\ *G Display the addresses and contents of the source
\ ** and destination.
  ."  source: " over .lword  over c@ ." (" .byte ." )"
  ."  dest: " dup .lword  dup c@ ." (" .byte ." )"
;

: (VerifyFlash)	\ src dest len --
\ *G Verify len bytes from memory at src to Flash at dest.
  3dup s= if  3drop exit  endif
  cr ."  Verify failed"
  0 ?do
    over c@ over c@ <> if
      cr 2 spaces .src/dest
      key? ?leave
    then
    i $FFF and 0= if  ." ."  endif
    1 + swap  1 + swap
  1 +loop
  2drop
;

: ProgramFlash	\ src dest len --
\ *G Using the RAM memory buffer at *\i{src}, program the Flash
\ ** at *\i{dest} with *\i{len} bytes. The relevant Flash sectors
\ ** are erased, the Flash is programmed, and the result verified.
  cr ." Erasing " 2dup (EraseFlash)
  0 #PrgErrs !
  cr ." Programming "  3dup (ProgFlash)
  space #PrgErrs ? ." errors"
  cr ." Verifying "  (VerifyFlash)
;


\ ======
\ *> ###
\ ======

