\ ReprogApp.fth - reprogram LPC2xxx Flash

((
Copyright (c) 2004, 2008, 2009, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
At end of download, check for a partial block to program.

Change max program size to depend on CPU.

Change history
==============
20100323 SFP006 LM3s9b9x conversion.
20100125 SFP005 LPC17xx conversion.
20090205 SFP004 LPC2388 does not erase last sector at $0007:D000
20081008 SFP003 Minor documentation changes.
20060701 SFP002 Added CopyFlash.
20050627 MPE002 Now uses sector tables.
20040212 MPE001 First release
))

only forth definitions  decimal


\ ****************
\ *S Flash handler
\ ****************

cell buffer: #PrgErrs	\ -- addr
\ *G Holds error count

: FlashTiming	\ --
  CLD_clockspeed @ #1000000 u/ 1-	\ MHz-1
  _SysCtl sysUSECRL + !			\ timing
;

: FlErase1k	\ addr -- ior
\ *G Erase a 1kb Flash sector.
  FlashTiming
  $-400 and _FMC tuck fmcFMA + !	\ base address
  $A442:0002 over fmcFMC + !		\ key + bit 1
  begin dup fmcFMC + @ bit1 and 0= until
  drop 0
;

: FlProg128	\ src dest -- ior
  FlashTiming
  $-80 and swap -3 and			\ dest/src alignment
  2dup  _FMC fmcFWBn +			\ -- dest src dest src fwbn
  #32 0 do
    over @ over !  4+ swap 4+ swap
  loop
  drop 2drop swap _FMC			\ -- src dest fmc
  2dup fmcFMA + !			\ dest address
  $A442:0001 over fmcFMC2 + !		\ command
  begin dup fmcFMC2 + @ bit0 and 0= until
  drop #128 s= 0=
;

: (EraseFlash)	\ addr len --
\ *G Erase the given range of Flash.
  bounds ?do
    i FlErase1k if  #PrgErrs incr  endif
  /FlashSector +loop
;

: EraseFlash	\ --
\ *G Erase the Flash from start to end.
  Flashbase				\ base address
  CLD_#FlSectors @ /FlashSector *	\ length
  (EraseFlash)
;

: toFlash	\ src dest --
\ *G Handle 128 bytes at src to go to dest in Flash.
  FlProg128 if  #PrgErrs incr  endif
;


\ *****************
\ *S Numeric output
\ *****************

: .nibble	\ n --
\ *G Convert a nibble to a hex ASCII digit and display it.
  $0F and  dup 9 >
  if  7 +  then
  $30 + emit
;

: .WORD		\ w --
\ *G Display w as a 4 digit unsigned hexadecimal number.
  dup #12 rshift .nibble
  dup 8 rshift .nibble
  dup 4 rshift .nibble
  .nibble
;

: .dword	\ x --
\ *G Display x as an 8 digit unsigned hexadecimal number.
  dup #16 rshift .word .word
;


\ ***************
\ *S Reprogrammer
\ ***************

l: m1
  ", Erasing "
l: m2
  ",  done "
l: m3
  ", Start Xmodem TX "
[defined] REBOOT [if]
l: m4
  ", Press R to reboot, other to reprogram "
[else]
l: m4
  ", Reset or press key to reprogram"
[then]
l: m5
  ", Errors"
l: m6
  ", No "

: $.	\ c-addr --
\ *G Display counted string.
  count type
;

: .PrgErrs
\ *G Display number of errors.
  cr  #PrgErrs @ .word  bl emit
  m5 $.
;

: initReprog	\ --
\ *G Get the setup data and calculate the requirements
  init-ser #50 ms
;

: ReprogFlash	\ --
\ *G Erase and program the Flash.
  begin
    initReprog
    #PrgErrs off
    cr m1 $.  #20 ms
    EraseFlash
    m2 $.  .PrgErrs  #PrgErrs off
    cr m3 $.  $01A emit			\ send message and trigger
    Flashbase dup $00400000 + Bin-Up drop
    .PrgErrs
    #100 ms
    cr m4 $.
[defined] REBOOT [if]
    key $DF and dup emit  [char] R =
  until
  #100 ms
  reboot
[else]
    key drop
  again
[then]
;

\ SFP002...
l: msg7
  ", Copy "

: (CopyFlash)	\ src dest len --
\ *G Copy one Flash area to another. The parameters are as for *\fo{CMOVE}.
  initReprog
  cr m1 $.  #20 ms			\ "Erase" message
  #PrgErrs off
  2dup (EraseFlash) .PrgErrs  #10 ms	\ -- src dest len
  #PrgErrs @
  if  3drop exit  endif
  cr msg7 $.				\ "Copy" message
  0 do					\ -- src dest
    2dup FlProg128
    if  #PrgErrs incr  endif
    swap #128 + swap #128 +
  #128 +loop
  2drop
  .PrgErrs  #20 ms			\ error count
;

: CopyFlash	\ src dest len --
\ *G Copy the flash. The parameters are as for *\fo{CMOVE}.
\ ** The process is repeated until there are no errors, and
\ ** the system is then rebooted.
  begin
    3dup (CopyFlash)  #PrgErrs @ 0=
  until
  #100 ms  reboot
;
\ ...SFP002


\ ======
\ *> ###
\ ======

