\ Primitives.fth - Forth primitives from CodeCortex.fth

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20100125 MPE001 Cortex first version.
))

only forth definitions

CODE S=		\ c-addr1 c-addr2 u -- flag
\ *G Compare two same-length strings/memory blocks, returning TRUE if
\ ** they are identical.
  ldmfd   psp ! { r0, r1 }		\ r0=adr2, r1=adr1
  mov .s  tos, tos
  b .eq   S=X                           \ TRUE as len=0
L: S=1
  ldrb    r2, [ r0 ], # 1
  ldrb    r3, [ r1 ], # 1
  cmp     r2, r3
  b .ne   S=X                           \ FALSE as mismatched chars
  sub .s  tos, tos, # 1
  b .ne   S=1                           \ if count=0 then TRUE
L: S=X
  ite .eq
    mvn     tos, # 0			\ TRUE
    mov     tos, # 0			\ FALSE
  bx      lr
END-CODE

code cmove	\ asrc adest len --
\ *G Copy *\i{len} bytes of memory forwards from *\i{asrc} to *\i{adest}.
  mov .s  r3, tos                	\ r3=len
  ldmfd   psp ! { r0, r1, tos }		\ r0=addr2, r1=addr1, r3=len, tos restored
  it .eq
    bx      link			\ return if len=0, nothing to do
  orr     r2, r1, r0			\ check alignment
  orr     r2, r2, r3			\ R2 := adr1|addr2|len
  and .s  r4, r2, # 3			\ will be zero if aligned
  b .ne   cmovx3			\ if not aligned
L: CMOVX1
  ldr     r2, [ r1 ], # 4
  sub .s  r3, r3, # 4			\ hoisted for ARM9
  str     r2, [ r0 ], # 4
  b .ne   CMOVX1
  bx      link
L: CMOVX3
  ldrb    r2, [ r1 ], # 1
  sub .s  r3, r3, # 1			\ hoisted for ARM9
  strb    r2, [ r0 ], # 1
  b .ne   CMOVX3
  bx      link
END-CODE

CODE (DO)	\ limit index --
\ *G The run time action of *\fo{DO} compiled on the target.
  ldr     r1, [ psp ], # 4		\ get limit
L: PDO
  ldr     r2, [ link, # -1 ]		\ get LEAVE address
  orr .s  r2, r2, # 1			\ force Thumb bit
  add .s  link, link, # 4		\ update link
  add .s  r1, r1, # $80000000
  sub .s  r0, tos, r1

  stmfd   rsp ! { r0, r1, r2 }		\ push LEAVE then limit, then index on ret. stack
  ldr     tos, [ psp ], # 4		\ update tos
  bx      lr
END-CODE

CODE (?DO) 	\ limit index --
\ *G The run time action of *\fo{?DO} compiled on the target.
  ldr     r1, [ psp ], # 4		\ get limit
  cmp     r1, tos			\ check not equal
  b .ne   pdo				\ take DO ?
  ldr     tos, [ psp ], # 4		\ update tos
  ldr     r0, [ link, # -1 ]		\ get leave destination
  orr .s  r0, r0, # 1			\ set Thumb bit
  bx      r0				\ and take leave branch
END-CODE

asmcode
here is-action-of constant
\ *G The runtime code for a CONSTANT.
  str     tos, [ psp, # -4 ] !		\ save TOS
  ldr     tos, [ link, # -1 ]		\ get const val/addr from afer CALL
  pop     { pc }
END-CODE

code di		\ --
\ *G Disable interruots.
  cps     .id .i
  bx      lr
end-code

code dfi	\ --
\ *G Disable fault exceptions.
  cps     .id .f
  bx      lr
end-code

CODE WITHIN 	\ n1|u1 n2|u2 n3|u3 -- flag
\ *G The ANS version of WITHIN?.
\ ** This word uses unsigned arithmetic, so that signed compares are
\ ** treated as existing on a number circle.
  ldmfd   psp ! { r0, r1 }              \ tos = n3, r0 = n2, r1 = n1
  sub .s  tos, tos, r0
  sub .s  r1, r1, r0
  cmp     r1, tos
  ite .lo
    mvn     tos, # 0
    mov     tos, # 0
  bx      lr
END-CODE
