\ LPC1114uf.ctl - MPE LPC1114 Umbilical Forth target control file

((
Copyright (c) 2004, 2008, 2011, 2013

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 80 631441
fax: +44 (0)23 80 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20131029 SFP003 Updated for Segger J-Link
20110629 SFP002 Converted for LPC1114
20081127 MPE001 Converted for v7 and Stamp compiler
))


\ =========
\ *! lpc1114ufjl
\ *T Umbilical Forth for LPC1114 User Flash
\ =========
\ *P The control file LPC1114uf.ctl generates an Umbilical
\ ** Forth for Philips LPC1114 systems. Programming is done
\ ** by a Segger J-Link. Umbilical Forth comms are also handled
\ ** through the J-Link.

only forth definitions decimal


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

include %CpuDir%/macros		\ compiler and assembler macros

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE                   \ turn compiler on

only forth definitions          \ starting search order
  no-heads			\ heads not needed for Umbilical Forth
  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if]
  +xrefs			\ Not Stamp/IRTC compilers
[then]
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ processor type
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 26 bit range.
  Hex-I32			\ also generate Intel Hex file


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N LPC type being used
\ ======================

#1114 equ LPCtype	\ -- n
\ *G Define the LPC type being used. This is used in the
\ ** CPU specific code such as the IAP access code to
\ ** select CPU specific data and routines.
#32 kb equ /LPCflash	\ -- u
\ *G Define the size of the internal Flash. This is used
\ ** to control the use of  the Flash sector table.
#48 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.


\ =====================
\ *N Speed selection
\ =====================

#12 Mhz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
#48 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.

#115200 equ console0-speed	\ Serial port speed in BPS.

20 equ console-port		\ Designate serial port for terminal (0/1).
\ *G Ports 0 and 1 are the on-chip UARTs. The internal/external USB device
\ ** is port 2, and bit-banged ports are defined from 10 onwards.
\ ** The PSUART for a Segger J-Link is port 20.
0 equ useUART0?	\ -- n
\ *G Compile UART0 code if non-zero.

#1 equ tick-ms			\ TIMEBASE tick in ms


\ ======================================
\ *N Stack, task and interrupt selection
\ ======================================

\ define stack and user area sizes for TASKs
$0F0 equ UP-SIZE		\ size of each task's user area
$0F0 equ SP-SIZE		\ size of each task's data stack
$0F0 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$30 equ max-depth		\ max depth for UF data stack before UF barfs


\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SWI nestings permitted
				\ 0 is ok if SWIs are unused


\ =============
\ *N Memory map
\ =============

  $0000:0000 $FFFF:FFFF udata section catchall
  $0000:0000 $0000:7FFF cdata section LPC1114ufjl	\ code section
  $1000:0200 $1000:0FFF idata section PROGd	\ 4k IDATA
  $1000:1000 $1000:1FFF udata section PROGu	\ 4k UDATA
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP
    $7FFFFFF0 equ IAPentry			\   Thumb code entry point

interpreter
: prog LPC1114ufjl ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED


\ =====================
\ *N Software selection
\ =====================

 1 equ umbilical?               \ true = Umbilical Forth, zero = standalone
   0 umbilical? and equ SPI?	\ true for SPI Umbilical, false for serial
   SPI? 0= equ Serial?		\ true for Serial Umbilical, false for SPI
 1 equ init-idata?		\ true if IDATA to be initialised

\ Kernel components
 0 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

interpreter
: AtCold	\ xt --
\ *G Most Umbilical Forth systems do not use the *\fo{ColdChain}
\ ** mechanism provided in *\i{Kernel62.fth} for standalone
\ ** Forths. To prevent compilation problems, define *\fo{AtCold}
\ ** as *\fo{DROP}. Compilers built from December 2008 onwards
\ ** will warn you if *\fo{AtCold} is used before being defined
\ ** in the target code.
  drop
;
target

\ *****************
\ default constants
\ *****************

8 equ BSOUT			  \ backspace character
cell equ cell			  \ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM0def		\ M0 generic equates
  include %CpuDir%/sfrLPC11xx  		\ Special function registers.
  include %HwDir%/startLPC11xx		\ startup code
  include %CommonDir%/messages		\ Umbilical messages
  include %CpuDir%/minM0M1		\ main code defs.
  include %CommonDir%/UFkernel		\ Forth high level kernel

  include %CpuDir%/IntCortex		\ interrupt handlers
\  include %CommonDir%/cqueues		\ character queues
\  include %CpuDir%/Drivers/serLPC11xxi	\ interrupt RX serial driver
\  include %CpuDir%/Drivers/serLPC11xxp	\ polled RX serial driver
  include %CpuDir%/Drivers/serPSUART	\ pseudo UART driver

umbilical? [if]
  Synonym wait-byte  psKey0		\ Say which XTL drivers to use
  Synonym send-byte  psEmit0
  Synonym Wait-Byte? psKey?0
  Synonym Init-XTL   Init-Ser
  include %CommonDir%/targend		\ message driver
[then]


\ *******************
\ *S Application code
\ *******************

: App		\ -- ; N.B. remove INIT-SER for interactive testing
[ init-idata? ] [if]  init-idata  [then]
[ umbilical? 0= ] [if]			\ if final application
  init-ser				\ initialise UART
  SerConsole opvec !			\ select I/O devices
  SerConsole ipvec !
[then]
  $0A base c!				\ default number conversion base
  cr cr ." MicroProcessor Engineering Umbilical Forth"
  begin
    cr  $0A 0
    do  i $030 + emit  loop		\ display numbers 0..9
    cr ." Press any key to continue " key drop
  again
;

umbilical? [if]
: runUmb	\ --
\ *G Starts Umbilical Forth with additional initialisation.
[ init-idata? ] [if]  init-idata  [then]
  run-umbilical
;
[then]


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CommonDir%/ufLib
end-libs

\ Force binary file to 512 byte unit.
cdata

\  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
\  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
ExcVecs 0				\ -- addr csum
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $14 + @ +
  over $18 + @ +
  negate
swap $1C + !


\ *******************************************************************
\ Define what the system will do at power up.
\ For the final application, use UMBILICAL? = 0
\ and TARGEND.FTH will not be included when the interactive Umbilical
\ Forth is no longer needed. RUN-UMBILICAL is defined in TARGEND.FTH.
\ *******************************************************************

decimal

umbilical? [if]
  make-turnkey runUmb			\ cold start to Umbilical Forth
\ Segger J-Link - Windows, OS X, Linux
  jlink LPC1114/302 1000 Project.jlink	\ device, speed, project file
  useSWD				\ use SWD rather than the default JTAG
  PSUART0 set-sjl-link			\ select Umbilical link through J-Link

\ Windows - real serial port
\  c" COM4:" console0-speed serial	\ define link driver
\  c" dtr=off rts=off" ser-control	\ define DTR and RTS
\ OS X - Keyspan HS19 USB adapter
\  c" /dev/cu.KeySerial1" console0-speed serial	\ define link driver
\  c" 1 dtr  1 rts" ser-control		\ define DTR and RTS

  umbilical-forth                       \ switch to interactive mode
[else]
  make-turnkey App			\ cold start to application

  jlink LPC1114/302 1000 Project.jlink	\ device, speed, project file
  useSWD				\ use SWD rather than the default JTAG
  afterwards  download-sections

  finis
[then]

FINIS					\ catch all in case ...


\ ======
\ *> ###
\ ======
