\ IAP11xx.fth - LPC11xx IAP routines

((
Copyright (c) 2004, 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20151128 MPE004 Reduced in size by removing unused code.
20110617 MPE003 LPC11xx conversion.
20100125 MPE002 LPC17xx conversion.
20040809 MPE001 Released for use with LPC2xxx bootloaders
                v1.52/1.63.
))

decimal

\ ======
\ *! iap11xx
\ *T NXP LPC11xx IAP routines
\ ======
\ *P The IAP is accessed by calling a Thumb routine at the
\ ** IAPentry address with the address of a command block in R0
\ ** and the address of a status/result block (RAM) in R1.
\ ** All the IAPxxx words return a 0 result on success. Note
\ ** also that all interrupts are disabled for the duration
\ ** of an IAP call, and therefore that ticker interrupts will
\ ** not be serviced. In particular, the sector erase command
\ ** may take 400ms, and a write of a 512 byte line may take 1ms.

5 cells buffer: IAPcmd	\ -- addr ; max 5 cells
\ *G Command input buffer for IAP routines.
4 cells buffer: IAPres	\ -- addr ; max 4 cells
\ *G Result output buffer from IAP routines.

code IAP	\ *cmd *res --
\ *G The primitive to call the IAP routines. Interrupts are
\ ** disabled for the duration of the IAP call.
\ save lr
  push    { lr }
\ Disable interrupts
  mrs     r2, PRIMASK			\ save and disable interrupts
  push    { r2 }
  cps     .id .i
\ save high regs
  push    { lr }
  mov     r0, r8
  mov     r1, r9
  mov     r2, r10
  mov     r3, r11
  mov     r4, r12
  push    { r0-r4 }			\ save state
\ Set up parameters and call the IAP Thumb routine
  ldmia   psp ! { r0 }			\ R0 = *cmd
  mov     r1, tos			\ R1 = *res
  push    { r6 }
  ldr     r3, ^IAP			\ get entry address
  blx     r3
  pop    { r6 }
  ldmia   psp ! { tos }			\ restore TOS
\ restore high regs
  pop     { r0-r4 }			\ restore state
  mov     r8, r0
  mov     r9, r1
  mov     r10, r2
  mov     r11, r3
  mov     r12, r4
  pop     { r0 }
  mov     lr, r0
\ restore interrupts
  pop     { r2 }
  msr     PRIMASK r2
\ restore lr/pc
  pop     { pc }
end-code

align l: ^IAP
  $1FFF:1FF1 ,				\ entry address, Thumb mode

: IAPprep	\ start end -- res
\ *G Prepare sectors for erase/write.
  IAPcmd
  #50 over !				\ -- start end addr ; cmd#
  rot over cell + !			\ -- end addr ; start
  tuck  2 cells + !			\ -- addr ; end
  IAPres IAP  IAPres @
;

: IAPcopy	\ Rsrc Fdest len -- res
\ *G Copy/program len bytes from Rsrc in RAM to Fdest in Flash.
  IAPcmd
  #51 over !				\ cmd
  tuck  3 cells + !			\ len
  tuck  cell + !			\ Fdest
  tuck  2 cells + !			\ Rsrc
  prog-speed #1000 u/ over 4 cells + !	\ -- addr ; cclk/1000
  IAPres IAP  IAPres @
;

: IAPerase	\ start end -- res
\ *G Erase the inclusive range of sectors.
  IAPcmd
  #52 over !				\ cmd
  tuck  2 cells + !			\ end
  tuck  cell + !			\ start
  prog-speed #1000 u/ over 3 cells + !	\ cclk/1000
  IAPres IAP  IAPres @
;

((
: IAPcheck	\ start end -- res
\ *G Blank check the inclusive range of sectors.
  IAPcmd
  #53 over !				\ cmd
  tuck  2 cells + !			\ end
  tuck  cell + !			\ start
  IAPres IAP  IAPres @
;

: IAPcompare	\ Rsrc Fdest len -- res
\ *G Compare len bytes from Rsrc in RAM to Fdest in Flash.
  IAPcmd
  #56 over !				\ cmd
  tuck  3 cells + !			\ len
  tuck  cell + !			\ Fdest
  tuck  2 cells + !			\ Rsrc
  IAPres IAP  IAPres @
;

FullIAP? [if]

: IAPbootver	\ -- bootver
\ *G Return the 16 bit boot code version. The high byte is the
\ ** major version and the low byte is the minor version.
  IAPcmd
  #55 over !
  IAPres IAP  IAPres cell + w@
;

: IAPpartno	\ -- part#
\ *G Return the NXP part number.
  IAPcmd
  #54 over !
  IAPres IAP  IAPres cell + @
;

: IAPserialno	\ -- addr
\ *G Return a pointer to the 128 bit (4 cells) NXP LPC1xxx serial
\ ** number.
  IAPcmd
  #58 over !
  IAPres IAP  IAPres
;

[then]
))

\ ======
\ *> ###
\ ======

decimal

