\ Primitives.fth - Forth primitives from CodeM0M1.fth

((
Copyright (c) 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20110628 MPE002 Cortex-M0 conversion.
20100125 MPE001 Cortex-M3 first version.
))

only forth definitions

((
code cmove	\ asrc adest len --
\ *G Copy *\i{len} bytes of memory forwards from *\i{asrc} to *\i{adest}.
  mov .s  r3, tos                	\ r3=len
  ldmia   psp ! { r0, r1, tos }		\ r0=addr2, r1=addr1, r3=len, tos restored
  eq, if,
    bx      link			\ return if len=0, nothing to do
  endif,
  mov .s  r2, r0
  orr .s  r2, r2, r1			\ check alignment
  orr .s  r2, r2, r3			\ R2 := adr1|addr2|len
  mov .s  r4, # 3
  and .s  r4, r4, r2,			\ will be zero if aligned
  b .ne   cmovx3			\ if not aligned
L: CMOVX1
  ldr     r2, [ r1 ]
  str     r2, [ r0 ]
  add .s  r1, r1, # 4
  add .s  r0, r0, # 4
  sub .s  r3, r3, # 4
  b .ne   CMOVX1
  bx      lr
L: CMOVX3
  ldrb    r2, [ r1 ]
  strb    r2, [ r0 ]
  add .s  r1, r1, # 1
  add .s  r0, r0, # 1
  sub .s  r3, r3, # 1
  b .ne   CMOVX3
  bx      lr
END-CODE
))

((
CODE BRANCH	\ --
\ *G The run time action of unconditional branches compiled on the target.
\ ** The branch target address is in-line and must have the T bit set.
\ ** INTERNAL.
l: takebranch
  mov .s  r1, # 1
  mov     r0, link
  bic .s  r0, r1
  ldr     r2, [ r0 ]			\ get address and branch
  bx      r2
END-CODE

CODE ?BRANCH	\ n --
\ *G The run time action of conditional branches compiled on the target.
\ ** The branch target address is in-line and must have the T bit set.
\ ** INTERNAL.
  mov .s  r1, tos
  ldmia   psp ! { tos }
  b .eq   takebranch
l: skipbranch
  mov     r0, link
  add .s  r0, # 4
  bx      r0
END-CODE
))

CODE (DO)	\ limit index --
\ *G The run time action of *\fo{DO} compiled on the target.
\ ** The branch target address is in-line and must have the T bit set.
\ ** INTERNAL.
  ldmia   psp ! { r1 }			\ get limit
L: PDO
  mov .s  r4, # 1
  lsl .s  r4, r4, # #31			\ r4 := $8000:0000
  mov     r3, link
  sub .s  r3, r3, # 1			\ clear T bit
  ldr     r2, [ r3 ]			\ get LEAVE address, compiler sets T bit
  add .s  r1, r1, r4			\ limit+$8000:0000
  sub .s  r0, tos, r1			\ index-limit-$8000:0000

  push    { r0, r1, r2 }		\ push LEAVE then limit, then index on ret. stack
  ldmia   psp ! { tos }			\ update tos
l: skipbranch
  mov     r0, link
  add .s  r0, # 4
  bx      r0
END-CODE

CODE (?DO) 	\ limit index --
\ *G The run time action of *\fo{?DO} compiled on the target.
\ ** The branch target address is in-line and must have the T bit set.
\ ** INTERNAL.
  ldmia   psp ! { r1 }			\ get limit
  cmp     r1, tos			\ check not equal
  b .ne   pdo				\ take DO ?
  ldmia   psp ! { tos }			\ update tos
l: takebranch
  mov .s  r1, # 1
  mov     r0, link
  bic .s  r0, r1
  ldr     r2, [ r0 ]			\ get address and branch
  bx      r2
END-CODE

asmcode
here is-action-of constant
\ *G The runtime code for a CONSTANT.
  sub .s  psp, psp, # 4			\ save TOS
  str     tos, [ psp, # 0 ]
  mov     r0, link			\ LINK is high register
  sub .s  r0, r0, # 1			\ remove T bit
  ldr     tos, [ r0, # 0 ]		\ get data address (in-line)
  pop     { pc }
END-CODE

code di		\ --
\ *G Disable interruots.
  cps     .id .i
  bx      lr
end-code

((
code dfi	\ --
\ *G Disable fault exceptions.
  cps     .id .f
  bx      lr
end-code
))

CODE WITHIN 	\ n1|u1 n2|u2 n3|u3 -- flag
\ *G The ANS version of WITHIN?.
\ ** This word uses unsigned arithmetic, so that signed compares are
\ ** treated as existing on a number circle.
  mov     r2, tos			\ save tos
  ldmia   psp ! { r0, r1 }              \ r2 = n3, r0 = n2, r1 = n1
  mov .s  tos, # 0			\ assume false
  sub .s  r2, r2, r0
  sub .s  r1, r1, r0
  cmp     r1, r2
  lo, if,
    mvn .s  tos, tos
  endif,
  bx      lr
END-CODE

code um/mod	\ ud1 u2 -- urem uquot
\ *G Full 64 by 32 unsigned division subroutine.
\ ** This routine uses a loop for code size.
  ldmia   psp ! { r0 }			\ dividend high
  ldr     r1, [ psp ]			\ dividend low
  mov .s  r4, # #32			\ loop counter
  mov .s  r2, # 0			\ always 0
  begin,
\    udiv64_step
    add .s  r1, r1, r1			\ 64 bit shift of dividend
    adc .s  r0, r0, r0
    mov     r3, r2			\ must not affect flags
    adc .s  r3, r3, r2			\ preserve dividend bit 63, R2=0
    sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
    adc .s  r3, r3, r3			\ success if bit 0 or 1 set
    ne, if,
      add .s  r1, r1, # 1		\ succeeded, update quotient
    else,
      add .s  r0, r0, tos		\ failed, undo subtraction
    endif,
    sub .s  r4, r4, # 1
  eq, until,
  mov     tos, r1			\ move quotient
  str     r0, [ psp ]			\ move remainder
  bx      lr
end-code

: u/		\ u1 u2 -- uquot
  0 swap um/mod nip  ;

