\ StartLPC11xx.fth - Cortex startup code for LPC11xx family

((
Copyright (c) 2009, 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20110614 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startlpc11xx
\ *T Cortex-M0 start up for LPC11xx
\ ===========
\ *P The file *\i{Cortex/Hardware/LPC11xx/startLPC11xx.fth} contains
\ ** the start up code for LPC17xx devices. Start up code for other
\ ** Cortex-M0 and M1 implementations can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M0
\ ** is in *\i{Cortex/StartCortex.fth}.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1. *\b{Do not} use this to set the ISP checksum!
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. This
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

interpreter
: genSysPLL	\ -- mval pval
\ *G INTERPRETER: Generate the data for the SYSPLLCTRL register.
  system-speed xtal-speed u< if
    cr ." system speed less than xtal speed" abort
  endif
  system-speed xtal-speed u/ 1-		\ MVAL-1
  dup 31 u> if
    cr ." MVAL out of range 1..32" abort
  endif
  0  over 1+ xtal-speed *  4 0 do	\ -- mval pval fclkout
    dup 1 i 1+ lshift * 156 MHz u< 0= ?leave
    swap 1+ swap
  loop
  drop
;

: gen-FlTim	\ -- flcfg
\ *G Generate the Flash configuration register value.
  system-speed 1- #20000000 /
;
target

system-speed xtal-speed <> [if]
  genSysPLL equ Pval  equ Mval
  Pval 5 lshift  Mval or  equ SYSPLLCTRLval

\ calculate the system-speed
  xtal-speed Mval 1+ * equ system-speed	\ PLL output clock
  system-speed 1 Pval 1+ lshift * equ cco-speed
[then]
  gen-FlTim equ FLASHCFGval

_SYSCON PDRUNCFG + equ _PDRUNCFG

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
  $22 _PDRUNCFG bic!			\ clear IRC and SYOSC power down bits

  bit3 _SYSCON SYSAHBCLKCTRL + or!	\ enable FMC register access
  _FMC FLASHCFG + dup @			\ configure Flash timing
    -4 and  FLASHCFGval or
  swap !

  _SYSCON				\ point at LPC System Control Regs
  \ Start oscillators, main clock = IRC
  0 over MAINCLKUEN + !			\ clear update enable
  0 over MAINCLKSEL + !			\ src=0=IRC,1=PLLin,2=WDT,3=PLLout
  1 over MAINCLKUEN + !			\ set update enable
  \ Configure PLL
  bit7 _PDRUNCFG or!			\ power down PLL
\  bit7 over PDRUNCFG + or!		\ power down PLL
  0 over SYSPLLCLKUEN + !		\ clear update enable
  1 over SYSPLLCLKSEL + !		\ sel=0=IRC,1=SysXtal
  1 over SYSPLLCLKUEN + !		\ set update enable
  SYSPLLCTRLval over SYSPLLCTRL + !	\ set PLL configuration
  bit7 _PDRUNCFG bic!			\ power up PLL
\  bit7 over PDRUNCFG + bic!		\ power up PLL
  \ Set AHB, wait for System Osc, select System Osc
  1 over SYSAHBCLKDIV + !		\ divider = 1
  begin					\ wait for PLL
    dup SYSPLLSTAT + @ bit0 and
  until
  0 over MAINCLKUEN + !			\ clear update enable
  3 over MAINCLKSEL + !			\ src=0=IRC,1=PLLin,2=WDT,3=PLLout
  1 over MAINCLKUEN + !			\ set update enable

  drop
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    setClocks
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc11      11        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

