\ EMC-DRAM-LPC1788.fth - LPC1788 family external memory controller setup

((
Copyright (c) 2010, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140117 BJC001	Modified for LPC1788 family devices which use a rather
		different pin assignment scheme from other LCP1xxx and LPC2xxx
		devices. See sfrLPC1788.fth for family members.
))

only forth definitions
decimal

\ ===========
\ *! EMC-DRAM-LPC1788
\ *T LPC1788 External Memory
\ ===========
\ *P This file assumes that the external memory is a SDRAM chip configured
\ ** as 2M x 16bits x 4 banks, with base address fixed by the use of /DYCS0
\ ** at 0xA000.0000 through A0FF.FFFF.
\ ** The DRAM region has 15 'echoes' above this range.

\ *P IMPORTANT:  The timings here ASSUME 95MHz CPU clock.  SDRAM access may
\ ** be unnecessarily slow if clock speed is reduced.

\ *P NOTE: The dynamic RAM + controller consumes up to 100mA when active, so if
\ ** this memory is not needed, disabling it will save significant power.
\ ** Do this by defining DISABLE_SDRAM as TRUE.

[UNDEFINED] DISABLE_SDRAM [IF]
  FALSE   equ   DISABLE_SDRAM   \ default to enabled
[ENDIF]
\ *P Much of this code is derived from the function *\b{ConfigureEMC()}
\ ** in the Olimex start-up code in the file
\ ** *\i{u-boot-oli2468\u-boot-1.3.1-olimex\board\lpc_e2468\lowlevel_init.c}.

\ *P The LPC-1788 pin usage is as follows:
\ *E SDRAM
\ ** P2.16     CASn
\ ** P2.17     RASn
\ ** P2.18     SDclk
\ ** P2.20     DyCS0/SDCS
\ ** P2.24     SDclkEn
\ ** P2.28     DQMN0
\ ** P2.29     DQMN1
\ ** P3.0..15  Dn
\ ** P4.0..14  An
\ ** P4.25     /WE

\ *P The flash memory & static RAM controllers are not enabled

$A000:0000 constant SDRAMbase	\ -- addr
 $100:0000 constant SDRAMsize	\ 16MB
\ *G Base address & size of the SDRAM block (both hardware determined).

8				\ cmddly = 8
8 8 lshift or			\ fbclkdly = 8
8 16 lshift or			\ clkout0dly = 8
  equ EMCDLYCTLval

0 #14 lshift			\ 16 bit data
0 #12 lshift or			\ high performance
2   9 lshift or			\ 8M*16, 4 banks, row=12, col=9
1   7 lshift or
  equ initDynCfg0

 $0303 equ initDynRasCas0    \ SDRAM RAS & CAS delay registers
\    ^------------ RAS latency, clocks
\   ^------------- reserved (=0)
\  ^-------------- CAS latency, clocks
\ ^--------------- reserved (=0)

3                 \ burst length = 8
3  4  lshift or   \ cas latency = 3 cycles
  equ SDramMode

\ Refresh period is 64 ms / 2 ^ rows.
\ Refresh timer is set at CCLK/16.
system-speed #64 #1000 */ 4096 / 16  / equ initDynRfrsh
\ *G Refresh counter for 64 ms period, 12 bit row, 16 clocks per unit.

: initEMC	\ --
\ *G Initialise the EMC interface for SDRAM only.
\ ** Performed at power up.
\ power up and pin config
  ~pcEMC _PCONP or!			\ enable EMC interface
  0 _EMCCLKSEL !				\ EMC interface runs at sys-clock speed
\ set the IO pins for EMC use
  _IOCON			\ base
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P2_16 + !	\ P2.16 is EMC CAS
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P2_17 + !	\ P2.16 is EMC RAS
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P2_18 + !	\ P2.18 is EMC Clk[0]
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P2_20 + !	\ P2.20 is EMC DYCS0
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P2_24 + !	\ P2.24 is EMC CKE0
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P2_28 + !	\ P2.28 is EMC DYCS0
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P2_29 + !	\ P2.29 is EMC DYCS0

  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_00 + !	\ P3.0 is EMC data D0 (type D)
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_01 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_02 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_03 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_04 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_05 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_06 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_07 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_08 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_09 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_10 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_11 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_12 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_13 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_14 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P3_15 + !	\ P3.15 is EMC data D15 (type D)

  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_00 + !	\ P4.0 is EMC addr A0 (type D)
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_01 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_02 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_03 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_04 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_05 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_06 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_07 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_08 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_09 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_10 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_11 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_12 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_13 + !
  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_14 + !	\ P4.14 is EMC addr A14 (type D)

  IO_PERIPH_FUNC1 +hysteresis over IOCON_P4_25 + !	\ P4.25 is EMC /WE
  drop							\ .. base

\ EMC Dynamic SDRAM setup
\ Timings have been checked for AMIC part A43L3616AV-7F
\ @96MHz, 1 CCLK ~ 10.4ns
  EMCDLYCTLval _EMCDLYCTL !		\ set delays
  _EMC
  $01 over EMCControl + !		\ enabled
  $01 over EMCDynamicReadConfig + !	\ command delayed strategy
  $0000 over EMCConfig + !		\ little-endian, CLKOUT=CCLK
  initDynRasCas0 over EMCDynamicRasCas0 + !
  $01 over EMCDynamictRP + !		\ 1+1, >20ns = 2 clk
  $03 over EMCDynamictRAS + !		\ 3+1, >45ns = 4 clk
  $07 over EMCDynamictSREX + !		\ 7+1, >80-100ns = 8 clk
  $01 over EMCDynamictAPR + !		\ min 1 clock
  $07 over EMCDynamictDAL + !		\ 7+1, >75ns
  $03 over EMCDynamictWR + !		\ 3+1
  $02 over EMCDynamictRC + !		\ 2+1 min 2 clocks
  $05 over EMCDynamictRFC + !		\ 5+1, >63ns = 6 clocks
  $05 over EMCDynamictXSR + !		\ 5+1 (use Trc >63ns) = 6 clocks
  $01 over EMCDynamictRRD + !		\ 0+1, >14ns = 2 clk
  $03 over EMCDynamictMRD + !		\ 3+1 min 2 clocks

  initDynCfg0 over EMCDynamicConfig0 + !
  #100 ms
  3 7 lshift  3 or  over EMCDynamicControl + !	\ NOP cmd + clocks
  #200 ms
  2 7 lshift  3 or  over EMCDynamicControl + !	\ PRECHARGE cmd + clocks
  1 over EMCDynamicRefresh + !		\ min refresh period
  #128 0 do loop			\ wait at least 128 bus clocks - 512?
  initDynRfrsh over EMCDynamicRefresh + !	\ final refresh period
  1 7 lshift  3 or  over EMCDynamicControl + !	\ MODE cmd + clocks
  DMB0base SDramMode #12 lshift or @ drop	\ set mode by reading
  0 over EMCDynamicControl + !			\ NORMAL cmd
  bit19 over EMCDynamicConfig0 + or!		\ enable buffers
  tick-ms ms				\ settle
  drop					\ discard base address
;

DISABLE_SDRAM FALSE = [IF]
  ' initEMC AtCold
[ENDIF]

((
\ Original C code
void SDRAM_Init (void)
{
#ifndef  OLIMEX
#define DATABUS_32
#endif
  /* Enable EMC clock*/
  PCONP_bit.PCEMC = 1;
  /*The EMC uses the same clock as the CPU*/
  EMCCLKSEL = 0;
  /*Assign pins to SDRAM controller*/
  IOCON_P2_16 = 0x21;
  IOCON_P2_17 = 0x21;
  IOCON_P2_18 = 0x21;
  IOCON_P2_20 = 0x21;
  IOCON_P2_24 = 0x21;
  IOCON_P2_28 = 0x21;
  IOCON_P2_29 = 0x21;

#ifdef  DATABUS_32
  IOCON_P2_30 = 0x21;
  IOCON_P2_31 = 0x21;
#endif

  IOCON_P3_00 = 0x21;
  IOCON_P3_01 = 0x21;
  IOCON_P3_02 = 0x21;
  IOCON_P3_03 = 0x21;
  IOCON_P3_04 = 0x21;
  IOCON_P3_05 = 0x21;
  IOCON_P3_06 = 0x21;
  IOCON_P3_07 = 0x21;
  IOCON_P3_08 = 0x21;
  IOCON_P3_09 = 0x21;
  IOCON_P3_10 = 0x21;
  IOCON_P3_11 = 0x21;
  IOCON_P3_12 = 0x21;
  IOCON_P3_13 = 0x21;
  IOCON_P3_14 = 0x21;
  IOCON_P3_15 = 0x21;

#ifdef  DATABUS_32
  IOCON_P3_16 = 0x21;
  IOCON_P3_17 = 0x21;
  IOCON_P3_18 = 0x21;
  IOCON_P3_19 = 0x21;
  IOCON_P3_20 = 0x21;
  IOCON_P3_21 = 0x21;
  IOCON_P3_22 = 0x21;
  IOCON_P3_23 = 0x21;
  IOCON_P3_24 = 0x21;
  IOCON_P3_25 = 0x21;
  IOCON_P3_26 = 0x21;
  IOCON_P3_27 = 0x21;
  IOCON_P3_28 = 0x21;
  IOCON_P3_29 = 0x21;
  IOCON_P3_30 = 0x21;
  IOCON_P3_31 = 0x21;
#endif

  IOCON_P4_00 = 0x21;
  IOCON_P4_01 = 0x21;
  IOCON_P4_02 = 0x21;
  IOCON_P4_03 = 0x21;
  IOCON_P4_04 = 0x21;
  IOCON_P4_05 = 0x21;
  IOCON_P4_06 = 0x21;
  IOCON_P4_07 = 0x21;
  IOCON_P4_08 = 0x21;
  IOCON_P4_09 = 0x21;
  IOCON_P4_10 = 0x21;
  IOCON_P4_11 = 0x21;
  IOCON_P4_12 = 0x21;
  IOCON_P4_13 = 0x21;
  IOCON_P4_14 = 0x21;
  IOCON_P4_25 = 0x21;
  /*Init SDRAM controller*/
  EMCDLYCTL_bit.CMDDLY = 0x8;
  /*Set data read delay*/
  EMCDLYCTL_bit.FBCLKDLY = 0x8;
  /**/
  EMCDLYCTL_bit.CLKOUT0DLY = 0x8;
  //EMCDLYCTL_bit.CLKOUT1DLY = 0x0;
  EMCControl      = 1;         // enable EMC
  EMCDynamicReadConfig = 1;
  EMCDynamicRasCas0_bit.CAS = 3;
  EMCDynamicRasCas0_bit.RAS = 3;
  EMCDynamictRP = P2C(SDRAM_TRP);
  EMCDynamictRAS = P2C(SDRAM_TRAS);
  EMCDynamictSREX = P2C(SDRAM_TXSR);
  EMCDynamictAPR = SDRAM_TAPR;
  EMCDynamictDAL = SDRAM_TDAL+P2C(SDRAM_TRP);
  EMCDynamictWR = SDRAM_TWR;
  EMCDynamictRC = P2C(SDRAM_TRC);
  EMCDynamictRFC = P2C(SDRAM_TRFC);
  EMCDynamictXSR = P2C(SDRAM_TXSR);
  EMCDynamictRRD = P2C(SDRAM_TRRD);
  EMCDynamictMRD = SDRAM_TMRD;
#ifdef  DATABUS_32
  EMCDynamicConfig0 = 0x0004680;        // 13 row, 9 - col, SDRAM, 32 bit data bus
#else
  EMCDynamicConfig0 = 0x0000680;        // 13 row, 9 - col, SDRAM, 16 bit data bus
#endif
  // JEDEC General SDRAM Initialization Sequence
  // DELAY to allow power and clocks to stabilize ~100 us
  // NOP
  EMCDynamicControl = 0x0183;
  for(volatile Int32U i = 200*30; i;i--);
  // PALL
  EMCDynamicControl_bit.I = 2;
  EMCDynamicRefresh = 1;
  for(volatile Int32U i= 256; i; --i); // > 128 clk
   EMCDynamicRefresh = P2C(SDRAM_REFRESH) >> 4;
  // COMM
  EMCDynamicControl_bit.I = 1;
  // Burst 4, Sequential, CAS-3
#ifdef  DATABUS_32
  volatile unsigned long Dummy = *(volatile unsigned short *)
                                  ( (uint32_t)&SDRAM_BASE_ADDR + (0x32UL << (13) ) );
#else
  volatile unsigned long Dummy = *(volatile unsigned short *)
                                  ( (uint32_t)&SDRAM_BASE_ADDR + (0x33UL << (12) ) );
#endif
  // NORM
  EMCDynamicControl = 0x0000;
  EMCDynamicConfig0_bit.B = 1;
  for(volatile Int32U i = 100000; i;i--);
}
))


\ ======
\ *> ###
\ ======

decimal

