\ ExApp1766.ctl - Example application for Olimex LPC-1766STK board

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20110727 MPE001 First release.
))


\ ============
\ *! exapp1766
\ ============

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************]

1 constant MPE?	\ true is built at MPE
0 constant KDE?	\ true if built at KIMA

MPE? [if]	\ if built on an MPE development box

c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives
mc" %ExampleDir%/FATfiler"		\ where FAT file system lives
                setMacro FatFiler
[defined] Target_386_Windows [if]
c" c:/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]
[defined] Target_386_Linux [if]
c" ~/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]
[then]

KDE? [if]
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M3			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
   Hex-I32			\ produce Intel Hex-32 obj format

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N LPC1766 definitions
\ =====================

#1700 equ LPCtype	\ -- 1700
\ *G Define the LPC type, using 1700 as the generic type
#256 kb equ /LPCflash	\ -- u
\ *G Define the size of the internal Flash. This is used
\ ** to control the use of  the Flash sector table.

1 equ PCLKSELbug?	\ --
\ *G As of January 2010, revision '-' versions of the LPC17xx
\ ** devices must set the PCLKSELx registers before PLL0 is
\ ** enabled. Leave this flag true until you have checked the
\ ** latest errata sheet for your LPC17xx device.
1 equ PLL0bug?	\ --
\ *G As of January 2010, revision '-' versions of the LPC17xx
\ ** devices do not disconnect and disable PLL0 when entering
\ ** Deep Sleep and Power-Down modes.
1 equ MCPWMbug?	\ --
\ *G As of January 2010, revision '-' versions of the LPC17xx
\ ** devices have non-functional MCI0/1/2 inputs.

#128 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

#12 MHz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
#4 MHz equ irc-speed		\ -- hz
\ *G Internal RC oscillator clock rate in HZ.
#32768 equ rtc-speed		\ -- hz
\ *G External RTC oscillator clock rate in HZ.

#96 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.
#48 MHz equ usb-speed		\ -- hz
\ *G Requested clock speed to USB peripheral.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $1000:0000..$1000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0000:0000.

  $0000:0000 $0003:FFFF cdata section ExApp1766	\ code section in boot Flash
  $1000:0200 $1000:0FFF idata section PROGd	\ 4k IDATA
  \ ISP uses RAM $1000:0000..1000:01FF
    $1000:0000 equ ISPRAM
  $1000:1000 $1000:7FFF udata section PROGu	\ 12k+16kb UDATA
  \ IAP uses top 32 bytes of RAM
    #32 reserve equ ISPdata			\   32 bytes at top for IAP and ISP

interpreter
: prog ExApp1766  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUART3?	\ -- n
\ *G Set non-zero to compile code for UART3,
\ ** device *\fo{Console3}.
#9600 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 0..3 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CortexDef		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrLPC17xx		\ LPC17xx special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startLPC17xx		\ startup code
0 equ FastCmove?
  include %CpuDir%/CodeCortex		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootLPC17xx \ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault handlers - needs serial
  include %CommonDir%/CQueues		\ character queues
  include %CpuDir%/Drivers/serLPC17xxqi \ RX/TX queued interrupt driver
\  include %CpuDir%/Drivers/serLPC17xxp	\ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM3		\ local variables

tasking? [if]
  include %CpuDir%/MultiCortex		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickLPC17xx
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickLPC17xx
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile ExApp1766.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M3"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  include %HwDir%/IAP17xx		\ IAP code
  include %HwDir%/FlashTables		\ Flash sector tables
  include %HwDir%/ReProg/ReFlash	\ Reflash system
  include %HwDir%/FlashProg		\ higher level Flash routines

  include %CommonDir%/DebugTools	\ debug tools

  include %FatFiler%/Drivers/spiLPC17xxhard	\ SPI via SSP1
  include %FatFiler%/SSP1768.bld	\ FAT file system

  include %CpuDir%/Drivers/gpioLPC17xx	\ easy GPIO drivers
  include %CpuDir%/Drivers/adcLPC17xx	\ ADC driver
  include %CpuDir%/Drivers/dacLPC17xx	\ DAC driver
  include %HwDir%/ExAppTask		\ Example task
  ' startApp AtCold			\ start task at power up

\  include %HwDir%/NokiaSSP0		\ low level LCD driver

((
  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%/XmodemTxRx	\ Xmodem driver
  include %CommonDir%/RomForth/iodef	\ link i/o
  include %CommonDir%/RomForth/textfile	\ AIDE textfile support
\  variable lseconds
\  include %ExampleDir%\UnixTime
))


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\benchmarks\xbench32	\ performance tests

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
ExcVecs 0				\ -- addr csum
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $14 + @ +
  over $18 + @ +
  negate
swap $1C + !

\ Set the reset address for ELF
  ' StartCortex setBoot

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======

