NOKIA_6100_LCD [if]

\ ----- led check -----

#21 equ LED_BIT

: toggle_led \ --
  1 LED_BIT lshift >r
  _FIO FIO0PIN + @
  r@ and if
    r@ _FIO FIO0CLR + !
  else
    r@ _FIO FIO0SET + !
  then
  r> drop
;

: start_led \ --
   +RTCint
   _FIO FIO0DIR +
   dup @
   1 LED_BIT lshift or swap !
   assign toggle_led to-do RTCticker
;

' start_led AtCold

\ -------- spi -------------

: init_lcd_spi \ ----
#12 _SSP0 SSPCPSR + !
  9 1 -         \ bits size
  0 4 lshift or \ mode, 00->SPI
  1 6 lshift or \ CPHA
  1 7 lshift or \ CPOL
  0 8 lshift or \ clocks per bit
  _SSP0 SSPCR0 + w!
  0             \ loop back mode
  1 1 lshift or \ SSE SSP enable
  0 2 lshift or \ MS 0->master
  0 3 lshift or \ SOD slave output disable
  _SSP0 SSPCR1 + c!
  3 #23 1 set_function \ p1.23 MISO0
  3 #20 1 set_function \ p1.20 SCK0
  3 #21 1 set_function \ p1.20 SSEL0
  3 #24 1 set_function \ p1.24 MOSI0
;

: spi0_write \ n1 -- ; send n1 and receive n2
  _SSP0  SSPDR + w!
  begin
\    pause
    _SSP0 SSPSR + c@
    1 4 lshift and 0=
  until
;

: spi0_exch \ n1 -- n2; send n1 and receive n2
  spi0_write
  _SSP0 SSPDR + w@
;

$100 equ DOFS \ data offset

: WriteSPIdata \ n --
  DOFS or spi0_write
;

: WriteSPIcommand \ n --
  $ff and spi0_write
;

\ --------------- Epson S1D15G10 Command Set  -------------

#define LCD_DISON       $0af
#define LCD_DISOFF      $0ae
#define LCD_DISNOR      $0a6
#define LCD_DISINV      $0a7
#define LCD_COMSCN      $0bb
#define LCD_DISCTL      $0ca
#define LCD_SLPIN       $095
#define LCD_SLPOUT      $094
#define LCD_PASET       $075
#define LCD_CASET       $015
#define LCD_DATCTL      $0bc
#define LCD_RGBSET8     $0ce
#define LCD_RAMWR       $05c
#define LCD_RAMRD       $05d
#define LCD_PTLIN       $0a8
#define LCD_PTLOUT      $0a9
#define LCD_RMWIN       $0e0
#define LCD_RMWOUT      $0ee
#define LCD_ASCSET      $0aa
#define LCD_SCSTART     $0ab
#define LCD_OSCON       $0d1
#define LCD_OSCOFF      $0d2
#define LCD_PWRCTR      $020
#define LCD_VOLCTR      $081
#define LCD_VOLUP       $0d6
#define LCD_VOLDOWN     $0d7
#define LCD_TMPGRD      $082
#define LCD_EPCTIN      $0cd
#define LCD_EPCOUT      $0cc
#define LCD_EPMWR       $0fc
#define LCD_EPMRD       $0fd
#define LCD_EPSRRD1     $07c
#define LCD_EPSRRD2     $07d
#define LCD_NOP         $025

#define LCD_XSIZE       131
#define LCD_YSIZE       131

\ --- startup sequences ---
\ from 0 to $ff -> control, from $100 to $1ff
\ sequence tables
\ In spite of big effort to use the 8 bits color mode, it was IMPOSSIBLE
\ to move the S1D15G10 to initialise on this size!
\

here
0 ,                     \ count
LCD_DISCTL   w,
    0 DOFS + w,
   32 DOFS + w,
    0 DOFS + w,
    0 DOFS + w,
LCD_COMSCN   w,
    1 DOFS + w,
LCD_TMPGRD   w,
    0 DOFS + w,
LCD_OSCON    w,
LCD_SLPOUT   w,
LCD_VOLCTR   w,
   37 DOFS + w,
    1 DOFS + w,
LCD_PWRCTR   w,
  $0f DOFS + w,
LCD_DISINV   w,
LCD_DATCTL   w,
    0 DOFS + w,         \ invers normal orientation; scan across cols, then rows
    0 DOFS + w,         \ RGB arrangement (RGB all rows/cols)
\    4 DOFS + w,         \ RGB arrangement (RGB row 1, BGR row 2)
\    1 DOFS + w,         \ 8 bit-color display
    2 DOFS + w,         \ 12 bit-color display
\    4 DOFS + w,         \ 16 bit-color display
LCD_RGBSET8  w,          \ only necessary for 8 bits color
\ color table
\ RED
    0 DOFS + w,
    2 DOFS + w,
    4 DOFS + w,
    6 DOFS + w,
    8 DOFS + w,
   10 DOFS + w,
   12 DOFS + w,
   15 DOFS + w,
\ GREEN
    0 DOFS + w,
    2 DOFS + w,
    4 DOFS + w,
    6 DOFS + w,
    8 DOFS + w,
   10 DOFS + w,
   12 DOFS + w,
   15 DOFS + w,
 \ BLUE
    0 DOFS + w,
    4 DOFS + w,
    9 DOFS + w,
   15 DOFS + w,
LCD_PASET    w,
    2 DOFS + w,
  131 DOFS + w,
LCD_CASET    w,
    0 DOFS + w,
  131 DOFS + w,
LCD_RAMWR    w,     \ start RAM write
    0 DOFS + w,
    0 DOFS + w,
    0 DOFS + w,
LCD_NOP      w,
here over - cell - over !
constant LCD_SEQUENCE_TABLE1

here
0 ,                     \ count
LCD_NOP      w,
here over - cell - over !
constant LCD_SEQUENCE_TABLE2

: send_sequence \ table --
  dup cell+ swap @ bounds do
    i w@ spi0_write
  2 +loop
;

: init_lcd \ --
  start_timer1
  init_lcd_spi
  0 OBJ_LCD_RESET put
  5 ms_delay
  OBJ_LCD_RESET dup get 0= swap put    \ invert bit
  1 OBJ_LCD_BLANK put
  20 ms_delay
  LCD_SEQUENCE_TABLE1 send_sequence
\  LCD_SEQUENCE_TABLE2 send_sequence
((
  #18000 0 do
    28 DOFS + spi0_write           \ all on
  loop
))
  LCD_NOP spi0_write
  150 ms_delay
  LCD_DISON spi0_write
;

' init_lcd AtCold

: adj_v \ ---;  simple interactive function to adjust the voltages
  3 32
  begin
    key case
      bl of 2drop exit endof
      [char] u of 1+ endof
      [char] d of 1- endof
      [char] U of swap 1+ swap endof
      [char] D of swap 1- swap endof
    endcase
    LCD_VOLCTR spi0_write
    2dup .  .
    2dup DOFS or spi0_write
    DOFS or spi0_write
  again
;

: write_pixel \ x y color -- ; color follows the 12bit color code
  >r
  dup  0<          if r> drop 2drop exit then
  dup  LCD_YSIZE > if r> drop 2drop exit then
  over 0<          if r> drop 2drop exit then
  over LCD_XSIZE > if r> drop 2drop exit then
  LCD_CASET WriteSPICommand
  over WriteSpidata
  over WriteSpidata
  LCD_PASET WriteSPICommand
  dup WriteSpidata
  WriteSpidata
  drop
  r>
  LCD_RAMWR WriteSPICommand
  dup 4 rshift $ff and WriteSpidata
  dup $0f and 4 lshift over 8 rshift $0f and or WriteSpidata
  $ff and  WriteSpidata
;

: fill_area \ x1 y1 x2 y2 color ---
  >r
  rot swap
  LCD_PASET WriteSPICommand
  over WriteSpidata
  dup  WriteSpidata
  swap - >r
  LCD_CASET WriteSPICommand
  over WriteSpidata
  dup  WriteSpidata
  swap - 1+ r> 1+ *
  r> swap
  LCD_RAMWR WriteSPICommand
  2/ 1+ 0 do
    dup 4 rshift $ff and WriteSpidata
    dup $0f and 4 lshift over 8 rshift $0f and or WriteSpidata
    dup $ff and  WriteSpidata
  loop
  drop
  LCD_NOP spi0_write
;

LCD_XSIZE equ DISP_XSIZE
LCD_YSIZE equ DISP_YSIZE

[then]

\ --------------------------------------------------------------------------------

QUARTER_VGA_OLED [if]

\ simple OLED 1/4 VGA interface
\ Uses the 2,8" CHI MEI EL CORPORATION C0283QGLD-T module, unfortunately no more available after may 2010

: reset_oled
   OBJ_OLED_RESET dup object_init
   0 2dup swap put
   100000 do_delay
   0= swap put
;

: index! \ n
   card_base w!
;

: param! \ n
   card_base 2+ w!
;

: init_oled \ --  ; 
reset_oled
$23 index!
  2 index!     0 param!
\  3 index!  $130 param!  \ AM=0 horizontal is 240
  3 index!  $131 param!  \ AM=1 horizontal is 320
$10 index!     0 param!
$71 index! $2780 param!  \ gamma regs
$72 index! $3380 param!
$73 index! $1d18 param!
$74 index! $1f11 param!
$75 index! $2419 param!
$76 index! $1a14 param!
$77 index! $211a param!
$78 index! $2013 param!
  5 index!     1 param!
;

' init_oled AtCold

: xxx \ n --
  $22 index!
  #240 #320 * 0 do
    dup param!
  loop
  drop
;

: red
$f800 xxx ;
: green
$7c0 xxx ;
: blue
$1f xxx ;

: qqq  \ n ---
  begin
    red
    dup do_delay
    green
    dup do_delay
    blue
    dup do_delay
  key? until
  drop
;

: set_oled_ptr \ n --
  $20 index! dup $ff and param!
  $21 index! 8 rshift param!
;

\ ------------ 1/4 VGA ------------------

320 equ H_SIZE_FULL
240 equ V_SIZE_FULL

variable dot_color $1f dot_color !

: write_pixel \ x y color ---
  >r
  dup  0<            if r> drop 2drop exit then
  dup  V_SIZE_FULL > if r> drop 2drop exit then
  over 0<            if r> drop 2drop exit then
  over H_SIZE_FULL > if r> drop 2drop exit then
  H_SIZE_FULL 1- rot - 8 lshift + set_oled_ptr    \ horizontal is inverted adressed 0-319
  $22 index! r> param!
;

: fill_area { x1 y1 x2 y2 color -- }
  y2 y1 ?do
    H_SIZE_FULL x2 - 8 lshift i + set_oled_ptr  \ put to x2 y1 because of x axis is reverse
    $22 index! 
    x2 x1 - 0 ?do
      color param!
    loop
  loop 
;

H_SIZE_FULL equ DISP_XSIZE
V_SIZE_FULL equ DISP_YSIZE

[then]
