\ StartLPC17xx.fth - Cortex startup code for LPC17xx family

((
Copyright (c) 2009, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20100112 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startlpc17xx
\ *T Cortex start up for LPC17xx
\ ===========
\ *P The file *\i{Cortex/Hardware/LPC17xx/startLPC17xx.fth} contains
\ ** the start up code for LPC17xx devices. Start up code for other
\ ** Cortex-M3 implementations, e.g. STM32, can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M3
\ ** is in *\i{Cortex/StartCortex.fth}.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1. *\b{Do not} use this to set the ISP checksum!
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. This
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

\ *P Find the clock divider that generates the CCO clock at 288 Mhz.
\ ** The value in the CCLKCFG register is one less than this.
#288000000 system-speed / 1- equ CCLKCFGval	\ -- x
\ *G The CCLKCFG register initial value.
CCLKCFGval 1+ system-speed * equ cco-speed	\ -- hz
\ *G The actual CCO frequency that will be generated by the
\ ** calculated configuration values.

interpreter
: genPLL0	\ -- mval nval
\ *G INTERPRETER: Generate the data for the PLL0CFG register.
\ ** For the moment, we set N=1 (nval=0), and calculate the Mval
\ ** only.
  system-speed xtal-speed u< if
    cr ." system speed less than xtal speed" abort
  endif
  cco-speed xtal-speed 2* / 1-		\ Mval-1
  dup 6 u< if
    cr ." MVAL too small; MVAL=" . abort
  endif
  dup #511 u> if
    cr ." MVAL too large; MVAL=" . abort
  endif
  0					\ Nval-1
;
: gen-FlTim	\ -- flcfg
\ *G Generate the Flash configuration register value.
  system-speed 1- #20000000 /
;
target

system-speed xtal-speed <> [if]
  genPLL0 equ Nval  equ Mval
  Nval #16 lshift  Mval or  equ PLL0CFGval
\ recalculate the system-speed
  xtal-speed Nval 1+ /			\ PLL input clock
  Mval 1+ 2* *				\ PLL output speed
    equ cco-real			\ CCO speed
  cco-real CCLKCFGval 1+ /
    equ system-speed			\ system speed this will give
  usb-speed [if]
    cco-real usb-speed / 1- equ USBCLKval
  [else]
    $0F equ USBCLKval
  [then]
[then]
  gen-FlTim #12 lshift $003A or equ FlCfgval

PCLKSELbug? [if]
L: ^PCLKSEL0val
\ *G This location contains the initial value of the PCLKSEL0
\ ** register if the equate *\fo{PCLKSELbug?} is set non-zero.
  0 ,
L: ^PCLKSEL1val
\ *G This location contains the initial value of the PCLKSEL0
\ ** register if the equate *\fo{PCLKSELbug?} is set non-zero.
  0 ,
[then]

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
  _SYSCON				\ point at LPC System Control Regs

[ PCLKSELbug? ] [if]			\ set PCLKSELx now
  ^PCLKSEL0val @ over PCLKSEL0 + !
  ^PCLKSEL1val @ over PCLKSEL1 + !
[then]

  \ Main oscillator set up
  $0020 over SCSreg + !			\ enable main oscillator
  begin					\ wait for it
    dup SCSreg + @ bit6 and
  until

  \ Set core clock with PLL0.
  \ PLL1 for USB is set up by the USB code.
  CCLKCFGval over CCLKCFG + !		\ set up CCO -> Core clock divider
  $0001 over CLKSRCSEL + !		\ %01 for main oscillator
  PLL0CFGval over PLL0CFG + !		\ set PLL divider
  $0001 over PLL0CON + !		\ enable PLL, do not connect
  $AA over PLL0FEED + !			\ feed sequence
  $55 over PLL0FEED + !
  begin					\ wait for lock
    dup PLL0STAT + @ bit26 and
  until

  $0003 over PLL0CON + !		\ enable and connect PLL
  $AA over PLL0FEED + !			\ feed sequence
  $55 over PLL0FEED + !

  \ Set the Flash memory timing
  FlCfgval over FlashCfg + !

  drop
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

((
: +led1		\ -- ; turn on LED1, pin -> 0
  bit25 _FIO1 FIOCLR + !		\ P1.25 -> 1
;
: -led1		\ -- ; turn off LED1, pin -> 1
  bit25 _FIO1 FIOSET + !		\ P1.25 -> 1
;

: +led2		\ -- ; turn on LED2, pin -> 0
  bit4 _FIO0 FIOCLR + !			\ P0.4 -> 1
;
: -led2		\ -- ; turn off LED2, pin -> 1
  bit4 _FIO0 FIOSET + !			\ P0.4 -> 1
;

: initLEDs	\ -- ; initialise LED1 on P1.25, LED2 on P0.4
\ Initialise LEDs for testing, P0.4 and P1.25 default to GPIO.
  ~pcGPIO _PCONP or!			\ keeps simulator happy
  $0000:0300 _PINSEL0 bic!		\ force GPIO mode for P0.4
  $000C:0000 _PINSEL3 bic!		\ force GPIO mode for P1.25
  bit4  _FIO0 FIODIR + or!		\ P0.4 is output
  bit25 _FIO1 FIODIR + or!		\ P1.25 is output
  +led1 +led2
;
))

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    setClocks setNVIC
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

