\ Reprogram LPC40xx over serial line.

((
Copyright (c) 2003, 2004, 2005, 2006, 2008, 2010, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20150703 SFP004 Converted for LPC40xx
20100125 SFP004 Converted for LPC17xx
20081008 SFP003 Converted for LPC23xx/24xx/28xx
20060818 SFP002 Added UART selection.
20040809 SFP001 Added equate FULLIAP?
20030728 SFP000 First cut
))

\ =============
\ *! reprog40xx
\ *T Reprogramming the LPC40xx serially
\ =============
\ *S Introduction
\ *P *\i{Reprog40xx.ctl} is the control file for the serial loader
\ ** which reprograms the on-chip Flash memory.
\ ** Because the on-chip Flash cannot be accessed during
\ ** programming, and because the initial loader code may
\ ** itself be replaced, the Flash programming code is copied
\ ** into RAM for execution. The only exit from the code is a
\ ** reset of the CPU to execute the newly Flashed program. All
\ ** interrupts are disabled during reprogramming, and so
\ ** polled serial drivers are used.

\ *P Access to the reprogramming software is defined in
\ ** *\i{ReFlash.fth} for the main system code and in
\ ** this control file. The two files must match.

\ *P Return is by rebooting the system. The contents of the
\ ** internal RAM and Flash are destroyed, therefore any data
\ ** that must be preserved should be saved externally to the
\ ** chip.


only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

c" ../../.."	setmacro CpuDir
c" ../../../.."	setmacro CommonDir
c" ."		setmacro AppDir
c" .."		setmacro HwDir


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to show output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ words start 32bit aligned
  T2-mode			\ Cortex-M processor type
  no-heads			\ do not generate heads
  no inlining			\ saves some space
  -ram-table			\ saves some space
  +LeafCalls			\ saves some space

0         equ false
false not equ true


\ ****************
\ *S Configuration
\ ****************

#1700 equ LPCtype	\ -- 1700
\ *G Define the LPC type, using 1700 as the generic type
#512 kb equ /LPCflash	\ -- u
\ *G Max size of LPC40xx internal Flash

#96000000 equ system-speed	\ -- Hz
\ *G System operating speed in HZ.

0 equ FullIAP?		\ -- n
\ *G Set this equate non-zero to compile additional IAP routines.
\ ** See *\i{IAP40xxx.FTH} for more details.


\ **********
\ Memory map
\ **********

LPCtype #1700 = [if]
$1000:0200 $1000:0FFF cdata section Reprog40xx
interpreter
: prog  Reprog40xx  ;
target
[then]

$1000:1000 $1000:11FF idata section PROGd
$1000:1200 $1000:13FF udata section PROGu
$1000:0000 $1000:01FF udata section Xbuff	\ 512 byte Xmodem buffer
  sec-base equ Xbase			\ base address of buffer
  sec-len equ Xlen			\ length of buffer
  Xbase Xlen + equ Xtop			\ top of buffer

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ********************
\ *S Items of interest
\ ********************
\ *P The stacks have already been set up by the calling program.

\ *[
include %CpuDir%\CortexDef		\ Cortex CPU equates
include %CpuDir%\sfrLPC40xx		\ Special function registers
\ *]

l: CLD1			\ -- addr
\ *G Filled in later with the xt of *\fo{ReprogFlash}. This
\ ** label marks the start of the RAM code area.
  0 ,
l: CLD_CopyFlash	\ -- addr
\ *G Filled in later with the xt of *\fo{CopyFlash}
  0 ,
l: CLD_UART		\ -- addr
\ *G Filled in later with 0 or UART base address to use.
\ ** If set to 0, the default is UART0. This allows the calling
\ ** application to select which UART to use.
  0 ,
l: CLD_clockspeed	\ -- addr
\ *G Contains the clock speed to use. Filled in by the calling
\ ** application. Default is 96 MHz.
  96 Mhz ,
l: CLD_#FlSectors	\ -- addr
\ *G Number of sectors in the flash. Default is 30 for 512k
\ ** devices.
  #30 ,

compiler
: prog-speed	\ -- hz
\ *G A macro to fetch the system clock speed from the entry table.
  CLD_clockspeed @  ;
target

\ *[
include %AppDir%\primitives		\ Forth primitives from CODEARM.FTH
include %AppDir%\MinSerLPC40xxp		\ polled serial line driver
include %AppDir%\delays			\ software delays
include %CpuDir%\Drivers\rebootLPC17xx	\ reboot using watchdog
\ *]

synonym ser-emit emit
\ *G A synonym required by the Xmodem code.
synonym ser-key? key?
\ *G A synonym required by the Xmodem code.
synonym ser-key  key
\ *G A synonym required by the Xmodem code.

\ *[
include %AppDir%\MinXmodemRx		\ Xmodem receiver
include %HwDir%\IAP40xx			\ IAP access
include %HwDir%\FlashTables		\ Flash sector tables
include %AppDir%\ReprogApp		\ Application
\ *]


\ *******
\ The end
\ *******

make-turnkey ReprogFlash		\ start up action
\ *G Define the action of the reprogramming code.
' CopyFlash CLD_CopyFlash !
\ *G Define the action of the Flash copy code.

decimal

FINIS


\ ******
\ *> ###
\ ******
