\ NuTiny120.ctl - standalone Forth for Nuvoton NUC120LE3AN

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20110706 MPE001 First release.
))


\ ============
\ *! nuc120le3
\ ============

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if] +xrefs [then]	\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb-2 processor type and register usage
  Hex-I32			\ produce Intel Hex-32 obj format
  +LongCalls			\ permit standalone Forth to handle far calls

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N NUC140 definitions
\ =====================

#128 kb equ /NUCflash	\ -- u
\ *G Define the size of the internal Flash. This is used
\ ** to control the use of  the Flash sector table.

#48 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

#12 Mhz equ xtal-speed	\ -- hz
\ *G Master external oscillator crystal clock rate in HZ.
#12 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
#32768 equ xrtc-speed	\ -- hz
\ *G External RTC crystal speed.
#22118400 equ ihso-speed	\ -- hz
\ *G Frequency of internal high speed oscillator. This is trimmed
\ ** to within 1% for normal temperatures.
#10 kHz equ ilso-speed	\ -- hz
\ *G Frequency of internal low speed oscillator.

system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $1000:0000..$1000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0000:0000.

  $0000:0000 equ link-address	\ Address at which code will start\be linked at
  $0000:0000 $0001:EFFF cdata section NUC120	\ code section in boot Flash
  $2000:0000 $2000:0FFF idata section PROGd	\ 4k IDATA
  $2000:1000 $2000:1FFF udata section PROGu	\ 4k UDATA

interpreter
: prog NUC120  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0F0 equ UP-SIZE		\ size of each task's user area
$0F0 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.
0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Port 0 is the on-chip UART. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\   true to include semaphores
   0 equ test-multi?		\   true to include test code
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM0Def		\ Cortex-M0 generic equates and SFRs
  include %CpuDir%/sfrNUC1xx		\ LPC11xx special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startNUC12x		\ NUC14x specific startup code
  include %CpuDir%/CodeM0M1		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootNUC1xx \ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
\  include %CpuDir%/FaultCortex		\ fault handlers - needs serial
\  include %CommonDir%/CQueues		\ character queues
\  include %CpuDir%/Drivers/serNUC1xxi \ queued RX, polled TXinterrupt driver
  include %CpuDir%/Drivers/serNUC1xxp	\ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
\  include %CpuDir%/LocalCM0		\ local variables

tasking? [if]
  include %CpuDir%/MultiCM0		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickCM0
  ' start-timers AtCold
[else]
\  include %CommonDir%/Delays		\ time delays
\  include %CpuDir%/Drivers/SysTickCM0
\  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/Sfp32CM0		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile nutiny120.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M0"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

: delGPIO	\ --
  1000000 0 do loop
;

: testGPIO	\ -- ; flash LED on PB0
  $03 _GCR GPB_MFP + bic!		\ select PB0/1 to GPIO mode
  $03 _GPIOB GPIOn_PMD + bic!		\ select PB0 to output
  $01 _GPIOB GPIOn_PMD + or!		\ select PB0 to output
  begin
    1 _GPIOB GPIOn_DOUT + bic!		\ set low
    delGPIO
    1 _GPIOB GPIOn_DOUT + or!		\ set high
    delGPIO
  again
;
' testGPIO AtCold


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\benchmarks\xbench32	\ performance tests

\ include %ExampleDir%\tester		\ John Hayes' ANS test engine
\ include %ExampleDir%\5-1-a		\ MPE test engine
\ include %ExampleDir%\5-1-b		\ MPE tests

\ include %ExampleDir%\orbits		\ Sun-Earth-Moon simulation
\ include %ExampleDir%\bench		\ Sieve of Eratosthenes
\ include %ExampleDir%\str_loc		\ structures and local variables
\ include %ExampleDir%\checksum		\ checksum / crc
\ include %ExampleDir%\buildno		\ build number

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

\ checksum the interrupt vectors
ExcVecs 0				\ -- addr csum
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $14 + @ +
  over $18 + @ +
  negate
swap $1C + !

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======

