;/**************************************************************************//**
; * @file     startup_NUC1xx.s
; * @brief    CMSIS Cortex-M0 Core Device Startup File
; *           for the Nuvoton NUC1xx Device Series
; * @version  V1.02
; * @date     25. May 2011
; *------- <<< Use Configuration Wizard in Context Menu >>> ------------------
; *
; * @note
; * Copyright (C) 2010 ARM Limited. All rights reserved.
; *
; * @par
; * ARM Limited (ARM) is supplying this software for use with Cortex-M 
; * processor based microcontrollers.  This file can be freely distributed 
; * within development tools that are supporting such ARM based processors. 
; *
; * @par
; * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
; * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
; * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
; * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
; * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
; *
; ******************************************************************************/


; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000200

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
                DCD     BOD_IRQHandler            ; 16+ 0: Brownout low voltage detected interrupt
                DCD     WDT_IRQHandler            ; 16+ 1: Watch Dog Timer interrupt
                DCD     EINT0_IRQHandler          ; 16+ 2: External signal interrupt from PB.14 pin
                DCD     EINT1_IRQHandler          ; 16+ 3: External signal interrupt from PB.15 pin
                DCD     GPAB_IRQHandler           ; 16+ 4: External signal interrupt from PA[15:0] / PB[13:0]
                DCD     GPCDE_IRQHandler          ; 16+ 5: External interrupt from PC[15:0]/PD[15:0]/PE[15:0]
                DCD     PWMA_IRQHandler           ; 16+ 6: PWM0 or PWM2 interrupt
                DCD     PWMB_IRQHandler           ; 16+ 7: PWM1 or PWM3 interrupt
                DCD     TMR0_IRQHandler           ; 16+ 8: Timer 0 interrupt
                DCD     TMR1_IRQHandler           ; 16+ 9: Timer 1 interrupt
                DCD     TMR2_IRQHandler           ; 16+10: Timer 2 interrupt
                DCD     TMR3_IRQHandler           ; 16+11: Timer 3 interrupt
                DCD     UART02_IRQHandler         ; 16+12: UART0 interrupt
                DCD     UART1_IRQHandler          ; 16+13: UART1 interrupt
                DCD     SPI0_IRQHandler           ; 16+14: SPI0 interrupt
                DCD     SPI1_IRQHandler           ; 16+15: SPI1 interrupt
                DCD     SPI2_IRQHandler           ; 16+16: SPI2 interrupt
                DCD     SPI3_IRQHandler           ; 16+17: SPI3 interrupt
                DCD     I2C0_IRQHandler           ; 16+18: I2C0 interrupt
                DCD     I2C1_IRQHandler           ; 16+19: I2C1 interrupt
                DCD     CAN0_IRQHandler           ; 16+20: CAN0 interrupt
                DCD     0                         ; 16+21: Reserved
                DCD     0                         ; 16+22: Reserved
                DCD     USB_IRQHandler            ; 16+23: USB FS Device interrupt
                DCD     PS2_IRQHandler            ; 16+24: PS2 interrupt
                DCD     ACMP_IRQHandler           ; 16+25: Analog Comparator-0 or Comaprator-1 interrupt
                DCD     PDMA_IRQHandler           ; 16+26: PDMA interrupt
                DCD     I2S_IRQHandler            ; 16+27: I2S interrupr
                DCD     PWRWU_IRQHandler          ; 16+28: Clock controller interrupt for chip wake up from power-down state
                DCD     ADC_IRQHandler            ; 16+29: ADC0/1 interrupt
                DCD     0                         ; 16+30: Reserved
                DCD     RTC_IRQHandler            ; 16+31: Real time clock interrupt


                AREA    |.text|, CODE, READONLY


; Reset Handler

Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                IMPORT  SystemInit
                IMPORT  __main
                LDR     R0, =SystemInit
                BLX     R0
                LDR     R0, =__main
                BX      R0
                ENDP


; Dummy Exception Handlers (infinite loops which can be modified)                

NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC

                EXPORT  BOD_IRQHandler            [WEAK]
                EXPORT  WDT_IRQHandler            [WEAK]
                EXPORT  EINT0_IRQHandler          [WEAK]
                EXPORT  EINT1_IRQHandler          [WEAK]
                EXPORT  GPAB_IRQHandler           [WEAK]
                EXPORT  GPCDE_IRQHandler          [WEAK]
                EXPORT  PWMA_IRQHandler           [WEAK]
                EXPORT  PWMB_IRQHandler           [WEAK]
                EXPORT  TMR0_IRQHandler           [WEAK]
                EXPORT  TMR1_IRQHandler           [WEAK]
                EXPORT  TMR2_IRQHandler           [WEAK]
                EXPORT  TMR3_IRQHandler           [WEAK]
                EXPORT  UART02_IRQHandler         [WEAK]
                EXPORT  UART1_IRQHandler          [WEAK]
                EXPORT  SPI0_IRQHandler           [WEAK]
                EXPORT  SPI1_IRQHandler           [WEAK]
                EXPORT  SPI2_IRQHandler           [WEAK]
                EXPORT  SPI3_IRQHandler           [WEAK]
                EXPORT  I2C0_IRQHandler           [WEAK]
                EXPORT  I2C1_IRQHandler           [WEAK]
                EXPORT  CAN0_IRQHandler           [WEAK]
                EXPORT  USB_IRQHandler            [WEAK]
                EXPORT  PS2_IRQHandler            [WEAK]
                EXPORT  ACMP_IRQHandler           [WEAK]
                EXPORT  PDMA_IRQHandler           [WEAK]
                EXPORT  I2S_IRQHandler            [WEAK]
                EXPORT  PWRWU_IRQHandler          [WEAK]
                EXPORT  ADC_IRQHandler            [WEAK]
                EXPORT  RTC_IRQHandler            [WEAK]

BOD_IRQHandler
WDT_IRQHandler
EINT0_IRQHandler
EINT1_IRQHandler
GPAB_IRQHandler
GPCDE_IRQHandler
PWMA_IRQHandler
PWMB_IRQHandler
TMR0_IRQHandler
TMR1_IRQHandler
TMR2_IRQHandler
TMR3_IRQHandler
UART02_IRQHandler
UART1_IRQHandler
SPI0_IRQHandler
SPI1_IRQHandler
SPI2_IRQHandler
SPI3_IRQHandler
I2C0_IRQHandler
I2C1_IRQHandler
CAN0_IRQHandler
USB_IRQHandler
PS2_IRQHandler
ACMP_IRQHandler
PDMA_IRQHandler
I2S_IRQHandler
PWRWU_IRQHandler
ADC_IRQHandler
RTC_IRQHandler

                B       .

                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB
                
                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit
                
                ELSE
                
                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap
__user_initial_stackheap

                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END
