\ StartNUC14x.fth - Cortex startup code for Nuvoton NUC14x family

((
Copyright (c) 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20110705 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startnuc1xx
\ *T Cortex-M0 start up for Nuvoton BUC1xx
\ ===========
\ *P The file *\i{Cortex/Hardware/NUC1xx/startNUC1xx.fth} contains
\ ** the start up code for Nuvoton NUC1xx devices. Start up code for other
\ ** Cortex-M0 and M1 implementations can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M0
\ ** is in *\i{Cortex/StartCortex.fth}.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1. *\b{Do not} use this to set the ISP checksum!
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. This
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

internal
: UnProtRegs	\ --
\ *G Unprotect the system registers.
  _GCR REQWRPROT +  $059 over !  $016 over !  $088 swap !
;

: ProtRegs	\ --
\ *G protect the system registers.
  0 _GCR REQWRPROT + !
;

: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLL
\ ** as required.
  UnProtRegs
  _CLK
  $0F over PWRCON + or!			\ enable all four clocks
\  $0000:C22E over PLLCON + !		\ 48 MHz PLL output
\  $3A over CLKSEL0 + !			\ Main clock = xtal
\  $00 swap CLKDIV + !			\ all divide by 1
\  100 0 do loop				\ settling delay
\ The CLKSTATUS register cannot be read on all devices, e.g. NUC140VE3.
\ To use the PLL, the PLLCON register PD bit must be zero on some devices, e.g. NUC140VE33.
\  begin
\    _CLK CLKSTATUS + @ bit2 and
\  until
\  $3A _CLK CLKSEL0 + !			\ Main clock = PLL
;

((
: delGPIO	\ --
  1000000 0 do loop
;

: testGPIO	\ -- ; flash LED on PB0
  $03 _GCR GPB_MFP + bic!		\ select PB0/1 to GPIO mode
  $03 _GPIOB GPIOn_PMD + bic!		\ select PB0 to output
  $01 _GPIOB GPIOn_PMD + or!		\ select PB0 to output
  begin
    1 _GPIOB GPIOn_DOUT + bic!		\ set low
    delGPIO
    1 _GPIOB GPIOn_DOUT + or!		\ set high
    delGPIO
  again
;
))

((
: testUART	\ --
  init-ser
  begin
    [char] A seremit0
  again
;
))

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    setClocks
\    testGPIO
\    testUART
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc11      11        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

