\ serRP2040p.fth - polled serial driver for Raspberry Pico RP2040.

((
Copyright (c) 2023
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)7803 903 612
       +34 649 662 974
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20230703 MPE001 First release  for RP2040
))

only forth definitions  decimal

\ ===============
\ *! serRP2040p
\ *T Rasperry Pico RP2040 polled serial
\ ===============
\ *P The file *\i{Cortex/Drivers/serRP2040p.fth} contains a polled
\ ** serial driver for Raspberry Pico RP2040 and compatible devices.

: +FaultConsole	( -- )  ;
\ *G Because this is a polled driver, *\fo{+FaultConsole} for
\ ** fault handling is a *\fo{NOOP}. See *\i{Cortex\FaultCortex.fth}
\ ** for more details.


\ ********
\ *S UART0
\ ********

useUART0? [if]

system-speed console0-speed 16 * / equ BAUDRATE_INTEGER
\ *G Integer part of baud rate divisor for UART0.
\ ** For 115200 baud at 12 MHz, this is
\ ** floor(12e6/(16*115200)) = 6

system-speed 4 * console0-speed / 63 and equ BAUDRATE_FRACTIONAL
\ *G Fractional part of baud rate divisor for UART0.
\ ** For 115200 baud at 12 MHz, this is
\ ** floor(mod(64*12e6/(16*115200), 64)) = 32

: init-ser0	\ --
\ *G Initialise UART0 on GP0 and GP1.

\ Set baud rate divider

  BAUDRATE_INTEGER    Un.UARTIBRD _UART0 + !
  BAUDRATE_FRACTIONAL Un.UARTFBRD _UART0 + !

\ Select 8N1 and enable FIFOs

  3 5 lshift 1 4 lshift or Un.UARTLCR_H _UART0 + !

\ RXE: Receive enable, TXE: transmit enable, UARTEN: UART enable.

  $301 Un.UARTCR _UART0 + !

\ Select UART special function for GP0 and GP1

  2 iob0.GPIO0_CTRL _DONE + !
  2 iob0.GPIO1_CTRL _DONE + !
;

: serEmit0	\ char --
\ *G *\fo{EMIT} for UART0.
  begin					\ -- char uart
    Un.UARTFR _UART0 + @ BIT5 and 0=	\ TXFF: Transmit FIFO full bit
  until
  Un.UARTDR _UART0 + c!
;

: serKey?0	\ -- flag
\ *G *\fo{KEY?} for UART0.
  Un.UARTFR _UART0 + @ BIT4 and 0=  	\ RXFE: Receive FIFO empty bit
;

Tasking? [if]
: serKey0	\ -- char
\ *G *\fo{KEY} for UART0.
  begin
    serKey?0 invert
  while
    pause
  repeat
  Un.UARTDR _UART0 + c@
;
[else]
: serKey0	\ -- char
\ *G *\fo{KEY} for UART0.
  begin
    serKey?0
  until
  Un.UARTDR _UART0 + c@
;
[then]

: serType0	\ caddr len --
\ *G *\fo{TYPE} for UART0.
  bounds ?do  i c@ serEmit0  loop
;

: serCR0	\ --
\ *G *\fo{CR} for UART0.
  $0D serEmit0  $0A serEmit0
;

create Console0	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for UART0.
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

console-port 0 = [if]
  console0 constant console
\ *G *\fo{CONSOLE} is the device used by the Forth system for interaction.
\ ** It may be changed by one of the phrases of the form:
\ *C   <device>  dup opvec !  ipvec !
\ *C   <device>  SetConsole
[then]

[then]


\ ************************
\ *S System initialisation
\ ************************

: init-ser	\ --
\ *G Initialise all serial ports
  [ useUART0? ] [if]  init-ser0  [then]
  [ useUART1? ] [if]  init-ser1  [then]
;


\ ======
\ *> ###
\ ======

decimal

