\ PWMdisco072.fth - Cortex-M0 PWM for coloured LEDs

((
Copyright (c) 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20141126 MPE001 first cut.
))

only forth definitions
decimal

\ ===========
\ *! pwmdisco072
\ *T PWM on Discovery board LEDs
\ ===========

((
#include "stm32f0xx.h"
#include "core_cm0.h"

void TIM1_BRK_UP_TRG_COM_IRQHandler();                      // ISR prototype

int main() {
    RCC->AHBENR |= RCC_AHBENR_GPIOAEN;                      // Enable GPIOA clock
    GPIOA->MODER |= GPIO_MODER_MODER8_1;                    // Pin A8 mode = Alt Function
    GPIOA->AFR[1] |= 0b0010;                                // Pin A8 alt function = AF2 (TIM1)
    GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR8;               // Pin A8 speed = 50MHz

    RCC->APB2ENR |= RCC_APB2ENR_TIM1EN;                     // Enable clock
    TIM1->PSC = 23;                                         // Prescaler = 23 (1 tick per half microsecond)
    TIM1->ARR = 19999;                                      // Auto-reload = PWM frequency in half microseconds = 10ms period
    TIM1->CCMR1 |= TIM_CCMR1_OC1M_1 | TIM_CCMR1_OC1M_2;     // OC1M = 110 for PWM Mode 1 output on ch1
    TIM1->CCMR1 |= TIM_CCMR1_OC1PE;                         // Output 1 preload enable
    TIM1->CR1 |= TIM_CR1_ARPE;                              // Auto-reload preload enable
    TIM1->CCER |= TIM_CCER_CC1E;                            // Enable output for ch1
    TIM1->CCR1 = 1800;                                      // CCR1 = Duty cycle in half microseconds = high for 900us
    TIM1->EGR |= TIM_EGR_UG;                                // Force update
    TIM1->SR &= ~TIM_SR_UIF;                                // Clear the update flag
    TIM1->DIER |= TIM_DIER_UIE;                             // Enable interrupt on update event
    NVIC_EnableIRQ(TIM1_BRK_UP_TRG_COM_IRQn);               // Enable IRQ
    TIM1->CR1 |= TIM_CR1_CEN;                               // Enable counter

    while(1) {
    }
}

void TIM1_BRK_UP_TRG_COM_IRQHandler() {
    if(TIM1->SR & TIM_SR_UIF != 0) {                        // If update flag is set
        // do stuff here if needed
    }
    TIM1->SR &= ~TIM_SR_UIF;                                // Interrupt has been handled
}

Too simple: when using the advanced timer you must set the
MOE main output enable bit. Turns out I wasn't using an
advanced timer on my F4's, and made a poor assumption when
setting up PWM on my F0 :)
))

((
PC 6 pio: LDU	\ -- struct ; LD3 on TIM3_CH1 AF0
\ *G Red/Upper LED.
PC 9 pio: LDR	\ -- struct ; LD5 on TIM3_CH4 AF0
\ *G Green/Right LED.
PC 7 pio: LDD	\ -- struct ; LD6 on TIM3_CH2 AF0
\ *G Blue/Down LED.
PC 8 pio: LDL	\ -- struct ; LD4 on TIM3_CH3 AF0
\ *G Orange/Left LED.
))

4096 value PWMmax	\ -- u
\ *G Assume a 12 bit PWM (0..4095)
100 value PWMHz		\ -- Hz
\ *G Desired repetition rate.

: initPWM	\ --
\ *G Initialise PWM channels
  0 LDU isFunction			\ controlled by Timer
  0 LDR isFunction
  0 LDD isFunction
  0 LDL isFunction
  $0000:0002 _RCC  2dup rccAPB1en + or!	\ TIM3 enabled
  rccAPB1rst + bic!
  _TIM3
  409
;

\ ======
\ *> ###
\ ======

decimal

