\ STM32F072disco.ctl - STM32F072RB discovery board standalone Forth

((
Copyright (c) 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140205 MPE001 First release for STM32F0xx.
))


\ ================
\ *! stm32f072disco
\ *T Control file for STM32F072 Discovery board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32F0/STM32F072disco.ctl}
\ ** produces a standalone Forth for the STM32F072 Discovery board.
\ ** The serial port is connected to USART1 on pins PA9 (Tx) and
\ ** PA10 (Rx).
\ ** You cannot use the easy GPIO code for UART initialisation unless
\ ** the GPIO ports are clocked and taken out of reset in the start
\ ** up code.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

\ *P You can easily use the low-cost ST-LINK/V2 JTAG unit.
\ ** The ST-LINK Utility software is a
\ ** free download from the ST website *\f{www.st.com}.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

1 constant MPE?
0 constant XYZ?

MPE? [if]	\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives
c" C:/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
mc" %ExampleDir%/FATfiler"			\ where the FAT file system lives
                setmacro FATfiler
[then]

XYZ? [if]
  ...
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
  UF2-format			\ also produce UF2 file
\  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N STM32F0 variant definitions
\ ======================

$0800:0000 equ FlashBase	\ -- addr
\ *G Start address of Flash. The bottom 2kb (the vector area) is
\ ** mirrored at $0000:0000 for booting.

#128 kb equ /Flash	\ -- len
\ *G Size of Flash.
2 kb equ /FlashPage	\ -- len
\ *G Size of a Flash Page.
1 equ KeepPages		\ -- u
\ *G Set this non-zero for the number of pages at the end of Flash
\ ** that are reserved for configuration data. Often set to 1 or 2
\ ** by systems that use PowerNet.
FlashBase /Flash + /FlashPage KeepPages * - equ CfgFlash  \ -- len
\ *G Base address of the configuration Flash area.

$1FFF:C800 equ /InfoBase	\ -- addr
\ *G Base address of system memory information block.

#12 kb equ /SysMem		\ -- len
\ *G Size of system memory block.

$1FFF:F800 equ OptionBytes	\ -- addr
\ *G Base address of option bytes
#16 equ /OptionBytes		\ -- len
\ *G Number of option bytes

#64 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system clocks are generated from PLLs.
\ ** How to set them up is non-obvious and is mostly documented
\ ** by the ST demonstration code in the file *\i{system_stm32f0x2.c}.
\ ** If you are not going to use an existing setup, copy and rename
\ ** one of the existing *\i{startSTM32F0xx.fth} files.

8 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ. This is the HSI
\ ** internal oscillator which has better than +/-1% accuracy and
\ ** is more accurate than the internal 48 MHz oscillator.
48 MHz equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 48 MHz.
1 equ APBdiv	\ -- u
\ *G Division ratio of the APB clock from the system clock.
\ ** This may not be more than 48 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APBdiv / equ APB-speed	\ -- hz
\ *G APB bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2kb (the
\ ** vector area) is mirrored at $0000:0000.

  $0800:0000 $0801:DFFF cdata section STM32F072disco	\ code section in boot Flash
  $2000:0000 $2000:1FFF idata section PROGd	\ 4k IDATA
  $2000:2000 $2000:3FFF udata section PROGu	\ 12k UDATA

interpreter
: prog STM32F072disco  ;		\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================
\ *P These equates calculate how much RAM is needed for the USER area,
\ ** return stack and data stack of each task, plus an area used
\ ** by Forth interrupts that share a common USER (and optionally)
\ ** common stacks. This can be done to reduce the overall amount
\ ** of RAM needed for interrupt stacks. Since interrupts can occur
\ ** in any task, switching stacks for high level interrupts will
\ ** reduce the total RAM requirements.

$0F0 equ UP-SIZE		\ size of each task's user area
$0F0 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#115200 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUSART4?	\ -- n
\ *G Set non-zero to compile code for USART4,
\ ** device *\fo{Console4}.
115200 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

1 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..4 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM0def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F072		\ STM32F072 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F072	\ start up code
  include %CpuDir%/CodeM0M1		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootSTM32	\ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
\  include %CommonDir%/CQueues		\ character queues
\  include %CpuDir%/Drivers/serSTM32F0xxqi \ RX/TX queued interrupt driver
: selio-ser1	\ --
\ *G Example to perform clock and pin selection for USART1 on PA9/10.
\ ** You cannot use the easy GPIO code for this unless the GPIO ports
\ ** are clocked and taken out of reset in the start up code.
\ Clocks and reset
  _RCC
  $0000 $0003 2 pick rccCFGR3 + setMask	\ PCLK is UART clock
  bit17 over rccAHBen + or!		\ enable clock to GPIOA
  bit17 over rccAHBrst + bic!		\ take it out of reset
  bit14 over rccAPB2en + or!		\ enable clock to USART1
  bit14 over rccAPB2rst + bic!		\ take it out of reset
  drop
\ Default on PA9/10
  _GPIOA >r
  $0028:0000 $003C:0000 r@ gpioMODER + setMask	\ alternate function on 9/10
  $0000:0110 $0000:0FF0 r> gpioAFRH + setMask	\ AF1 for both
;
  include %CpuDir%/Drivers/serSTM32F0xxp \ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM0		\ local variables

tasking? [if]
  include %CpuDir%/MultiCM0		\ multi-tasker
[else]
  : pause  ;
[then]

  include %CpuDir%/drivers/gpioSTM32F0xx \ easy pin access
  include %CommonDir%/Delays
  include %CpuDir%/Drivers/SysTickDisco072
\  include %HwDir%/SysTickDisco072
  ' start-clock AtCold

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32CM0		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]


\ *************
\ *S End of kernel
\ *************

buildfile STM32F072disco.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M0"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%/RomForth/iodef	\ link i/o
  include %CommonDir%/RomForth/textfile	\ AIDE textfile support

  include %HwDir%/FlashProg		\ ISP programming
  include %HwDir%/ReProg/ReFlash	\ ReFlash utility
\  include %CpuDir%/drivers/rtcSTM32	\ RTC driver
  include %CpuDir%/Drivers/spiSTM32F0hard \ SPI2 driver
  include %HwDir%/l3gd20		\ L3GD20 MEMS driver

  include %CommonDir%/DebugChan		\ AIDE aux display handler


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%/benchmarks/xbench32	\ performance tests

\ include %CommonDir%/TESTS/ANS/tester		\ John Hayes' ANS test engine
\ include %CommonDir%/TESTS/MPE/5-1-a		\ MPE test engine
\ include %ExampleDir%/TESTS/MPE/5-1-b		\ MPE tests

\ include %CommonDir%/TESTS/MPE/orbits		\ Sun-Earth-Moon simulation
\ include %CommonDir%/TESTS/MPE/bench		\ Sieve of Eratosthenes
\ include %CommonDir%/TESTS/MPE/str_loc		\ structures and local variables
\ include %CommonDir%/TESTS/MPE/checksum	\ checksum / crc
\ include %CommonDir%/TESTS/MPE/buildno		\ build number

\ include %CommonDir%/TESTS/MPE/testtask
\ include %CommonDir%/TESTS/MPE/testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

decimal

update-build

afterwards sh "C:\MyApps\ST-Linkv4.0.0\ST-LINK Utility\ST-LINK_CLI.exe"  -P STM32F072DISCO.hex  -Rst

FINIS


\ ======
\ *> ###
\ ======
