\ SysTickDisco072.fth - Cortex-M0 ticker for STM32F072B-DISCO board

((
Copyright (c) 2010, 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140207 SFP003 Converted for STM32F072B-DISCO board.
20110218 MPE002 Added timebase extensions.
))

only forth definitions
decimal

\ ===========
\ *! systickdisco072
\ *T System Ticker
\ ===========
\ *P The system ticker uses the Cortex-M0 *\i{SysTick} timer.
\ ** Although the *\i{SysTick} timer is common to all Cortex-M0
\ ** devices, there is no guarantee that they have the same
\ ** clocking arrangements. Consequently there are different
\ ** drivers for different implementations.

\ *P The STM32 implementation is fed with the system clock and
\ ** has an option to divide it by 8, controlled by bit 2 of the
\ ** SysTick Control and Status Register (ARM DDI 0337, Rev E,
\ ** page 8-8).
\ *(
\ *B Bit2=0: SysTick clock is HCLK/8.
\ *B Bit2=1: SysTick clock is HCLK.
\ *)
\ *P This code always uses HCLK/8.


1 value LedActive?	\ -- flag
\ *P If *\fo{LedActive?} returns non-zero, the ticker is used
\ ** to produce a rotating pattern on the four LEDs.

[undefined] ticks [if]
0 value ticks	\ -- ms
\ *G A counter that returns a 32 bit number of milliseconds that
\ ** eventually wraps.
[then]

PC 6 pio: LDU	\ -- struct
\ *G Red/Upper LED.
PC 9 pio: LDR	\ -- struct
\ *G Green/Right LED.
PC 7 pio: LDD	\ -- struct
\ *G Blue/Down LED.
PC 8 pio: LDL	\ -- struct
\ *G Orange/Left LED.

: SysTicker	\ --
\ *G Ticker ISR action.
  tick-ms addr ticks +!			\ update ticker
  LedActive? if
    case  ticks $7FF and		\ 0..2047
      $000 of  LDL clrPin  LDU setPin  endof
      $200 of  LDU clrPin  LDR setPin  endof
      $400 of  LDR clrPin  LDD setPin  endof
      $600 of  LDD clrPin  LDL setPin  endof
    endcase
  then
;
' SysTicker SysTickvec# EXC: SysTickISR	\ -- addr
\ *G Setting the high level ISR.

tick-ms system-speed 8 / #1000 */ 1- equ /SysTick	\ -- x
\ +G Initial value of the SysTick Reload register.

internal
: initLD	\ struct --
  dup isOutput setPin
;
external

: start-clock	\ --
\ *G Initialise the system ticker to run with a period of
\ ** *\fo{tick-ms} milliseconds.
  LDU initLD  LDR initLD  LDD initLD  LDL initLD
  _SCS
  $C000:0000 $FF00:0000 2 pick scsSHPR3 + setMask \ lowest priority interrupt
  /SysTick over stRVR + !		\ set ticker rate
  0 over stCVR + !			\ clear current value
  $0003 over stCSR + !			\ enable it
\ bit 0 = enable, bit 1 = interrupt, bit 2 = clock source (1=sysclk, 0=sysclk/8)
  drop
  ei
;

: stop-clock	\ --
\ *G Stop the sytem ticker.
  bit0 _SCS stCSR + bic!  ;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]


\ ======
\ *> ###
\ ======

decimal

