\ Mcp23S17.fth - read and write I/O pins.

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

The minimum procedure to get it working as a simple output port is;
1. Reset
2. Send SPI bytes 0x40,0x0A,0xA0 (configures IOCON reg)
3. Send SPI bytes 0x40,0x10,0x00 (set PORT1 direction all outs)
4. Send SPI bytes 0x40,0x1A (sets PORT1 address ready to receive a byte)
5. (Now every SPI byte sent just writes direct to PORT1 outputs)


To do
=====

Change history
==============
20120508 SFP001 First cut.
))

only forth definitions
decimal

\ ===========
\ *! mcp23s17
\ *T MCP23S17 I/O Expander.
\ ===========
\ *P The MCP23S17 I/O expander is an SPI device that provides two
\ ** 8-bit I/O ports. In this code, the extended addressing feature
\ ** is used so that up to eight devices can be used with the same chip
\ ** select line. The IOCON.BANK signal is left at the power up
\ ** value of 0.

\ *P The MCP23S17 errata sheet notes that if A2=1, the extended
\ ** addressing *\b{must} be used.

\ *P This code uses the low-level SPI driver in
\ ** *\i{Cortex/Hardware/STM32/spiSTM32-P107-8.fth}.

\ I/O addresses for BANK=0
$00 equ rIODIRA
$01 equ rIODIRB
$02 equ rIPOLA
$03 equ rIPOLB
$04 equ rGPINTENA
$05 equ rGPINTENB
$06 equ rDEFVALA
$07 equ rDEFVALB
$08 equ rINTCONA
$09 equ rINTCONB
$0A equ rIOCON
\ $0B equ rIOCON
$0C equ rGPPUA
$0D equ rGPPUB
$0E equ rINTFA
$0F equ rINTFB
$10 equ rINTCAPA
$11 equ rINTCAPB
$12 equ rGPIOA
$13 equ rGPIOB
$14 equ rOLATA
$15 equ rOLATB

0 value CurrS17	\ -- u ; 0..7
\ *G Holds the current device. Set by you as required.

: read8-S17	\ reg -- b
\ *G Read 8 bits from the register.
  [spi3
    $41 CurrS17 1 lshift or writeSPI3	\ select read
    writeSPI3				\ register
    readSPI3				\ data
  spi3]
;

: write8-S17	\ b reg --
\ *G Write 8 bits to the register.
  [spi3
     $40 CurrS17 1 lshift or writeSPI3	\ select write
     writeSPI3				\ register
     writeSPI3				\ data
  spi3]
;

: resetSPI	\ --
\ *G The SPI bus must be reset after initialising the SPI driver
\ ** and before using the bus. This avoids transients upsetting
\ ** the MCP23S17 device.
  rst_low  20 ms  rst_high 20 ms
;

: initS17	\ --
\ *G Initialise the current MCP23S17 device.
  $28 rIOCON write8-S17			\ SEQOP=1, HAEN=1
  $00 rIODIRA write8-S17		\ port A all outputs
  $FF rIODIRB write8-S17		\ port B all inputs
;

: initS17all	\ --
\ *G Initialise all MCP23S17s that may be on the bus.
  CurrS17  8 0
  do  i to CurrS17  initS17  loop
  to CurrS17
;


\ ********
\ *S Tests
\ ********

1 [if]

2 value msdelay

: tr		\ reg --
  $55 over write8-s17  read8-s17 .byte space
;

: trall		\ --
  cr  $15 0 do  i tr  loop  ;

: trax		\ --
  CurrS17  8 0 do
    cr ." Dev=" i .  i to CurrS17  trall
  loop
  to CurrS17
;

: ta		\ -- ; waggle port A
  0 to CurrS17
  begin
    $00 rGPIOA write8-S17  msdelay ms
    $FF rGPIOA write8-S17  msdelay ms
    key?
  until
;

: tb		\ -- ; waggle port A
  0 to CurrS17
  begin
    $55 rGPIOA write8-S17  200 ms
    $AA rGPIOA write8-S17  200 ms
    key?
  until
;

: tc		\ -- ; waggle port A
  begin
    256 0 do
      i rGPIOA write8-s17
      key? ?leave
      400 ms
    loop
    key?
  until
;


\ ======
\ *> ###
\ ======

decimal

