\ STM32-P107.ctl - Olimex STM32-P107 board standalone Forth

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20120320 MPE002 STM32-P107 conversion
20091223 MPE001 First release for STM32F1xx.
))


\ ================
\ *! stm32p107
\ *T Control file for Olimex STM32-P107 board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32/STM32-P107.ctl}
\ ** produces a standalone Forth for the Olimex STM32-P107 board.
\ ** The serial port is connected to USART2 on pins PD5 (Tx) and
\ ** PD6 (Rx).

\ *P This target code was tested on a Rev B board. Note that the
\ ** manual may contain the Rev A schematic. Be sure to download
\ ** the correct schematic from the Olimex web site.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

\ *P You can easily use the low-cost ST-LINK/V2 JTAG unit.
\ ** The ST-LINK Utility software is a
\ ** free download from the ST website *\f{www.st.com}.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

1 constant MPE?
0 constant XYZ?

MPE? [if]	\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Lib"
		setmacro LibDir		\ Library code
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives
mc" %LibDir%/PowerNet"
		setmacro IpStack	\ where PowerNet lives
mc" %LibDir%/FATfiler"			\ where the FAT file system lives
                setmacro FATfiler
[then]

XYZ? [if]
  ...
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M3			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N STM32F1 variant definitions
\ ======================

$0800:0000 equ FlashBase	\ -- addr
\ *G Start address of Flash. The bottom 2kb (the vector area) is
\ ** mirrored at $0000:0000 for booting.

#256 kb equ /Flash	\ -- len
\ *G Size of Flash.
2 kb equ /FlashPage	\ -- len
\ *G Size of a Flash Page.
\ *(
\ *B 1kb for low-density devices (up to 128kb)
\ *B 1kb for medium-density devices (up to 256kb)
\ *B 2kb for high high-density devices (up to 512k)
\ *B 2kb for connectivity line devices (up to 256kb)
\ *)
1 equ KeepPages		\ -- u
\ *G Set this non-zero for the number of pages at the end of Flash
\ ** that are reserved for configuration data. Often set to 1 or 2
\ ** by systems that use PowerNet.
FlashBase /Flash + /FlashPage KeepPages * - equ CfgFlash  \ -- len
\ *G Base address of the configuration Flash area.

$1FFF:B000 equ /InfoBase	\ -- addr
\ *G Base address of system memory information block.
\ *(
\ *B $1FFF:F000 for low/med/high-density devices,
\ *B $1FFF:B000 for connectivity line devices.
\ *)
#18 kb equ /SysMem		\ -- len
\ *G Size of system memory block.
\ *(
\ *B 2kb for low/med/high-density devices,
\ *B 18kb for connectivity line devices.
\ *)

$1FFF:F800 equ OptionBytes	\ -- addr
\ *G Base address of option bytes
#16 equ /OptionBytes		\ -- len
\ *G Number of option bytes

#128 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system, Ethernet and USB clocks are generated from PLLs.
\ ** How to set them up is non-obvious and is mostly documented
\ ** by the ST demonstration code in the file *\i{system_stm32f10x.c}.
\ ** If you are not going to use an existing setup, copy and rename
\ ** one of the existing *\i{startSTM32F1xx.fth} files.

25 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
72 MHz equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 72 MHz.
1 equ APB2div	\ -- u
\ *G Division ratio of the APB2 clock from the system clock.
\ ** This may not be more than 72 MHz.
2 equ APB1div	\ -- u
\ *G Division ratio of the APB1 clock from the system clock.
\ ** This may not be more than 36 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APB2div / equ APB2-speed	\ -- hz
\ *G APB2 bus speed.
system-speed APB1div / equ APB1-speed	\ -- hz
\ *G APB1 bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2kb (the
\ ** vector area) is mirrored at $0000:0000.

\  $0800:0000 equ link-address	\ Address at which code will start\be linked at
  $0800:0000 $0807:FFFF cdata section STM32P107	\ code section in boot Flash
  $2000:0000 $2000:0FFF idata section PROGd	\ 4k IDATA
  $2000:1000 $2000:FFFF udata section PROGu	\ 60k UDATA

interpreter
: prog STM32P107  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

0 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

1 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#115200 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

2 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..6 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $8000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM3Def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F10xConn	\ STM32F105/7 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F107	\ STM32F107 startup code
1 equ FastCmove?
  include %CpuDir%/CodeCortex		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootSTM32	\ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
  include %CommonDir%/CQueues		\ character queues
  include %CpuDir%/Drivers/serSTM32F10xqi \ RX/TX queued interrupt driver
\  include %CpuDir%/Drivers/serSTM32F10xp \ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM3		\ local variables

tasking? [if]
  include %CpuDir%/MultiCortex		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before SysTickxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickSTM32
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays
  include %CpuDir%/Drivers/SysTickSTM32
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]


\ *************
\ *S End of kernel
\ *************

buildfile STM32P107.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M3"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  include %CpuDir%/drivers/gpioSTM32	\ easy pin access
  include %HwDir%/FlashProg		\ ISP programming
  include %HwDir%/ReProg/ReFlash	\ ReFlash utility
  include %CpuDir%/drivers/rtcSTM32	\ RTC driver
\  include %HwDir%/spiSTM32-P107-9bb	\ SPI to UEXT NOKIA display
\  include %HwDir%/spiSTM32-P107-8	\ SPI to UEXT MOD-LGS

1 equ FATtest?
  include %HwDir%/spiSD-P107dma		\ SPI3/DMA SD card primitives
  include %FATfiler%/spiSDgen.bld	\ FAT file system
  include %FATfiler%/DelDirs		\ Disk erase code
  include %HwDir%/Mcp23S17-SD		\ SPI to UEXT MOD-LGS

((
  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
\  include %CommonDir%/XmodemTxRx	\ Xmodem driver
  include %CommonDir%/RomForth/iodef	\ link i/o
  include %CommonDir%/RomForth/textfile	\ AIDE textfile support
))

PowerNet? [if]
1 equ CPU=Cortex			\ to compile Cortex specific code
1 equ RMII?				\ PHY is in RMII mode
  include %AppDir%/P107PNcfg		\ PowerNet configuration
  include %IpStack%/NetCode		\ network order stuff
  include %AppDir%/WebConfig		\ configuration tools
  include %CpuDir%/drivers/EtherSTM32F10x \ Ethernet driver
  include %IpStack%/PowerNet.bld	\ PowerNet build
  include %IpStack%/Examples/WebPost	\ application update
[else]
(( \ for testing
1 equ XC?
1 equ CPU=Cortex			\ to compile Cortex specific code
1 equ EtherDiags?
1 equ RMII?
1 equ Sniff?
1 equ Pingit?
  include %ExampleDir%/NetCode		\ network order stuff
  include %CpuDir%/drivers/EtherSTM32F10x \ Ethernet driver
))
[then]


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%/benchmarks/xbench32	\ performance tests

\ include %CommonDir%/TESTS/ANS/tester		\ John Hayes' ANS test engine
\ include %CommonDir%/TESTS/MPE/5-1-a		\ MPE test engine
\ include %ExampleDir%/TESTS/MPE/5-1-b		\ MPE tests

\ include %CommonDir%/TESTS/MPE/orbits		\ Sun-Earth-Moon simulation
\ include %CommonDir%/TESTS/MPE/bench		\ Sieve of Eratosthenes
\ include %CommonDir%/TESTS/MPE/str_loc		\ structures and local variables
\ include %CommonDir%/TESTS/MPE/checksum	\ checksum / crc
\ include %CommonDir%/TESTS/MPE/buildno		\ build number

\ include %CommonDir%/TESTS/MPE/testtask
\ include %CommonDir%/TESTS/MPE/testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======
