\ hackS17.fth - 8 bit bit-banged SPI driver for MCP23S17

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20120508 SFP001 Started.
))

only forth definitions
decimal

\ ================
\ *! hacks17
\ *T SPI Driver for Olimex STM32-P107 board
\ ================
\ *P This code was tested on an Olimex STM32-P107 board which uses
\ ** an STM32F107VCT6 CPU in LQFP 100 pin format. The SPI I/O is
\ ** attached to the UEXT connector.

\ *E STM32-P107 UEXT Connector
\ ** Pin  Port  Function     Pin  Port  Function
\ ** 1          3v3          2          0v
\ ** 3    PD8   USART3 Tx    4    PD9   USART3 Rx
\ ** 5    PB8   I2C1 SCL     6    PB9   I2C1 SDA
\ ** 7    PC11  SPI3 MISO    8    PC12  SPI3 MOSI
\ ** 9    PC10  SPI3 SCK     10   PB15  CS_UEXT


\ ****************
\ *S SPI interface
\ ****************
\ *P This code requires *\i{Cortex/Drivers/gpioSTM32.fth}. All
\ ** GPIO ports are clocked and enabled at startup.

3 8 pio: SPI_RST	\ PD8 GPIO output
2 11 pio: MISO3		\ PC11 AF input
2 12 pio: MOSI3		\ PC12 AF output 50MHz - may be >10MHz
2 10 pio: SCK3		\ PC10 AF output 50MHz - may be >10MHz
1 15 pio: nCSUEXT	\ PB15 GPIO output 10MHz

200 value /spiDelay	\ -- x
\ *G Value used for SPI half clock delay. The loop takes about
\ ** 5 clocks per iteration on a Cortex-M3 and 6 on an ARM.

: spiDelay	\ --
\ *G Wait half an SPI clock.
  /spiDelay begin 1- dup 0=  until  drop
;

: sck_low	\ --
\ *G Set SCK pin low.
  SCK3 clrPin
;

: sck_high      \ --
\ Set SCK pin high.
  SCK3 setPin
;

: mosi_low	\ --
\ *G Set MOSI pin low.
  MOSI3 clrPin
;

: mosi_high	\ --
\ *G Set MOSI pin high.
  MOSI3 setPin
;

: ss_low	\ --
\ *G Set SPI CS pin low.
  nCSUEXT clrPin
;

: ss_high	\ --
\ *G Set SPI CS pin high.
  nCSUEXT setPin
;

: rst_low	\ --
\ *G Set LCD RST pin low.
  SPI_RST clrPin
;

: rst_high	\ --
\ *G Set LCD RST pin high.
  SPI_RST setPin
;

: miso@		\ -- 0/1
\ *G Read MISO pin state.
  MISO3 getPin
;

: read_spi8	\ -- b ; msb first
\ *G Read 8 bits from the SPI bus. CPOL=0, CPHA=0.
  0  8 0 do
    1 lshift
    spiDelay  miso@ or  sck_high  spiDelay  sck_low
  loop
;

: write_spi8	\ b -- ; msb first
\ *G Write 8 bits to the SPI bus. CPOL=0, CPHA=0
  8 0 do
    dup $80 and
    if  mosi_high  else  mosi_low  endif
    spiDelay  sck_high  spiDelay  sck_low
    1 lshift
  loop
  drop
;

: initUEXTpins	\ --
\ *G Initialise the pins. CPOL=0, CPHA=0.
  SPI_RST isOutput
  MISO3 isInput
  Output10MHz MOSI3 isFunction
  Output10MHz SCK3 isFunction
  Output10MHz nCSUEXT isFunction
  rst_high  ss_high  sck_low  mosi_low
;
' initUEXTpins AtCold

: [spi3		\ --
\ *G Start a transaction by asserting CS.
  spidelay  ss_low  spidelay
;

: spi3]		\ --
\ *G Finish a transaction by de-asserting CS.
  spidelay  ss_high  spidelay
;


\ I/O addresses for BANK=0
$00 equ rIODIRA
$01 equ rIODIRB
$02 equ rIPOLA
$03 equ rIPOLB
$04 equ rGPINTENA
$05 equ rGPINTENB
$06 equ rDEFVALA
$07 equ rDEFVALB
$08 equ rINTCONA
$09 equ rINTCONB
$0A equ rIOCON
\ $0B equ rIOCON
$0C equ rGPPUA
$0D equ rGPPUB
$0E equ rINTFA
$0F equ rINTFB
$10 equ rINTCAPA
$11 equ rINTCAPB
$12 equ rGPIOA
$13 equ rGPIOB
$14 equ rOLATA
$15 equ rOLATB

0 value CurrS17	\ -- u ; 0..7
\ *G Holds the current device. Set by you as required.

: read8-S17	\ reg -- b
\ *G Read 8 bits from the register.
  [spi3
    $41 CurrS17 1 lshift or write_spi8	\ select read
    write_spi8				\ register
    read_spi8				\ data
  spi3]
;

: write8-S17	\ b reg --
\ *G Write 8 bits to the register.
  [spi3
     $40 CurrS17 1 lshift or write_spi8	\ select write
     write_spi8				\ register
     write_spi8				\ data
  spi3]
;

: resetSPI	\ --
  rst_low  20 ms  rst_high 20 ms
;

: initS17	\ --
\ *G Initialise the current MCP23S17 device
  $28 rIOCON write8-S17			\ SEQOP=1, HAEN=1
  $00 rIODIRA write8-S17		\ port A all outputs
  $FF rIODIRB write8-S17		\ port B all inputs
;

: initS17all	\ --
\ *G Initialise all MCP23S17s that may be on the bus.
  CurrS17  8 0
  do  i to CurrS17  initS17  loop
  to CurrS17
;

\ ********
\ *S Tests
\ ********

1 [if]

2 value msdelay

: tr		\ reg --
  $55 over write8-s17  read8-s17 .byte space
;

: trall		\ --
  cr  $15 0 do  i tr  loop  ;

: trax		\ --
  CurrS17  8 0 do
    cr ." Dev=" i .  i to CurrS17  trall
  loop
  to CurrS17
;

: ta		\ -- ; waggle port A
  0 to CurrS17
  begin
    $00 rGPIOA write8-S17  msdelay ms
    $FF rGPIOA write8-S17  msdelay ms
    key?
  until
;

: tb		\ -- ; waggle port A
  0 to CurrS17
  begin
    $55 rGPIOA write8-S17  200 ms
    $AA rGPIOA write8-S17  200 ms
    key?
  until
;

: tc		\ -- ; waggle port A
  begin
    256 0 do
      i rGPIOA write8-s17
      key? ?leave
      400 ms
    loop
    key?
  until
;

: tsck		\ --
  begin  sck_low spidelay sck_high spidelay   key? until ;
: tmosi		\ --
  begin  mosi_low spidelay mosi_high spidelay   key? until ;
: tcs		\ --
  begin  ss_low spidelay ss_high spidelay   key? until ;
: trst		\ --
  begin  rst_low spidelay rst_high spidelay   key? until ;

[then]


\ ======
\ *> ###
\ ======

decimal
