\ spiSTM32-P107-8.fth - 8 bit SPI driver for STM32-P107 UEXT

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20120508 SFP002 Added reset line.
20120501 SFP001 Started.
))

only forth definitions
decimal

\ ================
\ *! sppistm32p107
\ *T SPI Driver for Olimex STM32-P107 board
\ ================
\ *P This code was tested on an Olimex STM32-P107 board which uses
\ ** an STM32F107VCT6 CPU in LQFP 100 pin format.

\ *E STM32-P107 UEXT Connector
\ ** Pin  Port  Function     Pin  Port  Function
\ ** 1          3v3          2          0v
\ ** 3    PD8   USART3 Tx    4    PD9   USART3 Rx
\ ** 5    PB8   I2C1 SCL     6    PB9   I2C1 SDA
\ ** 7    PC11  SPI3 MISO    8    PC12  SPI3 MOSI
\ ** 9    PC10  SPI3 SCK     10   PB15  CS_UEXT

\ *E MOD-LG UEXT Connector
\ ** Pin        Function     Pin        Function
\ ** 1          3v3          2          0v
\ ** 3          SPI_RST      4          --
\ ** 5          --           6          --
\ ** 7          MISO         8          MOSI
\ ** 9          SCK          10         CS


\ ****************
\ *S Configuration
\ ****************

interpreter
: genSPIBR	\ speed clock -- br
\ *G Generate an SPI baud rate value that is less than or equal
\ ** to the given speed.
  /  -1 swap  8 0 do			\ -- flag div
    dup 1 i 1+ lshift <=
    if  nip i swap  leave  endif
  loop
  drop  dup -1 = abort" Cannot generate SPI baud rate"
;
target


\ ****************
\ *S SPI interface
\ ****************
\ *P The port setup code requires *\i{Cortex/Drivers/gpioSTM32.fth}.
\ ** All GPIO ports are clocked and enabled at startup.

( 200 kHz ) 2 Mhz equ spi3-speed	\ -- hz
\ *G Speed of SPI3
apb1-speed spi3-speed genSPIBR 3 lshift equ spiBR3	\ -- x
\ *G Bit mask for SPI3 baud rate.
system-speed spi3-speed / 5 / 1 max equ /spi3delay	\ -- u
\ *G Number of iterations for delays around CS operations.
$0304 spiBR3 or equ /initSPI3_CR1	\ -- x
\ *G 2 lines unidirectional master, 8 bit MSB, no CRC, CPOL=0,
\ ** CPHA=0. N.B. SSM and SSI must be set in master mode.

[undefined] selio-spi3 [if]
3 8 pio: SPI3_RST	\ PD8 GPIO output ; SFP002
2 11 pio: MISO3		\ PC11 AF input
2 12 pio: MOSI3		\ PC12 AF output 50MHz - may be >10MHz
2 10 pio: SCK3		\ PC10 AF output 50MHz - may be >10MHz
1 15 pio: nCSUEXT	\ PB15 GPIO output 10MHz

: selio-spi3	\ --
\ *G Example to perform clock and pin selection for SPI3.
\ ** This version is used if the word *\fo{selio-spi} has not
\ ** already been defined.
  bit28 _AFIO afMAPR + or!		\ SPI3_REMAP = 1
  SPI3_RST isOutput  SPI3_RST setPin	\ Reset high ; SFP002
  MISO3 isInput
  AFOutput50MHz MOSI3 isFunction
  AFOutput50MHz SCK3 isFunction
  Output10MHz nCSUEXT isFunction
  nCSUEXT setPin			\ CS high
  \ SPI3 is on APB1
  _RCC
  bit15 over rccAPB1en + or!		\ enable SPI3 clock
  bit15 over rccAPB1rst + bic!		\ take it out of reset
  drop
;
[then]

: rst_low	\ --
\ *G Set the SPI reset line low.
  SPI3_RST clrPin
;

: rst_high	\ --
\ *G Set the SPI reset line high.
  SPI3_RST setPin
;

: spi3delay	\ --
\ *G Because the CS line is controlled by software, software
\ ** delays are used around changing the CS line.
  /spi3delay begin  1- dup 0=  until drop
;

: [spi3		\ --
\ *G Start a transaction by asserting CS.
  spi3delay  nCSUEXT clrPin  spi3delay
;

: spi3]		\ --
\ *G Finish a transaction by de-asserting CS.
  spi3delay  nCSUEXT setPin  spi3delay
;

: spi3xchg	\ xout -- xin
\ *G Transmit and read bytes.
  _SPI3
  tuck spiDR + w!			\ set data
  begin					\ wait for TXE=1 and RXNE=1
    dup spiSR + w@ $03 and $03 <>
   while
    pause
  repeat
  spiDR + w@				\ read data
;

: readSPI3	\ -- xin
\ *G Read a byte from the SPI line.
  $FF spi3xchg
;

: writeSPI3	\ xout --
\ *G Write a byte to the SPI line.
  spi3xchg drop
;

: init-spi3	\ --
\ *G Initialise SPI3.
  selio-spi3
  _SPI3
  0 over spiCR1 + w!			\ disable SPI3
  /initSPI3_CR1 over spiCR1 + w!	\ 2 lines uni master, 8 bit MSB, no CRC
  0 over spiCR2 + w!			\ no interrupts
  bit6 /initSPI3_CR1 or over spiCR1 + w! \ SPE=1, 2 lines uni master, 8 bit MSB, no CRC
  drop
;
' init-spi3 AtCold


\ *****
\ Tests
\ *****

1 [if]
_SPI3 constant _SPI3

: tcs		\ -- ; waggle CS line
  begin
    [spi3  $AA spi3xchg drop  spi3]  1 ms  key?
  until
;
[then]


\ ======
\ *> ###
\ ======

decimal
