\ StartSTM32F103.fth - Cortex startup code for STM32

((
Copyright (c) 2009, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20100112 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startstm32
\ *T Cortex start up for STM32
\ ===========

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard
[then]

init-s0 tos-cached? sp-guard + cells -
  equ real-init-s0	\ -- addr
\ The data stack pointer set at start up.

$010					\ Enable prefetch, no half-cycle
system-speed 1 - #24000000 / $07 and or	\ Wait states in bits 2:0
  equ /initACR	\ -- x

system-speed xtal-speed u> dup [if]
  drop  system-speed xtal-speed / 2-	\ PLLMUL
  #18 lshift  $0001:0000 or		\ PLL control in bits 21:16
[then]
  equ /initPLLmul

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLL
\ ** as required.
\ ** By default, the AHB and APB2 busses are run at SYSCLK,
\ ** and APB1 is run at SYSCLK/2.
  _RCC
  $0000:0001 over rccCR + or!		\ enable HSI only
  $0743:FFFF over rccCFGR + bic!	\ force to default config
  $0000:0400 over rccCFGR + or!		\ bits 10:8 -> 100, APB1 SYSCLK/2
  $0109:0000 over rccCR + bic!		\ clear PLLON, CSSON, HSEON
  $0004:0000 over rccCR + bic!		\ clear HSEBYP (HSEON=0 now)
  $007F:0000 over rccCFGR + bic!	\ clear PLLSRC, PLLXTPRE, PLLMUL and USBPRE/OTGFSPRE
  $009F:0000 over rccCIR + !		\ Disable  ints, clear pending bits
  $0001:0000 over rccCR + or!		\ set HSEON
  begin
    dup rccCR + @ $0002:0000 and	\ wait for HSERDY
  until
  /initACR _FPEC flACR + !		\ Enable prefetch and set wait states

  /initPLLmul $003F:0000 2 pick rccCFGR + setMask	\ configure PLL (bits 21:16)
  bit24 over rccCR + or!		\ enable PLL
  begin					\ wait for PLL ready
    dup rccCR + @ bit25 and
  until
  bit1 over rccCFGR + or!		\ SW=10, SYSCLK=PLL
  begin					\ wait for PLL selected
    dup rccCFGR + @ $0C and $08 =
  until

  drop
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation can also be performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    setClocks setNVIC
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

