\ startSTM32F107.fth - Cortex startup code for Olimex STM32-P107 board

((
Copyright (c) 2009, 2010, 2011, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20120322 MPE002 STM32F107 conversion
20100112 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startstm32f107
\ *T Cortex start up for STM32F107
\ ===========

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard
[then]

init-s0 tos-cached? sp-guard + cells -
  equ real-init-s0	\ -- addr
\ The data stack pointer set at start up.

interpreter
: prediv4	\ u -- x
\ *G Convert a divider value to a 4 bit pattern as used for HPRE.
  case
    1 of  %0000  endof
    2 of  %1000  endof
    4 of  %1001  endof
    8 of  %1010  endof
    16 of  %1011  endof
    64 of  %1100  endof
    128 of  %1101  endof
    256 of  %1110  endof
    512 of  %1111  endof
      true abort" Invalid AHB prescaler setting"
  endcase
;

: prediv3	\ u -- x
\ *G Convert a divider value to a 3 bit pattern as used for PPRE1 and PPRE2.
  case
    1 of  %000  endof
    2 of  %100  endof
    4 of  %101  endof
    8 of  %110  endof
    16 of  %111  endof
      true abort" Invalid PPREx prescaler setting"
  endcase
;

: pllmul1	\ u -- x
\ *G Convert a multiplier value to the 4 bit pattern as used for
\ ** PLLMUL.
  case
    4 of  %0010  endof
    5 of  %0011  endof
    6 of  %0100  endof
    7 of  %0101  endof
    8 of  %0110  endof
    9 of  %0111  endof
      true abort" Invalid PLLMUL1 setting"
  endcase
;

: pllmulx	\ u -- x
\ *G Convert a multiplier value to the 4 bit pattern as used for
\ ** PLL2MUL and PLL3MUL.
  case
     8 of  %0110  endof
     9 of  %0111  endof
    10 of  %1000  endof
    11 of  %1001  endof
    12 of  %1010  endof
    13 of  %1011  endof
    14 of  %1100  endof
    16 of  %1110  endof
    20 of  %1111  endof
      true abort" Invalid PLLMULX setting"
  endcase
;
target

system-speed 72 MHz = [if]
\ *P In order to set up for 72MHz CPU speed and RMII Ethernet
\ ** operation, we need to generate 8 Mhz on PLL2 as the input
\ ** to the main PLL which has a multiplier of 9.
\ *(
\ *B PLL2 configuration: *\br{}
\ ** PLL2CLK = (HSE / 5) * 8 = 40 MHz
\ *B PREDIV2 configuration: *\br{}
\ ** PREDIV2 = 5 ; HSE = 25 MHz, o/p = 5 MHz
\ *B PLL2MUL configuration *\br{}
\ ** PLL2MUL = 8, PLL2CLK = 40MHz
\ *B PREDIV1 configuration: *\br{}
\ ** PREDIV1 = 5, PREDIV1CLK = PLL2CLK / 5 = 8 MHz
\ *B PLLMUL configuration *\br{}
\ ** PLLMUL = 9, PLLCLK = 8MHz * 9 = 72 MHz
\ *B PLL3MUL configuration *\br{}
\ ** PLL3MUL = 10, PLL3CLK = 50MHz for MCO
\ *)
0					\ also the reset value
$0B 24 lshift or			\ MCO from PLL3 (50MHz for RMII)
0 22 lshift or				\ USB divider
9 pllmul1 18 lshift or			\ PLLin * 9 ; PLLin = PLL2 = 8MHz
					\ bit17 set from bits3:0 in rccCFGR2
1 16 lshift or				\ PLLSRC from PREDIV1
AHBdiv prediv4 4 lshift or		\ AHB clock
APB1div prediv3 8 lshift or		\ APB1 (slow) clock
APB2div prediv3 11 lshift or		\ APB2 (fast) clock
  equ initCFGRval		\ -- x
\ *G Bit settings for the RCC_CFGR register.
0
bit16 or				\ PREDIV1 from PLL2
10 pllmulx 12 lshift or			\ PLL3MUL
8 pllmulx 8 lshift or			\ PLL2MUL
5 1- 4 lshift or			\ PREDIV2
5 1- or					\ PREDIV1
  equ initCFGR2val		\ -- x
\ *G Bit settings for the RCC_CFGR2 register.
[then]

\ *P The number of wait states required by the Flash depends on
\ ** the operating frequency.
system-speed 1- 24 MHz / equ FLWS	\ -- u
\ *G The number of Flash wait states required.

bit4 ( PRFTBE ) FLWS or  equ initACRval	\ -- x
\ *G Initial value written to the FLASH_ACR register

((
: wastetime	\ --
  2000000 0 do loop
;

: blinky	\ --
  bit4 _RCC
    2dup rccAPB2en + or!		\ enable clock to GPIOC
    rccAPB2rst + bic!			\ remove reset from GPIOC
  $3300:0000 $FF00:0000 _GPIOC gp.CRL + setMask	\ 50MHz GP p-p o/p
  wastetime
  begin
    $0040:0080 _GPIOC gp.BSRR + !
    wastetime
    $0080:0040 _GPIOC gp.BSRR + !
    wastetime
  again
;
))

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required. PLL1 and PLL2 are used to generate the main
\ ** 72MHz clock. By default, the AHB and APB2 busses are run at
\ ** SYSCLK, and APB1 is run at SYSCLK/2.
\ ** PLL3 is used to generate 50MHz for Ethernet.
  _RCC

\ ensure internal clock is running
  bit0 over rccCR + or!			\ set HSION
  begin
    dup rccCR + @ bit1 and		\ wait for HSIRDY
  until
\ start external clock
  bit16 over rccCR + or!		\ set HSEON
  begin
    dup rccCR + @ bit17 and		\ wait for HSERDY
  until

\ Set up the Flash configuration
  initACRval _FPEC flACR + !		\ Enable caches and set wait states

\ set prescalers and PLL configurations
  initCFGRval over rccCFGR + !		\ set AHB, APB1, APB2 prescalers
  initCFGR2val over rccCFGR2 + !	\ PLL2/3 and dividers

\ start PLL2
  bit26 over rccCR + or!		\ set PLL2ON
  begin
    dup rccCR + @ bit27 and		\ wait for PLL2RDY
  until
\ start main PLL
  bit24 over rccCR + or!		\ set PLLON
  begin
    dup rccCR + @ bit25 and		\ wait for PLLRDY
  until

\ switch to the PLL output
  $0002 $0003 2 pick rccCFGR + setMask	\ set SW=2
  begin
    dup rccCFGR + @ $000C and $0008 =	\ wait for SWS=2
  until

\ can be moved to Cortex/Drivers/EtherSTM32F10x.fth
\ start PLL3
  bit28 over rccCR + or!		\ set PLL3ON
  begin
    dup rccCR + @ bit29 and		\ wait for PLL3RDY
  until
\ PLL3 output on MC0
  bit2 bit0 or over			\ -- rcc mask rcc
    2dup rccAPB2rst + bic!		\ port A and AFIO out of reset
    rccAPB2en + or!			\ port A and AFIO enabled
  $0B00:0000 $0F00:0000 2 pick rccCFGR + setMask	\ MCO from PLL3
  %1011 %1111 _GPIOA gp.CRH + setMask	\ PA8 to alt config output 50MHz

  drop
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation can also be performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    setClocks setNVIC
\ initGPIO tleds
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

