\ STM32F3nucleo.ctl - STM32F303K8T6 Nucleo board standalone Forth

((
Copyright (c) 2009, 2011, 2012, 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  901-313-4312

Adptation to the Synics environment by Kurt Heinz

Copyright (c) 2009-2017
Synics AG

tel: +41 (0)44 840 40 60
net: info@synics.ch
web: www.synics.ch

To do
=====

Change history
==============
20171019 kh002  Flash programming is adapted to STM32F30x, project simplified
20171005 kh002  Modify to use modern version of sfrSTM32F30x.fth extracted from STM32F30x.svd
20170831 kh001  Tentative to migrate to stm32f306
20120113 SFP002 Moved USART I/O selection to control file.
20091223 MPE001 First release.
))


\ ================
\ *! stm32f303nucleo32
\ *T Control file for STM32F303 Nucleo 32 board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32F3/STM32F3nucleo32.ctl}
\ ** produces a standalone Forth for the STM32F303 Nucleo 32 board.
\ ** The serial port is connected to USART2 on pins PA2 (USART2 Tx) and
\ ** PA15 (USART2 Rx). This port is connected to the Nucleo board's
\ ** USB COM port. Because the system uses the HSI oscillator, you
\ ** may find restrictions on the baud rate. By default, we use
\ ** 38400 baud.

\ *P When you modify this file for your own hardware do not
\ ** forget to update the GPIO pin assignments and alternate function
\ ** selections.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

0 constant Synics?
1 constant MPE?

\ Synics macros for a flat directory structure
Synics? [if]
c" ."           setmacro CommonDir      \ where the common code lives
c" ."           setmacro CpuDir         \ where the CPU specific code lives
c" ."           setmacro HWDir          \ where board specific code lives
c" ."           setmacro Drivers        \ drivers
c" ."           setmacro ReFlash        \ reflash
c" ."           setmacro ExampleDir     \ Examples
c" ."           setmacro AppDir         \ where application code lives
[then]

MPE? [if]
c" ../../../Common" 	setmacro CommonDir	\ where the common code lives
c" ../.."		setmacro CpuDir		\ CPU specific files
c" ."			setmacro HwDir		\ Board hardware files
c" ../../Drivers"	setmacro Drivers	\ drivers
c" ./Reflash"           setmacro ReFlash        \ reflash
c" ../../Examples"	setmacro ExampleDir	\ example files
c" ."			setmacro AppDir		\ application files
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros		\ macros needed by the cross-compiler


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: nucleo.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if] +xrefs [then]    \ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M4                     \ Thumb2 processor type and register usage
  +LongCalls                    \ permit calls outside 25 bit range.
  Hex-I32                       \ also produce Intel Hex-32 obj format
  +SaveCdataOnly                \ no data area image files

 0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N STM32F3 variant definitions
\ ======================
\ *P The board uses an STM32F303K8T6 32 pin chip. It has 64 kb
\ ** Flash, 12k SRAM and 4k CCM RAM.

1 equ PrefetchOK?       \ -- flag
\ *G If zero, the instruction prefetch queue does not work.

$0800:0000 equ FlashBase	\ -- addr
\ *G Start address of Flash. The bottom 2 kb (the vector area) is
\ ** mirrored at $0000:0000 for booting.
#64  kb equ /Flash      \ -- len
\ *G Size of Flash.
2 kb equ /FlashPage	\ -- len
\ *G Size of a Flash Page.
0 equ KeepPages		\ -- u
\ *G Set this non-zero for the number of pages at the end of Flash
\ ** that are reserved for configuration data. Often set to 1 or 2
\ ** by systems that use PowerNet.
FlashBase /Flash + /FlashPage KeepPages * - equ CfgFlash  \ -- len
\ *G Base address of the configuration Flash area.

$2000:0000 equ SRAMbase	\ -- addr
\ *G Base of SRAM
#12 kb equ /SRAM	\ -- len
\ *G Size of SRAM
$1000:0000 equ CCMbase	\ -- addr
\ *G Base of CCM SRAM
#4 kb equ /CCM	\ -- len
\ *G Size of CCM SRAM

$1FFF:D800 equ SysBase	\ -- addr
\ *G Base address of system memory information block.
#8 kb equ /SysMem		\ -- len
\ *G Size of system memory block.

$1FFF:F800 equ OptionBytes	\ -- addr
\ *G Base address of option bytes
#8 equ /OptionBytes		\ -- len
\ *G Number of option bytes

#128 cells equ /ExcVecs \ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system and USB clocks are generated from a PLL. The PLL
\ ** input clock must be in the range 1-2 MHz. The upper
\ ** end is preferred to reduce PLL jitter. The PLL output
\ ** frequency must be in the range 64..432 MHz.
\ ** To use USB at 48 MHz, you need to choose a PLL frequency
\ ** that is an integer multiple of 48 MHz. The system speed
\ ** is also an integer division of the VCO speed.

8 MHz equ xtal-speed    \ -- hz
\ *G Master oscillator crystal clock rate in HZ.
8 MHz equ HSI-speed    \ -- hz
\ *G Master RC oscillator clock rate in HZ.
HSI-speed 2 / equ vco-in-speed          \ -- hz
\ *G Requested VCO input frequency.
64 MHz equ vco-out-speed       \ -- hz
\ *G Requested VCO output frequency. Must not be greater than
\ ** 72 MHz.
64 MHz equ system-speed                \ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values.

vco-out-speed vco-in-speed / equ PLLN   \ -- u
\ *G Multiply the VCO input frequency by this value to produce the VCO
\ ** output frequency.

1 equ AHBdiv    \ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 168 MHz @ stm32f4x,
\ ** 120 MHz @ stm32f2x, 72 MHz @ stm32f3x.
1 equ APB2div   \ -- u
\ *G Division ratio of the APB2 clock from the system clock.
\ ** This may not be more than 84 MHz, 72 MHz @ stm32f3x.
2 equ APB1div   \ -- u
\ *G Division ratio of the APB1 clock from the system clock.
\ ** This may not be more than 42 MHz, 36 MHz @ stm32f3x.

system-speed AHBdiv / equ AHB-speed     \ -- hz
\ *G AHB bus speed.
system-speed APB2div / equ APB2-speed   \ -- hz
\ *G APB2 bus speed.
system-speed APB1div / equ APB1-speed   \ -- hz
\ *G APB1 bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2kb (the
\ ** vector area) is mirrored at $0000:0000. The CPU has 12 kb
\ ** of SRAM plus an 8kb CCM RAM
  $0800:0000 $0800:FFFF       cdata section STM32F3nucleo32 \ code section in boot Flash
  $2000:1000 equ UDATA_BOTTOM	\ boundary between UDATA and IDATA
  $2000:0000 UDATA_BOTTOM 1 - idata section PROGd           \ IDATA
  UDATA_BOTTOM $2000:2FFF     udata section PROGu           \ UDATA
  CCMbase dup /CCM 1- +       udata section CCM

interpreter
: prog STM32F3nucleo32  ;		\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE               \ size of each task's user area
$0100 equ SP-SIZE               \ size of each task's data stack
$0200 equ RP-SIZE               \ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size                 \ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard                 \ can underflow the data stack by this amount

$0100 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs                     \ number of IRQ stacks,
                                \ shared by all IRQs (1 min)
0 equ #SVCs                     \ number of SVC nestings permitted
                                \ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

0 equ useUSART1?        \ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

1 equ useUSART2?        \ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#38400 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART3?        \ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUART4?        \ -- n
\ *G Set non-zero to compile code for UART4,
\ ** device *\fo{Console4}.
115200 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

2 equ console-port      \ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..6 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#10 equ tick-ms         \ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?               \ nz to use cold chain mechanism
 1 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size         \   for internal consistency check
   0 equ event-handler?         \   true to include event handler
   0 equ message-handler?       \   true to include message handler
   1 equ semaphores?            \ true to include semaphores
 0 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?                  \ true for software floating point
 0 equ FullCase?                \ true to include ?OF END-CASE NEXTCASE extensions
 1 equ target-locals?           \ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?                  \ true if BLOCK needed
 $0000 equ sizeofheap           \ 0=no heap, nz=size of heap
   1 equ heap-diags?            \   true to include diagnostic code
 0 equ paged?                   \ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?             \ true if ANS ENVIRONMENT system required

 0 equ PowerNet?        \ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell                           \ size of a cell (32 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM4Def               \ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F30x         \ equates extracted from
  include %CpuDir%/StackDef             \ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG  \ top of memory for UNUSED
  include %HwDir%/startSTM32F3nucleo32	\ STM32F302 startup code
1 equ FastCmove?
  include %CpuDir%/CodeCortex           \ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %Drivers%/rebootCM4           \ reboot using CM4 core
  include %CpuDir%/IntCortex            \ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex          \ fault exception handlers for NVIC
  include %CommonDir%/CQueues           \ character queues

[undefined] selio-ser2 [if]
: selio-ser2    \ --
\ *G Initialise USART2 for operation on GPIO pins PA15 (Rx), PA2 (Tx)
  _RCC
  bit17 over rcc.AHBENR  + or!                    \ enable clock to GPIOA
  bit17 over rcc.AHBRSTR  + bic!                  \ take it out of reset
  bit17 over rcc.APB1ENR  + or!                   \ enable clock to USART2
  bit17 over rcc.APB1RSTR + bic!                  \ take it out of reset
  drop

  \ Default RS-232 port as UART2 on PA2 and PA15
  _GPIOA >r
  $8000:0020 $C000:0030 r@ gpio.MODER   + setMask  \ alternate function on PA15, PA2
  $40000:0010 $C000:0030 r@ gpio.OSPEEDR + setMask  \ 25MHz
  $0000:0010 $0000:0030 r@ gpio.PUPDR   + setMask  \ pull-up on PA2 (Tx)
  $0000:0700 $0000:0F00 r@ gpio.AFRL    + setMask  \ AF7 for both (AFRL2)
  $7000:0000 $F000:0000 r> gpio.AFRH    + setMask  \ AF7 for both (AFRH15)
;
[then]
  include %Drivers%/serSTM32F3xxqi \ RX/TX queued interrupt driver
  include %CommonDir%/Devtools          \ DUMP .S etc development tools
  include %CommonDir%/Voctools          \ ORDER VOCS etc
  include %CommonDir%/methods           \ target support for methods
  include %CpuDir%/LocalCM3             \ local variables

tasking? [if]
  include %CpuDir%/MultiCortex          \ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase          \ time base common code, MUST be before SysTickxxx
  include %CommonDir%/Delays            \ time delays
  include %Drivers%/SysTickCM4
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays
  include %Drivers%/SysTickCM4
  ' start-clock AtCold
[then]
\  include direct_timers

environment? [if]
  include %CommonDir%/environ           \ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32            \ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex          \ floating point primitives - coded version
  include %CommonDir%/sfp32com          \ common floating point code
[then]

\ *************
\ *S End of kernel
\ *************

buildfile STM32F3nucleo32.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner       \ --
  cr ." ********************************"
;

: .CPU          \ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M4"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH     \ -- ; marker
;

\ *******************
\ *S Application code
\ *******************

  include %Drivers%/gpioSTM32F30x  \ easy GPIO access
  include %HwDir%/FlashProg        \ Flash programming code
\  include %ReFlash%/ReFlash        \ ReFlash utility
((
  include strings
  include sqrt
  include sincos
  include mapping
  include i2cSTM32Fx_drv
  include i2c_devices
  include object
  include Nucleo_mapping           \ map to physical application
  include sort
  include timers
  include background
  include application_control
))

((
  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%/XmodemTxRx        \ Xmodem driver
  include %CommonDir%/RomForth/iodef    \ link i/o
  include %CommonDir%/RomForth/textfile \ AIDE textfile support
))

\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%/benchmarks/xbench32      \ performance tests

\ include %CommonDir%/TESTS/ANS/tester          \ John Hayes' ANS test engine
\ include %CommonDir%/TESTS/MPE/5-1-a           \ MPE test engine
\ include %ExampleDir%/TESTS/MPE/5-1-b          \ MPE tests

\ include %CommonDir%/TESTS/MPE/orbits          \ Sun-Earth-Moon simulation
\ include %CommonDir%/TESTS/MPE/bench           \ Sieve of Eratosthenes
\ include %CommonDir%/TESTS/MPE/str_loc         \ structures and local variables
\ include %CommonDir%/TESTS/MPE/checksum        \ checksum / crc
\ include %CommonDir%/TESTS/MPE/buildno         \ build number

\ include %CommonDir%/TESTS/MPE/testtask
\ include %CommonDir%/TESTS/MPE/testheap


\ ***************
\ *S Finishing up
\ ***************

cr ." Total USER area size is " next-user @ dup .
UP-SIZE >= [if]
  cr ." Too much user variables"
  hack-abort
[then]

libraries       \ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
                                        \ NOW, rather than by FINIS
  here $1FF +  $-0200 and org           \ force to 512 byte boundary for ISP loader

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build

afterwards sh "C:\MyApps\ST-Linkv4.0.0\ST-LINK Utility\ST-LINK_CLI.exe"  -P STM32F3nucleo32.hex  -Rst

FINIS


\ ======
\ *> ###
\ ======

