((
Copyright (c) 2013-2016
Synics AG

tel: +41 (0)44 840 40 60
net: info@synics.ch
web: www.synics.ch
))

\ ----- application control ------

: defer@ \ adr -- cfa;
\ *G Get the code of a deferred word adressed by ADR
   2 cells + @ @
;

: .deferred_word \ adr --
\ *G Print the function name of the content of a deferred word
\ *P Use in the form: ' mydeferredword .deferred_word
  2 cells + @ @ >name .name
;

\ ------------ concept for dynamic functions -----------------
\
\ Idea: replacing several defered functions like for instance DO_ACTION1, DO_ACTION2 by a dynamic function list, where DO_ACTION1 and DO_ACTION2
\       are added to the list. As an advantage the content of the list may be changed at run time.
\       Each function of the list makes an evolution step on <list_adr> EXEC_FUNCTLIST so it can be added in a task (typically background task).
\       A function of the list must not contain any unpredictible waiting events like KEY.
\       Each function must be stack neutral ( MY_FUNCTION / -- ; )
\       If a function let one of more values on the stack, the stack is recovered and the function is removed from the list after execution.
\
\ Ressources:
\       Each function requires a  VARIABLE containing the HEAP pointer, the HEAP size requeres n+2 cells where n is the number of functions of
\       the list
\
\ 2016-02-24 kh remove_funct modified, new feature; if the function inside the list is gives one or more value on stack, it is removed form list
\               This allows to make self-removing function, one stack value = hara kiri
\ 2015-09-23 kh comments added
\ 2015-09-04 kh add add_funct- to replace a function instead of having it twice with different content (interactive syndrom during tests...)
\ 2014-11-07 kh first trials
\ 2014-11-07 kh all written functions checked

struct functionlist
   cell field functlist_maxsize
   cell field functlist_size
end-struct

123 value FUNCTLIST_THROWCODE

: init_functlist \ n adr --;
\ *G Create an empty function list or erase the existing one giving by ADR,
\ ** with N as the max number of functions
 dup @ free drop                   \ free it in case it was in use
 over cells functionlist + allocate if
   ." no room on heap" 0 throw
 then over !                       \ heap pointer to adr
 @ swap over functlist_maxsize !   \ notify max size
 0 over      functlist_size    !   \ empty the list
 drop
;

: add_funct \ cfa adr --;
\ *G Add the function to the end of the execution list.
  @  dup 0= if ." list was not initialized" FUNCTLIST_THROWCODE throw then
  dup functlist_maxsize @
  over functlist_size @  > if
    2dup dup functlist_size @ cells functionlist + + ! \ add at the end
    1 over functlist_size +!
  else
    ." list full" FUNCTLIST_THROWCODE throw
  then
  2drop
;

: (remove_funct) \ cfa adr_ptr --;
\ *G Remove first occurance of function in list and pack list.
\ ** No error if the function was not found.
  dup functlist_size @ 0 ?do
    dup functionlist + i cells + @
    2 pick = if
      dup functionlist + i cells +  \ pack the list
      dup cell+ swap                \ ....-- cfa adr from to
      2 pick functlist_size @ i 1+ - cells  \ ....-- cfa adr from to (size-(i+1))*cell
      cmove
      -1 over functlist_size +!     \ reduce list size
      leave
    then
  loop
  2drop
;

: remove_funct \ cfa adr --;
  @ dup 0= if
    ." list was not initialized"
    FUNCTLIST_THROWCODE throw
  then
  (remove_funct)
;

: add_funct- \ cfa adr --;
\ *G Add funct exclusive
  2dup remove_funct add_funct
;

: .functlist \ adr --;
\ *G Display the actual content of the list
  @ dup 0= if ." list was not initialized" FUNCTLIST_THROWCODE throw then
  dup functlist_size @ 0 ?do
    cr dup functionlist + i cells + @ >name .name
  loop
  drop
;

: exec_functlist \ adr --;
\ *G Execute each function in list separated by a PAUSE.
\ *P Typical usage: add following in a cyclic task loop : CYCLIC_DESCRIPTOR EXEC_FUNCTLIST
\ ** where CYCLIC_DESCRIPTOR is a variable and its content was initialized with: 10 CYCLIC_DESCRIPTOR INIT_FUNCTLIST
\ ** for a list with max 10 elements
  @ dup 0= if drop exit then  \ exit quiet if list pointer was not initialized
  dup functlist_size @ 0 ?do
    dup functionlist + i cells + @      \ ... -- adr cfa_i
    sp@ >r
    dup execute
    sp@ r@ = if                         \ check stack diff
      r> 2drop                          \ function was stack neutral
    else
      r> sp!                            \ ... -- adr cfa_i; recover data stack
      over (remove_funct)               \ remove this fucnt from list
      leave                             \ leave as functlist_size is modified
    then
    pause
  loop
  drop
;

: ?reboot \ t/f --;
\ *G  force reboot on TRUE:
\ ** Usage: s" PATCH_MARKER" pad place pad find ?reboot . \ force a system restart if the PATCH_MARKER was found
  if
    reboot
  then
;

((
\ ----- action list test code ----

variable action_list1

10 action_list1 init_functlist

' noop action_list1 add_funct

action_list1 .functlist

: foo1 cr ." do foo1 ";  ' foo1 action_list1 add_funct
: foo2 cr ." do foo2 ";  ' foo2 action_list1 add_funct
: foo3 cr ." do foo3 ";  ' foo3 action_list1 add_funct
: foo4 cr ." do foo4 ";  ' foo4 action_list1 add_funct
: foo5 cr ." do foo5 ";  ' foo5 action_list1 add_funct
: foo6 cr ." do foo6 ";  ' foo6 action_list1 add_funct

action_list1 .functlist
action_list1 ' foo3  over  remove_funct .functlist

action_list1 exec_functlist

))

