\ ------ quicksort ----------
\ Original from FORTH Dimension Vol. 5-6
\
\ This is a 2 ways sorter. A 3 ways would be better for sorting things with a lot of similar elements.
\ For teh FORTH words, the 2 ways sorter is perfect, each word is usually unique

((
: QUICK ( a[m],a[n] --- )
    2DUP OVER - 2/ -2 AND + @ >R ( take middle value as pivot)
    2DUP SWAP
    BEGIN
      BEGIN DUP @ R@  PRECEDES WHILE 2+ REPEAT SWAP
      BEGIN R@ OVER @ PRECEDES WHILE 2- REPEAT SWAP
      2DUP U< NOT
      IF 2DUP 2DUP @ >R @ SWAP ! R> SWAP ! SWAP 2- SWAP 2+ THEN
      2DUP U<
    UNTIL R> DROP   ( m,n,j,i)   ( m<=i<=j<=n )
    ROT 2OVER 2OVER - + > IF 2SWAP THEN
    2DUP U< IF RECURSE ELSE 2DROP THEN   ( shorter part)
    2DUP U< IF RECURSE ELSE 2DROP THEN ; ( longer  part)
))

\ see http://www.sorting-algorithms.com/ for more sortung algorithms

defer precedes  \ a1,a2 -- f; vectored execution
\ basic compare function for sorting: if vectored element a1>a2 then f=true

: quick \  a[m],a[n] --;
\ Sort cell structured array between pointer a[m] and a[n], a[m] is start, a[n] is end
    2dup over - 2/ -4 and +
    @ >r      \ align down, take middle value as pivot  *(a[n]-a[m]/2) -> r
    2dup swap                \ ... a[m], a[n], a[m], a[n], a[m]
    begin
      begin
        dup @ r@  precedes
      while
        cell+
      repeat
      swap
      begin
        r@ over @ precedes
      while
        cell-
      repeat
      swap
      2dup u< not
      if
        2dup 2dup @ >r @ swap ! r> swap !
        swap cell- swap cell+
      then
      2dup u<
    until r> drop   \ m,n,j,i --  m<=i<=j<=n
    rot 2over 2over - + > if 2swap then
    2dup u< if
      recurse       \ shorter part
    else
      2drop
    then
    2dup u< if
      recurse       \ longer  part
    else
      2drop
    then
;

: (SWORDS)         \ -- adr n
\ *G Display the names of all definitions in the wordlist at the
\ ** top of the search-order.
  copy-threads                          \ copy out & normalize
  0 >r                                  \ get the number of names
  begin
    max-nfa dup                         \ while still one left
  while
    r> 1+ >r
    n>link @c origin- swap !		        \ find next
  repeat                                \ loop to count names
  2drop
  r@ cells allocate                     \ make room for a pointer list
  if 2drop drop cr ." not enough heap room for " r> .  ." pointers" exit then
  dup 2>r
  copy-threads                          \ copy out & normalize
  begin
    max-nfa dup                         \ while still one left
  while
    r> 2dup !
    cell+ >r
    n>link @c origin- swap !		\ find next
  repeat
  2drop
  r> drop
  r> r>
;

: .words \ adr n --
  cr
  0 ?do
    dup i cells + @ .name
    out @ c/l > if  cr  then            \ new line if needed
    pause
  loop
  drop
;

\ ---

: alpha_comp \ adr1 adr2 -- t/f
\ Compares both counted string addressed by adr1/2. The comparison criteria is ASCII code
  count $1f and      \ limit to name max name size
  rot count $1f and  \ .. adr1' n1 adr2' n2
  dup 3 pick min 0
  dup -rot           \ set a 0 as default
  do                 \ check on the lowest string size
    2 pick i + c@
    5 pick i + c@
    2dup > if
      2drop drop 1 leave
    else
      < if
        drop 2 leave
      then
    then
  loop
  dup 0= if               \ if 0, no difference found within the begin of text, check string lenght
    drop
    nip
    rot drop
    <                     \ decision on the longuest
  else
    1 = nip nip nip nip
  then
;

: alpha_comp- \ adr1 adr2 -- t/f
  swap alpha_comp
;

variable sort_array

: swords \ --
  assign alpha_comp- to-do precedes
  (swords)
  over sort_array !
  2dup 1- cells over +
  ['] quick cr .exec_time cr
  .words
  sort_array @ free drop
;

: swords- \ --
  assign alpha_comp to-do precedes
  (swords)
  over sort_array !
  2dup 1- cells over +
  ['] quick cr .exec_time cr
  .words
  sort_array @ free drop
;

((
: .NAME         \ nfa --
\ *G The correct way to display a definition's name given an NFA.
  countc $1F and typec space
;

))

: debug_alpha \ n1 n2 -- f
  cr .s key bl = if quit then
  alpha_comp
;
\ assign debug_alpha to-do precedes

