\ StartSTM32F407.fth - Cortex startup code for STM32F407VGT6 Discovery board

((
Copyright (c) 2009, 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20170904 kh     adapted to stm32f302
20120720 MPE002 Updated for external oscillators.
20100112 MPE001 First release.
))

only forth definitions  decimal

\ =================
\ *! startstm32f302
\ *T Cortex start up for STM32F302
\ =================

l: ExcVecs      \ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec     \ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if                                \ If not the stack top
    swap 1 or swap                      \   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1         \ holds xt of main word
  0 ,                                   \ fixed by MAKE-TURNKEY

proc DefExc2    b  $    end-code        \ NMI
proc DefExc3    b  $    end-code        \ HardFault
proc DefExc4    b  $    end-code        \ MemManage
proc DefExc5    b  $    end-code        \ BusFault
proc DefExc6    b  $    end-code        \ UsageFault
proc DefExc11   b  $    end-code        \ SVC
proc DefExc12   b  $    end-code        \ DbgMon
proc DefExc14   b  $    end-code        \ PendSV
proc DefExc15   b  $    end-code        \ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]               \ if no guard value is set
0 equ sp-guard
[then]

init-s0 tos-cached? sp-guard + cells -
  equ real-init-s0      \ -- addr
\ The data stack pointer set at start up.

interpreter
: prediv4       \ u -- x
\ *G Convert a divider value to a 4 bit pattern as used for HPRE.
  case
    1   of  %0000  endof
    2   of  %1000  endof
    4   of  %1001  endof
    8   of  %1010  endof
    16  of  %1011  endof
    64  of  %1100  endof
    128 of  %1101  endof
    256 of  %1110  endof
    512 of  %1111  endof
      true abort" Invalid AHB prescaler setting"
  endcase
;

: prediv3       \ u -- x
\ *G Convert a divider value to a 3 bit pattern as used for PPRE1 and PPRE2.
  case
    1 of  %000  endof
    2 of  %100  endof
    4 of  %101  endof
    8 of  %110  endof
    16 of  %111  endof
      true abort" Invalid PPREx prescaler setting"
  endcase
;

\ ---- stsm32ff40x -------
: PLLPval       \ -- x
\ *G Return the required PLLP bit pattern from the equate *\fo{PLLP}.
  case PLLP
    2 of  %00  endof
    4 of  %01  endof
    6 of  %10  endof
    8 of  %11  endof
      true abort" Invalid PLLP setting"
  endcase
;
target

0
1               0 lshift or      \ SW 0:1       1=HSE
$0f             4 lshift or      \ HPRE 4:7
7               8 lshift or      \ PPRE1 8:10
7               11 lshift or     \ PPRE1 11:13
3               15 lshift or     \ PLLSRC 15:16 %10=HSE
1               17 lshift or     \ PLLXTPRE 17  0=no prescaler
$0f             18 lshift or     \ PLLMUL 18:21
0               22 lshift or     \ USBPRE
1               23 lshift or     \ I2SSRC 0=internal clock
0               24 lshift or     \ MCO 24:30 setup
1               31 lshift or     \ PLLNODIV
  equ initCFGRmask

2   equ SW_SOURCE
$0c equ SWS_MASK

0                                \ SW 0:1       HSI first
AHBdiv  prediv4 4  lshift or     \ HPRE 4:7
APB1div prediv3 8  lshift or     \ PPRE1 8:10
APB2div prediv3 11 lshift or     \ PPRE1 11:13
0               15 lshift or     \ PLLSRC 15:16 %10=HSE, %01=HSI %00=HSI/2
0               17 lshift or     \ PLLXTPRE 17  0=no prescaler
PLLN 2 -        18 lshift or     \ PLLMUL 18:21
0               22 lshift or     \ USBPRE
0               23 lshift or     \ I2SSRC 0=internal clock
0               24 lshift or     \ MCO 24:30 setup
0               31 lshift or     \ PLLNODIV
  equ initCFGRval


\ *P The number of wait states required by the Flash depends on
\ ** the operating voltage.
  30 MHz equ /FlUnit    \ for 2.7 to 3.6 volts
\ 24 MHz equ /FlUnit    \ for 2.4 to 2.7 volts
\ 18 MHz equ /FlUnit    \ for 2.1 to 2.4 volts
\ 16 MHz equ /FlUnit    \ for 1.8 to 2.1 volts
system-speed 1- /FlUnit / equ FLWS      \ -- u
\ *G The number of Flash wait states required.

\ PrefetchOK? [if] bit8 ( PRFTEN ) [else] 0 [then]  \ No prefetch on rev A devices  STM32F407
PrefetchOK? [if] bit4 ( PRFTBE ) [else] 0 [then]  \ No prefetch on rev A devices STM32F30x
  FLWS or
  equ initACRval        \ -- x
\ *G Initial value written to the Flash ACR register.

[undefined] HSE-ExtClk? [if]
0 equ HSE-ExtClk?       \ -- 0
\ *G If not already defined, this equate is set to zero to select
\ ** HSE using the internal crystal driver.
[then]

internal

: setClocks     \ --
\ *G Enable the clocks, set the bus dividers, and enable the PLL
\ ** as required.
\ ** By default, the AHB and APB2 busses are run at SYSCLK,
\ ** and APB1 is run at SYSCLK/2.
  _RCC
  bit16 bit19 or bit24 or over rcc.CR + bic!  \ clear PLLON, HSEON, CSSON
  begin
    dup rcc.CR + @ bit25 and 0=            \ wait on PLLRDY=0
  until
  \ Set up the Flash configuration
  initACRval _Flash fl.ACR + !            \ Enable caches and set wait states
  initCFGRval over rcc.CFGR + !            \ setup dividers
  bit24 bit19 or over rcc.CR + or!         \ set PLLON, CCSON
  begin
    dup rcc.CR + @ bit25 and               \ wait on PLLRDY
  until
  initCFGRval SW_SOURCE or over rcc.CFGR + or! \ once stable, choose as clk source
  begin
    dup rcc.CFGR + @ SWS_MASK and
    SW_SOURCE 2 lshift =
    drop -1
  until

  drop
;

: setNVIC       \ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !              \ Vector Table Offset Reg.
;

: +FPU          \ --
\ *G Enable the FPU by enabling full access to CP10 and CP11.
  $00F0:0000 _SCS scsCPACR + or!
;

: StartCortex   \ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation can also be performed here.
  begin
    INT_STACK_TOP SP_main sys!          \ set SP_main for interrupts
    INIT-R0 SP_process sys!             \ set SP_process for tasks
    2 control sys!                      \ switch to SP_process
    REAL-INIT-S0 set-sp                 \ Allow for cached TOS and guard space
    INIT-U0 up!                         \ USER area
    setClocks
    setNVIC
  \  +FPU
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec       \ Define initial return stack
' StartCortex ResetVec# SetExcVec       \ Define startup word
DefExc2       2         SetExcVec
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,      \ s0
  init-r0 ,                             \ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

