\ Down counter high level timer system
((

 Copyright (c) 2007-2012
 Synics AG
 Pumpwerstrasse 15
 CH-8105 Regensdorf

 tel: +41 (0)44 840 40 60
 net: info@synics.ch
 web: www.synics.ch

 Release:
 2014-06-20 KH timers have now a link list
 2010-03-14 KH timers using maximum resolution are moved to DIRECT_TIMERS.FTH
 2009-06-24 KH short delay words included in this timer
 2009-04-08 KH changed DO_TIMERS to DOWN-TIMERS to make it compatible with timebase.fth
 2006-05-23 KH converted from 68k project

))

decimal

25 constant MAX_TIMERS    \ this is the reserved cells array size for a fix # of timers (deterministic)

align here 0 , equ timer_count \ this cell in ROM is the total number of timers in use
align here 0 , constant timer_link  \ link list pointer for timers, first val is 0

variable timer_array MAX_TIMERS cells cell - allot-ram

: timer_step \ adr --, decrement cell if different from 0 of -1
  dup @  0=  if drop exit then
  dup @ -1 = if drop exit then
  -1 swap +! ;

: running? \ adr -- n; check timer for running
  dup @ 0= if drop 0 exit then
  @ -1 <> ;

: elapsed? \ adr --
  @ 0=
;

: down-timers \ --, compatible with low level words
  timer_count @ 0 ?do
    i cells timer_array + timer_step
  loop ;

: clear-down-timers \ --, all timers on elapsed
  timer_array timer_count @ cells 0 fill
;

\ ' clear-down-timers AtCold \ no need as moved to IRAM

interpreter

: build_timer  \ -- <name> ; -- n ; build a static named timer
  create
  here                             \ remember code field
  timer_array timer_count @ cells + , 1 timer_count +!
  timer_link dup @ ,
  !                                \ store link value
  does> @
;

target

: delay \ n timer --;
  tuck !
  begin
   pause
   dup @ 0=
  until drop ;

build_timer timer1
build_timer timer2
build_timer timer3
build_timer timer4

\ ------ timer debugging -------------

: .timers \ --;
\ *G Print a list the declared timers
  timer_link @
  timer_count @ 0 do
     dup 2 cells -     \ this offset is compiler specific!
     >name cr .name
     cell+ @
     dup 0= if leave then
  loop
  drop
;

: .running_timers \ --;
\ *G Print a list the declared timers
  timer_link @
  timer_count @ 0 do
    dup @ running? if
      dup 2 cells -     \ this offset is compiler specific!
      >name cr .name
    then
    cell+ @
    dup 0= if leave then
  loop
  drop
;


\ ----- non interrupt timer update ----

variable downtimers_accumulator

: update_timers \ --;
  ticks downtimers_accumulator @ <> if
    down-timers
    1 downtimers_accumulator +!
  then
;

: init_downtimers \ --;
  ticks downtimers_accumulator !
;

\ ------------ simplifed timer modell ---------------

\ For periodic tasks with non-cumulative error permitted.
\ The timing bases is TICKS


: periode_over_nc? \ n adr -- t/f;
\ *G Check if n milisecs of the timer at adr is elapsed.
\ ** The result is true or false
\ ** If true, the new time step is stored in adr.
\ ** The cumulative error is compensated
  2dup @ ticks swap - swap > if
    +! true             \ cumulative errors are avoided
  else
    2drop false
  then
;

\ for periodic tasks where a cumulative error is allowed

: periode_over? \ n adr -- t/f;
\ *G Check if n milisecs of the timer at adr is elapsed.
\ ** The result is true or false
\ ** If true, the new time step is stored in adr.
  ticks over @ over swap -
  3 roll > if
    swap ! true         \ the time where this word was called and true is now the new reference
  else
    2drop false
  then
;

\ ---- debug -----------
0 [if]

200 value test_rate

variable mytime

: tictac \ ---
  begin
    pause
    test_rate mytime time_over? if
      cr ." tic
    then
  key? until
;

[then]
