\ FSMCeval.fth - Configure FSMC for SRAM and LCD on STM3240G-EVAL board

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! fsmceval
\ *T External SRAM and LCD
\ ===========
\ *P The file *\i{Cortex\Hardware\STM32F4\FSMCeval.fth} sets up
\ ** the STM32F4 FSMC unit for the SRAM and LCD on the STM3240G-EVAL
\ ** board.

((
 NOR/SRAM Controller functions
 =============================

 The following sequence should be followed to configure the FSMC to interface with
 SRAM, PSRAM, NOR or OneNAND memory connected to the NOR/SRAM Bank:

   1. Enable the clock for the FSMC and associated GPIOs using the following functions:
          RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);
          RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOx, ENABLE);

   2. FSMC pins configuration
       - Connect the involved FSMC pins to AF12 using the following function
          GPIO_PinAFConfig(GPIOx, GPIO_PinSourcex, GPIO_AF_FSMC);
       - Configure these FSMC pins in alternate function mode by calling the function
          GPIO_Init();

   3. Declare a FSMC_NORSRAMInitTypeDef structure, for example:
          FSMC_NORSRAMInitTypeDef  FSMC_NORSRAMInitStructure;
      and fill the FSMC_NORSRAMInitStructure variable with the allowed values of
      the structure member.

   4. Initialize the NOR/SRAM Controller by calling the function
          FSMC_NORSRAMInit(&FSMC_NORSRAMInitStructure);

   5. Then enable the NOR/SRAM Bank, for example:
          FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM2, ENABLE);

   6. At this stage you can read/write from/to the memory connected to the NOR/SRAM Bank.
))

\        function         port
3 4  pio: pinNOE	\ PD4
3 5  pio: pinNWE	\ PD5
4 0  pio: pinNBL0	\ PE0
4 1  pio: pinNBL1	\ PE1

6 9  pio: pinNE2	\ PG9, SRAM
6 10 pio: pinNE3	\ PG10, LCD

4 3  pio: pinA19	\ PE3
3 13 pio: pinA18	\ PD13
3 12 pio: pinA17	\ PD12
3 11 pio: pinA16	\ PD11
6 5  pio: pinA15	\ PG5
6 4  pio: pinA14	\ PG4
6 3  pio: pinA13	\ PG3
6 2  pio: pinA12	\ PG2
6 1  pio: pinA11	\ PG1
6 0  pio: pinA10	\ PG0
5 15 pio: pinA9		\ PF15
5 14 pio: pinA8		\ PF14
5 13 pio: pinA7		\ PF13
5 12 pio: pinA6		\ PF12
5 5  pio: pinA5		\ PF5
5 4  pio: pinA4		\ PF4
5 3  pio: pinA3		\ PF3
5 2  pio: pinA2		\ PF2
5 1  pio: pinA1		\ PF1
5 0  pio: pinA0		\ PF0

3 10 pio: pinD15	\ PD10
3 9  pio: pinD14	\ PD9
3 8  pio: pinD13	\ PD8
4 15 pio: pinD12	\ PE15
4 14 pio: pinD11	\ PE14
4 13 pio: pinD10	\ PE13
4 12 pio: pinD9		\ PE12
4 11 pio: pinD8		\ PE11
4 10 pio: pinD7		\ PE10
4 9  pio: pinD6		\ PE9
4 8  pio: pinD5		\ PE8
4 7  pio: pinD4		\ PE7
3 1  pio: pinD3		\ PD1
3 0  pio: pinD2		\ PD0
3 15 pio: pinD1		\ PD15
3 14 pio: pinD0		\ PD14

create FSMCpins	\ -- addr
\ *G A zero terminated list of GPIO pin structures.
  pinNOE ,	pinNWE ,	pinNBL0 ,	pinNBL1 ,
  pinNE2 ,	pinNE3 ,
  pinA19 ,	pinA18 ,	pinA17 ,	pinA16 ,
  pinA15 ,	pinA14 ,	pinA13 ,	pinA12 ,
  pinA11 ,	pinA10 ,	pinA9 ,		pinA8 ,
  pinA7 ,	pinA6 ,		pinA5 ,		pinA4 ,
  pinA3 ,	pinA2 ,		pinA1 ,		pinA0 ,

  pinD15 ,	pinD14 ,	pinD13 ,	pinD12 ,
  pinD11 ,	pinD10 ,	pinD9 ,		pinD8 ,
  pinD7 ,	pinD6 ,		pinD5 ,		pinD4 ,
  pinD3 ,	pinD2 ,		pinD1 ,		pinD0 ,
  0 ,

\ SRAM style devices are connected to Bank 1. This is subdivided
\ into four regions.
\   6000:0000  NOR/PSRAM1
\   6400:0000  NOR/PSRAM2
\   6800:0000  NOR/PSRAM3
\   6C00:0000  NOR/PSRAM4

\ BCRx regs
\ bit 19	CBURSTRW, 0= async, 1=PSRAM synchronous burst
\		on Write, 1=Flash burst

\ bit 15	ASYNCWAIT, true if wait signal needed
\		0 = no NWAIT, 1 = use NWAIT
\ bit 14	EXTMOD, 1 = extended mode enabled, use BWTRx
\ bit 13	WAITEN, 1 = use NWAIT in Flash bursts
\ bit 12	WREN, 0 = read only, 1 = write enabled
\ bit 11	WAITCFG, 0 = NWAIT before wait, 1 = NWAIT during
\ bit 10	WRAPMOD, no effect
\ bit 9		WAITPOL, 0 = NWAIT active low, 1 = NWAIT active high
\ bit 8		BURSTEN, 1 = enable synchronous bursts

\ bit 6		FACCEN, 1 = Enable NOR Flash access
\ bit 5:4	MWID, 00 = 8 bits, 01 = 16 bits, 10, 11 = reserved
\ bit 3:2	MTYPE, 00 = SRAM/ROM, 01 = PSRAM, 10 = NorFlash/OneNAND Flash
\ bit 1		MUXEN, 0 = not multipled, 1 = addr/data multiplexed
\ bit 0		MBKEN, 0 = bank disabled, 1 = bank enabled

$0008:FF7F equ BCRx_mask
$0000:1010 equ BCR2_SRAM_nowait
$0000:5010 equ BCR3_SRAM_nowait		\ LCD: EXTMOD is set

\ BTRx regs - read/write timing if EXTMOD=0, read timing if EXTMOD=1
\ BWTRx regs - write timing if BCRx EXTMOD=1
\ bit 29:28	ACCMOD, if BCRX EXTMOD=1, 0..3 = Access Mode A..D
\ bit 27:24	DATLAT, latency for sync burst mode NOR Flash,
\		0..15 -> 2..17 Flash clock periods
\ bit 23:20	CLKDIV, don't care if async. BTRx only.
\		1..15 -> 2..16 HCLK periods
\ bit 19:16	BUSTRURN, Multiplexed NOR Flash only, bus turn
\		around duration, 0..15 HCLK cycles
\ bit 15:8	DATST, data transfer phase , 1..255 HCLK cycles
\ bit 7:4	ADDHLD, Address hold duration after DATST, 1..15 HCLKs
\ bit 3:0	ADDSET, Address set up duration, 0..15 HCLKs

$3FFF:FFFF equ BTRx_mask
$0010:0311 equ BTR2_SRAM	\ 1Mx16 SRAM
$0000:17F2 equ BTR3_SRAM	\ LCD: read timing, 150ns + 150ns @ HCLK=6ns
$3FF0:FFFF equ BWTRx_mask
$0000:0912 equ BWTR3_SRAM	\ LCD: write timing, 50ns + 50ns @ HCLK=6ns

\ IS61WV102416 SRAM, assume 20ns (10ns on the ST eval board at MPE).
\ tRC  20 ns    read cycle
\ tAA  20 ns    address valid to read data valid
\ tDOE  8 ns    OE valid to data valid
\ tOHA  2.5 ns  Output hold time
\ tWC  20 ns    write cycle
\ tWE  12 ns    WE cycle

: enFSMC	\ --
\ *G Enable access to the FSMC.
  _RCC
  bit0 over rccAHB3RSTR + bic!		\ remove reset
  bit0 over rccAHB3ENR + or!		\ enable clock
  drop
;

: initFSMC	\ --
\ *G Initialise the FSMC for the 16 bit RAM on NE2 and the FPGA
\ ** on NE3.
  enFSMC
\ set up the pins for the FSMC
  FSMCpins begin
    dup @
   while
    #12 over @ isFunction		\ AF12
    100MHZ_hs over @ isPinSpeed		\ 100 MHz
    0 over @ isPinOD			\ push-pull
    dup @ NotPulled			\ no pull up/down
    cell +
  repeat
  drop
\ set up the FSMC - assume documented reset values are bogus
  _FSMC >r
  BCR2_SRAM_nowait BCRx_mask r@ fsmc.BCR2 + setMask	\ SRAM control reg
  BTR2_SRAM BTRx_mask r@ fsmc.BTR2 + setMask		\ SRAM timing reg
  BCR3_SRAM_nowait BCRx_mask r@ fsmc.BCR3 + setMask	\ LCD control reg
  BTR3_SRAM BTRx_mask r@ fsmc.BTR3 + setMask		\ LCD read timing reg
  BWTR3_SRAM BWTRx_mask r@ fsmc.BWTR3 + setMask		\ LCD write timing reg
  bit0 r@ fsmc.BCR2 + or!		\ enable
  bit0 r@ fsmc.BCR3 + or!		\ enable
  r> drop
;
' initFSMC atCold			\ run at power up

$6400:0000 equ _ESRAM2
$6800:0000 equ _ELCD


\ ************
\ *S Test code
\ ************

1 [if]

_FSMC constant _FSMC		\ -- addr
_ESRAM2 constant _ESRAM2	\ -- addr
_ELCD constant _ELCD		\ -- addr

: trs		\ --
\ *G SRAM read test
  _ESRAM2
  begin  dup @ drop  key? until
  drop
;

: tws		\ --
\ *G SRAM write test
  _ESRAM2
  begin  $012345678 over !  key? until
  drop
;

: trl		\ --
\ *G LCD read test
  _ELCD
  begin  dup w@ drop  key? until
  drop
;

: twl		\ --
\ *G LCD write test
  _ELCD
  begin  $0123 over w!  key? until
  drop
;

[then]


\ ======
\ *> ###
\ ======

decimal

