\ HwEther-E407.fth - Ethernet hardware set up for Olimex STM32-E407

((
Copyright (c) 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20131008 SFP002 Refactored from Ethernet driver
20120125 SFP001 First release
))

decimal


\ ==============
\ *! hwethere407
\ *T Olimex STM32-E407 Ethernet specifics
\ ==============
\ *S Introduction
\ *P The Ethernet code in *\i{Cortex\Drivers\HwEther-E407.fth}
\ ** is derived from sample C drivers from the ST MCD Application
\ ** Team; see *\f{http://www.st.com}.
\ ** The code was tested on the Olimex STM32-E407 board.

\ ** In order to ease porting the Ethernet code to other boards,
\ ** the hardware specific set-up code is separated from the main
\ ** STM32F4 Ethernet driver. The word *\fo{initEtherHw) performs
\ ** the set up and is used by the main driver.
\ ** This code requires the GPIO driver
\ ** *\i{Cortex\Drivers\gpioSTM32F4xx.fth}.


\ **************************
\ *S Default hardware set up
\ **************************
\ *P RMII mode, 50 MHz oscillator for PHY.

((
  ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1, AF11
  ETH_MDIO -------------------------> PA2, AF11
  ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7, AF11

  ETH_PPS_OUT ----------------------> PB5, AF11
  ETH_MII_TXD3 ---------------------> PB8, AF11

  ETH_MDC --------------------------> PC1, AF11
  ETH_MII_TXD2 ---------------------> PC2, AF11
  ETH_MII_TX_CLK -------------------> PC3, AF11
  ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4, AF11
  ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5, AF11

  ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11, AF11
  ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13, AF11
  ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14, AF11

  ETH_MII_CRS ----------------------> PH2, AF11
  ETH_MII_COL ----------------------> PH3, AF11
  ETH_MII_RXD2 ---------------------> PH6, AF11
  ETH_MII_RXD3 ---------------------> PH7, AF11

  ETH_MII_RX_ER --------------------> PI10, AF11
))

PA 3 pio: PA3		\ PA3 for PHY interrupt
PG 6 pio: PG6		\ PG6 for PHY NRST

PA 1 pio: PA1		\ ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1, AF11
PA 2 pio: PA2		\ ETH_MDIO -------------------------> PA2, AF11
PA 7 pio: PA7		\ ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7, AF11
PC 1 pio: PC1		\ ETH_MDC --------------------------> PC1, AF11
PC 4 pio: PC4		\ ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4, AF11
PC 5 pio: PC5		\ ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5, AF11
PG 11 pio: PG11		\ ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11, AF11
PG 13 pio: PG13		\ ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13, AF11
PG 14 pio: PG14		\ ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14, AF11

\ 1 5 pio: PB5		\ ETH_PPS_OUT ----------------------> PB5, AF11
\ 1 8 pio: PB8		\ ETH_MII_TXD3 ---------------------> PB8, AF11
\ 2 2 pio: PC2		\ ETH_MII_TXD2 ---------------------> PC2, AF11
\ 2 3 pio: PC3		\ ETH_MII_TX_CLK -------------------> PC3, AF11
\ 7 2 pio: PH2		\ ETH_MII_CRS ----------------------> PH2, AF11
\ 7 3 pio: PH3		\ ETH_MII_COL ----------------------> PH3, AF11
\ 7 6 pio: PH6		\ ETH_MII_RXD2 ---------------------> PH6, AF11
\ 7 7 pio: PH7		\ ETH_MII_RXD3 ---------------------> PH7, AF11
\ 8 10 pio: PI10		\ ETH_MII_RX_ER --------------------> PI10, AF11

: isEtherPin	\ pin --
\ *G Set the pin to AF11 and 100MHz.
  11 over isFunction  100MHZ_hs swap isPinSpeed
;

: initEtherHw	\ --
\ *G Set up the clocks and pins required for Ethernet operation
\ ** in RMII mode.
\ GPIO already clocked by gpioSTM32F4xx.fth.

  bit14 _RCC 2dup
  rccAPB2ENR + or!  rccAPB2RSTR + bic!	\ enable SYSCFG clock, out of reset

\  PA8					\ configure PA8 for MCO
\  0 over isFunction  0 over isPinOD
\  dup notPulled  100MHZ_hs swap isPinSpeed
\  $0040:0000 $0760:0000 _RCC rccCFGR + setMask \ divide by 1 from HSE

  PA3 isInput				\ PHY interrupt
  PG6 isOutput				\ PHY NRST

  PA1 isEtherPin			\ all pins to AF11, Ethernet
  PA2 isEtherPin
  PA7 isEtherPin
\  PB5 isEtherPin
\  PB8 isEtherPin
  PC1 isEtherPin
\  PC2 isEtherPin
\  PC3 isEtherPin
  PC4 isEtherPin
  PC5 isEtherPin
  PG11 isEtherPin
  PG13 isEtherPin
  PG14 isEtherPin
\  PH2 isEtherPin
\  PH3 isEtherPin
\  PH6 isEtherPin
\  PH7 isEtherPin
\  PI10 isEtherPin
((
  PB14 dup isInput  dup notPulled
  $0100 _SYSCFG sysEXTICR4 + !		\ connect PB14 to EXTI
  bit14 _EXTI  2dup extiFTSR + !	\ happens on falling edge
  extiIMR + !				\ enable interrupt
))

  bit25 _RCC rccAHB1RSTR + or!		\ MAC into reset
  bit23 _SYSCFG sysPMC + or!
  bit25 _RCC rccAHB1RSTR + bic!		\ MAC out of reset
  $0E00:0000 _RCC rccAHB1ENR + or!	\ enable MAC clocks except PTP
  _ETH
  bit0 over ethDMABMR + or!		\ software reset of DMA and MAC
  begin
    dup ethDMABMR + @ bit0 and 0=
  until
  drop
  PG6 clrPin  10 ms  PG6 setpin  10 ms	\ hardware reset PHY
;


\ ******
\ *> ###
\ ******

decimal

