\ HwEther3240geval.fth - Ethernet hardware set up for STM3240G-EVAL board.

((
Copyright (c) 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20131008 SFP002 Refactored from Ethernet driver
20120125 SFP001 First release
))

decimal


\ ==============
\ *! hwether3240geval
\ *T STM3240G-EVAL Ethernet specifics
\ ==============
\ *S Introduction
\ *P The Ethernet set up code in *\i{Cortex\Drivers\HwEther3240geval.fth}
\ ** is derived from sample C drivers from the ST MCD Application
\ ** Team; see *\f{http://www.st.com}.
\ ** The code was tested on the STM3240G-EVAL board.

\ *P The STM3240G-EVAL board is supplied pre-configured with the
\ ** PHY set to use MII communication. The board must be modified
\ ** to use RMII. This code is for MII mode only. In order to use
\ ** the default jumper settings, this code outputs 25MHz on the
\ ** MCO (PA8) pin. In order to ease porting this code to other
\ ** boards, the hardware specific set-up code is in the word
\ ** *\fo{initEtherHw) which is only used if not already defined.
\ ** This default code requires the GPIO driver
\ ** *\i{Cortex\Drivers\gpioSTM32F4xx.fth}.


\ **************************
\ *S Default hardware set up
\ **************************
\ *P MII mode, 25 MHz from MCO.

((
  ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1, AF11
  ETH_MDIO -------------------------> PA2, AF11
  ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7, AF11

  ETH_PPS_OUT ----------------------> PB5, AF11
  ETH_MII_TXD3 ---------------------> PB8, AF11

  ETH_MDC --------------------------> PC1, AF11
  ETH_MII_TXD2 ---------------------> PC2, AF11
  ETH_MII_TX_CLK -------------------> PC3, AF11
  ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4, AF11
  ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5, AF11

  ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11, AF11
  ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13, AF11
  ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14, AF11

  ETH_MII_CRS ----------------------> PH2, AF11
  ETH_MII_COL ----------------------> PH3, AF11
  ETH_MII_RXD2 ---------------------> PH6, AF11
  ETH_MII_RXD3 ---------------------> PH7, AF11

  ETH_MII_RX_ER --------------------> PI10, AF11
))

0 1 pio: PA1		\ ETH_MII_RX_CLK/ETH_RMII_REF_CLK---> PA1, AF11
0 2 pio: PA2		\ ETH_MDIO -------------------------> PA2, AF11
0 7 pio: PA7		\ ETH_MII_RX_DV/ETH_RMII_CRS_DV ----> PA7, AF11
1 5 pio: PB5		\ ETH_PPS_OUT ----------------------> PB5, AF11
1 8 pio: PB8		\ ETH_MII_TXD3 ---------------------> PB8, AF11
2 1 pio: PC1		\ ETH_MDC --------------------------> PC1, AF11
2 2 pio: PC2		\ ETH_MII_TXD2 ---------------------> PC2, AF11
2 3 pio: PC3		\ ETH_MII_TX_CLK -------------------> PC3, AF11
2 4 pio: PC4		\ ETH_MII_RXD0/ETH_RMII_RXD0 -------> PC4, AF11
2 5 pio: PC5		\ ETH_MII_RXD1/ETH_RMII_RXD1 -------> PC5, AF11
6 11 pio: PG11		\ ETH_MII_TX_EN/ETH_RMII_TX_EN -----> PG11, AF11
6 13 pio: PG13		\ ETH_MII_TXD0/ETH_RMII_TXD0 -------> PG13, AF11
6 14 pio: PG14		\ ETH_MII_TXD1/ETH_RMII_TXD1 -------> PG14, AF11
7 2 pio: PH2		\ ETH_MII_CRS ----------------------> PH2, AF11
7 3 pio: PH3		\ ETH_MII_COL ----------------------> PH3, AF11
7 6 pio: PH6		\ ETH_MII_RXD2 ---------------------> PH6, AF11
7 7 pio: PH7		\ ETH_MII_RXD3 ---------------------> PH7, AF11
8 10 pio: PI10		\ ETH_MII_RX_ER --------------------> PI10, AF11

0 8 pio: PA8		\ AF0, used for MCO - requires SYSCFG
1 14 pio: PB14		\ used as EXTINT from PHY

: isEtherPin	\ pin --
\ *G Set the pin to AF11 and 50MHz.
  11 over isFunction  50MHZ_fs swap isPinSpeed
;

: initEtherHw	\ --
\ *G Set up the clocks and pins required for Ethernet operation
\ ** in MII mode for the STM3240G-EVAL board.
\ GPIO already clocked by gpioSTM32F4xx.fth.

  bit14 _RCC 2dup
  rccAPB2ENR + or!  rccAPB2RSTR + bic!	\ enable SYSCFG clock, out of reset

  PA8					\ configure PA8 for MCO
  0 over isFunction  0 over isPinOD
  dup notPulled  100MHZ_hs swap isPinSpeed
  $0040:0000 $0760:0000 _RCC rccCFGR + setMask \ divide by 1 from HSE

  PA1 isEtherPin			\ all pins to AF11, Ethernet
  PA2 isEtherPin
  PA7 isEtherPin
  PB5 isEtherPin
  PB8 isEtherPin
  PC1 isEtherPin
  PC2 isEtherPin
  PC3 isEtherPin
  PC4 isEtherPin
  PC5 isEtherPin
  PG11 isEtherPin
  PG13 isEtherPin
  PG14 isEtherPin
  PH2 isEtherPin
  PH3 isEtherPin
  PH6 isEtherPin
  PH7 isEtherPin
  PI10 isEtherPin
((
  PB14 dup isInput  dup notPulled
  $0100 _SYSCFG sysEXTICR4 + !		\ connect PB14 to EXTI
  bit14 _EXTI  2dup extiFTSR + !	\ happens on falling edge
  extiIMR + !				\ enable interrupt
))

  bit25 _RCC rccAHB1RSTR + or!		\ MAC into reset
\ [ RMII? ] [if]			\ MII only
\  bit23 _SYSCFG sysPMC + or!
\ [else]
  bit23 _SYSCFG sysPMC + bic!
\ [then]
  bit25 _RCC rccAHB1RSTR + bic!		\ MAC out of reset
  $0E00:0000 _RCC rccAHB1ENR + or!	\ enable MAC clocks except PTP
  _ETH
  bit0 over ethDMABMR + or!		\ software reset of DMA and MAC
  begin
    dup ethDMABMR + @ bit0 and 0=
  until
  drop
;


\ ******
\ *> ###
\ ******

decimal

