\ SP429disco.ctl - STM32F429I Discovery board standalone Forth with SockPuppet interface

((
Copyright (c) 2011, 2012, 2014, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20160226 SFP005 Updated for MPE style.
20160103 HPO004 Updated for STM32F429I Discovery Board.
20140130 SFP003 Updated for Stamp compiler.
20120113 SFP002 Moved USART I/O selection to control file.
20091223 MPE001 First release.
))

\ ================
\ *! sp429disco.ctl
\ *T Control file for STM32F429I Discovery board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32F4/SP429disco.ctl}
\ ** produces a standalone Forth for the STM32F4 Discovery board.
\ ** There is a Sockpuppet interface via SVC calls to the C library
\ ** that initialises the board.
\ ** The serial port is connected to USART1 on pins PA9 (USART1-Tx)
\ ** and PA10 (USART1-Rx). Note that many TTL->RS232/USB adapters
\ ** are labelled with respect to the PC, not the TTL directions.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

only forth definitions  decimal

\ ******************************
\ Define the default directories
\ ******************************

1 constant MPE?         \ MPE macros

MPE? [if]       \ MPE macros
c" ../../.."	setmacro XCroot
mc" %XCroot%/Common"
                setmacro CommonDir	\ where the common kernel code lives
mc" %XCroot%/Cortex"
                setmacro CpuDir		\ where the CPU specific code lives
mc" %XCroot%/Lib"
                setmacro LibDir         \ where library code lives
mc" %XCroot%/Examples"
		setmacro ExampleDir     \ Examples
c" ."
		setmacro HWDir          \ where board specific code lives
c" ."
		setmacro AppDir         \ where application code lives
mc" %LibDir%/FATfiler"
		setmacro FFDir          \ where FATFiler lives
mc" %LibDir%/PowerNet"
		setmacro IpStack        \ where PowerNet lives
mc" %LibDir%/PowerView"
		setmacro GuiDir         \ GUI program
mc" %LibDir%/SockPuppet"
		setmacro SPdir		\ SockPuppet demo and examples
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros

\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if] +xrefs [then]    \ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M4                     \ Thumb2 processor type and register usage
  +LongCalls                    \ permit standalone Forth to handle
                                \ calls outside 25 bit range.
  Hex-I32                       \ also produce Intel Hex-32 obj format
  +SaveCdataOnly                \ no data area image files

0 equ false
-1 equ true

\ *******************
\ *S Configure target
\ *******************

\ ==============================
\ *N STM32F4 variant definitions
\ ==============================

#429 equ STM32type	\ -- x
\ *G Define CPU type for Flash programming.

#1024 kb equ /Flash      \ -- len
\ *G Size of Flash.

#128 cells equ /ExcVecs \ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system and USB clocks are generated from a PLL. The PLL
\ ** input clock must be in the range 1-2 MHz. The upper
\ ** end is preferred to reduce PLL jitter. The PLL output
\ ** frequency must be in the range 64..432 MHz.
\ ** To use USB at 48 MHz, you need to choose a PLL frequency
\ ** that is an integer multiple of 48 MHz. The system speed
\ ** is also an integer division of the VCO speed.

8 MHz equ xtal-speed    \ -- hz
\ *G Master oscillator crystal clock rate in HZ.
2 MHz equ vco-in-speed          \ -- hz
\ *G Requested VCO input frequency.
360 MHz equ vco-out-speed       \ -- hz
\ *G Requested VCO output frequency.
\ ** Suitable VCO speeds for use with USB are 48, 96, 144, 192,
\ ** 240, 288, 336, 384, 432 MHz.
180 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values. The VCO output frequency
\ ** can only be divided by 2, 4, 6 or 8.

xtal-speed vco-in-speed / equ PLLM      \ -- u
\ *G Divide the *\fo{xtal-speed} by the VCO input speed to produce
\ ** the required divisor.
vco-out-speed vco-in-speed / equ PLLN   \ -- u
\ *G Multiply the VCO input frequency by this value to produce the VCO
\ ** output frequency.
vco-out-speed system-speed / equ PLLP   \ -- u
\ *G Divide the VCO output by this ration to produce the system
\ ** clock. The VCO output frequency can only be divided by 2, 4,
\ ** 6 or 8.
vco-out-speed 48 MHz / equ PLLQ \ -- u
\ *G At a suitable VcoClk, this gives a USB OTG FS clock of 48 MHz.
\ ** Suitable VCO speeds are 48, 96, 144, 192, 240, 288, 336,
\ ** 384, 432 MHz.

1 equ AHBdiv    \ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** The AHB clock may not be more than 180 MHz.
2 equ APB2div   \ -- u
\ *G Division ratio of the APB2 clock from the system clock.
\ ** The APB2 clock may not be more than 90 MHz.
4 equ APB1div   \ -- u
\ *G Division ratio of the APB1 clock from the system clock.
\ ** The APB1 clock may not be more than 45 MHz.

system-speed AHBdiv / equ AHB-speed     \ -- hz
\ *G AHB bus speed.
system-speed APB2div / equ APB2-speed   \ -- hz
\ *G APB2 bus speed.
system-speed APB1div / equ APB1-speed   \ -- hz
\ *G APB1 bus speed.

\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2kb (the
\ ** vector area) is mirrored at $0000:0000.

  $0810:0000 $0817:FFFF cdata section SP429disco  \ code section in boot Flash, top 1 Mbyte
  $2002:0000 $2002:1FFF idata section PROGd     \ 8k IDATA , top 128K
  $2002:2000 $2002:FFFF udata section PROGu     \ 56k UDATA, top 128K

interpreter
: prog SP429disco  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE               \ size of each task's user area
$0100 equ SP-SIZE               \ size of each task's data stack
$0080 equ FP-SIZE		\ size of each task's float stack
$0180 equ RP-SIZE               \ size of each task's return stack
up-size rp-size + sp-size + fp-size +
  equ task-size                 \ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard                 \ can underflow the data stack by this amount

$0100 equ TIB-LEN               \ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
1 equ #IRQs                     \ number of IRQ stacks,
                                \ shared by all IRQs (1 min)
0 equ #SVCs                     \ number of Forth SVC nestings permitted
                                \ 0 is ok if Forth SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUSART1?        \ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART2?        \ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART3?        \ -- n
\ *G Set non-zero to compile code for USART3, device *\fo{Console3}.
\ ** Do not enable UART3 - it uses the Data lines of the SDRAM
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUART4? \ -- n
\ *G Set non-zero to compile code for UART4,
\ ** device *\fo{Console4}.
#9600 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

1 equ console-port      \ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..6 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#10 equ tick-ms         \ -- ms
\ *G Timebase tick in ms.

\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?               \ nz to use cold chain mechanism
 1 equ tasking?                 \ true if multitasker needed
   6 cells equ tcb-size         \   for internal consistency check
   0 equ event-handler?         \   true to include event handler
   0 equ message-handler?       \   true to include message handler
   1 equ semaphores?            \ true to include semaphores
 0 equ timebase?                \ true for TIMEBASE code
 0 equ softfp?                  \ true for software floating point
 0 equ FullCase?                \ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?           \ true if target local variable sources needed
 0 equ romforth?                \ true for ROMForth handler
 0 equ blocks?                  \ true if BLOCK needed
 $0000 equ sizeofheap           \ 0=no heap, nz=size of heap
   1 equ heap-diags?            \   true to include diagnostic code
 0 equ paged?                   \ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?             \ true if ANS ENVIRONMENT system required

 0 equ PowerNet?        \ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.

\ *****************
\ default constants
\ *****************

cell equ cell                           \ size of a cell (16 bits)
0 equ false
-1 equ true

\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM4Def               \ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F4xx         \ STM32F4 special function registers
  include %CpuDir%/StackDef             \ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG  \ top of memory for UNUSED
  include %HwDir%/startSP429disco	\ STM32F429 startup code
1 equ FastCmove?
  include %CpuDir%/CodeCortex           \ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootCM4    \ reboot using CM4 core
\  include %CpuDir%/IntCortex            \ interrupt handlers for NVIC
\  include %CpuDir%/FaultCortex          \ fault exception handlers for NVIC
: selio-ser1	\ --
\ *G Example to perform clock and pin selection for USART1 on PA9 (TX) and
\ ** PA10 (RX).
\ Clocks and reset
  _RCC
  bit0 over rccAHB1ENR + or!		\ enable clock to GPIOA
  bit0 over rccAHB1RSTR + bic!		\ take it out of reset
  bit4 over rccAPB2ENR + or!		\ enable clock to USART1
  bit4 over rccAPB2RSTR + bic!		\ take it out of reset
  drop
\ Default on PA9/10
  _GPIOA >r
  $0028:0000 $003C:0000 r@ gpioMODER + setMask	\ alternate function on 9/10
  $0000:0770 $0000:0FF0 r> gpioAFRH + setMask	\ AF7 for both
;
\  include %CommonDir%/CQueues           \ character queues
\  include %CpuDir%/Drivers/serSTM32F429qi \ RX/TX queued interrupt driver
  include %CpuDir%/Drivers/serSTM32F4xxp \ polled serial driver
  include %CommonDir%/Devtools          \ DUMP .S etc development tools
  include %CommonDir%/Voctools          \ ORDER VOCS etc
  include %CommonDir%/methods           \ target support for methods
  include %CpuDir%/LocalCM3             \ local variables

tasking? [if]
  include %CpuDir%/MultiCortex          \ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

  include %SPdir%/SockPuppetInterface   \ access C library from Forth
  include %SPdir%/Dylink		\ access shared C variables from Forth

timebase? [if]
  include %CommonDir%/timebase          \ time base common code, MUST be before SysTickxxx
  include %CommonDir%/Delays            \ time delays
[else]
  include %CommonDir%/Delays
[then]


environment? [if]
  include %CommonDir%/environ           \ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32            \ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex          \ floating point primitives - coded version
  include %CommonDir%/sfp32com          \ common floating point code
[then]

\ *************
\ *S End of kernel
\ *************

buildfile SP429disco.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner       \ --
  cr ." **********************************************"
;

: .CPU          \ -- ; display CPU type
  .banner
  cr ." MPE SockPuppet PowerForth for STM32F429I Disco"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH     \ -- ; marker
;

\ *******************
\ *S Application code
\ *******************

  include %CpuDir%/Drivers/gpioSTM32F42x	\ easy GPIO access
\  include %HwDir%/FlashProg		\ Flash programming code
\  include %HwDir%/ReProg/ReFlash	\ ReFlash utility

\  include %FFDir%/STM32F4disco.bld	\ FATFiler build file

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
  include %CommonDir%/RomForth/iodef    \ link i/o
  include %CommonDir%/RomForth/textfile \ AIDE textfile support

  include %SPdir%/Touchscreen		\ higher level touchscreen words
  include %GuiDir%/STM32F429disco.bld	\ display bitmaps etc

((
  cr ." Type : 'DemoMenu'  or  'DemoPics'"

  AppSignOnMessage

  CheckForButtonPressed
))

PowerNet? [if]
\  1 equ CPU=Cortex     \ to compile Cortex specific code
  0 equ FilePages?                      \ no file system of HTTP page storage
  include %AppDir%/PNconfig             \ PowerNet configuration
  include %IpStack%/NetCode             \ network order stuff
  include %AppDir%/WebConfig            \ configuration tools
  include %CpuDir%/drivers/EtherSTM32   \ Ethernet driver
  include %IpStack%/PowerNet.bld        \ PowerNet build
  include %IpStack%/Examples/WebPost    \ application update

[else]
((
1 equ CPU=Cortex        \ to compile Cortex specific code
1 equ EthDiags?
1 equ Sniff?
1 equ Pingit?
  include %ExampleDir%/NetCode          \ network order stuff
  include %CpuDir%/drivers/EtherSTM32   \ Ethernet driver
))
[then]

\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%/benchmarks/xbench32      \ performance tests

  include %CommonDir%/debugchan		\ Use additional AIDE displays


\ ***************
\ *S Finishing up
\ ***************

libraries       \ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 128 byte unit for REFLASH.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $07F +  $-080 and org		\ force to 128 byte boundary for REFLASH

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build

afterwards sh "C:\MyApps\ST-Linkv4.0.0\ST-LINK Utility\ST-LINK_CLI.exe"  -P SP429base.hex  -Rst
afterwards sh "C:\MyApps\ST-Linkv4.0.0\ST-LINK Utility\ST-LINK_CLI.exe"  -P SP429DISCO.hex  -Rst

FINIS

\ ======
\ *> ###
\ ======

