\ STM32-E407.ctl - Olimex STM32-E407 board standalone Forth with PowerNet

((
Copyright (c) 2011, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20130913 MPE003 Olimex STM32-E407.
20120125 MPE002 STM3240G-EVAL and PowerNet
20091223 MPE001 First release for STM32F1xx.
))


\ ================
\ *! stm32e407
\ *T Control file for Olimex STM32-E407 board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32F4/STM32-E407.ctl}
\ ** produces a standalone Forth for the Olimex STM32-E407 board.
\ ** The serial port is connected to USART3 on pins PB10 (Tx) and
\ ** PB11 (Rx), so allowing programming with the serial bootloader
\ ** as well as the ST-LINK or other JTAG/SWD device. Using a TTL
\ ** level USB to serial cable, e.g. FTDI's TTL-232R-RPi, is very
\ ** convenient.
\ *(
\ *B Yellow -> TX3
\ *B Orange -> RX3
\ *B Black -> Gnd
\ *)

\ *P If you want to use the STM32 Flash Tool with the ST bootloader,
\ ** set the Boot pin links as below:
\ *(
\ *B B0_1
\ *B B1_0 (default)
\ *)
\ *P After programming for the first time, restore the boot pins to:
\ *(
\ *B B0_0 (default)
\ *B B1_0 (default)
\ *)

\ *P If you modify this control file for your own hardware do not
\ ** forget to update the GPIO pin assignments and alternate
\ ** function selections.


only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

1 constant MPE?		\ select macro set

MPE? [if]	\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives
mc" %ExampleDir%/FATfiler"
		setmacro FFDir		\ FAT file system
mc" %ExampleDir%/PowerView"
		setmacro GuiDir		\ where GUI code lives
[defined] Target_386_OSX [if]
  c" /Users/stephen/buildkit.dev/software/AddOns/PowerNet/Dev"
[then]
[defined] Target_386_Windows [if]
  c" /buildkit.dev/software/AddOns/PowerNet/Dev"
[then]
[defined] Target_386_Linux [if]
  c" /home/stephen/buildkit.dev/software/AddOns/PowerNet/Dev"
[then]
		setmacro IpStack	\ where PowerNet lives
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M4			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N STM32F4 variant definitions
\ ======================

0 equ PrefetchOK?	\ -- flag
\ *G The instruction prefetch queue does not work on Rev A devices.

#512 kb equ /Flash	\ -- len
\ *G Size of Flash that is used/erased by the reprogrammer.

$0800:0000 equ FlashBase	\ -- addr
\ *G Base address of the Flash.

#128 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system and USB clocks are generated from a PLL. The PLL
\ ** input clock must be in the range 1-2 MHz. The upper
\ ** end is preferred to reduce PLL jitter. The PLL output
\ ** frequency must be in the range 64..432 MHz.
\ ** To use USB at 48 MHz, you need to choose a PLL frequency
\ ** that is an integer multiple of 48 MHz. The system speed
\ ** is also an integer division of the VCO speed.

12 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
1 MHz equ vco-in-speed		\ -- hz
\ *G Requested VCO input frequency.
336 MHz equ vco-out-speed	\ -- hz
\ *G Requested VCO output frequency.
168 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values. The VCO output frequency
\ ** can only be divided by 2, 4, 6 or 8.

xtal-speed vco-in-speed / equ PLLM	\ -- u
\ *G Divide the *\fo{xtal-speed} by the VCO input speed to produce
\ ** the required divisor.
vco-out-speed vco-in-speed / equ PLLN	\ -- u
\ *G Multiply the VCO input frequency by this value to produce the VCO
\ ** output frequency.
vco-out-speed system-speed / equ PLLP	\ -- u
\ *G Divide the VCO output by this value to produce the system
\ ** clock. The VCO output frequency can only be divided by 2, 4,
\ ** 6 or 8.
vco-out-speed 48 MHz / equ PLLQ	\ -- u
\ *G At VcoClk=336 MHz, this gives a USB OTG FS clock of 48 MHz.
\ ** Suitable VCO speeds are 48, 96, 144, 192, 240, 288, 336,
\ ** 384, 432 MHz.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 168 MHz.
2 equ APB2div	\ -- u
\ *G Division ratio of the APB2 clock from the system clock.
\ ** This may not be more than 84 MHz.
4 equ APB1div	\ -- u
\ *G Division ratio of the APB1 clock from the system clock.
\ ** This may not be more than 42 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APB2div / equ APB2-speed	\ -- hz
\ *G APB2 bus speed.
system-speed APB1div / equ APB1-speed	\ -- hz
\ *G APB1 bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2kb (the
\ ** vector area) is mirrored at $0000:0000.

\  $0800:0000 equ link-address	\ Address at which code will start\be linked at
  $0800:0000 $0807:FFFF cdata section STM32-E407	\ code section in boot Flash
  $2000:0000 $2000:0FFF idata section PROGd	\ 4k IDATA
  $2000:1000 $2001:FFFF udata section PROGu	\ 124k UDATA
  $1000:0000 $1000:FFFF udata section CCMRAM	\ 64k UDATA

interpreter
: prog STM32-E407  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0300 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

0 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

1 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
#115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUART4?	\ -- n
\ *G Set non-zero to compile code for UART4,
\ ** device *\fo{Console4}.
#9600 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

3 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..6 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ *[
\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required
 \ *]

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.
\ ** Note that the heap must also be used (32 kb = $8000).
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

\ *[
  include %CpuDir%/CM4Def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F4xx		\ STM32F4 special function registers
[sections CCMRAM cdata			\ improves performance by about 10%
  include %CpuDir%/StackDef		\ Reserve default task and stacks
sections]

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F407	\ STM32F407 startup code
1 equ FastCmove?
1 equ dsqrt?
  include %CpuDir%/CodeCortex		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootCM4	\ reboot using CM4 core
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
  include %CommonDir%/CQueues		\ character queues
\ *]

: selio-ser3	\ --
\ *G Initialise USART3 clocks and pins for operation on GPIO pins
\ ** PB10/11. Note that the serial initialisation code cannot use
\ ** the easy pin I/O code as the serial initialisation code is
\ ** run before the cold chain is run.
  _RCC
  bit1 over rccAHB1ENR + or!		\ enable clock to GPIOB
  bit1 over rccAHB1RSTR + bic!		\ take it out of reset
  bit18 over rccAPB1ENR + or!		\ enable clock to USART3
  bit18 over rccAPB1RSTR + bic!		\ take it out of reset
  drop
\ Default on PB10/11
  _GPIOB >r
  $00A0:0000 $00F0:0000 r@ gpioMODER + setMask	\ alternate function on 10/11
  $0000:7700 $0000:FF00 r> gpioAFRH + setMask	\ AF7 for both
;

\ *[
  include %CpuDir%/Drivers/serSTM32F4xxqi \ RX/TX queued interrupt driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM3		\ local variables

tasking? [if]
  include %CpuDir%/MultiCortex		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before SysTickxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]
\ *]


\ *************
\ *S End of kernel
\ *************

buildfile STM32-E407.no
\ *G The host file that contains the build numbering.
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ --
\ *G Display CPU type and build information.
  .banner
  cr ." MPE ROM PowerForth for Cortex-M4"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ --
\ *G Marker and claim.
;


\ *******************
\ *S Application code
\ *******************

\ *[
  include %CpuDir%/Drivers/rtcSTM32F4xx	\ RTC
  include %HwDir%/FlashProg		\ Flash programming code
  include %HwDir%/ReProg/ReFlash	\ ReFlash utility
  include %CpuDir%/Drivers/gpioSTM32F4xx  \ easy GPIO access

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
\  include %CommonDir%/XmodemTxRx	\ Xmodem driver
  include %CommonDir%/RomForth/iodef	\ link i/o
  include %CommonDir%/RomForth/textfile	\ AIDE textfile support

  include %FFDir%/STM32-E407.bld	\ SD card and FAT file system

PowerNet? [if]
1 equ CPU=Cortex	\ to compile Cortex specific code
1 equ RMII?		\ Ethernet operates with RMII to the PHY.
0 equ EEPROMconfig?	\ Set non-zero to use EEPROM configuration.
1 equ Flashconfig?	\ Set non-zero to use Flash configuration.
  include %AppDir%/EvalPNcfg		\ PowerNet configuration
  include %IpStack%/NetCode		\ network order stuff
  include %AppDir%/WebConfig		\ configuration tools
0 equ sniff?
  include %HwDir%/HwEther-E407		\ Ethernet set up
  include %CpuDir%/drivers/EtherSTM32F4xx \ Ethernet driver
  include %IpStack%/PowerNet.bld	\ PowerNet build
  include %IpStack%/Examples/WebPost	\ application update
[else]
((
1 equ XC?
1 equ CPU=Cortex			\ to compile Cortex specific code
1 equ RMII?		\ Ethernet operates with RMII to the PHY.
1 equ EtherDiags?
1 equ Sniff?
1 equ Pingit?
  include %ExampleDir%/NetCode		\ network order stuff
  include %HwDir%/HwEther-E407		\ Ethernet set up
  include %CpuDir%/drivers/EtherSTM32F4xx \ Ethernet driver
))
[then]
\ *]


\ **********************
\ *S Select example code
\ **********************
\ *[
\ include %ExampleDir%/benchmarks/xbench32	\ performance tests

\ include %CommonDir%/TESTS/ANS/tester		\ John Hayes' ANS test engine
\ include %CommonDir%/TESTS/MPE/5-1-a		\ MPE test engine
\ include %ExampleDir%/TESTS/MPE/5-1-b		\ MPE tests

\ include %CommonDir%/TESTS/MPE/orbits		\ Sun-Earth-Moon simulation
\ include %CommonDir%/TESTS/MPE/bench		\ Sieve of Eratosthenes
\ include %CommonDir%/TESTS/MPE/str_loc		\ structures and local variables
\ include %CommonDir%/TESTS/MPE/checksum	\ checksum / crc
\ include %CommonDir%/TESTS/MPE/buildno		\ build number

\ include %CommonDir%/TESTS/MPE/testtask
\ include %CommonDir%/TESTS/MPE/testheap
\ *]

\ ***************
\ *S Finishing up
\ ***************
\ *[

((
: startAll	\ --
\ *G Start selected software.
  initFATfs abort" Can't start file system"
  PowerNet
;
))

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

\ Define device, speed (0=auto) and project file
\ Download the target image when compilation finishes
  jlink STM32F407ZG 0 Project.jlink
  afterwards  download-sections		\ download image
\  afterwards  initJLink		\ and start JLink again

update-build
FINIS
\ *]

\ ======
\ *> ###
\ ======

