\ STM32F4disco.ctl - STM32F407 Discovery board standalone Forth

((
Copyright (c) 2011, 2012, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140130 SFP003 Updated for Stamp compiler.
20120113 SFP002 Moved USART I/O selection to control file.
20091223 MPE001 First release.
))


\ ================
\ *! stm32f4disco
\ *T Control file for STM32F4 Discovery board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32F4/STM32F4disco.ctl}
\ ** produces a standalone Forth for the STM32F4 Discovery board.
\ ** The serial port is connected to USART3 on pins PD8 (USART3-Tx)
\ ** and PD9 (USART3-Rx). Note that many TTL->RS232/USB adapters
\ ** are labelled with respect to the PC, not the TTL directions.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

1 constant MPE?		\ MPE macros
0 constant OMRT?	\ OpenMRT macros

MPE? [if]	\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives
c" ../../../Examples/FATfiler"
		setmacro FFDir		\ where FATFiler lives
c" c:/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]

OMRT? [if]
c" c:\working\xArmCortex\Common"
		setmacro CommonDir	\ where the common code lives
c" c:\working\xArmCortex/Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."  setmacro HWDir			\ where board specific code lives
c" c:\working\xArmCortex/Examples"
		setmacro ExampleDir	\ examples
c" ."  setmacro AppDir			\ where application code lives
c" c:\working\xArmCortex/FATFiler"
		setmacro FFDir		\ where FATFiler lives
c" c:\working\xArmCortex\PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if] +xrefs [then]	\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M4			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ==============================
\ *N STM32F4 variant definitions
\ ==============================

0 equ PrefetchOK?	\ -- flag
\ *G The instruction prefetch queue does not work on Rev A devices.

#512 kb equ /Flash	\ -- len
\ *G Size of Flash.

#128 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system and USB clocks are generated from a PLL. The PLL
\ ** input clock must be in the range 1-2 MHz. The upper
\ ** end is preferred to reduce PLL jitter. The PLL output
\ ** frequency must be in the range 64..432 MHz.
\ ** To use USB at 48 MHz, you need to choose a PLL frequency
\ ** that is an integer multiple of 48 MHz. The system speed
\ ** is also an integer division of the VCO speed.

8 MHz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
2 MHz equ vco-in-speed		\ -- hz
\ *G Requested VCO input frequency.
336 MHz equ vco-out-speed	\ -- hz
\ *G Requested VCO output frequency.
168 MHz equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values. The VCO output frequency
\ ** can only be divided by 2, 4, 6 or 8.

xtal-speed vco-in-speed / equ PLLM	\ -- u
\ *G Divide the *\fo{xtal-speed} by the VCO input speed to produce
\ ** the required divior.
vco-out-speed vco-in-speed / equ PLLN	\ -- u
\ *G Multiply the VCO input frequency by this value to produce the VCO
\ ** output frequency.
vco-out-speed system-speed / equ PLLP	\ -- u
\ *G Divide the VCO output by this ration to produce the system
\ ** clock. The VCO output frequency can only be divided by 2, 4,
\ ** 6 or 8.
vco-out-speed 48 MHz / equ PLLQ	\ -- u
\ *G At VcoClk=336 MHz, this gives a USB OTG FS clock of 48 MHz.
\ ** Suitable VCO speeds are 48, 96, 144, 192, 240, 288, 336,
\ ** 384, 432 MHz.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 168 MHz.
2 equ APB2div	\ -- u
\ *G Division ratio of the APB2 clock from the system clock.
\ ** This may not be more than 84 MHz.
4 equ APB1div	\ -- u
\ *G Division ratio of the APB1 clock from the system clock.
\ ** This may not be more than 42 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APB2div / equ APB2-speed	\ -- hz
\ *G APB2 bus speed.
system-speed APB1div / equ APB1-speed	\ -- hz
\ *G APB1 bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2kb (the
\ ** vector area) is mirrored at $0000:0000.

Stamp? [if]
  $0800:0000 $0801:DFFF cdata section STM32F4disco	\ code section in boot Flash
[else]
  $0800:0000 $0807:FFFF cdata section STM32F4disco	\ code section in boot Flash
[then]
  $2000:0000 $2000:1FFF idata section PROGd	\ 8k IDATA
  $2000:2000 $2000:FFFF udata section PROGu	\ 56k UDATA

interpreter
: prog STM32F4disco  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
$0200 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

0 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

1 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUART4?	\ -- n
\ *G Set non-zero to compile code for UART4,
\ ** device *\fo{Console4}.
#9600 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

3 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..6 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#10 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM4Def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F4xx		\ STM32F4 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F407	\ STM32F407 startup code
0 equ FastCmove?
  include %CpuDir%/CodeCortex		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootCM4	\ reboot using CM4 core
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
  include %CommonDir%/CQueues		\ character queues
: selio-ser3	\ --
\ *G Initialise USART3 for operation on GPIO pins PD8 (Tx), PD9(Rx).
\ ** Note that the serial initialisation code cannot use
\ ** the easy pin I/O code as the serial initialisation code is
\ ** run before the cold chain is run.
  _RCC
  bit3 over rccAHB1ENR + or!		\ enable clock to GPIOD
  bit3 over rccAHB1RSTR + bic!		\ take it out of reset
  bit18 over rccAPB1ENR + or!		\ enable clock to USART3
  bit18 over rccAPB1RSTR + bic!		\ take it out of reset
  drop
\ Default on PD8/9
  _GPIOD >r
  $000A:0000 $000F:0000 r@ gpioMODER + setMask	\ alternate function on 8/9
  $0000:0077 $0000:00FF r> gpioAFRH + setMask	\ AF7 for both
;
  include %CpuDir%/Drivers/serSTM32F4xxqi \ RX/TX queued interrupt driver
\  include %CpuDir%/Drivers/serSTM32F4xxp \ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM3		\ local variables

tasking? [if]
  include %CpuDir%/MultiCortex		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before SysTickxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]


\ *************
\ *S End of kernel
\ *************

buildfile STM32F4disco.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M4"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  include %CpuDir%/Drivers/gpioSTM32F4xx  \ easy GPIO access
  include %HwDir%/FlashProg		\ Flash programming code
  include %HwDir%/ReProg/ReFlash	\ ReFlash utility

\  include %FFDir%/STM32F4disco.bld	\ FATFiler build file

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
\  include %CommonDir%/XmodemTxRx	\ Xmodem driver
  include %CommonDir%/RomForth/iodef	\ link i/o
  include %CommonDir%/RomForth/textfile	\ AIDE textfile support

PowerNet? [if]
\  1 equ CPU=Cortex	\ to compile Cortex specific code
  0 equ FilePages?			\ no file system of HTTP page storage
  include %AppDir%/PNconfig		\ PowerNet configuration
  include %IpStack%/NetCode		\ network order stuff
  include %AppDir%/WebConfig		\ configuration tools
  include %CpuDir%/drivers/EtherSTM32	\ Ethernet driver
  include %IpStack%/PowerNet.bld	\ PowerNet build
  include %IpStack%/Examples/WebPost	\ application update

[else]
((
1 equ CPU=Cortex	\ to compile Cortex specific code
1 equ EthDiags?
1 equ Sniff?
1 equ Pingit?
  include %ExampleDir%/NetCode		\ network order stuff
  include %CpuDir%/drivers/EtherSTM32	\ Ethernet driver
))
[then]



\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%/benchmarks/xbench32	\ performance tests

\ include %CommonDir%/TESTS/ANS/tester		\ John Hayes' ANS test engine
\ include %CommonDir%/TESTS/MPE/5-1-a		\ MPE test engine
\ include %ExampleDir%/TESTS/MPE/5-1-b		\ MPE tests

\ include %CommonDir%/TESTS/MPE/orbits		\ Sun-Earth-Moon simulation
\ include %CommonDir%/TESTS/MPE/bench		\ Sieve of Eratosthenes
\ include %CommonDir%/TESTS/MPE/str_loc		\ structures and local variables
\ include %CommonDir%/TESTS/MPE/checksum	\ checksum / crc
\ include %CommonDir%/TESTS/MPE/buildno		\ build number

\ include %CommonDir%/TESTS/MPE/testtask
\ include %CommonDir%/TESTS/MPE/testheap

  include %CommonDir%/debugchan


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 512 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $1FF +  $-0200 and org		\ force to 512 byte boundary for ISP loader

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build
FINIS


\ ======
\ *> ###
\ ======

