\ StartSTM32F429-HI.fth - Cortex startup code for STM32F429 CPUs to run in the high 1 M byte of FLASH

((
Copyright (c) 2009, 2010, 2011, 2012, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20120720 MPE002 Updated for external oscillators.
20100112 MPE001 First release.
))

only forth definitions  decimal

\ =================
\ *! startstm32f407
\ *T Cortex start up for STM32F407
\ =================

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard
[then]

init-s0 tos-cached? sp-guard + cells -
  equ real-init-s0	\ -- addr
\ The data stack pointer set at start up.

0 [if]   \ initialisation is done in the C system

interpreter
: prediv4	\ u -- x
\ *G Convert a divider value to a 4 bit pattern as used for HPRE.
  case
    1 of  %0000  endof
    2 of  %1000  endof
    4 of  %1001  endof
    8 of  %1010  endof
    16 of  %1011  endof
    64 of  %1100  endof
    128 of  %1101  endof
    256 of  %1110  endof
    512 of  %1111  endof
      true abort" Invalid AHB prescaler setting"
  endcase
;

: prediv3	\ u -- x
\ *G Convert a divider value to a 3 bit pattern as used for PPRE1 and PPRE2.
  case
    1 of  %000  endof
    2 of  %100  endof
    4 of  %101  endof
    8 of  %110  endof
    16 of  %111  endof
      true abort" Invalid PPREx prescaler setting"
  endcase
;

: PLLPval	\ -- x
\ *G Return the required PLLP bit pattern from the equate *\fo{PLLP}.
  case PLLP
    2 of  %00  endof
    4 of  %01  endof
    6 of  %10  endof
    8 of  %11  endof
      true abort" Invalid PLLP setting"
  endcase
;
target

0
AHBdiv prediv4 4 lshift or
APB1div prediv3 10 lshift or
APB2div prediv3 13 lshift or
  equ initCFGRval		\ -- x
\ *G Bit settings for the RCC_CFGR register.
$0000:FCF0 equ initCFGRmask	\ -- x
\ *G Bit mask for the RCC_CFGR register.

PLLQ #24 lshift
bit22 or		\ select HSE
PLLPval #16 lshift or
PLLN 6 lshift or
PLLM or
  equ initPLLCFGRval	\ --
\ *G Initial bit settings for PLL values in RCC_PLLCFGR.
$0F43:7FFF equ initPLLCFGRmask	\ --
\ *G Bit mask settings for PLL values in RCC_PLLCFGR.

\ *P The number of wait states required by the Flash depends on
\ ** the operating voltage.
  30 MHz equ /FlUnit	\ for 2.7 to 3.6 volts
\ 24 MHz equ /FlUnit	\ for 2.4 to 2.7 volts
\ 18 MHz equ /FlUnit	\ for 2.1 to 2.4 volts
\ 16 MHz equ /FlUnit	\ for 1.8 to 2.1 volts
system-speed 1- /FlUnit / equ FLWS	\ -- u
\ *G The number of Flash wait states required.

PrefetchOK? [if] bit8 ( PRFTEN ) [else] 0 [then]  \ No prefetch on rev A devices
bit10 ( DCEN ) or  bit9 ( ICEN ) or  FLWS or
  equ initACRval	\ -- x
\ *G Initial value written to the Flash ACR register.

[undefined] HSE-ExtClk? [if]
0 equ HSE-ExtClk?	\ -- 0
\ *G If not already defined, this equate is set to zero to select
\ ** HSE using the internal crystal driver.
[then]

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLL
\ ** as required.
\ ** By default, the AHB and APB2 busses are run at SYSCLK,
\ ** and APB1 is run at SYSCLK/2.
\ ** HPO: maybe not anymore, it follows the C config.

  $0000:0001 _RCC rccCR + or!		\ enable HSI only
  0 _RCC rccCFGR + !			      \ force to reset config
  $0109:0000 _RCC rccCR + bic!   \ clear PLLON, CSSON, HSEON
  $2400:3010 _RCC rccPLLCFGR + !	\ reset PLLCFGR register

[ HSE-ExtClk? ] [if]
\  $0004:0000 _RCC rccCR + or!		\ set HSEBYP (HSEON=0 now)
[else]
  $0004:0000 _RCC rccCR + bic!		\ clear HSEBYP (HSEON=0 now)
[then]
  $00BF:0000 _RCC rccCIR + !		\ Disable  ints, clear pending bits

  bit16 _RCC rccCR + or!		\ set HSEON
  begin
    _RCC rccCR + @ bit17 and		\ wait for HSERDY
  until

  bit28 _RCC rccAPB1ENR + or!		\ enable PWR module
  bit14 _PWR pwrCR + or!		\ scale 1 mode, whatever that means

\ set prescalers and PLL configuration and start the PLL
  initCFGRval initCFGRmask 2 pick rccCFGR + setMask	\ set AHB, APB1, APB2 prescalers

  initPLLCFGRval initPLLCFGRmask _RCC rccPLLCFGR + setMask	\ set PLL config
  bit24 _RCC rccCR + or!		\ set PLLON
  begin
    _RCC rccCR + @ bit25 and		\ wait for PLLRDY
  until

\ Set up the Flash configuration
  initACRval _FPEC flACR + !		\ Enable caches and set wait states
\ switch to the PLL output
  $0002 $0003 _RCC rccCFGR + setMask	\ set SW=2

  begin
    _RCC rccCFGR + @ $000C and $0008 =	\ wait for SWS=2
  until
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

: +FPU		\ --
\ *G Enable the FPU by enabling full access to CP10 and CP11.
  $00F0:0000 _SCS scsCPACR + or!
;

[then]   \ initialisation is done in the C system

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation can also be performed here.
  begin
  \ HPO use the C stacks :
\    INT_STACK_TOP SP_main sys!	\ set SP_main for interrupts
\    INIT-R0 SP_process sys!		\ set SP_process for tasks
\    2 control sys!			      \ switch to SP_process
    REAL-INIT-S0 set-sp			   \ Allow for cached TOS and guard space
    INIT-U0 up!				      \ USER area

    \ setClocks

\    setNVIC
\    +FPU
    CLD1 @ execute
  again
;
external

\ INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word HPO Note : we need this!
((
DefExc2       2         SetExcVec
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec
))

\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

