\ LP1294.ctl - Launchpad EK-TM4C1294 board standalone Forth

((
Copyright (c) 2011, 2012, 2014, 2016, 2019
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20190204 SFP005 TM4C1294 conversion
20160102 SFP004 STM32F429 Discovery conversion.
20140130 SFP003 Updated for Stamp compiler.
20120113 SFP002 Moved USART I/O selection to control file.
20091223 MPE001 First release.
))


\ =================
\ *! lp1294
\ *T Control file for Launchpad EK-TM4C1294 board
\ =================
\ *P The control file *\i{Cortex/Hardware/STM32F4/LP1294.ctl}
\ ** produces a standalone Forth for the Launchpad EK-TM4C1294 board.
\ ** The serial port is connected to USART1 on pins PA9 (USART1-Tx)
\ ** and PA10 (USART1-Rx). These are free pins on the Discovery board.
\ ** The serial port may alternatively be connected to USART3 on
\ ** pins PD8 (USART3-Tx) and PD9 (USART3-Rx), but this conflicts
\ ** with the SDRAM connections.
\ ** Note that many TTL->RS232/USB adapters
\ ** are labelled with respect to the PC, not the TTL directions.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

1 constant MPE?		\ MPE macros

MPE? [if]	\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives
c" ../../../Examples/FATfiler"
		setmacro FFDir		\ where FATFiler lives
c" c:/buildkit.dev/software/AddOns/PowerNet/Dev"
		setmacro IpStack	\ where PowerNet lives
[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
Stamp? 0= [if] +xrefs [then]	\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M4			\ Thumb2 processor type and register usage
  +LongCalls			\ permit standalone Forth to handle
  				\ calls outside 25 bit range.
  setBinExt .bin		\ binary files have .bin extension
\  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files
  -const-visible		\ CONST = EQU

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 1 equ hardfp?			\ true for hardware floating point
 1 equ fpstack?			\ true for a separate floating point stack,
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required
 1 equ DeferPrompt?		\ true if DEFERred prompt needed.

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ ==============================
\ *N TM4C variant definitions
\ ==============================

#1294 equ TM4type	\ -- x
\ *G Define CPU type for Flash programming.

#1024 kb equ /Flash	\ -- len
\ *G Size of Flash.

#128 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system and USB clocks are generated from a PLL. The PLL
\ ** input clock must be in the range 4-30 MHz. The upper
\ ** end is preferred to reduce PLL jitter. The PLL output
\ ** frequency must be in the range 64..432 MHz.
\ ** To use USB at 60 MHz, you need to choose a PLL frequency
\ ** that is an integer multiple of 60 MHz, usually 480 MHz, The system speed
\ ** is also an integer division of the VCO speed.

25 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ. This is the main
\ ** oscillator (MOSC) in the data sheet. This assumed to be the
\ ** frequency used when the PLL is bypassed.
5 MHz equ vco-in-speed		\ -- hz
\ *G Requested VCO input frequency, which should be in the range
\ ** 4-30 MHz.
480 MHz equ vco-out-speed	\ -- hz
\ *G Requested VCO output frequency.
120 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values.

xtal-speed vco-in-speed / equ PLLQN	\ -- u
\ *G Divide the *\fo{xtal-speed} by the VCO input speed to produce
\ ** the required divior. *\fo{PLLQN} must be less than 32. N is
\ ** always set to 0 (divide by 1).
vco-out-speed vco-in-speed / equ PLLMINT	\ -- u
\ *G Multiply the VCO input frequency by this value to produce the VCO
\ ** output frequency. Must be less than 1024.
vco-out-speed 60 MHz / equ PLLUSB	\ -- u
\ *G At suitable VcoClk, this gives a USB OTG FS clock of 60 MHz.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2kb (the
\ ** vector area) is mirrored at $0000:0000.

  $0000:0000 $000F:FFFF cdata section LP1294	\ code section in boot Flash
  $2000:0000 $2000:FFFF idata section PROGd	\ 64k IDATA
  $2001:0000 $2003:FFFF udata section PROGu	\ 192k UDATA

interpreter
: prog LP1294  ;		\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0100 equ UP-SIZE		\ size of each task's user area
$0100 equ SP-SIZE		\ size of each task's data stack
fpstack? [if]
$0100 equ FP-SIZE		\ size of each task's float stack
[else]
$0000 equ FP-SIZE
[then]
$0200 equ RP-SIZE		\ size of each task's return stack
rp-size fp-size + sp-size + up-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
#9600 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.
115200 equ console2-speed
\ *G *\fo{Console2} speed in BPS.


0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..6 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM4Def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrTM4C1294		\ Special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startLP1294		\ startup code
1 equ FastCmove?
  include %CpuDir%/CodeCortex		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %CpuDir%/Drivers/rebootCM4	\ reboot using CM4 core
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
  include %CommonDir%/CQueues		\ character queues
  include %CpuDir%/Drivers/serTM4C12xxqi \ RX/TX queued interrupt driver
\  include %CpuDir%/Drivers/serTM4C12xxp \ polled serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
  include %CpuDir%/LocalCM3		\ local variables

tasking? [if]
  include %CpuDir%/MultiCortex		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before SysTickxxx
  include %CommonDir%/Delays		\ time delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-timers AtCold
[else]
  include %CommonDir%/Delays
  include %CpuDir%/Drivers/SysTickCM4
  ' start-clock AtCold
[then]

environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/sfp32Cortex		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]

hardfp? [if]
  include %CpuDir%/VFP32SX
[then]


\ *************
\ *S End of kernel
\ *************

buildfile LP1294.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M4"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

  include %CpuDir%/Drivers/gpioTM4C12xx  \ easy GPIO access
\  include %HwDir%/ReProg/delays
  include %HwDir%/FlashProg		\ Flash programming code
  include %HwDir%/ReProg/ReFlash	\ ReFlash utility

\  include %FFDir%/STM32F4disco.bld	\ FATFiler build file

  Synonym Ser-Key? key?
  Synonym Ser-Key  key
  Synonym Ser-Emit emit
\  include %CommonDir%/XmodemTxRx	\ Xmodem driver
  include %CommonDir%/RomForth/iodef	\ link i/o
  include %CommonDir%/RomForth/textfile	\ AIDE textfile support

PowerNet? [if]
\  1 equ CPU=Cortex	\ to compile Cortex specific code
  0 equ FilePages?			\ no file system of HTTP page storage
  include %AppDir%/PNconfig		\ PowerNet configuration
  include %IpStack%/NetCode		\ network order stuff
  include %AppDir%/WebConfig		\ configuration tools
  include %CpuDir%/drivers/EtherSTM32	\ Ethernet driver
  include %IpStack%/PowerNet.bld	\ PowerNet build
  include %IpStack%/Examples/WebPost	\ application update

[else]
((
1 equ CPU=Cortex	\ to compile Cortex specific code
1 equ EthDiags?
1 equ Sniff?
1 equ Pingit?
  include %ExampleDir%/NetCode		\ network order stuff
  include %CpuDir%/drivers/EtherSTM32	\ Ethernet driver
))
[then]



\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%/benchmarks/xbench32	\ performance tests

  include %CommonDir%/debugchan
\ include %CpuDir%/Tests/TestAAPCS	\ test AAPCS interfaces

\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibCortex
  include %CommonDir%/LIBRARY
end-libs

\ Force binary file to 128 byte unit for REFLASH.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $07F +  $-080 and org		\ force to 128 byte boundary for REFLASH

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

afterwards sh D:\MyApps\TI\LMFlashProg\LMFlash.exe  -i ICDI -o 0  -v -r  C:\buildkit.dev\software\ROM\Cortex\Hardware\TM4C\LP1294.bin

update-build
FINIS


\ ======
\ *> ###
\ ======

