\ StartLM3s9bxx.fth - Cortex startup code for Luminary LM3S9Bx family

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20100317 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startlm3s9bx
\ *T Cortex start up for LM3S9Bx
\ ===========
\ *P The file *\i{Cortex/Hardware/LM3S9Bx/startLM3S9Bx.fth} contains
\ ** the start up code for LM3S9Bx devices. Start up code for other
\ ** Cortex-M3 implementations, e.g. STM32, can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M3
\ ** is in *\i{Cortex/StartCortex.fth}.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1. *\b{Do not} use this to set the ISP checksum!
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. This
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

internal

((
code ROMcall-1/1	\ param1 priind secind -- res
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r0, [ psp ], # 4		\ r0 = param
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code

code ROMcall-2/1	\ param1 param2 priind secind -- res
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r1, [ psp ], # 4		\ r1 = param2
  ldr     r0, [ psp ], # 4		\ r0 = param1
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code

code ROMcall-3/1	\ param1 param2 param3 priind secind -- res
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r2, [ psp ], # 4		\ r2 = param3
  ldr     r1, [ psp ], # 4		\ r1 = param2
  ldr     r0, [ psp ], # 4		\ r0 = param1
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code
))

#define SYSCTL_XTAL_16MHZ  0x00000540  // External crystal is 16 MHz
#define SYSCTL_OSC_MAIN    0x00000000  // Osc source is main osc
#define SYSCTL_USE_PLL     0x00000000  // System clock is the PLL clock
#define SYSCTL_SYSDIV_10   0x04C00000  // Processor clock is osc/pll /10 - 20 MHz
#define SYSCTL_SYSDIV_5    0x02400000  // Processor clock is osc/pll /5  - 40 MHz
#define SYSCTL_SYSDIV_4    0x01C00000  // Processor clock is osc/pll /4  - 50 MHz
#define SYSCTL_SYSDIV_2_5  0xC1000000  // Processor clock is pll / 2.5   - 80 MHz

SYSCTL_SYSDIV_4 equ SYSCTL_SYSDIV_XX

SYSCTL_XTAL_16MHZ SYSCTL_OSC_MAIN or SYSCTL_USE_PLL or SYSCTL_SYSDIV_XX or
  equ ClockParam

: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
  ClockParam #13 #23 RomCall-1/1 drop
;

((
\ These equates are used to get the clock running using the
\ 16 MHz Main oscillator and the PLL. The SYSDIV value for the
\ PLL is also defined here.

#20000000 equ system-speed	\ -- hz

bit27  $0F #23 lshift or  bit22 or  bit20 or
$1F 6 lshift or  bit5 or bit4 or  bit1 or bit0 or
  invert equ #RCCclr
$09 #23 lshift  bit22 or
bit13 or  bit11 or  $0F 6 lshift or
  equ #RCCset

: waitabit
  $0008:0000 begin 1- dup 0= until drop	\ wait a bit
;

: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
  \ -- start the main oscillator at 16 MHz
  _SysCtl dup sysRCC + @		\ read RCC
  #RCCclr and				\ clear required bits
  #RCCset or				\ set required bits
  swap sysRCC + !			\ write back
  waitabit
  \ -- set up the PLL
  bit22 _SysCtl sysRCC + or!		\ set USESYSDIV bit
  waitabit
  bit13 _SysCtl sysRCC + bic!		\ clear PWRDN bit
  waitabit
  bit11 _SysCtl sysRCC + bic!		\ clear BYPASS bit
;
))

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area

[ _GPIOA _GPIOA_ahb = ] [if]
    $1FF _SysCtl sysGPIOHBCTL + !	\ All GPIOs on AHB
[else]
    0 _SysCtl sysGPIOHBCTL + !		\ All GPIOs on APB
[then]
    setClocks setNVIC
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

