\ startLP1294.fth - Cortex startup code for TI TM4C1294 Launchpad

((
Copyright (c) 2010, 2019
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20190207 SFP002 TM4C1294 conversion.
20100317 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startlm3s9bx
\ *T Cortex start up for LM3S9Bx
\ ===========
\ *P The file *\i{Cortex/Hardware/TM4C/startLP1294.fth} contains
\ ** the start up code for TM4C1294 devices. Start up code for other
\ ** Cortex-M implementations, e.g. STM32, can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M
\ ** is in *\i{Cortex/StartCortex.fth}.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. This
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

internal

((
code ROMcall-1/1	\ param1 priind secind -- res
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r0, [ psp ], # 4		\ r0 = param
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code

code ROMcall-2/1	\ param1 param2 priind secind -- res
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r1, [ psp ], # 4		\ r1 = param2
  ldr     r0, [ psp ], # 4		\ r0 = param1
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code

code ROMcall-3/1	\ param1 param2 param3 priind secind -- res
  mov     r0, # $010			\ primary table base
  movt    r0, # $0100
  ldr     r1, [ psp ], # 4		\ r1 = PriInd
  lsl .s  r1, r1, # 2
  ldr     r3, [ r0 ++ r1 ]		\ table pointer
  lsl .s  tos, tos, # 2			\ tos = SecInd
  ldr     r4, [ r3 ++ tos ]		\ function pointer
  ldr     r2, [ psp ], # 4		\ r2 = param3
  ldr     r1, [ psp ], # 4		\ r1 = param2
  ldr     r0, [ psp ], # 4		\ r0 = param1
  push    { r4-r12 lr }
  blx     r4
  pop     { r4-r12 lr }
  mov     tos, r0
  bx      lr
end-code
))

interpreter
: setmemtim	\ system-speed -- memtim0val
  dup #16000000 <=
  if  drop  $0020:0020 exit  endif
  dup #40000000 <=
  if  drop  $0081:0081 exit  endif
  dup #60000000 <=
  if  drop  $00C2:00C2 exit  endif
  dup #80000000 <=
  if  drop  $0103:0103 exit  endif
  dup #100000000 <=
  if  drop  $0144:0144 exit  endif
  dup #120000000 <=
  if  drop  $0185:0185 exit  endif
  1 abort" Can't figure out memory timing"
;
target
0 equ OSYSDIV 	\ -- 0
\ *G Divide by 1 when the PLL is bypassed
vco-out-speed system-speed / 1- equ PSYSDIV \ -- x
\ *G Divide by n when PLL in use

$0300:0000 OSYSDIV 9 lshift or  PSYSDIV or  equ CLKCFGval1  \ -- x
\ *G Value required first for RSCLKCFG register. PLL clock source
\ ** is the main oscillator MOSC, 25 MHz for the Launchpad. The
\ ** bypass clock source is the PIOSC, 16 MHz.

$8000:0000 ( en) $0000:0000 or ( sysclk)  #100 1- or equ DIVSCLKval  \ -- x
\ *G Value required to coutput sysclk/100 on PQ4.

system-speed setmemtim $0010:0010 or equ MEMTIM0val	\ -- x
\ *G Value required for MEMTIM0 register

: udelay	\ --
  #10 0 do loop
;

: PN0>1		\ --
  _GPION				\ set PQ4
  $FF over gpio.DATA + 4 + !		\ 1 -> bit0
  drop  udelay
;

: PN0>0		\ --
  _GPION				\ clr PQ4
  $00 over gpio.DATA + 4 + !		\ 0 -> bit4
  drop  udelay
;

((
: waggle	\ --
  begin  PN0>1 PN0>0  again
;
))

: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required. The PIOSC is the current clock.

\ Put SysClk/100 on a GPIO pin, PQ4
  _SYSCTL
  $7FFF over sc.RCGCGPIO + or!		\ clock and enable all GPIO ports
  0 over sc.ALTCLKCFG + !		\ PIOSC = Alt clock
  drop  udelay
  _GPION				\ set PN0
  bit0 over gpio.DIR + or!		\ set as output, 2 ma drive by default
  bit0 over gpio.DEN + or!		\ digital enable
  bit0 over gpio.ODR + bic!		\ no open drain
  bit0 over gpio.PUR + bic!		\ no pull ups
  bit0 over gpio.PDR + bic!		\ no pull downs
  drop
\ Initialise parameters
  _SYSCTL
  %11010 over sc.MOSCCTL + !		\ HF + powerdown
  CLKCFGval1 over sc.RSCLKCFG + !	\ divisors
  PLLMINT over sc.PLLFREQ0 + !		\ PLL multiplier
  PLLQN over sc.PLLFREQ1 + !		\ PLL input divider
  MEMTIM0val over sc.MEMTIM0 + !	\ memory wait states
  $C000:0000 over sc.RSCLKCFG + or!	\ use new values for PLL and memory WS

  bit3 over sc.MOSCCTL + bic!		\ Power up main oscillator
  begin  dup sc.RIS + @ bit8 and  until	\ wait until ready. Clk still PIOSC.
  bit23 over sc.PLLFREQ0 + or!		\ enable PLL
  begin  dup sc.PLLSTAT + @ 1 and  until  \ wait for LOCK
  bit28 over sc.RSCLKCFG + or!		\ switch SYSCLK to PLL output
  drop
;

: setNVIC	\ --
\ *G Initialise the NVIC. By default it points into Flash.
  ExcVecs _SCS scsVTOR + !		\ Vector Table Offset Reg.
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area

    setClocks setNVIC
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
\  true
\ [else]
\   umbilical? 0=
\ [then]
\ [if]
\ initial values of user variables
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

