\ FlashXMC11xx.fth - Program XMC11xx Flash in 16 byte units.

((
Copyright (c) 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20151218 SFP001 First release.
))

\ ***************************
\ *! flashxmc11xx
\ *T XMC11xx Flash operations
\ ***************************
\ *P Although some of these routines are not the most efficient
\ ** for these CPUs, they are designed to be easily expanded
\ ** for future parts with as yet unknown sector sizes. This
\ ** code is for Flash with a fixed sector/page size. The page size
\ ** is defined by *\fo{/FlashPage} before the file
\ ** *\i{FlashXMC11xx.fth} is compiled.

4 kb equ /FlashSector	\ -- u
\ *G Size of a Flash sector.
#256 equ /FlashPage	\ -- u
\ *G Size of a Flash page.
#16 equ /FlashBlock	\ -- u
\ *G Size of a Flash block.


\ *************
\ *S Primitives
\ *************

internal

: FlAction	\ action --
\ *G Start the given Flash action. INTERNAL.
  _NVM nvm.PROG + !
;

: FlFinish	\ --
\ *G Wait until the current action finishes and clear it. INTERNAL.
  _NVM begin 				\ -- _nvm
    dup nvm.STATUS + @ bit0 and 0=
  until
  0 swap nvm.PROG + !
;

: FlPageErase	\ addr --
\ *G Erase a 256 byte page. INTERNAL.
  $092 FlAction  0 swap !  FlFinish
;

: FlWriteBlock	\ src dest --
\ *G Write a 16 byte block to a 16 byte aligned address. INTERNAL.
  $091 FlAction
  /FlashBlock bounds do  		\ write 16 bytes
    dup @ i !  cell +
  cell +loop
  drop  FlFinish
;

external


\ ***********
\ *S User API
\ ***********

: FlErase	\ addr len --
\ *G Erase the Flash sectors in the given range.
  bounds ?do  i flPageErase  /FlashPage +loop
;

: FlProg	\ src dest len --
\ *G Program the Flash at *\i{dest} from memory at *\i{src}.
\ ** *\i{Src} must be 4-byte aligned.
\ ** *\i{Dest} is forced to a 16 byte boundary.
  swap #15 + #-16 and swap
  bounds ?do			\ -- src
    dup i FlWriteBlock  /FlashBlock +
  /FlashBlock +loop
  drop
;


\ ======
\ *> ###
\ ======

decimal

