\ Microbit.ctl - Forth for micro:bit with mbed library

((
Copyright (c) 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20161106 MPE001 First release.
))


\ ===========
\ *! microbit
\ ===========

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

\ MPE macros
c" ../../../Common"
		setmacro CommonDir	\ where the common code lives
c" ../../../Cortex"
		setmacro CpuDir		\ where the CPU specific code lives
c" ."		setmacro HWDir		\ where board specific code lives
c" ../../../Examples"
		setmacro ExampleDir	\ Examples
c" ."    	setmacro AppDir		\ where application code lives


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb-2 processor type and register usage
  Hex-I32			\ produce Intel Hex-32 obj format
  +LongCalls			\ permit standalone Forth to handle far calls
  +SaveCdataOnly		\ only save CDATA sections

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

1 equ microbit?		\ -- flag
\ *G Choose hardware to be the BBC micro:bit.


\ =====================
\ *N nRF51 definitions
\ =====================

#48 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

#16 Mhz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
#16 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.


\ =============
\ *N Memory map
\ =============
\ *P The Forth code is placed above the C wrapper, which ends at
\ ** about $0002:C000 in Flash and goes up to $0003:FFFF. RAM is
\ ** available in the top 4 kb of RAM.

\ *P The Flash memory starts at $0000:0000.

  $0000:0000 $FFFF:FFFF udata section catchall
  $0002:C000 $0003:FFFF cdata section Microbit	\ code section above mbed code
  $2000:3000 $2000:31FF idata section PROGd	\ 0.5 kb IDATA
  $2000:3200 $2000:37FF udata section PROGu	\ 1.5 kb UDATA

interpreter
: prog Microbit  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================

$0F0 equ UP-SIZE		\ size of each task's user area
$0F0 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Port 0 is the on-chip UART. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\   true to include semaphores
   0 equ test-multi?		\   true to include test code
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM0Def		\ Cortex-M0 generic equates and SFRs
  include %HwDir%/sfrNRF51svd		\ nRF51 special function registers
  include %HwDir%/MBStackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startMicrobit		\ startup code
  include %CpuDir%/CodeM0M1		\ low level kernel definitions
  include %CommonDir%/kernel62          \ high level kernel definitions
  include %HwDir%/SPcalls		\ Sockpuppet calls
  include %HwDir%/serMicrobit		\ serial driver
  include %CommonDir%/Devtools		\ DUMP .S etc development tools
  include %CommonDir%/Voctools		\ ORDER VOCS etc
  include %CommonDir%/methods		\ target support for methods
\  include %CpuDir%/LocalCM0		\ local variables

tasking? [if]
  include %CpuDir%/MultiCM0		\ multi-tasker, MUST be before TIMEBASE
[else]
  : pause  ;
[then]

\ *** nRF51 DOES NOT HAVE A SYSTICK TIMER
timebase? [if]
  include %CommonDir%/timebase		\ time base common code, MUST be before TICKxxx
  include %CommonDir%/Delays		\ time delays
[else]
  include %CommonDir%/Delays		\ time delays
[then]


environment? [if]
  include %CommonDir%/environ		\ ENVIRONMENT?
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]

softfp? [if]
  include %CpuDir%/Sfp32CM0		\ floating point primitives - coded version
  include %CommonDir%/sfp32com		\ common floating point code
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile microbit.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ********************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE ROM PowerForth for Cortex-M0"
  cr version$ $. space BuildDate$ $.
  .banner
;
external

: ANS-FORTH	\ -- ; marker
;


\ *******************
\ *S Application code
\ *******************

\  include %HwDir%/IAP11xx		\ IAP code
\  include %HwDir%/FlashTables		\ Flash sector tables
\  include %HwDir%/ReProg/ReFlash	\ Reflash system
\  include %HwDir%/FlashProg		\ higher level Flash routines

\  include %CpuDir%/Drivers/gpioNRF51xxx	\ easy GPIO drivers


\ **********************
\ *S Select example code
\ **********************

\ include %ExampleDir%\benchmarks\xbench32	\ performance tests

\ include %CommonDir%\widinfo
\ include %CommonDir%\taskinfo
\ include %ExampleDir%\testtask
\ include %ExampleDir%\testheap


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CommonDir%/LIBRARY
end-libs

decimal

\ XREF DUP                              \ where is DUP used?
\ XREF-ALL                              \ full cross reference
\ XREF-UNUSED                           \ find unused words

update-build

\ ((
\ Define device, speed (0=auto) and project file
\ Download the target image when compilation finishes
jlink nRF51822 0 Project.jlink		\ JLINK and device setup
useSWD					\ use SWD rather than the default JTAG
-download-prompts			\ no prompting for download
afterwards  download-sections		\ download image after FINIS
\ ))

FINIS


\ ======
\ *> ###
\ ======

