\ serNRF51xxpx.fth - polled RX, polled TX, serial communication drivers

((
Copyright (c) 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20161007 MPE001 First release.
))

only forth definitions  decimal

\ ==============
\ *! sernrf51xxxp
\ *T NRF51xxx polled serial driver
\ ==============

\ *P This driver provides polled serial comms for nRF51xxx CPUs
\ ** using the internal UARTs. Because there are alternate pin
\ ** assignments for some UARTs, you *\b{must} check that the
\ ** supplied code pin setup is correct for your board.

\ *P The defaults in this file are for the Nordic NRF51 DK board,
\ ** which uses an nRF51422.


\ ****************
\ *S Configuration
\ ****************
\ *P The following equates control which UARTs are required
\ ** and initialised. If the equates are undefined, they are
\ ** generated and set.

[undefined] useUART0? [if]
1 equ useUART0?	\ -- n
\ *G Compile UART00 code if non-zero.
\ ** Default P0.09 = TxD, P0.11 = RxD, P0.08 = RTS, P0.10 = CTS.
[then]

[undefined] nRF51DK? [if] 0 equ nRF51DK? [then]
[undefined] microbit? [if] 0 equ microbit? [then]


\ ********
\ *S Tools
\ ********

#define Baud1200   0x0004F000
#define Baud2400   0x0009D000
#define Baud4800   0x0013B000
#define Baud9600   0x00275000
#define Baud14400  0x003B0000
#define Baud19200  0x004EA000
#define Baud28800  0x0075F000
#define Baud38400  0x009D5000
#define Baud57600  0x00EBF000
#define Baud76800  0x013A9000
#define Baud115200 0x01D7E000
#define Baud230400 0x03AFB000
#define Baud250000 0x04000000
#define Baud460800 0x075F7000
#define Baud921600 0x0EBED000
#define Baud1M     0x10000000

interpreter
: genBRG	\ baud -- brg
  case
    1200   of Baud1200   endof
    2400   of Baud2400   endof
    4800   of Baud4800   endof
    9600   of Baud9600   endof
    14400  of Baud14400  endof
    19200  of Baud19200  endof
    28800  of Baud28800  endof
    38400  of Baud38400  endof
    57600  of Baud57600  endof
    76800  of Baud76800  endof
    115200 of Baud115200 endof
    230400 of Baud230400 endof
    250000 of Baud250000 endof
    460800 of Baud460800 endof
    921600 of Baud921600 endof
    1 MHz  of Baud1M     endof
  endcase
;
target


\ ********
\ *S UART0
\ ********
\ *P The default pins are P0.09 = TxD (o/p=1), P0.11 = RxD (i/p), P0.08 = RTS (o/p),
\ ** and P0.10 = CTS (i/p).

useUART0? [if]

[undefined] init-ser0 [if]
console0-speed genBRG equ BRG0val

nRF51DK? [if]
: init-ser0	\ --
\ *G Initialise serial port 0 for an nRF51DK. This version is
\ ** used if *\fo{init-ser0} has not already been defined and
\ ** *\fo{nRF51DK?} is true.
  \ Outputs: RTS = P0.8, TXD = P0.9,
  bit8 bit9 or _GPIO gpio.OUTSET + !
  1 _GPIO gpio.PIN_CNF[%s] + 8 cells + ! \ o/p standard drive
  1 _GPIO gpio.PIN_CNF[%s] + 9 cells + ! \ o/p standard drive
  \ Inputs: CTS = P0.10, RXD = P0.11,
  $0C _GPIO gpio.PIN_CNF[%s] + 10 cells + ! \ pull up, connect, i/p
  $0C _GPIO gpio.PIN_CNF[%s] + 11 cells + ! \ pull up, connect, i/p

  0 _UART0 un.ENABLE + !		\ disable UART
  8 _UART0 un.PSELRTS + !		\ select pins
  9 _UART0 un.PSELTXD + !
  10 _UART0 un.PSELCTS + !
  11 _UART0 un.PSELRXD + !
  1 _UART0 un.CONFIG + !		\ no parity, RTS/CTS flow control
  BRG0val _UART0 un.BAUDRATE + !	\ set baudrate
  4 _UART0 un.ENABLE + !		\ enable UART
  1 _UART0 un.TASKS_STARTTX + !		\ start TX
  1 _UART0 un.TASKS_STARTRX + !		\ start RX
  0 _UART0 un.EVENTS_RXDRDY + !		\ RX not ready yet
  bl _UART0 un.TXD + !			\ dummy send
;
[then]
microbit? [if]
: init-ser0	\ --
\ *G Initialise serial port 0 for a micro:bit. This version is
\ ** used if *\fo{init-ser0} has not already been defined and
\ ** *\fo{microbit?} is true.
  \ Outputs: TXD = P0.24,
  bit8 bit9 or _GPIO gpio.OUTSET + !
  1 _GPIO gpio.PIN_CNF[%s] + #24 cells + ! \ o/p standard drive
  \ Inputs: RXD = P0.25,
  $0C _GPIO gpio.PIN_CNF[%s] + 25 cells + ! \ pull up, connect, i/p

  0 _UART0 un.ENABLE + !		\ disable UART
  24 _UART0 un.PSELTXD + !
  25 _UART0 un.PSELRXD + !
  0 _UART0 un.CONFIG + !		\ no parity, no RTS/CTS flow control
  BRG0val _UART0 un.BAUDRATE + !	\ set baudrate
  4 _UART0 un.ENABLE + !		\ enable UART
  1 _UART0 un.TASKS_STARTTX + !		\ start TX
  1 _UART0 un.TASKS_STARTRX + !		\ start RX
  0 _UART0 un.EVENTS_RXDRDY + !		\ RX not ready yet
  bl _UART0 un.TXD + !			\ dummy send
;
[then]

[then]

: seremit0	\ char --
\ *G Transmit a character on UART0.
  _UART0 begin
    dup un.EVENTS_TXDRDY + @ 1 and	\ wait for TX RDY
  until
  0 over un.EVENTS_TXDRDY + !		\ clear TX RDY
  un.TXD + !				\ send char
;
: sertype0	\ c-addr len --
\ *G Transmit a string on UART0.
  bounds ?do  i c@ seremit0  loop
;

: sercr0	\ --
\ *G Issue a CR/LF pair to UART0.
  #13 seremit0  #10 seremit0
;
: serkey?0	\ -- flag
\ *G Return true if UART0 has a character available.
  _UART0 un.EVENTS_RXDRDY + @ 1 and
;
: serkey0	\ -- char
\ *G Wait for a character on UART0.
  _UART0 begin				\ wait for TX RDY
    dup un.EVENTS_RXDRDY + @ 1 and
  until
  0 over un.EVENTS_RXDRDY + !		\ clear TX RDY
  un.RXD + @				\ read

;

create Console0	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for UART0.
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line
console-port 0 = [if]
  console0 constant console
\ *G *\fo{CONSOLE} is the device used by the Forth system for interaction.
\ ** It may be changed by one of the phrases of the form:
\ *C   <device>  dup opvec !  ipvec !
\ *C   <device>  SetConsole
[then]

[then]


\ ************************
\ *S System initialisation
\ ************************

: init-ser	\ --
\ *G Initialise all serial ports
  [ useUART0? ] [if]  init-ser0  [then]
;


\ ======
\ *> ###
\ ======

decimal
