\ startMicrobit.fth - startup code for Micro:bit/Sockpuppet

((
Copyright (c) 2010, 2011, 2012, 2014, 2015, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20161107 SFP006 Micro:bit conversion.
20151209 MPE005 Conversion for STM32F031K6 Nucleo board.
20140524 RDO004 R. Doering did STM32F030R8 Nucleo board adaptation
20140205 MPE003 STM32F072 conversion
20120322 MPE002 STM32F107 conversion
20100112 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startmicrobit
\ *T Cortex start up for Microbit.
\ ===========

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M0+
\ ** Technical Reference Manual (ARM DDI0484). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick

code setForthVM	\ --
\ *G Massage R13/SP to set up UP/SP/RP.
  sub  sp, sp, # UP-SIZE
  mov  up, sp				\ set up UP
  mov  psp, sp				\ set up PSP
  sub .s psp, psp, # 2 cells		\ set up PSP
  sub  sp, sp, # SP-SIZE		\ now the final  set of SP/RSP
  bx   lr
end-code

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation can also be performed here.
\ ** All the GPIO blocks are enabled.
  defSPcalls				\ save passed parameter
  setForthVM				\ set up Forth VM
  begin
    serCr s" foobar" serType
    1000 mbSleep
\    CLD1 @ execute
  again
;
external

\ INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec

$AAAA:5555    7         SetExcVec	\ magic number in unused vector


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-RP  0 ,				\ RP
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

