\ startnRF51.fth - Cortex-M0+ startup code for nRF51 DK.

((
Copyright (c) 2010, 2011, 2012, 2014, 2015, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20161010 SFP006 NRF51 DK conversion.
20151209 MPE005 Conversion for STM32F031K6 Nucleo board.
20140524 RDO004 R. Doering did STM32F030R8 Nucleo board adaptation
20140205 MPE003 STM32F072 conversion
20120322 MPE002 STM32F107 conversion
20100112 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startnrf51
\ *T Cortex start up for NRF51 CPUs.
\ ===========

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M0+
\ ** Technical Reference Manual (ARM DDI0484). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard
[then]

init-s0 tos-cached? sp-guard + cells -
  equ real-init-s0	\ -- addr
\ The data stack pointer set at start up.


: ManualSetup?	\ -- flag
\ See ARMv6M_ARM_DDI0419x
  $F000:0FE0 @ $FF and $01 = if		\ PID0
    $F000:0FE4 @ $0F and 0= if		\ PID1
      $F000:0FEC @ $F0 and 0= if	\ PID3
        case  $F000:0FEC @ $F0 and
          $00 of  true exit  endof
          $10 of  true exit  endof
          $30 of  true exit  endof
        endcase
      then
    then
  then
  0
;

: CheckDevice	\ --
  ManualSetup? if
    $C007:FFDF $4000:0504 !		\ undocumented
    $0000:8000 $4000:6C18 !		\ undocumented
  then
;

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required. the PLL IS used to generate the main 48 MHz
\ ** clock. By default, the AHB and APB busses are run at
\ ** SYSCLK
  $03 _POWER pwr.RAMON + or!		\ Make sure ALL RAM banks are powered on
  $03 _POWER pwr.RAMONB + or!
  _CLOCK
  0 over clk.EVENTS_HFCLKSTARTED + !	\ clear started event
  1 over clk.TASKS_HFCLKSTART + !	\ start HF clock
  begin
    dup clk.EVENTS_HFCLKSTARTED + @ 1 and
  until
  drop
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation can also be performed here.
\ ** All the GPIO blocks are enabled.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    CheckDevice				\ allow for errors in early chips
    setClocks
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-RP  0 ,				\ RP
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

