\ K60intro.man

\ ===========
\ *! k60intro
\ *T Introduction
\ ===========
\ *P This manual documents the MPE PowerForth system for the Kinetis
\ ** K60 hardware.
\ ** The code was built and tested for a Freescale Tower System
\ ** with the TWR-K60N512 and TWR-SER modules. The CPU board
\ ** must be jumpered for the 50MHz module on the the TWR-SER
\ ** board and the Ethernet PHY on the TWR-SER module set up for
\ ** RMII operation.

\ *P When you program the board using the P&E or Freescale tools,
\ ** you may have to power cycle the board.


\ **********************
\ *S Tower System set up
\ **********************
\ *P At MPE, the system is powered by a USB cable plugged into
\ ** the primary elevator card.

\ *P The Forth console runs on RS232 (Tx, Rx only) at 115200
\ ** connected to the RS232 DB9 on the TWR-SER module. This
\ ** is connected to UART3 on the K60.

\ *P If used with Powernet, an Ethernet cable is connected to the
\ ** Ethernet connector on the TWR-SER board.

\ *P The jumpers on the MPE boards are set as shown below.
\ *E TWR-K60N512
\ ** J1    on/closed
\ ** J2    off/open
\ ** J6    Jumper on 2-3, clocked at 50MHz from TWR-SER
\ ** J8    on/closed
\ ** J9    Jumper 2-3 if coin cell connected
\ ** J10   off/open
\ ** J12   off/open

\ *E TWR-SER
\ ** J2    Jumper 3-4 (centre) for 50MHz
\ ** J5    default
\ ** J6    all open, no PHY interrupt
\ ** J10   Jumper 1-2
\ ** J11   all open, no OTG interrupt
\ ** J12   Jumper 9-10, RMII selected
\ ** J13   all open
\ ** J15   Jumper 1-2 for RS232
\ ** J16   Jumper 1-2
\ ** J17   Jumper 1-2 for RS232
\ ** J18   all open, RTS/CTS unused
\ ** J19   Jumper 1-2 for RS232

\ *P To put the initial Forth system into the K60, connect
\ ** your usual debugger or BDM unit to the TWR-K60N512
\ ** module and Flash the board with the file *\i{K60TWR.IMG}.
\ ** MPE .IMG files are simple memory images.
\ ** Power cycle the board, and the Forth should sign on
\ ** at the serial port.

\ =====================
\ *S Terminal emulators
\ =====================
\ *P You can also use almost any terminal emulator,
\ ** including the Windows HyperTerm.

\ *P AIDE is MPE's Integrated Development Environment (IDE)
\ ** that includes a simple editor for your source code
\ ** and a terminal emulator (PowerTerm) tuned for use with
\ ** the PowerForth on the board. AIDE is provided with MPE
\ ** development tools and is what we use with our cross
\ ** compilers.

\ *P Set the terminal emulator to 115200 baud, 8 data bits,
\ ** no parity, 1 or 2 stop bits.


\ ==================
\ *S Using the Forth
\ ==================
\ *P The interactive Forth on the board can use AIDE as a file
\ ** server for source code. Note that code compiled on the
\ ** target Forth:
\ *(
\ *B is always in RAM,
\ *B and the on-board compiler is very simple. The code
\ ** produced by the cross compiler is very much faster and
\ ** shorter.
\ *)

\ *P The real virtue of the on-board compiler is for configuration
\ ** and testing purposes. The on-board interpreter is also
\ ** heavily used as a scripting engine by the web server. We
\ ** also use AIDE to set the clock. In the PowerTerm window,
\ ** press Ctrl-D. This sends Forth words that set the clock to
\ ** your current time and date.

\ *P If you have the PowerNet demo file *\i{K60twrNet.img},
\ ** put it into the Flash. If you are using AIDE with the
\ ** file server enabled, you can do this by typing
\ *C   REFLASH
\ *P and selecting *\i{K60twrNet.img} when the file selecter
\ ** dialog opens. If you are not using AIDE, you will have
\ ** to start the XModem 128/checksum download manually. If
\ ** this fails, you will have to go back to your BDM/JTAG
\ ** tools. After the board signs on and an Ethernet cable
\ ** is connected, type:
\ *C   PowerNet
\ ** to start PowerNet. You will need a DHCP server on your
\ ** network to give PowerNet an IP address and other network
\ ** details. After a few seconds, PowerNet will announce its
\ ** IP address and other details.

\ *P Point a browser at the IP address, and you will see a simple
\ ** web page served.

\ *P If you have a Telnet client, connect it to port 5023 on the
\ ** board. This will log in to another Forth console.


\ ===================
\ *S About the manual
\ ===================
\ *P This manual is derived directly from the Forth source code
\ ** used to generate the on-chip Forth. The full source code
\ ** is supplied with the MPE VFX Forth ARM/Cortex Cross Compiler.
\ ** Consequently the documentation includes some words that
\ ** do not have target entries in the on-chip Forth.

\ *P Some words and code routines are marked in the documentation
\ ** as *\fo{INTERNAL}. These are factors used by other words and do
\ ** not have dictionary entries in the standalone Forth. They
\ ** are only accessible to users of the VFX Forth ARM Cross
\ ** Compiler. This also applies to definitions of the form:
\ *C n EQU <name>
\ *C PROC <name>
\ *C L: <name>


\ ====================
\ *S Technical support
\ ====================
\ *P Technical support is available from your supplier in
\ ** first instance, or from MicroProcessor Engineering.
\ *E tel: +44 (0)23 8063 1441
\ ** fax: +44 (0)23 8033 9691
\ ** net: mpe@mpeforth.com
\ **      tech-support@mpeforth.com
\ ** web: www.mpeforth.com
\ **
\ ** From North America, our telephone and fax numbers are:
\ **   011 44 23 8063 1441
\ **   011 44 23 8033 9691
\ **   901-313-4312 (rings in UK)


\ ============
\ *S Licensing
\ ============
\ *P If you want to distribute applications built with the code you
\ ** have two options.
\ *(
\ *B Purchase an MPE VFX Forth ARM/Cortex Cross Compiler. This includes
\ ** the full source code for ARM and Cortex CPUs and unlimited
\ ** application distribution rights, provided that you do not
\ ** ship an open Forth interpreter. If you need to ship an
\ ** open Forth interpreter for engineering and maintenance
\ ** access, you must get permission from MPE in writing (a
\ ** fax will do), and this is normally provided free of charge.
\ *B Purchase an OEM PowerForth license, either on a quantity
\ ** basis or an unlimited basis. This license includes rights
\ ** to redistribute the software manuals in PDF and HTML form.
\ *)


\ ======
\ *> ###
\ ======

