\ DemoColor429I-Disco.fth - GUI demonstration code for STM32F429I-Discovery board
\ STM32F4 CPU and 240x320 pixel colour LCD

((
Copyright (c) 1999, 2000, 2005, 2012, 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050319 MPE001 Overhauled and factored out
))

decimal

\ ======
\ *> sockpuppet
\ *S Forth demo code
\ ======
\ *P The example code in *\i{Lib\PowerView\DemoColor429I-Disco.fth}
\ ** illustrates an application of the Sockpuppet system.

\ ** The example defines a menus and its actions.
\ ** The menu position is given in absolute pixel coordinates.
\ ** The positions of the internal elements are relative to
\ ** the top left hand corner of the menu. A title bar is
\ ** generated only if a caption is defined. The menu exits
\ ** when a menu item with a negative ID is executes its action.


\ ================
\ *N Menu examples
\ ================

: TestGeneric \ c-addr len --
\ *G The given string is displayed at the bottom of the screen.
   2>r
   Font24 BSP_LCD_SetFont
   LCD_COLOR_BLUE BSP_LCD_SetTextColor
   LCD_COLOR_WHITE BSP_LCD_SetBackColor
   0 300 2r> sout
;

\ *P The following code is for the actions of the main menu and
\ ** for the menu itself. The menu exit action is performed by
\ ** the button that displays the images.
\ *[
: test1		( -- )  s" One           " TestGeneric ;
: test2		( -- )  s" Two           " TestGeneric ;
: test3		( -- )  s" Three         " TestGeneric ;
: test4		( -- )  s" Four      Exit" TestGeneric  DemoPics  ;

20 20 200 230 MENU MainMenu  Font16x12x1 MENUFONT
  CAPTION" MPE PowerView 2"  Font16x16x1 FONT
  20  36 130 32 BUTTON" Button 1"  RUNS test1
  20  76 130 32 BUTTON" Button 2"  RUNS test2
  20 116 130 32 BUTTON" Button 3"  RUNS test3
  20 156 130 32 BUTTON" Images 4"  RUNS test4  -1 setID
END-MENU
\ *]

: drawMainMenu	\ --
\ *G Draw the menu *\fo{MainMenu}.
  LCD_COLOR_BLUE setBColor
  initlcd clear-lcd
  MainMenu DrawMenu
;

Image: image2 %GuiDir%\Pics\image04.bmp
\ *G Construction.
Image: mpehouse %GuiDir%\Pics\mpehouse.bmp
\ *G The MPE building.
Image: bertie %GuiDir%\Pics\bertie.bmp
\ *G The MPE robot. There's a bug in the C code that draws this.

: DemoMenu    \ --
\ *G The Menu demonstration, which exits after displaying the images.
   tsInit				\ calibrate the touchscreen if not done
   LCD_COLOR_LTGRAY setBColor
   clear-lcd
   LCD_COLOR_BLACK setBColor
   LCD_COLOR_YELLOW setTColor
   2 50 mpehouse load-image
   MainMenu dup DrawMenu
   TouchScreenAction
;

: tapScreen	\ --
\ Wait for the screen to be tapped.
  tsWaitUp tsWaitDown  cr ." Tap detected"
;

: DemoPics	\ --
\ *G The bitmap images demonstration
  tsInit

  LCD_COLOR_BLUE lcdClear
  LCD_COLOR_BLUE BSP_LCD_SetBackColor

  LCD_COLOR_RED BSP_LCD_SetTextColor
  0 133 z" Tap the screen" CENTER_MODE  BSP_LCD_DisplayStringAt
  tapScreen

\  LCD_COLOR_GREEN BSP_LCD_SetTextColor
\  0 163 z" Thank you :-)" CENTER_MODE  BSP_LCD_DisplayStringAt
\  tapScreen

  LCD_COLOR_BLACK lcdClear
  20 0 image2 load-image			\ 320 x 240
  tapScreen
  LCD_COLOR_GREEN lcdClear
  2 50 mpehouse load-image			\ 236 x 189
  tapScreen
  LCD_COLOR_CYAN lcdClear
  32 30 bertie load-image			\ 175 x 264
  tapScreen
\  LCD_COLOR_WHITE lcdClear
;

: AppSignOn	\ --
\ *G Display a signon message on the LCD
  1 SAPI_SetLeds    \ just the green LED

  Font24 BSP_LCD_SetFont
  LCD_COLOR_BLACK BSP_LCD_SetBackColor

\ Work around for bug in C library that fails to set the start position correctly
  LCD_COLOR_BLACK BSP_LCD_SetTextColor
  10 z" VFX Forth" CENTER_MODE BSP_LCD_DisplayStringAtLineMode
  LCD_COLOR_YELLOW BSP_LCD_SetTextColor
  10 z" VFX Forth" CENTER_MODE BSP_LCD_DisplayStringAtLineMode

  Font12 BSP_LCD_SetFont
  LCD_COLOR_BLUE BSP_LCD_SetTextColor
  23 z" Press the Blue button to start..." CENTER_MODE BSP_LCD_DisplayStringAtLineMode
;

: RunDemos	\ --
\ *G Run the LCD and Touchscreen demo until a serial-line key is
\ ** pressed. The check uses *\fo{KEY?}.
  begin
    pause
    SAPI_GetKeyButton if
      cr ." USER button pressed"
      begin  DemoMenu  key? until
    then
  key? until
  cr
;

: doForthApp	\ --
\ *G Run the signon message *\fo{AppSignon} and do *\fo{RunDemos}.
\ ** Performed at cold start. To get out to the Forth interpreter
\ ** press a key in the terminal emulator, and press button 4
\ ** (Images).
  AppSignon RunDemos
; ' doForthApp AtCold


\ ======
\ *> ###
\ ======

decimal
