\ MinM0M1.fth - ARM Cortex-M0/M1 Umbilical Forth minimal code definitions

((
Released for use with the MPE Forth Cross Compiler by:

Copyright (c) 2000, 2008, 2010, 2011
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


Change history
==============
20140107 SFP013 Corrected FM/MOD.
20110628 SFP012 Cortex-M0 conversion.
20100204 SFP011 Cortex-M3 conversion.
20081128 SFP010 Added S=.
20000620 SFP009 Matched USER to updated compiler [build 561]
20000608 SFP008 Converted to v6.1 generic I/O model
))

only forth definitions  decimal


\ =========
\ *! minm0m1
\ *T Minimal Umbilical code definitions
\ =========
\ *P The file *\i{Cortex/MinM0M1.fth} contains the minimum code
\ ** definitions required to support Umbilical Forth for Cortex-M0/M1
\ ** variants. If additional words are required, they may be copied
\ ** to a new file from *\i{Cortex/CodeM0M1.fth} or from
\ ** *\i{Common/Kernel62.fth}.

\ *****************
\ *S Register usage
\ *****************
\ *P For Cortex-M0/M1 the following register usage is the default:
\ *E   r15         pc      program counter
\ **   r14         link    link register; bit0=1=Thumb, usually set
\ **   r13         rsp     return stack pointer
\ **   r12         --
\ **   r11         up      user area pointer
\ **   r10         --
\ **   r9          lp      locals pointer
\ **   r8          --
\ **   r7          tos     cached top of stack
\ **   r6          psp     data stack pointer
\ **   r0-r5       scratch
\ *P The VFX optimiser reserves R0 and R1 for internal operations.
\ ** *\fo{CODE} definitions must use R7 as TOS with NOS pointed
\ ** to by R6 as a full descending stack in ARM terminology.
\ ** R0..R5, R12 are free for use by *\fo{CODE} definitions and
\ ** need not be preserved or restored. You should assume that
\ ** any register can be affected by other words.


\ ***************
\ *S Flow of control
\ ***************

CODE (DO)	\ limit index --
\ *G The run time action of *\fo{DO} compiled on the target.
\ ** The branch target address is in-line and must have the T bit set.
\ ** INTERNAL.
  ldmia   psp ! { r1 }			\ get limit
L: PDO
  mov .s  r4, # 1
  lsl .s  r4, r4, # #31			\ r4 := $8000:0000
  mov     r3, link
  sub .s  r3, r3, # 1			\ clear T bit
  ldr     r2, [ r3 ]			\ get LEAVE address, compiler sets T bit
  add .s  r1, r1, r4			\ limit+$8000:0000
  sub .s  r0, tos, r1			\ index-limit-$8000:0000

  push    { r0, r1, r2 }		\ push LEAVE then limit, then index on ret. stack
  ldmia   psp ! { tos }			\ update tos
l: skipbranch
  mov     r0, link
  add .s  r0, # 4
  bx      r0
END-CODE

CODE (?DO) 	\ limit index --
\ *G The run time action of *\fo{?DO} compiled on the target.
\ ** The branch target address is in-line and must have the T bit set.
\ ** INTERNAL.
  ldmia   psp ! { r1 }			\ get limit
  cmp     r1, tos			\ check not equal
  b .ne   pdo				\ take DO ?
  ldmia   psp ! { tos }			\ update tos
l: takebranch
  mov .s  r1, # 1
  mov     r0, link
  bic .s  r0, r1
  ldr     r2, [ r0 ]			\ get address and branch
  bx      r2
END-CODE

CODE EXECUTE 	\ xt --
\ *G Execute the code described by the XT. This is a Forth equivalent
\ ** to an assembler JSR/CALL instruction.
  mov .s  r0, # 1
  orr .s  r0, tos		        \ move CFA, setting Thumb bit
  ldmia   psp ! { tos }
  bx      r0				\ execute CFA - link contains ret addr of execute
END-CODE


\ **************************
\ *S Stack operations and maths
\ **************************

: NOOP  ;	\ --
\ *G A NOOP, null instruction.

: DROP 		\ x --
\ *G Lose the top data stack item and promote NOS to TOS.
  drop  ;

CODE WITHIN? 	\ n1 n2 n3 -- flag
\ *G Return TRUE if N1 is within the range N2..N3.
\ ** This word uses signed arithmetic.
  ldmia   psp ! { r0, r1 }
  mov .s  r2, # 0
  mov .s  r3, # 0
  cmp     r1, r0
  ge, if,
    mov .s  r2, # 1
  endif,
  cmp     r1, tos
  le, if,
    mov .s  r3, # 1
  endif,
  mov .s  tos, # 0		\ flag=0
  tst     r2, r3
  ne, if,
    mvn .s  tos, tos		\ flag=-1
  endif,
  next,
END-CODE

CODE WITHIN 	\ n1|u1 n2|u2 n3|u3 -- flag
\ *G The ANS version of WITHIN?.
\ ** This word uses unsigned arithmetic, so that signed compares are
\ ** treated as existing on a number circle.
  mov     r2, tos			\ save tos
  ldmia   psp ! { r0, r1 }              \ r2 = n3, r0 = n2, r1 = n1
  mov .s  tos, # 0			\ assume false
  sub .s  r2, r2, r0
  sub .s  r1, r1, r0
  cmp     r1, r2
  lo, if,
    mvn .s  tos, tos
  endif,
  next,
END-CODE


\ *****************
\ *S Multiplication
\ *****************

get-tos 7 =  get-psp 6 =  and [if]
code UM*	\ u1 u2 -- ud
\ *G Perform unsigned-multiply between two numbers and return double
\ ** result.
  mov     r0, tos			\ r0=u2
  ldr     r1, [ psp, # 0 ]		\ r1=u1
  push    { psp }
\ build result in r7:r6
\ build temps in r5:r4:r3:r2
  lsl .s  r2, r1 # #16
  lsr .s  r2, r2 # #16			\ r2=u1l
  lsr .s  r3, r1 # #16			\ r3=u1h
  lsl .s  r4, r0 # 16
  lsr .s  r4, r4 # 16			\ r4=u2l
  lsr .s  r5, r0 # 16			\ r5=u2h
\ scratch = r0, r1
\ multiply low portions
  mov     r6, r2
  mul .s  r6, r4			\ r6=u1l*u2l
\ multiply high portions
  mov     r7, r3
  mul .s  r7, r5			\ r7=u1h*u2h
\ multiply and accumulate u1h * u2l
  mov     r0, r3
  mul .s  r0, r4			\ r0=u1h*u2l=xxxx.yyyy
  lsr .s  r1, r0 # #16			\ r1=r0h
  lsl .s  r0, r0 # #16			\ r1:r0=0000.xxxx:yyyy.0000
  add .s  r6, r6, r0			\ add into result
  adc .s  r7, r7, r1
\ multiply and accumulate u2h * u1l
  mov     r0, r5
  mul .s  r0, r2			\ r0=u2h*u1l=xxxx.yyyy
  lsr .s  r1, r0 # #16			\ r1=r0h
  lsl .s  r0, r0 # #16			\ r1:r0=0000.xxxx:yyyy.0000
  add .s  r6, r6, r0			\ add into result
  adc .s  r7, r7, r1

  mov    r5, r6
  pop    { psp }
  str    r5, [ psp, # 0 ]

  next,
end-code
[then]

: *		\ n1 n2 -- n3
\ *G Standard signed multiply. N3 = n1 * n2.
  *  ;

: m*		\ n1 n2 -- d
\ *G Signed multiply yielding double result.
  2dup xor >r				\ sign of result
  abs swap abs swap um*			\ process unsigned
  r> 0<					\ apply sign of result
  if  dnegate  endif
;


\ ***********
\ *S Division
\ ***********
\ *P ARM Cortex-M0 provides no division instructions.

((
macro: udiv64_step	\ --
\ *G Cross compiler macro to perform one step of the unsigned
\ ** 64 bit by 32 bit division
  add .s  r1, r1, r1			\ 64 bit shift of dividend
  adc .s  r0, r0, r0
  mov     r3, r2			\ must not affect flags
  adc .s  r3, r3, r2			\ preserve dividend bit 63, R2=0
  sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
  adc .s  r3, r3, r3			\ success if bit 0 or 1 set
  ne, if,
    add .s  r1, r1, # 1			\ succeeded, update quotient
  else,
    add .s  r0, r0, tos			\ failed, undo subtraction
  endif,
;m
))
code um/mod	\ ud1 u2 -- urem uquot
\ *G Full 64 by 32 unsigned division subroutine.
\ ** This routine uses a loop for code size.
  ldmia   psp ! { r0 }			\ dividend high
  ldr     r1, [ psp ]			\ dividend low
  mov .s  r4, # #32			\ loop counter
  mov .s  r2, # 0			\ always 0
  begin,
\    udiv64_step
    add .s  r1, r1, r1			\ 64 bit shift of dividend
    adc .s  r0, r0, r0
    mov     r3, r2			\ must not affect flags
    adc .s  r3, r3, r2			\ preserve dividend bit 63, R2=0
    sub .s  r0, r0, tos			\ trial subtraction, carry set (no borrow) if ok
    adc .s  r3, r3, r3			\ success if bit 0 or 1 set
    ne, if,
      add .s  r1, r1, # 1		\ succeeded, update quotient
    else,
      add .s  r0, r0, tos		\ failed, undo subtraction
    endif,
    sub .s  r4, r4, # 1
  eq, until,
  mov     tos, r1			\ move quotient
  str     r0, [ psp ]			\ move remainder
  next,
end-code

: fm/mod	\ d n -- rem quot ; floored division
\ *G Perform a signed division of double number *\i{d} by single
\ ** number *\i{n} and return remainder and quotient using floored
\ ** division. See the ANS Forth specification for more details
\ ** of floored division.
  dup >r				\ sign of divisor
  2dup xor >r				\ sign of quotient
  >r					\ divisor
  dabs r@ abs um/mod $7FFF:FFFF and	\ unsigned divide, truncate overflow ; SFP013
  swap r> 0< ?negate swap		\ remainder takes sign of divisor
  r> 0< if				\ if quotient negative
    negate				\ apply sign of quotient
    over if				\ if remainder non-zero
      1-				\ decrement quotient
      r@ rot - swap			\ rem := divisor - rem
    endif
  endif
  r> drop
;

: sm/rem	\ d n -- rem quot ; symmetric division
\ *G Perform a signed division of double number *\i{d} by single
\ ** number *\i{n} and return remainder and quotient using
\ ** symmetric (normal) division.
  over >r                               \ save sign of dividend
  >r  dabs  r@ abs  um/mod              \ unsigned division
  r> r@ xor ?negate                     \ correct sign of quot.
  swap r> ?negate swap                  \ correct sign of rem.
;

: /mod          \ n1 n2 -- rem quot
\ *G Signed symmetric division of N1 by N2 single-precision
\ ** returning remainder and quotient. Symmetric.
  >r s>d r> sm/rem
;

: / 		\ n1 n2  -- n3
\ *G Standard signed division operator. n3 = n1/n2. Symmetric.
  >r s>d r> sm/rem nip
;

: u/ 		\ u1 u2  -- u3
\ *G Unsigned division operator. u3 = u1/u2.
  0 swap um/mod nip
;

: MOD 		\ n1 n2 -- n3
\ *G Return remainder of division of N1 by N2. n3 = n1 mod n2.
  /mod drop  ;

: */MOD 	\ n1 n2 n3 -- n4 n4
\ *G Multiply n1 by n2 to give a double precision result, and then
\ ** divide it by n3 returning the remainder and quotient. The point
\ ** of this operation is to avoid loss of precision.
  >r m* r> sm/rem
;

: */ 		\ n1 n2 n3 -- n4
\ *G Multiply n1 by n2 to give a double precision result, and then
\ ** divide it by n3 returning the quotient. The point
\ ** of this operation is to avoid loss of precision.
  */mod nip
;

: M/ 		\ d n1 -- n2
\ *G Signed divide of a double by a single integer.
  sm/rem nip
;


\ *********************
\ *S Miscellaneous math
\ *********************

CODE D+ 	\ d1 d2 -- d3
\ *G Add two double precision integers.
  ldmia   psp ! { r0-r2 }		\ r0 = d2 low, r1 = d1 high, r2 = d1 low
  add .s  r0, r0, r2			\ d3l = d2l + d1l
  adc .s  tos, tos, r1			\ d3h = d2h + d1h
  sub .s  psp, psp, # 4
  str     r0, [ psp ]			\ push d3l
  next,
END-CODE

CODE D- 	\ d1 d2 -- d3
\ *G Subtract two double precision integers. D3=D1-D2.
  ldmia   psp ! { r0-r2 }		\ r0 = d2 low, r1 = d1 high, r2 = d1 low
  sub .s  r0, r2, r0			\ d3l = d2l - d1l
  sbc .s  r1, r1, tos			\ d3h = d2h - d1h
  mov     tos, r1
  sub .s  psp, psp, # 4
  str     r0, [ psp ]			\ push d3l
  next,
END-CODE

CODE DNEGATE 	\ d1 -- -d1
\ *G Negate a double number.
L: DNEG1
  ldr     r0, [ psp ]			\ r0 = d1l, tos = d1h
  mov .s  r1, # 0
  rsb .s  r0, r0, # 0			\ negate low
  sbc .s  r1, r1, tos			\ negate high carrying through
  mov     tos, r1
  str     r0, [ psp ]
  next,
END-CODE

CODE ?NEGATE 	\ n1 flag -- n1|n2
\ *G If flag is negative, then negate n1.
  mov .s  tos, tos			\ set processor flags
  ldmia   psp ! { tos }
  mi, if,
    rsb .s  tos, tos, # 0		\ negate n1 if flag=true
  endif,
  next,
END-CODE

CODE ?DNEGATE 	\ d1 flag -- d1|d2
\ *G If flag is negative, then negate d1.
  mov .s  tos, tos			\ set processor flags
  ldmia   psp ! { tos }			\ discard flag
  b .mi   DNEG1
  next,
END-CODE

CODE ABS	\ n -- u
\ *G If n is negative, return its positive equivalent (absolute value).
  cmp     tos, # 0
  mi, if,
    rsb .s  tos, tos, # 0
  endif,
  next,
END-CODE

CODE DABS 	\ d -- ud
\ *G If d is negative, return its positive equivalent (absolute value).
  mov .s  tos, tos			\ set processor flags
  b .mi   DNEG1				\ negate d if flag=true
  bx      r14				\ restore link and exit
END-CODE

CODE ROLL 	\ xu xu-1 .. x0 u -- xu-1 .. x0 xu
\ *G Rotate the order of the top N stack items by one place such that
\ ** the current top of stack becomes the second item and the Nth item
\ ** becomes TOS. See also *\fo{ROT}.
  lsl .s  r0, tos # 2			\ r0=position of xu in bytes
  cmp     r0, # 0			\ u is valid?
  le, if,
    ldmia   psp ! { tos }		\   no - get new tos
    bx      link			\   no - exit
  endif,
  ldr     tos, [ psp ++ r0 ]		\ put xu in tos
L: ROLL1
  sub .s  r1, r0, # 4
  ldr     r2, [ psp ++ r1 ]
  str     r2, [ psp ++ r0 ]
  mov     .s r0, r1
  b .ne   ROLL1
  add .s  psp, psp, # 4
  next,
END-CODE


\ **********
\ *S Strings
\ **********

code cmove	\ asrc adest len --
\ *G Copy *\i{len} bytes of memory forwards from *\i{asrc} to *\i{adest}.
  mov .s  r3, tos                	\ r3=len
  ldmia   psp ! { r0, r1, tos }		\ r0=addr2, r1=addr1, r3=len, tos restored
  eq, if,
    bx      link			\ return if len=0, nothing to do
  endif,
  mov .s  r2, r0
  orr .s  r2, r2, r1			\ check alignment
  orr .s  r2, r2, r3			\ R2 := adr1|addr2|len
  mov .s  r4, # 3
  and .s  r4, r4, r2,			\ will be zero if aligned
  b .ne   cmovx3			\ if not aligned
L: CMOVX1
  ldr     r2, [ r1 ]
  str     r2, [ r0 ]
  add .s  r1, r1, # 4
  add .s  r0, r0, # 4
  sub .s  r3, r3, # 4
  b .ne   CMOVX1
  next,
L: CMOVX3
  ldrb    r2, [ r1 ]
  strb    r2, [ r0 ]
  add .s  r1, r1, # 1
  add .s  r0, r0, # 1
  sub .s  r3, r3, # 1
  b .ne   CMOVX3
  next,
END-CODE

CODE CMOVE>	\ c-addr1 c-addr2 u --
\ *G As *\fo{CMOVE} but working in the opposite direction,
\ ** copying the last character in the string first.
  mov .s  r3, tos                	\ r3=len
  ldmia   psp ! { r0, r1, tos }		\ r0=addr2, r1=addr1, r3=len, tos restored
  eq, if,
    bx      link			\ return if len=0, nothing to do
  endif,
  add .s  r0, r0, r3			\ addr2+len
  add .s  r1, r1, r3			\ addr1+len
  mov .s  r2, r0
  orr .s  r2, r2, r1			\ check alignment
  orr .s  r2, r2, r3			\ R2 := adr1|addr2|len
  mov .s  r4, # 3
  and .s  r4, r4, r2,			\ will be zero if aligned
  b .ne   CMV>3				\ byte by byte if not aligned
L: CMV>1	\ copy by 4 byte units
  sub .s  r1, r1, # 4
  sub .s  r0, r0, # 4
  ldr     r2, [ r1 ]
  str     r2, [ r0 ]
  sub .s  r3, r3, # 4
  b .ne   CMV>1
  next,
L: CMV>3	\ copy byte by byte
  sub .s  r1, r1, # 1
  sub .s  r0, r0, # 1
  ldrb    r2, [ r1 ]
  strb    r2, [ r0 ]
  sub .s  r3, r3, # 1
  b .ne   CMV>3
  next,
END-CODE

CODE FILL	\ c-addr u char --
\ *G Fill LEN bytes of memory starting at ADDR with the byte information
\ ** specified as CHAR.
  mov .s  r2, tos
  ldmia   psp ! { r0, r1, tos }		\ r2=char, r0=count, r1=addr
  cmp     r0, # 0
  b .eq   FILX
L: FIL1
  strb    r2, [ r1 ]
  add .s  r1, r1, # 1
  sub .s  r0, r0, # 1
  b .ne FIL1
L: FILX
  next,
END-CODE

: erase		\ c-addr u -- ; wipe memory
\ *G Set U bytes of memory starting at C-ADDR with zeros.
  0 fill
;

CODE S=		\ c-addr1 c-addr2 u -- flag
\ *G Compare two same-length strings/memory blocks, returning TRUE if
\ ** they are identical.
  ldmia   psp ! { r0, r1 }		\ r0=adr2, r1=adr1
  mov .s  tos, tos
  b .eq   S=X                           \ TRUE as len=0
L: S=1
  ldrb    r2, [ r0 ]
  ldrb    r3, [ r1 ]
  add .s  r0, r0, # 1
  add .s  r1, r1, # 1
  cmp     r2, r3
  b .ne   S=X                           \ FALSE as mismatched chars
  sub .s  tos, tos, # 1
  b .ne   S=1                           \ if count=0 then TRUE
L: S=X
  eq, if,
    mov .s  tos, # 1
    neg .s  tos, tos			\ TRUE
  else,
    mov .s  tos, # 0			\ FALSE
  then,
  next,
END-CODE

CODE (") 	\ -- a-addr ; return address of string, skip over it
\ *G Return the address of a counted string that is inline after the
\ ** CALLING word, and adjust the CALLING word's return address to
\ ** step over the inline string. The adjusted return address will
\ ** be at a four byte boundary.
\ ** See the definition of *\fo{(.")} for an example.
  sub .s  psp, psp, # 4			\ save TOS
  str     tos, [ psp, # 0 ]
  ldr     tos, [ rsp, # 0 ]		\ get return address of CALLER
  sub .s  tos, tos, # 1			\ remove Thumb bit
  ldrb    r1, [ tos, # 0 ]		\ length byte
  add .s  r1, r1, tos			\ r1=address+length
  add .s  r1, r1, # 4			\ +1 for count byte, +3 to align
  mov .s  r2, # 3			\ -4 AND -> 3 BIC
  bic .s  r1, r1, r2
  add .s  r1, r1, # 1			\ restore Thumb bit
  str     r1, [ rsp, # 0 ]		\ update return address
  next,
END-CODE

: (C")		\ -- c-addr
\ *G The run time action compiled by C".
  (")
;

: (S")		\ -- c-addr u
\ *G The run time action compiled by S".
  (") count
;


\ *******************************
\ *S Return address manipulations
\ *******************************

CODE >RR 	\ x -- ; R: -- x
\ *G Push the current top item of the data stack onto the top of the
\ ** return stack as a return address
  mov .s  r0, # 1			\ set the T bit
  orr .s  tos, tos, r0
  push    { tos }
  ldmia   psp ! { tos }
  next,
END-CODE

CODE RR> 	\ -- x ; R: x --
\ *G Pop the caller's return address from the return stack.
  sub .s  psp, psp, # 4
  str     tos, [ psp ]
  pop     { tos }
  mov .s  r0, # 1			\ clear the T bit
  bic .s  tos, tos, r0
  next,
END-CODE

CODE RR@ 	\ --  x  ; R:  x -- x
\ *G Copy the top item from the return stack to the data stack.
  sub .s  psp, psp, # 4
  str     tos, [ psp ]
  ldr     tos, [ rsp ]
  mov .s  r0, # 1			\ clear the T bit
  bic .s  tos, tos, r0
  next,
END-CODE


\ ************************************
\ *S Umbilical versions of defining words
\ ************************************

asmcode
here is-action-of constant
\ *G The runtime code for a CONSTANT.
  sub .s  psp, psp, # 4			\ save TOS
  str     tos, [ psp, # 0 ]
  mov     r0, link			\ LINK is high register
  sub .s  r0, r0, # 1			\ remove T bit
  ldr     tos, [ r0, # 0 ]		\ get data address (in-line)
  pop     { pc }
END-CODE

asmcode
here is-action-of variable
\ *G The runtime action for a VARIABLE.
  sub .s  psp, psp, # 4			\ save TOS
  str     tos, [ psp, # 0 ]
  mov     r0, link			\ LINK is high register
  sub .s  r0, r0, # 1			\ remove T bit
  ldr     tos, [ r0, # 0 ]		\ get address
  pop     { pc }
END-CODE

asmcode
here is-action-of value
\ *G The runtime action of a VALUE.
  sub .s  psp, psp, # 4			\ save TOS
  str     tos, [ psp, # 0 ]
  mov     r0, link			\ LINK is high register
  sub .s  r0, r0, # 1			\ remove T bit
  ldr     tos, [ r0, # 0 ]		\ get address
  ldr     tos, [ tos ]			\ get data
  pop     { pc }
END-CODE

asmcode
here is-action-of user
\ *G The runtime action of a USER variable.
  sub .s  psp, psp, # 4			\ save TOS
  str     tos, [ psp, # 0 ]
  mov     r0, link			\ LINK is high register
  sub .s  r0, r0, # 1			\ remove T bit
  ldr     tos, [ r0, # 0 ]		\ get offset
  add     tos, tos, up			\ add user pointer to user offset
  pop     { pc }
end-code

interpreter
: u#		\ "<name>"-- u
\ *G An *\fo{INTERPRETER} word that returns the index of the
\ ** *\fo{USER} variable whose name follows, e.g.
\ *C   u# S0
  ' >body
;
target

: CRASH		\ --  ; used as action of DEFER
\ *G The default action of a DEFERed word. A NOOP.
;

asmcode
here is-action-of DEFER	\ Comp: "<spaces>name" -- ; Run: i*x -- j*x
\ *G The runtime action of a DEFERred word.
  mov     r0, link			\ LINK is high register
  sub .s  r0, r0, # 1			\ remove T bit
  ldr     r0, [ r0, # 0 ]		\ get data address
  ldr     r1, [ r0, # 0 ]		\ get xt
  mov .s  r2, # 1
  orr .s  r1, r1, r2			\ force T bit
  pop     { r3 }			\ get return address
  mov     link, r3
  bx      r1
END-CODE


\ ****************
\ *S Display words
\ ****************

: SPACE         \ --
\ *G Output a blank space (ASCII 32) character.
  bl emit
;

: SPACES        \ n --
\ *G Output 'n' spaces, where 'n' > 0.
\ ** If 'n' < 0, no action is taken.
  0 max 0 ?do  space  loop
;

: .nibble	\ n --
\ *G Convert a nibble to a hex ASCII digit and display it.
  $0F and  dup 9 >
  if  7 +  then
  $30 + emit
;

: .BYTE         \ b --
\ *G Display the byte *\i{b} as a 2 digit hex number.
  dup 4 rshift .nibble  .nibble
;

: .WORD		\ w --
\ *G Display w as a 4 digit unsigned hexadecimal number.
  dup #12 rshift .nibble
  dup 8 rshift .nibble
  dup 4 rshift .nibble
  .nibble
;

: .dword	\ x --
\ *G Display x as an 8 digit unsigned hexadecimal number.
  dup #16 rshift .word .word
;

: .lword	\ x --
\ *G A synonym for *\fo{.DWORD} above.
  .dword
;


\ ****************
\ *S Miscellaneous
\ ****************

: setMask	\ value mask addr -- ; cell operation
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;

: init-io	\ addr --
\ *G Copy the contents of the I/O set up table to an I/O device.
\ ** Each element of the table is of the form addr (cell) followed
\ ** by data (cell). The table is terminated by an address of 0.
  begin
    dup @
   while
    dup 2@ !  2 cells +
  repeat
  drop
;


\ ======
\ *> ###
\ ======

