\ MPE PowerForth Cortex-M3+ 32/32 software floating point package

((
Copyright (c) 2000, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE


********************
Resource definitions
********************

For Cortex-M0/M1 the following register usage is the default:
  r15         pc      program counter
  r14         link    link register; bit0=1=Thumb, usually set
  r13         rsp     return stack pointer
  r12         --
  r11         up      user area pointer
  r10         --
  r9          lp      locals pointer
  r8          --
  r7          tos     cached top of stack
  r6          psp     data stack pointer
  r0-r5       scratch

32-bit mantissa and 32-bit exponent

Change History
==============
20100114 MPE003 Cortex conversion from ARM
20000713 SFP002 Updated for v6.1, corrected D>>N for shift count <= 0
19981029 MSD001 Delete all of the *common* FP stuff as it is now in the
         MSD001 common sub-directory.  (Not flagged.)
))


only forth definitions
decimal

\ =========
\ *> softfp
\ =========
\ *S Cortex coded primitives
\ *P The software floating point pack requires several support
\ ** primitives. High level versions are provided in *\i{SFP16HI.FTH}
\ ** and *\i{SFP32HI.FTH} for 16 and 32 bit targets. Some targets
\ ** have coded versions in the CPU directory and these will provide
\ ** much better performance. The support file should be compiled
\ ** before the common file.


\ ******************
\ Cortex shift routines
\ ******************

code <-S	\ n1 carry-in-flag -- n2 carry-out-flag
\ *G Perform a left shift, applying the carry in to the l.s. bit and
\ ** returning the carry out as 1 or 0.
  ldr     r0, [ psp ]			\ get n1
  lsr .s  tos, tos # 1			\ put input flag in X bit
  adc .s  r0, r0, r0			\ shift n1 left with X, carry to X
  cs, if,
    mov .s  tos, # 1			\ move carry flag into tos
  else,
    mov .s  tos, # 0
  endif,
  str     r0, [ psp ]			\ stash n2
  next,
end-code

code S->	\ n1 carry-in-flag -- n2 carry-out-flag
\ *G Perform a right shift, applying the carry in to the m.s. bit and
\ ** returning the carry out as 1 or 0.
  ldr     r0, [ psp ]			\ get n1
  mov .s  r1, # 1
  and .s  tos, tos, r1			\ make flag 0/1
  ror .s  tos, tos, r1			\ make flag 00000000 or 80000000
  lsr .s  r0, r0 # 1			\ shift n1 right, carry to X
  cs, if,
    orr .s  r0, r0, tos			\ add carry in to n1>>1
    mov .s  tos, # 1			\ move carry flag into tos
  else,
    orr .s  r0, r0, tos			\ add carry in to n1>>1
    mov .s  tos, # 0
  endif,
  str     r0, [ psp ]			\ stash n2
  next,
end-code

code d<<1	\ xd -- xd<<1
\ *G One bit double left shift.
  ldr     r0, [ psp ]			\ fetch low word
  add .s  r0, r0, r0			\ shift low word left
  adc .s  tos, tos, tos			\ shift high word left
  str     r0, [ psp ]			\ store low word
  next,
end-code

code d>>1	\ xd -- xd>>1
\ *G One bit double right shift.
  ldr     r0, [ psp ]			\ fetch low word, d=tos:r0
  lsr .s  tos, tos # 1			\ shift high word right
  adc .s  r1, r1, r1			\ R1 = xxxx1 according to old bit 0
  lsl .s  r1, r1 # #31			\ R1 -> 0/$8000:0000
  lsr .s  r0, r0, # 1			\ shift low word right
  orr .s  r0, r0, r1			\ or in the carry bit
  str     r0, [ psp ]			\ store low word
  next,
end-code

code D>>N	\ d m -- d>>m
\ *G N bit double right shift.
  mov .s  r2, tos			\ r2 = m
  ldmia   psp ! { tos }			\ tos = dh
  ldr     r0, [ psp ]			\ fetch low word, d=tos:r0
  cmp     r2, # 0
  ne, if,
    begin,
      lsr .s  tos, tos # 1		\ shift high word right
      adc .s  r1, r1, r1		\ R1 -> xxxx1 according to old bit 0
      lsl .s  r1, r1 # #31		\ R1 -> 0/$8000:0000
      lsr .s  r0, r0, # 1		\ shift low word right
      orr .s  r0, r0, r1		\ or in the carry bit
      sub .s  r2, r2, # 1
    eq, until,
  endif,
  str     r0, [ psp ]                   \ store low word
  next,
end-code


\ ======
\ *> ###
\ ======

