\ Serial drivers for XTL (Xcompiler Transfer Link)

((
Copyright (c) 2003, 2005, 2009, 2012

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============

))

\ ==========
\ *> jtag430
\ ==========
\ *************************
\ *S Debug control switches
\ *************************
\ *P These switches can be useful when you are having trouble
\ ** with a new FET and Umbilical Forth.

variable <debug-FET>	\ -- addr ; true to generate FET debug info
  <debug-FET> off

also cc/i
: +FET-debug	cc/i	( -- )  <debug-FET> on  ;
\ *G Turn on debugging of the FET link.
: -FET-debug	cc/i	( -- )  <debug-FET> off  ;
\ *G Turn off debugging of the FET link.
: FET-debug?	cc/i	( -- flag )  <debug-FET> @  ;
\ *G Return true if FET debugging is enabled.
previous definitions


\ *************************
\ *S Umbilical Forth set up
\ *************************

only forth also c-c definitions
variable FET-base-addr	\ base address of target virtual UART
  $2F8 FET-base-addr !

also cc/i
: VUART		cc/i	\ -- addr(t)
\ *G Return the target base address of the four-byte region of
\ ** RAM used as a virtual UART.
  FET-base-addr @
;

: is-VUART	cc/i	\ addr(t) --
\ *G Set the target base address of the four-byte region of
\ ** RAM used as a virtual UART.
  FET-base-addr !
;
c-c previous definitions

also cc/c
required: VUART
: VUART		cc/c	\ --
\ *G Once the VUART base address has been set, it can be compiled
\ ** into target code as a literal.
  FET-base-addr @
  general-literal  [last-req]
;
c-c previous definitions

also sections.voc also c-c definitions

: vuart		\ -- vuart 
  fet-base-addr @
;

\ ************************
\ Character read and write
\ ************************

\ =====
\ Tools
\ =====

: c@(fet)	\ addr(t) -- b
\ Read byte. CPU must be stopped.
  { | temp[ cell ] -- }
  5 0 do
    dup temp[ 1 FET_READ MSP430_Memory 0= if
      drop temp[ c@  unloop exit
    endif
  loop
cr ." Reinitialising FET after mem read errors"
  Close-FET 100 ms Open-FET  Run-FET-target
  5 0 do
    dup temp[ 1 FET_READ MSP430_Memory 0= if
      drop temp[ c@  unloop exit
    endif
  loop
  drop  -1 ?FET-Error
;

: c!(fet)	\ b addr(t) --
\ Write byte. CPU must be stopped.
  { | temp[ cell ] -- }
  swap temp[ c!
  5 0 do
    dup temp[ 1 FET_WRITE MSP430_Memory 0= if
      drop  unloop exit
    endif
  loop
cr ." Reinitialising FET after mem write errors"
  Close-FET 100 ms Open-FET  Run-FET-target
  5 0 do
    dup temp[ 1 FET_WRITE MSP430_Memory 0= if
      drop  unloop exit
    endif
  loop
  drop  -1 ?FET-Error
;

: vc@(fet)	\ offset -- b ; read byte from offset in VUART
\ Read byte at offset in block. CPU must be stopped.
  vuart + c@(fet)
;

: vc!(fet)	\ b offset -- ; write byte at offset in VUART
\ Write byte at offset in block. CPU must be stopped.
  vuart + c!(fet)
;

: Query-FET-Escape	\ --
\ Abort out if the ^B or ^C keys have been pressed.
  key? if
    case  key
      2 of  1 abort"  ^B - FET user break"  endof
      3 of  1 abort"  ^C - FET user break"  endof
    endcase
  then
;


\ ======
\ PSUART - V1
\ ======

\ Offsets to registers used by FET comms link.
\ RO = host read-only
\ WO = host write-only
\ RW = host read/write
0 constant data(h>t)		\ WO, host to target data
1 constant status(h>t)		\ RW, host to target status
2 constant data(t>h)		\ RO, target to host data
3 constant status(t>h)		\ RW, target to host status

: (FET-Send-byte)	\ b --
  begin
    Suspend-FET
    status(h>t) vc@(fet)
   while
    Restart-FET
    Query-FET-Escape  2 ms
  repeat
  data(h>t) vc!(fet)
  $FF status(h>t) vc!(fet)
  Restart-FET
;

: ?cr		\ --
\ Do a CR if near end of line.
  out @ #72 u>
  IF  pcr  ENDIF
;

: FET-Send-byte		\ b --
\ Send byte from host down to target, with debug option.
  <debug-FET> @ if
    dup (FET-Send-byte)
    ?cr  ." <" .byte ." >"		\ do AFTER transmission
  else
    (FET-Send-byte)
  endif
;


variable vchar-in	\ -- addr ; last character read
variable vchar-avail	\ -- addr ; true if character available

: (FET-query-byte)	\ -- f
  status(t>h) vc@(fet) $FF =
  dup vchar-avail !  dup if
    data(t>h) vc@(fet) vchar-in !
    0 status(t>h) vc!(fet)
  endif
;

: FET-query-byte	\ -- f
\ Return non-zero if a character is available, and check for a key press.
  Query-FET-Escape
  vchar-avail @ if
    true
  else
    Suspend-FET (FET-query-byte) Restart-FET
  endif
;

: FET-get-byte	\ -- b
\ Return the latest character immediately.  
  vchar-in @  vchar-avail off
  <debug-FET> @
  if  ?cr  dup ." [" .byte ." ]"  endif
;

: FET-wait-byte	\ -- b
\ Wait for a character up from the target and return it.
  begin  FET-query-byte 0= while  100 microsleep  repeat
  FET-get-byte
;


\ **********************
\ Other link definitions
\ **********************

: FET-link-flush	\ -- ; flush any pending input characters
  begin
    FET-query-byte
  while
    FET-get-byte drop
  repeat
;

: .FET-link-state       \ -- ; show serial link state
  FET-id-data .FET-string .z$
  ."  target link on "  fet-port c@ [char] U =
  if  ." USB"  else ." LPT" fet-port c@ emit  endif
  ."  and transfer address $" fet-base-addr @ .word
  [char] . emit cr
  ." FET DLL version = $" FET-dll-version @ .dword
  [char] . emit cr
\  ." Type 430RUN to free-run the CPU."
;

: FET-link-open         \ -- ; start comms as default
  Open-FET  Run-FET-Target
  FET-base-addr @
  FET-id-data .FET-RAM-end @ >= if
    cr ." FET link I/O address ($"
    FET-base-addr @ .word
    ." ) out of range. " cr
    abort
  then
;

: FET-link-close	\ -- ; close FET link
  Close-FET
;

: FET-link-reset	\ -- ; reset XTL link
;

: set-FET	\ -- ; set FLASH Emulation Tool parallel drivers
  assign FET-link-open to-do link-open
  assign FET-link-close to-do link-close
  assign FET-query-byte to-do query-byte
  assign FET-wait-byte to-do wait-byte
  assign FET-send-byte to-do send-byte
  assign FET-link-reset to-do link-reset
  assign FET-link-flush to-do link-flush
  assign .FET-link-state to-do .link-state
;

: FET-hook	\ -- ; sets up cord-hook
  Via-FET
  Open-FET
  vchar-avail off
  Run-FET-Target
\  reset430
;

((
#256 buffer: FETdef$	\ -- caddr
\ Buffer holding port definition string.
))

: setFETport	\ port#|c$ --
\ Set up the FET port buffer for the required port#. Small positive
\ integers are LPT port numbers, large integers are device string
\ addresses, and negative numbers are USB ports.
  dup 0< if
    drop  s" USB" FET-port zplace
  else
    dup hApp u> if			\ it's a string
      count FET-port zplace
    else
      [char] 0 + FET-port !		\ which parallel port
    endif
  endif
;

: FETdefine	\ port# -- ; e.g. 1 FETLINK
  set-FET				\ link via FET driver
\  assign FET-umbilical  to-do emulator-umbilical
\  assign -FET-umbilical to-do -emulator-umbilical
  [ also umbilical ]
  assign (download-FET-code) to-do download-section
  assign FET-hook to-do cord-hook
  [ previous ]
  setFETport				\ which parallel or USB port
  FET-defined on			\ definition done
;

: FETlink	\ port# -- ; e.g. 1 FETLINK
  FETdefine  open-FET
;



\ ****************
\ *S FET selection
\ ****************
\ *P These are used within the compiler to select how the FET is
\ ** connected.

also cc/i	\ this code added to the cross interpreter
: FETdefine	cc/i	\ port# -- ; e.g. USB FETLINK
\ *G Defines which FET is to be used and where it is connected, e.g
\ ** for a USB FET, use:
\ *C   USB FETlink
\ *P whereas for a parallel port FET, use:
\ *C   LPT1 FETlink
\ *P For other ports, e.g. virtual COM ports, use:
\ *C   FET" COM10" FETlink
\ *P The FET is not opened.
  FETdefine
;

: FETlink	cc/i	\ port# -- ; e.g. USB FETLINK
\ *G Defines which FET is to be used and where it is connected, e.g
\ ** for a USB FET, use:
\ *C   USB FETlink
\ *P whereas for a parallel port FET, use:
\ *C   LPT1 FETlink
\ *P For other ports, e.g. virtual COM ports, use:
\ *C   FET" COM10" FETlink
\ *P The FET is opened.
  fetlink
;

: LPT1	cc/i ( -- 1 ) 1  ;
\ *G Select an FET connected on LPT1.
: LPT2	cc/i ( -- 2 ) 2  ;
\ *G Select an FET connected on LPT2.
: LPT3	cc/i ( -- 3 ) 3  ;
\ *G Select an FET connected on LPT3.
: USB	cc/i ( -- -1 ) -1  ;
\ *G Select an FET connected on USB.

((
: FET"	cc/i	\ -- c$
 \ *G Select a FET port (usually virtual). Use in the form:
 \ *C  FET" link"
 \ *P e.g.
 \ *C  FET" COM10" FETlink
  [char] " word count -white
  FETdef$ place  FETdef$
;
))

c-c only forth also c-c definitions


\ ======
\ *> ###
\ ======

decimal
