\ FETmem.fth - Texas Instruments Flash Emulator Tool (FET) drivers

((
Copyright (c) 2003, 2005, 2009, 2012, 2015

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20150722 SFP004 Updated CONFIG-FET.
20120824 SFP003 Added FET debug switch.
20090511 SFP002 Vfx/XC7 and USB FET conversion.
20051027 SFP001 Reduced number of state polls.
))

\ ==========
\ *! jtag430
\ *T JTAG and the MSP430
\ ==========

\ ============================
\ *S JTAG library installation
\ ============================
\ *P TI provides a standard shared library interface to their FET
\ ** (Flash Emulation Tool) products. This interface is used by
\ ** other manufacturers of such tools. These tools use a standard
\ ** 14 pin IDC cable interface to the target hardware. Most use
\ ** the JTAG interface and more recently a Spy-bi-Wire interface
\ ** has been introduced using the same cable.

\ *P The compiler has been tested with the TI MSP-FETU430IF USB
\ ** Debug Interface and the Launchpad with an MSP430G2553 CPU.
\ ** We have not knowingly tested Spy-bi-Wire.

\ ==========
\ *N Windows
\ ==========
\ *P Compatible programmers provide a DLL called *\i{MSP430.dll},
\ ** and may require more. These DLLs must be installed where the
\ ** compiler can reach them, either in the *\i{Compiler} folder
\ ** (recommended) containing *\i{xMSP430xx.exe} or in one of the
\ ** Windows system folders. You *\b{must} use a set of DLLs that
\ ** matches your hardware!

\ *P Although we can provide some drivers, it is always preferable to
\ ** download the latest drivers and DLLs yourself. The latest
\ ** versions of the debug tool and USB drivers can be found at
\ *C   http://www.ti.com/tool/MSPDS
\ *P Download the Developers Package and the Stand-alone Driver
\ ** Installer and install the software.


\ ---------------------------
\ *H TI USB FET MSP-FET430UIF
\ ---------------------------
\ *P You can download current drivers from:
\ *C   http://processors.wiki.ti.com/index.php/MSP430_JTAG_Interface_USB_Driver

\ ** Install according to the instructions. Plug in the adapter,
\ ** install the drivers, and then use Control Panel -> Device Manager
\ ** to check that a new COM port has appeared. If the COM port
\ ** shows an error, reinstall.

\ *P After driver installation, copy the DLLs to the *\i{Compiler}
\ ** folder from which the compiler is run.

\ ------------
\ *H Launchpad
\ ------------
\ *P If you are not worried about having up to date drivers, a
\ ** reasonble set is provided in the *\i{Compiler\LaunchpadG2}
\ ** folder.

\ *P Download the Launchpad code and drivers from:
\ *C   http://www.ti.com/tool/msp-exp430g2et
\ *P or
\ *C   http://www.ti.com/tool/msp-exp430g2
\ *P after clicking the "Get Software" button.

\ *P The Launchpad's integrated
\ ** FET is similar to the MSP-FET430UIF and includes a virtual
\ ** serial port. Remove the Launchpad if connected, navigate to
\ ** the folder containing *\i{Launchpad_Driver.exe} and run that
\ ** file. Not much will appear to happen. Connect the Launchpad,
\ ** let Windows rattle around a bit and Windows should tell you
\ ** that the driver is installed. Then use Control Panel -> Device
\ ** Manager to check that a new COM port has appeared. If the COM
\ ** port shows an error, uninstall and reinstall the driver. Some
\ ** users recommend rebooting the PC after installation.

\ *P After driver installation, copy the DLLs *\i{MSP430.dll} and
\ ** *\i{HIL.dll} to the *\i{Compiler} folder from which the
\ ** compiler is run.

\ -----------------------
\ *H Olimex MSP-JTAG-TINY
\ -----------------------
\ *P Download the drivers from
\ *C   http://www.olimex.com
\ *P Plug in the adapter and install the drivers from the folder.
\ ** Note that this requires the v2 drivers and the current package
\ ** is v3.13 in 2018. Not recommended.

\ *P After driver installation, copy *\b{all} the DLLs to the *\i{Compiler}
\ ** folder from which the compiler is run.


\ ========
\ *N Linux
\ ========
\ *P At the time of writing we do not have Linux drivers for the
\ ** interfaces we use. TI provide some drivers at:
\ *C   http://processors.wiki.ti.com/index.php/MSP430_JTAG_Interface_USB_Driver


\ ===========
\ *N Mac OS X
\ ===========
\ *P At the time of writing we do not have Mac OS X drivers for the
\ ** interfaces we use. TI provide some drivers at:
\ *C   http://processors.wiki.ti.com/index.php/MSP430_JTAG_Interface_USB_Driver


\ **************************
\ *S Defining a virtual UART
\ **************************
\ *P In Umbilical Forth the serial link driver used by a
\ ** standalone Forth is usually replaced by a virtual UART which
\ ** provides a pseudo-serial comms link for Umbilical Forth
\ ** without using any MSP430 peripherals or additional pins. All
\ ** it needs is four bytes of RAM and the JTAG pins. The driver
\ ** for the virtual UART is in *\i{MSP430\Drivers\VUART.FTH}. The
\ ** Umbilical Forth host is told where the VUART RAM is by the
\ ** directive *\fo{IS-VUART}. The control file will/should
\ ** contain code of the form below:
\ *E \ Offsets to registers used by FET comms link.
\ ** \ RO = host read-only
\ ** \ WO = host write-only
\ ** \ RW = host read/write
\ ** 0 equ data(h>t)         \ WO, host to target data
\ ** 1 equ status(h>t)       \ RW, host to target status
\ ** 2 equ data(t>h)         \ RO, target to host data
\ ** 3 equ status(t>h)       \ RW, target to host status
\ ** 4 equ /VUART            \ size of VUART data block
\ **
\ ** idata
\ ** here is-VUART  /VUART allot    \ reserve space
\ ** cdata
\ *P Once this definition has been made, the base address of the
\ ** virtual UART is available as *\fo{VUART}.

decimal

[forth]  also c-c  also sections.voc  also c-c definitions
decimal

[-sin

\ variable <running>	\ -- addr ; Holds nz if CPU running
\  -1 <running> !
variable <DEFAULT-FET-VCC>
  #3000 <DEFAULT-FET-VCC> !
variable VccOffAtClose	\ -- addr ; holds nz to turn off VCC at close
  1 VccOffAtClose !
variable FET-DLL-version
variable FET-defined	\ -- addr ; true when connection specified
variable FET-opened	\ -- addr ; true after FET DLL is initialised correctly
variable FET-run-state
variable FET-run-cycles

0 value DebugFET?	\ -- flag
also cc/i
: +DebugFET	( -- )  1 to DebugFET?  ;
: -DebugFET	( -- )  0 to DebugFET?  ;
previous definitions

: initFETvars	\ -- ; SFP002
\ initialise FET variables at cold start.
\  -1 <running> !
  #3000 <DEFAULT-FET-VCC> !
  FET-DLL-version off
  FET-defined off
  FET-opened off
  FET-run-state off
  FET-run-cycles off
  1 VccOffAtClose !
;
' initFETvars AtCold

FET-id-size buffer: FET-id-data
#256 buffer: FET-Port	\ -- addr
\ Holds string identifying device ; parallel port number of FET
  s" USB" FET-port zplace
#512 buffer: FET-data-buffer	\ -- addr ; max Flash block size

$10000 constant MAX-MAIN-FET-ADDR

0 value LastFETerror
\ Used by FETLINK.FTH for host/target comms
\ variable FET-host-tx-count	\ -- addr ; chars received
\ variable FET-host-rx-count	\ -- addr ; chars sent

: crx		\ -- ; debug version of cr
  cr  key? drop  #100 ms
;

: Display-FET-Error	\ --
  MSP430_Error_Number dup to LastFETerror
  dup 0= if  drop exit  endif
  DebugFET? if  .rs  then
  cr ." Flash Emulation Tool Error: " cr   \ Issue error message
  MSP430_Error_String .z$ cr
  key? drop  #100 ms
;

: ?FET-Error	\ flag --
  if  Display-FET-Error  .rs  abort  endif
;

4 0 Callback: UFWcallback	\ msg wparam lparam handle --
	
: (UFWcallback)	{: msg wparam lparam handle -- :}
  [io  ConsoleDev setIo
  decimal  cr
  case  msg
    0 of  ." Initializing Update Bootloader"  endof
    1 of  ." Erasing mapped interrupt vectors"  endof
    2 of  ." Erasing firmware memory section"  endof
    3 of  ." Program new firmware"  endof
    4 of  ." One data block of the new firmware was successfully programmed" endof
    5 of  ." Exit Update Bootlader and reboot firmware"  endof
    6 of  ." Update was successfully finished"  endof
    7 of  ." An error occured during firmware update"  endof
    8 of  ." An error occured during firmware update"  endof
          ." Unknown response"
  endcase
  io]
;
assign (UFWcallback) to-callback UFWcallback

: UpdateFETfirmware	\ --
  NULL NULL 0 GetModulehandle MSP430_FET_FwUpdate if
    cr ." Update failed"
  else
    cr ." Update succeeded"
  then
  cr ." Please re-run compiler" key? drop #1000 ms bye
;

: Init-FET	\  --
\ ." Initialize" crx
  MSP430.dll 0= abort" Cannot access FET interface MSP430 shared library"
  FET-port FET-DLL-version MSP430_Initialize ?FET-Error
  FET-DLL-version @ -1 -3 either= if
    cr ." FET firmware must be updated, Proceed? (y/n) " y/n? if
      UpdateFETfirmware
    then
  then
;

: Init-FET-Vcc	\ --
\ <DEFAULT-FET-VCC> @ . ." Vcc" crx
  <DEFAULT-FET-VCC> @ MSP430_VCC ?FET-Error
;

: Identify-FET	\ --
\ ." Identify" crx
  Z" DEVICE_UNKNOWN" Z" " 0 0 0 MSP430_OpenDevice ?FET-Error
  FET-id-data FET-id-size MSP430_GetFoundDevice ?FET-Error
;

: .ConnectedTo	\ -- ; display MSP430 type
  cr ." Connected to " FET-id-data .FET-string .z$
;

: config-FET-protocol	\ -- ; automatic
  INTERFACE_MODE 3 MSP430_Configure ( ?FET-Error ) drop
;

: config-FET-verification	\ --
  VERIFICATION_MODE 1 MSP430_Configure ( ?FET-Error ) drop
;

: Config-FET	\ --
\ ." Config" crx
  config-FET-protocol config-FET-verification
;

: eraseMain	\ caddr len --
  10 0 do
    2dup  ERASE_MAIN -rot MSP430_Erase 0=
    if  2drop unloop exit  endif
    50 ms
  loop
  2drop  1 ?FET-Error
;

: eraseCheck	\ caddr len --
  10 0 do
    2dup MSP430_EraseCheck 0=
    if  2drop unloop exit  endif
    50 ms
  loop
  2drop  1 ?FET-Error
;

: Erase-FET	\ --
\ ." Erase" crx
  FET-id-data .FET-main-start w@
  MAX-MAIN-FET-ADDR over -		\ -- caddr len

\ ." Erase" crx
  2dup eraseMain
\ ." EraseCheck" crx
  eraseCheck
;

: (Open-FET)	\  --
\ ." Opening FET" cr
  Init-FET
\  Init-FET-VCC
  Identify-FET
  Config-FET
\  .ConnectedTo
;

: Open-FET	\  --
  FET-opened @ 0=
  if  (Open-FET) FET-opened on  endif
;

: (Close-FET)	\ --
\ Close FET regardless of the state variables.
\ ." Closing FET" crx
\  -1 <running> !	 \ SFP001
  0 MSP430_Close ?FET-Error
;

: Close-FET	\ --
\ Close the FET if it is already open
  FET-opened @
  if  (Close-FET)  FET-opened off  then
;
' Close-FET AtExit

: Get430State	\ mode --
\ Read MSP430 state. For mode=0 the CPU is not stopped,
\ for mode=1 it is stopped.
\ ." State" crx
  FET-run-state swap FET-run-cycles .s MSP430_State .s ?FET-error
\  FET-run-state @ FET_RUNNING = <running> !
;

: GetCPUState	\ mode -- state
\ Read MSP430 state. For mode=0 the CPU is not stopped,
\ for mode=1 it is stopped.
\ ." State" crx
  FET-run-state swap FET-run-cycles MSP430_State ?FET-error
  FET-run-state @
;

: Run-FET-Target	\ --
\ ." Reset ALL" crx
  ALL_RESETS 1 0 MSP430_Reset ?FET-Error
\  FET-host-tx-count off  FET-host-rx-count off
;

: Suspend-FET	\ --
  begin
    1 GetCPUState FET_RUNNING =
  while
    100 microsleep
  repeat
;

: Restart-FET	\ --
\ ." Restart" crx
  #500 0 do
    FREE_RUN 0 MSP430_Run 0=
    if  unloop exit  endif
    #100 microsleep
  loop
  1 ?FET-Error
\  FREE_RUN 0 MSP430_Run ?FET-Error
;

: [F		\ -- ; R: -- flag ; flag=0 if previously frozen
\ +G The code between *\fo{[F} and *\fo{F]} is executed with the
\ +* target frozen. Because state information is preserved on the
\ +* return stack, these words must be used in pairs inside the
\ +* same definition.
  r>  1 GetCPUstate >r  suspend-FET  >r
;

: F]		\ -- ; R: flag -- ; Unfreeze target if flag is true
\ +G The code between *\fo{[F} and *\fo{F]} is executed with the
\ +* target frozen. Because state information is preserved on the
\ +* return stack, these words must be used in pairs inside the
\ +* same definition.
  r>
  r> FET_RUNNING = 
  if  restart-FET  endif
  >r
;


#16 cells buffer: registers[]	\ -- addr
\ Contains the 430 registers

: reset430	\ -- ; reset target across link
\ ." Reset RST" crx
  RST_RESET 1 0 MSP430_Reset ?FET-error
\  FET-host-tx-count off  FET-host-rx-count off
;

: 430Run	\ --
\ Free run the CPU. The JTAG signals are not released.
  FREE_RUN 0 MSP430_Run ?FET-Error
;

: 430Go		\ --
\ Free run the CPU. The JTAG signals are released.
  FREE_RUN 1 MSP430_Run ?FET-Error
;

: .430Arch	\ b -- ; display MSP architecture/ISA
  case
    0 of  ." Non-X"  endof
    1 of  ." X"  endof
    2 of  ." X v2"  endof
     dup . ." is unknown"
  endcase
;


\ ***********************
\ *S JTAG interface words
\ ***********************
\ *P The JTAG interface provides a simple set of words that
\ ** can be used to aid debugging. The words below are available
\ ** when the cross-compiler is interpreting.

also cc/i
: GET-VCC	cc/i	\ -- mv ; get VCC in millivolts
\ *G Get the current VCC setting in millivolts.
  <DEFAULT-FET-VCC> @
;

: SET-VCC	cc/i	\ mv -- ; define VCC in millivolts
\ *G Define the current VCC setting in millivolts. This only
\ ** changes the setting until the FET is reinitialised.
  <DEFAULT-FET-VCC> !
;

: Apply-Vcc	\ mv --
\ *G Define the current VCC setting in millivolts and apply
\ ** it to the FET.
\ ." Apply VCC" crx
  dup <DEFAULT-FET-VCC> ! MSP430_VCC ?FET-Error
;

: reset430	cc/i	\ --
\ *G Reset target across link. JTAG is not released.
  reset430
;

: 430Restart	cc/i	\ --
\ *G Restart the CPU in free run mode.
  Restart-FET
;

: 430Run	cc/i	\ --
\ *G Free run the CPU. The JTAG signals *\b{are not} released.
  430Run
;

: 430Go		cc/i	\ --
\ *G Free run the CPU. The JTAG signals *\b{are} released.
  430Go
;

: 430open	cc/i	\ --
\ *G Opens FET access if not already opened.
  Open-FET
;

: 430close	cc/i	\ --
\ *G Closes FET access if not already closed.
  Close-FET
;

: 430reopen	cc/i	\ --
\ *G Forces an open of the FET. Occasionally useful when
\ ** debugging FETs.
  Close-FET  100 ms  Open-FET
\  (open-fet) fet-opened on
;

: .430State	cc/i	\ --
\ *G Display the CPU registers.
  cr
." a" key? drop
  0 Get430State  FET-run-state @
." b" key? drop
  1 Get430State
." c" key? drop
  case
    FET_STOPPED of  ." stopped"  endof
    FET_RUNNING of  ." running"  endof
    FET_SINGLE_STEPPED of  ." single-stepped"  endof
    FET_BREAKPOINT of  ." breakpointed"  endof
      ." ???"
  endcase
\ ." Read Regs" crx
  registers[]  $FFFF FET_READ MSP430_Registers ?FET-error
  cr ." Registers"
  registers[]  #16 0 do
    i 7 and 0= if  cr ." R" i .  endif
    dup @ .word space  cell +
    i 3 and 3 = if  space  endif
  loop
  drop
\ ." FreeRun 1" crx
  FREE_RUN 1 MSP430_Run ?FET-Error
  FET_RUNNING FET-run-state !
;

\ See FET-DEVICE in UsbJtagDll.fth for structure details.

: .430cpu	cc/i	\ -- ; display CPU information
\ *G Display CPU info.
  base @  decimal
  Fet-id-data
  dup w@ $AA55 <> abort" Invalid ID block"
  cr ." ID = $" dup 2+ w@ .word 7 spaces dup 4 + .z$
  cr ." MSP430 Arch      " dup .FET-cpuArch c@ .430Arch
  cr ." Main Flash from $" dup #36 + w@ .word
     ."  to $" dup .FET-main-end @ dup if  .dword  else  drop ." FFFF"   endif
\    ."  to $FFFF"
  cr ." Info Flash from $" dup #38 + w@ .word
     ."  to " dup .FET-info-end w@ .word
  cr ." RAM from        $" dup #58 + w@ .word
    ."  to $" dup #40 + w@ .word
  cr ." RAM2 from       $" dup .FET-RAM2-start w@ .word
    ."  to $" dup .FET-RAM2-end w@ .word
  cr ." Has FRAM        $" dup .FET-HasFramMemory w@ .word
  cr ." # Breakpoints    " dup #42 + w@ u.
  cr ." Emulation level  " dup #44 + w@ u.
  cr ." Clock level      " dup #46 + w@ u.
  cr ." LCD starts at   $" dup #48 + w@ .word
  cr ." LCD ends at     $" dup #50 + w@ .word
  cr ." Vcc min (mv)     " dup #52 + w@ u.
  cr ." Vcc max (mv)     " dup #54 + w@ u.
  cr ." Has TEST/VPP     " dup #56 + w@ .
  drop
  base !
;

: UpdateFETfirmware	\ --
  UpdateFETfirmware
;
previous definitions


\ ************************
\ ***  Interface Code  ***
\ ************************

decimal


\ =====================
\ FET access primitives
\ =====================

: FET-read-mem	\ addr(t) len --
\ Read len bytes at target address addr(t) to the
\ data buffer.
\ ." Read mem" crx
  FET-data-buffer swap FET_READ MSP430_Memory ?FET-Error
;

: FET-write-mem \ addr(t) len --
\ Write len bytes from the data buffer to target memory
\ at addr(t).
\ ." write mem" crx
  FET-data-buffer
  swap FET_WRITE MSP430_Memory ?FET-Error
;

: FET-erase	\ addr(t) len --
\ Erase the given segment whose starting address and length
\ are given.
\ ." Erase seg" crx
  ERASE_SEGMENT -rot MSP430_Erase ?FET-Error
;

: (InFlash?)	\ addr(t) -- flag
\ Returns true if addr(t) is in Flash, which is assumed
\ to be the range $1000..$FFFF.
  $01000 $10000 within
;

 defer InFlash?	\ addr(t) -- flag
\ Returns true if addr(t) is in Flash, which is assumed
\ to be the range $1000..$FFFF.
  assign (inFlash?) to-do InFlash?

: (FindSegment)	\ addr(t) -- addr(t)' len
\ Find the start address and length of the Flash segment
\ containing addr(t).
  dup $01200 u< if			\ possible info Flash?
    dup $01100 u< if			\ segA or segB?
      $FF80 and  $080			\ 128 byte SegA/B
    else
      $FF00 and  $100			\ 256 byte SegN
    endif
  else
    $FE00 and  $200			\ 512 byte segment
  endif
;

defer FindSegment	\ addr(t) -- addr(t)' len
\ Find the start address and length of the Flash segment
\ containing addr(t).
  assign (FindSegment) to-do FindSegment


\ ===============
\ Read operations
\ ===============

: FET-c@(ms)	\ addr(t) section -- b
  drop
  1 FET-read-mem
  FET-data-buffer c@
;

: FET-w@(ms)	\ addr(t) section -- w ; read word from target image
  drop
  2 FET-read-mem
  FET-data-buffer >r
  r@ c@ r> 1+ c@
  2bytes>word  			        \ merge
;

: FET-l@(ms)	\ addr(t) section -- l ; read long from target image
  sec_badmem throw
;


\ ================
\ Write operations
\ ================

: FET-c!(ms)	\ b addr(t) section -- ; write byte to target image
  drop
  dup InFlash? if
    dup 0 FET-c@(ms) $FF <> if		\ -- b addr(t)
      dup FindSegment tuck 2dup 2>r	\ -- b addr(t) len seg len ; R: -- seg len
      FET-read-mem			\ read segment ; -- b addr(t) len ; R: -- seg len
      1- and				\ offset of addr(t) in seg/len
      FET-data-buffer + c!		\ overwrite byte ; -- ; R: -- seg len
      2r@ FET-erase			\ Erase segment
      2r> FET-write-mem			\ put block back
      exit
    endif
  endif
  \ -- w addr(t)
  swap FET-data-buffer c!
  1 FET-write-mem
;

: wi!		\ w addr -- ; store word as target image in host
  swap word>2bytes			\ -- addr b1 b0
  rot >r  r@ c!  r> 1+ c!
;

: FET-w!(ms)	\ w addr(t) section -- ; write word to target image
  drop
  dup InFlash? if
    dup 0 FET-w@(ms) $FFFF <> if	\ -- w addr(t)
      dup FindSegment tuck 2dup 2>r	\ -- w addr(t) len seg len ; R: -- seg len
      FET-read-mem			\ read segment ; -- w addr(t) len ; R: -- seg len
      1- and FET-data-buffer + wi!	\ overwrite byte ; -- ; R: -- seg len
      2r@ FET-erase			\ erase segment
      2r> FET-write-mem			\ put block back
      exit
    endif
  endif
  \ -- w addr(t)
  swap FET-data-buffer wi!
  2 FET-write-mem
;

: FET-l!(ms)	\ l addr(t) section -- ; write long to target image
  sec_badmem throw
;


\ ===================================
\ Memory operations that halt the CPU
\ ===================================

: FET-c@(us)	\ addr(t) section -- b
  [F  FET-c@(ms)  F]
;

: FET-w@(us)	\ addr(t) section -- w ; read word from target image
  [F  FET-w@(ms)  F]
;

: FET-l@(us)	\ addr(t) section -- l ; read long from target image
\  suspend-FET  FET-l@(ms)  restart-FET
  sec_badmem throw
;

: FET-c!(us)	\ b addr(t) section -- ; write byte to target image
  [F  FET-c!(ms)  F]
;

: FET-w!(us)	\ w addr(t) section -- ; write word to target image
  [F  FET-w!(ms)  F]
;

: FET-l!(us)	\ l addr(t) section -- ; write long to target image
\  suspend-FET  FET-l!(ms)  restart-FET
  sec_badmem throw
;

: open-FET(ms)	\ -- ; open section
  FET-defined @ 0= abort" OPEN: FET not defined"
  FET-opened @ 0= abort" OPEN: FET not initialised"
  MS-handle off		\ set offset to 0 ; reset by IN-EMULATOR
;

: close-FET(ms)	 \ -- ; close section
  MS-handle on
;

create MSIO-FET	\ -- addr ; vector for I/O to host memory
  ' open-FET(ms) ,
  ' close-FET(ms) ,
  ' save-gen(ms) ,
  ' load-gen(ms) ,
  ' read-gen(ms) ,
  ' write-gen(ms) ,
  ' org-mem(ms) ,
  ' here-mem(ms) ,
  ' allot-mem(ms) ,
  ' FET-c@(ms) ,
  ' FET-w@(ms) ,
  ' FET-l@(ms) ,
  ' no64support ,
  ' FET-c!(ms) ,
  ' FET-w!(ms) ,
  ' FET-l!(ms) ,
  ' no64support ,

: +FET-umbilical	\ -- ; set FET I/O to return to target mode
  ['] FET-c@(us)  MSIO-FET MSIO-c@(t) !
  ['] FET-w@(us)  MSIO-FET MSIO-w@(t) !
  ['] FET-l@(us)  MSIO-FET MSIO-l@(t) !
  ['] no64support MSIO-FET MSIO-x@(t) !
  ['] FET-c!(us)  MSIO-FET MSIO-c!(t) !
  ['] FET-w!(us)  MSIO-FET MSIO-w!(t) !
  ['] FET-l!(us)  MSIO-FET MSIO-l!(t) !
  ['] no64support MSIO-FET MSIO-x!(t) !
;

: -FET-umbilical	\ -- ; set FET I/O to return to host mode
  ['] FET-c@(ms)  MSIO-FET MSIO-c@(t) !
  ['] FET-w@(ms)  MSIO-FET MSIO-w@(t) !
  ['] FET-l@(ms)  MSIO-FET MSIO-l@(t) !
  ['] no64support MSIO-FET MSIO-x@(t) !
  ['] FET-c!(ms)  MSIO-FET MSIO-c!(t) !
  ['] FET-w!(ms)  MSIO-FET MSIO-w!(t) !
  ['] FET-l!(ms)  MSIO-FET MSIO-l!(t) !
  ['] no64support MSIO-FET MSIO-x!(t) !
;

: [FET		\ -- x
\ The CPU is stopped and the previous state returned.
  0 Get430State
  FET-run-state @ FET_RUNNING = dup
  if  1 Get430State  endif		\ force a halt
  -FET-umbilical
;

: FET]		\ x --
\ The CPU is restarted if the previous state demands it.
  if
    FREE_RUN 1 MSP430_Run ?FET-Error	\ JTAG released
    +FET-umbilical
  endif
;

((
create FET-test-data
 $31 c, $40 c, $00 c, $03 c, $B2 c, $40 c, $80 c, $5A c,
 $20 c, $01 c, $D2 c, $D3 c, $22 c, $00 c, $E2 c, $D3 c,
 $32 c, $00 c, $D2 c, $E3 c, $21 c, $00 c, $E2 c, $E3 c,
 $31 c, $00 c, $3F c, $40 c, $E8 c, $FD c, $1F c, $83 c,
 $FE c, $23 c, $F7 c, $3F c,
36 constant FET-test-data-len

create FET-test-vectors
 $00 c, $F0 c,          \ reset vector
2 constant FET-vec-data-len

$F000 constant FET-start-addr
$FFFE constant FET-vec-addr

: test-FET \ --
  49 FET-port !
  open-FET
  erase-FET
  FET-test-data FET-data-buffer
  FET-test-data-len cmove

  FET-start-addr FET-test-data-len FET-write-mem

  FET-start-addr FET-vec-addr 0 fet-w!(ms)

  Run-FET-target
;
))


\ ***********
\ Diagnostics
\ ***********

: .fet		\ --
\ Display FET status information
  cr ." FET-opened " FET-opened @ .
;


\ *********************
\ downloading a section
\ *********************

: FET_download_code	{ buff b-len taddr -- }
\ ." download" crx
  taddr buff  b-len FET_WRITE MSP430_Memory ?FET-Error
;

: FET_upload_code	{ buff b-len taddr -- }
\ ." upload" crx
  taddr buff  b-len FET_READ  MSP430_Memory ?FET-Error
;

: (download-FET-code)	{ sect | buff used vbuff -- }
\ download memory image
  Close-FET  #100 ms  Open-FET
  Suspend-FET
  sect savesize(ms) dup -> used         \ find size of section contents
  get-img-buff -> buff                  \ put contents of section in buffer
  used allocate abort" Can't get verify buffer" -> vbuff
  -download-prompts? if
    erase-FET  #20 ms			\ wipe flash
  else
    cr ." Erase Flash? (y/n): " y/n? if
      erase-FET				\ wipe flash
      cr ."   Flash erased" key? drop #20 ms
    endif
  endif
<<<<<<< HEAD:software/ROM/Cortex/VUART2/FETmem.fth
  cr ." Downloading to FET, addr: " sect MS-low @ .word ." , len: " used .word  key? drop 20 ms
=======
  cr ." Downloading to FET, addr: " sect MS-low @ .word ." , len: " used .word  key? drop #20 ms
>>>>>>> 45fb850f9faeb5b22395423eca5510505a1a6eb7:software/COMPILER/MSP430v2/jtag/FETmem2.fth
  sect MS-low @  buff  used  sect read-gen(ms)
  buff  used  sect MS-low @		\ -- buff len addr(t)
  FET_download_code
  cr ." Verifying"  key? drop #20 ms
  vbuff used sect MS-low @		\ read back
  cr FET_upload_code
  buff used vbuff used compare if	\ compare
    ."   Verify failed"
  else
    ."   Verify good"
  endif
  cr  key? drop #20 ms
  vbuff free drop			\ release buffers
  buff free-img-buff
  Close-FET
;


\ ****************************
\ Words accessible by the user
\ ****************************

: Via-FET	\ -- ; set up base address of section I/O for FET
\ Define actions for memory functions called by the cross-compiler.
  [ also sections.voc ]
  MSIO-FET change-curr-io(ms)		\ switch to FET I/O
  [ previous ]
;

also cc/i
: via-FET	cc/i  via-FET  ;
\ *G Memory accesses during cross compilation or interpretation
\ ** are directed to the FET.

: FETdownload	cc/i	\ --
\ *G Download the cross-compiled memory image(s) to the target
\ ** through the FET. It is normally only necessary to download
\ ** the *\fo{CDATA} section(s). The FET is left open, the CPU is
\ ** reset and running.
  download-sections
  100 ms  Open-FET
  Run-FET-Target
;

: +FET-umbilical  cc/i	\ --
\ *G When the host needs to read or write memory, it will halt
\ ** the CPU, perform the operation and restart the CPU.
  +FET-umbilical
;

: -FET-umbilical  cc/i	\ --
\ *G When the host needs to read memory, the CPU is assumed to be
\ ** halted.
  -FET-umbilical
;
previous definitions

sin]


\ ======
\ *> ###
\ ======

decimal
