\ NXP LPC40xx special function registers

((
Copyright (c) 2010..2013, 2014, 2015, 2016, 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


Absolute addresses start with an '_' underscore character.
Offsets and masks do not.


To do
=====

Change history
==============
20170216 SFP006 Incorporated more changes from BJC.
20160825 BJC005 Corrected base address for UART2
20160302 BJC004 Corrected i2s register names for DAO & DAI
20160111 BJC003 Added bit defs for SSP CR0 register
20151211 BJC002 Added bit definitions for SSP status register
20150619 BJC001 Modified sfrLPC1788.fth for LPC40xx family devices, with new
		IOCON IO pin configuration registers.
))


\ =============
\ *! sfrlpc40xx
\ *T Peripheral Register definition files
\ =============
\ *P Each CPU family we support has a peripheral register
\ ** definition file, called *\i{sfrxxx.fth}. You will find
\ ** them in your *\i{Cortex} or {ARM} directories. For example
\ ** the one for the NXP LPC40xx family is called *\i{sfrLPC40xx.fth}.

\ *P Common Cortex-M4 core definitions are in *\i{CM4def.fth}.

\ *P Peripheral blocks are all defined using a common format.
\ ** The base address is defined in an *\fo{EQU}ate with a
\ ** leading underscore in the name. The registers are defined
\ ** as offsets from the base address. The following example
\ ** is for the LPC17xx watchdog.
\ *E #define _WD  $4000:0000
\ **   $00 equ WDMOD
\ **   $04 equ WDTC
\ **   $08 equ WDFEED
\ **   $0C equ WDTV
\ **   $10 equ WDCLKSEL
\ *P Code supplied by MPE always uses the register names in these
\ ** files.

\ *P Files for particular families may also define *\fo{INTERPRETER}
\ ** extensions that ease peripheral set up.

\ *P PIN FUNCTION MAPPING
\ *P NXP have kept many register addresses and peripherals the same
\ ** as for the LPC24xx family, HOWEVER the pin function mapping
\ ** uses a completely different scheme.  The differences force us
\ ** to use different words and syntax to set up pins.  This affects
\ ** many library driver files that would otherwise be compatible.

\ ==========
\ RAM blocks
\ ==========

#define _SRAM     $1000:0000
#define _AHBSRAM0 $2000:0000
#define _AHBSRAM1 $2000:4000

\ EMC
$8000:0000 equ SMB0base		\ four 64 Mb static memory banks
$9000:0000 equ SMB1base
$9800:0000 equ SMB2base
$9C00:0000 equ SMB3base

$A000:0000 equ DMB0base		\ four 256 Mb dynamic memory banks
$B000:0000 equ DMB1base
$C000:0000 equ DMB2base
$D000:0000 equ DMB3base

\ *****************
\ Interrupt numbers
\ *****************

decimal
#16 equ WDT_Vec#
#17 equ T0_vec#		\ Timerx
#18 equ T1_vec#
#19 equ T2_vec#
#20 equ T3_vec#
#21 equ U0_vec#		\ UARTx
#22 equ U1_vec#
#23 equ U2_vec#
#24 equ U3_vec#
#25 equ PWM1_vec#	\ PWM0
#26 equ I2C0_vec#	\ I2Cx
#27 equ I2C1_vec#
#28 equ I2C2_vec#
	\ 29 is unused
#30 equ SSP0_vec#	\ SSPx
#31 equ SSP1_vec#
#32 equ PLL0_vec#	\ PLL0
#33 equ RTC_vec#	\ RTC
#34 equ E0_vec#		\ EINTx
#35 equ E1_vec#
#36 equ E2_vec#
#37 equ E3_vec#
#38 equ ADC_vec#	\ ADC
#39 equ BOD_vec#	\ Brown Out Detector
#40 equ USB_vec#	\ USB
#41 equ CAN_vec#	\ CAN
#42 equ DMA_vec#	\ DMA
#43 equ I2S_vec#	\ I2S
#44 equ ENET_vec#	\ Ethernet
#45 equ SDC_vec#	\ SD Card
#46 equ MCPWM_vec#	\ Motor Control
#47 equ QEI_vec#	\ Quadrature Encoder Interface
#48 equ PLL1_vec#	\ PLL1
#49 equ USBact_vec#	\ USB activity; USB_NEED_CLK
#50 equ CANact_vec#	\ CAN activity
#51 equ U4_vec#		\ USART 4
#52 equ SSP2_vec#
#53 equ LCDC_vec#	\ LCD controller
#54 equ GPIO_vec#	\ GPIO Interrupts
#55 equ PWM0_vec#
#56 equ EEPROM_vec#	\ EEPROM events


\ **************
\ System Control
\ **************

#define _SYSCON	$400F:C000
  $00 equ FlashCfg	\ $400F:C000

  $40 equ MEMMAP	\ 0x400FC040

\ Main clock PLL
  $80 equ PLL0CON	\ $400F:c080
  $84 equ PLL0CFG	\ $400F:c084
  $88 equ PLL0STAT	\ $400F:c088
  $8C equ PLL0FEED	\ $400F:c08c

\ USB clock PLL
  $A0 equ PLL1CON	\ $400F:c0A0
  $A4 equ PLL1CFG	\ $400F:c0A4
  $A8 equ PLL1STAT	\ $400F:c0A8
  $AC equ PLL1FEED	\ $400F:c0Ac

  $C0 equ PCON		\ $400F:c0c0
  $C4 equ PCONP		\ $400F:c0c4
  $C8 equ PCONP1	\ $400F:c0c8

  $100 equ EMCCLKSEL	\ 0x400FC100

  $104 equ CCLKSEL	\ $400F:C104
  $108 equ USBCLKSEL	\ $400F:C108
  $10C equ CLKSRCSEL	\ $400F:C10C

  $110 equ CANsleepClr
  $114 equ CANwakeFlags

  $140 equ EXTINT	\ $400F:C140
  $148 equ EXTMODE	\ $400F:C148
  $14C equ EXTPOLAR	\ $400F:C14C

  $180 equ RSID		\ $400F:C180

  $188 equ MATRIXARB	\ $400F:C188

  $1A0 equ SCS		\ $400F:C1A0 ; NXP name
  $1A0 equ SCSreg	\ $400F:C1A0 ; renamed to avoid conflict with ARM SCS

  $1A8 equ PCLKSEL	\ $400F:C1A8

  $1B0 equ PBOOST	\ $400F:C1B0

  $1B4 equ SPIFICLKSEL	\ 0x400FC1B4
  $1B8 equ LCDCFG	\ $400F:C1B8

  $1C0 equ USBIntSt	\ $400F:C1C0 USB Interrupt Status
  $1C4 equ DMAReqSel	\ $400F:C1C4 DMA Request Select, UART/Timer
  $1C8 equ CLKOUTCFG	\ $400F:C1C8

  $1CC equ RSTCON0	\ 0x400FC1CC
  $1D0 equ RSTCON1	\ 0x400FC1D0

  $1DC equ EMCDLYCTL	\ 0x400FC1DC
  $1E0 equ EMCCAL	\ 0x400FC1E0

#define _PCON		$400F:C0C0
#define _PCONP		$400F:C0C4
#define _PCONP1		$400F:C0C8
#define _SCSreg		$400F:C1A0 	\ renamed to avoid conflict with ARM SCS
#define _PCLKSEL0	$400F:C1A8
#define _PCLKSEL1	$400F:C1AC
#define _LCDCFG   	$400F:C1B8
#define _UsbIntSt	$400F:C1C0
#define _DmaReqSel	$400F:C1C4

#define _EMCDLYCTL	0x400F:C1DC
#define _EMCCAL		0x400F:C1E0
#define _EMCCLKSEL  	0x400F:C100

\ ---------------
\ PCONP bit masks
\ ---------------

decimal
bit0 equ ~pcLCD
bit1 equ ~pcTim0
bit2 equ ~pcTim1
bit3 equ ~pcUART0
bit4 equ ~pcUART1
bit5 equ ~pcPWM0
bit6 equ ~pcPWM1
bit7 equ ~pcI2C0
bit8 equ ~pcUART4
bit9 equ ~pcRTC
bit10 equ ~pcSSP1
bit11 equ ~pcEMC
bit12 equ ~pcADC
bit13 equ ~pcCAN1
bit14 equ ~pcCAN2
bit15 equ ~pcGPIO
bit16 equ ~pcSPIFI
bit17 equ ~pcMCPWM
bit18 equ ~pcQEI
bit19 equ ~pcI2C1
bit20 equ ~pcSSP2
bit21 equ ~pcSSP0
bit22 equ ~pccTim2
bit23 equ ~pccTim3
bit24 equ ~pcUART2
bit25 equ ~pcUART3
bit26 equ ~pcI2C2
bit27 equ ~pcI2S
bit28 equ ~pcSDC
bit29 equ ~pcGPDMA
bit30 equ ~pcENET
bit31 equ ~pcUSB

\ ---------------
\ PCONP1 bit mask
\ ---------------

bit3 equ PCCMP


\ ===============================
\ Pin configuration and selection
\ ===============================

#define _IOCON	0x4002C000	\ base for the individual IO pin register

  $000 equ IOCON_P0_00
  $004 equ IOCON_P0_01
  $008 equ IOCON_P0_02
  $00C equ IOCON_P0_03
  $010 equ IOCON_P0_04
  $014 equ IOCON_P0_05
  $018 equ IOCON_P0_06
  $01C equ IOCON_P0_07
  $020 equ IOCON_P0_08
  $024 equ IOCON_P0_09
  $028 equ IOCON_P0_10
  $02C equ IOCON_P0_11
  $030 equ IOCON_P0_12
  $034 equ IOCON_P0_13
  $038 equ IOCON_P0_14
  $03C equ IOCON_P0_15
  $040 equ IOCON_P0_16
  $044 equ IOCON_P0_17
  $048 equ IOCON_P0_18
  $04C equ IOCON_P0_19
  $050 equ IOCON_P0_20
  $054 equ IOCON_P0_21
  $058 equ IOCON_P0_22
  $05C equ IOCON_P0_23
  $060 equ IOCON_P0_24
  $064 equ IOCON_P0_25
  $068 equ IOCON_P0_26
  $06C equ IOCON_P0_27
  $070 equ IOCON_P0_28
  $074 equ IOCON_P0_29
  $078 equ IOCON_P0_30
  $07C equ IOCON_P0_31
  $080 equ IOCON_P1_00
  $084 equ IOCON_P1_01
  $088 equ IOCON_P1_02
  $08C equ IOCON_P1_03
  $090 equ IOCON_P1_04
  $094 equ IOCON_P1_05
  $098 equ IOCON_P1_06
  $09C equ IOCON_P1_07
  $0A0 equ IOCON_P1_08
  $0A4 equ IOCON_P1_09
  $0A8 equ IOCON_P1_10
  $0AC equ IOCON_P1_11
  $0B0 equ IOCON_P1_12
  $0B4 equ IOCON_P1_13
  $0B8 equ IOCON_P1_14
  $0BC equ IOCON_P1_15
  $0C0 equ IOCON_P1_16
  $0C4 equ IOCON_P1_17
  $0C8 equ IOCON_P1_18
  $0CC equ IOCON_P1_19
  $0D0 equ IOCON_P1_20
  $0D4 equ IOCON_P1_21
  $0D8 equ IOCON_P1_22
  $0DC equ IOCON_P1_23
  $0E0 equ IOCON_P1_24
  $0E4 equ IOCON_P1_25
  $0E8 equ IOCON_P1_26
  $0EC equ IOCON_P1_27
  $0F0 equ IOCON_P1_28
  $0F4 equ IOCON_P1_29
  $0F8 equ IOCON_P1_30
  $0FC equ IOCON_P1_31
  $100 equ IOCON_P2_00
  $104 equ IOCON_P2_01
  $108 equ IOCON_P2_02
  $10C equ IOCON_P2_03
  $110 equ IOCON_P2_04
  $114 equ IOCON_P2_05
  $118 equ IOCON_P2_06
  $11C equ IOCON_P2_07
  $120 equ IOCON_P2_08
  $124 equ IOCON_P2_09
  $128 equ IOCON_P2_10
  $12C equ IOCON_P2_11
  $130 equ IOCON_P2_12
  $134 equ IOCON_P2_13
  $138 equ IOCON_P2_14
  $13C equ IOCON_P2_15
  $140 equ IOCON_P2_16
  $144 equ IOCON_P2_17
  $148 equ IOCON_P2_18
  $14C equ IOCON_P2_19
  $150 equ IOCON_P2_20
  $154 equ IOCON_P2_21
  $158 equ IOCON_P2_22
  $15C equ IOCON_P2_23
  $160 equ IOCON_P2_24
  $164 equ IOCON_P2_25
  $168 equ IOCON_P2_26
  $16C equ IOCON_P2_27
  $170 equ IOCON_P2_28
  $174 equ IOCON_P2_29
  $178 equ IOCON_P2_30
  $17C equ IOCON_P2_31
  $180 equ IOCON_P3_00
  $184 equ IOCON_P3_01
  $188 equ IOCON_P3_02
  $18C equ IOCON_P3_03
  $190 equ IOCON_P3_04
  $194 equ IOCON_P3_05
  $198 equ IOCON_P3_06
  $19C equ IOCON_P3_07
  $1A0 equ IOCON_P3_08
  $1A4 equ IOCON_P3_09
  $1A8 equ IOCON_P3_10
  $1AC equ IOCON_P3_11
  $1B0 equ IOCON_P3_12
  $1B4 equ IOCON_P3_13
  $1B8 equ IOCON_P3_14
  $1BC equ IOCON_P3_15
  $1C0 equ IOCON_P3_16
  $1C4 equ IOCON_P3_17
  $1C8 equ IOCON_P3_18
  $1CC equ IOCON_P3_19
  $1D0 equ IOCON_P3_20
  $1D4 equ IOCON_P3_21
  $1D8 equ IOCON_P3_22
  $1DC equ IOCON_P3_23
  $1E0 equ IOCON_P3_24
  $1E4 equ IOCON_P3_25
  $1E8 equ IOCON_P3_26
  $1EC equ IOCON_P3_27
  $1F0 equ IOCON_P3_28
  $1F4 equ IOCON_P3_29
  $1F8 equ IOCON_P3_30
  $1FC equ IOCON_P3_31
  $200 equ IOCON_P4_00
  $204 equ IOCON_P4_01
  $208 equ IOCON_P4_02
  $20C equ IOCON_P4_03
  $210 equ IOCON_P4_04
  $214 equ IOCON_P4_05
  $218 equ IOCON_P4_06
  $21C equ IOCON_P4_07
  $220 equ IOCON_P4_08
  $224 equ IOCON_P4_09
  $228 equ IOCON_P4_10
  $22C equ IOCON_P4_11
  $230 equ IOCON_P4_12
  $234 equ IOCON_P4_13
  $238 equ IOCON_P4_14
  $23C equ IOCON_P4_15
  $240 equ IOCON_P4_16
  $244 equ IOCON_P4_17
  $248 equ IOCON_P4_18
  $24C equ IOCON_P4_19
  $250 equ IOCON_P4_20
  $254 equ IOCON_P4_21
  $258 equ IOCON_P4_22
  $25C equ IOCON_P4_23
  $260 equ IOCON_P4_24
  $264 equ IOCON_P4_25
  $268 equ IOCON_P4_26
  $26C equ IOCON_P4_27
  $270 equ IOCON_P4_28
  $274 equ IOCON_P4_29
  $278 equ IOCON_P4_30
  $27C equ IOCON_P4_31
  $280 equ IOCON_P5_00
  $284 equ IOCON_P5_01
  $288 equ IOCON_P5_02
  $28C equ IOCON_P5_03
  $290 equ IOCON_P5_04

\ GPIO Configuration bits
\ There are different options associated with different GPIO pin types,
\ so the IOCON registers vary.  The data sheet lists IOCON types:
\ 	type D - standard digital pins
\ 	type A - pins with analogue functionality
\ 	type I - pins with I2C functionality
\ 	type U - pins with USB functionality (and little else)
\ 	type W - pins as D, but with glitch filter

\ The equates below can be OR'd together to configure an IO pin
\ Check with the data sheet to ensure that you only use options allowed
\ for the pin you are setting!  In some cases options are mandatory
\ (e.g. bit7 in IOCON type W) or mutually exclusive (e.g. HI drive & OD).

\ LPC specific names
\ Pin Function bits (0:2) - applicable to ALL IOCON register types
0 equ ~IO_GPIO		\ input function 0 is always as a GPIO pin
1 equ ~IOPFUNC1		\ peripheral functions supported vary for each pin
2 equ ~IOPFUNC2
3 equ ~IOPFUNC3
4 equ ~IOPFUNC4
5 equ ~IOPFUNC5
6 equ ~IOPFUNC6
7 equ ~IOPFUNC7

\ Generic names - "Alternate Function"
\ Pin Function bits (0:2) - applicable to ALL IOCON register types
0 equ AF0		\ input function 0 is always as a GPIO pin
1 equ AF1		\ peripheral functions supported vary for each pin
2 equ AF2
3 equ AF3
4 equ AF4
5 equ AF5
6 equ AF6
7 equ AF7

\ Pin Pull-up / Pull-down resistor bits (4:3) - applicable to IOCON register types D, A, W
   0 equ ~IO_NOPULL	\ no pull resistor - use when port is an output
BIT3 equ ~IO_PULLDN
BIT4 equ ~IO_PULLUP
$18  equ ~IO_PULLREP	\ repeater mode, port pulled to retain current input state

BIT5 equ ~IO_HYST_EN	\ input hysteresis enable - applicable to IOCON reg. types D, W

BIT6 equ ~IO_INVERT	\ invert port logic - i.e. pin state when read.  IOCON types D, A, I, W

BIT7 equ ~IO_DIGITAL	\ set as an digital input (rather than analogue) - only for IOCON type A
BIT7 equ ~IO_W		\ MUST be set for IOCON type W

BIT8 equ ~IO_NOFILTER	\ disable glitch filter for IOCON types A, W.
BIT8 equ ~IO_HS		\ High Speed I2C mode for IOCON type I

BIT9 equ ~IO_FAST_SLEW	\ DISable output slew rate control - see data.  IOCON types D, W.
BIT9 equ ~IO_HIDRIVE	\ increase output drive - see data.  IOCON type I only.

BIT10 equ ~IO_OD_OUT	\ Sets (simulated) open drain mode for output.  IOCON types D, W

BIT16 equ ~IO_DAC	\ set analogue output mode, IOCON type A only.


\ ==============
\ GPIO registers
\ ==============
\ The original GPIO format is not supported.
\ We only support the new FIO format.

#define _FIO0 0x20098000
#define _FIO1 0x20098020
#define _FIO2 0x20098040
#define _FIO3 0x20098060
#define _FIO4 0x20098080
#define _FIO5 0x200980A0
  $00 equ FIODIR
  $10 equ FIOMASK
  $14 equ FIOPIN
  $18 equ FIOSET
  $1C equ FIOCLR

#define _GPIO0int  $4002:8000
  $080 equ IOIntSt
  $084 equ IO0IntStR
  $088 equ IO0IntStF
  $08C equ IO0IntClr

  $090 equ IO0IntEnR
  $094 equ IO0IntEnF

  $0A4 equ IO2IntStR
  $0A8 equ IO2IntStF
  $0AC equ IO2IntClr

  $0B0 equ IO2IntEnR
  $0B4 equ IO2IntEnF


\ ========
\ Ethernet
\ ========

$2008:4000 equ _EMAC
  $0000 equ emMAC1
  $0004 equ emMAC2
  $0008 equ emIPGT
  $000C equ emIPGR
  $0010 equ emCLRT
  $0014 equ emMAXF
  $0018 equ emSUPP
  $001C equ emTEST
  $0020 equ emMCFG
  $0024 equ emMCMD
  $0028 equ emMADR
  $002C equ emMWTD
  $0030 equ emMRDD
  $0034 equ emMIND

  $0040 equ emSA0
  $0044 equ emSA1
  $0048 equ emSA2

  $0100 equ emCommand
  $0104 equ emStatus
  $0108 equ emRxDesc
  $010C equ emRxSt
  $0110 equ emRxDesc#
  $0114 equ emRxProdInd
  $0118 equ emRxConsInd
  $011C equ emTxDesc
  $0120 equ emTxSt
  $0124 equ emTxDesc#
  $0128 equ emTxProdInd
  $012C equ emTxConsInd

  $0158 equ emTSV0
  $015C equ emTSV1
  $0160 equ emTSV2

  $0170 equ emFlowCtrlCnt
  $0174 equ emFlowCtrlSt

  $0200 equ emRxFiltCtrl
  $0204 equ emRxFiltWolSt
  $0208 equ emRxFiltWolClr

  $0210 equ emHashFiltL
  $0214 equ emHashFiltH

  $0FE0 equ emIntSt
  $0FE4 equ emIntEn
  $0FE8 equ emIntClr
  $0FEC equ emIntSet

  $0FF4 equ emPwrDn
  $0FFC equ emModId


\ =======================
\ USB - Device, Host, OTG
\ =======================

#define _USB	$2008:C000	\ USB control base address
#define _USBdev	$2008:C000	\ USB device control base address

\ Host controller
  $000 equ hcRevision
  $004 equ hcControl
  $008 equ hcCmdSt
  $00C equ hcIntSt
  $010 equ hcIntEn
  $014 equ hcIntDis
  $018 equ hcHCCA
  $01C equ hcPerCurrED
  $020 equ hcCtrlHeadED
  $024 equ hcCtrlCurrED
  $028 equ hcBulkHeadED
  $02C equ hcBulkCurrED
  $030 equ hcDoneHead
  $034 equ hcFmInterval
  $038 equ hcFmRemaining
  $03C equ hcFmNumber
  $040 equ hcPerStart
  $044 equ hcLsThreshold
  $048 equ hcRhDescA
  $04C equ hcRhDescB
  $050 equ hcRhSt
  $054 equ hcRhPortSt1
  $058 equ hcRhPortSt2
  $05C equ hcModID

\ OTG registers
  $100 equ otgIntSt
  $104 equ otgIntEn
  $108 equ otgIntSet
  $10C equ otgIntClr
  $110 equ otgStCtrl
  $110 equ USBPortSel
  $114 equ orgTmr

\ Bits in USBPortSel/otgStCtrl - these are clock enable bits
  bit1	equ ~USBCLK_DEV_CLK
  bit3  equ ~USBCLK_PORTSEL_CLK
  bit4  equ ~USBCLK_AHB_CLK

\ Device Interrupt Registers
  $200 equ USBDevIntSt
  $204 equ USBDevIntEn
  $208 equ USBDevIntClr
  $20C equ USBDevIntSet

\ SIE Command Registers
  $210 equ USBDevCmdCode
  $214 equ USBDevCmdData

\ Data Transfer Registers
  $218 equ USBDevRxData
  $21C equ USBDevTxData
  $220 equ USBDevRxLen
  $224 equ USBDevTxLen
  $228 equ USBDevCtrl

\ Device Interrupt Registers
  $22C equ USBDevIntPri		\ interrupt priority

\ Endpoint Interrupt Registers
  $230 equ USBDevEpIntSt	\ r/o, 16 bit pairs, low=RX, high=TX
  $234 equ USBDevEpIntEn	\ r/w
  $238 equ USBDevEpIntClr	\
  $23C equ USBDevEpIntSet	\ w/o
  $240 equ USBDevEpIntPri	\ w/o
\ Endpoint Realization Registers
  $244 equ USBDevEpRe		\ r/w, 32 single bits
  $248 equ USBDevEpIndx		\ w/o, 5 bit endpoint number, 0..31
  $24C equ USBDevEpSize		\ r/w, 10 bit max packet size

\ DMA Registers
  $250 equ USBDevDmaReqSt	\ r/o, single bit per endpoint, 0..31
  $254 equ USBDevDmaReqClr	\ w/o
  $258 equ USBDevDmaReqSet	\ w/o

\  $27C equ USBDevDcRev
  $280 equ USBDevCaHead
  $284 equ USBDevEpDmaSt
  $288 equ USBDevEpDmaEn
  $28C equ USBDevEpDmaDis
  $290 equ USBDevDmaIntSt	\ r/o
  $294 equ USBDevDmaIntEn	\ r/w
  $2A0 equ USBDevEotIntSt	\ r/o
  $2A4 equ USBDevEotIntClr	\ w/o
  $2A8 equ USBDevEotIntSet	\ w/o
  $2AC equ USBDevDdrIntSt	\ r/o
  $2B0 equ USBDevDdrIntClr	\ w/o
  $2B4 equ USBDevDdrIntSet	\ w/o
  $2B8 equ USBDevSysErrIntSt	\ r/o
  $2BC equ USBDevSysErrIntClr	\ w/o
  $2C0 equ USBDevSysErrIntSet	\ w/o

  $2FC equ USBDevModId

\ OTG I2C registers
  $300 equ usbI2cRx		\ r/o
  $300 equ usbI2cTx		\ w/o
  $304 equ usbI2cSt		\ r/w
  $308 equ usbI2cCtrl		\ r/w
  $30C equ usbI2cClkHi		\ r/w
  $310 equ usbI2cClkLo		\ w/o

\ Clock control
\ for USB host/dev
  $FF4 equ USBClkCtrl
  $FF8 equ USBClkSt
\ for use with OTG - same registers, different names
  $FF4 equ otgClkCtrl
  $FF8 equ otgClkSt


\ =====
\ UARTs
\ =====

#define _UART0	$4000:C000	\ APB0
#define _UART1	$4001:0000	\ APB0
#define _UART2	$4009:8000	\ APB1 - BJC005
#define _UART3	$4009:C000	\ APB1
#define _UART4  $400A:4000	\ APB1
#define _USART4 $400A:4000	\ APB1
  $00 equ UnRBR
  $00 equ UnTHR
  $00 equ UnDLL
  $04 equ UnIER
  $04 equ UnDLM
  $08 equ UnIIR
  $08 equ UnFCR
  $0C equ UnLCR
  $14 equ UnLSR
  $1C equ UnSCR
  $20 equ UnACR
  $24 equ UnICR
  $28 equ UnFDR
  $30 equ UnTER			\ - NOT U4
  
  $4C equ UnRS485Ctl		\ new, U1 only
  $50 equ UnRS485AdrMatch	\ new, U1 only
  $54 equ UnRS485Dly		\ new, U1 only
  $58 equ UnFIFOLVL		\ new in LPC17xx

  $10 equ U1MCR			\ - U1 only
  $18 equ UnMSR			\ - U1 only
  $18 equ U1MSR			\ - U1 only

\ USART 4 Only
  $2C equ U4OSR
  $48 equ U4SCICTRL
  $58 equ U4SYNCCTRL
  
\   UARTx LCR Register bits
     2  equ ~UnLCR-7BIT
     3  equ ~UnLCR-8BIT
  BIT2  equ ~UnLCR-SBS
  BIT3  equ ~UnLCR-PE
  BIT3 BIT5 or 
        equ UnLCR-PARITY1
  BIT3 BIT4 or BIT5 or
        equ UnLCR-PARITY0
  BIT7  equ ~UnLCR-DLAB
  BIT3 BIT4 or BIT5 or not
        equ UnLCR-PMASK
        
\   UARTx IER bits
  BIT0  equ ~UnIER-RBRIE    \ receiver (Rx character) interrupt enable
  BIT1  equ ~UnIER-THREIE   \ Tx FIFO threshold int enable
  BIT2  equ ~UnIER-RXIE     \ Rx Line status (Rx error) interrupt
  
\   UARTx IIR bits & fields
  BIT0  equ ~UnIIR-INSTATUS \ 0=interrupt pending
  BIT1 BIT2 and BIT3 and
        equ ~UnIIR-INTID    \ interrupt ID

\   UARTx LSR Register bits
  BIT0  equ ~UnLSR-RDR
  BIT1  equ ~UnLSR-OE
  BIT2  equ ~UnLSR-PE
  BIT3  equ ~UnLSR-FE
  BIT4  equ ~UnLSR-BI
  BIT5  equ ~UnLSR-THRE
  BIT6  equ ~UnLSR-TEMT
  BIT7  equ ~UnLSR-RXFE

\   UARTx FCR Register bits & fields
  BIT0  equ FCR-FIFOEN~
  BIT1  equ FCR-RxFIFO-RES~
  BIT2  equ FCR-TxFIFO-RES~
  BIT3  equ FCR-DMA-EN~
  0 6 lshift equ FCR-RxTRIG-1~
  1 6 lshift equ FCR-RxTRIG-4~
  2 6 lshift equ FCR-RxTRIG-8~
  3 6 lshift equ FCR-RxTRIG-14~

\   UARTx RS485 control reg bits
  BIT0  equ ~Un485-NMMEN    \ mutidrop mode
  BIT1  equ ~Un485-RXDIS
  BIT2  equ ~Un485-AADEN
  BIT4  equ ~Un485-DCTRL
  BIT5  equ ~Un485-OINV
  
\   UART1 MCR bits & fields
  BIT0 BIT1 or BIT4 or BIT6 or BIT7 or
        equ   ~U1MCRmask
  BIT0  equ   ~U1MCR-DTR
  BIT1  equ   ~U1MCR-RTS
  BIT6  equ   ~U1MCR-RTSAUTO
  BIT7  equ   ~U1MCR-CTSAUTO


\ ======
\ CAN1/2
\ ======

#define _CANram	 $4003:8000	\ APB0, 2kb

#define _CANfilter  $4003:C000	\ APB0, central registers
  $00 equ cfAFMR
  $04 equ cfSFF_sa
  $08 equ cfSFF_GRP_sa
  $0C equ cfEFF_sa
  $10 equ cfEFF_GRP_sa
  $14 equ cfEndOfTable
  $18 equ cfLutErrAd
  $1C equ cfLutErr

#define _CANgen  $4004:0000	\ APB0
  $00 equ cgTSR
  $04 equ cgRSS
  $08 equ cgMisc

#define _CAN1 $4004:4000	\ APB0
#define _CAN4 $4004:8000	\ APB0
  $00 equ CANnMOD
  $04 equ CANnCMR
  $08 equ CANnGSR
  $0C equ CANnICR
  $10 equ CANnIER
  $14 equ CANnBTR
  $18 equ CANnEWL
  $1C equ CANnSR
  $20 equ CANnRFS
  $24 equ CANnRID
  $28 equ CANnRDA
  $2C equ CANnRDB

  $30 equ CANnTFI1
  $34 equ CANnTID1
  $38 equ CANnTDA1
  $3C equ CANnTDB1

  $40 equ CANnTFI2
  $44 equ CANnTID2
  $48 equ CANnTDA2
  $4C equ CANnTDB2

  $50 equ CANnTFI3
  $54 equ CANnTID3
  $58 equ CANnTDA3
  $5C equ CANnTDB3

#define _CANSleepClr $400F:C110
#define _CANWakeFlags $400F:C114


\ ======
\ CMP0/1
\ ======

#define _CMP	$4002:0000	\ APB0
  $00 equ cmpCTRL
  $04 equ cmpCTRL0
  $08 equ cmpCTRL1


\ ========
\ SSP0/1/2
\ ========

#define _SSP0	$4008:8000	\ APB1
#define _SSP1	$4003:0000	\ APB0
#define _SSP2	$400A:C000	\ APB1
  $00 equ sspCR0
  $04 equ sspCR1
  $08 equ sspDR
  $0C equ sspSR
  $10 equ sspCPSR
  $14 equ sspIMSC
  $18 equ sspRIS
  $1C equ sspMIS
  $20 equ sspICR
  $24 equ sspDMACR

\ Bits in the Status Register
  bit0  equ ~SSPSR-TFE
  bit1  equ ~SSPSR-TNF
  bit2  equ ~SSPSR-RNE
  bit3  equ ~SSPSR-RFF
  bit4  equ ~SSPSR-BSY
  
\ Bits in CR0
  #7    equ ~SSPCR0-DSS8
  #15   equ ~SSPCR0-DSS16  
  bit6  equ ~SSPCR0-CPOL
  bit7  equ ~SSPCR0-CPHA

\ ========
\ I2C0/1/2
\ ========

#define _I2C0	$4001:C000	\ APB0
#define _I2C1	$4005:C000	\ APB0
#define _I2C2	$400A:0000	\ APB1
  $00 equ I2CONSET
  $04 equ I2STAT
  $08 equ I2DAT
  $0C equ I2ADR
  $10 equ I2SCLH		\ 16 bit
  $14 equ I2SCLL		\ 16 bit
  $18 equ I2CONCLR
  \ new for LPC17xx
  $1C equ i2MMCTRL
  $20 equ i2ADR1
  $24 equ i2ADR2
  $28 equ i2ADR3
  $2C equ i2DataBuffer
  $30 equ i2MASK0
  $34 equ i2MASK1
  $38 equ i2MASK2
  $3C equ i2MASK3

\ Bits in the I2CONSET and I2CONCLR registers
  bit6 equ ~I2EN
  bit5 equ ~STA
  bit4 equ ~STO
  bit3 equ ~SI
  bit2 equ ~AA

#100 equ err_I2C_ACK	\ --n
\ *G Error code returned on ACK failure.


\ ===
\ I2S
\ ===

#define _I2S $400A:8000		\ APB1
  $00 equ i2sDAO
  $04 equ i2sDAI
  $08 equ i2sTXFIFO
  $0C equ i2sRXFIFO
  $10 equ i2sSTATE
  $14 equ i2sDMA1
  $18 equ i2sDMA2
  $1C equ i2sIRQ
  $20 equ i2sTXRATE
  $24 equ i2sRXRATE
  $28 equ i2sTXBITRATE
  $2C equ i2sRXBITRATE
  $30 equ i2sTXMODE
  $34 equ i2sRXMODE


\ ===
\ RIT
\ ===

#define _RIT $400B:0000		\ APB1
  $00 equ ritCompVal
  $04 equ ritMask
  $08 equ ritCtrl
  $0C equ ritCounter


\ ======
\ Timers
\ ======

#define _TIMER0	$4000:4000	\ APB0
#define _TIMER1	$4000:8000	\ APB0
#define _TIMER2 $4009:0000	\ APB1
#define _TIMER3 $4009:4000	\ APB1
  $00 equ TnIR
  $04 equ TnTCR
  $08 equ TnTC
  $0C equ TnPR
  $10 equ TnPC
  $14 equ TnMCR
  $18 equ TnMR0
  $1C equ TnMR1
  $20 equ TnMR2
  $24 equ TnMR3
  $28 equ TnCCR
  $2C equ TnCR0
  $30 equ TnCR1
  $34 equ TnCR2
  $38 equ TnCR3
  $3C equ TnEMR

  $70 equ TnCTCR


\ ===
\ PWM
\ ===

#define _PWM0	$4001:4000	\ APB0
#define _PWM1	$4001:8000	\ APB0
  $00 equ PWMnIR
  $04 equ PWMnTCR
  $08 equ PWMnTC
  $0C equ PWMnPR
  $10 equ PWMnPC
  $14 equ PWMnMCR
  $18 equ PWMnMR0
  $1C equ PWMnMR1
  $20 equ PWMnMR2
  $24 equ PWMnMR3
  $28 equ PWMnCCR
  $2C equ PWMnCR0
  $30 equ PWMnCR1
  $34 equ PWMnCR2
  $38 equ PWMnCR3
  $3C equ PWMnEMR
  $40 equ PWMnMR4
  $44 equ PWMnMR5
  $48 equ PWMnMR6
  $4C equ PWMnPCR
  $50 equ PWMnLER

  $70 equ PWMnCTCR


\ =====
\ MCPWM
\ =====

#define _MCPWM $400B:8000	\ APB1
  $00 equ mcCON
  $04 equ mcCON_SET
  $08 equ mcCON_CLR
  $0C equ mcCAPCON
  $10 equ mcCAPCON_SET
  $14 equ mcCAPCON_CLR
  $18 equ mcTC0
  $1C equ mcTC1
  $20 equ mcTC2
  $24 equ mcLIM0
  $28 equ mcLIM1
  $2C equ mcLIM2
  $30 equ mcMAT0
  $34 equ mcMAT1
  $38 equ mcMAT2
  $3C equ mcDT
  $40 equ mcCP
  $44 equ mcCAP0
  $48 equ mcCAP1
  $4C equ mcCAP2
  $50 equ mcINTEN
  $54 equ mcINTEN_SET
  $58 equ mcINTEN_CLR
  $5C equ mcCNTCON
  $60 equ mcCNTCON_SET
  $64 equ mcCNTCON_CLR
  $68 equ mcINTF
  $6C equ mcINTF_SET
  $70 equ mcINTF_CLR
  $74 equ mcCAP_CLR


\ ========================
\ QEI - quadrature encoder
\ ========================

#define _QEI $400B:C000		\ APB1
  $00 equ qeiCON
  $04 equ qeiSTAT
  $08 equ qeiCONF
  $0C equ qeiPOS
  $10 equ qeiMaxPos
  $14 equ qeiCmPos0
  $18 equ qeiCmPos1
  $1C equ qeiCmPos2
  $20 equ qeiInxCnt
  $24 equ qeiInxCmp
  $28 equ qeiLoad
  $2C equ qeiTime
  $30 equ qeiVel
  $34 equ qeiCap
  $38 equ qeiVelComp
  $3C equ qeiFilter

  $FDC equ qeiIES
  $FD8 equ qeiIEC
  $FE0 equ qeiIntStat
  $FE4 equ qeiIE
  $FE8 equ qeiClr
  $FEC equ qeiSet


\ ===
\ RTC
\ ===

#define _RTC	$4002:4000	\ APB0
  $00 equ rtcILR
  $04 equ rtcCTC
  $08 equ rtcCCR
  $0C equ rtcCIIR
  $10 equ rtcAMR
  $14 equ rtcCTIME0
  $18 equ rtcCTIME1
  $1C equ rtcCTIME2
  $20 equ rtcSEC
  $24 equ rtcMIN
  $28 equ rtcHOUR
  $2C equ rtcDOM
  $30 equ rtcDOW
  $34 equ rtcDOY
  $38 equ rtcMONTH
  $3C equ rtcYEAR
  $40 equ rtcCISS
  $44 equ rtcGPR0	\ new
  $48 equ rtcGPR1	\ new
  $4C equ rtcGPR2	\ new
  $50 equ rtcGPR3	\ new
  $54 equ rtcGPR4	\ new
  $58 equ rtcAUXEN	\ new
  $5C equ rtcAUX	\ new
  $60 equ rtcALSEC
  $64 equ rtcALMIN
  $68 equ rtcALHOUR
  $6C equ rtcALDOM
  $70 equ rtcALDOW
  $74 equ rtcALDOY
  $78 equ rtcALMON
  $7C equ rtcALYEAR
  $80 equ rtcPREINT
  $84 equ rtcPREFRAC


\ ========
\ Watchdog
\ ========

#define _WD	$4000:0000	\ APB0
  $00 equ WDMOD
  $04 equ WDTC
  $08 equ WDFEED
  $0C equ WDTV
  $10 equ WDCLKSEL

  $14 equ WDWARNINT
  $18 equ WDWINDOW
  
\   Bits in WDMOD
  BIT0  equ ~WDEN     \ enable bit
  BIT1  equ ~WDRESET  \ watchdog timeout does reset
  BIT2  equ ~WDTOF    \ bit set if watchdog timeout or feed error ocurred
                      \ used to test if a wd restart happened
  BIT3  equ ~WDINT    \ bit set on warning interrupt
  BIT4  equ ~WDPROTECT  \ set for windowed mode


\ ===
\ ADC
\ ===

4095	equ	ADC_MAX_RDG			\ full-scale reading

#define _ADC0	$4003:4000	\ APB0

  $00 equ ADnCR
  $04 equ ADnGDR
  $0C equ ADnINTEN
  $10 equ ADnDR0
  $14 equ ADnDR1
  $18 equ ADnDR2
  $1C equ ADnDR3
  $20 equ ADnDR4
  $24 equ ADnDR5
  $28 equ ADnDR6
  $2C equ ADnDR7
  $30 equ ADnSTAT

\ bits in ADCnGDR
  bit30 equ ~ADCGDR_Orun
  bit31	equ ~ADCGDR_Done
  $FFF0	equ ADC_RES_MASK		\ mask for the result bits
  4     equ ADC_RES_SHIFT		\ bits to shift to right-justify
  
\ bits in ADCnCR
BIT17 BIT18 or BIT19 or BIT20 or BIT22 or BIT23 or BIT28 or BIT19 or
BIT30 or BIT31 or     \ unused bits in ADC_CR
not EQU ADCCR_MASK    \ mask for all the USED bits

$FFFFFFF0 equ ADCCR_NO_CHLS \ mask off all the channel bits  

  bit0  equ ~ADC_SEL_CH0
  bit1  equ ~ADC_SEL_CH1
  bit2  equ ~ADC_SEL_CH2
  bit3  equ ~ADC_SEL_CH3
  bit4  equ ~ADC_SEL_CH4
  bit5  equ ~ADC_SEL_CH5
  bit6  equ ~ADC_SEL_CH6
  bit7  equ ~ADC_SEL_CH7
  bit16 equ ~ADCCR_BURST
  bit21 equ ~ADCCR_PDOWN
  bit24 equ ~ADCCR_STARTNOW
  bit27 equ ~ADCCR_EDGE


\ ===
\ DAC
\ ===

#define _DAC0	$4008:C000	\ APB1
  $00 equ DAnCR
  $04 equ DAnCtrl
  $08 equ DAnCntVal


\ *************************
\ MCI Memory card interface
\ *************************

$400C:0000 equ _MCI		\ MCI base address
  $000 equ mciPwr		\ Power control
  $004 equ mciClk		\ Clock control
  $008 equ mciArg		\ Command argument
  $00C equ mciCmd		\ Command
  $010 equ mciRespCmd		\ response to command
  $014 equ mciResp0		\ response 0
  $018 equ mciResp1
  $01C equ mciResp2
  $020 equ mciResp3
  $024 equ mciDatTim		\ data timer
  $028 equ mciDatLen		\ data control
  $02C equ mciDatCtrl		\ data control
  $030 equ mciDatCnt		\ data counter
  $034 equ mciSt		\ status
  $038 equ mciClr		\ clear
  $03C equ mciMask0		\ interrupt mask
  $040 equ mciMask1		\   ... according to Keil

  $048 equ mciFifoCnt		\ FIFO count

  $080 equ mciFifo		\ FIFO data, $080 to $0BC


\ **********************
\ DMA controller - GPDMA
\ **********************

$2008:0000 equ _GPDMA
  $0000 equ dmaIntSt
  $0004 equ dmaIntTCSt
  $0008 equ dmaIntTCClr
  $000C equ dmaIntErrSt
  $0010 equ dmaIntErrClr
  $0014 equ dmaRawIntTCSt
  $0018 equ dmaRawIntErrSt
  $001C equ dmaEnbldChans
  $0020 equ dmaSoftBReq
  $0024 equ dmaSoftSReq
  $0028 equ dmaSoftLBReq
  $002C equ dmaSoftLSReq
  $0030 equ dmaConfig
  $0034 equ dmaSync

  $0100 equ dmaChan0
  $0120 equ dmaChan1
  $0140 equ dmaChan2
  $0160 equ dmaChan3
  $0180 equ dmaChan4
  $01A0 equ dmaChan5
  $01C0 equ dmaChan6
  $01E0 equ dmaChan7
    $000 equ dmaChSrcAddr
    $004 equ dmaChDestAddr
    $008 equ dmaChLLI
    $00C equ dmaChCtrl
    $010 equ dmaChConfig

  $0100 equ dma0SrcAddr
  $0104 equ dma0DestAddr
  $0108 equ dma0LLI
  $010C equ dma0Ctrl
  $0110 equ dma0Config

  $0120 equ dma1SrcAddr
  $0124 equ dma1DestAddr
  $0128 equ dma1LLI
  $012C equ dma1Ctrl
  $0130 equ dma1Config

  $0140 equ dma2SrcAddr
  $0144 equ dma2DestAddr
  $0148 equ dma2LLI
  $014C equ dma2Ctrl
  $0150 equ dma2Config

  $0160 equ dma3SrcAddr
  $0164 equ dma3DestAddr
  $0168 equ dma3LLI
  $016C equ dma3Ctrl
  $0170 equ dma3Config

  $0180 equ dma4SrcAddr
  $0184 equ dma4DestAddr
  $0188 equ dma4LLI
  $018C equ dma4Ctrl
  $0190 equ dma4Config

  $01A0 equ dma5SrcAddr
  $01A4 equ dma5DestAddr
  $01A8 equ dma5LLI
  $01AC equ dma5Ctrl
  $01B0 equ dma5Config

  $01C0 equ dma6SrcAddr
  $01C4 equ dma6DestAddr
  $01C8 equ dma6LLI
  $01CC equ dma6Ctrl
  $01D0 equ dma6Config

  $01E0 equ dma7SrcAddr
  $01E4 equ dma7DestAddr
  $01E8 equ dma7LLI
  $01EC equ dma7Ctrl
  $01F0 equ dma7Config

\   bit defs for DMAxCtrl - OR as required
  0   18 lshift  EQU DMA_SRC_WID8~
  1   18 lshift  EQU DMA_SRC_WID16~
  2   18 lshift  EQU DMA_SRC_WID32~
  
  0   21 lshift  EQU DMA_DEST_WID8~
  1   21 lshift  EQU DMA_DEST_WID16~
  2   21 lshift  EQU DMA_DEST_WID32~
  
  0   12 lshift  EQU DMA_SRC_BURST1
  1   12 lshift  EQU DMA_SRC_BURST4
  2   12 lshift  EQU DMA_SRC_BURST8
  3   12 lshift  EQU DMA_SRC_BURST16
  4   12 lshift  EQU DMA_SRC_BURST32
  5   12 lshift  EQU DMA_SRC_BURST64
  6   12 lshift  EQU DMA_SRC_BURST128
  7   12 lshift  EQU DMA_SRC_BURST256
  
  0   15 lshift  EQU DMA_DEST_BURST1
  1   15 lshift  EQU DMA_DEST_BURST4
  2   15 lshift  EQU DMA_DEST_BURST8
  3   15 lshift  EQU DMA_DEST_BURST16
  4   15 lshift  EQU DMA_DEST_BURST32
  5   15 lshift  EQU DMA_DEST_BURST64
  6   15 lshift  EQU DMA_DEST_BURST128
  7   15 lshift  EQU DMA_DEST_BURST256
  
          BIT26  EQU DMA_SI~   \ source address increment
          BIT27  EQU DMA_DI~   \ destination address increment
          BIT31  EQU DMA_TCINT~  \ terminal count interrupt enable
          

\   bit defs for DMAxConfig - OR as required
          BIT0   EQU DMA_CH_ENABLE~

  0    1 lshift  EQU DMA_SRC_REQ0~
  1    1 lshift  EQU DMA_SRC_REQ1~
  2    1 lshift  EQU DMA_SRC_REQ2~
  3    1 lshift  EQU DMA_SRC_REQ3~
  4    1 lshift  EQU DMA_SRC_REQ4~
  5    1 lshift  EQU DMA_SRC_REQ5~
  6    1 lshift  EQU DMA_SRC_REQ6~
  7    1 lshift  EQU DMA_SRC_REQ7~
  8    1 lshift  EQU DMA_SRC_REQ8~
  9    1 lshift  EQU DMA_SRC_REQ9~
 10    1 lshift  EQU DMA_SRC_REQ10~
 11    1 lshift  EQU DMA_SRC_REQ11~
 12    1 lshift  EQU DMA_SRC_REQ12~
 13    1 lshift  EQU DMA_SRC_REQ13~
 14    1 lshift  EQU DMA_SRC_REQ14~
 15    1 lshift  EQU DMA_SRC_REQ15~

  0    6 lshift  EQU DMA_DEST_REQ0~
  1    6 lshift  EQU DMA_DEST_REQ1~
  2    6 lshift  EQU DMA_DEST_REQ2~
  3    6 lshift  EQU DMA_DEST_REQ3~
  4    6 lshift  EQU DMA_DEST_REQ4~
  5    6 lshift  EQU DMA_DEST_REQ5~
  6    6 lshift  EQU DMA_DEST_REQ6~
  7    6 lshift  EQU DMA_DEST_REQ7~
  8    6 lshift  EQU DMA_DEST_REQ8~
  9    6 lshift  EQU DMA_DEST_REQ9~
 10    6 lshift  EQU DMA_DEST_REQ10~
 11    6 lshift  EQU DMA_DEST_REQ11~
 12    6 lshift  EQU DMA_DEST_REQ12~
 13    6 lshift  EQU DMA_DEST_REQ13~
 14    6 lshift  EQU DMA_DEST_REQ14~
 15    6 lshift  EQU DMA_DEST_REQ15~  
\ DMA transfer modes
  0   11 lshift  EQU DMA_MEM>MEM~     \ Note: 'MEM' means memory
  1   11 lshift  EQU DMA_MEM>PER~     \ 'PER' means peripheral
  2   11 lshift  EQU DMA_PER>MEM~
  3   11 lshift  EQU DMA_PER>PER~
  4   11 lshift  EQU DMA_PER>PERC~
  5   11 lshift  EQU DMA_MEM>PERC~    \ 'PERC' means peripheral, and this
  6   11 lshift  EQU DMA_PERC>MEM~    \ controls transfer i.e. not TRANSFERSIZE
  7   11 lshift  EQU DMA_PERC>PER~    \ field in dmaxCtrl
  
          BIT14  EQU DMA_CONF_IE~  
          BIT15  EQU DMA_CONF_ITC~  


\ ***
\ EMC
\ ***

$2009:C000 equ	_EMC
  $000 equ EMCControl
  $004 equ EMCStatus
  $008 equ EMCConfig
  $020 equ EMCDynamicControl
  $024 equ EMCDynamicRefresh
  $028 equ EMCDynamicReadConfig
  $030 equ EMCDynamictRP
  $034 equ EMCDynamictRAS
  $038 equ EMCDynamictSREX
  $03C equ EMCDynamictAPR
  $040 equ EMCDynamictDAL
  $044 equ EMCDynamictWR
  $048 equ EMCDynamictRC
  $04C equ EMCDynamictRFC
  $050 equ EMCDynamictXSR
  $054 equ EMCDynamictRRD
  $058 equ EMCDynamictMRD
  $080 equ EMCStaticExtendedWait
  $100 equ EMCDynamicConfig0
  $104 equ EMCDynamicRasCas0
  $120 equ EMCDynamicConfig1
  $124 equ EMCDynamicRasCas1
  $140 equ EMCDynamicConfig2
  $144 equ EMCDynamicRasCas2
  $160 equ EMCDynamicConfig3
  $164 equ EMCDynamicRasCas3
  $200 equ EMCStaticConfig0
  $204 equ EMCStaticWaitWen0
  $208 equ EMCStaticWaitOen0
  $20C equ EMCStaticWaitRd0
  $210 equ EMCStaticWaitPage0
  $214 equ EMCStaticWaitWr0
  $218 equ EMCStaticWaitTurn0
  $220 equ EMCStaticConfig1
  $224 equ EMCStaticWaitWen1
  $228 equ EMCStaticWaitOen1
  $22C equ EMCStaticWaitRd1
  $230 equ EMCStaticWaitPage1
  $234 equ EMCStaticWaitWr1
  $238 equ EMCStaticWaitTurn1
  $240 equ EMCStaticConfig2
  $244 equ EMCStaticWaitWen2
  $248 equ EMCStaticWaitOen2
  $24C equ EMCStaticWaitRd2
  $250 equ EMCStaticWaitPage2
  $254 equ EMCStaticWaitWr2
  $258 equ EMCStaticWaitTurn2
  $260 equ EMCStaticConfig3
  $264 equ EMCStaticWaitWen3
  $268 equ EMCStaticWaitOen3
  $26C equ EMCStaticWaitRd3
  $270 equ EMCStaticWaitPage3
  $274 equ EMCStaticWaitWr3
  $278 equ EMCStaticWaitTurn3


\ **************
\ LCD Controller
\ **************

$2008:8000 equ _LCD				\ register base address
0x400FC1B8 equ _LCD_CFG

  $0000 equ LCD_TIMH
  $0004 equ LCD_TIMV
  $0008 equ LCD_POL
  $000C equ LCD_LE
  $0010 equ LCD_UPBASE
  $0014 equ LCD_LPBASE
  $0018 equ LCD_CTRL
  $001C equ LCD_INTMSK
  $0020 equ LCD_INTRAW
  $0024 equ LCD_INTSTAT
  $0028 equ LCD_INTCLR
  $002C equ LCD_UPCURR
  $0030 equ LCD_LPCURR
  $0200 equ LCD_PAL_BASE
  $0800 equ CRSR_IMG_BASE
  $0C00 equ CRSR_CTRL
  $0C04 equ CRSR_CFG
  $0C08 equ CRSR_PAL_BASE			\ base of 128 x 16 bit entries
  $0C10 equ CRSR_XY
  $0C14 equ CRSR_CLIP
  $0C20 equ CRSR_INTMSK
  $0C24 equ CRSR_INTCLR
  $0C28 equ CRSR_INTRAW
  $0C2C equ CRSR_INTSTAT

\ LCD_CTRL register bit definitions
  bit0 	equ ~LCDCTRL_EN
  0	equ ~LCDCTRL_1BPP
  2	equ ~LCDCTRL_2BPP
  4	equ ~LCDCTRL_4BPP
  6	equ ~LCDCTRL_8BPP
  8	equ ~LCDCTRL_16BPP
 #10	equ ~LCDCTRL_24BPP
 #12	equ ~LCDCTRL_16BPP_565
 #14	equ ~LCDCTRL_12BPP_444
  0	equ	~LCDCTRL_COLOUR
 bit4	equ	~LCDCTRL_MONO
  0	equ	~LCDCTRL_STN
 bit5	equ	~LCDCTRL_TFT
  0     equ ~LCDCTRL_MONO4
 bit6   equ ~LCDCTRL_MONO8
 bit7   equ ~LCDCTRL_DUALPANEL
  0	equ ~LCDCTRL_RGB
 bit8	equ ~LCDCTRL_BGR
  0	equ ~LCDCTRL_LE_BYTE		\ little-ended display buffer memory byte ordering
 bit9	equ ~LCDCTRL_BE_BYTE		\ big endian ''
  0     equ ~LCDCTRL_LE_PIXEL		\ little endian pixel packing
 bit10	equ ~LCDCTRL_BE_PIXEL		\ big endian ''
  0	equ ~LCDCTRL_LCD_OFF
 bit11	equ ~LCDCTRL_LCD_ON		\ power up the display
  0	equ ~LCDCTRL_VINT_VSYNC		\ vertical interrupt on start of vertical sync
 bit12	equ ~LCDCTRL_VINT_BPORCH 	\ '' on start of back porch
 bit13	equ ~LCDCTRL_VINT_ACTVID	\ '' on start of active video
bit12 bit13 or
	equ ~LCDCTRL_VINT_FPORCH	\ '' on start of front porch
0	equ ~LCDCTRL_FIFO_WM4		\ a LCD DMA request occurs when a FIFO has 4 or more locations free
bit16	equ ~LCDCTRL_FIFO_WM8		\ a LCD DMA request occurs when a FIFO has 8 or more locations free

bit16	equ ~LCDLE_ENABLE		\ LCD_LE register, enable bit

\ LCD_TIMH - horizontal timing register bit definition macros
\ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
interpreter
: +LCD_TIMH_PPL		\ n ppl | n'; merge pixels per line
  #16 / 1-				\ convert to register scaling
  2 lshift or
;

: +LCD_TIMH_HS_WIDTH	\ n width | n'; merge horizontal sync pulse width, in pixel clocks
  1-					\ convert to register scaling
  8 lshift or
;

: +LCD_TIMH_HF_PORCH	\ n horiz_f_porch | n'; merge horizontal front porch period, pixels
  1-					\ remove the register offset
  #16 lshift or
;

: +LCD_TIMH_HB_PORCH	\ n horiz_back_porch | n'; merge horizontal back porch period, pixels
  1-
  #24 lshift or
;
target

\ LCD_TIMV vertical timing register bit definition macros
\ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
interpreter
: +LCD_TIMV_LPP		\ n lpp | n'; merge the lines per panel (display rows)
  1-					\ remove the register offset
  or
;

: +LCD_TIMV_VS_WIDTH	\ n pwidth | n'; merge the vertical sync pulse width, in lines
  #10 lshift or
;

: +LCD_TIMV_VF_PORCH	\ n lpp | n'; merge the vertical front porch period, lines
  #16 lshift or
;

: +LCD_TIMV_VB_PORCH	\ n lpp | n'; merge the vertical back porch period, lines
  #24 lshift or
;
target


\ LCD_POL clock and signal polarity register bit definition macros
\ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
interpreter
: +LCD_POL_PCD		\ n divider | n'; set clock divisor this sets BOTH LO AND HI fields
  dup $3E0 and	#22 lshift		\ get ms 5 bits, shift to align
  swap $01F and				\ handle ls 5 bits
  or or													\ merge
;

: +LCD_POL_EXTCLK	\ n | n'; merge external clock bit
  bit5 or
;

: +LCD_POL_ACB		\ n acb | n'; merge acb value
  6 lshift or
;

: +LCD_POL_INV_VS	\ n | n'; select inverted sync (active low)
  bit11 or
;

: +LCD_POL_INV_HS	\ n | n'; select inverted horiz. sync (low active)
  bit12 or
;

: +LCD_POL_INV_PCLK	\ n | n'; select inverted panel clock (falling edge)
  bit13 or
;

: +LCD_POL_INV_OE	\ n | n'; select inverted output enable (active low)
  bit14 or
;

: +LCD_POL_CPL		\ n clocks_per_line | n'; merge clocks per line CPL value
  #16 lshift or
;

: +LCD_POL_BCD		\ n | n'; select bypassed pixel clock divider
  bit26 or
;

target


\ *********************
\ On Chip EEPROM Memory
\ *********************

 $0020:0000	equ	_EEPROM

  $80 equ EECMD				\ command register $0020:0080
  $84 equ EEADDR			\ address register
  $88 equ EEWDATA			\ write data register
  $8C equ EERDATA			\ read data register
  $90 equ EEWSTATE			\ wait state register $0020:0090
  $94 equ EECLKDIV			\ clock divider register
  $98 equ EEPWRDWN			\ power down register
  _EEPROM EEPWRDWN + equ _EEPWRDWN
 $FE4 equ EEINTEN			\ EEPROM interrupt register $0020:0FE4
 $FD8 equ EEENCLR			\ interrupt clear register $0020:0FD8
 $FDC equ EEENSET			\ interrupt enable set register $0020:0FDC
 $FE0 equ EESTAT			\ EEPROM interrupt status register $0020:0FE0
 _EEPROM EESTAT + equ _EESTAT
 $FE8 equ EESTATCLR		\ status register clear register $0020:0FE8
 $FEC equ EESTATSET   \ status register set bit register $0020:0FEC

\ Command codes for EECMD register
 $00 equ EE_CMD_8bREAD
 $01 equ EE_CMD_16bREAD
 $02 equ EE_CMD_32bREAD
 $03 equ EE_CMD_8bWRITE
 $04 equ EE_CMD_16bWRITE
 $05 equ EE_CMD_32bWRITE
 $06 equ EE_CMD_EP_PAGE			\ erase, program one page
BIT3 equ EE_CMD_PREFETCH		\ .. to merge with above, if required

\ Bits in EESTAT / EESTATCLR / EESTATSET
bit26 equ ~EESTAT_RDWR
bit28 equ ~EESTAT_PROG



\ *********************
\ Hardware CRC Generator
\ *********************

$2009:0000 equ _CRC-GEN    \ base address

  $00 equ CRCMODE
  $04 equ CRCSEED   \ this sets the initial value & resets CRC gen on writing
  $08 equ CRCVAL    \ the calculated value
  $08 equ CRCDATA   \ write byte, word or cell to add to CRC
  
\   Bits for CRC mode
  $0000   equ ~CRC_POLY-CCITT
  $0001   equ ~CRC_POLY-16
  $0002   equ ~CRC_POLY-32
  
  BIT2    equ ~CRC-BIT_RVS_WR
  BIT3    equ ~CRC-CMPL_WR
  BIT4    equ ~CRC-BIT_RVS_SUM
  BIT5    equ ~CRC-CMPL_SUM


\ ***********************
\ *S Miscellaneous macros
\ ***********************

interpreter
: PCLKdiv	\ divider -- x
\ *G Given a peripheral clock divider ratio (1, 2, 4, 6, 8) return
\ ** the bit pattern for the PCLKSELx register bits.
  case
    1 of  1  endof	\ 0001 -> 01
    2 of  2  endof	\ 0010 -> 10
    4 of  0  endof	\ 0100 -> 00
    6 of  3  endof	\ 		CAN peripherals
    8 of  3  endof	\ 1000 -> 11	non-CAN peripherals
      1 abort" Invalid peripheral clock divider"
  endcase
;

(( ---- these interpreted words are now defined by default, uncomment if required ----
: kHz		\ n -- Hz
\ *G Multiply by 1000.
  #1000 *  ;
: MHz		\ n -- Hz
\ *G Multiply by 1,000,000.
  #1000000 *  ;
))

: setMask	\ value mask addr -- ; cell operation
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;

: DALIGNED	\ n -- n'
\ *G Align to a multiple of 8 bytes (double cell).
  7 + -8 and
;
target


\ ======
\ *> ###
\ ======

decimal
