\ sfrRP2040.fth - peripheral register addresses

((
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)7803 903612
email: tech-support@vfxforth.com
web:   http://www.vfxforth.com
Skype: mpe_sfp

This file produced by the MPE SVDconv utility
on Wed, 24 May 2023 16:53:32     .


CPU: CM0PLUS, r0p1
  Endian: little
  MPU: true
  FPU: false
  NVIC priority bits: 2
  Vendor Systick Config: false
))


\ =================================
\ Read/Modify/Write address aliases
\ =================================
\ When these valaues are added to a peripheral register address,
\ the normal store behaviour is converted to a read-modify-write
\ operation. ALIAS_RW is just there for completeness. See section
\ 2.1.2 Atomic Register Access in the RP2040 datasheet.

$0000 equ ALIAS_RW		\ normal write
$1000 equ ALIAS_XOR		\ set bits in value cause toggle
$2000 equ ALIAS_SET		\ set bits in value are set
$3000 equ ALIAS_CLR		\ set bits in value are cleared


\ ====================================== 
\ XIP_CTRL - QSPI flash execute-in-place block
\ ====================================== 

$1400:0000 equ _XIP_CTRL
#22 equ XIP_IRQ_IRQn
  $0000 equ xip.CTRL             \ Cache control
  $0004 equ xip.FLUSH            \ Cache Flush control
  $0008 equ xip.STAT             \ Cache Status
  $000C equ xip.CTR_HIT          \ Cache Hit counter
                                 \ A 32 bit saturating counter that increments upon each cache hit,
                                 \ i.e. when an XIP access is serviced directly from cached data.
                                 \ Write any value to clear.
  $0010 equ xip.CTR_ACC          \ Cache Access counter
                                 \ A 32 bit saturating counter that increments upon each XIP access,
                                 \ whether the cache is hit or not. This includes noncacheable accesses.
                                 \ Write any value to clear.
  $0014 equ xip.STREAM_ADDR      \ FIFO stream address
  $0018 equ xip.STREAM_CTR       \ FIFO stream control
  $001C equ xip.STREAM_FIFO      \ FIFO stream data
                                 \ Streamed data is buffered here, for retrieval by the system DMA.
                                 \ This FIFO can also be accessed via the XIP_AUX slave, to avoid exposing
                                 \ the DMA to bus stalls caused by other XIP traffic.


\ ====================================== 
\ XIP_SSI - DW_apb_ssi has the following features:
\ * APB interface – Allows for easy integration into a DesignWare Synthesizable Components for AMBA 2 implementation.
\ * APB3 and APB4 protocol support.
\ * Scalable APB data bus width – Supports APB data bus widths of 8, 16, and 32 bits.
\ * Serial-master or serial-slave operation – Enables serial communication with serial-master or serial-slave peripheral devices.
\ * Programmable Dual/Quad/Octal SPI support in Master Mode.
\ * Dual Data Rate (DDR) and Read Data Strobe (RDS) Support - Enables the DW_apb_ssi master to perform operations with the device in DDR and RDS modes when working in Dual/Quad/Octal mode of operation.
\ * Data Mask Support - Enables the DW_apb_ssi to selectively update the bytes in the device. This feature is applicable only in enhanced SPI modes.
\ * eXecute-In-Place (XIP) support - Enables the DW_apb_ssi master to behave as a memory mapped I/O and fetches the data from the device based on the APB read request. This feature is applicable only in enhanced SPI modes.
\ * DMA Controller Interface – Enables the DW_apb_ssi to interface to a DMA controller over the bus using a handshaking interface for transfer requests.
\ * Independent masking of interrupts – Master collision, transmit FIFO overflow, transmit FIFO empty, receive FIFO full, receive FIFO underflow, and receive FIFO overflow interrupts can all be masked independently.
\ * Multi-master contention detection – Informs the processor of multiple serial-master accesses on the serial bus.
\ * Bypass of meta-stability flip-flops for synchronous clocks – When the APB clock (pclk) and the DW_apb_ssi serial clock (ssi_clk) are synchronous, meta-stable flip-flops are not used when transferring control signals across these clock domains.
\ * Programmable delay on the sample time of the received serial data bit (rxd); enables programmable control of routing delays resulting in higher serial data-bit rates.
\ * Programmable features:
\ - Serial interface operation – Choice of Motorola SPI, Texas Instruments Synchronous Serial Protocol or National Semiconductor Microwire.
\ - Clock bit-rate – Dynamic control of the serial bit rate of the data transfer; used in only serial-master mode of operation.
\ - Data Item size (4 to 32 bits) – Item size of each data transfer under the control of the programmer.
\ * Configured features:
\ - FIFO depth – 16 words deep. The FIFO width is fixed at 32 bits.
\ - 1 slave select output.
\ - Hardware slave-select – Dedicated hardware slave-select line.
\ - Combined interrupt line - one combined interrupt line from the DW_apb_ssi to the interrupt controller.
\ - Interrupt polarity – active high interrupt lines.
\ - Serial clock polarity – low serial-clock polarity directly after reset.
\ - Serial clock phase – capture on first edge of serial-clock directly after reset.
\ ====================================== 

$1800:0000 equ _XIP_SSI
  $0000 equ ssi.CTRLR0           \ Control register 0
  $0004 equ ssi.CTRLR1           \ Master Control register 1
  $0008 equ ssi.SSIENR           \ SSI Enable
  $000C equ ssi.MWCR             \ Microwire Control
  $0010 equ ssi.SER              \ Slave enable
  $0014 equ ssi.BAUDR            \ Baud rate
  $0018 equ ssi.TXFTLR           \ TX FIFO threshold level
  $001C equ ssi.RXFTLR           \ RX FIFO threshold level
  $0020 equ ssi.TXFLR            \ TX FIFO level
  $0024 equ ssi.RXFLR            \ RX FIFO level
  $0028 equ ssi.SR               \ Status register
  $002C equ ssi.IMR              \ Interrupt mask
  $0030 equ ssi.ISR              \ Interrupt status
  $0034 equ ssi.RISR             \ Raw interrupt status
  $0038 equ ssi.TXOICR           \ TX FIFO overflow interrupt clear
  $003C equ ssi.RXOICR           \ RX FIFO overflow interrupt clear
  $0040 equ ssi.RXUICR           \ RX FIFO underflow interrupt clear
  $0044 equ ssi.MSTICR           \ Multi-master interrupt clear
  $0048 equ ssi.ICR              \ Interrupt clear
  $004C equ ssi.DMACR            \ DMA control
  $0050 equ ssi.DMATDLR          \ DMA TX data level
  $0054 equ ssi.DMARDLR          \ DMA RX data level
  $0058 equ ssi.IDR              \ Identification register
  $005C equ ssi.SSI_VERSION_ID   \ Version ID
  $0060 equ ssi.DR0              \ Data Register 0 (of 36)
  $00F0 equ ssi.RX_SAMPLE_DLY    \ RX sample delay
  $00F4 equ ssi.SPI_CTRLR0       \ SPI control
  $00F8 equ ssi.TXD_DRIVE_EDGE   \ TX drive edge


\ ====================================== 
\ SYSINFO - TX drive edge
\ ====================================== 

$4000:0000 equ _SYSINFO
  $0000 equ si.CHIP_ID           \ JEDEC JEP-106 compliant chip identifier.
  $0004 equ si.PLATFORM          \ Platform register. Allows software to know what environment it is running in.
  $0040 equ si.GITREF_RP2040     \ Git hash of the chip source. Used to identify chip version.


\ ====================================== 
\ SYSCFG - Register block for various chip control signals
\ ====================================== 

$4000:4000 equ _SYSCFG
  $0000 equ sc.PROC0_NMI_MASK    \ Processor core 0 NMI source mask
                                 \ Set a bit high to enable NMI from that IRQ
  $0004 equ sc.PROC1_NMI_MASK    \ Processor core 1 NMI source mask
                                 \ Set a bit high to enable NMI from that IRQ
  $0008 equ sc.PROC_CONFIG       \ Configuration for processors
  $000C equ sc.PROC_IN_SYNC_BYPASS \ For each bit, if 1, bypass the input synchronizer between that GPIO
                                 \ and the GPIO input register in the SIO. The input synchronizers should
                                 \ generally be unbypassed, to avoid injecting metastabilities into processors.
                                 \ If you're feeling brave, you can bypass to save two cycles of input
                                 \ latency. This register applies to GPIO 0...29.
  $0010 equ sc.PROC_IN_SYNC_BYPASS_HI \ For each bit, if 1, bypass the input synchronizer between that GPIO
                                 \ and the GPIO input register in the SIO. The input synchronizers should
                                 \ generally be unbypassed, to avoid injecting metastabilities into processors.
                                 \ If you're feeling brave, you can bypass to save two cycles of input
                                 \ latency. This register applies to GPIO 30...35 (the QSPI IOs).
  $0014 equ sc.DBGFORCE          \ Directly control the SWD debug port of either processor
  $0018 equ sc.MEMPOWERDOWN      \ Control power downs to memories. Set high to power down memories.
                                 \ Use with extreme caution


\ ====================================== 
\ MEMPOWERDOWN - Control power downs to memories. Set high to power down memories.
\ Use with extreme caution
\ ====================================== 

$4000:8000 equ _MEMPOWERDOWN
#33 equ CLOCKS_IRQ_IRQn
  $0000 equ clk.CLK_GPOUT0_CTRL  \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0004 equ clk.CLK_GPOUT0_DIV   \ Clock divisor, can be changed on-the-fly
  $0008 equ clk.CLK_GPOUT0_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $000C equ clk.CLK_GPOUT1_CTRL  \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0010 equ clk.CLK_GPOUT1_DIV   \ Clock divisor, can be changed on-the-fly
  $0014 equ clk.CLK_GPOUT1_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $0018 equ clk.CLK_GPOUT2_CTRL  \ Clock control, can be changed on-the-fly (except for auxsrc)
  $001C equ clk.CLK_GPOUT2_DIV   \ Clock divisor, can be changed on-the-fly
  $0020 equ clk.CLK_GPOUT2_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $0024 equ clk.CLK_GPOUT3_CTRL  \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0028 equ clk.CLK_GPOUT3_DIV   \ Clock divisor, can be changed on-the-fly
  $002C equ clk.CLK_GPOUT3_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $0030 equ clk.CLK_REF_CTRL     \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0034 equ clk.CLK_REF_DIV      \ Clock divisor, can be changed on-the-fly
  $0038 equ clk.CLK_REF_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ The glitchless multiplexer does not switch instantaneously (to avoid glitches), so software should poll this register to wait for the switch to complete. This register contains one decoded bit for each of the clock sources enumerated in the CTRL SRC field. At most one of these bits will be set at any time, indicating that clock is currently present at the output of the glitchless mux. Whilst switching is in progress, this register may briefly show all-0s.
  $003C equ clk.CLK_SYS_CTRL     \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0040 equ clk.CLK_SYS_DIV      \ Clock divisor, can be changed on-the-fly
  $0044 equ clk.CLK_SYS_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ The glitchless multiplexer does not switch instantaneously (to avoid glitches), so software should poll this register to wait for the switch to complete. This register contains one decoded bit for each of the clock sources enumerated in the CTRL SRC field. At most one of these bits will be set at any time, indicating that clock is currently present at the output of the glitchless mux. Whilst switching is in progress, this register may briefly show all-0s.
  $0048 equ clk.CLK_PERI_CTRL    \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0050 equ clk.CLK_PERI_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $0054 equ clk.CLK_USB_CTRL     \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0058 equ clk.CLK_USB_DIV      \ Clock divisor, can be changed on-the-fly
  $005C equ clk.CLK_USB_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $0060 equ clk.CLK_ADC_CTRL     \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0064 equ clk.CLK_ADC_DIV      \ Clock divisor, can be changed on-the-fly
  $0068 equ clk.CLK_ADC_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $006C equ clk.CLK_RTC_CTRL     \ Clock control, can be changed on-the-fly (except for auxsrc)
  $0070 equ clk.CLK_RTC_DIV      \ Clock divisor, can be changed on-the-fly
  $0074 equ clk.CLK_RTC_SELECTED \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $0078 equ clk.CLK_SYS_RESUS_CTRL \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $007C equ clk.CLK_SYS_RESUS_STATUS \ Indicates which SRC is currently selected by the glitchless mux (one-hot).
                                 \ This slice does not have a glitchless mux (only the AUX_SRC field is present, not SRC) so this register is hardwired to 0x1.
  $0080 equ clk.FC0_REF_KHZ      \ Reference clock frequency in kHz
  $0084 equ clk.FC0_MIN_KHZ      \ Minimum pass frequency in kHz. This is optional. Set to 0 if you are not using the pass/fail flags
  $0088 equ clk.FC0_MAX_KHZ      \ Maximum pass frequency in kHz. This is optional. Set to 0x1ffffff if you are not using the pass/fail flags
  $008C equ clk.FC0_DELAY        \ Delays the start of frequency counting to allow the mux to settle
                                 \ Delay is measured in multiples of the reference clock period
  $0090 equ clk.FC0_INTERVAL     \ The test interval is 0.98us * 2**interval, but let's call it 1us * 2**interval
                                 \ The default gives a test interval of 250us
  $0094 equ clk.FC0_SRC          \ Clock sent to frequency counter, set to 0 when not required
                                 \ Writing to this register initiates the frequency count
  $0098 equ clk.FC0_STATUS       \ Frequency counter status
  $009C equ clk.FC0_RESULT       \ Result of frequency measurement, only valid when status_done=1
  $00A0 equ clk.WAKE_EN0         \ enable clock in wake mode
  $00A4 equ clk.WAKE_EN1         \ enable clock in wake mode
  $00A8 equ clk.SLEEP_EN0        \ enable clock in sleep mode
  $00AC equ clk.SLEEP_EN1        \ enable clock in sleep mode
  $00B0 equ clk.ENABLED0         \ indicates the state of the clock enable
  $00B4 equ clk.ENABLED1         \ indicates the state of the clock enable
  $00B8 equ clk.INTR             \ Raw Interrupts
  $00BC equ clk.INTE             \ Interrupt Enable
  $00C0 equ clk.INTF             \ Interrupt Force
  $00C4 equ clk.INTS             \ Interrupt status after masking & forcing


\ ====================================== 
\ RESETS - Interrupt status after masking & forcing
\ ====================================== 

$4000:C000 equ _RESETS
  $0000 equ rst.RESET            \ Reset control. If a bit is set it means the peripheral is in reset. 0 means the peripheral's reset is deasserted.
  $0004 equ rst.WDSEL            \ Watchdog select. If a bit is set then the watchdog will reset this peripheral when the watchdog fires.
  $0008 equ rst.RESET_DONE       \ Reset done. If a bit is set then a reset done signal has been returned by the peripheral. This indicates that the peripheral's registers are ready to be accessed.


\ ====================================== 
\ PSM - Reset done. If a bit is set then a reset done signal has been returned by the peripheral. This indicates that the peripheral's registers are ready to be accessed.
\ ====================================== 

$4001:0000 equ _PSM
  $0000 equ psm.FRCE_ON          \ Force block out of reset (i.e. power it on)
  $0004 equ psm.FRCE_OFF         \ Force into reset (i.e. power it off)
  $0008 equ psm.WDSEL            \ Set to 1 if this peripheral should be reset when the watchdog fires.
  $000C equ psm.DONE             \ Indicates the peripheral's registers are ready to access.


\ ====================================== 
\ DONE - Indicates the peripheral's registers are ready to access.
\ ====================================== 

$4001:4000 equ _DONE
#29 equ IO_IRQ_BANK0_IRQn
  $0000 equ iob0.GPIO0_STATUS    \ GPIO status
  $0004 equ iob0.GPIO0_CTRL      \ GPIO control including function select and overrides.
  $0008 equ iob0.GPIO1_STATUS    \ GPIO status
  $000C equ iob0.GPIO1_CTRL      \ GPIO control including function select and overrides.
  $0010 equ iob0.GPIO2_STATUS    \ GPIO status
  $0014 equ iob0.GPIO2_CTRL      \ GPIO control including function select and overrides.
  $0018 equ iob0.GPIO3_STATUS    \ GPIO status
  $001C equ iob0.GPIO3_CTRL      \ GPIO control including function select and overrides.
  $0020 equ iob0.GPIO4_STATUS    \ GPIO status
  $0024 equ iob0.GPIO4_CTRL      \ GPIO control including function select and overrides.
  $0028 equ iob0.GPIO5_STATUS    \ GPIO status
  $002C equ iob0.GPIO5_CTRL      \ GPIO control including function select and overrides.
  $0030 equ iob0.GPIO6_STATUS    \ GPIO status
  $0034 equ iob0.GPIO6_CTRL      \ GPIO control including function select and overrides.
  $0038 equ iob0.GPIO7_STATUS    \ GPIO status
  $003C equ iob0.GPIO7_CTRL      \ GPIO control including function select and overrides.
  $0040 equ iob0.GPIO8_STATUS    \ GPIO status
  $0044 equ iob0.GPIO8_CTRL      \ GPIO control including function select and overrides.
  $0048 equ iob0.GPIO9_STATUS    \ GPIO status
  $004C equ iob0.GPIO9_CTRL      \ GPIO control including function select and overrides.
  $0050 equ iob0.GPIO10_STATUS   \ GPIO status
  $0054 equ iob0.GPIO10_CTRL     \ GPIO control including function select and overrides.
  $0058 equ iob0.GPIO11_STATUS   \ GPIO status
  $005C equ iob0.GPIO11_CTRL     \ GPIO control including function select and overrides.
  $0060 equ iob0.GPIO12_STATUS   \ GPIO status
  $0064 equ iob0.GPIO12_CTRL     \ GPIO control including function select and overrides.
  $0068 equ iob0.GPIO13_STATUS   \ GPIO status
  $006C equ iob0.GPIO13_CTRL     \ GPIO control including function select and overrides.
  $0070 equ iob0.GPIO14_STATUS   \ GPIO status
  $0074 equ iob0.GPIO14_CTRL     \ GPIO control including function select and overrides.
  $0078 equ iob0.GPIO15_STATUS   \ GPIO status
  $007C equ iob0.GPIO15_CTRL     \ GPIO control including function select and overrides.
  $0080 equ iob0.GPIO16_STATUS   \ GPIO status
  $0084 equ iob0.GPIO16_CTRL     \ GPIO control including function select and overrides.
  $0088 equ iob0.GPIO17_STATUS   \ GPIO status
  $008C equ iob0.GPIO17_CTRL     \ GPIO control including function select and overrides.
  $0090 equ iob0.GPIO18_STATUS   \ GPIO status
  $0094 equ iob0.GPIO18_CTRL     \ GPIO control including function select and overrides.
  $0098 equ iob0.GPIO19_STATUS   \ GPIO status
  $009C equ iob0.GPIO19_CTRL     \ GPIO control including function select and overrides.
  $00A0 equ iob0.GPIO20_STATUS   \ GPIO status
  $00A4 equ iob0.GPIO20_CTRL     \ GPIO control including function select and overrides.
  $00A8 equ iob0.GPIO21_STATUS   \ GPIO status
  $00AC equ iob0.GPIO21_CTRL     \ GPIO control including function select and overrides.
  $00B0 equ iob0.GPIO22_STATUS   \ GPIO status
  $00B4 equ iob0.GPIO22_CTRL     \ GPIO control including function select and overrides.
  $00B8 equ iob0.GPIO23_STATUS   \ GPIO status
  $00BC equ iob0.GPIO23_CTRL     \ GPIO control including function select and overrides.
  $00C0 equ iob0.GPIO24_STATUS   \ GPIO status
  $00C4 equ iob0.GPIO24_CTRL     \ GPIO control including function select and overrides.
  $00C8 equ iob0.GPIO25_STATUS   \ GPIO status
  $00CC equ iob0.GPIO25_CTRL     \ GPIO control including function select and overrides.
  $00D0 equ iob0.GPIO26_STATUS   \ GPIO status
  $00D4 equ iob0.GPIO26_CTRL     \ GPIO control including function select and overrides.
  $00D8 equ iob0.GPIO27_STATUS   \ GPIO status
  $00DC equ iob0.GPIO27_CTRL     \ GPIO control including function select and overrides.
  $00E0 equ iob0.GPIO28_STATUS   \ GPIO status
  $00E4 equ iob0.GPIO28_CTRL     \ GPIO control including function select and overrides.
  $00E8 equ iob0.GPIO29_STATUS   \ GPIO status
  $00EC equ iob0.GPIO29_CTRL     \ GPIO control including function select and overrides.
  $00F0 equ iob0.INTR0           \ Raw Interrupts
  $00F4 equ iob0.INTR1           \ Raw Interrupts
  $00F8 equ iob0.INTR2           \ Raw Interrupts
  $00FC equ iob0.INTR3           \ Raw Interrupts
  $0100 equ iob0.PROC0_INTE0     \ Interrupt Enable for proc0
  $0104 equ iob0.PROC0_INTE1     \ Interrupt Enable for proc0
  $0108 equ iob0.PROC0_INTE2     \ Interrupt Enable for proc0
  $010C equ iob0.PROC0_INTE3     \ Interrupt Enable for proc0
  $0110 equ iob0.PROC0_INTF0     \ Interrupt Force for proc0
  $0114 equ iob0.PROC0_INTF1     \ Interrupt Force for proc0
  $0118 equ iob0.PROC0_INTF2     \ Interrupt Force for proc0
  $011C equ iob0.PROC0_INTF3     \ Interrupt Force for proc0
  $0120 equ iob0.PROC0_INTS0     \ Interrupt status after masking & forcing for proc0
  $0124 equ iob0.PROC0_INTS1     \ Interrupt status after masking & forcing for proc0
  $0128 equ iob0.PROC0_INTS2     \ Interrupt status after masking & forcing for proc0
  $012C equ iob0.PROC0_INTS3     \ Interrupt status after masking & forcing for proc0
  $0130 equ iob0.PROC1_INTE0     \ Interrupt Enable for proc1
  $0134 equ iob0.PROC1_INTE1     \ Interrupt Enable for proc1
  $0138 equ iob0.PROC1_INTE2     \ Interrupt Enable for proc1
  $013C equ iob0.PROC1_INTE3     \ Interrupt Enable for proc1
  $0140 equ iob0.PROC1_INTF0     \ Interrupt Force for proc1
  $0144 equ iob0.PROC1_INTF1     \ Interrupt Force for proc1
  $0148 equ iob0.PROC1_INTF2     \ Interrupt Force for proc1
  $014C equ iob0.PROC1_INTF3     \ Interrupt Force for proc1
  $0150 equ iob0.PROC1_INTS0     \ Interrupt status after masking & forcing for proc1
  $0154 equ iob0.PROC1_INTS1     \ Interrupt status after masking & forcing for proc1
  $0158 equ iob0.PROC1_INTS2     \ Interrupt status after masking & forcing for proc1
  $015C equ iob0.PROC1_INTS3     \ Interrupt status after masking & forcing for proc1
  $0160 equ iob0.DORMANT_WAKE_INTE0 \ Interrupt Enable for dormant_wake
  $0164 equ iob0.DORMANT_WAKE_INTE1 \ Interrupt Enable for dormant_wake
  $0168 equ iob0.DORMANT_WAKE_INTE2 \ Interrupt Enable for dormant_wake
  $016C equ iob0.DORMANT_WAKE_INTE3 \ Interrupt Enable for dormant_wake
  $0170 equ iob0.DORMANT_WAKE_INTF0 \ Interrupt Force for dormant_wake
  $0174 equ iob0.DORMANT_WAKE_INTF1 \ Interrupt Force for dormant_wake
  $0178 equ iob0.DORMANT_WAKE_INTF2 \ Interrupt Force for dormant_wake
  $017C equ iob0.DORMANT_WAKE_INTF3 \ Interrupt Force for dormant_wake
  $0180 equ iob0.DORMANT_WAKE_INTS0 \ Interrupt status after masking & forcing for dormant_wake
  $0184 equ iob0.DORMANT_WAKE_INTS1 \ Interrupt status after masking & forcing for dormant_wake
  $0188 equ iob0.DORMANT_WAKE_INTS2 \ Interrupt status after masking & forcing for dormant_wake
  $018C equ iob0.DORMANT_WAKE_INTS3 \ Interrupt status after masking & forcing for dormant_wake


\ ====================================== 
\ DORMANT_WAKE_INTS3 - Interrupt status after masking & forcing for dormant_wake
\ ====================================== 

$4001:8000 equ _DORMANT_WAKE_INTS3
#30 equ IO_IRQ_QSPI_IRQn
  $0000 equ qspi.GPIO_QSPI_SCLK_STATUS \ GPIO status
  $0004 equ qspi.GPIO_QSPI_SCLK_CTRL \ GPIO control including function select and overrides.
  $0008 equ qspi.GPIO_QSPI_SS_STATUS \ GPIO status
  $000C equ qspi.GPIO_QSPI_SS_CTRL \ GPIO control including function select and overrides.
  $0010 equ qspi.GPIO_QSPI_SD0_STATUS \ GPIO status
  $0014 equ qspi.GPIO_QSPI_SD0_CTRL \ GPIO control including function select and overrides.
  $0018 equ qspi.GPIO_QSPI_SD1_STATUS \ GPIO status
  $001C equ qspi.GPIO_QSPI_SD1_CTRL \ GPIO control including function select and overrides.
  $0020 equ qspi.GPIO_QSPI_SD2_STATUS \ GPIO status
  $0024 equ qspi.GPIO_QSPI_SD2_CTRL \ GPIO control including function select and overrides.
  $0028 equ qspi.GPIO_QSPI_SD3_STATUS \ GPIO status
  $002C equ qspi.GPIO_QSPI_SD3_CTRL \ GPIO control including function select and overrides.
  $0030 equ qspi.INTR            \ Raw Interrupts
  $0034 equ qspi.PROC0_INTE      \ Interrupt Enable for proc0
  $0038 equ qspi.PROC0_INTF      \ Interrupt Force for proc0
  $003C equ qspi.PROC0_INTS      \ Interrupt status after masking & forcing for proc0
  $0040 equ qspi.PROC1_INTE      \ Interrupt Enable for proc1
  $0044 equ qspi.PROC1_INTF      \ Interrupt Force for proc1
  $0048 equ qspi.PROC1_INTS      \ Interrupt status after masking & forcing for proc1
  $004C equ qspi.DORMANT_WAKE_INTE \ Interrupt Enable for dormant_wake
  $0050 equ qspi.DORMANT_WAKE_INTF \ Interrupt Force for dormant_wake
  $0054 equ qspi.DORMANT_WAKE_INTS \ Interrupt status after masking & forcing for dormant_wake


\ ====================================== 
\ PADS_BANK0 - Interrupt status after masking & forcing for dormant_wake
\ ====================================== 

$4001:C000 equ _PADS_BANK0
  $0000 equ pdb0.VOLTAGE_SELECT  \ Voltage select. Per bank control
  $0004 equ pdb0.GPIO0           \ Pad control register
  $0008 equ pdb0.GPIO1           \ Pad control register
  $000C equ pdb0.GPIO2           \ Pad control register
  $0010 equ pdb0.GPIO3           \ Pad control register
  $0014 equ pdb0.GPIO4           \ Pad control register
  $0018 equ pdb0.GPIO5           \ Pad control register
  $001C equ pdb0.GPIO6           \ Pad control register
  $0020 equ pdb0.GPIO7           \ Pad control register
  $0024 equ pdb0.GPIO8           \ Pad control register
  $0028 equ pdb0.GPIO9           \ Pad control register
  $002C equ pdb0.GPIO10          \ Pad control register
  $0030 equ pdb0.GPIO11          \ Pad control register
  $0034 equ pdb0.GPIO12          \ Pad control register
  $0038 equ pdb0.GPIO13          \ Pad control register
  $003C equ pdb0.GPIO14          \ Pad control register
  $0040 equ pdb0.GPIO15          \ Pad control register
  $0044 equ pdb0.GPIO16          \ Pad control register
  $0048 equ pdb0.GPIO17          \ Pad control register
  $004C equ pdb0.GPIO18          \ Pad control register
  $0050 equ pdb0.GPIO19          \ Pad control register
  $0054 equ pdb0.GPIO20          \ Pad control register
  $0058 equ pdb0.GPIO21          \ Pad control register
  $005C equ pdb0.GPIO22          \ Pad control register
  $0060 equ pdb0.GPIO23          \ Pad control register
  $0064 equ pdb0.GPIO24          \ Pad control register
  $0068 equ pdb0.GPIO25          \ Pad control register
  $006C equ pdb0.GPIO26          \ Pad control register
  $0070 equ pdb0.GPIO27          \ Pad control register
  $0074 equ pdb0.GPIO28          \ Pad control register
  $0078 equ pdb0.GPIO29          \ Pad control register
  $007C equ pdb0.SWCLK           \ Pad control register
  $0080 equ pdb0.SWD             \ Pad control register


\ ====================================== 
\ PADS_QSPI - Pad control register
\ ====================================== 

$4002:0000 equ _PADS_QSPI
  $0000 equ pdqspi.VOLTAGE_SELECT \ Voltage select. Per bank control
  $0004 equ pdqspi.GPIO_QSPI_SCLK \ Pad control register
  $0008 equ pdqspi.GPIO_QSPI_SD0 \ Pad control register
  $000C equ pdqspi.GPIO_QSPI_SD1 \ Pad control register
  $0010 equ pdqspi.GPIO_QSPI_SD2 \ Pad control register
  $0014 equ pdqspi.GPIO_QSPI_SD3 \ Pad control register
  $0018 equ pdqspi.GPIO_QSPI_SS  \ Pad control register


\ ====================================== 
\ XOSC - Controls the crystal oscillator
\ ====================================== 

$4002:4000 equ _XOSC
  $0000 equ xosc.CTRL            \ Crystal Oscillator Control
  $0004 equ xosc.STATUS          \ Crystal Oscillator Status
  $0008 equ xosc.DORMANT         \ Crystal Oscillator pause control
                                 \ This is used to save power by pausing the XOSC
                                 \ On power-up this field is initialised to WAKE
                                 \ An invalid write will also select WAKE
                                 \ WARNING: stop the PLLs before selecting dormant mode
                                 \ WARNING: setup the irq before selecting dormant mode
  $000C equ xosc.STARTUP         \ Controls the startup delay
  $001C equ xosc.COUNT           \ A down counter running at the xosc frequency which counts to zero and stops.
                                 \ To start the counter write a non-zero value.
                                 \ Can be used for short software pauses when setting up time sensitive hardware.


\ ====================================== 
\ PLL_SYS - A down counter running at the xosc frequency which counts to zero and stops.
\ To start the counter write a non-zero value.
\ Can be used for short software pauses when setting up time sensitive hardware.
\ ====================================== 

$4002:8000 equ _PLL_SYS
  $0000 equ pll.CS               \ Control and Status
                                 \ GENERAL CONSTRAINTS:
                                 \ Reference clock frequency min=5MHz, max=800MHz
                                 \ Feedback divider min=16, max=320
                                 \ VCO frequency min=750MHz, max=1600MHz
  $0004 equ pll.PWR              \ Controls the PLL power modes.
  $0008 equ pll.FBDIV_INT        \ Feedback divisor
                                 \ (note: this PLL does not support fractional division)
  $000C equ pll.PRIM             \ Controls the PLL post dividers for the primary output
                                 \ (note: this PLL does not have a secondary output)
                                 \ the primary output is driven from VCO divided by postdiv1*postdiv2


\ ====================================== 
\ PLL_USB - Controls the PLL post dividers for the primary output
\ (note: this PLL does not have a secondary output)
\ the primary output is driven from VCO divided by postdiv1*postdiv2
\ ====================================== 

$4002:C000 equ _PLL_USB


\ ====================================== 
\ BUSCTRL - Register block for busfabric control signals and performance counters
\ ====================================== 

$4003:0000 equ _BUSCTRL
  $0000 equ busctl.BUS_PRIORITY  \ Set the priority of each master for bus arbitration.
  $0004 equ busctl.BUS_PRIORITY_ACK \ Bus priority acknowledge
  $0008 equ busctl.PERFCTR0      \ Bus fabric performance counter 0
  $000C equ busctl.PERFSEL0      \ Bus fabric performance event select for PERFCTR0
  $0010 equ busctl.PERFCTR1      \ Bus fabric performance counter 1
  $0014 equ busctl.PERFSEL1      \ Bus fabric performance event select for PERFCTR1
  $0018 equ busctl.PERFCTR2      \ Bus fabric performance counter 2
  $001C equ busctl.PERFSEL2      \ Bus fabric performance event select for PERFCTR2
  $0020 equ busctl.PERFCTR3      \ Bus fabric performance counter 3
  $0024 equ busctl.PERFSEL3      \ Bus fabric performance event select for PERFCTR3


\ ====================================== 
\ UART0 - Bus fabric performance event select for PERFCTR3
\ ====================================== 

$4003:4000 equ _UART0
#36 equ UART0_IRQ_IRQn
  $0000 equ Un.UARTDR            \ Data Register, UARTDR
  $0004 equ Un.UARTRSR           \ Receive Status Register/Error Clear Register, UARTRSR/UARTECR
  $0018 equ Un.UARTFR            \ Flag Register, UARTFR
  $0020 equ Un.UARTILPR          \ IrDA Low-Power Counter Register, UARTILPR
  $0024 equ Un.UARTIBRD          \ Integer Baud Rate Register, UARTIBRD
  $0028 equ Un.UARTFBRD          \ Fractional Baud Rate Register, UARTFBRD
  $002C equ Un.UARTLCR_H         \ Line Control Register, UARTLCR_H
  $0030 equ Un.UARTCR            \ Control Register, UARTCR
  $0034 equ Un.UARTIFLS          \ Interrupt FIFO Level Select Register, UARTIFLS
  $0038 equ Un.UARTIMSC          \ Interrupt Mask Set/Clear Register, UARTIMSC
  $003C equ Un.UARTRIS           \ Raw Interrupt Status Register, UARTRIS
  $0040 equ Un.UARTMIS           \ Masked Interrupt Status Register, UARTMIS
  $0044 equ Un.UARTICR           \ Interrupt Clear Register, UARTICR
  $0048 equ Un.UARTDMACR         \ DMA Control Register, UARTDMACR
  $0FE0 equ Un.UARTPERIPHID0     \ UARTPeriphID0 Register
  $0FE4 equ Un.UARTPERIPHID1     \ UARTPeriphID1 Register
  $0FE8 equ Un.UARTPERIPHID2     \ UARTPeriphID2 Register
  $0FEC equ Un.UARTPERIPHID3     \ UARTPeriphID3 Register
  $0FF0 equ Un.UARTPCELLID0      \ UARTPCellID0 Register
  $0FF4 equ Un.UARTPCELLID1      \ UARTPCellID1 Register
  $0FF8 equ Un.UARTPCELLID2      \ UARTPCellID2 Register
  $0FFC equ Un.UARTPCELLID3      \ UARTPCellID3 Register


\ ====================================== 
\ UART1 - UARTPCellID3 Register
\ ====================================== 

$4003:8000 equ _UART1
#37 equ UART1_IRQ_IRQn


\ ====================================== 
\ SPI0 - UARTPCellID3 Register
\ ====================================== 

$4003:C000 equ _SPI0
#34 equ SPI0_IRQ_IRQn
  $0000 equ spi.SSPCR0           \ Control register 0, SSPCR0 on page 3-4
  $0004 equ spi.SSPCR1           \ Control register 1, SSPCR1 on page 3-5
  $0008 equ spi.SSPDR            \ Data register, SSPDR on page 3-6
  $000C equ spi.SSPSR            \ Status register, SSPSR on page 3-7
  $0010 equ spi.SSPCPSR          \ Clock prescale register, SSPCPSR on page 3-8
  $0014 equ spi.SSPIMSC          \ Interrupt mask set or clear register, SSPIMSC on page 3-9
  $0018 equ spi.SSPRIS           \ Raw interrupt status register, SSPRIS on page 3-10
  $001C equ spi.SSPMIS           \ Masked interrupt status register, SSPMIS on page 3-11
  $0020 equ spi.SSPICR           \ Interrupt clear register, SSPICR on page 3-11
  $0024 equ spi.SSPDMACR         \ DMA control register, SSPDMACR on page 3-12
  $0FE0 equ spi.SSPPERIPHID0     \ Peripheral identification registers, SSPPeriphID0-3 on page 3-13
  $0FE4 equ spi.SSPPERIPHID1     \ Peripheral identification registers, SSPPeriphID0-3 on page 3-13
  $0FE8 equ spi.SSPPERIPHID2     \ Peripheral identification registers, SSPPeriphID0-3 on page 3-13
  $0FEC equ spi.SSPPERIPHID3     \ Peripheral identification registers, SSPPeriphID0-3 on page 3-13
  $0FF0 equ spi.SSPPCELLID0      \ PrimeCell identification registers, SSPPCellID0-3 on page 3-16
  $0FF4 equ spi.SSPPCELLID1      \ PrimeCell identification registers, SSPPCellID0-3 on page 3-16
  $0FF8 equ spi.SSPPCELLID2      \ PrimeCell identification registers, SSPPCellID0-3 on page 3-16
  $0FFC equ spi.SSPPCELLID3      \ PrimeCell identification registers, SSPPCellID0-3 on page 3-16


\ ====================================== 
\ SPI1 - PrimeCell identification registers, SSPPCellID0-3 on page 3-16
\ ====================================== 

$4004:0000 equ _SPI1
#35 equ SPI1_IRQ_IRQn


\ ====================================== 
\ I2C0 - DW_apb_i2c address block
\ 
\ List of configuration constants for the Synopsys I2C hardware (you may see references to these in I2C register header; these are *fixed* values, set at hardware design time):
\ 
\ IC_ULTRA_FAST_MODE ................ 0x0
\ IC_UFM_TBUF_CNT_DEFAULT ........... 0x8
\ IC_UFM_SCL_LOW_COUNT .............. 0x0008
\ IC_UFM_SCL_HIGH_COUNT ............. 0x0006
\ IC_TX_TL .......................... 0x0
\ IC_TX_CMD_BLOCK ................... 0x1
\ IC_HAS_DMA ........................ 0x1
\ IC_HAS_ASYNC_FIFO ................. 0x0
\ IC_SMBUS_ARP ...................... 0x0
\ IC_FIRST_DATA_BYTE_STATUS ......... 0x1
\ IC_INTR_IO ........................ 0x1
\ IC_MASTER_MODE .................... 0x1
\ IC_DEFAULT_ACK_GENERAL_CALL ....... 0x1
\ IC_INTR_POL ....................... 0x1
\ IC_OPTIONAL_SAR ................... 0x0
\ IC_DEFAULT_TAR_SLAVE_ADDR ......... 0x055
\ IC_DEFAULT_SLAVE_ADDR ............. 0x055
\ IC_DEFAULT_HS_SPKLEN .............. 0x1
\ IC_FS_SCL_HIGH_COUNT .............. 0x0006
\ IC_HS_SCL_LOW_COUNT ............... 0x0008
\ IC_DEVICE_ID_VALUE ................ 0x0
\ IC_10BITADDR_MASTER ............... 0x0
\ IC_CLK_FREQ_OPTIMIZATION .......... 0x0
\ IC_DEFAULT_FS_SPKLEN .............. 0x7
\ IC_ADD_ENCODED_PARAMS ............. 0x0
\ IC_DEFAULT_SDA_HOLD ............... 0x000001
\ IC_DEFAULT_SDA_SETUP .............. 0x64
\ IC_AVOID_RX_FIFO_FLUSH_ON_TX_ABRT . 0x0
\ IC_CLOCK_PERIOD ................... 100
\ IC_EMPTYFIFO_HOLD_MASTER_EN ....... 1
\ IC_RESTART_EN ..................... 0x1
\ IC_TX_CMD_BLOCK_DEFAULT ........... 0x0
\ IC_BUS_CLEAR_FEATURE .............. 0x0
\ IC_CAP_LOADING .................... 100
\ IC_FS_SCL_LOW_COUNT ............... 0x000d
\ APB_DATA_WIDTH .................... 32
\ IC_SDA_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff
\ IC_SLV_DATA_NACK_ONLY ............. 0x1
\ IC_10BITADDR_SLAVE ................ 0x0
\ IC_CLK_TYPE ....................... 0x0
\ IC_SMBUS_UDID_MSB ................. 0x0
\ IC_SMBUS_SUSPEND_ALERT ............ 0x0
\ IC_HS_SCL_HIGH_COUNT .............. 0x0006
\ IC_SLV_RESTART_DET_EN ............. 0x1
\ IC_SMBUS .......................... 0x0
\ IC_OPTIONAL_SAR_DEFAULT ........... 0x0
\ IC_PERSISTANT_SLV_ADDR_DEFAULT .... 0x0
\ IC_USE_COUNTS ..................... 0x0
\ IC_RX_BUFFER_DEPTH ................ 16
\ IC_SCL_STUCK_TIMEOUT_DEFAULT ...... 0xffffffff
\ IC_RX_FULL_HLD_BUS_EN ............. 0x1
\ IC_SLAVE_DISABLE .................. 0x1
\ IC_RX_TL .......................... 0x0
\ IC_DEVICE_ID ...................... 0x0
\ IC_HC_COUNT_VALUES ................ 0x0
\ I2C_DYNAMIC_TAR_UPDATE ............ 0
\ IC_SMBUS_CLK_LOW_MEXT_DEFAULT ..... 0xffffffff
\ IC_SMBUS_CLK_LOW_SEXT_DEFAULT ..... 0xffffffff
\ IC_HS_MASTER_CODE ................. 0x1
\ IC_SMBUS_RST_IDLE_CNT_DEFAULT ..... 0xffff
\ IC_SMBUS_UDID_LSB_DEFAULT ......... 0xffffffff
\ IC_SS_SCL_HIGH_COUNT .............. 0x0028
\ IC_SS_SCL_LOW_COUNT ............... 0x002f
\ IC_MAX_SPEED_MODE ................. 0x2
\ IC_STAT_FOR_CLK_STRETCH ........... 0x0
\ IC_STOP_DET_IF_MASTER_ACTIVE ...... 0x0
\ IC_DEFAULT_UFM_SPKLEN ............. 0x1
\ IC_TX_BUFFER_DEPTH ................ 16
\ ====================================== 

$4004:4000 equ _I2C0
#39 equ I2C0_IRQ_IRQn
  $0000 equ i2c.IC_CON           \ I2C Control Register. This register can be written only when the DW_apb_i2c is disabled, which corresponds to the IC_ENABLE[0] register being set to 0. Writes at other times have no effect.
                                 \ 
                                 \ Read/Write Access: - bit 10 is read only. - bit 11 is read only - bit 16 is read only - bit 17 is read only - bits 18 and 19 are read only.
  $0004 equ i2c.IC_TAR           \ I2C Target Address Register
                                 \ 
                                 \ This register is 12 bits wide, and bits 31:12 are reserved. This register can be written to only when IC_ENABLE[0] is set to 0.
                                 \ 
                                 \ Note: If the software or application is aware that the DW_apb_i2c is not using the TAR address for the pending commands in the Tx FIFO, then it is possible to update the TAR address even while the Tx FIFO has entries (IC_STATUS[2]= 0). - It is not necessary to perform any write to this register if DW_apb_i2c is enabled as an I2C slave only.
  $0008 equ i2c.IC_SAR           \ I2C Slave Address Register
  $0010 equ i2c.IC_DATA_CMD      \ I2C Rx/Tx Data Buffer and Command Register; this is the register the CPU writes to when filling the TX FIFO and the CPU reads from when retrieving bytes from RX FIFO.
                                 \ 
                                 \ The size of the register changes as follows:
                                 \ 
                                 \ Write: - 11 bits when IC_EMPTYFIFO_HOLD_MASTER_EN=1 - 9 bits when IC_EMPTYFIFO_HOLD_MASTER_EN=0 Read: - 12 bits when IC_FIRST_DATA_BYTE_STATUS = 1 - 8 bits when IC_FIRST_DATA_BYTE_STATUS = 0 Note: In order for the DW_apb_i2c to continue acknowledging reads, a read command should be written for every byte that is to be received; otherwise the DW_apb_i2c will stop acknowledging.
  $0014 equ i2c.IC_SS_SCL_HCNT   \ Standard Speed I2C Clock SCL High Count Register
  $0018 equ i2c.IC_SS_SCL_LCNT   \ Standard Speed I2C Clock SCL Low Count Register
  $001C equ i2c.IC_FS_SCL_HCNT   \ Fast Mode or Fast Mode Plus I2C Clock SCL High Count Register
  $0020 equ i2c.IC_FS_SCL_LCNT   \ Fast Mode or Fast Mode Plus I2C Clock SCL Low Count Register
  $002C equ i2c.IC_INTR_STAT     \ I2C Interrupt Status Register
                                 \ 
                                 \ Each bit in this register has a corresponding mask bit in the IC_INTR_MASK register. These bits are cleared by reading the matching interrupt clear register. The unmasked raw versions of these bits are available in the IC_RAW_INTR_STAT register.
  $0030 equ i2c.IC_INTR_MASK     \ I2C Interrupt Mask Register.
                                 \ 
                                 \ These bits mask their corresponding interrupt status bits. This register is active low; a value of 0 masks the interrupt, whereas a value of 1 unmasks the interrupt.
  $0034 equ i2c.IC_RAW_INTR_STAT \ I2C Raw Interrupt Status Register
                                 \ 
                                 \ Unlike the IC_INTR_STAT register, these bits are not masked so they always show the true status of the DW_apb_i2c.
  $0038 equ i2c.IC_RX_TL         \ I2C Receive FIFO Threshold Register
  $003C equ i2c.IC_TX_TL         \ I2C Transmit FIFO Threshold Register
  $0040 equ i2c.IC_CLR_INTR      \ Clear Combined and Individual Interrupt Register
  $0044 equ i2c.IC_CLR_RX_UNDER  \ Clear RX_UNDER Interrupt Register
  $0048 equ i2c.IC_CLR_RX_OVER   \ Clear RX_OVER Interrupt Register
  $004C equ i2c.IC_CLR_TX_OVER   \ Clear TX_OVER Interrupt Register
  $0050 equ i2c.IC_CLR_RD_REQ    \ Clear RD_REQ Interrupt Register
  $0054 equ i2c.IC_CLR_TX_ABRT   \ Clear TX_ABRT Interrupt Register
  $0058 equ i2c.IC_CLR_RX_DONE   \ Clear RX_DONE Interrupt Register
  $005C equ i2c.IC_CLR_ACTIVITY  \ Clear ACTIVITY Interrupt Register
  $0060 equ i2c.IC_CLR_STOP_DET  \ Clear STOP_DET Interrupt Register
  $0064 equ i2c.IC_CLR_START_DET \ Clear START_DET Interrupt Register
  $0068 equ i2c.IC_CLR_GEN_CALL  \ Clear GEN_CALL Interrupt Register
  $006C equ i2c.IC_ENABLE        \ I2C Enable Register
  $0070 equ i2c.IC_STATUS        \ I2C Status Register
                                 \ 
                                 \ This is a read-only register used to indicate the current transfer status and FIFO status. The status register may be read at any time. None of the bits in this register request an interrupt.
                                 \ 
                                 \ When the I2C is disabled by writing 0 in bit 0 of the IC_ENABLE register: - Bits 1 and 2 are set to 1 - Bits 3 and 10 are set to 0 When the master or slave state machines goes to idle and ic_en=0: - Bits 5 and 6 are set to 0
  $0074 equ i2c.IC_TXFLR         \ I2C Transmit FIFO Level Register This register contains the number of valid data entries in the transmit FIFO buffer. It is cleared whenever: - The I2C is disabled - There is a transmit abort - that is, TX_ABRT bit is set in the IC_RAW_INTR_STAT register - The slave bulk transmit mode is aborted The register increments whenever data is placed into the transmit FIFO and decrements when data is taken from the transmit FIFO.
  $0078 equ i2c.IC_RXFLR         \ I2C Receive FIFO Level Register This register contains the number of valid data entries in the receive FIFO buffer. It is cleared whenever: - The I2C is disabled - Whenever there is a transmit abort caused by any of the events tracked in IC_TX_ABRT_SOURCE The register increments whenever data is placed into the receive FIFO and decrements when data is taken from the receive FIFO.
  $007C equ i2c.IC_SDA_HOLD      \ I2C SDA Hold Time Length Register
                                 \ 
                                 \ The bits [15:0] of this register are used to control the hold time of SDA during transmit in both slave and master mode (after SCL goes from HIGH to LOW).
                                 \ 
                                 \ The bits [23:16] of this register are used to extend the SDA transition (if any) whenever SCL is HIGH in the receiver in either master or slave mode.
                                 \ 
                                 \ Writes to this register succeed only when IC_ENABLE[0]=0.
                                 \ 
                                 \ The values in this register are in units of ic_clk period. The value programmed in IC_SDA_TX_HOLD must be greater than the minimum hold time in each mode (one cycle in master mode, seven cycles in slave mode) for the value to be implemented.
                                 \ 
                                 \ The programmed SDA hold time during transmit (IC_SDA_TX_HOLD) cannot exceed at any time the duration of the low part of scl. Therefore the programmed value cannot be larger than N_SCL_LOW-2, where N_SCL_LOW is the duration of the low part of the scl period measured in ic_clk cycles.
  $0080 equ i2c.IC_TX_ABRT_SOURCE \ I2C Transmit Abort Source Register
                                 \ 
                                 \ This register has 32 bits that indicate the source of the TX_ABRT bit. Except for Bit 9, this register is cleared whenever the IC_CLR_TX_ABRT register or the IC_CLR_INTR register is read. To clear Bit 9, the source of the ABRT_SBYTE_NORSTRT must be fixed first; RESTART must be enabled (IC_CON[5]=1), the SPECIAL bit must be cleared (IC_TAR[11]), or the GC_OR_START bit must be cleared (IC_TAR[10]).
                                 \ 
                                 \ Once the source of the ABRT_SBYTE_NORSTRT is fixed, then this bit can be cleared in the same manner as other bits in this register. If the source of the ABRT_SBYTE_NORSTRT is not fixed before attempting to clear this bit, Bit 9 clears for one cycle and is then re-asserted.
  $0084 equ i2c.IC_SLV_DATA_NACK_ONLY \ Generate Slave Data NACK Register
                                 \ 
                                 \ The register is used to generate a NACK for the data part of a transfer when DW_apb_i2c is acting as a slave-receiver. This register only exists when the IC_SLV_DATA_NACK_ONLY parameter is set to 1. When this parameter disabled, this register does not exist and writing to the register's address has no effect.
                                 \ 
                                 \ A write can occur on this register if both of the following conditions are met: - DW_apb_i2c is disabled (IC_ENABLE[0] = 0) - Slave part is inactive (IC_STATUS[6] = 0) Note: The IC_STATUS[6] is a register read-back location for the internal slv_activity signal; the user should poll this before writing the ic_slv_data_nack_only bit.
  $0088 equ i2c.IC_DMA_CR        \ DMA Control Register
                                 \ 
                                 \ The register is used to enable the DMA Controller interface operation. There is a separate bit for transmit and receive. This can be programmed regardless of the state of IC_ENABLE.
  $008C equ i2c.IC_DMA_TDLR      \ DMA Transmit Data Level Register
  $0090 equ i2c.IC_DMA_RDLR      \ I2C Receive Data Level Register
  $0094 equ i2c.IC_SDA_SETUP     \ I2C SDA Setup Register
                                 \ 
                                 \ This register controls the amount of time delay (in terms of number of ic_clk clock periods) introduced in the rising edge of SCL - relative to SDA changing - when DW_apb_i2c services a read request in a slave-transmitter operation. The relevant I2C requirement is tSU:DAT (note 4) as detailed in the I2C Bus Specification. This register must be programmed with a value equal to or greater than 2.
                                 \ 
                                 \ Writes to this register succeed only when IC_ENABLE[0] = 0.
                                 \ 
                                 \ Note: The length of setup time is calculated using [(IC_SDA_SETUP - 1) * (ic_clk_period)], so if the user requires 10 ic_clk periods of setup time, they should program a value of 11. The IC_SDA_SETUP register is only used by the DW_apb_i2c when operating as a slave transmitter.
  $0098 equ i2c.IC_ACK_GENERAL_CALL \ I2C ACK General Call Register
                                 \ 
                                 \ The register controls whether DW_apb_i2c responds with a ACK or NACK when it receives an I2C General Call address.
                                 \ 
                                 \ This register is applicable only when the DW_apb_i2c is in slave mode.
  $009C equ i2c.IC_ENABLE_STATUS \ I2C Enable Status Register
                                 \ 
                                 \ The register is used to report the DW_apb_i2c hardware status when the IC_ENABLE[0] register is set from 1 to 0; that is, when DW_apb_i2c is disabled.
                                 \ 
                                 \ If IC_ENABLE[0] has been set to 1, bits 2:1 are forced to 0, and bit 0 is forced to 1.
                                 \ 
                                 \ If IC_ENABLE[0] has been set to 0, bits 2:1 is only be valid as soon as bit 0 is read as '0'.
                                 \ 
                                 \ Note: When IC_ENABLE[0] has been set to 0, a delay occurs for bit 0 to be read as 0 because disabling the DW_apb_i2c depends on I2C bus activities.
  $00A0 equ i2c.IC_FS_SPKLEN     \ I2C SS, FS or FM+ spike suppression limit
                                 \ 
                                 \ This register is used to store the duration, measured in ic_clk cycles, of the longest spike that is filtered out by the spike suppression logic when the component is operating in SS, FS or FM+ modes. The relevant I2C requirement is tSP (table 4) as detailed in the I2C Bus Specification. This register must be programmed with a minimum value of 1.
  $00A8 equ i2c.IC_CLR_RESTART_DET \ Clear RESTART_DET Interrupt Register
  $00F4 equ i2c.IC_COMP_PARAM_1  \ Component Parameter Register 1
                                 \ 
                                 \ Note This register is not implemented and therefore reads as 0. If it was implemented it would be a constant read-only register that contains encoded information about the component's parameter settings. Fields shown below are the settings for those parameters
  $00F8 equ i2c.IC_COMP_VERSION  \ I2C Component Version Register
  $00FC equ i2c.IC_COMP_TYPE     \ I2C Component Type Register


\ ====================================== 
\ I2C1 - I2C Component Type Register
\ ====================================== 

$4004:8000 equ _I2C1
#40 equ I2C1_IRQ_IRQn


\ ====================================== 
\ ADC - Control and data interface to SAR ADC
\ ====================================== 

$4004:C000 equ _ADC
#38 equ ADC_IRQ_FIFO_IRQn
  $0000 equ adc.CS               \ ADC Control and Status
  $0004 equ adc.RESULT           \ Result of most recent ADC conversion
  $0008 equ adc.FCS              \ FIFO control and status
  $000C equ adc.FIFO             \ Conversion result FIFO
  $0010 equ adc.DIV              \ Clock divider. If non-zero, CS_START_MANY will start conversions
                                 \ at regular intervals rather than back-to-back.
                                 \ The divider is reset when either of these fields are written.
                                 \ Total period is 1 + INT + FRAC / 256
  $0014 equ adc.INTR             \ Raw Interrupts
  $0018 equ adc.INTE             \ Interrupt Enable
  $001C equ adc.INTF             \ Interrupt Force
  $0020 equ adc.INTS             \ Interrupt status after masking & forcing


\ ====================================== 
\ PWM - Simple PWM
\ ====================================== 

$4005:0000 equ _PWM
#20 equ PWM_IRQ_WRAP_IRQn
  $0000 equ pwm.CH0_CSR          \ Control and status register
  $0004 equ pwm.CH0_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $0008 equ pwm.CH0_CTR          \ Direct access to the PWM counter
  $000C equ pwm.CH0_CC           \ Counter compare values
  $0010 equ pwm.CH0_TOP          \ Counter wrap value
  $0014 equ pwm.CH1_CSR          \ Control and status register
  $0018 equ pwm.CH1_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $001C equ pwm.CH1_CTR          \ Direct access to the PWM counter
  $0020 equ pwm.CH1_CC           \ Counter compare values
  $0024 equ pwm.CH1_TOP          \ Counter wrap value
  $0028 equ pwm.CH2_CSR          \ Control and status register
  $002C equ pwm.CH2_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $0030 equ pwm.CH2_CTR          \ Direct access to the PWM counter
  $0034 equ pwm.CH2_CC           \ Counter compare values
  $0038 equ pwm.CH2_TOP          \ Counter wrap value
  $003C equ pwm.CH3_CSR          \ Control and status register
  $0040 equ pwm.CH3_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $0044 equ pwm.CH3_CTR          \ Direct access to the PWM counter
  $0048 equ pwm.CH3_CC           \ Counter compare values
  $004C equ pwm.CH3_TOP          \ Counter wrap value
  $0050 equ pwm.CH4_CSR          \ Control and status register
  $0054 equ pwm.CH4_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $0058 equ pwm.CH4_CTR          \ Direct access to the PWM counter
  $005C equ pwm.CH4_CC           \ Counter compare values
  $0060 equ pwm.CH4_TOP          \ Counter wrap value
  $0064 equ pwm.CH5_CSR          \ Control and status register
  $0068 equ pwm.CH5_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $006C equ pwm.CH5_CTR          \ Direct access to the PWM counter
  $0070 equ pwm.CH5_CC           \ Counter compare values
  $0074 equ pwm.CH5_TOP          \ Counter wrap value
  $0078 equ pwm.CH6_CSR          \ Control and status register
  $007C equ pwm.CH6_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $0080 equ pwm.CH6_CTR          \ Direct access to the PWM counter
  $0084 equ pwm.CH6_CC           \ Counter compare values
  $0088 equ pwm.CH6_TOP          \ Counter wrap value
  $008C equ pwm.CH7_CSR          \ Control and status register
  $0090 equ pwm.CH7_DIV          \ INT and FRAC form a fixed-point fractional number.
                                 \ Counting rate is system clock frequency divided by this number.
                                 \ Fractional division uses simple 1st-order sigma-delta.
  $0094 equ pwm.CH7_CTR          \ Direct access to the PWM counter
  $0098 equ pwm.CH7_CC           \ Counter compare values
  $009C equ pwm.CH7_TOP          \ Counter wrap value
  $00A0 equ pwm.EN               \ This register aliases the CSR_EN bits for all channels.
                                 \ Writing to this register allows multiple channels to be enabled
                                 \ or disabled simultaneously, so they can run in perfect sync.
                                 \ For each channel, there is only one physical EN register bit,
                                 \ which can be accessed through here or CHx_CSR.
  $00A4 equ pwm.INTR             \ Raw Interrupts
  $00A8 equ pwm.INTE             \ Interrupt Enable
  $00AC equ pwm.INTF             \ Interrupt Force
  $00B0 equ pwm.INTS             \ Interrupt status after masking & forcing


\ ====================================== 
\ TIMER - Controls time and alarms
\ time is a 64 bit value indicating the time in usec since power-on
\ timeh is the top 32 bits of time & timel is the bottom 32 bits
\ to change time write to timelw before timehw
\ to read time read from timelr before timehr
\ An alarm is set by setting alarm_enable and writing to the corresponding alarm register
\ When an alarm is pending, the corresponding alarm_running signal will be high
\ An alarm can be cancelled before it has finished by clearing the alarm_enable
\ When an alarm fires, the corresponding alarm_irq is set and alarm_running is cleared
\ To clear the interrupt write a 1 to the corresponding alarm_irq
\ ====================================== 

$4005:4000 equ _TIMER
#16 equ TIMER_IRQ_0_IRQn
#17 equ TIMER_IRQ_1_IRQn
#18 equ TIMER_IRQ_2_IRQn
#19 equ TIMER_IRQ_3_IRQn
  $0000 equ tim.TIMEHW           \ Write to bits 63:32 of time
                                 \ always write timelw before timehw
  $0004 equ tim.TIMELW           \ Write to bits 31:0 of time
                                 \ writes do not get copied to time until timehw is written
  $0008 equ tim.TIMEHR           \ Read from bits 63:32 of time
                                 \ always read timelr before timehr
  $000C equ tim.TIMELR           \ Read from bits 31:0 of time
  $0010 equ tim.ALARM0           \ Arm alarm 0, and configure the time it will fire.
                                 \ Once armed, the alarm fires when TIMER_ALARM0 == TIMELR.
                                 \ The alarm will disarm itself once it fires, and can
                                 \ be disarmed early using the ARMED status register.
  $0014 equ tim.ALARM1           \ Arm alarm 1, and configure the time it will fire.
                                 \ Once armed, the alarm fires when TIMER_ALARM1 == TIMELR.
                                 \ The alarm will disarm itself once it fires, and can
                                 \ be disarmed early using the ARMED status register.
  $0018 equ tim.ALARM2           \ Arm alarm 2, and configure the time it will fire.
                                 \ Once armed, the alarm fires when TIMER_ALARM2 == TIMELR.
                                 \ The alarm will disarm itself once it fires, and can
                                 \ be disarmed early using the ARMED status register.
  $001C equ tim.ALARM3           \ Arm alarm 3, and configure the time it will fire.
                                 \ Once armed, the alarm fires when TIMER_ALARM3 == TIMELR.
                                 \ The alarm will disarm itself once it fires, and can
                                 \ be disarmed early using the ARMED status register.
  $0020 equ tim.ARMED            \ Indicates the armed/disarmed status of each alarm.
                                 \ A write to the corresponding ALARMx register arms the alarm.
                                 \ Alarms automatically disarm upon firing, but writing ones here
                                 \ will disarm immediately without waiting to fire.
  $0024 equ tim.TIMERAWH         \ Raw read from bits 63:32 of time (no side effects)
  $0028 equ tim.TIMERAWL         \ Raw read from bits 31:0 of time (no side effects)
  $002C equ tim.DBGPAUSE         \ Set bits high to enable pause when the corresponding debug ports are active
  $0030 equ tim.PAUSE            \ Set high to pause the timer
  $0034 equ tim.INTR             \ Raw Interrupts
  $0038 equ tim.INTE             \ Interrupt Enable
  $003C equ tim.INTF             \ Interrupt Force
  $0040 equ tim.INTS             \ Interrupt status after masking & forcing


\ ====================================== 
\ WATCHDOG - Interrupt status after masking & forcing
\ ====================================== 

$4005:8000 equ _WATCHDOG
  $0000 equ wdg.CTRL             \ Watchdog control
                                 \ The rst_wdsel register determines which subsystems are reset when the watchdog is triggered.
                                 \ The watchdog can be triggered in software.
  $0004 equ wdg.LOAD             \ Load the watchdog timer. The maximum setting is 0xffffff which corresponds to 0xffffff / 2 ticks before triggering a watchdog reset (see errata RP2040-E1).
  $0008 equ wdg.REASON           \ Logs the reason for the last reset. Both bits are zero for the case of a hardware reset.
  $000C equ wdg.SCRATCH0         \ Scratch register. Information persists through soft reset of the chip.
  $0010 equ wdg.SCRATCH1         \ Scratch register. Information persists through soft reset of the chip.
  $0014 equ wdg.SCRATCH2         \ Scratch register. Information persists through soft reset of the chip.
  $0018 equ wdg.SCRATCH3         \ Scratch register. Information persists through soft reset of the chip.
  $001C equ wdg.SCRATCH4         \ Scratch register. Information persists through soft reset of the chip.
  $0020 equ wdg.SCRATCH5         \ Scratch register. Information persists through soft reset of the chip.
  $0024 equ wdg.SCRATCH6         \ Scratch register. Information persists through soft reset of the chip.
  $0028 equ wdg.SCRATCH7         \ Scratch register. Information persists through soft reset of the chip.
  $002C equ wdg.TICK             \ Controls the tick generator


\ ====================================== 
\ RTC - Register block to control RTC
\ ====================================== 

$4005:C000 equ _RTC
#41 equ RTC_IRQ_IRQn
  $0000 equ rtc.CLKDIV_M1        \ Divider minus 1 for the 1 second counter. Safe to change the value when RTC is not enabled.
  $0004 equ rtc.SETUP_0          \ RTC setup register 0
  $0008 equ rtc.SETUP_1          \ RTC setup register 1
  $000C equ rtc.CTRL             \ RTC Control and status
  $0010 equ rtc.IRQ_SETUP_0      \ Interrupt setup register 0
  $0014 equ rtc.IRQ_SETUP_1      \ Interrupt setup register 1
  $0018 equ rtc.RTC_1            \ RTC register 1.
  $001C equ rtc.RTC_0            \ RTC register 0
                                 \ Read this before RTC 1!
  $0020 equ rtc.INTR             \ Raw Interrupts
  $0024 equ rtc.INTE             \ Interrupt Enable
  $0028 equ rtc.INTF             \ Interrupt Force
  $002C equ rtc.INTS             \ Interrupt status after masking & forcing


\ ====================================== 
\ ROSC - Interrupt status after masking & forcing
\ ====================================== 

$4006:0000 equ _ROSC
  $0000 equ rosc.CTRL            \ Ring Oscillator control
  $0004 equ rosc.FREQA           \ The FREQA & FREQB registers control the frequency by controlling the drive strength of each stage
                                 \ The drive strength has 4 levels determined by the number of bits set
                                 \ Increasing the number of bits set increases the drive strength and increases the oscillation frequency
                                 \ 0 bits set is the default drive strength
                                 \ 1 bit set doubles the drive strength
                                 \ 2 bits set triples drive strength
                                 \ 3 bits set quadruples drive strength
  $0008 equ rosc.FREQB           \ For a detailed description see freqa register
  $000C equ rosc.DORMANT         \ Ring Oscillator pause control
                                 \ This is used to save power by pausing the ROSC
                                 \ On power-up this field is initialised to WAKE
                                 \ An invalid write will also select WAKE
                                 \ Warning: setup the irq before selecting dormant mode
  $0010 equ rosc.DIV             \ Controls the output divider
  $0014 equ rosc.PHASE           \ Controls the phase shifted output
  $0018 equ rosc.STATUS          \ Ring Oscillator Status
  $001C equ rosc.RANDOMBIT       \ This just reads the state of the oscillator output so randomness is compromised if the ring oscillator is stopped or run at a harmonic of the bus frequency
  $0020 equ rosc.COUNT           \ A down counter running at the ROSC frequency which counts to zero and stops.
                                 \ To start the counter write a non-zero value.
                                 \ Can be used for short software pauses when setting up time sensitive hardware.


\ ====================================== 
\ VREG_AND_CHIP_RESET - control and status for on-chip voltage regulator and chip level reset subsystem
\ ====================================== 

$4006:4000 equ _VREG_AND_CHIP_RESET
  $0000 equ vrcr.VREG            \ Voltage regulator control and status
  $0004 equ vrcr.BOD             \ brown-out detection control
  $0008 equ vrcr.CHIP_RESET      \ Chip reset control and status


\ ====================================== 
\ TBMAN - Testbench manager. Allows the programmer to know what platform their software is running on.
\ ====================================== 

$4006:C000 equ _TBMAN
  $0000 equ tbm.PLATFORM         \ Indicates the type of platform in use


\ ====================================== 
\ DMA - DMA with separate read and write masters
\ ====================================== 

$5000:0000 equ _DMA
#27 equ DMA_IRQ_0_IRQn
#28 equ DMA_IRQ_1_IRQn
  $0000 equ dma.CH0_READ_ADDR    \ DMA Channel 0 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0004 equ dma.CH0_WRITE_ADDR   \ DMA Channel 0 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0008 equ dma.CH0_TRANS_COUNT  \ DMA Channel 0 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $000C equ dma.CH0_CTRL_TRIG    \ DMA Channel 0 Control and Status
  $0010 equ dma.CH0_AL1_CTRL     \ Alias for channel 0 CTRL register
  $0014 equ dma.CH0_AL1_READ_ADDR \ Alias for channel 0 READ_ADDR register
  $0018 equ dma.CH0_AL1_WRITE_ADDR \ Alias for channel 0 WRITE_ADDR register
  $001C equ dma.CH0_AL1_TRANS_COUNT_TRIG \ Alias for channel 0 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0020 equ dma.CH0_AL2_CTRL     \ Alias for channel 0 CTRL register
  $0024 equ dma.CH0_AL2_TRANS_COUNT \ Alias for channel 0 TRANS_COUNT register
  $0028 equ dma.CH0_AL2_READ_ADDR \ Alias for channel 0 READ_ADDR register
  $002C equ dma.CH0_AL2_WRITE_ADDR_TRIG \ Alias for channel 0 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0030 equ dma.CH0_AL3_CTRL     \ Alias for channel 0 CTRL register
  $0034 equ dma.CH0_AL3_WRITE_ADDR \ Alias for channel 0 WRITE_ADDR register
  $0038 equ dma.CH0_AL3_TRANS_COUNT \ Alias for channel 0 TRANS_COUNT register
  $003C equ dma.CH0_AL3_READ_ADDR_TRIG \ Alias for channel 0 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0040 equ dma.CH1_READ_ADDR    \ DMA Channel 1 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0044 equ dma.CH1_WRITE_ADDR   \ DMA Channel 1 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0048 equ dma.CH1_TRANS_COUNT  \ DMA Channel 1 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $004C equ dma.CH1_CTRL_TRIG    \ DMA Channel 1 Control and Status
  $0050 equ dma.CH1_AL1_CTRL     \ Alias for channel 1 CTRL register
  $0054 equ dma.CH1_AL1_READ_ADDR \ Alias for channel 1 READ_ADDR register
  $0058 equ dma.CH1_AL1_WRITE_ADDR \ Alias for channel 1 WRITE_ADDR register
  $005C equ dma.CH1_AL1_TRANS_COUNT_TRIG \ Alias for channel 1 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0060 equ dma.CH1_AL2_CTRL     \ Alias for channel 1 CTRL register
  $0064 equ dma.CH1_AL2_TRANS_COUNT \ Alias for channel 1 TRANS_COUNT register
  $0068 equ dma.CH1_AL2_READ_ADDR \ Alias for channel 1 READ_ADDR register
  $006C equ dma.CH1_AL2_WRITE_ADDR_TRIG \ Alias for channel 1 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0070 equ dma.CH1_AL3_CTRL     \ Alias for channel 1 CTRL register
  $0074 equ dma.CH1_AL3_WRITE_ADDR \ Alias for channel 1 WRITE_ADDR register
  $0078 equ dma.CH1_AL3_TRANS_COUNT \ Alias for channel 1 TRANS_COUNT register
  $007C equ dma.CH1_AL3_READ_ADDR_TRIG \ Alias for channel 1 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0080 equ dma.CH2_READ_ADDR    \ DMA Channel 2 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0084 equ dma.CH2_WRITE_ADDR   \ DMA Channel 2 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0088 equ dma.CH2_TRANS_COUNT  \ DMA Channel 2 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $008C equ dma.CH2_CTRL_TRIG    \ DMA Channel 2 Control and Status
  $0090 equ dma.CH2_AL1_CTRL     \ Alias for channel 2 CTRL register
  $0094 equ dma.CH2_AL1_READ_ADDR \ Alias for channel 2 READ_ADDR register
  $0098 equ dma.CH2_AL1_WRITE_ADDR \ Alias for channel 2 WRITE_ADDR register
  $009C equ dma.CH2_AL1_TRANS_COUNT_TRIG \ Alias for channel 2 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $00A0 equ dma.CH2_AL2_CTRL     \ Alias for channel 2 CTRL register
  $00A4 equ dma.CH2_AL2_TRANS_COUNT \ Alias for channel 2 TRANS_COUNT register
  $00A8 equ dma.CH2_AL2_READ_ADDR \ Alias for channel 2 READ_ADDR register
  $00AC equ dma.CH2_AL2_WRITE_ADDR_TRIG \ Alias for channel 2 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $00B0 equ dma.CH2_AL3_CTRL     \ Alias for channel 2 CTRL register
  $00B4 equ dma.CH2_AL3_WRITE_ADDR \ Alias for channel 2 WRITE_ADDR register
  $00B8 equ dma.CH2_AL3_TRANS_COUNT \ Alias for channel 2 TRANS_COUNT register
  $00BC equ dma.CH2_AL3_READ_ADDR_TRIG \ Alias for channel 2 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $00C0 equ dma.CH3_READ_ADDR    \ DMA Channel 3 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $00C4 equ dma.CH3_WRITE_ADDR   \ DMA Channel 3 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $00C8 equ dma.CH3_TRANS_COUNT  \ DMA Channel 3 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $00CC equ dma.CH3_CTRL_TRIG    \ DMA Channel 3 Control and Status
  $00D0 equ dma.CH3_AL1_CTRL     \ Alias for channel 3 CTRL register
  $00D4 equ dma.CH3_AL1_READ_ADDR \ Alias for channel 3 READ_ADDR register
  $00D8 equ dma.CH3_AL1_WRITE_ADDR \ Alias for channel 3 WRITE_ADDR register
  $00DC equ dma.CH3_AL1_TRANS_COUNT_TRIG \ Alias for channel 3 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $00E0 equ dma.CH3_AL2_CTRL     \ Alias for channel 3 CTRL register
  $00E4 equ dma.CH3_AL2_TRANS_COUNT \ Alias for channel 3 TRANS_COUNT register
  $00E8 equ dma.CH3_AL2_READ_ADDR \ Alias for channel 3 READ_ADDR register
  $00EC equ dma.CH3_AL2_WRITE_ADDR_TRIG \ Alias for channel 3 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $00F0 equ dma.CH3_AL3_CTRL     \ Alias for channel 3 CTRL register
  $00F4 equ dma.CH3_AL3_WRITE_ADDR \ Alias for channel 3 WRITE_ADDR register
  $00F8 equ dma.CH3_AL3_TRANS_COUNT \ Alias for channel 3 TRANS_COUNT register
  $00FC equ dma.CH3_AL3_READ_ADDR_TRIG \ Alias for channel 3 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0100 equ dma.CH4_READ_ADDR    \ DMA Channel 4 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0104 equ dma.CH4_WRITE_ADDR   \ DMA Channel 4 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0108 equ dma.CH4_TRANS_COUNT  \ DMA Channel 4 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $010C equ dma.CH4_CTRL_TRIG    \ DMA Channel 4 Control and Status
  $0110 equ dma.CH4_AL1_CTRL     \ Alias for channel 4 CTRL register
  $0114 equ dma.CH4_AL1_READ_ADDR \ Alias for channel 4 READ_ADDR register
  $0118 equ dma.CH4_AL1_WRITE_ADDR \ Alias for channel 4 WRITE_ADDR register
  $011C equ dma.CH4_AL1_TRANS_COUNT_TRIG \ Alias for channel 4 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0120 equ dma.CH4_AL2_CTRL     \ Alias for channel 4 CTRL register
  $0124 equ dma.CH4_AL2_TRANS_COUNT \ Alias for channel 4 TRANS_COUNT register
  $0128 equ dma.CH4_AL2_READ_ADDR \ Alias for channel 4 READ_ADDR register
  $012C equ dma.CH4_AL2_WRITE_ADDR_TRIG \ Alias for channel 4 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0130 equ dma.CH4_AL3_CTRL     \ Alias for channel 4 CTRL register
  $0134 equ dma.CH4_AL3_WRITE_ADDR \ Alias for channel 4 WRITE_ADDR register
  $0138 equ dma.CH4_AL3_TRANS_COUNT \ Alias for channel 4 TRANS_COUNT register
  $013C equ dma.CH4_AL3_READ_ADDR_TRIG \ Alias for channel 4 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0140 equ dma.CH5_READ_ADDR    \ DMA Channel 5 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0144 equ dma.CH5_WRITE_ADDR   \ DMA Channel 5 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0148 equ dma.CH5_TRANS_COUNT  \ DMA Channel 5 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $014C equ dma.CH5_CTRL_TRIG    \ DMA Channel 5 Control and Status
  $0150 equ dma.CH5_AL1_CTRL     \ Alias for channel 5 CTRL register
  $0154 equ dma.CH5_AL1_READ_ADDR \ Alias for channel 5 READ_ADDR register
  $0158 equ dma.CH5_AL1_WRITE_ADDR \ Alias for channel 5 WRITE_ADDR register
  $015C equ dma.CH5_AL1_TRANS_COUNT_TRIG \ Alias for channel 5 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0160 equ dma.CH5_AL2_CTRL     \ Alias for channel 5 CTRL register
  $0164 equ dma.CH5_AL2_TRANS_COUNT \ Alias for channel 5 TRANS_COUNT register
  $0168 equ dma.CH5_AL2_READ_ADDR \ Alias for channel 5 READ_ADDR register
  $016C equ dma.CH5_AL2_WRITE_ADDR_TRIG \ Alias for channel 5 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0170 equ dma.CH5_AL3_CTRL     \ Alias for channel 5 CTRL register
  $0174 equ dma.CH5_AL3_WRITE_ADDR \ Alias for channel 5 WRITE_ADDR register
  $0178 equ dma.CH5_AL3_TRANS_COUNT \ Alias for channel 5 TRANS_COUNT register
  $017C equ dma.CH5_AL3_READ_ADDR_TRIG \ Alias for channel 5 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0180 equ dma.CH6_READ_ADDR    \ DMA Channel 6 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0184 equ dma.CH6_WRITE_ADDR   \ DMA Channel 6 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0188 equ dma.CH6_TRANS_COUNT  \ DMA Channel 6 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $018C equ dma.CH6_CTRL_TRIG    \ DMA Channel 6 Control and Status
  $0190 equ dma.CH6_AL1_CTRL     \ Alias for channel 6 CTRL register
  $0194 equ dma.CH6_AL1_READ_ADDR \ Alias for channel 6 READ_ADDR register
  $0198 equ dma.CH6_AL1_WRITE_ADDR \ Alias for channel 6 WRITE_ADDR register
  $019C equ dma.CH6_AL1_TRANS_COUNT_TRIG \ Alias for channel 6 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $01A0 equ dma.CH6_AL2_CTRL     \ Alias for channel 6 CTRL register
  $01A4 equ dma.CH6_AL2_TRANS_COUNT \ Alias for channel 6 TRANS_COUNT register
  $01A8 equ dma.CH6_AL2_READ_ADDR \ Alias for channel 6 READ_ADDR register
  $01AC equ dma.CH6_AL2_WRITE_ADDR_TRIG \ Alias for channel 6 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $01B0 equ dma.CH6_AL3_CTRL     \ Alias for channel 6 CTRL register
  $01B4 equ dma.CH6_AL3_WRITE_ADDR \ Alias for channel 6 WRITE_ADDR register
  $01B8 equ dma.CH6_AL3_TRANS_COUNT \ Alias for channel 6 TRANS_COUNT register
  $01BC equ dma.CH6_AL3_READ_ADDR_TRIG \ Alias for channel 6 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $01C0 equ dma.CH7_READ_ADDR    \ DMA Channel 7 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $01C4 equ dma.CH7_WRITE_ADDR   \ DMA Channel 7 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $01C8 equ dma.CH7_TRANS_COUNT  \ DMA Channel 7 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $01CC equ dma.CH7_CTRL_TRIG    \ DMA Channel 7 Control and Status
  $01D0 equ dma.CH7_AL1_CTRL     \ Alias for channel 7 CTRL register
  $01D4 equ dma.CH7_AL1_READ_ADDR \ Alias for channel 7 READ_ADDR register
  $01D8 equ dma.CH7_AL1_WRITE_ADDR \ Alias for channel 7 WRITE_ADDR register
  $01DC equ dma.CH7_AL1_TRANS_COUNT_TRIG \ Alias for channel 7 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $01E0 equ dma.CH7_AL2_CTRL     \ Alias for channel 7 CTRL register
  $01E4 equ dma.CH7_AL2_TRANS_COUNT \ Alias for channel 7 TRANS_COUNT register
  $01E8 equ dma.CH7_AL2_READ_ADDR \ Alias for channel 7 READ_ADDR register
  $01EC equ dma.CH7_AL2_WRITE_ADDR_TRIG \ Alias for channel 7 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $01F0 equ dma.CH7_AL3_CTRL     \ Alias for channel 7 CTRL register
  $01F4 equ dma.CH7_AL3_WRITE_ADDR \ Alias for channel 7 WRITE_ADDR register
  $01F8 equ dma.CH7_AL3_TRANS_COUNT \ Alias for channel 7 TRANS_COUNT register
  $01FC equ dma.CH7_AL3_READ_ADDR_TRIG \ Alias for channel 7 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0200 equ dma.CH8_READ_ADDR    \ DMA Channel 8 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0204 equ dma.CH8_WRITE_ADDR   \ DMA Channel 8 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0208 equ dma.CH8_TRANS_COUNT  \ DMA Channel 8 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $020C equ dma.CH8_CTRL_TRIG    \ DMA Channel 8 Control and Status
  $0210 equ dma.CH8_AL1_CTRL     \ Alias for channel 8 CTRL register
  $0214 equ dma.CH8_AL1_READ_ADDR \ Alias for channel 8 READ_ADDR register
  $0218 equ dma.CH8_AL1_WRITE_ADDR \ Alias for channel 8 WRITE_ADDR register
  $021C equ dma.CH8_AL1_TRANS_COUNT_TRIG \ Alias for channel 8 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0220 equ dma.CH8_AL2_CTRL     \ Alias for channel 8 CTRL register
  $0224 equ dma.CH8_AL2_TRANS_COUNT \ Alias for channel 8 TRANS_COUNT register
  $0228 equ dma.CH8_AL2_READ_ADDR \ Alias for channel 8 READ_ADDR register
  $022C equ dma.CH8_AL2_WRITE_ADDR_TRIG \ Alias for channel 8 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0230 equ dma.CH8_AL3_CTRL     \ Alias for channel 8 CTRL register
  $0234 equ dma.CH8_AL3_WRITE_ADDR \ Alias for channel 8 WRITE_ADDR register
  $0238 equ dma.CH8_AL3_TRANS_COUNT \ Alias for channel 8 TRANS_COUNT register
  $023C equ dma.CH8_AL3_READ_ADDR_TRIG \ Alias for channel 8 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0240 equ dma.CH9_READ_ADDR    \ DMA Channel 9 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0244 equ dma.CH9_WRITE_ADDR   \ DMA Channel 9 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0248 equ dma.CH9_TRANS_COUNT  \ DMA Channel 9 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $024C equ dma.CH9_CTRL_TRIG    \ DMA Channel 9 Control and Status
  $0250 equ dma.CH9_AL1_CTRL     \ Alias for channel 9 CTRL register
  $0254 equ dma.CH9_AL1_READ_ADDR \ Alias for channel 9 READ_ADDR register
  $0258 equ dma.CH9_AL1_WRITE_ADDR \ Alias for channel 9 WRITE_ADDR register
  $025C equ dma.CH9_AL1_TRANS_COUNT_TRIG \ Alias for channel 9 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0260 equ dma.CH9_AL2_CTRL     \ Alias for channel 9 CTRL register
  $0264 equ dma.CH9_AL2_TRANS_COUNT \ Alias for channel 9 TRANS_COUNT register
  $0268 equ dma.CH9_AL2_READ_ADDR \ Alias for channel 9 READ_ADDR register
  $026C equ dma.CH9_AL2_WRITE_ADDR_TRIG \ Alias for channel 9 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0270 equ dma.CH9_AL3_CTRL     \ Alias for channel 9 CTRL register
  $0274 equ dma.CH9_AL3_WRITE_ADDR \ Alias for channel 9 WRITE_ADDR register
  $0278 equ dma.CH9_AL3_TRANS_COUNT \ Alias for channel 9 TRANS_COUNT register
  $027C equ dma.CH9_AL3_READ_ADDR_TRIG \ Alias for channel 9 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0280 equ dma.CH10_READ_ADDR   \ DMA Channel 10 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $0284 equ dma.CH10_WRITE_ADDR  \ DMA Channel 10 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $0288 equ dma.CH10_TRANS_COUNT \ DMA Channel 10 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $028C equ dma.CH10_CTRL_TRIG   \ DMA Channel 10 Control and Status
  $0290 equ dma.CH10_AL1_CTRL    \ Alias for channel 10 CTRL register
  $0294 equ dma.CH10_AL1_READ_ADDR \ Alias for channel 10 READ_ADDR register
  $0298 equ dma.CH10_AL1_WRITE_ADDR \ Alias for channel 10 WRITE_ADDR register
  $029C equ dma.CH10_AL1_TRANS_COUNT_TRIG \ Alias for channel 10 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $02A0 equ dma.CH10_AL2_CTRL    \ Alias for channel 10 CTRL register
  $02A4 equ dma.CH10_AL2_TRANS_COUNT \ Alias for channel 10 TRANS_COUNT register
  $02A8 equ dma.CH10_AL2_READ_ADDR \ Alias for channel 10 READ_ADDR register
  $02AC equ dma.CH10_AL2_WRITE_ADDR_TRIG \ Alias for channel 10 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $02B0 equ dma.CH10_AL3_CTRL    \ Alias for channel 10 CTRL register
  $02B4 equ dma.CH10_AL3_WRITE_ADDR \ Alias for channel 10 WRITE_ADDR register
  $02B8 equ dma.CH10_AL3_TRANS_COUNT \ Alias for channel 10 TRANS_COUNT register
  $02BC equ dma.CH10_AL3_READ_ADDR_TRIG \ Alias for channel 10 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $02C0 equ dma.CH11_READ_ADDR   \ DMA Channel 11 Read Address pointer
                                 \ This register updates automatically each time a read completes. The current value is the next address to be read by this channel.
  $02C4 equ dma.CH11_WRITE_ADDR  \ DMA Channel 11 Write Address pointer
                                 \ This register updates automatically each time a write completes. The current value is the next address to be written by this channel.
  $02C8 equ dma.CH11_TRANS_COUNT \ DMA Channel 11 Transfer Count
                                 \ Program the number of bus transfers a channel will perform before halting. Note that, if transfers are larger than one byte in size, this is not equal to the number of bytes transferred (see CTRL_DATA_SIZE).
                                 \ 
                                 \ When the channel is active, reading this register shows the number of transfers remaining, updating automatically each time a write transfer completes.
                                 \ 
                                 \ Writing this register sets the RELOAD value for the transfer counter. Each time this channel is triggered, the RELOAD value is copied into the live transfer counter. The channel can be started multiple times, and will perform the same number of transfers each time, as programmed by most recent write.
                                 \ 
                                 \ The RELOAD value can be observed at CHx_DBG_TCR. If TRANS_COUNT is used as a trigger, the written value is used immediately as the length of the new transfer sequence, as well as being written to RELOAD.
  $02CC equ dma.CH11_CTRL_TRIG   \ DMA Channel 11 Control and Status
  $02D0 equ dma.CH11_AL1_CTRL    \ Alias for channel 11 CTRL register
  $02D4 equ dma.CH11_AL1_READ_ADDR \ Alias for channel 11 READ_ADDR register
  $02D8 equ dma.CH11_AL1_WRITE_ADDR \ Alias for channel 11 WRITE_ADDR register
  $02DC equ dma.CH11_AL1_TRANS_COUNT_TRIG \ Alias for channel 11 TRANS_COUNT register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $02E0 equ dma.CH11_AL2_CTRL    \ Alias for channel 11 CTRL register
  $02E4 equ dma.CH11_AL2_TRANS_COUNT \ Alias for channel 11 TRANS_COUNT register
  $02E8 equ dma.CH11_AL2_READ_ADDR \ Alias for channel 11 READ_ADDR register
  $02EC equ dma.CH11_AL2_WRITE_ADDR_TRIG \ Alias for channel 11 WRITE_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $02F0 equ dma.CH11_AL3_CTRL    \ Alias for channel 11 CTRL register
  $02F4 equ dma.CH11_AL3_WRITE_ADDR \ Alias for channel 11 WRITE_ADDR register
  $02F8 equ dma.CH11_AL3_TRANS_COUNT \ Alias for channel 11 TRANS_COUNT register
  $02FC equ dma.CH11_AL3_READ_ADDR_TRIG \ Alias for channel 11 READ_ADDR register
                                 \ This is a trigger register (0xc). Writing a nonzero value will
                                 \ reload the channel counter and start the channel.
  $0400 equ dma.INTR             \ Interrupt Status (raw)
  $0404 equ dma.INTE0            \ Interrupt Enables for IRQ 0
  $0408 equ dma.INTF0            \ Force Interrupts
  $040C equ dma.INTS0            \ Interrupt Status for IRQ 0
  $0414 equ dma.INTE1            \ Interrupt Enables for IRQ 1
  $0418 equ dma.INTF1            \ Force Interrupts for IRQ 1
  $041C equ dma.INTS1            \ Interrupt Status (masked) for IRQ 1
  $0420 equ dma.TIMER0           \ Pacing (X/Y) Fractional Timer
                                 \ The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
  $0424 equ dma.TIMER1           \ Pacing (X/Y) Fractional Timer
                                 \ The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
  $0428 equ dma.TIMER2           \ Pacing (X/Y) Fractional Timer
                                 \ The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
  $042C equ dma.TIMER3           \ Pacing (X/Y) Fractional Timer
                                 \ The pacing timer produces TREQ assertions at a rate set by ((X/Y) * sys_clk). This equation is evaluated every sys_clk cycles and therefore can only generate TREQs at a rate of 1 per sys_clk (i.e. permanent TREQ) or less.
  $0430 equ dma.MULTI_CHAN_TRIGGER \ Trigger one or more channels simultaneously
  $0434 equ dma.SNIFF_CTRL       \ Sniffer Control
  $0438 equ dma.SNIFF_DATA       \ Data accumulator for sniff hardware
                                 \ Write an initial seed value here before starting a DMA transfer on the channel indicated by SNIFF_CTRL_DMACH. The hardware will update this register each time it observes a read from the indicated channel. Once the channel completes, the final result can be read from this register.
  $0440 equ dma.FIFO_LEVELS      \ Debug RAF, WAF, TDF levels
  $0444 equ dma.CHAN_ABORT       \ Abort an in-progress transfer sequence on one or more channels
  $0448 equ dma.N_CHANNELS       \ The number of channels this DMA instance is equipped with. This DMA supports up to 16 hardware channels, but can be configured with as few as one, to minimise silicon area.
  $0800 equ dma.CH0_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0804 equ dma.CH0_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0840 equ dma.CH1_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0844 equ dma.CH1_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0880 equ dma.CH2_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0884 equ dma.CH2_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $08C0 equ dma.CH3_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $08C4 equ dma.CH3_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0900 equ dma.CH4_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0904 equ dma.CH4_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0940 equ dma.CH5_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0944 equ dma.CH5_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0980 equ dma.CH6_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0984 equ dma.CH6_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $09C0 equ dma.CH7_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $09C4 equ dma.CH7_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0A00 equ dma.CH8_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0A04 equ dma.CH8_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0A40 equ dma.CH9_DBG_CTDREQ   \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0A44 equ dma.CH9_DBG_TCR      \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0A80 equ dma.CH10_DBG_CTDREQ  \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0A84 equ dma.CH10_DBG_TCR     \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer
  $0AC0 equ dma.CH11_DBG_CTDREQ  \ Read: get channel DREQ counter (i.e. how many accesses the DMA expects it can perform on the peripheral without overflow/underflow. Write any value: clears the counter, and cause channel to re-initiate DREQ handshake.
  $0AC4 equ dma.CH11_DBG_TCR     \ Read to get channel TRANS_COUNT reload value, i.e. the length of the next transfer


\ ====================================== 
\ USBCTRL_DPRAM - DPRAM layout for USB device.
\ ====================================== 

$5010:0000 equ _USBCTRL_DPRAM
  $0000 equ usbdp.SETUP_PACKET_LOW \ Bytes 0-3 of the SETUP packet from the host.
  $0004 equ usbdp.SETUP_PACKET_HIGH \ Bytes 4-7 of the setup packet from the host.
  $0008 equ usbdp.EP1_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $000C equ usbdp.EP1_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0010 equ usbdp.EP2_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0014 equ usbdp.EP2_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0018 equ usbdp.EP3_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $001C equ usbdp.EP3_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0020 equ usbdp.EP4_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0024 equ usbdp.EP4_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0028 equ usbdp.EP5_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $002C equ usbdp.EP5_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0030 equ usbdp.EP6_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0034 equ usbdp.EP6_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0038 equ usbdp.EP7_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $003C equ usbdp.EP7_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0040 equ usbdp.EP8_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0044 equ usbdp.EP8_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0048 equ usbdp.EP9_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $004C equ usbdp.EP9_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0050 equ usbdp.EP10_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0054 equ usbdp.EP10_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0058 equ usbdp.EP11_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $005C equ usbdp.EP11_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0060 equ usbdp.EP12_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0064 equ usbdp.EP12_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0068 equ usbdp.EP13_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $006C equ usbdp.EP13_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0070 equ usbdp.EP14_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0074 equ usbdp.EP14_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0078 equ usbdp.EP15_IN_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $007C equ usbdp.EP15_OUT_CONTROL \ Bytes 4-7 of the setup packet from the host.
  $0080 equ usbdp.EP0_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $0084 equ usbdp.EP0_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $0088 equ usbdp.EP1_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $008C equ usbdp.EP1_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $0090 equ usbdp.EP2_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $0094 equ usbdp.EP2_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $0098 equ usbdp.EP3_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $009C equ usbdp.EP3_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00A0 equ usbdp.EP4_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00A4 equ usbdp.EP4_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00A8 equ usbdp.EP5_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00AC equ usbdp.EP5_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00B0 equ usbdp.EP6_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00B4 equ usbdp.EP6_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00B8 equ usbdp.EP7_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00BC equ usbdp.EP7_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00C0 equ usbdp.EP8_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00C4 equ usbdp.EP8_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00C8 equ usbdp.EP9_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00CC equ usbdp.EP9_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00D0 equ usbdp.EP10_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00D4 equ usbdp.EP10_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00D8 equ usbdp.EP11_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00DC equ usbdp.EP11_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00E0 equ usbdp.EP12_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00E4 equ usbdp.EP12_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00E8 equ usbdp.EP13_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00EC equ usbdp.EP13_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00F0 equ usbdp.EP14_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00F4 equ usbdp.EP14_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00F8 equ usbdp.EP15_IN_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.
  $00FC equ usbdp.EP15_OUT_BUFFER_CONTROL \ Buffer control for both buffers of an endpoint. Fields ending in a _1 are for buffer 1.
                                 \ Fields ending in a _0 are for buffer 0. Buffer 1 controls are only valid if the endpoint is in double buffered mode.


\ ====================================== 
\ USBCTRL_REGS - USB FS/LS controller device registers
\ ====================================== 

$5011:0000 equ _USBCTRL_REGS
#21 equ USBCTRL_IRQ_IRQn
  $0000 equ usbreg.ADDR_ENDP     \ Device address and endpoint control
  $0004 equ usbreg.ADDR_ENDP1    \ Interrupt endpoint 1. Only valid for HOST mode.
  $0008 equ usbreg.ADDR_ENDP2    \ Interrupt endpoint 2. Only valid for HOST mode.
  $000C equ usbreg.ADDR_ENDP3    \ Interrupt endpoint 3. Only valid for HOST mode.
  $0010 equ usbreg.ADDR_ENDP4    \ Interrupt endpoint 4. Only valid for HOST mode.
  $0014 equ usbreg.ADDR_ENDP5    \ Interrupt endpoint 5. Only valid for HOST mode.
  $0018 equ usbreg.ADDR_ENDP6    \ Interrupt endpoint 6. Only valid for HOST mode.
  $001C equ usbreg.ADDR_ENDP7    \ Interrupt endpoint 7. Only valid for HOST mode.
  $0020 equ usbreg.ADDR_ENDP8    \ Interrupt endpoint 8. Only valid for HOST mode.
  $0024 equ usbreg.ADDR_ENDP9    \ Interrupt endpoint 9. Only valid for HOST mode.
  $0028 equ usbreg.ADDR_ENDP10   \ Interrupt endpoint 10. Only valid for HOST mode.
  $002C equ usbreg.ADDR_ENDP11   \ Interrupt endpoint 11. Only valid for HOST mode.
  $0030 equ usbreg.ADDR_ENDP12   \ Interrupt endpoint 12. Only valid for HOST mode.
  $0034 equ usbreg.ADDR_ENDP13   \ Interrupt endpoint 13. Only valid for HOST mode.
  $0038 equ usbreg.ADDR_ENDP14   \ Interrupt endpoint 14. Only valid for HOST mode.
  $003C equ usbreg.ADDR_ENDP15   \ Interrupt endpoint 15. Only valid for HOST mode.
  $0040 equ usbreg.MAIN_CTRL     \ Main control register
  $0044 equ usbreg.SOF_WR        \ Set the SOF (Start of Frame) frame number in the host controller. The SOF packet is sent every 1ms and the host will increment the frame number by 1 each time.
  $0048 equ usbreg.SOF_RD        \ Read the last SOF (Start of Frame) frame number seen. In device mode the last SOF received from the host. In host mode the last SOF sent by the host.
  $004C equ usbreg.SIE_CTRL      \ SIE control register
  $0050 equ usbreg.SIE_STATUS    \ SIE status register
  $0054 equ usbreg.INT_EP_CTRL   \ interrupt endpoint control register
  $0058 equ usbreg.BUFF_STATUS   \ Buffer status register. A bit set here indicates that a buffer has completed on the endpoint (if the buffer interrupt is enabled). It is possible for 2 buffers to be completed, so clearing the buffer status bit may instantly re set it on the next clock cycle.
  $005C equ usbreg.BUFF_CPU_SHOULD_HANDLE \ Which of the double buffers should be handled. Only valid if using an interrupt per buffer (i.e. not per 2 buffers). Not valid for host interrupt endpoint polling because they are only single buffered.
  $0060 equ usbreg.EP_ABORT      \ Device only: Can be set to ignore the buffer control register for this endpoint in case you would like to revoke a buffer. A NAK will be sent for every access to the endpoint until this bit is cleared. A corresponding bit in `EP_ABORT_DONE` is set when it is safe to modify the buffer control register.
  $0064 equ usbreg.EP_ABORT_DONE \ Device only: Used in conjunction with `EP_ABORT`. Set once an endpoint is idle so the programmer knows it is safe to modify the buffer control register.
  $0068 equ usbreg.EP_STALL_ARM  \ Device: this bit must be set in conjunction with the `STALL` bit in the buffer control register to send a STALL on EP0. The device controller clears these bits when a SETUP packet is received because the USB spec requires that a STALL condition is cleared when a SETUP packet is received.
  $006C equ usbreg.NAK_POLL      \ Used by the host controller. Sets the wait time in microseconds before trying again if the device replies with a NAK.
  $0070 equ usbreg.EP_STATUS_STALL_NAK \ Device: bits are set when the `IRQ_ON_NAK` or `IRQ_ON_STALL` bits are set. For EP0 this comes from `SIE_CTRL`. For all other endpoints it comes from the endpoint control register.
  $0074 equ usbreg.USB_MUXING    \ Where to connect the USB controller. Should be to_phy by default.
  $0078 equ usbreg.USB_PWR       \ Overrides for the power signals in the event that the VBUS signals are not hooked up to GPIO. Set the value of the override and then the override enable to switch over to the override value.
  $007C equ usbreg.USBPHY_DIRECT \ This register allows for direct control of the USB phy. Use in conjunction with usbphy_direct_override register to enable each override bit.
  $0080 equ usbreg.USBPHY_DIRECT_OVERRIDE \ Override enable for each control in usbphy_direct
  $0084 equ usbreg.USBPHY_TRIM   \ Used to adjust trim values of USB phy pull down resistors.
  $008C equ usbreg.INTR          \ Raw Interrupts
  $0090 equ usbreg.INTE          \ Interrupt Enable
  $0094 equ usbreg.INTF          \ Interrupt Force
  $0098 equ usbreg.INTS          \ Interrupt status after masking & forcing


\ ====================================== 
\ PIO0 - Programmable IO block
\ ====================================== 

$5020:0000 equ _PIO0
#23 equ PIO0_IRQ_0_IRQn
#24 equ PIO0_IRQ_1_IRQn
  $0000 equ pio.CTRL             \ PIO control register
  $0004 equ pio.FSTAT            \ FIFO status register
  $0008 equ pio.FDEBUG           \ FIFO debug register
  $000C equ pio.FLEVEL           \ FIFO levels
  $0010 equ pio.TXF0             \ Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
  $0014 equ pio.TXF1             \ Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
  $0018 equ pio.TXF2             \ Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
  $001C equ pio.TXF3             \ Direct write access to the TX FIFO for this state machine. Each write pushes one word to the FIFO. Attempting to write to a full FIFO has no effect on the FIFO state or contents, and sets the sticky FDEBUG_TXOVER error flag for this FIFO.
  $0020 equ pio.RXF0             \ Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
  $0024 equ pio.RXF1             \ Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
  $0028 equ pio.RXF2             \ Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
  $002C equ pio.RXF3             \ Direct read access to the RX FIFO for this state machine. Each read pops one word from the FIFO. Attempting to read from an empty FIFO has no effect on the FIFO state, and sets the sticky FDEBUG_RXUNDER error flag for this FIFO. The data returned to the system on a read from an empty FIFO is undefined.
  $0030 equ pio.IRQ              \ State machine IRQ flags register. Write 1 to clear. There are 8 state machine IRQ flags, which can be set, cleared, and waited on by the state machines. There's no fixed association between flags and state machines -- any state machine can use any flag.
                                 \ 
                                 \ Any of the 8 flags can be used for timing synchronisation between state machines, using IRQ and WAIT instructions. The lower four of these flags are also routed out to system-level interrupt requests, alongside FIFO status interrupts -- see e.g. IRQ0_INTE.
  $0034 equ pio.IRQ_FORCE        \ Writing a 1 to each of these bits will forcibly assert the corresponding IRQ. Note this is different to the INTF register: writing here affects PIO internal state. INTF just asserts the processor-facing IRQ signal for testing ISRs, and is not visible to the state machines.
  $0038 equ pio.INPUT_SYNC_BYPASS \ There is a 2-flipflop synchronizer on each GPIO input, which protects PIO logic from metastabilities. This increases input delay, and for fast synchronous IO (e.g. SPI) these synchronizers may need to be bypassed. Each bit in this register corresponds to one GPIO.
                                 \ 0 -> input is synchronized (default)
                                 \ 1 -> synchronizer is bypassed
                                 \ If in doubt, leave this register as all zeroes.
  $003C equ pio.DBG_PADOUT       \ Read to sample the pad output values PIO is currently driving to the GPIOs. On RP2040 there are 30 GPIOs, so the two most significant bits are hardwired to 0.
  $0040 equ pio.DBG_PADOE        \ Read to sample the pad output enables (direction) PIO is currently driving to the GPIOs. On RP2040 there are 30 GPIOs, so the two most significant bits are hardwired to 0.
  $0044 equ pio.DBG_CFGINFO      \ The PIO hardware has some free parameters that may vary between chip products.
                                 \ These should be provided in the chip datasheet, but are also exposed here.
  $0048 equ pio.INSTR_MEM0       \ Write-only access to instruction memory location 0
  $004C equ pio.INSTR_MEM1       \ Write-only access to instruction memory location 1
  $0050 equ pio.INSTR_MEM2       \ Write-only access to instruction memory location 2
  $0054 equ pio.INSTR_MEM3       \ Write-only access to instruction memory location 3
  $0058 equ pio.INSTR_MEM4       \ Write-only access to instruction memory location 4
  $005C equ pio.INSTR_MEM5       \ Write-only access to instruction memory location 5
  $0060 equ pio.INSTR_MEM6       \ Write-only access to instruction memory location 6
  $0064 equ pio.INSTR_MEM7       \ Write-only access to instruction memory location 7
  $0068 equ pio.INSTR_MEM8       \ Write-only access to instruction memory location 8
  $006C equ pio.INSTR_MEM9       \ Write-only access to instruction memory location 9
  $0070 equ pio.INSTR_MEM10      \ Write-only access to instruction memory location 10
  $0074 equ pio.INSTR_MEM11      \ Write-only access to instruction memory location 11
  $0078 equ pio.INSTR_MEM12      \ Write-only access to instruction memory location 12
  $007C equ pio.INSTR_MEM13      \ Write-only access to instruction memory location 13
  $0080 equ pio.INSTR_MEM14      \ Write-only access to instruction memory location 14
  $0084 equ pio.INSTR_MEM15      \ Write-only access to instruction memory location 15
  $0088 equ pio.INSTR_MEM16      \ Write-only access to instruction memory location 16
  $008C equ pio.INSTR_MEM17      \ Write-only access to instruction memory location 17
  $0090 equ pio.INSTR_MEM18      \ Write-only access to instruction memory location 18
  $0094 equ pio.INSTR_MEM19      \ Write-only access to instruction memory location 19
  $0098 equ pio.INSTR_MEM20      \ Write-only access to instruction memory location 20
  $009C equ pio.INSTR_MEM21      \ Write-only access to instruction memory location 21
  $00A0 equ pio.INSTR_MEM22      \ Write-only access to instruction memory location 22
  $00A4 equ pio.INSTR_MEM23      \ Write-only access to instruction memory location 23
  $00A8 equ pio.INSTR_MEM24      \ Write-only access to instruction memory location 24
  $00AC equ pio.INSTR_MEM25      \ Write-only access to instruction memory location 25
  $00B0 equ pio.INSTR_MEM26      \ Write-only access to instruction memory location 26
  $00B4 equ pio.INSTR_MEM27      \ Write-only access to instruction memory location 27
  $00B8 equ pio.INSTR_MEM28      \ Write-only access to instruction memory location 28
  $00BC equ pio.INSTR_MEM29      \ Write-only access to instruction memory location 29
  $00C0 equ pio.INSTR_MEM30      \ Write-only access to instruction memory location 30
  $00C4 equ pio.INSTR_MEM31      \ Write-only access to instruction memory location 31
  $00C8 equ pio.SM0_CLKDIV       \ Clock divisor register for state machine 0
                                 \ Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
  $00CC equ pio.SM0_EXECCTRL     \ Execution/behavioural settings for state machine 0
  $00D0 equ pio.SM0_SHIFTCTRL    \ Control behaviour of the input/output shift registers for state machine 0
  $00D4 equ pio.SM0_ADDR         \ Current instruction address of state machine 0
  $00D8 equ pio.SM0_INSTR        \ Read to see the instruction currently addressed by state machine 0's program counter
                                 \ Write to execute an instruction immediately (including jumps) and then resume execution.
  $00DC equ pio.SM0_PINCTRL      \ State machine pin control
  $00E0 equ pio.SM1_CLKDIV       \ Clock divisor register for state machine 1
                                 \ Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
  $00E4 equ pio.SM1_EXECCTRL     \ Execution/behavioural settings for state machine 1
  $00E8 equ pio.SM1_SHIFTCTRL    \ Control behaviour of the input/output shift registers for state machine 1
  $00EC equ pio.SM1_ADDR         \ Current instruction address of state machine 1
  $00F0 equ pio.SM1_INSTR        \ Read to see the instruction currently addressed by state machine 1's program counter
                                 \ Write to execute an instruction immediately (including jumps) and then resume execution.
  $00F4 equ pio.SM1_PINCTRL      \ State machine pin control
  $00F8 equ pio.SM2_CLKDIV       \ Clock divisor register for state machine 2
                                 \ Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
  $00FC equ pio.SM2_EXECCTRL     \ Execution/behavioural settings for state machine 2
  $0100 equ pio.SM2_SHIFTCTRL    \ Control behaviour of the input/output shift registers for state machine 2
  $0104 equ pio.SM2_ADDR         \ Current instruction address of state machine 2
  $0108 equ pio.SM2_INSTR        \ Read to see the instruction currently addressed by state machine 2's program counter
                                 \ Write to execute an instruction immediately (including jumps) and then resume execution.
  $010C equ pio.SM2_PINCTRL      \ State machine pin control
  $0110 equ pio.SM3_CLKDIV       \ Clock divisor register for state machine 3
                                 \ Frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
  $0114 equ pio.SM3_EXECCTRL     \ Execution/behavioural settings for state machine 3
  $0118 equ pio.SM3_SHIFTCTRL    \ Control behaviour of the input/output shift registers for state machine 3
  $011C equ pio.SM3_ADDR         \ Current instruction address of state machine 3
  $0120 equ pio.SM3_INSTR        \ Read to see the instruction currently addressed by state machine 3's program counter
                                 \ Write to execute an instruction immediately (including jumps) and then resume execution.
  $0124 equ pio.SM3_PINCTRL      \ State machine pin control
  $0128 equ pio.INTR             \ Raw Interrupts
  $012C equ pio.IRQ0_INTE        \ Interrupt Enable for irq0
  $0130 equ pio.IRQ0_INTF        \ Interrupt Force for irq0
  $0134 equ pio.IRQ0_INTS        \ Interrupt status after masking & forcing for irq0
  $0138 equ pio.IRQ1_INTE        \ Interrupt Enable for irq1
  $013C equ pio.IRQ1_INTF        \ Interrupt Force for irq1
  $0140 equ pio.IRQ1_INTS        \ Interrupt status after masking & forcing for irq1


\ ====================================== 
\ PIO1 - Interrupt status after masking & forcing for irq1
\ ====================================== 

$5030:0000 equ _PIO1
#25 equ PIO1_IRQ_0_IRQn
#26 equ PIO1_IRQ_1_IRQn


\ ====================================== 
\ SIO - Single-cycle IO block
\ Provides core-local and inter-core hardware for the two processors, with single-cycle access.
\ ====================================== 

$D000:0000 equ _SIO
#31 equ SIO_IRQ_PROC0_IRQn
#32 equ SIO_IRQ_PROC1_IRQn
  $0000 equ scio.CPUID           \ Processor core identifier
                                 \ Value is 0 when read from processor core 0, and 1 when read from processor core 1.
  $0004 equ scio.GPIO_IN         \ Input value for GPIO pins
  $0008 equ scio.GPIO_HI_IN      \ Input value for QSPI pins
  $0010 equ scio.GPIO_OUT        \ GPIO output value
  $0014 equ scio.GPIO_OUT_SET    \ GPIO output value set
  $0018 equ scio.GPIO_OUT_CLR    \ GPIO output value clear
  $001C equ scio.GPIO_OUT_XOR    \ GPIO output value XOR
  $0020 equ scio.GPIO_OE         \ GPIO output enable
  $0024 equ scio.GPIO_OE_SET     \ GPIO output enable set
  $0028 equ scio.GPIO_OE_CLR     \ GPIO output enable clear
  $002C equ scio.GPIO_OE_XOR     \ GPIO output enable XOR
  $0030 equ scio.GPIO_HI_OUT     \ QSPI output value
  $0034 equ scio.GPIO_HI_OUT_SET \ QSPI output value set
  $0038 equ scio.GPIO_HI_OUT_CLR \ QSPI output value clear
  $003C equ scio.GPIO_HI_OUT_XOR \ QSPI output value XOR
  $0040 equ scio.GPIO_HI_OE      \ QSPI output enable
  $0044 equ scio.GPIO_HI_OE_SET  \ QSPI output enable set
  $0048 equ scio.GPIO_HI_OE_CLR  \ QSPI output enable clear
  $004C equ scio.GPIO_HI_OE_XOR  \ QSPI output enable XOR
  $0050 equ scio.FIFO_ST         \ Status register for inter-core FIFOs (mailboxes).
                                 \ There is one FIFO in the core 0 -> core 1 direction, and one core 1 -> core 0. Both are 32 bits wide and 8 words deep.
                                 \ Core 0 can see the read side of the 1->0 FIFO (RX), and the write side of 0->1 FIFO (TX).
                                 \ Core 1 can see the read side of the 0->1 FIFO (RX), and the write side of 1->0 FIFO (TX).
                                 \ The SIO IRQ for each core is the logical OR of the VLD, WOF and ROE fields of its FIFO_ST register.
  $0054 equ scio.FIFO_WR         \ Write access to this core's TX FIFO
  $0058 equ scio.FIFO_RD         \ Read access to this core's RX FIFO
  $005C equ scio.SPINLOCK_ST     \ Spinlock state
                                 \ A bitmap containing the state of all 32 spinlocks (1=locked).
                                 \ Mainly intended for debugging.
  $0060 equ scio.DIV_UDIVIDEND   \ Divider unsigned dividend
                                 \ Write to the DIVIDEND operand of the divider, i.e. the p in `p / q`.
                                 \ Any operand write starts a new calculation. The results appear in QUOTIENT, REMAINDER.
                                 \ UDIVIDEND/SDIVIDEND are aliases of the same internal register. The U alias starts an
                                 \ unsigned calculation, and the S alias starts a signed calculation.
  $0064 equ scio.DIV_UDIVISOR    \ Divider unsigned divisor
                                 \ Write to the DIVISOR operand of the divider, i.e. the q in `p / q`.
                                 \ Any operand write starts a new calculation. The results appear in QUOTIENT, REMAINDER.
                                 \ UDIVISOR/SDIVISOR are aliases of the same internal register. The U alias starts an
                                 \ unsigned calculation, and the S alias starts a signed calculation.
  $0068 equ scio.DIV_SDIVIDEND   \ Divider signed dividend
                                 \ The same as UDIVIDEND, but starts a signed calculation, rather than unsigned.
  $006C equ scio.DIV_SDIVISOR    \ Divider signed divisor
                                 \ The same as UDIVISOR, but starts a signed calculation, rather than unsigned.
  $0070 equ scio.DIV_QUOTIENT    \ Divider result quotient
                                 \ The result of `DIVIDEND / DIVISOR` (division). Contents undefined while CSR_READY is low.
                                 \ For signed calculations, QUOTIENT is negative when the signs of DIVIDEND and DIVISOR differ.
                                 \ This register can be written to directly, for context save/restore purposes. This halts any
                                 \ in-progress calculation and sets the CSR_READY and CSR_DIRTY flags.
                                 \ Reading from QUOTIENT clears the CSR_DIRTY flag, so should read results in the order
                                 \ REMAINDER, QUOTIENT if CSR_DIRTY is used.
  $0074 equ scio.DIV_REMAINDER   \ Divider result remainder
                                 \ The result of `DIVIDEND % DIVISOR` (modulo). Contents undefined while CSR_READY is low.
                                 \ For signed calculations, REMAINDER is negative only when DIVIDEND is negative.
                                 \ This register can be written to directly, for context save/restore purposes. This halts any
                                 \ in-progress calculation and sets the CSR_READY and CSR_DIRTY flags.
  $0078 equ scio.DIV_CSR         \ Control and status register for divider.
  $0080 equ scio.INTERP0_ACCUM0  \ Read/write access to accumulator 0
  $0084 equ scio.INTERP0_ACCUM1  \ Read/write access to accumulator 1
  $0088 equ scio.INTERP0_BASE0   \ Read/write access to BASE0 register.
  $008C equ scio.INTERP0_BASE1   \ Read/write access to BASE1 register.
  $0090 equ scio.INTERP0_BASE2   \ Read/write access to BASE2 register.
  $0094 equ scio.INTERP0_POP_LANE0 \ Read LANE0 result, and simultaneously write lane results to both accumulators (POP).
  $0098 equ scio.INTERP0_POP_LANE1 \ Read LANE1 result, and simultaneously write lane results to both accumulators (POP).
  $009C equ scio.INTERP0_POP_FULL \ Read FULL result, and simultaneously write lane results to both accumulators (POP).
  $00A0 equ scio.INTERP0_PEEK_LANE0 \ Read LANE0 result, without altering any internal state (PEEK).
  $00A4 equ scio.INTERP0_PEEK_LANE1 \ Read LANE1 result, without altering any internal state (PEEK).
  $00A8 equ scio.INTERP0_PEEK_FULL \ Read FULL result, without altering any internal state (PEEK).
  $00AC equ scio.INTERP0_CTRL_LANE0 \ Control register for lane 0
  $00B0 equ scio.INTERP0_CTRL_LANE1 \ Control register for lane 1
  $00B4 equ scio.INTERP0_ACCUM0_ADD \ Values written here are atomically added to ACCUM0
                                 \ Reading yields lane 0's raw shift and mask value (BASE0 not added).
  $00B8 equ scio.INTERP0_ACCUM1_ADD \ Values written here are atomically added to ACCUM1
                                 \ Reading yields lane 1's raw shift and mask value (BASE1 not added).
  $00BC equ scio.INTERP0_BASE_1AND0 \ On write, the lower 16 bits go to BASE0, upper bits to BASE1 simultaneously.
                                 \ Each half is sign-extended to 32 bits if that lane's SIGNED flag is set.
  $00C0 equ scio.INTERP1_ACCUM0  \ Read/write access to accumulator 0
  $00C4 equ scio.INTERP1_ACCUM1  \ Read/write access to accumulator 1
  $00C8 equ scio.INTERP1_BASE0   \ Read/write access to BASE0 register.
  $00CC equ scio.INTERP1_BASE1   \ Read/write access to BASE1 register.
  $00D0 equ scio.INTERP1_BASE2   \ Read/write access to BASE2 register.
  $00D4 equ scio.INTERP1_POP_LANE0 \ Read LANE0 result, and simultaneously write lane results to both accumulators (POP).
  $00D8 equ scio.INTERP1_POP_LANE1 \ Read LANE1 result, and simultaneously write lane results to both accumulators (POP).
  $00DC equ scio.INTERP1_POP_FULL \ Read FULL result, and simultaneously write lane results to both accumulators (POP).
  $00E0 equ scio.INTERP1_PEEK_LANE0 \ Read LANE0 result, without altering any internal state (PEEK).
  $00E4 equ scio.INTERP1_PEEK_LANE1 \ Read LANE1 result, without altering any internal state (PEEK).
  $00E8 equ scio.INTERP1_PEEK_FULL \ Read FULL result, without altering any internal state (PEEK).
  $00EC equ scio.INTERP1_CTRL_LANE0 \ Control register for lane 0
  $00F0 equ scio.INTERP1_CTRL_LANE1 \ Control register for lane 1
  $00F4 equ scio.INTERP1_ACCUM0_ADD \ Values written here are atomically added to ACCUM0
                                 \ Reading yields lane 0's raw shift and mask value (BASE0 not added).
  $00F8 equ scio.INTERP1_ACCUM1_ADD \ Values written here are atomically added to ACCUM1
                                 \ Reading yields lane 1's raw shift and mask value (BASE1 not added).
  $00FC equ scio.INTERP1_BASE_1AND0 \ On write, the lower 16 bits go to BASE0, upper bits to BASE1 simultaneously.
                                 \ Each half is sign-extended to 32 bits if that lane's SIGNED flag is set.
  $0100 equ scio.SPINLOCK0       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0104 equ scio.SPINLOCK1       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0108 equ scio.SPINLOCK2       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $010C equ scio.SPINLOCK3       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0110 equ scio.SPINLOCK4       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0114 equ scio.SPINLOCK5       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0118 equ scio.SPINLOCK6       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $011C equ scio.SPINLOCK7       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0120 equ scio.SPINLOCK8       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0124 equ scio.SPINLOCK9       \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0128 equ scio.SPINLOCK10      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $012C equ scio.SPINLOCK11      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0130 equ scio.SPINLOCK12      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0134 equ scio.SPINLOCK13      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0138 equ scio.SPINLOCK14      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $013C equ scio.SPINLOCK15      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0140 equ scio.SPINLOCK16      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0144 equ scio.SPINLOCK17      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0148 equ scio.SPINLOCK18      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $014C equ scio.SPINLOCK19      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0150 equ scio.SPINLOCK20      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0154 equ scio.SPINLOCK21      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0158 equ scio.SPINLOCK22      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $015C equ scio.SPINLOCK23      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0160 equ scio.SPINLOCK24      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0164 equ scio.SPINLOCK25      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0168 equ scio.SPINLOCK26      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $016C equ scio.SPINLOCK27      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0170 equ scio.SPINLOCK28      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0174 equ scio.SPINLOCK29      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $0178 equ scio.SPINLOCK30      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.
  $017C equ scio.SPINLOCK31      \ Reading from a spinlock address will:
                                 \ - Return 0 if lock is already locked
                                 \ - Otherwise return nonzero, and simultaneously claim the lock
                                 \ 
                                 \ Writing (any value) releases the lock.
                                 \ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
                                 \ The value returned on success is 0x1 << lock number.


\ ====================================== 
\ PPB - Reading from a spinlock address will:
\ - Return 0 if lock is already locked
\ - Otherwise return nonzero, and simultaneously claim the lock
\ 
\ Writing (any value) releases the lock.
\ If core 0 and core 1 attempt to claim the same lock simultaneously, core 0 wins.
\ The value returned on success is 0x1 << lock number.
\ ====================================== 

$E000:0000 equ _PPB
  $E010 equ ppb.SYST_CSR         \ Use the SysTick Control and Status Register to enable the SysTick features.
  $E014 equ ppb.SYST_RVR         \ Use the SysTick Reload Value Register to specify the start value to load into the current value register when the counter reaches 0. It can be any value between 0 and 0x00FFFFFF. A start value of 0 is possible, but has no effect because the SysTick interrupt and COUNTFLAG are activated when counting from 1 to 0. The reset value of this register is UNKNOWN.
                                 \ To generate a multi-shot timer with a period of N processor clock cycles, use a RELOAD value of N-1. For example, if the SysTick interrupt is required every 100 clock pulses, set RELOAD to 99.
  $E018 equ ppb.SYST_CVR         \ Use the SysTick Current Value Register to find the current value in the register. The reset value of this register is UNKNOWN.
  $E01C equ ppb.SYST_CALIB       \ Use the SysTick Calibration Value Register to enable software to scale to any required speed using divide and multiply.
  $E100 equ ppb.NVIC_ISER        \ Use the Interrupt Set-Enable Register to enable interrupts and determine which interrupts are currently enabled.
                                 \ If a pending interrupt is enabled, the NVIC activates the interrupt based on its priority. If an interrupt is not enabled, asserting its interrupt signal changes the interrupt state to pending, but the NVIC never activates the interrupt, regardless of its priority.
  $E180 equ ppb.NVIC_ICER        \ Use the Interrupt Clear-Enable Registers to disable interrupts and determine which interrupts are currently enabled.
  $E200 equ ppb.NVIC_ISPR        \ The NVIC_ISPR forces interrupts into the pending state, and shows which interrupts are pending.
  $E280 equ ppb.NVIC_ICPR        \ Use the Interrupt Clear-Pending Register to clear pending interrupts and determine which interrupts are currently pending.
  $E400 equ ppb.NVIC_IPR0        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
                                 \ Note: Writing 1 to an NVIC_ICPR bit does not affect the active state of the corresponding interrupt.
                                 \ These registers are only word-accessible
  $E404 equ ppb.NVIC_IPR1        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
  $E408 equ ppb.NVIC_IPR2        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
  $E40C equ ppb.NVIC_IPR3        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
  $E410 equ ppb.NVIC_IPR4        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
  $E414 equ ppb.NVIC_IPR5        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
  $E418 equ ppb.NVIC_IPR6        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
  $E41C equ ppb.NVIC_IPR7        \ Use the Interrupt Priority Registers to assign a priority from 0 to 3 to each of the available interrupts. 0 is the highest priority, and 3 is the lowest.
  $ED00 equ ppb.CPUID            \ Read the CPU ID Base Register to determine: the ID number of the processor core, the version number of the processor core, the implementation details of the processor core.
  $ED04 equ ppb.ICSR             \ Use the Interrupt Control State Register to set a pending Non-Maskable Interrupt (NMI), set or clear a pending PendSV, set or clear a pending SysTick, check for pending exceptions, check the vector number of the highest priority pended exception, check the vector number of the active exception.
  $ED08 equ ppb.VTOR             \ The VTOR holds the vector table offset address.
  $ED0C equ ppb.AIRCR            \ Use the Application Interrupt and Reset Control Register to: determine data endianness, clear all active state information from debug halt mode, request a system reset.
  $ED10 equ ppb.SCR              \ System Control Register. Use the System Control Register for power-management functions: signal to the system when the processor can enter a low power state, control how the processor enters and exits low power states.
  $ED14 equ ppb.CCR              \ The Configuration and Control Register permanently enables stack alignment and causes unaligned accesses to result in a Hard Fault.
  $ED1C equ ppb.SHPR2            \ System handlers are a special class of exception handler that can have their priority set to any of the priority levels. Use the System Handler Priority Register 2 to set the priority of SVCall.
  $ED20 equ ppb.SHPR3            \ System handlers are a special class of exception handler that can have their priority set to any of the priority levels. Use the System Handler Priority Register 3 to set the priority of PendSV and SysTick.
  $ED24 equ ppb.SHCSR            \ Use the System Handler Control and State Register to determine or clear the pending status of SVCall.
  $ED90 equ ppb.MPU_TYPE         \ Read the MPU Type Register to determine if the processor implements an MPU, and how many regions the MPU supports.
  $ED94 equ ppb.MPU_CTRL         \ Use the MPU Control Register to enable and disable the MPU, and to control whether the default memory map is enabled as a background region for privileged accesses, and whether the MPU is enabled for HardFaults and NMIs.
  $ED98 equ ppb.MPU_RNR          \ Use the MPU Region Number Register to select the region currently accessed by MPU_RBAR and MPU_RASR.
  $ED9C equ ppb.MPU_RBAR         \ Read the MPU Region Base Address Register to determine the base address of the region identified by MPU_RNR. Write to update the base address of said region or that of a specified region, with whose number MPU_RNR will also be updated.
  $EDA0 equ ppb.MPU_RASR         \ Use the MPU Region Attribute and Size Register to define the size, access behaviour and memory type of the region identified by MPU_RNR, and enable that region.