\ sfrSTM32F103xx.fth - peripheral definitions for STM32F103xx

((
Copyright (c) 2019
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))



\ ======================================
\ FSMC - Flexible static memory controller
\ ======================================

$A000:0000 equ _FSMC
#64 equ FSMC_IRQn
#64 equ FSMCvec#
  $0000 equ fsmcBCR1             \ SRAM/NOR-Flash chip-select control register          1
  $0004 equ fsmcBTR1             \ SRAM/NOR-Flash chip-select timing register          1
  $0008 equ fsmcBCR2             \ SRAM/NOR-Flash chip-select control register          2
  $000C equ fsmcBTR2             \ SRAM/NOR-Flash chip-select timing register          2
  $0010 equ fsmcBCR3             \ SRAM/NOR-Flash chip-select control register          3
  $0014 equ fsmcBTR3             \ SRAM/NOR-Flash chip-select timing register          3
  $0018 equ fsmcBCR4             \ SRAM/NOR-Flash chip-select control register          4
  $001C equ fsmcBTR4             \ SRAM/NOR-Flash chip-select timing register          4
  $0060 equ fsmcPCR2             \ PC Card/NAND Flash control register          2
  $0064 equ fsmcSR2              \ FIFO status and interrupt register          2
  $0068 equ fsmcPMEM2            \ Common memory space timing register          2
  $006C equ fsmcPATT2            \ Attribute memory space timing register          2
  $0074 equ fsmcECCR2            \ ECC result register 2
  $0080 equ fsmcPCR3             \ PC Card/NAND Flash control register          3
  $0084 equ fsmcSR3              \ FIFO status and interrupt register          3
  $0088 equ fsmcPMEM3            \ Common memory space timing register          3
  $008C equ fsmcPATT3            \ Attribute memory space timing register          3
  $0094 equ fsmcECCR3            \ ECC result register 3
  $00A0 equ fsmcPCR4             \ PC Card/NAND Flash control register          4
  $00A4 equ fsmcSR4              \ FIFO status and interrupt register          4
  $00A8 equ fsmcPMEM4            \ Common memory space timing register          4
  $00AC equ fsmcPATT4            \ Attribute memory space timing register          4
  $00B0 equ fsmcPIO4             \ I/O space timing register 4
  $0104 equ fsmcBWTR1            \ SRAM/NOR-Flash write timing registers          1
  $010C equ fsmcBWTR2            \ SRAM/NOR-Flash write timing registers          2
  $0114 equ fsmcBWTR3            \ SRAM/NOR-Flash write timing registers          3
  $011C equ fsmcBWTR4            \ SRAM/NOR-Flash write timing registers          4


\ ======================================
\ PWR - Power control
\ ======================================

$4000:7000 equ _PWR
#17 equ PVD_IRQn
#17 equ PVDvec#
  $0000 equ pwrCR                \ Power control register          (PWR_CR)
  $0004 equ pwrCSR               \ Power control register          (PWR_CR)


\ ======================================
\ RCC - Reset and clock control
\ ======================================

$4002:1000 equ _RCC
#17 equ PVD_IRQn
#17 equ PVDvec#
  $0000 equ rccCR               \ Clock control register
  $0004 equ rccCFGR             \ Clock configuration register          (RCC_CFGR)
  $0008 equ rccCIR              \ Clock interrupt register          (RCC_CIR)
  $000C equ rccAPB2RSTR         \ APB2 peripheral reset register          (RCC_APB2RSTR)
    $0C equ rccAPB2rst          \ APB2 peripheral reset register          (RCC_APB2RSTR)
  $0010 equ rccAPB1RSTR         \ APB1 peripheral reset register          (RCC_APB1RSTR)
  $0014 equ rccAHBENR           \ AHB Peripheral Clock enable register          (RCC_AHBENR)
  $0018 equ rccAPB2ENR          \ APB2 peripheral clock enable register          (RCC_APB2ENR)
    $18 equ rccAPB2en		\ APB2 clock enable
  $001C equ rccAPB1ENR          \ APB1 peripheral clock enable register          (RCC_APB1ENR)
  $0020 equ rccBDCR             \ Backup domain control register          (RCC_BDCR)
  $0024 equ rccCSR              \ Control/status register          (RCC_CSR)


\ ======================================
\ GPIOA - General purpose I/O
\ ======================================

$4001:0800 equ _GPIOA
$4001:0C00 equ _GPIOB
$4001:1000 equ _GPIOC
$4001:1400 equ _GPIOD
$4001:1800 equ _GPIOE
$4001:1C00 equ _GPIOF
$4001:2000 equ _GPIOG
  $0000 equ gp.CRL               \ Port configuration register low	(GPIOn_CRL)
  $0004 equ gp.CRH               \ Port configuration register high     (GPIOn_CRL)
  $0008 equ gp.IDR               \ Port input data register             (GPIOn_IDR)
  $000C equ gp.ODR               \ Port output data register            (GPIOn_ODR)
  $0010 equ gp.BSRR              \ Port bit set/reset register          (GPIOn_BSRR)
  $0014 equ gp.BRR               \ Port bit reset register              (GPIOn_BRR)
  $0018 equ gp.LCKR              \ Port configuration lock register


\ ======================================
\ AFIO - Alternate function I/O
\ ======================================

$4001:0000 equ _AFIO
  $0000 equ afEVCR               \ Event Control Register          (AFIO_EVCR)
  $0004 equ afMAPR               \ AF remap and debug I/O configuration          register (AFIO_MAPR)
  $0008 equ afEXTICR1            \ External interrupt configuration register 1          (AFIO_EXTICR1)
  $000C equ afEXTICR2            \ External interrupt configuration register 2          (AFIO_EXTICR2)
  $0010 equ afEXTICR3            \ External interrupt configuration register 3          (AFIO_EXTICR3)
  $0014 equ afEXTICR4            \ External interrupt configuration register 4          (AFIO_EXTICR4)
  $001C equ afMAPR2              \ AF remap and debug I/O configuration          register


\ ======================================
\ EXTI - EXTI
\ ======================================

$4001:0400 equ _EXTI
#18 equ TAMPER_IRQn
#22 equ EXTI0_IRQn
#23 equ EXTI1_IRQn
#24 equ EXTI2_IRQn
#25 equ EXTI3_IRQn
#26 equ EXTI4_IRQn
#39 equ EXTI9_5_IRQn
#56 equ EXTI15_10_IRQn
  $0000 equ exti.IMR             \ Interrupt mask register          (EXTI_IMR)
  $0004 equ exti.EMR             \ Event mask register (EXTI_EMR)
  $0008 equ exti.RTSR            \ Rising Trigger selection register          (EXTI_RTSR)
  $000C equ exti.FTSR            \ Falling Trigger selection register          (EXTI_FTSR)
  $0010 equ exti.SWIER           \ Software interrupt event register          (EXTI_SWIER)
  $0014 equ exti.PR              \ Pending register (EXTI_PR)


\ ======================================
\ DMA1 - DMA controller
\ ======================================

$4002:0000 equ _DMA1
#27 equ DMA1_Channel1_IRQn
#28 equ DMA1_Channel2_IRQn
#29 equ DMA1_Channel3_IRQn
#30 equ DMA1_Channel4_IRQn
#31 equ DMA1_Channel5_IRQn
#32 equ DMA1_Channel6_IRQn
#33 equ DMA1_Channel7_IRQn
  $0000 equ dma.ISR              \ DMA interrupt status register          (DMA_ISR)
  $0004 equ dma.IFCR             \ DMA interrupt flag clear register          (DMA_IFCR)
  $0008 equ dma.CCR1             \ DMA channel configuration register          (DMA_CCR)
  $000C equ dma.CNDTR1           \ DMA channel 1 number of data          register
  $0010 equ dma.CPAR1            \ DMA channel 1 peripheral address          register
  $0014 equ dma.CMAR1            \ DMA channel 1 memory address          register
  $001C equ dma.CCR2             \ DMA channel configuration register          (DMA_CCR)
  $0020 equ dma.CNDTR2           \ DMA channel 2 number of data          register
  $0024 equ dma.CPAR2            \ DMA channel 2 peripheral address          register
  $0028 equ dma.CMAR2            \ DMA channel 2 memory address          register
  $0030 equ dma.CCR3             \ DMA channel configuration register          (DMA_CCR)
  $0034 equ dma.CNDTR3           \ DMA channel 3 number of data          register
  $0038 equ dma.CPAR3            \ DMA channel 3 peripheral address          register
  $003C equ dma.CMAR3            \ DMA channel 3 memory address          register
  $0044 equ dma.CCR4             \ DMA channel configuration register          (DMA_CCR)
  $0048 equ dma.CNDTR4           \ DMA channel 4 number of data          register
  $004C equ dma.CPAR4            \ DMA channel 4 peripheral address          register
  $0050 equ dma.CMAR4            \ DMA channel 4 memory address          register
  $0058 equ dma.CCR5             \ DMA channel configuration register          (DMA_CCR)
  $005C equ dma.CNDTR5           \ DMA channel 5 number of data          register
  $0060 equ dma.CPAR5            \ DMA channel 5 peripheral address          register
  $0064 equ dma.CMAR5            \ DMA channel 5 memory address          register
  $006C equ dma.CCR6             \ DMA channel configuration register          (DMA_CCR)
  $0070 equ dma.CNDTR6           \ DMA channel 6 number of data          register
  $0074 equ dma.CPAR6            \ DMA channel 6 peripheral address          register
  $0078 equ dma.CMAR6            \ DMA channel 6 memory address          register
  $0080 equ dma.CCR7             \ DMA channel configuration register          (DMA_CCR)
  $0084 equ dma.CNDTR7           \ DMA channel 7 number of data          register
  $0088 equ dma.CPAR7            \ DMA channel 7 peripheral address          register
  $008C equ dma.CMAR7            \ DMA channel 7 memory address          register


\ ======================================
\ DMA2 - Memory address
\ ======================================

$4002:0400 equ _DMA2
#72 equ DMA2_Channel1_IRQn
#73 equ DMA2_Channel2_IRQn
#74 equ DMA2_Channel3_IRQn
#75 equ DMA2_Channel4_5_IRQn


\ ======================================
\ SDIO - Secure digital input/output      interface
\ ======================================

$4001:8000 equ _SDIO
#65 equ SDIO_IRQn
  $0000 equ sdio.POWER           \ Bits 1:0 = PWRCTRL: Power supply control          bits
  $0004 equ sdio.CLKCR           \ SDI clock control register          (SDIO_CLKCR)
  $0008 equ sdio.ARG             \ Bits 31:0 = : Command argument
  $000C equ sdio.CMD             \ SDIO command register          (SDIO_CMD)
  $0010 equ sdio.RESPCMD         \ SDIO command register
  $0014 equ sdio.RESPI1          \ Bits 31:0 = CARDSTATUS1
  $0018 equ sdio.RESP2           \ Bits 31:0 = CARDSTATUS2
  $001C equ sdio.RESP3           \ Bits 31:0 = CARDSTATUS3
  $0020 equ sdio.RESP4           \ Bits 31:0 = CARDSTATUS4
  $0024 equ sdio.DTIMER          \ Bits 31:0 = DATATIME: Data timeout          period
  $0028 equ sdio.DLEN            \ Bits 24:0 = DATALENGTH: Data length          value
  $002C equ sdio.DCTRL           \ SDIO data control register          (SDIO_DCTRL)
  $0030 equ sdio.DCOUNT          \ Bits 24:0 = DATACOUNT: Data count          value
  $0034 equ sdio.STA             \ SDIO status register          (SDIO_STA)
  $0038 equ sdio.ICR             \ SDIO interrupt clear register          (SDIO_ICR)
  $003C equ sdio.MASK            \ SDIO mask register (SDIO_MASK)
  $0048 equ sdio.FIFOCNT         \ Bits 23:0 = FIFOCOUNT: Remaining number of          words to be written to or read from the          FIFO
  $0080 equ sdio.FIFO            \ bits 31:0 = FIFOData: Receive and transmit          FIFO data


\ ======================================
\ RTC - Real time clock
\ ======================================

$4000:2800 equ _RTC
#19 equ RTC_IRQn
#57 equ RTCAlarm_IRQn
  $0000 equ rtc.CRH              \ RTC Control Register High
  $0004 equ rtc.CRL              \ RTC Control Register Low
  $0008 equ rtc.PRLH             \ RTC Prescaler Load Register          High
  $000C equ rtc.PRLL             \ RTC Prescaler Load Register          Low
  $0010 equ rtc.DIVH             \ RTC Prescaler Divider Register          High
  $0014 equ rtc.DIVL             \ RTC Prescaler Divider Register          Low
  $0018 equ rtc.CNTH             \ RTC Counter Register High
  $001C equ rtc.CNTL             \ RTC Counter Register Low
  $0020 equ rtc.ALRH             \ RTC Alarm Register High
  $0024 equ rtc.ALRL             \ RTC Alarm Register Low


\ ======================================
\ BKP - Backup registers
\ ======================================

$4000:6C04 equ _BKP
  $0000 equ bkp.DR1              \ Backup data register (BKP_DR)
  $0004 equ bkp.DR2              \ Backup data register (BKP_DR)
  $0008 equ bkp.DR3              \ Backup data register (BKP_DR)
  $000C equ bkp.DR4              \ Backup data register (BKP_DR)
  $0010 equ bkp.DR5              \ Backup data register (BKP_DR)
  $0014 equ bkp.DR6              \ Backup data register (BKP_DR)
  $0018 equ bkp.DR7              \ Backup data register (BKP_DR)
  $001C equ bkp.DR8              \ Backup data register (BKP_DR)
  $0020 equ bkp.DR9              \ Backup data register (BKP_DR)
  $0024 equ bkp.DR10             \ Backup data register (BKP_DR)
  $003C equ bkp.DR11             \ Backup data register (BKP_DR)
  $0040 equ bkp.DR12             \ Backup data register (BKP_DR)
  $0044 equ bkp.DR13             \ Backup data register (BKP_DR)
  $0048 equ bkp.DR14             \ Backup data register (BKP_DR)
  $004C equ bkp.DR15             \ Backup data register (BKP_DR)
  $0050 equ bkp.DR16             \ Backup data register (BKP_DR)
  $0054 equ bkp.DR17             \ Backup data register (BKP_DR)
  $0058 equ bkp.DR18             \ Backup data register (BKP_DR)
  $005C equ bkp.DR19             \ Backup data register (BKP_DR)
  $0060 equ bkp.DR20             \ Backup data register (BKP_DR)
  $0064 equ bkp.DR21             \ Backup data register (BKP_DR)
  $0068 equ bkp.DR22             \ Backup data register (BKP_DR)
  $006C equ bkp.DR23             \ Backup data register (BKP_DR)
  $0070 equ bkp.DR24             \ Backup data register (BKP_DR)
  $0074 equ bkp.DR25             \ Backup data register (BKP_DR)
  $0078 equ bkp.DR26             \ Backup data register (BKP_DR)
  $007C equ bkp.DR27             \ Backup data register (BKP_DR)
  $0080 equ bkp.DR28             \ Backup data register (BKP_DR)
  $0084 equ bkp.DR29             \ Backup data register (BKP_DR)
  $0088 equ bkp.DR30             \ Backup data register (BKP_DR)
  $008C equ bkp.DR31             \ Backup data register (BKP_DR)
  $0090 equ bkp.DR32             \ Backup data register (BKP_DR)
  $0094 equ bkp.DR33             \ Backup data register (BKP_DR)
  $0098 equ bkp.DR34             \ Backup data register (BKP_DR)
  $009C equ bkp.DR35             \ Backup data register (BKP_DR)
  $00A0 equ bkp.DR36             \ Backup data register (BKP_DR)
  $00A4 equ bkp.DR37             \ Backup data register (BKP_DR)
  $00A8 equ bkp.DR38             \ Backup data register (BKP_DR)
  $00AC equ bkp.DR39             \ Backup data register (BKP_DR)
  $00B0 equ bkp.DR40             \ Backup data register (BKP_DR)
  $00B4 equ bkp.DR41             \ Backup data register (BKP_DR)
  $00B8 equ bkp.DR42             \ Backup data register (BKP_DR)
  $0028 equ bkp.RTCCR            \ RTC clock calibration register          (BKP_RTCCR)
  $002C equ bkp.CR               \ Backup control register          (BKP_CR)
  $0030 equ bkp.CSR              \ BKP_CSR control/status register          (BKP_CSR)


\ ======================================
\ IWDG - Independent watchdog
\ ======================================

$4000:3000 equ _IWDG
  $0000 equ iwdgKR               \ Key register (IWDG_KR)
  $0004 equ iwdgPR               \ Prescaler register (IWDG_PR)
  $0008 equ iwdgRLR              \ Reload register (IWDG_RLR)
  $000C equ iwdgSR               \ Status register (IWDG_SR)


\ ======================================
\ WWDG - Window watchdog
\ ======================================

$4000:2C00 equ _WWDG
#16 equ WWDG_IRQn
  $0000 equ wwdgCR               \ Control register (WWDG_CR)
  $0004 equ wwdgCFR              \ Configuration register          (WWDG_CFR)
  $0008 equ wwdgSR               \ Status register (WWDG_SR)


\ ======================================
\ TIM1 - Advanced timer
\ ======================================

$4001:2C00 equ _TIM1
#40 equ TIM1_BRK_TIM9_IRQn
#41 equ TIM1_UP_TIM10_IRQn
#42 equ TIM1_TRG_COM_TIM11_IRQn
#43 equ TIM1_CC_IRQn
$4000:0400 equ _TIM3
#45 equ TIM3_IRQn

$4000:0800 equ _TIM4
#46 equ TIM4_IRQn

$4000:0C00 equ _TIM5
#66 equ TIM5_IRQn

$4000:1400 equ _TIM7
#71 equ TIM7_IRQn

$4001:3400 equ _TIM8
#59 equ TIM8_BRK_TIM12_IRQn
#60 equ TIM8_UP_TIM13_IRQn
#61 equ TIM8_TRG_COM_TIM14_IRQn
#62 equ TIM8_CC_IRQn

$4001:5400 equ _TIM11
#42 equ TIM1_TRG_COM_TIM11_IRQn
$4000:1800 equ _TIM12
#59 equ TIM8_BRK_TIM12_IRQn
$4000:1C00 equ _TIM13
#60 equ TIM8_UP_TIM13_IRQn
$4000:2000 equ _TIM14
#61 equ TIM8_TRG_COM_TIM14_IRQn

  $0000 equ tim.CR1              \ control register 1
  $0004 equ tim.CR2              \ control register 2
  $0008 equ tim.SMCR             \ slave mode control register
  $000C equ tim.DIER             \ DMA/Interrupt enable register
  $0010 equ tim.SR               \ status register
  $0014 equ tim.EGR              \ event generation register
  $0018 equ tim.CCMR1_Output     \ capture/compare mode register (output          mode)
  $0018 equ tim.CCMR1_Input      \ capture/compare mode register 1 (input          mode)
  $001C equ tim.CCMR2_Output     \ capture/compare mode register (output          mode)
  $001C equ tim.CCMR2_Input      \ capture/compare mode register 2 (input          mode)
  $0020 equ tim.CCER             \ capture/compare enable          register
  $0024 equ tim.CNT              \ counter
  $0028 equ tim.PSC              \ prescaler
  $002C equ tim.ARR              \ auto-reload register
  $0034 equ tim.CCR1             \ capture/compare register 1
  $0038 equ tim.CCR2             \ capture/compare register 2
  $003C equ tim.CCR3             \ capture/compare register 3
  $0040 equ tim.CCR4             \ capture/compare register 4
  $0048 equ tim.DCR              \ DMA control register
  $004C equ tim.DMAR             \ DMA address for full transfer
  $0030 equ tim.RCR              \ repetition counter register
  $0044 equ tim.BDTR             \ break and dead-time register


\ ======================================
\ TIM2 - General purpose timer
\ ======================================

$4000:0000 equ _TIM2
#44 equ TIM2_IRQn
  $0000 equ tim.CR1                  \ control register 1
  $0004 equ tim.CR2                  \ control register 2
  $0008 equ tim.SMCR                 \ slave mode control register
  $000C equ tim.DIER                 \ DMA/Interrupt enable register
  $0010 equ tim.SR                   \ status register
  $0014 equ tim.EGR                  \ event generation register
  $0018 equ tim.CCMR1_Output         \ capture/compare mode register 1 (output          mode)
  $0018 equ tim.CCMR1_Input          \ capture/compare mode register 1 (input          mode)
  $001C equ tim.CCMR2_Output         \ capture/compare mode register 2 (output          mode)
  $001C equ tim.CCMR2_Input          \ capture/compare mode register 2 (input          mode)
  $0020 equ tim.CCER                 \ capture/compare enable          register
  $0024 equ tim.CNT                  \ counter
  $0028 equ tim.PSC                  \ prescaler
  $002C equ tim.ARR                  \ auto-reload register
  $0034 equ tim.CCR1                 \ capture/compare register 1
  $0038 equ tim.CCR2                 \ capture/compare register 2
  $003C equ tim.CCR3                 \ capture/compare register 3
  $0040 equ tim.CCR4                 \ capture/compare register 4
  $0048 equ tim.DCR                  \ DMA control register
  $004C equ tim.DMAR                 \ DMA address for full transfer


\ ======================================
\ TIM9 - General purpose timer
\ ======================================

$4001:4C00 equ _TIM9
#40 equ TIM1_BRK_TIM9_IRQn
  $0000 equ tim.CR1                  \ control register 1
  $0004 equ tim.CR2                  \ control register 2
  $0008 equ tim.SMCR                 \ slave mode control register
  $000C equ tim.DIER                 \ DMA/Interrupt enable register
  $0010 equ tim.SR                   \ status register
  $0014 equ tim.EGR                  \ event generation register
  $0018 equ tim.CCMR1_Output         \ capture/compare mode register 1 (output          mode)
  $0018 equ tim.CCMR1_Input          \ capture/compare mode register 1 (input          mode)
  $0020 equ tim.CCER                 \ capture/compare enable          register
  $0024 equ tim.CNT                  \ counter
  $0028 equ tim.PSC                  \ prescaler
  $002C equ tim.ARR                  \ auto-reload register
  $0034 equ tim.CCR1                 \ capture/compare register 1
  $0038 equ tim.CCR2                 \ capture/compare register 2


\ ======================================
\ TIM10 - General purpose timer
\ ======================================

$4001:5000 equ _TIM10
#41 equ TIM1_UP_TIM10_IRQn
  $0000 equ tim.CR1                  \ control register 1
  $0004 equ tim.CR2                  \ control register 2
  $000C equ tim.DIER                 \ DMA/Interrupt enable register
  $0010 equ tim.SR                   \ status register
  $0014 equ tim.EGR                  \ event generation register
  $0018 equ tim.CCMR1_Output         \ capture/compare mode register (output          mode)
  $0018 equ tim.CCMR1_Input          \ capture/compare mode register (input          mode)
  $0020 equ tim.CCER                 \ capture/compare enable          register
  $0024 equ tim.CNT                  \ counter
  $0028 equ tim.PSC                  \ prescaler
  $002C equ tim.ARR                  \ auto-reload register
  $0034 equ tim.CCR1                 \ capture/compare register 1


\ ======================================
\ TIM6 - Basic timer
\ ======================================

$4000:1000 equ _TIM6
#70 equ TIM6_IRQn
  $0000 equ tim.CR1                  \ control register 1
  $0004 equ tim.CR2                  \ control register 2
  $000C equ tim.DIER                 \ DMA/Interrupt enable register
  $0010 equ tim.SR                   \ status register
  $0014 equ tim.EGR                  \ event generation register
  $0024 equ tim.CNT                  \ counter
  $0028 equ tim.PSC                  \ prescaler
  $002C equ tim.ARR                  \ auto-reload register


\ ======================================
\ I2C1 - Inter integrated circuit
\ ======================================

$4000:5400 equ _I2C1
#47 equ I2C1_EV_IRQn
#48 equ I2C1_ER_IRQn
$4000:5800 equ _I2C2
#49 equ I2C2_EV_IRQn
#50 equ I2C2_ER_IRQn
  $0000 equ i2c.CR1              \ Control register 1
  $0004 equ i2c.CR2              \ Control register 2
  $0008 equ i2c.OAR1             \ Own address register 1
  $000C equ i2c.OAR2             \ Own address register 2
  $0010 equ i2c.DR               \ Data register
  $0014 equ i2c.SR1              \ Status register 1
  $0018 equ i2c.SR2              \ Status register 2
  $001C equ i2c.CCR              \ Clock control register
  $0020 equ i2c.TRISE            \ TRISE register


\ ======================================
\ SPI1 - Serial peripheral interface
\ ======================================

$4001:3000 equ _SPI1
#51 equ SPI1_IRQn
$4000:3800 equ _SPI2
#52 equ SPI2_IRQn
$4000:3C00 equ _SPI3
#67 equ SPI3_IRQn
  $0000 equ spi.CR1              \ control register 1
  $0004 equ spi.CR2              \ control register 2
  $0008 equ spi.SR               \ status register
  $000C equ spi.DR               \ data register
  $0010 equ spi.CRCPR            \ CRC polynomial register
  $0014 equ spi.RXCRCR           \ RX CRC register
  $0018 equ spi.TXCRCR           \ TX CRC register
  $001C equ spi.I2SCFGR          \ I2S configuration register
  $0020 equ spi.I2SPR            \ I2S prescaler register


\ ======================================
\ USART1 - Universal synchronous asynchronous receiver      transmitter
\ ======================================

$4001:3800 equ _USART1
#53 equ USART1_IRQn
#53 equ USART1vec#
$4000:4400 equ _USART2
#54 equ USART2_IRQn
$4000:4800 equ _USART3
#55 equ USART3_IRQn
$4000:4C00 equ _UART4
#68 equ UART4_IRQn
$4000:5000 equ _UART5
#69 equ UART5_IRQn
  $0000 equ usSR                \ Status register
  $0004 equ usDR		\ TX and RX data regs
    $0004 equ usTDR
    $0004 equ usRDR
  $0008 equ usBRR               \ Baud rate register
  $000C equ usCR1               \ Control register 1
  $0010 equ usCR2               \ Control register 2
  $0014 equ usCR3               \ Control register 3
  $0018 equ usGTPR              \ Guard time and prescaler register


\ ======================================
\ ADC1 - Analog to digital converter
\ ======================================

$4001:2400 equ _ADC1
#34 equ ADC_IRQn
  $0000 equ adc.SR               \ status register
  $0004 equ adc.CR1              \ control register 1
  $0008 equ adc.CR2              \ control register 2
  $000C equ adc.SMPR1            \ sample time register 1
  $0010 equ adc.SMPR2            \ sample time register 2
  $0014 equ adc.JOFR1            \ injected channel data offset register          x
  $0018 equ adc.JOFR2            \ injected channel data offset register          x
  $001C equ adc.JOFR3            \ injected channel data offset register          x
  $0020 equ adc.JOFR4            \ injected channel data offset register          x
  $0024 equ adc.HTR              \ watchdog higher threshold register
  $0028 equ adc.LTR              \ watchdog lower threshold  register
  $002C equ adc.SQR1             \ regular sequence register 1
  $0030 equ adc.SQR2             \ regular sequence register 2
  $0034 equ adc.SQR3             \ regular sequence register 3
  $0038 equ adc.JSQR             \ injected sequence register
  $003C equ adc.JDR1             \ injected data register x
  $0040 equ adc.JDR2             \ injected data register x
  $0044 equ adc.JDR3             \ injected data register x
  $0048 equ adc.JDR4             \ injected data register x
  $004C equ adc.DR               \ regular data register


\ ======================================
\ ADC2 - Analog to digital converter
\ ======================================

$4001:2800 equ _ADC2
#34 equ ADC_IRQn
$4001:3C00 equ _ADC3
#63 equ ADC3_IRQn
  $0000 equ adc.SR                   \ status register
  $0004 equ adc.CR1                  \ control register 1
  $0008 equ adc.CR2                  \ control register 2
  $000C equ adc.SMPR1                \ sample time register 1
  $0010 equ adc.SMPR2                \ sample time register 2
  $0014 equ adc.JOFR1                \ injected channel data offset register          x
  $0018 equ adc.JOFR2                \ injected channel data offset register          x
  $001C equ adc.JOFR3                \ injected channel data offset register          x
  $0020 equ adc.JOFR4                \ injected channel data offset register          x
  $0024 equ adc.HTR                  \ watchdog higher threshold          register
  $0028 equ adc.LTR                  \ watchdog lower threshold          register
  $002C equ adc.SQR1                 \ regular sequence register 1
  $0030 equ adc.SQR2                 \ regular sequence register 2
  $0034 equ adc.SQR3                 \ regular sequence register 3
  $0038 equ adc.JSQR                 \ injected sequence register
  $003C equ adc.JDR1                 \ injected data register x
  $0040 equ adc.JDR2                 \ injected data register x
  $0044 equ adc.JDR3                 \ injected data register x
  $0048 equ adc.JDR4                 \ injected data register x
  $004C equ adc.DR                   \ regular data register


\ ======================================
\ CAN - Controller area network
\ ======================================

$4000:6400 equ _CAN
#35 equ CAN1_TX_IRQn
#36 equ CAN1_RX0_IRQn
#37 equ CAN1_RX1_IRQn
#38 equ CAN1_SCE_IRQn
  $0000 equ can.CAN_MCR          \ CAN_MCR
  $0004 equ can.CAN_MSR          \ CAN_MSR
  $0008 equ can.CAN_TSR          \ CAN_TSR
  $000C equ can.CAN_RF0R         \ CAN_RF0R
  $0010 equ can.CAN_RF1R         \ CAN_RF1R
  $0014 equ can.CAN_IER          \ CAN_IER
  $0018 equ can.CAN_ESR          \ CAN_ESR
  $001C equ can.CAN_BTR          \ CAN_BTR
  $0180 equ can.CAN_TI0R         \ CAN_TI0R
  $0184 equ can.CAN_TDT0R        \ CAN_TDT0R
  $0188 equ can.CAN_TDL0R        \ CAN_TDL0R
  $018C equ can.CAN_TDH0R        \ CAN_TDH0R
  $0190 equ can.CAN_TI1R         \ CAN_TI1R
  $0194 equ can.CAN_TDT1R        \ CAN_TDT1R
  $0198 equ can.CAN_TDL1R        \ CAN_TDL1R
  $019C equ can.CAN_TDH1R        \ CAN_TDH1R
  $01A0 equ can.CAN_TI2R         \ CAN_TI2R
  $01A4 equ can.CAN_TDT2R        \ CAN_TDT2R
  $01A8 equ can.CAN_TDL2R        \ CAN_TDL2R
  $01AC equ can.CAN_TDH2R        \ CAN_TDH2R
  $01B0 equ can.CAN_RI0R         \ CAN_RI0R
  $01B4 equ can.CAN_RDT0R        \ CAN_RDT0R
  $01B8 equ can.CAN_RDL0R        \ CAN_RDL0R
  $01BC equ can.CAN_RDH0R        \ CAN_RDH0R
  $01C0 equ can.CAN_RI1R         \ CAN_RI1R
  $01C4 equ can.CAN_RDT1R        \ CAN_RDT1R
  $01C8 equ can.CAN_RDL1R        \ CAN_RDL1R
  $01CC equ can.CAN_RDH1R        \ CAN_RDH1R
  $0200 equ can.CAN_FMR          \ CAN_FMR
  $0204 equ can.CAN_FM1R         \ CAN_FM1R
  $020C equ can.CAN_FS1R         \ CAN_FS1R
  $0214 equ can.CAN_FFA1R        \ CAN_FFA1R
  $021C equ can.CAN_FA1R         \ CAN_FA1R
  $0240 equ can.F0R1             \ Filter bank 0 register 1
  $0244 equ can.F0R2             \ Filter bank 0 register 2
  $0248 equ can.F1R1             \ Filter bank 1 register 1
  $024C equ can.F1R2             \ Filter bank 1 register 2
  $0250 equ can.F2R1             \ Filter bank 2 register 1
  $0254 equ can.F2R2             \ Filter bank 2 register 2
  $0258 equ can.F3R1             \ Filter bank 3 register 1
  $025C equ can.F3R2             \ Filter bank 3 register 2
  $0260 equ can.F4R1             \ Filter bank 4 register 1
  $0264 equ can.F4R2             \ Filter bank 4 register 2
  $0268 equ can.F5R1             \ Filter bank 5 register 1
  $026C equ can.F5R2             \ Filter bank 5 register 2
  $0270 equ can.F6R1             \ Filter bank 6 register 1
  $0274 equ can.F6R2             \ Filter bank 6 register 2
  $0278 equ can.F7R1             \ Filter bank 7 register 1
  $027C equ can.F7R2             \ Filter bank 7 register 2
  $0280 equ can.F8R1             \ Filter bank 8 register 1
  $0284 equ can.F8R2             \ Filter bank 8 register 2
  $0288 equ can.F9R1             \ Filter bank 9 register 1
  $028C equ can.F9R2             \ Filter bank 9 register 2
  $0290 equ can.F10R1            \ Filter bank 10 register 1
  $0294 equ can.F10R2            \ Filter bank 10 register 2
  $0298 equ can.F11R1            \ Filter bank 11 register 1
  $029C equ can.F11R2            \ Filter bank 11 register 2
  $02A0 equ can.F12R1            \ Filter bank 4 register 1
  $02A4 equ can.F12R2            \ Filter bank 12 register 2
  $02A8 equ can.F13R1            \ Filter bank 13 register 1
  $02AC equ can.F13R2            \ Filter bank 13 register 2


\ ======================================
\ DAC - Digital to analog converter
\ ======================================

$4000:7400 equ _DAC
  $0000 equ dac.CR               \ Control register (DAC_CR)
  $0004 equ dac.SWTRIGR          \ DAC software trigger register          (DAC_SWTRIGR)
  $0008 equ dac.DHR12R1          \ DAC channel1 12-bit right-aligned data          holding register(DAC_DHR12R1)
  $000C equ dac.DHR12L1          \ DAC channel1 12-bit left aligned data          holding register (DAC_DHR12L1)
  $0010 equ dac.DHR8R1           \ DAC channel1 8-bit right aligned data          holding register (DAC_DHR8R1)
  $0014 equ dac.DHR12R2          \ DAC channel2 12-bit right aligned data          holding register (DAC_DHR12R2)
  $0018 equ dac.DHR12L2          \ DAC channel2 12-bit left aligned data          holding register (DAC_DHR12L2)
  $001C equ dac.DHR8R2           \ DAC channel2 8-bit right-aligned data          holding register (DAC_DHR8R2)
  $0020 equ dac.DHR12RD          \ Dual DAC 12-bit right-aligned data holding          register (DAC_DHR12RD), Bits 31:28 Reserved, Bits 15:12          Reserved
  $0024 equ dac.DHR12LD          \ DUAL DAC 12-bit left aligned data holding          register (DAC_DHR12LD), Bits 19:16 Reserved, Bits 3:0          Reserved
  $0028 equ dac.DHR8RD           \ DUAL DAC 8-bit right aligned data holding          register (DAC_DHR8RD), Bits 31:16 Reserved
  $002C equ dac.DOR1             \ DAC channel1 data output register          (DAC_DOR1)
  $0030 equ dac.DOR2             \ DAC channel2 data output register          (DAC_DOR2)


\ ======================================
\ DBG - Debug support
\ ======================================

$E004:2000 equ _DBG
  $0000 equ dbg.IDCODE           \ DBGMCU_IDCODE
  $0004 equ dbg.CR               \ DBGMCU_CR


\ ======================================
\ CRC - CRC calculation unit
\ ======================================

$4002:3000 equ _CRC
  $0000 equ crcDR                \ Data register
  $0004 equ crcIDR               \ Independent Data register
  $0008 equ crcCR                \ Control register


\ ======================================
\ FLASH - FLASH
\ ======================================

$4002:2000 equ _FLASH
$4002:2000 equ _FPEC
#20 equ FLASH_IRQn
  $0000 equ flACR                \ Flash access control register
  $0004 equ flKEYR               \ Flash key register
  $0008 equ flOPTKEYR            \ Flash option key register
  $000C equ flSR                 \ Status register
  $0010 equ flCR                 \ Control register
  $0014 equ flAR                 \ Flash address register
  $001C equ flOBR                \ Option byte register
  $0020 equ flWRPR               \ Write protection register


\ ======================================
\ NVIC - Nested Vectored Interrupt      Controller
\ ======================================

$E000:E000 equ _NVIC
  $0004 equ nvic.ICTR            \ Interrupt Controller Type          Register
  $0F00 equ nvic.STIR            \ Software Triggered Interrupt          Register
  $0100 equ nvic.ISER0           \ Interrupt Set-Enable Register
  $0104 equ nvic.ISER1           \ Interrupt Set-Enable Register
  $0180 equ nvic.ICER0           \ Interrupt Clear-Enable          Register
  $0184 equ nvic.ICER1           \ Interrupt Clear-Enable          Register
  $0200 equ nvic.ISPR0           \ Interrupt Set-Pending Register
  $0204 equ nvic.ISPR1           \ Interrupt Set-Pending Register
  $0280 equ nvic.ICPR0           \ Interrupt Clear-Pending          Register
  $0284 equ nvic.ICPR1           \ Interrupt Clear-Pending          Register
  $0300 equ nvic.IABR0           \ Interrupt Active Bit Register
  $0304 equ nvic.IABR1           \ Interrupt Active Bit Register
  $0400 equ nvic.IPR0            \ Interrupt Priority Register
  $0404 equ nvic.IPR1            \ Interrupt Priority Register
  $0408 equ nvic.IPR2            \ Interrupt Priority Register
  $040C equ nvic.IPR3            \ Interrupt Priority Register
  $0410 equ nvic.IPR4            \ Interrupt Priority Register
  $0414 equ nvic.IPR5            \ Interrupt Priority Register
  $0418 equ nvic.IPR6            \ Interrupt Priority Register
  $041C equ nvic.IPR7            \ Interrupt Priority Register
  $0420 equ nvic.IPR8            \ Interrupt Priority Register
  $0424 equ nvic.IPR9            \ Interrupt Priority Register
  $0428 equ nvic.IPR10           \ Interrupt Priority Register
  $042C equ nvic.IPR11           \ Interrupt Priority Register
  $0430 equ nvic.IPR12           \ Interrupt Priority Register
  $0434 equ nvic.IPR13           \ Interrupt Priority Register
  $0438 equ nvic.IPR14           \ Interrupt Priority Register


\ ======================================
\ USB - Universal serial bus full-speed device      interface
\ ======================================

$4000:5C00 equ _USB
#58 equ USB_FS_WKUP_IRQn
  $0000 equ usbEP0R              \ endpoint 0 register
  $0004 equ usbEP1R              \ endpoint 1 register
  $0008 equ usbEP2R              \ endpoint 2 register
  $000C equ usbEP3R              \ endpoint 3 register
  $0010 equ usbEP4R              \ endpoint 4 register
  $0014 equ usbEP5R              \ endpoint 5 register
  $0018 equ usbEP6R              \ endpoint 6 register
  $001C equ usbEP7R              \ endpoint 7 register
  $0040 equ usbCNTR              \ control register
  $0044 equ usbISTR              \ interrupt status register
  $0048 equ usbFNR               \ frame number register
  $004C equ usbDADDR             \ device address
  $0050 equ usbBTABLE            \ Buffer table address