\ sfrSTM32F10x.fth - peripheral definitions for STM32

((
Copyright (c) 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal


\ ===========
\ *! sfrstm32f10x
\ *T STM32F10x memory and peripherals
\ ===========

\ ******
\ Family
\ ******

1 equ STM32lowd		\ low density devices
2 equ STM32medd		\ medium density devices
3 equ STM32highd	\ high density devices
4 equ STM32conn		\ connectivity device


\ *************************
\ Peripheral base addresses
\ *************************

\ Peripheral and SRAM base address in the bit-band alias region
#define _PERIPH_BB        0x42000000
#define _SRAM_BB          0x22000000

\ Peripheral and SRAM base address in the primary region
#define _SRAM             0x20000000
#define _PERIPH           0x40000000

\ FSMC registers base address
#define _FSMC             0xA0000000

\ Peripheral memory map
#define _APB1      _PERIPH
#define _APB2      _PERIPH 0x10000 +
#define _AHB       _PERIPH 0x20000 +

#define _TIM2             _APB1 0x0000 +
#define _TIM3             _APB1 0x0400 +
#define _TIM4             _APB1 0x0800 +
#define _TIM5             _APB1 0x0C00 +
#define _TIM6             _APB1 0x1000 +
#define _TIM7             _APB1 0x1400 +
#define _RTC              _APB1 0x2800 +
#define _WWDG             _APB1 0x2C00 +
#define _SPI2             _APB1 0x3800 +
#define _SPI3             _APB1 0x3C00 +
#define _I2C1             _APB1 0x5400 +
#define _I2C2             _APB1 0x5800 +
#define _CAN              _APB1 0x6400 +
#define _BKP              _APB1 0x6C00 +
#define _PWR              _APB1 0x7000 +
#define _DAC              _APB1 0x7400 +

#define _EXTI             _APB2 0x0400 +
#define _ADC1             _APB2 0x2400 +
#define _ADC2             _APB2 0x2800 +
#define _TIM1             _APB2 0x2C00 +
#define _SPI1             _APB2 0x3000 +
#define _TIM8             _APB2 0x3400 +
#define _ADC3             _APB2 0x3C00 +

#define _SDIO             _PERIPH 0x18000 +

#define _DMA1             _AHB 0x0000 +
#define _DMA1_Channel1    _AHB 0x0008 +
#define _DMA1_Channel2    _AHB 0x001C +
#define _DMA1_Channel3    _AHB 0x0030 +
#define _DMA1_Channel4    _AHB 0x0044 +
#define _DMA1_Channel5    _AHB 0x0058 +
#define _DMA1_Channel6    _AHB 0x006C +
#define _DMA1_Channel7    _AHB 0x0080 +
#define _DMA2             _AHB 0x0400 +
#define _DMA2_Channel1    _AHB 0x0408 +
#define _DMA2_Channel2    _AHB 0x041C +
#define _DMA2_Channel3    _AHB 0x0430 +
#define _DMA2_Channel4    _AHB 0x0444 +
#define _DMA2_Channel5    _AHB 0x0458 +

#define _CRC              _AHB 0x3000 +

\ Flash Option Bytes base address
#define _OB               0x1FFFF800

\ FSMC Bankx registers base address
#define _FSMC_Bank1_R     _FSMC 0x0000 +
#define _FSMC_Bank1E_R    _FSMC 0x0104 +
#define _FSMC_Bank2_R     _FSMC 0x0060 +
#define _FSMC_Bank3_R     _FSMC 0x0080 +
#define _FSMC_Bank4_R     _FSMC 0x00A0 +

\ Debug MCU registers base address
#define _DBGMCU          0xE0042000


\ ********************
\ CRC calculation unit
\ ********************

$4002:3000 equ _CRC		\ _AHB1 0x3000 +
  $00 equ crcDR
  $04 equ crcIDR
  $08 equ crcCR


\ *******************
\ PWR - Power Control
\ *******************

$4000:7000 equ _PWR		\ _APB1 0x7000 +
  $00 equ pwrCR
  $04 equ pwrCSR


\ ***********************
\ Reset and Clock Control
\ ***********************

#define _RCC              _AHB 0x1000 +
\ RCC base address

\ Register offsets
$00 equ rccCR		\ Control Reg
$04 equ rccCFGR		\ Clock Configuration
$08 equ rccCIR		\ Clock Interrupt
$0C equ rccAPB2rst	\ ABP2 reset
$10 equ rccAPB1rst	\ APB1 reset
$14 equ rccAHBen	\ AHB clock enable
$18 equ rccAPB2en	\ APB2 clock enable
$1C equ rccAPB1en	\ APB1 clock enable
$20 equ rccBDCR		\ Backup Domain Control Reg
$24 equ rccCSR		\ Control and Status Reg


\ **********************
\ Alternate function I/O
\ **********************

\ Base address
#define _AFIO             _APB2 0x0000 +

\ Register offsets
$00 equ afEVCR
$04 equ afMAPR
$08 equ afEXTICR1
$0C equ afEXTICR2
$10 equ afEXTICR3
$14 equ afEXTICR4


\ ********************
\ Independent Watchdog
\ ********************

#define _IWDG             _APB1 0x3000 +

  $00 equ iwdgKR
  $04 equ iwdgPR
  $08 equ iwdgRLR
  $0C equ iwdgSR


\ ***********************
\ Flash memory controller
\ ***********************

\ Flash Program Erase Controller (FPEC) registers base address
#define _FPEC            _AHB 0x2000 +

\ Register offsets
$00 equ flACR
$04 equ flKEYR
$08 equ flOPTKEYR
$0C equ flSR
$10 equ flCR
$14 equ flAR

$1C equ flOBR
$20 equ flWRPR

$00A5 equ flRDPRTkey	\ keys needed to write registers
$45670123 equ flKEY1
$CDEF89AB equ flKEY2


\ ***********
\ GPIO Blocks
\ ***********

\ Base addresses
#define _GPIOA            _APB2 0x0800 +
#define _GPIOB            _APB2 0x0C00 +
#define _GPIOC            _APB2 0x1000 +
#define _GPIOD            _APB2 0x1400 +
#define _GPIOE            _APB2 0x1800 +
#define _GPIOF            _APB2 0x1C00 +
#define _GPIOG            _APB2 0x2000 +

\ Register offsets
$00 equ gp.CRL		\ Port configuration low, bits 0..7
$04 equ gp.CRH		\ Port configuration high, bits 8..15
$08 equ gp.IDR
$0C equ gp.ODR
$10 equ gp.BSRR
$14 equ gp.BRR
$18 equ gp.LCKR


\ ************
\ USART blocks
\ ************

#define _USART1           _APB2 0x3800 +
#define _USART2           _APB1 0x4400 +
#define _USART3           _APB1 0x4800 +
#define _UART4            _APB1 0x4C00 +
#define _UART5            _APB1 0x5000 +

$00 equ usSR		\ status reg
$04 equ usDR		\ TX and RX data regs
  $04 equ usTDR
  $04 equ usRDR
$08 equ usBRR		\ baud rate divisor
$0C equ usCR1		\ control reg 1..3
$10 equ usCR2
$14 equ usCR3
$18 equ usGPTR		\ guard time and prescaler


\ ************************
\ Exception vector numbers
\ ************************
\ Equates of the form xxxVec# refer to Exception Vector position,
\ whereas those of the form xxxInt# refer to Interrupt#.
\ Nearly all MPE code uses the Vec# form. To convert to Int#,
\ subtract 16 from the Vec#.

#16 equ WWDGvec#
#17 equ PVDvec#
#18 equ TamperVec#
#19 equ RTCvec#
#20 equ FlashVec#
#21 equ RCCvec#
#22 equ ExtI0Vec#
#23 equ ExtI1Vec#
#24 equ ExtI2Vec#
#25 equ ExtI3Vec#
#26 equ ExtI4Vec#
#27 equ DMAC1Vec#
#28 equ DMAC2Vec#
#29 equ DMAC3Vec#
#30 equ DMAC4Vec#
#31 equ DMAC5Vec#
#32 equ DMAC6Vec#
#33 equ DMAC7Vec#
#34 equ ADCvec#
#35 equ USB_HPvec#	#35 equ CAN_TXvec#
#36 equ USB_LPvec#	#36 equ CAN_RX0vec#
#37 equ CAN_RX1vec#
#38 equ CAN_SCEvec#
#39 equ ExtI9_5vec#
#40 equ TIM1_BRKvec#
#41 equ TIM1_UPvec#
#42 equ TIM1_TRGvec#
#43 equ TIM1_CCvec#
#44 equ TIM2vec#
#45 equ TIM3vec#
#46 equ TIM4vec#
#47 equ I2C1_EVvec#
#48 equ I2C1_ERvec#
#49 equ I2C2_EVvec#
#50 equ I2C2_ERvec#
#51 equ SPI1vec#
#52 equ SPI2vec#
#53 equ USART1vec#
#54 equ USART2vec#
#55 equ USART3vec#
#56 equ ExtI15_10vec#
#57 equ RTCalarmVec#
#58 equ USBwakeVec#


\ ***********************
\ *S Miscellaneous macros
\ ***********************

interpreter
: kHz		\ n -- Hz
\ *G Multiply by 1000.
  #1000 *  ;
: MHz		\ n -- Hz
\ *G Multiply by 1,000,000.
  #1000000 *  ;

: setMask	\ value mask addr -- ; cell operation
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;
target


\ ======
\ *> ###
\ ======

decimal
