\ sfrSTM32F10xConn.fth - peripheral definitions for STM32F105/7 Connectivity

((
Copyright (c) 2010, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal


\ ===========
\ *! sfrstm32f10xconn
\ *T STM32F105/7 memory and peripherals
\ ===========

\ ******
\ Family
\ ******

4 equ STM32conn		\ connectivity device


\ ********************************
\ Memory/Peripheral base addresses
\ ********************************

$0800:0000 equ _FLASH		\ up to 1 Mb

$1FFF:B000 equ _SYSTEM		\ 18 kb    to 1FFF:F7FF
$1FFF:F800 equ _OPTION		\ 16 bytes to 1FFF:F80F

$2000:0000 equ _SRAM		\ legacy name

$4000:0000 equ _PERIPH		\ legacy name

$4200:0000 equ _PERIPH_BB       \ bit-band region
$2200:0000 equ _SRAM_BB         \ bit-band region


\ ******************************
\ Misc peripheral base addresses
\ ******************************

\ FSMC registers base address
$A000:0000 equ _FSMC

\ Peripheral memory map
#define _APB1      _PERIPH
#define _APB2      _PERIPH 0x10000 +
#define _AHB       _PERIPH 0x20000 +

\ #define _SPI2             _APB1 0x3800 +
\ #define _SPI3             _APB1 0x3C00 +
#define _I2C1             _APB1 0x5400 +
#define _I2C2             _APB1 0x5800 +
#define _CAN              _APB1 0x6400 +
#define _BKP              _APB1 0x6C00 +
#define _DAC              _APB1 0x7400 +

\ #define _EXTI             _APB2 0x0400 +
#define _ADC1             _APB2 0x2400 +
#define _ADC2             _APB2 0x2800 +
\ #define _TIM1             _APB2 0x2C00 +
\ #define _SPI1             _APB2 0x3000 +
\ #define _TIM8             _APB2 0x3400 +
#define _ADC3             _APB2 0x3C00 +

#define _SDIO             _PERIPH 0x18000 +

\ #define _DMA1             _AHB 0x0000 +
\ #define _DMA1_Channel1    _AHB 0x0008 +
\ #define _DMA1_Channel2    _AHB 0x001C +
\ #define _DMA1_Channel3    _AHB 0x0030 +
\ #define _DMA1_Channel4    _AHB 0x0044 +
\ #define _DMA1_Channel5    _AHB 0x0058 +
\ #define _DMA1_Channel6    _AHB 0x006C +
\ #define _DMA1_Channel7    _AHB 0x0080 +
\ #define _DMA2             _AHB 0x0400 +
\ #define _DMA2_Channel1    _AHB 0x0408 +
\ #define _DMA2_Channel2    _AHB 0x041C +
\ #define _DMA2_Channel3    _AHB 0x0430 +
\ #define _DMA2_Channel4    _AHB 0x0444 +
\ #define _DMA2_Channel5    _AHB 0x0458 +

\ #define _CRC              _AHB 0x3000 +

\ Flash Option Bytes base address
#define _OB               0x1FFFF800

\ FSMC Bankx registers base address
#define _FSMC_Bank1_R     _FSMC 0x0000 +
#define _FSMC_Bank1E_R    _FSMC 0x0104 +
#define _FSMC_Bank2_R     _FSMC 0x0060 +
#define _FSMC_Bank3_R     _FSMC 0x0080 +
#define _FSMC_Bank4_R     _FSMC 0x00A0 +

\ Debug MCU registers base address
#define _DBGMCU          0xE0042000


\ ***
\ DMA
\ ***

$4002:0000 equ _DMA1
$4002:0400 equ _DMA2
  $000 equ dmaISR		\ interrupt status
  $004 equ dmaIFCR		\ interrupt flag clear (write 1)
\ channel 1 - 20 bytes per DMA channel
  $008 equ dmaCCR1
  $00C equ dmaCNDTR1
  $010 equ dmaCPAR1
  $014 equ dmaCMAR1
\ channel 2
  $01C equ dmaCCR2
  $020 equ dmaCNDTR2
  $024 equ dmaCPAR2
  $028 equ dmaCMAR2
\ channel 3
  $030 equ dmaCCR3
  $034 equ dmaCNDTR3
  $038 equ dmaCPAR3
  $03C equ dmaCMAR3
\ channel 4
  $044 equ dmaCCR4
  $048 equ dmaCNDTR4
  $04C equ dmaCPAR4
  $050 equ dmaCMAR4
\ channel 5
  $058 equ dmaCCR5
  $05C equ dmaCNDTR5
  $060 equ dmaCPAR5
  $064 equ dmaCMAR5
\ channel 6
  $06C equ dmaCCR6
  $070 equ dmaCNDTR6
  $074 equ dmaCPAR6
  $078 equ dmaCMAR6
\ channel 7
  $080 equ dmaCCR7
  $084 equ dmaCNDTR7
  $088 equ dmaCPAR7
  $08C equ dmaCMAR7


\ ********************
\ CRC calculation unit
\ ********************

$4002:3000 equ _CRC		\ _AHB1 0x3000 +
  $00 equ crcDR
  $04 equ crcIDR
  $08 equ crcCR


\ *******************
\ PWR - Power Control
\ *******************

$4000:7000 equ _PWR		\ _APB1 0x7000 +
  $00 equ pwrCR
  $04 equ pwrCSR


\ ***********************
\ Reset and Clock Control
\ ***********************

$4002:1000 equ _RCC		\ AHB

\ Register offsets
$00 equ rccCR		\ Control Reg
$04 equ rccCFGR		\ Clock Configuration
$08 equ rccCIR		\ Clock Interrupt
$0C equ rccAPB2rst	\ ABP2 reset
$10 equ rccAPB1rst	\ APB1 reset
$14 equ rccAHBen	\ AHB clock enable
$18 equ rccAPB2en	\ APB2 clock enable
$1C equ rccAPB1en	\ APB1 clock enable
$20 equ rccBDCR		\ Backup Domain Control Reg
$24 equ rccCSR		\ Control and Status Reg
$28 equ rccAHBrst	\ AHB reset Reg, bit14=Ether, bit12=USB_OTG
$2C equ rccCFGR2	\ Clock Config Reg 2


\ **********************
\ Alternate function I/O
\ **********************

\ Base address
$4001:0000 equ _AFIO		\ APB2

\ Register offsets
$00 equ afEVCR
$04 equ afMAPR
$08 equ afEXTICR1
$0C equ afEXTICR2
$10 equ afEXTICR3
$14 equ afEXTICR4

$1C equ afMAPR2


\ ********************
\ Independent Watchdog
\ ********************

$4000:3000 equ _IWDG		\ _APB1

  $00 equ iwdgKR
  $04 equ iwdgPR
  $08 equ iwdgRLR
  $0C equ iwdgSR


\ *****************
\ Windowed Watchdog
\ *****************

$4000:2C00 equ _WWDG		\ _APB1 0x2C00 +
  $00 equ wwdgCR
  $04 equ wwdgCFR
  $08 equ wwdgSR


\ ===
\ RTC
\ ===

$4000:2800 equ _RTC		\ _APB1 + 0x2800
  $000 equ rtcCRH		\ Control Reg High
  $004 equ rtcCRL		\ Control Reg Low
  $008 equ rtcPRLH		\ Prescaler Load Reg high
  $00C equ rtcPRLL		\ Prescaler Load Reg Low
  $010 equ rtcDIVH		\ Prescaler Divide Reg High
  $014 equ rtcDIVL		\ Prescaler Divide Reg Low
  $018 equ rtcCNTH		\ Counter Reg High
  $01C equ rtcCNTL		\ Counter Reg Low
  $020 equ rtcALRH		\ Alarm Reg High
  $024 equ rtcALRL		\ Alarm Reg Low




\ ***********************
\ Flash memory controller
\ ***********************

\ Flash Program Erase Controller (FPEC) registers base address
$4002:2000 equ _FPEC		\ _AHB

\ Register offsets
$00 equ flACR
$04 equ flKEYR
$08 equ flOPTKEYR
$0C equ flSR
$10 equ flCR
$14 equ flAR

$1C equ flOBR
$20 equ flWRPR

$00A5 equ flRDPRTkey	\ keys needed to write registers
$45670123 equ flKEY1
$CDEF89AB equ flKEY2


\ ***********
\ GPIO Blocks
\ ***********

$4001:0800 equ _GPIOA		\ APB2
$4001:0C00 equ _GPIOB		\ APB2
$4001:1000 equ _GPIOC		\ APB2
$4001:1400 equ _GPIOD		\ APB2
$4001:1800 equ _GPIOE		\ APB2
$4001:1C00 equ _GPIOF		\ APB2
$4002:0800 equ _GPIOG		\ APB2

\ Register offsets
$00 equ gp.CRL			\ Port configuration low, bits 0..7
$04 equ gp.CRH			\ Port configuration high, bits 8..15
$08 equ gp.IDR
$0C equ gp.ODR
$10 equ gp.BSRR
$14 equ gp.BRR
$18 equ gp.LCKR


\ ****************************
\ External Event and Interrupt - EXTI
\ ****************************

$4001:0400 equ _EXTI		\ APB2
  $00 equ extiIMR
  $04 equ extiEMR
  $08 equ extiRTSR
  $0C equ extiFTSR
  $10 equ extiSWIER
  $14 equ extiPR


\ ******
\ Timers
\ ******
\ Not all timers have all registers

$4001:2C00 equ _TIM1		\ APB2
$4000:0000 equ _TIM2		\ APB1
$4000:0400 equ _TIM3		\ APB1
$4000:0800 equ _TIM4		\ APB1
$4000:0C00 equ _TIM5		\ APB1
$4000:1000 equ _TIM6		\ APB1
$4000:1400 equ _TIM7		\ APB1
$4001:3400 equ _TIM8		\ APB2
$4001:4C00 equ _TIM9		\ APB2
$4001:5000 equ _TIM10		\ APB2
$4001:5400 equ _TIM11		\ APB2
$4000:1800 equ _TIM12		\ APB1
$4000:1C00 equ _TIM13		\ APB1
$4000:2000 equ _TIM14		\ APB1
  $00 equ timCR1
  $04 equ timCR2
  $08 equ timSMCR
  $0C equ timDIER
  $10 equ timSR
  $14 equ timEGR
  $18 equ timCCMR1
  $1C equ timCCMR2
  $20 equ timCCER
  $24 equ timCNT
  $28 equ timPSC
  $2C equ timARR
  $30 equ timRCR
  $34 equ timCCR1
  $38 equ timCCR2
  $3C equ timCCR3
  $40 equ timCCR4
  $44 equ timBDTR
  $48 equ timDCR
  $4C equ timDMAR
  $50 equ timOR


\ ************
\ USART blocks
\ ************

$4001:3800 equ _USART1		\ APB2
$4000:4400 equ _USART2		\ APB1
$4000:4800 equ _USART3		\ APB1
$4000:4C00 equ _UART4		\ APB1
$4000:5000 equ _UART5		\ APB1

$00 equ usSR		\ status reg
$04 equ usDR		\ TX and RX data regs
  $04 equ usTDR
  $04 equ usRDR
$08 equ usBRR		\ baud rate divisor
$0C equ usCR1		\ control reg 1..3
$10 equ usCR2
$14 equ usCR3
$18 equ usGPTR		\ guard time and prescaler


\ **********
\ SPI blocks
\ **********

$4001:3000 equ _SPI1	\ _APB2 0x3000 +
$4000:3800 equ _SPI2	\ _APB1 0x3800 +
$4000:3C00 equ _SPI3	\ _APB1 0x3C00 +
  $000 equ spiCR1
  $004 equ spiCR2
  $008 equ spiSR
  $00C equ spiDR
  $010 equ spiCRCPR
  $014 equ spiRXCRCR
  $018 equ spiTXCRCR
  $01C equ spiI2SCFGR
  $020 equ spiI2SPR


\ **************
\ Ethernet block
\ **************

$4002:8000 equ _ETH		\ AHB

\ MAC control
  $000 equ ethMACCR	\ config
  $004 equ ethMACFFR	\ frame filter
  $008 equ ethMACHTHR	\ hash table high
  $00C equ ethMACHTLR	\ hash table low
  $010 equ ethMACMIIAR	\ MII address
  $014 equ ethMACMIIDR	\ MII data
  $018 equ ethMACFCR	\ flow control
  $01C equ ethMACVLANTR	\ VLAN tag

  $028 equ ethMACRWUFFR	\ remote wake up frame filter
  $02C equ ethMACPMTCSR	\ PMT control status

\  $034 equ ethMACDBGR	\ debug
  $038 equ MACSR	\ MAC interrupt status

  $03C equ ethMACIMR	\ interrupt mask
  $040 equ ethMACA0HR	\ addr 0 high
  $044 equ ethMACA0LR	\ addr 0 low
  $048 equ ethMACA1HR	\ addr 1 high
  $04C equ ethMACA1LR	\ addr 1 low
  $050 equ ethMACA2HR	\ addr 2 high
  $054 equ ethMACA2LR	\ addr 2 low
  $058 equ ethMACA3HR	\ addr 3 high
  $05C equ ethMACA3LR	\ addr 3 low

\ MMC control
  $100 equ ethMMCCR	\ MMC control
  $104 equ ethMMCRIR	\ MMC receive interrupt
  $108 equ ethMMCTIR	\ MMC transmit interrupt
  $10C equ ethMMCRIMR	\ MMC receive interrupt mask
  $110 equ ethMMCTIMR	\ MMC transmit interrupt mask

  $14C equ ethMMCTGFSCCR	\ transmitted good frames ...
  $150 equ ethMMCTGFMSCCR

  $168 equ ethMMCCTGFCR

  $194 equ ethMMCRFCECR
  $198 equ ethMMCRFAECR

  $1C4 equ ethMMCRGUFCR

\ IEEE1588 time stamp
  $700 equ ethPTPTSCR
  $704 equ ethPTPSSIR
  $708 equ ethPTPTSHR
  $70C equ ethPTPTSLR
  $710 equ ethPTPTSHUR
  $714 equ ethPTPTSLUR
  $718 equ ethPTPTSAR
  $71C equ ethPTPTTHR
  $720 equ ethPTPTTLR

  $728 equ ethPTPTSSR
  $72C equ ethPTPPPSCR

\ DMA control
  $1000 equ ethDMABMR	\ bus mode
  $1004 equ ethDMATPDR	\ transmit poll demand
  $1008 equ ethDMARPDR	\ receive poll demand
  $100C equ ethDMARDLAR	\ receive desc list addr
  $1010 equ ethDMATDLAR	\ transmit desc list addr
  $1014 equ ethDMASR	\ status
  $1018 equ ethDMAOMR	\ operation mode
  $101C equ ethDMAIER	\ interrupt enable
  $1020 equ ethDMAMFBOCR \ missed frame ...
\  $1024 equ ethDMARSWTR	\ receive status watchdog timer

  $1048 equ ethDMACHTDR	\ current host tx desc
  $104C equ ethDMACHRDR	\ current host rx desc
  $1050 equ ethDMACHTBAR \ current host tx buffer address
  $1054 equ ethDMACHRBAR \ current host rx buffer address


\ ************************
\ Exception vector numbers
\ ************************
\ Equates of the form xxxVec# refer to Exception Vector position,
\ whereas those of the form xxxInt# refer to Interrupt#.
\ Nearly all MPE code uses the Vec# form. To convert to Int#,
\ subtract 16 from the Vec#.

#16 equ WWDGvec#
#17 equ PVDvec#
#18 equ TamperVec#
#19 equ RTCvec#
#20 equ FlashVec#
#21 equ RCCvec#
#22 equ ExtI0Vec#
#23 equ ExtI1Vec#
#24 equ ExtI2Vec#
#25 equ ExtI3Vec#
#26 equ ExtI4Vec#
#27 equ DMAC1Vec#
#28 equ DMAC2Vec#
#29 equ DMAC3Vec#
#30 equ DMAC4Vec#
#31 equ DMAC5Vec#
#32 equ DMAC6Vec#
#33 equ DMAC7Vec#
#34 equ ADCvec#
#35 equ CAN1_TXvec#
#36 equ CAN1_RX0vec#
#37 equ CAN1_RX1vec#
#38 equ CAN1_SCEvec#
#39 equ ExtI9_5vec#
#40 equ TIM1_BRKvec#
#41 equ TIM1_UPvec#
#42 equ TIM1_TRGvec#
#43 equ TIM1_CCvec#
#44 equ TIM2vec#
#45 equ TIM3vec#
#46 equ TIM4vec#
#47 equ I2C1_EVvec#
#48 equ I2C1_ERvec#
#49 equ I2C2_EVvec#
#50 equ I2C2_ERvec#
#51 equ SPI1vec#
#52 equ SPI2vec#
#53 equ USART1vec#
#54 equ USART2vec#
#55 equ USART3vec#
#56 equ ExtI15_10vec#
#57 equ RTCalarmVec#
#58 equ USBwakeVec#

#66 equ TIM5Vec#
#67 equ SPI3Vec#
#68 equ UART4Vec#
#69 equ UART5Vec#
#70 equ TIM6Vec#
#71 equ TIM7Vec#
#72 equ DMA2C1Vec#
#73 equ DMA2C2Vec#
#74 equ DMA2C3Vec#
#75 equ DMA2C4Vec#
#76 equ DMA2C5Vec#
#77 equ ETHvec#
#78 equ ETH_WKUPvec#
#79 equ CAN2_TXvec#
#80 equ CAN2_RX0vec#
#81 equ CAN2_RX1vec#
#82 equ CAN2_SCEvec#
#83 equ OTG_FSvec#


\ ***********************
\ *S Miscellaneous macros
\ ***********************

interpreter
: setMask	\ value mask addr -- ; cell operation
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;
target


\ ======
\ *> ###
\ ======

decimal
