\ sfrSTM32F3xx.fth - peripheral definitions for STM32F30x/31x

((
Copyright (c) 2010, 2012, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal


\ ===========
\ *! sfrstm32f3xx
\ *T STM32F30x/31x memory and peripherals
\ ===========


\ ********************************
\ Memory/Peripheral base addresses
\ ********************************

$0800:0000 equ _FLASH		\ up to 256 kb

$1000:0000 equ _CCM		\ 8 kb

$1FFF:D800 equ _SYSTEM		\ 8 kb to 1FFF:F7FF
$1FFF:F800 equ _OPTION		\ 2 kb  to 1FFF:FFFF

$2000:0000 equ _SRAM		\ 40 kb
$2200:0000 equ _SRAM_BB         \ bit-band region

$4000:0000 equ _APB1
$4001:0000 equ _APB2

$5000:0000 equ _AHB3
$4800:0000 equ _AHB2
$4002:0000 equ _AHB1
$4200:0000 equ _PERIPH_BB       \ bit-band region


\ *******************
\ PWR - Power Control
\ *******************

$4000:7000 equ _PWR		\ _APB1 0x7000 +
  $00 equ pwrCR
  $04 equ pwrCSR


\ ***********************
\ Reset and Clock Control
\ ***********************

$4002:1000 equ _RCC		\ AHB1

\ Register offsets
$00 equ rccCR		\ Control Reg
$04 equ rccCFGR		\ Clock Configuration
$08 equ rccCIR		\ Clock Interrupt
$0C equ rccAPB2rst	\ ABP2 reset
$10 equ rccAPB1rst	\ APB1 reset
$14 equ rccAHBen	\ AHB clock enable
$18 equ rccAPB2en	\ APB2 clock enable
$1C equ rccAPB1en	\ APB1 clock enable
$20 equ rccBDCR		\ Backup Domain Control Reg
$24 equ rccCSR		\ Control and Status Reg
$28 equ rccAHBrst	\ AHB reset Reg, bit14=Ether, bit12=USB_OTG
$2C equ rccCFGR2	\ Clock Config Reg 2
$30 equ rccCFGR3	\ Clock Config Reg 3


\ ***********
\ GPIO Blocks
\ ***********

$4800:0000 equ _GPIOA		\ AHB2
$4800:0400 equ _GPIOB
$4800:0800 equ _GPIOC
$4800:0C00 equ _GPIOD
$4800:1000 equ _GPIOE
$4800:1400 equ _GPIOF


\ Register offsets
  $00 equ gpioMODER		\ mode register
  $04 equ gpioOTYPER		\ output type
  $08 equ gpioOSPEEDR		\ output speed
  $0C equ gpioPUPDR		\ pull up / pull down
  $10 equ gpioIDR		\ input data reg
  $14 equ gpioODR		\ output data reg
  $18 equ gpioBSRR		\ bit set / reset as 32 bit register
    $18 equ gpioBSRRL		\ low 16 bits to set
    $1A equ gpioBSRRH		\ high 16 bits to clear
  $1C equ gpioLCKR		\ configuration lock
  $20 equ gpioAFRL		\ alternate function low (4 bits per I/O pin)
  $24 equ gpioAFRH		\ alternate function high (4 bits per I/O pin)
  $28 equ gpioBRR		\ port bit reset


\ ***
\ DMA
\ ***

$4002:0000 equ _DMA1		\ AHB1
$4002:0400 equ _DMA2		\ AHB1
  $000 equ dmaISR		\ interrupt status
  $004 equ dmaIFCR		\ interrupt flag clear (write 1)
\ channel 1 - 20 bytes per DMA channel
  $008 equ dmaCCR1
  $00C equ dmaCNDTR1
  $010 equ dmaCPAR1
  $014 equ dmaCMAR1
\ channel 2
  $01C equ dmaCCR2
  $020 equ dmaCNDTR2
  $024 equ dmaCPAR2
  $028 equ dmaCMAR2
\ channel 3
  $030 equ dmaCCR3
  $034 equ dmaCNDTR3
  $038 equ dmaCPAR3
  $03C equ dmaCMAR3
\ channel 4
  $044 equ dmaCCR4
  $048 equ dmaCNDTR4
  $04C equ dmaCPAR4
  $050 equ dmaCMAR4
\ channel 5
  $058 equ dmaCCR5
  $05C equ dmaCNDTR5
  $060 equ dmaCPAR5
  $064 equ dmaCMAR5
\ channel 6
  $06C equ dmaCCR6
  $070 equ dmaCNDTR6
  $074 equ dmaCPAR6
  $078 equ dmaCMAR6
\ channel 7
  $080 equ dmaCCR7
  $084 equ dmaCNDTR7
  $088 equ dmaCPAR7
  $08C equ dmaCMAR7


\ *******************************
\ System Configuration Controller
\ *******************************

$4001:0000 equ _SYSCFG		\ APB2
$4001:0000 equ _COMP		\ APB2
$4001:0000 equ _OPAMP		\ APB2
  $00 equ sysCFGR1		\ memory remap register, offset: 0x00
  $04 equ sysRCR		\ CCM RAM protection
  $08 equ sysEXTICR1		\ external interrupt configuration reg, offset: 0x08-0x14
  $0C equ sysEXTICR2
  $10 equ sysEXTICR3
  $14 equ sysEXTICR4
  $18 equ sysCFGR1		\ config reg 2

  $1C equ comp1CSR
  $20 equ comp2CSR
  $24 equ comp3CSR
  $28 equ comp4CSR
  $2C equ comp5CSR
  $30 equ comp6CSR
  $34 equ comp7CSR

  $38 equ opamp1CSR
  $3C equ opamp2CSR
  $40 equ opamp3CSR
  $44 equ opamp4CSR


\ ****************************
\ External Event and Interrupt - EXTI
\ ****************************

$4001:0400 equ _EXTI
  $00 equ extiIMR1
  $04 equ extiEMR1
  $08 equ extiRTSR1
  $0C equ extiFTSR1
  $10 equ extiSWIER1
  $14 equ extiPR1

  $20 equ extiIMR2
  $24 equ extiEMR2
  $28 equ extiRTSR2
  $2C equ extiFTSR2
  $30 equ extiSWIER2
  $34 equ extiPR2


\ ******************************
\ Ananlogue to Digital Converter - ADC
\ ******************************

$5000:0000 equ _ADC1
$5000:0100 equ _ADC2
$5000:0300 equ _ADC12		\ common regs
$5000:0400 equ _ADC3
$5000:0500 equ _ADC4
$5000:0700 equ _ADC34		\ common regs

\ master/slave registers
  $00 equ adcISR
  $04 equ adcIER
  $08 equ adcCR
  $0C equ adcCFGR

  $14 equ adcSMPR1
  $18 equ adcSMPR2

  $20 equ adcTR1
  $24 equ adcTR2
  $28 equ adcTR3

  $30 equ adcSQR1
  $34 equ adcSQR2
  $38 equ adcSQR3
  $3C equ adcSQR4
  $40 equ adcDR

  $4C equ adcJSQR

  $60 equ adcOFR1
  $64 equ adcOFR2
  $68 equ adcOFR3
  $6C equ adcOFR4

  $80 equ adcJDR1
  $84 equ adcJDR2
  $88 equ adcJDR3
  $8C equ adcJDR4

  $A0 equ adcAWD2CR
  $A4 equ adcAWD3CR

\ Common registers
  $00 equ adcCSR

  $08 equ adcCCR
  $0C equ adcCDR


\ *****************************
\ Digital to Analogue Converter - DAC
\ *****************************

$4000:7400 equ _DAC
  $00 equ dacCR
  $04 equ dacSWTRIGR
  $08 equ dacDHR12R1
  $0C equ dacDHR12L1
  $10 equ dacDHR8R1
  $14 equ dacDHR12R2
  $18 equ dacDHR12L2
  $1C equ dacDHR8R2
  $20 equ dacDHR12RD
  $24 equ dacDHR12LD
  $28 equ dacDHR8RD
  $2C equ dacDOR1
  $30 equ dacDOR2
  $44 equ dacSR


\ ******
\ Timers
\ ******
\ Not all timers have all registers

$4001:2C00 equ _TIM1		\ APB2
$4000:0000 equ _TIM2		\ APB1
$4000:0400 equ _TIM3		\ APB1
$4000:0800 equ _TIM4		\ APB1

$4000:1000 equ _TIM6		\ APB1
$4000:1400 equ _TIM7		\ APB1
$4001:3400 equ _TIM8		\ APB2

$4001:4000 equ _TIM15		\ APB2
$4001:4400 equ _TIM16		\ APB2
$4001:4800 equ _TIM17		\ APB2
  $00 equ timCR1
  $04 equ timCR2
  $08 equ timSMCR
  $0C equ timDIER
  $10 equ timSR
  $14 equ timEGR
  $18 equ timCCMR1
  $1C equ timCCMR2
  $20 equ timCCER
  $24 equ timCNT
  $28 equ timPSC
  $2C equ timARR
  $30 equ timRCR
  $34 equ timCCR1
  $38 equ timCCR2
  $3C equ timCCR3
  $40 equ timCCR4
  $44 equ timBDTR
  $48 equ timDCR
  $4C equ timDMAR

  $54 equ timCCMR3
  $58 equ timCCR5
  $5C equ timCCR6


\ ********************
\ Independent Watchdog
\ ********************

$4000:3000 equ _IWDG		\ _APB1

  $00 equ iwdgKR
  $04 equ iwdgPR
  $08 equ iwdgRLR
  $0C equ iwdgSR
  $10 equ iwdgWINR


\ *****************
\ Windowed Watchdog
\ *****************

$4000:2C00 equ _WWDG		\ _APB1
  $00 equ wwdgCR
  $04 equ wwdgCFR
  $08 equ wwdgSR


\ ===
\ RTC
\ ===

$4000:2800 equ _RTC		\ APB1
  $000 equ rtcTR		\ time
  $004 equ rtcDR		\ date
  $008 equ rtcCR		\ control
  $00C equ rtcISR		\ initialisation
  $010 equ rtcPRER		\ prescaler
  $014 equ rtcWUTR		\ wakeup timer

  $01C equ rtcALRMAR		\ alarm A
  $020 equ rtcALRMBR		\ alarm B
  $024 equ rtcWPR		\ write protect
  $028 equ rtcSSR		\ subsecond
  $02C equ rtcSHIFTR		\ shift control
  $030 equ rtcTSTR		\ timestamp time
  $034 equ rtcTSDR		\ timestamp date
  $038 equ rtcTSSSR		\ timestamp subsecond
  $03C equ rtcCALR		\ calibration
  $040 equ rtcTAFCR		\ tamper, alt func cfg
  $044 equ rtcALRMASSR		\ alarm A subsecond
  $048 equ rtcALRMBSSR		\ alarm B subsecond

  $050 equ rtcBKP0R		\ backup BBRMA $50..8C (64 bytes)


\ ***
\ I2C
\ ***

$4000:5400 equ _I2C1		\ APB1
$4000:5800 equ _I2C2		\ APB1
  $00 equ i2cCR1
  $04 equ i2cCR2
  $08 equ i2cOAR1
  $0C equ i2cOAR2
  $10 equ i2cTIMINGR
  $14 equ i2cTIMEOUTR
  $18 equ i2cISR
  $1C equ i2cICR
  $20 equ i2cPECR
  $24 equ i2cRXDR
  $28 equ i2cTXDR


\ **********
\ SPI blocks
\ **********

$4001:3000 equ _SPI1		\ APB2
$4000:3800 equ _SPI2		\ APB1
$4000:3C00 equ _SPI3		\ APB1
  $000 equ spiCR1
  $004 equ spiCR2
  $008 equ spiSR
  $00C equ spiDR
  $010 equ spiCRCPR
  $014 equ spiRXCRCR
  $018 equ spiTXCRCR
  $01C equ spiI2SCFGR
  $020 equ spiI2SPR


\ ************
\ USART blocks
\ ************

$4001:3800 equ _USART1		\ APB2
$4000:4400 equ _USART2		\ APB1
$4000:4800 equ _USART3		\ APB1
$4000:4C00 equ _UART4		\ APB1
$4000:5000 equ _UART5		\ APB1
  $00 equ usCR1
  $04 equ usCR2
  $08 equ usCR3
  $0C equ usBRR
  $10 equ usGTPR
  $14 equ usRTOR
  $18 equ usRQR
  $1C equ usISR
  $20 equ usICR
  $24 equ usRDR
  $28 equ usTDR


\ ************************
\ Touch Sensing Controller - TSC
\ ************************

$4002:4000 equ _TSC		\ AHB1
  $00 equ tscCR
  $04 equ tscIER
  $08 equ tscICR
  $0C equ tscISR
  $10 equ tscIOHCR

  $18 equ tscIOASCR

  $20 equ tscIOSCR

  $28 equ tscIOCCR

  $30 equ tscIOGCSR
  $34 equ tscIOG1CR
  $38 equ tscIOG2CR
  $3C equ tscIOG3CR
  $40 equ tscIOG4CR
  $44 equ tscIOG5CR
  $48 equ tscIOG6CR
  $4C equ tscIOG7CR
  $50 equ tscIOG8CR


\ *****
\ bxCAN
\ *****

$4000:6400 equ _bxCAN
  $00 equ canMCR
  $04 equ canMSR
  $08 equ canTSR
  $0C equ canRF0R
  $10 equ canRF1R
  $14 equ canIER
  $18 equ canESR
  $1C equ canBTR

  $180 equ canTI0R
  $184 equ canTDT0R
  $188 equ canTDL0R
  $18C equ canTDH0R
  $190 equ canTI1R
  $194 equ canTDT1R
  $198 equ canTDL1R
  $19C equ canTDH1R
  $1A0 equ canTI2R
  $1A4 equ canTDT2R
  $1A8 equ canTDL2R
  $1AC equ canTDH2R

  $1B0 equ canRI0R
  $1B4 equ canRDT0R
  $1B8 equ canRDL0R
  $1BC equ canRDH0R
  $1C0 equ canRI1R
  $1C4 equ canRDT1R
  $1C8 equ canRDL1R
  $1CC equ canRDH1R

  $200 equ canFMR
  $204 equ canFM1R

  $20C equ canFS1R

  $214 equ canFFA1R

  $21C equ canFA1R

  $240 equ canF0R1	$240 equ canFxR1
  $244 equ canF0R2	$244 equ canFxR2
\ ...
  $318 equ canF27R1
  $31C equ canF27R2


\ ***
\ USB
\ ***

$4000:6000 equ _USBSRAM		\ 512 bytes
$4000:5C00 equ _USBdev		\ device peripheral
  $00 equ usbEP0R
  $04 equ usbEP1R
  $08 equ usbEP2R
  $0C equ usbEP3R
  $10 equ usbEP4R
  $14 equ usbEP5R
  $18 equ usbEP6R
  $1C equ usbEP7R

  $40 equ usbCNTR
  $44 equ usbISTR
  $48 equ usbFNR
  $4C equ usbDADDR
  $50 equ usbBTABLE


\ ************************
\ Exception vector numbers
\ ************************
\ Equates of the form xxxVec# refer to Exception Vector position,
\ whereas those of the form xxxInt# refer to Interrupt#.
\ Nearly all MPE code uses the Vec# form. To convert to Int#,
\ subtract 16 from the Vec#.

#16 equ WWDGvec#
#17 equ PVDvec#
#18 equ TamperVec#
#19 equ RTCvec#
#20 equ FlashVec#
#21 equ RCCvec#
#22 equ ExtI0Vec#
#23 equ ExtI1Vec#
#24 equ ExtI2Vec#	#24 equ TSCVec#
#25 equ ExtI3Vec#
#26 equ ExtI4Vec#
#27 equ DMAC1Vec#
#28 equ DMAC2Vec#
#29 equ DMAC3Vec#
#30 equ DMAC4Vec#
#31 equ DMAC5Vec#
#32 equ DMAC6Vec#
#33 equ DMAC7Vec#
#34 equ ADCvec#
#35 equ CAN1_TXvec#	#35 equ USB_HPVec#
#36 equ CAN1_RX0vec#	#36 equ USB_LPVec#
#37 equ CAN1_RX1vec#
#38 equ CAN1_SCEvec#
#39 equ ExtI9_5vec#
#40 equ TIM1_BRKvec#
#41 equ TIM1_UPvec#
#42 equ TIM1_TRGvec#
#43 equ TIM1_CCvec#
#44 equ TIM2vec#
#45 equ TIM3vec#
#46 equ TIM4vec#
#47 equ I2C1_EVvec#	#47 equ EXTI23Vec#
#48 equ I2C1_ERvec#
#49 equ I2C2_EVvec#	#49 equ EXTI24Vec#
#50 equ I2C2_ERvec#
#51 equ SPI1vec#
#52 equ SPI2vec#
#53 equ USART1vec#	#53 equ EXTI25Vec#
#54 equ USART2vec#	#54 equ EXTI26Vec#
#55 equ USART3vec#	#55 equ EXTI28Vec#
#56 equ ExtI15_10vec#
#57 equ RTCalarmVec#
#58 equ USBwakeVec#
#59 equ TIM8_BRKVec#
#60 equ TIM8_UPVec#
#61 equ TIM8_TRG_COMVec#
#62 equ TIM8_CCVec#
#63 equ ADC3Vec#

#67 equ SPI3Vec#
#68 equ UART4Vec#
#69 equ UART5Vec#
#70 equ TIM6Vec#	#70 equ DACUNDERVec#
#71 equ TIM7Vec#
#72 equ DMA2C1Vec#
#73 equ DMA2C2Vec#
#74 equ DMA2C3Vec#
#75 equ DMA2C4Vec#
#76 equ DMA2C5Vec#
#77 equ ADC4vec#

#80 equ COMP123vec#
#81 equ COMP456vec#
#82 equ COMP7vec#

#90 equ USB_HPVec#
#91 equ USB_LPVec#
#92 equ USB_WKUPVec#

#97 equ FPUVec#


\ ***********************
\ *S Miscellaneous macros
\ ***********************

interpreter
: setMask	\ value mask addr -- ; cell operation
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;
target


\ ======
\ *> ###
\ ======

decimal
