\ sfrSTM32F4xx.fth - peripheral definitions for STM32F4 devices

((
Copyright (c) 2011, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

Parts contributed by
OpenMRT, Cape Town, South Africa


To do
=====

Change history
==============
20120316 MRT004 - added OTG_FS and RRC stuff
20120316 MRT003 - added DMA and SIO stuff
20120309 MRT002 - beautify and doublecheck code, added SDIO references
20120101 MPE001 - original release
))

only forth definitions
decimal


\ ===============
\ *! sfrstm32f4xx
\ *T STM32F4xx memory and peripherals
\ ===============

\ ********************************
\ Memory/Peripheral base addresses
\ ********************************


$A000:0000 equ _AHB3
$5000:0000 equ _AHB2
$4002:0000 equ _AHB1

$4002:4000 equ _BKPSRAM		\ 4 kb - battery backed

$4001:0000 equ _APB2		\ max 84 MHz
$4000:0000 equ _APB1		\ max 42 MHz
$4000:0000 equ _PERIPH		\ legacy name

$2001:C000 equ _SRAM2		\ 16 kb
$2000:0000 equ _SRAM1		\ 112 kb
$2000:0000 equ _SRAM		\ legacy name
$1FFF:0000 equ _SYSTEM		\ 30 kb
$1000:0000 equ _DATARAM		\ 64k CCM
$0800:0000 equ _FLASH		\ up to 1 Mb

$4200:0000 equ _PERIPH_BB       \ bit-band region
$2200:0000 equ _SRAM_BB         \ bit-band region

$1FFF:0000 equ SysBase		\ Base address of system memory  block.
$1FFF:7800 equ OTPbase		\ Base address of OTP memory block.
$1FFF:C000 equ OptBase		\ Base address of option memory block.


\ ******************************
\ Misc peripheral base addresses
\ ******************************
\ When a #define is commented out, the device registers is put
\ out in detail
\ FSMC registers base address
\ $A000:0000 equ _FSMC

\ Peripheral memory map

\ #define _TIM2             _APB1 0x0000 +
\ #define _TIM3             _APB1 0x0400 +
\ #define _TIM4             _APB1 0x0800 +
\ #define _TIM5             _APB1 0x0C00 +
\ #define _TIM6             _APB1 0x1000 +
\ #define _TIM7             _APB1 0x1400 +

\ #define _TIM12            _APB1 0x1800 +
\ #define _TIM13            _APB1 0x1C00 +
\ #define _TIM14            _APB1 0x2000 +

\ #define _RTC              _APB1 0x2800 +
\ #define _IWDG             _APB1 0x3000 +
\ #define _WWDG             _APB1 0x2C00 +

#define _I2S2ext          _APB1 0x3400 +

\ #define _SPI2             _APB1 0x3800 +
\ #define _SPI3             _APB1 0x3C00 +

#define _I2S3ext          _APB1 0x4000 +

#define _I2C1             _APB1 0x5400 +
#define _I2C2             _APB1 0x5800 +
#define _CAN              _APB1 0x6400 +
#define _BKP              _APB1 0x6C00 +
\ #define _PWR              _APB1 0x7000 +
#define _DAC              _APB1 0x7400 +

\ #define _EXTI             _APB2 0x0400 +
\ #define _TIM1             _APB2 0x2C00 +
\ #define _SPI1             _APB2 0x3000 +
\ #define _TIM8             _APB2 0x3400 +

\ #define _CRC              _AHB 0x3000 +

\ Flash Option Bytes base address
#define _OB               0x1FFFF800

\ FSMC Bankx registers base address
\ #define _FSMC_Bank1_R     _FSMC 0x0000 +
\ #define _FSMC_Bank1E_R    _FSMC 0x0104 +
\ #define _FSMC_Bank2_R     _FSMC 0x0060 +
\ #define _FSMC_Bank3_R     _FSMC 0x0080 +
\ #define _FSMC_Bank4_R     _FSMC 0x00A0 +

\ Debug MCU registers base address
#define _DBGMCU          0xE0042000


\ ********************
\ CRC calculation unit
\ ********************

$4002:3000 equ _CRC		\ _AHB1 0x3000 +
  $00 equ crcDR
  $04 equ crcIDR
  $08 equ crcCR


\ *******************
\ PWR - Power Control
\ *******************

$4000:7000 equ _PWR		\ _APB1 0x7000 +
  $00 equ pwrCR
  $04 equ pwrCSR


\ ***********************
\ Reset and Clock Control
\ ***********************

$4002:3800 equ _RCC		\ RCC base address

\ Register offsets
$00 equ rccCR		\ Control Reg
$04 equ rccPLLCFGR	\ Clock Configuration
$08 equ rccCFGR		\ Clock Configuration
$0C equ rccCIR		\ Clock Interrupt
$10 equ rccAHB1RSTR	\ AHB1 reset
$14 equ rccAHB2RSTR	\ AHB2 reset
$18 equ rccAHB3RSTR	\ AHB3 reset

$20 equ rccAPB1RSTR	\ APB1 reset
$24 equ rccAPB2RSTR	\ APB2 reset

$30 equ rccAHB1ENR	\ AHB1 clock enable
$34 equ rccAHB2ENR	\ AHB2 clock enable
$38 equ rccAHB3ENR	\ AHB3 clock enable

$40 equ rccAPB1ENR	\ APB1 clock enable
$44 equ rccAPB2ENR	\ APB2 clock enable

$50 equ rccAHB1LPENR	\ AHB1 low power clock enable
$54 equ rccAHB2LPENR	\ AHB2 low power clock enable
$58 equ rccAHB3LPENR	\ AHB3 low power clock enable

$60 equ rccAPB1LPENR	\ APB1 low power clock enable
$64 equ rccAPB2LPENR	\ APB2 low power clock enable

$70 equ rccBDCR		\ Backup Domain Control Reg
$74 equ rccCSR		\ Control and Status Reg

$80 equ rccSSCGR
$84 equ rccPLLI2SCFGR


\ ********************
\ Independent Watchdog
\ ********************

$4000:3000 equ _IWDG		\ _APB1 0x3000 +

  $00 equ iwdgKR
  $04 equ iwdgPR
  $08 equ iwdgRLR
  $0C equ iwdgSR


\ *****************
\ Windowed Watchdog
\ *****************

$4000:2C00 equ _WWDG		\ _APB1 0x2C00 +
  $00 equ wwdgCR
  $04 equ wwdgCFR
  $08 equ wwdgSR


\ ===
\ RTC
\ ===

$4000:2800 equ _RTC
  $000 equ rtcTR		\ time
  $004 equ rtcDR		\ date
  $008 equ rtcCR		\ control
  $00C equ rtcISR		\ initialisation
  $010 equ rtcPRER		\ prescaler
  $014 equ rtcWUTR		\ wakeup timer
  $018 equ rtcCALIBR		\ calibration
  $01C equ rtcALRMAR		\ alarm A
  $020 equ rtcALRMBR		\ alarm B
  $024 equ rtcWPR		\ write protect
  $028 equ rtcSSR		\ subsecond
  $02C equ rtcSHIFTR		\ shift control
  $030 equ rtcTSTR		\ timestamp time
  $034 equ rtcTSDR		\ timestamp date
  $038 equ rtcTSSSR		\ timestamp subsecond
  $03C equ rtcCALR		\ calibration
  $040 equ rtcTAFCR		\ tamper, alt func cfg
  $044 equ rtcALRMASSR		\ alarm A subsecond
  $048 equ rtcALRMBSSR		\ alarm B subsecond

  $050 equ rtcBKP0R		\ backup BBRMA $50..9F (80 bytes)



\ ***********************
\ Flash memory controller
\ ***********************

\ Flash Program Erase Controller (FPEC) registers base address
$4002:3C00 equ _FPEC		\ _AHB1 0x3c00 +

\ Register offsets
$00 equ flACR
$04 equ flKEYR
$08 equ flOPTKEYR
$0C equ flSR
$10 equ flCR
$14 equ flAR

$1C equ flOBR
$20 equ flWRPR

$00A5 equ flRDPRTkey	\ keys needed to write registers
$45670123 equ flKEY1
$CDEF89AB equ flKEY2


\ ***********
\ GPIO Blocks
\ ***********

$4002:0000 equ _GPIOA		\ AHB1 + 0x0000
$4002:0400 equ _GPIOB
$4002:0800 equ _GPIOC
$4002:0C00 equ _GPIOD
$4002:1000 equ _GPIOE
$4002:1400 equ _GPIOF
$4002:1800 equ _GPIOG
$4002:1C00 equ _GPIOH
$4002:2000 equ _GPIOI

\ Register offsets
  $00 equ gpioMODER		\ mode register
  $04 equ gpioOTYPER		\ output type
  $08 equ gpioOSPEEDR		\ output speed
  $0C equ gpioPUPDR		\ pull up / pull down
  $10 equ gpioIDR		\ input data reg
  $14 equ gpioODR		\ output data reg
  $18 equ gpioBSRR		\ bit set / reset as 32 bit register
    $18 equ gpioBSRRL		\ low 16 bits to set
    $1A equ gpioBSRRH		\ high 16 bits to clear
  $1C equ gpioLCKR		\ configuration lock
  $20 equ gpioAFRL		\ alternate function low (4 bits per I/O pin)
  $24 equ gpioAFRH		\ alternate function high (4 bits per I/O pin)


\ *******************************
\ System Configuration Controller
\ *******************************

$4001:3800 equ _SYSCFG		\ APB2 + 0x3800
  $00 equ sysMEMRMP		\ memory remap register, offset: 0x00
  $04 equ sysPMC		\ peripheral mode configuration reg, offset: 0x04
  $08 equ sysEXTICR1		\ external interrupt configuration reg, offset: 0x08-0x14
  $0C equ sysEXTICR2
  $10 equ sysEXTICR3
  $14 equ sysEXTICR4

  $20 equ sysCMPCR		\ Compensation cell control reg, offset: 0x20


\ ****************************
\ External Event and Interrupt - EXTI
\ ****************************

$4001:3C00 equ _EXTI
  $00 equ extiIMR
  $04 equ extiEMR
  $08 equ extiRTSR
  $0C equ extiFTSR
  $10 equ extiSWIER
  $14 equ extiPR


\ ******
\ Timers
\ ******
\ Not all timers have all registers

$4001:0000 equ _TIM1
$4000:0000 equ _TIM2
$4000:0400 equ _TIM3
$4000:0800 equ _TIM4
$4000:0C00 equ _TIM5
$4000:1000 equ _TIM6
$4000:1400 equ _TIM7
$4001:0400 equ _TIM8
$4001:4000 equ _TIM9
$4001:4400 equ _TIM10
$4001:4800 equ _TIM11
$4000:1800 equ _TIM12
$4000:1C00 equ _TIM13
$4000:2000 equ _TIM14
  $00 equ timCR1
  $04 equ timCR2
  $08 equ timSMCR
  $0C equ timDIER
  $10 equ timSR
  $14 equ timEGR
  $18 equ timCCMR1
  $1C equ timCCMR2
  $20 equ timCCER
  $24 equ timCNT
  $28 equ timPSC
  $2C equ timARR
  $30 equ timRCR
  $34 equ timCCR1
  $38 equ timCCR2
  $3C equ timCCR3
  $40 equ timCCR4
  $44 equ timBDTR
  $48 equ timDCR
  $4C equ timDMAR
  $50 equ timOR


\ ************
\ USART blocks
\ ************

$4001:1000 equ _USART1		\ APB2 + 0x1000, PA9, PA10
$4000:4400 equ _USART2		\ APB1 + 0x4400
$4000:4800 equ _USART3		\ APB1 + 0x4800, PB10/11 & PC10/11
$4000:4C00 equ _UART4		\ APB1 + 0x4C00
$4000:5000 equ _UART5		\ APB1 + 0x5000
$4001:1400 equ _USART6		\ APB2 + 0x1400

$00 equ usSR		\ status reg
$04 equ usDR		\ TX and RX data regs
  $04 equ usTDR
  $04 equ usRDR
$08 equ usBRR		\ baud rate divisor
$0C equ usCR1		\ control reg 1..3
$10 equ usCR2
$14 equ usCR3
$18 equ usGPTR		\ guard time and prescaler


\ **********
\ SPI blocks
\ **********

\ #define _SPI1             _APB2 0x3000 +
\ #define _SPI2             _APB1 0x3800 +
\ #define _SPI3             _APB1 0x3C00 +

$4001:3000 equ _SPI1	\ APB2
$4000:3800 equ _SPI2	\ APB1
$4000:3C00 equ _SPI3	\ APB1
  $00 equ spiCR1
  $04 equ spiCR2
  $08 equ spiSR
  $0C equ spiDR
  $10 equ spiCRCPR
  $14 equ spiRXCRCR
  $18 equ spiTXCRCR
  $1C equ spiI2SCFGR
  $20 equ spiI2SPR


\ **************
\ Ethernet block
\ **************

$4002:8000 equ _ETH

\ MAC control
  $000 equ ethMACCR	\ config
  $004 equ ethMACFFR	\ frame filter
  $008 equ ethMACHTHR	\ hash table high
  $00C equ ethMACHTLR	\ hash table low
  $010 equ ethMACMIIAR	\ MII address
  $014 equ ethMACMIIDR	\ MII data
  $018 equ ethMACFCR	\ flow control
  $01C equ ethMACVLANTR	\ VLAN tag

  $028 equ ethMACRWUFFR	\ remote wake up frame filter
  $02C equ ethMACPMTCSR	\ PMT control status

  $034 equ ethMACDBGR	\ debug

  $03C equ ethMACIMR	\ interrupt mask
  $040 equ ethMACA0HR	\ addr 0 high
  $044 equ ethMACA0LR	\ addr 0 low
  $048 equ ethMACA1HR	\ addr 1 high
  $04C equ ethMACA1LR	\ addr 1 low
  $050 equ ethMACA2HR	\ addr 2 high
  $054 equ ethMACA2LR	\ addr 2 low
  $058 equ ethMACA3HR	\ addr 3 high
  $05C equ ethMACA3LR	\ addr 3 low

\ MMC control
  $100 equ ethMMCCR	\ MMC control
  $104 equ ethMMCRIR	\ MMC receive interrupt
  $108 equ ethMMCTIR	\ MMC transmit interrupt
  $10C equ ethMMCRIMR	\ MMC receive interrupt mask
  $110 equ ethMMCTIMR	\ MMC transmit interrupt mask

  $14C equ ethMMCTGFSCCR	\ transmitted good frames ...
  $150 equ ethMMCTGFMSCCR

  $168 equ ethMMCCTGFCR

  $194 equ ethMMCRFCECR
  $198 equ ethMMCRFAECR

  $1C4 equ ethMMCRGUFCR

\ IEEE1588 time stamp
  $700 equ ethPTPTSCR
  $704 equ ethPTPSSIR
  $708 equ ethPTPTSHR
  $70C equ ethPTPTSLR
  $710 equ ethPTPTSHUR
  $714 equ ethPTPTSLUR
  $718 equ ethPTPTSAR
  $71C equ ethPTPTTHR
  $720 equ ethPTPTTLR

  $728 equ ethPTPTSSR
  $72C equ ethPTPPPSCR

\ DMA control
  $1000 equ ethDMABMR	\ bus mode
  $1004 equ ethDMATPDR	\ transmit poll demand
  $1008 equ ethDMARPDR	\ receive poll demand
  $100C equ ethDMARDLAR	\ receive desc list addr
  $1010 equ ethDMATDLAR	\ transmit desc list addr
  $1014 equ ethDMASR	\ status
  $1018 equ ethDMAOMR	\ operation mode
  $101C equ ethDMAIER	\ interrupt enable
  $1020 equ ethDMAMFBOCR \ missed frame ...
  $1024 equ ethDMARSWTR	\ receive status watchdog timer

  $1048 equ ethDMACHTDR	\ current host tx desc
  $104C equ ethDMACHRDR	\ current host rx desc
  $1050 equ ethDMACHTBAR \ current host tx buffer address
  $1054 equ ethDMACHRBAR \ current host rx buffer address


\ **********
\ USB_OTG_FS
\ **********
\  [$5000:0000 - $5003:FFFF]

_AHB2 $0000:0000 +  equ _OTG_FS

\ -- CSR Memory Map

\ core global registers - $0000

  $000 equ otgfsGOTGCTL
  $004 equ otgfsGOTGINT
  $008 equ otgfsGAHBCFG
  $00C equ otgfsGUSBCFG
  $010 equ otgfsGRSTCTL
  $014 equ otgfsGINTSTS
  $018 equ otgfsGINTMSK
  $01C equ otgfsGRXSTSR           \ (Host/Device differs)
  $020 equ otgfsGRXSTSP           \ (Host/Device differs)
  $024 equ otgfsGRXFSIZ
  $028 equ otgfsHNPTXFSIZ
  $028 equ otgfsDIEPTXF0
  $02C equ otgfsHNPTXSTS
  $038 equ otgfsGCCFG
  $03C equ otgfsCID
  $100 equ otgfsHPTXFSIZ
  $104 equ otgfsDIEPTXF1
  $108 equ otgfsDIEPTXF2
  $10C equ otgfsDIEPTXF3

\ host-mode registers - $0400
\  Host global registers
\  Host port CSRs
\  Host channel-specific registers

  $400 equ otgfsHCFG
  $404 equ otgfsHFIR
  $408 equ otgfsHFNUM
  $410 equ otgfsHPTXSTS
  $414 equ otgfsHAINT
  $418 equ otgfsHAINTMSK
  $440 equ otgfsHPRT
  $500 equ otgfsHCCHAR0
  $520 equ otgfsHCCHAR1
  $540 equ otgfsHCCHAR2
  $560 equ otgfsHCCHAR3
  $580 equ otgfsHCCHAR4
  $5A0 equ otgfsHCCHAR5
  $5C0 equ otgfsHCCHAR6
  $5E0 equ otgfsHCCHAR7
  $508 equ otgfsHCINT0
  $528 equ otgfsHCINT1
  $548 equ otgfsHCINT2
  $568 equ otgfsHCINT3
  $588 equ otgfsHCINT4
  $5A8 equ otgfsHCINT5
  $5C8 equ otgfsHCINT6
  $5E8 equ otgfsHCINT7
  $50C equ otgfsHCINTMSK0
  $52C equ otgfsHCINTMSK1
  $54C equ otgfsHCINTMSK2
  $56C equ otgfsHCINTMSK3
  $58C equ otgfsHCINTMSK4
  $5AC equ otgfsHCINTMSK5
  $5CC equ otgfsHCINTMSK6
  $5EC equ otgfsHCINTMSK7
  $510 equ otgfsHCTSIZ0
  $530 equ otgfsHCTSIZ1
  $550 equ otgfsHCTSIZ2
  $570 equ otgfsHCTSIZ3
  $590 equ otgfsHCTSIZ4
  $5B0 equ otgfsHCTSIZ5
  $5D0 equ otgfsHCTSIZ6
  $5F0 equ otgfsHCTSIZ7

\ device-mode registers - $0800
\  Device global registers
\  Device endpoint-specific registers

  $800 equ otgfsDCFG
  $804 equ otgfsDCTL
  $808 equ otgfsDSTS
  $810 equ otgfsDIEPMSK
  $814 equ otgfsDOEPMSK
  $818 equ otgfsDAINT
  $81C equ otgfsDAINTMSK
  $828 equ otgfsDVBUSDIS
  $82C equ otgfsDVBUSPULSE
  $834 equ otgfsDIEPEMPMSK
  $900 equ otgfsDIEPCTL0
  $908 equ othfsDIEPINT0
  $910 equ otgfsDIEPTSIZ0
  $918 equ otgfsDTXFSTS0
  $920 equ otgfsDIEPCTL1
  $928 equ otgfsDIEPINT1
  $930 equ otgfsDIEPTSIZ1
  $938 equ otgfsDTXFSTS1
  $940 equ otgfsDIEPCTL2
  $948 equ otgfsDIEPINT2
  $950 equ otgfsDIEPTSIZ2
  $958 equ otgfsDTXFSTS2
  $960 equ otgfsDIEPCTL3
  $968 equ otgfsDIEPINT3
  $970 equ otgfsDIEPTSIZ3
  $978 equ otgfsDTXFSTS3
  $B00 equ otgfsDOEPCTL0
  $B08 equ otgfsDOEPINT0
  $B10 equ otgfsDOEPTSIZ0
  $B20 equ otgfsDOEPCTL1
  $B28 equ otgfsDOEPINT1
  $B30 equ otgfsDOEPTSIZ1
  $B40 equ otgfsDOEPCTL2
  $B48 equ otgfsDOEPINT2
  $B50 equ otgfsDOEPTSIZ2
  $B60 equ otgfsDOEPCTL3
  $B68 equ otgfsDOEPINT3
  $B70 equ otgfsDOEPTSIZ3

\ power and clock-gating registers - $0E00

  $E00 equ otgfsPCGCCTL

\ Data FIFO (DFIFO) access registers
\  Device EP 0 / Host channel 0 FIFO - $1000
\  Device EP 1 / Host channel 1 FIFO - $2000
\  Device EP 2 / Host channel 2 FIFO - $3000
\  Device EP 3 / Host channel 3 FIFO - $4000
\                Host channel 4 FIFO - $5000
\                Host channel 5 FIFO - $6000
\                Host channel 6 FIFO - $7000
\                Host channel 7 FIFO - $8000


\ **********
\ DMA1, DMA2
\ **********
\ DMA1 :  [ $4002:6000 - $4002:63FF ]
\ DMA2 :  [ $4002:6400 - $4002:67FF ]

_AHB1 $0000:6000 +  equ _DMA1
_AHB1 $0000:6400 +  equ _DMA2

  $00 equ dmaLISR      \ Low interrupt status register
  $04 equ dmaHISR      \ High interrupt status register
  $08 equ dmaLIFCR     \ Low interrupt flag clear register
  $0C equ dmaHIFCR     \ High interrupt flag clear register

  $10 equ dmaS0CR      \ Stream 0 configuration register
  $14 equ dmaS0NDTR    \ Stream 0 number of data register
  $18 equ dmaS0PAR     \ Stream 0 peripheral address register
  $1C equ dmaS0M0AR    \ Stream 0 memory 0 address register
  $20 equ dmaS0M1AR    \ Stream 0 memory 1 address register
  $24 equ dmaS0FCR     \ Stream 0 FIFO control register

  $28 equ dmaS1CR      \ Stream 1
  $2C equ dmaS1NDTR
  $30 equ dmaS1PAR
  $34 equ dmaS1M0AR
  $38 equ dmaS1M1AR
  $3C equ dmaS1FCR

  $40 equ dmaS2CR      \ Stream 2
  $44 equ dmaS2NDTR
  $48 equ dmaS2PAR
  $4C equ dmaS2M0AR
  $50 equ dmaS2M1AR
  $54 equ dmaS2FCR

  $58 equ dmaS3CR      \ Stream 3
  $5C equ dmaS3NDTR
  $60 equ dmaS3PAR
  $64 equ dmaS3M0AR
  $68 equ dmaS3M1AR
  $6C equ dmaS3FCR

  $70 equ dmaS4CR      \ Stream 4
  $74 equ dmaS4NDTR
  $78 equ dmaS4PAR
  $7C equ dmaS4M0AR
  $80 equ dmaS4M1AR
  $84 equ dmaS4FCR

  $88 equ dmaS5CR      \ Stream 5
  $8C equ dmaS5NDTR
  $90 equ dmaS5PAR
  $94 equ dmaS5M0AR
  $98 equ dmaS5M1AR
  $9C equ dmaS5FCR

  $A0 equ dmaS6CR      \ Stream 6
  $A4 equ dmaS6NDTR
  $A8 equ dmaS6PAR
  $AC equ dmaS6M0AR
  $B0 equ dmaS6M1AR
  $B4 equ dmaS6FCR

  $B8 equ dmaS7CR      \ Stream 7
  $BC equ dmaS7NDTR
  $C0 equ dmaS7PAR
  $C4 equ dmaS7M0AR
  $C8 equ dmaS7M1AR
  $CC equ dmaS7FCR


\ ***
\ ADC
\ ***

_APB2 0x2000 + equ _ADC1
_APB2 0x2100 + equ _ADC2
_APB2 0x2200 + equ _ADC3
  0x00 equ adcSR
  0x04 equ adcCR1
  0x08 equ adcCR2
  0x0C equ adcSMPR1
  0x10 equ adcSMPR2
  0x14 equ adcJOFR1
  0x18 equ adcJOFR2
  0x1C equ adcJOFR3
  0x20 equ adcJOFR4
  0x24 equ adcHTR
  0x28 equ adcLTR
  0x2C equ adcSQR1
  0x30 equ adcSQR2
  0x34 equ adcSQR3
  0x38 equ adcJSQR
  0x3C equ adcJDR1
  0x40 equ adcJDR2
  0x44 equ adcJDR3
  0x48 equ adcJDR4
  0x4C equ adcDR

_APB2 0x2300 + equ _ADCcommon
  0x00 equ adcCSR
  0x04 equ adcCCR
  0x08 equ adcCDR


\ ***********************
\ SDIO Secure Digital I/O (APB2)
\ ***********************
\  [ $4001:2C00 - $4001:2FFF ]

_APB2 $2C00 + equ _SDIO

  $00 equ sdPOWER
  $04 equ sdCLKCR
  $08 equ sdARG
  $0C equ sdCMD
  $10 equ sdRESPCMD
  $14 equ sdRESP1
  $18 equ sdRESP2
  $1C equ sdRESP3
  $20 equ sdRESP4
  $24 equ sdDTIMER
  $28 equ sdDLEN
  $2C equ sdDCTRL
  $30 equ sdDCOUNT
  $34 equ sdSTA
  $38 equ sdICR
  $3C equ sdMASK
  $48 equ sdFIFOCNT
  $80 equ sdFIFO


\ ****
\ FSMC
\ ****

$A000:0000 equ _FSMC
  $000 equ fsmc.BCR1
  $008 equ fsmc.BCR2
  $010 equ fsmc.BCR3
  $018 equ fsmc.BCR4

  $004 equ fsmc.BTR1
  $00C equ fsmc.BTR2
  $014 equ fsmc.BTR3
  $01C equ fsmc.BTR4

  $104 equ fsmc.BWTR1
  $10C equ fsmc.BWTR2
  $114 equ fsmc.BWTR3
  $11C equ fsmc.BWTR4

  $060 equ fsmc.PCR2
  $080 equ fsmc.PCR3
  $0A0 equ fsmc.PCR4

  $064 equ fsmc.SR2
  $084 equ fsmc.SR3
  $0A4 equ fsmc.SR4

  $068 equ fsmc.PMEM2
  $088 equ fsmc.PMEM3
  $0A8 equ fsmc.PMEM4

  $06C equ fsmc.PATT2
  $08C equ fsmc.PATT3
  $0AC equ fsmc.PATT4

  $0B0 equ fsmc.PIO4

  $074 equ fsmc.ECCR2
  $094 equ fsmc.ECCR3

\ Alternates
\ #define _FSMC_Bank1_R     _FSMC 0x0000 +
\ #define _FSMC_Bank1E_R    _FSMC 0x0104 +
\ #define _FSMC_Bank2_R     _FSMC 0x0060 +
\ #define _FSMC_Bank3_R     _FSMC 0x0080 +
\ #define _FSMC_Bank4_R     _FSMC 0x00A0 +


\ ************************
\ Exception vector numbers
\ ************************
\ Equates of the form xxxVec# refer to Exception Vector position,
\ whereas those of the form xxxInt# refer to Interrupt#.
\ Nearly all MPE code uses the Vec# form. To convert to Int#,
\ subtract 16 from the Vec#.

#16 equ WWDGvec#
#17 equ PVDvec#
#18 equ TamperVec#
#19 equ RTCvec#
#20 equ FlashVec#
#21 equ RCCvec#
#22 equ ExtI0Vec#
#23 equ ExtI1Vec#
#24 equ ExtI2Vec#
#25 equ ExtI3Vec#
#26 equ ExtI4Vec#
#27 equ DMA1S0Vec#
#28 equ DMA1S1Vec#
#29 equ DMA1S2Vec#
#30 equ DMA1S3Vec#
#31 equ DMA1S4Vec#
#32 equ DMA1S5Vec#
#33 equ DMA1S6Vec#
#34 equ ADCvec#
#35 equ CAN1_TXvec#
#36 equ CAN_RX0vec#
#37 equ CAN_RX1vec#
#38 equ CAN_SCEvec#
#39 equ ExtI9_5vec#
#40 equ TIM1_BRK_TIM9vec#
#41 equ TIM1_UP_TIM10vec#
#42 equ TIM1_TRG_COM_TIM11vec#
#43 equ TIM1_CCvec#
#44 equ TIM2vec#
#45 equ TIM3vec#
#46 equ TIM4vec#
#47 equ I2C1_EVvec#
#48 equ I2C1_ERvec#
#49 equ I2C2_EVvec#
#50 equ I2C2_ERvec#
#51 equ SPI1vec#
#52 equ SPI2vec#
#53 equ USART1vec#
#54 equ USART2vec#
#55 equ USART3vec#
#56 equ ExtI15_10vec#
#57 equ RTCalarmVec#
#58 equ OTG_FS_WKUPVec#
#59 equ TIM8_BRK_TIM12Vec#
#60 equ TIM8_UP_TIM13vec#
#61 equ TIM8_TRG_COM_TIM14
#62 equ TIM8_CCVec#
#63 equ DMA1S7Vec#
#64 equ FSMCVec#
#65 equ SDIOVec#
#66 equ TIM5Vec#
#67 equ SPI3Vec#
#68 equ UART4Vec#
#69 equ UART5Vec#
#70 equ TIM6_DACVec#
#71 equ TIM7Vec#
#72 equ DMA2S0Vec#
#73 equ DMA2S1Vec#
#74 equ DMA2S2Vec#
#75 equ DMA2S3Vec#
#76 equ DMA2S4Vec#
#77 equ ETHvec#
#78 equ ETH_WKUPvec#
#79 equ CAN2_TXvec#
#80 equ CAN2_RX0vec#
#81 equ CAN2_RX1vec#
#82 equ CAN2_SCEvec#
#83 equ OTG_FSvec#
#84 equ DMA2S5vec#
#85 equ DMA2S6vec#
#86 equ DMA2S7vec#
#87 equ USART6vec#
#88 equ I2C3_EVvec#
#89 equ I2C3_ERvec#
#90 equ OTG_HS_EP1_OUTvec#
#91 equ OTG_HS_EP1_INvec#
#92 equ OTG_HS_EP1_WKUPvec#
#93 equ OTG_HSvec#
#94 equ DCMIvec#
#95 equ CRYPvec#
#96 equ HASH_RNGvec#
#97 equ FPUvec#


\ ***********************
\ *S Miscellaneous macros
\ ***********************

interpreter
: setMask	\ value mask addr -- ; cell operation
\ *G Clear the *\i{mask} bits at *\i{addr} and set (or) the
\ ** bits defined by *\i{value}.
  tuck @				\ -- value addr mask x
  swap invert and			\ -- value addr x'
  rot or				\ -- addr x''
  swap !
;
target


\ ======
\ *> ###
\ ======

decimal
