\ SysTickDisco030Nucleo.fth - Cortex-M0 ticker for STM32F030 Nucleo board

((
Copyright (c) 2010, 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140524 RDO004 R. Doering: Adapted to STM32F030 on STM32F030 Nucleo board.
20140207 SFP003 Converted for STM32F072B-DISCO board.
20110218 MPE002 Added timebase extensions.
))

only forth definitions
decimal

\ ===========
\ *> drivers
\ ===========
\ *S System Ticker
\ ****************
\ *P The system ticker uses the Cortex-M0 *\i{SysTick} timer.
\ ** Although the *\i{SysTick} timer is common to all Cortex-M0
\ ** devices, there is no guarantee that they have the same
\ ** clocking arrangements. Consequently there are different
\ ** drivers for different implementations.

\ *P The STM32 implementation is fed with the system clock and
\ ** has an option to divide it by 8, controlled by bit 2 of the
\ ** SysTick Control and Status Register (ARM DDI 0337, Rev E,
\ ** page 8-8).
\ *(
\ *B Bit2=0: SysTick clock is CCLK/8.
\ *B Bit2=1: SysTick clock is CCLK.
\ *)
\ *P This code always uses CCLK.

tick-ms system-speed #1000 */ 1- equ /SysTick	\ -- x
\ *G Initial value of the SysTick Reload register.

[undefined] ticks [if]
0 value ticks	\ -- u
\ *G Returns a 32 bit number of milliseconds that eventually
\ ** wraps.
[then]

variable LedActive	\ -- flag
\ *P If *\fo{LedActive} contains non-zero, the ticker is used
\ ** to flash the green LED.
  1 LedActive !

PB 3 pio: GreenLED
\ *G Green LED on Nucleo-031K6 maps to PB.3.

: SysTicker	\ --
\ *G Ticker ISR action.
  tick-ms addr ticks +!			\ update ticker
  LedActive @ if
    case  ticks $7FF and		\ 0..1023
      $000 of  GreenLED clrPin    endof
      $400 of  GreenLED setPin    endof
    endcase
  then
;
' SysTicker SysTickvec# EXC: SysTickISR	\ -- addr
\ *G Setting the high level ISR.

: start-clock	\ --
\ *G Initialise the system ticker to run with a period of
\ ** *\fo{tick-ms} milliseconds.
  GreenLED dup isOutput setPin
  _SCS
  /SysTick over stRVR + !		\ set ticker rate
  $0007 over stCSR + !			\ enable, bit 0 = enable, bit 1=interrupt
					\ bit 2 = clock source (1=sysclk, 0=sysclk/8)
  $FF over $0D23 + c!			\ set SysTick priority to lowest
  drop
  ei
;

: stop-clock	\ --
\ *G Stop the sytem ticker.
  bit0 _SCS stCSR + bic!  ;

: ms            \ n --
\ *G Waits for n milliseconds.
  ticks + begin
    pause  dup ticks - 0<
  until
  drop
;


\ ======
\ *> ###
\ ======

decimal

