\ SysTickFRDMKL25Z.fth - Cortex-M0 ticker

((
Copyright (c) 2010, 2011, 2012, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140707 MPE006 Conversion to Freescale FRDM-KL25Z board.
20120517 MPE005 Update for timebase option.
20110704 MPE004 Generic CM0 version.
20110627 MPE003 LPC11xx conversion.
20110218 MPE002 Added timebase extensions.
20100125 MPE001 First release.
))

only forth definitions
decimal

\ ===========
\ *! systick
\ *T System Ticker
\ ===========
\ *P The system ticker uses the Cortex-M0 *\i{SysTick} timer.
\ ** Although the *\i{SysTick} timer is common to all Cortex-M0
\ ** devices, there is no guarantee that they have the same
\ ** clocking arrangements. Consequently there are different
\ ** drivers for different implementations.

\ *P The CMO implementation is fed with the system clock and
\ ** has an option controlled by bit 2 of the
\ ** SysTick Control and Status Register (ARM DDI 0337, Rev E,
\ ** page 8-8).
\ *(
\ *B Bit2=0: SysTick clock is vendor defined
\ *B Bit2=1: SysTick clock is CCLK.
\ *)
\ *P This code always uses CCLK.

PTB 18 pio: led1	\ -- struct
\ *G Red LED.
PTB 19 pio: led2	\ -- struct
\ *G Green LED.
PTD  1 pio: led3	\ -- struct
\ *G Blue LED.

variable LedActive	\ -- flag
\ *G If *\fo{LedActive} contains non-zero, the ticker is used
\ ** to produce a sequence on the tri-colour LEDs.
  1 LedActive !
variable <ticks>
\ *G Holds the current ticker value.

: ticks		\ -- ms
\ *G Returns a 32 bit number of milliseconds that eventually
\ ** wraps.
  <ticks> @
;
compiler
: ticks		\ -- ms
  <ticks> @
;
target

: SysTicker	\ --
\ *G High level version of ticker ISR.
  tick-ms <ticks> +!			\ update ticker
  LedActive @ if
    case  ticks $7FF and		\ 0..2047
      $000 of  led3 setPin  endof		\ off
      $200 of  led3 setPin  led1 clrPin  endof	\ red
      $400 of  led1 setPin  led2 clrPin  endof	\ green
      $600 of  led2 setPin  led3 clrPin  endof	\ blue
    endcase
  then
;
' SysTicker SysTickvec# EXC: SysTickISR	\ -- addr
\ *G Setting the high level ISR.

tick-ms system-speed #1000 */ 1- equ /SysTick	\ -- x
\ *G Initial value of the SysTick Reload register.

: start-clock	\ --
\ *G Initialise the system ticker to run with a period of
\ ** *\fo{tick-ms} milliseconds.
  led1 isOutput  led2 isOutput  led3 isOutput
  _SCS
  /SysTick over stRVR + !		\ set ticker rate
  $0007 over stCSR + !			\ enable it
\ bit 0 = enable, bit 1 = interrupt, bit 2 = clock source (1=sysclk, 0=sysclk/2)
  $FF over $0D23 + c!			\ set SysTick priority to lowest
  drop
  ei
;

: stop-clock	\ --
\ *G Stop the sytem ticker.
  bit0 _SCS stCSR + bic!  ;

: later         \ n -- n'
\ *G Generates the timebase value for termination in
\ ** n millseconds time.
  ticks +
;

: timedout?     \ n -- flag ; true if timed out
\ *G Flag is returned true if the timebase value n has timed out.
\ ** *\fo{TIMEDOUT?} does not call *\fo{PAUSE}.
  ticks - 0<                            \ must use circular arithmetic
;

compiler
: later         \ n -- n'
  ticks +
;

: timedout?     \ n -- flag ; true if timed out
  ticks - 0<                            \ must use circular arithmetic
;
target

: ms            \ n --
\ *G Waits for n milliseconds.
  later begin
    pause  dup timedout?
  until
  drop
;


\ ======
\ *> ###
\ ======

decimal

