\ SysTickXMC2Go.fth - Cortex-M0 ticker with LEDs for XMC2Go board.

((
Copyright (c) 2010, 2011, 2012, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20151218 MPE006 Infineon XMC1100 XMC2Go conversion.
20120517 MPE005 Update for timebase option.
20110704 MPE004 Generic CM0 version.
20110627 MPE003 LPC11xx conversion.
20110218 MPE002 Added timebase extensions.
20100125 MPE001 First release.
))

only forth definitions
decimal

\ ===========
\ *! systick
\ *T System Ticker
\ ===========
\ *P The system ticker uses the Cortex-M0 *\i{SysTick} timer.
\ ** Although the *\i{SysTick} timer is common to all Cortex-M0
\ ** devices, there is one clocking option. This code does not use
\ ** that option and so may be used by all CM0 devices.

\ *P The CMO implementation is fed with the system clock and
\ ** has an option controlled by bit 2 of the
\ ** SysTick Control and Status Register (ARM DDI 0337, Rev E,
\ ** page 8-8).
\ *(
\ *B Bit2=0: SysTick clock is vendor defined
\ *B Bit2=1: SysTick clock is CCLK.
\ *)
\ *P This code always uses CCLK.

tick-ms system-speed #1000 */ 1- equ /SysTick	\ -- x
\ *G Initial value of the SysTick Reload register.

[undefined] ticks [if]
0 value ticks		\ -- ms
\ *P Returns a milliseconds value that eventually wraps.
[then]

1 value LedActive?	\ -- flag
\ *G If *\fo{LedActive?} returns non-zero, the ticker is used
\ ** to produce a pattern on the LEDs.

P1 0 pio: LED1	\ -- struct
\ *G LED1 is on P1.0.
P1 1 pio: LED2	\ -- struct
\ *G LED2 is on P1.1.

: SysTicker	\ --
\ *G High level version of ticker ISR.
  tick-ms addr ticks +!			\ update ticker
  LedActive? if
    case  ticks $7FF and		\ 0..2047
      $000 of  LED1 clrPin  LED2 setPin  endof
      $400 of  LED1 setPin  LED2 clrPin  endof
    endcase
  then
[ timebase? ] [if]
  do-timers				\ run timebase code
[then]
;
' SysTicker SysTickvec# EXC: SysTickISR	\ -- addr
\ *G Setting the high level ISR.

: start-clock	\ --
\ *G Initialise the system ticker to run with a period of
\ ** *\fo{tick-ms} milliseconds.
  LED1 isOutput  LED2 isOutput		\ set up GPIO for LEDs
  _SCS
  /SysTick over stRVR + !		\ set ticker rate
  $0007 over stCSR + !			\ enable it
\ bit 0 = enable, bit 1 = interrupt, bit 2 = clock source (1=sysclk, 0=sysclk/x)
  $FF over $0D23 + c!			\ set SysTick priority to lowest
  drop
  ei
;

: stop-clock	\ --
\ *G Stop the sytem ticker.
  bit0 _SCS stCSR + bic!  ;

[undefined] ms [if]
: ms            \ n --
\ *G Waits for n milliseconds.
  ticks +
  begin
[ tasking? ] [if]  pause  [then]
    dup ticks - 0<
  until
  drop
;
[then]


\ ********************
\ Time base extensions
\ ********************

timebase? [if]
: start-timers  \ -- ; Start internal time clock
\ *G Initialise the TIMEBASE system.
  stop-clock init-timers start-clock
;

: stop-Timers   \ -- ; disable timer system
\ *G Stop the TIMEBASE ticker.
  stop-clock
;
[then]


\ ======
\ *> ###
\ ======

decimal

