\ gpioLPC11xx.fth - easy access to LPC11xx GPIO pins.

((
Copyright (c) 2011, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20110608 SFP001 Adapted from LPC17xx version.
))

only forth definitions
decimal

\ ===========
\ *! gpiolpc11xx
\ *T LPC11xx GPIO utilities
\ ===========
\ *P The code in *\i{Cortex\gpioLPC11xx.fth} provides utility
\ ** words for accessing the GPIO pins on a bit by bit basis.
\ ** The code is written for ease of use rather than performance.


\ ****************
\ *S Configuration
\ ****************
\ *P The equates in this block are defaults used if the equates
\ ** have not been defined before this file is compiled.

interpreter
: within?  within?  ;
target

_GPIO0data equ PortDataBase	\ -- addr
\ *G Base address of port definitions for data.
_GPIO0cfg equ PortCfgBase	\ -- addr
\ *G Base address of port definitions for data.
$1:0000 equ /PortBlock	\ -- len
\ *G Number of bytes per port for peripheral registers.
16 equ #PortShift	\ -- u
\ *G Number of bits to shift to offset by *\fo{/PortBlock}.


\ ********************
\ *S Defining I/O pins
\ ********************
\ *P FIO/GPIO acess is defined using a bit number. Bits 0..11 form
\ ** P0.0 to P0.11, bits 32..43 form P1.0 to P1.11 and so on.

target-only
: PIO:		\ port# bit# -- ; -- iobit#
\ *G Define a port I/O bit by name. For example
\ *C   2 11 PIO: P2.11
\ ** defines an I/O bit on port 2, bit 11. At run time, the I/O
\ ** bit number is returned.
  swap 5 lshift or constant  ;
host&target
interpreter
: PIO:		\ port# bit# -- ; -- iobit#
  swap 5 lshift or constant  ;
target


\ ****************
\ *S IO pin access
\ ****************

compiler
: >IOdata	\ iobit# -- addr
\ Used to form the address of the data port for the I/O bit.
  dup $1F and 4 swap lshift		\ -- iobit# mask<<2
  swap 5 rshift #Portshift lshift	\ -- mask offset
  + PortDataBase +			\ -- address
;
: >IOcfg	\ iobit# -- mask address
\ Forms a mask and base address for configuration registers.
  dup $1F and 1 swap lshift		\ -- iobit# mask
  swap 5 rshift #Portshift lshift	\ -- mask offset
  PortCfgBase +				\ -- address
;
target

: writePin	\ x iobit# --
\ *G Set the pin low if x=0, otherwise set it high.
  swap 0<> swap >IOdata ! ;

: setPin	\ iobit# --
\ *G Set the pin high.
  -1 swap writePin  ;

: clrPin	\ iobit# --
\ *G Set the pin low.
  0 swap writePin  ;

: getPin	\ iobit# -- 0/1
\ *G Read the pin state.
  >IOdata @ 0<> 1 and  ;

: IOdata	\ iobit# -- addr
  >IOdata  ;


\ ***********************
\ *S IO pin configuration
\ ***********************

#12 equ bits/port

create GPIOcfgs
l: PIO0cfg
  IOCON_RESET_PIO0_0 ,
  IOCON_PIO0_1 ,
  IOCON_PIO0_2 ,
  IOCON_PIO0_3 ,
  IOCON_PIO0_4 ,
  IOCON_PIO0_5 ,
  IOCON_PIO0_6 ,
  IOCON_PIO0_7 ,
  IOCON_PIO0_8 ,
  IOCON_PIO0_9 ,
  IOCON_SWCLK_PIO0_10 ,
  IOCON_R_PIO0_11 ,
l: PIO1cfg
  IOCON_R_PIO1_0 ,
  IOCON_R_PIO1_1 ,
  IOCON_R_PIO1_2 ,
  IOCON_SWD_PIO1_3 ,
  IOCON_PIO1_4 ,
  IOCON_PIO1_5 ,
  IOCON_PIO1_6 ,
  IOCON_PIO1_7 ,
  IOCON_PIO1_8 ,
  IOCON_PIO1_9 ,
  IOCON_PIO1_10 ,
  IOCON_PIO1_11 ,
l: PIO2cfg
  IOCON_PIO2_0 ,
  IOCON_PIO2_1 ,
  IOCON_PIO2_2 ,
  IOCON_PIO2_3 ,
  IOCON_PIO2_4 ,
  IOCON_PIO2_5 ,
  IOCON_PIO2_6 ,
  IOCON_PIO2_7 ,
  IOCON_PIO2_8 ,
  IOCON_PIO2_9 ,
  IOCON_PIO2_10 ,
  IOCON_PIO2_11 ,
l: PIO3cfg
  IOCON_PIO3_0 ,
  IOCON_PIO3_1 ,
  IOCON_PIO3_2 ,
  IOCON_PIO3_3 ,
  IOCON_PIO3_4 ,
  IOCON_PIO3_5 ,

: isInput	\ iobit# --
\ *G Set the pin as an input.
  >IOcfg GPIOnDIR + bic!  ;

: isOutput	\ iobit# --
\ *G Set the pin as an input.
  >IOcfg GPIOnDIR + or!  ;

: >IOmode	\ iobit# -- addr
\ *G Return the address of the I/O bit's mode register.
  dup 5 rshift bits/port cells *	\ offset of port base
  swap $1F and cells +			\ offset to port.x mode register
  PIO0cfg + @				\ fetch offset from GPIOcfg table
  _IOCONFIG +				\ add base address
;

: isPinCon	\ selbits iobit# --
\ *G Set the pin configuration 0..3. By default, nearly all pins
\ ** are in I/O pin mode after reset. For most function registers,
\ ** use 6 bits consisting of hysteresis, mode and function.
\ ** Function 0 is for GPIO use in most but not all cases.
  >IOmode !  ;

bit5 constant HYS	\ -- mask
\ *G Bit mask to set hysteresis in a pin configuration register.

$00 3 lshift constant NotPulled		\ -- mode#
\ *G Used as the mode# with *\fo{isPinCon} above to define how
\ ** the pin operates as an input.
$01 3 lshift constant PulledDown	\ -- mode#
\ *G Used as the mode# with *\fo{isPinCon} above to define how
\ ** the pin operates as an input.
$02 3 lshift constant PulledUp		\ -- mode#
\ *G Used as the mode# with *\fo{isPinCon} above to define how
\ ** the pin operates as an input.
$03 3 lshift constant RptrMode		\ -- mode#
\ *G Used as the mode# with *\fo{isPinCon} above to define how
\ ** the pin operates as a repeater.

\ *P The function of each pin is defined by a 3 bit field in
\ ** bits 2:0 of the bit pattern used with *\fo{isPinCon}, e.g.
\ *C   1 8 PIO: PIO1.8
\ *C   HYS PulledUp or 1 or  PIO1.8 isPinCon


\ ============
\ *S Test code
\ ============

((
\ *[
decimal

1 25 pio: led1
0  4 pio: led2
1 18 pio: Usb_Link_Led
2  9 pio: Usb_Connect_Led
0 23 pio: But1
2 13 pio: But2

: tled1		\ --
  led1 isOutput
  begin
    led1 clrPin  200 ms
    led1 setPin  200 ms
  key? until
;

: tled2		\ --
  led2 isOutput
  begin
    led2 clrPin  200 ms
    led2 setPin  200 ms
  key? until
;
\ *]
))


\ ======
\ *> ###
\ ======

decimal
