\ serKL25p.fth - polled serial driver for Kinetis KL25.

((
Copyright (c) 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20141122 MPE002 Corrected SERKEY0.
20140530 MPE001 First release.
))

only forth definitions  decimal

\ ===============
\ *! serKL25p
\ *T Freescale KL25 polled serial
\ ===============
\ *P The file *\i{Cortex/Drivers/serKL25p.fth} contains a polled
\ ** serial driver for Freescal KL25 and compatible devices.

: +FaultConsole	( -- )  ;
\ *G Because this is a polled driver, *\fo{+FaultConsole} for
\ ** fault handling is a *\fo{NOOP}. See *\i{Cortex\FaultCortex.fth}
\ ** for more details.


\ ********
\ *S UART0
\ ********

useUART0? [if]
\ Baud = systemspeed / 2 ( oversample * divisor )
\ Divisor = systemspeed / ( 2 * oversample * baud )
\         = systemspeed / ( 32 * baud )

8 equ /OVL0	\ -- x
\ *G Oversample value for UART0

system-speed console0-speed /
/OVL0 2* /mod swap /OVL0 >=
[if] 1+ [then]
equ U0divisor
\ *G Baud rate divisor for UART0.
U0divisor $1FFF and
dup 8 rshift equ /U0BDH	\ -- x
\ *G UnBDH initial value.
     $FF and equ /U0BDL	\ -- x
\ *G UnBDL initial value.

: init-ser0	\ --
\ *G Initialise UART0 on PTA1 and PTA2.
  $0401:0000 $0C01:0000 _SIM simSOPT2 + setMask	\ UART0SRC = 01, source=PLL/2
  bit10 _SIM simSCGC4 + or!		\ enable UART clock
  _PORTA
  $0200 over portPCR1 + !		\ Alt 2, no ints
  $0200 over portPCR2 + !		\ Alt 2, no ints
  drop
  _UART0
  0 over UnC2 + c!			\ disable
  0 over UnC1 + c!			\ 8 bit, no parity
  0 over UnC3 + c!			\ no Tx invert, no ints
  0 over UnS2 + c!			\ no Rx invert
  /OVL0 1- over U0C4 + c!		\ oversample by X
  /U0BDH over UnBDH + c!		\ baud rate divisor
  /U0BDL over UnBDL + c!
  $0C over UnC2 + c!			\ Enable Tx, Rx
  drop
;

: serEmit0	\ char --
\ *G *\fo{EMIT} for UART0.
  _UART0 begin				\ -- char uart
    dup UnS1 + c@ bit7 and
  until
  UnD + c!
;

: serKey?0	\ -- flag
\ *G *\fo{KEY?} for UART0.
  _UART0 UnS1 + c@ bit5 and 0<>
;

: serKey0	\ -- char
\ *G *\fo{KEY} for UART0.
  _UART0
  $0F over UnS1 + c!			\ clear error flags
  begin					\ -- uart
    dup UnS1 + c@ bit5 and 0=
   while
    pause
  repeat
  UnD + c@
;

: serType0	\ caddr len --
\ *G *\fo{TYPE} for UART0.
  bounds ?do  i c@ serEmit0  loop
;

: serCR0	\ --
\ *G *\fo{CR} for UART0.
  $0D serEmit0  $0A serEmit0
;

create Console0	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for UART0.
  ' serkey0 ,		\ -- char ; receive char
  ' serkey?0 ,		\ -- flag ; check receive char
  ' seremit0 ,		\ -- char ; display char
  ' sertype0 ,		\ caddr len -- ; display string
  ' sercr0 ,		\ -- ; display new line

console-port 0 = [if]
  console0 constant console
\ *G *\fo{CONSOLE} is the device used by the Forth system for interaction.
\ ** It may be changed by one of the phrases of the form:
\ *C   <device>  dup opvec !  ipvec !
\ *C   <device>  SetConsole
[then]

[then]


\ ************************
\ *S System initialisation
\ ************************

: init-ser	\ --
\ *G Initialise all serial ports
  [ useUART0? ] [if]  init-ser0  [then]
  [ useUART1? ] [if]  init-ser1  [then]
  [ useUART2? ] [if]  init-ser2  [then]
;


\ ======
\ *> ###
\ ======

decimal

