\ serPSUART.fth - polled pseudo-UART communication drivers

((
Copyright (c) 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20131029 MPE001 First release.
))

only forth definitions


\ ==============
\ *! serpsuart
\ *T Polled PSUART serial driver
\ ==============
\ *P This driver provides polled comms for systems using a Segger
\ ** J-Link or other JTAG/SWD unit to implement the host side.

struct /PSUART-RAM	\ -- len
\ *G Describes the four byte block used to communicate with the
\ ** host.
\ *[
  1 field pr.status-h>t	\ set by host when data is ready
  			\ cleared by target after data has been read
  1 field pr.data-h>t	\ written by host
  1 field pr.status-t>h	\ set by target when data is ready
  			\ cleared by host after data has been read
  1 field pr.data-t>h	\ written by target
end-struct
\ *]

uhere label PSUART0  ( -- addr )  /PSUART-RAM uallot
\ *G Reserve the four byte block.


\ ********************
\ *S Serial primitives
\ ********************

internal

: +FaultConsole	( -- )  ;
\ *G Because this is a polled driver, *\fo{+FaultConsole} for
\ ** fault handling is a *\fo{NOOP}. See *\i{Cortex/FaultCortex.fth}
\ ** for more details.

: psEmit0	\ char --
\ *G Transmit a character on the UART.
  PSUART0
  begin					\ -- char base
[ tasking? ] [if] pause [then]
    dup pr.status-t>h c@ 0=		\ wait until previous byte taken
  until
  tuck pr.data-t>h c!			\ set data
  $FF swap pr.status-t>h c!		\ flag it
;

: psType0	\ caddr len --
\ *G Transmit a string on the UART.
  bounds
  ?do  i c@ psEmit0  loop
;

: psCr0		\ --
\ *G Transmit a CR/LF pair on the UART.
  $0D psEmit0  $0A psEmit0
;

: psKey?0	\ -- t/f
\ *G Return true if the UART has a character available to read.
  PSUART0 pr.status-h>t c@ 0<>
;

: psKey0	\ -- char
\ *G Wait for a character to come available on the UART and
\ ** return the character.
  PSUART0
  begin					\ wait for previous byte to be taken
[ tasking? ] [if]  pause  [then]
    dup pr.status-h>t c@
  until					\ -- base
  dup pr.data-h>t c@			\ -- base char ; read data
  0 rot pr.status-h>t c!		\ clear flag
;

external

create ConsolePS	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for UART0.
  ' psKey0 ,		\ -- char ; receive char
  ' psKey?0 ,		\ -- flag ; check receive char
  ' psEmit0 ,		\ -- char ; display char
  ' psType0 ,		\ caddr len -- ; display string
  ' psCr0 ,		\ -- ; display new line
console-port 20 = [if]
  consolePS constant console
\ *G *\fo{CONSOLE} is the device used by the Forth system for interaction.
\ ** It may be changed by one of the phrases of the form:
\ *C   <device>  dup opvec !  ipvec !
\ *C   <device>  SetConsole
[then]

[then]


\ ************************
\ *S System initialisation
\ ************************

: init-ser	\ --
\ *G Initialise all serial ports
  0 PSUART0 !
;


\ ======
\ *> ###
\ ======

decimal
