\ l3gd20.fth - interface to MEMS gyro sensor using SPI.

((
Copyright (c) 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! l3gd20
\ *T L3GD20 MEMS Gyro
\ ===========
\ *P The STM32F072B Discovery board includes an L3GD20 3-axis
\ ** MEMS gyro. The ST documentation is a bit sparse. We use
\ ** the SPI2 peripheral to talk to the gyro. The MEMS code
\ ** is in the file *\i{Examples\l3gd20.fth} and the SPI driver
\ ** is in the file *\i{spiSTM32F0hard.fth}.

PC 1 pio: MEMS_INT1	\ -- struct
\ *G MEMS INT1 connected here.
PC 2 pio: MEMS_INT2
\ *G MEMS INT2 connected here.

: L3GD20_Write1	\ b reg --
\ *G Write one byte to the given register.
  CSlo_MEMS				\ select chip
  $3F and spi!				\ r/w=0, mbyte=0, write address
  spi!					\ write data
  CShi_MEMS				\ deselect chip
;

: L3GD20_Read1	\ reg -- b
\ *G Read one byte from the given register.
  CSlo_MEMS				\ select chip
  $3F and $80 or spi!			\ r/w=1, mbyte=0, write address
  spi@					\ read data
  CShi_MEMS				\ deselect chip
;

: L3GD20_ReadN	\ caddr len reg --
\ *G Read *\i{len} bytes to *\i{caddr} starting from the given register.
  over 0=
  if  drop 2drop exit  endif
  CSlo_MEMS				\ select chip
  $3F and $C0 or spi!			\ r/w=1, mbyte=1, write reg
  bounds do  spi@ i c!  loop		\ read into buffer
  CShi_MEMS				\ deselect chip
;

\ L3GD20 register definitions

$0F equ L3GD20.WHO_AM_I
$20 equ L3GD20.CTRL_REG1
$21 equ L3GD20.CTRL_REG2
$22 equ L3GD20.CTRL_REG3
$23 equ L3GD20.CTRL_REG4
$24 equ L3GD20.CTRL_REG5
$25 equ L3GD20.REFCAP
$26 equ L3GD20.OUT_TEMP
$27 equ L3GD20.STATUS_REG
$28 equ L3GD20.OUT_X_L
$2A equ L3GD20.OUT_Y_L
$2C equ L3GD20.OUT_Z_L
$2E equ L3GD20.FIFO_CTRL_REG
$2F equ L3GD20.FIFO_SRC_REG
$30 equ L3GD20.INT1_CFG
$31 equ L3GD20.INT1_SRC
$32 equ L3GD20.INT1_THS_XH
$33 equ L3GD20.INT1_THS_XL
$34 equ L3GD20.INT1_THS_YH
$35 equ L3GD20.INT1_THS_YL
$36 equ L3GD20.INT1_THS_ZH
$37 equ L3GD20.INT1_THS_ZL
$38 equ L3GD20.INT1_DURATION

: setMaskL3	\ value mask reg --
\ *G Clear the *\i{mask} bits in *\i{reg}. Write *\i{value} into
\ ** the mask bits in the given L3GD20 register.
  tuck L3GD20_Read1			\ -- value reg mask x
  swap invert and			\ -- value reg x'
  rot or				\ -- addr x''
  swap L3GD20_Write1
;

: borL3		\ mask reg --
\ *G Set the bits in mask in the given register.
  over swap setMaskL3
;

: bbicL3		\ mask reg --
\ *G Clear the bits in mask in the given register.
  0 -rot setMaskL3
;

: L3GD20_RebootCmd	\ --
\ *G Reboot memory content of L3GD20.
  $80 L3GD20.CTRL_REG5 borL3		\ BOOT=1
;

: L3GD20_GetDataStatus	\ -- b
\ *G Get status for L3GD20 data.
  L3GD20.STATUS_REG L3GD20_Read1
;

((
variable L3exc-count

: ExtI0_1-isr
  _EXTI extiPR +  bit0 over !		\ clear EXTI0
  @ bit1 and if
    L3GD20.INT1_SRC L3GD20_Read1 drop	\ read to clear
    bit1 _EXTI extiPR + !		\ clear EXTI1
    1 L3exc-count +!
  endif
;
' ExtI0_1-isr ExtI0_1Vec# Exc: ExtI0-interrupt

: L3GD20_EXTI_Config	\ --
\ *G Configure the INT1 MEMS Interrupt line and GPIO in EXTI mode.
\ ** The GPIO and SPI blocks are already enabled.
  bit0 _RCC				\ enable SYSCFG
    2dup rccAPB2en + or!
    rccAPB2rst + bic!
  MEMS_INT1 dup isInput  NotPulled
  MEMS_INT2 dup isInput  NotPulled
  $0020 $00F0 _SYSCFG sysEXTICR1 + setMask	\ interrupt on PC1 is EXTI1
  bit1 _EXTI
    2dup extiIMR + or!				\ EXTI1 interrupt mode
    extiFTSR + or!				\ EXTI1 falling trigger

  $0F ExtI0_1Vec# setPri			\ lowest priority
  ExtI0_1Vec# enInt				\ enable interrupt
;

: L3GD20_INT_Config	\ --
\ *G Configure L3GD20 interrupts.
  $2A L3GD20.INT1_CFG L3GD20_Write1	\ Enable from high portion of each axis.
  3   L3GD20.INT1_THS_XH L3GD20_Write1	\ set axis threshold to $300
  0   L3GD20.INT1_THS_XL L3GD20_Write1
  3   L3GD20.INT1_THS_YH L3GD20_Write1
  0   L3GD20.INT1_THS_YL L3GD20_Write1
  3   L3GD20.INT1_THS_ZH L3GD20_Write1
  0   L3GD20.INT1_THS_ZL L3GD20_Write1
  $10 L3GD20.INT1_DURATION L3GD20_Write1
;
))

: init-L3GD20	\ --
\ *G Initialise L3GD20.
  initSPI2
  MEMS_INT1 dup isInput  NotPulled
  MEMS_INT2 dup isInput  NotPulled
  $0F L3GD20.CTRL_REG1 L3GD20_Write1	\ DR=00, BW=00, PD=1, En=1
  $00 L3GD20.CTRL_REG2 L3GD20_Write1	\ HPM=00, HPCF=0000
  $00 L3GD20.CTRL_REG3 L3GD20_Write1	\ all defaults
  $90 L3GD20.CTRL_REG4 L3GD20_Write1	\ BDU=0, BLE=0, FS1/0=00, SIM=0
  $40 L3GD20.CTRL_REG5 L3GD20_Write1	\ FIFO_EN=1, HPen=0
  $00 L3GD20.FIFO_CTRL_REG L3GD20_Write1	\ FIFO Bypass mode
\  L3GD20_EXTI_Config
\  L3GD20_INT_Config
;

: L3GD20_INT1InterruptCmd	\ flag --
\ *G Enable or disable INT1 interrupt.
  0<> $80 and $80 L3GD20.CTRL_REG3 setMaskL3
;

: L3GD20_INT2InterruptCmd	\ --
\ *G Enable or disable INT2 interrupt.
  0<> $08 and $08 L3GD20.CTRL_REG3 setMaskL3
;


\ *********************
\ *S Test and Demo code
\ *********************

: tr	  \ --
\ *G Display the L3GD20 identification register.
  L3GD20.WHO_AM_I L3GD20_Read1 .byte
;

6 buffer: AxisData	\ -- addr
\ *G Buffer for axis data.

: ReadAxes	\ --
\ *G Read the axes into the *\fo{AxisData} buffer. The data
\ ** consists of three 16 bit signed values for the X, Y and Z
\ ** axes.
  begin  L3GD20.STATUS_REG L3GD20_Read1 bit3 and  until
  AxisData 6 L3GD20.OUT_X_L L3GD20_ReadN
;

: .Axes		\ --
\ *G Display the axis data.
  cr  AxisData
  ." X = " dup w@s .  2+
  ." Y = " dup w@s .  2+
  ." Z = " dup w@s .  2+
  drop
;

: ReadTest	\ --
\ *G Display the axis data until a key is pressed.
  init-l3gd20
  begin
    ReadAxes .Axes  100 ms
    key?
  until
;

PA 0 pio: UserButton	\ -- struct
\ *G Pin definition for the User/Wake button on PA0.

: LedsOff	\ --
\ *G Turn off the four LEDs.
  LDU clrPin  LDL clrPin  LDD clrPin  LDR clrPin
;

: Xval		\ -- n
\ *G Return the X value from the last reading of the axes.
  AxisData w@s
;

: Yval		\ -- n
\ *G Return the Y value from the last reading of the axes.
  AxisData 2 + w@s
;

: Zval		\ -- n
\ *G Return the Z value from the last reading of the axes.
  AxisData 4 + w@s
;

: onlyLED	\ struct
\ *G Turn on the given LED only and wait 250 ms.
  LedsOff  setPin  250 ms
;

: DemoMEMS	\ --
\ *G Demo for the MEMS sensor.
  cr ." The LEDs will follow the motion of the board"
  cr ." until you press the USER button"
  LedActive off  LedsOff		\ disable the LED ticker
  init-l3gd20
  UserButton isInput
  begin
    ReadAxes
    Xval abs Yval Abs > if
      Xval 40 >
      if  LDR onlyLED  endif
      Xval -40 <
      if  LDL onlyLED  endif
    else
      Yval 40 >
      if  LDU onlyLED  endif
      Yval -40 <
      if  LDD onlyLED  endif
    endif
    pause
    UserButton getPin
  until
  begin  pause  UserButton getPin 0=  until
  LedsOff  LedActive on
  cr
;


\ ======
\ *> ###
\ ======

decimal
