\ FaultCortex.fth - Exception fault handling

((
Copyright (c) 2010, 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140703 SFP002 Updated to trigger AIDE if including.
20110628 SFP001 Extended for Cortex-M0/M1.
))

only forth definitions
decimal

\ ==============
\ *! faultcortex
\ *T Exception Fault handling
\ ==============
\ *P The code in *\i{Cortex/FaultCortex.fth} provides debugging
\ ** facilities for when a fault occurs that triggers an exception.


\ **************************
\ *S Fault handler framework
\ **************************

struct /AppFrame	\ -- len
\ *G Structure defining what is saved on the application's
\ ** stack.
  int exa.r0
  int exa.r1
  int exa.r2
  int exa.r3
  int exa.r12
  int exa.r14
  int exa.PC
  int exa.PSR
end-struct

struct /MainFrame	\ -- len
  int exm.r4
  int exm.r5
  int exm.r6
  int exm.r7
  int exm.r8
  int exm.r9
  int exm.r10
  int exm.r11
  int exm.ExcRet	\ LR after entry
end-struct

struct /ExData	\ -- len
\ *G A structure holding data saved on entry to the exception
\ ** routine
  int exd.R13flt	\ Faulting stack on entry
  int exd.R13main	\ main stack on entry
  int exd.R13process	\ process stack on entry
  int exd.R13state	\ main stack after state save
  int exd.LRmain	\ LR of main stack on entry
end-struct

/ExData reserve constant ExData	\ -- addr
\ *G Holds additional data about the fault.

Not-M0/M1? [if]
PROC FltEntry	\ --
\ *G This is the template code for Cortex-M3+ fault handlers.
\ ** R0..R3 have already been saved by the hardware
\ --- save fault data ---
  mvl     r3, # ExData			\ point to save buffer
  mrs     r0, SP_main			\ save stack pointers
  str     r0, [ r3, # 0 exd.R13main ]
  mrs     r1, SP_process
  str     r1, [ r3, # 0 exd.R13process ]
  str     lr, [ r3, # 0 exd.LRmain ]	\ save LR
  tst     lr, # bit2
  ite .eq
    str     r0, [ r3, # 0 exd.R13flt ]	\ faulting stack is SP_main
    str     r1, [ r3, # 0 exd.R13flt ]	\ faulting stack is SP_process
\ --- save state --- must match /MainFrame structure above
  push    { r4-r11, r14 }		\ registers not saved by core + link
  str     r13, [ r3, # 0 exd.R13state ]
  sub     rsp, rsp, # up-size sp-size +	\ make space for USER area and data stack

\ --- set up Forth ---
  add   up, rsp, # sp-size		\ USER area at top
  sub	psp, up, # sp-guard cells	\ generate new PSP
  sub	r4, psp, # tos-cached? cells	\ generate new S0
  str   r4, [ up, # s0-offset ]		\   that is compatible with SP!
  str   rsp, [ up, # r0-offset ]	\ set R0
\ --- call high level handler ---
l: FltCall
  bl      ' noop			\ call the high level handler
\ --- restore state ---
  add     rsp, rsp, # up-size sp-size +	\ remove space for USER area and data stack
  pop     { r4-r11, pc }		\ restore and exit
end-code
chere FltEntry - equ /FltEntry		\ size of template
FltCall FltEntry - equ /FltCall		\ offset to call instruction
[else]
PROC FltEntry	\ --
\ *G This is the template code for Cortex-M0/M1 fault handlers.
\ ** R0..R3 have already been saved by the hardware
\ --- save fault data ---
  ldr     r3, ^ExData			\ point to save buffer
  mrs     r0, SP_main			\ save stack pointers
  str     r0, [ r3, # 0 exd.R13main ]
  mrs     r1, SP_process
  str     r1, [ r3, # 0 exd.R13process ]
  mov     r2, lr
  str     r2, [ r3, # 0 exd.LRmain ]	\ save LR
  mov .s  r0, # bit2
  tst     r2, r0
  eq, if,
    ldr     r0, [ r3, # 0 exd.R13main ]	\ faulting stack is SP_main
  else,
    ldr     r0, [ r3, # 0 exd.R13Process ]	\ faulting stack is SP_process
  endif,
  str     r0, [ r3, # 0 exd.R13flt ]	\ faulting stack pointer
\ --- save state --- must match /MainFrame structure above
  mov     r0, r8
  mov     r1, r9
  mov     r2, r10
  mov     r3, r11
  push    { r0-r3, r14 }		\ R8-R11 not saved by core + link
  push    { r4-r7 }			\ R4-R7

  ldr     r3, ^ExData			\ point to save buffer
  mov     r0, r13
  str     r0, [ r3, # 0 exd.R13state ]
  sub     rsp, rsp, # up-size sp-size +	\ make space for USER area and data stack

\ --- set up Forth ---
  mov     r0, rsp
  add .s  r0, r0, # sp-size		\ USER area at top
  mov     up, r0
  mov     psp, r0
  sub .s  psp, psp, # sp-guard cells	\ generate new PSP
  sub .s  r4, psp, # tos-cached? cells	\ generate new S0
  str     r4, [ r0, # s0-offset ]	\   that is compatible with SP!
  mov     r1, rsp
  str     r1, [ r0, # r0-offset ]	\ set R0
\ --- call high level handler ---
l: FltCall
  bl      ' noop			\ call the high level handler
\ --- restore state ---
  add     rsp, rsp, # up-size sp-size +	\ remove space for USER area and data stack
  pop     { r4-r7 }			\ original R4-R7
  pop     { r0-r3 }			\ original R8-11
  mov     r8, r0
  mov     r9, r1
  mov     r10, r2
  mov     r11, r3
  pop     { pc }			\ exit
end-code
align l: ^ExData
  ExData ,
chere FltEntry - equ /FltEntry		\ size of template
FltCall FltEntry - equ /FltCall		\ offset to call instruction
[then]

interpreter
: FLT:		\ xt vec# -- ; -- isr
\ *G Creates a fault handler that runs the given Forth word.
\ ** At run time the entry point of the ISR is returned. Use in
\ ** the form:
\ *C   ' <action> <vec#> FLT: <actionISR>
\ *P The example below will define a handler for the HardFault
\ ** exception that runs the Forth word *\fo{HFhandler}.
\ *C   ' HFhandler 3 FLT: HFentry
  swap align  chere /FltEntry callot	\ -- vec# xt isr ; reserve space
  FltEntry over /FltEntry move		\ copy template
  tuck /FltCall + !call			\ -- vec# isr ; form call instruction
  tuck swap setExcVec
  label					\ returns entry point
;
target

: .item		\ addr -- addr+4
\ *G Display a cell item at addr and increment addr.
  dup @ .dword space  cell +  ;
: .items	\ addr n -- addr+4n
\ *G Display n items at addr and increment addr.
  0 ?do  .item  loop  ;

: AppItems	\ -- addr
\ *G The address of fault data saved on the application's stack.
  ExData exd.R13flt @  ;
: MainItems	\ -- addr
\ *G The address of fault data saved on SP_main.
  ExData exd.R13state @  ;
: AppRSP	\ -- addr
\ *G The Forth return stack pointer at the fault.
  AppItems 8 cells +  ;
Cortex-M0/M1? [if]
: AppPSP	\ -- addr
\ *G The Forth data stack pointer at the fault for Cortex-M3+.
  MainItems exm.R6 @  ;
[else]
: AppPSP	\ -- addr
\ *G The Forth data stack pointer at the fault for Cortex-M3+.
  AppItems exa.R12 @  ;
[then]
: AppUP		\ -- addr
\ *G The Forth UP at the fault.
  MainItems exm.R11 @  ;
: AppTOS	\ -- x
\ *G The Forth TOS at the fault.
  MainItems exm.R7 @  ;
: AppPC	\ -- x
\ *G The PC at the fault.
  AppItems exa.PC @  ;

: .FltFrame	\ --
\ *G Display the CPU state pointed to by the data frame.
  AppItems				\ from the faulting stack
  cr ." === Registers ==="
  cr ." PSR = " dup exa.PSR @ .dword	\ PSR
  cr ." R0  = "  dup exa.R0 4 .items	\ R0..R3
  drop
  MainItems exm.R4			\ from SP_main
  cr ." R4  = "  4 .items		\ R4..R7
  cr ." R8  = "  4 .items		\ R8..R11
  drop
  AppItems				\ from the faulting stack
  cr ." R12 = "  dup exa.r12 .item drop	\ R12
  dup 8 cells + .dword space		\ R13 before fault
  exa.R14 2 .items drop			\ R14 R15
;

[undefined] ip>nfa [if]
: name?         \ addr -- flag
\ *G Check to see if the supplied address is a valid NFA.
\ ** This word is implementation dependent.
\ ** A valid NFA for MPE embedded systems satisfies the following:
\ *(
\ *B All characters within string are printable ASCII within range 33..126.
\ *B String Length is non-zero in range 1..31 and bit 7 is set, ignore bits 6, 5.
\ *)
  dup aligned over <>
  if  drop  FALSE exit  endif
  count					\ c-addr u --
  dup  $9F and  $81 $A0 within 0=	\ NFA first byte = 1SIxxxxx, count = xxxxx
  					\ mask           = 10011111
  if  2drop  0  exit  then
  $01F and bounds ?do
    i c@ #33 #127 within 0=		\ check all ascii chars
    if  unloop  FALSE exit  then
  loop
  TRUE
;

: ip>nfa	\ addr -- nfa
\ *G Attempt to move backwards from an address within a definition
\ ** to the relevant NFA.
  2-				\ NFA must be at least 'n' bytes backwards
  begin
    dup name? 0=
   while
    1-
  repeat
;
[then]

: check-aligned	\ addr -- addr'
\ *G Check addr for cell alignment, report and correct if
\ ** misaligned.
  dup aligned over <>
  if  ." (Misaligned) "  aligned  then
;

: ?Clip32	\ xsp xspTop -- xsp xspTop'
\ *G Clip the stack display to 32 items.
  2dup swap - #32 cells u> if
    drop  dup #32 cells +
    cr ." Clipped to 32 items"
  endif
;

[defined] Umbilical? [if]
' (do) equ start-of-code
[else]
' and equ start-of-code
[then]
prog sec-end  equ end-of-code

: .rsFault	\ --
\ *G Display the return stack of the fault, assuming
\ ** the Forth RSP=R13 and RUP=R11.
  cr  AppRSP				\ -- rsp
  cr ." RSP = " dup .dword space check-aligned
  AppUP r0-offset + @			\ -- rsp rsTop
  ."   R0 = " dup .dword space check-aligned
\ Note that for negative steps, +LOOP processes the limit value.
  cell -				\ rsTop is not used
  cr ." --- Return stack high ---"
  ?Clip32 ?do
    cr i .dword space  i @ dup .dword space
    dup start-of-code end-of-code within
    if  ip>nfa .name  else  drop  endif
  cell negate +loop
  cr ." --- Return stack low --- "
;

: .psFault	\ --
\ *G Display the data stack indicated by the frame, assuming
\ ** the Forth RSP=AppPSP and RUP=R11.
  cr  AppPSP				\ -- psp
  cr ." PSP = " dup .dword space  check-aligned
  AppUP s0-offset + @			\ -- psp psTop
  ."   S0 = " dup .dword space  check-aligned
\ Because of the stack caching S0 is a cell LOWER than PSP
\ for an empty stack.
  2dup cell + u>
  if  2drop  cr ." Data stack underflowed"  exit  endif
  2dup cell + =
  if  2drop  cr ." Data stack empty"  exit  endif
  2dup swap - SP-SIZE u>
  if  2drop  cr ." Data stack overflowed"  exit  endif
  cr ." --- Data stack high ---"	\ -- psp psTop/S0
  2dup <> if
    cell - ?Clip32 do			\ for -ve steps +LOOP runs the limit value
      cr i .dword space  i @ .dword
      cell negate
    +loop
  else
    2drop
  endif
  cr ." --- Data stack low --- "
  cr ." rTOS/R7  " AppTOS .dword
;


\ *************************
\ *S Default Fault handlers
\ *************************
\ *P The interrupt structure of the Cortex-Mx is such that the
\ ** simplest way to display exception information without large
\ ** code and RAM overheads is to use polled operation of the
\ ** comms link without interrupts. By default, the fault handler
\ ** only transmits, it does not use *\fo{KEY}. If your serial
\ ** driver normally uses interrupt-driven transmit routines, you
\ ** must provide the word *\fo{+FaultConsole} which switches it
\ ** into polled operation.

\ *P By default, there is no return from a fault handler, the
\ ** system is reset using *\fo{REBOOT}. Not all implementations
\ ** of *\fo{REBOOT} are equal. Rebooting by triggering a
\ ** watchdog always works.

: showFault	\ --
\ *G Generic fault handler.
  console dup ipvec !  opvec !  +FaultConsole	\ use fault mode console
  decimal
  10000 0 do loop			\ pause to avoid comms error ; SFP002
  5 0 do  5 emit  loop			\ trigger host error to stop INCLUDE server; SFP002
  end-load
  cr
  cr ." Exception " IPSR sys@ . ." at " AppPC .dword
  .FltFrame  .rsFault  .psFault
  cr ." Restarting system ..."  10000 0 do loop
[defined] reboot [if] reboot [else] ExcVecs 4 + @ execute [then]
  0
;

' showFault 2 FLT: NMIisr
' showFault 3 FLT: HFisr
Cortex-M0/M1? 0= [if]
' showFault 4 FLT: MFisr
' showFault 5 FLT: BFisr
' showFault 6 FLT: UFisr
' showFault #12 FLT: DFisr

: EnableFaults	\ --
\ Enable the fault handlers.
  $0007:0000 _SCS scsSHCSR + !  ;
' EnableFaults AtCold
[then]


\ *****************************
\ *S Intepreting the crash dump
\ *****************************
\ *P The example below comes from typing
\ *C  55 0 !
\ *P on the Forth console. The device is an NXP LPC2388 and address
\ ** zero is Flash to which writes have not been permitted. There
\ ** are only minor differences between the ARM example here and the
\ ** fault display for Cortex-M devices.

\ *E 55 0 !
\ ** DAbort exception at PC = 0000:1C64
\ ** === Registers ===
\ ** CPSR = 6000:001F
\ ** R0  = 0000:0037 0000:1C60 0000:0021 0000:0021
\ ** R4  = 0000:0070 0005:FD8C 4000:0298 0000:000C
\ ** R8  = 0000:1C60 0000:0000 0000:0000 4000:FEE0
\ ** R12 = 4000:FED4 4000:FDCC 0000:4FAC 0000:1C6C
\ **
\ ** RSP = 4000:FDCC   R0 = 4000:FDE0
\ ** --- Return stack high ---
\ ** 4000:FDDC 0000:51E0 QUIT
\ ** 4000:FDD8 4000:FED0
\ ** 4000:FDD4 0000:0000
\ ** 4000:FDD0 4000:FD94
\ ** 4000:FDCC 0000:3244 CATCH
\ ** --- Return stack low ---
\ **
\ ** PSP = 4000:FED4   S0 = 4000:FED4
\ ** --- Data stack high ---
\ ** --- Data stack low ---
\ ** rTOS/R10  0000:0000
\ ** Restarting system ...

\ *P The exception occurred in the instruction at $1C64. It was
\ ** a data abort exception, which means that it was a data
\ ** load or store at an invalid address.

\ *P The return stack dump shows that *\fo{CATCH} was used, in turn
\ ** called by *\fo{QUIT}, the text interpreter.

\ *P Further interpretation requires some knowledge of the use of
\ ** the CPU registers.

\ *****************
\ *N Register usage
\ *****************
\ ---------
\ *H Cortex
\ ---------
\ *P For Cortex-M the following register usage is the default:
\ *E   r15         pc      program counter
\ **   r14         link    link register; bit0=1=Thumb, usually set
\ **   r13         rsp     return stack pointer
\ **   r12         psp     data stack pointer
\ **   r11         up      user area pointer
\ **   r10         --
\ **   r9          lp      locals pointer
\ **   r8          --
\ **   r7          tos     cached top of stack
\ **   r0-r6       scratch
\ *P The VFX optimiser reserves R0 and R1 for internal operations.
\ ** *\fo{CODE} definitions must use R10 as TOS with NOS pointed to by
\ ** R12 as a full descending stack in ARM terminology. R0..R8 are
\ ** free for use by *\fo{CODE} definitions and need not be preserved or
\ ** restored. You should assume that any register can be affected
\ ** by other words.
\ ---------
\ *H ARM32
\ ---------
\ *P On the ARM the following register usage is the default:
\ *E   r15         pc      program counter
\ **   r14         link    link register
\ **   r13         rsp     return stack pointer
\ **   r12         psp     data stack pointer
\ **   r11         up      user area pointer
\ **   r10         tos     cached top of stack
\ **   r9          lp      locals pointer
\ **   r0-r8       scratch
\ *P The VFX optimiser reserves R0 and R1 for internal operations.
\ ** *\fo{CODE} definitions must use R10 as TOS with NOS pointed to by
\ ** R12 as a full descending stack in ARM terminology. R0..R8 are
\ ** free for use by *\fo{CODE} definitions and need not be preserved or
\ ** restored. You should assume that any register can be affected
\ ** by other words.

\ =============================
\ *N Interpreting the registers
\ =============================
\ *P Using the ARM example above, we can learn more.
\ *E DAbort exception at PC = 0000:1C64
\ ** === Registers ===
\ ** CPSR = 6000:001F
\ ** R0  = 0000:0037 0000:1C60 0000:0021 0000:0021
\ ** R4  = 0000:0070 0005:FD8C 4000:0298 0000:000C
\ ** R8  = 0000:1C60 0000:0000 0000:0000 4000:FEE0
\ ** R12 = 4000:FED4 4000:FDCC 0000:4FAC 0000:1C6C

\ *P The exception occurred in the instruction at $1C64. It was
\ ** a data abort exception, which means that it was a data
\ ** load or store at an invalid address.

\ *E TOS = R10 = 0000:0000
\ ** UP  = R11 = 4000:FEE0
\ ** PSP = R12 = 4000:FED4
\ ** RSP = R13 = 4000:FDCC

\ *P In general, UP > PSP > RSP. In this case that's good. TOS=0,
\ ** which we would expect from the phrase:
\ *C   55 0 !

\ *P We now switch back to the cross compiler, which you did leave
\ ** running, didn't you? Since we now know that $1C64 is in *\fo{!},
\ ** we can disassemble it.
\ *E dis !
\ ** !
\ ** ( 0000:1C60 0100BCE8 ..<h )  ldmia r12 ! { r0 }
\ ** ( 0000:1C64 00008AE5 ...e )  str r0, [ r10, # $00 ]
\ ** ( 0000:1C68 0004BCE8 ..<h )  ldmia r12 ! { r10 }
\ ** ( 0000:1C6C 0EF0A0E1 .p a )  mov PC, LR
\ ** 16 bytes, 4 instructions.
\ **  ok

\ *P From this, we can see that the offending instruction is
\ *C ( 0000:1C64 00008AE5 ...e )  str r0, [ r10, # $00 ]
\ *P Since R10 is 0, we now know that it was attempting a
\ ** write to 0, which is not permitted.

\ *P Provided that you keep words small, the register contents
\ ** at the crash point, with the stack contents and the disassembly
\ ** often provide enough information to reconstruct the state of
\ ** stack on entry to the word.


\ ======
\ *> ###
\ ======


decimal
