\ LPC1114FN28.ctl - standalone Forth for LPC1114 in DIP28

((
Copyright (c) 2011, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20151127 MPE003 Converted to use IRC oscillator with standalone DIP28 chip.
20150316 MPE002 Updated for Olimex LPC-P1114 board.
20110628 MPE001 First release.
))


\ ==============
\ *! lpc1114fn28
\ ==============

only forth definitions  decimal


\ **************************
\ *S Define directory macros
\ **************************
\ *P The MPE cross compilers contain a useful text macro system.
\ ** Macros allow easy porting of control files when projects are
\ ** moved from their default locations. Each directory macro
\ ** defines where a particular set of files are located.

\ *[
c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ..\.."		setmacro CpuDir		\ CPU specific files
c" ..\..\Examples"	setmacro ExampleDir	\ example files
\ *]


\ **************************
\ Turn on the cross compiler
\ **************************

\ *[

include %CpuDir%/Macros

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  -LongCalls			\ no calls outside 25 bit range
  +FlashCompile			\ target compiles to Flash
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true

\ *]

\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N LPC1114 definitions
\ =====================

#1114 equ LPCtype	\ -- 1100
\ *G Define the LPC type, using 1100 as the generic type
#32 kb equ /LPCflash	\ -- u
\ *G Define the size of the internal Flash. This is used
\ ** to control the use of  the Flash sector table.

#48 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

0 Mhz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ. Set to zero if
\ ** there is no crystal.
12 Mhz equ irc-speed	\ -- hz
\ *G IRC oscillator clock rate in HZ. See data sheet for speed.
#48 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $1000:0000..$1000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0000:0000.

  $0000:0000 $0000:7FFF cdata section liteLPC1114FN28	\ code section in boot Flash
  $1000:0000 $1000:087F udata section PROGu	\ 2k+ UDATA
  $1000:0880 $1000:0FDF idata section PROGd	\ 2k - 32 IDATA
    						\   32 bytes at top for IAP

interpreter
: prog liteLPC1114FN28  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

$0000:7000 equ INFOSTART   	\ kernel status is saved here.
$0000:7800 equ APPSTART		\ application data is saved here.
$0000:7FFF equ INFOEND		\ end of kernel/application status data.
$1000:0000 equ RAMSTART		\ start of RAM
$1000:1000 equ RAMEND		\ end of RAM
$0000:0000 equ FLASHSTART  	\ start of Main Flash
$0010:8000 equ FLASHEND		\ end of possible main Flash
$0000:4000 equ APPFLASHSTART  	\ start of application flash
$0000:7000 equ APPFLASHEND  	\ end of application flash
APPFLASHEND APPFLASHSTART - equ /APPFLASH	\ size of application flash

APPFLASHSTART TargetFlashStart	\ sets HERE at kernel start up

\ ============================
\ *N Stack and user area sizes
\ ============================

$0F0 equ UP-SIZE		\ size of each task's user area
$0A0 equ SP-SIZE		\ size of each task's data stack
$0F0 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Port 0 is the on-chip UART. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
\ *[
 1 equ Tiny?			\ nz to make a minimal kernel.
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\   true to include semaphores
   0 equ test-multi?		\   true to include test code
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
\ *]


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM0Def		\ Cortex-M0 generic equates and SFRs
  include %CpuDir%/sfrLPC11xx		\ LPC11xx special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startLPC11xx_IRC	\ startup code
l: crcslot
  0 ,					\ the kernel CRC
l: crcstart
  include %CpuDir%/CodeM0lite		\ low level kernel definitions
  include %CpuDir%/Drivers/FlashLPC11xx	\ Flash programming code
  include %CpuDir%/kernel72lite	\ high level kernel definitions
  include %CpuDir%/Drivers/rebootLPC11xx \ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
\  include %CpuDir%/FaultCortex		\ fault handlers - needs serial
  include %CpuDir%/Drivers/serLPC11xxp \ polled RX, polled TX
  include %CpuDir%/Dump			\ DUMP .S etc development tools

  include %CpuDir%/Drivers/gpioLPC11xx	\ easy pin access
  include %CpuDir%/Drivers/SysTickLPC11xx \ ticker
  ' start-clock AtCold

tasking? [if]
  include %CpuDir%/MultiCM0lite		\ multitasker
[then]

\ *]


\ ****************
\ *S End of kernel
\ ****************
\ *P After the main kernel has been built, some version data is
\ ** laid down for use by the sign-on code.

buildfile liteLPC1114FN28.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ******************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE Forth Lite for LPC1114FN28"
  cr version$ $. space BuildDate$ $.
  cr ." Copyright (C) 2014 MicroProcessor Engineering Ltd."
  .banner
;
external
\ *]


\ *******************
\ *S Application code
\ *******************
\ *P This code is not essential, but makes life very much easier.

\ *[
  include %CpuDir%/include		\ include from AIDE

RAMEND constant RP-END			\ end of available RAM
RAMEND equ RP-END			\ end of available RAM

\ *]



\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CpuDir%/LIBRARY
end-libs

\ Force binary file to 256 byte unit.
cdata
  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
  here $0FF +  $-0100 and org		\ force to 256 byte boundary for ISP loader

\ checksum the interrupt vectors
ExcVecs 0				\ -- addr csum
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $14 + @ +
  over $18 + @ +
  negate
swap $1C + !

decimal

\ Add a kernel checksum
crcstart here crcslot crc32 checksum
/DefStart 128 > [if]
  .( DEFSTART area too big ) abort
[then]

update-build				\ update build number file

FINIS					\ all done

\ *]


\ ======
\ *> ###
\ ======

