\ liteSTP812sa.ctl - standalone Forth for LPC812 on Space Time Productions

((
Copyright (c) 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140709 MPE002 Converted for STP LPC812 board.
20110628 MPE001 First release.
))


\ ==========
\ *! lpc812
\ ==========

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************
\ Make a copy of the MPE macros and adjust them for your directory
\ layout. If you have several developers with different layouts,
\ build a constant and a macro set for each developer. Only one
\ constant should be set non-zero.

1 constant MPE?	\ true if built at MPE

MPE? [if]	\ if built on an MPE development box

c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ..\.."		setmacro CpuDir		\ CPU specific files

[then]


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb-2 processor type and register usage
  -LongCalls			\ forbid standalone Forth to handle far calls
  +FlashCompile			\ target compiles to Flash
  setBinExt .bin		\ binary files have .bin extension
  +SaveCdataOnly		\ no data area image files
  Hex-I32			\ also produce Intel Hex-32 obj format

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ =====================
\ *N LPC812 definitions
\ =====================

#812 equ LPCtype	\ -- 812
\ *G Define the LPC type, using 800 as the generic type
#16 kb equ /LPCflash	\ -- u
\ *G Define the size of the internal Flash. This is used
\ ** to control the use of  the Flash sector table.

#48 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

#12 Mhz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
#24 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $1000:0000..$1000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0000:0000.

  $0000:0000 equ link-address	\ Address at which code will start\be linked at
  $0000:0000 $0000:3EFF cdata section liteSTP812sa	\ code section in Flash
  $1000:0000 $1000:07FF udata section PROGu	\ UDATA
  $1000:0800 $1000:0FDF idata section PROGd	\ IDATA
						\   32 bytes at top for IAP

interpreter
: prog liteSTP812sa  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

$0000:3F00 equ INFOSTART   	\ kernel info ; 4 pages of 64 bytes
$0000:3F80 equ APPSTART		\ app info
$0000:3FFF equ INFOEND
$1000:0000 equ RAMSTART
$1000:1000 equ RAMEND
$0000:0000 equ FLASHSTART  	\ start of Main Flash
$0000:4000 equ FLASHEND		\ end of possible main Flash
$0000:3800 equ APPFLASHSTART  	\ start of application flash
$0000:3F80 equ APPFLASHEND  	\ end of application flash
APPFLASHEND APPFLASHSTART - equ /APPFLASH	\ size of application flash

APPFLASHSTART TargetFlashStart	\ sets HERE at kernel start up

RAMEND #32 - equ RP-END		\ end of available RAM


\ ============================
\ *N Stack and user area sizes
\ ============================

$0F0 equ UP-SIZE		\ size of each task's user area
$0F0 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ useUART1?	\ -- n
\ *G Set non-zero to compile code for UART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUART2?	\ -- n
\ *G Set non-zero to compile code for UART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 0..2 are the on-chip UARTs.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ Tiny?			\ nz for minimal kernel
 0 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked



\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM0Def		\ Cortex-M0 generic equates and SFRs
  include %CpuDir%/sfrLPC81x		\ LPC81x special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startLPC81x		\ startup code
l: crcslot
  0 ,
l: crcstart
  include %CpuDir%/CodeM0lite		\ low level kernel definitions
  include %CpuDir%/Drivers/FlashLPC81x	\ Flash programming code
  include %CpuDir%/kernel72lite		\ high level kernel definitions
  include %CpuDir%/Drivers/rebootCM0	\ reboot using NVIC
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
\  include %CpuDir%/FaultCortex		\ fault handlers - needs serial
  include %CpuDir%/Drivers/serLPC81xp	\ polled serial driver
  include %CpuDir%/Dump			\ DUMP .S etc development tools

  include %CpuDir%/Drivers/SysTickLPC812
  ' start-clock to-do start-action

tasking? [if]
  include %CpuDir%/MultiCM0lite		\ multitasker
[then]



\ ****************
\ *S End of kernel
\ ****************

buildfile liteSTP812sa.no
l: BuildDate$
  Date$,

internal
: .CPU		\ -- ; display CPU type
  cr ." MPE Forth Lite 2 for LPC812 " BuildDate$ $.
  cr ." Copyright (C) 2014 MicroProcessor Engineering Ltd."
;
external


\ *******************
\ *S Application code
\ *******************

  include %CpuDir%/include		\ include from AIDE


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CpuDir%/LIBRARY
end-libs

  flush-idata                           \ force IDATA sections to be laid
  					\ NOW, rather than by FINIS
\  here $3F +  $-040 and org		\ force to 64 byte boundary for ISP loader

\ checksum the interrupt vectors
ExcVecs 0				\ -- addr csum
  over $00 + @ +
  over $04 + @ +
  over $08 + @ +
  over $0C + @ +
  over $10 + @ +
  over $14 + @ +
  over $18 + @ +
  negate
swap $1C + !

decimal

\ Add a kernel checksum
crcstart here crcslot crc32 checksum
/DefStart 124 > [if]
  .( DEFSTART area too big ) abort
[then]

update-build
FINIS


\ ======
\ *> ###
\ ======

