\ StartLPC81x.fth - Cortex startup code for LPC81x family

((
Copyright (c) 2009, 2010, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140707 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startlpc81x
\ *T Cortex-M0 start up for LPC81x
\ ===========
\ *P The file *\i{Cortex/Hardware/LPC81x/startLP81x.fth} contains
\ ** the start up code for LPC81x devices. Start up code for other
\ ** Cortex-M0 and M1 implementations can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M0
\ ** is in *\i{Cortex/StartCortex.fth}.

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file.
  /ExcVecs allot&erase

L: CLD1		\ holds xt of main word
\ *G Holds the xt of the main action after initial booting.
\ ** For a standalone Forth, this is usually the xt of *\fo{COLD}.
  0 ,					\ fixed by MAKE-TURNKEY

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1. *\b{Do not} use this to set the ISP checksum!
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. This
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required.
\ CTS=P0.14 i/p, RTS=P0.9 o/p, RXD=P0.0 i/p, TXD=P0.4 o/p ; CTS-lpc=RTS-ftdi RTS-lpc=CTS-ftdi
  $0000:0011 _SM smPINENABLE0 + !	\ disable ACMP1 in, XTALIN.

  $0000:ED70 _SYSCON PDRUNCFG + !	\ ACMP off, PLL on, XTAL off
  1 _SYSCON SYSAHBCLKDIV + !		\ main clock divided by 1
  $0001:C0FF _SYSCON SYSAHBCLKCTRL + !	\ Clock UARTs, SWM, GPIO, I2C, Flash, ROM, RAM, SYS

  0 _SYSCON SYSPLLCLKSEL + !		\ 0=irc, 1=sysclk, 2=wdt, 3=mainclock
  0 _SYSCON SYSPLLCLKUEN + !		\ Tickle Update Register
  1 _SYSCON SYSPLLCLKUEN + !
  $03 _SYSCON SYSPLLCTRL + !		\ IRC mul by 4 for 48 MHz, P=1. Divide by 2 for 24 MHz
  begin					\ wait for lock
    _SYSCON SYSPLLSTAT + @ 1 and
  until

  $03 _SYSCON MAINCLKSEL + !		\ 0=irc, 1=PLLin, 2=wdt, 3=PLLout
  0 _SYSCON MAINCLKUEN + !		\ Tickle Update Register
  1 _SYSCON MAINCLKUEN + !
;

((
: waitabit	\ --
  100000 0 do loop
;

: test		\ --
  init-ser0
  waitabit sercr0
  25 0 do
    waitabit  [char] A seremit0
    waitabit  [char] B seremit0
  loop
  waitabit sercr0
  console dup opvec ! ipvec !
  begin
    25 0 do
      waitabit  [char] X emit
      waitabit  [char] Y emit
    loop
    cr ." hello" cr
  again
;
))

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    setClocks
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
DefExc11      11        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
\ L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-RP 0 ,                          \ rp
\ L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

