\ liteSTM32F030uf.ctl - Umbilical Forth STM32F030RB discovery board

((
Copyright (c) 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140409 MPE001 First release for STM32F0xx.
))


\ ================
\ *! stm32f072liteuf
\ *T Umbilical Forth for STM32F030 Discovery board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32F0/liteSTM32F030uf.ctl}
\ ** produces an Umbilical Forth for the STM32F030 Discovery board.
\ ** The serial port is connected to USART1 on pins PA9 (Tx) and
\ ** PA10 (Rx).
\ ** You cannot use the easy GPIO code for UART initialisation unless
\ ** the GPIO ports are clocked and taken out of reset in the start
\ ** up code.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

\ *P You can easily use the low-cost ST-LINK/V2 JTAG unit.
\ ** The ST-LINK Utility software is a
\ ** free download from the ST website *\f{www.st.com}.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ."			setmacro CpuDir		\ CPU specific files
c" ..\Examples"		setmacro ExampleDir	\ example files


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

Only Forth Definitions          \ default search order

  no-heads                      \ No heads on target
  No-log			\ No output log
  Rommed                        \ Split ROM/RAM system
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  -LongCalls			\ no calls outside 25 bit range
   Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N STM32F0 variant definitions
\ ======================

$0800:0000 equ FlashBase	\ -- addr
\ *G Start address of Flash. The bottom 2kb (the vector area) is
\ ** mirrored at $0000:0000 for booting.

#64 kb equ /Flash	\ -- len
\ *G Size of Flash.
1 kb equ /FlashPage	\ -- len
\ *G Size of a Flash Page.
1 equ KeepPages		\ -- u
\ *G Set this non-zero for the number of pages at the end of Flash
\ ** that are reserved for configuration data. Often set to 1 or 2
\ ** by systems that use PowerNet.
FlashBase /Flash + /FlashPage KeepPages * - equ CfgFlash  \ -- len
\ *G Base address of the configuration Flash area.

$1FFF:EC00 equ /InfoBase	\ -- addr
\ *G Base address of system memory information block.

#3 kb equ /SysMem		\ -- len
\ *G Size of system memory block.

$1FFF:F800 equ OptionBytes	\ -- addr
\ *G Base address of option bytes
#8 equ /OptionBytes		\ -- len
\ *G Number of option bytes

#64 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 reserved by ARM.

\ *P The system clocks are generated from PLLs.
\ ** How to set them up is non-obvious and is mostly documented
\ ** by the ST demonstration code in the file *\i{system_stm32f0x2.c}.
\ ** If you are not going to use an existing setup, copy and rename
\ ** one of the existing *\i{startSTM32F0xx.fth} files.

8 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ. This is the HSI
\ ** internal oscillator which has better than +/-1% accuracy and
\ ** is more accurate than the internal 48 MHz oscillator.
48 MHz equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 48 MHz.
1 equ APBdiv	\ -- u
\ *G Division ratio of the APB clock from the system clock.
\ ** This may not be more than 48 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APBdiv / equ APB-speed	\ -- hz
\ *G APB bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2 kb (the
\ ** vector area) is mirrored at $0000:0000. The top 4 kb (two
\ ** pages) is used to save autostart and application linkage
\ ** information.

  $0800:0000 $0800:7FFF cdata section liteSTM32F030sa	\ code section in boot Flash
  $2000:0000 $2000:0FFF idata section PROGd	\ 4k IDATA
  $2000:1000 $2000:1FFF udata section PROGu	\ 4k UDATA

interpreter
: prog liteSTM32F030sa  ;		\ synonym
target

PROG PROGd PROGu  CDATA		\ use Code for HERE , and so on


\ ============================
\ *N Stack and user area sizes
\ ============================
\ *P These equates calculate how much RAM is needed for the USER area,
\ ** Return stack and data stack of each task, plus an area used
\ ** by Forth interrupts that share a common USER (and optionally)
\ ** common stacks. This can be done to reduce the overall amount
\ ** of RAM needed for interrupt stacks. Since interrupts can occur
\ ** in any task, switching stacks for high level interrupts will
\ ** reduce the total RAM requirements.

$040 equ UP-SIZE		\ size of each task's user area
$080 equ SP-SIZE		\ size of each task's data stack
$080 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount


SP-SIZE 2 rshift 2 - equ max-depth	\ max depth for UF data stack

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused


\ ==========================
\ *N Serial and ticker rates
\ ==========================

0 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

1 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#115200 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUSART4?	\ -- n
\ *G Set non-zero to compile code for USART4,
\ ** device *\fo{Console4}.
115200 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

2 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..4 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

 1 equ umbilical?               \ true = Umbilical Forth, zero = standalone
   0 umbilical? and equ SPI?	\ true for SPI Umbilical, false for serial
   SPI? 0= equ Serial?		\ true for Serial Umbilical, false for SPI
 1 equ init-idata?		\ true if IDATA to be initialised

\ Kernel components
 0 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
   0 equ event-handler?		\   true to include event handler
   0 equ message-handler?	\   true to include message handler
   1 equ semaphores?		\ true to include semaphores
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   0 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
 0 equ MPE-SET?			\ compatibility with MPE v5 targets
 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

interpreter
: AtCold	\ xt --
\ *G Most Umbilical Forth systems do not use the *\fo{ColdChain}
\ ** mechanism provided in *\i{Kernel62.fth} for standalone
\ ** Forths. To prevent compilation problems, define *\fo{AtCold}
\ ** as *\fo{DROP}. Compilers built from December 2008 onwards
\ ** will warn you if *\fo{AtCold} is used before being defined
\ ** in the target code.
  drop
;
target


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM0def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F051		\ STM32F030 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks
PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F030	\ start up code
  include %CpuDir%/minM0M1		\ code definitions
: selio-ser1	\ --
\ *G Example to perform clock and pin selection for USART1 on PA9/10.
\ ** You cannot use the easy GPIO code for this unless the GPIO ports
\ ** are clocked and taken out of reset in the start up code.
\ Clocks and reset
  _RCC
  $0000 $0003 2 pick rccCFGR3 + setMask	\ PCLK is UART clock
  bit17 over rccAHBen + or!		\ enable clock to GPIOA
  bit17 over rccAHBrst + bic!		\ take it out of reset
  bit14 over rccAPB2en + or!		\ enable clock to USART1
  bit14 over rccAPB2rst + bic!		\ take it out of reset
  drop
\ AF1 on PA9/10
  _GPIOA >r
  $0028:0000 $003C:0000 r@ gpioMODER + setMask	\ alternate function on 9/10
    
  $0000:0110 $0000:0FF0 r> gpioAFRH + setMask	\ AF1 for both (9/10)
   
;

: selio-ser2	\ --
\ *G Example to perform clock and pin selection for USART2 on PA2/3.
\ ** You cannot use the easy GPIO code for this unless the GPIO ports
\ ** are clocked and taken out of reset in the start up code.
\ Clocks and reset
  _RCC
  $0000 $0003 2 pick rccCFGR3 + setMask	\ PCLK is UART clock
  bit17 over rccAHBen + or!		\ enable clock to GPIOA
  bit17 over rccAHBrst + bic!		\ take it out of reset
  bit17 over rccAPB1en + or!		\ enable clock to USART2
  bit17 over rccAPB1rst + bic!		\ take it out of reset
  drop
\ AF1 on PA2/3
  _GPIOA >r
  \ value    mask       _GPIO  MODER    
  $0000:00A0 $0000:00F0 r@ gpioMODER + setMask      \ alternate function on 2/3  
  $0000:1100 $0000:FF00 r> gpioAFRL + setMask   \ AF1 for 2/3
 
;
include %CpuDir%/Drivers/serSTM32F030R8p \ polled serial driver
Include %CpuDir%/UFKernel		\ Umbilical Forth high level kernel
umbilical? [if]
  Synonym wait-byte  serkey2		\ Say which XTL drivers to use
  Synonym send-byte  seremit2
  Synonym Wait-Byte? serkey?2
  Synonym Init-XTL   Init-Ser
  include %CpuDir%/MESSAGES		\ Umbilical message equates
  include %CpuDir%/Targend		\ message driver
[then]


\ *************
\ *S End of kernel
\ *************

: .CPU          \ -- ; sign on
  ." MPE STM32F030 "
  [ Umbilical? ] [if] ." Umbilical " [else] ." Production " [then]
  ." Forth"
;


\ **************************
\ Load application code here
\ **************************

\ include %AppDir%\MyApp		\ you can use a macro for APPDIR


\ *******************
\ Load libraries here
\ *******************

libraries
  include %CpuDir%/libM0M1
  include %CPuDir%/UFlib
end-libs


\ **************************************
\ Define what happens at target power up
\ **************************************

((
: test-UART	\ -- char
  ipvec @ >r  opvec @ >r		\ currently points to VUART
  init-ser				\ initialise UART
  console0 dup ipvec !  opvec !		\ switch o/p to UART
  ." Is the UART working? " key
  r> opvec !  r> ipvec !
;
))

Umbilical? [if]
  make-turnkey run-umbilical            \ in TARGEND.FTH

\ Segger J-Link - Windows, OS X, Linux
\  jlink STM32F030RB 1000 Project.jlink	\ device, speed, project file
\  useSWD				\ use SWD rather than the default JTAG
\  PSUART0 set-sjl-link			\ select Umbilical link through J-Link
\ Windows - USB serial port
  c" \\.\COM48" 57600 serial	\ define link driver
  c" dtr=off rts=off" ser-control	\ define DTR and RTS
\ OS X - Keyspan HS19 USB adapter
\  c" /dev/cu.KeySerial1" console0-speed serial	\ define link driver
\  c" 1 dtr  1 rts" ser-control		\ define DTR and RTS

  cr ." You only need to download the LITESTM32F030UF section."
  cr ." Program the Flash using the ST-Link utility"
  cr ." Then answer N to the erase the Flash question."
  cr
  cr ." When you get the T:OK prompt, type .CPU to run"
  cr ." the word .CPU from this file."
  cr
\  cr ." Press any key to continue"  key drop
\  cr

  umbilical-forth                       \ start interactive session
[else]
: test-start    \ --
  init-ser				\ initialise the serial port for our use
  console ipvec !  console opvec !	\ I/O goes to VUART
  c" This is the STM32F030 Chip speaking!"
  count 0
  do  dup c@ emit 1+  loop  \  drop
  begin
    key emit
  again
;

  Make-turnkey test-start		\ application start up word
[then]

finis

