\ liteSTM32F030Nucleosa.ctl - STM32F030 Nucleo board standalone "Lite" Forth

((
Copyright (c) 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20151209 MPE003 COnverted to STM32F031K6 Nucleo board
20140524 RDO002 RDoering: Adapted to STM32F030Nucleo with STM32F030R8 Core
20140205 MPE001 First release for STM32F0xx.
))


\ ================
\ *! litestm32f031sa
\ *T Control file for STM32F031K6 Nucleo board
\ ================
\ *P The control file *\i{CortexLite/Hardware/Nucleo/liteSTM32F031NucleoSA.ctl}
\ ** produces a standalone Forth for the STM32F031K6 Nucleo board.
\ ** The serial port is connected to USART1 on pins PA2 (Tx) and
\ ** PA15 (Rx). These connect to the on-board ST-Link V2-1 and are
\ ** accessed as a USB serial port.

\ ** You cannot use the easy GPIO code for UART initialisation unless
\ ** the GPIO ports are clocked and taken out of reset in the start
\ ** up code.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

\ ** The ST-LINK Utility software is a
\ ** free download from the ST website *\f{www.st.com}.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ..\.."			setmacro CpuDir		\ CPU specific files
c" ..\Common"		setmacro CommonDir	\ common high level files
c" ..\Examples"		setmacro ExampleDir	\ example files


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  -LongCalls			\ no calls outside 25 bit range
  +FlashCompile			\ target compiles to Flash
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N STM32F0 variant definitions
\ ======================

$0800:0000 equ FlashBase	\ -- addr
\ *G Start address of Flash. The bottom 2kb (the vector area) is
\ ** mirrored at $0000:0000 for booting.

#32 kb equ /Flash	\ -- len
\ *G Size of Flash.
1 kb equ /FlashPage	\ -- len
\ *G Size of a Flash Page.
1 equ KeepPages		\ -- u
\ *G Set this non-zero for the number of pages at the end of Flash
\ ** that are reserved for configuration data. Often set to 1 or 2
\ ** by systems that use PowerNet.
FlashBase /Flash + /FlashPage KeepPages * - equ CfgFlash  \ -- len
\ *G Base address of the configuration Flash area.

$1FFF:EC00 equ /InfoBase	\ -- addr
\ *G Base address of system memory information block.

#3 kb equ /SysMem		\ -- len
\ *G Size of system memory block.

$1FFF:F800 equ OptionBytes	\ -- addr
\ *G Base address of option bytes
#8 equ /OptionBytes		\ -- len
\ *G Number of option bytes

#64 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 slots reserved by ARM.

\ *P The system clocks are generated from PLLs.
\ ** How to set them up is non-obvious and is mostly documented
\ ** by the ST demonstration code in the file *\i{system_stm32f0x2.c}.
\ ** If you are not going to use an existing setup, copy and rename
\ ** one of the existing *\i{startSTM32F0xx.fth} files.

8 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ. This is the HSI
\ ** internal oscillator which has better than +/-1% accuracy and
\ ** is more accurate than the internal 48 MHz oscillator.
48 MHz equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 48 MHz.
1 equ APBdiv	\ -- u
\ *G Division ratio of the APB clock from the system clock.
\ ** This may not be more than 48 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APBdiv / equ APB-speed	\ -- hz
\ *G APB bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2 kb (the
\ ** vector area) is mirrored at $0000:0000. The top 4 kb (two
\ ** pages) is used to save autostart and application linkage
\ ** information.

  $0800:0000 $0800:3FFF cdata section liteSTM32F031sa	\ code section in boot Flash
  $2000:0000 $2000:07FF idata section PROGd	\ 2k IDATA
  $2000:0800 $2000:0FFF udata section PROGu	\ 2k UDATA

interpreter
: prog liteSTM32F031sa  ;		\ synonym
target

PROG PROGd PROGu  CDATA		\ use Code for HERE , and so on

$0800:7C00 equ INFOSTART   	\ 1 page of 1 kb
$0800:7E00 equ APPSTART
$0800:7FFF equ INFOEND
$2000:0000 equ RAMSTART
$2000:1000 equ RAMEND
$0800:0000 equ FLASHSTART  	\ start of Main Flash
$0800:8000 equ FLASHEND		\ end of possible main Flash
$0800:4000 equ APPFLASHSTART  	\ start of application flash
$0800:7C00 equ APPFLASHEND  	\ end of application flash
$0000:6000 equ /APPFLASH	\ size of application flash

APPFLASHSTART TargetFlashStart	\ sets HERE at kernel start up

\ ============================
\ *N Stack and user area sizes
\ ============================

$0F0 equ UP-SIZE		\ size of each task's user area
$0A0 equ SP-SIZE		\ size of each task's data stack
$0F0 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#115200 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUSART4?	\ -- n
\ *G Set non-zero to compile code for USART4,
\ ** device *\fo{Console4}.
115200 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

1 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..4 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ Tiny?			\ nz to make a minimal kernel.
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked
\ 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required

 0 equ PowerNet?	\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM0def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F030		\ STM32F031 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks
PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F031	\ start up code
l: crcslot
  0 ,
l: crcstart
  include %CpuDir%/CodeM0lite		\ low level kernel definitions
  include %CpuDir%/Drivers/FlashSTM32	\ Flash programming code
  include %CpuDir%/kernel72lite		\ high level kernel definitions
  include %CpuDir%/Drivers/rebootSTM32	\ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
\  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
: selio-ser1	\ --
\ *G Example to perform clock and pin selection for USART1 on PA2/15.
\ ** Provide your own version, e.g. in the control file.
\ Clocks and reset
  _RCC
  $0000 $0003 2 pick rccCFGR3 + setMask	\ PCLK is UART clock
\  bit17 over rccAHBen + or!		\ enable clock to GPIOA - done at power up
\  bit17 over rccAHBrst + bic!		\ take it out of reset
  bit14 over rccAPB2en + or!		\ enable clock to USART1
  bit14 over rccAPB2rst + bic!		\ take it out of reset
  drop
\ Default on PA2/15
  _GPIOA >r
  $8000:0020 $C000:0030 r@ gpioMODER + setMask	\ alternate function on 2/15
  $0000:0100 $0000:0F00 r@ gpioAFRL + setMask	\ AF1 for PA2
  $1000:0000 $F000:0000 r> gpioAFRH + setMask	\ AF1 for PA15
;
  include %CpuDir%/Drivers/serSTM32F0xxp \ polled serial driver
  include %CpuDir%/Dump			\ DUMP .S etc development tools

  include %CpuDir%/Drivers/gpioSTM32F0xx \ easy pin access
\  include %CpuDir%/Drivers/SysTickCM0	\ ticker
  include %CpuDir%/Drivers/SysTick031Nucleo	\ ticker
  ' start-clock AtCold

tasking? [if]
  include %CpuDir%/MultiCM0lite		\ multitasker
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile liteSTM32F031.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ****************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE PowerForth for STM32F031"
  cr version$ $. space BuildDate$ $.
  .banner
;
external


\ *******************
\ *S Application code
\ *******************

  \ include %HwDir%/ReProg/ReFlash	\ ReFlash utility
  include %CpuDir%/include		\ include from AIDE
  \ include %HwDir%/Drivers/spiSTM32F0hard	\ SPI2 driver
  \ include %HwDir%/Examples/l3gd20	\ L3GD20 MEMS driver

progu sec-end cdata constant RP-END	\ end of available RAM


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CpuDir%/LIBRARY
end-libs

decimal

\ Add a kernel checksum
crcstart here crcslot crc32 checksum
/DefStart 124 > [if]
  .( DEFSTART area too big ) abort
[then]

update-build
FINIS


\ ======
\ *> ###
\ ======
