\ startSTM32F031.fth - Cortex startup code for STM32F031K6 Nucleo board

((
Copyright (c) 2010, 2011, 2012, 2014, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20151209 MPE005 Conversion for STM32F031K6 Nucleo board.
20140524 RDO004 R. Doering did STM32F030R8 Nucleo board adaptation
20140205 MPE003 STM32F072 conversion
20120322 MPE002 STM32F107 conversion
20100112 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startstm32f051
\ *T Cortex start up for STM32F030
\ ===========

l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long. The
\ ** equate is defined in the control file. Note that this table
\ ** table must be correctly aligned as described in the Cortex-M3
\ ** Technical Reference Manual (ARM DDI0337, rev E page 8-21). If
\ ** placed at 0 or the start of Flash, you'll usually be fine.
  /ExcVecs allot&erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector number to the given address.
\ ** Note that for vectors other than 0, the Thumb bit is forced
\ ** to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  cells ExcVecs + !  ;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc4	b  $	end-code	\ MemManage
proc DefExc5	b  $	end-code	\ BusFault
proc DefExc6	b  $	end-code	\ UsageFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc12	b  $	end-code	\ DbgMon
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard
[then]

init-s0 tos-cached? sp-guard + cells -
  equ real-init-s0	\ -- addr
\ The data stack pointer set at start up.

interpreter
: prediv4	\ u -- x
\ *G Convert a divider value to a 4 bit pattern as used for HPRE.
  case
    1 of  %0000  endof
    2 of  %1000  endof
    4 of  %1001  endof
    8 of  %1010  endof
    16 of  %1011  endof
    64 of  %1100  endof
    128 of  %1101  endof
    256 of  %1110  endof
    512 of  %1111  endof
      true abort" Invalid AHB prescaler setting"
  endcase
;

: prediv3	\ u -- x
\ *G Convert a divider value to a 3 bit pattern as used for PPRE.
  case
    1 of  %000  endof
    2 of  %100  endof
    4 of  %101  endof
    8 of  %110  endof
    16 of  %111  endof
      true abort" Invalid PPREx prescaler setting"
  endcase
;

: pllmul1	\ u -- x
\ *G Convert a multiplier value to the 4 bit pattern as used for
\ ** PLLMUL.
  dup 2 16 within? 0=  abort" Invalid PLLMUL setting"
  2-
;
target

\ *P We set up for 48 MHz CPU speed from the HSI oscillator.
\ ** NOTE: F03x/05x has a fixed /2 divide for the HSI BEFORE it
\ ** is used as clock source for the PLL.
\ ** We take this into account when calculating PLLMUL
0					\ also the reset value
$04 24 lshift or			\ MCO from SystemClock
system-speed xtal-speed 2 / / pllmul1
 18 lshift or				\ PLLin * x ; PLLin = 8MHz
					\ bit17 set from bits3:0 in rccCFGR2
%01 15 lshift or			\ PLLSRC from HSI/PREDIV
APBdiv prediv3 8 lshift or		\ APB clock
AHBdiv prediv4 4 lshift or		\ AHB clock
  equ initCFGRval		\ -- x
\ *G Bit settings for the RCC_CFGR register.
0
0 or				\ prediv = 0 FOR NO DIVIDE
  equ initCFGR2val		\ -- x
\ *G Bit settings for the RCC_CFGR2 register.
[then]

\ *P The number of wait states required by the Flash depends on
\ ** the operating frequency.
system-speed 1- 24 MHz / equ FLWS	\ -- u
\ *G The number of Flash wait states required.

bit4 ( PRFTBE ) FLWS or  equ initACRval	\ -- x
\ *G Initial value written to the FLASH_ACR register

internal
: setClocks	\ --
\ *G Enable the clocks, set the bus dividers, and enable the PLLs
\ ** as required. the PLL IS used to generate the main 48 MHz
\ ** clock. By default, the AHB and APB busses are run at
\ ** SYSCLK
  _RCC

\ ensure internal clock is running
  bit0 over rccCR + or!			\ set HSION
  begin
    dup rccCR + @ bit1 and		\ wait for HSIRDY
  until
((
\ start external clock
  bit16 over rccCR + or!		\ set HSEON
  begin
    dup rccCR + @ bit17 and		\ wait for HSERDY
  until
))

\ Set up the Flash configuration
  initACRval _FPEC flACR + !		\ Enable caches and set wait states

\ set prescalers and PLL configurations
  initCFGRval over rccCFGR + !		\ set AHB, APB1, APB2 prescalers
  initCFGR2val over rccCFGR2 + !	\ PLL2/3 and dividers

\ start main PLL
  bit24 over rccCR + or!		\ set PLLON
  begin
    dup rccCR + @ bit25 and		\ wait for PLLRDY
  until

\ switch to the PLL output
  $0002 $0003 2 pick rccCFGR + setMask	\ set SW=2
  begin
    dup rccCFGR + @ $000C and $0008 =	\ wait for SWS=2
  until

  drop
;

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation can also be performed here.
\ ** All the GPIO blocks are enabled.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    $007E:0014 _RCC rccAHBen + or!	\ enable GPIOs, FLIT and SRAM
    $007E:0000 _RCC rccAHBrst + bic!	\ no reset to GPIOs
    setClocks
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec
DefExc3       3         SetExcVec
DefExc4       4         SetExcVec
DefExc5       5         SetExcVec
DefExc11      11        SetExcVec
DefExc12      12        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-RP  0 ,				\ RP
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ ======
\ *> ###
\ ======

decimal

