\ liteSTM32F051sa.ctl - STM32F051 discovery board standalone "Lite" Forth

((
Copyright (c) 2014, 2016, 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Our thanks to Dr. Marcello Presulli for the code changes.

To do
=====

Change history
==============
20161214 MAP003 STM32F051 conversion.
20140707 MPE002 Updated for Lite Forth v2.
20140205 MPE001 First release for STM32F0xx.
))


\ ================
\ *! stm32f051litesa
\ *T Control file for STM32F051 Discovery board
\ ================
\ *P Every Forth project has a control file, which is similar to the
\ ** project file in other languages. The control file tells the
\ ** cross compiler all about the target. Not all of it is
\ ** documented here, but it is all commented in the source code.

\ *P The control file *\i{CortexLite/Hardware/STM32F051Bdisco/liteSTM32F051sa.ctl}
\ ** produces a standalone Forth for the STM32F051B Discovery board.
\ ** The control files for other boards are very similar in structure,
\ ** and different in the details of the CPU and board specific
\ ** options. Read the specific code for your CPU and board.

\ *P The serial port is connected to USART1 on pins PA9 (Tx -> PC Rx) and
\ ** PA10 (Rx -> PC Tx).
\ ** You cannot use the easy GPIO code for UART initialisation unless
\ ** the GPIO ports are clocked and taken out of reset in the start
\ ** up code.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

\ *P To Flash the board, you can use the low-cost ST-LINK/V2 JTAG unit, or use
\ ** the one integrated into the ST Discovery boards.
\ ** The ST-LINK Utility software is a
\ ** free download from the ST website *\f{www.st.com}.

only forth definitions  decimal


\ **************************
\ *S Define directory macros
\ **************************
\ *P The MPE cross compilers contain a useful text macro system.
\ ** Macros allow easy porting of control files when projects are
\ ** moved from their default locations. Each directory macro
\ ** defines where a particular set of files are located.

\ *[
c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ..\.."		setmacro CpuDir		\ CPU specific files
c" ..\..\Examples"	setmacro ExampleDir	\ example files
\ *]


\ *****************************
\ *S Turn on the cross compiler
\ *****************************

\ *[

include %CpuDir%/Macros		\ macros needed by the cross-compiler

CROSS-COMPILE			\ Turn host Forth into a cross-compiler

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  -LongCalls			\ no calls outside 25 bit range
  +FlashCompile			\ target compiles to Flash
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true

\ *]

\ *******************
\ *S Configure target
\ *******************

\ ==============================
\ *N STM32F0 variant definitions
\ ==============================

$0800:0000 equ FlashBase	\ -- addr
\ *G Start address of Flash. The bottom 2kb (the vector area) is
\ ** mirrored at $0000:0000 for booting.

#64 kb equ /Flash	\ -- len
\ *G Size of Flash.
1 kb equ /FlashPage	\ -- len
\ *G Size of a Flash Page.
1 equ KeepPages		\ -- u
\ *G Set this non-zero for the number of pages at the end of Flash
\ ** that are reserved for configuration data. Often set to 1 or 2
\ ** by systems that use PowerNet.
FlashBase /Flash + /FlashPage KeepPages * - equ CfgFlash  \ -- len
\ *G Base address of the configuration Flash area.

$1FFF:EC00 equ /InfoBase	\ -- addr
\ *G Base address of system memory information block.

#3 kb equ /SysMem		\ -- len
\ *G Size of system memory block.

$1FFF:F800 equ OptionBytes	\ -- addr
\ *G Base address of option bytes
#16 equ /OptionBytes		\ -- len
\ *G Number of option bytes

#64 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 slots reserved by ARM.

\ *P The system clocks are generated from PLLs.
\ ** How to set them up is non-obvious and is mostly documented
\ ** by the ST demonstration code in the file *\i{system_stm32f0x2.c}.
\ ** If you are not going to use an existing setup, copy and rename
\ ** one of the existing *\i{startSTM32F0xx.fth} files.

8 MHz equ xtal-speed		\ -- hz
\ *G Master oscillator crystal clock rate in HZ. This is the HSI
\ ** internal oscillator which has better than +/-1% accuracy and
\ ** is more accurate than the internal 48 MHz oscillator.
32 MHz equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 48 MHz.
1 equ APBdiv	\ -- u
\ *G Division ratio of the APB clock from the system clock.
\ ** This may not be more than 48 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APBdiv / equ APB-speed	\ -- hz
\ *G APB bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000. The bottom 2 kb (the
\ ** vector area) is mirrored at $0000:0000. The top 4 kb (two
\ ** pages) is used to save autostart and application linkage
\ ** information.

\ *[
  $0800:0000 $0800:7FFF cdata section liteSTM32F051sa	\ code
  $2000:0000 $2000:0FFF udata section PROGu	\ 4k UDATA RAM
  $2000:1000 $2000:1FFF idata section PROGd	\ 4k IDATA RAM

interpreter
: prog liteSTM32F051sa  ;		\ synonym
target

PROG PROGd PROGu  CDATA		\ use Code for HERE , and so on

$0800:FC00 equ INFOSTART   	\ kernel status is saved here.
$0800:FE00 equ APPSTART		\ application data is saved here.
$0800:FFFF equ INFOEND		\ end of kernel/application status data.
$2000:0000 equ RAMSTART		\ start of RAM
$2000:2000 equ RAMEND		\ end of RAM
$0800:0000 equ FLASHSTART  	\ start of Main Flash
$0801:0000 equ FLASHEND		\ end of possible main Flash
$0800:8000 equ APPFLASHSTART	\ start of application flash
$0800:FC00 equ APPFLASHEND  	\ end of application flash
APPFLASHEND APPFLASHSTART - equ /APPFLASH	\ size of application flash

APPFLASHSTART TargetFlashStart	\ sets HERE at kernel start up

\ *]


\ ============================
\ *N Stack and user area sizes
\ ============================

\ *[
$0F0 equ UP-SIZE		\ size of each task's user area
$0F0 equ SP-SIZE		\ size of each task's data stack
$0F0 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused
\ *]


\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#115200 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

0 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUSART4?	\ -- n
\ *G Set non-zero to compile code for USART4,
\ ** device *\fo{Console4}.
115200 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

1 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..4 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================
\ *P With 128 kb of Flash we can select a comfortable set of
\ ** software and still have plenty of space for application
\ ** code.

\ *[
 0 equ Tiny?			\ nz to make a minimal kernel.
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
 6 cells equ tcb-size		\ for internal consistency check
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?			\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked

 0 equ ENVIRONMENT?		\ true if ANS ENVIRONMENT system required
 0 equ PowerNet?		\ -- flag
\ *G Set non-zero to compile the PowerNet TCP/IP stack.

\ *]


\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

\ *[
  include %CpuDir%/CM0def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F072		\ STM32F072 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks
PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F051	\ start up code
l: crcslot
  0 ,					\ the kernel CRC
l: crcstart
  include %CpuDir%/CodeM0lite		\ low level kernel definitions
  include %CpuDir%/Drivers/FlashSTM32	\ Flash programming code
  include %CpuDir%/kernel72lite		\ high level kernel definitions
  include %CpuDir%/Drivers/rebootSTM32	\ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
\ *]
: selio-ser1	\ --
\ *G Example to perform clock and pin selection for USART1 on PA9/10.
\ ** You cannot use the easy GPIO code for this unless the GPIO ports
\ ** are clocked and taken out of reset in the start up code.
\ Clocks and reset
  _RCC
  $0000 $0003 2 pick rccCFGR3 + setMask	\ PCLK is UART clock
  bit17 over rccAHBen + or!		\ enable clock to GPIOA
  bit17 over rccAHBrst + bic!		\ take it out of reset
  bit14 over rccAPB2en + or!		\ enable clock to USART1
  bit14 over rccAPB2rst + bic!		\ take it out of reset
  drop
\ Default on PA9/10
  _GPIOA >r
  $0028:0000 $003C:0000 r@ gpioMODER + setMask	\ alternate function on 9/10
  $0000:0110 $0000:0FF0 r> gpioAFRH + setMask	\ AF1 for both

\ * $4800:0800 >r	\ * $4800:0800 equ PORTC_Base
\ * %01 8 2* lshift  %01 9 2* lshift or r> !
\ * $4800:0818 >r	\ * $4800:0818 equ PORTC_BSRR
\ * 1 8 lshift 1 9 lshift or r> !
;
\ *[
  include %CpuDir%/Drivers/serSTM32F0xxp \ polled serial driver
  include %CpuDir%/Dump			\ DUMP .S etc development tools

  include %CpuDir%/Drivers/gpioSTM32F0xx \ easy pin access
  include %CpuDir%/Drivers/SysTickDisco072
  ' start-clock AtCold

tasking? [if]
  include %CpuDir%/MultiCM0lite		\ multitasker
[then]


\ *]


\ *************
\ *S End of kernel
\ *************
\ *P After the main kernel has been built, some version data is
\ ** laid down for use by the sign-on code.

\ *[

buildfile liteSTM32F051sa.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." ****************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE Forth Lite for STM32F051"
  cr version$ $. space BuildDate$ $.
  cr ." Copyright (C) 2017 MicroProcessor Engineering Ltd."
  .banner
;
external
\ *]


\ *******************
\ *S Application code
\ *******************
\ *P This code is not essential, but makes life very much easier.

\ *[
  include %HwDir%/ReProg/ReFlash	\ ReFlash utility
  include %CpuDir%/include		\ include from AIDE

RAMEND constant RP-END	\ end of available RAM
\ *]


\ ***************
\ *S Finishing up
\ ***************
\ *[

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CpuDir%/LIBRARY
end-libs

decimal

\ Add a kernel checksum
crcstart here crcslot crc32 checksum
/DefStart 128 > [if]
  .( DEFSTART area too big ) abort
[then]

update-build				\ update build number file

FINIS					\ all done

\ *]


\ ======
\ *> ###
\ ======
