\ Reprogram STM32 over serial line.

((
Copyright (c) 2003, 2004, 2005, 2006, 2008, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20100131 SFP005 Converted for STM32.
20100125 SFP004 Converted for LPC17xx
20081008 SFP003 Converted for LPC23xx/24xx/28xx
20060818 SFP002 Added UART selection.
20040809 SFP001 Added equate FULLIAP?
20030728 SFP000 First cut
))

\ =============
\ *! reprogstm32
\ *T Reprogramming the STM32 serially
\ =============
\ *S Introduction
\ *P *\i{ReprogSTM32.ctl} is the control file for the serial
\ ** loader which reprograms the on-chip Flash memory.
\ ** Because the loader code may itself be replaced, the Flash
\ ** programming code is copied into RAM for execution. The only
\ ** exit from the code is a reset of the CPU to execute the
\ ** newly Flashed program. All interrupts are disabled during
\ ** reprogramming, and so polled serial drivers are used.

\ *P Access to the reprogramming software is defined in
\ ** *\i{ReFlash.fth} for the main system code and in
\ ** this control file and the two must match.

\ *P Return is by rebooting the system. The contents of the
\ ** internal RAM and Flash are destroyed, therefore any data
\ ** that must be preserved should be saved externally to the
\ ** chip.


only forth definitions


\ ******************************
\ Define the default directories
\ ******************************

c" ../../.."	setmacro CpuDir
c" ../../../.."	setmacro CommonDir
c" ."		setmacro AppDir
c" .."		setmacro HwDir


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs                        \ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Cortex-M processor type
  no-heads			\ do not generate heads
  no inlining			\ saves some space
  -ram-table			\ saves some space
  +LeafCalls			\ saves some space
  +SaveCdataOnly		\ no data area image files

0         equ false
false not equ true

((
$2000:0000 $2000:0BFF cdata section ReprogSTM32
interpreter
: prog  ReprogSTM32  ;
target
$2000:0C00 $2000:0C7F idata section PROGd
$2000:0C80 $2000:0CFF udata section PROGu
$2000:0D00 $2000:0DFF udata section Xbuff	\ 256 byte Xmodem buffer
  sec-base equ Xbase			\ base address of buffer
  sec-len equ Xlen			\ length of buffer
  Xbase Xlen + equ Xtop			\ top of buffer
))
$2000:0000 $2000:09FF cdata section ReprogSTM32F0
interpreter
: prog  ReprogSTM32F0  ;
target
$2000:0A00 $2000:0A1F idata section PROGd
$2000:0A20 $2000:0A3F udata section PROGu
$2000:0A40 $2000:0ABF udata section Xbuff	\ 128 byte Xmodem buffer
  sec-base equ Xbase			\ base address of buffer
  sec-len equ Xlen			\ length of buffer
  Xbase Xlen + equ Xtop			\ top of buffer

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on


\ ********************
\ *S Items of interest
\ ********************
\ *P The stacks have already been set up by the calling program.

\ *[
include %CpuDir%\CM0def			\ Cortex CPU equates
include %CpuDir%\sfrSTM32F072		\ Special function registers
\ *]

l: CLD1			\ -- addr
\ *G Filled in later with the xt of *\fo{ReprogFlash}
  0 ,
l: CLD_CopyFlash	\ -- addr
\ *G Filled in later with the xt of *\fo{CopyFlash}
  0 ,
l: CLD_UART		\ -- addr
\ *G Filled in later with 0 or UART base address to use.
\ ** If set to 0, the default is UART1. This allows the calling
\ ** application to select which UART to use.
  0 ,
l: CLD_clockspeed	\ -- addr
\ *G Contains the clock speed to use. Filled in by the calling
\ ** application. Default is 48 MHz.
  48 Mhz ,
l: CLD_/Flash		\ -- addr
\ *G Flash size in bytes. Default is 128kb
  #128 kb ,
l: CLD_/FlashPage	\ -- addr
\ *G Size of each page in the flash. Default is 2 kb.
  2 kb ,

$0800:0000 equ FlashBase	\ -- addr
\ *G Base address of the Flash.

compiler
: prog-speed	\ -- hz
  CLD_clockspeed @  ;
: /Flash	\ -- len
  CLD_/Flash @  ;
: /FlashPage	\ -- len
  CLD_/FlashPage @  ;
target

\ *[
include %AppDir%\primitives		\ Forth primitives from CodeM0M1.fth
include %AppDir%\MinSerSTM32p		\ polled serial line driver
include %AppDir%\delays			\ software delays
include %CpuDir%\Drivers\rebootSTM32	\ reboot
\ *]

synonym ser-emit emit
\ *G A synonym required by the Xmodem code.
synonym ser-key? key?
\ *G A synonym required by the Xmodem code.
synonym ser-key  key
\ *G A synonym required by the Xmodem code.

\ *[
include %AppDir%\MinXmodemRx		\ Xmodem receiver
include %AppDir%\ReprogApp		\ Application
\ *]


\ *******
\ The end
\ *******

make-turnkey ReprogFlash		\ start up action
\ *G Define the action of the reprogramming code.
' CopyFlash CLD_CopyFlash !
\ *G Define the action of the Flash copy code.

decimal

FINIS


\ ******
\ *> ###
\ ******
