\ STM32F4disco.ctl - STM32F407 Discovery board standalone Forth

((
Copyright (c) 2011, 2012, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20140703 SFP004 Lite compiler conversion.
20140130 SFP003 Updated for Stamp compiler.
20120113 SFP002 Moved USART I/O selection to control file.
20091223 MPE001 First release.
))


\ ================
\ *! litestm32f4discosa
\ *T Control file for STM32F4 Discovery board
\ ================
\ *P The control file *\i{Cortex/Hardware/STM32F4/STM32F4disco.ctl}
\ ** produces a standalone Forth for the STM32F4 Discovery board.
\ ** The serial port is connected to USART3 on pins PD8 (USART3-Tx)
\ ** and PD9 (USART3-Rx). Note that many TTL->RS232 USB adapters
\ ** are labelled with respect to the PC, not the TTL directions.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

\ *P To Flash the board, you can use the low-cost ST-LINK/V2 JTAG unit, or use
\ ** the one integrated into the ST Discovery boards.
\ ** The ST-LINK Utility software is a
\ ** free download from the ST website *\f{www.st.com}.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ..\.."		setmacro CpuDir		\ CPU specific files
c" ..\..\Examples"	setmacro ExampleDir	\ example files


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  +LongCalls			\ calls outside 25 bit range
  +FlashCompile			\ target compiles to Flash
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ==============================
\ *N STM32F4 variant definitions
\ ==============================

0 equ PrefetchOK?	\ -- flag
\ *G The instruction prefetch queue does not work on Rev A devices.

$0800:0000 equ FlashBase	\ -- addr
\ *G Start address of Flash. The bottom 2kb (the vector area) is
\ ** mirrored at $0000:0000 for booting.

#512 kb equ /Flash	\ -- len
\ *G Size of Flash.

$1FFF:0000 equ /InfoBase	\ -- addr
\ *G Base address of system memory information block.

#30 kb equ /SysMem		\ -- len
\ *G Size of system memory block.

$1FFF:C000 equ OptionBytes	\ -- addr
\ *G Base address of option bytes
#16 equ /OptionBytes		\ -- len
\ *G Number of option bytes

#128 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 slots reserved by ARM.

\ *P The system and USB clocks are generated from a PLL. The PLL
\ ** input clock must be in the range 1-2 MHz. The upper
\ ** end is preferred to reduce PLL jitter. The PLL output
\ ** frequency must be in the range 64..432 MHz.
\ ** To use USB at 48 MHz, you need to choose a PLL frequency
\ ** that is an integer multiple of 48 MHz. The system speed
\ ** is also an integer division of the VCO speed.

8 MHz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ.
2 MHz equ vco-in-speed		\ -- hz
\ *G Requested VCO input frequency.
336 MHz equ vco-out-speed	\ -- hz
\ *G Requested VCO output frequency.
168 MHz equ system-speed		\ -- hz
\ *G Requested CPU clock speed in HZ. Note that you must
\ ** calculate the PLL values. The VCO output frequency
\ ** can only be divided by 2, 4, 6 or 8.

xtal-speed vco-in-speed / equ PLLM	\ -- u
\ *G Divide the *\fo{xtal-speed} by the VCO input speed to produce
\ ** the required divisor.
vco-out-speed vco-in-speed / equ PLLN	\ -- u
\ *G Multiply the VCO input frequency by this value to produce the VCO
\ ** output frequency.
vco-out-speed system-speed / equ PLLP	\ -- u
\ *G Divide the VCO output by this value to produce the system
\ ** clock. The VCO output frequency can only be divided by 2, 4,
\ ** 6 or 8.
vco-out-speed 48 MHz / equ PLLQ	\ -- u
\ *G At VcoClk=336 MHz, this gives a USB OTG FS clock of 48 MHz.
\ ** Suitable VCO speeds are 48, 96, 144, 192, 240, 288, 336,
\ ** 384, 432 MHz.

1 equ AHBdiv	\ -- u
\ *G Division ratio of the AHB clock from the system clock.
\ ** This may not be more than 168 MHz.
2 equ APB2div	\ -- u
\ *G Division ratio of the APB2 clock from the system clock.
\ ** This may not be more than 84 MHz.
4 equ APB1div	\ -- u
\ *G Division ratio of the APB1 clock from the system clock.
\ ** This may not be more than 42 MHz.

system-speed AHBdiv / equ AHB-speed	\ -- hz
\ *G AHB bus speed.
system-speed APB2div / equ APB2-speed	\ -- hz
\ *G APB2 bus speed.
system-speed APB1div / equ APB1-speed	\ -- hz
\ *G APB1 bus speed.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $0800:0000 and is mirrored at
\ ** $0000:0000. The sector from $0800:C000 to $0800:FFFF is
\ ** used to store autostart and application linkage* information.

  $0800:0000 $0800:BFFF cdata section liteSTM32F4disco	\ code section in boot Flash
  $2000:0000 $2000:2FFF udata section PROGu	\ 12k UDATA
  $2000:3000 $2000:3FFF idata section PROGd	\ 4k IDATA

interpreter
: prog liteSTM32F4disco  ;			\ synonym
target

PROG PROGd PROGu  CDATA                 \ use Code for HERE , and so on

$0800:C000 equ INFOSTART   	\ start of system data
$0800:C800 equ APPSTART		\ start of application data
$0800:FFFF equ INFOEND		\ end of sys/app data sector
$2000:0000 equ RAMSTART		\ start of main RAM
$2002:0000 equ RAMEND		\ end of main RAM
$0800:0000 equ FLASHSTART  	\ start of Main Flash
$0810:0000 equ FLASHEND		\ end of possible main Flash
$0801:0000 equ APPFLASHSTART  	\ start of application flash
$0808:0000 equ APPFLASHEND  	\ end of application flash
$0000:F000 equ /APPFLASH	\ size of application flash

APPFLASHSTART TargetFlashStart	\ sets HERE at kernel start up

\ ============================
\ *N Stack and user area sizes
\ ============================

$00F0 equ UP-SIZE		\ size of each task's user area
$00F0 equ SP-SIZE		\ size of each task's data stack
$0100 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

\ $2001:0000 task-size - equ INIT-T0

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
2 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

0 equ useUSART1?	\ -- n
\ *G Set non-zero to compile code for USART1,
\ ** device *\fo{Console1}.
#115200 equ console1-speed
\ *G *\fo{Console1} speed in BPS.

0 equ useUSART2?	\ -- n
\ *G Set non-zero to compile code for USART2,
\ ** device *\fo{Console2}.
#9600 equ console2-speed
\ *G *\fo{Console2} speed in BPS.

1 equ useUSART3?	\ -- n
\ *G Set non-zero to compile code for USART3,
\ ** device *\fo{Console3}.
115200 equ console3-speed
\ *G *\fo{Console3} speed in BPS.

0 equ useUART4?	\ -- n
\ *G Set non-zero to compile code for UART4,
\ ** device *\fo{Console4}.
#9600 equ console4-speed
\ *G *\fo{Console4} speed in BPS.

3 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Ports 1..6 are the on-chip UARTs. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#10 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 1 equ ColdChain?		\ nz to use cold chain mechanism
 1 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked



\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ***************
\ *S Kernel files
\ ***************

  include %CpuDir%/CM4Def		\ Cortex generic equates and SFRs
  include %CpuDir%/sfrSTM32F4xx		\ STM32F4 special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks

PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startSTM32F407	\ STM32F407 startup code
l: crcslot
  0 ,
l: crcstart
  include %CpuDir%/CodeM0lite		\ low level kernel definitions
  include %CpuDir%/Drivers/FlashSTM32F4	\ Flash programming code
  include %CpuDir%/kernel72lite		\ high level kernel definitions
  include %CpuDir%/Drivers/rebootCM4	\ reboot using CM4 core
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
: selio-ser3	\ --
\ *G Initialise USART3 for operation on GPIO pins PD8 (Tx), PD9(Rx).
\ ** Note that the serial initialisation code cannot use
\ ** the easy pin I/O code as the serial initialisation code is
\ ** run before the cold chain is run.
\ Clocks and reset
  _RCC
  bit3 over rccAHB1ENR + or!		\ enable clock to GPIOD
  bit3 over rccAHB1RSTR + bic!		\ take it out of reset
  bit18 over rccAPB1ENR + or!		\ enable clock to USART3
  bit18 over rccAPB1RSTR + bic!		\ take it out of reset
  drop
\ Default on PD8/9
  _GPIOD >r
  $000A:0000 $000F:0000 r@ gpioMODER + setMask	\ alternate function on 8/9
  $0000:0077 $0000:00FF r> gpioAFRH + setMask	\ AF7 for both
;
  include %CpuDir%/Drivers/serSTM32F4xxp \ polled serial driver
  include %CpuDir%/Dump			\ DUMP .S etc development tools

  include %CpuDir%/Drivers/gpioSTM32F4xx \ easy pin access
  include %CpuDir%/Drivers/SysTickCM4	\ system ticker
  ' start-clock AtCold

tasking? [if]
  include %CpuDir%/MultiCM0lite		\ multitasker
[then]


\ *************
\ *S End of kernel
\ *************

buildfile liteSTM32F407discoSA.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." **************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE Forth Lite for STM32F4"
  cr version$ $. space BuildDate$ $.
  cr ." Copyright (C) 2014 MicroProcessor Engineering Ltd."
  .banner
;
external


\ *******************
\ *S Application code
\ *******************

  include %HwDir%/ReProg/ReFlash	\ ReFlash utility
  include %CpuDir%/include		\ include from AIDE

RAMEND constant RP-END	\ end of available RAM


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CpuDir%/LIBRARY
end-libs

decimal

\ Add a kernel checksum
crcstart here crcslot crc32 checksum
/DefStart 124 > [if]
  .( DEFSTART area too big ) abort
[then]

update-build
FINIS


\ ======
\ *> ###
\ ======

