\ liteXMC2GoSA.ctl - standalone Forth for XMC1100-0064 in XMC2Go

((
Copyright (c) 2014, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20151218 MPE003 Converted to XMC2Go board
20140524 RDO002 RDoering: Adapted to STM32F030Nucleo with STM32F030R8 Core
20140205 MPE001 First release for STM32F0xx.
))


\ ================
\ *! litesxmc2gosa
\ *T Control file for Infineon XMC2Go board
\ ================
\ *P The control file *\i{CortexLite/Hardware/XMC2Go/liteXMC2GoSA.ctl}
\ ** produces a standalone Forth for the Infineon XMC2Go board.
\ ** The serial port is connected to USIC0_CH0 through the on-board
\ ** J-Link VCOM USB serial port. The Flash is programmed by the
\ ** cross-compiler through the on-board J-Link.

\ ** The J-LINK software is a free download from the Segger website
\ ***\f{https://www.segger.com/jlink-software.html}. The MPE Lite
\ ** compiler can access the on-board J-Link directly once the drivers
\ ** have been installed and *\i{JLinkARM.dll} has been copied to the
\ ** directory containing the compiler *\i{xArmCortexLite.exe}.

\ *P You cannot use the easy GPIO code for UART initialisation unless
\ ** the GPIO ports are clocked and taken out of reset in the start
\ ** up code.

\ *P When you modify this file for your own hardware do not forget
\ ** to update the GPIO pin assignments and alternate function
\ ** selections.

only forth definitions  decimal


\ ******************************
\ Define the default directories
\ ******************************

c" ."			setmacro AppDir		\ application files
c" ."			setmacro HwDir		\ Board hardware files
c" ..\.."		setmacro CpuDir		\ CPU specific files
c" ..\Common"		setmacro CommonDir	\ common high level files
c" ..\Examples"		setmacro ExampleDir	\ example files


\ ***************************************
\ Load compiler extensions such as macros
\ ***************************************

include %CpuDir%/Macros


\ *********************************************************
\ Turn on the cross compiler and define CPU and log options
\ *********************************************************

\ file: PROG.log                \ uncomment to send log to a file

CROSS-COMPILE

only forth definitions          \ default search order

  no-log                        \ uncomment to suppress output log
  rommed                        \ split ROM/RAM target
  interactive                   \ enter interactive mode at end
  +xrefs			\ enable cross references
  align-long                    \ code is 32bit aligned
  Cortex-M0			\ Thumb2 processor type and register usage
  -LongCalls			\ no calls outside 25 bit range
  +FlashCompile			\ target compiles to Flash
  Hex-I32			\ also produce Intel Hex-32 obj format
  +SaveCdataOnly		\ no data area image files

0 equ false
-1 equ true


\ *******************
\ *S Configure target
\ *******************

\ ======================
\ *N XMC11xx definitions
\ ======================

1000:1000 equ FlashBase	\ -- addr
\ *G Start address of Flash.

#1100 equ XMCtype	\ -- 1100
\ *G Define the XMC type, using 1100 as the generic type
#64 kb equ /XMCflash	\ -- u
\ *G Define the size of the internal Flash.

#48 cells equ /ExcVecs	\ -- len
\ *G Size of the exception/interrupt vector table. There are
\ ** 16 vectors reserved by ARM.

0 Mhz equ xtal-speed	\ -- hz
\ *G Master oscillator crystal clock rate in HZ. Set to zero if
\ ** there is no crystal.
64 Mhz equ irc-speed	\ -- hz
\ *G Internal oscillator clock rate in HZ. See data sheet for
\ ** more details.

#32 MHz equ system-speed	\ -- hz
\ *G Requested CPU clock speed in HZ.
system-speed 2 * equ periph-speed	\ -- hz
\ *G Fast peripheral clock speed.
system-speed equ prog-speed	\ -- hz
\ *G Clock speed assumed when programming the Flash.
\ ** Do not change this unless you really know what you are doing.


\ =============
\ *N Memory map
\ =============
\ *P If you are using the *\fo{Reflash} code in the *\i{ReProg}
\ ** folder, note that the Flash reprogramming code uses RAM
\ ** from $2000:0000..$2000:0FFF and its mirrors. Ensure that
\ ** your stacks are outside this region.

\ *P The Flash memory starts at $1000:1000. The top 256 bytes (one
\ ** page) is used to save autostart and application linkage
\ ** information.
\  $0000:0000 $FFFF:FFFF udata section CATCHALL	\ for debugging with J-Link
  $1000:1000 $1000:7FFF cdata section liteXMC2GoSA	\ code section in Flash
  $2000:0000 /ExcVecs 2* +			\ allow for RAM vectors
             $2000:0FFF idata section PROGd	\ 4k IDATA
  $2000:1000 $2000:3FFF udata section PROGu	\ 12k UDATA

interpreter
: prog liteXMC2GoSA  ;		\ synonym
target

PROG PROGd PROGu  CDATA		\ use Code for HERE , and so on

$1001:0F00 equ INFOSTART   	\ 1 page of 256 bytes
$1001:0F80 equ APPSTART
$1001:0FFF equ INFOEND
$2000:0000 equ RAMSTART
$2000:1000 equ RAMEND
$1000:1000 equ FLASHSTART  	\ start of Main Flash
$1001:1000 equ FLASHEND		\ end of possible main Flash
$1000:8000 equ APPFLASHSTART  	\ start of application flash
$1001:0F00 equ APPFLASHEND  	\ end of application flash
APPFLASHEND APPFLASHSTART -
           equ /APPFLASH	\ size of application flash

APPFLASHSTART TargetFlashStart	\ sets HERE at kernel start up


\ ============================
\ *N Stack and user area sizes
\ ============================

$0F0 equ UP-SIZE		\ size of each task's user area
$0A0 equ SP-SIZE		\ size of each task's data stack
$0F0 equ RP-SIZE		\ size of each task's return stack
up-size rp-size + sp-size +
  equ task-size			\ size of TASK data area
\ define the number of cells of guard space at the top of the data stack
#2 equ sp-guard			\ can underflow the data stack by this amount

$0100 equ TIB-LEN		\ terminal i/p buffer length

\ define nesting levels for interrupts and SWIs.
1 equ #IRQs			\ number of IRQ stacks,
				\ shared by all IRQs (1 min)
0 equ #SVCs			\ number of SVC nestings permitted
				\ 0 is ok if SVCs are unused

\ ==========================
\ *N Serial and ticker rates
\ ==========================

1 equ useUART0?	\ -- n
\ *G Set non-zero to compile code for UART0,
\ ** device *\fo{Console0}.
#115200 equ console0-speed
\ *G *\fo{Console0} speed in BPS.

0 equ console-port	\ -- n ; Designate serial port for terminal (0..n).
\ *G Port 0 is the on-chip UART. The internal USB device
\ ** is port 10, and bit-banged ports are defined from 20 onwards.

#1 equ tick-ms		\ -- ms
\ *G Timebase tick in ms.


\ =====================
\ *N Software selection
\ =====================

\ Kernel components
 0 equ Tiny?			\ nz to make a minimal kernel.
 1 equ ColdChain?		\ nz to use cold chain mechanism
 0 equ tasking?			\ true if multitasker needed
   6 cells equ tcb-size		\   for internal consistency check
 0 equ timebase?		\ true for TIMEBASE code
 0 equ softfp?			\ true for software floating point
 0 equ FullCase?		\ true to include ?OF END-CASE NEXTCASE extensions
 0 equ target-locals?		\ true if target local variable sources needed
 0 equ romforth?		\ true for ROMForth handler
 0 equ blocks?			\ true if BLOCK needed
 $0000 equ sizeofheap		\ 0=no heap, nz=size of heap
   1 equ heap-diags?		\   true to include diagnostic code
 0 equ paged?			\ true if ROM or RAM is paged/banked



\ *****************
\ default constants
\ *****************

cell equ cell				\ size of a cell (16 bits)
0 equ false
-1 equ true


\ ************
\ *S Kernel files
\ ************

  include %CpuDir%/CM0Def		\ Cortex-M0 generic equates and SFRs
  include %CpuDir%/sfrXMC1100		\ LPC11xx special function registers
  include %CpuDir%/StackDef		\ Reserve default task and stacks
PROGd  sec-top 1+ equ UNUSED-TOP  PROG	\ top of memory for UNUSED
  include %HwDir%/startXMC11xx		\ startup code
l: crcslot
  0 ,
l: crcstart
  include %CpuDir%/CodeM0lite		\ low level kernel definitions
  include %CpuDir%/Drivers/FlashXMC11xx	\ Flash programming code
  include %CpuDir%/kernel72lite		\ high level kernel definitions
  include %CpuDir%/Drivers/rebootXMC1xxx \ reboot using watchdog
  include %CpuDir%/IntCortex		\ interrupt handlers for NVIC
\  include %CpuDir%/FaultCortex		\ fault exception handlers for NVIC
  include %CpuDir%/Drivers/serXMC1xxxp	\ polled serial driver
  include %CpuDir%/Dump			\ DUMP .S etc development tools

  include %CpuDir%/Drivers/gpioXMC1xxx	\ easy pin access
  include %CpuDir%/Drivers/SysTickXMC2Go	\ ticker
  ' start-clock AtCold

tasking? [if]
  include %CpuDir%/MultiCM0lite		\ multitasker
[then]

SIZEOFHEAP [if]
  include %CommonDir%/heap32		\ memory allocation set
[then]


\ ****************
\ *S End of kernel
\ ****************

buildfile liteXMC2Go.no
l: version$
  build$,
l: BuildDate$
  DateTime$,

internal
: .banner	\ --
  cr ." *************************"
;

: .CPU		\ -- ; display CPU type
  .banner
  cr ." MPE PowerForth for XMC2Go"
  cr version$ $. space BuildDate$ $.
  .banner
;
external


\ *******************
\ *S Application code
\ *******************

  include %CpuDir%/include		\ include from AIDE

progu sec-end cdata constant RP-END	\ end of available RAM


\ ***************
\ *S Finishing up
\ ***************

libraries	\ to resolve common forward references
  include %CpuDir%/LibM0M1
  include %CpuDir%/LIBRARY
end-libs

decimal

\ Add a kernel checksum
crcstart here crcslot crc32 checksum
/DefStart 124 > [if]
  .( DEFSTART area too big ) abort
[then]

update-build

\ ((
\ Define device, speed (0=auto) and project file
\ Download the target image when compilation finishes
jlink XMC1100-0064 0 Project.jlink	\ JLINK and device setup
useSWD					\ use SWD rather than the default JTAG
-download-prompts			\ no prompting for download
afterwards  download-sections		\ download image after FINIS
\ ))

FINIS


\ ======
\ *> ###
\ ======
