\ StartXMC11xx.fth - Cortex startup code for Infineon XMC11xx family

((
Copyright (c) 2009, 2010, 2011, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20151212 SFP002 Conversion to XMC1100 family.
20110614 MPE001 First release.
))

only forth definitions  decimal

\ ===========
\ *! startlxmc11xx
\ *T Cortex-M0 start up for XMC11xx
\ ===========
\ *P The file *\i{Cortex/Hardware/XMC1xxx/startXMC11xx.fth} contains
\ ** the start up code for XMC11xx devices. Start up code for other
\ ** Cortex-M0 and M1 implementations can be found in parallel
\ ** directories. The absolute minimum code to start a Cortex-M0
\ ** is in *\i{Cortex/StartCortex.fth}.

interpreter
: FillVT,	\ --
\ *G Fill the vector table.
  /ExcVecs 0 do
    $4800 /ExcVecs 4 - 2 rshift or w,	\ LDR  r0, [ pc, # /ExcVecs 4 - ]
    $4700 w,				\ BX   r0
  4 +loop
;
target
l: ExcVecs	\ -- addr ; start of vector table
\ *G The exception vector table is *\fo{/ExcVecs} bytes long,
\ ** four bytes per entry. *\fo{/ExcVecs} is defined in the
\ ** control file. Note that the vector table for the Infineon
\ ** XMC devices has a very peculiar layout as a result of design
\ ** choices for the ROM bootloader. Apart from the first few
\ ** entries, the exception vector table contains executable code.
\ ** This table is copied to the start of RAM.
  FillVT,
l: VecEntries	\ -- addr ; start of entry table
\ *G The Vector Entry is the same length as the exception vector
\ ** table and contains the absolute entry address + 1 of the
\ ** exception handler. This table is copied to RAM after the
\ ** exception vector table.
  /ExcVecs allot&Erase

interpreter
: SetExcVec	\ addr exc# --
\ *G Set the given exception vector entry to the given address.
\ ** The Thumb bit is forced to 1.
  dup if				\ If not the stack top
    swap 1 or swap			\   set the Thumb bit
  endif
  dup 1 > if
    cells VecEntries + !
  else
    cells ExcVecs + !
  then
;
target

L: CLD1		\ holds xt of main word
  0 ,					\ fixed by MAKE-TURNKEY

proc DefExc2	b  $	end-code	\ NMI
proc DefExc3	b  $	end-code	\ HardFault
proc DefExc11	b  $	end-code	\ SVC
proc DefExc14	b  $	end-code	\ PendSV
proc DefExc15	b  $	end-code	\ SysTick


\ Calculate the initial value for the data stack pointer.
\ We allow for TOS being in a register and guard space.

[undefined] sp-guard [if]		\ if no guard value is set
0 equ sp-guard	\ -- +n
\ *G The number of cells reserved as guard space on the Forth
\ ** data stack. Usually set to two in the control file. This
\ ** value allows the data stack to underflow by the given amount
\ ** before other data is corrupted.
[then]

init-s0 tos-cached? sp-guard + cells - equ real-init-s0  \ -- addr
\ *G The data stack pointer value set at start up.

irc-speed 64 MHz <> [if]
  cr ." I don't know how to handle this clock speed" abort
[then]

\ *E Clock setting values
\ **
\ **              64MHz
\ ** MCK = --------------------
\ **                     FDIV
\ **         2 ( IDIV + ------ )
\ **                     256
\ **
\ ** Assume FDIV=0, then IDIV = 32MHz/MCLK

32 MHz system-speed / equ IDIVval
\ *G Calculated value of IDIV.
$00 17 lshift $0001:0000 or IDIVval 8 lshift or  $00 or equ CLKCRval
\ *G Written as CLK_VAL1 in the Flash.
$0008 equ CGATCLR0val
\ *G Written as CLK_VAL2 in the Flash.

: StartCortex	\ -- ; never exits
\ *G Set up the Forth registers and start Forth. Other primary
\ ** hardware initialisation is also performed here.
  begin
    INT_STACK_TOP SP_main sys!		\ set SP_main for interrupts
    INIT-R0 SP_process sys!		\ set SP_process for tasks
    2 control sys!			\ switch to SP_process
    REAL-INIT-S0 set-sp			\ Allow for cached TOS and guard space
    INIT-U0 up!				\ USER area
    ExcVecs $2000:0000 /ExcVecs 2* cmove \ Copy vectors to RAM
    CLD1 @ execute
  again
;
external

INT_STACK_TOP StackVec# SetExcVec	\ Define initial return stack
' StartCortex ResetVec# SetExcVec	\ Define startup word
DefExc2       2         SetExcVec	\ set default exception handlers
DefExc3       3         SetExcVec
CLKCRval      $1000:1010 !		\ CLK_VAL1
CGATCLR0val   $1000:1014 !		\ CLK_VAL2
DefExc11      11        SetExcVec
DefExc14      14        SetExcVec
DefExc15      15        SetExcVec


\ ------------------------------------------
\ reset values for user and system variables
\ ------------------------------------------

[undefined] umbilical? [if]
L: USER-RESET
  init-s0 tos-cached? sp-guard + cells - ,	\ s0
  init-r0 ,				\ r0
  0 ,  0 ,                              \ #tib, 'tib
  0 ,  0 ,                              \ >in, out
  $0A ,  0 ,                            \ base, hld

\ initial values of system variables
L: INIT-FENCE 0 ,                       \ fence
L: INIT-DP 0 ,                          \ dp
L: INIT-RP  0 ,				\ RP
L: INIT-VOC-LINK 0 ,                    \ voc-link
[then]


\ *****************
\ *S SCU protection
\ *****************

: unlockBPS	\ --
\ *G Unlock the SCU Bit Protection Scheme.
  $00C0 _GCU gcu.PASSWD + !  ;

: lockBPS	\ --
\ *G Lock the SCU Bit Protection Scheme.
  $00C3 _GCU gcu.PASSWD + !  ;


\ ======
\ *> ###
\ ======

decimal

