\ IntroLite.man

((
Copyright (c) 2013, 2014, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)2380 631441
fax: +44 (0)2380 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

Change History
==============
Added LPC1114FN28 and Infineon XMC2Go.
))

\ ************
\ *! introlite
\ *T Introduction
\ ************
\ *P This manual documents the MPE Forth Lite kernel for ARM
\ ** Cortex processors. The Lite kernel is not the same as the
\ ** PowerForth kernel supplied with the Professional, Standard
\ ** and Stamp compilers. The Lite kernel is smaller and compiles
\ ** directly to Flash, which has required changes to the kernel.
\ ** These changes are discussed later.

\ *******************
\ *S Supported boards
\ *******************
\ *P Supported boards are the ST STM32F072B
\ ** Discovery, ST STM32F4 Discovery (with the STM32F407 chip),
\ ** the Freescale Freedom KL25Z, the NXP LPC1114FN28, and the
\ ** Infineon XMC2Go. Preliminary releases for
\ ** other boards may also be included.

\ *P The following discuss getting a cross-compiled binary image
\ ** into the boards.

\ =============================
\ *N STM32F072B Discovery board
\ =============================
\ *P The MPE Forth kernel uses the hardware USART1 for serial
\ ** communications to talk to the Forth kernel on the board.
\ ** We use an FTDI USB to 3v3 TTL serial cable for this:
\ *C   http://www.ftdichip.com/Products/Cables/RPi.htm
\ *P Connect the cable to the Discovery board:
\ *(
\ *B Black - Gnd,
\ *B Yellow - PA9 - PC Rx, STM32 USART1 Tx,
\ *B Orange - PA10 - PC Tx, STM32 USART1 Rx.
\ *)
\ *P Set your terminal emulator, e.g. PowerTerm in AIDE, to
\ ** 115200 baud, 8 data bits, no parity.

\ *P Power the board using the central (ST Link v2) USB connector.

\ *P After cross compiling the control file
\ ** *\i{Cortex/Hardware/STM32F072Bdisco/liteSTM32F072sa.ctl}
\ ** the image file will be
\ ** *\i{Cortex/Hardware/STM32F072Bdisco/liteSTM32F072sa.hex}

\ *P To program the Flash on the Discovery board for the first time,
\ ** use the STM32 ST-LINK utility. See:
\ *C   http://www.st.com/web/en/catalog/tools/PF258168

\ ==========================
\ *N STM32F4 Discovery board
\ ==========================
\ ** The serial port is connected to USART3 on pins PD8 (USART3-Tx)
\ ** and PD9 (USART3-Rx).
\ ** We use an FTDI USB to 3v3 TTL serial cable for this:
\ *C   http://www.ftdichip.com/Products/Cables/RPi.htm
\ *P Connect the cable to the Discovery board:
\ *(
\ *B Black - Gnd,
\ *B Yellow - PD8 - PC Rx, STM32F4 USART3 Tx,
\ *B Orange - PD9 - PC Tx, STM32F4 USART3 Rx.
\ *)
\ *P Set your terminal emulator, e.g. PowerTerm in AIDE, to
\ ** 115200 baud, 8 data bits, no parity.

\ *P Power the board using the ST Link v2 USB connector.

\ *P After cross compiling the control file
\ ** *\i{Cortex/Hardware/STM32F407disco/liteSTM32F407discoSA.ctl}
\ ** the image file will be
\ ** *\i{Cortex/Hardware/STM32F407disco/liteSTM32F072.hex}.

\ *P To program the Flash on the Discovery board for the first time,
\ ** use the STM32 ST-LINK utility. See:
\ *C   http://www.st.com/web/en/catalog/tools/PF258168

\ =====================================
\ *N Freescale Freedom FRDM-KL25Z board
\ =====================================
\ *P The Forth uses UART0 on the FRDM-KL25Z board connected to
\ ** the on-board serial to USB adapter. The board presents
\ ** itself as a USB memory stick and a comms port. To use these,
\ ** download the latest OpenSDA firmware drivers from
\ *C   http://www.pemicro.com/opensda/
\ *P and install them. After installation on Windows, you will see
\ ** the memory stick as a drive, and a new comms port will be
\ ** available to AIDE.

\ *P When you put a new binary file into the new drive, the
\ ** board's second processor checks the file and Flashes it
\ ** into the target. The compiler uses this facility. It is
\ ** automated by adding the following two
\ ** lines before *\fo{FINIS} in the control file:
\ *C   afterwards  fdel F:\FRDMKL25Zsa.bin
\ *C   afterwards  fcopy FRDMKL25Zsa.bin F:\FRDMKL25Zsa.bin
\ *P Change the drive letter as required.

\ *P After cross compiling the control file
\ ** *\i{Cortex/Hardware/KinetisKL20/liteFRDMKL25Zsa.ctl} copy the
\ ** binary file *\i{Cortex/Hardware/KinetisKL20/FRDMKL25Zsa.bin}
\ ** to the Flash drive. The board will reboot and the new Forth
\ ** file will be running. By default, the Forth Lite kernel
\ ** continuously changes the colour of the tri-colour LED.

\ *P The terminal emulator should be set to 115200 baud.
\ ** After the file has been copied to the Flash drive, it is
\ ** programmed into the target. When the target reboots, the
\ ** Forth prompt appears.


\ ==============
\ *N LPC1114FN28
\ ==============
\ *P This chip is a Cortex-M0 in a DIP28 package with 32 kb of
\ ** Flash and 4 kb of RAM. The chip runs at 48 MHz from the
\ ** internal oscllator. Comms are handled by UART0 at 115200
\ ** baud, connected to RXD (PIO1.6) and TXD (PIO1.7).

\ *P Providing that your hardware has buttons for both the
\ ** reset pin and the boot pin, you can use FlashMagic to
\ ** program the Flash.  See:
\ *C   http://www.flashmagictool.com/


\ ==================
\ *N Infineon XMC2Go
\ ==================
\ *P This tiny board includes an XMC1100-0064 CPU with 64k of
\ ** Flash and 16 kb RAM, plus an on-board J-Link device with
\ ** a virtual COM port. The MPE cross compiler supports the
\ ** J-Link directly and programs the Flash at the end of
\ ** compilation.

\ *P You must install the Segger J-Link tools from
\ *C   https://www.segger.com/jlink-software.html
\ *P and then copy the file *\i{JLinkARM.dll} into the directory
\ ** containing the cross compiler. You must use the Segger tools
\ ** of version 5.02h or later.


\ ==============================
\ *N STM32F031K6 Nucleo-32 board
\ ==============================
\ *P The MPE Forth kernel uses the hardware USART1 for serial
\ ** communications to talk to the Forth kernel on the board
\ ** through the on-board ST-Link v2.1 which provides a USB
\ ** virtual serial port.

\ *P Set your terminal emulator, e.g. PowerTerm in AIDE, to
\ ** 115200 baud, 8 data bits, no parity.

\ *P Power the board using the central (ST Link v2.1) USB connector.

\ *P After cross compiling the control file
\ ** *\i{Cortex/Hardware/Nucleo-F031K6/liteSTM32F031NucleoSA.ctl}
\ ** the image file will be
\ ** *\i{Cortex/Hardware/Nucleo-F031K6/LITESTM32F031SA.hex}

\ *P To program the Flash on the Discovery board for the first time,
\ ** use the STM32 ST-LINK utility. See:
\ *C   http://www.st.com/web/en/catalog/tools/PF258168


\ ===================
\ *N STP LPC812 board
\ ===================
\ *P This is a minimalist board with 16 kb of Flash. It is used
\ ** with the FlashMagic utility in a conventional cross-compile,
\ ** download to Flash, and then test cycle. It *\b{is} possible
\ ** to squeeze the Flash programming code onto the board, but
\ ** that does not leave enough space for a real application.

\ *P The board's UART0 is connected to an FTDI serial to USB
\ ** converter. The kernel runs the UART at 115200 baud. Most
\ ** versions of Windows include a driver for the FTDI adapters.

\ *P After cross compiling the control file
\ ** *\i{Cortex/Hardware/LPC81x/liteSTP812sa.ctl}
\ ** the image file will be
\ ** *\i{Cortex/Hardware/LPC81x/liteSTP812sa.hex}

\ *P To program the Flash on board, move the jumper by the
\ ** reset button so that both pins are covered. Press the
\ ** reset button and then use use the FlashMagic utility. See:
\ *C   http://www.flashmagictool.com/
\ *P After the Flash has been programmed, move the jumper so
\ ** that one pin is uncovered; it is convenient to leave the
\ ** jumper on the other pin. The press the reset button to
\ ** start the Forth application.


\ ***********************
\ *S Producing the kernel
\ ***********************
\ *P The Forth kernel is built using a cross compiler running
\ ** in the AIDE environment. AIDE contains three primary windows:
\ ** compiler, terminal emulator and text editor. The compilation
\ ** results can be seen in the compiler window. During compilation,
\ ** the compilation results can be seen in the compiler/tool window.
\ ** The process below is very similar for all boards except for the
\ ** use of vendor-specific tools.

\ *P To compile the standalone Forth kernel, find the  button for
\ **  your board, e.g. "STM32F072 Lite SA", in AIDE's main toolbar
\ ** and click it. After successful compilation, program the Flash
\ ** with the file *\i{LITESTM32F072SA.hex} using the STM32 ST-LINK
\ ** utility. See:
\ *C   http://www.st.com/web/en/catalog/tools/PF258168
\ *P Other boards will have other utilities to Flash the board.

\ *P After programming the board, the new kernel will be running.
\ ** To talk to the kernel, use AIDE's PowerTerm terminal emulator.
\ ** The kernel talks to PowerTerm through the FTDI cable.
\ ** Finding this UART can require a little botheration. There
\ ** are two ways to do this.
\ *( 1
\ *B Go to Windows Control Panel -> System -> Device Manager -> Ports.
\ ** One of the entries will be something like:
\ ** *\b{USB Serial Port (COM14)}.
\ ** COM14 will be the UART you will use.
\ *B Go to AIDE's PowerTerm configuration dialog. Select the drop
\ ** down list in the COM Port# group. It will list all the available
\ ** COM ports by COM number and name. The one you want includes:
\ ** *\b{COMxx USB Serial Port}.
\ *)
\ *P Now select the required port in the PowerTerm configuration
\ ** dialog and ensure that it is set to 115200 baud, 8 data bits, 1
\ ** stop bit, no parity and file server is enabled. Save the
\ ** configuration and press OK.
\ ** Start the PowerTerm connection and you should be talking to
\ ** the kernel.

\ *P Once you have the Forth kernel running on the target, you
\ ** can use Forth's internal flash utility to flash a newly
\ ** compiled Forth image. Type:
\ *C   REFLASH
\ *P The Flash will be erased, and AIDE will present you with
\ ** a file dialog box. Find the file *\i{LITESTM32F072SA.img}
\ ** (*\b{not} the .hex file)), select it and the file will
\ ** be downloaded and flashed. If everything goes wrong, just
\ ** start again with the Flash utility.


\ *******************
\ *S About the kernel
\ *******************
\ *P The Forth kernel you have made and installed is a cut-down
\ ** version of the full PowerForth kernel. It has extensions so
\ ** that code is compiled directly into Flash memory. When
\ ** you want to add code, you can compile it directly on the
\ ** Forth kernel by using the phrase
\ *C   include <filename>
\ *P on the target Forth command line. AIDE will then try to
\ ** deliver the file to the kernel. If the file cannot be
\ ** found, AIDE will let you change the default directory
\ ** (the commonest problem) or correct the spelling.

\ *P To start over just type *\fo{EMPTY}, the application
\ ** region of the Flash will be cleaned, and the CPU will be
\ ** rebooted. To reuse your compiled code at the next reboot
\ ** or power up, use the word *\fo{COMMIT}.
\ ** If you just want to preserve the code, use:
\ *C   0 COMMIT
\ *P If you want the Forth to run a word, say *\fo{APP}, use:
\ *C   ' APP COMMIT

\ *P To support compilation to Flash, the kernel uses a compilation
\ ** buffer (usually 512 bytes) in RAM. The use of this buffer is
\ ** mostly transparent to the user.


\ **********
\ *S Gotchas
\ **********

\ =================
\ *N Flash problems
\ =================
\ *P If you forget to use *\fo{COMMIT} and *\fo{EMPTY} appropriately,
\ ** the Flash may not be correctly erased and application
\ ** compilation may fail. Try to use *\fo{EMPTY} and if that fails,
\ ** reinstall a new kernel.


\ ===============================
\ *N Flash kernel is non-standard
\ ===============================
\ *P The Lite kernel is not completely to the ANS and Forth-2012
\ ** standards.

\ *P A particular change is that the Forth word *\fo{IMMEDIATE} is
\ ** not present. Instead, precede the the word with *\fo{IMM} and
\ ** will be immediate. This change is a result of the particulars
\ ** of Flash implementations and their controllers.


\ ==============================
\ *N Building tables with CREATE
\ ==============================
\ *P The word *\fo{CREATE} switches on compilation to the RAM
\ ** buffer. When you just need to build a table, you must
\ ** remember to turn the buffer off to flush the contents to
\ ** Flash.
\ *E create MyTable   \ -- addr
\ **   5 ,  6 ,  7 ,  8 ,          \ lay down data
\ ** -BuffComp                     \ flush buffer to Flash


\ ********************
\ *S Technical support
\ ********************
\ *P Technical support for the Lite compilers is done in our
\ ** spare time. If you have problems, send us an email to
\ *C   tech-support@mpeforth.com


\ ******
\ *> ###
\ ******

