\ Generic XC6 Umbilical Forth high level kernel, UFkernel.fth

((
Copyright (c) 1988-1998
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)2380 631441
fax: +44 (0)2380 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))

harvard? 0= [if]
compiler
: @c		@  ;
: c@c		c@  ;
: 2@c		2@  ;
: cmovec	cmove  ;
target
[then]


\ **************
\ User variables
\ **************

host
variable next-user(t)
  next-user(t) off

interpreter
: next-user	\ -- offset
  next-user(t) @(h)
;

: +user		\ size -- ; -- addr(t)
  next-user(t) @(h) user
  next-user(t) @(h) + next-user(t) !(h)
;
target

tcb-size +user SELF		\ task identifier and TCB
next-user equ s0-offset
cell +user S0			\ base of data stack
next-user equ r0-offset
cell +user R0                   \ base of return stack
cell +user base                 \ number conversion base
cell +user out                  \ output counter
cell +user opvec		\ output device
cell +user ipvec		\ input device


\ ****************
\ System constants
\ ****************

optimised? 0= [if]
 0 CONSTANT 0
-1 constant -1
[then]


\ ***********************
\ Generic I/O definitions
\ ***********************

((
Generic I/O allows the Forth words KEY KEY? EMIT TYPE CR to
use any I/O device. The user variables IPVEC and OPVEC contain
pointers to the current device structure. For example, input can
be from a serial channel, and output can be to an LCD screen. The
selection can be changed at any time by the application, and
because IPVEC and OPVEC are USER variables, different tasks may
have different I/O devices.

The generic I/O structure consists of any array of five (six for
Harvard targets) xts. The XTS are for the words that perorm the
following basic functions.

cell	KEY action
cell	KEY? action
cell	EMIT action
cell	TYPE action
cell	CR action
cell	TYPEC action	Harvard targets only

The CR and TYPE actions are provided to ease implementations
of devices such as LCD output in which CR does not naturally
correspond to 13 EMIT 10 EMIT, and for which TYPE will be much
faster than repeated EMITs.

The generic I/O dispatcher handles all use of OUT for the output
functions. OUT is manipulated before the action is performed so
that special cases can update OUT themselves.

When creating a new device driver, make an array that contains
the pointers. The following code is taken from the 8051 serial
driver.

cdata	\ this table goes in CODE space
create SerConsole	\ -- addr ; OUT managed by upper driver
  ' (serkey) ,		\ -- char ; receive char, must schedule if tasking
  ' (serkey?) ,		\ -- flag ; check receive char
  ' (seremit) ,		\ -- char ; display char
  ' (sertype) ,		\ caddr len -- ; display string
  ' (sercr) ,		\ -- ; display new line
  ' (sertypec) ,	\ caddr len -- ; display string from CDATA space

To select serial input, the phrase
  SerConsole IpVec !
is all that is needed.
))

harvard? [if]
: key	        \ -- char ; receive char
  ipvec @ ( 0 cells + ) @c execute
;
: KEY?		\ -- flag ; check receive char
  ipvec @ cell +  @c execute
;
: EMIT		\ -- char ; display char
  1 out c+!
  opvec @ [ 2 cells ] literal + @c execute
;
: TYPE		\ caddr len -- ; display string
  dup out c+!
  opvec @ [ 3 cells ] literal + @c execute
;
: CR		\ -- ; display new line
  0 out c!
  opvec @ [ 4 cells ] literal + @c execute
;
: TYPEC		\ caddr len -- ; display string
  dup out c+!
  opvec @ [ 5 cells ] literal + @c execute
;
[else]
: key	        \ -- char ; receive char
  ipvec @ ( 0 cells + ) @ execute
;
: KEY?		\ -- flag ; check receive char
  ipvec @ cell+  @ execute
;
: EMIT		\ -- char ; display char
  1 out c+!
  opvec @ [ 2 cells ] literal + @ execute
;
: TYPE		\ caddr len -- ; display string
  dup out c+!
  opvec @ [ 3 cells ] literal + @ execute
;
: CR		\ -- ; display new line
  0 out c!
  opvec @ [ 4 cells ] literal + @ execute
;
[then]


\ ******************************
\ Miscellaneous Word definitions
\ ******************************
((
: $.		\ c-addr -- ; display counted string
  count type
;
))
harvard? [if]
: (.")          \ -- ; displays in-line data
  (") countc typec
;
[else]
: (.")          \ -- ; displays in-line data
  (") count type
;
[then]

init-idata? [if]
3 cells equ #idhead

: INIT-IDATA	\ --
  init-ram                              \ copy IDATA sections to their destinations
  begin                                 \ len, addr, pageid, len data, ..., 0,
    dup @c
   while                                \ len<>0
    dup #idhead +  over 2@c             \ source, dest, len
    cmovec                              \ copy
    dup @c +  #idhead +                 \ step to next block
  repeat
  drop
;
[then]



