\ include.fth - INCLUDE using AIDE as the file server.

((
Copyright (c) 1991..2004, 2005, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====
Permit file nesting.

Change history
==============
20131219 MPE007 Updated for Lite compiler.
20051105 MPE006 Force interpretation on error.
20040505 MPE005 Modified for AIDE 2.500 onwards.
20021004 SFP004 Refactored to use CATCH/THROW
20020917 SFP003 Refactored to reduce size
20000530 SFP002 Converted to v6.1 targets
19990509 SFP001 Rewritten for AIDE/PowerTerm.
                Removed support for USE
First Release: 29/04/91
))

\ ===========
\ *! includeaide
\ *T Compile source code from AIDE
\ ===========
\ *P The file *\i{include.fth} provides support for compiling a
\ ** source file from the AIDE server.

decimal

$01 equ START
$06 equ ACK
$15 equ NAK

$12 equ ^R                                         \ read command code
$17 equ ^W                                         \ write
$15 equ ^U                                         \ $using
$01 equ ^A                                         \ use
$06 equ ^F                                         \ $from
$05 equ ^E                                         \ error

variable disk-error	\ -- addr
\ *G Receives transfer error status from the host.

: end-load      \ -- ; switch back to keyboard input
\ *G This word is automatically performed at the end of a download
\ ** to tidy up the comms.
  xon/xoff off                  	\ turn off handshaking
  echoing on                    	\ turn echoing back on
;

internal

: WAIT-ACK	\ --
\ +G Wait for ACK character at the end of part of a transfer.
  begin  key ack =  until
;

: WAIT-ACK/NACK	\ -- flag
\ +G Wait for NACk or ACK, return true for NACK.
  0
  begin
    drop				\ drop old flag
    key dup nak = tuck                  \ -- t/f char t/f
    swap ack =  or                      \ -- t/f t/f'
  until
;

: SEND-BLOCK#	\ n --
\ +G Send block number to server as two bytes.
  dup emit  8 rshift emit		\ send block no., low byte first
\  0 #256 um/mod swap                   \ split into high & low bytes
\  emit emit
  wait-ack                              \ wait for ack
;

: SYNCH-TO-HOST	\ --
\ +G Sync host to target
  begin  key start =  until		\ receive bytes until start code
  begin  key? while  key drop  repeat	\ flush the input
  ack emit				\ acknowledge as end of synch.
;

: SEND$		\ $addr char --
\ +G Send a trigger character and a counted string to the host.
  emit					\ send trigger
  synch-to-host				\ synchronise
  count dup send-block#			\ send string length
  type					\ send string
  ack emit				\ send ack to complete transfer
  wait-ack/nack disk-error !		\ get status response
;

: file-error    \ n --
\ +G Handle an error when a file is being *\fo{INCLUDE}d.
  ^e emit				\ trigger host error
  CR ." Error # " .  CR			\ show error
  end-load                              \ clean up
;

external

Tiny? [if]

: include       \ "<filename>" -- ; load file from host
\ *G Compile a file across the serial line from the AIDE file server.
\ ** Use in the form:
\ *C   include <filename>
\ *P The filename extension must be supplied.
  bl word                               \ get file name
  dup count upper                       \ convert to upper case
  ^f send$                      	\ send file name
  -1 send-block#                   	\ send last-p# - legacy
  1 send-block#                   	\ send first-p# - legacy
  synch-to-host                 	\ synchronise
  echoing off                   	\ not echoing
  xon/xoff on                   	\ initiate handshaking
[ blocks? ] [if]
  blk off
[then]
  0 <source> !  [compile] [		\ set up
  begin
    query				\   get user input
    ['] interpret catch ?dup if		\   interpret line
      file-error			\    display error
      s0 @ sp!				\    clean data stack
      [compile] [			\    force interpretation ; MPE006
    then
    echoing @
  until                                 \   do next line
;

[else]

0 value ShowLines?	\ -- flag
\ *G Set this non-zero to display source lines during compilation.
internal

: $include	\ $addr -- ; compile host file, counted string
\ +G Given a counted string representing a file name, compile
\ +* the file from AIDE.
  ^f send$                      	\ send file name
  -1 send-block#                   	\ send last-p# - legacy
  1 send-block#                   	\ send first-p# - legacy
  synch-to-host                 	\ synchronise
  echoing off                   	\ not echoing
  xon/xoff on                   	\ initiate handshaking
[ blocks? ] [if]
  blk off
[then]
  0 <source> !  [compile] [		\ set up
  begin
    query				\   get user input
    ShowLines?
    if  cr source type  endif
    ['] interpret catch ?dup if		\   interpret line
      file-error			\    display error
      s0 @ sp!				\    clean data stack
      [compile] [			\    force interpretation ; MPE006
    then
    echoing @
  until                                 \   do next line
;

external

: include       \ "<filename>" -- ; load file from host
\ *G Compile a file across the serial line from the AIDE file server.
\ ** Use in the form:
\ *C   include <filename>
\ *P The filename extension must be supplied.
  bl word                               \ get file name
  dup count upper                       \ convert to upper case
  $include				\ compile from file
;

[then]


\ ======
\ *> ###
\ ======

decimal

