\ sfrKL25Z4.fth - peripherals of Freescale KL25Z4 CPUs

((
Copyright (c) 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


Derived from MKL25Z4.h
Version:             rev. 1.6, 2013-04-05
Reference manual:    KL25P80M48SF0RM, Rev.3, Sep 2012


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ============
\ *! sfrkl25z4
\ *T Freescale KL25Z4 peripheral definitions
\ ============


\ *******************************
\ *S Interrupt Number Definitions
\ *******************************
enum IRQInterruptIndex {
  INT_Initial_Stack_Pointer    = 0,                /**< Initial stack pointer */
  INT_Initial_Program_Counter  = 1,                /**< Initial program counter */
  INT_NMI                      = 2,                /**< Non-maskable interrupt */
  INT_Hard_Fault               = 3,                /**< Hard fault exception */

  INT_SVCall                   = 11,               /**< A supervisor call exception */
  INT_PendableSrvReq           = 14,               /**< PendSV exception - request for system level service */
  INT_SysTick                  = 15,               /**< SysTick interrupt */
  INT_DMA0                     = 16,               /**< DMA channel 0 transfer complete/error interrupt */
  INT_DMA1                     = 17,               /**< DMA channel 1 transfer complete/error interrupt */
  INT_DMA2                     = 18,               /**< DMA channel 2 transfer complete/error interrupt */
  INT_DMA3                     = 19,               /**< DMA channel 3 transfer complete/error interrupt */
  INT_FTFA                     = 21,               /**< FTFA command complete/read collision interrupt */
INT_LVD_LVW                  = 22,               /**< Low Voltage Detect; Low Voltage Warning */
  INT_LLW                      = 23,               /**< Low Leakage Wakeup */
  INT_I2C0                     = 24,               /**< I2C0 interrupt */
  INT_I2C1                     = 25,               /**< I2C0 interrupt 25 */
  INT_SPI0                     = 26,               /**< SPI0 interrupt */
  INT_SPI1                     = 27,               /**< SPI1 interrupt */
  INT_UART0                    = 28,               /**< UART0 status/error interrupt */
  INT_UART1                    = 29,               /**< UART1 status/error interrupt */
  INT_UART2                    = 30,               /**< UART2 status/error interrupt */
  INT_ADC0                     = 31,               /**< ADC0 interrupt */
  INT_CMP0                     = 32,               /**< CMP0 interrupt */
INT_TPM0                     = 33,               /**< TPM0 fault; overflow and channels interrupt */
INT_TPM1                     = 34,               /**< TPM1 fault; overflow and channels interrupt */
INT_TPM2                     = 35,               /**< TPM2 fault; overflow and channels interrupt */
  INT_RTC                      = 36,               /**< RTC interrupt */
  INT_RTC_Seconds              = 37,               /**< RTC seconds interrupt */
  INT_PIT                      = 38,               /**< PIT timer interrupt */

  INT_USB0                     = 40,               /**< USB0 interrupt */
  INT_DAC0                     = 41,               /**< DAC0 interrupt */
  INT_TSI0                     = 42,               /**< TSI0 interrupt */
  INT_MCG                      = 43,               /**< MCG interrupt */
  INT_LPTimer                  = 44,               /**< LPTimer interrupt */

  INT_PORTA                    = 46,               /**< Port A interrupt */
  INT_PORTD                    = 47,               /**< Port D interrupt */
};


\ ******
\ *S ADC
\ ******

$4003:B000 const _ADC
$4003:B000 const _ADC0
  $00 const adcSC1	\ ADC Status and Control Registers 1, array offset: 0x0, array step: 0x4 */
  $00 const adcSC1A
  $04 const adcSC1B
  $08 const adcCFG1	\ ADC Configuration Register 1, offset: 0x8 */
  $0C const adcCFG2	\ ADC Configuration Register 2, offset: 0xC */
  $10 const adcR0	\ ADC Data Result Register, array offset: 0x10, array step: 0x4 */
  $14 const adcR1
  $18 const adcCV1	\ Compare Value Registers, offset: 0x18 */
  $1C const adcCV2      \ Compare Value Registers, offset: 0x1C */
  $20 const adcSC2      \ Status and Control Register 2, offset: 0x20 */
  $24 const adcSC3      \ Status and Control Register 3, offset: 0x24 */
  $28 const adcOFS      \ ADC Offset Correction Register, offset: 0x28 */
  $2C const adcPG       \ ADC Plus-Side Gain Register, offset: 0x2C */
  $30 const adcMG       \ ADC Minus-Side Gain Register, offset: 0x30 */
  $34 const adcCLPD     \ ADC Plus-Side General Calibration Value Register, offset: 0x34 */
  $38 const adcCLPS     \ ADC Plus-Side General Calibration Value Register, offset: 0x38 */
  $3C const adcCLP4     \ ADC Plus-Side General Calibration Value Register, offset: 0x3C */
  $40 const adcCLP3     \ ADC Plus-Side General Calibration Value Register, offset: 0x40 */
  $44 const adcCLP2     \ ADC Plus-Side General Calibration Value Register, offset: 0x44 */
  $48 const adcCLP1     \ ADC Plus-Side General Calibration Value Register, offset: 0x48 */
  $4C const adcCLP0     \ ADC Plus-Side General Calibration Value Register, offset: 0x4C */

  $54 const adcCLMD     \ ADC Minus-Side General Calibration Value Register, offset: 0x54 */
  $58 const adcCLMS     \ ADC Minus-Side General Calibration Value Register, offset: 0x58 */
  $5C const adcCLM4     \ ADC Minus-Side General Calibration Value Register, offset: 0x5C */
  $60 const adcCLM3     \ ADC Minus-Side General Calibration Value Register, offset: 0x60 */
  $64 const adcCLM2     \ ADC Minus-Side General Calibration Value Register, offset: 0x64 */
  $68 const adcCLM1     \ ADC Minus-Side General Calibration Value Register, offset: 0x68 */
  $6C const adcCLM0     \ ADC Minus-Side General Calibration Value Register, offset: 0x6C */

((
/* ----------------------------------------------------------------------------
   -- BP
   ---------------------------------------------------------------------------- */

/** BP - Peripheral register structure */
typedef struct BP_MemMap {
  uint32_t CTRL;                                   /**< FlashPatch Control Register, offset: 0x0 */
  uint8_t RESERVED_0[4];
  uint32_t COMP[2];                                /**< FlashPatch Comparator Register 0..FlashPatch Comparator Register 1, array offset: 0x8, array step: 0x4 */
  uint8_t RESERVED_1[4032];
  uint32_t PID4;                                   /**< Peripheral Identification Register 4., offset: 0xFD0 */
  uint32_t PID5;                                   /**< Peripheral Identification Register 5., offset: 0xFD4 */
  uint32_t PID6;                                   /**< Peripheral Identification Register 6., offset: 0xFD8 */
  uint32_t PID7;                                   /**< Peripheral Identification Register 7., offset: 0xFDC */
  uint32_t PID0;                                   /**< Peripheral Identification Register 0., offset: 0xFE0 */
  uint32_t PID1;                                   /**< Peripheral Identification Register 1., offset: 0xFE4 */
  uint32_t PID2;                                   /**< Peripheral Identification Register 2., offset: 0xFE8 */
  uint32_t PID3;                                   /**< Peripheral Identification Register 3., offset: 0xFEC */
  uint32_t CID0;                                   /**< Component Identification Register 0., offset: 0xFF0 */
  uint32_t CID1;                                   /**< Component Identification Register 1., offset: 0xFF4 */
  uint32_t CID2;                                   /**< Component Identification Register 2., offset: 0xFF8 */
  uint32_t CID3;                                   /**< Component Identification Register 3., offset: 0xFFC */
} volatile *BP_MemMapPtr;
))


\ ===
\ CMP
\ ===

$4007:3000 const _CMP0
\ $4007:3008 const _CMP1
\ $4007:3010 const _CMP2
  \ byte registers
  $00 const cmpCR0		\ CMP Control Register 0
  $01 const cmpCR1		\ CMP Control Register 1
  $02 const cmpFPR		\ CMP Filter Period Register
  $03 const cmpSCR		\ CMP Status and Control Register
  $04 const cmpDACCR		\ DAC Control Register
  $05 const cmpMUXCR		\ MUX Control Register


\ ===
\ DAC
\ ===

$4003:F000 const _DAC0
  \ byte registers
  $00 const dacDATL		\ 16 16-bit buffers
  $01 const dacDATH

  $20 const dacSR
  $21 const dacC0
  $22 const dacC1
  $23 const dacC2


\ ===
\ DMA
\ ===

$4000:8000 const _DMA0
  $0100 const dmaSAR0
  $0104 const dmaDAR0
  $0108 const dmaDSR_BCR0
  $010C const dmaDCR0

  $0110 const dmaSAR1
  $0114 const dmaDAR1
  $0118 const dmaDSR_BCR1
  $011C const dmaDCR1

  $0120 const dmaSAR2
  $0124 const dmaDAR2
  $0128 const dmaDSR_BCR2
  $012C const dmaDCR2

  $0130 const dmaSAR3
  $0134 const dmaDAR3
  $0138 const dmaDSR_BCR3
  $013C const dmaDCR3

$4002:1000 const _DMAMUX
  $00 const dmamuxCHCFG0
  $01 const dmamuxCHCFG1
  $02 const dmamuxCHCFG2
  $03 const dmamuxCHCFG3


\ ===
\ DWT
\ ===

$E000:1000 const _DWT	\ header file
\ $F000:1000 const _DWT	\ manual
  $000 const dwtCTRL

  $01C const dwtPCSR
  $020 const dwtCOMP0
  $024 const dwtMASK0
  $028 const dwtFCT0

  $030 const dwtCOMP1
  $034 const dwtMASK1
  $038 const dwtFCT1


\ ====
\ PORT - GPIO control
\ ====

$4004:9000 const _PORTA
$4004:A000 const _PORTB
$4004:B000 const _PORTC
$4004:C000 const _PORTD
$4004:D000 const _PORTE
  $000 const portPCR		\ Pin Control Register n, array offset: 0x0, array step: 0x4 */
    $000 const portPCR0
    $004 const portPCR1
    $008 const portPCR2
    $00C const portPCR3
    $010 const portPCR4
    $014 const portPCR5
    $018 const portPCR6
    $01C const portPCR7
    $020 const portPCR8
    $024 const portPCR9
    $028 const portPCR10
    $02C const portPCR11
    $030 const portPCR12
    $034 const portPCR13
    $038 const portPCR14
    $03C const portPCR15
    $040 const portPCR16
    $044 const portPCR17
    $048 const portPCR18
    $04C const portPCR19
    $050 const portPCR20
    $054 const portPCR21
    $058 const portPCR22
    $05C const portPCR23
    $060 const portPCR24
    $064 const portPCR25
    $068 const portPCR26
    $06C const portPCR27
    $070 const portPCR28
    $074 const portPCR29
    $078 const portPCR30
    $07C const portPCR31

  $080 const portGPCLR		\ Global Pin Control Low Register, offset: 0x80 */
  $084 const portGPCHR		\ Global Pin Control High Register, offset: 0x84 */

  $0A0 const portISFR		\ Interrupt Status Flag Register, offset: 0xA0 */


\ ====
\ GPIO - GPIO use
\ ====
\ Both the Freescale and the ARM/MPE names are provided.

$400F:F000 const _PTA
$400F:F040 const _PTB
$400F:F080 const _PTC
$400F:F0C0 const _PTD
$400F:F100 const _PTE
  $0000 const ptPDOR		\ Port Data Output Register, offset: 0x0 */
  $0004 const ptPSOR		\ Port Set Output Register, offset: 0x4 */
  $0008 const ptPCOR		\ Port Clear Output Register, offset: 0x8 */
  $000C const ptPTOR		\ Port Toggle Output Register, offset: 0xC */
  $0010 const ptPDIR		\ Port Data Input Register, offset: 0x10 */
  $0014 const ptPDDR		\ Port Data Direction Register, offset: 0x14 */

$400F:F000 const _GPIOA
$400F:F040 const _GPIOB
$400F:F080 const _GPIOC
$400F:F0C0 const _GPIOD
$400F:F100 const _GPIOE

$F80F:F000 const _FGPIOA	\ faster
$F80F:F040 const _FGPIOB
$F80F:F080 const _FGPIOC
$F80F:F0C0 const _FGPIOD
$F80F:F100 const _FGPIOE


\ ====
\ FTFA - Flash Memory Module
\ ====

$4002:0000 const _FTFA
  \ byte registers
  $00 const ftfaFSTAT		\ Flash Status Register, offset: 0x0 */
  $01 const ftfaFCNFG		\ Flash Configuration Register, offset: 0x1 */
  $02 const ftfaFSEC		\ Flash Security Register, offset: 0x2 */
  $03 const ftfaFOPT		\ Flash Option Register, offset: 0x3 */
  $04 const ftfaFCCOB3		\ Flash Common Command Object Registers, offset: 0x4 */
  $05 const ftfaFCCOB2		\ Flash Common Command Object Registers, offset: 0x5 */
  $06 const ftfaFCCOB1		\ Flash Common Command Object Registers, offset: 0x6 */
  $07 const ftfaFCCOB0		\ Flash Common Command Object Registers, offset: 0x7 */
  $08 const ftfaFCCOB7		\ Flash Common Command Object Registers, offset: 0x8 */
  $09 const ftfaFCCOB6		\ Flash Common Command Object Registers, offset: 0x9 */
  $0A const ftfaFCCOB5		\ Flash Common Command Object Registers, offset: 0xA */
  $0B const ftfaFCCOB4		\ Flash Common Command Object Registers, offset: 0xB */
  $0C const ftfaFCCOBB		\ Flash Common Command Object Registers, offset: 0xC */
  $0D const ftfaFCCOBA		\ Flash Common Command Object Registers, offset: 0xD */
  $0E const ftfaFCCOB9		\ Flash Common Command Object Registers, offset: 0xE */
  $0F const ftfaFCCOB8		\ Flash Common Command Object Registers, offset: 0xF */
  $10 const ftfaFPROT3		\ Program Flash Protection Registers, offset: 0x10 */
  $11 const ftfaFPROT2		\ Program Flash Protection Registers, offset: 0x11 */
  $12 const ftfaFPROT1		\ Program Flash Protection Registers, offset: 0x12 */
  $13 const ftfaFPROT0		\ Program Flash Protection Registers, offset: 0x13 */


\ ===
\ I2C
\ ===

$4006:6000 const _I2C0
$4006:7000 const _I2C1
  $00 const i2cA1		\ I2C Address Register 1, offset: 0x0 */
  $01 const i2cF		\ I2C Frequency Divider register, offset: 0x1 */
  $02 const i2cC1		\ I2C Control Register 1, offset: 0x2 */
  $03 const i2cS		\ I2C Status Register, offset: 0x3 */
  $04 const i2cD		\ I2C Data I/O register, offset: 0x4 */
  $05 const i2cC2		\ I2C Control Register 2, offset: 0x5 */
  $06 const i2cFLT		\ I2C Programmable Input Glitch Filter register, offset: 0x6 */
  $07 const i2cRA		\ I2C Range Address register, offset: 0x7 */
  $08 const i2cSMB		\ I2C SMBus Control and Status register, offset: 0x8 */
  $09 const i2cA2		\ I2C Address Register 2, offset: 0x9 */
  $0A const i2cSLTH		\ I2C SCL Low Timeout Register High, offset: 0xA */
  $0B const i2cSLTL		\ I2C SCL Low Timeout Register Low, offset: 0xB */

\ ====
\ LLWU - Low Leakage Wake Up
\ ====

$4007:C000 const _LLWU
  $00 const llwuPE1		\ LLWU Pin Enable 1 Register, offset: 0x0 */
  $01 const llwuPE2		\ LLWU Pin Enable 2 Register, offset: 0x1 */
  $02 const llwuPE3		\ LLWU Pin Enable 3 Register, offset: 0x2 */
  $03 const llwuPE4		\ LLWU Pin Enable 4 Register, offset: 0x3 */
  $04 const llwuME		\ LLWU Module Enable Register, offset: 0x4 */
  $05 const llwuF1		\ LLWU Flag 1 Register, offset: 0x5 */
  $06 const llwuF2		\ LLWU Flag 2 Register, offset: 0x6 */
  $07 const llwuF3		\ LLWU Flag 3 Register, offset: 0x7 */
  $08 const llwuFILT1		\ LLWU Pin Filter 1 register, offset: 0x8 */
  $09 const llwuFILT2		\ LLWU Pin Filter 2 register, offset: 0x9 */

\ =====
\ LPTMR - Low Power Timer
\ =====

$4004:0000 const _LPTMR0
  $00 const lptmCSR		\ Low Power Timer Control Status Register, offset: 0x0 */
  $04 const lptmPSR		\ Low Power Timer Prescale Register, offset: 0x4 */
  $08 const lptmCMR		\ Low Power Timer Compare Register, offset: 0x8 */
  $0C const lptmCNR		\ Low Power Timer Counter Register, offset: 0xC */


\ ===
\ MCG - Multipurpose Clock Generator
\ ===

$4006:4000 const _MCG
  $00 const mcgC1		\ MCG Control 1 Register, offset: 0x0 */
  $01 const mcgC2		\ MCG Control 2 Register, offset: 0x1 */
  $02 const mcgC3		\ MCG Control 3 Register, offset: 0x2 */
  $03 const mcgC4		\ MCG Control 4 Register, offset: 0x3 */
  $04 const mcgC5		\ MCG Control 5 Register, offset: 0x4 */
  $05 const mcgC6		\ MCG Control 6 Register, offset: 0x5 */
  $06 const mcgS		\ MCG Status Register, offset: 0x6 */

  $08 const mcgSC		\ MCG Status and Control Register, offset: 0x8 */

  $0A const mcgATCVH		\ MCG Auto Trim Compare Value High Register, offset: 0xA */
  $0B const mcgATCVL		\ MCG Auto Trim Compare Value Low Register, offset: 0xB */
  $0C const mcgC7		\ MCG Control 7 Register, offset: 0xC */
  $0D const mcgC8		\ MCG Control 8 Register, offset: 0xD */
  $0E const mcgC9		\ MCG Control 9 Register, offset: 0xE */
  $0F const mcgC10		\ MCG Control 10 Register, offset: 0xF */


\ ===
\ MCM - Miscellaneous Control Module
\ ===

$F000:3000 const _MCM
  $08 const mcmPLASC		\ Crossbar switch (AXBS) slave configuration, offset: 0x8 */
  $0A const mcmPLAMC		\ Crossbar switch (AXBS) master configuration, offset: 0xA */
  $0C const mcmPLACR		\ Platform Control Register, offset: 0xC */

  $40 const mcmCPO		\ Compute Operation Control Register, offset: 0x40 */


\ ===
\ MTB
\ ===

$F000:0000 const _MTB
  $0000 const mtbPOSITION	\ MTB Position Register, offset: 0x0 */
  $0004 const mtbMASTER		\ MTB Master Register, offset: 0x4 */
  $0008 const mtbFLOW		\ MTB Flow Register, offset: 0x8 */
  $000C const mtbBASE		\ MTB Base Register, offset: 0xC */

  $0F00 const mtbMODECTRL	\ Integration Mode Control Register, offset: 0xF00 */

  $0FA0 const mtbTAGSET		\ Claim TAG Set Register, offset: 0xFA0 */
  $0FA4 const mtbTAGCLEAR	\ Claim TAG Clear Register, offset: 0xFA4 */

  $0FB0 const mtbLOCKACCESS	\ Lock Access Register, offset: 0xFB0 */
  $0FB4 const mtbLOCKSTAT	\ Lock Status Register, offset: 0xFB4 */
  $0FB8 const mtbAUTHSTAT	\ Authentication Status Register, offset: 0xFB8 */
  $0FBC const mtbDEVICEARCH	\ Device Architecture Register, offset: 0xFBC */

  $0FC8 const mtbDEVICECFG	\ Device Configuration Register, offset: 0xFC8 */
  $0FCC const mtbDEVICETYPID	\ Device Type Identifier Register, offset: 0xFCC */
  $0FD0 const mtbPERIPHID4	\ Peripheral ID Register, array offset: 0xFD0, array step: 0x4 */
  $0FD4 const mtbPERIPHID5
  $0FD8 const mtbPERIPHID6
  $0FDC const mtbPERIPHID7
  $0FE0 const mtbPERIPHID0
  $0FE4 const mtbPERIPHID1
  $0FE8 const mtbPERIPHID2
  $0FEC const mtbPERIPHID3
  $0FF0 const mtbCOMPID0	\ Component ID Register, array offset: 0xFF0, array step: 0x4 */
  $0FF4 const mtbCOMPID1
  $0FF8 const mtbCOMPID2
  $0FFC const mtbCOMPID3


\ ====
\ NVIC
\ ====
\ See CM0def.fth - SCS System Control SPace


\ ===
\ OSC - Oscillator
\ ===

$4006:5000 const _OSC
  $000 const oscCR		\ OSC Control Register, offset: 0x0 */


\ ===
\ PIT - Programmable Interval Timer - not like 68k/Coldfire
\ ===

$4003:7000 const _PIT
  $0000 const pitMCR		\ PIT Module Control Register, offset: 0x0 */

  $00E0 const pitLTMR64H	\ PIT Upper Lifetime Timer Register, offset: 0xE0 */
  $00E4 const pitLTMR64L	\ PIT Lower Lifetime Timer Register, offset: 0xE4 */

  $0100 const pitLDVAL0		\ Timer Load Value Register, array offset: 0x100, array step: 0x10 */
  $0104 const pitCVAL0		\ Current Timer Value Register, array offset: 0x104, array step: 0x10 */
  $0108 const pitTCTRL0		\ Timer Control Register, array offset: 0x108, array step: 0x10 */
  $010C const pitTFLG0		\ Timer Flag Register, array offset: 0x10C, array step: 0x10 */
  $0110 const pitLDVAL1
  $0114 const pitCVAL1
  $0118 const pitTCTRL
  $011C const pitTFLG1
((
  $0120 const pitLDVAL2
  $0124 const pitCVAL2
  $0128 const pitTCTRL2
  $012C const pitTFLG2
  $0130 const pitLDVAL3
  $0134 const pitCVAL3
  $0138 const pitTCTRL3
  $013C const pitTFLG3
))


\ ===
\ PMC - Power Management Controller
\ ===

$4007:D000 const _PMC
  $00 const pmcLVDSC1		\ Low Voltage Detect Status and Control 1 Register, offset: 0x0 */
  $01 const pmcLVDSC2		\ Low Voltage Detect Status and Control 2 Register, offset: 0x1 */
  $02 const pmcREGSC		\ Regulator Status and Control Register, offset: 0x2 */



\ ==
\ RCM - Reset Control Module
\ ==

$4007:F000 const _RMC
  $00 const rcmSRS0		\ System Reset Status Register 0, offset: 0x0 */
  $01 const rcmSRS1		\ System Reset Status Register 1, offset: 0x1 */

  $04 const rcmRPFC		\ Reset Pin Filter Control register, offset: 0x4 */
  $05 const rcmRPFW		\ Reset Pin Filter Width register, offset: 0x5 */


\ ===
\ ROM
\ ===

$F000:2000 const _ROM
  $0000 const romENTRY0		\ Entry, array offset: 0x0, array step: 0x4 */
  $0004 const romENTRY1
  $0008 const romENTRY2
  $0000 const romTABLEMARK	\ End of Table Marker Register, offset: 0xC */

  $0FCC const romSYSACCESS	\ System Access Register, offset: 0xFCC */
  $0FD0 const romPERIPHID4	\ Peripheral ID Register, offset: 0xFD0 */
  $0FD4 const romPERIPHID5	\ Peripheral ID Register, offset: 0xFD4 */
  $0FD8 const romPERIPHID6	\ Peripheral ID Register, offset: 0xFD8 */
  $0FDC const romPERIPHID7	\ Peripheral ID Register, offset: 0xFDC */
  $0FE0 const romPERIPHID0	\ Peripheral ID Register, offset: 0xFE0 */
  $0FE4 const romPERIPHID1	\ Peripheral ID Register, offset: 0xFE4 */
  $0FE8 const romPERIPHID2	\ Peripheral ID Register, offset: 0xFE8 */
  $0FEC const romPERIPHID3	\ Peripheral ID Register, offset: 0xFEC */
  $0FF0 const romCOMPID0	\ Component ID Register, array offset: 0xFF0, array step: 0x4 */
  $0FF4 const romCOMPID1
  $0FF8 const romCOMPID2
  $0FFC const romCOMPID3


\ ===
\ RTC - Real Time Clock
\ ===

$4003:D000 const _RTC
  $000 const rtcTSR		\ RTC Time Seconds Register, offset: 0x0 */
  $004 const rtcTPR		\ RTC Time Prescaler Register, offset: 0x4 */
  $008 const rtcTAR		\ RTC Time Alarm Register, offset: 0x8 */
  $00C const rtcTCR		\ RTC Time Compensation Register, offset: 0xC */
  $010 const rtcCR		\ RTC Control Register, offset: 0x10 */
  $014 const rtcSR		\ RTC Status Register, offset: 0x14 */
  $018 const rtcLR		\ RTC Lock Register, offset: 0x18 */
  $01C const rtcCCR		\ RTC Chip Configuration Register, offset: 0x1C */


\ ===
\ SCB
\ ===
\ See CM0def.fth


\ ===
\ SIM - System Integration Module
\ ===

$4004:7000 const _SIM
  $0000 const simSOPT1		\ System Options Register 1, offset: 0x0 */
  $0004 const simSOPT1CFG	\ SOPT1 Configuration Register, offset: 0x4 */

  $1004 const simSOPT2		\ System Options Register 2, offset: 0x1004 */

  $100C const simSOPT4		\ System Options Register 4, offset: 0x100C */
  $1010 const simSOPT5		\ System Options Register 5, offset: 0x1010 */
\  $1014 const simSOPT6		\ System Options Register 6, offset: 0x1014 */
  $1018 const simSOPT7		\ System Options Register 7, offset: 0x1018 */

  $1024 const simSDID		\ System Device Identification Register, offset: 0x1024 */
\ $1028 const simSCGC1		\ System Clock Gating Control Register 1, offset: 0x1028 */
\  $102C const simSCGC2		\ System Clock Gating Control Register 2, offset: 0x102C */
\  $1030 const simSCGC3		\ System Clock Gating Control Register 3, offset: 0x1030 */
  $1034 const simSCGC4		\ System Clock Gating Control Register 4, offset: 0x1034 */
  $1038 const simSCGC5		\ System Clock Gating Control Register 5, offset: 0x1038 */
  $103C const simSCGC6		\ System Clock Gating Control Register 6, offset: 0x103C */
  $1040 const simSCGC7		\ System Clock Gating Control Register 7, offset: 0x1040 */
  $1044 const simCLKDIV1	\ System Clock Divider Register 1, offset: 0x1044 */
\  $1048 const simCLKDIV2	\ System Clock Divider Register 2, offset: 0x1048 */
  $104C const simFCFG1		\ Flash Configuration Register 1, offset: 0x104C */
  $1050 const simFCFG2		\ Flash Configuration Register 2, offset: 0x1050 */
  $1054 const simUIDH		\ Unique Identification Register High, offset: 0x1054 */
  $1058 const simUIDMH		\ Unique Identification Register Mid-High, offset: 0x1058 */
  $105C const simUIDML		\ Unique Identification Register Mid Low, offset: 0x105C */
  $1060 const simUIDL		\ Unique Identification Register Low, offset: 0x1060 */

  $1100 const simCOPC		\ COP Control Register, offset: 0x1100 */
  $1104 const simSRVCOP		\ Service COP Register, offset: 0x1104 */


\ ===
\ SMC - System Mode COntrol
\ ===

$4007:E000 const _SMC
  $00 const smcPMPROT		\ Power Mode Protection register, offset: 0x0 */
  $01 const smcPMCTRL		\ Power Mode Control register, offset: 0x1 */
  $02 const smcSTOPCTRL		\ Stop Control Register, offset: 0x2 */
  $03 const smcPMSTAT		\ Power Mode Status register, offset: 0x3 */


\ ===
\ SPI
\ ===

$4007:6000 const _SPI0
$4007:7000 const _SPI1
  $00 const spiC1		\ SPI control register 1, offset: 0x0 */
  $01 const spiC2		\ SPI control register 2, offset: 0x1 */
  $02 const spiBR		\ SPI baud rate register, offset: 0x2 */
  $03 const spiS		\ SPI status register, offset: 0x3 */

  $05 const spiD		\ SPI data register, offset: 0x5 */

  $07 const spiM		\ SPI match register, offset: 0x7 */


\ =======
\ SysTick
\ =======
\ See CM0def.fth


\ =======================================
\ TPM/FTM - Timer/PWM - Flex Timer Module
\ =======================================

$4003:8000 const _FTM0
$4003:9000 const _FTM1
$4003:A000 const _FTM2
  $000 const ftmSC		\ Status and Control, offset: 0x0 */
  $004 const ftmCNT		\ Counter, offset: 0x4 */
  $008 const ftmMOD		\ Modulo, offset: 0x8 */
  $00C const ftmC0SC		\ Channel (n) Status and Control, array offset: 0xC, array step: 0x8 */
  $010 const ftmC0V		\ Channel (n) Value, array offset: 0x10, array step: 0x8 */
  $014 const ftmC1SC
  $018 const ftmC1V
  $01C const ftmC2SC
  $020 const ftmC2V
  $024 const ftmC3SC
  $028 const ftmC3V
  $02C const ftmC4SC
  $030 const ftmC4V
  $034 const ftmC5SC
  $038 const ftmC5V

  $050 const ftmSTATUS		\ Capture and Compare Status, offset: 0x50 */

  $084 const ftmCONF		\ Configuration, offset: 0x84 */


$4003:8000 const _TPM0
$4003:9000 const _TPM1
$4003:A000 const _TPM2
  $000 const tpmSC		\ Status and Control, offset: 0x0 */
  $004 const tpmCNT		\ Counter, offset: 0x4 */
  $008 const tpmMOD		\ Modulo, offset: 0x8 */
  $00C const tpmC0SC		\ Channel (n) Status and Control, array offset: 0xC, array step: 0x8 */
  $010 const tpmC0V		\ Channel (n) Value, array offset: 0x10, array step: 0x8 */
  $014 const tpmC1SC
  $018 const tpmC1V
  $01C const tpmC2SC
  $020 const tpmC2V
  $024 const tpmC3SC
  $028 const tpmC3V
  $02C const tpmC4SC
  $030 const tpmC4V
  $034 const tpmC5SC
  $038 const tpmC5V

  $050 const tpmSTATUS		\ Capture and Compare Status, offset: 0x50 */

  $084 const tpmCONF		\ Configuration, offset: 0x84 */


\ ===
\ TSI - Touch Sense Input
\ ===

$4004:5000 const _TSI0
  $000 const tsiGENCS		\ General Control and Status Register, offset: 0x0 */
  $004 const tsiDATA		\ TSI DATA Register, offset: 0x4 */
  $008 const tsiTSHD		\ TSI Threshold Register, offset: 0x8 */


\ ====
\ UART
\ ====

$4006:A000 const _UART0
$4006:B000 const _UART1
$4006:C000 const _UART2
  $000 const UnBDH		\ UART Baud Rate Registers:High, offset: 0x0 */
  $001 const UnBDL		\ UART Baud Rate Registers: Low, offset: 0x1 */
  $002 const UnC1		\ UART Control Register 1, offset: 0x2 */
  $003 const UnC2		\ UART Control Register 2, offset: 0x3 */
  $004 const UnS1		\ UART Status Register 1, offset: 0x4 */
  $005 const UnS2		\ UART Status Register 2, offset: 0x5 */
  $006 const UnC3		\ UART Control Register 3, offset: 0x6 */
  $007 const UnD		\ UART Data Register, offset: 0x7 */
  $008 const UnC4		\ UART Control Register 4, offset: 0x8 */

  $008 const U0MA1		\ UART Match Address Registers 1, offset: 0x8 */
  $009 const U0MA2		\ UART Match Address Registers 2, offset: 0x9 */
  $00A const U0C4		\ UART Control Register 4, offset: 0xA */
  $00B const U0C5		\ UART Control Register 5, offset: 0xB */


\ =======
\ USB OTG
\ =======

$4007:2000 const _USB0
  $0000 const usbPERID		\ Peripheral ID Register, offset: 0x0 */
  $0004 const usbIDCOMP		\ Peripheral ID Complement Register, offset: 0x4 */
  $0008 const usbREV		\ Peripheral Revision Register, offset: 0x8 */
  $000C const usbADDINFO	\ Peripheral Additional Info Register, offset: 0xC */
  $0010 const usbOTGISTAT	\ OTG Interrupt Status Register, offset: 0x10 */
  $0014 const usbOTGICR		\ OTG Interrupt Control Register, offset: 0x14 */
  $0018 const usbOTGSTAT	\ OTG Status Register, offset: 0x18 */
  $001C const usbOTGCTL		\ OTG Control Register, offset: 0x1C */

  $0080 const usbISTAT		\ Interrupt Status Register, offset: 0x80 */
  $0084 const usbINTEN		\ Interrupt Enable Register, offset: 0x84 */
  $0088 const usbERRSTAT	\ Error Interrupt Status Register, offset: 0x88 */
  $008C const usbERREN		\ Error Interrupt Enable Register, offset: 0x8C */
  $0090 const usbSTAT		\ Status Register, offset: 0x90 */
  $0094 const usbCTL		\ Control Register, offset: 0x94 */
  $0098 const usbADDR		\ Address Register, offset: 0x98 */
  $009C const usbBDTPAGE1	\ BDT Page Register 1, offset: 0x9C */
  $00A0 const usbFRMNUML	\ Frame Number Register Low, offset: 0xA0 */
  $00A4 const usbFRMNUMH	\ Frame Number Register High, offset: 0xA4 */
  $00A8 const usbTOKEN		\ Token Register, offset: 0xA8 */
  $00AC const usbSOFTHLD	\ SOF Threshold Register, offset: 0xAC */
  $00B0 const usbBDTPAGE2	\ BDT Page Register 2, offset: 0xB0 */
  $00B4 const usbBDTPAGE3	\ BDT Page Register 3, offset: 0xB4 */

  $00C0 const usbENDPTs		\ Endpoint Control Register, array offset: 0xC0, array step: 0x4 */
    $00C0 const usbENDPT0
    $00C4 const usbENDPT1
    $00C8 const usbENDPT2
    $00CC const usbENDPT3
    $00D0 const usbENDPT4
    $00D4 const usbENDPT5
    $00D8 const usbENDPT6
    $00DC const usbENDPT7
    $00E0 const usbENDPT8
    $00E4 const usbENDPT9
    $00E8 const usbENDPT10
    $00EC const usbENDPT11
    $00F0 const usbENDPT12
    $00F4 const usbENDPT13
    $00F8 const usbENDPT14
    $00FC const usbENDPT15
  $0100 const usbUSBCTRL	\ USB Control Register, offset: 0x100 */
  $0104 const usbOBSERVE	\ USB OTG Observe Register, offset: 0x104 */
  $0108 const usbCONTROL	\ USB OTG Control Register, offset: 0x108 */
  $010C const usbUSBTRC0	\ USB Transceiver Control Register 0, offset: 0x10C */

  $0114 const USBFRMADJUST	\ Frame Adjust Register, offset: 0x114 */



\ ======
\ *> ###
\ ======

decimal

