((
Compiler: MPE Forth 6 ARM
Autor: Alberto Pasquale

LPC2106 Modbus Driver
The driver supports the most commun Modbus functions used.
   1  Read Coil  
   2  Read Digital Status Inputs
   3  Read Holding Register 
   4  Read Input Register
   5  Set  Coil 
   6  Set Holding Register
   8  Loop Back Diagnostics
  15  Set Multiple Coils
  16  Set Multiple Registers 
The implementation does not follow the standard 3 1/2 byte transmition timeou to detect the frame
In real life most timeouts work just fine
To use it on a multidrop line RS485, line control needs to be added at the begining and end of the 
SEND word
))
\ Modbus UART1

\ Dumy variables, replace with variables used in the application
: CVARIABLE [COMPILE] VARIABLE -3 ALLOT ;
CVARIABLE <STATUS1>
CVARIABLE <STATUS2> 
VARIABLE VAR1
VARIABLE VAR2
VARIABLE VAR3
VARIABLE VAR4
\ =================================================================================
VARIABLE <MB>		\ STATE
VARIABLE <MB.ID>	\ ID for com3 or com4
VARIABLE START.REGISTER	\ start register on the querry
VARIABLE NUMBER.OFF	\ number of registers requested
VARIABLE LENGTH		\ size of the incoming querry
VARIABLE RX{}		\ count for RX TX
VARIABLE TX{}
300 BUFFER: MBRXBUFFER	\ transmit and receive buffers 
300 BUFFER: MBTXBUFFER	

\ =================================================================================
\ Register tables includes a scaling fraction to adjust numbers to 16 bit integers
\ address  factor divisor  ( 0 0 indicates a byte variable
CREATE MB.I.TBL

VAR1 , 1 , 10 ,		\ Divide by 10
VAR2 , 1 , 1 ,
VAR3 , 1 , 1 ,
VAR4 , 1 , 1 ,

4 EQU MAX.I.REGISTER

\ ----------------------------------------------------------------------------------
CREATE MB.H.TBL

VAR1 , 1 , 10 ,		\ Divide by 10
VAR2 , 1 , 1 ,
VAR3 , 1 , 1 ,
VAR4 , 1 , 1 ,

4 EQU MAX.H.REGISTER
\ ----------------------------------------------------------------------------------
\ Bit Tables  byte address , mask 
CREATE COIL.TBL
<STATUS1> , $01 ,
<STATUS1> , $02 ,
<STATUS1> , $04 ,
<STATUS1> , $08 ,
<STATUS1> , $10 ,
<STATUS1> , $20 ,
<STATUS1> , $40 ,
<STATUS1> , $80 ,
<STATUS2> , $01 ,
<STATUS2> , $02 ,
<STATUS2> , $04 ,
<STATUS2> , $08 ,
<STATUS2> , $10 ,
<STATUS2> , $20 ,
<STATUS2> , $40 ,
<STATUS2> , $80 ,

16 EQU MAX.COIL

CREATE DIN.TBL
<STATUS1> , $01 ,
<STATUS1> , $02 ,
<STATUS1> , $04 ,
<STATUS1> , $08 ,
<STATUS1> , $10 ,
<STATUS1> , $20 ,
<STATUS1> , $40 ,
<STATUS1> , $80 ,
<STATUS2> , $01 ,
<STATUS2> , $02 ,
<STATUS2> , $04 ,
<STATUS2> , $08 ,
<STATUS2> , $10 ,
<STATUS2> , $20 ,
<STATUS2> , $40 ,
<STATUS2> , $80 ,

16 EQU MAX.DIN

\ ====================================================================================== 
\ Little endian to big endian & misaligned words
\ Modbus buffers have words not aligned in even addresses, and also requires big endian 
\ format.
\ CRC16 with a seed of $FFFF needs to be stored in little endian format, so that the 
\ crc of the buffer + the crc is allways zero, but is not allways align on even bytes

: bW! ( w addr -- ) OVER 8 RSHIFT OVER 1+ C! C! ;
: ><W@ ( addr -- w ) COUNT 8 LSHIFT SWAP C@ OR ;
: ><W! ( w addr -- ) OVER 8 RSHIFT OVER C! 1+ C! ;

\ =================================================================================
\ CRC16 Extracted from MPE source files
: +CRC16 ( crc b -- crc' )
  8 0 DO
    2DUP XOR  1 AND 
    IF SWAP 1 RSHIFT $A001 XOR SWAP ELSE SWAP 1 RSHIFT SWAP THEN 
    1 RSHIFT 
    LOOP DROP
;

: CRC16	( addr len -- crc' )
  $FFFF -ROT BOUNDS ?DO  I C@ +CRC16  LOOP $FFFF AND
;
\ Store the crc at the end of the buffer
: CRC16! ( addr len -- addr len+2 )
  2DUP >R >R 2+ R> R> 2DUP + >R  CRC16  R>   bW! ;
  

\ ========================================================================================
\ Buffer operators

\ Store a byte in the receive buffer and increment pointer
: RXC! ( c -- ) RX{} @ DUP 1+ RX{} ! MBRXBUFFER  + C!  ;

\ Retrieve an indexed byte from the receive buffer
: RXC@ ( index -- byte ) PAUSE MBRXBUFFER  + C@ ;

\ Retrieve an indexed word from the receive buffer
: RXW@ ( index -- word ) PAUSE MBRXBUFFER  + ><W@ ;


\ store a byte in the transmit buffer and increment the pointer
: TXC! ( c -- ) PAUSE MBTXBUFFER TX{} @ + C! 1 TX{} +! ;

\ store a word in the transmit buffer and increment pointer
: TXW! ( c -- ) PAUSE MBTXBUFFER  TX{} @ + ><W! 2 TX{} +! ;


\ Set index pointers to zero
: ZERO.RX ( -- ) 0 RX{} ! ;

: ZERO.TX ( -- ) 0 TX{} ! ;

\ get the last byte stored in the receive buffer
: LAST.BYTE ( -- c ) RX{} @ 1- RXC@ ;

\ CRC the transmit buffer and append the CRC
: CRCTX (  -- addr count+2 ) MBTXBUFFER  TX{} @ CRC16! ;

\ ====================================================================================================
\ Register operators 
\ The tables have a pointer to variables and the numerator and denominator 
\ of a scaling fraction, so 32 bit variables can be scaled for the 16 bit Modbus format 
\ If the fraction is 0 0, the variable is treated as a  byte

: MB.IREG@ ( index -- n ) 
  12 * MB.I.TBL + DUP 4+ @ 
  IF DUP @ @ SWAP 4 + 2@ SWAP */  ABS $FFFF MIN 	\ force it to an unsigned 16 bits
  ELSE @ C@ THEN ;


: MB.IREG! ( n index -- )
  12 * MB.I.TBL +  DUP @ >R 
  4+ DUP @  IF 2@ */ R> ! ELSE DROP R> C! THEN ;


: MB.HREG@ ( index -- n ) 
  12 * MB.H.TBL + DUP 4+ @ 
  IF DUP @ @ SWAP 4 + 2@ SWAP */ $FFFF MIN 
  ELSE @ C@ THEN ;


: MB.HREG! ( n index -- ) 
  12 * MB.H.TBL +  DUP @ >R 
  4+ DUP @  IF 2@ */ R> ! ELSE DROP R> C! THEN ;

\ bit tables have an address and a mask

: MB.COIL@ ( index -- flag ) 3 <<n COIL.TBL + 2@ TEST-BIT 0= 0= ;
: MB.COIL! ( flag index -- ) 3 <<n COIL.TBL + 2@  ROT IF SET-BIT ELSE RESET-BIT THEN ; 

: MB.DIN@ ( index -- flag ) 3 <<n DIN.TBL + 2@ TEST-BIT 0= 0= ; 
: MB.DIN! ( flag index -- ) 3 <<n DIN.TBL + 2@  ROT IF SET-BIT ELSE RESET-BIT THEN ;

  
\ Catch the wrong querrys c=illegal code
: ILLEGAL.RESPONSE ( c -- )
   0 RXC@ TXC! 1 RXC@ $80 OR TXC! TXC!  			\ function with the high bit set, code
;

\ Read Coils 
: READ.COIL ( -- )
  2 RXW@ START.REGISTER ! 4 RXW@ NUMBER.OFF !				\ load variables
  START.REGISTER @ NUMBER.OFF @ + MAX.COIL >				\ check range
  IF 2 ILLEGAL.RESPONSE  EXIT THEN 					\ out of range
  0 RXW@ TXW!								\ copy ID and function
  NUMBER.OFF @ 8 /MOD SWAP IF 1+ THEN DUP TXC!				\ byte count
  0 DO									\ for all bytes needed
    0 8 0 DO 2/  							\ do 8 bits for each
       NUMBER.OFF @ IF START.REGISTER @ MB.COIL@ IF $80 OR THEN THEN 	\ if withing range, if not, padd with 0
       NUMBER.OFF @ 1- 0 MAX NUMBER.OFF !				\ keep count
       1 START.REGISTER +!
       LOOP TXC!							\ store each byte
  LOOP    
;

\ Same thing for digital 
: READ.DIN ( -- )
  2 RXW@ START.REGISTER ! 4 RXW@ NUMBER.OFF !
  START.REGISTER @ NUMBER.OFF @ + MAX.DIN >
  IF 2 ILLEGAL.RESPONSE  EXIT THEN 
  0 RXW@ TXW!
  NUMBER.OFF @ 8 /MOD SWAP IF 1+ THEN DUP TXC!				\ byte count
  0 DO
    0  8 0 DO 2/  
       NUMBER.OFF @ IF START.REGISTER @ MB.DIN@ IF $80 OR THEN THEN 
       NUMBER.OFF @ 1- 0 MAX NUMBER.OFF ! 1 START.REGISTER +!
       LOOP TXC!
  LOOP    
;

\ Read holding registers
: READ.H.REGISTER ( -- )
  2 RXW@ 4 RXW@ 2DUP + MAX.H.REGISTER > IF 2DROP 2 ILLEGAL.RESPONSE  EXIT THEN \ range checking
  0 RXW@  TXW! DUP  2* TXC!						\ byte count
  BOUNDS DO I MB.HREG@ TXW! LOOP					
 ;

\ Read input registers
: READ.I.REGISTER ( -- )
  2 RXW@ 4 RXW@ 2DUP + MAX.I.REGISTER > IF 2DROP 2 ILLEGAL.RESPONSE  EXIT THEN \ range checking
  0 RXW@  TXW! DUP  2* TXC!						\ byte count
  BOUNDS DO I MB.IREG@ TXW! LOOP
;

\ ===============================================================================================================

\ Transfer the first 6 bytes from the querry to the response
: COPY.6 ( -- )
  MBRXBUFFER  MBTXBUFFER  6 CMOVE 6 TX{} ! ;

: LOOP.BACK ( -- ) COPY.6 ;
  
\ Set one coil
: SET.COIL ( -- )
  2 RXW@ MAX.COIL 1- > IF 2 ILLEGAL.RESPONSE  EXIT THEN 
  4 RXW@ 2 RXW@ MB.COIL! COPY.6
;

\ Set a single holding register
: SET.H.REGISTER ( -- )
  2 RXW@ MAX.H.REGISTER 1- > IF 2 ILLEGAL.RESPONSE  EXIT THEN 
  4 RXW@ 2 RXW@ MB.HREG! COPY.6
;

\ Set multiple coils
: SET.MCOILS ( -- ) 
  2 RXW@ DUP START.REGISTER ! 4 RXW@ DUP NUMBER.OFF !
  + MAX.COIL > IF  2 ILLEGAL.RESPONSE  EXIT THEN
  6 RXC@ 0 DO 
    7 I + RXC@ 
        8 0 DO 
            NUMBER.OFF @ IF DUP 1 AND START.REGISTER @ MB.COIL! THEN 2/
	    NUMBER.OFF @ 1- 0 MAX NUMBER.OFF ! 1 START.REGISTER +!
	LOOP DROP
   LOOP COPY.6	
;

\ Set multiple registers
: SET.MREGISTERS ( -- )
  2 RXW@ DUP START.REGISTER ! 4 RXW@ DUP NUMBER.OFF !
  + MAX.H.REGISTER > IF  2 ILLEGAL.RESPONSE  EXIT THEN
  NUMBER.OFF @ 0 DO
  I 2* 7 + RXW@ START.REGISTER @ I + MB.HREG! LOOP 
  COPY.6
;




\ DO the work here and send responces
: PROCES.FUNCTION ( -- )
  ZERO.TX 						\ start with a clean buffer
  MBRXBUFFER RX{} @  CRC16  IF  ZERO.RX EXIT THEN 	\ bad crc, nothing to do 
  1 RXC@ 						\ get the function and process it
  CASE
   1 OF READ.COIL  ENDOF
   2 OF READ.DIN  ENDOF
   3 OF READ.H.REGISTER ENDOF
   4 OF READ.I.REGISTER ENDOF
   5 OF SET.COIL ENDOF
   6 OF SET.H.REGISTER ENDOF
   8 OF LOOP.BACK ENDOF
  15 OF SET.MCOILS ENDOF
  16 OF SET.MREGISTERS ENDOF
  ENDCASE
  ;

\ send a response
: MB.RESPONSE ( -- ) 
  0 RXC@ 0= TX{} @ 0= OR IF EXIT THEN  CRCTX TYPE ;	\ if it was a broadcast, or nothing to said, exit

  


\ ==============================================================================================================
\ States to get the frame

\ If it is our ID or 0, go next state else reset for begining of frame
: GET.ID (  -- ) 
  LAST.BYTE  DUP <MB.ID> @ = SWAP 0 = OR			
  IF 2 <MB> !  ELSE  ZERO.RX 0 <MB> ! THEN 
;

\ functions 1 thru 6 are fix length, go wait for the end (added function 8 and 17 )
\ 15 and 16 have the byte count in byte 7
: GET.FUNCTION (  -- )
  LAST.BYTE  
  DUP 1 8 WITHIN? IF DROP 4 <MB> !  8 LENGTH ! EXIT THEN 
  DUP 15 16 WITHIN? IF DROP 3 <MB> ! 7  LENGTH ! EXIT THEN  
  17 = IF 4 <MB> ! 4 LENGTH ! EXIT THEN 
  0 <MB> ! 
 ;

\ if it is the byte count, add 9 for the overhead and go wait for them
: GET.LENGTH (  -- )
  RX{} @ LENGTH @ = 
  IF   LAST.BYTE 9 + LENGTH ! 4 <MB> ! THEN 
;

\ if at the end, go answer them
: WAIT.LENGTH ( -- )
  RX{} @ LENGTH @ = IF  5 <MB> ! THEN 
;

\ Modbus loop
\ On a timer gap, reset everithing to begining of frame

CREATE MB.EXECUTE.TBL
' ZERO.RX ,
' GET.ID ,
' GET.FUNCTION ,
' GET.LENGTH ,
' WAIT.LENGTH ,

: MB.EXECUTE ( -- ) <MB> @ CELLS MB.EXECUTE.TBL + @ EXECUTE ; 

: SLAVE.MODBUS ( -- )
   1 <MB> ! ZERO.RX 
   100 LATER						\ initial long gap
  BEGIN 						\ task loop
     BEGIN PAUSE KEY? UNTIL 				\ wait til byte received
     EXPIRED IF 1 <MB> !  ZERO.RX  THEN			\ if time gap, reset to begining of frame
     KEY  RXC! 						\ store the byte
     MB.EXECUTE
     <MB> @ 5 = IF					\ final state, go to work  
       PROCES.FUNCTION MB.RESPONSE 
       0 <MB> ! ZERO.RX  
     THEN 2 LATER					\ time gap
  AGAIN	
;

: CONSOLE! ( n -- ) DUP IPVEC ! OPVEC ! ;

: RUN.MODBUS.TASK ( -- )
  MODBUS.TASK START: CONSOLE1 CONSOLE! 1 <MB.ID> ! 
  BEGIN SLAVE.MODBUS AGAIN ;
  
    