\ Lunar Landing Simulation - Originally by Eric Peters
\ This version adapted from Ahl, BASIC Computer Games
\ Warning:  not Y2K tested.  In fact, not very much checked at all.

: -n?  ( -- flag )  KEY DUP [CHAR] n =  SWAP [CHAR] N = OR 0= ;

: announce  ( -- flag )
   PAGE ." Rocket: Lunar Landing Simulation"
     CR ." ------- ----- ------- ----------" ;

: instructions  ( flag -- )
   CR ." Do you want instructions (y/n)?" -n?  PAGE
   IF
   CR ." You are landing on the moon and have taken over manual"
   CR ." control 500 feet above a good landing spot.  You have a"
   CR ." downward velocity of 50 ft/sec.  120 units of fuel remain."
   CR
   CR ." Here are the rules that govern your space vehicle:"
   CR ." (1) After each second, the height, velocity, and remaining"
   CR ."     fuel will be reported."
   CR ." (2) After the report, a '?' will be typed.  Enter the"
   CR ."     number of units of fuel you wish to burn during the"
   CR ."     next second.  Each unit of fuel will slow your descent"
   CR ."     by 1 ft/sec."
   CR ." (3) The maximum thrust of your engine is 30 ft/sec/sec or"
   CR ."     30 units of fuel per second."
   CR ." (4) When you contact the lunar surface, your descent engine"
   CR ."     will automatically be cut off and you will be given a"
   CR ."     report of your landing speed and remaining fuel."
   CR ." (5) If you run out of fuel, the '?' will no longer appear,"
   CR ."     but your second-by-second report will continue until"
   CR ."     you contact the lunar surface."
   CR
   THEN ;

: luck  ( -- )
   CR ." Beginning landing procedure....  G o o d   L u c k ! ! !"
   CR
   CR ." Sec Feet Spd Fuel Plot of distance"
   CR ;

VARIABLE time
VARIABLE height
VARIABLE velocity
VARIABLE fuel
VARIABLE units

 5 CONSTANT moon  \ velocity increase due to moon's gravity
30 CONSTANT max-units  \ of fuel for each input

: init ( -- )
     0 time !
  5000 height !  ( 10ths of feet)
    50 velocity !
   120 fuel ! ;

\ Report
\ String representation of n to so many decimal places
: (.d)  ( n places -- str len )
   >R DUP ABS
   0 <# R> 0 ?DO # LOOP [CHAR] . HOLD #S ROT SIGN #> ;

: .0R  ( n places width -- )
    >R  (.d)  R> OVER - 0 MAX SPACES TYPE ;

: report  ( -- )
   CR time @ 2 .r  height @ 1 6 .0r  velocity @ 4 .r
      fuel @ 0 MAX 4 .r
   ."  I" height @ 130 / SPACES [CHAR] * EMIT ;

\ Command
\ Set throttle
: gets  ( - pad u )  PAD DUP 84 ACCEPT  -TRAILING ;
: getd  ( -- d f )  0. gets >NUMBER  NIP 0= ;
: geti  ( -- n )  getd DROP  D>S ;
: clamp  ( n low high -- n' )  ROT MIN MAX ;
: get-units  ( -- n )
   0 units !
   fuel @ 0>
   IF  ."   ? "
      geti  0 fuel @ clamp  max-units MIN DUP units !
      ." =" DUP .
   ELSE  -1 fuel +!  1000 MS 0  THEN ;

\ Result
: result  ( fuel -- vel )
   NEGATE DUP fuel +!
   velocity @ DUP >R + moon + DUP R> + 10 2 */  NEGATE height +! ;

: ?fuel  ( -- )  fuel @ 0= IF  CR ." **** OUT OF FUEL ****"  THEN ;

: descent  ( -- prev-height )
   report
   BEGIN  height @  get-units result height @ 0> WHILE
     NIP  velocity !  1 time +!  ?fuel  report
   REPEAT  DROP ;

: good  ( -- )
   CR ." Congratulations!  A perfect landing!"
   CR ." Your license will be renewed.......later." ;

: ok  ( -- )
   CR ." An ok landing - you survived." ;

: bad  ( -- )
   CR ." ***** Sorry, but you blew it!!!!"
   CR ." Appropriate condolences will be sent to your next of kin." ;

\ Seconds to touchdown
\ Square of n
: sqr  ( n -- n^2 )  DUP * ;

\ square root after Wil Baden
: sqrt   ( n1 -- n2 )  0 TUCK  ?DO  1+  DUP 2* 1+  +LOOP ;

\ Common after Andrew Haley
: common  ( a b c -- q )
   SWAP >R
   * -4 * R@ sqr + sqrt
   R@ 0< IF NEGATE THEN
   R> + NEGATE 2/ ;

\ Roots to 2 places
: roots  ( a b c -- r1 r2 )
   DUP  3 PICK  2>R  common
   DUP 100 R> */  SWAP R> 100 ROT */ ;

\ With acceleration
: s.ec1  ( a b c -- secs )  roots NIP ;

\ Without acceleration
: s.ec2  ( d -- secs )  10 velocity @ */ ;

: touchdown  ( prev-height -- sec )
   CR  ." *** Contact *** "
   CR ." Touchdown at "
   >R  units @ 5 <>
   IF  moon units @ - 10 2 */ velocity @ 10 * R> NEGATE s.ec1
   ELSE  R> s.ec2  THEN
   time @ 100 * OVER +  2 0 .0r  ."  seconds" ;

: last  ( sec -- vel )
   CR ." Landing velocity = "
   moon units @ - *  velocity @ 100 * +
   DUP 2 0 .0r ."  feet/second"
   fuel @ 0 MAX fuel !  CR fuel ? ." units of fuel remaining."
   100 / ;

: rites ( vel -- )
   DUP    0= IF good ELSE
   DUP   2 < IF ok   ELSE
                bad  THEN THEN
   DROP ;

: -another  ( -- flag )
   CR CR ." Another mission?" -n? 0= ;

: rocket  ( -- )  announce  instructions
   BEGIN  init luck descent touchdown last rites  -another UNTIL ;

rocket

((
Leo Wong  hello@albany.net
http://www.albany.net/~hello/
))
