\ DOUBLES.FTH - Double number support by MicroProcessor Engineering

((
Copyright (c) 1990, 1991, 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050407 MPE001 Split into 16 and 32 bit versions.
		The 32 bit version is the master.
))

decimal

\ ==========
\ *! doubles
\ *T Double and quad numbers
\ ==========
\ *P The files *\i{Examples\Doubles.16.fth} and *\i{Examples\Doubles.32.fth}
\ ** implement double and some quad precision number support for 16
\ ** and 32 bit systems using Forth primitives and high level definitions.
\ ** To obtain better performance some definitions should be coded.
\ ** These are indicated in the source code. Alternative versions
\ ** of this file are implemented as *\i{DOUBLES.XXX.FTH} where XXX
\ ** is the processor code.

\ *P The code in these files can be used with the MPE Forth 4/5/6
\ ** and VFX cross compilers. Some minor alterations may be rquired
\ ** for *\fo{TSUMH} in the version 5.xx compilers.


hex

compiler
: u2/           \ u -- u/2
\ U2/ is not needed if D2/ and uD2/ are coded
  1 rshift
;
target

: d2*           \ d -- d*2
  2dup d+
;

[undefined] d2/ [if]	\ code this
: d2/           \ d -- d/2
  dup >r  2/
  swap  u2/  r> 1 and
  if  08000 or  endif
  swap
;
[then]

[undefined] ud2/ [if]	\ code this
: uD2/          \ ud -- ud/2
  dup >r  u2/
  swap  u2/  r> 1 and
  if  08000 or  endif
  swap
;
[then]

: d0=           \ d -- t/f
  or 0=
;

: d0<>          \ d -- t/f
  or 0<>
;

: D0<           \ d -- t/f
  nip 0<
;

: D=            \ d1 d2 -- t/f
  d- d0=
;

: D<            \ d1 d2 -- t/f
  rot 2dup =
  IF  -rot d- 0<  ELSE  > nip  THEN
  nip
;

: D>            \ d1 d2 -- t/f
  2SWAP D<
;

: DU<           \ ud1 ud2 -- t/f
  ROT SWAP 2DUP U<                      \ ud1l ud2l ud1h ud2h f
  IF   2DROP 2DROP  -1
  ELSE  <>
        IF   2DROP  0  ELSE  U<  THEN
  THEN
;

: DU>           \ ud1 ud2 -- t/f
  ROT SWAP 2DUP U>                      \ ud1l ud2l ud1h ud2h f
  IF   2DROP 2DROP  -1
  ELSE  <>
        IF   2DROP  0  ELSE  U>  THEN
  THEN
;


\ dmin dmax dumin dumax  2constant 2variable

: DMIN          \ d1 d2 -- d3
  2OVER 2OVER D>
  IF   2SWAP   THEN
  2DROP
;

: DMAX          \ d1 d2 -- d3
  2OVER 2OVER D<
  IF   2SWAP   THEN
  2DROP
;

: DUMIN         \ d1 d2 -- d3
  2OVER 2OVER DU>
  IF   2SWAP   THEN
  2DROP
;

: DUMAX         \ d1 d2 -- d3
  2OVER 2OVER DU<
  IF   2SWAP   THEN
  2DROP
;

: 2CONSTANT     \ dn -- ; -- dn [child]
   CREATE  , ,  DOES> 2@  ;

: 2VARIABLE     \ -- ; -- addr [child]
  create  0 , 0 ,  ;


\ q@ q! q+ qnegate ?qnegate qabs

\ not needed if DUM/MOD is coded
: q@            \ addr -- q ; low address on top of stack
  >r
  r@ 6 + @
  r@ 4 +  @
  r@ 2+  @
  r>     @
;

\ not needed if DUM/MOD is coded
: q!            \ q addr -- ; top of stack to low address
  >r
  r@     !
  r@ 2+  !
  r@ 4 + !
  r> 6 + !
;

\ Q+ is not needed if DUM* is coded
: q+            \ q1 q2 -- q3
  7 roll 0  5 roll 0  d+  >r            \ add least significant, save carry
  6 roll 0  5 roll 0  d+  r> 0 d+  >r   \ add next sig and carry, save carry
  5 roll 0  5 roll 0  d+  r> 0 d+  >r   \ add next sig and carry, save carry
  4 roll  4 roll  +  r> +               \ add most sig and carry  
;

\ not needed if DUM/MOD is coded
: q2*           \ q -- q*2
  2over 2over                           \ QDUP
\ 3 pick  3 pick  3 pick  3 pick        \ alternative QDUP
  q+
;

\ This should be coded
: qnegate       \ q -- -q ; negate by invert and add 1
  3 roll  not  1+ dup >r                \ 4OS: invert and add one, save carry
  3 roll  not                           \ 3OS:
  3 roll  not                           \ 2OS
  3 roll  not                           \ TOS
  r> if  exit  endif                    \ done if no carry
  rot 1+ dup >r  -rot                   \ update 3OS
  r> if  exit  endif                    \ done if no carry
  swap 1+ swap                          \ update 2OS
  over if  exit  endif                  \ done if no carry
  1+                                   \ update TOS  
;

: ?qnegate      \ q t/f -- q'
  0<
  if  qnegate  endif
;

: qabs          \ q -- |q|
  dup 0<
  if  qnegate  endif
;


\ dum* d* dm*

\ Code this
: dum*          \ ud1 ud2 -- uq
  over  4 pick  um*  >r >r              \ l1*l2
  swap  2 pick  um*  >r >r              \ h1*l2
  rot over um* >r >r                    \ l1*h2
  um* 0 0 2swap                         \ h1*h2
  0 r> r> 0  q+                         \ + l1*h2
  0 r> r> 0  q+                         \ + h1*l2
  r> r> 0 0  q+                         \ + l1*l2
;

: d*            \ d1 d2 -- d3
  dum* 2drop
;

: dm*           \ d1 d2 -- q
  2 pick over xor >r                    \ save sign of result
  dabs 2swap dabs dum*                  \ unsigned multiply
  r> ?qnegate                           \ apply sign
;


\ dum/mod du/ dm/mod

\ TSUMH is not needed if DUM/MOD is coded. Note that TSUMH is used by DUM/MOD
\ and if DUM/MOD is used in an interrupt routine, arrangements must be 
\ made to preserve the previous contents of TSUMH. This can be done by
\ adding the following to the start of DUM/MOD
\       TSUMH Q@ >R >R >R >R
\ and to the end og DUM/MOD
\       R> R> R> R> TSUMH Q!
\
\ create tsumh  8 allot         \ build this in RAM: ModForth version
\ variable tsumh  6 allot-ram   \ v 5.x cross compiler version
4 cells buffer: tsumh		\ v 6.x cross compilers

hex

\ Coding DUM/MOD can give an enormous increase in speed.
: dum/mod       \ uq ud -- udrem udquot
  >r >r  tsumh q!  r> r>                \ save dividend
  #32 0                                 \ 32 times
  do  tsumh q@  q2*                     \ shift dividend left one bit
      dup >r  tsumh q!                  \ preserving carry
      2dup tsumh 2@ du> 0=              \ if divisor less than top 32 bits
      if  tsumh 2@ 2over d- tsumh 2!    \ substract divisor
          1 tsumh 6 + +!                \ increment quotient
          r> drop
      else  r> 0<                       \ test preserved carry
            if  tsumh 2@ 2over d- tsumh 2!      \ substract divisor
                1 tsumh 6 + +!          \ increment quotient
            endif
      endif
  loop
  2drop                                 \ discard divisor
  tsumh 2@  tsumh 4 + 2@                \ return remainder and quotient
;

: du/           \ ud1 ud2 -- udquot
  dum/mod 2swap 2drop
;


\ DM/MOD obeys the quotient and remainder rules for floored division
\ as required by the Forth-83 specification.

: dm/mod        \ q d -- drem dquot
  2dup >r >r                            \ save divisor
  2 pick over xor >r                    \ save sign quotient
  dup >r                                \ save sign divisor
  >r >r  qabs  r> r> dabs  dum/mod      \ make abs and divide
  2swap  r> ?dnegate  2swap             \ set sign remainder
  r> 0<                                 \ sign of quotient
  if  dnegate                           \ apply it
      2over d0<>                        \ adjust remainder ?
      if  0,1 d-                        \ adjust quotient
          2swap r> r> 2swap d- 2swap    \ adjust remainder
          exit                          \ and get out
      endif
  endif
  r> r> 2drop                           \ discard divisor
;


\ d>q d/mod d/ dmod d*/mod d*/

: d>q           \ d -- q
  dup 0< dup
;

: d/mod         \ d1 d2 -- drem dquot
  >r >r  d>q  r> r>  dm/mod
;

: d/            \ d1 d2 -- d3
  d/mod 2swap 2drop
;

: dmod          \ d1 d2 -- d3
  d/mod 2drop
;

: d*/mod        \ d1 d2 d3 -- drem dquot
  >r >r  dm*  r> r>  dm/mod
;

: d*/           \ d1 d2 d3 -- d4
  d*/mod 2swap 2drop
;


\ ======
\ *> ###
\ ======

decimal


