\ TSEPOS 7 36 7 \

\ 29F010/40 Driver code


\ Assuming 16 bit bus using 2 devices, using 040 devices but only the top
\ segment of each (hardware fault).

\ 00000 .. 1FFFF               128k Section#1   SA0, Rom A + B    Kernel
\ 20000 .. 3FFFF               128k Section#2                     User

\ =======================================================================
( **                                                                    )
( ** Globals                                                            )
( **                                                                    )
\ =======================================================================

create  cmEraseSectorA  6 ,
                        $AA , $05555 2* ,
                        $55 , $02AAA 2* ,
                        $80 , $05555 2* ,
                        $AA , $05555 2* ,
                        $55 , $02AAA 2* ,
                        $30 , $00000 2* ,

create  cmEraseSectorB  6 ,
                        $AA , $05555 2* 1+ ,
                        $55 , $02AAA 2* 1+ ,
                        $80 , $05555 2* 1+ ,
                        $AA , $05555 2* 1+ ,
                        $55 , $02AAA 2* 1+ ,
                        $30 , $00000 2* 1+ ,

create  cmWriteRomA     4 ,
                        $AA , $05555 2* ,
                        $55 , $02AAA 2* ,
                        $A0 , $05555 2* ,
                        0   , 0         ,       \ data \ address

create  cmWriteRomB     4 ,
                        $AA , $05555 2* 1 + ,
                        $55 , $02AAA 2* 1 + ,
                        $A0 , $05555 2* 1 + ,
                        0   , 0         1 + ,       \ data \ address

12 cells buffer:        cmCommand

\ =======================================================================
( **                                                                    )
( ** Utilities                                                          )
( **                                                                    )
\ =======================================================================

: @++                   \ addr -- addr+cell (addr)
  dup cell+ swap @
;

code (FlashWrite)       \ addr #regs --
        mov.l   @sp+    er0             \ er0 = address, tos = #regs to do
l: fwl
        mov.l   @er0+   er2
        mov.l   @er0+   er1
        mov.b   r2l     @er1
        sub.l   # 1     tos
        bne     fwl
l: wlp
        mov.b   @er1    r2l
        and.l   # $80   er2
        cmp.l   # $80   er2
        bne     wlp
        mov.l   @sp+    tos
        next,
end-code

here equ fwend

code (FlashWrite2)       \ addr #regs --
        mov.l   @sp+    er0             \ er0 = address, tos = #regs to do
l: fwl2
        mov.l   @er0+   er2
        mov.l   @er0+   er1
        mov.b   r2l     @er1
        sub.l   # 1     tos
        bne     fwl2
l: wlp2
        mov.b   @er1    r2h
        cmp.b   r2l     r2h
        bne     wlp2
        mov.l   @sp+    tos
        next,
end-code

here equ fwend2

fwend  ' (flashwrite)  -   buffer:         fw
fwend2 ' (flashwrite2) -   buffer:         fw2

: FlashWrite    \ addr #regs --
  di fw execute ei
;

: FlashWrite2    \ addr #regs --
  di fw2 execute ei
;

: (EraseSector)         { sa rom -- }
  rom 1 and
  if cmEraseSectorB else cmEraseSectorA then
  @++ dup >r 2* cells cmCommand swap move
  sa #15 LSHIFT
  rom 1 and +   cmCommand 11 cells + !
  cmCommand r> FlashWrite
;

: EraseSector           \ sector --
  dup 0 (EraseSector)
  1 (ERaseSector)
;

: WriteByte             { data address -- }
  address 1 and if
    cmWriteRomB else cmWriteRomA then
  @++ dup >r 2* cells cmCommand swap move
  data    cmCommand 6 cells + !
  address cmCommand 7 cells + !
  cmCommand r> FlashWrite2
;

\ =======================================================================
( **                                                                    )
( ** API                                                                )
( **                                                                    )
\ =======================================================================

: Flash!                \ data address --
  WriteByte
;

: InitialiseFLASH     \ --
  ['] (flashwrite)  fw  fwend  2 pick - move
  ['] (flashwrite2) fw2 fwend2 2 pick - move
;

: KillUserFlash
  7 erasesector
;
