\ TSEPOS 73 24 46 \

\ TSECFGEXPR    '^[ \t]*{\(\ \**}|{code}|{:}'
\ TSECFGOPTS    'ix'

\ T             title
\ P             begin paragraph (ends after any non * line)
\ H             a general heading
\ S             Section sub-title
\ D             Definition, next word is title, all following text
\               and * lines are descriptions
\ N             Forced newline
\ C             Description comment for Forth word
\ E             An Edit Comment line

\ =======================================================================
( **                                                                    )
( *T CANREAL                                                            )
( **                                                                    )
( *P This file provides a set of words to act has a hardware abstraction)
( ** layer for the i82527 drivers when using the physical device on the )
( ** MPE H8 Board.                                                      )
( **                                                                    )
\ =======================================================================

: CAN-INIT(CANREAL)     \ --
( ** Set up the base register pointer and provide default Can config.   )
  CSEXT CAN_BASE !
  PowerOnConfigure
;

: CAN!(CANREAL)         \ val offset --
( ** Store a value in a given i82527 register.                          )
  CAN_BASE @ + c!
;


: CAN@(CANREAL)         \ offset -- val
( ** Fetch the 8 bit contents of an i82527 register.                    )
  CAN_BASE @ + c@
;

: CAN-FLUSH(CANREAL)    \ --
( ** Flush any pending hardware writes to the can device.               )
;

: dc@                   \ addr -- dat
( ** Perform a register fetch twice discarding the first result.        )
  dup can_base @ + c@ drop
      can_base @ + c@
;

: fc@                   \ addr -- dat
( ** Perform a register fetch via the fast read interface.              )
  can_base @ + c@ drop          \ dummy read to set address
  can_base @ 4 + c@             \ read from high speed register
;

: dualcanread           \ --
( ** Change the default can read mode to use the dc@ primitive.         )
  assign dc@ to-do can@
;

: fastcanread           \ --
( ** Change the default can read mode to use the fc@ primitive.         )
  assign fc@ to-do can@
;

\ ============================================================================

assign  CAN-INIT(CANREAL) to-do    CAN-INIT     \ Path up i82527 driver layer
assign  CAN!(CANREAL)     to-do    CAN!
assign  fc@               to-do    CAN@
assign  CAN-FLUSH(CANREAL) to-do   CAN-FLUSH

