\ ANSfile.fth

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal

\ ======
\ *! ansfile
\ *T ANS File word set
\ ======
\ *P This group of words provides the ANS file word set using the
\ ** native file system facilities.


\ **********************
\ *S File access methods
\ **********************
\ *P These are ignored at present, but are provided for compatibility.

: bin           \ fam -- 'fam
\ *G Modify a file-access method to include BINARY.
;

1 constant r/o	\ -- fam
\ *G Specifiy a read-only file.

2 constant w/o	\ -- fam
\ *G Get Writeonly fam.

3 constant r/w	\ -- fam
\ *G Get ReadWrite fam.


\ *********
\ *S Basics
\ *********

: FSerr:	\ err# -- ; n -- 0|err#
\ *G The underlying file system primitives return zero on error.
\ ** The children of this word return 0 on success or a given
\ ** error code which can in turn be used as a throw code.
  create
    ,
  does>
    swap
    if  drop 0  else  @  endif
;

\ *[ 
#-258 FSerr: ?err_open		\ n -- n'
#-412 FSerr: ?err_create	\ n -- n'
#-414 FSerr: ?err_read		\ n -- n'
#-415 FSerr: ?err_write		\ n -- n'
#-416 FSerr: ?err_close		\ n -- n'
#-417 FSerr: ?err_seek		\ n -- n'
#-418 FSerr: ?err_delete	\ n -- n'
#-419 equ    err_handle		\ -- n
#-420 equ    err_resize		\ -- n
\ *] 

: Open-File	\ c-addr u fam -- fileid ior
\ *G Create a file on disk, returning a 0 ior for success and
\ ** a file id.
  drop open_f dup ?err_open
;

: Create-File	\ c-addr u fam -- fileid ior
\ *G Create a file on disk, returning a 0 ior for success and
\ ** a file id.
  drop  2dup 1 1 create_f 0=
  if  2drop #-412  exit  endif
  open_f dup ?err_create
;

: delete-file   \ c-addr u -- ior
\ *G Delete a named file from disk, and return ior=0 on success.
  delete_f ?err_delete
;

: Read-File     \ caddr u fileid -- u2 ior
\ *G Read data from a file. The number of bytes actually read is
\ ** returned as u2, and ior is returned 0 for a successful read.
\ ** Note that all reads with zero length succeed.
  over 0=
  if  3drop 0  exit  endif
  read_f dup ?err_read
;

: Write-File    \ caddr u fileid -- ior
\ *G Write a block of memory to a file. Note that all writes
\ ** with zero length succeed.
  over 0=
  if  3drop 0  exit  endif
  write_f ?err_write
;

: Close-File    \ fileid -- ior
\ *G Close an open file.
  close_f ?err_close
;

: file-position \ fileid -- ud ior
\ *G Return file position, and return ior=0 on success.
  dup ->valid-otb @ VALID_OTB <> if
    drop -1 -1 err_handle  exit
  endif
  ->r/w-data-ptr @ 0  0
;

: Reposition-File	\ ud fileid -- ior
\ *G Set file position, and return ior=0 on success.
  nip lseek_f ?err_seek
;

: file-size     \ fileid -- ud ior
\ *G Get size in bytes of an open file as a double number,
\ ** and return ior=0 on success.
  dup ->valid-otb @ VALID_OTB <> if
    drop -1 -1 err_handle  exit
  endif
  fcb-addr ->file-size @ 0  0
;

: Resize-File	\ ud fileid -- ior
\ *G Set the size of the file to ud, an unsigned double number. After
\ ** using *\fo{RESIZE-FILE}, the result returned by *\fo{FILE-POSITION} may
\ ** be invalid.
  3drop err_resize
;


\ ======
\ *> ###
\ ======

decimal


