\ D-cache.fth - disk cache

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
20050407 SFP002 Incorporated CEM updates.
))

decimal

\ ======
\ *! dcache
\ *T Disk Cache
\ ======
\ *P The disk cache reduces the number of accesses made to the physical
\ ** drive.

\ move to STRUCTS.FTH
((
struct CACHE-INFO	\ -- len
\ *G Data structure controlling each cached disk sector.
\ *[ 
  int ->cache-link	\ link to next structure in chain; MUST be first
  int ->mem-block       \ pointer to data
  int ->disk-block	\ disk block number
  int ->cache-valid	\ nz = cache valid
  int ->cache-time	\ last read/write time
  int ->cache-dirty	\ nz = modified
end-struct
\ *] 
))

$125a5aff equ CACHE_VALID
$125a5a5a equ CACHE_DIRTY
$00000000 equ CACHE_CLEAN

#CACHE_BLOCKS CACHE-INFO * equ CACHE_HDR_SIZE
#CACHE_BLOCKS BLOCK_SIZE * equ CACHE_SIZE

CACHE_SIZE buffer: cache-memory	\ -- addr
\ *G The cache data blocks.
CACHE_HDR_SIZE buffer: cache-hdr-table	\ -- addr
\ *G The cache lookup table.

variable active-c-block	\ -- addr
\ *G Anchors active block chain.
variable free-c-blocks	\ -- addr
\ *G Anchors free block chain.


\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

: init-cache	\ --
\ *G Initialise the disk cache.
\  cr ." Init Disk cache ... " 
  cache-sema init-sema
  cache-hdr-table CACHE_HDR_SIZE erase	\ clear file control block table
  cache-memory CACHE_SIZE erase
  0 active-c-block !
  0
  #CACHE_BLOCKS 0 do			\ -- last_cache_hr
    i CACHE-INFO * cache-hdr-table +	\ -- last_cache_hdr curr_cache_hdr
    i BLOCK_SIZE * cache-memory + over ->mem-block !
    tuck ->cache-link !
    ->cache-link
  loop
  free-c-blocks !
\  ." done"
;

: release-cache	\ struct --
\ *G Put the given cache structure back onto the free chain. 
\ ** Exit quietly if the structure cannot be found.
  active-c-block
  begin					\ -- struct s2
    2dup ->cache-link @ <>		\  MUST be first
   while
    ->cache-link @ dup 0=		\ if at end of chain
    if  2drop  exit  endif		\   get out quietly
  repeat
\ struct prev -- ; take the next link, and place it in the previous link
  over ->cache-valid off
  over ->cache-link @  swap ->cache-link !
  free-c-blocks @ over ->cache-link !	\ add to free chain
  free-c-blocks !
;

: release-all-cache	\ --
\ *G Put all cache structures back on the free chain.
  di
  active-c-block ->cache-link @
  begin
    ?dup
   while
    dup ->cache-link @      	\ get next link
    swap release-cache
  repeat
  ei
;

\ -----------------------------------------------------------------
\ ENTRY, a possible oldest cache structure

: (get-cache)	\ -- struct|0
\ *G Get a new cache structure, or return 0 if one cannot be found.
  free-c-blocks @ dup if
    dup ->cache-link @ free-c-blocks !	\ unhook from free chain
    active-c-block @ over ->cache-link ! \ add to active chain
    dup active-c-block !
  endif
;

: get-oldest-cache	\ struct --
\ *G If the cache item is dirty (needs to be written back), write
\ ** it back and mark it as unused.
  dup ->cache-dirty @ CACHE_DIRTY = if
    dup ->disk-block @
    over ->mem-block @ write-block drop
  endif
  ->disk-block off
;

: get-cache	\ ?struct -- struct
\ *G If there is a free cache block, discard the given one and
\ ** use a free one, otherwise reuse it.
  free-c-blocks @ if			\ any free cache blocks ?
    drop				\ YES don't need old cache
    (get-cache)				\ try to get fresh cache
  else					\ NO
    dup get-oldest-cache		\ oldest cache
  endif
;

((
: blk-in-cache?	{ l_blk | l_struct l_oldest l_found -- struct t/f )
\ ARM - 208 bytes
  -1 -> l_oldest
  0 -> l_struct
  false -> l_found
  active-c-block
  begin
    @ ?dup				\ still in chain
   while
    dup ->disk-block @ l_blk =		\ this block
    if  true exit  endif
    dup ->cache-time @ l_oldest u< if
      dup -> l_struct
      l_struct ->cache-time @ -> l_oldest
    endif           
  repeat
  l_struct l_found
;
))
: blk-in-cache?	\ blk# -- struct t/f
\ *G If the given block is in the disk cache, return its cache
\ ** structure and true, otherwise return the oldest cache
\ ** structure and false.
\ ARM - 128 bytes
  >r
  0 -1					\ dummy oldest structure and time
  active-c-block
  begin					\ -- olds oldt prev
    @ dup				\ still in chain
   while				\ -- olds oldt struct
    dup ->disk-block @ r@ = if		\ this block
      r> drop
      nip nip true  exit
    endif
    2dup ->cache-time @ swap u< if
      nip nip				\ discard olds, oldt
      dup dup ->cache-time @ swap	\ -- olds' oldt' struct=olds
    endif           
  repeat
  2drop 0				\ -- olds 0
  r> drop
;

: read-blk>c	\ blk struct -- cache
\ *G Read the given block into cache, returning the actual cache
\ ** structure used, or zero if no cache is available.
\ ARM - 104 bytes
  get-cache dup if	           	\ -- blk cache ; able to get cache ?
    CACHE_CLEAN over ->cache-dirty !	\ signal it's clean
    CACHE_VALID over ->cache-valid !	\ signal it's valid
    ticks over ->cache-time !		\ record the time
    2dup ->disk-block !			\ record the block
    tuck ->mem-block @ read-f-block	\ read block, returns -1 if good
\ Should this block be released if the physical read fails?
    and					\ -- cache|0
  else
    nip
  endif
;

: (read-blk)	\ blk -- struct
\ *G If the block is already in cache, return its cache structure,
\ ** otherwise read it into cache, returning the cache structure
\ ** used or zero for failure.
  dup blk-in-cache? if
    nip                 		\ YES, lose block, keep structure
  else
    read-blk>c          		\ NO, read into cache
  endif
;
    
: read-p-blk	\ lba buff start size -- t/f
\ *G Read a partial block/sector into the given buffer.
\ ** Return true for success.
\ a partial block read ; ARM - 128 bytes
  3 pick DISK_SIZE u< not		\ SFP002
  if  2drop 2drop 0  exit  endif	\ SFP002
  cache-sema request
  swap >r  rot (read-blk)		\ -- buff size cache ; R: -- start ; read and return cache structure
  ticks over ->cache-time !		\ record the time
  ->mem-block @ r> + -rot		\ -- start' buff size
  cmove
  true 
  cache-sema signal
;

((
: read-f-blk	{ l_lba l_buff | -- t/f }
\ a full block read ; ARM - 196 bytes
  cache-sema request
  l_lba blk-in-cache? if		\ this block's in cache
    ->mem-block @ l_buff BLOCK_SIZE cmove \ copy from cache
    cache-sema signal
    true
  else
    cache-sema signal
    drop
    l_lba l_buff read-f-block		\ no, so just read full block
  endif
;
))
: read-f-blk	\ lba buff -- t/f
\ *G Read a full block/sector into the given buffer.
\ ** Return true for success.
\ a full block read ; ARM - 96 bytes
  over DISK_SIZE u< not			\ SFP002
  if  2drop 0  exit  endif		\ SFP002
  cache-sema request
  over blk-in-cache? if			\ -- lba buff struct ; this block's in cache
    rot drop				\ discard lba
    ->mem-block @ swap BLOCK_SIZE cmove \ copy from cache
    cache-sema signal
    true
  else
    drop				\ discard cache struct
    cache-sema signal
    read-f-block			\ no, so just read full block
  endif
;


\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

((
: write-p-blk	{ l_blk l_buffer l_start l_end | l_struct -- t/f  }
\ ARM - 228 bytes
  cache-sema request
  l_blk (read-blk) dup if
    -> l_struct				\ find a cache block
    l_buffer                            \ source
    l_struct ->mem-block @  l_start +   \ destination
    l_end l_start -                     \ amount
    cmove                               \ write to cache
    ticks l_struct ->cache-time !       \ record time in cache
    CACHE_DIRTY l_struct ->cache-dirty !    \ mark as dirty
    true
  endif
  cache-sema signal
;
))
: write-p-blk	\ blk buffer start end -- t/f
\ *G Write a partial block/sector from the given buffer.
\ ** Return true for success.
  3 pick DISK_SIZE u< not		\ SFP002
  if  2drop 2drop 0  exit  endif	\ SFP002
  cache-sema request
  over -				\ -- blk buffer start len
  swap 2swap swap (read-blk) dup if	\ -- len start buffer struct ; find a cache block
    dup >r  ->mem-block @  rot +  	\ -- len buffer mem+start
    rot cmove				\ write to cache
    ticks r@ ->cache-time !		\ record time in cache
    CACHE_DIRTY r> ->cache-dirty !	\ mark as dirty
    true
  else
    nip nip nip
  endif
  cache-sema signal
;

((
: write-f-blk	{ l_lba l_buff | -- t/f }
\ does a full block write to disk, replaces WRITE-BLOCK
\ ARM - 164 bytes
  cache-sema request
  l_lba blk-in-cache? if          	\ is there a partial in cache ?
    release-cache               	\ YES, well release it
  else
    drop
  endif
  cache-sema signal
  l_lba l_buff write-block        	\ write full block to disk
;
))
: write-f-blk	\ lba buff -- t/f
\ *G Does a full block write to disk, replaces *\fo{WRITE-BLOCK}.
\ ** Return true for success.
  over DISK_SIZE u< not			\ SFP002
  if  2drop 2drop 0  exit  endif	\ SFP002
  cache-sema request
  over blk-in-cache? if          	\ is there a partial in cache ?
    release-cache               	\ YES, well release it
  else
    drop
  endif
  cache-sema signal
  write-block		        	\ write full block to disk
;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

((
: sync_vol	{ | l_struct -- }
\ ARM - 252 bytes
  cache-sema request
  active-c-block ->cache-link @
  begin
    ?dup
  while
    -> l_struct
    l_struct ->cache-valid @ CACHE_VALID =
    l_struct ->cache-dirty @ CACHE_DIRTY = and if
      l_struct ->disk-block @
      l_struct ->mem-block @ write-block drop
      l_struct ->cache-dirty off
      ticks l_struct ->cache-time !	\ record time in cache
    endif
    l_struct ->cache-link @		\ get next link
  repeat
  cache-sema signal
;
))
: sync_vol	\ --
\ *G Synchronise the cache and the disk by writing out any modified
\ ** blocks.
\ ARM - 144 bytes
  cache-sema request
  active-c-block
  begin
    @ dup
  while
    dup ->cache-valid @ CACHE_VALID = if
      dup ->cache-dirty @ CACHE_DIRTY = if
        dup ->disk-block @
        over ->mem-block @ write-block drop
        dup ->cache-dirty off
        ticks over ->cache-time !		\ record time in cache
      endif
    endif
    ->cache-link
  repeat
  drop
  cache-sema signal
;

((
: time_sync_vol	{ | l_struct l_expired -- }
\ ARM - 300 bytes
  ticks VOL_TIMEOUT - -> l_expired
  cache-sema request
  active-c-block ->cache-link @
  begin
    ?dup
   while
    -> l_struct
    l_struct ->cache-valid @ CACHE_VALID =
    l_struct ->cache-dirty @ CACHE_DIRTY = and 
    l_struct ->cache-time @ l_expired u< and if
      l_struct ->disk-block @
      l_struct ->mem-block @ write-block drop
      l_struct ->cache-dirty off	\ mark as clean
      ticks l_struct ->cache-time !	\ record time in cache
    endif
    l_struct ->cache-link @		\ get next link
  repeat
  cache-sema signal
;
))

TaskDiskFlush DiskCacheMode = [if]

: time_sync_vol	\ --
\ *G Write out any modified cache blocks that have been in cache
\ ** loger than *\fo{VOL_TIMEOUT} milliseconds.
\ ARM - 188 bytes
  cache-sema request
  ticks VOL_TIMEOUT -			\ -- expired
  active-c-block			\ -- expired prev
  begin
    @ dup
   while				\ -- expired curr
    dup ->cache-valid @ CACHE_VALID = if
      dup ->cache-dirty @ CACHE_DIRTY = if
        2dup ->cache-time @ swap u< if
          dup ->disk-block @
          over ->mem-block @ write-block drop
          dup ->cache-dirty off		\ mark as clean
          ticks over ->cache-time !	\ record time in cache
        endif
      endif
    endif
    ->cache-link			\ next link
  repeat
  2drop
  cache-sema signal
;

[then]


\ ********************
\ *S Cache diagnostics
\ ********************
\ *P This code is only compiled if the equate *\fo{FSdiags?} is
\ ** set non-zero.

FSdiags? [if]

((
: (show-cache)	{ | l_struct -- }
\ ARM - 260 bytes
  begin
    ->cache-link @ ?dup
   while
    -> l_struct
    cr
    l_struct .hex
    l_struct ->cache-valid @ CACHE_VALID =
    if  ." V "  else  ." N "  endif
    l_struct ->cache-dirty @ CACHE_DIRTY =
    if  ." D "  else  ." C "  endif
    l_struct ->mem-block @ .hex space
    l_struct ->disk-block @ .decimal space
    l_struct ->cache-time @ .decimal space
    l_struct
  repeat
;
))
: (show-cache)	\ anchor --
\ *G Display the cache blocks in a particular chain.
\ ARM - 172 bytes
  begin
    @ dup
   while
    cr  dup .hex
    dup ->cache-valid @ CACHE_VALID =
    if  ." V "  else  ." N "  endif
    dup ->cache-dirty @ CACHE_DIRTY =
    if  ." D "  else  ." C "  endif
    dup ->mem-block @ .hex space
    dup ->disk-block @ .decimal space
    dup ->cache-time @ .decimal space
    ->cache-link
  repeat
  drop
;

: show-cache	\ --
\ *G Display the cache blocks in the active chain.
  active-c-block (show-cache)
;

: show-f-cache	\ --
\ *G Display the cache blocks in the free chain.
  free-c-blocks (show-cache)
;

((
: t-c1
  #30 0 do
    i (read-blk) cr .hex
  loop
;
))

[then]


\ ======
\ *> ###
\ ======

decimal


