\ Disk24C512.fth

((
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ =============
\ *! disk24c512
\ *T 24C512 Serial EEPROM Disk
\ =============
\ *P This file requires the I2C driver:
\ *C   Examples\I2C\Devices\AT24C512drv.fth

\ *P Assuming that a single device is used gives a disk size of
\ ** 64 kbytes.

\ *P Disk sectors are referenced by a block number, 0..DISK_SIZE-1.

\ *P Other example disk drivers may be found in the folder
\ ** *\i{%FileSysDir%\Drivers}.

#64 kb BLOCK_SIZE / equ #SECTORS/DISK	\ -- n
\ *G Number of sectors per disk.

\ API
: show-drive-details	\ addr --
\ *G Display drive details from given data buffer
  ?free
  cr ." AT24C512 - 64 kb"
;

\ API
: (drive-id)	\ -- buff
\ *G Allocates a 512 byte structure that includes the disk size in
\ ** sectors. For IDE and CompactFlash drives, this word should use
\ ** the IDE-ID ($EC) command.
  BLOCK_SIZE allocate if  		\ grab buffer
    ." Can't allocate "
    drop 0
  else
    dup BLOCK_SIZE erase
    #SECTORS/DISK over ID.#SECTORS + l!(le)
  endif
;

((
\ API
: drive-id	\ --
  (drive-id) ?dup if
    dup show-drive-details
    ?free
  else
    cr ." Error reading drive "
  endif
;     
))

\ API
: write-block	\ blk# caddr -- flag ; 0=bad
\ *G Write the buffer at *\b{caddr} to the disk sector *\b{blk#}.
\ cr ." Writing block " over .  dup $080 dump
  drive-access request
  BLOCK_SIZE  rot over *		\ -- caddr #512 eeaddr
  ['] SEPROM! catch dup
  if  nip nip nip  endif
  0=
  drive-access signal
;

\ API
: read-f-block	\ blk# caddr -- t/f ; 0=bad
\ *G Read the disk sector *\b{blk#} into the buffer at *\b{caddr}.
  drive-access request
  BLOCK_SIZE  rot over *		\ -- caddr #512 eeaddr
  ['] SEPROM@ catch dup
  if  nip nip nip  endif
  0=
  drive-access signal
;
        
\ API
: erase-block	\ blk# -- t/f
\ *G Erase (set to zero) the given disk block.
  BLOCK_SIZE allocate
  if  drop false  exit  endif
  dup BLOCK_SIZE erase
  tuck write-block
  swap ?free
;
        
\ API
: InitDisk	\ --
\ *G Initialise disk access hardware
;


\ ======
\ *> ###
\ ======

decimal


