\ FILECODE.FTH - Hardware specific code

\ *P Because this code is used so much in the system, it is
\ ** worth coding it for maximum efficiency. One of the
\ ** following equates may be defined (any value) in the control
\ ** file to compile CPU specific code.
\ *D CPU=ARM      All ARM variants
\ *D CPU=H8/300H  Hitachi H8/300H and H8S
\ *D CPU=68K      Motorola 68xxx CPUs
\ *D CPU=386      Intel 386 and above (i32)
\ *P If no CPU specific equate is defined, the high level code
\ ** will be used.


\ *****************************
\ *S H8S and H8/300H primitives
\ *****************************

[defined] H8/300H [if]

CODE f-FILL	\ w-addr u long -- ; 32-bit fast fill
\ *G Fill an aligned buffer with the the 32 bit long.
  mov.l @sp+ er1   mov.l @sp+ er0	\ er0=addr, er1=len, tos=char
	
  mov.l er1 er1                         \ if len<>0
  ne, if,
    mov.l # 4 er2			\ can only sub.w rs rd
    begin,
      mov.l tos @er0			\ store byte
      adds # 4 er0			\ increment address
      sub.l er2 er1			\ decrement count
    eq, until,
  endif,
  mov.l @sp+ tos
  next,
END-CODE

code f-block-xfer	\ buff size port --
\ read block of data from a memory port to memory block in 16 bit units.
  mov.l @sp+ er1			\ buffer size
  mov.l @sp+ er0			\ buffer base address
  begin,
    mov.w @tos r2			\ read port
    mov.w r2 @er0			\ write to buffer
    inc.l # 2 er0			\ inc buffer address
    dec.l # 2 er1			\ decrement count
  eq, until,
  mov.l @sp+ tos
  next,
end-code

\ write block of data from memory block to port
code f-block-tx		\ buff size port --
  mov.l @sp+ er1			\ buffer size
  mov.l @sp+ er0			\ buffer base address
  begin,
    mov.w @er0+ r2			\ read buffer
    mov.w r2 @tos			\ write to port
    dec.l # 2 er1			\ decrement count
  eq, until,
  mov.l @sp+ tos
  next,
end-code

[then]


\ ******************************
\ *S Default high level versions
\ ******************************

[undefined] f-block-xfer [if]
: f-block-xfer	\ buff size port --
\ *G Read a block of data from a memory port to a memory block
\ ** in 16 bit units.
  -rot  bounds ?do			\ -- port
    dup w@ i w!
  2 +loop
  drop
;
[then]

[undefined] f-block-tx [if]
: f-block-tx	\ buff size port --
\ *G Write a block of data from a memory block to a memory port
\ ** in 16 bit units.
  -rot  bounds ?do			\ -- port
    i w@ over w!
  2 +loop
  drop
;
[then]


\ *********************
\ *S Disk LED functions
\ *********************

: disk-led-on	\ --
\ *G Turn on a disk access LED.
;

: disk-led-off	\ --
\ *G Turn off a disk access LED.
;

: disk-rd-led-on	\ --
\ *G Turn on a disk access LED.
;

: disk-rd-led-off	\ --
\ *G Turn off a disk access LED.
;

