\ Extent.fth - handling non-contiguous files.

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ =========
\ *! extent
\ *T Extents
\ =========
\ *P To permit a file to be non-contiguous, each File Control Block
\ ** allows a file to be in up to ten different sections of the
\ ** disk, referred to as *\b{extents}. An extent is defined by the
\ ** following structure in *\i{STRUCTS.FTH}.
\ *E struct file-extents  \ -- len
\ **   int  ->extent-start      \ location of data (block number)
\ **   int  ->extent-size       \ size of data, (number of blocks)
\ ** end-struct

\ *P If more than ten extents are needed, another block is used to
\ ** contain additional extents. These are referred to as
\ ** *\b{indirect extents} which are controlled by the
\ ** following structure in *\i{STRUCTS.FTH}.
\ *E struct indir-extent
\ **   int  ->ind-extent-start
\ **   int  ->ind-logical-end
\ ** end-struct

: new-extent-needed	{ l_blocks l_fid | -- blocks }
\ *G A returned positive number is the number of blocks required.
  l_fid get-r/w-b-ptr     		\ -- current-block
  l_blocks +              		\ new expected end
  l_fid fcb-addr ->file-blocks @ -
;

compiler
: need-extent?	\ blocks fid -- t/f
\ *G COMPILER: Return TRUE if more space is needed in the file,
\ ** i.e. it needs to be extended.
  new-extent-needed 0>
;
target

: file-extent	{ l_ext-no l_fld -- block size }
\ *G Return the start and length in blocks of the extent number
\ ** for the file number.
  l_ext-no cells 2* -> l_ext-no
  l_fld ->extent-start-0 l_ext-no + @
  l_fld ->extent-size-0 l_ext-no + @
;

: fcb-free-extent?	{ l_buff | l_fcb l_ext -- t/f }
\ *G *\b{Buff} is the address of a *\fo{file-extension} structure.
\ ** Return true if the extent it points to is free.
  l_buff ->ext-no @ file-extents * -> l_ext	\ get extent offset
  l_buff ->ext-fid @ fcb-addr -> l_fcb
  l_fcb l_ext + ->extent-start-0 @		\ -- fcb, ext, start
  l_fcb l_ext + ->extent-size-0 @ or 0=
;

: next-extent?	{ l_buff | l_found -- t/f }
\ *G *\b{Buff} is the address of a *\fo{file-extension} structure.
\ ** Return true if there is a free extent in the FCB it points to.
  FALSE -> l_found
  NO-OF-EXTENTS 0 do
    i l_buff ->ext-no !
    l_buff fcb-free-extent? if  		\ found an unused extent
      TRUE -> l_found
      leave
    endif
  loop
  l_found
;

: end-of-existing-extent	{ l_buff | l_fcb l_ext -- end_block }
\ *G *\b{Buff} is the address of a *\fo{file-extension} structure.
\ ** Return the block number of the block immediately after the
\ ** end of the previous extent. This is used to determine whether
\ ** or not the last extent can be increased in size or a new
\ ** extent is needed.
  l_buff ->ext-no @ 1-  file-extents * -> l_ext \ get extent offset
  l_buff ->ext-fid @ fcb-addr -> l_fcb
  l_fcb l_ext + ->extent-start-0 @        	\ -- fcb, ext, start
  l_fcb l_ext + ->extent-size-0 @ +		\ end of last extent
;

: add-to-existing-extent?	{ l_buff | -- t/f }
\ *G *\b{Buff} is the address of a *\fo{file-extension} structure.
\ ** Return true if the previous extent can be extended.
  l_buff end-of-existing-extent
  l_buff ->ext-start @            		\ proposed start of next extent
  =                             		\ does it match?
;

: extend-extent	{ l_buff | l_fcb l_ext -- }
\ *G *\b{Buff} is the address of a *\fo{file-extension} structure.
\ ** Extend the existing previous extent.
  l_buff ->ext-no @ 1- file-extents * -> l_ext	\ get extent offset
  l_buff ->ext-fid @ fcb-addr -> l_fcb
  l_buff ->ext-size @ l_fcb l_ext + ->extent-size-0 +!
  l_buff ->ext-size @ l_fcb ->file-blocks +!	\ read existing file size
  lseconds @ l_fcb ->file-last-mod !
;

\ move to FCB.FTH?
: update-fcb	{ l_fcb | -- t/f }
\ *G Copy the RAM FCB to the FLD on the disk.
  fld-update request
  l_fcb ->fld-no @ fld-d-block        	\ disk block
  l_fcb                               	\ source
  l_fcb ->fld-no @ fld-m-block        	\ start
  dup FLD-REC +                       	\ end
  write-p-blk
  fld-update signal
;

: set-extent	{ l_buff | l_ext l_fcb -- }
\ *G *\b{Buff} is the address of a *\fo{file-extension} structure.
\ ** Use it to set the selected extent in the FCB.
  l_buff ->ext-no @ file-extents * -> l_ext	\ get extent offset
  l_buff ->ext-fid @ fcb-addr -> l_fcb
  l_buff ->ext-start @ l_fcb l_ext + ->extent-start-0 !
  l_buff ->ext-size @  l_fcb l_ext + ->extent-size-0 !
  l_buff ->ext-size @  l_fcb ->file-blocks +!	\ read existing file size
  lseconds @  l_fcb ->file-last-mod !
;


\ *******************
\ *S Indirect Extents
\ *******************

BLOCK_SIZE equ INDIR_SIZE	\ -- u
\ *G Size of indirect extension block.
INDIR_SIZE INDIR-EXTENT / equ #INDIR-EXTENT	\ -- n
\ *G Number of indirect extents per indirect extension block.
INDIR-EXTENT buffer: *ext-buff	\ -- addr
\ *G Buffer for handling indirect extents.

: need-indirect-extent	{ l_*struct l_fid | l_*ind_block l_oc -- t/f }
\ *G Adds a new indirect extent to a file as required.
\ ** *\b{Struct} is the address of a *\fo{file-extension} structure.
  FALSE -> l_oc
  l_fid fcb-addr ->indirect-block -> l_*ind_block

  l_*ind_block @ 0= if	\ need first indirect block ?
    #1 need-blocks l_*ind_block !	\ allocate new indirect extent block
    l_*ind_block @ erase-block drop
    l_fid fcb-addr update-fcb drop
  endif

  INDIR_SIZE 0 do			\ find empty indirect block
    l_*ind_block @			\ block
    *ext-buff				\ buffer
    i 					\ start
    INDIR-EXTENT			\ size
    read-p-blk if
      *ext-buff ->ind-extent-start @ 0= if	\ new block

        l_*struct ->ext-start @		\ get selected block start
\       ." Starting @" dup .decimal space l_*struct ->ext-size @ .decimal ." blocks "
	*ext-buff ->ind-extent-start !	\ save physical start
	l_fid fcb-addr ->file-blocks @	\ current size
	l_*struct ->ext-size @ + dup	\ plus extension
	l_fid fcb-addr ->file-blocks !	\ save new file blocks
	*ext-buff ->ind-logical-end !	\ save new logical end

	l_*ind_block @			\ block
	*ext-buff			\ buffer
	i 				\ start
	i INDIR-EXTENT +		\ end
	write-p-blk drop
	l_fid fcb-addr update-fcb -> l_oc
	leave
				
      endif
    else
      leave
    endif
  INDIR-EXTENT +loop
  l_oc
;

: get-indirect-extent	{ l_fid l_remaining | l_indi l_req l_hd_block -- phys  }
\ *G *\b{l_remaining} is the number of blocks remaining - 1.
\ ** Return the physical address of the logical block via indirection.
  -1 -> l_hd_block
  l_fid fcb-addr ->indirect-block @ -> l_indi
  l_fid get-r/w-b-ptr -> l_req
\ cr ." Seeking " l_req .decimal ." remaining " l_remaining .decimal 
  INDIR_SIZE 0 do	
    l_indi				\ block
    *ext-buff				\ buffer
    i 					\ start
    INDIR-EXTENT			\ size
    read-p-blk if
      *ext-buff ->ind-extent-start @ 0<> if	\ indirect extent defined
        l_req *ext-buff ->ind-logical-end @ - dup 0< if		\ get selected block start
          drop
	  *ext-buff ->ind-extent-start @ l_remaining + -> l_hd_block
\         cr ." Indir block @" l_hd_block .decimal l_remaining .decimal
	  leave
        else
	  -> l_remaining
        endif
      else
        leave				\ non defined
      endif
    else
      leave				\ block read error
    endif
  INDIR-EXTENT +loop
  l_hd_block ;


: delete-indir-extent	{ l_indirect_blk l_start -- }
\ *G Deletes all associated indirect blocks.
  INDIR_SIZE 0 do			\ find used indirect blocks
    l_indirect_blk			\ block
    *ext-buff				\ buffer
    i  					\ start
    INDIR-EXTENT			\ size
    read-p-blk if
      *ext-buff ->ind-extent-start @ 0<> if	\ indirect extent defined
        *ext-buff ->ind-extent-start @		\ get physical start
\       cr ." Releasing from " dup .decimal
        *ext-buff ->ind-logical-end @ l_start -	\ get length
\       dup .decimal ." blocks " 
        free-blocks
        *ext-buff ->ind-logical-end @ -> l_start 
      else
        leave				\ not defined
      endif
    else
      leave				\ block read error
    endif
  INDIR-EXTENT +loop
  l_indirect_blk #1 free-blocks		\ release indirect list block
;


\ *******************
\ *S Extending a file
\ *******************

: annex-file	{ l_blocks l_fid | l_buff l_start l_last_extent -- t/f }
\ *G Extends a file by the given number of blocks, returning true
\ ** for success.
  l_blocks l_fid new-extent-needed -> l_blocks
  l_blocks 0> not			\ do we actually need any extra blocks ?
  if  TRUE exit  endif

  FILE-EXTENSION allocate
  if  drop false  exit  endif
  -> l_buff

  drive-control request
  l_fid fcb-addr ->file-expansion @ if	\ check allowed extension
    l_fid l_buff ->ext-fid !		\ save fid
    l_fid fcb-addr ->file-expansion @ l_blocks max	\ allocate required or minimum space
\ cr ." Getting " dup .decimal
    l_buff ->ext-size !			\ save size
    l_buff next-extent? -> l_last_extent
    l_buff ->ext-size @ need-blocks -> l_start
    l_start l_buff ->ext-start !
    l_start -1 <> if
\ ." blocks @ " l_start .
      l_buff add-to-existing-extent? if
\ cr ." add to extent "
        l_buff extend-extent		\ add to existing extent
        l_fid fcb-addr update-fcb	\ now update FLD on disk
      else
        l_last_extent if
\ cr ." new extent "
          l_buff set-extent		\ create new extent
          l_fid fcb-addr update-fcb	\ update FLD on disk
        else
\ ." indirect extents "
          l_buff l_fid need-indirect-extent dup 0= if
            l_start l_buff ->ext-size @ free-blocks \ failed
          endif
        endif
      endif
    else
\ ." no free blocks "
      false
    endif
  else
\ ." Not allowed to increase "
    false
  endif
  drive-control signal
  l_buff ?free
;


\ ======
\ *> ###
\ ======

decimal


