\ FCB.FTH - File Control Block words.

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ ======
\ *! fcb
\ *T File Control Blocks
\ ======

\ ************************
\ *S Variables and buffers
\ ************************

SYS_NFCB FLD-REC * equ FCB-SIZE	\ -- len
\ *G Size of FCB table.
FCB-SIZE buffer: FCB-TABLE	\ -- addr
\ *G RAM table containing FCBs loaded from disk FLDs.

TASK_NCFILE open-table-struct * equ OPENT-SIZE	\ -- len
\ *G Size of open file table.
OPENT-SIZE buffer: OPEN-TABLE	\ -- addr
\ *G RAM table of open file structures.

FLD-REC cell - equ FCB_DATA_REC	\ -- len
\ *G Size of data in an FCB/FLD. The first element in the structure
\ ** must be preserved as it is the link field.
FLD-REC buffer: cdir-buffer	\ -- addr
\ *G Scratch FCB/FLD buffer.

variable active-fcbs	\ -- addr
\ *G Anchors chain of FCBs in use.
variable free-fcbs	\ -- addr
\ *G Anchors chain of unused FCBs.
variable active-otbs	\ -- addr
\ *G Anchors chain of open file structures in use.
variable free-otbs	\ -- addr
\ *G Anchors chain of unused open file structures.


\ ******************
\ *S Read/Write FLDs
\ ******************

: fld-d-block	\ fld -- blk
\ *G Returns disk block of required FLD.
  FLD/BLOCK / FLD0_BLOCK +
;

: fld-m-block	\ fld -- offset
\ *G Returns block offest of required FLD
  FLD/BLOCK mod FLD-REC *
;

: rewrite-fld	{ l_fld l_buff -- t/f }
\ *G Update specific FLD to disk via cache
  l_fld fld-d-block		\ disk block
  l_buff			\ destination
  l_fld fld-m-block		\ start
  dup FLD-REC +			\ end
  write-p-blk
;

: read-in-fld { l_fld l_buff | -- t/f }
\ *G Gead in specific FLD from disk via cache
  l_fld fld-d-block		\ disk block
  l_buff cell+			\ destination
  l_fld fld-m-block cell+	\ start
  FCB_DATA_REC			\ size
  read-p-blk
;


\ *****************
\ *S FCB primitives
\ *****************

\ Redo to avoid bug when ACTIVE-FCBS contains 0.
: existing-fcb?	\ fld# -- struct|0
\ *G If the FLD number is already in RAM, return its address,
\ ** otherwise return zero.
  active-fcbs			\ test active chain
  begin				\ -- fld s2
    2dup ->fld-no @ <>		\ see if file already open 
   while
    ->next-fcb @
    dup 0= if			\ if at end of chain
      nip exit
    endif			\   get out quietly
  repeat
  nip				\ lose fld
;

(( unused
: fn_open?	\ fn -- t/f
\ use the Fn to find if named file is open
  existing-fcb? 0<>	\ find out if there's already a file control block.
;
))

: get-fcb	\ -- struct|0
\ *G Get a new FCB structure and return its address or 0 if no
\ ** free FCBs are available.
  free-fcbs @ dup if
    [i					\ make this safe!
    dup @ free-fcbs !			\ unhook from free chain
    active-fcbs @ over ->next-fcb !	\ add to active chain
    dup active-fcbs !
    i]
  endif
;

: create-fcb	{ l_fld | l_fcb -- FCB|0 }
\ *G Return file control block address if successful, otherwise
\ ** return 0.
  l_fld existing-fcb? ?dup 0= if	\ already there, return struct
    get-fcb dup if			\ 1. find free FCB
      -> l_fcb
      l_fld l_fcb read-in-fld drop
      l_fld l_fcb ->fld-no !		\ FLD in the fcb
      1 l_fcb ->open-tables !		\ number of tasks using this file
      l_fcb
    endif
  else
    \   ." Already open "
    dup ->open-tables incr
  endif
;

: release-fcb	\ struct --
\ *G Put an FCB structure back on the free chain; exit quietly
\ ** if the FCB is not on the active chain.
  active-fcbs
  begin                                 \ -- struct s2
    2dup ->next-fcb @ <>                \ ->next-fcb MUST be first
   while
    ->next-fcb @
    dup 0=				\ if at end of chain
    if  2drop  exit  endif		\   get out quietly
  repeat
\ struct prev -- ; take the next link, and place it in the previous link
  over ->next-fcb @  swap ->next-fcb !
  free-fcbs @ over ->next-fcb !		\ add to free chain
  free-fcbs !
;

: dec-fcb-open	\ fcb -- t/f
\ *G Decrement the number of open instances of this FCB.
\ ** Returns true if all access closed, i.e. no instance
\ ** is still open.
  ->open-tables dup @ dup if
    1- tuck swap !
    0=                  		\ all closed ?
  else
    2drop
    true                		\ already closed
  endif
 ;

: init-fcbs	\ --
\ *G Initialise the table of FCBs.
  fcb-table FCB-SIZE erase		\ clear file control block table
  0  fcb-table FCB-SIZE bounds
  do  i ->next-fcb !  i  FLD-REC +loop
  free-fcbs !
  0 active-fcbs !
;


\ ***************************
\ *S Open Table Blocks (OTBs)
\ ***************************
\ *P Because a file may have been opened more than once, some
\ ** data must be held for each open instance. An Open Table Block
\ ** holds the instance data and the address of the FCB for the
\ ** file. The OTB address is used as handle (FID) of the file.

: init-otbs	\ --
\ *G Initialise the OTB structures.
  OPEN-TABLE OPENT-SIZE erase     	\ clear file open table
  0  OPEN-TABLE OPENT-SIZE bounds do
    i ->next-otb !
    i ->next-otb
  OPEN-TABLE-STRUCT +loop
  free-otbs !
  0 active-otbs !
;

: create-otb     \ -- fid|0
\ *G Get a free OTB and return its address or 0 if one cannot
\ ** be found.
  free-otbs @ dup if
    [i					\ make this safe!
    dup ->next-otb @ free-otbs !	\ unhook from free chain
    active-otbs @ over ->next-otb !	\ add to active timer chain
    VALID_OTB over ->valid-otb !
    dup active-otbs !
    i]
  endif
;

: release-otb	\ fid --
\ *G Put the OTB back on the free chain if it is active, 
\ ** otherwise do nothing.
  active-otbs
  begin					\ -- struct s2
    2dup ->next-otb @ <>		\ MUST be first
   while
    ->next-otb @ dup 0=			\ if at end of chain
    if  2drop  exit  endif		\   get out quietly
  repeat
\ struct prev -- ; take the next link, and place it in the previous link
  over ->fcb-number off
  over ->valid-otb off
  over ->next-otb @  swap ->next-otb !
  free-otbs @ over ->next-otb !		\ add to free chain
  free-otbs !
;

compiler
: fcb-addr	\ fid -- fcb
\ *G COMPILER: Returns the FCB address from the FID.
  ->fcb-number @
;
    
: fid-file-size	\ fid -- size
\ *G COMPILER: Returns the file size given the FID.
  fcb-addr ->file-size @
;

: set-f-r/w-ptr	\ value fid --
\ *G COMPILER: Set the file read/write pointer.
  ->r/w-data-ptr !
;
target

: get-f-r/w-ptr	\ fid -- value
\ *G Get the file read/write pointer.
  dup if
    ->r/w-data-ptr @
  endif
;

: get-r/w-b-ptr	\ fid -- block
\ *G Get the n-th block in the file which includes the data at
\ ** the read/write pointer.
  dup if
    ->r/w-data-ptr @ BSdiv
  endif ;

: get-r/w-pb-ptr	\ fid -- offset
\ *G Get the offset in the block in the file which includes the
\ ** data at the read/write pointer.
  dup if
    ->r/w-data-ptr @ BSmod
  endif
;

(( unused
: next-f-r/w-ptr	\ fid --
  dup if
    ->r/w-data-ptr BLOCK_SIZE swap +!
  endif
;
))

compiler
: inc-f-r/w-ptr	\ step fid --
\ *G Add *\b{step} to the file's read/write pointer.
\  ->r/w-data-ptr tuck @ + swap !
  ->r/w-data-ptr +!
;
target

: r/w-ptr-inrange?	\ fid -- t/f
\ *G Return true if the file read/write pointer is within
\ ** the number of allocated blocks.
  dup get-r/w-b-ptr
  swap fcb-addr ->file-blocks @ <=
;

: r-ptr-inrange?	\ fid -- t/f
\ *G Return true if the file read/write pointer is within
\ ** the current file size.
  dup get-f-r/w-ptr
  swap fid-file-size <=
;

(( moved to file-main.fth
: r/w-block	{ l_fid | l_hd-block l_block l_fcb -- phys-blk } 
    -1 -> l_hd-block
    l_fid get-r/w-b-ptr -> l_block  
    l_fid fcb-addr -> l_fcb
    l_fcb ->file-blocks @ l_block > if  \ test if pointer inside allocated blocks
        l_fcb ->extent-start-0 EXTENT_LIST bounds do
            i ->extent-size @ ?dup if   \ get extent size
                l_block swap - dup 0< if \ found extent
                    drop        
                    i ->extent-start @ l_block + \ get extent start 
                    -> l_hd-block
                    leave
                else
                    -> l_block          \ rem-blocks
                endif
            else
                leave
            endif
        FILE-EXTENTS +loop
        l_fcb ->indirect-block @ 0<>    \ indirect extent defined
        l_hd-block -1 = and if          \ not found so far
            l_fid l_block get-indirect-extent -> l_hd-block
        endif
    else
        -2 -> l_hd-block        	\ need more blocks allocated
    endif
    l_hd-block
;
))


\ ======
\ *> ###
\ ======

decimal


