\ FSconfig.fth - default file system configuration

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ ===========
\ *! fsconfig
\ *T Configuring the file system
\ ===========
\ *P The file *\i{<FileSys>\FSconfig.fth} contains the default
\ ** configuration for the file system. To set your own configuration,
\ ** copy *\i{<FileSys>\FSconfig.fth} and rename it. You can put it
\ ** where you like, so placing it in your main application source
\ ** folder is sensible. Then compile your file *\b{before} 
\ ** *\i{FileSys.bld}, which will ignore the default file if
\ ** it finds a previously loaded configuration.


\ **************
\ *S Diagnostics
\ **************

1 equ FSdiags?	\ -- n
\ *G Set non-zero to compile diagnostic and test code.


\ ********************
\ *S Sector/Block size
\ ********************
\ *P The file system operates on fixed-size blocks on the disk.
\ ** The size of each block is normally 512 bytes. It can be
\ ** changed as required. If the block size is changed, you
\ ** will need to redefine the compiler macros in this section.
\ ** These macros use shift and logical operations on the 
\ ** assumption that these are faster than division operations.
\ ** You can change these macros to suit your CPU architecture,
\ ** or replace them by words if the block size is not a power
\ ** of two.

#512 equ BLOCK_SIZE	\ -- u
\ *G Block size in bytes on disk.

compiler
: BSdiv		\ size -- #blocks
\ *G Given a length, return the number of complete blocks required.
\ ** Equivalent to:
\ *C   BLOCK_SIZE /
  9 rshift
;

: BSmod		\ size -- rem
\ *G Given a length, return the number of bytes in the last block.
\ ** Equivalent to:
\ *C   BLOCK_SIZE mod
  BLOCK_SIZE 1- and
;

: BS/mod	\ size -- rem #blocks
\ *G Given a length, return the number of bytes in the last block
\ ** and the number of complete blocks. Equivalent to:
\ *C   BLOCK_SIZE /mod
  dup BSmod  swap BSdiv
;

: BSblks	\ size -- #blocks
\ *G Given a size in bytes, return the number of blocks required
\ ** to contain them. Unlike *\fo{BSdiv}, this allows for a partly
\ ** used last block.
  BLOCK_SIZE 1- +  BSdiv
;
target


\ *******************
\ *S Disk information
\ *******************
\ *P These equates affect the proportion of the disk used for
\ ** management and how much RAM is required.

#7 equ SYS_NFCB	       		\ max number of different concurrently open files
#14 equ TASK_NCFILE    		\ max number of concurrent open files

#10 equ NO-OF-EXTENTS		\ DO NOT CHANGE

\ these are predefined
00 equ BOOT_BLOCK0
01 equ BOOT_BLOCK1
02 equ ROOT_BLOCK
03 equ ROOT_DIR_BLOCK
04 equ USAGE_BLOCK

01 equ DIR_BLOCK_SIZE

\ flag in FLD to show file type
$5a01 equ FILE-FTYPE
$5a33 equ DIR-FTYPE
$5a58 equ SYS-FTYPE

((
#16 equ MIN_EXP_SIZE
MIN_EXP_SIZE #1024 *	equ MIN_EXPANSION
0			equ NO_EXPANSION
))


\ ****************
\ *S Filing system
\ ****************

BLOCK_SIZE equ MIN_FILE_SIZE	\ -- u
\ *G Defines how much space is initially allocated when a file is
\ ** created.

#32 equ MAX_FILES	\ -- u
\ *G Maximum number of files on the drive.
3 equ PATH-LIMIT	\ -- u
\ *G Maximum nesting of directory levels.

char / equ DIR-DELIMIT	\ -- char
\ *G Defines the character used as a directory name delimiter.

char . equ THIS-DIR	\ -- char
\ *G Defines the character used to mean "this directory".

#11 equ PATH-LENGTH	\ -- u
\ *G Maximum length of a file name.


\ *************
\ *S Disk Cache
\ *************
\ *P The disk can operate with a choice of caching modes, which
\ ** provide trade-offs between disk performance, RAM usage,
\ ** determinism and interrupt latency.

0 equ NoDiskCache	\ -- 0
\ *G No Disk Cache. All disk activity is to the drive. Lowest RAM
\ ** usage and lowest performance. This may be the best solution
\ ** for systems with limited RAM and drives with their own cache.
1 equ CloseDiskFlush	\ -- 1
\ *G The disk cache is only written back on demand by the word
\ ** *\fo{SYNC-VOL}, or when a file is closed, or when a directory
\ ** is created.
2 equ TaskDiskFlush	\ -- 3
\ *G A separate task *\fo{DiskTask} executes *\fo{SYNC-VOL} when
\ ** required. When data is written out depends on *\fo{SyncMs} 
\ ** and *\fo{SyncOnIdle} below.

CloseDiskFlush equ DiskCacheMode	\ -- n
\ *G Define the required disk cache mode.

#4 equ #CACHE_BLOCKS	\ -- n
\ *G Number of disk blocks that are cached in RAM.

#250 equ SyncMs	\ -- ms
\ *G Time in milliseconds after a write after which the disk cache
\ ** will be flushed.
0 equ SyncOnIdle	\ -- n
\ *G If non-zero *\fo{SYNC-VOL} is executed *\fo{SyncMs}
\ ** milliseconds after the last disk write, otherwise
\ ** it is executed *\fo{SyncMs} milliseconds after the
\ ** first disk write.

#15000 equ VOL_TIMEOUT	\ -- ms
\ *G Number of milliseconds after a write before the cache is
\ ** flushed.

#240000 equ TAIL_TIMEOUT        \ four minutes


\ *************************
\ *S Disk information block
\ *************************
\ *P The file system was originally written to use CompactFlash
\ ** cards or ATAPI drives. These return a disk identification
\ ** block in little endian format. See *\i{Specs\cfspec1_4.pdf}
\ ** Table 40 for the gory details. This format is retained for
\ ** all hardware, regardless of host CPU and attached
\ ** hardware. At present, the only *\b{required} data is the
\ ** number of sectors on the disk.

#57 2* equ ID.#SECTORS	\ -- offset
\ *G Offset of the 32 bit number of sectors on the disk.


\ ***********
\ Global data - do not change
\ ***********

0 value DISK_SIZE
0 value FLD0_BLOCK
0 value FLD-SIZE
0 value USAGE_BLOCK0
0 value USAGE_TABLE_SIZE
0 value DATA_BLOCK0

BLOCK_SIZE buffer: bit_block_buffer


\ ***********************
\ *S End of configuration
\ ***********************
\ *P This must be the last section of the configuration file.

1 equ FSconfigured?	\ -- n
\ *G Must be defined non-zero after all configuration information
\ ** has been defined.


\ ======
\ *> ###
\ ======

decimal


