\ FSdiags.fth - File system diagnostic tools

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ ======
\ *! fsdiags
\ *T File system diagnostic tools
\ ======
\ *P The diagnostic code requires the file FSTOOLS.FTH, which is
\ ** compiled if the equate *\fo{FSdiags?} is set non-zero.

: sectors>kb	\ #blks -- kb
  BLOCK_SIZE #1024 */
;

: df-v		\ --
\ *G Display disk information. N.B. The free space information
\ ** is incorrect for small drives.
\ ARM - 536 bytes
  base @ >r  decimal
  cr ." Disk size " DISK_SIZE sectors>kb 0 .r ." kb"
  cr
  BLOCK_SIZE allocate if
    drop ." Alloc error " exit
  endif
  0 swap				\ -- #free buff
  USAGE_BLOCK0 USAGE_TABLE_SIZE bounds ?do
    i over read-hd-block drop		\ read in block
    BLOCK_SIZE 0 do
      dup i + @ -1 <> if		\ area not totally used
        dup i + @
        #32 0 do			\ -- #free buff x
	  1 i lshift over and 0=
	  if  rot 1+ -rot  endif
        loop
	drop
      endif
    cell +loop
  loop
  ?free
  sectors>kb dup 0 .r ." kb, "
  #100 DISK_SIZE */ 0 .r ." % free"
  cr
  r> base !
;

: list-fcb	\ --
\ *G Display all open files.
  active-fcbs
  cr
  ." FCB   FLD Op T   Blk   loc0  loc1  loc2  loc3  loc4  loc5  loc6  loc7  exp" 
  begin
    @ ?dup
   while
    dup cr  .hex space
    dup ->fld-no @ .
    dup ->open-tables @ .
    dup show-file-details
  repeat
  cr
;

: list-otb	\ --
\ *G Display all open file handle details.
  active-otbs
  begin
    @ ?dup
   while
    cr
    dup ." OTB " .hex
    dup ->fcb-number @ dup ." FCB " .hex
    ->fld-no @ ." FLD " .decimal
    dup ->r/w-data-ptr @ ."  R/W pointer " .
  repeat
  cr
;

: fdump		{ l_*name | l_handle l_*buff -- }
\ *G Dump (memory display) the file whose name (counted string) is given.
  l_*name count open_f dup 0= abort" Can't open "
  -> l_handle
  BLOCK_SIZE allocate abort" Can't allocate "
  -> l_*buff
  begin
    l_*buff BLOCK_SIZE erase
    l_handle get-f-r/w-ptr 
    l_*buff BLOCK_SIZE 2/ l_handle read_f if
      cr cr
      ." Block " dup BLOCK_SIZE /mod .decimal ." offset " .decimal ." Ptr " .decimal
      l_*buff BLOCK_SIZE 2/ dump
      cr ." Press any to continue or D to end " cr
      case  key UPC
        [char] D of  true  endof
        [char] S of
          0 l_handle lseek_f
	  if  ." Back to start" endif
          false
	endof
        [char] E of
          -513 l_handle lseek_f
	  if ." Moving to last 512 bytes" endif
          false
	endof
          false swap
      endcase
    else
      drop true
    endif
  until
  l_*buff ?free
  l_handle close_f drop
  ." Done "
;

((
: (show-ftype)	\ code --
  case
    FILE-FTYPE of  ."  F"  endof
    DIR-FTYPE  of  ."  D"  endof
    SYS-FTYPE  of  ."  S"  endof
  endcase
;
))

: (show-ftype2) ( code -- )
  case
    FILE-FTYPE of  ." -rw-rw-rw-"  endof
    DIR-FTYPE  of  ." drw-rw-rw-"  endof
    SYS-FTYPE  of  ." -r-s------"  endof
  endcase
;

[undefined] (show-time2) [if]
: (show-time2)	( n -- )  . ." lseconds "  ;
[then]

: show-file-details2	\ buff --
  base @ >r  decimal
  dup ->file-type w@ (show-ftype2)
  dup ->fld-no @ 4 .r space		\ show fld
  ." S9040    " ." group   "
  dup ->file-size @ 8 .r space
  dup ->file-last-mod @ (show-time2)
  drop
  r> base !
;

: list		{ | l_buff l_fld-buff -- }
\ *G Display a directory listing in UNIX format.
  BLOCK_SIZE allocate abort" Alloc error "
  -> l_buff
  FLD-REC allocate abort" Alloc error "
  -> l_fld-buff
  working-dir-blk l_buff read-hd-block 0= if
    ." Error reading "  l_buff ?free exit
  endif
  0
  begin
    dup l_buff + ->file-name @ dup 0<> swap -1 <> and if
      cr
      dup l_buff + ->file-number @ l_fld-buff read-in-fld drop
      l_fld-buff show-file-details2
      dup l_buff + ->file-name count space type
    endif
    dir-struct +
    dup BLOCK_SIZE >=
  until
  drop cr
  l_fld-buff ?free
  l_buff ?free
;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\ a clatter of test words for file extensions

((
variable jerry1
variable jerry2

: ccreate ( -- )
	s" fred" 100 100 create_f .
	s" billy" 100 100 create_f .
	s" fred" open_f jerry1 !
	s" billy" open_f jerry2 ! ;

: cdelete ( -- )
	s" fred" delete_f .
	s" billy" delete_f . ;

: ll { | l_*buff -- }
	$100 allocate if
		drop exit
	endif
	-> l_*buff
	00
	l_*buff $fe bounds ?do
		dup i c!
		1+
	loop
	drop
	l_*buff $fe jerry1 @ write_f .
	l_*buff ?free ;

$2fe equ TEST_REC_SIZE

: kk { | l_*buff -- }
	TEST_REC_SIZE allocate if
		drop exit
	endif
	-> l_*buff
	00
	l_*buff TEST_REC_SIZE bounds ?do
		dup i c!
		1+
	loop
	drop
	l_*buff TEST_REC_SIZE jerry2 @ write_f .
	l_*buff ?free ;

: jj
	#15 0 do
		." ll>" ll
		." kk>" kk cr
	loop ;
))


\ ======
\ *> ###
\ ======

decimal


