\ Tools.fth - File system command line tools

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ ======
\ *! fstools
\ *T Command line tools
\ ======


\ *****************************
\ *S File and Directory display
\ *****************************

variable u_tail-timer

: tail_f	{ l_*name l_len | l_handle l_*buff l_key-wait -- }
\ *G List/Type the given file in ASCII format. Suitable for ASCII files.
  l_*name l_len open_f dup 0= abort" Can't open "
  -> l_handle
  BLOCK_SIZE allocate abort" Can't allocate "
  -> l_*buff
  cr
  TAIL_TIMEOUT later u_tail-timer !
  begin
    l_*buff BLOCK_SIZE l_handle read_f ?dup if
      l_*buff over type
      BLOCK_SIZE = if
        begin
          key?
          u_tail-timer @ expired or
        until
        key? if
          case  key upc
            ACR      of  true  endof
            [char] E of
              cr ." ----- Skip ----- " cr cr
              #-512 l_handle lseek_f drop false
            endof
              false swap
          endcase
          TAIL_TIMEOUT later u_tail-timer !   \ reset timer
        else
          false
        endif
      else
        false
      endif
    else
      #250 later -> l_key-wait
      begin
        #25 ms
        key? l_key-wait expired or
      until
      key? if
        TAIL_TIMEOUT later u_tail-timer !
        key upc ACR =
      else
        false
      endif
    endif
    u_tail-timer @ expired or
  until
  l_*buff ?free
  l_handle close_f drop
  cr cr ." --- Done ------ "
;

: tail		\ -- ; TAIL <filename>
\ *G List/Type a file in ASCII format. Suitable for ASCII files.
  bl word count tail_f
;
    
: pwd		\ --
\ *G Display the current working directory.
  *cwd @ ?dup 0=
  if  DIR-DELIMIT emit  else  $.  endif
;

: cd		\ -- ; CD <name>
\ *G Change directory to that given by the following text (if any).
\ ** Then display the working directory.
  bl word count dup if 
    ch-dir 0=
    if  ." - Failed: "  endif
  else
    2drop
  endif
  pwd
;
	
: 5.r		\ n --
\ *G Display n as 5 digits plus a space.
  5 .r space
;

: .ExtData	\ addr --
\ *G Display the extent data starting at *\b{addr}.
   EXTENT_LIST bounds
   do  i @ 5.r  file-extents +loop
;

: show-file-details	\ fcb --
\ *G Displays extended file information from an FCB/FLD structure.
  base @ >r decimal
  dup ->file-blocks @ 5.r
  dup ->extent-start-0 .ExtData
  cr #8 spaces
  dup ->indirect-block @ 4 .r space
  dup ->file-expansion @ 4 .r space
  2 spaces
  dup ->extent-size-0 .ExtData
  drop
  r> base !
;

: ls		{ | l_buff l_fld-buff -- }
\ *G List the files in the working directory with a lot of
\ ** diagnostic information.
  BLOCK_SIZE allocate abort" Alloc error "
  -> l_buff
  FLD-REC allocate abort" Alloc error "
  -> l_fld-buff
  working-dir-blk l_buff read-hd-block 0= abort" Error reading "
  0
  cr ." Directory listing " pwd
  cr ." FLD Name      #Blks  Loc0  Loc1  Loc2  Loc3  Loc4  Loc5  Loc6  Loc7  Loc8  Loc9"
  cr ." -------------------------------------------------------------------------------"
  begin
    dup l_buff + ->file-name @ dup 0<> swap -1 <> and if
      cr
      dup l_buff + ->file-number @ 3 .r
      dup l_buff + ->file-name count path-length over - spaces type
      dup l_buff + ->file-number @ l_fld-buff read-in-fld drop
      l_fld-buff show-file-details
    endif
    dir-struct +
    dup BLOCK_SIZE >=
  until
  drop  cr
  l_fld-buff ?free
  l_buff ?free
;


\ *******************************
\ *S Host to target file transfer
\ *******************************
\ *P These tools are compiled if the Xmodem file transfer tools
\ ** have been compiled. They allow files to be copied to and from
\ ** the host over the *\fo{CONSOLE} (serial) link. The ANS file
\ ** interface and the Xmodem code in *\i{COMMON\XmodenTxRx.fth}
\ ** are required. Host file selection must be performed by the
\ ** host's communication tools. If you are using AIDE's PowerTerm
\ ** tool, this is built in.

[defined] RecvXmodem
[defined] Bin-Down
or [if]
0 value fh	\ -- fileid
\ *G temorary file handle
[then]

[defined] RecvXmodem [if]

: wf128		\ -- 
\ *G Write 128 bytes from the host to the target file given by
\ ** *\fo{FH}.
  X-buffer #128 fh write-file drop
  #128 cur-address +!
;

: Host>File	\ caddr len --
\ *G Copy a host file to the named target file, which will be
\ ** created using the name given by *\i{caddr/len}.
  ['] from-buffer >body @ >r
  assign wf128 to-do from-buffer
  r/w create-file if
    drop  cr ." Can't create file"
  else
    -> fh
    0 $7FFFFFFF RecvXmodem
    cr ." Transfer "
    if  ." failed, "  else ." good, "  endif
    . ." bytes transferred"
    fh close-file drop
  endif
  r> to-do from-buffer
;

[then]

[defined] Bin-Down [if]

: rf128		\ -- 
\ *G Read 128 bytes from the target file given by *\fo{FH}.
  X-buffer #128 $04 fill		\ pad with ^D
  X-buffer #128 fh read-file 2drop
  #128 cur-address +!
;

: File>Host	\ caddr len --
\ *G Copy the named target file to the host.
  ['] from-buffer >body @ >r
  assign rf128 to-do to-buffer
  r/o open-file if
    drop  cr ." Can't open file"
  else
    dup -> fh  file-size nip if		\ -- usize
      drop  cr ." Can't get file size"
    else
      dup 0= if
        cr ." Empty file"
      else
        cr ." Transferring " dup . ." bytes"
        0 swap Bin-Down
      endif
    endif
    fh close-file drop
  endif
  r> to-do from-buffer
;

[then]


\ ======
\ *> ###
\ ======

decimal


