\ HARNESS.FTH - CEM File System interface harness

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under licence by:
Modifications Copyright (c) 2002, 2003, 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
))

decimal


\ ==========
\ *! harness
\ *T Portability harness
\ ==========
\ *P The file *\i{%FileSysDir%\Harness.fth} provides portability
\ ** and architecture specific definitions for the file system.


\ ****************
\ *S Miscellaneous
\ ****************

[undefined] y/n? [if]
: y/n?		\ -- flag
\ *G Wait for a key, echo it and return true if it was a 'Y'.
  key dup emit upc [char] Y =
;
[then]

[undefined] .hex [if]
Synonym .hex .dword	\ n --
\ *G Display n as an 8 digit hex number.
[then]

[undefined] .decimal [if]
: .decimal	( n -- )	base @ swap  decimal  0 .r  base !  ;
\ *G Display in decimal.
[then]

2 field-type short	\ n -- n+2 ; addr -- addr+n
\ *G Defines a 2-byte field in a structure definition.


\ **********************************
\ *S Little Endian memory operations
\ **********************************
\ *P These words are usually implmented as compiler macros, and
\ ** are provided for interchange of storage media between big
\ ** and little endian media. They are also necessary when using
\ ** CompactFlash, ATA or ATAPI drives to avoid endian and data
\ ** alignment problems.

big-endian? [if]
compiler
: c!(le)	\ b caddr --
\ *G Little endian byte store.
  c!
;

: w!(le)	\ w caddr --
\ *G Little endian 16 bit unaligned word store.
  2dup c!  swap 8 rshift swap 1+ c!
;

: l!(le)	\ x caddr --
\ *G Little endian 32 bit unaligned store.
  2dup w!(le)  swap #16 rshift swap 2+ w!(le)
;

: c@(le)	\ caddr -- b
\ *G Little endian byte fetch.
  c@
;

: w@(le)	\ caddr -- w
\ *G Little endian unaligned 16 bit word fetch.
  dup c@  swap 1+ c@ 8 lshift or
;

: l@(le)	\ caddr -- x
\ *G Little endian unaligned 32 bit word fetch.
  dup w@(le)  swap 2+ w@(le)  #16 lshift or
;
target
[then]

little-endian? [if]
compiler
: c!(le)	\ b caddr --
  c!
;

: w!(le)	\ w caddr --
\ *G Little endian 16 bit unaligned word store.
  w!
;

: l!(le)	\ x caddr --
\ *G Little endian 32 bit unaligned store.
  2dup w!  swap #16 rshift swap 2+ w!
;

: c@(le)	\ caddr -- b
\ *G Little endian byte fetch.
  c@
;

: w@(le)	\ caddr -- w
\ *G Little endian unaligned 16 bit word fetch.
  dup c@  swap 1+ c@ 8 lshift or
;

: l@(le)	\ caddr -- x
\ *G Little endian unaligned 32 bit word fetch.
  dup w@(le)  swap 2+ w@(le)  #16 lshift or
;
target
[then]


\ *******************
\ *S File system data
\ *******************

[undefined] lseconds [if]
variable lseconds	\ -- addr
\ *G Holds UNIX LSECONDS since ...
[then]

cell +user DIR-BLOCK	\ -- addr
\ *G Pointer to a directory block.
cell +user *CWD		\ -- addr
\ *G Pointer to current work directory name structure.

\ Access semaphores
semaphore drive-access	\ -- addr
\ *G Interlock for access to the low level disk driver.
semaphore cache-sema	\ -- addr
\ *G Interlock for access to the disk cache software.
semaphore drive-control	\ -- addr
\ *G Interlock for access to the drive management software.
semaphore fld-update
\ *G Interlock for access to the file management software.

: init-sema	\ sema --
\ *G Initialise a counted semaphore.
  0 1 rot 2!				\ count=1, owner=0
;

compiler
: working-dir-blk	\ -- addr
  dir-block @
;

: set-working-dir	\ addr --
  dir-block !
;
target

decimal


\ *******
\ *S Heap
\ *******

: allocate	\ u -- addr ior
\ *G A redefinition of *\fo{ALLOCATE} that issues a console
\ ** warning message if it fails.
  allocate dup
  if  cr ." ALLOCATE failed"  endif
;

: free		\ addr -- ior
\ *G A redefinition of *\fo{ALLOCATE} that issues a console
\ ** warning message if it fails.
  free dup
  if  cr ." FREE failed"  endif
;

: ?free		\ addr -- 
\ *G A convenvenient version of *\fo{FREE} that does nothing
\ ** if addr is zero and does not return an error code.
  ?dup
  if  free drop  endif
;


\ **********
\ *S Strings
\ **********

: extract-string	\ caddr len char -- caddr' len'
\ *G Given a string return the substring between the delimiters,
\ ** ignoring leading delimiters.
  dup >r skip  2dup r> scan  nip -
;

: jump-string	\ caddr len char -- caddr' len'
\ *G Ignore leading delimiters, and return the string to the
\ ** right of the delimiter character including the delimiter.
  dup >r skip  r> scan
;


\ ======
\ *> ###
\ ======

decimal

