
#30 #60 * equ WORLD-TZ-ADJUST   \ convert to 1/2 hour offsets

: adjust-for-world ( -- change )
  world-tz WORLD-TZ-ADJUST / $ff and ;

: show-time ( -- )
  lseconds @ (show-time) ;

: (show-time2) ( lseconds -- )  \ displays in UNIX file format
  dup get-date base @ decimal
  month @ disp-month date @ 2 .0r
  swap longtotime drop space
  2 .0r ." :" 2 .0r drop
  base ! ;

: disp-month ( n -- )
  case
    1   of ." Jan" endof
    2   of ." Feb" endof
    3   of ." Mar" endof
    4   of ." Apr" endof
    5   of ." May" endof
    6   of ." Jun" endof
    7   of ." Jul" endof
    8   of ." Aug" endof
    9   of ." Sep" endof
    #10 of ." Oct" endof
    #11 of ." Nov" endof
    #12 of ." Dec" endof
  endcase
  space ;
    
: (show-date) ( ls -- )
  get-date base @ decimal
  date @ 2 .0r ." /" month @ 2 .0r ." /" year @ .
  base ! ;

: (show-time) ( lseconds -- )
  dup (show-date)
  [show=time] ;

: [show=time] ( lseconds -- )
  base @ decimal
  swap longtotime d-of-week show-day space
  2 .0r ." :" 2 .0r ." :" 2 .0r
  base ! ;

: longtotime ( ls -- sec min hr da )
  world-tz +
  #60 /mod    \ seconds
  #60 /mod    \ mins
  #24 /mod    \ hours
  7 /mod      \ day
  drop ;      \ no. of weeks

: (d-since-0)   ( lseconds -- n )
  world-tz +
  #60 /mod    \ seconds
  #60 /mod    \ mins
  #24 /mod    \ hours
  nip nip nip ;

\ no of days since 0 hour, used for holidays
: d-since-0 ( -- n )    
        Lseconds @ (d-since-0) ;

: d-of-week	\ dow -- dow ; assume monday is day zero
                \ no thursday is day zero 2/3/95
  4 - dup 0< if 7 + endif	\ adjust day number, 0 was THURSDAY 
  mask ;                        \ generate bit mask 


: min-in-day    ( -- min )      \ used for offline timezone table
        Lseconds @ longtotime
        drop                		\ lose day of week
        #60 * +
        nip ;               		\ lose seconds

: mask ( n -- bp )
        1 swap lshift ;

: whichday  ( -- )
        Lseconds @ longtotime
        nip nip nip
        d-of-week
        show-day ;
        
: show-day ( N -- )
        case
            01 of ." Mon" endof \ Mo "
            02 of ." Tue" endof \ Tu "
            04 of ." Wed" endof \ We "
            08 of ." Thu" endof \ Th "
            10 of ." Fri" endof \ Fr "
            20 of ." Sat" endof \ Sa "
            40 of ." Sun" endof \ Su "
        endcase ;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
variable date
variable month
variable year

#365 equ d/year        \ number of normal days in year
#214 equ 1/2year       \ number of days in "half year"
#59 equ d_to_march     \ number of days to after leap year
#58 equ d_to_leap      \ number of day to leap year extra
#61 equ d/2month       \ number of days per two months 
#31 equ d/month        \ number of days per "month"
#0  equ first-mnth
#7  equ mid-mnth

\ table of days in year per month
create mon_tab  
#000 , #031 , #060 , #091 , #121 , #152 ,
#182 , #213 , #244 , #274 , #305 , #335 ,

\ --------------------------------------------------------------
hex

: get-date  ( lseconds -- )
        world-tz +
        get-year
        get-month ;

        
\ extracts number of days from longseconds and converts to year
: get-year ( h -- days-left-in-year )
        (d-since-0)         \ get number of days since longsec start
        (get-year) ;
                
: (get-year) ( d_since_0 -- days_left )
        d/year /mod     \ d, y
        tuck 1+ 4 /     \ work out no. of leap days since 1970
        -               \ remove from day count
        dup 0< if       \ at end of year, 
            over 3 - 4 mod 0= if    \ causes problem at end of leap year
                1+
            endif
            d/year +    \ adjust day
            swap 1-     \ adjust year
        else
            swap        \ year on top
        endif
        dup #1970 + year !      \ save year
        2- 4 mod 0<>            \ find non leap years
        over d_to_leap > and if \ AND after feb 28
            1+                  \ add extra day
        endif ;

: get-month ( days_left -- )
        dup d_to_march > if
            1+
        endif
        dup 1/2year mod                 \ days in half year
        swap 1/2year >= if              \ days after July 31
            mid-mnth                    \ set month
        else
            first-mnth                  \ set month
        endif
        over                            \ get days in half year
        d/2month / 2* +                     \ month
        over                                \ get days in half year
        d/2month mod d/month / + 1+
        month ! \ month
        d/2month mod d/month mod 1+
        date ! ;    \ get days in month

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

\ convert normal time to longsec
\ the data is coming in as 2 digit BCD, from the clock chip
: get-lseconds { l_*struct | l_year l_month l_date -- longsec }
        l_*struct ->year c@ s-bcd>hex dup #70 < if
            #30 +               \ must be after 1999
        else
            #70 -               \ must be before 2000
        endif -> l_year
        l_*struct ->month c@ s-bcd>hex -> l_month
        l_*struct ->date c@ s-bcd>hex 1-
        l_month 1- cells mon_tab + @ + -> l_date    \ now add days in that month    [ y m d ]

        l_month 2 >                 \ get month, after feb      
        l_year 2+ 4 mod 0<> and if  \ adjust leap year  
            l_date 1- -> l_date                     
        endif
        l_year 1+ 4 / l_date + -> l_date            

        l_year #31536000 *                      \ add seconds in a year
        l_date #86400 * +                       \ add seconds in a day
        l_*struct ->hrs c@ s-bcd>hex #3600 * +  \ add seconds in hour
        l_*struct ->mins c@ s-bcd>hex #60 * +       \ add seconds in minute
        l_*struct ->secs c@ s-bcd>hex +         \ add seconds
        world-tz -
        ;                                           \       [ sl ]

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

: set-short-clock { | l_*struct -- }  \ convert from longseconds and set the clock chip
    rtc-struct allocate if
        exit
    endif
    -> l_*struct
    lseconds @ get-date        \ generate year, month and date from longseconds
    year @ #1900 - dup #99 > if
        #100 -
    endif
    s-hex>bcd l_*struct ->year c!   \ year
    month @ s-hex>bcd l_*struct ->month c!  \ month
    date @  s-hex>bcd l_*struct ->date c!   \ date
    lseconds @ longtotime
    s-hex>bcd l_*struct ->day c!    \ day off week
    s-hex>bcd l_*struct ->hrs c!    \ hours
    s-hex>bcd l_*struct ->mins c!   \ minutes
    s-hex>bcd l_*struct ->secs c!   \ seconds
    l_*struct set-clock             \ write to RTC chip
    l_*struct free drop
    ;


: set-long-clock { | l_*struct -- }  \ convert from longseconds and set the clock chip
    rtc-struct allocate if
        exit
    endif
    -> l_*struct
    l_*struct read-clock
    l_*struct get-lseconds lseconds !
    l_*struct free drop ;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
: bcd>hex ( bcd -- hex )
       $10 /mod
       $10 /mod
       $10 /mod
       $10 /mod
       $10 /mod
       $10 /mod
            #1000000 *
       swap #100000 * +
       swap #10000 * +
       swap #1000 * +
       swap #100 * +
       swap #10 * + 
            + ;

: s-bcd>hex ( bcd -- hex )
    $10 /mod #10 * + ;

: s-hex>bcd ( hex -- bcd )
    #10 /mod $10 * + ;

