\ DIR_PARSE.FTH - Filename parsing functions

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
20050407 CEM002 Incorporated CEM updates.
))

decimal

\ ===========
\ *! dirparse
\ *T Filename parsing
\ ===========

: get-path	\ *name len -- *name2 len2
\ *G Return the string up to (to the left of) the next directory
\ ** separator.
  DIR-DELIMIT extract-string
;

: get-next-path	\ *name len -- next-addr next-len
\ *G Return the string to the right of the first path, including
\ ** any director separator.
  DIR-DELIMIT jump-string
;

(( unused
: looking-file?	\ *name len -- t/f
\ \ *G Returns TRUE if looking for a file and FALSE for a directory.
  get-next-path nip 0=
;
))

: from-root?	\ *name len -- t/f
\ *G Returns true if the first character of the string is a 
\ ** directory separator.
  0<> if
    c@ DIR-DELIMIT =
  else
    drop false
  endif
;

: get-fid	{ l_*name l_q-len l_dir-buff | l_f-num l_wild -- file-num|0 }
\ *G Look for file name in a directory block buffer. If the file is found
\ ** return its index, otherwise return 0.
  false -> l_f-num
  l_*name s" *" s= if				\ wild card
    l_*name l_q-len [char] * skip -> l_q-len -> l_*name
    true -> l_wild
  else
    false -> l_wild
  endif
  begin
    l_wild if					\ wild card
      l_dir-buff ->file-name count l_*name l_q-len search nip nip 0<>
    else
      l_dir-buff ->file-name count l_*name l_q-len compare 0= \ read from directory list
    endif
    dup if					\ found
      l_dir-buff ->file-number @
\     cr ." Found" dup .word space
      -> l_f-num
    endif
    l_dir-buff DIR-STRUCT + -> l_dir-buff	\ next directory entry
    l_dir-buff ->file-number @ true = or	\ test last entry
  until
  l_f-num
;
		
: find-file?	{ l_dir-blk l_*name l_len | l_*buff -- fn|0 }
\ *G Look for file name in a directory block. If the file is found
\ ** return its index, otherwise return 0.
  BLOCK_SIZE allocate if
    drop 0 exit
  endif
  -> l_*buff
  l_dir-blk l_*buff 0 BLOCK_SIZE read-p-blk if	\ read in directory name block
    l_*name l_len get-path dup if
      l_*buff get-fid
    else
      \	." No path "
    endif
  else
    ." DIR read error "
    0
  endif
  l_*buff ?free
;

\ -----------------------------------------------------

: no-of-paths	\ caddr len -- n
\ *G Count the number of directory levels in file name.
  false -rot					\ returned if too many paths
  PATH-LIMIT 0 do				\ -- 0 caddr len
    2dup get-path nip 0= if
      rot drop					\ lose bad outcome
      i -rot					\ put number of steps
      leave
    endif
    get-next-path
  loop
  2drop
;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

: set-default-dir	\ --
\ *G Set the working directory to the root directory.
  ROOT_DIR_BLOCK set-working-dir
  *cwd @ ?free  *cwd off
;
	
: step-next-dir		\ *name len buff -- *name1 len1 dir-blk
\ *G Step to the next directory in the pathname (using the 
\ ** given directory block buffer), make that the current
\ ** directory, and return the remaining path name and directory
\ ** block number.
  -rot get-next-path
  rot ->extent-start-0 @
  dup set-working-dir
;

: get_fn	{ l_*name l_len | l_dir l_fld l_*fld -- fld-num|0 }
\ *G Given a path/file name, return the file index number if found
\ ** or zero if not found. Directory path numbers are negated.
  l_*name l_len from-root? if
    set-default-dir
  endif
  working-dir-blk -> l_dir
  FLD-REC allocate if
    drop FALSE exit
  endif
  -> l_*fld
  l_*name l_len no-of-paths 1 swap do
\ ." Path " i .
    l_dir l_*name l_len find-file? ?dup if \ read dir blk ; -- fld-num
      -> l_fld
      l_fld l_*fld read-in-fld drop
      l_*fld ->file-type w@
      case				\ read FLD and return 
        FILE-FTYPE of
\         ." file "
          i 1 <> if			\ not the end of path, but a file name 
            0 leave			\ return failure code
          else
	    l_fld leave
	  endif
        endof
        DIR-FTYPE of
\	  ." dir "
	  i 1 = if			\ last path section
            l_fld negate  leave		\ returns -ve FID number
	  else
	    l_*name l_len l_*fld step-next-dir
	    -> l_dir -> l_len -> l_*name
	  endif
        endof
\	  ." strange "
	  drop l_fld leave		\ probably a SYS file, or FLD missing !
      endcase
    else
\     ." not found "
      0 leave				\ file not found
    endif
  -1 +loop
  l_*fld ?free
;

: find-next-fld	{ | l_fld l_*buff -- fld-no }
\ *G Find the next free FLD number, returning -1 if there is
\ ** no more space, or 0 if buffer memory is exhausted.
  -1 -> l_fld
\ cr ." Finding next FLD entry "
  BLOCK_SIZE allocate if
    0 exit
  endif
  -> l_*buff
  FLD0_BLOCK FLD-SIZE bounds ?do	\ search all FLD blocks
\   cr ." block " i .
    i l_*buff read-f-blk 0= if
      leave
    endif
    BLOCK_SIZE 0 ?do
\     i FLD-REC / .
      i l_*buff + ->file-type w@ 0= if
        j FLD0_BLOCK - FLD/BLOCK * i FLD-REC / +
	-> l_fld
	leave
      endif
    FLD-REC +loop
    l_fld -1 <> if
      leave
    endif
  loop
  l_*buff ?free
  l_fld
;


\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
((
1) find first path F-NUM in root dir
2) lookup descriptor for that F-NUM
3) if it's file, open it
  3a) check for existing FCB
  3a1) NO, allocate FCB
  3a2) YES, use existing FCB
  3b) create task open table
4) if it's a Dir
  4a) ensure applied name not expecting a file.
  4b) get that dir table
  4c) get next path and repeat above.
))

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

(( unused
: break-dir	\ *name len --
  begin
    DIR-DELIMIT skip			\ find first \ 
    dup
   while
    2dup DIR-DELIMIT scan
    2swap
    2 pick - type cr
  repeat
  2drop
;
))

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
\ used for PWD function

((
: inc-str-len ( N -- )
  *cwd @ c@ +	*cwd @ c! ;
))
compiler
: inc-str-len	\ n --
\ *G COMPILER: increase size of working directory string.
  *cwd @ c+!
;
target

: add-delimit-str	\ --
\ *G Add a directory separator to the working directory string.
  *cwd @ count + 1- c@ DIR-DELIMIT <> if	\ CEM002
    DIR-DELIMIT *cwd @ count + c!
    1 inc-str-len
  endif					\ CEM002
;

: clr-dir-str	\ --
\ *G Clear the working directory string.
  0 *cwd @ c!
;

: append-dir-str	\ caddr len --
\ *G Append the string to the working directory string.
  tuck
  *cwd @ count + swap cmove	\ cat strings together
  inc-str-len 			\ increase length
;							
	
: move-back-dir	\ --
\ *G Move back one level in the working directory string.
  *cwd @ count
  1- 0 swap ?do
    dup i + c@ DIR-DELIMIT = if
      i *cwd @ c!
      leave
    endif
  -1 +loop
  drop
;

: back-dir?	\ *name -- t/f
\ *G Return true if the start of the name is '..'.
  s" .." s= ;

: remove-dir-hdr	\ *name len -- '*name 'len
\ *G Remove a directory header from the name.
  dup 3 min /string		\ short new string
;


\ ======
\ *> ###
\ ======

decimal


