\ file-bitmap.fth - disk sector usage

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ ======
\ *! fsbitmap
\ *T Sector Bitmap
\ ======
\ *P These functions are used for indicating disk usage.
\ ** Space is indicated by a bitmap where one bit represents
\ ** one block (512 bytes) on the media. When a block is in use,
\ ** the appropiate bit in the BITMAP table will be set to one.
\ ** For groups of 32 or more blocks, these groups will always
\ ** start on a multiple of 32 blocks.

\ *P The bitmap is set up so that block 0 is indicated by the
\ ** top bit of the first byte.

variable p-blocks	\ -- addr
\ *G Holds groups of less than 32 blocks required.
variable no-32		\ -- addr
\ *G Holds number of 32 block groups required.
variable b-ptr		\ -- addr 
\ *G Holds disk buffer search pointer.
variable grab/free	\ -- addr
\ *G Holds true for grab, false for release

BLOCK_SIZE 8 * equ BLOCKS/BIT_TABLE	\ -- n
\ *G Number of bits (blocks) per block.
BLOCKS/BIT_TABLE equ BLOCKS/GROUP	\ -- n
\ *G Synonym.
$FFFFFFFF equ 32BLOCKS	\ -- mask
\ *G Indicator for 32 blocks.


(( moved to STRUCTS.FTH
struct block-alloc
    int ->last-amount
    int ->#blocks
    int ->start-block
    int ->start-address
    int ->first-amount
end-struct
))

((
: .BlockAlloc	\ struct --
  cr ."   AllocBlock"
  cr ."   Last Amount " dup ->last-amount @ .
  cr ."   # blocks    " dup ->#blocks @ .
  cr ."   Start Block " dup ->start-block @ .
  cr ."   Start Addr  " dup ->start-address @ .
  cr ."   First Amount " dup ->first-amount @ .
  drop
;
))

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

((
compiler
: bitmask	\ n -- mask
\ *G Creates an n-bit mask in the low bits.
  1 swap lshift 1- 			
;
target

: topBitmask	\ n -- mask
  dup bitmask  #32 rot - lshift
;
))
: topBitmask	\ n -- pattern
\ *G Creates an n-bit mask starting at the highest bit.
\ *B  1 bit is $80000000
\ *B  4 bits is $f0000000
\ *B  16 bits is $ffff0000
  1 over lshift 1- 			
  swap #32 swap - lshift
;

: (find-partial-block) { l_data l_size | l_block-no -- start-block-no }
\ *G Finds partial block of SIZE anywhere in 32 blocks.
\ ** Returns -1 if it won't fit.
  -1 -> l_block-no
  l_size #32 ?do
    1 l_size lshift 1-  i l_size - lshift
    l_data and 0= if
      #32 i - -> l_block-no
      leave
    endif
  -1 +loop
  l_block-no
;

: find-<32blocks	\ -- start|-1
\ *G Search the current bitmap block for the number of blocks
\ ** held in variable *\fo{P-BLOCK}, returning -1 if NOT FOUND,
\ ** otherwise returning the start block.
\ *B no find of continue => -1 resets "start block" pointer, continue search
\ *B no find of small block => -1 continue search
\ *B find of continue => 0 all found and done
\ *B find of small block => ptr all found done, set "start block" pointer
  bit_block_buffer b-ptr @ + @ dup -1 <> if	\ read 32 blocks at pointer
\ ." some space "
    p-blocks @					\ number of disk blocks required
    no-32 @ if					\ continuation of blocks
      topBitmask 
      and 0<> 					\ test required area free
    else					\ small block on own
      \ ." on own "
      (find-partial-block) 
    endif
  endif
;

: find-free-blocks	{ l_req-blocks | l_start-block l_32-found -- start-block }
\ *G Find the required number of blocks, returning the start
\ ** block, or -1 if space cannot be found.
  l_req-blocks #32 /mod no-32 !		\ number of complete 32 blocks
  p-blocks !				\ number of partial 32 blocks
  0 -> l_32-found			\ number of 32 blocks found
  -1 -> l_start-block
  USAGE_BLOCK0 USAGE_TABLE_SIZE bounds ?do
\ cr ." Read " i u.
\    i bit_block_buffer read-hd-block drop	\ read in block
    i bit_block_buffer read-hd-block 0=	\ read in block
    if  -1  unloop exit  endif		\ get out on error
    b-ptr off				\ start block pointer
    begin
      no-32 @ l_32-found - 0<> if	\ complete 32 blocks to find
	bit_block_buffer b-ptr @ + @ 0= if	\ area free
	  l_start-block -1 = if		\ first of the free blocks
\ cr ." Found 1st " 
	    b-ptr @ 08 *
	    i USAGE_BLOCK0 - BLOCKS/BIT_TABLE * +
\ dup .
	    -> l_start-block		\ record it
	  endif
	  l_32-found 1+ -> l_32-found	\ another block found
	else
	  -1 -> l_start-block		\ cancel find and restart area search
	  0 -> l_32-found		\ 
	endif
      else
	find-<32blocks dup -1 = if	\ reset and continue search
	  -> l_start-block		
	  0 -> l_32-found		\ 
	else				\ all done
	  no-32 @ if			\ multiple blocks
\ ." Continue block free, done "
	    drop
	  else				\ single block 
\  ." Partial block free, done " dup .
	    b-ptr @ 08 * +
	    i USAGE_BLOCK0 - BLOCKS/BIT_TABLE * + -> l_start-block
	  endif
	  -1 -> l_req-blocks
	endif
      endif
      b-ptr @ cell+ dup b-ptr ! BLOCK_SIZE >=	\ these blocks searched
      l_req-blocks -1 = or		\ OR all required found
    until
    l_req-blocks -1 = if
      leave
    endif
  loop
  l_req-blocks -1 = if			\ Yes, find required
    l_start-block
  else
    -1					\ No, could not find
  endif
;

: block-order	{ l_start l_amount | l_buff -- buffer }
\ *G Set up the data structure required to grab or release a
\ ** number of blocks starting at a given block.
\ cr ." Block-Order ... "
  block-alloc allocate if
    drop  false exit
  endif
  -> l_buff
  l_start BLOCKS/GROUP /mod			\ start position block number
  USAGE_BLOCK0 + l_buff ->start-block !
  dup l_buff ->start-address !
  BLOCKS/GROUP swap - dup l_buff ->first-amount ! \ space remaining
  l_amount swap -				\ additional amount of space required
  dup 0< if					\ it'll fit one page
    drop 0
    l_amount l_buff ->first-amount !
  endif
  BLOCKS/GROUP /mod
  l_buff ->#blocks !
  l_buff ->last-amount !
  l_buff
\ cr ." B-O done"
;

compiler
: adjust-blk-addr	\ addr1 -- addr2
\ *G COMPILER: Convert a block number to address a cell containing its
\ ** indicator bit. Equivalent to:
\ *C   #32 / cells
  5 rshift  2 lshift
;

: 32mod		\ n -- mod(n,32)
\ *G COMPILER: Given a block number, return the bit number of its
\ ** indicator bit in a cell. Equivalent to:
\ *C   #32 mod
  $1F and
;
target

: take/free	\ data mask -- data'
\ *G Apply the bit mask to grab or free the required blocks in
\ ** *\b{data}.
  grab/free @ if			\ true for release
    not and
  else					\ false for take
    or
  endif ;

: take/free32	\ -- mask
\ *G Return the data value to grab/free 32 blocks.
  grab/free @ if			\ true for release
    0
  else					\ false for take
    32BLOCKS
  endif ;

: grab-first-block	\ start-block start-address first-amount --
\ *G Using the starting bitmap block *\b{start-block}, grab/free
\ ** *\b{first-amount} blocks from *\b{start-address} onwards.
  2 pick				\ get start block
\ cr ." first U block " dup .
\  bit_block_buffer read-hd-block drop	\ read in block
  bit_block_buffer read-hd-block 0=	\ read in block ; CEM002
  if  2drop drop  exit  endif		\ CEM002
\ ." Marking  "
  over adjust-blk-addr bit_block_buffer + \ start address of 32 bit units
  over adjust-blk-addr 			\ last address of 32 bits units
  bounds
\ 2dup ." From " .lword ."  To " .lword
  ?do
    take/free32 i !
  cell +loop
					\ s-addr f-amount
  dup 32mod ?dup if			\ need a partial block
\ ."  And " 				\ s-addr fa famod
    topBitmask
\ ." Mask " dup .hex .s	
    2 pick 32mod rshift			\ data to write
    -rot 				\ b-map s-addr f-amount 
    adjust-blk-addr swap adjust-blk-addr +	\ address to write
    bit_block_buffer +
\ 2dup swap .lword ."  @ " .lword
    dup @ rot take/free swap !
  else					\ don't need partial block
    2drop
  endif
\ bit_block_buffer $080 dump
  bit_block_buffer write-hd-block drop	\ write out block
;

: grab-32complete-blocks	\ block --
\ *G Grab/free the complete block of the usage bit map.
  dup bit_block_buffer read-hd-block drop	\ read in block
  bit_block_buffer BLOCK_SIZE bounds do
    take/free32 i !
  cell +loop
  bit_block_buffer write-hd-block drop	\ write back block
;

: grab-last-block	\ last-amount last-block --
\ *G Grab/free the given amount in the last required block
\ ** of the usage bitmap.
\ cr ." Last block " dup u.
  tuck
  bit_block_buffer read-hd-block drop	\ read in block
  dup #32 / cells dup
\ ." To " dup .
  bit_block_buffer swap bounds ?do
    take/free32 i !
  cell +loop

  swap 32mod topBitmask
  swap bit_block_buffer + 
\ 2dup swap .lword ."  @" .lword
  dup @ rot take/free swap !
\ cr ." Last block " dup u.
  bit_block_buffer write-hd-block drop	\ write back block
;

: allocate-blocks	{ l_buff -- } 
\ *G Using the data structure created by *\fo{BLOCK-ORDER},
\ ** grab/free the blocks and *\fo{FREE} the buffer.
\ dup .BlockAlloc
  l_buff ->start-block @
  l_buff ->start-address @
  l_buff ->first-amount @ grab-first-block
	
  l_buff ->start-block @ 1+
  l_buff ->#blocks @ bounds ?do
     \ cr ." full block no.  " i .
     i grab-32complete-blocks
  loop
	
  l_buff ->last-amount @ ?dup if
    l_buff ->start-block @ 1+
    l_buff ->#blocks @ + grab-last-block
  endif
  l_buff ?free
;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

: need-blocks	\ amount -- start
\ *G This attempts to find a contigous area of required space
\ ** and allocates it. *\b{Amount} is the mumber of blocks required.
\ ** *\b{Start} is the starting block number on the media, or -1
\ ** if space cannot be found.
  dup find-free-blocks dup true <> if
    \ cr ." Found start block @ " dup .
    tuck swap				\ keep start pointer
    grab/free off			\ flag to grab blocks
    block-order allocate-blocks
  else
    cr ." Error finding space "
    nip
  endif ;

: free-blocks	\ start amount --
\ *G This will release the required number of blocks from the
\ ** start point.
  grab/free on				\ flag to release blocks
  block-order allocate-blocks
;


\ **************
\ *S Diagnostics
\ **************

FSdiags? [if]

: d-fbm		{ | l_*buff -- }
\ *G Display the disk bitmap.
  BLOCK_SIZE allocate if
    ." Alloc error " exit
  endif
  -> l_*buff
  USAGE_BLOCK0 USAGE_TABLE_SIZE bounds ?do
    i l_*buff read-hd-block if		\ read in block
      cr ." Block" i .
      l_*buff BLOCK_SIZE dump
      cr ." Press key " key upc [char] D = if
	unloop l_*buff ?free  exit
      endif
    else
      cr ." Disk error"
      unloop l_*buff ?free  exit
    endif
  loop
  l_*buff ?free
;

[then]


\ ======
\ *> ###
\ ======

decimal


