\ file-main.fth - Native API

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ ===========
\ *! filemain
\ *T Native API
\ ===========
\ *P The file *\i{%FileSysDir%\file-main.fth} builds the native API
\ ** of the file system in preparation for the ANS layer.

\ *P File identifiers (shows as FID) are the OTB address when files
\ ** are open.

: open_f#	\ fn -- fid|0
\ *G Given a file number, open it and return the FID address or 0
\ ** if the operation fails.
  drive-control request 
  create-fcb dup if 			\ -- fcb,
    create-otb dup if			\ -- fcb, fid
      tuck ->fcb-number !		\ put fcb number in open table
      $0 over set-f-r/w-ptr		\ reset r/w ptr
    else
\      ." no spare OTBs"
      nip				\ lose fcb
    endif
\  else
\    ." no spare FCBs "
  endif
  drive-control signal
;

: open_f	\ caddr len -- fid|0
\ *G Open the named file and return its FID
\ ** or zero if the operation fails.
  get_fn dup if				\ -- fld
    open_f# 
\  else
\    ." file not found "
  endif
;

(( unused
: file-open?	\ *name len -- t/f
  get_fn dup
  if  fn_open?  endif
;
))

: close_f	\ fid -- code
\ *G Close the file, returning: 0 on error; -1 if fully closed;
\ ** or -2 if there is still an open instance.
  dup ->valid-otb @ VALID_OTB <>
  if  drop 0  exit  endif
  sync_vol
  drive-control request
  dup ->fcb-number @ ?dup if		\ get FCB number from open table
    swap release-otb			\ -- fcb
    dup dec-fcb-open if			\ all links removed
      release-fcb
      -1
    else				\ other process has file open
      drop				\ lose FCB
      -2				\ 	File not completely closed
    endif	
  else
    drop
    false				\ error closing
  endif
  drive-control signal 
;

compiler
: file-not-exist?	\ caddr len -- t/f
\ *G COMPILER: Return true if the file does not exist.
  get_fn 0=
;
target

: spllt-path&file	{ l_*name l_len | l_*tname l_tlen -- name len }
\ *G Split Dir path and filename, returning the file name part.
  begin
    l_*name l_len DIR-DELIMIT skip -> l_tlen -> l_*tname
    l_*tname l_tlen DIR-DELIMIT scan -> l_len -> l_*name
    l_len 0=
  until
  l_*tname l_tlen
;

((
: (adjust-blocks)	\ len -- blocks
\  BLOCK_SIZE /mod swap 0<>		\ round up to complete blocks
  BS/mod swap				\ round up to complete blocks
  if  1+  endif
;
))
compiler
: (adjust-blocks)	\ len -- blocks
\ *G COMPILER: Convert a length to the minimum number of blocks
\ ** required to hold that much data.
  BSblks
;
target

: adjust-blocks		\ len -- blocks
\ *G Convert a length to the minimum number of blocks
\ ** required to hold that much data. Always returns
\ ** at least one.
  (adjust-blocks) 1 max
;

: required-blocks	\ len fid -- blocks
\ *G Returns the number of blocks required to write len
\ ** bytes from the current file position.
  get-r/w-pb-ptr + adjust-blocks
;


\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

: need-dir-$buffer	\ --
\ *G *\fo{ALLOCATE} a working directory buffer if one has not
\ ** already been allocated.
  *cwd @ 0= if				\ no buffer defined
    PATH-STRING allocate if
      drop 2drop drop	false exit
    endif
    *cwd !
    clr-dir-str				\ initialise string length
  endif
;

: (ch-dir)	\ caddr len -- t/f
\ *G Switch to the given working directory and return true. If the
\ ** change cannot be made, set the default directory and return 0.
  2dup get_fn dup 0< if			\ directories return -ve
    drive-control request
    negate cdir-buffer read-in-fld drop
    cdir-buffer ->extent-start-0 @ set-working-dir
    drive-control signal
    need-dir-$buffer			\ allocate a string name buffer
    2dup from-root? if			\ new string defined from root ?
      clr-dir-str
    else
      over back-dir? if			\ move to previous dir
        move-back-dir			\ adjust stored string
        remove-dir-hdr			\ remove header from new string
      endif
      add-delimit-str			\ put dir delimiter after existing string
    endif
    append-dir-str			\ add new string to existing dir
    true
  else
    drop 2drop
    set-default-dir
    FALSE
  endif
;
	
: ch-dir	\ caddr len -- t/f
\ *G Switch to the given working directory and return true. If the
\ ** change cannot be made, set the default directory and return 0.
  2dup s" /" compare 0= if		\ root dir ?
    2drop  set-default-dir  TRUE
  else
    (ch-dir)	\ NO, find file
  endif
;

: add-to-dir	{ l_fld l_*name l_len l_block | l_success -- flag }
\ *G Add the given file number and name to the given directory
\ ** block, returning true for success.
  false -> l_success
  BLOCK_SIZE allocate abort" Alloc error "
  l_block over read-f-blk drop
  dup BLOCK_SIZE bounds ?do			\ find next directory slot
    i ->file-name @ dup 0= swap -1 = or if	\ free or unused slot
      l_fld l_*name l_len i write-filename
      l_block over write-f-blk -> l_success
      leave
    endif
  dir-struct +loop
  ?free
  l_success
;

: remove-file-dir	{ l_fld l_block | l_success -- flag }
\ *G Remove the given file number from the given directory block,
\ ** returning true for success.
  false -> l_success
  BLOCK_SIZE allocate abort" Alloc error "
  l_block over read-f-blk drop
  dup BLOCK_SIZE bounds ?do		\ find next directory slot
    i ->file-number @ l_fld = if	\ file numbers match?
      i dir-struct bounds ?do		\ now clear the entry
	i off					
      cell +loop
      l_block over write-f-blk drop
      true -> l_success
      leave
    endif
  dir-struct +loop
  ?free
  l_success
;

: (create_f)	{ l_*name l_len l_size l_extent | l_start l_*buff -- t/f }
\ *G In the current directory create a file with the given name,
\ ** initial size and extension. Return true for success.
\ ** The file is not opened.
  BLOCK_SIZE allocate if
    drop 0 exit
  endif
  -> l_*buff
  l_size adjust-blocks -> l_size
  l_extent (adjust-blocks) -> l_extent
  drive-control request 
  l_size need-blocks -> l_start		\ find data space
  find-next-fld dup -1 = if
\    ." No more file FLDs"		\ find next fld
    drive-control signal 
    l_*buff ?free	
    drop  false exit
  endif
  dup l_*name l_len working-dir-blk add-to-dir	\ find & record filename
  0= if
    drive-control signal 
    l_*buff ?free	
\    ." No more directory space "
    drop  false exit
  endif
  l_*buff over 0 FILE-FTYPE l_size l_extent l_start (create-fld)	
  l_*buff rewrite-fld			\ store FLD
  drive-control signal
  l_*buff ?free	
;

: create_f	{ l_*name l_len l_size_b l_extent_b | l_*tname l_tlen -- t/f }
\ *G Create a file with the given name, initial size and extension.
\ ** The file name may include existing directories.
\ ** The file is not opened.
\ ** Return true for success.
  l_*name l_len file-not-exist?		\ name does not exist
  l_len PATH-STRING <= and if		\ AND name not TOO long
    l_*name l_len spllt-path&file -> l_tlen -> l_*tname
    l_len l_tlen - ?dup if		\ any path ?
      l_*name swap ch-dir 0= if		\ change to that
        FALSE exit			\ couldn't
      endif
    endif
    l_tlen PATH-LENGTH <= if
      l_*tname l_tlen l_size_b l_extent_b (create_f)	\ now create file
    else
      FALSE
    endif
  else
    FALSE
  endif
;

: (create_d)	{ l_*name l_len | l_start l_fld l_prev-fld l_*buff -- t/f }
\ *G In the current directory create a subdirecory with the given name.
\ ** Return true for success.
  s" ." get_fn abs -> l_prev-fld	\ directory numbers are negative
  BLOCK_SIZE allocate if
    drop exit
  endif
  -> l_*buff
  drive-control request 
  DIR_BLOCK_SIZE need-blocks -> l_start	\ find data space
  find-next-fld dup -1 = if
\    ." No more dir FLDs"		\ find next fld
    drive-control signal 
    l_*buff ?free
    drop false exit
  endif
  -> l_fld

  l_fld l_*name l_len working-dir-blk add-to-dir	\ find & record filename
 \ 0= abort" No more directory space "
   0= if
    drive-control signal 
    l_*buff ?free
    false exit
  endif

  l_*buff l_fld BLOCK_SIZE DIR-FTYPE DIR_BLOCK_SIZE 0 l_start (create-fld)
  l_fld l_*buff rewrite-fld drop	\ store FLD

  l_*buff BLOCK_SIZE $FF fill		\ mark dir as unused
  l_fld s" ." 		l_*buff write-filename
  l_prev-fld s" .."	l_*buff dir-struct + write-filename
  l_start l_*buff write-f-blk		\ clear this dir

  drive-control signal
  l_*buff ?free
  sync_vol
;

: create_d	\ caddr len -- t/f
\ *G Create a directory with the given name.
\ ** The name may include existing directories.
\ ** Return true for success.
  2dup file-not-exist? if
    (create_d)
  else
    2drop  false
  endif
;

: rename_f	{ *old-name old-len *new-name new-len | l_fld -- t/f }
\ *G Rename a file from the old name to the new name.
\ ** Return true for success.
  *new-name new-len no-of-paths 1 =
  *old-name old-len no-of-paths 1 = and 0=
  if  false exit  endif
  *new-name new-len get_fn 0<>		\ new name exist ?
  *old-name old-len get_fn -> l_fld 	\ OR old name not exists ?
  l_fld 0= or
  if  false exit  endif
  drive-control request 
  l_fld working-dir-blk remove-file-dir if	\ remove name from dir list
    l_fld *new-name new-len working-dir-blk add-to-dir
  else
    false
  endif
  drive-control signal
;

: (delete_f)	{ l_fn | l_*buff l_size -- t/f }
\ *G Delete the given file number.
\ ** Return true for success.
  FLD-REC allocate
  if  0 exit  endif
  -> l_*buff
  0 -> l_size
  drive-control request 
\  cr ." delete dir " dup .decimal
  l_fn working-dir-blk remove-file-dir drop	\ remove directory entry
  l_fn l_*buff read-in-fld drop		\ load FLD from disk
  NO-OF-EXTENTS 0 ?do
    i l_*buff file-extent dup l_size + -> l_size
    2dup or if
      \ cr ." delete extent " i . 2dup .decimal .decimal
      free-blocks			\ remove bitmap entry
    else
      2drop leave			\ all extents found
    endif
  loop
  l_*buff ->indirect-block @ ?dup if
    l_size delete-indir-extent
  endif
\ cr ." delete fld "
  l_*buff FLD-REC erase
  l_fn l_*buff rewrite-fld
  l_*buff ?free
  drive-control signal
;

: delete_f	\ name len -- t/f
\ *G Delete the given file by name.
\ ** Return true for success.
  get_fn ?dup if
    (delete_f)
  else
    false
  endif
;

: delete-files	{ l_*name l_len -- t/f }
\ *G Delete a group of files if the first character of the name
\ ** is '*', otherwise delete a single file.
\ ** Return true for success.
  l_*name s" *" s= if			\ wild card
    begin
      l_*name l_len delete_f
    0= until
    true
  else
    l_*name l_len delete_f
  endif
;

: lseek_f	\ ptr fid -- t/f
\ *G Seek to the given location in the file.
\ ** Return true for success.
\ *D PTR=0    set to start
\ *D PTR=-1   set to end of written data
\ *D PTR=+ve  from start of file
\ *D PTR=-ve  from end of data
  dup ->valid-otb @ VALID_OTB <> if
    2drop  0 exit
  endif
  dup fcb-addr ->file-blocks @ 0<> if	\ see if exists
    over 0< if				\ test ptr
      dup fid-file-size 1+ rot + dup 0< if
	drop  0
      endif
      swap	 			\ set PTR to end of data area
    endif
    tuck set-f-r/w-ptr			\ set new pointer
    r/w-ptr-inrange?			\ test it
  else
    2drop  false
  endif
;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

: file-read-area	{ l_fid l_len | l_*c-pos -- b-start size }
\ *G Caculate how much can be read from the current position with
\ ** a single block read.
  l_fid get-f-r/w-ptr -> l_*c-pos
  l_fid get-r/w-pb-ptr			\ get read start offset
  l_len l_*c-pos + 			\ new end ptr
  l_fid fid-file-size min		\ end of file ptr
  l_*c-pos -				\ max amount required to read
  BLOCK_SIZE min dup BLOCK_SIZE = if	\ only if full size
    over -				\ less the start pos = Max this read
  endif
  2dup + BLOCK_SIZE > if		\ does the read straddle
    drop				\ lose size
    BLOCK_SIZE over -
  endif	
;

: r/w-block	{ l_fid | l_hd-block l_block l_fcb -- phys-blk } 
\ *G Return the block number containing the current file read/write
\ ** pointer location. Return -1 on error or -2 if the pointer
\ ** is beyond the end of file.
    -1 -> l_hd-block
    l_fid get-r/w-b-ptr -> l_block  
    l_fid fcb-addr -> l_fcb
    l_fcb ->file-blocks @ l_block > if  \ test if pointer inside allocated blocks
        l_fcb ->extent-start-0 EXTENT_LIST bounds do
            i ->extent-size @ ?dup if   \ get extent size
                l_block swap - dup 0< if \ found extent
                    drop        
                    i ->extent-start @ l_block + \ get extent start 
                    -> l_hd-block
                    leave
                else
                    -> l_block          \ rem-blocks
                endif
            else
                leave
            endif
        FILE-EXTENTS +loop
        l_fcb ->indirect-block @ 0<>    \ indirect extent defined
        l_hd-block -1 = and if          \ not found so far
            l_fid l_block get-indirect-extent -> l_hd-block
        endif
    else
        -2 -> l_hd-block        	\ need more blocks allocated
    endif
    l_hd-block
;

: read_f	{ l_buf l_len l_fid | l_read-size l_start-buff -- len }
\ *G Read len bytes into buff from the given file, returning
\ ** the number of bytes actually written.
  l_fid ->valid-otb @ VALID_OTB <> if
    0 exit
  endif
  l_buf -> l_start-buff
  l_len l_fid required-blocks 0 ?do					
    l_fid r/w-block			\ get physical block to hd read from pointer
    l_fid r-ptr-inrange? 0= if 		\ test in read range of stored data
      drop  leave			\ outside read range, return 0 bytes read
    else				\ -- block
      l_buf
      l_fid l_len file-read-area -> l_read-size
      l_read-size 0= if			\ work out which part of block to read
        drop 2drop  leave
      endif
      l_read-size read-d-block 0= if	\ read block from disk
        leave				\ error reading			
      endif
      l_len l_read-size - -> l_len	\ dec length
      l_read-size l_buf + -> l_buf	\ inc buffer pointer
      l_read-size l_fid inc-f-r/w-ptr	\ move logical disk pointer
    endif
  loop
  l_buf l_start-buff -			\ amount actually read
;

\ \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\

: update-size&time	{ l_fid | l_size l_fcb -- t/f }
\ *G Update the file size and modification time.
  l_fid ->valid-otb @ VALID_OTB <> if
    0 exit
  endif
  l_fid get-f-r/w-ptr -> l_size 
  l_fid fcb-addr -> l_fcb
  l_fcb ->file-size @ l_size < if
    l_size l_fcb ->file-size !
  endif
  lseconds @ l_fcb ->file-last-mod !
  l_fcb update-fcb			\ change on disk
;		

: write_f	{ l_buf l_len l_fid | l_size l_start l_req-blk -- len }
\ *G Write len bytes from buff from the given file, returning
\ ** the number of bytes actually written.
  l_fid ->valid-otb @ VALID_OTB <> if
    0 exit
  endif
  l_buf -> l_start
  l_len l_fid required-blocks -> l_req-blk	\ get number of blocks required
  l_req-blk l_fid need-extent? if	\ need more space for this file
    l_req-blk l_fid annex-file 0= if	\ YES, try and grab some
      0 exit				\ couldn't, so leave
    endif
  endif
  l_req-blk 0 do	
    l_fid r/w-block dup 0< if		\ get block to write to from pointer
      drop leave			\ error with allocation
    endif
    l_buf				\ buffer
    l_fid get-r/w-pb-ptr		\ start pointer
    dup l_len + BLOCK_SIZE min		\ end pointer
    2dup swap - -> l_size		\ size
    write-d-block 0= if			\ write block
      leave
    endif
    l_len l_size - -> l_len		\ dec length
    l_size l_buf + -> l_buf		\ inc buffer pointer
    l_size l_fid inc-f-r/w-ptr		\ move logical disk pointer
  loop
  l_fid update-size&time drop		\ update FCB and FLD last mod date & file size
  l_buf l_start -			\ number of blocks written
;

: zero-file	\ l_fid -- t/f
\ *G Reduce the file to zero size without changing the disk
\ ** allocation. Return true for success.
  dup ->valid-otb @ VALID_OTB <> if
    drop 0 exit
  endif
  fcb-addr				\ get fcb number
  0 over ->file-size !
  lseconds @ over ->file-last-mod !
  update-fcb				\ change on disk
;		

: InitFileTask	\ --
\ *G Before using the file system, each task must run this word.
  ROOT_DIR_BLOCK set-working-dir
  *cwd off
;

TaskDiskFlush DiskCacheMode = [if]

: doDiskTask	\ --
\ *G The action of the disk service task.
  ConsoleIO decimal
  cr ." Disk Service task running"
  InitFileTask
  begin
    tick-ms ms  time_sync_vol
  again
;

Task DiskTask	\ -- addr
\ *G The disk service task.

[then]

: InitFileSys	\ --
\ *G Before using the file system, this word must be run at
\ ** power up.
  drive-control init-sema
  drive-access init-sema
  fld-update init-sema
  cache-sema init-sema

  InitDisk				\ initialise disk hardware
  Init-Cache				\ initialise disk cache
  InitFileTask				\ initialise USER variables
  init-fcbs				\ setup the FILE CONTROL BLOCK list
  init-otbs				\ setup the OPEN TABLE CONTROL BLOCK list
  inited-disk				\ mount the disk
[ TaskDiskFlush DiskCacheMode = ] [if]
  ['] doDiskTask DiskTask initiate	\ launch disk service task
[then]
;
' InitFileSys AtCold


\ ======
\ *> ###
\ ======

decimal


