\ INIT-DISk.FTH - Disk formatting and initialisation

((
Copyright (c) 1999-2001
CEM Systems Ltd, Belfast

Distributed under license by MicroProcessor Engineering.
A big thank you to Jeremy Moller and George Redpath of CEM.

Extensions and modifications:
Copyright (c) 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20050318 MPE001 Refactored source files.
))

decimal

\ ===========
\ *! initdisk
\ *T Disk formatting and initialisation
\ ==========

 0 equ DISK_OK
-1 equ DISK_WRONG_FORMAT
-2 equ DISK_ERROR

: (get-disk-info)	\ -- oc
\ *G Read the disk root block and save the disk format details.
\ ** Returns: 0 OK, -1 wrong format, -2 read error
  cr ." Checking disk: "
  ROOT_BLOCK cdir-buffer read-f-block 0=
  if  ." - Failed"  DISK_ERROR  exit  endif
  ." format "
  cdir-buffer ->CEM_FORMAT @ CEM_FORMAT = if
    cdir-buffer ->USAGE_BLOCK0 @ to USAGE_BLOCK0
    cdir-buffer ->vol-size @ dup to DISK_SIZE 		\ number of blocks per disk
    BLOCKS/BIT_TABLE / 1 max	 to USAGE_TABLE_SIZE	\ size of bit map table
    cdir-buffer ->FLD0_BLOCK @ 	 to FLD0_BLOCK		\ start of fld table
    MAX_FILES FLD/BLOCK / 	 to FLD-SIZE
    cdir-buffer ->DATA_BLOCK0 @  to DATA_BLOCK0		\ start of files
    set-default-dir					\ set-default-dir
    DISK_OK
    ." - OKay"
  else
    DISK_WRONG_FORMAT ." - NOT CEM format"
  endif
;

: init-root-block	{ l_buff | l_dest -- }
\ *G Initialise the root block on the disk.
  BLOCK_SIZE allocate abort" ALLOC ERROR "
  -> l_dest 
  l_dest BLOCK_SIZE erase
  USAGE_BLOCK to USAGE_BLOCK0
  USAGE_BLOCK0 l_dest ->USAGE_BLOCK0 !	\ start of bit map table
  l_buff #57 2* + l@(le)		\ -- #sectors
  dup to DISK_SIZE
  dup l_dest ->vol-size !		\ number of blocks per disk
  BLOCKS/BIT_TABLE / 1 max		\ size of bit map table
  dup to USAGE_TABLE_SIZE
  USAGE_BLOCK0 +
  dup l_dest ->FLD0_BLOCK !		\ start of fld table
  dup to FLD0_BLOCK
  MAX_FILES FLD/BLOCK /
  dup to FLD-SIZE
  +
  dup to DATA_BLOCK0
  l_dest ->DATA_BLOCK0 !		\ start of files
  lseconds @ 
  l_dest ->init-time !			\ time of 
  CEM_FORMAT l_dest ->CEM_FORMAT !
  ROOT_BLOCK l_dest write-block 0= abort" Write error "
  l_dest free abort" Error free "
;

: write-filename	{ fld *name len buff -- }
\ *G Write the given file number and name to the buffer.
  *name len get-path -> len -> *name
  fld 	buff ->file-number !
  len	buff ->file-name c!
  *name	buff ->file-name 1+ len cmove
;

: init-root-dir	\ --
\ *G Initialise the root directory.
  cdir-buffer
  dup BLOCK_SIZE $0ff fill
  02 s" ."          2 pick dir-struct * 4 pick + write-filename 
  00 s" bitmap.sys" 2 pick dir-struct * 4 pick + write-filename 
  01 s" flist.sys"  2 pick dir-struct * 4 pick + write-filename
  working-dir-blk over write-block 0= abort" error writing"
  drop
;

: init-vol-bitmap	\ --
\ *G Initialise the disk bitmap.
\ cr ." wiping bitmap"
  USAGE_BLOCK0 USAGE_TABLE_SIZE bounds ?do
    i erase-block 0=
    if  cr ." Error ERASE " leave  endif
  loop
\ cr ." grabbing"
  grab/free off				\ flag to grab blocks
  0 DATA_BLOCK0 block-order
\ dup 20 dump
  allocate-blocks \ mark the first few blocks as used
\ cr ." done"
;

: (create-fld)	{ l_buff l_fld l_bytes l_type l_blocks l_expan l_start -- }
\ *G Initialise a file entry in a buffer.
  l_buff FLD-REC erase
  l_blocks BLOCK_SIZE * l_bytes < abort" Size definition error "
  l_fld		l_buff ->fld-no !
  l_blocks	l_buff ->file-blocks !
  l_bytes 	l_buff ->file-size !
  l_type	l_buff ->file-type w!
  l_expan	l_buff ->file-expansion !
  l_start	l_buff ->extent-start-0 !
  l_blocks	l_buff ->extent-size-0 !
  lseconds @	l_buff ->file-last-mod !
;

: create-fld	{ l_buff l_fld l_bytes l_type l_blocks l_expan l_start -- }
\ *G Given a directory block buffer, initialise the given FLD entry.
  l_fld fld-m-block l_buff +
  l_fld l_bytes l_type l_blocks l_expan l_start (create-fld)
;

: init-fld	\ --
\ *G Initialise the system FLDs.
  FLD0_BLOCK FLD-SIZE bounds ?do	\ clear all FLD blocks
    i erase-block 0=
    if  cr ." Error ERASE " leave  endif	
  loop
  Block_size allocate abort" ALLOC error "
  dup BLOCK_SIZE erase
  dup 0 USAGE_TABLE_SIZE BLOCK_SIZE * SYS-FTYPE USAGE_TABLE_SIZE 0 usage_block0 create-fld
  dup 1 FLD-SIZE BLOCK_SIZE * SYS-FTYPE FLD-SIZE 0 FLD0_BLOCK create-fld
  dup 2 BLOCK_SIZE DIR-FTYPE 1 0 ROOT_DIR_BLOCK create-fld
  FLD0_BLOCK over write-block drop
  ?free
;

: show-drive	\ --
\ *G Display the drive details.
  Disk_size		cr ." Disk size            " .
  ROOT_DIR_BLOCK	cr ." Root starts          " .
  usage_block0		cr ." Bit map starts       " .
  FLD0_BLOCK		cr ." FLD starts           " .
  DATA_BLOCK0		cr ." Data area starts     " .
  USAGE_TABLE_SIZE	cr ." Bit map table size   " .
  FLD-SIZE		cr ." File descriptor size " .
  cr
;

: init-vol	\ --
\ *G Initialise (format) the disk.
  (drive-id) dup			\ read drive
  active-fcbs @ 0= and if		\ files not open
    release-all-cache
    dup init-root-block
    ?free
    show-drive 
    init-root-dir			\ setup root directory on disk
    init-vol-bitmap			\ setup usage bit map on disk
    init-fld				\ setup File List Descriptor on disk
    init-fcbs				\ setup the FILE CONTROL BLOCK list
    init-otbs				\ setup the OPEN TABLE CONTROL BLOCK list
  else
    0= if 
      cr ." Error reading disk "
    else
      cr ." Files still open "
    endif
  endif ;

: inited-disk	\ --
\ *G Performed at power up in *\fo{InitFileSys}, checks the
\ ** disk format and, if invalid prompts the user to reformat
\ ** disk.
  (get-disk-info) ?dup if
    dup DISK_WRONG_FORMAT = if
      cr ." Do you wish to format this disk? (y/n)"
      y/n? if
	cr ." Initialising disk"
	init-vol
      else
	cr ." Replace disk "
      endif
    endif
    DISK_ERROR = if
      cr ." Disk not usable "
    endif
  endif ;

((
\ this is done at boot up, to read the disk format details
: get-disk-info	\ --
  drive-control signal 			\ initialise semaphore
  fld-update signal			\ initialise semaphore
  init-fcbs				\ setup the FILE CONTROL BLOCK list
  init-otbs				\ setup the OPEN TABLE CONTROL BLOCK list
  inited-disk
;
))


\ ************
\ *S Test code
\ ************

FSdiags? [if]

BLOCK_SIZE buffer: td[

: .block	\ blk# --
\ *G Read a disk block and display the first part of the data.
  cr ." Blk = " dup .
  td[ read-f-block  ." Res = " .
  td[ ( $080 ) BLOCK_SIZE dump
;

: .SysBlocks	\ --
\ *G Display the system data blocks.
  DATA_BLOCK0 0
  ?do  i .block  loop
;

: Test-disk-speed	{ l_*dest | l_time -- }
\ *G Check disk performance.
  cr ." Testing Disk performance "
  ticks -> l_time
  ." W1 "
  DISK_SIZE 2 - l_*dest write-block if
    ." W2 "
    DISK_SIZE 2 - l_*dest write-block 0= if
      ." error"
    endif
  else
    ." error"
  endif
  ticks l_time -  . ." mS"
;

[then]


\ ======
\ *> ###
\ ======

decimal


