\ SINE and COSINE words from Kurt Heinz, Synics

((
These words provide a simple implementation of sine, cosine,
and tangent functions.

Input angles are in degrees * ten, thus 45.1 degrees is
represented as 451. If this is to be changed, some hard-coded
values will have to be changed.

Output for SIN and COS is scaled in the range 1=10,000 which
is defined by the value of the constant OSF (output scaling
factor). Output for TAN is scaled in TAN to be 1/100 of that
for SIN and COS.
))

\ scaled integer SIN function

decimal

10000 constant osf      \ scaling constant

variable XS             \ square of the scaling angle


\ z (sin)

: z             \ a b -- m ; m=10000-ax*x/b .. a common term
  xs @ swap / negate  osf */  osf +  ;

: (sin)         \ theta -- 10000*sin ; -15708 < theta < 15708
  dup dup osf */ xs !                   \ save x^2
  osf                                   \ start with 1=10,000
  72 z  42 z  20 z  6 z                 \ compute series terms
  osf */  ;                             \ finally mult by x


\ range reduction

: ?mirror       \ theta -- theta' ; reduce range
  dup 900 >
  if  1800 swap -  endif  ;

: reduce        \ theta -- theta' ; range reduction
  3600 mod  dup 0<
  if  3600 +  endif
  dup 1800 <
  if  ?mirror
  else  1800 - ?mirror negate
  endif  ;


\ sin cos tan in .1 degree steps

: sin           \ theta -- 10000*sin
  reduce  17453 1000 */                \ deg to rad * 10000
  (sin)  ;

: cos           \ theta -- 10000*cos
  3600 mod  900 swap - sin  ;

: tan           \ theta -- 100*tan
  dup sin  swap cos  dup -18 17 within?
  if  1 min -1 max  swap 1 min -1 max  *  30000 *
      dup 0=  if  drop 30000  endif
  else  100 swap */
  endif  ;


