\ Microchip 25LC series SPI EEPROM driver

((
Copyright (c) 1999

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)2380 631441
fax: +44 (0)2380 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

HOLD and WP lines are assumed to be tied high. No software control
for these lines has been added as they are not standard SPI lines.
MSB first, data is valid on the rising edge of the clock
))


\ ---------------------------------------------------------------------------
\ split a 16 bit address into 2 bytes high byte on top of stack
\ ---------------------------------------------------------------------------
 
: addr_split_25lc	\ addr -- addr1 addr2
  dup				\ -- addr addr
  $00ff AND			\ -- addr addr1
  swap				\ -- addr1 addr
  $ff00	AND 8 rshift		\ -- addr1 addr2
;

\ ---------------------------------------------------------------------------
\ write enable
\ ---------------------------------------------------------------------------

: wren_25lc	\ --
  ss_low		\ --		; enable chip
  $006 write_byte_spi	\ -- 		; send instruction to enable write
  ss_high		\ -- 		; disable chip
;

\ ---------------------------------------------------------------------------
\ write disable
\ ---------------------------------------------------------------------------

: wrdi_25lc	\ --
  ss_low	    	\ --		; enable chip
  $004 write_byte_spi	\ --		; send instruction to disable write
  ss_high		\ -- 		; disable chip
;

\ ---------------------------------------------------------------------------
\ write status register
\ ---------------------------------------------------------------------------

: wrsr_25lc		\ byte --
  wren_25lc
  ss_low			\ -- byte		; enable chip
  $001 write_byte_spi		\ -- byte		; send instruction to write to status register
  write_byte_spi		\ --			; send byte
  ss_high			\ --			; disable chip
;

\ ---------------------------------------------------------------------------
\ write a byte to a memory location
\ ---------------------------------------------------------------------------

: 25lc!		\ byte addr --
  wren_25lc
  ss_low	  		\ -- byte addr			; chip enable
  addr_split_25lc	        \ -- byte addr1 addr2	        ; split 16bit address into 2 bytes
  $002 write_byte_spi		\ -- byte addr1 addr2		; send write instruction
  write_byte_spi		\ -- byte addr1			; send high byte of address
  write_byte_spi		\ -- byte			; send low byte of address
  write_byte_spi		\ --				; send data byte
  ss_high			\ 				; disable chip
;

\ ---------------------------------------------------------------------------
\ read status register
\ ---------------------------------------------------------------------------

: rdsr_25lc		\ -- byte
  ss_low	  		\		; enable chip
  $005 write_byte_spi		\		; send instruction to read status register
  read_byte_spi			\ -- byte	; read data byte
  ss_high			\ -- byte	; disable chip
;

\ ---------------------------------------------------------------------------
\ read a byte from a memory location
\ ---------------------------------------------------------------------------

: 25lc@			\ addr -- byte
  ss_low   			\ -- addr		; enable chip
  addr_split_25lc		\ -- addr1 addr2	; split 16 bit address into 2 byte
  $003 write_byte_spi		\ -- addr1 addr2	; send write instruction
  write_byte_spi		\ -- addr1		; send high byte of address
  write_byte_spi		\ -- 			; send low byte of address
  read_byte_spi			\ -- byte		; read data byte
  ss_high			\			; disable chip
;

\ ---------------------------------------------------------------------------

