\ SPI driver for Parallel port

((
Copyright (c) 1999

MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)2380 631441
fax: +44 (0)2380 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))


\ Parallel Port  ----------------------- SPI 

\ strobe     (pin 1 ) ------------------- SCK
\ auto feed  (pin 14 ) -----------------  MOSI
\ initialise (pin 16) ------------------- MISO  
\ select in  (pin 17) ------------------- SS
\ D0 (pin2)  (pin 2) -------------------- RST

\ ----------------------------------------------------------------------------
\ Parallel port SPI addresses
\ ----------------------------------------------------------------------------

$008 constant sck		\ mask of sck
$004 constant mosi	        \ mask of mosi bit
$040 constant miso	        \ mask of miso bit
$002 constant ss		\ mask of slave select bit / chip select

$010 constant rst		\ mask of reset bit - used by some microcontrollers
#010 value uS-delay		\ default delay to 10uS

\ ----------------------------------------------------------------------------
\ VFX / PFW22 Harness
\ ----------------------------------------------------------------------------

[undefined] extern [if]		\ check for PFW2.2
: GetTickCount
  [ also Winfunctions ] WinGetTickCount [previous]
;
[else]					\ assume PFW/VFX
extern DWORD PASCAL GetTickCount( void );
[then]

: ms		\ time --
  GetTickCount +
  begin dup GetTickCount - 0< until 
  drop
;

variable <delay>

: delay		\ -- ; short wait
  <delay> @ 0 
  ?do  noop  loop
;

: calc_1ms	\ -- n ; n is number required by DELAY for 1 ms
  #2000    			\ -- delay_value
  begin				\ -- delay_value			       
    dup <delay> !	        \ -- delay_value			; set the delay value
    GetTickCount		\ -- delay_value ms_value1		; get the ms counter
    delay			\ -- delay_value ms_value1	        ;
    GetTickCount 	        \ -- delay_value ms_value1 ms_value2  	; get the ms counter
    swap - abs			\ -- delay_value ms_diff		; interval  
    dup #200 <			\ -- delay_value ms_diff flag
   while			\ -- delay_value ms_diff		; if delay too short 
    drop 2*			\ -- delay_value'			; scale delay value up		        
  repeat		
  / 				\ -- delay_per_cycle			; delay per cycle
;

: set_delay	\ uS -- ; value passed in sets delay to that number of uS
  calc_1ms 1000 */ 		\ -- 
  <delay> !
;  

\ ----------------------------------------------------------------------------
\ basic spi definitions, to set spi lines high, low and read
\ ----------------------------------------------------------------------------

: sck_low	\ --
  sck invert
  ppd@ AND ppd!
  delay
;

: sck_high      \ --
  ppd@ sck OR ppd!
  delay
;

: mosi_low	\ --
  mosi invert
  ppd@ AND ppd!
  delay
;

: mosi_high	\ --
  ppd@ mosi OR ppd!
  delay
;

: ss_low	\ --
  ss invert
  ppd@ AND ppd!
  delay
;

: ss_high	\ --
  ppd@ ss OR ppd!
  delay
;

: rst_low	\ --
  rst invert		
  ppd@ AND ppd!
  delay
;

: rst_high	\ --
  ppd@ rst OR ppd!
  delay
;

: miso@		\ -- bit
  pps@ miso AND			\ -- bit		; read miso bit then AND
  0 = if $000 else $001 endif	\ -- bit'		; if miso bit set put 1 on stack else put on 0
  delay
;

\ ----------------------------------------------------------------------------

