\ Tasking.fth - ROM PowerForth Multi-Tasker Example

((
Copyright (c) 1988-1991, 2006
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   htpp://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20060704 MPE001 Overhauled for current compilers.
))

only forth definitions
decimal

\ ===========
\ *! taskexample
\ *T Multitasker examples
\ ===========
\ *P This example demonstrates the event and message handling
\ ** facilities of the multitasker. In order to compile this code
\ ** the following equates must be true in your control file.
\ *E 1 equ tasking?             \ true if multitasker needed
\ **   1 equ event-handler?     \   true to include event handler
\ **   1 equ message-handler?   \   true to include message handler


\ ****************
\ *S Example tasks
\ ****************
\ *P Note that these examples are for target code up from v6.1
\ ** onwards.

\ *P A task whose action is *\fo{ACTION} waits until a message is
\ ** sent to it, and then displays the ID of itself, the ID of the
\ ** sender, and the message. The message is a number, which may be
\ ** the message itself, the address of a data block, or even the
\ ** xt of a word to *\fo{EXECUTE}.

[undefined] ConsoleIO [if]
: ConsoleIO	\ --
\ *G Define the default debug device for tasks.
  console dup ipvec ! opvec !
;
[then]

: action        \ --
\ *G Task action, an endless loop after the initialisation.
\ ** N.B. Tasks that use *\fo{KEY}, *\fo{EMIT} and friends
\ ** *\b{must} must define an I/O device first. See *\fo{ConsoleIO}
\ ** above.
  ConsoleIO  decimal		\ force output to default console
  begin
    get-message                 \ wait for message
    cr ." Task @ " self .       \ show who received it
    swap                        \ swap message and task
    ." received message # " u.  \ show message
    ." from task @ " .          \ and task
    cr
  again                         \ go get next message
;

\ *P The event handler for the example tasks just displays that
\ ** it has been triggered. Note that the event run flag is not
\ ** reset by this event handler. The task will thus wake up
\ ** because this bit is never reset. Add *\fo{CLR-EVENT-RUN}
\ ** to reset the event run flag in the task status.

: eventful      \ --
\ *G Just shows triggered task number.
  cr  ." Event handler for task @ " self . cr
;

Task ExTask1	\ -- addr
\ *G Example task.
Task ExTask2	\ -- addr
\ *G Example task.

: do-tasks      \ --
\ *G *\fo{DO-TASKS} sets up the actions and event handlers of
\ ** the two example tasks, and starts everything running.
  ['] eventful ExTask1 to-event		\ event handler
  ['] action ExTask1 initiate		\ launch task action
  ['] eventful ExTask2 to-event		\ event handler
  ['] action ExTask2 initiate		\ launch task action
;

\ *P Now you can use *\fo{SEND-MESSAGE} and *\fo{SET-EVENT}, e.g.
\ *E   task SET-EVENT
\ **   message task SEND-MESSAGE


\ ======
\ *> ###
\ ======

decimal
