\ 8583DRV.FTH - interface with a PCF8583 Real Time Clock

((
Copyright (c) 1999, 2005
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

To do
=====

Change history
==============
20050412 MPE001 Updated for latest I2C drivers.
))

\ ==========
\ *! 8583drv
\ *T PCF8583 Real Time Clock
\ ==========
\ *P The code in *\i{Examples\I2C\Devices\8583DRV.FTH} provides
\ ** a high level driver for the Philips P8583 Real Time Clock chip.

decimal

$0A0 constant 8583_id0
\ *G device address, including 4 MSBs = Manufacturers prefix
\ ** (A0 for 8583), bits 4,3,2 = device number, LSB=R/W (R=1, W=0).
 						
\ ----------------------
\ RTC internal registers 
\ ----------------------

$000 equ control			
$001 equ hundreds		
$002 equ seconds	
$003 equ minutes	
$004 equ hours		
$005 equ year_date	
$006 equ weekday_month
$010 equ yearoffset_address1
$011 equ yearoffset_address2
$012 equ rambase			\ base address for free RAM

: set_address	\ w_address devaddr --
\ *G Sets word pointer address using devaddr and w_address, where
\ ** w_address addresses the RTC registers and memory.
  +ack_i2c  start_i2c		\ -- w_address devaddr ; send start condition
  $FE and write_byte_i2c	\ -- w_address ; send device address 
  write_byte_i2c		\ -- ; send device address
  stop_i2c
;

: read_8583	\ devaddr -- data 
\ *G Reads a byte from the 8583 using the previously set
\ ** data address.
  +ack_i2c  start_i2c	   	  	\ 	 	   	; send start condition	
  $001 or write_byte_i2c		\ -- 			; set LSB ( read mode ) and send d_address
  -ack_i2c  read_byte_i2c		\ -- data 	  	; write w_address     
  stop_i2c				\ -- data		; send stop condition
;

: rtc@		\ w_address devaddr -- byte  
\ *G Fetches a byte of data at a given RTC address.
  tuck set_address  read_8583
;

: rtc!		\ data w_address devaddr --
\ *G Writes a data-byte on the 8583 using (w_address, d_address, data )
\ *D w_address: address in the RTC registers & memory
\ *D data: The data you are sending
  +ack_i2c  start_i2c		\ -- data w_address devaddr ; send start condition
  $0FE and write_byte_i2c	\ -- data w_address ; send device address   
  write_byte_i2c		\ -- data ; send word address   
  write_byte_i2c               	\ -- ; send data
  stop_i2c			\ send stop condition
;						       


: resetclock	\ devaddr --
\ *G Initialise control register, default is for clock stopped.
  >r
  $083 control r@ rtc! 	\ 50% timer duty = off, 50% alarm duty = off
  			\ alarm enable = off, mask flag = off
			\ function mode = 32.768Khz, hold last count = off
			\ stop counting = on
  $000 hundreds r@ rtc!
  $000 seconds r@ rtc!
  $000 minutes r@ rtc!
  $000 hours r@ rtc!
  $000 year_date r@ rtc!
  $000 weekday_month r> rtc!
;

: stopclock	\ devaddr --
\ *G Stop the clock. In the control register, bit 7 is set,
\ ** all other bits are perseved 
  >r
  control r@ rtc@ $080 OR	\ --  data 	; read control register mask and set bit8  
  control r> rtc!		\		; store new value in control register
;

: startclock	\ devaddr --
\ *G Start the clock. In the control register, bit 7 is cleared
\ ** and all other bits are perserved
  >r  control r@ rtc@  $7F and  control r> rtc!
;

: latchclock	\ devaddr --
\ *G Latch the the clock, clock still counts but the clock
\ ** registers are frozen to enable a read. Control register
\ ** bit 7 is set, all other bits are perserved
  >r  control r@ rtc@  $040 OR  control r> rtc!
;

: unlatchclock	\ devaddr --
\ *G Un-latch the the clock, clock registers are un-frozen
\ ** and updated with latest count values. Control register
\ ** bit 7 is cleared, all other bits are perserved
  >r  control r@ rtc@  $BF and  control r> rtc!
;

: number!	\ byte address devaddr --  
\ *G Store a binary value in BCD format at address
  rot bin2bcd -rot rtc!
;

: number@	\ address devaddr -- bin_byte 
\ *G Fetches a value from an address given and converts it from
\ ** BCD to binary.
  rtc@ bcd2bin
;  

: date!		\ dd mm yyyy devaddr --   
\ *G Store the date.
  >r
  r@ stopclock
  year_date $000 r@ rtc!	\ -- dd mm yyyy	; clear year_date register  
  dup				\ -- dd mm yyyy yyyy ; duplicate the year
  $0ff AND	        	\ -- dd mm yyyy low_byte ; mask low byte 
  swap				\ -- dd mm low_byte yyyy ; swap items
  8 rshift	        	\ -- dd mm low_byte high_byte ; shift high byte into LS byte			
  yearoffset_address1 r@ rtc!	\ -- dd mm low_byte ; store high byte 
  yearoffset_address2 r@ rtc!	\ -- dd mm ; store low byte
  swap bin2bcd	        	\ -- mm dd ; convert from bin to bcd 
  year_date r@ rtc! 		\ -- mm ; store new year_date value
\ weekday_month register
  bin2bcd			\ -- mm ; convert from bin to bcd
  weekday_month r@ rtc!		\ -- mm ; store new weekday_month value   
  r> startclock
;

: date@		\ devaddr -- dd mm yyyy
\ *G Fetch the date. The clock must be latched before this word
\ ** is used.
  >r
  \ year_date register
  year_date r@ rtc@	        \ -- year_date ; fetch year_date 
  dup				\ -- year_date year_date ; duplicate top item
  $0C0 AND			\ -- year_date yy ; mask off bits 6 & 7
  6 rshift		        \ -- year_date yy ; shift right 6  

  yearoffset_address1 r@ rtc@  	\ -- year_date yy yoffset1 ; fetch year offset 1 
  8 lshift		        \ -- year_date yy yoffset1 ; shift year offset 1 into high byte
  yearoffset_address2 r@ rtc@   \ -- year_date yy yoffset1 yoffset2 ; fetch year offset 2 and store in low byte
  or				\ -- year_date yy yoffset ; or both bytes to give 1 double year offset
  +			        \ -- year_date yyyy ; add year offset to current year count

  swap			        \ -- yyyy year_date ; swap the top 2 items
  $03f AND			\ -- yyyy dd ; mask off the 6 LSB's 
  bcd2bin			\ -- yyyy dd ; convert from bcd to bin
									
  \ weekday_month register
  weekday_month r> rtc@	        \ -- yyyy dd weekday_month ; fetch weekday_month 
  $01F AND			\ -- yyyy dd mm ; mask off 4 LSB's
  bcd2bin			\ -- yyyy dd mm ; convert from bcd to bin
				  
  rot				\ -- dd mm yyyy ; swap top 2 items  
;				  

: time!		\ secs mins hrs devaddr --
\ *G Store the time.
  >r
  r@ stopclock	 					
  $03f AND			\ mask off 6 LSB's to force 24hrs format
  hours r@ number!		\ -- secs mins ; store hours value  
  minutes r@ number!    	\ -- secs ; store minutes value	       	  
  seconds r@ number!		\ store seconds value
  r> startclock
;

: time@		\ devaddr -- secs mins hrs  
\ *G Fetch the time. The clock must be latched before this word
\ ** is used.
  >r
  seconds r@ number@		\ -- secs 
  minutes r@ number@		\ -- secs mins   
  hours r> number@  $03f AND	\ -- secs mins hrs ; remove am/pm flag & 24hr flag   
;

: time&date	\ -- secs mins hrs dd mm yyyy
\ *G Fetch time and date, but latch clock to ensure a stable reading.
\ ** The clock is assumed to be the the first one at 8583_ID0.
  8583_id0 >r			\ assume date comes from first RTC
  r@ latchclock			\ stop register updates
  r@ time@		        \ -- secs mins hrs				
  r@ date@		        \ -- secs mins hrs dd mm yyyy
  r> unlatchclock	        \ restart register updates
;

: SetRTC	\ secs mins hrs dd mm yyyy --
\ *G Set time and date, but latch clock to ensure a stable setting.
\ ** The clock is assumed to be the the first one at 8583_ID0.
  8583_id0 dup latchclock
  dup >r date! r@ time!
  r> unlatchclock		\ restart register updates
;

: .2digs	\ u --
\ *G Display u as a two digit unsigned number
  0 <# # # #> type
;

: .4digs	\ u --
\ *G Display u as a four digit unsigned number
  0 <# # # # # #> type
;

: .d2		\ u --
\ *G Display u as a two digit unsigned number with a trailing "/"
\ ** character.
  .2digs  [char] / emit
;

: .t2		\ u --
\ *G Display u as a two digit unsigned number with a trailing ":"
\ ** character.
  .2digs  [char] : emit
;

: .date		\ day month year --
\ *G Display the date. Format is "dd/mm/yyyy".
  rot .d2  swap .d2  .4digs
;

: .USdate	\ day month year --
\ *G Display date in US format.  Format is "mm/dd/yyyy".
  swap .d2  swap .d2  .4digs
;

: .time 	\ secs mins hrs --
\ *G Display the time. Format is "hh:mm:ss".
  .t2 .t2 .2digs
;						        

: .time&date	\ secs mins hrs dd mm yyyy --		
\ *G Display the time & date.
  cr   ." The date is " 
  .date     
  cr   ." The time is "		
  .time  
;

: .current	\ --
\ *G Display current time and date.
  time&date .time&date
;


\ ======
\ *> ###
\ ======

decimal

