\ I2C790DRV.FTH - I2C Bus Hardware Level Driver for MPE ARM Dev. Kit

((
Copyright (c) 2000, 2005
MicroProcessor Engineering Ltd
133 Hill Lane
Southampton
SO1 5AF
England
tel: +44 (0)23 80 631441
fax: +44 (0)23 80 339691

and

Invend Technologies Ltd
Bradley House
Locks Hill
Rochford
Essex
SS4 1BB
England

The I2C bus consists of a bi-directional clock line and a
bi-directional data line.

Each line is pulled high via a 1k resistor. The LH77790 IO
doesn't have bi-directional port pins so one diode protected
output pin and one input pin are tied together for each line.

Port_C.7 is the data line output driver
Port_B.1 is the data line input pin

Port_C.6 is the clock line output driver
Port_B.0 is the clock line input pin

This requires
  Port_B  i/p
  Port_Ch o/p
The default initialisation ($92) also sets
  Port_A  o/p
  Port_Cl o/p
))

decimal

20 equ I2CPeriod	\ -- us
\ *G Define the I2C bit time in microseconds.

I2Cperiod system-speed 1000000 */ equ I2Cclocks	\ -- n
\ *G The number of CPU clocks in a bit time.

6 equ clocks/mem	\ --
\ *G The number of CPU clocks per 32 bit memory access. Tune this to
\ ** your hardware set up (cache, memory width and speed).
\ ** For this hardware, RAM is 16 bit and not cached.

1 equ clocks/ins	\ -- n
\ *G The number of CPU clocks per instruction. Tune this to
\ ** your hardware set up (cache, memory width and speed).
\ ** For this hardware, instructions are cached.

2 clocks/ins *  1 clocks/mem * equ clocks/callret	\ -- n
\ *G Approximate number of clocks for low level CALL and RET.


\ *****************************
\ *S Timing using "n 0 DO LOOP"
\ *****************************

((
31 clocks/ins *  10 clocks/mem *  + equ clocks/DO
\ *G the number of clocks needed to execute DO at the
\ ** start of DO..LOOP including parameter passing.
\ **  2   reg	2
\ **  3   str	6 + 2 mems
\ **  1/3 stm	2 + 3 mems
\ **  5   ldr	15 + 5 mems
\ **  2   flow  6
\ **  =================
\ **  		31 + 10 mems

9 clocks/ins *  2 clocks/mem *  + equ clocks/LOOP
\ *G the number of clocks needed to execute LOOP at the
\ ** end of DO..LOOP.
\ *E  1   ldr   3 + mem
\ **  1   add   1
\ **  1   str   2 + mem
\ **  1   bvc   3
\ **  =================
\ **            9 + 2 mems

I2Cclocks 4 / clocks/DO - clocks/LOOP / equ /I2Cqbit	\ -- n
\ *G The number n for "n 0 DO LOOP" to generate an I2C
\ ** quarter bit time.

/I2Cqbit 1 < [if]
  cr ." *** I2C requested rate too fast for CPU ***" abort
[then]

: I2Cdelay	\ --
\ *G Wait one quarter of an I2C bit. Tune the equates
\ ** above for a one quarter bit time.
  /I2Cqbit 0 do loop
;

1000000 value /TestDelay

: TestDelay	\ --
  start-clock
  ticks
  I2Cdelay
  ticks swap -
  cr . ." ms"
;
))


\ *****************************
\ *S Timing using "BEGIN UNTIL"
\ *****************************
\ *P The short delays are produced by a software delay routine
\ ** UDELAY which takes five clock cycles per iteration.

clocks/callret  6 clocks/ins * +  2 clocks/mem * +
  equ clocks/setup	\ -- n
\ *G Number of clocks required to set up for the delay.
\ *E Large value
\ ** 1    call/ret
\ ** 1    str   2 + 1 mem
\ ** 2    ldr   6 + 2 mems
\ ** ==================
\ **            5 + 3 mems
\ *E Small value
\ ** 1    call/ret
\ ** 1    str   2 + mem
\ ** 1    mov   1
\ ** 1    ldr   3 + mem
\ ** ==================
\ **            6 + 2 mems

I2Cclocks 4 /				\ quarter bit time
clocks/setup -				\ less call overhead
6 clocks/ins * /			\ loop count
  equ /I2Cqbit
\ *G The number of loops to generate an I2C quarter bit time.

/I2Cqbit 1 < [if]
  cr ." *** I2C requested rate too fast for CPU ***" abort
[then]

isLeaf
: I2Cdelay	\ n --
\ *G Waits for n iterations to produce a short delay. This
\ ** takes five clocks per iteration with build 770 of the
\ ** VFX compiler. When condition flag optimisation is added
\ ** to the compiler the number of cycles per iteration will
\ ** change.
  /I2Cqbit
  begin  1- dup 0= until		\ 5 clocks per iteration
  drop
;

((
: TestDelay	\ --
  start-clock
  ticks
  I2Cdelay
  ticks swap -
  cr . ." ms"
;
))


\ **************
\ *S Port driver
\ **************

\ internal

: sda_high      \ --
\ *G Set Data Line High.
  $0F _PPI_CTLR c!
;

: sda_low       \ --
\ *G Set Data Line Low
  $0E _PPI_CTLR c!
;

: write_sda	\ b -- ; output to SDA
\ *G Write the SDA bit.
  if  sda_high  else  sda_low  endif
;

: read_sda	\ -- 0|1
\ *G Read Level on Data Line.
  _PPI_PB c@ 1 rshift 1 and
;

: scl_high      \ --
\ *G Set Clock Line High.
  $0D _PPI_CTLR c!
;

: scl_low       \ --
\ *G Set Clock Line Low.
  $0C _PPI_CTLR c!
;

: read_scl	\ -- 0|1
\ *G Read Level on Clock Line
  _PPI_PB c@ 1 and
;

: init-i2c	\ --
\ *G Initialise the I2C ports and set SDA and SCL high
  $92 _PPI_CTLR c!
  sda_high  scl_high			\ possible stop condition
;
' init-i2c AtCold

\ external

