\ I2C bit-bang driver for EB55 board

((
Copyright (c) 1999
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)2380 631441
fax: +44 (0)2380 339691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
))

\ *! i2c_eb55
\ *T EB55 I2C driver
\ ===============
\ *S Introduction
\ ===============
\ *P The I2C interface is a software bit-banging system
\ ** using PIO bits PA1 for SCL and PA2 for SDA. The
\ ** initialisation code sets up these bits as open
\ ** drain outputs.

\ *P The driver code in I2CEB55DRV.FTH must be compiled
\ ** before the generic drive code in I2CBASE.FTH and the
\ ** device specific files (e.g. AT24C512.FTH) are
\ ** compiled.

0 equ TestI2C?	\ -- n
\ *G Set this equate non-zero to include the test code
\ ** at the end of the file. This equate also controls
\ ** the compilation of I2C test code in other files.

bit1 equ SCLmask	\ -- mask ; for SCL bit
1 equ #SCLshift		\ -- n; right shifts for SCL to bit0
bit2 equ SDAmask	\ -- mask ; for SDA bit
2 equ #SDAshift		\ -- n; right shifts for SDA to bit0
SDAmask SCLmask or
  equ I2Cmask		\ -- mask ; for SDA and SCL bits

l: I2C_init_table
\ address			data
  _APMC_BASE apmc_pcer + ,	bit13 ,		\ enable PIOA clocks
  _PIOA_BASE pio_per + ,	I2Cmask ,	\ enable PIO for I2C
  _PIOA_BASE pio_oer + ,	I2Cmask ,	\ set as outputs
  _PIOA_BASE pio_mder + ,	I2Cmask ,	\ with open drain
  _PIOA_BASE pio_ifdr + ,	I2Cmask ,	\ disable glitch filters
  _PIOA_BASE pio_idr + ,	I2Cmask ,	\ no interrupts
  _PIOA_BASE pio_sodr + ,	I2Cmask ,	\ both bits high
  0 ,		            			\ terminator
  
: initI2C	\ --
\ *G Initialise the I2C hardware.
  I2C_init_table init-io
  start-timers
; ' initI2C atCold


\ =========
\ *S Timing
\ =========

decimal

20 equ I2CPeriod	\ -- us
\ *G Define the I2C bit time in microseconds.

I2Cperiod system-speed 1000000 */ equ I2Cclocks	\ -- n
\ *G The number of CPU clocks in a bit time.

10 equ clocks/ins	\ -- n
\ *G The number of CPU clocks per instruction. Tune this to
\ ** your hardware set up (memory width and speed)

12 clocks/ins * equ clocks/DO
\ *G the number of clocks needed to execute DO at the
\ ** start of DO..LOOP including parameter passing.

3 clocks/ins * equ clocks/LOOP
\ *G the number of clocks needed to execute LOOP at the
\ ** end of DO..LOOP.

I2Cclocks 4 / clocks/DO - clocks/LOOP / equ /I2Cqbit	\ -- n
\ *G The number n for "n 0 DO LOOP" to generate an I2C
\ ** quarter bit time.

/I2Cqbit 1 < [if]
  cr ." *** I2C requested rate too fast for CPU ***" abort
[then]

: I2Cdelay	\ --
\ *G Wait one quarter of an I2C bit. Tune the equates
\ ** above for a one quarter bit time.
  /I2Cqbit 0 do loop
;


\ ====================
\ *S I2C bit functions
\ ====================

: read_scl	\ -- bit
\ *G Read SCL bit and return in l.s. bit. No delays.
  _PIOA_BASE PIO_PDSR + @ SCLmask and #SCLshift rshift
;

: write_scl	\ b -- ; output clock bit
\ *G Write to SCL with no delays.
  _PIOA_BASE swap if
    SCLmask swap PIO_SODR + !
  else
    SCLmask swap PIO_CODR + !
  endif
;

: SCL_low	\ --
\ *G Set SCL low. A quarter cycle delay is performed
\ ** before and after the transition.
  I2Cdelay
  SCLmask _PIOA_BASE PIO_CODR + !
  I2Cdelay
;

: SCL_high	\ --
\ *G Set SCL high. A quarter cycle delay is performed
\ ** before and after the transition.
  I2Cdelay
  SCLmask _PIOA_BASE PIO_SODR + !
  I2Cdelay
;

: read_sda	\ -- bit
\ *G Read SCL bit and return in l.s. bit.
  _PIOA_BASE PIO_PDSR + @ SDAmask and #SDAshift rshift
;

: write_sda	\ b -- ; output to SDA
\ *G Write the SDA bit.
  _PIOA_BASE swap if
    SDAmask swap PIO_SODR + !
  else
    SDAmask swap PIO_CODR + !
  endif
;

: sda_low	\ --
\ *G Set SDA low.
  SDAmask _PIOA_BASE PIO_CODR + !
;

: sda_high	\ --
\ *G Set SDA high.
  SDAmask _PIOA_BASE PIO_SODR + !
;


\ *********
\ test code
\ *********

TestI2C? [if]
: tcl		\ -- ; rattle the SCL line
  begin  scl_low scl_high  key? until
;

: tda		\ -- ; rattle the SCL line
  begin  sda_low sda_high  key? until
;
[then]


\ ======
\ *> ###
\ ======

