\ I2C Link Level Control (LLC) driver

((
Copyright (c) 1999, 2002
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20020928 SFP001 Tightened timing and factorisation.
		Defined I2C error code(s).
		Added switches to control ACK generation and
		checking.
))


\ ==========
\ *! i2cbase
\ *T I2C generic primitives
\ ==========
\ *P The code in *\i{I2CBASE.FTH} provides the following
\ ** words for constructing drivers for I2C devices:
\ *D START_I2C Generate a START condition
\ *D STOP_I2C  Generate a STOP condition
\ *D READ_BYTE_I2C Read a byte from the I2C lines
\ *D WRITE_BYTE_I2C Write a byte to the I2C lines
\ *D +ACK_I2C Generate and test the ACK bit
\ *D -ACK_I2C Do not generate and test the ACK bit.

\ *P The code assumes that bit banging primitives
\ ** are used to generate the I2C signals, and that
\ ** these are provided by low level code. Examples
\ ** of the low level code are provided in files
\ ** named *\i{I2CxxxDRV.FTH} where xxx indicates the
\ ** hardware.

decimal

[defined] VFXForth [if]
  ErrDef err_I2C_ACK	"I2C: Acknowledge bit set"
[else]
  #100 equ err_I2C_ACK
[then]

: start_i2c   	\ --
\ *G Define a start condition, bring SDA high-low while SCL is high
  sda_high scl_high  sda_low
  I2Cdelay I2Cdelay
;

: stop_i2c  	\ --
\ *G Define a stop condition, bring SDA low-high while SCL is high
  sda_low scl_high  sda_high
  I2Cdelay I2Cdelay
;

1 value ack_i2c?	\ -- n
\ *G If a non-zero value is used, the system will
\ ** generate the ACK bit on SDA for reads, otherwise SDA
\ ** is left high. For a non-zero value on writes, the
\ ** system will check the ACK bit and *\fo{THROW} if it is non-zero.

: +ack_i2c	\ --
\ *G check ACK cycles.
  1 to ack_i2c?
;

: -ack_i2c	\ --
\ *G Do not check ACK cycles.
  0 to ack_i2c?
;

: read_byte_i2c		\ -- in_byte			
\ *G Read a byte, MSB first. If the value I2C_ACK? is
\ ** non-zero, pull SDA low during the ACK bit, otherwise
\ ** leave it high
  scl_low sda_high
  0	        			\ blank data byte
  8 0 do				\ 8 bits to read
    scl_high
    1 lshift  read_sda or 		\ shift, read, merge
    scl_low
  loop
  ack_i2C?
  if  sda_low  else  sda_high  endif
  scl_high scl_low
  sda_high
;			    	

: write_byte_i2c	\ byte --
\ *G Write a byte by shifting, MSB first.
\ ** If ACK_I2C? is set, read ack bit on SDA, and THROW if the
\ ** ACK bit is high.
  scl_low
  8 0 do				\ 8 bits to write
    dup $080 AND write_sda		\ extract MSB and write it
    scl_high scl_low
    1 lshift				\ shift for next bit
  loop
  drop
  sda_high  scl_high
  ack_i2c? if
    read_sda if
      stop_i2c  err_i2c_ack throw	\ make I2C lines safe and THROW
    endif
  endif
  scl_low
;		      


\ ======
\ *> ###
\ ======

