\ Miscellaneous math functions

((
Copyright (c) 2001, 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20080808 SFP001 DocGenned.
))


\ =======
\ *! math
\ *T more math operations
\ =======
\ *P This file provides additional math operations. All code is
\ ** in high level Forth. Use these words as a model.


\ **********************
\ *S Doubles and triples
\ **********************

[undefined] m* [if]
: m*		\ n1 n2 -- d
\ *G Multiply two signed numbers to profuce a signed double.
  2dup xor >r
  abs swap abs um*
  r> ?dnegate
;
[then]

: ut*		\ ud1 u2 -- ut
\ *G Unsigned multiply of a double by a single to produce a triple.
  dup >r  swap >r			\ -- ud1l u2 ; R: -- u2 ud1h
  um*					\ -- ut1lm ; R: -- u2 ud1h
  r> r> um* >r 0			\ -- dutlm utm 0 ; R: -- uth
  swap 0 d+				\ -- ut ; R: -- uth
  r> +					\ -- ut
;

: ut/		\ ut1 u2 -- ud
\ *G Unsigned divide of a triple by a single to produce a double
\ ** quotient.
  dup >r um/mod  r> swap >r  um/mod  nip r> ;

: um*/		\ ud1 u2 u3 -- ud2
\ *G Unsigned multiply of *\i{ud1} by *\i{u2} and divide by
\ ** *\i{u3}. The intermediate product is a triple number.
  >r ut*  r> ut/
;

[undefined] m*/ [if]
: m*/		\ d1 n1 n2 -- d2
\ *G Signed multiply of *\i{d1} by *\i{n2} and divide by
\ ** *\i{n3}. The intermediate product is a triple number.
  2 pick 2 pick xor  over xor >r	\ -- d1 n1 n2 ; R: -- sign
  >r >r  dabs  r> abs ut*  r> abs ut/
  r> 0<
  if  dnegate  then
;
[then]

[undefined] dsqrt [if]
32bit? [if] 32 [else] 16 [then] equ #bits	\ -- u
\ *G Number of bits in a cell.

: dsqrt		\ d -- n
\ *G Single square root of a double number. This word requires
\ ** local variables. Coded versions are *\b{much} faster and
\ ** are available in some implementations. This version is
\ ** only compiled if *\fo{dsqrt} is not present.
\ ** This high level code was taken from comp.lang.forth and we
\ ** apologise for losing the attribution to the original author.
  { | a p -- }
  0 to a  0 to p			\ a=error accumulator, p=result
  #bits 0 do				\ -- d
    a 2* over 0< if 1+ then to a	\ shift MSB of d into a
    2dup d+				\ shift d
    a 2* over 0< if 1+ then to a	\ shift MSB of d into a
    2dup d+				\ shift d
    p 2* to p				\ shift p one place
    a p 2* > if
      a p 2* 1+ - to a
      p 1+ to p
    then
  loop
  2drop  p
;
[then]


\ ****************
\ *S Miscellaneous
\ ****************

: lg2		\ n -- lg2(n)
\ *G Integer base 2 logarithm.
  0 swap 2/
  begin  dup while  2/  swap 1+ swap  repeat
  drop
;

: ^^		\ n m -- n^m
\ *G Integer raise *\i{n} to the power *\i{m}.
  dup 0=
  if  2drop 1  exit  then
  abs swap >r  1
  begin
    over 1 and
    if  r@ *  then
    r> dup * >r
    swap 2/ tuck
  0= until
  nip  r> drop
;


\ *E Parity can be generated by XORing the upper and lower 16 bits,
\ ** then the upper and lower 8 bits of the resulting 16 bits
\ **   "   "    "    "    "   4  "    "  "      "      8 bits
\ **   "   "    "    "    "   2  "    "  "      "      4 bits
\ **   "   "    "    "    "   1  "    "  "      "      2 bits

: P2		\ x -- flag x
\ *G Generate an odd parity flag. Taken from comp.lang.forth.
\ ** The flag is non-zero if the number of '1' bits in the word
\ ** is odd.
  DUP >R
[ 32bit? ] [if]
  DUP 16 rshift XOR
[then]
  DUP  8 rshift XOR
  DUP  4 rshift XOR
  DUP  2 rshift XOR
  DUP  1 rshift XOR
  1 AND
  R>
;

: sqrt		\ n1 -- n2
\ *G return the square root of a single number.
  0 TUCK
  ?DO  1+  DUP 2* 1+  +LOOP
;

: magnitude	\ x y -- sqrt(x^2+y^2)
\ *G Calculate sqrt(x^2+y^2) to produce the vector magnitude.
\ ** This version uses *\fo{dsqrt}. A simple version with less
\ ** range that only uses *\fo{sqrt} is:
\ *C : magnitude   dup *  swap *  + sqrt  ;
  dup m*  rot dup m* d+ dsqrt
;


\ ======
\ *> ###
\ ======

