\ DelDirs.fth - delete directories and erase disk

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! deldirs
\ *T deleting directories
\ ===========
\ *P The code in *\i{DelDirs.fth} allows you to delete directories
\ ** recursively. By deleting from the root directory, the whole
\ ** disk can be erased.

: entValid?	\ -- flag
\ *G Are we at a valid directory entry? In terms of DOS attributes,
\ ** we only reject smudged entries and volume labels.
  dirsmudged?  $08 attrib? or		\ smudged or a volume name (long name)
  0 dir[] uaLEw@ $0202E = or		\ or '.' folder name
  0=					\ none of the above
;

: entNext	\ -- flag
\ *G Step to the next valid directory entry. Return zero
\ ** at the end of the directory, otherwise non-zero.
  begin
    OKnext  dir? 0<> and			\ next entry exists?
   while
    dir++  dir?
   while
     entValid?				\ usable entry: okay
    if  1 exit  then
  repeat then
  0
;

: toEnt		\ --
\ *G Skip to first valid entry.
  entValid? 0= if  entNext drop  then
;

: thisDir?	\ -- flag
\ *G Return true if name is '.'
  dfname 1 =  swap c@ [char] . =  and
;

compiler
: delFileEnt	\ --
\ *G Delete the current directory entry, which must be
\ ** for a file.
  _delete
;
target

: delDirEnt	\ --
\ *G Delete the current directory entry, which must be
\ ** for a directory.
  parent? if  exit  endif
  thisDir? if  exit  endif
  dircluster @ >r  get-es 2>r		\ save directory positions
  nodir dirnest				\ select directory
  delCurrDir				\ delete directory contents (yes, forward ref)
  2r> set-es  r> dircluster !		\ restore directory positions
  _delete				\ delete directory file
;

: delEntry	\ --
\ *G Delete the current directory entry.
  $0B dir[] c@ $010 and
  if  delDirEnt  else  delFileEnt  endif
;

: delCurrDir	\ --
\ *G Delete all the files and directories in the current directory.
  get-es 2>r
  dirhome  toEnt entValid? if
    begin
      dir?				\ 0 means end of directory
     while
      delEntry  entNext 0=
    until then
  endif
  2r> set-es  flushall
;

: delDisk	\ --
\ *G Delete all the files on the disk
  rootdir  delCurrDir
;

: _$rmd		\ caddr len -- ior
\ *G Delete a directory and its contents. Unprotected.
  setdir
  if  2drop err_BadPathName exit  then
  $08 _test-file if			\ not a volume ID, but hidden is ok.
    folder?
    if  delDirEnt 0  else  err_BadDir  then
  else
    err_DirNotFound
  then
;

: $rmd		\ caddr len -- ior
\ *G Delete a directory and its contents. Protected.
  +FileLock [dir
  ['] _$rmd catch ?dup
  if  nip nip  endif
  dir] -FileLock
;

((
: cd+		\ caddr len -- ior
\ *G Open new directory. *\fo{CD+} changes the directory up or down
\ ** a level.
  $08 _test-file if			\ not a volume ID, but hidden is ok.
    folder? if
      sel-folder 0			\ SFP021
    else
      err_BadDir			\ 1 = not a directory
    then
  else
    err_DirNotFound			\ 2 = directory not found
  then
;
))




\ ======
\ *> ###
\ ======

decimal
