\ PlainDir.fth - simple directory listing

((
Copyright (c) 2006 Brad Eckert
email: brad1@tinyboot.com

Copyright (c) 2007, 2008, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


Revision: 0.003
Test platform: VFX Forth for Windows

To do
=====

Change history
==============
20130313 MPE005 Added PWD. Moved PWD and CWD to FatCore.fth.
20090216 MPE004 Added selection equates, see FatCore.fth.
20081211 MPE003 Added CWD.
20081024 MPE002 Made directory listing prettier.
20070510 SFP001 Split from core file.
))

only forth definitions
decimal

\ ===========
\ *! plaindir
\ *T Simple Directory listing
\ ===========
\ ********************
\ *S Directory listing
\ ********************
\ *P The directory listing is designed to fit on a small screen
\ ** so it usually lists just filenames and not very many at that.
\ ** The cursor position is marked using a method different from
\ ** that used to mark highlighting. For example, a different
\ ** background shade for highlights and foreground shade for
\ ** the cursor.

FATopen? 0= [if]  internal  [then]

\ ********************
\ *S File size display
\ ********************
\ *P Display size in bytes, Kbytes or Mbytes. Sized to fit in a
\ ** 6-char field. K and M units are 2^10 and 2^20 scale.

: #n		\ n u --
\ *G Display n as u digits.
  0 swap FOR # NEXT
;

: ##		\ n --
\ *G Display n as two or more digits.
  0 <# # #S #> type ;

: 10^n		\ n -- 10^n
\ *G Generate 10^n.
  1 begin				\ -- n res
    over
   while
    #10 * >r 1- r>
  repeat
  nip
;

: _size		\ u scale c -- int 0
\ *G Show size split in int and frac parts.
   >r dup 2/ 0 2swap um* d+                                              \ round
   dup 0 #s #> nip 4 swap - <# r> hold swap >r             ( frac digits | int )
   dup if dup >r 10^n um* nip r> #n 2drop [char] . hold        \ fractional part
   else 2drop then r> 0 #n ;                                      \ integer part

: (size)	\ u -- addr len
\ *G Convert file size using 6 chars or less
   <#
   dup $-10000 and if                                                    \ scale?
   dup $-100000 and if $1000 [char] M else $400000 [char] K then     \ select range
   _size else 0 #n then #s #> ;                               \ 0 to 65535 bytes

: .size		( u -- ) (size) type ;
\ *G Display file size.


\ =====================
\ *S File date and time
\ =====================

: unp		\ u bits -- field u'
\ *G Unpack the low *\i{bits} bits of *\i{u} as *\i{field}
\ ** with the shifted accumulator *\i{u'}.
  >r -1 r@ lshift invert over and swap r> rshift
;

: .filedate	\ u --
\ *G Prints a 16-bit date in yyyy.mm.dd format.
  5 unp 4 unp 1980 + ## ." ." ## ." ." ##
;

: .filetime	\ u --
\ *G Prints a 16-bit time in hh:mm:ss format.
  5 unp 6 unp ## ." :" ## ." :" 2* ##
;

: d-date	( -- )  24 dir[] uaLEw@ .filedate  ;
\ *G Display file date.

: d-time	( -- )	22 dir[] uaLEw@ .filetime  ;
\ *G Display file time.


\ =====================
\ *S Display operations
\ =====================

[undefined] >pos [if]
: >pos          \ +n --
\ *G Place cursor on current line to column n if possible.
  out @ - spaces
;
[then]

: d-name	( -- )	dfullname type  ;
\ *G Display filename.

: d-size	\ --
\ *G Display file size or "folder".
  folder?
  if  s" FOLDER" type  else  dsize .size  then
;

: zcurs		\ --
\ *G Zero cursor position.
   dcurs @ FOR dirnext drop NEXT  nodir
;

variable dirmode  \ -- addr
\ *G Set non-zero to get detailed directory listing.
  1 dirmode !
variable +Dirs	\ -- addr
\ *G Set non-zero to display directories in listings.
  1 +Dirs !

\ 0         1         2         3         4         5
\ 012345678901234567890123456789012345678901234567890123456789
\ XREF.XRF       2008.08.01 14:48:00 109.8K 2

: _dirline	\ row -- row
\ *G Display filename on one line.
  dirlines over - dcurs @ =		\ highlight cursor?
  if  ." >>>"  then
  dirmode @ if
    cr d-name
    #15 >pos d-date space d-time
    #36 >pos d-size  #44 >pos cluster0 .
  else
    d-name space
  endif
;

: _dir		\ live? -- #lines
\ *G Display short list of file names.
  >r  get-es todir DirValid? if
    0 ( dirlines ) 1024
    begin
      dir? over 0<> and
     while
      r@ if  _dirline  then
      N++ dirnext 0=
      if  drop 1  then
      1-
    repeat
    drop
  else
    0                               \ no valid entries
  then
  r> drop  >r set-es r>
;

FATopen? 0= [if]  external  [then]

: dirfine	\ --
\ *G Display detailed listing for one entry.
  zcurs
  d-name  space d-date  space d-time
  space d-size  space cluster0 .
;

: delete	\ --
\ *G Deletes the highlighted file and repaints the directory.
  zcurs _delete flushall dirnext drop dir
;

: dir		\ --
\ *G Standard directory listing.
  dirhome  1 _dir drop
;


 \ ********************************
 \ *S Directory scanning operations
 \ ********************************
 \ *P Directories can be scanned recursively to count folders,
 \ ** files, and bytes. The byte tally is somewhat misleading
 \ ** because some files are hidden and the actual file sizes
 \ ** should be rounded up to the nearest cluster boundary.

((
: di{		\ -- entry sector 0
 \ *G Starts a directory scanning operation.
  get-es dirhome 0
;

: }di		\ entry sector n -- n
 \ *G Finishes a directory scanning operation, restoring the
 \ ** state at the start of the operation.
  -rot set-es
;

: dirbytes	\ -- n
 \ *G Counts the number of bytes in the current directory.
  di{
  begin  dirnext while  dsize +  repeat
  }di
;

: dcnt		\ e s S P f -- e s S P f
 \ *G WTF?
  >r  N++  r@ if
    1+ 2over get-es d=
    if  r> 1+ >r  then
  then
  r>
;

: dirspan	\ -- m n
 \ *G Calculates the file position and total number of files in
 \ ** the directory.
  di{  dcurs @ -1
  begin  dirnext while  dcnt  repeat
  drop >r  }di  r>
;

variable dfiles		\ -- addr
 \ *G Number of files in this directory.
variable dfolders	\ -- addr
 \ *G Number of folders in this directory.
variable dbytes		\ -- addr
 \ *G Number of bytes in this directory.

: scanfold	\ cluster --
 \ *G Recursively scan a directory at the given cluster.
  get-es  rot dirstart dirhome
  begin
    dirValid?
   while
    folder? parent? 0= and if
      1 dfolders +!  cluster0 recurse
    else
      1 dfiles +!  dsize dbytes +!
    then
    dirnext 0=
  until then
  set-es
;

: getsize	\ --
 \ *G Tally up the features of this folder.
  0 dfiles !  0 dfolders !  0 dbytes !
  dircluster @ scanfold
;

: stats		\ --
 \ *G Display properties of the current folder.
  getsize
  dfiles @ . ." Files, " dfolders @ . ." Folders, " dbytes @ . ." Bytes."
  cr ." Disk usage: " UsedClusters c>b . ." of " TotalClusters c>b . ." bytes."
;
))


(( currently unused
: dcbump	( offset -- )	dcurs @ + 0 max dcurs !  ;
 \ *G Step directory cursor by given offset.

: cursatend	( -- n )   0 _dir dcurs @ - 1-  ;

: diralign	\ --
 \ *G Attempt to align ROI to cursor.
  begin
    dcurs @ 0<
   while
    dirprev       \ cursor is before ROI
    if  1 dcurs +!  else  0 dcurs ! exit  then \ error: reset cursor to top
  repeat
  begin
    [ dirlines 1- ] literal dcurs @ <
   while
    dirnext
    if  -1 dcurs +!  else  [ dirlines 1- ] literal dcurs ! exit  then
  repeat
;

: _dirdn	\ flag --
 \ *G Line down (toward end)
  diralign cursatend ?dup if
    0<
    if  -1 dcbump  else 1 dcbump  then
  else
    dirnext drop cursatend		\ scroll down
    if  dirprev drop  then		\ but not if it shows blank space
  then
  _dir drop
;

: _dirup	\ flag --
 \ *G Line up (toward beginning).
  diralign  dcurs @                    \ top of page?
  if   -1 dcurs +!                     \ no, bump the cursor
  else dirprev drop                    \ scroll the page if possible
  then
  _dir drop
;

: dirup  ( -- )  1 _dirup ;
: dirdn  ( -- )  1 _dirdn ;
: dirpu  ( -- )  dirlines FOR 0 _dirup NEXT  dir ;
: dirpd  ( -- )  dirlines FOR 0 _dirdn NEXT  dir ;
))




\ ======
\ *> ###
\ ======

decimal

