\ VFXharn.fth - compatibility harness for VFX Forth for Windows.

((
Copyright (c) 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

\ ==========
\ *! vfxharn
\ *T VFX Forth harness
\ ==========
\ *P The harness files permit you to optimise the FAT file
\ ** system for speed or code size.

decimal

\ ******************
\ *S Sector handling
\ ******************
\ *P How you code these will depend on the host CPU and Forth
\ ** implementation.

: secs>bytes	\ u -- #byes
\ *G Multiply a sector number by the number of bytes per sector.
\ ** Optimise this for performance.
  secshift lshift
;

: bytes>secs	\ #bytes -- #secs
\ *G Divide a number of bytes by the number of bytes per sector,
\ ** returning the number of complete sectors.
\ ** Optimise this for performance.
  secshift rshift
;

: bytes>offsecs	\ #bytes -- offset #secs
\ *G Divide a number of bytes by the number of bytes per sector,
\ ** returning the offset in a sector and the number of complete
\ ** sectors.
\ ** Optimise this for performance.
  dup secsizemask and  swap secshift rshift
;


\ ********
\ *S Tools
\ ********
\ *P These tools should be compiled as required.

((
: buffer:	\ u -- ; -- addr
\ *G Reserve *\i{u} bytes of uninitialised read/write data.
  create allot
;

: scan		\ addr len char -- addr' len'
\ *G Find char in string.
  >r
  begin
    dup
   while
    over c@ r@ <>
   while
    1 /string
  repeat then
  r> drop
;

: upc		\ char -- char'
\ *G Convert *\i{char} to upper case.
  dup [char] a [char] z 1+ within
  if  32 -  then
;
))

: FOR		\ -- ; n -- ; R: -- n'
\ *G Starts a loop that is executed one or more times. At runtime,
\ ** the loop is entered with the loop count on the stack. In the
\ ** body of the loop the loop index is on the return stack and
\ ** counts down from n-1 to zero. Use in the form:
\ *C  ( -- n ) FOR ... NEXT ( -- )
  postpone begin  postpone dup  postpone while  postpone 1-  postpone >r
; immediate

: NEXT		\ -- ; R: n -- ; -- [n]
\ *G Terminates a *\fo{[m ... m]} structure, discarding the loop
\ ** index.
  postpone r> postpone repeat postpone drop
; immediate

: ucompare	\ a1 u1 a2 u2 -- notsame?
\ *G Case insensitive string compare. Returns zero for a match.
  rot over <>
  if  drop 2drop 1  exit  then		\ not the same length
  FOR					\ -- a1 a2
    swap count upc rot count upc rot <>	\ --
    if  r> drop  2drop 1  exit then
  NEXT
  2drop 0
;

: ucmove	\ src dest len --
\ *G As *\fo{CMOVE} but characters are converted to upper case.
  bounds
  ?do  dup c@ upc i c!  1+  loop
  drop
;

: N++		\ n1 x -- n1+1 x
\ *G Increment *\i{n1}.
  swap 1+ swap
;

: byte-split	\ w -- lo hi
\ *G Split a 16 bit item into its low and high bytes.
  dup #255 and swap 8 rshift #255 and
;

: byte-join	\ lo hi -- w
\ *G Convert two bytes into a 16 bit item.
  8 lshift +
;

: word-split	\ x -- wlo whi
\ *G Split a 32 bit item into its low and high 16 bit items.
  dup #65535 and swap 16 rshift #65535 and
;

: word-join	\ wlo whi -- x
\ *G Convert two 16 bit items into a 32 bit item.
  $FFFF and #16 lshift swap $FFFF and or
;

: qlog2		\ n -- log2[n]
\ *G Given *\i{n}, a power of two, return its power.
  -1 swap
  begin  dup while  2/ N++  repeat
  drop
;

: @C @ ;
\ *G Fetch cell from code space (e.g. for tables).

-412 #ErrDef err_create		"File create error"
-413 #ErrDef err_open		"File open error"
-414 #ErrDef err_read		"File read error"
-415 #ErrDef err_write		"File write error"
-416 #ErrDef err_close		"File close error"
-417 #ErrDef err_seek		"File seek error"
\ -418 #ErrDef err_dict		"Dictionary corrupt at DICTBAD"
-419 #ErrDef err_handle		"File handle bad"
-420 #ErrDef err_NoHandles	"No more file handles"
-421 #ErrDef err_RootFull	"No room left in the root directory"
-422 #ErrDef err_FATparams	"Error in FAT parameters"
-423 #ErrDef err_FileSize	"File size longer than 32-bit"
-424 #ErrDef err_FileNotFound	"File not found"
-425 #ErrDef err_BadDir		"Not a valid directory"
-426 #ErrDef err_DirNotFound	"Directory not found"
-427 #ErrDef err_w/o		"Attempt to read W/O file"
-428 #ErrDef err_r/o		"Attempt to write R/O file"
-429 #ErrDef err_BadPathName	"Invalid created pathname"
-430 #ErrDef err_BadFileName	"Invalid created filename"
-431 #ErrDef err_DirFull	"Full directory"
-432 #ErrDef err_NotDir		"Invalid directory name"
-433 #ErrDef err_BadPos		"Reposition beyond end of file"
-434 #ErrDef err_RawRead	"Error reading raw sector"
-435 #ErrDef err_RawWrite	"Error writing raw sector"


(( embedded systems etc.
\ renumbered IORs:
\  err_read     1/2/3 = error reading device (hardware specific from rawread)
\  err_write     4/5/6 = error writing device (hardware specific from rawwrite)
7 equ err_NoHandles		\ not enough free file handles
8 equ err_RootFull		\ no room left in the root directory
9 equ err_FATparams		\ error in FAT parameters
10 equ err_FileSize		\ file size longer than 32-bit
11 equ err_FileNotFound		\ file was not found
12 equ err_BadDir		\ not a valid directory
13 equ err_DirNotFound		\ directory was not found
14 equ err_w/o			\ 14 = attempt to read W/O file
15 equ err_r/o			\ attempt to write R/O file
16 equ err_BadPathName		\ invalid created pathname
17 equ err_BadFileName		\ invalid created filename
18 equ err_DirFull		\ full directory
19 equ err_NotDir		\ invalid directory name
20 equ err_BadPos		\ Reposition beyond end of file

\ should be:
\ renumbered IORs:
\ xxx equ err_FATfiler
\  err_read     1/2/3 = error reading device (hardware specific from rawread)
\  err_write     4/5/6 = error writing device (hardware specific from rawwrite)
  6 +
  dup equ err_NoHandles  1+	\ not enough free file handles
  dup equ err_RootFull  1+	\ no room left in the root directory
  dup equ err_FATparams  1+	\ error in FAT parameters
  dup equ err_FileSize  1+	\ file size longer than 32-bit
  dup equ err_FileNotFound  1+	\ file was not found
  dup equ err_BadDir  1+	\ not a valid directory
  dup equ err_DirNotFound  1+	\ directory was not found
  dup equ err_w/o  1+		\ 14 = attempt to read W/O file
  dup equ err_r/o  1+		\ attempt to write R/O file
  dup equ err_BadPathName  1+	\ invalid created pathname
  dup equ err_BadFileName  1+	\ invalid created filename
  dup equ err_DirFull  1+	\ full directory
  dup equ err_NotDir  1+	\ invalid directory name
  dup equ err_BadPos  1+	\ Reposition beyond end of file
drop
))


\ *********************
\ *S Alignment checking
\ *********************
\ *P Some CPUs require aligned memory accesses. The code here
\ ** checks alignment at run-time.

1 [if] \ Test for unaligned memory access:
: @
  dup 3 and if
    .rs  1 abort" Unaligned 32-bit read"
  endif
  @ ;
: !   dup 3 and if  .rs 1 abort" Unaligned 32-bit write"  endif ! ;
: W@  dup 1 and if  .rs 1 abort" Unaligned 16-bit read"  endif w@ ;
: W!  dup 1 and if  .rs 1 abort" Unaligned 16-bit write" endif w! ;
[then]


\ *********************************
\ *S Embedded systems compatibility
\ *********************************
\ *P Embedded systems may have words *\fo{TARGET} and *\fo{HOST}
\ ** that may be ignored.
\ *[
[undefined] target [if]
: target ;
: host ;
[then]
\ *]

: cvariable  variable  ;	\ -- ; -- addr
\ *G In embedded systems, *\fo{CVARIABLE} can be used to reduce
\ ** RAM use.

: @P	@  ;	\ addr -- x
: W@P   w@  ;
: c@p	c@  ;

: @C	@  ;	\ addr -- x
: W@C   w@  ;
: c@C	c@  ;


\ **************************
\ *S Unaligned memory access
\ **************************
\ *P Systems that require aligned accesses for 16 and 32 bit values
\ ** must provide unaligned access words. Similarly, big-endian CPUs
\ ** must provide little-endian access as all the FAT data structure
\ ** items are little-endian items.

: uaLEw@	\ caddr -- w
  dup c@  swap 1+ c@ 8 lshift or
;

: uaLE@		\ caddr -- x
  dup uaLEw@  swap 2+ uaLEw@ #16 lshift or
;

: uaLEw!	\ w caddr --
  2dup c!  swap 8 rshift swap 1+ c!
;

: uaLE!		\ w caddr --
  2dup uaLEw!  swap #16 rshift swap 2+ uaLEw!
;


\ ======
\ *> ###
\ ======

decimal


