\ ARP.fth - ARP for Ethernet systems

((
Copyright (c) 1997-2005, 2007, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999

  CEM Systems Ltd, Belfast, 2000-2001

To do
=====

Change history
==============
20090206 SFP005	Removed ARPfailed.
		Added SendGratuitousArp.
		Major overhaul (unmarked).
		Protection against gratuitous ARPs.
20070214 SFP004 ARP code moved to ARP.fth to help with DHCP.
		Added ARPfailed.
		IPROUTE_LIFE = 0 = infinite life.
20050216 EGS003 The (N) nightmare.
20040801 CJD002 Prevented leak on ARP error.
20030621 SFP001 Fetches from IPADDRESS are network order.
20000221 SJC000 Use network order ! rather than a CMOVE to copy
                IPAddress to network packet. 4 CMOVE assumes that
                the host variable IPAddress is big-endian.
))


\ ===========
\ *! arp
\ *T ARP handler
\ ===========
\ *P *\i{ARP.fth} handles ARP packets for Ethernet and DHCP.

(( for DocGen
$E1 equ ETHER_PORT	\ -- n
\ *G Port identifiers for Ethernet devices are in the range
\ ** $E1..EF
))

create EtherBCastAddress	\ -- addr
\ *G Holds the Ethernet broadcast address FF-FF-FF-FF-FF-FF.
  $FFFFFFFF ,(n) $FFFF w,(n)

create EtherUnkAddress	\ --
\ *G Holds the Ethernet "unknown" address, all zeros.
  0 ,(n)  0 w,(n)

: TurnEtherPacketRound	\ *eth_hdr --
\ *G Return the packet to the sender.
  dup ether_src over ether_dest CopyEthAdd	\ source -> dest
  EtherAddress swap ether_src CopyEthAdd	\ me -> source
;

: HandleArpRequest	\ *pb ad *eth *arp --
\ *G Returns reply to an incoming ARP request.
  dup ether_sender			\ pointer to sender area
  over ether_target			\ pointer to target area
  ETHER_ADDRESS_LEN IP_ADDRESS_LEN +	\ length of sender data
  cmove					\ move sender into target
  \ -- *pb ad *eth *arp
  EtherAddress 				\ now add my Ethernet address as sender
  over ether_sender			\ pointer to sender area
  CopyEthAdd
  \ -- *pb ad *eth *arp
  IPAddress @(n)			\ now add my IP address as sender ; SFP001
  swap ip_sender !(n)			\ pointer to sender area
  \ -- *pb ad *eth
  ARPOP_REPLY swap ar_op w!(n)		\ set the op code to reply
  \ -- *pb ad
  TurnEtherPacketRound			\ now swap the addresses round
  TxEnetQ enqueue			\ and finally send the packet
;

XC? [if]
: AddEtherRoute	\ *etheraddr ipaddr --
\ *G Add an Ethernet route given a pointer to a 6 byte Ethernet address
\ ** and an IP address.
[else]
: rtAddEtherRoute	\ *etheraddr ipaddr --
[then]
  add_route dup if			\ add ip address to routing table
    \ -- *etheraddr rt
    ETHER_PORT over iproute_port w!	\ set port as Ethernet
    ROUTE_LIFE over setRouteLife	\ restart lifetime ; SFP004
    iproute_ethadd copyEthAdd		\ save Ethernet address
  else
    2drop
  endif
;
XC? 0= [if] assign rtAddEtherRoute to-do AddEtherRoute  [endif]

: HandleArpReply	\ *pb *arp --
\ *G Handle an incoming ARP reply - so extract info.
  dup ether_sender			\ Ethernet address of respondent
  swap ip_sender @(n)			\ IP address of respondent
  AddEtherRoute				\ *pb
  FreeQB				\ all done - so free up the packet
;

\ SFP005...
: ArpForMe?	\ *arp -- flag ; nz=for me
\ *G Given the address of an incoming ARP payload, return true
\ ** if the packet should be handled.
  dup ip_target @(n) IPAddress @(n) = if	\ for us
    dup ip_target @(n) over ip_sender @(n) <>	\ not a looped Gratuitous ARP
    if  drop true  exit  endif			\  so we want it
  endif
  drop 0
;
\ ...SFP005

-short-branches
: EtherArpPacket	{ *pb | ad ln *eth *arp -- }
\ *G Process an incoming ARP packet. The pbuf holds the received
\ ** Ethernet packet. We use the incoming packet buffer for the reply.
\ cr ." In EtherArpPacket"
  *pb p_dat -> ad			\ set up data address
  *pb p_len w@ -> ln			\ and data length
  ln arp_hdr arp_ip_data + < if		\ check packet is long enough
[ diags? ] [if]  ." ARP Short packet"  [then]	\ packet too small
    *pb FreeQB				\ so free up the packet ; CJD002
  else
    ad ether_hdr + -> *eth		\ OK go to op code
    *eth ar_op w@(n) ARPOP_REQUEST = if	\ if a request, check for us
      *eth arp_hdr +			\ point to arp packet data start
      -> *arp
      *arp ArpForMe? if			\ SFP005
        *pb ad *eth *arp
	HandleArpRequest		\ so go send a reply
      else				\ nope not for us so we can ignore it
        *pb FreeQB			\ so free up the packet
      endif
    else
      *eth ar_op w@(n) ARPOP_REPLY = if	\ is this a reply?
        *eth arp_hdr +			\ point to arp packet data start
        -> *arp
        *arp ArpForMe? if		\ SFP005
          *pb *arp
          HandleArpReply		\ so handle the reply
        else				\ nope not for us so we can ignore it
          *pb FreeQB			\ so free up the packet
	endif
      else
[ diags? ] [if]
        cr ." ARP unhandled opcode"	\ don't handle this type (YET)
[then]
        *pb FreeQB			\ so free up the packet
      endif
    endif
  endif
;

create ProtoArpHdr	\ -- addr
\ *G Prototype ARP request header.
  ARPHARD_ETHER w,(n)			\ Ethernet
  ARPPROTO_IP w,(n)			\ IP
  6 c,  4 c,				\ Ethernet & IP address sizes
  ARPOP_REQUEST w,(n)

: FillArpData	\ ipaddr *arpdata --
  >r					\ -- ipaddr ; R: -- *arpdata
  EtherAddress r@ ether_sender CopyEthAdd	\ sender Ethernet address
  IpAddress @(n) dup r@ ip_sender !(n)	\ sender IP address
  over =					\ broadcast if gratuitous ARP
  if  EtherBCastAddress else  EtherUnkAddress  endif
  r@ ether_target copyEthAdd	\ pointer to sender area
  r> ip_target !(n)		\ target IP address
;

: SendArpRequest	\ ipaddr --
\ *G Formats and sends out an ARP request packet. This is always a
\ ** broadcast.
\ debug_cr "" In SendArpRequest" debug_count&type
  GetPbuf dup 0=			\ get a buffer for the packet
  if  2drop exit  endif

  \ -- ipaddr pbuf
  dup p_dat				\ Ethernet header at start
  EtherAddress over ether_src CopyEthAdd		\ my Ethernet address as sender
  EtherBCastAddress over ether_dest CopyEthAdd	\ broadcast Ethernet address as dest
  ETHERTYPE_ADDRESS_RESOLUTION		\ $0806
    swap ether_type w!(n)		\ set ether pkt type to ARP

  \ -- ipaddr pbuf
  dup p_dat ether_hdr +			\ ARP header follows
  ProtoArpHdr swap arp_hdr cmove	\ fill in the ARP header structure

  \ -- ipaddr pbuf
  tuck p_dat ether_hdr + arp_hdr + FillArpData

  \ -- pbuf
  ether_hdr arp_hdr + arp_ip_data + over \ total packet length
  2dup !PBDataLen  p_len w!
  TxEnetQ enqueue			\ and finally send the packet
;

: SendGratuitousArp	\ --
\ *G Send a "Gratuitous ARP Request".
  IpAddress @(n) SendArpRequest
;


\ ======
\ *> ###
\ ======

