\ *! intro
\ *T Introduction
\ ========

\ *******************
\ *S What Do You Get?
\ *******************
\ *P PowerNet is a TCP/IP networking stack code written in Forth
\ ** with support for:
\ *( 1
\ *B Ethernet Layer. Reference driver for SMSC LAN91C9x series of
\ ** Ethernet chips. Includes a packet sniffer to test reception and
\ ** Ping to test transmit and receive. Additional Ethernet drivers
\ ** can be found in the CPU-specific drivers directories supplied
\ ** with your Forth cross-compiler.
\ *B SLIP Layer. Uses a serial Interface and multiple connections
\ ** are supported
\ *B Routing Table. IP/Hardware routing table, with life-timers on routes.
\ *B IP Layers.
\ *( a
\ *B ICMP handler. ECHO Request coded for PING, other codes have stubs.
\ *B UDP handler. Unconnected data packets.
\ *B DHCP client. Permits almost automatic configuration.
\ *B DNS client. Converts URLs to IP addresses.
\ *B SNTP client. Used to synchronise the local clock.
\ *B TCP handler. Connected data streams.
\ *)
\ *B Socket Layer. The sockets interface being used is based on
\ ** BSD/Winsock
\ *B Services Layer
\ *( a
\ *B Primitive TFTP client (Trivial File Transfer Protocol)
\ *B Multi-threaded Telnet server for Forth Interpreter over TCP/IP
\ *B Multi-threaded HTTP server with ASP and CGI.
\ *B Modbus server template.
\ *B Echo server
\ *)
\ *B Examples
\ *( a
\ *B Sending mail by SMTP.
\ *B ModBus client and server templates.
\ *B POST handler with application binary updater.
\ *B Configuration tools.
\ *)
\ *)


\ ****************
\ *S Documentation
\ ****************
\ *P The main commentary on the code and all the glossaries are
\ ** generated directly from the source by the DocGen utility supplied
\ ** with MPEs VFX Forth for Windows. Both PDF and HTML versions are
\ ** provided.

\ *P The code and its *\i{DocGen} documentation should be considered
\ ** the primary reference. Please read it.


\ **************
\ *S Source Tree
\ **************
\ *E PowerNet
\ ** |
\ ** +------ ARP                      Address Resolution Protocol
\ ** +------ DEFINES.FTH              Various IP Constants
\ ** +------ DHCP.FTH                 DHCP client
\ ** +------ DIAGS.FTH                Diagnostic / Test routines
\ ** +------ DNS.fth                  DNS client
\ ** +------ GLOBALS.FTH              Global Vars/Buffers
\ ** +------ ICMP.FTH                 ICMP Packet handlers
\ ** +------ IP.FTH                   LowLevel IP access
\ ** +------ NETCODE.FTH		 Network and CPU dependent code
\ ** +------ POWERNET.BLD             Stack Build File
\ ** +------ PRIMITIV.FTH             Useful code fragments
\ ** +------ ROUTING.FTH              Routing table handler
\ ** +-------SNTP.fth                 SNTP client.
\ ** +------ STRUCTS.FTH              Various IP/BSD Structures
\ ** +------ TCP.FTH                  TCP Protocol Handler
\ ** +------ UDP.FTH                  UDP Protocol Handler
\ ** +------ USERVARS.FTH             Required System Variables
\ ** |
\ ** +------ ETHER
\ ** |       +------ ETHTASK.FTH      Ethernet TxRx Dispatcher Task
\ ** |       +------ SMC91C9X.FTH     Reference Ethernet Driver
\ ** |
\ ** +------ SLIP
\ ** |       +------ SLIPCOM.FTH      SLIP interface
\ ** |
\ ** |------ EXAMPLES
\ ** |       +------- SMTPmail        SMTP mail examples
\ ** |       +------- WebConfig.fth   Application Configuration
\ ** |       +------- WebPost.fth     POST and binary update
\ ** |       +------- MBusClnt.fth    Modbus framework
\ ** |
\ ** +------ SOCKETS
\ ** |       +------ BSD.FTH          BSD Sockets API
\ ** |       +------ SOCKPRIM.FTH     Low Level Socket struct manip.
\ ** |
\ ** +------ QUEUES
\ ** |       +------ PBUFFERS.FTH     Queue Buffer handlers
\ ** |       +------ QPRIMS.FTH       En/DeQueue code
\ ** |       +------ QUEUES.FTH       Describe/initialise queues
\ ** |       +------ RAM.FTH          Describe TxRx RAM Useage
\ ** |
\ ** +------ SERVICES
\ ** |       +------ HTTP.FTH         MultiThreaded Web Server
\ ** |       +------ MbusSrvr         ModBus Server framework
\ ** |       +------ Pages.fth        Example web page handlers
\ ** |       +------ Servers.fth      Server core code
\ ** |       +------ TcpEcho          TCP Echo Server
\ ** |       +------ TELNET.FTH       Multi-threaded Telnet Server
\ ** |       +------ TFTP.FTH         Simple TFTP Server
\ ** |
\ ** +------ TestPages		 Example web pages
\ ** |
\ ** +------ CodeManual		 PDF and HTML manuals


\ ======
\ *> ###
\ ======
