\ DNS.fth - DNS client implementation for UDP

((
Copyright (c) 2010, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
))


\ ======
\ *! dnsclient
\ *T DNS client
\ ======
\ *P The code in *\fo{DNS.fth} implements a simple DNS client.
\ ** The focus of the design is on minimising overall RAM usage.
\ ** All data is transferred using UDP.

\ *P The code is compiled after the BSD layer because it makes
\ ** heavy use of that layer.

\ *P The main documentation for DNS is contained in RFCs 1034 and
\ ** 1035. The language is abstruse at best.


\ ****************
\ *S Configuration
\ ****************
\ *P These equates and data are only used if not already defined.

[undefined] DNSdebug? [if]
1 equ DNSdebug?		\ -- x
\ *G Set this non-zero to display DNS debug information during
\ ** execution.
[then]

[undefined] DNSport# [if]
#53 equ DNSport#	\ -- port#
\ *G The standard DNS port number on the server.
[then]

[undefined] #DNS [if]
3 equ #DNS		\ -- u
\ *G Maximum number of DNS attempts if no response.
[then]

[undefined] /DNSms [if]
5000 equ /DNSms		\ -- ms
\ *G Maximum time (in milliseconds) allowed for a DNS transaction.
[then]

[undefined] /DNS [if]
#512 equ /DNS		\ -- u
\ *G Maximum size of the DNS payload.
[then]

[undefined] DNSserver [if]
variable DNSserver	\ -- addr
\ *G Holds DNS server or 0 if not configured yet.
  DNSserver off
[then]


\ ************************
\ *S Queries and responses
\ ************************
\ *P For the full details of DNS queries and responses, see RFCs
\ ** 1034 and 1035. What follows in this documentation is a small
\ ** subset.

\ *P The header (RFC 1035, 4.1.1) contains the following fields:
\ *E                                     1  1  1  1  1  1
\ **       0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                      ID                       |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |QR|   Opcode  |AA|TC|RD|RA|   Z    |   RCODE   |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                    QDCOUNT                    |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                    ANCOUNT                    |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                    NSCOUNT                    |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                    ARCOUNT                    |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+

\ *P A query section contains the following fields, QNAME
\ ** (RFC 1035, 4.1.2), QTYPE (RFC 1035, 3.2.2) and QCLASS
\ ** (RFC 1035, 3.2.4/5).

\ *P The question section is used to carry the "question" in most queries,
\ ** i.e., the parameters that define what is being asked.  The section
\ ** contains QDCOUNT (usually 1) entries, each of the following format:
\ *E       0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                                               |
\ **     /                     QNAME                     /
\ **     /                                               /
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                     QTYPE                     |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                     QCLASS                    |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+

\ *P where:
\ *D QNAME           a domain name represented as a sequence of labels, where
\ **                 each label consists of a length octet followed by that
\ **                 number of octets.  The domain name terminates with the
\ **                 zero length octet for the null label of the root.  Note
\ **                 that this field may be an odd number of octets; no
\ **                 padding is used.
\ *D QTYPE           a two octet code which specifies the type of the query.
\ **                 The values for this field include all codes valid for a
\ **                 TYPE field, together with some more general codes which
\ **                 can match more than one type of RR.
\ *D QCLASS          a two octet code that specifies the class of the query.
\ **                 For example, the QCLASS field is IN for the Internet.

\ *P The answer, authority, and additional sections all share the same
\ ** format: a variable number of resource records, where the number of
\ ** records is specified in the corresponding count field in the header.
\ ** Each resource record has the following format:
\ *E                                     1  1  1  1  1  1
\ **       0  1  2  3  4  5  6  7  8  9  0  1  2  3  4  5
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                                               |
\ **     /                                               /
\ **     /                      NAME                     /
\ **     |                                               |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                      TYPE                     |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                     CLASS                     |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                      TTL                      |
\ **     |                                               |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     |                   RDLENGTH                    |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--|
\ **     /                     RDATA                     /
\ **     /                                               /
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+

\ *P where:
\ *D NAME            a domain name to which this resource record pertains.
\ **
\ *D TYPE            two octets containing one of the RR type codes.  This
\ **                 field specifies the meaning of the data in the RDATA
\ **                 field.
\ **
\ *D CLASS           two octets which specify the class of the data in the
\ **                 RDATA field.
\ **
\ *D TTL             a 32 bit unsigned integer that specifies the time
\ **                 interval (in seconds) that the resource record may be
\ **                 cached before it should be discarded.  Zero values are
\ **                 interpreted to mean that the RR can only be used for the
\ **                 transaction in progress, and should not be cached.
\ *D RDLENGTH        an unsigned 16 bit integer that specifies the length in
\ **                 octets of the RDATA field.
\ **
\ *D RDATA           a variable length string of octets that describes the
\ **                 resource.  The format of this information varies
\ **                 according to the TYPE and CLASS of the resource record.
\ **                 For example, the if the TYPE is A and the CLASS is IN,
\ **                 the RDATA field is a 4 octet ARPA Internet address.

\ *P In order to reduce the size of messages, the domain system utilizes a
\ ** compression scheme which eliminates the repetition of domain names in a
\ ** message.  In this scheme, an entire domain name or a list of labels at
\ ** the end of a domain name is replaced with a pointer to a prior occurance
\ ** of the same name.

\ *P The pointer takes the form of a two octet sequence:
\ *E     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
\ **     | 1  1|                OFFSET                   |
\ **     +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+

\ *P The first two bits are ones.  This allows a pointer to be distinguished
\ ** from a label, since the label must begin with two zero bits because
\ ** labels are restricted to 63 octets or less.  (The 10 and 01 combinations
\ ** are reserved for future use.)  The OFFSET field specifies an offset from
\ ** the start of the message (i.e., the first octet of the ID field in the
\ ** domain header).  A zero offset specifies the first byte of the ID field,
\ ** etc.


\ ********
\ *S Tools
\ ********

4 buffer: DNSid	\ -- addr
\ *G Holds the DNS transaction number. The number is incremented
\ ** before use so that a good response should have the same
\ ** number.

create DNSQtemplate	\ -- addr
\ *G Template for a DNS query.
  $0000 w,(n)				\ ID
  $0100 w,(n)				\ RD
  $0001 w,(n)				\ 1 question
  0 w,(n)  0 w,(n)  0 w,(n)

#12 equ /DNSQtemplate	\ -- len
\ *G Length of the DNS query template.

: genDNSQhead	\ addr -- addr
\ *G Generate a DNS query header at *\i{addr} and return the
\ ** address just after the header.
  >r
  DNSQtemplate r@ /DNSQtemplate cmove	\ copy template
  DNSid @ 1+  dup r@ w!(n)  DNSid !	\ set transaction number
  r> /DNSQtemplate +			\ next addresss
;

: c!++		\ addr b -- addr+1
\ *G Saves *\i{b} at *\i{addr} and increments *\i{addr}.
  over c!  1+  ;

: genLabel	\ caddr len addr -- addr'
\ *G Add a label (n + n bytes) to the text at addr, returning
\ ** the next label address.
  over c!++  -rot bounds
  ?do  i c@ c!++  loop
;

: genDNSname		\ caddr len addr -- addr'
\ *G Generate a name as a sequence of labels.
  >r  begin
    dup
   while				\ -- caddr len ; R: -- addr
    [char] . split$RL r> genLabel >r	\ next left label
  repeat
  r> genLabel				\ lay terminator
;

: genDNSquestion	\ caddr len addr -- addr'
\ *G Generate the question section of a query.
  genDNSname
  $0001 over w!(n)  2+			\ QTYPE
  $0001 over w!(n)  2+			\ QCLASS
;


: skipDNSQ	\ caddr len -- caddr' len'
\ *G Skip the query portion of a DNS reponse.
  0 scan dup 				\ to end of text
  if  5 /string  endif			\ 1 for zero, 4 for data
;

: skipRRname	\ caddr len -- caddr' len'
\ *G Step over the name field in a resource record (RR).
  over c@ $C0 and $C0 = if		\ compresed pointer?
    2 /string
  else
    0 scan dup
    if  1 /string  endif
  endif
;

: cleanDNSq	\ hs buffer --
\ *G Clean up the query by releasing the buffer and closing the
\ ** socket.
  ?dup if  free drop  endif		\ release buffer
  ?dup if  closesocket drop  endif	\ close UDP socket
;


\ *************
\ *S User words
\ *************
\ *P After acquisition of the DNS server's IP address, either by
\ ** specification or by DHCP, the only function provided is to
\ ** convert a domain name string, e.g. *\i{www.google.com} into
\ ** an IPv4 address.

: DNSquery	\ caddr len -- ip|0
\ *G Send a UDP DNS query message requesting the IP address of the
\ ** given domain name. Returns the IP address for success, or zero
\ ** on failure.
  { | hs *dns tid rxlen -- }

  0 -> hs  /DNS allocate		\ DNS query buffer
  if  3drop 0 exit  endif
  -> *dns
  DNSserver @(n) 0 DNSport# UDPconnect if  \ -- caddr len hs
    hs *dns cleanDNSq  3drop 0 exit
  endif
  -> hs

  *dns genDNSQhead genDNSquestion >r	\ form query
  DNSid @ $FFFF and -> tid		\ save transaction number
  hs *dns dup r> swap - 0		\ -- hs buff len 0
  send SOCKET_ERROR = if
    hs *dns cleanDNSq  0 exit
  endif

  /DNSms later begin 			\ -- ticks ; wait response
    hs pollSocket #16 <=
   while
    dup expired
  until then
  drop
  hs pollsocket #16 <= if		\ len <= 16 is bad response?
    hs *dns cleanDNSq  0 exit
  endif

  *dns /DNS erase			\ wipe buffer
  hs *dns /DNS 0 recv 0 max -> rxlen
  0					\ assume bad response
  *dns w@(n) tid = if			\ transaction ID good
    *dns 2+ w@(n) $F80F and $8000 = if	\ good response
      *dns 4 + w@(n) 1 = if		\ query copied back
        *dns 6 + w@(n) 1 >= if		\ got a response
          *dns rxlen 12 /string		\ -- 0 caddr len ; skip header
          skipDNSQ			\ skip query
          skipRRname 8 /string		\ skip RR name, type, class, TTL
          over w@(n) 4 = if		\ IPv4?
            drop nip  2 + @(n)		\ discard len and 0, return address
          else
            2drop			\ discard caddr len, leave zero
          endif
        endif
      endif
    endif
  endif

  hs *dns cleanDNSq
;

((
: tdns
  s" www.demon.net" DNSquery
;
))
