\ SMC91C9x Ethernet driver

((
Copyright (c) 1998-2003, 2005, 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20081125 MGS008 Final fix for lock up problem in transmitter.
20081125 SFP007 Added ETHERLINK? for PowerNet v4.5.
20080826 SFP006 Major overhaul for error detection in transmit.
20080625 SFP005 Clear TX errors at start of SEND_ETHER_PKT by
		enabling transmission. Some refactoring.
20050302 SFP004 Implemented ISTX? fully.
		Made QUEUETX a macro.
		Added GENERICIP? conditional.
20030630 SFP003 Fixed Odd/Even problem in GET_ETHER_PKT
20020627 SFP002 Updated documentation.
20020618 SFP001 Revamped for tidiness and PowerNet v3+.
))


\ ===========
\ *! Smc91C9x
\ *T SMC LAN91C92/4/6 Ethernet Driver Code
\ ===========
\ *S Introduction
\ *P The file *\i{Smc91C9x.fth} contains the hardware driver layer
\ ** for the Standard MicroSystems LAN91C92/4/6 Ethernet controller
\ ** chips as used on the MPE ARM Development Kit, MPE/Hiden SA1110
\ ** StrongBox and other MPE boards.
\ ** The data sheets for these devices may be found at www.smsc.com and
\ ** the part number for the MPE board is LAN91C96I, which can be used
\ ** at 5v or 3.3v.

\ *P The controller has 64 8 bit registers organised as four banks of
\ ** 16.

\ *P The code is written for portability rather than speed and
\ ** uses byte accesses only. This permits the code to be used
\ ** without change on big and little endian CPUs regardless of
\ ** the bus interface width (8 or 16 bits).

\ *S Hardware gotchas
\ *P This code assumes a CPU with byte addressing. A cell
\ ** addressed machine, e.g. most DSPs, will require considerable
\ ** changes to the source code for memory buffer transfers.

\ *P When operating with fast CPUs note the following:
\ *(
\ *B The cycle time of the chip is 185ns (min. back to back access),
\ ** but the read access time is only 40ns. This is detailed in the
\ ** data sheet.
\ *B At least 400ns is required between a data register access and
\ ** a pointer register access. At least 400ns is required after
\ ** writing the pointer register and accessing the data register.
\ ** See the data sheet.
\ *B The words *\fo{EC@} and *\fo{EC!} and the optional *\fo{EW@}
\ ** and *\fo{EW!} 16 bit words by default assume that the SMC chip
\ ** is memory mapped and that
\ ** that the SMC memory area is contiguous. If you have a 16 or 32
\ ** bit bus and one or more low address lines are ignored you must
\ ** adapt these words yourself to suit your hardware.
\ *)

\ ****************
\ *S Configuration
\ ****************
\ *P The following must be defined before the file *\i{DRIVERS\SMC91C9x.FTH}
\ ** is compiled.
((
interpreter
: const equ ;	\ n -- ; -- n
\ *G If you are using a standalone target with heads and you want
\ ** interactive access to all the registers and bit masks, define
\ ** *\fo{CONST} as *\fo{CONSTANT}, otherwise by default *\fo{CONST}
\ ** is defined as *\fo{EQU}.
target

$50000000 const EtherBase	\ -- addr
\ *G Define the base address of the Ethernet controller.

0 equ SMC16?	\ -- flag
\ *G If SMC16 is non-zero, the driver will use 16 bit register
\ ** accesses where possible, otherwise it will use two 8 bit
\ ** accesses. SMC16 should only be set non-zero for little
\ ** endian CPUs.

1 equ fastCPU?	\ -- n
\ *G Set this value false if no software intervention is
\ ** required to meet the 400ns timing requirement before and after
\ ** changing the pointer register.

0 equ smcDiags?	\ -- flag
\ *G Set this equate true to compile diagnostic code for
\ ** register dumping and so on. False by default.

0 equ eeprom?	\ -- flag
\ *G Set this equate true if the LAN91C9x has an attached EEPROM
\ ** for configuration data storage. False by default.

0 equ GenericIP?	\ -- flag
\ *G Set this equate true if the Generic IP device structure
\ ** defined in *\i{ETHERCOM.FTH} is required. This is only
\ ** required for systems using multiple IP devices in future
\ ** releases of PowerNet.

1 equ sniff?	\ -- flag
\ *G Set this equate true to compile the packet sniffer code,
\ ** which can be used to test Ethernet reception.

create EtherAddress	\ -- addr
\ *G Holds the Ethernet MAC address (six bytes). Note that you
\ ** must obtain these from the IEEE (www.ieee.org) or from other
\ ** sources.
  $00 c, $10 c, $8B c, $F1 c, $44 c, $20 c,	\ six bytes for 48 bit MAC address
					\ you must obtain these yourself!

create IpAddress	\ -- addr
\ *G Holds the Ethernet IP address (four bytes).
  192 c,  168 c,  1 c,  251 c,		\ assign these as required
))


\ ************
\ *S Constants
\ ************

( *D BankSelect         Offset of bank select register                  )
( **                                                                    )
( *H Bank0 Registers                                                    )
( *D TCRL               Transmit Control Register - lo byte             )
( *D TCRH               Transmit Control Register - hi byte             )
( *D TXSTATUS           Transmit Status Register                        )
( *D RCR                Receive Control Register                        )
\ *D MIR                Memory Information Register
( *D MCR                Memory Control Register                         )
( **                                                                    )
( *H Bank1 Registers                                                    )
( *D Configl            Configuration Register - lo byte                )
( *D Configh            Configuration Register - hi byte                )
( *D IA0                Hardware Address, Ethernet MSB                  )
( *D IA1                Hardware addr                                   )
( *D IA2                Hardware addr                                   )
( *D IA3                Hardware addr                                   )
( *D IA4                Hardware addr                                   )
( *D IA5                Hardware addr                                   )
( *D Ctrh               Control Register - hi byte                      )
( **                                                                    )
( *H Bank2 Registers                                                    )
( *D MMU                Memory management unit cmd reg                  )
( *D PNR                Packet number register                          )
( *D ARR                Allocation result register.                     )
( *D Pointerl           Memory pointer register                         )
( *D Pointerh           Memory pointer register                         )
( *D Datal              Reg to send packets too...                      )
( *D Datah              Reg to send packets too...                      )
( *D IntStatus                                                          )
( *D IntMask                                                            )
( **                                                                    )
( *H Misc. Constants                                                    )
( *D RelRx              MMU Command to release memory from an rx        )
( *D RxRd               SMC Command to read received packet             )
( *D TxWr               SMC Command to write to tx area of ram          )
( *D AllocIntMask       Memory allocated mask                           )
( *D MaxMsgSize         Max 802.3 Ether frame size in bytes             )
( *D AllocTx                                                            )
( *D bit0               Bitmask                                         )
( *D bit1               Bitmask                                         )
( *D bit2               Bitmask                                         )
( *D bit3               Bitmask                                         )
( *D bit4               Bitmask                                         )
( *D bit5               Bitmask                                         )
( *D bit6               Bitmask                                         )
( *D bit7               Bitmask                                         )
( *D EvenTx             Control byte for tx of even bytes               )
( *D OddTx              Control byte for tx of odd bytes                )
( *D RxTask#            Multi Task ID                                   )
( **                                                                    )

\ =======================================================================

\ Common to all banks
$0E const BankSelect
$00 const Bank0
$01 const Bank1
$02 const Bank2
$03 const Bank3

\ Bank 0 registers
$00 const TCRL
$01 const TCRH
$02 const TxStatus
$05 const RCR
$08 const MIR
  $08 const MIRl
  $09 const MIRh
$0A const MCR

\ Bank 1 registers
$00 const Configl
$01 const Configh
$04 const IA0
$05 const IA1
$06 const IA2
$07 const IA3
$08 const IA4
$09 const IA5
$0A const gprl
$0B const gprh
$0C const Ctrl
$0D const Ctrh

\ Bank 2 registers
$00 const MMU
$02 const PNR
$03 const ARR
$06 const Pointerl
$07 const Pointerh
$08 const Datal
$09 const Datah
$0C const IntStatus
$0D const IntMask

\ Various bits and bobs...
$E0 const RxRd
$40 const TxWr
$08 const AllocIntMask
#1530 const MaxMsgSize

\ MMU commands
$80 const RelRx
$20 const AllocTx

[undefined] bit0 [if]
$01 const bit0
$02 const bit1
$04 const bit2
$08 const bit3
$10 const bit4
$20 const bit5
$40 const bit6
$80 const bit7
[then]

$00 const EvenTx
$20 const OddTx
$20 const OddRx			\ SFP003
$01 const RxTask#


\ =======================================================================
( **                                                                    )
( *S Hardware Interface Layer                                           )
( **                                                                    )
\ =======================================================================
\ *P The default code is for a memory-mapped device at base
\ ** address *\fo{EtherBase}.

: ec!           \ b offset --
\ *G Set an 8 bit register contents to *\i{b}, in the selected bank.
  EtherBase + c!
;

: ec@           \ offset -- val
\ *G Read an 8 bit register in the selected bank.
  EtherBase + c@
;

((
SMC16? [if]
: ew!           \ w offset --
\ *G For LITTLE-ENDIAN CPUs only.
\ ** Set a 16 bit register contents to *\i{w}, in the selected bank.
  EtherBase + w!
;

: ew@           \ offset -- val
\ *G For LITTLE-ENDIAN CPUs only.
\ ** Read a 16 bit register in the selected bank.
  EtherBase + w@
;
[then]
))

optimised? [if]
compiler
: ec!           \ val offset --
\ *G VFX optimising compilers will probably produce shorter
\ ** and faster code by using a compiler macro for *\fo{EC!}.
  EtherBase + c!
;

: ec@           \ offset -- val
\ *G VFX optimising compilers will probably produce shorter
\ ** and faster code by using a compiler macro for *\fo{EC@}.
  EtherBase + c@
;

((
SMC16? [if]
: ew!           \ val offset --
\ *G VFX optimising compilers will probably produce shorter
\ ** and faster code by using a compiler macro for *\fo{EW!}.
  EtherBase + w!
;

: ew@		\ offset -- val
\ *G VFX optimising compilers will probably produce shorter
\ ** and faster code by using a compiler macro for *\fo{EW@}.
  EtherBase + w@
;
[then]
))

target
[then]

fastCPU? [if]
: 400ns		\ --
\ *G For fast CPUs, use this hardware dependent word to ensure
\ ** at least 400ns between back to back Ethernet chip accesses.
\ ** This is required for correct operation of the pointer register.
\ ** The default version here assumes 4 instructions per iteration
\ ** at 5ns per instruction, and the call/return and set up overheads
\ ** are ignored. See *\fo{FASTCPU?} above.
  #20 0 do loop
;
[else]
compiler
: 400ns		\ --
\ *G Use this version if your CPU takes at least 400 ns between back
\ ** to back Ethernet chip accesses. See *\fo{FASTCPU?} above.
;
target
[then]

: SetBank	\ bank -- ; bank = 0 to 3
\ *G Select the active register bank.
  $03 and  BankSelect EtherBase + c!
;


\ ==============
\ *S Diagnostics
\ ==============

smcDiags? [if]

: .bank         \ --
( *G Display register contents for currently selected SMC bank.         )
  $10 0
  do  i ec@ .byte space  loop
  cr
;

: .reg          \ --
( *G Display contents of all SMC Ethernet Controller's registers        )
  base @  hex
  cr ." Offset  " $10 0 do i .byte space loop cr cr
  #4 0 do
    i setbank  ." Bank " i . space  .bank
  loop
  base !
;

[then]


\ ===============
\ *S Driver Layer
\ ===============

: eISR@		\ -- bmask
\ *G Read the bank 2 interrupt status register (IST).
  bank2 setbank   IntStatus ec@
;

: eMMU!		\ cmd --
\ *G Send a command to the bank 2 MMU register.
  bank2 setbank  MMU ec!		\ send command to chip
;

: eTCRl@	\ -- bmask
\ *G Read the bank 0 TCRl register.
  Bank0 SetBank  Tcrl ec@
;

: EnEtherTx	\ --
\ *G Enable Ethernet transmission. Selects bank 0.
  Bank0 SetBank
  Tcrl ec@ 1 or Tcrl ec!
;

: eSoftRst	\ --
\ *G Initiate software reset - needed to clear Tx lock up.
   bank0 setbank  $80 RCR ec!
   2 ms  0 RCR ec!
;

: init-SMC	\ --
\ *G Initialise SMC chip.
  eSoftRst				\ MGS008
  $40 eMMU!				\ Reset MMU
  bank1 SetBank
  $00 Configl ec!  $00 Configh ec!	\ Active HI ints, 10BaseT
  1 ms					\ SMSC drivers do this
  Bank0 SetBank  $06 MCR ec!		\ reserves 6*256=1536 bytes for TX
;

\ SFP005...
: init-EtherTx  \ --
\ *G Enable Ethernet TX module.
  EnEtherTx
  Bank1 SetBank  $08 Ctrh ec!    	\ set auto release of transmit memory
;
\ ...SFP005

: init-EtherRx  \ --
\ *G Enable Ethernet RX module.
  Bank0 SetBank
  $03 Rcr ec!                    \ enable receiver and strip rx crc
  Bank2 SetBank
  bit0 IntMask ec!               \ enable rx ints on ether chip
;

: etheradd>CS   \ addr --
\ *G Store 6 byte Ethernet address into SMC chip from memory buffer.
((
  cr ." SMC Ether Device, Hardware Address "
  Bank1 SetBank
  dup c@ dup .byte IA0 ec! 1+ [char] - emit
  dup c@ dup .byte IA1 ec! 1+ [char] - emit
  dup c@ dup .byte IA2 ec! 1+ [char] - emit
  dup c@ dup .byte IA3 ec! 1+ [char] - emit
  dup c@ dup .byte IA4 ec! 1+ [char] - emit
      c@ dup .byte IA5 ec!
))
  Bank1 SetBank  IA0 6 bounds
  do  dup c@ i ec!  1+  loop
  drop
;

: InitEther	\ --
\ *G Perform a full initialisation of the chip, enabling
\ ** Rx and Tx, and setting up the Ethernet MAC address
\ ** from *\fo{ETHERADDRESS}.
  init-SMC  EtherAddress etheradd>CS
  init-EtherRx init-EtherTx
;

: EtherLink?	\ -- flag
\ *G Return true if the Ethernet link is established.
  Bank0 SetBank  TxStatus 1+ ec@ bit6 and 0<>
;

: AllocTxMem	\ n -- mask
\ *G Allocate *\i{n} bytes of memory to hold a packet for transmission.
\ ** Return the contents of the ARR register, or -1 for a fatal error.
  $10 +                         	\ we will add to the msg length later
  #8 rshift				\ allocate in 256 byte chuncks
  AllocTx or eMMU!			\ send command
  begin
    eISR@ AllocIntMask and 0=		\ loop until bit set, in bank 2
   while
[ tasking? ] [if]  pause  [then]
    eTCRl@ 1 and 0=			\ fatal TX error, TXENA=0
    if  InitEther  -1 exit  endif	\ chip reset, must return bit 7 set
  repeat
  ARR ec@
;

: GetTxMem	\ n -- ior
\ *G Allocate *\i{n} bytes of memory to hold a packet for transmission.
  5 0 do				\ -- n
    dup AllocTxMem dup $80 and 0= if	\ -- n arr
      PNR ec!  drop 0  unloop exit
    endif
    drop  #10 ms
  loop
  drop -1
;

: WritePacket   \ addr len --
\ *G Write an Ethernet frame to chip for transmission.
  >r                            \ store len for later
  $00 Datah ec!                 \ write status word
  $00 Datah ec!
  r@ 6 +                        \ len+6,=status,length and control word
  $FFFE and                     \ ensure an even number
  dup 8 rshift  swap $FF and	\ split len to two 8 bit units - faster on ARM
  Datah ec! Datah ec!           \ store low then high count byte
  r@ bounds                     \ addr len bounds for loop
  do  i c@ Datah ec!  loop	\ get buffer entry, store to chip
  r> 1 and                      \ was length odd ?
  if                            \ y:
    OddTx Datah ec!             \ packet control byte
  else                          \ n:
    $00 Datah ec!               \ dummy write
    EvenTx Datah ec!            \ packet control byte
  endif
;

: IsRx?         \ -- t|f
\ *G Check if incoming data is present, returning true if a
\ ** packet is available for *\fo{get_ether_pkt} below.
  eISR@ 1 and 0<>
;

: DiscardRX     \ --
\ *G Throw away pending input packet due to lack of memory.
  RelRx eMMU!			\ remove and release memory from rx
  begin  MMU ec@ 1 and while  pause  repeat	\ check busy bit
;

: get_ether_pkt \ *dest maxlen -- len
\ *G Get pending receive packet to a buffer.
\ ** Note that data is valid only when *\fo{ISRX?} returns true,
\ ** so that you must poll with *\fo{ISRX?} before using
\ ** *\fo{GET_ETHER_PKT}.
  Bank2 SetBank
  $00 Pointerl ec!              \ reset pointer value
  RxRd Pointerh ec!             \ read rx ram area, auto inc low pntr
  400ns				\ See SMC91C9x data sheet
  Datah ec@ drop                \ discard low status
  Datah ec@ drop                \ discard high status
  Datah ec@ Datah ec@ 8 lshift or \ 2 * 8 bit -> 16 bit number ; includes control info
  $07FF and                     \ byte count only uses 11 bits - always even
  $06 -                         \ already read 4 bytes, ignore last control bytes
  min				\ -- addr len ; prevent buffer overrun
\ SFP003...
  2dup bounds ?do		\ loop for even bytes of message
    datah ec@  i c!		\ copy byte from Ether chip ram to buffer
  loop
  tuck				\ -- len addr len
  datah ec@			\ potential last odd byte
  datah ec@ OddRx and if	\ control byte, check if odd count
    -rot + c!			\ stash last byte
    1+				\ increment count
  else
    drop 2drop
  endif
\ ...SFP003
  DiscardRX			\ remove and release memory from rx
;

: IsTx?		\ n -- flag
\ *G Return true if a packet of length n can be sent.
  #255 +  8 rshift			\ required 256 byte packets
  Bank0 SetBank  MIRh ec@ <=
;

: send_ether_pkt        \ addr len --
\ *G Send packet from supplied buffer.
\ ** The word will block until sufficient packet memory is available.
\ ** Note that you should read incoming packets regularly, otherwise
\ ** it is possible to get into a situation in which *\fo{SEND_ETHER_PKT}
\ ** blocks for ever because there is not enough space in the
\ ** device for the transmission packet. Theoretically, this should
\ ** not happen because 1536 bytes are reserved for transmission,
\ ** but ...
  $40 max			\ ensure packet is at least 64 bytes
  EnEtherTx 			\ SFP005
  dup GetTxMem			\ get memory for transmission
  if  2drop initEther exit  endif
  Bank2 SetBank
  $00 Pointerl ec!		\ reset offset value
  TxWr Pointerh ec!		\ set to write Tx ram area
  400ns				\ See SMC91C9x data sheet
  WritePacket			\ send packet to ether chip
  $C0 eMMU!			\ enqueue packet command
;


\ ******************
\ *S Attached EEPROM
\ ******************

eeprom? [if]

: ee!           \ word offset --
\ *G Write a 16 bit word into the EEPROM at EEPROM address
\ ** *\i{offset}. The EEPROM may not be present in all SMC91C9x
\ ** implementations.
  Bank2 SetBank  pointerl ec!
  Bank1 SetBank  dup 255 and gprl ec!
  8 rshift gprh ec!
  ctrl ec@  $F8 and $5 or  ctrl ec!
  begin  ctrl ec@ 1 and  0= until
  ctrl ec@  $F8 and  ctrl ec!
;

: ee@           \ offset -- word
\ *G Read a 16 bit word from the EEPROM at EEPROM address
\ ** *\i{offset}. The EEPROM may not be present in all SMC91C9x
\ ** implementations.
  Bank2 SetBank pointerl ec!
  Bank1 SetBank
  ctrl ec@  $F8 and $6 or  ctrl ec!
  begin  ctrl ec@ 2 and  0= until
  ctrl ec@  $F8 and  ctrl ec!
  gprh ec@ 8 lshift  gprl ec@ or
;

[then]


\ ****************************************
\ *S Generic I/O for PowerNet v3 and above
\ ****************************************
\ *P The code in this section is only compiled if the equate
\ ** *\fo{GenericIP?) has been defined and is non-zero.

\ *P The layout and usage of this structure is defined in the
\ ** file *\i{COMMON\ETHERCOM.FTH}.
\ *E create IPdevice
\ **   ' MyInit ,       \ 0: initialisation
\ **   ' MyTerm ,	\ 1: shutdown
\ **   ' MyRx? ,	\ 2: receive test
\ **   ' MyRx		\ 3: receive packet
\ **   ' MyTx? ,	\ 4: transmit test
\ **   ' MyTx ,		\ 5: transmit packet
\ **   ' MyGetAddr ,    \ 6: Get device addresses
\ **   ' MySetAddr ,    \ 7: Set device addresses
\ **   ' MySave ,	\ 8: Save IP device state

[undefined] GenericIP? [if]
0 equ GenericIP?
[then]

GenericIP? [if]

: GetAddrs	\ -- ipaddr macaddr 0
\ *G For Generic I/O *\fo{IPDGetAddr} function
  IpAddress EtherAddress 0
;

: SetAddrs	\ ipaddr|0 macaddr|0 mode --
\ *G Set up the Ethernet MAC and the IP addresses.
\ ** At present mode is always 0, but will be used in
\ ** future releases to indicate data formats. If
\ ** ipaddr or macaddr are zero, the stored data
\ ** will not be changed.
  drop					\ flags
  ?dup if				\ macaddr
    dup EtherAddress 6 cmove
    etheradd>CS
  endif
  ?dup if				\ ipaddr
    IpAddress 4 cmove
  endif
;

create SMCvector	\ -- addr
\ *G The device vector needed by PowerNet v3+ and
\ ** *\i{COMMON\ETHERCOM.FTH}.
  ' InitEther ,         \ initialisation
  ' noop ,	        \ shutdown
  ' IsRx? ,	        \ receive test
  ' get_ether_pkt ,     \ receive packet
  ' IsTx? ,	        \ transmit test
  ' send_ether_pkt ,    \ transmit packet
  ' GetAddrs ,		\ Get device addresses
  ' SetAddrs ,	        \ Set device addresses
  ' noop ,	        \ Save IP device state

SMCvector constant IPDevice	\ -- addr
\ *G *\fo{IPDevice} is the default device name required by
\ ** *\i{ETHERCOM.FTH}. If you have multiple ports, only one
\ ** should be named *\fo{IPDevice}.

[then]


\ **************
\ *S System test
\ **************
\ *P This code is only compiled if the equate *\fo{SNIFF?}
\ ** is non-zero.

sniff? [if]

1536 buffer: pbuff	\ -- addr
\ *G Buffer for SNIFF.

: sniff		\ --
\ *G Listens to the network and displays all the traffic that
\ ** has a broadcast destination or is for this device.
\ ** *\fo{SNIFF} can be used to test reception.
  InitEther
  begin
    isrx? if
      pbuff 1536 erase
      pbuff 1536 get_ether_pkt
      pbuff swap 1 max dump
    then
[ tasking? ] [if] pause [then]
    key?
  until
;

[then]


\ ******
\ *> ###
\ ******


