\ MBusTst.fth - ModBus TCP Client Test code

((
Copyright (c) 2008
Flygt AB
and
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
))

decimal


\ =========
\ *! modbusclient
\ *T ModBus client
\ =========
\ ***********************
\ *S ModBus specific data
\ ***********************
\ *P These data definitions are required by each server task. The data
\ ** is allocated at the start of the task and released when the task is
\ ** *\fo{TERMINATE}d. The chain *\fo{SVCHAIN} links all the service tasks.

#502 equ ModBusPort#	\ -- n ; standard is 502
\ *G Define the port used for the Modbus server. The standard
\ ** port is 502.

#300 buffer: MBusData	\ -- addr
\ *GBuffer for reply data

struct /MBAP	\ -- len
\ *G ModBus MBAP header.
  2 field ma.tid		\ transaction identifier
  2 field ma.prot		\ protocol, 0=mbus
  2 field ma.len		\ following data length in bytes
  1 field ma.uin		\ unit identifier
end-struct

\ *********************
\ *S ModBus Client code
\ *********************

: +lrcMB	\ crc b -- crc'
\ *G Update ModBus LRC16 for one byte
  $FF and xor
  8 0 do
    dup 1 and if
      1 rshift $A001 xor		\ XOR in polynomial to crc
    Else
      1 rshift				\ just shift crc
    Endif
  Loop
;

: MBcrc		\ caddr len -- crc16
  $FFFF -rot  bounds
  ?do  i c@ +lrcMB  loop
  $FFFF and
;

((
create tframe
  $02 c,  $07 c,

tframe 2 MBcrc -> $1241

16 bit CRC is sent low byte first, $41, $12
))

create MBserver	\ -- addr
\ *G Holds the IP address of the Modbus server.
  #192 c,  #168 c,  0 c,  #214 c,

[undefined] TCPConnectTo [if]
: TCPConnectTo	\ ipaddr port# -- hsocket|0
\ *G Attempt to create a TCP socket and connect to a server.
\ ** *\i{IPADDR} is an IP address and *\i{PORT#} is the requested port.
\ ** ModBus servers are almost always found on port 502 (decimal).
  { | sin[ SOCKADDR_IN ] -- }

  AF_INET SOCK_STREAM IPPROTO_TCP socket \ Create a socket
  dup INVALID_SOCKET = if
\ cr ." WC: can't create socket"
    2drop drop 0  exit
  then
\ cr ." socket created"

  -rot					\ -- hs ipaddr port#
  sin[ dup SOCKADDR_IN erase		\ -- hs ipaddr port# sin[
  AF_INET over sin_family w!(n)		\ fill in address type
  tuck sin_port w!(n)			\ port number
  sin_addr !(n)				\ and IP address
  \ -- hs

  dup sin[ SOCKADDR_IN connect if      	\ Attempt to connect
\ cr ." WC: can't connect"
    CloseSocket drop  0 exit		\ failed
  then
\ cr ." connection established"
;
[then]

: OpenMBConn	\ -- socket|0
\ *G Open a connection to the poll server, returning the socket
\ ** number or zero on error. The return value is also placed in
\ ** *\fo{MY_HSOCKET}. A maximum of five attempts are made to
\ ** connect.
  5 0 do
    MBserver @(n) ModBusPort# TCPConnectTo
    dup my_hsocket !  dup
    if  unloop exit  endif
    drop  #2000 ms
\ cr ." CONNECT failed in OpenMBConn "
  loop
  0
;

: CloseConn	\ --
\ *G Close the poll connection. The socket handle is in *\fo{MY_HSOCKET}.
  my_Hsocket @ Disconnect drop
;

: GotResponse?	\ -- n
\ *G Return non-zero if the socket has data waiting to be read.
  my_HSocket @ pollSocket 0 max
;

: WaitResponse	\ ms -- flag ; 0=success
\ *G Wait for a response (socket has input data) for the given number
\ ** of milliseconds. Return true if the socket times out.
  later
  begin
    GotResponse? 0=
   while
    dup expired
  until then
  timedout?
;

create TestAPDU
  $00 w,(n)  $00 w,(n)			\ transaction id and protocol
  $06 w,(n)  $05 c,			\ data bytes in PDU, unit id.
\ ------------------------------
  $08 c,  $05 c,  $00 c,  $AC c,	\ PDU
  $FF c,  $00 c,  $4C c,  $82 c,	\ last two bytes are CR lo, hi

: SendTest	\ --
\ *G Send the test packet above
  my_hsocket @ TestAPDU #15 0 send .
;

: GetReply	\ --
  my_hsocket @ MBusData #15 0 recv .
;

((
: ReadResponse	\ --
\ *G Read the response into *\fo{HTTPbuff}.
  my_HSocket @ HTTPbuff /HTTPbuff 0 recv
  0 max HTTPcnt !
\ HTTPbuff HTTPcnt @ dump
;
))



\ **************
\ *S Diagnostics
\ **************
\ *P This code is only compiled if the *\fo{EQU}ate *\fo{DIAGS?}
\ ** is set non-zero.

diags? [if]


[then]


\ ******
\ *> ###
\ ******

