\ SMTP.FTH - SMTP Primitives

((
Copyright (c) 2001, 2003
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20030510 SFP001 Converted from VFX Forth demo.
))


\ =======
\ *! smtp
\ *T SMTP Primitives
\ ======
\ *P This code can send any arbitary text via standard
\ ** eMail systems using the Simple Mail Transfer protocol
\ ** (SMTP).

\ *P Recommended reading includes:-
\ *D RFC_821    Simple Mail Transfer Protocol
\ *D RFC_822    ARPA Internet Text Messages


1 value smtpdiags?	\ -- n
\ *G Set this non-zero to get diagnostic information.

#200 equ SMTPwait	\ -- ms
\ *G Number of milliseconds to wait for an SMTP response from
\ ** the server. If you are using a slow connection you may
\ ** have to increase this value.

#256 equ MAXRXSIZE	\ -- n
\ *G Size of the response buffer for messages from the SMTP
\ ** server.

MAXRXSIZE buffer: SMTPin	\ -- addr
\ *G The SMTP receive buffer.

: SMTP::Wait	\ hsock --
\ *G Wait for a response from the server for up to SMTPWAIT
\ ** milliseconds.
  { | numchars -- }
  SMTPwait later swap
  begin					\ -- timer hsock
    dup FIONREAD addr numchars ioctlsocket
    SOCKET_ERROR <>			\ good response?
   while
    numchars SOCKET_ERROR <>		\ no socket error?
   while
    numchars 0=
   while
    over expired
  until then then then
  2drop
;

: SMTP::Error?	\ hsock -- flag
\ *G Read a response back from the SMTP server via the
\ ** supplied socket and return a TRUE flag if the server's response
\ ** indicates an error.
  dup SMTP::Wait  
  SMTPin MAXRXSIZE 2dup erase		\ clear buffer
  0 recv dup SOCKET_ERROR =		\ read from socket, error?
  if  drop TRUE exit  then
  					\ -- len
  smtpdiags?
  if  ." RCV :" SMTPin over type cr  then
  drop
  SMTPin c@ [char] 5 =			\ SMTP errors start '5xx'
;

: SMTP::Connect \ ipaddr port# -- hsocket|0
\ *G Attempt to create a socket and connect to an SMTP Server.
\ ** IPADDR is an ipaddress and PORT# is the requested port.
\ ** SMTP Servers are almost always found on port 25 (decimal).
  { | sin[ SOCKADDR_IN ] -- }

  AF_INET SOCK_STREAM IPPROTO_TCP socket \ Create a socket
  dup INVALID_SOCKET =
  if  2drop drop 0  exit  then

  -rot					\ -- hs ipaddr port#
  sin[ dup SOCKADDR_IN erase		\ -- hs ipaddr port# sin[
  AF_INET over sin_family w!(n)		\ fill in address type
  tuck sin_port w!(n)			\ port number
  sin_addr !(n)				\ and IP address
  					\ -- hs
  dup sin[ SOCKADDR_IN connect if      	\ Attempt to connect
    CloseSocket drop  0 exit		\ failed
  then

  dup SMTP::Error? if			\ Handle response. On
    CloseSocket drop  0 exit		\ connection SMTP servers
  then					\ should say "hello"
;

: SMTP::Disconnect	\ hsock --
\ *G Disconnect from a SMTP discussion by closing our socket.
  Disconnect drop
;

: (SMTPWrite)	\ hsock c-addr u --
\ *G Write a buffer out via a socket. 
  #0 send drop
;

: SMTP::Write   \ hsock c-addr u --
\ *G Write a buffer out via a socket. Used to send text strings to the
\ ** SMTP Server.
  smtpdiags?
  if  2dup type  then
  (SMTPWrite)
;

create crlf$	\ -- addr
\ *G A counted string holding a CR/LF pair
  2 c,  #13 c,  #10 c,

: (SMTP::WriteLn)	\ hsock c-addr len mode --
\ *G As 'SMTP::Write' but followed by a CR/LF pair. Mode is
\ ** a TCP flag which is normally 0, but is set to TCP_PSH
\ ** by SMTP::WRITEFIELD below.
  >r					\ preserve mode
  smtpdiags?
  if  2dup type cr  then
  2 pick >r				\ preserve hsock
  dup					\ len may be 0
  if  (SMTPWrite)  else  2drop drop  endif
  r> crlf$ count r> send drop
;

: SMTP::WriteLn	\ hsock c-addr u --
\ *G As 'SMTP::Write' but followed by a CR/LF pair.
  0 (SMTP::WriteLn)
;

: SMTP::WriteField      \ hsock field-addr field-len val-addr val-len -- flag
\ *G Write a 'field' to the SMTP server. In this case a 'field' consists
\ ** of a 'field name' and a 'text value' followed by an EOL. Any field 
\ ** written prompts the server to make a response which is checked for
\ ** an error condition to form the return flag.
  2>r  2 pick >r			\ hsock field-addr field-len --
  					\ R: -- val-addr val-len hsock
  SMTP::Write				\ write field text
  r@ S"  " SMTP::Write			\ write space
  r> dup 2r> TCP_PSH (SMTP::Writeln)	\ write value with CR/LF.
  SMTP::Error?				\ and check for error
;


\ ======
\ *> ###
\ ======

decimal

