\ Reprogram.fth - client code for PillarComm server

((
Copyright (c) 2014, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Copyright (c) 2014, 2015
Pillarhouse Ltd.

To do
=====

Change history
==============
20150217 SFP002 Revamp for serial emulation.
20140131 SFP001 First cut.
))

decimal

\ ===============
\ *! reprogram
\ ===============
\ *P The code in *\i{<pnet>\Examples\TargetServer\Reprogram} implements
\ ** reprogramming code for a system driven by a PC issuing commands
\ ** across a serial line simulated by an Ethernet connection.

\ *P Command parsing and status responses are handled by your
\ ** application code. The user variables *\fo{my_hsocket} and
\ ** friends are assumed to work as expected.


1 value TCdiags?	\ -- n
\ *G Set this non-zero to get diagnostic information about the
\ ** server.

compiler
: [tc		\ --
\ *G A compiler macro used to surround diagnostic code, and terminated
\ ** by *\fo{TC]}.
\ *E   [TC  ." debug message"  TC]
  postpone TCdiags?  postpone if
;
: tc]
\ *G Terminates a *\fo{[TC ... TC]} structure.
  postpone endif
;
target


64 buffer: Cmd$	\ -- addr
\ *G Buffer in which to build commands. Counted string
128 buffer: Response$
\ *G Buffer for responses. Counted string.

: extractDec	\ caddr len -- caddr' len' u
\ *G Extract a number in decimal from the start of the
\ ** string, returning the remaining string starting at the first
\ ** non-numeric character and the converted number.
  base @ >r  decimal
  0 0 2swap >number 2swap drop
  r> base !
;

: -leading	\ caddr len -- caddr' len'
\ *G Remove leading blanks from the string.
  begin
    dup
   while
    over c@ bl =
   while
    1 /string
  repeat then
;


\ **********************
\ *S Reading a new image
\ **********************
\ *P A new program is read into a buffer called *\fo{ImageBuffer}.
\ ** the first four cells are reserved and used as follows:
\ *D len1   length sent by host PC
\ *D CRC1   CRC32 calculated by host
\ *D len2   length read by target
\ *D CRC2   CRC32 calculated by target

struct /ibh	\ -- len
\ *G Image buffer header structure
  int ibh.len1
  int ibh.crc1
  int ibh.len2
  int ibh.crc2
  0 field ibh.prog
end-struct

: +CRC32	\ crc b -- crc'
\ *G Update CRC32 with one byte.
  8 0 do
    2dup xor  1 and if
      swap 1 rshift $EDB8:8320 xor swap	\ XOR in polynomial to crc
    Else
      swap 1 rshift swap		\ just shift crc
    Endif
    1 rshift				\ shift b
  Loop
  drop					\ discard b
;

: genCRC32	\ crc caddr len -- crc'
\ *G Generate a 32 bit CRC for a block of memory, given an initial CRC.
\ ** value.
  bounds				\ -- crc addr+len addr
  ?do  i c@ +crc32  loop
;

: genCRC32-0	\ caddr len -- crc
\ *G Generate a CRC32 with an initial value of 0.
  0 -rot genCRC32
;

: readProgData	\ caddr len -- ior ; 0=success
\ *G Read the new program data into the image buffer. Return 0 on
\ ** success.
  begin
    dup
   while
    2dup
    my_hsocket @ -rot #1024 min 0 recv	\ -- caddr len len'
    dup SOCKET_ERROR =
    if  drop 2drop  -1 exit  then
    dup ImageBuffer ibh.len2 +!		\ update length read
    /string
  repeat
  nip
;

: do-NPel		\ --
\ *G Handle length of new program.
  expected-length @ ImageBuffer ibh.len1 !
;

: do-NPec		\ --
\ *G Handle expected CRC of new program.
  expected-crc @ ImageBuffer ibh.crc1 !
;

: do-NPG		\ --
\ *G Read bytes from *\fo{my_hsocket} into *\fo{ImageBuffer}.
  ImageBuffer dup ibh.prog  ibh.len1 @	\ start of prog data and length
  readProgData drop			\ --
  ImageBuffer ibh.prog			\ start address
  ImageBuffer ibh.len2 @		\ length read
  genCRC32-0 ImageBuffer ibh.crc2 !	\ generate CRC
;

: do-NPst	\ --
\ *G Return a status message.
  ImageBuffer ibh.len2 @
  ImageBuffer ibh.crc1 @
  ImageBuffer ibh.crc2 @
  check-image-message
;

: do-NPF	\ --
  ImageBuffer ibh.prog			\ start address of image
  FlashBase				\ start address of Flash
  ImageBuffer ibh.len2 @		\ length of image
  CopyFlash				\ ends with a reboot
;



\ ******
\ *> ###
\ ******

