\ PNMISC.FTH - String and Time support for WWW protocols

((
Copyright (c) 2001-2003, 2005, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

This code has been licensed from:
  CEM Systems Ltd, Belfast, 2000-2002


To do
=====

Change history
==============
20081216 SFP003 Added -LEADING, IS=, ISEARCH.
20060610 SFP002 Added CSPLIT and RIGHT-STRING.
20060228 SFP001 Added IS= case insensitive version of S=.
		Recoded STRING>N.
))

decimal

\ =============
\ *! websupport
\ =============
\ **************
\ *T WWW Support
\ **************
\ *P The source code for these string and time functions
\ ** is in *\i{PNmisc.fth}.


\ **********
\ *S Strings
\ **********

[undefined] -leading [if]
: -leading      \ c-addr u -- c-addr u
\ *G Modify a string address/length pair to ignore leading spaces.
  bl skip  ;
[then]

[undefined] addchar [if]
: addchar       \ char string --
\ *G Add the character to the end of the counted string
  tuck  count + c!  1 swap c+!
;
[then]

[undefined] append [if]
: append	\ c-addr u $dest --
\ *G Add the string described by C-ADDR U to the counted string at
\ ** $DEST.
  >r
  tuck  r@ count +  swap cmove          \ add source to end
  r> c+!                                \ add length to count
;
[then]

[undefined] is= [if]
: is=		\ c-addr1 c-addr2 u -- flag
\ *G Compare two same-length strings/memory blocks, returning TRUE if
\ ** they are identical. The comparison is case insensitive.
  bounds ?do
    dup c@ upc  i c@ upc <>
    if  drop 0  unloop exit  endif
    1+
  loop
  drop -1
;
[then]

[undefined] str= [if]
: str=		\ addr1 len1 addr2 len2 -- flag
\ *G Compare two addr/len memory blocks, returning TRUE if
\ ** they are identical both in length and contents.
\ ** The comparison is case sensitive.
  rot over = if				\ -- addr1 addr2 len2
    s=  exit
  endif
  3drop 0
;
[then]

[undefined] isearch [if]
: (isearch)	{ c-addr1 u1 c-addr2 u2 -- c-addr3 u3 flag }
  c-addr1 u1 u2 - 1+ bounds     \ Search for first char of $2 in $1.  Note that
  ?do                           \ we only check LEN($1)-LEN($2)+1 chars of $1.
    i c@ upc  c-addr2 c@ upc  = if	\ Compare next char of $1 with first char of $2.
      i c-addr2 u2 is= if       \ Is the whole of $2 present?
        i u1 i c-addr1 - - true \ Yes - build return parameters...
        unloop exit             \ ...and bail out.
      then
    then
  loop
  c-addr1 u1 false            \ End of loop and first char not found - no match.
;

: ISEARCH        \ c-addr1 u1 c-addr2 u2 -- c-addr3 u3 f
\ *G Search the string *\i{c-addr1/u1} for the string (\i{c-addr2/u2}.
\ ** If a match is found return *\i{c-addr3/u3}, the address of
\ ** the start of the match and the number of characters remaining
\ ** in *\i{c-addr1/u1}, plus flag *\i{f} set to true. If no match
\ ** was found return *\i{c-addr1/u1} and *\i{f=0}. Case insensitive
\ ** for English.
  2 pick over < if      \ Is $1 shorter than $2?
    2drop false exit    \ Yes - $2 *can't* be in $1.
  then
  dup 0 <= if           \ Is $2 zero length?
    2drop true exit     \ Yes - we have found it at the start of $1.
  then
  2 pick 0 <= if        \ Is $1 zero length?
    2drop false exit    \ Yes - string not found.
  then
  (isearch)
;
[then]

: left-string \ caddr len char -- caddr len'
\ *G Given a string and a delimiter, return the string to
\ ** the left of the delimiter.
  -rot  2dup swap >r >r  rot		\ -- addr len char ; R: -- addr len
  scan nip				\ find delimiter
  r> swap -  r> swap
;

((
: right-string \ caddr len char -- caddr len'
\ *G Given a string and a delimiter, return the string to
\ ** the right of the delimiter.
  scan dup
  if  1 /string  endif
;
))

: extract-string	\ caddr len char -- caddr' len'
\ *G Given a string, return the substring between the delimiters,
\ ** ignoring leading delimiters.
  dup >r skip
  2dup r> scan
  nip -
;

: jump-string	\ caddr len char -- caddr' len'
\ *G Ignore leading delimiters, and return the string to the
\ ** right of the next delimiter character including the delimiter.
  dup >r skip  r> scan
;

[undefined] split [if]
: split		\ addr len char -- laddr llen raddr rlen
\ *G Extract a substring at the start of addr/len, returning
\ ** *\i{raddr/rlen} the string remaining after char and the substring
\ ** *\i{laddr/llen} which includes all characters before but not
\ ** including *\i{char}. If the string does not contain the
\ ** character, *\i{raddr} is *\i{addr+len} and *\i{rlen}=0.
  rot dup 2swap scan			\ -- addr addr' len'
  dup 0= if				\ can't split
    -rot tuck over -			\ -- 0 addr' addr len
    2swap swap				\ -- addr len addr' 0
  else
    over swap				\ -- addr addr' addr' len'
    1 /string 2swap			\ -- addr'' len'' addr addr'
    over -				\ -- addr'' len'' addr sslen
    2swap				\ -- addr sslen addr'' len''
  endif
;
[then]

: split$LR	\ addr len char -- laddr llen raddr rlen
\ *G Extract a substring at the start of *\i{addr/len}, returning
\ ** *\i{raddr/rlen} the string remaining after char and the substring
\ ** *\i{laddr/llen} which includes all characters before but not
\ ** including *\i{char}. If the string does not contain the
\ ** character, *\i{raddr} is *\i{addr+len} and *\i{rlen}=0.
  split  ;

: split$RL	\ addr len char -- raddr rlen laddr llen
\ *G As *\fo{split$LR} but the left string is returned topmost.
  split$LR 2swap  ;

((
: csplit	\ addr len char -- raddr rlen laddr llen
\ *G Extract a substring at the start of addr/len, returning
\ ** the string raddr/rlen which includes char (if found) and
\ ** the string laddr/llen which contains the text to left of char.
\ ** If the string does not contain the character, raddr is
\ ** addr+len and rlen=0.
  >r 2dup r> scan                       \ -- addr len addr' len'
  2swap 2 pick -                        \ -- addr' len' addr len-len'
;
))

\ SFP001...
: string>n	\ addr len radix -- value
\ *G Converts an ASCII string to a number in the given base.
\ ** Characters are converted to upper case before conversion.
  0 2swap				\ -- radix 0 addr len
  over c@ [char] - = dup >r		\ test for negative
  if  1 /string  then			\ step over '-' char
  bounds ?do				\ -- radix value
    over i c@ upc swap digit 0= ?leave	\ -- radix value n
    -rot over *	rot +			\ -- radix value'
  loop
  nip
  r> ?negate
;
\ ...SFP001

: string>dec	\ caddr len -- n
\ *G Converts an ASCII string as a decimal number.
  #10 string>n
;

: string>hex	\ caddr len -- n
\ *G Converts an ASCII string as a hexadecimal number.
  #16 string>n
;

: dec>string	\ n -- caddr len
\ *G *\i{N} is converted to a signed decimal string.
  base @ >r decimal
  s>d tuck dabs <# #s rot sign #>
  r> base !
;

: hex>string	\ n -- caddr len
\ *G *\i{N} is converted to an unsigned hexadecimal string
\ ** with a leading $ character
  base @ >r hex
  0 <# #s [char] $ hold #>
  r> base !
;

\ SFP001...
[undefined] is= [if]
: is=		\ c-addr1 c-addr2 u -- flag
\ *G Compare two same-length strings/memory blocks, returning TRUE if
\ ** they are identical. The comparison is case insensitive.
  bounds ?do
    dup c@ upc  i c@ upc <>
    if  drop 0  unloop exit  endif
    1+
  loop
  drop -1
;
[then]
\ ...SFP001


\ ****************
\ *S Time and date
\ ****************
\ *P These functions rely on the ANS Forth word
\ ** *\fo{TIME&DATE ( -- s m h dd mm yyyy )} and the non-standard
\ ** *\fo{DOW ( -- dow, 0=Sun)} to get the day of the week.

[undefined] .AnsiDate [if]

[undefined] dow [if]
2 constant dow	\ -- 2
[then]

[undefined] time&date [if]
: time&date	\ -- s m h dd mm yy
  0 55 11  22 4 2003
;
[then]

create days$	\ -- addr
\ *G String containing 3 character text for the days of the week.
  ", SunMonTueWedThuFriSat"

create months	\ -- addr
\ *G String containing 3 character text for the months.
  z", JanFebMarAprMayJunJulAugSepOctNovDec"

: .dow		\ --
\ *G Display day of week.
  dow 3 * days$ 1+ + 3 type
;


: .2r		\ n --
\ *G Display n as a two digit number with leading zeros.
  s>d <# # # #> type
;

: .4r		\ n --
\ *G Display n as a four digit number with leading zeros.
  s>d <# # # # # #> type
;

: .Time&Date	\ --
\ *G Display the system time
\ ** The format is:
\ *C   hh:mm:ss dd Mmm yyyy
  base @ decimal
  time&date				\ -- s m h dd mm yy
  dup #100 <				\ correct two digit year clocks
  if  #2000 +  endif
  rot .2r space				\ day - generate date
  swap 1- 3 * months + 3 type space	\ month
  .4r space				\ year
  .2r ." :"			 	\ hours - generate time
  .2r ." :"				\ minutes
  .2r					\ seconds
  base !
;

: .zone		\ zone --
  if  3 spaces  else  ." GMT"  endif
;

: .AnsiDate	\ zone --
\ *G Display the day of week, date and time. If zone is 0
\ ** GMT is displayed. The format is:
\ *C   dow, hh:mm:ss dd Mmm yyyy [GMT]
  .dow ." , " .time&date space .zone space
;

[then]


\ **************
\ To be included
\ **************

((
: IncludeMem	\ caddr len --
\ *G *\fo{INCLUDE} a block of memory as if it were a file.
  begin
    dup
   while
    $0A split 2>r			\ look for line feed
    dup if
      2dup + 1- c@ $0D =
      if  1-  endif
      dup
      if  evaluate  else  2drop  endif
    endif
    2r>
  repeat
  2drop
;
))


\ ************
\ *S Test code
\ ************

diags? [if]

\ create test$	\ -- caddr
\   ", get home.htm?send=submit&form=mine"

[then]


\ ======
\ *> ###
\ ======

decimal

