\ QPRIMS.FTH - definitions for packet queue handling

((
Copyright (c) 1997-2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002


To do
=====

Change history
==============
))

\ =========
\ *! qprims
\ *T Queues
\ =========
\ *P The PowerNet *\fo{QUEUE} data structure is the head of a
\ ** linked list of queue buffer data structures, usually
\ ** a *\fo{PBUF} or an I/O buffer. The first cell of anything
\ ** that is added to a *\fo{QUEUE} *\b{must} have a link field
\ ** at offset 0, the first cell.

struct queue	\ -- size
\ *S QUEUE structure
  cell field *q_head		\ pointer to head of queue
  cell field *q_tail		\ pointer to tail of queue
  cell field q_len		\ current length of queue
  cell field q_maxlen		\ maximum length of queue
\  cell field q_drops		\ entries dropped because of full queue
end-struct

: QLock		\ -- ; lock out interrupts
\ *G If QUEUEs are used in interrupts, or if the scheduler
\ ** is interrupt driven, place your locking code here.
\ ** Do not forget to change the *\fo{COMPILER ... TARGET} macros too.
\  mask-ints
;

: QUnLock	\ -- ; reenable interrupts
\ *G If QUEUEs are used in interrupts, or if the scheduler
\ ** is interrupt driven, place your locking code here.
\ ** Do not forget to change the *\fo{COMPILER ... TARGET} macros too.
\  unmask-ints
;
compiler
: QLock		\ -- ; lock out interrupts
\  mask-ints
;

: QUnLock	\ -- ; reenable interrupts
\  unmask-ints
;
target

: PeekQ		\ *queue -- *head
\ *G Return the first item in the queue without removing it.
\ ** Useful for testing and handling partially full/empty buffers.
  *q_head @
;
compiler
: PeekQ		\ *queue -- *head
  *q_head @
;
target

[undefined] Dequeue [if]
: Dequeue	\ *queue -- *head|0 ; 0 indicates nothing to remove
\ *G Remove the next item from the queue.
  QLock
  dup *q_head @ tuck if			\ -- head queue
    over @ over *q_head !		\ -- head queue ; replace head
    dup q_len @ 1-			\ -- head queue len ; decrement queue count
    2dup swap q_len !			\ -- head queue len'
    0= if				\ -- head queue ; if we removed the last one
      dup *q_tail off			\ -- head queue ; then zero the tail pointer
    endif
  then
  drop					\ -- head|0
  QUnLock
;
[then]

[undefined] Enqueue [if]
: Enqueue	\ *buffer *queue --
\ *G Add buffer to queue.
  QLock
  over off				\ ensure no *next in *buffer
  dup *q_tail @ if			\ if tail exists
    2dup *q_tail @ !			\ set buffer at tail
  endif
  2dup *q_tail !			\ set buffer as new tail
  dup *q_head @ 0= if			\ if head does not exist
    2dup *q_head !			\ make this one the head
  endif
  1 swap q_len +!			\ increase queue count
  drop					\ discard buffer
  QUnlock
;
[then]

: InitQ		\ *queue --
\ *G Initialise/reset the queue.
  queue erase
;


\ ======
\ *> ###
\ ======

