\ QUEUES.FTH - definitions for packet queues

((
Copyright (c) 1997-2003, 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002

To do
=====

Change history
==============
20080408 MPE001 Overhauled for new structures and aligned CPUs.
))

\ =========
\ *! queues
\ *T Queue diagnostic routines
\ =========
\ *P The code in *\i{QUEUES/Qdiags.fth} is only compiled if the equate
\ ** *\fo{DIAGS?} is non-zero.

diags? [if]

\ some diagnostic code
\ NB this has to build a list in free RAM
\ so that it can successfully list ALL queues

queue aligned equ /qdhead	\ -- len
\ *G The head of the list is the original queue header forced
\ ** to an aligned length.
cell BUFFHDRSIZE + aligned equ /qdentry	\ -- len
\ *G Each entry in the list consists of a link field, plus the
\ ** original PBUF header. The entry must be aligned.

-short-branches
: .q		\ *queue --
\ *G Display queue info.
  /qdhead NUMFREEPBUFS /qdentry * +	\ plus header for each entry + head/tail
  allocate if
    2drop  cr ." No memory!!"  exit
  endif					\ *q *ram -- ;
  dup >r				\ save RAM pointer
  2dup queue cmove			\ move queue header
  /qdhead +				\ bump RAM past header
  begin
    swap @ swap				\ point to next entry
    over				\ is there a buffer on the queue ??
  while
    2dup !				\ store address first
    cell +				\ bump dest past address pointer
    2dup BUFFHDRSIZE cmove		\ move header
    /qdentry cell - +			\ then bump pointer
  repeat
  nip r@				\ -- end start ; retrieve RAM pointer
  cr					\ display head data
  ." Head " dup *q_head @ .hex
  ." Tail " dup *q_tail @ .hex
  ." Len " dup q_len @ .hex
  /qdhead +				\ -- end first ; step over head
  -1 -rot ?do				\ -- curr ; start check value at -1
    cr
    i @ .dword space			\ show original address
    i cell +				\ base of PBUF_HDR
    dup p_type w@ BUFFTYPEMASK and .word space	\ type
    dup p_type w@ BUFFSIZEMASK and .word space	\ length
    dup p_datalen w@ .word space		\ char count
    dup p_data w@ .hex space			\ data offset
    dup *p_next @ .dword space			\ finally print the link address
    *p_next @  over =			\ get *next, same as check value?
    if  ." Queue looped!"  endif
    drop i @				\ get new check value
    halt? ?leave
  /qdentry +loop
  drop
  r> free drop				\ finally free RAM back to heap
;
+short-branches

: @qlen		\ *queue -- len
\ *G Count queue length.
  0 swap			\ start count at 0
  #100 0 do 			\ -- len *q ; set limit so we don't crash
    *q_head @ dup 0= ?leave
    swap 1+ swap
  loop
  drop				\ lose last address (0)
;

: .qlen		\ *queue txtaddr --
\ *G Display the queue length.
  count type
  @qlen dup #100 =
  if  drop -1  endif
  .hex space
;

: .qs		\ --
\ *G Display the lengths of the FreeIoQ and the FreePbufQ.
  cr FreePBufQ "" FreePb: " .qlen
[defined] TxEnetQ [if]
  cr TxEnetQ "" TxEnet: " .qlen
[then]
[defined] IPinQ [if]
  cr IPinQ "" IPin: " .qlen
[then]
[defined] TxSlipQ [if]
  cr IPinQ "" TxSlip: " .qlen
[then]
;

[then]


\ ======
\ *> ###
\ ======

