\ FTP.fth - FTP Server

((
Copyright (c) 2013
Rutherford Appleton Labs

Copyright (c) 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

  http://en.wikipedia.org/wiki/File_Transfer_Protocol
  http://slacksite.com/other/ftp.html	active vs passive
  http://cr.yp.to/ftp.html
  http://www.ipswitch.com/support/ws_ftp-server/guide/v5/ftpservTOC.html
  http://ftpguide.com/index.htm


To do
=====

Change history
==============
20130321 SFP003 Corrections. Added shutdown actions.
		Added additional cleanup to PORT if socket or
		file are open.
20130315 SFP002 Release for alpha testing.
20130111 SFP001 Started.
))

decimal


\ ======
\ *! ftp
\ *T FTP Server
\ ======
\ *P The Powernet FTP server is a multi-threaded server
\ ** that can accept multiple FTP connections.
\ ** The model used is described in *\i{SERVERS.FTH}. The
\ ** code for the command channel is based on the Telnet code.

\ *P The objectives of the design are to use "not a lot" of RAM,
\ ** and to keep the code size down by not providing many bells and
\ ** whistles. A side effect of the low RAM usage is that directory
\ ** listings are rather slow. Initially, the intention was only to
\ ** support the requirements of RFC959, the primary FTP
\ ** specification. The only client that actually works fully in
\ ** this mode is WS_FTP. In order to support clients that are not
\ ** so forgiving, a few extra commands are supported. The FTP
\ ** server has been tested with:
\ *(
\ *B Ipswitch WS_FTP Pro (2007). This has been our Windows FTP
\ ** client of choice for many years. Windows only.
\ *B WinSCP. A functional free tool. Windows only.
\ *B FileZilla. A functional free tool. Windows, Linux and Mac OS X.
\ *)

\ *P The use of the *\i{FATfiler} file system code is assumed.
\ ** Because this is not a fully thread-safe file system and only
\ ** has a single "working directory" for all threads, this FTP
\ ** server is not suitable for use with FTP clients that assume
\ ** a Unix-style operating system. Such clients include the FTP
\ ** client built into Finder on Mac OS X. FileZilla works well
\ ** on the Mac, as well as on Linux and Windows.


\ ***********
\ *S FTP data
\ ***********

struct /FTPdata	\ -- n
\ *G The standard service data structure is extended for FTP.
  /SVdata +			\ common beginning with SVdata
  int ftpState			\ FTP state: 0 before login
  int ftpPassive?		\ true if passive mode
  int ftpBinary?		\ true for binary transfers
  int ftpQuit?			\ true if QUIT received.
  int ftpDataIP			\ IPv4 of FTP data socket (PORT cmd), native endian
  int ftpDataPort		\ Port# of FTP data socket, native endian
  int ftpDataSock		\ handle of FTP data socket
  int ftpDataState		\ Data transfer machine state
  int ftpDataFile		\ Data file handle
  int ftpLine?			\ true for complete command line
  int ftpLineLen		\ Length of command line
  max_path field ftpSrc$	\ file name from
  max_path field ftpDest$	\ file name to
  512 field ftpRxTxBuff		\ transfer buffer
end-struct

: cleanFTP	\ *sv -- *sv
\ *G Clean up an FTP service block before it is released.
  dup ftpDataSock @ if
    dup ftpDataSock @ closesocket drop
    dup ftpDataSock off
  endif
  dup ftpDataFile @ if
    dup ftpDataFile @ close-file drop
    dup ftpDataFile off
  endif
;

compiler
: my_ftpState		\ -- addr
\ *G Holds the state of the FTP system.
  MySVD ftpState  ;
: my_ftpPassive?	\ -- addr
\ *G Holds the passive flag.
  MySVD ftpPassive?  ;
: my_ftpBinary?	\ -- addr
\ *G Holds the binary flag.
  MySVD ftpBinary?  ;
: my_ftpQuit?	\ -- addr
\ *G Holds the QUIT flag.
  MySVD ftpQuit?  ;
: my_ftpDataIP		\ -- addr
\ *G Holds the IP adddress of the data channel.
  MySVD ftpDataIP  ;
: my_ftpDataPort	\ -- addr
\ *G Holds the port number of the data channel.
  MySVD ftpDataPort  ;
: my_ftpDataSock	\ -- addr
\ *G Holds the socket handle of the data channel.
  MySVD ftpDataSock  ;
: my_ftpDataState	\ -- addr
\ *G Holds the state number of the data channel.
  MySVD ftpDataState  ;
: my_ftpDataFile	\ -- addr
\ *G Holds the handle of the file being read/written.
  MySVD ftpDataFile  ;
: my_ftpLine?		\ -- addr
\ *G Holds true if a complete line can be processed.
  MySVD ftpLine?  ;
: my_ftpLineLen		\ -- addr
\ *G Holds the current length of the line excluding terminators.
  MySVD ftpLineLen  ;
: my_ftpSrc$		\ -- addr
\ *G Buffer that holds a source file path as a counted string.
  MySVD ftpSrc$  ;
: my_ftpDest$		\ -- addr
\ *G Buffer that holds a destination file path as a counted string.
  MySVD ftpDest$  ;
: my_ftpBuff		\ -- addr
\ *G Buffer that holds FTP data for transfer.
  MySVD ftpRxTxBuff  ;
target


\ *******************
\ *S FTP vectored I/O
\ *******************
\ *P The FTP server establishes its own generic I/O for the
\ ** data and command channels. The data channel operations are
\ ** written for minimum RAM usage and use the sockets for
\ ** buffering.

\ ==============
\ *N Data socket
\ ==============

: checkSocket	\ hs -- ior
\ *G The *\i{ior} is returned non-zero if the socket is invalid
\ ** or not in TCPS_ESTABLISHED state.
  get_socket_addr dup 0=		\ socket is valid and has struct or not
  if  drop -3 exit  endif		\ error if 0
  so_*cb @ dup 0=			\ check for a valid control block
  if  drop -2 exit  endif		\ error if 0
  t_state w@ TCPS_ESTABLISHED <>	\ get tcp STATE
;

: sockKey?	\ hs -- #chars
\ *G Return the number of available characters from the socket.
\ ** If an error has occurred, 1 is returned.
  dup checkSocket
  if  drop 1  exit  endif
  pollSocket dup SOCKET_ERROR =
  if  drop 1  endif
;

: sockKey 	\ hs -- char
\ *G Return a character from a socket. On a socket error, an LF
\ ** is returned.
  { | char[ cell ] -- ch }
  begin  dup sockKey? 0= while  pause  repeat
  dup checkSocket			\ error?
  if  pause  drop $0A exit  endif
  char[ 1 0 recv SOCKET_ERROR =		\ read one character
  if  pause  $0A exit  endif		\ error
  char[ c@				\ return character read
;

: sockType	\ caddr len hs --
\ *G Send a string/buffer to the socket.
  dup checkSocket
  if  drop 2drop  exit  endif
  -rot 0 send drop
;

: sockEmit	\ char hs --
\ *G Send a character to the socket.
  { | char[ cell ] -- }
  swap char[ c!
  char[ 1 rot sockType
;

l: FTPcrlf		\ -- caddr
  #13 c,  #10 c,

: sockCr	\ hs --
\ *G Send a CR/LF pair to the socket.
  FTPcrlf 2 rot sockType
;

\ *P The following five words are used to provide generic I/O on
\ ** the FTP data channel.
\ *[
: FTPdataKey	( -- char )	my_ftpDataSock @ sockKey  ;
: FTPdataKey?	( -- flag )	my_ftpDataSock @ sockKey?  ;
: FTPdataEmit	( char -- )	my_ftpDataSock @ sockEmit  ;
: FTPdataType	( caddr len -- ) my_ftpDataSock @ sockType  ;
: FTPdataCr	( -- )		my_ftpDataSock @ sockCr  ;
\ *]

create ConFTPdata	\ -- addr ; OUT managed by upper driver
\ *G Function despatch table for FTP data channel I/O.
\ ** *\fo{OUT} is managed by the upper level driver.
  ' FTPdataKey ,	\ -- char ; receive char
  ' FTPdataKey? ,	\ -- flag ; check receive char
  ' FTPdataEmit ,	\ -- char ; display char
  ' FTPdataType ,	\ caddr len -- ; display string
  ' FTPdataCr ,		\ -- ; display new line

: [FTPdataIo	\ -- ; R: --  ipvec opvec
\ *G Redirects console I/O to the FTP data channel.
\ ** Use in the form:
\ *C   [FTPdataIo ... io]
  r>  ipvec @ >r opvec @ >r  >r
  ConFTPdata dup ipvec ! opvec !
;


\ =================
\ *N Command socket
\ =================

create ConsoleFTP	\ -- addr ; OUT managed by upper driver
\ *G Function despatch table for FTP command channel I/O.
\ ** *\fo{OUT} is managed by the upper level driver.
  ' SVkey ,		\ -- char ; receive char
  ' SVkey? ,		\ -- flag ; check receive char
  ' SVEmit ,		\ -- char ; display char
  ' SVtype ,		\ caddr len -- ; display string
  ' SVcr ,		\ -- ; display new line

: Init-ConsoleFTP	\ --
\ *G Initialise the command channel. Note that the
\ ** FTP socket must have been set up and and the private
\ ** service area initialised.
  Init-ConsoleSV
;

: FTPio		\ --
\ *G Select the FTP command channel as the console.
  ConsoleFTP dup opvec ! ipvec !
;

: checkFTP	\ -- ior
\ *G Return non-zero if there is an error in the FTP command
\ ** channel.
  my_hsocket @ checkSocket
;


\ *************************************
\ *S Sampling the command channel input
\ *************************************

: -ftpLine	\ --
\ *G Reset the FTP line input.
  my_ftpLine? off  my_ftpLineLen off
  MySVD SVtibBuff 'tib !		\ initialise TIB
  #tib off
;

: +ftpLine	\ --
\ *G Mark that a complete line is available.
  my_ftpLine? on
;

: ftpBS		\ --
\ *G The backspace operation for input.
  my_ftpLineLen @
  if  -1 my_ftpLineLen +!  endif
;

: +ftpCmdChar	\ char --
\ *G Add the character to the command line being assembled.
  my_ftpLineLen @ /SVtib 2- < if	\ if space
    MySVD dup SVtibBuff  swap ftpLineLen @ + c!	\ add character
    1 my_ftpLineLen +!			\ count it
  else
    drop  +ftpLine
  endif
;

: ?FTPacceptable	\ --
\ *G Process the next service input character on the command channel.
\ ** Input is terminated by LF, and CR is ignored. This satisfies
\ ** the requirements of DOS, Windows, Unices and the TCP/IP NVT
\ ** (Network Virtual Terminal).
  my_ftpLine? @ if  exit  endif
  SVKey? if
    case SVKey				\ get character
      8 of ftpBS  endof			\ BS
      9 of  bl +ftpCmdChar  endof	\ TAB, convert to space
      $0D of  CheckFTP drop  endof	\ CR, ignore
      $0A of  +ftpLine  endof		\ LF, terminator
      $0C of  checkFTP drop  endof	\ FF, treat like CR
      $7F of  ftpBS  endof		\ DEL
        dup +ftpCmdChar			\ stash char
    endcase
  endif
;


\ *********************
\ *S Diagnostic control
\ *********************

1 value FTPdiags?	\ -- n
\ *G Set this non-zero to get diagnostic information.

XC? [if]
compiler
: [ftp  ( -- )  FTPdiags? if  [io consoleio decimal  ;
\ *G A *\fo{COMPILER} macro used to surround debug code, and
\ ** terminated by *\fo{FTP]}.
\ *E   [FTP  ." debug message"  FTP]
: ftp]  ( -- )  io]  endif  ;
\ *G A *\fo{COMPILER} macro that terminates an *\fo{[FTP ... FTP]}
\ ** structure.
target
[else]
: [ftp		\ --
  postpone FTPdiags?  postpone if  postpone [io  postpone consoleio  postpone decimal
; immediate
: ftp]		\ --
   postpone io]   postpone endif
; immediate
[then]

: .fdLine	\ caddr len --
\ *G Display FTP text with leading CR on Forth console. If
\ ** *\fo{FTPdiags?} is set to zero, no action is taken.
  FTPdiags? if
    consoleio decimal
    cr ." FTP Cmd Socket " my_hsocket @ . ." :"  type  ." :"
    FTPio
  else
    2drop
  endif
;


\ ****************************
\ *S Directory listing for FTP
\ ****************************
\ *P Each line of the display is "sort of" in Unix ls format.

\ *E ----------   1 owner    group         1803128 Jul 10 10:18 ls-lR.Z
\ ** d---------   1 owner    group               0 May  9 19:45 Softlib

[undefined] months [if]
create months	\ -- addr
\ *G String containing 3 character text for the months.
  z", JanFebMarAprMayJunJulAugSepOctNovDec"
[then]

: .ftpDate	\ --
\ *G Display the current directory entry's date in the form:
\ *C   Mmm dd  yyyy
\ *P e.g.
\ *C   Apr 30  2012
  24 dir[] uaLEw@ 			\ 16 bit date
  5 unp 4 unp 1980 +			\ -- dd mm yyyy
  swap 1- 0 max 11 min			\ -- dd yyyy mm'
  3 * months + 3 type 			\ -- dd yyyy ; month
  swap 3 .r  2 spaces  0 .r		\ day year
;

: .ftpDirLine	\ --
\ *G Display a directory entry in FTP format. FTP clients get the
\ ** size information from this format.
  folder?
  if  ." d"  else  ." -"  endif
  ." --------- 1 owner group "
  dsize 13 .r space
  .ftpDate space d-name cr
;

: .ftpFile	\ --
\ *G List the file data for the last file found.
  savedDirEnt 2@ set-es		\ restore directory entry
  .ftpDirLine
;

: .ftpdir	\ --
\ *G Display a list of files in FTP format.
  dirhome  get-es todir  DirValid? if
    begin
      dir?				\ valid entry
     while				\ --
      .ftpDirLine  dirnext 0=
    until then
  then
  set-es
;

: .ftpDirNlst	\ --
\ *G Display a list of file names in FTP NLST format.
  dirhome  get-es todir  DirValid? if
    begin
      dir?				\ valid entry
     while				\ --
      folder? 0=
      if  d-name cr  then
      dirnext 0=
    until then
  then
  set-es
;


\ *****************
\ *S Status returns
\ *****************

: .ftpResp	\ caddr len --
\ *G Send the string plus a CR/LF pair to the command socket and
\ ** optionally to the console.
  2dup type cr
  [ftp  2dup cr type  ftp]
  2drop
;

: .ftp150	\ --
\ *G Return status 150 - about to transfer data
  s" 150 File status okay." .ftpResp
;

: .ftp200	\ --
\ *G Return status 200 - good command.
  s" 200 Command good." .ftpResp
;

: .ftp202	\ --
\ *G Return status 202 - not needed
  s" 202 not needed here" .ftpResp
;

: .ftp211	\ --
\ *G Return status 211 - not available
  s" 211 No status available." .ftpResp
;

: .ftp230	\ --
\ *G Return status 230 - user logged in.
  s" 230 user logged in" .ftpResp
;

: .ftp250	\ --
\ *G Return status 250 - good file command.
  s" 250 Requested file action okay, completed." .ftpResp
;

: .ftp226	\ --
\ *G Return status 226 - transfer successful.
  s" 226 File transfer successful" .ftpResp
;

: .ftp331	\ --
\ *G Return status 331 - password needed
  s" 331 Password needed." .ftpResp
;

: .ftp350	\ --
\ *G Return status 350 - need more info.
  s" 350 Requested action needs more information." .ftpResp
;

: .ftp425	\ --
\ *G Return status 425 - can't make data connection.
  s" 425 Can't open data connection" .ftpResp
;

: .ftp426	\ --
\ *G Return status 426 - aborted.
  s" 426 Connection closed; transfer aborted." .ftpResp
;

: .ftp450	\ --
\ *G Return FTP error code 450 - action not taken.
  s" 450 Requested file action not taken." .ftpResp
;

: .ftp451	\ --
\ *G Return FTP error code 451.
  s" 451 Error; action aborted" .ftpResp
;

: .ftp502	\ --
\ *G Return FTP error code 502 command not implemented.
  s" 502 Command not implemented." .ftpResp
;

: .ftp504	\ --
\ *G Return FTP error code 504.
  s" 504 Command not implemented for that parameter." .ftpResp
;

: .ftp550	\ --
\ *G Return FTP error code 550.
  s" 550 Requested action not taken." .ftpResp
;


\ *************************
\ *S Data socket operations
\ *************************

interpreter
: [sm		\ -- 0
\ *G Starts the definition of a state machine's states.
  0  ;
: smState	\ n -- n+1
\ *G Defines the next state as an *\fo{EQU} and increments the
\ ** state number.
  dup equ  1+  ;
: sm]		\ n --
\ *G Finishes the state machine and defines an equate of the
\ ** number of states.
  equ  ;
target

\ *P The FTP data transfer state machine.
\ *[
[sm
  smState ftpDtIdle		\ No data transfer in progress
  smState ftpListening		\ PASV mode, wait connection
  smState ftpConnected		\ wait transfer command
  smState ftpReadSendData	\ RETRieve
  smState ftpRecvWriteData	\ STORe
  smState ftpErrorState		\ it's bad if we get here!
sm] #FTPsm	\ -- n
\ *]

\ *P Select the the FTP data socket states.
\ *[
: goFTPidle	\ --
  ftpDtIdle my_ftpDataState !  ;
: goFTPlistening	\ --
  ftpListening my_ftpDataState !  ;
: goFTPconnected	\ --
  ftpConnected my_ftpDataState !  ;
: goFTPReadSend	\ --
  ftpReadSendData my_ftpDataState !  ;
: goFTPRecvWrite	\ --
  ftpRecvWriteData my_ftpDataState !  ;
\ *]

[undefined] umin [if]
: umin		\ u1 u2 -- u1|u2
\ *G Minimum of two unsigned values.
  2dup u>
  if  swap  then
  drop
;
[then]

: closeDataFile	\ --
\ *G Perform an emergency close of the data file if it is open.
  my_ftpDataFile if
    my_ftpDataFile @ close-file drop
    my_ftpDataFile off
  endif
;

: hasConn?	\ hs -- ior true | 0
\ *G Return true and an ior if the socket has an error or a
\ ** completed connection. For a good connection, *\i{ior} is
\ ** non-zero.
  get_socket_addr dup 0=
  if  drop -1 -1 exit  endif
  dup skValid? 0=
  if   drop -2 -1 exit  endif
  TCPConnQ?
  if  0 -1 exit  endif
  0
;

: newDataSocket	\ -- hs|0
\ *G Create a new FTP data socket and set it to listen.
\ ** The socket handle is also stored in the FTP service
\ ** structure.
  TCPsocket dup if
    dup Listen drop			\ use default local port
  endif
;

: discDataSocket	\ --
\ *G If open, flush the socket transmit data and disconnect the
\ ** FTP data socket. This is a graceful close unless the socket
\ ** has failed.
  my_ftpDataSock @ if
    my_ftpDataSock @ checksocket if
      my_ftpDataSock @ closeSocket drop
    else
      my_ftpDataSock @ waitSocketSent  #200 ms
      my_ftpDataSock @ disconnect drop
    endif
    my_ftpDataSock off
  endif
;

: closeDataSocket	\ --
\ *G If open, close the FTP data socket. This is not a graceful close.
  my_ftpDataSock @ if
    my_ftpDataSock @ closeSocket drop
    my_ftpDataSock off
  endif
;

: termDataStream	\ --
\ *G Gracefully shut down a STREAM mode data transfer and return
\ ** to FTPDTIDLE state.
  closeDataFile  discDataSocket  goFTPidle
;

: closeDataStream	\ --
\ *G Hurriedly shut down a STREAM mode data transfer and return
\ ** to FTPDTIDLE state.
  closeDataFile  closeDataSocket  goFTPidle
;

: CheckFTPdata	\ -- ior
\ *G Return non-zero if the FTP data socket is in error.
  my_ftpDataSock @ get_socket_addr 0=
;

: FTPdataFailed?	\ -- ior
\ *G Return non-zero if the FTP data socket is not in established
\ ** state. If not established, the data socket is closed and we
\ ** return to FTPDTIDLE state.
  my_ftpDataSock @ checkSocket dup
  if  discDataSocket  endif
;

: doDTidle	\ --
\ *G The action in FTPDTIDLE state. Check the data socket. If it
\ ** fails, close it.
  CheckFTPdata if			\ failed
[ftp  cr ." Closing FTP data socket in FTPDTIDLE"  ftp]
    discDataSocket
  endif
;

: doDTlistening	\ --
\ *G The action in FTPLISTENING state.
  my_ftpDataSock @ hasConn? if
    if					\ socket failure
\ [ftp  cr ." Closing FTP data socket in FTPLISTENING"  ftp]
      closeDataSocket  goFTPidle  exit	\ and need to start again
    endif
    my_ftpDataSock @ 0 0 saccept
    dup SOCKET_ERROR = if
\ [ftp  cr ." ACCEPT failed in FTPLISTENING"  ftp]
      drop
      closeDataSocket  goFTPidle  exit	\ and need to start again
    endif
    \ -- hsnew
\ [ftp  cr ." ACCEPT returns " dup . ." in FTPLISTENING"  ftp]
    my_ftpDataSock @ closeSocket drop	\ close original listener
    my_ftpDataSock !			\ set new active socket
    goFTPconnected			\ now connected
  endif
;

: doDTconnected	\ --
\ *G The action in FTPCONNECTED state.
  FTPdataFailed? if
\ [ftp  cr ." FTP data socket not TCPS_ESTABLISHED in FTPCONNECTED"  ftp]
    goFTPidle
    exit
  endif
;

: doDTReadSend	\ --
\ *G The action in FTPREADSENDDATA state.
  my_ftpBuff #512 my_ftpDataFile @ read-file if	\ -- len
\ [ftp  cr ." DODTRREADSEND Cannot read file"  ftp]
    drop termDataStream .ftp451  exit
  endif
\ [ftp  cr ." Read " dup . ." bytes from file " my_ftpDataFile @ . ." for socket " my_ftpDataSock @ .  ftp]
  dup if
    my_ftpBuff swap begin		\ -- caddr len
      2dup my_ftpDataSock @ -rot 0 send
      dup SOCKET_ERROR = if		\ -- caddr len len'
        drop 2drop  termDataStream .ftp426  exit
      endif
      /string 0 max dup 0=
    until
    2drop
\ [ftp  cr ." Bytes sent"  ftp]
  else
    drop  termDataStream .ftp226
  endif
;

: doDTRecvWrite	\ --
\ *G The action in FTPRECVWRITEDATA state.
  my_ftpDataSock @ pollsocket		\ -- len|SOCKET_ERROR
  dup SOCKET_ERROR = if			\ closure means finished
    drop  termDataStream .ftp226 exit
  endif
  dup if				\ -- len
    #512 umin >r
    my_ftpDataSock @ my_ftpBuff r> 0 recv
    dup SOCKET_ERROR = if		\ -- len #read|SOCKET_ERROR
      drop  termDataStream .ftp450 exit
    endif
    dup if				\ -- #read
\ [ftp  cr ." Writing " dup . ." bytes to file"  ftp]
      my_ftpBuff over my_ftpDataFile @ write-file if
        drop  termDataStream .ftp450 exit
      endif
    endif
  endif
  drop
;

: doDTerror	\ --
\ *G The action in FTPERRORSTATE state.
  termDataStream
;

create FTPdataActions	\ -- addr
\ *G A table of xts corresponding to the FTP data channel state.
  ' doDTidle ,
  ' doDTlistening ,
  ' doDTconnected ,
  ' doDTReadSend ,
  ' doDTRecvWrite ,
  ' doDTerror ,

: doFTPdata	\ --
\ *G We have a data socket. Execute the action for the state.
  my_ftpDataState @ ftpErrorState min
  cells FTPdataActions + @ execute
;

: waitDataConnected	\ -- ior
\ *G Wait for a passive mode data connection to be made. Return
\ ** non-zero on failure.
  my_ftpDataState @ ftpListening <>	\ check listening socket
  if  -3 exit  endif			\ no listening socket
  FTP_DATA_ESTAB_MS later 1 or begin
    TcpIdleMs ms  doDTlistening		\ check connection
    my_ftpDataState @ ftpConnected <>
   while
    dup timedout?
  until then
  drop
  my_ftpDataState @ ftpConnected <>	\ 0 on success
;

: waitDataEstablished	\ -- ior
\ *G Wait for a passive mode data connection to get to TCP state
\ ** *\fo{TCPS_ESTABLISHED}. Return non-zero on failure.
  FTP_DATA_ESTAB_MS later 1 or begin
    my_ftpDataSock @ -Listen? if
      TCPS_ESTABLISHED = if		\ we have a connection
        goFTPconnected  drop 0 exit	\ and are now connected
      else
        drop -2 exit
      endif
    endif
    TcpIdleMs ms  dup timedout?
  until
  drop -1
;


\ *********************
\ *S Command processing
\ *********************
\ *P An FTP command exists on a single line. The first token
\ ** identifies the command. Each command identifier is a Forth
\ ** word which parses any more data needed by the command.

[undefined] parse-name [if]
: parse-name	\ "text" -- caddr len
\ *G Return the next space-delimited string from the input stream.
  bl parse-word
;
[then]

: parse-path	\ "<pathname>" -- caddr len
\ *G Return the next space-delimited path name from the input
\ ** stream. Clip it to *\fo{MAX_PATH 1-} bytes.
  parse-name bl skip -trailing max_path 1- umin
;

: getSrcParam	\ --
\ *G Read the parameter and store it as the source name.
  parse-path my_ftpSrc$ place
;

: getDestParam	\ --
\ *G Read the parameter and store it as the destination name.
  parse-path my_ftpDest$ place
;

: getSrcDir	\ --
\ *G Read the parameter and store it as the source name. If the
\ ** name ends in a '/' character, remove it. Some FTP clients
\ ** terminate directory names with a '/', which can confuse
\ ** the FAT file system.
  parse-path
  dup 1 > if				\ single '/' is ok
    2dup + 1- c@ [char] / =		\ if last char '/'
    if  1-  then			\ remove it
  then
  my_ftpSrc$ place
;

: SrcParam	\ -- addr len
\ *G Return the string saved as the source parameter.
  my_ftpSrc$ count
;

: FTPannounce	\ --
\ *G Issues FTP signon message.
  ." 220 " .PNET ."  FTP server" cr
;

: +decByte	\ u caddr len -- u' caddr' len'
\ *G Accumulate the next byte of a comma separated set of numbers,
\ ** e.g. 1,25,33,4. The accumulator *\i{u} is shifted left by
\ ** 8 bits and the next number in the text is added. The updated
\ ** accumulator and remaining text are returned.
  base @ >r  decimal
  rot 8 lshift >r
  0 0 2swap >number dup
  if  1 /string  endif
  2swap drop r> + -rot
  r> base !
;

: .decByte	\ u -- u'
\ *G Display the top byte of *\i{u} as a decimal number and shift
\ ** it left by 8 bits.
  base @ >r  decimal
  dup #24 rshift 0 .r  8 lshift
  r> base !
;

: .comma	\ --
\ *G Display a comma.
  [char] , emit
;

: FTPabort426	\ -- 426
\ *G Close the data socket and data file and command with a 426
\ ** response.
\ [ftp  cr ." No data socket " SrcParam type  ftp]
  closeDataStream .ftp426  426
;

: setDataSock	\ -- ior
\ *G Depending on the mode, check or establish a data connection.
\ ** Return non-zero on error.
  my_ftpPassive? @ if			\ PASV used, socket listening
\ [ftp  cr ." Passive"  ftp]
    my_ftpDataState @ ftpConnected <> if  \ still waiting
\ [ftp  cr ." Not connected"  ftp]
      waitDataConnected	if		\ wait some more
\ [ftp  cr ." Connect failed"  ftp]
        FTPabort426 exit
      endif
    endif
    waitDataEstablished	if		\ wait some more
\ [ftp  cr ." Not established"  ftp]
      FTPabort426 exit
    endif
    FTPdataFailed? if
\ [ftp  cr ." data failed"  ftp]
      FTPabort426 exit
    endif
  else					\ PORT used
\ [ftp  cr ." Active"  ftp]
    my_ftpDataIP @ 0 my_ftpDataPort @
    TCPconnect swap my_ftpDataSock !  if
\ [ftp  cr ." Cannot ACTIVE connect"  ftp]
      my_ftpDataSock off
      closeDataStream .ftp425  425 exit
    endif
  endif
  0
;

: .quo		( -- ) [char] " emit  ;
\ *G Display a double quotes.

: .pwdResp	\ --
\ *G Display the PWD response with no CR/LF.
  ." 257 "  .quo pwd ." /" .quo
;

: .257SrcResp	\ --
\ *G A 257 response with the source buffer
  ." 257 " .quo SrcParam type .quo
;


\ *********************
\ *S Login and security
\ *********************
\ *P Access to FTP is provided through the FTP verbs (words)
\ ** *\fo{USER} *\fo{PASS} and optionally *\fo{ACCT}.

defer doFTPuser	\ --
\ *G Process the FTP USER command. The default accepts any
\ ** user. Once the user and password have been confirmed,
\ ** the *\fo{ftpState} variable can be set to 1.
defer doFTPpass	\ --
\ *G Process the FTP PASS command. The default accepts any
\ ** password. Once the user and password have been confirmed,
\ ** the *\fo{ftpState} variable can be set to 1.
defer doFTPacct	\ --
\ *G Process the FTP ACCT command. The default accepts any
\ ** password. Once the user and password have been confirmed,
\ ** the *\fo{ftpState} variable can be set to 1.

: (FTPuser)	\ --
\ *G The default action performed for the USER command. All users
\ ** are accepted.
  parse-name 2drop			\ get user name
  .ftp331
;  assign (FTPuser) to-do doFTPuser

: (FTPpass)	\ --
\ *G The default action performed for the PASS command. All passwords
\ ** are accepted.
  parse-name 2drop			\ get password
  .ftp230  1 my_ftpState !		\ set state non-zero
;  assign (FTPpass) to-do doFTPpass

: (FTPacct)	\ --
\ *G The default action performed for the ACCT command. All accounts
\ ** are accepted.
  parse-name 2drop			\ get account
  .ftp202
;  assign (FTPacct) to-do doFTPacct


\ ***************************
\ *S Implemented FTP commands
\ ***************************
\ *P FTP commands all start with a command name ("verb" in FTP
\ ** parlance). Each is implemented as a Forth word in the *\fo{FTPvoc}
\ ** vocabulary.

vocabulary FTPvoc	\ --
\ *G Vocabulary holding FTP commands for execution.
' FTPvoc >body @ constant FTPwid	\ -- x
\ *G Wordlist holding FTP commands for execution.

: FTPinterpret	\ --
\ *G Interpret the current line containing an FTP command.
  decimal
  >in off  my_ftpLineLen @ #tib !	\ make buffer active
  source .fdLine			\ display command on console?
  parse-name 2dup upper			\ get command name/verb
  FTPwid search-wordlist		\ look it up
  if  execute  else .ftp502  endif
  -ftpLine
;

also FTPvoc definitions
\ *G Start of FTP command definitions.

: USER		\ -- ; USER <name>
\ *G Handle the USER command.
  doFTPuser  \ -ftpLine
;
: PASS		\ -- ; PASS <password>
\ *G Handle the PASS command.
  doFTPpass  \ -ftpLine
;
: ACCT		\ -- ; ACCT <password>
\ *G Handle the ACCT command.
  doFTPacct  \ -ftpLine
;
: SYST		\ -- ; SYST
\ *G Handle the SYST command.
[ftp  cr ." 215 " .PNET  ftp]
  ." 215 " .PNET cr  \ -ftpLine
;
: STAT		\ -- ; STAT
\ *G Process the STAT command. Commands with parameters are
\ ** rejected.
  bl word c@
  if  .ftp504  else  .ftp211  endif
\  -ftpLine
;
: HELP		\ -- ; HELP [<param]
\ *G The HELP command just describes the system.
  ." 200 MPE Embedded PowerNet written with VFX Forth." cr
\  -ftpLine
;
: NOOP		\ -- ; NOOP
\ *G The NOOP command just returns good.
  .ftp200  \ -ftpLine
;
: STRU		\ -- ; STRU F
\ *G Obsolete command - we just accept F.
  char upc [char] F =
  if  .ftp200  else  .ftp504  endif
\  -ftpLine
;
: MODE		\ -- ; MODE S
\ *G Obsolete command - we just accept S.
  char upc [char] S =
  if  .ftp200  else  .ftp504  endif
\  -ftpLine
;
: TYPE		\ -- ; TYPE <params>
\ *G Process the TYPE command. Valid parameters are
\ *C  A, A N, I, L 8
  case  char upc
    [char] A of  my_ftpBinary? off  .ftp200  endof
    [char] I of  my_ftpBinary? on  .ftp200  endof
    [char] L of  char 8 =
          if  my_ftpBinary? on .ftp200  else .ftp504  endif
      endof
      .ftp504
  endcase
\  -ftpLine
;
: QUIT		\ -- ; QUIT
\ *G Process the QUIT command.
  .ftp200 ( -ftpLine )  500 ms  my_FTPquit? on
;
: ABOR		\ -- ; ABOR
\ *G Process the ABORt command.
  my_ftpDataSock @ 0<>
  my_ftpDataFile @ 0<>  or if
    termDataStream  .ftp426
  endif
  .ftp226  \ -ftpLine
;
: PORT		\ -- ; PORT a1,a2,a3,a4,ph,pl
\ *G Process the PORT command. The IP address and port
\ ** are saved in *\fo{my_ftpDataIP} and *\fo{my_ftpDataPort}.
  my_ftpDataSock @			\ close previous connection
  if  termDataStream  endif
  0 parse-name
  +decByte +decByte +decByte +decByte
  rot my_ftpDataIP !  0 -rot
  +decByte +decByte
  2drop my_ftpDataPort !
  my_ftpPassive? off
  .ftp200  \ -ftpLine
;

: PASV		\ -- ; PASV
\ *G Process the PASV command. RFC 0959 does not document PASV
\ ** well. See *\f{http://cr.yp.to/ftp/retr.html} for more details.
\ ** A successful response is of the form:
\ *C   227 Entering Passive Mode (a1,a2,a3,a4,ph,pl)
  my_ftpDataSock @			\ close previous connection
  if  termDataStream  endif
  NewDataSocket dup 0= if		\ new data socket listener
\ [ftp  cr ." NEWDATASOCKET failed"  ftp]
    drop goFTPidle .ftp451 exit
  endif
  my_ftpDataSock !

  my_ftpPassive? on  goFTPlistening	\ mark in passive mode and listening

  ." 227 Entering Passive Mode ("	\ response code
  IPaddress @(n) 4 0			\ our IP address
  do .decByte .comma  loop		\ a1,a2,a3,a4,
  drop
  my_ftpDataSock @
  get_socket_addr so_portloc w@		\ port number
  dup my_ftpDataPort !  #16 lshift	\ port is 16 bit
  .decByte .comma .decByte  drop  	\ ph,pl
  ." )" cr  \ -ftpLine
[ftp  cr ." FTP PASV good " IPaddress .IPnet space my_ftpDataPort @ .  ftp]
;

: RETR		\ -- ; RETR <filepathname>
\ *G Process the RETRieve command. Starts transmission of a server
\ ** file to the FTP client.
  getSrcParam  SrcParam r/o open-file
  swap my_ftpDataFile ! if
\ [ftp  cr ." Cannot open " SrcParam type  ftp]
    termDataStream .ftp550  exit
  endif
  .ftp150
  setDataSock 0= if			\ ensure data connection
\ [ftp  cr ." Switch to FTPREADSENDDATA"  ftp]
    goFTPReadSend			\ transfer performed by doDTReadSend
  endif
\  -ftpLine
;
: STOR		\ -- ; STOR <filepathname>
  parse-path 2dup my_ftpDest$ place	\ file name
\ [ftp  cr ." File name :" 2dup type ." :"  ftp]
  r/w create-file swap my_ftpDataFile ! if
\ [ftp  cr ." Cannot create " my_ftpDest$ count type  ftp]
    termDataStream .ftp450  exit
  endif
  .ftp150
  setDataSock 0=			\ ensure data connection
  if  goFTPRecvWrite  endif		\ transfer performed by doDTRecvWrite
\  -ftpLine
;

: CWD		\ -- ; CWD dirpath
\ *G Process the CWD command.
  getSrcDir  SrcParam $cwd
  if  .ftp550  else  .ftp250  endif
\  -ftpLine
;
: XCWD	CWD  ;	\ -- ; XCWD dirpath
\ *G Process the XCWD command.
: PWD		\ -- ; CWD dirpath
\ *G Process the PWD command.
  .pwdResp cr
[ftp  cr .pwdResp  ftp]
\  -ftpLine
;
: XPWD	PWD  ;	\ -- ; XCWD dirpath
\ *G Process the XPWD command.
: LIST		\ -- ; LIST <filespec/dirspec>
\ *G Process the LIST command. The parameter is optional.
\ ** If there is no parameter, the current directory is
\ ** listed. If the parameter is a file, the details of
\ ** that file are listed. If the parameter is a directory,
\ ** the files and directories are listed in "sort of" the Unix
\ ** *\i{ls} format.
  getSrcParam  SrcParam dup if
    2dup fileExist? if			\ single file listing
      2drop  ['] .ftpFile
    else
      $cwd 0= if			\ directory listing
        ['] .ftpDir
      else
        closeDataStream			\ for passive mode
        .ftp550 -ftpLine exit		\ bad parameter
      endif
    endif
  else
    2drop  ['] .ftpDir
  endif					\ -- xt ; listing word
  .ftp150				\ initial response
  setDataSock 0= if			\ ensure data connection
    [FTPdataIo  execute  io]		\ list on data connection
    discDataSocket 			\ close data connection
    .ftp226				\ final response
  else
    drop
  endif
  goFTPidle  \ -ftpLine
;
: NLST		\ -- ; LIST <filespec/dirspec>
\ *G Process the LIST command. The parameter is optional.
\ ** If there is no parameter, the current directory is
\ ** listed. If the parameter is a directory, only the files
\ ** are listed by returning one name per line.
  getSrcParam  my_ftpSrc$ c@ if		\ if there's a parameter
    SrcParam $cwd if
      .ftp550 -ftpLine goFTPidle exit	\ bad parameter
    endif
  endif
  .ftp150				\ initial response
  setDataSock 0= if			\ ensure data connection
    [FTPdataIo  .ftpDirNlst  io]	\ list on data connection
    discDataSocket 			\ close data connection
    .ftp226				\ final response
  endif
  goFTPidle  \ -ftpLine
;
: RNFR		\ -- ; RNFR <filespec>
\ *G Process the RNFR command, checking that the file exists.
  getSrcParam  SrcParam fileExist?
  if  .ftp350  else  .ftp450  0 my_ftpSrc$ c!  then
  -ftpLine
;
: RNTO		\ -- ; RNTO <filespec>
\ *G Process RNTO, the second part of the file rename operation.
  getDestParam  my_ftpDest$ c@ if
    SrcParam  my_ftpDest$ count rename-file
    if  .ftp550  else  .ftp250  then
  else
    .ftp550
  endif
\  -ftpLine
;

: DELE		\ -- ; DELE <filespec>
\ *G Process the file delete command.
  getSrcParam  my_ftpSrc$ c@ if
    SrcParam delete-file
    if  .ftp550  else  .ftp250  then
  else
    .ftp550
  endif
\  -ftpLine
;

: MKD		\ -- ; MKD <pathspec>
\ *G Process the make directory command.
  getSrcDir  my_ftpSrc$ c@ if
    SrcParam mkDirEx if
      .ftp550
    else
[ftp  cr .257SrcResp  ftp]
      .257SrcResp cr
    then
  else
    .ftp550
  endif
\  -ftpLine
;
: XMKD	( -- )	MKD  ;

: RMD		\ -- ; RMD <pathspec>
\ *G Process the delete directory command.
  getSrcDir  my_ftpSrc$ c@ if
    SrcParam $rmd
    if  .ftp550  else  .ftp250  then
  else
    .ftp550
  endif
\  -ftpLine
;
: XRMD	( -- )	RMD  ;

previous definitions
\ *G End of FTP command definitions.


\ ********************
\ *S FTP service tasks
\ ********************

((
: (TestSV)	\ --
  my_hsocket @ get_socket_addr 0=
  if  cr my_hsocket @ . ." bad" exit  endif
  my_socket @ so_*cb @ 0=		\ has control block?
  if  cr my_hsocket @ . ." no control block" exit  endif
  my_socket @ so_*service @ 0= 		\ has service block?
  if  cr my_hsocket @ . ." no service block" exit  endif
  my_socket @ so_*service @ SVdone @
  if  cr my_hsocket @ . ." SVdone set" exit  endif
  my_socket @ TcpState@ TCPS_ESTABLISHED <>
  if  cr my_hsocket @ . ." not established" exit  endif
;

: TestSV	\ --
  [ftp  (TestSV)  ftp]
;
))

: FTPcommands	\ --
\ *G Empty the return stack, store 0 in *\fo{SOURCE-ID}, and
\ ** enter interpretation state. *\fo{FTPcommands} repeatedly
\ ** inputs an command line and *\fo{FTPinterpret}s it. Note that
\ ** any task that uses *\fo{FTPcommands} must initialise *\fo{'TIB},
\ ** *\fo{BASE}, *\fo{IPVEC}, and *\fo{OPVEC}.
  0 to source-id  state off		\ set up
  my_ftpPassive? off  my_ftpBinary? off
  decimal
  -ftpLine
  begin
[ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
    FTPio ?FTPacceptable		\ process input
    my_ftpLine?	@ if			\ check command channel input
      ['] FTPinterpret catch drop	\   interpret line
    endif
    my_ftpDataSock @ if			\ if data socket open
      ['] doFTPdata catch drop		\ process any data
    endif
    pause
    checkFTP  my_FTPquit? @ or
  until
\ [ftp  cr ." FTPcommands done " checkFTP . my_FTPquit? @ .  ftp]
  discDataSocket			\ disconnect gracefully
;

: FTPservice	\ --
\ *G The FTP service task launched for each established
\ ** Telnet connection.
  SVstartup
  Init-ConsoleFTP FTPio
  #100 ms
  ['] cleanFTP MySVD SVappClean !	\ set shutdown action
  my_ftpstate off			\ not logged in yet
  FTPdataPort# my_ftpDataPort !		\ default data port
  my_ftpDataSock off			\ no data socket yet
  FTPannounce				\ FTP sign on
  FTPcommands				\ process FTP commands
  SVshutdown
;

SVsingle? 0= [if]			\ if multi-threaded
: #FTPconns	\ -- u
\ *G Return the number of FTP connections. This is the number
\ ** FTP service tasks running. Each one needs two sockets.
   0 SVchain
   begin
     @ dup
    while
     dup SVsk# @ get_socket_addr	\ @ missing ; MGS003
     so_service @ service_ftp =		\ if FTP
     if  swap 1+ swap  endif		\ count it
   repeat
   drop
;
[then]


\ *********************
\ *S FTP listening task
\ *********************

: FTPserverPass	\ --
\ *G One iteration through the FTP server.
  my_hsocket @ >r			\ preserve listening socket#
  begin
    r@ isMySocket
    my_hsocket @ 0 0 Saccept		\ wait for connection or failure
    dup SOCKET_ERROR <>
   while
[ SVsingle? ] [if]
    isMySocket  FTPservice
[else]
    ['] FTPservice StartService		\ launch another service
[then]
  repeat
  drop  r> closeSocket drop		\ socket error
;

: FTPserver	\ -- ; stay here forever
\ *G The FTP listening task.
  s" FTP" SRVRstartup
  begin
[ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
    service_ftp /FTPdata FTPcmdPort# ServiceCreate
    FTPserverPass  pause
[ SVsingle? 0= ] [if]
    begin
      #FTPconns #FTPmaxConns >=
     while
      100 ms
    repeat
[then]
  again
;

0 value FTPtask	\ -- 0|task
\ *G Returns 0 or the FTP server task if running.

: startFTPserver	\ --
\ *G Start the FTP server task.
  FTPtask 0= if
    ['] FTPServer SVinitiate to FTPtask
  endif
;

: stopFTPserver	\ --
\ *G Stop the FTP server.
  FTPtask if
    FTPtask SVterminate  0 -> FTPtask
    cr ." FTP server stopped"
  endif
;


\ **************
\ *S Diagnostics
\ **************
\ *P This code is only compiled if the *\fo{EQU}ate *\fo{DIAGS?}
\ ** is set non-zero.

diags? [if]

: +ftp  initfatfs . powernet  ;
\ *G Start the file system and PowerNet.

: .FTPchain	\ --
\ *G Display data about the FTP sockets.
   FTPtask 0= if
     cr ." FTP not running"  exit
   endif
   cr ." Listening on socket " FTPtask my_hsocket his @ .
   cr ." Serving on sockets "
   SVchain
   begin
     @ dup
    while
     dup SVsk# @ get_socket_addr	\ @ missing ; MGS003
     so_service @ service_ftp =
     if  dup SVsk# @ .  endif
   repeat
   drop
;

[then]


\ ******
\ *> ###
\ ******

((
FTP error numbers by order

110 MARK yyyy = mmmm
120 Service ready in nnn minutes.
125 Data connection already open; transfer starting.
150 File status okay; about to open data connection.

200 Command okay.
202 Command not implemented, superfluous at this site.
211 System status, or system help reply.
212 Directory status.
213 File status.
214 Help message.
On how to use the server or the meaning of a particular
non-standard command. This reply is useful only to the
human user.
215 NAME system type.
Where NAME is an official system name from the list in the
Assigned Numbers document.
220 Service ready for new user.
221 Service closing control connection.
Logged out if appropriate.
225 Data connection open; no transfer in progress.
226 Closing data connection.
Requested file action successful (for example, file
transfer or file abort).
227 Entering Passive Mode (h1,h2,h3,h4,p1,p2).
230 User logged in, proceed.
250 Requested file action okay, completed.
257 "PATHNAME" created.

331 User name okay, need password.
332 Need account for login.
350 Requested file action pending further information.

421 Service not available, closing control connection.
This may be a reply to any command if the service knows it
must shut down.
425 Cant open data connection.
426 Connection closed; transfer aborted.
450 Requested file action not taken.
File unavailable (e.g., file busy).
451 Requested action aborted: local error in processing.
452 Requested action not taken.
Insufficient storage space in system.

500 Syntax error, command unrecognized.
This may include errors such as command line too long.
501 Syntax error in parameters or arguments.
502 Command not implemented.
503 Bad sequence of commands.
504 Command not implemented for that parameter.
530 Not logged in.
532 Need account for storing files.
550 Requested action not taken.
File unavailable (e.g., file not found, no access).
551 Requested action aborted: page type unknown.
552 Requested file action aborted.
Exceeded storage allocation (for current directory or
dataset).
553 Requested action not taken.
File name not allowed.
))
((
BlockAddr   BlockType   BlockSize
4000:4104      USED     0000:0600
4000:4708      USED     0000:0600
4000:4D0C      USED     0000:0600
4000:5310      USED     0000:0600
4000:5914      USED     0000:002C
4000:5944      USED     0000:0010
4000:5958      USED     0000:0010
4000:596C      USED     0000:0050
4000:59C0      USED     0000:0400
4000:5DC4      USED     0000:0400
4000:61C8      USED     0000:0400
4000:65CC      USED     0000:002C
4000:65FC      USED     0000:0010
4000:6610      USED     0000:0010
4000:6624      USED     0000:0050
4000:6678      USED     0000:002C
4000:66A8      USED     0000:0010
4000:66BC      USED     0000:0010
4000:66D0      USED     0000:0050
4000:6724      USED     0000:002C
4000:6754      USED     0000:0010
4000:6768      USED     0000:0010
4000:677C      USED     0000:0050
4000:67D0      USED     0000:0018
4000:67EC      USED     0000:002C
4000:681C      USED     0000:0010
4000:6830      USED     0000:0010
4000:6844      USED     0000:0050
4000:6898      USED     0000:0454
4000:6CF0      USED     0000:0400
4000:70F4      USED     0000:0B68
4000:7C60      USED     0000:002C
4000:7C90      USED     0000:0010
4000:7CA4      USED     0000:0010
4000:7CB8      USED     0000:0058
4000:7D14      FREE     0000:0154
4000:7E6C      USED     0000:0454
4000:82C4      USED     0000:0400
4000:86C8      USED     0000:0B68
4000:9234      FREE     0000:0054
4000:928C      USED     0000:002C
4000:92BC      USED     0000:0010
4000:92D0      USED     0000:0014
4000:92E8      FREE     0000:02AC
4000:9598      USED     0000:0050
4000:95EC      USED     0000:002C
4000:961C      USED     0000:0010
4000:9630      FREE     0000:0118
4000:974C      USED     0000:0050
4000:97A0      FREE     0000:295C
4000:C100      LAST     0000:0000

11996 bytes free in 5 blocks
20772 bytes used in 45 blocks
32768 bytes total
32768 is SIZEOFHEAP ok
))