\ HTTP.FTH - HTTP server for PowerNet v4.x

((
Copyright (c) 2001-2005, 2006, 2007, 2008, 2009, 2010, 2012, 2013
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


The original code was licensed from:
  CEM Systems Ltd, Belfast, 2000-2002
  Many thanks to Jeremy Moller and George Redpath
Any faults are MPE's responsibility.

Graham Stevenson of Oxford Net Solutions performed
much testing of the server, defined the solution
to the error page issues, and sniffed many packets.

To do
=====
Rewrite INTERPSCRIPT.

Change history
==============
20130913 SFP022 Removed search order change in script interpreter.
		It caused too many problems when the connection is
		closed by the browser.
		Checked for file being served in CLEANHTTP.
20130912 SFP021	Checked release of semaphores when a service is
		placed on the kill chain.
20120306 SFP020 Corrected .HDLINE .
20101208 SFP019 Improved robustness of file script processor.
20101124 SFP018 Made the home page user configurable.
20100526 SFP017 Forced Key/Value reads to use RAM.
20090605 SFP016 Improved bad socket detection.
20090430 SFP015	Added minimal RAM version with one connection.
20081216 SFP014 Added header processing.
		Enhanced scripting support for forms.
		READHEADERS only reads when first called.
		Renamed decodeCGI$ to decodeURL$.
20081215 FVD013 Changed page cache control, added CSS type.
20081208 SFP012 Conversion to file system and line input.
20081202 SFP011 Reduced default number of QVARs to 8.
20081124 SFP010 Added wait for IP address.
20080619 SFP009 Refactored page serving code for easier handling
		of POST requests.
		Replaced FINDPAGE with SEARCHPAGE.
		ASPREQUEST no longer returns status.
		Removed redundant deferred words.
		Removed authentication mechanism.
20080228 SFP008	Removed wait code because of new close-down system.
20080222 SFP007 Added check for listen error.
20070228 SFP006 Refixed QVAR case insensitivity for Flash-based apps.
		CGI string data is now decoded before storage.
20061123 MPE005 Fixed GETQVAR.
		Fixed QVAR case insensitivity for Flash-based apps.
20050901 MPE004 Corrected (SCRIPTENGINE).
20050222 SFP003 Updated for RAM-based systems
20030709 SFP002 Improved type testing.
20030510 GS_001 Revamped error page handler.
))


\ =======
\ *! http
\ =======
\ **************
\ *T HTTP Server
\ **************
\ *P The Powernet HTTP server is a multi-threaded server
\ ** that can accept multiple connections limited only by
\ ** available heap space. For details of the server
\ ** architecture see *\i{Servers.fth}.

\ *P Web pages may be served from memory or from the FAT file
\ ** system. If you are using the FAT file system, you can
\ ** configure the root directory for web pages and the name of
\ ** the home page. If you do not specify them, they will default
\ ** to *\i{\PAGES} and *\i{/home.htm}.
\ *E create pagedir$  ", \HTTP"	\ -- caddr
\ ** \ *G The base directory for pages as a counted string.
\ ** \ ** The directory name must not end in a separator.
\ ** create HomePage$  ", /home.asp"	\ -- caddr
\ ** \ *G Holds the counted string for the home page.


\ *********************
\ *S HTTP specific data
\ *********************
\ *P These data definitions are required by each HTTP task. The data
\ ** is allocated at the start of the task and released when the task is
\ ** *\fo{TERMINATE}d. The chain *\fo{SVCHAIN} links all the service tasks.

\ *P For details of *\fo{QVAR}s see the CGI section below.

(( for DocGen
#80 equ HTTPPort#	\ -- n ; standard is 80
\ *G Define the port used for the HTTP server. The standard
\ ** port is 80. Moved to *\i{PNconfig.fth}.
))

#8 equ #QVARS	\ -- n
\ *G Number of *\fo{QVAR}s in each connection and the common area.
#20 equ /QvarName	\ -- n
\ *G Length including count byte of a *\fo{QVAR}'s name.
#20 equ /QvarData	\ -- n
\ *G Length including count byte of a *\fo{QVAR}'s data area.

struct /QVarRec		\ -- n
\ *G Structure of a single *\fo{QVAR}.
  /QvarName field QV.Name	\ counted string
  /QvarData field QV.Data	\ counted string
end-struct

#QVARS /QVarRec * equ /QVARS	\ -- n
\ *G Size of a QVAR buffer.

struct /HTTPdata	\ -- n
\ *G The standard service data structure is extended for HTTP.
  /SVdata +			\ common beginning with SVdata
  int httpTimeout		\ timeout target
\ int httpLogin			\ login value
  int httpFlags			\ web status flags
  int httpError			\ error status
  int httpScriptId		\ scripting language identifier
  int httpOpFlags		\ operation flags
  int httpClength		\ received HTTP Content Length
  /PageDef field httpFilePage	\ file page control structure ; SFP012
  /QVARS field httpVars		\ HTTP connection variables
end-struct

0 equ NO_SCRIPT		\ -- 0
\ *G No selected script language identifier.
-1 equ FORTH_SCRIPT	\ -- -1
\ *G Selected script language identifier for Forth.

$80000000 equ WEBS_CLOSE	\ -- mask
\ *G Status bit to close the connection.
$40000000 equ WEBS_SCRIPTERR	\ -- mask
\ *G Status bit for a script error.
$00000100 equ WEBS_ASP		\ -- mask
\ *G Status bit for ASP processing.
$00000002 equ WEBS_KEEP_ALIVE
\ *G Status bit for "keep alive" handling.

0 equ PLAIN_TYPE	\ -- 0
\ *G The data type indicator for PLAIN data.
1 equ GIF_TYPE		\ -- 1
\ *G The data type indicator for GIF data.
2 equ HTML_TYPE		\ -- 2
\ *G The data type indicator for HTML data.
3 equ ASP_TYPE		\ -- 3
\ *G The data type indicator for ASP data - HTML data with server
\ ** side scripting. Output *\b{is not} buffered.
4 equ JPEG_TYPE		\ -- 4
\ *G The data type indicator for JPEG data.
5 equ XML_TYPE		\ -- 5
\ *G The data type indicator for XML data.
6 equ ASMX_TYPE		\ -- 6
\ *G The data type indicator for ASMX data in web services - XML
\ ** data with server side scripting.
7 equ ASPX_TYPE		\ -- 7
\ *G The data type indicator for ASP data - XML data with server
\ ** side scripting. Output *\b{is} buffered.
8 equ CSS_TYPE		\ -- 8
\ *G The data type indicator for CSS data.

xc? [if] compiler [then]
: WebErrCode	\ -- addr
\ *G Return the address of the service's error code.
  MySVD httpError
;

: WebStatus	\ -- addr
\ *G Return the address of the service's status flags.
  MySVD httpFlags
;

: WebVars	\ -- addr
\ *G Return the address of the service's connection variables.
  MySVD httpVars
;

: WebScriptId	\ -- addr
\ *G Return the address of the service's script identifier.
  MySVD httpScriptId
;

: WebOpFlags	\ -- addr
\ *G Return the address of the service's operation flags.
\ *D bit0	set if primary headers have been read.
  MySVD httpOpFlags
;
xc? [if] target [then]

: WebPageDef	\ -- struct
\ *G Return the address of the file page definition structure.
\ ** This word is forward referenced.
  MySVD httpFilePage
;

: TestWebFlag	\ mask -- n|0
\ *G Test the mask bits in the service's status flag cell.
  WebStatus @ and
;

: SetWebFlag	\ mask --
\ *G Set the mask bits in the service's status flag cell.
  WebStatus tuck  @ or swap !
;

: ClrWebFlag	\ mask --
\ *G Clear the mask bits in the service's status flag cell.
  invert  WebStatus tuck  @ and swap !
;

: SetHttpTimeout	\ time --
\ *G Set the timeout target for HTTP. A value of 0 indicates
\ ** no timing.
  MySVD httpTimeout !
;

: GetHttpTimeout	\ -- time
\ *G Get the timeout target for HTTP. A value of 0 indicates
\ ** no timing.
  MySVD httpTimeout @
;


\ ====================
\ *S HTTP vectored I/O
\ ====================

\ ================
\ *N Stream socket
\ ================
\ *P The HTTP server establishes its own generic I/O
\ ** based on that in *\i{SERVERS.FTH} in order to handle
\ ** special character processing in the future.

create ConsoleHTTP	\ -- addr ; OUT managed by upper driver
\ *G Function despatch table for HTTP I/O.
\ ** OUT is managed by the upper level driver.
  ' SVkey ,		\ -- char ; receive char
  ' SVkey? ,		\ -- flag ; check receive char
  ' SVemit ,		\ -- char ; display char
  ' SVtype ,		\ caddr len -- ; display string
  ' SVcr ,		\ -- ; display new line

: HTTPio	\ --
\ *G Select HTTP as the console.
  ConsoleHTTP dup opvec ! ipvec !
;

: Init-ConsoleHTTP	\ --
\ *G Initialise for console I/O by HTTP. Note that the
\ ** HTTP must have been set up and and the private
\ ** service area initialised.
  Init-ConsoleSV
;


\ ============================
\ *N Output to a memory buffer
\ ============================
\ *P For correct handling of error responses, some browsers require
\ ** the HTTP "Content-Length" field to be defined. This means that
\ ** the output length must be known before sending. Consequently,
\ ** error messages are buffered before transmission.

\ *P N.B. All output to the buffer is unchecked for overflow.
\ ** Checking is the responsibility of the application.

\ Move to USERVARS.FTH!
cell +user my_opbuff	\ -- addr
\ *G Holds the current output buffer, the first cell holding
\ ** the buffer length.

: BuffEmit	\ char --
\ *G Send a character to the buffer.
  my_opbuff @				\ -- char buff
  dup @ 				\ -- char buff len
  2dup 1+ swap !			\ update count
  + cell + c!				\ stash character
;

: BuffType	\ caddr len --
\ *G Send a string to the buffer.
  tuck					\ save length
  my_opbuff @ dup @ + cell +		\ address to store to
  swap cmove				\ save string
  my_opbuff @ +!			\ update length
;

: BuffCr	\ --
\ *G Send a CR/LF pair to the buffer.
  #13 BuffEmit  #10 BuffEmit
;

create ConsoleBuff	\ -- addr ; OUT managed by upper driver
\ *G Function despatch table for HTTP buffered I/O.
\ ** OUT is managed by the upper level driver.
  ' SVkey ,		\ -- char ; receive char
  ' SVkey? ,		\ -- flag ; check receive char
  ' BuffEmit ,		\ -- char ; display char
  ' BuffType ,		\ caddr len -- ; display string
  ' BuffCr ,		\ -- ; display new line

: Buff$		\ -- addr len
\ *G Return the address and length of the HTTP output buffer.
  my_opbuff @ dup @  swap cell+ swap
;

(( unused
: BuffIo	\ --
 \ *G Select HTTP buffered output as the console.
  ConsoleBuff dup opvec ! ipvec !
;
))

: Init-ConsoleBuff	\ len -- addr|0
\ *G Initialise for console I/O by the HTTP output buffer.
\ ** *\i{Len} is the required size of the buffer and the address
\ ** of the buffer is returned for success, or zero is returned
\ ** if the buffer could not be allocated. The first cell of
\ ** the buffer contains the length used, the rest is for data.
  allocate
  if  drop 0  else  dup off  endif	\ prep buffer
  dup my_opbuff !			\ address or 0
;

: Term-ConsoleBuff	\ --
\ *G Terminate buffer I/O by freeing the buffer.
\  my_opbuff @ ?free  0 my_opbuff !
  my_opbuff @ free drop  0 my_opbuff !
;


\ *********************
\ *S Diagnostic control
\ *********************

1 value httpDiags?	\ -- n
\ *G Set this non-zero to get diagnostic information.

XC? [if]
compiler
: [hd	httpDiags? if consoleio decimal  ;
\ *G A *\fo{COMPILER} macro used to surround debug code, and
\ ** terminated by *\fo{HD]}.
\ *E   [HD  ." debug message"  HD]
: hd]	HttpIo  endif  ;
\ *G Terminates a *\fo{[HD ... HD]} structure.
target
[else]
: [hd		\ --
  postpone httpDiags?  postpone if  postpone consoleio  postpone decimal
; immediate
: hd]
   postpone HttpIo   postpone endif
; immediate
[then]

: .hdLine	\ caddr len --
\ *G Display text with leading CR on Forth console. If
\ ** *\fo{httpDiags?} is set to zero, no action is taken.
  httpDiags? if
    consoleio decimal
    cr ." Socket " my_hsocket @ . ." :"  type  ." :"
    HttpIo
  else
    2drop
  endif
;


\ *********************
\ *S Transmit Utilities
\ *********************

PDATA_MAX equ WEB_SIZE	\ -- n
\ *G Maximum web content sent in one packet.

: WebSend	\ caddr len --
\ *G Send an arbitrary sized data block to the HTTP client.
  0 max
  begin		\ -- caddr len
    dup 0<>  WebErrCode @ 0= and
   while
    dup WEB_SIZE min >r
    over my_hsocket @ swap r> 0 send
    dup SOCKET_ERROR =
    if  SOCKET_ERROR WebErrCode !  endif
    0 max /string
    pause
  repeat
  2drop
;


\ **************
\ *S CGI Support
\ **************
\ *P Common Gateway Interface (CGI) defines how data is passed
\ ** between a browser (client) and a server. The equivalent of
\ ** a variable is a name/value pair (e.g. submit=send). Such
\ ** pairs are sent by the browser after filling in a form.

\ *P CGI variables are "URLencoded" as key/value pairs of the
\ ** form:
\ *C   key1=value1&key2=value2&...

\ *P When a form response is sent with a GET message, the CGI
\ ** variables are sent after the URI separated from it by a
\ ** '?' character. When a form response is sent with a POST
\ ** message, the CGI variables appear in the body of the
\ ** message.

\ *P When a GET message is used, PowerNet receives the CGI
\ ** variables before it knows what to do with them. They are
\ ** saved in structures called *\fo{QVAR}s, which hold counted
\ ** strings. Numeric values are converted from text by the
\ ** appropriate Forth words. Name comparision is case-insensitive.

\ *P PowerNet manages two sets of these variables. The first is
\ ** a common set, which may be used to hold system data such as the
\ ** host's IP address. The second set is allocated per connection
\ ** as part of the service data, and only exists for the duration
\ ** of the connection.


(( repeated documentation
#20 equ /QvarName	\ -- n
\ *G Length including count byte of a *\fo{QVAR}'s name.
#20 equ /QvarData	\ -- n
\ *G Length including count byte of a *\fo{QVAR}'s data area.

struct /QVarRec		\ -- n
\ *G Structure of a single *\fo{QVAR}.
  /QvarName field QV.Name
  /QvarData field QV.Data
end-struct

#32 equ #QVARS	\ -- n
\ *G Number of *\fo{QVAR}s in each connection and the common area.
#QVARS /QVarRec * equ /QVARS	\ -- n
\ *G Size of a *\fo{QVAR} buffer.
))

/QVARS buffer: commonQvars	\ -- addr
\ *G The buffer area for the common *\fo{QVAR}s.

$80000000 equ NO_VAR_SET	\ -- n
\ *G Indicator returned when a *\fo{QVAR} has not been set.

/QvarName 1 chars - equ /QV.name	\ -- n
\ *G Maximum length of a *\fo{QVAR}'s name.
/QvarData 1 chars - equ /QV.data	\ -- n
\ *G Maximum length of a *\fo{QVAR}'s string data.

: (.Qvars)	\ addr --
\ *G Display the variables in the given table.
  /QVARS bounds ?do
    i QV.Name @ if
      cr i QV.Name $. ." =" i QV.Data $.
    endif
  /QVarRec +loop
;

: .Qvars		\ --
\ *G Display the common and connection variables.
  SVchain
  begin
    @ dup
   while
    dup SVsk# @ get_socket_addr
    so_service @ service_http = if
      cr ." Connection on socket " dup SVsk# @ .
      dup httpvars (.Qvars)
    endif
    pause
  repeat
  drop

  cr ." Common:"
  commonQvars (.Qvars)
;

: (freeqvar)	\ table -- addr|0
\ *G Find a free *\fo{QVAR} in the given table, and return its address
\ ** or zero if no free space is available.
  0
  swap /QVARS bounds ?do
    i QV.Name c@ 0= if
      drop i  leave
    endif
  /QVarRec +loop
;

: freeQvar	\ -- addr|0
\ *G Find a free *\fo{QVAR} for the current connection, and return its address
\ ** or zero if no free space is available.
  WebVars (freeQvar)
  dup 0= if
     ConsoleIO
     ." ERROR: Too many QVARs !"	\ var table full
     HTTPio
  endif
;

: freeCommonQvar	\ -- addr|0
\ *G Find a free *\fo{QVAR} in the common *\fo{QVAR}s, and return its address
\ ** or zero if no free space is available.
  CommonQVars (freeQvar)
;

: (findQvar)	\ caddr len table -- addr|0
\ *G Try to find a *\fo{QVAR} in the given table. Case insensitive.
  rot 0 swap 2swap			\ -- 0 caddr len table
  /qvars bounds do			\ -- 0 caddr len
    dup i QV.name c@ = if
      over i QV.name count is= if
        rot drop  i -rot  leave
      endif
    endif
  /QVarRec +loop
  2drop
;

: findQvar	\ caddr len -- addr|0
\ *G Try to find the given *\fo{QVAR} name, returning the address
\ ** if found or zero if not found. Case insensitive.
  /QV.name min				\ trim length
  my_socket @ if
    my_socket @ so_service @ service_http = if
      2dup WebVars (findqvar) ?dup if
        nip nip  exit
      endif
    endif
  endif
  CommonQvars (findqvar)
;

: %xx>char	\ caddr len -- caddr' len' char
\ *G Convert the three character sequence "%xy" as a hexadecimal
\ ** two-digit number. Step over the string.
  over >r  3 /string  r> 1+ 2 string>hex
;

: decodeURL$	\ caddr len dest dlen --
\ *G Converts the source string *\i{caddr/len} from a URL-encoded
\ ** string to a decoded counted string in the buffer *\i{dest/dlen}.
\ ** No error checking is performed.
\ ** Decoding converts '+' characters to a space and
\ ** "%ab" ('%' follwed by two hex digits) sequences to their
\ ** character codes.
  over >r  over + >r  1+		\ -- caddr len next ; R: -- start end
  begin
    over 0 >				\ while chars left in source
   while
    dup r@ u<				\ while not at end
   while
    >r
    case  over c@
      [char] + of  1 /string  bl  endof
      [char] % of  %xx>char  endof
        -rot 1 /string rot
    end-case
    r> tuck c!  1+
  repeat then
  nip nip r> drop r>			\ -- next start
  tuck - 1- swap c!			\ stash count
;

: setQvarData	\ caddr len qvar --
\ *G Use the given URL encoded string *\i{caddr/len) to set the
\ ** data area of the given *\i{qvar}.
  QV.data /QVarData decodeURL$
;

: setQstring	\ name nlen string slen --
\ *G Set the connection *\fo{QVAR} *\i{name/nlen} to contain
\ ** *\i{string/slen}. If the name already exists in the
\ ** connection or common *\fo{QVAR}s it is overwritten. If the
\ ** *\fo{QVAR} does not exist it is created in the connection's
\ ** *\fo{QVAR} set. If there is no space for it, the request is
\ ** ignored.
  2swap					\ -- string slen name namelen
  2dup findQvar dup if			\ already exists?
    nip nip				\   discard name
    setQvarData				\   set string
  else					\ -- string slen' name namelen 0
    drop freeqvar dup if		\ -- string slen' name namelen addr|0
      dup >r  QV.name place		\   set name
      r> setQvarData			\   set string
    else
      drop 2drop 2drop
    endif
  endif
;

: setCommonQstring	\ name nlen string slen --
\ *G Set the common *\fo{QVAR} *\i{name/nlen} to contain *\i{string/slen}.
\ ** If the *\fo{QVAR} does not exist it is created. If there is no
\ ** space for it, the request is ignored.
  2swap					\ -- string slen name namelen
  2dup CommonQVars (findQvar) dup if	\ already exists?
    nip nip				\   discard name
    setQvarData				\   set string
  else
    drop  CommonQvars (freeQvar) dup if	\ free slot?
      dup >r  QV.name place		\   set name
      r> setQvarData			\   set string
    else
      drop 2drop 2drop			\   throw all away
    endif
  endif
;

: GetQstring	\ name nlen -- caddr len
\ *G Return the text for a string. If the variable cannot be found,
\ ** "???" is returned.
  findQvar ?dup 0= if
    s" ???"
  else
    QV.data count
  endif
;

: .Qstring	\ name nlen --
\ *G Output the text for a *\fo{QVAR} using *\fo{GetQstring} above.
  GetQstring type
;

: websetvars	\ caddr len --
\ *G Process a query string. A query string has the form:
\ *C name=value&name=value...
\ *P *\fo{WEBSETVARS} can be used with any query string.
\ [hd cr ." WebSetVars :" 2dup type ." :"  hd]
  begin
    dup
   while
    [char] = split  [char] & split 2>r	\ -- name nlen data dlen ; R: -- rem remlen
\ [hd  cr ." Name:" 2over type ." :  Value:" 2dup type ." :"  hd]
    setQstring
    2r>
  repeat
  2drop
\ [hd  .Qvars  cr ." WebSetVars done"  hd]
;

: WebQueryVars	\ caddr len --
\ *G This is used by GET with query packets.
  [char] ? jump-string dup
  if  1 /string  endif
  websetvars
;

: init-CommonQvars	\ --
\ *G Initialise the common *\fo{QVAR}s.
  commonQvars /QVARS erase
;

: init-WebVars	\ --
\ *G Initialise the connection *\fo{QVAR}s.
  WebVars /QVARS erase
;


\ ================
\ *N Numeric QVARS
\ ================

: setQvar	\ caddr len n --
\ *G Set the given *\fo{QVAR} to *\i{n}, which is held as a
\ ** signed decimal string.
  dec>string setQstring
;

: getQvar	\ caddr len -- n
\ *G Return the value held in the *\fo{QVAR} as a signed decimal
\ ** number. If the *\fo{QVAR} does not exist *\fo{NO_VAR_SET} is
\ ** returned. If the string cannot be converted, zero is returned.
  findQvar ?dup 0= if			\ MPE005
    NO_VAR_SET
  else
    QV.data count string>dec
  endif
;

((
\ *P CEM specific numeric items are commented out.

: QvarString?	\ caddr len -- type
\ *G Return true if the variable is a string variable.
  drop  c@ upc [char] S =
;

: setqvar	\ name nlen value --
  -rot					\ -- value name nlen
  2dup findQvar ?dup if			\ already exists ?
    nip nip				\  discard name
    QV.data !				\  set new value
  else
    freeqvar ?dup if			\ room?
      dup >r  QV.name place		\  set name
      r> QV.data !			\  set value
    else
      2drop drop			\  quietly ignore
    endif
  endif
;

: setCommonQvar	\ name nlen value --
  -rot					\ -- value name nlen
  2dup findQvar dup if			\ already exists ?
    nip nip				\  discard name
    QV.data !				\  set new value
  else
    drop freeCommonQvar dup if		\ room?
      dup >r  QV.name place		\  set name
      r> QV.data !			\  set value
    else
      drop 2drop drop			\  quietly ignore
    endif
  endif
;

: getdata	\ cname -- cdata|NO_VAR_SET
  count findQvar ?dup 0= if
    NO_VAR_SET
  else
    QV.data
  endif
;

: getvar	\ cname -- value/*cname
  dup getdata
  swap count qvarstring? 0=		\ not a string
  over NO_VAR_SET <> and 		\ and valid
  if  @	 endif
;

: setvar	\ value cname --
  count rot setqvar
;
))


\ **************
\ *S ASP Support
\ **************
\ *P ASP stands for "Active Server Pages". In PowerNet, these are
\ ** HTML pages which are modified by PowerNet when served. See
\ ** *\i{Examples\PowerNet\TestPages\thanks.asp} for an example.

\ *P The script language is Forth itself. Inside an HTML document,
\ ** scripting commands (Forth source) are contained inside tags
\ ** of the form:
\ *C  <% Forth_code %>
\ *P It is important that the script delimiters *\fo{<%} and *\fo{%>} are
\ ** surrounded by white space, otherwise the very simple parser
\ ** will fail. Before any scripting can be performed, the first
\ ** command must be on one line:
\ *C   <% language=forthscript %>
\ *P After that, Forth source code can be interpreted. Note that
\ ** any CGI variables (QVARS above) are available. For example,
\ ** if a form was submitted with a GET request:
\ *C   GET form1.asp?sname=Robert&send=submit
\ *P You can display the data using a script such as:
\ *C   <% s" sname" .qstring %>

\ *P Pages can be served from a linear memory image, or from the
\ ** FAT file system. When scripts are served from memory, each
\ ** script section *\b{must} be on a single line. When serving
\ ** scripts from files, the script section can extend over
\ ** several lines.

: script_code	\ -- caddr len
\ *G Returns the command to select Forth as the scripting language.
  s" language=forthscript"
;

: asp_header$	\ -- caddr len
\ *G ASP command header string.
  s" <%"
;

: asp_tail$	\ -- caddr len
\ *G ASP command tail string.
  s" %>"
;

MemPages? [if]

: ScriptEngine	\ caddr len --
\ *G Processes the string as Forth source. The data stack is
\ ** checked on return to ensure system integrity.
  0 -rot  sp@ >r			\ -- guard caddr len ; R: -- sp
  ['] evaluate catch if			\ THROW occurred
    WEBS_SCRIPTERR SetWebFlag		\ mark in status
    2drop drop  r> drop			\ clean data stack ; MPE004
  else					\ no THROW
    r> sp!  2drop drop			\ force data stack
  endif
  SVflushOP pause
;

: AspProcess	\ caddr len --
\ *G If Forth has been selected as the script language,
\ ** pass the string to *\fo{SCRIPTENGINE}, otherwise try to
\ ** find the Forth script command. In practice, this
\ ** means that the language selection command must be
\ ** on its own as the first script section.
\ [hd  cr ." AspProcess:" 2dup type ." :"  hd]
  WebScriptId @ NO_SCRIPT = if
    script_code search
    if	FORTH_SCRIPT WebScriptId !  endif
    2drop
  else
    scriptengine			\ try to process script
  endif
;

: AspRequest	\ caddr len --
\ *G Extract script section and try to process it. All output is
\ ** done using *\fo{TYPE} and *\fo{WebSend}.
\ [hd  cr ." AspRequest on socket " my_hsocket @ .  my_socket @ .dword  dvars  hd]
  NO_SCRIPT WebScriptId !
  begin
    2dup asp_header$ search		\ -- caddr len *s slen t/f
    WebErrCode @ 0= and
   while				\ -- caddr len *s slen
    swap >r >r				\ -- caddr len ; R: -- *s slen
    r@ - WebSend			\ send out first part
    r> r> swap 2 /string		\ step over '<%' ; -- *s slen
    2dup asp_tail$ search 0= if		\ -- *s *slen *r rlen
\     [hd  ." ASP Frame Error "  hd]
      2drop 2drop  exit
    endif
    swap >r >r				\ -- *s slen ; R: -- *r rlen
    r@ - AspProcess			\ process ASP string
    r> r> swap 2 /string		\ step over tail
  repeat
  2swap 2drop				\ discard original string
  dup
  if  WebSend  else  2drop  endif	\ now output the rest
  cr cr					\ blank line
;

[then]


\ ******************
\ *S Header scanning
\ ******************
\ *P HTTP headers are processed by building a list of actions and
\ ** strings. The action is an xt and the string is theheader
\ ** text that we are interested in. Each action has the stack
\ ** effect
\ *C   caddr len --
\ *P where *\i{caddr/len} is the string after the header and the
\ ** trailing colon.
\ *E create UploadHdrs	\ -- addr
\ **   ' doCLength ,  ," Content-Length" align
\ **   ' doCType1 ,   ," Content-Type" align
\ **   0 , 0 ,


: CheckName	\ src slen name nlen -- flag
\ *G Return true if the start of the string *\i{src/slen} contains
\ ** the entire name *\i{name/nlen}.
  rot  2dup >
  if  2drop 2drop false exit  endif
  min is=
;

: CheckHeader	\ src slen name nlen -- flag
\ *G Return true if the start of the string *\i{src/slen} contains
\ ** the entire name *\i{name/nlen} and a trailing semi-colon.
  2over 2>r
  CheckName 0=
  if  2r> 2drop 0 exit  endif
  2r> [char] : scan nip
;

: doHeader	\ caddr len list --
\ *G Given a string, check it against the given list of headers.
  begin
    dup @				\ check for end of list
   while
    3dup cell+ count CheckHeader if
      >r  [char] : scan dup		\ strip off :
      if  1 /string  endif
      r> @ execute  exit
    endif
    cell + count + aligned		\ step over xt and string
  repeat
  3drop
;

: ProcessHeaders	\ list --
\ *G We have already received the GET/POST line. Process each header
\ ** line and finish at the first blank line, ready for the message
\ ** body. This can be used for the header blocks in forms
\ ** as well as the for the first header.
  >r  begin
    SVquery  #tib @			\ not a blank line
\ [io consoleio  cr source type  io]
   while
    source r@ doHeader
  repeat
  r> drop
;

: doCLength	\ caddr len --
\ *G Process the string to extract the content length data,
\ ** a decimal number. A valid results is saved in the HTTP
\ ** service data's *\fo{httpClength} field.
  base @ >r  decimal
  -leading -trailing
  case  (integer?)
    1 of  MySVD httpClength !  endof
    2 of  2drop  endof
  endcase
  r> base !
;

variable FormType	\ -- addr
\ *G Holds the status extracted from the first *\fo{Content-Type}
\ ** header. This set is true if the header includes
\ ** "multipart/form-data".

#80 buffer: Boundary$	\ -- addr
\ *G Holds the boundary string.

: nextChar	\ caddr len -- caddr' len' char
\ *G Get the next character from the string and step on.
  over c@ -rot  1 /string  rot
;

: ?MultiPart	\ caddr len --
\ *G Check for the *\i{multipart/form-data} field.
  s" multipart/form-data" isearch FormType !  2drop
;

: ExtractValue	\ caddr len dest --
\ *G Given a string starting after the name portion of a name/value
\ ** pair, e.g. *\i{name="text"}, save the value text without
\ ** any quote marks at *\i{dest}. If there is no value text,
\ ** the destination is left unchanged.
  -rot  NextChar [char] = <>
  if  2drop drop  exit  endif
  -leading				\ now we're at the start
  over c@ [char] " =			\ quote or space delimited?
  if 1 /string [char] " else bl endif
  left-string rot place
;

: ?Boundary	\ caddr len --
\ *G Check for the *\i{boundary=xxx} field.
  s" boundary" dup >r isearch if	\ look for boundary string
\ [io consoleio cr ." Found boundary" io]
    r> /string -leading -trailing
   Boundary$ ExtractValue
  else
    2drop  r> drop
  endif
;

: doCType1	\ caddr len --
\ *G Process the string to extract the content type data,
\ ** which is the multipart form item and the boundary string.
  2dup ?MultiPart ?Boundary
;

create BaseHdrs	\ -- addr
\ *G Contains a list of the basic header fields to process.
  ' doCLength ,  ," Content-Length" align
  ' doCType1 ,   ," Content-Type" align
  0 , 0 ,

BaseHdrs value DefHdrs	\ -- addr
\ *G Holds the default header list for file and memory pages.

: ReadHeaders	\ --
\ *G Process the headers after the HTTP command line up to the
\ ** first blank line. *\b{Do not} use this word for reading part
\ ** headers. If the response headers have already been read,
\ ** this word is a no-op.
  WebOpFlags @ bit0 and 0= if
    -1 MySVD httpClength !		\ no headers yet
    0 Boundary$ c!  FormType off
    DefHdrs ProcessHeaders		\ read basic set
    bit0 WebOpFlags or!			\ mark primary headers read
  endif
;


\ ***********************
\ *S Form body processing
\ ***********************
\ *P Words are provided for use with form results submitted
\ ** as a POST request to an ASP page. You can use these with
\ ** simple URL encoded or multipart forms. An example of
\ ** multipart form handling is the file upload application
\ ** in *\i{Examples\WebPost.fth}. This example includes boundary
\ ** handling and parsing code.

\ ========
\ *N Tools
\ ========
\ *P Correct operation of this code requires a Content-Length
\ ** header in the request. In many cases, the body of a POST
\ ** request is a very long line. Because this server is designed
\ ** for systems with limited RAM, key/value pairs are *\fo{not}
\ ** read as a single line, but are read and parsed character by
\ ** character. Keys are limited to 31 characters, and values
\ ** to *\fo{/SVtib - 32} characters. The service's *\fo{TIB}
\ ** is where they are buffered.

: BodyLeft	\ -- addr
\ *G Returns the address holding the remaining size of the HTTP
\ ** body. If no Content-Length field has been found, *\fo{ReadHeaders}
\ ** will have set this to -1.
  MySVD httpClength
;

: -BodyLeft	\ n --
\ *G Reduce the remaining content by *\i{n}.
  BodyLeft @ swap - 0 max BodyLeft !
;

\ sname=pohh&smail=sfp%40mpeforth.com&state=happy&send=Submit

compiler
: CGIname$	\ -- caddr
\ *G Buffer for name assembly.
  SVtib  ;				\ SFP017
: CGIval$	\ -- caddr
\ *G Buffer for data assembly.
  SVtib #32 +  ;			\ SFP017
target

: CGIend?	\ -- flag
\ *G Return true if the body content is exhausted.
  BodyLeft @ 0 max 0=
;

: CGIkey	\ -- char
\ *G Read the next character. If the content is exhausted or there
\ ** has been a socket error, LF is returned.
  CGIend? if  $0A exit  endif
  SVkey  1 -BodyLeft
;

\ cell buffer: foxed
: ReadKey	\ -- ior
\ *G Read the key portion of a key/value pair, returning zero
\ ** for success.
  0 CGIname$ ( dup foxed ! ) c!  begin
    case CGIkey
      $0A of  -1 exit  endof
      [char] = of  0 exit  endof
        CGIname$ addchar  0
    endcase
    CGIend?
  until
  -2
;

: ReadValue	\ -- ior
\ *G Read the value portion of a key/value pair, returning zero
\ ** for success.
  0 CGIval$ c!  begin
    case CGIkey
      $0A of  -1 exit  endof
      [char] & of  0 exit  endof
        CGIval$ addchar  0
    endcase
    CGIend?
  until
  0
;


\ ====================
\ *N Application words
\ ====================
\ *P These words will mostly be used inside ASP scripts.
\ ** Note that the key and value strings are not URL decoded.
\ ** You can use *\fo{decodeURL$} to perform the decode and
\ ** copy in one operation.

\ *P It is assumed that you control both the RAM usage of the
\ ** PowerNet server and that you control the scripts it serves.
\ ** We are not trying to reimplement the Apache server!

\ *P To read the pairs, call *\fo{ReadNextPair} and inspect
\ ** the return result. If it is good, examine and process
\ ** the *\fo{PairName} and *\fo{PairValue} strings. Because
\ ** you control the scripts, it is sensible to use key names
\ ** that do not require URL decoding. Especially for user input,
\ ** some key values may need decoding, e.g. email addresses.
\ ** This decision is left up to you.

: ReadNextPair	\ -- ior
\ *G Read the next key/value pair and return 0 on success.
\ ** -1 is returned for an unexpected end of input or socket
\ ** error, and -2 is returned if input was truncated before
\ ** the key terminator.
  ReadKey ?dup if exit endif
  ReadValue
  pause
;

: PairName	\ -- caddr len
\ *G Return the last key/value pair's key text.
  CGIname$ count
;

: PairValue	\ -- caddr len
\ *G Return the last key/value pair's value text.
  CGIval$ count
;

: DumpPairs	\ --
\ *G A diagnostic tool for use in scripts. Read and dump all
\ ** the key/value pairs without preserving them.
  begin
    ReadNextPair 0=
   while
    ." Key=" PairName type ."  Value=" PairValue type ." <br>"
  repeat
;

((
: HTTPquery	\ -- ; fetch line into TIB
 \ *G Read a line of text from the connection into the input buffer.
 \ ** The line size is limited by the *\fo{httpClength} field,
 \ ** which is reduced by the size of the line read.
  'tib @ /SVtib 2-			\ buffer
  BodyLeft @ -1 <>			\ clip to remaining content size
  if  BodyLeft @ umin  endif
  SVaccept dup #tib !  >in off		\ read line
  2+ -Bodyleft				\ allow for CR/LF, reduce
;
))



\ *****************************
\ *S HTTP headers and responses
\ *****************************
\ *P This section deals with extracting the HTTP command data
\ ** and with pages served from memory.

: web-file-name	\ caddr len -- caddr len'
\ *G Return the string up to the next '?' character. Then, if
\ ** the file name starts "HTTP://.../" step over "HTTP://...".
\ ** This situation can occur with proxy servers.
  [char] ? left-string
  dup 7 > if
    over dup s" HTTP://" s=
    swap s" http://" s=
    or if
      7 /string  [char] / jump-string
    endif
  endif
;

: test-web-type	\ caddr len -- type
\ *G Return the type code associated with the file extension,
\ ** e.g ".htm" or ".asp".
  [char] . scan if
    case
      dup s" .htm" s=
      ?of  drop HTML_TYPE  endof
      dup s" .gif" s=
      ?of  drop GIF_TYPE  endof
      dup s" .jpg" s=
      ?of  drop JPEG_TYPE  endof
      dup s" .asp" s=
      ?of  drop ASP_TYPE  WEBS_ASP SetWebFlag  endof
      dup s" .css" s=			\ FVD013
      ?of  drop CSS_TYPE  endof
        drop  PLAIN_TYPE
    end-case
  else
    drop  PLAIN_TYPE
  endif
;

internal

: .HTTP#	\ n --
\ *G Output the HTTP string and code line.
  ." HTTP/1.1 "  . cr
;

: .HTTPserver	\ --
\ *G Output the HTTP server name line.
  ." Server: " .PNET cr
;

: .HTTPdate	\ --
\ *G Output an HTTP date line.
  ." Date: "  0 .AnsiDate cr
;

: .HTTPar-none	\ --
\ *G Output the "accept-ranges: none" line.
  ." Accept-ranges: none " cr
;

: .HTTPcontent	\ type --
\ *G Output the text string for the curent type. Unknown types
\ ** generate "text/plain".
  ." Content-type: "
  case
    GIF_TYPE of ." image/gif" endof
    HTML_TYPE of ." text/html" endof
    ASP_TYPE of ." text/html" endof
    JPEG_TYPE of ." image/jpeg" endof
    CSS_TYPE of ." text/css" endof	\ FVD013
      ." text/plain"
  endcase
  cr
;

external

: .close/keep	\ --
\ *G Output the default connection type.
[ SVsingle? ] [if]
  ." Connection: Close " cr
[else]
  ." Connection: Keep-Alive " cr
[then]
;

: WebResponse	\ datalen type code --
\ *G Output a suitable HTTP header for this type of data.
  base @ >r decimal
  .HTTP# .HTTPserver .HTTPdate
  .HTTPar-none
  dup .HTTPcontent
  ASP_TYPE = if			\ -- len
    ." Connection: Close " cr
    ." Pragma: no-cache" cr
    ." Cache-control: no-cache" cr
    drop
  else
    .close/keep				\ SFP015
    ." Content-length: " . cr
    ." Cache-control: public, max-age=86400" cr	\  FVD013
  endif					\ -- type
  cr					\ blank line marks end of header
  r> base !
  SVflushOP				\ force output
;

[undefined] Null$ [if]
create Null$	\ -- addr
\ *G A null string which may be used as a counted
\ ** string or a zero-terminated string.
  0 ,
[then]

: .ErrHead	\ len err# --
\ *G Output the error header. If len is non-zero, it is
\ ** used for a "Content-Length" field.
  dup .HTTP# .HTTPserver .HTTPdate
  .HTTPar-none  HTML_TYPE .HTTPContent
  .close/keep				\ SFP015
  swap ?dup
  if  ." Content-Length: " . cr  endif
  #401 = if
    ." WWW-Authenticate: Basic realm=PowerNet " cr
  endif
  cr					\ separate header and body
  SVflushOP				\ flush this out
;

: .ErrBody	\ caddr1 len1 caddr2 len2 err# --
\ *G Output a web error body using the given strings
\ ** and error number. The string caddr2/len2 contains the
\ ** error description, e.g. "Page not found". If len2 is zero
\ ** no error text body is sent and caddr1/len1 is discarded.
\ ** The string caddr1/len1 is displayed if len1 is non-zero,
\ ** and is usually the resource name that caused the problem.
  over if
    ." <HTML><head><title>PowerNet Error: </title></head>" cr
    ." <body><h2>Error: " . ." </h2>" cr
    ." <P>" type  ." </P>" cr
    dup if
      ." <h2>Resource: </h2>" cr
      ." <P>" type  ." </P>" cr
    else
      2drop
    endif
    ." </body></HTML>" cr
  else
    2drop 2drop drop
  endif
  cr cr					\ blank line
;

: weberror	\ caddr1 len1 caddr2 len2 err# --
\ *G Display a web error message using the given strings
\ ** and error number. The string caddr2/len2 contains the
\ ** error description, e.g. "Page not found". If len2 is zero
\ ** no error text body is sent and caddr1/len1 is discarded.
\ ** The string caddr1/len1 is displayed if len1 is non-zero,
\ ** and is usually the resource name that caused the problem.
\ [hd  cr ." Sending Error# " dup . 2 pick 2 pick type  hd]
  base @ >r decimal
  $400 Init-ConsoleBuff if		\ typical required < #400 bytes
    dup >r				\ save err#
    opvec @ >r  ConsoleBuff opvec !	\ place body in buffer
    .ErrBody				\ generate buffer data
    r> opvec !				\ switch o/p back to socket
    Buff$				\ -- caddr len ; body in buffer
    dup r> .ErrHead wait-socket-empty	\ send head as one packet
    WebSend wait-socket-empty		\ send body as second packet
    Term-ConsoleBuff			\ release buffer
  else
    drop 2drop 2drop			\ abandon response
  endif
  r> base !
;

[undefined] HomePage$ [if]		\ SFP018
create HomePage$  ", /home.htm"
\ *G If not already defined, *\fo{HomePage$} is set to contain
\ ** the counted string *\i{/home.htm}.
[then]

: CheckHomePage	\ caddr len -- caddr' len'
\ *G Check for a home page string of the forms "/ " or "/" and
\ ** if found replace with "/home.htm".
  2dup -trailing s" /" str=			\ home page checks
  if  2drop  HomePage$ count  endif
;

MemPages? [if]
: ServeSMem	\ caddr len type --
\ *G Serve a page from memory, according to its page type
\ ** extracted from the page name.
  ReadHeaders				\ read HTTP headers
  2dup #200 WebResponse  ASP_TYPE =	\ success response, check type
  if  AspRequest  else  WebSend  endif
;
[then]


\ ****************
\ *S Serving files
\ ****************

FilePages? [if]

#256 equ /FileUnit	\ -- len
\ *G The size of a part of a file processed at once.

: FileQuery	\ -- ; fetch line into TIB
\ *G Reset the input source specification to the console and
\ ** accept a line of text into the input buffer.
\  0 to source-id                        \ mark source as keyboard
  'tib @ /SVtib 2- FileAccept #tib !	\ "tib-len 1-" was #80
  >in off
;

\ We need a custom version of INTERPRET here that checks for %> as
\ we cannot rely on tools/users to enforce a trailing space.

\ SFP022...
-short-branches
: AspInterpret	\ --
\ *G Process the current input line as if it is text entered at
\ ** the keyboard.
[defined] FlushCache [if]
  FlushCache
[then]
  begin
    ?stack  bl word dup c@
  while					\ -- addr
    dup count asp_tail$ str=		\ -- addr flag
    if  drop  r> drop exit  then	\ exit from INTERPSCRIPT.
    find ?dup if
      0< state @ and			\ flag<>-1 for IMMEDIATE words
      if  compile,  else  execute  then
    else
      number? ?dup ?undef
      state @
      if  #literal  else  drop  then
    then
  repeat
  drop
;
+short-branches
\ ...SFP022

: InterpScript	\ --
\ *G Interpret a section of ForthScript which may extend over
\ ** several lines.
  begin
    AspInterpret			\ may cause an exit ; SFP022
    FileQuery  FileErr? FileEOF? or
  until
;

: sendPrev	\ addr --
\ *G Send the source line before this address.
  source drop  tuck - dup
  if  type  else  2drop  endif
;

semaphore InterpSem	\ -- addr
\ *G Exclusive access semaphore for the Forth interpreter.
  1 InterpSem !

: FileScript	\ --
\ *G Read and process a file with server side scripting until EOF
\ ** or an error. The file is already open and is not closed.
  begin
    FileQuery
    FileErr? 0=
   while
    FileEOF? 0=
   while
    source asp_header$ search if	\ -- *s slen
      over sendPrev			\ send previous part of line
      WebScriptId @ NO_SCRIPT = if
        script_code search
        if  FORTH_SCRIPT WebScriptId !  endif
        asp_tail$ search 0= if
          WEBS_SCRIPTERR SetWebFlag	\ mark error in status
          2drop exit
        endif
        2 /string type cr		\ transmit tail
      else				\ -- *s slen
        InterpSem request
        nip 2-  #tib @ swap - >in !	\ interpreter starts after '<%'
        ['] InterpScript catch drop	\ SFP019 ; SFP022
        source >in @ /string type	\ transmit tail
        InterpSem signal
      endif
    else
      type cr
    endif
  repeat then
;

: AspFileReq	\ len --
\ *G Read and process a file of length *\i{len} with server side
\ ** scripting. The file is already open. All input is done using
\ ** the *\fo{FileCon} Generic I/O device (see *\i{FATcore.fth}.
\ ** All output is done using *\fo{TYPE} and *\fo{WebSend}.
\ [io  consoleio  cr ." AspFileReq on socket " my_hsocket @ .  my_socket @ .dword  .qvars  io]
  drop					\ discard file length
  ipvec @ >r  source-id >r		\ preserve source info
  MyFID w@ to source-id			\ new Forth source
  FileCon ipvec !			\ set file input
  NO_SCRIPT WebScriptId !		\ no script marker yet
  ['] FileScript catch drop
  r> to source-id  r> ipvec !		\ restore source info
;

: badFS		\ x y z --
\ *G Discard three items and set error flags.
  3drop  WEBS_CLOSE SetWebFlag  WEBS_KEEP_ALIVE ClrWebFlag
;

: (FileSend)	\ len buff --
\ *G Serve the file using the given buffer.
  swap					\ -- buff len
  begin
   dup
  while
   2dup
   /FileUnit min MyFID w@ read-file	\ -- buff len len' flag
   if  badFS exit  endif
   dup 0=
   if  badFS exit  endif
   dup >r  - 0 max  over r> WebSend
   pause
  repeat
  2drop
;

: FileSend	\ len --
\ *G Serve a plain file. without scripting. The file is open and
\ ** the details are in the given *\fo{/PageDef} structure.
  /FileUnit allocate if			\ -- struct buff
    2drop
  else
    dup >r (FileSend) r> free drop
  endif
;

: ServeFile	\ len type --
\ *G Serve a page from a file, according to its page type
\ ** extracted from the page name. The request headers are read
\ ** if they have not already been read.
  ReadHeaders				\ read the HTTP command headers
  2dup #200 WebResponse  ASP_TYPE =	\ success response, check type
  if  AspFileReq  else  FileSend  endif
  SVflushOP  CloseFileCon		\ flush buffers, close file
;

[then]


\ ********************
\ *S HTTP service task
\ ********************

: ?CloseHTTP	\ --
\ *G Run after processing input to see if the connection
\ ** should be closed.
[ SVsingle? ] [if]
  WEBS_CLOSE SetWebFlag
[else]
  WEBS_KEEP_ALIVE TestWebFlag
  WEBS_ASP TestWebFlag 0= and if
    HTTP_KEEPALIVE_TIME later SetHttpTimeout
  else
    WEBS_CLOSE SetWebFlag
  endif
[then]
;

: Serve404	\ qaddr qlen --
\ *G Serve a "not found" page.
  [hd  cr ." Not found: " 2dup web-file-name type  hd]
  s" Page not found " #404 weberror
;

: ServePage	\ qaddr qlen --
\ *G Serve a page defined by the string *\i{qaddr/qlen} which
\ ** may include CGI vars. The string is the command line with
\ ** the command removed.
  WEBS_KEEP_ALIVE SetWebFlag		\ Assume a "keep-alive" connection
  2dup web-file-name CheckHomePage	\ -- qaddr qlen fname flen
  2dup test-web-type >r			\ determine content type from name
  SearchPage				\ -- qaddr qlen struct|0 type|0 ; R: -- ptype
  case
     NoPage of  drop  Serve404  endof
[ FilePages? ] [if]
     FilePage of  nip nip  PG.len @ r@ ServeFile  endof
[then]
[ MemPages? ] [if]
     SMemPage of  nip nip  PG.len 2@ r@ ServeSMem  endof
[then]
     xtPage of  PG.addr @ execute  endof
       2drop s" Method not implemented" #501 weberror 0
  endcase
  r> drop
;

: http-cmd	\ caddr len -- caddr' len'
\ *G Deal with the command line after the command has been
\ ** recognised. The input is the complete line. The output
\ ** is the line without the command.
  2dup .hdLine				\ show GET line

((
  2dup WebAuthenticate ?dup 0= if
    2drop  Null$ 0 s" Invalid login " #401 weberror  exit
  endif
  WebLogin !				\ stash good login result
))
  BL jump-string			\ skip the command
  BL extract-string			\ extract filename and query string
  WebOpFlags off			\ start of transaction.
;

: http-get	\ caddr len --
\ *G Process a GET command. The input string is the
\ ** complete input line containing the GET command.
  http-cmd  2dup WebQueryVars  ServePage
;

semaphore PostSem	\ -- addr
\ *G Controls access to the single POST handler. POST requests
\ ** are serialised because they may change the state of the
\ ** system.
  1 PostSem !				\ exclusive access

: http-post	\ caddr len --
\ *G Process a POST command. The input string is the
\ ** complete input line containing the POST command.
  PostSem request  http-cmd ServePage  PostSem signal
;

: ParseHTTP	\ caddr len --
\ *G Process received HTTP command line. At present we only deal
\ ** with GET and POST commands.
  WebOpFlags off			\ start of transaction.
  dup 4 <				\ basic sanity check ; SFP021
  if  2drop exit  endif
  over s" GET " s=
  if  http-get  exit  endif
  over s" POST " s=
  if  http-post  exit  endif
  2drop
;

: doHTTPinput	\ --
\ *G Process any pending input.
  SVkey? if				\ if anything to do
    HTTP_KEEPALIVE_TIME later 1 or SetHttpTimeout
    echoing off  xon/xoff off  out off
    SVquery  CheckSV if
      WEBS_CLOSE SetWebFlag
      [hd  s" bad socket close" .SVmessage  hd]
    else
      source ParseHTTP  ?CloseHTTP
      WebErrCode @ if
        WEBS_CLOSE SetWebFlag
	[hd  s" closing on error" .SVmessage  hd]
      endif
    endif
  endif
;

\ SFP021...
: cleanHTTP	\ *sv -- *sv
\ *G Cleans up the HTTP system when a task is shut down from the
\ ** kill chain.
[ FilePages? ] [if]
  dup SVtask @ MyFID his @		\ holds handle of file being served ; SFP022
  dup -1 <>				\ -1 is closed
  if  $FFFF and close-file  then
  drop
[then]
[defined] InterpSem [if]
  InterpSem @ 0= if			\ locked?
    dup SVtask @			\ task that owns the service
    InterpSem cell + @ =		\ is task owning the semaphore?
    if  0 1 InterpSem 2!  endif		\ yes, reset it
  endif
[then]
[defined] PostSem [if]
  PostSem @ 0= if			\ locked?
    dup SVtask @			\ task that owns the service
    PostSem cell + @ =			\ is task owning the semaphore?
    if  0 1 PostSem 2!  endif		\ yes, reset it
  endif
[then]
;
\ ...SFP021

: HTTPService	\ --
\ *G The HTTP service action or task launched for each established
\ ** HTTP connection.
  SVstartup
  Init-ConsoleHTTP  HTTPio
[ FilePages? ] [if]
  -1 MyFID !				\ mark no file handle ; SFP022
[then]
  ['] cleanHTTP MySVD SVappClean !	\ set shutdown action ; SFP021
  HTTP_KEEPALIVE_TIME later SetHttpTimeout
  0 WebErrCode !  -1 ClrWebFlag  Init-WebVars

  begin
[ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
    pause
    checkSV 0=				\ check for socket failure ; SFP016
   while
    doHTTPinput				\ check for input line
    GetHttpTimeout ?dup if
      timedout?
      if   WEBS_CLOSE SetWebFlag  endif
    endif
    SVdone?  WEBS_CLOSE TestWebFlag or
  until then

  SVshutdown
;


\ **********************
\ *S HTTP listening task
\ **********************
\ *P Listening tasks or actions are spawned when the HTTP server
\ ** gets a connection.

: HTTPServer	\ -- ; stay here forever
\ *G The HTTP listening task.
  s" HTTP" SRVRstartup
  init-CommonQvars
  begin
[ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
    service_http /HTTPdata HTTPport# ServiceCreate
    my_hsocket @ >r			\ preserve listening socket#
    begin
      r@ isMySocket
      my_hsocket @ 0 0 Saccept		\ wait for connection or failure
      dup SOCKET_ERROR <>
     while
[ SVsingle? ] [if]
      isMySocket  HTTPService
[else]
      ['] HTTPService StartService	\ launch another HTTP service
[then]
      #10 ms
    repeat
    drop  r> closeSocket drop
    pause
  again
;

0 value HTTPtask	\ -- 0|task
\ *G Returns 0 or the HTTP server task if running.

: RunHTTPtask	\ --
\ *G Start the HTTP server task.
  HTTPtask 0= if
    ['] HTTPserver SVinitiate to HTTPtask
  endif
;

: StopHTTPtask	\ --
\ *G Stop the HTTP server.
  HTTPtask if
    HTTPtask SVterminate  0 -> HTTPtask
    cr ." HTTP server stopped"
  endif
;


\ ************************
\ *S Notes on memory usage
\ ************************
\ *P The majority of page requests are made using a GET request.
\ ** For these, only the first line of the header needs to be
\ ** scanned and the request is contained in a single packet
\ ** whose maximum size is defined in lower layers of the
\ ** Powernet system. Consequently a fixed size packet
\ ** buffer is used for HTTP input.

\ *P When handling POST requests, e.g. for Web Services, the
\ ** input data is not in the headers, but is contained in the
\ ** body of the message. The size of this body is defined
\ ** by the *\i{Content-Length} header. When handling POST
\ ** messages, the whole of the incoming message must be
\ ** read, the body extracted and passed to the message
\ ** handler. In addition, some handlers may need to process
\ ** the message header. For example, Web Services need
\ ** to use the *\i{SOAPaction} header before SOAP version 1.2.

\ *P Scripting for output messages introduces the problem that
\ ** the size of the message body is not (in general) known until
\ ** the script output has been generated. This means that the
\ ** *\i{Content-Length} header (sent before the body) cannot
\ ** be formed until the body has been generated.

\ *P To avoid the memory overhead of buffering script output, for
\ ** ASP file requests no *\i{Content-Length}
\ ** header is generated and the connection is closed after the
\ ** response has been sent to indicate that the message is
\ ** complete. For web services, ASPX pages are served and
\ ** are buffered, and a *\i{Content-Length} header is generated.

\ *P Error messages such as "404 Not found" are always buffered
\ ** to produce a valid *\i{Content-Length} header because some
\ ** browsers require this header for error messages.

\ *P The *\i{Content-Length} header is not always required for
\ ** HTTP version 1.1 and above. However, if HTTP 1.0 clients
\ ** have to be supported the *\i{Content-Length} header must be
\ ** provided. In this case all scripted operations must be
\ ** buffered. See RFC2616 for more detils.


\ *****************
\ *S Authentication
\ *****************
\ *P Before PowerNet v4.4, authentication was provided by a
\ ** deferred word. Nobody reported using it, so it has been
\ ** removed to save memory. This section shows how to put it
\ ** back if required. All the code to be added can be found
\ ** towards the end of *\i{HTTP.FTH}.

\ *P More flexibility is provided in v4.4 because it is much
\ ** easier to parse headers (see *\i{Examples\WebPost.fth}),
\ ** and scripting provides more choice as to which files are
\ ** secured and which are public.

0 [if]

\ *P Add the line below to the *\fo{/HTTPdata} structure.
\ *C   int httpLogin                    \ login value

\ *P Add the following definitions.
: WebLogin	\ -- addr
\ *G Return the address of the service's login result.
  mySVD httpLogin
;

defer WebAuthenticate	\ caddr len -- res|0
\ *G Given a GET string, returns a non-zero code for permission
\ ** to carry on. If permission is refused, zero is returned.

: (WebAuthenticate)	\ caddr len -- res|0
\ *G The default action of *\fo{WebAuthenticate} always returns true.
  2drop true
;
assign (WebAuthenticate) to-do WebAuthenticate

\ *P Restore the following code to *\fo{HTTP-CMD}.
\ *[
  2dup WebAuthenticate ?dup 0= if
    2drop  Null$ 0 s" Invalid login " #401 weberror  exit
  endif
  WebLogin !				\ stash good login result
\ *]

[then]

0 [if]

\ === authorization levels === \

: authLevel ( -- level )	\ 0 = none, 1 = user, 3 = sys
  WebOpFlags @ 4 rshift 3 and
;

0 equ AuthNone#
1 equ AuthUser#
3 equ AuthSystem#

create UserPasswd  16 allot

: doAuthorization	\ caddr len --
  2dup .hdline
  -leading -trailing
  2dup S" Basic " CheckName 0= if 2drop exit endif
  6 /string -leading
  2dup .hdline
  2dup CfgBuff EbPassword$ + count istr= if
    $30 WebOpFlags or!
  endif
  2dup UserPasswd count istr= if
    $10 WebOpFlags or!
  endif
  2drop
;

create BaseHdrs	\ -- addr
\ *G Contains a list of the basic header fields to process.
  ' doCLength ,  ," Content-Length" align
  ' doCType1 ,   ," Content-Type" align
  ' doAuthorization ,	," Authorization" align
  0 , 0 ,

[then]


\ ======
\ *> ###
\ ======

decimal

