\ KimaServer.fth - Kima TCP Server.

((
Copyright (c) 2008, 2009, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20120214 SFP004 Converted from Modbus server.
20090528 SFP003 Changed server and service for new LISTEN and SACCEPT.
20081203 MGS002 Bug fixes and improved error checking.
20081124 SFP001	Added wait for IP address.
))

decimal


\ =========
\ *! kimaserver
\ =========
\ ****************
\ *T Kima Server
\ ****************
\ *P The Powernet KIMA server is a multithreaded server
\ ** that can accept multiple connections.
\ ** The model used is described in *\i{Servers.fth}.

\ *P Treat this code as an Alpha test template for your own
\ ** extensions.


\ ***********************
\ *S KIMA specific data
\ ***********************
\ *P These data definitions are required by each server task. The data
\ ** is allocated at the start of the task and released when the task is
\ ** *\fo{TERMINATE}d. The chain *\fo{SVCHAIN} links all the service tasks.

7 equ KimaPort#	\ -- n ; standard is 502
\ *G Define the port used for the Kima server. The standard
\ ** Echo port is 7.

struct /KimaData	\ -- len
\ *G The  server uses a standard service data structure.
\ *[
  /SVdata +			\ reuse existing service structure
end-struct
\ *]


\ *********************
\ *S Kima service tasks
\ *********************

: (sendKimaData)	\ --
  cr ." Data every second" cr
;

defer sendKimaData	\ --
\ *G A hook filled in by the application. Executed everyn}
\ ** includes the MBAP header, the data and the two-byte CRC.
  assign (sendKimaData) to-do sendKimaData

: FlushSkin	\ --
\ *G Flush input from the service socket.
  begin
    CheckSV 0=				\ MGS002
   while
    key?
   while
    key drop
  repeat then
;

: KimaService	\ --
\ *G The Kima service task launched for each established
\ ** Kima connection.
  SVstartup
  Init-ConsoleSV Console=SV		\ we'll use KEY, EMIT and friends
  #100 ms

  begin
 [ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
    sendKimaData
    1000 ms
    FlushSkin
    pause  CheckSV
  until

  SVshutdown
;


\ ************************
\ *S Kima listening task
\ ************************

: KimaServer	\ -- ; stay here forever
\ *G The Kima listening task.
  ConsoleIO decimal			\ default debug I/O device
  WaitForIPaddress			\ SFP007
  cr ." Kima server started" cr
  begin
[ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
    service_modbus /KimaData KimaPort# ServiceCreate
    my_hsocket @ >r			\ preserve listening socket#
    begin
      r@ isMySocket
      my_hsocket @ 0 0 Saccept		\ wait for connection or failure
      dup SOCKET_ERROR <>
     while
[ SVsingle? ] [if]
      isMySocket  KimaService
[else]
      ['] KimaService StartService	\ launch another service
[then]
    repeat
    drop  r> isMySocket			\ set listening socket
    my_hsocket @ closeSocket drop	\ close after error
    pause
  again
;

0 value KimaTask	\ -- 0|task
\ *G Returns 0 or the Kima server task if running.

: RunKima	\ --
\ *G Start the Kima server task.
  KimaTask 0= if
    ['] KimaServer SVinitiate to KimaTask
  endif
;

: StopKima	\ --
\ *G Stop the Kima server.
  KimaTask if
    KimaTask SVterminate  0 -> KimaTask
    cr ." Kima server stopped"
  endif
;


\ **************
\ *S Diagnostics
\ **************
\ *P This code is only compiled if the *\fo{EQU}ate *\fo{DIAGS?}
\ ** is set non-zero.

diags? [if]


[then]


\ ******
\ *> ###
\ ******

