\ MBusSrvr.fth - ModBus TCP Server.

((
Copyright (c) 2008
Flygt AB

and

Copyright (c) 2008, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

To do
=====

Change history
==============
20090528 SFP003 Changed server and service for new LISTEN and SACCEPT.
20081203 MGS002 Bug fixes and improved error checking.
20081124 SFP001	Added wait for IP address.
))

decimal


\ =========
\ *! modbus
\ =========
\ ****************
\ *T ModBus Server
\ ****************
\ *P The Powernet ModBus server is a multithreaded server
\ ** that can accept multiple connections.
\ ** The model used is described in *\i{Servers.fth}.

\ *P Treat this code as a Beta test template for your own
\ ** extension.


\ ***********************
\ *S ModBus specific data
\ ***********************
\ *P These data definitions are required by each server task. The data
\ ** is allocated at the start of the task and released when the task is
\ ** *\fo{TERMINATE}d. The chain *\fo{SVCHAIN} links all the service tasks.

#502 equ ModBusPort#	\ -- n ; standard is 502
\ *G Define the port used for the Modbus server. The standard
\ ** port is 502.

struct /ModBusData	\ -- len
\ *G The ModBus server uses a standard service data structure
\ ** with an addition buffer for incoming ADPUs.
\ *[
  /SVdata +			\ reuse existing service structure
  #300 field APDUin		\ incoming APDU
end-struct
\ *]


\ ***********************
\ *S ModBus service tasks
\ ***********************

defer doModBusIn	\ apduin len -- apduout len'
\ *G A hook filled in by the application. Given an incomping APDU,
\ ** process it and return an outgoing APDU which will be sent.
\ ** *\i{Apduin/out} point to the start of the MBAP header. */i{Len}
\ ** includes the MBAP header, the data and the two-byte CRC.
  assign noop to-do doModBusIn

struct /MBAP	\ -- len
\ *G ModBus MBAP header.
  2 field ma.tid		\ transaction identifier
  2 field ma.prot		\ protocol, 0=mbus
  2 field ma.len		\ following data length in bytes
  1 field ma.uin		\ unit identifier
end-struct

: MyAPDUin	\ -- addr
\ *G Returns the address of the received APDU.
  MySVD APDUin
;

Semaphore ModBusSem
\ *G Serialises Modbus access if there are multiple connections.
  1 ModBusSem !

: ReadMBAP	\ -- flag ; 0 = good
\ *G Read the MBAP header. Return 0 on success.
  MyAPDUin dup /MBAP bounds		\ Read MBAP heaper
  do  key i c!  loop
  ma.prot w@(n) 0<>			\ check ModBus
;

: ReadPDU	\ -- flag ; 0 = good
\ *G Read the PDU. Return 0 on success.
  MyAPDUin dup /MBAP +			\ start of PDU
  swap ma.len w@(n) bounds		\ length of PDU
  ?do  key i c!  loop
  CheckSV 0=				\ MGS002
;

: FlushMBin	\ --
\ *G Flush input from the Modbus socket.
  begin
    CheckSV 0=				\ MGS002
   while
    key?
   while
    key drop
  repeat then
;

: ProcessAPDU	\ --
\ *G Read, process and return an APDU transaction.
  ReadMBAP if  FlushMBin exit  endif
  ReadPDU if  FlushMBin exit  endif
  MyAPDUin dup ma.len w@(n) /MBAP + 2+  doModBusIn  type
  FlushMBin
;

: ModBusService	\ --
\ *G The ModBus service task launched for each established
\ ** ModBus connection.
  SVstartup
  Init-ConsoleSV Console=SV
  #100 ms

  begin
 [ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
   key? if
      ModBusSem request
      ProcessAPDU
      ModBusSem signal
    endif
    pause  CheckSV
  until

  SVshutdown
;


\ ************************
\ *S Modbus listening task
\ ************************

: ModBusServer	\ -- ; stay here forever
\ *G The ModBus listening task.
  ConsoleIO decimal			\ default debug I/O device
  WaitForIPaddress			\ SFP007
  cr ." ModBus server started" cr
  begin
[ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
    service_modbus /ModBusData ModBusPort# ServiceCreate
    my_hsocket @ >r			\ preserve listening socket#
    begin
      r@ isMySocket
      my_hsocket @ 0 0 Saccept		\ wait for connection or failure
      dup SOCKET_ERROR <>
     while
[ SVsingle? ] [if]
      isMySocket  ModbusService
[else]
      ['] ModbusService StartService	\ launch another service
[then]
    repeat
    drop  r> isMySocket			\ set listening socket
    my_hsocket @ closeSocket drop	\ close after error
    pause
  again
;

0 value ModBusTask	\ -- 0|task
\ *G Returns 0 or the Modbus server task if running.

: RunModBus	\ --
\ *G Start the ModBus server task.
  ModBusTask 0= if
    ['] ModBusServer SVinitiate to ModBusTask
  endif
;

: StopModBus	\ --
\ *G Stop the ModBus server.
  ModBusTask if
    ModBusTask SVterminate  0 -> ModBusTask
    cr ." ModBus server stopped"
  endif
;


\ **************
\ *S Diagnostics
\ **************
\ *P This code is only compiled if the *\fo{EQU}ate *\fo{DIAGS?}
\ ** is set non-zero.

diags? [if]


[then]


\ ******
\ *> ###
\ ******

