\ Pages.fth - Page tools for PowerNet v4.x

((
Copyright (c) 2001-2003, 2005, 2007, 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20081208 SFP004 Added file pages.
20080619 SFP003 Recoded for v4.4.
20071107 SFP002 Updated for VFX host use.
20050222 SFP001 Updated for RAM-based systems.
))


\ ========
\ *! pages
\ *T Web page handling
\ ========
\ *P Examples are given here for pages in memory and pages in
\ ** files. Feel free to experiment with mixed operation, but
\ ** note that the code in *\i{<PNet>\Services\Pages.fth} is
\ ** maintained by MPE and may/will change with future releases,
\ ** so we suggest that you keep changes in a separate file.


\ ****************
\ *S Configuration
\ ****************
\ *P The following two equates are only used if they are not
\ ** already defined.

[undefined] MemPages? [if]
1 equ MemPages?	\ -- n
\ *G Set this non-zero to compile the memory pages code.
[then]
[undefined] FilePages? [if]
1 equ FilePages?	\ -- n
\ *G Set this non-zero to compile the file pages code.
[then]


\ ******************
\ *S Data structures
\ ******************
\ *P Since Web pages may be stored in a file system, other mass
\ ** storage or memory, a common data structure is used based
\ ** on the needs of pages stored in CPU memory. Pages stored
\ ** in CPU memory are identified by name in the *\fo{HTTPPAGES.VOC}
\ ** vocabulary. Executing the word returns the address of the
\ ** data structure and its type.

struct /PageDef	\ -- n
\ *G The structure used for a page definition.
  int PG.type			\ page type, see below
  int PG.len			\ page length, PG.ADDR must follow
  int PG.addr			\ page address, xt or handle
end-struct

\ *P The following equates define the types of pages that are
\ ** available. They are stored in the *\fo{PG.TYPE} field of
\ ** a *\fo{/PageDef} structure.

0 equ NoPage	\ -- 0
\ *G Page not found or unusable.
1 equ FilePage	\ -- n
\ *G The page is in a file.
2 equ SMemPage	\ -- n
\ *G The page is in memory (e.g. Flash) and need not be released.
3 equ xtPage	\ -- n
\ *G The page is created by executing a Forth word, whose xt is in
\ ** the *\fo{PG.addr} field.
4 equ HMemPage	\ -- n
\ *G The page has been loaded into the heap.


vocabulary HTTPpages.voc	\ --
\ *G The default vocabulary used to hold page data.

[undefined] voc>wid [if]
: voc>wid       \ xt(voc) -- wid
\ *G Return the WID of a vocabulary whose XT is supplied.
\ ** This definition is implementation dependent.
  >body @
;
[then]

: FindVocPage	\ caddr len -- struct|0 type|0
\ *G Find a page from its name *\i{caddr/len}, returning the data
\ ** structure address *\i{struct} and a type code *\i{type}. If
\ ** the page cannot be found, both return values are zero. The
\ ** action is to look up the name in the *\fo{HTTPpages.voc}
\ ** vocabulary and extract the page type from the *\fo{/PAGEDEF}
\ ** structure.
  { | nb[ #32 ] -- }			\ SFP001
  #31 min  nb[ place  nb[ count		\ isolate name ; SFP001
  2dup upper				\ word names are upper case.
  ['] HTTPpages.voc voc>wid search-wordlist
  if  execute  else  0 0  endif
;


\ *******************
\ *S Executable pages
\ *******************

xc? [if] interpreter [then]
: xtPage:	\ xt "<name> -- ; -- struct type
\ *G Creates a page which later executes the given action, e.g.
\ *C   ' doNewApp xtPage: /NewApp.asp
\ *P The specified action must have the stack effect
\ *C   qaddr qlen --
\ *P where the input is the rest of the line after GET or POST.
  also HTTPpages.voc definitions
  create				\ Forthname entry
    xtPage ,  0 ,  ,
    previous definitions
  does>
   dup PG.type @
;
xc? [if] target [then]


\ ***************
\ *S Memory pages
\ ***************

MemPages? [if]

xc? [if] interpreter [then]
: MemPage:	\ "<Forthname>" "<filename>" -- ; -- struct type
\ *G An *\fo{INTERPRETER} definition that creates *\fo{<Forthname>} in the
\ ** *\fo{HTTPPAGES.VOC} vocabulary and a *\fo{/PAGEDEF} structure, and then
\ ** loads file *\i{<filename>} into the dictionary as data. At run-time
\ ** the address of the *\fo{/PAGEDEF} structure is returned. When the
\ ** HTTP server is running it will serve the data in response
\ ** to a GET request for Forthname.
\ *E MemPage: /home.htm %IPSTACK%\powernet.htm
  also HTTPpages.voc definitions
  create				\ Forthname entry
    here  SMemPage ,			\ PG.type
    0 ,  dup /PageDef + ,		\ dummy PG.len, PG.addr
    data-file swap PG.len !		\ load file, set length
    previous definitions
  does>
   dup PG.type @
;
xc? [if] target [then]


\ ***********************
\ *N Example memory pages
\ ***********************
\ *P These files are compiled by default unless the *\fo{equ}ate
\ ** *\fo{TestPages?} exists and is set to zero.

[defined] TestPages?
[if]  TestPages?  [else]  1  [then]
[if]
MemPage: /home.htm %IPSTACK%\TestPages\home.htm
\ *G Creates embedded memory page *\i{/HOME.HTM} from the data in
\ ** *\i{%IPSTACK%\home.htm}. The leading '/' is required as
\ ** it is not removed by the command parser.
MemPage: /bgimage.jpg %IPSTACK%\TestPages\bgimage.jpg
\ *G The background image for *\i{POWERNET.HTM}.
MemPage: /favicon.ico %IPSTACK%\TestPages\favicon.ico
\ *G The page icon for these pages.
MemPage: /contact.htm %IPSTACK%\TestPages\contact.htm
\ *G Creates embedded memory page *\i{/CONTACT.HTM} from the data
\ ** in *\i{%IPSTACK%\contact.htm}.
MemPage: /form1.htm %IPSTACK%\TestPages\form1.htm
\ *G A simple form.
MemPage: /thanks.htm %IPSTACK%\TestPages\thanks.htm
\ *G The response to *\i{FORM1.HTM}.
MemPage: /form2.htm %IPSTACK%\TestPages\form2.htm
\ *G A simple form with ASP and GET handling.
MemPage: /form3.htm %IPSTACK%\TestPages\form3.htm
\ *G A simple form with ASP and POST handling.
MemPage: /thanks.asp %IPSTACK%\TestPages\thanks.asp
\ *G The scripted response to *\i{FORM2.HTM}.
MemPage: /thanks3.asp %IPSTACK%\TestPages\thanks3.asp
\ *G The scripted response to *\i{FORM3.HTM}.
align
[then]

[then]	\ MemPages?


\ *************
\ *S File pages
\ *************
\ *P The defaults for this section assume that the FAT file system
\ ** in in use.

FilePages? [if]

[undefined] pagedir$ [if]
create PageDir$	\ -- addr
\ *G The base directory for pages. This will be prepended to
\ ** the page name given to *\fo{SearchPage}. The string is held
\ ** as a counted string in the buffer. Make sure that your
\ ** application sets *\fo{PageDir$} to point to its own pages.
\ ** The directory name must not end in a separator. The version
\ ** here sets the contents to *\i{\PAGES} and is only used if
\ ** *\fo{PageDir$} is undefined here.
  ", \PAGES" align
[then]

: dirSep?	\ char -- flag
  dup [char] / =  swap [char] \ = or
;

: PrepFilename	\ caddr len --
\ *G Prepare a URL to be a file name. For Windows/DOS file systems,
\ ** convert '/' characters to '\'. For Unices, do nothing.
  bounds ?do
    i c@ dirSep?
    if  [char] \ i c!  endif
  loop
;

[undefined] [IO [if]
: [io		\ -- ; R: -- ipvec opvec
\ *G Save the current I/O devices on the return stack.
  r>  ipvec @ >r opvec @ >r  >r
;

: io]		\ -- ; R: ipvec opvec --
\ *G Restore the I/O devices from the return stack.
  r>  r> opvec ! r> ipvec !  >r
;
[then]

: FindFilePage	\ caddr len -- struct|0 type|0
\ *G The action of *\fo{SearchPage} for pages in files.
\ ** Find a page into memory from its name *\i{caddr/len},
\ ** returning the structure address and length.
  MAX_PATH allocate
  if  drop 2drop  0 0 exit  endif
  >r
  pagedir$ count r@ place		\ base directory
  over c@ dirSep? 0=			\ if file name does not start with a separator
  if  [char] \ r@ addchar  endif	\ add separator
  r@ append				\ add file name
  r@ count 2dup PrepFilename		\ apply OS requirements
  r/o open-file  r> free drop		\ open file
  if  drop 0 0  exit  endif
  dup MyFID w!  0 MyFID 2 + w!		\ set handle for file input
  WebPageDef				\ -- handle struct
  2dup PG.addr !			\ save handle
  swap file-size 2drop  over PG.len !	\ -- struct ; save file size
  FilePage  2dup swap PG.type !		\ set page type
((
[con
cr ." struct=" WebPageDef .dword
cr ." Handle=" WebPageDef PG.addr @ .dword
cr ." Len=   " WebPageDef PG.len @ .dword
cr ." Type=  " WebPageDef PG.type @ .dword
20 ms  io]
))
;


((
: RelFilePage	\ struct --
 \ *G Free off the resources for a file page
  PG.addr @ close-file drop
;
))

[then]


\ ***************
\ *S Page look up
\ ***************

FilePages? [if]
: SearchPage	\ caddr len -- struct|0 type|0
\ *G Find a page from its name *\i{caddr/len}, returning the data
\ ** structure address *\i{struct} and a type code *\i{type}. If
\ ** the page cannot be found, both return values are zero. The
\ ** action is to look up the name in the *\fo{HTTPpages.voc}
\ ** vocabulary and extract the page type from the *\fo{/PAGEDEF}
\ ** structure. If file pages are required, the page directory
\ ** is searched.
  2dup FindVocPage dup			\ look for page in HTTPpages.voc
  if  2swap 2drop exit  endif
  2drop
  FindFilePage				\ look for page on disk
;
[else]
: SearchPage	\ caddr len -- struct|0 type|0
  FindVocPage  ;
[then]


\ ======
\ *> ###
\ ======

decimal

