\ TFTP Receiver Code

((
Copyright (c) 1997-2003
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====
Refactor, Document, Test and Release.

Change history
==============
20030313 SFP001 Moved task definition from build file.
))

\ ==========
\ *! tftp
\ *T TFTP receiver
\ ==========
\ *P The TFTP server receives files from a remote client.
\ ** Only one connection at a time is supported. See
\ ** SERVICES\TFTP.FTH.

\ *P In order to use the TFTP server, you must define the
\ ** actions required of the TFTP events.

\ *P N.B This is alpha test code.

decimal


\ **************
\ *S Ident Block
\ **************

create $MODULE	\ -- addr
\ *G Module name string.
  ", TFTP Server"
create $VERSION	\ -- addr
\ *G Version string.
  ", 3.00"
create $COPYRIGHT	\ -- addr
\ *G Copyright string
  ", Microprocessor Engineering Limited"


\ **************
\ *S Global data
\ **************

0 VALUE TFTP_Socket	\ -- n
\ *G Socket used by TFTP server.

#512 VALUE TFTP_BLOCKSIZE	\ -- n
\ *G TFTP data block size.

TFTP_BLOCKSIZE 4 + VALUE TFTP_MAXPACKETSIZE	\ -- n
\ *G TFTP data packet size.

0 VALUE *TFTP_PACKET	\ -- addr
\ *G Current TFTP packet address.

0 VALUE TFTP_PACKETSIZE	\ -- n
\ *G Current TFTP packet data block size.

0 VALUE blockid	\ -- n
\ *G If blockid is non-zero  then we are connected.

SOCKADDR_IN buffer: sai[	\ -- addr
\ When connected this describes our client.

STATE_IDLE VALUE TFTPState	\ -- n
\ *G Current State Machine state ID.


\ *****************************
\ *S TFTP State Machine equates
\ *****************************

$000 constant STATE_IDLE                \ IDLE
$010 constant STATE_STARTRX             \ A *load-to-server* has started
$011 constant STATE_ACKDATA             \ acknowledge client data
$012 constant STATE_RXDATA              \ receive data from client
$013 constant STATE_FINISHEDRX          \ finished upload
$020 constant STATE_STARTTX             \ A *load-to-client* has started
$021 constant STATE_TXDATA              \ transmit data to client
$022 constant STATE_WAITFORACK          \ wait for client acknowledge
$023 constant STATE_FINISHEDTX          \ finished download

$001 constant TFTP_RRQ                  \ Read ReQuest
$002 constant TFTP_WRQ                  \ Write ReQuest
$003 constant TFTP_DATA                 \ DATA packet
$004 constant TFTP_ACK                  \ ACKnowledge
$005 constant TFTP_ERROR                \ ERROR condition


\ ***************
\ *S Event action place-holders and defaults
\ ***************
\ *P The reciver uses these three DEFERred words to
\ ** handle incoming TFTP events.

defer TEVENT::BeginReceive	\ c-addr u -- okay?
\ *G Process a STARTRX (upload to server) request.
defer TEVENT::RxData		\ addr len --
\ *G Process a receive data request. The parameters
\ ** describe the TFTP data block.
defer TEVENT::FinishedRx	\ --
\ Process a FINISHEDRX request.

:NONAME		\ c-addr u -- okay?
\ *G Default to handle the string that comes with the STARTRX
\ ** (upload to server) request. Just display the string
\ ** and return true to carry on.
  cr ." TEVENT::BeginReceive: " type
  TRUE
; to-do TEVENT::BeginReceive

:NONAME		\ addr len --
\ *G Default to process a received data block. Just
\ ** show the address and length.
  cr ." TEVENT::RxData: " . ." bytes at $" .dword
; to-do TEVENT::RxData

:NONAME		\ --
\ Default to process a FINISHEDRX request.
  cr ." TEVENT::FinishedRx"
; to-do TEVENT::FinishedRx


\ ****************
\ *S Utility Words
\ ****************

: DiscardTFTP   \ --
\ *G Discard the current TFTP packet.
  *TFTP_PACKET ?free  0 to *TFTP_PACKET
;

: AbortTFTP     \ c-addr u --
\ *G Display the TFTP error message and discard the current packet.
  cr ." TFTP Error:" type
  DiscardTFTP  STATE_IDLE to TFTPState
;

: zcount        \ zaddr -- zaddr len
\ *G A version of COUNT for zero terminated strings, returning
\ ** the address of the first character and the length.
  dup 
  begin
    dup c@ 0<>
   while
     1+
  repeat
  over -
;


\ **********************
\ *S TFTP State Handlers
\ **********************

: smIDLE        { | namelen -- }
\ *G The TFTP system is idle. Perform a lot of error
\ ** checks and report them if the equate DIAGS? is true.
  0 to blockid

  SOCKADDR_IN -> namelen
  sai[ SOCKADDR_IN erase

  *TFTP_PACKET if
[ diags? ] [if]
    cr ." WARNING: smIDLE Entered with an allocated TFTP_Packet"
[then]
    DiscardTFTP
  then

  TFTP_MAXPACKETSIZE allocate 0= if
    to *TFTP_PACKET

    TFTP_Socket *TFTP_PACKET TFTP_MAXPACKETSIZE
    0 sai[ addr namelen recvfrom -> TFTP_PACKETSIZE

    TFTP_PACKETSIZE SOCKET_ERROR = if
        cr ." ERROR: TFTP Packet Received was too big!"
        DiscardTFTP
        exit
    then

    TFTP_Socket sai[ SOCKADDR_IN connect
    SOCKET_ERROR = if
        cr ." ERROR: Failed connect to remote host."
        DiscardTFTP
        exit
    then

    *TFTP_PACKET w@(n)
    case
      TFTP_WRQ of  STATE_STARTRX to TFTPState  endof
      TFTP_RRQ of  STATE_STARTTX to TFTPState  endof
        cr ." WARNING: Unrecognized TFTP Start frame"
        DiscardTFTP
    endcase

  else

    drop S" Failed to allocate start buffer from smIDLE" AbortTFTP

  then
;

: smSTARTRX	\ --
\ *G Handles start of receiving a file.
  *TFTP_PACKET 2+ zcount TEVENT::BeginReceive if
    DiscardTFTP
    STATE_ACKDATA to TFTPState
  else
    S" Refused Connection for RX" AbortTFTP
  then
;

: smSTARTTX     \ --
\ *G Handles start of a (refused) transmission request.
  cr ." TFTPState: StartTx"
  *TFTP_PACKET 2+ TFTP_PACKETSIZE 2 - DUMP
  DiscardTFTP
  smIDLE to TFTPState
;

: smFINISHEDRX  \ --
\ *G Handles the end of file reception.
  TEVENT::FinishedRX
  STATE_IDLE to TFTPState
;

: smFINISHEDTX  \ --
\ *G Handles the end of file transmission.
  cr ." TFTPState: FinishedTx"
;

: smRXDATA      \ --
\ *G Handles a received data packet.
  { | new_sai[ SOCKADDR_IN ] namelen -- }
  SOCKADDR_IN -> namelen
  new_sai[ SOCKADDR_IN erase

  *TFTP_PACKET if
    cr ." WARNING: smRXDATA Entered with an allocated TFTP_Packet"
    DiscardTFTP
  then

  TFTP_MAXPACKETSIZE allocate 0= if
    to *TFTP_PACKET

    TFTP_Socket *TFTP_PACKET TFTP_MAXPACKETSIZE
    0 new_sai[ addr namelen recvfrom -> TFTP_PACKETSIZE

    TFTP_PACKETSIZE SOCKET_ERROR = if
        S" Received TFTP Packet was too big (in smRXDATA)" AbortTFTP
        exit
    then

    new_sai[ SOCKADDR_IN sai[ SOCKADDR_IN compare if
        S" WARNING: Discarded packet from alien host in smRXDATA" AbortTFTP
        exit
    then

    *TFTP_PACKET 2+ w@(n)
    blockid <> if
        S" Out of sequence packet, retries are not allowed"
        AbortTFTP
    then

    *TFTP_PACKET 4 + TFTP_PACKETSIZE 4 - TEVENT::RxData
    DiscardTFTP

    STATE_ACKDATA to TFTPState

  else
    S" Failed to allocate buffer in smRXDATA" AbortTFTP
  then

;

: smTXDATA      \ --
\ *G Handles transmission of a data packet.
  cr ." TFTPState: TxData"
;

: smACKDATA     \ --
\ *G Handles a data acknowledge.
  { | pkt[ 4 ] }
  TFTP_ACK      pkt[ w!(n)
  blockid       pkt[ 2+ w!(n)
  TFTP_Socket pkt[ 4 0 send SOCKET_ERROR = if
    S" ERROR: Failed to ACKDATA" AbortTFTP
    exit
  then

  blockid 0>                                    \ if block > 0 then we are connected
  TFTP_PACKETSIZE 4 - TFTP_BLOCKSIZE <> AND if  \ and data packet < block size
    blockid 1+ to blockid                       \ then we are done!
    STATE_FINISHEDRX to TFTPState
  else
    blockid 1+ to blockid
    STATE_RXDATA to TFTPState
  then
;

: smWAITFORACK  \ --
\ *G Handles waiting for a data acknowledge.
  cr ." TFTPState: WAIT4ACK"
;


\ =======================================================================
( **                                                                    )
( ** TFTP Server Task                                                   )
( **                                                                    )
\ =======================================================================

: (TFTPTask)    \ --
  ConsoleIO  decimal			\ default debug I/O device
  cr ." TFTPtask running"
  STATE_IDLE to TFTPState		\ Initial State
  begin
[ diags? ] [if] ?StackEmpty [then]	\ check stack depth
    TFTPState case
      STATE_IDLE        of smIDLE       endof
      STATE_STARTRX     of smSTARTRX    endof
      STATE_STARTTX     of smSTARTTX    endof
      STATE_RXDATA      of smRXDATA     endof
      STATE_WAITFORACK  of smWAITFORACK endof
      STATE_TXDATA      of smTXDATA     endof
      STATE_ACKDATA     of smACKDATA    endof
      STATE_FINISHEDRX  of smFINISHEDRX endof
      STATE_FINISHEDTX  of smFINISHEDTX endof
        STATE_IDLE to TFTPState
    endcase
    pause
  again
;

task TFTPTask	\ -- addr
\ *G The TFTP task data structure.
cdata

: TFTP_Install	\ -- flag
\ *G Starts the TFTP server, returning true for success or
\ ** false for failure. SUBJECT TO CHANGE.
  { | sai[ SOCKADDR_IN ] }

  cr
  cr $MODULE $. space $VERSION $.
  cr $COPYRIGHT $.

  \ Create a socket for TFTP Server Channel, Abort if socket() failed
  AF_INET SOCK_DGRAM IPPROTO_UDP socket
  dup -> TFTP_Socket  INVALID_SOCKET =
  if  cr ." Failed TFTP Install"  false  exit  then

  \ Now Bind to Port #69
  sai[ SOCKADDR_IN erase
  AF_INET     sai[ sin_family w!(n)
  #69         sai[ sin_port w!(n)
  INADDR_ANY  sai[ sin_addr !(n)

  TFTP_Socket sai[ SOCKADDR_IN bind
  SOCKET_ERROR = if
    FALSE  exit
  then

  \ Start main task

  ['] (TFTPTask) TFTPTask initiate	\ assign action and start task

  cr ." TFTP Running ..."
  TRUE
;


\ ========================
\ *S Event Action Handlers
\ ========================
\ *P This code is currently commented out. Read it as an
\ ** application example.

(( part of C to Forth compilation
defer act-on-tftp-line	\ addr len -- ; process line received
  assign evaluate to-do act-on-tftp-line

: (TEVENT::BeginReceive)        \ c-addr u -- okay
  cr ." Receiving TFTP Code : " type space
  true
;

: (TEVENT::RxData)              \ addr len --
  begin
    dup 0 >
  while
    2dup $0A scan               \ addr len addrA lenpastA --
    2swap                       \ addrA lenpastA addr len --
    2 pick -                    \ addrA lenpastA addr 'len --
    2dup +  bl swap c!		\ add trailing space at end
    2swap 2>r act-on-tftp-line 2r>
    1 /string
  repeat
  2drop
  [char] . emit
;

: (TEVENT::FinishedRx)          \ --
  [EOF] ."  Complete" cr
;

assign (TEVENT::BeginReceive)  to-do TEVENT::BeginReceive
assign (TEVENT::RxData)        to-do TEVENT::RxData
assign (TEVENT::FinishedRx)    to-do TEVENT::FinishedRx


\ Test code
: (t1) cr [char] ' emit type [char] ' emit ;

: t1   assign (t1) to-do act-on-tftp-line ;
))


\ ======
\ *> ###
\ ======

decimal


