\ TCPECHO.FTH - top level vectored I/O for TCP Echo Socket

((
Copyright (c) 1997-2003, 2008, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20090602 SFP004 Rewritten for v4.8 and new BSD functions.
20081124 SFP003	Added wait for IP address.
20080306 SFP002 Kluge in CheckEchoState.
20030327 SFP001 Updated and retested.
))


\ ==========
\ *! tcpecho
\ *T TCP Echo socket
\ ==========
\ *P The Echo server just returns anything it receives to the
\ ** sender. It is a simple test of TCP transmission and
\ ** reception. Only one connection at a time is supported.

\ *P To save code space, the Echo server uses the server
\ ** system in *\i{SERVICES\Servers.fth} and *\fo{Saccept}
\ ** from the BSD layer.

\ *P As discussed in the Telnet chapter, HyperTerminal PE
\ ** is a reasonable test client.

decimal

TCP_PSH value EchoFlags	\ -- n
\ *G For fast response for use with an interactive terminal
\ ** set this to TCP_PUSH. For fast bulk response, set this
\ ** to zero.

((
7 equ EchoPort#	\ -- n
\ *G Port on which Echo server listens.
\ ** Moved to *\i{PNconfig.fth}.
))

$100 equ EBLen	\ -- n
\ *G Size of the Echo receive buffer.
EBLen buffer: EchoBuffer	\ -- addr
\ *G The Echo receive buffer.

#60000 equ /EchoMs	\ -- ms
\ *G Max delay after a connection before the socket is closed
\ ** if there is no transmission.

: echoEst?	\ hs -- flag
\ *G Return true if the socket is in ESTABLISHED state.
  get_socket_addr dup if
    so_*cb @ dup if
      t_state w@ TCPS_ESTABLISHED =
    endif
  endif
;

: echoWait	\ hs -- flag
\ *G Wait for something to be received up to the timeout period.
\ ** *\i{Flag} is returned true if the socket is still ESTABLISHED
\ ** and there has been no timeout.
  /EchoMs later >r
  begin
    r@ timedout? 0=
   while
    #10 ms
    dup pollSocket
  until then
  echoEst?  r> timedout? 0= and
;

: echoResponse	\ hs --
\ *G Read the input and respond.
  dup pollSocket dup 0> if		\ -- hs len
    EBlen min >r
    dup EchoBuffer r@ 0 recv drop
    EchoBuffer r> EchoFlags send drop
  else
    drop
  endif
;

: echoClose	\ hs --
\ *G Close the echo connection.
  dup echoEst? if
   disconnect drop
  else
   closeSocket drop
  endif
;

: EchoService	\ hs --
\ *G Service an Echo connection.
  begin
    dup echoWait
   while
    dup echoResponse
  repeat
  echoClose
;

: DoRunEchoSocket	\ --
\ *G The Echo task.
  s" Echo" SRVRstartup
  begin
    service_Echo 0 EchoPort# ServiceCreate
    begin
[ diags? ] [if]  ?StackEmpty  [then]	\ check stack depth
      my_hsocket @ 0 0 Saccept		\ wait for connection or failure
      dup SOCKET_ERROR <>
     while
      EchoService
    repeat
    drop
    my_hsocket @ closeSocket drop
    pause
  again
;

task TcpEchoTask	\ -- addr
\ *G The task running the Echo socket.

: RunEchoSockTask	\ --
\ *G Start the Echo task.
  ['] DoRunEchoSocket TcpEchoTask initiate
;


\ ======
\ *> ###
\ ======

decimal
