\ DEFINES.FTH - definitions for Forth Sockets

((
Copyright (c) 1997-2001
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002
  Hiden Analytical, 2002

To do
=====

Change history
==============
))

decimal

\ ==========
\ *! defines
\ *T System wide equates
\ ==========
\ *P This chapter documents *\i{DEFINES.FTH} which contains definitions
\ ** of constants and equates used throughout PowerNet.


\ **************************
\ *S Application definitions
\ **************************

0 equ INVALID_SOCKET	\ -- 0
\ *G Words returning a socket number return zero to indicate
\ ** an invalid socket.

-1 equ SOCKET_ERROR	\ -- -1
\ *G Many BSD layer and lower layers return -1 for an
\ ** error because a return of 0 as a length is valid.
\ ** It also permits systems to use *\fo{0<} as an error
\ ** check. *\b{Do not change}.


\ *************************************
\ *S Standard TCP/IP and Winsock values
\ *************************************
\ *P PowerNet provides a number of standard equates taken from
\ ** TCP/IP definitions and the Winsock API. These are not
\ ** documented here. If you are interested see *\i{DEFINES.FTH}.

\ *P Although a large number of these equates are defined,
\ ** and are not currently used, they occupy no memory in
\ ** the target system because these are *\fo{EQU}ates rather than
\ ** *\fo{CONSTANT}s.

\ All Windows Sockets error EQUs are biased by WSABASEERR.
10000	equ WSABASEERR

\ Windows Sockets definitions of regular Microsoft C error equs
WSABASEERR 4 +	equ WSAEINTR
WSABASEERR 9 +	equ WSAEBADF
WSABASEERR 13 +	equ WSAEACCES
WSABASEERR 14 +	equ WSAEFAULT
WSABASEERR 22 +	equ WSAEINVAL
WSABASEERR 24 +	equ WSAEMFILE

\ Windows Sockets definitions of regular Berkeley error equs
WSABASEERR 35 +	equ WSAEWOULDBLOCK
WSABASEERR 36 +	equ WSAEINPROGRESS
WSABASEERR 37 +	equ WSAEALREADY
WSABASEERR 38 +	equ WSAENOTSOCK
WSABASEERR 39 +	equ WSAEDESTADDRREQ
WSABASEERR 40 +	equ WSAEMSGSIZE
WSABASEERR 41 +	equ WSAEPROTOTYPE
WSABASEERR 42 +	equ WSAENOPROTOOPT
WSABASEERR 43 +	equ WSAEPROTONOSUPPORT
WSABASEERR 44 +	equ WSAESOCKTNOSUPPORT
WSABASEERR 45 +	equ WSAEOPNOTSUPP
WSABASEERR 46 +	equ WSAEPFNOSUPPORT
WSABASEERR 47 +	equ WSAEAFNOSUPPORT
WSABASEERR 48 +	equ WSAEADDRINUSE
WSABASEERR 49 +	equ WSAEADDRNOTAVAIL
WSABASEERR 50 +	equ WSAENETDOWN
WSABASEERR 51 +	equ WSAENETUNREACH
WSABASEERR 52 +	equ WSAENETRESET
WSABASEERR 53 +	equ WSAECONNABORTED
WSABASEERR 54 +	equ WSAECONNRESET
WSABASEERR 55 +	equ WSAENOBUFS
WSABASEERR 56 +	equ WSAEISCONN
WSABASEERR 57 +	equ WSAENOTCONN
WSABASEERR 58 +	equ WSAESHUTDOWN
WSABASEERR 59 +	equ WSAETOOMANYREFS
WSABASEERR 60 +	equ WSAETIMEDOUT
WSABASEERR 61 +	equ WSAECONNREFUSED
WSABASEERR 62 +	equ WSAELOOP
WSABASEERR 63 +	equ WSAENAMETOOLONG
WSABASEERR 64 +	equ WSAEHOSTDOWN
WSABASEERR 65 +	equ WSAEHOSTUNREACH
WSABASEERR 66 +	equ WSAENOTEMPTY
WSABASEERR 67 +	equ WSAEPROCLIM
WSABASEERR 68 +	equ WSAEUSERS
WSABASEERR 69 +	equ WSAEDQUOT
WSABASEERR 70 +	equ WSAESTALE
WSABASEERR 71 +	equ WSAEREMOTE

WSABASEERR 101 + equ WSAEDISCON

\ Extended Windows Sockets error equ definitions
WSABASEERR 91 +	equ WSASYSNOTREADY
WSABASEERR 92 +	equ WSAVERNOTSUPPORTED
WSABASEERR 93 +	equ WSANOTINITIALISED

\ Error return codes from gethostbyname() and gethostbyaddr()
\ (when using the resolver). Note that these errors are
\ retrieved via WSAGetLastError() and must therefore follow
\ the rules for avoiding clashes with error numbers from
\ specific implementations or language run-time systems.
\ For this reason the codes are based at WSABASEERR+1001.
\ Note also that [WSA]NO_ADDRESS is defined only for
\ compatibility purposes.

\ Authoritative Answer: Host not found
WSABASEERR 1001 + equ WSAHOST_NOT_FOUND

\ Non-Authoritative: Host not found, or SERVERFAIL
WSABASEERR 1002 + equ WSATRY_AGAIN

\ Non recoverable errors, FORMERR, REFUSED, NOTIMP */
WSABASEERR 1003 + equ WSANO_RECOVERY
WSANO_RECOVERY	equ NO_RECOVERY

\ Valid name, no data record of requested type */
WSABASEERR 1004 + equ WSANO_DATA
WSANO_DATA	equ NO_DATA

\ no address, look for MX record */
WSANO_DATA	equ WSANO_ADDRESS
WSANO_ADDRESS	equ NO_ADDRESS

\ Protocols
decimal
0		equ	IPPROTO_IP			\ dummy for IP
1		equ	IPPROTO_ICMP			\ control message protocol
2		equ	IPPROTO_GGP			\ gateway^2 (deprecated)
6		equ	IPPROTO_TCP			\ tcp
12		equ	IPPROTO_PUP			\ pup
17		equ	IPPROTO_UDP			\ user datagram protocol
22		equ	IPPROTO_IDP			\ xns idp
77		equ	IPPROTO_ND			\ UNOFFICIAL net disk proto

255		equ	IPPROTO_RAW			\ raw IP packet
256		equ	IPPROTO_MAX

\ Port/socket numbers: network standard functions
7		equ IPPORT_ECHO
9		equ IPPORT_DISCARD
11		equ IPPORT_SYSTAT
13		equ IPPORT_DAYTIME
15		equ IPPORT_NETSTAT
21		equ IPPORT_FTP
23		equ IPPORT_TELNET
25		equ IPPORT_SMTP
37		equ IPPORT_TIMESERVER
42		equ IPPORT_NAMESERVER
43		equ IPPORT_WHOIS
57		equ IPPORT_MTP
67		equ IPPORT_BOOTPSERVER		\ BOOTP/DHCP server
68		equ IPPORT_BOOTPCLIENT		\ BOOTP/DHCP client
69		equ IPPORT_TFTP
137		equ IPPORT_NBNS			\ NetBIOS Name Service
138		equ IPPORT_NBDS			\ NetBIOS Datagram Service
139		equ IPPORT_NBSS			\ NetBIOS Session Service

\ Port/socket numbers: host specific functions
77		equ IPPORT_RJE
79		equ IPPORT_FINGER
87		equ IPPORT_TTYLINK
95		equ IPPORT_SUPDUP

\ UNIX TCP sockets
512		equ IPPORT_EXECSERVER
513		equ IPPORT_LOGINSERVER
514		equ IPPORT_CMDSERVER
520		equ IPPORT_EFSSERVER

\ UNIX UDP sockets
512		equ IPPORT_BIFFUDP
513		equ IPPORT_WHOSERVER
520		equ IPPORT_ROUTESERVER		\ 520+1 also used

\ Ports < IPPORT_RESERVED are reserved for
\ privileged processes (e.g. root).
1024		equ	IPPORT_RESERVED

\ Link numbers

155		equ	IMPLINK_IP
156		equ	IMPLINK_LOWEXPER
158		equ	IMPLINK_HIGHEXPER

\ Definitions of bits in internet address integers.
\ On subnets, the decomposition of addresses to host and net parts
\ is done according to subnet mask, not the masks here.

\ equ IN_CLASSA(i) (((long)(i) & 0x80000000) == 0)
$ff000000	equ	IN_CLASSA_NET
$00ffffff	equ	IN_CLASSA_HOST
24		equ	IN_CLASSA_NSHIFT
128		equ	IN_CLASSA_MAX

\ equ IN_CLASSB(i) (((long)(i) & 0xc0000000) == 0x80000000)
$ffff0000	equ	IN_CLASSB_NET
$0000ffff	equ	IN_CLASSB_HOST
16		equ	IN_CLASSB_NSHIFT
65536		equ	IN_CLASSB_MAX

\ equ IN_CLASSC(i) (((long)(i) & 0xe0000000) == 0xc0000000)
$ffffff00	equ	IN_CLASSC_NET
$000000ff	equ	IN_CLASSC_HOST
8		equ	IN_CLASSC_NSHIFT

\ equ INADDR_ANY (u_long)0x00000000
$00000000	equ	INADDR_ANY	\ -- 0
$7f000001	equ	INADDR_LOOPBACK
$ffffffff	equ	INADDR_BROADCAST
$ffffffff	equ	INADDR_NONE

\ Options for use with [gs]etsockopt at the IP level.
1		equ	IP_OPTIONS			\ set/get IP per-packet options
2		equ	IP_MULTICAST_IF			\ set/get IP multicast interface
3		equ	IP_MULTICAST_TTL		\ set/get IP multicast timetolive
4		equ	IP_MULTICAST_LOOP		\ set/get IP multicast loopback
5		equ	IP_ADD_MEMBERSHIP		\ add  an IP group membership
6		equ	IP_DROP_MEMBERSHIP		\ drop an IP group membership

1		equ	IP_DEFAULT_MULTICAST_TTL	\ normally limit m'casts to 1 hop
1		equ	IP_DEFAULT_MULTICAST_LOOP	\ normally hear sends if a member
20		equ	IP_MAX_MEMBERSHIPS		\ per socket; must fit in one pbuf

\ Types

1		equ	SOCK_STREAM			\ stream socket
2		equ	SOCK_DGRAM			\ datagram socket
3		equ	SOCK_RAW			\ raw-protocol interface
4		equ	SOCK_RDM			\ reliably-delivered message
5		equ	SOCK_SEQPACKET			\ sequenced packet stream

\ Option flags per-socket.
$0001		equ	SOPT_DEBUG			\ turn on debugging info recording
$0002		equ	SOPT_ACCEPTCONN			\ socket has had listen()
$0004		equ	SOPT_REUSEADDR			\ allow local address reuse
$0008		equ	SOPT_KEEPALIVE			\ keep connections alive
$0010		equ	SOPT_DONTROUTE			\ just use interface addresses
$0020		equ	SOPT_BROADCAST			\ permit sending of broadcast msgs
$0040		equ	SOPT_USELOOPBACK		\ bypass hardware when possible
$0080		equ	SOPT_LINGER			\ linger on close if data present
$0100		equ	SOPT_OOBINLINE			\ leave received OOB data in line


\ Additional options.
$1001		equ	SOPT_SNDBUF			\ send buffer size
$1002		equ	SOPT_RCVBUF			\ receive buffer size
$1003		equ	SOPT_SNDLOWAT			\ send low-water mark
$1004		equ	SOPT_RCVLOWAT			\ receive low-water mark
$1005		equ	SOPT_SNDTIMEO			\ send timeout
$1006		equ	SOPT_RCVTIMEO			\ receive timeout
$1007		equ	SOPT_ERROR			\ get error status and clear
$1008		equ	SOPT_TYPE			\ get socket type

\ Options for connect and disconnect data and options.  Used only by
\ non-TCP/IP transports such as DECNet, OSI TP4, etc.
$7000		equ	SOPT_CONNDATA
$7001		equ	SOPT_CONNOPT
$7002		equ	SOPT_DISCDATA
$7003		equ	SOPT_DISCOPT
$7004		equ	SOPT_CONNDATALEN
$7005		equ	SOPT_CONNOPTLEN
$7006		equ	SOPT_DISCDATALEN
$7007		equ	SOPT_DISCOPTLEN

\ Option for opening sockets for synchronous access.
$7008		equ	SOPT_OPENTYPE

10		equ	SOPT_SYNCHRONOUS_ALERT
20		equ	SOPT_SYNCHRONOUS_NONALERT

\ Address families.
0		equ	AF_UNSPEC			\ unspecified
1		equ	AF_UNIX				\ local to host (pipes, portals)
2		equ	AF_INET				\ internetwork: UDP, TCP, etc.
3		equ	AF_IMPLINK			\ arpanet imp addresses
4		equ	AF_PUP				\ pup protocols: e.g. BSP
5		equ	AF_CHAOS			\ mit CHAOS protocols
6		equ	AF_IPX				\ IPX and SPX
6		equ	AF_NS				\ XEROX NS protocols
7		equ	AF_ISO				\ ISO protocols
7		equ	AF_OSI				\ AF_ISO OSI is ISO
8		equ	AF_ECMA				\ european computer manufacturers
9		equ	AF_DATAKIT			\ datakit protocols
10		equ	AF_CCITT			\ CCITT protocols, X.25 etc
11		equ	AF_SNA				\ IBM SNA
12		equ	AF_DECnet			\ DECnet
13		equ	AF_DLI				\ Direct data link interface
14		equ	AF_LAT				\ LAT
15		equ	AF_HYLINK			\ NSC Hyperchannel
16		equ	AF_APPLETALK			\ AppleTalk
17		equ	AF_NETBIOS			\ NetBios-style addresses
17		equ	AF_ROUTE			\ Internal routing protocol
18		equ	AF_VOICEVIEW			\ VoiceView
18		equ	AF_LINK				\ Link layer interface

19		equ AF_MAX

\ Protocol families, same as address families for now.
decimal
0		equ	PF_UNSPEC
1		equ	PF_UNIX
2		equ	PF_INET
3		equ	PF_IMPLINK
4		equ	PF_PUP
5		equ	PF_CHAOS
6		equ	PF_NS
6		equ	PF_IPX
7		equ	PF_ISO
7		equ	PF_OSI
8		equ	PF_ECMA
9		equ	PF_DATAKIT
10		equ	PF_CCITT
11		equ	PF_SNA
12		equ	PF_DECnet
13		equ	PF_DLI
14		equ	PF_LAT
15		equ	PF_HYLINK
16		equ	PF_APPLETALK
17		equ	PF_VOICEVIEW

19		equ	PF_MAX

\ Level number for (get/set)sockopt() to apply to socket itself.
$ffff		equ SOL_SOCKET				\ options for socket level

\ Maximum queue length specifiable by listen.

5		equ	SOMAXCONN
$1		equ	MSG_OOB				\ process out-of-band data
$2		equ	MSG_PEEK			\ peek at incoming message
$4		equ	MSG_DONTROUTE			\ send without using routing tables

16		equ	MSG_MAXIOVLEN

$8000		equ	MSG_PARTIAL			\ partial send or recv for message xport

\ Define equ based on RFC883, used by gethostbyxxxx() calls.

1024		equ	MAXGETHOSTSTRUCT

\ Define flags to be used with the WSAAsyncSelect() call.

$01		equ	FD_READ
$02		equ	FD_WRITE
$04		equ	FD_OOB
$08		equ	FD_ACCEPT
$10		equ	FD_CONNECT
$20		equ	FD_CLOSE

$0001		equ	IFF_UP				\ interface is up
$0002		equ	IFF_BROADCAST			\ broadcast address valid
$0004		equ	IFF_DEBUG			\ turn on debugging
$0008		equ	IFF_LOOPBACK			\ is a loopback net
$0010		equ	IFF_POINTOPOINT			\ interface is point-to-point link
$0020		equ	IFF_NOTRAILERS			\ avoid use of trailers
$0040		equ	IFF_RUNNING			\ resources allocated
$0080		equ	IFF_NOARP			\ no address resolution protocol
$0100		equ	IFF_PROMISC			\ receive all packets
$0200		equ	IFF_ALLMULTI			\ receive all multicast packets
$0400		equ	IFF_OACTIVE			\ transmission in progress
$0800		equ	IFF_SIMPLEX			\ can't hear own transmissions
$1000		equ	IFF_LINK0			\ per link layer defined bit
$2000		equ	IFF_LINK1			\ per link layer defined bit
$4000		equ	IFF_LINK2			\ per link layer defined bit
$8000		equ	IFF_MULTICAST			\ supports multicast

\ IF types
decimal
6		equ	IFT_ETHERNET			\ IP over ethernet - csmacd
18		equ	IFT_T1				\ IP over T1 link
20		equ	IFT_ISDNBASIC			\ IP over basic rate ISDN
21		equ	IFT_ISDNPRIMARY			\ IP over primat rate ISDN
23		equ	IFT_PPP				\ IP over PPP link
28		equ	IFT_SLIP			\ IP over generic TTY


\ define some Ethernet packet types
$0800		equ	ETHERTYPE_IP			\ basic IP protocol packet
$0806		equ	ETHERTYPE_ADDRESS_RESOLUTION	\ address resolution packet

$06		equ	ETHER_ADDRESS_LEN		\ bytes in ethernet address
$04		equ	IP_ADDRESS_LEN			\ bytes in IP address

\ now define some packet dependent codes
\ ARP op codes
$0001		equ	ARPOP_REQUEST			\ request to resolve address
$0002		equ	ARPOP_REPLY			\ response giving hardware address
$0003		equ	ARPOP_REVREQUEST		\ request protocol address from hardware
$0004		equ	ARPOP_REVREPLY			\ response giving protocol address
$0008		equ	ARPOP_INVREQUEST		\ request to identify peer
$0009		equ	ARPOP_INVREPLY			\ response identifying peer

\ hardware address format definitions
$0001		equ	ARPHARD_ETHER			\ ethernet format hardware address
$000F		equ	ARPHARD_FRELAY			\ frame relay format hardware address

\ protocol address format definitions
$0800		equ	ARPPROTO_IP			\ ethernet format protocol


\ **************
\ BOOTP and DHCP
\ **************

\ BOOTP opcodes
1 EQU BOOTPREQUEST
2 EQU BOOTPREPLY

\ DHCP message types in DHCP_OPT_MSGTYPE
1 EQU DHCPDISCOVER	\ Client broadcast to locate available servers.
2 EQU DHCPOFFER		\ Server to client in response to DHCPDISCOVER with
			\ offer of configuration parameters.
3 EQU DHCPREQUEST	\ Client message to servers either (a) requesting
			\ offered parameters from one server and implicitly
			\ declining offers from all others, (b) confirming
			\ correctness of previously allocated address after,
			\ e.g., system reboot, or (c) extending the lease on a
			\ particular network address.
4 EQU DHCPDECLINE	\ Client to server indicating network address is already
			\ in use.
5 EQU DHCPACK		\  Server to client with configuration parameters,
			\ including committed network address.
6 EQU DHCPNAK		\ Server to client indicating client's notion of network
			\ address is incorrect (e.g., client has moved to new
			\ subnet) or client's lease as expired
7 EQU DHCPRELEASE	\ Client to server relinquishing network address and
			\ cancelling remaining lease.
8 equ DHCPINFORM	\ Client to server, asking only for local configuration
			\ parameters; client already has externally configured
			\ network address.

\ DHCP option numbers
1  EQU DHCP_OPT_SUBNETMASK
3  EQU DHCP_OPT_ROUTER
6  EQU DHCP_OPT_DOMAINSERVERS
12 EQU DHCP_OPT_HOSTNAME
15 EQU DHCP_OPT_DOMAINNAME
23 EQU DHCP_OPT_IP_TTL
26 EQU DHCP_OPT_MTU
28 EQU DHCP_OPT_BROADCASTADDR
37 EQU DHCP_OPT_TCP_TTL
42 EQU DHCP_OPT_SNTP
50 EQU DHCP_OPT_REQUESTADDR
51 EQU DHCP_OPT_LEASE
53 EQU DHCP_OPT_MSGTYPE
54 EQU DHCP_OPT_SERVERID
55 EQU DHCP_OPT_PARAMREQ
58 EQU DHCP_OPT_T1
59 EQU DHCP_OPT_T2
60 EQU DHCP_OPT_CLASSID
61 EQU DHCP_OPT_CLIENTID
81 EQU DHCP_OPT_CLIENTDOMAIN

$FFFFFFFF EQU DHCP_INFINITE

\ HOSTNAME length is NetBIOS name length - 1
\ to allow for WINS resource type byte
\ Watch this if it is used with BUFFER: or other
\ words that do not ALIGN

15 EQU HOSTNAME_LEN

\ Automatic Private IP Addressing See. Q220874
$A9FE0000 equ APIPA_ADDR
$A9FEFFFF equ APIPA_BROADCAST
$FFFF0000 equ APIPA_MASK

$C0A80001 equ WINDOWS_ICS_ADDR  \ Windows Internet Connection Sharing



\ ======
\ *> ###
\ ======

decimal

