\ DIAGS.FTH - dump diags for the Forth Sockets Interface

((
Copyright (c) 1997-2003, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002
  Hiden Analytical, 2002

To do
=====

Change history
==============
20070302 SFP003 Replaced .IN with .IPADDRESS.
		Moved checksum code from TCP.FTH.
20030512 GS_002 Added diags for timers
20030319 GS_001 Changed .socket to display tcpcb instead of just state
))

\ ========
\ *! diags
\ *T PowerNet diagnostic tools
\ ========

decimal

: check_tcp_cksum	{ *ip | *tcp buff[ #12 ] -- flag }
\ *G Check incoming TCP cksum, returning true for a good checkum.
  *ip ip_hdr + -> *tcp			\ start of tcp buffer
  *tcp #12 - buff[ #12 cmove		\ save original buffer
  *tcp #8 - *tcp #12 - 8 cmove		\ move incoming IP addresses
  IPPROTO_TCP *tcp 4 - w!(n)		\ put protocol into pseudo-hdr
  *ip GetTCPPktLen			\ total TCP packet length
  dup *tcp 2- w!(n)			\ into pseudohdr
  *tcp #12 - swap #12 + cksum		\ now compute the checksum
  0=					\ OK if we return 0
  buff[ *tcp #12 - #12 cmove		\ restore original buffer
;

(( unused
: calc_tcp_cksum	{ *ip | *tcp buff[ #12 ] -- cksum }
\ \ *G Calculate incoming TCP cksum, returning true for a good checkum.
  *ip ip_hdr + -> *tcp			\ start of tcp buffer
  *tcp #12 - buff[ #12 cmove		\ save original buffer
  *tcp #8 - *tcp #12 - 8 cmove		\ move incoming IP addresses
  IPPROTO_TCP *tcp 4 - w!(n)		\ put protocol into pseudo-hdr
  *ip GetTCPPktLen			\ total TCP packet length
  dup *tcp 2- w!(n)			\ into pseudohdr
  *tcp #12 - swap #12 + cksum		\ now compute the checksum
  buff[ *tcp #12 - #12 cmove		\ restore original buffer
;
))

: dump_line	\ address bytes --
\ *G Display a small memory area as a single line of bytes
\ ** followed by the ASCII equivalent.
  2dup bounds
  ?do  space i c@ .byte  loop
  2 spaces
  bounds ?do
    space i c@ $7F and dup $20 <=
    if  drop $2e  endif
    emit
  loop
;

: .protocol	\ protocol --
\ *G Display the protocol type.
  case
    IPPROTO_UDP	of ." UDP"	endof
    IPPROTO_IP	of ." IP"	endof
    IPPROTO_TCP	of ." TCP"	endof
	dup .
  endcase
;

: .socket_type	\ type --
\ *G Display the socket type.
  case
    SOCK_DGRAM	of ." Datagram"	endof
    SOCK_STREAM	of ." Stream"	endof
    SOCK_RAW	of ." Raw"	endof		\ not supported (yet!!)
	dup .
  endcase
;

: .iphdr	\ addr --
\ *G Display an ip header.
  cr ." Version         "  dup ip_v_hl c@ $10 / .
  cr ." Header Length   "  dup ip_v_hl c@ $10 mod 4* .decimal
  cr ." Type of service $" dup ip_tos c@ .byte
  cr ." Packet length   "  dup ip_len w@(n) .decimal
  cr ." ID              $" dup ip_id w@(n) .word
  cr ." Flags/FragOff   $" dup ip_off w@(n) .word
  cr ." Time to live    "  dup ip_ttl c@ .decimal
  cr ." Protocol        $" dup ip_p c@ .byte
  cr ." Checksum        $" dup ip_sum w@(n) .word
  cr ." Src  Address    "  dup ip_src .IPnet
  cr ." Dest Address    "  dup ip_dest .IPnet
  drop
  cr
;

: .udp		\ *ip --
\ *G Prints UDP packet information.
  cr ." UDP "
  ip_hdr +
  cr ." Src  Port       "  dup uh_sport w@(n) .decimal
  cr ." Dest Port       "  dup uh_dport w@(n) .decimal
  cr ." Length          $" dup uh_ulen w@(n) .hex
  cr ." Checksum        $" dup uh_sum w@(n) .hex
  dup udp_hdr + over uh_ulen w@(n) ?dump
  drop
  cr
;

tcp? [if]

: .tcpFlags	{ fl -- }
\ *G Display flag word contents.
  fl .word
  ."  len=" fl #12 rshift 4* .decimal space
  fl TCP_URG and if ." Urg, " endif
  fl TCP_ACK and if ." Ack, " endif
  fl TCP_PSH and if ." Psh, " endif
  fl TCP_RST and if ." Rst, " endif
  fl TCP_SYN and if ." Syn, " endif
  fl TCP_FIN and if ." Fin, " endif
;

xc? [if]
: .tcp		{ *ip | *tcp -- }
\ *G Prints TCP packet information.
[else]
: (.tcp)	{ *ip | *tcp -- }
[then]
  cr ." TCP "
  *ip ip_hdr + -> *tcp
  cr ." Src  Port       " *tcp th_sport w@(n) .decimal
  cr ." Dest Port       " *tcp th_dport w@(n) .decimal
  cr ." Seq             " *tcp th_seq @(n) .decimal
  cr ." Ack             " *tcp th_ack @(n) .decimal
  cr ." Flags           " *tcp th_hdrlen w@(n) .tcpflags
  cr ." Window size     " *tcp th_win w@(n) .decimal
  cr ." Checksum        $" *tcp th_sum w@(n) .word space
      *ip check_tcp_cksum
      if  ." OK"  else  ." Bad"  endif
  cr ." Urgent offset   " *tcp th_urp w@(n) .decimal
  cr
  *tcp >TCPData			\ addr of data
  *ip GetTCPDataLen		\ get data length
      over .hex dup .hex
      $80 min ?dump		\ display addr & len then dump data
  cr
;
xc? 0= [if]  assign (.tcp) to-do .tcp  [then]

: .tcpcb	{ *cb -- }
\ *G Prints TCP packet information.
  cr ." TCP Control block "
  cr ." MSS        " *cb t_maxseg @ .decimal
  cr ." rcv_win    " *cb rcv_lastwin @ .decimal
  cr ." snd_una    " *cb snd_una @ .decimal
  cr ." snd_nxt    " *cb snd_nxt @ .decimal
  cr ." Tx Buff    " *cb *tx_buffer @ .hex
  cr ." Tx In      " *cb *tx_in @ .hex
  cr ." Tx Out     " *cb *tx_out @ .hex
  cr ." Tx Len     " *cb tx_len @ .hex
  cr ." State      " *cb t_state w@ .tcpstate
\ GS_002...
  cr ." Timers     " *cb t_ackdelayed
  	5 0 do dup @ .decimal cell+ loop drop
	." now=" ticks .decimal
\ ...GS_002
;

[then]

: .sockaddr_in  \ ^sockaddr_in --
\ *G Display the contents of a SOCKADDR_IN or SOCKADDR
\ ** structure.
  cr ." Address Family  : " dup sin_family w@(n) .hex
  cr ." Port            : " dup sin_port w@(n) .hex
  cr ." Address         : "     sin_addr .IPnet
;

: show-socket	\ hs --
\ *G Short form of display of socket data.
  get_socket_addr ?dup if		\ socket available ???
    cr
    dup so_handle w@ 2 .r space
    dup so_protocol w@ .protocol space
    dup so_iploc .IPloc ." :"
    dup so_portloc w@ .
    #28 >pos
    dup so_iprem .IPloc ." :"
    dup so_portrem w@ .
    #50 >pos
    dup so_protocol w@ IPPROTO_TCP = if
      dup so_*cb @ t_state w@ .tcpstate	\ retrieve control block pointer
    else
      dup so_state w@ .word
    endif
    drop
  endif
;

: netstat	\ --
\ *G Display short-form data about all non-closed sockets.
  base @ >r decimal
  MAXSOCKETS 1 do
    i show-socket
  loop
  cr
  r> base !
;

: .ippkt	{ *ip -- }
\ *G Prints IP packet information.
  *ip .iphdr
  case  *ip ip_p c@			\ now get protocol
    IPPROTO_UDP of  *ip .udp  endof
[ tcp? ] [if]
    IPPROTO_TCP of  *ip .tcp  endof
[then]
  endcase
  cr
;

: .etheradd	\ addr --
\ *G Display an Ethernet MAC address at addr.
  ETHER_ADDRESS_LEN 0 ?do
    i if  [char] - emit  endif
    i over + c@ .byte
  loop
  drop
;

: .route	\ *rt --
\ *G Show the routing table entry contents.
  find_route ?dup if
    cr ." Route entry "
    cr ." IP Address  " dup iproute_addr .IPloc
    cr ." Port        " dup iproute_port w@ .byte
    cr ." Life        " dup iproute_life @ ticks - #1000 / .decimal ." seconds"
\    cr ." Count       " dup iproute_count w@ .word
    cr ." Ether Add   " dup iproute_ethadd .etheradd
    drop
  else
    cr ." No route entry"
  endif
;

: .routes	\ --
\ *G Show all routing table entries.
  IpRoutes RT_LEN bounds do
    i iproute_addr @ ?dup
    if  .route  endif
  iproute_struct +loop
;

: .qd		\ *q --
\ *G List the queue data.
  dup .dword  dup *q_head @ 0=
  if  drop ."  empty"  else  .q  endif
;

: (.socket)	\ *sk --
\ *G Display the supplied socket structure.
  cr ." Skt at   " dup .hex
  cr ." Handle   " dup so_handle w@ .
  cr ." Type     " dup so_type w@ .socket_type
  cr ." Protocol " dup so_protocol w@ .protocol
  cr ." Local    " dup so_iploc .IPloc
  cr ." Loc Port " dup so_portloc w@ dup .decimal ."  0x" .hex
  cr ." Remote   " dup so_iprem .IPloc
  cr ." Rem Port " dup so_portrem w@ dup .decimal ."  0x" .hex
  cr ." Inqueue  " dup so_*inq @ .qd
  cr ." Outqueue " dup so_*outq @ .qd
  cr ." State    " dup so_state w@ .
[ tcp? ] [if]
  dup so_protocol w@ IPPROTO_TCP = if
\    dup so_*cb @ t_state w@ .tcpstate 	\ retrieve control block pointer
    dup so_*cb @ .tcpcb			\ retrieve control block pointer
  else
    dup so_state w@ .word
  endif
[else]
  dup so_state w@ .word
[then]
\  cr ." Rem Enet " dup so_etherrem .etheradd
  cr ." *Service " dup so_*service @ .hex
  cr ." Service  " dup so_service @ .hex
  drop
;

: .socket	\ HSOCKET --
\ *G Long form display from the supplied socket handle.
  get_socket_addr dup if
    (.socket)
  else
    drop  cr ." Invalid socket handle"
  endif
  cr
;

: .k	{ | *sk -- }
\ *G Show socket states in short form.
  MAXSOCKETS 1+ 1 do
    i get_socket_addr ?dup if
      -> *sk
      cr i .  *sk so_protocol w@ .protocol space
[ tcp? ] [if]
      *sk so_protocol w@ IPPROTO_TCP = if
        *sk so_*cb @ 			\ retrieve control block pointer
        t_state w@ dup .tcpstate
        dup TCPS_ESTABLISHED = if
	  ."  on " *sk so_portloc w@ .decimal
	  ."  to " *sk so_iprem .IPloc
	  ." :"    *sk so_portrem w@ .decimal
        endif
	dup TCPS_LISTEN = if
	  ."  on " *sk so_portloc w@ .decimal
	endif
	drop
      endif
[then]
    endif
  loop
;

: .lk		\ --
\ *G Show all active socket states in long form.
  MAXSOCKETS 1+ 1 do
    i get_socket_addr ?dup
    if  (.socket)  endif
  loop
;

: .err		\ --
\ *G Display current socket error.
  GetSocketError
  case
    4 of ." WSAEINTR" endof
    9 of ." WSAEBADF" endof
    13 of ." WSAEACCES" endof
    14 of ." WSAEFAULT" endof
    22 of ." WSAEINVAL" endof
    24 of ." WSAEMFILE" endof
    35 of ." WSAEWOULDBLOCK" endof
    36 of ." WSAEINPROGRESS" endof
    37 of ." WSAEALREADY" endof
    38 of ." WSAENOTSOCK" endof
    39 of ." WSAEDESTADDRREQ" endof
    40 of ." WSAEMSGSIZE" endof
    41 of ." WSAEPROTOTYPE" endof
    42 of ." WSAENOPROTOOPT" endof
    43 of ." WSAEPROTONOSUPPORT" endof
    44 of ." WSAESOCKTNOSUPPORT" endof
    45 of ." WSAEOPNOTSUPP" endof
    46 of ." WSAEPFNOSUPPORT" endof
    47 of ." WSAEAFNOSUPPORT" endof
    48 of ." WSAEADDRINUSE" endof
    49 of ." WSAEADDRNOTAVAIL" endof
    50 of ." WSAENETDOWN" endof
    51 of ." WSAENETUNREACH" endof
    52 of ." WSAENETRESET" endof
    53 of ." WSAECONNABORTED" endof
    54 of ." WSAECONNRESET" endof
    55 of ." WSAENOBUFS" endof
    56 of ." WSAEISCONN" endof
    57 of ." WSAENOTCONN" endof
    58 of ." WSAESHUTDOWN" endof
    59 of ." WSAETOOMANYREFS" endof
    60 of ." WSAETIMEDOUT" endof
    61 of ." WSAECONNREFUSED" endof
    62 of ." WSAELOOP" endof
    63 of ." WSAENAMETOOLONG" endof
    64 of ." WSAEHOSTDOWN" endof
    65 of ." WSAEHOSTUNREACH" endof
    66 of ." WSAENOTEMPTY" endof
    67 of ." WSAEPROCLIM" endof
    68 of ." WSAEUSERS" endof
    69 of ." WSAEDQUOT" endof
    70 of ." WSAESTALE" endof
    71 of ." WSAEREMOTE" endof
    101 of ." WSAEDISCON" endof
    91 of ." WSASYSNOTREADY" endof
    92 of ." WSAVERNOTSUPPORTED" endof
    93 of ." WSANOTINITIALISED" endof
    1001 of ." WSAHOST_NOT_FOUND" endof
    1002 of ." WSATRY_AGAIN" endof
    1003 of ." WSANO_RECOVERY" endof
    1004 of ." WSANO_DATA" endof
  endcase
  cr
;

: z		\ --
\ *G Dump 64kb of memory at 0000:0000 and time the result.
\ ** Mostly used for Telnet checking and performance testing.
  ticks
  0 $10000 dump
  ticks swap - .decimal ." ms"
;

: zz		\ --
\ *G Perform Z above until a key is pressed.
  begin  z  key? until
;


\ ======
\ *> ###
\ ======

decimal

