\ ICMP.FTH - definitions for raw ICMP packet handling

((
Copyright (c) 1997-2001, 2003, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002
  Hiden Analytical, 2002

To do
=====

Change history
==============
20070228 SFP002 Refaction and renamed ICMPMPCKSUM? to ICMPCKSUM.
20030222 SFP001 Incorporated CEM changes
))


\ =======
\ *! icmp
\ *T ICMP handling
\ =======
\ *P The ICMP implementation is fairly minimal.
\ ** Only the implemented functions are documented. Many
\ ** diagnostics can be enabled by setting the equate
\ ** *\fo{ICMPMON?} to non-zero.

\ *P See *\i{ICMP.FTH}.

ICMPmon? [if]

: NetUnreachable	\ *icmp --
  ." - net"  $30 dump			\ just dump the problem
;

: HostUnreachable	\ *icmp --
  ." - host"  $30 dump			\ just dump the problem
;

: ProtocolUnreachable	\ *icmp --
  ." - proto"  $30 dump			\ just dump the problem
;

: PortUnreachable	\ *icmp --
  ." - port"  $30 dump			\ just dump the problem
;

: FragNeeded&DFSet	\ *icmp --
  ." - frag"  $30 dump			\ just dump the problem
;

: SourceRouteFailed	\ *icmp --
  ." - route"  $30 dump			\ just dump the problem
;

: DestUnreachable	\ *pbuf *icmp --
  cr ." DestUnreachable "
  case  dup icmp_code c@		\ get the sub-code
    0 of NetUnreachable		endof
    1 of HostUnreachable	endof
    2 of ProtocolUnreachable	endof
    3 of PortUnreachable	endof
    4 of FragNeeded&DFSet	endof
    5 of SourceRouteFailed	endof
	 drop				\ lost the *icmp
  endcase
  FreeQB				\ and finally lose the original pbuf
;

: EchoReply		\ *pb *icmp --
  cr ." EchoReply"  ShowPacket
  FreeQB				\ just clean up for now
;

: SourceQuench		\ *pbuf *icmp --
  cr ." SourceQuench"  ShowPacket
  FreeQB				\ just clean up for now
;

: Redirect		\ *pbuf *icmp -- ;
  cr ." Redirect"  drop
  FreeQB				\ just clean up for now
;

: TimeExceeded		\ *pbuf *icmp -- ;
  cr ." TimeExceeded"  drop
  FreeQB				\ just clean up for now
;

: ParamProblem		\ *pbuf *icmp -- ;
  cr ." ParamProblem"  drop
  FreeQB				\ just clean up for now
;

: TimeStamp		\ *pbuf *icmp -- ;
  cr ." TimeStamp"  drop
  FreeQB				\ just clean up for now
;

: TimestampReply	\ *pbuf *icmp -- ;
  cr ." TimestampReply" drop
  FreeQB				\ just clean up for now
;

: InfoRequest		\ *pbuf *icmp -- ;
  cr ." InfoRequest"  drop
  FreeQB				\ just clean up for now
;

: InfoReply		\ *pbuf *icmp -- ;
  cr ." InfoReply"  drop
  FreeQB				\ just clean up for now
;

[then]

decimal

((
: ICMPcksum	\ *pb -- flag ; 0 for a pass
\ \ *G Generate the ICMP checksum. When used to check an ICMP header,
\ \ ** it will return 0 for a good checksum.
  dup PBDataStart			\ get data pointer
  swap PBDataLen			\ get data len
  cksum					\ go do the checksum for the data
;
))
: ICMPcksum	\ *pb -- flag ; 0 for a pass
\ *G Generate the ICMP checksum. When used to check an ICMP header,
\ ** it will return 0 for a good checksum.
  PBDataStart				\ get data pointer
  dup ip_hdr - ip_len w@(n) ip_hdr -
  cksum					\ go do the checksum for the data
;

: EchoRequest		\ *pbuf *icmp -- ; got a ping
\ *G Handle a PING request.
[ ICMPmon? ] [if]  cr ." Ping "  [then]
  swap >r				\ save pbuf
  0 over icmp_type c!			\ 0 for echo response
  0 over icmp_cksum w!(n)		\ clear the checksum
  r@ ICMPcksum				\ generate checksum ; SFP002
  swap icmp_cksum w!(n)			\ now update the checksum
  r@ PBHdrStart 			\ pointer to ip hdr
  >r
  r@ ip_src @(n)			\ get source address
  r@ ip_dest @(n)			\ and dest
  r@ ip_src !(n)			\ and swap them over
  r@ ip_dest !(n)
  #255 r@ ip_ttl c!			\ set a time_to_live value ; SFP001
  0 r@ ip_sum w!(n)			\ clear the cksum to calculate
  r@ dup c@ $0F and 4* cksum		\ calculate the new iphdr cksum
  r> ip_sum w!(n)			\ and add to the ip header
  ip_hdr negate r@ p_data w+!		\ dec start of data
  ip_hdr r@ p_datalen w+!		\ bump length of data
  r> (IPSend) drop			\ go send the response
;

-short-branches
: RxICMPPacket		\ *pbuf -- ; OK so we got an IP packet
\ *G Handle an incoming ICMP packet.
[ ICMPmon? ] [if]  cr ." ICMP in"  [then]
  dup ICMPcksum if
[ ICMPmon? ] [if]
    cr "" ICMP Cksum error" count type
    dup $60 dump
[then]
    FreeQB  exit			\ lose all the data
  endif
  dup PBDataStart			\ -- *pb *icmp
  case  dup icmp_type c@		\ -- *pb *icmp type ; get message type
[ ICMPmon? ] [if]
    0  of  EchoReply  endof
    3  of  DestUnreachable  endof
    4  of  SourceQuench  endof
    5  of  Redirect  endof
    11 of  TimeExceeded  endof
    12 of  ParamProblem  endof
    13 of  Timestamp  endof
    14 of  TimestampReply  endof
    15 of  InfoRequest  endof
    16 of  InfoReply  endof
[then]
    8  of  EchoRequest  endof		\ * implemented
           \ -- *pb *icmp type
           nip  swap FreeQB		\ -- type ; discard PBUF
  endcase
;
+short-branches


\ ======
\ *> ###
\ ======

decimal

