\ IP.FTH - definitions for raw IP packet handling

((
Copyright (c) 1997-2003, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002

To do
=====

Change history
==============
20070228 SFP003 Moved FILL_IPHDR and IPID words from PRIMITIV.FTH.
		Delocalled words.
20070224 SFP002 Changed ISIPFORME to accept broadcasts even when
		IPADDRESS is set. Thic change is necessary for UDP
		protocols such as DHCP.
20030621 SFP001 Fetches from IPADDRESS are network order.
))


\ =====
\ *! ip
\ *T Basic IP layer
\ =====

decimal


\ ********
\ *S Tools
\ ********

\ SFP002...
: isIPforme	\ *pbuf -- flag
\ *G Flag true if this packet is for me.
  PBHdrStart ip_dest @(n) dup 0=	\ get destination address
  if  exit  endif			\ sanity check 0.0.0.0 is illegal
  dup IPAddress @(n) =			\  same as mine?
  if  drop true  exit  endif		\ yes, done; no, check broadcast
  EnetIPmask @(n) dup			\ if subnet mask
  if  invert tuck and =  exit  endif	\ ~mask = (ip & ~mask)
  drop  $FF and $FF =			\ simple broadcast check
;
\ ...SFP002

\ SFP003...
variable ipid	\ -- addr ; for outgoing packets
\ *G IP packet identifier, incremented by one for
\ ** each outgoing IP packet.

: SetIPhdr	\ len ipsrc ipdest ipproto *iphdr --
\ *G Initialise and fill in an IP header with no options.
  $45 over ip_v_hl c!  0 over ip_tos c!	\ version and TOS, no options
  ipid dup @ 1 + dup rot !  $FFFF and	\ update IPID
  over ip_id w!(n)
  $4000 over ip_off w!(n)		\ NO FRAGMENTATION
  $FF over ip_ttl c!			\ time to live
  tuck ip_p c!				\ get protocol into the ip header
  tuck ip_dest !(n)			\ destination
  tuck ip_src !(n)			\ source
  tuck ip_len w!(n)			\ length
  0 over ip_sum w!			\ zero checksum
  dup ip_hdr cksum			\ calculate the ip header checksum
  swap ip_sum w!(n)			\ add checksum to the ip hdr
;


: pb>IPH	\ *ip *pb --
\ *G Step the PBuf data pointers back to the IP header from the
\ ** TCP or UDP header.
  tuck - swap				\ -- offset *pb
  2dup p_data w!			\ set start of data
  tuck p_pkthdr w!			\ -- *pb ; set start of packet
  ip_hdr over p_datalen w+!		\ bump the data length
  dup p_datalen w@  swap p_len w!	\ update total packet length
;
\ ...SFP003


\ *********************
\ *S Sending IP packets
\ *********************

defer send>other	\ *pb *rt -- len
\ *G This allows other routes to be patched in to the stack
\ ** without changing the stack source.

: NoOther	\ *pb *rt -- len
\ *G The default action of *\fo{SEND>OTHER}.
  drop					\ lose the route
  FreeQB				\ discard the pbuf
  WSAHOST_NOT_FOUND SetSocketError	\ flag error code
  SOCKET_ERROR				\ return error (usually -1)
;
assign NoOther to-do send>other

\ SFP003...
XC? [if]
: (IPSend)	\ *pb -- len|-1
\ *G Send the data in the supplied PBuf (chain).
\ ** *PB is the first pbuf in the chain (only 1 allowed)
\ ** Returns the number of bytes sent if OK else *\fo{SOCKET_ERROR}.
[else]
: rt(IPSend)	\ *pb -- len|-1
[then]
\ cr ." In (IpSend)"
\ dup PBDataStart ether_hdr - $60 dump
  dup PBDataStart ip_dest @(n)		\ get dest IP address
\ cr ." Going to " dup .IPv4
  find_route dup if			\ do we know which port to use?
    case				\ -- *pb *rt
      dup iproute_port w@ DEV_MASK and	\ which port protocol
[ slip? ] [if]
\ cr ." On SLIP port"
      SLIP_MASK of  send>slip  endof	\ SLIP port, ret len
[then]
[ ethernet? ] [if]
\ cr ." On ETHER port"
      ETHER_MASK of  send>ether  endof	\ Ether port, ret len
[then]
        drop  send>other		\ needs to go to another port?
    end-case
  else					\ -- *pb *rt
\ cr ." Can't Route packet"
    NoOther
  endif
;
XC? 0= [if]  assign rt(IPsend) to-do (IPsend)  [then]

: IPSend	\ *sk *pb -- len|SOCKET_ERROR
\ *G Send the data in the supplied PBuf (chain).
\ ** Returns number of bytes sent if OK else SOCKET_ERROR.
  dup PBDataStart ip_hdr -  dup >r	\ IP header is before data
  over pb>IPH				\ backup to IP header

  dup PBDataLen				\ number of bytes
  rot dup so_iploc @			\ source IP address
  swap dup so_iprem @			\ destination IP address
  swap so_protocol w@			\ protocol
  r> SetIPhdr				\ set up IP header

  (IPSend)				\ and finally send this to the I/O device
;
\ ...SFP003


\ ***********************
\ *S Receiving IP packets
\ ***********************

defer RouteIP	\ *pb --
\ *G This IP packet is not for me! Vectored routing allows later
\ ** application dependent patching up. The default action is to
\ ** discard the packet with *\fo{FreeQB}.
assign FreeQB to-do RouteIP

defer CanRouteIP?	\ *pb -- T|F
\ *G Returns true if we can route this packet.
\ ** Vectored routing allows later application dependent
\ ** patching up.

: NoCanRoute	\ *pb -- False
\ *G The default action of CANROUTEIP?
  drop 0
;
assign NoCanRoute to-do CanRouteIP?

: ProcessPacket	\ *pb --
\ *G Process an incoming packet
  case  dup PBHdrStart ip_p c@			\ get protocol
    IPPROTO_ICMP of  RxICMPPacket  endof	\ pass on to the ICMP handler
    IPPROTO_UDP  of  RxUDPPacket  endof		\ pass on to the UDP handler
    IPPROTO_TCP  of  RxTCPPacket  endof		\ pass on to the TCP handler
      drop FreeQB				\ we can't handle this type yet
  end-case
;

PacketTask? [if]

Task PacketTask	\ -- addr
\ *G The task that handles incoming packets.

: DoIncoming	\ --
\ *G The action of the incoming packet task.
  ConsoleIO decimal			\ default debug I/O device
  cr ." Incoming Packet Task running"
  begin
[ diags? ] [if] ?StackEmpty [then]	\ check stack depth
    IPinQ dequeue ?dup if
      ProcessPacket
    endif
    pause
  again
;

: RunIncoming	\ --
\ *G Start the incoming packet task.
  ['] DoIncoming PacketTask initiate
;

: GoodIPPacket	{ *pb -- }
\ *G Process a good packet.
  *pb isIPforme if			\ check if this ip address is for me
    *pb IPinQ enqueue
  else
    ip_hdr *pb unbumpPBuffData		\ back over header again
    *pb RouteIP
  endif
;

[else]

: GoodIPPacket	\ *pb --
\ *G Process a good packet.
  dup isIPforme if			\ check if this ip address is for me
\ dup PBHdrStart ip_dest @(n) ipAddress @(n) =
\ if  dup showpacket  endif
    ProcessPacket
  else
    ip_hdr over unbumpPBuffData		\ back over header again
    RouteIP
  endif
;

XC? [if]
compiler
: RunIncoming	\ --
\ *G Start the incoming packet task.
;
target
[else]
: RunIncoming	\ --
; immediate
[then]

[then]

xc? [if] compiler [then]
: BadIpPacket	\ *pbuf --
\ *G A header check failed, so discard the packet.
  FreeQB				\ free up the whole pbuf chain
;

: IPHdrLen	\ *ip -- len
\ *G Find the length of the IP header.
  ip_v_hl c@ $0F and 4*
;

: +IPhdr	\ *ip -- addr
\ *G Step from the IP header to the next item, e.g. TCP header.
  dup IPHdrLen +
;
xc? [if] target [then]

: RxIpPacket	\ *pb --
\ *G Process a received IP packet.
\ dup showpacket
  dup PBHdrStart			\ -- *pb *ip ; address of start of IP header
  dup ip_v_hl c@ $F0 and $40 = if	\ version MUST be 4
    tuck ip_v_hl c@ $0F and 4*		\ -- *ip *pb len ; length of header
    over 2dup p_pkthdrlen w!		\ -- *ip *pb len *pb ; packet header len
    2dup p_data w+!			\ bump start of data
    over negate swap p_datalen w+!	\ -- *ip *pb len ; dec length of data
    rot swap cksum if 			\ -- *pb ; calculate the ip hdr checksum
[ diags? ] [if]  cr ." Bad CKSum"  [then]
      BadIpPacket
    else				\ ip header checksum was OK
      GoodIpPacket
    endif
  else					\ version error
\   cr ." Bad Version"
\   over $20 dump
    drop  BadIpPacket
  endif
;


\ ======
\ *> ###
\ ======

decimal

