\ ROUTING.FTH - this file handles IP routing

((
Copyright (c) 1997-2003, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002
  Hiden Analytical, 2002

To do
=====

Change history
==============
20070215 SFP002 Added route life functions.
		(FIND_ROUTE) gives a default route for broadcasts
                to IP addresses of the form a.b.c.255.
20030621 SFP001 Fetches from IPADDRESS are network order.
))

\ ==========
\ *! routing
\ *T Routing packets
\ ==========
\ *P The routing table is statically allocated. Configuration data
\ ** is in *\i{PNconfig.fth}.

((
#32 constant MAX_IPADDRS	\ -- n
\ *G No of entries in the  routing table (not dynamic yet).
))
iproute_struct MAX_IPADDRS * equ RT_LEN	\ -- n
\ *G Size of the routing table.
RT_LEN Buffer: IpRoutes		\ -- addr
\ *G This is the main routing table.

((
#7200000 constant ROUTE_LIFE	\ -- ms
\ *G Two hours in milliseconds.
#1000 constant ROUTE_SAMPLE_MS	\ -- ms
\ *G How often routes are tested im milliseconds.
))

variable RouteTicker	\ -- addr
\ *G Holds the time to sample route expiry next.

((
create EnetIPMask	\ -- addr
\ *G IP mask for addresses on Ethernet port.
\ ** The data is in network order.
  0 ,
create IPGateway	\ -- addr
\ *G Gateway attached to Ethernet port.
\ ** The data is in network order.
  0 ,
))

((
STRUCT iproute_struct	\ -- size ; of structure
\ *G Routing table entry. The fields have the prefix "IPROUTE_".
  4 FIELD iproute_addr			\ ip address for this route
  4 FIELD iproute_life			\ number of milliseconds til discarded
  6 FIELD iproute_ethadd		\ ether address (if applicable)
  2 FIELD iproute_port			\ port identifier 01/2/3=scc1/2/3,EE=ethernet
  aligned
END-STRUCT
))

create defRoute	\ --
\ *G IPROUTE structure for the default route.
  $FFFFFFFF ,(n)			\ Limited Broadcast IP address
  0 ,					\ infinite life
  $FFFF w,  $FFFF w,  $FFFF w,		\ Ethernet broadcast
  $E1 w,				\ port identifier.

\ SFP002...
: SetRouteLife	\ ms route --
\ *G Set the route's life to end *\i{ms} from now.
  swap later 1 or			\ destination time forced non-zero
  swap iproute_life !			\ set life
;

compiler
: SetInfLife	\ route --
\ *G Set the route to have an infinite life.
  iproute_life off
;
target

((
: ExtendLife	\ ms route --
\ *G Extend the life of the given route by *\i{ms} if the life
\ ** is not infinite.
  dup iproute_life @ if			\ -- ms route ; life=nz=timed
    dup iproute_life @ ticks -		\ -- ms route rem
    rot max swap SetRouteLife
  else					\ life=0=infinite
    2drop
  endif
;
))
\ ...SFP002

: InitRoutes	\ --
\ *G Initialise routing table.
  IpRoutes RT_LEN erase			\ wipe data structure
  ROUTE_SAMPLE_MS later RouteTicker !	\ start timer
;

: (find_route)	\ ip -- addr|0
\ *G Get routing table entry address (if available).
  dup $FF and $FF =			\ check for broadcast ; SFP002
  if  drop DefRoute exit  endif		\ return default for broadcasts ; SFP002
  0					\ assume not available
  IpRoutes RT_LEN bounds do		\ -- ip 0
    over i iproute_addr @ = if
      drop i leave
    endif
  iproute_struct +loop
  nip					\ lose IP address
;

: AskForRoute	\ ip -- addr|0
\ *G See if we can make a route.
[ ethernet? ] [if]
  AskForEtherRoute			\ deliberate forward reference
[else]
  drop 0				\ return no route
[then]
;

\ so see if we can get a route via the gateway
: ViaGateway?	{ ip | iptmp *rt -- addr|0 }
\ *G Returns gateway route or none.
\ rewrite without locals!
  0 -> *rt
  IPGateway @(n) -> iptmp		\ get gateway address
  iptmp if				\ OK so its NOT 0.0.0.0 !!
    iptmp -1 <> if			\ make sure its NOT 255.255.255.255 either
      iptmp (find_route) ?dup if	\ see if we already know about the gateway
        -> *rt				\ so use it
      else
        iptmp AskForRoute -> *rt	\ go try to get this route
      endif
    endif
  endif
  *rt 					\ return the answer
;

: CanWeRoute	{ ip -- addr|0 ; returns route or 0 }
\ *G Handles routing via the subnet mask or gateway (if one exists).
  EnetIPMask @(n)			\ get current subnet mask
  ip over and				\ mask down to requested subnet
  swap IPAddress @(n) and		\ mask our ether port down to requested subnet ; SFP001
  =					\ now is this the same ????
  if					\ yep
    ip AskForRoute			\ so go do an arp request for it
  else					\ nope
    ip ViaGateway?			\ so see if we can get a route via the gateway
  endif
;

: find_route	\ ip -- addr|0
\ *G Get table entry address (if available)
  dup (find_route) ?dup if     		\ first see if we have a route
    nip					\ lose IP address & exit
  else
    CanWeRoute				\ see if we can get one then
  endif
;

: empty_route?	\ -- addr|0
\ *G Get table entry address (if available).
  0
  IpRoutes RT_LEN bounds do
    i iproute_addr @ 0= if
      drop i leave			\ OK this one is empty
    endif
  iproute_struct
  +loop
;

: add_route	\ ip -- addr|0
\ *G Get table entry address (if available).
  dup (find_route) dup if		\ already exists?
    nip					\ lose IP address, return struct addr
  else
    drop				\ discard 0
    empty_route? dup if
      dup iproute_struct erase		\ wipe it to start with
      tuck !				\ add ip address & ret struct addr
    else
      2drop 0				\ lose IP address & ret 0
    endif
  endif
;

[-short-branches
: expire_routes?	\ --
\ *G Decrement the route entry timers and expire any
\ ** routes that have timed out.
\ N.B. TIMEDOUT? does NOT call PAUSE.
  RouteTicker @ timedout? if		\ only every so often
    IpRoutes RT_LEN bounds do
      i iproute_addr @ if		\ 0 = empty slot
        i iproute_life @ ?dup if	\ 0 = infinite life
          timedout? if			\ if clock ticks expired
            i iproute_struct erase	\ wipe it when expired
          endif
        endif
      endif
    iproute_struct +loop
    ROUTE_SAMPLE_MS later RouteTicker !	\ restart route sample timer
  endif
;
short-branches]


\ ======
\ *> ###
\ ======

