\ STRUCTS.FTH - structure definitions for Forth Sockets

((
Copyright (c) 1997-2001, 2007, 2008, 2009
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002
  Hiden Analytical, 2002

To do
=====

Change history
==============
20091102 SFP007 Updated TCPCB.
20090518 SFP006 Moved TCPCB structure here.
20090505 SFP005 Added service length to socket structure.
20090210 SFP004	Reordered.
20080305 SFP003 Undid SFP002 after refactoring.
20080228 SFP002 Added service data.
20070214 SFP001 Removed IPROUTE_COUNT from IPROUTE_STRUCT.
		Reordered IPROUTE_STRUCT to reduce size.
20000221 SJC000 Added padding fields to some structure to ensure
                alignment of required data fields. Architectures
                such as ARM require that data is aligned to it's
                own size, i.e. byte accesses on any address, word
                accesses on even addresses and dword access can
                only occur on 4 byte aligned addresses.
))


\ ==========
\ *! structs
\ *T TCP/IP data structures
\ ==========
\ *P The definitions are in the file *\i{STRUCTS.FTH}.

\ *P You will find the books TCP/IP Illustrated, Volumes 1 and 2
\ ** useful when exploring the PowerNet data structures.

decimal


\ *********************
\ *S Primary structures
\ *********************
\ *P These structures are used in almost all systems.

ethernet? [if]
STRUCT ether_hdr	\ -- size ; of ether_hdr structure
\ *G Ethernet header. The fields have the prefix "ETHER_".
  6 FIELD ether_dest			\ destination ethernet address
  6 FIELD ether_src			\ source ethernet address
  2 FIELD ether_type			\ packet type (0x0800 for IP)
END-STRUCT

STRUCT arp_hdr		\ -- size ; of arp_hdr structure
\ *G ARP header. The fields have the prefix "AR_".
  2 FIELD ar_hard			\ format of hardware address
  2 FIELD ar_pro			\ format of protocol address
  1 FIELD ar_hln			\ length of hardware address
  1 FIELD ar_pln			\ length of protocol address
  2 FIELD ar_op				\ op code (see definitions)
END-STRUCT

STRUCT arp_ip_data	\ -- size ; of arp_ip_data structure
\ *G ARP data.
  6 FIELD ether_sender			\ sender ethernet address
  4 FIELD ip_sender			\ sender protocol address
  6 FIELD ether_target			\ target ethernet address
  4 FIELD ip_target			\ target protocol address
END-STRUCT
[else]
#14 equ ether_hdr			\ required for other files to compile
[then]

STRUCT icmp_hdr		\ -- size ; of icmp header structure
\ *G ICMP header structure. The fields have the prefix "ICMP_".
  1 FIELD icmp_type			\ type
  1 FIELD icmp_code			\ code
  2 FIELD icmp_cksum			\ checksum
END-STRUCT

STRUCT ip_hdr		\ -- size
\ *G IP header structure. The fields have the prefix "IP_".
  1 FIELD ip_v_hl			\ version and header len (*4)
  1 FIELD ip_tos			\ type of service
  2 FIELD ip_len			\ total packet length
  2 FIELD ip_id				\ identification
  2 FIELD ip_off			\ flags and fragment offset
  1 FIELD ip_ttl			\ time to live
  1 FIELD ip_p				\ protocol
  2 FIELD ip_sum			\ header checksum
  4 FIELD ip_src			\ source ip address
  4 FIELD ip_dest			\ destination ip address
END-STRUCT

STRUCT udp_hdr		\ -- size ; of udp header structure
\ *G UDP header structure. The fields have the prefix "UH_".
  2 FIELD uh_sport			\ source port
  2 FIELD uh_dport			\ destination port
  2 FIELD uh_ulen			\ udp length
  2 FIELD uh_sum			\ checksum
END-STRUCT

STRUCT (socket)		\ -- size ; of (socket) structure
\ *G The main socket structure. The fields have the prefix "SO_".
  2 FIELD so_handle			\ the socket handle associated with this struct
  2 FIELD so_type			\ socket type
  2 FIELD so_portloc			\ local port to use
  2 FIELD so_portrem			\ remote port to use
  2 FIELD so_protocol			\ saved protocol requested
  2 FIELD so_state			\ internal state flags ; $FFFF = close me
  4 FIELD so_iploc			\ local ip address
  4 FIELD so_iprem			\ remote ip address
  4 FIELD so_*inq			\ *input  queue
  4 FIELD so_*outq			\ *output queue
  4 FIELD so_*cb			\ pointer to a control block
  int so_*service			\ pointer to service specific data
  int so_service			\ service (Telnet, HTTP ...)
  int so_/service			\ service data length ; SFP005
END-STRUCT

tcp? [if]
STRUCT tcpcb	\ -- size
\ *G TCP control block structure. Notes:
\ *(
\ *B Sequence and ACK numbers are unsigned 32 bit numbers that
\ ** can wrap. They *\b{must} be compared using circular
\ ** arithmentic.
\ *)
  2 FIELD t_state		\ TCP connection state
  2 field t_override		\ transmit overrides (TCP_PSH, TCP_URG)
  aligned
\ send sequence variables
  4 FIELD snd_una		\ sequence number of first unacknowledged byte
  4 FIELD snd_nxt		\ sequence number of next byte to send
  4 FIELD snd_up		\ send urgent pointer (unsupported)
  4 FIELD snd_iss		\ initial send sequence number from SYN ; SFP037
  4 FIELD snd_win		\ send window size
\ receive sequence variables
  4 FIELD rcv_nxt		\ expected next receive sequence number
  4 field rcv_lastwin		\ last window size sent
\ additional retransmit variables for sliding window
  4 FIELD *tx_buffer		\ holds data to be sent (see snd_una)
				\ points to start of unacknowledged data
  4 FIELD *tx_in		\ pointer to waiting data
				\ i.e. to end of buffered data
  4 FIELD *tx_out		\ pointer to next data to send
				\ i.e. start of untransmitted data
  4 FIELD tx_len		\ length of waiting data
\ transmit timing stuff
  4 FIELD t_ackdelayed		\ set when ack required, ack (+data ?) sent when expires
  4 FIELD t_txdelayed		\ set when data ready, data sent when expires
  4 FIELD t_txretry		\ set when data ready, cleared when ALL data acked
  4 FIELD t_connect		\ timer for connections - MUST complete before timeout
  4 FIELD t_disconnect		\ disconnect msl timer
  4 FIELD t_#retries		\ number of retries
  4 FIELD t_maxseg		\ max segment size to send ; SFP007
\ listening sockets
  4 field listen_q		\ TCP connection queue.
END-STRUCT

\ Unused by Powernet, but present in most hosted implementations.
\  2 FIELD t_timer1		\ tcp timers (more to follow)
\  2 FIELD t_rxtshift		\ backoff ????
\  2 FIELD t_rxtcur		\ current retransmission timeout
\  2 FIELD t_dupacks		\ #consecutive duplicate ACK's received
\  2 FIELD t_force		\ 1 if forcing out a byte (persist/OOB)
\  2 FIELD t_flags		\ default flags (bits, unused)
\  2 field t_rfu			\ padding
\  4 FIELD snd_wl1		\ window update seg seq no
\  4 FIELD snd_wl2		\ window update seg ack no
\  4 FIELD rcv_wnd		\ receive window
\  4 FIELD rcv_up		\ receive urgent pointer
\  4 FIELD irs			\ initial receive sequence number
\ additional receive variables
\  4 FIELD rcv_adv		\ advertised window by other end
\  4 FIELD snd_max		\ highest sequence number sent
\  4 FIELD t_idle		\ inactivity time
\  4 FIELD t_rtt			\ round-trip time
\  4 FIELD t_rtseq		\ sequence number being timed
\  4 FIELD t_srtt		\ smoothed round-trip time
\  4 FIELD t_rttvar		\ variance in round-trip time
\  4 FIELD t_rttmin		\ minumum rtt allowed
\  4 FIELD max_sndwnd		\ largest window peer has offered
\ segment queueing
\  4 FIELD *rxseqQ		\ queue of received segments
[then]

((
\ Unused for single IP connection of any type. Needed only when
\ there are multiple IP connections.
STRUCT port_struct	\ -- size ; of structure
\ *G Port control data. The fields have the prefix "PORT_".
\ ** Unused by systems with only a single port.
  1 FIELD port_id			\ port identifier 01/2/3=scc1/2/3,EE=ethernet
  1 FIELD port_status			\ current status
  2 FIELD port_count			\ 16 bit packet counter
  4 FIELD port_ipaddr			\ ip address for this port
  4 FIELD port_ipmask			\ ip mask for this port
END-STRUCT
))

STRUCT iproute_struct	\ -- size ; of structure
\ *G Routing table entry. The fields have the prefix "IPROUTE_".
\ ** The entries are added to when a new IP address is received.
  4 FIELD iproute_addr			\ ip mask for this port
  4 FIELD iproute_life			\ number of milliseconds til discarded
  6 FIELD iproute_ethadd		\ ether address (if applicable)
  2 FIELD iproute_port			\ port identifier 01/2/3=scc1/2/3,EE=ethernet
  aligned
END-STRUCT

\ define size of the largest data block
PSIZE ether_hdr - ip_hdr - udp_hdr - equ PDATA_MAX
\ *G The size of the largest data block.

STRUCT SOCKADDR_IN	\ -- size
\ *G Used to hold socket family (always AF_INET), port
\ ** number and IP address. The fields have the prefix "SIN_".
  2 FIELD sin_family
  2 FIELD sin_port
  4 FIELD sin_addr
  8 FIELD sin_zero
END-STRUCT


\ ******************
\ *S SNMP structures
\ ******************

snmp? [if]
STRUCT TAGLIST_ENTRY	\ -- size
\ *G SNMP Tx/Rx taglist entry definition.
\ ** The fields have the prefix "SNMP_".
  2 FIELD	SNMP_tag		\ the actual tag value
  2 FIELD	SNMP_var_len		\ length of variable
  4 FIELD	*SNMP_var		\ pointer to the snmp variable contents
END-STRUCT
[then]


((
Per basic active socket
=======================
$0028			$0028	\ socket structure
$0010 * 2		$0020   \ InpQ + OutQ
                        -----
                        $0048
Per active TCP socket
=====================
$0048			$0048	\ base socket
$0010 * 2		$0020	\ RcvQ and SndQ
$0B68 ( TCPTXBUFFSIZE )	$0B68	\ Transmit queue = 2*1460
			-----
			$0BD0	\ $061C for 1460 byte

Heap memory allocation (main users)
======================
$0600 * NUMFREEBUFs	$1800	\ 4 free PBUFs
$0600 * 1               $0600	\ reserved TX PBUF
$0BD0			$0BD0	\ Telnet socket
$0114			$0114	\ /TNdata - Telnet service data
$0BD0			$0BD0	\ HTTP socket
$09E4			$09E4	\ /HTTPdata - HTTP service data
$0400			$0400	\ Telnet task
$0400			$0400	\ HTTP task
			-----
			$4898
))

\ ======
\ *> ###
\ ======

decimal

