\ TCP.FTH - definitions for TCP packet handling

((
Copyright (c) 1997-2006, 2007, 2008, 2009, 2011, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of the PowerNet code base have been licensed from:
  DelaComms Ltd, 1996-1999
  Oxford Net Solutions, 2000-2007
  CEM Systems Ltd, Belfast, 2000-2002
  Hiden Analytical, Warrington UK, 2002

To do
=====
Add timeouts in the connection queue.

Rewrite Tcp_Close and Tcp_Open.

Move (CloseSocket) to SOCKPRIM.FTH.

Refactor and De-local more.

Factor out the SEQ/ACK code.

Allow a closed socket to stay alive a bit to handle the final
transaction properly. The setting of SO_STATE in (CloseSocket)
should be performed in doTCPclosed after some time.

SENDBUFF>TCP always sends an ACK - is this correct?

SENDTOTCP does not handle TCP_PUSH correctly in flags, T_OVERRIDE
field must be set to get immediate transmission.

Change history
==============
20120205 SFP051 Restored kill chain, undid part of SFP049.
20110729 SFP050 Changed way the window size is generated.
20110718 SFP049 Update handler for CLOSING, FIN_WAIT_1 and FIN_WAIT_2.
		Factored FREEUPSOCKET.
20110531 SFP048 Added GENWINDOWSIZE
20110504 SFP047 Factoring for user definable window size.
		Refactoring.
20110502 SFP046 Correction after SFP042.
20091102 SFP045 Added MSS parsing to SYN handling.
		Added MSS handling to SendTCPPkt.
20090608 SFP044 Added duplicate detection to connection queue.
20090605 SFP043 Added RST handler to doTCPSynReceived.
20090601 SFP042 Overhauled transmit buffer release.
		Factored out TCP state changes.
		Refactored (CloseSocket).
20090520 SFP041 Fixed delayed transmit buffer allocation.
20090518 SFP040 Moved TCPCB structure to structs.fth.
		Moved TCPCB creation/deletion to SOCKPRIM.FTH.
20090507 SFP039	Delayed allocation of transmit buffers to save RAM.
20081205 SFP038 Overhauled SENDNOSOCKET and DOTCPCLOSED to be
		more robust and standard compliant.
20080410 SFP037	ACK and SEQ fixes for retry recovery.
20080325 SFP036 Rewrote SendToTcp for error recovery.
		Added socket close in doTcpRetry?.
20080320 SFP035 Corrected doTcpLastAck
20080318 VRW034 Turned off retry timer in DoTcpRetry? to prevent
		a) multiple resends
		b) sequence/ack errors
20080317 SFP033 Added (CloseSocket). Moved final close to TcpIdle.
		This avoids race conditions when closing sockets
		and higher level code still uses data in the socket,
		control block and service data areas.
		Corrected DoTcpRetry?.
20080307 SFP032 Change to avoid name conflict.
20080303 SFP031 Refactored service chain stuff for hosted systems.
20080228 SFP030 Overhauled memory release strategy.
		Corrected socket ranges.
20070814 MGS029 See DOTCPESTABLISHED for numbering change.
20070318 EGS028 Added ability to use receiver's send window to limit sends
20070302 SFP027 Moved some diagnostic to DIAGS.FTH.
20070102 EGS026 First fix of ack problems
20060706 RMS025 Added max retries to prevent deadlock on failed sends.
20050918 SFP024 Added protection to DISCARDTCPTCB.
20050303 SFP023 Removed *RXSEGQ.
20050216 MPE022 Hunt for missing (N)s in TCP header access
20030515 SFP021 Delocalled some words
20030514 EGS020 Fixed DOTCPRETRY sequence number handling from EGS017
20030513 EGS019 Added an Ack flag with Fin when calling tcp_close
20030513 EGS018 Added a Rst handler in Fin_Wait1 to force closure on receipt
20030513 EGS017 Fixed snd_nxt & snd_una problem
		nxt was being stomped on by an incoming ack of 0 in the syn packets
		una was then always 1 byte behind nxt, such that when a valid ack
	 	came in una was never the same as nxt, therefore the 'tx_retry' counter
		was always running. Then after retry time (5 secs) nxt was set to una
		causing a single lost data byte on the next outgoing packet
20030512 EGS016 Removed ack when in TimeWait - caused Ack firestorm
20030512 EGS015 Moved idle timer to TcpEstablished - was stopping MSL Timouts !!
20030424 EGS014 Removed redundant ACK in incoming FIN
20030424 EGS013 Removed spurious ACK of incoming ACK
20030422 SFP012 Factored ForceTcpReset.
20030415 SBD011 If a packet is not transmitted by SendTcpPkt during a resend
		the snd_nxt pointer gets left set back to snd_una and the
		tx buffer out pointer is set back to the start of the buffer.
		As far as the protocol is concerned it is as if the original
		packet had not been sent.
		If the original packet is then acknowledged ackntxbytes checks that
		there are characters in the buffer to acknowledge, but
		DoTcpAcknowledge just sets snd_una to th_ack - so the buffer pointer
		does not advance but snd_una does.
		The fix for this is to make ackntxbytes return the number of bytes
		acknowledged and to increment snd_una by this value.
20030328 SFP010 Changed use of T_FLAGS field
20030327 EGS009 Changed buffer size & added pkt size
20030327 EGS008 Added common TcpCksum and also used this in Rx
20030319 EGS007 Initial performance review
20030319 EGS006 Minor fixes following SFP Review
20030319 SFP005 Minor refactoring
20030318 EGS004 More changes from CEM incorporated
20030318 EGS003 Bug fix - identified by Hiden
20030318 EGS002 Numerous changes to integrate in CEM modifications
20030317 EGS001	Prevent disconnect delay when in Listen state
20000221 SJC000 Modified diagnostic output to only be built when
                TCPDEBUG? is set.
))


\ ======
\ *! tcp
\ *T TCP layer
\ ======
\ *P You will find the books TCP/IP Illustrated, Volume 1
\ ** (Stevens) and Volume 2 (Wright & Stevens) useful when
\ ** exploring the PowerNet TCP code, especially for details
\ ** of sequence and ack numbers.

\ ********************
\ *S TCP configuration
\ ********************
\ *P The configuration below can now be found in *\i{PNconfig.fth}.

((
0 equ TCPDEBUG	\ -- n
\ *G Set this flag non-zero to generate debug messages.

#1460 equ TCPDATASIZE	\ -- n ; EGS009
\ *G Transmit buffer size of pbuf less tcp hdr(ish).
#1460 2 * equ TCPTXBUFFSIZE	\ -- n
\ *G Size of the retransmission buffer for a TCP socket.
\ ** One buffer of this size is allocated from the heap
\ ** when a TCP socket is created.
#1460 equ TCPWINDOWSIZE	\ -- n ; EGS002
\ *G TCP window size.

#100 equ TXDELAYTIME	\ -- ms
\ *G timer for delayed transmit (mSecs), default is 100 milliseconds.
#10 equ ACKDELAYTIME	\ -- ms
\ *G timer for delayed ack (mSecs), default is 10 milliseconds.
#5000 equ TXRETRYTIME	\ -- ms
\ *G timer for transmit retries (mSecs), default is 5 seconds.
#30000 equ TCPCONNECTTIME	\ -- ms
\ *G timer for incoming connections to complete (mSecs), default is 30 seconds.
#30000 equ TCPMSLTIME	\ -- ms
\ *G timer for maximum segment lifetime (mSecs, default is 30 secs).
#7200000 equ TCPIDLETIME	\ -- ms
\ *G timer for idle disconnection (mSecs, default is 2 hours).
#12 equ TCPMAXRETRIES	\ -- n
\ *G Maximum number of transmission retries before returning
\ ** SOCKET_ERROR, default is 12.
\ ** If a receiving socket fails to accept new data, the total
\ ** timeout will be TXRETRYTIME*TCPMAXRETRIES, the default being
\ ** 60 seconds.
))

create MSSopt	\ -- addr
\ *G Maximum Segment Size option.
  $0204 w,(n)  defMSS w,(n)	\ Option 2 = Maximum Segment Size


\ **************************
\ *S Unknown socket requests
\ **************************

variable NoTcpHandle	\ -- addr
\ *G Holds the handle of a socket used to handle unknown
\ ** TCP requests.

: Init-NoTcp-Socket	\ -- res|SOCKET_ERROR
\ *G Initialise a socket structure to handle unknown TCP requests.
\ ** *\fo{SOCKET_ERROR} is returned if the socket cannot be initialised.
  NoTcpHandle off
  AF_INET SOCK_STREAM IPPROTO_TCP socket ?dup 0=
  if  SOCKET_ERROR exit  endif
  dup AF_INET 0 INADDR_ANY bindTo
  tuck SOCKET_ERROR = if	\ -- res hs
    CloseSocket drop		\ better free it all up again
  else
    NoTcpHandle !		\ so save handle
  endif
;

-short-branches


\ *****************************
\ *S TCP structures and equates
\ *****************************
\ *P Most equates are not documented.

STRUCT tcp_hdr	\ -- size ; of tcp header structure
\ *G TCP header structure.
  2 FIELD th_sport		\ source port
  2 FIELD th_dport		\ destination port
  4 FIELD th_seq		\ 32 bit sequence number
  4 FIELD th_ack		\ 32 bit acknowledgement number
  1 FIELD th_hdrlen		\ 4 bits of hdr len + 4 reserved
  1 FIELD th_flags		\ 6 (lo) bits of flag data (see equates below)
  2 FIELD th_win		\ 16 bit window size
  2 FIELD th_sum		\ 16 bit TCP cksum
  2 FIELD th_urp		\ 16 bit urgent offset
				\ options (if present) then data follows
END-STRUCT

\ TCP flag bit definitions. Unless otherwise marked, a set bit
\ does not affect the sequence numbering.
$01 equ	TCP_FIN		\ consumes a sequence number
$02 equ	TCP_SYN		\ consumes a sequence number
$04 equ	TCP_RST
$08 equ	TCP_PSH
$10 equ	TCP_ACK
$20 equ	TCP_URG

TCP_FIN TCP_ACK or equ TCP_FIN/ACK
TCP_SYN TCP_ACK or equ TCP_SYN/ACK
TCP_RST TCP_ACK or equ TCP_RST/ACK

(( Moved to structs.fth ; SFP040
STRUCT tcpcb	\ -- size
\ *G TCP control block structure is defined in *\i{structs.fth}.
  ...
END-STRUCT
))

\ define the TCP states. Some parts of PowerNet rely on the range
\ 0..4 being as below. DO NOT CHANGE unless you really know what
\ you are doing!
#0 equ	TCPS_CLOSED		\ closed
#1 equ	TCPS_LISTEN		\ listening for connection (passive open)
#2 equ	TCPS_SYN_SENT		\ have sent syn (active open)
#3 equ	TCPS_SYN_RECEIVED	\ have sent & received syn : awaiting ACK
#4 equ	TCPS_ESTABLISHED	\ established (data transfer)
#5 equ	TCPS_CLOSE_WAIT		\ received FIN, waiting for application close
#6 equ	TCPS_FIN_WAIT_1		\ have closed, sent FIN : awaiting ACK & FIN
#7 equ	TCPS_CLOSING		\ simultaneous close : awaiting ACK
#8 equ	TCPS_LAST_ACK		\ received FIN, have closed : awaiting ACK
#9 equ	TCPS_FIN_WAIT_2		\ have close : awaiting FIN
#10 equ	TCPS_TIME_WAIT		\ 2MSL wait state after active close
#11 equ #TcpStates		\ number of TCP states

\ define the TCP t_flags bits
$0001 equ TF_ACKNOW		\ ack peer immediately
$0002 equ TF_DELACK		\ ack, but try to delay it
$0004 equ TF_NODELAY		\ don't delay packets to coalesce
$0008 equ TF_NOOPT		\ don't use tcp options
$0010 equ TF_SENTFIN		\ have sent FIN
$0020 equ TF_REQ_SCALE		\ have/will request window scaling
$0040 equ TF_RCVD_SCALE		\ other side has requested scaling
$0080 equ TF_REQ_TSTMP		\ have/will request timestamps
$0100 equ TF_RCVD_TSTMP		\ a timestamp was received in SYN
$0200 equ TF_SACK_PERMIT	\ other side said I could SACK

: TcpState@	\ *sk -- state
\ *G Get socket state.
  so_*cb @ t_state w@
;

diags? [if]
: .tcpstate	\ n --
\ *G Display tcp socket state. Only compiled if DIAGS? is non-zero.
  case
    TCPS_CLOSED	      of  ." Closed"	    endof
    TCPS_LISTEN	      of  ." Listening"	    endof
    TCPS_SYN_SENT     of  ." Syn_Sent"	    endof
    TCPS_SYN_RECEIVED of  ." Syn_Received"  endof
    TCPS_ESTABLISHED  of  ." Connected"     endof
    TCPS_CLOSE_WAIT   of  ." Close_Wait"    endof
    TCPS_FIN_WAIT_1   of  ." Fin_Wait_1"    endof
    TCPS_CLOSING      of  ." Closing"	    endof
    TCPS_LAST_ACK     of  ." Last_Ack"	    endof
    TCPS_FIN_WAIT_2   of  ." Fin_Wait_2"    endof
    TCPS_TIME_WAIT    of  ." Time_Wait"     endof
  endcase
;

: ShowTcpState	\ *sk --
\ *G Display the state for the connection
  cr TcpState@ .tcpstate
;
[then]


\ **************************************
\ *S TCP structure creation and deletion
\ **************************************

: initTcpTxBuff	\ buff *cb --
\ *G Use *\i{buff} to initialise or reset the TCP transmit buffer.
  2dup *tx_buffer !			\ the transmit data buffer (snd_una)
  2dup *tx_in !				\ input  pointer at the beginning
  tuck *tx_out !			\ output pointer at the beginning
  tx_len off				\ & zero the data count
;

: -TcpTxBuff	\ *cb --
\ *G *\fo{FREE} the TCP transmit buffer and reset the transmit
\ ** pointers.
  dup *tx_buffer @ free drop  0 swap initTcpTxBuff
;

: goCLOSED	\ *sk --
\ *G Place socket in closed state.
  dup so_*cb @ if			\ if control block ok
    dup so_*cb @			\ -- *sk *cb
    dup t_connect off			\ stop timers
    dup t_disconnect off
    TCPS_CLOSED over t_state w!		\ new state = closed
    -TcpTxBuff				\ remove transmit buffer
  endif
  $FFFF swap so_state w!		\ mark closed for TCPIDLE.
;


\ SFP033..
: (CloseSocket)	\ *sk --
\ *G Prepare a socket for closure. Final closure and memory recovery
\ ** is performed in the TCP idle action to avoid race hazards.
[ TCPDEBUG? ] [if]
\  cr ." (CloseSocket): handle = " dup so_handle w@ .
\  ." from " r@ ip>nfa .name
[then]
  0 over so_portrem w!			\ ensure we don't have any dest to match
  0 over so_iprem !
  dup skTCP? if				\ is this a TCP socket?
    dup goCLOSED			\ SFP042
  endif
  $FFFF swap so_state w!		\ mark closed for TCPIDLE.
;
\ ...SFP033

\ SFP039...
: ChkTcpTxBuff	\ *cb -- ior
\ *G If the TCPCB does not have a transmit buffer, create it.
\ ** Return 0 if the buffer exists or has been created. We assume
\ ** that the TCPCB is valid.
  dup *tx_buffer @			\ check buffer exists
  if  drop 0 exit  endif		\ success ; SFP041
  TCPTXBUFFSIZE allocate		\ no, buffer, allocate it
  if  2drop -1 exit  endif		\ failed
  swap initTcpTxBuff
  0
;
\ ...SFP039


\ *****************
\ *S TCP header use
\ *****************

XC? [if] compiler [then]
: GetTCPHdrLen	\ *tcp -- len
\ *G Extract length of TCP header (varies with options).
  th_hdrlen c@ $F0 and			\ top 4 bits = no. of 32 bit units
  #2 rshift				\ convert to bytes ; 4 rshift 2 lshift
;
: GetTCPPktLen	\ *ip -- len ; SFP005
\ *G Extract length of TCP packet
  ip_len w@(n)				\ length of IP pkt
  ip_hdr -				\ iplen - iphdr = tcp pkt len
;
XC? [if] target [then]

: >TCPData	\ *tcp -- *tcpdata
\ *G Get pointer to TCP packet data.
  dup GetTCPHdrLen +			\ add dynamic header length
;

: GetTCPDataLen	\ *ip -- len ; SFP005
\ *G Extract length of TCP data packet.
  dup GetTCPPktLen			\ length of tcp packet
  swap ip_hdr + GetTCPHdrLen		\ get TCP header length
  - 					\ tcppktlen - tcphdr = tcp data len
;

: GetMSS	\ *tcp defmss -- mss ; SFP045
\ *G If the TCP header in a SYN or SYN/ACK packet contains an
\ ** MSS option return it, otherwise return the default value.
\ ** the output is limited to the range *\fo{0..defMSS}.
  swap dup GetTCPHdrLen		\ -- 536 *tcp hdrlen
  tcp_hdr /string 0 max bounds ?do	\ -- defmss
    case  i c@				\ inspect option number
      2 of				\ 2=MSS
          drop  i 2+ w@(n)  leave
      endof
      0 of  leave  endof		\ 0=end of options
      1 of  1  endof			\ 1=NOOP, step by one
          i 1+ c@ swap			\ step by length field
    endcase
  +loop
  #64 max defMSS min			\ clip range
;

: GetMSSsyn	\ *tcp -- mss ; SFP045
\ *G If the TCP header in a SYN packet contains an MSS option
\ ** return it, otherwise return the default value of 536.
\ ** Used by servers to read the incoming MSS in a SYN packet.
  #536 getMSS
;

: GetMSSsyn/ack	\ *tcp -- mss ; SFP045
\ *G If the TCP header in a SYN packet contains an MSS option
\ ** return it, otherwise return the default value of 536.
\ ** Used by clients to read the incoming MSS in a SYN/ACK packet.
  #536 getMSS
;


\ ************************
\ *S TCP checksum handling
\ ************************

: TcpCksum	\ *tcp tcplen iprem iploc -- cksum
\ *G A generic TCP checksum function. The parameters supplied
\ ** are used to form a 'pseudoheader' from which the actual
\ ** checksum is generated.
  IPPROTO_TCP PHcksum
;

: GenTcpCksum	\ *sk *tcp len --
\ *G Generate the TCP checksum using a prepared header and
\ ** socket data for the IP addresses. Len is the
\ ** length of the full TCP packet.
  rot >r
  over 0 swap th_sum w!(n)		\ zero the cksum BEFORE computing
  over swap				\ -- *tcp *tcp len ; R: -- *sk
  r> dup so_iprem @			\ remote ip address
  swap so_iploc @			\ local ip address
  TcpCksum 				\ go calculate it
  swap th_sum w!(n)			\ finally add the cksum to tcp pkt
;


\ ******************
\ *S TCP window size
\ ******************
\ *P The words in this section are the defaults for transmitting
\ ** the TCP receive window size to the remote end.

\ *P PowerNet queues input packets in the socket structure before
\ ** passing them to the BSD layer. Usually, ACKs are delayed.
\ ** There are three ways to implement receive window handling,
\ ** selected by the equate *\fo{genWinSize?} in the PowerNet
\ ** configuration file.
\ *(
\ *B *\fo{genWinSize?}=0. The receive window is always set
\ ** to *\fo{TCPWINDOWSIZE}.
\ *B *\fo{genWinSize?}=1.
\ ** By default, we assume that if all the input packets have
\ ** not been consumed by the time the ACK is sent, the system
\ ** is under heavy load and we do not want any more input for
\ ** the moment. This is indicated by setting the window size
\ ** to zero. Otherwise, the receive window is set to
\ ** *\fo{TCPWINDOWSIZE}. Although this "bang bang" approach is
\ ** very crude, it works well unless many packets are being sent
\ ** from PowerNet, as can happen when web pages are served from
\ ** memory.
\ *B *\fo{genWinSize?}=2. The word *\fo{genWindowSize ( *sk *tcp -- )}
\ ** is provided by you to suit your application. See *\i{tcp.fth}
\ ** for the examples.
\ *)

genWinSize? 0 = [if]		\ SFP047 SFP050
: genWindowSize	\ *sk *tcp --
\ *G Given a socket *\i{*sk} and a TCP header at *\i{*tcp},
\ ** calculate the TCP window size and place it in the TCP
\ ** header. This version is used when *\fo{genWinSize?}=0,
\ ** and implements the previous constant window size.
  swap so_*cb @ >r			\ control block
  TCPWINDOWSIZE dup			\ window size
  rot th_win w!(n)			\ to header
  r> rcv_lastwin !			\ to control block
;
[then]

genWinSize? 1 = [if]		\ SFP047 SFP050
: skMoreIp?	\ *sk -- flag
\ *G Return true if the socket has an input packet waiting.
  dup so_*cb @ t_state w@ TCPS_ESTABLISHED <>
  if  drop 0 exit  endif
  so_*inq @ dup
  if  PeekQ  endif
;

: NextWinSize	\ *sk -- size
\ *G Return the TCP Window size to be used for the packet send.
  skMoreIp?
  if 0 else TCPWINDOWSIZE endif
;

((
: checkRxWin	\ *sk --
 \ *G If the receive window is set to 0 and the delayed ACK timer
 \ ** is not running, start the delayed ACK timer.
  so_*cb @  dup rcv_lastwin @ 0= if	\ receive window = 0
    dup t_ackdelayed @ 0= if		\ ACK not set
      setAckDelay exit			\ set timer
    endif
  endif
  drop
;
))
: genWindowSize	\ *sk *tcp --
\ *G Given a socket *\i{*sk} and a TCP header at *\i{*tcp},
\ ** calculate the TCP window size and place it in the TCP
\ ** header. This version is used when *\fo{genWinSize?}=1,
\ ** and implements the "bang bang" control mechanism.
  swap dup so_*cb @ >r			\ control block
  NextWinSize dup			\ window size
  rot th_win w!(n)			\ to header
  r> rcv_lastwin !			\ to control block
;
[then]


\ ******************************
\ *S TCP transmission primitives
\ ******************************

XC? [if]  compiler  [then]
: #unsent	\ *cb -- n
\ *G Number of bytes waiting to be sent.
\ ** If no transmit buffer has been allocated, this returns 0.
  dup *tx_in @ swap *tx_out @ -
;

: #unacked	\ *cb -- n ; SBD011
\ *G Returns the number of sent bytes awaiting acknowledgement.
\ ** If no transmit buffer has been allocated, this returns 0.
  dup *tx_out @ swap *tx_buffer @ -
;
XC? [if]  target  [then]

\ SFP048...
: setAckDelay	\ *cb --
\ *G Set the ACK delay timer.
  ACKDELAYTIME later 1 or		\ non-zero target time
  swap t_ackdelayed !			\ restart the delayed ack timer
;

: checkRxWin	\ *sk --
\ *G If the receive window is set to 0 and the delayed ACK timer
\ ** is not running, start the delayed ACK timer.
  so_*cb @  dup rcv_lastwin @ 0= if	\ receive window = 0
    dup t_ackdelayed @ 0= if		\ ACK not set
      setAckDelay exit			\ set timer
    endif
  endif
  drop
;

: checkTxUnsent	\ *cb --
\ *G The TCP transmit primitive *\fo{SendTcpPkt) always clears
\ ** the delayed ACK timer. If there is no more data to send,
\ ** clear the delayed transmit flag and set the delayed ACK
\ ** timer if if the receive window is zero.
  dup #unsent 0= if			\ no more waiting to go?
    dup t_txdelayed off			\ so reset the delayed tx timer
    dup rcv_lastwin @ 0= if		\ if window=0
      dup setAckDelay			\ set ACK delay
    else
      dup t_ackdelayed off      	\ clear ACK delay
    endif
  endif
  drop
;
\ ...SFP048

: MakeTCPhdr	\ fl ol *sk *tcp --
\ *G Take data from the socket *\i{*sk} and create a TCP header
\ ** at *\i{*tcp}. The TCP header checksum is set to 0.
\ fl ol *sk *tcp -- ; from the socket structure
  over so_portloc w@
    over th_sport w!(n)			\ local port number
  over so_portrem w@
    over th_dport w!(n)			\ remote port number
\ fl ol *sk *tcp -- ; from the TCP control block
  swap dup >r so_*cb @ swap		\ retrieve control block pointer ; SFP047
  over snd_nxt @ over th_seq !(n)	\ initial seq number
  over rcv_nxt @ over th_ack !(n)	\ next expected seq no
  swap snd_up @ over th_urp w!(n)	\ urgent pointer
\ fl ol *tcp -- ; from the TCP control block
  swap tcp_hdr + 2 rshift 4 lshift	\ form hdrlen byte, in 32 bit units
    over th_hdrlen c!
  swap over th_flags c!			\ flags
  r> over genWindowSize			\ calculate Window Size in header ; SFP047
  0 swap th_sum w!(n)			\ zero the checksum BEFORE computing
;

[-short-branches
: SendTcpPkt { *sk fl *opt ol *dat dl | *pb *tcp pl -- res }
\ *G Transmit a TCP packet directly through the IP layer.
\ *D *sk points to the socket struct to be used
\ *D fl the flag bits to use
\ *D *opt,ol are the option pointer/length (if non-zero)
\ *D *dat, dl are the data pointer/length (if non-zero)
\ *D res = SOCKET_ERROR on failure, or no. of bytes sent
\ ** on success
[ tcpdebug? ] [if]  cr ." SendTCPPkt started"  [then]

  GetTxPbuf ?dup 0= if			\ get a buffer for the packet
\  #500 GetPbufWait ?dup 0= if		\ get a buffer for the packet
[ TCPDEBUG? ] [if] cr ." no TCP Tx buffer "  [then]
\    cr ." no TCP Tx buffer "
    SOCKET_ERROR  exit
  endif
  -> *pb				\ so save pointer to it

  *sk so_*cb @ t_maxseg @ dl min -> dl	\ clip to MSS.

  0 p_dat ether_hdr + ip_hdr +        \ skip Ether and IP headers
  dup *pb !*PBdata	              \ set buffer pointer to the TCP header
  *pb + -> *tcp                       \ this is the start of the tcp header
  tcp_hdr ol + dl + -> pl             \ total packet length
  pl *pb !PBDataLen	              \ set buffer char count for packet length

  fl ol *sk *tcp MakeTCPhdr           \ build the header but not the checksum yet

\ Add the header options and data
  ol if                               \ copy any optional data
    *opt *tcp tcp_hdr + ol cmove
  endif
  dl if                               \ and copy any data
    *dat *tcp tcp_hdr + ol + dl cmove
  endif

  *sk *tcp pl GenTcpCksum
  *sk so_*cb @ t_ackdelayed off       \ reset the delayed ack timer

  *sk *pb IPSend                      \ go send the IP packet & return result

[ tcpdebug? ] [if]  cr ." SendTCPPkt done " [then]
;
short-branches]

: SendTcpOpts	\ *sk fl *opt ol -- ; SFP005
\ *G Send a packet that consists of the flags
\ ** and options without data.
  0 0 SendTcpPkt drop
;

: SendTcpFlags	\ *sk fl -- ; SFP005
\ *G Send a packet that just consists of the flags,
\ ** without options or data.
  0 0 0 0 SendTcpPkt drop
;

: ForceTCPReset	\ *sk -- ; SFP012
\ *G Force a socket reset, and then close the socket.
  dup TCP_RST SendTcpFlags		\ so send reset
  TcpRstTime ms
  (CloseSocket)				\ forced closure ; SFP033
;

: sent<txbuff	\ *cb len --
\ *G Bump the transmit buffer after len bytes sent from txbuffer.
  over #unsent min dup if		\ bytes to send ??
    swap *tx_out +!			\ now bump the out pointer
  else   				\ EGS003
    2drop				\ ...EGS003
  endif
;

: SendBuff>TCP	{ *cb *sk | nb -- }
\ *G Send any buffered data out. No action is taken if
\ ** there is nothing to send. Use this only in the TCP
\ ** state machine.
\ EGS028...
  *cb snd_win @ *cb #unacked -		\ room left in receiver's window
  2- 0 max				\ leave 2 for IE6, but not < 0
  *cb #unsent min  ?dup 0=		\ anything to send?
  if  exit  endif
\ ...EGS028
  *cb ChkTcpTxBuff			\ ensure we have buffers ; SFP039
  if  *sk ForceTCPreset exit  endif

  TCPDATASIZE min -> nb  		\ EGS004 EGS009

[ TCPDEBUG? ] [if]
  cr ." sending tcp buffer " *cb snd_nxt @ .decimal space nb .dword
[then]
  *sk *cb t_override w@ TCP_ACK or 0 0 	\ EGS009 SFP010
  *cb *tx_out @ nb SendTcpPkt 0> if
    *cb nb sent<txbuff			\ bump out pointer
    nb *cb snd_nxt +!			\ bump snd_nxt
  endif
  TXRETRYTIME later 1 or *cb t_txretry !  \ so restart the tx retry timer
;

: SendAck>TCP	{ *cb *sk | nb -- }
\ *G Send any buffered data out with an ACK.
  *cb #unsent if 			\ now see if we have anything to send
    *cb *sk SendBuff>TCP
    *cb checkTxUnsent			\ check timers ; SFP048
\    *cb #unsent 0= if			\ no more waiting? ; EGS002...
\      *cb t_txdelayed off		\ so reset the delayed tx timer
\      *cb t_ackdelayed off      	\ & ack_delay
\    endif				\ ...EGS002
  else					\ no data so just send an Ack
    *sk TCP_ACK SendTcpFlags		\ send flag bits for ack ; SFP005
  endif
;

\ SFP036...
: skValid?	\ *sk -- flag
\ *G Returns true if the socket structure is a valid TCP socket.
  dup so_handle @ if			\ not in close process
    dup so_*cb @ if			\ has a control block. i.e. is TCP
      drop true  exit
    endif
  endif
  drop 0
;

: skCanSend?	\ *sk -- flag
\ *G Returns true if the socket structure is valid, the socket is
\ ** in TCPS_ESTABLISHED state and has not exceeded the maximum
\ ** number of retry attempts.
  dup so_handle @ if			\ not in close process
    dup so_*cb @ if			\ has a control block. i.e. is TCP
      so_*cb @
      dup t_state w@ TCPS_ESTABLISHED =	if
        dup t_#retries @ TCPMAXRETRIES <= if
          drop true  exit
        endif
      endif
    endif
  endif
  drop 0
;

: sendtoTCP	{ hsocket *bf len flags | bufflen *sk *cb nb -- res }
\ *G Move user data into the socket's transmit buffer, waiting
\ ** in the word until all data has been transferred to the
\ ** buffer. This may involve packet transmission while waiting
\ ** for the buffer to empty. Because the socket may be closed
\ ** during transmission, the socket must be checked before each
\ ** buffer fill operation.
\ ** Note that parts of this code must not be pre-empted by
\ ** the scheduler.
  hsocket get_socket_addr dup -> *sk	\ get socket address
  so_*cb @ -> *cb			\ get control block address
  len -> bufflen			\ preserve original buffer address
  begin
    *sk skCanSend? 0=
    if  SOCKET_ERROR exit  endif	\ failed
    *cb ChkTcpTxBuff
    if  SOCKET_ERROR exit  endif	\ failed
\ N.B. the below must NOT be interrupted by the TCP task
    *cb tx_len @ TCPTXBUFFSIZE < if	\ space in retransmission buffer?
      \ here we do the block move into the retransmission buffer
      *bf				\ source address
      *cb *tx_in @ 			\ dest address = start of data
      TCPTXBUFFSIZE *cb tx_len @ -	\ space left
      len min dup -> nb			\ compute bytes we can transfer
      cmove				\ copy nb bytes into txbuffer
      nb addr *bf +!			\ bump source address
      nb *cb *tx_in +!			\ bump dest buffer address
      nb *cb tx_len +!			\ bump lengths
      nb negate addr len +!
    endif
    len
  while
    0 *cb t_txdelayed !			\ force immediate transmission
\ N.B. the above must NOT be interrupted by the TCP task
    pause
  repeat
  *cb tx_len @ TCPDATASIZE >= 		\ buffer has one or more complete packets
  flags *cb t_override w@ or TCP_PSH and \ merge default flags, check PSH
  or if					\ No delay if PSH or large
    0 *cb t_txdelayed !			\ force immediate transmission
  else
    *cb t_txdelayed @ 0= if		\ data NOT already waiting ??
      TXDELAYTIME later 1 or		\ add new delay time
      *cb t_txdelayed !    		\ & start delay period
    endif
  endif
\ N.B. the above must NOT be interrupted by the TCP task
  bufflen
;
\ ...SFP036


\ ***********************
\ *S TCP state primitives
\ ***********************

compiler
: isTCP_Fin	\ *tcp -- T|F
\ *G Set if FIN flag bit is set.
  th_flags c@ TCP_FIN and
;

: isTCP_Syn	\ *tcp -- T|F ; set if flag bit is set
\ *G Set if SYN flag bit is set.
  th_flags c@ TCP_SYN and
;

: isTCP_Rst	\ *tcp -- T|F ; set if flag bit is set
\ *G Set if RST flag bit is set.
  th_flags c@ TCP_RST and
;

: isTCP_Psh	\ *tcp -- T|F ; set if flag bit is set
\ *G Set if PSH flag bit is set.
  th_flags c@ TCP_PSH and
;

: isTCP_Ack	\ *tcp -- T|F ; set if flag bit is set
\ *G Set if ACK flag bit is set.
  th_flags c@ TCP_ACK and
;

: isTCP_Urg	\ *tcp -- T|F ; set if flag bit is set
\ *G Set if URG flag bit is set.
  th_flags c@ TCP_URG and
;
target
XC? 0= [if]
: isTCP_Fin	\ *tcp -- T|F
  th_flags c@ TCP_FIN and
;

: isTCP_Syn	\ *tcp -- T|F ; set if flag bit is set
  th_flags c@ TCP_SYN and
;

: isTCP_Rst	\ *tcp -- T|F ; set if flag bit is set
  th_flags c@ TCP_RST and
;

: isTCP_Psh	\ *tcp -- T|F ; set if flag bit is set
  th_flags c@ TCP_PSH and
;

: isTCP_Ack	\ *tcp -- T|F ; set if flag bit is set
  th_flags c@ TCP_ACK and
;

: isTCP_Urg	\ *tcp -- T|F ; set if flag bit is set
  th_flags c@ TCP_URG and
;
[then]

: isTCP_SYN/ACK	\ *tcp -- flag ; SFP005
\ *G Set if SYN and ACK flags are both set.
  th_flags c@ TCP_SYN/ACK and TCP_SYN/ACK =
;

((
see RFC793, page 64
    If the state is CLOSED (i.e., TCB does not exist) then
      all data in the incoming segment is discarded.  An incoming
      segment containing a RST is discarded.  An incoming segment not
      containing a RST causes a RST to be sent in response.  The
      acknowledgment and sequence field values are selected to make the
      reset sequence acceptable to the TCP that sent the offending
      segment.

      If the ACK bit is off, sequence number zero is used,
        <SEQ=0><ACK=SEG.SEQ+SEG.LEN><CTL=RST,ACK>

      If the ACK bit is on,
        <SEQ=SEG.ACK><CTL=RST>

      Return.
))
[-short-branches
\ SFP038...
: SendNoSocket	{ *pb *tcp | *ip *sk *cb -- }
\ *G Handles sending the necessary RST replies when a connection
\ ** request is made for a TCP socket we don't know about.
\ ** Go send an RST message. The packet buffer is freed.
  NoTcpHandle @ ?dup if
    get_socket_addr -> *sk		\ convert to socket structure
    *sk so_*cb @ -> *cb			\ retrieve control block pointer
    IpAddress @(n) *sk so_iploc !	\ in case it's the first

    *tcp isTCP_RST 0= if		\ ignore RST packets
      *pb PBHdrStart dup -> *ip
      ip_src @(n)			\ get source IP address
      *sk so_iprem !			\  as socket rem address
      *tcp th_sport w@(n)		\ get the source port number
      *sk so_portrem w!			\   as socket rem port
      *tcp th_dport w@(n)		\ get the dest port number
      *sk so_portloc w!			\   as socket local port

      *tcp th_seq @(n) *cb rcv_nxt !	\ get incoming seq number into rcv seq
      *tcp isTCP_ACK if
        *tcp th_ack @(n) *cb snd_nxt !
        *sk TCP_RST SendTcpFlags
      else
        0 *cb snd_nxt !			\ SEQout=0
        *ip GetTCPDataLen *cb rcv_nxt +!  \ update rcv seq, ACK=SEQin+LEN
        *sk TCP_RST/ACK SendTcpFlags
      endif
      0 *sk so_portrem w!		\ lose the socket rem port
      0 *sk so_iprem !			\ and the remote ip address
    endif
    0 *sk so_portloc w!			\ finally lose the socket loc port
  endif
  *pb FreeQB
;

: set2MSL	\ *cb --
\ *G Start the 2xMSL timer for disconnection.
  TCPMSLTIME 2* later 1 or swap t_disconnect !  ;

\ SFP047...
: setIdle	\ *cb --
\ *G Start the disconnect timer with the IDLE period.
  TCPIDLETIME later 1 or swap t_disconnect !  ;

: setConn	\ *cb --
\ *G Start the connect timer with the TCPCONNECTTIME period.
  TCPCONNECTTIME later 1 or swap t_connect !  ;
\ ...SFP047

: goSYN_REC	\ *sk --
\ *G Put the given socket into TCPS_SYN_RECEIVED state.
\ ** This transition is performed by a server.
  { | tmp[ 4 ] -- }
[ TCPDEBUG? ] [if] cr ." Enter SYN_RECEIVED state " [then]
  $0204 tmp[ w!(n)			\ 4 byte MSS option
  dup so_*cb @ t_maxseg @ tmp[ 2+ w!(n)	\ segment size
  dup TCP_SYN/ACK  tmp[ 4 SendTcpOpts	\ send standard option field
  so_*cb @
  TCPS_SYN_RECEIVED over t_state w!	\ new state = syn_recvd
  setConn				\ connection timeout ; SFP047
\  TCPCONNECTTIME later 1 or swap t_connect ! \ connection timeout
;

: goESTABLISHED	\ *sk --
\ *G Move the socket into ESTABLISHED state.
[ TCPDEBUG? ] [if] cr ." Enter ESTABLISHED state " [then]
  so_*cb @
  dup t_connect off			\ stop connect timer
  dup setIdle				\ start the idle disconnect timer ; SFP047
  TCPS_ESTABLISHED swap t_state w!	\ new state = established
;

: goLAST_ACK	\ *sk --
\ *G Move the socket into LAST_ACK state from CLOSE_WAIT.
  dup TCP_FIN/ACK SendTcpFlags		\ auto-shutdown app ; SFP005
  so_*cb @  1 over snd_nxt +!		\ FIN consumes a sequence number ; SFP037
  TCPS_LAST_ACK over t_state w!  set2MSL
;

: goTIME_WAIT	\ *sk --
\ *G Move the socket into TIME_WAIT state.
  so_*cb @  TCPS_TIME_WAIT over t_state w!  set2MSL
;

: goCLOSING	\ *sk --
\ *G Move the socket into CLOSING state from FIN_WAIT1
  so_*cb @  TCPS_CLOSING over t_state w!  set2MSL
;

: goFIN_WAIT_1	\ *sk --
\ *G Move socket into FIN_WAIT_1 state from SYN_RECVD or ESTABLISHED.
  dup TCP_FIN/ACK SendTcpFlags
  so_*cb @  1 over snd_nxt +!		\ FIN consumes a sequence number ; SFP037
  TCPS_FIN_WAIT_1 over t_state w!	\ new state = Fin wait 1
  dup set2MSL
  -TcpTxBuff				\ early release of TCP TX buffer
;

: goFIN_WAIT_2	\ *sk --
\ *G Move the socket into FIN_WAIT_2 state.
  so_*cb @  TCPS_FIN_WAIT_2 over t_state w!	\ SFP046
  set2MSL
;

: goCLOSE_WAIT	\ *sk --
\ *G Move the socket into CLOSE_WAIT state after receiving a FIN
\ ** in ESTABLISHED.
  dup TCP_ACK SendTcpFlags		\ ACK rx of FIN ; SFP005
  so_*cb @  TCPS_CLOSE_WAIT over t_state w!
  dup set2MSL
  -TcpTxBuff				\ early release of TCP TX buffer
;

: goSYN_SENT	\ *sk --
\ *G Move the socket into SYN_SENT state from CLOSED.
  dup TCP_SYN MSSopt 4 SendTcpOpts	\ send segment size option ; SFP045
  so_*cb @
  dup setConn				\ connection timeout
\  TCPCONNECTTIME later 1 or over t_connect !	\ connection timeout
  TCPS_SYN_SENT swap t_state w!
;


\ ***************************
\ *S LISTEN connection queues
\ ***************************
\ *P From PowerNet v4.8 onwards, listening sockets save
\ ** connection attempts (receipt of SYN) on a linked list of
\ ** connection items. When the application, e.g. a Telnet client,
\ ** wants to use a connection, it inspects the queue, and
\ ** transfers the next item on the list to a new socket in
\ ** SYN_RECEIVED state. The new socket waits until the socket
\ ** gets to ESTABLISHED state at which point the new connection
\ ** can be used to transfer data.

\ *P Compared to the previous design, listening sockets use less
\ ** RAM and, because connection attempts are stored, the design
\ ** is kinder to single-threaded servers.

\ SFP042...
struct /ConnItem	\ -- len
\ *G Holds unprocessed connection information after a SYN packet
\ ** has been received by a listening socket. All entries are in
\ ** native order.
  cell field ci.link	\ link field, must be first
  4 field ci.seq#	\ starting sequence number, must be second
  4 field ci.mss	\ MSS option from IP header
\ The following fields can be directly compared to permit
\ retries to be detected.
  4 field ci.iprem	\ remote IP address
  4 field ci.iploc	\ local IP address (allows for routing)
  2 field ci.portrem	\ remote port
  2 field ci.portloc	\ local port (allows for routing)
end-struct

diags? [if]
: .ci		\ *ci --
\ *G Display the connection queue item.
  cr       dup ci.iprem .iploc  ." /"  dup ci.portrem w@ .
  ." -> "  dup ci.iploc .iploc  ." /"  ci.portloc w@ .
;

: .cq		\ hs --
\ *G Display the socket's connection queue.
  get_socket_addr dup if
    so_*cb @ dup if
      listen_q
      begin  @ dup while  dup .ci  repeat
    endif
  endif
  drop
;
[then]

: TCPConnQ?	\ *sk -- flag
\ *G Return true if there is a connection on the queue.
  so_*cb @ dup if
    listen_q @ 0<>
  endif
;

: #TCPConnQ	\ *sk -- n
\ *G Return the number of connections on the queue.
  so_*cb @ dup if
    0 swap listen_q			\ -- n *ci
    begin  @ dup while  swap 1+ swap  repeat	\ count
    drop
  endif
;

\ SFP044...
: ciSame?	\ *ci1 *ci2 -- flag
\ *G Return true if the two connection items are for the same
\ ** connection, i.e. a retry has has occurred.
  swap ci.iprem swap ci.iprem		\ point to start of blocks
  /ConnItem 0 ci.iprem -		\ length of block
  s=
;

: onConnQ?	\ *ci *sk -- flag
\ *G Return true if the connection is already on the socket's
\ ** connection queue.
  so_*cb @ listen_q begin
    @ dup
   while				\ -- *ci *cix
    2dup ciSame?
    if  2drop -1 exit  endif
  repeat
  2drop 0
;

: >TCPConnQ	\ *tcp *sk --
\ *G Add an entry to a listening socket's connection queue.
\ ** On failure no action is taken and we assume that the
\ ** connecting socket will retry.
  dup #TCPConnq /ListenQ >= if		\ check max connections
    2drop exit				\ discard if too many
  endif
  /ConnItem allocate if
    3drop exit				\ -- *tcp *sk *ci | --
  endif
  2dup 2>r
  over so_iploc @ over ci.iploc !	\ local IP address and port
  swap so_portloc w@ over ci.portloc w!	\ -- *tcp *ci
  over ip_hdr - ip_src @(n) over ci.iprem !	\ remote IP address and port
  over th_sport w@(n) over ci.portrem w!
  over th_seq @(n) 1+ over ci.seq# !	\ initial sequence number
  swap getMSSsyn swap ci.mss !		\ MSS option
  2r> swap  2dup onConnQ? if		\ -- *ci *sk ; already on queue?
    drop  free drop  exit		\ discard item
  endif
  so_*cb @ listen_q AddEndLink		\ add item to end of queue
\ [con  cr ." >ConnQ good"  con] 2 ms
;
\ ...SFP044

: TCPConnQ>	\ *skl *ska --
\ *G Take a connection from the listening socket *\i{*skl} and
\ ** transfer it to the accepting socket *\i{*ska} which enters
\ ** TCPS_SYN_RECEIVED state.
  swap so_*cb @ listen_q dup @ 		\ -- *ska lq *ci
  tuck swap delLink  swap		\ -- *ci *ska ; remove connection from queue
  over ci.iprem @ over so_iprem !	\ copy data
  over ci.iploc @ over so_iploc !
  over ci.portrem w@ over so_portrem w!
  over ci.portloc w@ over so_portloc w!
  over ci.seq# @ over so_*cb @ rcv_nxt !
  over ci.mss @ over so_*cb @ t_maxseg !
\ cr ." SYN sets MSS to " dup so_*cb @ t_maxseg @ .decimal
  swap free drop  goSYN_REC		\ discard connection, change state
;
\ ...SFP042


\ *********************
\ *S TCP state handlers
\ *********************

: DoTcpClosed	{ *pb *sk | *ip *tcp *cb -- }
\ *G Handle incoming packet while in CLOSED state.
  *pb PBHdrStart -> *ip
  *ip ip_hdr + -> *tcp
  *sk so_*cb @ -> *cb			\ retrieve control block pointer
  *tcp isTCP_RST 0= if			\ EGS006
    *tcp th_seq @(n) *cb rcv_nxt !	\ get incoming seq number into rcv seq
    *tcp isTCP_ACK if
      *tcp th_ack @(n) *cb snd_nxt !
      *sk TCP_RST SendTcpFlags
    else
      0 *cb snd_nxt !			\ SEQout=0
      *ip GetTCPDataLen *cb rcv_nxt +!  \ update rcv seq, ACK=SEQin+LEN
      *sk TCP_RST/ACK SendTcpFlags
    endif
    0 *sk so_portrem w!			\ now ensure we don't have any dest to match
    0 *sk so_iprem !
  endif
  *pb FreeQB
;
\ ...SFP038

: DoTcpListen	{ *pb *sk | *tcp *cb -- }
\ *G Handle incoming packet while in listening state (server).
[ TCPDEBUG? ] [if] cr ." DoTcpListen started" [then]
  *pb PBHdrStart ip_hdr + -> *tcp
  *tcp isTCP_Syn if			\ incoming SYN?
    *tcp *sk >TCPConnQ			\ place it on connection queue
    *pb FreeQB
  else
    *pb *tcp SendNoSocket		\ reject all others
  endif
[ TCPDEBUG? ] [if] cr ." DoTcpListen done" [then]
;

: DoTcpSynReceived	{ *pb *sk | *tcp *cb -- }
\ *G Handle incoming packet while in SYN received state (server).
  *pb PBHdrStart ip_hdr + -> *tcp
  *tcp isTCP_Rst if			\ check for a reset ; SFP043
    *pb FreeQB  *sk (CloseSocket)  exit
  endif
  *tcp isTCP_Ack if
    *sk so_*cb @ -> *cb			\ retrieve control block pointer
    *tcp th_seq @(n) *cb rcv_nxt !	\ incoming seq number into rcv seq
    *tcp th_ack @(n)			\ get incoming ack number
    dup *cb snd_nxt !  *cb snd_una !	\ into send seq and initial una
    *sk goESTABLISHED			\ SFP042
  endif
  *pb FreeQB
;

: DoTcpSynSent	{ *pb *sk | *tcp *cb -- }
\ *G Handle incoming packet while in SYN sent state (client).
[ TCPDEBUG? ] [if]
    cr ." DoTcpSynSent " ( *pb $50 dump )
[then]
  *pb PBHdrStart ip_hdr + -> *tcp
  *tcp isTCP_SYN/ACK if
    *sk so_*cb @ -> *cb			\ retrieve control block pointer

    *tcp getMSSsyn/ack *cb t_maxseg !	\ get MSS from SYN/ACK ; SFP045
\ cr ." SYN/ACK sets MSS to " *cb t_maxseg @ .decimal
    *tcp th_seq @(n) 1+			\ incoming seq number +1 for SYN
    *cb rcv_nxt !			\ into rcv seq
    *tcp th_ack @(n)			\ get incoming ack number
    dup *cb snd_nxt !			\ resync una with nxt
    *cb snd_una !

    *cb *sk SendAck>TCP			\ so send ack
    *sk goESTABLISHED			\ SFP042
  endif
  *pb FreeQB
;
short-branches]

: acktxbytes	\ *cb nb -- nb' ; SBD011
\ *G Removes n bytes from the transmit buffer.
[ TCPDEBUG? ] [IF]
\  cr *cb .hex  nb .hex
\  ." Task: " up@ .hex
[THEN]
  0 max over #unacked min  dup		\ -- *cb nb' nb' ; range check
  if  over t_#retries off  endif	\ ack, so zero retry counter ; RMS025
  over tx_len @ over - >r		\ num bytes to be moved
  over *tx_buffer @ dup 2 pick + swap	\ from here to start
  r> move				\ shift data down
  tuck negate swap			\ -- nb' -nb' *cb
  2dup *tx_out +!			\ reduce out pointer
  2dup *tx_in +!			\ reduce in pointer
  tx_len +!				\ reduce numbytes in buffer
;

: DoTcpAcknowledge	{ *tcp *cb *sk | nb -- }
\ *G Handles incoming ACK. See Wright & Stevens about ack numbers.
[ TCPDEBUG? ] [IF] ." Ack " [then]
  *tcp th_win w@(n)			\ receiver's window size ; EGS028
  *cb snd_win !				\ save as send window ; EGS028
  *tcp th_ack @(n)			\ get incoming ack number
  *cb snd_una @ -			\ how many bytes being ack'd
  dup 1 <				\ check for duplicate or out of order ACK ; EGS026
  if  drop  exit  endif			\ EGS026
  -> nb					\ how many bytes ack'd ; EGS017 SFP037
  *cb ChkTcpTxBuff			\ check TX buffer ; SFP039
  if  *sk ForceTCPreset exit  endif	\ failed
  *cb nb acktxbytes			\ remove acknowledged bytes from tx buffer
  *cb snd_una +!			\ set new unacknowledged seq
\ Comment out next line?
\  *cb *sk SendBuff>TCP			\ now send any waiting data with the ACK
  *cb snd_una @				\ set if unacknowledged seq
  *cb snd_nxt @	=			\ is same as nxt seq
  if  *cb t_txretry off  endif		\ reset the retry counter as there's no more data
[ TCPDEBUG? ] [if] ( *cb .tcpcb ) [then]
;

[-short-branches
: DoTcpEstablished	{ *pb *sk | *ip *tcp *cb tdlen -- }
\ *G Handle incoming packet while in established state.
[ TCPDEBUG? ] [if] cr ." DoTcpEstablished : " [then]
  *pb PBHdrStart -> *ip
  *ip ip_hdr + -> *tcp
  *ip GetTCPDataLen -> tdlen		\ length of TCP data
  *sk so_*cb @ -> *cb			\ retrieve control block pointer
  *cb setIdle				\ restart the idle disconnect timer ; EGS015 SFP047
\  TCPIDLETIME later *cb t_disconnect !	\ restart the idle disconnect timer ; EGS015
  *tcp isTCP_Syn if			\ EGS002...
[ TCPDEBUG? ] [if] cr ." Est SYN_RST" [then]
    1 *cb rcv_nxt +! 			\ SYN uses up 1 byte, send ack+1 ; SFP037
    *sk ForceTCPReset			\ SFP012
    *pb FreeQB				\ and free up this buffer
    exit
  endif					\ ...EGS002
  *tcp isTCP_Rst if			\ then check for a reset
    *sk (CloseSocket)			\ Forced closure ; SFP033
    *pb FreeQB				\ and free up this buffer
    exit				\ EGS002
  endif
  *tcp isTCP_Fin if
    tdlen if				\ there might be some final data
[ TCPDEBUG? ] [if]
      cr ." RxData_Last" tdlen .decimal space
      *tcp th_seq @(n) .hex ."  "
[then]
      *tcp th_seq @(n) tdlen +		\ get incoming seq number + num bytes
      *cb rcv_nxt !			\ into rcv seq
      *pb *sk so_*inq @ enqueue		\ add data to queue
    else
      *pb FreeQB
    endif				\ ...EGS002
    1 *cb rcv_nxt +! 			\ FIN uses up 1 byte, send ack+1 ; MGS029
    *sk goCLOSE_WAIT			\ sends ACK, now in CLOSE_WAIT
    pause  *sk skValid?			\ socket may change, so validate ; SFP036
    if  *sk goLAST_ACK  endif		\ now to LAST_ACK, sends FIN ; SFP042
    exit
  endif

  tdlen if				\ if len non-zero
[ TCPDEBUG? ] [if]  cr ." RxData : "  [then]
    \ See if this is the expected segment
    *tcp th_seq @(n) *cb rcv_nxt @ = if
[ TCPDEBUG? ] [if]  ." Next pkt "  [then]
      *tcp th_seq @(n) tdlen +		\ get incoming seq number + num bytes
      *cb rcv_nxt !			\ into rcv seq
      *cb setAckDelay			\ restart the delayed ack timer ; SFP048
\      ACKDELAYTIME later *cb t_ackdelayed !	\ restart the delayed ack timer
      *tcp *cb *sk DoTcpAcknowledge	\ Go handle the ack
      *pb *sk so_*inq @ enqueue		\ finally queue packet for handling
      exit
    endif
    \ Not the expected segment. If it was the previous one, an
    \ outgoing ACK has been lost.
    *tcp th_seq @(n) *cb rcv_nxt @ - 0<	\ less than the expected segment ??
    if					\ yes so it MUST be a retransmission
[ TCPDEBUG? ] [if]  ." Retransmission, seq=" *tcp th_seq @(n) u.  [then]
      *cb rcv_nxt @ tdlen -	 	\ EGS002
      *tcp th_seq @(n) = if		\ MPE022
[ TCPDEBUG? ] [if]  ." try Re-ack "  [then]
        *cb setAckDelay			\ restart the delayed ack timer ; SFP048
\        ACKDELAYTIME later *cb t_ackdelayed !  \ restart the delayed ack timer
        *tcp *cb *sk DoTcpAcknowledge	\ Go handle the ack
      else
[ TCPDEBUG? ] [if]  ." dropped "  [then]
      endif				\ ...EGS002
      *pb FreeQB  exit			\ silently discard the packet
    endif
[ TCPDEBUG? ] [if]  ." Early segment "  [then]
    *pb FreeQB  exit			\ silently discard the packet
  endif

\ If we get here, there is no data in the received packet.
  *tcp isTCP_Ack if			\ so is it an ACK?
\   ACKDELAYTIME later *cb t_ackdelayed !	\ restart the delayed ack timer EGS007, del EGS013
    *tcp *cb *sk DoTcpAcknowledge	\ Go handle the ack
    *pb FreeQB  exit
  endif

\ If we get here we cannot parse the packet.
[ TCPDEBUG? ] [if]  ." Unknown packet, dropped "  [then]
  *pb FreeQB
;
short-branches]

: DoTcpLastAck	{ *pb *sk | *tcp *cb -- }
\ *G Handle incoming packet while in last ACK state.
  *pb PBHdrStart ip_hdr +
  dup -> *tcp isTCP_Ack
  *tcp isTCP_Rst or if			\ EGS002
    *sk so_*cb @ -> *cb			\ retrieve control block pointer
    *tcp th_seq @(n)			\ get incoming seq number
    *cb rcv_nxt !			\ into rcv seq
    *sk (CloseSocket)			\ new state = closed ; SFP030 SFP033 SFP035
  endif
  *pb FreeQB
;

\ SFP049...
[-short-branches
: DoTcpFinWait1	{ *pb *sk | *ip *tcp *cb -- }
\ *G Handle incoming packet while in FIN_WAIT1 state.
  *pb PBHdrStart -> *ip
  *ip ip_hdr + -> *tcp
  *sk so_*cb @ -> *cb			\ retrieve control block pointer ; EGS002

  *tcp th_seq @(n)			\ get incoming seq number
  *ip GetTCPDataLen +			\ + data bytes in this packet
  *cb rcv_nxt !				\ into rcv seq to drop incoming data

\ EGS018...
  *tcp isTCP_Rst if			\ waiting for incoming FIN ; EGS002
    *sk (CloseSocket)			\ Forced closure ; EGS002 SFP030 SFP033
    *pb FreeQB  exit			\ SFP030
  endif
\ ...EGS018

  *tcp isTCP_Fin if			\ simultaneous close
    1 *cb rcv_nxt +!			\ FIN consumes sequence number ; SFP037
    *sk TCP_ACK SendTcpFlags		\ return ACK ; SFP037
    *tcp isTCP_Ack if			\ ack received as well?
       *sk goTIME_WAIT
    else
       *sk goCLOSING			\ SFP042
    endif
    *pb FreeQB exit
  endif

  *tcp isTCP_Ack if
    *tcp th_ack @(n)			\ get incoming ack number
    *cb snd_nxt !			\ into send seq for next send
    *sk goFIN_WAIT_2			\ SFP042
  endif
  *pb FreeQB
;

: DoTcpFinWait2	{ *pb *sk | *ip *tcp *cb -- }
\ *G Handle incoming packet while in FIN_WAIT2 state.
  *pb PBHdrStart dup -> *ip
  ip_hdr + -> *tcp
  *sk so_*cb @ -> *cb			\ retrieve control block pointer EGS002
  *tcp isTCP_Rst if			\ waiting for incoming FIN EGS002
    *sk (CloseSocket)			\ Forced closure ; EGS002 SFP030 SFP033
    *pb FreeQB  exit
  endif
  *tcp isTCP_SYN if
    1 *cb rcv_nxt +!			\ SYN consumes a sequence number ; SFP037
    *sk ForceTCPreset			\ SFP012
    *pb FreeQB  exit
  endif
  *tcp isTCP_Fin if			\ waiting for incoming FIN
\ cr ." doTcpFinWait2, got FIN"
    *tcp th_seq @(n) 1+			\ incoming seq number +1 for FIN
    *ip GetTCPDataLen +			\ size of TCP data
    *cb rcv_nxt !			\ into rcv seq
    *tcp isTCP_ACK if
      *tcp th_ack @(n) 			\ get incoming ack number
      *cb snd_nxt !			\ into send seq for next send
    endif
    *sk TCP_ACK SendTcpFlags		\ SFP037
    *sk goTIME_WAIT			\ SFP042
  endif
  *pb FreeQB
;
short-branches]

: DoTcpClosing	{ *pb *sk | *tcp *cb -- }
\ *G Handle incoming packet while in closing state.
  *pb PBHdrStart ip_hdr + -> *tcp	\ TCP header
  *sk so_*cb @ -> *cb			\ retrieve control block pointer

  *tcp isTCP_Rst if			\ waiting for incoming FIN
    *sk (CloseSocket)			\ Forced closure
    *pb FreeQB  exit
  endif

  *tcp isTCP_Ack if
    *tcp th_seq @(n)			\ get incoming seq number
    *cb rcv_nxt !			\ into rcv seq
    *tcp th_ack @(n)			\ get incoming ack number
    *cb snd_nxt !			\ into send seq for next send
    *sk goTIME_WAIT			\ SFP042
  endif

  *pb FreeQB
;
\ ...SFP049

: DoTcpCloseWait	\ *pb *sk --
\ *G Handle incoming packet while in CLOSE_WAIT state.
  drop  FreeQB
;

: DoTcpTimeWait	{ *pb *sk | *tcp *cb -- }
\ *G Handle incoming packet while in timed wait state.
  *pb PBHdrStart ip_hdr + -> *tcp	\ EGS002...
  *sk so_*cb @ -> *cb			\ retrieve control block pointer
  *tcp isTCP_Rst if
    TcpRstTime ms			\ JGM
    *sk (CloseSocket)			\ immediate forced closure ; SFP030 SFP033
  else
    *tcp isTCP_SYN if
      1 *cb rcv_nxt +!			\ SYN consumes a sequence number ; SFP037
      *sk ForceTCPreset			\ SFP012
    endif
  endif					\ ...EGS002
  *pb FreeQB
;

[-short-branches
: IncTcpPacket	{ *pb hs | *sk *cb -- }
\ *G Process an incoming packet destined for hs.
  hs get_socket_addr dup -> *sk 0= 	\ socket invalid?
  if  *pb FreeQB  exit  endif
  *sk so_*cb @ dup -> *cb 0=		\ check for a control block
  if  *pb FreeQB  exit  endif
[ TCPDEBUG? ] [if]  cr ." TcpIn, hs=" hs . *sk ShowTcpState  [then]
  *cb t_state w@			\ get tcp STATE
  case
    TCPS_CLOSED       of  *pb *sk DoTcpClosed       endof
    TCPS_LISTEN       of  *pb *sk DoTcpListen       endof
    TCPS_SYN_SENT     of  *pb *sk DoTcpSynSent      endof
    TCPS_SYN_RECEIVED of  *pb *sk DoTcpSynReceived  endof
    TCPS_ESTABLISHED  of  *pb *sk DoTcpEstablished  endof
    TCPS_CLOSE_WAIT   of  *pb *sk DoTcpCloseWait    endof
    TCPS_FIN_WAIT_1   of  *pb *sk DoTcpFinWait1     endof
    TCPS_CLOSING      of  *pb *sk DoTcpClosing      endof
    TCPS_LAST_ACK     of  *pb *sk DoTcpLastAck      endof
    TCPS_FIN_WAIT_2   of  *pb *sk DoTcpFinWait2     endof
    TCPS_TIME_WAIT    of  *pb *sk DoTcpTimeWait     endof
  endcase
[ TCPDEBUG? ] [if]  *sk ShowTcpState  [then]	\ just display the state
;
short-branches]

[-short-branches
XC? [if]
: RxTCPPacket	{ *pb | *ip *tcp hs *q bf -- }
\ *G Process an incoming TCP packet.
[else]
: rtRxTCPPacket	{ *pb | *ip *tcp hs *q bf -- }
[then]
  *pb PBHdrStart -> *ip
  *ip ip_hdr + -> *tcp
[ TCPDEBUG? ] [if]  cr ." RxTCPPacket started"  [then]
\ EGS008...
\ check incoming tcp cksum
  *tcp *ip GetTCPPktLen			\ *tcp pktlen
  *ip ip_src @(n)			\ get source address
  *ip ip_dest @(n)			\ get dest address
  TcpCksum if				\ go calculate cksum (good pkt = 0)
[ TCPDEBUG? ] [if]  cr ." Bad incoming TCP cksum"  [then]
    *pb FreeQB  exit			\ so just lose all the data & quit
  endif
\ ...EGS008

  *ip ip_src @(n)			\ get source address
  *tcp th_sport w@(n)			\ and source port number
  *ip ip_dest @(n)			\ get dest address
  *tcp th_dport w@(n)			\ now get the destination port number
  find_hsocket_tcpport			\ see if we have a matching socket/port
  ?dup if				\ OK so split buffer into queues
    -> hs				\ save socket handle
    hs get_socket_inq
    dup -> *q if	\ convert to *q
[ TCPDEBUG? ] [if] cr ." Have input queue" [then]
      *tcp >TCPData *pb - 		\ offset to data start
      *pb !*PBdata			\ set data pointer into buffer
      *ip GetTCPDataLen
      *pb !PBDataLen			\ data length
      *pb hs IncTcpPacket		\ so handle incoming packet for this socket
    else
[ TCPDEBUG? ] [if] cr ." No input queue!" [then]
      *pb FreeQB			\ no *q so just lose all the data
    endif
  else
    ip_hdr *pb unbumpPBuffData		\ set data to include ip hdr again
    *pb CanRouteIP? if			\ No socket so try to route the packet
[ TCPDEBUG? ] [if] cr ." Routing input " [then]
      *pb RouteIP
    else
[ TCPDEBUG? ] [if] cr ." No Tcp Socket!!" [then]
      *pb *tcp SendNoSocket		\ go send no socket message
    endif
  endif
[ TCPDEBUG? ] [if] flushDebug cr ." RxTCPPacket done"  [then]
;
XC? 0= [if]  assign rtRxTCPPacket to-do RxTCPPacket  [then]
short-branches]


\ *********************
\ *S TCP timer handling
\ *********************

: DoTcpRetry?	\ *cb *sk -- ; SFP021
\ *G Send data if tx retry timer expiration.
  dup skValid? 0=
  if  2drop  exit  endif
  over t_txretry @ ?dup 0= if		\ waiting to send?
    SendBuff>TCP			\ so send buffered data
  else
    timedout? if			\ timeout expired?
      swap				\ -- *sk *cb
      dup t_txretry off			\ prevent second pass ; VRW034
      dup t_#retries incr		\ RMS025
      dup t_#retries @ TCPMAXRETRIES < if
        dup *tx_buffer @ over *tx_out !	\ reset buffer back to start
        dup snd_una @  over snd_nxt !	\ then use snd_una as start ; SFP037
        swap SendBuff>TCP		\ so send buffered data
      else
        drop (CloseSocket)
      endif
    else
      2drop
    endif
  endif
;

: DoTcpDelayedData?	\ *cb *sk -- ; SFP021
\ *G Send data if the transmit delay timer has expired.
  dup skValid? 0=
  if  2drop  exit  endif
  over t_txdelayed @ ?dup 0= if		\ waiting to send?
    SendBuff>TCP			\ so send buffered data
  else
    timedout? if			\ timeout expired yet?
      over swap SendBuff>TCP		\ so send buffered data
      dup checkTxUnsent			\ set/clear timers ; SFP048
\      dup #unsent 0= if			\ no more waiting?
\        dup t_txdelayed off		\ so reset the delayed tx timer
\      endif
      drop
    else
      2drop
    endif
  endif
;

: DoTcpDelayedAck?	\ *cb *sk -- ; SFP021
\ *G Send ack/data if ack delay timer expiration.
  dup skValid? 0=
  if  2drop  exit  endif
  over t_ackdelayed @ dup if		\ something to send?
    timedout? if			\ -- *cb *sk ; timeout expired?
      tuck SendAck>TCP			\ -- *sk ; send ack (and data if any) ; SFP048
      checkRxWin			\ reset the delayed ack timer ; SFP048
\      over swap SendAck>TCP		\ send ack (or buffered data if any)
\      t_ackdelayed off			\ reset the delayed ack timer
    else
      2drop
    endif
  else
    3drop
  endif
;

: DoTcpDisconDelay	\ *cb *sk -- ; SFP021
\ *G Handle disconnect delay timer expiration.
  dup skValid? 0=
  if  2drop  exit  endif
  over t_disconnect @ dup 0= if		\ timer not set?
    3drop
  else
    timedout? if			\ -- *cb *sk ; timed out?
      swap t_disconnect off		\ clear the timer ; SFP030
      (CloseSocket)			\ new state = closed ; SFP030 SFP033
    else
      2drop
    endif
  endif
;

: DoTcpConnectTimeout	\ *cb *sk -- ; SFP021
\ *G Handles timeout for failed connections.
  dup skValid? 0=
  if  2drop  exit  endif
  over t_connect @ dup 0= if		\ only if timeout is running
    3drop
  else
    timedout? if			\ -- *cb *sk ; timed out?
      swap t_connect off		\ MUST stop timeout ; SFP030
      (CloseSocket)			\ new state = closed ; SFP030 SFP033
    else
      2drop
    endif
  endif
;

[undefined] TcpIdleMs [if]
#5 tick-ms max equ TcpIdleMs	\ -- ms ; SFP030
\ *G Interval in miliseconds between TCP idle checks. Using this
\ ** timer mechanism reduces the CPU load at the expense of
\ ** transmission performance on local networks. Setting the
\ ** value of *\fo{TcpIdleMs} to zero turns the mechanism off.
\ ** The value set here in *\i{tcp.fth} is only used if
\ ** *\fo{TcpIdleMs} has not been defined in your *\i{PNconfig.fth}
\ ** file.
[then]

TcpIdleMs [if]
0 value TcpIdleTimer	\ -- time ; SFP030
\ *G Time for next TCPIDLE check.
[then]

[-short-branches
: TcpIdle	{ | *sk *cb -- }
\ *G Handles TCP idle time processing.
[ TcpIdleMs ] [if]
  TcpIdleTimer timedout? 0=
  if  exit  endif
  TcpIdleMs later 1 or -> TcpIdleTimer
[then]
  MAXSOCKETS 1+ 1 do			\ sockets are numbered 1..n ; SFP030
    i get_socket_addr ?dup if		\ socket available?
      dup -> *sk			\ so save address
      so_state w@ $FFFF = if		\ final shut down check
        *sk FreeupSocket		\   releases socket memory
      else
        *sk skTCP? if			\ is this a tcp socket ??
          *sk so_*cb @ ?dup if		\ retrieve control block pointer
            -> *cb
            *cb t_state w@ TCPS_LISTEN <> if 	\ EGS001
              *cb *sk DoTcpDelayedData?		\ handle delayed data timer
              *cb *sk DoTcpDelayedAck?		\ handle delayed ack timer
              *cb *sk DoTcpRetry?		\ handle tx retries timer
              *cb *sk DoTcpConnectTimeout	\ handle bad connections
	      *cb *sk DoTcpDisconDelay		\ handle disconnect timeout
            endif				\ EGS001
          endif
        endif
      endif
    endif
  loop
;
short-branches]


\ *******************************
\ *S Primitives for the BSD layer
\ *******************************

[-short-branches
: Tcp_Open	{ *sk | *cb res -- res }
\ *G Allows an application to open the TCP connection.
\ ** N.B. This is a *\b{blocking} function. It will *\b{not} return
\ ** until connected or timed out.
  SOCKET_ERROR -> res			\ assume the worst
  *sk so_*cb @ ?dup			\ is socket type TCP and connected?
  if
    -> *cb
    *cb t_state w@ TCPS_CLOSED =	\ only if closed
    if					\ if not return with SOCKET_ERROR
[ TCPDEBUG? ] [if]  cr ." Tcp_Open "  [then]
     *sk goSYN_SENT			\ SFP042
      begin
        pause
        *cb t_state w@
        TCPS_ESTABLISHED = if		\ connected?
          *cb *sk SendAck>TCP		\ we should ack the connection? ; MPE022
          0 -> res
          1				\ flag for exit
	else
          *cb t_state w@ TCPS_CLOSED =	\ closed
	endif
      until
    endif
  endif
  res					\ returns 0 if the request is in hand
;

: Tcp_Close	{ *sk | *cb res -- res }
\ *G Allows an application to close the TCP connection cleanly.
\ ** This is the TCP primitive for the BSD *\fo{disconnect}
\ ** function.
  SOCKET_ERROR -> res			\ assume the worst
  *sk so_*cb @ ?dup if			\ is socket type TCP and connected ??
    -> *cb
    *cb t_state w@
    dup TCPS_ESTABLISHED =		\ it is fully connected
    swap TCPS_SYN_RECEIVED = or		\ or syn_recvd
    if					\ if not return with SOCKET_ERROR
[ TCPDEBUG? ] [if]  cr ." Tcp_Close "  [then]
      *sk goFIN_WAIT_1			\ SFP042
      0 -> res
    else
      *cb t_state w@ TCPS_SYN_SENT = if	\ syn sent state closes immediately
        *sk (CloseSocket)		\ SFP030 SFP033
        0 -> res
      endif
    endif
  endif
  res					\ returns 0 if the request is in hand
;
short-branches]


\ *********************
\ *S TCP initialisation
\ *********************

: TcpInit	\ --
\ *G Initialise the TCP layer.
  Init-NoTcp-Socket drop		\ bad socket responses
;


\ *********************
\ *S Checksum test code
\ *********************

0 [if]

create tckbuff
  $00 c,  $01 c,  $02 c,  $03 c,  $04 c,  $05 c,  $06 c,  $07 c,
  $08 c,  $09 c,  $0A c,  $0B c,  $0C c,  $0D c,  $0E c,  $0F c,
  $10 c,  $11 c,  $12 c,  $13 c,  $14 c,  $15 c,  $16 c,  $17 c,
  $18 c,  $19 c,  $1A c,  $1B c,  $1C c,  $1D c,  $1E c,  $1F c,

: tck1  tckbuff #32 cksum .dword  ;
\ ARM - $0EFF
\ CF  - $0EFF

((
Bad SYN/ACK
EtherOut# 58
2000:1E9C  00 1A A0 8E FB C0 00 10  8B F1 44 53 08 00 45 00  .. .{@...qDS..E.
2000:1EAC  00 2C 00 05 40 00 FF 06  F8 CD C0 A8 00 CD C0 A8  .,..@...xM@(.M@(
2000:1EBC  00 DB 00 50 DC BF 3E 41  3B A4 1A 59 86 BC 60 12  .[.P\?>A;$.Y.<`.
2000:1ECC  05 B4 73 0E 00 00 02 04  05 B4 46 46 43 46 47 45  .4s......4FFCFGE

Ethernet header 14 bytes
00 1A A0 8E FB C0 00 10  8B F1 44 53 08 00

IP header 20 bytes, IPlen=$002C=44 bytes
45 00 00 2C  00 05 40 00  FF 06 F8 CD  C0 A8 00 CD
C0 A8 00 DB

TCP header, 20 bytes+4 option bytes
00 50 DC BF  3E 41 3B A4  1A 59 86 BC  60 12 05 B4
73 0E 00 00  02 04 05 B4
))

create PackBuff2
\ Ethernet header 14 bytes
  $00 c, $1A c, $A0 c, $8E c, $FB c, $C0 c, $00 c, $10 c,
  $8B c, $F1 c, $44 c, $53 c, $08 c, $00 c,
\ IP header 20 bytes, IPlen=$002C=44 bytes
  $45 c, $00 c, $00 c, $2C c,  $00 c, $05 c, $40 c, $00 c,
  $FF c, $06 c, $F8 c, $0CD c, $C0 c, $A8 c, $00 c, $0CD c,
  $C0 c, $A8 c, $00 c, $DB c,
\ TCP header, 20 bytes+4 option bytes
  $00 c, $50 c, $DC c, $BF c,  $3E c, $41 c, $3B c, $A4 c,
  $1A c, $59 c, $86 c, $BC c,  $60 c, $12 c, $05 c, $B4 c,
  $00 c, $00 c, $00 c, $00 c,  $02 c, $04 c, $05 c, $B4 c,
\ Checksum should be 175F, not 730E

\ : TcpCksum	\ *tcp tcplen iprem iploc -- cksum

: tck2		\ --
  PackBuff2 #14 + #20 +			\ TCP header
  #24					\ total length
  $C0A8:00DB				\ 192.168.0.219
  $C0A8:00CD				\ 192.168.0.205
  TcpCksum .dword
;

create PackBuff3
\ padding
  0 c, 0 c,
\ Ethernet header 14 bytes
  $00 c, $1A c, $A0 c, $8E c, $FB c, $C0 c, $00 c, $10 c,
  $8B c, $F1 c, $44 c, $53 c, $08 c, $00 c,
\ IP header 20 bytes, IPlen=$002C=44 bytes
  $45 c, $00 c, $00 c, $2C c,  $00 c, $05 c, $40 c, $00 c,
  $FF c, $06 c, $F8 c, $0CD c, $C0 c, $A8 c, $00 c, $0CD c,
  $C0 c, $A8 c, $00 c, $DB c,
\ TCP header, 20 bytes+4 option bytes
  $00 c, $50 c, $DC c, $BF c,  $3E c, $41 c, $3B c, $A4 c,
  $1A c, $59 c, $86 c, $BC c,  $60 c, $12 c, $05 c, $B4 c,
  $00 c, $00 c, $00 c, $00 c,  $02 c, $04 c, $05 c, $B4 c,
\ Checksum should be 175F, not 730E

: tck3		\ --
  PackBuff3 2 + #14 + #20 +		\ TCP header
  #24					\ total length
  $C0A8:00DB				\ 192.168.0.219
  $C0A8:00CD				\ 192.168.0.205
  TcpCksum .dword
;


[then]


\ ======
\ *> ###
\ ======

decimal

