\ UDP.FTH - definitions for UDP packet handling

((
Copyright (c) 1997-2003, 2007
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

Portions of this code have been licensed from:
  DelaComms Ltd. and Oxford Network Solutions 1996-1999
and:
  CEM Systems Ltd, Belfast, 2000-2002
  Hiden Analytical, 2002, 2003

To do
=====

Change history
==============
20070828 VRW003 Added UDP service despatcher and generic SendDatagram.
20070228 SFP002 Added UDPcksum.
20070214 SFP001 Added DHCP changes to RxUDPpacket.
))


\ ======
\ *! udp
\ *T UDP layer
\ ======

0 p_dat  ether_hdr +  ip_hdr +  udp_hdr +
  equ >UdpDataPos	\ -- offset
\ *G The UDP data offset in an outgoing UDP packet.


: UDPcksum	\ *hdr len ipsrc ipdest -- cksum
\ *G A generic UDP checksum function. The parameters supplied
\ ** are used to form a 'pseudoheader' from which the acual
\ ** checksum is generated.
  IPPROTO_UDP PHcksum dup 0=
  if  drop  $FFFF  endif
;

: UDPSend	{ hs *pb | *ip *sk -- len|err }
\ *G Send data on a connected UDP socket. *\i{HS} is a handle to
\ ** a UDP socket handle. *\i{*PB} is the address of a PBuff
\ ** pointer already filled with the data to send (as UDP data).
\ ** Returns the number of bytes sent if OK else SOCKET_ERROR.
  udp_hdr *pb unbumpPBuffData		\ point data to udp header
  hs get_socket_addr -> *sk		\ get socket structure
  *sk so_portloc w@			\ get local port number
  *pb PBDataStart uh_sport w!(n)	\ into UDP header
  *sk so_iploc @			\ and local ip address
  *pb PBDataStart #12 - !(n)		\ into UDP pseudo-header

  *sk so_portrem w@			\ and remote port number
  *pb PBDataStart uh_dport w!(n)	\ into UDP header
  *sk so_iprem @			\ and remote ip address
  *pb PBDataStart 8 - !(n)		\ into UDP pseudo-header

  *sk so_protocol w@			\ get protocol
  *pb PBDataStart 4 - w!(n)		\ into UDP pseudo-header

  *pb p_datalen w@			\ UDP packet length
  dup *pb PBDataStart uh_ulen w!(n)	\ into UDP header
  *pb PBDataStart 2 - w!(n)		\ into UDP pseudo-header

  0 *pb PBDataStart uh_sum w!(n)	\ zero cksum in UDP header
  *pb PBDataStart 12 -			\ MUST add in the pseudo header
  *pb PBDataLen 12 +			\ MUST add in the pseudo header length
  cksum					\ do header checksum
  *pb PBDataStart uh_sum w!(n)		\ and store it
\ *pb $80 dump
  *sk *pb IPSend			\ go send the IP packet & return result
;

: SendDatagram	{ *pb ipaddr | *ip *udp -- }
\ *G Send data as a UDP packet without using a socket. The
\ ** parameters are *\i{*pb}, a pointer to a PBuff containing
\ ** the UDP data, and *\i{ipaddr}, the IP address to send to.
\ cr ." SendDatagram "
  *pb 0=
  if  exit  then

  *pb PBDataStart  dup -> *udp		\ points to UDP header
  ip_hdr - -> *ip			\ IP header is before UDP header

  \ Build the UDP header
  0 *udp uh_sum w!(n)			\ zero cksum in udp header
  *udp					\ start of UDP header
  *pb p_datalen w@  dup *udp uh_ulen w!(n) \ UDP length into header
  IPaddress @(n) ipaddr			\ src and dest addresses
  UDPcksum *udp uh_sum w!(n)		\ calculate checksum

  \ Now build the IP header
  *ip *pb pb>IPH			\ Step back to the IP header
  *pb PBDataLen				\ number of bytes
  IpAddress @(n)			\ source IP address
  ipaddr				\ destination IP address
  IPPROTO_UDP				\ protocol
  *ip SetIPhdr				\ set up IP header

\ *ip #512 dump
  *pb (IPSend) drop			\ and finally send this
;

: GetUDPAddress	\ *pbuf -- *udp
\ *G Extract start of UDP data from IP packet.
  PBDataStart
;
compiler
: GetUDPAddress	\ *pbuf -- *udp ; extract start of UDP data from IP packet
  PBDataStart
;
target

: GetUDPLen	\ *udp -- len
\ *G Extract length of UDP packet.
  uh_ulen w@(n)				\ read length and convert to host order
  udp_hdr -				\ less the length of the header
;

: >UDPData	\ *udp -- *udpdata
\ *G Extract pointer to UDP packet data.
  udp_hdr +				\ simply add the header length
;
compiler
: >UDPData	\ *udp -- *udpdata
  udp_hdr +				\ simply add the header length
;
target
(( unused
: check_udp_cksum	\ *pbuf -- T|F
 \ *G Check an incoming UDP checksum.
  drop -1
;
))

variable UDPServiceChain	\ -- addr
\ *G This variable anchors the chain that will be walked to find
\ ** out what to do with a UDP packet that has arrived.

struct /UDPService	\ -- len
\ *G Describes the UDP service and how to run the service.
\ *[
  int us.next		\ Link to next service in chain
  int us.port		\ UDP Port that this service handles
  int us.xtReceive	\ Word that will process this packet
  int us.xtIdle		\ Word that will run idle action
end-struct
\ *]

: UDPprocessRx	\ *pb *udp --
\ *G Walk the UDP service chain and execute the first receive
\ ** action for the port. If no association is found, try to
\ ** route the packet.
  UDPServiceChain  begin		\ -- *pb *udp next
    @ dup
   while				\ -- *pb *udp us.item
    over uh_dport w@(n)			\ port to use
    over us.port @ = if			\ packet matches this service's port
      nip us.xtReceive @ execute  exit	\ Run service word
    endif
    us.next				\ step to next service
  repeat
  2drop					\ -- *pb
  ip_hdr over unbumpPBuffData		\ set data to include ip hdr again
  RouteIP				\ Attempt to route packet
;

-short-branches
XC? [if]
: RxUDPPacket	{ *pb | *udp hs *q bf -- }
\ *G Process a received UDP packet.
[else]
: rtRxUDPPacket	{ *pb | *udp hs *q bf -- }
[then]
  *pb GetUDPAddress -> *udp
  *pb PBHdrStart ip_dest @(n)		\ get dest address
  IPPROTO_UDP				\ and protocol
  *udp uh_dport w@(n)			\ now get the destination port number
  find_hsocket_port			\ see if we have a matching socket/port
  ?dup if				\ OK so split buffer into queues
    -> hs				\ save socket handle
    hs get_socket_inq dup -> *q	if	\ convert to *q only if a queue exists
      *pb PBHdrStart ip_src @(n)	\ get source address
      hs get_socket_addr so_iprem !	\ update the socket rem address
      *udp uh_sport w@(n)		\ now get the source port number
      hs get_socket_addr so_portrem w!	\ update the socket rem port
      *udp >UDPData *pb - 		\ offset to data start
      *pb !*PBdata			\ set data pointer into buffer
      *udp GetUDPLen *pb !PBDataLen	\ data length
      *pb *q enqueue			\ finally enqueue the buffer
    else
      *pb FreeQB			\ no *q so just lose all the data
    endif
  else
    *pb *udp UDPprocessRX
  endif
;
XC? 0= [if]  assign rtRxUDPPacket to-do RxUDPPacket  [then]
+short-branches


\ ======
\ *> ###
\ ======

decimal

