\ BMPfiles.fth - reading BMP files

((
Copyright (c) 2001, 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691


To do
=====

Change history
==============
20120808 SFP001 Rewritten - we have enough memory now.
))

0 constant turnkey?

\ *P Useful references for the various bitmap file formats are:
\ *C  http://en.wikipedia.org/wiki/BMP_file_format
\ *C  http://atlc.sourceforge.net/bmp.html
\ *C  http://www.fileformat.info/format/bmp/egff.htm
\ *C  http://expertsetup.com/~brian/bmp/index.html

ErrDef err_cantload	"Can't load BMP file"


\ *********************
\ *S Dialog definitions
\ *********************

NextID: IDD_DIALOG1
NextID: IDC_EDIT1
NextID: IDC_BUTTON1
NextID: IDC_EDIT2
NextID: IDC_BUTTON2
NextID: IDC_IMAGE
NextID: IDC_PREVIEW
NextID: IDC_LCDtype
NextID: IDC_FlipV
NextID: IDC_BigEnd
NextID: IDC_NoHeader

\ *P COMBOBOX id, x, y, w, h [[ , style [[ , exstyle ]] ]]
CBS_DROPDOWNLIST winconstant: CBS_DROPDOWNLIST
CBS_HASSTRINGS winconstant: CBS_HASSTRINGS

IDD_DIALOG1 DIALOG DISCARDABLE  0, 0, 334, 362
STYLE DS_MODALFRAME | WS_POPUP | WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU | WS_VISIBLE
CAPTION "MPE LCD Bitmap Converter v2.0"
FONT 8, 100, 0, "MS Sans Serif"
BEGIN
    CONTROL         "",IDC_IMAGE,"Static",SS_BITMAP,	7,7,320,240

    LTEXT           "BMP input file",IDC_STATIC,	7,271,64,11
    EDITTEXT        IDC_EDIT1,				73,268,205,13,  ES_AUTOHSCROLL
    PUSHBUTTON      "Browse",IDC_BUTTON1,		283,268,44,15

    LTEXT           "LCD output file",IDC_STATIC,	7,289,64,11
    EDITTEXT        IDC_EDIT2,				73,287,205,13,  ES_AUTOHSCROLL
    PUSHBUTTON      "Browse",IDC_BUTTON2,		283,286,44,15

    LTEXT           "LCD format",IDC_STATIC,		7,307,64,11
    COMBOBOX        IDC_LCDtype,			73,305,205,100, CBS_DROPDOWNLIST | CBS_HASSTRINGS
    PUSHBUTTON      "Flip vert",IDC_FlipV,		283,305,44,15, BS_AUTOCHECKBOX | BS_LEFTTEXT
    PUSHBUTTON      "BigEndian",IDC_BigEnd,		283,321,44,15, BS_AUTOCHECKBOX | BS_LEFTTEXT
    PUSHBUTTON      "No header",IDC_NoHeader,		283,337,44,15, BS_AUTOCHECKBOX | BS_LEFTTEXT

    DEFPUSHBUTTON   "OK",IDOK,				7,341,50,14
    PUSHBUTTON      "Exit",IDCANCEL,			59,341,50,14
    PUSHBUTTON	    "Preview",IDC_PREVIEW,		111,341,50,14
END

0 value AnError?	\ -- flag
\ *G Returns true when an error has occurred.
NULL Value hBMPcnv			\ file selection dialog
\ *G The handle of the main dialog.

: wipeBMPfilename	\ --
\ *G Clear the BMP input file box.
  hBMPcnv IDC_EDIT1 LB_RESETCONTENT 0 0 SendDlgItemMessage  drop
;

: zSetBMPfilename	\ z$ --
\ *G Set the BMP input file box.
  hBMPcnv IDC_EDIT1 rot SetDlgItemText drop
;

: zBMPfilename		\ buffer --
\ *G Get the BMP file name.
  0 over c!
  hBMPcnv IDC_Edit1 rot MAX_PATH GetDlgItemText drop
;

: wipeOPfilename	\ --
\ *G Clear the output file box.
  hBMPcnv IDC_EDIT2 LB_RESETCONTENT 0 0 SendDlgItemMessage  drop
;

: zSetOPfilename	\ z$ --
\ *G Set the output file box.
  hBMPcnv IDC_EDIT2 rot SetDlgItemText drop
;

: zOPfilename		\ buffer --
\ *G Get the BMP file name.
  0 over c!
  hBMPcnv IDC_Edit2 rot MAX_PATH GetDlgItemText drop
;

: emptyCB	\ hCb --
\ *G Empty a ComboBox
  CB_RESETCONTENT 0 0 SendMessage drop
;

: zAddCB	\ z$ hCb --
\ *G Add a line to a ComboBox.
  swap >r CB_ADDSTRING 0 r> SendMessage drop
;

: selCB		\ index hCb --
\ *G Select the given index in a ComboBox.
  swap >r
  CB_SETCURSEL r> 0 SendMessage drop
;

: getCBsel	\ hdlg id -- sel
\ *G Get the current index from a ComboBox
  CB_GETCURSEL 0 0 SendDlgItemMessage
;

: initCBlist	\ table hdlg id --
\ *G Initialise the given combobox using the given table.
  GetDlgItem				\ ComboBox handle
  dup emptyCB				\ force empty
  begin					\ -- table hdlg
    over c@
   while
    2dup zAddCB				\ add string to CB
    swap zcount + 1+ aligned swap	\ step to next one
  repeat
  over cell + @ over selCB		\ set default index
  2drop
;


\ *********************
\ *S BMP converter data
\ *********************

\ Compression types
0 constant BI_RGB			\ most common
1 constant BI_RLE8			\ eek!
2 constant BI_RLE4			\ eek!
3 constant BI_BITFIELDS			\ has 3 bitfield definitions
4 constant BI_JPEG			\ JPEG or RLE_24 for BITMAPCOREHEADER2
5 constant BI_PNG			\ PNG
6 constant BI_ALPHABITFIELDS		\ has four bitfield definitions (Win CE)

\ Input BMP file
MAX_PATH cell+ buffer: szBMPinpDir	\ default directory
0 value ^BMPfile			\ address of BMP file in memory
0 value /BMPfile			\ size of BMP file in memory
0 value ^BMPheader			\ address of BMP header in memory
0 value ^BMPinfo			\ address of BMP info structure in memory
0 value /BMPinfo			\ size of BMP info structure in memory
0 value ^ColourTable			\ address of colour table in memory
0 value /ColourTable			\ size of colour table ; Clr_table_size
0 value ^PixData			\ address of pixel data in memory
0 value /PixData			\ size of pixel data in memory
0 value #bppIn				\ bits per pixel (input)
0 value PixWidth			\ Image width in pixels (signed)
0 value PixHeight			\ Image height in pixels (signed)

\ Intermediate MPE format in memory
0 value ^GenForm			\ address of generic format data
					\ cells of ccrrggbb - little endian
					\ cc=colour index if colourtable is used
0 value /GenForm			\ length reserved for generic format data

\ LCD output file
MAX_PATH cell+  buffer: szLCDoutDir	\ default directory
0 value #bppOut				\ bits per pixel (output)
0 value OP_File				\ output file buffer
0 value Conv_Buff			\ primary conversion buffer
0 value Conv_buff2
0 value ^LCDimage			\ address of output LCD image
0 value /LCDimage			\ LCD image size in bytes
0 value /LCDrow				\ row length in bytes

\ Switches
1 value GenLCDheader?			\ 1 to generate LCD header
0 value BigEndian?			\ 1 to generate big-endian output data
0 value FlipVert?			\ 1 to invert image from top to bottom
2 value LCDformat			\ 0..n format of LCD output file.

struct /BMPheader
  2 field bfType
  4 field bfSize
  2 field bfRsvd1
  2 field bfRsvd2
  4 field bfOffs
end-struct

struct /BMP_Info_40	\ -- len
\ *G The structure of the common BITMAPINFOHEADER.
  4 field biSize			\ Structure size (40)
  4 field biWidth			\ the bitmap width in pixels (signed integer)
  4 field biHeight			\ the bitmap height in pixels (signed integer)
  2 field biPlanes			\ always 1
  2 field biBitCount			\ bits per pixel, 1, 4, 8, 16, 24, 32
  					\ 4: colour table
  					\ 8: colour table
  					\ 16: if not BI_BITFIELDS, 0:5:5:5
  					\ 24: 8:8:8
  					\ 32: 0:8:8:8:
  4 field biCompression
  4 field biSizeImage
  4 field biXPelsPerMetre
  4 field biYPelsPerMetre
  4 field biClrUsed			\ number of colours in colour table
  4 field biClrImportant
end-struct

struct /BMP_Info_12	\ -- len
\ *G The structure of the common BITMAPCOREHEADER (not yet supported).
  4 +					\ use biSize, (12)
  2 field bich.width			\ width in pixels
  2 field bich.height			\ height in pixels
  2 field bich.Planes			\ always 1
  2 field bich.BitCount			\ bits per pixel, 1, 4, 8, 24
end-struct

struct /LCDheader	\ - len
\ *G The standard 4 cell header used by MPE.
  int lcd.PixHeight			\ cell: height in pixels
  int lcd.PixWidth			\ cell: width in pixels
  int lcd./Row				\ cell: row size in bytes
  int lcd./Image			\ cell: size of image
end-struct


\ *****************
\ *S BMP primitives
\ *****************

: errorbox	\ message --
\ *G Launch a message box containing the text given. Use only for
\ ** error messages.
  -1 to AnError?
  HWND_DESKTOP swap Z" LCD BMP Converter Error" MB_OK MessageBox drop
;

: .badBMPformat	\ --
\ *G Error box for unhandled BMP format.
  z" I can't handle this bitmap format yet" errorbox
;

: .badIPpixwidth	\ --
\ *G Error box for unhandled input pixel size format.
  z" I can't handle this input pixel size yet" errorbox
;

: .badColourTable	\ --
\ *G Error box for incorrect colour table size
  z" Wrong size of colour table" errorbox
;

: .BadAllocate	\ --
\ *G Error box for memory allocation error
  z" Out of memory" errorbox
;

: setBMPdata40	\ --
\ We have a 40 byte info header. Set up the data pointers.
\ The colour table starts after the info header
  ^BMPinfo
  dup /BMPinfo + to ^ColourTable
  dup biClrUsed @ 2 lshift to /ColourTable
  dup biSizeImage @ to /PixData		\ size of Pixel Data
  dup biBitCount w@ to #bppIn		\ bits per pixel
  dup biWidth @ to PixWidth		\ image width
  dup biHeight @ to PixHeight		\ image height
  drop
;

: setBMPdata	\ --
\ *G Parse the BMP file and set up the BMP pointers and data.
  ^BMPfile to ^BMPheader		\ header at start of file
  ^BMPheader dup bfOffs @ + to ^PixData	\ pixel data
  ^BMPfile /BMPheader + to ^BMPinfo	\ address of info structure
  ^BMPinfo biSize @ to /BMPinfo		\ size of info struct, usually 40
  case  /BMPinfo
    40 of  setBMPdata40  endof
      z" Unknown BMP info size" errorbox
  endcase
;

: Can'tLoad	\ -- err#
\ *G Error box for bad BMP file.
  Z" Unable to load BMP File" errorbox  err_cantload
;

: loadBMPfile	\ -- ior
\ *G Load BMP file into memory
  {: | zname[ MAX_PATH 1 + ] -- ior :}
  0 -> AnError?				\ global error flag is cleared
  zname[ zBMPfilename
  zname[ zcount ['] inhale catch if	\ -- caddr len ior
    2drop  Can'tLoad exit
  endif
  dup 0= if				\ -- caddr len
    2drop  Can'tLoad exit
  endif
  to /BMPfile to ^BMPfile
  setBMPdata				\ set up data pointers
  0
;

: freeBMPfile	\ --
\ *G Release BMP file memory and data.
  ^BMPfile if
    burp  0 to ^BMPfile  0 to /BMPfile
  endif
;

: .BMPheader	\ --
\ *G Display BMP file header.
  cr
  ^BMPheader
  dup bfType cr ." BMP File Type ('BM') " 2 type
  dup bfSize @ cr ." BMP File is " . ." Bytes Long"
  dup bfRsvd1 w@ cr ." First Reserved # (0) " .
  dup bfRsvd2 w@ cr ." Second Reserved # (0) " .
  bfOffs @ cr ." BMP Offset Data Area " .lword ."  Hex"
  cr
;

: .BMPinfo40	\ --
\ *G Display the 40 byte BITMAPINFOHEADER data.
  cr
  ^BMPinfo
  dup biSize @ cr ." Info Header is " . ." Bytes Long"
  dup biWidth @ cr ." Bitmap is " . ." Pixels Wide"
  dup biHeight @ cr ." Bitmap is " . ." Pixels High"
  dup biPlanes w@ cr ." Colour Planes (1) " .
  dup biBitCount w@ cr ." Bitmap has " . ." Bits Per Pixel"
  dup biCompression @ cr ." Compression Format "
  case
    BI_RGB of ." RGB" endof
    BI_RLE8 of ." RLE8" endof
    BI_RLE4 of ." RLE4" endof
    BI_BITFIELDS of ." BITFIELDS" endof
    BI_JPEG of ." JPEG"  endof
    BI_PNG  of ." PNG"  endof
  end-case
  dup biSizeImage @ cr ." Image is " . ." Bytes Long"
  dup biXPelsPerMetre @ cr . ."  X Pels Per Metre"
  dup biYPelsPerMetre @ cr . ."  Y Pels Per Metre"
  dup biClrUsed @ cr . ." Colours Used"
  biClrImportant @ cr . ." Important Colours"
;

: .BMPinfo	\ --
\ *G Display BMP info header data
  case  ^BMPinfo biSize @
    40 of  .BMPinfo40  endof
      z" Unknown BMP info size" errorbox
  endcase
;

: .ColourSize	\ --
\ *G Display colour table size
  cr /ColourTable 2 rshift . ." items in Colour table"
;

: .ColourTable	\ --
\ *G Dump the colour table
  .ColourSize
  cr /ColourTable 0 ?do
    cr ." Clr#" i 2 rshift 3 .r space ^ColourTable i + @ .lword  ( 00rrggbb )
  cell +loop
;

: .PixData	\ --
\ *G Dump the pixel data
  cr  ^PixData  ^BMPinfo biSizeImage @ 0 ?do
    dup c@ dup 4 rshift .byte space	\ high nibble
    $0F and .byte 			\ low nibble
    i 7 and 7 <>
    if  ."  | "  else  cr  endif
    1 +
  loop
  drop
;

: .BMPdata	\ --
  .BMPheader
  .BMPinfo
  .ColourTable
;

\ *********************************
\ *S Convert to intermediate format
\ *********************************

: initGenMem	\ -- ior
\ *G Create the generic format memory. Return zero on success.
  PixWidth PixHeight * 4 * to /GenForm
  /GenForm allocate if			\ bad
    drop -1
  else					\ good
    to ^GenForm  0
  endif
;

: termGenMem	\ --
\ *G Release the generic format memory.
  ^GenForm if
    ^GenForm free drop
    0 to ^GenForm  0 to /GenForm
  endif
;

: cval>gen	\ index -- ccrrggbb
\ *G Convert a colour table index to generic format.
  dup cells ^ColourTable + @ $00FF:FFFF and
  swap 24 lshift or
; doNotSin

: convPixPair	\ src dest -- src' dest'
\ *G Convert two 4-bit indices in a byte to generic format.
  over c@ dup 4 rshift $0F and		\ -- src dest pixpair hpix
  cval>gen 2 pick !			\ -- src dest pixpair
  $0F and cval>gen  over cell+ !	\ -- src dest
  swap 1+ swap 8 +			\ -- src' dest'
; doNotSin

: convPixOdd	\ src dest -- src' dest'
\ *G Convert a single 4-bit index in a byte to generic format.
  over c@ 4 rshift $0F and cval>gen over !
  swap 1+ swap 4 +
;

: BMPlineAlign	\ end start -- next
\ *G Align a lign address so that the line length is a multiple
\ ** of 4 bytes.
  tuck -				\ length of line
  3 + -4 and 				\ four byte aligned (BMP requirement)
  + 					\ form next
;

: conv4>gen	\ --
\ *G Convert a 4 bit BMP to generic format.
  /ColourTable 64 <>			\ 16 entries
  if  .badColourTable exit  endif
  ^PixData ^GenForm
  PixHeight 0 ?do			\ -- src dest ; for each line
    over >r				\ save src start address of line
    PixWidth -2 and 0			\ even number of pixels
    ?do  convPixPair  2 +loop
    PixWidth 1 and 			\ last if PixWidth is odd
    if  convPixOdd  endif
    swap r> BMPlineAlign swap		\ 4 byte align line
  loop
  2drop
;

: conv8>gen	\ --
\ *G Convert an 8 bit BMP to generic format.
  /ColourTable 1024 <>			\ 256 entries
  if  .badColourTable exit  endif
  ^PixData ^GenForm
  PixHeight 0 ?do			\ -- src dest ; for each line
    over >r				\ save start address of line
    PixWidth 0 ?do			\ -- src dest
      over c@ cval>gen over !
      swap 1+ swap 4 +
    loop
    swap r> BMPlineAlign swap		\ 4 byte align line
  loop
  2drop
;

: c555>gen	\ w -- 00rrggbb
\ *G Convert 5:5:5 to 8:8:8
  dup 10 rshift $1F and 19 lshift	\ -- w 00rr0000
  over 5 rshift $1F and 11 lshift or	\ -- w 00rrgg00
  swap $1F and 3 lshift or		\ -- 00rrggbb
;

: conv16>gen	\ --
\ *G Convert a 16 bit BMP to generic format.
  ^PixData ^GenForm
  PixHeight 0 ?do			\ -- src dest ; for each line
    over >r				\ save start address of line
    PixWidth 0 ?do			\ -- src dest
      over w@ c555>gen over !
      swap 2+ swap 4 +
    loop
    swap r> BMPlineAlign swap		\ 4 byte align line
  loop
  2drop
;

: conv24>gen	\ --
\ *G Convert a 24 bit BMP to generic format.
  ^PixData ^GenForm
  PixHeight 0 ?do			\ -- src dest ; for each line
    over >r				\ save start address of line
    PixWidth 0 ?do			\ -- src dest
      over @ $00FF:FFFF and over !	\ use x86 non-aligned fetches
      swap 3 + swap 4 +
    loop
    swap r> BMPlineAlign swap		\ 4 byte align line
  loop
  2drop
;

: conv32>gen	\ --
\ *G Convert a 32 bit BMP to generic format.
  ^PixData ^GenForm			\ -- src dest
  PixHeight PixWidth * 4 * cmove
;

: conv>gen	\ --
  case #bppIn
     4 of  conv4>gen  endof
     8 of  conv8>gen  endof
    16 of  conv16>gen  endof
    24 of  conv24>gen  endof
    32 of  conv32>gen  endof
      .badIPpixwidth
  endcase
;


\ ***************************
\ *S Convert to output format
\ ***************************

: w!(le)	\ w caddr --
\ *G Little-endian 16 bit unaligned word store.
  2dup c!  swap 8 rshift swap 1+ c!
;

: l!(le)	\ x caddr --
\ *G Little endian 32 bit unaligned store.
  2dup w!(le)  swap #16 rshift swap 2+ w!(le)
;

: w!(be)         \ w addr --
\ *G Big-Endian 16 bit store.
  over ( $FF and ) over 1 + c!
  swap 8 rshift swap c!
;

: l!(be)	\ x addr --
\ *G Big-Endian 32 bit store.
  2dup 3 + c!  swap 8 rshift swap
  2dup 2 + c!  swap 8 rshift swap
  2dup 1 + c!  swap 8 rshift swap
  c!
;

: c!lcd		\ b addr --
\ *G Write a byte to LCD memory.
  c!
;

: w!lcd		\ w addr --
\ *G Write a 16 bit word to LCD memory. If big-endian output is
\ ** required, the bytes are swapped.
  BigEndian?
  if  w!(be)  else  w!(le)  endif
;

: l!lcd		\ x addr --
\ *G Write a 32 bit word to LCD memory. If big-endian output is
\ ** required, the bytes are swapped.
  BigEndian?
  if  l!(be)  else  l!(le)  endif
;


: +LCDheader	\ addr -- addr'
\ *G Increase by size of the LCD header if any.
  GenLCDheader?
  if  /LCDheader +  endif
;

: writeLCDfile	\ --
\ *G Write the LCD output file.
  {: | temp[ MAX_PATH 1 + ] -- :}
  temp[ zOPfilename
  ^LCDimage /LCDimage +LCDheader  temp[ zcount
  ['] exhale catch if
    z" bad write to LCD output file" errorbox
  endif
;

: .success	\ --
\ *G Display success box.
  AnError? if exit then
  hBmpCnv
  Z" Bitmap Converted Successfully" Z" LCD BMP Converter Finished"
  MB_OK MessageBox drop
;

[-short-branches
: setLCDheader	\ addr1 -- addr2
\ *G Generate the MPE header at addr1 and step over it to addr2.
  GenLCDheader? if
    PixHeight over lcd.PixHeight l!lcd
    PixWidth  over lcd.PixWidth l!lcd
    /LCDrow   over lcd./Row l!lcd
    /LCDimage over lcd./Image l!lcd
    /LCDheader +
  endif
;
short-branches]

: LineData	\ line# -- addr
\ *G Given a line number, return its start address in the generic
\ ** image array.
  PixWidth cells *  ^GenForm +
; doNotSin

: getLCDmem	\ /img -- addr
\ *G Given the LCD output image size, allocate memory for it
\ ** and the header.
  +LCDheader allocate if
    drop 0  .BadAllocate
  endif
  dup to ^LCDimage			\ -- img* ; start
;

: freeLCDmem	\ --
\ *G FREE the output file memory.
  ^LCDimage free drop
;


\ ===============
\ *N 2 bit output
\ ===============
\ *P The output is a stream of bytes with 4 pixels per byte.
\ ** A line is byte aligned.

defer cc>2bit	\ ccrrggbb -- 2b
\ *G Convert a generic format item to a 2 bit value.

: (cc>2bit)	\ ccrrggbb -- 2b
\ *G Convert a generic format item to a 2 bit value. This version
\ ** assumes *\i{cc} is a 4 bit colour index. In the monochrome
\ ** conversions seen, only colour values 0, 7, 8, F were used.
  24 rshift $0F and
  case
     0 of  3  endof		\ 0000:0000
     8 of  2  endof		\ 0080:8080
     7 of  1  endof		\ 00C0:C0C0
    $F of  0  endof		\ 00FF:FFFF
    \ -------------
     1 of  2  endof		\ 0080:0000
     2 of  2  endof		\ 0000:8000
     4 of  2  endof		\ 0000:0080
     3 of  2  endof		\ 0080:8000
     6 of  2  endof		\ 0000:8080
     5 of  2  endof		\ 0080:0080
     9 of  1  endof		\ 00FF:0000
    $A of  1  endof		\ 0000:FF00
    $C of  1  endof		\ 0000:00FF
    $B of  1  endof		\ 00FF:FF00
    $E of  1  endof		\ 0000:FFFF
    $D of  1  endof		\ 00FF:00FF
  end-case
;
assign (cc>2bit) to-do cc>2bit

: gen>2bitLine	\ *img line# --
  LineData swap  PixWidth 0 ?do		\ -- *pix *img
    over
    dup            @ cc>2bit 6 lshift
    over 1 cells + @ cc>2bit 4 lshift or
    over 2 cells + @ cc>2bit 2 lshift or
    swap 3 cells + @ cc>2bit          or
    over c!lcd
    swap 4 cells + swap 1 +
  4 +loop
  2drop
;

: gen>2bitImage	\ *img --
  FlipVert? if
    0 PixHeight 1-
    ?do  dup i gen>2bitLine  /LCDrow +  -1 +loop
  else
    PixHeight 0
    ?do  dup i gen>2bitLine  /LCDrow +  loop
  endif
  drop
;

: gen>2bit	\ --
\ *G Convert generic image to 2 bit grayscale and write the file.
\  ." gen>2bit" cr
  PixWidth 3 + -4 and 4 /  to /LCDrow	\ row size in bytes
  PixHeight /LCDrow *  dup to /LCDimage	\ image size in bytes
  getLCDmem dup if			\ get memory for file image
    setLCDheader			\ MPE LCD header
    gen>2bitImage			\ generate LCD image
    writeLCDfile			\ write mem to file
    freeLCDmem				\ free mem
  else
    drop  .BadAllocate
  endif
;

\ ===================
\ *N 12 bit RGB 4:4:4
\ ===================
\ *P The output consists of sets of three bytes containing even
\ ** and odd pixels:
\ *C  r0g0 b0r1 g1b1
\ *P If there is an odd number of pixels in a line, the r1g1b1
\ ** set of the last pair are dummies.

: genpix>444	\ ccrrggbb -- 00000rgb
  dup 20 rshift $0F and 8 lshift	\ top 4 bits of red
  dup 12 rshift $0F and 4 lshift or	\ top 4 bits of green
  swap 4 rshift $0F and or		\ top 4 bits of blue
;

: gen>444line	\ *img line# --
  LineData swap
  PixWidth -2 and 0 ?do			\ -- *pix *img
    over @ genpix>444			\ pix444-0
    dup >r  4 rshift over + c!lcd	\ save top 8 bits of pix444-0
    over cell + @ genpix>444
    r>  over >r				\ -- *pix *img pix444-0 pix444-1
    swap $0F and 4 lshift		\ bottom 4 bits of pix444-0
    swap 8 rshift $0F and 		\ top 4 bits of pix444-1
    or over 1+ c!lcd
    r> $FF and over 2 + c!lcd		\ bottom 8 bits of pix444-1
    swap 2 cells + swap 3 +
  2 +loop
  PixWidth 1 and if
    over @ genpix>444
    dup >r  4 rshift over + c!lcd	\ top 8 bits
    r> 4 lshift over 1 + c!lcd		\ bottom 4 bits
    0 over 2 + c!lcd			\ dummy
    swap cell + swap 3 +
  endif
  2drop
;

: gen>444Image	\ *img --
  FlipVert? if
    0 PixHeight 1-
    ?do  dup i gen>444Line  /LCDrow +  -1 +loop
  else
    PixHeight 0
    ?do  dup i gen>444Line  /LCDrow +  loop
  endif
  drop
;

: gen>444	\ --
\ *G Convert the generic image to 12 bit colour and write the
\ ** file. Two pixels are packed into three bytes.
\  cr ." gen>444" cr
  PixWidth 1 + -2 and 3 2 */ to /LCDrow	\ row size in bytes
  PixHeight /LCDrow *  dup to /LCDimage	\ image size in bytes
  getLCDmem dup if			\ get memory for file image
    setLCDheader			\ MPE LCD header
    gen>444Image			\ generate LCD image
    writeLCDfile			\ write mem to file
    freeLCDmem				\ free mem
  else
    drop  .BadAllocate
  endif
;


\ ===================
\ *N 16 bit RGB 5:6:5
\ ===================

: genpix>565	\ ccrrggbb -- w
\ *G Convert a generic colour to 16 bit 5:6:5 format.
  dup 19 rshift $1F and 11 lshift	\ top 5 bits of red
  dup 10 rshift $3F and 5 lshift or	\ top 6 bits of green
  swap 3 rshift $1F and or		\ top 5 bits of blue
; doNotSin

((
: .PixLcd	\ *pix *lcd -- *pix *lcd
  ." Pix=" over .dword  ."  LCD=" dup .dword cr
;
))

: gen>565line	\ *lcd line# --
\ *G Convert the generic line specified by *\i{line#} to LCD format
\ ** at *\i{*lcd}.
  LineData swap
  PixWidth 0 ?do			\ -- *pix *lcd
    over @ genpix>565 over w!lcd
    swap cell + swap 2 +
  loop
  2drop
; doNotSin

: gen>565Image	\ *lcd --
\ *G Convert the generic format image to LCD format at *\i{*lcd}.
  FlipVert? if
    0 PixHeight 1-
    ?do  dup i gen>565Line  /LCDrow +  -1 +loop
  else
    PixHeight 0
    ?do  dup i gen>565Line  /LCDrow +  loop
  endif
  drop
;

: gen>565	\ --
\ *G Convert the generic image to 16 bit colour and write the
\ ** file.
\  cr ." gen>565" cr
  PixWidth 2* to /LCDrow		\ row size in bytes
  PixHeight /LCDrow *  dup to /LCDimage	\ image size in bytes
  getLCDmem dup if			\ get memory for file image
    setLCDheader			\ MPE LCD header
    gen>565Image			\ generate LCD image
    writeLCDfile			\ write mem to file
    freeLCDmem				\ free mem
  else
    drop  .BadAllocate
  endif
;


\ ================
\ *N Output driver
\ ================

create LCDtypes	\ -- addr
\ *G Table of strings that identify the available output types.
  z", 2 bit grayscale, e.g. GiantPlus GPG3224TWE3" align
  z", 12 bit RGB 4:4:4 colour, e.g. Nokia 6100" align
  z", 16 bit RGB 5:6:5 colour, e.g. ST MB785" align
  0 , 2 ,				\ end, default index

create LCDxts	\ -- addr
\ *G Table of xts that generate the LCD output files.
  ' gen>2bit ,				\ 2 bit grayscale
  ' gen>444 ,				\ 4:4:4
  ' gen>565 ,				\ 5:6:5
  0 ,  ' noop ,

: gen>LCDfile	\ --
\ *G Convert the generic format to the selected LCD format.
  AnError? if exit endif
  hBMPcnv IDC_LCDtype getCBsel dup 0< if
    z" No output type selection" errorbox  exit
  endif
  cells LCDxts + @ execute		\ execute selected action
;

: CheckBppIn	\ -- ior
\ *G Check the input bits per pixel.
  case #bppIn
     4 of  0  endof
     8 of  0  endof
    16 of  0  endof
    24 of  0  endof
    32 of  0  endof
      -1 swap
  endcase
;

: CheckInfoHeader40	\ -- ior
\ *G Check a BMP file 40 byte header.
  ^BMPinfo
  dup biWidth @ 640 > if
    drop  z" Wider than 640 pixels" errorbox  -2 exit
  endif
  dup biHeight @ 480 > if
    drop  z" Higher than 480 pixels" errorbox  -3 exit
  endif
  dup biCompression @ BI_RGB <> if
    drop  z" Bad compression type" errorbox  -4 exit  then
  true
  drop  CheckBppIn -6 and
;

: ?NotBMPfile	\ flag -- ior
\ *G Show an error box if flag is true.
  if  z" Bad BMP signature" errorbox  1  else  0  endif
;

: CheckBMPdata	\ -- ior
\ *G Check the BMP file headers.
  ^BMPheader c@ 'B' <> ?NotBMPfile ?dup if  exit  endif
  ^BMPheader 1+ c@ 'M' <> ?NotBMPfile ?dup if  exit  endif
  case  ^BMPinfo biSize @
    40 of  CheckInfoHeader40  endof
      drop  z" I can't handle this header size yet" errorbox  -1
  end-case
;

: BMPoptions	\ --
\ *G Read the BMP option checkboxes.
  hBMPcnv IDC_FlipV    GetCheckState to FlipVert?
  hBMPcnv IDC_BigEnd   GetCheckState to BigEndian?
  hBMPcnv IDC_NoHeader GetCheckState 0= to GenLCDHeader?
;

: setBMPoptions	\ --
\ *G Write the BMP option checkboxes.
  hBMPcnv IDC_FlipV    FlipVert? SetCheckState
  hBMPcnv IDC_BigEnd   BigEndian? SetCheckState
  hBMPcnv IDC_NoHeader GenLCDHeader? 0= SetCheckState
;

[-short-branches
: Main		\ --
\ *G The main action of the conversion. Read the bitmap file and
\ ** convert it to the specified LCD format.
  loadBMPfile if exit then
  CheckBMPdata 0= if
    BMPoptions
\    .BMPdata
\    .PixData
    initGenMem if			\ get memory for generic format
      .BadAllocate
    else
      conv>gen  			\ convert BMP to generic
      gen>LCDfile			\ generic to LCD file
      termGenMem
      .success
    endif
  then
  freeBMPfile
;
short-branches]

((
: cleanup
  ['] burp catch drop
;
' cleanup atexit
))


\ *******************
\ *S INI file actions
\ *******************

[undefined] szConfigName [if]
: BMPiniFile	\ -- caddr len
  INIfile$ count
;
[else]
: BMPiniFile	\ -- caddr len
  szConfigName zcount
;
[then]

: loadBMPini	\ --
\ *G Load the BMP INI file data.
  BMPiniFile Ini.Open if  exit  endif
  base @ >r  decimal
  s" BmpCnv" Ini.Section
  s" FlipVert" 1 Ini.ReadInt to FlipVert?
  s" BigEndian" 0 Ini.ReadInt to BigEndian?
  s" GenLCDheader" 1 Ini.ReadInt to GenLCDHeader?
  s" LCDformat" 2 Ini.ReadInt to LCDformat
  s" BmpInputDir"  szBMPinpDir MAX_PATH s" ." Ini.ReadZStr
  s" LcdOutputDir" szLCDoutDir MAX_PATH s" ." Ini.ReadZStr
  Ini.Close
  setBMPoptions
  r> base !
;

: saveBMPini	\ --
\ *G Save the BMP INI file data.
  BMPiniFile Ini.Open if  exit  endif
  base @ >r  decimal
  s" BmpCnv" Ini.Section
  s" FlipVert" FlipVert? Ini.WriteInt
  s" BigEndian" BigEndian? Ini.WriteInt
  s" GenLCDheader" GenLCDHeader? Ini.WriteInt
  s" LCDformat" LCDformat Ini.WriteInt
  s" BmpInputDir"  szBMPinpDir Ini.WriteZStr
  s" LcdOutputDir" szLCDoutDir Ini.WriteZStr
  Ini.Close
  r> base !
;


\ *****************
\ *S Dialog actions
\ *****************

: showBMPfile	\ zname --
\ *G Display the BMP file in the image box.
  0 swap IMAGE_BITMAP			\ #0 filestr type --
  0 0	 				\ #0 filestr type #0 #0 --
  LR_LOADFROMFILE 			\ #0 filestr type #0 #0 lparam --
  LoadImage				\ -- hbmp
  hBMPcnv IDC_IMAGE rot		\ hdlg item-id hbmp --
  STM_SETIMAGE IMAGE_BITMAP rot		\ hdlg item-id message wparam hbmp --
  SendDlgItemMessage drop
;

((
: AskOpenFile	\  zcap zdir pBuffer -- ior ; 0=success
\ *G Run the file open dialog with zcap as the caption (may be 0),
\ ** zdir as a zero terminated string holding the default directory,
\ ** and pBuffer receiving the fully qualified path name selected.
\ ** Ior is returned 0 if a good selection was made. The output
\ ** pBuffer is only updated if a good selection is made.
  ['] GetOpenFileName (AskFile)
;

: AskSaveFile	\  zcap zdir pBuffer -- ior ; 0=success
\ *G Run the file open dialog with zcap as the caption (may be 0),
\ ** zdir as a zero terminated string holding the default directory,
\ ** and pBuffer receiving the fully qualified path name selected.
\ ** Ior is returned 0 if a good selection was made. The output
\ ** pBuffer is only updated if a good selection is made.
  ['] GetSaveFileName (AskFile)
;
))

\ browse for the file to scan
: BMP_File_Select	\ --
  {: | ofn[ openfilename ] zname[ MAX_PATH 1 + ] tmp[ MAX_PATH 1 + ]  -- :}
  MAX_PATH tmp[ GetCurrentDirectory drop        \ GetOpenFileName changes it!
  0 zname[ c!
  wipeBMPfilename
  ofn[ openfilename erase
  openfilename ofn[ openfilename.lStructSize !
  zname[ ofn[ openfilename.lpstrFile !
  MAX_PATH ofn[ openfilename.nMaxFile !
  Z" Select BMP file to scan" ofn[ openfilename.lpstrTitle !
  OFN_FILEMUSTEXIST ofn[ openfilename.Flags !
  szBMPinpDir ofn[ openfilename.lpstrInitialDir !
  ofn[ GetOpenFileName if
    MAX_PATH szBMPinpDir GetCurrentDirectory drop
  endif
  ofn[ openfilename.lpstrFile @			\ filestr --
  dup zSetBMPfilename showBMPfile
  tmp[ SetCurrentDirectory drop			\ restore
;

: OP_File_Select	\ --
  {: | ofn[ openfilename ] zname[ MAX_PATH 1 + ] tmp[ MAX_PATH 1 + ] -- :}
  MAX_PATH tmp[ GetCurrentDirectory drop        \ GetOpenFileName changes it!
  0 zname[ c!
  wipeOPfilename
  ofn[ openfilename erase
  ofn[ openfilename.lStructSize openfilename swap !
  zname[ ofn[ openfilename.lpstrFile !
  MAX_PATH ofn[ openfilename.nMaxFile !
  Z" Select LCD output file" ofn[ openfilename.lpstrTitle !
  szLCDoutDir ofn[ openfilename.lpstrInitialDir !
  ofn[ GetSaveFileName if
    MAX_PATH szLCDoutDir GetCurrentDirectory drop
  endif
  ofn[ openfilename.lpstrFile @ zSetOPfilename	\ filestr --
  tmp[ SetCurrentDirectory drop			\ restore
;

: Preview	\ --
  {: | zname[ MAX_PATH 1+ ] -- :}
  zname[ zBMPfilename  zname[ showBMPfile
;

4 1 callback: BrowserDialogProc

[-short-branches
: (BrowserDialogProc)	{: hdlg message wparam lparam -- ior :}
  case message

    WM_INITDIALOG of
      hdlg -> hBMPcnv  hdlg +modeless	\ tell VFX its modeless
      loadBMPini
      LCDtypes hdlg IDC_LCDtype initCBlist
      1
    endof

    WM_COMMAND of
      case wparam LOWORD
        IDCANCEL    of  hdlg sendClose  1  endof
	IDC_BUTTON1 of  BMP_File_Select  1  endof
	IDC_BUTTON2 of  OP_File_Select  1  endof
	IDC_PREVIEW of  preview  1  endof
	IDOK        of  Main  1  endof
          drop 0
      end-case
    endof

    WM_CLOSE of
      saveBMPini
      [ turnkey? ] [IF]
        0 ExitProcess drop  1
      [ELSE]
        hDlg DestroyWindow drop  hdlg -ModeLess  0 -> hBMPcnv  1
      [THEN]
    endof

      drop 0
  end-case
; assign (BrowserDialogProc) to-callback BrowserDialogProc
short-branches]


\ **********
\ *S Turnkey or not
\ **********

turnkey? [IF]
\ *G For turnkey versions.

: runBMPCnv	\ --
\ *G Start the dialog if not already started.
  hBMPcnv 0= if
    IDD_DIALOG1 BrowserDialogProc HWND_DESKTOP ModelessDlgRun
  then
;


: WinMain	\ hInstance hPInst lpstrCommand nShow -- res
\ *G The main appication word.
  4drop  RunCnv  appIdle  ExitCode @
;
assign WinMain to-do EntryPoint

save BmpCnv

[ELSE]

: runBMPCnv	\ --
\ *G Start the dialog if not already started.
  hBMPcnv 0= if
    IDD_DIALOG1 BrowserDialogProc hWndMain ModelessDlgRun
  then
; doNotSin

: closeBMPcnv	\ --
\ *G Stop the dialog if open.
  hBMPcnv if  hBMPcnv sendClose  endif
; doNotSin

: toggleBMPcnv	\ --
  hBMPcnv
  if  closeBMPcnv  else  runBMPcnv  endif
; doNotSin

[THEN]
