\ ILI932x.fth - ILI 9320/5 LCD on STM3240G-EVAL board

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! ili932x
\ *T ILI932x LCD controller graphics
\ ===========
\ *P The STM32xx external bus interface is set up in another file.
\ ** For the STM3240G-EVAL board, this is
\ ** */i{Cortex/Hardware/STM32F4/FSMCeval.fth}.

\ *P The display is memory mapped at two locations, each taking
\ ** 16 bit data:
\ *(
\ *B _ELCD + 0 - index/register
\ *B _ELCD + 2 - data
\ *)
\ *P The base address *\fo{_ELCD} is $6800:0000 for the STM3240G-EVAL
\ ** board and the display connected by 16 bit data to the FSMC_NE3
\ ** select.


\ ****************
\ *S Configuration
\ ****************
\ *P Please note that the ILI932x controller documentation refers
\ ** to the panel as 240 by 320, i.e portrait is the default.
\ ** Because of the register format, the controller must be set
\ ** up using datasheet vertical for the longer dimension. Some
\ ** registers, e.g. R20h are restricted to 8 bits.

\ *P By default, the code here operates the display in landscape
\ ** format, width=320, height=240 pixels.

320 constant /ScreenV	\ -- u
\ *G Screen vertical size
240 constant /ScreenH	\ -- u
\ *G Screen horizontal size.
/ScreenV /ScreenH * constant /Screen	\ -- u
\ *G Number of pixels on the screen.

0 constant Portrait?	\ -- x
\ *G True (non-zero) for portrait mode with the long edge vertical.
Portrait? 0= [if]	\ landscape
  /ScreenV 1- constant XMax
  /ScreenH 1- constant YMax
[then]
Portrait? [if]		\ portrait
  /ScreenH 1- constant XMax
  /ScreenV 1- constant YMax
[then]

1 constant OrgTopLeft?	\ -- x
\ *G True (non-zero) for screen 0,0 at the top left. If set zero,
\ ** 0,0 is at bottom left. The GUI system needs this set to 1.


\ *************
\ *S Primitives
\ *************

variable X	\ -- addr
\ *G Holds the current X position
variable Y	\ -- addr
\ *G Holds the current Y position
variable dX	\ -- addr
\ *G Holds the current X increment, usually -1, 0, or 1.
variable dY	\ -- addr
\ *G Holds the current Y increment, usually -1, 0, or 1.
variable FColor	\ -- addr
\ *G Holds the current foreground colour.
\ ** Colours are 16 bit RGB values in 5:6:5 form *\i{rrrrrggggggbbbbb}.
  $FFFF Fcolor !
variable BColor	\ -- addr
\ *G Holds the current background colour.
\ ** Colours are 16 bit RGB values in 5:6:5 form *\i{rrrrrggggggbbbbb}.
  $0000 BColor !

interpreter
: RGB		\ rr gg bb -- c565
\ *G Convert the three 0..255 RGB values to the internal 5:6:5 format.
\ ** INTERPRET ONLY.
  rot  3 rshift $1F and 11 lshift	\ -- gg bb c5xx
  rot  2 rshift $3F and 5 lshift or	\ -- bb c56x
  swap 3 rshift $1F and or		\ -- c565
;

: >RGB		\ c565 -- rr gg bb
\ *G Convert the internal 5:6:5 format to the three 0..255 RGB values.
\ ** INTERPRET ONLY.
  dup 11 rshift $1F and 3 lshift swap	\ -- rr c565
  dup  5 rshift $3F and 2 lshift swap	\ -- rr gg c565
                $1F and 3 lshift	\ -- rr gg bb
;
target

0xFFFF constant WHITE
0x0000 constant BLACK
\ 0xF7DE constant GRAY
0x001F constant BLUE
0x051F constant BLUE2
0xF800 constant RED
0xF81F constant MAGENTA
0x07E0 constant GREEN
0x7FFF constant CYAN
0xFFE0 constant YELLOW

$F0 $F8 $F0 RGB constant LTGRAY
$DF $DF $DF RGB constant GRAY

: setFColor      \ colour --
\ *G Set the foreground colour.
  Fcolor !
;

: SetBColor  \ colour --
\ *G Set the background colour.
  Bcolor !
;

: setTColor	\ colour --
\ *G Set the text drawing colour. Same as *\fo{setFColor}
  FColor !
;

: XY>		\ -- x y
\ *G Return the current X and Y coordinates.
  X @ Y @
;

: >XY		\ x y --
\ *G Set the current X and Y coordinates
  Y !  X !
;
compiler
: XY>		X @  Y @  ;
: >XY		Y !  X !  ;
target

: .XY		\ --
  ." XY = " X ? Y ?
;

: lcdReg@	\ reg -- w
\ *G Read a 16 bit value from an LCD register.
  _elcd tuck w!  2+ w@
;

: lcdReg!	\ w reg --
\ *G Write a 16 bit value to an LCD register.
  _elcd tuck w!  2+ w!
;

: lcdRAM!	\ w --
\ *G Write w to GRAM.
  _elcd 2+ w!
;
compiler
: lcdRAM!	( w -- )  _elcd 2+ w!  ;
target

: lcdRAMprep	\ --
\ *G Prepare GRAM for writing.
  34 _elcd w!				\ select reg 34/22h
;

: setCursor	\ x y --
\ *G Set the GRAM cursor position.
[ Portrait? ] [if]
  [ orgTopLeft? 0= ] [if] Ymax swap - [then]
  swap
[else]
  [ orgTopLeft? ] [if] Ymax swap - [then]
[then]
  32 lcdReg!  33 lcdReg!
;

: lcdClear	\ colour --
\ *G Clear the screen to the given colour.
  0 0 setCursor
  lcdRAMprep  /Screen 0
  do  dup lcdRAM!  loop
  drop
;

: clear-lcd	\ --
\ *G Clear the screen to the current background colour.
  BColor @ lcdClear
  0 0 setCursor  0 0 >XY
;

: setPixel	\ x y color --
\ *G Plot a pixel at *\i{x,y} in the given *\i{color}.
  >r  setCursor  lcdRAMprep  r> lcdRAM!
;

Portrait? 0= [if]
: fillX		\ u --
\ *G Write u pixels in the current foreground colour.
  lcdRAMprep
  0 ?do  FColor @ lcdRAM!  loop
;

: fillRect	\ w h --
\ *G Fill the area of given width and height at the present X,Y
\ ** position using the current foreground colour.
  { w h -- }
  w if
    h 0 ?do
      X @ Y @ i + setCursor  w fillX
    loop
  endif
;
[then]

Portrait? [if]
: fillY		\ u --
\ *G Write u pixels in the current foreground colour.
  lcdRAMprep
  0 ?do  FColor @ lcdRAM!  loop
;

: fillRect	\ w h --
\ *G Fill the area of given width and height at the present X,Y
\ ** position using the current foreground colour.
  { w h -- }
  h if
    w 0 ?do
      X @ Y @ i + setCursor  h fillY
    loop
  endif
;
[then]

: gline		\ xdest ydest --
\ *G Draw a line from the current position to *\i{Xdest,Ydest}.
  { | x0 y0 x1 y1 stepx stepy -- }
\ cr .xy  ."   dest = " over . dup .
  to y1  to x1				\ end
  X @ to x0  Y @ to Y0			\ start
  y1 y0 - dY !  x1 x0 - dX !
  dX @ 0<
  if  dX @ negate dX !  -1 to stepX  else  1 to stepX  endif
  dY @ 0<
  if  dY @ negate dY !  -1 to stepY  else  1 to stepY  endif
  dX @ 1 lshift dX !  dY @ 1 lshift dY ! \ scale
\ cr ." dXY, stepX/Y " dX ? dY ? stepX . stepY .
  dX @ dY @ > if
    dY @ dX @ 1 rshift -		\ initial error
    begin
      x0 y0 FColor @ setPixel
      x0 x1 <>
     while
      dup 0 >= if
        y0 stepY + to y0  dX @ -
      endif
      x0 stepX + to x0  dy @ +
    repeat
    drop
  else
    dX @ dY @ 1 rshift -
    begin
      x0 y0 FColor @ setPixel
      y0 y1 <>
     while
      dup 0 >= if
        x0 stepX + to x0  dY @ -
      endif
      y0 stepY + to y0  dX @ +
    repeat
    drop
  endif
  x0 X !  y0 Y !
;

: hline		\ x --
\ *G Draw a horizontal line until the destination X is reached.
\ ** This word is much faster than *\i{gline} above.
  dup X @ > if
    dup X @ -				\ -- x w
  else
    dup  X @ over -  swap X !		\ -- x w
  endif
  1 fillRect  X !
;

: h-line        \ x y len --
\ *G Draw a horizontal line of length *\i{len} pixels starting
\ ** at *\i{x,y}.
  -rot >XY  X @ + hline
;

: vline		\ y --
\ *G Draw a vertical line until the destination Y is reached.
\ ** This word is much faster than *\i{gline} above.
  dup Y @ > if
    dup Y @ -				\ -- y h
  else
    dup  Y @ over -  swap Y !		\ -- y h
  endif
  1 swap fillRect  Y !
;

: v-line        \ x y len --
\ *G Draw a vertical line of length *\i{len} pixels starting
\ ** at *\i{x,y}.
  -rot >XY  Y @ + vline
;

: line		\ xd yd --
\ *G Draw a line from the current position to *\i{Xd,Yd}.
\ ** This word uses whichever of the previous line words
\ ** gives the best results.
  over X @ =				\ vertical line
  if  nip vline exit  endif
  dup Y @ = 				\ horizontal line
  if  drop hline exit  endif
  gline
;

: rect		\ w h --
\ *G Draw a rectangle of given width and height at the current
\ ** position.
  { | x0 y0 x1 y1 -- }
  X @ dup to x0  rot + to x1
  Y @ dup to y0  + to y1
  x1 hline  y1 vline  x0 hline  y0 vline
;

: filledRect	\ w h --
\ *G Draw a filled rectangle of given width and height at the
\ ** current position.
  fillRect
;

: absRect	\ x y w h --
\ *G Draw a rectangle of given width and height at the given
\ ** position.
  2swap >XY  rect
;

: absFilledRect	\ x y w h --
\ *G Draw a filled rectangle of given width and height at the
\ ** given position.
  2swap >XY  fillRect
;

: quadrant	{ cx cy rad fclr -- }
\ Draw 4 points for the circle centred at *\i{cx,cy} with
\ offset *\i{rad} in colour *\i{fclr}.
  cx cy rad + fclr setPixel
  cx cy rad - fclr setPixel
  cx rad + cy fclr setPixel
  cx rad - cy fclr setPixel
;

defer octant	\ cx cy dx dy fclr --
\ Draw 8 points for the circle centred at *\i{cx,cy} with
\ offsets *\i{dx,dy} in colour *\i{fclr}.

: octpoints	{ cx cy dx dy fclr -- }
\ Draw 8 points for the circle centred at *\i{cx,cy} with
\ offsets *\i{dx,dy} in colour *\i{fclr}.
  cx dx +  cy dy +  fclr setPixel
  cx dx -  cy dy +  fclr setPixel
  cx dx +  cy dy -  fclr setPixel
  cx dx -  cy dy -  fclr setPixel
  cx dy +  cy dx +  fclr setPixel
  cx dy -  cy dx +  fclr setPixel
  cx dy +  cy dx -  fclr setPixel
  cx dy -  cy dx -  fclr setPixel
;

: octline	\ x1 y1 x2 y2 fclr --
  xy> 2>r  Fcolor @ >r
  Fcolor !
  2swap >xy  line
  r> FColor !  2r> >xy
;

: octlines	{ cx cy dx dy fclr -- }
\ Draw 4 lines for the circle centred at *\i{cx,cy} with
\ offsets *\i{dx,dy} in colour *\i{fclr}.
  cx dx -  cy dy +  cx dx +  cy dy +  fclr octline
  cx dx -  cy dy -  cx dx +  cy dy -  fclr octline
  cx dy -  cy dx +  cx dy +  cy dx +  fclr octline
  cx dy -  cy dx -  cx dy +  cy dx -  fclr octline
;

: (circle)	\ radius --
\ *G Draw a circle of the given radius, using the current
\ ** position as the centre. The current position is not
\ ** changed.
  { rad | ddfX ddfY currX currY -- }

  0 -> ddfX  rad 2* negate -> ddfY
  0 -> currX  rad -> currY

  X @ Y @ rad FColor @ quadrant
  1 rad -  begin			\ -- f ; fractional error
    currX currY <
   while
    dup 0 >= if
      currY 1- -> currY
      ddfY 2+ dup -> ddfY  +
    endif
    currX 1+ -> currX
    ddfX 2+ dup -> ddfX  + 1+
    X @ Y @ currX currY Fcolor @ octant
  repeat
  drop
;

: circle	\ radius --
\ *G Draw a circle of the given radius, using the current
\ ** position as the centre. The current position is not
\ ** changed.
  assign octpoints to-do octant  (circle)
;

: filledCircle	{ radius | xc yc -- }
\ *G Draw a filled circle of the given radius, using the current
\ ** position as the centre. The current position is not
\ ** changed.
  xy> to yc to xc
  assign octlines to-do octant
  radius (circle)
  xc radius - yc >xy  xc radius + yc line
  xc yc >xy
;

: sparkleCircle	\ radius
\ *G Draw a series of circles up to the given radius, using the
\ ** current position as the centre. The current position is not
\ ** changed.
  X @ Y @ FColor @ setPixel		\ centre
  1+ 1 ?do  i circle  loop
;


\ *****************
\ *S Initialisation
\ *****************

: initLCD9320	\ --
  \ Start Initial Sequence
  0x8000 229 lcdReg!		\ Set the internal vcore voltage
  0x0001 0   lcdReg!		\ Start internal OSC.
  0x0100 1   lcdReg!		\ set SS and SM bit
  0x0700 2   lcdReg!		\ set 1 line inversion
  0x1030 3   lcdReg!		\ set GRAM write direction and BGR=1.
  0x0000 4   lcdReg!		\ Resize register - no resizing
  0x0202 8   lcdReg!		\ set the back porch and front porch
  0x0000 9   lcdReg!		\ set non-display area refresh cycle ISC[3:0]
  0x0000 10  lcdReg!		\ FMARK function
  0x0000 12  lcdReg!		\ RGB interface setting
  0x0000 13  lcdReg!		\ Frame marker Position
  0x0000 15  lcdReg!		\ RGB interface polarity

  \ Power On sequence
  0x0000 16  lcdReg!		\ SAP, BT[3:0], AP, DSTB, SLP, STB
  0x0000 17  lcdReg!		\ DC1[2:0], DC0[2:0], VC[2:0]
  0x0000 18  lcdReg!		\ VREG1OUT voltage
  0x0000 19  lcdReg!		\ VDV[4:0] for VCOM amplitude
  200 ms			\ Dis-charge capacitor power voltage (200ms)
  0x17B0 16  lcdReg!		\ SAP, BT[3:0], AP, DSTB, SLP, STB
  0x0137 17  lcdReg!		\ DC1[2:0], DC0[2:0], VC[2:0]
  50 ms
  0x0139 18  lcdReg!		\ VREG1OUT voltage
  50 ms
  0x1d00 19  lcdReg!		\ VDV[4:0] for VCOM amplitude
  0x0013 41  lcdReg!		\ VCM[4:0] for VCOMH
  50 ms
  0x0000 32  lcdReg!		\ GRAM horizontal Address
  0x0000 33  lcdreg!		\ GRAM Vertical Address

  \ Adjust the Gamma Curve (9320)
  0x0006 48 lcdReg!
  0x0101 49 lcdReg!
  0x0003 50 lcdReg!
  0x0106 53 lcdReg!
  0x0b02 54 lcdReg!
  0x0302 55 lcdReg!
  0x0707 56 lcdReg!
  0x0007 57 lcdReg!
  0x0600 60 lcdReg!
  0x020b 61 lcdReg!

  \ Set GRAM area
  0x0000 80 lcdReg!		\ Horizontal GRAM Start Address
  0x00EF 81 lcdReg!		\ Horizontal GRAM End Address
  0x0000 82 lcdReg!		\ Vertical GRAM Start Address
  0x013F 83 lcdReg!		\ Vertical GRAM End Address
  0x2700 96 lcdReg!		\ Gate Scan Line
  0x0001 97 lcdReg!		\ NDL,VLE, REV
  0x0000 106 lcdReg!		\ set scrolling line

  \ Partial Display Control
  0 128 lcdReg!
  0 129 lcdReg!
  0 130 lcdReg!
  0 131 lcdReg!
  0 132 lcdReg!
  0 133 lcdReg!

  \ Panel Control
  0x0010 144 lcdReg!
  0x0000 146 lcdReg!
  0x0003 147 lcdReg!
  0x0110 149 lcdReg!
  0x0000 151 lcdReg!
  0x0000 152 lcdReg!

  \ Set GRAM write direction and BGR = 1
  \ I/D=01 (Horizontal : increment, Vertical : decrement)
  \ AM=1 (address is updated in vertical writing direction)
  0x1018 3 lcdReg!

  0x0173 7 lcdReg!		\ 262K color and display ON
;

: initLCD9325	\ --
  \ Start Initial Sequence
  0x0001 0   lcdReg!		\ Start internal OSC.
  0x0100 1   lcdReg!		\ set SS and SM bit
  0x0700 2   lcdReg!		\ set 1 line inversion
  0x1030 3   lcdReg!		\ set GRAM write direction (++) and BGR=1.
  0x0000 4   lcdReg!		\ Resize register - no resizing
  0x0202 8   lcdReg!		\ set the back porch and front porch
  0x0000 9   lcdReg!		\ set non-display area refresh cycle ISC[3:0]
  0x0000 10  lcdReg!		\ FMARK function
  0x0001 12  lcdReg!		\ RGB interface setting
  0x0000 13  lcdReg!		\ Frame marker Position
  0x0000 15  lcdReg!		\ RGB interface polarity

  \ Power On sequence
  0x0000 16  lcdReg!		\ SAP, BT[3:0], AP, DSTB, SLP, STB
  0x0000 17  lcdReg!		\ DC1[2:0], DC0[2:0], VC[2:0]
  0x0000 18  lcdReg!		\ VREG1OUT voltage
  0x0000 19  lcdReg!		\ VDV[4:0] for VCOM amplitude
  200 ms			\ Dis-charge capacitor power voltage (200ms)
  0x17B0 16  lcdReg!		\ SAP, BT[3:0], AP, DSTB, SLP, STB
  0x0137 17  lcdReg!		\ DC1[2:0], DC0[2:0], VC[2:0]
  50 ms
  0x0139 18  lcdReg!		\ VREG1OUT voltage
  50 ms
  0x1d00 19  lcdReg!		\ VDV[4:0] for VCOM amplitude
  0x0013 41  lcdReg!		\ VCM[4:0] for VCOMH
  50 ms
  0x0000 32  lcdReg!		\ GRAM horizontal Address
  0x0000 33  lcdreg!		\ GRAM Vertical Address

  \ Adjust the Gamma Curve (9325)
    0x0004 $030 lcdReg!
    0x0007 $031 lcdReg!
    0x0006 $032 lcdReg!
    0x0206 $035 lcdReg!
    0x0408 $036 lcdReg!
    0x0507 $037 lcdReg!
    0x0200 $038 lcdReg!
    0x0707 $039 lcdReg!
    0x0504 $03C lcdReg!
    0x0F02 $03D lcdReg!

((  \ Adjust the Gamma Curve (9325)
  0x0007 48 lcdReg!
  0x0302 49 lcdReg!
  0x0105 50 lcdReg!
  0x0206 53 lcdReg!
  0x0808 54 lcdReg!
  0x0206 55 lcdReg!
  0x0504 56 lcdReg!
  0x0007 57 lcdReg!
  0x0105 60 lcdReg!
  0x0808 61 lcdReg!
))

  \ Set GRAM area
  0x0000 80 lcdReg!		\ Horizontal GRAM Start Address
  0x00EF 81 lcdReg!		\ Horizontal GRAM End Address
  0x0000 82 lcdReg!		\ Vertical GRAM Start Address
  0x013F 83 lcdReg!		\ Vertical GRAM End Address
  0xA700 96 lcdReg!		\ Gate Scan Line (GS=1, scan direction is G320~G1)
  0x0001 97 lcdReg!		\ NDL,VLE, REV
  0x0000 106 lcdReg!		\ set scrolling line

  \ Partial Display Control
  0 128 lcdReg!
  0 129 lcdReg!
  0 130 lcdReg!
  0 131 lcdReg!
  0 132 lcdReg!
  0 133 lcdReg!

  \ Panel Control
  0x0010 144 lcdReg!
  0x0000 146 lcdReg!
  0x0003 147 lcdReg!
  0x0110 149 lcdReg!
  0x0000 151 lcdReg!
  0x0000 152 lcdReg!

  \ Set GRAM write direction and BGR = 1
  \ I/D=01 (Horizontal : increment, Vertical : increment)
  \ AM=0 (address is updated in horizontal writing direction)
  0x1038 3 lcdReg!

  0x0133 7 lcdReg!		\ 262K color and display ON
;

0 value lcdid	\ -- x
\ *G returns the LCD identifier.

: initLCD	\ --
  case  0 lcdReg@ dup to lcdid
    $0000:9320 of  initLCD9320  endof
    $0000:9325 of  initLCD9325  endof
      1 abort" Unknown/No LCD controller"
  endcase
;


\ ***************
\ *S Font display
\ ***************
\ *P Character drawing is based on the *\fo{drawChar} word defined
\ ** *\i{FontDefs.fth}. In the implementations here, this word
\ ** uses the current X,Y position and the stack effect is:
\ *C   char --
\ *P X is incremented by the character width.

(( for DocGen
: DrawChar	\ char --
\ *G Draw the character at the current X,Y position. Note
\ ** that *\fo{DrawChar} just calls the routine given in the
\ ** current font table. The parameters for this word may vary
\ ** between CPU and LCD panel.
  CurrFont @ f.xtDrawChar @ execute
;

struct /Font	\ -- len
\ *G The font table data structure from FontDefs.fth
\ *[
  int f.link		\ link to previous font
  int f.height		\ height of character in pixels
  int f.width		\ width of character in pixels
  int f.mwidth		\ width of character in bytes/row
  int f.first		\ ASCII code of first character
  int f.end		\ ASCII code of last+1 character
  int f.charsize	\ size of character in bytes
  int f.xtDrawChar	\ xt of character drawing routine
  			\ char vbuff bytes/row --
  0 field f.map		\ start of character table
end-struct
\ *]
))

\ *P The word *\fo{drawCharGen} below can be used with any font
\ ** as it takes the font information from the current font table.
\ ** Where drawing speed is particularly important, you may care
\ ** to implement your own font-dependent words.


\ *P Three implementations are given here:
\ *(
\ *B The first is for reference only and may be useful when
\ ** developing code for other panels and CPUs. It depends only
\ ** on the word *\fo{setPixel ( x y color -- )}.
\ *B The second is optimised for landscape mode operation of the
\ ** controller.
\ *B The third is optimised for portrait mode operation of the
\ ** controller.
\ *)


\ ===========================
\ *N Reference implementation
\ ===========================
\ *P This code is easy to port to a new CPU or panel.

0 [if]

: drawPixel	\ flag --
\ *G Draw a pixel at the current position. If flag is non-zero,
\ ** the foreground colour is used, otherwise the background
\ ** colour is used.
  XY> rot
  if  FColor @  else  BColor @ endif
  setPixel
;

: drawCharBits	\ byte u --
\ *G Draw the u top bits from the byte.
  swap $080 rot 0 do			\ -- byte mask
    2dup and drawPixel
    1 X +!  1 rshift
  loop
  2drop
;

: drawCharRow	\ font ^charmap --
\ *G Draw one row of pixels.
  X @ >r
  over f.width @ -8 and 3 rshift 0 ?do	\ -- font ^charmap ; complete bytes
    dup c@ 8 drawCharBits  1+
  loop
  over f.width @ 7 and dup >r if
    dup c@ r@ drawCharBits
  endif
  r> drop
  r> X !  2drop
;

: drawCharGen	\ char --
\ *G The generic routine.
  XY> 2>r
  CurrFont @ tuck char>map		\ -- font ^charmap
  over f.height @ 0 do			\ -- font ^charmap
    2dup drawCharRow
    over f.mwidth @ +  1 Y +!
  loop
  2r> >XY  over f.width @ X +!
  2drop
;

[then]


\ **********************
\ *N Landscape operation
\ **********************

Portrait? 0= [if]

: drawCharBits	\ byte u --
\ *G Draw the u top bits from the byte.
  swap $080 rot 0 do			\ -- byte mask
    2dup and
    if  FColor @  else  BColor @  endif
    lcdRAM!  1 rshift
  loop
  2drop
;

: drawCharRow	\ font ^charmap --
\ *G Draw one row of pixels.
  XY> setCursor  lcdRAMprep
  over f.width @ -8 and 3 rshift 0 ?do	\ -- font ^charmap ; complete bytes
    dup c@ 8 drawCharBits  1+
  loop
  over f.width @ 7 and dup >r if
    dup c@ r@ drawCharBits
  endif
  r> drop  2drop
;

: drawCharGen	\ char --
\ *G A generic routine for landscape use.
  XY> 2>r
  CurrFont @ tuck char>map		\ -- font ^charmap
  over f.height @ 0 do			\ -- font ^charmap
    2dup drawCharRow
    over f.mwidth @ +  1 Y +!
  loop
  2r> >XY  over f.width @ X +!
  2drop
;

[then]


\ *********************
\ *N Portrait operation
\ *********************

Portrait? [if]

: drawCharCol	\ font ^charmap col# --
\ *G Draw one column of pixels.
  dup >r  3 rshift +				\ -- font ^charmap'
  r@ 7 and 7 swap - 1 swap lshift		\ -- font ^charmap' mask
  X @ r> + Y @ setCursor  lcdRAMprep		\ start point
  rot dup >r f.mwidth @ -rot			\ -- /row ^charmap' mask
  r> f.height @ 0 ?do				\ -- /row ^charmap' mask
    over c@ over and
    if  FColor @  else  BColor @  endif
    lcdRAM!  -rot  over +  rot			\ -- /row ^charmap'' mask
  loop
  3drop
;

: drawCharGen	\ char --
\ *G A generic routine for portrait use.
  XY> 2>r
  CurrFont @ tuck char>map		\ -- font ^charmap
  over f.width @ 0 do			\ -- font ^charmap
    2dup i drawCharCol
  loop
  2r> >XY  over f.width @ X +!
  2drop
;

[then]


\ =====================
\ *N Higher level words
\ =====================
\ *P These words are used by the GUI code.

: .fchar        \ px py char --
\ *G Write a character at the given pixel address, which is
\ ** stepped to the nearest byte boundary. Uses the implementation
\ ** dependent word *\fo{drawChar}.
  -rot >XY  DrawChar
;

: sout          \ px py caddr len --
\ *G Write a string at the given pixel address, which is
\ ** stepped to the nearest byte boundary.
  bounds ?do			\ -- x y
    2dup i c@ .fchar
    swap /fwidth + swap
  loop
  2drop
;


\ *****************
\ *S Image handling
\ *****************
\ *P Before use, images created on a PC must be converted to
\ ** the format required by the particular LCD panel. There
\ ** are tools to do this in the *\i{Tools\BMPcnv} folder.
\ ** The images must be in 16 bit RGB 565 format.

struct /LCDheader	\ -- len
\ *G The four cell header at the start of MPE image files for
\ ** PowerView 2.
\ *[
  int lcd.PixHeight	\ cell: height in pixels
  int lcd.PixWidth	\ cell: width in pixels
  int lcd./Row		\ cell: row size in bytes
  int lcd./Image	\ cell: size of image
  0 field lcd.Image	\ LCD image starts here
end-struct
\ *]


\ ======================
\ *N Landscape operation
\ ======================

Portrait? 0= [if]

: imageRowX	\ ^img #pixels --
\ *G Write the given number of pixels in the X direction starting
\ ** at the current X Y position.
  XY> setCursor  lcdRAMprep
  2* bounds ?do  i w@ lcdRAM!  2 +loop
;

: load-image	\ x y imageaddr --
\ *G Display a graphics image in memory at *\i{imageaddr} at pixel
\ ** position *\i{x,y}. If part of the image is off the screen, the
\ ** displayed image is clipped to the screen size.
  { | %width %height %rowsize %imgsize %pad -- }
  -rot  2dup >XY 2>r
  dup lcd.PixHeight @ to %height
  dup lcd.PixWidth @ to %width
  dup lcd./Row @ to %rowsize
  dup lcd./Image @ to %imgsize
  lcd.Image				\ -- ^image
  %width X @ + Xmax >
  if  Xmax X @ - to %width  then
  %height Y @ + Ymax >
  if  Ymax Y @ - to %height  then
  %height 0 ?do				\ -- ^image
    dup %width imageRowX
    %rowsize +  1 Y +!
  loop
  drop  2r> >XY
;

[then]


\ =====================
\ *N Portrait operation
\ =====================

Portrait? [if]

: imageColY	\ ^img col# #pixels --
\ *G Write the *\i{#pixels} pixels in the Y direction for
\ ** column *\i{col#} of the image starting at the current X,Y
\ ** position. The parameter *\i{^img} is the address of the
\ ** image header.
  XY> setCursor  lcdRAMprep  >r		\ -- ^img col# ; R: -- #pixels
  over lcd.Image swap 2* +		\ -- ^img addr ; start address of column
  swap lcd./Row @ swap			\ -- rowsize addr
  r> 0 ?do				\ -- rowsize addr
     dup w@ lcdRAM!  over +
  loop
  2drop
;

: load-image	\ x y imageaddr --
\ *G Display a graphics image in memory at *\i{imageaddr} at pixel
\ ** position *\i{x,y}. If part of the image is off the screen, the
\ ** displayed image is clipped to the screen size.
  { | %width %height %rowsize %imgsize %pad -- }
  -rot  2dup >XY  2>r
  dup lcd.PixHeight @ to %height
  dup lcd.PixWidth @ to %width
  dup lcd./Row @ to %rowsize
  dup lcd./Image @ to %imgsize
  %width X @ + Xmax >
  if  Xmax X @ - to %width  then
  %height Y @ + Ymax >
  if  Ymax Y @ - to %height  then
  %width 0 ?do				\ -- ^image
    dup i %height imageColY  1 X +!
  loop
  drop  2r> >XY
;

[then]

((
create testimg1
  data-file %GuiDir%\Pics\temp.c565
constant /testimg1

create testimg2
  data-file %GuiDir%\Pics\200Hx70W.c565
constant /testimg2
))


\ ======
\ *> ###
\ ======

decimal

