\ NokiaSSP0.fth - SPI for Nokia LCD on Olimex LPC1766-STK

((
Copyright (c) 2007, 2008, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20100422 MPE001 Conversion started.
))

only forth definitions
decimal


\ =================
\ *! nokiassp0
\ *T SPI drivers for Nokia LCD
\ =================
\ *P THIS IS PRE-RELEASE CODE.

\ *P The Nokia display is connected as follows.
\ *E   LPC1766       dir PSx  PMx  Display
\ **   P1.20/SCK0    o/p 3/3       SCK
\ \ **   P1.21/SSEL0   o/p 3/0       CS in GPIO mode
\ **   P1.21/SSEL0   o/p 3/0       CS automatic
\ **   P1.23/MISO0   i/p 3/3  3/2  DIO direct input
\ **   P1.24/MOSI0   o/p 3/3       DIO via 1k resistor
\ **   P3.25         o/p 7/0       LCD_RST
\ **   P3.26/PWM1.3  o/p 7/3       LCD_BL, high for full on


\ ***************************
\ *S Setup and initialisation
\ ***************************

((
3 [ps		\ P1 - PINSEL3 values for SSP0 without SSEL
  20 +ps		\ SCK
  23 +ps		\ MISO
  24 +ps		\ MOSI
0 [psm]
  21 +ps		\ CS
ps] equ LCD_PS3val  equ LCD_PS3mask
\ *G P1.20, 21, 23 and 24 values for PINSEL3.
))
3 [ps		\ P1 - PINSEL3 values for SSP0 without SSEL
  20 +ps		\ SCK
  21 +ps		\ SSEL0/CS
  23 +ps		\ MISO
  24 +ps		\ MOSI
ps] equ LCD_PS3val  equ LCD_PS3mask
\ *G P1.20, 21, 23 and 24 values for PINSEL3.

0 [ps 25 +ps  3 [psm] 26 +ps  ps]	\ P3 - PINSEL7 selections
  equ LCD_PS7val  equ LCD_PS7mask
\ *G P3.25 and P3.26 values for PINSEL7.

: initLCD_BL	\ --
\ *G Initialise the LCD backlight by setting up the PWM.
  _PWM1
  bit1 over PWMnTCR + !			\ initial reset
  bit1 over PWMnMCR + !			\ reset on MR0
  bit3 over PWMnPCR + bic!		\ single edge control on PWM3
  bit11 over PWMnPCR + or!		\ enable PWM3
  0 over PWMnPR + !			\ prescale by 1
  $FF over PWMnMR0 + !			\ 8 bit resolution, 0..255
  bit0 over PWMnLER + or!		\ enable match on MR0
  $00 over PWMnMR3 + !			\ initial value 0
  bit3 over PWMnLER + or!		\ enable match on MR3
  bit3 over PWMnTCR + or!		\ enable PWM
  bit1 over PWMnTCR + bic!		\ release reset
  bit0 over PWMnTCR + or!		\ enable counting
  drop
;

((
             CCLK
freq = ------------------
       pdiv * pres * (scr+1)

In this case we force PDIV=1.
))
interpreter
: genSSPdiv	\ freq -- /pres /scr
  system-speed swap / 2 begin		\ -- div pres
    2dup / #256 u>
   while
    2+
  repeat
  tuck / 1-
;
target

100 kHz genSSPdiv  equ initSSP0scr  equ initSSP0pres
\ *G Calculate the initial values of SCR and the prescaler.
initSSP0scr 8 lshift $0008 or equ initSSP0CR0
\ *G Initial value of CR0 for 9 bit use.

: initLCD_SPI	\ --
\ *G Initialise the SPI in 9 bit mode using SSP0.
\ ** Power up and pin selection has already been done.
  _SSP0
  0 over sspCR1 + !			\ CR1 set up
  initSSP0CR0 over sspCR0 + !		\ CR0
  initSSP0pres over sspCPSR + !		\ prescaler
  0 over sspIMSC + !			\ no interrupts
  0 over sspDMACR + !			\ no DMA
  bit1 over sspCR1 + or!		\ enable module
  8 0 do				\ flush Rx FIFO
    dup sspDR + @ drop
  loop
  drop
;

: setLCD_BL	\ u8 --
\ *G Set the LCD backlight level. The useful range is usually
\ ** $40 (off) to $60 (full on).
  _PWM1
  swap over PWMnMR3 + !		\ set level
  bit3 over PWMnLER + or!	\ enable latch
  drop
;

PCLKSELbug? [if]
  $0400 $0C00 ^PCLKSEL1val setMask	\ SSP0 PCLK=CCLK
[then]

: initLCD	\ --
\ *G Initialise SSP0 in 9 bit mode for the LCD driver. Set up the
\ ** required GPIOs and PWM as well.
  _PCONP
  ~pcSPI over bic!			\ disable legacy SPI
  ~pcPWM1 ~pcSSP0 or swap or!		\ enable PWM1 and SSP0
[ PCLKSELbug? 0= ] [if]
  $0400 $0C00 _PCLKSEl1 setMask		\ SSP0 PCLK=CCLK
[then]
  LCD_PS3val LCD_PS3mask _PINSEL3 setMask	\ pin functions
  $4000 $C000 _PINMODE3 setMask		\ no pull up/down on P1.23/MISO
  LCD_PS7val LCD_PS7mask _PINSEL7 setMask
\  bit21 _FIO1  2dup FIOSET + !  FIODIR + or!	\ P1.21 is high CS o/p
  bit25 _FIO3  2dup FIOCLR + !  FIODIR + or!	\ P3.25 is low  LCD_RST o/p

  initLCD_SPI  initLCD_BL
  5 ms  bit25 _FIO3 FIOSET + !  5 ms	\ clear LCD reset
  $55 setLCD_BL  100 ms
;


\ ****************
\ *S SPI transfers
\ ****************
\ *P These routines have been optimised for speed over space
\ ** considerations. For high speed use with SD cards, you will
\ ** get best performance using interrupt driven or DMA driven
\ ** operation.

(( SSP status register
Bit	Symbol	Description			Reset value
0 	TFE	Transmit FIFO Empty. This bit is 1 is the Transmit FIFO is
		empty, 0 if not.		1
1 	TNF	Transmit FIFO Not Full. This bit is 0 if the Tx FIFO is
		full, 1 if not.			1
2	RNE	Receive FIFO Not Empty. This bit is 0 if the Receive
		FIFO is empty, 1 if not.	0
3	RFF	Receive FIFO Full. This bit is 1 if the Receive
		FIFO is full, 0 if not.		0
4	BSY	Busy. This bit is 0 if the SSP controller is idle,
		or 1 if it is currently sending/receiving a frame
		and/or the Tx FIFO is not empty. 0
))

compiler
: lcdXchg	\ bout -- bin
\ *G Write data to SSP0 and return the received byte.
  _SSP0				\ -- bout *ssp
  begin				\ Wait for FIFO to be empty
    dup SSPSR + @ bit0 and
  until
  tuck SSPDR + !		\ -- *ssp ; Write to device
  begin				\ Wait for bus activity to finish
    dup SSPSR + @ bit4 bit2 or and bit2 =
  until
  SSPDR + @ $FF and		\ return received byte
;
target

IsLeaf : lcd!	\ b --
\ *G Write 9 bit data to the SSP and discard the received byte.
  lcdXchg drop  ;

: lcdCmd!	\ b --
\ *G Write a command byte to the LCD.
  $FF and lcd!  ;
: lcdDat!	\ b --
\ *G Write a data byte to the LCD.
  $100 or lcd!  ;

IsLeaf : lcd@	\ -- b
\ *G Write an idle byte to the SSP and return the received byte.
  $FFFF lcdXchg  ;

IsLeaf : lcdIdle	\ --
\ *G Send an 0x1FF byte to the SSP and discard the received byte.
  $FFFF lcdXchg drop  ;

IsLeaf : lcdWr	\ addr -- addr'
\ *G Write byte at *\i{addr} to SPI and increment pointer.
  dup c@ $100 or lcdXchg drop  1+  ;

IsLeaf : lcdRd	\ addr -- addr'
\ *G Read byte from SPI, put it at *\i{addr}, increment pointer.
  $FFFF lcdXchg over c!  1+  ;

((
: +lcdCS	\ --
\ *G Assert CS.
  bit21 _FIO1 FIOCLR + !  ;		\ Set pin low
: -lcdCS	\ --
\ *G Negate CS.
  bit21 _FIO1 FIOSET + !  ;		\ Set pin high
))
: +lcdRST	\ --
\ *G Assert LCD reset (go low).
  bit25 _FIO3 FIOCLR + !  ;
: -lcdRST	\ --
\ *G Remove LCD reset (go high).
  bit25 _FIO3 FIOSET + !  ;

\ --------------- Epson S1D15G10 Command Set  -------------

#define LCD_DISON       $0af
#define LCD_DISOFF      $0ae
#define LCD_DISNOR      $0a6
#define LCD_DISINV      $0a7
#define LCD_COMSCN      $0bb
#define LCD_DISCTL      $0ca
#define LCD_SLPIN       $095
#define LCD_SLPOUT      $094
#define LCD_PASET       $075
#define LCD_CASET       $015
#define LCD_DATCTL      $0bc
#define LCD_RGBSET8     $0ce
#define LCD_RAMWR       $05c
#define LCD_RAMRD       $05d
#define LCD_PTLIN       $0a8
#define LCD_PTLOUT      $0a9
#define LCD_RMWIN       $0e0
#define LCD_RMWOUT      $0ee
#define LCD_ASCSET      $0aa
#define LCD_SCSTART     $0ab
#define LCD_OSCON       $0d1
#define LCD_OSCOFF      $0d2
#define LCD_PWRCTR      $020
#define LCD_VOLCTR      $081
#define LCD_VOLUP       $0d6
#define LCD_VOLDOWN     $0d7
#define LCD_TMPGRD      $082
#define LCD_EPCTIN      $0cd
#define LCD_EPCOUT      $0cc
#define LCD_EPMWR       $0fc
#define LCD_EPMRD       $0fd
#define LCD_EPSRRD1     $07c
#define LCD_EPSRRD2     $07d
#define LCD_NOP         $025

#define LCD_XSIZE       131
#define LCD_YSIZE       131

$100 equ DOFS \ data offset


\ --- startup sequences ---
\ from 0 to $ff -> control, from $100 to $1ff
\ sequence tables
\ In spite of big effort to use the 8 bits color mode, it was IMPOSSIBLE
\ to move the S1D15G10 to initialise on this size!
\

interpreter
: [LCDseq	\ -- addr ; -- addr
  create  here  0 ,  ;
: LCDseq]	\ addr --
  here over - cell - swap !  ;
target

: sendLCDseq	\ addr --
  dup cell+ swap @ bounds
  ?do  i w@ lcd!  2 +loop
;

[LCDseq LCD_initseq	\ -- addr
LCD_DISCTL   w,
    0 DOFS + w,
   32 DOFS + w,
    0 DOFS + w,
    0 DOFS + w,
LCD_COMSCN   w,
    1 DOFS + w,
LCD_TMPGRD   w,
    0 DOFS + w,
LCD_OSCON    w,
LCD_SLPOUT   w,
LCD_VOLCTR   w,
   37 DOFS + w,
    1 DOFS + w,
LCD_PWRCTR   w,
  $0f DOFS + w,
LCD_DISINV   w,
LCD_DATCTL   w,
    0 DOFS + w,         \ invers normal orientation; scan across cols, then rows
    0 DOFS + w,         \ RGB arrangement (RGB all rows/cols)
\    4 DOFS + w,         \ RGB arrangement (RGB row 1, BGR row 2)
\    1 DOFS + w,         \ 8 bit-color display
    2 DOFS + w,         \ 12 bit-color display
\    4 DOFS + w,         \ 16 bit-color display
LCD_RGBSET8  w,          \ only necessary for 8 bits color
\ color table
\ RED
    0 DOFS + w,
    2 DOFS + w,
    4 DOFS + w,
    6 DOFS + w,
    8 DOFS + w,
   10 DOFS + w,
   12 DOFS + w,
   15 DOFS + w,
\ GREEN
    0 DOFS + w,
    2 DOFS + w,
    4 DOFS + w,
    6 DOFS + w,
    8 DOFS + w,
   10 DOFS + w,
   12 DOFS + w,
   15 DOFS + w,
 \ BLUE
    0 DOFS + w,
    4 DOFS + w,
    9 DOFS + w,
   15 DOFS + w,
LCD_PASET    w,
    2 DOFS + w,
  131 DOFS + w,
LCD_CASET    w,
    0 DOFS + w,
  131 DOFS + w,
LCD_RAMWR    w,     \ start RAM write
    0 DOFS + w,
    0 DOFS + w,
    0 DOFS + w,
LCD_NOP      w,
LCDseq]

((
here
0 ,                     \ count
LCD_NOP      w,
here over - cell - over !
constant LCD_SEQUENCE_TABLE2
))

: startLCD	\ --
\  +lcdCS
  LCD_initseq sendLCDseq
  LCD_NOP lcdCmd!
  #150 ms
  LCD_DISON lcdCmd!
\  -lcdCS
;


\ =========
\ Test Code
\ =========

: tcmd		\ -- ; test command write
  begin
\    +lcdcs
    $025 lcd!
\    -lcdcs
    tick-ms ms
  key? until
;


\ ======
\ *> ###
\ ======

decimal
