\ 429I-DiscoSockPuppet.fth - SockPuppet graphics interface STM32F429I-Discovery board

((
Copyright (c) 2012
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! ili932x
\ *T ILI932x LCD controller graphics
\ ===========
\ *P The STM32xx external bus interface is set up in another file.
\ ** For the STM3240G-EVAL board, this is
\ ** */i{Cortex/Hardware/STM32F4/FSMCeval.fth}.

\ *P The display is memory mapped at two locations, each taking
\ ** 16 bit data:
\ *(
\ *B _ELCD + 0 - index/register
\ *B _ELCD + 2 - data
\ *)
\ *P The base address *\fo{_ELCD} is $6800:0000 for the STM3240G-EVAL
\ ** board and the display connected by 16 bit data to the FSMC_NE3
\ ** select.


\ ****************
\ *S Configuration
\ ****************
\ *P Please note that the ILI932x controller documentation refers
\ ** to the panel as 240 by 320, i.e portrait is the default.
\ ** Because of the register format, the controller must be set
\ ** up using datasheet vertical for the longer dimension. Some
\ ** registers, e.g. R20h are restricted to 8 bits.

\ *P By default, the code here operates the display in landscape
\ ** format, width=320, height=240 pixels.

320 constant /ScreenV	\ -- u
\ *G Screen vertical size
240 constant /ScreenH	\ -- u
\ *G Screen horizontal size.
/ScreenV /ScreenH * constant /Screen	\ -- u
\ *G Number of pixels on the screen.

-1 constant Portrait?	\ -- x
\ *G True (non-zero) for portrait mode with the long edge vertical.
Portrait? 0= [if]	\ landscape
  /ScreenV 1- constant XMax
  /ScreenH 1- constant YMax
[then]
Portrait? [if]		\ portrait
  /ScreenH 1- constant XMax
  /ScreenV 1- constant YMax
[then]

1 constant OrgTopLeft?	\ -- x
\ *G True (non-zero) for screen 0,0 at the top left. If set zero,
\ ** 0,0 is at bottom left. The GUI system needs this set to 1.


\ *************
\ *S Primitives
\ *************

variable X	\ -- addr
\ *G Holds the current X position
variable Y	\ -- addr
\ *G Holds the current Y position
variable dX	\ -- addr
\ *G Holds the current X increment, usually -1, 0, or 1.
variable dY	\ -- addr
\ *G Holds the current Y increment, usually -1, 0, or 1.
variable FColor	\ -- addr
\ *G Holds the current foreground colour.
\ ** Colours are 16 bit RGB values in 8:8:8:8 form *\i{ttrrggbb}.
  $FFFFFFFF Fcolor !
variable BColor	\ -- addr
\ *G Holds the current background colour.
\ ** Colours are 16 bit RGB values in 8:8:8:8 form *\i{ttrrggbb}.
  $FF000000 BColor !

interpreter
: RGB		\ rr gg bb -- c8888
\ *G Convert the three 0..255 RGB values to the internal 8:8:8:8 format.
\ ** INTERPRET ONLY.
  rot  $FF and 16 lshift	\ -- gg bb 00rr:0000
  rot  $FF and 8 lshift or	\ -- bb 00rr:gg00
  swap $FF and or		\ -- 00rr:ggbb
  $FF000000 or       		\ -- aarr:ggbb ; add transparency
;

: >RGB		\ c8888 -- rr gg bb
\ *G Convert the internal 8:8:8:8 format to the three 0..255 RGB values.
\ ** INTERPRET ONLY.
  dup 16 rshift $FF and swap	\ -- rr c8888
  dup  8 rshift $FF and swap	\ -- rr gg c888
                $FF and 	\ -- rr gg bb
;
target

$C0 $C0 $C0 RGB constant LCD_COLOR_LTGRAY

\ *G Colour constants
$FF0000FF constant LCD_COLOR_BLUE
$FF00FF00 constant LCD_COLOR_GREEN
$FFFF0000 constant LCD_COLOR_RED
$FF00FFFF constant LCD_COLOR_CYAN
$FFFF00FF constant LCD_COLOR_MAGENTA
$FFFFFF00 constant LCD_COLOR_YELLOW
$FF8080FF constant LCD_COLOR_LIGHTBLUE
$FF80FF80 constant LCD_COLOR_LIGHTGREEN
$FFFF8080 constant LCD_COLOR_LIGHTRED
$FF80FFFF constant LCD_COLOR_LIGHTCYAN
$FFFF80FF constant LCD_COLOR_LIGHTMAGENTA
$FFFFFF80 constant LCD_COLOR_LIGHTYELLOW
$FF000080 constant LCD_COLOR_DARKBLUE
$FF008000 constant LCD_COLOR_DARKGREEN
$FF800000 constant LCD_COLOR_DARKRED
$FF008080 constant LCD_COLOR_DARKCYAN
$FF800080 constant LCD_COLOR_DARKMAGENTA
$FF808000 constant LCD_COLOR_DARKYELLOW
$FFFFFFFF constant LCD_COLOR_WHITE
$FFD3D3D3 constant LCD_COLOR_LIGHTGRAY
$FF808080 constant LCD_COLOR_GRAY
$FF404040 constant LCD_COLOR_DARKGRAY
$FF000000 constant LCD_COLOR_BLACK
$FFA52A2A constant LCD_COLOR_BROWN
$FFFFA500 constant LCD_COLOR_ORANGE
$FF000000 constant LCD_COLOR_TRANSPARENT

: setFColor      \ colour --
\ *G Set the foreground colour.
  dup Fcolor !
  BSP_LCD_SetTextColor
;

: SetBColor  \ colour --
\ *G Set the background colour.
  dup Bcolor !
  BSP_LCD_SetBackColor
;

: setTColor	\ colour --
\ *G Set the text drawing colour. Same as *\fo{setFColor}
   setFColor
;

: XY>		\ -- x y
\ *G Return the current X and Y coordinates.
  X @ Y @
;

: >XY		\ x y --
\ *G Set the current X and Y coordinates
  Y !  X !
;
compiler
: XY>		X @  Y @  ;
: >XY		Y !  X !  ;
target

: .XY		\ --
  ." XY = " X ? Y ?
;

: setCursor	\ x y -- \
\ *G Set the GRAM cursor position.
   >XY
;

: lcdClear	\ colour --
\ *G Clear the screen to the given colour.
  0 0 setCursor
  SAPI_BSP_LCD_Clear
;

: clear-lcd	\ --
\ *G Clear the screen to the current background colour.
  BColor @ lcdClear
  0 0 setCursor  0 0 >XY
;

: setPixel	\ x y color --
\ *G Plot a pixel at *\i{x,y} in the given *\i{color}
  >r  2dup >XY  r> BSP_LCD_DrawPixel
;

Portrait? 0= [if]
: fillX		\ u --
\ *G Write u pixels in the current foreground colour.
   0 ?do  XY> FColor @ setPixel  1 X +!  loop
;

: fillRect	\ w h --
\ *G Fill the area of given width and height at the present X,Y
\ ** position using the current foreground colour.
  { w h -- }
  w if
    h 0 ?do
      X @ Y @ i + setCursor  w fillX
    loop
  endif
;
[then]

Portrait? [if]
: fillY		\ u --
\ *G Write u pixels in the current foreground colour.
  0 ?do  XY> FColor @ setPixel  1 Y +!  loop
;

: fillRect	\ w h --
\ *G Fill the area of given width and height at the present X,Y
\ ** position using the current foreground colour.
  { w h -- }
  h if
    w 0 ?do
      X @ Y @ i + setCursor  h fillY
    loop
  endif
;
[then]

((
: gline		\ xdest ydest --
\ *G Draw a line from the current position to *\i{Xdest,Ydest}.
  { | x0 y0 x1 y1 stepx stepy -- }
\ cr .xy  ."   dest = " over . dup .
  to y1  to x1				\ end
  X @ to x0  Y @ to Y0			\ start
  y1 y0 - dY !  x1 x0 - dX !
  dX @ 0<
  if  dX @ negate dX !  -1 to stepX  else  1 to stepX  endif
  dY @ 0<
  if  dY @ negate dY !  -1 to stepY  else  1 to stepY  endif
  dX @ 1 lshift dX !  dY @ 1 lshift dY ! \ scale
\ cr ." dXY, stepX/Y " dX ? dY ? stepX . stepY .
  dX @ dY @ > if
    dY @ dX @ 1 rshift -		\ initial error
    begin
      x0 y0 FColor @ setPixel
      x0 x1 <>
     while
      dup 0 >= if
        y0 stepY + to y0  dX @ -
      endif
      x0 stepX + to x0  dy @ +
    repeat
    drop
  else
    dX @ dY @ 1 rshift -
    begin
      x0 y0 FColor @ setPixel
      y0 y1 <>
     while
      dup 0 >= if
        x0 stepX + to x0  dY @ -
      endif
      y0 stepY + to y0  dX @ +
    repeat
    drop
  endif
  x0 X !  y0 Y !
;

: hline		\ x --
\ *G Draw a horizontal line until the destination X is reached.
\ ** This word is much faster than *\i{gline} above.
  dup X @ > if
    dup X @ -				\ -- x w
  else
    dup  X @ over -  swap X !		\ -- x w
  endif
  1 fillRect  X !
;

: vline		\ y --
\ *G Draw a vertical line until the destination Y is reached.
\ ** This word is much faster than *\i{gline} above.
  dup Y @ > if
    dup Y @ -				\ -- y h
  else
    dup  Y @ over -  swap Y !		\ -- y h
  endif
  1 swap fillRect  Y !
;
))

: gline		\ xdest ydest --
\ *G Draw a line from the current position to *\i{Xdest,Ydest}.
   XY> 2swap BSP_LCD_DrawLine
;

: hline		\ x --
\ *G Draw a horizontal line until the destination X is reached.
\ ** This word is much faster than *\i{gline} above.
   abs  XY> rot BSP_LCD_DrawHLine
;

: vline		\ y --
\ *G Draw a vertical line until the destination Y is reached.
\ ** This word is much faster than *\i{gline} above.
   abs  XY> rot BSP_LCD_DrawVLine
;

: h-line        \ x y len --
\ *G Draw a horizontal line of length *\i{len} pixels starting
\ ** at *\i{x,y}.
  >r  >XY  r> hline
;

: v-line        \ x y len --
\ *G Draw a vertical line of length *\i{len} pixels starting
\ ** at *\i{x,y}.
  >r  >XY  r> vline
;

: line		\ xd yd --
\ *G Draw a line from the current position to *\i{Xd,Yd}.
\ ** This word uses whichever of the previous line words
\ ** gives the best results.
  over X @ =				\ vertical line
  if  nip vline exit  endif
  dup Y @ = 				\ horizontal line
  if  drop hline exit  endif
  swap X @ + swap  Y @ + gline
;

: rect		\ w h --
\ *G Draw a rectangle of given width and height at the current
\ ** position.
   XY> 2swap BSP_LCD_DrawRect
;

: filledRect	\ w h --
\ *G Draw a filled rectangle of given width and height at the
\ ** current position.
   XY> 2swap BSP_LCD_FillRect
;

: absRect	\ x y w h --
\ *G Draw a rectangle of given width and height at the given
\ ** position.
  BSP_LCD_DrawRect
;

: absFilledRect	\ x y w h --
\ *G Draw a filled rectangle of given width and height at the
\ ** given position.
  BSP_LCD_FillRect
;

: quadrant	{ cx cy rad fclr -- }
\ Draw 4 points for the circle centred at *\i{cx,cy} with
\ offset *\i{rad} in colour *\i{fclr}.
  cx cy rad + fclr setPixel
  cx cy rad - fclr setPixel
  cx rad + cy fclr setPixel
  cx rad - cy fclr setPixel
;

defer octant	\ cx cy dx dy fclr --
\ Draw 8 points for the circle centred at *\i{cx,cy} with
\ offsets *\i{dx,dy} in colour *\i{fclr}.

: octpoints	{ cx cy dx dy fclr -- }
\ Draw 8 points for the circle centred at *\i{cx,cy} with
\ offsets *\i{dx,dy} in colour *\i{fclr}.
  cx dx +  cy dy +  fclr setPixel
  cx dx -  cy dy +  fclr setPixel
  cx dx +  cy dy -  fclr setPixel
  cx dx -  cy dy -  fclr setPixel
  cx dy +  cy dx +  fclr setPixel
  cx dy -  cy dx +  fclr setPixel
  cx dy +  cy dx -  fclr setPixel
  cx dy -  cy dx -  fclr setPixel
;

: octline	\ x1 y1 x2 y2 fclr --
  xy> 2>r  Fcolor @ >r
  setFColor
  2swap >xy  line
  r> setFColor  2r> >xy
;

: octlines	{ cx cy dx dy fclr -- }
\ Draw 4 lines for the circle centred at *\i{cx,cy} with
\ offsets *\i{dx,dy} in colour *\i{fclr}.
  cx dx -  cy dy +  cx dx +  cy dy +  fclr octline
  cx dx -  cy dy -  cx dx +  cy dy -  fclr octline
  cx dy -  cy dx +  cx dy +  cy dx +  fclr octline
  cx dy -  cy dx -  cx dy +  cy dx -  fclr octline
;

: (circle)	\ radius --
\ *G Draw a circle of the given radius, using the current
\ ** position as the centre. The current position is not
\ ** changed.
  { rad | ddfX ddfY currX currY -- }

  0 -> ddfX  rad 2* negate -> ddfY
  0 -> currX  rad -> currY

  X @ Y @ rad FColor @ quadrant
  1 rad -  begin			\ -- f ; fractional error
    currX currY <
   while
    dup 0 >= if
      currY 1- -> currY
      ddfY 2+ dup -> ddfY  +
    endif
    currX 1+ -> currX
    ddfX 2+ dup -> ddfX  + 1+
    X @ Y @ currX currY Fcolor @ octant
  repeat
  drop
;

: circle	\ radius --
\ *G Draw a circle of the given radius, using the current
\ ** position as the centre. The current position is not
\ ** changed.
   >r  XY>  r>  BSP_LCD_DrawCircle
;

: filledCircle	{ radius | xc yc -- }
\ *G Draw a filled circle of the given radius, using the current
\ ** position as the centre. The current position is not
\ ** changed.
   >r  XY>  r>  BSP_LCD_FillCircle
;

: sparkleCircle	\ radius
\ *G Draw a series of circles up to the given radius, using the
\ ** current position as the centre. The current position is not
\ ** changed.
  X @ Y @ FColor @ setPixel		\ centre
  1+ 1 ?do  i circle  loop
;


\ *****************
\ *S Initialisation
\ *****************

0 value lcdid	\ -- x
\ *G returns the LCD identifier.

: initLCD	\ --
   \ *G No action - already initialised by the C SockPuppet code
;


\ ***************
\ *S Font display
\ ***************
\ *P Character drawing is based on the *\fo{drawChar} word defined
\ ** *\i{FontDefs.fth}. In the implementations here, this word
\ ** uses the current X,Y position and the stack effect is:
\ *C   char --
\ *P X is incremented by the character width.

(( for DocGen
: DrawChar	\ char --
\ *G Draw the character at the current X,Y position. Note
\ ** that *\fo{DrawChar} just calls the routine given in the
\ ** current font table. The parameters for this word may vary
\ ** between CPU and LCD panel.
  CurrFont @ f.xtDrawChar @ execute
;

struct /Font	\ -- len
\ *G The font table data structure from FontDefs.fth
\ *[
  int f.link		\ link to previous font
  int f.height		\ height of character in pixels
  int f.width		\ width of character in pixels
  int f.mwidth		\ width of character in bytes/row
  int f.first		\ ASCII code of first character
  int f.end		\ ASCII code of last+1 character
  int f.charsize	\ size of character in bytes
  int f.xtDrawChar	\ xt of character drawing routine
  			\ char vbuff bytes/row --
  0 field f.map		\ start of character table
end-struct
\ *]
))

\ *P The word *\fo{drawCharGen} below can be used with any font
\ ** as it takes the font information from the current font table.
\ ** Where drawing speed is particularly important, you may care
\ ** to implement your own font-dependent words.


\ *P Three implementations are given here:
\ *(
\ *B The first is for reference only and may be useful when
\ ** developing code for other panels and CPUs. It depends only
\ ** on the word *\fo{setPixel ( x y color -- )}.
\ *B The second is optimised for landscape mode operation of the
\ ** controller.
\ *B The third is optimised for portrait mode operation of the
\ ** controller.
\ *)


\ ===========================
\ *N Reference implementation
\ ===========================
\ *P This code is easy to port to a new CPU or panel.

0 [if]

: drawPixel	\ flag --
\ *G Draw a pixel at the current position. If flag is non-zero,
\ ** the foreground colour is used, otherwise the background
\ ** colour is used.
  XY> rot
  if  FColor @  else  BColor @ endif
  setPixel
;

: drawCharBits	\ byte u --
\ *G Draw the u top bits from the byte.
  swap $080 rot 0 do			\ -- byte mask
    2dup and drawPixel
    1 X +!  1 rshift
  loop
  2drop
;

: drawCharRow	\ font ^charmap --
\ *G Draw one row of pixels.
  X @ >r
  over f.width @ -8 and 3 rshift 0 ?do	\ -- font ^charmap ; complete bytes
    dup c@ 8 drawCharBits  1+
  loop
  over f.width @ 7 and dup >r if
    dup c@ r@ drawCharBits
  endif
  r> drop
  r> X !  2drop
;

: drawCharGen	\ char --
\ *G The generic routine.
  XY> 2>r

  2r> >XY  over f.width @ X +!
  2drop
;

[then]



\ =====================
\ *N Higher level words
\ =====================
\ *P These words are used by the GUI code.

: .fchar        \ px py char --
\ *G Write a character at the given pixel address, which is
\ ** stepped to the nearest byte boundary. Uses the implementation
\ ** dependent word *\fo{drawChar}.
  -rot >XY  DrawChar
;

$100 constant |StringPad|
|StringPad| Buffer: StringPad[]

: LCD_DisplayStringAtMode  \ px py caddr len mode --
\ *G Write a string at the given pixel address using the given mode, which is
\ ** stepped to the nearest byte boundary
   >r
   StringPad[] |StringPad| erase
   |StringPad| 2- min 0 max StringPad[] place
   ( px py -- )  StringPad[] count drop  r>  BSP_LCD_DisplayStringAt
;

: sout          \ px py caddr len --
\ *G Write a string at the given pixel address, which is
\ ** stepped to the nearest byte boundary
   LEFT_MODE LCD_DisplayStringAtMode
;


\ *****************
\ *S Image handling
\ *****************
\ *P Before use, images created on a PC must be converted to
\ ** the format required by the particular LCD panel. There
\ ** are tools to do this in the *\i{Tools\BMPcnv} folder.
\ ** The images must be in 16 bit RGB 565 format.

struct /LCDheader	\ -- len
\ *G The four cell header at the start of MPE image files for
\ ** PowerView 2.
\ *[
  int lcd.PixHeight	\ cell: height in pixels
  int lcd.PixWidth	\ cell: width in pixels
  int lcd./Row		\ cell: row size in bytes
  int lcd./Image	\ cell: size of image
  0 field lcd.Image	\ LCD image starts here
end-struct
\ *]


\ ======================
\ *N Landscape operation
\ ======================

Portrait? 0= [if]

: load-image	\ x y imageaddr --
\ *G Display a graphics image in memory at *\i{imageaddr} at pixel
\ ** position *\i{x,y}. If part of the image is off the screen, the
\ ** displayed image is clipped to the screen size.
   BSP_LCD_DrawBitmap
;

[then]


\ =====================
\ *N Portrait operation
\ =====================

Portrait? [if]

: load-image	\ x y imageaddr --
\ *G Display a graphics image in memory at *\i{imageaddr} at pixel
\ ** position *\i{x,y}. If part of the image is off the screen, the
\ ** displayed image is clipped to the screen size.
   BSP_LCD_DrawBitmap
;

[then]

((
create testimg1
  data-file %GuiDir%\Pics\temp.c565
constant /testimg1

create testimg2
  data-file %GuiDir%\Pics\200Hx70W.c565
constant /testimg2
))


\ ======
\ *> ###
\ ======

decimal

