((
*****************************
Configure FSMC for SRAM & LCD
FSMC = Flexible Static Memory Controller
*****************************

External SRAM and LCD operations
================================

External memory controller operations
NB : FSMC controls LCD, and FMC controls SDRAM

Configure FSMC to interface with LCD and SDRAM.
1. Enable the clock for the FSMC - see enFSMC
2. All GPIO registers are enabled by initGPIO in gpioSTM32F42x.fth
3. Configure all involved FSMC pins on Alt. Fn 12 - see initFSMC
4. For LCD, configure BCR3 & BTR3 registers
))

\          function
PD  4 pio: pinNOE
PD  5 pio: pinNWE
PE  0 pio: pinNBL0
PE  1 pio: pinNBL1

PD  7 pio: pinNE1  \ SDRAM uses this
\ PG  9 pio: pinNE2  \ not used
PG 10 pio: pinNE3	\ LCD
\ LCD RESET pin is connected directly to NRST on DISC1 board.

PB  5 pio: pinSDCKE1
PB  6 pio: pinSDNE1
PC  0 pio: pinSDNWE
PG  8 pio: pinSDCLK
PG 15 pio: pinSDNCAS
PG  4 pio: pinBA0
PG  5 pio: pinBA1
PF 11 pio: pinNRAS

PG  1 pio: pinA11
PG  0 pio: pinA10
PF 15 pio: pinA9
PF 14 pio: pinA8
PF 13 pio: pinA7
PF 12 pio: pinA6
PF  5 pio: pinA5
PF  4 pio: pinA4
PF  3 pio: pinA3
PF  2 pio: pinA2
PF  1 pio: pinA1
PF  0 pio: pinA0

PD 10 pio: pinD15
PD  9 pio: pinD14
PD  8 pio: pinD13
PE 15 pio: pinD12
PE 14 pio: pinD11
PE 13 pio: pinD10
PE 12 pio: pinD9
PE 11 pio: pinD8
PE 10 pio: pinD7
PE  9 pio: pinD6
PE  8 pio: pinD5
PE  7 pio: pinD4
PD  1 pio: pinD3
PD  0 pio: pinD2
PD 15 pio: pinD1
PD 14 pio: pinD0

create FSMCpins	\ -- addr
\ Zero terminated list of GPIO pin structures.
pinNOE    , pinNWE    , 		\ used only by LCD
pinNBL0   , pinNBL1   ,
pinNE1    , ( pinNE2 , ) pinNE3  ,	\ NE2 not used
pinSDCKE1 , pinSDNE1  , pinSDNWE ,
pinSDCLK  , pinSDNCAS , pinNRAS  ,
pinBA0    , pinBA1    ,

pinA11 , pinA10 , pinA9  , pinA8  ,
pinA7  , pinA6  , pinA5  , pinA4  ,
pinA3  , pinA2  , pinA1  , pinA0  ,

pinD15 ,	pinD14 ,	pinD13 ,	pinD12 ,
pinD11 ,	pinD10 ,	pinD9  ,	pinD8  ,
pinD7  ,	pinD6  ,	pinD5  ,	pinD4  ,
pinD3  ,	pinD2  ,	pinD1  ,	pinD0  ,
0 ,

: initFSMCpins		\ for LCD and SDRAM
  FSMCpins		\ address of zero-terminated pin list
   begin 			\ set up the pins for the FSMC
    dup @
   while				\ stops when last 0 entry found
    12 over @ isFunction		\ AF12
    100MHZ_hs over @ isPinSpeed	\ 100 MHz
    0 over @ isPinOD		\ push-pull
    dup @ NotPulled		\ no pull up/down
    cell +
  repeat  drop
;

((
SRAM type devices are connected to Bank 1, subdivided into 4 regions.
   6000:0000  NOR/PSRAM1 - used for external 64 Mb SRAM
   6400:0000  NOR/PSRAM2 - used for external 64 Mb SRAM
   6800:0000  NOR/PSRAM3 - used for external LCD
   6C00:0000  NOR/PSRAM4

ILI9325 LCD Configuration (BCR3 and BTR3)
=========================================

BCR3 regs : see p1553 of RM0090
  bit 19  CBURSTRW  = 0 for async PSRAM
  bit 15  ASYNCWAIT = 0 since ASYNCWAIT not supported
  bit 14  EXTMOD    = 0, 1 for EXTMOD dis-, en-abled
  bit 13  WAITEN    = 0 no effect on async
  bit 12  WREN      = 1 write enabled
  bit 11  WAITCFG   = 0 [since no ASYNCWAIT]
  bit 10  WRAPMOD   = 0 no effect
  bit  9  WAITPOL   = 0 [since no ASYNCWAIT]
  bit  8  BURSTEN   = 0 synchronous bursts disabled
  bit  7  reserved  = 1 [omitted from register mask]
  bit  6  FACCEN    = 0 NOR Flash access disabled
  bits5:4 MWID      = 01 for 16 bits
  bits3:2 MTYPE     = 00 for SRAM
  bit  1  MUXEN     = 0  addr/data not multiplexed
  bit  0  MBKEN     = 1 ENABLE FSMC
So bits 12, 7, 4 & 0 need to be set; hence BCR3 = $00001091

Settings for BTR3
-----------------
If EXTMOD is set to 1, different timings for write can be set in BWTRx.
If EXTMOD = 0, same timings [in BTRx] are used for read & write.
ILI9325 LCD works OK with same R & W timings [BTR3 = $0000:0504], but
is still exactly as fast with $0000:0605, so use that for safety.
No need to define or set BWTR3.

Choice of address for _ELCD
---------------------------
FSMC uses memory bank 1 - 4 in blocks of 16 MB from
addresses $6000:0000 to $6FFF:FFFF (RM0090 p543).
_NE3 (LCD chip select) is 3rd block - $6800:0000 to $6BFF:FFFF, so ANY
address in this range will trigger FSMC_NE3 and the external LCD.
))

 $0000:1091 equ BCR3	\ LCD: EXTMOD is NOT set, so BWTR3 unnecessary
			\ EN FSMC bit is set here, not in initFSMC

\                            type   R/W   ------ timings -------
\ ILI9325                   \                 ns      HCLK @ 6ns
 $0000:0605 equ BTR3	\ LCD    R    36     30    25     1

 $6800:0000 constant _ELCD	\ $6BFF:FFF0 constant _ELCD also OK -see above
\ _ELCD is sued by LCDreg! and LCDreg@

: enFSMC	\ -- ; enable clock for FSMC & FMC
  _RCC
  bit0 over rccAHB3RSTR + bic!	\ remove reset
  bit0 over rccAHB3ENR  +  or!	\ enable clock
  drop
;

: *FSMC	\ disable clock
  _RCC
  bit0 over rccAHB3RSTR + bic!	\ remove reset
  bit0 over rccAHB3ENR  + bic!	\ disable clock
  drop
;

: initFSMC  	\ -- ; Initialise FSMC for LCD
  initFSMCpins			\ for FSMC and FMC
  BTR3  _FSMC fsmc.BTR3  +  !	\ LCD read timing reg
  BCR3  _FSMC fsmc.BCR3  +  !	\ LCD control reg & enable FSMC/LCD
;


\ SDRAM Configuration
\ ===================

\ **** DO NOT CHANGE THESE VALUES !!! ****
\ Follow configuration instructions on p1661 of RM0090.
\ All commands issued simultaneously to CBT1 and CBT2.

$0019    equ /SDRAMCLK		\ Start SDRAM clock
$001A    equ PALL		  	\ All Bank Precharge
$011B    equ AREF			\ Auto-refresh = 8
$601C    equ LMR			\ Load Mode Register
3000     equ REFRC			\ refresh rate count

: initFMC		\ initialise FMC for SDRAM
   $000029D4 _FMC fmc_SDCR1 + !	\ control reg for Bank 1
   $000029D4 _FMC fmc_SDCR2 + !	\ control reg for Bank 2
   $02226462 _FMC fmc_SDTR1 + !	\ timings for Bank 1
   $02226462 _FMC fmc_SDTR2 + !	\ timings for Bank 2
   /SDRAMCLK _FMC fmc_SDCMR + !	\ start clock (normal mode)
   8375 do_delay			\ wait for 200 us [p. 1661]
   PALL      _FMC fmc_SDCMR + !	\ All Bank Precharge command
   AREF      _FMC fmc_SDCMR + !	\ Auto-refresh commands
   LMR       _FMC fmc_SDCMR + !	\ Load Mode Register
   REFRC     _FMC fmc_SDRTR + !	\ refresh rate count
;

: initFSMC&FMC		\ both use same clock
\   bit10 _SYSCFG sysMEMRMP + or!	\ no memory remap [p295 of RM0090]
   enFSMC
   initFSMCpins
   initFSMC
   initFMC
;

 ' initFSMC&FMC atCold


