\ Demo.man - how to build the SockPuppet demonstration

\ =============
\ *> sockpuppet
\ *S Building the demonstration
\ =============
\ *P The demonstration code is for an STM32F429I Discovery board,
\ ** which includes a QVGA colour display.

\ *P The Forth sources are provided in the Forth cross-compiler
\ ** distribution in the *\i{Lib/SockPuppet} directory. The Forth
\ ** toolchain is the MPE Forth cross compiler.

\ *P The demonstration C source code is provided as a ZIP file
\ ** with the cross compiler download. The C
\ ** toolchain is the version of GCC maintained by ARM at:
\ *C   https://launchpad.net/gcc-arm-embedded

\ *P Flash programming is performed by ST's STM32 ST-Link utility,
\ ** usually found at:
\ *C   http://www2.st.com/content/st_com/en/products
\ *C   /embedded-software/development-tool-software/stsw-link004.html
\ *P The ST-Link software and drivers must be installed before the
\ ** board can be programmed.


\ =======================
\ *N Building the C layer
\ =======================
\ *P Windows Batch files are supplied in the *\i{C_files} folder:
\ *(
\ *B MakeAll.bat  make the C library system
\ *B MakeAllFlash.bat  make the C library system and download
\ ** it to the target Flash.
\ *)
\ *P These files trundle around the distribution to run the make
\ ** systems, gcc and to program the Flash.

\ *P The C system is based on the STM32F4 LCD system by Pierpaolo
\ ** Bagnasco, available from:
\ *C   http://www.pierpaolobagnasco.com/category/stm32f4xx/
\ *C   http://www.pierpaolobagnasco.com/2014/07/13/stm32f429-discovery-display/

\ *P Pierpaolo uses Eclipse to auto-generate makefiles so that the
\ ** ST supplied libraries can be compiled using the GNU C ARM command
\ ** line interface tools
\ *C   http://thehackerworkshop.com/?p=1056
\ *P To avoid being tied to any particular IDE, the makefiles are now
\ ** edited manually. If you want to know the gory details of editing
\ ** makefiles produced by Eclipse, Google is your friend.

\ *P The following files have been modified or added:
\ *(
\ *B C_files\src\main.c  (modified)
\ *B C_files\src\ SockPuppetInterface.c/h (added)
\ *B C_files\src\stm32f4xx_it.c (modified)
\ *)

\ *P *\i{Main.c} initialises hardware components that we are using,
\ ** then enters an infinite loop testing for the presence of the
\ ** Forth system in the high 1 Mbyte of FLASH memory. If found,
\ ** the C system jumps to the StartCortex entry point in the
\ ** Forth system.

\ *P *\i{SockPuppetInterface.c/h} define the SockPuppet interface
\ ** for the C toolchain.

\ *P *\i{stm32f4xx_it.c} defines the SVC exception handler that
\ ** jumps to the function in the SVC_syscall_table corresponding
\ ** to the SVC number.

\ *P The C compiler used is the GNU Tools ARM Embedded\5.2 2015q4
\ ** gcc-arm-none-eabi-5_2-2015q4-20151219-win32.exe available from
\ ** here:
\ *C   https://launchpad.net/gcc-arm-embedded/5.0/5-2015-q4-major

\ *P The makefile Make.exe is from GNU ARM Eclipse\Build Tools\2.6-201507152002
\ ** gnuarmeclipse-build-tools-win32-2.6-201507152002-setup.exe available from
\ ** here:
\ *C http://sourceforge.net/projects/gnuarmeclipse/files/Build%20Tools/
\ *P A copy of make.exe is included in the C_files\Debug\ directory.

\ *P The output file is *\i{C_files\Debug\Display.hex} and runs from location
\ ** 0x0800000. 1 Mbyte of Flash and 128K bytes of RAM are allocated for
\ ** the C system. This is grotesquely over the top but works. For a
\ ** production environment the memory map should be edited.


\ ============================
\ *N Building the Forth system
\ ============================
\ *P There is a control file for the SockPuppet demo:
\ *C   Cortex/Hardware/STM32F4/SP429disco.ctl
\ *P Edit this file so that the line
\ *C  afterwards sh "C:\MyApps\ST-Linkv3.8.1\ST-LINK Utility\ST-LINK_CLI.exe"  -P SP429DISCO.hex  -Rst
\ *P contains the correct path to the command line version of the
\ ** ST-Link utility.

\ *P Set up a new AIDE project to compile this file or compile it
\ ** directly. At the end the Forth image will have been programmed
\ ** and the application run.


\ ========================
\ *N Required target files
\ ========================
\ *P The build processes should have programmed the required files.
\ ** In case you just have the hex files, you can program them into
\ ** the board using the ST-Link utility. Find the files
\ *(
\ *B Display.hex - the C image. Usually in *\i{C_files\Debug\Display.hex}.
\ *B SP429DISCO.hex - the Forth image. Usually in
\ ** *\i{<xArmCortex>\Cortex\Hardware\STM32F4\SP429DISCO.hex}.
\ *)


\ =============
\ *N Memory map
\ =============
\ *P The STM32F429 chip has 2 Mbytes of Flash and 256 Kbytes of
\ ** RAM on chip. The available memory is divided equally between
\ ** the C and the Forth portions:
\ *E C System Flash      0x08000000 0x100000
\ ** Forth System Flash  0x08100000 0x100000
\ ** C System RAM        0x20000000 0x020000
\ ** Forth System RAM    0x20020000 0x020000

\ *P The STM32F429 fetches the stack and PC from address 0 at
\ ** reset, and 0x400 bytes from 0x08000000 are initially mapped
\ ** to address 0, allowing the chip to effectively boot from the
\ ** start of the C system Flash at 0x08000000.

\ *P The C system checks the location 4 bytes offset from the
\ ** start of the Forth system Flash, 0x08100004, which contains the
\ ** address of the word *\fo{StartCortex} in the Forth system.
\ ** Note that the value at this address always has bit 0 set to 1,
\ ** indicating Thumb code, since the entire chip runs in Thumb mode.
\ ** The value at the start of the Forth system Flash, 0x08100000,
\ ** normally contains the address to be used for the stack, but
\ ** the xArmCortex Forth start-up code has been modified not to
\ ** use this value, but to continue to use the C system stack.
\ ** This allows C library functions to be called in their own
\ ** stack environment.

\ *P If the value at 0x08100004 is not 0xFFFFFFFF or 0x00000000
\ ** the C system hands control over to the Forth system, which
\ ** can then access C library functions via the SockPuppet interface.

\ ======
\ *> ###
\ ======

