/**
  ******************************************************************************
  * @file    main.c
  * @author  Pierpaolo Bagnasco
  * @version V0.1
  * @date    15-July-2014
  * @brief   This file provides an example of how to use the LCD of the
  * 		 STM32F429-DISCOVERY.
  ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f429i_discovery_lcd.h"
#include "stm32f4xx_hal_rtc.h"
#include "stm32f429xx.h"
#include <stdlib.h>

void Touchscreen_CalibrationPlus( void );
void Touchscreen_Calibration( void );

/* Defines related to Clock configuration */
#define RTC_ASYNCH_PREDIV  0x7F   /* LSE as RTC clock */
#define RTC_SYNCH_PREDIV   0x00FF /* LSE as RTC clock */

#define FORTH_ADDR (0x08100000U)    // the start address of the Forth system

unsigned long maincounter = 0;	// counting the main while loop, test only
RTC_HandleTypeDef RTC_Handle;

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);

/* Private functions ---------------------------------------------------------*/

uint32_t app_restart_requested = 0;

typedef int (*fptr)(void);

fptr MyFunctionPtr;

void user_app_launch( uint32_t* appAddress )
{
    uint32_t MyAddress;

    MyAddress = (uint32_t)appAddress + 4;
    MyFunctionPtr = (fptr)( *( (uint32_t*)MyAddress ) );
    MyFunctionPtr();
}

uint8_t user_app_present( uint32_t* appAddress )
{
    uint32_t MyAddress;
    MyAddress = (uint32_t)appAddress + 4;
    MyFunctionPtr = (fptr)( *( (uint32_t*)MyAddress ) );

    if( ( (uint32_t)MyFunctionPtr != 0xFFFFFFFF ) && ( (uint32_t)MyFunctionPtr != 0x00000000 ) )
    {
	return( 1 );
    }else{
	return( 0 );
    }
}

void EnableAllClocks( void )
{
    RCC->APB2ENR |= RCC_APB2ENR_TIM1EN;                     // Enable clock
    TIM1->PSC = 23;                                         // Prescaler = 23 (1 tick per half microsecond)
    TIM1->ARR = 19999;                                      // Auto-reload = PWM frequency in half microseconds = 10ms period
    TIM1->CCMR1 |= TIM_CCMR1_OC1M_1 | TIM_CCMR1_OC1M_2;     // OC1M = 110 for PWM Mode 1 output on ch1
    TIM1->CCMR1 |= TIM_CCMR1_OC1PE;                         // Output 1 preload enable
    TIM1->CR1 |= TIM_CR1_ARPE;                              // Auto-reload preload enable
    TIM1->CCER |= TIM_CCER_CC1E;                            // Enable output for ch1
    TIM1->CCR1 = 1800;                                      // CCR1 = Duty cycle in half microseconds = high for 900us
    TIM1->EGR |= TIM_EGR_UG;                                // Force update
    TIM1->SR &= ~TIM_SR_UIF;                                // Clear the update flag
    TIM1->DIER |= TIM_DIER_UIE;                             // Enable interrupt on update event
//    NVIC_EnableIRQ(TIM1_BRK_UP_TRG_COM_IRQn);               // Enable IRQ
    TIM1->CR1 |= TIM_CR1_CEN;                               // Enable counter
}


/**
 * @brief   Main program
 * @param  None
 * @retval None
 */
int main(void)
{
    HAL_Init();

    /* Configure the system clock */
    SystemClock_Config();

    BSP_LED_Init( LED3 );
    BSP_LED_Init( LED4 );
    BSP_LED_Off( LED3 );
    BSP_LED_On( LED4 );

    BSP_PB_Init( BUTTON_KEY , BUTTON_MODE_GPIO );
/*
    if( BSP_PB_GetState( BUTTON_MODE_GPIO ) )
    {
        BSP_LED_On( LED3 );
    }
    else
    {
        BSP_LED_Off( LED3 );
    }
*/

    /* Configure RTC prescaler and RTC data registers as follows:
    - Hour Format = Format 24
    - Asynch Prediv = Value according to source clock
    - Synch Prediv = Value according to source clock
    - OutPut = Output Disable
    - OutPutPolarity = High Polarity
    - OutPutType = Open Drain */
    RTC_Handle.Instance = RTC;
    RTC_Handle.Init.HourFormat = RTC_HOURFORMAT_24;
    RTC_Handle.Init.AsynchPrediv = RTC_ASYNCH_PREDIV;
    RTC_Handle.Init.SynchPrediv = RTC_SYNCH_PREDIV;
    RTC_Handle.Init.OutPut = RTC_OUTPUT_DISABLE;
    RTC_Handle.Init.OutPutPolarity = RTC_OUTPUT_POLARITY_HIGH;
    RTC_Handle.Init.OutPutType = RTC_OUTPUT_TYPE_OPENDRAIN;

    if(HAL_RTC_Init(&RTC_Handle) != HAL_OK)
    {
      /* Initialization Error */
      // Error_Handler();
    }

    EnableAllClocks();

//    Touchscreen_CalibrationPlus();

    BSP_LCD_Init();
    BSP_LCD_LayerDefaultInit(0, (uint32_t) LCD_FRAME_BUFFER);
//	BSP_LCD_SetLayerWindow(0, 0, 0, 240, 160);
    BSP_LCD_SetLayerVisible(0, ENABLE);

    BSP_LCD_SelectLayer(0);
    BSP_LCD_SetBackColor(LCD_COLOR_BLACK);
    BSP_LCD_SetTextColor(LCD_COLOR_GREEN);
    BSP_LCD_DisplayOn();

    BSP_LCD_Clear(LCD_COLOR_BLACK);

    while( 1 )
    {
        BSP_LCD_Clear(LCD_COLOR_BLACK);
	BSP_LCD_DisplayStringAtLineMode(1, (uint8_t *) "MPE", CENTER_MODE);
	BSP_LCD_DisplayStringAtLineMode(2, (uint8_t *) "SockPuppet", CENTER_MODE);
	BSP_LCD_DisplayStringAtLineMode(3, (uint8_t *) "Demo", CENTER_MODE);
//	BSP_LCD_DisplayStringAtLineMode(5, (uint8_t *) "2016 Jan 23", CENTER_MODE);

	if ( user_app_present( (uint32_t*)FORTH_ADDR ) == 1)
	{
            BSP_LCD_SetFont(&Font20);
	    BSP_LCD_SetTextColor(LCD_COLOR_GREEN);
	    BSP_LCD_DisplayStringAtLineMode(  6, (uint8_t *) "Starting App", CENTER_MODE);
            BSP_LCD_SetTextColor(LCD_COLOR_BLUE);
	    BSP_LCD_DisplayStringAtLineMode(  7, (uint8_t *) "VFX Forth for ARM", CENTER_MODE);
            BSP_LCD_DisplayStringAtLineMode(  8, (uint8_t *) "UART1 @ 115200", CENTER_MODE);
            BSP_LCD_DisplayStringAtLineMode(  9, (uint8_t *) "PA9 for U1 TX", CENTER_MODE);
            BSP_LCD_DisplayStringAtLineMode( 10, (uint8_t *) "PA10 for U1 RX", CENTER_MODE);
            // HAL_Delay( 500 );
	    user_app_launch( (uint32_t*)FORTH_ADDR );
	}else{
	    BSP_LCD_SetTextColor(LCD_COLOR_RED);
	    BSP_LCD_DisplayStringAtLineMode( 8, (uint8_t *) "No App found", CENTER_MODE);
	}

    	maincounter++;
	HAL_Delay( 800 );
	BSP_LCD_Clear(LCD_COLOR_BLACK);
	HAL_Delay( 200 );
    }
}

/**
 * @brief  System Clock Configuration
 *         The system Clock is configured as follow :
 *            System Clock source            = PLL (HSE)
 *            SYSCLK(Hz)                     = 180000000
 *            HCLK(Hz)                       = 180000000
 *            AHB Prescaler                  = 1
 *            APB1 Prescaler                 = 4
 *            APB2 Prescaler                 = 2
 *            HSE Frequency(Hz)              = 8000000
 *            PLL_M                          = 8
 *            PLL_N                          = 360
 *            PLL_P                          = 2
 *            PLL_Q                          = 7
 *            VDD(V)                         = 3.3
 *            Main regulator output voltage  = Scale1 mode
 *            Flash Latency(WS)              = 5
 *         The LTDC Clock is configured as follow :
 *            PLLSAIN                        = 192
 *            PLLSAIR                        = 4
 *            PLLSAIDivR                     = 8
 * @param  None
 * @retval None
 *
 * COPYRIGHT(c) 2014 STMicroelectronics
 */
void SystemClock_Config(void)
{
    RCC_ClkInitTypeDef RCC_ClkInitStruct;
    RCC_OscInitTypeDef RCC_OscInitStruct;
    RCC_PeriphCLKInitTypeDef PeriphClkInitStruct;

    /* Enable Power Control clock */
    __PWR_CLK_ENABLE();

    /* The voltage scaling allows optimizing the power consumption when the device is
       clocked below the maximum system frequency, to update the voltage scaling value
       regarding system frequency refer to product datasheet.  */
    __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

    /*##-1- System Clock Configuration #########################################*/
    /* Enable HSE Oscillator and activate PLL with HSE as source */
    RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
    RCC_OscInitStruct.HSEState = RCC_HSE_ON;
    RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
    RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
    RCC_OscInitStruct.PLL.PLLM = 8;
    RCC_OscInitStruct.PLL.PLLN = 360;
    RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
    RCC_OscInitStruct.PLL.PLLQ = 7;
    HAL_RCC_OscConfig(&RCC_OscInitStruct);

    HAL_PWREx_ActivateOverDrive();

    /* Select PLL as system clock source and configure the
       HCLK, PCLK1 and PCLK2 clocks dividers */
    RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
    RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
    RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
    RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
    RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;
    HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5);

    /*##-2- LTDC Clock Configuration ###########################################*/
    /* LCD clock configuration */
    /* PLLSAI_VCO Input = HSE_VALUE/PLL_M = 1 Mhz */
    /* PLLSAI_VCO Output = PLLSAI_VCO Input * PLLSAIN = 192 Mhz */
    /* PLLLCDCLK = PLLSAI_VCO Output/PLLSAIR = 192/4 = 48 Mhz */
    /* LTDC clock frequency = PLLLCDCLK / RCC_PLLSAIDIVR_8 = 48/8 = 6 Mhz */
    PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_LTDC;
    PeriphClkInitStruct.PLLSAI.PLLSAIN = 192;
    PeriphClkInitStruct.PLLSAI.PLLSAIR = 4;
    PeriphClkInitStruct.PLLSAIDivR = RCC_PLLSAIDIVR_8;
    HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct);
}
