\ Touchscreen.fth - STM32F429I touchscreen functions

((
Copyright (c) 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

Please note that MPE files have hard tabs set to 8. Hence, tabs
are at columns 1, 9, 17, 25 ...

To do
=====

Change history
==============
))

only forth definitions
decimal

\ ==============
\ *! touchscreen
\ *T STM32F429I touchscreen functions
\ ==============

0 value tsInitialised	\ -- flag
\ *G Returns true when the touchscreen has been initialised.

2variable tsLastPos	\ -- addr
\ *G Holds the position of the last touchscreen press as 32 bit
\ ** x and y values. The y coordinate is at the low address so:
\ *C   tsLastPos 2@ ( -- x y )

: tsInit    \ --
\ *G Initialise the touchscreen.
  tsInitialised 0= if
    __SAPI_34_Touchscreen_Cal FN
    -1 to tsInitialised
  then
;

interpreter
: uint16_t  2 field  ;
target

struct /TS_State	\ -- len
\ *G Structure used to hold the touchscreen state.
  uint16_t ts.TouchDetected
  uint16_t ts.X
  uint16_t ts.Y
  uint16_t ts.Z
end-struct

/TS_State Buffer: TS_State[]	\ -- addr
\ *G A global buffer holding the touchscreen state.

: tsKey?	\ -- flag
\ *G Return non-zero if the touchscreen is being pressed.
  tsInit
  TS_State[] SAPI_33_BSP_TS_GetState
  TS_State[] ts.TouchDetected w@
;

: tsWaitDown	\ --
\ *G Wait until the screen has been touched. Some debounce should
\ ** be added.
  begin  tskey? 0= while  20 ms  repeat
;

: tsWaitUp	\ --
\ *G Wait until the screen is no longer being touched. Some
\ ** debounce should be added.
  begin  tskey? while  20 ms  repeat
;

: tsWaitKey	\ -- x y
\ *G Wait until the screen has been touched and return the
\ ** position.
  tsWaitDown
  TS_State[] dup ts.X w@  swap ts.Y w@
;

: tsWaitPress	\ --
\ *G Wait until a screen press takes place and save the x,y
\ ** coordinates in the variable *\fo{tsLastPos}.
  tsWaitDown
  TS_State[] dup ts.X w@  swap ts.Y w@
  tsLastPos 2!
;

((
: tsKey    \ -- x y -1 | 0
\ *G Return the x,y coordinate of the last place touched on the
\ ** touchscreen, and true if still pressed. If the touchscreen
\ ** is not being pressed, just return zero.
  tsWaitDown
  TS_State[] dup ts.TouchDetected w@ if	\ touched
    dup ts.X w@  swap ts.Y w@  -1
  else
    drop  0
  then
;
))


\ ************
\ *S Test code
\ ************
\ *P Edit the conditional compilation at the end of the file
\ ** to enable or disable the test code.

1 [if]

: ttts   \ --
\ *G display the X and Y coordinates when the touchscreen is pressed
  cr ." Press any key to exit, touch the touchscreen to show (x,y) coordinates
  begin
    tsWaitPress  tsLastPos 2@
    cr ." x y = " swap . .  tsWaitUp
  key? until
;

[then]


\ ======
\ *> ###
\ ======

decimal
