\ CdcConfigSTM32F0x2.fth - USB Communications Device Class configuration

((
Copyright (c) 2008, 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20171203 SFP001 Updated to use ST INF file.
))

only forth definitions
decimal

\ ==========
\ *! cdcconfig
\ *T USB CDC configuration
\ =========
\ *P The file *\i{CDCconfig.fth} contains a USB configuration for
\ ** a Communications Device Class (CDC) application.


\ ************************
\ *S Primary Configuration
\ ************************
\ *P Note that interface numbers must be a contiguous set
\ ** starting at 0.

0 equ dbgUHW?	\ -- flag
\ *G Set true to get debug information from the USB hardware
\ ** layer. Some of the debug output *\b{must} be done using
\ ** polled serial drivers.
0 equ dbgCore?	\ -- flag
\ *G Set true to get debug information from the USB core layer.
\ ** Some of the debug output *\b{must} be done using polled
\ ** serial drivers.

0 equ AllBulkNakInt?	\ -- flag
\ *G Set true if Bulk NAK interrupts are controlled globally,
\ ** e.g. as for many NXP devices.
1 equ  EPBulkNakInt?	\ flag
\ *G Set true if Bulk NAK interrupts and handling are controlled
\ ** for each endpoint e.g. as for STM32F0x2 devices.

0 equ UsbPowered?	\ -- 0/1 ; bus powered?
\ *G Set to one if bus powered.
0 equ UsbDMA?		\ -- 0/1 ; uses DMA?
\ *G Set to one if any endpoints other than EP0 use DMA. At the
\ ** moment we assume that EP0 does not use DMA.
$0000:0000 equ UsbDMAEP	\ -- x ; DMA endpoints

#64 equ /maxpacket0	\ -- u
\ *G Maximum size of packets on endpoint 0.
#64 equ /maxpacket	\ -- u
\ *G Maximum size of packets on endpoints other than 0.

1 equ EP0inFrameCheck?	\ -- flag
\ *G STM32F0x2 IN endpoints send a packet when EPVALID status
\ ** is set. This means that a check in the frame interrupt is
\ ** required for data items longer than *\fo{/maxpacket0} bytes.

#32 equ #EPs	\ -- u
\ *G Number of endpoints we are are going to deal with. On a
\ ** 32 bit CPU, there are 32 bits per word. We need separate
\ ** bits for IN and OUT, so there's a practical maximum here
\ ** of 32, and USB supports 16 endpoints, each with IN and
\ ** OUT capability. Endpoint 0 is always IN and OUT capable.
4 equ #USBifs		\ -- u
\ *G Number of interfaces we can expand to eventually.

0 equ HID?	\ -- flag
\ *G True if a HID device, e.g. mouse, is to be supported.
0 equ MSC?	\ -- flag
\ *G True if a mass storage device is to be supported.
1 equ CDC?	\ -- flag
\ *G True if a communications device class (CDC) is to be supported.
\ ** The most common of these is a USB serial port.
0 equ AUDIO?	\ -- flag
\ *G True if an audio device is to be supported.
1 equ CLASS?	\ -- flag
\ *G Set this to 1 if any of the classes above are set to 1.

CDC? [if]
0 equ CdcCIf#		\ -- b
\ *G Interface number of the CDC control interface.
1 equ CdcDIf#		\ -- b
\ *G Interface number of the CDC data interface.
$81 equ CDCIinEP	\ -- ep
\ *G Optional Interrupt IN event notification endpoint (CIF).
$82 equ CDCBinEP	\ -- ep
\ *G CDC Bulk IN endpoint (DIF).
$03 equ CDCBoutEP	\ -- ep
\ *G CDC Bulk OUT endpoint (DIF).
0 equ dbgCDC?		\ -- flag
\ *G set non-zero to get debug information from the CDC driver.
1 equ CdcBulkinFrameCheck?	\ -- flag
\ *G STM32F0x2 IN endpoints must send a packet when EPVALID status
\ ** is set. This means that a check in the frame interrupt is
\ ** required.

CDCIinEP EP>mask equ CDCIinEPMask	\ -- mask
\ *G Core layer bit mask for the CDC Interrupt IN endpoint.
\ ** Used to avoid run-time calculation.
CDCBinEP EP>mask equ CDCBinEPMask	\ -- mask
\ *G Core layer bit mask for the CDC bulk IN endpoint.
\ ** Used to avoid run-time calculation.
CDCBoutEP EP>mask equ CDCBoutEPMask	\ -- mask
\ *G Core layer bit mask for the CDC bulk OUT endpoint.
\ ** Used to avoid run-time calculation.
[then]


\ ******************
\ *S USB Descriptors
\ ******************
\ *P This section contains the USB descriptors. There is a descriptor
\ ** for the device, for its configuration, and for various strings.

\ *P Strings are accessed using a "string index". Aprt from zero, which
\ ** has a predefined meaning to define the language, string indices
\ ** (1..255) are defined by the application. Here the indices are the
\ ** offset in bytes from the start of a string table. If you need more
\ ** than 255 bytes of string descriptors, change the index to be the
\ ** offset into a table of string descriptor addresses.

Create CfgDesc	\ -- addr
\ *G The configuration descriptor has a primary configuration, one
\ ** interface, and two endpoints. If there is more than one
\ ** configuration, additional ones must follow the first one.
\ -- configuration
\ USB Configuration Descriptor
  9 c,   \ bLength: Configuration Descriptor size
  $02 c, \ USB_CONFIGURATION_DESCRIPTOR_TYPE
  0 lew, \ total size, this + interfaces + x endpoints - filled in later
  2 c,   \ bNumInterfaces: 2 interface
  1 c,   \ bConfigurationValue
  0 c,   \ iConfiguration
  $C0 c, \ bmAttributes: self powered
  $64 c, \ MaxPower 0 mA

\ -- interface 1, CDC control interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  CdcCIf# c,		\ CDC control interface number
  0 c,			\ no alternate setting
  1 c,			\ number of endpoints
  2 c,			\ Class - Communication Device Class
  2 c,			\ Subclass - ACM Abstract Control Model
  1 c,			\ protocol - modem, needed by Macs
  0 c,			\ no interface string
\ Header Functional Descriptor
  5 c,   \ bLength: Endpoint Descriptor size
  $24 c, \ bDescriptorType: CS_INTERFACE
  $00 c, \ bDescriptorSubtype: Header Func Desc
  $10 c, \ bcdCDC: spec release number
  $01 c,
\ Call Management Functional Descriptor
  5 c,   \ bFunctionLength
  $24 c, \ bDescriptorType: CS_INTERFACE
  $01 c, \ bDescriptorSubtype: Call Management Func Desc
  $00 c, \ bmCapabilities: D0+D1
  $01 c, \ bDataInterface: 1
\ -- Abstract Control Management Functional Descriptor
  4 c,			\ length
  $024 c,		\ type - CS_INTERFACE
  2 c,			\ subtype - ACM
  2 c,			\ bmCapabilities - SET/GET_LINE_xxx
\ -- Union Functional Descriptor
  5 c,			\ length
  $024 c,		\ type - CS_INTERFACE
  6 c,			\ subtype - union
  CdcCIf# c,		\ bMasterInterface - control interface number
  CdcDIf# c,		\ bSlaveInterface - data interface number
\ -- Interrupt IN Descriptor - event notification (optional)
  7 c,			\ desc size
  5 c,			\ endpoint descriptor
  CDCIinEP c,		\ Endpoint I/O+number
  3 c,			\ EP type - interrupt
  ( /maxpacket ) 8 lew,	\ max packet size
  #50 c,		\ poll interval in ms

\ -- interface 2, CDC data interface
  9 c,			\ descriptor size
  4 c,			\ interface descriptor
  CdcDIf# c,		\ CDC data interface number
  0 c,			\ no alternate setting
  2 c,			\ number of endpoints
  #10 c,		\ Class - CDC Data Interface
  0 c,			\ Subclass - none
  0 c,			\ protocol - none
  0 c,			\ no interface string
\ -- BULK endpoint IN
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  CDCBinEP c,		\ dir and ep# - IN
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- BULK endpoint OUT
  7 c,			\ descriptor size
  5 c,			\ endpoint descriptor
  CDCBoutEP c,		\ dir and ep# - OUT 5
  2 c,			\ type - bulk
  /MaxPacket lew,	\ max packet size
  0 c,			\ no latency for full speed bulk endpoints
\ -- end, or insert more configuration descriptors
here CfgDesc -  CfgDesc 2+ lew!		\ set config. desc. length
  0 lew,		\ needed by software
calign

create StrDesc	\ -- addr
\ *G String descriptor and following strings.
l: Str0		\ -- string 0, language
  4 c,			\ descriptor size
  3 c,			\ string
  $0409 lew,		\ type - US English
l: Str1		\ -- string 1, manufacturer - must match .INF file
  usb", ST Microelectronics"
align l: str2	\ -- string 2, product
  usb", MPE CDC class demo"
\ l: Str1		\ -- string 1, manufacturer - must match .INF file
\   usb", MPE - www.mpeforth.com"
\ align l: str2	\ -- string 2, product
\   usb", MPE CDC class demo"
align l: str3	\ -- string3, serial number
  usb", 0123456789AB0001"	\ 16 digits
		\ -- end
  0 lew,		\ needed by software

Str1 Str0 - equ Str1idx
Str2 Str0 - equ Str2idx
Str3 Str0 - equ Str3idx

create DevDesc	\ -- addr
\ *G The device descriptor.
  #18 c,		\ descriptor size
  1 c,			\ device descriptor
  $0200 lew,		\ USB 2.0
  2 c,			\ CDC class
  0 c,			\ subclass is specified in the interface
  0 c,			\ protocol is specified in the interface
  /MaxPacket0 c,	\ endpoint 0 max size
  $0483 lew,		\ Vendor ID (from www.usb.org) - VID=0x0483 for ST
  $5740 lew,		\ Product ID -
\  $0D59 lew,		\ Vendor ID (from www.usb.org) - VID=0D59 for MPE developers
\  $FFF3 lew,		\ Product ID - PID=FFF0..FFFF for MPE developers
  $0001 lew,		\ Release ID - assigned by vendor
  3 c,			\ Manufacturer string index
  2 c,			\ Product string index
  1 c,			\ serial number string index - 0=none
  1 c,			\ number of configurations
\ -- end
  0 lew,		\ needed by software


\ ***************************
\ *S Serial number formatting
\ ***************************
\ *P If the word *\fo{unit#} is defined, it should return a
\ ** 32 bit unit serial number. This is used by the USB code
\ ** to return a serial number to the host. If the word
\ ** *\fo{unit#} has not been defined, a default fixed serial
\ ** number is returned.

((
$1FFF:F7AC equ _udevid	\ -- addr
\ *G Address holds 96 bit unique ID

: unit#		\ -- u
\ *G Returns a "fairly unique" hardware ID as single 32-bit number.
\ ** Specific to STM32F072
  _udevid dup @ over cell + @ xor swap 2 cells + @ xor
;
))

[defined] unit# [if]
#34 buffer: UTFbuf	\ -- addr
\ *G Holds a string descriptor with up to 16 little-endian Unicode
\ ** 16-bit characters.

: usbSerialNum	\ -- caddr len
\ *G Returns the default device USB serial number if *\fo{unit#}
\ ** is defined.
  Str1 UTFbuf #34 cmove		\ copy string to buffer
  #34 UTFbuf c!			\ serial number length
  unit#  UTFbuf #18 + #16 bounds do	\ -- sn
    dup #28 rshift $0F and		\ top 4 bits
    dup 9 >				\ convert to hex digit
    if  7 +  endif
    [char] 0 + i c!			\ char in first of two bytes
    4 lshift				\ ready for next 4 bits
  2 +loop
  drop  UTFbuf #34
;
[else]
: usbSerialNum	\ -- caddr len
\ *G Returns the default device USB serial number if *\i{unit#}
\ ** is not defined.
  Str1 dup c@
;
[then]


\ *****************************
\ *S String descriptor handling
\ *****************************

: USBStr@	\ index -- caddr len
\ *G Return the USB string descriptor corresponding to the given
\ ** string index. By convention the following order is used:
\ *D 0    language identifier - do not change
\ *D 1    serial number
\ *D 2    product
\ *D 3    manufacturer
\ *P If you configure your products with serial numbers extracted
\ ** from non-USB strings, e.g. MAC numbers, modify this word to
\ ** perform the string extraction as required.
  case
    0 of  Str0 dup c@  endof
    1 of  usbSerialNum  endof
    2 of  Str2 dup c@  endof
    3 of  Str1 dup c@  endof
      0 0 rot
  endcase
;


\ ======
\ *> ###
\ ======

decimal
