\ devSTM32F072disco.fth - USB set up for STM32F072B Discovery board

((
Copyright (c) 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! devstm32f072disco
\ *T USB set up for STM32F072B Discovery
\ ===========
\ *P The file *\i{devSTM32F072disco.fth} provides USB initialisation
\ ** and LED drivers for an STM32F072B Discovery board.
\ ** It will require minor changes for other STM32F0 devices and
\ ** boards, in particular in the USB Connect function and LEDs.

\ USB - CDC or MSC only
  include %UsbDir%/UsbDefs		\ USB definitions/equates
\  include %CommonDir%/CQueues		\ character queues
  include %UsbDir%/VCOMdev		\ UART device emulation
((
  include %UsbDir%\MscConfig		\ MSC configuration file
  include %UsbDir%\UsbHw2xxx		\ USB hardware layer for LPC2148
  include %UsbDir%\UsbBase		\ USB base operations
MSC? [if]
  include %UsbDir%\MscDrv		\ Mass Storage Class
[then]
CDC? [if]
  include %UsbDir%\VCOMdev		\ UART device emulation
  include %UsbDir%\CdcDrv		\ Communications Device Class
[then]
  include %UsbDir%\UsbEP0		\ USB control and initialisation
))

\ **********************
\ *S USB interrupt level
\ **********************

0 equ USBpri#	\ -- u
\ *G Slot/Priority (0..15, 0=high) used for the USB interrupt handler.


\ *******
\ *S LEDs
\ *******
\ *P The hardware uses two LEDs to indicate USB activity (LED1) and
\ ** disk activity (LED2).

: usbLED1on	\ --
\ *G Turn LED1 on.
  LDL setPin  ;
: usbLED1off	\ --
\ *G Turn LED1 off.
  LDL clrPin  ;
: usbLED2on	\ --
\ *G Turn LED2 on.
  LDR setPin  ;
: usbLED2off	\ --
\ *G Turn LED2 off.
  LDR clrPin  ;

: initLEDs	\ --
\ *G Initialise the two LEDs to off, output=low. These are on
\ ** P0.10,11. Executed at power up.
  LedActive off  usbLED1off  usbLED2off
  LDU clrPin  LDD clrPin
;
' initLEDs AtCold



\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary STM32F0xx USB setup is performed in
\ ** *\i{UsbHwSTM32F0xx.fth}. Because STM32F0xx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.

: udelay	\ --
\ *G Short delay to let hardware react.
  1000 0 do loop
;

: USBhwInit	\ --
\ *G Enable access to the USB peripheral.
  _RCC
  bit16 over rccCR2 + or!		\ enable HSI48
  begin					\ wait until ready
    dup rccCR2 + @ bit17 and
  until
  bit7 over rccCFGR3 + bic!		\ select HSI48 for USB
\ switch to the HSI48 output
  $0003 $0003 2 pick rccCFGR + setMask	\ set SW=3
  begin
    dup rccCFGR + @ $000C and $000C =	\ wait for SWS=3
  until
  bit23 over rccAPB1EN + or!		\ enable USB
  bit23 over rccAPB1RST + bic!		\ clear USB reset
  bit27 over rccAPB1EN + or!		\ enable CRS
  bit27 over rccAPB1RST + bic!		\ clear CRS reset
  drop

  _USBdev
  $0001 over usbCNTR + !		\ reset USB peripheral
  udelay
  $0000 over usbCNTR + !
  drop

  _CRS
  $0000:2040 over crsCR + !		\ default trim, autotrim
  $2022:BB7F over crsCFGR + !		\ src=SOF, 1 ms, 48 MHz
  $0000:000F over crsICR + !		\ clear interrupts
  bit5 over crsCR + or!			\ default trim + CEN
  drop
;

: .CRS		\ --
\ *G Display CRS state.
  _CRS #16 pdump
;

: usbHWconnect	\ --
\ *G Connect pull up on USB DP line.
  bit15 _USBdev usbBCDR + or!
;

: usbHWdisconnect	\ --
\ *G Disconnect pull up on USB DP line.
  bit15 _USBdev usbBCDR + bic!
;

_USBSRAM constant _USBSRAM	\ -- addr
\ *G Base address of 768 bytes of USB packet memory, which must
\ ** be addressed/read/written in 16 bit units.
_USBdev constant _USBdev	\ -- addr
\ *G Base address of USB peripheral.
_CRS constant _CRS		\ -- addr
\ *G Base address of USB Clock Recovery System, which tunes the
\ ** HSI48 oscillator using the SOF pulses.


\ =========
\ Main code
\ =========

\ USB driver

create usb:dev
  18 c,  \ bLength
  $01 c, \ USB_DEVICE_DESCRIPTOR_TYPE
  $00 c,
  $02 c, \ bcdUSB = 2.00
  $02 c, \ bDeviceClass: CDC
  $00 c, \ bDeviceSubClass
  $00 c, \ bDeviceProtocol
  $40 c, \ bMaxPacketSize0
  $83 c,
  $04 c, \ idVendor = 0x0483
  $40 c,
  $57 c, \ idProduct = 0x7540
  $00 c,
  $02 c, \ bcdDevice = 2.00
  1 c,     \ Index of string descriptor describing manufacturer
  2 c,     \ Index of string descriptor describing product
  3 c,     \ Index of string descriptor describing the device's serial number
  $01 c, \ bNumConfigurations
calign

create usb:conf  \ total length = 67 bytes
\ USB Configuration Descriptor
  9 c,   \ bLength: Configuration Descriptor size
  $02 c, \ USB_CONFIGURATION_DESCRIPTOR_TYPE
  67 c,  \ VIRTUAL_COM_PORT_SIZ_CONFIG_DESC
  0 c,
  2 c,   \ bNumInterfaces: 2 interface
  1 c,   \ bConfigurationValue
  0 c,   \ iConfiguration
  $C0 c, \ bmAttributes: self powered
  $32 c, \ MaxPower 0 mA
\ Interface Descriptor
  9 c,   \ bLength: Interface Descriptor size
  $04 c, \ USB_INTERFACE_DESCRIPTOR_TYPE
  $00 c, \ bInterfaceNumber: Number of Interface
  $00 c, \ bAlternateSetting: Alternate setting
  $01 c, \ bNumEndpoints: One endpoints used
  $02 c, \ bInterfaceClass: Communication Interface Class
  $02 c, \ bInterfaceSubClass: Abstract Control Model
  $01 c, \ bInterfaceProtocol: Common AT commands
  $00 c, \ iInterface:
\ Header Functional Descriptor
  5 c,   \ bLength: Endpoint Descriptor size
  $24 c, \ bDescriptorType: CS_INTERFACE
  $00 c, \ bDescriptorSubtype: Header Func Desc
  $10 c, \ bcdCDC: spec release number
  $01 c,
\ Call Management Functional Descriptor
  5 c,   \ bFunctionLength
  $24 c, \ bDescriptorType: CS_INTERFACE
  $01 c, \ bDescriptorSubtype: Call Management Func Desc
  $00 c, \ bmCapabilities: D0+D1
  $01 c, \ bDataInterface: 1
\ ACM Functional Descriptor
  4 c,   \ bFunctionLength
  $24 c, \ bDescriptorType: CS_INTERFACE
  $02 c, \ bDescriptorSubtype: Abstract Control Management desc
  $02 c, \ bmCapabilities
\ Union Functional Descriptor
  5 c,   \ bFunctionLength
  $24 c, \ bDescriptorType: CS_INTERFACE
  $06 c, \ bDescriptorSubtype: Union func desc
  $00 c, \ bMasterInterface: Communication class interface
  $01 c, \ bSlaveInterface0: Data Class Interface
\ Endpoint 2 Descriptor
  7 c,   \ bLength: Endpoint Descriptor size
  $05 c, \ USB_ENDPOINT_DESCRIPTOR_TYPE
  $82 c, \ bEndpointAddress: (IN2)
  $03 c, \ bmAttributes: Interrupt
  8 c,   \ VIRTUAL_COM_PORT_INT_SIZE
  0 c,
  $FF c, \ bInterval:
\ Data class interface descriptor
  9 c,   \ bLength: Endpoint Descriptor size
  $04 c, \ USB_INTERFACE_DESCRIPTOR_TYPE
  $01 c, \ bInterfaceNumber: Number of Interface
  $00 c, \ bAlternateSetting: Alternate setting
  $02 c, \ bNumEndpoints: Two endpoints used
  $0A c, \ bInterfaceClass: CDC
  $00 c, \ bInterfaceSubClass:
  $00 c, \ bInterfaceProtocol:
  $00 c, \ iInterface:
\ Endpoint 3 Descriptor
  7 c,   \ bLength: Endpoint Descriptor size
  $05 c, \ USB_ENDPOINT_DESCRIPTOR_TYPE
  $03 c, \ bEndpointAddress: (OUT3)
  $02 c, \ bmAttributes: Bulk
  64 c,  \ VIRTUAL_COM_PORT_DATA_SIZE
  0 c,
  $00 c, \ bInterval: ignore for Bulk transfer
\ Endpoint 1 Descriptor
  7 c,   \ bLength: Endpoint Descriptor size
  $05 c, \ USB_ENDPOINT_DESCRIPTOR_TYPE
  $81 c, \ bEndpointAddress: (IN1)
  $02 c, \ bmAttributes: Bulk
  64 c,  \ VIRTUAL_COM_PORT_DATA_SIZE
  0 c,
  $00 c, \ bInterval
calign

create usb:langid
  4 c, 3 c,  \ USB_STRING_DESCRIPTOR_TYPE,
  $0409 w, \ LangID = U.S. English

create usb:vendor
  40 c, 3 c,  \ USB_STRING_DESCRIPTOR_TYPE,
  char M w, char e w, char c w, char r w, char i w, char s w, char p w,
  bl     w, char ( w, char S w, char T w, char M w, char 3 w, char 2 w,
  char F w, char 1 w, char 0 w, char x w, char ) w,

create usb:product
  36 c, 3 c,  \ USB_STRING_DESCRIPTOR_TYPE,
  char F w, char o w, char r w, char t w, char h w, bl     w, char S w,
  char e w, char r w, char i w, char a w, char l w, bl     w, char P w,
  char o w, char r w, char t w,

create usb:line
  hex 00 c, C2 c, 01 c, 00 c, 01 c, 00 c, 08 c, 00 c, decimal

$1FFF:F7AC equ _udevid	\ -- addr
\ *G Address holds 96 bit unique ID
$1FFF:F7CC equ _flashkb	\ -- addr
\ *G Holds the flash size in kilobytes.
: hwid		\ -- u
\ *G Returns a "fairly unique" hardware ID as single 32-bit number.
  _udevid dup @ over cell + @ xor swap 2 cells + @ xor
;

: flash-kb ( -- u )  \ return size of flash memory in KB
  _flashkb w@ ;

: even		\ addr -- addr'
  1+ -2 and
;

: usb-pma	( pos -- addr ) _USBSRAM + ;
\ *G Convert an offset/position in the packet RAM to an absolute
\ ** address.
compiler
: usb-pma	( pos -- addr ) _USBSRAM + ;
target
: usb-pma!	( u pos -- )  usb-pma w! ;
\ *G Store 16 bits into offset/position in the packet RAM.
: usb-pma@ ( pos -- u ) usb-pma w@ ;
\ *G Fetch 16 bits from offset/position in the packet RAM.
((
: usb-pmac@	\ pos -- b
\ *G Fetch 8 bits from offset/position in the packet RAM.
  dup $FFFE and usb-pma@  swap 1 and
  if  8 rshift  else  $FF and  then
;
: usb-pmac! ( b pos -- )  \ careful, can't write high bytes separately
  dup 1 and if
    1- dup usb-pma@ rot 8 lshift or swap
  then usb-pma! ;
))

: ep-reg	\ ep n -- addr
\ *G Return the address in packet RAM of reg n (0..3) of ep (0..7)
  2* swap $0F and 8 * + usb-pma ;

\ *P Each endpoint has an 8 byte control area of four buffers for
\ ** transmit and receive control.
\ *[
0 equ EPR_ADDR_TX
1 equ EPR_COUNT_TX
2 equ EPR_ADDR_RX
3 equ EPR_COUNT_RX
\ *]

: ep-addr	( ep -- addr )  $0F and  cells _USBdev usbEP0R + + ;
\ *G Return the address in the peripheral block of the peripheral
\ ** register for endpoint ep.
compiler
: ep-addr	( ep -- addr )  $0F and  cells _USBdev usbEP0R + + ;
target

\ endpoint status values, both TX and RX
0 equ epDISABLED
1 equ epSTALL
2 equ epNAK
3 equ epVALID

\ endpoint type, both TX and RX
0 equ epBULK
1 equ epCONTROL
2 equ epISO
3 equ epINTERRUPT

: reg!wait	\ --
  20 0 do loop
;

: rxstat!	\ ep u --
\ *G Set stat_rx without toggling/setting any other fields.
  swap ep-addr >r
  3 and 12 lshift  r@ w@ tuck  xor	\ -- rval rval^mask ; R: -- *ep
\  R^rrseekT^ttnnnn
\  5432109876543210
  %0011:0000:0000:0000 and swap		\ -- rval^mask' rval ; R: -- *ep
  %0000:1111:0000:1111 and		\
  %1000:0000:1000:0000 or
  or r> w! reg!wait
;

: txstat!	\ ep u --
\ *G Set stat_tx without toggling/setting any other fields.
  swap ep-addr >r
  3 and 4 lshift  r@ w@ tuck  xor	\ -- orig mask ; R: -- addr
\  R^rrseekT^ttnnnn
\  5432109876543210
  %0000:0000:0011:0000 and swap		\ -- mask' orig
  %0000:1111:0000:1111 and		\ zero bits that toggle on 1
  %1000:0000:1000:0000 or		\ set bits that clear on zero
  or r> w! reg!wait
;

: rxclear	\ ep --
\ *G Clear the endpoint receive CTR_RX bit.
  ep-addr dup w@  $0F0F and  $0080 or  swap w! reg!wait  ;

: txclear	\ ep --
\ *G Clear the endpoint transmit CTR_TX bit.
  ep-addr dup w@  $0F0F and  $8000 or  swap w! reg!wait  ;

: ep-reset-rx#	\ ep --
\ *G Clear the RX count register.
  $8400 over EPR_COUNT_RX ep-reg w! reg!wait
  epVALID rxstat!
;

: ep-reset-tx#	\ ep --
\ *G Clear the TX count register.
  $0000 swap EPR_COUNT_TX ep-reg w! reg!wait  ;

2variable usb-pend
  0 0 usb-pend 2!
18 buffer: usb-serial

: set-sernum	\ -- addr
\ *G Fill serial number in as UTF-16 descriptor.
  base @ >r  hex
  hwid 0 <# 8 0 do # loop #> ( -- addr 8 )
  0 do  dup c@ i 1+ 2* usb-serial + w! 1+  loop
  drop
  usb-serial $0312 over w!		\ length and type
  r> base !  ;

: send-data	\ addr n --
cr ." > " over .dword space dup .word space ." - "
  usb-pend 2! ;

: USBReadEP	\ ep# addr -- #read ; bit7=dir, bits3:0=num
 \ *G Read endpoint data from endpoint *\i{ep#} to *\i{addr}.
 \ ** Return the number of bytes read.
 \ ** Note that data is transferred in units of two bytes.
  swap $0F and dup >r			\ -- addr ep# ; R: -- ep#
  dup EPR_ADDR_RX ep-reg w@		\ -- addr ep# rxpma ; R: -- ep#
  usb-pma -rot				\ -- src addr ep# ; R: -- ep#
  EPR_COUNT_RX ep-reg w@ $03FF and 	\ -- src addr len ; R: -- ep#
  dup >r even  bounds ?do		\ -- src ; R: -- ep# len
    dup w@ i w!  2+
  2 +loop
  drop  r> r>				\ -- len ep#
  dup ep-reset-rx# rxclear		\ -- len
;

: USBWriteEP	\ ep# addr len -- cnt
 \ *G Write data *\i{addr/len} to host from endpoint *\i{ep#}.
 \ ** Return the number of bytes written.
  dup >r				\ ep# addr len ; R: -- len
  rot $0F and dup >r			\ addr len ep# ; R: -- len ep#
  epr_addr_tx ep-reg w@ usb-pma -rot	\ txpma addr len ; R: -- len ep#
  even bounds ?do			\ -- txpma
    i w@  over w! 2+
  2 +loop				\ -- txpma'
  drop					\ -- ; R: -- len ep#
  r> r> swap				\ -- len ep#
  2dup EPR_COUNT_TX ep-reg w! reg!wait	\ set TX length in EPn data
  epVALID txstat!			\ -- len ; start write
;

UAlign
#64 buffer: SetupPacket	\ -- addr
#64 buffer: EP0Buf	\ -- addr
#64 buffer: EPnBuf	\ -- addr

((
interpreter
: byte	 1 field  ;
: hword  2 field  ;
target

struct /USBSetupPacket	\ -- len
\ *G USB Default Control Pipe Setup Packet structure.
\ *[
  byte  usp.bmRequestType	\ bit7=Dir,0=to dev, bit6:5=Type, bit4:0=Recipient
  byte  usp.bRequest
  hword usp.wValue
  hword usp.wIndex
  hword usp.wLength
end-struct
\ *]
))

: sendZLPin	\ ep# --
\ *G Send a zero length packet from the given endpoint.
  0 0 USBWriteEP drop
;

: USBStatusInStage	\ --
\ *G USB Request - Status In Stage.
  $80 sendZLPin
;

: USBStatusOutStage	\ --
\ *G USB Request - Status Out Stage.
  $00 EP0Buf USBReadEP drop
;

: .ep0/istr	\ --
\ *G Display the EP0 endpoint register and the ISTR
  ." EP0R " 0 ep-addr @ .word ."  ISTR " _usbdev usbISTR + w@ .word space
;

: .ep0-pma	\ --
\ *G Display the EP0 packet memory registers
  cr ." EP0-PMA "
  _usbDev usbBTABLE + w@ usb-pma	\ table address
  8 bounds do
    i w@ .word space
  2 +loop
;

: send-desc	\ --
\ *G Send a descriptor from EP0.
  case  SetupPacket usp.wValue w@
    $0100 of usb:dev     18 endof
    $0200 of usb:conf    67 endof
    $0300 of usb:langid  4  endof
    $0301 of usb:vendor  40 endof
    $0302 of usb:product 36 endof
    $0303 of set-sernum  18 endof
      0 0 rot
  endcase				\ -- caddr len
  $80 -rot USBWriteEP drop
;

create usb:init
  $0080 w,  \ ADDR0_TX   control - rx: 64b @ +040/080, tx: 64b @ +080/100
  $0000 w,  \ COUNT0_TX
  $0040 w,  \ ADDR0_RX
  $8400 w,  \ COUNT0_RX
  $00C0 w,  \ ADDR1_TX   bulk - tx: 64b @ +0C0/180
  $0000 w,  \ COUNT1_TX
  $0000 w,  \ ADDR1_RX
  $0000 w,  \ COUNT1_RX
  $0140 w,  \ ADDR2_TX   interrupt - tx: 8b @ +140/280
  $0000 w,  \ COUNT2_TX
  $0000 w,  \ ADDR2_RX
  $0000 w,  \ COUNT2_RX
  $0000 w,  \ ADDR3_TX   bulk - rx: 64b @ +100/200
  $0000 w,  \ COUNT3_TX
  $0100 w,  \ ADDR3_RX
  $8400 w,  \ COUNT3_RX

: set-EPnR	\ x ep --
\ *G Set the USB endpoint register
  over $070F and  over ep-addr w! reg!wait	\ set bits that are directly written
  over #12 rshift over swap rxstat!	\ set RX status bits
  swap #4 rshift txstat!		\ set TX status bits
;

: usb-reset ( -- )
  512 0 do  0 i usb-pma!  2 +loop	\ wipe region
  0 _USBdev usbBTABLE + w!		\ EP table starts at packet buffer 0
  usb:init  32 0 do			\ initialise 4 endpoint buffer sets
    dup w@  i _USBSRAM + w!  2+
  2 +loop  drop
  $3210 0 set-EPnR		\ RX valid, CONTROL, TX stall, EP0
  $0021 1 set-EPnR		\ RX dis, BULK, TX NAK, EP1
  $0622 2 set-EPnR		\ RX dis, INTERRUPT, TX NAK, EP2
  $3003 3 set-EPnR		\ RX valid, BULK, TX dis, EP3
  $80 _USBdev usbDADDR + w! 	\ bit 7 = USB enable, address=0
;

create zero 0 ,

variable DevAddr	\ -- addr
\ *G Holds device address to be set.

: USBSetAddress	\ addr --
\ *G Set the USB assigned address and enable the device
  $7F and  $80 or  DevAddr !
\  $7F and  $80 or  _USBdev usbDADDR + w!
;

: ep-setup	\ ep --
\ *G Handle a setup packet, sent from host to config this device.
cr ." ep-setup "
  drop					\ always on EP0
  $00 SetupPacket UsbReadEP drop	\ Read endpoint 0
SetupPacket 8 bounds do  i c@ .byte space  loop
  case  SetupPacket usp.bRequest c@
    $00 of  $80 zero 2 USBWriteEP drop endof
    $05 of
          SetupPacket usp.wValue c@ >r
          USBStatusInStage
          r> USBSetAddress
        endof
    $06 of
cr ." SDI " .ep0/istr
          send-desc
cr ." SDO " .ep0/istr
    endof
      USBStatusInStage
  endcase
;

variable tx.pend
  tx.pend off
variable usb.ticks
  usb.ticks off

: ep-out	\ ep --
\ *G Outgoing packets, sent from host to this device
cr ." ep-out "
  dup if				\ ignore endpoint 0
    EPnbuf UsbReadEP dup .		\ -- len ; read endpoint n
    UsbInpQ0 cqroom min			\ -- len' ; clip to capacity
    EPnbuf swap bounds ?do
      i c@ UsbInpQ0 (>cqueue)
    loop
  else					\ EP0
    EP0buf UsbReadEP .			\ Read endpoint 0
  then
;

: send-next	\ --
\ *G Send the next part of a block of data being sent to the
\ ** host from device EP0.
cr ." SNI " .ep0/istr
  usb-pend 2@				\ -- caddr len
  dup if
    #64 min 				\ -- caddr len'
    $80 -rot USBWriteEP			\ -- len''
    usb-pend 2@ rot dup negate d+ usb-pend 2!
  else
    2drop
cr .ep0/istr
    $80 dup ep-reset-tx#  txclear
cr .ep0/istr
  endif
cr ." SNX " .ep0/istr
;

: ep-in		\ ep --
\ *G Incoming polls, sent from this device to host
cr ." ep-in "
  $0F and  dup if			\ -- ep ; EP1..7
    0 usb.ticks !  0 tx.pend !
    UsbOutQ0 cqchars ?dup if
      EPnBuf over #64 min		\ -- ep len addr len
      bounds ?do			\ -- ep len ; queue>buffer
        UsbOutQ0 (cqueue>) i c!
      loop
      EPnBuf swap USBWriteEP drop
    else				\ -- ep ; EP0
      dup ep-reset-tx#  txclear
    then
  else					\ -- ep ; EP0
    send-next
  then
  drop
;

: usb-ctr	\ istr --
cr ." usb-ctr "
cr ." S " .ep0/istr
  dup $0F and  swap $10 and if		\ -- ep ; OUT or SETUP packet
    dup ep-addr w@ $800 and
    if  ep-setup  else  ep-out then
  else
    ep-in
  then
cr ." F " .ep0/istr
cr \ .ep0-pma
;

: usb-poll
\ *G Main USB driver polling.
\    tx.pend @ if			\ if TX pending
\      1 usb.ticks +!
\      usb.ticks @ 10000 u>		\ waited 10 secs?
\      if  initVCOM0  then		\ clear buffers
\    then
\ main poll
  _USBdev usbISTR + w@
$9E1F and dup $8000 and
if  cr dup .word  then
  dup $8000 and if			\ good endpoint transfer
    usb-ctr  exit
  then
  dup $1000 and if			\ bit 12; USB wake up
    %1000 _USBdev usbCNTR + wbic!
    $EFFF _USBdev usbISTR + w!		\ clear interrupt
    drop  exit
  then
  dup $0800 and if			\ bit 11; USB suspend
    %1000 _USBdev usbCNTR + wor!	\ suspend and low power modes
    $F7FF _USBdev usbISTR + w!		\ clear interrupt
    drop  exit
  then
  dup $0400 and if			\ bit 10; USB reset
    usb-reset
    $FBFF _USBdev usbISTR + w!		\ clear interrupt
cr cr ." RESET" cr
    drop exit
  then
  dup $0200 and if			\ bit 9; SOF
    tx.pend @ if			\ if TX pending
      1 usb.ticks +!
      usb.ticks @ 10000 u>		\ waited 10 secs?
      if  initVCOM0  then		\ clear buffers
    then
    $FDFF _USBdev usbISTR + w!		\ clear interrupt
    DevAddr @ if
      DevAddr @ _USBdev usbDADDR + w!
      0 DevAddr !
      cr ." SOF " .ep0/istr
    then
    drop exit
  then
  drop
;

((
: USBinterrupt	\ --
\ *G The USB interrupt handler. It calls *\fo{usb-poll} above to
\ ** permit better factoring.
  usbLED1on
  usb-poll
  usbLED1off
;
' USBinterrupt USB_EXTI_18vec# EXC: USB_ISR	\ -- addr
\ *G The entry point for the USB interrupt.
))
: usbIntInit	\ --
\ *G Initialise USB hardware interrupt handler.
  $9E00 _USBdev usbCNTR + w! 		\ enable USB ints
\  USB_EXTI_18vec# EnInt			\ enable USB handler
;


: usb-io	\ --
\ *G Start up USB
  usbHWinit usb-reset
  initVCOM0
  usbIntInit
  usbHWconnect
  2000000 0 do  usb-poll  key? ?leave  loop
;

: +usb	usb-io  ;
: -usb  usbHWdisconnect  ;

: WDUMP		\ addr len -- ; dump 16 bit half words
\ *G Display (dump) len bytes of memory starting at addr
\ ** as 16 bit half-words.
[defined] FlushKeys [if] FlushKeys [then]	\ SFP002
  bounds ?do				\ get limits
    cr  i .lword  space   i 8 + i       \ print address
    do  space i w@ .word  2 +loop	\ first eight bytes
    space  i $010 + i 8 +               \ gap
    do  space i w@ .word  2 +loop	\ second eight bytes
\    2 spaces  i $010 + i                \ gap
\    do  i w@ dup $FF and .ascii  8 rshift .ascii  2 +loop  \ ascii o/p
    halt? ?leave			\ quit or halt
  $010 +loop                            \ until done
  cr  					\ restore base
;

: .usb		\ offset len --
  swap _USBSRAM + swap wdump
;

: .usb1		0 $100 .usb  ;
: .dev		_usbdev $100 pdump ;


\ ======
\ *> ###
\ ======

decimal

