\ VCOMdev - USB CDC class generic I/O device

((
Copyright (c) 2009, 2010
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone numbers are:
       011 44 23 8063 1441
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20100906 SFP003 Allowed queue sizes to be defined elsewhere.
20090803 SFP002 Improved initialisation.
20090723 SFP001 Initial release.
))

decimal

\ =========
\ *> cdcdrv
\ *S Example CDC UART
\ =========
\ *P The code in *\i{VCOMdev.fth} is an example of a simulated
\ ** UART. It appears in the target Forth as another generic I/O
\ ** device. This code must be compiled *\b{before} the CDC
\ ** driver in *\i{CdcDrv.fth}.


\ *******************
\ *N CDC Virtual UART
\ *******************
\ *P The USB UART simulation relies on character queues and
\ ** therefore the words in this section must not be called
\ ** inside interrupt routines.

[undefined] /UsbInpQ0 [if]
#256 equ /UsbInpQ0	\ must be power of two
\ *G Size of the input queue if not previously defined.
[then]
[undefined] /UsbOutQ0 [if]
#128 equ /UsbOutQ0	\ must be power of two
\ *G Size of the output queue if not previously defined.
[then]

uAlign iAlign
/UsbInpQ0 cqueue: UsbInpQ0	\ -- addr
\ *G Receiver circular byte queue.
/UsbOutQ0 cqueue: UsbOutQ0	\ -- addr
\ *G Transmitter circular byte queue.

: usbEmit0	\ char --
\ *G Send a character to the USB host.
  UsbOutQ0 >cqueue
;

: usbType0	\ caddr len --
\ *G Send a string to the USB host.
  bounds ?do  i c@ UsbOutQ0 >cqueue  loop
;

: usbCr0	\ --
\ *G Send a CR/LF pair to the USB host.
  $0D usbEmit0  $0A usbEmit0
;

: usbKey?0	\ -- flag
\ *G Return true if a character is available from the USB host.
  UsbInpQ0 cqnotempty?
;

: usbKey0	\ -- char
\ *G Wait for a character from the USB host.
  UsbInpQ0 cqueue>  10 ms
;

create UsbConsole0	\ -- addr ; OUT managed by upper driver
\ *G Generic I/O device for first USB UART.
  ' usbkey0 ,		\ -- char ; receive char
  ' usbkey?0 ,		\ -- flag ; check receive char
  ' usbemit0 ,		\ -- char ; display char
  ' usbtype0 ,		\ caddr len -- ; display string
  ' usbcr0 ,		\ -- ; display new line

console-port 10 = [if]
  UsbConsole0 constant console
[then]


\ *****************
\ *N USB insulation
\ *****************
\ *P The words in this layer are used in *\i{CdcDrv.fth} within
\ ** the USB interrupt handlers. They isolate the UART code from
\ ** its implementation. Some of these words can be implemented
\ ** as compiler macros.

compiler
: cdcRoomIn?0	\ len -- true
\ *G Return true if there is room for *\i{len} more characters
\ ** in the input queue.
  UsbInpQ0 cqroom?
;

: #cdcTxOut0	\ -- len
\ *G Return the number of characters that can be sent.
  UsbOutQ0 cqchars
;
target

: cdcRcv0	\ caddr len --
\ *G Receive *\i{len} characters from the buffer at *\i{caddr}.
  UsbInpQ0 -rot				\ -- q caddr lenr
  bounds
  ?do  i c@ over (>cqueue)  loop	\ save characters
  drop
;

: cdcTxmit0	\ caddr len --
  UsbOutQ0 -rot  bounds ?do	\ -- queue ; fill buffer
    dup (cqueue>) i c!
  loop
  drop
;

: initVCOM0	\ --
  UsbInpQ0 init-cqueue  UsbOutQ0 init-cqueue	\ initialise character queues
;
' initVCOM0 AtCold			\ prevents problems with early use ; SFP002

((
struct /cqueue	\ -- size ; character queue structure in idata, buffer in udata
\ *G Circular queue data structure.
\ *[
  int >qhead		\ Offset of head, where characters are taken from
  int >qtail		\ Offset of tail, where characters are put
  int >qchars		\ Number of characters in the queue
  int >qmask		\ Mask to apply to pointers
  ptr >qbuffer		\ Base address of character buffer
end-struct
\ *]
))

: .usbQs	\ --
  cr ." UsbInpQ0 "  UsbInpQ0 >qchars @ .  UsbInpQ0 >qbuffer @ $40 dump
  cr ." UsbOutQ0 "  UsbOutQ0 >qchars @ .  UsbOutQ0 >qbuffer @ $40 dump
;


\ ======
\ *> ###
\ ======

decimal
