((
Copyright (c) 2008, 2011, 2014
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
20140402 BJC002 Conversion to CCJ-SP05 PCB using LPC1788 chip.
		Should be easily portable to other boards with similar chips.
		NOTE: Main differences between this file and LPC17xx is pin
		configuration & clock register bits.  Otherwise driver same
		as LPC17xx.
20110817 SFP001 Conversion to device connection of Olimex
                LPC-1766STK board.
))

only forth definitions
decimal

\ ===========
\ *! devCCJ1788.fth
\ *T USB device set up for CCJ-SP05 board
\ ===========
\ *P The file *\i{USB_BSP_CCJ-SP05.fth} provides USB device mode
\ ** initialisation and LED drivers for an CCJ-SP05 LPC-1788
\ ** board. It will require minor changes for other LPC devices
\ ** and boards, in particular in the USB Connect function and
\ ** LEDs.
\ ** See also *\i{sfrLPC1788} for details of compatible family members.


\ *******
\ *S LEDs
\ *******
\ *P This board uses the hardware driven 'up' LED, so these words are stubs.

((
: initLEDs	\ --
\ *G Initialise the two LEDs to off, output=high.
  LED1val LED1mask  _PINSEL3 setMask	\ configure LED1
  LED2val LED2mask  _PINSEL0 setMask	\ configure LED2
  LED1bit _FIO1
  2dup FIODIR + or!			\ set DIR bits
  FIOSET + !				\ set high
  LED2bit _FIO0
  2dup FIODIR + or!			\ set DIR bits
  FIOSET + !				\ set high
;
' initLEDs AtCold
))

: usbLED1on	\ --
\ *G Turn LED1 on.
;
: usbLED1off	\ --
\ *G Turn LED1 off.
;
: usbLED2on	\ --
\ *G Turn LED2 on.
;
: usbLED2off	\ --
\ *G Turn LED2 off.
;


\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary LPC1788 USB setup is performed in
\ ** *\i{UsbHwLPC2xxx.fth}. Because LPC17xx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.
\ ** these port assignments are for the USB1 port.
\ *E USB_UP_LED   P1.18	  (IO type D)
\ ** USB_CONNECT  P1.19		''
\ ** VBUS         P1.30		''	type A	** requires board mod on prototypes **
\ ** D+           P0.29		(IO type *U*)
\ ** D-           P0.30		''

~USBCLK_DEV_CLK ~USBCLK_PORTSEL_CLK or ~USBCLK_AHB_CLK or
  equ		~USB_ALL_CLOCKS

~USBCLK_DEV_CLK ~USBCLK_AHB_CLK or
  equ   ~USB_RUN_CLOCKS

: USBhwInit	\ --
\ *G Enable access to the USB Device peripheral.
\ ** The main clock PLL is set in the start up code.
  ~pcusb _PCONP  or!			\ usb power on

  _SYSCON
  USBCLKval BIT8 or			\ merge bit to enable USB clock from main PLL (0)
	over USBCLKSEL + !		\ USB clock divider from PLL0
  [i
  0 over PLL1CON + !			\ disable PLL1
  $AA over PLL1FEED + !			\ feed sequence
  $55 over PLL1FEED + !
  i]
  drop

  _USB
  begin					\ turn on clocks
    ~USB_ALL_CLOCKS over USBClkCtrl + !	\ enable AHB, Portsel, USB device clocks
    pause
    dup USBClkSt + @			\ check status
    ~USB_ALL_CLOCKS and ~USB_ALL_CLOCKS =
  until
  0 over USBPortSel + !			\ use USB1 port for device
  ~USB_RUN_CLOCKS over USBClkCtrl + !	\ just AHB & USB device clocks (portsel now hidden)
  drop

  \ configure IO pins
  _IOCON
  IO_PERIPH_FUNC1 over IOCON_P0_29 + !		\ USB D+
  IO_PERIPH_FUNC1 over IOCON_P0_30 + !		\ USB D-
  IO_PERIPH_FUNC1 over IOCON_P1_18 + !		\ 'up' LED indicator
  IO_PERIPH_FUNC2 +DIGITAL over IOCON_P1_30 + !	\ USB Vbus sense

\ NOTE: If the LCD display is connected and USB port 1 is used,
\ the hardware control of the pull-up resistor (soft connect) is
\ not available due to pin conflict.  Connect is controlled using GPIO
  IO_GPIO_FUNC over IOCON_P1_19 + !	\ soft connect (bus pull up) output
  BIT19 _FIO1 FIODIR + or!		\ make output

  IO_GPIO_FUNC over IOCON_P0_14 + !	\ fix for for prototype PCBs
  drop
;

\ not needed if software connect works right or is available
\ comment these defs if soft connect is used.
: USBhwconnect	\ --
\ *G Connect to the USB bus.
  BIT19 _FIO1 FIOCLR + !
;
: USBhwdisconnect	\ --
\ *G Disconnect from the USB bus.
  BIT19 _FIO1 FIOSET + !
;


\ ======
\ *> ###
\ ======

decimal

