\ devLCI2148.fth - USB set up for LanBox-LCI P2148 board

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! devlci2148
\ *T USB set up for LanBox-LCI
\ ===========
\ *P The file *\i{devLCI2148.fth} provides USB initialisation
\ ** and LED drivers for an LanBox-LCI board.
\ ** It will require minor changes for other LPC devices and
\ ** boards, in particular in the USB Connect function and LEDs.


\ **********************
\ *S USB interrupt level
\ **********************

0 equ USBpri#	\ -- u
\ *G Slot/Priority (0..15, 0=high) used for the USB interrupt handler.


\ *******
\ *S USB LED and other GPIO
\ *******
\ *P The hardware uses a LED to indicate USB activity (LED1), a
\ ** DMXE (TX), 18 digouts, and 6 analog inputs
\ ** LED1 is turned on during the interrupt service routine.


bit31 equ LED1mask		\ P0.31
bit22 equ DMXEmask		\ P0.22
bit7  equ Out18mask		\ P0.7
bit21 equ Out19mask		\ P0.21
bit13 equ dtr1mask		\ P0.13
bit14 bit15 or bit16 or bit20 or equ buttonsMask

LED1mask equ P0HighMask
DMXEmask Out18mask or Out19mask or dtr1mask or equ P0LowMask

P0HighMask P0LowMask or equ P0OutMask

$0C00C000 equ GPIO_PINSEL0	\ P0.7, .13
$C0003C00 equ GPIO_PINSEL1	\ P0.21, .22, .31


: initIo	\ --
\ *G Initialise  LED to off, output=high. Other Io to LOW
\ ** Executed at power up.
  $C _PINSEL2 bic!			\ disable trace/debug ports
  GPIO_PINSEL0 _PINSEL0 bic!		\ configure pins for GPIO
  GPIO_PINSEL1 _PINSEL1 bic!		\ configure pins for GPIO

  _FIO					\ -- fiobase
  P0HighMask over FIO0SET + !		\ set appropriate P0 pins
  P0LowMask over FIO0CLR + !		\ clear appropriate P0 pins
  P0OutMask over FIO0DIR + or!		\ set to output
  $FFFF0000 over FIO1CLR + !		\ clear P1.16 - .31
  $FFFF0000 over FIO1DIR + or!		\ set to output
((
  i2c? 0= if				\ no i2c? then also no buttons
    buttonsMask over FIO0CLR + !	\ clear pins
    buttonsMask over FIO0DIR + or!	\ set to output
  then
))
  drop					\ --
\  init-ad
;
' initIo AtCold

: usbLED1on	( -- )	LED1mask _FIO0 FIOCLR + !  ;
: usbLED1off	( -- )	LED1mask _FIO0 FIOSET + !  ;

((
: usbLED1on ( -- )  LED1mask _FIO0CLR !  ;
: usbLED1off ( -- )  LED1mask _FIO0SET !  ;
: usbLED1? ( -- on? )  _FIO0PIN @ LED1mask and 0=  ;

: DMXEon ( -- )  DMXEmask _FIO0SET !  ;
: DMXEoff ( -- )  DMXEmask _FIO0CLR !  ;
: DMXE? ( -- on? )  _FIO0PIN @ DMXEmask and 0<>  ;

: DTR1on ( -- )  DTR1mask _FIO0SET !  ;
: DTR1off ( -- )  DTR1mask _FIO0CLR !  ;   	\ output high
: DTR1? ( -- on? )  _FIO0PIN @ DTR1mask and 0<>  ;

: Out18on ( -- )  Out18mask _FIO0SET !  ;
: Out18off ( -- )  Out18mask _FIO0CLR !  ;
: Out18? ( -- on? )  _FIO0PIN @ Out18mask and 0<>  ;

: Out19on ( -- )  Out19mask _FIO0SET !  ;
: Out19off ( -- )  Out19mask _FIO0CLR !  ;
: Out19? ( -- on? )  _FIO0PIN @ Out19mask and 0<>  ;


DTR1mask Out18mask or Out19mask or equ fio0-DOUTmask

: DOUT ( u -- )
  0					\ -- u fio0pins )
  over $0:0001 and if DTR1mask or then
  over $2:0000 and if Out18mask or then
  over $4:0000 and if Out19mask or then
  swap $1:FFFE and 1 rshift		\ -- fio0pins fio1pins )
  fio0-DOUTmask invert			\ -- fio0pins fio1pins fio0mask )

  _fio [I				\ -- fio0pins fio1pins fio0mask _fio )
    tuck FIO0MASK + !			\ -- fio0pins fio1pins _fio )
    tuck FIO1PIN 2+ + w!		\ -- fio0pins _fio )
    tuck FIO0PIN + !			\ -- _fio )
    0 over FIO0MASK + !			\ -- _fio )
  I] drop				\ -- )
;


: btn1? ( -- flag ) _FIO0PIN @ bit15 and 0= ;
: btn2? ( -- flag ) _FIO0PIN @ bit16 and 0= ;
: btn3? ( -- flag ) _FIO0PIN @ bit20 and 0= ;
: btn4? ( -- flag ) _FIO0PIN @ bit14 and 0= ;
: sw0? ( -- flag ) _FIO0PIN @ bit12 and 0= ;
: sw1? ( -- flag ) _FIO0PIN @ bit6 and 0= ;

compiler
: (anin) cells _ADC0 $10 + + w@ $3F bic $20 or 16 lshift ;
target

: anin1 ( -- u ) 1 (anin) ;
: anin2 ( -- u ) 2 (anin) ;
: anin3 ( -- u ) 3 (anin) ;
: anin4 ( -- u ) 4 (anin) ;
: anin5 ( -- u ) 6 (anin) ;
: anin6 ( -- u ) 7 (anin) ;


: init-ad	\ --
\ *G Initialise A/D convertor
  $00001000 $E01FC0C4 or!			\ enable bit12 clock & pwr _PCON
  $00200000 $E0034000 or!			\ enable ADC in the AD0CR register (bit PDN)

  $f00 $DE or $ffff $E0034000 setMask 		\ set AD clock speed (div16  en ch1-4,6-7)
  $00010000 $00010000 $E0034000 setmask		\ set AD auto bit16

  $00000F00 $00000F00 $E002c000 setMask		\ PINSEL0 bits 8:11 (ad6/7)val 0f
  $15040000 $3F0C0000 $E002c004 setMask		\ PINSEL1 bits 18:19, 24:29 (ad4, ad1-3)
;
))


\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary LPC2xxx USB setup is performed in
\ ** *\i{UsbHwLPC2xxx.fth}. Because LPC2xxx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.

#48000000 calcPLL 5 lshift or equ PLL1CfgVal	\ -- x
\ *G Value of the PLL1CFG register needed to generate a 48MHz
\ ** USB clock.

: USBhwInit	\ --
\ *G Enable access to the USB peripheral.
  bit23 _FIO0 FIOCLR + !		\ set P0.23 low (when output)
  bit23 _FIO0 FIODIR + or!		\ set P0.23 to output (when GPIO)

  $00000000 $C000C000 _PINSEL1 setmask	\ switch P0.23 (vbus) and P0.31 (usbled) to gpio

  _SYSCON				\ power up and configure 48Mhz PLL
  bit31 over PCONP + or!		\ power up
  PLL1CfgVal over PLL1CFG + !		\ FOsc = 12MHz, P = 2, M = 4
  [i
  bit0 over PLL1CON + !			\ set PLLE bit
  $AA over PLL1FEED + !			\ feed sequence
  $55 over PLL1FEED + !
  i]
  begin					\ wait for lock
    dup PLL1STAT + @ bit10 and
  until
  [i
  bit0 bit1 or over PLL1CON + !		\ set PLLC bit as well
  $AA over PLL1FEED + !			\ feed sequence
  $55 over PLL1FEED + !
  i]
  drop
;

: USBhwconnect	\ --
\ *G Connect to the USB bus.
  $4000 _PINSEL1 or!			\ switch P0.23 to USB VBUS
;

: USBhwdisconnect	\ --
\ *G Disconnect from the USB bus.
  bit23 _FIO0 FIOCLR + !		\ set P0.23 low (when output)
  bit23 _FIO0 FIODIR + or!		\ set P0.23 to output (when GPIO)
  $C000 _PINSEL1 bic!			\ switch P0.23 to GPIO/FIO
;


\ ======
\ *> ###
\ ======

decimal

