\ devP2148.fth - USB set up for Olimex P2148 board

((
Copyright (c) 2008
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! devp2148
\ *T USB set up for Olimex P2148
\ ===========
\ *P The file *\i{devP2148.fth} provides USB initialisation
\ ** and LED drivers for an Olimex P2148 board.
\ ** It will require minor changes for other LPC devices and
\ ** boards, in particular in the USB Connect function and LEDs.


\ **********************
\ *S USB interrupt level
\ **********************

0 equ USBpri#	\ -- u
\ *G Slot/Priority (0..15, 0=high) used for the USB interrupt handler.


\ *******
\ *S LEDs
\ *******
\ *P The hardware uses two LEDs to indicate USB activity (LED1) and
\ ** disk activity (LED2).

\ *P The Olimex board has user LEDs, LED1 (P0.10) and LED2 (P0.11).
\ ** LED1 is turned on during the interrupt service routine. LED2
\ ** is used during disk accesses.

bit10 equ LED1mask			\ P0.10
bit11 equ LED2mask			\ P0.11
LED1mask LED2mask or equ LEDiomask	\ both P0.10 and P0.11
$00F00000 equ LEDpinmask		\ bits in PINSEL0.

: initLEDs	\ --
\ *G Initialise the two LEDs to off, output=high. These are on
\ ** P0.10,11. Executed at power up.
  bit0 _SYSCON SCS + or!		\ enable fast I/O for GPIO0
  LEDpinmask _PINSEL0 bic!		\ clear bits 20..23 for GPIO use
  LEDIOmask _FIO
  2dup FIO0DIR + or!			\ set DIR bits
  FIOSET + !				\ set high
;
' initLEDs AtCold

: usbLED1on	\ --
\ *G Turn LED1 on.
  LED1mask _FIO FIO0CLR + !  ;
: usbLED1off	\ --
\ *G Turn LED1 off.
  LED1mask _FIO FIO0SET + !  ;
: usbLED2on	\ --
\ *G Turn LED2 on.
  LED2mask _FIO FIO0CLR + !  ;
: usbLED2off	\ --
\ *G Turn LED2 off.
  LED2mask _FIO FIO0SET + !  ;


\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary LPC2xxx USB setup is performed in
\ ** *\i{UsbHwLPC2xxx.fth}. Because LPC2xxx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.

#48000000 calcPLL 5 lshift or equ PLL1CfgVal	\ -- x
\ *G Value of the PLL1CFG register needed to generate a 48MHz
\ ** USB clock.

: USBhwInit	\ --
\ *G Enable access to the USB peripheral.
  _PINSEL1 dup @			\ pin select operations
\    $C000C000 invert and  $40004000 or	\ Select USB Link, UP_LED, VBUS - Keil
    $C000C000 invert and  $80004000 or	\ Select USB Link, CONNECT, VBUS - Olimex P2148
  swap !

  _SYSCON				\ power up and configure 48Mhz PLL
  bit31 over PCONP + or!		\ power up
  PLL1CfgVal over PLL1CFG + !		\ FOsc = 12MHz, P = 2, M = 4
  [i
  bit0 over PLL1CON + !			\ set PLLE bit
  $AA over PLL1FEED + !			\ feed sequence
  $55 over PLL1FEED + !
  i]
  begin					\ wait for lock
    dup PLL1STAT + @ bit10 and
  until
  [i
  bit0 bit1 or over PLL1CON + !		\ set PLLC bit as well
  $AA over PLL1FEED + !			\ feed sequence
  $55 over PLL1FEED + !
  i]
  drop
;


\ ======
\ *> ###
\ ======

decimal

