\ devSTM32F072disco.fth - USB set up for STM32F072B Discovery board

((
Copyright (c) 2017
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)


To do
=====

Change history
==============
))

only forth definitions
decimal

\ ===========
\ *! devstm32f072disco
\ *T USB set up for STM32F072B Discovery
\ ===========
\ *P The file *\i{devSTM32F072disco.fth} provides USB initialisation
\ ** and LED drivers for an STM32F072B Discovery board.
\ ** It will require minor changes for other STM32F0 devices and
\ ** boards, in particular in the USB Connect function and LEDs.


\ *******
\ *S LEDs
\ *******
\ *P The hardware uses two LEDs to indicate USB activity (LED1) and
\ ** disk activity (LED2).

: usbLED1on	\ --
\ *G Turn LED1 on.
  LDL setPin  ;
: usbLED1off	\ --
\ *G Turn LED1 off.
  LDL clrPin  ;
: usbLED2on	\ --
\ *G Turn LED2 on.
  LDR setPin  ;
: usbLED2off	\ --
\ *G Turn LED2 off.
  LDR clrPin  ;

: initLEDs	\ --
\ *G Initialise the two LEDs to off, output=low. These are on
\ ** P0.10,11. Executed at power up.
  LedActive off  usbLED1off  usbLED2off
  LDU clrPin  LDD clrPin
;
' initLEDs AtCold



\ **********************
\ *S USB hardware set up
\ **********************
\ *P The primary STM32F0xx USB setup is performed in
\ ** *\i{UsbHwSTM32F0xx.fth}. Because STM32F0xx variants and hardware
\ ** differ, the actual hardware initialisation is performed here.

: udelay	\ --
\ *G Short delay to let hardware react.
  1000 0 do loop
;

: USBhwInit	\ --
\ *G Enable access to the USB peripheral.
  _RCC
  bit16 over rccCR2 + or!		\ enable HSI48
  begin					\ wait until ready
    dup rccCR2 + @ bit17 and
  until
  bit7 over rccCFGR3 + bic!		\ select HSI48 for USB
\ switch to the HSI48 output
  $0003 $0003 2 pick rccCFGR + setMask	\ set SW=3
  begin
    dup rccCFGR + @ $000C and $000C =	\ wait for SWS=3
  until
  bit23 over rccAPB1EN + or!		\ enable USB
  bit23 over rccAPB1RST + bic!		\ clear USB reset
  bit27 over rccAPB1EN + or!		\ enable CRS
  bit27 over rccAPB1RST + bic!		\ clear CRS reset
  drop

  _USBdev
  $0001 over usbCNTR + !		\ reset USB peripheral
  udelay
  $0000 over usbCNTR + !
  drop

  _CRS
  $0000:2040 over crsCR + !		\ default trim, autotrim
  $2022:BB7F over crsCFGR + !		\ src=SOF, 1 ms, 48 MHz
  $0000:000F over crsICR + !		\ clear interrupts
  bit5 over crsCR + or!			\ default trim + CEN
  drop
;

: .CRS		\ --
\ *G Display CRS state.
  _CRS #16 pdump
;

: WDUMP		\ addr len -- ; dump 16 bit half words
\ *G Display (dump) len bytes of memory starting at addr
\ ** as 16 bit half-words.
[defined] FlushKeys [if] FlushKeys [then]	\ SFP002
  bounds ?do				\ get limits
    cr  i .lword  space   i 8 + i       \ print address
    do  space i w@ .word  2 +loop	\ first eight bytes
    space  i $010 + i 8 +               \ gap
    do  space i w@ .word  2 +loop	\ second eight bytes
\    2 spaces  i $010 + i                \ gap
\    do  i w@ dup $FF and .ascii  8 rshift .ascii  2 +loop  \ ascii o/p
    halt? ?leave			\ quit or halt
  $010 +loop                            \ until done
  cr  					\ restore base
;

: .usb		\ offset len --
\ *G Display USB packet memory.
  swap _USBSRAM + swap wdump
;

: .usb1		0 $100 .usb  ;
\ *G Display the first 256 bytes of USB packet memory.
: .dev		_usbdev $100 pdump ;
\ *G Display the first 64 registers (256 bytes) of USB peripheral space.


\ ======
\ *> ###
\ ======

decimal

