\ sfrNRF51.fth - peripherals for Nordic NRF51 devices
\ derived from nrf51.h, v522, 23 Feb 2016

((
Copyright (c) 2016
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel:   +44 (0)23 8063 1441
fax:   +44 (0)23 8033 9691
email: mpe@mpeforth.com
       tech-support@mpeforth.com
web:   http://www.mpeforth.com
Skype: mpe_sfp

From North America, our telephone and fax numbers are:
       011 44 23 8063 1441
       011 44 23 8033 9691
       901 313 4312 (North American access number to UK office)

Please note that MPE files have hard tabs set to 8. Hence, tabs
are at columns 1, 9, 17, 25 ...

To do
=====

Change history
==============
))

only forth definitions
decimal


\ ===========
\ *! sfrnrf51
\ *T Nordic NRF51 peripherals
\ ===========

\ ************************
\ Exception vector numbers
\ ************************
\ Equates of the form xxxVec# refer to Exception Vector position,
\ whereas those of the form xxxInt# refer to Interrupt#.
\ Nearly all MPE code uses the Vec# form. To convert to Int#,
\ subtract 16 from the Vec#.

\ nrf51 Specific Interrupt Numbers
#16 equ POWER_CLOCK_IRQn	\ 0  POWER_CLOCK
#17 equ RADIO_IRQn		\ 1  RADIO
#18 equ UART0_IRQn		\ 2  UART0
#19 equ SPI0_TWI0_IRQn		\ 3  SPI0_TWI0
#20 equ SPI1_TWI1_IRQn		\ 4  SPI1_TWI1

#22 equ GPIOTE_IRQn		\ 6  GPIOTE
#23 equ ADC_IRQn		\ 7  ADC
#24 equ TIMER0_IRQn		\ 8  TIMER0
#25 equ TIMER1_IRQn		\ 9  TIMER1
#26 equ TIMER2_IRQn		\ 10 TIMER2
#27 equ RTC0_IRQn		\ 11  RTC0
#28 equ TEMP_IRQn		\ 12  TEMP
#29 equ RNG_IRQn		\ 13  RNG
#30 equ ECB_IRQn		\ 14  ECB
#31 equ CCM_AAR_IRQn		\ 15  CCM_AAR
#32 equ WDT_IRQn		\ 16  WDT
#33 equ RTC1_IRQn		\ 17  RTC1
#34 equ QDEC_IRQn		\ 18  QDEC
#35 equ LPCOMP_IRQn		\ 19  LPCOMP
#36 equ SWI0_IRQn		\ 20  SWI0
#37 equ SWI1_IRQn		\ 21  SWI1
#38 equ SWI2_IRQn		\ 22  SWI2
#39 equ SWI3_IRQn		\ 23  SWI3
#40 equ SWI4_IRQn		\ 24  SWI4
#41 equ SWI5_IRQn		\ 25  SWI5


\ =========================
\ Peripheral base addresses
\ =========================

$4000:0000 equ NRF_POWER_BASE
$4000:0000 equ NRF_CLOCK_BASE
$4000:0000 equ NRF_MPU_BASE
$4000:0000 equ NRF_AMLI_BASE
$4000:1000 equ NRF_RADIO_BASE
$4000:2000 equ NRF_UART0_BASE
$4000:3000 equ NRF_SPI0_BASE
$4000:3000 equ NRF_TWI0_BASE
$4000:4000 equ NRF_SPI1_BASE
$4000:4000 equ NRF_TWI1_BASE
$4000:4000 equ NRF_SPIS1_BASE
$4000:4000 equ NRF_SPIM1_BASE
$4000:6000 equ NRF_GPIOTE_BASE
$4000:7000 equ NRF_ADC_BASE
$4000:8000 equ NRF_TIMER0_BASE
$4000:9000 equ NRF_TIMER1_BASE
$4000:A000 equ NRF_TIMER2_BASE
$4000:B000 equ NRF_RTC0_BASE
$4000:C000 equ NRF_TEMP_BASE
$4000:D000 equ NRF_RNG_BASE
$4000:E000 equ NRF_ECB_BASE
$4000:F000 equ NRF_AAR_BASE
$4000:F000 equ NRF_CCM_BASE
$4001:0000 equ NRF_WDT_BASE
$4001:1000 equ NRF_RTC1_BASE
$4001:2000 equ NRF_QDEC_BASE
$4001:3000 equ NRF_LPCOMP_BASE
$4001:4000 equ NRF_SWI_BASE
$4001:E000 equ NRF_NVMC_BASE
$4001:F000 equ NRF_PPI_BASE
$1000:0000 equ NRF_FICR_BASE
$1000:1000 equ NRF_UICR_BASE
$5000:0000 equ NRF_GPIO_BASE

((
typedef struct {
  __IO uint32_t  SCK;                               /*!< Pin select for SCK.                                                   */
  __IO uint32_t  MOSI;                              /*!< Pin select for MOSI.                                                  */
  __IO uint32_t  MISO;                              /*!< Pin select for MISO.                                                  */
} SPIM_PSEL_Type;

typedef struct {
  __IO uint32_t  PTR;                               /*!< Data pointer.                                                         */
  __IO uint32_t  MAXCNT;                            /*!< Maximum number of buffer bytes to receive.                            */
  __I  uint32_t  AMOUNT;                            /*!< Number of bytes received in the last transaction.                     */
} SPIM_RXD_Type;

typedef struct {
  __IO uint32_t  PTR;                               /*!< Data pointer.                                                         */
  __IO uint32_t  MAXCNT;                            /*!< Maximum number of buffer bytes to send.                               */
  __I  uint32_t  AMOUNT;                            /*!< Number of bytes sent in the last transaction.                         */
} SPIM_TXD_Type;

typedef struct {
  __O  uint32_t  EN;                                /*!< Enable channel group.                                                 */
  __O  uint32_t  DIS;                               /*!< Disable channel group.                                                */
} PPI_TASKS_CHG_Type;

typedef struct {
  __IO uint32_t  EEP;                               /*!< Channel event end-point.                                              */
  __IO uint32_t  TEP;                               /*!< Channel task end-point.                                               */
} PPI_CH_Type;
))


\ =====
\ POWER
\ =====

$4000:0000 equ _POWER
\ Tasks
  0x078 equ pwr.CONSTLAT	\ Enable constant latency mode
  0x07C equ pwr.LOWPWR		\ Enable low power mode (variable latency)
\ Events
  0x108 equ pwr.POFWARN		\ Power failure warning
\ Registers
  0x304 equ pwr.INTENSET	\ Enable interrupt
  0x308 equ pwr.INTENCLR	\ Disable interrupt
  0x400 equ pwr.RESETREAS  	\ Reset reason
  0x428 equ pwr.RAMSTATUS  	\ RAM status register
  0x500 equ pwr.SYSTEMOFF  	\ System OFF register
  0x510 equ pwr.POFCON  	\ Power failure comparator configuration
  0x51C equ pwr.GPREGRET  	\ General purpose retention register
  0x524 equ pwr.RAMON  		\ RAM on/off register (this register is retained)
  0x544 equ pwr.RESET  		\ Reset configuration register
  0x554 equ pwr.RAMONB  	\ RAM on/off register (this register is retained)
  0x578 equ pwr.DCDCEN  	\ DC/DC enable register
  0xA08 equ pwr.DCDFORCE	\ DC/DC power-up force register

((
typedef struct {                                    /*!< POWER Structure                                                       */
  __I  uint32_t  RESERVED0[30];
  __O  uint32_t  TASKS_CONSTLAT;                    /*!< Enable constant latency mode.                                         */
  __O  uint32_t  TASKS_LOWPWR;                      /*!< Enable low power mode (variable latency).                             */
  __I  uint32_t  RESERVED1[34];
  __IO uint32_t  EVENTS_POFWARN;                    /*!< Power failure warning.                                                */
  __I  uint32_t  RESERVED2[126];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[61];
  __IO uint32_t  RESETREAS;                         /*!< Reset reason.                                                         */
  __I  uint32_t  RESERVED4[9];
  __I  uint32_t  RAMSTATUS;                         /*!< Ram status register.                                                  */
  __I  uint32_t  RESERVED5[53];
  __O  uint32_t  SYSTEMOFF;                         /*!< System off register.                                                  */
  __I  uint32_t  RESERVED6[3];
  __IO uint32_t  POFCON;                            /*!< Power failure configuration.                                          */
  __I  uint32_t  RESERVED7[2];
  __IO uint32_t  GPREGRET;                          /*!< General purpose retention register. This register is a retained
                                                         register.                                                             */
  __I  uint32_t  RESERVED8;
  __IO uint32_t  RAMON;                             /*!< Ram on/off.                                                           */
  __I  uint32_t  RESERVED9[7];
  __IO uint32_t  RESET;                             /*!< Pin reset functionality configuration register. This register
                                                         is a retained register.                                               */
  __I  uint32_t  RESERVED10[3];
  __IO uint32_t  RAMONB;                            /*!< Ram on/off.                                                           */
  __I  uint32_t  RESERVED11[8];
  __IO uint32_t  DCDCEN;                            /*!< DCDC converter enable configuration register.                         */
  __I  uint32_t  RESERVED12[291];
  __IO uint32_t  DCDCFORCE;                         /*!< DCDC power-up force register.                                         */
} NRF_POWER_Type;
))


\ =====
\ CLOCK
\ =====

$4000:0000 equ _CLOCK
\ Tasks
  0x000 equ clk.HFCLKSTART  	\ Start HFCLK crystal oscillator
  0x004 equ clk.HFCLKSTOP  	\ Stop HFCLK crystal oscillator
  0x008 equ clk.LFCLKSTART  	\ Start LFCLK source
  0x00C equ clk.LFCLKSTOP  	\ Stop LFCLK source
  0x010 equ clk.CAL  		\ Start calibration of LFCLK RC oscillator
  0x014 equ clk.CTSTART  	\ Start calibration timer
  0x018 equ clk.CTSTOP 		\ Stop calibration timer
\ Events
  0x100 equ clk.HFCLKSTARTED 	\ HFCLK oscillator started
  0x104 equ clk.LFCLKSTARTED 	\ LFCLK started
  0x10C equ clk.DONE 		\ Calibration of LFCLK RC oscillator complete event
  0x110 equ clk.CTTO 		\ Calibration timer timeout
\ Registers
  0x304 equ clk.INTENSET 	\ Enable interrupt
  0x308 equ clk.INTENCLR 	\ Disable interrupt
  0x408 equ clk.HFCLKRUN 	\ Status indicating that HFCLKSTART task has been triggered
  0x40C equ clk.HFCLKSTAT 	\ Which HFCLK source is running
  0x414 equ clk.LFCLKRUN 	\ Status indicating that LFCLKSTART task has been triggered
  0x418 equ clk.LFCLKSTAT 	\ Which LFCLK source is running
  0x41C equ clk.LFCLKSRCCOPY 	\ Copy of LFCLKSRC register, set when LFCLKSTART task was triggered
  0x518 equ clk.LFCLKSRC 	\ Clock source for the LFCLK
  0x538 equ clk.CTIV 		\ Calibration timer interval
  0x550 equ clk.XTALFREQ 	\ Crystal frequency

((
typedef struct {                                    /*!< CLOCK Structure                                                       */
  __O  uint32_t  TASKS_HFCLKSTART;                  /*!< Start HFCLK clock source.                                             */
  __O  uint32_t  TASKS_HFCLKSTOP;                   /*!< Stop HFCLK clock source.                                              */
  __O  uint32_t  TASKS_LFCLKSTART;                  /*!< Start LFCLK clock source.                                             */
  __O  uint32_t  TASKS_LFCLKSTOP;                   /*!< Stop LFCLK clock source.                                              */
  __O  uint32_t  TASKS_CAL;                         /*!< Start calibration of LFCLK RC oscillator.                             */
  __O  uint32_t  TASKS_CTSTART;                     /*!< Start calibration timer.                                              */
  __O  uint32_t  TASKS_CTSTOP;                      /*!< Stop calibration timer.                                               */
  __I  uint32_t  RESERVED0[57];
  __IO uint32_t  EVENTS_HFCLKSTARTED;               /*!< HFCLK oscillator started.                                             */
  __IO uint32_t  EVENTS_LFCLKSTARTED;               /*!< LFCLK oscillator started.                                             */
  __I  uint32_t  RESERVED1;
  __IO uint32_t  EVENTS_DONE;                       /*!< Calibration of LFCLK RC oscillator completed.                         */
  __IO uint32_t  EVENTS_CTTO;                       /*!< Calibration timer timeout.                                            */
  __I  uint32_t  RESERVED2[124];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[63];
  __I  uint32_t  HFCLKRUN;                          /*!< Task HFCLKSTART trigger status.                                       */
  __I  uint32_t  HFCLKSTAT;                         /*!< High frequency clock status.                                          */
  __I  uint32_t  RESERVED4;
  __I  uint32_t  LFCLKRUN;                          /*!< Task LFCLKSTART triggered status.                                     */
  __I  uint32_t  LFCLKSTAT;                         /*!< Low frequency clock status.                                           */
  __I  uint32_t  LFCLKSRCCOPY;                      /*!< Clock source for the LFCLK clock, set when task LKCLKSTART is
                                                         triggered.                                                            */
  __I  uint32_t  RESERVED5[62];
  __IO uint32_t  LFCLKSRC;                          /*!< Clock source for the LFCLK clock.                                     */
  __I  uint32_t  RESERVED6[7];
  __IO uint32_t  CTIV;                              /*!< Calibration timer interval.                                           */
  __I  uint32_t  RESERVED7[5];
  __IO uint32_t  XTALFREQ;                          /*!< Crystal frequency.                                                    */
} NRF_CLOCK_Type;
))


\ ===
\ MPU
\ ===

$4000:0000 equ _MPU
  0x528 equ mpu.PERR0 		\ Definition of peripherals in memory region 0
  0x52C equ mpu.RLENR0 		\ Length of RAM region 0
  0x600 equ mpu.PROTENSET0 	\ Protection bit enable set register
  0x604 equ mpu.PROTENSET1 	\ Protection bit enable set register
  0x608 equ mpu.DISABLEINDEBUG 	\ Disable protection mechanism in debug mode
  0x60C equ mpu.PROTBLOCKSIZE 	\ Protection block size

((
typedef struct {                                    /*!< MPU Structure                                                         */
  __I  uint32_t  RESERVED0[330];
  __IO uint32_t  PERR0;                             /*!< Configuration of peripherals in mpu regions.                          */
  __IO uint32_t  RLENR0;                            /*!< Length of RAM region 0.                                               */
  __I  uint32_t  RESERVED1[52];
  __IO uint32_t  PROTENSET0;                        /*!< Erase and write protection bit enable set register.                   */
  __IO uint32_t  PROTENSET1;                        /*!< Erase and write protection bit enable set register.                   */
  __IO uint32_t  DISABLEINDEBUG;                    /*!< Disable erase and write protection mechanism in debug mode.           */
  __IO uint32_t  PROTBLOCKSIZE;                     /*!< Erase and write protection block size.                                */
} NRF_MPU_Type;
))


\ ====
\ AMLI
\ ====

$4000:0000 equ _AMLI
  $E00 equ aml.CPU0
  $E04 equ aml.SPIS1
  $E08 equ aml.RADIO
  $E0C equ aml.ECB
  $E10 equ aml.CCM
  $E14 equ aml.AAR
((
typedef struct {
  __IO uint32_t  CPU0;                              /*!< Configurable priority configuration register for CPU0.                */
  __IO uint32_t  SPIS1;                             /*!< Configurable priority configuration register for SPIS1.               */
  __IO uint32_t  RADIO;                             /*!< Configurable priority configuration register for RADIO.               */
  __IO uint32_t  ECB;                               /*!< Configurable priority configuration register for ECB.                 */
  __IO uint32_t  CCM;                               /*!< Configurable priority configuration register for CCM.                 */
  __IO uint32_t  AAR;                               /*!< Configurable priority configuration register for AAR.                 */
} AMLI_RAMPRI_Type;

typedef struct {                                    /*!< AMLI Structure                                                        */
  __I  uint32_t  RESERVED0[896];
  AMLI_RAMPRI_Type RAMPRI;                          /*!< RAM configurable priority configuration structure.                    */
} NRF_AMLI_Type;
))


\ =====
\ RADIO
\ =====

$4000:1000 equ _RADIO
\ Tasks
  0x000 equ rad.TXEN 		\ Enable RADIO in TX mode
  0x004 equ rad.RXEN 		\ Enable RADIO in RX mode
  0x008 equ rad.START 		\ Start RADIO
  0x00C equ rad.STOP 		\ Stop RADIO
  0x010 equ rad.DISABLE 	\ Disable RADIO
  0x014 equ rad.RSSISTART 	\ Start the RSSI and take one single sample of the receive signal strength
  0x018 equ rad.RSSISTOP 	\ Stop the RSSI measurement
  0x01C equ rad.BCSTART 	\ Start the bit counter
  0x020 equ rad.BCSTOP 		\ Stop the bit counter
\ Events
  0x100 equ rad.READY 		\ RADIO has ramped up and is ready to be started
  0x104 equ rad.ADDRESS 	\ Address sent or received
  0x108 equ rad.PAYLOAD 	\ Packet payload sent or received
  0x10C equ rad.END 		\ Packet sent or received
  0x110 equ rad.DISABLED 	\ RADIO has been disabled
  0x114 equ rad.DEVMATCH 	\ A device address match occurred on the last received packet
  0x118 equ rad.DEVMISS 	\ No device address match occurred on the last received packet
  0x11C equ rad.RSSIEND 	\ Sampling of receive signal strength complete. A new RSSI sample is ready for readout from the RSSISAMPLE register.
  0x128 equ rad.BCMATCH 	\ Bit counter reached bit count value specified in the BCC register
\ Registers
  0x200 equ rad.SHORTS 		\ Shortcut register
  0x304 equ rad.INTENSET 	\ Enable interrupt
  0x308 equ rad.INTENCLR 	\ Disable interrupt
  0x400 equ rad.CRCSTATUS 	\ CRC status
  0x408 equ rad.RXMATCH 	\ Received address
  0x40C equ rad.RXCRC 		\ CRC field of previously received packet
  0x410 equ rad.DAI 		\ Device address match index
  0x504 equ rad.PACKETPTR 	\ Packet pointer
  0x508 equ rad.FREQUENCY 	\ Frequency
  0x50C equ rad.TXPOWER 	\ Output power
  0x510 equ rad.MODE 		\ Data rate and modulation
  0x514 equ rad.PCNF0 		\ Packet configuration register 0
  0x518 equ rad.PCNF1 		\ Packet configuration register 1
  0x51C equ rad.BASE0 		\ Base address 0
  0x520 equ rad.BASE1 		\ Base address 1
  0x524 equ rad.PREFIX0 	\ Prefixes bytes for logical addresses 0-3
  0x528 equ rad.PREFIX1 	\ Prefixes bytes for logical addresses 4-7
  0x52C equ rad.TXADDRESS 	\ Transmit address select
  0x530 equ rad.RXADDRESSES 	\ Receive address select
  0x534 equ rad.CRCCNF 		\ CRC configuration
  0x538 equ rad.CRCPOLY 	\ CRC polynomial
  0x53C equ rad.CRCINIT 	\ CRC initial value
  0x540 equ rad.TEST 		\ Test features enable register
  0x544 equ rad.TIFS 		\ Inter Frame Spacing in us
  0x548 equ rad.RSSISAMPLE 	\ RSSI sample
  0x550 equ rad.STATE 		\ Current radio state
  0x554 equ rad.DATAWHITEIV 	\ Data whitening initial value
  0x560 equ rad.BCC 		\ Bit counter compare
  0x600 equ rad.DAB[0] 		\ Device address base segment 0
  0x604 equ rad.DAB[1] 		\ Device address base segment 1
  0x608 equ rad.DAB[2] 		\ Device address base segment 2
  0x60C equ rad.DAB[3] 		\ Device address base segment 3
  0x610 equ rad.DAB[4] 		\ Device address base segment 4
  0x614 equ rad.DAB[5] 		\ Device address base segment 5
  0x618 equ rad.DAB[6] 		\ Device address base segment 6
  0x61C equ rad.DAB[7] 		\ Device address base segment 7
  0x620 equ rad.DAP[0] 		\ Device address prefix 0
  0x624 equ rad.DAP[1] 		\ Device address prefix 1
  0x628 equ rad.DAP[2] 		\ Device address prefix 2
  0x62C equ rad.DAP[3] 		\ Device address prefix 3
  0x630 equ rad.DAP[4] 		\ Device address prefix 4
  0x634 equ rad.DAP[5] 		\ Device address prefix 5
  0x638 equ rad.DAP[6] 		\ Device address prefix 6
  0x63C equ rad.DAP[7] 		\ Device address prefix 7
  0x640 equ rad.DACNF 		\ Device address match configuration
  0x724 equ rad.OVERRIDE0 	\ Trim value override register 0
  0x728 equ rad.OVERRIDE1 	\ Trim value override register 1
  0x72C equ rad.OVERRIDE2 	\ Trim value override register 2
  0x730 equ rad.OVERRIDE3 	\ Trim value override register 3
  0x734 equ rad.OVERRIDE4 	\ Trim value override register 4
  0xFFC equ rad.POWER 		\ Peripheral power control
((
typedef struct {                                    /*!< RADIO Structure                                                       */
  __O  uint32_t  TASKS_TXEN;                        /*!< Enable radio in TX mode.                                              */
  __O  uint32_t  TASKS_RXEN;                        /*!< Enable radio in RX mode.                                              */
  __O  uint32_t  TASKS_START;                       /*!< Start radio.                                                          */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop radio.                                                           */
  __O  uint32_t  TASKS_DISABLE;                     /*!< Disable radio.                                                        */
  __O  uint32_t  TASKS_RSSISTART;                   /*!< Start the RSSI and take one sample of the receive signal strength.    */
  __O  uint32_t  TASKS_RSSISTOP;                    /*!< Stop the RSSI measurement.                                            */
  __O  uint32_t  TASKS_BCSTART;                     /*!< Start the bit counter.                                                */
  __O  uint32_t  TASKS_BCSTOP;                      /*!< Stop the bit counter.                                                 */
  __I  uint32_t  RESERVED0[55];
  __IO uint32_t  EVENTS_READY;                      /*!< Ready event.                                                          */
  __IO uint32_t  EVENTS_ADDRESS;                    /*!< Address event.                                                        */
  __IO uint32_t  EVENTS_PAYLOAD;                    /*!< Payload event.                                                        */
  __IO uint32_t  EVENTS_END;                        /*!< End event.                                                            */
  __IO uint32_t  EVENTS_DISABLED;                   /*!< Disable event.                                                        */
  __IO uint32_t  EVENTS_DEVMATCH;                   /*!< A device address match occurred on the last received packet.          */
  __IO uint32_t  EVENTS_DEVMISS;                    /*!< No device address match occurred on the last received packet.         */
  __IO uint32_t  EVENTS_RSSIEND;                    /*!< Sampling of the receive signal strength complete. A new RSSI
                                                         sample is ready for readout at the RSSISAMPLE register.               */
  __I  uint32_t  RESERVED1[2];
  __IO uint32_t  EVENTS_BCMATCH;                    /*!< Bit counter reached bit count value specified in BCC register.        */
  __I  uint32_t  RESERVED2[53];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for the radio.                                              */
  __I  uint32_t  RESERVED3[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED4[61];
  __I  uint32_t  CRCSTATUS;                         /*!< CRC status of received packet.                                        */
  __I  uint32_t  RESERVED5;
  __I  uint32_t  RXMATCH;                           /*!< Received address.                                                     */
  __I  uint32_t  RXCRC;                             /*!< Received CRC.                                                         */
  __I  uint32_t  DAI;                               /*!< Device address match index.                                           */
  __I  uint32_t  RESERVED6[60];
  __IO uint32_t  PACKETPTR;                         /*!< Packet pointer. Decision point: START task.                           */
  __IO uint32_t  FREQUENCY;                         /*!< Frequency.                                                            */
  __IO uint32_t  TXPOWER;                           /*!< Output power.                                                         */
  __IO uint32_t  MODE;                              /*!< Data rate and modulation.                                             */
  __IO uint32_t  PCNF0;                             /*!< Packet configuration 0.                                               */
  __IO uint32_t  PCNF1;                             /*!< Packet configuration 1.                                               */
  __IO uint32_t  BASE0;                             /*!< Radio base address 0. Decision point: START task.                     */
  __IO uint32_t  BASE1;                             /*!< Radio base address 1. Decision point: START task.                     */
  __IO uint32_t  PREFIX0;                           /*!< Prefixes bytes for logical addresses 0 to 3.                          */
  __IO uint32_t  PREFIX1;                           /*!< Prefixes bytes for logical addresses 4 to 7.                          */
  __IO uint32_t  TXADDRESS;                         /*!< Transmit address select.                                              */
  __IO uint32_t  RXADDRESSES;                       /*!< Receive address select.                                               */
  __IO uint32_t  CRCCNF;                            /*!< CRC configuration.                                                    */
  __IO uint32_t  CRCPOLY;                           /*!< CRC polynomial.                                                       */
  __IO uint32_t  CRCINIT;                           /*!< CRC initial value.                                                    */
  __IO uint32_t  TEST;                              /*!< Test features enable register.                                        */
  __IO uint32_t  TIFS;                              /*!< Inter Frame Spacing in microseconds.                                  */
  __I  uint32_t  RSSISAMPLE;                        /*!< RSSI sample.                                                          */
  __I  uint32_t  RESERVED7;
  __I  uint32_t  STATE;                             /*!< Current radio state.                                                  */
  __IO uint32_t  DATAWHITEIV;                       /*!< Data whitening initial value.                                         */
  __I  uint32_t  RESERVED8[2];
  __IO uint32_t  BCC;                               /*!< Bit counter compare.                                                  */
  __I  uint32_t  RESERVED9[39];
  __IO uint32_t  DAB[8];                            /*!< Device address base segment.                                          */
  __IO uint32_t  DAP[8];                            /*!< Device address prefix.                                                */
  __IO uint32_t  DACNF;                             /*!< Device address match configuration.                                   */
  __I  uint32_t  RESERVED10[56];
  __IO uint32_t  OVERRIDE0;                         /*!< Trim value override register 0.                                       */
  __IO uint32_t  OVERRIDE1;                         /*!< Trim value override register 1.                                       */
  __IO uint32_t  OVERRIDE2;                         /*!< Trim value override register 2.                                       */
  __IO uint32_t  OVERRIDE3;                         /*!< Trim value override register 3.                                       */
  __IO uint32_t  OVERRIDE4;                         /*!< Trim value override register 4.                                       */
  __I  uint32_t  RESERVED11[561];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_RADIO_Type;
))


\ ====
\ UART
\ ====

$4000:2000 equ _UART0
\ Tasks
  0x000 equ un.STARTRX 		\ Start UART receiver
  0x004 equ un.STOPRX 		\ Stop UART receiver
  0x008 equ un.STARTTX 		\ Start UART transmitter
  0x00C equ un.STOPTX 		\ Stop UART transmitter
  0x01C equ un.SUSPEND 		\ Suspend UART
\ Events
  0x100 equ un.CTS 		\ CTS is activated (set low). Clear To Send.
  0x104 equ un.NCTS 		\ CTS is deactivated (set high). Not Clear To Send.
  0x108 equ un.RXDRDY 		\ Data received in RXD
  0x11C equ un.TXDRDY 		\ Data sent from TXD
  0x124 equ un.ERROR 		\ Error detected
  0x144 equ un.RXTO 		\ Receiver timeout
\ Registers
  0x300 equ un.INTEN 		\ Enable or disable interrupt
  0x304 equ un.INTENSET 	\ Enable interrupt
  0x308 equ un.INTENCLR 	\ Disable interrupt
  0x480 equ un.ERRORSRC 	\ Error source
  0x500 equ un.ENABLE 		\ Enable UART
  0x508 equ un.PSELRTS 		\ Pin select for RTS
  0x50C equ un.PSELTXD 		\ Pin select for TXD
  0x510 equ un.PSELCTS 		\ Pin select for CTS
  0x514 equ un.PSELRXD 		\ Pin select for RXD
  0x518 equ un.RXD 		\ RXD register
  0x51C equ un.TXD 		\ TXD register
  0x524 equ un.BAUDRATE 	\ Baud rate
  0x56C equ un.CONFIG 		\ Configuration of parity and hardware flow control
  0xFFC equ un.POWER

((
typedef struct {                                    /*!< UART Structure                                                        */
  __O  uint32_t  TASKS_STARTRX;                     /*!< Start UART receiver.                                                  */
  __O  uint32_t  TASKS_STOPRX;                      /*!< Stop UART receiver.                                                   */
  __O  uint32_t  TASKS_STARTTX;                     /*!< Start UART transmitter.                                               */
  __O  uint32_t  TASKS_STOPTX;                      /*!< Stop UART transmitter.                                                */
  __I  uint32_t  RESERVED0[3];
  __O  uint32_t  TASKS_SUSPEND;                     /*!< Suspend UART.                                                         */
  __I  uint32_t  RESERVED1[56];
  __IO uint32_t  EVENTS_CTS;                        /*!< CTS activated.                                                        */
  __IO uint32_t  EVENTS_NCTS;                       /*!< CTS deactivated.                                                      */
  __IO uint32_t  EVENTS_RXDRDY;                     /*!< Data received in RXD.                                                 */
  __I  uint32_t  RESERVED2[4];
  __IO uint32_t  EVENTS_TXDRDY;                     /*!< Data sent from TXD.                                                   */
  __I  uint32_t  RESERVED3;
  __IO uint32_t  EVENTS_ERROR;                      /*!< Error detected.                                                       */
  __I  uint32_t  RESERVED4[7];
  __IO uint32_t  EVENTS_RXTO;                       /*!< Receiver timeout.                                                     */
  __I  uint32_t  RESERVED5[46];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for UART.                                                   */
  __I  uint32_t  RESERVED6[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED7[93];
  __IO uint32_t  ERRORSRC;                          /*!< Error source. Write error field to 1 to clear error.                  */
  __I  uint32_t  RESERVED8[31];
  __IO uint32_t  ENABLE;                            /*!< Enable UART and acquire IOs.                                          */
  __I  uint32_t  RESERVED9;
  __IO uint32_t  PSELRTS;                           /*!< Pin select for RTS.                                                   */
  __IO uint32_t  PSELTXD;                           /*!< Pin select for TXD.                                                   */
  __IO uint32_t  PSELCTS;                           /*!< Pin select for CTS.                                                   */
  __IO uint32_t  PSELRXD;                           /*!< Pin select for RXD.                                                   */
  __I  uint32_t  RXD;                               /*!< RXD register. On read action the buffer pointer is displaced.
                                                         Once read the character is consumed. If read when no character
                                                          available, the UART will stop working.                               */
  __O  uint32_t  TXD;                               /*!< TXD register.                                                         */
  __I  uint32_t  RESERVED10;
  __IO uint32_t  BAUDRATE;                          /*!< UART Baudrate.                                                        */
  __I  uint32_t  RESERVED11[17];
  __IO uint32_t  CONFIG;                            /*!< Configuration of parity and hardware flow control register.           */
  __I  uint32_t  RESERVED12[675];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_UART_Type;
))


\ ===
\ SPI
\ ===

$4000:3000 equ _SPI0
$4000:4000 equ _SPI1		\ S = SPIS1 only, M = Master from .h file
\ Tasks
  0x010 equ spi.START		\ M Start SPI transaction
  0x014 equ spi.STOP		\ M Stop SPI transaction
  0x01C equ spi.SUSPEND		\ Suspend SPI transaction
  0x020 equ spi.RESUME		\ M Resume SPI transaction.                                               */
  0x024 equ spi.ACQUIRE		\ S Acquire SPI semaphore
  0x028 equ spi.RELEASE 	\ S Release SPI semaphore, enabling the SPI slave to acquire it
\ Events
  0x104 equ spi.END 		\ S Granted transaction completed
  0x104 equ spi.STOPPED		\ M Transaction completed
  0x108 equ spi.READY		\ TXD byte sent and RXD byte received
  0x110 equ spi.ENDRX		\ M End of RXD buffer reached.
  0x120 equ spi.ENDTX		\ M End of TXD buffer reached.
  0x128 equ spi.ACQUIRED 	\ S Semaphore acquired
  0x140 equ spi.STARTED		\ M Transaction started.
\ Registers
  0x200 equ spi.SHORTS 		\ S Shortcut register
  0x300 equ spi.INTEN 		\ Enable or disable interrupt
  0x304 equ spi.INTENSET 	\ Enable interrupt
  0x308 equ spi.INTENCLR 	\ Disable interrupt
  0x400 equ spi.SEMSTAT 	\ S Semaphore status register
  0x440 equ spi.STATUS 		\ S Status from last transaction
  0x500 equ spi.ENABLE 		\ Enable SPI
  0x508 equ spi.PSELSCK 	\ Pin select for SCK
  0x50C equ spi.PSELMOSI 	\ Pin select for MOSI
  0x510 equ spi.PSELMISO 	\ Pin select for MISO
  0x514 equ spi.PSELCSN 	\ S Pin select for CSN
  0x518 equ spi.RXD 		\ RXD register
  0x51C equ spi.TXD 		\ TXD register
  0x524 equ spi.FREQUENCY 	\ SPI frequency
  0x534 equ spi.RXDPTR 		\ S RXD data pointer
  0x538 equ spi.MAXRX 		\ S Maximum number of bytes in receive buffer
  0x53C equ spi.AMOUNTRX 	\ S Number of bytes received in last granted transaction
  0x544 equ spi.TXDPTR 		\ S TXD data pointer
  0x548 equ spi.MAXTX 		\ S Maximum number of bytes in transmit buffer
  0x54C equ spi.AMOUNTTX 	\ S Number of bytes transmitted in last granted transaction
  0x554 equ spi.CONFIG 		\ Configuration register
  0x55C equ spi.DEF 		\ S Default character. Character clocked out in case of an ignored transaction.
  0x5C0 equ spi.ORC 		\ S Over-read character
  0xFFC equ spi.POWER
((
typedef struct {                                    /*!< SPIM Structure                                                        */
  __I  uint32_t  RESERVED0[4];
  __O  uint32_t  TASKS_START;                       /*!< Start SPI transaction.                                                */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop SPI transaction.                                                 */
  __I  uint32_t  RESERVED1;
  __O  uint32_t  TASKS_SUSPEND;                     /*!< Suspend SPI transaction.                                              */
  __O  uint32_t  TASKS_RESUME;                      /*!< Resume SPI transaction.                                               */
  __I  uint32_t  RESERVED2[56];
  __IO uint32_t  EVENTS_STOPPED;                    /*!< SPI transaction has stopped.                                          */
  __I  uint32_t  RESERVED3[2];
  __IO uint32_t  EVENTS_ENDRX;                      /*!< End of RXD buffer reached.                                            */
  __I  uint32_t  RESERVED4[3];
  __IO uint32_t  EVENTS_ENDTX;                      /*!< End of TXD buffer reached.                                            */
  __I  uint32_t  RESERVED5[10];
  __IO uint32_t  EVENTS_STARTED;                    /*!< Transaction started.                                                  */
  __I  uint32_t  RESERVED6[109];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED7[125];
  __IO uint32_t  ENABLE;                            /*!< Enable SPIM.                                                          */
  __I  uint32_t  RESERVED8;
  SPIM_PSEL_Type PSEL;                              /*!< Pin select configuration.                                             */
  __I  uint32_t  RESERVED9[4];
  __IO uint32_t  FREQUENCY;                         /*!< SPI frequency.                                                        */
  __I  uint32_t  RESERVED10[3];
  SPIM_RXD_Type RXD;                                /*!< RXD EasyDMA configuration and status.                                 */
  __I  uint32_t  RESERVED11;
  SPIM_TXD_Type TXD;                                /*!< TXD EasyDMA configuration and status.                                 */
  __I  uint32_t  RESERVED12;
  __IO uint32_t  CONFIG;                            /*!< Configuration register.                                               */
  __I  uint32_t  RESERVED13[26];
  __IO uint32_t  ORC;                               /*!< Over-read character.                                                  */
  __I  uint32_t  RESERVED14[654];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_SPIM_Type;
))
((
typedef struct {                                    /*!< SPI Structure                                                         */
  __I  uint32_t  RESERVED0[66];
  __IO uint32_t  EVENTS_READY;                      /*!< TXD byte sent and RXD byte received.                                  */
  __I  uint32_t  RESERVED1[126];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED2[125];
  __IO uint32_t  ENABLE;                            /*!< Enable SPI.                                                           */
  __I  uint32_t  RESERVED3;
  __IO uint32_t  PSELSCK;                           /*!< Pin select for SCK.                                                   */
  __IO uint32_t  PSELMOSI;                          /*!< Pin select for MOSI.                                                  */
  __IO uint32_t  PSELMISO;                          /*!< Pin select for MISO.                                                  */
  __I  uint32_t  RESERVED4;
  __I  uint32_t  RXD;                               /*!< RX data.                                                              */
  __IO uint32_t  TXD;                               /*!< TX data.                                                              */
  __I  uint32_t  RESERVED5;
  __IO uint32_t  FREQUENCY;                         /*!< SPI frequency                                                         */
  __I  uint32_t  RESERVED6[11];
  __IO uint32_t  CONFIG;                            /*!< Configuration register.                                               */
  __I  uint32_t  RESERVED7[681];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_SPI_Type;
))


\ ===
\ TWI
\ ===

$4000:3000 equ _TWI0
$4000:4000 equ _TWI1
\ Tasks
  0x000 equ twi.STARTRX 	\ Start TWI receive sequence
  0x008 equ twi.STARTTX 	\ Start TWI transmit sequence
  0x014 equ twi.STOP 		\ Stop TWI transaction
  0x01C equ twi.SUSPEND 	\ Suspend TWI transaction
  0x020 equ twi.RESUME 		\ Resume TWI transaction
\ Events
  0x104 equ twi.STOPPED 	\ TWI stopped
  0x108 equ twi.RXDREADY 	\ TWI RXD byte received
  0x11C equ twi.TXDSENT 	\ TWI TXD byte sent
  0x124 equ twi.ERROR 		\ TWI error
  0x138 equ twi.BB 		\ TWI byte boundary, generated before each byte that is sent or received
\ Registers
  0x200 equ twi.SHORTS 		\ Shortcut register
  0x300 equ twi.INTEN 		\ Enable or disable interrupt
  0x304 equ twi.INTENSET 	\ Enable interrupt
  0x308 equ twi.INTENCLR 	\ Disable interrupt
  0x4C4 equ twi.ERRORSRC 	\ Error source
  0x500 equ twi.ENABLE 		\ Enable TWI
  0x508 equ twi.PSELSCL 	\ Pin select for SCL
  0x50C equ twi.PSELSDA 	\ Pin select for SDA
  0x518 equ twi.RXD 		\ RXD register
  0x51C equ twi.TXD 		\ TXD register
  0x524 equ twi.FREQUENCY 	\ TWI frequency
  0x588 equ twi.ADDRESS 	\ Address used in the TWI transfer
  0xFFC equ twi.POWER
((
typedef struct {                                    /*!< TWI Structure                                                         */
  __O  uint32_t  TASKS_STARTRX;                     /*!< Start 2-Wire master receive sequence.                                 */
  __I  uint32_t  RESERVED0;
  __O  uint32_t  TASKS_STARTTX;                     /*!< Start 2-Wire master transmit sequence.                                */
  __I  uint32_t  RESERVED1[2];
  __O  uint32_t  TASKS_STOP;                        /*!< Stop 2-Wire transaction.                                              */
  __I  uint32_t  RESERVED2;
  __O  uint32_t  TASKS_SUSPEND;                     /*!< Suspend 2-Wire transaction.                                           */
  __O  uint32_t  TASKS_RESUME;                      /*!< Resume 2-Wire transaction.                                            */
  __I  uint32_t  RESERVED3[56];
  __IO uint32_t  EVENTS_STOPPED;                    /*!< Two-wire stopped.                                                     */
  __IO uint32_t  EVENTS_RXDREADY;                   /*!< Two-wire ready to deliver new RXD byte received.                      */
  __I  uint32_t  RESERVED4[4];
  __IO uint32_t  EVENTS_TXDSENT;                    /*!< Two-wire finished sending last TXD byte.                              */
  __I  uint32_t  RESERVED5;
  __IO uint32_t  EVENTS_ERROR;                      /*!< Two-wire error detected.                                              */
  __I  uint32_t  RESERVED6[4];
  __IO uint32_t  EVENTS_BB;                         /*!< Two-wire byte boundary.                                               */
  __I  uint32_t  RESERVED7[3];
  __IO uint32_t  EVENTS_SUSPENDED;                  /*!< Two-wire suspended.                                                   */
  __I  uint32_t  RESERVED8[45];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for TWI.                                                    */
  __I  uint32_t  RESERVED9[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED10[110];
  __IO uint32_t  ERRORSRC;                          /*!< Two-wire error source. Write error field to 1 to clear error.         */
  __I  uint32_t  RESERVED11[14];
  __IO uint32_t  ENABLE;                            /*!< Enable two-wire master.                                               */
  __I  uint32_t  RESERVED12;
  __IO uint32_t  PSELSCL;                           /*!< Pin select for SCL.                                                   */
  __IO uint32_t  PSELSDA;                           /*!< Pin select for SDA.                                                   */
  __I  uint32_t  RESERVED13[2];
  __I  uint32_t  RXD;                               /*!< RX data register.                                                     */
  __IO uint32_t  TXD;                               /*!< TX data register.                                                     */
  __I  uint32_t  RESERVED14;
  __IO uint32_t  FREQUENCY;                         /*!< Two-wire frequency.                                                   */
  __I  uint32_t  RESERVED15[24];
  __IO uint32_t  ADDRESS;                           /*!< Address used in the two-wire transfer.                                */
  __I  uint32_t  RESERVED16[668];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_TWI_Type;
))


\ ======
\ GPIOTE
\ ======

$4000:6000 equ _GPIOTE
\ Tasks
\ Events
\ Registers
  0x000 equ gte.OUT[0] 		\ Task for writing to pin specified in CONFIG[0].PSEL. Action on pin is configured in CONFIG[0].POLARITY.
  0x004 equ gte.OUT[1] 		\ Task for writing to pin specified in CONFIG[1].PSEL. Action on pin is configured in CONFIG[1].POLARITY.
  0x008 equ gte.OUT[2] 		\ Task for writing to pin specified in CONFIG[2].PSEL. Action on pin is configured in CONFIG[2].POLARITY.
  0x00C equ gte.OUT[3] 		\ Task for writing to pin specified in CONFIG[3].PSEL. Action on pin is configured in CONFIG[3].POLARITY.
  0x100 equ gte.IN[0] 		\ Event generated from pin specified in CONFIG[0].PSEL
  0x104 equ gte.IN[1] 		\ Event generated from pin specified in CONFIG[1].PSEL
  0x108 equ gte.IN[2] 		\ Event generated from pin specified in CONFIG[2].PSEL
  0x10C equ gte.IN[3] 		\ Event generated from pin specified in CONFIG[3].PSEL
  0x17C equ gte.PORT 		\ Event generated from multiple input pins
  0x300 equ gte.INTEN 		\ Enable or disable interrupt
  0x304 equ gte.INTENSET 	\ Enable interrupt
  0x308 equ gte.INTENCLR 	\ Disable interrupt
  0x510 equ gte.CONFIG[0] 	\ Configuration for OUT[n] task and IN[n] event
  0x514 equ gte.CONFIG[1] 	\ Configuration for OUT[n] task and IN[n] event
  0x518 equ gte.CONFIG[2] 	\ Configuration for OUT[n] task and IN[n] event
  0x51C equ gte.CONFIG[3] 	\ Configuration for OUT[n] task and IN[n] event
  0xFFC equ gte.POWER
((
typedef struct {                                    /*!< GPIOTE Structure                                                      */
  __O  uint32_t  TASKS_OUT[4];                      /*!< Tasks asssociated with GPIOTE channels.                               */
  __I  uint32_t  RESERVED0[60];
  __IO uint32_t  EVENTS_IN[4];                      /*!< Tasks asssociated with GPIOTE channels.                               */
  __I  uint32_t  RESERVED1[27];
  __IO uint32_t  EVENTS_PORT;                       /*!< Event generated from multiple pins.                                   */
  __I  uint32_t  RESERVED2[97];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[129];
  __IO uint32_t  CONFIG[4];                         /*!< Channel configuration registers.                                      */
  __I  uint32_t  RESERVED4[695];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_GPIOTE_Type;
))

\ ===
\ ADC
\ ===

$4000:7000 equ _ADC
\ Tasks
  0x000 equ adc.START 		\ Start a new ADC conversion
  0x004 equ adc.STOP 		\ Stop ADC
\ Events
  0x100 equ adc.END 		\ An ADC conversion is completed
\ Registers
  0x300 equ adc.INTEN 		\ Enable or disable interrupt
  0x304 equ adc.INTENSET 	\ Enable interrupt
  0x308 equ adc.INTENCLR 	\ Disable interrupt
  0x400 equ adc.BUSY 		\ ADC busy (conversion in progress)
  0x500 equ adc.ENABLE  	\ Enable ADC. When enabled, the ADC will acquire access to the analog input pins specified in the CONFIG register.
  0x504 equ adc.CONFIG 		\ ADC configuration
  0x508 equ adc.RESULT 		\ Result of the previous ADC conversion
  0xFFC equ adc.POWER

((
typedef struct {                                    /*!< ADC Structure                                                         */
  __O  uint32_t  TASKS_START;                       /*!< Start an ADC conversion.                                              */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop ADC.                                                             */
  __I  uint32_t  RESERVED0[62];
  __IO uint32_t  EVENTS_END;                        /*!< ADC conversion complete.                                              */
  __I  uint32_t  RESERVED1[128];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED2[61];
  __I  uint32_t  BUSY;                              /*!< ADC busy register.                                                    */
  __I  uint32_t  RESERVED3[63];
  __IO uint32_t  ENABLE;                            /*!< ADC enable.                                                           */
  __IO uint32_t  CONFIG;                            /*!< ADC configuration register.                                           */
  __I  uint32_t  RESULT;                            /*!< Result of ADC conversion.                                             */
  __I  uint32_t  RESERVED4[700];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_ADC_Type;
))


\ =====
\ TIMER
\ =====

$4000:8000 equ _TIMER0
$4000:9000 equ _TIMER1
$4000:A000 equ _TIMER2
\ Tasks
  0x000 equ tim.START 		\ Start Timer
  0x004 equ tim.STOP 		\ Stop Timer
  0x008 equ tim.COUNT 		\ Increment Timer (Counter mode only)
  0x00C equ tim.CLEAR 		\ Clear time
  0x010 equ tim.SHUTDOWN 	\ Shut down timer
  0x040 equ tim.CAPTURE[0] 	\ Capture Timer value to CC[0] register
  0x044 equ tim.CAPTURE[1] 	\ Capture Timer value to CC[1] register
  0x048 equ tim.CAPTURE[2] 	\ Capture Timer value to CC[2] register
  0x04C equ tim.CAPTURE[3] 	\ Capture Timer value to CC[3] register
\ Events
  0x140 equ tim.COMPARE[0] 	\ Compare event on CC[0] match
  0x144 equ tim.COMPARE[1] 	\ Compare event on CC[1] match
  0x148 equ tim.COMPARE[2] 	\ Compare event on CC[2] match
  0x14C equ tim.COMPARE[3] 	\ Compare event on CC[3] match
\ Registers
  0x200 equ tim.SHORTS 		\ Shortcut register
  0x304 equ tim.INTENSET 	\ Enable interrupt
  0x308 equ tim.INTENCLR 	\ Disable interrupt
  0x504 equ tim.MODE 		\ Timer mode selection
  0x508 equ tim.BITMODE 	\ Configure the number of bits used by the TIMER
  0x510 equ tim.PRESCALER 	\ Timer prescaler register
  0x540 equ tim.CC[0] 		\ Capture/Compare register 0
  0x544 equ tim.CC[1] 		\ Capture/Compare register 1
  0x548 equ tim.CC[2] 		\ Capture/Compare register 2
  0x54C equ tim.CC[3] 		\ Capture/Compare register 3
  0xFFC equ tim.POWER

((
typedef struct {                                    /*!< TIMER Structure                                                       */
  __O  uint32_t  TASKS_START;                       /*!< Start Timer.                                                          */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop Timer.                                                           */
  __O  uint32_t  TASKS_COUNT;                       /*!< Increment Timer (In counter mode).                                    */
  __O  uint32_t  TASKS_CLEAR;                       /*!< Clear timer.                                                          */
  __O  uint32_t  TASKS_SHUTDOWN;                    /*!< Shutdown timer.                                                       */
  __I  uint32_t  RESERVED0[11];
  __O  uint32_t  TASKS_CAPTURE[4];                  /*!< Capture Timer value to CC[n] registers.                               */
  __I  uint32_t  RESERVED1[60];
  __IO uint32_t  EVENTS_COMPARE[4];                 /*!< Compare event on CC[n] match.                                         */
  __I  uint32_t  RESERVED2[44];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for Timer.                                                  */
  __I  uint32_t  RESERVED3[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED4[126];
  __IO uint32_t  MODE;                              /*!< Timer Mode selection.                                                 */
  __IO uint32_t  BITMODE;                           /*!< Sets timer behaviour.                                                 */
  __I  uint32_t  RESERVED5;
  __IO uint32_t  PRESCALER;                         /*!< 4-bit prescaler to source clock frequency (max value 9). Source
                                                         clock frequency is divided by 2^SCALE.                                */
  __I  uint32_t  RESERVED6[11];
  __IO uint32_t  CC[4];                             /*!< Capture/compare registers.                                            */
  __I  uint32_t  RESERVED7[683];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_TIMER_Type;
))


\ ===
\ RTC
\ ===

$4000:B000 equ _RTC0
$4001:1000 equ _RTC1
$4002:4000 equ _RTC2

\ Tasks
  0x000 equ rtc.START 		\ Start RTC COUNTER
  0x004 equ rtc.STOP 		\ Stop RTC COUNTER
  0x008 equ rtc.CLEAR 		\ Clear RTC COUNTER
  0x00C equ rtc.TRIGOVRFLW 	\ Set COUNTER to 0xFFFFF0
\ Events
  0x100 equ rtc.TICK 		\ Event on COUNTER increment
  0x104 equ rtc.OVRFLW 		\ Event on COUNTER overflow
  0x140 equ rtc.COMPARE[0] 	\ Compare event on CC[0] match
  0x144 equ rtc.COMPARE[1] 	\ Compare event on CC[1] match
  0x148 equ rtc.COMPARE[2] 	\ Compare event on CC[2] match
  0x14C equ rtc.COMPARE[3] 	\ Compare event on CC[3] match
\ Registers
  0x300 equ rtc.INTEN 		\ Enable or disable interrupt
  0x304 equ rtc.INTENSET 	\ Enable interrupt
  0x308 equ rtc.INTENCLR 	\ Disable interrupt
  0x340 equ rtc.EVTEN 		\ Enable or disable event routing
  0x344 equ rtc.EVTENSET 	\ Enable event routing
  0x348 equ rtc.EVTENCLR 	\ Disable event routing
  0x504 equ rtc.COUNTER 	\ Current COUNTER value
  0x508 equ rtc.PRESCALER 	\ 12 bit prescaler for COUNTER frequency
  				\ (32768/(PRESCALER+1)).Must be written
  				\ when RTC is stopped
  0x540 equ rtc.CC[0] 		\ Compare register 0
  0x544 equ rtc.CC[1] 		\ Compare register 1
  0x548 equ rtc.CC[2] 		\ Compare register 2
  0x54C equ rtc.CC[3] 		\ Compare register 3
  0xFFC equ rtc.POWER

((
typedef struct {                                    /*!< RTC Structure                                                         */
  __O  uint32_t  TASKS_START;                       /*!< Start RTC Counter.                                                    */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop RTC Counter.                                                     */
  __O  uint32_t  TASKS_CLEAR;                       /*!< Clear RTC Counter.                                                    */
  __O  uint32_t  TASKS_TRIGOVRFLW;                  /*!< Set COUNTER to 0xFFFFFFF0.                                            */
  __I  uint32_t  RESERVED0[60];
  __IO uint32_t  EVENTS_TICK;                       /*!< Event on COUNTER increment.                                           */
  __IO uint32_t  EVENTS_OVRFLW;                     /*!< Event on COUNTER overflow.                                            */
  __I  uint32_t  RESERVED1[14];
  __IO uint32_t  EVENTS_COMPARE[4];                 /*!< Compare event on CC[n] match.                                         */
  __I  uint32_t  RESERVED2[109];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[13];
  __IO uint32_t  EVTEN;                             /*!< Configures event enable routing to PPI for each RTC event.            */
  __IO uint32_t  EVTENSET;                          /*!< Enable events routing to PPI. The reading of this register gives
                                                         the value of EVTEN.                                                   */
  __IO uint32_t  EVTENCLR;                          /*!< Disable events routing to PPI. The reading of this register
                                                         gives the value of EVTEN.                                             */
  __I  uint32_t  RESERVED4[110];
  __I  uint32_t  COUNTER;                           /*!< Current COUNTER value.                                                */
  __IO uint32_t  PRESCALER;                         /*!< 12-bit prescaler for COUNTER frequency (32768/ (PRESCALER+1) ).
                                                         Must be written when RTC is STOPed.                                   */
  __I  uint32_t  RESERVED5[13];
  __IO uint32_t  CC[4];                             /*!< Capture/compare registers.                                            */
  __I  uint32_t  RESERVED6[683];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_RTC_Type;
))


\ ====
\ TEMP
\ ====

$4000:C000 equ _TEMP
\ Tasks
  0x000 equ tmp.START 		\ Start temperature measurement
  0x004 equ tmp.STOP  		\ Stop temperature measurement
\ Events
  0x100 equ tmp.DATARDY		\ Temperature measurement complete, data ready
\ Registers
  0x300 equ tmp.INTEN  		\ Enable or disable interrupt
  0x304 equ tmp.INTENSET 	\ Enable interrupt
  0x308 equ tmp.INTENCLR 	\ Disable interrupt
  0x508 equ tmp.TEMP 		\ Temperature in C

((
typedef struct {                                    /*!< TEMP Structure                                                        */
  __O  uint32_t  TASKS_START;                       /*!< Start temperature measurement.                                        */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop temperature measurement.                                         */
  __I  uint32_t  RESERVED0[62];
  __IO uint32_t  EVENTS_DATARDY;                    /*!< Temperature measurement complete, data ready event.                   */
  __I  uint32_t  RESERVED1[128];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED2[127];
  __I  int32_t   TEMP;                              /*!< Die temperature in degC, 2's complement format, 0.25 degC pecision.   */
  __I  uint32_t  RESERVED3[700];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_TEMP_Type;
))


\ ===
\ RNG
\ ===

$4000:D000 equ NRF_RNG_BASE
\ Tasks
  0x000 equ rng.START 		\ Task starting the random number generator
  0x004 equ rng.STOP 		\ Task stopping the random number generator
\ Events
  0x100 equ rng.VALRDY 		\ Event being generated for every new random number written to the VALUE register
\ Registers
  0x200 equ rng.SHORTS 		\ Shortcut register
  0x300 equ rng.INTEN 		\ Enable or disable interrupt
  0x304 equ rng.INTENSET 	\ Enable interrupt
  0x308 equ rng.INTENCLR 	\ Disable interrupt
  0x504 equ rng.CONFIG 		\ Configuration register
  0x508 equ rng.VALUE 		\ Output random number
  0xFFC equ rng.POWER
((
typedef struct {                                    /*!< RNG Structure                                                         */
  __O  uint32_t  TASKS_START;                       /*!< Start the random number generator.                                    */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop the random number generator.                                     */
  __I  uint32_t  RESERVED0[62];
  __IO uint32_t  EVENTS_VALRDY;                     /*!< New random number generated and written to VALUE register.            */
  __I  uint32_t  RESERVED1[63];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for the RNG.                                                */
  __I  uint32_t  RESERVED2[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register                                         */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register                                       */
  __I  uint32_t  RESERVED3[126];
  __IO uint32_t  CONFIG;                            /*!< Configuration register.                                               */
  __I  uint32_t  VALUE;                             /*!< RNG random number.                                                    */
  __I  uint32_t  RESERVED4[700];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_RNG_Type;
))


\ ===
\ ECB
\ ===

$4000:E000 equ _ECB
\ Tasks
  0x000 equ ecb.STARTECB 	\ Start ECB block encrypt
  0x004 equ ecb.STOPECB 	\ Abort a possible executing ECB operation
\ Events
  0x100 equ ecb.ENDECB 		\ ECB block encrypt complete
  0x104 equ ecb.ERRORECB 	\ ECB block encrypt aborted because of a STOPECB task or due to an error
\ Registers
  0x304 equ ecb.INTENSET 	\ Enable interrupt
  0x308 equ ecb.INTENCLR 	\ Disable interrupt
  0x504 equ ecb.ECBDATAPTR 	\ ECB block encrypt memory pointers
  0xFFC equ ecb.POWER
((
typedef struct {                                    /*!< ECB Structure                                                         */
  __O  uint32_t  TASKS_STARTECB;                    /*!< Start ECB block encrypt. If a crypto operation is running, this
                                                         will not initiate a new encryption and the ERRORECB event will
                                                          be triggered.                                                        */
  __O  uint32_t  TASKS_STOPECB;                     /*!< Stop current ECB encryption. If a crypto operation is running,
                                                         this will will trigger the ERRORECB event.                            */
  __I  uint32_t  RESERVED0[62];
  __IO uint32_t  EVENTS_ENDECB;                     /*!< ECB block encrypt complete.                                           */
  __IO uint32_t  EVENTS_ERRORECB;                   /*!< ECB block encrypt aborted due to a STOPECB task or due to an
                                                         error.                                                                */
  __I  uint32_t  RESERVED1[127];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED2[126];
  __IO uint32_t  ECBDATAPTR;                        /*!< ECB block encrypt memory pointer.                                     */
  __I  uint32_t  RESERVED3[701];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_ECB_Type;
))


\ ===
\ AAR
\ ===

$4000:F000 equ _AAR
\ Tasks
\ Events
\ Registers
  0x000 equ aar.START 		\ Start resolving addresses based on IRKs specified in the IRK data structure
  0x008 equ aar.STOP 		\ Stop resolving addresses
  0x100 equ aar.END 		\ Address resolution procedure complete
  0x104 equ aar.RESOLVED 	\ Address resolved
  0x108 equ aar.NOTRESOLVED 	\ Address not resolved
  0x304 equ aar.INTENSET 	\ Enable interrupt
  0x308 equ aar.INTENCLR 	\ Disable interrupt
  0x400 equ aar.STATUS 		\ Resolution status
  0x500 equ aar.ENABLE 		\ Enable AAR
  0x504 equ aar.NIRK  		\ Number of IRKs
  0x508 equ aar.IRKPTR 		\ Pointer to IRK data structure
  0x510 equ aar.ADDRPTR 	\ Pointer to the resolvable address
  0x514 equ aar.SCRATCHPTR 	\ Pointer to data area used for temporary storage
  0xFFC equ aar.POWER

((
typedef struct {                                    /*!< AAR Structure                                                         */
  __O  uint32_t  TASKS_START;                       /*!< Start resolving addresses based on IRKs specified in the IRK
                                                         data structure.                                                       */
  __I  uint32_t  RESERVED0;
  __O  uint32_t  TASKS_STOP;                        /*!< Stop resolving addresses.                                             */
  __I  uint32_t  RESERVED1[61];
  __IO uint32_t  EVENTS_END;                        /*!< Address resolution procedure completed.                               */
  __IO uint32_t  EVENTS_RESOLVED;                   /*!< Address resolved.                                                     */
  __IO uint32_t  EVENTS_NOTRESOLVED;                /*!< Address not resolved.                                                 */
  __I  uint32_t  RESERVED2[126];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[61];
  __I  uint32_t  STATUS;                            /*!< Resolution status.                                                    */
  __I  uint32_t  RESERVED4[63];
  __IO uint32_t  ENABLE;                            /*!< Enable AAR.                                                           */
  __IO uint32_t  NIRK;                              /*!< Number of Identity root Keys in the IRK data structure.               */
  __IO uint32_t  IRKPTR;                            /*!< Pointer to the IRK data structure.                                    */
  __I  uint32_t  RESERVED5;
  __IO uint32_t  ADDRPTR;                           /*!< Pointer to the resolvable address (6 bytes).                          */
  __IO uint32_t  SCRATCHPTR;                        /*!< Pointer to a scratch data area used for temporary storage during
                                                         resolution. A minimum of 3 bytes must be reserved.                    */
  __I  uint32_t  RESERVED6[697];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_AAR_Type;
))


\ ===
\ CCM
\ ===

$4000:F000 equ _CCM
\ Tasks
  0x000 equ ccm.KSGEN 		\ Start generation of key-stream. This operation will stop by itself when completed.
  0x004 equ ccm.CRYPT 		\ Start encryption/decryption. This operation will stop by itself when completed.
  0x008 equ ccm.STOP 		\ Stop encryption/decryption
\ Events
  0x100 equ ccm.ENDKSGEN 	\ Key-stream generation complete
  0x104 equ ccm.ENDCRYPT 	\ Encrypt/decrypt complete
  0x108 equ ccm.ERROR 		\ CCM error event
\ Registers
  0x200 equ ccm.SHORTS 		\ Shortcut register
  0x304 equ ccm.INTENSET 	\ Enable interrupt
  0x308 equ ccm.INTENCLR 	\ Disable interrupt
  0x400 equ ccm.MICSTATUS 	\ MIC check result
  0x500 equ ccm.ENABLE 		\ Enable
  0x504 equ ccm.MODE 		\ Operation mode
  0x508 equ ccm.CNFPTR 		\ Pointer to data structure holding AES key and NONCE vector
  0x50C equ ccm.INPTR 		\ Input pointer
  0x510 equ ccm.OUTPTR 		\ Output pointer
  0x514 equ ccm.SCRATCHPTR 	\ Pointer to data area used for temporary storage
  0xFFC equ ccm.POWER

((
typedef struct {                                    /*!< CCM Structure                                                         */
  __O  uint32_t  TASKS_KSGEN;                       /*!< Start generation of key-stream. This operation will stop by
                                                         itself when completed.                                                */
  __O  uint32_t  TASKS_CRYPT;                       /*!< Start encrypt/decrypt. This operation will stop by itself when
                                                         completed.                                                            */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop encrypt/decrypt.                                                 */
  __I  uint32_t  RESERVED0[61];
  __IO uint32_t  EVENTS_ENDKSGEN;                   /*!< Keystream generation completed.                                       */
  __IO uint32_t  EVENTS_ENDCRYPT;                   /*!< Encrypt/decrypt completed.                                            */
  __IO uint32_t  EVENTS_ERROR;                      /*!< Error happened.                                                       */
  __I  uint32_t  RESERVED1[61];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for the CCM.                                                */
  __I  uint32_t  RESERVED2[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[61];
  __I  uint32_t  MICSTATUS;                         /*!< CCM RX MIC check result.                                              */
  __I  uint32_t  RESERVED4[63];
  __IO uint32_t  ENABLE;                            /*!< CCM enable.                                                           */
  __IO uint32_t  MODE;                              /*!< Operation mode.                                                       */
  __IO uint32_t  CNFPTR;                            /*!< Pointer to a data structure holding AES key and NONCE vector.         */
  __IO uint32_t  INPTR;                             /*!< Pointer to the input packet.                                          */
  __IO uint32_t  OUTPTR;                            /*!< Pointer to the output packet.                                         */
  __IO uint32_t  SCRATCHPTR;                        /*!< Pointer to a scratch data area used for temporary storage during
                                                         resolution. A minimum of 43 bytes must be reserved.                   */
  __I  uint32_t  RESERVED5[697];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_CCM_Type;
))


\ ===
\ WDT
\ ===

$4001:0000 equ _WDT
\ Tasks
  0x000 equ wdt.START 		\ Start the watchdog
\ Events
  0x100 equ wdt.TIMEOUT 	\ Watchdog timeout
\ Registers
  0x304 equ wdt.INTENSET 	\ Enable interrupt
  0x308 equ wdt.INTENCLR 	\ Disable interrupt
  0x400 equ wdt.RUNSTATUS 	\ Run status
  0x404 equ wdt.REQSTATUS 	\ Request status
  0x504 equ wdt.CRV 		\ Counter reload value
  0x508 equ wdt.RREN 		\ Enable register for reload request registers
  0x50C equ wdt.CONFIG 		\ Configuration register
  0x600 equ wdt.RR[0] 		\ Reload request 0
  0x604 equ wdt.RR[1] 		\ Reload request 1
  0x608 equ wdt.RR[2] 		\ Reload request 2
  0x60C equ wdt.RR[3] 		\ Reload request 3
  0x610 equ wdt.RR[4] 		\ Reload request 4
  0x614 equ wdt.RR[5] 		\ Reload request 5
  0x618 equ wdt.RR[6] 		\ Reload request 6
  0x61C equ wdt.RR[7] 		\ Reload request 7
  0xFFC equ wdt.POWER
((
typedef struct {                                    /*!< WDT Structure                                                         */
  __O  uint32_t  TASKS_START;                       /*!< Start the watchdog.                                                   */
  __I  uint32_t  RESERVED0[63];
  __IO uint32_t  EVENTS_TIMEOUT;                    /*!< Watchdog timeout.                                                     */
  __I  uint32_t  RESERVED1[128];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED2[61];
  __I  uint32_t  RUNSTATUS;                         /*!< Watchdog running status.                                              */
  __I  uint32_t  REQSTATUS;                         /*!< Request status.                                                       */
  __I  uint32_t  RESERVED3[63];
  __IO uint32_t  CRV;                               /*!< Counter reload value in number of 32kiHz clock cycles.                */
  __IO uint32_t  RREN;                              /*!< Reload request enable.                                                */
  __IO uint32_t  CONFIG;                            /*!< Configuration register.                                               */
  __I  uint32_t  RESERVED4[60];
  __O  uint32_t  RR[8];                             /*!< Reload requests registers.                                            */
  __I  uint32_t  RESERVED5[631];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_WDT_Type;
))


\ ====
\ QDEC
\ ====

$4001:2000 equ _QDEC
\ Tasks
  0x000 equ qdc.START 		\ Task starting the quadrature decoder
  0x004 equ qdc.STOP 		\ Task stopping the quadrature decoder
  0x008 equ qdc.READCLRACC 	\ Read and clear ACC and ACCDBL
\ Events
  0x100 equ qdc.SAMPLERDY 	\ Event being generated for every new sample value written to the SAMPLE register
  0x104 equ qdc.REPORTRDY 	\ Non-null report ready
  0x108 equ qdc.ACCOF 		\ ACC or ACCDBL register overflow
\ Registers
  0x200 equ qdc.SHORTS 		\ Shortcut register
  0x300 equ qdc.INTEN 		\ Enable or disable interrupt
  0x304 equ qdc.INTENSET 	\ Enable interrupt
  0x308 equ qdc.INTENCLR 	\ Disable interrupt
  0x500 equ qdc.ENABLE 		\ Enable the quadrature decoder
  0x504 equ qdc.LEDPOL 		\ LED output pin polarity
  0x508 equ qdc.SAMPLEPER 	\ Sample period
  0x50C equ qdc.SAMPLE 		\ Motion sample value
  0x510 equ qdc.REPORTPER 	\ Number of samples to be taken before a REPORTRDY event is generated
  0x514 equ qdc.ACC 		\ Register accumulating the valid transitions
  0x518 equ qdc.ACCREAD  	\ Snapshot of the ACC register, updated by the READCLRACC task
  0x51C equ qdc.PSELLED 	\ GPIO pin number to be used as LED output
  0x520 equ qdc.PSELA 		\ GPIO pin number to be used as Phase A input
  0x524 equ qdc.PSELB 		\ GPIO pin number to be used as Phase B input
  0x528 equ qdc.DBFEN 		\ Enable input debounce filters
  0x540 equ qdc.LEDPRE 		\ Time period the LED is switched ON prior to sampling
  0x544 equ qdc.ACCDBL 		\ Register accumulating the number of detected double transitions
  0x548 equ qdc.ACCDBLREAD 	\ Snapshot of the ACCDBL, updated by the READCLRACC task
  0xFFC equ qdc.POWER
((
typedef struct {                                    /*!< QDEC Structure                                                        */
  __O  uint32_t  TASKS_START;                       /*!< Start the quadrature decoder.                                         */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop the quadrature decoder.                                          */
  __O  uint32_t  TASKS_READCLRACC;                  /*!< Transfers the content from ACC registers to ACCREAD registers,
                                                         and clears the ACC registers.                                         */
  __I  uint32_t  RESERVED0[61];
  __IO uint32_t  EVENTS_SAMPLERDY;                  /*!< A new sample is written to the sample register.                       */
  __IO uint32_t  EVENTS_REPORTRDY;                  /*!< REPORTPER number of samples accumulated in ACC register, and
                                                         ACC register different than zero.                                     */
  __IO uint32_t  EVENTS_ACCOF;                      /*!< ACC or ACCDBL register overflow.                                      */
  __I  uint32_t  RESERVED1[61];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for the QDEC.                                               */
  __I  uint32_t  RESERVED2[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[125];
  __IO uint32_t  ENABLE;                            /*!< Enable the QDEC.                                                      */
  __IO uint32_t  LEDPOL;                            /*!< LED output pin polarity.                                              */
  __IO uint32_t  SAMPLEPER;                         /*!< Sample period.                                                        */
  __I  int32_t   SAMPLE;                            /*!< Motion sample value.                                                  */
  __IO uint32_t  REPORTPER;                         /*!< Number of samples to generate an EVENT_REPORTRDY.                     */
  __I  int32_t   ACC;                               /*!< Accumulated valid transitions register.                               */
  __I  int32_t   ACCREAD;                           /*!< Snapshot of ACC register. Value generated by the TASKS_READCLEACC
                                                         task.                                                                 */
  __IO uint32_t  PSELLED;                           /*!< Pin select for LED output.                                            */
  __IO uint32_t  PSELA;                             /*!< Pin select for phase A input.                                         */
  __IO uint32_t  PSELB;                             /*!< Pin select for phase B input.                                         */
  __IO uint32_t  DBFEN;                             /*!< Enable debouncer input filters.                                       */
  __I  uint32_t  RESERVED4[5];
  __IO uint32_t  LEDPRE;                            /*!< Time LED is switched ON before the sample.                            */
  __I  uint32_t  ACCDBL;                            /*!< Accumulated double (error) transitions register.                      */
  __I  uint32_t  ACCDBLREAD;                        /*!< Snapshot of ACCDBL register. Value generated by the TASKS_READCLEACC
                                                         task.                                                                 */
  __I  uint32_t  RESERVED5[684];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_QDEC_Type;
))


\ ======
\ LPCOMP
\ ======

$4001:3000 equ _LPCOMP

\ Tasks
  0x000 equ lpc.START 		\ Start comparator
  0x004 equ lpc.STOP 		\ Stop comparator
  0x008 equ lpc.SAMPLE 		\ Sample comparator value
\ Events
  0x100 equ lpc.READY 		\ LPCOMP is ready and output is valid
  0x104 equ lpc.DOWN 		\ Downward crossing
  0x108 equ lpc.UP 		\ Upward crossing
  0x10C equ lpc.CROSS 		\ Downward or upward crossing
\ Registers
  0x200 equ lpc.SHORTS 		\ Shortcut register
  0x300 equ lpc.INTEN 		\ Enable or disable interrupt
  0x304 equ lpc.INTENSET 	\ Enable interrupt
  0x308 equ lpc.INTENCLR 	\ Disable interrupt
  0x400 equ lpc.RESULT 		\ Compare result
  0x500 equ lpc.ENABLE 		\ Enable LPCOMP
  0x504 equ lpc.PSEL 		\ Input pin select
  0x508 equ lpc.REFSEL 		\ Reference select
  0x50C equ lpc.EXTREFSEL 	\ External reference select
  0x520 equ lpc.ANADETECT 	\ Analog detect configuration
  0xFFC equ lpc.POWER
((
typedef struct {                                    /*!< LPCOMP Structure                                                      */
  __O  uint32_t  TASKS_START;                       /*!< Start the comparator.                                                 */
  __O  uint32_t  TASKS_STOP;                        /*!< Stop the comparator.                                                  */
  __O  uint32_t  TASKS_SAMPLE;                      /*!< Sample comparator value.                                              */
  __I  uint32_t  RESERVED0[61];
  __IO uint32_t  EVENTS_READY;                      /*!< LPCOMP is ready and output is valid.                                  */
  __IO uint32_t  EVENTS_DOWN;                       /*!< Input voltage crossed the threshold going down.                       */
  __IO uint32_t  EVENTS_UP;                         /*!< Input voltage crossed the threshold going up.                         */
  __IO uint32_t  EVENTS_CROSS;                      /*!< Input voltage crossed the threshold in any direction.                 */
  __I  uint32_t  RESERVED1[60];
  __IO uint32_t  SHORTS;                            /*!< Shortcuts for the LPCOMP.                                             */
  __I  uint32_t  RESERVED2[64];
  __IO uint32_t  INTENSET;                          /*!< Interrupt enable set register.                                        */
  __IO uint32_t  INTENCLR;                          /*!< Interrupt enable clear register.                                      */
  __I  uint32_t  RESERVED3[61];
  __I  uint32_t  RESULT;                            /*!< Result of last compare.                                               */
  __I  uint32_t  RESERVED4[63];
  __IO uint32_t  ENABLE;                            /*!< Enable the LPCOMP.                                                    */
  __IO uint32_t  PSEL;                              /*!< Input pin select.                                                     */
  __IO uint32_t  REFSEL;                            /*!< Reference select.                                                     */
  __IO uint32_t  EXTREFSEL;                         /*!< External reference select.                                            */
  __I  uint32_t  RESERVED5[4];
  __IO uint32_t  ANADETECT;                         /*!< Analog detect configuration.                                          */
  __I  uint32_t  RESERVED6[694];
  __IO uint32_t  POWER;                             /*!< Peripheral power control.                                             */
} NRF_LPCOMP_Type;
))


\ ===
\ SWI
\ ===

0x40014000 equ _SWI0 Software interrupt
0x40015000 equ _SWI1 Software interrupt
0x40016000 equ _SWI2 Software interrupt
0x40017000 equ _SWI3 Software interrupt
0x40018000 equ _SWI4 Software interrupt
0x40019000 equ _SWI5 Software interrupt


\ ====
\ NVMC
\ ====

$4001:E000 equ _NVMC
\ Registers
  0x400 equ nvm.READY 		\ Ready flag
  0x504 equ nvm.CONFIG 		\ Configuration register
  0x508 equ nvm.ERASEPAGE 	\ Register for erasing a page in Code area
  0x508 equ nvm.ERASEPCR1 	\ Register for erasing a page in Code region 1. Equivalent to ERASEPAGE.
  0x50C equ nvm.ERASEALL 	\ Register for erasing all non-volatile user memory
  0x510 equ nvm.ERASEPCR0 	\ Register for erasing a page in Code region 0
  0x514 equ nvm.ERASEUICR 	\ Register for erasing User Information Configuration Registers

((
typedef struct {                                    /*!< NVMC Structure                                                        */
  __I  uint32_t  RESERVED0[256];
  __I  uint32_t  READY;                             /*!< Ready flag.                                                           */
  __I  uint32_t  RESERVED1[64];
  __IO uint32_t  CONFIG;                            /*!< Configuration register.                                               */

  union {
    __IO uint32_t  ERASEPCR1;                       /*!< Register for erasing a non-protected non-volatile memory page.        */
    __IO uint32_t  ERASEPAGE;                       /*!< Register for erasing a non-protected non-volatile memory page.        */
  };
  __IO uint32_t  ERASEALL;                          /*!< Register for erasing all non-volatile user memory.                    */
  __IO uint32_t  ERASEPCR0;                         /*!< Register for erasing a protected non-volatile memory page.            */
  __IO uint32_t  ERASEUICR;                         /*!< Register for start erasing User Information Congfiguration Registers. */
} NRF_NVMC_Type;
))


\ ===
\ PPI
\ ===

$4001:F000 equ _PPI
\ Tasks
  8 equ /CHGTASK		\ size of CHG task area; 4 groups
    $000 equ ppi.CHGEN
    $004 equ ppi.CHGDIS
  0x000 equ ppi.CHGT[0]		\ channel group task 0
  0x008 equ ppi.CHGT[1]		\ channel group task 1
  0x010 equ ppi.CHGT[2]		\ channel group task 2
  0x018 equ ppi.CHGT[3]		\ channel group task 3
\ Registers
  0x500 equ ppi.CHEN 		\ Channel enable register
  0x504 equ ppi.CHENSET 	\ Channel enable set register
  0x508 equ ppi.CHENCLR 	\ Channel enable clear register
  8 equ /CHEP			\ size of a channel endpoint
    $000 equ ppi.CHEEP		\ offset of event endpoint
    $004 equ ppi.CHTEP		\ offset of task endpoint
  0x510 equ ppi.CHEP[0]		\ channel 0 endpoint
  0x800 equ ppi.CHG[0] 		\ Channel group 0
  0x804 equ ppi.CHG[1] 		\ Channel group 1
  0x808 equ ppi.CHG[2] 		\ Channel group 2
  0x80C equ ppi.CHG[3] 		\ Channel group 3


\ ====
\ FICR
\ ====

$1000:0000 equ _FICR
\ Registers
  0x010 equ fic.CODEPAGESIZE 	\ Code memory page size
  0x014 equ fic.CODESIZE 	\ Code memory size
  0x028 equ fic.CLENR0 		\ Length of Code region 0 in bytes Deprecated
  0x02C equ fic.PPFC 		\ Pre-programmed factory Code present Deprecated
  0x034 equ fic.NUMRAMBLOCK 	\ Number of individually controllable RAM blocks
  0x038 equ fic.SIZERAMBLOCKS 	\ RAM block size, in bytes
  0x038 equ fic.SIZERAMBLOCK[0] \ Size of RAM block 0, in bytes Deprecated
  0x03C equ fic.SIZERAMBLOCK[1] \ Size of RAM block 1, in bytes Deprecated
  0x040 equ fic.SIZERAMBLOCK[2] \ Size of RAM block 2, in bytes Deprecated
  0x044 equ fic.SIZERAMBLOCK[3] \ Size of RAM block 3, in bytes Deprecated
  0x05C equ fic.CONFIGID 	\ Configuration identifier
  0x060 equ fic.DEVICEID[0] 	\ Device identifier
  0x064 equ fic.DEVICEID[1] 	\ Device identifier
  0x080 equ fic.ER[0] 		\ Encryption Root, word 0
  0x084 equ fic.ER[1] 		\ Encryption Root, word 1
  0x088 equ fic.ER[2] 		\ Encryption Root, word 2
  0x08C equ fic.ER[3] 		\ Encryption Root, word 3
  0x090 equ fic.IR[0] 		\ Identity Root, word 0
  0x094 equ fic.IR[1] 		\ Identity Root, word 1
  0x098 equ fic.IR[2] 		\ Identity Root, word 2
  0x09C equ fic.IR[3] 		\ Identity Root, word 3
  0x0A0 equ fic.DEVICEADDRTYPE 	\ Device address type
  0x0A4 equ fic.DEVICEADDR[0] 	\ Device address 0
  0x0A8 equ fic.DEVICEADDR[1] 	\ Device address 1
  0x0AC equ fic.OVERRIDEEN 	\ Override enable
  0x0B0 equ fic.NRF_1MBIT[0] 	\ Override value for NRF_1MBIT mode
  0x0B4 equ fic.NRF_1MBIT[1] 	\ Override value for NRF_1MBIT mode
  0x0B8 equ fic.NRF_1MBIT[2] 	\ Override value for NRF_1MBIT mode
  0x0BC equ fic.NRF_1MBIT[3] 	\ Override value for NRF_1MBIT mode
  0x0C0 equ fic.NRF_1MBIT[4] 	\ Override value for NRF_1MBIT mode
  0x0EC equ fic.BLE_1MBIT[0] 	\ Override value for BLE_1MBIT mode
  0x0F0 equ fic.BLE_1MBIT[1] 	\ Override value for BLE_1MBIT mode
  0x0F4 equ fic.BLE_1MBIT[2] 	\ Override value for BLE_1MBIT mode
  0x0F8 equ fic.BLE_1MBIT[3] 	\ Override value for BLE_1MBIT mode
  0x0FC equ fic.BLE_1MBIT[4] 	\ Override value for BLE_1MBIT mode

((
typedef struct {                                    /*!< FICR Structure                                                        */
  __I  uint32_t  RESERVED0[4];
  __I  uint32_t  CODEPAGESIZE;                      /*!< Code memory page size in bytes.                                       */
  __I  uint32_t  CODESIZE;                          /*!< Code memory size in pages.                                            */
  __I  uint32_t  RESERVED1[4];
  __I  uint32_t  CLENR0;                            /*!< Length of code region 0 in bytes.                                     */
  __I  uint32_t  PPFC;                              /*!< Pre-programmed factory code present.                                  */
  __I  uint32_t  RESERVED2;
  __I  uint32_t  NUMRAMBLOCK;                       /*!< Number of individualy controllable RAM blocks.                        */

  union {
    __I  uint32_t  SIZERAMBLOCK[4];                 /*!< Deprecated array of size of RAM block in bytes. This name is
                                                         kept for backward compatinility purposes. Use SIZERAMBLOCKS
                                                          instead.                                                             */
    __I  uint32_t  SIZERAMBLOCKS;                   /*!< Size of RAM blocks in bytes.                                          */
  };
  __I  uint32_t  RESERVED3[5];
  __I  uint32_t  CONFIGID;                          /*!< Configuration identifier.                                             */
  __I  uint32_t  DEVICEID[2];                       /*!< Device identifier.                                                    */
  __I  uint32_t  RESERVED4[6];
  __I  uint32_t  ER[4];                             /*!< Encryption root.                                                      */
  __I  uint32_t  IR[4];                             /*!< Identity root.                                                        */
  __I  uint32_t  DEVICEADDRTYPE;                    /*!< Device address type.                                                  */
  __I  uint32_t  DEVICEADDR[2];                     /*!< Device address.                                                       */
  __I  uint32_t  OVERRIDEEN;                        /*!< Radio calibration override enable.                                    */
  __I  uint32_t  NRF_1MBIT[5];                      /*!< Override values for the OVERRIDEn registers in RADIO for NRF_1Mbit
                                                         mode.                                                                 */
  __I  uint32_t  RESERVED5[10];
  __I  uint32_t  BLE_1MBIT[5];                      /*!< Override values for the OVERRIDEn registers in RADIO for BLE_1Mbit
                                                         mode.                                                                 */
} NRF_FICR_Type;
))


\ ====
\ UICR
\ ====

$1000:1000 equ _UICR
  0x000 equ uic.CLENR0 		\ Length of code region 0
  0x004 equ uic.RBPCONF 	\ Read back protection configuration
  0x008 equ uic.XTALFREQ 	\ Reset value for XTALFREQ in CLOCK, see CLOCK chapter
  0x010 equ uic.FWID 		\ Firmware ID
  0x014 equ uic.BOOTLOADERADDR 	\ Bootloader address
  0x018 equ uic.NRFFW[1] 	\ Reserved for Nordic firmware design
((
  0x01C equ uic.NRFFW[2] 	\ Reserved for Nordic firmware design
  0x020 equ uic.NRFFW[3] 	\ Reserved for Nordic firmware design
  0x024 equ uic.NRFFW[4] 	\ Reserved for Nordic firmware design
  0x028 equ uic.NRFFW[5] 	\ Reserved for Nordic firmware design
  0x02C equ uic.NRFFW[6] 	\ Reserved for Nordic firmware design
  0x030 equ uic.NRFFW[7] 	\ Reserved for Nordic firmware design
  0x034 equ uic.NRFFW[8] 	\ Reserved for Nordic firmware design
  0x038 equ uic.NRFFW[9] 	\ Reserved for Nordic firmware design
  0x03C equ uic.NRFFW[10] 	\ Reserved for Nordic firmware design
  0x040 equ uic.NRFFW[11] 	\ Reserved for Nordic firmware design
  0x044 equ uic.NRFFW[12] 	\ Reserved for Nordic firmware design
  0x048 equ uic.NRFFW[13] 	\ Reserved for Nordic firmware design
  0x04C equ uic.NRFFW[14] 	\ Reserved for Nordic firmware design
  0x050 equ uic.NRFHW[0] 	\ Reserved for Nordic hardware design
  0x054 equ uic.NRFHW[1] 	\ Reserved for Nordic hardware design
  0x058 equ uic.NRFHW[2] 	\ Reserved for Nordic hardware design
  0x05C equ uic.NRFHW[3] 	\ Reserved for Nordic hardware design
  0x060 equ uic.NRFHW[4] 	\ Reserved for Nordic hardware design
  0x064 equ uic.NRFHW[5] 	\ Reserved for Nordic hardware design
  0x068 equ uic.NRFHW[6] 	\ Reserved for Nordic hardware design
  0x06C equ uic.NRFHW[7] 	\ Reserved for Nordic hardware design
  0x070 equ uic.NRFHW[8] 	\ Reserved for Nordic hardware design
  0x074 equ uic.NRFHW[9] 	\ Reserved for Nordic hardware design
  0x078 equ uic.NRFHW[10] 	\ Reserved for Nordic hardware design
  0x07C equ uic.NRFHW[11] 	\ Reserved for Nordic hardware design
))
  0x080 equ uic.CUSTOMER[0] 	\ Reserved for customer
((
  0x084 equ uic.CUSTOMER[1] 	\ Reserved for customer
  0x088 equ uic.CUSTOMER[2] 	\ Reserved for customer
  0x08C equ uic.CUSTOMER[3] 	\ Reserved for customer
  0x090 equ uic.CUSTOMER[4] 	\ Reserved for customer
  0x094 equ uic.CUSTOMER[5] 	\ Reserved for customer
  0x098 equ uic.CUSTOMER[6] 	\ Reserved for customer
  0x09C equ uic.CUSTOMER[7] 	\ Reserved for customer
  0x0A0 equ uic.CUSTOMER[8] 	\ Reserved for customer
  0x0A4 equ uic.CUSTOMER[9] 	\ Reserved for customer
  0x0A8 equ uic.CUSTOMER[10] 	\ Reserved for customer
  0x0AC equ uic.CUSTOMER[11] 	\ Reserved for customer
  0x0B0 equ uic.CUSTOMER[12] 	\ Reserved for customer
  0x0B4 equ uic.CUSTOMER[13] 	\ Reserved for customer
  0x0B8 equ uic.CUSTOMER[14] 	\ Reserved for customer
  0x0BC equ uic.CUSTOMER[15] 	\ Reserved for customer
  0x0C0 equ uic.CUSTOMER[16] 	\ Reserved for customer
  0x0C4 equ uic.CUSTOMER[17] 	\ Reserved for customer
  0x0C8 equ uic.CUSTOMER[18] 	\ Reserved for customer
  0x0CC equ uic.CUSTOMER[19] 	\ Reserved for customer
  0x0D0 equ uic.CUSTOMER[20] 	\ Reserved for customer
  0x0D4 equ uic.CUSTOMER[21] 	\ Reserved for customer
  0x0D8 equ uic.CUSTOMER[22] 	\ Reserved for customer
  0x0DC equ uic.CUSTOMER[23] 	\ Reserved for customer
  0x0E0 equ uic.CUSTOMER[24] 	\ Reserved for customer
  0x0E4 equ uic.CUSTOMER[25] 	\ Reserved for customer
  0x0E8 equ uic.CUSTOMER[26] 	\ Reserved for customer
  0x0EC equ uic.CUSTOMER[27] 	\ Reserved for customer
  0x0F0 equ uic.CUSTOMER[28] 	\ Reserved for customer
  0x0F4 equ uic.CUSTOMER[29] 	\ Reserved for customer
  0x0F8 equ uic.CUSTOMER[30] 	\ Reserved for customer
  0x0FC equ uic.CUSTOMER[31] 	\ Reserved for customer
)

((
typedef struct {                                    /*!< UICR Structure                                                        */
  __IO uint32_t  CLENR0;                            /*!< Length of code region 0.                                              */
  __IO uint32_t  RBPCONF;                           /*!< Readback protection configuration.                                    */
  __IO uint32_t  XTALFREQ;                          /*!< Reset value for CLOCK XTALFREQ register.                              */
  __I  uint32_t  RESERVED0;
  __I  uint32_t  FWID;                              /*!< Firmware ID.                                                          */

  union {
    __IO uint32_t  NRFFW[15];                       /*!< Reserved for Nordic firmware design.                                  */
    __IO uint32_t  BOOTLOADERADDR;                  /*!< Bootloader start address.                                             */
  };
  __IO uint32_t  NRFHW[12];                         /*!< Reserved for Nordic hardware design.                                  */
  __IO uint32_t  CUSTOMER[32];                      /*!< Reserved for customer.                                                */
} NRF_UICR_Type;
))


\ ====
\ GPIO
\ ====

$5000:0000 equ _GPIO
  0x504 equ gp.OUT 		\ Write GPIO port
  0x508 equ gp.OUTSET 		\ Set individual bits in GPIO port
  0x50C equ gp.OUTCLR 		\ Clear individual bits in GPIO port
  0x510 equ gp.IN 		\ Read GPIO port
  0x514 equ gp.DIR 		\ Direction of GPIO pins
  0x518 equ gp.DIRSET 		\ DIR set register
  0x51C equ gp.DIRCLR 		\ DIR clear register
  0x700 equ gp.PIN_CNF[0]  	\ Configuration of GPIO pins
  0x704 equ gp.PIN_CNF[1]  	\ Configuration of GPIO pins
  0x708 equ gp.PIN_CNF[2]  	\ Configuration of GPIO pins
  0x70C equ gp.PIN_CNF[3]  	\ Configuration of GPIO pins
  0x710 equ gp.PIN_CNF[4]  	\ Configuration of GPIO pins
  0x714 equ gp.PIN_CNF[5]  	\ Configuration of GPIO pins
  0x718 equ gp.PIN_CNF[6]  	\ Configuration of GPIO pins
  0x71C equ gp.PIN_CNF[7]  	\ Configuration of GPIO pins
  0x720 equ gp.PIN_CNF[8]  	\ Configuration of GPIO pins
  0x724 equ gp.PIN_CNF[9]  	\ Configuration of GPIO pins
  0x728 equ gp.PIN_CNF[10] 	\ Configuration of GPIO pins
  0x72C equ gp.PIN_CNF[11] 	\ Configuration of GPIO pins
  0x730 equ gp.PIN_CNF[12] 	\ Configuration of GPIO pins
  0x734 equ gp.PIN_CNF[13] 	\ Configuration of GPIO pins
  0x738 equ gp.PIN_CNF[14] 	\ Configuration of GPIO pins
  0x73C equ gp.PIN_CNF[15] 	\ Configuration of GPIO pins
  0x740 equ gp.PIN_CNF[16] 	\ Configuration of GPIO pins
  0x744 equ gp.PIN_CNF[17] 	\ Configuration of GPIO pins
  0x748 equ gp.PIN_CNF[18] 	\ Configuration of GPIO pins
  0x74C equ gp.PIN_CNF[19] 	\ Configuration of GPIO pins
  0x750 equ gp.PIN_CNF[20] 	\ Configuration of GPIO pins
  0x754 equ gp.PIN_CNF[21] 	\ Configuration of GPIO pins
  0x758 equ gp.PIN_CNF[22] 	\ Configuration of GPIO pins
  0x75C equ gp.PIN_CNF[23] 	\ Configuration of GPIO pins
  0x760 equ gp.PIN_CNF[24] 	\ Configuration of GPIO pins
  0x764 equ gp.PIN_CNF[25] 	\ Configuration of GPIO pins
  0x768 equ gp.PIN_CNF[26] 	\ Configuration of GPIO pins
  0x76C equ gp.PIN_CNF[27] 	\ Configuration of GPIO pins
  0x770 equ gp.PIN_CNF[28] 	\ Configuration of GPIO pins
  0x774 equ gp.PIN_CNF[29] 	\ Configuration of GPIO pins
  0x778 equ gp.PIN_CNF[30] 	\ Configuration of GPIO pins
  0x77C equ gp.PIN_CNF[31] 	\ Configuration of GPIO pins
((
typedef struct {                                    /*!< GPIO Structure                                                        */
  __I  uint32_t  RESERVED0[321];
  __IO uint32_t  OUT;                               /*!< Write GPIO port.                                                      */
  __IO uint32_t  OUTSET;                            /*!< Set individual bits in GPIO port.                                     */
  __IO uint32_t  OUTCLR;                            /*!< Clear individual bits in GPIO port.                                   */
  __I  uint32_t  IN;                                /*!< Read GPIO port.                                                       */
  __IO uint32_t  DIR;                               /*!< Direction of GPIO pins.                                               */
  __IO uint32_t  DIRSET;                            /*!< DIR set register.                                                     */
  __IO uint32_t  DIRCLR;                            /*!< DIR clear register.                                                   */
  __I  uint32_t  RESERVED1[120];
  __IO uint32_t  PIN_CNF[32];                       /*!< Configuration of GPIO pins.                                           */
} NRF_GPIO_Type;
))


\ ======
\ *> ###
\ ======

decimal

