\ ARM/Cortex disassembler for MPE v7 cross compilers

((
File: XDasmCortex.fth

Copyright (c) 1998, 1999, 2004, 2006, 2008, 2009, 2010, 2011, 2014, 2015
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: tech-support@mpeforth.com

Usage:

DISASM/al       \ addr len --
  Disassemble from addr for len bytes. NEXT is displayed as individual
  instructions.

disasm/ft       \ from to --
  Disassemble from address FROM to address TO. The NEXT macro is displayed
  as a NEXT, macro.

disasm/f        \ addr --
  Disassemble from address FROM. The NEXT, macro is displayed as a NEXT,
  macro. The display stops when the first NEXT, macro is encountered with
  no outstanding forward branch, or if the forward branch is over 256 bytes.

dasm            \ -- ; disassemble word; DASM <word>
  Disassemble from the cfa of <word>. The NEXT, macro is displayed as a NEXT,
  macro. The display stops when the first NEXT, macro is encountered with
  no outstanding forward branch, or if the forward branch is over 256 bytes.

All the utility words can be found in the xDasmCortex vocabulary.

For encoding details see ARM documents DDI0403D for Cortex-M and
DDI0406C for ARM32 and Cortex-A/R.

Change History
==============
20150617 SFP016 Added some hint instructions.
20140729 SFP014 Added BLX to ARM disassembly.
20111214 SFP013 Added Cortex-M4 instructions, ARMv7E-M
20100406 SFP012 Fixed CBZ, CBNZ.
20091220 SFP011 Cortex-M3 conversion.
20080225 SFP010 Improved NEXT detection.
20061130 SFP009 Added Thumb-1 decoder.
20040218 SFP008	Showed number of instructions disassembled.
SFP007	21 June 2000	Added address display to LDR rx, [ pc, # n ]
SFP006	1 Feb 2000	Added more high level decompilation
SFP005	25 Jan 2000	Made ?NEXT work for code and colon definitions
SFP004  11 July 1999    Enhanced branch target display to use symbol table
                        Added length display
SFP003  30 July 1998    Modified XDASM to find labels as well as words
SFP002  7 July 1998     H8/300H version
SFP001  29 June 1998    Converted for cross compiler
))


\ turn off FP to avoid problems with binary conversion

integers

\ Disassembler vocabulary for hiding

only FORTH  also c-c DEFINITIONS  DECIMAL

VOCABULARY xDasmCortex

also xDasmCortex DEFINITIONS


\ ***************
\ Auxiliary words
\ ***************

: digits        \ d n -- d' ; add n digits to the output stream
  0
  ?do  #  loop
;

: <C            \ char --- n ; sign extend byte to 32/64bits
  dup $80 and negate or
;

: <w            \ w -- n ; sign extend word to 32/64 bits
  dup $8000 and negate or
;

: u<=           \ u1 u2 -- flag
  u> not
;

: u>=           \ u1 u2 -- flag
  u< not
;

: (search)   { c-addr1 u1 c-addr2 u2 -- c-addr3 u3 flag }
  c-addr1 u1 u2 - 1+ bounds     \ Search for first char of $2 in $1.  Note that
  ?do                           \ we only check LEN($1)-LEN($2)+1 chars of $1.
    i c@  c-addr2 c@  =         \ Compare next char of $1 with first char of $2.
    if
      i c-addr2 u2 s=           \ Is the whole of $2 present?
      if
        i u1 i c-addr1 - - true \ Yes - build return stack parameters...
        unloop exit             \ ...and bail out.
      then
    then
  loop
  c-addr1 u1 false            \ End of loop and first char not found - no match.
;

: ANSSEARCH     ( c-addr1 u1 c-addr2 u2  --  c-addr3 u3 flag  )
  2 pick over <         \ Is $1 shorter than $2?
  if
    2drop false exit    \ Yes - $2 *can't* be in $1.
  then
  dup 0 <=              \ Is $2 zero length?
  if
    2drop true exit     \ Yes - we have found it at the start of $1.
  then
  2 pick 0 <=           \ Is $1 zero length?
  if
    2drop false exit    \ Yes - string not found.
  then
  (search)
;

\ *************************
\ DISASSEMBLER main portion - target address handling
\ *************************

VARIABLE (DisAddr)	\ current disassembly address
VARIABLE ToDisAddr      \ to address in from to words
variable StartAddr      \ address of start of opcode
variable branch-target  \ highest branch target address
variable next-found     \ true when NEXT found
variable next-addr	\ set when NEXT found
variable <itcount>	\ set by IT instruction to #ins
			\ # of instructions in complete IT block

: DisAddr       \ -- addr ; current disassembly address
   (DisAddr) @ ;

: SET-DisAddr   \ addr -- ; set current disassembly addr to addr
   (DisAddr) ! ;

: +DisAddr      \ n -- ; increment current disassembly address by n
   (DisAddr) +! ;

: >branch-target        \ addr -- ; insert if higher than current value
  dup branch-target @ u>
  if    branch-target !
  else  drop
  endif
;


\ **********************
\ opcode stream handling
\ **********************

#32 constant /ByteStream        \ -- n ; maximum number of bytes to buffer

/ByteStream buffer: ByteStream  \ -- addr ; buffer for stored opcodes

variable ^Stream        \ -- addr ; holds next free slot in byte stream
  ByteStream ^Stream !

: byte>stream   \ b -- ; add byte to stream
  ^Stream @ c!  1 ^Stream +!  ;

: INIT-VARS     \ --
  ByteStream /ByteStream erase
  ByteStream ^Stream !
;

: .ByteStream   \ -- ; dump the contents of the opcode stream
  ^Stream @ ByteStream
  ?do  i c@ .byte  loop
;

: .AsciiStream   \ -- ; dump the ascii contents of the opcode stream
  ^Stream @ ByteStream
  ?do  i c@ .ascii  loop
;


\ ******************
\ DISASSEMBLY OUTPUT
\ ******************

#29 CONSTANT DisOpCol    \ starting column for opcode

#128 Buffer: (DisLine$) \ line buffer for disassembly
VARIABLE DisPos         \ output position for opcode info

: DisLine$      \ -- addr cnt
  (DisLine$) 80 ;

: >DISLINE      \ adr cnt -- ; Add opcode information to disassembly line buffer.
  DisLine$ DisPos @ /STRING
  ROT MIN DUP
  DisPos  +!                           \ increment print pos in dis buffer
  CMOVE ;

: $>dis         \ string --
  count >disline  ;

: num>DIS      \ long --
  0 <# # #s #> >disline  ;

: $num>dis	\ long --
  c" $" $>dis  num>dis  ;

: #num>dis	\ long --
  c" #" $>dis  num>dis  ;

: dec>dis	\ long --
\ Display u unsigned as a decimal.
  base @ >r  decimal  num>dis  r> base !  ;

: hex>dis	\ long --
  base @ >r  hex  num>dis  r> base !  ;

: snum>DIS     \ long --
  s>d tuck  dabs <# # #s rot sign #> >disline  ;

variable char-holder

: CHAR>DIS      \ char --
   char-holder c!
   char-holder 1 >DISLINE  ;

: [>DIS         ( --- )
   C" [ " $>dis  ;

: ]>DIS         ( --- )
   C"  ]" $>dis  ;

((
: :>DIS         ( --- )
   ASCII : CHAR>DIS ;

: +>DIS         ( --- )
   ASCII + CHAR>DIS ;

: ->DIS         ( --- )
   ASCII - CHAR>DIS ;
))

: bl>dis        \ --
  bl char>dis  ;

: ">dis        \ --
  [char] " char>dis  ;

: ,>DIS         \ --
  C" , " $>dis  ;

: #>dis         \ --
  C" # $" $>dis  ;

: DISINIT       \ -- ; Initialize disassembly output buffer to current
                \ address and output positions for dump and opcode.
  DisLine$ BLANK                        \ initialise o/p buffer
  <itcount> @ 0<> 2 and DisPos !	\ no characters in buffer
  init-vars
  DisAddr StartAddr !			\ instruction start address
;

: disclose      \ -- ; add finishing text
; immediate



\ *************************************
\ Fetching opcodes from the code stream
\ *************************************

: @BYTE         \ -- byte
  DisAddr C@c(t)
  dup byte>stream
  1 +DisAddr  ;

: @WORD         \ -- word
  @byte @byte
\  little-endian? if  swap  endif
  [ also sections.voc ] 2bytes>word [ previous ]  ;

: @24b          \ -- 00aabbcc
  big-endian? if
    0 @byte @byte @byte
  else
    @byte @byte @byte 0
  endif
  [ also sections.voc ] 4bytes>long [ previous ]  ;

: @LONG         \ -- long
  @byte @byte @byte @byte
  [ also sections.voc ] 4bytes>long [ previous ]  ;


\ *****************
\ Immediate display
\ *****************

: .imm          \ l --
  #>dis  num>dis  ;

: .imms         \ l --
  #>dis  snum>dis  ;


\ **************
\ Opcode display
\ **************

: s+    \ dest src - dest'
  count rot 2dup +
  >r swap move r>  ;

create cond     ", aaaa"
create opcode   ", bbbb"
create sbit     ", c"
create abit     ", d"
create ubit     ", e"
create rbit     ", f"
create bbit     ", g"
create wbit     ", h"
create pbit     ", i"
create lbit     ", j"
create nbit     ", k"
create Rn       ", llll"
create Rd       ", mmmm"
create Rs       ", nnnn"
create Rm       ", oooo"
create hbit     ", p"
create 8bit-immed+rot   ", qqqqqqqqqqqq"
create 12bit-immed      ", rrrrrrrrrrrr"
create shift    ", ss"
create shift-immed      ", ttttt"
create RdHi     ", uuuu"
create RdLo     ", vvvv"
create reg-list ", wwwwwwwwwwwwwwww"
create swi-number       ", xxxxxxxxxxxxxxxxxxxxxxxx"
create mask     ", yyyy"
create 24bit-offset ", zzzzzzzzzzzzzzzzzzzzzzzz"
create 4bit-SBZ ", 0000"
create 7bit-SBZ ", 0000000"
create SBO      ", 1111"
create 12bit-SBZ ", 000000000000"
\ 2 reserved for undefined instruction
create Hi-offset        ", 3333"
create Lo-offset        ", 4444"

: +cond         \ addr - addr'
  cond s+  ;

: +opcode       \ addr - addr'
  opcode s+  ;

: +sbit         \ addr - addr'
  sbit s+  ;

: +abit         \ addr - addr'
  abit s+  ;

: +ubit         \ addr - addr'
  ubit s+  ;

: +rbit         \ addr - addr'
  rbit s+  ;

: +bbit         \ addr - addr'
  bbit s+  ;

: +wbit         \ addr - addr'
  wbit s+  ;

: +pbit         \ addr - addr'
  pbit s+  ;

: +lbit         \ addr - addr'
  lbit s+  ;

: +nbit         \ addr - addr'
  nbit s+  ;

: +hbit         \ addr - addr'
  hbit s+  ;

: +Rn           \ addr - addr'
  Rn s+  ;

: +Rd           \ addr - addr'
  Rd s+  ;

: +Rs           \ addr - addr'
  Rs s+  ;

: +Rm           \ addr - addr'
  Rm s+  ;

: +8bit-immed+rot       \ addr - addr'
  8bit-immed+rot s+  ;

: +12bit-immed  \ addr - addr'
  12bit-immed s+  ;

: +shift        \ addr - addr'
  shift s+  ;

: +shift-immed  \ addr - addr'
  shift-immed s+  ;

: +RdHi         \ addr - addr'
  RdHi s+  ;

: +RdLo         \ addr - addr'
  RdLo s+  ;

: +reg-list     \ addr - addr'
  reg-list s+  ;

: +SBO          \ addr - addr'
  SBO s+  ;

: +mask         \ addr - addr'
  mask s+  ;

: +4bit-SBZ     \ addr - addr'
  4bit-SBZ s+  ;

: +7bit-SBZ     \ addr - addr'
  7bit-SBZ s+  ;

: +12bit-SBZ    \ addr - addr'
  12bit-SBZ s+  ;

: +24bit-offset \ addr - addr'
  24bit-offset s+  ;

: +swi-number   \ addr - addr'
  swi-number s+  ;

: +Hi-offset    \ addr - addr'
  Hi-offset s+  ;

: +Lo-offset    \ addr - addr'
  Lo-offset s+  ;

variable curr-instr
0 value T2insbase	\ -- addr
\ Base (start) address of the T2 instruction.

32 constant i-size

i-size buffer: i-buff

struct igroup-struct
  i-size field encoding
  cell   field mask-out
  cell   field ibits
end-struct

variable mask-out-temp
variable ibits-temp

: make-masks
  0 mask-out-temp !
  0 ibits-temp !
  i-buff i-size bounds
  ?do
    case i c@
      [char] 0 of endof
      [char] 1 of
                1 i-size i i-buff - - 1- lshift
                ibits-temp @ or
                ibits-temp !
               endof
      drop 1 i-size i i-buff - - 1- lshift
      mask-out-temp @ or
      mask-out-temp !

    end-case
  loop
  mask-out-temp @ invert ,
  ibits-temp @ ,
;

: igroup
  create
    i-buff - i-size <>
    if
      true abort" group def. not long enough "
    then
    i-buff here i-size
    dup allot
    move
    make-masks
  does>
    dup mask-out @
    curr-instr @ and
    over ibits @ =
    dup 0=
    if  nip  then
;

i-buff +cond C" 001" s+ +opcode +sbit +Rn +Rd +8bit-immed+rot igroup group0
i-buff +cond C" 000" s+ +opcode +sbit +Rn +Rd +shift-immed +shift C" 0" s+ +Rm igroup group1
i-buff +cond C" 000" s+ +opcode +sbit +Rn +Rd +Rs C" 0" s+ +shift C" 1" s+ +Rm igroup group2
i-buff +cond C" 000000" s+ +abit +sbit +Rd +Rn +Rs C" 1001" s+ +Rm igroup group3
i-buff +cond C" 00001" s+ +ubit +abit +sbit +RdHi +RdLo +Rs C" 1001" s+ +Rm igroup group4
i-buff +cond C" 00010" s+ +rbit C" 00" s+ +SBO +Rd +12bit-SBZ igroup group5
i-buff +cond C" 00110" s+ +rbit C" 10" s+ +mask +SBO +8bit-immed+rot igroup group6
i-buff +cond C" 00010" s+ +rbit C" 10" s+ +mask +SBO +7bit-SBZ C" 0" s+ +Rm igroup group7
i-buff +cond C" 00010010" s+ +SBO +SBO +SBO C" 0001" s+ +Rm igroup group8
i-buff +cond C" 00010010" s+ +SBO +SBO +SBO C" 0011" s+ +Rm igroup group8a	\ SFP014
i-buff +cond C" 010" s+ +pbit +ubit +bbit +wbit +lbit +Rn +Rd +12bit-immed igroup group9
i-buff +cond C" 011" s+ +pbit +ubit +bbit +wbit +lbit +Rn +Rd +shift-immed +shift C" 0" s+ +Rm igroup group10
i-buff +cond C" 000" s+ +pbit +ubit C" 1" s+ +wbit +lbit +Rn +Rd +Hi-offset C" 1" s+ +sbit +hbit C" 1" s+ +Lo-offset igroup group11
i-buff +cond C" 000" s+ +pbit +ubit C" 0" s+ +wbit +lbit +Rn +Rd +4bit-SBZ C" 1" s+ +sbit +hbit C" 1" s+ +Rm igroup group12
i-buff +cond C" 00010" s+ +bbit C" 00" s+ +Rn +Rd +4bit-SBZ C" 1001" s+ +Rm igroup group13
i-buff +cond C" 100" s+ +pbit +ubit +sbit +wbit +lbit +Rn +reg-list igroup group14
i-buff +cond C" 1110222222222222222222202222" s+ igroup group15
i-buff +cond C" 1110222222222222222222212222" s+ igroup group16
i-buff +cond C" 1102222222222222222222222222" s+ igroup group17
i-buff C" 1111101" s+ +lbit +24bit-offset igroup group18a	\ SFP014
i-buff +cond C" 101" s+ +lbit +24bit-offset igroup group18
i-buff +cond C" 1111" s+ +swi-number igroup group19
i-buff +cond C" 0112222222222222222222212222" s+ igroup group20
i-buff +cond c" 00010110" s+ +SBO +Rd +SBO c" 0001" s+ +Rm igroup groupCLZ
i-buff C" 111101010111xxxxxxxxxxxxxxxxxxxx" s+ igroup groupHints1

create one-buff
  ", 11111111111111111111111111111111"

: match \ group pattern -- group value ; get value from instruction
  >r dup i-size r@ count anssearch
  if
    nip r@ count nip -
    base @ binary
    0 0 one-buff 1+ r> count nip >number 2drop drop
    swap base !
    over lshift
    curr-instr @ and
    swap rshift
  else
    true abort" Can't find expected instruction field"
  then
;

create cond-codes       \ -- addr
  ", .eq .ne .cs .cc .mi .pl .vs .vc .hi .ls .ge .lt .gt .le .al .nv "

: .condCode	\ cc --
  dup $0E <> if				\ ignore .AL
    $0F and 4* cond-codes + 1+ 4 >disline
  else
    drop
  then
;

: .cond         \ group -- group ; display condition code
  cond match .condCode
;

create data-opcodes     \ -- addr
  ", and eor sub rsb add adc sbc rsc tst teq cmp cmn orr mov bic mvn "
\ Arm 0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F

: .opcode       \ group -- group ; data processing opcode name
  opcode match
  $0F and 4* data-opcodes + 1+  4 >disline
;

: .sbit         \ group -- group ; display status bit
  sbit match
  if
    C" .s " $>dis
  then
;

: .rbit         \ group -- group ; display correct status register
  rbit match
  if
    C" spsr"
  else
    C" cpsr"
  then
  $>dis
;

: .lbit         \ group -- group ; display load/store type
  lbit match
  if
    C" ld"
  else
    C" st"
  then
  $>dis
;

: .bbit         \ group -- group ; display byte part of opcode
  bbit match
  if
    C" b "
  else
    C"  "
  then
  $>dis
;

: .wbit         \ group -- group ; display writeback !
  wbit match
  if C"  !" $>dis then
;

create regs     \ -- addr
  ", r0  r1  r2  r3  r4  r5  r6  r7  r8  r9  r10 r11 r12 SP  LR  PC  "

: .reg          \ reg# --
  $0F and 4* regs + 1+ 4 -TRAILING >disline
;

: .Rn
  Rn match .reg
;

: .Rd
  Rd match .reg
;

: .Rs
  Rs match .reg
;

: .Rm
  Rm match .reg
;

: .RdHi
  RdHi match .reg
;

: .RdLo
  RdLo match .reg
;

variable start-reg
variable reg-seq
variable printed-reg?

: .reg-list     \ group -- group
  -1 start-reg !
  -1 reg-seq !
  false printed-reg? !
  reg-list match
  $10 0 do
    dup $01 and                 \ Register to be printed?
    if                          \ YES
      reg-seq @ 0<              \   are we doing a sequence?
      if                        \   NO
        printed-reg? @          \     have we printed a register yet?
        if                      \     YES
          ,>dis                 \       print a comma between registers
        else                    \     NO
          true printed-reg? !   \       print nothing, but update the flag
        then                    \     ENDIF
        i .reg                  \     print the register
        i start-reg !           \     mark as start of sequence
      else                      \   YES
        i $0F =                 \     is this the last register, r15?
        if                      \     YES
          C" -" $>dis i .reg    \       force the end of the sequence
        then                    \     ENDIF
      then                      \   ENDIF
      i reg-seq !               \   Indicate possibly doing a sequence now
    else                        \ NO
      reg-seq @ 0 >=            \   were we doing a sequence?
      if                        \   YES
        reg-seq @ start-reg @ <>        \     Was the sequence more than one reg?
        if                      \     YES
          C" -" $>dis           \       display end of sequence
          reg-seq @ .reg
        then                    \     ENDIF
        -1 reg-seq !            \     reset sequence var
      then                      \   ENDIF
      -1 start-reg !            \   reset start of sequence var
    then                        \ ENDIF
    1 rshift
  loop
  drop
;

: .8bit-immed+rot
  8bit-immed+rot match
  dup $FF and swap $F00 and 8 rshift
  0 ?do
    2 ror
  loop
  .imm
;

create shift-codes
  ", lsl lsr asr ror "

: .shift        \ group -- group
  shift match
  $03 and 4* shift-codes + 1+ 4 >disline
;

: .shift-immed  \ group -- group
  shift-immed match
  swap shift match rot tuck
  0=
  if
    $03 =
    if
      C" rrx" $>dis
    then
    drop
  else
    $03 and 4* shift-codes + 1+ 4 >disline
    .imm
  then
;

: .mask
  mask match
  dup 1 and if  C" _c" $>dis  endif
  dup 2 and if  C" _x" $>dis  endif
  dup 4 and if  C" _s" $>dis  endif
      8 and if  C" _f" $>dis  endif
;

: .data-proc-regs       \ group -- group
  opcode match
  case
    dup $08 $0B within? ?of drop .Rn ,>dis endof        \ tst teq cmp cmn
    $0D                  of .sbit .Rd ,>dis endof       \ mov
    $0F                  of .sbit .Rd ,>dis endof       \ mvn
    drop .sbit .Rd ,>dis .Rn ,>dis
  end-case
;

: .name(s)>dis  \ symbol --
\ Output a name string to the disassembly line
  >name(s)
  dup  1+ swap c@(s) #31 and
  bounds ?do
    i c@(s) char>dis
  loop
;

\ SFP006...
: is-xt?		\ xt(t) symbol -- flag ; true if xt matches symbol
  dup resolved? if
    @(s) =
  else
    2drop 0
  endif
;

#256 buffer: StepBuff	\ -- addr

: $stepOver		\ -- ; display and step over string at DISADDR
  DisAddr StepBuff  over c@c(t) 1+ 0 do	\ collect string
    over i + c@c(t)  over i + c!
  loop
  2drop
  bl>dis ">dis  StepBuff $>dis  ">dis	\ display string
  Disaddr count(t) + aligned(t)		\ step over string
  (DisAddr) !
;

: z$stepOver		\ -- ; display and step over string at DISADDR
  DisAddr StepBuff  over c@c(t) 1+ 0 do	\ collect string
    over i + c@c(t)  over i + c!
  loop
  2drop
  bl>dis ">dis  StepBuff $>dis  ">dis	\ display string
  Disaddr count(t) + 1+ aligned(t)	\ step over string
  (DisAddr) !
;

: .dest		\ -- ; display inline data as branch target SFP008
  bl>dis  @long dup >branch-target num>dis
;

: .frame	\ -- ; display local frame data
  C"  #args = " $>dis  DisAddr w@c(t) num>dis
  C" , #locals = " $>dis  DisAddr 2+ w@c(t) num>dis  C"  (cells)" $>dis
  cell(t) (DisAddr) +!
;

: .lit		\ -- ; display inline data
  bl>dis  @long num>dis
;

: .xt		\ xt(t) --
  bl>dis
  dup xt>symbol ?dup if
    nip  .name(s)>dis
  else
    num>dis
  endif
;

: tabbed	\ -- ; step to next disassembler tab
  #8 Dispos @ #7 and - 0		\ to next tab stop
  ?do  bl>dis  loop
;

: ShowBranchTarget      \ abs24 -- ; SFP007
\ Show Target name of a branch address where possible.
  dup xt>symbol ?dup if
    tabbed .name(s)>dis
  endif

\ process special cases such as stepping over strings
  case
    dup target-sym LIT      is-xt? ?of  drop  .lit  endof
    dup target-sym branch   is-xt? ?of  drop  .dest  endof
    dup target-sym ?branch  is-xt? ?of  drop  .dest  endof
    dup target-sym (S")     is-xt? ?of  drop  $stepOver  endof
    dup target-sym (C")     is-xt? ?of  drop  $stepOver  endof
    dup target-sym (Z")     is-xt? ?of  drop  z$stepOver  endof
    dup target-sym (.")     is-xt? ?of  drop  $stepOver  endof
    dup target-sym (ABORT") is-xt? ?of  drop  $stepOver  endof
    dup target-sym (DO)     is-xt? ?of  drop  .dest  endof
    dup target-sym (?DO)    is-xt? ?of  drop  .dest  endof
\   dup target-sym VAL      is-xt? ?of  drop  .lit  endof
    dup target-sym VAL@     is-xt? ?of  drop  .lit  endof
    dup target-sym VAL!     is-xt? ?of  drop  .lit  endof
    dup target-sym MAKELVS  is-xt? ?of  drop  .frame  endof
    dup target-sym LV       is-xt? ?of  drop  .lit  endof
    dup target-sym LV@      is-xt? ?of  drop  .lit  endof
    dup target-sym LV!      is-xt? ?of  drop  .lit  endof
\    dup target-sym (COMPILE) is-xt? ?of drop  @word .xt  endof
  endcase
;
\ ...SFP006

: ?load-abs	\ -- ; add to LDR/STR(B) rx, [ pc, # n ]
  curr-instr @
  $0F2F0000 and			\ ignore cond, rd, immval
  $050F0000 = if		\ check with LDR rx, [ pc, # n ]
    tabbed  C" ( @$" $>dis
    DisAddr 4 +			\ PC value to use
    curr-instr @ $00000FFF and	\ extract offset
    curr-instr @ $00800000 and	\ extract U bit
    if  +  else  -  endif	\ apply offset
    dup num>dis			\ display address
    C"  = $" $>dis
    @c(t) num>dis		\ and data value
    C"  )" $>dis
  endif
;

: .opt4		\ --
  c" $" $>dis
  curr-instr @ $0F and hex>dis
;

defer CheckAS/VFP?	\ -- flag ; ASIMD and VFP instruction decoder

: do-main_ARM
  DisAddr to T2Insbase
  @long curr-instr !
  CheckAS/VFP? if  exit  endif		\ ASIMD and VFP check
  case
    groupHints1 ?of			\ SFP016
      case  curr-instr @ $00F0 and
        $0010 of  c" clrex " $>dis  drop  endof
        $0040 of  c" dsb  # " $>dis .opt4  drop  endof
        $0050 of  c" dmb  # " $>dis .opt4  drop  endof
        $0060 of  c" isb  # " $>dis .opt4  drop  endof
          c" undef_hint " $>dis  nip
      endcase
    endof
    groupCLZ    ?of
                        c" clz " $>dis .cond  .Rd ,>dis .Rm  drop
                endof

    group3      ?of
                        abit match
                        if
                          C" mla " $>dis .cond .sbit .Rd ,>dis .Rm ,>dis .Rs ,>dis .Rn drop
                        else
                          C" mul " $>dis .cond .sbit .Rd ,>dis .Rm ,>dis .Rs drop
                        then
                endof

    group4      ?of
                        ubit match
                        if
                          [char] s
                        else
                          [char] u
                        then
                        char>dis
                        abit match
                        if
                          C" mlal "
                        else
                          C" mull "
                        then
                        $>dis
                        .cond .sbit .RdLo ,>dis .RdHi ,>dis .Rm ,>dis .Rs drop
                endof

    group5      ?of
                        C" mrs " $>dis .cond .Rd ,>dis .rbit drop
                endof

    group6      ?of
                        C" msr " $>dis .cond .rbit .mask ,>dis .8bit-immed+rot drop
                endof

    group7      ?of
                        C" msr " $>dis .cond .rbit .mask ,>dis .Rm drop
                endof

    group8      ?of
                        C" bx " $>dis .cond .Rm drop
                endof

    group8a ?of		\ SFP014
      C" blx " $>dis .cond .Rm drop
    endof

    group0      ?of
                        .opcode .cond .data-proc-regs .8bit-immed+rot drop
                endof

    group1      ?of
                        .opcode .cond .data-proc-regs .Rm bl>dis .shift-immed drop
                endof

    group2      ?of
                        .opcode .cond .data-proc-regs .Rm bl>dis .shift .Rs drop
                endof

    group9      ?of
                        .lbit C" r" $>dis .bbit .cond .Rd ,>dis [>dis .Rn
                        pbit match 0=
                        if
                          ]>dis
                        then
                        ,>dis 12bit-immed match
                        swap ubit match 0=
                        if
                          0 rot -
                        else
                          swap
                        then
                        .imms
                        pbit match
                        if
                          ]>dis
                        then
                        .wbit drop
                endof

    group10     ?of
                        .lbit C" r" $>dis .bbit .cond .Rd ,>dis [>dis .Rn
                        pbit match 0=
                        if
                          ]>dis
                        then
                        ,>dis ubit match
                        if
                          C" ++ "
                        else
                          C" -- "
                        then
                        $>dis .Rm bl>dis .shift-immed
                        pbit match
                        if
                          ]>dis
                        then
                        .wbit drop
                endof

    group13     ?of
                        C" swp" $>dis .bbit .cond .Rd ,>dis .Rm ,>dis [>dis .Rn ]>dis drop
                endof

    group11     ?of
                        .lbit c" r" $>dis sbit match
                        if
                          c" s" $>dis
                        then
                        hbit match
                        if
                          c" h "
                        else
                          c" b "
                        then
                        $>dis
                        .cond .Rd ,>dis [>dis .Rn pbit match 0=
                        if
                          ]>dis
                        then
                        ,>dis
                        Hi-Offset match 4 lshift
                        swap Lo-Offset match rot or
                        swap ubit match 0=
                        if
                          0 rot -
                        else
                          swap
                        then
                        .imms
                        pbit match
                        if
                          ]>dis
                        then
                        .wbit drop
                endof

    group12     ?of
                        .lbit c" r" $>dis sbit match
                        if
                          c" s" $>dis
                        then
                        hbit match
                        if
                          c" h "
                        else
                          c" b "
                        then
                        $>dis
                        .cond .Rd ,>dis [>dis .Rn pbit match 0=
                        if
                          ]>dis
                        then
                        ,>dis ubit match
                        if
                          c" ++ "
                        else
                          c" -- "
                        then
                        $>dis .Rm
                        pbit match
                        if
                          ]>dis
                        then
                        .wbit drop
                endof

    group14     ?of
                        .lbit c" m" $>dis
                        ubit match
                        if
                          c" i"
                        else
                          c" d"
                        then
                        $>dis
                        pbit match
                        if
                          c" b "
                        else
                          c" a "
                        then
                        $>dis .cond .Rn .wbit
                        c"  { " $>dis .reg-list c"  }" $>dis
                        sbit match
                        if
                         c"  ^" $>dis
                        then
                        drop
                endof

    group15     ?of
                        drop c" Coprocessor data processing" $>dis
                endof

    group16     ?of
                        drop c" Coprocessor register transfers" $>dis
                endof

    group17     ?of
                        drop c" Coprocessor load and store" $>dis
                endof

    group18a ?of	\ SFP014
      c" blx " $>dis   24bit-offset match	\ -- group offset24
      dup $0080:0000 and negate or		\ -- group soffset32
      2 lshift  swap lbit match			\ extend, check bit 1
      if  $02 or  endif				\ -- soffset32'
      8 + StartAddr @ +				\ branch target address
      dup .imm  ShowBranchTarget		\ display it
    endof

    group18     ?of
                        lbit match
                        if
                          c" bl "
                        else
                          c" b "
                        then
                        $>dis .cond
                        24bit-offset match dup 8 lshift 0<
                        if
                          $ff000000 or
                        else
                          $ffffff and
                        then
                        2 + cells StartAddr @ +         \ branch target address
                        dup .imm  dup ShowBranchTarget  \ display it
                        swap lbit match
                        if
                          2drop
                        else
                          drop
                          >branch-target
                        then
                endof

    group19     ?of
                        c" swi " $>dis .cond swi-number match .imm drop
                endof

    group20     ?of
                        drop c" Undefined instruction" $>dis
                endof
  end-case
  ?load-abs		\ check for LDR rx, [ pc, # n ]
;

: undef	\ --
  c" UNDEF " $>dis  ;

: showUndef	\ $op --
  drop  undef  ;


\ ************************************
\ Thumb1 and 16 bit Thumb2 disassembly
\ ************************************

: -itcount	\ --
\ +G Count down number of instructions remaining in IT block.
  <itcount> @
  if  -1 <itcount> +!  endif
;

: undefT	\ --
  c" UNDEF " $>dis  ;

: badSuffix	\ --
  c" Unhandled BL(X) suffix " $>dis  ;

: $op>		\ caddr -- opcode
\ +G Put string in output buffer and return current opcode.
  $>dis  curr-instr @  ;

: .reg,		\ n --
\ +G Apply four-bit mask and display register with a trailing comma.
  .reg ,>dis  ;

: .loreg	\ n --
\ +G Apply three-bit mask and display register.
  7 and .reg
;

: .loreg,	\ n --
\ +G Apply three-bit mask and display register with a trailing comma.
  .loreg ,>dis
;

: .s/it		\ --
\ +G Display .S if not in IT block.
  <itcount> @ 0=
  if  c" .s " $>dis  endif
;

: .RdRn		\ --
\ +G Display two low registers in the form
\ *C   Rd, Rn
  curr-instr @
  dup .loreg,  3 rshift .loreg		\ Rd, Rm
;

: .RdRdRn		\ --
\ +G Display two low registers in three register form
\ *C   Rd, Rd, Rn
  curr-instr @ dup .loreg, dup .loreg,
  3 rshift .loreg
;

: .RdRn,		\ --
\ +G Display two low registers with more to come.
\ *C  Rd, Rn, ...
  curr-instr @
  dup .loreg,  3 rshift .loreg,		\ Rd, Rm,
;

: .RdRnImm3	\ caddr --
\ +G Display two low registers and a three bit immediate.
  $>dis .s/it .RdRn,	 		\ opc Rd, Rm,
  curr-instr @ 6 rshift 7 and .imm	\ # imm3
;

: .RdRnImm5	\ caddr --
\ +G Display two low registers and a five bit immediate.
  $>dis .s/it .RdRn,			\ opc Rd, Rm,
  curr-instr @ 6 rshift $1F and .imm	\ # imm5
;

: .RdRnRm	\ caddr --
\ +G Display three low registers.
  $>dis .s/it .RdRn,			\ opc Rd, Rn,
  curr-instr @ 6 rshift .loreg		\ Rm
;

: .RdImm8	\ caddr --
  $op> dup 8 rshift .loreg,  $FF and .imm  ;

: .SRdImm8	\ caddr --
  $op> .s/it dup 8 rshift .loreg,  $FF and .imm  ;

: .SRdRdImm8	\ caddr --
  $op> .s/it dup 8 rshift dup .loreg, .loreg,  $FF and .imm  ;

: .swi/bkpt		\ caddr -- ; SWI and BKPT instructions
\ +G decode SWI instructions.
  $op> $FF and .imm
;

create 16T1ops	\ -- addr
\ Th  0   1   2   3   4   5   6   7   8   9   A   B   C   D   E   F
  ", and eor lsl lsr asr adc sbc ror tst neg cmp cmn orr mul bic mvn "

%1111:1111:0100:1111 constant 16T1Sflags	\ -- mask
\ A set bit indicates that the opcode sets the flags *and* should
\ display ".S".

: .s16T1ops	\ index --
\ Display ".S" if required by index into 16 T1 ops.
  dup >r
  1 $0F rot - lshift 16T1Sflags and if
    .s/it
    case  r@
      $09 of  .RdRn  endof
      $0F of  .RdRn  endof
        drop  .RdRdRn
    end-case
  else
    .RdRn
  endif
  r> drop
;

: dT1dpiF5	\ --
  curr-instr @ 6 rshift $0F and		\ bits 9:6 are table index
  dup 2 lshift 16T1ops 1+ + 4 >disline	\ output opcode
  .s16T1ops				\ output ".S" and registers
;

: dT1dpiF8	\ caddr -- ; string is opcode
\ +G Display two hi/lo registers in two register form.
  $op>
  dup 7 and  over $0080 and 4 rshift or \ extract Rdh
  .reg,  3 rshift .reg		\ Rdh, Rmh
;

: dT1dpiF8-3	\ caddr -- ; string is opcode
\ +G Display two hi/lo registers in three register form.
  $op>
  dup 7 and  over $0080 and 4 rshift or \ extract Rdh
  dup .reg, .reg,  3 rshift .reg	\ Rdh, Rdh, Rmh
;

create 8T1mems	\ -- ; memory opcodes
\ Th   0     1     2     3     4     5     6     7 - 6 bytes per opcode
  ", str   strh  strb  ldrsb ldr   ldrh  ldrb  ldrsh "

: dT1memF2	\ --
\ +G Display load/store 3 regs instruction.
  curr-instr @
  dup 9 rshift 7 and 6 * 8T1mems 1+ +	\ pointer to opcode
  6 -trailing >disline bl>dis		\ display with space
  dup .loreg,				\ Rd,
  [>dis dup 3 rshift .loreg,		\ [ Rn,
  6 rshift .loreg ]>dis			\ Rm ]
;

: dT1bx		\ -- ; b(l)x  rm
  curr-instr @ dup $0007 and if
    drop undef exit
  endif
  dup $0080 and
  if  c" blx " else c" bx " then $>dis
  3 rshift $0F and .reg
;

: [MemImm5	\ caddr -- imm5
  $op>					\ opc
  dup .loreg,				\ rd,
  [>dis  dup 3 rshift .loreg,		\ [ rn,
  6 rshift $1F and			\ -- imm5
;

: imm]		\ imm5*x --
  .imm ]>dis
;

: .SPrel	\ caddr --
  $op>  dup 8 rshift .loreg,
  c" [ SP, " $>dis
  $FF and 2 lshift .imm  ]>dis
;

: .PCtarget	\ offset --
  c"  ( @$" $>dis			\ )
  T2insbase 4 + -4 and + dup num>dis
  c" =$" $>dis l@c(t) num>dis
  c"  ) " $>dis
;
((
: .PCtarget	\ offset --
  c"  ( $" $>dis			\ )
  T2insbase 4 + -4 and + num>dis
  c"  ) " $>dis
;
))

: .PCrel	\ caddr --
  $op>  dup 8 rshift .loreg,
  c" [ PC, " $>dis
  $FF and 2 lshift dup .imm
  ]>dis  .PCtarget
;

: .TRegMask	\ mask --
\ +G Display 8 bit register mask.
  8 0 do
    dup 1 and
    if  i .loreg bl>dis  endif
    1 rshift
  loop
  drop
;

: .ld/stmia	\ caddr --
  $op>  dup 8 rshift .loreg
  c" !, { " $>dis .TregMask  c" }" $>dis
;

: .pop		\ caddr --
  $op>  c" { " $>dis		\ start
  dup .TregMask			\ show low regs
  $0100 and			\ check for PC
  if  c" pc " $>dis  endif
  c" }" $>dis
;

: .push		\ caddr --
  $op>  c" { " $>dis		\ start
  dup .TregMask			\ show low regs
  $0100 and			\ check for PC
  if  c" lr " $>dis  endif
  c" }" $>dis
;

: Dest11	\ -- dest
  curr-instr @ $7FF and 1 lshift	\ offset
  dup $800 and				\ sign extend
  if  $FFFFF000 or  endif
  DisAddr 2+ +
;

: Dest8		\ -- dest
  curr-instr @ $FF and 1 lshift		\ offset
  dup $100 and				\ sign extend
  if  $FFFFFE00 or  endif
  DisAddr 2+ +
;

: .BranchTarget	\ addr --
\ +G Display branch target and set continuations.
  dup .imm  dup ShowBranchTarget  >branch-target  ;

: .condBr8	\ --
\ +G Display 8 bit conditional branch.
  c" b " $op> 8 rshift .condCode
  Dest8 .BranchTarget
;

: dTbr11	\ --
\ +G Decode 11 bit unconditional branch.
  c" b " $>dis  Dest11 .BranchTarget
;

: .bl/blxoff11	\ --
\ +G Decode BL/BLX # label instruction pair.
  curr-instr @ dup $1800 and $1000 <>	\ check H field
  if  drop badSuffix exit  endif
  $7FF and dup $400 and			\ sign extend 11 bit offset
  if  $FFFFF800 or  then  #12 lshift	\ and shift
  DisAddr 2+ +				\ add PC+4
  @word dup curr-instr !		\ second instruction
  case  dup $1800 and			\ inspect H field
    $1800 of  c"  bl "  endof
    $0800 of  c"  blx "  endof
    c" bad BL/X " swap
  endcase
  $>dis  $7FF and 1 lshift +		\ opcode, target
  .BranchTarget
;

: .blx/undef	\ --
\ +G Undefined instruction or BLX second instruction.
  curr-instr @ 1 and
  if  Undef  else  badSuffix  endif
;

: .SP/PCimm8	\ caddr --
  $op>  dup 8 rshift .loreg,		\ opc Rd
  dup $0800 and if			\ SP/PC,
    c" SP, " $>dis
    $FF and 2 lshift .imm		\ # imm8
  else
    c" PC, " $>dis
    $FF and 2 lshift dup .imm		\ # imm8
    .PCtarget
  then
;

: dTadjustSP	\ --
\ +G Decode "adjust stack" instructions.
  Curr-instr @ dup $0080 and
  if c" sub SP, SP, " else c" add SP, SP, " then $>dis
  $7F and 2 lshift .imm
;

: Imm1+5	\ -- imm6
  curr-instr @ dup 3 rshift $01F and	\ bits 7:3 -> 4:0
  swap 9 rshift 1 and 5 lshift or	\ bit    9 -> 5
;

: .cbx		\ $ --
\  $>dis  Imm1+5
  $op> .loreg, Imm1+5			\ SFP012
  1 lshift T2insbase + 4 +		\ -- dest
  .BranchTarget
;

: .itx		\ --
  c" it" $>dis  1  curr-instr @
  dup 4 rshift $0F and  swap $0F and	\ -- cnt cond mask
  begin
    dup $08 <>				\ %1000 = no more
   while
    over 1 and  over 3 rshift 1 and =
    if  c" t"  else  c" e"  then $>dis
    1 lshift $0F and			\ shift bit mask
    rot 1+ -rot				\ update counter
  repeat
  drop  bl>dis .condCode
  1+ <itcount> !			\ instruction count +1
;

: .itx/nop	\ --
\ ITxxx, NOP, SEV, WFE, WFI, YIELD
  case  curr-instr @ $FF and		\ bottom 8 bits
    $10 of  c" yield " $>dis  endof
    $30 of  c" wfi " $>dis  endof
    $20 of  c" wfe " $>dis  endof
    $40 of  c" sev " $>dis  endof
    $00 of  c" nop " $>dis  endof
      .itx
  endcase
;

: .cpxx		\ $ --
  $>dis  curr-instr @ $010 and
  if  c" .id  "  else  c" .ie  "  endif  $>dis
  curr-instr @ $02 and
  if  c" .i " $>dis  endif
  curr-instr @ $01 and
  if  c" .f " $>dis  endif
;

: decodeT11	\ -- flag ; true if processed
\ +G Decode instructions selected by the top 11 bits.
\ ** These are all Thumb-2 instructions.
  -1					\ assume processed
  case  curr-instr @ $FFE0 and		\ top 11 bit decode
   $B660 of  c" cps " .cpxx  endof
      nip 0 swap			\ not processed
  endcase
;

: .T1RdRn	\ $ --
  $>dis  .RdRn  ;

: decodeT10	\ -- flag ; true if processed
\ +G Decode instructions selected by the top 10 bits.
\ ** These are all Thumb-2 instructions.
  -1					\ assume processed
  case  curr-instr @ $FFC0 and		\ top 10 bit decode
   $BA00 of  c" rev " .T1RdRn  endof
   $BA40 of  c" rev16 " .T1RdRn  endof
   $BAC0 of  c" revsh " .T1RdRn  endof
   $B240 of  c" sxtb " .T1RdRn  endof
   $B200 of  c" sxth " .T1RdRn  endof
   $B2C0 of  c" uxtb " .T1RdRn  endof
   $B280 of  c" uxth " .T1RdRn  endof
      nip 0 swap			\ not processed
  endcase
;

: decodeT8	\ -- flag ; true if processed
\ +G Decode instructions selected by the top 8 bits.
  -1					\ assume processed
  case  curr-instr @ $FF00 and		\ top 8 bit decode
    $4400 of  c" add " dT1dpiF8-3  endof
    $4500 of  c" cmp " dT1dpiF8  endof
    $4600 of  c" mov " dT1dpiF8  endof
    $4700 of  dT1bx  endof
    $B000 of  dTadjustSP  endof
    $BE00 of  c" bkpt " .swi/bkpt  endof
    $DF00 of  c" svc/swi " .swi/bkpt  endof
    $B900 of  c" cbnz " .cbx  endof
    $BB00 of  c" cbnz " .cbx  endof
    $B100 of  c" cbz " .cbx  endof
    $B300 of  c" cbz " .cbx  endof
    $BF00 of  .itx/nop  endof		\ ITxxx, NOP, SEV, WFE, WFI, YIELD
      nip 0 swap			\ not processed
  endcase
;

: decodeT6	\ -- flag ; true if processed
\ +G Decode instructions selected by the top 6 bits.
  -1					\ assume processed
  case  curr-instr @ $FC00 and		\ top 6 bit decode
    $4000 of  dT1dpiF5  endof
      nip 0 swap			\ not processed
  endcase
;

: decodeT7	\ -- flag ; true if processed
\ +G Decode instructions selected by the top 7 bits.
  -1					\ assume processed
  case  curr-instr @ $FE00 and		\ top 7 bit decode
    $1800 of  c" add " .RdRnRm  endof
    $1A00 of  c" sub " .RdRnRm  endof
    $1C00 of  c" add " .RdRnImm3  endof
    $1E00 of  c" sub " .RdRnImm3  endof
    $BC00 of  c" pop " .pop  endof
    $B400 of  c" push " .push  endof
      nip 0 swap			\ not processed
  endcase
;

: decodeT5	\ -- flag ; true if processed
\ +G Decode instructions selected by the top 5 bits.
  -1					\ assume processed
  case  curr-instr @ $F800 and		\ top 5 bit decode
    $0000 of  c" lsl " .RdRnImm5  endof
    $0800 of  c" lsr " .RdRnImm5  endof
    $1000 of  c" asr " .RdRnImm5  endof
    $2000 of  c" mov " .SRdImm8  endof
    $2800 of  c" cmp " .RdImm8  endof
    $3000 of  c" add " .SRdRdImm8  endof
    $3800 of  c" sub " .SRdRdImm8  endof
    $4800 of  c" ldr " .PCrel  endof
    $6000 of  c" str " [MemImm5 4* Imm]  endof
    $6800 of  c" ldr " [MemImm5 4* Imm]  endof
    $7000 of  c" strb " [MemImm5 Imm]  endof
    $7800 of  c" ldrb " [MemImm5 Imm]  endof
    $8000 of  c" strh " [MemImm5 2* Imm]  endof
    $8800 of  c" ldrh " [MemImm5 2* Imm]  endof
    $9000 of  c" str " .SPrel  endof
    $9800 of  c" ldr " .SPrel  endof
    $C000 of  c" stmia " .ld/stmia  endof
    $C800 of  c" ldmia " .ld/stmia  endof
    $E000 of  dTbr11  endof
    $E800 of  .blx/undef  endof
      nip 0 swap			\ not processed
  endcase
;

: decodeT4	\ -- flag ; true if processed
\ +G Decode instructions selected by the top 4 bits.
  -1					\ assume processed
  case  curr-instr @ $F000 and		\ top 4 bit decode
    $5000 of  dT1memF2  endof
    $A000 of  c" add " .SP/PCimm8 endof
    $D000 of  .condBr8  endof
    $F000 of  .bl/blxoff11  endof
      nip 0 swap			\ not processed
  endcase
;


\ *************************
\ Thumb2 32 bit disassembly
\ *************************
\ See ARM DDI 0405C for page references.

\ Instruction groups are defined as a table of /Thumb2Items below.
\ A row of four zeros terminates the table.

\ Thumb2 Instructions are decoded by applying a bit mask to the
\ current instruction, comparing that to a required value, and
\ then applying additional tests. The additional tests are usually
\ to test whether a register is R15, which selects special case
\ decoding.

struct /Thumb2Item	\ -- len
\ Size of an entry in the list below.
  int t2i.mask		\ mask applied to instruction
  int t2i.value		\ value after masking for match
  int t2i.test		\ additional test specification
  int t2i.xt		\ word executed when match and tests succeed
  #16 field t2i.name	\ instruction name, counted string
end-struct


\ =================================
\ Testing the instruction structure
\ =================================

: Rd8=PC?	\ -- flag
\ Return true if Rd8 is the PC
  curr-instr @ 8 rshift $0F and $0F =
;

: Rd8<>PC?	\ -- flag
\ Return true if Rd8 is not the PC
  curr-instr @ 8 rshift $0F and $0F <>
;

: Rt12=PC?	\ -- flag
\ Return true if Rt12 is the PC
  curr-instr @ #12 rshift $0F and $0F =
;

: Rt12<>PC?	\ -- flag
\ Return true if Rt12 is not the PC
  curr-instr @ #12 rshift $0F and $0F <>
;

: Rn16=PC?	\ -- flag
\ Return true if Rn16 is the PC
  curr-instr @ #16 rshift $0F and $0F =
;

: Rn16<>PC?	\ -- flag
\ Return true if Rn16 is not the PC
  curr-instr @ #16 rshift $0F and $0F <>
;

: CheckTest	\ addr -- flag
\ The t2i.test cell defines adddtional tests. Decode succeeds
\ if all tests return true.
\   bits1:0  Rt12=15  %00=return true
\                     %10=return true if Rt<>15
\                     %11=return true if Rt=15
\   bits3:2  Rn16=15  %00=return true
\                     %10=return true if Rn<>15
\                     %11=return true if Rn=15
\   bits5:4  Rd8=15   %00=return true
\                     %10=return true if Rd<>15
\                     %11=return true if Rd=15
  t2i.test @ true			\ -- tests flag
  case  over $3 and
    %1:0 of  Rt12<>PC? and  endof
    %1:1 of  Rt12=PC? and  endof
  endcase
  case  over 2 rshift $3 and
    %1:0 of  Rn16<>PC? and  endof
    %1:1 of  Rn16=PC? and  endof
  endcase
  case  over 4 rshift $3 and
    %1:0 of  Rd8<>PC? and  endof
    %1:1 of  Rd8=PC? and  endof
  endcase
  nip
; doNotSin

: ExecItem	\ addr --
\ Execute the xt of the list, passing the string to the xt.
  dup t2i.name swap t2i.xt @ execute  ; doNotSin

: DecodeList	\ addr --
\ Decode an list of opcode masks and values. If no matching
\ entry is found, the instruction is assumed to be undefined.
 begin
    dup t2i.xt @			\ xt=0=end
   while
    dup t2i.mask @ curr-instr @ and	\ apply mask
    over t2i.value @ = if		\ match to value?
      dup CheckTest			\ and additional tests pass?
      if   execItem  exit  endif	\ pass string to xt
    endif
    /Thumb2Item +
  repeat
  drop Undef
;

: DecodeListRaw	\ addr -- flag ; nz = found
\ Decode an list of opcode masks and values. If no matching
\ entry is found, the instruction is assumed to be undefined.
\ dup body> >name cr .name  key? drop
  begin					\ -- addr
    dup t2i.xt @			\ xt=0=end
   while
\ dup t2i.name count cr 2 spaces type  key? drop
    dup t2i.mask @ curr-instr @ and	\ apply mask
    over t2i.value @ = if		\ match to value?
      dup CheckTest if			\ and additional tests pass?
        t2i.xt @ execute  true  exit	\ run xt
      endif
    endif
    /Thumb2Item +
  repeat
  drop 0
;


\ =======
\ Parsing
\ =======
\ The mask and value are created by by parsing a string of the
\ form
\   0011xx100xx
\ where the '0' and '1' characters indicate that they should be
\ part of the mask and what the value should be. The 'x' character
\ indicates "don't care", and applies 0 bits to both the mask and
\ the value. All other characters are ignored so that '_' and
\ ':' can be used as group separators.

#64 buffer: insmask	\ -- addr
\ scratch buffer for building the mask.
#64 buffer: insvalue	\ -- addr
\ scratch buffer for building the mask.
variable maskCnt	\ -- addr
\ Holds number of bits processed. Used for checking

: ?maskErr	\ flag --
\ bad mask if flag is nz
  abort" Incomplete mask/value string" ;

: +maskchar	\ bitmask -- bitmask'
  dup 0= ?maskErr
  1 rshift  maskCnt incr  ;

: xmv		\ bitmask -- bitmask'
\ Deal with 'x' character.
  dup insmask bic!  dup insvalue bic!  +maskChar  ;

: 0mv		\ bitmask -- bitmask'
  dup insmask or!  dup insvalue bic!  +maskChar  ;

: 1mv		\ bitmask -- bitmask'
  dup insmask or!  dup insvalue or!  +maskChar  ;

: setDMask	\ bitmask char -- bitmask'
\ Check the character, apply as required to the mask and value
\ using the bit mask, and change the bitmask as required.
  case
   [char] 0 of  0mv  endof
   [char] 1 of  1mv  endof
   [char] x of  xmv  endof
   [char] X of  xmv  endof
  endcase
;

: makeMV	\ bitmask -- bitmask'
\ Collect and process a mask/value pair from the input stream.
  maskCnt off  parse-name bounds
  ?do  i c@ setDMask  loop
  ?maskErr
;

: makeMV32,	\ --
\ Collect and lay down a 32 bit mask/value pair.
  $8000:0000 makeMV  insmask @ ,  insvalue @ ,  ;

: mvtests,	\ --
\ Evaluate the test value and comma it in.
  parse-name evaluate ,  ;

: mvxt,		\ --
\ Comma in the xt of the word following.
  ' ,  ;

: mvname,	\ --
\ Lay down the instruction name. Keeps trailing spaces.
  here >r  #16 allot&erase
  [char] " parse -leading #15 min r> place  ;

: mv"		\ --
\ Build a decode table entry.
  makemv32, mvtests, mvxt, mvname,  ;

: mv:		\ --
\ Syntactic sugar to start a table
  create  ;

: mvR:		\ --
\ Syntactic sugar to start a table
  create  ;

: ;mv		\ --
\ Syntactic sugar to end a table
  0 ,  0 ,  0 ,  0 ,  0 ,
  does> decodeList
;

: ;mvR		\ -- ; -- flag
\ Syntactic sugar to end a table
  0 ,  0 ,  0 ,  0 ,  0 ,
  does> decodeListRaw
;


\ =====
\ Tools
\ =====

: unpredictable	\ --
  c" UNPREDICTABLE " $>dis
;

: Rd0		\ --
\ Decode the register at bits 3:0.
  curr-instr @ .reg
;

: Rd0,		\ --
\ Decode the register at bits 3:0. followed by comma and space.
  Rd0 ,>dis
;

: Rd0=PC	\ --
\ Check that Rd0=15.
  curr-instr @ $000F and $000F <>
  if  unpredictable  endif
;

: Rm0	Rd0  ;
: Rm0,	Rd0,  ;

: Rd8		\ --
\ Decode the register at bits 11:8.
  curr-instr @ 8 rshift .reg
;

: Rd8,		\ --
\ Decode the register at bits 11:8. followed by comma and space.
  Rd8 ,>dis
;

: Rd8=PC	\ --
\ Check that Rd8=15.
  Rd8<>PC? if  unpredictable  endif
;

: Rt12		\ --
\ Decode the register at bits 15:12.
  curr-instr @ 12 rshift .reg
;

: Rt12,		\ --
\ Decode the register at bits 15:12. followed by comma and space.
  Rt12 ,>dis
;

: Rn16		\ --
\ Decode the register at bits 19:16.
  curr-instr @ #16 rshift .reg
;

: Rn16,		\ --
\ Decode the register at bits 19:16. followed by comma and space.
  Rn16 ,>dis
;

: Rn16=PC	\ --
\ Check that Rn16=15.
  Rn16<>PC? if  unpredictable  endif
;

: wb!		\ --
\ Display writeback indicator.
  curr-instr @ $0020:0000 and		\ W bit 21
  if  c" ! " $>dis  endif
;

: wb!8		\ --
\ Display writeback indicator.
  curr-instr @ $0000:0100 and
  if  c" !" $>dis  endif
  bl>dis
;

: [Rn16]	\ --
\ Display indexed with no offset.
  [>dis Rn16 ]>dis
;

: [Rn16+off]	\ offset --
\ Display preindexed form
  [>dis Rn16, .imms  ]>dis
;

: [Rn16]+off	\ offset --
\ Display postindexed form
  [>dis Rn16 ]>dis  ,>dis .imms
;

: Rn16Ind+off	\ offset --
\ Display pre or post indexed forms.
  curr-instr @ $0100:0000 and		\ P bit 24
  if  [Rn16+off] wb!  else  [Rn16]+off  endif
;

: Rn16IndMemlo	\ offset --
\ Display pre or post indexed forms for modes where PUW are in
\ bits 10:8.
  curr-instr @ $0000:0400 and		\ P bit 24
  if  [Rn16+off] wb!8  else  [Rn16]+off  endif
;

: wb21,		\ --
\ If the W bit 21 is set output "!, " otherwise just ", "
  curr-instr @ $0020:0000 and
  if  c" !" $>dis  endif
  ,>dis
;

: T2Imm8	\ -- u
\ Return instruction bits 7:0.
  curr-instr @ $FF and  ;

: T2Imm8*4	\ -- u
\ Return instruction bits 7:0 scaled by 4.
  T2Imm8 2 lshift  ;

: T2Imm12	\ -- u
\ Return instruction bits 11:0.
  curr-instr @ $0FFF and  ;

: +/-U		\ u -- n
\ If the U bit at bit 23 is zero, negate u.
  curr-instr @ $0080:0000 and 0=
  if  negate  endif
;

: +/-U9		\ u -- n
\ If the U bit at bit 9 is zero, negate u.
  curr-instr @ $0000:0200 and 0=
  if  negate  endif
;

: Sbit20	\ --
\ Display state of S bit if set.
  curr-instr @ $0010:0000 and
  if  c" .s " $>dis  endif
;

: checkSbit20	\ --
  curr-instr @ $0010:0000 and 0=
  if  unpredictable  endif
;

: GetImm5	\ -- imm5
\ Get immediate value from bits 14:12 and 7:6.
  curr-instr @
  dup $0000:00C0 and 6 rshift
  swap $0000:7000 and #10 rshift or	\ -- imm5
;

: #Imm5		\ --
  GetImm5 .imm  ;

: ?Shift32	\ n -- n'
  dup 0=  if  32 +  endif
;

: ShowShift5	\ imm5 type --
  case
    0 of  c" lsl " $>dis .imm  endof
    1 of  c" lsr " $>dis ?Shift32 .imm  endof
    2 of  c" asr " $>dis ?Shift32 .imm  endof
    3 of
      dup 0= if
        drop  c" rrx # 1 " $>dis
      else
        c" ror " $>dis .imm
      endif
    endof
      c" Eek! Report to MPE" $>dis  nip
  endcase
;

: GetShift5	\ -- imm5 type
  curr-instr @
  dup  $0000:00C0 and 6 rshift
  over $0000:7000 and #10 rshift or	\ -- opc imm5
  swap $0000:0030 and 4 rshift
;

: ?shift5
  GetShift5  2dup or 0=			\ don't show LSL # 0
  if  2drop exit  endif
  bl>dis ShowShift5
;

: T2Rd8Rm0Rn16	\ $ -- ; SFP013
\ Display instruction of the form:  QADD  Rd, Rm, Rn
  $>dis  Rd8, Rn16, Rm0  ;

: T2Rd8Rn16Rm0	\ $ --
\ Display instruction of the form:  MUL  Rd, Rn, Rm
  $>dis  Rd8, Rn16, Rm0  ;

: T2SRd8Rn16Rm0	\ $ --
\ Display instruction of the form:  LSL  [ .s ] Rd, Rn, Rm
  $>dis Sbit20  Rd8, Rn16, Rm0  ;

: T2SRd8Rn16Rm0sh5	\ $ --
\ Display instruction of the form:  AND  Rd, Rn, Rm lsl # n
  $>dis Sbit20  Rd8, Rn16, Rm0 ?shift5  ;

: T2cmpRn16Rm0sh5	\ $ --
  $>dis  Rn16, Rm0 ?shift5  checkSbit20
;

: T2opSRd8Rm0	\ $ --
  $>dis  Sbit20  Rd8, Rm0
;

: T2opSRd8Rm0sh5	\ $ --
  $>dis  Sbit20  Rd8, Rm0 ?shift5
;

: T2shiftRd8Rm0imm5a	\ $ --
  $>dis  Sbit20  Rd8, Rm0, #Imm5
;

: T2shiftRd8Rm0imm5b	\ $ --
  $>dis  Sbit20  Rd8, Rm0, GetImm5 ?shift32 .imm
;

: (cp8#)	\ --
\ Display coprocessor number from bits 11:8.
  c" CP" $>dis  curr-instr @ 8 rshift $0F and dec>dis
;

: cp8#		\ --
\ Display coprocessor number from bits 11:8, followed by a space
  (cp8#)  bl>dis  ;

: cp8#,		\ --
\ Display coprocessor number from bits 11:8, followed by a comma and space
  (cp8#)  ,>dis  ;

create cpregs	\ -- addr
  ", cr0 cr1 cr2 cr3 cr4 cr5 cr6 cr7 cr8 cr9 cr10cr11cr12cr13cr14cr15"

: (.cpreg)	\ reg# --
  $0F and 4* cpregs + 1+ 4 -TRAILING >disline
;

: .cpreg	\ reg# --
  (.cpreg) bl>dis  ;

: .cpreg,	\ reg# --
  (.cpreg) ,>dis  ;

: CRm0		\ --
\ Display CPreg at bits 3:0.
  curr-instr @ .cpreg
;

: CRm0,		\ --
\ Display CPreg at bits 3:0.
  curr-instr @ .cpreg,
;

: CRd12,	\ --
\ Display CPreg at bits 15:12.
  curr-instr @ #12 rshift .cpreg,
;

: CRn16,	\ --
\ Display CPreg at bits 19:16.
  curr-instr @ #16 rshift .cpreg,
;

: form138	\ -- x
\ extract the 1:3:8 12 bit immediate/constant.
  curr-instr @
  dup #26 rshift 1 and #11 lshift	\ bit 26     -> 11
  over #12 rshift $07 and 8 lshift or	\ bits 14:12 -> 10:8
  swap $FF and or			\ bits 7:0   -> 7:0
;

: Const138	\ --
\ Display 12 bit constant. See page A5-16.
  form138 dup $0C00 and if		\ inspect bits 11:10
    dup $007F and $080 or		\ unrotated value in bits 6:0
    swap $0F80 and 7 rshift		\ shift count in bits 11:7
    ror
  else
    case dup $0300 and			\ inspect bits 9:8
      $0000 of  $0FF and  endof
      $0100 of  $0FF and dup #16 lshift or  endof
      $0200 of  $0FF and dup #24 lshift swap #8 lshift or  endof
      $0300 of  $0FF and dup 8 lshift or  dup #16 lshift or  endof
    endcase
  endif
  .imm
;

: Imm138	\ --
\ Display 12 bit immediate literal
  form138 .imm
;

: T2SRd8Rn16Const138	\ $ --
  $>dis Sbit20  Rd8, Rn16, Const138
;

: T2cmpRn16Const138	\ $ --
  $>dis  Rn16, Const138  checkSbit20
;

: T2SopRd8Const138	\ $ --
  $>dis  Sbit20  Rd8, Const138
;

: T2opRd8Rn16Imm138	\ $ --
  $>dis  Rd8, Rn16, Imm138
;

: formImm16	\ -- x
\ extract the 4:1:3:8 16 bit immediate/constant.
  curr-instr @
  dup 16 rshift $0F and 12 lshift
  over 26 rshift 1 and 11 lshift or
  over 12 rshift $07 and 8 lshift or
  swap $FF and or
;

: Imm16		\ --
  #>dis  formImm16 num>dis  ;

: T2opRd8Imm16	\ $ --
  $>dis  Rd8, Imm16  ;

: bits4:0	\ -- x
  curr-instr @ $01F and
;

: satImm	\ --
  bits4:0 1+ .imm  ;

: satShift	\ --
  curr-instr @ $0020:0000 and if
    c"  asr " $>dis  #Imm5
  else
    curr-instr @ $0000:70C0 and
    if  c"  lsl " $>dis  #Imm5  endif
  endif
;

: T2xsat	\ $ --
  $>dis  Rd8, satImm ,>dis Rn16 satShift  ;

: T2xsat16	\ $ --
  $>dis  Rd8, satImm ,>dis Rn16  ;

: T2xbfx	\ $ --
  $>dis  Rd8, Rn16, #Imm5 ,>dis  satImm  ;

: formOff21	\ -- n
\ Form a 21 bit offset from the current instruction.
\ Page A6-40 encoding T3, conditional branches with a 21 bit offset.
\ The last bit of 21 is an implicit 0. Offset=S:J2:J1:imm6:imm11:'0'.
  curr-instr @
  dup $07FF and 1 lshift			\ bits 10:0  -> 11:1	imm11
  over #16 rshift $03F and 12 lshift or		\ bits 21:16 -> 17:12	imm6
  over #13 rshift 1 and #18 lshift or		\ bit  13    -> 18	J1
  over #11 rshift 1 and #19 lshift or		\ bit  11    -> 19	J2
  swap #26 rshift 1 and #20 lshift tuck or	\ bit  13    -> 20	S
  swap negate or				\ sign extend
;

: T2cond22	\ -- x
\ Get the condition code from bits 25:22.
  curr-instr @ #22 rshift $0F and  ;

: T2SysReg	\ --
  c" SYS" $>dis T2imm8 num>dis  ;

: opt0		\ $ --
\ Show 4 bit option code from bits 3:0
  $>dis  Curr-instr @ $0F and .imm
;

: [Rn16Rm0lsl]	\ --
\ Display registers for load/store
  [>dis  Rn16, Rm0
  curr-instr @ $030 and 4 rshift ?dup
  if  c" , lsl " $>dis .imm bl>dis  endif
  ]>dis
;

: PC+off12	\ -- addr
  T2insbase 4 + -4 and T2Imm12 +/-U +  ;

: .PCrel12	\ --
  c"  ( $" $>dis PC+off12 num>dis c"  ) " $>dis  ;

: PClit12	\ --
  T2Imm12 +/-U [Rn16+off] .PCrel12  ;

: T2memPClit	\ $ --
\ 12 bit indexed PC relative signed offset.
  $>dis  PClit12  ;

: T2memRt12Rn16Imm12	\ $ --
\ 12 bit indexed positive offset.
  $>dis  Rt12,  T2Imm12 [Rn16+off]  ;

: T2memRt12PClit12	\ $ --
\ 12 bit indexed PC relative signed offset.
  $>dis  Rt12, PClit12  ;

: T2memRt12Rn16Imm8	\ $ --
\ 12 bit indexed offset.
  $>dis  Rt12, T2Imm8 +/-U9 Rn16IndMemlo  ;

: T2memRn16Rm0lsl	\ $ --
  $>dis  [Rn16Rm0lsl]  ;

: T2memRt12Rn16Rm0lsl	\ $ --
  $>dis  Rt12, [Rn16Rm0lsl]  ;

: T2memRn16Imm+12	\ $ -- ; e.g. PLD/PLDW
  $>dis  T2Imm12 [Rn16+off]  ;

: T2memRn16Imm-8	\ $ --
  $>dis  T2imm8 negate [Rn16+off]  ;

: T20op		\ $ -- ; No operands
  $>dis  ;

: rot2		\ --
  curr-instr @ $030 and ?dup
  if  ,>dis  1 rshift .imm  endif
;

: T2Rd8Rm0rot2	\ $ --
  $>dis  Rd8, Rm0 rot2  ;

: T2Rd8Rn16Rm0rot2	\ $ --
  $>dis  Rd8, Rn16, Rm0 rot2  ;

: T2Rd8Rm0	\ $ --
  $>dis  Rd8, Rm0  ;

: T2Rd8Rn16Rm0Ra12	\ $ --
  $>dis  Rd8, Rn16, Rm0, Rt12  ;

: T2Rd12Rd8Rn16Rm0	\ $ --
  $>dis  Rt12, Rd8, Rn16, Rm0  ;


\ ========================
\ Long multiply and divide - A5-31
\ ========================

mv: T2lmul/div		\ -- ; A5-31
\   mask/value------------------------- test    xt                name "
\   xxxxxxxx:xIIIxxxx:xxxxxxxx:IIIIxxxx  RdRnRt
mv" xxxxxxxx:x000xxxx:xxxxxxxx:0000xxxx %000000 T2Rd12Rd8Rn16Rm0  smull "  \ A6-214 T1
mv" xxxxxxxx:x001xxxx:xxxxxxxx:1111xxxx %000000 T2Rd8Rn16Rm0      sdiv "   \ A6-210 T1
mv" xxxxxxxx:x010xxxx:xxxxxxxx:0000xxxx %000000 T2Rd12Rd8Rn16Rm0  umull "  \ A6-269 T1
mv" xxxxxxxx:x011xxxx:xxxxxxxx:1111xxxx %000000 T2Rd8Rn16Rm0      udiv "   \ A6-267 T1
mv" xxxxxxxx:x100xxxx:xxxxxxxx:0000xxxx %000000 T2Rd12Rd8Rn16Rm0  smlal "  \ A6-269 T1

mv" xxxxxxxx:x100xxxx:xxxxxxxx:1000xxxx %000000 T2Rd12Rd8Rn16Rm0  smlalbb "  \ A7-439
mv" xxxxxxxx:x100xxxx:xxxxxxxx:1001xxxx %000000 T2Rd12Rd8Rn16Rm0  smlalbt "
mv" xxxxxxxx:x100xxxx:xxxxxxxx:1010xxxx %000000 T2Rd12Rd8Rn16Rm0  smlaltb "
mv" xxxxxxxx:x100xxxx:xxxxxxxx:1011xxxx %000000 T2Rd12Rd8Rn16Rm0  smlaltt "

mv" xxxxxxxx:x100xxxx:xxxxxxxx:1100xxxx %000000 T2Rd12Rd8Rn16Rm0  smlald "  \ A7-441
mv" xxxxxxxx:x100xxxx:xxxxxxxx:1101xxxx %000000 T2Rd12Rd8Rn16Rm0  smlaldx "  \ A7-441

mv" xxxxxxxx:x101xxxx:xxxxxxxx:1100xxxx %000000 T2Rd12Rd8Rn16Rm0  smlsld "  \ A7-447
mv" xxxxxxxx:x101xxxx:xxxxxxxx:1101xxxx %000000 T2Rd12Rd8Rn16Rm0  smlsldx "  \ A7-447

mv" xxxxxxxx:x110xxxx:xxxxxxxx:0000xxxx %000000 T2Rd12Rd8Rn16Rm0  umlal "  \ A6-268 T1
mv" xxxxxxxx:x110xxxx:xxxxxxxx:0110xxxx %000000 T2Rd12Rd8Rn16Rm0  umaal "  \ A7-536
;mv


\ ================================
\ Multiply and multiply accumulate - A5-30, 0403D p186
\ ================================

: doSMLAxx	\ $ -- ; 0403D p433
  drop
  case  curr-instr @ $0030 and
    $0000 of  c" smlabb " T2Rd8Rn16Rm0Ra12  endof
    $0010 of  c" smlabt " T2Rd8Rn16Rm0Ra12  endof
    $0020 of  c" smlatb " T2Rd8Rn16Rm0Ra12  endof
    $0030 of  c" smlatt " T2Rd8Rn16Rm0Ra12  endof
  endcase
;

: doSMULxx	\ $ -- ; 0403D p433
  drop
  case  curr-instr @ $0030 and
    $0000 of  c" smulbb " T2Rd8Rn16Rm0  endof
    $0010 of  c" smulbt " T2Rd8Rn16Rm0  endof
    $0020 of  c" smultb " T2Rd8Rn16Rm0  endof
    $0030 of  c" smultt " T2Rd8Rn16Rm0  endof
  endcase
;

: doSMLADx	\ $ -- ; 0403D p435
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smlad " T2Rd8Rn16Rm0Ra12  endof
    $0010 of  c" smladx " T2Rd8Rn16Rm0Ra12  endof
  endcase
;

: doSMUADx	\ $ -- ; 0403D p453
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smuad " T2Rd8Rn16Rm0  endof
    $0010 of  c" smuadx " T2Rd8Rn16Rm0  endof
  endcase
;

: doSMLAWx	\ $ -- ; 0403D p443
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smlawb " T2Rd8Rn16Rm0Ra12  endof
    $0010 of  c" smlawt " T2Rd8Rn16Rm0Ra12  endof
  endcase
;

: doSMULWx	\ $ -- ; 0403D p458
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smulwb " T2Rd8Rn16Rm0  endof
    $0010 of  c" smulwt " T2Rd8Rn16Rm0  endof
  endcase
;

: doSMLSDx	\ $ -- ; 0403D p445
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smlsd " T2Rd8Rn16Rm0Ra12  endof
    $0010 of  c" smlsdx " T2Rd8Rn16Rm0Ra12  endof
  endcase
;

: doSMUSDx	\ $ -- ; 0403D p459
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smusd " T2Rd8Rn16Rm0  endof
    $0010 of  c" smusdx " T2Rd8Rn16Rm0  endof
  endcase
;

: doSMMLAx	\ $ -- ; 0403D p449
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smmla " T2Rd8Rn16Rm0Ra12  endof
    $0010 of  c" smmlar " T2Rd8Rn16Rm0Ra12  endof
  endcase
;

: doSMMULx	\ $ -- ; 0403D p459
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smmul " T2Rd8Rn16Rm0  endof
    $0010 of  c" smmulr " T2Rd8Rn16Rm0  endof
  endcase
;

: doSMMLSx	\ $ -- ; 0403D p450
  drop
  case  curr-instr @ $0010 and
    $0000 of  c" smmls " T2Rd8Rn16Rm0Ra12  endof
    $0010 of  c" smmlsr " T2Rd8Rn16Rm0Ra12  endof
  endcase
;


mv: T2mul/mulacc	\ -- ; A5-30
\   mask/value------------------------- test    xt                name "
\   xxxxxxxx:xIIIxxxx:xxxxxxxx:00IIxxxx  RdRnRt
mv" xxxxxxxx:x000xxxx:xxxxxxxx:0000xxxx %000010 T2Rd8Rn16Rm0Ra12  mla "	 \ A6-146 T1
mv" xxxxxxxx:x000xxxx:xxxxxxxx:0000xxxx %000011 T2Rd8Rn16Rm0      mul "	 \ A6-160 T2
mv" xxxxxxxx:x000xxxx:xxxxxxxx:xx01xxxx %000000 T2Rd8Rn16Rm0Ra12  mls "	 \ A6-147 T1

mv" xxxxxxxx:x001xxxx:xxxxxxxx:00xxxxxx %000010 doSMLAxx          smlaxx " \ A7-433
mv" xxxxxxxx:x001xxxx:xxxxxxxx:00xxxxxx %000011 doSMULxx          smulxx " \ A7-455

mv" xxxxxxxx:x010xxxx:xxxxxxxx:000xxxxx %000010 doSMLADx          smlaxx " \ A7-435
mv" xxxxxxxx:x010xxxx:xxxxxxxx:000xxxxx %000011 doSMUADx          smulxx " \ A7-453

mv" xxxxxxxx:x011xxxx:xxxxxxxx:000xxxxx %000010 doSMLAWx          smlawx " \ A7-435
mv" xxxxxxxx:x011xxxx:xxxxxxxx:000xxxxx %000011 doSMULWx          smulwx " \ A7-453

mv" xxxxxxxx:x100xxxx:xxxxxxxx:000xxxxx %000010 doSMLSDx          smlsdx " \ A7-435
mv" xxxxxxxx:x100xxxx:xxxxxxxx:000xxxxx %000011 doSMUSDx          smusdx " \ A7-453

mv" xxxxxxxx:x101xxxx:xxxxxxxx:000xxxxx %000010 doSMMLAx          smmlax " \ A7-449
mv" xxxxxxxx:x101xxxx:xxxxxxxx:000xxxxx %000011 doSMMULx          smmulx " \ A7-451

mv" xxxxxxxx:x110xxxx:xxxxxxxx:000xxxxx %000000 doSMMLSx          smmlax " \ A7-449

mv" xxxxxxxx:x111xxxx:xxxxxxxx:0000xxxx %000010 T2Rd8Rn16Rm0Ra12  usada8 " \ A7-545
mv" xxxxxxxx:x111xxxx:xxxxxxxx:0000xxxx %000011 T2Rd8Rn16Rm0      usad8 "  \ A7-546

;mv


\ =================================
\ Parallel ADD/SUB - A5-182 (0403D)
\ =================================

mv: T2Padd/sub	\ -- ; A5-182 (0403D)
\   mask/value------------------------- test    xt                  name "
\   xxxxxxxx:xIIIxxxx:xxxxxxxx:xxIIxxxx  RdRnRt
mv" xxxxxxxx:xxxxxxxx:xxxxxxxx:xxxxxxxx %000010 T20op             UNDEFINED "
mv" xxxxxxxx:x001xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      sadd16 "
mv" xxxxxxxx:x010xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      sasx "
mv" xxxxxxxx:x110xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      ssax "
mv" xxxxxxxx:x101xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      ssub16 "
mv" xxxxxxxx:x000xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      sadd8 "
mv" xxxxxxxx:x100xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      ssub8 "

mv" xxxxxxxx:x001xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      qadd16 "
mv" xxxxxxxx:x010xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      qasx "
mv" xxxxxxxx:x110xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      qsax "
mv" xxxxxxxx:x101xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      qsub16 "
mv" xxxxxxxx:x000xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      qadd8 "
mv" xxxxxxxx:x100xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      qsub8 "
;mv

: doPadd/sub	\ $ --
  drop T2Padd/sub
;

mv: T2Padd/subU	\ -- ; A5-182 (0403D)
\   mask/value------------------------- test    xt                  name "
\   xxxxxxxx:xIIIxxxx:xxxxxxxx:xxIIxxxx  RdRnRt
mv" xxxxxxxx:xxxxxxxx:xxxxxxxx:xxxxxxxx %000010 T20op             UNDEFINED "
mv" xxxxxxxx:x001xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      uadd16 "
mv" xxxxxxxx:x010xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      uasx "
mv" xxxxxxxx:x110xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      usax "
mv" xxxxxxxx:x101xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      usub16 "
mv" xxxxxxxx:x000xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      uadd8 "
mv" xxxxxxxx:x100xxxx:xxxxxxxx:xx00xxxx %000011 T2Rd8Rn16Rm0      usub8 "

mv" xxxxxxxx:x001xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      uqadd16 "
mv" xxxxxxxx:x010xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      uqasx "
mv" xxxxxxxx:x110xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      uqsax "
mv" xxxxxxxx:x101xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      uqsub16 "
mv" xxxxxxxx:x000xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      uqadd8 "
mv" xxxxxxxx:x100xxxx:xxxxxxxx:xx01xxxx %000011 T2Rd8Rn16Rm0      uqsub8 "

mv" xxxxxxxx:x001xxxx:xxxxxxxx:xx10xxxx %000011 T2Rd8Rn16Rm0      uhadd16 "
mv" xxxxxxxx:x010xxxx:xxxxxxxx:xx10xxxx %000011 T2Rd8Rn16Rm0      uhasx "
mv" xxxxxxxx:x110xxxx:xxxxxxxx:xx10xxxx %000011 T2Rd8Rn16Rm0      uhsax "
mv" xxxxxxxx:x101xxxx:xxxxxxxx:xx10xxxx %000011 T2Rd8Rn16Rm0      uhsub16 "
mv" xxxxxxxx:x000xxxx:xxxxxxxx:xx10xxxx %000011 T2Rd8Rn16Rm0      uhadd8 "
mv" xxxxxxxx:x100xxxx:xxxxxxxx:xx10xxxx %000011 T2Rd8Rn16Rm0      uhsub8 "
;mv

: doPadd/subU	\ $ --
  drop T2Padd/subU
;

\ ========================
\ Data processing register
\ ========================
\ A5-28, A5-24 (0403D)

mv: T2dpr	\ -- ; A5-28
\   mask/value------------------------- test    xt                  name "
\   xxxxxxxx:IIIIxxxx:xxxxxxxx:IIIIxxxx  RdRnRt
mv" xxxxxxxx:xxxxxxxx:xxxxxxxx:xxxxxxxx %000010 T20op               UNDEFINED "
mv" xxxxxxxx:000xxxxx:xxxxxxxx:0000xxxx %000000 T2SRd8Rn16Rm0    lsl "  \ A6-136 T2
mv" xxxxxxxx:001xxxxx:xxxxxxxx:0000xxxx %000000 T2SRd8Rn16Rm0    lsr "  \ A6-140 T2
mv" xxxxxxxx:010xxxxx:xxxxxxxx:0000xxxx %000000 T2SRd8Rn16Rm0    asr "  \ A6-38 T2
mv" xxxxxxxx:011xxxxx:xxxxxxxx:0000xxxx %000000 T2SRd8Rn16Rm0    ror "  \ A6-196 T2
mv" xxxxxxxx:0000xxxx:xxxxxxxx:1xxxxxxx %001011 T2Rd8Rn16Rm0rot2 sxtah " \ A6-256 T2 ; SFP013
mv" xxxxxxxx:0000xxxx:xxxxxxxx:1xxxxxxx %001111 T2Rd8Rm0rot2     sxth " \ A6-256 T2 ; SFP013
mv" xxxxxxxx:0001xxxx:xxxxxxxx:1xxxxxxx %001011 T2Rd8Rn16Rm0rot2 uxtah " \ A6-256 T2 ; SFP013
mv" xxxxxxxx:0001xxxx:xxxxxxxx:1xxxxxxx %001111 T2Rd8Rm0rot2     uxth " \ A6-274 T2 ; SFP013
mv" xxxxxxxx:0010xxxx:xxxxxxxx:1xxxxxxx %001011 T2Rd8Rn16Rm0rot2 sxtab16 " \ A7-507 ; SFP013
mv" xxxxxxxx:0010xxxx:xxxxxxxx:1xxxxxxx %001111 T2Rd8Rm0rot2     sxtb16 " \ A7-513 ; SFP013
mv" xxxxxxxx:0011xxxx:xxxxxxxx:1xxxxxxx %001011 T2Rd8Rn16Rm0rot2 uxtab16 " \ A7-555 ; SFP013
mv" xxxxxxxx:0011xxxx:xxxxxxxx:1xxxxxxx %001111 T2Rd8Rm0rot2     uxtb16 " \ A7-561 ; SFP013
mv" xxxxxxxx:0100xxxx:xxxxxxxx:1xxxxxxx %001011 T2Rd8Rn16Rm0rot2 sxtab " \ A7-505 ; SFP013
mv" xxxxxxxx:0100xxxx:xxxxxxxx:1xxxxxxx %001111 T2Rd8Rm0rot2     sxtb " \ A6-254 T2
mv" xxxxxxxx:0101xxxx:xxxxxxxx:1xxxxxxx %001011 T2Rd8Rn16Rm0rot2 uxtab " \ A7-553 ; SFP013
mv" xxxxxxxx:0101xxxx:xxxxxxxx:1xxxxxxx %001111 T2Rd8Rm0rot2     uxtb " \ A6-274 T2
mv" xxxxxxxx:1xxxxxxx:xxxxxxxx:00xxxxxx %000011 doPadd/sub       Padd/sub " \ 5-182 ; SFP013
mv" xxxxxxxx:1xxxxxxx:xxxxxxxx:01xxxxxx %000011 doPadd/subU      Padd/subU " \ 5-183 ; SFP013
\ A5-27, DDI0403D
mv" xxxxxxxx:1000xxxx:xxxxxxxx:1000xxxx %000011 T2Rd8Rm0Rn16     qadd "  \ A7-391
mv" xxxxxxxx:1000xxxx:xxxxxxxx:1001xxxx %000011 T2Rd8Rm0Rn16     qdadd "  \ A7-395
mv" xxxxxxxx:1000xxxx:xxxxxxxx:1010xxxx %000011 T2Rd8Rm0Rn16     qsub "  \ A7-398
mv" xxxxxxxx:1000xxxx:xxxxxxxx:1011xxxx %000011 T2Rd8Rm0Rn16     qdsub "  \ A7-396
mv" xxxxxxxx:1010xxxx:xxxxxxxx:1000xxxx %000011 T2Rd8Rn16Rm0     sel "  \ A7-425
\ A5-29
mv" xxxxxxxx:1001xxxx:xxxxxxxx:1000xxxx %000011 T2Rd8Rm0         rev "  \ A6-191 T2
mv" xxxxxxxx:1001xxxx:xxxxxxxx:1001xxxx %000011 T2Rd8Rm0         rev16 "  \ A6-192 T2
mv" xxxxxxxx:1001xxxx:xxxxxxxx:1010xxxx %000011 T2Rd8Rm0         rbit "  \ A6-190 T1
mv" xxxxxxxx:1001xxxx:xxxxxxxx:1011xxxx %000011 T2Rd8Rm0         revsh "  \ A6-193 T2
mv" xxxxxxxx:1011xxxx:xxxxxxxx:1000xxxx %000011 T2Rd8Rm0         clz "  \ A6-57 T1
;mv



\ =========
\ Load word
\ =========
\ A5-22

mv: T2ldw	\ -- ; A5-22
\   mask/value------------------------- test    xt                  name "
\   xxxxxxxI:Ixxxxxxx:xxxxIIII:IIxxxxxx  RdRnRt
mv" xxxxxxx0:1xxxxxxx:xxxxxxxx:xxxxxxxx %001000 T2memRt12Rn16Imm12  ldr "  \ A6-88 T3
mv" xxxxxxx0:0xxxxxxx:xxxx1xx1:xxxxxxxx %001000 T2memRt12Rn16Imm8   ldr "  \ A6-88 T4
mv" xxxxxxx0:0xxxxxxx:xxxx1100:xxxxxxxx %001000 T2memRt12Rn16Imm8   ldr "  \ A6-88 T4
\ ---
mv" xxxxxxx0:0xxxxxxx:xxxx1110:xxxxxxxx %001000 T2memRt12Rn16Imm8   ldrt " \ A6-133 T1
\ ---
mv" xxxxxxx0:0xxxxxxx:xxxx0000:00xxxxxx %001000 T2memRt12Rn16Rm0lsl ldr "  \ A6-92 T2
\ ---
mv" xxxxxxx0:xxxxxxxx:xxxxxxxx:xxxxxxxx %001100 T2memRt12PClit12    ldr "  \ A6-90 T2
;mv


\ =======================================
\ Load halfword, unallocated memory hints
\ =======================================

mv: T2ldh/memhints	\ -- ; A5-23
\   mask/value------------------------- test  xt                  name "
mv" xxxxxxx0:1xxxxxxx:xxxxxxxx:xxxxxxxx %1010 T2memRt12Rn16Imm12  ldrh "  \ A6-110 T2
mv" xxxxxxx0:0xxxxxxx:xxxx1xx1:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrh "  \ A6-110 T3
mv" xxxxxxx0:0xxxxxxx:xxxx1100:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrh "  \ A6-110 T3
\ ---
mv" xxxxxxx0:0xxxxxxx:xxxx1110:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrht " \ A6-116 T1
\ ---
mv" xxxxxxx0:xxxxxxxx:xxxxxxxx:xxxxxxxx %1110 T2memRt12PClit12    ldrh "  \ A6-112 T1
\ ---
mv" xxxxxxx0:0xxxxxxx:xxxx0000:00xxxxxx %1010 T2memRt12Rn16Rm0lsl ldrh "  \ A6-114 T2
\ --
mv" xxxxxxx1:1xxxxxxx:xxxxxxxx:xxxxxxxx %1010 T2memRt12Rn16Imm12  ldrsh " \ A6-126 T1
mv" xxxxxxx1:0xxxxxxx:xxxx1xx1:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrsh " \ A6-126 T2
mv" xxxxxxx1:0xxxxxxx:xxxx1100:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrsh " \ A6-126 T2
\ ---
mv" xxxxxxx1:0xxxxxxx:xxxx1110:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrsht " \ A6-132 T1
\ ---
mv" xxxxxxx1:xxxxxxxx:xxxxxxxx:xxxxxxxx %1110 T2memRt12PClit12    ldrsh " \ A6-128 T1
\ ---
mv" xxxxxxx1:0xxxxxxx:xxxx0000:00xxxxxx %1010 T2memRt12Rn16Rm0lsl ldrsh " \ A6-130 T2
\ ---
mv" xxxxxxx0:1x1xxxxx:xxxxxxxx:xxxxxxxx %1011 T2memRn16Imm+12     pldw "  \ A6-176 T1
mv" xxxxxxx0:0x1xxxxx:xxxx1100:xxxxxxxx %1011 T2memRn16Imm-8      pldw "  \ A6-176 T2
mv" xxxxxxxx:xxxxxxxx:xxxxxxxx:xxxxxxxx %0011 T20op UnallocHint "
;mv


\ =======================
\ Load byte, memory hints
\ =======================

mv: T2ldb/memhints	\ -- ; A5-24
\   mask/value------------------------- test  xt                  name "
mv" xxxxxxx0:1xxxxxxx:xxxxxxxx:xxxxxxxx %1010 T2memRt12Rn16Imm12  ldrb "  \ A6-94 T2
mv" xxxxxxx0:0xxxxxxx:xxxx1xx1:xxxxxxxx %1000 T2memRt12Rn16Imm8   ldrb "  \ A6-94 T3
mv" xxxxxxx0:0xxxxxxx:xxxx1100:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrb "  \ A6-94 T3
\ ---
mv" xxxxxxx0:0xxxxxxx:xxxx1110:xxxxxxxx %1000 T2memRt12Rn16Imm8   ldrbt " \ A6-100 T1
\ ---
mv" xxxxxxx0:xxxxxxxx:xxxxxxxx:xxxxxxxx %1110 T2memRt12PClit12    ldrb "  \ A6-96 T1
\ ---
mv" xxxxxxx0:0xxxxxxx:xxxx0000:00xxxxxx %1010 T2memRt12Rn16Rm0lsl ldrb "  \ A6-98 T2
\ ---
mv" xxxxxxx1:1xxxxxxx:xxxxxxxx:xxxxxxxx %1010 T2memRt12Rn16Imm12  ldrsb " \ A6-118 T1
mv" xxxxxxx1:0xxxxxxx:xxxx1xx1:xxxxxxxx %1000 T2memRt12Rn16Imm8   ldrsb " \ A6-118 T2
mv" xxxxxxx1:0xxxxxxx:xxxx1100:xxxxxxxx %1010 T2memRt12Rn16Imm8   ldrsb " \ A6-118 T2
\ ---
mv" xxxxxxx1:0xxxxxxx:xxxx1110:xxxxxxxx %1000 T2memRt12Rn16Imm8   ldrsbt " \ A6-124 T1
\ ---
mv" xxxxxxx1:xxxxxxxx:xxxxxxxx:xxxxxxxx %1110 T2memRt12PClit12    ldrsb "  \ A6-120 T1
\ ---
mv" xxxxxxx1:0xxxxxxx:xxxx0000:00xxxxxx %1010 T2memRt12Rn16Rm0lsl ldrsb "  \ A6-122 T2
\ --- PLD, PLDW
mv" xxxxxxx0:1x0xxxxx:xxxxxxxx:xxxxxxxx %1011 T2memRn16Imm+12     pld "  \ A6-176 T1
mv" xxxxxxx0:1x1xxxxx:xxxxxxxx:xxxxxxxx %1011 T2memRn16Imm+12     pldw "  \ A6-176 T1
mv" xxxxxxx0:0x0xxxxx:xxxx1100:xxxxxxxx %1011 T2memRn16Imm-8      pld "  \ A6-176 T2
mv" xxxxxxx0:0x1xxxxx:xxxx1100:xxxxxxxx %1011 T2memRn16Imm-8      pldw "  \ A6-176 T2
mv" xxxxxxx0:xxxxxxxx:xxxxxxxx:xxxxxxxx %1111 T2memPClit          pld "  \ A6-176 T2
\ ---
mv" xxxxxxx0:0xxxxxxx:xxxx0000:00xxxxxx %1011 T2memRn16Rm0lsl     pld "  \ A6-180 T2
\ ---
mv" xxxxxxx1:1xxxxxxx:xxxxxxxx:xxxxxxxx %1011 T2memRn16Imm+12     pli "  \ A6-182 T1
mv" xxxxxxx1:0xxxxxxx:xxxx1100:xxxxxxxx %1011 T2memRn16Imm-8      pli "  \ A6-182 T2
mv" xxxxxxx1:xxxxxxxx:xxxxxxxx:xxxxxxxx %1111 T2memPClit          pli "  \ A6-182 T3
\ ---
mv" xxxxxxx1:0xxxxxxx:xxxx0000:00xxxxxx %1011 T2memRn16Rm0lsl     pli "  \ A6-180 T2
;mv


\ ======================
\ Store single data item
\ ======================
\ A5-25

mv: T2storesingle	\ -- ; A5-25
\   mask/value------------------------- test    xt                  name "
\   xxxxxxxx:IIIxxxxx:xxxxIxxx:xxxxxxxx  RdRnRt
mv" xxxxxxxx:100xxxxx:xxxxxxxx:xxxxxxxx %001000 T2memRt12Rn16Imm12  strb "
mv" xxxxxxxx:000xxxxx:xxxx1110:xxxxxxxx %001000 T2memRt12Rn16Imm8   strbt "
mv" xxxxxxxx:000xxxxx:xxxx1xxx:xxxxxxxx %001000 T2memRt12Rn16Imm8   strb "
\ ---
mv" xxxxxxxx:000xxxxx:xxxx0xxx:xxxxxxxx %001000 T2memRt12Rn16Rm0lsl strb "
\ ---
mv" xxxxxxxx:101xxxxx:xxxxxxxx:xxxxxxxx %001000 T2memRt12Rn16Imm12  strh "
mv" xxxxxxxx:001xxxxx:xxxx1110:xxxxxxxx %001000 T2memRt12Rn16Imm8   strht "
mv" xxxxxxxx:001xxxxx:xxxx1xxx:xxxxxxxx %001000 T2memRt12Rn16Imm8   strh "
\ ---
mv" xxxxxxxx:001xxxxx:xxxx0xxx:xxxxxxxx %001000 T2memRt12Rn16Rm0lsl strh "
\ ---
mv" xxxxxxxx:110xxxxx:xxxxxxxx:xxxxxxxx %001000 T2memRt12Rn16Imm12  str "
mv" xxxxxxxx:010xxxxx:xxxx1110:xxxxxxxx %001000 T2memRt12Rn16Imm8   strt "
mv" xxxxxxxx:010xxxxx:xxxx1xxx:xxxxxxxx %001000 T2memRt12Rn16Imm8   str "
\ ---
mv" xxxxxxxx:010xxxxx:xxxx0xxx:xxxxxxxx %001000 T2memRt12Rn16Rm0lsl str "
;mv


\ ==========================
\ Branches and misc. control
\ ==========================
\ page A5-18

: T2miscctrl	\ -- ; A5-19
  case  curr-instr @ 4 rshift $0F and	\ bits 7:4
    $02 of  c" clrex " $>dis  endof	\ A6-56
    $04 of  c" dsb " opt0  endof	\ A6-70
    $05 of  c" dmb " opt0  endof	\ A6-68
    $06 of  c" isb " opt0  endof	\ A6-76
      Undef
  endcase
;

: T2hintins	\ -- ; A5-19
  curr-instr @ $0700 and		\ bits 10:8 not 000
  if  Undef exit  endif
  curr-instr @ $00F0 and $00F0 =	\ bits 7:4 = '1111'
  if  c" dbg " opt0 exit  endif
  case  curr-instr @ $0FF and		\ bits 7:0
    $00 of  c" nop " $>dis  endof	\ A6-167, enc T2
    $01 of  c" yield " $>dis  endof	\ A6-278, enc T2
    $02 of  c" wfe " $>dis  endof	\ A6-276, enc T2
    $03 of  c" wfi " $>dis  endof	\ A6-277, enc T2
    $04 of  c" sev " $>dis  endof	\ A6-212, enc T2
      Undef
  endcase
;

: T2msrreg	\ --
  c" msr " $>dis  T2SysReg ,>dis  Rn16  ;

: T2mrs		\ --
  c" mrs " $>dis  Rd8, T2SysReg  ;

: T2cbr21	\ --
\ Page A6-40 encoding T3, conditional branches with a 21 bit offset.
\ The last bit of 21 is an implicit 0. Offset=S:J2:J1:imm6:imm11:'0'.
  c" b " $>dis  T2cond22 .condCode
  formoff21 T2insbase +	4 +		\ dest address
  .BranchTarget
;

: T2op2=0x0	\ --
  curr-instr @ $07F0:7000 and $07F0:2000 =
  if  Undef exit  endif			\ permanently UNDEFINED
  curr-instr @ $0380:0000 and $0380:0000 <>
  if  T2cbr21 exit  endif		\ A6-40 enc T3
  case  curr-instr @ #20 rshift $7F and	\ Bits 26:20
    %0111000 of  T2msrreg  endof	\ A6-159
    %0111001 of  T2msrreg  endof
    %0111010 of  T2hintins  endof	\ A5-19
    %0111011 of  T2miscctrl  endof	\ A5-19
    %0111110 of  T2mrs  endof		\ A6-158
    %0111111 of  T2mrs  endof		\ A6-158
      Undef
  endcase
;

: destOff25	\ --
\ Page A6-40 encoding T4, branches with a 25 bit offset.
\ Page A6-49 encoding T1, branch/link with a 25 bit offset.
  T2insBase >callDestT2
  dup .imm ( drop ) ShowBranchTarget
;

: T2branch	\ --
\ Page A6-40 encoding T4, branches with a 25 bit offset.
  c" b " $>dis  destOff25  ;

: T2bl		\ --
\ Page A6-49 encoding T1, branch/link with a 25 bit offset.
  c" bl " $>dis  destOff25  ;

: T2blx		\ --
  c" blx " $>dis  destOff25  ;

: T2br/miscctrl	\ --
  case  curr-instr @ #12 rshift 7 and	\ look at op2
   %000 of  T2op2=0x0  endof
   %010 of  T2op2=0x0  endof
   %001 of  T2branch  endof		\ Enc T4, page A6-40
   %011 of  T2branch  endof		\ Enc T4, page A6-40
   %101 of  T2bl  endof			\ Enc T1, page A6-49
   %111 of  T2bl  endof			\ Enc T1, page A6-49
   %110 of  T2blx  endof		\ Enc T1, page A6-49
   %110 of  T2blx  endof		\ Enc T1, page A6-49
     Undef
  endcase
;


\ =================================
\ Data processing - plain immediate
\ =================================
\ page A5-17

: T2opRd8Lab138	\ $ flag -- ; 1=add,0=subtract
  swap $>dis
  Rd8, form138 swap 0= ?negate		\ offset
  T2insBase 4 + -4 and +		\ dest = Align(PC,4)+offset
  dup .imm  ( drop ) ShowBranchTarget	\ display
;

: T2dppi00000	\ --
  Rn16<>PC? if
    c" add " T2opRd8Rn16Imm138
  else
    c" adr " 1 T2opRd8Lab138
  endif
;

: T2dppi01010	\ --
  Rn16<>PC? if
    c" sub " T2opRd8Rn16Imm138
  else
    c" adr " 0 T2opRd8Lab138
  endif
;

: bfxLsb/Wid	\ --
  GetImm5 dup .imm ,>dis  bits4:0	\ -- lsbit msbit
  swap - 1+ .imm			\ width=msbit-lsbit+1
;

: T2bfi		\ --
  c" bfi " $>dis  Rd8, Rn16, bfxLsb/Wid  ;

: T2bfc		\ --
  c" bfc " $>dis  Rd8, bfxLsb/Wid  ;

: T2dpplimm	\ --
  case  curr-instr @ #20 rshift %11111 and
    %00000 of  T2dppi00000  endof
    %00100 of  c" mov " T2opRd8Imm16  endof
    %01010 of  T2dppi01010  endof
    %01100 of  c" movt " T2opRd8Imm16  endof
    %10000 of  c" ssat " T2xsat  endof
    %10010 of
      curr-instr @ $70C0 and if		\ 14:12, 7:6 non-zero
        c" ssat " T2xsat
      else
        c" ssat16 " T2xsat16
      endif
    endof
    %10100 of  c" sbfx " T2xbfx  endof
    %10110 of  Rn16<>PC? if T2bfi else T2bfc endif  endof
    %11000 of  c" usat " T2xsat  endof
    %11010 of
      curr-instr @ $70C0 and if		\ 14:12, 7:6 non-zero
        c" usat " T2xsat
      else
        c" usat16 " T2xsat16
      endif
    endof
    %11100 of  c" ubfx " T2xbfx  endof
      Undef
  endcase
;


\ ====================================
\ Data processing - modified immediate
\ ====================================
\ page A5-14

mv: T2dpmodimm	\ -- ; A5-14
\   mask/value------------------------- test    xt                  name "
\   xxxxxxxI:IIIIxxxx:xxxxxxxx:xxxxxxxx  RdRnRt
mv" xxxxxxx0:000xxxxx:xxxxxxxx:xxxxxxxx %100000 T2SRd8Rn16Const138  and "  \ A6-32
mv" xxxxxxx0:000xxxxx:xxxxxxxx:xxxxxxxx %110000 T2cmpRn16Const138   tst "  \ A6-262
mv" xxxxxxx0:001xxxxx:xxxxxxxx:xxxxxxxx %000000 T2SRd8Rn16Const138  bic "  \ A6-44
mv" xxxxxxx0:010xxxxx:xxxxxxxx:xxxxxxxx %001000 T2SRd8Rn16Const138  orr "  \ A6-172 T1
mv" xxxxxxx0:010xxxxx:xxxxxxxx:xxxxxxxx %001100 T2SopRd8Const138    mov "  \ A6-148 T2
mv" xxxxxxx0:011xxxxx:xxxxxxxx:xxxxxxxx %001000 T2SRd8Rn16Const138  orn "  \ A6-168 T1
mv" xxxxxxx0:011xxxxx:xxxxxxxx:xxxxxxxx %001100 T2SopRd8Const138    mvn "  \ A6-162 T1
mv" xxxxxxx0:100xxxxx:xxxxxxxx:xxxxxxxx %100000 T2SRd8Rn16Const138  eor "  \ A6-72 T1
mv" xxxxxxx0:100xxxxx:xxxxxxxx:xxxxxxxx %110000 T2cmpRn16Const138   teq "  \ A6-260 T1
mv" xxxxxxx1:000xxxxx:xxxxxxxx:xxxxxxxx %100000 T2SRd8Rn16Const138  add "  \ A6-22 T3
mv" xxxxxxx1:000xxxxx:xxxxxxxx:xxxxxxxx %110000 T2cmpRn16Const138   cmn "  \ A6-58 T1
mv" xxxxxxx1:010xxxxx:xxxxxxxx:xxxxxxxx %000000 T2SRd8Rn16Const138  adc "  \ A6-18 T1
mv" xxxxxxx1:011xxxxx:xxxxxxxx:xxxxxxxx %000000 T2SRd8Rn16Const138  sbc "  \ A6-204 T1
mv" xxxxxxx1:101xxxxx:xxxxxxxx:xxxxxxxx %100000 T2SRd8Rn16Const138  sub "  \ A6-244 T3
mv" xxxxxxx1:101xxxxx:xxxxxxxx:xxxxxxxx %110000 T2cmpRn16Const138   cmp "  \ A6-62 T2
mv" xxxxxxx1:110xxxxx:xxxxxxxx:xxxxxxxx %000000 T2SRd8Rn16Const138  rsb "  \ A6-200 T2
;mv


\ ====================
\ CDP instructions - 1
\ ====================

: opc7:5	\ --
  curr-instr @ 5 rshift $07 and $num>dis bl>dis
;

: opc7:4,	\ --
  curr-instr @ 4 rshift $0F and $num>dis ,>dis
;

: opc23:21,	\ --
  curr-instr @ #21 rshift $07 and $num>dis ,>dis
;

: opc23:20,	\ --
  curr-instr @ #20 rshift $0F and $num>dis ,>dis
;

: T2cp2regs	\ $ --
  $>dis  cp8#, opc7:4, Rt12, Rn16, CRm0  ;

: T2ldc/stc	\ $ --
  $>dis cp8#, Crd12,
  curr-instr @ $01A0:0000 and		\ PUW bits
  $0080:0000 = if			\ 010 is for option
    [Rn16]  c" , {} " $>dis  T2Imm8 num>dis
  else
    T2Imm8*4 +/-U Rn16Ind+off
  endif
;

: T2cdpx	\ $ --
  $>dis  cp8#, opc23:20, Crd12, Crn16, Crm0, opc7:5  ;

: T2mcrx/mrcx	\ --
  $>dis cp8#, opc23:21, Rt12, CRn16, CRm0, opc7:5  ;

mv: T2cdpins	\ -- ; A5-32
\   mask/value------------------------- test    xt              name "
\   xxxxxxII:IIIIxxxx:xxxxxxxx:xxxIxxxx  RdRnRt
mv" xxx0xx00:0100xxxx:xxxxxxxx:xxxxxxxx %000000	T2cp2regs	mcrr "
mv" xxx1xx00:0100xxxx:xxxxxxxx:xxxxxxxx %000000	T2cp2regs	mcrr2 "
mv" xxx0xx00:0101xxxx:xxxxxxxx:xxxxxxxx %000000	T2cp2regs	mrrc "
mv" xxx1xx00:0101xxxx:xxxxxxxx:xxxxxxxx %000000	T2cp2regs	mrrc2 "
mv" xxxxxx00:0x0xxxxx:xxxxxxxx:xxxxxxxx %000000	$>dis		Undefined "

mv" xxx0xx0x:x0x0xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	stc "		\ A6-216
mv" xxx0xx0x:x1x0xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	stcl "
mv" xxx1xx0x:x0x0xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	stc2 "
mv" xxx1xx0x:x1x0xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	stc2l "

mv" xxx0xx0x:x0x1xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	ldc "		\ A6-80
mv" xxx0xx0x:x1x1xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	ldcl "
mv" xxx1xx0x:x0x1xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	ldc2 "
mv" xxx1xx0x:x1x1xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldc/stc	ldc2l "

mv" xxx0xx10:xxxxxxxx:xxxxxxxx:xxx0xxxx %000000	T2cdpx		cdp "		\ A6-54
mv" xxx1xx10:xxxxxxxx:xxxxxxxx:xxx0xxxx %000000	T2cdpx		cdp2 "

mv" xxx0xx10:xxx0xxxx:xxxxxxxx:xxx1xxxx %000000	T2mcrx/mrcx	mcr "		\ A6-142
mv" xxx1xx10:xxx0xxxx:xxxxxxxx:xxx1xxxx %000000	T2mcrx/mrcx	mcr2 "
mv" xxx0xx10:xxx1xxxx:xxxxxxxx:xxx1xxxx %000000	T2mcrx/mrcx	mrc "		\ A6-154
mv" xxx1xx10:xxx1xxxx:xxxxxxxx:xxx1xxxx %000000	T2mcrx/mrcx	mrc2 "
;mv


\ ==================================
\ Data processing - shifted register
\ ==================================
\ A5-26

mv: T2dp/shift	\ -- ; A5-26
\   mask/value------------------------- test    xt                 name "
\   xxxxxxxI:IIIxxxxx:xxxxxxxx:xxxxxxxx  RdRnRt
mv" xxxxxxx0:000xxxxx:xxxxxxxx:xxxxxxxx %100000	T2SRd8Rn16Rm0sh5   and "	\ A6-32 T2
mv" xxxxxxx0:0001xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    tst "	\ A6-264 T2
mv" xxxxxxx0:0000xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    tst (unpred) "
mv" xxxxxxx0:001xxxxx:xxxxxxxx:xxxxxxxx %000000	T2SRd8Rn16Rm0sh5   bic "	\ A6-46 T2
mv" xxxxxxx0:010xxxxx:xxxxxxxx:xxxxxxxx %001000	T2SRd8Rn16Rm0sh5   orr "	\ A6-174
mv" xxxxxxx0:010xxxxx:x000xxxx:0000xxxx %001100	T2opSRd8Rm0sh5     mov "	\ A6-150 T3
mv" xxxxxxx0:010xxxxx:xxxxxxxx:xx00xxxx %001100	T2shiftRd8Rm0imm5a lsl "	\ A6-134 T2
mv" xxxxxxx0:010xxxxx:xxxxxxxx:xx01xxxx %001100	T2shiftRd8Rm0imm5b lsr "	\ A6-134 T2
mv" xxxxxxx0:010xxxxx:xxxxxxxx:xx10xxxx %001100	T2shiftRd8Rm0imm5b asr "	\ A6-134 T2
mv" xxxxxxx0:010xxxxx:x000xxxx:0011xxxx %001100	T2opSRd8Rm0        rrx "	\ A6-198 T1
mv" xxxxxxx0:010xxxxx:xxxxxxxx:xx11xxxx %001100	T2shiftRd8Rm0imm5a ror "	\ A6-194 T1
mv" xxxxxxx0:011xxxxx:xxxxxxxx:xxxxxxxx %001000	T2SRd8Rn16Rm0sh5   orn "	\ A6-170 T1
mv" xxxxxxx0:011xxxxx:xxxxxxxx:xxxxxxxx %001100	T2opSRd8Rm0sh5     mvn "	\ A6-164 T2
mv" xxxxxxx0:100xxxxx:xxxxxxxx:xxxxxxxx %100000	T2SRd8Rn16Rm0sh5   eor "	\ A6-74 T2
mv" xxxxxxx0:1001xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    teq "	\ A6-261 T1
mv" xxxxxxx0:1000xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    teq (unpred)"
mv" xxxxxxx0:1100xxxx:xxxxxxxx:xx00xxxx %101010	T2SRd8Rn16Rm0sh5   pkhbt "	\ SFP013
mv" xxxxxxx0:1100xxxx:xxxxxxxx:xx10xxxx %101010	T2SRd8Rn16Rm0sh5   pkhtb "	\ SFP013
mv" xxxxxxx1:000xxxxx:xxxxxxxx:xxxxxxxx %100000	T2SRd8Rn16Rm0sh5   add "	\ A6-24 T3
mv" xxxxxxx1:0001xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    cmn "	\ A6-60 T2
mv" xxxxxxx1:0000xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    cmn (unpred)"
mv" xxxxxxx1:010xxxxx:xxxxxxxx:xxxxxxxx %000000	T2SRd8Rn16Rm0sh5   adc "	\ A6-20 T2
mv" xxxxxxx1:011xxxxx:xxxxxxxx:xxxxxxxx %000000	T2SRd8Rn16Rm0sh5   sbc "	\ A6-206 T2
mv" xxxxxxx1:101xxxxx:xxxxxxxx:xxxxxxxx %100000	T2SRd8Rn16Rm0sh5   sub "	\ A6-246 T2
mv" xxxxxxx1:1011xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    cmp "	\ A6-64 T3
mv" xxxxxxx1:1010xxxx:xxxxxxxx:xxxxxxxx %110000	T2cmpRn16Rm0sh5    cmp (unpred)"
mv" xxxxxxx1:110xxxxx:xxxxxxxx:xxxxxxxx %000000	T2SRd8Rn16Rm0sh5   rsb "	\ A6-202 T1
;mv

\ PKHBT, PKHTB  Rd8, Rn16, Rm0 [ shift 5 ]
\ =============================
\ Dual and exclusive operations
\ =============================
\ A5-21

: T2strex	\ $ --
  $>dis  Rd8, Rt12, T2Imm8*4 [Rn16+off]  ;

: T2ldrex	\ $ --
  $>dis  Rt12, T2Imm8*4 [Rn16+off]  Rd8=PC  ;

: T2strdImm	\ $ --
  $>dis  Rt12, Rd8, T2Imm8*4 +/-U Rn16Ind+off
  Rn16=PC? if
    T2Imm8*4 +/-U .PCtarget
  endif
;

: T2strexx	\ $ --
  $>dis  Rd0, Rt12, [Rn16]  Rd8=PC  ;

: T2ldrexx	\ $ --
  $>dis  Rt12, [Rn16]  Rd8=PC Rd0=PC  ;

: T2tbb		\ $ --
  $>dis  [>dis  Rn16, Rm0  ]>dis  ;

: T2tbh		\ $ --
  $>dis  [>dis  Rn16, Rm0,  c" lsl # 1" $>dis  ]>dis
;

mv: T2dual/ex	\ -- ; A5-21
\   mask/value------------------------- test    xt              name "
\   xxxxxxxI:IxIIxxxx:xxxxxxxx:IIIIxxxx  RdRnRt
mv" xxxxxxx0:0x00xxxx:xxxxxxxx:xxxxxxxx %000000	T2strex		strex "		\ A6-234 T1
mv" xxxxxxx0:0x01xxxx:xxxxxxxx:xxxxxxxx %000000	T2ldrex		ldrex "		\ A6-106 T1
mv" xxxxxxx0:xx10xxxx:xxxxxxxx:xxxxxxxx %000000	T2strdImm	strd "		\ A6-232 T1
mv" xxxxxxx1:xxx0xxxx:xxxxxxxx:xxxxxxxx %000000	T2strdImm	strd "		\ A6-232 T1
mv" xxxxxxx0:xx11xxxx:xxxxxxxx:xxxxxxxx %000000	T2strdImm	ldrd "		\ A6-102 T1
mv" xxxxxxx1:xxx1xxxx:xxxxxxxx:xxxxxxxx %000000	T2strdImm	ldrd "		\ A6-104 T1
mv" xxxxxxx0:1x00xxxx:xxxxxxxx:0100xxxx %000000	T2strexx	strexb "	\ A6-235 T1
mv" xxxxxxx0:1x00xxxx:xxxxxxxx:0101xxxx %000000	T2strexx	strexh "	\ A6-236 T1
mv" xxxxxxx0:1x01xxxx:xxxxxxxx:0000xxxx %000000	T2tbb		tbb "		\ A6-258 T1
mv" xxxxxxx0:1x01xxxx:xxxxxxxx:0001xxxx %000000	T2tbh		tbh "		\ A6-258 T1
mv" xxxxxxx0:1x01xxxx:xxxxxxxx:0100xxxx %000000	T2ldrexx	ldrexb "	\ A6-107 T1
mv" xxxxxxx0:1x01xxxx:xxxxxxxx:0101xxxx %000000	T2ldrexx	ldrexh "	\ A6-108 T1
;mv


\ =======
\ LDM/STM
\ =======

: .T2RegMask	\ mask --
\ +G Display 16 bit register mask.
  c" { " $>dis
  16 0 do
    dup 1 and
    if  i .reg bl>dis  endif
    1 rshift
  loop
  drop  c" }" $>dis
;

: WbR13?	\ -- flag
\ True if Rn=13 and W=1.
  curr-instr @ $002F:0000 and $002D:0000 =
;

: T2push/pop	\ op$ --
  $op> .T2RegMask
;

: T2ldmx/stmx	\ op$ --
  $op> dup #16 rshift .reg wb21, .T2RegMask
;

: T2stmia	\ --
  c" stm " $op> dup #16 rshift .reg wb21, .T2RegMask
;

: T2pop/ldmia	\ --
  WbR13? if
    c" pop " T2push/pop
  else
    c" ldm " T2ldmx/stmx
  endif
;

: T2push/stmdb	\ --
  WbR13? if
    c" push " T2push/pop
  else
    c" stmdb " T2ldmx/stmx
  endif
;


: T2ldm/stm	\ --
\ Decode LDMxx, STMxx, POP, PUSH from page A5-20
  case  curr-instr @ $0190:0000 and
   $0080:0000 of  c" stm " T2ldmx/stmx  endof
   $0090:0000 of  T2pop/ldmia  endof
   $0100:0000 of  T2push/stmdb  endof
   $0110:0000 of  c" ldmdb " T2ldmx/stmx  endof
     Undef
  endcase
;


\ ==============
\ Primary Decode
\ ==============
\ A5-13

: >pT2m,	\ op1 op2 op -- mask
\ Convert op1 op2 and op to a mask and comma it in. This just
\ makes translation of table A5-9 (Cortex-M) easier. See A6-9
\ for Cortex-A/R.
  #15 lshift  		\ op:  bit 0 -> bit 15
  swap #20 lshift or	\ op2: bit 0 -> bit 20
  swap #27 lshift or	\ op1: bit 0 -> bit 27
  ,
;

create Thumb2List	\ -- addr
\ This table contains four items (cells) per entry. These items are
\ applied to the current 32 bit instruction.
\ 1) mask
\ 2) value after mask
\ 3) reserved (0)
\ 4) xt of action if value matches after masking.
\ Note that %x may be an FP number, so use : to prevent problems.
  %:11 %1100100 %:0 >pT2m,  %:01 %0000000 %:0 >pT2m,  0 ,  ' T2ldm/stm ,      \ A5-20
  %:11 %1100100 %:0 >pT2m,  %:01 %0000100 %:0 >pt2m,  0 ,  ' T2dual/ex ,      \ A5-21
  %:11 %1100000 %:0 >pT2m,  %:01 %0100000 %:0 >pT2m,  0 ,  ' T2dp/shift ,     \ A5-26
  %:11 %1000000 %:0 >pT2m,  %:01 %1000000 %:0 >pT2m,  0 ,  ' T2cdpins ,       \ A5-32

  %:11 %0100000 %:1 >pT2m,  %:10 %0000000 %:0 >pT2m,  0 ,  ' T2dpmodimm ,     \ A5-14
  %:11 %0100000 %:1 >pT2m,  %:10 %0100000 %:0 >pT2m,  0 ,  ' T2dpplimm ,      \ A5-17
  %:11 %0000000 %:1 >pT2m,  %:10 %0000000 %:1 >pT2m,  0 ,  ' T2br/miscctrl ,  \ A5-18
  %:11 %1110001 %:0 >pT2m,  %:11 %0000000 %:0 >pT2m,  0 ,  ' T2storesingle ,  \ A5-25
  %:11 %1100111 %:0 >pT2m,  %:11 %0000001 %:0 >pT2m,  0 ,  ' T2ldb/memhints , \ A5-24
  %:11 %1100111 %:0 >pT2m,  %:11 %0000011 %:0 >pT2m,  0 ,  ' T2ldh/memhints , \ A5-23
  %:11 %1100111 %:0 >pT2m,  %:11 %0000101 %:0 >pT2m,  0 ,  ' T2ldw ,          \ A5-22
  %:11 %1100111 %:0 >pT2m,  %:11 %0000111 %:0 >pT2m,  0 ,  ' Undef ,
\  %:11 %1110001 %:0 >pT2m,  %:11 %0010000 %:0 >pT2m,  0 ,  ' T2ASIMDld/st     \ A/R A7-7 SFPxxx
  %:11 %1110000 %:0 >pT2m,  %:11 %0100000 %:0 >pT2m,  0 ,  ' T2dpr ,          \ A5-28
  %:11 %1111000 %:0 >pT2m,  %:11 %0110000 %:0 >pT2m,  0 ,  ' T2mul/mulacc ,   \ A5-30
  %:11 %1111000 %:0 >pT2m,  %:11 %0111000 %:0 >pT2m,  0 ,  ' T2lmul/div ,     \ A5-31
  %:11 %1000000 %:0 >pT2m,  %:11 %1000000 %:0 >pT2m,  0 ,  ' T2cdpins ,       \ A5-32
  0 ,  0 ,  0 ,  0 ,

: disThumb32	\ --
\ Primary Thumb2 decode, implementing table A5-9 on page A5-13.
  Thumb2List begin
    dup t2i.xt @			\ xt=0=end
   while				\ -- struct
    dup t2i.mask @ curr-instr @ and	\ apply mask
    over t2i.value @ = if		\ match to value?
      t2i.xt @ execute  exit
    endif
    4 cells +
  repeat
  drop Undef
;


\ *****************
\ Thumb1 and Thumb2
\ *****************

0 value ShowOpcodes?	\ -- x
\ Non-zero to display opcodes.

also cc/i
: +ShowOpcodes	cc/i	\ --
  1 to ShowOpcodes?  ;
: -ShowOpcodes	cc/i	\ --
  0 to ShowOpcodes?  ;
previous definitions

: CheckT2	\ -- flag
\ Return true if the current instruction is a Thumb2 32 bit instruction.
  case curr-instr @ $F800 and
    $F800 of  true  endof
    $F000 of  true  endof
    $E800 of  true  endof
      false swap
  endcase
;

: makeIns32	\ --
\ Convert the current instruction into a 32 bit instruction.
  curr-instr @ #16 lshift @word or curr-instr !
;

: do-main_T	\ -- ; disassemble one Thumb instruction
  DisAddr to T2Insbase
  @word curr-instr !
  CheckT2 if
    makeIns32  ShowOpcodes?
    if  cr ." Instr=" curr-instr @ .dword  endif
    CheckAS/VFP? if  exit  endif
    disThumb32 exit
  endif
  ShowOpcodes?
  if  cr ." Instr=" curr-instr @ .word  endif
  decodeT11 if  exit  endif
  decodeT10 if  exit  endif
  decodeT8  if  exit  endif
  decodeT7  if  exit  endif
  decodeT6  if  exit  endif
  decodeT5  if  exit  endif
  decodeT4  if  exit  endif
  undef
;


\ *************
\ ASIMD and VFP - See DDI0406C
\ *************

variable ASIMD-ubit	\ -- addr

: SizeCode	\ -- size ; 0..3
  curr-instr @ #20 rshift $03 and	\ 0..3
;

: .dt21:20	\ -- ; display data size
  SizeCode 8 swap lshift dec>dis bl>dis
;

: .s/udt21:20	\ -- ; display integer data type
  ASIMD-ubit @
  if  c" .u"  else  c" .s"  endif
  $>dis  .dt21:20
;

: .idt21:20	\ -- ; display integer data type
  c" i" $>dis  .dt21:20
;

: .sdt21:20	\ -- ; display integer data type
  c" s" $>dis  .dt21:20
;

: MVreg@	\ -- reg#
  curr-instr @ dup $0F and		\ low four bits
  swap 1 rshift $010 and or 		\ plus M bit
;

: NVreg@	\ -- reg#
  curr-instr @ dup #16 rshift $0F and	\ low four bits
  swap 3 rshift $010 and or 		\ plus N bit
;

: DVreg@	\ -- reg#
  curr-instr @ dup #12 rshift $0F and	\ low four bits
  swap #18 rshift $010 and or		\ plus D bit
;

: .Q/Dreg	\ reg# --
  curr-instr @ $0000:0040 and
  if   1 rshift  c" Q"  else  c" D"  endif
  $>dis  dec>dis
;

: .QVreg	\ reg# --
   1 rshift  c" Q" $>dis  dec>dis  ;

: .DVreg	\ reg# --
  c" D" $>dis  dec>dis  ;

: .Am		\ -- ; display Am as Q/D from bit 6
  MVreg@ .Q/Dreg  ;

: .An		\ -- ; display An as Q/D from bit 6
  NVreg@ .Q/Dreg  ;

: .Ad		\ -- ; display Ad as Q/D from bit 6
  DVreg@ .Q/Dreg  ;

: .Qm		\ -- ; display Qm
  MVreg@ .QVreg  ;

: .Qn		\ -- ; display Qn
  NVreg@ .QVreg  ;

: .Qd		\ -- ; display Qd
  DVreg@ .QVreg  ;

: .Dm		\ -- ; display Dm
  MVreg@ .DVreg  ;

: .Dn		\ -- ; display Dn
  NVreg@ .DVreg  ;

: .Dd		\ -- ; display Dd
  DVreg@ .DVreg  ;

: MSreg@	\ -- reg#
  curr-instr @ dup $0F and 1 lshift	\ top four bits
  swap 5 rshift $001 and or 		\ plus M bit
;

: NSreg@	\ -- reg#
  curr-instr @ dup
  #16 rshift $0F and 1 lshift		\ top four bits
  swap 7 rshift $001 and or 		\ plus N bit
;

: DSreg@	\ -- reg#
  curr-instr @ dup
  #12 rshift $0F and 1 lshift		\ top four bits
  swap #22 rshift $001 and or		\ plus D bit
;

: .Sreg		\ reg# -- ; display Sx
  c" S" $>dis  dec>dis  ;

: .Sm		\ -- ; display Sm
  MSreg@ .Sreg  ;
: .Sn		\ -- ; display Sn
  NSreg@ .Sreg  ;
: .Sd		\ -- ; display Sd
  DSreg@ .Sreg  ;

: .Fm		\ -- ; display Fm
  curr-instr @ $0100 and
  if  .Dm  else  .Sm  endif
;
: .Fn		\ -- ; display Fm
  curr-instr @ $0100 and
  if  .Dn  else  .Sn  endif
;
: .Fd		\ -- ; display Fm
  curr-instr @ $0100 and
  if  .Dd  else  .Sd  endif
;

: .index	\ index --
  c" [" $>dis  dec>dis  c" ] " $>dis
;

: .Dm[x]1	\ --
  case SizeCode
    1 of  MVreg@ dup $07 and .DVreg  $18 and 3 rshift .index  endof
    2 of  MVreg@ dup $0F and .DVreg  $10 and 4 rshift .index  endof
     c" Invalid size code" $>dis
  endcase
;

: showVVV-s/u-q/d	\ $ --
  $>dis .s/udt21:20 .Ad ,>dis  .An ,>dis  .Am
;

: showVVV-q/d		\ $ --
  $>dis .Ad ,>dis  .An ,>dis .Am
;

: showVVV-idt-q/d	\ $ --
\ cr ." showVVV-idt-q/d" key? drop
  $>dis .idt21:20  .Ad ,>dis  .An ,>dis .Am
;

: showVVV-f32-q/d	\ $ --
\ cr ." showVVV-f32-q/d" key? drop
  $>dis  c" .f32 " $>dis  .Ad ,>dis  .An ,>dis  .Am
;

: showQDD-s/u		\ $ --
  $>dis .s/udt21:20 .Qd ,>dis  .Dn ,>dis  .Dm
;

: showQQD-s/u		\ $ --
  $>dis .s/udt21:20 .Qd ,>dis  .Qn ,>dis  .Dm
;

: showDQQ-idt		\ $ --
  $>dis .idt21:20  .Dd ,>dis  .Qn ,>dis .Qm
;

: showQDD[x]-idt	\ $ --
  $>dis  .idt21:20  .Qd ,>dis  .Dn ,>dis  .Dm[x]1
;

: showVEXT		\ --
  c" vext " $>dis  c"  .8 " $>dis .Ad ,>dis  .An ,>dis  .Am ,>dis
  c"  # #" $>dis  curr-instr @ 8 rshift $0F and dec>dis
;

: listlen9:8	\ -- len
  curr-instr @ 8 rshift 3 and 1+
;

: .DnList	\ -- ; taken from Vn and len9:8
  c"  { " $>dis
  NVreg@  listlen9:8 0
  do  .DVreg bl>dis  loop
  c" } " $>dis
;

: showVTBx	\ --
  curr-instr @ $0040 and
  if  c" vtbx .8 "  else  c" vtbl .8 "  endif
  $>dis  .Dd  .DnList  .Dm
;

: .imm4Size	\ -- index
  curr-instr @ 16 rshift $0F and
  dup $01 and $01 =
  if  c"  .8 " $>dis  1 rshift  exit  endif
  dup $03 and $02 =
  if  c"  .16 " $>dis  2 rshift  exit  endif
  dup $07 and $04 =
  if  c"  .32 " $>dis  2 rshift  exit  endif
  drop c"  BadImm4 format" $>dis  0
;

: showVDUPscalar	\ --
  c" vdup " $>dis  .imm4Size
  .Ad ,>dis .Dm .index
;

: size19:18	\ -- 8/16/32/64
  curr-instr @ #18 rshift $03 and	\ 0..3
  8 swap lshift
;

: (.dt19:18)	\ -- ; display data size
  size19:18 dec>dis bl>dis
;

: .dt19:18	\ -- ; display data size
  c"  ." $>dis  (.dt19:18)  ;

: .idt19:18	\ -- ; display data size
  c"  .i" $>dis  (.dt19:18)  ;

: .sdt19:18:7	\ -- ; display data size
  curr-instr @ $0080 and
  if  c"  .u"  else  c"  .s"  endif
  $>dis  (.dt19:18)
;

: .sfdt19:18	\ --
  curr-instr @ $0400 and if
    curr-instr @ #18 rshift $03 and 2 =
    if  c" .f32 "  else  c" bad size/float "  endif
  else
    curr-instr @ #18 rshift $03 and case
      0 of  c" .s8 "  endof
      1 of  c" .s16 "  endof
      2 of  c" .s32 "  endof
        c" bad size/float " swap
    endcase
  endif
  $>dis
;

: .ufdt19:18	\ --
  curr-instr @ $0400 and if
    curr-instr @ #18 rshift $03 and 2 =
    if  c" .f32 "  else  c" bad size/float "  endif
  else
    curr-instr @ #18 rshift $03 and case
      0 of  c" .u8 "  endof
      1 of  c" .u16 "  endof
      2 of  c" .u32"  endof
        c" bad size/float " swap
    endcase
  endif
  $>dis
;

: .sudt19:18:7:6	\ --
  case  curr-instr @ 6 rshift 3 and
    0 of  c"  .BADop "  endof
    1 of  c"  .s"  endof
    2 of  c"  .s"  endof
    3 of  c"  .u"  endof
  endcase  $>dis
  case  curr-instr @ #18 rshift 3 and
    0 of  c" 16 "  endof
    1 of  c" 32 "  endof
    2 of  c" 64 "  endof
    3 of  c" BadSize "  endof
  endcase $>dis
;

: showVV	\ $ --
  $>dis  .Ad ,>dis  .Am
;

: showVV-idt	\ $ --
  $>dis .dt19:18  .Ad ,>dis  .Am
;

: showVV-sdt	\ $ --
  $>dis .sdt19:18:7  .Ad ,>dis  .Am
;

: showVV-sfdt	\ $ --
  $>dis .sfdt19:18  .Ad ,>dis  .Am
;

: showVV#0-sfdt	\ $ --
  $>dis .sfdt19:18  .Ad ,>dis  .Am  c" , # 0" $>dis
;

: showDQ-idt	\ $ --
  $>dis .idt19:18  .Dd ,>dis  .Qm
;

: showDQ-sudt	\ $ --
  $>dis .sudt19:18:7:6  .Dd ,>dis  .Qm
;

: showQD#size	\ $ --
  $>dis  .idt19:18  .Qd ,>dis  .Dm  c" , # #" $>dis (.dt19:18)
;

: showVCVT-h/s	\ $ --
  $>dis  curr-instr @ $0100 and if	\ half to single
    c"  .F32.F16 " $>dis  .Qd ,>dis  .Dm
  else
    c"  .F16:F32 " $>dis  .Dd ,>dis  .Qm
  endif
;

: showVV-ufdt	\ $ --
  $>dis  .ufdt19:18  .Ad ,>dis  .Am
;

: .suf32	\ --
  case  curr-instr @ 7 rshift $3 and
    2 of  c" .s32.f32 "  endof
    3 of  c" .u32.f32 "  endof
    0 of  c" .f32.s32 "  endof
    1 of  c" .f32.u32 "  endof
  endcase
  $>dis
;

: showVV-suf	\ $ --
  $>dis  .suf32  .Ad ,>dis  .Am
;

: VModImm@	\ -- imm8
  curr-instr @
  dup $0F and				\ efgh
  over #12 rshift $0070 and or		\ bcdefgh
  Thumb? if
    swap #21 rshift $0080 and or	\ abcdefgh
  else
    swap #17 rshift $0080 and or	\ abcdefgh
  endif
;

: cmode11:8@	\ -- abcd
  curr-instr @ 12 rshift $0F and
;

: op5@		\ -- x
  curr-instr @ $0010 and 0<>  ;

: .decVMI	\ n --
  c" #" $>dis  dec>dis  ;

: (.hexVMI)	\ u --
  base @ >r  hex  num>dis  r> base !  ;

: .hexVMI	\ u --
  c" $" $>dis  (.hexVMI) ;

: .rep8VMI	\ u --
  c" $" $>dis  4 0
  do  dup (.hexVMI) loop
  drop
;

: .aaaaaaaa8VMI	\ u --
  c" $" $>dis  8 0 do
    dup $80 and
    if  c" FF"  else  c" 00"  endif  $>dis
    1 lshift
  loop
  drop
;

(( from assembler
also forth definitions
defer fdropa	\ F: f -- ; -- ; used when FP pack installed
defer fdupa	\ F: f -- f f ; -- ; used when FP pack installed
defer f0=a	\ F: f -- ; -- flag ; used when FP pack installed
defer sf!a	\ F: f -- ; -- ; used when FP pack installed
previous definitions
))
also forth definitions
defer sf@a	\ F: -- f ; addr -- ; used when FP pack installed
defer (f.)a	\ F: f -- ; u --
previous definitions

: VMI>f32	\ u -- ieee32
  dup $80 and #24 lshift	\ -- u a...
  over $40 and
  if  $3E00:0000 or  else  $4000:0000 or  endif	\ -- u aBbbbbb...
  swap $3F and #19 lshift or			\ -- aBbbbbbc:defgh000:...
;

: .fVMI		\ u8 --
  { | temp[ cell ] -- }
  VMI>f32 temp[ !			\ extract pattern
  temp[ sf@a 6 (f.)a >disline
;

: .VModImm	\ -- ;
  c"  # " $>dis  VModImm@
  case  cmode11:8@
    0 of  .decVMI  endof
    1 of  .decVMI  endof
    2 of  8 lshift .hexVMI  endof
    3 of  8 lshift .hexVMI  endof
    4 of  #16 lshift .hexVMI  endof
    5 of  #16 lshift .hexVMI  endof
    6 of  #24 lshift .hexVMI  endof
    7 of  #24 lshift .hexVMI  endof
    8 of  dup #16 lshift or .hexVMI  endof
    9 of  dup #16 lshift or .hexVMI  endof
    #10 of  8 lshift dup #16 lshift or .hexVMI  endof
    #11 of  8 lshift dup #16 lshift or .hexVMI  endof
    #12 of  8 lshift $FF or .hexVMI  endof
    #13 of  #16 lshift $FFFF or .hexVMI  endof
    #14 of  op5@ if  .aaaaaaaa8VMI  else  .rep8VMI  endif  endof
    #15 of  op5@ if  drop undef  else  .fVMI  endif  endof
  endcase
;

: .Vdt	\ -- ;
  bl>dis
  case  cmode11:8@
    0 of  c" .i32"  endof
    1 of  c" .i32"  endof
    2 of  c" .i32"  endof
    3 of  c" .i32"  endof
    4 of  c" .i32"  endof
    5 of  c" .i32"  endof
    6 of  c" .i32"  endof
    7 of  c" .i32"  endof
    8 of  c" .i16"  endof
    9 of  c" .i16"  endof
    #10 of  c" .i16"  endof
    #11 of  c" .i16"  endof
    #12 of  c" .i32"  endof
    #13 of  c" .i32"  endof
    #14 of  op5@ if  c" .i64"  else  c" .i8"  endif  endof
    #15 of  op5@ if  c" .???" else  c" .f32"  endif  endof
  endcase
  $>dis
;

: showVmodImm-1	\ $ --
  .Vdt  .Ad ,>dis  .VmodImm
;

: imm6@		\ -- imm6
  curr-instr @ #16 rshift $003F and	\ imm6
;

: L:imm6@	\ -- L:imm6
  curr-instr @ dup #16 rshift $003F and	\ imm6
  swap $080 and 1 rshift or		\ L:imm6
;

: .vshSize1	\ l:imm6 -- #shift
  dup $078 and $008 = if
    c" 8 " $>dis  #16 swap -  exit
  endif
  dup $070 and $010 = if
    c" 16 " $>dis  #32 swap -  exit
  endif
  dup $060 and $020 = if
    c" 32 " $>dis  #64 swap -  exit
  endif
  dup $040 and if
    c" 64 " $>dis  $3F and  #64 swap -  exit
  endif
  drop c" Bad-Type/Size" $>dis  0
;

: .vshSize2	\ l:imm6 -- #shift
  dup $078 and $008 = if
    c" 8 " $>dis  8 -  exit
  endif
  dup $070 and $010 = if
    c" 16 " $>dis  #16 -  exit
  endif
  dup $060 and $020 = if
    c" 32 " $>dis  #32 -  exit
  endif
  dup $040 and if
    c" 64 " $>dis  $3F and  exit
  endif
  drop c" Bad-Size2" $>dis  0
;

: .vshU/S	\ --
  ASIMD-ubit @
  if  c" .u"  else  c" .s"  endif  $>dis
;

: .vsh.		\ --
  c" ." $>dis
;

: .vshI		\ --
  c" .I" $>dis  ;

: .vshUop
  ASIMD-ubit @ 0<>  curr-instr @ $0100 and 0<>  and
  if  c" .u"  else  c" .s"  then $>dis
;

: .VVimm	\ imm --
  .Ad ,>dis  .Am  c"  # #" $>dis dec>dis  ;

: .DQimm	\ imm --
  .Dd ,>dis  .Qm  c"  # #" $>dis dec>dis  ;

: .QDimm	\ imm --
  .Qd ,>dis  .Dm  c"  # #" $>dis dec>dis  ;

: showVVSH-u/s1	\ $ --
  $>dis  .vshU/S L:imm6@ .vshSize1  .VVimm  ;

: showVVSH-.1	\ $ --
  $>dis  .vsh. L:imm6@ .vshSize1  .VVimm  ;

: showVVSH-i1	\ $ --
  $>dis  .vshI L:imm6@ .vshSize1  .VVimm  ;


: showDQ#shift-i1	\ $ --
  $>dis  .vshI L:imm6@  .DQimm
;
: showDQ#shift-suop1	\ $ --
  $>dis  .vshUop L:imm6@ .vshSize1  .DQimm  ;

: showVVSH-i2	\ $ --
  $>dis  .vshI L:imm6@ .vshSize2  .VVimm  ;

: showVVSH-.2	\ $ --
  $>dis  .vsh. L:imm6@ .vshSize2  .VVimm  ;

: showVVSH-uop2	\ $ --
  $>dis  .vshuop L:imm6@ .vshSize2  .VVimm  ;

: showQD#shift-u/s2	\ $ --
  $>dis  .vshU/S L:imm6@ .vshsize2  .QDimm  ;

: .suf32-U:8	\ -- ; depends on U and bit 8
  bl>dis  curr-instr @ $100 and if	\ op=1
    ASIMD-ubit @ 0= if			\ op=1, U=0
      c" .s32.f32 "
    else				\ op=1, U=1
      c" .u32.f32 "
    endif
  else					\ op=0
    ASIMD-ubit @ 0= if			\ op=0, U=0
      c" .f32.s32 "
    else				\ op=0, U=1
      c" .f32.u32 "
    endif
  endif
  $>dis
;

: showVV#frac1	\ $ --
  $>dis  .suf32-U:8  imm6@ .VVimm
;

: .if8:21:20		\ --
  curr-instr @ $0100 and
  if  c"  .f"  else  c"  .i"  endif
  $>dis  .dt21:20
;

: showDDD[x]-i/f	\ $ --
  $>dis  .if8:21:20  .Dd ,>dis  .Dn ,>dis  .Dm[x]1
;

: showQQD[x]-i/f	\ $ --
  $>dis  .if8:21:20  .Qd ,>dis  .Qn ,>dis  .Dm[x]1
;

: showQDD[x]-s/u	\ $ --
  $>dis  .s/udt21:20  .Qd ,>dis  .Dn ,>dis  .Dm[x]1
;

: showDDD[x]-s	\ $ --
  $>dis  .sdt21:20  .Dd ,>dis  .Dn ,>dis  .Dm[x]1
;

: showQQD[x]-s	\ $ --
  $>dis  .sdt21:20  .Qd ,>dis  .Qn ,>dis  .Dm[x]1
;

: .f64/32-8	\ --
  curr-instr @ $0100 and
  if  c" .f64 "  else  c" .f32 "  then  $>dis
;

: ?.ARMcond	\ --
  Thumb? 0= if
    curr-instr @ #28 rshift .condCode
  endif
;

: showFFF-sz8	\ $ --
  $>dis ?.ARMcond  .f64/32-8  .Fd ,>dis  .Fn ,>dis  .Fm
;

: showFF-sz8	\ $ --
\ cr ." showFF-sz8 in" key? drop
  $>dis ?.ARMcond  .f64/32-8  .Fd ,>dis  .Fm
\ cr ." showFF-sz8 out" key? drop
;

: V4:4imm@	\ -- u8
  curr-instr @ dup $0F and		\ -- instr efgh
  swap $000F:0000 and 12 rshift or	\ -- abcdefgh
;

: .VFPimm	\ --
  V4:4imm@ .fVMI
;

: showFdVFPimm	\ $ --
  $>dis ?.ARMcond  .f64/32-8  .Fd  c" , # " $>dis .VFPimm
;

: showVCVT-1632	\ $ -- ; VCVTx .f32.f16 Sd Sm
  $>dis ?.ARMcond  curr-instr @ $0001:0000 and
  if c" .f16.f32 " else c" .f32.f16 " then $>dis
  .Sd ,>dis  .Sm
;

: showVCMPx-6432	\ $ --
  $>dis ?.ARMcond  .f64/32-8
  curr-instr @ $0001:0000 and if
    .Fd  c" , # 0.0e0 " $>dis
  else
    .Fd ,>dis  .Fm
  endif
;

: showVCVT-3264	\ $ -- ; VCT .f64.f32 Dd Sm
  $>dis ?.ARMcond
  curr-instr @ $0000:0100 and if	\ double to single
    c" .f32.f64 " $>dis  .Sd ,>dis .Dm
  else					\ single to double
    c" .f64.f32 " $>dis  .Dd ,>dis .Sm
  endif
;

: showVCVTx-s32f64	\ $ --
  $>dis ?.ARMcond  c" .s32.f64 " $>dis  .Sd ,>dis .Dm
;
: showVCVTx-s32f32	\ $ --
  $>dis ?.ARMcond  c" .s32.f32 " $>dis  .Sd ,>dis .Sm
;
: showVCVTx-u32f64	\ $ --
  $>dis ?.ARMcond  c" .u32.f64 " $>dis  .Sd ,>dis .Dm
;
: showVCVTx-u32f32	\ $ --
  $>dis ?.ARMcond  c" .u32.f32 " $>dis  .Sd ,>dis .Sm
;
: showVCVT-f64s32 	\ $ --
  $>dis ?.ARMcond  c" .f64.s32 " $>dis  .Dd ,>dis .Sm
;
: showVCVT-f64u32 	\ $ --
  $>dis ?.ARMcond  c" .f64.u32 " $>dis  .Dd ,>dis .Sm
;
: showVCVT-f32s32 	\ $ --
  $>dis ?.ARMcond  c" .f32.s32 " $>dis  .Sd ,>dis .Sm
;
: showVCVT-f32u32 	\ $ --
  $>dis ?.ARMcond  c" .f32.u32 " $>dis  .Sd ,>dis .Sm
;

: .<sf>		\ --
  curr-instr @ $0100 and
  if  c" .f64"  else  c" .f32"  endif $>dis
;

: .<Td>		\ --
  curr-instr @ dup #15 rshift $02 and	\ u (bit 16) -> bit 1
  swap 7 rshift $01 and or		\ sx (bit 7) -> bit 0
  case
    0 of  c" .s16"  endof
    2 of  c" .u16"  endof
    1 of  c" .s32"  endof
    3 of  c" .u32"  endof
  endcase
  $>dis
;

: .#<fbits>	\ --
  c"  # #" $>dis
  curr-instr @ $0080 and
  if  #32  else  #16  endif
  curr-instr @ dup $0F and 1 lshift	\ top 4 bits
  swap $20 and 5 rshift or		\ bottom bit
  -  dec>dis
;

: showVCVT-fltfix	\ $ -- ; VCVT .Td/.Tx xD xD #fbits
  $>dis ?.ARMcond
  curr-instr @ $0004:0000 and		\ op = bit 18
  if  .<Td> .<sf>  else  .<sf> .<Td>  then
  .Fd .Fd .#<fbits>
;

defer run-disASdpiT
defer run-disASdpiA

mv: disVVscalarT	\ -- ; A7.4.3, p263
\   mask/value------------------------- test    xt              name "
\   111U1111:1xBBxxxx:xxxxAAAA:x1x0xxxx %000000	showVmodImm-1	IGN  "
mv" 111x1111:1x11xxxx:xxxxxxxx:x1x0xxxx %000000	run-disASdpiT	IGN  "
mv" 11101111:1xxxxxxx:xxxx000x:x1x0xxxx %000000	showDDD[x]-i/f	vmla  "	\ A.8.8.388, p933, T1/A1
mv" 11111111:1xxxxxxx:xxxx000x:x1x0xxxx %000000	showQQD[x]-i/f	vmla  "
mv" 11101111:1xxxxxxx:xxxx010x:x1x0xxxx %000000	showDDD[x]-i/f	vmls  "
mv" 11111111:1xxxxxxx:xxxx010x:x1x0xxxx %000000	showQQD[x]-i/f	vmls  "
mv" 111x1111:1xxxxxxx:xxxx0010:x1x0xxxx %000000	showQDD[x]-s/u	vmlal  " \ A.8.8.388, p933, T2/A2
mv" 111x1111:1xxxxxxx:xxxx0110:x1x0xxxx %000000	showQDD[x]-s/u	vmlsl  " \ A.8.8.388, p933, T2/A2
mv" 11101111:1xxxxxxx:xxxx0011:x1x0xxxx %000000	showQDD[x]-s/u	vqdmlal  " \ T2/A2
mv" 11101111:1xxxxxxx:xxxx0111:x1x0xxxx %000000	showQDD[x]-s/u	vqdmlsl  " \ T2/A2
mv" 11101111:1xxxxxxx:xxxx100x:x1x0xxxx %000000	showDDD[x]-i/f	vmul  "
mv" 11111111:1xxxxxxx:xxxx100x:x1x0xxxx %000000	showQQD[x]-i/f	vmul  "
mv" 11101111:1xxxxxxx:xxxx010x:x1x0xxxx %000000	showDDD[x]-i/f	vmls  "
mv" 111x1111:1xxxxxxx:xxxx1010:x1x0xxxx %000000	showQDD[x]-s/u	vmull  "
mv" 11101111:1xxxxxxx:xxxx1011:x1x0xxxx %000000	showQDD[x]-s/u	vqdmull  " \ T2/A2
mv" 11101111:1xxxxxxx:xxxx1100:x1x0xxxx %000000	showDDD[x]-s	vqdmulh  " \ T2/A2
mv" 11111111:1xxxxxxx:xxxx1100:x1x0xxxx %000000	showQQD[x]-s	vqdmulh  " \ T2/A2
mv" 11101111:1xxxxxxx:xxxx1101:x1x0xxxx %000000	showDDD[x]-s	vqrdmulh  " \ T2/A2
mv" 11111111:1xxxxxxx:xxxx1101:x1x0xxxx %000000	showQQD[x]-s	vqrdmulh  " \ T2/A2
;mv

mv: disVVscalarA	\ -- ; A7.4.3, p263
\   mask/value------------------------- test    xt              name "
\   111U1111:1xBBxxxx:xxxxAAAA:x1x0xxxx %000000	showVmodImm-1	IGN  "
mv" 1111001x:1x11xxxx:xxxxxxxx:x1x0xxxx %000000	run-disASdpiA	IGN  "
mv" 11110010:1xxxxxxx:xxxx000x:x1x0xxxx %000000	showDDD[x]-i/f	vmla  "	\ A.8.8.388, p933, T1/A1
mv" 11110011:1xxxxxxx:xxxx000x:x1x0xxxx %000000	showQQD[x]-i/f	vmla  "
mv" 11110010:1xxxxxxx:xxxx010x:x1x0xxxx %000000	showDDD[x]-i/f	vmls  "
mv" 11110011:1xxxxxxx:xxxx010x:x1x0xxxx %000000	showQQD[x]-i/f	vmls  "
mv" 1111001x:1xxxxxxx:xxxx0010:x1x0xxxx %000000	showQDD[x]-s/u	vmlal  " \ A.8.8.388, p933, T2/A2
mv" 1111001x:1xxxxxxx:xxxx0110:x1x0xxxx %000000	showQDD[x]-s/u	vmlsl  " \ A.8.8.388, p933, T2/A2
mv" 11110010:1xxxxxxx:xxxx0011:x1x0xxxx %000000	showQDD[x]-s/u	vqdmlal  " \ T2/A2
mv" 11110010:1xxxxxxx:xxxx0111:x1x0xxxx %000000	showQDD[x]-s/u	vqdmlsl  " \ T2/A2
mv" 11110010:1xxxxxxx:xxxx100x:x1x0xxxx %000000	showDDD[x]-i/f	vmul  "
mv" 11110011:1xxxxxxx:xxxx100x:x1x0xxxx %000000	showQQD[x]-i/f	vmul  "
mv" 11110010:1xxxxxxx:xxxx010x:x1x0xxxx %000000	showDDD[x]-i/f	vmls  "
mv" 1111001x:1xxxxxxx:xxxx1010:x1x0xxxx %000000	showQDD[x]-s/u	vmull  "
mv" 11110010:1xxxxxxx:xxxx1011:x1x0xxxx %000000	showQDD[x]-s/u	vqdmull  " \ T2/A2
mv" 11110010:1xxxxxxx:xxxx1100:x1x0xxxx %000000	showDDD[x]-s	vqdmulh  " \ T2/A2
mv" 11110011:1xxxxxxx:xxxx1100:x1x0xxxx %000000	showQQD[x]-s	vqdmulh  " \ T2/A2
mv" 11110010:1xxxxxxx:xxxx1101:x1x0xxxx %000000	showDDD[x]-s	vqrdmulh  " \ T2/A2
mv" 11110011:1xxxxxxx:xxxx1101:x1x0xxxx %000000	showQQD[x]-s	vqrdmulh  " \ T2/A2
;mv

mv: disVVshiftT	\ -- ; A7.4.4, p264
\   mask/value------------------------- test    xt              name "
\   111U1111:1xIIIxxx:xxxxAAAA:LBx1xxxx %000000	showVmodImm-1	IGN  "
mv" 111x1111:1xxxxxxx:xxxx0000:xxx1xxxx %000000	showVVSH-u/s1	vshr "
mv" 111x1111:1xxxxxxx:xxxx0001:xxx1xxxx %000000	showVVSH-u/s1	vsra "
mv" 111x1111:1xxxxxxx:xxxx0010:xxx1xxxx %000000	showVVSH-u/s1	vrshr "
mv" 11111111:1xxxxxxx:xxxx0100:xxx1xxxx %000000	showVVSH-.1	vsri "
mv" 11101111:1xxxxxxx:xxxx0101:xxx1xxxx %000000	showVVSH-i2	vshl "
mv" 11111111:1xxxxxxx:xxxx0101:xxx1xxxx %000000	showVVSH-.2	vsli "
mv" 11101111:1xxxxxxx:xxxx011x:xxx1xxxx %000000	showVVSH-uop2	vqshl "
mv" 11111111:1xxxxxxx:xxxx011x:xxx1xxxx %000000	showVVSH-uop2	vqshlu "
\   111U1111:1xIIIxxx:xxxxAAAA:LBx1xxxx %000000	showVmodImm-1	IGN  "
mv" 11101111:1xxxxxxx:xxxx1000:00x1xxxx %000000	showDQ#shift-i1	vshrn "
mv" 11101111:1xxxxxxx:xxxx1000:01x1xxxx %000000	showDQ#shift-i1	vrshrn "
mv" 11101111:1xxxxxxx:xxxx100x:00x1xxxx %000000	showDQ#shift-suop1  vqshrn "
mv" 11111111:1xxxxxxx:xxxx100x:00x1xxxx %000000	showDQ#shift-suop1  vqshrun "
mv" 11101111:1xxxxxxx:xxxx100x:01x1xxxx %000000	showDQ#shift-suop1  vqrshrn "
mv" 11111111:1xxxxxxx:xxxx100x:01x1xxxx %000000	showDQ#shift-suop1  vqrshrun "
mv" 111x1111:1xxxxxxx:xxxx1010:00x1xxxx %000000	showQD#shift-u/s2  vshll "	\ T1/A1
mv" 111x1111:1xxxxxxx:xxxx111x:0xx1xxxx %000000	showVV#frac1	vcvt "
;mv

mv: disVVshiftA	\ -- ; A7.4.4, p264
\   mask/value------------------------- test    xt              name "
\   1111001U:1xIIIxxx:xxxxAAAA:LBx1xxxx %000000	showVmodImm-1	IGN  "
mv" 1111001x:1xxxxxxx:xxxx0000:xxx1xxxx %000000	showVVSH-u/s1	vshr "
mv" 1111001x:1xxxxxxx:xxxx0001:xxx1xxxx %000000	showVVSH-u/s1	vsra "
mv" 1111001x:1xxxxxxx:xxxx0010:xxx1xxxx %000000	showVVSH-u/s1	vrshr "
mv" 11110011:1xxxxxxx:xxxx0100:xxx1xxxx %000000	showVVSH-.1	vsri "
mv" 11110010:1xxxxxxx:xxxx0101:xxx1xxxx %000000	showVVSH-i2	vshl "
mv" 11110011:1xxxxxxx:xxxx0101:xxx1xxxx %000000	showVVSH-.2	vsli "
mv" 11110010:1xxxxxxx:xxxx011x:xxx1xxxx %000000	showVVSH-uop2	vqshl "
mv" 11110011:1xxxxxxx:xxxx011x:xxx1xxxx %000000	showVVSH-uop2	vqshlu "
\   1111001U:1xIIIxxx:xxxxAAAA:LBx1xxxx %000000	showVmodImm-1	IGN  "
mv" 11110010:1xxxxxxx:xxxx1000:00x1xxxx %000000	showDQ#shift-i1	vshrn "
mv" 11110010:1xxxxxxx:xxxx1000:01x1xxxx %000000	showDQ#shift-i1	vrshrn "
mv" 11110010:1xxxxxxx:xxxx100x:00x1xxxx %000000	showDQ#shift-suop1  vqshrn "
mv" 11110011:1xxxxxxx:xxxx100x:00x1xxxx %000000	showDQ#shift-suop1  vqshrun "
mv" 11110010:1xxxxxxx:xxxx100x:01x1xxxx %000000	showDQ#shift-suop1  vqrshrn "
mv" 11110011:1xxxxxxx:xxxx100x:01x1xxxx %000000	showDQ#shift-suop1  vqrshrun "
mv" 1111001x:1xxxxxxx:xxxx1010:00x1xxxx %000000	showQD#shift-u/s2  vshll "	\ T1/A1
mv" 1111001x:1xxxxxxx:xxxx111x:0xx1xxxx %000000	showVV#frac1	vcvt "
;mv

mv: disVmodImmT	\ -- ; A7.4.6, p267
\   mask/value------------------------- test    xt              name "
\   111a1111:1x000bcd:xxxxCCCC:0xP1efgh %000000	showVmodImm-1	IGN  "
mv" 111x1111:1x000xxx:xxxx0xx0:0x01xxxx %000000	showVmodImm-1	vmov "
mv" 111x1111:1x000xxx:xxxx0xx1:0x01xxxx %000000	showVmodImm-1	vorr "
mv" 111x1111:1x000xxx:xxxx10x0:0x01xxxx %000000	showVmodImm-1	vmov "
mv" 111x1111:1x000xxx:xxxx10x1:0x01xxxx %000000	showVmodImm-1	vorr "
mv" 111x1111:1x000xxx:xxxx11xx:0x01xxxx %000000	showVmodImm-1	vmov "

mv" 111x1111:1x000xxx:xxxx0xx0:0x11xxxx %000000	showVmodImm-1	vmvn "
mv" 111x1111:1x000xxx:xxxx0xx1:0x11xxxx %000000	showVmodImm-1	vbic "
mv" 111x1111:1x000xxx:xxxx10x0:0x11xxxx %000000	showVmodImm-1	vmvn "
mv" 111x1111:1x000xxx:xxxx10x1:0x11xxxx %000000	showVmodImm-1	vbic "
mv" 111x1111:1x000xxx:xxxx110x:0x11xxxx %000000	showVmodImm-1	vmvn "
mv" 111x1111:1x000xxx:xxxx1110:0x11xxxx %000000	showVmodImm-1	vmov "
;mv

mv: disVmodImmA	\ -- ; A7.4.6, p267
\   mask/value------------------------- test    xt              name "
\   1111001x:1x000bcd:xxxxCCCC:0xP1efgh %000000	showVmodImm-1	IGN  "
mv" 1111001x:1x000xxx:xxxx0xx0:0x01xxxx %000000	showVmodImm-1	vmov "
mv" 1111001x:1x000xxx:xxxx0xx1:0x01xxxx %000000	showVmodImm-1	vorr "
mv" 1111001x:1x000xxx:xxxx10x0:0x01xxxx %000000	showVmodImm-1	vmov "
mv" 1111001x:1x000xxx:xxxx10x1:0x01xxxx %000000	showVmodImm-1	vorr "
mv" 1111001x:1x000xxx:xxxx11xx:0x01xxxx %000000	showVmodImm-1	vmov "

mv" 1111001x:1x000xxx:xxxx0xx0:0x11xxxx %000000	showVmodImm-1	vmvn "
mv" 1111001x:1x000xxx:xxxx0xx1:0x11xxxx %000000	showVmodImm-1	vbic "
mv" 1111001x:1x000xxx:xxxx10x0:0x11xxxx %000000	showVmodImm-1	vmvn "
mv" 1111001x:1x000xxx:xxxx10x1:0x11xxxx %000000	showVmodImm-1	vbic "
mv" 1111001x:1x000xxx:xxxx110x:0x11xxxx %000000	showVmodImm-1	vmvn "
mv" 1111001x:1x000xxx:xxxx1110:0x11xxxx %000000	showVmodImm-1	vmov "
;mv

mv: disVVmiscT	\ -- ; A7.4.5, p265
\   mask/value------------------------- test    xt              name "
\   11111111:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11111111:1x11xx00:xxxx0001:0xx0xxxx %000000	showVV-idt	vrev16 "
mv" 11111111:1x11xx00:xxxx0000:1xx0xxxx %000000	showVV-idt	vrev32 "
mv" 11111111:1x11xx00:xxxx0000:0xx0xxxx %000000	showVV-idt	vrev64 "
mv" 11111111:1x11xx00:xxxx0010:xxx0xxxx %000000	showVV-idt	vpaddl "
mv" 11111111:1x11xx00:xxxx0100:0xx0xxxx %000000	showVV-idt	vcls "
mv" 11111111:1x11xx00:xxxx0100:1xx0xxxx %000000	showVV-idt	vclz "
mv" 11111111:1x11xx00:xxxx0101:0xx0xxxx %000000	showVV-idt	vcnt "
mv" 11111111:1x110000:xxxx0101:1xx0xxxx %000000	showVV		vmvn "
mv" 11111111:1x11xx00:xxxx0110:xxx0xxxx %000000	showVV-sdt	vpadal "
mv" 11111111:1x11xx00:xxxx0111:0xx0xxxx %000000	showVV-idt	vqabs "
mv" 11111111:1x11xx00:xxxx0111:1xx0xxxx %000000	showVV-idt	vqneg "
\   11111111:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11111111:1x11xx01:xxxx0x00:0xx0xxxx %000000	showVV#0-sfdt	vcgt "
mv" 11111111:1x11xx01:xxxx0x00:1xx0xxxx %000000	showVV#0-sfdt	vcge "
mv" 11111111:1x11xx01:xxxx0x00:0xx0xxxx %000000	showVV#0-sfdt	vceq "
mv" 11111111:1x11xx01:xxxx0x01:0xx0xxxx %000000	showVV#0-sfdt	vcgt "
mv" 11111111:1x11xx01:xxxx0x01:1xx0xxxx %000000	showVV#0-sfdt	vcle "
mv" 11111111:1x11xx01:xxxx0x10:0xx0xxxx %000000	showVV#0-sfdt	vclt "
mv" 11111111:1x11xx01:xxxx0x11:0xx0xxxx %000000	showVV-sfdt	vneg "
mv" 11111111:1x11xx01:xxxx0x11:1xx0xxxx %000000	showVV-sfdt	vabs "
\   11111111:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11111111:1x11xx10:xxxx0000:0xx0xxxx %000000	showVV		vswp "
mv" 11111111:1x11xx10:xxxx0000:1xx0xxxx %000000	showVV-idt	vtrn "
mv" 11111111:1x11xx10:xxxx0001:0xx0xxxx %000000	showVV-idt	vuzp "
mv" 11111111:1x11xx10:xxxx0001:1xx0xxxx %000000	showVV-idt	vzip "
mv" 11111111:1x11xx10:xxxx0010:00x0xxxx %000000	showDQ-idt	vmovn "
mv" 11111111:1x11xx10:xxxx0010:01x0xxxx %000000	showDQ-sudt	vqmovun "
mv" 11111111:1x11xx10:xxxx0010:1xx0xxxx %000000	showDQ-sudt	vqmovn "
mv" 11111111:1x11xx10:xxxx0011:00x0xxxx %000000	showQD#size	vshll "
mv" 11111111:1x11xx10:xxxx011x:00x0xxxx %000000	showVCVT-h/s	vcvt "
\   11111111:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11111111:1x11xx11:xxxx010x:0xx0xxxx %000000	showVV-ufdt	vrecpe "
mv" 11111111:1x11xx11:xxxx010x:1xx0xxxx %000000	showVV-ufdt	vrsqrte "
mv" 11111111:1x111011:xxxx011x:xxx0xxxx %000000	showVV-suf	vcvt "
;mv

mv: disVVmiscA	\ -- ; A7.4.5, p265
\   mask/value------------------------- test    xt              name "
\   11110011:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11110011:1x11xx00:xxxx0001:0xx0xxxx %000000	showVV-idt	vrev16 "
mv" 11110011:1x11xx00:xxxx0000:1xx0xxxx %000000	showVV-idt	vrev32 "
mv" 11110011:1x11xx00:xxxx0000:0xx0xxxx %000000	showVV-idt	vrev64 "
mv" 11110011:1x11xx00:xxxx0010:xxx0xxxx %000000	showVV-idt	vpaddl "
mv" 11110011:1x11xx00:xxxx0100:0xx0xxxx %000000	showVV-idt	vcls "
mv" 11110011:1x11xx00:xxxx0100:1xx0xxxx %000000	showVV-idt	vclz "
mv" 11110011:1x11xx00:xxxx0101:0xx0xxxx %000000	showVV-idt	vcnt "
mv" 11110011:1x110000:xxxx0101:1xx0xxxx %000000	showVV		vmvn "
mv" 11110011:1x11xx00:xxxx0110:xxx0xxxx %000000	showVV-sdt	vpadal "
mv" 11110011:1x11xx00:xxxx0111:0xx0xxxx %000000	showVV-idt	vqabs "
mv" 11110011:1x11xx00:xxxx0111:1xx0xxxx %000000	showVV-idt	vqneg "
\   11110011:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11110011:1x11xx01:xxxx0x00:0xx0xxxx %000000	showVV#0-sfdt	vcgt "
mv" 11110011:1x11xx01:xxxx0x00:1xx0xxxx %000000	showVV#0-sfdt	vcge "
mv" 11110011:1x11xx01:xxxx0x00:0xx0xxxx %000000	showVV#0-sfdt	vceq "
mv" 11110011:1x11xx01:xxxx0x01:0xx0xxxx %000000	showVV#0-sfdt	vcgt "
mv" 11110011:1x11xx01:xxxx0x01:1xx0xxxx %000000	showVV#0-sfdt	vcle "
mv" 11110011:1x11xx01:xxxx0x10:0xx0xxxx %000000	showVV#0-sfdt	vclt "
mv" 11110011:1x11xx01:xxxx0x11:0xx0xxxx %000000	showVV-sfdt	vneg "
mv" 11110011:1x11xx01:xxxx0x11:1xx0xxxx %000000	showVV-sfdt	vabs "
\   11110011:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11110011:1x11xx10:xxxx0000:0xx0xxxx %000000	showVV		vswp "
mv" 11110011:1x11xx10:xxxx0000:1xx0xxxx %000000	showVV-idt	vtrn "
mv" 11110011:1x11xx10:xxxx0001:0xx0xxxx %000000	showVV-idt	vuzp "
mv" 11110011:1x11xx10:xxxx0001:1xx0xxxx %000000	showVV-idt	vzip "
mv" 11110011:1x11xx10:xxxx0010:00x0xxxx %000000	showDQ-idt	vmovn "
mv" 11110011:1x11xx10:xxxx0010:01x0xxxx %000000	showDQ-sudt	vqmovun "
mv" 11110011:1x11xx10:xxxx0010:1xx0xxxx %000000	showDQ-sudt	vqmovn "
mv" 11110011:1x11xx10:xxxx0011:00x0xxxx %000000	showQD#size	vshll "
mv" 11110011:1x11xx10:xxxx011x:00x0xxxx %000000	showVCVT-h/s	vcvt "
\   11110011:1x11xxAA:xxxx0BBB:BBx0xxxx %000000	disVVmisc	IGN  "
mv" 11110011:1x11xx11:xxxx010x:0xx0xxxx %000000	showVV-ufdt	vrecpe "
mv" 11110011:1x11xx11:xxxx010x:1xx0xxxx %000000	showVV-ufdt	vrsqrte "
mv" 11110011:1x111011:xxxx011x:xxx0xxxx %000000	showVV-suf	vcvt "
;mv

mv: disVVVsameT	\ -- ; A7.4.1, p 260
\   mask/value------------------------- test    xt              name "
\   IIIUIIII:IxCCxxxx:xxxxAAAA:xxxBxxxx  RdRnRt
mv" 11101111:0x00xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vand "
mv" 11101111:0x01xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbic "
mv" 11101111:0x10xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vorr "
mv" 11101111:0x11xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vorn "
mv" 11111111:0x00xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	veor "
mv" 11111111:0x01xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbsl "
mv" 11111111:0x10xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbit "
mv" 11111111:0x11xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbif "
mv" 111x1111:0xxxxxxx:xxxx0000:xxx0xxxx %000000	showVVV-s/u-q/d	vhadd "
mv" 111x1111:0xxxxxxx:xxxx0010:xxx0xxxx %000000	showVVV-s/u-q/d	vhsub "
mv" 111x1111:0xxxxxxx:xxxx0000:xxx1xxxx %000000	showVVV-s/u-q/d	vqadd "
mv" 111x1111:0xxxxxxx:xxxx0001:xxx0xxxx %000000	showVVV-s/u-q/d	vrhadd "
mv" 111x1111:0xxxxxxx:xxxx0010:xxx1xxxx %000000	showVVV-s/u-q/d	vqsub "
mv" 111x1111:0xxxxxxx:xxxx0011:xxx0xxxx %000000	showVVV-s/u-q/d	vcgt "
mv" 111x1111:0xxxxxxx:xxxx0011:xxx1xxxx %000000	showVVV-s/u-q/d	vcge "
mv" 111x1111:0xxxxxxx:xxxx0100:xxx0xxxx %000000	showVVV-s/u-q/d	vshl "
mv" 111x1111:0xxxxxxx:xxxx0100:xxx1xxxx %000000	showVVV-s/u-q/d	vqshl "
mv" 111x1111:0xxxxxxx:xxxx0101:xxx0xxxx %000000	showVVV-s/u-q/d	vrshl "
mv" 111x1111:0xxxxxxx:xxxx0101:xxx1xxxx %000000	showVVV-s/u-q/d	vqrshl "
mv" 111x1111:0xxxxxxx:xxxx0110:xxx0xxxx %000000	showVVV-s/u-q/d	vmax "
mv" 111x1111:0xxxxxxx:xxxx0110:xxx1xxxx %000000	showVVV-s/u-q/d	vmin "
mv" 111x1111:0xxxxxxx:xxxx0111:xxx0xxxx %000000	showVVV-s/u-q/d	vabd "
mv" 111x1111:0xxxxxxx:xxxx0111:xxx1xxxx %000000	showVVV-s/u-q/d	vaba "
\   mask/value------------------------- test    xt              name "
\   IIIUIIII:IxCCxxxx:xxxxAAAA:xxxBxxxx  RdRnRt
mv" 11101111:0xxxxxxx:xxxx1000:xxx0xxxx %000000	showVVV-idt-q/d	vadd "
mv" 11111111:0xxxxxxx:xxxx1000:xxx0xxxx %000000	showVVV-idt-q/d	vsub "
mv" 11101111:0xxxxxxx:xxxx1000:xxx1xxxx %000000	showVVV-idt-q/d	vtst "
mv" 11111111:0xxxxxxx:xxxx1000:xxx1xxxx %000000	showVVV-idt-q/d	vceq "
mv" 11101111:0xxxxxxx:xxxx1001:xxx0xxxx %000000	showVVV-idt-q/d	vmla "
mv" 11111111:0xxxxxxx:xxxx1001:xxx0xxxx %000000	showVVV-idt-q/d	vmls "
mv" 11101111:0xxxxxxx:xxxx1001:xxx1xxxx %000000	showVVV-idt-q/d	vmul "
mv" 11111111:0xxxxxxx:xxxx1001:xxx1xxxx %000000	showVVV-idt-q/d	vmulp "
mv" 111x1111:0xxxxxxx:xxxx1010:x0x0xxxx %000000	showVVV-s/u-q/d	vpmax "
mv" 111x1111:0xxxxxxx:xxxx1010:x0x1xxxx %000000	showVVV-s/u-q/d	vpmin "
mv" 111x1111:0xxxxxxx:xxxx1010:x1x0xxxx %000000	showUndef	vpmax "
mv" 111x1111:0xxxxxxx:xxxx1010:x1x1xxxx %000000	showUndef	vpmin "
mv" 11101111:0xxxxxxx:xxxx1011:xxx0xxxx %000000	showVVV-idt-q/d	vqdmulh "
mv" 11111111:0xxxxxxx:xxxx1011:xxx0xxxx %000000	showVVV-idt-q/d	vqrdmulh "
mv" 11101111:0xxxxxxx:xxxx1011:x0x1xxxx %000000	showVVV-idt-q/d	vpadd "
mv" 11101111:0xxxxxxx:xxxx1011:x1x1xxxx %000000	showUndef	vpadd "
mv" 11101111:0x0xxxxx:xxxx1100:xxx1xxxx %000000	showVVV-f32-q/d	vfma "
mv" 11101111:0x1xxxxx:xxxx1100:xxx1xxxx %000000	showVVV-f32-q/d	vfms "
\   mask/value------------------------- test    xt              name "
\   IIIUIIII:IxCCxxxx:xxxxAAAA:xxxBxxxx  RdRnRt
mv" 11101111:0x00xxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vadd "
mv" 11101111:0x01xxxx:xxxx1101:xxx0xxxx %000000	showUndef	vadd "
mv" 11101111:0x10xxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vsub "
mv" 11101111:0x11xxxx:xxxx1101:xxx0xxxx %000000	showUndef	vsub "
mv" 11111111:0x00xxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vpadd "
mv" 11111111:0x01xxxx:xxxx1101:xxx0xxxx %000000	showUndef	vpadd "
mv" 11111111:0x1xxxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vabd "
mv" 11101111:0x00xxxx:xxxx1101:xxx1xxxx %000000	showVVV-f32-q/d	vmla "
mv" 11101111:0x01xxxx:xxxx1101:xxx1xxxx %000000	showUndef	vmla "
mv" 11101111:0x10xxxx:xxxx1101:xxx1xxxx %000000	showVVV-f32-q/d	vmls "
mv" 11101111:0x11xxxx:xxxx1101:xxx1xxxx %000000	showUndef	vmls "
mv" 11111111:0x0xxxxx:xxxx1101:xxx1xxxx %000000	showVVV-f32-q/d	vmul "
mv" 11101111:0x0xxxxx:xxxx1110:xxx0xxxx %000000	showVVV-f32-q/d	vceq "
mv" 11111111:0x0xxxxx:xxxx1110:xxx0xxxx %000000	showVVV-f32-q/d	vcge "
mv" 11111111:0x1xxxxx:xxxx1110:xxx0xxxx %000000	showVVV-f32-q/d	vcgt "
mv" 11111111:0x00xxxx:xxxx1110:xxx1xxxx %000000	showVVV-f32-q/d	vacge "
mv" 11111111:0x10xxxx:xxxx1110:xxx1xxxx %000000	showVVV-f32-q/d	vacgt "
mv" 11101111:0x00xxxx:xxxx1111:xxx0xxxx %000000	showVVV-f32-q/d	vmax "
mv" 11101111:0x10xxxx:xxxx1111:xxx0xxxx %000000	showVVV-f32-q/d	vmin "
mv" 11101111:0xx1xxxx:xxxx1111:xxx0xxxx %000000 showUndef	vmax "
;mv

mv: disVVVsameA	\ -- ; DDI0406C, pp260/1, table A7-9
\   mask/value------------------------- test    xt              name "
\   IIIUIIII:IxCCxxxx:xxxxAAAA:xxxBxxxx  RdRnRt
mv" 11110010:0x00xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vand "
mv" 11110010:0x01xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbic "
mv" 11110010:0x10xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vorr "
mv" 11110010:0x11xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vorn "
mv" 11110011:0x00xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	veor "
mv" 11110011:0x01xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbsl "
mv" 11110011:0x10xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbit "
mv" 11110011:0x11xxxx:xxxx0001:xxx1xxxx %000000	showVVV-q/d	vbif "
mv" 1111001x:0xxxxxxx:xxxx0000:xxx0xxxx %000000	showVVV-s/u-q/d	vhadd "
mv" 1111001x:0xxxxxxx:xxxx0010:xxx0xxxx %000000	showVVV-s/u-q/d	vhsub "
mv" 1111001x:0xxxxxxx:xxxx0000:xxx1xxxx %000000	showVVV-s/u-q/d	vqadd "
mv" 1111001x:0xxxxxxx:xxxx0001:xxx0xxxx %000000	showVVV-s/u-q/d	vrhadd "
mv" 1111001x:0xxxxxxx:xxxx0010:xxx1xxxx %000000	showVVV-s/u-q/d	vqsub "
mv" 1111001x:0xxxxxxx:xxxx0011:xxx0xxxx %000000	showVVV-s/u-q/d	vcgt "
mv" 1111001x:0xxxxxxx:xxxx0011:xxx1xxxx %000000	showVVV-s/u-q/d	vcge "
mv" 1111001x:0xxxxxxx:xxxx0100:xxx0xxxx %000000	showVVV-s/u-q/d	vshl "
mv" 1111001x:0xxxxxxx:xxxx0100:xxx1xxxx %000000	showVVV-s/u-q/d	vqshl "
mv" 1111001x:0xxxxxxx:xxxx0101:xxx0xxxx %000000	showVVV-s/u-q/d	vrshl "
mv" 1111001x:0xxxxxxx:xxxx0101:xxx1xxxx %000000	showVVV-s/u-q/d	vqrshl "
mv" 1111001x:0xxxxxxx:xxxx0110:xxx0xxxx %000000	showVVV-s/u-q/d	vmax "
mv" 1111001x:0xxxxxxx:xxxx0110:xxx1xxxx %000000	showVVV-s/u-q/d	vmin "
mv" 1111001x:0xxxxxxx:xxxx0111:xxx0xxxx %000000	showVVV-s/u-q/d	vabd "
mv" 1111001x:0xxxxxxx:xxxx0111:xxx1xxxx %000000	showVVV-s/u-q/d	vaba "
\   mask/value------------------------- test    xt              name "
\   1111001U:0xCCxxxx:xxxxAAAA:xxxBxxxx  RdRnRt
mv" 11110010:0xxxxxxx:xxxx1000:xxx0xxxx %000000	showVVV-idt-q/d	vadd "
mv" 11110011:0xxxxxxx:xxxx1000:xxx0xxxx %000000	showVVV-idt-q/d	vsub "
mv" 11110010:0xxxxxxx:xxxx1000:xxx1xxxx %000000	showVVV-idt-q/d	vtst "
mv" 11110011:0xxxxxxx:xxxx1000:xxx1xxxx %000000	showVVV-idt-q/d	vceq "
mv" 11110010:0xxxxxxx:xxxx1001:xxx0xxxx %000000	showVVV-idt-q/d	vmla "
mv" 11110011:0xxxxxxx:xxxx1001:xxx0xxxx %000000	showVVV-idt-q/d	vmls "
mv" 11110010:0xxxxxxx:xxxx1001:xxx1xxxx %000000	showVVV-idt-q/d	vmul "
mv" 11110011:0xxxxxxx:xxxx1001:xxx1xxxx %000000	showVVV-idt-q/d	vmulp "
mv" 1111001x:0xxxxxxx:xxxx1010:x0x0xxxx %000000	showVVV-s/u-q/d	vpmax "
mv" 1111001x:0xxxxxxx:xxxx1010:x0x1xxxx %000000	showVVV-s/u-q/d	vpmin "
mv" 1111001x:0xxxxxxx:xxxx1010:x1x0xxxx %000000	showUndef	vpmax "
mv" 1111001x:0xxxxxxx:xxxx1010:x1x1xxxx %000000	showUndef	vpmin "
mv" 11110010:0xxxxxxx:xxxx1011:xxx0xxxx %000000	showVVV-idt-q/d	vqdmulh "
mv" 11110011:0xxxxxxx:xxxx1011:xxx0xxxx %000000	showVVV-idt-q/d	vqrdmulh "
mv" 11110010:0xxxxxxx:xxxx1011:x0x1xxxx %000000	showVVV-idt-q/d	vpadd "
mv" 11110010:0xxxxxxx:xxxx1011:x1x1xxxx %000000	showUndef	vpadd "
mv" 11110010:0x0xxxxx:xxxx1100:xxx1xxxx %000000	showVVV-f32-q/d	vfma "
mv" 11110010:0x1xxxxx:xxxx1100:xxx1xxxx %000000	showVVV-f32-q/d	vfms "
\   mask/value------------------------- test    xt              name "
\   IIIUIIII:IxCCxxxx:xxxxAAAA:xxxBxxxx  RdRnRt
mv" 11110010:0x00xxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vadd "
mv" 11110010:0x01xxxx:xxxx1101:xxx0xxxx %000000	showUndef	vadd "
mv" 11110010:0x10xxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vsub "
mv" 11110010:0x11xxxx:xxxx1101:xxx0xxxx %000000	showUndef	vsub "
mv" 11110011:0x00xxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vpadd "
mv" 11110011:0x01xxxx:xxxx1101:xxx0xxxx %000000	showUndef	vpadd "
mv" 11110011:0x1xxxxx:xxxx1101:xxx0xxxx %000000	showVVV-f32-q/d	vabd "
mv" 11110010:0x00xxxx:xxxx1101:xxx1xxxx %000000	showVVV-f32-q/d	vmla "
mv" 11110010:0x01xxxx:xxxx1101:xxx1xxxx %000000	showUndef	vmla "
mv" 11110010:0x10xxxx:xxxx1101:xxx1xxxx %000000	showVVV-f32-q/d	vmls "
mv" 11110010:0x11xxxx:xxxx1101:xxx1xxxx %000000	showUndef	vmls "
mv" 11110011:0x0xxxxx:xxxx1101:xxx1xxxx %000000	showVVV-f32-q/d	vmul "
mv" 11110010:0x0xxxxx:xxxx1110:xxx0xxxx %000000	showVVV-f32-q/d	vceq "
mv" 11110011:0x0xxxxx:xxxx1110:xxx0xxxx %000000	showVVV-f32-q/d	vcge "
mv" 11110011:0x1xxxxx:xxxx1110:xxx0xxxx %000000	showVVV-f32-q/d	vcgt "
mv" 11110011:0x00xxxx:xxxx1110:xxx1xxxx %000000	showVVV-f32-q/d	vacge "
mv" 11110011:0x10xxxx:xxxx1110:xxx1xxxx %000000	showVVV-f32-q/d	vacgt "
mv" 11110010:0x00xxxx:xxxx1111:xxx0xxxx %000000	showVVV-f32-q/d	vmax "
mv" 11110010:0x10xxxx:xxxx1111:xxx0xxxx %000000	showVVV-f32-q/d	vmin "
mv" 11110010:0xx1xxxx:xxxx1111:xxx0xxxx %000000 showUndef	vmax "
;mv

mv: disVVVdiffT	\ -- ; DDI0406C, table A7-10, p262
\   mask/value------------------------- test    xt              name "
\   111U1111:1xBBxxxx:xxxxAAAA:xxxxxxxx  RdRnRt
mv" 111x1111:1xxxxxxx:xxxx0000:x0x0xxxx %000000	showQDD-s/u	vaddl "
mv" 111x1111:1xxxxxxx:xxxx0001:x0x0xxxx %000000	showQQD-s/u	vaddw "
mv" 111x1111:1xxxxxxx:xxxx0010:x0x0xxxx %000000	showQDD-s/u	vsubl "
mv" 111x1111:1xxxxxxx:xxxx0011:x0x0xxxx %000000	showQQD-s/u	vsubw "
mv" 11101111:1xxxxxxx:xxxx0100:x0x0xxxx %000000	showDQQ-idt	vaddhn "
mv" 11111111:1xxxxxxx:xxxx0100:x0x0xxxx %000000	showDQQ-idt	vraddhn "
mv" 111x1111:1xxxxxxx:xxxx0100:x0x0xxxx %000000	showQDD-s/u	vabal "
mv" 11101111:1xxxxxxx:xxxx0110:x0x0xxxx %000000	showDQQ-idt	vsubhn "
mv" 11111111:1xxxxxxx:xxxx0110:x0x0xxxx %000000	showDQQ-idt	vrsubhn "
mv" 111x1111:1xxxxxxx:xxxx0111:x0x0xxxx %000000	showQDD-s/u	vabdl "
mv" 111x1111:1xxxxxxx:xxxx1000:x0x0xxxx %000000	showQDD-s/u	vmlal "
mv" 111x1111:1xxxxxxx:xxxx1010:x0x0xxxx %000000	showQDD-s/u	vmlsl "
mv" 11101111:1xxxxxxx:xxxx0011:x1x0xxxx %000000	showQDD[x]-idt	vqdmlal "
mv" 11101111:1xxxxxxx:xxxx0111:x1x0xxxx %000000	showQDD[x]-idt	vqdmlsl "
mv" 111x1111:1xxxxxxx:xxxx1100:x0x0xxxx %000000	showQDD-s/u	vmull "
mv" 111x1111:1xxxxxxx:xxxx1110:x0x0xxxx %000000	showQDD-s/u	vmullp "
mv" 11101111:1xxxxxxx:xxxx1101:x0x0xxxx %000000	showQDD-s/u	vqdmull "
mv" 11101111:1xxxxxxx:xxxx1011:x1x0xxxx %000000	showQDD[x]-idt	vqdmull "
mv" 111x1111:1xxxxxxx:xxxx1100:x0x0xxxx %000000	showVVV-s/u-q/d	vmul "
mv" 111x1111:1xxxxxxx:xxxx1110:x0x0xxxx %000000	showVVV-s/u-q/d	vmulp "
;mv

mv: disVVVdiffA	\ -- ; DDI0406C, table A7-10, p262
\   mask/value------------------------- test    xt              name "
\   111U1111:1xBBxxxx:xxxxAAAA:xxxxxxxx  RdRnRt
mv" 1111001x:1xxxxxxx:xxxx0000:x0x0xxxx %000000	showQDD-s/u	vaddl "
mv" 1111001x:1xxxxxxx:xxxx0001:x0x0xxxx %000000	showQQD-s/u	vaddw "
mv" 1111001x:1xxxxxxx:xxxx0010:x0x0xxxx %000000	showQDD-s/u	vsubl "
mv" 1111001x:1xxxxxxx:xxxx0011:x0x0xxxx %000000	showQQD-s/u	vsubw "
mv" 11110010:1xxxxxxx:xxxx0100:x0x0xxxx %000000	showDQQ-idt	vaddhn "
mv" 11110011:1xxxxxxx:xxxx0100:x0x0xxxx %000000	showDQQ-idt	vraddhn "
mv" 1111001x:1xxxxxxx:xxxx0100:x0x0xxxx %000000	showQDD-s/u	vabal "
mv" 11110010:1xxxxxxx:xxxx0110:x0x0xxxx %000000	showDQQ-idt	vsubhn "
mv" 11110011:1xxxxxxx:xxxx0110:x0x0xxxx %000000	showDQQ-idt	vrsubhn "
mv" 1111001x:1xxxxxxx:xxxx0111:x0x0xxxx %000000	showQDD-s/u	vabdl "
mv" 1111001x:1xxxxxxx:xxxx1000:x0x0xxxx %000000	showQDD-s/u	vmlal "
mv" 1111001x:1xxxxxxx:xxxx1010:x0x0xxxx %000000	showQDD-s/u	vmlsl "
mv" 11110010:1xxxxxxx:xxxx0011:x1x0xxxx %000000	showQDD[x]-idt	vqdmlal "
mv" 11110010:1xxxxxxx:xxxx0111:x1x0xxxx %000000	showQDD[x]-idt	vqdmlsl "
mv" 1111001x:1xxxxxxx:xxxx1100:x0x0xxxx %000000	showQDD-s/u	vmull "
mv" 1111001x:1xxxxxxx:xxxx1110:x0x0xxxx %000000	showQDD-s/u	vmullp "
mv" 11110010:1xxxxxxx:xxxx1101:x0x0xxxx %000000	showQDD-s/u	vqdmull "
mv" 11110010:1xxxxxxx:xxxx1011:x1x0xxxx %000000	showQDD[x]-idt	vqdmull "
mv" 1111001x:1xxxxxxx:xxxx1100:x0x0xxxx %000000	showVVV-s/u-q/d	vmul "
mv" 1111001x:1xxxxxxx:xxxx1110:x0x0xxxx %000000	showVVV-s/u-q/d	vmulp "
;mv

mvR: disASdpiT	\ -- ; A7.4 p259
\   mask/value------------------------- test    xt              name "
\   xxxxxxxx:AAAAAxxx:xxxxBBBB:CCCCxxxx  RdRnRt
mv" 11101111:1x11xxxx:xxxxxxxx:xxx0xxxx %000000	showVEXT	IGN1 "
mv" 11111111:1x11xxxx:xxxx0xxx:xxx0xxxx %000000	disVVmiscT	IGN2 "
mv" 11111111:1x11xxxx:xxxx10xx:xxx0xxxx %000000	showVTBx	IGN3 "
mv" 11111111:1x11xxxx:xxxx11xx:0xx0xxxx %000000	showVDUPscalar	IGN4 "
mv" 111x1111:0xxxxxxx:xxxxxxxx:xxxxxxxx %000000	disVVVsameT	IGN5 "
mv" 111x1111:1x000xxx:xxxxxxxx:0xx1xxxx %000000	disVmodImmT	IGN6 "
mv" 111x1111:1x001xxx:xxxxxxxx:0xx1xxxx %000000	disVVshiftT	IGN7 "
mv" 111x1111:1x01xxxx:xxxxxxxx:0xx1xxxx %000000	disVVshiftT	IGN8 "
mv" 111x1111:1x1xxxxx:xxxxxxxx:0xx1xxxx %000000	disVVshiftT	IGN9 "
mv" 111x1111:1xxxxxxx:xxxxxxxx:1xx1xxxx %000000	disVVshiftT	IGN10 "
mv" 111x1111:1x0xxxxx:xxxxxxxx:x0x0xxxx %000000	disVVVdiffT	IGN11 "
mv" 111x1111:1x10xxxx:xxxxxxxx:x0x0xxxx %000000	disVVVdiffT	IGN12 "
mv" 111x1111:1x0xxxxx:xxxxxxxx:x1x0xxxx %000000	disVVscalarT	IGN13 "
mv" 111x1111:1x0xxxxx:xxxxxxxx:x1x0xxxx %000000	disVVscalarT	IGN14 "
;mvR

mvR: disASdpiA	\ -- ; A7.4 p259
\   mask/value------------------------- test    xt              name "
\   xxxxxxxx:AAAAAxxx:xxxxBBBB:CCCCxxxx  RdRnRt
mv" 11110010:1x11xxxx:xxxxxxxx:xxx0xxxx %000000	showVEXT	IGN1 "
mv" 11110011:1x11xxxx:xxxx0xxx:xxx0xxxx %000000	disVVmiscA	IGN2 "
mv" 11110011:1x11xxxx:xxxx10xx:xxx0xxxx %000000	showVTBx	IGN3 "
mv" 11110011:1x11xxxx:xxxx11xx:0xx0xxxx %000000	showVDUPscalar	IGN4 "
mv" 1111001x:0xxxxxxx:xxxxxxxx:xxxxxxxx %000000	disVVVsameA	IGN5 "
mv" 1111001x:1x000xxx:xxxxxxxx:0xx1xxxx %000000	disVmodImmA	IGN6 "
mv" 1111001x:1x001xxx:xxxxxxxx:0xx1xxxx %000000	disVVshiftA	IGN7 "
mv" 1111001x:1x01xxxx:xxxxxxxx:0xx1xxxx %000000	disVVshiftA	IGN8 "
mv" 1111001x:1x1xxxxx:xxxxxxxx:0xx1xxxx %000000	disVVshiftA	IGN9 "
mv" 1111001x:1xxxxxxx:xxxxxxxx:1xx1xxxx %000000	disVVshiftA	IGN10 "
mv" 1111111x:1x0xxxxx:xxxxxxxx:x0x0xxxx %000000	disVVVdiffA	IGN11 "
mv" 1111111x:1x10xxxx:xxxxxxxx:x0x0xxxx %000000	disVVVdiffA	IGN12 "
mv" 1111111x:1x0xxxxx:xxxxxxxx:x1x0xxxx %000000	disVVscalarA	IGN13 "
mv" 1111111x:1x0xxxxx:xxxxxxxx:x1x0xxxx %000000	disVVscalarA	IGN14 "
;mvR

assign disASdpiT to-do run-disASdpiT
assign disASdpiA to-do run-disASdpiA

mv: disVFPdpiT	\ -- ; DDI0406C, table A7-16, A7-17 p270
\   mask/value------------------------- test    xt              name "
\            opc1opc2          o3  opc4
\   111T1110:OOOOOOOO:xxxx101x:OOx0OOOO  RdRnRt
mv" 11101110:0x00xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vmla "  \ T2/A2
mv" 11101110:0x00xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vmls "
mv" 11101110:0x01xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vnmla "  \ T1/A1
mv" 11101110:0x01xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vnmls "
mv" 11101110:0x10xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vnmul "  \ T2/A2
mv" 11101110:0x10xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vmul "  \ T2/A2
mv" 11101110:0x11xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vadd "  \ T2/A2
mv" 11101110:0x11xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vsub "  \ T2/A2
mv" 11101110:1x00xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vdiv "  \ T2/A2
mv" 11101110:1x01xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vfnma "  \ T1/A1
mv" 11101110:1x01xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vfnms "  \ T1/A1
mv" 11101110:1x10xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vfma "  \ T2/A2
mv" 11101110:1x10xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vfms "  \ T2/A2
\            opc1opc2          o3  opc4
\   111T1110:1x11OOOO:xxxx101x:OOx0OOOO  RdRnRt
mv" 11101110:1x11xxxx:xxxx101x:x0x0xxxx %000000	showFdVFPimm	vmov "  \ T2/A2
mv" 11101110:1x110000:xxxx101x:01x0xxxx %000000	showFF-sz8	vmov "  \ T2/A2
mv" 11101110:1x110000:xxxx101x:11x0xxxx %000000	showFF-sz8	vabs "  \ T2/A2
mv" 11101110:1x110001:xxxx101x:01x0xxxx %000000	showFF-sz8	vabs "  \ T2/A2
mv" 11101110:1x110001:xxxx101x:11x0xxxx %000000	showFF-sz8	vsqrt "
mv" 11101110:1x11001x:xxxx101x:01x0xxxx %000000	showVCVT-1632	vcvtb "
mv" 11101110:1x11001x:xxxx101x:11x0xxxx %000000	showVCVT-1632	vcvtt "
mv" 11101110:1x11010x:xxxx101x:01x0xxxx %000000	showVCMPx-6432	vcmp "
mv" 11101110:1x11010x:xxxx101x:11x0xxxx %000000	showVCMPx-6432	vcmpe "
mv" 11101110:1x110111:xxxx101x:11x0xxxx %000000	showVCVT-3264	vcvt "
\ mv" 11101110:1x110111:xxxx101x:11x0xxxx %000000	showVCVT-3264	vcvt "
mv" 11101110:1x111101:xxxx1011:11x0xxxx %000000	showVCVTx-s32f64 vcvt "
mv" 11101110:1x111101:xxxx1011:01x0xxxx %000000	showVCVTx-s32f64 vcvtr "
mv" 11101110:1x111101:xxxx1010:11x0xxxx %000000	showVCVTx-s32f32 vcvt "
mv" 11101110:1x111101:xxxx1010:01x0xxxx %000000	showVCVTx-s32f32 vcvtr "
mv" 11101110:1x111100:xxxx1011:11x0xxxx %000000	showVCVTx-u32f64 vcvt "
mv" 11101110:1x111100:xxxx1011:01x0xxxx %000000	showVCVTx-u32f32 vcvtr "
mv" 11101110:1x111100:xxxx1010:11x0xxxx %000000	showVCVTx-u32f32 vcvt "
mv" 11101110:1x111100:xxxx1010:01x0xxxx %000000	showVCVTx-u32f32 vcvtr "
mv" 11101110:1x111000:xxxx1011:11x0xxxx %000000	showVCVT-f64s32 vcvt "
mv" 11101110:1x111000:xxxx1011:01x0xxxx %000000	showVCVT-f64u32 vcvt "
mv" 11101110:1x111000:xxxx1010:11x0xxxx %000000	showVCVT-f32s32 vcvt "
mv" 11101110:1x111000:xxxx1010:01x0xxxx %000000	showVCVT-f32u32 vcvt "
mv" 11101110:1x111x1x:xxxx101x:x1x0xxxx %000000	showVCVT-fltfix vcvt "
;mv

mv: disVFPdpiA	\ -- ; DDI0406C, table A7-16, A7-17 p270
\   mask/value------------------------- test    xt              name "
\            opc1opc2          o3  opc4
\   xxxx1110:OOOOOOOO:xxxx101x:OOx0OOOO  RdRnRt
mv" xxxx1110:0x00xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vmla "  \ T2/A2
mv" xxxx1110:0x00xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vmls "
mv" xxxx1110:0x01xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vnmla "  \ T1/A1
mv" xxxx1110:0x01xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vnmls "
mv" xxxx1110:0x10xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vnmul "  \ T2/A2
mv" xxxx1110:0x10xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vmul "  \ T2/A2
mv" xxxx1110:0x11xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vadd "  \ T2/A2
mv" xxxx1110:0x11xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vsub "  \ T2/A2
mv" xxxx1110:1x00xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vdiv "  \ T2/A2
mv" xxxx1110:1x01xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vfnma "  \ T1/A1
mv" xxxx1110:1x01xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vfnms "  \ T1/A1
mv" xxxx1110:1x10xxxx:xxxx101x:x0x0xxxx %000000	showFFF-sz8	vfma "  \ T2/A2
mv" xxxx1110:1x10xxxx:xxxx101x:x1x0xxxx %000000	showFFF-sz8	vfms "  \ T2/A2
\            opc1opc2          o3  opc4
\   111T1110:1x11OOOO:xxxx101x:OOx0OOOO  RdRnRt
mv" xxxx1110:1x11xxxx:xxxx101x:x0x0xxxx %000000	showFdVFPimm	vmov "  \ T2/A2
mv" xxxx1110:1x110000:xxxx101x:01x0xxxx %000000	showFF-sz8	vmov "  \ T2/A2
mv" xxxx1110:1x110000:xxxx101x:11x0xxxx %000000	showFF-sz8	vabs "  \ T2/A2
mv" xxxx1110:1x110001:xxxx101x:01x0xxxx %000000	showFF-sz8	vabs "  \ T2/A2
mv" xxxx1110:1x110001:xxxx101x:11x0xxxx %000000	showFF-sz8	vsqrt "
mv" xxxx1110:1x11001x:xxxx101x:01x0xxxx %000000	showVCVT-1632	vcvtb "
mv" xxxx1110:1x11001x:xxxx101x:11x0xxxx %000000	showVCVT-1632	vcvtt "
mv" xxxx1110:1x11010x:xxxx101x:01x0xxxx %000000	showVCMPx-6432	vcmp "
mv" xxxx1110:1x11010x:xxxx101x:11x0xxxx %000000	showVCMPx-6432	vcmpe "
mv" xxxx1110:1x110111:xxxx101x:11x0xxxx %000000	showVCVT-3264	vcvt "
\ mv" xxxx1110:1x110111:xxxx101x:11x0xxxx %000000	showVCVT-3264	vcvt "
mv" xxxx1110:1x111101:xxxx1011:11x0xxxx %000000	showVCVTx-s32f64 vcvt "
mv" xxxx1110:1x111101:xxxx1011:01x0xxxx %000000	showVCVTx-s32f64 vcvtr "
mv" xxxx1110:1x111101:xxxx1010:11x0xxxx %000000	showVCVTx-s32f32 vcvt "
mv" xxxx1110:1x111101:xxxx1010:01x0xxxx %000000	showVCVTx-s32f32 vcvtr "
mv" xxxx1110:1x111100:xxxx1011:11x0xxxx %000000	showVCVTx-u32f64 vcvt "
mv" xxxx1110:1x111100:xxxx1011:01x0xxxx %000000	showVCVTx-u32f32 vcvtr "
mv" xxxx1110:1x111100:xxxx1010:11x0xxxx %000000	showVCVTx-u32f32 vcvt "
mv" xxxx1110:1x111100:xxxx1010:01x0xxxx %000000	showVCVTx-u32f32 vcvtr "
mv" xxxx1110:1x111000:xxxx1011:11x0xxxx %000000	showVCVT-f64s32 vcvt "
mv" xxxx1110:1x111000:xxxx1011:01x0xxxx %000000	showVCVT-f64u32 vcvt "
mv" xxxx1110:1x111000:xxxx1010:11x0xxxx %000000	showVCVT-f32s32 vcvt "
mv" xxxx1110:1x111000:xxxx1010:01x0xxxx %000000	showVCVT-f32u32 vcvt "
mv" xxxx1110:1x111x1x:xxxx101x:x1x0xxxx %000000	showVCVT-fltfix vcvt "
;mv

: Rn#		\ -- reg#
  curr-instr @ #16 rshift $0F and
;

: Rm#		\ -- reg#
  curr-instr @ $0F and
;

: Rt#		\ -- reg#
  curr-instr @ #12 rshift $0F and
;

: Rt2#		\ -- reg#
  curr-instr @ #16 rshift $0F and
;

: .Rn-V		\ --
  Rn# .reg
;

: .Rm-V		\ --
  Rm# .reg
;

: .Rt-V		\ --
  Rt# .reg
;

: .Rt2-V	\ --
  Rt2# .reg
;

: .wb21		\ --
  curr-instr @ $0020:0000 and
  if  c" ! " $>dis  endif
;

: #regs@	\ -- u
  curr-instr @ $FF and
;

: .list-V	\ --
  c" { " $>dis
  curr-instr @ $0100 and if
    DVreg@ #regs@ 2/ bounds
    ?do  i .Dvreg bl>dis  loop
  else
    DSreg@ #regs@ bounds
    ?do  i .Sreg bl>dis  loop
  endif
  c" } " $>dis
;

: showVSTMxx	\ $ --
  $>dis ?.ARMcond  .Rn-V bl>dis  .wb21  .list-V
;

: immVldrstr@	\ -- u
  curr-instr @ $FF and 2 lshift
;

: .Rn+/-#imm	\ --
  c" [ " $>dis  .Rn-V  curr-instr @ $0080:0000 and
  if  c" , # #"  else  c" , - # #" then $>dis
  immVldrstr@ dec>dis
  c"  ] " $>dis
;

: .PCrelV	\ --
  Rn# #15 = if
    c"  ( $" $>dis			\ )
    StartAddr @  Thumb?
    if  4 +  else  8 +  endif
    immVldrstr@  curr-instr @ $0080:0000 and
    if  +  else  -  then
    base @ >r  hex  num>dis  r> base !
    c"  )" $>dis
  endif
;


: showVLDSTR	\ $ --
  $>dis ?.ARMcond .Fd ,>dis .Rn+/-#imm  .PCrelV
;

mv: disAS/VFPextrlsiT	\ -- ; DDI0406C, table A7-19, A7-17 p272
\   mask/value------------------------- test    xt              name "
\            opc1opc2          o3  opc4
\   1110110O:OOOOxxxx:xxxx101x:xxxxxxxx  RdRnRt
mv" 11101100:1xx0xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vstmia "
mv" 11101101:0xx0xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vstmdb "
mv" 11101101:xx00xxxx:xxxx101x:xxxxxxxx %000000 showVLDSTR	vstr "
mv" 11101100:1xx1xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vldmia "
mv" 11101101:0x01xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vldmdb "
mv" 11101101:0x11xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vldmdb "
mv" 11101101:xx01xxxx:xxxx101x:xxxxxxxx %000000 showVLDSTR	vldr "
;mv

mv: disAS/VFPextrlsiA	\ -- ; DDI0406C, table A7-19, A7-17 p272
\   mask/value------------------------- test    xt              name "
\            opc1opc2          o3  opc4
\   1110110O:OOOOxxxx:xxxx101x:xxxxxxxx  RdRnRt
mv" xxxx1100:1xx0xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vstmia "
mv" xxxx1101:0xx0xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vstmdb "
mv" xxxx1101:xx00xxxx:xxxx101x:xxxxxxxx %000000 showVLDSTR	vstr "
mv" xxxx1100:1xx1xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vldmia "
mv" xxxx1101:0x01xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vldmdb "
mv" xxxx1101:0x11xxxx:xxxx101x:xxxxxxxx %000000 showVSTMxx	vldmdb "
mv" xxxx1101:xx01xxxx:xxxx101x:xxxxxxxx %000000 showVLDSTR	vldr "
;mv

\ VST1 list
\ type=0111, sets=1, regs=1, inc=1, inc=step per reg
\      1010,      2,      2      1
\      0110,      3,      3      1
\      0010,      4       4      1
\ VST2 list
\ type=1000,      1       2      1
\      1001,      1       2      2
\      0011,      2       4      2
\ VST3 list
\ type=0100,      1       3      1
\      0101,      1       3      2
\ VST4 list
\ type=0000,      1       4      1
\      0001,      1       4      2

struct /VLDSTitem	\ -- len
  int vit.type
  int vit.#regs
  int vit.inc
end-struct

create VLD/STlist	\ -- addr
\ type     regs inc
\ VST1
  %0111 ,  1 ,  1 ,
  %1010 ,  2 ,  1 ,
  %0110 ,  3 ,  1 ,
  %0010 ,  4 ,  1 ,
\ VST2
  %1000 ,  2 ,  1 ,
  %1001 ,  2 ,  2 ,
  %0011 ,  4 ,  2 ,
\ VST3
  %0100 ,  3 ,  1 ,
  %0101 ,  3 ,  2 ,
\ VST4
  %0000 ,  4 ,  1 ,
  %0001 ,  4 ,  2 ,
  -1    ,  0 ,  0 ,	\ terminator


: search-VLDSTlist	\ -- reg# #regs inc
  DVreg@
  curr-instr @ 8 rshift $0F and		\ -- reg# type
  VLD/STlist begin			\ -- reg# type addr
    dup vit.type @ -1 <>
   while
    2dup vit.type @ <>
   while
    /VLDSTitem +
  repeat then
  nip  dup vit.#regs @  swap vit.inc @	\ -- reg# #regs inc
;


: .VLDSTx-list	\ --
  c"  { " $>dis  search-VLDSTlist	\ -- reg# #regs inc
  swap 0 ?do				\ -- reg# inc
    over .DVreg bl>dis  tuck + swap
  loop
  2drop  c"  } " $>dis
;

: (.size)	\ 0..3 --
  c"  ." $>dis
  8 swap lshift dec>dis bl>dis
;

: size7:6	\ -- u ; 0..3
  curr-instr @ 6 rshift $03 and
;

: size11:10	\ -- u ; 0..3
  curr-instr @ #10 rshift $03 and
;

: .size7:6	\ -- ; for VLD/ST
  size7:6 (.size)
;

: .size11:10	\ -- ; for VLD/ST
  size11:10 (.size)
;

: .align5:4	\ --
  curr-instr @ 4 rshift $03 and
  dup if
    c" {@" $>dis
    32 swap lshift dec>dis
    c" } " $>dis
  else
    drop
  endif
;

: .VLDSTX-mem/align	\ --
  c"  [ " $>dis .Rn-V .align5:4 c"  ] " $>dis
;

: .wb/Rm	\ --
  case  Rm#
    #15 of  endof
    #13 of  c"  ! " $>dis  endof
      .Rm-V
  endcase
;

: .{Dd[index]}	\ --
  c"  { " $>dis
  .Dd c" [" $>dis
  curr-instr @ 4 rshift $0F and		\ index_align
  case  size11:10
    0 of  1 rshift  endof
    1 of  2 rshift  endof
    2 of  3 rshift  endof
    3 of  drop -1  endof
  endcase
  dec>dis  c" ] } " $>dis
;

create VLD/STlist-seall	\ -- addr
\ opc-T            regs inc
\ VLD1
%1100:0000:0000 ,  1 ,  1 ,
%1100:0010:0000 ,  2 ,  1 ,
\ VLD2
%1101:0000:0000 ,  2 ,  1 ,
%1101:0010:0000 ,  2 ,  2 ,
\ VLD3
%1110:0000:0000 ,  3 ,  1 ,
%1110:0010:0000 ,  3 ,  2 ,
\ VLD4
%1111:0000:0000 ,  4 ,  1 ,
%1111:0010:0000 ,  4 ,  2 ,
-1 , 0 , 0 ,

: search-VLDseall	\ -- reg# #regs inc
  DVreg@  curr-instr @ $0F20 and	\ -- reg# type
  VLD/STlist-seall begin		\ -- reg# type addr
    dup vit.type @ -1 <>
   while
    2dup vit.type @ <>
   while
    /VLDSTitem +
  repeat then
  nip  dup vit.#regs @  swap vit.inc @	\ -- reg# #regs inc
;

: .VLDx-listseall	\ --
  c"  { " $>dis  search-VLDseall	\ -- reg# #regs inc
  swap 0 ?do				\ -- reg# inc
    over .DVreg c" [] " $>dis  tuck + swap
  loop
  2drop  c"  } " $>dis
;

: .align7:4	\ --
  curr-instr @ 4 rshift $03 and
  case  size11:10
    0 of  drop  endof
    1 of  $03 and $01 = if  c" {@16}" $>dis  endif  endof
    2 of  $07 and $03 = if  c" {@32}" $>dis  endif  endof
      nip
  endcase
;

: .align-a4	\ --
  curr-instr @ $0010 and if		\ a=1
    c"  {@" $>dis
    8 size7:6 lshift dec>dis
    c" } " $>dis
  endif
;

: .VLDSTX-mem/ind/align	\ --
  c"  [ " $>dis .Rn-V .align7:4 c"  ] " $>dis
;

: .VLDST-memseall	\ --
  c"  [ " $>dis .Rn-V .align-a4 c"  ] " $>dis
;

: showVLDSTu-me	\ $ --
  $>dis  .size7:6 .VLDSTx-list .VLDSTX-mem/align  .wb/Rm
;

: showVLDSTu-se1l	\ $ --
  $>dis  .size11:10 .{Dd[index]} .VLDSTX-mem/ind/align  .wb/Rm
;

: showVLDSTu-seall	\ $ --
  $>dis  .size7:6 .VLDx-listseall .VLDST-memseall  .wb/Rm
;

mv: disASeleT	\ -- ; DDI0406C, table A7-20, A7-7 p273
\   mask/value------------------------- test    xt              name "
\   11111001:AxL0xxxx:xxxxBBBB:xxxxxxxx  RdRnRt
mv" 11111001:0x00xxxx:xxxx0010:xxxxxxxx %000000 showVLDSTu-me	vst1 "
mv" 11111001:0x10xxxx:xxxx0010:xxxxxxxx %000000 showVLDSTu-me	vld1 "
mv" 11111001:0x00xxxx:xxxx011x:xxxxxxxx %000000 showVLDSTu-me	vst1 "
mv" 11111001:0x10xxxx:xxxx011x:xxxxxxxx %000000 showVLDSTu-me	vld1 "
mv" 11111001:0x00xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-me	vst1 "
mv" 11111001:0x10xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-me	vld1 "
mv" 11111001:0x00xxxx:xxxx0011:xxxxxxxx %000000 showVLDSTu-me	vst2 "
mv" 11111001:0x10xxxx:xxxx0011:xxxxxxxx %000000 showVLDSTu-me	vld2 "
mv" 11111001:0x00xxxx:xxxx100x:xxxxxxxx %000000 showVLDSTu-me	vst2 "
mv" 11111001:0x10xxxx:xxxx100x:xxxxxxxx %000000 showVLDSTu-me	vld2 "
mv" 11111001:0x00xxxx:xxxx010x:xxxxxxxx %000000 showVLDSTu-me	vst3 "
mv" 11111001:0x10xxxx:xxxx010x:xxxxxxxx %000000 showVLDSTu-me	vld3 "
mv" 11111001:0x00xxxx:xxxx000x:xxxxxxxx %000000 showVLDSTu-me	vst4 "
mv" 11111001:0x10xxxx:xxxx000x:xxxxxxxx %000000 showVLDSTu-me	vld4 "
\   mask/value------------------------- test    xt              name "
\   11111001:AxL0xxxx:xxxxBBBB:xxxxxxxx  RdRnRt
mv" 11111001:1x00xxxx:xxxx0x00:xxxxxxxx %000000 showVLDSTu-se1l	vst1 "
mv" 11111001:1x10xxxx:xxxx0x00:xxxxxxxx %000000 showVLDSTu-se1l	vld1 "
mv" 11111001:1x00xxxx:xxxx1000:xxxxxxxx %000000 showVLDSTu-se1l	vst1 "
mv" 11111001:1x10xxxx:xxxx1000:xxxxxxxx %000000 showVLDSTu-se1l	vld1 "
mv" 11111001:1x00xxxx:xxxx0x01:xxxxxxxx %000000 showVLDSTu-se1l	vst2 "
mv" 11111001:1x10xxxx:xxxx0x01:xxxxxxxx %000000 showVLDSTu-se1l	vld2 "
mv" 11111001:1x00xxxx:xxxx1001:xxxxxxxx %000000 showVLDSTu-se1l	vst2 "
mv" 11111001:1x10xxxx:xxxx1001:xxxxxxxx %000000 showVLDSTu-se1l	vld2 "
mv" 11111001:1x00xxxx:xxxx0x10:xxxxxxxx %000000 showVLDSTu-se1l	vst3 "
mv" 11111001:1x10xxxx:xxxx0x10:xxxxxxxx %000000 showVLDSTu-se1l	vld3 "
mv" 11111001:1x00xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-se1l	vst3 "
mv" 11111001:1x10xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-se1l	vld3 "
mv" 11111001:1x00xxxx:xxxx0x11:xxxxxxxx %000000 showVLDSTu-se1l	vst4 "
mv" 11111001:1x10xxxx:xxxx0x11:xxxxxxxx %000000 showVLDSTu-se1l	vld4 "
mv" 11111001:1x00xxxx:xxxx1011:xxxxxxxx %000000 showVLDSTu-se1l	vst4 "
mv" 11111001:1x10xxxx:xxxx1011:xxxxxxxx %000000 showVLDSTu-se1l	vld4 "
\   mask/value------------------------- test    xt              name "
\   11111001:AxL0xxxx:xxxxBBBB:xxxxxxxx  RdRnRt
mv" 11111001:1x10xxxx:xxxx1100:xxxxxxxx %000000 showVLDSTu-seall vld1 "
mv" 11111001:1x10xxxx:xxxx1101:xxxxxxxx %000000 showVLDSTu-seall vld2 "
mv" 11111001:1x10xxxx:xxxx1110:xxxxxxxx %000000 showVLDSTu-seall vld3 "
mv" 11111001:1x10xxxx:xxxx1111:xxxxxxxx %000000 showVLDSTu-seall vld4 "
;mv

mv: disASeleA	\ -- ; DDI0406C, table A7-20, A7-7 p273
\   mask/value------------------------- test    xt              name "
\   11110100:AxL0xxxx:xxxxBBBB:xxxxxxxx  RdRnRt
mv" 11110100:0x00xxxx:xxxx0010:xxxxxxxx %000000 showVLDSTu-me	vst1 "
mv" 11110100:0x10xxxx:xxxx0010:xxxxxxxx %000000 showVLDSTu-me	vld1 "
mv" 11110100:0x00xxxx:xxxx011x:xxxxxxxx %000000 showVLDSTu-me	vst1 "
mv" 11110100:0x10xxxx:xxxx011x:xxxxxxxx %000000 showVLDSTu-me	vld1 "
mv" 11110100:0x00xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-me	vst1 "
mv" 11110100:0x10xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-me	vld1 "
mv" 11110100:0x00xxxx:xxxx0011:xxxxxxxx %000000 showVLDSTu-me	vst2 "
mv" 11110100:0x10xxxx:xxxx0011:xxxxxxxx %000000 showVLDSTu-me	vld2 "
mv" 11110100:0x00xxxx:xxxx100x:xxxxxxxx %000000 showVLDSTu-me	vst2 "
mv" 11110100:0x10xxxx:xxxx100x:xxxxxxxx %000000 showVLDSTu-me	vld2 "
mv" 11110100:0x00xxxx:xxxx010x:xxxxxxxx %000000 showVLDSTu-me	vst3 "
mv" 11110100:0x10xxxx:xxxx010x:xxxxxxxx %000000 showVLDSTu-me	vld3 "
mv" 11110100:0x00xxxx:xxxx000x:xxxxxxxx %000000 showVLDSTu-me	vst4 "
mv" 11110100:0x10xxxx:xxxx000x:xxxxxxxx %000000 showVLDSTu-me	vld4 "
\   mask/value------------------------- test    xt              name "
\   11110100:AxL0xxxx:xxxxBBBB:xxxxxxxx  RdRnRt
mv" 11110100:1x00xxxx:xxxx0x00:xxxxxxxx %000000 showVLDSTu-se1l	vst1 "
mv" 11110100:1x10xxxx:xxxx0x00:xxxxxxxx %000000 showVLDSTu-se1l	vld1 "
mv" 11110100:1x00xxxx:xxxx1000:xxxxxxxx %000000 showVLDSTu-se1l	vst1 "
mv" 11110100:1x10xxxx:xxxx1000:xxxxxxxx %000000 showVLDSTu-se1l	vld1 "
mv" 11110100:1x00xxxx:xxxx0x01:xxxxxxxx %000000 showVLDSTu-se1l	vst2 "
mv" 11110100:1x10xxxx:xxxx0x01:xxxxxxxx %000000 showVLDSTu-se1l	vld2 "
mv" 11110100:1x00xxxx:xxxx1001:xxxxxxxx %000000 showVLDSTu-se1l	vst2 "
mv" 11110100:1x10xxxx:xxxx1001:xxxxxxxx %000000 showVLDSTu-se1l	vld2 "
mv" 11110100:1x00xxxx:xxxx0x10:xxxxxxxx %000000 showVLDSTu-se1l	vst3 "
mv" 11110100:1x10xxxx:xxxx0x10:xxxxxxxx %000000 showVLDSTu-se1l	vld3 "
mv" 11110100:1x00xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-se1l	vst3 "
mv" 11110100:1x10xxxx:xxxx1010:xxxxxxxx %000000 showVLDSTu-se1l	vld3 "
mv" 11110100:1x00xxxx:xxxx0x11:xxxxxxxx %000000 showVLDSTu-se1l	vst4 "
mv" 11110100:1x10xxxx:xxxx0x11:xxxxxxxx %000000 showVLDSTu-se1l	vld4 "
mv" 11110100:1x00xxxx:xxxx1011:xxxxxxxx %000000 showVLDSTu-se1l	vst4 "
mv" 11110100:1x10xxxx:xxxx1011:xxxxxxxx %000000 showVLDSTu-se1l	vld4 "
\   mask/value------------------------- test    xt              name "
\   11110100:AxL0xxxx:xxxxBBBB:xxxxxxxx  RdRnRt
mv" 11110100:1x10xxxx:xxxx1100:xxxxxxxx %000000 showVLDSTu-seall vld1 "
mv" 11110100:1x10xxxx:xxxx1101:xxxxxxxx %000000 showVLDSTu-seall vld2 "
mv" 11110100:1x10xxxx:xxxx1110:xxxxxxxx %000000 showVLDSTu-seall vld3 "
mv" 11110100:1x10xxxx:xxxx1111:xxxxxxxx %000000 showVLDSTu-seall vld4 "
;mv

: showSnRt	\ $ --
  $>dis  .Sn ,>dis .Rt-V
;

: showRtSn	\ $ --
  $>dis  .Rt-V ,>dis .Sn
;

: .VFPspecreg	\ --
  curr-instr @ $000F:0000 and # 16 rshift
  case
    $00 of  c" FPSID"  endof
    $01 of  c" FPSCR"  endof
    $08 of  c" FPEXC"  endof
    $09 of  c" FPINST"  endof
    $0A of  c" FPINST2"  endof
      c" FP???"
  endcase $>dis
;

: showSpecRt	\ $ --
  $>dis  .VFPspecreg ,>dis .Rt-V
;

: showRtSpec	\ $ --
  $>dis  .Rt-V ,>dis .VFPspecreg
;

: get22:21:6:5	\ -- abcd
  curr-instr @ dup
  #19 rshift $0C and
  swap 5 rshift $03 and or
;

: idx/size22:21:6:5	\ -- index size
  get22:21:6:5 dup $08 and
  if  $07 and  8 exit  endif
  dup $09 and $01 =
  if  1 rshift $03 and  16 exit  endif
  dup $0B and  $00 =
  if  2 rshift $01 and  32 exit  endif
  drop 0 32
;

: .sizeX	\ size -- ; 8/16/32/64
  c" ." $>dis dec>dis  bl>dis
;

: .sizeSUX	\ size -- ; 8/16/32/64
  dup #32 = if
    c" ." $>dis
  else
    curr-instr @ $0080 and
    if  c" u"  else  c" s"  then $>dis
  endif
  dec>dis  bl>dis
;

: .size-be	\ -- ; b = bit 22, e = bit 5
  curr-instr @ dup
  #21 rshift $02 and
  swap 5 rshift $01 and or
  case
    0 of  #32  endof
    1 of  #16  endof
    2 of  8  endof
     64 swap
  endcase
  .sizeX
;

: show-szD[x]Rt	\ $ --
  $>dis  idx/size22:21:6:5 .sizeX .Dn .index ,>dis  .Rt-V
;

: show-szRtD[x]	\ $ --
  $>dis  idx/size22:21:6:5 .sizeSUX  .Rt-V ,>dis .Dn .index
;

: .Adsz21	\ --
  curr-instr @ $0020:0000 and
  if  .Qd  else  .Dd  then
;

: show-szAdRt	\ $ --
  $>dis  .size-be  .Adsz21 ,>dis .Rt-V
;

mv: disARMcoreT	\ -- ; DDI0406C, table A7-22, A7-8, p276
\   mask/value------------------------- test    xt              name "
\   11101110:AAALxxxx:xxxx101C:xBB1xxxx  RdRnRt
mv" 11101110:0000xxxx:xxxx1010:xxx1xxxx %000000 showSnRt	vmov "
mv" 11101110:0001xxxx:xxxx1010:xxx1xxxx %000000 showRtSn	vmov "
mv" 11101110:1110xxxx:xxxx1010:xxx1xxxx %000000 showSpecRt	vmsr "
mv" 11101110:1111xxxx:xxxx1010:xxx1xxxx %000000 showRtSpec	vmrs "
mv" 11101110:0xx0xxxx:xxxx1011:xxx1xxxx %000000 show-szD[x]Rt	vmov "
mv" 11101110:xxx1xxxx:xxxx1011:xxx1xxxx %000000 show-szRtD[x]	vmov "
mv" 11101110:1xx0xxxx:xxxx1010:xxx1xxxx %000000 show-szAdRt	vdup "
;mv

mv: disARMcoreA	\ -- ; DDI0406C, table A7-22, A7-8, p276
\   mask/value------------------------- test    xt              name "
\   xxxx1110:AAALxxxx:xxxx101C:xBB1xxxx  RdRnRt
mv" xxxx1110:0000xxxx:xxxx1010:xxx1xxxx %000000 showSnRt	vmov "
mv" xxxx1110:0001xxxx:xxxx1010:xxx1xxxx %000000 showRtSn	vmov "
mv" xxxx1110:1110xxxx:xxxx1010:xxx1xxxx %000000 showSpecRt	vmsr "
mv" xxxx1110:1111xxxx:xxxx1010:xxx1xxxx %000000 showRtSpec	vmrs "
mv" xxxx1110:0xx0xxxx:xxxx1011:xxx1xxxx %000000 show-szD[x]Rt	vmov "
mv" xxxx1110:xxx1xxxx:xxxx1011:xxx1xxxx %000000 show-szRtD[x]	vmov "
mv" xxxx1110:1xx0xxxx:xxxx1010:xxx1xxxx %000000 show-szAdRt	vdup "
;mv

: showSmSm1RtRt2	\ $ --
  $>dis  MSreg@ dup .Sreg ,>dis  1+ .Sreg ,>dis  .Rt-V ,>dis  .Rt2-V
;

: showRtRt2SmSm1	\ $ --
  $>dis   .Rt-V ,>dis  .Rt2-V ,>dis  MSreg@ dup .Sreg ,>dis  1+ .Sreg
;

: showDmRtRt2	\ $ --
  $>dis  .Dm ,>dis  .Rt-V ,>dis  .Rt2-V
;

: showRtRt2Dm	\ $ --
  $>dis   .Rt-V ,>dis  .Rt2-V ,>dis  .Dm
;

mv: disARMcore64T	\ -- ; DDI0406C, table A7-23, A7-9, p277
\   mask/value------------------------- test    xt              name "
\   11101100:010xxxxx:xxxx101C:OOOOxxxx  RdRnRt
mv" 11101100:0100xxxx:xxxx1010:00x1xxxx %000000 showSmSm1RtRt2	vmov "
mv" 11101100:0101xxxx:xxxx1010:00x1xxxx %000000 showRtRt2SmSm1	vmov "
mv" 11101100:0100xxxx:xxxx1011:00x1xxxx %000000 showDmRtRt2	vmov "
mv" 11101100:0101xxxx:xxxx1011:00x1xxxx %000000 showRtRt2Dm	vmov "
;mv

mv: disARMcore64A	\ -- ; DDI0406C, table A7-23, A7-9, p277
\   mask/value------------------------- test    xt              name "
\   11101100:010xxxxx:xxxx101C:OOOOxxxx  RdRnRt
mv" xxxx1100:0100xxxx:xxxx1010:00x1xxxx %000000 showSmSm1RtRt2	vmov "
mv" xxxx1100:0101xxxx:xxxx1010:00x1xxxx %000000 showRtRt2SmSm1	vmov "
mv" xxxx1100:0100xxxx:xxxx1011:00x1xxxx %000000 showDmRtRt2	vmov "
mv" xxxx1100:0101xxxx:xxxx1011:00x1xxxx %000000 showRtRt2Dm	vmov "
;mv

: CheckASdpi?	\ -- flag ; 0 = not an ASIMD DPI instruction
  Curr-Instr @ Thumb? if
    dup $1000:0000 and 0<> ASIMD-ubit !
    $EF00:0000 and $EF00:0000 = dup
    if  disASdpiT drop  endif
  else
    dup $0100:0000 and 0<> ASIMD-ubit !
    $FE00:0000 and $F200:0000 = dup
    if  disASdpiA drop endif
  endif
;

: CheckVFPdpi?	\ -- flag ; 0 = not an VFP DPI instruction; A7.5, p270, DDI0406C
  Curr-Instr @ Thumb? if
    $EE00:0E10 and $EE00:0A00 = if
      Curr-Instr @ $1000:0000 and
      if  Undef  else  disVFPdpiT  endif
      true
    else  0 endif
  else
    $0F00:0E10 and $0E00:0A00 = if
      Curr-Instr @ $F000:0000 and $F000:0000 =
      if  Undef  else  disVFPdpiA  endif
      true
    else  0 endif
  endif
;

: checkAS/VFPextrlsi?	\ -- flag ; 0 = not extension register load/store ins
  Thumb? if
    Curr-Instr @ $EE00:0E00 and $EC00:0A00 = if
      Curr-Instr @ $1000:0000 and
      if  Undef  else  disAS/VFPextrlsiT  endif
      true
    else  0  endif
  else
    Curr-Instr @ $0E00:0E00 and $0C00:0A00 = if
      Curr-Instr @ $F000:0000 and $F000:0000 =
      if  Undef  else  disAS/VFPextrlsiA  endif
      true
    else  0  endif
  endif
;

: checkASele?	\ -- flag ; 0 = not ASIMD element or structure ins
  Thumb? if
    Curr-Instr @ $FF10:0000 and $F900:0000 =
    dup if  disASeleT  endif
  else
    Curr-Instr @ $FF10:0000 and $F400:0000 =
    dup if  disASeleA  endif
  endif
;

: checkARMcore?	\ -- flag ; 0 = not 8/16/32 bit ARM core reg transfer
  Thumb? if
    Curr-Instr @ $EF00:0E10 and $EE00:0A10 = if
      Curr-Instr @ $1000:0000 and
      if  Undef  else  disARMcoreT  endif
      true
    else  0  endif
  else
    Curr-Instr @ $0F00:0E10 and $0E00:0A10 = if
      Curr-Instr @ $F000:0000 and $F000:0000 =
      if  Undef  else  disARMcoreA  endif
      true
    else  0  endif
  endif
;

: checkARMcore64?	\ -- flag ; 0 = not 64 bit core reg transfer
  Thumb? if
    Curr-Instr @ $EFE0:0E00 and $EC40:0A00 = if
      Curr-Instr @ $1000:0000 and
      if  Undef  else  disARMcore64T  endif
      true
    else  0  endif
  else
    Curr-Instr @ $0FE0:0E00 and $0C40:0A00 = if
      Curr-Instr @ $F000:0000 and $F000:0000 =
      if  Undef  else  disARMcore64A  endif
      true
    else  0  endif
  endif
;

: (CheckAS/VFP?)	\ -- flag ; 0 = not an ASIMD or VFP instruction
  CheckASdpi? if  true exit  endif
  CheckVFPdpi? if  true exit  endif
  checkARMcore? if  true exit  endif
  checkARMcore64? if  true exit  endif
  checkAS/VFPextrlsi? if  true exit  endif
  checkASele? if  true exit  endif
  0
;  assign (CheckAS/VFP?) to-do CheckAS/VFP?

\ *********************
\ Dealing with NEXT/RET
\ *********************

: ?NEXT         \ addr(t) -- flag ; true if string is NEXT
  Thumb? if
    <itcount> @ 0= if
      case  w@c(t)
        $4770 of  true  endof		\ bx r14
        $46F7 of  true  endof		\ mov  pc, lr/r14
        $BD00 of  true  endof		\ pop { pc }
          0 swap
      endcase
    else
      drop 0
    endif
  else
    l@c(t)
    dup  $E8BD:8000 =			\ ldmia r13 ! { pc }
    over $E49D:F004 = or		\ ldr pc, [ r13 ], # $04 ; SFP010
    over $E1A0:F00E = or		\ mov pc, link
    swap $E12F:FF1E = or		\ bx link
  endif
  dup
  if  next-found on  endif
;


\ **********************
\ Organising the display
\ **********************

: .DISLINE      \ -- ; output the dump information and dissassembly
  cr  ." ( "  StartAddr @ .dword  space  .ByteStream space .AsciiStream ."  )"
  DisOpCol >pos
  DisLine$ -TRAILING TYPE ;

((
: DASHES        \ n --
   0 2DUP +
   IF DO [char] - EMIT LOOP ELSE 2DROP THEN ;

: ?LABEL        \ addr -- nfa true / false
   >NAME DUP -1 <> DUP 0=
   IF  NIP  THEN  ;

: LABEL>DIS	\ nfa --
   CR 72 DASHES
   CR 34 SPACES
\   NAME>  ID.
   .name
   CR 72 DASHES ;
))

: ft-init-dis   \ from to -- ; initialise disassembly
  DisplayStart
  Thumb? if
    1+ -2 and			\ align
    ToDisAddr !			\ set end of disassembly
    1+ -2 and			\ align
    SET-DisAddr			\ set start of disassembler
  else
    3 + -4 and			\ align
    ToDisAddr !			\ set end of disassembly
    3 + -4 and			\ align
    SET-DisAddr			\ set start of disassembler
  endif
  <itcount> off			\ reset IT instruction count
  next-found off                \ reset next found flag
  branch-target off		\ reset max branch target
;

: al-init-dis   \ addr len -- ; initialise disassembly
  over + ft-init-dis            \ convert addr/len to from/to
;

: dis-cont?     \ -- flag ; true if in disassembly range
  DisAddr ToDisAddr @ u<        \ in range
  halt? not  and                \ and halt key not pressed
;

: dasm-done?    \ -- t/f ; true if dasm finished
  next-found @                  \ next found
  if  branch-target @
      disaddr u<                \ and no branch beyond this position
      if  -1
      else  next-found off      \ beyond, so reset next-found flag
            branch-target @     \ return true if gap
            disaddr $100 + u>   \ is greater than 256 bytes (arbitrary)
      endif
  else  0
  endif
;

: get-cfa       \ -- cfa flag ; get CFA of word name in input stream
  get-token  cr dup count type
  <xfind> on  find(t)_ans  <xfind> off
  dup 0= if				\ if not a target word, try a label
    drop                                \ discard flag
    'labels(s) @ #threads(s)+ v-find(s) \ look for a label
  endif
  dup ?undef                            \ error if nothing found
  swap
  dup resolved? ?undef  @(s)            \ replace symbol address by value
  swap
;

: (disasm)      \ --
  base @ >r  hex
  DISINIT
  Thumb?
  if  do-main_T  else  do-main_ARM  endif
  disclose  -itcount
  r> base !
;

: 1DISASM       \ --
  depth >r
  DisAddr ?NEXT
  IF
    (DISASM)  ( NEXT>DIS )
  ELSE
    (DISASM)
  THEN
  DisAddr ToDisAddr @ u<=
  IF .DISLINE THEN
  depth r> <> if
    cr ." Stack fault in disassembler"
  endif
;

: (dasm)        \ -- ; disassemble code, range already set      SFP004
  DisAddr                               \ start address
  0
  BEGIN
      Dis-cont?  dasm-done? not  and
    while
      1DISASM
      1+
  repeat
  cr  DisAddr rot - u. ." bytes, "	\ show length
  u. ." instructions."			\ SFP002
;

: al-dasm       \ addr len -- ; disassemble code over the given range
  al-init-dis
  (dasm)
  cr
;

: ft-dasm       \ from to -- ; disassemble code over the given range
  ft-init-dis
  (dasm)
  CR
;


\ ******************
\ Words for the user
\ ******************

also C-C DEFINITIONS DECIMAL

: xDISASM/al    \ addr len --
\ *G Disassemble from addr for len bytes.
  al-init-dis
  BEGIN
      Dis-cont?
    WHILE
      (DISASM) .DISLINE
  REPEAT
;

: xdisasm/ft    \ from to --
\ *G Disassemble from address FROM to address TO.
  ft-init-dis
  BEGIN
      Dis-cont?
    WHILE
      1DISASM
  REPEAT
  CR
;

: xdisasm/f     \ addr --
\ *G Disassemble from address FROM. The NEXT, macro is displayed as a NEXT,
\ ** macro. The display stops when the first NEXT, macro is encountered with
\ ** no outstanding forward branch, or if the forward branch is over 256 bytes.
  chere(t) ft-init-dis
  (dasm)
;

: xdasm         \ -- ; DASM <word>
\ *G Disassemble a word. Use in the form:
\ *C   DASM <word>
  get-cfa swap                  \ find CFA or word
  dup $200 +                    \ restrict range to 512 bytes
  ft-init-dis                   \ set up from and to addresses
  (dasm)                        \ do disassembly
  0>                            \ was it immediate ?
  If                            \ yes, say so
    cr 5 spaces ." IMMEDIATE"
  Endif
  CR
;

: xdis
  xdasm  ;

: xsee
  xdasm  ;

only forth  also c-c definitions


\ ******************************************************************
\ Expose the user words for cross interpretation and Umbilical Forth
\ ******************************************************************

also cc/i

: xDISASM/al    cc/i    \ addr len --
  xDISASM/AL  ;

: xdisasm/ft    cc/i    \ from to --
  xdisasm/ft  ;

: xdisasm/f     cc/i    \ addr --
  xdisasm/f  ;

: xdasm         cc/i    \ -- ; disassemble word; XDASM <word>
  xdasm  ;

: xdis          cc/i    \ -- ; disassemble word; XDIS <word>
  xdasm  ;

: xsee          cc/i    \ -- ; disassemble word; XSEE <word>
  xdasm  ;

: disasm/al	cc/i	\ addr len --
  xDISASM/AL  ;

: disasm/ft	cc/i	\ from to --
  xdisasm/ft  ;

: disasm/f	cc/i	\ addr --
  xdisasm/f  ;

: dasm          cc/i    \ -- ; disassemble word; DASM <word>
  xdasm  ;

: dis           cc/i    \ -- ; disassemble word; DIS <word>
  xdasm  ;

: see           cc/i    \ -- ; disassemble word; SEE <word>
  xdasm  ;

previous definitions
decimal

