\ XoptCortex.fth - Optimiser for ARM/Cortex-M cross compiler v7

((
Copyright (c) 2000-2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018
MicroProcessor Engineering
133 Hill Lane
Southampton SO15 5AF
England

tel: +44 (0)23 8063 1441
fax: +44 (0)23 8033 9691
net: mpe@mpeforth.com
     tech-support@mpeforth.com
web: www.mpeforth.com

From North America, our telephone and fax numbers are:
  011 44 23 8063 1441
  011 44 23 8033 9691

PLEASE NOTE OUR PHONE NUMBER CHANGE


To do
=====
Add -! and friends. Then modify +! to check for a negative literal,
which is usually -1. Convert these to -! operations.

Cortex literal addresses for store operations can often be
generated using an ADR instruction. N.B. Align(PC,4).

Add a register preload using LDMIA for code generator that know
how many inputs will used. This saves a lot of space, especially
if PSP is a high register.

Code generator for ?NEGATE ( n1 flag -- n2 ). Not often used
but speeds up high level math routines.

Be more aggressive in getting stack items into registers. This will
avoid duplicate loads (3 cycles) at the expense of occasional extra
stores (2 cycles).

Reduce use of fixed registers R0 and R1 so that these can go into
the stack pool. See the 386 changes.

Use R7 as DO..LOOP update register. This will save 4 cycles per
loop, and 2/3 cycles per I. If both items are in registers, we can
reduce I to an ADD (1 cycle) and LOOP/+LOOP to two ADDs (2 cycles).

Clarify and check alignment in ", and friends.

Improve literal generation. LDR rx, [ PC, # n ] costs three cycles.
Many can be two ALU instructions if done better.

Add new detection for "reg1 reg2 + @/!" from 386.

Check/add M+ code generation - check D+ also.

Remove shifterops to get (say) "ADD dest, reg1, reg2 ASR # n".

Add MAKEBESTTOS and MAKEBESTNOS from the 68k/386 compilers.

Add the register move elimination from the 68HC12/MSP430 compiler.


History
=======
ARM and Cortex
--------------
20180702 SFP129 Corrected CM0 local variable stores.
20171113 SFP128	Added WOR! WAND! WBIC! WXOR!
20170823 SFP127	Corrected C" and friends.
20170524 SFP126 Added FSP! intrinsic
20170420 SFP125 Improved CM3+ small literals (0..255).
20161106 SFP124 C_UP! does not touch R0 for Thumb devices. This
		permits an AAPCS C call to start Forth.
20160824 SFP123 Added code generators for DI and EI, DFI and EFI.
20160802 SFP122 Removed use of SWP instruction for Cortex A/R.
20160322 SFP121 Added Z" support.
20160118 SFP120 Freed R8 in ARM32 for use as FSP.
20150528 SFP119 Added DSB#0F and ISB#0F intrinsics.
20150224 SFP118	Exposed +c for multiprecision adds.
20150115 SFP117 Added restrictions when running ARM32 code on
		Cortex-A/R.
20140826 SFP116 Fixed >BODY and friends.
20140718 SFP115 Update for VFX Forth for ARM Linux kernel.
20140628 SFP114 Corrected EXECUTE for Cortex-M0/M1.
		Corrected
20140309 SFP113 Corrected sign extended fetches for Cortex-M0/M1.
20140211 SFP112 Corrected compare for negative literals.
20131216 SFP111 Corrected NUWWIDEN.
20120816 SFP110	Added WFI and WFE intrinsics.
20120612 SFP109 Refactored string routines and added escapes.
20111117 SFP108 Added byte reversal intrinsics.
20110614 SFP107 Cortex M0/M1 update.
20100406 SFP106 Added short/long branch selection for Thumb.
20100208 SFP105 Added .S to ADDSTOREOP operations.
		ARM: Replaced LDM/STM with LDR/STR for single registers.
20100125 SFP104 Cortex STRx has no literal form!
		Corrected C_/T2 and C_U/T2
20100123 SFP103 Added SET-SP intrinsic.
20100111 SFP102 Added SYS@ and SYS! intrinsics.
20091029 SFP101 Removed switches for ARM mode before ARM7TDMI

ARM only
--------
20090818 SFP080 Improved use of BIC instruction.
20090724 SFP079 Restored code generators for U<= and U>=.
20090722 SFP078 Added forward reference protection to run-times.
20090710 SFP077 Permitted far calls with +LONGCALLS.
20090709 SBD076 Added code to handle more far call conditions.
20090218 SFP075 Added more loop alignment exposure.
20080816 SFP074 S>D by 31 ARSHIFT. Updated M+.
20080809 SFP073 Corrected divide by powers of two for negative dividends.
20080606 SFP072 DocGenned.
20071004 SFP071 Added LDR/STR selection over LDM/STM.
		STR is faster than STM for one register on ARM9 and
		above in all cases. LDR is faster than LDM on ARM9
		for dest<>PC. There is no measured impact on
		performance for ARM7 devices. However, this needs
		to be checked for XScale devices.
20070904 SFP070 Refactored C_SHUFFLE.
		Restored EXECUTE optimiser.
20070301 SFP069 Improved local variable entry and exit code generation.
20060920 SFP068 Protect INLINING and +LEAFCALLS.
20060329 SFP067 Corrected <lit> + cbranch.
20050323 SFP066 Corrected (SHIFTER) when RD is not the same as RNOS.
20050303 SFP065 Corrected FINDDEST2 when out of registers.
20050223 SFP064 Corrected C_W! for systems without STRH.
20041109 SFP063 Added loop head alignment.
		Added optimisers for ON/OFF, INCR/DECR, +! and friends.
		Changed literal pool handling to end of word.
20041108 SFP062 Added flag set optimisation for AND IF and dyads
20041005 SFP061 Forced forward branches to use FFFFFF offset to help
		Umbilical Forth Flash systems.
20040530 SFP060 Load ALL multiply referenced literals into registers.
20040219 SFP059 Added delayed save of return stack to elimiate
		memory operations for words that don't need it.
		Directives are:
		  +LEAFCALLS -LEAFCALLS LEAFCALLS? ISLEAF
20040218 SFP058 Made non-immediate right shifts check for shift count
		of 0 (no action) because ARM treats this as shift=32.
20040207 SFP057 Removed target optimisation of >BODY and friends
		if the long calls switch is active.
20030717 SFP056 Deleted @LIT and reverted to R#>A.
		Made R#>A prevent binary inlining.
20030519 SFP055 Prevented binary inlining when literal pool
		has entries when ';' is encountered.
20030509 SFP054 Take advantage of RSB for subtracts.
		Modified DUP OVER and TUCK to load literals that
		do not fit into 12 bit format. This is NOT worthwhile
		for deep stack items, nor is it worth doing for
		COMLOAD2 and friends, probably because it increases
		the amount of shuffle code.
		Improved code generation of local arrays.
20030319 SFP053 Fixed M* and UM* which use the SMULL and UMULL
		instructions which do NOT accept a literal.
20030120 SFP052 Corrected (COMDYAD) for LIT + optimisations.
20021030 SFP051 Corrected ABS after screwup using SDCMP0,
20021008 SFP050	Added BOR! BAND! BXOR! BBIC!.
		Restored /STRING optimiser for parse performance.
20021004 SFP049 Exposed the ARSHIFT code generator.
20020916 SFP048 Changed CMP1s to use CMP instruction rather than MOV .S
		because the carry flag is not always cleared.
20020710 SFP047 Fixed problem in fetch/store operators for 16 bit operations.
20020618 SFP046 Added 3DUP
20010716 SFP045	Added OR! AND! XOR! BIC!
20010330 SFP044 Added Z",x(T)
		Added ROR and ROL code generators.
20001019 SFP043 Corrected xxxWIDEN to match 386 compiler
		Modified string generation primitives to match token compiler.
		Modified VALUE DEFER xVARIABLE to force alignment
20000713 SFP042 Added ?NEGATE as a macro for floating point pack
20000710 SFP041 Added WVARIABLE
		Added and removed /STRING because of code increase
20000620 SFP040 Reduced size of USER variables; BL douser, offset
		Rewrote C_PRIM_* because the first book lied!
		MUL rd, rm, rs  MUST NOT have rd=rm!
		Enhanced *
		Added / for simple values.
20000618 SFP039 Added C_VAL.
		Corrected LITPLUS-SEQ.
20000606 SFP038	Brought up to v6.1.
		Removed DTC? STC? compilation options.
20000204 SFP037	Added compatibility noops for
		+SHORT-BRANCES -SHORT-BRANCHES
		Added EXECUTE
20000202 SFP036	Updated FORCENEWxx to preserve working stack depth
		Enhanced TO-DO C_USERCHILD
20000202 SFP035	Refactored entry/exit code, corrected C_CREATE(CC)
		Added UM* M*
		Modified S_XCHG
		Added FLUSHLITPOOL to C_BRANCH< C_BRANCH>
		Enhanced S>D for literals
20000201 SFP034	Inline code generation of MAKELVs
		Exposed UNLOOP
		Added MIN MAX generators
20000131 SFP033	Added >BODY NxxWIDEN RPICK D+ D- M+
		Fixed bug in +LOOP
		Improved ?LEAVE
20000130 SFP032	Added literal pool
20000130 SFP031	Added UP@ SP@ RP@ LP@ UP! SP! RP! LP!
		Added CELL+ CELL- S>D
20000130 SFP030	Re/moved #CHANGED(WS)
20000130 SFP029	Added RFROM and DOES> optimisations
20000129 SFP028	Added ON OFF generators
		Added SEQRES field to optimisation structure,
		made LITPLUS optimiser track source register
20000127 SFP027	Added LSHIFT and RSHIFT code generators
		Enhanced FINDDEST2 C_*
		Changed 1+ 2+ 4+ to macros
		Made RELLVs interrupt safe
20000126 SFP026	Added LIT + and LIT + LIT optimisations
		Changed OPTMATCH? to OPT? to bring in line with other optimisers
		Added COMPILE-NEW and brought up to v6.06
19990614 SFP025 Moved COMPILE-CALL and friends from XASMARM.FTH because
                of dependencies in children of CREATE.
19990524 SFP024 Changed use of ABS12BITLIT? to IMM12? in C_LV
                Replaced all 12 bit literal tests by IMM12?
                Replaced use of R#>A with @LIT for PC offset after IMM12?
19990523 SFP023 Added C_CVARIABLE
19990425 SFP022 Add C_FIELD-CHILD
19990311 SFP021 Add C_THERE
19981120 MSD020 Add CHARS.
19981120 MSD019 Removed common code to the new PREGEN.FTH file. (Not flagged.)
SFP018  30 Oct 1998     Corrected shuffle routines. All completed moves,
                        exchanges, or inplaces, now set the working stack
                        resource to rDUMMY.
SFP017  23 Oct 1998     Corrected compile time literal handling
SFP016  22 Oct 1998     Updated C_*
SFP015  14 Oct 1998     Overhauled CASE handling
                        Added inlining
SFP014  7 Oct 1998      Corrected C_RELLVS for new model
SFP013  13 Sep 1998     Added >DOES(T)
SFP012  18 Aug 1998     Added C_BUFFER:
SFP011  17 Aug 1998     Added code in UNMARK to substitute a dummy resource
                        in the working stack. This prevents a register being
                        falsely copied to the output stack when it is reused as
                        a scratch register.
                        Updated C_VAL@ C_VAL! C_USERCHILD
SFP010  15 Aug 1998     Added PAGE-COMPILE-REF for interpage calls
SFP009  22 July 1998    Prevented GETSHUFFLEREG from finding TOS
SFP008  18 July 1998    Added data stack PUSH/POP optimisations in shuffles.
                        Added +! literal optimisation
SFP007  16 July 1998    Fixed forward reference problems in COMPILE-xxx and
                        DOLITERALxxx
SFP006  7 July 1998     H8/300H conversion
SFP005  6 July 1998     Fixed >NUMBER(T) to be base sensitive
SFP004  2 July 1998     Moved XC: words and analogues from XDTC41-4
                        Fixed conditional branches taking literal values
SFP003  29 June 1998    Converted to cross compiler version
SFP002  4 June 1998     Converted from using strings to access assembler directly
SFP001  3 May 1998      First cut
))

\ ==========
\ *! xoptcortex
\ *T ARM/Cortex VFX code generator
\ ==========
\ *P In the main the VFX code generator is a black box. There are
\ ** however a few control switches which are useful in limited cases.
\ ** Note that because of the efficiency of the VFX code generator, it
\ ** is very rarely necessary to use assembler code. The main
\ ** exceptions are when dealing with very CPU specific code such as
\ ** the cache control routines.

\ *************
\ *S Directives
\ *************

: xoptCortex  ;	\ -- ; marker

only forth  also c-c  definitions


\ ****************
\ Optimiser errors
\ ****************

ErrDef opt_invalid_op	"invalid operation - internal software error"
ErrDef opt_ws_oflow	"working stack overflow"
ErrDef opt_no_lits	"no free literals"
ErrDef opt_no_regs	"No free registers"
ErrDef opt_bad_size	"invalid size operator"
ErrDef opt_bad_scratch	"couldn't free scratch resource"
ErrDef opt_cant_shuffle	"Can't shuffle - internal software error"
ErrDef opt_neg_pick	"negative literal input to PICK"
ErrDef opt_bad_set	"Attempted to execute SETx - internal software error"
ErrDef opt_bad_ws	"working stack unsafe - internal software error"
ErrDef opt_abort	"Optimiser abort"
ErrDef opt_interpage	"Can't handle inter-page forward reference"
ErrDef opt_lit_toofar	"literal pool - literal too far from instruction"
ErrDef opt_bad_move	"ISE: bad move operation"
\ ErrDef opt_bad_movef	"ISE: bad move operation"
ErrDef opt_fwd_todo	"target of TO-DO is not resolved"
ErrDef opt_noLinkSave	"LINK register not saved"
ErrDef opt_LeafError	"LEAF set in error"
ErrDef opt_LeafInline	"Cannot use INLINING and +LEAFCALLS together"
ErrDef opt_LongCalls	"Call out of range - use +LONGCALLS"
ErrDef opt_NoLitPool	"Literal pool not implemented for Cortex-M3+"
ErrDef opt_LitPoolM0	"Enable literal pool for Cortex-M0/M1"
ErrDef opt_SYSm		"SYSm register must be a literal"
ErrDef opt_T2Intrinsic	"This intrinsic is for Thumb2 only"
ErrDef opt_ARMIntrinsic	"This intrinsic is for ARM32 only"


\ ******************************************
\ Compatibility directives for common kernel
\ ******************************************

variable <ARMunpredictable>	\ -- addr ; SFP117
\ If true, the Cortex-A restrictions on ARM32 code are used.

variable <short-branches>
  <short-branches> on

also cc/i
: +short-branches	cc/i	\ --
\ *G You save memory by compiling branches with an 8 bit offset.
\ ** This is the default condition.
  <short-branches> on  ;
: -short-branches	cc/i	\ --
\ *G You may need this for big conditional structures.
  <short-branches> off  ;
: [-short-branches	cc/i	\ -- x
\ *G You may need this for big conditional structures.
\ *C   [-short-branches ... short-branches]
  <short-branches> @  <short-branches> off  ;
: [+short-branches	cc/i	\ -- x
\ *G You can save memory by compiling branches with an 8 bit offset.
\ *C   [+short-branches ... short-branches]
  <short-branches> @  <short-branches> on  ;
: [short-branches	cc/i	\ -- x
\ *G Save the current short/long branch state.
\ *C   [short-branches ... short-branches]
  <short-branches> @  ;
: short-branches]	cc/i	\ x --
\ *G Restore the saved short/long branch state.
  <short-branches> !  ;
previous definitions


\ ******************
\ Target description
\ ******************

1 constant tos-cached?  \ -- n ; #stack items cached in registers at a boundary

: cells(t)      \ n -- n'
  cell(t) *  ;


\ ********************
\ Assembler management
\ ********************

: start-asm     \ --
  [+assembler] asm_start asm-init [previous]
;

: flush-asm     \ --
  [+assembler] asm_end asm_start [previous]       \ flush assembler
;

: a[            \ -- ; start of assembler sequence
  also asm-access  ;  immediate

: ]a            \ -- ; end of assembler sequence
  previous  ;  immediate


\ *******************************************
\ High level branches - marking and resolving
\ These branch targets are 32 bit absolute
\ *******************************************

: mrk>          \ -- addr ; mark start of forward branch
  chere(t)                              \ address to patch up later
  0 l,c(t)  ;                           \ lay dummy value

: >res          \ addr -- ; resolve forward branch
  flush-asm                             \ flush assembler
  chere(t) swap l!c(t)  ;               \ fix up destination

: mrk<          \ -- addr ; mark end of backward branch
  flush-asm                             \ flush assembler
  chere(t)  ;                           \ address to branch to

: <res          \ addr -- ; resolve backward branch
  l,c(t)  ;                             \ lay target address


\ ****************
\ Control switches
\ ****************

variable debugging   debugging off
\ variable generating  generating on
\ variable optimising  optimising on

also cc/i
: +debug-opt	cc/i  debugging on  ;
: -debug-opt	cc/i  debugging off  ;
previous definitions


\ **********************
\ Parsing quoted strings
\ **********************

[undefined] "xxx" [if]
: "xxx"         \ -- addr len ; parse out string of form "<text>"
  [char] " parse 2drop  [char] " parse  ;
[then]

: lay-string    \ -- ; compile a name into the dictionary as a counted string
  "xxx" here  over 1+ allot place
;

: lay-asm:      \ addr -- addr' ; compile a string as an an assembler macro, putting xt at addr
  :noname over !  cell+                 \ generate : portion, put xt at addr, step on
  smudge  !csp                          \ invert SMUDGE that EXIT will do, set error checking
  also asm-access                       \ enable assmbler vocabulary
  "xxx" evaluate                        \ use the string
  previous
  smudge  postpone ;
;


\ ********************
\ Optimisation control
\ ********************

((
The code generator is NOT intended to be a full optimiser. It is intended to
be used with two additional modules, a token optimiser and an instruction
scheduler. The token optimiser is a pass that converts the code into a token
sequence, and then modifies it to use more efficient sequences. For example
the sequence "lit + @" may be translated to "indexed@" which will then produce
shorter code. Similarly, phrases such as "var lit1 + lit2 +" can be folded to
a single literal "var+lit1+lit2" generated at compile time.

Despite this, there are some sequences which are simple to recognise at
compile time, and to optimise simply. The main one of these is the "compare
condbranch" pair. During a normal comparison, a code tail is produced which
only has to be processed by the conditional branch code. The code generator
detects this pair, and removes the comparison epilogue and the branch
prologue.

These simple optimisations are controlled by a data structure
SEQBEG          holds HERE before the generated code sequence, so
                that the optimiser can be extended to multiple look backs
SEQEND          holds HERE after code is generated
SEQBKUP         holds the start of the code to backup (can be removed with ALLOT)
SEQID           holds a sequence type indicator
SEQFLAGS        holds information used later
SEQPREV         holds information from a previous sequence
SEQPTR          holds a pointer to the comparison data structure
SEQRES		holds a resource needed for substitution
))

struct /sequence
  int .seqbeg
  int .seqend
  int .seqbkup
  int .seqid
  int .seqflags
  int .seqprev                  \ SFP026
  int .seqptr
  int .seqres			\ SFP028
end-struct

/sequence buffer: seqbeg        \ -- addr ; returns address of SEQBEG

: seqend        \ -- addr
  seqbeg .seqend  ;

: seqbkup       \ -- addr
  seqbeg .seqbkup  ;

: seqid         \ -- addr
  seqbeg .seqid  ;

: seqflags      \ -- addr
  seqbeg .seqflags  ;

: seqprev       \ -- addr ; SFP026
  seqbeg  .seqprev  ;

: seqptr        \ -- addr
  seqbeg .seqptr  ;

: seqres	\ -- addr ; SFP028
  seqbeg .seqres  ;

0 constant noOptID                      \ no optimisation
1 constant 1CMPID                       \ Comparison of 1 item, e.g. 0=
2 constant 2CMPID                       \ comparison of 2 items, e.g. <
3 constant QDUPID                       \ ?DUP
4 constant QDUPZEID                     \ ?DUP 0= etc
5 constant LITPLUSID                    \ LIT <x> + followed by @ or !
6 constant LITPLUSLITID                 \ LIT <x> + then LIT <y> +
7 constant DOESID                       \ DOES> >R
8 constant RFROMID                      \ R> DROP, R> IF etc
9 constant FLAG1SETID			\ AND IF and friends ; SFP062

: start-seq     \ seqID -- ; mark the start of a sequence
  seqid !  chere(t) seqbeg !  ;

: start-bkup    \ -- ; mark the start of the code which will be removed
  chere(t) seqbkup !  ;

: end-seq       \ -- ; mark the end of the sequence and code to be removed
  chere(t) seqend !  ;

: [opt		\ seqid -- ; SFP062
  start-seq start-bkup
;

: opt]		\ -- ; SFP062
  end-seq
;

: seq-backup    \ -- ; remove the optimisable sequence - safely
  seqbkup @ seqend @ - callot(t)  ;

: reset-opt     \ -- ; resets optimiser
  -1 seqend !  nooptID seqid !  ;       \ will never be matched

: optimisable?  \ -- flag ; true if optimisation ends here
  seqend @ chere(t) =  ;

: optid?        \ seqid -- flag ; true if sequence ID matches
  seqid @ =  ;

: opt?     	\ seqid -- flag ; true if previous sequence ends here
  optid?  optimisable? and  ;

: .sequence	\ --
  cr seqid @
  case
    0 of  ." NoOptID"  endof
    1 of ." 1CMPID"  endof
    2 of ." 2CMPID"  endof
    3 of ." QDUPID"  endof
    4 of ." QDUPZEID"  endof
    5 of ." LITPLUSID"  endof
    6 of ." LITPLUSLITID"  endof
    7 of ." DOESID"  endof
    8 of ." RFROMID"  endof
    9 of ." FLAG1SETID"  endof
  endcase
;

reset-opt


\ *********************
\ Structure definitions
\ *********************

\ ********************************************************
\ A resource is somewhere to keep data about a stack item,
\ it may be a register, or it may be on the stack itself.
\ ********************************************************

struct /resource        \ -- ; -- len ;
        int res-flags           \ zero if unused
        int res-refcount        \ number of stack items represented
        int res-type            \ data type
        int res-value           \ value of literal/address/associated data
        int res-reslink         \ link to previous resource
    0 field res-name            \ was resource name, now end of modifiable data
        int lxt                 \ xt of word to execute when LONG
        int wxt                 \ xt of word to execute when WORD
        int bxt                 \ xt of word to execute when BYTE
        int ixt                 \ xt of word to execute when ADDRESS with 0 offset  @(ERx)
        int oxt                 \ xt of word to execute when ADDRESS with n offset  n @(ERx)
end-struct

0 res-name constant /res-var    \ size of resource variable area
/resource constant /res-data    \ size of resource data area


\ The following bit masks are used by flags field in a resource structure
%00000001 constant bit0
%00000010 constant bit1
%00000100 constant bit2
%00001000 constant bit3
%00010000 constant bit4
%00100000 constant bit5
%01000000 constant bit6
%10000000 constant bit7

bit0 constant rmask     \ -- n ; register bit mask, if clear memory assumed
bit1 constant br        \ -- n ;  has 8 bit form
bit2 constant wr        \ -- n ;  has 16 bit form
bit3 constant lr        \ -- n ;  has 32 bit form
bit4 constant xr        \ -- n ;  has 64 bit form
bit5 constant litmask   \ -- n ; is a literal, value field contains literal
bit6 constant deepmask  \ -- n ; is in deep stack, value field contains index into stack model
bit7 constant srcmask   \ -- n ; is the source of a stack shuffle

rmask  lr or  wr or  br or
  constant rlwb         \ -- n ; register with 32, 16, 8 bit forms
rmask  lr or  wr or
  constant rlw          \ -- n ; register with 32, 16 bit forms

rmask  xr or  lr or  wr or  br or
  constant fmask        \ -- n ; initial flag mask

: init-resource \ addr -- ; initialise a resource
  dup  res-flags dup @ fmask and swap !
  dup  res-refcount off
  dup  res-type off
  dup  res-value off
  drop
;

defer bad-report

variable res-link       \ -- addr ; holds address of last resource declared

: say-bad       \ -- ; error if executed
cr ." SAY-BAD probably in " r> ip>nfa .name
bad-report
\  1 abort" Invalid xXT operation - internal software error"  ;
  opt_invalid_op throw  ;

defer alit>a    \ -- ; how to use an address literal is defined later
  assign say-bad to-do alit>a           \ in case is never set

: lay-alit      \ addr -- addr' ; set pointer to address literal handling
  ['] alit>a over !  cell+  ;

: bad           \ addr -- addr' ; set pointer to be invalid
  ['] say-bad over !  cell+  ;

: res-allot     \ flags -- ; lay data section of resource and initialise the flags field
  here                                  \ address of new resource
  /res-data allot&erase                 \ reserve data space
  res-link @ over res-reslink !         \ old value to link field
  dup res-link !                        \ set new value
  res-flags !                           \ and initialise flags field
;

: resource:     \ index -- ; -- addr
  create
    here                                        \ address of data area
    deepmask res-allot                          \ reserve and link data space
    tuck res-value !                            \ stash index
    lxt lay-asm: lay-asm: lay-asm: bad bad drop \ long, word, byte strings
;

: reg-lwb:      \ -- ; -- addr ; register with Long, Word, Byte, 0 address, n address forms
  create
    here
    rlwb res-allot                              \ reserve data space
    lxt lay-asm: lay-asm: lay-asm: lay-asm: lay-asm: drop \ long, word, byte, 0 indirect, n indirect strings
;

((
: reg-lw:       \ -- ; -- addr ; register with Long, Word forms
  create
    here
    rlw res-allot                               \ reserve and link data space
    lxt lay-asm: lay-asm: bad lay-asm: lay-asm: drop     \ long, word strings
;
))

: res-lit:      \ -- ; -- addr ; literal
  create
    here
    litmask res-allot                           \ reserve and link data space
    lxt bad bad bad lay-alit bad drop
;


\ ********************
\ Resource definitions
\ ********************

\ Registers - N.B. the leading spaces MUST be provided
reg-lwb: rR0  " r0" " r0" " r0" " [ r0 ]" " [ r0, # ]"
reg-lwb: rR1  " r1" " r1" " r1" " [ r1 ]" " [ r1, # ]"
reg-lwb: rR2  " r2" " r2" " r2" " [ r2 ]" " [ r2, # ]"
reg-lwb: rR3  " r3" " r3" " r3" " [ r3 ]" " [ r3, # ]"
reg-lwb: rR4  " r4" " r4" " r4" " [ r4 ]" " [ r4, # ]"
reg-lwb: rR5  " r5" " r5" " r5" " [ r5 ]" " [ r5, # ]"
reg-lwb: rR6  " r6" " r6" " r6" " [ r6 ]" " [ r6, # ]"
reg-lwb: rR7  " r7" " r7" " r7" " [ r7 ]" " [ r7, # ]"
reg-lwb: rR8  " r8" " r8" " r8" " [ r8 ]" " [ r8, # ]"
reg-lwb: rR9  " r9" " r9" " r9" " [ r9 ]" " [ r9, # ]"
reg-lwb: rR10 " r10" " r10" " r10" " [ r10 ]" " [ r10, # ]"
reg-lwb: rR11 " r11" " r11" " r11" " [ r11 ]" " [ r11, # ]"
reg-lwb: rR12 " r12" " r12" " r12" " [ r12 ]" " [ r12, # ]"
reg-lwb: rR13 " r13" " r13" " r13" " [ r13 ]" " [ r13, # ]"
reg-lwb: rR14 " r14" " r14" " r14" " [ r14 ]" " [ r14, # ]"
reg-lwb: rR15 " r15" " r15" " r15" " [ r15 ]" " [ r15, # ]"

\ ARM compiler:
\   On the ARM the following register usage is assumed
\   r15         pc      program counter
\   r14         link    link register
\   r13         rsp     return stack pointer
\   r12         psp     data stack pointer
\   r11         up      user area pointer ; see SFP124
\   r10         tos     cached top of stack
\   r9          lp      locals pointer
\   r8          fsp	float stack pointer ; SFP120
\   r2-r7       scratch
\   r1          move    used by generic sdmove, routine
\   r0          system  used by shuffle routine, may be used in code definitions

\ Cortex-M3+ compiler:
\   On the Cortex/ARM the following register usage is assumed
\   r15         pc      program counter
\   r14         link    link register
\   r13         rsp     return stack pointer
\   r12         psp     data stack pointer
\   r11         up      user area pointer ; see SFP124
\   r10         --	RFU
\   r9          lp      locals pointer
\   r8          fsp	float stack pointer ; SFP120
\   r7          tos     cached top of stack
\   r0-r6       scratch

\ Cortex-M0/M1 compiler:
\   On the Cortex-M0/M1 the following register usage is assumed
\   r15         pc      program counter
\   r14         link    link register
\   r13         rsp     return stack pointer
\   r12         --
\   r11         up      user area pointer ; see SFP124
\   r10         --	RFU
\   r9          lp      locals pointer
\   r8          fsp	float stack pointer ; SFP120
\   r7          tos     cached top of stack
\   r6          psp     data stack pointer
\   r0-r5       scratch

0 value RegMode	\ -- 0..x
\ Carries offsets, 0, 4, 8 ... for ARM, T2 and M0 modes that need different
\ register sets.

: mres		\ tab -- res
\ Given a table, return the resource for the current mode.
  RegMode + @  ;

\                       ARM     CM3+    CM01
create PSPtab   	rR12 ,  rR12 ,  rR6 ,
create TOStab		rR10 ,  rR7 ,   rR7 ,
create newTOStab	rR7 ,   rR6 ,   rR5 ,	\ SFP120
create newNOStab	rR6 ,   rR5 ,   rR4 ,	\ SFP120
\ create newTOStab	rR8 ,   rR6 ,   rR5 ,
\ create newNOStab	rR7 ,   rR5 ,   rR4 ,

: rPC	( -- addr )  rR15  ;	\ fixed
: rLINK	( -- addr )  rR14  ;	\ fixed
: rRSP	( -- addr )  rR13  ;	\ fixed
: rPSP	( -- addr )  PSPtab mres  ;	\ for Cortex-M3+, rPSP should be a high register
: rUP	( -- addr )  rR11  ;
: rLP	( -- addr )  rR9  ;
: rTOS	( -- addr )  TOStab mres  ;

: defnewTOS     \ -- addr ; synonym for when new TOS is required
  newTOStab mres  ;

: defnewNOS     \ -- addr ; synonym for when new NOS is required
  newNOStab mres  ;

\ Deep stack  N.B. the index into the stack model MUST be provided
tos-cached?
     dup resource: DS1   " [ psp ]"        " [ psp ]"        " [ psp ]"
  1+ dup resource: DS2   " [ psp, # $04 ]" " [ psp, # $04 ]" " [ psp, # $04 ]"
  1+ dup resource: DS3   " [ psp, # $08 ]" " [ psp, # $08 ]" " [ psp, # $08 ]"
  1+ dup resource: DS4   " [ psp, # $0C ]" " [ psp, # $0C ]" " [ psp, # $0C ]"
  1+ dup resource: DS5   " [ psp, # $10 ]" " [ psp, # $10 ]" " [ psp, # $10 ]"
  1+ dup resource: DS6   " [ psp, # $14 ]" " [ psp, # $14 ]" " [ psp, # $14 ]"
  1+ dup resource: DS7   " [ psp, # $18 ]" " [ psp, # $18 ]" " [ psp, # $18 ]"
  1+ dup resource: DS8   " [ psp, # $1C ]" " [ psp, # $1C ]" " [ psp, # $1C ]"
  1+ dup resource: DS9   " [ psp, # $20 ]" " [ psp, # $20 ]" " [ psp, # $20 ]"
  1+ dup resource: DS10  " [ psp, # $24 ]" " [ psp, # $24 ]" " [ psp, # $24 ]"
  1+ dup resource: DS11  " [ psp, # $28 ]" " [ psp, # $28 ]" " [ psp, # $28 ]"
  1+ dup resource: DS12  " [ psp, # $2C ]" " [ psp, # $2C ]" " [ psp, # $2C ]"
  1+ dup resource: DS13  " [ psp, # $30 ]" " [ psp, # $30 ]" " [ psp, # $30 ]"
  1+ dup resource: DS14  " [ psp, # $34 ]" " [ psp, # $34 ]" " [ psp, # $34 ]"
  1+ dup resource: DS15  " [ psp, # $38 ]" " [ psp, # $38 ]" " [ psp, # $38 ]"
  1+ dup resource: DS16  " [ psp, # $3C ]" " [ psp, # $3C ]" " [ psp, # $3C ]"
  1+ dup resource: DS17  " [ psp, # $40 ]" " [ psp, # $40 ]" " [ psp, # $40 ]"
  1+ dup resource: DS18  " [ psp, # $44 ]" " [ psp, # $44 ]" " [ psp, # $44 ]"
  1+ dup resource: DS19  " [ psp, # $48 ]" " [ psp, # $48 ]" " [ psp, # $48 ]"
  1+ dup resource: DS20  " [ psp, # $4C ]" " [ psp, # $4C ]" " [ psp, # $4C ]"
  1+ dup resource: DS21  " [ psp, # $50 ]" " [ psp, # $50 ]" " [ psp, # $50 ]"
  1+ dup resource: DS22  " [ psp, # $54 ]" " [ psp, # $54 ]" " [ psp, # $54 ]"
  1+ dup resource: DS23  " [ psp, # $58 ]" " [ psp, # $58 ]" " [ psp, # $58 ]"
drop

#-99 resource: DR1>  " [ rsp ], # 4 ] "  " [ rsp ], # 4 ] "  " [ rsp ], # 4 ] "
#-98 resource: >DR1  " [ rsp, # -4 ] !"  " [ rsp, # -4 ] !"  " [ rsp, # -4 ] !"
#-99 resource: DS1>  " [ psp ], # 4 ] "  " [ psp ], # 4 ] "  " [ psp ], # 4 ] "
#-98 resource: >DS1  " [ psp, # -4 ] !"  " [ psp, # -4 ] !"  " [ psp, # -4 ] !"


\ Literals
res-lit:  LITP1
res-lit:  LITP2
res-lit:  LITP3
res-lit:  LITP4
res-lit:  LITP5
res-lit:  LITP6
res-lit:  LITP7
res-lit:  LITP8
res-lit:  LITP9
res-lit:  LITP10
res-lit:  LITP11
res-lit:  LITP12
res-lit:  LITP13
res-lit:  LITP14
res-lit:  LITP15
res-lit:  LITP16

res-lit:  rDUMMY        \ never select this in a table, this resource is
                        \ reserved for use as an unused resource to prevent
                        \ the stack shuffle from mis-using a source register
                        \ which is moved and then re-used as a scratch reg.

\ return stack
-1 resource: DR1   " [ rsp ]"        " [ rsp ]"        " [ rsp ]"
-2 resource: DR2   " [ rsp, # $04 ]" " [ rsp, # $04 ]" " [ rsp, # $04 ]"
-3 resource: DR3   " [ rsp, # $08 ]" " [ rsp, # $08 ]" " [ rsp, # $08 ]"
-4 resource: DR4   " [ rsp, # $0C ]" " [ rsp, # $0C ]" " [ rsp, # $0C ]"
-5 resource: DR5   " [ rsp, # $10 ]" " [ rsp, # $10 ]" " [ rsp, # $10 ]"
-6 resource: DR6   " [ rsp, # $14 ]" " [ rsp, # $14 ]" " [ rsp, # $14 ]"


\ resource tables - lists of resources under various conditions
create resTabM0	  \ -- addr
\ holds available register resources in preference order
  rR7 ,         rR5 ,    		\ TOS *MUST* be first
  rR4 ,         rR3 ,            rR2 ,
  0 ,					\ ends with a 0
\ r12, r10, r8 can be used for scratch. N.B R8=FSP

create resTabT2   \ -- addr
\ holds available register resources in preference order
  rR7 ,         rR6 ,            rR5 ,    	\ TOS *MUST* be first
  rR4 ,         rR3 ,            rR2 ,
  0 ,					\ ends with a 0
\ r10, r8 can be used for scratch. N.B R8=FSP

create resTabARM  \ -- addr
  rR10 ,      ( rR8 , )          rR7 ,  \ TOS *MUST* be first
  rR6 ,         rR5 ,            rR4 ,
  rR3 ,         rR2 ,
  0 ,					\ ends with a 0

create resPtrs   resTabARM ,  resTabT2 ,  resTabM0 ,

: resource-table	\ -- addr
  resPtrs mres  ;


#16 constant /stack-picture     \ -- n ; number of cells on the working stack
                                \ and the initial model

create smT2	\ -- addr ; holds Thumb2 stack model for input and output
  rR7 ,         DS1 ,           DS2 ,           DS3 ,
  DS4 ,         DS5 ,           DS6 ,           DS7 ,
  DS8 ,         DS9 ,           DS10 ,          DS11 ,
  DS12 ,        DS13 ,          DS14 ,          DS15 ,
  DS16 ,        DS17 ,          DS18 ,          DS19 ,
  DS20 ,        DS21 ,          DS22 ,          DS23 ,
  0 ,
create smARM	\ -- addr ; holds ARM stack model for input and output
  rR10 ,        DS1 ,           DS2 ,           DS3 ,
  DS4 ,         DS5 ,           DS6 ,           DS7 ,
  DS8 ,         DS9 ,           DS10 ,          DS11 ,
  DS12 ,        DS13 ,          DS14 ,          DS15 ,
  DS16 ,        DS17 ,          DS18 ,          DS19 ,
  DS20 ,        DS21 ,          DS22 ,          DS23 ,
  0 ,

create smTab
  smARM ,  smT2 ,  smT2 ,

: stack-model	\ -- addr
  smTab mres  ;

/stack-picture 1+ cells
  buffer: stack-picture \ -- addr ; the working stack, initialised from STACK-MODEL

/stack-picture 1+ cells
 buffer: dest-picture   \ -- addr ; output stack picture after stack shuffling

create litpool  \ -- addr ; holds table of literal resources
  litp1 ,       litp2 ,         litp3 ,         litp4 ,
  litp5 ,       litp6 ,         litp7 ,         litp8 ,
  litp9 ,       litp10 ,        litp11 ,        litp12 ,
  litp13 ,      litp14 ,        litp15 ,        litp16 ,
  0 ,


\ ******************************
\ Working stack access operators
\ ******************************

: in(ws)        \ n -- addr ; get address of nth slot in working stack
  cells stack-picture +  ;

: nget(ws)      \ n -- ; -- res ; get nth resource from working stack
  create  ,  does> @ in(ws) @  ;

: nput(ws)      \ n -- ; res -- ; set nth resource in working stack
  create  ,  does> @ in(ws) !  ;

0 nget(ws) tos-res      0 nput(ws) set-tos
1 nget(ws) nos-res      1 nput(ws) set-nos
2 nget(ws) 3os-res      2 nput(ws) set-3os
3 nget(ws) 4os-res      3 nput(ws) set-4os

variable #changed(ws)   \ -- addr ; stack depth change since start of word
variable #on-stack(ws)  \ -- addr ; number of items on stack
variable #max-stack(ws) \ -- addr ; max input number of items

: init-stack(ws)        \ -- ; initialise the stack picture
  stack-model stack-picture /stack-picture cells cmove
  0 #changed(ws) !
[ tos-cached? ] [if]
\ The next section is only required if TOS is cached
  tos-res init-resource
  1 tos-res res-refcount !
  1 #on-stack(ws) !  1 #max-stack(ws) !
[else]
\ this section is required if TOS is not cached
  0 #on-stack(ws) !  0 #max-stack(ws) !
[then]
;

init-stack(ws)

: set-max-stack(ws)     \ -- ; set maximum input stack depth
  #on-stack(ws) @ #max-stack(ws) @ max #max-stack(ws) !
;

: push(ws)      \ res -- ; add a resource to the top of the working stack
  #on-stack(ws) @ /stack-picture >=
  opt_ws_oflow ?throw
  stack-picture dup cell+ /stack-picture 1- cells cmove>
  dup stack-picture !                   \ add as TOS
  res-refcount incr                     \ update reference count
  #on-stack(ws) incr
  #changed(ws) incr			\ SFP030
;

: drop(ws)      \ -- ; drop the top item of the working stack
  stack-picture @ res-refcount decr     \ decrement reference count
  stack-picture cell+ stack-picture /stack-picture cells cmove
  #on-stack(ws) decr
  #changed(ws) decr			\ SFP030
;

: pop(ws)       \ -- res ; remove a resource from the top of the working stack
  stack-picture @  drop(ws)
;

: nip(ws)       \ -- ; remove NOS
  pop(ws)  drop(ws)  push(ws)  ;

: min-depth(ws) \ n -- ; set minimum depth of stack
  #on-stack(ws) @ max                   \ set minimum input required
  dup #on-stack(ws) !
\ all resources USED on stack have minimum reference count of 1
  0
  ?do
    i in(ws) @ res-refcount
    dup @ 1 max swap !
  loop
  set-max-stack(ws)
;

: in-range(ws)  \ n -- res | 0 ; 0 if the nth item is not on the working stack
  dup /stack-picture u<
  if  in(ws) @ 0<>  else  drop 0  endif
;

\ ***************************************
\ Output and Model stack access operators
\ ***************************************

: in(os)        \ n -- addr ; get address of nth slot in output stack
  cells dest-picture +  ;

: in(ms)        \ n -- addr ; get address of nth slot in model stack
  cells stack-model +  ;


\ *************************
\ Resource access operators
\ *************************

: reg?          \ res -- flag ; true if register
  res-flags @ rmask and 0<>  ;

: byte?         \ res -- flag ; true if resource has byte form
  res-flags @ br and 0<>  ;

: word?         \ res -- flag ; true if resource has byte form
  res-flags @ wr and 0<>  ;

: lit?          \ res -- flag ; true if resource is a literal
  res-flags @ litmask and 0<>  ;

((
: reg/lit?      \ res -- flag ; true if resource is register or literal
  res-flags @ [ rmask litmask or ] literal and 0<>  ;
))
: deep?         \ res -- flag ; true if resource is in deep stack
  res-flags @ deepmask and 0<>  ;

: source?       \ res -- flag ; true if resource is a source for a shuffle
  res-flags @ srcmask and 0<>  ;

: count=1?      \ res -- flag ; true if ref count=1
  res-refcount @ 1 =  ;

: free?         \ res -- flag ; true if count=0
  res-refcount @ 0=  ;

: dreg?         \ res -- flag ; true if reg and count=1, can be used as dest
  dup reg? swap count=1? and  ;

: unmarkable?   \ res -- flag ; true if this resource is source with count=1
  dup source? swap count=1? and  ;

: mark          \ res -- ; set source bit
  srcmask swap res-flags or!  ;

: set-dummy(ws) \ n -- ; set nth item of the working stack to rDUMMY    SFP018
  rDUMMY swap in(ws) !
;

: unmark        \ res -- ; reset source bit, remove resource from working stack
  srcmask over res-flags bic!		\ clear source flag in resource
  #on-stack(ws) @ 0 ?do			\ walk the stack
    i in(ws) @ over =                   \ if that resource is being cleared
    if  i set-dummy(ws)  endif          \   put rDUMMY in its place     SFP018
  loop
  drop                                  \ discard res
;

: getres        \ table -- res | 0 ; res1, res2, ... resn, 0
  begin
    dup @ dup                           \ get resource
   while                                \ 0 marks end of chain
    free?                               \ resource free?
    if  @  exit  endif                  \ yes, return it
    cell+                               \ no, step to next
  repeat
  nip  ;

: GetLitRes     \ -- res ; get a free literal resource
  LitPool GetRes dup 0= opt_no_lits ?throw  ;

: GetRegRes     \ -- res ; get a free register resource
  Resource-table GetRes dup 0= opt_no_regs ?throw  ;

: GetShuffleReg \ -- res | 0 ; used in the shuffle routine
  Resource-Table cell+ GetRes  ;        \ Skips TOS, which *MUST* be first

: GetNewReg     \ -- res | 0 ; get a new register
  Resource-Table GetRes  ;

: -GetNewReg	\ reg -- res|0 ; SFP073
\ Get a new register that is not the specified one.
  dup res-refcount incr
  GetNewReg
  swap res-refcount decr
;

: --GetNewReg	\ reg1 reg2 -- res|0 ; SFP073
\ Get a new register that is not either of the specified ones.
  over res-refcount incr  dup res-refcount incr
  GetNewReg
  swap res-refcount decr  swap res-refcount decr
;


\ **************************
\ Stack display and checking
\ **************************

: .res-name     \ res -- ; display resource name
  body> >name .name  ;

: .resource     \ res -- ; list resouce
  cr
  dup .res-name
  #8 >pos  ." RC = " dup res-refcount @ .
  dup lit?
  if  #16 >pos  ." LIT " dup res-value @ .  endif
  dup source?
  if  #32 >pos  ." source"  endif
  drop
;

: .resources    \ -- ; list current resources
  cr ." Resource list"
  res-link
  begin
    @ ?dup
   while
    dup .resource
    res-reslink
  repeat
;

: check(ws)     \ -- ; Check working stack
  #on-stack(ws) @ 0< if
    cr ." Compiler working stack underflow"
    #on-stack(ws) @ .  opt_abort throw
  endif
  #on-stack(ws) @ /stack-picture > if
    cr ." Compiler working stack too deep"
    #on-stack(ws) @ .  opt_abort throw
  endif
;

: .ws           \ -- ; display working stack
  check(ws)
  #on-stack(ws) @
  cr ." Depth = " dup .
     ."  max depth was " #max-stack(ws) @ .
     ."  change so far " #changed(ws) @ .
  0 max  0
  ?do  i in(ws) @ .resource  loop
;
assign .ws to-do bad-report

: .os           \ -- ; display output stack
  #on-stack(ws) @
  cr ." Depth = " dup .
  0 max  0
  ?do  i in(os) @ .resource  loop
;

: check-stack(ws)       \ -- ; check current resources after shuffle
  res-link
  begin
    @ ?dup
   while
    dup rTOS <> if
      dup res-refcount @ if
        .resources  opt_bad_ws throw
      endif
    endif
    res-reslink
  repeat
;


\ *************************************
\ Putting items in the assembler output
\ *************************************

: l>a           \ res -- ; add LONG string to ASM
  lxt @ execute  ;

: w>a           \ res -- ; add WORD string to ASM
  wxt @ execute  ;

: b>a           \ res -- ; add BYTE string to ASM
  bxt @ execute  ;

: i>a           \ res -- ; use as address with 0 offset (indirect address)
  ixt @ execute  ;

: o>a           \ n res -- ; use as address with n offset (offset address)
  oxt @ execute  ;

: sz>a          \ size res -- ; add SIZEd string to ASM$
  lxt swap cells + @ execute  ;

: n>a           \ n -- n ; add number to assembler output
;               \ consumed by assembler opcode

0 constant LSIZE
1 constant WSIZE
2 constant BSIZE

: size-error    \ -- ; size error detected
  opt_bad_size throw  ;

: use-a         \ -- ; display assembler string
  flush-asm  ;


\ **************************************
\ CPU assembler specific string handling
\ **************************************

: r#>a          \ res -- n ; add literal resource to ASM$, either immediate or address
  res-value @  not-inlined  ;		\ SFP056

: #>a           \ -- ; add " # " to output string
  a[ # ]a  ;

: #l>a          \ res -- ; puts out value of literal
  a[ # ]a  res-value @  ;

: gl>a          \ res -- ; copes with #, reg, DS for PUSH
  dup lit?
  if  a[ # ]a res-value @  else  l>a  endif  ;

: gsz>a         \ size res -- [n] ; copes with #, reg, DS
  dup lit?
  if  #l>a  nip  else  sz>a  endif  ;

: raddr>a      \ n res -- n ; add offset address to output string e.g. 10 [ESI]
  o>a ;

: +0[reg]       \ res -- ; add offset address to output string e.g. 0 [ESI]
  i>a  ;

: +cell[reg]    \ res -- ; add offset address to output string e.g. 4 [ESI]
  cell(t) swap o>a  ;

: +n[reg]	\ n res -- ; add offset address to output string e.g. n [ESI]
  o>a  ;

: 12bitoff?	\ n -- flag ; true if offset fits in 12bit ARM format
  abs $FFF u> 0=  ;

: 8bitoff?	\ n -- flag ; true if offset fits in 8bit ARM format
  abs $FF u> 0=  ;

: T2memOff?	\ n -- flag ; true if offset fits Cortex+12/-8 bit format
  #-255 #4096 within  ;

: Imm8?		\ n -- flag ; true if 0..255
  #256 u<  ;

also assembler
: #genLit?	\ n -- flag
\ True if value fits in 12bit ARM or 12bit Cortex const format.
  Thumb? if
    Cortex-M0/M1?			\ SFP107
    if  Imm8?  else  T2Const12?  endif
  else
    Imm12?
  endif
;

: #memOffLong?	\ n -- flag
\ True if value fits in 12bit ARM or 12/8bit Cortex memory offset format.
  Thumb? if
    Cortex-M0/M1?			\ SFP107
    if  Imm8?  else  T2memOff?  endif
  else
    12bitoff?
  endif
;

: #memOffShort?	\ n -- flag
\ True if value fits in 8bit ARM or 12/8bit Cortex memory offset format.
  Thumb? if
    Cortex-M0/M1?			\ SFP107
    if  Imm8? else  T2memOff?  endif
  else
    8bitoff?
  endif
;
previous

: resGenLit?     \ res -- flag
\ True if immediate value fits in 12bit ARM or 12bit Cortex const format.
  dup lit? if
    res-value @ #genLit?
  else
    drop  false
  endif
;

: absResGenLit?  \ res -- flag
\ True if abs of immediate value fits in 12bit ARM/Cortex format
  dup lit? if
    res-value @ abs #genLit?
  else
    drop  false
  endif
;

: print-sd      \ src des -- ; prints type of src and dest resources
  dup reg? if ." dest=reg= " endif
  dup lit? if ." dest=lit= " endif
  dup deep? if ." dest=deep= " endif
  .res-name
  dup reg? if ."  src=reg= " endif
  dup lit? if ."  src=lit= " endif
  dup deep? if ."  src=deep= " endif
  .res-name
;


\ ****************
\ Literal handling
\ ****************

0 value GuardedFlags?	\ -- flag
\ +G Set when flags must be preserved, e.g. in a shuffle.

\ SFP032...
: ArmLitLoadIns,	\ dest-res -- ^ins ; lay literal load instruction
\ A DEFERed word that loads a literal from the pool into a destination.
\ resource. Used as a factor of C_LITPOOL. It is deferred because it is
\ CPU dependent. Assignment of the action is usually performed by the )
\ code generator during the cross compiler build.
  Thumb? if
    >r  chere(t)			\ return instruction address
    a[ LDR ]a  r> l>a  a[ [ PC, # $FF 4* ] ]a  use-a	\ FF can be updated in flash
    exit
  endif
  >r  chere(t)				\ return instruction address
  a[ LDR ]a  r> l>a  a[ [ PC, # $FFF ] ]a  use-a	\ FFF can be updated in flash
;
assign ArmLitLoadIns, to-do litloadins,

: ArmResLoadIns		\ ^const ^ins -- ; resolve load offset
\ A DEFERed word that resolves a load instruction laid by LITLOADINS,
\ with an unkown offset. Used as a factor of FLUSHLITPOOL. It is deferred
\ because it is CPU dependent. Assignment of the action is usually
\ performed by the code generator during the cross compiler build.
  Thumb? if
\    Cortex-M0/M1? 0= opt_NoLitPool ?throw
    tuck 4 + -  2+ -4 and  2 rshift	\ offset, align(pc,4), scaled
    dup #255 u> opt_lit_toofar ?throw	\ -- ^ins offset ; range check
    over w@c(t)				\ -- ^ins offset' opc
    dup $F800 and $4800 <> abort" Bad M0/M1 LDR opcode"
    $FF00 and				\ remove current offset
    or  swap w!c(t)			\ merge offset to instruction
    exit
  endif
  tuck 8 + - dup 12bitoff?		\ allow for PC+8
\  0= abort" Literal pool too distant
  0= opt_lit_toofar ?throw
  dup 0< if
    abs $FFF and			\ clip to 12 bits
    over l@c(t) $FF7FF000 and		\ clear U bit and offset
    or					\ merge opcode, offset
  else
    $FFF and				\ clip to 12 bits
    over l@c(t) $FF7FF000 and		\ clear U bit and offset
    or  $00800000 or			\ merge opcode, offset, set U bit
  endif
  swap l!c(t)				\ put instruction back
;
assign ArmResLoadIns to-do resloadins

variable <use-litpool>	\ controls use of the literal pool
  <use-litpool> on

also cc/i
: +use-litpool	\ -- ; enable use of literal pool
\ *G Enables use of the literal pool, which results in shorter code
\ ** at the expense of more data cache activity. This is usually a
\ ** better option than generating four-instruction sequences.
\ ** *\fo{+USE-LITPOOL} is the default for the ARM instruction set
\ ** because of the significant reduction  in code size.
\ ** *\fo{+USE-LITPOOL} is not permitted for Cortex-M3 onwards,
\ ** but is required for Cortex-M0/M1.
  Thumb? if				\ SFP107
    Not-M0/M1? opt_NoLitPool ?throw
  endif
  <use-litpool> on
;

: -use-litpool	\ -- ; disable use of literal pool
\ *G Disables use of the literal pool. Immediate values that cannot
\ ** be represented by an ARM 12 bit literal may take up to four
\ ** instructions. *\fo{+USE-LITPOOL} is the default for the ARM,
\ ** Cortex-M0 and Cortex-M1 instruction sets.
  <use-litpool> off  ;
previous definitions

: pLoadLitCM01	{ reg n -- }
\ CM0/1 ; Load n into a register.
  n #256 u< if
    a[ MOV .s ]a  reg l>a  a[ # ]a n n>a  use-a  exit
  endif
  <use-litpool> @ if
    n reg c_litpool  exit
  endif
  opt_LitPoolM0 throw
;

: pLoadLitCM3+	{ reg n -- }
\ CM3+ ; Load n into a register.
  GuardedFlags? 0= if			\ SFP125
    n #256 u< if
      a[ MOV .s ]a  reg l>a  a[ # ]a n n>a  use-a  exit
    endif
  endif
  a[ MVL ]a  reg l>a  a[ # ]a n n>a  use-a
;

: pLoadLitARM	{ reg n -- }
\ ARM32 ; Load n into a register.
  n [+assembler] imm12? [previous] if
    a[ MOV ]a  reg l>a  a[ # ]a n n>a  use-a  exit
  endif
  n 0<  n 1+ abs [+assembler] imm12? [previous] and if
    a[ MVN ]a  reg l>a  a[ # ]a n 1+ abs n>a  use-a  exit
  endif
\ add test for two instruction sequence here.
  <use-litpool> @ if
    n reg c_litpool  exit
  endif
  a[ MVL ]a  reg l>a  n n>a  use-a
;

: PrimLoadLit	\ res n -- ; load n into a register
\ Cortex-M3 onwards does not use the literal pool, because the
\ compiler generates a MOV/MOVT pair instead with the MVL
\ pseudo-instruction. ARM32 also uses MVL. For Cortex-M0/M1
\ it's nasty - just use the literal pool.
  Thumb? if
    Cortex-M0/M1?			\ SFP107
    if  pLoadLitCM01  else  pLoadLitCM3+  endif
    exit
  endif
  pLoadLitARM
;

: ResLoadLit	\ reg reslit -- ; load n into a register
  res-value @ PrimLoadLit
;
\ ...SFP032


\ ****************
\ Move and compare
\ ****************

: sdMoveARM/M3,	{ sres dres -- }	\ SFP032
\ use r1 as a scratch register
  case
    sres reg? dres reg? and     ?of a[ MOV ]a dres l>a sres l>a use-a endof
    sres lit? dres reg? and     ?of dres sres ResLoadLit endof
    sres deep? dres reg? and    ?of a[ LDR ]a dres l>a sres gl>a use-a endof
    sres reg? dres deep? and    ?of a[ STR ]a sres l>a dres l>a use-a endof
    sres lit? dres deep? and    ?of rR1 sres ResLoadLit
                                    a[ STR r1, ]a dres l>a use-a
                                endof
    sres deep? dres deep? and   ?of a[ LDR r1, ]a sres gl>a use-a
                                    a[ STR r1, ]a dres gl>a use-a
                                endof
    opt_bad_move throw	\ true abort" ISE: not a valid move operation"
  endcase
;

\ SFP107...
: deep>regM0	{ sres dres -- }
\ copy deep sres to register dres.
  sres DR1> = if
    a[ pop .n  {{ dres l>a }} ]a use-a
  else
    a[ LDR ]a  dres l>a  sres gl>a  use-a
  endif
;

: reg>deepM0	{ sres dres -- }
\ sres is a register, copy it to a deep location.
  dres >DR1 = if
    a[ push .n  {{ sres l>a }} ]a use-a
  else
    a[ STR ]a sres l>a  dres l>a use-a
  endif
;

: sdMoveM0/M1,	{ sres dres -- }
\ use r1 as a scratch register
  case
    sres reg? dres reg? and     ?of a[ MOV ]a dres l>a sres l>a use-a endof
    sres lit? dres reg? and     ?of dres sres ResLoadLit endof
    sres deep? dres reg? and    ?of sres dres deep>regM0  endof
    sres reg? dres deep? and    ?of sres dres reg>deepM0  endof
    sres lit? dres deep? and ?of
      rR1 sres ResLoadLit  rR1 dres reg>deepM0
    endof
    sres deep? dres deep? and ?of
      sres rR1 deep>regM0  rR1 dres reg>deepM0
    endof
      opt_bad_move throw	\ true abort" ISE: not a valid move operation"
  endcase
;

: sdmove,
  M0/M1?
  if  sdmoveM0/M1,  else  sdMoveARM/M3,  endif
;
\ ...SFP107

: sdcmp0,	{ sres reg -- ; SFP048 }
\ compare with 0, setting status flags, uses reg as a scratch if necessary
  case
    sres reg? ?of
      a[ CMP ]a  sres l>a  a[ # 0 ]a  use-a
    endof
    sres lit? ?of
      a[ MVL ]a  reg l>a  sres #l>a  use-a
      a[ CMP ]a  reg l>a  a[ # 0 ]a  use-a
    endof
      a[ LDR ]a  reg l>a  sres gl>a  use-a
      a[ CMP ]a  reg l>a  a[ # 0 ]a  use-a
  end-case
;

: CopyTOS       \ res -- ; copy TOS to new register resource which becomes TOS
  pop(ws) over sdmove,  push(ws)  ;


\ *********************************************
\ Shuffling the stack at a basic block boundary
\ *********************************************

((
N.B. ***WARNING*** Ugly code follows!

At the end of a basic block, the array STACK-PICTURE holds the current state
of the data stack. The variable #CHANGED(WS) tells us by how many cells the
data stack has grown (+ve) or shrunk (-ve) since the stack was last in model
form as defined by STACK-MODEL. If the stack has grown, additional space must
allocated on the stack *before* all the stack items are shuffled onto the stack
in model order. If the stack has shrunk, the stack items must be reordered into
model form, and the stack reduced *after* the shuffle.

This means that in the case of a changed stack, translation is necessary from
before shuffle locations to after shuffle locations.

The position of the data on the working stack is held in the array
STACK-PICTURE, whose contents are adjusted for a growing stack. The required
output picture is held in DEST-PICTURE, which is initialised from STACK-MODEL,
and then adjusted if the stack is to shrink.

The algorithm for shuffling the stack is as follows:
  mark all resources in STACK-PICTURE as sources
  find all dest=source, already in place
    unmark if refcount=1
  begin
    find exchanges (source=dest(dest) with refcounts=1)
      lay exchange, unmark source and dest
    lay moves to free resources
      decrement source refcount
      unmark if source refcount=0
    if sources left, and no changes made
      get a scratch resource
      copy most used source in output to scratch
      lay moves as above
      free scratch
      if still no changes made
        panic!
  until all sources unmarked

Note that the strategy in this optimiser is to delay code generation for as
long as possible, and not to move the stack pointer unless absolutely
necessary. All the problems are passed to the stack shuffling code, which
is ugly and contains heuristics designed to cope with the peculiarities
of the target CPU. However, if it can cope with an 386/486/Pentium, it
can probably cope with most CPUs.
))

struct /temp-res        \ -- len ; holds information during renumbering
  int temp-resource
  int temp-refcount
end-struct

/stack-picture /temp-res * buffer: xlat-buffer

\ Renumber the working stack to allow for the stack pointer having been
\ moved BEFORE the shuffle takes place. For entries in deep stack only,
\ the reference count of the entry is preserved, along with the resource
\ it must be transferred to. When the whole stack has been scanned and the
\ list built, it is scanned again, and the deep stack entries are replaced
\ by their new entries, and the reference count is transferred.
: prepare-stack(ws)     \ -- ; updates the current stack items to allow for #CHANGED(WS)
  check(ws)

\ Save the reference counts of the working deep stack entries
  xlat-buffer  #on-stack(ws) @ 0 ?do	\ -- temp
     i in(ws) @ tuck deep? if           \ -- res temp ; copy if in deep stack
       \ save the new resource in TEMP-RESOURCE
       over res-value @                 \ current deep stack index
       #changed(ws) @ abs +             \ update index by stack change
       in(ms) @                         \ get new resource
       over temp-resource !             \ this will be the resource afterwards
       \ save the reference count in TEMP-REFCOUNT
       over res-refcount @  over temp-refcount !
       \ update the temporary buffer pointer
       /temp-res +                      \ step to next item
     endif
     nip                                \ discard resource
  loop
  drop                                  \ discard temp

\ Zero the reference counts of the deep stack entries. These will
\ be restored later, but we need to ensure that any previous values
\ that are now unused are set to zero.
  #on-stack(ws) @ 0 ?do
     i in(ws) @ dup deep?               \ zero it if in deep stack
     if  res-refcount off  else  drop  endif
  loop

\ Update the contents of the working stack, ensuring that the
\ reference counts of the new deep stack positions are also
\ updated.
  xlat-buffer  #on-stack(ws) @ 0 ?do	\ for each item
    i in(ws) @ deep? if                 \ deep stack item?
      dup temp-resource @               \ get the new resource
      dup i in(ws) !                    \ make it current
      over temp-refcount @              \ get the required reference count
      swap res-refcount !               \ and set it in the resource
      /temp-res +                       \ step on
    endif
  loop
  drop
;

\ If the stack pointer is moved BEFORE the shuffle, the STACK-MODEL already
\ represents the output picture. However, if the stack pointer is to be
\ moved afterwards, the output stack picture must be modified.
: prepare-dest(ws)      \ -- ; prepare the output stack picture
  stack-model dest-picture /stack-picture cells cmove
  #changed(ws) @ 0< if
    #on-stack(ws) @ 0 ?do
      i in(ms) @ deep? if               \ if in deep stack
        i #changed(ws) @ abs + in(ms) @ \ take new item from model
        i in(os) !                      \ and store in destination
      endif
    loop
  endif
;

: mark-sources          \ -- ; mark all items on the working stack
  #on-stack(ws) @ 0
  ?do  i in(ws) @ mark  loop
;

: growPSP	\ u --
\ Increase the size of the data stack by n bytes.
  Thumb? if
    Cortex-M0/M1? if
      >r  a[ SUB .s psp, psp, # ]a r> n>a  use-a
      exit
    endif
  endif
  >r  a[ SUB psp, psp, # ]a r> n>a  use-a
;

: shrinkPSP	\ u --
\ Decrease the size of the data stack by n bytes.
  Thumb? if
    Cortex-M0/M1? if
      >r  a[ ADD .s psp, psp, # ]a r> n>a  use-a
      exit
    endif
  endif
  >r  a[ ADD psp, psp, # ]a r> n>a  use-a
;

variable ^DS1-move      \ -- addr ; address at which last LDR reg, [ psp ] was laid

also assembler
: shrink-opt?   \ -- flag ; true if optimisation available for LDR reg, [ psp ]
  Thumb? 0= if				\ ARM32
    ^DS1-move @ 4+ chere(t) =		\ must be 4 bytes previous
  else
    Cortex-M0/M1? 0= if			\ SFP107
      ^DS1-move @ 4+ chere(t) = dup if	\ must be 4 bytes previous
        drop  ^DS1-move @ T2x32@(t)	\ opcode
        $FFF0:0FFF and $F8D0:0000 =	\ LDR reg1, [ reg2, # 0 ] ; 12 bit form
      endif
    else
      ^DS1-move @ 2+ chere(t) = dup if	\ must be 2 bytes previous
        drop  ^DS1-move @ w@c(t)	\ opcode, should be LDR Rt, [ Rn, # 0 ]
        $FFF8 and			\ extract opcode + Rn + #
        $6800 get-psp 3 lshift or	\ expected
        =
      endif
    endif
  endif
;

: remove-shrink \ -- ; replace LDR reg [ psp ] by LDR reg [ psp ], # 4
  debugging @
  if   cr ." remove shrink"  endif
  Thumb? if
    Cortex-M0/M1? if			\ SFP107
      ^DS1-move @ dup w@c(t)		\ LDR Rt, [Rn,#0]
      dup 7 and swap 3 rshift 7 and	\ -- addr Rt Rn
      8 lshift  1 rot lshift or  $C800 or \ LDMIA Rn, { Rt }
      swap w!c(t)
    else
      ^DS1-move @ dup T2x32@(t)		\ must be LDR Rt, [ Rn, # 0 ] - enc T3
      $000F:F000 and  $F850:0B04 or	\ offset=4, P=0, U=1, W=1, enc T4
      swap T2x32!(t)
    endif
  else
    ^DS1-move @ dup l@c(t)
    $FEFFFFFF and $4 or			\ make post incremented by 4
    swap l!c(t)
  endif
  -1 ^DS1-move !                        \ inhibit future conversions
;
previous

: shrink-cell   \ -- ; lay code to remove a cell from the stack
  shrink-opt? if                        \ optimisation available?
    remove-shrink                       \ convert
  else
    $04 shrinkPSP
\    a[ ADD psp, psp, # $04 ]a  use-a    \ or lay code
  endif
;

: shrink-stack  \ -- ; drop items from stack
  case  #changed(ws) @ abs cells(t)     \ number of bytes to change stack by
    0 of    endof
    4 of  shrink-cell  endof
        shrinkPSP
\      >r  a[ ADD psp, psp, # ]a r> n>a  use-a
  end-case
;

variable ^grow-cell     \ -- addr ; address at which SUB psp, psp, # 4 was laid
  -1 ^grow-cell !                       \ will never happen

: grow-cell     \ -- ; add a cell to the stack pointer
  chere(t) ^grow-cell !
  4 growPSP
\  a[ SUB psp, psp, # 4 ]a  use-a
;

: grow-cell?    \ -- true ; true if previous instruction was SUB psp, psp, # 4
  chere(t) ^grow-cell @ 4+ =
;

: remove-grow   \ -- ; backup by size of SUB psp, psp, # 4
  -4 callot(t)  -1 ^grow-cell !
;

: grow-stack    \ -- ; add items to stack
  debugging @
  if   cr ." > grow stack by "  endif
  case  #changed(ws) @ abs cells(t)     \ number of bytes to change stack by
    debugging @
    if   dup . ." bytes"  endif
    0 of    endof
    4 of  grow-cell  endof
        growPSP
\      >r  a[ SUB  psp, psp, # ]a r> n>a use-a
  end-case
  debugging @
  if   cr ." < grow stack"  endif
;

: pre-balance(ws)       \ -- ; update stack pointer BEFORE shuffling the stack
  #changed(ws) @ 0> if                  \ if more cells are needed on the stack
    grow-stack
    prepare-stack(ws)
  endif
  prepare-dest(ws)                      \ set up the output stack picture
  mark-sources                          \ marking working stack items
;

: post-balance(ws)      \ -- ; update stack pointer AFTER shuffling the stack
  #changed(ws) @ 0<                     \ if fewer cells are needed on the stack
  if  shrink-stack  endif
;

: 0shuffle      \ -- ; clean up empty stack
  debugging @
  if   cr ." > 0shuffle"  endif
  #changed(ws) @ abs 1- cells(t) ?dup if
    >r a[ LDR  rTOS l>a  [ psp, # r> ]  ]a use-a
  else
    chere(t) ^DS1-move !                \ mark for later optimisation
    a[ LDR  rTOS l>a  [ psp, # 0 ]  ]a  use-a
  endif
;

: 1shuffle      \ -- ; clean up stack at a boundary
  debugging @
  if   cr ." > 1shuffle"  endif
  tos-res rTOS <> if
    tos-res DS1 =                       \ check for later optimisation
    if  chere(t) ^DS1-move !  endif
    tos-res rTOS sdmove,
    tos-res dup unmark  res-refcount off
    rTOS set-tos  1 rTOS res-refcount !
  endif
;

: dec-source    \ res -- ; decrement the ref count of a source
  dup count=1?                          \ only one use?
  if  dup unmark  endif                 \ y: all done with source
  res-refcount decr  ;                  \ count down

variable shuffled       \ -- addr ; set by any shuffle routine that does something

: find-inplace  \ -- ; find all sources already in right place
  #on-stack(ws) @ 0 ?do
    i in(ws) @ i in(os) @ = if          \ source = dest?
      i in(ws) @ dup source?            \ must still be a source
      swap count=1? and if              \ and only one reference
        debugging @ if
          cr i in(ws) @ .res-name ."  is in place"
        endif
        i in(ws) @ dec-source           \ decrement ref count and unmark
        shuffled incr
      endif
    endif
  loop
;

: on-os?        \ res -- pos -1 | 0 ; determine where resource is
  0 swap                                \ assume not there
  #on-stack(ws) @ 0 ?do
    i in(os) @ over =
    if  2drop  i true  unloop  exit  endif
  loop
  drop  ;

: on-ws?        \ res -- pos -1 | 0
  0 swap                                \ assume not there
  #on-stack(ws) @ 0 ?do
    i in(ws) @ over =
    if  2drop  i true  unloop exit  endif
  loop
  drop  ;

: s_swap        { src-reg dest-ds -- ; lay code to swap }
  a[ SWP ]a  src-reg l>a  src-reg l>a  dest-ds gl>a  use-a
;

: s_xchg        { src-res dest-res -- ; lay code to exchange two items }
  debugging @ if
    cr ." s_xchg " src-res dest-res print-sd
  endif
  ARM32? if
    <ARMunpredictable> @ 0= if		\ SFP122
      src-res reg?  dest-res DS1 = and if	\ SFP035
        src-res dest-res s_swap  exit
      endif
      dest-res reg? src-res DS1 = and if	\ SFP035
        dest-res src-res s_swap  exit
      endif
    endif
  endif
  src-res rR0 sdmove,			\ rs -> r
  dest-res src-res sdmove,              \ rd -> rs, may use r1
  rR0 dest-res sdmove,                  \ r -> rd
;

: find-exchange { | src dest -- ; find all sources whose dest is the source }
  #on-stack(ws) @ 0 ?do
    i in(ws) @ -> src
    src unmarkable? if
      i in(os) @ -> dest
      dest unmarkable? if               \ both sources, both count=1
        src on-os? if                   \ source on dest stack?
          dest on-ws? if                \ dest on source stack?
            = if                        \ at same place?
              debugging @ if
                cr ." Exchanging " src .res-name ."  and " dest .res-name
              endif
              src dest s_xchg           \ lay code
              src  dup unmark res-refcount off  \ unmark both
              dest dup unmark res-refcount off  \ unmark both
              i set-dummy(ws)           \ remove source from working stack      SFP018
              shuffled incr
            endif
          endif
        endif
      endif
    endif
  loop
;

: move,         { sres dres -- ; move source to dest }
  debugging @
  if  cr ." move, " sres dres print-sd  endif

  sres DR1 = dres reg? and if		\ if move from TOR (rare)
    sres count=1? if                    \ and it's the last one?
      Thumb? if
        a[ POP {{ dres l>a }} ]a use-a
      else
        a[ LDR ]a dres l>a a[ [ rsp ], # $04 ]a use-a      \ y:, =pop=ldmfd
      endif
      exit
    endif
  endif

  Cortex-M0/M1? 0= if			\ SFP114
    sres reg?  dres DS1 = and           \ STR reg, [ psp ] ?
    grow-cell? and if                   \ after SUBS psp, psp, # 4 ?
      remove-grow
      a[ STR ]a  sres l>a  a[ [ psp, # $-04 ] ! ]a  use-a \ =push=stmfd
      exit
    endif
  endif
  sres DS1 =  dres reg?  and            \ if  LDR reg, [ psp ]
  if  chere(t) ^DS1-move !  endif       \ we may be able to optimise it later
  debugging @
  if  ." Move with SDMOVE," cr  endif
  sres dres sdmove,
;

: find-move     \ -- ; lay available moves
  #on-stack(ws) @ 0 ?do
    i in(ws) @ source?                  \ source must be marked
    i in(os) @ source? 0= and if        \ dest must not be a source
      i in(ws) @  i in(os) @  2dup = if \ check for null move
        2drop
      else
        debugging @ if
          cr ." Moving from " i in(ws) @ .res-name ."  to " i in(os) @ .res-name
        endif
        over swap  move,  dec-source    \ copy and count down
        i set-dummy(ws)                 \ remove source from working stack      SFP018
        shuffled incr
      endif
    endif
  loop
;

: #sources      \ -- n ; return number of marked sources
  0  #on-stack(ws) @ 0 ?do
    i in(ws) @ source?
    if  1+  endif
  loop
;

: GetMaxOS      \ -- res ; get the output stack item with the highest ref-count
  0 0                                   \ -- dummy-res dummy-refcount
  #on-stack(ws) @ 0 ?do
    i in(os) @ res-refcount @ over >
    i in(os) @ source? and if
      2drop  i in(os) @ dup res-refcount @
    endif
  loop
  drop
;

: replace(ws)   \ oldres newres -- ; replace old by new on working stack
  swap  #on-stack(ws) @ 0 ?do
    i in(ws) @ over =
    if  over i in(ws) !  endif
  loop
  2drop
;

: GetScratch    \ -- newres ; Find a scratch resource and copy to it
  GetMaxOS                              \ find dest that is most used source
  GetShuffleReg ?dup if                 \ can we use a spare register?
    2dup sdmove,                        \ MOV scratch, maxos
  else                                  \ n:  res --
    dup reg? if                         \ if maxos is register
      Thumb? if
        a[ PUSH  {{ dup l>a }} ]a use-a
      else
        a[ STR ] ]a  dup l>a a[ [ rsp, # -4 ] ! ]a  use-a \ push maxos
      endif
    else
      dup rR0 sdmove,                   \ MOV rER0 maxos0
      Thumb? if
        a[ push  {{ r0 }} ]a use-a
      else
        a[ str  r0, [ rsp, # -4 ] ! ]a use-a	\ SFP101 for ARM9
      endif
    endif
    DR1
  endif
  over res-refcount @                   \ copy refcount into new resource
  over res-refcount !
  2dup replace(ws)                      \ replace all source occurrences by new
  swap  dup unmark  res-refcount off    \ clear source and refcount on old
  dup mark                              \ mark as source
;

: keep-moving   \ -- ; keep moving until there's nothing left to do
  begin
    shuffled @
    find-move
    shuffled @ =
  until
;

: FreeScratch   \ res -- ; free up the scratch resource
  res-refcount @
\  abort" Couldn't free scratch resource"  ;
  opt_bad_scratch ?throw
;

: nshuffle      \ -- ; shuffle the general stack
  begin
    shuffled off
    find-inplace
    find-exchange
    find-move
    #sources dup if                     \ if shuffles left to do
      shuffled @ 0=                     \ did it manage to shuffle?
      if                                \ n: cyclic stack, we'll use a new resource
        drop                            \ discard remaining number of sources
        GetScratch                      \ get a scratch resource
        keep-moving                     \ let data be moved
        FreeScratch                     \ free up the scratch resource
        shuffled @ 0=                   \ check for disaster
        opt_cant_shuffle ?throw
        #sources                        \ see if there's more to do
      endif
    endif
    0=
  until
;

: (c_shuffle)     \ -- ; SFP070
\ Clean up stack at a boundary. Performs no checks.
  -1 ^grow-cell !                       \ initialise data stack push optimisation
  -1 ^DS1-move !                        \ initialise data stack pop optimisation
  pre-balance(ws)
  case  #on-stack(ws) @
    0 of  0shuffle  endof
    1 of  1shuffle  endof
          nshuffle
  endcase
  post-balance(ws)
  init-stack(ws)
;

: c_shuffle     \ --
\ Clean up stack at a boundary. Perform resource check.
\ Output debug info if enabled.
  0 to GuardedFlags?			\ regular shuffle
  debugging @ if
    cr ." > c_shuffle"
  endif
  (c_shuffle) check-stack(ws)		\ SFP070
  debugging @ if
    cr ." < c_shuffle"
  endif
;

: popTOSopM0	\ -- opcode
\ Returns the Cortex M0/M1 opcode to pop TOS from the data stack,
\ usually  LDMIA r6! { r7 } = $CE80.
  $C800 get-psp 8 lshift or		\ LDMIA rn !
  1 get-tos lshift or			\ { tos }
;

: c_GuardedShuffle     \ --
\ When the stack is shuffled after a compare and before a branch,
\ the CCR must be preserved. On CPUs for which the MOV instruction
\ sets flags, a special version of c_shuffle is needed. The
\ shuffle on the ARM doesn't currently affect the status flags.
\ Cortex code density can be improved by permitting MOV .S instructions
\ in many cases.
  1 to GuardedFlags?			\ protected
  debugging @ if
    cr ." > c_guardedShuffle"
  endif
  M0/M1? if
    chere(t)
    a[  mrs  r0, 0   push .n {{ r0 }}  ]a use-a	\ 6 bytes
    (c_shuffle) check-stack(ws)
    a[  pop .n {{ r0 }}   msr  0 r0  ]a use-a	\ 6 bytes
    chere(t) swap -			\ gap
    case
      #12 of  #-12 callot(t)  endof	\ null shuffle?
      #14 of  chere(t) 8 - w@c(t) popTOSopM0 =
              if  #-14 callot(t)  popTOSopM0 w,c(t)  endif
      endof
    endcase
  else
    (c_shuffle) check-stack(ws)
  endif
  debugging @ if
    cr ." < c_guardedShuffle"
  endif
  0 to GuardedFlags?
;

: c_flushcode   \ -- ; force stack and optimiser to known state
  reset-opt  c_shuffle
;


\ ***************************************************************************
\ Copying literal when modified at run time. If a compile time operation
\ is performed on a multiply referenced literal, ALL copies are changed.                SFP0017
\ To prevent this, only literals with a reference count of 1 may be modified.
\ ***************************************************************************

: ?NewLitNeeded \ n -- ; replaces item n on stack if refcount<>1 ; SFP017
  in(ws) dup @                          \ preserve address, get resource
  dup lit?  swap count=1? 0= and        \ if not a destination (count=1)
  if                                    \ -- ^stack
    GetLitRes                           \ -- ^stack resource
    over @ res-value @ over res-value ! \ copy literal value to new
    over @ res-refcount decr            \ decrement old reference count
    1 over res-refcount !               \ set new reference count to 1
    swap !                              \ replace resource on stack
  else
    drop
  endif
;

: ?NewTOSlit    \ -- ; ensure literal TOS can be updated ; SFP017
  0 ?NewLitNeeded
;

: ?NewNOSlit    \ -- ; ensure literal NOS can be updated ; SFP017
  1 ?NewLitNeeded
;


\ ******************************************
\ Register requests that may cause a shuffle
\ ******************************************

: ForceNewTOS           \ -- ; get new register as TOS
  GetNewReg ?dup 0= if
    #on-stack(ws) @  c_shuffle  min-depth(ws)	\ SFP036
    defnewTOS
  endif
  push(ws)
;

: ForceNewTOS+NOS       \ -- ; get two new registers for TOS and NOS
  GetNewReg ?dup if
    dup res-refcount incr               \ so that GetNewReg won't find it
    GetNewReg
    over res-refcount decr              \ remove previous protection
    ?dup 0= if
      drop
      #on-stack(ws) @  c_shuffle  min-depth(ws)	\ SFP036
      defnewTOS  defnewNOS
    endif
  else
    #on-stack(ws) @  c_shuffle  min-depth(ws)	\ SFP036
    defnewTOS defnewNOS
  endif
  push(ws)  push(ws)
;


\ ***************
\ Compiling words
\ ***************

\ ---------------------------------
\ Preservation of the link register
\ ---------------------------------

variable EntryMode	\ -- addr ; 0 to generate link save
  1 EntryMode !
variable LinkSaved	\ -- addr ; 0 if LINK not saved
  0 LinkSaved !
variable LeafForced	\ -- addr ; 0 if word must be leaf
  0 LeafForced !

: :Entry,	\ -- ; lay colon procedure entry
  Thumb? if
    a[ push .n  {{ link }} ]a use-a
  else
    a[ str  link, [ rsp, # -4 ] ! ]a use-a	\ ARM9 ; SFP101
  endif
  LinkSaved on				\ SFP059
;

: :Exit,	\ -- ; lay colon procedure exit code ; SFP059
  LinkSaved @ if
    Thumb? if
      a[ pop .n {{ pc }} ]a use-a
    else
      a[ ldr  pc, [ rsp ], # 4 ]a  use-a	\ ARM9
    endif
  else
    Thumb? if
      a[ bx  link ]a use-a
    else
      a[ mov pc, link ]a  use-a
    endif
  endif
\  flushlitpool				\ SFP063
;

: ?SaveLink	\ -- ; save LINK if not already done ; SFP059
  LeafForced @ if  exit  endif
  LinkSaved @ 0=
  if  :Entry,  endif
;

: ?LinkSaved	\ -- ; error if LINK not saved ; SFP059
  LeafForced @ 0= if
    LinkSaved @ 0= opt_noLinkSave ?throw
  endif
;

: ?LeafError	\ --
  LeafForced @ opt_LeafError ?throw
;

: +ForceLeaf	\ -- ; force this word to be a leaf
  1 LeafForced !
;

: -ForceLeaf	\ -- ; reset leaf handling
  0 LeafForced !
;


\ ----------------------
\ Start and end of words
\ ----------------------

: c_:           \ -- ; initialise optimiser and compile code for :
  init-stack(ws)  reset-opt  start-asm
  LinkSaved off  EntryMode @ 0=
  if  :Entry,  endif
;

: c_exit        \ -- ; compile code for EXIT ; COMPILER inhibits inlining
  c_shuffle  reset-opt  :Exit,
;

: c_;		\ -- ; compile code for EXIT at ";" ; SFP038 SFP059
  c_shuffle  reset-opt
  #lp-entries
  if  not-inlined  endif
  LinkSaved @ 0=
  if  not-inlined  endif
  :Exit,
  FlushLitpool				\ SFP063
  -ForceLeaf
;


\ --------------------------
\ Words involved with CREATE
\ --------------------------
\ The code laid down for a child of a defining word is
\    PUSH link | BL runtime | data
\ For CREATE this will be
\    PUSH link | BL runtime | address -->-- data
\ For both Thumb2 and ARM, the PUSH instruction is forced to be
\ a 4-byte instruction as this simplifies alignment in XC7. The
\ instruction at offset 4 is a BL instruction.

\ When far calls outside the usual range are required
\   ARM 24+2 = signed 26 bits
\   T2  24+1 = signed 25 bits
\   T1  22+1 = signed 23 bits
\    PUSH link | longcall | data

\ For T2 the full long sequence is:
\    PUSH .n { link }		\ $+00: save link
\    MVL32+1 r0, # dest		\ $+02: load dest to r0
\    BLX  r0			\ $+0A: call routine
\				\ $+0C: link points here

\ For ARM the full long sequence is:
\    STR  link, [ rsp ], # -4	\ $+00: save link
\    MOV  link, pc		\ $+04: link points to $+0C
\    LDR  pc, [ link ], # 4	\ $+08: branch to contents of $+0C
\    dest address		\ $+0C: holds destination address
\				\ $+10: link points here

variable <LongCalls?>	\ -- addr
\ True to enable calls beyond the normal range of BL.
  <LongCalls?> off

: ?LongCalls	\ -- ; check long calls enabled
  <LongCalls?> @ 0= opt_LongCalls ?throw
;

also assembler
: BLrange?	\ dest -- flag ; SFP077
\ Return true if the target address is range for a BL instruction.
  Thumb? if
    chere(t) 4 + - 25bitT2?
  else
    chere(t) 8 + - 26bit?	\ can do a relative branch if in range
  endif
;

: BL_T2,	\ ref -- ; lay BL instruction
  $F000:D000 swap +Trel24,  ;

: BL_ARM,	\ ref -- ; lay BL instruction
  $EB000000 swap +rel24,  ;
previous

: BLrunT2,	\ r-t-sym -- ; lay a T2 BL call
\ Compile call to the run-time action of a symbol as a BL instruction.
  Cortex-M0/M1? if			\ SFP107
    a[ push .n {{ link }}  nop ]a use-a
  else
    a[ str  link, [ rsp, # -4 ] ! ]a use-a	\ four bytes for T2
  endif
  asm-run-ref BL_T2,  ;

: BLrunARM,	\ r-t-sym -- ; lay an ARM BL call ; SFP078
\ Compile call to the run-time action of a symbol as a BL instruction.
\  a[ stmfd  rsp ! { link } ]a use-a		\ four bytes for ARM7
  a[ str  link, [ rsp, # -4 ] ! ]a use-a	\ four bytes for ARM9
  asm-run-ref BL_ARM,
;

: BLrun,	\ r-t-sym -- ; lay an ARM BL call ; SFP078
\ Compile call to the run-time action of a symbol as a BL instruction.
  Thumb? if  BLrunT2,  else  BLrunARM,  endif  ;

: compile-call	\ r-t-sym --
\ Used to construct a child of a defining word. It lays a call to
\ the run-time action of the defining word. This run-time action
\ may be forward referenced. The address after the code sequence
\ must be cell aligned because it will contain data.
\ We need to preserve the link register so that the DOES> portion
\ of a CREATE can return directly to where the defining word was
\ called
  dup 2 cells - Run-Time? 0=		\ run-time unresolved?
  if  BLrun, exit  endif		\   all forward refs short
  dup @(s) BLrange?			\ short call possible?
  if  BLrun, exit  endif		\   use it
  ?LongCalls				\ check long calls enabled
  Thumb? if
    Cortex-M0/M1? if
      >r  chere(t) 3 and if		\ align, 12 bytes follow NOP
        a[  nop .n  ]a use-a		\ $+00
      endif
      \ The code sequence is constructed this way so LR in the called
      \ word always points to the next inline address.
      a[  ldr      r0, $ 4 +		\ $+00: load from slot
          b        $ 6 +		\ $+02: skip slot
          dxl      r> asm-run-ref	\ $+04: must be aligned
          nop .n			\ $+08
          blx      r0			\ $+0A: call
      ]a use-a				\ $+0C
    else
      >r
      a[ PUSH .n {{ link }} ]a use-a		\ $+00: save link
      a[ MVL32+1 r0, # ]a r> asm-run-ref use-a	\ $+02: load dest to any reg except TOS
      a[ BLX  r0 ]a use-a			\ $+0A: call routine, link points to $+0C
    endif
  else
    a[ str  link, [ rsp, # -4 ] ! ]a use-a	\ $+00: four bytes for ARM
    a[ mov  link, pc ]a use-a			\ $+04: link points to $+0C
    a[ ldr  pc, [ link ], # 4 ]a use-a		\ $+08: branch to contents of $+0C, link points to $+010
    asm-run-ref [+assembler] absolute [previous] \ $+0C: <dest>
  endif
;

\ SFP025 ...

: compile-run   \ symbol --
  2 cells+ compile-call  ;

also assembler
: T2hasBLins?	\ addr -- flag
\ Return true if location contains a BL opcode.
  T2x32@(t) $F000:D000 and $F000:D000 =  ;
previous

: ARMhasBLins?	\ addr -- flag
\ Return true if location contains a BL opcode.
  l@c(t) $FF000000 and $EB000000 =  ;

: hasBLins?	\ addr -- flag
\ Return true if location contains a BL opcode.
  Thumb? if  T2hasBLins?  else  ARMhasBLins?  endif  ;

also assembler
: patch-call    \ runtime cfa -- ; patch CFA with CALL/JSR instruction
  Thumb? if
    dup 4 + T2hasBLins?			\ is it BL for in-range call?
    if  4 + fix_Trel24  else  2 + fix_mov32+1  endif
  else
    4 +  dup ARMhasBLins?		\ is it BL for in-range call?
    if  fix_r24  else  8 + fix_abs  endif
  endif
;
previous

: patch-child   \ runtime cfa -- ; patch CFA of child of defining word
  patch-call  ;
\ ...SFP025


: >body(t)      \ xt[t] -- pfa[t] ; see also C_>BODY
  single-section?			\ SFP115
  if  8 +  exit  endif
  Thumb? if
    dup 4 + T2hasBLins?			\ BL at offset 4
    if  8 +  else #12 +  endif
  else
    dup 4 + ARMhasBLins? 		\ if not BL instruction
    if  8 +  else  #16 + endif		\ skip extra MOV LDR instructions
  endif
  @c(t)
;

: >does(t)      \ xt -- runtime
\ Finds runtime address of child of defining word
  Thumb? if
    dup 4 + T2hasBLins? if
      4 + >callDestT2			\ get dest of BL instruction
    else
      2 + >Dest32			\ get target of MOV32, rx # addr+1
    endif
  else
    dup 4 + ARMhasBLins? if		\ if not BL instruction
      4 + >callDestARM
    else
      #12 + l@c(t)			\ skip MOV and LDR, fetch absolute address
    endif
  endif
;

: c_create(cc)  \ -- addr ; code generated by interpreted CREATE
  single-section? if
    Thumb? if
      a[ str  link, [ rsp, # -4 ] ! ]a use-a  LinkSaved on
      the-label DOCREATE asm-ref BL_T2,	\ reference label
    else
      a[ str  link, [ rsp, # -4 ] ! ]a use-a  LinkSaved on
      the-label DOCREATE asm-ref BL_ARM, \ reference label
    endif
    chere(t)				\ address returned by CREATE
  else
    target-sym CONSTANT compile-run
    cdata? if                           \ in dictionary section?
      chere(t) cell(t) +
    else
      align(t) here(t)
    endif
    dup ,c(t)                           \ lay address inline in code space
  endif
;

: DODOES,       \ -- ; lay the entry code needed at the start of the
                \ run-time code defined by DOES>
  init-stack(ws)  reset-opt  start-asm
  LinkSaved on					\ CREATE always saves LINK ; SFP059
  DOESID start-seq  start-bkup			\ SFP029
  single-section? if
    Thumb? if
      a[ str  tos, [ psp, # -4 ] ! ]a use-a	\ SFP101
      a[ bic tos, link, # 1 ]a use-a		\ new TOS is PFA is return address
    else
      a[ str  tos, [ psp, # -4 ] ! ]a use-a	\ ARM9 ; SFP101
      a[ mov tos, link ]a use-a			\ new TOS is PFA is return address
    endif
  else
    Thumb? if
      Cortex-M0/M1? if				\ SFP107
        a[ sub .s  psp, psp, # 4
           str     tos, [ psp, # 0 ]
           mov     r0, link
           sub .s  r0, # 1
           ldr     tos, [ r0, # 0 ]
        ]a use-a
      else
        a[ str  tos, [ psp, # -4 ] ! ]a use-a	\ save old TOS
        a[ ldr tos, [ link, # -1 ] ]a use-a	\ get CREATE address
      endif
    else
      a[ str  tos, [ psp, # -4 ] ! ]a use-a	\ ARM9 ; SFP101
      a[ ldr tos, [ link ] ]a use-a		\ get CREATE address
    endif
  endif
  end-seq					\ SFP029
;

: c_does>_>r    \ -- ; back up and lay new code ; SFP035
  single-section? if
    Thumb? if
      a[ bic r1, link, # 1 ]a use-a		\ new value is PFA is return address
      a[ push .n  {{ r1 }} ]a use-a
    else
      a[ str  link, [ rsp, # -4 ] ! ]a use-a	\ ARM9 ; SFP101
    endif
  else
    Thumb? if
      Cortex-M0/M1? if				\ SFP107
        a[ mov     r1, link
           sub .s  r1, # 1
           ldr     r0, [ r1, # 0 ]
           push .n  {{ r0 }}
        ]a use-a
      else
        a[ ldr r0, [ link, # -1 ] ]a  use-a	\ get CREATE address
        a[ push .n  {{ r0 }} ]a use-a
      endif
    else
      a[ ldr r1, [ link ] ]a  use-a		\ get CREATE address ; SFP101
      a[ str  r1, [ rsp, # -4 ] ! ]a use-a	\ ARM9
    endif
  endif
;


\ ***********************************
\ Referencing a target word or symbol
\ ***********************************

\ Set up data for inlining control - default is for 32-bit mode

4 #:entry !             \ size of COLON defn. entry code
4 #:exit !              \ size of COLON defn. exit code
0 #code-entry !         \ size of CODE defn. entry code
4 #code-exit !          \ size of CODE defn. exit code

: ARMentry/exit		\ --
  4 #:entry !				\ size of COLON defn. entry code
  4 #:exit !				\ size of COLON defn. exit code
  0 #code-entry !			\ size of CODE defn. entry code
  4 #code-exit !			\ size of CODE defn. exit code
;

: ThumbEntry/Exit	\ --
  2 #:entry !			\ size of COLON defn. entry code
  2 #:exit !			\ size of COLON defn. exit code
  0 #code-entry !		\ size of CODE defn. entry code
  2 #code-exit !		\ size of CODE defn. exit code
;

: (Legacy-mode)	\ -- ; select 32 bit mode for classic ARM
  ARMentry/Exit
  0 cells -> RegMode			\ register selection
  #10 set-TOSreg  #12 set-PSPreg	\ switch assembler
  rR7 res-refcount off  rR6 res-refcount off
  init-stack(ws)
;

: (T2-mode)	\ -- ; select Thumb-2 register operation
  ThumbEntry/Exit
  1 cells -> RegMode				\ register selection
  #7 set-TOSreg  #12 set-PSPreg
  rR10 res-refcount off  rR6 res-refcount off
  init-stack(ws)
;

: (M0M1-mode)	\ -- ; select Cortex-M0/M1 register operation
  ThumbEntry/Exit
  2 cells -> RegMode				\ register selection
  #7 set-TOSreg  #6 set-PSPreg
  rR10 res-refcount off  rR12 res-refcount off
  init-stack(ws)
;

-1 value ISB?	\ -- flag
\ Non-zero to generate ISB after write to CONTROL reg.
\ The Keil simulator cannot cope!

\ Set instruction set defaults
Thumb-2 (T2-mode)  <use-litpool> off   <ARMunpredictable> off

also cc/i

: +ISBcontrol	cc/i	\ --
\ Generate an ISB instruction after writing to the CONTROL
\ register.
  -1 -> ISB?  ;
: -ISBcontrol	cc/i	\ --
\ Do not generate an ISB instruction after writing to the CONTROL
\ register.
  0 -> ISB?  ;

: T2-mode	cc/i	\ -- ; select 32 bit mode
\ *G Selects generation of code using the full Thumb-2
\ ** instruction set and a register allocation model optimised
\ ** for Thumb-2. This is the default.
  Thumb-2 (T2-mode)  <use-litpool> off  ;

: Cortex-M0	cc/i	\ --
\ *G Select Cortex-M0 code generation.
  Cortex-M0 (M0M1-mode) <use-litpool> on  1 -> ISB?  ;
: Cortex-M1	cc/i	\ --
\ *G Select Cortex-M1 code generation.
  Cortex-M1 (M0M1-mode)  <use-litpool> on  1 -> ISB?  ;
: Cortex-M3	cc/i	\ --
\ *G Select Cortex-M3 code generation.
  Cortex-M3 (T2-mode)  <use-litpool> off  -1 -> ISB?  ;
: Cortex-M4	cc/i	\ --
\ *G Select Cortex-M4 code generation with the integer DSP
\ ** extension.
  Cortex-M4 (T2-mode)  <use-litpool> off  -1 -> ISB?  ;
: Cortex-M4F	cc/i	\ --
\ *G Select Cortex-M4F code generation with the integer DSP
\ ** and single precision VFP extensions.
  Cortex-M4F (T2-mode)  <use-litpool> off  -1 -> ISB?  ;

: ARM-mode    cc/i    \ -- ; select 32 bit mode
\ *G Selects generation of code using the ARM 32 bit instruction
\ ** set and a register allocation model optimised for Thumb-2.
\ ** This switch is for code generation with interworking between
\ ** ARM32 and Thumb code.
  ARM32 (T2-mode)  <use-litpool> on  ;

: Legacy-mode    cc/i    \ -- ; select 32 bit mode
\ *G Selects generation of code using the ARM 32 bit instruction set
\ ** and the register allocation model used by previous ARM compilers
\ ** that do not support Thumb-2. Selecting this mode provides
\ ** compatibility with previous ARM-only source code that may
\ ** include assembler.
  ARM32 (Legacy-mode)  <use-litpool> on  ;

: ARM7TDMI	[+interpreter] Legacy-mode [previous]  ;
\ *G A synonym for *\fo{Legacy-mode}.
: 32bit-mode	[+interpreter] Legacy-mode [previous]  ;
\ *G A synonym for *\fo{Legacy-mode}.

\ SFP117...
: +ARM32-Cortex-A/R	cc/i	( -- )  <ARMunpredictable> on  ;
\ *G ARM32 code that must run on Cortex-A and Cortex-R cores must
\ ** allow for increased restrictions in the register usage,
\ ** especially for the LDRx and STRx instructions.
: -ARM32-Cortex-A/R	cc/i	( -- )  <ARMunpredictable> off  ;
\ *G Remove the restrictions introduced by *\fo{+ARM32-Cortex-A/R}.
\ ...SFP117

\ *P To support standalone Forths in which RAM or auxiliary Flash
\ ** may be more than 16/32 Mb away from the primary Flash area,
\ ** optimisation of *\fo{>BODY} and *\fo{TO-DO} can be controlled
\ ** by the *\fo{xLongCallsx} directives below. These are
\ ** necessary for processors such as the Philips LPC families,
\ ** in which the Flash and RAM locations in memory are fixed.
\ ** Note that *\fo{+LongCalls} is only required for an interactive
\ ** Forth or when Forth calls to code outside the normal 16/32Mb
\ ** branch range are required.

: +LongCalls	cc/i	\ --
\ *G Permit long call sequences.
  <LongCalls?> on  ;
: -LongCalls	cc/i	\ --
\ *G Forbid long call sequences (default).
  <LongCalls?> off  ;
: LongCall?	cc/i	\ -- flag ; long call sequences permitted?
  <LongCalls?> @  ;
: LongCalls?	cc/i	\ -- flag
\ *G Long call sequences permitted?
  <LongCalls?> @  ;

: inlining	cc/i	\ u -- ; base version with protection
\ *G Specifies that words of less than *\i{u} bytes can be binary
\ ** inlinined. Use zero to disable binary inlining. Note that
\ ** inlining interacts with the *\fo{+LeafCalls} directive.
\ ** The use of *\fo{INLINING} is no longer recommended.
  dup if
    EntryMode @ opt_LeafInline ?throw
  endif
  inlining
;

\ *P Cortex and ARM CPUs save subroutine return addresses in the LINK
\ ** register. If a word calls another word, the LINK register
\ ** must be preserved, usually on the return stack. Faster and
\ ** shorter code can be generated by only saving the LINK register
\ ** as required. We refer to this as the *\fo{LeafCall}
\ ** optimisation. All branches ensure that the LINK register
\ ** is saved in case a later CALL/BL is made. If you know that
\ ** a call will not be made, use *\fo{ISLEAF} before
\ ** *\fo{: <name>} to inhibit the save of the LINK register, e.g.
\ *E IsLeaf : uDelay   \ n -- ; short delay
\ **   begin  dup 1- 0= until
\ **   drop
\ ** ;
\ *P The default is *\fo{+LeafCalls}.

: +LeafCalls	cc/i	\ -- ; enable late LINK saving
\ *G When used, the LINK register is only saved if it has to be.
\ ** Default.
  Inlining? opt_LeafInline ?throw	\ SFP068
  1 EntryMode !
;
: -LeafCalls	cc/i	\ -- ; disable late LINK saving
\ *G Forces colon definitions to save the LINK register on entry.
  0 EntryMode !  ;
: LeafCalls?	cc/i	\ -- flag ; test for late LINK saving
\ *G Returns non-zero if late LINK saves are enabled.
  EntryMode @  ;
: IsLeaf	cc/i	\ -- ; next word is a LEAF
\ *G Marks the next colon definition as being a leaf node, i.e. it
\ ** makes no subroutine calls.
  Inlining? opt_LeafInline ?throw	\ SFP068
  1 LeafForced !
;
previous definitions

0 value AlignReturn?	\ -- flag
\ Set before a Thumb call whose return address must be aligned.
\ ARM32 calls are always aligned. The flag is reset when detected.
\ This flag is set when the call is followed by a 32 bit value
\ that must lie on a 32 bit boundary.

: +AlignReturn	\ --
\ Set the alignment flag if in Thumb mode.
  Thumb?
  if  -1 -> AlignReturn?  endif
;

: shortCall	\ symbol -- ; SFP077
\ Lay a call using a BL instruction.
  not-inlined				\ only needed if CALL relative
  ?LeafError ?SaveLink
  >r
  Thumb? if
    AlignReturn? if
      chere(t) 2 and if			\ 4 bytes follow NOP
\ cr ." Generating NOP for BL alignment, " chere(t) .dword .source-line
        a[ NOP .n ]a use-a
      endif
      0 -> AlignReturn?
    endif
  endif
  start-macro
    bl r> 0 ref
  end-macro
;

: longCallM0/M1	\ symbol --
  >r  chere(t) 2 and if			\ align, 12 bytes follow NOP
    a[  nop .n  ]a use-a
  endif
  a[  ldr      r0, $ 6 +		\ load from slot
      b        $ 6 +			\ skip slot
      dxl      r> 1 ref			\ must be aligned
      nop .n
      blx      r0			\ call
  ]a use-a
  0 -> AlignReturn?
;

: longCallCM3+	\ symbol --
  >r  AlignReturn? if
    chere(t) 2 and 0= if		\ 10 bytes follow NOP
\ cr ." Generating NOP for BLX alignment, " chere(t) .dword .source-line
      a[ NOP .n ]a use-a
    endif
    0 -> AlignReturn?
  endif
  a[ MVL32+1 r0, # ]a  r> 1 ref  use-a	\ $+00: load dest to any reg except TOS
  a[ BLX  r0 ]a use-a			\ $+08: call routine
;

: longCallARM	\ symbol --
  a[ mov  link, pc ]a use-a		\ $+00: link points to $+08
  a[ ldr  pc, [ link ], # 4 ]a use-a	\ $+04: branch to contents of $+08
  1 ref [+assembler] absolute [previous] \ $+08: <dest>
;

: longCall	\ symbol -- ; SFP077
  not-inlined				\ only needed if CALL relative
  ?LeafError ?SaveLink			\ SFP059
  Thumb? if
    Cortex-M0/M1?
    if  longCallM0/M1 else  longCallCM3+  endif
  else
    longCallARM
  endif
;

: Compile-Ref	\ symbol -- ; SFP015 SFP077
  c_shuffle  reset-opt                  \ flush stack
  dup inline? if
    inline,(t)
  else
    dup resolved? if
      <LongCalls?> @ if
        dup @(s) BLrange?
        if  shortCall  else  longCall  endif
      else
        shortCall
      endif
    else
      shortCall
    endif
  endif
;

: Compile-Run-Time-Ref  \ symbol --
  debugging @
  if  cr ." >compile-run-time-ref"  endif
  c_shuffle  reset-opt  not-inlined	\ flush stack   SFP024
  2 cells+ 0 run-time-ref
  >r
  start-macro
    bl r>
  end-macro
;


\ **********************************
\ value literals and symbol literals
\ **********************************

: c_lit         \ n -- ; compile code for resolved literal      \ SFP026
  LITPLUSID opt? if			\ if it follows a LIT + sequence        SFP035
\ cr ." LITPLUSLITID"
    LITPLUSLITID seqid !                \ transform it into a LIT + LIT sequence
    seqflags @ seqprev !                \ preserve previous literal value
  else
    reset-opt
  endif
  GetLitRes tuck res-value !            \ put value in literal resource
  push(ws)				\ make TOS	SFP030
;

: DoLiteral,    \ val -- ; lay code for literal handling
  c_lit  ;

: FwdLitARM	\ symbol --
  >r  start-macro
    str  tos, [ psp # -4 ] !		\ ARM9
    adr tos, # r> fwd_adr ref		\ needs START-MACRO
  end-macro
;

: FwdLitCM3+	\ symbol --
  >r  start-macro
    str  tos, [ psp # -4 ] !
    mvl32  tos, # r> fwd_mov32 ref	\ needs START-MACRO
  end-macro
;

: FwdLitM0/M1	\ symbol --
  >r  chere(t) 3 and if			\ align
    a[  nop .n  ]a use-a
  endif
  a[  sub .s   psp, psp, # 4
      str      tos, [ psp, # 0 ]
      ldr      tos, $ 4 +		\ load from slot
      b        $ 6 +			\ skip slot
      dxl      r> 1 ref
  ]a use-a
;

: genFwdLit	\ symbol --
  c_shuffle reset-opt  Thumb? if
    Cortex-M0/M1?
    if  FwdLitM0/M1  else  FwdLitCM3+  endif
  else
    FwdLitARM
  endif
;

: doliteralsym, \ symbol -- ; lay symbol as a literal
  dup resolved? if                      \ if already defined
    @(s) c_lit  exit			\ can apply all optimisations
  endif
  genFwdLit
;

: doliteralcfa, \ symbol -- ; lay symbol XT as a literal
  dup resolved? if                      \ if already defined
    @(s) >ca(t) c_lit  exit		\ can apply all optimisations
  endif
  genFwdLit
;

: c_c/v		\ symbol -- ; compile reference to CONSTANT or VARIABLE ; SFP038
  c/v@ c_lit
;


\ ****************
\ Inter-page calls
\ ****************

: Page-Compile-Ref      \ section symbol --
  dup resolved? 0= opt_interpage ?throw	\ forward reference not permitted
  @(s) c_lit                            \ symbol address
  [ also sections.voc ]
  MS-pageid @ c_lit                     \ page identifier
  [ previous ]
  target-sym PAGE-EXECUTE compile-ref
;


\ *****************
\ String primitives
\ *****************
\ These words are here because they are target sensitive.

\ SFP109 ...

: $,D(T)	\ caddr --
\ Lay down a counted string, allowing for ?B-D.
  lay-target?  If  dup dup c@ 1+ cmove-c,d(t)  Endif
  ?b-d  If  dup c@ 1+ allot  even(h)  Endif
  drop
; doNotSin

: $,C(T)	\ caddr --
\ Lay down a counted string, allowing for ?B-D.
  lay-target?
  If  dup dup c@ 1+ cmove-c,c(t)  Endif
  ?b-d
  If  dup c@ 1+ allot  even(h)  Endif
  drop
; doNotSin

: z$,D(T)	\ caddr --
\ Given a counted string, compile it into IDATA space as a zero
\ terminated string
  lay-target?
  If  dup count cmove-c,d(t)  0 c,d(t)  Endif
  ?b-d
  If  dup count >r  here r@ move  r> allot  0 c,  even(h)  Endif
  drop
; doNotSin

: z$,C(T)	\ caddr --
\ Given a counted string, compile it into CDATA space as a zero
\ terminated string
  lay-target?
  If  dup count cmove-c,c(t)  0 c,c(t)  Endif
  ?b-d
  If  dup count >r  here r@ move  r> allot  0 c,  even(h)  Endif
  drop
; doNotSin

: ",D(T)	\ -- ; lay down a " delimited string
  [interp] c"  $,D(T)			\ SFP127
\  [char] " word  $,D(T)
;

: ",C(T)	\ -- ; lay down a " delimited string
  [interp] c"  $,C(T)			\ SFP127
\  [char] " word  $,C(T)
;

: \",D(T)	\ -- ; lay down a " delimited escaped string
  [interp] C\"  $,D(T)
;

: \",C(T)	\ -- ; lay down a " delimited escaped string
  [interp] C\"  $,C(T)
;

: z",D(T)	\ -- ; compile string into IDATA space
  [interp] c"  z$,D(T)			\ SFP127
\  [char] " word  z$,d(t)
;

: z",C(T)	\ -- ; compile string into CDATA space
  [interp] c"  z$,C(T)			\ SFP127
\  [char] " word  z$,c(t)
;

: z\",D(T)	\ -- ; compile string into IDATA space
  [interp] C\"  z$,d(t)
;

: z\",C(T)	\ -- ; compile string into CDATA space
  [interp] C\" z$,c(t)
;

\ ...SFP109


\ ---------------
\ Stack operators
\ ---------------

\ SFP054...
: XferRes	\ src dest --
\ Copy resource from src to dest and update the working stack.
  2dup sdmove,				\ copy it
  over res-refcount @			\ copy reference count
  over res-refcount !
  over res-refcount off			\ src is now unused
  replace(ws)				\ replace all
;

: ?LoadLit	\ n --
\ If the nth item on the working stack is a literal with a
\ reference count of 2 or more, it is put in a working register
\ to avoid further literal loads.
  in(ws) @				\ get resource
  dup lit? if				\ if a literal
    dup res-refcount @ 1 > if		\ that is multiply referenced
      GetNewReg ?dup if			\ and there's a free register
        over swap XferRes		\ transfer
      endif
    endif
  endif
  drop
;

: ?LoadTosLit	\ --
  0 ?loadLit  ;
: ?LoadNosLit	\ --
  1 ?LoadLit  ;
\ ...SFP054

: c_dup         \ -- ; compile code for DUP
  1 min-depth(ws)  reset-opt		\ SFP030
  tos-res push(ws)
  ?LoadTosLit				\ SFP054
;
((
  tos-res reg? 0= if			\ SFP107
    GetNewReg ?dup if
      tos-res swap XferRes
    endif
  endif
))


\ C_?DUP moved to make optimisation easier

: c_2dup        \ -- ; compile code for 2DUP
  2 min-depth(ws)  reset-opt
  tos-res nos-res push(ws) push(ws)
  ?LoadTosLit ?LoadNosLit		\ SFP054
;

: c_3dup        \ -- ; compile code for 3DUP ; SFP046
  3 min-depth(ws)  reset-opt
  tos-res nos-res 3os-res push(ws) push(ws) push(ws)  ;

: c_drop        \ -- ; compile code for DROP ; SFP029
  1 min-depth(ws)			\ SFP030
  RFROMID opt? if
    seq-backup
    a[ add  rsp, rsp, # 4  ]a  use-a
  endif
  reset-opt  drop(ws)  ;

: c_2drop       \ -- ; compile code for 2DROP
  2 min-depth(ws)  reset-opt		\ SFP030
  drop(ws) drop(ws)  ;

: c_over        \ -- ; compile code for OVER
  2 min-depth(ws)  reset-opt		\ SFP030
  nos-res push(ws)
  ?LoadTosLit				\ SFP054
;

: c_2over       \ -- compile code for 2OVER
  4 min-depth(ws)  reset-opt		\ SFP030
  4os-res push(ws)  4os-res push(ws)  ;

: c_swap        \ -- ; compile code for SWAP
  2 min-depth(ws)  reset-opt
  nos-res tos-res set-nos set-tos  ;

: c_2swap       \ compile code for 2SWAP
  4 min-depth(ws)  reset-opt
  pop(ws) pop(ws) pop(ws) pop(ws)
  2swap
  push(ws) push(ws) push(ws) push(ws)  ;

: c_nip         \ -- ; compile code for NIP
  2 min-depth(ws)  reset-opt		\ SFP030
  pop(ws) drop(ws) push(ws)  ;

: c_tuck        \ -- ; compile code for TUCK
  2 min-depth(ws)  reset-opt		\ SFP030
  pop(ws) pop(ws)                       \ tos nos --
  over push(ws) push(ws) push(ws)       \ put back
  ?LoadTosLit				\ SFP054
;

: c_rot         \ -- ; compile code for ROT
  3 min-depth(ws)  reset-opt
  pop(ws) pop(ws) pop(ws) -rot push(ws) push(ws) push(ws)  ;

: c_-rot        \ -- ; compile code for -ROT
  3 min-depth(ws)  reset-opt
  pop(ws) pop(ws) pop(ws) rot push(ws) push(ws) push(ws)  ;

: c_2pick       \ -- ; compile code for 2 PICK
  3 min-depth(ws)  reset-opt			\ SFP030
  3os-res push(ws)  ;

: c_pick        \ -- ; generate code for PICK
  1 min-depth(ws)  reset-opt
  tos-res lit? if
    pop(ws) res-value @			\ remove literal from working stack	\ SFP030
    dup 0< opt_neg_pick ?throw		\ Negative input to PICK?
    dup in-range(ws) if                 \ required source on the working stack
      in(ws) @ push(ws) ?LoadTosLit	\ SFP054
      exit
    endif
    c_lit                               \ can't optimise, put literal back
  endif
  c_shuffle
  Thumb? if
    Cortex-M0/M1? if			\ SFP107
      a[ lsl .s  tos, tos, # 2 ]a use-a
      a[ ldr     tos, [ psp ++ tos ] ]a use-a
      exit
    endif
  endif
  a[ ldr tos, [ psp, tos .lsl # 2 ] ]a use-a
;

: c_roll        \ --
  target-sym ROLL compile-ref  ;


\ --------------------
\ LITPLUSxx optimisers
\ --------------------

\ SFP026...
: ?rem-litplus12	\ -- n res ; remove LIT + code if possible ; SFP028
  LITPLUSID opt? if
\ cr ." Got LITPLUSID12 x "
    seqflags @ #memOffLong? if
      seq-backup  seqflags @  seqres @	\ remove LIT + code, get n, res
    else
      0 tos-res
    endif
  else
    0 tos-res
  endif
;

: ?rem-litplus8	\ -- n res ; remove LIT + code if possible ; SFP028
  LITPLUSID opt? if
\ cr ." Got LITPLUSID8 x "
    seqflags @ #memOffShort? if
      seq-backup  seqflags @  seqres @	\ remove LIT + code, get n, res
    else
      0 tos-res
    endif
  else
    0 tos-res
  endif
;

: sz?rem-lit+ARM/M3	\ size -- n res ; remove LIT + code if possible
  WSIZE =
  if  ?rem-litplus8  else  ?rem-litplus12  endif
;

: sz?rem-lit+M0/M1	\ size -- n res ; remove LIT + code if possible
  LITPLUSID opt? if
    case
      LSIZE of  $80  endof
      WSIZE of  $40  endof
      BSIZE of  $20  endof
    endcase
    seqflags @ swap u< if
      seq-backup  seqflags @  seqres @	\ remove LIT + code, get n, res
    else
      0 tos-res
    endif
  else
    drop  0 tos-res
  endif
;

: sz?rem-litplus	\ size -- n res ; remove LIT + code if possible
  Thumb? if
    Cortex-M0/M1?
    if  sz?rem-lit+M0/M1  else  sz?rem-lit+ARM/M3  endif
  else
    sz?rem-lit+ARM/M3
  endif
;

: ?rem-lit+lit  \ reslit resreg -- n ; remove LIT + code if possible, and return offset ; SFP039
		\ N.B. modified from standard code in other optimisers
  LITPLUSLITID opt? if
\ cr ." Got LITPLUSLITID x"
\ cr ." New  Lit " over res-value @ 4 .r  ."  Reg " dup .res-name
\ cr ." Prev Lit " seqprev @        4 .r  ."  Src " seqres @ .res-name
    >r					\ save source regster ; SFP039
    res-value @ seqprev @ +		\ resulting literal may be
    dup #memOffLong?			\  an address offset
    swap #GenLit? and 			\  or a literal
    if
      r> drop				\ source already set ; SFP039
      seq-backup  seqprev @		\ remove LIT + code, get value
    else
      r> seqres !			\ define source ; SFP039
      0
    endif
\ cr ." Comb Lit Reg " seqres @  .res-name
  else
    seqres !				\ define source ; SFP039
    drop  0
  endif
;

: litplus-seq   \ reslit resreg -- ; start a LITPLUSID sequence ; SFP028
		\ N.B. modified from standard code in other optimisers
  over swap ?rem-lit+lit		\ deal with previous LIT + sequence ; SFP039
  over res-value +!
\ cr ." LITPLUSID "  dup res-value @ .dword
  LITPLUSID start-seq                   \ start sequence
  res-value @ seqflags !                \ save literal value
  start-bkup                            \ mark start of code
;

: nplus-seq	\ n srcres -- ; start a LITPLUSID sequence ; SFP033
		\ Used by USERCHILD
  LITPLUSID start-seq                   \ start sequence
  seqres !  seqflags !			\ save resource and lit value
  start-bkup                            \ mark start of code
;

: ?beg-reglitplus       { reslit src dest xt -- flag }
			\ start sequence if xt is of +, dest is reg, dest=src
			\ N.B. modified from standard code in other optimisers
  xt ['] + =
  dest reg? and
\  src dest = and
  reslit lit? and
  if  reslit src litplus-seq  true  else  reset-opt  false  endif
;

: ?end-reglitplus       \ flag -- ; end sequence if xt is of +
			\ N.B. modified from standard code in other optimisers
  if  end-seq  endif
;
\ ...SFP026


\ --------------------
\ Arithmetic operators
\ --------------------

: 3opins,       { rd r1 r2 opc -- }
\ lays a three operand instruction.
  opc execute  rd l>a  r1 l>a  r2 gl>a  use-a
;

: 2opins,	{ rd rs opc -- }
\ lays a two operand instruction.
  opc execute  rd l>a  rs gl>a  use-a
;

: 3Sopins,       { rd r1 r2 opc -- }
\ lays a three operand instruction with status set.
  opc execute a[ .s ]a  rd l>a  r1 l>a  r2 gl>a  use-a
;

: 2Sopins,	{ rd rs opc -- }
\ lays a two operand instruction with status set.
  opc execute a[ .s ]a  rd l>a  rs gl>a  use-a
;

\ old
: ComLoad2      \ -- r1 r2
\ Gets two operands of a commutative dyad into registers, returning
\ the resources r1 and r2. The second resource may be a 12 bit
\ literal for a shifter operand.
\ look for at least one of TOS and NOS already in a register
  tos-res reg? if			\ TOS already in register
    tos-res
    nos-res reg?  nos-res resGenLit? or if
      nos-res
    else
      nos-res rR1 sdmove,  rR1
    endif
    exit
  endif

  nos-res reg? if			\ NOS already in register
    nos-res
    tos-res reg?  tos-res resGenLit? or if
      tos-res
    else
      tos-res rR0 sdmove,  rR0
    endif
    exit
  endif

\ Neither TOS nor NOS in a register, look for literals
  tos-res resGenLit? if
    nos-res rR1 sdmove,
    rR1 tos-res
    exit
  endif
  nos-res resGenLit? if
    tos-res rR0 sdmove,
    rR0 nos-res
    exit
  endif

\ Just do it the hard way
  tos-res rR0 sdmove,
  nos-res rR1 sdmove,
  rR0 rR1
;

: tos/R0	\ -- reg
\ If TOS is not a register, copy it to R0. Return the register.
  tos-res reg? if
    tos-res
  else
    tos-res rR0 sdmove,  rR0
  endif
;

: nos/R1	\ -- reg
\ If NOS is not a register, copy it to R1. Return the register.
  nos-res reg? if
    nos-res
  else
    nos-res rR1 sdmove,  rR1
  endif
;

: ComLoad2Reg	\ -- r1 r2 ; SFP053
\ Gets two operands of a commutative dyad into registers.
\ Literals are loaded into registers.
  tos/R0 nos/R1
;

: FindDest2     \ -- res ; gets a suitable dreg for a two operand instruction	SFP027
  tos-res dreg?  tos-res rTOS = and
  if  tos-res  exit  endif

  nos-res dreg?  nos-res rTOS = and
  if  nos-res  exit  endif

  rTOS free?
  if  rTOS  exit  endif

  tos-res dreg?
  if  tos-res  exit  endif

  nos-res dreg?
  if  nos-res  exit  endif

  GetNewReg ?dup 0= if
    c_shuffle  2 min-depth(ws)		\ SFP030, SFP065
    defnewNOS
  endif
;

: +FlagOpt	\ -- ; flag set optimisation
  FLAG1SETID [opt opt]  ;

: dyadExit	\ res --
\ Clean up from a dyad, leaving res as the new TOS.
  drop(ws) drop(ws)  push(ws)  ;

: dyadExitFlags	\ res
\ Clean up from a dyad, leaving res as the new TOS.
\ Flags are marked as set.
  dyadExit +FlagOpt  ;

\ litplus-seq   \ reslit resreg --

: add2opM0/M1	{ dest src -- }
  a[ add .s ]a  dest l>a  src gl>a  use-a
;

: add3opM0/M1	{ rd rn rmImm3 -- }
  a[ add .s ]a  rd l>a  rn l>a  rmImm3 gl>a  use-a
;

: plusM0/M1	{ dest src -- }
\ Generate code for + for M0/M1. Dest is always a reg.
  src reg? if
    dest src add2opM0/M1  dest dyadExitFlags  exit
  endif
  src lit? if
    src res-value @ #256 u< if
      src dest litplus-seq
      dest src add2opM0/M1
      end-seq
      drop(ws) drop(ws)  dest push(ws)
      exit
    endif
  endif
  src rR0 sdmove,
  dest rR0 add2opM0/M1
  dest dyadExitFlags
;

: comDyadARM	{ opc xt -- }
\ Code generate ARM/CM3 code for a commutative dyad (a OP b = b OP a)
  FindDest2 dup >r  ComLoad2		\ produce dest, src, Rb/lit
  2dup swap r@ xt ?beg-reglitplus >r   	\ start sequence if dest is reg and xt is of +	SFP026
  r@					\ if this is a reg+lit sequence ; SFP052
  if  nip  seqres @ swap  endif		\ replace src with that for LIT+ sequence ; SFP052
  opc 3opins,
  r> ?end-reglitplus			\ end sequence if defined ; SFP026
  xt ['] + <>				\ check for LIT+ sequence
  if  FLAG1SETID [opt opt]  endif	\ SFP062
  drop(ws) drop(ws)   r> push(ws)       \ update working stack
;

: glit+M0/M1	{ dest src resimm -- }
\ Dest is a dreg. src is NOT a dreg.
  resimm res-value @ 8 u< if
    src reg? if
      resimm src litplus-seq
      dest src resimm add3opM0/M1
      end-seq  exit
    endif
  endif
  resimm res-value @ #256 u< if
    src reg? if
      resimm src litplus-seq
      src dest sdmove,
    else
      src dest sdmove,
      resimm dest litplus-seq
    endif
    dest resimm add2opM0/M1
    end-seq  exit
  endif
  dest					\ dest
  src reg?
  if  src  else  src rR0 sdmove, rR0  endif  \ src/r1
  resimm rR1 sdmove,  rR1		\ imm
  add3opM0/M1  +FlagOpt
;

: comDyad+M0/M1	{ opc xt -- }
\ Code generate + for M0/M1
  tos-res dreg? if
    tos-res nos-res plusM0/M1  exit
  endif
  nos-res dreg? if
    nos-res tos-res plusM0/M1
    exit
  endif
\ Neither is a dreg. If one is a reg, reserve that as the source.
  GetNewReg ?dup 0= if
    c_shuffle  2 min-depth(ws) opc xt recurse  exit
  endif
  >r
\ check for three regs
  nos-res reg? tos-res reg? and if
    r@ tos-res nos-res add3opM0/M1
    r> dyadExitFlags  exit
  endif
\ check for 3 and 8 bit immediates
  tos-res lit? if
    r@ nos-res tos-res glit+M0/M1  r> dyadExit  exit
  endif
  nos-res lit? if
    r@ tos-res nos-res glit+M0/M1  r> dyadExit  exit
  endif
\ do it the hard way
  r@ tos/R0 nos/R1  add3opM0/M1
  r> dyadExitFlags
;

: comDyadRegM0/M1	{ opc xt -- }
\ Code generate register operations (not +) for M0/M1 code
\ cr ." comDyadRegM0/M1 "
  tos-res dreg? if
    tos-res nos/R1 opc 2opins,
    tos-res dyadExitFlags  exit
  endif
  nos-res dreg? if
    nos-res tos/R0 opc 2opins,
    nos-res dyadExitFlags exit
  endif
\ Neither is a dreg. If one is a reg, reserve that as the source.
  GetNewReg ?dup 0= if
    c_shuffle  2 min-depth(ws) opc xt recurse  exit
  endif
  >r
  tos-res reg? if
    nos-res r@ sdmove,
    r@ tos-res opc 2opins,
    r> dyadExitFlags  exit
  endif
  nos-res reg? if
    tos-res r@ sdmove,
    r@ nos-res opc 2opins,
    r> dyadExitFlags  exit
  endif
  tos-res r@ sdmove,
  nos-res rR1 sdmove,
  r@ rR1 opc 2opins,
  r> dyadExitFlags  exit
;

: (comdyad)     { opc xt -- ; code generates for a commutative dyad (a OP b = b OP a) }
                \ use of this factor permits recursion when GetRegRes fails
  2 min-depth(ws)			\ SFP030

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if     \ can do everything at compile time
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    tos-res res-value @  nos-res res-value @
    xt execute                          \ perform compile time function
    nos-res res-value !
    drop(ws)  reset-opt  exit 		\ SFP026
  endif

\ Now deal with everything else
  M0/M1? if
    xt ['] + = if
      opc xt comDyad+M0/M1
    else
      opc xt comDyadRegM0/M1
    endif
  else
    opc xt comDyadARM
  endif
;

: comdyad       \ -- ; -- ; COMDYAD C_+ + "ADD"
  create
    ' ,                                 \ get XT of compile time action
    here  0 ,  lay-asm: drop            \ for xt of opcode action
  does>
    dup cell+ @  swap @                 \ -- opc xt
    (comdyad)                           \ use common code
;

: +c  +  ;

comdyad (c_+)   +    "ADD .S "
comdyad c_+cl	+    "ADD .S "
comdyad (c_and) and  "AND .S "
comdyad c_or    or   "ORR .S "
comdyad c_xor   xor  "EOR .S "
comdyad c_+ch	+c   "ADC .S "

\ Some instances of ADD and AND where tos-res is a -ve literal that
\ doesn't fit into the 12bit format, can be replaced with the SUB or
\ BIC instructions. In this case ABS TOS-RES or NOT TOS-RES is used
\ as the literal (if it fits into the 12bit format). Avoids the
\ need for a MOV/MVL instruction.

: c_+		\ --
\ Deal with special cases before general
  2 min-depth(ws)
  tos-res lit? if
    nos-res lit? 0= if			\ NOS not a lit
      tos-res res-value @ 0< if		\ TOS -ve
        tos-res absResGenLit? if		\ TOS -ve, abs is 12 bit,
          tos-res resGenLit? 0= if	\     but -ve is not
            reset-opt	\ SFP030
            FindDest2 dup >r		\ save Rd
            nos-res reg? if
              nos-res
            else
              nos-res rR0 sdmove,  rR0
            endif
            swap
            a[ SUB ]a  l>a  l>a  tos-res res-value @ abs #>a n>a  use-a
            drop(ws) drop(ws) r> push(ws)	\ update working stack
            exit
          endif
        endif
      endif
    endif
  endif
  (c_+)
;

\ SFP080...

: (c_biclit)	\ --
\ Generates a BIC Rd, Rn, # lit12 instruction.
  2 min-depth(ws)  reset-opt		\ SFP030
  FindDest2 dup >r			\ save Rd
  nos-res reg? if
    nos-res
  else
    nos-res rR0 sdmove,  rR0
  endif
  \ -- rd rnos
  M0/M1? if
    2dup <> if
      >r dup >r
      a[ mov ]a  r> l>a  r> l>a  use-a
    else
      drop
    endif
    \ -- rd
    >r
    a[ mov .s ]a  rR1 l>a  a[ # ]a tos-res res-value @ invert  use-a
    a[ bic .s ]a  r> l>a  rR1 l>a  use-a
  else
    swap
    a[ BIC ]a  l>a  l>a  tos-res res-value @ invert #>a n>a  use-a
  endif
  drop(ws) drop(ws)  r> push(ws)	\ update working stack
;

: invLit?	\ res -- flag
\ RES is a literal resource. Return true if it will not fit into
\ IMM12 format, but its complement (inverse) will.
  dup res-value @ 0< if
    dup res-value @ invert #genLit? if
      #genLit? 0=  exit
    endif
   endif
  drop 0
;

: c_and		\ --
  2 min-depth(ws)
  nos-res lit? tos-res lit? and 0= if
    nos-res lit? if
      nos-res tos-res set-nos set-tos	\ swap TOS and NOS
    endif
    tos-res lit? if
      tos-res invLit? if
        (c_biclit)  exit
      endif
    endif
  endif
  (c_and)
;


: NonComLoad2   \ -- nos tos ; gets two operands of a nomcommutative dyad into regs
  nos/R1
  tos-res reg?  tos-res resGenLit? or if
    tos-res
  else
    tos-res rR0 sdmove,  rR0
  endif
;

: subM0/M1	{ dest opc xt -- }
  dest  nos/r1  tos-res lit? if		\ -- dest src1
    tos-res res-value @ 8 u< if
      tos-res opc 3opins,
    else				\ -- dest src1
      tos-res res-value @ #256 u< if
        2dup <>			\ get src1 into dest
        if  over sdmove,  else  drop  endif
        tos-res opc 2opins,
      else
        tos/r0 opc 3opins,
      endif
    endif
  else				\ -- dest src1
    tos/r0 opc 3opins,
  endif
;

: (noncomdyad)  { opc xt -- ; code generates for a noncommutative dyad (a OP b != b OP a) }
                \ use of this factor permits recursion when GetRegRes fails
  2 min-depth(ws)  reset-opt		\ SFP030

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if     \ can do everything at compile time
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    nos-res res-value @  tos-res res-value @
    xt execute                          \ perform compile time function
    nos-res res-value !
    drop(ws)  exit
  endif

  FindDest2 dup >r                      \ save Rd
  Not-M0/M1? if				\ -- dest
    NonComLoad2				\ -- dest src1 sr2/lit
    opc 3opins,
  else
    opc xt subM0/M1
  endif
  drop(ws) drop(ws)   r> push(ws)       \ update working stack
;

: noncomdyad    \ -- ; -- ; COMDYAD C_- - "SUB"
  create
    ' ,                                 \ get XT of compile time action
    here  0 ,  lay-asm: drop            \ for xt of opcode action
  does>
    dup cell+ @  swap @                 \ -- opc xt
    (noncomdyad)                        \ use common code
;

noncomdyad (c_-)     -     "SUB .s "	\ SFP054
noncomdyad c_-cl     -     "SUB .S "
noncomdyad c_-ch     -     "SBC .s "
noncomdyad c_swap-   swap- "RSB .s "	\ SFP054
\ noncomdyad c_swap-cl swap- "RSB .S "	\ SFP054
\ noncomdyad c_swap-ch swap- "RSC "	\ SFP054

: c_-		\ -- ; SFP054
  Not-M0/M1? if
    nos-res lit?
    if  c_swap c_swap-  else  (c_-)  endif
  else
    (c_-)
  endif
;

: Load1		\ -- reg ; gets operands of a monad into reg
  tos-res reg?				\ TOS already in register
  if  tos-res  exit  endif
  tos-res rR0 sdmove,  rR0
;

: FindDest1     \ -- res ; gets a suitable dreg for a one operand instruction	SFP027
  rTOS free?
  if  rTOS  exit  endif

  tos-res dreg?
  if  tos-res  exit  endif

  GetNewReg ?dup 0=
  if  c_shuffle  rTOS  endif
;

: (monad)       { xt opc arg -- ; generate code for a monadic instruction OPC tos, tos, arg }
  1 min-depth(ws)  reset-opt

  tos-res lit? if			\ do it at compile time?
    ?NewTOSLit                          \ ensure TOS is free for modification ; SFP017
    tos-res res-value  dup @  xt execute  swap !
    exit
  endif

  FindDest1 dup  Load1 >r  >r		\ form dest and source registers
  opc execute  r> l>a  r> l>a  arg execute  use-a	\ OP tos, tos, arg
  drop(ws)  push(ws)			\ set dest as TOS
;

: monad         \ -- ; -- ; generate code for a monadic instruction OPC tos, arg
  create
    ' ,                                 \ xt for compile time action
    here  0 ,  0 ,			\ instruction, argument
    lay-asm: lay-asm: drop              \ get required actions
  does>
    dup @  swap cell+ dup @  swap cell+ @       \ -- xt opc arg
    (monad)
;

monad c_1-      1-     "SUB .s "  " # 1"
monad c_2-      2-     "SUB .s "  " # 2"
monad c_4-      4-     "SUB .s "  " # 4"
monad c_2*      2*     "MOV .s "  " .LSL # 1"
monad c_2/      2/     "MOV .s "  " .ASR # 1"
monad c_u2/     u2/    "MOV .s "  " .LSR # 1"
monad c_negate  negate "RSB .s "  " # 0"
monad c_invert  not    "MVN .s "  " "
monad c_not     not    "MVN .s "  " "
monad c_4*      4*     "MOV .s "  " .LSL # 2"
monad c_4/      4/     "MOV .s "  " .ASR # 2"
monad c_u4/     u4/    "MOV .s "  " .LSR # 2"

: c_1+		\ -- ; SFP026
  1 c_lit  c_+  ;
: c_2+		\ -- ; SFP026
  2 c_lit  c_+  ;
: c_4+		\ -- ; SFP026
  4 c_lit  c_+  ;


\ ---------------------------
\ Logical and shift operators   SFP027
\ ---------------------------

: amrk>         \ -- sbrid ; mark the start (opcode address) of a forward branch
  chere(t)  ;

: >ares         \ sbrid --
  fix_ATbranch
;

: genShiftARM	{ xt opc | srd srnos srtos -- }
\ Generate shift code for ARM and CM3
  FindDest2 NonComLoad2			\ -- rd nos tos
  -> srtos  -> srnos  -> srd
\ SFP058...
  srtos lit?  xt ['] lshift = or if	\ LIT or LSHIFT needs no protection
    a[ MOV .s ]a  srd l>a  srnos l>a  opc execute  srtos gl>a  use-a
  else
    Thumb? if
      a[ CMP ]a  srtos l>a  a[ # 0 ]a use-a  \ check for shift=0
      srd srnos <> if
        a[ ITE .ne
           MOV   srd l>a  srnos l>a  opc execute  srtos gl>a
           MOV   srd l>a  srnos l>a
        ]a use-a
      else
        a[ IT .ne
           MOV  srd l>a  srnos l>a  opc execute  srtos gl>a
        ]a use-a
      endif
    else
      a[ CMP ]a  srtos l>a  a[ # 0 ]a use-a  \ check for shift=0
      a[ MOV .NE ]a  srd l>a  srnos l>a  opc execute  srtos gl>a  use-a
      srd srnos <> if
        a[ MOV .EQ ]a  srd l>a  srnos l>a  use-a
      endif
    endif
  endif
\ ...SFP058
  drop(ws) drop(ws)   srd push(ws)       \ update working stack
;

: dregShift	\ -- srd
\ Get NOS into a dreg
  nos-res dreg? if
    nos-res
  else
    GetNewReg ?dup 0= if
      c_shuffle 2 min-depth(ws) GetnewReg
    endif
  endif
;

: shiftLitM0	{ opc | dest srnos -- srd }
\ CM0/1 shift using a literal
  dregShift -> dest  nos/R1 -> srnos
  dest srnos tos-res opc 3Sopins,
  dest
;

: shiftOtherM0	{ opc | dest -- srd }
\ CM0/1 shift not using a literal
  dregShift dup -> dest  nos-res <>
  if  nos-res dest sdmove,  endif
  dest tos/R0 opc 2Sopins,
  dest
;

: shiftM0	\ opc -- srd
\ M0 shift. If TOS is a lit, it is not zero.
  tos-res lit?
  if  shiftLitM0  else  shiftOtherM0  endif
;

: genShiftM0M1	{ xt opc -- }
\ Generate shift code for M0 and M1.
\ If TOS is a lit, it is not zero.
  xt ['] ror = if
    opc shiftOtherM0
  else
    opc shiftM0
  endif
  drop(ws) drop(ws)  push(ws)		\ update working stack
;

: (shifter)     { xt opcA opcT -- }	\ SFP066
\ generate code for given shift instruction
  2 min-depth(ws)  reset-opt		\ SFP030

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if     \ can do everything at compile time
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    nos-res res-value @  tos-res res-value @
    xt execute                          \ perform compile time function
    nos-res res-value !
    drop(ws)  exit
  endif

\ check range of literal
  tos-res lit? if
    tos-res res-value @
    dup 0= if
      cr ." Urk! Literal shift by 0, *** ignored ***"
      .source-line
      drop  drop(ws)  exit
    endif
    dup #31 u> if
      cr ." Urk! Literal shift > 31, *** replaced by 32 ***"
      .source-line
      drop #32
    endif
    tos-res res-value !
  endif

  Thumb? if
    Cortex-M0/M1? if
      xt opcA genShiftM0M1		\ M0/M1
    else
      xt opcT genshiftARM		\ M3+
    endif
  else
    xt opcA genShiftARM			\ ARM
  endif
;

: shifter       \ -- ; -- ; generate code for a shift instruction OPC nos, tos
  create
    ' ,                                 \ xt for compile time action
    here  0 , 0 ,			\ instruction slots
    lay-asm: lay-asm: drop		\ ARM then T2
  does>
    dup @  swap cell+ dup @  swap cell+ @	\ -- xt opcA opcT
    (shifter)
;

\                 interp    ARM     Cortex-M3
shifter c_lshift  lshift    "LSL "  ".LSL "
shifter c_rshift  rshift    "LSR "  ".LSR "
shifter c_arshift arshift   "ASR "  ".ASR "
shifter c_ror	  ror       "ROR "  ".ROR "	\ SFP044

: c_rol		\ -- ; = shift right by 32-n	SFP044
  #32 c_lit  c_swap  c_-  c_ror
;


\ --------------
\ Multiplication
\ --------------

: RegLoad2      \ -- nos tos ; gets two operands of a dyad into regs
  nos-res reg? if			\ NOS already in register
    nos-res
  else
    nos-res rR1 sdmove,  rR1
  endif
  tos-res reg? if			\ TOS already in register
    tos-res
  else
    tos-res rR0 sdmove,  rR0
  endif
;

: prim*T2	{ rd rn rt -- }		\ SFP107
  Cortex-M0/M1? if
    rd rt = if
      a[ MUL .s ]a  rd l>a  rt l>a  rn l>a  use-a  exit
    endif
    rd rn = if
      a[ MUL .s ]a  rd l>a  rn l>a  rt l>a  use-a  exit
    endif
    rt rd sdmove,
    a[ MUL .s ]a  rd l>a  rd l>a  rn l>a  use-a
  else
    a[ MUL ]a  rd l>a  rn l>a  rt l>a  use-a
  endif
;

: prim*ARM	{ rd rn rt -- }
  rd rn <> if
    a[ MUL ]a  rd l>a  rn l>a  rt l>a  use-a	  \ MUL rd, rnos, rtos
  else
    rd rt <> if
      a[ MUL ]a  rd l>a  rt l>a  rn l>a  use-a	  \ MUL rd, rtos, rnos
    else
      rn rR0 <> if
        rt rR0 sdmove,				  \ MOV r0, rtos
        a[ MUL ]a  rd l>a  rR0 l>a  rn l>a  use-a \ MUL rd, r0, rnos
      else
        rn rR1 = abort" Out of registers in MUL!"
        rt rR1 sdmove,				  \ MOV r1, rtos
        a[ MUL ]a  rd l>a  rR1 l>a  rn l>a  use-a \ MUL rd, r1, rnos
      endif
    endif
  endif
;

: c_prim_*	{ | rd rn rt -- ; generate multiply primitive }
  FindDest2 -> rd
  RegLoad2 -> rt  -> rn
  rd rn rt Thumb?
  if  prim*T2  else  prim*ARM  endif
  drop(ws) drop(ws)   rd push(ws)	\ update working stack
;

: MulByShiftAdd	\ n -- ; ADD tos, tos, tos lsl # n
  >r
  FindDest1 dup  Load1 dup >r >r >r	\ -- dest ; R: -- n tos tos dest
  a[ ADD ]a  r> l>a  r> l>a  r> l>a a[ .LSL # ]a r> n>a  use-a
  drop(ws)  push(ws)
;

: MulLitARM/M3	\ -- ; multiply by a lteral for ARM/M3
    case  pop(ws) res-value @		\ SFP030
         0 of  drop(ws)  0 c_lit  endof	\ result is literal 0	\ SFP030
         1 of  noop  endof		\ result is nos*1=nos
         2 of  c_2*  endof
	 3 of  1 MulByShiftAdd    endof
         4 of  2 c_lit  c_lshift  endof
	 5 of  2 MulByShiftAdd    endof
         8 of  3 c_lit  c_lshift  endof
	 9 of  3 MulByShiftAdd    endof
       #16 of  4 c_lit  c_lshift  endof
       #32 of  5 c_lit  c_lshift  endof
       #64 of  6 c_lit  c_lshift  endof
      #128 of  7 c_lit  c_lshift  endof
      #256 of  8 c_lit  c_lshift  endof
    $10000 of  #16 c_lit  c_lshift  endof
  $1000000 of  #24 c_lit  c_lshift  endof
               c_lit  c_prim_*
    end-case
;

: MulLitM0/M1	\ -- ; multiply by a lteral for M0/M1
    case  pop(ws) res-value @		\ SFP030
         0 of  drop(ws)  0 c_lit  endof	\ result is literal 0	\ SFP030
         1 of  noop  endof		\ result is nos*1=nos
         2 of  c_2*  endof
         4 of  2 c_lit  c_lshift  endof
         8 of  3 c_lit  c_lshift  endof
       #16 of  4 c_lit  c_lshift  endof
       #32 of  5 c_lit  c_lshift  endof
       #64 of  6 c_lit  c_lshift  endof
      #128 of  7 c_lit  c_lshift  endof
      #256 of  8 c_lit  c_lshift  endof
    $10000 of  #16 c_lit  c_lshift  endof
  $1000000 of  #24 c_lit  c_lshift  endof
               c_lit  c_prim_*
    end-case
;

: c_*           \ -- ; generate code for * ; SFP027
  2 min-depth(ws)  reset-opt

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    tos-res res-value @  nos-res res-value @
    * nos-res res-value !
    drop(ws)  exit			\ SFP030
  endif

\ process multiplication by literal
  nos-res lit?                          \ put single literal in TOS
  if  c_swap  endif			\ as multiplication is commutative

  tos-res lit? if
    M0/M1?
    if  MulLitM0/M1  else  MulLitARM/M3  endif
    exit
  endif

\ if all else fails
  c_prim_*
;

\ SFP035...
: (mul64s)	{ xt opc -- }
  2 min-depth(ws)  reset-opt
  tos-res lit?  nos-res lit? and if
    pop(ws) res-value @
    pop(ws) res-value @
    xt execute
    swap c_lit c_lit  exit
  endif

  ComLoad2Reg  drop(ws) drop(ws) >r >r	\ -- rs1 rs2 ; load inputs into registers ; SFP053
  ForceNewTos+NOS			\ assign output registers
  opc execute  nos-res l>a  tos-res l>a  r> l>a  r> l>a  use-a
;

: mul64s	\ -- ; MUL64S <name> "instruction"
  create
    ' ,					\ xt
    here  0 ,  lay-asm: drop            \ opc
  does>
    dup @ swap cell+ @ (mul64s)
;

mul64s (c_um*) um* "UMULL "
mul64s (c_m*)  m*  "SMULL "

: c_um*		\ --
  M0/M1?
  if  target-sym UM* compile-ref  else  (c_um*)  endif
;

: c_m*		\ --
  M0/M1?
  if  target-sym M* compile-ref  else  (c_m*)  endif
;
\ ...SFP035


\ --------
\ Division
\ --------

: shuffleTOS+1	\ -- tos scr
\ Shuffle, return TOS and a scratch register
  c_shuffle tos-res GetNewReg
;

: GetTOS	\ -- tos
\ Get TOS into a dreg.
  tos-res dreg?
  if  tos-res exit  endif
  GetNewReg dup
  if  dup CopyTOS exit  endif
  drop  c_shuffle tos-res
;

: GetTOS+1	\ -- tos reg
\ We need TOS in a dreg and a scratch register.
  tos-res dreg? if
    GetNewReg dup if
      tos-res swap
    else
      drop shuffleTOS+1
    endif
  else
    GetNewReg dup if
      dup -GetNewReg
      dup if
        over CopyTOS
      else
        2drop shuffleTOS+1
      endif
    else
      drop shuffleTOS+1
    endif
  endif
;

: c_div2^n	\ 1..31 -- ; SFP073
\ Generate a signed symmetric divide by 2^n. The steps are:
\   generate an n bit mask from the top bit of the dividend
\   shift the mask to the bottom bits
\   add the mask to the dividend
\   arithmetic shift right
\     1  x.......   .......x
\     2  xx......   ......xx
\     3  xxx.....   .....xxx
\ We have to do it this way because a simple ARSHIFT is equivalent
\ to floored division, and we need symmetric division.
  { u | rtos rtemp -- }
  u 1 = if
    GetTOS -> rtos
    a[ add ]a  rtos l>a  rtos l>a  rtos l>a  a[ .lsr # #31 ]a use-a
    a[ mov ]a  rtos l>a  rtos l>a a[ .asr # 1 ]a use-a
    exit
  endif

  GetTOS+1 -> rtemp -> rtos		\ TOS in dreg and a scratch reg
  a[ mov ]a  rtemp l>a  rtos l>a a[ .asr # ]a u 1- n>a  use-a
  a[ add ]a  rtos l>a  rtos l>a  rtemp l>a  a[ .lsr # ]a #32 u -  use-a
  a[ mov ]a  rtos l>a  rtos l>a a[ .asr # ]a u n>a  use-a
;

\ SFP104...
: (NcDyadReg)	{ opc xt -- ; code generates for a noncommutative dyad (a OP b != b OP a) }
                \ use of this factor permits recursion when GetRegRes fails
  2 min-depth(ws)  reset-opt		\ SFP030

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if     \ can do everything at compile time
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    nos-res res-value @  tos-res res-value @
    xt execute                          \ perform compile time function
    nos-res res-value !
    drop(ws)  exit
  endif

  FindDest2 dup >r                      \ save Rd
  RegLoad2
  opc 3opins,
  drop(ws) drop(ws)   r> push(ws)       \ update working stack
;

: NcDyadReg	\ -- ; -- ; NCDYADREG c_/T2 / " SDIV "
  create
    ' ,                                 \ get XT of compile time action
    here  0 ,  lay-asm: drop            \ for xt of opcode action
  does>
    dup cell+ @  swap @                 \ -- opc xt
    (NcDyadReg)				\ use common code
;

NcDyadReg c_/T2   /  " SDIV "
NcDyadReg c_u/T2  u/  " UDIV "
\ ...SFP104

: c_prim_/	\ -- ; lay division primitive ; SFP040
  Thumb?
  if  c_/T2  else  target-sym / compile-ref  endif
;

: c_/		\ -- ; generate code for / ; SFP040
  2 min-depth(ws)  reset-opt

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    nos-res res-value @  tos-res res-value @
    / nos-res res-value !
    drop(ws)  exit
  endif

  Thumb? if				\ SFP107
    Cortex-M0/M1? if
      target-sym / compile-ref  exit
    endif
  endif

\ process division by literal
  tos-res lit? if
\ SFP073...
    case  pop(ws) res-value @		\ SFP030
         0 of  true abort" Urk! Division by literal zero"  endof
         1 of  noop  endof		\ result is nos/1=nos
         2 of  1 c_div2^n  endof
         4 of  2 c_div2^n  endof
         8 of  3 c_div2^n  endof
       #16 of  4 c_div2^n  endof
       #32 of  5 c_div2^n  endof
       #64 of  6 c_div2^n  endof
      #128 of  7 c_div2^n  endof
      #256 of  8 c_div2^n  endof
      #512 of  9 c_div2^n  endof
      $400 of  #10 c_div2^n  endof
      $800 of  #11 c_div2^n  endof
     $1000 of  #12 c_div2^n  endof
     $2000 of  #13 c_div2^n  endof
     $4000 of  #14 c_div2^n  endof
     $8000 of  #15 c_div2^n  endof
    $10000 of  #16 c_div2^n  endof
  $1000000 of  #24 c_div2^n  endof
               c_lit  c_prim_/
    end-case
\ ...SFP073
    exit
  endif

\ if all else fails
  c_prim_/
;

: c_u/		\ -- ; code generator for U/
  2 min-depth(ws)  reset-opt

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    nos-res res-value @  tos-res res-value @
    u/ nos-res res-value !
    drop(ws)  exit
  endif

  Thumb? if				\ SFP107
    Cortex-M0/M1? 0= if
      c_u/T2  exit
    endif
  endif

  target-sym U/ compile-ref
;


\ ----------------
\ Memory operators
\ ----------------

: tosPCrel?	{ maxoff -- flag }	\ SFP107
\ TOS is a literal address. Return true if it can be reached
\ by a PC relative load.
  Not-M0/M1? if
    tos-res res-value @ chere(t) 8 + -	\ offset
    abs maxoff u> not			\ Does TOS fit relxx format? SFP024, SFP047
  else
    tos-res res-value @ dup 3 and if
      drop 0
    else
      chere(t) 8 + - dup 0< if
        drop 0
      else
        $3FC u<
      endif
    endif
  endif
;

: (fetchop)     { size maxoff opc -- ; compile code for fetch operations }
  1 min-depth(ws)

  tos-res lit? if                                       \ TOS is a lit?
    GetNewReg ?dup 0=                                   \   Get new tos reg
    if  c_shuffle  size maxoff opc recurse  exit  endif	\ SFP047
    >r  maxoff tosPCrel? if
      opc execute  r@ l>a  pop(ws) r#>a ( @lit ) use-a  \     LDR newtos, oldtos value  \ SFP024
    else                                                \   NO
      pop(ws) rR0 sdmove,                               \     MOV r0, # oldtos value
      opc execute  r@ l>a  rR0 +0[reg]  use-a		\     LDR newtos, [ r0 ]
    endif                                               \   ENDIF
    r> push(ws)  reset-opt  exit			\   push newtos and exit	\ SFP026
  endif                                                 \ ENDIF

  tos-res dreg? if                                      \ TOS is a dreg?
    size sz?rem-litplus >r >r				\   ?remove lit +		\ SFP026
    opc execute  tos-res l>a  r> r> +n[reg]  use-a 	\   LDR tos, [ tos + n ]	\ SFP026
    reset-opt  exit					\				\ SFP026
  endif                                                 \ ENDIF

  reset-opt						\ SFP026

  GetNewReg ?dup 0=                                     \ New resource required.
  if  c_shuffle  size maxoff opc recurse  exit  endif	\ SFP047

  tos-res reg? if                                       \ TOS is a reg?
    >r
    opc execute  r@ l>a  pop(ws) +0[reg]  use-a         \   LDR newtos, [ oldtos ]
    r> push(ws)                                         \   push newtos
  else                                                  \ NO - deep
    CopyTOS                                             \   LDR newtos, [ psp # n] (oldtos)
    opc execute  tos-res l>a  tos-res +0[reg]  use-a    \   LDR newtos, [ newtos ]
  endif                                                 \ ENDIF
;

\ SFP113...
: [reg++r1]	\ res -- ; lay [ reg ++ r1 ]
  >r  a[ [ ]a  r> l>a  a[ ++ r1 ] ]a
;

: fetchop-M0/M1	{ size maxoff opc -- }
\ compile code for fetch operations
  1 min-depth(ws)

  tos-res lit? if                                       \ TOS is a lit?
    GetNewReg ?dup 0=                                   \   Get new tos reg
    if  c_shuffle  size maxoff opc recurse  exit  endif	\ SFP047
    >r
    pop(ws) rR0 sdmove,                                 \     MOV r0, # oldtos value
    a[ mov .s  r1, # 0 ]a use-a
    opc execute  r@ l>a  a[ [ r0 ++ r1 ] ]a use-a	\     LDRSx newtos, [ r0 ++ r1 ]
    r> push(ws)  reset-opt  exit			\   push newtos and exit	\ SFP026
  endif                                                 \ ENDIF

  tos-res dreg? if                                      \ TOS is a dreg?
    a[ mov .s  r1, # 0 ]a use-a
    opc execute  tos-res l>a  tos-res [reg++r1]  use-a 	\   LDRSx tos, [ tos ++ r1 ]	\ SFP026
    reset-opt  exit					\				\ SFP026
  endif                                                 \ ENDIF

  reset-opt						\ SFP026

  GetNewReg ?dup 0=                                     \ New resource required.
  if  c_shuffle  size maxoff opc recurse  exit  endif	\ SFP047

  tos-res reg? if                                       \ TOS is a reg?
    >r
    a[ mov .s  r1, # 0 ]a use-a
    opc execute  r@ l>a  pop(ws) [reg++r1]  use-a	\   LDRSx newtos, [ oldtos ++ r1]
    r> push(ws)                                         \   push newtos
  else                                                  \ NO - deep
    CopyTOS                                             \   LDR newtos, [ psp # n] (oldtos)
    a[ mov .s  r1, # 0 ]a use-a
    opc execute  tos-res l>a  tos-res [reg++r1]  use-a	\   LDR newtos, [ newtos ++ r1 ]
  endif                                                 \ ENDIF
;

: fetchop       \ max_offset size -- ; -- ; max_offset xsize FETCHOP <name> "instruction"
  create
    , ,					\ size, max_offset ; SFP047
    here  0 ,				\ will be opc
    swap ,				\ true if special for Cortex-M0/M1
    lay-asm: drop
  does>
    dup @  swap cell+ dup @  swap cell+ dup @	\ size maxoff addr opc --
    Not-M0/M1? if
      nip (fetchop)
    else
      swap cell+ @
      if  fetchop-M0/M1  else  (fetchop)  endif
    endif
;

0 $0FFF LSIZE fetchop c_@      "LDR "
0 $00FF WSIZE fetchop c_w@     "LDRH "
0 $0FFF BSIZE fetchop c_c@     "LDRB "
1 $00FF WSIZE fetchop c_w@s    "LDRSH "
1 $00FF BSIZE fetchop c_c@s    "LDRSB "
\ ...SFP113

: (storeop)     { size maxoff opc -- ; compile code for store operations }
  2 min-depth(ws)					\ SFP026 SFP030

  tos-res dreg?						\ Case where TOS is a dreg
  if  size sz?rem-litplus  else  0 tos-res  endif	\ save offset

  nos-res reg? if                                       \ NOS is a reg?
    nos-res                                             \   use it as value
  else                                                  \ NO
    nos-res rR0 sdmove,  rR0				\   move deep or lit into rR0
    reset-opt						\ SFP026
  endif                                                 \ ENDIF
\ -- n areg dreg

  tos-res dreg? if                                      \ Case where TOS is a dreg
    -rot >r >r >r
    opc execute  r> l>a  r> r> +n[reg]  use-a		\   STR nos [ tos + n ]		SFP026
    drop(ws) drop(ws)  reset-opt  exit			\ SFP026
  endif

  nip nip  >r						\ stash source register

  tos-res reg? if                                       \ Case where TOS is a reg
    opc execute  r> l>a  pop(ws) +0[reg]  use-a         \   STR nos [ tos ]
    drop(ws)  reset-opt  exit				\ SFP026
  endif

  tos-res deep? if                                      \ Case where TOS is deep
    pop(ws) rR1 sdmove,                                 \   LDR r1, [ tos ]
    opc execute  r> l>a  rR1 +0[reg]  use-a             \   STR nos, [ r1 ]
    drop(ws)  reset-opt  exit				\ SFP026
  endif

\ SFP104...
  Thumb? 0= if				\ Cortex has no literal form for STRx
    tos-res lit? if
      tos-res res-value @ chere(t) 8 + -
      abs maxoff u> not if		\ Does TOS fit relxx format? ; SFP024, SFP047
        opc execute  r> l>a  pop(ws) r#>a ( @lit ) use-a  \   STR nos, tos ; SFP024
        drop(ws)  reset-opt  exit
      endif
    endif
  endif

  pop(ws) rR1 sdmove,			\   MOV r1, tos
  opc execute  r> l>a  rR1 +0[reg]  use-a \ STR nos, [ r1 ]
  drop(ws)  reset-opt			\ SFP026
\ ...SFP104
;

: storeop       \ max_offset size -- ; -- ; max_offset xsize STOREOP <name> "instruction"
  create
    , ,                                 \ size, max_offset
    here  0 ,  lay-asm: drop            \ opcode
  does>
    dup @  swap cell+ dup @  swap cell+ @ (storeop)	\ SFP047
;

$0FFF LSIZE storeop c_!    "STR "
$00FF WSIZE storeop c_w!   "STRH "
$0FFF BSIZE storeop c_c!   "STRB "

: (addstoreop)	\ size maxoff opld opadd opst -- ; SFP063
\ Compile code for +! and friends
  { size maxoff opld opadd opst | wreg hasLit? -- }

  2 min-depth(ws)

  M0/M1? if				\ SFP107
    maxoff $8000:0000 and 0<> -> hasLit?	\ if ADD rx, # valid
  else
    -1 -> hasLit?			\ ADD rx, # for all ARM and M3/M4
  endif
  maxoff $FFFF and -> maxoff

\ Form address and offset in reg/r1
  tos-res dreg? if
    size sz?rem-litplus
  else
    tos-res reg? if
      0 tos-res
    else
      tos-res rR1 sdmove,  0 rR1
    endif
  endif
\ -- n mreg

\ Form source data in reg/lit
  nos-res reg? if
    nos-res
  else
    nos-res resGenLit? hasLit? and if	\ SFP107
      nos-res
    else
      nos-res rR0 sdmove,  rR0
    endif
  endif
\ -- n mreg sreg

\ Find a working register
  over rR1 <> if
    rR1 -> wreg
  else
    dup rR0 <> if
      rR0 -> wreg
    else
      GetNewReg ?dup 0=
      if  c_shuffle  2 min-depth(ws)  rR0  endif
      -> wreg
    endif
  endif
\ -- n areg sreg

  -rot 2>r				\ -- sreg ; R: -- n areg
  opld execute  wreg l>a  2r@ +n[reg]  use-a
  >r  opadd execute  wreg l>a  wreg l>a  r> gl>a  use-a
  opst execute  wreg l>a  2r> +n[reg]  use-a

  drop(ws) drop(ws)  reset-opt
;

: addstoreop	\ max_offset size -- ; --
\ max_offset xsize STOREOP <name> "instruction"
  create
    , ,                                 \ size, max_offset
    here  0 , 0 , 0 ,
    lay-asm: lay-asm: lay-asm:		\ load op store
    drop            \ opcode
  does>
    dup @
    swap cell+ dup @
    swap cell+ dup @
    swap cell+ dup @
    swap cell+ @
    (addstoreop)
;

\ SFP063...
$8000:0FFF LSIZE addstoreop c_+!     "LDR  "  "ADD .s "  "STR  "
$8000:00FF WSIZE addstoreop c_w+!    "LDRH "  "ADD .s "  "STRH "
$8000:0FFF BSIZE addstoreop c_c+!    "LDRB "  "ADD .s "  "STRB "

$0000:0FFF LSIZE addstoreop c_or!    "LDR  "  "ORR .s "  "STR  "
$0000:0FFF LSIZE addstoreop c_and!   "LDR  "  "AND .s "  "STR  "
$0000:0FFF LSIZE addstoreop c_xor!   "LDR  "  "EOR .s "  "STR  "
$0000:0FFF LSIZE addstoreop c_bic!   "LDR  "  "BIC .s "  "STR  "

$0000:00FF WSIZE addstoreop c_wor!   "LDRH "  "ORR .s "  "STRH "	\ SFP128
$0000:00FF WSIZE addstoreop c_wand!  "LDRH "  "AND .s "  "STRH "	\ SFP128
$0000:00FF WSIZE addstoreop c_wxor!  "LDRH "  "EOR .s "  "STRH "	\ SFP128
$0000:00FF WSIZE addstoreop c_wbic!  "LDRH "  "BIC .s "  "STRH "	\ SFP128

$0000:0FFF BSIZE addstoreop c_bor!   "LDRB "  "ORR .s "  "STRB "
$0000:0FFF BSIZE addstoreop c_band!  "LDRB "  "AND .s "  "STRB "
$0000:0FFF BSIZE addstoreop c_bxor!  "LDRB "  "EOR .s "  "STRB "
$0000:0FFF BSIZE addstoreop c_bbic!  "LDRB "  "BIC .s "  "STRB "
\ ...SFP063

: (on/off)	\ n -- ; SFP063
\ Compile code for ON and OFF
  1 min-depth(ws)

\ Form address and offset in reg/r1
  tos-res dreg? if
    ?rem-litplus12
  else
    tos-res reg? if
      0 tos-res
    else
      tos-res rR1 sdmove,  0 rR1
    endif
  endif
\ -- n offset mreg

  rot dup >r  0< if
    M0/M1? if				\ SFP107
      a[ mov .s  r0, # r> abs ]a use-a
      a[ neg .s  r0, r0  ]a use-a
    else
      a[ MVN .s  r0, # ]a  r> not n>a  use-a
    endif
  else
    a[ MOV .s  r0, # ]a  r> n>a  use-a
  endif
  2>r
  a[ STR  r0, ]a  2r> +n[reg]  use-a

  drop(ws)  reset-opt
;

: on/off	\ n -- ; -- ; n ON/OFF <name> ; SFP063
  create
    ,
  does>
    @ (on/off)
;

 0 on/off c_off				\ SFP063
-1 on/off c_on				\ SFP063

: (incr/decr)	\ n opld opadd opst -- ; SFP063
\ Compile code for INCR/DECR and friends
  { n opld opadd opst -- }

  1 min-depth(ws)

\ Form address and offset in reg/r1
  tos-res dreg? if
    ?rem-litplus12
  else
    tos-res reg? if
      0 tos-res
    else
      tos-res rR1 sdmove,  0 rR1
    endif
  endif
\ -- offset mreg

  2>r					\ -- ; R: -- n areg
  opld execute   a[ r0, ]a  2r@ +n[reg]  use-a
  opadd execute  a[ r0, r0, # ]a n n>a   use-a
  opst execute   a[ r0, ]a  2r> +n[reg]  use-a

  drop(ws)  reset-opt
;

: incr/decr	\ n -- ; -- ; SFP063
\ n INCR/DECR <name> "ld" "add" "st"
  create
    ,					\ n
    here  0 , 0 , 0 ,
    lay-asm: lay-asm: lay-asm:		\ load op store
    drop
  does>
    dup @
    swap cell+ dup @
    swap cell+ dup @
    swap cell+ @
    (incr/decr)
;

1 incr/decr c_incr	"LDR  "  "ADD .s "  "STR  "	\ SFP063
1 incr/decr c_decr	"LDR  "  "SUB .s "  "STR  "	\ SFP063

: c_2@          \ -- ; compile code for 2@
  1 min-depth(ws)
  tos-res dreg? 0= if
    ForceNewTOS+NOS					\ SFP030
    3os-res reg? if
      a[ LDR ]a  tos-res l>a  3os-res +0[reg]   use-a   \ LDR tosreg, [ 3osreg ]
      a[ LDR ]a  nos-res l>a  3os-res +cell[reg]  use-a \ LDR nosreg, [ 3osreg # 4 ]
    else
      3os-res rR0 sdmove,                               \ MOV r0, 3os-res
      a[ LDR ]a  tos-res l>a  rR0 +0[reg]  use-a        \ LDR tosreg, [ r0 ]
      a[ LDR ]a  nos-res l>a  rR0 +cell[reg]  use-a     \ LDR nosreg, [ r0, # 4 ]
    endif
    pop(ws) pop(ws) drop(ws) push(ws) push(ws)          \ dispose of 3os	\ SFP030
    reset-opt  exit
  endif
  ?rem-litplus12 2>r
  ForceNewTOS						\ nos-res=old tos-res=dreg	\ SFP030
  a[ LDR ]a  tos-res l>a  2r@ +n[reg]  use-a			\ LDR tosreg, [ nosreg ]
  a[ LDR ]a  nos-res l>a  2r> swap cell(t) + swap +n[reg]  use-a \ LDR nosreg, [ nosreg # 4 ]
;

: c_2!          \ compile code for 2!
  3 min-depth(ws)  reset-opt

  tos-res reg? 0=
  if  rR1 CopyTos  endif

  nos-res reg? if
    a[ STR ]a  nos-res l>a  tos-res +0[reg]  use-a      \ STR nosreg, [ tosreg ]
  else
    nos-res rR0 sdmove,                                 \ MOV r0, nosres
    a[ STR r0 ]a  tos-res +0[reg]  use-a                \ STR r0, [ tosreg ]
  endif

  3os-res reg? if
    a[ STR ]a  3os-res l>a  tos-res +cell[reg]  use-a   \ STR 3os-reg, [ tosreg # 4 ]
  else
    3os-res rR0 sdmove,                                 \ MOV r0, 3os-res
    a[ STR r0 ]a  tos-res +cell[reg]  use-a             \ STR r0, [ tosreg # 4 ]
  endif

  drop(ws) drop(ws) drop(ws)		\ SFP030
;


\ **************************************************
\ Comparisons - N.B. these affect the optimiser data
\ **************************************************

: cmp,          { src dest -- ; lay compare instruction }
  src absResGenLit?  src res-value @ 0< and if
    M0/M1? if
      a[ MOV .s  r0, # ]a src res-value @ abs n>a  use-a	\ SFP112
\      src rR0 sdmove,			\ del SFP112
      a[ CMN ]a dest l>a  rR0 l>a  use-a
    else
      a[ CMN ]a dest l>a  #>a src res-value @ abs n>a  use-a
    endif
  else
    a[ CMP ]a dest l>a  src gl>a  use-a
  endif
;

: genCMP2       \ -- switched? ; lay compare code for two operands
\ TOS in register?
  tos-res reg? if			\ CMP tosreg, nosreg/abs12bitlit,
    nos-res reg? nos-res absResGenLit? or
    if
      nos-res tos-res cmp,              \ CMP tosreg, nosreg/abs12bitlit
    else
      nos-res rR0 sdmove,               \ MOV r0, nos
      rR0 tos-res cmp,                  \ CMP tosreg, r0
    endif
    drop(ws) drop(ws)  1  exit          \ switched
  endif

\ NOS in register?
  nos-res reg? if			\ CMP nosreg, tos
    tos-res absResGenLit? if
      tos-res nos-res cmp,              \ CMP nosreg, abs12bitlit
    else
      tos-res rR0 sdmove,               \ MOV r0, tos
      rR0 nos-res cmp,                  \ CMP nosreg, r0
    endif
    drop(ws) drop(ws)  0 exit           \ not switched
  endif

\ Both operands literals? - done here as abs12bitlits are identified below
  tos-res lit?  nos-res lit? and if	\ Code generated by next section
    cr ." Urk! Comparison of two literals " .source-line
  endif

\ NOS abs12bit literal?
  nos-res absResGenLit? if
    tos-res rR0 sdmove,                 \ MOV r0, tos
    nos-res rR0 cmp,                    \ CMP r0, nos
    drop(ws) drop(ws)  1 exit           \ switched
  endif

\ TOS abs12bit literal?
  tos-res absResGenLit? if
    nos-res rR0 sdmove,                 \ MOV r0, nos
    tos-res rR0 cmp,                    \ CMP r0, tos
    drop(ws) drop(ws)  0 exit           \ not switched
  endif

\ TOS and NOS either both deep, both lits, or deep/lit combo
  tos-res rR0 sdmove,                   \ MOV r0, tos
  nos-res rR1 sdmove,                   \ MOV r1, nos
  rR0 rR1 cmp,                          \ CMP r1, r0
  drop(ws) drop(ws)  0                  \ not switched
;

\ Not necessary to preserve status flags on the ARM as the shuffle doesn't
\ currently affect the flags.
: GuardedForceNewTOS    \ -- ; gets new TOS reg, preserving status flags
  GetNewReg ?dup 0= if			\ need register for result
    #on-stack(ws) @  c_shuffle  min-depth(ws)	\ SFP036
    GetNewReg				\ shuffle always permits a new register
  endif
  push(ws)                              \ make new register TOS
;

: genWFF	\ struct --
\ Generate a well formed flag (0/-1) using the condition code pair
\ that the struxcture points at.
  >r  Thumb? if
    Cortex-M0/M1? if			\ SFP107
      amrk>
      a[ b ]a r> @ execute  chere(t) 2+ use-a
      a[ mov .s  tos-res l>a # 0 ]a use-a
      amrk>
      a[ b ]a  chere(t) 2+ use-a
      swap >ares
      a[ mov .s  tos-res l>a  # 1
         neg .s  tos-res l>a  tos-res l>a
      ]a use-a
      >ares
    else
      a[ ite  r> @ execute
         mvn   tos-res l>a  # 0
         mov   tos-res l>a  # 0
      ]a use-a
    endif
  else
    a[ mvn ]a  r@ @ execute tos-res l>a  a[ # 0 ]a  use-a
    a[ mov ]a  r> cell+ @ execute tos-res l>a  a[ # 0 ]a  use-a
  endif
;

: (cmp2)        \ struct -- ; lays comparision code
                \ struct points to the comparison operator data structure (four strings)
  2 min-depth(ws)			\ SFP030
  2CMPID start-seq                      \ mark start of sequence
  dup      seqptr !

\ For the ARM the first and second opcodes generate the normal true MVN and
\ false MOV instructions, the third and fourth opcodes generate the switched
\ instruction sequence. The tail after the comparison is:
\ MVN .xx tos, # 0
\ MOV .yy tos, # 0

  genCMP2                               \ lay comparison code	\ SFP030
  dup seqflags !                        \ set whether comparison is switched
                                        \ -- struct switched? ; see later
  GuardedForceNewTOS			\ SFP030

\ Select which opcodes to lay, according to switched, and lay code
\ -- struct switched?
  if  2 cells +  endif                  \ step over opcodes
  start-bkup                            \ mark start of code to be removed by optimiser
  genWFF				\ lay sequence
  end-seq                               \ end of optimisable sequence
;

: cmp2          \ -- ; -- ; CMP2 C_< "SETL  "  "SETG  " works for tos nos OP
                \ the first opcode defines the instruction for comparing tos OP nos
                \ the second permits code where nos OP tos is better. The third
                \ and fourth opcodes are used by the branch generators
  create
    here  6 cells allot
    lay-asm: lay-asm:                   \ SETxx opcodes for CMP nos, tos  CMP tos, nos
    lay-asm: lay-asm:                   \ branch opcodes for CMP nos, tos  CMP nos, tos
    lay-asm: lay-asm: drop
  does>
    (cmp2)
;

\                  for >                for > if
\           returning a flag            optimising a branch
\           normal        switched
\           TRUE  FALSE   TRUE  FALSE   normal  switched
cmp2 c_=   ".EQ " ".NE " ".EQ " ".NE "  "B .NE "  "B .NE "
cmp2 c_<>  ".NE " ".EQ " ".NE " ".EQ "  "B .EQ "  "B .EQ "
cmp2 c_<   ".LT " ".GE " ".GT " ".LE "  "B .GE "  "B .LE "
cmp2 c_>   ".GT " ".LE " ".LT " ".GE "  "B .LE "  "B .GE "
cmp2 c_<=  ".LE " ".GT " ".GE " ".LT "  "B .GT "  "B .LT "
cmp2 c_>=  ".GE " ".LT " ".LE " ".GT "  "B .LT "  "B .GT "
cmp2 c_u<  ".LO " ".HS " ".HI " ".LS "  "B .HS "  "B .LS "
cmp2 c_u>  ".HI " ".LS " ".LO " ".HS "  "B .LS "  "B .HS "
cmp2 c_u<= ".LS " ".HI " ".HS " ".LO "  "B .HI "  "B .LO "
cmp2 c_u>= ".HS " ".LO " ".LS " ".HI "  "B .LO "  "B .HI "

: (cmp1)        \ struct -- ; lays comparision code
                \ struct points to the comparison operator data structure (four strings)
  1 min-depth(ws)
  1CMPID start-seq                      \ mark for optimiser
  dup seqptr !                          \ structure holds opcode strings
  0 seqflags !                          \ not switched yet
  pop(ws) rR0 sdcmp0,			\ lay compare code	\ SFP030, SFP048
  GuardedForceNewTOS                    \ get a free register for the result, preserving CCR	\ SFP030
  start-bkup                            \ start of code to be removed
  genWFF				\ generate flag
  end-seq                               \ end of optimisable sequence                   <-----------
;

: cmp1          \ -- ; -- ; CMP2 C_< "SETL  "  "SETG  " works for tos nos OP
                \ the first opcode defines the instruction for comparing tos OP nos
                \ the second is a dummy for the optimiser. The third
                \ and fourth opcodes are used by the branch generators.
                \ Note that the data structure is the same as for CMP2
  create
    here  6 cells allot
    lay-asm: lay-asm:                   \ SETxx opcodes for CMP nos, tos  CMP tos, nos
    lay-asm: lay-asm:                   \ branch opcodes for CMP nos, tos  CMP nos, tos
    lay-asm: lay-asm: drop
  does>
    (cmp1)
;

: SETx          \ -- ; error if executed
\  1 abort" SETx should never execute - internal software error"
  opt_bad_set throw
;

cmp1 (c_0=)  ".EQ "  ".NE "  "SETx "  "SETx "  "B .NE "  "B .EQ "
cmp1 c_0<>   ".NE "  ".EQ "  "SETx "  "SETx "  "B .EQ "  "B .NE "
cmp1 c_0<    ".MI "  ".PL "  "SETx "  "SETx "  "B .PL "  "B .MI "
cmp1 c_0>    ".GT "  ".LE "  "SETx "  "SETx "  "B .LE "  "B .GT "

: c_0=          \ -- ; magic available with 0= for ?DUP 0= and ?DUP 0= IF
  QDUPID opt? if
    seqbeg @  seqbkup @                 \ preserve start info
    (c_0=)                              \ process
    seqbkup !  seqbeg !                 \ restore start info
    QDUPZEID seqid !                    \ mark optimisation type
  else
    (c_0=)
  endif
;


\ **********************
\ Return stack operators
\ **********************

: c_>r          \ -- ; compile code for >R ; SFP029
  1 min-depth(ws)
  DOESID opt? if
    seq-backup  c_does>_>r
    reset-opt  1 +rs-count
    exit
  endif
  ?SaveLink  Thumb? if
    tos-res reg? if
     a[ PUSH  {{ pop(ws) l>a }} ]a use-a
    else
      pop(ws) rR0 sdmove,
     a[ PUSH  {{ rR0 l>a }} ]a use-a
    endif
  else
    tos-res reg? if
      a[ STR ]a  pop(ws) l>a  a[ [ rsp, # -4 ] ! ]a  use-a	\ SFP030
    else
      pop(ws) rR0 sdmove,				        \ SFP030
\      a[ STMFD rsp ! { r0 } ]a  use-a		\ ARM7
      a[ str  r0, [ rsp, # -4 ] ! ]a use-a	\ ARM9 ; SFP101
    endif
  endif
  reset-opt
  1 +rs-count
;

: c_r@          \ -- ; compile code for R@
  ForceNewTOS  reset-opt		\ SFP030
  ?SaveLink				\ SFP059
  a[ LDR ]a  tos-res l>a  DR1 l>a  use-a
  1 ?inline-rs
;

: c_r>          \ -- ; compile code for R> ; SFP029
  ForceNewTOS  reset-opt		\ SFP030
  ?SaveLink				\ SFP059
  RFROMID start-seq  start-bkup         \ SFP029
  Thumb? if
    a[ POP {{ tos-res l>a }} ]a use-a
  else
    a[ LDR ]a  tos-res l>a a[ [ rsp ], # 4 ]a  use-a
  endif
  end-seq  -1 +rs-count
;

: c_2>r         \ -- ; compile code for 2>R
  2 min-depth(ws)  reset-opt		\ SFP030
  ?SaveLink				\ SFP059
  tos-res reg? if
    pop(ws)
  else
    pop(ws) rR1 sdmove,
    rR1
  endif
  >r
  tos-res reg? if			\ was nos-res
    pop(ws)
  else
    pop(ws) rR0 sdmove,  rR0
  endif
  >r
  Thumb? if
    a[ PUSH  {{ r> l>a }} ]a use-a	\ nos
    a[ PUSH  {{ r> l>a }} ]a use-a	\ tos
  else
    a[ STR ]a r> l>a  a[ [ rsp, # -4 ] ! ]a  use-a        \ nos
    a[ STR ]a r> l>a  a[ [ rsp, # -4 ] ! ]a  use-a        \ tos
  endif
  2 +rs-count
;

: c_2r@         \ -- ; compile code for 2R@
  ForceNewTOS+NOS  reset-opt		\ SFP030
  ?SaveLink				\ SFP059
  a[ LDR ]a  tos-res l>a  DR1 l>a  use-a
  a[ LDR ]a  nos-res l>a  DR2 l>a  use-a
  2 ?inline-rs
;

: c_2r>         \ -- ; compile code for 2R>
  ForceNewTOS+NOS  reset-opt		\ SFP030
  ?SaveLink				\ SFP059
  Thumb? if
    a[ POP {{ tos-res l>a }} ]a  use-a
    a[ POP {{ nos-res l>a }} ]a use-a
  else
    a[ LDR ]a  tos-res l>a  a[ [ rsp ], # 4 ]a  use-a
    a[ LDR ]a  nos-res l>a  a[ [ rsp ], # 4 ]a  use-a
  endif
  -2 +rs-count
;


\ -----------------------------------------------
\ low level branches - marking, resolving, laying
\ -----------------------------------------------
\ -----------------------------------------
\ Label handling - CPU specific when hosted
\ -----------------------------------------

((
Branches are handled by generating addresses which are resolved by the assembler.
These addresses are referred to as branch identifiers, denoted by branchid in the
stack comments. and SBRID refers to the location (source) of a branch, and a DBRID
refers to the target (destination) of a branch.

A source is generated before the opcode is executed, and therefore is the address
of the B .xx instruction. This opcode can be inspected by the resolution code. For
a I386 CPU, the offset is 8 bits unless the opcode is 0F, E8, or E9 in which case
the offset is 32 bits. The opcode is one byte long unless it starts with 0F, in
which case the opcode is two bytes long.
))

: ?s/l		\ -- ; select 8 or 20 bit forward branch ; SFP106
  Thumb? if
    Cortex-M0/M1? 0= if			\ SFP107
      <short-branches> @ 0=
      if  a[ .W ]a  endif
    endif
  endif
;


\ SFP063...
variable top-mask
  3 top-mask !

: LoopAlign	\ --
\ Align with NOPs to head of loop.
  begin		\ Align code on a boundary by inserting NOPs
    chere(t) top-mask @ and
   while
    a[ NOP ]a use-a			\ NOP
  repeat
;

also cc/i

: LoopAlignment	cc/i	\ n --
\ *G Set loop starts, e.g. *\fo{BEGIN..XXX} and *\fo{DO..LOOP}
\ ** to be aligned on an n-byte boundary, where n must be a
\ ** power of two. This is useful to force the heads of loops
\ ** onto a cache line or memory buffer boundary. The default is 4.
\ ** For a Philips LPC CPU DO..LOOP performance can be
\ ** improved 10-20% in some cases by using n=16.
\ *E   #16 LoopAlignment     \ set to 16 byte boundary
\ **   0   LoopAlignment     \ revert to lowest setting
  1 max  1- top-mask !
;

: LoopAlignment?	cc/i	\ --
\ *G Return the current loop alignment setting.
  top-mask @ 1+
;

: LoopAlign	cc/i	\ --
\ *G Align with NOPs to head of loop. Used to align loop heads
\ ** in assembler code.
  Flush-asm LoopAlign
;

previous definitions

: amrk<         \ -- dbrid ; mark the destination of backward branch
  LoopAlign chere(t)
;
\ ...SFP063

: <ares         \ dbrid -- dbrid ; resolve a backward branch - DBRID is consumed by opcode
;

: c_mrk_branch< \ -- dbrid ; generate the address to resolve a backward branch
  c_shuffle  reset-opt                  \ flush stack to known state
  ?SaveLink				\ SFP059
  amrk<					\ define address
;

: >c_res_branch \ sbrid -- ; resolve a forward branch
  c_shuffle  reset-opt                  \ flush the working stack
  >ares                                 \ resolve branch
;

: c_mrk_branch> \ -- sbrid ; generate the address to resolve a forward branch
  c_shuffle  reset-opt                  \ flush stack to known state
  ?SaveLink				\ SFP059
  amrk>					\ define address
;

: c_?dup        \ -- ; lay code for ?DUP ; here to make author's life easier
  1 min-depth(ws)
  QDUPID start-seq                      \ mark this sequence for the optimiser
  c_shuffle                             \ flush the working stack
  tos-res rR0 sdcmp0,                   \ sets flags ; SFP048
  start-bkup                            \ mark start of optimisable sequence
  ['] c_0<> seqptr !  seqflags off      \ identify Jxx type for later use
  Thumb? if
    Cortex-M0/M1? if			\ SFP107
      a[ ne, if,
           sub .s  psp, psp, # 4
           str     tos, [ psp, # 0 ]
         endif,
      ]a use-a
    else
      a[ it .ne
         str tos, [ psp, # -4 ] !
      ]a use-a
    endif
  else
    a[ str .ne  tos, [ psp, # -4 ] ! ]a use-a	\ ARM9
  endif
  end-seq                               \ mark end of optimisable sequence
;

: lay-optbranch \ -- ; lays Bxx defined by comparison structure
  seqptr @  seqflags @                  \ if the branch has been switched, use 6th, else 5rd operation in CMPx structure
  if  5  else  4  endif                 \ by skipping 5 or 4 strings
  0 do  cell+  loop                     \ now points to the correct branch opcode
  @ execute                             \ generate the code
;

: optbranch<    \ dbrid -- ; optimise a branch after a conditional
  seq-backup                            \ back up to just after the compare
  drop(ws)                              \ discard TOS, to be replaced by test of flags
  c_GuardedShuffle                      \ does not affect the flags
  lay-optbranch  <ares  use-a           \ lay down the branch code
  reset-opt                             \ reset the optimiser
;

: qdupbranch<    \ dbrid --
\ Optimise a branch after ?DUP. N.B. ?DUP causes a SHUFFLE.
  seq-backup                            \ back up to just after the compare
  Thumb? if
    Cortex-M0/M1? if
      a[  eq, if,
            ldmia psp, ! {{ tos }}
          endif,
      ]a use-a
    else
      a[ it   .eq
         ldr  tos, [ psp ], # 4
      ]a use-a
    endif
  else
    a[ ldr .eq tos, [ psp ], # 4 ]a use-a	\ ARM ; lay DROP code for TOS=0 case ; SFP101
  endif
  a[ B .EQ ]a  <ares  use-a             \ lay JMP to BEGIN etc
  init-stack(ws)  reset-opt             \ reset the optimiser
;

: qdupzebranch< \ dbrid -- ; optimise a branch after ?DUP 0=
  seq-backup                            \ back up to just after the compare
  a[ B .NE ]a  <ares  use-a
  init-stack(ws)  reset-opt
  c_drop
;

: flag1branch<	\ dbrid -- ; optimise a branch after a dyad ; SFP062
\  seq-backup                            \ back up to just after the code ; del SFP067
  drop(ws)                              \ discard TOS, to be replaced by test of flags
  c_GuardedShuffle                      \ does not affect the flags
  a[ B .EQ ]a  <ares  use-a             \ lay JMP to BEGIN etc
  reset-opt                             \ reset the optimiser
;

: optbranch>    \ -- sbrid ; optimise a branch after a conditional
  seq-backup                            \ back up to just after the compare
  drop(ws)                              \ discard TOS, to be replaced by test of flags
  c_GuardedShuffle                      \ does not affect the flags
  ?SaveLink				\ SFP059
  amrk>
  lay-optbranch ?s/l chere(t) 4 +  use-a \ lay down the branch code ; SFP061 SFP106
  reset-opt                             \ reset the optimiser
;

: qdupbranch>    \ -- sbrid
\ Optimise a branch after ?DUP. N.B. ?DUP causes a SHUFFLE.
  seq-backup                            \ back up to just after the compare
  Thumb? if
    Cortex-M0/M1? if
      a[  eq, if,
            ldmia psp, ! {{ tos }}
          endif,
      ]a use-a
    else
      a[ it .eq
        ldr  tos, [ psp ], # 4
      ]a use-a
    endif
  else
    a[ ldr .eq tos, [ psp ], # 4 ]a use-a	\ ARM9
  endif
  ?SaveLink				\ SFP059
  amrk>
  a[ B .EQ ]a ?s/l chere(t) 4 +  use-a	\ lay JMP to ENDIF/ELSE etc ; SFP061 SFP106
  init-stack(ws)  reset-opt             \ reset the optimiser
;

: qdupzebranch> \ -- sbrid ; optimise a branch after ?DUP 0=
  seq-backup                            \ back up to just after the compare
  ?SaveLink				\ SFP059
  amrk>
  a[ B .NE ]a ?s/l chere(t) 4 +  use-a	\ SFP061 SFP106
  init-stack(ws)  reset-opt
  c_drop
;

: flag1branch>	\ -- sbrid ; optimise a branch after a dyad ; SFP062
  drop(ws)                              \ discard TOS, to be replaced by test of flags
  c_GuardedShuffle                      \ does not affect the flags
  ?SaveLink
  amrk>
  a[ B .EQ ]a ?s/l chere(t) 4 +  use-a	\ lay JMP to ENDIF/ELSE etc ; SFP106
  reset-opt                             \ reset the optimiser
;

: c_branch<     \ dbrid -- ; lay BRANCH instructions
  ?LinkSaved				\ SFP059
  c_shuffle  reset-opt                  \ flush the working stack
  a[ B ]a  <ares  use-a
;

: c_?branch<    \ dbrid -- ; lay ?BRANCH instructions
  ?LinkSaved				\ SFP059
  1 min-depth(ws)			\ SFP030
  chere(t) seqend @ = if
    2CMPID optid?  1CMPID optid? or if	\ try < UNTIL or 0< UNTIL
      optbranch<  exit
    endif
    FLAG1SETID optid? if		\ SFP062, SFP067
      seq-backup flag1branch<  exit	\ SFP062, SFP067
    endif				\ SFP062
    LITPLUSID optid? if			\ SFP062, SFP067
      flag1branch<  exit		\ SFP062, SFP067
    endif				\ SFP062, SFP067
    QDUPID optid? if			\ try ?DUP UNTIL
      qdupbranch<  exit
    endif
    QDUPZEID optid? if			\ try ?DUP 0= UNTIL
      qdupzebranch<  exit
    endif
  endif
  tos-res lit? if
    cr ." Urk! Conditional branch tests literal value" .source-line
  endif
  pop(ws) rR0 sdcmp0,			\ sets flags ; SFP048
  c_GuardedShuffle  reset-opt           \ flush the working stack
  a[ B .EQ ]a  <ares  use-a
;

: c_branch>     \ -- sbrid ; forward referenced branch
  c_shuffle  reset-opt                  \ flush the working stack
  ?SaveLink				\ SFP059
  amrk>
  a[ B ]a ?s/l chere(t) 4 +  use-a	\ SFP061 SFP106
;

: c_?branch>    \ -- sbrid ; forward referenced branch
  1 min-depth(ws)			\ SFP030
  chere(t) seqend @ = if		\ optimisation available
    2CMPID optid?  1CMPID optid? or if	\ try < IF  or 0< IF
      optbranch>  exit                  \ optimise the branch
    endif
    FLAG1SETID optid? if		\ SFP062, SFP067
      seq-backup flag1branch>  exit	\ SFP062, SFP067
    endif				\ SFP062
    LITPLUSID optid? if			\ SFP062, SFP067
      flag1branch>  exit		\ SFP062, SFP067
    endif				\ SFP062, SFP067
    QDUPID optid? if			\ try ?DUP IF
      qdupbranch>  exit                 \ optimise the branch
    endif
    QDUPZEID optid? if			\ try ?DUP 0= IF
      qdupzebranch>  exit               \ optimise the branch
    endif
  endif
  tos-res lit? if
    cr ." Urk! Conditional branch tests literal value" .source-line
  endif
  pop(ws) rR0 sdcmp0,			\ sets flags ; SFP048
  c_GuardedShuffle  reset-opt           \ flush the working stack
  ?SaveLink				\ SFP059
  amrk>
  a[ B .EQ ]a  ?s/l chere(t) 4 +  use-a	\ SFP061 SFP106
;


\ ---------------
\ CASE statements
\ ---------------

variable NextCaseTarg   \ -- addr ; holds address of CASE

: c_case        \ -- dbrid
  NextCaseTarg @  c_mrk_branch< NextCaseTarg !
;

: c_of          \ -- sbrid ; MUST match with >C_RES_BRANCH and C_?BRANCH>
                \ lays equivalent of OVER = IF DROP
  2 min-depth(ws)			\ removes 1 item before branch	\ SFP030
  nos-res  genCMP2 drop  push(ws)       \ lays down the compare code, but leaves NOS intact
  c_GuardedShuffle  reset-opt           \ reorder the stack, does not affect flags
  ?SaveLink				\ SFP059
  amrk>                                 \ lay the forward branch
  a[ B .NE ]a ?s/l chere(t) 4 +  use-a	\ SFP061 SFP106
  c_drop                                \ lay code for DROP
;

: c_?of         \ -- sbrid
  c_?branch>
;

: c_endof       \ sbrid1 -- sbrid2
  c_branch>  swap >c_res_branch
;

: c_dropcase    \ addr --
  NextCaseTarg !
;

: c_endcase     \ addr --
  >c_res_branch
;

: c_nextcasejump \ -- ; lay W. BRA, opcode
  NextCaseTarg @ c_branch<
;

: c_nextcasefix \ addr --
  NextCaseTarg !
;


\ -------------------
\ DO LOOP and friends
\ -------------------

: c_unloop      \ -- ; compile code for UNLOOP
  reset-opt
  a[ ADD rsp, rsp,  # #12 ]a  use-a
;

: c_do          \ -- mrk> dbrid ; compile code for DO
  2 min-depth(ws)
  c_shuffle  reset-opt
\  ?SaveLink				\ required for FAST inlined DO
  +AlignReturn  target-sym (DO) compile-ref
  MRK>  amrk<
\  not-inlined				\ required for FAST inlined DO
;

: c_?do         \ -- mrk> dbrid ; compile code for ?DO
  2 min-depth(ws)
  c_shuffle  reset-opt
\  ?SaveLink				\ required for FAST inlined DO
  +AlignReturn  target-sym (?DO) compile-ref
  MRK>  amrk<
\  not-inlined				\ required for FAST inlined DO
;

: >resARM/Cx	\ mrk> --
  flush-asm  chere(t) 			\ -- mrk> dest
  Thumb? if  1 or  endif		\ T bit as required
  swap l!c(t)				\ fix up
;

: c_loop        \ mrk> dbrid -- ; compile code for LOOP
  c_shuffle  reset-opt                  \ shuffle data stack
  a[ LDR r0, [ rsp ] ]a use-a           \ fetch index
  a[ ADD .S r0, r0, # 1 ]a  use-a       \ increment index by one
  a[ STR .VC r0, [ rsp ] ]a  use-a      \ no overflow, store index
  a[ B .VC ]a  <ares  use-a             \ no overflow, branch back to start
  a[ ADD rsp, rsp, # #12 ]a  use-a      \ drop 3 items from return stack
  >resARM/Cx				\ resolve forward branch
;

: c_+loop       \ mrk> dbrid -- ; compile code for +LOOP
  1 min-depth(ws)
  tos-res absResGenLit? if
    pop(ws) res-value @ >r              \ remove literal value for later	\ SFP030
    c_shuffle  reset-opt                \ shuffle leaves TOS in register
    a[ LDR r0, [ rsp ] ]a  use-a        \ fetch index
    r@ 0< if				\ update index
      a[ SUB .S r0, r0, # ]a r> abs n>a  use-a
    else
      a[ ADD .S r0, r0, # ]a r> n>a  use-a
    endif
    a[ STR .VC r0, [ rsp ] ]a  use-a    \ no overflow, store index
  else
    tos-res reg?			\ force tos to register	\ SFP033
    if  tos-res  else  tos-res rR1 sdmove,  rR1  endif		\ SFP033
    >r
    a[ LDR r0, [ rsp ] ]a use-a         \ fetch index
    a[ ADD .S r0, r0, ]a  r> l>a  use-a	\ increment index	\ SFP033
    a[ STR .VC r0, [ rsp ] ]a  use-a    \ no overflow, store index
    drop(ws)  c_shuffle  reset-opt	\ shuffle does not affect flags	\ SFP033
  endif
  a[ B .VC ]a  <ares  use-a             \ no overflow, branch back to start
  a[ ADD rsp, rsp, # #12 ]a  use-a      \ drop 3 items from return stack
  >resARM/Cx				\ resolve forward branch
;

: c_i           \ -- ; compile code for I
  ForceNewTOS                           \ somewhere for result
  reset-opt				\ SFP030
  DR1 tos-res sdmove,                   \ LDR newreg, [ rsp, # 0 ]
  DR2 rR0 sdmove,                       \ LDR r0, [ rsp, # 4 ]
  a[ ADD ]a  tos-res l>a  tos-res l>a  a[ r0 ]a  use-a  \ ADD newreg, newreg, r0
;

: c_j           \ -- ; compile code for J
  ForceNewTOS                           \ somewhere for result
  reset-opt				\ SFP030
  DR4 tos-res sdmove,                   \ LDR newreg, [ rsp, # 12 ]
  DR5 rR0 sdmove,                       \ LDR r0, [ rsp, # 16 ]
  a[ ADD ]a  tos-res l>a  tos-res l>a  a[ r0 ]a  use-a  \ ADD newreg, newreg, r0
;

: c_leave       \ -- ; compile code for LEAVE
  c_shuffle  reset-opt
  a[ ADD rsp, rsp, # 8 ]a  use-a        \ remove limit, index from return stack
  Thumb? if
\    Cortex-M0/M1? if
      a[ pop .n  {{ pc }} ]a use-a	\ SFP107
\    else
\      a[ ldr pc, [ rsp ], # 4 ]a use-a	\ SFP101
\    endif
  else
    a[ ldr pc, [ rsp ], # 4 ]a use-a	\ ARM9 ; SFP101
  endif
  not-inlined				\ requires absolute address
;

: c_?leave      \ -- ; compile code for ?LEAVE ; SFP033 SFP107
  1 min-depth(ws)  not-inlined		\ requires absolute address
  pop(ws) rR0 sdcmp0,			\ set flags on TOS ; SFP048
  c_GuardedShuffle  reset-opt		\ SFP107
  Thumb? if
    Not-M0/M1? if			\ SFP107
      a[ itt .ne			\ M3+ has ITT instruction.
         ADD  rsp, rsp, # 8		\ remove limit, index  - if old TOS<>0
         pop .n  {{ pc }}		\ SFP107
\         ldr  pc, [ rsp ], # 4
      ]a use-a
      exit
    endif
    \ M0/M1 without ITT
    a[  ne, if,
          add     rsp, rsp, # 8		\ remove limit, index  - if old TOS<>0
          pop .n  {{ pc }}		\ go to end address
        endif,
    ]a use-a
    exit
  endif
  \ ARM32
  a[ ADD .NE rsp, rsp, # 8 ]a  use-a    \ remove limit, index  - if old TOS<>0
  a[ ldr .ne  pc, [ rsp ], # 4 ]a use-a	\ ARM9 ; SFP101
;


\ ***********************
\ Local variable handling
\ ***********************

: c_lvARM/M3	\ offset -- ; generate code for LVx address
  dup >r  abs #GenLit? if		\ SFP024, SFP054
    r@ 0< if
      a[ SUB ]a  tos-res l>a  a[ LP, # ]a  r> abs n>a  use-a
    else
      a[ ADD ]a  tos-res l>a  a[ LP, # ]a  r> n>a  use-a
    endif
  else
    a[ MVL ]a tos-res l>a  #>a  r> n>a  use-a
    a[ ADD ]a tos-res l>a  tos-res l>a  a[ LP ]a  use-a
  endif
;

: c_lvM0/M1	\ offset -- ; generate code for LVx address
  dup >r abs #256 u< if
    a[ mov ]a  tos-res l>a  rLP l>a  use-a
    r@ 0< if
      a[ SUB .s ]a  tos-res l>a  tos-res l>a  a[ # ]a r> abs n>a  use-a
    else
      a[ ADD .s ]a  tos-res l>a  tos-res l>a  a[ # ]a r> abs n>a  use-a
    endif
  else
    tos-res r> primLoadLit
    a[ ADD ]a tos-res l>a  tos-res l>a  a[ LP ]a  use-a
  endif
;

: c_lv          \ offset -- ; generate code for LVx address
  ForceNewTOS  reset-opt  M0/M1?
  if  c_lvM0/M1  else  c_lvARM/M3  endif
;

: c_lv@ARM/M3	\ offset -- ; generate code for LVx fetch
  dup >r #memOffLong? if
    a[ LDR ]a tos-res l>a  a[ [ LP, # r> n>a ] ]a  use-a
  else
    a[ MVL ]a tos-res l>a  #>a  r> n>a  use-a
    a[ LDR ]a tos-res l>a  a[ [ LP, ++ ]a  tos-res l>a  a[ ] ]a  use-a
  endif
;

: c_lv@M0/M1	\ offset -- ; generate code for LVx fetch
  c_lvM0/M1
  a[ LDR ]a tos-res l>a  tos-res +0[reg]  use-a
;

: c_lv@         \ offset -- ; generate code for LVx fetch
  ForceNewTOS  reset-opt  M0/M1?
  if  c_lv@M0/M1  else  c_lv@ARM/M3  endif
;

: c_lv!ARM/M3	\ offset -- ; generate code for LVx store
  tos-res reg? if
    dup >r #memOffLong? if
      a[ STR ]a  pop(ws) l>a  a[ [ LP, # r> n>a ] ]a  use-a
    else
      a[ MVL r1 ]a  #>a r> n>a  use-a
      a[ STR ]a  pop(ws) l>a  a[ [ LP, ++ r1 ] ]a  use-a
    endif
  else
    pop(ws) rR0 sdmove,
    dup >r #memOffLong? if
      a[ STR r0 [ LP, # r> n>a ] ]a  use-a
    else
      a[ MVL r1 ]a  #>a r> n>a  use-a
      a[ STR r0 [ LP, ++ r1 ] ]a  use-a
    endif
  endif
;

: c_lv!M0/M1	\ offset -- ; generate code for LVx store
\ cr ." c_lv!M0/M1 in"
  ForceNewTos c_lvM0/M1  tos-res >r  nos/R1 >r
  a[ STR ]a  r> l>a  r> +0[reg] use-a
  drop(ws) drop(ws)
\ cr ." c_lv!M0/M1 out"
;

: c_lv!         \ offset -- ; generate code for LVx store
  1 min-depth(ws)  reset-opt  M0/M1?
  if  c_lv!M0/M1  else  c_lv!ARM/M3  endif
;

\ SFP034 ...
variable <fastlvs>
  <fastlvs> on
variable <FixedLVs>
  <FixedLVs> on
variable <LVadjType>
  <LVadjType> off
also cc/i
: +FastLVs	cc/i	\ -- ; enable fast local variables
\ *G Enables generation of inline local variable entry code. This
\ ** is  the default condition, and is strongly recommended.
  <fastlvs> on  ;
: -FastLVs	cc/i	\ -- ; disable fast local variables
\ *G Disables generation of inline local variable entry code.
  <fastlvs> off  ;
: +FixedLVs	cc/i	\ -- ; enable faster local variable entry code
\ *G Enables generation of faster and shorter local variable entry
\ ** code. The only penalty is that run-time adjustment of the stack
\ ** frame is much more difficult and may require more stack space.
\ ** This is  the default condition, and is strongly recommended.
  <FixedLVs> on  ;
: -FixedLvs	cc/i	\ -- ; disable faster local variable entry code
\ *G Disables generation of faster/shorter inline local variable
\ ** entry code.
  <FixedLVs> off  ;
: SmallFrames	cc/i	\ -- ; max 512 bytes of LVs
\ *G *\b{Cortex only}: A local variable frame is restricted
\ ** to a bit less than 512 bytes. This is enough for
\ ** embedded systems use and results in smaller/faster entry code.
\ ** Requires both *\fo{+FixedLVs} and *\fo{+FastLVs}.
  0 <LVadjType> !  ;
: LargeFrames	cc/i	\ -- ; max 4096 bytes of LVs
\ *G *\b{Cortex only}: A local variable frame is restricted
\ ** to a bit less than 4096 bytes. This is enough for
\ ** all hosted use.
\ ** Requires both *\fo{+FixedLVs} and *\fo{+FastLVs}.
  1 <LVadjType> !  ;

previous definitions

: forceGenLit	\ +n -- +n'
  3 + -4 and				\ round up to 4 byte unit
;

variable makelvstyle	\ -- addr ; true for inlined MAKELVS

: copyLVargs	\ #args -- ; in address unit
\ cr ." copyLVargs"
  cell(t) / dup min-depth(ws)		\ stops reference count screw up
  dup if	\ if parameters need copying
\ cr ." parameters = " dup .
    a[ MOV  r0, rsp ]a  use-a		\ preserve old RSP, SDMOVE, will use R1
    dup begin				\ will need #args later
      1- dup in(ws) @ ( dup .resource ) >DR1 sdmove,	\ push the arguments, deepest first
      dup 0=
    until
    drop  0 ?do  drop(ws) loop		\ remove #args from working stack
    rR0 >DR1 sdmove,			\ add frame control data: previous RSP
  else		\ no parameters to copy
    drop  Thumb? if
      rRSP rR0 sdmove,			\ Cortex cannot push R13 directly
      rR0 >DR1 sdmove,			\ preserve old RSP
    else
      <ARMunpredictable> @ if		\ SFP117
        rRSP rR0 sdmove,		\ Cortex-A/R cannot push R13 directly
        rR0 >DR1 sdmove,		\ preserve old RSP
      else
        rRSP >DR1 sdmove,		\ preserve old RSP
      endif
    endif
  endif
\ cr ." set frame"
  M0/M1? if				\ SFP107
    a[ mov  r0, lp ]a use-a		\ LP to low reg
    a[ push .n {{ r0 }} ]a use-a
  else
    rLP >DR1 sdmove,			\ add frame control data: previous LP
  endif
  a[ MOV    lp, rsp ]a  use-a		\ set new LP
\ cr ." CopyLVargs out"
;

: genT2frame	\ #frame -- addr(t) ; in address units - for Thumb2
\ cr ." genT2frame"
  aligned(t) >r  <FixedLVs> @ if
    chere(t)				\ patch location
    <LVadjType> @ if
      a[ SUB .W   rsp, rsp, # ]a  r> n>a  use-a	\ drop frame by size
    else
      a[ SUB .N   rsp, rsp, # ]a  r> n>a  use-a	\ drop frame by size
    endif
  else
    chere(t)
    a[ MVL32 r0, # ]a  r> n>a  use-a
    a[ SUB   rsp, rsp, r0 ]a  use-a	\ drop frame by size
  endif
;

\ : #genLit?	\ n -- flag

: Force12bitLit	\ size -- size' ; SFP115
  begin  dup #genLit? 0= while  cell(t) +  repeat
;

: genARMframe	\ #frame -- addr(t) ; in address units - for ARM32
  aligned(t) >r  <FixedLVs> @ if
    chere(t)				\ patch location
    r> Force12bitLit >r			\ SFP115
    a[ SUB   rsp, rsp, # ]a  r> n>a  use-a	\ drop frame by size
  else
    chere(t)
    a[ MVL32 r0, # ]a  r> n>a  use-a
    a[ SUB   rsp, rsp, r0 ]a  use-a	\ drop frame by size
  endif
;

: genLVframe	\ #frame -- addr(t) ; in address units
  Thumb?
  if  genT2frame  else  genARMframe  endif
;

: makeLvsM0/M1	\ #args #frame -- addr(t)

: makeLvsT2/Arm	\ #args #frame -- addr(t)
  >r					\ preserve frame size, #frame
  CopyLVargs				\ copy the arguments
  Thumb?				\ generate local frame
  if  r> genT2frame  else  r> genARMframe  endif
;

: c_makelvs     \ #args #frame -- addr(t) ; generate code for MAKELVS
		\ #args and #frame are in target address units
  ?SaveLink
  over 4 cells <=  <fastlvs> @ and
  dup makelvstyle ! if			\ mark style in use
    Cortex-M0/M1?
    if  makeLvsM0/M1  else  makeLvT2/Arm  then
    c_shuffle  reset-opt		\ flush stack, must do because C_SHUFFLE will not
    					\ reorganise stacks with holes and #CHANGED -ve
  else
    target-sym MAKELVS compile-ref	\ compile local entry word
    chere(t) -rot			\ needed when frame is adjusted at compile time
    swap 4/ w,c(t)  4/ w,c(t)		\ #args then #frame
  then
;

cr ." Installing ARM optimiser MAKELVS patchup"
\ for VFX cross compiler
: PatchLocalSize(t)	\ #bytes/aus ^patch -- ; sets frame size ; SFP069
\ cr ." Patch size is " over .
  makelvstyle @ if
    <FixedLVs> @ if	\ faster/shorter?
      Thumb? if
        swap abs swap			\ in case negative
        <LVadjType> @			\ patch  sub  rsp, rsp, # u
        if  patchT2Imm138  else  patchT1Imm7*4  endif
      else
        swap abs Force12bitLit swap	\ SFP115
        tuck l@c(t) $FFFFF000 and	\ -- ^patch +n opcode' ; remove lit12 from opcode
        [+assembler] +immop [previous]	\ -- ^patch opcode''
        swap l!c(t)			\ -- ; set modified opcode
      endif
    else		\ easy
      swap abs swap			\ in case negative
      [+assembler] Thumb?
      if  fix_mov32  else  fix_adr  endif	\ store #bytes at patch address
      [previous]
    endif
  else
    swap addr(t)/cell u/ swap 2+ w!c(t)	\ store #cells at patch address +2
  endif
;
(( for hosted VFX system
: o_PatchLocalSize	\ -- ; set #locals to size defined by LVCOUNT
  makelvstyle @
  if
    [ also system ]
    lvcount @ negate cell-		\ #locals in bytes
    Patch-Address @ !
    [ previous ]
  else
    (PatchLocalSize)
  endif
;
))
\ ...SFP034

: c_rellvs      \ -- ; generate code for RELLVS ; SFP027 SFP069
  c_shuffle  reset-opt			\ for when we use MEMPOOL
  <FixedLVs> @ Thumb? 0= and if
    a[ LDMIA LP, {{ LP, rsp }} ]a use-a	\ cheaper, faster
  else
    M0/M1? if
      a[  mov     r0, LP
          ldmia   r0 ! {{ r1 r2 }}
          mov     lp, r1		\ must reload LP before RSP
          mov     rsp, r2		\ to avoid interrupt problems
      ]a use-a
    else
      a[ MOV  r0, LP ]a use-a		\ must update LP before RSP
      a[ LDR  LP, [ r0 ] ]a use-a	\ to be interrupt safe
      a[ LDR  rsp, [ r0, # 4 ] ]a  use-a
    endif
  endif
;


\ ******************************
\ Words for Pointer manipulation
\ ******************************

\ SFP031...
: c_reg@	\ res -- ; -- addr
  create
    ,
  does>
    reset-opt
    @					\ get src resource
    GetNewReg ?dup 0=			\ get new register
    if  c_shuffle  GetNewReg  endif	\ -- src new
    tuck sdmove,  push(ws)
;

rUP  c_reg@ c_up@
rLP  c_reg@ c_lp@
rRSP c_reg@ (c_rp@)

: c_rp@		\ -- ; SFP059
  ?SaveLink  (c_rp@)
;

: c_reg!	\ res -- ; addr --
  create
    ,
  does>
    1 min-depth(ws)  reset-opt
    M0/M1? if				\ SFP107
      tos/R0 swap @ sdmove,  drop(ws)
    else
      @ pop(ws)				\ -- dest src
      swap sdmove,
    endif
;

((
rUP  c_reg! c_up!
rLP  c_reg! c_lp!
rRSP c_reg! (c_rp!)
))

rUP  c_reg! (c_up!)
rLP  c_reg! c_lp!
rRSP c_reg! (c_rp!)

: tos/Reg	\ -- reg ; SFP124
\ If TOS is not a register, copy it to a working register (not R0).
\ Return the register.
  tos-res reg? if
    tos-res
  else
    GetRegRes  tos-res over sdmove,
  then
;

: c_up!		\ -- ; SFP124
  Thumb? if
    Cortex-M0/M1? 0=  tos-res lit? and if
      rR11 tos-res res-value @ pLoadLitCM3+  drop(ws)
    else
      tos/Reg rR11 sdmove,  drop(ws)
    then
  else
    (c_up!)
  then
;

: c_rp!		\ -- ; SFP059
  ?SaveLink  1 min-depth(ws)
  Thumb? if
    tos-res reg? 0= if
      GetNewReg ?dup 0= if
        c_shuffle  1 min-depth(ws)  tos-res
      else
        tos-res over sdmove,
      endif
      rR13 sdmove,
    else
      pop(ws) rR13 sdmove,
    endif
  else
    (c_rp!)
  endif
;

: c_sp@         \ -- n
  c_shuffle  reset-opt
  Thumb? if
    Cortex-M0/M1? if
      a[ sub .s  psp, psp, # 4
         str     tos, [ psp, # 0 ]
      ]a use-a
    else
      a[ str  tos, [ psp, # -4 ] ! ]a use-a
    endif
  else
    a[ str  tos, [ psp, # -4 ] ! ]a use-a	\ ARM9 ; SFP101
  endif
  a[ mov   tos, psp      ]a  use-a
;

: c_sp!         \ n --
  1 min-depth(ws)  tos-res lit? if
    pop(ws) res-value @ >r  c_shuffle
    M0/M1? if
      rPSP r> pLoadLitCM01
      a[ ldmia psp ! {{ tos }} ]a use-a
    else
      a[
        mvl32 psp, # r>
        ldr   tos, [ psp ], # 4
      ]a use-a
    endif
  else
    Thumb? if
      c_shuffle
      Cortex-M0/M1? if
        a[  mov .s  psp, tos
            ldmia   psp ! {{ tos }}
        ]a use-a
      else
        a[  mov   psp, tos
            ldr   tos, [ psp ], # 4
        ]a use-a
      endif
    else
      c_shuffle  a[
        mov   psp, tos
        ldr   tos, [ psp ], # 4		\ ARM9
      ]a use-a
    endif
  endif
  reset-opt
;
\ ...SFP031


\ *****************************
\ Miscellaneous code generators
\ *****************************

: c_nop		\ --
\  a[ NOP ]a  use-a
;

Required: >BODY
: c_>body       \ -- ; compile code for >BODY ; SFP033
  <LongCalls?> @ if
    target-sym >BODY compile-ref  exit
  endif
  8 c_lit  c_+				\ step over STM BL ; SFP101
  single-section? 0=
  if  c_@  endif			\ get address
;

required: execute
: c_execute     \ -- ; SFP037 SFP070 SFP077
  1 min-depth(ws) reset-opt

\ handle literal
  tos-res lit? if
\ cr ." LITERAL EXECUTE" .source-line
    pop(ws) res-value @ >r
    c_shuffle
    not-inlined ?LeafError ?SaveLink	\ relative call, leaf check, save link
    r@ BLrange? if			\ check for call by BL
      a[ bl ]a  r> n>a  use-a  exit
    endif
    Thumb? if
      Cortex-M0/M1? if			\ SFP114
        rR0 r> 1+ pLoadLitCM01
        a[ BLX  r0 ]a use-a
      else
        a[ MVL32+1 r0, # r> ]a use-a
        a[ BLX  r0 ]a use-a
      endif
    else
      a[ mov link, pc ]a use-a		\ $+00: link points to $+08
      a[ ldr pc, [ link ], # 4 ]a use-a	\ $+04: branch to contents of $+08
      r> ,c(t)				\ $+08: destination address
    endif
    exit
  endif

\ find a register for the target address
  tos-res dreg? tos-res rTOS <> and if
    pop(ws)
  else
    GetShuffleReg ?dup if		\ must not use rTOS
      pop(ws) over sdmove,
    else
      c_shuffle  pop(ws)
    endif
  endif
  dup res-refcount incr			\ stop shuffle using this register
  (c_shuffle)
  dup res-refcount decr			\ and release it
  check-stack(ws)
  not-inlined ?LeafError ?SaveLink	\ relative call, leaf check, save link
  Thumb? if
    >r
    Not-M0/M1? if			\ SFP107
      a[ orr ]a  r@ l>a  r@ l>a  a[ # 1 ]a use-a	\ force bit 0
    else
      a[ mov .s     r0, # 1 ]a use-a
      a[ orr .s ]a  r@ l>a  r@ l>a  rR0 l>a  use-a
    endif
    a[ blx ]a  r> l>a  use-a
  else
    a[ mov  link, pc ]a  use-a		\ set link to return to $+8
    >r  a[ mov  pc, ]a  r> l>a  use-a	\ set pc
  endif
;


\ The following 4 optimisers are to handle the 0 and sign extension of
\ items already on the stack. They have the stack picture:
\   nn..n0 n -- nn'..n0
\ where nn is sign or zero extended as required. Thus functions are provided
\ to permit promotion of items less than one cell in size when fetched from
\ memory or when passed on the stack to a C function.

: c_nsbwiden    \ -- ; Signed byte widen Nth Item on stack ; SFP043
  1 min-depth(ws)  reset-opt
  tos-res lit? if
    pop(ws) res-value @ >r		\ literal value
    r@ in(ws) @ dreg? if
      r> in(ws) @ >r			\ nth resource on stack
      a[ mov ]a  r@ l>a  r@ l>a a[ .lsl # #24 ]a  use-a
      a[ mov ]a  r@ l>a  r> l>a a[ .asr # #24 ]a  use-a
      exit
    endif
    r@ in(ws) @ lit? if
      r@ ?NewLitNeeded
      r> in(ws) @ 			\ nth resource on stack
      dup res-value @ dup $80 and
      if  $FFFFFF00 or  endif
      swap res-value !
      exit
    endif
    r@ in(ws) @ count=1? if
      r> in(ws) @ >r
        a[ ldr  r0, ]a  r@ l>a       use-a
        a[ mov  r0, r0 .lsl # #24 ]a  use-a
        a[ mov  r0, r0 .asr # #24 ]a  use-a
        a[ str  r0, ]a  r> l>a       use-a
	exit
    endif
    r@ in(ws) @ GetNewReg dup >r sdmove,	\ R: -- n reg
    a[ mov ]a  r@ l>a  r@ l>a a[ .lsl # #24 ]a  use-a
    a[ mov ]a  r@ l>a  r@ l>a a[ .asr # #24 ]a  use-a
    r>  r@ in(ws) @ res-refcount decr		\ reduce original refcount
    dup r> in(ws) !				\ replace old with new resource
    1 swap res-refcount !			\ set count of new resource to 1
    exit
  endif
  c_shuffle  1 min-depth(ws)
  a[ ldr  r0, [ psp ++ tos .lsl # 2 ] ]a  use-a
  a[ mov  r0, r0 .lsl # #24           ]a  use-a
  a[ mov  r0, r0 .asr # #24           ]a  use-a
  a[ str  r0, [ psp ++ tos .lsl # 2 ] ]a  use-a
  drop(ws)
;

: c_nswwiden    \ -- ; Signed word widen Nth Item on stack	SFP033
  1 min-depth(ws)  reset-opt
  tos-res lit? if
    pop(ws) res-value @ >r		\ literal value
    r@ in(ws) @ dreg? if
      r> in(ws) @ >r			\ nth resource on stack
      a[ mov ]a  r@ l>a  r@ l>a a[ .lsl # #16 ]a  use-a
      a[ mov ]a  r@ l>a  r> l>a a[ .asr # #16 ]a  use-a
      exit
    endif
    r@ in(ws) @ lit? if
      r@ ?NewLitNeeded
      r> in(ws) @ 			\ nth resource on stack
      dup res-value @ dup $8000 and
      if  $FFFF0000 or  endif
      swap res-value !
      exit
    endif
    r@ in(ws) @ count=1? if
      r> in(ws) @ >r
        a[ ldr  r0, ]a  r@ l>a       use-a
        a[ mov  r0, r0 .lsl # #16 ]a  use-a
        a[ mov  r0, r0 .asr # #16 ]a  use-a
        a[ str  r0, ]a  r> l>a       use-a
	exit
    endif
    r@ in(ws) @ GetNewReg dup >r sdmove,	\ R: -- n reg
    a[ mov ]a  r@ l>a  r@ l>a a[ .lsl # #16 ]a  use-a
    a[ mov ]a  r@ l>a  r@ l>a a[ .asr # #16 ]a  use-a
    r>  r@ in(ws) @ res-refcount decr		\ reduce original refcount
    dup r> in(ws) !				\ replace old with new resource
    1 swap res-refcount !			\ set count of new resource to 1
    exit
  endif
  c_shuffle  1 min-depth(ws)
  a[ ldr  r0, [ psp ++ tos .lsl # 2 ] ]a  use-a
  a[ mov  r0, r0 .lsl # #16           ]a  use-a
  a[ mov  r0, r0 .asr # #16           ]a  use-a
  a[ str  r0, [ psp ++ tos .lsl # 2 ] ]a  use-a
  drop(ws)
;

: c_nubwiden    \ -- ; Unsigned byte widen Nth Item on stack	SFP033
  1 min-depth(ws)  reset-opt
  tos-res lit? if
    pop(ws) res-value @ >r		\ literal value
    r@ in(ws) @ dreg? if
      r> in(ws) @ >r			\ nth resource on stack
      a[ and ]a  r@ l>a  r> l>a  a[ # $FF ]a  use-a
      exit
    endif
    r@ in(ws) @ lit? if
      r@ ?NewLitNeeded
      r> in(ws) @ 			\ nth resource on stack
      dup res-value @ dup $80 and
      if  $FF and  endif
      swap res-value !
      exit
    endif
    r@ in(ws) @ count=1? if
      r> in(ws) @ >r
        a[ ldr  r0, ]a  r@ l>a    use-a
        a[ and  r0, r0, # $FF ]a  use-a
        a[ str  r0, ]a  r> l>a    use-a
	exit
    endif
    r@ in(ws) @ GetNewReg dup >r sdmove,	\ R: -- n reg
    a[ and ]a  r@ l>a  r@ l>a  a[ # $FF ]a  use-a
    r>  r@ in(ws) @ res-refcount decr		\ reduce original refcount
    dup r> in(ws) !				\ replace old with new resource
    1 swap res-refcount !			\ set count of new resource to 1
    exit
  endif
  c_shuffle  1 min-depth(ws)
  a[ ldr  r0, [ psp ++ tos .lsl # 2 ] ]a  use-a
  a[ and  r0, r0, # $FF              ]a  use-a
  a[ str  r0, [ psp ++ tos .lsl # 2 ] ]a  use-a
  drop(ws)
;

: c_nuwwiden	\ -- ; Unsigned Word widen Nth Item on stack ; SFP033
  { | rd rs -- }
  1 min-depth(ws)  reset-opt
  tos-res lit? if
    pop(ws) res-value @ >r		\ literal value
    r@ in(ws) @ dreg? if
      r> in(ws) @ >r			\ nth resource on stack
      a[ bic ]a  r@ l>a  r@ l>a  a[ # $FF000000 ]a  use-a
      a[ bic ]a  r@ l>a  r> l>a  a[ # $00FF0000 ]a  use-a
      exit
    endif
    r@ in(ws) @ lit? if
      r@ ?NewLitNeeded
      r> in(ws) @ 			\ nth resource on stack
      dup res-value @ dup $80 and
      if  $0000FFFF and  endif
      swap res-value !
      exit
    endif
    r@ in(ws) @ count=1? if
      r> in(ws) @ >r
        a[ ldr  r0, ]a  r@ l>a          use-a
        a[ bic  r0, r0, # $FF000000 ]a  use-a
        a[ bic  r0, r0, # $00FF0000 ]a  use-a
        a[ str  r0, ]a  r> l>a          use-a
	exit
    endif
    r@ in(ws) @ GetNewReg dup >r sdmove,	\ R: -- n reg
    a[ bic ]a  r@ l>a  r@ l>a a[ # $FF000000 ]a  use-a
    a[ bic ]a  r@ l>a  r@ l>a a[ # $00FF0000 ]a  use-a
    r>  r@ in(ws) @ res-refcount decr		\ reduce original refcount
    dup r> in(ws) !				\ replace old with new resource
    1 swap res-refcount !			\ set count of new resource to 1
    exit
  endif
  c_shuffle  1 min-depth(ws)
  GetNewReg >r				\ SFP111
  a[ ldr  r0, [ psp ++ tos .lsl # 2 ]        ]a  use-a
  a[ bic ]a  r@ l>a  r@ l>a  a[ # $FF000000 ]a  use-a
  a[ bic ]a  r@ l>a  r@ l>a  a[ # $00FF0000 ]a  use-a
  a[ str  r0, [ psp ++ tos .lsl # 2 ]       ]a  use-a
  r> drop				\ SFP111
  drop(ws)
;

: c_rpick	{ | rd rs -- }
\ compile code for rpick ; SFP033
  1 min-depth(ws)
  ?SaveLink				\ SFP059
  tos-res lit? if
    pop(ws) res-value @ 2 lshift >r		\ offset into return stack
    ForceNewTos					\ destination
    a[ LDR ]a  tos-res l>a  a[ [ RSP, # r> n>a ] ]a  use-a
  else
    FindDest1 -> rd  Load1 -> rs		\ get TOS in register
    a[ MOV  R0, ]a  rs l>a a[ .LSL # 2 ]a  use-a
    a[ LDR ]a  rd l>a  a[ [ RSP ++ R0 ] ]a  use-a
    drop(ws)  rd push(ws)  reset-opt
  endif
;

: c_d+          \ -- ; generate code for D+             \ SFP030
  M0/M1? if
    target-sym D+ compile-ref  exit
  endif
  4 min-depth(ws)
  tos-res lit?  3os-res lit? and
  nos-res lit?  4os-res lit? and  or
  if
    target-sym D+ compile-ref
  else
    c_swap  c_2swap  c_-rot  c_+cl	\ low add
    GuardedFlags? >r  1 -> GuardedFlags? \ literals must not generate flags ; SFP125
    c_-rot  c_+ch			\ high add
    r> -> GuardedFlags?
  endif
;

: c_d-          \ -- ; generate code for D-             \ SFP030
  M0/M1? if
    target-sym D- compile-ref  exit
  endif
  4 min-depth(ws)
  tos-res lit?  3os-res lit? and
  nos-res lit?  4os-res lit? and  or
  if
    target-sym D- compile-ref
  else
    c_swap  c_2swap  c_-rot  c_swap c_-cl  c_-rot  c_swap c_-ch
  endif
;

: absRegRes	\ res -- ; negate the register resource
  >r
  a[ CMP ]a  r@ l>a  a[ # 0 ]a  use-a
  Thumb? if
    Cortex-M0/M1? if
      a[ mi, if,
         RSB .s  r@ l>a  r> l>a  # 0
         endif,
      ]a use-a
    else
      a[ IT .MI
         RSB  r@ l>a  r> l>a  # 0
      ]a use-a
    endif
  else
    a[ RSB .MI ]a  r@ l>a  r> l>a  a[ # 0 ]a  use-a
  endif
;

: c_abs         \ -- ; compile code for ABS
  1 min-depth(ws)  reset-opt
  tos-res lit? if                       \ do it at compile time?
    ?NewTOSLit                          \ ensure TOS is free for modification ; SFP017
    tos-res res-value  dup @  abs  swap !
    exit
  endif
  tos-res dreg? if
    tos-res absRegRes  exit
  endif
  GetNewReg ?dup if
    >r
    pop(ws) r@ sdmove,			\ SFP048, SFP059
    r@ absRegRes
    r> push(ws)
  else
    c_shuffle  recurse
  endif
;

: (minmax)	{ xt opctrue opcfalse | rs1 rs2 -- }
\ Code generator for MIN and MAX
  2 min-depth(ws)  reset-opt		\ SFP030

\ Deal with 2 literals first, more optimisation available
  tos-res lit?  nos-res lit? and if     \ can do everything at compile time
    ?NewNOSLit                          \ ensure NOS is free for modification ; SFP017
    tos-res res-value @  nos-res res-value @
    xt execute                          \ perform compile time function
    nos-res res-value !
    drop(ws)  exit
  endif

  ComLoad2  -> rs2  -> rs1		\ operands to registers
\ cr ." Generating compare"
  a[ CMP ]a  rs1 l>a  rs2 gl>a  use-a	\ compare
  drop(ws) drop(ws)
  ForceNewTos				\ result in new TOS
  Thumb? if
    Not-M0/M1? if			\ SFP107
      \ Cortex M2/M3+
      tos-res rs1 <>  tos-res rs2 <> and if
        a[ ite ]a  opctrue execute  use-a
        a[ MOV ]a  tos-res l>a  rs1 gl>a  use-a
        a[ MOV ]a  tos-res l>a  rs2 gl>a  use-a
        exit
      endif
      tos-res rs1 <> if
        a[ it ]a   opctrue execute  use-a
        a[ MOV ]a  tos-res l>a  rs1 gl>a  use-a
        exit
      endif
      tos-res rs2 <> if
        a[ it ]a   opcfalse execute  use-a
        a[ MOV ]a  tos-res l>a  rs2 gl>a  use-a
        exit
      endif
      true abort" ISE in (MINMAX) for M3"
      exit
    endif
    \ Cortex M0/M1
    tos-res rs1 <>  tos-res rs2 <> and if
      amrk>  a[ b ]a opcfalse execute  chere(t) 4 +  use-a
      a[ MOV .s ]a  tos-res l>a  rs1 gl>a  use-a
      amrk>  a[ b ]a  chere(t) 4 +  use-a  swap >ares
      a[ MOV .s ]a  tos-res l>a  rs2 gl>a  use-a
      >ares
      exit
    endif
    tos-res rs1 <> if
      amrk>  a[ b ]a opcfalse execute  chere(t) 4 +  use-a
      a[ MOV .s ]a  tos-res l>a  rs1 gl>a  use-a
      >ares
      exit
    endif
    tos-res rs2 <> if
      amrk>  a[ b ]a opctrue execute  chere(t) 4 +  use-a
      a[ MOV .s ]a  tos-res l>a  rs2 gl>a  use-a
      >ares
      exit
    endif
    true abort" ISE in (MINMAX) for M0/M1"
    exit
  endif
  \ ARM32
  tos-res rs1 <> if
    \ cr ." generating RS1, MOV " tos-res .res-name  space rs1 .res-name
    a[ MOV ]a  opctrue execute   tos-res l>a  rs1 gl>a  use-a
  endif
  tos-res rs2 <> if
    \ cr ." generating RS2, MOV " tos-res .res-name  space rs2 .res-name
    a[ MOV ]a  opcfalse execute  tos-res l>a  rs2 gl>a  use-a
  endif
;

: minmax       \ -- ; -- ;
  create
    ' ,                                 \ get XT of compile time action
    here  0 , 0 ,		        \ space for opcode actions
    lay-asm: lay-asm: drop   		\ get xts of opcode actions
  does>
    dup @				\ xt
    swap cell+ dup @			\ opctrue
    swap cell+ @			\ opcfalse
    (minmax)				\ use common code
;

minmax c_tmin  min   ".LT " ".GE "
minmax c_tmax  max   ".GT " ".LE "
minmax c_tumin umin  ".LO " ".HS "
minmax c_tumax umax  ".HI " ".LS "


\ ***********************************************
\ Words usually compiled as high level references
\ ***********************************************

: c_+string
  target-sym +STRING compile-ref
;

: c_>number
  target-sym >NUMBER compile-ref
;

: c_compare
  target-sym COMPARE compile-ref
;

: c_move
  target-sym MOVE compile-ref
;

: c_-trailing
  target-sym -TRAILING compile-ref
;

: c_c,
  target-sym C, compile-ref
;

: c_w,
  target-sym W, compile-ref
;

: c_,
  target-sym , compile-ref
;

: c_here
  target-sym HERE compile-ref
;

: c_there       \ SFP021
  target-sym HERE compile-ref
;

: c_allot
  target-sym ALLOT compile-ref
;

: c_blank
  target-sym BLANK compile-ref
;

: c_erase
  target-sym ERASE compile-ref
;

: c_word
  target-sym WORD compile-ref
;

: c_create
  target-sym CREATE compile-ref
;

: c_(;code)
  target-sym (;CODE) compile-ref
;

: c_s"		\ -- ; SFP043
  target-sym (S") compile-ref  ",c(t)  0 ?force-even
;

: c_c"		\ -- ; SFP043
  target-sym (C") compile-ref  ",c(t)  0 ?force-even
;

required: (Z")
: c_z"		\ -- ; SFP121
  target-sym (Z") compile-ref  ",c(t)  0 c,c(t)  0 ?force-even
;

: c_s\"		\ -- ; SFP109
  target-sym (S") compile-ref  \",c(t)  0 ?force-even
;

: c_c\"         \ -- ; SFP109
  target-sym (C") compile-ref  \",c(t)  0 ?force-even
;

: c_z\"		\ -- ; SFP121
  target-sym (Z") compile-ref  \",c(t)  0 c,c(t)  0 ?force-even
;

: c_."		\ -- ; SFP043
  target-sym (.") compile-ref  ",c(t)  0 ?force-even
;

: c_abort"	\ -- ; SFP043
  target-sym (ABORT") compile-ref  ",c(t)  0 ?force-even
;

: c_",          \ --
  target-sym ", compile-ref
;

: c_compile-word        \ symbol -- ; SFP026
  doliteralcfa,				\ lay xt from this symbol
  target-sym COMPILE, compile-ref       \ reference to COMPILE,
;

: c_compile_new	\ -- ; SFP026
  BL-WORD/UC FIND(T) ?undef             \ must be defined
  doliteralcfa,
  target-sym COMPILE, compile-ref
;


\ ***********************************************
\ Defining word and compilation structure support
\ ***********************************************

: c_defer       \ -- ; lay DEFER structure
  target-sym DEFER compile-run
  ?rom @(m) if
    iAlign(t)  ihere(t) ,c(t)
    target-sym CRASH  dup resolved? 0= xcerr_fwdref ?throw  @(s) ,d(t)
  else
    target-sym CRASH compile-ref
  endif
;

: c_(to-do)	\ -- ; SFP036
  BL-WORD/UC FIND(T) ?undef             \ must be defined
  dup resolved? 0= opt_fwd_todo ?throw	\ and resolved
  Thumb? if
    1 c_lit  c_or			\ force bit0=1 for Thumb
  endif
  @(s) >body(t) c_lit  c_!
;

: c_user        \ -- ; lay USER structure
  target-sym USER compile-run
  VFXuservars? @
  if  chere(t) cell(t) + ,c(t)  then	\ make compatible with embedded CREATE   SFP019
  ,c(t)
;

\ SFP107...
: userChildCM01	\ offset --
\ Generate code for child of USER. This version codes for
\ Cortex-M0 and Cortex-M1.
  reset-opt
  a[ MOV ]a  tos-res l>a  a[ UP ]a use-a
  c_lit  c_+
;

: userChildARM	\ offset --
\ Generate code for child of USER. This version codes for CM3+
\ and ARM.
  dup >r
  abs [+assembler] imm12? [previous] if	\ SFP054
    r@ 0< if
      a[ SUB ]a  tos-res l>a  a[ UP, # ]a  r> abs n>a  use-a
    else
      r@ rUP nplus-seq			\ can use LITPLUS sequence
        a[ ADD ]a  tos-res l>a  a[ UP, # ]a  r> n>a  use-a
      end-seq
    endif
  else
    tos-res r> PrimLoadLit		\ SFP036
    a[ ADD ]a tos-res l>a  tos-res l>a  a[ UP ]a  use-a
  endif
; doNotSin

: c_userchild	\ symbol -- ; generate code for child of USER ; SFP027
  ForceNewTOS  reset-opt  user@		\ -- offset
  Thumb? if
    Cortex-M0/M1?
    if  userChildCM01  else  userChildARM  endif
  else
    userChildARM
  endif
; doNotSin

\ ...SFP107

: c_value       \ n -- addr(t) ; compile structure of a VALUE.
  target-sym VALUE compile-run
  ?Rom @(m) if                          \ compile *data* structure for a VALUE
    iAlign(t)  ihere(t)                 \ address at which data is stored
    dup ,c(t)                           \ lay pointer to RAM
  else
    here(t)                             \ address of following data
  endif
  swap ,d(t)                            \ lay initial value
;

: c_val@        \ val-addr --
  c_lit  c_@  ;

: c_val!        \ val-addr --
  c_lit  c_!  ;

: c_val		\ val-addr -- ; SFP039
  c_lit  ;

: c_2variable   \ -- addr(t) ; compile code and data structure of 2VARIABLE
  target-sym VARIABLE compile-run
  ?rom @(m) If
    iAlign(t)  ihere(t) dup ,c(t)  0 ,d(t)  0 ,d(t)
  Else
    chere(t)  0 ,c(t)  0 ,c(t)
  Endif
;

: c_variable    \ -- addr(t) ; compile code and data structure of VARIABLE
  target-sym VARIABLE compile-run
  ?rom @(m) If
    Ialign(t)  ihere(t) dup ,c(t)  0 ,d(t)
  Else
    chere(t)  0 ,c(t)
  Endif
;

: c_Wvariable	\ -- addr(t) ; compile code and data structure of WVARIABLE ; SFP041
  target-sym VARIABLE compile-run
  ?rom @(m) If
    ?even(r)  ihere(t) dup ,c(t)  0 w,d(t)
  Else
    chere(t)  0 w,c(t)
  Endif
;

: c_Cvariable   \ -- addr(t) ; compile code and data structure of CVARIABLE ; SFP019
  target-sym VARIABLE compile-run
  ?rom @(m) If
    ihere(t) dup ,c(t)  0 c,d(t)
  Else
    chere(t)  0 c,c(t)
  Endif
;

: c_buffer:     \ -- addr(t)
\ compile code and data structure of BUFFER:
  ?rom @(m) If
    target-sym CONSTANT compile-run
    ualign(t)  uhere(t) dup ,c(t)
  Else
    target-sym VARIABLE compile-run
    chere(t)
  Endif
;

: c_constant    \ n --
  target-sym CONSTANT compile-run  ,c(t)  ;

: c_2constant   \ d --
  target-sym 2CONSTANT compile-run  ,c(t) ,c(t)  ;

: c_struct      \ -- addr(t)
  target-sym CONSTANT compile-run  chere(t)  0 ,c(t)  ;

: c_end-struct  \ addr(t) size --
  swap !c(t)  ;

: c_field       \ size --
  target-sym FIELD compile-run  ,c(t)  ;

: c_field-child \ symbol -- ; compile child of FIELD as LIT +  ; SFP022
  field@ ?dup
  if  c_lit  c_+  endif
;


\ ****************************
\ *S ARM and Cortex intrinsics
\ ****************************
\ *P Intrinsics are what compiler people call special code
\ ** generators that provide access to CPU-specific operations.
\ ** The indicator *\fo{C_} in the names below is just for
\ ** internal use by the compiler. The words you use are without
\ ** *\fo{C_}.

\ *P The intrinsics in this section are available for all CPUs.

((
c_arshift arshift	\ x u -- x'
\ *G Arithmetic right shift.
c_ror ror		\ x u -- x'
\ *G Rotate right.
fetchop c_w@s		\ addr -- sw
\ *G 16 bit fetch and then sign extended.
fetchop c_c@s		\ addr -- sb
\ *G 8 bit fetch and then sign extended.
addstoreop c_w+!	\ n addr --
\ *G As *\fo{+!} but for a 16 bit item.
addstoreop c_c+!	\ n addr --
\ *G As *\fo{+!} but for an 8 bit item.
addstoreop c_or!	\ mask addr --
\ *G *\fo{OR} the *\i{mask} into the data at *\i{addr}.
addstoreop c_and!	\ mask addr --
\ *G *\fo{AND} the *\i{mask} into the data at *\i{addr}.
addstoreop c_xor!	\ mask addr --
\ *G *\fo{XOR} the *\i{mask} into the data at *\i{addr}.
addstoreop c_bic!	\ mask addr --
\ *G Clear the *\i{mask} bits in the data at *\i{addr}.

addstoreop c_bor!	\ bmask caddr --
\ *G *\fo{OR} the *\i{bmask} into the 8 bit data at *\i{caddr}.
addstoreop c_band!	\ bmask caddr --
\ *G *\fo{AND} the *\i{bmask} into the 8 bit data at *\i{caddr}.
addstoreop c_bxor!	\ bmask caddr --
\ *G *\fo{XOR} the *\i{bmask} into the 8 bit data at *\i{caddr}.
addstoreop c_bbic!	\ bmask caddr --
\ *G Clear the *\i{bmask} bits in the 8 bit data at *\i{caddr}.

rUP  c_reg@ c_up@	\ -- addr
\ *G Return the address of the *\fo{USER} area.
rLP  c_reg@ c_lp@	\ -- addr
\ *G Return the local variable pointer.
rUP  c_reg! c_up!	\ addr --
\ *G Set the *\fo{USER} area pointer.
rLP c_reg! c_lp!	\ addr --
\ *G Set the local variable pointer.
))

\ SFP123...
required: di
required: ei
required: dfi
required: efi

: c_di		\ -- ; disable interrupts
\ *G For Cortex parts, lays *\fo{CPS .ID .I}. For ARM parts, lays
\ ** a call to the word *\fo{DI}.
  Thumb? if
    a[ CPS .ID .I ]a use-a
  else
    target-sym DI compile-ref
  endif
;

: c_ei		\ -- ; enable interrupts
\ *G For Cortex parts, lays *\fo{CPS .IE .I}. For ARM parts, lays
\ ** a call to the word *\fo{EI}.
  Thumb? if
    a[ CPS .IE .I ]a use-a
  else
    target-sym EI compile-ref
  endif
;

: c_dfi		\ -- ; disable fast interrupts
\ *G For Cortex parts, lays *\fo{CPS .ID .F}. For ARM parts, lays
\ ** a call to the word *\fo{DFI}.
  Thumb? if
    a[ CPS .ID .F ]a use-a
  else
    target-sym DFI compile-ref
  endif
;

: c_efi		\ -- ; enable fast interrupts
\ *G For Cortex parts, lays *\fo{CPS .IE .F}. For ARM parts, lays
\ ** a call to the word *\fo{EFI}.
  Thumb? if
    a[ CPS .IE .F ]a use-a
  else
    target-sym EFI compile-ref
  endif
;
\ ...SFP123

required: FSP!
: c_fsp!	\ -- ; set R8
\ *G Set the floating point stack pointer R8, e.g.
\ *C   <address> FSP!
  1 min-depth(ws)
  pop(ws) rR8 sdmove,
;


\ *****************************
\ *S Cortex-specific intrinsics
\ *****************************
\ *P Intrinsics are what compiler people call special code
\ ** generators that provide access to CPU-specific operations.
\ ** The indicator *\fo{C_} in the names below is just for
\ ** internal use by the compiler. The words you use are without
\ ** *\fo{C_}.

: ?T2intrinsic	\ -- ; error if not T2
  Thumb? 0= opt_T2intrinsic ?throw
;

required: sys@
: c_sys@	\ -- ; <lit> SYS@
\ *G Intrinsic for for MRS reg, SYSm. Use this to read a system
\ ** register such as CONTROl or BASEPRI.
\ *C   BASEPRI sys@
  1 min-depth(ws) reset-opt  ?T2intrinsic
  tos-res lit? 0= opt_SYSm ?throw	\ SYS must be literal
\  GetNewReg dup 0= opt_no_regs ?throw
  getRegRes				\ error if no free register
  pop(ws) res-value @ >r  dup >r
  a[ MRS ]a  r> l>a  r>  use-a
  push(ws)
;

required: sys!
: c_sys!	\ -- ; <val> <lit> SYS!
\ *G Intrinsic for for MSR SYSm reg. Use this to write a system
\ ** register.
\ *C   2 CONTROL sys!
  2 min-depth(ws) reset-opt  ?T2intrinsic
  tos-res lit? 0= opt_SYSm ?throw
  nos-res reg? if
    nos-res
  else
    getRegRes				\ error if no free register
\    GetNewReg dup 0= opt_no_regs ?throw
    nos-res over sdmove,
  endif
  >r
  a[ MSR ]a  tos-res res-value @  r> l>a  use-a
  tos-res res-value @ #20 =		\ CONTROL reg?
  ISB? and if
    Cortex-M0/M1? if			\ lay ISB
      a[ ISB $0F ]a use-a
    else
      a[ ISB $0F ]a use-a
    endif
  endif
  drop(ws) drop(ws)			\ SFP103
;

required: set-sp
: c_set-sp	\ -- ; <lit> SET-SP
\ *G Used to set the Forth data stack pointer during initialisation.
\ ** Unlike the normal Forth *\fo{SP!}, the previous top of stack
\ ** is *\b{not} saved. After use of *\fo{SET-SP}, you can assume
\ ** nothing about the data stack except that it is empty.
  1 min-depth(ws) reset-opt  ?T2intrinsic
  Cortex-M0/M1? if			\ SFP107
    get-psp 8 u< if
      pop(ws) rPSP sdmove,
    else
      tos/R0 rPSP sdmove,  drop(ws)
    endif
  else
    tos-res lit? 0= opt_SYSm ?throw
    a[ mvl  psp, # ]a  pop(ws) res-value @  use-a
  endif
;

\ SFP108...
required: ByteRevL
: c_ByteRevL	\ --
\ *G Swap the bytes in a 32 bit long word.
  1 min-depth(ws) reset-opt  ?T2intrinsic
  tos-res lit? if			\ do it at compile time?
    ?NewTOSLit                          \ ensure TOS is free for modification ; SFP017
    tos-res res-value dup @  ByteRevL  swap !
    exit
  endif
  tos/r0 drop(ws) >r			\ R: -- rsrc
  GetRegRes dup push(ws) >r		\ R: -- rsrc rdest
  a[ rev ]a  r> l>a  r> l>a  use-a
;

required: ByteRevW
: c_ByteRevW	\ --
\ *G Swap the bytes in the low 16 bits of TOS and swap the bytes
\ ** in the upper 16 bits. This is safe to use to get a zero
\ ** extended result immediately after *\fo{W@}.
  1 min-depth(ws) reset-opt  ?T2intrinsic
  tos-res lit? if			\ do it at compile time?
    ?NewTOSLit                          \ ensure TOS is free for modification ; SFP017
    tos-res res-value dup @  ByteRevW  swap !
    exit
  endif
  tos/r0 drop(ws) >r			\ R: -- rsrc
  GetRegRes dup push(ws) >r		\ R: -- rsrc rdest
  a[ rev16 ]a  r> l>a  r> l>a  use-a
;

required: ByteRevWS
: c_ByteRevWS	\ --
\ *G Swap the bytes in the low 16 bits of TOS and sign extend
\ ** them to 32 bits.
  1 min-depth(ws) reset-opt  ?T2intrinsic
  tos-res lit? if			\ do it at compile time?
    ?NewTOSLit                          \ ensure TOS is free for modification ; SFP017
    tos-res res-value dup @  ByteRevWS  swap !
    exit
  endif
  tos/r0 drop(ws) >r			\ R: -- rsrc
  GetRegRes dup push(ws) >r		\ R: -- rsrc rdest
  a[ revsh ]a  r> l>a  r> l>a  use-a
;

required: ByteRevWZ
: c_ByteRevWZ	\ --
\ *G Swap the bytes in the low 16 bits of TOS and zero extend it.
  c_byteRevW
  #16 c_lit  c_lshift
  #16 c_lit  c_rshift
;
\ ...SFP108

\ SFP110...
required: WFE
: c_wfe		\ --
\ *G Flushes the stack and generates a *\fo{WFE .N} instruction.
  ?T2intrinsic  c_flushcode  a[ wfe .n ]a use-a
;

required: WFI
: c_wfi		\ --
\ *G Flushes the stack and generates a *\fo{WFI .N} instruction.
  ?T2intrinsic  c_flushcode  a[ wfi .n ]a use-a
;
\ ...SFP110

\ SFP119...
required: DSB#0F
: c_dsb#0F		\ --
\ *G Flushes the stack and generates a *\fo{DSB  # $0F} instruction.
  ?T2intrinsic  c_flushcode  a[ dsb # $0F ]a use-a
;

required: ISB#0F
: c_isb#0F		\ --
\ *G Flushes the stack and generates an *\fo{ISB  # $0F} instruction.
  ?T2intrinsic  c_flushcode  a[ isb # $0F ]a use-a
;
\ ...SFP119


\ ********************
\ Miscellaneous macros
\ ********************

: c_bounds      \ -- ; compile code for BOUNDS
  c_over c_+ c_swap  ;

: c_count       \ -- ; compile code for COUNT
  c_dup  c_1+ c_swap c_c@  ;

: c_cell        \ -- ; compile code for CELL
  cell(t) c_lit  ;

: c_cells+      \ -- ; compile code for CELLS+
  c_4* c_+  ;

: c_bl          #32 c_lit  ;
: c_null        null c_lit  ;
: c_MAX_PATH    max_path c_lit  ;
: c_TRUE        true c_lit  ;
: c_FALSE       false c_lit  ;

: c_s>d		\ -- ; SFP031 SFP035 SFP074
  tos-res lit? if
    tos-res res-value @ 0< c_lit  exit
  endif
\  c_dup c_0<
  c_dup  #31 c_lit  c_arshift
;

: c_m+          \ -- ; generate code for M+ ; SFP030
\  c_dup c_0< c_d+
  c_s>d c_d+				\ SFP074
;

: c_3drop	c_drop  c_2drop  ;	\ SFP031
: c_4drop	c_2drop  c_2drop  ;	\ SFP031

: c_/string	c_tuck c_- c_-rot c_+ c_swap  ;	\ SFP041

: c_?negate	c_0< c_?branch> c_negate >c_res_branch  ;	\ SFP042

\ **************
\ Compiler words
\ **************

only forth also c-c also compiler definitions  c-c

\   "name"      "compiler"      "analogue"
xc: @           c_@             @d(t)
xc: +string     c_+string       +string-(t)
xc: >number     c_>number       >number(t)
xc: c!          c_c!            c!d(t)
xc: c@          c_c@            c@d(t)
xc: compare     c_compare       compare(t)
xc: count       c_count         count(t)
xc: /string	c_/string	/string(t)	\ SFP050
xc: move        c_move          cmove-(t)
xc: -trailing   c_-trailing     -trailing(t)
xc: 2!          c_2!            2!d(t)
xc: 2@          c_2@            2@d(t)
xc: c,          c_c,            gc,(t)
xc: w,          c_w,            gw,(t)
xc: here        c_here          here(t)
xc: allot       c_allot         allot(t)
xc: blank       c_blank         blank-(t)
xc: erase       c_erase         erase-(t)
xc: word        c_word          word(t)
xc: incr        c_incr          incr(t)
xc: decr        c_decr          decr(t)
xc: cell        c_cell          cell(t)
xc: cell+       c_4+            4+
xc: cell-	c_4-		4-
xc: cells       c_4*            4*
xc: cells+      c_cells+        cells+(t)

xc: i           c_i             i
xc: j           c_j             j
xc: swap        c_swap          swap
xc: 2swap       c_2swap         2swap
xc: drop        c_drop          drop
xc: 2drop       c_2drop         2drop
xc: dup         c_dup           dup
xc: ?dup        c_?dup          ?dup
xc: 2dup        c_2dup          2dup
xc: 3dup	c_3dup		3dup	\ SFP046
xc: over        c_over          over
xc: 2over       c_2over         2over
xc: rot         c_rot           rot
xc: -rot        c_-rot          -rot
xc: nip         c_nip           nip
xc: tuck        c_tuck          tuck
xc: pick        c_pick          pick
xc: roll        c_roll          roll
xc: +           c_+             +
xc: +c          c_+ch           +	\ SFP118
xc: -           c_-             -
xc: 1+          c_1+            1+
xc: 2+          c_2+            2+
xc: 4+          c_4+            4+
xc: 1-          c_1-            1-
xc: 2-          c_2-            2-
xc: 4-          c_4-            4-
xc: 2*          c_2*            2*
xc: 4*          c_4*            4*
xc: 2/          c_2/            2/
xc: u2/         c_u2/           u2/
xc: 4/          c_4/            4/
xc: u4/         c_u4/           u4/
xc: negate      c_negate        negate
xc: ?negate	c_?negate	?negate	\ SFP042

xc: *           c_*             *
xc: um*		c_um*		um*	\ SFP035
xc: m*		c_m*		m*	\ SFP035
xc: /		c_/		/	\ SFP040
xc: u/		c_u/		u/

xc: w@          c_w@            w@d(t)
xc: =           c_=             =
xc: <>          c_<>            <>
xc: <           c_<             <
xc: >           c_>             >
xc: <=          c_<=            <=
xc: >=          c_>=            >=
xc: u<          c_u<            u<
xc: u>          c_u>            u>
xc: u>=		c_u>=		u>=	\ SFP079
xc: u<=		c_u<=		u<=	\ SFP079
xc: 0=          c_0=            0=
xc: 0<>         c_0<>           0<>
xc: 0<          c_0<            0<
xc: 0>          c_0>            0>
xc: >r          c_>r            >r
xc: r@          c_r@            r@
xc: r>          c_r>            r>
xc: 2>r         c_2>r           2>r
xc: 2r@         c_2r@           2r@
xc: 2r>         c_2r>           2r>

xc: and         c_and           and
xc: or          c_or            or
xc: xor         c_xor           xor
xc: invert      c_invert        not
xc: not         c_invert        not

xc: or!		c_or!		none(t)		\ SFP045
xc: and!	c_and!		none(t)		\ SFP045
xc: xor!	c_xor!		none(t)		\ SFP045
xc: bic!	c_bic!		none(t)		\ SFP045

xc: wor!	c_wor!		none(t)		\ SFP128
xc: wand!	c_wand!		none(t)		\ SFP128
xc: wxor!	c_wxor!		none(t)		\ SFP128
xc: wbic!	c_wbic!		none(t)		\ SFP128

xc: bor!	c_bor!		none(t)		\ SFP050
xc: band!	c_band!		none(t)		\ SFP050
xc: bxor!	c_bxor!		none(t)		\ SFP050
xc: bbic!	c_bbic!		none(t)		\ SFP050

xc: bounds      c_bounds        bounds
xc: abs         c_abs           abs
\ xc: min         c_min           min
\ xc: max         c_max           max
xc: min         c_tmin           min
xc: max         c_tmax           max

xc: chars       noop            noop            \ MSD020
xc: char+       c_1+            1+              \ assume char=byte=au

xc: d+          c_d+            d+              \ SFP033
xc: d-          c_d-            d-              \ SFP033
xc: m+          c_m+            m+              \ SFP033

xc: >body       c_>body         >body(t)        \ SFP033

xc: <<n         c_lshift        lshift             \ SFP027
xc: lshift      c_lshift        lshift             \ SFP027
xc: >>n         c_rshift        rshift             \ SFP027
xc: rshift      c_rshift        rshift             \ SFP027
xc: arshift     c_arshift       none(t)		\ SFP049
xc: rol		c_rol		rol		\ SFP044
xc: ror		c_ror		ror		\ SFP044

xc: on		c_on		on(t)		\ SFP028
xc: off		c_off		off(t)		\ SFP028

xc: s>d		c_s>d		s>d		\ SFP031
xc: unloop	c_unloop	unloop		\ SFP034
xc: execute	c_execute	none(t)		\ SFP037

\ MPEisms for PFW VFX and the C compiler
xc: bl          c_bl            bl              \ SFP025
xc: null        c_null          null            \ SFP025
xc: MAX_PATH    c_max_path      max_path        \ SFP025
xc: TRUE        c_true          true            \ SFP025
xc: FALSE       c_false         false           \ SFP025
xc: up@         c_up@           none(t)         \ SFP031
xc: up!         c_up!           none(t)         \ SFP031
xc: sp@         c_sp@           none(t)         \ SFP031
xc: sp!         c_sp!           none(t)         \ SFP031
xc: rp@         c_rp@           none(t)         \ SFP031
xc: rp!         c_rp!           none(t)         \ SFP031
xc: lp@		c_lp@		none(t)		\ SFP031
xc: lp!		c_lp!		none(t)		\ SFP031
xc: 3drop	c_3drop		3drop		\ SFP031
xc: 4drop	c_4drop		4drop		\ SFP031
xc: C@S         c_c@s           none(t)         \ SFP031
xc: W@S         c_w@s           none(t)         \ SFP031

xc: nsbwiden    c_nsbwiden      none(t)         \ SFP033
xc: nswwiden    c_nswwiden      none(t)         \ SFP033
xc: nubwiden    c_nubwiden      none(t)         \ SFP033
xc: nuwwiden    c_nuwwiden      none(t)         \ SFP033
xc: rpick       c_rpick         none(t)         \ SFP033

xc: [o/f]       c_flushcode     noop            \ flush ; optimiser control
xc: [o/s]       .ws             noop            \ show ; optimiser control
xc: [o/m]       c_nop		noop		\ marker

\ Intrinsics
xc: sys@	c_sys@		none(t)		\ SFP102
xc: sys!	c_sys!		none(t)		\ SFP102
xc: set-sp	c_set-sp	none(t)		\ SFP103
xc: ByteRevL	c_ByteRevL	ByteRevL	\ SFP108
xc: ByteRevW	c_ByteRevW	ByteRevW	\ SFP108
xc: ByteRevWS	c_ByteRevWS	ByteRevWS	\ SFP108
xc: ByteRevWZ	c_ByteRevWZ	ByteRevWZ	\ SFP108
xc: wfe		c_wfe		none(t)		\ SFP110
xc: wfi		c_wfi		none(t)		\ SFP110
xc: dsb#0f	c_dsb#0f	none(t)		\ SFP119
xc: isb#0f	c_isb#0f	none(t)		\ SFP119
xc: di		c_di		none(t)		\ SFP123
xc: ei		c_ei		none(t)		\ SFP123
xc: dfi		c_dfi		none(t)		\ SFP123
xc: efi		c_efi		none(t)		\ SFP123
xc: fsp!	c_fsp!		none(t)		\ SFP126
c-c  only forth  also c-c definitions


\ ****************************
\ *S Optimiser in-built macros
\ ****************************
((
\ *P The ARM Cortex VFX optimiser includes a number of special
\ ** cases which may not be present in the target code, but can
\ ** be easily written if required. In the main, these are present
\ ** to support peripheral register handling.
BOR! BAND! BXOR! BBIC! ( mask addr -- )
\ *G Logical operation on bytes (8 bits) in memory for CPUs with
\ ** 8 bit peripheral registers.
OR! AND! XOR! BIC! ( mask addr -- )
\ *G Logical operation on words (32 bits) in memory for CPUs with
\ ** 32 bit peripheral registers (the vast majority).
ARSHIFT ROL ROR ( x count -- x' )
\ *G Shift operations in the style of *\fo{LSHIFT}. *\fo{ARSHIFT}
\ ** performs an arithmetic right shift. *\fo{ROL} and *\fo{ROR}
\ ** are 32 bit circular left and right shifts.
))

\ ****************************
\ *S Efficient coding practice
\ ****************************
\ *P Writing good code for ARM and Cortex CPUs can double
\ ** performance by reducing instruction count and memory accesses.
\ ** This is particularly true for I/O operations involving bit
\ ** masking. Although register operations are nearly always
\ ** single-cycle operations, on most ARM/Cortex CPU implementations
\ ** memory loads (LDRx instructions) for ARM take three cycles,
\ ** Cortex loads take two cycles, taken branches need three
\ ** cycles, and memory stores (STRx instructions) take two
\ **  cycles. All of these have to be extended by any additional
\ ** memory overhead such as the effect of running with wait
\ ** states and/or 16 bit memory. A small amount of instruction
\ ** cache can make a huge difference to systems with slow memory.

\ *P ARM CPUs are not good at handling literals as these are assembled
\ ** 8 bits at a time. Thus an I/O address may result in four 32 bit
\ ** instructions. To reduce this, the code generator places literals
\ ** that cannot be coded in one instruction in a literal pool after
\ ** the end of the word.

\ *P Cortex-M3/4 CPUs have better literal handling plus additional
\ ** instructions for loading 16-bit immediates. A 32 bit literal
\ ** can be loaded into a register in two instructions (8 bytes).
\ ** For Cortex-M3/4, the code density benefit of a literal pool is
\ ** minimal, but the performance cost is high.

\ *P The following code fragments illustrate the benefits of efficient
\ ** coding. In MPE practice, absolute peripheral addresses are
\ ** indicated by a leading underscore character '_', whereas offsets
\ ** from a base address do not have an underscore. These examples are for
\ ** an NXP LPC2106 CPU with an Ethernet controller connected to
\ ** GPIO lines.

\ *P The first example is the simplistic operation.
\ *E : eth>read	\ -- ; turns data bus for reading
\ **   _IODIR @
\ **   $1FFE00FF and            \ set data bus to i/p
\ **   $E00000F0 or             \ set others to o/p
\ **   _IODIR !
\ ** ;
\ ** dis eth>read
\ ** ETH>READ
\ ** ldr r0, [ pc, # $1C ]   ( @$B3CC = $E0028008 )
\ ** ldr r8, [ r0, # $00 ]
\ ** ldr r0, [ pc, # $18 ]   ( @$B3D0 = $1FFE00FF )
\ ** and r8, r8, r0
\ ** ldr r0, [ pc, # $14 ]   ( @$B3D4 = $E00000F0 )
\ ** orr r8, r8, r0
\ ** ldr r1, [ pc, # $10 ]   ( @$B3D8 = $E0028008 )
\ ** str r8, [ r1, # $00 ]
\ ** mov pc, r14
\ ** 36 bytes, 9 instructions.
\ *P To this we have to add four 32 bit literals in the literal pool
\ ** for a total of 12 words or 48 bytes (excluding the final return).
\ ** At one cycle for the eight instructions we have to add two cycles
\ ** each for the three loads (+6) and one cycle for the write (+1).
\ ** The total is 48 bytes and 8+6+1=15 cycles.

\ *P The second example is written knowing that we have an ARM CPU and
\ ** with understanding of the GPIO structure.
\ *E : erdmode	\ -- ; turns data bus for reading
\ **   _GPIO
\ **   dup IODIR + @
\ **   edbmask invert and
\ **   swap IODIR + !
\ ** ;
\ ** dis erdmode
\ ** ERDMODE
\ ** ldr r8, [ pc, # $0C ]   ( @$B770 = $E0028000 )
\ ** ldr r7, [ r8, # $08 ]
\ ** bic r7, r7, # $FF00
\ ** str r7, [ r8, # $08 ]
\ ** mov pc, r14
\ ** 20 bytes, 5 instructions.
\ *P Again ignoring the final return we have four effective
\ ** instructions, one literal pool fetch and one store, resulting in
\ ** 20 bytes (vs 48) and 4+2+1=7 cycles (vs 15). Removing the
\ ** redundant OR has saved us four cycles and twelve bytes, and
\ ** loading the base address (literal) once has saved us another four
\ ** cycles and eight bytes.

\ *P Most peripheral operations require access to more than one
\ ** register, e.g. command/status/data. Loading the base address once
\ ** and then accessing individual registers by fixed offsets from the
\ ** base allows the VFX code generator to optimise more away at the
\ ** expense of slightly longer source code. Use of the built-in
\ ** optimiser macros can reduce the expansion of the source code.
\ ** Because the PC relative loads have a limited range, the code
\ ** generator builds a new literal pool for each word. Most source
\ ** files concentrate on a limited set of peripheral register base
\ ** addresses. You can avoid the size penalty of the literal pool by
\ ** providing a local pointer to the base address in the *\fo{CDATA} section
\ ** and referencing it throughout the source file. This will be
\ ** accessed by a PC relative load.
\ *E L: ^GPIOe
\ **   _GPIO ,
\ ** : erdmode2	\ -- ; turns data bus for reading
\ **   ^GPIOe @
\ **   dup IODIR + @
\ **   edbmask invert and
\ **   swap IODIR + !
\ ** ;
\ ** dis erdmode2
\ ** ERDMODE2
\ ** ldr r8, [ pc, # $-1C ]  ( @$B9F4 = $E0028000 )
\ ** ldr r7, [ r8, # $08 ]
\ ** bic r7, r7, # $FF00
\ ** str r7, [ r8, # $08 ]
\ ** mov pc, r14
\ ** 20 bytes, 5 instructions.

\ *P Thus the rule of thumb is to use the Forth stack to *\fo{DUP}
\ ** or *\fo{OVER} literals rather than to reference them several
\ ** times as literals, and to avoid use of literals that do not
\ ** fit in the ARM 4/8 bit literal format. By doing this you will
\ ** also make better use of future improvements to the VFX code
\ ** generator.


\ ======
\ *> ###
\ ======

